import React from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';
import styles from './styles.module.less';
import { Modal } from 'COMPONENT';
import { Radio } from 'antd';
import { LINK, MESSAGE, POSTER, validRadios, radioLabelMap } from './config';
import { createUid } from './utils';
import { Link, Message, Poster } from '../ShareBy';

const RadioGroup = Radio.Group;

const _infoPropTypes = PropTypes.arrayOf(
  PropTypes.shape({
    text: PropTypes.string,
    textAlign: PropTypes.oneOf(['left', 'center', 'right']),
    fontWeight: PropTypes.oneOf(['normal', 'bold']),
  }),
);

class ConfirmByShareWay extends React.Component {
  static propTypes = {
    title: PropTypes.string,
    loading: PropTypes.bool,
    onCancel: PropTypes.func.isRequired,
    radios: PropTypes.arrayOf(PropTypes.string),
    link: PropTypes.shape({
      url: PropTypes.string,
    }),
    message: PropTypes.shape({
      info: _infoPropTypes,
      template: PropTypes.string,
      onSend: PropTypes.func,
    }),
    poster: PropTypes.shape({
      infos: _infoPropTypes,
      url: PropTypes.string,
      tips: PropTypes.string,
    }),
  };

  static defaultProps = {
    title: '确认信息',
    loading: false,
    radios: validRadios,
    link: {},
    message: {},
    poster: {},
  };

  constructor(props) {
    super(props);

    this.updateRadio = this.updateRadio.bind(this);
    this.getRadioComponents = this.getRadioComponents.bind(this);

    const radioKeys = props.radios.filter((item) => validRadios.includes(item));
    if (radioKeys.length === 0) {
      throw new Error(`ConfirmByShareWay.props.radios must be one of ${validRadios}!`);
    }

    this.radioKeys = radioKeys;

    this.state = {
      radios: radioKeys.map((item) => ({ label: radioLabelMap[item], value: item })),
      selected: radioKeys[0],

      posterMounted: radioKeys[0] === POSTER,
      messageMounted: radioKeys[0] === MESSAGE,
      linkMounted: radioKeys[0] === LINK,

      iframeSrc: undefined,
    };

    this.mountedKeyMap = {
      [POSTER]: 'posterMounted',
      [MESSAGE]: 'messageMounted',
      [LINK]: 'linkMounted',
    };
  }

  componentDidMount() {
    if (this.state.iframeSrc) {
      return;
    }

    if (this.props.poster.url) {
      const key = createUid();

      sessionStorage.setItem(
        key,
        JSON.stringify({
          data: this.props.poster,
          config: {
            scale: 2,
          },
        }),
      );

      this.setState({
        iframeSrc: `${location.origin}/generatePicture/confirmInfo?key=${key}`,
      });
    }
  }

  componentDidUpdate() {
    if (this.state.iframeSrc) {
      return;
    }

    if (this.props.poster.url) {
      const key = createUid();

      sessionStorage.setItem(
        key,
        JSON.stringify({
          data: this.props.poster,
          config: {
            scale: 2,
          },
        }),
      );

      this.setState({
        iframeSrc: `${location.origin}/generatePicture/confirmInfo?key=${key}`,
      });
    }
  }

  updateRadio(e) {
    const value = e.target.value;
    this.setState({
      selected: value,
      [this.mountedKeyMap[value]]: true,
    });
  }

  getRadioComponents() {
    let radioComponents = [];

    this.radioKeys.forEach((radioKey) => {
      if (this.state[this.mountedKeyMap[radioKey]] || this.state.selected === radioKey) {
        switch (radioKey) {
          case LINK:
            radioComponents.push(
              <div className={classNames({ [styles.hide]: this.state.selected !== LINK })} key={LINK}>
                <Link {...this.props.link} />
              </div>,
            );
            break;

          case MESSAGE:
            radioComponents.push(
              <div className={classNames({ [styles.hide]: this.state.selected !== MESSAGE })} key={MESSAGE}>
                <Message {...this.props.message} />
              </div>,
            );
            break;

          case POSTER:
            radioComponents.push(
              <div className={classNames({ [styles.hide]: this.state.selected !== POSTER })} key={POSTER}>
                <Poster iframeSrc={this.state.iframeSrc} />
              </div>,
            );
            break;

          default:
        }
      }
    });

    return radioComponents;
  }

  render() {
    return (
      <Modal
        title={this.props.title}
        width={680}
        loading={this.state.loading || this.props.loading}
        onCancel={this.props.onCancel}
        footer={null}
      >
        {
          /*this.state.radios.length > 1 &&*/ <RadioGroup onChange={this.updateRadio} value={this.state.selected}>
            {this.state.radios.map((item) => (
              <div className={styles.templateItem} key={item.value}>
                <Radio value={item.value}>{item.label}</Radio>
              </div>
            ))}
          </RadioGroup>
        }

        {this.getRadioComponents()}
      </Modal>
    );
  }
}

ConfirmByShareWay.radioLink = LINK;
ConfirmByShareWay.radioMessage = MESSAGE;
ConfirmByShareWay.radioPoster = POSTER;

export default ConfirmByShareWay;
