/**
 * 修改学生姓名业务组件
 * @author zero
 */
import React, { Component } from 'react';
import { Input, Row, Col, message } from 'antd';
import PropTypes from 'prop-types';
import style from './index.less';
import Modal from 'COMPONENT/Modal';
import { renameStudentName } from 'SERVICE/contract';
import { runFunc } from 'UTIL/common';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

class ModalChangStuName extends Component {
  static PropTypes = {
    visible: PropTypes.bool, // 控制Modal显示与隐藏
    cusId: PropTypes.number.isRequired, // 客户id
    studentId: PropTypes.number.isRequired, // 学生id
    studentName: PropTypes.string.isRequired, // 学生姓名
    onCancel: PropTypes.func, // 取消事件
    onOk: PropTypes.func, // 确定事件,
    onMerge: PropTypes.func, // 学生姓名合并回调
  };

  static defaultProps = {
    visible: true,
    cusId: null,
    studentId: null,
    studentName: null,
    onCancel: () => {},
    onOk: () => {},
  };

  handleOk = () => {
    const name = this.inputRef.refs.input.value;
    if (!name) return message.warning('请输入新的学生姓名');
    if (name === this.props.studentName) {
      return message.warning('新旧学生姓名不能相同');
    }
    this.updateStudentName(name);
  };

  updateStudentName = async (studentName) => {
    const { cusId, studentId, onOk, onCancel } = this.props;
    const params = { latestCustomerId: cusId, id: studentId, name: studentName };
    const { resultCode } = await renameStudentName(params);
    if (resultCode === 0) {
      showMessage('更改成功！', 'success');
      runFunc(onOk, studentName);
      runFunc(onCancel);
    }
  };

  render() {
    const { visible, onCancel, studentName } = this.props;
    return (
      <Modal toggle title="更改姓名" visible={visible} width={400} onOk={this.handleOk} onCancel={onCancel}>
        <Row className={style.input_row}>
          <Col span={7}>
            <span className={style.input_row_span}>原学生姓名：</span>
          </Col>
          <Col span={16}>
            <Input disabled value={studentName} />
          </Col>
        </Row>
        <br />
        <Row className={style.input_row}>
          <Col span={7}>
            <span className={style.input_row_span}>新学生姓名：</span>
          </Col>
          <Col span={16}>
            <Input ref={(ref) => (this.inputRef = ref)} />
          </Col>
        </Row>
      </Modal>
    );
  }
}

export default ModalChangStuName;
