import React from 'react';
import PropTypes from 'prop-types';
import ModalEditCustomerContact from '../index';
import ConfirmByShareWay from 'BLOC/ConfirmByShareWay';
import { getSmsTemplate, getFrontEndConfig } from 'SERVICE/common';
import { sendCustomerContactChangeH5ConfirmSms } from 'SERVICE/customer';
import { errorsHandle, runFunc } from 'UTIL/common';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

const errorsHandler = errorsHandle();
const { radioPoster, radioMessage } = ConfirmByShareWay;

class Modal extends React.Component {
  static propTypes = {
    customerId: PropTypes.string.isRequired,
    customerName: PropTypes.string.isRequired,
    customerContact: PropTypes.string.isRequired,
    onClose: PropTypes.func.isRequired,
  };

  constructor(props) {
    super(props);

    this.loadData = this.loadData.bind(this);
    this.handleModify = this.handleModify.bind(this);
    this.sendMessage = this.sendMessage.bind(this);

    this.state = {
      modifyPhoneVisible: true,
      confirmInfoVisible: false,
      confirmInfoLoading: false,
      radios: [radioMessage, radioPoster],
      messageInfo: [{ text: '发送号码：' }, { text: `家长姓名：${props.customerName}` }],
      template: null,
      posterInfo: [
        { text: `客户：${props.customerName}` },
        { text: `原电话：${props.customerContact}` },
        { text: '新电话：' },
      ],
      url: null,
    };

    this.isNeedInit = true;
    this.urlStorage = null;

    this.changeLogId = null;
  }

  loadData(resolve) {
    let fetchList = [
      getSmsTemplate({
        templateCodeTag: 'SMS_CUS_CONTACT_H5_CONFIRM',
      }),
      getFrontEndConfig({
        module: 'crm',
        key: 'contact-change-confirm-h5-url',
      }),
    ];

    Promise.all(fetchList)
      .then(([template, url]) => {
        this.urlStorage = url.data || '';
        this.setState({ template: template.data.templateContent || '' });
      })
      .catch(errorsHandler)
      .finally(resolve);
  }

  async handleModify(changeLogId, newContactNum) {
    this.changeLogId = changeLogId;

    if (this.isNeedInit) {
      await new Promise(this.loadData);
      this.isNeedInit = false;
    }

    this.setState((state) => {
      state.url = this.urlStorage.replace('#{changeLogId}', changeLogId);
      state.messageInfo[0].text = state.messageInfo[0].text + newContactNum;
      state.posterInfo[2].text = state.posterInfo[2].text + newContactNum;
      state.modifyPhoneVisible = false;
      state.confirmInfoVisible = true;
      return state;
    });
  }

  async sendMessage(changeLogId) {
    this.setState({ confirmInfoLoading: true });

    let apiParams = {
      changeLogId: typeof changeLogId === 'object' ? this.changeLogId : changeLogId,
    };
    const res = await sendCustomerContactChangeH5ConfirmSms(apiParams).catch(errorsHandler);
    if (res) {
      showMessage('发送成功！');
      runFunc(this.props.onClose);
    } else {
      this.setState({ confirmInfoLoading: false });
    }
  }

  render() {
    const { props, state } = this;

    return (
      <div>
        {state.modifyPhoneVisible && (
          <ModalEditCustomerContact {...props} onOk={this.handleModify} onSend={this.sendMessage} />
        )}

        {state.confirmInfoVisible && (
          <ConfirmByShareWay
            title="确认信息"
            loading={state.confirmInfoLoading}
            onCancel={props.onClose}
            radios={state.radios}
            message={{
              infos: state.messageInfo,
              template: state.template,
              onSend: this.sendMessage,
            }}
            poster={{
              infos: state.posterInfo,
              url: state.url,
              tips: '请扫码确认相关信息',
            }}
          />
        )}
      </div>
    );
  }
}

export default Modal;
