import React from 'react';
import PropTypes from 'prop-types';
import { Modal, Form, Input } from 'COMPONENT';
import { runFunc, errorsHandle } from 'SRC/utils/common';
import showMessage from 'SRC/components/utils/globalTips/showMessage';
import { onlyRenameStudent } from 'SERVICE/student';

const FormItem = Form.Item;
const errorsHandler = errorsHandle();

class ModifyStudentName extends React.Component {
  static propTypes = {
    studentId: PropTypes.string.isRequired,
    studentName: PropTypes.string,
    customerId: PropTypes.string.isRequired,
  };

  constructor(props) {
    super(props);

    this.handleSubmit = this.handleSubmit.bind(this);
    this.submit = this.submit.bind(this);

    this.state = {
      loading: false,
    };

    this.formRef = null;
  }

  handleSubmit() {
    runFunc(this.formRef.submitFields);
  }

  async submit(values) {
    this.setState({ loading: true });
    const apiParams = Object.assign(
      {
        id: this.props.studentId,
        latestCustomerId: this.props.customerId,
      },
      values,
    );
    const res = await onlyRenameStudent(apiParams).catch(errorsHandler);
    if (res && res.resultCode === 0) {
      showMessage('保存成功！', 'success');
      runFunc(this.props.onCancel);
      GlobalEvent.emit('MODIFY_STUDENT_NAME_REFRESH_LIST');
    } else {
      this.setState({ loading: false });
    }
  }

  submitFail = () => {
    showMessage('请完善好所有信息后再提交！', 'error');
  };

  render() {
    return (
      <Modal
        title="修改学生姓名"
        loading={this.state.loading}
        onCancel={this.props.onCancel}
        onOk={this.handleSubmit}
        okBtnText="保存"
        widthSize="S"
      >
        <Form ref={(c) => (this.formRef = c)} onSubmit={this.submit} onSubmitFail={this.submitFail}>
          <FormItem
            name="name"
            initialValue={this.props.studentName}
            rules={[{ required: true, message: '学生姓名不能为空' }]}
            formItemProps={{
              label: '学生姓名',
              colon: false,
              labelCol: { span: 6 },
              wrapperCol: { span: 16 },
            }}
          >
            <Input placeholder="请输入学生姓名" />
          </FormItem>
        </Form>
      </Modal>
    );
  }
}

export default ModifyStudentName;
