/**
 * @description 选择学校业务组件
 * @author zero
 */

import React, { Component } from 'react';
import { Row, Col, Form, Input, Select as AntdSelect } from 'antd';
import { connect } from 'dva';
import PropTypes from 'prop-types';

import style from './index.less';
import Modal from 'COMPONENT/Modal';
import Select from 'COMPONENT/Select';
import { getSchoolByBlCampus, findOrganizationById } from 'SERVICE/common';

const { Item } = Form;
const { Option } = AntdSelect;

class ModalSelectSchool extends Component {
  state = {
    province_code: null, // 当前选择省份
    city_code: null, // 当前的市id
    schoolData: [], //    学校枚举数据
    schoolId: [], // 选择的学校id
    remarks: false, // 是否打开备注，即学校选了其它
  };

  cacheData = {}; // 缓存已选过省份的数据

  componentDidMount() {
    const { dispatch, provinceCityDict } = this.props;
    // 初始化所有省市数据，已有不再请求
    if (!provinceCityDict)
      dispatch({
        type: 'dict/initDict',
        payload: {
          list: [{ name: 'provinceCityDict', params: 'dict' }],
        },
      });
    // 根据blcampusId初始化省市值
    this.queryOrganizationById();
  }

  handleProvinceSelect = (code) => {
    const {
      form: { setFieldsValue },
    } = this.props;
    const { province_code } = this.state;
    this.cacheCity(code);
    province_code !== code && setFieldsValue({ city: undefined });
    this.setState({ province_code: code });
  };

  handleCitySelect = async (val) => {
    const params = { blCampus: val };
    const { schoolType } = this.props;
    if (schoolType) {
      // 学生学校不匹配提醒，根据学校类型过滤，多加 schoolType 字段过滤
      Object.assign(params, { schoolType });
    }
    const { value = {} } = await getSchoolByBlCampus(params);
    delete value[''];
    const _schoolData = Object.keys(value).map((key) => {
      return { text: value[key], val: key };
    });
    if (this.props.useOtherRemark) {
      _schoolData.push({ text: '其他', val: '-1' });
    }
    this.setState({ school_Original: value, schoolData: _schoolData, city_code: val });
  };

  handleSchoolSelect = (schoolId) => {
    // 若选了其它，打开备注输入框
    this.setState({ schoolId, remarks: schoolId === '-1' });
  };

  queryOrganizationById = async () => {
    const {
      blCampusId,
      form: { setFieldsValue },
      schoolId,
    } = this.props;
    if (blCampusId) {
      const { province, city } = await findOrganizationById({ id: blCampusId });
      // 根据city id值获取学习中心数据
      this.handleCitySelect(city.id);
      // this.cacheCity(province.id)
      this.setState({ province_code: province.id }, () => {
        setFieldsValue({ province: province.id, city: city.id, school: schoolId });
      });
    }
  };

  cacheCity = (code) => {
    const { provinceCityDict } = this.props;
    const procinData = provinceCityDict ? provinceCityDict.dict : [];
    // 筛选数据, 已有不筛选
    if (!this.cacheData[code]) {
      for (let i = 0, len = procinData.length; i < len; i += 1) {
        const { val, options } = procinData[i];
        if (code === val) this.cacheData[code] = options;
      }
    }
    return this.cacheData[code];
  };

  handleOk = () => {
    const {
      form: { validateFields },
      onOk,
    } = this.props;
    validateFields((err, valus) => {
      if (err) return;
      const { schoolId, city_code, province_code, school_Original } = this.state;
      // id: 学校id（学校为其它时id是-1）, name: 学校名字, remarks: 备注, cityId: 市id, provinceId: 省id
      onOk({
        id: schoolId,
        name: school_Original[schoolId],
        remarks: valus.remarks,
        cityId: city_code,
        provinceId: province_code,
      });
    });
  };

  render() {
    const { form, provinceCityDict, visible, onCancel } = this.props;
    const { getFieldDecorator } = form;
    const { province_code, schoolData, remarks } = this.state;
    const procinData = provinceCityDict ? provinceCityDict.dict : [];
    const cityData = this.cacheData[province_code] || this.cacheCity(province_code) || [];

    const formItemLayout = {
      labelCol: {
        xs: { span: 24 },
        sm: { span: 8 },
      },
      wrapperCol: {
        xs: { span: 24 },
        sm: { span: 16 },
      },
    };

    const school_formItemLayout = {
      labelCol: {
        xs: { span: 24 },
        sm: { span: 4 },
      },
      wrapperCol: {
        xs: { span: 24 },
        sm: { span: 20 },
      },
    };

    return (
      <Modal toggle title="学校选择" visible={visible} width={600} onOk={this.handleOk} onCancel={onCancel}>
        <Form>
          <Row gutter={8}>
            <Col span={12}>
              <Item {...formItemLayout} label="城市">
                {getFieldDecorator('province')(
                  <AntdSelect onSelect={this.handleProvinceSelect}>
                    {procinData.map((item) => {
                      const { val, label } = item;
                      return (
                        <Option value={val} key={val}>
                          {label}
                        </Option>
                      );
                    })}
                  </AntdSelect>,
                )}
              </Item>
            </Col>
            <Col span={12}>
              <Item>
                {getFieldDecorator('city')(
                  <Select dict={cityData} onSelect={this.handleCitySelect}>
                    {cityData.map((item) => {
                      const { val, text } = item;
                      return (
                        <Option value={val} key={val}>
                          {text}
                        </Option>
                      );
                    })}
                  </Select>,
                )}
              </Item>
            </Col>
            <Col span={24}>
              <Item {...school_formItemLayout} label="学校">
                {getFieldDecorator('school', { rules: [{ required: true, message: '请选择学校' }] })(
                  <Select
                    showSearch
                    dict={schoolData}
                    optionFilterProp="children"
                    onSelect={this.handleSchoolSelect}
                    notFoundContent="暂无学校数据"
                    placeholder="请选择学校"
                  />,
                )}
              </Item>
            </Col>
            {this.props.useOtherRemark && remarks && (
              <Col span={24}>
                <Item {...school_formItemLayout} label="备注">
                  {getFieldDecorator('remarks', { rules: [{ required: true, message: '请填写备注' }] })(<Input />)}
                </Item>
              </Col>
            )}
            {this.props.useOtherRemark && (
              <span className={style.tips_text}>若找不到您的学校，请选择其他并添加备注</span>
            )}
          </Row>
        </Form>
      </Modal>
    );
  }
}

ModalSelectSchool.propTypes = {
  visible: PropTypes.bool, // 控制Modal显示与隐藏
  blCampusId: PropTypes.oneOfType([PropTypes.string, PropTypes.number]).isRequired, // 已选学习中心ID
  schoolId: PropTypes.oneOfType([PropTypes.string, PropTypes.number]), // 学校id
  onCancel: PropTypes.func, // 取消事件
  onOk: PropTypes.func, // 确定事件
  useOtherRemark: PropTypes.bool,
  schoolType: PropTypes.string, // 学校类型字段，过滤学校
};

ModalSelectSchool.defaultProps = {
  visible: true,
  schoolId: null,
  useOtherRemark: true,
  onCancel: () => {},
  onOk: () => {},
};

export default connect((state) => ({ provinceCityDict: state.dict.provinceCityDict }))(
  Form.create()(ModalSelectSchool),
);
