import React from 'react';
import { Modal } from 'COMPONENT';
import UploadFile from 'SRC/BLoComponents/UploadFile';
import { cloneJsonArray, formatApiFileList } from './util';
import styles from './index.less';

const ossFile = { fileId: null, fileKey: null, fileName: null, fileType: null, size: null, src: null };
// 最大可上传数量
const MAX_FILE_LENGTH = 5;

/**
 * 上传附件业务组件
 *
 * @param props.editable {boolean} 是否允许编，默认true
 * @param props.maxFileLength {number} 最大可选张数
 * @param props.fileList {array} 附件列表(需组装成OSS格式的附件文件)
 * @param props.onclose {function} 关闭弹窗
 * @param props.onOk {function}
 */
class Index extends React.Component {
  static defaultProps = {
    editable: true,
    maxFileLength: MAX_FILE_LENGTH,
    fileList: [],
    validateFileTypes: [],
    onClose: () => {},
    onOk: () => {},
  };

  constructor(props) {
    super(props);

    this.state = {
      okBtnDisabled: false,
      fileList: [],
    };
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化讲座考勤照片
   */
  initContent = () => {
    let fileList = cloneJsonArray(this.props.fileList);
    const canPushEmptyFile = this.props.editable && fileList.length < this.props.maxFileLength;

    if (canPushEmptyFile) {
      fileList.push(ossFile);
    }

    this.setState({ fileList });
  };

  /**
   * 上传成功
   *
   * @param file {object} OSS附件文件
   * @param index {number} 文件列表索引
   */
  onUploadComplete = (file, index) => {
    let fileList = [...this.state.fileList];
    fileList[index] = { ...ossFile, ...file };

    const canPushEmptyFile = this.props.editable && fileList.length < this.props.maxFileLength;
    if (canPushEmptyFile) {
      fileList.push(ossFile);
    }

    this.setState({ fileList });
  };

  /**
   * 删除附件
   *
   * @param fileId {string} 附件ID
   * @param index {number} 文件列表索引
   */
  onDeleteComplete = (fileId, index) => {
    let fileList = [...this.state.fileList];
    fileList.splice(index, 1);

    const hasEmptyFile = fileList.some((file) => file.src === null);
    const canPushEmptyFile = this.props.editable && !hasEmptyFile && fileList.length < this.props.maxFileLength;

    if (canPushEmptyFile) {
      fileList.push(ossFile);
    }

    this.setState({ fileList });
  };

  /**
   * 保存
   */
  onOk = () => {
    this.props.onOk(formatApiFileList(this.state.fileList));
    this.props.onClose();
  };

  render() {
    const fileList = this.state.fileList || [];
    const okBtnDisabled = fileList.length === 0 || !fileList[0].src;

    return (
      <Modal
        title="选择文件"
        okText="确定"
        width={600}
        loading={false}
        okBtnDisabled={okBtnDisabled}
        okBtnShow={true}
        cancelBtnShow={true}
        onOk={this.onOk}
        onClose={this.props.onClose}
      >
        <ul className={styles.fileWrap}>
          {fileList.map((item, index) => {
            const value = item.src ? item : null;

            return (
              <li key={index}>
                <UploadFile
                  value={value}
                  text="上传附件"
                  params={{ fileType: 0 }}
                  size={160}
                  viewOnly={!this.props.editable}
                  validateFileTypes={this.props.validateFileTypes}
                  uploadComplete={(file) => this.onUploadComplete(file, index)}
                  deleteComplete={(fileId) => this.onDeleteComplete(fileId, index)}
                />
              </li>
            );
          })}
        </ul>
      </Modal>
    );
  }
}

export default Index;
