import { AutoComplete } from 'antd';
import React from 'react';
import PropTypes from 'prop-types';
import styles from './styles.module.less';
import { getStudentList } from 'SERVICE/student';
import { debounce } from 'lodash';

export function parseStudent(value) {
  // e.g. value = AAA(柠溪学习中心,肖玉芬)STU190529000012
  const reg = /(.+)\((.+)?,?(.+)?\)(.+)/;
  const res = value.match(reg);
  if (!res) return null;
  return {
    value: res[0],
    studentName: res[1],
    campusName: res[2],
    campusTeacher: res[3],
    studentId: res[4],
  };
}

/* 学生姓名自动补全搜索 */
export default class SearchStudentAutoComplete extends React.Component {
  static propTypes = {
    placeholder: PropTypes.string,
    value: PropTypes.string,
    onChange: PropTypes.func,
    onSelect: PropTypes.func,
    onStudentChange: PropTypes.func,
  };

  static defaultProps = {
    placeholder: '学生编号/学生姓名',
    value: '',
    onChange: (value) => {},
    onSelect: (value, option) => {},
    onStudentChange: (parsedStudentObj) => {},
  };

  constructor(props) {
    super(props);
    this.state = {
      studentList: [],
    };
    this.debounceSearch = debounce(this.handleSearch, 500);
  }

  componentWillReceiveProps(nextProps) {
    // 重置后清空学生列表
    if ('value' in nextProps && !nextProps.value) {
      this.setState({ studentList: [] });
    }
  }

  handleSearch = async (value) => {
    const params = { rows: 30, page: 1, sidx: 'id', sord: 'desc' };

    if ((value || '').toLowerCase().startsWith('stu')) {
      params.id = value;
    } else {
      params.name = value;
    }

    const res = await getStudentList(params);

    if (!res || res.resultCode !== 0 || !res.rows.length) {
      this.setState({
        studentList: [],
      });
    } else {
      this.setState({
        studentList: res.rows.map(({ id, name, blCampusName }) => ({
          text: `${name}(${blCampusName || ''})${id}`,
          value: `${name}(${blCampusName || ''})${id}`,
        })),
      });
    }
  };

  handleChange = (value) => {
    console.log(value);
    this.props.onChange && this.props.onChange(value);
  };

  handleSelect = (value, option) => {
    this.props.onSelect && this.props.onSelect(value, option);
    const parsedStudentObj = parseStudent(value);
    this.props.onStudentChange && this.props.onStudentChange(parsedStudentObj);
  };

  render() {
    return (
      <AutoComplete
        className={styles.autocomplete}
        style={{ width: this.props.width || '310px' }}
        autoFocus={true}
        backfill={true}
        placeholder={this.props.placeholder}
        dataSource={this.state.studentList}
        value={this.props.value}
        allowClear={true}
        onSearch={this.debounceSearch}
        onChange={this.handleChange}
        onSelect={this.handleSelect}
        notFoundContent={<p>找不到学生</p>}
      />
    );
  }
}
