import React from 'react';
import { connect } from 'dva';
import PropTypes from 'prop-types';
import { Modal } from 'COMPONENT';
import { Form } from 'antd';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

class ActionModal extends React.Component {
  static propTypes = {
    onSubmit: PropTypes.func.isRequired,
    onClose: PropTypes.func.isRequired,
    id: PropTypes.any.isRequired,
    title: PropTypes.string,
    size: PropTypes.string, //S|M|L
  };

  constructor(props) {
    super(props);
    this.state = {
      loading: false,
    };
  }

  handleSubmit = () => {
    this.props.form.validateFields(async (errors, form) => {
      if (errors) {
        return;
      }

      const { id, onSubmit, onClose } = this.props;
      this.setState({ loading: true });

      onSubmit({ ...form, id })
        .then((res) => {
          showMessage(res || '操作成功', 'info');
          onClose();
        })
        .catch((err) => {
          this.setState({ loading: false });
          console.error(err);
          showMessage('操作失败:' + err.text, 'error');
        });
    });
  };

  render() {
    //将本组件生成的FORM传递到子组件
    const childrenWithProps = React.Children.map(this.props.children, (child) => {
      return React.cloneElement(child, { form: this.props.form });
    });

    return (
      <Modal
        onOk={this.handleSubmit}
        onClose={this.props.onClose}
        title={this.props.title}
        widthSize={this.props.size || 'M'}
      >
        <Form>{childrenWithProps}</Form>
      </Modal>
    );
  }
}

export default connect()(Form.create()(ActionModal));
