/**
 * 按钮
 * @property {ReactNode} children - 内容
 * @property {string} [iconType] - 图标类型
 * @property {string} [type] - 类型，'primary'
 * @property {string} [size] - 大小，'small'|'xSmall'
 * @property {Function} [onClick] - 点击时的回调
 * @property {object} [...props] - 传递给antd Button的其他属性
 */
import React from 'react'
import ReactDOM from 'react-dom'
import { Button } from 'antd'
import './styles.less'
import Icon from '../Icon'
import classnames from 'classnames'

const sizeConfig = {
  'undefined': 'large',
  'large': 'large',
  'small': 'default',
  'xSmall': 'small',
};

class ButtonPlus extends React.Component {
  disabledClick = false;

  componentDidMount(){
    const { dataReportClick, dataReportParams } = this.props;

    // 设置埋点属性到DOM结构
    if(dataReportClick) {
      const btn = ReactDOM.findDOMNode(this.refs.btn);

      if(btn){
        btn.setAttribute("data-report-click", dataReportClick);

        if(dataReportParams){
          btn.setAttribute("data-report-params", JSON.stringify(dataReportParams));
        }
      }

    }
  }

  //防止多次快速点击
  onClick = async (e) => {
    const { onClick } = this.props;

    if (this.disabledClick) {
      return
    }
    this.disabledClick = true;
    typeof onClick === 'function' && onClick(e);
    setTimeout(() => {
      this.disabledClick = false;
    }, 300);
  }

  render() {
    const { children, type, iconType, size: sizeProp, disabled, style, className, ...props } = this.props
    let nativeProps = { ...this.props };

    delete nativeProps.dataReportClick;
    delete nativeProps.dataReportParams;
    delete nativeProps.size;
    delete nativeProps.iconType;

    const content = typeof children === 'string' && children.length === 2 && !iconType ? `${children[0]} ${children[1]}` : children
    const cls = {
      'Button': 1,
      [className]: !!className,
    };

    return (
      <Button
        type={type === 'primary' ? 'primary' : undefined}
        size={sizeConfig[sizeProp]}
        onClick={this.onClick}
        disabled={disabled === true ? true : undefined}
        style={style}
        className={classnames(cls)}
        {...nativeProps}
        ref='btn'
      >
        {iconType && <Icon className="Button-icon" type={iconType} size={sizeConfig[sizeProp] === 'large' ? 14 : 12} />}
        {content}
      </Button>
    )
  }
}

export default ButtonPlus
