import React from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';
import style from './styles.module.less';
import ReactDOM from "react-dom";

const classPrefix = 'ExpandHeight';

class ExpandHeight extends React.Component {
  constructor(props) {
    super(props);
    this.switch = this.switch.bind(this);
    this.open = this.open.bind(this);
    this.close = this.close.bind(this);
    this.isOpen = false;
    this.$expand = null;
    this.$expandBody = null;
    this.isbind = false;
  }

  static propTypes = {
    autoClose: PropTypes.bool,
    className: PropTypes.string,
    style: PropTypes.object,
    expandClassName: PropTypes.string,
    expandStyle: PropTypes.object,
  };

  static defaultProps = {
    autoClose: false,
  };

  static stopPropagation(e) {
    e.stopPropagation();
  }

  componentDidMount() {
    if (this.props.autoClose) {
      this.isbind = true;
      window.addEventListener('click', this.closeExpand, false);
    }
  }

  componentWillUnmount() {
    if (this.isbind) {
      window.removeEventListener('click', this.closeExpand, false);
    }
  }

  switch() {
    this.isOpen = !this.isOpen;
    if (this.isOpen) {
      this.open();
    } else {
      this.close();
    }
  }

  open() {
    this.isOpen = true;
    this.$expand.style.height =`${this.$expandBody.offsetHeight}px`;
  }

  close() {
    this.isOpen = false;
    this.$expand.style.height = '0px';
  }

  render() {
    return (
      <div className={classNames(style[classPrefix], this.props.className)}
           onClick={ExpandHeight.stopPropagation}
           ref={c => this.$expand = this.$expand || ReactDOM.findDOMNode(c)}
           style={this.props.style}>
        <div className={classNames(style[`${classPrefix}-body`], this.props.expandClassName)}
             ref={c => this.$expandBody = this.$expandBody || ReactDOM.findDOMNode(c)}
             style={this.props.expandStyle}>
          {this.props.children}
        </div>
      </div>
    )
  }
}

export default ExpandHeight;
