import React from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';
import { Form } from 'antd';
import style from './styles.module.less';

const FormItem = Form.Item;

const Item = (props) => {
  return (
    <div className={classNames(style.formItemWrap, props.wrapClassName)}>
      <FormItem help={props.help} {...props.formItemProps}>
        {
          props.form.getFieldDecorator(
            props.name,
            {
              initialValue: props.initialValue,
              rules: props.rules,
              ...props.fieldDecoratorOpt,
            }
          )(
            props.children
          )
        }
      </FormItem>
      {props.slot}
    </div>
  );
};

Item.propTypes = {
  initialValue: PropTypes.any,
  required: PropTypes.bool,
  beforeSnapshot: PropTypes.any,
  children: PropTypes.any,
  formItemProps: PropTypes.object,
  fieldDecoratorOpt: PropTypes.object,
  wrapClassName: PropTypes.string,
  slot: PropTypes.oneOfType([PropTypes.string, PropTypes.element]),
  rules: PropTypes.arrayOf(PropTypes.object),
  help: PropTypes.oneOfType([PropTypes.string, PropTypes.element]),
};

Item.defaultProps = {
  span: 1,
  initialValue: null,
  required: false,
  formItemProps: {},
  fieldDecoratorOpt: {},
  wrapClassName: null,
  slot: null,
  rules: undefined,
  help: undefined,
};

export default Item;
