import React from 'react';
import PropTypes from 'prop-types';
import { Form as AntdForm } from 'antd';
import { runFunc } from '../utils/common';
import Item from './Item';

class _Form extends React.Component {
  constructor(props) {
    super(props);
    this.submit = this.submit.bind(this);
  }

  static propTypes = {
    onSubmit: PropTypes.func.isRequired,
    onSubmitFail: PropTypes.func,
    autoComplete: PropTypes.bool,
    mapPropsToFields: PropTypes.func,
    onFieldsChange: PropTypes.func,
    onValueChange: PropTypes.func,
    className: PropTypes.string,
  };

  static defaultProps = {
    autoComplete: false,
  };

  submit(e) {
    e.preventDefault();
    this.props.form.validateFields((err, values) => {
      if (err) {
        runFunc(this.props.onSubmitFail, err);
      } else {
        runFunc(this.props.onSubmit, values);
      }
    });
  }

  render() {
    const childrenAfterFilter = Array.isArray(this.props.children) ?
      (this.props.children || []).filter(child => React.isValidElement(child))
      :
      [this.props.children];
    const mainChildren = React.Children.map(
      childrenAfterFilter,
      child => React.cloneElement(child, { form: this.props.form })
    );

    return (
      <AntdForm
        className={this.props.className}
        autoComplete={this.props.autoComplete ? 'on' : 'off' }
        onSubmit={this.submit}
      >
        {mainChildren}
      </AntdForm>
    )
  }
}

const NewForm = AntdForm.create({
  mapPropsToFields(props) {
    return runFunc(props.mapPropsToFields);
  },
  onFieldsChange(props, fields) {
    runFunc(props.onFieldsChange, fields);
  },
  onValuesChange(props, values) {
    runFunc(props.onValueChange, values);
  },
})(_Form);

class Form extends React.Component {
  constructor(props) {
    super(props);
    this.getForm = this.getForm.bind(this);
    this.getFieldDecorator = this.getFieldDecorator.bind(this);
    this.setFieldsValue = this.setFieldsValue.bind(this);
    this.getFieldsValue = this.getFieldsValue.bind(this);
    this.getFieldValue = this.getFieldValue.bind(this);
    this.validateFields = this.validateFields.bind(this);
    this.resetFields = this.resetFields.bind(this);
    this.submitFields = this.submitFields.bind(this);

    this.form = null;
  }

  getForm() {
    return this.form;
  }

  getFieldDecorator(...args) {
    this.form.getFieldDecorator(...args);
  }

  setFieldsValue(payload) {
    this.form.setFieldsValue(payload);
  }

  getFieldsValue(fieldKeyArr) {
    return this.form.getFieldsValue(fieldKeyArr);
  }

  getFieldValue(fieldKey) {
    return this.form.getFieldValue(fieldKey);
  }

  validateFields(...arg) {
    return this.form.validateFields(...arg);
  }

  resetFields() {
    this.form.resetFields();
  }

  submitFields(...arg) {
    this.form.validateFields({ force: true },(err, values) => {
      if (err) {
        runFunc(this.props.onSubmitFail, err);
      } else {
        runFunc(this.props.onSubmit, values, ...arg);
      }
    });
  }

  render() {
    return (
      <NewForm wrappedComponentRef={c => this.form = c.props.form} {...this.props}/>
    );
  }
}

Form.Item = Item;

export default Form;
