/**
 * 单行文本输入框，基于antd的Input，默认为大号，可能要实现一些格式化的输入效果（待完善）
 * @property {string} [size] - 大小，'small'|'xSmall'
 * @property {boolean} [wordCount] - 是否需要字数统计
 * @property {boolean} [isTrim] - 是否清空字数所有空格
 * @property {number} [maxLength] - 字数上限
 * @property {Object} [validator] - 校验器的配置参数
 */
import React from 'react'
import { Input as AntdInput } from 'antd'
import styles from './styles.less'
import Validator from '../Validator'
import classnames from 'classnames'

const UNSELECTABLE_STYLE = {
  userSelect: 'none',
  WebkitUserSelect: 'none',
}
const UNSELECTABLE_ATTRIBUTE = {
  unselectable: 'unselectable',
}
class Input extends React.Component {
  constructor(props) {
    super(props);
    this.handleChange = this.handleChange.bind(this);
  }
  focus() {
    this.input.focus();
  }
  blur() {
    this.input.blur();
  }
  saveInput = (node) => {
    this.input = node;
  }
  renderClear() {
    const { disabled } = this.props
    const { value } = this.props
    const cls = {
      [`Input-clear`]: 1,
      [`Input-clear-show`]: !disabled  && value ,
    }
    return (
      <span
        key="clear"
        style={UNSELECTABLE_STYLE}
        {...UNSELECTABLE_ATTRIBUTE}
        className={classnames(cls)}
        onClick={this.onClearInput}
      />
    )
  }

  onClearInput = () => {
    const e = {
      target: {
        value : null
      }
    }
    this.props.onChange(e)
  }
  handleChange(ele) {
    const { isTrim } = this.props
    let val = ele.target.value;
    if (val && val !== '') {
      // 过滤emoji
      ele.target.value = val.replace(/\uD83C[\uDF00-\uDFFF]|\uD83D[\uDC00-\uDE4F]/g, '');
      // 过滤空格
      if(isTrim){
        ele.target.value = val.replace(/\s/g, '');
      }
    }
    this.props.onChange(ele);
  }
  render() {
    const { size: sizeProp, validator, wordCount, style, className, maxLength, onChange, allowClear, ...props } = this.props
    let size = 'large'
    if (sizeProp === 'small') {
      size = 'default'
    } else if (sizeProp === 'xSmall') {
      size = 'small'
    }

    const cls = {
      Input: 1,
      [className]: !!className,
    }

    let input = <AntdInput
      size={size}
      maxLength={maxLength !== undefined && maxLength !== null ? (maxLength + '') : undefined}
      onChange={this.handleChange}
      {...props}
      ref={this.saveInput}/>;
    if (wordCount) {
      input = (
        <div className="Input-wordCount-wrap">
          {/* <AntdInput size={size} {...{...props, onChange:this.getValue}} /> */}
          {input}
          <div className="Input-wordCount"><span>{(props.value !== undefined && props.value !== null) ? props.value.length : 0}</span>/{maxLength || 20}</div>
        </div>
      )
    }
    if (allowClear) {
      input = (
        <div className="Input-clear-wrap">
          {input}
          {this.renderClear()}
        </div>
      )
    }
    input = (
      <div className={classnames(cls)} style={style}>
        {input}
      </div>
    )
    if (validator) {
      input = (
        <Validator value={this.props.value} {...validator}>
          {input}
        </Validator>
      )
    }

    return input
  }
}

export default Input
