import React, { cloneElement } from 'react'
import { findDOMNode } from 'react-dom'
import toArray from 'rc-util/lib/Children/toArray'
import Menu from 'rc-menu'
import scrollIntoView from 'dom-scroll-into-view'
import { saveRef } from '../utils'

export default class DropdownMenu extends React.Component {

  constructor(props) {
    super(props)
  }

  componentDidMount() {
    this.scrollActiveItemToView()
  }

  componentDidUpdate(prevProps) {
    const props = this.props
    if (!prevProps.visible && props.visible) {
      this.scrollActiveItemToView()
    }
  }

  scrollActiveItemToView = () => {
    const itemComponent = findDOMNode(this.firstActiveItem)
    if (itemComponent) {
      const scrollIntoViewOpts = {
        onlyScrollIfNeeded: true,
      }

      scrollIntoView(
        itemComponent,
        findDOMNode(this.menuRef),
        scrollIntoViewOpts
      )
    }
  }

  render() {
    const props = this.props
    const {
      menuItems,
      prefixCls,
      onMenuSelect,
    } = props
    if (menuItems && menuItems.length) {
      let clonedMenuItems = menuItems
      return (
        <Menu
          ref={saveRef(this, 'menuRef')}
          defaultActiveFirst={true}
          onClick={obj => onMenuSelect(obj.item.props.value, obj.item.props.children)}
          prefixCls={`${prefixCls}-menu`}
        >
          {clonedMenuItems}
        </Menu>
      )
    }
    return null
  }
}
