
import React from 'react'
import Input from '../Input'
import InputSelectTrigger from './components/InputSelectTrigger'
import styles from './styles.less'
import classnames from 'classnames'
import { Item as MenuItem } from 'rc-menu'
import { UNSELECTABLE_STYLE, UNSELECTABLE_ATTRIBUTE, saveRef } from './utils'
import { findFirstParent } from '../utils/dom'
import { propTypes, defaultProps } from './propType'
import ReactDOM from 'react-dom'

const prefixCls = 'InputSelect'

class InputSelect extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      open: true
    }
  }
  static defaultProps = defaultProps
  static propTypes = propTypes

  componentDidUpdate(prevProps, prevState) {
    if (!this.state.open && prevState.open) {
    }
  }

  setOpenState = (open) => {
    const { props, state } = this
    this.setState({ open })
  }

  getPopupContainer = () => {
    const $root = ReactDOM.findDOMNode(this.inputRef)
    return (
      this.props.getPopupContainer() ||
      findFirstParent($root, '.ant-modal-wrap') ||
      findFirstParent($root, '.Page-scroller') ||
      document.body
    )
  }

  onMenuSelect = (val, text) => {
    this.setOpenState(false)
    ReactDOM.findDOMNode(this.inputRef).blur()
    this.props.onSelect && this.props.onSelect({val, text})
  }

  onInput = (e) => {
    const { onChange } = this.props
    onChange && onChange(e)
  }
  onInputBlur = () => {
    this.setOpenState(false)
    this.props.onBlur && this.props.onBlur()
  }
  onInputFocus = () => {
    this.setOpenState(true)
    this.props.onFocus && this.props.onFocus()
  }
  getOptionsItem = (list) => {
    if(Array.isArray(list) && list.length){
      return (
        list.map(({val, text, disabled}) => (
          <MenuItem
            style={UNSELECTABLE_STYLE}
            attribute={UNSELECTABLE_ATTRIBUTE}
            value={val}
            key={val}
            disabled={disabled}
            title={text}
          >
            {text}
          </MenuItem>
        ))
      )
    }else {
      return (
        [
          <MenuItem
            style={UNSELECTABLE_STYLE}
            attribute={UNSELECTABLE_ATTRIBUTE}
            disabled
            value="NOT_FOUND"
            key="NOT_FOUND"
          >
            {this.props.noOptionsTips || '暂无数据'}
          </MenuItem>
        ]
      )
    }
  }
  render() {
    const { onSelect, list, className, dropDownSize, dropdownStyle, dropdownClassName, value, showSelect, getPopupContainer, noOptionsTips, ...extraInputProps } = this.props
    const { open } = this.state
    const cls = {
      SelectInput: 1,
      [className]: !!className,
    }
    const InputSelectTriggerShow = Boolean(open && value && showSelect);

    return (
      <InputSelectTrigger
        prefixCls={prefixCls}
        menuItems={this.getOptionsItem(list)}
        visible={InputSelectTriggerShow}
        onDropdownVisibleChange={this.onDropdownVisibleChange}
        getPopupContainer={this.getPopupContainer}
        onMenuSelect={this.onMenuSelect}
        ref={saveRef(this, 'inputSelectTriggerRef')}
        dropdownClassName={dropdownClassName}
        dropdownStyle={dropdownStyle}
        size={dropDownSize}
      >

        <Input
          value={value}
          ref={saveRef(this, 'inputRef')}
          {...extraInputProps}
          onBlur={this.onInputBlur}
          onFocus={this.onInputFocus}
          onChange={this.onInput}
        />
      </InputSelectTrigger>
    )
  }
}

export default InputSelect
