import React from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';
import style from './styles.module.less';
import PopConfirm from 'COMPONENT/PopConfirm';
import { row, column } from './config';

const ActionBtn = (props) => {
  return (
    props.disabled ?
      <a className={classNames(style.ActionBtn, style.disabled)}>{props.children}</a>
      :
      <a className={classNames(style.ActionBtn)} onClick={props.onClick}>{props.children}</a>

  );
};

const ListAction = (props) => {
  return (
    <div className={classNames(style.ListAction, style[props.direction])}>
      {
        props.config.map((cfg) => {
          if (cfg.needConfirm) {
            return (
              <PopConfirm
                content={cfg.confirmTips || "确定吗？"}
                onOk={cfg.onClick}
                key={cfg.text}>
                <ActionBtn disabled={cfg.disabled} key={cfg.text}>{cfg.text}</ActionBtn>
              </PopConfirm>
            );
          } else {
            return (
              <ActionBtn onClick={cfg.onClick} disabled={cfg.disabled} key={cfg.text}>{cfg.text}</ActionBtn>
            );
          }
        })
      }
    </div>
  );
};

ListAction.propTypes = {
  config: PropTypes.arrayOf(PropTypes.shape({
    text: PropTypes.string.isRequired,
    onClick: PropTypes.func.isRequired,
    disabled: PropTypes.bool,
    needConfirm: PropTypes.bool,
    confirmTips: PropTypes.string,
  })).isRequired,
  direction: PropTypes.oneOf([row, column]),
};
ListAction.defaultProps = {
  direction: row,
}

export default ListAction;
