import React from 'react'
import { Modal as AntdModal } from 'antd'
import PopTip from '../PopTip'
import PopConfirm from '../PopConfirm'
import Loading from '../Loading'
import Button from '../Button'
import ErrorTips from '../ErrorTips'
import './styles.less'
import classnames from 'classnames'
import PropTypes from 'prop-types'
import { SIZE_S, SIZE_M, SIZE_L, SIZE_XL, COLORS } from './config.js';

class Modal extends React.Component {

  static propTypes = {
    footerTipText: PropTypes.oneOfType([
      PropTypes.string,
      PropTypes.element,
    ]),
  };
  static defaultProps = {
    closable: true,
    okText: '确定',
    cancelText: '取消',
    loadingTip: '',
    themeConfig: {
      bgc: COLORS.white,
      bodyBgc: COLORS.white,
      footerTopShadow: false,
    },
    maskClosable: false
  }

  constructor(props) {
    super(props);
    this.state = {
      okTips: null,  //确定按钮提示信息
      okTipsVisible: false,  //确定按钮提示信息显示状态
      $container: document,  //承载弹框浮层的DOM元素
    };
    this.sizeConfig = {
      default: SIZE_L,
      s: SIZE_S,
      S: SIZE_S,
      m: SIZE_M,
      M: SIZE_M,
      l: SIZE_L,
      L: SIZE_L,
      xl: SIZE_XL,
      XL: SIZE_XL,
    };
    this.setModalWidth = this.setModalWidth.bind(this);
  }

  componentDidMount() {
    const modalWraps = document.querySelectorAll('.Modal')
    const $container = modalWraps[modalWraps.length - 1]
    this.setState({$container})
  }

  setModalWidth(width, widthSize) {
    if (widthSize) {
      return this.sizeConfig[widthSize] || this.sizeConfig.default;
    }
    return width || this.sizeConfig.default;
  }

  showOkTips = (text) => {
    this.setState({ okTips: text, okTipsVisible: true })
  }

  onOkTipsVisibleChange = (visible) => {
    if (!visible) {
      this.setState({ okTipsVisible: visible })
    }
  }

  onCancel = () => {
    if (this.props.cancelConfirm) {
      this._cancelConfirm.show()
    } else {
      this.cancel()
    }
  }

  cancel = () => {
    const { onCancel, onClose } = this.props
    if (onCancel) {
      onCancel()
    } else if (onClose) {
      onClose()
    }
  }

  close = () => {
    const { onCancel, onClose } = this.props
    if (onClose) {
      onClose()
    } else if (onCancel) {
      onCancel()
    }
  }

  render() {
    const { children, loading, loadingTip, okBtnDisabled, okBtnShow: okBtnShowProp, cancelBtnShow: cancelBtnShowProp,
      okText, cancelText, onOk, onCancel, onClose, closable, footer: footerProp, className, noLine, cancelConfirm,
      error, fixedLoading, toggle, visible, themeConfig, width, widthSize, footerTipText, showConfirmTip, confirmTip, maskClosable, ...props  } = this.props
    const { okTips, okTipsVisible, $container } = this.state
    const okBtnShow = okBtnShowProp === undefined ? true : okBtnShowProp
    const cancelBtnShow = cancelBtnShowProp === undefined ? true : cancelBtnShowProp
    const {
      content: cancelConfirmContent,
      okBtnText: cancelConfirmOkBtnText,
      cancelBtnText: cancelConfirmCancelBtnText
    } = cancelConfirm || {}
    const _toggle = toggle?visible:true
    let cancelBtn = <Button onClick={this.onCancel}>{cancelText}</Button>
    if (cancelConfirm) {
      cancelBtn = (
        <PopConfirm
          content={cancelConfirmContent}
          okBtnText={cancelConfirmOkBtnText}
          cancelBtnText={cancelConfirmCancelBtnText}
          onOk={this.cancel}
          ref={c => this._cancelConfirm = c}
          getPopupContainer={() => $container}
        >
          {cancelBtn}
        </PopConfirm>
      )
    }
    const footer = (footerProp !== undefined ? footerProp : (
      <div className="footerWrapper">
        {
          okBtnShow && !showConfirmTip &&
          <PopTip content={okTips} visible={okTipsVisible} onVisibleChange={this.onOkTipsVisibleChange} getPopupContainer={() => $container}>
            <Button type="primary" onClick={onOk} disabled={okBtnDisabled || loading}>{okText}</Button>
          </PopTip>
        }
        {
          okBtnShow && showConfirmTip &&
          <PopConfirm
          content={confirmTip}
          okBtnText='确定'
          cancelBtnText='取消'
          onOk={onOk}
          ref={c => this._cancelConfirm = c}
          getPopupContainer={() => $container}
        >
          <Button type="primary" disabled={okBtnDisabled || loading}>{okText}</Button>
        </PopConfirm>
        }
        {
          cancelBtnShow &&
          cancelBtn
        }
        {
          footerTipText &&
          <span className="footerTipText">{footerTipText}</span>
        }
      </div>
    ))

    const cls = {
      Modal: 1,
      'Modal-noLine': noLine,
      'Modal-fixedLoading': !!fixedLoading,
      'Modal-theme-bgc-white': themeConfig.bgc === COLORS.white,
      'Modal-theme-bgc-gray': themeConfig.bgc === COLORS.gray,
      'Modal-theme-body-bgc-white': themeConfig.bodyBgc === COLORS.white,
      'Modal-theme-body-bgc-gray': themeConfig.bodyBgc === COLORS.gray,
      'Modal-theme-footer-top-shadow': themeConfig.footerTopShadow,
      [className]: !!className,
    }

    return (
      <AntdModal
        visible={_toggle}
        maskClosable={maskClosable}
        wrapClassName={classnames(cls)}
        onCancel={this.close}
        footer={footer}
        closable={(onCancel || onClose) && closable}
        zIndex={900}
        width={this.setModalWidth(width, widthSize)}
        {...props}
        ref={c => this.modal = c}
      >
        {children}
        <Loading loading={loading} tip={loadingTip}/>
        {error &&
        <div className="Modal-error">
          <ErrorTips text={error}/>
        </div>
        }
      </AntdModal>
    )
  }
}

export default Modal
