import React from 'react';
import PropTypes from 'prop-types';
import Modal from '../../Modal';
import Button from '../../../Button';
import style from './styles.module.less';
import {
  runFunc,
} from '../../../utils/common';

class ModalCountdown extends React.Component {
  constructor(props) {
    super(props);

    this.updateCountdown = this.updateCountdown.bind(this);
    this.removeInterval = this.removeInterval.bind(this);
    this.onOk = this.onOk.bind(this);

    this.defaultCount = Math.floor(Math.abs(props.maxCount));

    this.state = {
      count: this.defaultCount,
      btnDisabled: this.defaultCount > 0,
    };

    this.interval = null;
  }

  static propTypes = {
    title: PropTypes.string,
    maxCount: PropTypes.number,
    buttonText: PropTypes.string,
    content: PropTypes.node,
    onOk: PropTypes.func,
    close: PropTypes.func.isRequired,
    widthSize: PropTypes.oneOf([ 'S', 'M', 'L', 'XL' ]),
  };

  static defaultProps = {
    title: '提示',
    maxCount: 10,
    buttonText: '确定',
    widthSize: 'S',
  };

  componentDidMount() {
    if (this.defaultCount > 0) {
      this.interval = setInterval(this.updateCountdown, 1000);
    }
  }

  componentWillUnmount() {
    this.removeInterval();
  }

  updateCountdown() {
    if (this.state.count === 0) {
      return this.removeInterval();
    }

    this.setState((prevState) => {
      prevState.count--;
      if (prevState.count === 0) {
        prevState.btnDisabled = false;
      }
      return prevState;
    });
  }

  removeInterval() {
    if (this.interval) {
      clearInterval(this.interval);
    }
  }

  onOk() {
    runFunc(this.props.onOk);
  }

  render() {
    return (
      <Modal
        title={this.props.title}
        widthSize={this.props.widthSize}
        onCancel={this.props.close}
        footer={
          <div className={style.footerWrap}>
            <Button onClick={this.onOk} type="primary" disabled={this.state.btnDisabled}>
              <span>{this.props.buttonText}</span>
              {this.state.count > 0 && (
                <span className={style.countNum}>{this.state.count}s</span>
              )}
            </Button>
          </div>
        }
      >
        {this.props.content}
      </Modal>
    )
  }
}

export default ModalCountdown;