import React from 'react';
import ReactDOM from 'react-dom';
import Modal from './Modal';
import {
  runFunc,
} from '../../utils/common';

/**
 * countdown
 *
 * @param config {Object}
 * @param config.title {string?}
 * @param config.buttonText {string?}
 * @param config.maxCount {number?}
 * @param config.content {'ReactNode'}
 * @param config.onOk {function?}
 * @param config.onCancel {function?}
 * @param config.widthSize {'S' | 'M' | 'L' | 'XL'?}
 *
 * @return instance {Object}
 * @return instance.destroy {function}
 * */
export default function countdown(config = {}) {
  const div = document.createElement('div');
  document.body.appendChild(div);

  let currentConfig = {
    ...config,
    close,
    onOk,
  };

  function close() {
    const unmountResult = ReactDOM.unmountComponentAtNode(div);
    if (unmountResult && div.parentNode) {
      div.parentNode.removeChild(div);
    }
    runFunc(config.onCancel);
  }

  function onOk() {
    runFunc(config.onOk);
    close();
  }

  function render(props) {
    ReactDOM.render(<Modal{...props}/>, div);
  }

  render(currentConfig);

  return {
    destroy: close,
  };
};