import React from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';
import style from './styles.module.less';
import PopoverPlus from 'COMPONENT/PopoverPlus';
import { runFunc } from "../utils/common";

const classPrefix = 'PopBtnGroups';

const BtnGroups = (props) => {
  return (
    <div className={style[`${classPrefix}-BtnGroups`]}>
      {
        props.groups.map((item, index) => {
          return item.disabled ?
            <div className={style[`${classPrefix}-BtnGroups-disabled`]} key={index}>{item.name}</div>
            :
            <div className={style[`${classPrefix}-BtnGroups-btn`]}
                 onClick={() => {
                   runFunc(item.onClick, props.data);
                   props.afterClick();
                 }}
                 key={index}>
              {item.name}
            </div>
        })
      }
    </div>
  );
};

class PopBtnGroups extends React.Component {
  constructor(props) {
    super(props);
    this.closePopover = this.closePopover.bind(this);
    this.state = {};
    this.$popover = null;
  }

  static propTypes = {
    groups: PropTypes.arrayOf(PropTypes.shape({
      name: PropTypes.oneOfType([PropTypes.string, PropTypes.element]),
      onClick: PropTypes.func,
      disabled: PropTypes.bool,
    })),
    data: PropTypes.any,
    popoverProps: PropTypes.any,
  };

  closePopover() {
    runFunc(this.$popover && this.$popover.close);
  }

  render() {
    return (
      <PopoverPlus
        ref={c => this.$popover = c}
        className={classNames(style[classPrefix], this.props.className)}
        style={this.props.style}
        content={
          <BtnGroups groups={this.props.groups} data={this.props.data} afterClick={this.closePopover}/>
        }
        popoverProps={this.props.popoverProps}
      >
        {this.props.children}
      </PopoverPlus>
    )
  }
}

export default PopBtnGroups;
