import React from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';
import style from './styles.module.less';
import { Popover } from 'antd';

const classPrefix = 'PopoverPlus';

class PopoverPlus extends React.Component {
  constructor(props) {
    super(props);
    this.switchVisible = this.switchVisible.bind(this);
    this.close = this.close.bind(this);
    this.state = {
      visible: false,
    };
  }

  static propTypes = {
    content: PropTypes.oneOfType([PropTypes.element, PropTypes.node]),
    initialVisible: PropTypes.bool,
    className: PropTypes.string,
    style: PropTypes.object,
    trigger: PropTypes.string,
    popoverProps: PropTypes.object,
  };

  static defaultProps = {
    initialVisible: false,
    trigger: 'click',
    popoverProps: {},
  };

  componentDidUpdate(prevProps, prevState, snapshot) {
    if (this.props.initialVisible && (this.props.initialVisible !== prevProps.initialVisible)) {
      this.setState({ visible: true });
    }
  }

  switchVisible() {
    this.setState((prevState) => ({ visible: !prevState.visible }));
  }

  close() {
    this.setState({ visible: false });
  }

  render() {
    return (
      <Popover
        overlayClassName={classNames(style[classPrefix], this.props.className)}
        overlayStyle={this.props.style}
        content={this.props.content}
        trigger={this.props.trigger}
        visible={this.state.visible}
        onVisibleChange={this.switchVisible}
        {...this.props.popoverProps}>
        {this.props.children}
      </Popover>
    )
  }
}

export default PopoverPlus;
