import React from 'react';
import styles from './styles.module.less';
import PropTypes from 'prop-types';
import classNames from 'classnames';
import { Popover } from 'antd';

const PopoverWrap = (props) => {
  const Children = props.children;

  return (props.visible && props.content) ? (
    <Popover
      content={props.content}
      overlayClassName={classNames(
        styles.popoverWrap,
        props.overlayClassName,
        styles[`${props.type}${props.deepBackground ? 'Deep' : ''}`],
        { [styles.deep]: props.deepFont }
      )}
      overlayStyle={props.overlayStyle}
    >
      {Children}
    </Popover>
  ) : (
    Children
  );
};

PopoverWrap.propTypes = {
  children: PropTypes.node.isRequired,
  visible: PropTypes.bool,
  content: PropTypes.node.isRequired,
  type: PropTypes.oneOf(['default', 'success', 'info', 'warning', 'danger']),
  overlayClassName: PropTypes.string,
  overlayStyle: PropTypes.object,
  deepFont: PropTypes.bool,
  deepBackground: PropTypes.bool,
};

PopoverWrap.defaultProps = {
  visible: false,
  type: 'default',
  deepFont: true,
  deepBackground: true,
};

export default PopoverWrap;