import React from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';
import style from './index.less';

const winQRCode = window.QRCode;

const classPrefix = 'QrCode';

class QrCode extends React.Component {
  constructor(props) {
    super(props);
    this.qrCodeObj = null;
  }

  static propTypes = {
    id: PropTypes.string.isRequired,
    text: PropTypes.string.isRequired,
    visible: PropTypes.bool,
    width: PropTypes.number,
    height: PropTypes.number,
    className: PropTypes.string,
    style: PropTypes.object,
  };

  static defaultProps = {
    visible: true,
    width: 100,
    height: 100,
    className: '',
    style: null,
  };

  static initQrCode(DOMId, text, width, height) {
    return new winQRCode(document.querySelector(`#${DOMId}`), {
      text,
      width,
      height,
      colorDark: "#000000",
      colorLight: "#ffffff",
      correctLevel: winQRCode.CorrectLevel.Q
    });
  };

  componentDidMount() {
    this.qrCodeObj = QrCode.initQrCode(this.props.id, this.props.text, this.props.width, this.props.height);
    document.querySelector(`#${this.props.id}`).title = '';
  }

  componentWillUnmount() {
    this.qrCodeObj = null;
  }

  render() {
    return (
      <div
        id={this.props.id}
        className={classNames(
          style[classPrefix],
          { hidden: !this.props.visible },
          this.props.className,
        )}
        style={this.props.style}/>
    )
  }
}

export default QrCode;
