import React from 'react';
import {Icon} from 'antd';
import styles from './styles.less';
import {objectDeepEquals} from "../../utils/basic/obj";

const UNLIMITED_TEXT = '不限';
const UNLIMITED_VAL = '';
const UNLIMITED_OBJ = {text: UNLIMITED_TEXT, val: UNLIMITED_VAL};
const is = (type, val) => Object.prototype.toString.call(val) === ('[object ' + type + ']');

class Options extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      showMoreBtn: false,
      expend: false,
      dict: this.getDict(props),
    }
  }

  componentDidMount() {
    this.updateMoreBtnShowStatus()
  }

  componentWillReceiveProps(nextProps) {
    if (!objectDeepEquals(this.props.dict, nextProps.dict)) {
      this.setState({ dict: this.getDict(nextProps) })
    }
  }

  componentDidUpdate(prevProps) {
    if (!objectDeepEquals(this.props.dict, prevProps.dict)) {
      this.updateMoreBtnShowStatus()
    }
  }

  getDict = props => {
    const { dict, hideUnlimited } = props
    let newDict = dict ? [...dict] : []
    if (!dict.find(val => val.val === UNLIMITED_VAL) && !hideUnlimited) {
      newDict.unshift(UNLIMITED_OBJ)
    }
    return newDict
  }

  updateMoreBtnShowStatus = () => {
    this.setState({
      showMoreBtn: this.$inner.offsetHeight > 50
    })
  }

  showOrHidden = () => {
    let {expend} = this.state;
    let $items = this.$inner.parentNode;
    this.setState({expend: !expend}, () => {
      if (this.state.expend) {
        $items.style.height = this.$inner.offsetHeight + 'px';
        $items.parentNode.querySelector('.Search-label').style.height = this.$inner.offsetHeight + 'px';
      } else {
        $items.style.height = '50px';
        $items.parentNode.querySelector('.Search-label').style.height = '50px';
      }
    });
  }

  multiSelect = (value, val, isClose) => {
    let {onChange, dict, hideUnlimited} = this.props;
    // 如果是字符串先转成数组
    if (is('String', value)) {
      value = value === '' ? [] : value.split(',');
    }
    // 反选
    if (isClose && value.indexOf(val) !== -1) {
      for (let i = 0; i < value.length; i++) {
        if (value[i] === val) {
          value.splice(i, 1);
          break;
        }
      }
      // 当反选到无选中项时 需选中“不限”
      value.length === 0 && !hideUnlimited && value.push(UNLIMITED_VAL);
    }
    // 选中
    else {
      // 如果选中“不限” 需要把其他选项清空
      if (val === UNLIMITED_VAL) {
        value = null;
      } else {
        // 当点击选项 如果存在“不限”需清除
        value = value.filter(v => v !== UNLIMITED_VAL);
        value.push(val);
        // 如果全选 要回到“不限”
        dict.filter(v => v.val !== UNLIMITED_VAL).length === value.length && !hideUnlimited && (value = [UNLIMITED_VAL])
      }
    }
    // 回调
    onChange(value && value.length > 0 ? value : null);
  }

  render() {
    let {isMultiSelect, onChange, value, hideUnlimited} = this.props;
    let {showMoreBtn, expend, dict} = this.state;
    // 当没有“不限”选项才加

    return (
      <div ref={e => {this.$inner = e; return this.$inner}} className={'Search-clearfix'} style={{ paddingBottom: 10 }} >
        {isMultiSelect &&
        <div>
          <div className={'Search-item multiSelect'}>
            可多选
          </div>
          {
            dict.map((item, key) => {
              const isActive = value !== undefined && value.indexOf(item.val) !== -1
              return (<div
                className={`Search-item ${item.val === '' ? 'noClose' : ''} ${isActive ? 'filter-active' : ''}`}
                key={key}
                onClick={isActive ? null : () => this.multiSelect(value, item.val)}>
                {item.text}
                {
                  value !== undefined && value.indexOf(item.val) !== -1 && item.val !== '' &&
                  <Icon className='icon' type='close' onClick={(e) => {
                    e.stopPropagation()
                    this.multiSelect(value, item.val, true)
                  }
                  } />
                }
              </div>)
            })
          }
        </div>
        }
        {!isMultiSelect &&
        <div>
          {
            dict.map((item, key) => {
              return (<div className={`Search-item noClose ${value === item.val || ( !value && !item.val ) ? 'filter-active' : ''}`}
                           key={key}
                           onClick={value === item.val ? null : () => onChange(item.val === UNLIMITED_VAL ? null : item.val)}>
                {item.text}
                {/* 单选不需要X按钮 */}
                {/*{*/}
                {/*JSON.stringify(value) === JSON.stringify(item) &&*/}
                {/*<Icon type='close'/>*/}
                {/*}*/}
              </div>)
            })
          }
        </div>
        }
        {showMoreBtn &&
        <div onClick={this.showOrHidden} className={'Search-showMore'}>
          <span>{!expend ? '更多' : '收起'}</span>
          <Icon type={expend ? 'up' : 'down'}/>
        </div>
        }
      </div>
    )
  }
}

export default Options;
