import React from 'react'
import { MULTI_TYPE } from "./common"
import Option from './Option'
import { Icon } from 'antd'

/**
 * 判断是否数组类型
 * @param target
 * @returns {boolean}
 */
const isArray = (target) => {
  return Object.prototype.toString.call(target).indexOf('Array') > -1
}

export default class extends React.PureComponent {
  static defaultProps = {
    hadOptionAll: true
  };

  defaultAll = {
    val: null,
    text: '全部'
  }

  state = {
    showMore: false
  }

  componentDidMount () {
    this.computedRow();
    window.addEventListener('resize', this.computedRow, false);
  }

  componentDidUpdate () {
    this.computedRow();
  }

  /**
   * 计算每行更多按钮的位置
   */
  computedRow = () => {
    const { showMoreButtons } = this.props
    if (!showMoreButtons || !this.ulDom) return
    let liDom = []
    const rowWidth = this.ulDom.offsetWidth
    let ulWidth = 0
    liDom = this.ulDom.getElementsByTagName('li')
    for (let i = 0, len = liDom.length; i < len; i += 1) {
      ulWidth += liDom[i].offsetWidth + 10
      if (ulWidth > rowWidth) {
        this.moreDom.style.display = ''
        this.moreDom.style.left = `${ulWidth - liDom[i].offsetWidth - 10}px`;
        return
      }
    }
    this.moreDom.style.display = 'none'
  }

  /**
   * 取消选择
   * @param val
   */
  unSelect = (val) => {
    const { onRowChange, rowIndex, type, value } = this.props
    if (type === MULTI_TYPE) {
      //如果是多选，要生成一个新的value数组
      const newValue = value.filter(item => item !== val)
      onRowChange(newValue, rowIndex)
    } else {
      onRowChange(val, rowIndex)
    }
  }

  /**
   * 选择
   * @param val
   */
  onSelect = (val) => {
    const { onRowChange, rowIndex, type, value } = this.props
    if (type === MULTI_TYPE) {
      //如果是多选，要生成一个新的子value数组
      const newValue = val === null ? [] : [...value, val]
      onRowChange(newValue, rowIndex)
    } else {
      onRowChange(val, rowIndex)
    }
  }

  /**
   * 判断dict中是否包含value值的项
   * @returns {*}
   */
  valueInDict = () => {
    const { value, type, dict } = this.props
    if (type === MULTI_TYPE) {
      if (isArray(value)) {
        return value.some(item => (~dict.findIndex(dt => dt.val === item)))
      }
    } else {
      return (~dict.findIndex(dt => dt.val === value))
    }
  }

  /**
   * 判断该项是否选中，一项都没有，则默认选中全部
   * @param val
   * @param option
   * @returns {*}
   */
  isActive = (val, option) => {
    const { value, type } = this.props
    if (this.valueInDict()) {
      if (type === MULTI_TYPE) {
        if (isArray(value)) {
          return (~value.indexOf(val))
        }
      } else {
        return val === value
      }
    } else {
      return option === this.defaultAll
    }
  }

  handelMoreButton = () => {
    this.setState({ showMore: !this.state.showMore })
  }

  render () {
    const { label, dict, type, hadOptionAll, showMoreButtons } = this.props
    const { showMore } = this.state

    //默认有一个全部选项
    const options = hadOptionAll ? [this.defaultAll, ...dict] : dict;

    const maxHeight = !showMoreButtons ? '' : showMore ? '500px' : '30px'

    return (
      <div className="Search2-row" style={{ maxHeight }}>
        <div className="Search2-label">{label}</div>
        <div className="Search2-options">
          <ul ref={ref => this.ulDom = ref}>
            {
              options.map(option => {
                return (
                  <Option
                    type={type}
                    key={option.val}
                    val={option.val}
                    text={option.text}
                    onSelect={this.onSelect}
                    unSelect={this.unSelect}
                    isRealOption={option !== this.defaultAll}
                    isActive={this.isActive(option.val, option)}
                    visible={option.visible == null ? true : option.visible}
                    disabled={option.disabled == null ? false : option.disabled} />
                )
              })
            }
          </ul>
          {
            showMoreButtons && <span onClick={this.handelMoreButton} ref={ref => this.moreDom = ref} className='Search2-row_showMoreBtn'>
              {
                showMore ? (
                  <span>收起<Icon className='icon' type="up" /></span>
                ) : (
                  <span>展开<Icon className='icon' type="down" /></span>
                )
              }
            </span>

          }
        </div>
      </div>
    )
  }
}
