import React from 'react'
import { Icon } from 'antd'
import classnames from 'classnames'
import PropTypes from 'prop-types'

import './index.less'
import Row from './Row'

/**
 * 判断是否数组类型
 * @param target
 * @returns {boolean}
 */
const isArray = (target) => {
  return Object.prototype.toString.call(target).indexOf('Array') > -1
}

export default class extends React.Component {
  constructor(props) {
    super(props)
    this.state = {
      moreRowShow: false,
    }
    this.moreRowHeight = 0 //更多的总高度
    this.emptyArr = []
    this.caCheValue = props.config.reduce((acc, cur, index) => {
      return {
        ...acc,
        [cur.field]: cur.value
      }
    }, {})
  }
  static propTypes = {
    config: PropTypes.array.isRequired,
    onChange: PropTypes.func.isRequired,
    rowNum: PropTypes.number,
    defaultExpand: PropTypes.bool,
    showMoreButtons: PropTypes.bool // 是否启用每行更多模式
  };
  static defaultProps = {
    rowNum: 4,
    defaultExpand: false,
    showMoreButtons: false
  };

  componentDidMount () {
    if (this.props.defaultExpand) {
      this.moreRowHeight = this.getMoreRowHeight();
      this.setState({ moreRowShow: true });
    }
  }

  //获取更多筛选项dom
  getMoreRowRef = (moreRowRef) => {
    if (moreRowRef) {
      this.moreRowRef = moreRowRef
    }
  }

  //获取更多筛选真实内容高度
  getMoreRowHeight = () => {
    const ulArr = Array.from(this.moreRowRef.querySelectorAll('ul'))
    return ulArr.reduce((acc, cur) => acc + cur.offsetHeight, 0)
  }

  toggleMoreRowShow = () => {
    this.moreRowHeight = this.getMoreRowHeight()
    this.setState({
      moreRowShow: !this.state.moreRowShow
    })
  }

  /**
   * 行的 val 改变后，通过field和value生成一个 keyValue 对象 ，通过onChange返回
   * @param val
   * @param rowIndex
   */
  onRowChange = (val, rowIndex) => {
    const { onChange, config } = this.props
    //生成keyValue
    const keyValue = config.reduce((acc, cur, index) => {
      if (index !== rowIndex) {
        return {
          ...acc,
          [cur.field]: cur.value
        }
      } else {
        return {
          ...acc,
          [cur.field]: val
        }
      }
    }, {})
    this.caCheValue = keyValue
    onChange(keyValue)
  }


  /**
   * 提供业务可以通过实例获取值
   * @returns
   */
  getConfigValue = () => {
    return this.caCheValue
  }

  render () {
    const { moreRowShow } = this.state;
    const { mode, config, style, rowNum, showMoreButtons } = this.props;
    //是否有更多筛选
    const hasMoreRow = config.length > rowNum;

    //最外层类名
    const wrapClassNames = classnames({
      Search2: true,
      'Search2-bright': mode !== 'dark',
      'Search2-dark': mode === 'dark',
    })

    return (
      <div className={wrapClassNames} style={style}>
        {/*默认展开筛选项*/}
        <div className="Search2-defaultRow">
          {
            config.map((row, index) => {
              if (index <= rowNum - 1) {
                let dict = row.dict
                if (!isArray(dict)) {
                  dict = this.emptyArr
                  // console.warn(`传入筛选器的配置项 dict 必须为数组`)
                }
                return (
                  <Row
                    type={row.type}
                    label={row.label}
                    dict={dict}
                    value={row.value}
                    key={index}
                    rowIndex={index}
                    showMoreButtons={row.showMoreButtons || showMoreButtons}
                    onRowChange={this.onRowChange}
                    hadOptionAll={row.hadOptionAll}
                  />
                )
              }
            })
          }
        </div>
        {/*默认收纳筛选项*/}
        {
          hasMoreRow &&
          <div
            className="Search2-moreRow"
            ref={this.getMoreRowRef}
            style={moreRowShow ? { maxHeight: this.moreRowHeight } : {}}
          >
            <div style={{ marginTop: 12 }}>
              {
                config.map((row, index) => {
                  if (index > rowNum - 1) {
                    let dict = row.dict
                    if (!isArray(dict)) {
                      dict = this.emptyArr
                      // console.warn(`传入筛选器的配置项 dict 必须为数组`)
                    }
                    return (
                      <Row
                        type={row.type}
                        label={row.label}
                        dict={dict}
                        value={row.value}
                        key={index}
                        rowIndex={index}
                        showMoreButtons={showMoreButtons}
                        onRowChange={this.onRowChange}
                        hadOptionAll={row.hadOptionAll}
                      />
                    )
                  }
                })
              }
            </div>
          </div>
        }
        {/*更多按钮*/}
        {
          hasMoreRow &&
          <p className="Search2-showMoreBtn">
            <span onClick={this.toggleMoreRowShow}>
              更多筛选
              <Icon type={moreRowShow ? 'up' : 'down'} style={{ marginLeft: 6, verticalAlign: 'text-bottom' }} />
            </span>
          </p>
        }
      </div>
    )
  }
}
