import React from 'react';
import ReactDOM from 'react-dom';
import './index.less';
import PropTypes from 'prop-types';
import Trigger from 'rc-trigger';
import Button from 'COMPONENT/Button';
import classNames from "classnames";
import { saveRef, getGridWidth } from "./utils";
import { findFirstParent } from '../utils/dom'
import { classPrefix, BUILT_IN_PLACEMENTS, PLACEMENTS_KEYS, defaultItemWidth } from './config';
import Popup from './Popup';

const ItemWrap = Popup.ItemWrap;

class SearchButton extends React.Component {
  constructor(props) {
    super(props);
    this.getPopupContainer = this.getPopupContainer.bind(this);
    this.setPopupState = this.setPopupState.bind(this);
    this.switchPopup = this.switchPopup.bind(this);
    this.state = {
      popupVisible: false,
    };
  }
  static propTypes = {
    maxGridEachRow: PropTypes.number,
    girdConfig: PropTypes.arrayOf(PropTypes.number),
    onSearch: PropTypes.func.isRequired,
    searchDisabled: PropTypes.bool,
    searchBtnLoading: PropTypes.bool,
    onReset: PropTypes.func.isRequired,
    resetDisabled: PropTypes.bool,
    getPopupContainer: PropTypes.func,
    dropdownClassName: PropTypes.string,
    dropdownStyle: PropTypes.object,
    dropdownPlacement: PropTypes.oneOf(PLACEMENTS_KEYS),
    popupTransitionName: PropTypes.string,
  };
  static defaultProps = {
    maxGridEachRow: 5,
    girdConfig: [],
    searchDisabled: false,
    searchBtnLoading: false,
    resetDisabled: false,
    getPopupContainer: function () {},
    dropdownClassName: '',
    dropdownStyle: null,
    dropdownPlacement: 'bottomRight',
    popupTransitionName: `Dropdown-slide-up`,
  };

  getPopupContainer() {
    const $root = ReactDOM.findDOMNode(this.rootRef)
    return (
      this.props.getPopupContainer() ||
      findFirstParent($root, '.ant-modal-wrap') ||
      findFirstParent($root, '.Page-scroller') ||
      document.body
    )
  }

  setPopupState(popupVisible) {
    this.setState({ popupVisible })
  };

  switchPopup() {
    this.setState((prevState) => ({
      popupVisible: !prevState.popupVisible,
    }));
  }

  render() {
    const popupCls = {
      [`${classPrefix}-dropdown`]: true,
      [this.props.dropdownClassName]: Boolean(this.props.dropdownClassName),
    };

    return (
      <Trigger
        ref={saveRef(this, 'triggerRef')}
        showAction={['click']}
        hideAction={['click']}
        prefixCls="Dropdown"
        builtinPlacements={BUILT_IN_PLACEMENTS}
        popupPlacement={this.props.dropdownPlacement}
        popupClassName={classNames(popupCls)}
        popupStyle={this.props.dropdownStyle}
        popupTransitionName={this.props.popupTransitionName}
        onPopupVisibleChange={this.setPopupState}
        popupVisible={this.state.popupVisible}
        getPopupContainer={this.getPopupContainer}
        popup={
          <Popup
            maxWidth={getGridWidth(this.props.maxGridEachRow) + 40}
            onSearch={this.props.onSearch}
            onReset={this.props.onReset}
            searchBtnLoading={this.props.searchBtnLoading}
            searchDisabled={this.props.searchDisabled}
            resetDisabled={this.props.resetDisabled}>
            {
              this.props.children.map((ItemComponent, index) => (
                ItemComponent ? (
                  <ItemWrap
                    width={this.props.girdConfig[index] ? getGridWidth(this.props.girdConfig[index]) : defaultItemWidth}
                    key={index}>
                    {ItemComponent}
                  </ItemWrap>
                ) : null
              ))
            }
          </Popup>
        }>
        <div className={classPrefix}>
          <Button onClick={this.switchPopup} iconType="search_mc_h" type="default">筛选</Button>
        </div>
      </Trigger>
    )
  }
}

export default SearchButton;
