import React from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';
import { Form } from 'antd';
import style from "./styles.module.less";

const FormItem = Form.Item;

const classPrefix = 'SearchFieldForm';

const SearchFieldForm = (props) => {
  const span = props.span >= 1 ? props.span : 1;

  return (
    <div className={classNames(
      style[classPrefix],
      style[`span-${span}`],
      props.className
    )} style={props.style}>
      <FormItem help={null} {...props.formItemProps}>
        {
          props.form.getFieldDecorator(
            props.name,
            {
              initialValue: props.initialValue,
              rules: [{ required: props.required }],
              ...props.fieldDecoratorOpt,
            }
          )(
            props.children
          )
        }
      </FormItem>
    </div>
  );
};

SearchFieldForm.propTypes = {
  name: PropTypes.string,
  span: PropTypes.number,
  initialValue: PropTypes.any,
  required: PropTypes.bool,
  beforeSnapshot: PropTypes.any,
  children: PropTypes.any,
  className: PropTypes.string,
  style: PropTypes.object,
  formItemProps: PropTypes.object,
  fieldDecoratorOpt: PropTypes.object,
};

SearchFieldForm.defaultProps = {
  span: 1,
  initialValue: null,
  required: false,
  formItemProps: {},
  fieldDecoratorOpt: {},
};

export default SearchFieldForm;
