import React from 'react';
import PropTypes from 'prop-types';
import style from './styles.module.less';
import { ExpandHeight } from '../../Expand';
import Button from "../../Button";
import TextLink from '../../TextLink';
import { runFunc } from "../../utils/common";

const classPrefix = 'SearchFilter';

class SearchFilter extends React.Component {
  constructor(props) {
    super(props);
    this.openExpand = this.openExpand.bind(this);
    this.closeExpand = this.closeExpand.bind(this);
    this.search = this.search.bind(this);
    this.reset = this.reset.bind(this);
    this.submit = this.submit.bind(this);
    this.$expand = null;
  }

  static propTypes = {
    onSearch: PropTypes.func,
    onReset: PropTypes.func,
    truncationNum: PropTypes.number,
    searchLoading: PropTypes.bool,
    searchDisabled: PropTypes.bool,
  };

  static defaultProps = {
    truncationNum: 1,
    searchLoading: false,
    searchDisabled: false,
  };

  static stopPropagation(e) {
    e.stopPropagation();
  }

  openExpand() {
    runFunc(this.$expand.open);
  }

  closeExpand() {
    runFunc(this.$expand.close);
  }

  search() {
    this.closeExpand();
    runFunc(this.props.onSearch);
  }

  reset() {
    runFunc(this.props.onReset);
  }

  submit(e) {
    e.preventDefault();
    this.props.form.validateFields((err, fieldsValue) => {
      if (err) {
        return;
      }
      runFunc(this.props.onSearch, fieldsValue);
    });
  }

  render() {
    const mainChildren = this.props.children.slice(0, this.props.truncationNum >= 1 ? this.props.truncationNum : 1);
    const isNeedExpand = this.props.children.length > this.props.truncationNum;

    return (
      <div className={style[classPrefix]} onClick={SearchFilter.stopPropagation}>
        <div className={style[`${classPrefix}-main`]}>
          {mainChildren}
          <div className={style[`${classPrefix}-main-tools`]}>
            {isNeedExpand && <Button onClick={this.openExpand}>更多筛选</Button>}
            <Button onClick={this.search}
                    loading={this.props.searchLoading}
                    disabled={this.props.searchDisabled}
                    type="primary">查询</Button>
            <Button onClick={this.reset}>重置</Button>
          </div>
        </div>
        <ExpandHeight className={style[`${classPrefix}-wrapper`]} ref={c => this.$expand = this.$expand || c}>
          <div className={style[`${classPrefix}-expand`]}>
            {this.props.children}
            <div className={style[`${classPrefix}-expand-footer`]}>
              <TextLink onClick={this.closeExpand}>收起</TextLink>
              <Button onClick={this.search}
                      loading={this.props.searchLoading}
                      disabled={this.props.searchDisabled}
                      type="primary">查询</Button>
              <Button onClick={this.reset}>重置</Button>
            </div>
          </div>
        </ExpandHeight>
      </div>
    )
  }
}

export default SearchFilter;
