import React from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';
import style from './index.less';
import fingerprintIcon from '../../assets/imgs/shareQRCode_fingerprint.png';

const winQRCode = window.QRCode;

class QRCode extends React.Component {
  constructor(props) {
    super(props);
    this.qrCode = null;
  }
  static classPrefix = 'QRCode';
  static propTypes = {
    id: PropTypes.string.isRequired,
    title: PropTypes.oneOfType([
      PropTypes.string,
      PropTypes.node,
    ]),
    text: PropTypes.string.isRequired,
    tips: PropTypes.string,
    tipsStyle: PropTypes.object,
    tipsVisible: PropTypes.bool,
    fingerprintVisible: PropTypes.bool,
  };
  static defaultProps = {
    title: '',
    tips: '长按识别二维码',
    tipsStyle: null,
    tipsVisible: true,
    fingerprintVisible: true,
  };
  // 初始化二维码
  static initQrCode(DOMId, text, width, height) {
    return new winQRCode(document.querySelector(`#${DOMId}`), {
      text,
      width,
      height,
      colorDark: "#000000",
      colorLight: "#ffffff",
      correctLevel: winQRCode.CorrectLevel.Q,
    });
  };
  componentDidMount() {
    this.qrCode = QRCode.initQrCode('QRCode-content', this.props.text, 160, 160);
    let $qrCode = document.querySelector('#QRCode-content');
    $qrCode.title = '';
    $qrCode.dataset.tag = this.props.text;
    $qrCode = null;
  }
  componentWillUnmount() {
    this.qrCode = null;
  }

  render() {
    const prefix = QRCode.classPrefix;
    const wrapStyle = this.props.fingerprintVisible ? null : { height: 470 };
    const wrapClassName = classNames({
      [style[`${prefix}-wrap`]]: true,
    }, this.props.className);

    return (
      <div className={style[prefix]}>
        <div id={this.props.id} className={wrapClassName} style={wrapStyle}>
          <div className={style[`${prefix}-title`]}>{this.props.title}</div>
          <div id="QRCode-content" className={style[`${prefix}-content`]}> </div>
          <div>
            {
              this.props.tipsVisible &&
              <div className={style[`${prefix}-tips`]} style={this.props.tipsStyle}>{this.props.tips}</div>
            }
            {
              this.props.fingerprintVisible &&
              <img className={style[`${prefix}-fingerprint`]} src={fingerprintIcon} alt="fingerprintIcon"/>
            }
          </div>
        </div>
      </div>
    )
  }
}

export default QRCode;
