import React from 'react';
import PropTypes from 'prop-types';
import style from './index.less';
import Modal from '../Modal';
import Button from '../Button';
import QRCode from './QRCode';
import QRCodeOnly from './QRCodeOnly';
import html2canvas from 'html2canvas';

// body容器
const Container = (props) => {
  const prefix = 'Container';
  return (
    <div className={style[prefix]}>
      <div className={style[`${prefix}-header`]}>
        <div className={style[`${prefix}-title`]}>{props.title}</div>
      </div>
      <div className={style[`${prefix}-body`]}>{props.children}</div>
    </div>
  );
};
// 构建底部按钮
const FooterBtn = (props) => {
  const {config, prefix, shareBottom} = props
  const noBtn = config.map(item => item.visible).every(item => item === false);
  if (noBtn) {
    return null;
  } else {
    return (
      <div className={style[`${prefix}-footer`]}>
        <div className={style[`${prefix}-footer-group`]}>
          {
            config.map((item, index) => {
              return item.visible ? <div className={style[`${prefix}-footer-group-item`]} key={index}>
                <Button
                  className={style[`${prefix}-group-btn`]}
                  type="primary"
                  onClick={item.onClick}>{item.text}</Button>
              </div> : '';
            })
          }
        </div>
        {shareBottom &&
        <div className={style[`${prefix}-footer-text`]}>
          {shareBottom}
        </div>
        }
      </div>
    );
  }
};

class ShareQRCode extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false,
      QRCodeOnlyVisible: false,
    };
    this.ShareQRCode = 'ShareQRCode';
    this.ShareQRCodeHD = 'ShareQRCodeHD';
    this.downloadQRCode = this.downloadQRCode.bind(this);
    this.downloadShare = this.downloadShare.bind(this);
    this.downloadHD = this.downloadHD.bind(this);
    this.footerConfig = [
      {
        text: '下载分享二维码',
        onClick:  this.downloadShare,
        visible: props.showShareBtn,
      },
      {
        text: '下载海报二维码',
        onClick: this.downloadHD,
        visible: props.showHDBtn,
      },
    ];
  }
  static classPrefix = 'ShareQRCode';
  static propTypes = {
    title: PropTypes.string,
    shareTitle: PropTypes.oneOfType([
      PropTypes.string,
      PropTypes.node,
    ]),
    shareBottom: PropTypes.oneOfType([
      PropTypes.string,
      PropTypes.node,
    ]),
    shareText: PropTypes.string.isRequired,
    fileName: PropTypes.string,
    visible: PropTypes.bool.isRequired,
    onCancel: PropTypes.func.isRequired,
    showShareBtn: PropTypes.bool,
    showHDBtn: PropTypes.bool,
    tips: PropTypes.string,
    tipsStyle: PropTypes.object,
    tipsVisible: PropTypes.bool,
    fingerprintVisible: PropTypes.bool,
  };
  static defaultProps = {
    title: '分享二维码',
    fileName: '二维码',
    shareBottom: '',
    showShareBtn: true,
    showHDBtn: true,
    tips: '长按识别二维码',
    tipsStyle: null,
    tipsVisible: true,
    fingerprintVisible: true,
  };

  // 下载二维码图片
  async downloadQRCode(DOMId, fileName, callback) {
    this.setState({ loading: true });
    html2canvas(document.querySelector(`#${DOMId}`), {
      logging: false,
      backgroundColor: null,
    }).then((canvas) => {
      const img = ShareQRCode.canvasToImage(canvas);
      ShareQRCode.download(img.src, fileName, callback);
      this.setState({ loading: false });
    });
  }
  downloadShare() {
    this.downloadQRCode(this.ShareQRCode, this.props.fileName);
  }
  downloadHD() {
    this.setState({ QRCodeOnlyVisible: true, }, () => {
      this.downloadQRCode(this.ShareQRCodeHD, `${this.props.fileName}_HD`, () => {
        this.setState({ QRCodeOnlyVisible: false });
      });
    });

  }
  // canvas转img
  static canvasToImage(canvas) {
    let image = new Image();
    image.crossOrigin = "Anonymous";
    image.src = canvas.toDataURL("image/png");
    return image;
  };
  // 下载文件
  static download(href, fileName, callback) {
    let a = document.createElement('a');
    a.setAttribute('href', href);
    a.setAttribute('download', fileName);
    document.body.appendChild(a);
    a.click();
    a.remove();
    if (callback) callback();
  };

  render() {
    const prefix = ShareQRCode.classPrefix;

    return (
      <div>
        {
          this.props.visible &&
          <Modal
            className={style[prefix]}
            loading={this.state.loading}
            widthSize="S"
            onCancel={this.props.onCancel}
            okBtnShow={false}
            cancelBtnShow={false}
            footer={<FooterBtn config={this.footerConfig} prefix={prefix} shareBottom={this.props.shareBottom}/>}
          >
            <Container title={this.props.title}>
              <QRCode
                id={this.ShareQRCode}
                title={this.props.shareTitle}
                text={this.props.shareText}
                tips={this.props.tips}
                tipsStyle={this.props.tipsStyle}
                tipsVisible={this.props.tipsVisible}
                fingerprintVisible={this.props.fingerprintVisible}
                className={this.props.className}
              />
              {
                this.props.showHDBtn &&
                <QRCodeOnly
                  id={this.ShareQRCodeHD}
                  text={this.props.shareText}
                  visible={this.state.QRCodeOnlyVisible}
                />
              }
            </Container>
          </Modal>
        }
      </div>
    )
  }
}

export default ShareQRCode;
