import React from 'react';
import PropTypes from 'prop-types';
import { Layout } from 'antd';
import Modal from '../../Modal';
import Wrapper from '../../Wrapper';
import style from './styles.module.less';

const { Sider, Content } = Layout;

// 标题组件
const Title = (props) => {
  return (
    <div className={style.title} style={props.style}>
      <span className={style.titleText}>{props.children}</span>
    </div>
  );
};

Title.propTypes = {
  style: PropTypes.object,
};

// 购物车弹窗
const ModalShoppingCart = (props) => {
  return (
    <Modal
      className={style.Modal}
      loading={props.loading}
      onCancel={props.onClose}
      onOk={props.onOk}
      okText={props.okText}
      themeConfig={{
        bgc: 'gray',
        footerTopShadow: true,
      }}
      widthSize="XL">
      <Layout className={style.Layout} style={{ height: props.layoutHeight }}>
        <Content className={style.Content}>
          <Wrapper
            header={<Title style={{ fontSize: 18 }}>{props.contentOptions.title}</Title>}
            bodyFitHeight
            autoScrollBar>
            <div className={style.WrapperBody}>
              {
                props.contentOptions.children
              }
            </div>
          </Wrapper>
        </Content>
        <Sider className={style.Sider} width={286}>
          <Wrapper
            header={<Title>{props.sideOptions.title}</Title>}
            bodyFitHeight
            autoScrollBar>
            <div className={style.WrapperBody}>
              {
                props.sideOptions.children
              }
            </div>
          </Wrapper>
        </Sider>
      </Layout>
    </Modal>
  );
};

ModalShoppingCart.propTypes = {
  loading: PropTypes.bool,
  onOk: PropTypes.func,
  okText: PropTypes.string,
  onClose: PropTypes.func.isRequired,
  modalProps: PropTypes.object,
  layoutHeight: PropTypes.number,
  contentOptions: PropTypes.shape({
    title: PropTypes.oneOfType([PropTypes.string, PropTypes.element]).isRequired,
    children: PropTypes.any,
  }).isRequired,
  sideOptions: PropTypes.shape({
    title: PropTypes.oneOfType([PropTypes.string, PropTypes.element]).isRequired,
    children: PropTypes.any,
  }).isRequired,
};

ModalShoppingCart.defaultProps = {
  loading: false,
  okText: '确认',
  layoutHeight: window.innerHeight - 104,
};

export default ModalShoppingCart;
