import React from 'react';
import PropTypes from 'prop-types';
import classNames from "classnames";
import Card from '../../Card';
import style from './styles.module.less';

class Selected extends React.Component {
  constructor(props) {
    super(props);
    this.switchIsShowMore = this.switchIsShowMore.bind(this);
    this.state = {
      isShowMore: false
    };
  }

  static propTypes = {
    onDelete: PropTypes.func,
    deleteBtnVisible: PropTypes.bool,
    showMoreBtnVisible: PropTypes.bool,
  };

  static defaultProps = {
    deleteBtnVisible: true,
    showMoreVisible: true,
  };

  switchIsShowMore() {
    this.setState((prevState) => ({ isShowMore: !prevState.isShowMore }));
  }

  render() {
    const RealChildren = React.Children.map(
      this.props.children,
      child => React.cloneElement(child, { isShowMore: this.state.isShowMore })
    );

    return (
      <Card className={classNames(style.SelectedCard, this.props.className)}>
        {
          this.props.deleteBtnVisible &&
          <div className={style.deleteBtn} onClick={this.props.onDelete} title="删除"/>
        }
        <div className={style.cardBody}>{RealChildren}</div>
        {
          this.props.showMoreVisible &&
          <div className={style.showMore}>
          <span className={style.showMoreBtn} onClick={this.switchIsShowMore}>
            <span>展开更多</span>
            <span className={classNames('DropdownMenu-arrow', { 'DropdownMenu-arrow-up': this.state.isShowMore })}/>
          </span>
          </div>
        }
      </Card>
    )
  }
}

export default Selected;
