import React from 'react';
import style from './index.less';
import PropTypes from 'prop-types';
import classNames from 'classnames';
import Popover from '../Popover';

const classPrefix = 'ShowMore';

const Wrap = (props) => {
  if (props.direction === 'row') {
    return (<span className={style[`${classPrefix}-Wrap-row`]}>{props.children}</span>);
  } else if (props.direction === 'col') {
    return (<div className={style[`${classPrefix}-Wrap-col`]}>{props.children}</div>);
  } else {
    return props.children;
  }
};

const Option = (props) => {
  return <div className={style[`${classPrefix}-Option`]}>{props.children}</div>
};

const ShowMore = (props) => {
  let prevData = [];
  let nextDate = [];
  if (props.data.length >= props.splitStartIndex) {
    prevData = props.data.slice(0, props.splitStartIndex);
    nextDate = props.data.slice(props.splitStartIndex);
  } else {
    prevData = props.data;
  }
  return (
    <div className={style[classPrefix]}>
      {
        prevData.length > 0 &&
        <Wrap direction={props.direction}>
          {prevData.map((item, index) => (<Option key={index}>{item}</Option>))}
        </Wrap>
      }
      {
        nextDate.length > 0 &&
        <Wrap direction={props.direction}>
          <Popover
            overlayClassName={classNames(
              style[`${classPrefix}-overlay`],
              { dark: props.theme === 'dark' },
              props.popoverClassName,
            )}
            content={nextDate.map((item, index) => (<Option key={index}>{item}</Option>))}>
            <span className={style[`${classPrefix}-text`]}>{props.text}</span>
          </Popover>
        </Wrap>
      }
    </div>
  );
};
ShowMore.propTypes = {
  data: PropTypes.arrayOf(PropTypes.string).isRequired,
  splitStartIndex: PropTypes.number,
  text: PropTypes.string,
  direction: PropTypes.string,
  theme: PropTypes.oneOf(['default', 'dark']),
  popoverClassName: PropTypes.string,
};
ShowMore.defaultProps = {
  splitStartIndex: 1,
  text: '更多',
  direction: 'col',
  theme: 'default',
  popoverClassName: null
};

export default ShowMore;
