import React from 'react';
import PropTypes from 'prop-types';
import { Select } from 'COMPONENT';
import debounce from 'lodash/debounce';
import { searchUser, searchUserByRoleCode } from 'SERVICE/customer';
import { errorsHandle } from 'UTIL/common';
import { runFunc } from 'UTIL/common';

export default class StaffSelect extends React.Component {
  static propTypes = {
    initialValue: PropTypes.shape({
      value: PropTypes.string,
      text: PropTypes.string,
    }),
    onChange: PropTypes.func.isRequired,
    placeholder: PropTypes.string,
    searchPlaceholder: PropTypes.string,
    searchNotFoundTips: PropTypes.string,
    searchType: PropTypes.string,
  };
  static defaultProps = {
    placeholder: '请选择员工',
    searchPlaceholder: '请先输入员工名称查找',
    searchNotFoundTips: '没有找到该员工',
    initialValue: {
      text: '',
      value: '',
    },
  };

  constructor(props) {
    super(props);
    this.state = {
      loading: false,
      data: null,
      fieldValue: this.props.initialValue.val,
    };
    this.handleSearch = debounce(this.doSearch, 1000);
  }

  componentDidMount() {
    this.props.initialValue.text && this.handleSearch(this.props.initialValue.text);
  }

  doSearch = async (name) => {
    if (!name || name.length < 1) {
      return;
    }

    this.setState({ loading: true });
    const rsp = await this.doCallApi(name).catch(errorsHandle());
    if (rsp && Array.isArray(rsp.data)) {
      const dict = rsp.data.map((item) => ({
        text: `${item.username}<${item.employeeNo}>`,
        subText: item.orgInfo,
        val: item.userId,
      }));
      this.setState({ data: dict }, this.updateFieldValue);
    }
    this.setState({ loading: false });
  };

  doCallApi = async (name) => {
    switch (this.props.searchType) {
      case 'ByRoleCode':
        const { roleCode } = this.props.searchParam;
        const res = await searchUserByRoleCode({ queryName: name, roleCode, number: 500 });
        //统一返回结果
        if (res && Array.isArray(res.data)) {
          res.data = res.data.map((item) => ({
            ...item,
            username: item.name,
          }));
        }
        return res;
      case 'All':
      default:
        //查询全部用户
        return await searchUser({ name });
    }
  };

  updateFieldValue = () => {
    const val = this.state.fieldValue;
    const index = this.state.data.findIndex((item) => item.val === val);
    if (index === -1) {
      this.setState({
        fieldValue: null,
      });
    }
  };

  handleChange = (selected) => {
    const index = this.state.data.findIndex((item) => item.val === selected);
    runFunc(this.props.onChange, index > -1 ? this.state.data[index] : '');
  };

  render() {
    return (
      <Select
        placeholder={this.props.placeholder}
        dict={this.state.data}
        showSearch
        allowClear
        searchPlaceholder={this.props.searchPlaceholder}
        searchNotFoundTips={this.props.searchNotFoundTips}
        onSearchKeyChange={this.handleSearch}
        onChange={this.handleChange}
        value={this.props.initialValue.value}
      />
    );
  }
}
