import React from 'react';
import ReactDOM from 'react-dom';
import PropTypes from 'prop-types';
import classNames from 'classnames';
import style from './index.less';
import { Tag } from 'antd';
import ExpandBtn from '../ExpandBtn';
import { runFunc } from '../utils/common';
import TagClose from '../TagClose'

const classPrefix = 'TempStorage';

class TempStorage extends React.Component {
  constructor(props) {
    super(props);
    this.updateComponent = this.updateComponent.bind(this);
    this.updateExpandBtnVisible = this.updateExpandBtnVisible.bind(this);
    this.updateContentHeight = this.updateContentHeight.bind(this);
    this.switchExpand = this.switchExpand.bind(this);
    this.closeExpand = this.closeExpand.bind(this);
    this.state = {
      expandBtnVisible: false,
      expand: false,
    };
    this.$content = null;
    this.$list = null;
    this.optionExpand = false;
  }

  componentDidMount() {
    window.addEventListener('resize', this.updateComponent, false);
    window.addEventListener('click', this.closeExpand, false);
  }

  componentDidUpdate(prevProps, prevState, snapshot) {
    this.updateComponent();
  }

  componentWillUnmount() {
    window.removeEventListener('resize', this.updateComponent, false);
    window.removeEventListener('click', this.switchExpand, false);
  }

  static propTypes = {
    className: PropTypes.string,
    // style: PropTypes.object,
    label: PropTypes.string,
    isLabelCount: PropTypes.bool,
    // labelRender: PropTypes.func,
    source: PropTypes.array.isRequired,
    optionTextKey: PropTypes.string.isRequired,
    onDelete: PropTypes.func,
    onChange: PropTypes.func,
    // optionRender: PropTypes.func,
    // maxRow: PropTypes.number,
    tagType: PropTypes.number,
  };

  static defaultProps = {
    // style: null,
    isLabelCount: false,
    // maxRow: 6,
    tagType: 1,           // tag的类型，1：antd-tag，2：TagClose，默认1
  };

  updateComponent() {
    this.updateExpandBtnVisible();
    this.updateContentHeight();
  }

  updateExpandBtnVisible() {
    if (this.$list.offsetHeight > 48 && !this.state.expandBtnVisible) {
      this.setState({ expandBtnVisible: true });
    } else if (this.$list.offsetHeight <= 48 && this.state.expandBtnVisible) {
      this.setState({
        expandBtnVisible: false,
        expand: false,
      });
      this.$content.style.height = '48px';
      this.optionExpand = false;
    }
  }

  updateContentHeight() {
    if (this.optionExpand) {
      this.$content.style.height = `${this.$list.offsetHeight}px`;
    }
  }

  switchExpand() {
    this.setState((prevState) => ({ expand: !prevState.expand }));
    this.$content.style.height = this.optionExpand ? '48px' : `${this.$list.offsetHeight}px`;
    this.optionExpand = !this.optionExpand;
  }

  closeExpand() {
    if (this.state.expand) {
      this.setState({ expand: false });
      this.$content.style.height = '48px';
      this.optionExpand = false;
    }
  }

  static stopPropagation(e) {
    e.stopPropagation();
  }

  render() {
    return (
      <div className={style[classPrefix]} onClick={TempStorage.stopPropagation}>
        <div className={classNames({
          [style[`${classPrefix}-wrapper`]]: true,
          'active': this.state.expand,
          [this.props.className]: this.props.className,
        })}>
          <div className={style[`${classPrefix}-main`]}>
            {
              this.props.label &&
              <div className={style[`${classPrefix}-label`]}>
                {this.props.label}
                {this.props.isLabelCount && `(${this.props.source.length})`}
                ：
              </div>
            }
            <div className={style[`${classPrefix}-content`]}
                 ref={c => this.$content = this.$content || ReactDOM.findDOMNode(c)}>
              <ul className={style[`${classPrefix}-list`]}
                  ref={c => this.$list = this.$list || ReactDOM.findDOMNode(c)}>
                {
                  this.props.source.map((item, index) => {
                    let key = item.id || `${item[this.props.optionTextKey]}_${index}`;
                    return (
                      <li className={style[`${classPrefix}-option`]}
                          key={key}>
                        {
                          this.props.tagType === 1 ? (
                            <Tag
                              closable
                              afterClose={() => { runFunc(this.props.onChange, item, index) }}
                              onClose={() => { runFunc(this.props.onDelete, item, index) }}
                              key={key}
                            >{item[this.props.optionTextKey]}</Tag>
                          ) : (
                            <TagClose
                              onClose={() => {runFunc(this.props.onDelete, item, index)}}
                            >{item[this.props.optionTextKey]}</TagClose>
                          )
                        }
                      </li>
                    )
                  })
                }
              </ul>
            </div>
          </div>
          <div className={style[`${classPrefix}-sub`]}>
            {
              this.state.expandBtnVisible &&
              <ExpandBtn active={this.state.expand} onClick={this.switchExpand}/>
            }
          </div>
        </div>
      </div>
    )
  }
}

export default TempStorage;
