import React from 'react';
import styles from './index.less';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { getSseKey } from 'SERVICE/common';
import { uuid } from 'COMPONENT/utils/basic/str';

export default class SelfUpload extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      list: [],
    };
  }

  triggerInput = () => {
    this.$input.click();
  };

  limitValidate(data, file) {
    let { size, type } = data;
    if (size && !(file.size / 1024 < size)) {
      showMessage(`文件大小超过${size}k`);
      return false;
    }
    if (type) {
      let limitType;
      type.split('/').forEach((v, k) => {
        if (file.type.includes(v)) {
          limitType = true;
        }
      });
      if (!limitType) {
        showMessage(`只能上传${type}文件`);
        return false;
      }
    }
    return true;
  }

  removeSuffix = (name) => {
    if (name) {
      let tempArr = name.split('.');
      return tempArr[tempArr.length - 1];
    }
    return '';
  };

  onChange = async (e) => {
    let list = [];
    let { hasExtension, beforeUpload } = this.props;
    let fileList = [...e.target.files];

    if (typeof beforeUpload === 'function') {
      fileList.forEach((v, k) => {
        let fileKey = this.getFileKey(v);
        let suffix = this.removeSuffix(v.name);
        if (hasExtension) {
          v['fileKey'] = fileKey + '.' + suffix;
        } else {
          v['fileKey'] = fileKey;
        }
      });
      let res = beforeUpload(fileList);
      if (res === false) {
        return false;
      } else if (res instanceof Array) {
        if (!this.checkLimit(res || fileList)) {
          return false;
        }
        this.getKeyAndMapRequest(res || fileList, list);
      } else {
        res
          .then(async (resolve) => {
            if (!this.checkLimit(resolve || fileList)) {
              return false;
            }
            this.getKeyAndMapRequest(resolve || fileList, list);
          })
          .catch((reject) => {
            return false;
          });
      }
    } else {
      if (!this.checkLimit(fileList)) {
        return false;
      }
      this.getKeyAndMapRequest(fileList, list);
    }
  };

  getFileKey(file, hasExtension) {
    let fileKey = uuid();
    if (hasExtension) {
      const extName = (file.type || '').split('/')[1];
      extName && (fileKey = `${fileKey}.${extName}`);
    }

    return fileKey;
  }

  checkLimit = (fileList) => {
    let { limit } = this.props;
    if (limit) {
      for (let item of fileList) {
        if (!this.limitValidate(limit, item)) {
          return false;
        }
      }
    }
    return true;
  };
  getKeyAndMapRequest = async (fileList, list) => {
    let configKey = await getSseKey();
    //  循环  uploadFile
    for (let file of fileList) {
      this.uploadFile(file, configKey, list);
    }
  };

  uploadFile = (file, configKey, list) => {
    let { name, type, size } = file;
    let { fileKey } = file;
    let formData = new FormData();
    formData.append('x:realname', name);
    formData.append('x:uploadtype', 'ipad');
    formData.append('x:uploadid', uuid());
    formData.append('key', fileKey);
    Object.keys(configKey).forEach((key, index) => {
      console.log(configKey);
      if (key != 'host' && (!this.props.ignoreExpires || key != 'Expires')) formData.append(key, configKey[key]);
    });
    formData.append('file', file);
    let info = {};
    let xhr = new XMLHttpRequest();
    this.props.onSetUploadStatus && this.props.onSetUploadStatus(true);
    xhr.open('POST', configKey.host, true);
    xhr.upload.onloadstart = (file) => {
      info = { ...info, fileKey, showProgress: true, percent: 0, name, type, size };
      this.update(info, list);
    };
    xhr.upload.onprogress = (file) => {
      info = { ...info, percent: (file.loaded * 100) / file.total, name, type, size };
      this.update(info, list);
    };
    xhr.onload = (res) => {
      if (res.target.status === 200) {
        info = {
          ...info,
          src: configKey.host + fileKey,
          showProgress: false,
          ...JSON.parse(res.target.response),
          name,
          type,
          size,
        };
      } else {
        info = {
          ...info,
          src: configKey.host + fileKey,
          showProgress: false,
          errorCode: res.target.status,
          name,
          type,
          size,
        };
      }
      this.props.onSetUploadStatus && this.props.onSetUploadStatus(false);
      this.update(info, list);
    };
    xhr.onerror = () => {
      info['error'] = true;
      info['showProgress'] = false;
      this.update({ ...info, name, type, size }, list);
      this.props.onSetUploadStatus && this.props.onSetUploadStatus(false);
    };
    xhr.send(formData);
  };
  update = (info, list) => {
    let { onChange, multiple = false } = this.props;
    if (multiple) {
      // let {list} = this.state;
      let tags = false;
      let index;
      for (let i = 0; i < list.length; i++) {
        if (list[i].fileKey === info.fileKey) {
          tags = true;
          index = i;
          break;
        }
      }
      if (tags) {
        list[index] = info;
      } else {
        list.push(info);
      }
      onChange(list);
    } else {
      onChange(info);
    }
  };

  render() {
    let { multiple = false } = this.props;
    return (
      <div onClick={this.triggerInput}>
        <input
          type="file"
          multiple={multiple}
          onClick={(event) => {
            event.target.value = null;
          }}
          ref={(e) => (this.$input = e)}
          onChange={this.onChange}
          className={styles.upload}
        />
        {this.props.children}
      </div>
    );
  }
}
