import React from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';
import style from './index.less';
import { Large, Normal, Small } from './config';
import Loading from '../Loading'

const classPrefix = 'Wrapper';

const Wrapper = (props) => {
  return (
    <div className={classNames(style[classPrefix], props.className)}>
      <div className={classNames(
        style[`${classPrefix}-scroller`],
        {
          'scroll-bar-large': props.scrollBarSize === Large,
          'scroll-bar-normal': props.scrollBarSize === Normal,
          'scroll-bar-small': props.scrollBarSize === Small,
        },
        props.autoScrollBar ? style[`${classPrefix}-autoScroller`] : ''
      )}>
        <div className={classNames(style[`${classPrefix}-content`], { fit: props.bodyFitHeight })}>
          {
            props.header &&
            <div className={style[`${classPrefix}-header`]}>{props.header}</div>

          }
          <div className={style[`${classPrefix}-body`]}>{props.children}</div>
          {
            props.footer &&
            <div className={style[`${classPrefix}-footer`]}>{props.footer}</div>
          }
        </div>
      </div>
      <Loading loading={props.loading} tip={props.loadingTip}/>
    </div>
  );
};

Wrapper.propTypes = {
  className: PropTypes.string,
  bodyFitHeight: PropTypes.bool,
  header: PropTypes.oneOfType([ PropTypes.node, PropTypes.element, PropTypes.func ]),
  footer: PropTypes.oneOfType([ PropTypes.node, PropTypes.element, PropTypes.func ]),
  autoScrollBar: PropTypes.bool,
  scrollBarSize: PropTypes.oneOf([Large, Normal, Small]),
  loading: PropTypes.bool,
  loadingTip: PropTypes.string,
};

Wrapper.defaultProps = {
  className: null,
  bodyFitHeight: false,
  autoScrollBar: true,
  scrollBarSize: 'Normal',
  loading: false,
  loadingTip: null,
};

export default Wrapper;
