import React from 'react';
import CompPage from 'DOC/DocPage';
import Section from 'DOC/Section';
import PropsTable from 'DOC/PropsTable';
import { Row, Col } from 'antd';
import FormItem from 'COMPONENT/FormItem';
import Input from 'COMPONENT/Input';
import DemoBase from './DemoBase';
import DemoAutoSelectAfter from './DemoAutoSelectAfter';
import DemoMultiSelect from './DemoMultiSelect';
import DemoBeforeNowDisabled from './DemoBeforeNowDisabled';
import DemoNoDataDirectCallback from './DemoNoDataDirectCallback';
import DemoFooterTipsRender from './DemoFooterTipsRender';

class Main extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      productType: 'SMALL_CLASS',
      classId: 'afea46d26a0b732f016a0b7618c7002a',
      noDataClassId: '2c908255669ad8fa01669b1f66910068',
    };
  }

  render() {
    const { productType, classId, noDataClassId } = this.state;

    return (
      <CompPage>
        <Section title="功能介绍">适用于双师/小班的上课时间选择弹窗。后期可拓展到其他产品类型。</Section>
        <Section title="例子">
          <Row>
            <Col span={12}>
              <FormItem name="产品类型" labelWidth={80}>
                <Input
                  placeholder="产品类型"
                  value={productType}
                  onChange={(e) => {
                    this.setState({ productType: e.target.value });
                  }}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="班级id" labelWidth={80}>
                <Input
                  placeholder="班级id"
                  value={classId}
                  onChange={(e) => {
                    this.setState({ classId: e.target.value });
                  }}
                />
              </FormItem>
            </Col>
          </Row>
          <h3>基本用法</h3>
          <DemoBase productType={productType} classId={classId} />
          <h3>选择当前与之后所有</h3>
          <DemoAutoSelectAfter productType={productType} classId={classId} />
          <h3>多选</h3>
          <DemoMultiSelect productType={productType} classId={classId} />
          <h3>当前日期之前的不可选</h3>
          <DemoBeforeNowDisabled productType={productType} classId={classId} />
          <h3>没数据时不打开弹窗</h3>
          <DemoNoDataDirectCallback productType={productType} classId={noDataClassId} />
          <h3>弹窗footer左侧提示语</h3>
          <DemoFooterTipsRender productType={productType} classId={classId} />
        </Section>
        <Section title="属性">
          <PropsTable>
            <tr>
              <td>title</td>
              <td>(可选) 弹窗标题</td>
              <td>String</td>
              <td>选择上课时间</td>
            </tr>
            <tr>
              <td>classId</td>
              <td>(必填) 班级Id（双师辅班Id/小班Id）</td>
              <td>String</td>
              <td>-</td>
            </tr>
            <tr>
              <td>productType</td>
              <td>(必填) 产品类型，现在只支持双师/小班</td>
              <td>String</td>
              <td>-</td>
            </tr>
            <tr>
              <td>autoSelectAfter</td>
              <td>(可选) 选中某个选项后，它后面的所有选项也会被选中</td>
              <td>Boolean</td>
              <td>false</td>
            </tr>
            <tr>
              <td>multiSelect</td>
              <td>(可选) 多选，当autoSelectAfter为true时忽略multiSelect</td>
              <td>Boolean</td>
              <td>false</td>
            </tr>
            <tr>
              <td>beforeNowDisabled</td>
              <td>(可选) 比当前日期之前的选项不可选</td>
              <td>Boolean</td>
              <td>false</td>
            </tr>
            <tr>
              <td>beforeNowDays</td>
              <td>(可选) beforeNowDisabled生效时，当前日期前n天的日期可选</td>
              <td>Number</td>
              <td>0</td>
            </tr>
            <tr>
              <td>onCancel</td>
              <td>(必填) 弹窗关闭</td>
              <td>Function</td>
              <td>-</td>
            </tr>
            <tr>
              <td>onConfirm</td>
              <td>(必填) 确认选择</td>
              <td>Function(selected, selectedKeys, list)</td>
              <td>-</td>
            </tr>
            <tr>
              <td>noDataDirectCallback</td>
              <td>(可选) 该属性有传值时，再无数据的时候会直接调用所传函数，不会打开弹窗</td>
              <td>Function</td>
              <td>-</td>
            </tr>
            <tr>
              <td>footerTipsRender</td>
              <td>(可选) 弹窗footer左侧提示语渲染</td>
              <td>Function(selectedKeys, list)</td>
              <td>-</td>
            </tr>
          </PropsTable>
        </Section>
      </CompPage>
    );
  }
}

export default Main;
