import React from 'react';
import { Row, Col } from 'antd';
import moment from 'moment';
import DatePicker from 'COMPONENT/DatePicker';

class Main extends React.Component {
  state = {
    startDate: null,
    endDate: null,
  };

  /**
   * 获取要禁用的开始日期值
   * @param  {string} startDate 过滤前的日期值
   * @return {boolean} 是否要禁用
   */
  getDisabledStartDate = (startDate) => {
    const { endDate } = this.state;
    if (!startDate || !endDate) {
      return false;
    }
    return moment(startDate).valueOf() > moment(endDate).valueOf();
  };

  /**
   * 获取要禁用的结束日期值
   * @param  {string} endDate   过滤前的日期值
   * @return {boolean}      是否要禁用
   */
  getDisabledEndDate = (endDate) => {
    const { startDate } = this.state;
    if (!endDate || !startDate) {
      return false;
    }
    return moment(endDate).valueOf() < moment(startDate).valueOf();
  };

  render() {
    const { startDate, endDate } = this.state;
    return (
      <div style={{ width: 420 }}>
        <Row gutter={20}>
          <Col span={12}>
            <DatePicker
              placeholder="开始日期"
              value={startDate}
              onChange={(val) => this.setState({ startDate: val })}
              disabledDate={this.getDisabledStartDate}
            />
          </Col>
          <Col span={12}>
            <DatePicker
              placeholder="结束日期"
              value={endDate}
              onChange={(val) => this.setState({ endDate: val })}
              disabledDate={this.getDisabledEndDate}
            />
          </Col>
        </Row>
      </div>
    );
  }
}

export default Main;
