import React from 'react';
import { connect } from 'dva';
import CompPage from 'DOC/DocPage';
import PropsTable from '../../PropsTable';
import FuncsTable from '../../FuncsTable';
import Sec from '../../Section';
import TextDemo from './TextDemo';
import IconDemo from './IconDemo';
import SmallDemo from './SmallDemo';
import CustomDemo from './CustomDemo';
import CustomColorDemo from './CustomColorDemo';
import DisabledDemo from './DisabledDemo';
import styles from '../../styles.less';
import styleVars from 'COMPONENT/theme/variables';

class Main extends React.Component {
  openPage = (params) => {
    const { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: params,
    });
  };

  render() {
    return (
      <CompPage>
        <Sec title="功能介绍">下拉菜单，用于展开更多操作。</Sec>
        <Sec title="例子">
          <h3>文字按钮</h3>
          <TextDemo />
          <h3>图标按钮</h3>
          <IconDemo />
          <h3>禁用</h3>
          <DisabledDemo />
          <h3>小号</h3>
          <SmallDemo />
          <h3>自定义颜色</h3>
          <CustomColorDemo />
          <h3>自定义内容</h3>
          <CustomDemo />
        </Sec>
        <Sec title="开发说明">暂无。</Sec>
        <Sec title="属性">
          <PropsTable>
            <tr>
              <td>text</td>
              <td>文字按钮内容</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>iconType</td>
              <td>图标按钮类型，设置了此属性时忽略text属性</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>hasArrow</td>
              <td>带有转动的箭头图标</td>
              <td>boolean</td>
              <td></td>
            </tr>
            <tr>
              <td>disabled</td>
              <td>禁用状态</td>
              <td>boolean</td>
              <td></td>
            </tr>
            <tr>
              <td>size</td>
              <td>大小，可选'small'</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>color</td>
              <td>按钮颜色，仅按钮内容为文字或者单色图标时适用</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>activeColor</td>
              <td>激活状态时按钮的颜色，仅按钮内容为文字或者单色图标时适用</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>disabledColor</td>
              <td>禁用状态时按钮的颜色，仅按钮内容为文字或者单色图标时适用</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>content</td>
              <td>自定义按钮内容</td>
              <td>ReactNode</td>
              <td></td>
            </tr>
            <tr>
              <td>activeContent</td>
              <td>激活状态的自定义按钮内容</td>
              <td>ReactNode</td>
              <td></td>
            </tr>
            <tr>
              <td>disabledContent</td>
              <td>禁用状态的自定义按钮内容</td>
              <td>ReactNode</td>
              <td></td>
            </tr>
            <tr>
              <td>isCustom</td>
              <td>
                是否为自定义按钮内容。为true时，content、activeContent、disabledContent属性生效，text、iconType属性失效
              </td>
              <td>boolean</td>
              <td></td>
            </tr>
            <tr>
              <td>menus</td>
              <td>菜单项配置</td>
              <td>
                <pre>
                  {`[
  {
    text: string - 文字
    disabled: boolean - 是否禁用
    hide: boolean - 是否隐藏
    onClick: Function - 点击菜单时的回调
  }
]`}
                </pre>
              </td>
              <td></td>
            </tr>
            <tr>
              <td>placement</td>
              <td>菜单弹出位置，'bottomLeft'|'bottomCenter'|'bottomRight'|'topLeft'|'topCenter'|'topRight'</td>
              <td>string</td>
              <td>'bottomLeft'</td>
            </tr>
            <tr>
              <td>trigger</td>
              <td>触发方式，默认为鼠标移入触发，可选'click'</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>className</td>
              <td>附加在根元素上的类名</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>style</td>
              <td>附加在根元素上的样式</td>
              <td>object</td>
              <td></td>
            </tr>
            <tr>
              <td>dropdownClassName</td>
              <td>附加在下拉菜单根元素上的类名</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>getPopupContainer</td>
              <td>获取承载弹框的DOM元素的函数</td>
              <td>DOMElement function</td>
              <td></td>
            </tr>
          </PropsTable>
        </Sec>
      </CompPage>
    );
  }
}

export default connect()(Main);
