import React from 'react';
import CompPage from 'DOC/DocPage';
import PropsTable from '../../PropsTable';
import FuncsTable from '../../FuncsTable';
import Sec from '../../Section';
import ViewDemo from './ViewDemo';
import EditDemo from './EditDemo';
import IconDemo from './IconDemo';
import TipsDemo from './TipsDemo';
import styles from '../../styles.less';
import TextLink from 'COMPONENT/TextLink';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import styleVars from 'COMPONENT/theme/variables';

class Main extends React.Component {
  render() {
    return (
      <CompPage ref={(c) => (this.pageRef = getWrappedInstance(c))}>
        <Sec title="功能介绍">
          常用的文件列表展示和操作，特点如下：
          <ul className={styles.ul}>
            <li>可自动识别文件类型并显示相应的图标</li>
            <li>分查看和编辑两种类型，查看包含文件预览和下载功能，编辑包含修改文件名和删除文件功能</li>
            <li>可显示上传进度条以及错误信息</li>
          </ul>
        </Sec>
        <Sec title="例子">
          <h3>查看</h3>
          <ViewDemo />
          <h3>编辑</h3>
          <EditDemo />
          <h3>自定义图标</h3>
          <IconDemo />
          <h3>信息提示</h3>
          <TipsDemo />
        </Sec>
        <Sec title="开发说明">暂无。</Sec>
        <Sec title="属性">
          <PropsTable>
            <tr>
              <td>className</td>
              <td>附加在根元素的类</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>style</td>
              <td>附加在根元素的样式</td>
              <td>object</td>
              <td></td>
            </tr>
            <tr>
              <td>type</td>
              <td>类型，可选'view'|'edit'</td>
              <td>string</td>
              <td>'edit'</td>
            </tr>
            <tr>
              <td>showIcon</td>
              <td>是否显示图标</td>
              <td>boolean</td>
              <td>true</td>
            </tr>
            <tr>
              <td>data</td>
              <td>列表数据</td>
              <td>
                <pre>
                  {`[
  {
    id: string - 文件ID，必需
    name: string - 文件名称，必需
    src: strign - 文件源地址
    showProgress: boolean - 是否显示上传进度条
    percent: number - 上传进度百分比
    error: string - 错误信息
    iconSrc: string - 自定义图标源地址
  }
]`}
                </pre>
              </td>
              <td></td>
            </tr>
            <tr>
              <td>onChange</td>
              <td>列表数据变化时的回调，参数为列表数据</td>
              <td>function(object)</td>
              <td></td>
            </tr>
            <tr>
              <td>onPreview</td>
              <td>预览时的回调，参数为文件预览地址和源地址</td>
              <td>{`function({ previewPath, src })`}</td>
              <td></td>
            </tr>
          </PropsTable>
        </Sec>
      </CompPage>
    );
  }
}

export default Main;
