import React from 'react';
import CompPage from 'DOC/DocPage';
import PropsTable from '../../PropsTable';
import FuncsTable from '../../FuncsTable';
import Sec from '../../Section';
import BasicDemo from './BasicDemo';
import MultiColorDemo from './MultiColorDemo';
import LibDemo from './LibDemo';
import styles from '../../styles.less';
import Icon from 'COMPONENT/Icon';

class Main extends React.Component {
  render() {
    return (
      <CompPage>
        <Sec title="功能介绍">
          图标，分为单色和多色，单色图标可随意设置颜色，多色图标区分普通状态、鼠标悬浮状态、激活状态。
        </Sec>
        <Sec title="例子">
          <h3>基本用法</h3>
          <BasicDemo />
          <h3>多色图标</h3>
          <MultiColorDemo />
          <h3>图标库</h3>
          <LibDemo />
        </Sec>
        <Sec title="开发说明">
          图标库添加图标时需按照一定的规则命名：
          <ul className={styles.ul}>
            <li>
              按形状分普通、圆背景、空心圆背景、方背景、空心方背景。命名分别添加后缀： <br />
              1、普通，不添加后缀，如：
              <Icon type="plus" size={16} />
              ，'plus' <br />
              2、圆背景，'_circle'，如：
              <Icon type="plus_circle" size={16} />
              ，'plus_circle' <br />
              3、方背景，'_square'，如：
              <Icon type="check_square" size={16} />
              ，'check_square' <br />
              4、空心圆背景，'_circle_o' <br />
              5、空心方背景，'_square_o'，如：
              <Icon type="plus_square_o" size={16} />
              ，'plus_square_o'
            </li>
            <li>
              按颜色分单色和多色。其中多色图标规则： <br />
              1、普通，添加'_mc'后缀，如：
              <Icon type="setting_mc" size={16} />
              ，'setting_mc' <br />
              2、鼠标悬浮状态，添加'_mc_h'后缀，如：
              <Icon type="setting_mc_h" size={16} />
              ，'setting_mc_h' <br />
              3、激活状态，添加'_mc_a'后缀，如：
              <Icon type="setting_mc_a" size={16} />
              ，'setting_mc_a'
            </li>
          </ul>
        </Sec>
        <Sec title="属性">
          <PropsTable>
            <tr>
              <td>type</td>
              <td>图标类型</td>
              <td>string</td>
              <td>-</td>
            </tr>
            <tr>
              <td>size</td>
              <td>大小（px），默认为普通字体大小</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>color</td>
              <td>颜色，仅对单色图标有效，默认为普通字体色</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>className</td>
              <td>附加在根元素的类名</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>style</td>
              <td>附加在根元素的样式</td>
              <td>object</td>
              <td></td>
            </tr>
          </PropsTable>
        </Sec>
      </CompPage>
    );
  }
}

export default Main;
