import React from 'react';
import CompPage from 'DOC/DocPage';
import PropsTable from '../../PropsTable';
import FuncsTable from '../../FuncsTable';
import Sec from '../../Section';
import BasicDemo from './BasicDemo';
import SizeDemo from './SizeDemo';
import DisabledDemo from './DisabledDemo';
import WordCountDemo from './WordCountDemo';
import styles from '../../styles.less';
import TextLink from 'COMPONENT/TextLink';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import styleVars from 'COMPONENT/theme/variables';

class Main extends React.Component {
  render() {
    return (
      <CompPage ref={(c) => (this.pageRef = getWrappedInstance(c))}>
        <Sec title="功能介绍">单行文本输入框，替代React自带的input组件。</Sec>
        <Sec title="例子">
          <h3>基本用法</h3>
          <BasicDemo />
          <h3>三种大小</h3>
          <SizeDemo />
          <h3>禁用状态</h3>
          <DisabledDemo />
          <h3>字数提示</h3>
          <WordCountDemo />
        </Sec>
        <Sec title="开发说明">暂无。</Sec>
        <Sec title="属性">
          <PropsTable>
            <tr>
              <td>value</td>
              <td>值</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>disabled</td>
              <td>禁用状态</td>
              <td>boolean</td>
              <td></td>
            </tr>
            <tr>
              <td>placeholder</td>
              <td>占位文字</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>size</td>
              <td>大小，可选'small'|'xSmall'</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>wordCount</td>
              <td>是否需要字数提示</td>
              <td>boolean</td>
              <td></td>
            </tr>
            <tr>
              <td>maxLength</td>
              <td>最大字数</td>
              <td>number</td>
              <td></td>
            </tr>
            <tr>
              <td>validator</td>
              <td>
                校验器配置，请参考
                <TextLink onClick={() => this.pageRef.openPage({ path: '/docValidator' })}>Validator</TextLink>
                组件的属性配置
              </td>
              <td>object</td>
              <td></td>
            </tr>
            <tr>
              <td>onPressEnter</td>
              <td>按下回车的回调</td>
              <td>Function(e)</td>
              <td></td>
            </tr>
            <tr>
              <td>className</td>
              <td>附加在根元素的类</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>style</td>
              <td>附加在根元素的样式</td>
              <td>object</td>
              <td></td>
            </tr>
          </PropsTable>
          <div className={styles.info}>
            其他事件属性同
            <TextLink
              onClick={() =>
                window.open('https://facebook.github.io/react/docs/events.html#supported-events', '_blank')
              }
            >
              React的表单元素通用事件
            </TextLink>
            。
          </div>
        </Sec>
      </CompPage>
    );
  }
}

export default Main;
