import React from 'react';
import Input from 'COMPONENT/Input';
import InputNumber from 'COMPONENT/InputNumber';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import DatePicker from 'COMPONENT/DatePicker';
import List from 'COMPONENT/List';
import TextLink from 'COMPONENT/TextLink';

const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;
const demoDict = [
  { val: '1', text: '选项 1' },
  { val: '2', text: '选项 2' },
  { val: '3', text: '选项 3' },
];
const initList = [{ input: null, number: null, select: null, multiSelect: null, date: null }];

class Main extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      list: initList, // {[object]} 列表数据
    };
  }

  /**
   * 修改行数据
   * @param  {number} idx 行序号
   * @param  {object} changed 要修改的数据键值对
   */
  updateRow = (idx, changed) => {
    this.setState({
      list: this.state.list.map((row, i) => {
        if (i === idx) {
          return { ...row, ...changed };
        }
        return row;
      }),
    });
  };

  /**
   * 移除行
   * @param  {object} record 行数据
   * @param  {number} index  行序号
   */
  removeRow = (record, index) => {
    this.setState({
      list: this.state.list.filter((row, i) => i !== index),
    });
  };

  /**
   * 添加行
   * @param  {object} record 行数据
   * @param  {number} index  行序号
   */
  addRow = (record, index) => {
    const { list } = this.state;
    let newList = [...list];
    newList.splice(index + 1, 0, initList);
    this.setState({ list: newList });
  };

  render() {
    const { list } = this.state;
    const columns = [
      {
        title: '文本',
        dataIndex: 'input',
        render: (text, record, index) => {
          return <Input value={record.input} onChange={(e) => this.updateRow(index, { input: e.target.value })} />;
        },
      },
      {
        title: '数字',
        dataIndex: 'number',
        render: (text, record, index) => {
          return <InputNumber value={record.number} onChange={(val) => this.updateRow(index, { number: val })} />;
        },
      },
      {
        title: '选项',
        dataIndex: 'select',
        render: (text, record, index) => {
          return (
            <Select dict={demoDict} value={record.select} onChange={(val) => this.updateRow(index, { select: val })} />
          );
        },
      },
      {
        title: '多选',
        dataIndex: 'multiSelect',
        render: (text, record, index) => {
          return (
            <MultiSelect
              dict={demoDict}
              value={record.multiSelect}
              onChange={(val) => this.updateRow(index, { multiSelect: val })}
            />
          );
        },
      },
      {
        title: '日期',
        dataIndex: 'date',
        render: (text, record, index) => {
          return <DatePicker value={record.date} onChange={(val) => this.updateRow(index, { date: val })} />;
        },
      },
      {
        title: '操作',
        key: 'action',
        fixed: 'right',
        render: (text, record, index) => {
          return (
            <span>
              <TextLink className="list-btn" onClick={() => this.removeRow(record, index)}>
                移除
              </TextLink>
              <TextLink className="list-btn" onClick={() => this.addRow(record, index)}>
                添加
              </TextLink>
            </span>
          );
        },
      },
    ];
    return <List dataSource={list} columns={columns} />;
  }
}

export default Main;
