import React from 'react';
import CompPage from 'DOC/DocPage';
import PropsTable from '../../PropsTable';
import FuncsTable from '../../FuncsTable';
import Sec from '../../Section';
import WidthDemo from './WidthDemo';
import CellsDemo from './CellsDemo';
import PagerDemo from './PagerDemo';
import EditDemo from './EditDemo';
import CheckDemo from './CheckDemo';
import ExpandDemo from './ExpandDemo';
import CardDemo from './CardDemo';
import styles from '../../styles.less';

class Main extends React.Component {
  render() {
    return (
      <CompPage>
        <Sec title="功能介绍">
          展示行列数据。特点有：
          <ul className={styles.ul}>
            <li>
              宽度适配方式为：1、列宽固定最小宽度；2、总宽度大于外层容器宽度时，横向滚动；3、总宽度小于外层容器宽度时，按比例拉伸；4、横向滚动时，首尾列可固定
            </li>
            <li>高度适配方式为：1、每行高度随内容伸缩；2、总高度大于外层容器高度时，纵向滚动</li>
            <li>单元格的内容及展现形式可自由定义</li>
            <li>集成了一些常用功能，如分页、行选择、行展开</li>
            <li>预设了一些常用格式的单元格，如字典项、金额、超长内容的展示</li>
          </ul>
        </Sec>
        <Sec title="例子">
          <h3>宽度适配</h3>
          <WidthDemo />
          <h3>单元格格式化</h3>
          <CellsDemo />
          <h3>分页</h3>
          <PagerDemo />
          <h3>编辑</h3>
          <EditDemo />
          <h3>行选择</h3>
          <CheckDemo />
          <h3>卡片款列表</h3>
          <CardDemo />
        </Sec>
        <Sec title="开发说明">
          <ul className={styles.ul}>
            <li>
              一般情况列宽使用几个固定尺寸：加小号（50px）、小号（70px）、中号（110px）、大号（142px）、加大号（210px），特殊情况可根据需求灵活设置
            </li>
            <li>表头高度需固定为48px（即展示单行文字时的高度），否则可能出现布局错乱</li>
            <li>
              当列表外层容器高度变化时，需要刷新纵向滚动状态。
              以下两种情况组件自动刷新：1、浏览器变化引起的高度变化；2、高度变化时引起了组件的重新渲染。
              此外的其他情况下需要手动调用组件的实例方法refreshScrollV来完成刷新。
            </li>
          </ul>
        </Sec>
        <Sec title="属性">
          <PropsTable>
            <tr>
              <td>dataSource</td>
              <td>
                数据源，为对象数组，每个对象描述一行数据，包含任意的键值对；每一行有一个唯一的key值，默认为该行的顺序号，可设置key字段来自定义该值
              </td>
              <td>object[]</td>
              <td>-</td>
            </tr>
            <tr>
              <td>columns</td>
              <td>列配置，具体参见columns配置说明</td>
              <td>object[]</td>
              <td>-</td>
            </tr>
            <tr>
              <td>loading</td>
              <td>加载状态</td>
              <td>boolean</td>
              <td></td>
            </tr>
            <tr>
              <td>pagination</td>
              <td>分页器配置，具体参见pagination配置说明</td>
              <td>object</td>
              <td></td>
            </tr>
            <tr>
              <td>onChange</td>
              <td>分页信息变化时的回调，参数为当前页码和每页数据条数</td>
              <td>{`function({ number, number })`}</td>
              <td></td>
            </tr>
            <tr>
              <td>rowSelection</td>
              <td>行选择框配置，具体参见rowSelection配置说明</td>
              <td>object</td>
              <td></td>
            </tr>
            <tr>
              <td>expandedRowRender</td>
              <td>展开行显示内容的渲染函数，参数为当前行数据</td>
              <td>{`function(object)`}</td>
              <td></td>
            </tr>
            <tr>
              <td>onExpand</td>
              <td>点击展开按钮时的回调，参数为当前操作的的行的展开状态和行数据</td>
              <td>{`function(boolean, object)`}</td>
              <td></td>
            </tr>
            <tr>
              <td>expandedRowKeys</td>
              <td>展开行的key集合（可同时展开多行时使用）</td>
              <td>[number|string]</td>
              <td></td>
            </tr>
            <tr>
              <td>onExpandedRowsChange</td>
              <td>行展开状态变化时的回调（可同时展开多行时使用）</td>
              <td>{`function([number|string])`}</td>
              <td></td>
            </tr>
            <tr>
              <td>expandedRowKey</td>
              <td>展开行的key（只能展开一行时使用）</td>
              <td>number|string</td>
              <td></td>
            </tr>
            <tr>
              <td>onExpandedRowChange</td>
              <td>行展开状态变化时的回调（只能展开一行时使用）</td>
              <td>{`function(number|string)`}</td>
              <td></td>
            </tr>
          </PropsTable>
          <h3>columns配置</h3>
          <PropsTable>
            <tr>
              <td>title</td>
              <td>列头显示内容</td>
              <td>ReactNode</td>
              <td></td>
            </tr>
            <tr>
              <td>dataIndex</td>
              <td>数据源取值的字段名</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>render</td>
              <td>自定义单元格渲染函数，参数分别为当前值，当前行数据，行序号</td>
              <td>ReactNode function(string, object, number)</td>
              <td></td>
            </tr>
            <tr>
              <td>width</td>
              <td>列的最小宽度（px）</td>
              <td>number</td>
              <td>110</td>
            </tr>
            <tr>
              <td>fixed</td>
              <td>固定列设置，可指定固定在左边或右边，'left' | 'right'</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>dict</td>
              <td>数据字典，指定数据值和显示文本的映射集合，渲染时将当前值转换为显示文本</td>
              <td>{`[{ val: string, text: string }]`}</td>
              <td></td>
            </tr>
            <tr>
              <td>lines</td>
              <td>限制最大行数，超出则显示省略号，并以气泡提示框的形式展示完整内容，可设置1~5行</td>
              <td>number</td>
              <td></td>
            </tr>
            <tr>
              <td>format</td>
              <td>格式化单元格内容，目前支持的类型有：'money'（金额）</td>
              <td>string</td>
              <td></td>
            </tr>
          </PropsTable>
          <h3>pagination配置</h3>
          <PropsTable>
            <tr>
              <td>current</td>
              <td>当前页码，从1开始</td>
              <td>number</td>
              <td></td>
            </tr>
            <tr>
              <td>pageSize</td>
              <td>每页数据条数</td>
              <td>number</td>
              <td>10</td>
            </tr>
            <tr>
              <td>total</td>
              <td>总数据条数</td>
              <td>number</td>
              <td>-</td>
            </tr>
            <tr>
              <td>pageSizeOptions</td>
              <td>每页条数可选项设置</td>
              <td>string[]</td>
              <td>{`[ '10', '20', '50' ]`}</td>
            </tr>
          </PropsTable>
          <h3>rowSelection配置</h3>
          <PropsTable>
            <tr>
              <td>selectedRowKeys</td>
              <td>选中行的key集合</td>
              <td>[number|string]</td>
              <td></td>
            </tr>
            <tr>
              <td>onChange</td>
              <td>选中行变化时的回调，参数为选中的行key集合、行数据集合</td>
              <td>{`function([number|string], [object])`}</td>
              <td></td>
            </tr>
          </PropsTable>
        </Sec>
        <Sec title="实例方法">
          <FuncsTable>
            <tr>
              <td>refreshScrollV</td>
              <td>刷新纵向滚动状态</td>
              <td></td>
            </tr>
          </FuncsTable>
        </Sec>
      </CompPage>
    );
  }
}

export default Main;
