import React from 'react';
import CompPage from 'DOC/DocPage';
import PropsTable from '../../PropsTable';
import FuncsTable from '../../FuncsTable';
import Sec from '../../Section';
import BasicDemo from './BasicDemo';
import FooterDemo from './FooterDemo';
import LoadingDemo from './LoadingDemo';
import ErrorDemo from './ErrorDemo';
import NetDemo from './NetDemo';
import styles from '../../styles.less';

class Main extends React.Component {
  render() {
    return (
      <CompPage>
        <Sec title="功能介绍">完全遮挡当前页面的对话框，用于与当前页面相关的操作或信息展示。</Sec>
        <Sec title="例子">
          <h3>基本用法</h3>
          <BasicDemo />
          <h3>自定义底部栏</h3>
          <FooterDemo />
          <h3>等待状态</h3>
          <LoadingDemo />
          <h3>错误提示</h3>
          <ErrorDemo />
          <h3>网络请求交互</h3>
          <NetDemo />
          <div className={styles.info}>
            本示例展示模态框中网络请求的一般场景：
            <br />
            1、打开模态框时，加载初始化数据，显示等待状态
            <br />
            2、提交数据时，显示等待状态
            <br />
            3、提交成功时，弹出自消失弹框提示操作成功，并关闭模态框
            <br />
            4、提交失败时，在操作按钮上方显示气泡弹框提示错误信息
          </div>
        </Sec>
        <Sec title="开发说明">
          <ul className={styles.ul}>
            <li>
              在路由切换时，由于模态框不能自动关闭，统一调用Modal的props.onCancel方法进行关闭，所以务必保证传入该属性，并且实现关闭模态框的功能。
            </li>
          </ul>
        </Sec>
        <Sec title="属性">
          <PropsTable>
            <tr>
              <td>title</td>
              <td>标题</td>
              <td>ReactNode</td>
              <td></td>
            </tr>
            <tr>
              <td>children</td>
              <td>内容</td>
              <td>ReactNode</td>
              <td></td>
            </tr>
            <tr>
              <td>width</td>
              <td>宽度</td>
              <td>number</td>
              <td>520</td>
            </tr>
            <tr>
              <td>footer</td>
              <td>
                底部栏内容，默认为确定和取消按钮（注：“确定”、“取消”指的是代表积极和消极两种含义的按钮，具体的文本可自定义。以下同义）
              </td>
              <td>ReactNode</td>
              <td></td>
            </tr>
            <tr>
              <td>onOk</td>
              <td>点击确定按钮时的回调，仅当footer为默认时有效</td>
              <td>function</td>
              <td></td>
            </tr>
            <tr>
              <td>onCancel</td>
              <td>点击取消按钮时的回调，仅当footer为默认时有效</td>
              <td>function</td>
              <td></td>
            </tr>
            <tr>
              <td>okBtnShow</td>
              <td>是否显示确定按钮，仅当footer为默认时有效</td>
              <td>boolean</td>
              <td>true</td>
            </tr>
            <tr>
              <td>cancelBtnShow</td>
              <td>是否显示取消按钮，仅当footer为默认时有效</td>
              <td>boolean</td>
              <td>true</td>
            </tr>
            <tr>
              <td>okText</td>
              <td>确定按钮的文字，仅当footer为默认时有效</td>
              <td>string</td>
              <td>'确定'</td>
            </tr>
            <tr>
              <td>cancelText</td>
              <td>取消按钮的文字，仅当footer为默认时有效</td>
              <td>string</td>
              <td>'取消'</td>
            </tr>
            <tr>
              <td>loading</td>
              <td>等待状态</td>
              <td>boolean</td>
              <td></td>
            </tr>
            <tr>
              <td>className</td>
              <td>附加在根元素的类名</td>
              <td>string</td>
              <td></td>
            </tr>
          </PropsTable>
        </Sec>
        <Sec title="实例方法">
          <FuncsTable>
            <tr>
              <td>showOkTips</td>
              <td>显示确定按钮的错误提示信息，参数为信息文本</td>
              <td>string</td>
            </tr>
          </FuncsTable>
        </Sec>
      </CompPage>
    );
  }
}

export default Main;
