import React from 'react';
import { Row, Col } from 'antd';
import moment from 'moment';
import MonthPicker from 'COMPONENT/MonthPicker';
import { momentToMonth, monthToMoment, MONTH_FORMAT } from 'COMPONENT/utils/form';

class Main extends React.Component {
  state = {
    startMonth: null,
    endMonth: null,
  };

  /**
   * 获取要禁用的开始月份值
   * @param  {string} startMonth 过滤前的月份值
   * @return {boolean} 是否要禁用
   */
  getDisabledStartMonth = (startMonth) => {
    const { endMonth } = this.state;
    if (!startMonth || !endMonth) {
      return false;
    }
    return moment(startMonth, MONTH_FORMAT).valueOf() > moment(endMonth, MONTH_FORMAT).valueOf();
  };

  /**
   * 获取要禁用的结束月份值
   * @param  {string} endMonth   过滤前的月份值
   * @return {boolean}      是否要禁用
   */
  getDisabledEndMonth = (endMonth) => {
    const { startMonth } = this.state;
    if (!endMonth || !startMonth) {
      return false;
    }
    return moment(endMonth, MONTH_FORMAT).valueOf() < moment(startMonth, MONTH_FORMAT).valueOf();
  };

  render() {
    const { startMonth, endMonth } = this.state;
    return (
      <div style={{ width: 420 }}>
        <Row gutter={20}>
          <Col span={12}>
            <MonthPicker
              placeholder="开始月份"
              value={startMonth}
              onChange={(val) => this.setState({ startMonth: val })}
              disabledMonth={(val) => this.getDisabledStartMonth(val)}
            />
          </Col>
          <Col span={12}>
            <MonthPicker
              placeholder="结束月份"
              value={endMonth}
              onChange={(val) => this.setState({ endMonth: val })}
              disabledMonth={(val) => this.getDisabledEndMonth(val)}
            />
          </Col>
        </Row>
      </div>
    );
  }
}

export default Main;
