import React from 'react';
import { connect } from 'dva';
import TextLink from 'COMPONENT/TextLink';
import CompPage from 'DOC/DocPage';
import PropsTable from '../../PropsTable';
import FuncsTable from '../../FuncsTable';
import Sec from '../../Section';
import BasicDemo from './BasicDemo';
import SizeDemo from './SizeDemo';
import DisableDemo from './DisableDemo';
import DisableValueDemo from './DisableValueDemo';
import StartEndDemo from './StartEndDemo';
import styles from '../../styles.less';
import styleVars from 'COMPONENT/theme/variables';

class Main extends React.Component {
  openPage = (params) => {
    const { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: params,
    });
  };

  render() {
    return (
      <CompPage>
        <Sec title="功能介绍">月份输入控件，可选择或者直接输入月份，格式为：YYYY-MM（年-月）。</Sec>
        <Sec title="例子">
          <h3>基本用法</h3>
          <BasicDemo />
          <h3>三种大小</h3>
          <SizeDemo />
          <h3>禁用状态</h3>
          <DisableDemo />
          <h3>输入限制</h3>
          <DisableValueDemo />
          <div className={styles.info}>本示例展示禁止选择当月之前的月份。</div>
          <h3>开始和结束月份</h3>
          <StartEndDemo />
          <div className={styles.info}>本示例展示限制开始月份不能超过结束月份。</div>
        </Sec>
        <Sec title="开发说明">
          <ul className={styles.ul}>
            <li>
              一般情况下，如果同时有开始月份和结束月份输入项（或其他名称不同，首尾制约的月份输入项），并且是要保存的，需要做输入限制，使输入的开始月份不超过结束月份
            </li>
          </ul>
        </Sec>
        <Sec title="属性">
          <PropsTable>
            <tr>
              <td>disabled</td>
              <td>禁用状态</td>
              <td>boolean</td>
              <td></td>
            </tr>
            <tr>
              <td>disabledMonth</td>
              <td>获取要禁用的月份</td>
              <td>boolean function(value: string)</td>
              <td></td>
            </tr>
            <tr>
              <td>getPopupContainer</td>
              <td>获取承载弹框的DOM元素的函数</td>
              <td>DOMElement function</td>
              <td></td>
            </tr>
            <tr>
              <td>onChange</td>
              <td>值变化时的回调</td>
              <td>function(value: string)</td>
              <td></td>
            </tr>
            <tr>
              <td>onFocus</td>
              <td>得到焦点时的回调</td>
              <td>function</td>
              <td></td>
            </tr>
            <tr>
              <td>onBlur</td>
              <td>失去焦点时的回调</td>
              <td>function</td>
              <td></td>
            </tr>
            <tr>
              <td>onClick</td>
              <td>点击事件回调</td>
              <td>function(e: object)</td>
              <td></td>
            </tr>
            <tr>
              <td>placeholder</td>
              <td>占位文字</td>
              <td>string</td>
              <td>'请选择月份'</td>
            </tr>
            <tr>
              <td>size</td>
              <td>大小，可选'small'|'xSmall'</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>className</td>
              <td>附加在根元素上的类名</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>dropdownClassName</td>
              <td>附加在下拉菜单根元素的类名</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>style</td>
              <td>附加在根元素上的样式</td>
              <td>object</td>
              <td></td>
            </tr>
            <tr>
              <td>dropdownStyle</td>
              <td>附加在下拉菜单根元素的样式</td>
              <td>object</td>
              <td></td>
            </tr>
            <tr>
              <td>title</td>
              <td>提示性的标题</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>validator</td>
              <td>
                校验器配置，请参考
                <TextLink onClick={() => this.openPage({ path: '/docValidator' })}>Validator</TextLink>组件的属性配置
              </td>
              <td>object</td>
              <td></td>
            </tr>
            <tr>
              <td>value</td>
              <td>月份值</td>
              <td>string</td>
              <td></td>
            </tr>
          </PropsTable>
        </Sec>
      </CompPage>
    );
  }
}

export default connect()(Main);
