import React from 'react';
import { connect } from 'dva';
import Button from 'COMPONENT/Button';
import CompPage from 'DOC/DocPage';
import PropsTable from '../../PropsTable';
import FuncsTable from '../../FuncsTable';
import Sec from '../../Section';
import styles from '../../styles.less';

class Main extends React.Component {
  openPage = (params) => {
    const { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: params,
    });
  };

  render() {
    return (
      <CompPage>
        <Sec title="功能介绍">
          工作区（为了方便表述，这里将系统界面中除去左侧菜单栏、上方顶部栏及标签页导航栏的主页面区域称为工作区。以下同义）中一个页面的通用框架。特点有：
          <ul className={styles.ul}>
            <li>
              布局包含四个区域：标题栏、顶部工具栏、主体内容区、底部工具栏；其中标题栏默认显示导航信息；各区域内容皆可自定义
            </li>
            <li>
              高度适配有两种模式： 1、适应内容高度，页面高度小于工作区时充满工作区，页面高度大于工作区时页面可滚动；
              2、适应窗口高度，页面高度始终充满工作区，内容超出时隐藏（一般会设置页面内的局部滚动来显示内容）
            </li>
            <li>包含通用的基础功能，如：标题栏导航信息、页面等待状态控制、底部工具栏操作按钮、错误信息提示框等</li>
          </ul>
        </Sec>
        <Sec title="例子">
          <h3>页面结构</h3>
          <Button type="primary" onClick={() => this.openPage({ path: '/docPageLayout' })}>
            查看示例
          </Button>
          <h3>适应工作区高度</h3>
          <Button type="primary" onClick={() => this.openPage({ path: '/docPageFitWindow' })}>
            查看示例
          </Button>
          <h3>页面层级</h3>
          <Button type="primary" onClick={() => this.openPage({ path: '/docPageLevel' })}>
            查看示例
          </Button>
          <h3>网络请求交互</h3>
          <Button type="primary" onClick={() => this.openPage({ path: '/docPageNet' })}>
            查看示例
          </Button>
          <div className={styles.info}>
            本示例展示页面中网络请求的一般场景：
            <br />
            1、打开页面时，加载初始化数据，显示等待状态
            <br />
            2、提交数据时，显示等待状态
            <br />
            3、提交成功时，弹出自消失弹框提示操作成功
            <br />
            4、提交失败时，在操作按钮上方显示气泡弹框提示错误信息
          </div>
        </Sec>
        <Sec title="开发说明">
          <ul className={styles.ul}>
            <li>
              标题栏非自定义时（即标题栏显示默认的导航信息），需要将页面的路由信息传入Page组件，如：
              {`route={this.props.route}`}
            </li>
          </ul>
        </Sec>
        <Sec title="属性">
          <PropsTable>
            <tr>
              <td>route</td>
              <td>路由信息</td>
              <td>object</td>
              <td></td>
            </tr>
            <tr>
              <td>fitWindowHeight</td>
              <td>是否为适应窗口高度模式</td>
              <td>boolean</td>
              <td></td>
            </tr>
            <tr>
              <td>loading</td>
              <td>页面等待状态</td>
              <td>boolean</td>
              <td></td>
            </tr>
            <tr>
              <td>error</td>
              <td>页面加载错误信息</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>title</td>
              <td>标题栏内容，为false时不显示标题栏</td>
              <td>boolean | ReactNode</td>
              <td>导航信息组件集合</td>
            </tr>
            <tr>
              <td>name</td>
              <td>页面名称，显示在导航栏中对应于当前页面的导航项</td>
              <td>ReactNode</td>
              <td>页面路由中配置的name</td>
            </tr>
            <tr>
              <td>headerToolBar</td>
              <td>顶部工具栏内容</td>
              <td>ReactNode</td>
              <td></td>
            </tr>
            <tr>
              <td>footer</td>
              <td>
                底部工具栏内容，为true时显示默认内容，即确定和取消按钮（注：“确定”、“取消”指的是代表积极和消极两种含义的按钮，具体的文本可自定义。以下同义）
              </td>
              <td>boolean | ReactNode</td>
              <td></td>
            </tr>
            <tr>
              <td>onOk</td>
              <td>点确定按钮时的回调函数，仅当footer为true时有效</td>
              <td>function</td>
              <td></td>
            </tr>
            <tr>
              <td>onCancel</td>
              <td>点取消按钮时的回调函数，仅当footer为true时有效</td>
              <td>function</td>
              <td></td>
            </tr>
            <tr>
              <td>okBtnShow</td>
              <td>是否显示确定按钮，仅当footer为true时有效</td>
              <td>boolean</td>
              <td>true</td>
            </tr>
            <tr>
              <td>cancelBtnShow</td>
              <td>是否显示取消按钮，仅当footer为true时有效</td>
              <td>boolean</td>
              <td>false</td>
            </tr>
            <tr>
              <td>okBtnText</td>
              <td>确定按钮的文字，仅当footer为true时有效</td>
              <td>string</td>
              <td>'提交'</td>
            </tr>
            <tr>
              <td>cancelBtnText</td>
              <td>取消按钮的文字，仅当footer为true时有效</td>
              <td>string</td>
              <td>'取消'</td>
            </tr>
            <tr>
              <td>className</td>
              <td>附加在根元素的类名</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>hasBackground</td>
              <td>是否有背景</td>
              <td>boolean</td>
              <td>true</td>
            </tr>
          </PropsTable>
        </Sec>
      </CompPage>
    );
  }
}

export default connect()(Main);
