import React from 'react';
import CompPage from 'DOC/DocPage';
import PropsTable from '../../PropsTable';
import FuncsTable from '../../FuncsTable';
import Sec from '../../Section';
import TriggerDemo from './TriggerDemo';
import IconDemo from './IconDemo';
import PlacementDemo from './PlacementDemo';
import ArrowDemo from './ArrowDemo';
import TipsDemo from './TipsDemo';
import ConfirmDemo from './ConfirmDemo';
import ActionDemo from './ActionDemo';
import styles from '../../styles.less';
import styleVars from 'COMPONENT/theme/variables';

class Main extends React.Component {
  render() {
    return (
      <CompPage>
        <Sec title="功能介绍">
          气泡弹框，悬浮于按钮或者其他交互元素上，一般用于较弱的信息提示，或简单快速的交互操作。
          <ul className={styles.ul}>
            <li>封装了提示框，常用于提交数据错误提示等场景</li>
            <li>封装了确认框，常用于须谨慎的操作前的提醒确认</li>
          </ul>
        </Sec>
        <Sec title="例子">
          <h3>两种触发方式</h3>
          <TriggerDemo />
          <h3>三种预设图标</h3>
          <IconDemo />
          <h3>弹出位置</h3>
          <PlacementDemo />
          <h3>箭头位置</h3>
          <ArrowDemo />
          <h3>提示框</h3>
          <TipsDemo />
          <h3>确认框</h3>
          <ConfirmDemo />
          <h3>用于交互操作</h3>
          <ActionDemo />
        </Sec>
        <Sec title="开发说明">
          弹框默认附加在document.body，当目标元素处于局部滚动的区域时，会出现弹框分离的情况，可通过getPopupContainer属性指定弹框附加的元素。
        </Sec>
        <Sec title="属性">
          <PropsTable>
            <tr>
              <td>title</td>
              <td>弹框标题</td>
              <td>ReactNode</td>
              <td></td>
            </tr>
            <tr>
              <td>content</td>
              <td>弹框内容</td>
              <td>ReactNode</td>
              <td></td>
            </tr>
            <tr>
              <td>trigger</td>
              <td>打开弹框的触发方式，不传为鼠标悬浮触发，可选'click'</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>icon</td>
              <td>弹框图标类型，可选'info' | 'warning' | 'error'</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>maxWidth</td>
              <td>弹框最大宽度（px）</td>
              <td>number</td>
              <td></td>
            </tr>
            <tr>
              <td>visible</td>
              <td>弹框显示状态</td>
              <td>boolean</td>
              <td></td>
            </tr>
            <tr>
              <td>onClose</td>
              <td>关闭弹框时的回调</td>
              <td>function</td>
              <td></td>
            </tr>
            <tr>
              <td>onVisibleChange</td>
              <td>弹框显示状态变化时的回调</td>
              <td>function(boolean)</td>
              <td></td>
            </tr>
            <tr>
              <td>placement</td>
              <td>
                弹框在目标元素上弹出的位置，可选项有 'topLeft' | 'top' | 'topRight' | 'leftTop' | 'left' | 'leftBottom'
                | 'rightTop' | 'right' | 'rightBottom' | 'bottomLeft' | 'bottom' | 'bottomRight'
              </td>
              <td>string</td>
              <td>'top'</td>
            </tr>
            <tr>
              <td>arrowPointAtCenter</td>
              <td>弹框的箭头固定指向目标元素中心</td>
              <td>boolean</td>
              <td></td>
            </tr>
            <tr>
              <td>getPopupContainer</td>
              <td>获取承载弹框的DOM元素，参数为目标元素，默认返回目标元素的父元素</td>
              <td>Element function(Element)</td>
              <td></td>
            </tr>
            <tr>
              <td>children</td>
              <td>目标元素内容</td>
              <td>ReactNode</td>
              <td>-</td>
            </tr>
            <tr>
              <td>footer</td>
              <td>弹框底部栏</td>
              <td>ReactNode</td>
              <td></td>
            </tr>
            <tr>
              <td>okBtnShow</td>
              <td>弹框是否显示确定按钮</td>
              <td>boolean</td>
              <td></td>
            </tr>
            <tr>
              <td>cancelBtnShow</td>
              <td>弹框是否显示取消按钮</td>
              <td>boolean</td>
              <td></td>
            </tr>
            <tr>
              <td>onOk</td>
              <td>弹框确定按钮点击回调</td>
              <td>function</td>
              <td></td>
            </tr>
            <tr>
              <td>onCancel</td>
              <td>弹框取消按钮点击回调</td>
              <td>function</td>
              <td></td>
            </tr>
            <tr>
              <td>okBtnText</td>
              <td>弹框确定按钮文字</td>
              <td>string</td>
              <td>'确定'</td>
            </tr>
            <tr>
              <td>cancelBtnText</td>
              <td>弹框取消按钮文字</td>
              <td>string</td>
              <td>'取消'</td>
            </tr>
            <tr>
              <td>overlayClassName</td>
              <td>附加在弹框根元素的类名</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>overlayStyle</td>
              <td>附加在弹框根元素的样式</td>
              <td>object</td>
              <td></td>
            </tr>
          </PropsTable>
          <h3>提示框属性</h3>
          <PropsTable>
            <tr>
              <td>content</td>
              <td>弹框内容</td>
              <td>ReactNode</td>
              <td></td>
            </tr>
            <tr>
              <td>icon</td>
              <td>弹框图标类型，可选'info' | 'warning' | 'error'</td>
              <td>string</td>
              <td>'warning'</td>
            </tr>
            <tr>
              <td>maxWidth</td>
              <td>弹框最大宽度（px）</td>
              <td>number</td>
              <td>180</td>
            </tr>
            <tr>
              <td>visible</td>
              <td>弹框显示状态</td>
              <td>boolean</td>
              <td></td>
            </tr>
            <tr>
              <td>onClose</td>
              <td>关闭弹框时的回调</td>
              <td>function</td>
              <td></td>
            </tr>
            <tr>
              <td>onVisibleChange</td>
              <td>弹框显示状态变化时的回调</td>
              <td>function(boolean)</td>
              <td></td>
            </tr>
            <tr>
              <td>placement</td>
              <td>弹框在目标元素上弹出的位置，可选项有 'topLeft' | 'top' | 'topRight'</td>
              <td>string</td>
              <td>'top'</td>
            </tr>
            <tr>
              <td>arrowPointAtCenter</td>
              <td>弹框的箭头固定指向目标元素中心</td>
              <td>boolean</td>
              <td></td>
            </tr>
            <tr>
              <td>getPopupContainer</td>
              <td>获取承载弹框的DOM元素，参数为目标元素，默认返回目标元素的父元素</td>
              <td>Element function(Element)</td>
              <td></td>
            </tr>
            <tr>
              <td>children</td>
              <td>目标元素内容</td>
              <td>ReactNode</td>
              <td>-</td>
            </tr>
            <tr>
              <td>btnText</td>
              <td>弹框确定按钮文字</td>
              <td>string</td>
              <td>'知道了'</td>
            </tr>
            <tr>
              <td>overlayClassName</td>
              <td>附加在弹框根元素的类名</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>overlayStyle</td>
              <td>附加在弹框根元素的样式</td>
              <td>object</td>
              <td></td>
            </tr>
          </PropsTable>
          <h3>确认框属性</h3>
          <PropsTable>
            <tr>
              <td>content</td>
              <td>弹框内容</td>
              <td>ReactNode</td>
              <td></td>
            </tr>
            <tr>
              <td>icon</td>
              <td>弹框图标类型，可选'info' | 'warning' | 'error'</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>maxWidth</td>
              <td>弹框最大宽度（px）</td>
              <td>number</td>
              <td></td>
            </tr>
            <tr>
              <td>visible</td>
              <td>弹框显示状态</td>
              <td>boolean</td>
              <td></td>
            </tr>
            <tr>
              <td>onVisibleChange</td>
              <td>弹框显示状态变化时的回调</td>
              <td>function(boolean)</td>
              <td></td>
            </tr>
            <tr>
              <td>placement</td>
              <td>弹框在目标元素上弹出的位置，可选项有 'topLeft' | 'top' | 'topRight'</td>
              <td>string</td>
              <td>'top'</td>
            </tr>
            <tr>
              <td>arrowPointAtCenter</td>
              <td>弹框的箭头固定指向目标元素中心</td>
              <td>boolean</td>
              <td></td>
            </tr>
            <tr>
              <td>getPopupContainer</td>
              <td>获取承载弹框的DOM元素，参数为目标元素，默认返回目标元素的父元素</td>
              <td>Element function(Element)</td>
              <td></td>
            </tr>
            <tr>
              <td>children</td>
              <td>目标元素内容</td>
              <td>ReactNode</td>
              <td>-</td>
            </tr>
            <tr>
              <td>onOk</td>
              <td>弹框确定按钮点击回调</td>
              <td>function</td>
              <td></td>
            </tr>
            <tr>
              <td>onCancel</td>
              <td>弹框取消按钮点击回调</td>
              <td>function</td>
              <td></td>
            </tr>
            <tr>
              <td>okBtnText</td>
              <td>弹框确定按钮文字</td>
              <td>string</td>
              <td>'确定'</td>
            </tr>
            <tr>
              <td>cancelBtnText</td>
              <td>弹框取消按钮文字</td>
              <td>string</td>
              <td>'取消'</td>
            </tr>
            <tr>
              <td>overlayClassName</td>
              <td>附加在弹框根元素的类名</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>overlayStyle</td>
              <td>附加在弹框根元素的样式</td>
              <td>object</td>
              <td></td>
            </tr>
          </PropsTable>
        </Sec>
      </CompPage>
    );
  }
}

export default Main;
