import React from 'react';
import Search from 'COMPONENT/Search';

const productTypeDict = [
  { val: 'oto', text: '一对一' },
  { val: 'otm', text: '一对多' },
];
const otmProductTypeDict = [
  { val: '2', text: '一对二' },
  { val: '3', text: '一对三' },
  { val: '4', text: '一对四' },
  { val: '5', text: '一对五' },
];
const gradeDict = [
  { val: '1', text: '幼儿园' },
  { val: '2', text: '一年级' },
  { val: '3', text: '二年级' },
];
const subjectDict = [
  { val: '1', text: '语文' },
  { val: '2', text: '数学' },
];
const otmProductYearDict = [
  { val: '1', text: '2017' },
  { val: '2', text: '2018' },
  { val: '3', text: '2019' },
];

class Main extends React.Component {
  state = {
    inputValue: null,
    productType: null,
    grade: null,
    otmProductType: null,
    subject: null,
    otmProductYear: null,
  };

  onInputChange = (inputValue) => {
    this.setState({ inputValue });
  };

  onSearch = (searchKeys) => {
    let newSearchKeys = { ...searchKeys };
    if (searchKeys.productType !== 'otm') {
      newSearchKeys = { ...newSearchKeys, otmProductType: null, otmProductYear: null };
    }
    this.setState(newSearchKeys);
    console.log(`查询条件：`, newSearchKeys);
  };

  render() {
    const { inputValue, productType, grade, otmProductType, subject, otmProductYear } = this.state;
    let conditions = [
      { title: '产品类型', field: 'productType', dict: productTypeDict },
      { title: '年级', field: 'grade', dict: gradeDict, isMultiSelect: true },
    ];
    let more = [{ title: '科目', field: 'subject', dict: subjectDict }];
    if (productType === 'otm') {
      conditions.push({ title: '一对多产品类型', field: 'otmProductType', dict: otmProductTypeDict });
      more.push({ title: '一对多产品年份', field: 'otmProductYear', dict: otmProductYearDict, isMultiSelect: true });
    }
    let moreTitle = [];
    let moreDict = {};
    let moreIsMultiSelect = {};
    more.forEach((m) => {
      moreTitle.push({ text: m.title, val: m.field });
      moreDict[m.field] = m.dict;
      moreIsMultiSelect[m.field] = m.isMultiSelect;
    });
    const moreCondition = { type: 'moreSelect', title: moreTitle, dict: moreDict, isMultiSelect: moreIsMultiSelect };
    conditions.push(moreCondition);

    const selectedDatas = {
      inputValue,
      productType,
      grade,
      otmProductType,
      subject,
      otmProductYear,
    };
    return (
      <div>
        <Search
          conditions={conditions}
          selectedDatas={selectedDatas}
          onSearch={this.onSearch}
          onInputChange={this.onInputChange}
          isShowResult={false}
          conditionWidth={150}
          placeholder="请输入条件搜索"
        />
      </div>
    );
  }
}

export default Main;
