import React from 'react';
import CompPage from 'DOC/DocPage';
import PropsTable from '../../PropsTable';
import FuncsTable from '../../FuncsTable';
import Sec from '../../Section';
import BasicDemo from './BasicDemo';
import AsyncSearchKeysDemo from './AsyncSearchKeysDemo';
import LinkageDemo from './LinkageDemo';
import styles from '../../styles.less';

const demoTitleCfg = {
  style: { marginTop: 30 },
};

class Main extends React.Component {
  render() {
    return (
      <CompPage>
        <Sec title="功能介绍">
          <ul className={styles.ul}>
            <li>输入型查询条件为统一输入框</li>
            <li>单选或者多选查询条件以平铺形式展现</li>
          </ul>
        </Sec>
        <Sec title="例子">
          <h3 {...demoTitleCfg}>基本用法</h3>
          <BasicDemo />
          <h3 {...demoTitleCfg}>异步设置默认值</h3>
          <AsyncSearchKeysDemo />
          <div className={styles.info}>异步设置默认值需要手动实现重置。</div>
          <h3 {...demoTitleCfg}>联动</h3>
          <LinkageDemo />
        </Sec>
        <Sec title="开发说明">暂无。</Sec>
        <Sec title="属性">
          <PropsTable>
            <tr>
              <td>conditions</td>
              <td>筛选条件配置</td>
              <td>
                <pre>
                  {`[
  {
    type: string - 类型，'default'|'moreSelect'，默认为'default'
    1、当type为'default'时:
      title: string - 筛选条件名称，必需
      field: string - 筛选条件字段值，必需
      dict: string - 选项字典，必需。格式为：
      [{
        text: string - 名称，必需
        val: strign - 值，必需
      }]
      isMultiSelect: boolean - 是否多选，默认为false
      hideUnlimited: boolean - 是否隐藏“不限”按钮，默认为false
    2、当type为'moreSelect'时，为更多筛选:
      title: object[] - 筛选条件集合，必需。格式为：
      [{
        text: string - 名称，必需
        val: string- 值，必需
      }]
      dict: object - 选项字典，必需。格式为：
      { key: object }，其中key对应title中的val，值为字典，格式为：[{ text, val }]
      isMultiSelect: object - 是否多选。格式为：
      { key: boolean }，其中key对应title中的val，值默认为false
      hideUnlimited: object - 是否隐藏“不限”按钮。格式为：
      { key: boolean }，其中key对应title中的val，值默认为false
  }
]`}
                </pre>
              </td>
              <td></td>
            </tr>
            <tr>
              <td>selectedDatas</td>
              <td>搜索条件键值对，其中输入框key固定为'inputValue'</td>
              <td>object</td>
              <td>-</td>
            </tr>
            <tr>
              <td>onSearch</td>
              <td>搜索回调，参数为搜索条件键值对</td>
              <td>function(object)</td>
              <td>-</td>
            </tr>
            <tr>
              <td>onReset</td>
              <td>点击重置时的回调，不传时默认将搜索条件值重置为最初传入的selectedDatas</td>
              <td>function</td>
              <td></td>
            </tr>
            <tr>
              <td>onInputChange</td>
              <td>输入框变化回调，参数为输入框值</td>
              <td>function(string)</td>
              <td>-</td>
            </tr>
            <tr>
              <td>isShowResult</td>
              <td>是否显示搜索结果</td>
              <td>boolean</td>
              <td>true</td>
            </tr>
            <tr>
              <td>conditionWidth</td>
              <td>筛选条件名称区域宽度（px）</td>
              <td>number</td>
              <td>100</td>
            </tr>
            <tr>
              <td>placeholder</td>
              <td>输入框占位文字</td>
              <td>string</td>
              <td>'请输入'</td>
            </tr>
          </PropsTable>
        </Sec>
      </CompPage>
    );
  }
}

export default Main;
