import React from 'react';
import Section from '../../Section';
import CompPage from 'DOC/DocPage';
import Demo from './demo';
import styles from '../../styles.less';
import PropsTable from '../../PropsTable';

export default class extends React.Component {
  render() {
    return (
      <CompPage>
        <Section title="功能介绍">
          <ul className={styles.ul}>
            <li>新的筛选器，用以替换以前的Search</li>
            <li>去掉了输入框，去掉了全部选项，去掉了超出一行收起展开功能</li>
            <li>更多筛选改为：超出四行默认收起所有更多项</li>
            <li>拥有多选和单选两种选项</li>
            <li>完全受控组件</li>
          </ul>
        </Section>
        <Section title="例子">
          <Demo />
        </Section>
        <Section title="属性">
          <PropsTable>
            <tr>
              <td>config</td>
              <td>筛选项配置</td>
              <td>Array&lt;object&gt; {'{label, type, dict, field, value}'}</td>
              <td>
                <pre>
                  <code>
                    {`示例数据：
                      [
                        {
                          label: '1行',
                          type: 'multi', //type为multi则为多选
                          dict: [
                            { text: '阿斯蒂芬', val: '1' },
                            { text: '白求恩', val: '2' },
                            { text: '切特', val: '3' },
                            { text: '帮我特', val: '4' },
                            { text: '帮我让她改变', val: '5' },
                            { text: '体育教育', val: '6' },
                            { text: '同意呢体育', val: '7' },
                            { text: '无人挺好', val: '8' },
                            { text: '你也内内', val: '9' },
                          ],
                          field: 'firstRow',
                          value: ['1', '2']
                        },
                        {
                          label: '2行',
                          dict: [
                            { text: '他嫩家', val: '1' },
                            { text: '日语课', val: '2' },
                            { text: '美人鱼', val: '3' },
                          ],
                          field: 'secondRow',
                          value: '2'
                        }
                      ]`}
                  </code>
                </pre>
              </td>
            </tr>
            <tr>
              <td>config.label</td>
              <td>左边的行文字</td>
              <td>String</td>
              <td>-</td>
            </tr>
            <tr>
              <td>config.type</td>
              <td>单选/多选</td>
              <td>String</td>
              <td>默认单选，multi为多选</td>
            </tr>
            <tr>
              <td>config.dict</td>
              <td>数据字典</td>
              <td>Array</td>
              <td>用于渲染选项。注意:每一项的val是用于react的key的，所以不能重复</td>
            </tr>
            <tr>
              <td>config.field</td>
              <td>onChange返回keyValue时，对应的key值</td>
              <td>String</td>
              <td>-</td>
            </tr>
            <tr>
              <td>config.value</td>
              <td>该筛选项的值</td>
              <td>-</td>
              <td>-</td>
            </tr>
            <tr>
              <td>onChange</td>
              <td>改变选中项的回调</td>
              <td>function(Object)</td>
              <td>触发时传入一个新的选中数据{'{ field: value }'}对象</td>
            </tr>
            <tr>
              <td>mode</td>
              <td>分为白色和深色(dark)主题，默认白色，只在背景颜色上有区别，</td>
              <td>String</td>
              <td>-</td>
            </tr>
            <tr>
              <td>style</td>
              <td>容器样式</td>
              <td>Object</td>
              <td>-</td>
            </tr>
          </PropsTable>
        </Section>
      </CompPage>
    );
  }
}
