import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import SearchButton from 'COMPONENT/SearchButton';
import Button from 'COMPONENT/Button';
import Input from 'COMPONENT/Input';
import Select from 'COMPONENT/Select';
import List from 'COMPONENT/List';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_XL } = List;

class Demo extends React.Component {
  constructor(props) {
    super(props);
    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.onSearch = this.onSearch.bind(this);
    this.onReset = this.onReset.bind(this);
    this.state = {
      loading: false,
      name: '',
      branch: null,
      campus: null,
      year: null,
      quarter: null,
      grade: null,
      subject: null,
    };
    this.columns = [
      { title: '序号', dataIndex: 'number', key: 'number', width: COL_WIDTH_S },
      { title: '班级名称', dataIndex: 'lectureName', key: 'lectureName', width: COL_WIDTH_XL },
      { title: '分公司', dataIndex: 'branchName', key: 'branchName', width: COL_WIDTH_M },
      { title: '学习中心', dataIndex: 'campusName', key: 'campusName', width: COL_WIDTH_M },
      { title: '年份', dataIndex: 'yearName', key: 'yearName', width: COL_WIDTH_M },
      { title: '季度', dataIndex: 'quarterName', key: 'quarterName', width: COL_WIDTH_M },
      { title: '年级', dataIndex: 'gradeName', key: 'gradeName', width: COL_WIDTH_M },
      { title: '科目', dataIndex: 'subjectName', key: 'subjectName', width: COL_WIDTH_M },
    ];
  }
  componentDidMount() {
    this.init();
  }

  async init() {
    this.setState({ loading: true });
    await new Promise(this.initDict);
    this.setState({ loading: false });
  }

  initDict(resolve) {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['branch', 'campus', 'year', 'quarter', 'grade', 'subject'],
        callback() {
          resolve();
        },
      },
    });
  }

  async onSearch() {
    this.setState({ loading: true });
    await new Promise((resolve) => {
      setTimeout(function () {
        resolve();
      }, 1000);
    });
    this.setState({ loading: false });
  }

  onReset() {
    showMessage('重置！', 'success');
  }

  render() {
    return (
      <Page
        loading={this.state.loading}
        route={this.props.route}
        headerToolBar={
          <div>
            <Button type="primary" style={{ marginRight: 20 }}>
              新增
            </Button>
            <Button type="default" style={{ marginRight: 20 }}>
              导出
            </Button>
            <SearchButton
              girdConfig={[2, 1, 1, 1, 1, 1, 3]}
              searchBtnLoading={this.state.loading}
              onSearch={this.onSearch}
              onReset={this.onReset}
            >
              <Input placeholder="请输入班级名称" />
              <Select dict={this.props.branchDict} placeholder="请选择分公司" />
              <Select dict={this.props.campusDict} placeholder="请选择学习中心" />
              <Select dict={this.props.yearDict} placeholder="请选择年份" />
              <Select dict={this.props.quarterDict} placeholder="请选择季度" />
              <Select dict={this.props.gradeDict} placeholder="请选择年级" />
              <Select dict={this.props.subjectDict} placeholder="请选择科目" />
            </SearchButton>
          </div>
        }
      >
        <List columns={this.columns} dataSource={[]} />
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    branchDict: state.dict.branch,
    campusDict: state.dict.campus,
    yearDict: state.dict.year,
    quarterDict: state.dict.quarter,
    gradeDict: state.dict.grade,
    subjectDict: state.dict.subject,
  };
};

export default connect(mapStateToProps)(Demo);
