import React from 'react';
import { connect } from 'dva';
import TextLink from 'COMPONENT/TextLink';
import CompPage from 'DOC/DocPage';
import PropsTable from '../../PropsTable';
import FuncsTable from '../../FuncsTable';
import Sec from '../../Section';
import BasicDemo from './BasicDemo';
import SizeDemo from './SizeDemo';
import GroupDemo from './GroupDemo';
import DisableDemo from './DisableDemo';
import SearchDemo from './SearchDemo';
import LinkageDemo from './LinkageDemo';
import CustomSelectModeDemo from './CustomSelectModeDemo';
import CustomSearchDemo from './CustomSearchDemo';
import TextInValueDemo from './TextInValueDemo';
import ServerSideSearchDemo from './ServerSideSearchDemo';
import styles from '../../styles.less';
import styleVars from 'COMPONENT/theme/variables';
import { SELECT_NOT_FOUND_TIPS } from 'COMPONENT/utils/form/config';

class Main extends React.Component {
  openPage = (params) => {
    const { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: params,
    });
  };

  render() {
    return (
      <CompPage>
        <Sec title="功能介绍">下拉选择框，替代React自带的select组件。</Sec>
        <Sec title="例子">
          <h3>基本用法</h3>
          <BasicDemo />
          <h3>三种大小</h3>
          <SizeDemo />
          <h3>选项分组</h3>
          <GroupDemo />
          <h3>禁用</h3>
          <DisableDemo />
          <h3>联动</h3>
          <LinkageDemo />
          <h3>自定义选择方式</h3>
          <CustomSelectModeDemo />
          <h3>搜索</h3>
          <SearchDemo />
          <h3>自定义搜索</h3>
          <CustomSearchDemo />
          <h3>显示选项以外的值</h3>
          <TextInValueDemo />
          <h3>服务器端搜索</h3>
          <ServerSideSearchDemo />
        </Sec>
        <Sec title="开发说明">暂无。</Sec>
        <Sec title="属性">
          <PropsTable>
            <tr>
              <td>className</td>
              <td>附加在根元素上的类名</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>dict</td>
              <td>选项字典，选项或选项组的集合</td>
              <td>
                <pre>
                  {`[
  {
    val: string - 值，仅选项使用，必需
    text: string - 显示文字，仅选项使用，必需
    disabled: boolean - 禁用状态，仅选项使用
    title: string - 提示性标题
    label: string - 组名，仅选项组使用，必需
    options: object[] - 选项集合，仅选项组使用，必需
  }
]`}
                </pre>
              </td>
              <td></td>
            </tr>
            <tr>
              <td>dropdownClassName</td>
              <td>附加在下拉菜单根元素的类名</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>disabled</td>
              <td>禁用状态</td>
              <td>boolean</td>
              <td></td>
            </tr>
            <tr>
              <td>filter</td>
              <td>自定义的搜索函数</td>
              <td>boolean function(searchKey: string, option: object)</td>
              <td></td>
            </tr>
            <tr>
              <td>getPopupContainer</td>
              <td>获取承载弹框的DOM元素的函数</td>
              <td>DOMElement function</td>
              <td></td>
            </tr>
            <tr>
              <td>noOptionTips</td>
              <td>无选项提示文字</td>
              <td>string</td>
              <td>{SELECT_NOT_FOUND_TIPS}</td>
            </tr>
            <tr>
              <td>searchNotFoundTips</td>
              <td>搜索无匹配结果时提示文字</td>
              <td>string</td>
              <td>{SELECT_NOT_FOUND_TIPS}</td>
            </tr>
            <tr>
              <td>searchPlaceholder</td>
              <td>搜索框的占位文字</td>
              <td>string</td>
              <td>'请输入内容搜索'</td>
            </tr>
            <tr>
              <td>onChange</td>
              <td>值变化时的回调</td>
              <td>function(value: string[])</td>
              <td></td>
            </tr>
            <tr>
              <td>onSelect</td>
              <td>选中值时的回调</td>
              <td>function(key: string)</td>
              <td></td>
            </tr>
            <tr>
              <td>onFocus</td>
              <td>得到焦点时的回调</td>
              <td>function</td>
              <td></td>
            </tr>
            <tr>
              <td>onBlur</td>
              <td>失去焦点时的回调</td>
              <td>function</td>
              <td></td>
            </tr>
            <tr>
              <td>onClick</td>
              <td>点击事件回调</td>
              <td>function(e: object)</td>
              <td></td>
            </tr>
            <tr>
              <td>placeholder</td>
              <td>占位文字</td>
              <td>string</td>
              <td>'请选择'</td>
            </tr>
            <tr>
              <td>showSearch</td>
              <td>是否可搜索</td>
              <td>boolean</td>
              <td></td>
            </tr>
            <tr>
              <td>size</td>
              <td>大小，'small'|'xSmall'</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>style</td>
              <td>附加在根元素上的样式</td>
              <td>object</td>
              <td></td>
            </tr>
            <tr>
              <td>selectable</td>
              <td>是否带有可选择的下拉菜单</td>
              <td>string</td>
              <td>true</td>
            </tr>
            <tr>
              <td>title</td>
              <td>提示性的标题</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>validator</td>
              <td>
                校验器配置，请参考
                <TextLink onClick={() => this.openPage({ path: '/docValidator' })}>Validator</TextLink>组件的属性配置
              </td>
              <td>object</td>
              <td></td>
            </tr>
            <tr>
              <td>value</td>
              <td>选中值</td>
              <td>{`string|{ val: string, text: string }`}</td>
              <td></td>
            </tr>
            <tr>
              <td>textInValue</td>
              <td>{`值包含选项文本，此时value使用{val, text}格式`}</td>
              <td>boolean</td>
              <td></td>
            </tr>
            <tr>
              <td>allowClear</td>
              <td>是否允许清除</td>
              <td>boolean</td>
              <td></td>
            </tr>
            <tr>
              <td>dropdownStyle</td>
              <td>附加在下拉菜单根元素的样式</td>
              <td>object</td>
              <td></td>
            </tr>
            <tr>
              <td>onSearchKeyChange</td>
              <td>搜索关键字变化时的回调，设置了该属性时默认的搜索失效</td>
              <td>function(searchKey: string)</td>
              <td></td>
            </tr>
          </PropsTable>
        </Sec>
      </CompPage>
    );
  }
}

export default connect()(Main);
