import React from 'react';
import { connect } from 'dva';
import CompPage from 'DOC/DocPage';
import PropsTable from '../../PropsTable';
import FuncsTable from '../../FuncsTable';
import Sec from '../../Section';
import BasicDemo from './BasicDemo';
import GroupDemo from './GroupDemo';
import styles from '../../styles.less';

class Main extends React.Component {
  openPage = (params) => {
    const { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: params,
    });
  };

  render() {
    return (
      <CompPage>
        <Sec title="功能介绍">
          排序按钮，用于切换排序方式。特点如下：
          <ul className={styles.ul}>
            <li>点击时依次切换排序状态为“由低到高”、“由高到低”、“清除排序”。</li>
            <li>提供按钮组，保证只有一种排序方式处于激活状态。</li>
          </ul>
        </Sec>
        <Sec title="例子">
          <h3>基本用法</h3>
          <BasicDemo />
          <h3>按钮组</h3>
          <GroupDemo />
        </Sec>
        <Sec title="开发说明">暂无。</Sec>
        <Sec title="属性">
          <PropsTable>
            <tr>
              <td>ascTitle</td>
              <td>升序状态下鼠标悬浮时的标题</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>children</td>
              <td>按钮内容</td>
              <td>ReactNode</td>
              <td></td>
            </tr>
            <tr>
              <td>className</td>
              <td>附加在根元素上的类名</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>descTitle</td>
              <td>降序状态下鼠标悬浮时的标题</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>onChange</td>
              <td>状态变化时的回调</td>
              <td>function(value: string)</td>
              <td></td>
            </tr>
            <tr>
              <td>style</td>
              <td>附加在根元素上的样式</td>
              <td>object</td>
              <td></td>
            </tr>
            <tr>
              <td>title</td>
              <td>默认状态下鼠标悬浮时的标题</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>value</td>
              <td>排序状态值，'asc'|'desc'</td>
              <td>string</td>
              <td></td>
            </tr>
          </PropsTable>
          <h3>按钮组属性</h3>
          <PropsTable>
            <tr>
              <td>children</td>
              <td>排序按钮集合</td>
              <td>ReactNode</td>
              <td></td>
            </tr>
            <tr>
              <td>className</td>
              <td>附加在根元素上的类名</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>onChange</td>
              <td>状态变化时的回调</td>
              <td>function(value: object)</td>
              <td></td>
            </tr>
            <tr>
              <td>style</td>
              <td>附加在根元素上的样式</td>
              <td>object</td>
              <td></td>
            </tr>
            <tr>
              <td>value</td>
              <td>排序状态值</td>
              <td>{`{name: string, type: string}`}</td>
              <td></td>
            </tr>
          </PropsTable>
        </Sec>
      </CompPage>
    );
  }
}

export default connect()(Main);
