import React from 'react';
import { connect } from 'dva';
import CompPage from 'DOC/DocPage';
import PropsTable from '../../PropsTable';
import FuncsTable from '../../FuncsTable';
import Sec from '../../Section';
import BasicDemo from './BasicDemo';
import ButtonStyleDemo from './ButtonStyleDemo';
import AlignDemo from './AlignDemo';
import styles from '../../styles.less';
import styleVars from 'COMPONENT/theme/variables';

class Main extends React.Component {
  openPage = (params) => {
    const { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: params,
    });
  };

  render() {
    return (
      <CompPage>
        <Sec title="功能介绍">
          标签页，用于局部切换内容。特点如下：
          <ul className={styles.ul}>
            <li>包含默认款、卡片款、按钮款、线条款。</li>
            <li>当页签个数过多时可滚动。</li>
            <li>其中卡片款和按钮款可关闭页签。</li>
            <li>其中按钮款有默认和大号两种尺寸。</li>
          </ul>
        </Sec>
        <Sec title="例子">
          <h3>默认款</h3>
          <BasicDemo />
          <h3>按钮款</h3>
          <ButtonStyleDemo />
          <h3>导航按钮对齐方式</h3>
          <AlignDemo />
        </Sec>
        <Sec title="开发说明">
          <div style={{ color: styleVars.highlightColor }}>
            目前仅完成默认款、按钮款的默认尺寸、不带滚动、不可增减。待完善。
          </div>
        </Sec>
        <Sec title="属性">
          <PropsTable>
            <tr>
              <td>activeKey</td>
              <td>激活状态的页签key，页签的key默认为顺序号，也可设置（见下面的页签属性）</td>
              <td>string|number</td>
              <td></td>
            </tr>
            <tr>
              <td>children</td>
              <td>页签集合</td>
              <td>ReactNode</td>
              <td></td>
            </tr>
            <tr>
              <td>className</td>
              <td>附加在根元素上的类名</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>defaultActiveKey</td>
              <td>首次加载时，默认激活状态的页签key，设置了activeKey属性时忽略。</td>
              <td>string|number</td>
              <td></td>
            </tr>
            <tr>
              <td>onChange</td>
              <td>当期激活的页签改变时的回调</td>
              <td>function(activeKey: string|number)</td>
              <td></td>
            </tr>
            <tr>
              <td>style</td>
              <td>附加在根元素上的样式</td>
              <td>object</td>
              <td></td>
            </tr>
            <tr>
              <td>type</td>
              <td>款式类型，'card'|'button'|'line'</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>navAlign</td>
              <td>导航按钮对齐方式，'center'|'right'</td>
              <td>string</td>
              <td></td>
            </tr>
          </PropsTable>
          <h3>页签属性</h3>
          <PropsTable>
            <tr>
              <td>children</td>
              <td>内容</td>
              <td>ReactNode</td>
              <td></td>
            </tr>
            <tr>
              <td>title</td>
              <td>标题</td>
              <td>ReactNode</td>
              <td></td>
            </tr>
            <tr>
              <td>className</td>
              <td>附加在根元素上的类名</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>style</td>
              <td>附加在根元素上的样式</td>
              <td>object</td>
              <td></td>
            </tr>
          </PropsTable>
        </Sec>
      </CompPage>
    );
  }
}

export default connect()(Main);
