import React from 'react';
import { connect } from 'dva';
import TextLink from 'COMPONENT/TextLink';
import CompPage from 'DOC/DocPage';
import PropsTable from '../../PropsTable';
import FuncsTable from '../../FuncsTable';
import Sec from '../../Section';
import BasicDemo from './BasicDemo';
import SizeDemo from './SizeDemo';
import DisableDemo from './DisableDemo';
import DisableValueDemo from './DisableValueDemo';
import styles from '../../styles.less';
import styleVars from 'COMPONENT/theme/variables';

class Main extends React.Component {
  openPage = (params) => {
    const { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: params,
    });
  };

  render() {
    return (
      <CompPage>
        <Sec title="功能介绍">时间输入控件，可选择或者直接输入时间，格式为：HH:mm（时:分）。</Sec>
        <Sec title="例子">
          <h3>基本用法</h3>
          <BasicDemo />
          <h3>三种大小</h3>
          <SizeDemo />
          <h3>禁用状态</h3>
          <DisableDemo />
          <h3>输入限制</h3>
          <DisableValueDemo />
          <div className={styles.info}>本示例展示禁止选择23:00-05:59时间段内所有时间。</div>
        </Sec>
        <Sec title="开发说明">暂无。</Sec>
        {/* 
          disabledHours: PropTypes.func,  //获取要禁用的小时
          disabledMinutes: PropTypes.func,  //获取要禁用的分钟
          getPopupContainer: PropTypes.func,  //获取承载弹框的DOM元素的函数
          onChange: PropTypes.func,  //值变化时的回调
          onFocus: PropTypes.func,  //得到焦点时的回调
          onBlur: PropTypes.func,  //失去焦点时的回调
          onClick: PropTypes.func,  //点击事件回调
          placeholder: PropTypes.string,  //占位文字
          size: PropTypes.oneOf(['small', 'xSmall']),  //大小
          title: PropTypes.string,  //提示性的标题
          value: PropTypes.string,  //选中值
          disabled: PropTypes.bool,  //禁用状态
          className: PropTypes.string,  //附加在根元素上的类名
          dropdownClassName: PropTypes.string,  //附加在下拉菜单根元素的类名
          style: PropTypes.object,  //附加在根元素上的样式
          dropdownStyle: PropTypes.object,  //附加在下拉菜单根元素的样式
          validator: PropTypes.object,  //校验器的配置参数，参考Validator组件的属性
        */}
        <Sec title="属性">
          <PropsTable>
            <tr>
              <td>value</td>
              <td>时间值，格式为：HH:mm</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>onChange</td>
              <td>值变化时的回调函数</td>
              <td>function(string)</td>
              <td></td>
            </tr>
            <tr>
              <td>size</td>
              <td>大小，可选'small'|'xSmall'</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>placeholder</td>
              <td>占位文字</td>
              <td>string</td>
              <td>'请选择时间'</td>
            </tr>
            <tr>
              <td>title</td>
              <td>提示性的标题</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>disabled</td>
              <td>禁用状态</td>
              <td>boolean</td>
              <td></td>
            </tr>
            <tr>
              <td>onFocus</td>
              <td>获得焦点时的回调函数</td>
              <td>function</td>
              <td></td>
            </tr>
            <tr>
              <td>onBlur</td>
              <td>失去焦点时的回调函数</td>
              <td>function</td>
              <td></td>
            </tr>
            <tr>
              <td>onClick</td>
              <td>点击事件回调</td>
              <td>function(e: object)</td>
              <td></td>
            </tr>
            <tr>
              <td>disabledHours</td>
              <td>获取不可选的小时</td>
              <td>
                function
                <br />
                {`@return {[number]} - 禁用的小时数组（0-23）`}
              </td>
              <td></td>
            </tr>
            <tr>
              <td>disabledMinutes</td>
              <td>获取不可选的分钟</td>
              <td>
                function
                <br />
                {`@param {number} - 当前选中的小时`}
                <br />
                {`@return {[number]} - 禁用的分钟数组（0-59）`}
              </td>
              <td></td>
            </tr>
            <tr>
              <td>getPopupContainer</td>
              <td>获取承载弹框的DOM元素的函数</td>
              <td>DOMElement function</td>
              <td></td>
            </tr>
            <tr>
              <td>className</td>
              <td>附加在根元素上的类名</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>dropdownStyle</td>
              <td>附加在下拉菜单根元素的样式</td>
              <td>object</td>
              <td></td>
            </tr>
            <tr>
              <td>dropdownClassName</td>
              <td>附加在下拉菜单根元素的类名</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>dropdownStyle</td>
              <td>附加在下拉菜单根元素的样式</td>
              <td>object</td>
              <td></td>
            </tr>
            <tr>
              <td>validator</td>
              <td>
                校验器配置，请参考
                <TextLink onClick={() => this.openPage({ path: '/docValidator' })}>Validator</TextLink>组件的属性配置
              </td>
              <td>object</td>
              <td></td>
            </tr>
          </PropsTable>
        </Sec>
      </CompPage>
    );
  }
}

export default connect()(Main);
