import React from 'react';
import TreeMultiSelect from 'COMPONENT/TreeMultiSelect';
import { debounce } from 'lodash';

let initDict = [
  { val: '1', text: '选项 1', isLeaf: false, parentVal: null },
  { val: '1-1', text: '选项 1-1', isLeaf: false, parentVal: '1' },
];

class Main extends React.Component {
  state = {
    value: null,
    dict: initDict,
    expandedKeys: [initDict[0].val],
  };

  dictVersion = 1;

  onLoadChildren = (treeNode) => {
    return new Promise((resolve) => {
      if (treeNode.props.children) {
        resolve();
        return;
      }
      setTimeout(() => {
        const key = treeNode.props.eventKey;
        this.setState({
          dict: [
            ...this.state.dict,
            { val: `${key}-1`, text: `选项 ${key}-1`, isLeaf: false, parentVal: key },
            { val: `${key}-2`, text: `选项 ${key}-2`, isLeaf: false, parentVal: key },
          ],
          expandedKeys: [...this.state.expandedKeys, key],
        });
        resolve();
      }, 500);
    });
  };

  onSearchKeyChange = (key) => {
    debounce(this.search, 300)(key);
  };

  search = (key) => {
    if (key) {
      const dictVersion = ++this.dictVersion;
      setTimeout(() => {
        if (this.dictVersion === dictVersion) {
          const fakeSearchResult = [
            { val: 'search_1', text: `${key}搜索结果1`, isLeaf: true, parentVal: null },
            { val: 'search_2', text: `${key}搜索结果2`, isLeaf: true, parentVal: null },
          ];
          this.setState({ dict: fakeSearchResult, expandedKeys: [] });
        }
      }, 500);
    } else {
      this.dictVersion++;
      this.setState({ dict: initDict, expandedKeys: [initDict[0].val] });
    }
  };

  render() {
    const { value, dict, expandedKeys } = this.state;
    return (
      <div style={{ width: 200 }}>
        <TreeMultiSelect
          dict={dict}
          value={value}
          onChange={(value) => this.setState({ value })}
          onLoadChildren={this.onLoadChildren}
          onSearchKeyChange={this.onSearchKeyChange}
          textInValue
          expandedKeys={expandedKeys}
          onExpandChange={(expandedKeys) => this.setState({ expandedKeys })}
        />
      </div>
    );
  }
}

export default Main;
