import React from 'react';
import { connect } from 'dva';
import TextLink from 'COMPONENT/TextLink';
import CompPage from 'DOC/DocPage';
import PropsTable from '../../PropsTable';
import FuncsTable from '../../FuncsTable';
import Sec from '../../Section';
import BasicDemo from './BasicDemo';
import SizeDemo from './SizeDemo';
import DisableDemo from './DisableDemo';
import ServerSideSearchDemo from './ServerSideSearchDemo';
import { SELECT_NOT_FOUND_TIPS } from 'COMPONENT/utils/form/config';

class Main extends React.Component {
  openPage = (params) => {
    const { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: params,
    });
  };

  render() {
    return (
      <CompPage>
        <Sec title="功能介绍">下拉选项为树状的多选框。</Sec>
        <Sec title="例子">
          <h3>基本用法</h3>
          <BasicDemo />
          <h3>三种大小</h3>
          <SizeDemo />
          <h3>禁用</h3>
          <DisableDemo />
          <h3>服务器端搜索</h3>
          <ServerSideSearchDemo />
        </Sec>
        <Sec title="开发说明">暂无。</Sec>
        <Sec title="属性">
          <PropsTable>
            <tr>
              <td>className</td>
              <td>附加在根元素上的类名</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>dict</td>
              <td>选项字典</td>
              <td>
                <pre>
                  {`[
  {
    val: string - 选项值，在整棵树中唯一
    text: string - 选项显示文字
    disabled: boolean - 选项禁用状态
    isLeaf: boolean - 是否叶子结点，仅动态加载结点数据时使用
    parentVal: string - 父结点值，根结点时为null
  }
]`}
                </pre>
              </td>
              <td></td>
            </tr>
            <tr>
              <td>dropdownClassName</td>
              <td>附加在下拉菜单根元素的类名</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>disabled</td>
              <td>禁用状态</td>
              <td>boolean</td>
              <td></td>
            </tr>
            <tr>
              <td>filter</td>
              <td>自定义的搜索函数</td>
              <td>boolean function(searchKey: string, option: object)</td>
              <td></td>
            </tr>
            <tr>
              <td>getPopupContainer</td>
              <td>获取承载弹框的DOM元素的函数</td>
              <td>DOMElement function</td>
              <td></td>
            </tr>
            <tr>
              <td>noOptionTips</td>
              <td>无选项提示文字</td>
              <td>string</td>
              <td>{SELECT_NOT_FOUND_TIPS}</td>
            </tr>
            <tr>
              <td>searchNotFoundTips</td>
              <td>搜索无匹配结果时提示文字</td>
              <td>string</td>
              <td>{SELECT_NOT_FOUND_TIPS}</td>
            </tr>
            <tr>
              <td>searchPlaceholder</td>
              <td>搜索框的占位文字</td>
              <td>string</td>
              <td>'请输入内容搜索'</td>
            </tr>
            <tr>
              <td>onChange</td>
              <td>值变化时的回调</td>
              <td>{`function(string[]|[{ val: string, text: string }])`}</td>
              <td></td>
            </tr>
            <tr>
              <td>onFocus</td>
              <td>得到焦点时的回调</td>
              <td>function</td>
              <td></td>
            </tr>
            <tr>
              <td>onBlur</td>
              <td>失去焦点时的回调</td>
              <td>function</td>
              <td></td>
            </tr>
            <tr>
              <td>onSelect</td>
              <td>选中值时的回调</td>
              <td>{`function(string|{ val: string, text: string })`}</td>
              <td></td>
            </tr>
            <tr>
              <td>onDeselect</td>
              <td>取消选中值时的回调</td>
              <td>{`function(string|{ val: string, text: string })`}</td>
              <td></td>
            </tr>
            <tr>
              <td>onClick</td>
              <td>点击事件回调</td>
              <td>function(e: object)</td>
              <td></td>
            </tr>
            <tr>
              <td>placeholder</td>
              <td>占位文字</td>
              <td>string</td>
              <td>'请选择'</td>
            </tr>
            <tr>
              <td>showSearch</td>
              <td>是否可搜索</td>
              <td>boolean</td>
              <td>true</td>
            </tr>
            <tr>
              <td>size</td>
              <td>大小，'small'|'xSmall'</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>style</td>
              <td>附加在根元素上的样式</td>
              <td>object</td>
              <td></td>
            </tr>
            <tr>
              <td>title</td>
              <td>提示性的标题</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>validator</td>
              <td>
                校验器配置，请参考
                <TextLink onClick={() => this.openPage({ path: '/docValidator' })}>Validator</TextLink>组件的属性配置
              </td>
              <td>object</td>
              <td></td>
            </tr>
            <tr>
              <td>value</td>
              <td>选中值</td>
              <td>{`string[]|[{ val: string, text: string }]`}</td>
              <td></td>
            </tr>
            <tr>
              <td>textInValue</td>
              <td>{`值包含选项文本，此时value使用[{val, text}]格式`}</td>
              <td>boolean</td>
              <td></td>
            </tr>
            <tr>
              <td>allowClear</td>
              <td>是否允许清除</td>
              <td>boolean</td>
              <td>true</td>
            </tr>
            <tr>
              <td>dropdownStyle</td>
              <td>附加在下拉菜单根元素的样式</td>
              <td>object</td>
              <td></td>
            </tr>
            <tr>
              <td>onLoadChildren</td>
              <td>触发加载子结点数据时的回调，设置了该属性即指定为动态加载结点数据</td>
              <td>Promise function(treeNode: object)</td>
              <td></td>
            </tr>
            <tr>
              <td>onSearchKeyChange</td>
              <td>搜索关键字变化时的回调，设置了该属性时默认的搜索失效</td>
              <td>function(searchKey: string)</td>
              <td></td>
            </tr>
            <tr>
              <td>expandedKeys</td>
              <td>展开的结点。只有一个一级结点时默认展开一级结点；多个一级结点时，默认不展开</td>
              <td>string[]</td>
              <td></td>
            </tr>
            <tr>
              <td>onExpandChange</td>
              <td>展开结点变化时的回调</td>
              <td>function(expandedKeys: string[])</td>
              <td></td>
            </tr>
          </PropsTable>
        </Sec>
      </CompPage>
    );
  }
}

export default connect()(Main);
