import React from 'react';
import { Row, Col } from 'antd';
import Input from 'COMPONENT/Input';
import Page from 'COMPONENT/Page';
import FormItem from 'COMPONENT/FormItem';
import Select from 'COMPONENT/Select';
import TextArea from 'COMPONENT/TextArea';
import Button from 'COMPONENT/Button';
import PopTip from 'COMPONENT/PopTip';

const dict = [
  { val: '2016', text: '2016' },
  { val: '2017', text: '2017' },
  { val: '2018', text: '2018' },
];

class Main extends React.Component {
  state = {
    name: null,
    year: null,
    remark: '',
    checking: false,
    tipsVisible: false,
  };

  errors = {};

  save = () => {
    const { checking } = this.state;
    if (!checking) {
      this.setState({ checking: true });
    }
    if (Object.values(this.errors).find((v) => v)) {
      this.setState({ tipsVisible: true });
      return;
    }
  };

  onTipsVisibleChange = (visible) => {
    if (!visible) {
      this.setState({ tipsVisible: visible });
    }
  };

  errorChange = (key, error) => {
    this.errors[key] = error;
  };

  render() {
    const { name, year, remark, checking, tipsVisible } = this.state;

    return (
      <Row style={{ maxWidth: 1000, marginTop: 30 }}>
        <Col span={12}>
          <FormItem name="名称" required={true}>
            <Input
              value={name}
              onChange={(e) => this.setState({ name: e.target.value })}
              validator={{
                rules: ['required'],
                silent: !checking,
                onErrorChange: (e) => this.errorChange('name', e),
              }}
            />
          </FormItem>
        </Col>
        <Col span={12}>
          <FormItem name="年份" required={true}>
            <Select
              dict={dict}
              value={year}
              onChange={(val) => this.setState({ year: val })}
              validator={{
                rules: ['required'],
                silent: !checking,
                onErrorChange: (e) => this.errorChange('name', e),
              }}
            />
          </FormItem>
        </Col>
        <Col span={24}>
          <FormItem name="备注">
            <TextArea rows={3} value={remark} onChange={(e) => this.setState({ remark: e.target.value })} />
          </FormItem>
        </Col>
        <Col span={24} style={{ textAlign: 'right', padding: '0 45px' }}>
          <PopTip content="请正确填写信息" visible={tipsVisible} onVisibleChange={this.onTipsVisibleChange}>
            <Button type="primary" onClick={this.save}>
              保存
            </Button>
          </PopTip>
        </Col>
      </Row>
    );
  }
}

export default Main;
