import React from 'react';
import CompPage from 'DOC/DocPage';
import PropsTable from '../../PropsTable';
import FuncsTable from '../../FuncsTable';
import Sec from '../../Section';
import BasicDemo from './BasicDemo';
import CustomTipsDemo from './CustomTipsDemo';
import FreeInputDemo from './FreeInputDemo';
import CustomCheckDemo from './CustomCheckDemo';
import KeepHeightDemo from './KeepHeightDemo';
import FormSaveDemo from './FormSaveDemo';
import StatusStyleDemo from './StatusStyleDemo';
import styles from '../../styles.less';
import styleVars from 'COMPONENT/theme/variables';

class Main extends React.Component {
  render() {
    return (
      <CompPage>
        <Sec title="功能介绍">
          表单输入项校验器。特点有：
          <ul className={styles.ul}>
            <li>适用于任意输入项目，对于常用的输入组件，提供内置的属性配置</li>
            <li>对于常用的校验提供预设的校验规则，如必填项、电话号码等</li>
            <li>支持自定义校验函数</li>
            <li>支持多条校验规则</li>
          </ul>
        </Sec>
        <Sec title="例子">
          <h3>基本用法</h3>
          <BasicDemo />
          <h3>自定义提示信息</h3>
          <CustomTipsDemo />
          <h3>自定义校验函数</h3>
          <CustomCheckDemo />
          <h3>不规则输入项的校验</h3>
          <FreeInputDemo />
          <h3>状态样式</h3>
          <StatusStyleDemo />
          <h3>保持高度</h3>
          <KeepHeightDemo />
          <div className={styles.info} style={{ marginTop: 15 }}>
            本示例展示错误信息的显示状态改变时，输入项高度保持不变，即不会撑开下面的元素。
          </div>
          <h3>表单保存校验交互综合示例</h3>
          <FormSaveDemo />
          <div className={styles.info}>
            本示例展示表单保存校验的一般交互过程：
            <br />
            1、初始时不显示错误信息
            <br />
            2、点击了保存按钮后进入校验状态，错误信息根据输入情况实时更新
          </div>
        </Sec>
        <Sec title="开发说明">暂无。</Sec>
        <Sec title="属性">
          <PropsTable>
            <tr>
              <td>className</td>
              <td>附加在根元素上的类名</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>children</td>
              <td>待校验的输入项目</td>
              <td>ReactNode</td>
              <td>-</td>
            </tr>
            <tr>
              <td>keepHeight</td>
              <td>保持原有高度，即提示信息不撑开高度</td>
              <td>boolean</td>
              <td></td>
            </tr>
            <tr>
              <td>rules</td>
              <td>
                校验规则集合，可以为三种类型：
                <br />
                1、string，使用预设的校验规则，现有'required'|'phone'
                <br />
                2、object，用于需要自定义预设校验规则提示信息的情况，key为规则名，value为提示信息
                <br />
                3、function，自定义的校验函数，参数为待校验的值，返回校验错误null|true|string
              </td>
              <td>any[]</td>
              <td>['required']</td>
            </tr>
            <tr>
              <td>style</td>
              <td>附加在根元素上的样式</td>
              <td>object</td>
              <td></td>
            </tr>
            <tr>
              <td>silent</td>
              <td>不显示错误信息及状态样式</td>
              <td>boolean</td>
              <td>false</td>
            </tr>
            <tr>
              <td>showTips</td>
              <td>显示错误信息，silent不为true时有效</td>
              <td>boolean</td>
              <td>true</td>
            </tr>
            <tr>
              <td>showStatus</td>
              <td>显示错误状态样式，silent不为true时有效</td>
              <td>boolean</td>
              <td>false</td>
            </tr>
            <tr>
              <td>value</td>
              <td>待校验的值</td>
              <td>any</td>
              <td></td>
            </tr>
            <tr>
              <td>error</td>
              <td>错误信息</td>
              <td>string</td>
              <td></td>
            </tr>
            <tr>
              <td>onErrorChange</td>
              <td>错误信息变化时的回调</td>
              <td>function(string)</td>
              <td></td>
            </tr>
          </PropsTable>
        </Sec>
      </CompPage>
    );
  }
}

export default Main;
