import React from 'react';
import CompPage from 'DOC/DocPage';
import Sec from 'DOC/Section';
import commonStyles from '../../styles.less';
// import styles from './styles.less';
import List from 'COMPONENT/List';
import FormItem from 'COMPONENT/FormItem';
import ColorSample from 'DOC/lessPages/ColorSample';
import Demo from 'DOC/lessPages/Demo';
import Row from 'DOC/Row';
import Col from 'DOC/FloatCol';
import { Switch, Steps, Progress, Breadcrumb, Alert, Tabs, Tooltip } from 'antd';
import Radio from 'COMPONENT/Radio';
import Checkbox from 'COMPONENT/Checkbox';
import Tag from 'COMPONENT/Tag';
import Button from 'COMPONENT/Button';
import IconButton from 'COMPONENT/IconButton';
import Input from 'COMPONENT/Input';
import InputNumber from 'COMPONENT/InputNumber';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import DatePicker from 'COMPONENT/DatePicker';
import DateRangePicker from 'COMPONENT/DateRangePicker';
import TagSelect from 'COMPONENT/TagSelect';
import TextArea from 'COMPONENT/TextArea';
import TimePicker from 'COMPONENT/TimePicker';
import Popover from 'COMPONENT/Popover';
import Modal from 'COMPONENT/Modal';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import showAlert from 'COMPONENT/utils/globalTips/showAlert';
const TabPane = Tabs.TabPane;
import styleVars from 'COMPONENT/theme/variables';

const list = [
  { col1: 'x1', col2: 'y1' },
  { col1: 'x2', col2: 'y2' },
];
const columns = [
  { title: '第一列', dataIndex: 'col1' },
  { title: '第二列', dataIndex: 'col2' },
  { title: '第三列', dataIndex: 'col3' },
];
const colStyle = {
  width: 200,
};
const colStyle2 = {
  width: 420,
};
const dict = [
  { val: '1', text: '选项1' },
  { val: '2', text: '选项2' },
  { val: '3', text: '选项3' },
];
const dictLong = [
  { val: '1', text: '选项1' },
  { val: '2', text: '选项2' },
  { val: '3', text: '选项3' },
  { val: '4', text: '选项3' },
  { val: '5', text: '选项3' },
  { val: '6', text: '选项3' },
  { val: '7', text: '选项3' },
  { val: '8', text: '选项3' },
  { val: '9', text: '选项3' },
  { val: '10', text: '选项3' },
  { val: '11', text: '选项3' },
  { val: '12', text: '选项3' },
];
const groupDict = [
  {
    label: '第一组',
    options: [
      { val: '1', text: '选项1' },
      { val: '2', text: '选项2' },
      { val: '3', text: '选项3' },
    ],
  },
  {
    label: '第二组',
    options: [
      { val: '4', text: '选项4' },
      { val: '5', text: '选项5' },
    ],
  },
];

const sampleStyle = {
  background: styleVars.basicColor,
  border: '1px solid ' + styleVars.basicColorBorder,
  width: 120,
  height: 120,
  float: 'left',
  marginRight: 30,
  textAlign: 'center',
  lineHeight: '120px',
};

class Main extends React.Component {
  state = {
    modalVisible: false,
  };

  close = () => {
    this.setState({ modalVisible: false });
  };

  render() {
    const { modalVisible } = this.state;

    return (
      <CompPage>
        <Sec title="简介">
          3种尺寸，适应不同大小的元素。
          <ul className={commonStyles.ul}>
            <li>
              大号 @borderRadiusLg<span className={commonStyles.tips}> - 用于模态框、较大的底框</span>
            </li>
            <li>
              标准 @borderRadius<span className={commonStyles.tips}> - 用于一般的表单控件、底框</span>
            </li>
            <li>
              小号 @borderRadiusSm<span className={commonStyles.tips}> - 用于标签、勾选按钮、工具提示等</span>
            </li>
          </ul>
        </Sec>
        <Sec title="样本">
          <Row>
            <div style={{ ...sampleStyle, borderRadius: styleVars.borderRadiusLg }}>大号</div>
            <div style={{ ...sampleStyle, borderRadius: styleVars.borderRadius }}>标准</div>
            <div style={{ ...sampleStyle, borderRadius: styleVars.borderRadiusSm }}>小号</div>
          </Row>
        </Sec>
        <Sec title="使用场景">
          <Demo title="大号" info="模态框">
            <Row>
              <Col style={colStyle}>
                <Button onClick={() => this.setState({ modalVisible: true })}>打开业务操作模态框</Button>
                {modalVisible && (
                  <Modal title="标题" onOk={this.close} onCancel={this.close}>
                    一些内容。
                  </Modal>
                )}
              </Col>
              <Col style={colStyle}>
                <Button onClick={() => showAlert('一些提示信息。')}>打开提示性模态框</Button>
              </Col>
            </Row>
          </Demo>
          <Demo title="大号" info="较大的底框">
            见系统主页面，以及欢迎页的内容板块。
          </Demo>
          <Demo title="标准" info="按钮、文本框、数字框、选择框、日期选择框、多行文本框">
            <Row>
              <Col style={colStyle}>
                <Button>按钮</Button>
              </Col>
            </Row>
            <Row>
              <Col style={colStyle}>
                <Input placeholder="请输入" />
              </Col>
              <Col style={colStyle}>
                <InputNumber placeholder="请输入数字" />
              </Col>
              <Col style={colStyle}>
                <Select dict={dict} />
              </Col>
              <Col style={colStyle}>
                <DatePicker placeholder="请选择日期" />
              </Col>
            </Row>
            <Row>
              <Col style={colStyle2}>
                <TextArea rows={3} placeholder="请输入" />
              </Col>
            </Row>
          </Demo>
          <Demo title="标准" info="气泡提示框、自动消失提示框">
            <Row>
              <Col style={colStyle}>
                <Popover trigger="click" content="一些提示信息">
                  <Button onClick={(e) => e.stopPropagation()}>打开气泡提示框</Button>
                </Popover>
              </Col>
              <Col style={colStyle}>
                <Button onClick={() => showMessage('系统错误')}>打开自动消失提示框</Button>
              </Col>
            </Row>
          </Demo>
          <Demo title="标准" info="大提醒框">
            <Alert
              message="一些说明信息"
              description="信息的详细描述信息的详细描述信息的详细描述"
              type="info"
              showIcon
            />
          </Demo>
          <Demo title="小号" info="标签、勾选按钮、工具提示">
            <Row>
              <Col style={{ ...colStyle, width: 80 }}>
                <Tag>标签</Tag>
              </Col>
              <Col style={colStyle2}>
                <TagSelect dict={dictLong} value={['1', '2']} disabled />
              </Col>
              <Col style={{ ...colStyle, width: 80 }}>
                <Checkbox>同意</Checkbox>
              </Col>
              <Col style={colStyle}>
                <Tooltip trigger="click" title="一些提示信息">
                  <Button onClick={(e) => e.stopPropagation()}>打开工具提示</Button>
                </Tooltip>
              </Col>
            </Row>
          </Demo>
        </Sec>
      </CompPage>
    );
  }
}

export default Main;
