import React from 'react';
import CompPage from 'DOC/DocPage';
import Sec from 'DOC/Section';
import commonStyles from '../../styles.less';
// import styles from './styles.less';
import ColorSample from 'DOC/lessPages/ColorSample';
import Demo from 'DOC/lessPages/Demo';
import Row from 'DOC/Row';
import Col from 'DOC/FloatCol';
import { Icon, Tag, Badge, Alert, Progress, Form } from 'antd';
import Input from 'COMPONENT/Input';
import styleVars from 'COMPONENT/theme/variables';
import FormItem from 'COMPONENT/FormItem';

const AntdFormItem = Form.Item;

const formItemLayout = {
  labelCol: {
    xs: { span: 24 },
    sm: { span: 5 },
  },
  wrapperCol: {
    xs: { span: 24 },
    sm: { span: 12 },
  },
};

const liTitleStyle = {
  fontSize: styleVars.fontSizeHeadingSm,
  margin: '10px 0',
};

class Main extends React.Component {
  render() {
    return (
      <CompPage>
        <Sec title="简介">
          与主色搭配的几种不同颜色，使用量较少，一般具有特定含义。
          <ul className={commonStyles.ul}>
            <div style={liTitleStyle}>危险红</div>
            <li>
              标准 @highlightColor
              <span className={commonStyles.tips}>
                {' '}
                - 用于错误信息或其他需要重点突出的信息，如系统错误、表单必填项提示、新消息通知
              </span>
            </li>
            <li>
              禁用 @highlightColorDisabled
              <span className={commonStyles.tips}> - 用于使用了危险红的元素处于禁用状态时</span>
            </li>
            <li>
              背景 @highlight_bg<span className={commonStyles.tips}> - 用于底色，相对于标准色含义上比较弱化</span>
            </li>
            <div style={liTitleStyle}>警告黄</div>
            <li>
              标准 @warningColor<span className={commonStyles.tips}> - 用于需要提醒的信息，如操作错误提示、确认</span>
            </li>
            <li>
              禁用 @warningColorDisabled
              <span className={commonStyles.tips}> - 用于使用了警告黄的元素处于禁用状态时</span>
            </li>
            <li>
              背景 @warning_bg<span className={commonStyles.tips}> - 用于底色，相对于标准色含义上比较弱化</span>
            </li>
            <div style={liTitleStyle}>信息蓝</div>
            <li>
              标准 @infoColor<span className={commonStyles.tips}> - 用于一般说明性信息，如详情信息、操作说明</span>
            </li>
            <li>
              禁用 @infoColorDisabled<span className={commonStyles.tips}> - 用于使用了信息蓝的元素处于禁用状态时</span>
            </li>
            <li>
              背景 @info_bg<span className={commonStyles.tips}> - 用于底色，相对于标准色含义上比较弱化</span>
            </li>
            <div style={liTitleStyle}>成功绿</div>
            <li>
              标准 @successColor<span className={commonStyles.tips}> - 用于比较正面的信息，如成功状态提示</span>
            </li>
            <li>
              禁用 @successColorDisabled
              <span className={commonStyles.tips}> - 用于使用了成功绿的元素处于禁用状态时</span>
            </li>
            <li>
              背景 @success_bg<span className={commonStyles.tips}> - 用于底色，相对于标准色含义上比较弱化</span>
            </li>
          </ul>
        </Sec>
        <Sec title="样本">
          <ColorSample title="危险红" style={{ background: styleVars.highlightColor }} />
          <ColorSample title="危险红（禁用）" style={{ background: styleVars.highlightColorDisabled }} />
          <ColorSample
            title="危险红（背景）"
            style={{ background: styleVars.highlightColorBg }}
            fontColor={styleVars.highlightColorDisabled}
          />
          <br />
          <ColorSample title="警告黄" style={{ background: styleVars.warningColor }} />
          <ColorSample title="警告黄（禁用）" style={{ background: styleVars.warningColorDisabled }} />
          <ColorSample
            title="警告黄（背景）"
            style={{ background: styleVars.warningColorBg }}
            fontColor={styleVars.warningColorDisabled}
          />
          <br />
          <ColorSample title="信息蓝" style={{ background: styleVars.infoColor }} />
          <ColorSample title="信息蓝" style={{ background: styleVars.infoColorDisabled }} />
          <ColorSample
            title="信息蓝"
            style={{ background: styleVars.infoColorBg }}
            fontColor={styleVars.infoColorDisabled}
          />
          <br />
          <ColorSample title="成功绿" style={{ background: styleVars.successColor }} />
          <ColorSample title="成功绿（禁用）" style={{ background: styleVars.successColorDisabled }} />
          <ColorSample
            title="成功绿（背景）"
            style={{ background: styleVars.successColorBg }}
            fontColor={styleVars.successColorDisabled}
          />
        </Sec>
        <Sec title="使用场景">
          <Demo title="图标">
            <Row>
              <Col>
                <span style={{ color: styleVars.highlightColor, fontSize: 24 }}>
                  <Icon type="close-circle" />
                </span>
              </Col>
              <Col>
                <span style={{ color: styleVars.warningColor, fontSize: 24 }}>
                  <Icon type="exclamation-circle" />
                </span>
              </Col>
              <Col>
                <span style={{ color: styleVars.infoColor, fontSize: 24 }}>
                  <Icon type="info-circle" />
                </span>
              </Col>
              <Col>
                <span style={{ color: styleVars.successColor, fontSize: 24 }}>
                  <Icon type="check-circle" />
                </span>
              </Col>
            </Row>
          </Demo>
          <Demo title="标签">
            <Row>
              <Col>
                <Tag color={styleVars.highlightColor}>错误</Tag>
              </Col>
              <Col>
                <Tag color={styleVars.warningColor}>警告</Tag>
              </Col>
              <Col>
                <Tag color={styleVars.infoColor}>信息</Tag>
              </Col>
              <Col>
                <Tag color={styleVars.successColor}>成功</Tag>
              </Col>
            </Row>
            <Row>
              <Col>
                <Tag color="red">错误</Tag>
              </Col>
              <Col>
                <Tag color="orange">警告</Tag>
              </Col>
              <Col>
                <Tag color="blue">信息</Tag>
              </Col>
              <Col>
                <Tag color="green">成功</Tag>
              </Col>
            </Row>
          </Demo>
          <Demo title="标记">
            <div style={{ marginBottom: 30 }}>
              <Badge count={25}>
                <Icon type="message" style={{ fontSize: 20 }} />
              </Badge>
              &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
              <Badge dot>
                <Icon type="message" style={{ fontSize: 20 }} />
              </Badge>
            </div>
            <Row>
              <Col>
                <Badge status="error" text="失败" />
              </Col>
              <Col>
                <Badge status="warning" text="等待" />
              </Col>
              <Col>
                <Badge status="success" text="成功" />
              </Col>
            </Row>
          </Demo>
          <Demo title="表单" info="必填项、校验结果提示（待完善）">
            <div style={{ maxWidth: 500, marginLeft: -50 }}>
              <FormItem name="姓名" required labelWidth={80}>
                <Input />
              </FormItem>
            </div>
          </Demo>
          <Demo title="大提醒框">
            <Alert message="出错了" description="信息的详细描述信息的详细描述信息的详细描述" type="error" showIcon />
            <br />
            <Alert message="请注意" description="信息的详细描述信息的详细描述信息的详细描述" type="warning" showIcon />
            <br />
            <Alert
              message="一些说明信息"
              description="信息的详细描述信息的详细描述信息的详细描述"
              type="info"
              showIcon
            />
            <br />
            <Alert
              message="恭喜您，注册成功！"
              description="信息的详细描述信息的详细描述信息的详细描述"
              type="success"
              showIcon
            />
          </Demo>
          <Demo title="进度条">
            <Progress percent={70} strokeWidth={5} status="exception" />
            <Progress percent={100} strokeWidth={5} />
          </Demo>
        </Sec>
      </CompPage>
    );
  }
}

export default Main;
