import React from 'react';
import CompPage from 'DOC/DocPage';
import Sec from 'DOC/Section';
import commonStyles from '../../styles.less';
// import styles from './styles.less';
import ColorSample from 'DOC/lessPages/ColorSample';
import Demo from 'DOC/lessPages/Demo';
import Row from 'DOC/Row';
import Col from 'DOC/FloatCol';
import { Radio, Checkbox, Switch, Tag, Icon } from 'antd';
import Button from 'COMPONENT/Button';
import IconButton from 'COMPONENT/IconButton';
import Input from 'COMPONENT/Input';
import InputNumber from 'COMPONENT/InputNumber';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import DatePicker from 'COMPONENT/DatePicker';
import DateRangePicker from 'COMPONENT/DateRangePicker';
import TagSelect from 'COMPONENT/TagSelect';
import TextArea from 'COMPONENT/TextArea';
import TimePicker from 'COMPONENT/TimePicker';
const RadioGroup = Radio.Group;
import styleVars from 'COMPONENT/theme/variables';

const colStyle = {
  width: 200,
};
const colStyle2 = {
  width: 420,
};

const avatarStyles = {
  width: 98,
  height: 98,
  background: styleVars.basicColor,
  border: '1px dashed ' + styleVars.basicColorBorder,
  textAlign: 'center',
  borderRadius: styleVars.borderRadius,
};

const avatarIconStyles = {
  color: styleVars.fontColorSecondary,
  fontSize: 28,
  marginTop: 34,
};

const dictLong = [
  { val: '1', text: '选项1' },
  { val: '2', text: '选项2' },
  { val: '3', text: '选项3' },
  { val: '4', text: '选项3' },
  { val: '5', text: '选项3' },
  { val: '6', text: '选项3' },
  { val: '7', text: '选项3' },
  { val: '8', text: '选项3' },
  { val: '9', text: '选项3' },
  { val: '10', text: '选项3' },
  { val: '11', text: '选项3' },
  { val: '12', text: '选项3' },
];

const dict = [
  { val: '1', text: '选项1' },
  { val: '2', text: '选项2' },
  { val: '3', text: '选项3' },
];

const groupDict = [
  {
    label: '第一组',
    options: [
      { val: '1', text: '选项1' },
      { val: '2', text: '选项2' },
      { val: '3', text: '选项3' },
    ],
  },
  {
    label: '第二组',
    options: [
      { val: '4', text: '选项4' },
      { val: '5', text: '选项5' },
    ],
  },
];

class Main extends React.Component {
  render() {
    return (
      <CompPage>
        <Sec title="简介">
          基础功能性颜色，不太显眼，通常用于普通的区块划分或交互状态的示意。
          <ul className={commonStyles.ul}>
            <li>
              标准 @basicColor<span className={commonStyles.tips}> - 用于底框、普通禁用状态的背景色</span>
            </li>
            <li>
              激活 @basicColorActive<span className={commonStyles.tips}> - 用于普通激活状态的背景色</span>
            </li>
            <li>
              边线 @basicColorBorder<span className={commonStyles.tips}> - 用于普通边线、分割线</span>
            </li>
          </ul>
        </Sec>
        <Sec title="样本">
          <ColorSample title="标准" style={{ background: styleVars.basicColor }} darkFont />
          <ColorSample title="激活" style={{ background: styleVars.basicColorActive }} darkFont />
          <ColorSample title="边线" style={{ background: styleVars.basicColorBorder }} darkFont />
        </Sec>
        <Sec title="使用场景">
          <Demo title="底框" info="背景、边线">
            <Row>
              <Col>
                <div style={avatarStyles}>
                  <Icon type="plus" style={avatarIconStyles} />
                </div>
              </Col>
              <Col>
                <div style={{ ...avatarStyles, borderRadius: '50%' }}>
                  <Icon type="plus" style={avatarIconStyles} />
                </div>
              </Col>
            </Row>
            <br />
            <div style={{ ...avatarStyles, width: 'auto', height: 200 }}>
              <Icon type="upload" style={{ ...avatarIconStyles, marginTop: 64 }} />
              <div style={{ color: styleVars.fontColorSecondary, marginTop: 10 }}>将文件拖拽到此处上传</div>
            </div>
          </Demo>
          <Demo title="普通禁用状态" info="表单控件禁用时的背景色">
            <Row>
              <Col style={colStyle}>
                <Input value="一些文字" disabled />
              </Col>
              <Col style={colStyle}>
                <InputNumber value={22} disabled />
              </Col>
              <Col style={colStyle}>
                <Select dict={dict} value="1" disabled />
              </Col>
              <Col style={colStyle}>
                <MultiSelect dict={dict} value={['1', '2']} disabled />
              </Col>
            </Row>
            <Row>
              <Col style={colStyle2}>
                <TagSelect dict={dictLong} value={['1', '2']} disabled />
              </Col>
            </Row>
            <Row>
              <Col style={colStyle}>
                <DatePicker value="2017-11-22" disabled />
              </Col>
              <Col style={colStyle}>
                <TimePicker value="19:30" disabled />
              </Col>
              <Col style={colStyle2}>
                <DateRangePicker value={['2017-10-26', '2017-11-22']} disabled />
              </Col>
            </Row>
            <Row>
              <Col style={colStyle}>
                <RadioGroup value="1" disabled>
                  <Radio value="1">单选项1</Radio>
                  &nbsp;&nbsp;
                  <Radio value="2">单选项2</Radio>
                </RadioGroup>
              </Col>
              <Col style={colStyle}>
                <Checkbox disabled>同意</Checkbox>
                <Checkbox checked disabled>
                  同意
                </Checkbox>
              </Col>
            </Row>
            <Row>
              <Col style={colStyle}>
                <Switch disabled />
              </Col>
            </Row>
            <Row>
              <Col style={colStyle2}>
                <TextArea rows={3} value="一些文字" disabled />
              </Col>
            </Row>
          </Demo>
          <Demo title="普通激活状态" info="下拉框中非选中的选项在鼠标悬浮时的背景色">
            <Row>
              <Col style={colStyle}>
                <Select dict={dict} />
              </Col>
              <Col style={colStyle}>
                <DatePicker placeholder="请选择日期" />
              </Col>
            </Row>
          </Demo>
        </Sec>
      </CompPage>
    );
  }
}

export default Main;
