import React from 'react';
import CompPage from 'DOC/DocPage';
import Sec from 'DOC/Section';
import commonStyles from '../../styles.less';
// import styles from './styles.less';
import List from 'COMPONENT/List';
import FormItem from 'COMPONENT/FormItem';
import ColorSample from 'DOC/lessPages/ColorSample';
import Demo from 'DOC/lessPages/Demo';
import Row from 'DOC/Row';
import Col from 'DOC/FloatCol';
import { Radio, Checkbox, Switch, Steps, Progress, Breadcrumb, Alert, Tabs } from 'antd';
import Button from 'COMPONENT/Button';
import IconButton from 'COMPONENT/IconButton';
import Input from 'COMPONENT/Input';
import InputNumber from 'COMPONENT/InputNumber';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import DatePicker from 'COMPONENT/DatePicker';
import DateRangePicker from 'COMPONENT/DateRangePicker';
import TagSelect from 'COMPONENT/TagSelect';
import TextArea from 'COMPONENT/TextArea';
import TimePicker from 'COMPONENT/TimePicker';
const TabPane = Tabs.TabPane;
import styleVars from 'COMPONENT/theme/variables';

const list = [
  { col1: '一些内容123', col2: '一些内容abc' },
  { col1: '一些内容123', col2: '一些内容abc' },
];
const columns = [
  { title: '第一列', dataIndex: 'col1' },
  { title: '第二列', dataIndex: 'col2' },
];
const colStyle = {
  width: 200,
};
const colStyle2 = {
  width: 420,
};
const dict = [
  { val: '1', text: '选项1' },
  { val: '2', text: '选项2' },
  { val: '3', text: '选项3' },
];
const dictLong = [
  { val: '1', text: '选项1' },
  { val: '2', text: '选项2' },
  { val: '3', text: '选项3' },
  { val: '4', text: '选项3' },
  { val: '5', text: '选项3' },
  { val: '6', text: '选项3' },
  { val: '7', text: '选项3' },
  { val: '8', text: '选项3' },
  { val: '9', text: '选项3' },
  { val: '10', text: '选项3' },
  { val: '11', text: '选项3' },
  { val: '12', text: '选项3' },
];
const groupDict = [
  {
    label: '第一组',
    options: [
      { val: '1', text: '选项1' },
      { val: '2', text: '选项2' },
      { val: '3', text: '选项3' },
    ],
  },
  {
    label: '第二组',
    options: [
      { val: '4', text: '选项4' },
      { val: '5', text: '选项5' },
    ],
  },
];

class Main extends React.Component {
  state = {
    selectVal: '1',
  };
  render() {
    const { selectVal } = this.state;
    return (
      <CompPage>
        <Sec title="简介">
          文字颜色，也可用于图标，按重要程度不同分4个等级。
          <ul className={commonStyles.ul}>
            <li>
              重要 @fontColorPrimary<span className={commonStyles.tips}> - 重点强调的内容，如标题、激活状态的项目</span>
            </li>
            <li>
              普通 @fontColor<span className={commonStyles.tips}> - 默认字体色</span>
            </li>
            <li>
              次要 @fontColorSecondary
              <span className={commonStyles.tips}> - 需要弱化的内容，如表格的表头、表单的字段标签</span>
            </li>
            <li>
              提示 @fontColorTips<span className={commonStyles.tips}> - 提示性的小字，如表单控件内的占位文字</span>
            </li>
          </ul>
        </Sec>
        <Sec title="样本">
          <ColorSample title="重要" style={{ background: styleVars.fontColorPrimary }} />
          <ColorSample title="普通" style={{ background: styleVars.fontColor }} />
          <ColorSample title="次要" style={{ background: styleVars.fontColorSecondary }} />
          <ColorSample title="提示" style={{ background: styleVars.fontColorTips }} />
        </Sec>
        <Sec title="使用场景">
          <Demo title="大提醒框" info="提示信息为重要，描述文字为普通">
            <Alert
              message="一些说明信息"
              description="信息的详细描述信息的详细描述信息的详细描述"
              type="info"
              showIcon
            />
          </Demo>
          <Demo title="导航" info="激活状态为重要，非激活状态为次要">
            <Breadcrumb>
              <Breadcrumb.Item>主页</Breadcrumb.Item>
              <Breadcrumb.Item>
                <a>产品列表</a>
              </Breadcrumb.Item>
              <Breadcrumb.Item>产品详情</Breadcrumb.Item>
            </Breadcrumb>
            <br />
            <br />
            <Tabs>
              <TabPane tab="产品1" key="1">
                产品1详情...
              </TabPane>
              <TabPane tab="产品2" key="2">
                产品2详情...
              </TabPane>
            </Tabs>
            <br />
            <br />
            <Tabs type="card">
              <TabPane tab="产品1" key="1">
                产品1详情...
              </TabPane>
              <TabPane tab="产品2" key="2">
                产品2详情...
              </TabPane>
            </Tabs>
          </Demo>
          <Demo title="表格" info="表头文字为次要，主体内容为普通">
            <List dataSource={list} columns={columns} />
          </Demo>
          <Demo title="表单" info="字段标签为次要，控件占位文字，选项分组名为提示，其他内容文字、选项文字为普通">
            <div style={{ maxWidth: 500, marginLeft: -50 }}>
              <FormItem name="文字项">内容内容内容</FormItem>
              <FormItem name="选择项">
                <Select dict={groupDict} value={selectVal} onChange={(val) => this.setState({ selectVal: val })} />
              </FormItem>
              <FormItem name="选择项">
                <TextArea rows={3} placeholder="请输入" />
              </FormItem>
            </div>
          </Demo>
        </Sec>
      </CompPage>
    );
  }
}

export default Main;
