import React from 'react';
import CompPage from 'DOC/DocPage';
import Sec from 'DOC/Section';
import commonStyles from '../../styles.less';
// import styles from './styles.less';
import ColorSample from 'DOC/lessPages/ColorSample';
import Demo from 'DOC/lessPages/Demo';
import Row from 'DOC/Row';
import Col from 'DOC/FloatCol';
import { Radio, Checkbox, Switch, Steps, Progress, Tag } from 'antd';
import Button from 'COMPONENT/Button';
import IconButton from 'COMPONENT/IconButton';
import Input from 'COMPONENT/Input';
import InputNumber from 'COMPONENT/InputNumber';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import DatePicker from 'COMPONENT/DatePicker';
import DateRangePicker from 'COMPONENT/DateRangePicker';
import TagSelect from 'COMPONENT/TagSelect';
import TextArea from 'COMPONENT/TextArea';
import TimePicker from 'COMPONENT/TimePicker';

const RadioGroup = Radio.Group;
const Step = Steps.Step;

import styleVars from 'COMPONENT/theme/variables';
console.log(styleVars);

const colStyle = {
  width: 200,
};
const colStyle1 = {
  width: 140,
};
const colStyle2 = {
  width: 420,
};
const dict = [
  { val: '1', text: '选项1' },
  { val: '2', text: '选项2' },
  { val: '3', text: '选项3' },
];
const dictLong = [
  { val: '1', text: '选项1' },
  { val: '2', text: '选项2' },
  { val: '3', text: '选项3' },
  { val: '4', text: '选项3' },
  { val: '5', text: '选项3' },
  { val: '6', text: '选项3' },
  { val: '7', text: '选项3' },
  { val: '8', text: '选项3' },
  { val: '9', text: '选项3' },
  { val: '10', text: '选项3' },
  { val: '11', text: '选项3' },
  { val: '12', text: '选项3' },
];
const groupDict = [
  {
    label: '第一组',
    options: [
      { val: '1', text: '选项1' },
      { val: '2', text: '选项2' },
      { val: '3', text: '选项3' },
    ],
  },
  {
    label: '第二组',
    options: [
      { val: '4', text: '选项4' },
      { val: '5', text: '选项5' },
    ],
  },
];
const inLineStyle = {
  lineHeight: '36px',
};

const inLineStyle2 = {
  lineHeight: '28px',
};
class Main extends React.Component {
  render() {
    return (
      <CompPage>
        <Sec title="简介">
          决定主题风格的颜色，包含1个标准色，及常用的派生色。
          <ul className={commonStyles.ul}>
            <li>
              标准 @primaryColor<span className={commonStyles.tips}> - 用于Logo、可点击元素、元素的激活状态等</span>
            </li>
            <li>
              禁用 @primaryColorDisabled<span className={commonStyles.tips}> - 主色的禁用状态</span>
            </li>
            <li>
              激活 @primaryColorActive<span className={commonStyles.tips}> - 主色的激活状态</span>
            </li>
            <li>
              背景 @primaryColorBg<span className={commonStyles.tips}> - 主色的背景状态</span>
            </li>
          </ul>
        </Sec>
        <Sec title="样本">
          <ColorSample title="激活" style={{ background: styleVars.primaryColorActive }} />
          <ColorSample title="标准" style={{ background: styleVars.primaryColor }} />
          <ColorSample title="禁用" style={{ background: styleVars.primaryColorDisabled }} />
          <ColorSample
            title="背景"
            style={{ background: styleVars.primaryColorBg }}
            fontColor={styleVars.primaryColorDisabled}
          />
        </Sec>
        <Sec title="使用场景">
          <Demo title="可点击元素" info="按钮、文字、图标">
            <Row>
              <Col>
                <Button type="primary">确 定</Button>
              </Col>
              <Col>
                <Button type="primary" disabled>
                  确 定
                </Button>
              </Col>
              <Col>
                <Button>取 消</Button>
              </Col>
              <Col>
                <Button disabled>取 消</Button>
              </Col>
              <Col style={inLineStyle}>
                <a>详情</a>
              </Col>
              <Col style={inLineStyle}>
                <a disabled>详情</a>
              </Col>
              <Col style={inLineStyle2}>
                <IconButton type="cross" activeType="cross_circle" />
              </Col>
              <Col style={inLineStyle2}>
                <IconButton type="cross" activeType="cross_circle" disabled />
              </Col>
              <Col style={inLineStyle}>
                <Tag closable>标签 1</Tag>
                <Tag closable onClose={(e) => e.preventDefault()}>
                  标签 2
                </Tag>
              </Col>
            </Row>
          </Demo>
          <Demo title="激活状态" info="悬浮激活的图标">
            <Row>
              <Col>
                <IconButton type="cross" title="删除" size={20} />
              </Col>
            </Row>
          </Demo>
          <Demo title="激活状态" info="表单控件鼠标移入时、下拉或按钮选项选中时">
            <Row>
              <Col style={colStyle}>
                <Input placeholder="文本框" />
              </Col>
              <Col style={colStyle}>
                <InputNumber placeholder="数字输入框" />
              </Col>
              <Col style={colStyle}>
                <Select dict={dictLong} placeholder="选择框" />
              </Col>
              <Col style={colStyle}>
                <MultiSelect dict={dict} placeholder="多选框" />
              </Col>
            </Row>
            <Row>
              <Col style={colStyle2}>
                <TagSelect dict={dictLong} placeholder="标签多选框" />
              </Col>
              <Col style={colStyle2}>
                <TagSelect dict={groupDict} value={['1', '2']} disabled />
              </Col>
            </Row>
            <Row>
              <Col style={colStyle}>
                <DatePicker placeholder="日期选择框" />
              </Col>
              <Col style={colStyle}>
                <TimePicker placeholder="时间选择框" />
              </Col>
              <Col style={colStyle2}>
                <DateRangePicker placeholder={['开始日期', '结束日期']} />
              </Col>
            </Row>
            <Row>
              <Col style={colStyle2}>
                <TextArea rows={3} placeholder="多行文本框" />
              </Col>
            </Row>
            <div style={{ marginBottom: 20 }}>
              <Radio checked>选中的单选按钮</Radio>
              &nbsp;&nbsp;&nbsp;&nbsp;
              <Radio checked disabled>
                选中且禁用
              </Radio>
              &nbsp;&nbsp;&nbsp;&nbsp;
              <Checkbox checked>选中的复选按钮</Checkbox>
              &nbsp;&nbsp;&nbsp;&nbsp;
              <Checkbox checked disabled>
                选中且禁用
              </Checkbox>
              &nbsp;&nbsp;&nbsp;&nbsp;
              <span style={{ display: 'inline-block', marginTop: 6 }}>
                <Switch checked />
                &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
                <Switch checked disabled />
              </span>
            </div>
          </Demo>
          <Demo title="激活状态" info="进度条、步骤条">
            <Progress percent={30} strokeWidth={5} />
            <br />
            <br />
            <br />
            <Steps current={1}>
              <Step title="已完成" />
              <Step title="进行中" />
              <Step title="待完成" />
            </Steps>
          </Demo>
        </Sec>
      </CompPage>
    );
  }
}

export default Main;
