import React from 'react';
import CompPage from 'DOC/DocPage';
import Sec from 'DOC/Section';
import commonStyles from '../../styles.less';
// import styles from './styles.less';
import List from 'COMPONENT/List';
import FormItem from 'COMPONENT/FormItem';
import ColorSample from 'DOC/lessPages/ColorSample';
import Demo from 'DOC/lessPages/Demo';
import Row from 'DOC/Row';
import Col from 'DOC/FloatCol';
import { Radio, Checkbox, Switch, Steps, Progress, Breadcrumb, Alert, Tabs, Tag, Badge, Tooltip, Icon } from 'antd';
import Button from 'COMPONENT/Button';
import IconButton from 'COMPONENT/IconButton';
import Input from 'COMPONENT/Input';
import InputNumber from 'COMPONENT/InputNumber';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import DatePicker from 'COMPONENT/DatePicker';
import DateRangePicker from 'COMPONENT/DateRangePicker';
import TagSelect from 'COMPONENT/TagSelect';
import TextArea from 'COMPONENT/TextArea';
import TimePicker from 'COMPONENT/TimePicker';
import Modal from 'COMPONENT/Modal';
import Block from 'COMPONENT/Block';
import Popover from 'COMPONENT/Popover';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import showAlert from 'COMPONENT/utils/globalTips/showAlert';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
const TabPane = Tabs.TabPane;
import styleVars from 'COMPONENT/theme/variables';

const list = [
  { col1: '一些文字123', col2: '一些文字abc' },
  { col1: '一些文字123', col2: '一些文字abc' },
];
const columns = [
  { title: '第一列', dataIndex: 'col1' },
  { title: '第二列', dataIndex: 'col2' },
];

const liTitleStyle = {
  fontSize: styleVars.fontSizeHeadingSm,
  margin: '10px 0',
};

const colStyle = {
  width: 200,
};
const colStyle1 = {
  width: 80,
};
const colStyle2 = {
  width: 420,
};
const dict = [
  { val: '1', text: '选项1' },
  { val: '2', text: '选项2' },
  { val: '3', text: '选项3' },
];
const dictLong = [
  { val: '1', text: '选项1' },
  { val: '2', text: '选项2' },
  { val: '3', text: '选项3' },
  { val: '4', text: '选项3' },
  { val: '5', text: '选项3' },
  { val: '6', text: '选项3' },
  { val: '7', text: '选项3' },
  { val: '8', text: '选项3' },
  { val: '9', text: '选项3' },
  { val: '10', text: '选项3' },
  { val: '11', text: '选项3' },
  { val: '12', text: '选项3' },
];
const groupDict = [
  {
    label: '第一组',
    options: [
      { val: '1', text: '选项1' },
      { val: '2', text: '选项2' },
      { val: '3', text: '选项3' },
    ],
  },
  {
    label: '第二组',
    options: [
      { val: '4', text: '选项4' },
      { val: '5', text: '选项5' },
    ],
  },
];

class Main extends React.Component {
  state = {
    modalVisible: false,
    multiSelsctVal: ['1', '2'],
  };

  close = () => {
    this.setState({ modalVisible: false });
  };

  render() {
    const { modalVisible, multiSelsctVal } = this.state;

    return (
      <CompPage>
        <Sec title="简介">
          用于文字和图标，包含正文字体2种尺寸、标题字体3种尺寸。
          <ul className={commonStyles.ul}>
            <div style={liTitleStyle}>正文</div>
            <li>
              标准 @fontSize<span className={commonStyles.tips}> - 默认字体大小</span>
            </li>
            <li>
              小号 @fontSizeSm
              <span className={commonStyles.tips}>
                {' '}
                - 小空间内的文字，如表格、页码栏、标签、标记、小号和加小号按钮、日期选项、开关控件；
                说明性文字，如表单校验提示、工具提示、气泡提示框、大提醒框的描述内容；
                弱化的文字，如选择框的分组名、时间选择框的非选中项
              </span>
            </li>
            <div style={liTitleStyle}>标题</div>
            <li>
              大号 @fontSizeHeadingLg<span className={commonStyles.tips}> - 文章的主标题、特殊强调的文字</span>
            </li>
            <li>
              标准 @fontSizeHeading<span className={commonStyles.tips}> - 模态框标题、主页面标题、内容板块标题</span>
            </li>
            <li>
              小号 @fontSizeHeadingSm
              <span className={commonStyles.tips}>
                {' '}
                - 区块标题，如表单标题；提示性模态框的内容文字，如警告框、确认框
              </span>
            </li>
          </ul>
        </Sec>
        <Sec title="样本">
          <span style={{ fontSize: styleVars.fontSizeHeadingLg }}>大标题</span>
          &nbsp;&nbsp;&nbsp;&nbsp;
          <span style={{ fontSize: styleVars.fontSizeHeading }}>标题</span>
          &nbsp;&nbsp;&nbsp;&nbsp;
          <span style={{ fontSize: styleVars.fontSizeHeadingSm }}>小标题</span>
          &nbsp;&nbsp;&nbsp;&nbsp;
          <span style={{ fontSize: styleVars.fontSize }}>标准</span>
          &nbsp;&nbsp;&nbsp;&nbsp;
          <span style={{ fontSize: styleVars.fontSizeSm }}>小号</span>
        </Sec>
        <Sec title="使用场景">
          <Demo title="小号" info="小空间内的文字之：表格、页码栏">
            <List dataSource={list} columns={columns} pagination={true} />
          </Demo>
          <Demo title="小号" info="小空间内的文字之：标签、标记、小号和加小号按钮、日期选项、开关控件">
            <div style={{ marginBottom: 20 }}>
              <Tag>标签</Tag>
              &nbsp;&nbsp;&nbsp;&nbsp;
              <Switch checkedChildren="开" unCheckedChildren="关" />
              &nbsp;&nbsp;&nbsp;&nbsp;
              <Button size="small">小号按钮</Button>
              &nbsp;&nbsp;&nbsp;&nbsp;
              <Button size="xSmall">加小号按钮</Button>
              &nbsp;&nbsp;&nbsp;&nbsp;
              <Badge count={25}>
                <Icon type="message" style={{ fontSize: 20 }} />
              </Badge>
              &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
              <Badge status="error" text="状态标记" />
            </div>
            <Row>
              <Col style={colStyle}>
                <DatePicker placeholder="请选择日期" />
              </Col>
              <Col style={colStyle2}>
                <TagSelect
                  dict={dict}
                  value={multiSelsctVal}
                  onChange={(val) => this.setState({ multiSelsctVal: val })}
                />
              </Col>
            </Row>
          </Demo>
          <Demo title="小号" info="说明性文字之：表单校验提示">
            待完善。
          </Demo>
          <Demo title="小号" info="说明性文字之：工具提示、气泡提示框">
            <Row>
              <Col style={colStyle}>
                <Tooltip trigger="click" title="一些提示信息">
                  <Button onClick={(e) => e.stopPropagation()}>打开工具提示</Button>
                </Tooltip>
              </Col>
              <Col style={colStyle}>
                <Popover trigger="click" content="一些提示信息">
                  <Button onClick={(e) => e.stopPropagation()}>打开气泡提示框</Button>
                </Popover>
              </Col>
            </Row>
          </Demo>
          <Demo title="小号" info="弱化的文字：选择框的分组名、时间选择框的非选中项、大提醒框的描述内容">
            <Row>
              <Col style={colStyle}>
                <Select dict={groupDict} />
              </Col>
              <Col style={colStyle}>
                <TimePicker placeholder="请选择时间" />
              </Col>
            </Row>
            <Row>
              <Col style={colStyle2}>
                <Alert
                  message="一些说明信息"
                  description="信息的详细描述信息的详细描述信息的详细描述"
                  type="info"
                  showIcon
                />
              </Col>
            </Row>
          </Demo>
          <Demo title="大标题" info="文章的主标题">
            见开发文档页面。
          </Demo>
          <Demo title="标题" info="模态框标题">
            <Button onClick={() => this.setState({ modalVisible: true })}>打开模态框</Button>
            {modalVisible && (
              <Modal title="标题" onOk={this.close} onCancel={this.close}>
                一些内容。
              </Modal>
            )}
          </Demo>
          <Demo title="标题" info="主页面、内容板块标题">
            见系统主页面，以及欢迎页的内容板块。
          </Demo>
          <Demo title="小标题" info="区块标题">
            <Block title="标题" style={{ marginTop: -20 }}>
              <p>任意内容任意内容</p>
              <p>任意内容任意内容任意内容任意内容</p>
              <p>任意内容任意内容任意内容任意内容任意内容任意内容</p>
            </Block>
          </Demo>
          <Demo title="小标题" info="提示性模态框的内容文字">
            <Row>
              <Col style={colStyle}>
                <Button onClick={() => showAlert('请保存当前的修改后再切换！')}>打开警告框</Button>
              </Col>
              <Col style={colStyle}>
                <Button onClick={() => showConfirm('确定删除吗？')}>打开确认框</Button>
              </Col>
            </Row>
          </Demo>
        </Sec>
      </CompPage>
    );
  }
}

export default Main;
