import React from 'react';
import CompPage from 'DOC/DocPage';
import Sec from 'DOC/Section';
import commonStyles from '../../styles.less';
// import styles from './styles.less';
import List from 'COMPONENT/List';
import FormItem from 'COMPONENT/FormItem';
import ColorSample from 'DOC/lessPages/ColorSample';
import Demo from 'DOC/lessPages/Demo';
import Row from 'DOC/Row';
import Col from 'DOC/FloatCol';
import { Radio, Checkbox, Switch, Steps, Progress, Breadcrumb, Alert, Tabs, Tooltip, Tag } from 'antd';
import Button from 'COMPONENT/Button';
import IconButton from 'COMPONENT/IconButton';
import Input from 'COMPONENT/Input';
import InputNumber from 'COMPONENT/InputNumber';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import DatePicker from 'COMPONENT/DatePicker';
import DateRangePicker from 'COMPONENT/DateRangePicker';
import TagSelect from 'COMPONENT/TagSelect';
import TextArea from 'COMPONENT/TextArea';
import TimePicker from 'COMPONENT/TimePicker';
import Popover from 'COMPONENT/Popover';
import Modal from 'COMPONENT/Modal';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
const TabPane = Tabs.TabPane;
import styleVars from 'COMPONENT/theme/variables';

const list = [
  { col1: 'x1', col2: 'y1' },
  { col1: 'x2', col2: 'y2' },
];
const columns = [
  { title: '第一列', dataIndex: 'col1' },
  { title: '第二列', dataIndex: 'col2' },
  { title: '第三列', dataIndex: 'col3' },
];
const colStyle = {
  width: 200,
};
const colStyle2 = {
  width: 420,
};
const dict = [
  { val: '1', text: '选项1' },
  { val: '2', text: '选项2' },
  { val: '3', text: '选项3' },
];
const dictLong = [
  { val: '1', text: '选项1' },
  { val: '2', text: '选项2' },
  { val: '3', text: '选项3' },
  { val: '4', text: '选项3' },
  { val: '5', text: '选项3' },
  { val: '6', text: '选项3' },
  { val: '7', text: '选项3' },
  { val: '8', text: '选项3' },
  { val: '9', text: '选项3' },
  { val: '10', text: '选项3' },
  { val: '11', text: '选项3' },
  { val: '12', text: '选项3' },
];
const groupDict = [
  {
    label: '第一组',
    options: [
      { val: '1', text: '选项1' },
      { val: '2', text: '选项2' },
      { val: '3', text: '选项3' },
    ],
  },
  {
    label: '第二组',
    options: [
      { val: '4', text: '选项4' },
      { val: '5', text: '选项5' },
    ],
  },
];

const sampleStyle = {
  background: '#fff',
  border: '1px solid ' + styleVars.basicColorBorder,
  width: 200,
  height: 100,
  borderRadius: styleVars.borderRadius,
  textAlign: 'center',
  lineHeight: '100px',
  float: 'left',
  marginRight: 30,
};

class Main extends React.Component {
  state = {
    modalVisible: false,
  };

  close = () => {
    this.setState({ modalVisible: false });
  };

  render() {
    const { modalVisible } = this.state;

    return (
      <CompPage>
        <Sec title="简介">
          3种阴影，针对不同类型的组件。
          <ul className={commonStyles.ul}>
            <li>
              下拉框 @shadowDropdown
              <span className={commonStyles.tips}> - 用于表单控件的下拉框，如选择框、日期选择框等</span>
            </li>
            <li>
              弹框 @shadowPopup
              <span className={commonStyles.tips}>
                {' '}
                - 用于弹出的对话框，如模态框、气泡提示框、确认框、自动消失提示框等
              </span>
            </li>
            <li>
              底框 @shadowPanel
              <span className={commonStyles.tips}> - 用于立体卡片效果的底框，如欢迎页的内容板块等</span>
            </li>
          </ul>
        </Sec>
        <Sec title="样本">
          <div style={{ padding: 30, overflow: 'hidden', background: styleVars.basicColor }}>
            <div style={{ ...sampleStyle, boxShadow: styleVars.shadowDropdown }}>下拉框</div>
            <div style={{ ...sampleStyle, boxShadow: styleVars.shadowPopup }}>弹框</div>
            <div style={{ ...sampleStyle, boxShadow: styleVars.shadowPanel, border: 'none' }}>底框</div>
          </div>
        </Sec>
        <Sec title="使用场景">
          <Demo title="下拉框" info="选择框、日期选择框">
            <Row>
              <Col style={colStyle}>
                <Select dict={dict} />
              </Col>
              <Col style={colStyle}>
                <DatePicker />
              </Col>
            </Row>
          </Demo>
          <Demo title="弹框" info="模态框、气泡提示框、自动消失提示框">
            <Row>
              <Col style={colStyle}>
                <Button onClick={() => this.setState({ modalVisible: true })}>打开模态框</Button>
                {modalVisible && (
                  <Modal title="标题" onOk={this.close} onCancel={this.close}>
                    一些内容。
                  </Modal>
                )}
              </Col>
              <Col style={colStyle}>
                <Popover trigger="click" content="一些内容。">
                  <Button onClick={(e) => e.stopPropagation()}>打开气泡提示框</Button>
                </Popover>
              </Col>
              <Col style={colStyle}>
                <Button onClick={() => showMessage('系统错误')}>打开自动消失提示框</Button>
              </Col>
            </Row>
          </Demo>
          <Demo title="底框">见系统主页面，以及欢迎页的内容板块。</Demo>
        </Sec>
      </CompPage>
    );
  }
}

export default Main;
