import { routerRedux } from 'dva/router';
import { login, getLoginUserInfo, getButtonAuthTags, getUserToken, getCookieCallsPhoneCheckFail } from 'SERVICE/common';
import { logout } from 'SERVICE/account';
import { existUserId, getUserOrgRoles, getAllSwitchByCampusId } from 'SERVICE/system';
import { urlBase, IndexAttribute } from 'CONST/config';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import importGrowingIO from 'SERVICE/growingIO';
import checkBrowser from 'SERVICE/checkBrowser';
import * as Qiyu from 'SERVICE/qiyukf';
import { getQiYuUserInfo } from 'SERVICE/oa';
import { OSS_BASE_PREFIX, OSS_PRD_PREFIX } from 'CONST/ossConfig';
import { getHeaders, runFunc } from 'UTIL/common';
import CrmNotification from './CrmNotification';
import { peiYou, twoTeacher, prd } from 'CONST/buildEnv';

// 判断是否是 Chrome 浏览器，否则阻塞进程
checkBrowser();

// 设置Index对象(用于本项目及eduboss项目)
function setGlobalObj(currentLoginUser, authFuncBtnTags, userRoleInfo) {
  if (!window.Index) {
    window.Index = {};
  }

  const isCurrentUserBlBrench = currentLoginUser.isBlBrench === 'true';
  const isCurrentUserBlCampus = currentLoginUser.isBlCampus === 'true';

  window.Index = {
    ...IndexAttribute,
    ...window.Index,
    dataDictCacheMap: {},
    userToken: null,
    OSS_BASE_PREFIX,
    OSS_PRD_PREFIX,
    authFuncBtnTags,
    currentLoginUser,
    getLoginUser: function () {
      return currentLoginUser;
    },
    getLoginUserRole: function () {
      return userRoleInfo;
    },
    isCurrentUserBlGroup: function () {
      return !isCurrentUserBlBrench && !isCurrentUserBlCampus;
    },
    isCurrentUserBlBrench: function () {
      return isCurrentUserBlBrench;
    },
    isCurrentUserBlCampus: function () {
      return isCurrentUserBlCampus;
    },
    isProductionNodeEnv: process.env && [prd, twoTeacher, peiYou].includes(process.env.BUILD_ENV),
    isProductionBuildEnv: process.env && [prd, twoTeacher, peiYou].includes(process.env.BUILD_ENV),
  };
}

function filterUserInfo(user) {
  let roleNames = user.roleName ? user.roleName.split(',') : [];
  let roleIds = user.roleId ? user.roleId.split(',') : [];

  const filterList = ['紧急状态管理员'];

  roleNames = roleNames.filter((ele, i) => {
    const isFiltered = filterList.indexOf(ele) > -1;
    if (isFiltered) {
      roleIds = [...roleIds.slice(0, i), ...roleIds.slice(i + 1)];
    }
    return !isFiltered;
  });

  return {
    ...user,
    roleName: roleNames.join(','),
    roleId: roleIds.join(','),
  };
}

// 登录账户模块，处理登录、用户信息、按钮权限等账户相关数据和逻辑
export default {
  namespace: 'account',
  state: {
    // 是否已登录
    isLogin: false,
    // 当前登录用户信息
    user: {},
    // 当前登录用户的按钮权限
    btnAuths: [],
    // 登录错误
    loginError: false,
    callCenterConfig: {},
    // crm通知模块state
    ...CrmNotification.state,
  },
  reducers: {
    /**
     * 更新状态
     * @param {object} state
     * @param  {object} payload 需要更新的状态键值对
     */
    updateState(state, { payload }) {
      return { ...state, ...payload };
    },
  },
  effects: {
    // 获取用户信息
    *getUserInfo(action, { call, put }) {
      const user = yield call(getLoginUserInfo, {}, true);

      const userId = (user || {}).userId || 'NULL';
      sessionStorage.setItem('USER_ID', userId);

      const headers = getHeaders();
      const X_EX_BOSS = headers['x-ex-boss'] === undefined ? '' : headers['x-ex-boss'];
      sessionStorage.setItem('X_EX_BOSS', X_EX_BOSS);
      user.isExBoss = !!X_EX_BOSS;

      const [authFuncBtnTags, { data: userOrgRoles }] = yield [
        call(getButtonAuthTags, {}, true),
        call(getUserOrgRoles, {}, true),
      ];

      let userRoleInfo = null;
      if (userOrgRoles) {
        userRoleInfo = {
          userOrgRoles,
          mainRole: userOrgRoles.find((role) => role.isMain === 0) || null,
        };
      }

      if (process.env) {
        if (process.env.BUILD_ENV && [prd, twoTeacher, peiYou].includes(process.env.BUILD_ENV)) {
          // 引入 growingio
          importGrowingIO({
            ...user,
            mainRole: userRoleInfo.mainRole,
          });
        }
      }

      setGlobalObj(user, authFuncBtnTags, userRoleInfo);

      yield put({
        type: 'updateState',
        payload: {
          user: filterUserInfo(user),
          userRoleInfo,
          btnAuths: authFuncBtnTags.split(','),
        },
      });

      // 登录成功
      yield put({
        type: 'updateState',
        payload: {
          isLogin: true,
        },
      });

      yield put({
        type: 'getOutcallSwitch',
        payload: {
          user,
        },
      });

      const roleCodes = user.roleCode;
      if (
        roleCodes.indexOf('RECEPTIONIST') !== -1 ||
        roleCodes.indexOf('CONSULTOR') !== -1 ||
        roleCodes.indexOf('STUDY_MANAGER') !== -1
      ) {
        const userToken = yield call(getUserToken);

        if (!window.WebSocket) {
          alert('WebSocket not supported by this browser!');
        }

        const sendData = '[{"oper": "login", "data": "' + userToken + '"}]';
        const ws = new WebSocket('ws://127.0.0.1:1818');
        ws.onerror = function (evt) {
          // console.log('error', evt)
          if (evt.data) {
            alert(evt.data);
          } else {
            showMessage('连接电话机失败,请检查是否安装电话机连接软件并且运行');
          }
        };
        ws.onopen = function () {
          ws.send(sendData);
          ws.close();
        };
        //获取来电token验证失败的数据，同时打开新窗体输入客户信息
        const map = yield call(getCookieCallsPhoneCheckFail);

        if (map) {
          map.forEach((key) => {
            const value = map[key];
            const callsPhonePage = urlBase + 'phoneCustomer.html';
            const url = callsPhonePage + '?phoneNumber=' + key + '&callsTime=' + value + '&token=' + userToken;
            window.open(url);
          });
        }
      }

      yield put({
        type: 'loadCrmNotificationList',
        payload: {},
      });

      yield put({
        type: 'initQiYuKeFu',
        payload: {},
      });

      /**
       * 轮询检测是否需要紧急登出
       */
      setInterval(() => {
        existUserId({ userId: user.userId }, true).then((exist) => {
          if (!exist) {
            logout();
          }
        });
      }, 30000);
    },
    // 登录
    *login({ payload: { code } }, { call, put }) {
      const res = yield call(login, { code });
      if (res) {
        sessionStorage.logined = 'true';
        yield put(routerRedux.push('/'));
      } else {
        yield put({
          type: 'updateState',
          payload: {
            loginError: true,
          },
        });
      }
    },
    // 更新当前用户按钮权限配置
    *updateBtnAuth({ payload: { callback } }, { call, put }) {
      const authFuncBtnTags = yield call(getButtonAuthTags, {}, true);

      if (!window.Index) {
        window.Index = {};
      }
      window.Index.authFuncBtnTags = authFuncBtnTags;

      let btnAuth = authFuncBtnTags.split(',');
      yield put({
        type: 'updateState',
        payload: {
          btnAuths: btnAuth,
        },
      });
      runFunc(callback, btnAuth);
    },
    *getOutcallSwitch({ payload: { user } }, { call, put }) {
      const switchResponseData = yield call(getAllSwitchByCampusId, { campusId: user.campusId }, true);
      let switchList = switchResponseData.data || [];
      let permission;
      let callCenterConfig = {
        hasConsultWorkbrench: true,
        hasOutcallWorkbrench: true,
      };

      // 获取所有配置的开关
      switchList = switchList.reduce((pre, cur) => {
        if (cur.items && cur.items.length) {
          pre.push(...cur.items);
        }
        return pre;
      }, []);

      // 登录后是学习中心，判断是有含有SC_CONSULTANT_CALL
      if (user.isBlCampus === 'true') {
        permission = switchList.find((item) => {
          return item.config === 'SC_CONSULTANT_CALL';
        });
        if (permission) {
          callCenterConfig.hasConsultWorkbrench = permission.flag;
        }
      }

      // 登录后是分公司，判断是有含有SC_BRANCH_CALLOUT
      if (user.isBlBrench === 'true') {
        permission = switchList.find((item) => {
          return item.config === 'SC_BRANCH_CALLOUT';
        });
        if (permission) {
          callCenterConfig.hasOutcallWorkbrench = permission.flag;
        }
      }

      // 设置外呼开关信息
      yield put({
        type: 'updateState',
        payload: {
          callCenterConfig,
        },
      });
    },
    /**
     * 初始化七鱼客服
     *
     * @return {IterableIterator<*>}
     */
    *initQiYuKeFu({ payload }, { call }) {
      try {
        const response = yield call(getQiYuUserInfo);

        Qiyu.initQiYu(response);
      } catch (e) {
        console.warn('IT客服初始化: 获取OA用户信息失败');
      }
    },
    // crm通知模块effects
    ...CrmNotification.effects,
  },
};
