import { stringify } from 'qs';
import { isString } from 'UTIL/lang/data-type';
import ExSocket from './socket';

const SOBOT_CALL_WS_URL = 'wss://ctiws.sobot.com/ws';
const SOCKET_CONNECT_MAX_COUNT = 5;

class SobotCallCenter {
  constructor(props) {
    const { agentId, companyId, appId, token, messageHandler } = props;

    this.props = {
      // 当前座席 id
      agentId,
      // 企业 id
      companyId,
      // 应用节点 id
      appId,
      // sobot 访问令牌
      token,
    };

    // socket 消息回调
    this.messageHandler = messageHandler;

    // socket 重连次数
    this.socketConnectCount = 0;

    // ExSocket 实例
    this.socket = null;
  }

  goOnline() {
    this.connect();
  }

  goOffline() {
    if (this.socket) {
      this.socketConnectCount = 0;
      this.socket.close();
    }
  }

  connect() {
    const queryStr = stringify(this.props);

    const socket = new ExSocket({
      url: `${SOBOT_CALL_WS_URL}?${queryStr}`,
      onOpen: (event) => {
        this.socketConnectCount = 0;
        this.messageHandler({
          resultType: 'online',
          message: 'socket已连接',
          data: {},
        });
      },
      onMessage: (event) => {
        const data = isString(event.data) ? JSON.parse(event.data) : {};

        this.messageHandler({
          resultType: 'message',
          message: '收到socket消息',
          data,
        });
      },
      onClose: (event) => {
        this.messageHandler({
          resultType: 'offline',
          message: 'socket已关闭',
          data: {},
        });
      },
      onError: (event) => {
        if (this.socketConnectCount <= SOCKET_CONNECT_MAX_COUNT) {
          this.socketConnectCount += 1;

          this.connect();
        }

        this.messageHandler({
          resultType: 'error',
          message: 'socket连接错误',
          data: {},
        });
      },
    });

    this.socket = socket;
  }
}

export default SobotCallCenter;
