import {
  loadCommonDict,
  loadBranchDict,
  loadCampusDict,
  loadCommonDictNew,
  loadBranchCampusDict,
  loadClassLevelDict,
  loadMainClassTcDict,
  loadAuxClassTcDict,
  loadSmallMainTc,
  loadSmallClassTcDict,
  loadAllMainClassTcDict,
  loadBaseStatusDict,
  loadProductTypeTree,
  loadClassRoomsByCampusId,
  loadBranchAuxClassTcDict,
  loadBranchSmallClassAuxTc,
  loadBranchCampusTree,
  loadAllBranchCampusTree,
  getCampusByUserBelongBranchApi,
  loadMiniClassTeacherListForClassChange,
  loadProvinceCityDict,
  getOutcallUser,
  getCityTreeDict,
  loadSubjectAliasDict,
} from 'UTIL/dict';
import { chargeType, deductType } from 'CONST/dicts';

/**
 * 配置: 加载字典
 * */
const dictConfig = {
  // 分公司
  branch: loadBranchDict,
  // 用户所属分公司所有学习中心
  branchAllCampus: getCampusByUserBelongBranchApi,
  // 学习中心
  campus: loadCampusDict,
  // 分公司学习中心
  branchCampus: loadBranchCampusDict,
  // 当前用户所属分公司学习中心树
  branchCampusTree: loadBranchCampusTree,
  // 所有分公司学习中心树
  allBranchCampusTree: loadAllBranchCampusTree,
  //产品类型
  productType() {
    return loadCommonDict('PRODUCT_TYPE');
  },
  // 产品性质(全部)
  productCharacter() {
    return loadCommonDict('PRODUCT_CHARACTER');
  },
  // 年份
  year() {
    return loadCommonDict('PRODUCT_VERSION');
  },
  // 月份
  month() {
    return loadCommonDict('MonthType');
  },
  // 季度
  quarter() {
    return loadCommonDict('PRODUCT_QUARTER');
  },
  // 季度类型
  quarterType() {
    return loadCommonDict('QuarterType');
  },
  // 年级
  grade() {
    return loadCommonDict('STUDENT_GRADE');
  },
  // 年级对接新接口
  gradeNew() {
    return loadCommonDictNew('STUDENT_GRADE');
  },
  // 科目
  subject() {
    return loadCommonDict('SUBJECT');
  },
  // 期数
  phase() {
    return loadCommonDict('SMALL_CLASS_PHASE');
  },
  // 星期数
  weekDay() {
    return loadCommonDict('WeekDay');
  },
  // 全部班级类型
  classTypeAll() {
    return loadCommonDict('CLASS_TYPE');
  },
  // 无效标签
  invalid_sign() {
    return loadCommonDict('INVALID_SIGN');
  },
  /**
   * 班级类型
   * @param prdType string (e.g. 'TWO_TEACHER')
   * */
  classType(prdType) {
    return loadCommonDict('CLASS_TYPE', prdType);
  },
  // 全部班级层次
  classLevelAll() {
    return loadClassLevelDict(null);
  },
  // 班级层次
  classLevel(classType) {
    return loadClassLevelDict(classType);
  },
  // 产品系列(全部)
  courseSeries() {
    return loadCommonDict('COURSE_SERIES');
  },
  // 产品系列
  courseSeriesSingle(classType) {
    return loadCommonDict('COURSE_SERIES', classType);
  },
  // 费用类型
  other_category() {
    return loadCommonDict('OTHER_CATEGORY');
  },
  // 所属产品类型
  other_of_category() {
    return loadCommonDict('OTHER_OF_CATEGORY');
  },
  // 产品组
  productGroup(parentId) {
    return loadCommonDict('PRODUCT_GROUP', parentId);
  },
  // 讲座状态
  lectureStatus() {
    return loadCommonDict('LectureClassStatus');
  },
  // 讲座考勤状态
  lectureAttendanceStatus() {
    return loadCommonDict('LectureClassAttendanceStatus');
  },
  // 讲座学生扣费状态
  lectureStdChargeStatus() {
    return loadCommonDict('LectureClassStudentChargeStatus');
  },
  // 双师主讲老师(登录账户能看到的数据)
  twoTcMainTc() {
    return loadMainClassTcDict();
  },
  // 双师辅导老师(登录账户能看到的数据)
  twoTcAuxTc() {
    return loadAuxClassTcDict();
  },
  // 所有双师主讲老师
  allTwoTcMainTc() {
    return loadAllMainClassTcDict();
  },
  // 基础状态('TRUE', 'FALSE')
  baseStatus() {
    return loadBaseStatusDict();
  },
  // 合同类型(全部)
  contractType() {
    return loadCommonDict('ContractType');
  },
  // (合同)支付状态
  paidStatue() {
    return loadCommonDict('ContractPaidStatus');
  },
  // 资源入口
  resEntrance() {
    return loadCommonDict('RES_ENTRANCE');
  },
  // 收款类型
  chargeType() {
    return chargeType;
  },
  // 扣费类型
  deductType() {
    return deductType;
  },
  // 课程状态(未上课、上课中、已完成)
  courseStatus() {
    return loadCommonDict('MiniClassStatus');
  },
  // 直播间
  liveRoom() {
    return loadCommonDict('LIVE_ROOM');
  },
  // 产品类型->产品系列/班级类型->班级层次 树
  productTypeTree() {
    return loadProductTypeTree();
  },
  // 合同产品状态(全部)
  contractProductStatus() {
    return loadCommonDict('ContractProductStatus');
  },
  // 通过学习中心id获取教室
  classRoomsByCampusId(campusId) {
    return loadClassRoomsByCampusId(campusId);
  },
  // 小班班主任(登录账户能看到的数据)
  smallClassMainTc() {
    return loadSmallMainTc();
  },
  // 小班任课老师(登录账户能看到的数据)
  smallClassAuxTc() {
    return loadSmallClassTcDict();
  },
  // `双师辅导老师`(所属分公司下所有学习中心，按学习中心分组)
  branchTwoTcAuxTc() {
    return loadBranchAuxClassTcDict();
  },
  // `小班任课老师`(所属分公司下所有学习中心，按学习中心分组)
  branchSmallClassAuxTc() {
    return loadBranchSmallClassAuxTc();
  },
  // `小班任课老师`(有权限分公司下所有学习中心, 按学习中心分组)
  authBranchSmallClassAuxTc() {
    return loadMiniClassTeacherListForClassChange();
  },
  // 优惠类型
  promotionType() {
    return loadCommonDict('PromotionType');
  },
  // 学生状态字典
  studentStatus() {
    return loadCommonDict('StudentStatus');
  },
  // 网络工作台分配状态字典（未分配，已分配）
  networkCustomerAllocateStatus() {
    return loadCommonDict('NetworkCustomerAllocateStatus');
  },

  // 流量平台-财务类型
  insightFinanceType() {
    return loadCommonDict('InsightFinanceType');
  },
  // 流量平台-活动类型
  insightActivityType() {
    return loadCommonDict('InsightActivityType');
  },
  // 省市数据
  provinceCityDict(isFull = false) {
    return loadProvinceCityDict(isFull);
  },
  // 审核状态
  evidenceAuditStatus() {
    return loadCommonDict('EvidenceAuditStatus');
  },
  // 审核状态
  adjustProject() {
    return loadCommonDict('ADJUST_PROJECT');
  },
  // 客户意向度
  customerIntention() {
    return loadCommonDict('CUSTOMER_SATISFICING');
  },
  // 客户意向产品
  customerSignType() {
    return loadCommonDict('CustomerSignType');
  },
  // 无效客户标签
  customerInvalidSign() {
    return loadCommonDict('INVALID_SIGN');
  },
  // 转班理由
  changeClassReasonDict() {
    return loadCommonDict('CHANGE_CLASS_REASON');
  },
  // 资源来源
  resType() {
    return loadCommonDict('RES_TYPE');
  },
  // 含有外呼角色的用户
  outcallUser() {
    return getOutcallUser();
  },
  // 证件类别
  certificateType() {
    return loadCommonDict('CERTIFICATE_TYPE');
  },
  // 城市树，省市，不含区
  cityTree() {
    return getCityTreeDict();
  },
  // (精英班)产品类别
  ecsClassType() {
    return loadCommonDict('ESC_CLASS_TYPE');
  },
  // 学习中心类型
  campusType() {
    return loadCommonDict('SearchDropClassCampusType');
  },
  /**
   * 科目别名
   *
   * @param params {Object<{ orgId: string, gradeId: string }>}
   * @return {Promise<{val: *, text: (ts.SymbolDisplayPartKind.aliasName|number|*)}[]|Array>}
   */
  subjectAliasDict(params) {
    return loadSubjectAliasDict(params);
  },
};

export default dictConfig;
