import { clone } from 'lodash';

// 用于筛选 productTypeTree 的产品系列、班级类型、班级层次
const productTypeTreeUtils = {
  /**
   * @param keyName String
   * @return Function
   * */
  concatByKeyName(keyName) {
    return (arr, current) => arr.concat(current[keyName]);
  },
  /**
   * @param keyName String
   * @return Function
   * */
  buildDict(keyName) {
    return function (dictTree, value) {
      if (dictTree) {
        if (value) {
          let temp = dictTree.find((item) => item.val === value);
          return temp ? temp[keyName] : null;
        } else {
          return dictTree.reduce(productTypeTreeUtils.concatByKeyName(keyName), []);
        }
      } else {
        return null;
      }
    };
  },
};
// 构建基于 productTypeTree 的产品系列(courseSeries)数据字典
const buildProductSeriesDict = productTypeTreeUtils.buildDict('courseSeries');
// 构建基于 productTypeTree 的班级类型(classType)数据字典
const buildClassTypeDict = productTypeTreeUtils.buildDict('classTypes');
// 构建基于 productTypeTree 的班级层次(classLevel)数据字典
const buildClassLevelDict = function (dictTree, prdTypeVal, classTypeVal) {
  if (dictTree) {
    if (prdTypeVal) {
      let classTypeTree = buildClassTypeDict(dictTree, prdTypeVal);
      return productTypeTreeUtils.buildDict('classLevels')(classTypeTree, classTypeVal);
    } else if (classTypeVal) {
      let classTypeTree = dictTree.reduce(productTypeTreeUtils.concatByKeyName('classTypes'), []);
      return productTypeTreeUtils.buildDict('classLevels')(classTypeTree, classTypeVal);
    } else {
      return dictTree
        .reduce(productTypeTreeUtils.concatByKeyName('classTypes'), [])
        .reduce(productTypeTreeUtils.concatByKeyName('classLevels'), []);
    }
  } else {
    return null;
  }
};

/**
 * 用于过滤 branchCampus(分公司/学习中心) 字典
 * @param branchVal Array | String
 * @param branchCampusDict Array
 * @return Array
 * */
function buildBranchCampusDict(branchVal, branchCampusDict) {
  if (branchVal && branchCampusDict) {
    return Array.isArray(branchVal)
      ? branchVal.map((val) => branchCampusDict.find((item) => item.val === val)).filter((item) => !!item)
      : branchCampusDict.find((item) => item.val === branchVal);
  } else {
    return branchCampusDict || [];
  }
}

// 用于过滤`branchTwoTcAuxTc`与`branchSmallClassAuxTc`
function buildAuxTcDict(branchVal, campusVal, auxTcTree) {
  let dict = branchVal ? buildAuxTcDictByBranch(branchVal, auxTcTree) : buildAuxTcDictByAll(auxTcTree);
  return campusVal ? buildAuxTcDictByCampus(campusVal, dict) : dict;
}

function buildAuxTcDictByBranch(branchVal, auxTcTree) {
  if (branchVal && auxTcTree) {
    if (Array.isArray(branchVal)) {
      let selected = branchVal.map((branch) => auxTcTree.find((item) => item.val === branch)).filter((item) => !!item);
      return buildAuxTcDictByAll(selected);
    } else {
      let single = auxTcTree.find((item) => item.val === branchVal);
      return single ? single.options : null;
    }
  }
  return auxTcTree || null;
}

function buildAuxTcDictByAll(auxTcTree) {
  if (auxTcTree && auxTcTree.length > 0) {
    return auxTcTree.reduce((total, current) => total.concat(current.options), []);
  }
  return null;
}

function buildAuxTcDictByCampus(campusVal, auxTcDict) {
  if (campusVal && auxTcDict) {
    if (Array.isArray(campusVal)) {
      return campusVal.map((val) => auxTcDict.find((item) => item.val === val)).filter((item) => !!item) || null;
    } else {
      let single = auxTcDict.find((item) => item.val === campusVal);
      return single ? [single] : null;
    }
  }
  return auxTcDict || null;
}

export { buildProductSeriesDict, buildClassTypeDict, buildClassLevelDict, buildBranchCampusDict, buildAuxTcDict };
