import { options } from './config';
/**
 * @param {boolean} isTargetMulti
 * */
function getTargetVal(isTargetMulti) {
  return isTargetMulti ? getTargetMultiVal : getTargetSingleVal;
}
/**
 * @param {Array.<Object>} relativeDict
 * @param {Array|string} targetVal
 * */
function getTargetSingleVal(relativeDict, targetVal) {
  let checkIndex = relativeDict.findIndex((dictOption) => dictOption.val === targetVal);
  return checkIndex === -1 ? null : targetVal;
}
/**
 * @param {Array.<Object>} relativeDict
 * @param {Array|string} targetVal
 * */
function getTargetMultiVal(relativeDict, targetVal) {
  let tagValArr = targetVal.filter((tagVal) => relativeDict.findIndex((dictOption) => dictOption.val === tagVal) > -1);
  return tagValArr.length > 0 ? tagValArr : null;
}

/**
 * @param {string} keyName
 * @return {function}
 * */
function concatByKeyName(keyName) {
  return (arr, current) => arr.concat(current[keyName]);
}

/**
 * @param {string} keyName
 * @return {function}
 * */
function createOptionByKeyName(keyName) {
  return (item) => ({
    label: item.text,
    val: item.val,
    [options]: item[keyName],
  });
}

/**
 * @param {string} keyName
 * @return {function}
 * */
function buildChildDict(keyName) {
  return function (dictTree, value) {
    if (dictTree) {
      let filterArr;
      let isArray = Array.isArray(value);
      if (isArray ? value.length > 0 : Boolean(value)) {
        filterArr = dictTree.filter((prdOption) => (isArray ? value.includes(prdOption.val) : prdOption.val === value));
      } else {
        filterArr = dictTree;
      }
      return filterArr.reduce(concatByKeyName(keyName), []);
    } else {
      return null;
    }
  };
}
// 适用于带label的options
function buildChildDictPlus(keyName) {
  return function (dictTree, value) {
    if (dictTree) {
      let filterArr;
      if (dictTree.some((item) => item[options])) {
        filterArr = dictTree.reduce(concatByKeyName(options), []);
      } else {
        filterArr = dictTree;
      }
      let isArray = Array.isArray(value);
      if (isArray ? value.length > 0 : Boolean(value)) {
        filterArr = filterArr.filter((prdOption) =>
          isArray ? value.includes(prdOption.val) : prdOption.val === value,
        );
      }
      return filterArr.map(createOptionByKeyName(keyName));
    } else {
      return null;
    }
  };
}

export { getTargetVal, concatByKeyName, buildChildDict, buildChildDictPlus };
