import { getDictCfg, updateCfg, parentCfg } from './config';
import { getTargetVal, buildChildDict, buildChildDictPlus, concatByKeyName } from './common';

/**
 * 获取`产品类型`关联关系字典
 *
 * @param type {string}
 * @return {function}
 * */
function getRelationsDict(type) {
  if (getDictCfg[type]) {
    return function (dictTree, ...values) {
      let dict = dictTree;
      getDictCfg[type].forEach((_type, index) => {
        dict = buildChildDict(_type)(dict, values[index]);
      });
      return dict;
    };
  } else {
    return function (dictTree) {
      return dictTree;
    };
  }
}
// 适用于需要带label的options
function getRelationsDictPlus(type) {
  if (getDictCfg[type]) {
    return function (dictTree, ...values) {
      let dict = dictTree;
      getDictCfg[type].forEach((_type, index) => {
        dict = buildChildDictPlus(_type)(dict, values[index]);
      });
      return dict;
    };
  } else {
    return function (dictTree) {
      return dictTree;
    };
  }
}

/**
 * 更新关联关系值
 *
 * @param {string} key
 * @param {?string} val
 * @param {Object=} options
 * @param {string} options.type
 * @param {Array.<Object>} options.dictTree
 * @param {Object} options.state
 * @param {Array.<Object>} options.map
 * @return {Object}
 * */
function getRelationPayload(key, val, options) {
  let payload = { [key]: val };
  if (val && options) {
    const { type, dictTree, state, map } = options;
    updateCfg[type].forEach((_type) => {
      let targetKey = map[_type];
      let targetVal = state[targetKey];
      if (targetVal) {
        let values = parentCfg[_type].map((item) => (type === item ? val : state[map[item]]));
        let _dict = getRelationsDict(_type)(dictTree, ...values);
        payload[targetKey] = getTargetVal(Array.isArray(targetVal))(_dict, targetVal);
      }
    });
  }
  return payload;
}
// 适用于需要带label的options
function getRelationPayloadPlus(key, val, options) {
  let payload = { [key]: val };
  if (val && options) {
    const { type, dictTree, state, map } = options;
    updateCfg[type].forEach((_type) => {
      let targetKey = map[_type];
      let targetVal = state[targetKey];
      if (targetVal) {
        let values = parentCfg[_type].map((item) => (type === item ? val : state[map[item]]));
        let _dict = getRelationsDictPlus(_type)(dictTree, ...values);
        if (_dict.some((item) => item.options)) {
          _dict = _dict.reduce(concatByKeyName('options'), []);
        }
        payload[targetKey] = getTargetVal(Array.isArray(targetVal))(_dict, targetVal);
      }
    });
  }
  return payload;
}

export { getRelationsDict, getRelationPayload, getRelationsDictPlus, getRelationPayloadPlus };
