import React from 'react';
import classNames from 'classnames';
import { connect } from 'dva';
import { Icon as AntdIcon } from 'antd';
import Loading from 'COMPONENT/Loading';
import Icon from 'COMPONENT/Icon';
import requestApi from 'SERVICE/course';
import commons from '../common.less';
import styles from './styles.module.less';

class CourseConflictNotice extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      // 排课冲突列表
      result: [],
    };

    this.renderConflictContent = this.renderConflictContent.bind(this);
    this.openCourseSchedulePage = this.openCourseSchedulePage.bind(this);
  }

  componentDidMount() {
    this.init();
  }

  /**
   * 初始化
   */
  async init() {
    this.setState({
      loading: true,
    });

    try {
      const response = await requestApi.getBICourseConflictData();

      if (response && response.data) {
        const { data, statusCode } = response.data || {};

        if (statusCode === 200) {
          this.setState({
            result: data.quarterlist || [],
          });
        }
      }
    } catch (e) {
    } finally {
      this.setState({
        loading: false,
      });
    }
  }

  /**
   * 渲染列表
   * @param list
   */
  renderConflictContent(list = []) {
    if (!Array.isArray(list) || list.length === 0) {
      return (
        <div className={styles.conflictContent}>
          <div className={styles.conflictEmpty}>
            <AntdIcon className={styles.emptyIcon} type="meh-o" />
            <span>暂无排班冲突信息</span>
          </div>
        </div>
      );
    }

    return (
      <div className={styles.conflictContent}>
        {list.map((item) => {
          const { quarterId, quarterName, orgList } = item;

          return (
            <ul className={styles.conflictCard} key={quarterId}>
              {orgList.map((subItem, index) => {
                return (
                  <li className={styles.conflictItem} key={subItem.orgId}>
                    <span className={classNames(styles.conflictQuarter, index > 0 ? styles.conflictQuarterOther : '')}>
                      {quarterName}
                    </span>
                    <span className={styles.conflictOrg}>{subItem.orgName}</span>
                    <span className={styles.conflictResult}>
                      课室冲突 {subItem.calssroomNum} / 老师冲突 {subItem.teacherNum} / 跨校风险 {subItem.campusNum}
                    </span>
                  </li>
                );
              })}
            </ul>
          );
        })}
      </div>
    );
  }

  /**
   * 打开智能排班页面
   */
  openCourseSchedulePage() {
    const { courseScheduleBIUrl, courseScheduleBIName } = this.props;

    const [, queryStr] = courseScheduleBIUrl.split('?path=');

    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path: '/next-compact',
        query: {
          path: queryStr,
          title: courseScheduleBIName,
        },
        title: courseScheduleBIName,
      },
    });
  }

  render() {
    const { courseScheduleBIUrl } = this.props;
    const { result, loading } = this.state;

    if (!courseScheduleBIUrl) {
      return null;
    }

    return (
      <section className={styles.wrapper}>
        <div className={commons.title}>
          <div className={commons.inner}>智能排班</div>
        </div>

        <div className={styles.moreBtn} onClick={this.openCourseSchedulePage}>
          <span>查看更多</span>
          <Icon className={styles.moreIcon} type="right" size={10} />
        </div>

        <Loading loading={loading} />

        {this.renderConflictContent(result)}
      </section>
    );
  }
}

const mapStateToProps = (state) => {
  const { menus } = state.main;

  const [{ rurl: courseScheduleBIUrl = '', name: courseScheduleBIName = '' } = {}] = menus.filter(
    (item) => item.id === 'BI-rptXhBossTotalSchedule',
  );

  return {
    courseScheduleBIUrl,
    courseScheduleBIName,
  };
};

export default connect(mapStateToProps)(CourseConflictNotice);
