import React from 'react';
import { connect } from 'dva';
import { browserHistory } from 'react-router';
import { OLDPAGEBASE } from 'CONST/config';
import commons from '../common.less';
import style from './style.less';
import TextLink from 'COMPONENT/TextLink';
import IconButton from 'COMPONENT/IconButton';
import Loading from 'COMPONENT/Loading';
import NoData from 'COMPONENT/NoData';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { getDonateItems, getDonateItemsValue, refreshDonateVoluntarily } from 'SERVICE/home';
import linkUrl from 'UTIL/linkUrl';

// 刷新时间的本地缓存key
const STORAGE_KEY_DONATION_REFRESH_TIME = 'STORAGE_KEY_DONATION_REFRESH_TIME';
// 结算率 moduleType 字符串
const CR =
  'DONATE_REMIND_MOD_MC_B_CR,DONATE_REMIND_MOD_MC_C_CR,DONATE_REMIND_MOD_OOO_B_CR,DONATE_REMIND_MOD_OOO_C_CR,DONATE_REMIND_MOD_OTM_B_CR,DONATE_REMIND_MOD_OTM_C_CR';
// 学习中心结算率 moduleType 字符串
const CRC = 'DONATE_REMIND_MOD_MC_C_CR,DONATE_REMIND_MOD_OOO_C_CR,DONATE_REMIND_MOD_OTM_C_CR';
class DonationNotice extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      moduleArr: [], // 提醒展示项
      moduleValueArr: [], // 提醒项数据
      data: [], // 拼接的数据
    };

    this.initPage = this.initPage.bind(this);
    this.loadData = this.loadData.bind(this);
    this.loadDonateValue = this.loadDonateValue.bind(this);
    this.formatValueParam = this.formatValueParam.bind(this);
    this.onRefresh = this.onRefresh.bind(this);
    this.formatResponse = this.formatResponse.bind(this);
    this.onItemClick = this.onItemClick.bind(this);
  }

  componentDidMount() {
    this.initPage();
  }

  /**
   * 初始化数据
   */
  initPage() {
    this.setState({ loading: true });
    this.loadData();
  }

  /**
   * 获取数据
   */
  loadData() {
    getDonateItems()
      .then((result) => {
        const data = result.data || [];
        const param = this.formatValueParam(data);

        this.setState(
          {
            moduleArr: data,
          },
          () => {
            this.loadDonateValue(param);
          },
        );
      })
      .catch((e) => {
        this.setState(
          {
            loading: false,
            moduleArr: [],
            moduleValueArr: [],
          },
          () => {
            this.formatResponse();
          },
        );
      });
  }

  /**
   * 获取乐捐值
   * @param param
   */
  loadDonateValue(param) {
    this.setState({ loading: true });

    getDonateItemsValue(param)
      .then((result) => {
        this.setState(
          {
            loading: false,
            moduleValueArr: result.data || [],
          },
          () => {
            this.formatResponse();
          },
        );
      })
      .catch((e) => {
        this.setState(
          {
            loading: false,
            moduleValueArr: [],
          },
          () => {
            this.formatResponse();
          },
        );
      });
  }

  /**
   * 格式化获取乐捐值的参数
   * @return {*} {moduleTypes: ['DONATE_REMIND_MOD_MC_ATT']}
   */
  formatValueParam(data = []) {
    let types = [];

    data.forEach((item) => {
      if (item.moduleType) {
        types.push(item.moduleType);
      }
    });

    return { moduleTypes: types };
  }

  /**
   * 刷新提醒项数据
   */
  onRefresh() {
    const refreshTime = this.getStorageRefreshTime();
    if (refreshTime && new Date().getTime() - refreshTime < 5 * 60 * 1000) {
      showMessage('刷新太频繁啦，系统限制刷新时间间隔为5分钟', 'error');
      return false;
    }

    if (this.state.loading) {
      return false;
    }

    this.setState({ loading: true });

    refreshDonateVoluntarily()
      .then(() => {
        setTimeout(() => {
          this.loadData();
        }, 3000);

        this.updateStorageRefreshTime();
      })
      .catch((e) => {
        this.setState({ loading: false });
      });
  }

  /**
   * 组装两个接口的数据
   * 将itemValueArr.items[]的itemValue拼接到itemArr.items[]
   */
  formatResponse() {
    const { moduleArr, moduleValueArr } = this.state;
    let data = [];

    if (!moduleArr || !moduleValueArr || moduleValueArr.length <= 0) {
      data = [];
    } else {
      // 全量模块
      const tempModule = [...moduleArr];
      // 部分模块
      const tempModuleValue = [...moduleValueArr];

      tempModuleValue.forEach((valueModule) => {
        // 先从moduleArr找到对应匹配的module
        let matchModule = null;

        tempModule.forEach((module) => {
          if (module.moduleType === valueModule.moduleType) {
            matchModule = module;
            return false;
          }
        });

        // 找到匹配module接着查找对应items
        if (matchModule) {
          let items = [];

          valueModule.items.forEach((valueItem) => {
            for (let item of matchModule.items) {
              if (item.itemType === valueItem.itemType) {
                item.itemValue = valueItem.itemValue || 0;
                items.push(item);
                break;
              }
            }
          });

          matchModule.items = items;

          data.push(matchModule);
        }
      });
    }

    this.setState({
      data: data,
    });
  }

  /**
   * 点击某项
   * 跳转url的参数: itemType {错误收款记录: 错误收款记录类型, 小班排课考勤提醒: 乐捐提醒类型}
   * @param item
   */
  onItemClick(item) {
    const { dispatch, menusData } = this.props;

    let hasAuth = false;
    if (item.jumpResId) {
      hasAuth = menusData.some((menu) => {
        return menu.id === item.jumpResId;
      });
    }

    if (!hasAuth) {
      showMessage('您暂无此权限，请与管理员联系！');
      return false;
    }

    let params = { itemType: item.itemType || '' };
    // 初始化的查询参数等
    const initSearchParam = item.jumpResParams || {};
    params = { ...params, ...initSearchParam };

    const menu = menusData.find((menuItem) => menuItem.id === item.jumpResId);
    if (menu) {
      let url = linkUrl.get(menu.newUrl, menu.rurl);
      url = decodeURIComponent(url);
      // 旧页面需要拼接URL参数 /oldPage?src=function/xxx/xxx.html&itemType=xxx
      if (url.indexOf('function') > -1) {
        let [, path] = url.split('?src=');
        if (path) {
          let query = '?';
          if (Object.keys(params).length) {
            for (let key in params) {
              query += `${key}=${params[key]}&`;
            }
            path += query;
          }
          browserHistory.push({
            pathname: OLDPAGEBASE,
            query: {
              src: path,
            },
          });
        }
      } else if (new RegExp(/^\/next-compact/).test(url)) {
        /* BI页面跳转url*/
        let [, path] = url.split('?path=');
        if (path) {
          if (Object.keys(params).length) {
            for (let key in params) {
              path += `&${key}=${params[key]}`;
            }
          }
          dispatch({
            type: 'main/openPage',
            payload: {
              path: '/next-compact',
              refresh: true,
              title: item.itemName,
              query: {
                path,
                title: item.itemName,
              },
            },
          });
        }
      } else if (url) {
        dispatch({
          type: 'main/openPage',
          payload: {
            path: url,
            params: params,
            refresh: true,
            mirrorId: item.itemId || '',
          },
        });
      }
    }
  }

  /**
   * 获取缓存刷新时间戳
   */
  getStorageRefreshTime() {
    if (!window.localStorage) {
      return null;
    }

    const time = window.localStorage.getItem(STORAGE_KEY_DONATION_REFRESH_TIME);

    if (!time) {
      return null;
    }

    return Number(time);
  }

  /**
   * 缓存上一次刷新时间戳
   */
  updateStorageRefreshTime() {
    if (window.localStorage) {
      localStorage.setItem(STORAGE_KEY_DONATION_REFRESH_TIME, new Date().getTime());
    }
  }

  render() {
    const { loading, data } = this.state;

    return (
      <section className={commons.div + ' ' + style.position_rel} style={{ height: 280 }}>
        <Loading loading={loading} />

        <div className={commons.title}>
          <span className={commons.inner}>乐捐提醒</span>
        </div>

        <div className={style.refresh__btn}>
          <IconButton
            type="question_circle"
            title="乐捐报表更新时间点：每天01:30、12:30、18:30、21:30四个时间点更新数据"
          />
          {/*<span className={style.refreshBtn__text} onClick={this.onRefresh}>
            点击刷新
          </span>
          <IconButton type="refresh_circle" title="点击刷新" onClick={this.onRefresh} />*/}
        </div>

        <div className={style.contentWrap} style={{ height: 200, overflow: 'auto' }}>
          {data && data.length > 0 ? (
            <ul className={style.noticeList}>
              {data.map((module, index) => {
                return (
                  <li key={index}>
                    <div className={style.li__left}>{module.moduleName}</div>
                    <div className={style.li__right}>
                      {module.items &&
                        module.items.map((item, i) => {
                          if (item.itemValue <= 0 && CR.indexOf(module.moduleType) === -1) {
                            return '';
                          } else if (CRC.indexOf(module.moduleType) > -1) {
                            return (
                              <div key={i} className={style.right__item}>
                                <span>{item.itemName}</span>
                                <TextLink
                                  className={style.item__number}
                                  onClick={(e) => {
                                    this.onItemClick(item);
                                  }}
                                >{`${item.itemValue}%`}</TextLink>
                              </div>
                            );
                          }

                          return (
                            <div key={i} className={style.right__item}>
                              <span>{item.itemName}</span>
                              <TextLink
                                className={style.item__number}
                                onClick={(e) => {
                                  this.onItemClick(item);
                                }}
                              >
                                {item.itemValue}
                              </TextLink>
                            </div>
                          );
                        })}
                    </div>
                  </li>
                );
              })}
            </ul>
          ) : (
            <NoData />
          )}
        </div>
      </section>
    );
  }
}

function mapStateToProps(state) {
  let menus = [];

  if (state.main && state.main.menus) {
    menus = state.main.menus;
  }

  return {
    menusData: menus,
  };
}

export default connect(mapStateToProps)(DonationNotice);
