import React from 'react';
import { Button, Col, Modal, Pagination, Row, Spin } from 'antd';
import styles from './styles.less';
import { deleteReply, getHasReadUsers, getReplyList, publishNoticeReply, saveHasReadNotice } from 'SERVICE/home';
import { connect } from 'dva';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import Comment from '../Comments/comments';
import TextArea from 'COMPONENT/TextArea';

class NoticeModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      hasReadUserAccount: [], //已读员工列表数据
      replyData: [], //评论列表
      noticeId: this.props.systemNoticeId, //每条系统公告id
      records: 0, //评论条数
      publish: '', //textarea发表框的内容
      isLoading: true,
      currentPage: null,
    };
  }

  componentDidMount() {
    this.getHasReadUserList();
  }

  /**
   * 获取已阅读人数
   * @returns {Promise.<void>}
   */
  async getHasReadUserList() {
    let { noticeId } = this.state;
    let hasReadData = await getHasReadUsers({ systemNoticeId: noticeId });
    let { rows, records } = await this.getNoticeReplyList();
    this.setState({ hasReadUserAccount: hasReadData, replyData: rows, records, isLoading: false });
  }

  /**
   * 获取回复列表
   * @param val
   * @returns {Promise.<{rows: *, records: *}>}
   */
  getNoticeReplyList = async (val) => {
    let replyData = await getReplyList({ systemNoticeId: this.state.noticeId, page: val ? val : 1, rows: 10 });
    let { rows, records } = replyData;
    this.setState({ replyData: rows, records, currentPage: val });
    return { rows, records };
  };
  /**
   * 显示更多
   */
  showAllReadUser = () => {
    this.showTotal.style.display = 'block';
    this.showSome.style.display = 'none';
  };
  /**
   * 隐藏更多
   */
  showSomeReadUser = () => {
    this.showTotal.style.display = 'none';
    this.showSome.style.display = 'block';
  };
  /**
   * 删除评论
   * @param e
   * @returns {Promise.<void>}
   */
  deleteReply = async (e) => {
    let id = e.target.dataset.id;
    let { replyData } = this.state;
    let userId = this.props.userId;
    let { currentPage } = this.state;
    deleteReply({ id, 'user.userId': userId }).then(
      () => {
        showMessage('删除成功', 'success');
        if (replyData.length - 1 == 0 && currentPage > 0) {
          this.getNoticeReplyList(currentPage - 1);
        } else {
          this.getNoticeReplyList(currentPage);
        }
      },
      () => {
        showMessage('删除失败');
      },
    );
  };
  /**
   * 发布评论
   * @returns {Promise.<void>}
   */
  publishReply = async () => {
    let { publish, noticeId, currentPage } = this.state;
    if (!publish) {
      showMessage('请输入您的评论');
      return;
    }
    let content = '<p>' + publish + '</p>';
    publishNoticeReply({ content, 'systemNotice.id': noticeId }).then(
      (res) => {
        showMessage('发布成功', 'success');
        this.getNoticeReplyList(1);
      },
      () => {
        showMessage('发布失败');
      },
    );
    this.setState({ publish: '' });
  };

  /**
   *点击已读
   */
  saveHasReadNotice = () => {
    const { noticeId } = this.state;
    const { handleCancel } = this.props;
    const apiParams = { 'systemNotice.id': noticeId };

    saveHasReadNotice(apiParams).then((res) => {
      handleCancel();
    });
  };

  render() {
    let { hasReadUserAccount, replyData, records, publish, isLoading, currentPage } = this.state;
    let { show } = this.props;
    let hasReadUserLen = hasReadUserAccount.length;
    let someHasReadUserAccount = [];
    if (hasReadUserLen > 20) {
      someHasReadUserAccount = hasReadUserAccount.slice(0, 20);
    }

    return (
      <Modal
        title="系统公告详情"
        visible={true}
        footer={null}
        onCancel={(e) => {
          this.props.handleCancel(true);
        }}
        width={800}
        className={styles.modal}
      >
        <div style={{ padding: 20 }}>
          <div className={styles.head}>{this.props.modalData.title} </div>
          <div className={styles.subHead}>
            作者: {this.props.modalData.createUserName}({this.props.modalData.createUserOrg})
            <span>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</span> 发布时间: {this.props.modalData.createTime}
          </div>
          <div
            className={styles.content}
            dangerouslySetInnerHTML={{
              __html: this.props.modalData.content,
            }}
          ></div>
          {this.props.modalData.realFileName && (
            <div className={styles.realFile}>
              附件: <a href={this.props.modalData.aliPath}>{this.props.modalData.realFileName}</a>
            </div>
          )}
          <div className={styles.readUser} style={{ minHeight: 40 }}>
            {isLoading && (
              <Spin>
                <div>已阅读人数：</div>
                <div>
                  <div>已读员工：</div>
                </div>
              </Spin>
            )}

            {!isLoading && (
              <div>
                <div>已阅读人数: {hasReadUserLen} 人</div>
                <div
                  ref={(dom) => {
                    this.showSome = dom;
                  }}
                  style={{ display: 'block' }}
                >
                  已读员工:
                  {hasReadUserLen < 20 &&
                    hasReadUserAccount.map((v, k) => {
                      return v.userName + '(' + v.userOrg + '),';
                    })}
                  {hasReadUserLen >= 20 &&
                    someHasReadUserAccount.map((v, k) => {
                      return v.userName + '(' + v.userOrg + '),';
                    })}
                  {hasReadUserLen >= 20 && (
                    <a href="#" onClick={this.showAllReadUser}>
                      更多
                    </a>
                  )}
                </div>
                <div
                  ref={(dom) => {
                    this.showTotal = dom;
                  }}
                  style={{ display: 'none' }}
                >
                  已读员工:
                  {hasReadUserAccount.map((v, k) => {
                    return v.userName + ',';
                  })}
                  {
                    <a href="#" onClick={this.showSomeReadUser}>
                      隐藏更多
                    </a>
                  }
                </div>
              </div>
            )}
          </div>
          <div className={styles.publish}>
            <Row gutter={16}>
              <Col span={4} className={styles.subPublish}>
                发布评论
              </Col>
              <Col span={16}>
                <TextArea rows={3} value={publish} onChange={(e) => this.setState({ publish: e.target.value })} />
              </Col>
              <Col span={4} className={styles.subPublish}>
                <Button type="primary" onClick={this.publishReply}>
                  发表
                </Button>
              </Col>
            </Row>
          </div>
          <div>
            <Comment data={replyData} deleteReply={this.deleteReply}>
              {records > 0 && (
                <Pagination current={currentPage || 1} total={records} onChange={this.getNoticeReplyList} />
              )}
            </Comment>
          </div>
          <div style={{ textAlign: 'end', marginTop: show ? 30 : 0 }}>
            {show && (
              <Button type="primary" onClick={this.saveHasReadNotice}>
                我知道了
              </Button>
            )}
          </div>
        </div>
      </Modal>
    );
  }
}

function mapStateToProps(state) {
  return {
    userId: state.account.user.userId,
  };
}

export default connect(mapStateToProps)(NoticeModal);
