import React from 'react';
import commons from '../common.less';
import styles from './styles.less';
import defaultPng from 'ASSET/imgs/45_45.svg';
import { getSystemNoticeList, getHasNotNotice } from 'SERVICE/home';
import Icon from 'COMPONENT/Icon';
import MoreSysNoticeModal from '../Component/MoreSysNoticeModal';

class SystemNotice extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      resultData: [], // 完整数据
      showMoreModal: false, // 显示更多弹窗
    };

    this.renderNoticeList = this.renderNoticeList.bind(this);
  }

  componentDidMount() {
    let { getNoticeById } = this.props;

    getSystemNoticeList().then((res) => {
      const data = res.rows || [];

      this.setState({
        resultData: data,
      });
    });

    getHasNotNotice().then((res) => {
      if (res.length > 0) {
        getNoticeById(res[0]['id'], true);
      }
    });
  }

  /**
   * 渲染列表
   * @param list
   */
  renderNoticeList(list = []) {
    if (list && list.length > 0) {
      return (
        <ul className={styles.systemNotice}>
          {list.map((item, index) => {
            return (
              <li key={index}>
                <div>
                  <img src={defaultPng} alt="" />
                </div>

                <div>
                  <span
                    style={{ cursor: 'pointer' }}
                    data-id={item.id}
                    onClick={(e) => this.props.getNoticeById(item.id)}
                  >
                    {item.title}
                  </span>
                  <br />
                  <span>
                    发布人 :<span>&nbsp;&nbsp;&nbsp;</span>
                    {item.createUserName}
                  </span>
                </div>

                <div>{item.createTime}</div>
              </li>
            );
          })}
        </ul>
      );
    }

    return '';
  }

  render() {
    const { resultData, showMoreModal } = this.state;
    const { displayNum = 1 } = this.props;

    const noticeData = resultData.slice(0, displayNum);

    return (
      <section className={commons.div} style={{ height: 100, border: 0, padding: 0, position: 'relative' }}>
        <div className={commons.title}>
          <div className={commons.inner}>系统公告</div>
        </div>

        <div
          className={styles.moreBtn}
          onClick={(e) => {
            this.setState({ showMoreModal: true });
          }}
        >
          <span>查看更多</span>
          <Icon type="right" size={10} className={styles.more__icon} />
        </div>

        <div style={{ height: displayNum > 1 ? 200 : 60, overflow: 'hidden' }} className={styles.systemNoticeWrap}>
          {this.renderNoticeList(noticeData)}
        </div>

        {showMoreModal && (
          <MoreSysNoticeModal
            onClose={() => {
              this.setState({ showMoreModal: false });
            }}
          >
            {this.renderNoticeList(resultData)}
          </MoreSysNoticeModal>
        )}
      </section>
    );
  }
}

export default SystemNotice;
