import React from 'react';
import { Col, Row, Tabs } from 'antd';

import moment from 'moment';
import { Select } from 'COMPONENT';

import echarts from 'ASSET/js/echarts.min';
import styles from '../styles.less';
import TopList from './TopList';

import { option, teacherClassFireDict, studyManagerClassFireDict } from '../option';
import { getTopListOfAchievement, getTopListOfClassFire, getHomeroomTeacherBonusTop } from 'SERVICE/home';

import { errorsHandle } from 'UTIL/common';

const TabPane = Tabs.TabPane;

/**
 * 星火-集团TOP10概览
 */
class TopTen extends React.Component {
  constructor(props) {
    super(props);

    const startDate = this.initStartAndEndDate()['startDate'];
    const endDate = this.initStartAndEndDate()['endDate'];

    this.state = {
      loading: true,
      defaultActiveKey: '1',
      targetDefaultActiveKey: '6',

      startDate: startDate, // 开始时间
      endDate: endDate, // 结束时间
      teacherStatus: 'TOTAL', // 教师课消类型
      manageStatus: 'TOTAL', // 学管师课消类型
      overviewStatus: 'TOTAL', // 营业指标概览 类型

      staffEcharts: null, // 员工图表对象
      consultantEcharts: null, // 咨询师图表对象
      studyManageEcharts: null, // 学管师图表对象
      teacherClassEcharts: null, // 教师课消图表对象
      manageClassEcharts: null, // 学管师课消图表对象
      homeroomTeacherEcharts: null, // 班主任图表对象

      staffAchivementData: [], // 员工top10
      consultantAchivementData: [], // 咨询师top10
      manageAchivementData: [], // 学管师top10
      homeroomTeacherData: [], // 班主任top10
      teacherClassData: [], // 教师课消top10
      manageClassData: [], // 学管课消top10
    };

    this.chartsOne = null;
    this.chartsTwo = null;
    this.chartsThree = null;
    this.chartsFour = null;
    this.chartsFive = null;
    this.chartsSix = null;
  }

  componentDidMount() {
    this.initContent();
  }

  componentWillUnmount() {
    window.removeEventListener('resize', this.onResize, false);
  }

  initContent = () => {
    let { staffEcharts } = this.state;
    staffEcharts = echarts.init(this.chartsOne);

    this.loadAchivementList().then(({ achivementApiData, xAxisData, seriesData }) => {
      option.xAxis.data = xAxisData;
      option.series[0].data = seriesData;
      staffEcharts.setOption(option);
      this.setState({ staffAchivementData: achivementApiData, staffEcharts });
    });

    window.addEventListener('resize', this.onResize, false);
  };

  initStartAndEndDate = () => {
    let endDate = moment().format('YYYY-MM-DD');
    let startDate = moment().startOf('month').format('YYYY-MM-DD');
    return { startDate, endDate };
  };

  /**
   * top10 业绩
   * @param roleCode   {string}  员工为空  咨询师CONSULTOR，学管师STUDY_MANAGER
   * @returns {Promise.<{achivementApiData: *, xAxisData: Array, seriesData: Array}>}
   */
  loadAchivementList = async (roleCode) => {
    const { startDate, endDate } = this.state;
    const achivementApiParams = {
      startDate,
      endDate,
      isQueryAll: true,
      roleCode: roleCode,
    };

    this.setState({ loading: true });
    const achivementApiData = await getTopListOfAchievement(achivementApiParams);
    let xAxisData = [];
    let seriesData = [];
    achivementApiData.forEach((v, k) => {
      xAxisData.push(v.username);
      seriesData.push(v.amount);
    });

    this.setState({ loading: false });
    return { achivementApiData, xAxisData, seriesData };
  };

  loadHomeTeacherBonusList = async () => {
    const achivementApiParams = {
      orgType: 'GROUP',
    };

    this.setState({ loading: true });
    const { data: homeroomTeacherData } = await getHomeroomTeacherBonusTop(achivementApiParams).catch(errorsHandle());
    let xAxisData = [];
    let seriesData = [];
    homeroomTeacherData.forEach((v, k) => {
      xAxisData.push(v.username);
      seriesData.push(v.totalBonusAmount);
    });

    this.setState({ loading: false });
    return { homeroomTeacherData, xAxisData, seriesData };
  };

  /**
   * top10 课消
   * @param productType {string} 一对一ONE_ON_ONE_COURSE，小班SMALL_CLASS，其他OTHERS，总课消TOTAL
   * @param type {string} 老师teacher，学管师studyManager
   * @returns {Promise.<{classFireApiData: *, xAxisData: Array, seriesData: Array}>}
   */
  loadClassFireList = async (productType, type) => {
    const { startDate, endDate } = this.state;
    const classFireApiParams = { startDate, endDate, isQueryAll: true, productType, type };

    const classFireApiData = await getTopListOfClassFire(classFireApiParams);
    let xAxisData = [];
    let seriesData = [];
    classFireApiData.forEach((v, k) => {
      xAxisData.push(v.username);
      seriesData.push(v.natureAmount);
    });

    return { classFireApiData, xAxisData, seriesData };
  };

  /**
   * top10 tab 栏切换
   * @param key
   */
  top10TabChange = (key) => {
    let { consultantEcharts, studyManageEcharts, homeroomTeacherEcharts, defaultActiveKey } = this.state;
    this.resetSingleOption(option);

    this.setState(
      {
        defaultActiveKey: key == 4 || key == 5 ? defaultActiveKey : key,
        customKey: key,
      },
      () => {
        if (key == 2 && !consultantEcharts) {
          consultantEcharts = echarts.init(this.chartsTwo);
          this.loadAchivementList('CONSULTOR').then(({ achivementApiData, xAxisData, seriesData }) => {
            option.xAxis.data = xAxisData;
            option.series[0].data = seriesData;
            consultantEcharts.setOption(option);
            this.setState({ consultantAchivementData: achivementApiData, consultantEcharts });
          });
        }

        if (key == 3 && !studyManageEcharts) {
          studyManageEcharts = echarts.init(this.chartsThree);
          this.loadAchivementList('STUDY_MANAGER').then(({ achivementApiData, xAxisData, seriesData }) => {
            option.xAxis.data = xAxisData;
            option.series[0].data = seriesData;
            studyManageEcharts.setOption(option);
            this.setState({ manageAchivementData: achivementApiData, studyManageEcharts });
          });
        }

        if (key == 6 && !homeroomTeacherEcharts) {
          homeroomTeacherEcharts = echarts.init(this.chartsSix);
          this.loadHomeTeacherBonusList().then(({ homeroomTeacherData, xAxisData, seriesData }) => {
            option.xAxis.data = xAxisData;
            option.series[0].data = seriesData;
            homeroomTeacherEcharts.setOption(option);
            this.setState({ homeroomTeacherData, homeroomTeacherEcharts });
          });
        }
      },
    );
  };

  /**
   * 选择TOP10 课消不同类型
   */
  onDictChange = (val, type) => {
    let { customKey, teacherClassEcharts, manageClassEcharts } = this.state;

    if (customKey == 4) {
      this.setState({ defaultActiveKey: '4' }, () => {
        if (!teacherClassEcharts) {
          teacherClassEcharts = echarts.init(this.chartsFour);
        }
        if (val === 'TOTAL') {
          this.setMultipleOption(option);
          this.loadClassFireList(val, type).then(({ classFireApiData, xAxisData, seriesData }) => {
            this.mapDataToMultipleOption(classFireApiData, xAxisData, option);
            teacherClassEcharts.setOption(option);
            this.setState({ teacherClassData: classFireApiData, teacherClassEcharts, teacherStatus: val });
          });
        } else {
          this.resetSingleOption(option, val);
          teacherClassEcharts.clear();
          this.loadClassFireList(val, type).then(({ classFireApiData, xAxisData, seriesData }) => {
            option.xAxis.data = xAxisData;
            option.series[0].data = seriesData;
            teacherClassEcharts.setOption(option);
            this.setState({ teacherClassData: classFireApiData, teacherClassEcharts, teacherStatus: val });
          });
        }
      });
    }

    if (customKey == 5) {
      this.setState({ defaultActiveKey: '5' }, () => {
        if (!manageClassEcharts) {
          manageClassEcharts = echarts.init(this.chartsFive);
        }

        if (val === 'TOTAL') {
          this.setMultipleOption(option);
          this.loadClassFireList(val, type).then(({ classFireApiData, xAxisData, seriesData }) => {
            this.mapDataToMultipleOption(classFireApiData, xAxisData, option);
            manageClassEcharts.setOption(option);
            this.setState({ manageClassData: classFireApiData, manageClassEcharts, manageStatus: val });
          });
        } else {
          this.resetSingleOption(option, val);
          manageClassEcharts.clear();
          this.loadClassFireList(val, type).then(({ classFireApiData, xAxisData, seriesData }) => {
            option.xAxis.data = xAxisData;
            option.series[0].data = seriesData;
            manageClassEcharts.setOption(option);
            this.setState({ manageClassData: classFireApiData, manageClassEcharts, manageStatus: val });
          });
        }
      });
    }
  };

  /**
   * 重置option对象部分属性
   */
  resetSingleOption = (option, val) => {
    option.series = [{ type: 'bar', stack: '总量', barWidth: 12, data: [] }];
    option.tooltip.formatter = '{b} <br/>{c}';
    option.legend.data = [];

    if (!val || val === 'ONE_ON_ONE_COURSE') {
      option.color = ['#5B63FF'];
      option.series[0].itemStyle = { emphasis: { color: '#5B63FF' } };
    } else if (val === 'SMALL_CLASS') {
      option.color = ['#DD93F7'];
      option.series[0].itemStyle = { emphasis: { color: '#DD93F7' } };
    } else {
      option.color = ['#FDE1FF'];
      option.series[0].itemStyle = { emphasis: { color: '#FDE1FF' } };
    }
  };

  /**
   * 设置总课消数据 option对象
   * @param option {object}
   */
  setMultipleOption = (option) => {
    option.color = ['#5B63FF', '#DD93F7', '#FDE9FF'];
    option.series = [
      {
        name: '一对一',
        type: 'bar',
        stack: '总量',
        barWidth: 12,
        data: [],
      },
      {
        name: '小班',
        type: 'bar',
        stack: '总量',
        barWidth: 12,
        data: [],
      },
      {
        name: '其他',
        type: 'bar',
        stack: '总量',
        barWidth: 12,
        data: [],
      },
    ];
    option.tooltip.formatter = '{a} <br/>{c}';
  };

  /**
   * 遍历总课消数据到option
   */
  mapDataToMultipleOption = (classFireApiData, xAxisData, option) => {
    option.xAxis.data = xAxisData;
    option.series[0].itemStyle = { emphasis: { color: '#5B63FF' } };
    option.series[1].itemStyle = { emphasis: { color: '#DD93F7' } };
    option.series[2].itemStyle = { emphasis: { color: '#FDE9FF' } };

    classFireApiData.forEach((v, k) => {
      option.series[0].data.push(v.natureAmountOne);
      option.series[1].data.push(v.natureAmountSmall);
      option.series[2].data.push(v.natureAmountOther);
    });
    option.legend.data = ['一对一', '小班', '其他'];
  };

  onResize = () => {
    const { staffEcharts, consultantEcharts, studyManageEcharts, teacherClassEcharts, manageClassEcharts } = this.state;

    if (staffEcharts) staffEcharts.resize();
    if (consultantEcharts) consultantEcharts.resize();
    if (studyManageEcharts) studyManageEcharts.resize();
    if (teacherClassEcharts) teacherClassEcharts.resize();
    if (manageClassEcharts) manageClassEcharts.resize();
  };

  render() {
    const { defaultActiveKey, teacherStatus, manageStatus } = this.state;

    return (
      <div className={styles.section}>
        <div className={styles.title}>
          <div className={styles.inner}>集团TOP10概览</div>
        </div>

        <div className={styles.tabChart}>
          <Tabs onChange={this.top10TabChange} activeKey={defaultActiveKey} className={styles.top10overview}>
            <TabPane tab="员工业绩TOP10" key="1">
              <Row gutter={48}>
                <Col span={14}>
                  <div
                    className={styles.echarts}
                    ref={(dom) => {
                      this.chartsOne = dom;
                    }}
                  />
                </Col>
                <Col span={10}>
                  <TopList data={this.state.staffAchivementData} />
                </Col>
              </Row>
            </TabPane>
            <TabPane tab="咨询师业绩TOP10" key="2">
              <Row gutter={48}>
                <Col span={14}>
                  <div
                    className={styles.echarts}
                    ref={(dom) => {
                      this.chartsTwo = dom;
                    }}
                  />
                </Col>
                <Col span={10}>
                  <TopList data={this.state.consultantAchivementData} />
                </Col>
              </Row>
            </TabPane>
            <TabPane tab="学管师业绩TOP10" key="3">
              <Row gutter={48}>
                <Col span={14}>
                  <div
                    className={styles.echarts}
                    ref={(dom) => {
                      this.chartsThree = dom;
                    }}
                  />
                </Col>
                <Col span={10}>
                  <TopList data={this.state.manageAchivementData} />
                </Col>
              </Row>
            </TabPane>
            <TabPane tab="班主任业绩TOP10" key="6">
              <Row gutter={48}>
                <Col span={14}>
                  <div
                    className={styles.echarts}
                    ref={(dom) => {
                      this.chartsSix = dom;
                    }}
                  />
                </Col>
                <Col span={10}>
                  <TopList data={this.state.homeroomTeacherData} type="homeroomTeacher" />
                </Col>
              </Row>
            </TabPane>
            <TabPane
              tab={
                <Select
                  allowClear={false}
                  placeholder="教师课消TOP10"
                  dict={teacherClassFireDict}
                  value={teacherStatus}
                  onSelect={(e) => this.onDictChange(e, 'teacher')}
                />
              }
              key="4"
            >
              <Row gutter={48}>
                <Col span={14}>
                  <div
                    className={styles.echarts}
                    ref={(dom) => {
                      this.chartsFour = dom;
                    }}
                  />
                </Col>
                <Col span={10}>
                  <TopList data={this.state.teacherClassData} />
                </Col>
              </Row>
            </TabPane>
            <TabPane
              tab={
                <Select
                  allowClear={false}
                  placeholder="学管师课消TOP10"
                  dict={studyManagerClassFireDict}
                  value={manageStatus}
                  onSelect={(e) => this.onDictChange(e, 'studyManager')}
                />
              }
              key="5"
            >
              <Row gutter={48}>
                <Col span={14}>
                  <div
                    className={styles.echarts}
                    ref={(dom) => {
                      this.chartsFive = dom;
                    }}
                  />
                </Col>
                <Col span={10}>
                  <TopList data={this.state.manageClassData} />
                </Col>
              </Row>
            </TabPane>
          </Tabs>
        </div>
      </div>
    );
  }
}

export default TopTen;
