import React from 'react';
import { connect } from 'dva';
import { List, Select, Page } from 'COMPONENT';
import { Col, Row, Tabs } from 'antd';
import { getListOfCash, getListOfClassFire, getSystemNoticeById, saveHasReadNotice } from 'SERVICE/home';
import CustomSvg from './Component/Svg';
import TopTen from './TopTen';
import TopTenSS from './TopTenSS';
import SystemNotice from './SystemNotice';
import NoticeModal from './NoticeModel/noticeModel';
import BICourseConflict from './BICourseConflict';
import DonationNotice from './DonationNotice';
import SysNoticeAndDownload from './SysNoticeAndDownload';
import { overviewClassForeDict } from './option';
import styles from './styles.less';

const TabPane = Tabs.TabPane;

const { COL_WIDTH_S, COL_WIDTH_M } = List;
// 临时添加是否显示双师Top10开关，防止BI接口未上线时切换
const SHOW_SS_TOP_TEN = true;

class Home extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      overviewStatus: 'TOTAL', //营业指标概览 类型
      noticeData: [], // 系统公告
      isVisiable: false, //
      modalData: {}, //系统公告弹出框数据
      systemNoticeId: null, // 系统公告id
      overviewClassFire: [], //营业指标概览 课消数据
      overviewCash: [], //营业指标概览 现金流数据
      current: 1, //营业指标概览 课消数据当前表格页码
      pageSize: 10, //课消
      cashCurrent: 1, //营业指标概览 现金流数据当前表格页码
      cashPageSize: 10, //现金流
      overviewLoading: true, //
      overviewClassLoading: true, //
      loading: true,
      targetDefaultActiveKey: '6',
    };
  }

  componentDidMount() {
    this.initContent();
  }

  initContent = () => {
    const { cashCurrent, cashPageSize } = this.state;

    this.setState({ loading: true });
    Promise.all([this.loadListOfCash(cashCurrent, cashPageSize)])
      .then(() => {
        this.setState({ loading: false });
      })
      .catch((e) => {
        this.setState({ loading: false });
      });
  };

  /**
   * 营业指标现金流数据
   * @returns {Promise.<void>}
   */
  async loadListOfCash(pages, pageSize) {
    let cashApiParams = {
      page: pages,
      rows: pageSize,
    };
    let cashApiData = await getListOfCash(cashApiParams);
    cashApiData.forEach((v, k) => {
      v.ranking = k + 1;
    });
    this.setState({ overviewCash: cashApiData, cashPageSize: pageSize, cashCurrent: pages, overviewLoading: false });
  }

  /**
   * 加载营业指标课消数据
   * @returns {Promise.<void>}
   */
  async loadListOfClassFire(pages, pageSize) {
    let type = this.state.overviewStatus;
    let classFireApiParams = {
      page: pages ? pages : this.state.current,
      rows: pageSize ? pageSize : this.state.pageSize,
      productType: type,
    };
    let apiData = await getListOfClassFire(classFireApiParams);
    apiData.forEach((v, k) => {
      v.ranking = k + 1;
    });
    this.setState({ overviewClassFire: apiData, current: pages, pageSize, overviewClassLoading: false });
    return { apiData };
  }

  /**
   * 选择营业指标课消类型
   * @param e
   */
  selectType = (e) => {
    this.setState({ overviewStatus: e, overviewClassLoading: true, targetDefaultActiveKey: '7' }, () => {
      this.loadListOfClassFire();
    });
  };

  /**
   * 营业指标 tab栏切换
   * @param key
   */
  targetTabChange = (key) => {
    let { overviewCash, cashCurrent, cashPageSize, targetDefaultActiveKey } = this.state;
    this.setState({ targetDefaultActiveKey: key == 7 ? targetDefaultActiveKey : key });
    if (key == 6 && overviewCash.length === 0) {
      this.loadListOfCash(cashCurrent, cashPageSize);
    }
  };

  /**
   * 点击每一条系统公告加载详情
   */
  async getNoticeById(id, show) {
    let apiData = await getSystemNoticeById({ id });
    this.setState({ modalData: apiData, isVisiable: true, systemNoticeId: id, show });
  }

  /**
   * 关闭公告详情弹窗
   *
   * @param isSaveHasRead {boolean} 是否需要调用接口标记已读 默认是false
   */
  handleCancel = (isSaveHasRead) => {
    const { systemNoticeId } = this.state;

    if (isSaveHasRead) {
      const apiParams = { 'systemNotice.id': systemNoticeId };
      saveHasReadNotice(apiParams);
    }

    this.setState({ isVisiable: false });
  };

  render() {
    let {
      targetDefaultActiveKey,
      modalData,
      isVisiable,
      systemNoticeId,
      overviewClassFire,
      overviewCash,
      overviewStatus,
      current,
      pageSize,
      cashCurrent,
      cashPageSize,
      overviewLoading,
      overviewClassLoading,
      loading,
      show,
    } = this.state;
    let len = overviewClassFire.length;
    let cashLen = overviewCash.length;
    let { btnAuths } = this.props;
    let copyBtnauths = [];
    if (btnAuths.length === 0) {
      copyBtnauths = ['SHOW_WELCOME_TOP10', 'SHOW_WELCOME_TOTAL_INCOME'];
    } else {
      copyBtnauths = [...btnAuths];
    }
    const columnsClassFire = [
      {
        title: '名次',
        dataIndex: 'ranking',
        key: 'ranking',
        width: COL_WIDTH_S,
      },
      {
        title: '分公司',
        dataIndex: 'brenchId',
        key: 'brenchId',
        width: COL_WIDTH_M,
      },
      {
        title: '学习中心',
        dataIndex: 'campusId',
        key: 'campusId',
        width: COL_WIDTH_M,
      },
      {
        title: '今日课消(小时)',
        dataIndex: 'todayNatureHours',
        key: 'todayNatureHours',
        width: COL_WIDTH_M,
      },
      {
        title: '今日扣款(元)',
        dataIndex: 'todayAmount',
        key: 'todayAmount',
        width: COL_WIDTH_M,
      },
      {
        title: '本月课消（小时）',
        dataIndex: 'consumeNatureHours',
        key: 'consumeNatureHours',
        width: COL_WIDTH_M,
      },
      {
        title: '本月扣款',
        key: 'consumeAmount',
        dataIndex: 'consumeAmount',
        width: COL_WIDTH_M,
      },
    ];
    const columnsCash = [
      {
        title: '名次',
        dataIndex: 'ranking',
        key: 'ranking',
        width: COL_WIDTH_S,
      },
      {
        title: '分公司',
        dataIndex: 'brenchId',
        key: 'brenchId',
        width: COL_WIDTH_M,
      },
      {
        title: '学习中心',
        dataIndex: 'campusId',
        key: 'campusId',
        width: COL_WIDTH_M,
      },
      {
        title: '今日新签(元)',
        dataIndex: 'today_new',
        key: 'today_new',
        width: COL_WIDTH_M,
      },
      {
        title: '今日续签(元)',
        dataIndex: 'today_re',
        key: 'today_re',
        width: COL_WIDTH_M,
      },
      {
        title: '今日新签直播(元)',
        dataIndex: 'today_live',
        key: 'today_live',
        width: COL_WIDTH_M,
      },
      {
        title: '今日现金流(元)',
        dataIndex: 'today',
        key: 'today',
        width: COL_WIDTH_M,
      },
      {
        title: '本月新签(元)',
        key: 'month_new',
        dataIndex: 'month_new',
        width: COL_WIDTH_M,
      },
      {
        title: '本月续签(元)',
        key: 'month_re',
        dataIndex: 'month_re',
        width: COL_WIDTH_M,
      },
      {
        title: '本月新签直播(元)',
        dataIndex: 'month_live',
        key: 'month_live',
        width: COL_WIDTH_M,
      },
      {
        title: '本月现金流(元)',
        key: 'month',
        dataIndex: 'month',
        width: COL_WIDTH_M,
      },
    ];

    return (
      <Page
        title={false}
        loading={loading}
        footer={false}
        route={this.props.route}
        className={styles.home}
        fitWindowHeight={true}
      >
        <CustomSvg />

        <div style={{ backgroundColor: '#f7f7f7' }}>
          <Row style={{ display: copyBtnauths.includes('SHOW_WELCOME_TOP10') ? 'block' : 'none', marginBottom: 20 }}>
            {this.props.isTwoTeacher && SHOW_SS_TOP_TEN ? <TopTenSS /> : <TopTen />}
          </Row>

          <Row gutter={16}>
            <Col span={12}>
              <DonationNotice />
            </Col>
            <Col span={12}>
              <SysNoticeAndDownload>
                <SystemNotice
                  displayNum={this.props.isBICourseConflictAuth ? 1 : 3}
                  getNoticeById={(val, show) => this.getNoticeById(val, show)}
                />
                {this.props.isBICourseConflictAuth && <BICourseConflict />}
              </SysNoticeAndDownload>
            </Col>
          </Row>

          <Row
            style={{ marginTop: 20, display: copyBtnauths.includes('SHOW_WELCOME_TOTAL_INCOME') ? 'block' : 'none' }}
          >
            <div className={styles.section}>
              <div className={`${styles.title} ${'overviewTitle'}`}>
                <div className={styles.inner}>营业指标概览</div>
              </div>
              <div className={styles.tabChart}>
                <Tabs
                  onChange={this.targetTabChange}
                  activeKey={targetDefaultActiveKey}
                  className={styles.cashoverview}
                >
                  <TabPane tab="现金流" key="6">
                    <List
                      dataSource={overviewCash}
                      columns={columnsCash}
                      loading={overviewLoading}
                      pagination={{
                        current: cashCurrent,
                        total: cashLen,
                        pageSize: cashPageSize,
                      }}
                      onChange={({ current, pageSize }) => this.loadListOfCash(current, pageSize)}
                      ref={(c) => (this._list = c)}
                    />
                  </TabPane>
                  <TabPane
                    tab={
                      <Select
                        className={styles.cashSelect}
                        allowClear={false}
                        placeholder="课消"
                        dict={overviewClassForeDict}
                        value={overviewStatus}
                        onSelect={(e) => this.selectType(e)}
                      />
                    }
                    key="7"
                  >
                    <List
                      dataSource={overviewClassFire}
                      columns={columnsClassFire}
                      loading={overviewClassLoading}
                      pagination={{
                        current,
                        total: len,
                        pageSize,
                      }}
                      onChange={({ current, pageSize }) => this.loadListOfClassFire(current, pageSize)}
                      ref={(c) => (this._list = c)}
                    />
                  </TabPane>
                </Tabs>
              </div>
            </div>
          </Row>
        </div>

        {isVisiable && (
          <NoticeModal
            modalData={modalData}
            systemNoticeId={systemNoticeId}
            handleCancel={this.handleCancel}
            show={show}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  // 首页不能通过`loginUser.isTwoTeacherSym`判断环境
  const user = state.account ? state.account.user : {};
  const isTwoTeacher = user.institution === 'two_teacher';

  const isXinghuo = user && user.institution === 'xinghuo';

  const { menus } = state.main;

  const [{ rurl: courseScheduleBIUrl = '' } = {}] = menus.filter((item) => item.id === 'BI-rptXhBossTotalSchedule');

  return {
    btnAuths: state.account.btnAuths,
    isTwoTeacher,
    isBICourseConflictAuth: isXinghuo && !!courseScheduleBIUrl,
  };
};

export default connect(mapStateToProps)(Home);
