import React from 'react';
import { connect } from 'dva';
import style from './index.less';
import Iframe from 'COMPONENT/Iframe';
import Loading from 'COMPONENT/Loading';
import { parseUrl } from 'UTIL/url';
import { runFunc } from 'UTIL/common';
import { getPath, logout } from './utils';
import { OLDPAGEBASE, NEXT_COMPACT } from 'CONST/config';
import { message } from 'antd';

const classPrefix = 'NextCompact';

class NextCompact extends React.Component {
  constructor(props) {
    super(props);

    this.updatePath = this.updatePath.bind(this);
    this.onIframeLoad = this.onIframeLoad.bind(this);
    this.handleMessage = this.handleMessage.bind(this);
    this.execute = this.execute.bind(this);
    this.openPage = this.openPage.bind(this);
    this.dispatch = this.dispatch.bind(this);
    this.showMessage = this.showMessage.bind(this);
    this.logout = this.logout.bind(this);

    this.XH = {
      userInfo: props.user,
      userRole: props.userRoleInfo,
      userAuth: props.btnAuths,
    };
    sessionStorage.setItem('XH', JSON.stringify(this.XH));

    this.state = {
      path: getPath(props.location),
      loading: false,
    };
    this.iframeWindow = null;
  }

  componentDidMount() {
    this.setState({ loading: true });
    // GlobalEvent.on('nextCompact:execute', this.execute);
    window.addEventListener('message', this.handleMessage, false);
  }

  componentDidUpdate(prevProps, prevState, snapshot) {
    this.updatePath(prevProps, this.props);
  }

  componentWillUnmount() {
    // GlobalEvent.off('nextCompact:execute', this.execute);
    window.addEventListener('message', this.handleMessage, false);
  }

  updatePath(prevProps, nextProps) {
    const { _activeTabId: prev_activeTabId, _tabPanes: prev_tabPanes } = prevProps;
    const { _activeTabId, _tabPanes } = nextProps;

    if (prev_activeTabId === _activeTabId) {
      const prev_tabPane = prev_tabPanes.find((tabPane) => tabPane.id === _activeTabId);
      const tabPane = _tabPanes.find((tabPane) => tabPane.id === _activeTabId);

      if (prev_tabPane && tabPane && prev_tabPane.rurl !== tabPane.rurl) {
        const path = getPath({ query: parseUrl(decodeURIComponent(tabPane.rurl)).params });
        this.setState(
          {
            loading: true,
            path: null,
          },
          () => {
            this.setState({ path });
          },
        );
      }
    } else {
      // 页面已存在，rurl有更新
      const prev_tabPane = prev_tabPanes.find((tabPane) => tabPane.id === _activeTabId);
      const tabPane = _tabPanes.find((tabPane) => tabPane.id === _activeTabId);
      if (prev_tabPane && tabPane) {
        if (
          prev_tabPane.activePageId === tabPane.activePageId &&
          prev_tabPane.id === tabPane.id &&
          prev_tabPane.rurl !== tabPane.rurl
        ) {
          const path = getPath({ query: parseUrl(decodeURIComponent(tabPane.rurl)).params });
          this.setState(
            {
              loading: true,
              path: null,
            },
            () => {
              this.setState({ path });
            },
          );
        }
      }
    }
  }

  onIframeLoad(contentWindow) {
    // 只往同域环境内注入XH对象
    if (!/(http|https):\/\/([\w.]+\/?)\S*/.test(this.state.path) || this.state.path.indexOf(location.host) > -1) {
      contentWindow.XH = this.XH;
    }

    this.iframeWindow = contentWindow;
    this.setState({ loading: false });
  }

  /**
   * 处理message事件
   *
   * event.data:
   * {
   *   type: string,
   *   data: any
   *   mark: 'NEXT_COMPACT',
   * }
   * */
  handleMessage(e) {
    if (e.data && e.data.mark === 'NEXT_COMPACT') {
      this.execute(e.data);
    }
  }

  execute({ type, data, callback }) {
    if (['openPage', 'dispatch', 'showMessage', 'logout'].includes(type)) {
      runFunc(this[type], data, callback);
    }
  }

  /**
   * 打开标签页
   *
   * openType {string} 页面类型[oldPage(rurl), nextCompact(rurl), normal(new_url)]
   * query {object} 页面参数 oldPage至少需传src, nextCompact至少需传path
   * path, params, refresh 均为normal(new_url)类型页面使用
   * */
  openPage({ path, params = {}, refresh = true, openType, query }) {
    if (openType === 'oldPage') {
      this.props.history.push({
        pathname: OLDPAGEBASE,
        query,
      });
    } else if (openType === 'nextCompact') {
      this.props.history.push({
        pathname: NEXT_COMPACT,
        query,
      });
      // FIXME: 打开`next-compact`后，menus没有自动切换
    } else {
      this.props.dispatch({
        type: 'main/openPage',
        payload: { path, params, refresh },
      });
    }
  }

  dispatch({ type, payload = {} }) {
    this.props.dispatch({
      type,
      payload,
    });
  }

  showMessage({ type, msg }) {
    if (type === 'error') {
      message.error(msg);
    } else {
      message.success(msg);
    }
  }

  logout() {
    logout();
  }

  render() {
    return (
      <div className={style[classPrefix]}>
        {this.state.path && (
          <Iframe id={this.state.path} name={this.state.path} src={this.state.path} onLoad={this.onIframeLoad} />
        )}
        <Loading loading={this.state.loading} />
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    _activeTabId: state.main.activeTabId,
    _tabPanes: state.main.tabpanes,
    user: state.account.user,
    userRoleInfo: state.account.userRoleInfo,
    btnAuths: state.account.btnAuths,
  };
};

export default connect(mapStateToProps)(NextCompact);
