/**
 * 以iframe内嵌旧系统页面，对新旧页面中的遮罩做同步处理
 * @property {string} url 旧系统页面URL
 * @property {[object]} listeners 旧页面添加特定的事件监听
 * [{
     eventType: string         事件类型 默认为click
     selector: string          DOM元素选择器
     func: function(element)   事件处理函数
 * }]
 */
import React from 'react';
import { urlBase } from 'CONST/config';
import styles from './styles.less';
import { PROXY_OLD_BOSS, PROXY_OLD_BOSS_URL } from 'CONST/config';

class Main extends React.Component {
  state = {
    masks: [], //弹框遮罩（iframe中的旧页面弹出框时，外部区域也要相应地显示遮罩，可以有多层遮罩）
  };

  /**
   * 添加遮罩
   */
  addMask = () => {
    const oldMasks = this.state.masks;
    this.setState({
      masks: [...oldMasks, { opacity: 0 }],
    });
    setTimeout(() => {
      this.setState({
        masks: [...oldMasks, { opacity: 1 }],
      });
    }, 10);
  };

  /**
   * 删除遮罩
   */
  removeMask = () => {
    const basicMasks = this.state.masks.slice(0, this.state.masks.length - 1);
    setTimeout(() => {
      this.setState({ masks: [...basicMasks, { opacity: 0 }] });
      setTimeout(() => {
        this.setState({ masks: [...basicMasks] });
      }, 150);
    }, 10);
  };

  /**
   * 当弹框完全显示或隐藏后，检查和修复遮罩，保证和弹框数目一致
   * @param  {number} modalLen 弹框数量
   * @param  {number} masksLen 遮罩数量
   */
  checkAndFixMask = (modalLen, masksLen) => {
    const count = masksLen - modalLen;
    if (count !== 0) {
      const basicMasks = this.state.masks.slice(0, this.state.masks.length - count);
      this.setState({ masks: [...basicMasks] });
    }
  };

  /**
   * iframe加载完成时，
   * 1、监听其内嵌页面的打开、关闭模态框事件，在新页面遮罩状态做相应处理
   * 2、监听特定的事件，用于旧页面与新页面交互
   * @param  {object} e 事件对象
   */
  onLoaded = (e) => {
    const { listeners, onIframeLoad } = this.props;
    const oldPageWindow = e.target.contentWindow;
    const jq = oldPageWindow.$;
    const addMask = this.addMask;
    this.jq = jq;
    jq('body')
      .on('show.bs.modal', '.modal', function () {
        // console.log('show.bs.modal')
        jq(this).css('overflow', 'auto');
        addMask();
      })
      .on('shown.bs.modal', '.modal', () => {
        // console.log('shown.bs.modal')
        const modalLen = jq('.modal.in').length;
        const masksLen = this.state.masks.length;
        this.checkAndFixMask(modalLen, masksLen);
      })
      .on('hide.bs.modal', '.modal', () => {
        // console.log('hide.bs.modal')
        this.removeMask();
      })
      .on('hidden.bs.modal', '.modal', () => {
        // console.log('hidden.bs.modal')
        const modalLen = jq('.modal.in').length;
        const masksLen = this.state.masks.length;
        this.checkAndFixMask(modalLen, masksLen);
      });

    if (listeners) {
      listeners.forEach((listener) => {
        const { eventType, selector, func } = listener;
        jq(e.target.contentWindow.document).on(eventType || 'click', selector, function () {
          func(this);
        });
      });
    }

    //通知旧页面，iframe加载且事件绑定完成
    oldPageWindow.onReactPageLoad && oldPageWindow.onReactPageLoad();
  };

  render() {
    const { url } = this.props;
    return (
      <div style={{ height: '100%' }}>
        <iframe
          frameBorder={0}
          width="100%"
          height="100%"
          marginHeight={0}
          marginWidth={0}
          src={`${PROXY_OLD_BOSS ? `/webapp/` : urlBase}iframeTemplate.html?${decodeURIComponent(url)}`}
          style={{ position: 'relative', zIndex: 4 }}
          onLoad={this.onLoaded}
        ></iframe>
        {this.state.masks.map((mask, i) => (
          <div key={i} className={styles.fixedFullscreenMask} style={{ opacity: mask.opacity }}></div>
        ))}
      </div>
    );
  }
}

export default Main;
