import React from 'react';
import { connect } from 'dva';
import styles from './styles.less';
import { DatePicker } from 'antd';
import { Loading, Button, List, SearchBar, Input, Select, TextLink } from 'COMPONENT';
import { loadCommonDictByGroup, loadCampusDict } from 'UTIL/dict';
import * as loaders from '../loaders';
import * as dicts from '../../../common/util/dicts';
import { formatDuringTime, splitDateString, calcIsSameDeliverTarget, trimParams } from '../utils';
import { dateToMoment } from 'COMPONENT/utils/form';
import ModalAddRecord from './ModalAddRecord';

const RangePicker = DatePicker.RangePicker;
const { COL_WIDTH_S, COL_WIDTH_M } = List;

/**
 * 跟进客户tab
 */
class CusList extends React.Component {
  constructor(props) {
    super(props);

    const { campusId } = props;

    this.initSearchParam = {
      studentName: null,
      contact: null,
      gradeId: null,
      resEntranceId: null,
      isAppointCome: null,
      deliverTargetId: null,
      dealStatus: null,
      intentionOfTheCustomerId: null,
      intentionCampusId: campusId,
      schoolName: null,
      customerName: null,
      tempTime: null,
    };

    this.state = Object.assign(
      {
        loading: false,
        searchExpand: false, // 搜索栏展开状态

        studentName: null, // 查询条件 学生姓名
        contact: null, // 查询条件 联系方式
        gradeId: null, // 查询条件 年级
        resEntranceId: null, // 查询条件 资源入口/资源来源
        isAppointCome: null, // 查询条件 是否诺到/是否上门
        deliverTargetId: null, // 查询条件 跟进人
        dealStatus: null, // 查询条件 跟进状态
        intentionOfTheCustomerId: null, // 查询条件 意向度
        intentionCampusId: null, // 查询条件 意向学习中心
        schoolName: null, // 查询条件 学校
        customerName: null, // 查询条件 客户姓名

        tempTime: null, // 临时查询条件 时间(根据时间类型设置不同的起止时间)

        blCampusDict: [], // 字典 学校
        gradeDict: [], // 字典 年级
        entranceDict: [], // 字典 资源入口
        intentionDict: [], // 字典 意向度
        cusTagDict: [], // 字典 客户标签
        deliverTargetDict: [], // 字典 跟进人

        list: [], // 列表数据
        totalRecords: 0, // 总记录数
        currentPage: 1, // 当前页码
        pageSize: 20, // 每页记录数
        currentRecord: null, // 当前选中行，用于操作时使用

        modalAddRecordShow: false, // 显示跟进弹窗
      },
      this.props.defaultSearchConditions || {},
    );
  }

  componentWillReceiveProps(nextProps) {
    const oldDefaultSearchConditions = this.props.defaultSearchConditions || {};
    const newDefaultSearchConditions = nextProps.defaultSearchConditions || {};
    // 不能直接比较两个对象，因此补充一个判断条件
    const hadDefaultSearchConditions =
      Object.keys(oldDefaultSearchConditions).length > 0 || Object.keys(newDefaultSearchConditions).length > 0;

    if (hadDefaultSearchConditions && oldDefaultSearchConditions !== newDefaultSearchConditions) {
      this.reset();
      Promise.resolve(this.setState(newDefaultSearchConditions));
      setTimeout(() => {
        this.onSearch();
      }, 50);
    }
  }

  /**
   * 从系统通知跳转过来的，初始化筛选项值
   */
  initTodoSKey = () => {
    const { isTodo, todoParams } = this.props.params || {};

    if (isTodo) {
      this.setState({ ...todoParams });
    }
  };

  onResolveTodo = (todoParams) => {
    this.setState(
      {
        ...this.initSearchParam,
        ...todoParams,
      },
      () => {
        this.onSearch();
      },
    );
  };

  initContent = async () => {
    await this.initialBranch();

    const category = 'STUDENT_GRADE,RES_ENTRANCE,CUSTOMER_SATISFICING,CustomerSignType';
    const searchParam = Object.assign(this.formatSearchParams(), this.props.defaultSearchConditions || {});
    const [listData, groupDict, deliverTargetDict] = await Promise.all([
      this.loadCusList(searchParam),
      loadCommonDictByGroup(category),
      loaders.loadDeliverTargetDict(),
    ]);
    const { list, totalRecords } = listData;

    this.setState({
      gradeDict: groupDict['STUDENT_GRADE'],
      entranceDict: groupDict['RES_ENTRANCE'],
      intentionDict: groupDict['CUSTOMER_SATISFICING'],
      cusTagDict: groupDict['CustomerSignType'],
      deliverTargetDict,
      list,
      totalRecords,
    });
  };

  /**
   * 初始化学习中心字典
   *
   * @return {Promise<void>}
   */
  async initialBranch() {
    let { branchId, campusId, defaultSearchConditions } = this.props;

    if (defaultSearchConditions) {
      branchId = defaultSearchConditions.branchId || branchId;
      campusId = defaultSearchConditions.campusId || campusId;
    }

    const blCampusDict = await loadCampusDict(branchId);

    this.setState(
      Object.assign(
        {
          intentionCampusId: campusId,
          blCampusDict,
        },
        defaultSearchConditions || {},
      ),
    );
  }

  /**
   * 搜索
   */
  onSearch = () => {
    const params = this.formatSearchParams();
    const trimmedParam = trimParams(params);

    this.setState({ loading: true });
    this.loadCusList({ ...trimmedParam, currentPage: 1 })
      .then(({ list, totalRecords, page }) => {
        this.setState({ loading: false, currentPage: 1, list, totalRecords });
      })
      .catch((e) => {
        this.setState({ loading: false, currentPage: 1, list: [], totalRecords: 0 });
      });
  };

  reset = () => {
    this.setState(this.initSearchParam);
  };

  /**
   * 刷新
   */
  refreshCurrentPage = () => {
    const params = this.formatSearchParams();

    this.setState({ loading: true });

    this.loadCusList(params)
      .then((response) => {
        const { list, totalRecords } = response;

        this.setState({ loading: false, list, totalRecords });
      })
      .catch((e) => {
        this.setState({ loading: false, list: [], totalRecords: 0 });
      });
  };

  /**
   * 翻页
   *
   * @param current {number} 页码
   * @param pageSize {number} 每页条数
   */
  onPaginationChange = ({ current, pageSize }) => {
    this.setState({ loading: true });

    const changed = {
      currentPage: this.state.pageSize !== pageSize ? 1 : current,
      pageSize,
    };

    const params = {
      ...this.formatSearchParams(),
      ...changed,
    };
    this.loadCusList(params).then(({ list, totalRecords }) => {
      this.setState({ loading: false, ...changed, list, totalRecords });
    });
  };

  /**
   * 获取跟进列表
   *
   * @param params
   */
  loadCusList = (params) => {
    return loaders.loadCusList(params);
  };

  /**
   * 格式化搜索参数
   */
  formatSearchParams = () => {
    const {
      studentName,
      contact,
      gradeId,
      resEntranceId,
      isAppointCome,
      deliverTargetId,
      dealStatus,
      intentionOfTheCustomerId,
      intentionCampusId,
      customerName,
      tempTime,
      currentPage,
      pageSize,
    } = this.state;

    let param = {
      studentName,
      contact,
      gradeId,
      resEntranceId,
      isAppointCome,
      deliverTargetId,
      dealStatus,
      intentionOfTheCustomerId,
      intentionCampusId,
      customerName,
      currentPage,
      pageSize,
    };
    const timeParam = formatDuringTime('5', tempTime);

    param = {
      ...param,
      createDateStart: timeParam.createDateStart,
      createDateEnd: timeParam.createDateEnd,
    };

    return param;
  };

  /**
   * 跟进
   *
   * @param record
   */
  onFollow = (record) => {
    this.setState({ currentRecord: record }, () => {
      this.toggleModalRecordShow();
    });
  };

  /**
   * 显示/隐藏跟进记录弹窗
   */
  toggleModalRecordShow = () => {
    this.setState((prevState) => {
      return {
        modalAddRecordShow: !prevState.modalAddRecordShow,
      };
    });
  };

  render() {
    const { userId } = this.props;
    const {
      loading,
      searchExpand,
      studentName,
      contact,
      gradeId,
      isAppointCome,
      dealStatus,
      intentionOfTheCustomerId,
      intentionCampusId,
      customerName,
      tempTime,
      gradeDict,
      intentionDict,
      blCampusDict,
      list,
      totalRecords,
      currentPage,
      pageSize,
    } = this.state;

    const columns = [
      { title: '客户姓名', dataIndex: 'cusName', key: 'cusName', width: COL_WIDTH_M },
      { title: '联系方式', dataIndex: 'contact', key: 'contact', width: 120 },
      { title: '学生姓名', dataIndex: 'studentName', key: 'studentName', width: COL_WIDTH_M },
      { title: '年级', dataIndex: 'gradeName', key: 'gradeName', width: COL_WIDTH_S },
      { title: '学校', dataIndex: 'schoolName', key: 'schoolName', width: COL_WIDTH_M },
      { title: '资源所属学习中心', dataIndex: 'blCampusName', key: 'blCampusName', width: COL_WIDTH_M },
      { title: '意向学习中心', dataIndex: 'intentionCampusName', key: 'intentionCampusName', width: COL_WIDTH_M },
      { title: '资源来源', dataIndex: 'resEntranceName', key: 'resEntranceName', width: 80 },
      { title: '最新跟进记录', dataIndex: 'lastRemark', key: 'lastRemark', width: 180, lines: 2 },
      {
        title: '最后跟进时间',
        dataIndex: 'lastFollowDate',
        key: 'lastFollowDate',
        width: 120,
        render: (text) => {
          return splitDateString(text);
        },
      },
      { title: '外呼跟进状态', dataIndex: 'dealStatusName', key: 'dealStatusName', width: COL_WIDTH_M },
      { title: '是否诺到', dataIndex: 'isAppointCome', key: 'isAppointCome', width: 80 },
      { title: '诺到日期', dataIndex: 'appointmentDate', key: 'appointmentDate', width: 160 },
      { title: '意向度', dataIndex: 'intentionOfTheCustomerName', key: 'intentionOfTheCustomerName', width: 80 },
      { title: '未跟进天数', dataIndex: 'countDay', key: 'countDay', width: COL_WIDTH_M },
      { title: '跟进人', dataIndex: 'deliverTargetName', key: 'deliverTargetName', width: COL_WIDTH_M },
      { title: '创建日期', dataIndex: 'createDate', key: 'createDate', width: 120 },
      {
        title: '操作',
        key: 'action',
        fixed: 'right',
        width: COL_WIDTH_M,
        render: (text, record, index) => {
          const isSameUser = calcIsSameDeliverTarget(record, userId);

          return [
            <TextLink key={1} className={styles.textLink} disabled={!isSameUser} onClick={() => this.onFollow(record)}>
              跟进
            </TextLink>,
          ];
        },
      },
    ];

    return (
      <div className={styles.wrap}>
        <div style={{ display: 'flex' }}>
          <SearchBar
            expand={searchExpand}
            onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
            onSearch={this.onSearch}
            onReset={this.reset}
          >
            <Input
              placeholder="学生姓名"
              value={studentName}
              onChange={(e) => this.setState({ studentName: e.target.value })}
            />
            <Input
              placeholder="联系方式"
              value={contact}
              onChange={(e) => this.setState({ contact: e.target.value })}
            />
            <Select
              placeholder="年级"
              dict={gradeDict}
              value={gradeId}
              onChange={(val) => this.setState({ gradeId: val })}
            />
            <Select
              placeholder="资源来源"
              dict={dicts.ss_CALLOUT_ENTRANCE_DICT}
              value={this.state.resEntranceId}
              onChange={(val) => this.setState({ resEntranceId: val })}
            />
            <Select
              placeholder="是否诺到"
              dict={dicts.SS_IS_APPOINT_COME_DICT}
              value={isAppointCome}
              onChange={(val) => this.setState({ isAppointCome: val })}
            />
            <Select
              placeholder="跟进人"
              dict={this.state.deliverTargetDict}
              value={this.state.deliverTargetId}
              onChange={(val) => this.setState({ deliverTargetId: val })}
            />
            <Select
              placeholder="外呼跟进状态"
              dict={dicts.SS_CALLOUT_DEAL_STATUS_DICT_UN_SIGN}
              value={dealStatus}
              onChange={(val) => this.setState({ dealStatus: val })}
            />
            <Select
              placeholder="意向度"
              dict={intentionDict}
              value={intentionOfTheCustomerId}
              onChange={(val) => this.setState({ intentionOfTheCustomerId: val })}
            />
            <Select
              placeholder="意向学习中心"
              showSearch={true}
              dict={blCampusDict}
              value={intentionCampusId}
              onChange={(val) => {
                this.setState({ intentionCampusId: val });
              }}
            />
            <Input
              placeholder="客户姓名"
              value={customerName}
              onChange={(e) => this.setState({ customerName: e.target.value })}
            />

            <RangePicker
              style={{ width: 280 }}
              placeholder={['创建日期(起)', '创建日期(止)']}
              format="YYYY-MM-DD"
              showTime={false}
              allowClear={true}
              size="large"
              value={tempTime ? [dateToMoment(tempTime[0]), dateToMoment(tempTime[1])] : []}
              onChange={(date, dateString) => {
                this.setState({ tempTime: dateString });
              }}
            />
          </SearchBar>
          <Button style={{ marginLeft: '-20px' }} onClick={this.refreshCurrentPage}>
            刷新
          </Button>
        </div>

        <List
          bordered={true}
          dataSource={list}
          columns={columns}
          pagination={{
            current: currentPage,
            total: totalRecords,
            pageSize,
          }}
          onChange={this.onPaginationChange}
          className={styles.customList}
        />

        {this.state.modalAddRecordShow && (
          <ModalAddRecord
            row={this.state.currentRecord}
            dict={{
              blCampusDict,
              intentionDict,
            }}
            onClose={(type) => {
              if (type === 'success') {
                this.refreshCurrentPage();
              }

              this.toggleModalRecordShow();
            }}
          />
        )}

        <Loading loading={loading} />
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  const user = state.account.user;
  const { userId } = user || {};
  const pathName = state.routing.locationBeforeTransitions.pathname;

  return {
    userId,
    params: state.main.pageParams[pathName] || {},
  };
};

export { CusList };
export default connect(mapStateToProps, null, null, { withRef: true })(CusList);
