import {
  getNewCallingWorkbench,
  getLastFollowRecord,
  updateLastFollowRecord,
  getProfessionalCalling,
  getResourceForCalling,
  assignResource,
} from 'SERVICE/ssCRM';
import { formatDeliverTargetDict } from './utils';
import { transformCalloutResourceToOldResource } from './CusSignedList/ModalCalloutPoolTree/util';

/**
 * 获取跟进人字典
 *
 * @return {Promise<*>}
 */
export async function loadDeliverTargetDict() {
  try {
    const response = await getProfessionalCalling();
    const { data } = response;

    return formatDeliverTargetDict(data);
  } catch (e) {
    return [];
  }
}

/**
 * 获取跟进客户列表
 *
 * @param param
 * @return {Promise<{totalRecords: *, page: *, list: *}>}
 */
export async function loadCusList(param) {
  const { currentPage, customerName } = param;
  let params = {
    ...param,
    pageNo: currentPage,
    cusName: customerName,
  };

  delete params.currentPage;
  delete params.customerName;

  try {
    const { records, rows, page } = await getNewCallingWorkbench(params);
    const data = rows || [];
    const totalRecords = records || 0;

    return { list: data, totalRecords, page };
  } catch (e) {
    return {
      list: [],
      totalRecords: 0,
      page: currentPage,
    };
  }
}

/**
 * 外呼 - 获取上一次跟进记录
 *
 * @param param.lastFollowUpId {string} 上一次跟进ID
 */
export async function loadLastFollowRecord(param) {
  try {
    const response = await getLastFollowRecord(param);
    const { data } = response;

    return data;
  } catch (e) {
    return {};
  }
}

/**
 * 外呼 - 保存跟进记录
 *
 * @param param
 * @return {Promise<Function>}
 */
export function saveLastFollowRecord(param) {
  return updateLastFollowRecord(param);
}

/**
 * 获取外呼专员角色的资源树
 *
 * @param param {object<{roleCode: *}>?} {CONSULTOR: 咨询师, OUTCALL_SPEC: 外呼专员}
 * @return {Promise<*|Array|Array>}
 */
export async function getCalloutResourceTree(param) {
  try {
    const response = await getResourceForCalling(param);
    const { data } = response;

    return transformCalloutResourceToOldResource(data);
  } catch (e) {
    return [];
  }
}

/**
 * 客户调配(双师)
 *
 * @param param.deliverTarget {string} 新调配资源池ID（树节点ID）
 * @param param.deliverType {string} 新调配资源池类型
 * @param param.sourceList {array} 被分配客户列表 [{cusId: '客户ID', studentId: '学生ID', deliverTarget: '原跟进人', deliverType: '原有跟进人类型'}]
 * @return {Promise<*>}
 */
export function doAllocateCustomerResource(param) {
  return assignResource(param);
}
