import React from 'react';
import { Row, Col } from 'antd';
import { Input, Select, FormItem } from 'COMPONENT';
import { addCusFormCfg, addCusModalRowStyle } from '../utils';
import { sexDict } from 'PAGE/common/dicts';
import SelectSchoolModal from 'PAGE/common/SelectSchoolModal';
import { loginUser } from 'UTIL/common';

const phoneRegExp = /^1\d{10}$/;

class NewStudent extends React.Component {
  state = {
    selectSchoolModalVis: false, // 选择学校弹框显示状态
    error: null, // 表单错误信息
  };

  onChange = (filedName, val) => {
    const { data, onChange } = this.props;
    onChange({ ...data, [filedName]: val });
  };

  /**
   * 手机号码改变
   *
   * @param e
   */
  onPhoneChange = (e) => {
    const val = e.target.value;
    const valid = phoneRegExp.test(val);
    const error = val && !valid ? '请输入正确手机号' : null;

    this.setState({ error });
    this.props.onErrorChange('contact', error);
    this.onChange('contact', e.target.value);
  };

  /**
   * 手机号码失焦
   *
   * @param e
   */
  onPhoneBlur = (e) => {
    const val = e.target.value;
    const value = val ? val.trim() : val;

    this.onChange('contact', value);
  };

  render() {
    const {
      gradeDict,
      data: { name, gradeId, schoolName, sex, contact, address, idCardNo },
      userProvince,
      userCity,
    } = this.props;
    const { selectSchoolModalVis, error } = this.state;
    const phoneSilent = !(contact && error);

    return (
      <Row style={addCusModalRowStyle}>
        <Col span={12}>
          <FormItem {...addCusFormCfg} name="学生姓名">
            <Input value={name} onChange={(e) => this.onChange('name', e.target.value)} />
          </FormItem>
        </Col>
        <Col span={12}>
          <FormItem {...addCusFormCfg} name="学生年级">
            <Select dict={gradeDict} value={gradeId} onChange={(val) => this.onChange('gradeId', val)} />
          </FormItem>
        </Col>
        {loginUser.isTwoTeacherSym() && (
          <Col span={12}>
            <FormItem {...addCusFormCfg} name="学生身份证">
              <Input value={idCardNo} onChange={(e) => this.onChange('idCardNo', e.target.value)} />
            </FormItem>
          </Col>
        )}
        <Col span={12}>
          <FormItem {...addCusFormCfg} name="学校">
            <Input
              readOnly
              value={schoolName}
              onClick={() => {
                this.setState({ selectSchoolModalVis: true });
              }}
            />
          </FormItem>
        </Col>
        <Col span={12}>
          <FormItem {...addCusFormCfg} name="性别">
            <Select dict={sexDict} value={sex} onChange={(val) => this.onChange('sex', val)} />
          </FormItem>
        </Col>
        <Col span={12}>
          <FormItem {...addCusFormCfg} name="联系方式">
            <Input
              value={contact}
              validator={{ rules: ['phone'], silent: phoneSilent }}
              onChange={this.onPhoneChange}
              onBlur={this.onPhoneBlur}
            />
          </FormItem>
        </Col>
        <Col span={12}>
          <FormItem {...addCusFormCfg} name="家庭住址">
            <Input value={address} onChange={(e) => this.onChange('address', e.target.value)} />
          </FormItem>
        </Col>

        {selectSchoolModalVis && (
          <SelectSchoolModal
            defaultProvince={userProvince}
            defaultCity={userCity}
            onConfirm={(school, schoolName) => {
              const { data, onChange } = this.props;
              onChange({ ...data, schoolId: school, schoolName });
              this.setState({ selectSchoolModalVis: false });
            }}
            onClose={() => this.setState({ selectSchoolModalVis: false })}
          />
        )}
      </Row>
    );
  }
}
export default NewStudent;
