import React from 'react';
import styles from './styles.less';
import { Block, FormItem, Input, Select, Section, TextArea, PopTip, Modal, Button } from 'COMPONENT';
import { getFieldWithFiller, getCommonErrorTips } from 'COMPONENT/utils/form';
import * as loaders from '../../util/loaders';
import { getDictTextByVal, loadCommonDict, loadCampusDict } from 'UTIL/dict';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import SelectSchoolModal from 'PAGE/common/SelectSchoolModal';
import { CUS_LIST_LOST } from '../../../consultant/FollowCustomer/dicts';
import { findOrganizationById } from 'SERVICE/common';

const formCfg = {
  labelWidth: 100,
  size: 'small',
};

class CusDetailBlock extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      editing: false, // 是否编辑状态
      checking: false, // 校验开启状态
      initData: null, // 备份数据，用于取消时回退

      studentName: null, // 学生姓名
      gradeId: null, // 年级
      schoolId: null, // 学校
      schoolName: null, // 学校名称
      cusName: null, // 客户姓名
      contact: null, // 客户电话
      resEntranceId: null, // 资源入口
      resEntranceName: null, // 资源入口名称
      blCampusId: null, // 资源所属学习中心ID
      blCampusName: null, // 资源所属学习中心名称
      intentionCampusId: null, // 意向学习中心ID
      intentionCampusName: null, // 意向学习中心名称
      lastDeliverName: null, // 分配人
      deliverTargetName: null, // 跟进人
      dealStatus: null, // 跟进状态
      dealStatusName: null, // 跟进状态名称
      remark: '', // 备注

      reactivated: '',
      preEntranceId: '',

      gradeDict: [], // 字典，年级
      entranceDict: [], // 字典，资源入口
      followStatusDict: [], // 字典，跟进状态
      campusDict: [], // 字典，所属学习中心
      initCampusDict: [], // 初始所属学习中心字典，用于取消时回退

      saveTipsVis: false, // 保存按钮提示气泡显示状态
      saveTips: null, // 保存按钮提示信息

      selectSchoolModalVis: false, // 选择学校弹框显示状态
      mergeStudentInfoModalVisible: false,
      mergeStudentInfoMsg: null,
      newStudentId: null,
      isMergeLost: null,
      provinceId: null,
      cityId: null,

      isFinishOrStopClass: false,
    };

    this.errors = {};
  }

  componentDidMount() {
    this.initContent();
  }

  initContent = () => {
    const { onLoadingStart, onLoadingEnd, cusId, studentId, deliverTarget } = this.props;
    const { organizationId, campusId, brenchId, isBlCampus, campusName } = this.props.user || {};

    // 判断是否学习中心
    const isCampus = isBlCampus === 'true' || isBlCampus === true;
    const userCampusOption = isCampus ? [{ val: campusId, text: campusName }] : [];

    onLoadingStart();
    Promise.all([
      loaders.loadCusDetail(cusId, studentId, deliverTarget),
      loadCommonDict('STUDENT_GRADE'),
      loadCommonDict('RES_ENTRANCE'),
      findOrganizationById({ id: campusId || brenchId }),
      loadCampusDict(brenchId),
    ])
      .then(([cusDetail, gradeDict, entranceDict, organizationInfo, campusDict]) => {
        onLoadingEnd();

        let provinceId = '';
        let cityId = '';
        if (organizationId === '000001') {
          provinceId = '440000';
          cityId = '441900';
        } else {
          const { city, province } = organizationInfo;
          provinceId = province ? province.id : '';
          cityId = city ? city.id : '';
        }

        const userCampusDict = isCampus ? userCampusOption : campusDict;

        this.setState({
          ...cusDetail,
          initData: cusDetail,
          gradeDict,
          entranceDict,
          campusDict: userCampusDict,
          initCampusDict: userCampusDict,
          provinceId,
          cityId,
        });
      })
      .catch(() => {
        onLoadingEnd();
      });
  };

  /**
   * 追加学习中心下拉选项，解决跨学习中心无权限问题
   *
   * @param campusDict {array} 权限学习中心字典
   * @param campusId {string} 追加的学习中心ID
   * @param campusName {string} 追加的学习中心名称
   */
  appendCampusDict = (campusDict = [], campusId = '', campusName = '') => {
    if (!campusId || !campusName) {
      return campusDict;
    }

    const appendDict = [{ val: campusId, text: campusName }];
    if (!campusDict || campusDict.length <= 0) {
      return appendDict;
    }

    const isExist = campusDict.some((item) => item.val === campusId);
    if (isExist) {
      return campusDict;
    }

    return campusDict.concat(appendDict);
  };

  cancel = () => {
    const { initData, initCampusDict } = this.state;
    this.setState({ editing: false, ...initData, campusDict: initCampusDict });
  };

  /**
   * 保存客户信息
   */
  onOk = () => {
    const { cusId, onLoadingStart, onLoadingEnd, studentId } = this.props;
    const { checking, studentName, gradeId, schoolId, cusName, remark, intentionCampusId, campusDict, initData } =
      this.state;
    if (!checking) {
      this.setState({ checking: true });
    }
    const errorTips = getCommonErrorTips(this.errors);
    if (errorTips) {
      this.showSaveTips(errorTips);
      return;
    }

    const details = { studentName, gradeId, schoolId, cusName, remark, intentionCampusId };

    onLoadingStart();
    loaders.saveCusDetail({ cusId, studentId, ...details }).then(
      (res) => {
        const data = res.data || {};
        const mergeStudentInfo = data.mergeStudentInfo;

        if (mergeStudentInfo) {
          this.setState({
            newStudentId:
              mergeStudentInfo.mergeSuccess && !mergeStudentInfo.lost ? mergeStudentInfo.newStudentId : null,
            mergeStudentInfoModalVisible: true,
            isMergeLost: mergeStudentInfo.lost,
            mergeStudentInfoMsg: mergeStudentInfo.mergeMessage,
          });
          if (!mergeStudentInfo.mergeSuccess) {
            return onLoadingEnd();
          }
        } else {
          showMessage('保存成功', 'success');
        }
        this.setState({
          loading: false,
          editing: false,
          checking: false,
          initData: { ...initData, ...details },
          initCampusDict: campusDict,
        });
        onLoadingEnd();
      },
      (e) => {
        if (e.text) {
          this.showSaveTips(e.text);
        } else {
          showMessage('保存失败');
        }
        onLoadingEnd();
      },
    );
  };

  showSaveTips = (text) => {
    this.setState({ saveTipsVis: true, saveTips: text || '请正确填写信息' });
  };

  errorChange = (key, error) => {
    this.errors[key] = error;
  };

  onMergeModalClose = () => {
    const { newStudentId, isMergeLost } = this.state;
    const { pageParams, onCloseTab, dispatch } = this.props;

    if (pageParams.onRefresh) {
      if (newStudentId) {
        pageParams.onRefresh({
          ...pageParams,
          studentId: newStudentId,
        });
      }

      if (isMergeLost === true) {
        dispatch({
          type: 'main/openPage',
          payload: {
            params: {
              initCusList: CUS_LIST_LOST,
            },
            refresh: true,
            path: '/SSConsultantFollowCustomer',
          },
        });
      }

      onCloseTab();
    }
  };

  render() {
    const { editable } = this.props;
    const {
      editing,
      checking,
      studentName,
      gradeId,
      schoolName,
      cusName,
      contact,
      resEntranceId,
      resEntranceName,
      blCampusId,
      blCampusName,
      intentionCampusId,
      intentionCampusName,
      lastDeliverName,
      deliverTargetName,
      dealStatus,
      dealStatusName,
      remark,
      gradeDict,
      entranceDict,
      campusDict,
      followStatusDict,
      saveTipsVis,
      saveTips,
      selectSchoolModalVis,
      mergeStudentInfoModalVisible,
      mergeStudentInfoMsg,
      provinceId,
      cityId,
    } = this.state;
    const blCampusDict = this.appendCampusDict(campusDict, blCampusId, blCampusName);
    const intentionCampusDict = this.appendCampusDict(campusDict, intentionCampusId, intentionCampusName);

    return (
      <div className={styles.wrap}>
        <Block>
          <Section
            title="客户详情"
            toolBar={
              editable &&
              (editing ? (
                <span>
                  <PopTip
                    visible={saveTipsVis}
                    content={saveTips}
                    onClose={() => this.setState({ saveTipsVis: false })}
                    getPopupContainer={(ele) => ele.parentNode}
                  >
                    <Button type="primary" style={{ marginRight: 20 }} onClick={this.onOk}>
                      保存
                    </Button>
                  </PopTip>
                  <Button onClick={this.cancel}>取消</Button>
                </span>
              ) : (
                <Button onClick={() => this.setState({ editing: true })}>编辑</Button>
              ))
            }
          >
            <div className={styles.inner}>
              <FormItem {...formCfg} name="学生姓名">
                {editing ? (
                  <Input value={studentName} onChange={(e) => this.setState({ studentName: e.target.value })} />
                ) : (
                  <div style={{ wordBreak: 'break-word' }}>{getFieldWithFiller(studentName)}</div>
                )}
              </FormItem>
              <FormItem {...formCfg} name="年级">
                {editing ? (
                  <Select dict={gradeDict} value={gradeId} onChange={(val) => this.setState({ gradeId: val })} />
                ) : (
                  <div style={{ wordBreak: 'break-word' }}>{getDictTextByVal(gradeDict, gradeId, true)}</div>
                )}
              </FormItem>
              <FormItem {...formCfg} name="学校">
                {editing ? (
                  <Input readOnly value={schoolName} onClick={() => this.setState({ selectSchoolModalVis: true })} />
                ) : (
                  <div style={{ wordBreak: 'break-word' }}>{getFieldWithFiller(schoolName)}</div>
                )}
              </FormItem>
              <FormItem {...formCfg} name="客户姓名" required={editing}>
                {editing ? (
                  <Input
                    value={cusName}
                    onChange={(e) => this.setState({ cusName: e.target.value })}
                    validator={{
                      rules: ['required'],
                      silent: !checking,
                      onErrorChange: (e) => this.errorChange('cusName', e),
                    }}
                  />
                ) : (
                  <div style={{ wordBreak: 'break-word' }}>{getFieldWithFiller(cusName)}</div>
                )}
              </FormItem>
              <FormItem {...formCfg} name="客户电话">
                {0 ? (
                  <Input
                    value={contact}
                    onChange={(e) => this.setState({ contact: e.target.value })}
                    validator={{
                      rules: ['required', 'phone'],
                      silent: !checking,
                      onErrorChange: (e) => this.errorChange('contact', e),
                    }}
                  />
                ) : (
                  <div style={{ wordBreak: 'break-word' }}>{getFieldWithFiller(contact)}</div>
                )}
              </FormItem>
              <FormItem {...formCfg} name="资源来源">
                {0 ? (
                  <Select
                    dict={entranceDict}
                    value={resEntranceId}
                    onChange={(val) => this.setState({ resEntranceId: val })}
                  />
                ) : (
                  <div style={{ wordBreak: 'break-word' }}>{getFieldWithFiller(resEntranceName)}</div>
                )}
              </FormItem>
              <FormItem {...formCfg} name="资源所属学习中心">
                {0 ? (
                  <Select
                    dict={blCampusDict}
                    value={blCampusId}
                    onChange={(val) => this.setState({ blCampusId: val })}
                  />
                ) : (
                  <div style={{ wordBreak: 'break-word' }}>{getDictTextByVal(blCampusDict, blCampusId, true)}</div>
                )}
              </FormItem>
              <FormItem {...formCfg} name="意向学习中心">
                {editing ? (
                  <Select
                    dict={intentionCampusDict}
                    value={intentionCampusId}
                    onChange={(val) => this.setState({ intentionCampusId: val })}
                  />
                ) : (
                  <div style={{ wordBreak: 'break-word' }}>
                    {getDictTextByVal(intentionCampusDict, intentionCampusId, true)}
                  </div>
                )}
              </FormItem>
              <FormItem {...formCfg} name="分配人">
                {0 ? (
                  <Input
                    value={lastDeliverName}
                    onChange={(e) => this.setState({ lastDeliverName: e.target.value })}
                    validator={{
                      rules: ['required'],
                      silent: !checking,
                      onErrorChange: (e) => this.errorChange('lastDeliverName', e),
                    }}
                  />
                ) : (
                  <div style={{ wordBreak: 'break-word' }}>{getFieldWithFiller(lastDeliverName)}</div>
                )}
              </FormItem>
              <FormItem {...formCfg} name="跟进人">
                {0 ? (
                  <Input
                    value={deliverTargetName}
                    onChange={(e) => this.setState({ deliverTargetName: e.target.value })}
                  />
                ) : (
                  <div style={{ wordBreak: 'break-word' }}>{getFieldWithFiller(deliverTargetName)}</div>
                )}
              </FormItem>
              <FormItem {...formCfg} name="跟进状态">
                {0 ? (
                  <Select
                    dict={followStatusDict}
                    value={dealStatus}
                    onChange={(val) => this.setState({ dealStatus: val })}
                  />
                ) : (
                  <div style={{ wordBreak: 'break-word' }}>{getFieldWithFiller(dealStatusName)}</div>
                )}
              </FormItem>
              <FormItem {...formCfg} name="备注">
                {editing ? (
                  <TextArea
                    style={{ wordBreak: 'break-word' }}
                    rows={3}
                    value={remark}
                    onChange={(e) => this.setState({ remark: e.target.value })}
                  />
                ) : (
                  <div style={{ wordBreak: 'break-word' }}>{getFieldWithFiller(remark)}</div>
                )}
              </FormItem>
            </div>
          </Section>
        </Block>

        {selectSchoolModalVis && (
          <SelectSchoolModal
            onConfirm={(school, schoolName) =>
              this.setState({ selectSchoolModalVis: false, schoolId: school, schoolName })
            }
            onClose={() => this.setState({ selectSchoolModalVis: false })}
            defaultProvince={provinceId}
            defaultCity={cityId}
          />
        )}

        {mergeStudentInfoModalVisible && (
          <Modal
            width={514}
            onCancel={() => {
              this.setState({ mergeStudentInfoModalVisible: false });
              this.onMergeModalClose();
            }}
            okBtnShow={false}
            cancelText="关闭"
          >
            <div style={{ padding: '40px 0 5px 0' }}>{mergeStudentInfoMsg}</div>
          </Modal>
        )}
      </div>
    );
  }
}

export default CusDetailBlock;
