import React from 'react';
import styles from './styles.less';
import Sec from '../../component/Sec';
import Record from '../../component/Record';
import { FormItem, DatePicker, TimePicker, Button, TextArea, InputNumber, PopTip, Select } from 'COMPONENT';
import * as loaders from '../../util/loaders';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { getCommonErrorTips } from 'COMPONENT/utils/form';
import { loadCommonDict } from 'UTIL/dict';
import moment from 'moment';
import { DATE_FORMAT } from 'COMPONENT/utils/form';
import { Row, Col } from 'antd';
import { SS_EXAM_TYPE_DICT, SS_RECORD_DEAL_STATUS_DICT, CHINESE, ENGLISH, MATHEMATICS } from '../../util/dicts';

const formCfg = {
  labelWidth: 110,
};

/**
 * 添加跟进记录字段
 */
const initFollowInfo = {
  remark: null, // 跟进内容
  satisficing: null, // 意向度
  customerSign: null, // 客户意向产品
  dealStatus: null, // 跟进状态
  nextFollowTime: null, // 下次跟进日期
  appointmentDate: null, // 预约上门日期
  appointmentTime: null, // 预约上门时间
  examTime: null, // 考试时间
  examTypeId: null, // 考试类型
  scoreChinese: null, // 语文成绩
  scoreEnglish: null, // 英语成绩
  scoreMatch: null, // 数学成绩
};

function range(start, end) {
  const result = [];

  for (let i = start; i < end; i++) {
    result.push(i);
  }

  return result;
}

/**
 * 客户跟进记录tab
 */
class RecordsBlock extends React.Component {
  constructor(props) {
    super(props);

    const { followList, studentId } = this.props;
    this.isListFollowed = !!followList;
    this.isNextDisabled = false;
    if (this.isListFollowed) {
      const currentIndex = followList.findIndex((ele) => ele.studentId === studentId);
      this.isNextDisabled = followList.length - 1 === currentIndex;
      this.nextStudent = this.isNextDisabled ? undefined : followList[currentIndex + 1];
    }

    this.state = {
      recordListKey: Date.now(),
      editing: true, // 添加记录开启状态
      checking: false, // 校验开启状态

      ...initFollowInfo,

      intentionDict: [], // 意向度字典
      cusTagDict: [], // 客户标签字典(应该删除'其他'选项)

      saveTipsVis: false, // {boolean} 保存按钮提示气泡显示状态
      saveTips: null, // {string} 保存按钮提示信息
    };

    this.errors = {};
  }

  componentDidMount() {
    this.initContent();
  }

  initContent = () => {
    Promise.all([loadCommonDict('CUSTOMER_SATISFICING'), loadCommonDict('CustomerSignType')])
      .then(([intentionDict, cusTagDict]) => {
        const cusTagResult = cusTagDict.filter((item) => item.val !== 'CUSTOMER_SIGN_TYPE_OTHER');

        this.setState({ intentionDict, cusTagDict: cusTagResult });
      })
      .catch((e) => {
        throw e;
      });
  };

  /**
   * 重置
   */
  cancel = () => {
    this.error = {};
    this.setState({
      ...initFollowInfo,
      checking: false,
    });
  };

  /**
   * 保存跟进记录
   *
   * @param followNext
   */
  onOk = (followNext) => {
    const { onLoadingStart, onLoadingEnd } = this.props;
    const { checking, appointmentDate, appointmentTime } = this.state;

    if (!checking) {
      this.setState({ checking: true });
    }

    const errorTips = getCommonErrorTips(this.errors);
    if (errorTips) {
      this.showSaveTips(errorTips);
      return;
    }

    if (appointmentDate && !appointmentTime) {
      this.showSaveTips('填写了预约上门日期必须填写预约上门时间');
      return;
    }

    const params = this.formatSaveParam();

    onLoadingStart();
    loaders.saveFollowRecord(params).then(
      () => {
        onLoadingEnd();
        showMessage('保存成功', 'success');

        // 保存成功触发全局事件，跟进客户-跟进列表接收事件刷新列表
        GlobalEvent.emit('SSFollowCustomer:addFollowRecord');

        this.setState({
          checking: false,
          recordListKey: Date.now(),
          ...initFollowInfo,
        });
        this.error = {};

        if (followNext) {
          this.followNext();
        }
      },
      (e) => {
        if (e.text) {
          this.showSaveTips(e.text);
        } else {
          showMessage('保存失败');
        }
        onLoadingEnd();
      },
    );
  };

  errorChange = (key, error) => {
    this.errors[key] = error;
  };

  showSaveTips = (text) => {
    this.setState({ saveTipsVis: true, saveTips: text || '请正确填写信息' });
  };

  /**
   * 组装提交参数
   * 跟进内容 意向度 客户意向产品 跟进状态 下次跟进日期 预约上门时间 考试日期 考试类型 语文成绩 数学成绩 英语成绩
   * tips: 最后需写死一个 appointmentType: 'FOLLOW_UP'
   */
  formatSaveParam = () => {
    const { cusId, studentId } = this.props;
    const {
      remark,
      satisficing,
      customerSign,
      dealStatus,
      nextFollowTime,
      appointmentDate,
      appointmentTime,
      examTime,
      examTypeId,
      scoreChinese,
      scoreEnglish,
      scoreMatch,
    } = this.state;
    const meetingTime = appointmentDate && appointmentTime ? `${appointmentDate} ${appointmentTime}:00` : null;
    const examResultList = [
      { courseId: CHINESE, courseName: '语文', score: scoreChinese },
      { courseId: ENGLISH, courseName: '英语', score: scoreEnglish },
      { courseId: MATHEMATICS, courseName: '数学', score: scoreMatch },
    ];

    const detail = {
      remark,
      satisficing,
      customerSign,
      dealStatus,
      nextFollowTime,
      meetingTime,
      examTime,
      examTypeId,
      examResultList,
    };

    return {
      ...detail,
      customerId: cusId,
      followStuId: studentId,
      appointmentType: 'FOLLOW_UP',
    };
  };

  /**
   * 禁用今天之后的时间
   *
   * @param value
   */
  disableDateAfterToday = (value) => {
    if (!value) {
      return false;
    }

    return moment(value).valueOf() > moment(moment().format(DATE_FORMAT)).valueOf();
  };

  /**
   * 禁用今天之前的时间
   *
   * @param value
   * @return {boolean}
   */
  disableDateBeforeToday = (value) => {
    if (!value) {
      return false;
    }

    return moment(value).valueOf() < moment(moment().format(DATE_FORMAT)).valueOf();
  };

  disabledHours = () => {
    return range(0, 6).concat([24]);
  };

  disabledMinutes = () => {
    return range(0, 60).filter((ele) => ele % 15 !== 0);
  };

  /**
   * 跟进下一个学生
   * tips: 客户列表的跟进人ID字段为`deliverTarget`或`deliverTargetId`
   * tips: 'main/openPage'的mirrorId字段会影响/CustomerDetailEdit/index的参数取值
   */
  followNext = () => {
    if (this.nextStudent) {
      const { dispatch, pageParams } = this.props;
      const { cusId, studentId, deliverTargetId, deliverTarget } = this.nextStudent;
      const params = {
        ...pageParams,
        cusId: cusId,
        studentId: studentId,
        deliverTarget: deliverTargetId || deliverTarget,
      };

      dispatch({
        type: 'main/openPage',
        payload: {
          path: '/SSCustomerDetailEdit',
          params,
          refresh: true,
        },
      });
    }
  };

  render() {
    const { cusId, studentId, deliverTarget } = this.props;
    const {
      recordListKey,
      editing,
      checking,
      intentionDict,
      cusTagDict,
      saveTipsVis,
      saveTips,
      remark,
      satisficing,
      dealStatus,
      customerSign,
      nextFollowTime,
      appointmentDate,
      appointmentTime,
      examTime,
      examTypeId,
      scoreChinese,
      scoreEnglish,
      scoreMatch,
    } = this.state;
    const canEditSource = dealStatus === 'EXAM_PASS' || dealStatus === 'EXAM_FAIL';

    return (
      <div className={styles.wrap}>
        {editing && (
          <div className={styles.editBlock}>
            <Row>
              <Col span="24">
                <FormItem {...formCfg} name="跟进内容" required>
                  <TextArea
                    rows={3}
                    wordCount
                    maxLength={1000}
                    value={remark}
                    onChange={(e) => this.setState({ remark: e.target.value })}
                    validator={{
                      rules: ['required'],
                      silent: !checking,
                      onErrorChange: (e) => this.errorChange('remark', e),
                    }}
                  />
                </FormItem>
              </Col>
            </Row>
            <Row>
              <Col span="8">
                <FormItem {...formCfg} name="意向度" required>
                  <Select
                    dict={intentionDict}
                    value={satisficing}
                    onChange={(val) => this.setState({ satisficing: val })}
                    validator={{
                      rules: ['required'],
                      silent: !checking,
                      onErrorChange: (e) => this.errorChange('satisficing', e),
                    }}
                  />
                </FormItem>
              </Col>
              <Col span="8">
                <FormItem {...formCfg} name="跟进状态" required>
                  <Select
                    dict={SS_RECORD_DEAL_STATUS_DICT}
                    value={dealStatus}
                    onChange={(val) => this.setState({ dealStatus: val })}
                    validator={{
                      rules: ['required'],
                      silent: !checking,
                      onErrorChange: (e) => this.errorChange('dealStatus', e),
                    }}
                  />
                </FormItem>
              </Col>
              <Col span="8">
                <FormItem {...formCfg} name="客户意向产品">
                  <Select
                    dict={cusTagDict}
                    value={customerSign}
                    onChange={(val) => this.setState({ customerSign: val })}
                    validator={{
                      rules: [],
                      silent: !checking,
                      onErrorChange: (e) => this.errorChange('customerSign', e),
                    }}
                  />
                </FormItem>
              </Col>
            </Row>
            <Row>
              <Col span="8">
                <FormItem {...formCfg} name="下次跟进日期" required>
                  <DatePicker
                    value={nextFollowTime}
                    onChange={(val) => this.setState({ nextFollowTime: val })}
                    validator={{
                      rules: ['required'],
                      silent: !checking,
                      onErrorChange: (e) => this.errorChange('nextFollowTime', e),
                    }}
                    className={styles.pickerInput}
                    disabledDate={this.disableDateBeforeToday}
                    style={{ width: '100%' }}
                  />
                </FormItem>
              </Col>
              <Col span="8">
                <FormItem {...formCfg} name="预约上门日期">
                  <div style={{ width: '100%', display: 'flex' }}>
                    <DatePicker
                      value={appointmentDate}
                      onChange={(val) => this.setState({ appointmentDate: val })}
                      className={styles.pickerInput}
                      disabledDate={this.disableDateBeforeToday}
                      style={{ width: '48%' }}
                    />
                    <TimePicker
                      value={appointmentTime}
                      onChange={(val) => this.setState({ appointmentTime: val })}
                      className={styles.pickerInput}
                      disabledHours={this.disabledHours}
                      disabledMinutes={this.disabledMinutes}
                      hideDisabledOptions={true}
                      style={{ width: '48%', marginLeft: '4%' }}
                    />
                  </div>
                </FormItem>
              </Col>
              <Col span="8">
                <FormItem {...formCfg} name="考试日期">
                  <DatePicker
                    value={examTime}
                    onChange={(val) => this.setState({ examTime: val })}
                    validator={{
                      rules: [],
                      silent: !checking,
                      onErrorChange: (e) => this.errorChange('examTime', e),
                    }}
                    className={styles.pickerInput}
                    disabledDate={this.disableDateAfterToday}
                    disabled={!canEditSource}
                    style={{ width: '100%' }}
                  />
                </FormItem>
              </Col>
            </Row>
            <Row>
              <Col span="6">
                <FormItem {...formCfg} name="考试类型">
                  <Select
                    dict={SS_EXAM_TYPE_DICT}
                    value={examTypeId}
                    onChange={(val) => this.setState({ examTypeId: val })}
                    validator={{
                      rules: [],
                      silent: !checking,
                      onErrorChange: (e) => this.errorChange('examTypeId', e),
                    }}
                  />
                </FormItem>
              </Col>
              <Col span="6">
                <FormItem {...formCfg} name="语文成绩">
                  <InputNumber
                    min={0}
                    max={100}
                    value={scoreChinese}
                    onChange={(value) => this.setState({ scoreChinese: value })}
                    disabled={!canEditSource}
                  />
                </FormItem>
              </Col>
              <Col span="6">
                <FormItem {...formCfg} name="数学成绩">
                  <InputNumber
                    min={0}
                    max={100}
                    value={scoreEnglish}
                    onChange={(value) => this.setState({ scoreEnglish: value })}
                    disabled={!canEditSource}
                  />
                </FormItem>
              </Col>
              <Col span="6">
                <FormItem {...formCfg} name="英语成绩">
                  <InputNumber
                    min={0}
                    max={100}
                    value={scoreMatch}
                    onChange={(value) => this.setState({ scoreMatch: value })}
                    disabled={!canEditSource}
                  />
                </FormItem>
              </Col>
            </Row>

            <div style={{ textAlign: 'center', marginTop: 20 }}>
              <PopTip
                visible={saveTipsVis}
                content={saveTips}
                onClose={() => this.setState({ saveTipsVis: false })}
                getPopupContainer={(ele) => ele.parentNode}
              >
                <Button type="primary" onClick={() => this.onOk()}>
                  保存
                </Button>
              </PopTip>

              <Button onClick={this.cancel} style={{ marginLeft: 20 }}>
                重置
              </Button>

              {this.isListFollowed && (
                <Button
                  type="primary"
                  onClick={() => this.onOk(true)}
                  style={{ marginLeft: 20 }}
                  disabled={this.isNextDisabled}
                >
                  保存并跟进下一个学生
                </Button>
              )}
            </div>
          </div>
        )}

        <Sec title="跟进记录">
          <Record cusId={cusId} stuId={studentId} deliverTarget={deliverTarget} key={recordListKey} />
        </Sec>
      </div>
    );
  }
}

export default RecordsBlock;
