import React from 'react';
import { connect } from 'dva';
import Modal from 'COMPONENT/Modal';
import styles from '../CustomerDetailView/styles.less';
import CusDetailBlock from '../CustomerDetailView/CusDetailBlock';
import RecordsBlock from '../CustomerDetailView/RecordsBlock';
import CusNewsBlock from '../component/CusNewsBlock';
import { Tabs } from 'antd';
import sectionStyle from 'COMPONENT/Section/styles.less';
import modalStyle from './index.less';

const TabPane = Tabs.TabPane;

class CustomerDetailModal extends React.Component {
  static defaultProps = {
    oldPageParams: {}, // 旧boss页面弹窗传过来的参数
  };

  constructor(props) {
    super(props);

    this.state = {
      loading: false,
    };
  }

  loadingStart = () => {
    this.setState({ loading: true });
  };

  loadingEnd = () => {
    this.setState({ loading: false });
  };

  render() {
    const { onCloseTab, dispatch, list, user, oldPageParams } = this.props;
    const { loading } = this.state;
    const { campusId, cusId, studentId, deliverTarget } = oldPageParams || {};
    const isManager = false;

    return (
      <Modal
        title="客户详情"
        width={1200}
        loading={loading}
        okBtnShow={false}
        cancelBtnShow={false}
        closable={true}
        onClose={this.props.onClose}
        className={modalStyle.modalWrap}
      >
        <CusDetailBlock
          cusId={cusId}
          studentId={studentId}
          deliverTarget={deliverTarget}
          editable={false}
          ref={(c) => (this.cusDetailBlockRef = c)}
          onLoadingStart={this.loadingStart}
          onLoadingEnd={this.loadingEnd}
          onCloseTab={onCloseTab}
          dispatch={dispatch}
          user={user}
        />

        <div style={{ marginTop: 15 }}>
          <div className="Section" style={sectionStyle}>
            <div className="Section-body custom-tabs" style={{ paddingBottom: '20px' }}>
              <Tabs className={styles.customTabs} tabBarStyle={{ border: 'none', margin: '10px 0 15px 0' }}>
                <TabPane key="records" tab="跟进记录">
                  <RecordsBlock
                    cusId={cusId}
                    stuId={studentId}
                    deliverTarget={deliverTarget}
                    ref={(c) => (this.recordsBlockRef = c)}
                    onLoadingStart={this.loadingStart}
                    onLoadingEnd={this.loadingEnd}
                    isManager={isManager}
                    followList={list}
                    onCloseTab={onCloseTab}
                    dispatch={dispatch}
                  />
                </TabPane>
                <TabPane key="cusNews" tab="客户动态">
                  <CusNewsBlock
                    cusId={cusId}
                    stuId={studentId}
                    campusId={campusId}
                    deliverTarget={deliverTarget}
                    ref={(c) => (this.cusNewsBlockRef = c)}
                    isManager={isManager}
                  />
                </TabPane>
              </Tabs>
            </div>
          </div>
        </div>
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    user: state.account.user,
  };
};

export default connect(mapStateToProps)(CustomerDetailModal);
