import React from 'react';
import Page from 'COMPONENT/Page';
import styles from './styles.less';
import CusDetailBlock from './CusDetailBlock';
import RecordsBlock from './RecordsBlock';
import CusNewsBlock from '../component/CusNewsBlock';
import { Tabs } from 'antd';
import sectionStyle from 'COMPONENT/Section/styles.less';

const TabPane = Tabs.TabPane;

class CusDetailPageMain extends React.Component {
  static defaultProps = {
    cusId: '', // 客户ID
  };

  constructor(props) {
    super(props);

    this.state = {
      loading: false,
    };
  }

  loadingStart = () => {
    this.setState({ loading: true });
  };

  loadingEnd = () => {
    this.setState({ loading: false });
  };

  render() {
    const { cusId, studentId, deliverTarget, isManager, campusId, onCloseTab, dispatch, list, user } = this.props;
    const { loading } = this.state;

    return (
      <Page route={this.props.route} title={false} className={styles.page} loading={loading}>
        <CusDetailBlock
          cusId={cusId}
          studentId={studentId}
          deliverTarget={deliverTarget}
          ref={(c) => (this.cusDetailBlockRef = c)}
          onLoadingStart={this.loadingStart}
          onLoadingEnd={this.loadingEnd}
          onCloseTab={onCloseTab}
          dispatch={dispatch}
          user={user}
        />

        <div style={{ marginTop: 15 }}>
          <div className="Section" style={sectionStyle}>
            <div className="Section-body custom-tabs" style={{ paddingBottom: '20px' }}>
              <Tabs className={styles.customTabs} tabBarStyle={{ border: 'none', margin: '10px 0 15px 0' }}>
                <TabPane key="records" tab="跟进记录">
                  <RecordsBlock
                    cusId={cusId}
                    stuId={studentId}
                    deliverTarget={deliverTarget}
                    ref={(c) => (this.recordsBlockRef = c)}
                    onLoadingStart={this.loadingStart}
                    onLoadingEnd={this.loadingEnd}
                    isManager={isManager}
                    followList={list}
                    onCloseTab={onCloseTab}
                    dispatch={dispatch}
                  />
                </TabPane>
                <TabPane key="cusNews" tab="客户动态">
                  <CusNewsBlock
                    cusId={cusId}
                    stuId={studentId}
                    campusId={campusId}
                    deliverTarget={deliverTarget}
                    ref={(c) => (this.cusNewsBlockRef = c)}
                    isManager={isManager}
                  />
                </TabPane>
              </Tabs>
            </div>
          </div>
        </div>
      </Page>
    );
  }
}

export default CusDetailPageMain;
