import React from 'react';
import styles from './styles.less';
import Sec from '../../component/Sec';
import * as loaders from '../../util/loaders';
import List from 'COMPONENT/List';
import { loadCommonDict } from 'UTIL/dict';

class RecordsBlockManage extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      list: [], // 我的跟进记录列表，
      intentionDict: [], // 意向度字典
    };

    this.errors = {};
  }

  initContent = async () => {
    const { cusId, stuId, campusId } = this.props;
    const [recordsLists, intentionDict] = await Promise.all([
      loaders.loadRecordsManage(campusId, cusId, stuId),
      loadCommonDict('CUSTOMER_SATISFICING'),
    ]).catch(() => {
      this.props.onLoadingEnd();
    });
    const { list } = recordsLists;

    this.setState({ list, intentionDict });
  };

  render() {
    const { list, intentionDict } = this.state;

    const columns = [
      { title: '意向度', dataIndex: 'intention', width: 70, dict: intentionDict },
      { title: '跟进时间', dataIndex: 'time', width: 150 },
      { title: '客户意向产品', dataIndex: 'tag', width: 150 },
      { title: '下次跟进日期', dataIndex: 'nextFollowTime', width: 150 },
      { title: '预约上门时间', dataIndex: 'meetingTime', width: 150 },
      { title: '跟进内容', dataIndex: 'content', width: 300 },
    ];

    return (
      <div className={styles.wrap}>
        {list.map((ele) => {
          return (
            <Sec key={ele.consultantName} title={`${ele.consultantName ? ele.consultantName + '的' : ''}跟进记录`}>
              <List dataSource={ele.followupRecords} columns={columns} />
            </Sec>
          );
        })}
      </div>
    );
  }
}

export default RecordsBlockManage;
