import React from 'react';
import styles from './styles.less';
import { List } from 'COMPONENT';
import { SS_EXAM_TYPE_DICT, CHINESE, ENGLISH, MATHEMATICS } from '../../util/dicts';
import * as loaders from '../../util/loaders';
import { formatScore } from './util';

/**
 * 跟进记录列表
 */
class Record extends React.Component {
  static defaultProps = {
    cusId: '', // 客户ID
    stuId: '', // 学生ID
    deliverTarget: '', // 跟进人
  };

  constructor(props) {
    super(props);

    this.state = {
      listLoading: false,
      list: [], // 列表数据
      totalRecords: 0, // 总记录数
      pageNo: 1, // 当前页码
      pageSize: 20, // 每页记录数
    };
  }

  componentDidMount() {
    this.initContent();
  }

  initContent = () => {
    this.loadList({ pageNo: 1 });
  };

  loadList = (changedParams = {}) => {
    const { pageNo, pageSize } = this.state;
    const pageParam = { pageNo, pageSize, ...changedParams };
    const params = this.formatParams(pageParam);

    this.setState({ listLoading: true });

    loaders
      .loadRecords(params)
      .then((response) => {
        const result = this.formatResponse(response);

        this.setState({
          listLoading: false,
          list: result.data,
          totalRecords: result.rowCount,
          pageNo: params.pageNo,
          pageSize: params.pageSize,
        });
      })
      .catch(() => {
        const result = this.formatResponse({});

        this.setState({
          listLoading: false,
          list: result.data,
          totalRecords: result.rowCount,
          pageNo: params.pageNo,
          pageSize: params.pageSize,
        });
      });
  };

  formatParams = (pageParam) => {
    const { pageNo, pageSize } = pageParam;
    const { cusId, stuId } = this.props;

    return {
      pageNo,
      pageSize,

      page: pageNo,
      rows: pageSize,
      customerId: cusId,
      studentId: stuId,
    };
  };

  formatResponse = (response = {}) => {
    const data = response.rows || [];
    const rowCount = response.records || 0;

    return {
      data: data,
      rowCount: rowCount,
    };
  };

  render() {
    const { list, listLoading, pageNo, totalRecords, pageSize } = this.state;

    const columns = [
      { title: '跟进人', dataIndex: 'createUserName', key: 'createUserName', width: 80 },
      { title: '跟进时间', dataIndex: 'createTime', key: 'createTime', width: 140 },
      { title: '跟进内容', dataIndex: 'remark', key: 'remark', width: 450, lines: 2 },
      { title: '跟进状态', dataIndex: 'dealStatusName', key: 'dealStatusName', width: 100 },
      { title: '意向度', dataIndex: 'intentionOfTheCustomerName', key: 'intentionOfTheCustomerName', width: 70 },
      { title: '客户意向产品', dataIndex: 'customerSignName', key: 'customerSignName', width: 120 },
      { title: '预约上门时间', dataIndex: 'meetingTime', key: 'meetingTime', width: 160 },
      { title: '下次跟进日期', dataIndex: 'nextFollowTime', key: 'nextFollowTime', width: 140 },
      { title: '考试日期', dataIndex: 'examTime', key: 'examTime', width: 120 },
      { title: '考试类型', dataIndex: 'examTypeId', key: 'examTypeId', width: 80, dict: SS_EXAM_TYPE_DICT },
      {
        title: '语文成绩',
        dataIndex: 'CHINESE',
        width: 80,
        render: (text, record) => {
          return formatScore(record.examResultList, CHINESE);
        },
      },
      {
        title: '数学成绩',
        dataIndex: 'ENGLISH',
        width: 80,
        render: (text, record) => {
          return formatScore(record.examResultList, ENGLISH);
        },
      },
      {
        title: '英语成绩',
        dataIndex: 'MATHEMATICS',
        width: 80,
        render: (text, record) => {
          return formatScore(record.examResultList, MATHEMATICS);
        },
      },
    ];

    return (
      <div className={styles.listWrap} style={{ height: 400 }}>
        <List
          dataSource={list}
          columns={columns}
          loading={listLoading}
          pagination={{
            current: pageNo || 1,
            total: totalRecords || 0,
            pageSize: pageSize || 20,
          }}
          onChange={({ current, pageSize }) => this.loadList({ pageNo: current, pageSize })}
          className={styles.cusList}
        />
      </div>
    );
  }
}

export default Record;
