import React from 'react';
import ReactDOM from 'react-dom';
import { connect } from 'dva';
import classnames from 'classnames';
import styles from './styles.less';
import utils from '../utils';
import MoreInfoPop from '../MoreInfoPop';
import { visitStatusColors } from '../../utils';
import { TextLink } from 'COMPONENT';
import { formatEditPageParam } from './util';

class Card extends React.Component {
  state = {
    mode: null,
  };

  componentDidMount() {
    this.$wrap = ReactDOM.findDOMNode(this.wrapRef);
    this.updateMode();
    window.addEventListener('resize', this.updateMode, false);
  }

  componentWillReceiveProps(nextProps) {
    if (nextProps.expandDate !== this.expandDate) {
      if (nextProps.expand && !this.props.expand) {
        this.setState({ mode: 'default' });
      } else {
        setTimeout(() => {
          this.updateMode();
        }, 500);
      }
    }
  }

  componentWillUnmount() {
    window.removeEventListener('resize', this.updateMode, false);
  }

  updateMode = () => {
    const w = this.$wrap.offsetWidth;
    this.setState({ mode: w < 0 ? 'small' : 'default' });
  };

  /**
   * tips：注意点击卡片的参数
   *
   * @param lines
   * @return {*}
   */
  getContent = (lines) => {
    const {
      data: { infos, cusId, stuId, createUserId, createUserName, name },
      onCardClick,
    } = this.props;
    const cls = {
      [styles.content]: 1,
      [styles.contentDefault]: 1,
    };
    const editParam = formatEditPageParam({ cusId, stuId, createUserId });
    const onClick = onCardClick !== undefined ? onCardClick : () => this.goDetail(editParam);

    return (
      <div className={classnames(cls)}>
        <div className={styles.headerInfo}>{name}</div>
        {lines >= 2 && (
          <div className={styles.title}>
            {onClick ? <TextLink onClick={onClick}>{createUserName}</TextLink> : <span>{createUserName}</span>}
          </div>
        )}
        {lines >= 3 && (
          <div className={styles.infos}>
            {infos.slice(0, lines - 2).map((info, idx) => {
              return (
                <div key={idx} className={styles.info}>
                  {info}
                </div>
              );
            })}
          </div>
        )}
      </div>
    );
  };

  getContentSmall = (lines) => {
    const {
      data: { title, createUserName },
    } = this.props;
    const cls = {
      [styles.content]: 1,
      [styles.contentSmall]: 1,
    };
    const titleCls = {
      [styles.title]: 1,
      [styles[`titleLines${lines - 1}`]]: 1,
    };

    return (
      <div className={classnames(cls)}>
        <div className={classnames(titleCls)}>{createUserName}</div>
      </div>
    );
  };

  onRefresh = (params) => {
    const { onRefresh } = this.props;
    onRefresh();
    this.goDetail(params);
  };

  goDetail = (params) => {
    const { dispatch } = this.props;

    params = {
      ...params,
      onRefresh: this.onRefresh,
    };

    dispatch({
      type: 'main/openPage',
      payload: {
        path: '/SSCustomerDetailEdit',
        params,
        refresh: true,
      },
    });
  };

  render() {
    const {
      data: { startTime, endTime, status, moreInfos, cusId, stuId, createUserId },
      nodeStartTime,
      onActiveTimeChange,
    } = this.props;
    const { mode } = this.state;
    const colors = visitStatusColors[status];
    const top = utils.getTimePx(nodeStartTime, startTime);
    const height = utils.getTimePx(startTime, endTime);
    const lines = Math.floor((height - 6) / 19);
    const editParam = formatEditPageParam({ cusId, stuId, createUserId });

    return (
      <MoreInfoPop
        data={moreInfos}
        onClick={(e) => {
          this.goDetail(editParam);
        }}
      >
        <div
          className={styles.wrap}
          style={{ borderColor: colors.primary, top, height }}
          onMouseEnter={() => onActiveTimeChange({ start: startTime, end: endTime })}
          onMouseLeave={() => onActiveTimeChange(null)}
          ref={(c) => (this.wrapRef = c)}
        >
          {mode ? (mode === 'small' ? this.getContentSmall(lines) : this.getContent(lines)) : null}
        </div>
      </MoreInfoPop>
    );
  }
}

export default connect()(Card);
