import React from 'react';
import classnames from 'classnames';
import styles from './styles.less';
import utils from '../utils';
import Card from '../Card';
import Icon from 'COMPONENT/Icon';
import { objectDeepEquals } from 'COMPONENT/utils/basic/obj';

class Node extends React.Component {
  shouldComponentUpdate(nextProps, nextState) {
    const diff = Object.keys(this.props).find(
      (key) => typeof this.props[key] !== 'function' && !objectDeepEquals(this.props[key], nextProps[key]),
    );

    if (diff) {
      return true;
    }

    return false;
  }

  render() {
    const {
      data: { isOverDay, startTime, endTime, cardList },
      maxCardCount,
      expand,
      onExpandChange,
      onActiveTimeChange,
      expandDate,
      $panel,
      onCardClick,
      onRefresh,
      isManager,
    } = this.props;
    const top = utils.getTimePx(utils.panelStartTime, startTime);
    const height = utils.getTimePx(startTime, endTime) + 6;
    const hasMore = cardList.length > maxCardCount;
    const showCount = hasMore ? maxCardCount - 1 : cardList.length;
    const cardWidthPercent = (1 / showCount) * 100 + '%';
    const moreBtnCls = {
      [styles.moreBtn]: 1,
      [styles.moreBtnHidden]: !hasMore && !expand,
    };

    return (
      <div className={styles.wrap} style={{ top, height }}>
        <div className={styles.cardWrap}>
          {cardList.map((card, idx) => {
            const cls = {
              [styles.card]: 1,
              [styles.cardHidden]: idx > showCount - 1,
            };

            return (
              <div key={idx} className={classnames(cls)} style={{ width: cardWidthPercent }}>
                <Card
                  data={card}
                  nodeStartTime={startTime}
                  onActiveTimeChange={onActiveTimeChange}
                  expandDate={expandDate}
                  expand={expand}
                  $panel={$panel}
                  onCardClick={onCardClick}
                  onRefresh={onRefresh}
                  isManager={isManager}
                />
              </div>
            );
          })}
        </div>
        <div className={classnames(moreBtnCls)} onClick={() => onExpandChange(!expand)}>
          <div className={styles.moreBtnInner}>
            <Icon type={expand ? 'left' : 'right'} />
          </div>
        </div>
      </div>
    );
  }
}

export default Node;
