import React from 'react';
import ReactDOM from 'react-dom';
import classnames from 'classnames';
import styles from './styles.less';
import utils from '../utils';
import moment from 'moment';
import Node from '../Node';
import { TIME_FORMAT } from 'COMPONENT/utils/form';

class SchedulePanel extends React.Component {
  state = {
    $panel: null,
  };

  componentDidMount() {
    this.setState({ $panel: ReactDOM.findDOMNode(this.wrapRef) });
  }

  render() {
    const {
      cols,
      currentTime,
      activeDate,
      activeStartTime,
      activeEndTime,
      data: { name, cardList },
      winW,
      expandDate,
      onExpandChange,
      onColActive,
      onActiveTimeChange,
      $scroller,
      onCardClick,
      enableCardAction,
      onRefresh,
      isManager,
    } = this.props;
    const { $panel } = this.state;
    const panelTimes = utils.getPanelTimes();
    const currentTimeLineTop = utils.getTimePx(utils.panelStartTime, currentTime) + 17;
    const currentHour = currentTime.slice(0, 2);
    const currentTimeMoment = moment(currentTime, TIME_FORMAT);
    const diffPrev = moment(currentHour, 'HH').diff(currentTimeMoment, 'minutes');
    const diffNext = moment(currentHour, 'HH').add(1, 'hours').diff(currentTimeMoment, 'minutes');
    const showCurrentTime = -diffPrev > 10 && diffNext > 10;

    return (
      <div className={styles.wrap} ref={(c) => (this.wrapRef = c)}>
        <div className={styles.currentTimeLine} style={{ top: currentTimeLineTop }} />
        {showCurrentTime && (
          <div className={styles.currentTime} style={{ top: currentTimeLineTop - 10 }}>
            {currentTime}
          </div>
        )}
        {activeStartTime && (
          <div
            className={styles.activeTimeLine}
            style={{ top: utils.getTimePx(utils.panelStartTime, activeStartTime) + 17 }}
          />
        )}
        {activeEndTime && (
          <div
            className={styles.activeTimeLine}
            style={{ top: utils.getTimePx(utils.panelStartTime, activeEndTime) + 17 }}
          />
        )}
        <div className={styles.left}>
          <div className={styles.name}></div>
          <div className={styles.timeLine}>
            {panelTimes.map((time, i) => {
              return (
                <div
                  key={time}
                  className={styles.time}
                  style={{ height: utils.minuteH * (i === panelTimes.length - 1 ? 40 : 60) }}
                >
                  {time}
                </div>
              );
            })}
          </div>
        </div>
        <div className={styles.mainWrap}>
          <div className={styles.main}>
            {cols.map((col) => {
              const cls = {
                [styles.col]: 1,
                [styles.active]: activeDate === col.date,
              };
              const colCardList = cardList.filter((card) => card.date === col.date);
              const nodeList = utils.getNodeList(colCardList);

              return (
                <div
                  key={col.date}
                  className={classnames(cls)}
                  style={{ flex: col.width }}
                  onMouseEnter={() => onColActive(col.date, true)}
                  onMouseLeave={() => onColActive(col.date, false)}
                  ref={(c) => (this.wrapRef = c)}
                >
                  <div className={styles.colInner}>
                    {nodeList.map((node, idx) => {
                      const nodeSize = expandDate ? (expandDate === col.date ? 'large' : 'small') : 'normal';
                      const maxCardCount = utils.getNodeMaxCardCount(winW, nodeSize);

                      return (
                        <Node
                          key={idx}
                          data={node}
                          maxCardCount={maxCardCount}
                          expand={nodeSize === 'large'}
                          onExpandChange={(expand) => onExpandChange(col.date, expand)}
                          onActiveTimeChange={onActiveTimeChange}
                          expandDate={expandDate}
                          $panel={$panel}
                          onCardClick={onCardClick}
                          onRefresh={onRefresh}
                          isManager={isManager}
                        />
                      );
                    })}
                  </div>
                </div>
              );
            })}
          </div>
        </div>
      </div>
    );
  }
}

export default SchedulePanel;
