import moment from 'moment';
import { DATE_FORMAT, TIME_FORMAT } from 'COMPONENT/utils/form';

const getWeekByDate = (date) => {
  const day = moment(date, DATE_FORMAT).day();
  return day === 0 ? 6 : day - 1;
};

const weekNames = ['周一', '周二', '周三', '周四', '周五', '周六', '周日'];

const minuteH = 0.8;

const panelStartTime = '06:00';
const panelEndTime = '23:00';

const isValidTime = (time) => {
  const timeValue = moment(time, TIME_FORMAT).valueOf();
  const panelStartTimeValue = moment(panelStartTime, TIME_FORMAT).valueOf();
  const panelEndTimeValue = moment(panelEndTime, TIME_FORMAT).valueOf();
  return panelStartTimeValue <= timeValue && timeValue < panelEndTimeValue;
};

const getPanelTimes = () => {
  return [
    '06:00',
    '07:00',
    '08:00',
    '09:00',
    '10:00',
    '11:00',
    '12:00',
    '13:00',
    '14:00',
    '15:00',
    '16:00',
    '17:00',
    '18:00',
    '19:00',
    '20:00',
    '21:00',
    '22:00',
    '23:00',
  ];
};

const isConflict = (start1, end1, start2, end2) => {
  return (
    (moment(start1, TIME_FORMAT).valueOf() < moment(end2, TIME_FORMAT).valueOf() &&
      moment(end1, TIME_FORMAT).valueOf() > moment(start2, TIME_FORMAT).valueOf()) ||
    (moment(start2, TIME_FORMAT).valueOf() < moment(end1, TIME_FORMAT).valueOf() &&
      moment(end2, TIME_FORMAT).valueOf() > moment(start1, TIME_FORMAT).valueOf())
  );
};

const getNodeList = (cardList) => {
  let nodeList = [];
  cardList.forEach((card) => {
    const { startTime, endTime } = card;
    let conflictNodes = nodeList.filter((node) => node && isConflict(startTime, endTime, node.startTime, node.endTime));

    if (conflictNodes.length === 0) {
      nodeList.push({ startTime, endTime, cardList: [card] });
    } else {
      let newStartTime = startTime;
      let newEndTime = endTime;
      let newCardList = [];

      conflictNodes.forEach((node) => {
        newStartTime = moment(node.startTime, TIME_FORMAT).isBefore(moment(newStartTime, TIME_FORMAT))
          ? node.startTime
          : newStartTime;
        newEndTime = moment(node.endTime, TIME_FORMAT).isAfter(moment(newEndTime, TIME_FORMAT))
          ? node.endTime
          : newEndTime;
        newCardList = [...newCardList, ...node.cardList];
        node['delete'] = true;
      });
      nodeList = nodeList.filter((node) => node.delete !== true);
      nodeList.push({ startTime: newStartTime, endTime: newEndTime, cardList: [...newCardList, card] });
    }
  });

  return nodeList;
};

const getTimePx = (start, end) => {
  const minutes = moment(end, TIME_FORMAT).diff(moment(start, TIME_FORMAT), 'minutes');

  return minutes * minuteH;
};

const nodeMaxCardCountMap = [
  { winSize: 'small', nodeSize: 'small', count: 2 },
  { winSize: 'small', nodeSize: 'normal', count: 3 },
  { winSize: 'small', nodeSize: 'large', count: 9999 },
  { winSize: 'normal', nodeSize: 'small', count: 3 },
  { winSize: 'normal', nodeSize: 'normal', count: 3 },
  { winSize: 'normal', nodeSize: 'large', count: 9999 },
];

const getWinSize = (winW) => {
  return winW < 1920 ? 'small' : 'normal';
};

const getNodeMaxCardCount = (winW, nodeSize) => {
  return nodeMaxCardCountMap.find((m) => m.winSize === getWinSize(winW) && m.nodeSize === nodeSize).count;
};

const panelsMaxCount = 2;

const ownerTypeNames = {
  teacher: '老师',
  student: '学生',
};

const addCusFormCfg = {
  labelWidth: 80,
  style: { padding: '0 25px' },
};

const addCusModalRowStyle = {
  marginRight: -22,
};

export default {
  getWeekByDate,
  weekNames,
  minuteH,
  getPanelTimes,
  panelStartTime,
  panelEndTime,
  getTimePx,
  getNodeList,
  nodeMaxCardCountMap,
  getNodeMaxCardCount,
  panelsMaxCount,
  ownerTypeNames,
  addCusFormCfg,
  addCusModalRowStyle,
  isValidTime,
};
