import {
  findCustomerStudentByCusStuId,
  findCustomerDynamicStatusByCustomerStudentId,
  updateCustomerStudent,
  saveConsultorFollowRecord,
  getStudentFollowupRecordsInCampus,
} from 'SERVICE/consultant';
import { queryCustomerStudentFollowingList } from 'SERVICE/ssCRM';

/**
 * 获取客户详情
 * @param cusId
 * @param studentId
 * @param deliverTarget
 */
export const loadCusDetail = async (cusId, studentId, deliverTarget) => {
  const response = await findCustomerStudentByCusStuId({ customerId: cusId, studentId, deliverTarget });
  const { data } = response || {};

  return data;
};

/**
 * 获取跟进记录
 */
export const loadRecords = async (params) => {
  return await queryCustomerStudentFollowingList(params);
};

export const loadRecordsManage = async (campusId, customerId, studentId) => {
  const res = await getStudentFollowupRecordsInCampus({
    campusId,
    customerId,
    studentId,
  });

  const list = res.data.map((ele) => {
    const consultantName = ele.consultantName;
    return {
      consultantName,
      followupRecords: (ele.followupRecords.length ? ele.followupRecords : []).map((d) => {
        return {
          time: !!d.createTime && d.createTime,
          consultantName: consultantName,
          intention: d.intentionOfTheCustomerId,
          tag: d.customerSignName,
          content: d.remark,
          deliverTargetName: d.createUserName,
          nextFollowTime: d.nextFollowTime,
          meetingTime: d.meetingTime,
        };
      }),
    };
  });
  return { list };
};

/**
 * 获取客户动态
 *
 * @param customerId
 * @param studentId
 * @param deliverTarget
 * @return {Promise<*>}
 */
export const loadCusNews = async (customerId, studentId, deliverTarget) => {
  const list = await findCustomerDynamicStatusByCustomerStudentId({
    customerId,
    studentId,
    deliverTarget,
  });

  return list.map((d) => {
    return { time: d.occourTime, content: d.description };
  });
};

/**
 * 保存客户详情
 * @param param
 */
export const saveCusDetail = async (param) => {
  const { cusId, cusName, studentId, studentName, gradeId, schoolId, intentionCampusId, remark } = param;
  const params = {
    cusId,
    cusName,
    studentId,
    studentName,
    gradeId,
    schoolId,
    intentionCampusId,
    remark,
  };

  return await updateCustomerStudent(params);
};

/**
 * 保存跟进记录
 */
export const saveFollowRecord = async (params) => {
  await saveConsultorFollowRecord(params);
  return true;
};
