import React from 'react';
import { connect } from 'dva';
import styles from './styles.less';
import { DatePicker, Modal as AntdModal } from 'antd';
import { Loading, Button, List, SearchBar, Input, Select, TextLink, Popover } from 'COMPONENT';
import { loadCampusDict, loadCommonDictByGroup } from 'UTIL/dict';
import loaders from '../loaders';
import dicts from '../dicts';
import { SS_DEAL_STATUS_DICT, SS_IS_APPOINT_COME_DICT } from '../../../common/util/dicts';
import {
  filterUnFetchListBySS,
  calcIsSigned,
  calcCanDoSign,
  formatDuringTime,
  formatPhoneNumber,
  splitDateString,
  formatDeliverTarget,
} from '../utils';
import { dateToMoment } from 'COMPONENT/utils/form';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import classNames from 'classnames';

const RangePicker = DatePicker.RangePicker;
const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;

/**
 * 跟进客户tab
 */
class CusList extends React.Component {
  constructor(props) {
    super(props);

    this.initTodoSKey = this.initTodoSKey.bind(this);
    this.onResolveTodo = this.onResolveTodo.bind(this);
    this.formatSearchParams = this.formatSearchParams.bind(this);
    this.showEmployeeTip = this.showEmployeeTip.bind(this);

    const { campusId } = this.props;

    this.initSearchParam = {
      studentName: null,
      contact: null,
      gradeId: null,
      resEntranceId: null,
      isAppointCome: null,
      deliverTarget: null,
      dealStatus: null,
      intentionOfTheCustomerId: null,
      intentionCampusId: campusId,
      schoolName: null,
      customerName: null,
      tempTimeType: '1',
      tempTime: null,
    };

    this.state = Object.assign(
      {
        loading: false,
        searchExpand: false, // 搜索栏展开状态

        studentName: null, // 查询条件 学生姓名
        contact: null, // 查询条件 联系方式
        gradeId: null, // 查询条件 年级
        resEntranceId: null, // 查询条件 资源入口/资源来源
        isAppointCome: null, // 查询条件 是否诺到/是否上门
        deliverTarget: null, // 查询条件 跟进人
        dealStatus: null, // 查询条件 跟进状态
        intentionOfTheCustomerId: null, // 查询条件 意向度
        intentionCampusId: '', // 查询条件 意向学习中心
        schoolName: null, // 查询条件 学校
        customerName: null, // 查询条件 客户姓名

        tempTimeType: '1', // 临时查询条件 时间类型
        tempTime: null, // 临时查询条件 时间(根据时间类型设置不同的起止时间)

        blCampusDict: [], // 字典 学校
        gradeDict: [], // 字典 年级
        entranceDict: [], // 字典 资源入口
        intentionDict: [], // 字典 意向度
        cusTagDict: [], // 字典 客户标签
        deliverTargetDict: [], // 字典 跟进人

        list: [], // 列表数据
        filteredList: [], // 未跟进列表？
        totalRecords: 0, // 总记录数
        currentPage: 1, // 当前页码
        pageSize: 20, // 每页记录数
        pageOrder: 'DESC', // 页面排序
        pageOrderField: 'create_time', // 排序字段

        currentRecord: null, // 当前选中行(暂时保留)
      },
      this.props.defaultSearchConditions || {},
    );
  }

  /**
   * 创建全局事件：
   *  global.GlobalEmitter.create('SSFollowCustomer').trigger('addFollowRecord');
   *  GlobalEvent.emit('SSFollowCustomer:addFollowRecord');
   * 监听全局事件：
   *  global.GlobalEmitter.create('SSFollowCustomer').listen('addFollowRecord', this.refreshCurrentPage);
   *  GlobalEvent.on('SSFollowCustomer:addFollowRecord', this.refreshCurrentPage);
   * 移除监听事件：
   *  global.GlobalEmitter.create('SSFollowCustomer').remove('addFollowRecord', this.refreshCurrentPage);
   *  GlobalEvent.off('SSFollowCustomer:addFollowRecord', this.refreshCurrentPage);
   */
  componentDidMount() {
    this.initTodoSKey();
    GlobalEvent.on('CrmNotification:resolveFollowCustomer', this.onResolveTodo);
    GlobalEvent.on('SSFollowCustomer:addFollowRecord', this.refreshCurrentPage);
  }

  componentWillUnmount() {
    GlobalEvent.off('CrmNotification:resolveFollowCustomer', this.onResolveTodo);
    GlobalEvent.off('SSFollowCustomer:addFollowRecord', this.refreshCurrentPage);
  }

  componentWillReceiveProps(nextProps) {
    const oldDefaultSearchConditions = this.props.defaultSearchConditions || {};
    const newDefaultSearchConditions = nextProps.defaultSearchConditions || {};
    // 不能直接比较两个对象，因此补充一个判断条件
    const hadDefaultSearchConditions =
      Object.keys(oldDefaultSearchConditions).length > 0 || Object.keys(newDefaultSearchConditions).length > 0;

    if (hadDefaultSearchConditions && oldDefaultSearchConditions !== newDefaultSearchConditions) {
      this.reset();
      Promise.resolve(this.setState(newDefaultSearchConditions));
      setTimeout(() => {
        this.search();
      }, 50);
    }
  }

  /**
   * 从系统通知跳转过来的，初始化筛选项值
   */
  initTodoSKey() {
    const { isTodo, todoParams } = this.props.params || {};

    if (isTodo) {
      this.setState({ ...todoParams });
    }
  }

  onResolveTodo(todoParams) {
    this.setState(
      {
        ...this.initSearchParam,
        ...todoParams,
      },
      () => {
        this.search();
      },
    );
  }

  async initialBranch() {
    let { branchId, campusId, defaultSearchConditions } = this.props;

    if (defaultSearchConditions) {
      branchId = defaultSearchConditions.branchId || branchId;
      campusId = defaultSearchConditions.campusId || campusId;
    }

    const blCampusDict = await loadCampusDict(branchId);

    this.setState(
      Object.assign(
        {
          intentionCampusId: campusId,
          blCampusDict,
        },
        defaultSearchConditions || {},
      ),
    );
  }

  initContent = async () => {
    await this.initialBranch();

    const { userId, userOrganizationId } = this.props;
    const searchParam = Object.assign(this.formatSearchParams(), this.props.defaultSearchConditions || {});
    const category = 'STUDENT_GRADE,RES_ENTRANCE,CUSTOMER_SATISFICING,CustomerSignType';
    const [listData, groupDict, deliverTargetDict] = await Promise.all([
      this.loadCusList(searchParam),
      loadCommonDictByGroup(category),
      loaders.loadDeliverTargetDict({ roleCode: 'CONSULTOR', organizationId: userOrganizationId }),
    ]);
    const { list, totalRecords } = listData;
    const filteredList = filterUnFetchListBySS(list, userId);

    this.setState({
      gradeDict: groupDict['STUDENT_GRADE'],
      entranceDict: groupDict['RES_ENTRANCE'],
      intentionDict: groupDict['CUSTOMER_SATISFICING'],
      cusTagDict: groupDict['CustomerSignType'],
      deliverTargetDict,
      list,
      totalRecords,
      filteredList,
    });
  };

  search = () => {
    this.setState({ loading: true });
    let params = this.formatSearchParams();
    this.loadCusList({ ...params, currentPage: 1 })
      .then(({ list, totalRecords, page }) => {
        const { userId } = this.props;
        const filteredList = filterUnFetchListBySS(list, userId);
        this.setState({ loading: false, currentPage: 1, list, totalRecords, filteredList });
      })
      .catch((e) => {
        this.setState({ loading: false, currentPage: 1, list: [], totalRecords: 0, filteredList: [] });
      });
  };

  reset = () => {
    this.setState(this.initSearchParam);
  };

  /**
   * 刷新
   */
  refreshCurrentPage = () => {
    const params = this.formatSearchParams();

    this.setState({ loading: true });

    this.loadCusList(params)
      .then((response) => {
        const { list, totalRecords } = response;
        const { userId } = this.props;
        const filteredList = filterUnFetchListBySS(list, userId);

        this.setState({ loading: false, list, totalRecords, filteredList });
      })
      .catch((e) => {
        this.setState({ loading: false, list: [], totalRecords: 0, filteredList: [] });
      });
  };

  onRefresh = (params) => {
    this.search();
    this.onPageToDetail(params);
  };

  onPaginationChange = ({ current, pageSize }, filters, sorter) => {
    this.setState({ loading: true });

    const changed = {
      currentPage: this.state.pageSize !== pageSize ? 1 : current,
      pageSize,
    };

    const params = {
      ...this.formatSearchParams(),
      ...changed,
    };
    const { userId } = this.props;
    this.loadCusList(params).then(({ list, totalRecords }) => {
      const filteredList = filterUnFetchListBySS(list, userId);
      this.setState({ loading: false, ...changed, list, totalRecords, filteredList });
    });
  };

  /**
   * 获取跟进列表
   * @param params
   */
  loadCusList = (params) => {
    return loaders.loadCusList(params);
  };

  /**
   * 格式化搜索参数
   */
  formatSearchParams() {
    const {
      studentName,
      contact,
      gradeId,
      resEntranceId,
      isAppointCome,
      deliverTarget,
      dealStatus,
      intentionOfTheCustomerId,
      intentionCampusId,
      schoolName,
      customerName,
      tempTimeType,
      tempTime,
      currentPage,
      pageSize,
      pageOrder,
      pageOrderField,
    } = this.state;

    let param = {
      studentName,
      contact,
      gradeId,
      resEntranceId,
      isAppointCome,
      deliverTarget,
      dealStatus,
      intentionOfTheCustomerId,
      intentionCampusId,
      schoolName,
      customerName,
      currentPage,
      pageSize,
      pageOrder,
      pageOrderField,
    };
    const timeParam = formatDuringTime(tempTimeType, tempTime);

    param = {
      ...param,
      ...timeParam,
    };

    return param;
  }

  /**
   * 跳转客户详情/编辑页面
   * tips: 'main/openPage'的mirrorId字段会影响/CustomerDetailEdit/index的参数取值
   * 查看：{cusId, studentId, detailEditable: false, followEditable: false, intentionCampusId, deliverTarget}
   * 编辑：{cusId, studentId, detailEditable: true, followEditable: true, deliverTarget, followList: filteredList}
   *
   * @param params
   */
  onPageToDetail = (params) => {
    const { dispatch, type } = this.props;

    params = {
      ...params,
      type,
      onRefresh: this.onRefresh,
    };

    const path = params.detailEditable ? 'SSCustomerDetailEdit' : 'SSCustomerDetailView';

    dispatch({
      type: 'main/openPage',
      payload: {
        path: `/${path}`,
        params,
        refresh: true,
      },
    });
  };

  /**
   * 签单
   * tips: 合同学习中心（跟进客户列表中取 意向学习中心`intentionCampusId`）
   * tips: 新生-跳转添加合同页面，合同类型为新签
   * tips: 老生-提示:当前学生已到达4次课消，请联系辅导老师进行签单, 若是学习中心配置了不限制签单开关，则允许签单
   */
  signNew = async (record) => {
    const { cusId, studentId, studentName, gradeId, schoolId, schoolName, intentionCampusId: blCampusId } = record;
    const query = {
      cusId,
      studentId,
      studentName,
      gradeId,
      schoolId,
      schoolName,
      blCampusId,
      contractType: 'NEW_CONTRACT',
    };

    const judgeResponse = await loaders.judgeIsOldStuForSsBoss({ studentId });
    const isOldStudent = judgeResponse && judgeResponse.data;

    const switchFlag = await loaders.getSwitchConfigFlag(blCampusId);
    const { isUnLimitSign } = switchFlag;

    if (isOldStudent && !isUnLimitSign) {
      showMessage('当前学生长期常规课程课消已达到4次，请联系辅导老师进行签单', 'error');
      return;
    }

    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path: `/signContract/${JSON.stringify(query)}`,
        refresh: true,
      },
    });

    window.__refreshParentEntrance = (params, refresh) => {
      this.props.dispatch({
        type: 'main/removeTab',
        payload: { id: this.props.activeTabId },
      });

      if (!refresh) {
        this.props.dispatch({
          type: 'main/openPage',
          payload: {
            path: '/SSConsultantFollowCustomer',
            params: {
              initCusList: dicts.CUS_LIST_LOST,
            },
            refresh: true,
          },
        });

        return;
      }

      this.search();
      this.signNew({ ...record, ...params });
    };
  };

  /**
   * 主动将资源流失
   *
   * @param record
   */
  confirmLost = (record) => {
    const { cusId, studentId } = record;

    this.setState({ loading: true });

    loaders.confirmLostCus({ cusId, stuId: studentId }).then(
      () => {
        showMessage('操作成功', 'success');
        this.search();
      },
      (e) => {
        showMessage(e.text || '操作失败');
        this.setState({ loading: false });
      },
    );
  };

  showEmployeeTip = () => {
    const tip =
      '该学生的客户号码属于员工号码，根据集团要求，内部员工号码不允许签单，请使用非员工号码录入资源然后签单。';
    AntdModal.info({
      title: tip,
      content: '',
      className: 'homAntdModal',
    });
  };

  render() {
    const { userId } = this.props;
    const {
      loading,
      searchExpand,
      studentName,
      contact,
      gradeId,
      resEntranceId,
      isAppointCome,
      dealStatus,
      intentionOfTheCustomerId,
      intentionCampusId,
      schoolName,
      customerName,
      tempTimeType,
      tempTime,
      gradeDict,
      entranceDict,
      intentionDict,
      blCampusDict,
      list,
      totalRecords,
      currentPage,
      pageSize,
      filteredList,
    } = this.state;

    const columns = [
      { title: '客户姓名', dataIndex: 'cusName', key: 'cusName', width: COL_WIDTH_M },
      {
        title: '联系方式',
        dataIndex: 'contact',
        key: 'contact',
        width: COL_WIDTH_M,
        render: (text, record) => {
          return formatPhoneNumber(text);
        },
      },
      {
        title: '学生姓名',
        dataIndex: 'studentName',
        key: 'studentName',
        width: COL_WIDTH_L,
        render: (text, record) => {
          const { isNew, isSigned, employee } = record;
          let flag = <div>&nbsp;</div>;

          if (isNew) {
            flag = <div className={styles.redDot} />;
          } else if (isSigned) {
            flag = <div className={styles.tag}>签</div>;
          }

          return (
            <div className={styles.stuNameWrap}>
              <div className={styles.flag}>{flag}</div>
              {text}
              {employee && (
                <div>
                  <span className={'employeeFlag'}>内部员工</span>
                </div>
              )}
            </div>
          );
        },
      },
      { title: '年级', dataIndex: 'gradeName', key: 'gradeName', width: COL_WIDTH_S },
      { title: '学校', dataIndex: 'schoolName', key: 'schoolName', width: COL_WIDTH_M },
      { title: '资源所属学习中心', dataIndex: 'blCampusName', key: 'blCampusName', width: COL_WIDTH_M },
      { title: '意向学习中心', dataIndex: 'intentionCampusName', key: 'intentionCampusName', width: COL_WIDTH_M },
      { title: '资源来源', dataIndex: 'resEntranceName', key: 'resEntranceName', width: 80 },
      { title: '跟进人', dataIndex: 'deliverTargetName', key: 'deliverTargetName', width: COL_WIDTH_M },
      { title: '最新跟进记录', dataIndex: 'lastRemark', key: 'lastRemark', width: 260, lines: 2 },
      {
        title: '最后跟进时间',
        dataIndex: 'lastFollowUpTime',
        key: 'lastFollowUpTime',
        width: 120,
        render: (text) => {
          return splitDateString(text);
        },
      },
      { title: '跟进状态', dataIndex: 'dealStatusName', key: 'dealStatusName', width: 80 },
      { title: '是否诺到', dataIndex: 'isAppointCome', key: 'isAppointCome', width: 80 },
      { title: '诺到日期', dataIndex: 'appointmentDate', key: 'appointmentDate', width: 160 },
      { title: '意向度', dataIndex: 'intentionOfTheCustomerName', key: 'intentionOfTheCustomerName', width: 80 },
      { title: '未跟进天数', dataIndex: 'noopDay', key: 'noopDay', width: 120, sorter: true },
      { title: '创建日期', dataIndex: 'createDate', key: 'createDate', width: 120, sorter: true },
      {
        title: '操作',
        key: 'action',
        fixed: 'right',
        width: COL_WIDTH_XL,
        render: (text, record, index) => {
          const { cusId, studentId, customerStatus, employee } = record;
          const deliverTarget = formatDeliverTarget(record);
          const editParam = {
            cusId,
            studentId,
            detailEditable: true,
            followEditable: true,
            deliverTarget,
            followList: filteredList,
          };
          const isSameUser = deliverTarget === userId;
          const isSigned = calcIsSigned(customerStatus);
          const canDoSign = calcCanDoSign(record, userId);

          return isSigned ? (
            <Popover content="该客户已与其他咨询师签约，点击后进入流失列表" icon="info" maxWidth={180}>
              <TextLink className={styles.textLink} disabled={!isSameUser} onClick={() => this.confirmLost(record)}>
                已被签约
              </TextLink>
            </Popover>
          ) : (
            <span>
              <TextLink
                className={styles.textLink}
                disabled={!isSameUser}
                onClick={() => this.onPageToDetail(editParam)}
              >
                跟进
              </TextLink>
              <TextLink
                className={classNames({
                  [styles.textLink]: true,
                  [styles.disabled]: !canDoSign,
                })}
                onClick={() => {
                  // 20221115版本去除内部员工无法签单限制
                  // 是否是内部员工
                  /*if (employee) {
                      this.showEmployeeTip()
                      return
                    }*/
                  if (!canDoSign) return;

                  this.signNew(record);
                }}
              >
                签单
              </TextLink>
            </span>
          );
        },
      },
    ];

    return (
      <div className={styles.wrap}>
        <div style={{ display: 'flex' }}>
          <SearchBar
            expand={searchExpand}
            onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
            onSearch={this.search}
            onReset={this.reset}
          >
            <Input
              placeholder="学生姓名"
              value={studentName}
              onChange={(e) => this.setState({ studentName: e.target.value })}
            />
            <Input
              placeholder="联系方式"
              value={contact}
              onChange={(e) => this.setState({ contact: e.target.value })}
            />
            <Select
              placeholder="年级"
              dict={gradeDict}
              value={gradeId}
              onChange={(val) => this.setState({ gradeId: val })}
            />
            <Select
              placeholder="资源来源"
              dict={entranceDict}
              value={resEntranceId}
              onChange={(val) => this.setState({ resEntranceId: val })}
            />
            <Select
              placeholder="是否诺到"
              dict={SS_IS_APPOINT_COME_DICT}
              value={isAppointCome}
              onChange={(val) => this.setState({ isAppointCome: val })}
            />
            <Select
              placeholder="跟进人"
              dict={this.state.deliverTargetDict}
              value={this.state.deliverTarget}
              onChange={(val) => this.setState({ deliverTarget: val })}
            />
            <Select
              placeholder="跟进状态"
              dict={SS_DEAL_STATUS_DICT}
              value={dealStatus}
              onChange={(val) => this.setState({ dealStatus: val })}
            />
            <Select
              placeholder="意向度"
              dict={intentionDict}
              value={intentionOfTheCustomerId}
              onChange={(val) => this.setState({ intentionOfTheCustomerId: val })}
            />
            <Select
              placeholder="意向学习中心"
              showSearch={true}
              dict={blCampusDict}
              value={intentionCampusId}
              onChange={(val) => {
                this.setState({ intentionCampusId: val });
              }}
            />
            <Input
              placeholder="学校"
              value={schoolName}
              onChange={(e) => this.setState({ schoolName: e.target.value })}
            />
            <Input
              placeholder="客户姓名"
              value={customerName}
              onChange={(e) => this.setState({ customerName: e.target.value })}
            />

            <Select
              placeholder="时间类型"
              dict={dicts.followTimeTypeDict}
              value={tempTimeType}
              onChange={(val) => this.setState({ tempTimeType: val })}
            />
            <RangePicker
              placeholder={['日期(起)', '日期(止)']}
              format="YYYY-MM-DD"
              showTime={false}
              allowClear={true}
              size="large"
              value={tempTime ? [dateToMoment(tempTime[0]), dateToMoment(tempTime[1])] : []}
              onChange={(date, dateString) => {
                this.setState({ tempTime: dateString });
              }}
            />
          </SearchBar>
          <Button style={{ marginLeft: '-20px' }} onClick={this.refreshCurrentPage}>
            刷新
          </Button>
        </div>

        <List
          dataSource={list}
          columns={columns}
          pagination={{
            current: currentPage,
            total: totalRecords,
            pageSize,
          }}
          onChange={this.onPaginationChange}
          bordered={true}
          className={styles.customList}
        />

        <Loading loading={loading} />
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  const user = state.account.user;
  const { userId, organizationId } = user || {};
  const pathName = state.routing.locationBeforeTransitions.pathname;

  return {
    userId,
    userOrganizationId: organizationId,
    activeTabId: state.main.activeTabId,
    btnAuths: state.account.btnAuths,
    params: state.main.pageParams[pathName] || {},
  };
};

export { CusList };
export default connect(mapStateToProps, null, null, { withRef: true })(CusList);
