import React from 'react';
import { connect } from 'dva';
import styles from './styles.less';
import CusList from '../CusList';
import CusSignedList from '../CusSignedList';
import CusLostList from '../CusLostList';
import { CUS_LIST_FOLLOW, CUS_LIST_SIGNED, CUS_LIST_LOST, CUS_TRANSFER, cusListDict } from '../dicts';
import { Button, Loading } from 'COMPONENT';
import AddCusModal from '../AddCusModal';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import loaders from '../loaders';
import { Tabs } from 'antd';
import sectionStyle from 'COMPONENT/Section/styles.less';

const TabPane = Tabs.TabPane;

/**
 * 跟进客户tab(tab配置在cusListDict)
 */
class CusListSec extends React.Component {
  constructor(props) {
    super(props);

    const params = this.props.params || {};
    const { initCusList } = params;
    const activeCusList = initCusList ? initCusList : CUS_LIST_FOLLOW;

    this.state = {
      loading: false,

      activeCusList: activeCusList, // 当前激活状态的客户列表key
      addCusModalVis: false, // 录入客户弹框显示状态
      canAddCus: false,

      searchingState: {},
      defaultSearchConditions: this.props.params ? this.props.params.defaultSearchConditions : {},
    };

    this.cusLists = {}; // 客户列表tab信息 {id: {ref, inited}}

    cusListDict.forEach((d) => {
      this.cusLists[d.val] = { ref: null, inited: false };
    });
  }

  componentWillReceiveProps(nextProps) {
    const newStr = nextProps.initCusList || '';

    if (this.state.activeCusList !== newStr) {
      this.changeCusList(newStr);
    }

    if (this.props.defaultSearchConditions !== nextProps.defaultSearchConditions) {
      setTimeout(() => {
        this.setState({ defaultSearchConditions: nextProps.defaultSearchConditions });
      }, 50);
    }
  }

  componentDidMount() {
    const { activeCusList } = this.state;
    const { brenchId } = this.props.user;
    this.setState({ loading: true });

    Promise.all([
      loaders.getCrmResourceVisibility({
        orgId: brenchId,
        rid: 'saveCalloutCustomer',
      }),
      this.cusLists[activeCusList].ref.initContent(),
    ])
      .then(([canAddCus]) => {
        this.cusLists[activeCusList].inited = true;
        this.setState({ loading: false, canAddCus });
      })
      .catch((e) => {
        this.setState({ loading: false, canAddCus: true });
      });
  }

  changeCusList = (type) => {
    if (this.cusLists[type].inited) {
      this.setState({ activeCusList: type });
    } else {
      this.setState({ loading: true });
      this.cusLists[type].ref
        .initContent()
        .then(() => {
          this.cusLists[type].inited = true;
          this.setState({ loading: false, activeCusList: type });
        })
        .catch((e) => {
          this.setState({ loading: false, activeCusList: type });
        });
    }
  };

  /***
   * tab操作按钮
   *
   * @return {*[]}
   */
  getCusListToolBar = () => {
    const { transformBtnAuth } = this.props;
    const { canAddCus, activeCusList } = this.state;
    const showTransform = activeCusList === CUS_LIST_SIGNED;

    return [
      canAddCus && (
        <Button key={1} iconType="plus" onClick={() => this.setState({ addCusModalVis: true })}>
          录入客户
        </Button>
      ),
      showTransform && transformBtnAuth && (
        <Button key={2} onClick={this.onTransform} style={{ marginLeft: 15 }}>
          已签资源调配
        </Button>
      ),
    ];
  };

  addCusSuccess = () => {
    const { activeCusList } = this.state;
    const ref = this.cusLists[activeCusList].ref;

    if (ref) {
      ref.search();
    }
  };

  /**
   * 已签资源调配
   */
  onTransform = () => {
    if (this.cusLists && this.cusLists['2']) {
      const $ref = this.cusLists['2'].ref;
      $ref.onShowTransformModal();
    }
  };

  render() {
    const { dispatch } = this.props;
    const { activeCusList, addCusModalVis, loading, defaultSearchConditions } = this.state;

    return (
      <div className="Section" style={sectionStyle}>
        <div className="Section-body custom-tabs">
          <Loading loading={loading} />

          <Tabs
            className={styles.customTabs}
            tabBarExtraContent={this.getCusListToolBar()}
            onChange={this.changeCusList}
            activeKey={activeCusList}
            tabBarStyle={{ border: 'none', margin: '19px 0 25px 0' }}
          >
            {cusListDict.map((c) => {
              let listComp;

              if (c.val === CUS_LIST_FOLLOW) {
                listComp = (
                  <CusList
                    ref={(comp) => {
                      if (comp) {
                        this.cusLists[c.val].ref = getWrappedInstance(comp);
                      }
                    }}
                    dispatch={dispatch}
                    changeCusList={this.changeCusList}
                    defaultSearchConditions={defaultSearchConditions}
                  />
                );
              } else if (c.val === CUS_LIST_SIGNED) {
                listComp = (
                  <CusSignedList
                    ref={(comp) => {
                      if (comp) {
                        this.cusLists[c.val].ref = getWrappedInstance(comp);
                      }
                    }}
                    dispatch={dispatch}
                    changeCusList={this.changeCusList}
                  />
                );
              } else if (c.val === CUS_LIST_LOST) {
                listComp = (
                  <CusLostList
                    ref={(comp) => {
                      if (comp) {
                        this.cusLists[c.val].ref = getWrappedInstance(comp);
                      }
                    }}
                    dispatch={dispatch}
                    changeCusList={this.changeCusList}
                  />
                );
              }

              return (
                <TabPane key={c.val} tab={c.text} forceRender={true}>
                  {listComp}
                </TabPane>
              );
            })}
          </Tabs>
        </div>

        {addCusModalVis && (
          <AddCusModal onSaveSuccess={this.addCusSuccess} onClose={() => this.setState({ addCusModalVis: false })} />
        )}
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  const btnAuth = state.account.btnAuths || [];
  const transformBtnAuth = btnAuth.indexOf('SSSignedResourceTransform') > -1;

  return {
    params: state.main.pageParams['/SSConsultantFollowCustomer'],
    user: state.account.user,
    transformBtnAuth,
  };
};

export default connect(mapStateToProps)(CusListSec);
