import React from 'react';
import { connect } from 'dva';
import styles from './styles.less';
import { DatePicker } from 'antd';
import { Loading, List, SearchBar, Input, Select } from 'COMPONENT';
import { loadCampusDict, loadCommonDictByGroup } from 'UTIL/dict';
import loaders from '../loaders';
import dicts from '../dicts';
import { formatDuringTime, formatPhoneNumber, splitDateString } from '../utils';
import { SS_DEAL_STATUS_DICT, SS_IS_APPOINT_COME_DICT } from '../../../common/util/dicts';
import { dateToMoment } from 'COMPONENT/utils/form';

const RangePicker = DatePicker.RangePicker;
const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L } = List;

/**
 * 流失客户
 */
class CusLostList extends React.Component {
  constructor(props) {
    super(props);

    const { campusId } = this.props;

    this.initSearchParam = {
      studentName: null,
      contact: null,
      gradeId: null,
      resEntranceId: null,
      isAppointCome: null,
      deliverTarget: null,
      dealStatus: null,
      intentionOfTheCustomerId: null,
      intentionCampusId: campusId,
      schoolName: null,
      customerName: null,
      tempTimeType: '6',
      tempTime: null,
    };

    this.state = {
      loading: false,
      searchExpand: false, // 搜索栏展开状态

      studentName: null, // 查询条件 学生姓名
      contact: null, // 查询条件 联系方式
      gradeId: null, // 查询条件 年级
      resEntranceId: null, // 查询条件 资源入口/资源来源
      isAppointCome: null, // 查询条件 是否诺到/是否上门
      deliverTarget: null, // 查询条件 跟进人
      dealStatus: null, // 查询条件 跟进状态
      intentionOfTheCustomerId: null, // 查询条件 意向度
      intentionCampusId: '', // 查询条件 意向学习中心
      schoolName: null, // 查询条件 学校
      customerName: null, // 查询条件 客户姓名

      tempTimeType: '6', // 临时查询条件 时间类型
      tempTime: null, // 临时查询条件 时间(根据时间类型设置不同的起止时间)

      blCampusDict: [], // 字典 学校
      gradeDict: [], // 字典 年级
      entranceDict: [], // 字典 资源入口
      intentionDict: [], // 字典 意向度
      deliverTargetDict: [], // 字典 跟进人

      list: [], // 列表数据 [{id, studentName, gradeId, schoolName, customerName, contact, resEntranceId}]
      totalRecords: 0, // 总记录数
      currentPage: 1, // 当前页码
      pageSize: 10, // 每页记录数
    };
  }

  async initialBranch() {
    let { branchId, campusId, defaultSearchConditions } = this.props;

    if (defaultSearchConditions) {
      branchId = defaultSearchConditions.branchId || branchId;
      campusId = defaultSearchConditions.campusId || campusId;
    }

    const blCampusDict = await loadCampusDict(branchId);

    this.setState({
      intentionCampusId: campusId,
      blCampusDict,
    });
  }

  initContent = async () => {
    await this.initialBranch();

    const { userOrganizationId } = this.props;
    const category = 'STUDENT_GRADE,RES_ENTRANCE,CUSTOMER_SATISFICING';
    const [listData, groupDict, deliverTargetDict] = await Promise.all([
      this.loadCusLostList(this.formatSearchParams()),
      loadCommonDictByGroup(category),
      loaders.loadDeliverTargetDict({ roleCode: 'CONSULTOR', organizationId: userOrganizationId }),
    ]);
    const { list, totalRecords } = listData;

    this.setState({
      gradeDict: groupDict['STUDENT_GRADE'],
      entranceDict: groupDict['RES_ENTRANCE'],
      intentionDict: groupDict['CUSTOMER_SATISFICING'],
      deliverTargetDict,
      list,
      totalRecords,
    });
  };

  search = () => {
    this.setState({ loading: true });

    const params = this.formatSearchParams();

    this.loadCusLostList({ ...params, currentPage: 1 })
      .then(({ list, totalRecords }) => {
        this.setState({ loading: false, currentPage: 1, list, totalRecords });
      })
      .catch(() => {
        this.setState({
          loading: false,
          currentPage: 1,
          list: [],
          totalRecords: 0,
        });
      });
  };

  reset = () => {
    this.setState(this.initSearchParam);
  };

  onPaginationChange = ({ current, pageSize }) => {
    const changed = {
      currentPage: this.state.pageSize !== pageSize ? 1 : current,
      pageSize,
    };

    this.setState({ loading: true });

    this.loadCusLostList({ ...this.formatSearchParams(), ...changed })
      .then(({ list, totalRecords }) => {
        this.setState({ loading: false, ...changed, list, totalRecords });
      })
      .catch(() => {
        this.setState({
          loading: false,
          ...changed,
          list: [],
          totalRecords: 0,
        });
      });
  };

  /**
   * 获取流失客户
   * @param params
   */
  loadCusLostList = (params) => {
    return loaders.loadCusLostList(params);
  };

  /**
   * 格式化搜索参数
   */
  formatSearchParams = () => {
    const {
      studentName,
      customerName,
      gradeId,
      schoolName,
      contact,
      resEntranceId,
      isAppointCome,
      deliverTarget,
      dealStatus,
      intentionOfTheCustomerId,
      intentionCampusId,
      tempTimeType,
      tempTime,
      currentPage,
      pageSize,
    } = this.state;

    let param = {
      studentName,
      customerName,
      gradeId,
      schoolName,
      contact,
      resEntranceId,
      isAppointCome,
      deliverTarget,
      dealStatus,
      intentionOfTheCustomerId,
      intentionCampusId,
      currentPage,
      pageSize,
    };
    const timeParam = formatDuringTime(tempTimeType, tempTime);

    param = {
      ...param,
      ...timeParam,
    };

    return param;
  };

  render() {
    const {
      loading,
      searchExpand,
      studentName,
      gradeId,
      schoolName,
      intentionCampusId,
      customerName,
      contact,
      resEntranceId,
      isAppointCome,
      dealStatus,
      intentionOfTheCustomerId,
      tempTimeType,
      tempTime,
      gradeDict,
      entranceDict,
      intentionDict,
      blCampusDict,
      list,
      totalRecords,
      currentPage,
      pageSize,
    } = this.state;

    const columns = [
      { title: '客户姓名', dataIndex: 'cusName', key: 'cusName', width: COL_WIDTH_M },
      {
        title: '联系电话',
        dataIndex: 'contact',
        key: 'contact',
        width: COL_WIDTH_M,
        render: (text) => {
          return formatPhoneNumber(text);
        },
      },
      { title: '学生姓名', dataIndex: 'studentName', key: 'studentName', width: COL_WIDTH_L },
      { title: '年级', dataIndex: 'gradeName', key: 'gradeName', width: COL_WIDTH_S },
      { title: '学校', dataIndex: 'schoolName', key: 'schoolName', width: COL_WIDTH_M },
      { title: '资源所属学习中心', dataIndex: 'blCampusName', key: 'blCampusName', width: COL_WIDTH_M },
      { title: '意向学习中心', dataIndex: 'intentionCampusName', key: 'intentionCampusName', width: COL_WIDTH_M },
      { title: '资源来源', dataIndex: 'resEntranceName', key: 'resEntranceName', width: 80 },
      { title: '跟进人', dataIndex: 'deliverTargetName', key: 'deliverTargetName', width: COL_WIDTH_M },
      { title: '最新跟进记录', dataIndex: 'lastFollowupRemark', key: 'lastFollowupRemark', width: 260, lines: 2 },
      {
        title: '最后跟进时间',
        dataIndex: 'lastFollowUpTime',
        key: 'lastFollowUpTime',
        width: 120,
        render: (text) => {
          return splitDateString(text);
        },
      },
      { title: '跟进状态', dataIndex: 'dealStatusName', key: 'dealStatusName', width: 80 },
      { title: '意向度', dataIndex: 'intentionOfTheCustomerName', key: 'intentionOfTheCustomerName', width: 80 },
      { title: '未跟进天数', dataIndex: 'noopDay', key: 'noopDay', width: 120, sorter: true },
      { title: '创建日期', dataIndex: 'createTime', key: 'createTime', width: 160, sorter: true },
      { title: '流失时间', dataIndex: 'lostCustomerTime', key: 'lostCustomerTime', width: 160 },
      { title: '流失原因', dataIndex: 'lostCustomerReason', key: 'lostCustomerReason', width: COL_WIDTH_L, lines: 2 },
    ];

    return (
      <div className={styles.wrap}>
        <SearchBar
          expand={searchExpand}
          onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
          onSearch={this.search}
          onReset={this.reset}
        >
          <Input
            placeholder="学生姓名"
            value={studentName}
            onChange={(e) => this.setState({ studentName: e.target.value })}
          />
          <Input placeholder="联系方式" value={contact} onChange={(e) => this.setState({ contact: e.target.value })} />
          <Select
            placeholder="年级"
            dict={gradeDict}
            value={gradeId}
            onChange={(val) => this.setState({ gradeId: val })}
          />
          <Select
            placeholder="资源来源"
            dict={entranceDict}
            value={resEntranceId}
            onChange={(val) => this.setState({ resEntranceId: val })}
          />
          <Select
            placeholder="是否诺到"
            dict={SS_IS_APPOINT_COME_DICT}
            value={isAppointCome}
            onChange={(val) => this.setState({ isAppointCome: val })}
          />
          <Select
            placeholder="跟进人"
            dict={this.state.deliverTargetDict}
            value={this.state.deliverTarget}
            onChange={(val) => this.setState({ deliverTarget: val })}
          />
          <Select
            placeholder="跟进状态"
            dict={SS_DEAL_STATUS_DICT}
            value={dealStatus}
            onChange={(val) => this.setState({ dealStatus: val })}
          />
          <Select
            placeholder="意向度"
            dict={intentionDict}
            value={intentionOfTheCustomerId}
            onChange={(val) => this.setState({ intentionOfTheCustomerId: val })}
          />
          <Select
            placeholder="意向学习中心"
            showSearch
            allowClear={true}
            dict={blCampusDict}
            value={intentionCampusId}
            onChange={(val) => {
              this.setState({ intentionCampusId: val });
            }}
          />
          <Input
            placeholder="学校"
            value={schoolName}
            onChange={(e) => this.setState({ schoolName: e.target.value })}
          />
          <Input
            placeholder="客户姓名"
            value={customerName}
            onChange={(e) => this.setState({ customerName: e.target.value })}
          />

          <Select
            placeholder="时间类型"
            dict={dicts.lostTimeTypeDict}
            value={tempTimeType}
            onChange={(val) => this.setState({ tempTimeType: val })}
          />
          <RangePicker
            placeholder={['日期(起)', '日期(止)']}
            format="YYYY-MM-DD"
            showTime={false}
            allowClear={true}
            size="large"
            value={tempTime ? [dateToMoment(tempTime[0]), dateToMoment(tempTime[1])] : []}
            onChange={(date, dateString) => {
              this.setState({ tempTime: dateString });
            }}
          />
        </SearchBar>

        <List
          dataSource={list}
          columns={columns}
          pagination={{
            current: currentPage,
            total: totalRecords,
            pageSize,
          }}
          onChange={this.onPaginationChange}
        />

        <Loading loading={loading} />
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  const user = state.account.user;
  const { userId, organizationId } = user || {};

  return {
    userId,
    userOrganizationId: organizationId,
  };
};

export default connect(mapStateToProps, null, null, { withRef: true })(CusLostList);
