import React from 'react';
import Modal from 'COMPONENT/Modal';
import styles from './styles.less';
import Select from 'COMPONENT/Select';
import FormItem from 'COMPONENT/FormItem';
import TextArea from 'COMPONENT/TextArea';
import { loadAllBranchCampusDict } from 'UTIL/dict';
import classnames from 'classnames';
import loaders from '../loaders';
import { getCommonErrorTips } from 'COMPONENT/utils/form';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

const formCfg = {
  labelWidth: 80,
};

const checkLen = (value) => {
  value = value || '';

  if (value.length > 200) {
    return `请勿超过200字`;
  }

  return null;
};

class TransferCampusModal extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false, // {boolean} 等待状态

      reason: '', // {string} 转校理由
      transToCampus: '', // {string} 转入学习中心

      transToCampusDict: [], // {object[]} 字典，转入学习中心

      checking: false, // {boolean} 校验开启状态
    };

    this.errors = {};
  }

  componentDidMount() {
    this.init();
  }

  init = async () => {
    this.setState({ loading: true });
    const promises = [loadAllBranchCampusDict()];
    const [dict] = await Promise.all(promises);
    this.setState({
      transToCampusDict: dict,
      loading: false,
    });
  };

  onOk = async () => {
    const { followingRecord, onConfirm, onClose } = this.props;
    const { checking, reason, transToCampus } = this.state;
    this.setState({ loading: true });
    try {
      if (!checking) {
        this.setState({ checking: true });
      }

      const errorTips = getCommonErrorTips(this.errors);
      if (errorTips) {
        this.modal.showOkTips(errorTips);
        this.setState({ loading: false });
        return;
      }

      const params = {
        applyReason: reason,
        customerId: followingRecord.id,
        oldDeliverTarget: followingRecord.deliverTarget,
        receiveCampusId: transToCampus,
        studentId: followingRecord.stuId,
      };

      await loaders.saveTransferCampus(params);
      showMessage('保存成功', 'success');
      onConfirm();
      onClose();
    } catch (e) {
      console.log(e);
      showMessage('保存失败:' + e.text, 'error');
      this.setState({ loading: false });
    }
  };

  errorChange = (key, error) => {
    this.errors[key] = error;
  };

  render() {
    const { onClose } = this.props;
    const { loading, reason, transToCampus, transToCampusDict, checking } = this.state;
    const modalCls = {
      [styles.wrap]: 1,
      [styles.modal]: 1,
    };

    return (
      <Modal
        className={classnames(modalCls)}
        loading={loading}
        title="转校申请"
        width={514}
        onOk={this.onOk}
        onClose={onClose}
        ref={(c) => (this.modal = getWrappedInstance(c))}
      >
        <div className={styles.main}>
          <FormItem {...formCfg} name="转校理由" required>
            <TextArea
              placeholder="请输入"
              rows={4}
              value={reason}
              onChange={(e) => this.setState({ reason: e.target.value })}
              validator={{
                rules: [{ required: '请填写转校理由' }, checkLen],
                silent: !checking,
                onErrorChange: (e) => this.errorChange('reason', e),
              }}
              maxLength={200}
              wordCount={true}
            />
          </FormItem>
          <FormItem {...formCfg} name="转入学习中心" required>
            <Select
              dict={transToCampusDict}
              value={transToCampus}
              onChange={(val) => this.setState({ transToCampus: val })}
              validator={{
                rules: ['required'],
                silent: !checking,
                onErrorChange: (e) => this.errorChange('transToCampus', e),
              }}
              showSearch
            />
          </FormItem>
        </div>
      </Modal>
    );
  }
}

export default TransferCampusModal;
