import React from 'react';
import styles from './styles.less';
import { loadVisitCusList } from '../loaders';
import { getWeekRange } from 'PAGE/consultant/common/VisitCusSchedule/utils';
import VisitCusSchedule from '../../../common/component/VisitCusSchedule';
import WeekSelect from '../../../common/component/WeekSelect';
import moment from 'moment';
import Loading from 'COMPONENT/Loading';

const { startDate: initStartDate, endDate: initEndDate } = getWeekRange(moment());

class VisitCusContent extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false, // {boolean} 内容加载状态
      startDate: initStartDate, // {string} 开始日期
      endDate: initEndDate, // {string} 结束日期
      list: [], // {object[]} 列表数据，
      rate: '',
      // {status, date, startTime, endTime, name, contract, studentName, gradeName, schoolName, headerInfo, title, infos, moreInfos}
    };
  }

  componentDidMount() {
    this.initContent();
  }

  initContent = async () => {
    const { startDate, endDate } = this.state;
    this.setState({ loading: true });
    const list = await loadVisitCusList({ startDate, endDate });
    const rate = this.getVisitedRateText(list);
    this.setState({ loading: false, list, rate });
  };

  goWeek = ({ startDate, endDate }) => {
    this.setState({ loading: true });

    loadVisitCusList({ startDate, endDate }).then(
      (list) => {
        const rate = this.getVisitedRateText(list);
        this.setState({ loading: false, startDate, endDate, list, rate });
      },
      (e) => {
        this.setState({ loading: false });
        showMessage(e.text || '预约上门客户数据加载失败');
      },
    );
  };

  search = () => {
    this.initContent();
  };

  getVisitedRateText = (list) => {
    return '' + (list.length ? list.filter((ele) => ele.isVisit === true).length : 0);
  };

  render() {
    const { loading, startDate, endDate, list, rate } = this.state;

    return (
      <div className={styles.section}>
        <div className={styles.sectionHeader}>
          <div className={styles.sectionTitle}>{`预约上门客户（上门率：${rate}/${list.length || 0}）`}</div>
        </div>
        <div className={styles.sectionBody}>
          <div className={styles.wrap}>
            <div className={styles.toolbar}>
              <WeekSelect value={{ startDate, endDate }} onChange={this.goWeek} />
            </div>
            <VisitCusSchedule startDate={startDate} endDate={endDate} list={list} onRefresh={this.search} />
          </div>
        </div>
        <Loading loading={loading} />
      </div>
    );
  }
}

export default VisitCusContent;
