import {
  getConsultorNextFollowing,
  getConsultorFollowingList,
  getConsultorSignUpList,
  getConsultorLostList,
  obtainCustomerInternet,
  obtainCustomerAgain,
  updateCustomerStudentStatus,
  submitStudentTransferApply,
  listStudentTransferApplyBySelf,
  listStudentTransferApplyForCampusAudit,
  crmResourceVisibility,
  canSign,
  getConsultorCustomerFolowup,
} from 'SERVICE/consultant';
import { judgeIsOldStuForSsBoss, allocateCustomerResource, getUserByRoleCodesSelection } from 'SERVICE/ssCRM';
import { deliverCustomer } from 'SERVICE/customer';
import { getSwitchConfigsByCodesAndCampusIds } from 'SERVICE/common';
import { CUS_LIST_FOLLOW, CUS_LIST_SIGNED, CUS_LIST_LOST, IS_VISIT_YES, IS_VISIT_NO } from './dicts';
import dicts from '../../common/component/VisitCusSchedule/dicts';
import { formateDict, getDictTextByVal } from 'UTIL/dict';
import moment from 'moment';
import { SWITCH_CONFIG_UNLIMITED_NEW_SIGN } from './utils';

const SC_SAVE_CONTRACT_2020 = 'SC_SAVE_CONTRACT_2020';

/**
 * 获取咨询师工作台跟进人字典
 *
 * @param params.roleCode {string} 用户角色
 * @param params.organizationId {string} 用户主组织ID
 * @return {Promise<Array>}
 */
const loadDeliverTargetDict = async (params) => {
  try {
    const response = await getUserByRoleCodesSelection(params);

    return formateDict(response);
  } catch (e) {
    return [];
  }
};

const loadPhoneCusList = async ({ startDate, endDate, branchId, campusId, consultantId }) => {
  try {
    const { data } = await getConsultorNextFollowing({
      dateStart: startDate,
      dateEnd: endDate,
      branchId,
      campusId,
      consultantId,
    });
    const list = [
      { id: 'stayFollow', text: '预约跟进', val: 0, startDate: 'nextFollowTime', endDate: 'nextFollowTimeEnd' },
      {
        id: 'directVisit',
        text: '直访',
        val: 0,
        resEntranceId: 'DIRECT_VISIT',
        startDate: 'createDateStart',
        endDate: 'createDateEnd',
      },
      { id: 'appointment', text: '预约上门', val: 0, startDate: 'appointmentTime', endDate: 'appointmentTimeEnd' },
      {
        id: 'callIn',
        text: '呼入',
        val: 0,
        resEntranceId: 'CALL_IN',
        startDate: 'createDateStart',
        endDate: 'createDateEnd',
      },
      { id: 'visited', text: '已上门', val: 0, startDate: 'visitComeDateStart', endDate: 'visitComeDateEnd' },
      {
        id: 'onLine',
        text: '网络',
        val: 0,
        resEntranceId: 'ON_LINE',
        startDate: 'createDateStart',
        endDate: 'createDateEnd',
      },
      { id: 'followed', text: '已跟进', val: 0, startDate: 'newFollowTime', endDate: 'newFollowTimeEnd' },
    ];

    return list.map((ele) => {
      return Object.assign(ele, {
        val: data[ele.id],
      });
    });
  } catch (e) {
    console.log(e);
  }
};

const loadPhoneCusListManage = async ({ startDate, endDate, branchId, campusId, consultantId }) => {
  try {
    const { data } = await getConsultorNextFollowing({
      dateStart: startDate,
      dateEnd: endDate,
      branchId,
      campusId,
      consultantId,
    });
    const list = [
      { id: 'stayFollow', text: '预约跟进', val: 0, startDate: 'nextFollowTime', endDate: 'nextFollowTimeEnd' },
      { id: 'appointment', text: '预约上门', val: 0, startDate: 'appointmentTime', endDate: 'appointmentTimeEnd' },
      { id: 'visited', text: '已上门', val: 0, startDate: 'visitComeDateStart', endDate: 'visitComeDateEnd' },
      { id: 'followed', text: '已跟进', val: 0, startDate: 'newFollowTime', endDate: 'newFollowTimeEnd' },
      {
        id: 'directVisit',
        text: '直访',
        val: 0,
        resEntranceId: 'DIRECT_VISIT',
        startDate: 'createDateStart',
        endDate: 'createDateEnd',
      },
      {
        id: 'callIn',
        text: '呼入',
        val: 0,
        resEntranceId: 'CALL_IN',
        startDate: 'createDateStart',
        endDate: 'createDateEnd',
      },
      {
        id: 'onLine',
        text: '网络',
        val: 0,
        resEntranceId: 'ON_LINE',
        startDate: 'createDateStart',
        endDate: 'createDateEnd',
      },
    ];

    return list.map((ele) => {
      return Object.assign(ele, {
        val: data[ele.id],
      });
    });
  } catch (e) {
    console.log(e);
  }
};

/**
 * 获取跟进客户列表
 *
 * @param param
 * @return {Promise<{totalRecords: *, page: *, list: *}>}
 */
const loadCusList = async (param) => {
  const { currentPage, pageSize, pageOrder, pageOrderField } = param;
  let params = {
    ...param,
    page: currentPage,
    rows: pageSize,
    sord: pageOrder,
    sidx: pageOrderField,
  };

  delete params.currentPage;
  delete params.pageSize;
  delete params.pageOrder;
  delete params.pageOrderField;

  const { records, rows, page } = await getConsultorFollowingList(params);
  const data = rows || [];
  const totalRecords = records || 0;
  const list = data.map((row) => {
    const { nextFollowupDate, customerStatus, dealStatus } = row;

    return {
      ...row,
      nextFollowTime: getDateByDateTime(nextFollowupDate),
      isSigned: customerStatus === 'FLOW_AWAY',
      unFetch: dealStatus === 'STAY_FOLLOW',
    };
  });

  return { list, totalRecords, page };
};

/**
 * 获取签单客户列表
 *
 * @param param
 * @return {Promise<{totalRecords: *, page: *, list: *}>}
 */
const loadCusSignedList = async (param) => {
  const { currentPage, pageSize } = param;
  const params = {
    ...param,
    page: currentPage,
    rows: pageSize,
  };

  delete params.currentPage;
  delete params.pageSize;

  try {
    const { records, rows, page } = await getConsultorSignUpList(params);

    return { list: rows || [], totalRecords: records || 0, page };
  } catch (e) {
    return { list: [], totalRecords: 0, page: currentPage };
  }
};

/**
 * 获取流失客户列表
 *
 * @param param
 * @return {Promise<{totalRecords: *, page: *, list: *}>}
 */
const loadCusLostList = async (param = {}) => {
  const { customerName, currentPage, pageSize } = param;
  const params = {
    ...param,
    cusName: customerName,
    page: currentPage,
    rows: pageSize,
  };

  delete params.customerName;
  delete params.currentPage;
  delete params.pageSize;

  const { records, rows, page } = await getConsultorLostList(params);

  return { list: rows || [], totalRecords: records || 0, page };
};

/**
 * 时间表
 *
 * @param startDate
 * @param endDate
 * @return {Promise<*>}
 */
const loadVisitCusList = async ({ startDate, endDate }) => {
  const { data } = await getConsultorCustomerFolowup({ dateStart: startDate, dateEnd: endDate });
  const cusList = data
    .filter((da) => !!da.meetingTime && da.meetingTime.split(' ').length > 1)
    .map((d) => {
      const timeArry = d.meetingTime.split(' ');
      const date = timeArry[0];
      const startTime = timeArry[1].slice(0, 5);

      const endMeetingTime = moment(d.meetingTime).add(1, 'hours').format('YYYY-MM-DD HH:mm:ss');
      const endTimeArry = endMeetingTime.split(' ');
      const endDate = endTimeArry[0];
      const endTime = endTimeArry[1].slice(0, 5);

      return {
        id: d.id,
        status: d.isVisit ? dicts.VISIT_STATUS_VISITED : dicts.VISIT_STATUS_UNVISIT,
        date,
        startTime,
        endTime,
        name: d.customerName,
        contract: d.cusMobile,
        studentName: d.followStuName,
        gradeName: d.gradeName,
        schoolName: d.schoolName,
        cusId: d.customerId,
        stuId: d.followStuId,
        isVisit: d.isVisit,
        isOverDay: date !== endDate,
        createUserId: d.createUserId,
        createUserName: d.createUserName,
      };
    });

  const cardList = cusList.map((c) => {
    const { startTime, name, status, contract, studentName, gradeName, schoolName, createUserName } = c;
    const moreInfos = [
      { label: '上门状态', content: getDictTextByVal(dicts.visitStatusDict, status) },
      { label: '上门时间', content: startTime },
      { label: '预约人', content: createUserName },
      { label: '客户电话', content: contract },
      { label: '学生姓名', content: studentName },
      { label: '学生年级', content: gradeName },
      { label: '学生学校', content: schoolName },
    ];

    return {
      ...c,
      headerInfo: startTime,
      title: name,
      infos: [],
      moreInfos,
    };
  });

  return cardList;
};

const getDateByDateTime = (dateTime) => {
  if (!dateTime) {
    return null;
  }
  return dateTime.slice(0, 10);
};

const loadTransToCampusDict = async () => {
  const dict = [{ val: '1', text: 'xx学习中心' }];
  return dict;
};

const saveTransferCampus = async (params) => {
  await submitStudentTransferApply(params);
  return true;
};

/**
 * 操作-获取
 * @param cusId
 * @param stuId
 * @return {Promise<boolean>}
 */
const obtainCustomer = async ({ cusId, studentId }) => {
  await obtainCustomerInternet({ customerId: cusId, studentId: studentId });
  return true;
};

const reObtainCustomer = async ({ id, stuId }) => {
  await obtainCustomerAgain({ customerId: id, studentId: stuId });
  return true;
};

const confirmLostCus = async ({ cusId, stuId }) => {
  await updateCustomerStudentStatus({ cusId: cusId, studentId: stuId, customerStatus: 'INVALID' }, true);
  return true;
};

const loadTransferApplyingList = async (params) => {
  const data = await listStudentTransferApplyBySelf(params);
  if (data.resultCode !== 0) {
    throw new Error(data.resultMessage);
  }
  return data;
};

const loadTransferOutAuditList = async (params) => {
  const data = await listStudentTransferApplyForCampusAudit(params);
  if (data.resultCode !== 0) {
    throw new Error(data.resultMessage);
  }
  return data;
};

const getCrmResourceVisibility = async (params) => {
  const res = await crmResourceVisibility(params);
  if (res.resultCode !== 0) {
    throw new Error(res.resultMessage);
  }
  return !!!res.data.hidden;
};

/**
 * 获取是否新签、是否不限制签单开关
 *
 * @param blCampusId 学习中心ID
 * @return {Promise<{isSignNew: boolean, isUnLimitSign: boolean}>}
 */
const getSwitchConfigFlag = async (blCampusId) => {
  let isSignNew = false;
  let isUnLimitSign = false;

  const codes = [SC_SAVE_CONTRACT_2020, SWITCH_CONFIG_UNLIMITED_NEW_SIGN].join(',');
  const param = { codes, campusIds: blCampusId, flag: true };
  const response = await getSwitchConfigsByCodesAndCampusIds(param);
  const { data } = response || {};

  if (data && data.length > 0) {
    data.forEach((item) => {
      const { itemCode, switchFlag } = item;

      if (itemCode === SC_SAVE_CONTRACT_2020 && switchFlag === true) {
        isSignNew = true;
      }

      if (itemCode === SWITCH_CONFIG_UNLIMITED_NEW_SIGN && switchFlag === true) {
        isUnLimitSign = true;
      }
    });
  }

  return {
    isSignNew,
    isUnLimitSign,
  };
};

/**
 * 客户资源调配(星火)
 *
 * @param param.cusId {string} 所选客户ID 'id1,id2'
 * @param param.deliverType {string} {PERSONAL_POOL: 分配到个人，CUSTOMER_RESOURCE_POOL: 分配到资源池}
 * @param param.deliverTarget {string} 跟进人ID
 * @return {Promise<*>}
 */
const doDeliverCustomer = (param) => {
  return deliverCustomer(param);
};

/**
 * 客户调配(双师)
 *
 * @param param.isSignedResources {boolean} 是否已签资源调配，{true: 是, false: 否}
 * @param param.deliverTarget {string} 新跟进人（树节点ID）
 * @param param.sourceList {array} 被分配客户列表 [{cusId: '客户ID', studentId: '学生ID', deliverTarget: '原跟进人', deliverType: '跟进人类型'}]
 * @return {Promise<*>}
 */
const doAllocateCustomerResource = (param) => {
  return allocateCustomerResource(param);
};

export default {
  loadDeliverTargetDict,
  loadCusList,
  loadCusSignedList,
  loadCusLostList,
  loadVisitCusList,
  loadPhoneCusList,
  loadTransToCampusDict,
  saveTransferCampus,
  obtainCustomer,
  reObtainCustomer,
  confirmLostCus,
  loadTransferApplyingList,
  loadTransferOutAuditList,
  getCrmResourceVisibility,
  loadPhoneCusListManage,
  canSign,
  judgeIsOldStuForSsBoss,
  getSwitchConfigFlag,
  doDeliverCustomer,
  doAllocateCustomerResource,
};
