import React from 'react';
import { Input } from 'antd';
import TagList from '../TagList';
import style from './index.less';

const SearchInput = Input.Search;

/**
 * 支持搜索的标签列表
 *
 * @param props.tagList {array} 完整的标签列表
 * @param props.placeholder {string} 占位符
 * @param props.onSelect {function} 选中标签
 */
class SearchTag extends React.Component {
  static defaultProps = {
    tagList: [],
    placeholder: null,
    onSelect: (val) => {},
  };

  constructor(props) {
    super(props);

    this.state = {
      inputValue: null,
      resultList: [],
    };
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化
   */
  initContent = () => {
    this.setState({
      resultList: this.props.tagList || [],
    });
  };

  /**
   * 输入input
   *
   * @param e
   */
  onInputChange = (e) => {
    const { tagList } = this.props;
    const inputValue = e.target.value || null;
    let result = tagList;

    if (inputValue) {
      result = tagList.filter((value) => value.indexOf(inputValue) > -1);
    }

    this.setState({
      inputValue: inputValue,
      resultList: result,
    });
  };

  /**
   * 点击标签选中
   *
   * @param tagName
   */
  onSelect = (tagName) => {
    this.props.onSelect(tagName);
  };

  render() {
    const { resultList } = this.state;
    const hasTag = resultList && resultList.length > 0;

    return (
      <div className={style.searchWrap}>
        <div style={{ padding: '0 10px', marginBottom: 5 }}>
          <SearchInput placeholder={this.props.placeholder} onChange={this.onInputChange} />
        </div>

        <div className={style.tagList}>
          {hasTag ? <TagList list={resultList} onTagClick={this.onSelect} /> : <p className={style.noDate}>暂无数据</p>}
        </div>
      </div>
    );
  }
}

export default SearchTag;
