import { getCustomerImportInfoList, editImportCustomer, batchImportCustomer } from 'SERVICE/ssCRM';

/**
 * 获取资源导入列表
 *
 * @param param
 * @return {Promise<{totalRecords: *, page: *, list: *}>}
 */
async function loadCustomerList(param = {}) {
  const {
    startTime,
    endTime,
    customerName,
    contact,
    schoolName,
    intentionCampusId,
    isAppointCome,
    currentPage,
    pageSize,
  } = param;
  const params = {
    ...param,
    page: currentPage,
    rows: pageSize,
    startDate: startTime,
    endDate: endTime,
    cusName: customerName,
    cusContact: contact,
    blSchoolName: schoolName,
    blSchool: intentionCampusId,
    hasAppointment: isAppointCome,
  };

  delete params.currentPage;
  delete params.pageSize;
  delete params.startTime;
  delete params.endTime;
  delete params.customerName;
  delete params.contact;
  delete params.schoolName;
  delete params.intentionCampusId;
  delete params.isAppointCome;

  const { records, rows, page } = await getCustomerImportInfoList(params);

  const data = rows || [];
  const list = data.map((item) => {
    return {
      ...item,
      contact: item.cusContact,
      schoolName: item.blSchoolName,
      isAppointCome: item.hasAppointment,
      appointmentDate: item.appointmentTime,
      lastRemark: item.remark,
      createDate: item.createTime,
    };
  });

  return { list: list, totalRecords: records, page };
}

/**
 * 执行删除
 *
 * @param params
 * @return {Promise<void>}
 */
async function deleteCustomer(params = {}) {
  return await editImportCustomer(params);
}

/**
 * 执行导入
 *
 * @param params
 * @return {Promise<*>}
 */
async function importCustomer(params) {
  return await batchImportCustomer(params);
}

export default {
  loadCustomerList,
  deleteCustomer,
  importCustomer,
};
