import React from 'react';
import Block from 'COMPONENT/Block';
import Modal from 'COMPONENT/Modal';
import { Row, Col } from 'antd';
import Select from 'COMPONENT/Select';
import FormItem from 'COMPONENT/FormItem';
import styles from './index.less';
import InputNumber from 'COMPONENT/InputNumber';
import Icon from 'COMPONENT/Icon';
import Loaders from '../../loaders';
import { PROJECT_NAME } from '../../../../../constants/config';

class AddTarget extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      brenchId: props.branchDict[0].val,
      productTypeId: PROJECT_NAME === 'twoteacher' ? 'TWO_TEACHER' : 'SMALL_CLASS',
      yearId: props.productYearDict.filter((year) => {
        return year.text === new Date().toString().slice(11, 15);
      })[0].val,
      quarterId: props.productQuarterDict[0].val,

      // 目标科次
      goalSubject: [
        { gradeId: '', subjectId: '', goalVal: null },
        { gradeId: '', subjectId: '', goalVal: null },
        { gradeId: '', subjectId: '', goalVal: null },
      ],
    };
  }

  onBrenchIdChange = (brenchId) => {
    this.setState({ brenchId });
  };

  onProductTypeIdChange = (productTypeId) => {
    this.setState({ productTypeId });
  };

  onYearIdChange = (yearId) => {
    this.setState({ yearId });
  };

  onQuarterIdChange = (quarterId) => {
    this.setState({ quarterId });
  };

  onGoalSubjectChange = (value, type, index) => {
    const { goalSubject } = this.state;
    let newGoalSubject = [];
    switch (type) {
      case 'gradeId':
        newGoalSubject = goalSubject.map((goal, curIndex) => {
          if (index === curIndex) {
            return Object.assign(goal, { gradeId: value });
          } else {
            return goal;
          }
        });
        this.setState({ goalSubject: newGoalSubject });
        break;
      case 'subjectId':
        newGoalSubject = goalSubject.map((goal, curIndex) => {
          if (index === curIndex) {
            return Object.assign(goal, { subjectId: value });
          } else {
            return goal;
          }
        });
        this.setState({ goalSubject: newGoalSubject });
        break;
      case 'goalVal':
        newGoalSubject = goalSubject.map((goal, curIndex) => {
          if (index === curIndex) {
            return Object.assign(goal, { goalVal: value });
          } else {
            return goal;
          }
        });
        this.setState({ goalSubject: newGoalSubject });
        break;
      default:
        return;
    }
  };

  addGoalSubject = () => {
    let { goalSubject } = this.state;
    goalSubject.push({ gradeId: '', subjectId: '', goalVal: null });
    this.setState({ goalSubject });
  };

  deleteGoalSubject = (index) => {
    let { goalSubject } = this.state;
    goalSubject.splice(index, 1);
    this.setState({ goalSubject });
  };

  save = async () => {
    this.setState({ loading: true });
    const { brenchId, productTypeId, yearId, quarterId, goalSubject: conditions } = this.state;
    const params = { brenchId, productTypeId, yearId, quarterId, conditions };
    try {
      await Loaders.addByBrenchApi(params).then(() => {
        this.props.closeModal();
        this.props.search();
        this.setState({ loading: false });
      });
    } catch (error) {
      this.setState({ loading: false });
    }
  };

  render() {
    const { loading, brenchId, productTypeId, yearId, quarterId, goalSubject } = this.state;
    const { branchDict, productTypeDict, productYearDict, productQuarterDict, gradeDict, subjectDict } = this.props;
    return (
      <Modal loading={loading} title="增加目标" onCancel={this.props.closeModal} onOk={this.save}>
        <Block title="基础信息">
          <Row>
            <Col span={12}>
              <FormItem name="分公司:" labelWidth={80} className={styles.formItem}>
                <Select allowClear={false} dict={branchDict} value={brenchId} onChange={this.onBrenchIdChange} />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="产品类型:" labelWidth={80} className={styles.formItem}>
                <Select
                  allowClear={false}
                  dict={productTypeDict}
                  value={productTypeId}
                  onChange={this.onProductTypeIdChange}
                />
              </FormItem>
            </Col>
          </Row>
          <Row>
            <Col span={12}>
              <FormItem name="产品年份:" labelWidth={80} className={styles.formItem}>
                <Select allowClear={false} dict={productYearDict} value={yearId} onChange={this.onYearIdChange} />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="季度:" labelWidth={80} className={styles.formItem}>
                <Select
                  allowClear={false}
                  dict={productQuarterDict}
                  value={quarterId}
                  onChange={this.onQuarterIdChange}
                />
              </FormItem>
            </Col>
          </Row>
        </Block>
        <Block title="目标科次">
          <Row>
            <Col span={7} className={styles.col}>
              年级
            </Col>
            <Col span={7} className={styles.col}>
              科目
            </Col>
            <Col span={7} className={styles.col}>
              目标科次
            </Col>
            <Col span={3} className={styles.col}></Col>
          </Row>
          {goalSubject.map((goal, index) => {
            const { gradeId, subjectId, goalVal } = goal;
            return (
              <Row>
                <Col span={7} className={styles.col}>
                  <Select
                    value={gradeId}
                    className={styles.select}
                    dict={gradeDict}
                    onChange={(value) => this.onGoalSubjectChange(value, 'gradeId', index)}
                  />
                </Col>
                <Col span={7} className={styles.col}>
                  <Select
                    value={subjectId}
                    className={styles.select}
                    dict={subjectDict}
                    onChange={(value) => this.onGoalSubjectChange(value, 'subjectId', index)}
                  />
                </Col>
                <Col span={7} className={styles.col}>
                  <InputNumber
                    // validator={{
                    //   rules: [
                    //     (value) => {
                    //       if(value % 1 !== 0 || value <= 0){
                    //         return '只能填写正整数'
                    //       }
                    //     }
                    //   ],
                    // }}
                    value={goalVal}
                    max={999999999}
                    style={{ margin: '10px 20px' }}
                    step={1}
                    // min={1}
                    precision={0}
                    onChange={(value) => this.onGoalSubjectChange(value, 'goalVal', index)}
                  />
                </Col>
                <Col span={3} className={styles.col} style={{ height: '56px' }}>
                  <Icon type="plus_circle" onClick={this.addGoalSubject} className={styles.icon}></Icon>
                  <Icon
                    type="minus_circle"
                    onClick={() => this.deleteGoalSubject(index)}
                    className={styles.icon}
                  ></Icon>
                </Col>
              </Row>
            );
          })}
        </Block>
      </Modal>
    );
  }
}

export default AddTarget;
