import React from 'react';
import Modal from 'COMPONENT/Modal';
import InputNumber from 'COMPONENT/InputNumber';
import { Row, Col } from 'antd';
import Loaders from '../../loaders';
import style from './index.less';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

class CampusTarget extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      campusGoalList: [], // 学习中心
    };
  }

  componentDidMount() {
    this.init();
  }

  init = async () => {
    this.setState({ loading: true });
    const { modalGoalId: goalId } = this.props;
    try {
      const campusGoalList = await Loaders.getCampusByGoalIdApi({ goalId });
      this.setState({ campusGoalList, loading: false });
    } catch (error) {
      this.setState({ loading: false });
    }
  };

  save = async () => {
    const { modalGoalVal } = this.props;
    if (modalGoalVal !== null && modalGoalVal !== this.reduceAllGoldVal()) {
      showMessage('各学习中心的目标名额之和与分公司目标不同，请重新填写学习中心目标');
      return;
    }
    this.setState({ loading: true });
    const { campusGoalList } = this.state;
    const { modalGoalId: goalId, modalBrenchId: brenchId } = this.props;
    const params = { goalId, brenchId, campusGoalList };
    try {
      await Loaders.updateGoalByCampusApi(params);
      showMessage('保存成功', 'success');
      this.setState({ loading: false });
      this.props.closeModal();
      this.props.search();
    } catch (error) {
      this.setState({ loading: false });
    }
  };

  reduceAllGoldVal = () => {
    const { campusGoalList } = this.state;
    return campusGoalList.reduce((pre, campusGoal) => {
      if (campusGoal.campusGoalVal === null) {
        return pre;
      } else {
        return pre + parseInt(campusGoal.campusGoalVal);
      }
    }, 0);
  };

  render() {
    const { loading, campusGoalList } = this.state;
    const { modalGoalId, modalGoalVal, closeModal } = this.props;

    return (
      <Modal
        loading={loading}
        title="人数限制"
        width={550}
        onCancel={closeModal}
        onOk={this.save}
        className={style.modal}
      >
        <Row className={style.row}>分公司目标:{modalGoalVal ? modalGoalVal : this.reduceAllGoldVal()}</Row>
        <Row className={style.row}>
          <Col span={12} className={style.col}>
            学习中心名称
          </Col>
          <Col span={12} className={style.col}>
            目标名额
          </Col>
        </Row>
        {campusGoalList.map((campusGoal, index) => {
          const { goalId, campusId, campusName, campusGoalId, campusGoalVal } = campusGoal;
          return (
            <Row className={style.row}>
              <Col span={12} className={style.col}>
                <div>{campusName}</div>
              </Col>
              <Col span={12} className={style.col}>
                <InputNumber
                  validator={{
                    rules: [
                      (value) => {
                        if (value % 1 !== 0 || value < 0) {
                          return '只能填写大于等于 0 的整数';
                        }
                      },
                    ],
                  }}
                  value={campusGoalVal}
                  onChange={(campusGoalVal) => {
                    const { campusGoalList } = this.state;
                    const newCampusGoals = campusGoalList.map((campusGoal, curIndex) => {
                      if (curIndex === index) {
                        return Object.assign({}, campusGoal, { campusGoalVal });
                      } else {
                        return campusGoal;
                      }
                    });
                    this.setState({ campusGoalList: newCampusGoals });
                  }}
                />
              </Col>
            </Row>
          );
        })}
        <Row style={{ fontSize: '14px;', color: '#bbb' }}>
          备注：学习中心名额指该学习中心需要承担该年级科目的目标数量
        </Row>
      </Modal>
    );
  }
}

export default CampusTarget;
