import react from 'react';
import Page from 'COMPONENT/Page';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import SearchBar from 'COMPONENT/SearchBar';
import { loadCommonDict, loadBranchDict } from 'UTIL/dict/';
import List from 'COMPONENT/List';
import Loaders from './loaders';
import { connect } from 'dva';
import InputNumber from 'COMPONENT/InputNumber';
import CampusTarget from './Modal/CampusTarget';
import { joinArrToStr } from 'UTIL/dict';
import AddTarget from './Modal/AddTarget';
import Button from 'COMPONENT/Button';
import styles from './index.less';
import { PROJECT_NAME } from '../../../constants/config';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

const productTypeDict = [
  { val: 'TWO_TEACHER', text: '双师' },
  { val: 'SMALL_CLASS', text: '小班' },
];

class TargetSetting extends react.Component {
  constructor(props) {
    super(props);

    this.state = {
      // 字典
      productYearDict: null,
      productQuarterDict: null,
      branchDict: null,
      gradeDict: null,
      subjectDict: null,
      goalTypeDict: null,

      // 值
      yearId: null,
      quarterId: null,
      brenchId: null,
      productTypeId: PROJECT_NAME === 'twoteacher' ? 'TWO_TEACHER' : 'SMALL_CLASS',
      gradeId: null,
      subjectId: null,
      goalTypeId: null,

      // 列表数据
      listData: [],
      goalTypeStatus: [],

      // 学习中心目标弹窗
      campusGoal: false,
      modalGoalId: null,
      modalGoalVal: null,
      modalBrenchId: null,

      //新增弹窗
      isAddTargetModal: false,
    };

    // 分页
    this.pagination = {
      current: 1,
      pageSize: 10,
      total: null,
    };

    // 列表
    this.columns = [
      { title: '产品年份', dataIndex: 'yearName' },
      { title: '产品季度', dataIndex: 'quarterName' },
      { title: '所属分公司', dataIndex: 'brenchName' },
      { title: '产品类型', dataIndex: 'productTypeName' },
      { title: '年级', dataIndex: 'gradeName' },
      { title: '科目', dataIndex: 'subjectName' },
      {
        title: '科次',
        dataIndex: 'goalVal',
        render: (text, record) => {
          const { goalTypeStatus } = this.state;
          let dom = '';
          goalTypeStatus.forEach((goal) => {
            if (goal.id === record.id && goal.isModify === true) {
              dom = (
                <InputNumber
                  precision={0}
                  value={goal.goalVal}
                  onChange={(goalVal) => {
                    const newGoalTypeStatus = goalTypeStatus.map((goal) => {
                      if (goal.id === record.id) {
                        return Object.assign({}, goal, { goalVal: !goalVal ? null : goalVal });
                      } else {
                        return goal;
                      }
                    });
                    this.setState({ goalTypeStatus: newGoalTypeStatus });
                  }}
                />
              );
            } else if (goal.id === record.id) {
              dom = record.goalVal === null ? '未设置' : record.goalVal;
            }
          });
          return dom;
        },
      },
      {
        title: '操作',
        dataIndex: 'id',
        render: (text, record) => {
          let dom = '';
          const { goalTypeStatus } = this.state;
          const { btnAuths } = this.props;
          goalTypeStatus.forEach((goal) => {
            const { id, isModify } = goal;
            if (id === record.id && isModify === false) {
              dom = (
                <div>
                  {btnAuths.includes('MODIFY_TARGET') && (
                    <span onClick={() => this.updateGoalVal(record)} className={styles.btn}>
                      修改
                    </span>
                  )}
                  {btnAuths.includes('CAMPUS_TARGET') && (
                    <span
                      onClick={() => {
                        this.setState({
                          campusGoal: true,
                          modalGoalId: record.id,
                          modalGoalVal: record.goalVal,
                          modalBrenchId: record.brenchId,
                        });
                      }}
                      className={styles.btn}
                    >
                      设置学习中心
                    </span>
                  )}
                </div>
              );
            } else if (id === record.id && isModify === true) {
              dom = (
                <div>
                  {btnAuths.includes('MODIFY_TARGET') && (
                    <span onClick={() => this.saveGoalVal(record)} className={styles.btn}>
                      保存
                    </span>
                  )}
                  {btnAuths.includes('CAMPUS_TARGET') && (
                    <span
                      onClick={() => {
                        this.setState({
                          campusGoal: true,
                          modalGoalId: record.id,
                          modalGoalVal: record.goalVal,
                          modalBrenchId: record.brenchId,
                        });
                      }}
                      className={styles.btn}
                    >
                      设置学习中心
                    </span>
                  )}
                </div>
              );
            }
          });
          return dom;
        },
      },
    ];
  }

  componentDidMount() {
    this.init();
  }

  init = async () => {
    const [productYearDict, productQuarterDict, branchDict, gradeDict, subjectDict, goalTypeDict] = await Promise.all([
      loadCommonDict('YEAR'),
      loadCommonDict('PRODUCT_QUARTER'),
      loadBranchDict(),
      loadCommonDict('STUDENT_GRADE'),
      loadCommonDict('SUBJECT'),
      loadCommonDict('PLAN_GOAL_TYPE'),
    ]);

    let currentYear = new Date().toString().slice(11, 15);
    let yearId = productYearDict.filter((year) => {
      return year.text === currentYear;
    })[0].val;

    this.setState(
      {
        productYearDict,
        productQuarterDict,
        branchDict,
        gradeDict,
        subjectDict,
        goalTypeDict,
        yearId,
        quarterId: productQuarterDict[0].val,
        brenchId: branchDict[0].val,
      },
      () => {
        this.search();
      },
    );
  };

  updateGoalVal = (record) => {
    const { goalTypeStatus } = this.state;
    const newGoalTypeStatus = goalTypeStatus.map((goal) => {
      if (goal.id === record.id) {
        return Object.assign({}, goal, { isModify: true });
      } else {
        return goal;
      }
    });
    this.setState({ goalTypeStatus: newGoalTypeStatus });
  };

  saveGoalVal = (record) => {
    const { goalTypeStatus } = this.state;
    let newGoalVal = 0;
    const newGoalTypeStatus = goalTypeStatus.map((goal) => {
      if (goal.id === record.id) {
        this.setState({ loading: true });
        newGoalVal = goal.goalVal;
        Loaders.updateByBrenchApi({ goalId: goal.id, goalVal: newGoalVal }).then(
          () => {
            const { listData } = this.state;
            let newListData = listData.map((row) => {
              if (row.id === record.id) {
                return Object.assign({}, row, { goalVal: newGoalVal });
              } else {
                return row;
              }
            });
            this.setState({ listData: newListData, loading: false });
            showMessage('保存成功', 'success');
          },
          () => {
            this.setState({ loading: false });
          },
        );
        return Object.assign({}, goal, { isModify: false });
      } else {
        return goal;
      }
    });
    this.setState({ goalTypeStatus: newGoalTypeStatus });
  };

  onProductYearChange = (yearId) => {
    this.setState({ yearId });
  };

  onProductQuarterChange = (quarterId) => {
    this.setState({ quarterId });
  };

  onBranchChange = (brenchId) => {
    this.setState({ brenchId });
  };

  onProductTypeChange = (productTypeId) => {
    this.setState({ productTypeId });
  };

  onGradeChange = (gradeId) => {
    this.setState({ gradeId });
  };

  onSubjectChange = (subjectId) => {
    this.setState({ subjectId });
  };

  onTargetChange = (goalTypeId) => {
    this.setState({ goalTypeId });
  };

  search = async () => {
    this.setState({ loading: true });
    const { yearId, quarterId, brenchId, productTypeId, gradeId, subjectId, goalTypeId } = this.state;
    const { current: pageNo, pageSize } = this.pagination;
    const params = {
      yearId,
      quarterId,
      brenchId,
      productTypeId,
      gradeId: joinArrToStr(gradeId),
      subjectId: joinArrToStr(subjectId),
      goalTypeId,
      pageNo,
      pageSize,
    };
    try {
      const { listData, total } = await Loaders.getListApi(params);
      const goalTypeStatus = listData.map((goal) => {
        return { id: goal.id, isModify: false, goalVal: goal.goalVal };
      });
      this.pagination = Object.assign({}, this.pagination, { total });
      this.setState({ listData, goalTypeStatus, loading: false });
    } catch (error) {
      this.setState({ loading: false });
    }
  };

  reset = () => {
    const { productYearDict, productQuarterDict, branchDict } = this.state;
    this.setState({
      yearId: productYearDict.filter((year) => {
        return year.text === new Date().toString().slice(11, 15);
      })[0].val,
      quarterId: productQuarterDict[0].val,
      brenchId: branchDict[0].val,
      productTypeId: PROJECT_NAME === 'twoteacher' ? 'TWO_TEACHER' : 'SMALL_CLASS',
      gradeId: null,
      subjectId: null,
      goalTypeId: null,
    });
  };

  onPaginationChange = (pagination) => {
    this.pagination = Object.assign({}, this.pagination, {
      pageSize: pagination.pageSize,
      current: pagination.current,
    });
    this.search();
  };

  closeModal = () => {
    this.setState({ campusGoal: false, isAddTargetModal: false });
  };

  render() {
    const { btnAuths } = this.props;
    const {
      loading,
      yearId,
      quarterId,
      brenchId,
      productTypeId,
      gradeId,
      subjectId,
      goalTypeId,
      productYearDict,
      productQuarterDict,
      branchDict,
      gradeDict,
      subjectDict,
      goalTypeDict,
      listData,
      campusGoal,
      modalGoalId,
      modalGoalVal,
      modalBrenchId,
      isAddTargetModal,
    } = this.state;
    return (
      <Page title="目标设置" loading={loading}>
        <div className={styles.searchBar}>
          <SearchBar onReset={this.reset} onSearch={this.search}>
            <Select
              allowClear={false}
              value={yearId}
              dict={productYearDict}
              placeholder="年份"
              onChange={this.onProductYearChange}
            />
            <Select
              allowClear={false}
              value={quarterId}
              dict={productQuarterDict}
              placeholder="产品季度"
              onChange={this.onProductQuarterChange}
            />
            <Select value={brenchId} dict={branchDict} placeholder="分公司" onChange={this.onBranchChange} />
            <Select
              allowClear={false}
              value={productTypeId}
              dict={productTypeDict}
              placeholder="产品类型"
              onChange={this.onProductTypeChange}
            />
            <MultiSelect value={gradeId} dict={gradeDict} placeholder="年级" onChange={this.onGradeChange} />
            <MultiSelect value={subjectId} dict={subjectDict} placeholder="科目" onChange={this.onSubjectChange} />
            <Select value={goalTypeId} dict={goalTypeDict} placeholder="指标类型" onChange={this.onTargetChange} />
          </SearchBar>
        </div>
        {btnAuths.includes('ADD_TARGET') && (
          <Button
            className={styles.button}
            onClick={() => {
              this.setState({ isAddTargetModal: true });
            }}
          >
            新增
          </Button>
        )}
        <List
          dataSource={listData}
          columns={this.columns}
          pagination={this.pagination}
          onChange={this.onPaginationChange}
        />
        {campusGoal && (
          <CampusTarget
            modalGoalId={modalGoalId}
            modalGoalVal={modalGoalVal}
            modalBrenchId={modalBrenchId}
            closeModal={this.closeModal}
            search={this.search}
          />
        )}
        {isAddTargetModal && (
          <AddTarget
            search={this.search}
            branchDict={branchDict}
            productTypeDict={productTypeDict}
            productYearDict={productYearDict}
            productQuarterDict={productQuarterDict}
            closeModal={this.closeModal}
            gradeDict={gradeDict}
            subjectDict={subjectDict}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    user: state.account.user,
    btnAuths: state.account.btnAuths || [],
  };
};

export default connect(mapStateToProps)(TargetSetting);
