import React from 'react';
import PropTypes from 'prop-types';
import ConfirmByShareWay from 'BLOC/ConfirmByShareWay';
import { desensitizationPhoneNumber } from 'UTIL/common';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { getConfirmingInfo, sendConfirmSmsMessage } from 'SERVICE/audit';
import Loading from 'COMPONENT/Loading';

const { radioPoster, radioMessage } = ConfirmByShareWay;

class AuditListModal extends React.Component {
  constructor(props) {
    super(props);

    let radios = [radioPoster /*, radioMessage*/];

    this.state = {
      loading: false,
      url: '',
      template: undefined,
      messageInfo: [
        { text: `发送号码：` },
        { text: `家长姓名：` },
        {
          text: `发送内容：【星火教育】家长您好！您的学生#学生姓名#正在申请退款，本次退款信息需要您确认，请您点击以下链接按提示完成操作。如有疑问请咨询星火教育#学习中心名字#，咨询电话#学习中心电话#。http://xiao100.cn/#短链接#`,
        },
      ],
      posterInfo: [{ text: `学生：` }, { text: `年级：` }, { text: `学习中心：` }],
      radios,
      tips: '请扫码确认相关信息',
      loadingStatus: true,
      hasData: false,
    };
  }

  static propTypes = {
    title: PropTypes.string,
    contractInfo: PropTypes.shape({
      refundWorkflowId: PropTypes.string.isRequired,
      contractNumber: PropTypes.string,
      customerName: PropTypes.string,
      studentName: PropTypes.string,
      signTime: PropTypes.string,
      campusName: PropTypes.string,
    }),
  };

  // 取消弹框
  onCancel = () => {
    this.props.onCancel();
  };

  // 发送短信
  sendMessage = async () => {
    try {
      const data = await sendConfirmSmsMessage({ refundWorkflowId: this.props.contractInfo.refundWorkflowId });
      if (data.data) {
        this.onCancel();
      } else {
        showMessage(data.resultMessage);
      }
    } catch (e) {
      showMessage(e.text);
    }
  };

  // 电话号码脱敏处理
  photoChange = (photo) => {
    if (photo.length === 11) {
      return `${photo[0]}${photo[1]}${photo[2]}****${photo[7]}${photo[8]}${photo[9]}${photo[10]}`;
    }
    return photo;
  };

  // 获取信息
  getMessageInfo = async () => {
    try {
      const { data } = await getConfirmingInfo({ refundWorkflowId: this.props.contractInfo.refundWorkflowId });
      if (!data) {
        showMessage('暂无数据');
        this.setState({ loadingStatus: false, hasData: false });
        setTimeout(() => {
          this.onCancel();
          GlobalEvent.emit('ONLINE_OFF_CONFIRM_REFRESH');
        }, 2000);
      } else {
        this.setState({
          messageInfo: [
            { text: `发送号码：${this.photoChange(data.contact)}` },
            { text: `家长姓名：${data.studentName ? `${data.studentName}家长` : ''}` },
            {
              text: `发送内容：【星火教育】家长您好！您的学生#学生姓名#正在申请退款，本次退款信息需要您确认，请您点击以下链接按提示完成操作。如有疑问请咨询星火教育#学习中心名字#，咨询电话#学习中心电话#。http://xiao100.cn/#短链接#`,
            },
          ],
          posterInfo: [
            { text: `学生：${data.studentName}` },
            { text: `年级：${data.gradeName}` },
            { text: `学习中心：${data.campusName}` },
          ],
          url: data.confirmUrl,
          loadingStatus: false,
          hasData: true,
        });
      }
    } catch (e) {
      this.setState({ loadingStatus: false, hasData: false });
      showMessage(e.text);
      setTimeout(() => {
        this.onCancel();
        GlobalEvent.emit('ONLINE_OFF_CONFIRM_REFRESH');
      }, 2000);
    }
  };

  componentDidMount() {
    this.getMessageInfo();
  }

  render() {
    const { hasData, loadingStatus } = this.state;
    return hasData ? (
      <ConfirmByShareWay
        title={this.props.title}
        loading={this.state.loading}
        onCancel={this.onCancel}
        radios={this.state.radios}
        link={{
          url: this.state.url,
        }}
        message={{
          infos: this.state.messageInfo,
          template: this.state.template,
          onSend: this.sendMessage,
        }}
        poster={{
          infos: /*this.state.posterInfo*/ [],
          url: this.state.url,
          tips: this.state.tips,
        }}
      />
    ) : (
      <Loading loading={loadingStatus} tip="加载中..." />
    );
  }
}

export default AuditListModal;
