import React from 'react';
import file from 'UTIL/file';
import PropTypes from 'prop-types';
import styles from './styles.module.less';
import { confirmRefund, getConfirmingNo } from 'SERVICE/audit';
import { getBankCardInfo } from 'SERVICE/common';
import { searchBankBranchLoader } from 'PAGE/common/loaders';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import Loading from 'COMPONENT/Loading';
import { debounce } from 'lodash';
import CommonInput from './CommonInput';

import { Modal } from 'COMPONENT';
import { Button, Form, Input, AutoComplete } from 'antd';
// import UploadImg from 'BLOC/UploadImg';
import BankTooltip from 'BLOC/BankTooltip';

const FormItem = Form.Item;
const Option = AutoComplete.Option;

const formItemLayout = {
  labelCol: { span: 6 },
  wrapperCol: { span: 18 },
};

const formatBankNumber = (cardValue, length = 31) => {
  cardValue = cardValue.replace(/[^\d]/g, '');
  const cardList = [];

  while (cardValue.length) {
    cardList.push(cardValue.slice(0, 4));
    cardValue = cardValue.slice(4);
  }

  return length == null ? cardList.join(' ') : cardList.join(' ').substring(0, length);
};

class OnlineOffConfirm extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      refundAccount: '',
      refundAccName: '',
      bankInfo: '',
      branchCode: '', // 支行联行号
      branchBankList: [],
      fileKey: '',
      confirmNo: '',
      fileObj: null,
      loadingStatus: true,
      contractInfo: {},
    };
    this.isComposing = false;
    this.onBranchBankSelect = this.onBranchBankSelect.bind(this);
    this.searchBankInfo = debounce(this.searchBankInfo, 300).bind(this);
    this.onAccountChange = this.onAccountChange.bind(this);
    this.onSearchChange = this.onSearchChange.bind(this);
    this.checkBankSelect = this.checkBankSelect.bind(this);
  }

  beforeUpload = (fileObj) => {
    if (!fileObj[0].type) return showMessage('请重新上传图片');
    file.onUploadFile(fileObj, this.aliOssImage);
    return fileObj;
  };

  handleUploadComplete = (val) => {
    this.setState({
      fileObj: val,
    });
  };

  handleDeleteComplete = () => {
    this.setState({
      fileObj: null,
    });
  };

  aliOssImage = (xhr, key) => {
    if (xhr) {
      this.setState({
        fileKey: key,
      });
    } else {
      showMessage('请重新上传图片');
    }
  };

  findBankInfo = async (e) => {
    const val = e.target.value.trim();
    if (val === '' || val === null) return false;

    try {
      const { resultCode, data } = await getBankCardInfo({
        bankCardNo: val.replaceAll(' ', ''),
        isFromAts: true /** 获取ATS数据源 */,
      });
      if (resultCode === 0) {
        const { openBranchCode, accountOpeningBranch } = data;
        let payload = {};
        if (openBranchCode && accountOpeningBranch) {
          payload = {
            bankInfo: accountOpeningBranch,
            branchCode: openBranchCode,
            branchBankList: [{ label: accountOpeningBranch, value: openBranchCode }],
          };
        } else {
          payload = {
            branchBankList: [],
          };
        }
        this.setState(payload);
      }
    } catch (e) {
      this.setState({
        branchBankList: [],
      });
      showMessage(e.text);
    }
  };

  onAccountChange(e) {
    this.setState({
      refundAccount: formatBankNumber(e.target.value.trim()),
    });
  }

  async searchBankInfo(query) {
    if (query.length >= 2) {
      try {
        const branchBankList = await searchBankBranchLoader(query);
        this.setState({ branchBankList });
      } catch (e) {
        showMessage(e, 'error');
      }
    }
  }

  onBranchBankSelect(label) {
    this.matchBank(label);
  }

  matchBank(text) {
    const { branchBankList } = this.state;
    let item;
    for (let i = 0, ii = branchBankList.length; i < ii; i++) {
      if (branchBankList[i].label === text) {
        item = branchBankList[i];
        break;
      }
    }
    if (item) {
      this.setState({ branchCode: item.value || '' });
    } else {
      this.setState({ branchCode: '', bankInfo: '' });
    }
  }

  checkBankSelect(e) {
    e.persist();
    let text = e.target.value.trim(/\s+/g, '');
    if (text) {
      this.matchBank(text);
    } else {
      this.setState({ bankInfo: '', branchCode: '' });
    }
  }

  onSearchChange(val) {
    this.setState({ bankInfo: val });
  }

  onChangeRefundAccName = (refundAccName) => {
    this.setState({ refundAccName: refundAccName?.trim()?.replace(/[^\u4e00-\u9fff]/g, '') });
  };

  initData = async () => {
    try {
      const { data } = await getConfirmingNo({ refundWorkflowId: this.props.contractInfo.refundWorkflowId });
      let loadingStatus = true;
      if (data.confirmNo) {
        loadingStatus = false;
      } else {
        showMessage('确认记录编码为空');
        setTimeout(() => {
          this.closePanel();
          GlobalEvent.emit('ONLINE_OFF_CONFIRM_REFRESH');
        }, 2000);
      }
      const { bankInfo, branchCode, refundAccount = '', refundAccName } = data;
      this.setState({
        branchCode,
        confirmNo: data.confirmNo,
        loadingStatus,
        bankInfo,
        refundAccName,
        refundAccount: formatBankNumber(refundAccount || ''),
        contractInfo: {
          bankInfo,
          refundAccount,
          refundAccName,
          totalAmount: data.totalAmount,
        },
      });
    } catch (e) {
      this.setState({
        loadingStatus: false,
        confirmNo: false,
        bankInfo: '',
        refundAccount: '',
        refundAccName: '',
        contractInfo: {
          ...this.props.contractInfo,
          totalAmount:
            parseFloat(this.props.contractInfo?.basicOperateAmount || '0') +
            parseFloat(this.props.contractInfo?.specialOperateAmount || '0'),
        },
      });
      showMessage(e.text);
      setTimeout(() => {
        this.closePanel();
        GlobalEvent.emit('ONLINE_OFF_CONFIRM_REFRESH');
      }, 2000);
    }
  };

  submitOnline = async () => {
    try {
      const { refundAccount = '', refundAccName, bankInfo, branchCode, confirmNo } = this.state;

      if (!confirmNo) {
        return showMessage('确认记录编码为空');
      } else if (!refundAccount.replace(/\s/g, '')) {
        return showMessage('请输入银行卡账号');
      } else if (refundAccount.replace(/\s/g, '').length < 15 || refundAccount.replace(/\s/g, '').length > 25) {
        return showMessage('请输入正确银行卡账号(长度大于15，小于26)');
      } else if (!refundAccName.trim()) {
        return showMessage('请输入银行卡账号名');
      } else if (!branchCode) {
        return showMessage('请输入搜索选择银行卡支行信息');
      }

      await confirmRefund({
        refundAccount: refundAccount.replace(/\s/g, ''),
        refundAccName,
        bankInfo,
        branchCode,
        confirmNo: confirmNo,
      });
      GlobalEvent.emit('ONLINE_OFF_CONFIRM_REFRESH');

      this.props.onReload && this.props.onReload(); // 重新加载
      this.closePanel();
    } catch (e) {
      showMessage(e.text);
    }
  };

  closePanel = () => {
    this.props.onCancel();
  };

  componentDidMount() {
    this.initData();
  }

  render() {
    const { confirmNo, loadingStatus, branchBankList, bankInfo } = this.state;
    const dataSource = branchBankList.map((item) => (
      <Option value={item.label} key={item.value}>
        {item.label}
      </Option>
    ));

    return confirmNo ? (
      <Modal
        title="线下确认"
        width={580}
        loading={this.state.loading}
        onCancel={this.closePanel}
        footer={
          <div>
            <Button type="primary" onClick={() => this.submitOnline()}>
              提交
            </Button>
            <Button onClick={this.closePanel}>关闭</Button>
          </div>
        }
      >
        <div className={styles.modalPanel}>
          <Form className={styles.modalPanelForm}>
            <FormItem {...formItemLayout} label={<span>退费金额</span>}>
              <div style={{ color: 'red' }}>{this.state.contractInfo.totalAmount}</div>
            </FormItem>
            <FormItem
              {...formItemLayout}
              label={
                <span>
                  退费账户名<b style={{ color: 'red', fontWeight: 'normal' }}>*</b>
                </span>
              }
            >
              {/*<div>{this.state.contractInfo?.refundAccName}</div>*/}
              <CommonInput
                placeholder="请输入退费账户名"
                maxLength="30"
                value={this.state.refundAccName}
                onChange={this.onChangeRefundAccName}
              />
            </FormItem>
            <FormItem
              {...formItemLayout}
              label={
                <span>
                  退费账号<b style={{ color: 'red', fontWeight: 'normal' }}>*</b>
                </span>
              }
            >
              {/*<div>{this.state.contractInfo?.refundAccount}</div>*/}
              <Input
                maxLength="31"
                placeholder="请输入退费账号"
                value={this.state.refundAccount}
                onChange={this.onAccountChange}
                onBlur={(e) => {
                  this.findBankInfo(e);
                }}
              />
            </FormItem>
            <FormItem
              {...formItemLayout}
              label={
                <span>
                  银行支行信息<b style={{ color: 'red', fontWeight: 'normal' }}>*</b>
                </span>
              }
            >
              {/*<div>{this.state.contractInfo?.bankInfo}</div>*/}
              <AutoComplete
                value={bankInfo}
                dataSource={dataSource}
                onSelect={this.onBranchBankSelect}
                onSearch={this.searchBankInfo}
                onChange={this.onSearchChange}
              >
                <Input size="large" onBlur={this.checkBankSelect} placeholder="请输入搜索银行支行信息" maxLength="50" />
              </AutoComplete>
              <BankTooltip placement="top" />
            </FormItem>
          </Form>
          <div style={{ color: 'red' }}>注：家长不方便线上确认时，校区可以通过线下沟通的方式代替家长进行退费确认</div>
        </div>
      </Modal>
    ) : (
      <Loading loading={loadingStatus} tip="加载中..." />
    );
  }
}

export default OnlineOffConfirm;
