import React, { PropTypes } from 'react';
import { debounce } from 'lodash';
import SearchBar from 'COMPONENT/SearchBar';
import { Select, Input, DatePicker } from 'COMPONENT';
import { statusEnum } from '../constants';
import { loadBranchCampusDict } from 'UTIL/dict';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { getUserForPerformance } from 'SERVICE/contract';

const getSearchInitValue = () => {
  return {};
};

class Search extends React.Component {
  static propTypes = {
    onSearch: PropTypes.func.isRequired,
  };

  constructor(props) {
    super(props);
    this.state = {
      attendanceList: [],
      campusList: [],
      searchParams: getSearchInitValue(),
    };
  }

  componentDidMount() {
    this.fetchAttendanceList();
    this.fetchUser();
    this.fetchComplusOptinons();
  }

  fetchUser = async (val) => {
    if (val && val.length > 1) {
      try {
        const dict = await getUserForPerformance({ term: val });
        this.setState({
          userList: dict.map((item) => ({ text: item.label, val: item.value, title: item.label })),
        });
      } catch (e) {}
    }
  };

  fetchComplusOptinons = async () => {
    try {
      const campusList = await loadBranchCampusDict();
      this.setState({ campusList });
    } catch (e) {
      showMessage('获取基础数据失败');
    }
  };

  fetchAttendanceList = () => {
    const attendanceList = Object.entries(statusEnum).map(([key, value]) => {
      return {
        val: key,
        text: value,
      };
    });
    this.setState({
      attendanceList,
    });
  };

  setSearchParams = (key, value) => {
    this.setState({
      searchParams: {
        ...this.state.searchParams,
        [key]: value,
      },
    });
  };

  resetSearchParams = () => {
    this.setState(
      {
        searchParams: getSearchInitValue(),
      },
      () => this.props.onSearch(this.state.searchParams),
    );
  };

  render() {
    const {
      searchParams,
      attendanceList,
      campusList,
      userList,
      searchParams: { applicantTimeStart, applicantTimeEnd, className, createUserId, campusId, approveStatus },
    } = this.state;

    return (
      <SearchBar onSearch={() => this.props.onSearch(searchParams)} onReset={this.resetSearchParams}>
        <DatePicker
          placeholder="发起开始日期"
          value={applicantTimeStart}
          onChange={(value) => {
            this.setSearchParams('applicantTimeStart', value);
          }}
        />
        <DatePicker
          placeholder="发起结束日期"
          disabledDate={(d) => {
            if (applicantTimeStart && d) {
              let start = new Date(applicantTimeStart).getTime();
              let end = new Date(d).getTime();
              return end < start;
            }
          }}
          value={applicantTimeEnd}
          onChange={(value) => {
            this.setSearchParams('applicantTimeEnd', value);
          }}
        />
        <Input
          placeholder="请输入课程名称"
          value={className}
          onChange={(e) => {
            this.setSearchParams('className', e.target.value);
          }}
        />
        <Select
          value={createUserId}
          placeholder="发起人"
          allowClear
          dict={userList}
          showSearch={true}
          searchPlaceholder="输入id搜索"
          onSearchKeyChange={debounce(this.fetchUser, 300)}
          onChange={(value) => {
            this.setSearchParams('createUserId', value);
          }}
        />
        <Select
          placeholder="上课学习中心"
          dict={campusList}
          value={campusId}
          onChange={(value) => {
            this.setSearchParams('campusId', value);
          }}
        />
        <Select
          placeholder="审核状态"
          dict={attendanceList}
          value={approveStatus}
          onChange={(value) => {
            this.setSearchParams('approveStatus', value);
          }}
        />
      </SearchBar>
    );
  }
}

export default Search;
