import React from 'react';
import { Page, List } from 'COMPONENT';
import Search from './Search';
import { TABLE_COLUMS } from './constants';
import { isString } from 'UTIL/lang/data-type';
import { getAttendanceAuditList } from 'SERVICE/audit';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

class SmallClassAttendanceApprove extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      tableData: [],
      pagination: {
        total: 0,
        current: 1,
        pageSize: 10,
      },
      searchParams: {},
    };
  }

  componentDidMount() {
    this.fetchAttendanceList();
  }

  onPageChange = (data) => {
    let pagination = this.state.pagination;
    let pageParams = Object.keys(pagination).reduce((pre, cur) => {
      pre[cur] = data[cur];
      return pre;
    }, {});

    this.setState(
      {
        pagination: pageParams,
      },
      this.fetchAttendanceList,
    );
  };

  onSearch = (searchParams) => {
    this.setState(
      {
        searchParams,
      },
      this.fetchAttendanceList,
    );
  };

  /**
   * @description: 请求审批列表
   */
  fetchAttendanceList = async () => {
    const { current: pageNum, pageSize } = this.state.pagination;
    let { searchParams } = this.state;

    // 输入框字段为空时不传该参数
    // 时间需要传时间戳
    searchParams = Object.keys(searchParams).reduce((pre, cur) => {
      if (this.formatValue(searchParams[cur])) {
        if (/(applicantTimeStart)|(applicantTimeEnd)/i.test(cur)) {
          pre[cur] = new Date(searchParams[cur]).getTime();
        } else {
          pre[cur] = searchParams[cur];
        }
      }
      return pre;
    }, {});

    let queryParams = {
      ...searchParams,
      pageSize,
      pageNum,
    };

    try {
      this.setRequestLoading(true);
      const res = await getAttendanceAuditList(queryParams);
      const { list, total } = res.data;

      this.setState({
        tableData: list || [],
        pagination: {
          ...this.state.pagination,
          total: total,
        },
      });
    } catch (e) {
      showMessage(e.text);
      this.setState({
        tableData: [],
        pagination: {
          ...this.state.pagination,
          total: 0,
        },
      });
    } finally {
      this.setRequestLoading(false);
    }
  };

  formatValue = (value) => {
    if (isString(value)) {
      return value.trim();
    }
    return value;
  };

  /**
   * @description 设置列表请求Loading状态
   * @param  {boolean} loading
   * @return {void}
   */
  setRequestLoading = (loading) => {
    this.setState({
      loading,
    });
  };

  render() {
    const { route } = this.props;
    const { loading, tableData, pagination } = this.state;

    return (
      <Page
        name={route.name}
        route={route}
        fitWindowHeight={true}
        subHeader={
          <div>
            <Search onSearch={this.onSearch} />
            <List
              dataSource={tableData}
              loading={loading}
              columns={TABLE_COLUMS}
              pagination={pagination}
              onChange={this.onPageChange}
            />
          </div>
        }
      ></Page>
    );
  }
}

export default SmallClassAttendanceApprove;
