import React from 'react';
import { Page, Wrapper, SearchBar, Select, Input, List, Modal, SearchFilter, SearchField } from 'COMPONENT';
import { loadBranchDict, loadBranchCampusDict, dealCampus } from 'UTIL/dict';
import { DatePicker, message, Button, Cascader } from 'antd';
import moment from 'moment';
import { connect } from 'dva';
import {
  pageRecordsApi,
  getClassroomResponseByCampusApi,
  exportRecordsApi,
  getClassroomResponseByOrgApi,
} from './loaders';
import PreviewImgs from 'COMPONENT/PreviewImgs';
import styles from './styles.module.less';

const { RangePicker } = DatePicker;
const dateFormat = 'YYYY/MM/DD';

class CaptureRecord extends React.Component {
  constructor(props) {
    super(props);
    let tempState = {
      loading: false,
      dataSource: [],
      campusDict: [], // 学习中心
      pageNum: 1,
      total: 0,
      pageSize: 20,
      imgShow: false,
      arrayList: [], // 预览图片
      currImage: '',
      resultDict: [
        { val: 0, text: '成功' },
        { val: 1, text: '失败' },
      ],
      takePhotoTypeDict: [
        { val: 'COMMON', text: '正式抓拍' },
        { val: 'BEFORE_TEST', text: '课前试拍' },
      ],
    };

    let time = new Date(new Date().getTime() - 24 * 60 * 60 * 1000);
    let startYear = time.getFullYear();
    let startMonth = time.getMonth() + 1;
    let startDay = time.getDate();

    this.columns = [
      { title: '设备号', dataIndex: 'deviceNo' },
      { title: '通道号', dataIndex: 'channelNo' },
      { title: '验证码', dataIndex: 'validateCode' },
      {
        title: '抓拍类型',
        dataIndex: 'takePhotoType',
        render: (current) => {
          if (current === 'COMMON') {
            return '正式抓拍';
          } else if (current === 'BEFORE_TEST') {
            return '课前试拍';
          }
        },
      },
      {
        title: '抓拍任务创建时间',
        dataIndex: 'createDtTime',
        render: (current) => {
          return CaptureRecord.getTimeStamp(true, current);
        },
      },
      { title: '分公司', dataIndex: 'branchName' },
      { title: '学习中心', dataIndex: 'blCampusName' },
      { title: '教室', dataIndex: 'classroomName' },
      { title: '小班名称', dataIndex: 'className' },
      { title: '上课时间', dataIndex: 'courseTime' },
      {
        title: '抓拍结果',
        dataIndex: 'result',
        render: (current) => {
          return +current === 0 ? '成功' : '失败';
        },
      },
      {
        title: '抓拍时间',
        dataIndex: 'captureTime',
        render: (current) => {
          return CaptureRecord.getTimeStamp(true, current);
        },
      },
      {
        title: '抓拍照片',
        dataIndex: 'imgUrls',
        render: (current) => {
          let tempImageList = current ? JSON.parse(current) : [];
          if (tempImageList.length) {
            return <a onClick={() => this.showImgPanel(tempImageList)}>查看图片</a>;
          } else {
            return <span>暂无抓拍图片</span>;
          }
        },
      },
      {
        title: '是否复用照片',
        dataIndex: 'reuse',
        render: (current) => {
          return Number(current) === 1 ? '是' : '否';
        },
      },
      { title: '延时（秒）', dataIndex: 'delayInterval' },
      { title: '错误码', dataIndex: 'errCode' },
      { title: '错误信息', dataIndex: 'errMsg' },
      { title: '重试次数', dataIndex: 'retries' },
      {
        title: '重试结果',
        dataIndex: 'retryResult',
        render: (current) => {
          let tempCurrent = +current;
          if (tempCurrent === 0) {
            return '未重试';
          } else if (tempCurrent === 1) {
            return '重试成功';
          } else if (tempCurrent === 2) {
            return '重试失败';
          }
        },
      },
      {
        title: '是否放弃重试',
        dataIndex: 'giveUpFlag',
        render: (current) => {
          return Number(current) === 1 ? '是' : '否';
        },
      },
      { title: '放弃重试原因', dataIndex: 'giveUpReasonDesc' },
      {
        title: '操作',
        dataIndex: 'classroomId',
        render: (current, record, index) => {
          return (
            <div>
              <a onClick={() => this.goToClassRoom(current)}>查看教室详情</a>
              <a onClick={() => this.goToMoritor(current, record, index)}>查看监控详情</a>
            </div>
          );
        },
      },
    ];

    // 导出的搜索条件
    this.exportSelect = {};
    // 搜索框默认值
    this.defaultSelectValue = {
      beginCreateTime: CaptureRecord.getTimeStamp(false, startYear, startMonth, startDay),
      endCreateTime: CaptureRecord.getTimeStamp(
        false,
        new Date().getFullYear(),
        new Date().getMonth() + 1,
        new Date().getDate(),
      ),
      defaultBeginCreateTime: CaptureRecord.getTimeStamp(false, startYear, startMonth, startDay),
      defaultEndCreateTime: CaptureRecord.getTimeStamp(
        false,
        new Date().getFullYear(),
        new Date().getMonth() + 1,
        new Date().getDate(),
      ),
      blCampusIds: [], // 学习中心id
      classroomId: '', // 教室id
      result: '', // 抓拍结果
      className: '', // 小班课室名称
      deviceNo: '', // 设备号
      validateCode: '', // 验证码
      channelNo: '', // 通道号
      takePhotoType: '', // 拍照类型
      classRoomDict: [], // 教室
    };
    this.state = {
      ...this.defaultSelectValue,
      ...tempState,
    };
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * @param {boolean=false} isShowTime 是否展示时分秒
   * @param year
   * @param month
   * @param day
   * @description 获取某个日期的时间戳
   * **/
  static getTimeStamp(isShowTime, year, month, day) {
    let time = new Date();
    let timeArr;
    if (!isShowTime) {
      timeArr = [year || time.getFullYear(), month || time.getMonth() + 1, day || time.getDate()];
      return `${timeArr[0]}-${CaptureRecord.noTenAddZero(timeArr[1])}-${CaptureRecord.noTenAddZero(timeArr[2])}`;
    } else {
      timeArr = new Date(year);
      return `${timeArr.getFullYear()}-${CaptureRecord.noTenAddZero(
        timeArr.getMonth() + 1,
      )}-${CaptureRecord.noTenAddZero(timeArr.getDate())} ${CaptureRecord.noTenAddZero(
        timeArr.getHours(),
      )}:${CaptureRecord.noTenAddZero(timeArr.getMinutes())}:${CaptureRecord.noTenAddZero(timeArr.getSeconds())}`;
    }
  }

  // 不逢十补零
  static noTenAddZero(val) {
    return val > 9 ? val : `0${val}`;
  }

  // 展示图片预览
  showImgPanel = (imgList) => {
    this.setState({
      arrayList: imgList,
      currImage: imgList[0],
      imgShow: true,
    });
  };

  // 跳转到教室管理
  goToClassRoom = (classroomId) => {
    if (!classroomId) return message.error('教室id为空');
    this.props.history.push({
      pathname: '/ClassRoomManagement',
      query: { classroomId: classroomId },
    });
  };

  // 跳转到监控详情
  goToMoritor = (classroomId, record, index) => {
    const { deviceNo, channelNo, validateCode, branchId, blCampusId } = record;
    const { dispatch } = this.props;
    if (!classroomId) return message.error('教室id为空');
    sessionStorage.setItem(
      '/classroomMonitor/manage',
      JSON.stringify({
        deviceNo,
        channelNo,
        validateCode,
        branchId,
        campusId: blCampusId,
        actionType: 'INIT_SEARCH',
      }),
    );
    dispatch({
      type: 'main/openPage',
      payload: {
        path: '/next-compact',
        refresh: true,
        title: '教室监控绑定管理',
        query: {
          path: '/xh-extra/classroomMonitor/manage',
          title: '教室监控绑定管理',
          timestamp: Date.now(),
        },
      },
    });
  };

  // 搜索
  onSearch = async (otherParams) => {
    if (!this.state.beginCreateTime && !this.state.endCreateTime) {
      return message.error('请先选择时间');
    }
    const [_branchId, _campusId] = this.state.blCampusIds || [];
    this.exportSelect = {
      courseDateStart: this.state.beginCreateTime,
      courseDateEnd: this.state.endCreateTime,
      blCampusId: _campusId || _branchId,
      classroomId: this.state.classroomId,
      result: this.state.result,
      className: this.state.className,
      deviceNo: this.state.deviceNo,
      validateCode: this.state.validateCode,
      channelNo: this.state.channelNo,
      takePhotoType: this.state.takePhotoType,
    };
    let tempParams = {
      ...this.exportSelect,
      pageSize: this.state.pageSize,
      pageNum: this.state.pageNum,
    };
    if (otherParams) {
      this.setState({
        ...otherParams,
      });
    }
    await this.getListData(Object.assign(tempParams, otherParams));
  };

  // 重置
  onReset = async () => {
    this.exportSelect = {};
    this.setState({
      ...this.defaultSelectValue,
      pageSize: 20,
      pageNum: 1,
    });
  };

  /**
   * @param {Object?} params 参数
   * @description 获取列表数据
   * **/
  getListData = async (params) => {
    let data = await pageRecordsApi(params || { pageSize: 20, pageNum: 1 });
    this.setState({
      dataSource: data.list,
      total: data.total,
    });
  };

  // 获取学习中心数据
  // 从select数据转化为cascader格式
  async loadDict() {
    const [allCampusDict = []] = await Promise.all([loadBranchCampusDict()]);

    const handleOptions2Children = (options = []) => {
      if (!Array.isArray(options)) return [];
      return options.map((option) => ({
        value: option.val,
        label: option.label || option.text,
        children:
          Array.isArray(option.options) && option.options.length > 0
            ? handleOptions2Children(option.options)
            : undefined,
      }));
    };
    const cascaderCampusDict = handleOptions2Children(allCampusDict);
    this.setState({
      campusDict: cascaderCampusDict,
    });
  }

  // 日期修改，antd本身的问题，选择当天会返回两个一样的时间戳
  timeOnchange = (date) => {
    if (date.length) {
      let tempStartTime, tempEndTime;
      tempStartTime = `${date[0]._d.getFullYear()}-${CaptureRecord.noTenAddZero(
        date[0]._d.getMonth() + 1,
      )}-${CaptureRecord.noTenAddZero(date[0]._d.getDate())}`;
      tempEndTime = `${date[1]._d.getFullYear()}-${CaptureRecord.noTenAddZero(
        date[1]._d.getMonth() + 1,
      )}-${CaptureRecord.noTenAddZero(date[1]._d.getDate())}`;
      this.setState({
        beginCreateTime: tempStartTime,
        endCreateTime: tempEndTime,
      });
    } else {
      this.setState({
        beginCreateTime: null,
        endCreateTime: null,
      });
    }
  };

  // 筛选框修改
  wrapInputOnchange = (key, value) => {
    this.setState({ [key]: value });
    if (key === 'blCampusIds') {
      const [_branchId, _campusId] = value || [];
      const _id = _campusId || _branchId;
      _id ? this.getClassRoomList(_id) : this.setState({ classRoomDict: [], classroomId: '' });
    }
  };

  // 初始化
  initContent = async () => {
    this.setState({ loading: true });
    await this.loadDict();
    await this.onSearch();
    this.setState({ loading: false });
  };

  // 获取教室列表
  getClassRoomList = async (campusId) => {
    // let data = await getClassroomResponseByCampusApi({campusId: campusId});
    // this.setState({
    //   classRoomDict: data
    // });
    const classRoomWithCampusGroup = await getClassroomResponseByOrgApi({ organizationId: campusId });
    // 后台不提供学习中心名字，只能从另一个学习中心列表接口数据拼接
    const searchForCampus = (allCampus = [], campusId) => {
      for (const campus of allCampus) {
        if (campus.value === campusId) {
          return { id: campusId, name: campus.label };
        } else if (Array.isArray(campus.children)) {
          const theOne = searchForCampus(campus.children, campusId);
          if (theOne) {
            return theOne;
          }
        }
      }
      return undefined;
    };
    Array.isArray(this.state.campusDict) &&
      Array.isArray(classRoomWithCampusGroup) &&
      classRoomWithCampusGroup.forEach((campus) => {
        const _campus = searchForCampus(this.state.campusDict, campus.campusId);
        if (_campus) {
          campus.label = _campus.name;
        }
      });
    this.setState({
      classRoomDict: classRoomWithCampusGroup,
    });
  };

  // 导出文件
  exportFile = async () => {
    if (this.state.total === 0) {
      return message.error('暂无数据导出');
    } else if (this.state.total > 5000) {
      return message.error('最多只能导出5000份数据');
    }
    let data = await exportRecordsApi({ ...this.exportSelect, pageSize: this.state.total });
    window.open(data);
  };

  render() {
    const {
      loading,
      dataSource,
      campusDict,
      classRoomDict,
      pageNum,
      total,
      pageSize,
      beginCreateTime,
      endCreateTime,
      defaultBeginCreateTime,
      defaultEndCreateTime,
      blCampusIds,
      classroomId,
      resultDict,
      result,
      className,
      imgShow,
      deviceNo,
      validateCode,
      channelNo,
      arrayList,
      currImage,
      takePhotoType,
      takePhotoTypeDict,
    } = this.state;
    return (
      <Page
        loading={loading}
        title="摄像头抓拍记录"
        headerToolBar={
          <Button size="large" onClick={this.exportFile}>
            导出
          </Button>
        }
        subHeader={
          <SearchFilter onSearch={() => this.onSearch({ pageNum: 1 })} onReset={this.onReset} truncationNum={5}>
            <SearchField span={2}>
              <RangePicker
                size="large"
                defaultValue={[moment(defaultBeginCreateTime, dateFormat), moment(defaultEndCreateTime, dateFormat)]}
                value={[
                  beginCreateTime ? moment(beginCreateTime, dateFormat) : null,
                  endCreateTime ? moment(endCreateTime, dateFormat) : null,
                ]}
                onChange={this.timeOnchange}
              />
            </SearchField>
            <SearchField span={1}>
              <Cascader
                size={'large'}
                options={campusDict}
                allowClear
                value={blCampusIds}
                placeholder="请选择学习中心"
                expandTrigger="hover"
                changeOnSelect
                displayRender={(label) => {
                  const [branchName, campusName] = label || [];
                  return campusName || branchName || '';
                }}
                onChange={(e) => this.wrapInputOnchange('blCampusIds', e)}
                showSearch
              />
            </SearchField>
            <SearchField span={1}>
              <Select
                dict={classRoomDict}
                value={classroomId}
                placeholder="请选择上课教室"
                onChange={(e) => this.wrapInputOnchange('classroomId', e)}
                showSearch={true}
              />
            </SearchField>
            <SearchField span={1}>
              <Select
                dict={resultDict}
                value={result}
                placeholder="请选择抓拍结果"
                onChange={(e) => this.wrapInputOnchange('result', e)}
              />
            </SearchField>
            <SearchField span={1}>
              <Input
                value={className}
                placeholder="小班名称"
                onChange={(e) => this.wrapInputOnchange('className', e.target.value)}
              />
            </SearchField>
            <SearchField span={1}>
              <Input
                value={deviceNo}
                placeholder="设备号"
                onChange={(e) => this.wrapInputOnchange('deviceNo', e.target.value)}
              />
            </SearchField>
            <SearchField span={1}>
              <Input
                value={validateCode}
                placeholder="验证码"
                onChange={(e) => this.wrapInputOnchange('validateCode', e.target.value)}
              />
            </SearchField>
            <SearchField span={1}>
              <Input
                value={channelNo}
                placeholder="通道号"
                onChange={(e) => this.wrapInputOnchange('channelNo', e.target.value)}
              />
            </SearchField>
            <SearchField span={1}>
              <Select
                dict={takePhotoTypeDict}
                value={takePhotoType}
                placeholder="抓拍类型"
                onChange={(e) => this.wrapInputOnchange('takePhotoType', e)}
                showSearch={true}
              />
            </SearchField>
          </SearchFilter>
        }
      >
        <List
          dataSource={dataSource}
          pagination={{
            current: pageNum,
            total,
            pageSize,
          }}
          columns={this.columns}
          onChange={({ current, pageSize }) => {
            this.onSearch({ pageNum: current, pageSize });
            this.setState({ pageNum: current, pageSize });
          }}
        />
        <Modal
          width={800}
          okBtnShow={false}
          cancelBtnShow={false}
          closable={true}
          toggle={true}
          visible={imgShow}
          onClose={() => this.setState({ imgShow: false })}
        >
          {imgShow && <PreviewImgs list={arrayList} activeSrc={currImage} imageClass={styles.imageClass} />}
        </Modal>
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    contractBlCampusId: state.account.user.campusId,
  };
};

export default connect(mapStateToProps)(CaptureRecord);
