import React from 'react';
import { Modal, FormItem, Input, Select, InputNumber, Button, TextArea } from 'COMPONENT';
import { storeName, AddEditModalTypes } from '../store/config';
import { connect } from 'dva';
import { Row, Col } from 'antd';
import UploadImg from 'BLOC/UploadImg';
import { statusDict, subStatusDict, statusMap } from '../dicts';
import style from './style.less';
import ImagePreviewModal from '../../modals/ImagePreview';
import { getAuditMsgForDetailChange } from 'SERVICE/basicInformationManage.js';
import { requestWrapper } from 'UTIL/promiseHandler';
import moment from 'moment';
const ModalFooterBtn = (props) => {
  const { onOk, onCancel, confirmBtnDisabled, isViewOnly, isAddMode, isGroupAdmin } = props;
  return (
    <div style={{ display: 'flex', justifyContent: 'flex-end' }}>
      {!isViewOnly && (
        <Button type="primary" onClick={onOk} disabled={confirmBtnDisabled}>
          {isAddMode || isGroupAdmin ? '保存' : '提交修改申请'}
        </Button>
      )}
      <Button onClick={onCancel}>{isViewOnly ? '关闭' : '取消'}</Button>
    </div>
  );
};

const initialState = {
  imagePreviewModalVisible: false,
  imagePreviewSrc: '',
  loading: false,
  approvalUsersDict: [],
  nodeId: '',
  auditUserId: '',
  reason: '',
};

class AddEditModal extends React.Component {
  state = {
    ...initialState,
  };

  componentDidMount() {
    if (this.props.CLASSROOM_CAMPUS_ADMIN && this.props.addEditModalType === AddEditModalTypes.Edit) {
      this.onStatusChange(this.props.status);
    }
  }

  close = () => {
    this.setState({
      ...initialState,
    });
    this.props.dispatch({
      type: `${storeName}/closeAddEditModal`,
    });
  };

  ok = () => {
    this.props.dispatch({
      type: `${storeName}/onAddEditModalOk`,
      payload: {
        nodeId: this.state.nodeId,
        auditUserId: this.state.auditUserId,
        reason: this.state.reason,
        needApproval: this.props.CLASSROOM_CAMPUS_ADMIN && this.props.addEditModalType === AddEditModalTypes.Edit,
      },
    });
  };

  updateProps = (params) => {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: params,
    });
  };

  updateValue = (type, val) => {
    const newVal = typeof val === 'string' ? val.trim() : val;
    this.updateProps({ [type]: newVal });
  };

  handleFileUpload = (info, index) => {
    this.props.dispatch({
      type: `${storeName}/updatePhoto`,
      payload: {
        info,
        index,
      },
    });
  };

  handleDoorRoomFileUpload = (info) => {
    this.props.dispatch({
      type: `${storeName}/updateDoorPhoto`,
      payload: {
        info,
      },
    });
  };

  handleFileDelete = (index) => {
    this.props.dispatch({
      type: `${storeName}/deletePhoto`,
      payload: {
        index,
      },
    });
  };

  handleDoorRoomFileDelete = () => {
    this.props.dispatch({
      type: `${storeName}/deleteDoorPhoto`,
    });
  };

  showImagePreviewModal = (imagePreviewSrc) => {
    this.setState({
      imagePreviewModalVisible: true,
      imagePreviewSrc,
    });
  };

  closeImagePreviewModal = () => {
    this.setState({
      imagePreviewModalVisible: false,
      imagePreviewSrc: '',
    });
  };

  onStatusChange = async (changeStatus) => {
    if (changeStatus || changeStatus == 0) {
      this.updateValue('status', changeStatus);
    }

    if (!this.props.isViewOnly) {
      this.setState({
        loading: true,
      });

      const [, res] = await requestWrapper(
        getAuditMsgForDetailChange({
          organizationId: this.props.organizationId,
          changeStatus: this.props.initialStatus == changeStatus ? 0 : 1,
        }),
      );

      if (res && res.data) {
        const { nodeId, approvalUsers } = res.data;
        this.setState({
          nodeId,
          approvalUsersDict: approvalUsers
            ? approvalUsers.map(({ userId: val, userName, roleName }) => {
                return { val, text: `${userName} ${roleName}` };
              })
            : [],
        });
      }

      this.setState({
        loading: false,
      });
    }
  };

  beforeUpload = async (files) => {
    return files.map((file) => {
      file['fileKey'] += file['name'];
      return file;
    });
  };

  render() {
    const {
      props,
      close,
      ok,
      updateValue,
      handleFileUpload,
      handleDoorRoomFileUpload,
      handleFileDelete,
      handleDoorRoomFileDelete,
    } = this;

    const submitDisable = !(
      props.classroom &&
      props.status &&
      props.organizationId &&
      props.classroomTypeId &&
      props.classMember &&
      props.classArea
    );

    const previewImageSrcs =
      props.deviceVoList && props.deviceVoList.photos ? props.deviceVoList.photos.map((photo) => photo.filePath) : [];

    const deviceVoList = props.deviceVoList ? props.deviceVoList : [];

    const isAddMode = props.addEditModalType === AddEditModalTypes.Add;
    const isEditMode = props.addEditModalType === AddEditModalTypes.Edit;

    let title = '教室详情';
    if (isAddMode) {
      title = '新增教室';
    }

    if (isEditMode) {
      title = '修改教室';
    }

    if (props.isViewOnly) {
      title = '教室详情';
    }

    const isWaiting = props.status == statusMap.waiting;
    const isAuditing = props.status == statusMap.auditing;

    return (
      <Modal
        className={style.container}
        loading={props.addEditModalLoading || this.state.loading}
        onClose={close}
        title={title}
        width={1000}
        footer={
          <ModalFooterBtn
            onOk={ok}
            onCancel={close}
            confirmBtnDisabled={submitDisable}
            isViewOnly={props.isViewOnly}
            isAddMode={isAddMode}
            isGroupAdmin={props.CLASSROOM_GROUP}
          />
        }
      >
        <div className={style.title}>教室基础信息</div>
        <Row gutter={20}>
          <Col span={24}>
            <FormItem labelWidth={150} name="工程设计图教室名称">
              <Input
                placeholder="工程设计图教室名称"
                maxLength={64}
                value={props.designName}
                onChange={(e) => updateValue('designName', e.target.value)}
                allowClear={true}
                disabled={!isAddMode && (props.isViewOnly || props.CLASSROOM_CAMPUS_ADMIN)}
              />
            </FormItem>
          </Col>
        </Row>
        <Row gutter={20}>
          <Col span={12}>
            <FormItem labelWidth={150} name="教室名称" required={true}>
              <Input
                placeholder="教室名称"
                maxLength={50}
                value={props.classroom}
                onChange={(e) => updateValue('classroom', e.target.value)}
                allowClear={true}
                disabled={props.isViewOnly}
              />
            </FormItem>
          </Col>
          <Col span={12}>
            <FormItem labelWidth={150} name="状态">
              <Select
                placeholder="状态"
                dict={
                  isAddMode || isWaiting || isAuditing || (props.CLASSROOM_GROUP && isEditMode)
                    ? statusDict
                    : subStatusDict
                }
                value={props.status}
                disabled={(props.CLASSROOM_GROUP && isEditMode) || isAddMode || props.isViewOnly || isWaiting}
                onChange={this.onStatusChange}
              />
            </FormItem>
          </Col>
        </Row>
        <Row gutter={20}>
          <Col span={12}>
            <FormItem labelWidth={150} name="学习中心" required={true}>
              <Select
                placeholder="学习中心"
                dict={props.campusDict}
                value={props.organizationId}
                onChange={(val) => updateValue('organizationId', val)}
                allowClear={true}
                showSearch
                disabled={props.isViewOnly || (props.CLASSROOM_CAMPUS_ADMIN && !isAddMode)}
              />
            </FormItem>
          </Col>
          <Col span={12}>
            <FormItem labelWidth={150} name="类型" required={true}>
              <Select
                placeholder="类型"
                dict={props.classroomTypeDict}
                value={props.classroomTypeId}
                onChange={(val) => updateValue('classroomTypeId', val)}
                allowClear={true}
                showSearch
                disabled={props.isViewOnly}
              />
            </FormItem>
          </Col>
        </Row>
        <Row gutter={20}>
          <Col span={12}>
            <FormItem labelWidth={150} name="面积（㎡）" required={true}>
              <InputNumber
                placeholder="面积（㎡）"
                min={0}
                max={99999999}
                value={props.classArea}
                onChange={(val) => updateValue('classArea', val)}
                allowClear={true}
                disabled={props.isViewOnly}
                required={true}
              />
            </FormItem>
          </Col>
          <Col span={12}>
            <FormItem labelWidth={150} name="适用人数" required={true}>
              <InputNumber
                placeholder="适用人数"
                min={0}
                max={9999999}
                value={props.classMember}
                onChange={(val) => updateValue('classMember', val)}
                allowClear={true}
                disabled={props.isViewOnly}
              />
            </FormItem>
          </Col>
        </Row>
        <FormItem labelWidth={150} name="教室全景照片">
          <div className={style.photos}>
            <Row gutter={20}>
              {props.photos.map((photo, index) => {
                return (
                  <Col span={4} key={index}>
                    <UploadImg
                      viewOnly={props.isViewOnly}
                      value={photo.file}
                      text="上传"
                      uploadComplete={(val) => handleFileUpload(val, index)}
                      deleteComplete={() => handleFileDelete(index)}
                      size={110}
                      qrCodeUploadVisible={false}
                      beforeUpload={this.beforeUpload}
                    />
                  </Col>
                );
              })}
            </Row>
          </div>
        </FormItem>
        <FormItem labelWidth={150} name="教室门牌照片">
          <div className={style.photos}>
            <UploadImg
              viewOnly={props.isViewOnly}
              value={props.doorPhoto && props.doorPhoto.file}
              text="上传"
              uploadComplete={(val) => handleDoorRoomFileUpload(val)}
              deleteComplete={handleDoorRoomFileDelete}
              size={110}
              qrCodeUploadVisible={false}
              beforeUpload={this.beforeUpload}
            />
          </div>
        </FormItem>
        <div className={style.title}>教室监控信息</div>
        {deviceVoList.length > 0 ? (
          deviceVoList.map(({ id, position, checked, deviceNo, channelNo, validateCode, oaNumber, photos = [] }) => {
            return (
              <div className={style.darkBlock} key={id}>
                {photos && photos.length > 0
                  ? photos.map(({ filePath, createTime }) => {
                      return (
                        <div>
                          <FormItem className={style.formControl} labelWidth={0}>
                            <img src={filePath} style={{ maxWidth: '100%' }} alt="监控照片" />
                          </FormItem>
                          <FormItem className={style.formControl} labelWidth={150} name="监控照片抓拍时间:">
                            {createTime ? moment(createTime).format('YYYY.MM.DD HH:mm:ss') : ''}
                          </FormItem>
                        </div>
                      );
                    })
                  : ''}
                <FormItem className={style.formControl} labelWidth={150} name="位置:">
                  {position}
                </FormItem>
                <FormItem className={style.formControl} labelWidth={150} name="关联OA单号:">
                  {oaNumber}
                </FormItem>
                <FormItem className={style.formControl} labelWidth={150} name="设备号:">
                  {deviceNo}
                </FormItem>
                <FormItem className={style.formControl} labelWidth={150} name="验证码:">
                  {validateCode}
                </FormItem>
                <FormItem className={style.formControl} labelWidth={150} name="通道号:">
                  {channelNo}
                </FormItem>
                <FormItem className={style.formControl} labelWidth={150} name="验收状态:">
                  {checked ? '验收通过' : '未验收'}
                </FormItem>
              </div>
            );
          })
        ) : (
          <div>该教室暂未绑定监控。</div>
        )}
        {isEditMode && props.CLASSROOM_CAMPUS_ADMIN && (
          <div className={style.auditBlock}>
            <FormItem labelWidth={150} name="申请理由" required={true}>
              <TextArea
                maxLength={199}
                value={this.state.reason}
                onChange={(e) => this.setState({ reason: e.target.value })}
              />
            </FormItem>
            <FormItem labelWidth={150} name="请选择审批人" required={true}>
              <Select
                placeholder="审批人"
                dict={this.state.approvalUsersDict}
                value={this.state.auditUserId}
                onChange={(auditUserId) => this.setState({ auditUserId })}
              />
            </FormItem>
          </div>
        )}

        <div className={style.tips}>
          {props.isViewOnly && !props.canEdit && (
            <div>如需修改教室详情信息，请联系学习中心班课主任或学习中心营运主任发起教室信息变更OA申请。</div>
          )}
          {!props.isViewOnly && isEditMode && (
            <div>
              <div>操作指引：</div>
              <div>1、同1学习中心教室名称不可重名</div>
              <div>2、如修改教室名称，需同步更新教室门牌照片</div>
              <div>3、如修改适用人数，需同步更新教室全景照片</div>
            </div>
          )}
          {props.isViewOnly && props.CLASSROOM_CAMPUS_ADMIN && (
            <div style={{ paddingRight: '20px' }}>
              <div>
                1、新增的教室默认状态为待审批，如需用于排课，教室状态需为“有效”，请前往教室列表，选中所需教室，点击批量有效，可发起教室状态变更OA申请；
              </div>
              <div>
                2、如已发起申请，可到OA查看申请进度，OA审批成功后，BOSS教室状态会同步更新，届时方可修改教室信息。
              </div>
            </div>
          )}
        </div>
        {this.state.imagePreviewModalVisible && (
          <ImagePreviewModal
            imageSrcs={previewImageSrcs}
            currentSrc={this.state.imagePreviewSrc}
            onClose={this.closeImagePreviewModal}
          />
        )}
      </Modal>
    );
  }
}

function mapStateToProps(state) {
  const btnAuth = state.account.btnAuths || [];
  const auditing = state[storeName].status == statusMap.auditing;

  return {
    addEditModalLoading: state[storeName].addEditModalLoading,
    addEditModalType: state[storeName].addEditModalType,
    designName: state[storeName].designName,
    classroom: state[storeName].classroom,
    status: state[storeName].status + '',
    organizationId: state[storeName].organizationId,
    classroomTypeId: state[storeName].classroomTypeId,
    classArea: state[storeName].classArea,
    classMember: state[storeName].classMember,
    photos: state[storeName].photos,
    doorPhoto: state[storeName].doorPhoto,
    deviceVoList: state[storeName].deviceVoList,
    campusDict: state.dict.campus || [],
    classroomTypeDict: state[storeName].classroomTypeDict,
    isViewOnly: state[storeName].isViewOnly,
    initialStatus: state[storeName].initialStatus,
    auditing,
    canEdit: btnAuth.includes('CLASSROOM_GROUP') || btnAuth.includes('CLASSROOM_CAMPUS_ADMIN'),
    CLASSROOM_GROUP: btnAuth.includes('CLASSROOM_GROUP'),
    CLASSROOM_CAMPUS_ADMIN: btnAuth.includes('CLASSROOM_CAMPUS_ADMIN') && !btnAuth.includes('CLASSROOM_GROUP'),
    CLASSROOM_CAMPUS_NORMAL: btnAuth.includes('CLASSROOM_CAMPUS_ADMIN'),
  };
}

export default connect(mapStateToProps)(AddEditModal);
