import React from 'react';
import { Modal, Select, Button } from 'COMPONENT';
import { storeName } from '../store/config';
import { connect } from 'dva';
import { Row, Col, Icon } from 'antd';
import { statusDict } from '../dicts';
import style from './style.less';
import classNames from 'classnames';
import { OSS_BASE_PREFIX } from 'CONST/ossConfig.js';

const EXCEL_URL = `${OSS_BASE_PREFIX}/批量导入教室模版.xlsx`;

const ModalFooterBtn = (props) => {
  const { onOk, onCancel, confirmBtnDisabled } = props;
  return (
    <div style={{ display: 'flex', justifyContent: 'flex-end' }}>
      <Button type="primary" onClick={onOk} disabled={confirmBtnDisabled}>
        导入
      </Button>
      <Button onClick={onCancel}>取消</Button>
    </div>
  );
};

function ImportModal(props) {
  function updateProps(params) {
    props.dispatch({
      type: `${storeName}/updateState`,
      payload: params,
    });
  }

  function updateValue(type, val) {
    updateProps({ [type]: val });
  }

  function close() {
    props.dispatch({
      type: `${storeName}/closeImportModal`,
    });
  }

  function ok() {
    props.dispatch({
      type: `${storeName}/onBatchImportModalOk`,
    });
  }

  function onUploadChange(e) {
    props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        importModal_File: e.target.files[0],
      },
    });
  }

  function removeFile() {
    props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        importModal_File: null,
      },
    });
  }

  return (
    <Modal
      loading={props.importModalLoading}
      onClose={close}
      title="批量导入教室"
      width={600}
      footer={
        <ModalFooterBtn
          onOk={ok}
          onCancel={close}
          confirmBtnDisabled={!(props.importModal_File && props.importModal_organizationId) || props.importModalLoading}
        />
      }
    >
      <Row className={style.formControl}>
        <Col span={2}>
          <Icon type="file-excel" className={style.excel} />
        </Col>
        <Col>
          <a className={style.download} href={EXCEL_URL} download>
            模板下载
          </a>
        </Col>
      </Row>
      {props.importModal_File ? (
        <div>
          <div className={style.uploadFileContainer}>
            <Row className={style.center}>
              <Col span={2}>
                <Icon type="file-excel" className={style.excel} />
              </Col>
              <Col className={style.fileName} span={20}>
                {props.importModal_File.name}
              </Col>
              <Col span={2}>
                <Icon onClick={removeFile} className={style.delete} type="close-circle" />
              </Col>
            </Row>
          </div>
          <div className={style.label}>选择教室所属学习中心场地</div>
          <Select
            className={style.formControl}
            placeholder="学习中心"
            dict={props.campusDict}
            value={props.importModal_organizationId}
            onChange={(val) => updateValue('importModal_organizationId', val)}
            allowClear={true}
            showSearch
          />
        </div>
      ) : (
        <div className={style.formControl}>
          <div>上传文件</div>
          <div className={style.uploadBtn}>
            <div className={style.uploadText}>上传文件</div>
            <input className={style.file} type="file" onChange={onUploadChange} />
          </div>
        </div>
      )}
    </Modal>
  );
}

function mapStateToProps(state) {
  return {
    importModalLoading: state[storeName].importModalLoading,
    importModal_File: state[storeName].importModal_File,
    importModal_organizationId: state[storeName].importModal_organizationId,
    campusDict: state.dict.campus,
  };
}

export default connect(mapStateToProps)(ImportModal);
