import React from 'react';
import { Modal, Select, Button, FormItem } from 'COMPONENT';
import { storeName } from '../store/config';
import { connect } from 'dva';
import { statusDict, statusMap } from '../dicts';
import style from './style.less';

const ModalFooterBtn = (props) => {
  const { onOk, onCancel, confirmBtnDisabled, needAudit } = props;
  return (
    <div>
      {needAudit ? (
        <div style={{ display: 'flex', justifyContent: 'flex-end' }}>
          <Button type="primary" onClick={onOk} disabled={confirmBtnDisabled}>
            发起审批
          </Button>
          <Button onClick={onCancel}>取消</Button>
        </div>
      ) : (
        <div style={{ display: 'flex', justifyContent: 'center' }}>
          <Button onClick={onCancel}>关闭</Button>
        </div>
      )}
    </div>
  );
};

function ReasonModal(props) {
  function updateProps(params) {
    props.dispatch({
      type: `${storeName}/updateState`,
      payload: params,
    });
  }

  function updateValue(type, val) {
    updateProps({ [type]: val });
  }

  function close() {
    props.dispatch({
      type: `${storeName}/closeReasonModal`,
    });
  }

  function ok() {
    props.dispatch({
      type: `${storeName}/onReasonModalOk`,
    });
  }

  return (
    <Modal
      loading={props.reasonModalLoading}
      onClose={close}
      title=""
      width={700}
      footer={
        <ModalFooterBtn onOk={ok} onCancel={close} confirmBtnDisabled={!props.auditUserId} needAudit={!props.status1} />
      }
    >
      <div className={style.modal}>
        {props.status1 && (
          <div>
            <h3>待审批教室状态修改成功</h3>
            <div className={style.content}>所选中的【待审批】教室状态已成功更改为【无效】</div>
          </div>
        )}

        {props.status2 && <h3 className={style.content}>有效教室需发起OA审批方可修改为无效</h3>}

        {props.status3 && (
          <div>
            <div style={{ marginLeft: '45px' }}>
              <h3>待审批教室状态修改成功！</h3>
              <div>所选中的【待审批】教室状态已成功更改为【无效】</div>
            </div>
            <h3 className={style.content}>有效教室需发起OA审批方可修改为无效：</h3>
          </div>
        )}

        {props.status4 && <h3 className={style.content}>所选教室需发起OA审批方可修改为有效：</h3>}

        {!props.status1 && (
          <FormItem name="请选择审批人" required={true}>
            <Select
              placeholder="审批人"
              dict={props.approvalUsersDict || []}
              value={props.auditUserId}
              onChange={(val) => updateValue('auditUserId', val)}
            />
          </FormItem>
        )}
      </div>
    </Modal>
  );
}

function mapStateToProps(state) {
  const { selectedRows, reasonModal_status } = state[storeName];

  const modifyStatusValid = statusMap.valid === reasonModal_status;
  return {
    reasonModalLoading: state[storeName].reasonModalLoading,
    approvalUsersDict: state[storeName].approvalUsersDict || [],
    auditUserId: state[storeName].auditUserId,
    classroomIds: state[storeName].classroomIds,
    status1: !modifyStatusValid && selectedRows.every((row) => row.status == statusMap.waiting), // 点击批量无效时：当所选教室都是【待审批】
    status2: !modifyStatusValid && selectedRows.every((row) => row.status == statusMap.valid), // 点击批量无效时：当所选教室都是【有效】
    status3:
      !modifyStatusValid &&
      selectedRows.some((row) => row.status == statusMap.waiting) &&
      selectedRows.some((row) => row.status == statusMap.valid), // 当所选教室既有【有效】也有【待审批
    status4: modifyStatusValid, // 点击批量有效时
  };
}

export default connect(mapStateToProps)(ReasonModal);
