import React from 'react';
import { connect } from 'dva';
import { Select, Input, SearchButton, InputNumber, Button, MultiSelect } from 'COMPONENT';
import { AddEditModalTypes, storeName } from '../store/config';
import { statusDict, statusMap, deviceStatusDict } from '../dicts';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

function Search(props) {
  function updateProps(params) {
    props.dispatch({
      type: `${storeName}/updateState`,
      payload: params,
    });
  }

  function updateValue(type, val) {
    updateProps({ [type]: val });
  }

  function onReset() {
    props.dispatch({
      type: `${storeName}/resetPage`,
    });
    props.onSearch();
  }

  function batchModifyStatus(status) {
    if (props.selectedRows.length < 1) {
      showMessage('请选中至少一个教室');
      return;
    }

    props.dispatch({
      type: `${storeName}/batchModifyStatus`,
      payload: {
        classroomIds: props.selectedRows.map((row) => row.id),
        status,
      },
    });
  }

  function batchImport() {
    props.dispatch({
      type: `${storeName}/openImportModal`,
    });
  }

  function openAddModal() {
    props.dispatch({
      type: `${storeName}/openAddEditModal`,
      payload: {
        addEditModalType: AddEditModalTypes.Add,
      },
    });
  }

  function onSearch() {
    props.dispatch({
      type: `${storeName}/resetListData`,
    });
    props.onSearch();
  }

  return (
    <div>
      {props.canEdit && (
        <span>
          <Button
            onClick={() => batchModifyStatus(statusMap.valid)}
            disabled={false}
            type="primary"
            style={{ marginRight: '20px' }}
          >
            批量有效
          </Button>
          <Button
            onClick={() => batchModifyStatus(statusMap.inValid)}
            disabled={false}
            type="primary"
            style={{ marginRight: '20px' }}
          >
            批量无效
          </Button>
          <Button onClick={openAddModal} disabled={false} type="primary" style={{ marginRight: '20px' }}>
            新增教室
          </Button>
          {props.CLASSROOM_GROUP && (
            <Button onClick={batchImport} disabled={false} type="primary" style={{ marginRight: '20px' }}>
              批量导入
            </Button>
          )}
        </span>
      )}
      <SearchButton
        // searchBtnLoading={props.searchBtnLoading}
        onSearch={onSearch}
        onReset={onReset}
      >
        <Input
          placeholder="工程设计图教室名称"
          value={props.sKey_designName}
          onChange={(e) => updateValue('sKey_designName', e.target.value)}
          allowClear={true}
        />
        <Input
          placeholder="教室名称"
          value={props.sKey_classroomName}
          onChange={(e) => updateValue('sKey_classroomName', e.target.value)}
          allowClear={true}
        />
        <Select
          placeholder="分公司"
          dict={props.branchDict}
          value={props.sKey_branchId}
          onChange={(val) => updateValue('sKey_branchId', val)}
          allowClear={true}
          showSearch
        />
        <Select
          placeholder="学习中心"
          dict={props.campusDict}
          value={props.sKey_campusId}
          onChange={(val) => updateValue('sKey_campusId', val)}
          allowClear={true}
          showSearch
        />
        <Select
          placeholder="类型"
          dict={props.classroomTypeDict}
          value={props.sKey_classTypeId}
          onChange={(val) => updateValue('sKey_classTypeId', val)}
          allowClear={true}
          showSearch
        />
        <InputNumber
          placeholder="面积下限"
          min={0}
          value={props.sKey_minArea}
          onChange={(val) => updateValue('sKey_minArea', val)}
          allowClear={true}
        />
        <InputNumber
          placeholder="面积上限"
          min={0}
          value={props.sKey_maxArea}
          onChange={(val) => updateValue('sKey_maxArea', val)}
          allowClear={true}
        />
        <MultiSelect
          placeholder="状态"
          dict={statusDict}
          value={props.sKey_statuses}
          onChange={(val) => updateValue('sKey_statuses', val)}
          allowClear={true}
          showSearch
          isMultiSelect={true}
        />
        <Select
          placeholder="是否已绑定摄像头"
          dict={deviceStatusDict}
          value={props.sKey_deviceStatus}
          onChange={(val) => updateValue('sKey_deviceStatus', val)}
          allowClear={true}
        />
      </SearchButton>
    </div>
  );
}

const mapStateToProps = (state) => {
  const btnAuth = state.account.btnAuths || [];

  return {
    // 筛选项值
    branchDict: state.dict.branch,
    campusDict: state.dict.campus,
    sKey_classroomName: state[storeName].sKey_classroomName,
    sKey_branchId: state[storeName].sKey_branchId,
    sKey_campusId: state[storeName].sKey_campusId,
    sKey_classTypeId: state[storeName].sKey_classTypeId,
    sKey_minArea: state[storeName].sKey_minArea,
    sKey_maxArea: state[storeName].sKey_maxArea,
    sKey_statuses: state[storeName].sKey_statuses,
    classroomTypeDict: state[storeName].classroomTypeDict,
    sKey_designName: state[storeName].sKey_designName,
    selectedRows: state[storeName].selectedRows,
    sKey_deviceStatus: state[storeName].sKey_deviceStatus,
    CLASSROOM_GROUP: btnAuth.includes('CLASSROOM_GROUP'),
  };
};

export default connect(mapStateToProps)(Search);
