/**
 * 教室管理
 */
import React from 'react';
import { connect } from 'dva';
import { Page, List } from 'COMPONENT';
import Search from './Search';
import AddEditModal from './AddEditModal';
import ImportModal from './ImportModal';
import ReasonModal from './ReasonModal';
import ImportResultModal from './ImportResultModal';
import { storeName } from './store/config';
import { Table } from 'antd';
import TextLink from 'COMPONENT/TextLink';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { statusMap, deviceStatusMap } from './dicts';

const { COL_WIDTH_M, COL_WIDTH_L } = List;

class ClassRoomManagement extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      pageLoading: true,
      scrollY: 0,
    };
  }

  componentDidMount() {
    this.initSearchDefault();
    this.loadList();
    this.initDict();
    this.calculateScrollY();
    this.hidePageLoading();
    this.props.history.listen((key, fn) => {
      if (fn.location.pathname === this.props.location.pathname) {
        if (fn.location.query && fn.location.query.classroomId) {
          this.findClassRoomByIdAndOpenEditModal({ id: fn.location.query.classroomId }, false);
        }
      }
    });
  }

  hidePageLoading = () => {
    this.setState({
      pageLoading: false,
    });
  };

  calculateScrollY = () => {
    const searchDomRect = document.querySelector('.Page-header').getClientRects()[0];
    // 搜索+分页高度
    let y = searchDomRect.top + searchDomRect.height + 150;
    this.setState({
      scrollY: `calc(100vh - ${y}px)`,
    });
  };

  updatePagination = ({ current: pageCurrent, pageSize }) => {
    let payload = {};
    if (pageSize && pageSize !== this.props.pageSize) {
      payload.pageCurrent = 1;
      payload.pageSize = pageSize;
    } else {
      payload.pageCurrent = pageCurrent;
    }
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload,
    });

    this.loadList();
  };

  onSelectRow = (selectedRowKeys, selectedRows) => {
    const brandIds = selectedRows.map((row) => row.organizationId);
    if (brandIds.length > 1) {
      const first = brandIds[0];
      for (let i = 1; i < brandIds.length; i++) {
        if (first !== brandIds[i]) {
          showMessage('所选教室必须全部为同一学习中心下才能一起发起！');
          return;
        }
      }
    }

    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        selectedRows,
        selectedRowKeys,
      },
    });
  };

  initDict = () => {
    new Promise((resolve) => {
      this.props.dispatch({
        type: 'dict/initDict',
        payload: {
          list: ['branch', 'campus'],
          resolve,
        },
      });
    });
    this.props.dispatch({
      type: `${storeName}/initCommonDict`,
    });
  };

  initSearchDefault = () => {
    this.props.dispatch({
      type: `${storeName}/initSearchDefault`,
    });
  };

  loadList = async () => {
    this.props.dispatch({
      type: `${storeName}/loadList`,
    });
  };

  findClassRoomByIdAndOpenEditModal = ({ id }, isViewOnly) => {
    this.props.dispatch({
      type: `${storeName}/findClassRoomByIdAndOpenEditModal`,
      payload: {
        id,
        isViewOnly,
      },
    });
  };

  inValidClassRoom = ({ id }) => {
    this.props.dispatch({
      type: `${storeName}/batchModifyStatus`,
      payload: {
        classroomIds: [id],
        status: statusMap.inValid,
      },
    });
  };

  render() {
    let {
      records,
      listLoading,
      rowCount,
      pageCurrent,
      pageSize,
      addEditModalVisible,
      importModalVisible,
      reasonModalVisible,
      importResultModalVisible,
      canEdit,
    } = this.props;
    let { pageLoading } = this.state;

    const columns = [
      { title: '教室编号', dataIndex: 'id', key: 'id', width: COL_WIDTH_L },
      { title: '工程设计图教室名称', dataIndex: 'designName', width: COL_WIDTH_L + 50 },
      { title: '教室名称', dataIndex: 'classroom', width: COL_WIDTH_M },
      { title: '所属学习中心', dataIndex: 'organizationName', width: COL_WIDTH_M },
      { title: '类型', dataIndex: 'classTypeName', width: COL_WIDTH_M },
      { title: '面积（㎡）', dataIndex: 'classArea', width: COL_WIDTH_M },
      { title: '适用人数', dataIndex: 'classMember', width: COL_WIDTH_M },
      { title: '创建人', dataIndex: 'creatorName', width: COL_WIDTH_M },
      { title: '创建时间', dataIndex: 'createTime', width: COL_WIDTH_L },
      { title: '修改人', dataIndex: 'modifierName', width: COL_WIDTH_M },
      { title: '修改时间', dataIndex: 'modifyTime', width: COL_WIDTH_L },
      { title: '状态', dataIndex: 'statusName', width: COL_WIDTH_M },
      {
        title: '是否已绑定摄像头',
        dataIndex: 'deviceList',
        width: COL_WIDTH_M,
        render: (deviceList) => {
          return deviceList ? '是' : '否';
        },
      },
      {
        title: '操作',
        key: 'action',
        width: COL_WIDTH_L,
        render: (text, record, index) => {
          let editViewText = '详情';
          if (canEdit && (record.status == statusMap.valid || record.status == statusMap.inValid)) {
            editViewText = '修改';
          }
          if (this.props.CLASSROOM_GROUP && record.status == statusMap.waiting) {
            editViewText = '修改';
          }

          const actions = [
            {
              text: editViewText,
              onClick: () => this.findClassRoomByIdAndOpenEditModal(record, editViewText === '详情'),
            },
          ];

          return (
            <span>
              {actions.map((action, i) => {
                return (
                  <TextLink key={i} className="list-btn" onClick={action.onClick}>
                    {action.text}
                  </TextLink>
                );
              })}
            </span>
          );
        },
      },
    ];

    return (
      <Page
        fitWindowHeight={true}
        loading={pageLoading}
        route={this.props.route}
        headerToolBar={<Search onSearch={this.loadList} canEdit={this.props.canEdit} />}
        subHeader={
          <span style={{ color: 'red' }}>
            {this.props.canEdit
              ? '新增的教室默认状态为待审批，如需用于排课，教室状态需为“有效”，请选中所需教室，点击批量有效，可发起教室状态变更OA申请。'
              : '如需新增教室详情信息，请联系学习中心班课主任或学习中心营运主任新增后，发起教室状态变更OA申请。'}
          </span>
        }
      >
        <Table
          rowKey="id"
          dataSource={records}
          columns={columns}
          loading={!pageLoading && listLoading}
          rowSelection={{
            selectedRowKeys: this.props.selectedRowKeys,
            onChange: this.onSelectRow,
            getCheckboxProps: (record) => ({
              disabled: record.status == statusMap.auditing,
            }),
          }}
          pagination={{
            howQuickJumper: true,
            pageSizeOptions: ['10', '20', '50', '100'],
            showSizeChanger: true,
            showTotal: (total) => `共 ${total} 条记录`,
            current: pageCurrent,
            total: rowCount,
            pageSize,
          }}
          scroll={{
            // x: 1800,
            y: this.state.scrollY,
          }}
          onChange={this.updatePagination}
        />
        {addEditModalVisible && <AddEditModal canEdit={this.props.canEdit} />}
        {importModalVisible && <ImportModal />}
        {importResultModalVisible && <ImportResultModal />}
        {reasonModalVisible && <ReasonModal />}
      </Page>
    );
  }
}

function mapStateToProps(state) {
  const btnAuth = state.account.btnAuths || [];
  return {
    records: state[storeName].records,
    rowCount: state[storeName].rowCount,
    pageSize: state[storeName].pageSize,
    pageCurrent: state[storeName].pageCurrent,
    selectedRowKeys: state[storeName].selectedRowKeys,
    listLoading: state[storeName].listLoading,
    addEditModalVisible: state[storeName].addEditModalVisible,
    importModalVisible: state[storeName].importModalVisible,
    reasonModalVisible: state[storeName].reasonModalVisible,
    importResultModalVisible: state[storeName].importResultModalVisible,
    canEdit: btnAuth.includes('CLASSROOM_GROUP') || btnAuth.includes('CLASSROOM_CAMPUS_ADMIN'),
    CLASSROOM_GROUP: btnAuth.includes('CLASSROOM_GROUP'),
  };
}

export default connect(mapStateToProps)(ClassRoomManagement);
