import { storeName, AddEditModalTypes } from './config';
import {
  getClassroomManageList,
  ClassroomNewControllerAdd,
  ClassroomNewControllerModify,
  ClassroomNewBatchModifyStatus,
  ClassroomNewImportClassroom,
  ClassroomNewFindById,
  applyBatchChangeClassroomStatus,
} from 'SERVICE/basicInformationManage.js';
import { clone } from 'lodash';
import { loadCommonDict } from 'UTIL/dict';
import { isEmpty } from './utils';
import { requestWrapper } from 'UTIL/promiseHandler';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

function initPhotos() {
  return [
    {
      filePath: '',
      fileSize: '',
      realFileName: '',
      file: null,
    },
    {
      filePath: '',
      fileSize: '',
      realFileName: '',
      file: null,
    },
    {
      filePath: '',
      fileSize: '',
      realFileName: '',
      file: null,
    },
    {
      filePath: '',
      fileSize: '',
      realFileName: '',
      file: null,
    },
    {
      filePath: '',
      fileSize: '',
      realFileName: '',
      file: null,
    },
  ];
}

function initDoorPhoto() {
  return {
    filePath: '',
    fileSize: '',
    realFileName: '',
    file: null,
  };
}

const defaultState = {
  sKey: {
    sKey_classroomName: '',
    sKey_branchId: '',
    sKey_campusId: '',
    sKey_classTypeId: '',
    sKey_minArea: '',
    sKey_maxArea: '',
    sKey_statuses: [],
    sKey_designName: '',
    sKey_deviceStatus: '',
  },
  list: {
    records: [],
    rowCount: 0,
    pageSize: 100,
    pageCurrent: 1,
    listLoading: false,
  },
  listData: {
    selectedRows: [],
    selectedRowKeys: [],
  },
  addEditModal: {
    addEditModalVisible: false,
    addEditModalType: '',
    addEditModalLoading: false,
    deviceVoList: [],
    isViewOnly: false,
    initialStatus: undefined,
  },
  addEditModalData: {
    id: '',
    designName: '',
    classroom: '',
    status: '2',
    organizationId: '',
    classroomTypeId: '',
    classArea: '',
    classMember: '',
    photos: initPhotos(),
    doorPhoto: initDoorPhoto(),
  },
  importModal: {
    importModalVisible: false,
    importModalLoading: false,
  },
  importModalData: {
    importModal_File: null,
    importModal_organizationId: '',
  },
  reasonModal: {
    reasonModalVisible: false,
    reasonModalLoading: false,
    approvalUsersDict: [],
  },
  reasonModalData: {
    classroomIds: [],
    nodeId: [],
    auditUserId: '',
    reasonModal_status: '',
    reasonModal_campusId: '',
    isAllModifySuccess: false,
    modifyStatus: '',
  },
  importResultModal: {
    importResultModalVisible: false,
  },
  importResultModalData: {
    totalNum: '',
    successNum: '',
    failNum: '',
    resultUrl: '',
  },
  dict: {
    classroomTypeDict: [],
  },
};
Object.freeze(defaultState);

export default {
  namespace: storeName,
  state: {
    // index
    ...clone(defaultState.sKey),
    ...clone(defaultState.list),
    ...clone(defaultState.listData),
    ...clone(defaultState.addEditModal),
    ...clone(defaultState.dict),
    ...clone(defaultState.addEditModalData),
    ...clone(defaultState.importModal),
    ...clone(defaultState.importModalData),
    ...clone(defaultState.reasonModal),
    ...clone(defaultState.reasonModalData),
    ...clone(defaultState.importResultModal),
    ...clone(defaultState.importResultModalData),
  },
  reducers: {
    updateState(state, action) {
      const { payload } = action;
      return {
        ...state,
        ...payload,
      };
    },

    updateList(state, { payload: { records, rowCount } }) {
      const mapRecords = records.map((record) => ({
        ...record,
        // some extra data
      }));

      return {
        ...state,
        records: mapRecords,
        rowCount,
      };
    },

    openAddEditModal(state, { payload: { addEditModalType } }) {
      return {
        ...state,
        addEditModalVisible: true,
        addEditModalType,
      };
    },

    closeAddEditModal(state) {
      return {
        ...state,
        ...clone(defaultState.addEditModal),
        ...clone(defaultState.addEditModalData),
        photos: initPhotos(),
        doorPhoto: initDoorPhoto(),
      };
    },
    updatePhoto(state, { payload: { info, index } }) {
      const photo = state.photos[index];
      photo.file = info;
      photo.realFileName = info.name;
      photo.fileSize = info.size;
      photo.filePath = info.src;

      return {
        ...state,
        photos: [...clone(state.photos)],
      };
    },
    updateDoorPhoto(state, { payload: { info } }) {
      const photo = state.doorPhoto;
      photo.file = info;
      photo.realFileName = info.name;
      photo.fileSize = info.size;
      photo.filePath = info.src;

      return {
        ...state,
        doorPhoto: { ...clone(photo) },
      };
    },

    deleteDoorPhoto(state) {
      return {
        ...state,
        doorPhoto: initDoorPhoto(),
      };
    },

    deletePhoto(state, { payload: { index } }) {
      const photo = state.photos[index];
      photo.file = null;
      photo.realFileName = '';
      photo.fileSize = '';
      photo.filePath = '';

      return {
        ...state,
        photos: [...clone(state.photos)],
      };
    },

    openImportModal(state) {
      return {
        ...state,
        importModalVisible: true,
      };
    },

    closeImportModal(state) {
      return {
        ...state,
        ...clone(defaultState.importModal),
        ...clone(defaultState.importModalData),
      };
    },

    openReasonModal(state) {
      return {
        ...state,
        reasonModalVisible: true,
      };
    },

    closeReasonModal(state) {
      return {
        ...state,
        ...clone(defaultState.reasonModal),
        ...clone(defaultState.reasonModalData),
        ...clone(defaultState.listData),
      };
    },
    openImportResultModal(state) {
      return {
        ...state,
        importResultModalVisible: true,
      };
    },
    closeImportResultModal(state) {
      return {
        ...state,
        ...clone(defaultState.importResultModal),
        ...clone(defaultState.importResultModalData),
      };
    },
    resetListData(state) {
      return {
        ...state,
        ...clone(defaultState.listData),
      };
    },
  },
  effects: {
    *resetPage(action, { put }) {
      const newDefaultState = Object.keys(defaultState).reduce((acc, key) => {
        if (key !== 'dict') {
          Object.assign(acc, defaultState[key]);
        }
        return acc;
      }, {});
      yield put({
        type: 'updateState',
        payload: {
          ...clone(newDefaultState),
        },
      });
    },
    *initCommonDict(action, { put }) {
      const [err, classroomTypeDict = []] = yield requestWrapper(loadCommonDict('CLASSROOM_TYPE'));
      if (!err) {
        yield put({
          type: 'updateState',
          payload: {
            classroomTypeDict,
          },
        });
      }
    },
    *loadList(action, { select, put }) {
      yield put({
        type: 'updateState',
        payload: {
          listLoading: true,
        },
      });
      const sKeys = yield select(searchValuesSelector);
      const apiParams = yield select((state) => {
        return {
          ...sKeys,
          sidx: 'modifyTime',
          sord: 'desc',
          page: state[storeName].pageCurrent,
          rows: state[storeName].pageSize,
          getTotal: true,
        };
      });
      const [err, response] = yield requestWrapper(getClassroomManageList(apiParams));
      if (!err) {
        yield put({
          type: 'updateList',
          payload: {
            records: response.rows || [],
            rowCount: response.records,
          },
        });
      }

      yield put({
        type: 'updateState',
        payload: {
          listLoading: false,
        },
      });
    },

    *onAddEditModalOk({ payload: { nodeId, auditUserId, reason, needApproval } }, { put, select }) {
      const apiParams = yield select(addEditModalDataSelector);
      // only in edit mode
      if (needApproval) {
        apiParams.nodeId = nodeId;
        apiParams.auditUserId = auditUserId;
        apiParams.reason = reason;
      }

      const requestFnName = yield select((state) => state[storeName].addEditModalType);
      if (requestFnName === AddEditModalTypes.Add) {
        delete apiParams.id;
      }
      const requestFnNameMap = {
        ClassroomNewControllerAdd,
        ClassroomNewControllerModify,
      };

      const [err] = yield requestWrapper(requestFnNameMap[requestFnName](apiParams));
      if (!err) {
        const message = needApproval ? '教室修改申请发起成功！您可前往OA查看工单进度' : '操作成功';
        showMessage(message, 'success');
      }
      yield put({
        type: 'closeAddEditModal',
      });
      yield put({
        type: 'loadList',
      });
    },

    *findClassRoomByIdAndOpenEditModal({ payload: { id, isViewOnly } }, { put }) {
      const [err, response] = yield requestWrapper(ClassroomNewFindById({ id }));
      if ((!err, response && response.data)) {
        const { data } = response;
        const photos = data.photos.map((photo) => {
          photo.file = {
            fileName: photo.realFileName,
            name: photo.realFileName,
            size: photo.fileSize,
            src: photo.filePath,
          };
          return photo;
        });
        photos.push(...initPhotos());
        photos.length = 5;

        let { doorPhoto } = data;
        if (doorPhoto) {
          doorPhoto.file = {
            fileName: doorPhoto.realFileName,
            name: doorPhoto.realFileName,
            size: doorPhoto.fileSize,
            src: doorPhoto.filePath,
          };
        } else {
          doorPhoto = {
            file: null,
          };
        }

        yield put({
          type: 'updateState',
          payload: {
            id,
            designName: data.designName,
            classroom: data.classroom,
            status: data.status,
            organizationId: data.organizationId,
            classroomTypeId: data.classroomTypeId,
            classArea: data.classArea,
            classMember: data.classMember,
            doorPhoto,
            deviceVoList: (data.deviceVoList || []).map((item) => ({
              ...item,
              photos: (item.photos || []).filter(Boolean),
            })),
            photos,
            isViewOnly,
            initialStatus: data.status,
          },
        });
        yield put({
          type: 'openAddEditModal',
          payload: {
            addEditModalType: isViewOnly ? AddEditModalTypes.View : AddEditModalTypes.Edit,
          },
        });
      }
    },

    *onBatchImportModalOk(action, { select, put }) {
      yield put({
        type: 'updateState',
        payload: {
          importModalLoading: true,
        },
      });
      const { file, organizationId } = yield select((state) => {
        return {
          file: state[storeName].importModal_File,
          organizationId: state[storeName].importModal_organizationId,
        };
      });
      const data = new FormData();
      data.append('file', file);
      data.append('organizationId', organizationId);
      const [err, response] = yield requestWrapper(ClassroomNewImportClassroom(data));
      if (!err && response && response.data) {
        const {
          data: { totalNum, successNum, failNum, resultUrl },
        } = response;
        yield put({
          type: 'updateState',
          payload: {
            totalNum,
            successNum,
            failNum,
            resultUrl,
          },
        });

        yield put({
          type: 'closeImportModal',
        });

        yield put({
          type: 'openImportResultModal',
        });
      }
    },

    *onImportResultModalOk(action, { put }) {
      yield put({
        type: 'closeImportResultModal',
      });
      yield put({
        type: 'loadList',
      });
    },

    *batchModifyStatus({ payload: { classroomIds, status } }, { select, put }) {
      const [err, response] = yield requestWrapper(ClassroomNewBatchModifyStatus({ classroomIds, status }));
      if (!err && response && response.data) {
        const { needApproveClassroomIds, campusId, auditUserMsg, modifyStatus } = response.data;
        if (modifyStatus == 0) {
          showMessage('教室状态变更成功', 'success');
          yield put({
            type: 'resetListData',
          });
          yield put({
            type: 'loadList',
          });
          return;
        } else if (modifyStatus == 1) {
          yield put({
            type: 'loadList',
          });
        }

        yield put({
          type: 'updateState',
          payload: {
            classroomIds: needApproveClassroomIds,
            nodeId: auditUserMsg && auditUserMsg.nodeId,
            approvalUsersDict:
              auditUserMsg && auditUserMsg.approvalUsers.map((user) => ({ val: user.userId, text: user.userName })),
            reasonModal_status: status,
            reasonModal_campusId: campusId,
            modifyStatus,
          },
        });

        yield put({
          type: 'openReasonModal',
        });
      }
    },

    *onReasonModalOk(action, { select, put }) {
      const apiParams = yield select((state) => {
        return {
          classroomIds: state[storeName].classroomIds,
          status: state[storeName].reasonModal_status,
          nodeId: state[storeName].nodeId,
          auditUserId: state[storeName].auditUserId,
          campusId: state[storeName].reasonModal_campusId,
        };
      });
      const [err] = yield requestWrapper(applyBatchChangeClassroomStatus(apiParams));
      if (!err) {
        yield put({
          type: 'closeReasonModal',
        });
        showMessage('申请发起成功！您可前往OA查看工单进度', 'success');
        yield put({
          type: 'loadList',
        });
      }
    },
  },
};

function searchValuesSelector(state) {
  const store = state[storeName];
  return Object.keys(defaultState.sKey)
    .filter((key) => !isEmpty(store[key]))
    .reduce((accumulate, key) => {
      const val = store[key];
      const newKey = key.replace('sKey_', '');
      accumulate[newKey] = val;
      return accumulate;
    }, {});
}

function addEditModalDataSelector(state) {
  const store = state[storeName];
  return Object.keys(defaultState.addEditModalData)
    .filter((key) => !isEmpty(store[key]))
    .reduce((accumulate, key) => {
      let val = store[key];
      if (key === 'photos') {
        val = val
          .filter((photo) => {
            return photo.file;
          })
          .map(({ realFileName, fileSize, filePath }) => {
            return {
              realFileName,
              fileSize,
              filePath,
            };
          });
      }
      if (key === 'doorPhoto') {
        if (!val.file) {
          return accumulate;
        }
        const { realFileName, fileSize, filePath } = val;
        val = {
          realFileName,
          fileSize,
          filePath,
        };
      }

      accumulate[key] = val;
      return accumulate;
    }, {});
}
