import React from 'react';
import { Modal, Button } from 'COMPONENT';
import style from './style.less';
import ImagePreview from '../../../../modals/ImagePreview';
import PropTypes from 'prop-types';
import moment from 'moment';

class ImagePreviewModal extends React.Component {
  static propTypes = {
    previewImageSrcs: PropTypes.array.isRequired,
    title: PropTypes.string.isRequired,
    previewSrc: PropTypes.string.isRequired,
    fileName: PropTypes.string.isRequired,
    time: PropTypes.number.isRequired,
    onClose: PropTypes.func.isRequired,
    name: PropTypes.string.isRequired,
  };

  state = {
    imagePreviewModalVisible: false,
  };

  closeImagePreviewModal = () => {
    this.setState({
      imagePreviewModalVisible: false,
    });
  };

  showImagePreviewModal = () => {
    this.setState({
      imagePreviewModalVisible: true,
    });
  };

  downLoadImage = () => {
    const { previewSrc } = this.props;
    if (!previewSrc) {
      return;
    }

    const fileName = this.props.fileName || previewSrc.split('/').pop();
    const extension = fileName.split('.').pop();

    let image = new Image();
    // 解决跨域 Canvas 污染问题
    image.setAttribute('crossOrigin', 'anonymous');
    image.onload = function () {
      let canvas = document.createElement('canvas');
      canvas.width = image.width;
      canvas.height = image.height;
      let context = canvas.getContext('2d');
      context.drawImage(image, 0, 0, image.width, image.height);
      let url = canvas.toDataURL(`image/${extension}`); //得到图片的base64编码数据
      let a = document.createElement('a'); // 生成一个a元素
      let event = new MouseEvent('click'); // 创建一个单击事件
      a.download = fileName || '默认图片'; // 设置图片名称
      a.href = url; // 将生成的URL设置为a.href属性
      a.dispatchEvent(event); // 触发a的单击事件
    };
    image.src = this.props.previewSrc;
  };

  render() {
    const { previewImageSrcs, title, previewSrc, fileName, time, onClose, name } = this.props;

    return (
      <Modal
        loading={false}
        onCancel={onClose}
        title={title}
        width={835}
        cancelText="关闭"
        footer={
          <div style={{ display: 'flex', justifyContent: 'center' }}>
            <div className={style.download} onClick={this.downLoadImage}>
              下载
            </div>
            <Button onClick={onClose}>取消</Button>
          </div>
        }
      >
        <div className={style.label}>创建人：{name}</div>
        <div className={style.label}>创建时间：{moment(time).format('YYYY-MM-DD hh:mm:ss')}</div>
        <img onClick={this.showImagePreviewModal} className={style.image} src={previewSrc} />
        {this.state.imagePreviewModalVisible && (
          <ImagePreview imageSrcs={previewImageSrcs} currentSrc={previewSrc} onClose={this.closeImagePreviewModal} />
        )}
      </Modal>
    );
  }
}

export default ImagePreviewModal;
