import React from 'react';
import { Upload } from 'COMPONENT';
import PropTypes from 'prop-types';
import style from './style.less';
import ImagePreviewModal from './ImagePreviewModal';
import FileList from './FileList';
import { connect } from 'dva';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

class Uploader extends React.Component {
  static propTypes = {
    onFileListChange: PropTypes.func.isRequired,
    fileList: PropTypes.array.isRequired,
  };

  state = {
    imagePreviewModalVisible: false,
    previewFileName: '',
    previewSrc: '',
    previewUserName: '',
    previewCreateTime: '',
    isMounted: false,
  };

  hideImagePreviewModal = () => {
    this.setState({
      imagePreviewModalVisible: false,
      previewSrc: '',
      previewFileName: '',
    });
  };

  onUploadChange = (files) => {
    const fileList = files.map((file) => {
      const extention = file.name.split('.').pop();
      const images = ['jpg', 'jpeg', 'png'];

      return {
        ...file,
        showProgress: true,
        id: file.fileKey,
        iconSrc: images.includes(extention) ? file.src : undefined,
      };
    });

    const oldFileListMap = this.props.fileList.reduce((acc, cur) => {
      const { id } = cur;
      acc[id] = cur;
      return acc;
    }, {});

    // filter new file and update new file
    const newFiles = [];
    for (const file of fileList) {
      const { id } = file;
      if (!oldFileListMap[id]) {
        newFiles.push(file);
      } else {
        oldFileListMap[id] = file;
      }
    }

    const allFileList = Object.keys(oldFileListMap)
      .map((id) => oldFileListMap[id])
      .concat(newFiles);

    this.props.onFileListChange(allFileList);
  };

  beforeUpload = (fileList) => {
    const fileLength = this.props.fileList.length + fileList.length;
    if (fileLength > 10) {
      showMessage('最多传10张', 'error');
      return false;
    }
    for (const file of fileList) {
      file['fileKey'] += '.' + file['name'].split('.').pop();
    }
    return fileList;
  };

  onPreview = (file) => {
    const { previewPath, src, fileName, createUserName, createTime } = file;
    if (fileName.split('.').pop() === 'pdf') {
      window.open(
        `https://xiao-zh-cdn.xiao100.com/public/multi/pdf-viewer/web/viewer.html?file=${encodeURI(src)}`,
        '_blank',
      );
      return;
    }

    this.setState({
      imagePreviewModalVisible: true,
      previewSrc: src,
      previewFileName: fileName,
      previewUserName: createUserName,
      previewCreateTime: createTime,
    });
  };

  render() {
    const previewImageSrcs = this.props.fileList
      .map((file) => file.src)
      .filter((src) => !!src)
      .filter((src) => src.includes('jpg') || src.includes('png'));

    return (
      <div>
        <FileList
          data={this.props.fileList}
          onChange={this.props.onFileListChange}
          type={this.props.canEdit ? 'edit' : 'view'}
          canDelete={true}
          onPreview={this.onPreview}
        />
        {this.props.canEdit && this.props.fileList.length < 10 && (
          <Upload
            multiple={true}
            limit={{ type: 'jpg/jpeg/png/pdf' }}
            onChange={this.onUploadChange}
            beforeUpload={this.beforeUpload}
          >
            <span className={style.upload}>上传文件</span>
          </Upload>
        )}
        {this.state.imagePreviewModalVisible && (
          <ImagePreviewModal
            previewImageSrcs={previewImageSrcs}
            onClose={this.hideImagePreviewModal}
            previewSrc={this.state.previewSrc}
            fileName={this.state.previewFileName}
            title={this.state.previewFileName}
            name={this.state.previewUserName}
            time={this.state.previewCreateTime}
          />
        )}
      </div>
    );
  }
}

function mapStateToProps(state) {
  const btnAuths = state.account.btnAuths || [];

  return {
    canEdit: btnAuths.includes('OFFICE_SPACE_EDIT'),
    userName: state.account.user.name,
  };
}

export default connect(mapStateToProps)(Uploader);
