import React from 'react';
import { Modal, FormItem, Input, InputNumber, Select, InputTag, TextArea } from 'COMPONENT';
import { storeName } from '../store/config';
import { connect } from 'dva';
import FooterButtons from './FooterButtons';
import { Row, Col, DatePicker } from 'antd';
import style from './style.less';
import Uploader from './Uploader';
import moment from 'moment';
import { findOfficeSpaceById, modifyOfficeSpaceManage } from 'SERVICE/basicInformationManage';
import { requestWrapper } from 'UTIL/promiseHandler';
import { isEmpty } from '../store/utils';
import classNames from 'classnames';

class DetailModal extends React.Component {
  state = {
    loading: true,
    submitDisable: false,
    initialFileIds: [],
    phoneError: false,
    lessorPhoneError: false,
    principalPhoneError: false,

    deleteFileIds: [],

    officeSpace: '',
    purposeName: '',
    statusName: '',
    organizationName: '',
    shareInfoList: [],
    address: '',

    spaceArea: undefined,
    classroomNumber: undefined,
    deskNumber: undefined,
    boothNumber: undefined,
    twoBoothNumber: undefined,
    threeBoothNumber: undefined,
    fileInfoList: [],
    oneOnOnePreProduction: undefined,
    miniClassPreProduction: undefined,
    preSquarePerformance: undefined,
    realSquarePerformance: undefined,
    fullRate: undefined,

    lessor: undefined,
    lessorPhone: undefined,
    yearlyRent: undefined,
    monthlyRent: undefined,
    administrativeFee: undefined,
    airConditionFee: undefined,
    startDate: undefined,
    endDate: undefined,

    openingDate: undefined,
    phone: undefined,
    principal: undefined,
    principalPhone: undefined,
    spaceRemark: undefined,
  };

  componentDidMount() {
    this.init();
  }

  init = async () => {
    const [, res] = await requestWrapper(findOfficeSpaceById({ id: this.props.officeSpaceId }));
    if (res && res.data) {
      const { openingDate, startDate, endDate, fileInfoList, ...rest } = res.data;
      if (openingDate) {
        rest.openingDate = openingDate === '0000-00-00' ? '' : moment(openingDate);
      }
      if (startDate) {
        rest.startDate = startDate === '0000-00-00' ? '' : moment(startDate);
      }
      if (endDate) {
        rest.endDate = endDate === '0000-00-00' ? '' : moment(endDate);
      }
      this.setState({
        ...rest,
        initialFileIds: fileInfoList.map((file) => file.id),
        fileInfoList: this.initFileList(fileInfoList),
      });
    }
    this.setState({
      loading: false,
    });
  };

  updateFileInfoList = (fileInfoList) => {
    this.setState({
      fileInfoList,
    });
  };

  close = () => {
    this.props.dispatch({
      type: `${storeName}/closeDetailModal`,
    });
  };

  ok = async () => {
    this.setState({
      loading: true,
    });

    const {
      loading,
      submitDisable,
      officeSpace,
      purposeName,
      statusName,
      organizationName,
      shareInfoList,
      address,
      initialFileIds,
      fileInfoList,
      phoneError,
      lessorPhoneError,
      principalPhoneError,

      startDate,
      endDate,
      openingDate,
      ...params
    } = this.state;

    const phoneKeys = ['phone', 'lessorPhone', 'principalPhone'];

    const canSubmit = phoneKeys.every((key) => {
      return !this.state[`${key}Error`];
    });

    if (!canSubmit) {
      this.setState({
        loading: false,
      });
      return;
    }

    // find deleted file ids
    const allFileIds = fileInfoList.map((file) => file.id).filter((id) => !!id);
    const deleteFileIds = initialFileIds.filter((oldId) => {
      return !allFileIds.includes(oldId.toString());
    });

    // filter Empty Value
    const updateParma = {
      ...params,
      id: this.props.officeSpaceId,
      addFileInfoList: this.formatFileList(fileInfoList),
      startDate: startDate ? startDate.format('YYYY-MM-DD') : '',
      endDate: endDate ? endDate.format('YYYY-MM-DD') : '',
      openingDate: openingDate ? openingDate.format('YYYY-MM-DD') : '',
      deleteFileIds,
    };

    const notEmptyParams = Object.keys(updateParma)
      .filter((key) => !isEmpty(updateParma[key]))
      .reduce((accumulate, key) => {
        accumulate[key] = updateParma[key];
        return accumulate;
      }, {});

    const [, res] = await requestWrapper(modifyOfficeSpaceManage(notEmptyParams));
    if (res) {
      this.close();
    } else {
      this.setState({
        loading: false,
      });
    }
  };

  updateValue = (key, length, isNumber, precision) => {
    return (e) => {
      const val = e ? (e.target ? e.target.value : e) : '';

      if (isNumber) {
        if (isNaN(val)) {
          return;
        }
        const isDecimal = val % 1 != 0;
        const maxLength = isDecimal ? length + precision + 1 : length;
        const isInValid = maxLength - val.toString().length < 0;
        if (isInValid) {
          return;
        }
      }

      let hasError = false;
      if (val) {
        const phoneKeys = ['phone', 'lessorPhone', 'principalPhone'];
        if (phoneKeys.includes(key)) {
          const tel = /^1\d{10}$/;
          if (!tel.test(val)) {
            hasError = true;
          }
        }
      }

      this.setState({
        [`${key}Error`]: hasError,
      });

      this.setState({
        [key]: typeof val === 'string' ? val.trim() : val,
      });
    };
  };

  /**
   * 获取要禁用的开始日期值
   * @param  {string} startDate 过滤前的日期值
   * @return {boolean} 是否要禁用
   */
  getDisabledStartDate = (startDate) => {
    const { endDate } = this.state;
    if (!startDate || !endDate) {
      return false;
    }
    return moment(startDate).valueOf() > moment(endDate).valueOf();
  };

  /**
   * 获取要禁用的结束日期值
   * @param  {string} endDate   过滤前的日期值
   * @return {boolean}      是否要禁用
   */
  getDisabledEndDate = (endDate) => {
    const { startDate } = this.state;
    if (!endDate || !startDate) {
      return false;
    }
    return moment(endDate).valueOf() < moment(startDate).valueOf();
  };

  initFileList = (fileInfoList) => {
    /**
     bucket: "xhboss-test"
     fileKey: "0adc3a8a-6a00-bf3c-56a1-4decf59a1f1d.jpg"
     iconSrc: "https://xhboss-test.oss-cn-shenzhen.aliyuncs.com/0adc3a8a-6a00-bf3c-56a1-4decf59a1f1d.jpg"
     id: "0adc3a8a-6a00-bf3c-56a1-4decf59a1f1d.jpg"
     mimeType: "image/jpeg"
     name: "bg1.jpg"
     object: "0adc3a8a-6a00-bf3c-56a1-4decf59a1f1d.jpg"
     percent: 100
     showProgress: true
     size: 71119
     src: "https://xhboss-test.oss-cn-shenzhen.aliyuncs.com/0adc3a8a-6a00-bf3c-56a1-4decf59a1f1d.jpg"
     type: "image/jpeg"
     x:fileid: "UPL000000037925"
     x:realname: "bg1.jpg"
     x:uploadid: "c8873bea-5b56-fb10-017b-87440cb44bc2"
     x:uploadtype: "ipad"
     x:userid: null
     */
    return fileInfoList.map(
      ({ id, filePath: src = '', fileSize: size, realFileName: name, createUserName, createTime }) => {
        const isPdf = src.toLowerCase().includes('pdf');

        let extension = '';
        if (src) {
          extension = src.split('.').pop();
        }

        return {
          id: id + '',
          iconSrc: isPdf ? '' : src,
          src,
          size,
          name,
          createUserName,
          createTime,
          percent: 100,
          showProgress: false,
          type: extension,
          isOld: true,
        };
      },
    );
  };

  formatFileList = (fileInfoList) => {
    return fileInfoList.map(
      ({ id, src: filePath, size: fileSize, name: realFileName, createUserName, createTime, isOld }) => {
        const fileInfo = {
          filePath,
          fileSize,
          realFileName,
        };

        if (isOld) {
          fileInfo.id = id + '';
          fileInfo.createUserName = createUserName;
          fileInfo.createTime = createTime;
        }

        return fileInfo;
      },
    );
  };

  render() {
    return (
      <Modal
        loading={this.state.loading}
        onClose={this.close}
        title="场地信息"
        width={1100}
        footer={<FooterButtons onOk={this.ok} onCancel={this.close} confirmBtnDisabled={this.state.submitDisable} />}
      >
        <div className={style.container}>
          <div className={style.hr}></div>
          <div className={style.title}>基础信息</div>
          <Row gutter={24}>
            <Col span={8}>
              <FormItem size="small" name="场地名称：">
                {this.state.officeSpace}
              </FormItem>
            </Col>
            <Col span={8}>
              <FormItem size="small" name="场地用途：">
                {this.state.purposeName}
              </FormItem>
            </Col>
            <Col span={8}>
              <FormItem size="small" name="场地状态：">
                {this.state.statusName}
              </FormItem>
            </Col>
          </Row>
          <Row gutter={24}>
            <Col span={8}>
              <FormItem size="small" name="所属组织:">
                {this.state.organizationName}
              </FormItem>
            </Col>
            <Col span={8}>
              <FormItem size="small" name="共用组织:">
                {this.state.shareInfoList
                  ? this.state.shareInfoList.map((item) => item.organizationName).join(',')
                  : '-'}
              </FormItem>
            </Col>
          </Row>
          <FormItem size="small" name="场地产权证地址:">
            {this.state.address}
          </FormItem>
          <div className={style.hr}></div>
          <div className={style.title}>产能信息</div>
          <Row gutter={24}>
            <Col span={8}>
              <FormItem labelWidth={110} name="套内面积">
                <InputNumber
                  precision={2}
                  size="small"
                  value={this.state.spaceArea}
                  onChange={this.updateValue('spaceArea', 9, true, 2)}
                  allowClear={true}
                  className={style.smallInput}
                  disabled={!this.props.canEdit}
                />
              </FormItem>
            </Col>
            <Col span={8}>
              <FormItem labelWidth={110} name="教室总数">
                <InputNumber
                  size="small"
                  value={this.state.classroomNumber}
                  onChange={this.updateValue('classroomNumber', 9, true)}
                  allowClear={true}
                  className={style.smallInput}
                  disabled={!this.props.canEdit}
                />
              </FormItem>
            </Col>
            <Col span={8}>
              <FormItem labelWidth={110} name="教室课桌总数">
                <InputNumber
                  size="small"
                  value={this.state.deskNumber}
                  onChange={this.updateValue('deskNumber', 9, true)}
                  allowClear={true}
                  className={style.smallInput}
                  disabled={!this.props.canEdit}
                />
              </FormItem>
            </Col>
          </Row>
          <FormItem labelWidth={110} name="卡座数">
            <Row gutter={24}>
              <Col span={8}>
                <InputTag
                  size="small"
                  value={this.state.boothNumber}
                  onChange={this.updateValue('boothNumber', 9, true)}
                  disabled={!this.props.canEdit}
                >
                  1对1
                </InputTag>
              </Col>
              <Col span={8}>
                <InputTag
                  size="small"
                  value={this.state.twoBoothNumber}
                  onChange={this.updateValue('twoBoothNumber', 8, true)}
                  disabled={!this.props.canEdit}
                >
                  1对2
                </InputTag>
              </Col>
              <Col span={8}>
                <InputTag
                  size="small"
                  value={this.state.threeBoothNumber}
                  onChange={this.updateValue('threeBoothNumber', 8, true)}
                  disabled={!this.props.canEdit}
                >
                  1对3
                </InputTag>
              </Col>
            </Row>
          </FormItem>
          <FormItem labelWidth={110} name="工程平面图">
            <Uploader fileList={[...this.state.fileInfoList]} onFileListChange={this.updateFileInfoList} />
          </FormItem>
          <Row gutter={24}>
            <Col span={8}>
              <FormItem labelWidth={110} name="一对一预估产值">
                <InputNumber
                  precision={2}
                  value={this.state.oneOnOnePreProduction}
                  size="small"
                  className={style.smallInput}
                  onChange={this.updateValue('oneOnOnePreProduction', 10, true, 2)}
                  allowClear={true}
                  disabled={!this.props.canEdit}
                />
              </FormItem>
            </Col>
            <Col span={8}>
              <FormItem labelWidth={110} name="小班预估产值">
                <InputNumber
                  precision={2}
                  value={this.state.miniClassPreProduction}
                  size="small"
                  className={style.smallInput}
                  onChange={this.updateValue('miniClassPreProduction', 10, true, 2)}
                  allowClear={true}
                  disabled={!this.props.canEdit}
                />
              </FormItem>
            </Col>
          </Row>
          <Row gutter={24}>
            <Col span={8}>
              <FormItem labelWidth={110} name="预估平效">
                <InputNumber
                  precision={2}
                  value={this.state.preSquarePerformance}
                  size="small"
                  className={style.smallInput}
                  onChange={this.updateValue('preSquarePerformance', 10, true, 2)}
                  allowClear={true}
                  disabled={!this.props.canEdit}
                />
              </FormItem>
            </Col>
            <Col span={8}>
              <FormItem labelWidth={110} name="实际平效">
                <InputNumber
                  precision={2}
                  value={this.state.realSquarePerformance}
                  size="small"
                  className={style.smallInput}
                  onChange={this.updateValue('realSquarePerformance', 10, true, 2)}
                  allowClear={true}
                  disabled={!this.props.canEdit}
                />
              </FormItem>
            </Col>
            <Col span={8}>
              <FormItem labelWidth={110} name="满座率%">
                <InputNumber
                  precision={2}
                  value={this.state.fullRate}
                  size="small"
                  className={style.smallInput}
                  onChange={this.updateValue('fullRate', 8, true, 2)}
                  allowClear={true}
                  disabled={!this.props.canEdit}
                />
              </FormItem>
            </Col>
          </Row>

          <div className={style.hr}></div>
          <div className={style.title}>租赁信息</div>
          <Row gutter={24}>
            <Col span={8}>
              <FormItem labelWidth={110} name="出租方">
                <Input
                  value={this.state.lessor}
                  size="small"
                  maxLength={50}
                  className={style.smallInput}
                  onChange={this.updateValue('lessor')}
                  allowClear={true}
                />
              </FormItem>
            </Col>
            <Col span={16}>
              <FormItem labelWidth={110} name="出租方联系电话">
                <Row>
                  <Col span={9}>
                    <Input
                      value={this.state.lessorPhone}
                      size="small"
                      className={classNames(style.smallInput, { [style.hasError]: this.state.lessorPhoneError })}
                      onChange={this.updateValue('lessorPhone')}
                      allowClear={true}
                    />
                  </Col>
                  <Col span={12}>
                    {this.state.lessorPhoneError && <span className={style.error}>请填写正确的联系电话</span>}
                  </Col>
                </Row>
              </FormItem>
            </Col>
          </Row>
          <Row gutter={24}>
            <Col span={8}>
              <FormItem labelWidth={110} name="年租金">
                <InputNumber
                  precision={2}
                  value={this.state.yearlyRent}
                  size="small"
                  className={style.smallInput}
                  onChange={this.updateValue('yearlyRent', 8, true, 2)}
                  allowClear={true}
                />
              </FormItem>
            </Col>
            <Col span={8}>
              <FormItem labelWidth={110} name="月租金">
                <InputNumber
                  precision={2}
                  value={this.state.monthlyRent}
                  size="small"
                  className={style.smallInput}
                  onChange={this.updateValue('monthlyRent', 8, true, 2)}
                  allowClear={true}
                />
              </FormItem>
            </Col>
          </Row>
          <Row gutter={24}>
            <Col span={8}>
              <FormItem labelWidth={110} name="管理费">
                <InputNumber
                  precision={2}
                  value={this.state.administrativeFee}
                  size="small"
                  className={style.smallInput}
                  onChange={this.updateValue('administrativeFee', 8, true, 2)}
                  allowClear={true}
                />
              </FormItem>
            </Col>
            <Col span={8}>
              <FormItem labelWidth={110} name="空调费">
                <InputNumber
                  precision={2}
                  value={this.state.airConditionFee}
                  size="small"
                  className={style.smallInput}
                  onChange={this.updateValue('airConditionFee', 8, true, 2)}
                  allowClear={true}
                />
              </FormItem>
            </Col>
          </Row>
          <Row gutter={24}>
            <Col span={8}>
              <FormItem labelWidth={110} name="合同开始日期">
                <DatePicker
                  format="YYYY-MM-DD"
                  value={this.state.startDate}
                  onChange={(startDate) => this.setState({ startDate })}
                  disabledDate={this.getDisabledStartDate}
                  allowClear={true}
                  style={{
                    width: '120px',
                  }}
                />
              </FormItem>
            </Col>
            <Col span={8}>
              <FormItem labelWidth={110} name="合同结束日期">
                <DatePicker
                  format="YYYY-MM-DD"
                  value={this.state.endDate}
                  onChange={(endDate) => this.setState({ endDate })}
                  disabledDate={this.getDisabledEndDate}
                  allowClear={true}
                  style={{
                    width: '120px',
                  }}
                />
              </FormItem>
            </Col>
          </Row>

          <div className={style.hr}></div>
          <div className={style.title}>其他信息</div>
          <Row gutter={24}>
            <Col span={8}>
              <FormItem labelWidth={110} name="开业日期">
                <DatePicker
                  format="YYYY-MM-DD"
                  value={this.state.openingDate}
                  onChange={(openingDate) => this.setState({ openingDate })}
                  disabledDate={this.getDisabledEndDate}
                  allowClear={true}
                  style={{
                    width: '120px',
                  }}
                />
              </FormItem>
            </Col>
            <Col span={16}>
              <FormItem labelWidth={110} name="联系电话">
                <Row>
                  <Col span={9}>
                    <Input
                      value={this.state.phone}
                      size="small"
                      className={classNames(style.smallInput, { [style.hasError]: this.state.phoneError })}
                      onChange={this.updateValue('phone')}
                      allowClear={true}
                    />
                  </Col>
                  <Col span={12}>
                    {this.state.phoneError && <span className={style.error}>请填写正确的联系电话</span>}
                  </Col>
                </Row>
              </FormItem>
            </Col>
          </Row>
          <Row gutter={24}>
            <Col span={8}>
              <FormItem labelWidth={110} name="负责人">
                <Input
                  value={this.state.principal}
                  maxLength={20}
                  size="small"
                  className={style.smallInput}
                  onChange={this.updateValue('principal')}
                  allowClear={true}
                />
              </FormItem>
            </Col>
            <Col span={16}>
              <FormItem labelWidth={110} name="负责人电话">
                <Row>
                  <Col span={9}>
                    <Input
                      value={this.state.principalPhone}
                      size="small"
                      className={classNames(style.smallInput, { [style.hasError]: this.state.principalPhoneError })}
                      onChange={this.updateValue('principalPhone')}
                      allowClear={true}
                    />
                  </Col>
                  <Col span={12}>
                    {this.state.principalPhoneError && <span className={style.error}>请填写正确的联系电话</span>}
                  </Col>
                </Row>
              </FormItem>
            </Col>
          </Row>
          <FormItem labelWidth={110} name="备注">
            <TextArea
              rows={6}
              maxLength={500}
              value={this.state.spaceRemark}
              size="small"
              onChange={this.updateValue('spaceRemark')}
            />
          </FormItem>
        </div>
      </Modal>
    );
  }
}

function mapStateToProps(state) {
  const btnAuths = state.account.btnAuths || [];

  return {
    canEdit: btnAuths.includes('OFFICE_SPACE_EDIT'),
    officeSpaceId: state[storeName].officeSpaceId,
  };
}

export default connect(mapStateToProps)(DetailModal);
