import React from 'react';
import { connect } from 'dva';
import { Select, Input, SearchBar, Button } from 'COMPONENT';
import { storeName } from '../store/config';
import { statusFinderDict } from '../dicts';
import PropTypes from 'prop-types';
import { TreeSelect } from 'antd';
import { listOrganizationTreeData } from '../loader';
import styles from './styles.less';

const initialState = {};

class Search extends React.Component {
  static propTypes = {
    onSearch: PropTypes.func.isRequired,
  };

  state = {
    ...initialState,
    treeDefaultExpandedKeys: [],
    treeData: [],
  };

  componentDidMount() {
    this.initDict();
  }

  initDict = async () => {
    const treeData = await listOrganizationTreeData();
    const treeDefaultExpandedKeys = treeData.length > 0 ? treeData.map((item) => item.key) : [];
    this.setState({
      treeData,
      treeDefaultExpandedKeys,
    });
  };

  updateValue = (key) => {
    return (e) => {
      const val = e ? (e.target ? e.target.value : e) : undefined;
      this.props.dispatch({
        type: `${storeName}/updateState`,
        payload: {
          [key]: typeof val === 'string' ? val.trim() : val,
        },
      });
    };
  };

  onReset = () => {
    this.props.dispatch({
      type: `${storeName}/resetSearch`,
    });
    this.setState({
      ...initialState,
    });
  };

  handleExport = () => {
    this.props.dispatch({
      type: `${storeName}/exportToExcel`,
    });
  };

  render() {
    return (
      <div className={styles.searchWarp}>
        <SearchBar
          style={{ width: '100%' }}
          // searchBtnLoading={props.searchBtnLoading}
          onSearch={this.props.onSearch}
          onReset={this.onReset}
        >
          <Input
            placeholder="模糊搜索场地名称"
            value={this.props.sKey_officeSpace}
            onChange={this.updateValue('sKey_officeSpace')}
            allowClear={true}
          />
          <TreeSelect
            size="large"
            allowClear
            value={this.props.sKey_organizationId}
            dropdownStyle={{ maxHeight: 800, overflow: 'auto' }}
            treeData={this.state.treeData}
            placeholder="所属组织"
            treeDefaultExpandedKeys={this.state.treeDefaultExpandedKeys}
            onChange={this.updateValue('sKey_organizationId')}
            showSearch
            treeNodeFilterProp="label"
          />
          <TreeSelect
            size="large"
            allowClear
            value={this.props.sKey_shareOrganizationId}
            dropdownStyle={{ maxHeight: 600, overflow: 'auto' }}
            treeData={this.state.treeData}
            placeholder="共用组织"
            treeDefaultExpandedKeys={this.state.treeDefaultExpandedKeys}
            onChange={this.updateValue('sKey_shareOrganizationId')}
            showSearch
            treeNodeFilterProp="label"
          />
          <Select
            placeholder="场地状态"
            dict={statusFinderDict}
            value={this.props.sKey_status}
            onChange={this.updateValue('sKey_status')}
            allowClear={true}
          />
        </SearchBar>
        <Button className={styles.export} onClick={this.handleExport}>
          导出excel
        </Button>
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    sKey_officeSpace: state[storeName].sKey_officeSpace,
    sKey_organizationId: state[storeName].sKey_organizationId,
    sKey_shareOrganizationId: state[storeName].sKey_shareOrganizationId,
    sKey_status: state[storeName].sKey_status,
  };
};

export default connect(mapStateToProps)(Search);
