import React from 'react';
import { connect } from 'dva';
import { List, TextLink } from 'COMPONENT';
import { storeName } from '../store/config';
import moment from 'moment';
import { Table } from 'antd';
import PropTypes from 'prop-types';

const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L } = List;

class TableList extends React.Component {
  static propTypes = {
    // id: PropTypes.string.isRequired,
    loadList: PropTypes.func.isRequired,
  };

  state = {
    scrollY: '',
  };

  componentDidMount() {
    setTimeout(() => {
      this.calculateScrollY();
    }, 10);
  }

  calculateScrollY = () => {
    const searchDomRect = document.querySelector('.Page-header').getClientRects()[0];
    // 搜索+分页高度
    let y = searchDomRect.top + searchDomRect.height + 150;

    this.setState({
      scrollY: `calc(100vh - ${y}px)`,
    });
  };

  updatePagination = ({ current: pageCurrent, pageSize }) => {
    let payload = {};
    if (pageSize && pageSize !== this.props.pageSize) {
      payload.pageCurrent = 1;
      payload.pageSize = pageSize;
    } else {
      payload.pageCurrent = pageCurrent;
    }
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload,
    });

    this.props.loadList();
  };

  openDetailModal = (record) => {
    this.props.dispatch({
      type: `${storeName}/openDetailModal`,
      payload: {
        officeSpaceId: record.id,
      },
    });
  };
  columns = [
    { title: '场地名称', dataIndex: 'officeSpace', width: COL_WIDTH_M },
    { title: '所属组织', dataIndex: 'organizationName', width: COL_WIDTH_M },
    {
      title: '共用组织',
      dataIndex: 'shareInfoList',
      width: COL_WIDTH_M + 20,
      render: (shareInfoList, record, index) => {
        return shareInfoList ? shareInfoList.map((item) => item.organizationName).join(',') : '-';
      },
    },
    { title: '场地状态', dataIndex: 'statusName', width: COL_WIDTH_M },
    { title: '1对1卡座数', dataIndex: 'boothNumber', width: COL_WIDTH_M, render: (val) => val || '-' },
    { title: '1对2卡座数', dataIndex: 'twoBoothNumber', width: COL_WIDTH_M, render: (val) => val || '-' },
    { title: '1对3卡座数', dataIndex: 'threeBoothNumber', width: COL_WIDTH_M, render: (val) => val || '-' },
    { title: '场地开业日期', dataIndex: 'openingDate', width: COL_WIDTH_M, render: (val) => val || '-' },
    { title: '最近更新时间', dataIndex: 'modifyTime', width: COL_WIDTH_M },
    {
      title: '操作',
      key: 'action',
      width: COL_WIDTH_M,
      render: (text, record, index) => {
        const actions = [{ text: '详情', onClick: () => this.openDetailModal(record) }];

        return (
          <span>
            {actions.map((action, i) => {
              return (
                <TextLink key={i} className="list-btn" onClick={action.onClick}>
                  {action.text}
                </TextLink>
              );
            })}
          </span>
        );
      },
    },
  ];

  render() {
    const { records, listLoading, rowCount, pageCurrent, pageSize } = this.props;
    const { scrollY } = this.state;

    return (
      <Table
        rowKey="id"
        dataSource={records}
        columns={this.columns}
        loading={listLoading}
        pagination={{
          howQuickJumper: true,
          pageSizeOptions: ['10', '20', '50'],
          showSizeChanger: true,
          showTotal: (total) => `共 ${total} 条记录`,
          current: pageCurrent,
          total: rowCount,
          pageSize,
        }}
        scroll={{
          // x: 1800,
          y: scrollY,
        }}
        onChange={this.updatePagination}
      />
    );
  }
}

const mapStateToProps = (state) => {
  return {
    records: state[storeName].records,
    rowCount: state[storeName].rowCount,
    pageSize: state[storeName].pageSize,
    pageCurrent: state[storeName].pageCurrent,
    listLoading: state[storeName].listLoading,
  };
};

export default connect(mapStateToProps)(TableList);
