import { storeName } from './config';
import { New, getOfficeSpaceManageListNew, getOfficeSpaceManageListNewExport } from 'SERVICE/basicInformationManage';
import { clone } from 'lodash';
import { isEmpty } from './utils';
import { requestWrapper } from 'UTIL/promiseHandler';

const defaultState = {
  sKey: {
    sKey_officeSpace: undefined,
    sKey_organizationId: undefined,
    sKey_shareOrganizationId: undefined,
    sKey_status: '',
  },
  list: {
    records: [],
    rowCount: 0,
    pageSize: 20,
    pageCurrent: 1,
    listLoading: false,
  },
  detailModal: {
    detailModalVisible: false,
    officeSpaceId: '',
  },
};
Object.freeze(defaultState);

export default {
  namespace: storeName,
  state: {
    ...clone(defaultState.sKey),
    ...clone(defaultState.list),
    ...clone(defaultState.detailModal),
  },
  reducers: {
    updateState(state, action) {
      const { payload } = action;
      return {
        ...state,
        ...payload,
      };
    },

    updateList(state, { payload: { records, rowCount } }) {
      const mapRecords = records.map((record) => ({
        ...record,
        // some extra data
      }));

      return {
        ...state,
        records: mapRecords,
        rowCount,
      };
    },

    openDetailModal(state, { payload }) {
      return {
        ...state,
        ...payload,
        detailModalVisible: true,
      };
    },

    closeDetailModal(state) {
      return {
        ...state,
        ...clone(defaultState.detailModal),
        ...clone(defaultState.detailModalData),
      };
    },
  },
  effects: {
    *resetSearch(action, { put }) {
      yield put({
        type: 'updateState',
        payload: {
          ...clone(defaultState.sKey),
        },
      });
    },
    *loadList(action, { select, put }) {
      yield put({
        type: 'updateState',
        payload: {
          listLoading: true,
        },
      });
      const sKeys = yield select(searchValuesSelector);

      // convert params
      if (sKeys.status) {
        sKeys.status = +sKeys.status;
      }
      const apiParams = yield select((state) => {
        return {
          ...sKeys,
          pageNum: state[storeName].pageCurrent,
          pageSize: state[storeName].pageSize,
        };
      });
      const [err, response] = yield requestWrapper(getOfficeSpaceManageListNew(apiParams));
      if (!err) {
        yield put({
          type: 'updateList',
          payload: {
            records: response.data.list || [],
            rowCount: response.data.total,
          },
        });
      }
      yield put({
        type: 'updateState',
        payload: {
          listLoading: false,
        },
      });
    },

    *exportToExcel(action, { select, put }) {
      const sKeys = yield select(searchValuesSelector);

      const apiParams = yield select((state) => {
        return {
          ...sKeys,
          pageNum: 1,
          pageSize: 999,
        };
      });
      const [err, response] = yield requestWrapper(getOfficeSpaceManageListNewExport(apiParams));
      if (!err && response.data) {
        window.open(response.data);
      }
    },
  },
};

function searchValuesSelector(state) {
  const store = state[storeName];
  return Object.keys(defaultState.sKey)
    .filter((key) => !isEmpty(store[key]))
    .reduce((accumulate, key) => {
      const val = store[key];
      const newKey = key.replace('sKey_', '');
      accumulate[newKey] = val;
      return accumulate;
    }, {});
}
