import { saveConsultorCustomerStudent, getStudentListByCusContact } from 'SERVICE/consultant';
import { getLoginUserProvinceCity } from 'SERVICE/system';

const addCustomer = async ({
  workBench,
  name,
  phone,
  entrance,
  source,
  sourceDetail,
  campus,
  follower,
  remark,
  checkedLinkStudents,
  newStudents,
  customerId,
  customerSign,
  callType,
  cusIdCardNo,
  cusIdCardType,
}) => {
  const studentImportVos =
    newStudents && newStudents.length > 0
      ? newStudents.map((stu) => {
          const { name, grade, cls, school, sex, birthday, contract, address, idCardNo } = stu;
          return {
            name: name.trim(),
            gradeId: grade,
            classes: cls,
            schoolId: school,
            sex: sex,
            bothday: birthday,
            contact: contract,
            address: address,
            idCardNo: idCardNo,
          };
        })
      : [];
  const params = {
    workBench,
    cusId: customerId,
    cusName: name,
    contact: phone,
    resEntranceId: entrance,
    cusType: source,
    cusOrg: sourceDetail,
    intentionCampusId: campus,
    deliverTarget: follower,
    remark: remark,
    oldStudents: checkedLinkStudents.map((stu) => {
      return { studentId: stu.id, delivertarget: stu.follower, schoolId: stu.schoolId, gradeId: stu.gradeId };
    }),
    studentImportVos,
    customerSign,
    callType,
    cusIdCardNo,
    cusIdCardType,
  };
  await saveConsultorCustomerStudent(params);
  return true;
};

const loadLinkStudents = async ({ phone }) => {
  try {
    const {
      data: { studentList, customerId },
    } = await getStudentListByCusContact({ contact: phone }, true);
    if (!studentList) {
      return { customerId: null, list: null };
    }
    let list = studentList.map((student) => {
      const {
        status,
        canFollow,
        studentId,
        gradeId,
        haveLabel,
        finishDay,
        schoolId,
        gradeName,
        studentType,
        studentName,
        schoolName,
        blCampusName,
      } = student;
      return {
        status,
        haveLabel,
        finishDay,
        gradeName,
        canFollow,
        schoolName,
        studentName,
        studentType,
        blCampusName,
        follower: null,
        id: studentId,
        gradeId: gradeId,
        name: studentName,
        schoolId: schoolId,
      };
    });
    /*    list = [
          ...list,
          {id: 'xx1', name: 'xxx1', gradeName: 'dd', schoolName: 'dddd', canFollow: true},
          {id: 'xx2', name: 'xxx2', gradeName: 'dd', schoolName: 'dddd', canFollow: true},
          {id: 'xx3', name: 'xxx3', gradeName: 'dd', schoolName: 'dddd', canFollow: false},
          ]*/
    /*  const list = [
        { id: '1', name: '王甜甜', grade: '402881e647ce90330147ce94ef900006', schoolName: '培正中学' },
        { id: '2', name: '李甜甜', grade: '402881e647ce90330147ce94ef900006', schoolName: '培正中学' },
      ]*/
    return { customerId, list };
  } catch (e) {
    throw e;
  }
};

const loadLoginUserProvinceCity = async () => {
  const apiData = await getLoginUserProvinceCity();
  const data = apiData ? apiData.data : {};
  return { province: data.provinceId || null, city: data.cityId || null };
};

export default {
  addCustomer,
  loadLinkStudents,
  loadLoginUserProvinceCity,
};
