import React from 'react';
import styles from './styles.less';
import { getFieldWithFiller, getCommonErrorTips } from 'COMPONENT/utils/form';
import loaders from '../loaders';
import { Block, FormItem, Input, Select, DatePicker, TextArea, PopTip, Modal, Section, Button } from 'COMPONENT';
import { getDictTextByVal, loadCommonDict, loadCommonDictNew, loadAllBranchDict, loadCampusDict } from 'UTIL/dict';
import { loadSourceDetailDict } from 'PAGE/common/loaders';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import SelectSchoolModal from 'PAGE/common/SelectSchoolModal';
import { CUS_LIST_LOST } from '../../../consultant/FollowCustomer/dicts';
import { findOrganizationById } from 'SERVICE/common';
import { loginUser, isIdCard } from 'UTIL/common';
import tips from 'COMPONENT/Validator/tips';
import { checkPhone } from 'COMPONENT/Validator/checkers';

const CERTIFICATE_TYPE_ID_CARD = 'CT_IDENTITY_CARD';

const formCfg = {
  labelWidth: 100,
  size: 'small',
};

function checkPhonePlus(value) {
  if (value && value.length > 0) {
    return checkPhone(value) && tips.phone;
  }
  return null;
}

class CusDetailBlock extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      editing: false, // {boolean} 是否编辑状态
      checking: false, // {boolean} 校验开启状态
      initData: null, // {object} 备份数据，用于取消时回退

      studentName: null, // {string} 学生姓名
      grade: null, // {string} 年级
      school: null, // {string} 学校
      schoolName: null, // {string} 学校名称
      schoolLevelName: null, // {string} 学校级别
      address: null, // {string} 家庭住址
      studentContract: null, // {string} 联系方式
      birthDay: null, // {string} 出生日期

      cusName: null, // {string} 客户姓名
      entrance: null, // {string} 资源入口
      source: null, // {string} 资源来源
      sourceDetail: null, // {string} 资源细分
      campus: null, // {string} 所属学习中心
      cusContract: null, // {string} 客户联系电话
      dispatcher: null, // {string} 分配人
      follower: null, // {string} 跟进人
      followStatus: null, // {string} 跟进状态
      followStatusName: null, // {string} 跟进状态名称
      branch: null, // {string} 所属分公司
      remark: '', // {string} 备注
      cusIdCardNo: '', // 客户身份证
      studentFatherPhone: null,
      studentMotherPhone: null,
      isImport: '1',
      introducer: '', // {string} 推荐人
      introductionType: '', // {string} 推荐类型

      reactivated: '',
      preEntranceId: '',
      preEntranceName: '',

      gradeDict: [], // {object[]} 字典，年级
      entranceDict: [], // {object[]} 字典，资源入口
      sourceDict: [], // {object[]} 字典，资源来源
      sourceDetailDict: [], // {object[]} 字典，资源细分
      followStatusDict: [], // {object[]} 字典，跟进状态
      branchDict: [], // {object[]} 字典，所属分公司
      campusDict: [], // {object[]} 字典，所属学习中心
      initBranchDict: [], // {object[]} 初始所属分公司字典，用于取消时回退
      initCampusDict: [], // {object[]} 初始所属学习中心字典，用于取消时回退

      saveTipsVis: false, // {boolean} 保存按钮提示气泡显示状态
      saveTips: null, // {string} 保存按钮提示信息

      selectSchoolModalVis: false, // {boolean} 选择学校弹框显示状态
      mergeStudentInfoModalVisible: false,
      mergeStudentInfoMsg: null,
      newStudentId: null,
      isMergeLost: null,
      provinceId: null,
      cityId: null,

      isFinishOrStopClass: false,

      isXH: loginUser.isXingHuoSym(),

      certificateTypeDict: [], // 字典，证件类型
      cusIdCardTypeName: '',
    };
    this.errors = {};
  }

  initContent = async () => {
    const { id, stuId, deliverTarget } = this.props;
    const { organizationId, campusId, brenchId } = this.props.user;
    const [cusDetail, gradeDict, entranceDict, sourceDict, branchDict, organizationInfo, certificateTypeDict] =
      await Promise.all([
        loaders.loadCusDetail(id, stuId, deliverTarget),
        loadCommonDictNew('STUDENT_GRADE'),
        loadCommonDict('RES_ENTRANCE'),
        loadCommonDict('RES_TYPE'),
        loadAllBranchDict(),
        findOrganizationById({ id: campusId || brenchId }),
        loadCommonDict('CERTIFICATE_TYPE'),
      ]);
    const { branch, cusIdCardTypeName } = cusDetail;
    let campusDict = [];
    if (branch) {
      campusDict = await loadCampusDict(branch);
    }
    let sourceDetailDict = [];
    if (cusDetail.source) {
      sourceDetailDict = await loadSourceDetailDict(cusDetail.source);
    }

    let provinceId = '';
    let cityId = '';
    if (organizationId === '000001') {
      provinceId = '440000';
      cityId = '441900';
    } else {
      const { city, province } = organizationInfo;
      provinceId = province.id;
      cityId = city.id;
    }

    const isFinishOrStopClass =
      cusDetail.entrance === 'ACTIVATE_FINISH_CLASS' || cusDetail.entrance === 'ACTIVATE_STOP_CLASS';

    const identityTypeList =
      this.state.cusIdCardType && this.state.cusIdCardType !== 'CT_IDENTITY_CARD'
        ? certificateTypeDict
        : [{ text: '身份证', val: 'CT_IDENTITY_CARD' }];
    this.setState({
      isFinishOrStopClass,
      ...cusDetail,
      initData: cusDetail,
      gradeDict,
      entranceDict,
      sourceDict,
      sourceDetailDict,
      branchDict,
      campusDict,
      initBranchDict: branchDict,
      initCampusDict: campusDict,
      provinceId,
      cityId,
      certificateTypeDict: identityTypeList,
      cusIdCardTypeName,
    });
    return cusDetail;
  };

  cancel = () => {
    const { initData, initBranchDict, initCampusDict } = this.state;
    this.setState({
      editing: false,
      ...initData,
      branchDict: initBranchDict,
      campusDict: initCampusDict,
    });
  };

  onOk = () => {
    const { id, onLoadingStart, onLoadingEnd, stuId } = this.props;
    const {
      checking,
      studentName,
      grade,
      school,
      address,
      studentContract,
      birthDay,
      cusName,
      source,
      sourceDetail,
      remark,
      cusIdCardNo,
      cusIdCardType,
      branchDict,
      campusDict,
      initData,
    } = this.state;
    if (!checking) {
      this.setState({ checking: true });
    }
    const errorTips = getCommonErrorTips(this.errors);
    if (errorTips) {
      this.showSaveTips(errorTips);
      return;
    }
    if (errorTips) {
      this.showSaveTips(errorTips);
      return;
    }

    const shouldReturn = (cusIdCardType && !cusIdCardNo) || (!cusIdCardType && cusIdCardNo);
    if (shouldReturn) {
      const msg = !cusIdCardType ? '请选择客户证件类型！' : '请填写客户证件号码！';
      return this.showSaveTips(msg);
    }

    // 校验身份证号码
    if (cusIdCardType) {
      if (cusIdCardType === CERTIFICATE_TYPE_ID_CARD && !isIdCard(cusIdCardNo)) {
        return this.showSaveTips('客户身份证格式错误');
      } else if (!cusIdCardNo.trim()) {
        return this.showSaveTips('客户证件号码错误');
      }
    }

    const details = {
      studentName,
      grade,
      school,
      address,
      studentContract,
      birthDay,
      cusName,
      source,
      sourceDetail,
      remark,
      cusIdCardNo,
      cusIdCardType,
      studentFatherPhone: this.state.studentFatherPhone,
      studentMotherPhone: this.state.studentMotherPhone,
    };
    onLoadingStart();
    loaders.saveCusDetail({ id, stuId, ...details }).then(
      (res) => {
        const data = res.data || {};
        const mergeStudentInfo = data.mergeStudentInfo;
        if (mergeStudentInfo) {
          this.setState({
            newStudentId:
              mergeStudentInfo.mergeSuccess && !mergeStudentInfo.lost ? mergeStudentInfo.newStudentId : null,
            mergeStudentInfoModalVisible: true,
            isMergeLost: mergeStudentInfo.lost,
            mergeStudentInfoMsg: mergeStudentInfo.mergeMessage,
          });
          if (!mergeStudentInfo.mergeSuccess) {
            return onLoadingEnd();
          }
        } else {
          showMessage('保存成功', 'success');
        }

        this.setState({
          checking: false,
          initData: { ...initData, ...details },
          initBranchDict: branchDict,
          initCampusDict: campusDict,
        });
        this.initContent()
          .catch(() => {
            // do nothing
          })
          .finally(() => {
            this.setState({ loading: false, editing: false });
            onLoadingEnd();
          });
      },
      (e) => {
        if (e.text) {
          this.showSaveTips(e.text);
        } else {
          showMessage('保存失败');
        }
        onLoadingEnd();
      },
    );
  };

  showSaveTips = (text) => {
    this.setState({ saveTipsVis: true, saveTips: text || '请正确填写信息' });
  };

  sourceChange = (val) => {
    const { onLoadingStart, onLoadingEnd } = this.props;
    if (val) {
      onLoadingStart();
      loadSourceDetailDict(val).then(
        (dict) => {
          this.setState({
            source: val,
            sourceDetail: null,
            sourceDetailDict: dict,
          });
          onLoadingEnd();
        },
        () => {
          onLoadingEnd();
        },
      );
    } else {
      this.setState({ source: null, sourceDetail: null, sourceDetailDict: [] });
    }
  };

  errorChange = (key, error) => {
    this.errors[key] = error;
  };

  branchChange = (val) => {
    const { onLoadingStart, onLoadingEnd } = this.props;
    if (val) {
      onLoadingStart();
      loadCampusDict(val).then(
        (dict) => {
          this.setState({ branch: val, campus: null, campusDict: dict });
          onLoadingEnd();
        },
        () => {
          onLoadingEnd();
        },
      );
    } else {
      this.setState({ branch: null, campus: null, campusDict: [] });
    }
  };

  onMergeModalClose = () => {
    const { newStudentId, isMergeLost } = this.state;
    const { pageParams, onCloseTab, dispatch } = this.props;
    if (pageParams.onRefresh) {
      if (newStudentId) {
        pageParams.onRefresh({
          ...pageParams,
          stuId: newStudentId,
        });
      }
      if (isMergeLost === true) {
        dispatch({
          type: 'main/openPage',
          payload: {
            params: {
              initCusList: CUS_LIST_LOST,
            },
            refresh: true,
            path: '/ConsultantFollowCustomer',
          },
        });
      }
      onCloseTab();
    }
  };

  render() {
    const { editable } = this.props;
    const {
      editing,
      checking,
      studentName,
      grade,
      schoolName,
      schoolLevelName,
      address,
      studentContract,
      birthDay,
      cusName,
      entrance,
      source,
      sourceDetail,
      campus,
      cusContract,
      dispatcher,
      isImport,
      follower,
      followStatus,
      followStatusName,
      branch,
      remark,
      cusIdCardNo,
      gradeDict,
      entranceDict,
      sourceDict,
      sourceDetailDict,
      campusDict,
      followStatusDict,
      branchDict,
      saveTipsVis,
      saveTips,
      selectSchoolModalVis,
      mergeStudentInfoModalVisible,
      mergeStudentInfoMsg,
      introducer,
      introductionType,
      provinceId,
      cityId,
      isFinishOrStopClass,
      preEntranceName,
      cusIdCardTypeName,
      cusIdCardType,
      partialCoverCusIdCardNo,
    } = this.state;
    console.log(isImport, '8888888888');
    return (
      <div className={styles.wrap}>
        <Block>
          <Section
            title="客户详情"
            toolBar={
              editable &&
              (editing ? (
                <span>
                  <PopTip
                    visible={saveTipsVis}
                    content={saveTips}
                    onClose={() => this.setState({ saveTipsVis: false })}
                    getPopupContainer={(ele) => ele.parentNode}
                  >
                    <Button type="primary" style={{ marginRight: 20 }} onClick={this.onOk}>
                      保存
                    </Button>
                  </PopTip>
                  <Button onClick={this.cancel}>取消</Button>
                </span>
              ) : (
                <Button onClick={() => this.setState({ editing: true })}>编辑</Button>
              ))
            }
          >
            <div className={styles.inner}>
              <FormItem {...formCfg} name="学生姓名">
                {editing && !isFinishOrStopClass ? (
                  <Input value={studentName} onChange={(e) => this.setState({ studentName: e.target.value })} />
                ) : (
                  <div style={{ wordBreak: 'break-word' }}>{getFieldWithFiller(studentName)}</div>
                )}
              </FormItem>
              <FormItem {...formCfg} name="年级">
                {editing && !isFinishOrStopClass ? (
                  <Select dict={gradeDict} value={grade} onChange={(val) => this.setState({ grade: val })} />
                ) : (
                  <div style={{ wordBreak: 'break-word' }}>{getDictTextByVal(gradeDict, grade, true)}</div>
                )}
              </FormItem>
              <FormItem {...formCfg} name="学校级别">
                <div style={{ wordBreak: 'break-word' }}>{getFieldWithFiller(schoolLevelName)}</div>
              </FormItem>
              <FormItem {...formCfg} name="学校">
                {editing ? (
                  <Input readOnly value={schoolName} onClick={() => this.setState({ selectSchoolModalVis: true })} />
                ) : (
                  <div style={{ wordBreak: 'break-word' }}>{getFieldWithFiller(schoolName)}</div>
                )}
              </FormItem>
              <FormItem {...formCfg} name="学生电话">
                {editing && !isFinishOrStopClass ? (
                  <Input
                    value={studentContract}
                    onChange={(e) => this.setState({ studentContract: e.target.value })}
                    validator={{
                      rules: ['phone'],
                      silent: !checking,
                      onErrorChange: (e) => this.errorChange('studentContract', e),
                    }}
                  />
                ) : (
                  <div style={{ wordBreak: 'break-word' }}>{getFieldWithFiller(studentContract)}</div>
                )}
              </FormItem>
              <FormItem {...formCfg} name="客户姓名" required={editing}>
                {editing && !isFinishOrStopClass ? (
                  <Input
                    value={cusName}
                    onChange={(e) => this.setState({ cusName: e.target.value })}
                    validator={{
                      rules: ['required'],
                      silent: !checking,
                      onErrorChange: (e) => this.errorChange('cusName', e),
                    }}
                  />
                ) : (
                  <div style={{ wordBreak: 'break-word' }}>{getFieldWithFiller(cusName)}</div>
                )}
              </FormItem>
              {this.state.isXH && (
                <FormItem {...formCfg} name="客户证件类型">
                  {editing ? (
                    <Select
                      dict={this.state.certificateTypeDict}
                      value={this.state.cusIdCardType}
                      onChange={(val) => {
                        this.setState({ cusIdCardType: val });
                      }}
                      disabled={Boolean(cusIdCardType && cusIdCardType !== 'CT_IDENTITY_CARD')}
                    />
                  ) : (
                    <div style={{ wordBreak: 'break-word' }}>
                      {getFieldWithFiller(cusIdCardTypeName || cusIdCardType)}
                    </div>
                  )}
                </FormItem>
              )}
              {loginUser.isXingHuoSym() && (
                <FormItem {...formCfg} name="客户证件号码">
                  {editing ? (
                    <Input
                      disabled={Boolean(cusIdCardType && cusIdCardType !== 'CT_IDENTITY_CARD')}
                      value={cusIdCardNo}
                      onChange={(e) => this.setState({ cusIdCardNo: e.target.value })}
                    />
                  ) : (
                    <div style={{ wordBreak: 'break-word' }}>{getFieldWithFiller(partialCoverCusIdCardNo)}</div>
                  )}
                </FormItem>
              )}
              <FormItem {...formCfg} name="资源入口">
                {0 ? (
                  <Select dict={entranceDict} value={entrance} onChange={(val) => this.setState({ entrance: val })} />
                ) : (
                  <div style={{ wordBreak: 'break-word' }}>{getDictTextByVal(entranceDict, entrance, true)}</div>
                )}
              </FormItem>
              {entrance && entrance === 'TRANSFER' && (
                <FormItem {...formCfg} name="推荐类型">
                  <div style={{ wordBreak: 'break-word' }}>{introductionType}</div>
                </FormItem>
              )}
              {entrance && entrance === 'TRANSFER' && (
                <FormItem {...formCfg} name="推荐人">
                  <div style={{ wordBreak: 'break-word' }}>{introducer}</div>
                </FormItem>
              )}
              {entrance && isFinishOrStopClass && (
                <FormItem {...formCfg} name="历史资源入口">
                  <div style={{ wordBreak: 'break-word' }}>{preEntranceName}</div>
                </FormItem>
              )}
              <FormItem {...formCfg} name="资源来源" required={editing}>
                {editing && !isFinishOrStopClass ? (
                  <Select
                    dict={sourceDict}
                    value={source}
                    onChange={this.sourceChange}
                    validator={{
                      rules: ['required'],
                      silent: !checking,
                      onErrorChange: (e) => this.errorChange('source', e),
                    }}
                  />
                ) : (
                  <div style={{ wordBreak: 'break-word' }}>{getDictTextByVal(sourceDict, source, true)}</div>
                )}
              </FormItem>
              <FormItem {...formCfg} name="来源细分" required={editing}>
                {editing && !isFinishOrStopClass ? (
                  <Select
                    dict={sourceDetailDict}
                    value={sourceDetail}
                    onChange={(val) => this.setState({ sourceDetail: val })}
                    validator={{
                      rules: ['required'],
                      silent: !checking,
                      onErrorChange: (e) => this.errorChange('sourceDetail', e),
                    }}
                  />
                ) : (
                  <div style={{ wordBreak: 'break-word' }}>
                    {getDictTextByVal(sourceDetailDict, sourceDetail, true)}
                  </div>
                )}
              </FormItem>
              <FormItem {...formCfg} name="所属学习中心">
                {0 ? (
                  <Select dict={campusDict} value={campus} onChange={(val) => this.setState({ campus: val })} />
                ) : (
                  <div style={{ wordBreak: 'break-word' }}>{getDictTextByVal(campusDict, campus, true)}</div>
                )}
              </FormItem>
              <FormItem {...formCfg} name="客户电话">
                {0 ? (
                  <Input
                    value={cusContract}
                    onChange={(e) => this.setState({ cusContract: e.target.value })}
                    validator={{
                      rules: ['required', 'phone'],
                      silent: !checking,
                      onErrorChange: (e) => this.errorChange('cusContract', e),
                    }}
                  />
                ) : (
                  <div style={{ wordBreak: 'break-word' }}>{getFieldWithFiller(cusContract)}</div>
                )}
              </FormItem>
              <FormItem {...formCfg} name="分配人">
                {0 ? (
                  <Input
                    value={dispatcher}
                    onChange={(e) => this.setState({ dispatcher: e.target.value })}
                    validator={{
                      rules: ['required'],
                      silent: !checking,
                      onErrorChange: (e) => this.errorChange('dispatcher', e),
                    }}
                  />
                ) : (
                  <div style={{ wordBreak: 'break-word' }}>{+isImport ? '-' : getFieldWithFiller(dispatcher)}</div>
                )}
              </FormItem>
              <FormItem {...formCfg} name="跟进人">
                {0 ? (
                  <Input value={follower} onChange={(e) => this.setState({ follower: e.target.value })} />
                ) : (
                  <div style={{ wordBreak: 'break-word' }}>{getFieldWithFiller(follower)}</div>
                )}
              </FormItem>
              <FormItem {...formCfg} name="跟进状态">
                {0 ? (
                  <Select
                    dict={followStatusDict}
                    value={followStatus}
                    onChange={(val) => this.setState({ followStatus: val })}
                  />
                ) : (
                  <div style={{ wordBreak: 'break-word' }}>{getFieldWithFiller(followStatusName)}</div>
                )}
              </FormItem>
              <FormItem {...formCfg} name="所属分公司">
                {0 ? (
                  <Select dict={branchDict} value={branch} onChange={this.branchChange} />
                ) : (
                  <div style={{ wordBreak: 'break-word' }}>{getDictTextByVal(branchDict, branch, true)}</div>
                )}
              </FormItem>
              <FormItem {...formCfg} name="出生日期">
                {editing ? (
                  <DatePicker value={birthDay} onChange={(val) => this.setState({ birthDay: val })} />
                ) : (
                  <div style={{ wordBreak: 'break-word' }}>{getFieldWithFiller(birthDay)}</div>
                )}
              </FormItem>
              <FormItem {...formCfg} name="家庭住址">
                {editing ? (
                  <Input value={address} onChange={(e) => this.setState({ address: e.target.value })} />
                ) : (
                  <div style={{ wordBreak: 'break-word' }}>{getFieldWithFiller(address)}</div>
                )}
              </FormItem>
              <FormItem {...formCfg} name="备注">
                {editing ? (
                  <TextArea
                    style={{ wordBreak: 'break-word' }}
                    rows={3}
                    value={remark}
                    onChange={(e) => this.setState({ remark: e.target.value })}
                  />
                ) : (
                  <div style={{ wordBreak: 'break-word' }}>{getFieldWithFiller(remark)}</div>
                )}
              </FormItem>

              {this.state.isXH && (
                <FormItem {...formCfg} name="父亲电话">
                  {editing ? (
                    <Input
                      value={this.state.studentFatherPhone}
                      onChange={(e) => this.setState({ studentFatherPhone: e.target.value })}
                      validator={{
                        rules: [checkPhonePlus],
                        silent: !checking,
                        onErrorChange: (e) => this.errorChange('studentFatherPhone', e),
                      }}
                    />
                  ) : (
                    <div style={{ wordBreak: 'break-word' }}>{getFieldWithFiller(this.state.studentFatherPhone)}</div>
                  )}
                </FormItem>
              )}
              {this.state.isXH && (
                <FormItem {...formCfg} name="母亲电话">
                  {editing ? (
                    <Input
                      value={this.state.studentMotherPhone}
                      onChange={(e) => this.setState({ studentMotherPhone: e.target.value })}
                      validator={{
                        rules: [checkPhonePlus],
                        silent: !checking,
                        onErrorChange: (e) => this.errorChange('studentMotherPhone', e),
                      }}
                    />
                  ) : (
                    <div style={{ wordBreak: 'break-word' }}>{getFieldWithFiller(this.state.studentMotherPhone)}</div>
                  )}
                </FormItem>
              )}
            </div>
          </Section>
        </Block>
        {selectSchoolModalVis && (
          <SelectSchoolModal
            onConfirm={(school, schoolName) => {
              const match = schoolName.match(/（([^（）]+)）$/);

              this.setState({
                selectSchoolModalVis: false,
                school,
                schoolName,
                schoolLevelName: match ? match[1] : '-',
              });
            }}
            onClose={() => this.setState({ selectSchoolModalVis: false })}
            defaultProvince={provinceId}
            defaultCity={cityId}
          />
        )}
        {mergeStudentInfoModalVisible && (
          <Modal
            width={514}
            onCancel={() => {
              this.setState({ mergeStudentInfoModalVisible: false });
              this.onMergeModalClose();
            }}
            okBtnShow={false}
            cancelText="关闭"
          >
            <div style={{ padding: '40px 0 5px 0' }}>{mergeStudentInfoMsg}</div>
          </Modal>
        )}
      </div>
    );
  }
}

export default CusDetailBlock;
