import React from 'react';
import styles from './styles.less';
import loaders from '../loaders';
import Loading from 'COMPONENT/Loading';

class CusNewsBlock extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      list: [], // 列表数据，[{ time, content }]
      loading: false,
    };
  }

  initContent = async () => {
    this.setState({ loading: true });
    const { id, stuId, deliverTarget, isManager, campusId } = this.props;
    try {
      const list = await (isManager
        ? loaders.loadCusNewsForManager(campusId, id, stuId)
        : loaders.loadCusNews(id, stuId, deliverTarget));
      this.setState({ list });
    } finally {
      this.setState({ loading: false });
    }
  };

  componentDidMount() {
    this.initContent();
  }

  render() {
    const { list, loading } = this.state;
    return (
      <div className={styles.wrap}>
        <div className={styles.inner}>
          <Loading loading={loading} />
          {list.length ? (
            list.map((li, idx) => {
              return (
                <div key={idx} className={styles.item}>
                  <div className={styles.time}>{li.time}</div>
                  <div className={styles.content}>{li.content}</div>
                </div>
              );
            })
          ) : (
            <p style={{ textAlign: 'center', color: '#ccc', fontSize: '18px' }}>暂无数据</p>
          )}
        </div>
      </div>
    );
  }
}
export default CusNewsBlock;
