import React from 'react';
import FormItem from 'COMPONENT/FormItem';
import { Row, Col, Tabs } from 'antd';
import loaders from '../loaders';
import styles from './styles.less';
import 'COMPONENT/Page/styles.less';
import Loading from 'COMPONENT/Loading';
import MoneyChangeLogs from './MoneyChangeLogs';
import EleAccChangeLogs from './EleAccChangeLogs';
import EleAccout from './EleAccout';
import EleContracts from './EleContracts';

const TabPane = Tabs.TabPane;

function moneyToUpper(num) {
  let strOutput = '';
  let strUnit = '仟佰拾亿仟佰拾万仟佰拾元角分';
  num += '00';
  const intPos = num.indexOf('.');
  if (intPos >= 0) num = num.substring(0, intPos) + num.substr(intPos + 1, 2);
  strUnit = strUnit.substr(strUnit.length - num.length);
  for (let i = 0; i < num.length; i++)
    strOutput += '零壹贰叁肆伍陆柒捌玖'.substr(num.substr(i, 1), 1) + strUnit.substr(i, 1);
  return strOutput
    .replace(/零角零分$/, '')
    .replace(/零[仟佰拾]/g, '零')
    .replace(/零{2,}/g, '零')
    .replace(/零([亿|万])/g, '$1')
    .replace(/零+元/, '元')
    .replace(/亿零{0,3}万/, '亿')
    .replace(/^元/, '零元');
}

const summaryCn = {
  totalAmount: '合同总费用(元)：',
  realPaidAmount: '实际付款(元)：',
  electronicPaidAmount: '电子账户付款(元)：',
  promotinAmount: '总优惠资金(元)：',
  planAmount: '待付款资金(元)：',
  oneOnOneRemainingAmount: '一对一剩余资金(元)：',
  oneOnOneRemainingNatureHour: '一对一剩余小时：',
  miniRemainingAmount: '小班剩余资金(元)：',
  miniRemainingNatureHour: '小班剩余小时：',
  otmRemainingAmount: '一对多剩余资金(元)：',
  otmRemainingNatureHour: '一对多剩余小时：',
  ecsRemainingAmount: '精英班剩余资金(元)：',
  electronicAccount: '电子账户余额(元)：',
  lectureRemainingAmount: '讲座剩余资金(元)：',
  lectureRemainingNatureHour: '讲座剩余小时：',
  remainingAmount: '剩余资金(元)：',
  // twoTeacherRemainingAmount: '双师剩余资金(元)：',
  // twoTeacherRemainingHour: '双师剩余课时：',
};

class EleAccInfoBlock extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: true,
      info: {},
      summary: {},
    };
  }

  componentDidMount() {
    this.setState({ loading: true });
    const { stuId } = this.props;
    if (!stuId) {
      return;
    }
    Promise.all([loaders.getStudentAccoutInfo({ studentId: stuId })]).then(([accoutInfo]) => {
      const info = {
        electronicAccount: accoutInfo.electronicAccount || 0,
        frozenAmount: accoutInfo.frozenAmount || 0,
        electronicAccountUpper: moneyToUpper(accoutInfo.electronicAccount) || '',
      };
      const summary = {
        totalAmount: accoutInfo.totalAmount,
        realPaidAmount: accoutInfo.realPaidAmount,
        electronicPaidAmount: accoutInfo.electronicPaidAmount,
        promotinAmount: accoutInfo.promotinAmount || 0,
        planAmount: accoutInfo.planAmount,
        oneOnOneRemainingAmount: accoutInfo.oneOnOneRemainingAmount,
        oneOnOneRemainingNatureHour: accoutInfo.oneOnOneRemainingNatureHour,
        miniRemainingAmount: accoutInfo.miniRemainingAmount,
        miniRemainingNatureHour: accoutInfo.miniRemainingNatureHour || 0,
        otmRemainingAmount: accoutInfo.otmRemainingAmount,
        otmRemainingNatureHour: accoutInfo.otmRemainingNatureHour,
        ecsRemainingAmount: accoutInfo.ecsRemainingAmount || 0,
        electronicAccount: accoutInfo.electronicAccount || 0,
        lectureRemainingAmount: accoutInfo.lectureRemainingAmount || 0,
        lectureRemainingNatureHour: accoutInfo.lectureRemainingNatureHour || 0,
        remainingAmount: accoutInfo.remainingAmount,
        // twoTeacherRemainingAmount: accoutInfo.twoTeacherRemainingAmount,
        // twoTeacherRemainingHour: accoutInfo.twoTeacherRemainingHour,
      };

      this.setState({
        loading: false,
        info,
        summary,
      });
    });
  }

  renderSummary = () => {
    const summaryList = Object.keys(summaryCn);
    const { summary } = this.state;
    return summaryList.map((ele) => {
      return (
        <Col key={ele} span={6} className={styles.customFormItem}>
          <FormItem name={summaryCn[ele] || ''}>{summary[ele] || 0}</FormItem>
        </Col>
      );
    });
  };

  render() {
    const { stuId } = this.props;
    const { loading, info } = this.state;
    return (
      <div>
        <Loading loading={loading} />
        <Row key="info">
          <Col span={24}>
            <h3 className={styles.header}>账户信息</h3>
          </Col>
          <Col span={24}>
            <Col span={6} className={styles.customFormItem}>
              <FormItem name="电子账户余额：">
                <span>{info.electronicAccount || 0}</span>元
                {info.frozenAmount ? (
                  <span style={{ marginLeft: '16px' }}>
                    （冻结： <span>{info.frozenAmount}</span>元）
                  </span>
                ) : (
                  ''
                )}
              </FormItem>
            </Col>
            <Col span={6} className={styles.customFormItem}>
              <FormItem name="金额大写：">
                <span>{info.electronicAccountUpper || ''}</span>
              </FormItem>
            </Col>
          </Col>
        </Row>
        <div className={styles.hr}></div>
        <Row key="summary">
          <Col span={24}>
            <h3 className={styles.header}>财务总览</h3>
          </Col>
          <Col span={24}>{this.renderSummary()}</Col>
        </Row>
        <Row key="details" style={{ marginTop: '20px' }}>
          <Col span={24}>
            <Tabs type="card" className={styles.customTabs}>
              <TabPane tab="学生签约合同列表" key="eleContracts">
                <EleContracts stuId={stuId}></EleContracts>
              </TabPane>
              <TabPane tab="资金变更记录" key="moneyChangeLogs">
                <MoneyChangeLogs stuId={stuId}></MoneyChangeLogs>
              </TabPane>
              <TabPane tab="电子账户变更记录" key="eleAccChangeLogs">
                <EleAccChangeLogs stuId={stuId}></EleAccChangeLogs>
              </TabPane>
              <TabPane tab="电子账户余额" key="eleAccout">
                <EleAccout stuId={stuId}></EleAccout>
              </TabPane>
            </Tabs>
          </Col>
        </Row>
      </div>
    );
  }
}

export default EleAccInfoBlock;
