import React from 'react';
import List from 'COMPONENT/List';
import loaders from '../loaders';
import 'COMPONENT/Page/styles.less';
import { loadCommonDict } from 'UTIL/dict';
import { Popover, Checkbox } from 'antd';
import styles from './styles.less';

class EleContractDetail extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      dataSource: [],
      productTypeDict: [],
      fundList: [],
      fundListEnable: false,
      chargeList: [],
      chargeListEnable: false,
      listLoading: false,
    };
  }

  init = async () => {
    this.setState({ listLoading: true });
    const { contractId } = this.props;
    if (!contractId) {
      return;
    }

    const [contract, productTypeDict, fundListRes, chargeListRes] = await Promise.all([
      loaders.getContractById({ contractId: contractId }),
      loadCommonDict('ProductType'),
      loaders.getMyFundList({ contractId: contractId, rows: 1000 }),
      loaders.getMyChargeList({ contractIdAccount: contractId, rows: 1000, fromSource: 'consultantFollowCustomer' }),
    ]);

    const dataSource = await Promise.all(
      contract.contractProductVos.map(async (contractProduct) => {
        const data = {
          ...contractProduct,
          saleChannelName: contract.saleChannelName,
          className: '',
          classBlCampusName: '',
        };
        if (contractProduct.isFrozen == 0) {
          data['statusName'] = '冻结';
        }
        if (contract.productType == 'SMALL_CLASS') {
          const miniClass = await loaders.getMiniClassByContractProductId({ contractProductId: contractProduct.id });
          data['className'] = miniClass.name;
          data['classBlCampusName'] = miniClass.blCampusName;
        } else if (contract.productType == 'TWO_TEACHER') {
          const twoTeacherClass = await loaders.findTwoTeacherTwoByContractProductId({ conProId: contractProduct.id });
          data['className'] = twoTeacherClass.name;
          data['classBlCampusName'] = twoTeacherClass.blCampusName;
        }
        return data;
      }),
    );

    this.setState({
      dataSource: dataSource,
      productTypeDict: productTypeDict,
      fundList: fundListRes.rows,
      chargeList: chargeListRes.rows,
      listLoading: false,
    });
  };

  componentDidMount() {
    this.init();
  }

  onFundListEnableChange = () => {
    const { fundListEnable } = this.state;
    this.setState({
      fundListEnable: !fundListEnable,
    });
  };

  onChargeListEnableChange = () => {
    const { chargeListEnable } = this.state;
    this.setState({
      chargeListEnable: !chargeListEnable,
    });
  };

  render() {
    const { contractId } = this.props;
    const { dataSource, productTypeDict, fundList, fundListEnable, chargeList, chargeListEnable, listLoading } =
      this.state;

    const columns = [
      { title: '产品类型', dataIndex: 'productType', dict: productTypeDict },
      { title: '产品名称', dataIndex: 'productionName' },
      {
        title: '班级名称',
        dataIndex: 'className',
        render: (text, row) => {
          if (text) {
            const prefix =
              row.productType == 'SMALL_CLASS' ? '小班名称：' : row.productType == 'TWO_TEACHER' ? '班级名称：' : '';
            return (
              <Popover
                content={
                  <div>
                    <div>
                      <span>{prefix}</span>
                      <span>{text}</span>
                    </div>
                    <div>
                      <span>学习中心：</span>
                      <span>{row.classBlCampusName}</span>
                    </div>
                  </div>
                }
                trigger="click"
                getPopupContainer={() => document.querySelector(`#${contractId}`)}
              >
                <span>{text}</span>
              </Popover>
            );
          }
          return '空';
        },
      },
      { title: '购买小时', dataIndex: 'natureQuantity' },
      { title: '单价', dataIndex: 'price' },
      { title: '产品总额', dataIndex: 'payment' },
      { title: '已分配资金', dataIndex: 'paidAmount' },
      { title: '优惠金额', dataIndex: 'promotionAmount' },
      { title: '已消费金额', dataIndex: 'consumeAmount' },
      { title: '剩余资金', dataIndex: 'remainingAmount' },
      { title: '支付状态', dataIndex: 'paidStatusName' },
      { title: '状态', dataIndex: 'statusName' },
      { title: '报读渠道', dataIndex: 'saleChannelName' },
    ];

    return (
      <div className="Page-scroller" style={{ margin: '0' }}>
        <List dataSource={dataSource} columns={columns} loading={listLoading} />
        <div className={styles.action}>
          <div style={{ flex: '1' }}>合同资金变更记录</div>
          <Checkbox onChange={this.onFundListEnableChange}>收费</Checkbox>
          <Checkbox onChange={this.onChargeListEnableChange}>扣费</Checkbox>
        </div>
        {fundListEnable && (
          <div key="fundList">
            {fundList.map((ele) => {
              const payType = ele.fundsPayType == 'WASH' ? '【收费冲销】' : '【收费】';
              return (
                <div key={ele.id}>
                  <span className={styles.fund}>{`${payType}：`}</span>
                  <span>{`${ele.transactionAmount}元，${ele.channelName}，${ele.chargeByWho}，${ele.receiptTime}`}</span>
                </div>
              );
            })}
          </div>
        )}
        {chargeListEnable && (
          <div key="chargeList">
            {chargeList.map((ele) => {
              let chargeName = ele.subject;
              if (!chargeName) {
                chargeName = ele.productName;
                if (ele.productTypeName == '小班' && ele.miniClassName) {
                  chargeName += ' ' + ele.miniClassName;
                }
              }
              const courseTime = !ele.courseTime ? '' : '上课时间：' + ele.courseTime + '，';
              const payType = ele.chargePayType == 'WASH' ? '扣费冲销' : '扣费';
              const operateUserName = ele.operateUserName ? ele.operateUserName : '系统扣款';

              return (
                <div key={ele.id}>
                  <span className={styles.charge}>{`【${payType}】：`}</span>
                  <span>{`${ele.amount}元，${chargeName}，${operateUserName}，${courseTime}${payType}时间：${ele.payTime}`}</span>
                </div>
              );
            })}
          </div>
        )}
      </div>
    );
  }
}
export default EleContractDetail;
