import React from 'react';
import loaders from '../loaders';
import 'COMPONENT/Page/styles.less';
import { Collapse, Row } from 'antd';
import Loading from 'COMPONENT/Loading';
import styles from './styles.less';
import EleContractDetail from './EleContractDetail';

const Panel = Collapse.Panel;

class EleContracts extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      dataSource: [],
      loading: false,
      expandedRecord: {},
    };
  }

  componentDidMount() {
    this.setState({ listLoading: true });
    const { stuId } = this.props;
    if (!stuId) {
      return;
    }
    Promise.all([loaders.findContractByStu({ studentId: stuId, rowNum: 1000 })]).then(([res]) => {
      this.setState({
        dataSource: res.rows,
        loading: false,
      });
    });
  }

  columns = [
    { title: '编号：', dataIndex: 'contractId', width: '14%' },
    { title: '签约日期：', dataIndex: 'signTime', width: '16%' },
    { title: '总额：', dataIndex: 'totalAmount', width: '14%' },
    { title: '优惠金额费：', dataIndex: 'promotionAmount', width: '14%' },
    { title: '已付费：', dataIndex: 'paidAmount', width: '14%' },
    { title: '待付费：', dataIndex: 'pendingAmount', width: '14%' },
    { title: '签单人：', dataIndex: 'signByWho', width: '14%' },
  ];

  onClick = (val) => {
    const { expandedRecord } = this.state;
    if (!expandedRecord[val]) {
      this.setState({
        expandedRecord: { ...expandedRecord, [val]: 1 },
      });
    }
  };

  render() {
    const { dataSource, loading, expandedRecord } = this.state;
    return (
      <div className="Page-scroller" style={{ margin: '0', maxHeight: '560px' }}>
        <Loading loading={loading} />
        {!dataSource.length ? (
          <div style={{ width: '100%', textAlign: 'center' }}>暂无数据</div>
        ) : (
          <Collapse accordion onChange={this.onClick}>
            {dataSource.map((ele) => {
              return (
                <Panel
                  key={ele.contractId}
                  id={ele.contractId}
                  header={
                    <div className={styles.row}>
                      {this.columns.map((column) => {
                        return (
                          <span style={{ width: column.width }}>
                            <span>{column.title}</span>
                            <span>{ele[column.dataIndex]}</span>
                          </span>
                        );
                      })}
                    </div>
                  }
                  key={ele.contractId}
                >
                  {expandedRecord[ele.contractId] && (
                    <EleContractDetail contractId={ele.contractId}></EleContractDetail>
                  )}
                </Panel>
              );
            })}
          </Collapse>
        )}
      </div>
    );
  }
}
export default EleContracts;
