import React from 'react';
import List from 'COMPONENT/List';
import loaders from '../loaders';
import 'COMPONENT/Page/styles.less';

class MoneyChangeLogs extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      current: 1,
      pageSize: 10,
      total: 0,
      dataSource: [],
      listLoading: false,
    };
  }

  componentDidMount() {
    this.setState({ listLoading: true });
    const { stuId } = this.props;
    if (!stuId) {
      return;
    }
    Promise.all([loaders.getStudentMoneyChanges({ studentId: stuId })]).then(([res]) => {
      this.setState({
        total: res.records,
        dataSource: res.rows,
        listLoading: false,
      });
    });
  }

  onChange = async ({ current = this.state.current, pageSize = this.state.pageSize }) => {
    const { stuId } = this.props;
    this.setState({ listLoading: true });
    const params = {
      studentId: stuId,
      rows: pageSize,
      page: current,
    };
    const res = await loaders.getStudentMoneyChanges(params);
    this.setState({
      current: current,
      pageSize: pageSize,
      total: res.records,
      dataSource: res.rows,
      listLoading: false,
    });
    return;
  };

  columns = [
    { title: '合同ID', dataIndex: 'contractId' },
    { title: '学生名字', dataIndex: 'studentName' },
    { title: '学习中心', dataIndex: 'campusName' },
    { title: '学管师', dataIndex: 'studyManagerName' },
    {
      title: '类型',
      dataIndex: 'type',
      render: (text) => {
        switch (text) {
          case 'INCOME':
            return '收费';
          case 'OUTCOME':
            return '扣费';
          default:
            return text || '';
        }
      },
    },
    {
      title: '途径',
      dataIndex: 'channel',
      render: (text) => {
        switch (text) {
          case 'WASH':
            return '收费冲销';
          default:
            return text || '';
        }
      },
    },
    { title: '产品类型', dataIndex: 'productTypeName' },
    { title: '班级名称', dataIndex: 'className' },
    {
      title: '扣费类型',
      dataIndex: 'chargeType',
      render: (text, row) => {
        if (row.chargePayType === 'WASH') {
          return `${text}扣费冲销`;
        }
        return text || '';
      },
    },
    { title: '金额', dataIndex: 'amount' },
    { title: '操作人', dataIndex: 'operateUserName' },
    { title: '冲销备注', dataIndex: 'washReason' },
    { title: '时间', dataIndex: 'happenedTime' },
  ];

  render() {
    const { current, pageSize, total, dataSource, listLoading } = this.state;
    return (
      <div className="Page-scroller" style={{ margin: '0', maxHeight: '560px' }}>
        <List
          dataSource={dataSource}
          columns={this.columns}
          pagination={{
            current: current,
            total: total,
            pageSize: pageSize,
          }}
          loading={listLoading}
          onChange={this.onChange}
        />
      </div>
    );
  }
}
export default MoneyChangeLogs;
