import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import styles from './styles.less';
import CusDetailBlock from './CusDetailBlock';
import RecordsBlock from './RecordsBlock';
import CusNewsBlock from './CusNewsBlock';
import { Tabs } from 'antd';
import PropTypes from 'prop-types';
import RecordsBlockManage from './RecordsBlockManage';
import sectionStyle from 'COMPONENT/Section/styles.less';
import StuInfoBlock from './StuInfoBlock/StuInfoBlock';
import EleAccInfoBlock from './EleAccInfoBlock/EleAccInfoBlock';
import CallRecord from 'PAGE/consultant/CallRecord';
import { CALL_DIRECT_CALL } from 'CONST/callConfig';
import getWrappedInstance from 'UTIL/getWrappedInstance';

const TabPane = Tabs.TabPane;

class CusDetailPageMain extends React.Component {
  static propTypes = {
    id: PropTypes.string.isRequired, // 客户ID
    detailEditable: PropTypes.bool, // 详情可编辑
    followEditable: PropTypes.bool, // 跟进记录可编辑
  };
  static defaultProps = {
    detailEditable: false,
    followEditable: false,
  };

  constructor(props) {
    super(props);

    this.state = {
      loading: false, // {boolean} 页面加载状态
      isReactivated: false,
    };

    this.audioAuth = props.btnAuth.indexOf('CALL_AUTH_PLAY_AUDIO') > -1;
    this.callRecordDefaultSKey = {
      studentId: props.stuId,
      userId: props.isManager ? null : props.user.userId,
      callDirection: CALL_DIRECT_CALL,
    };
  }

  componentDidMount() {
    this.setState({ loading: true });
    Promise.all([this.cusDetailBlockRef.initContent(), this.recordsBlockRef.initContent()]).then(([cusDetail]) => {
      this.setState({
        loading: false,
        isReactivated: !!cusDetail.reactivated,
      });
    });
  }

  loadingStart = () => {
    this.setState({ loading: true });
  };

  loadingEnd = () => {
    this.setState({ loading: false });
  };

  render() {
    const {
      id,
      stuId,
      detailEditable,
      followEditable,
      deliverTarget,
      isManager,
      campusId,
      pageParams,
      onCloseTab,
      dispatch,
      list,
      user,
    } = this.props;
    const hasConsultWorkbrench = this.props.callCenterConfig.hasConsultWorkbrench;
    const alreadyBind = this.props.alreadyBind;
    const { loading, isReactivated } = this.state;

    return (
      <Page route={this.props.route} title={false} className={styles.page} loading={loading}>
        <CusDetailBlock
          id={id}
          stuId={stuId}
          deliverTarget={deliverTarget}
          editable={detailEditable}
          ref={(c) => (this.cusDetailBlockRef = c)}
          onLoadingStart={this.loadingStart}
          onLoadingEnd={this.loadingEnd}
          pageParams={pageParams}
          onCloseTab={onCloseTab}
          dispatch={dispatch}
          user={user}
        />

        <div style={{ marginTop: 15 }}>
          <div className="Section" style={sectionStyle}>
            <div className="Section-body custom-tabs" style={{ paddingBottom: '20px' }}>
              <Tabs className={styles.customTabs} tabBarStyle={{ border: 'none', margin: '10px 0 15px 0' }}>
                <TabPane key="records" tab="跟进记录">
                  {isManager ? (
                    <RecordsBlockManage
                      id={id}
                      stuId={stuId}
                      campusId={campusId}
                      deliverTarget={deliverTarget}
                      editable={followEditable}
                      ref={(c) => (this.recordsBlockRef = getWrappedInstance(c))}
                      onLoadingStart={this.loadingStart}
                      onLoadingEnd={this.loadingEnd}
                      isManager={isManager}
                    />
                  ) : (
                    <RecordsBlock
                      id={id}
                      stuId={stuId}
                      deliverTarget={deliverTarget}
                      editable={followEditable}
                      ref={(c) => (this.recordsBlockRef = getWrappedInstance(c))}
                      onLoadingStart={this.loadingStart}
                      onLoadingEnd={this.loadingEnd}
                      isManager={isManager}
                      followList={list}
                      onCloseTab={onCloseTab}
                      dispatch={dispatch}
                      pageParams={pageParams}
                      callProps={this.props.callProps}
                      audioAuth={this.audioAuth}
                    />
                  )}
                </TabPane>
                <TabPane key="cusNews" tab="客户动态">
                  <CusNewsBlock
                    id={id}
                    stuId={stuId}
                    campusId={campusId}
                    deliverTarget={deliverTarget}
                    ref={(c) => (this.cusNewsBlockRef = c)}
                    isManager={isManager}
                  />
                </TabPane>
                {hasConsultWorkbrench && (
                  <TabPane key="callRecord" tab="通话记录">
                    <CallRecord searchKeys={this.callRecordDefaultSKey} audioAuth={this.audioAuth} />
                  </TabPane>
                )}
                {isReactivated && (
                  <TabPane key="stuInfo" tab="学生详情">
                    <StuInfoBlock stuId={stuId} />
                  </TabPane>
                )}
                {isReactivated && (
                  <TabPane key="eleAcc" tab="学生账务">
                    <EleAccInfoBlock stuId={stuId} />
                  </TabPane>
                )}
              </Tabs>
            </div>
          </div>
        </div>
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    callCenterConfig: state.account.callCenterConfig,
    alreadyBind: state.callCenter.alreadyBind,
  };
};
export default connect(mapStateToProps, null, null, { withRef: true })(CusDetailPageMain);
