import React from 'react';
import { connect } from 'dva';
import PropTypes from 'prop-types';
import styles from './styles.less';
import { List, ListAction } from 'COMPONENT';
import { runFunc } from 'UTIL/common';

const setWidth = List.createSetWidth();
class Record extends React.Component {
  constructor(props) {
    super(props);
    const initColumns = [
      { title: '跟进人', dataIndex: 'createUserName', key: 'createUserName', width: 80 },
      { title: '跟进时间', dataIndex: 'createTime', key: 'createTime', width: 120 },
      { title: '跟进内容', dataIndex: 'remark', key: 'remark', width: 380 },
      {
        title: '意向度',
        dataIndex: 'intentionOfTheCustomerName',
        key: 'intentionOfTheCustomerName',
        width: 70,
        dict: props.intentionDict,
      },
      { title: '客户意向产品', dataIndex: 'customerSignName', key: 'customerSignName', width: 120 },
    ];
    const { hasConsultWorkbrench, hasOutcallWorkbrench } = this.props.callCenterConfig;
    const alreadyBind = this.props.alreadyBind;
    const pageType = this.props.page;
    // 页面是咨询师并且开始咨询师外呼
    // 或者页面是外呼，并且开启外呼功能才显示
    const isShowCallBtn =
      (hasConsultWorkbrench && pageType == 'cousultant') || (hasOutcallWorkbrench && pageType == 'outcall');

    this.columns = (
      props.isOutCallResource
        ? initColumns
        : [
            ...initColumns,
            { title: '预约上门时间', dataIndex: 'meetingTime', key: 'meetingTime', width: 120 },
            { title: '下次跟进日期', dataIndex: 'nextFollowTime', key: 'nextFollowTime', width: 120 },
          ]
    ).concat([
      {
        title: '操作',
        dataIndex: 'callRecordVos',
        key: 'actionPlus',
        width: setWidth(4),
        render(text, record, index) {
          // 当前业务只存在一条通话记录
          const callRecord = (text || [])[0];
          return isShowCallBtn ? (
            <ListAction
              config={[
                {
                  text: '通话详情',
                  onClick: () => {
                    runFunc(props.handleCallDetail, callRecord);
                  },
                  disabled: !callRecord,
                  // disabled: !props.callProps.agentId || !callRecord,
                },
              ]}
            />
          ) : null;
        },
      },
    ]);
  }

  static propTypes = {
    list: PropTypes.array,
    intentionDict: PropTypes.array,
    handleCallDetail: PropTypes.func,
    callProps: PropTypes.shape({
      agentId: PropTypes.any,
      agentLoginStatus: PropTypes.any,
      callWay: PropTypes.any,
    }),
    isOutCallResource: PropTypes.bool,
  };

  render() {
    return <List dataSource={this.props.list} columns={this.columns} />;
  }
}

const mapStateToProps = (state) => {
  return {
    callCenterConfig: state.account.callCenterConfig,
    alreadyBind: state.callCenter.alreadyBind,
  };
};
export default connect(mapStateToProps)(Record);
