import React from 'react';
import { connect } from 'dva';
import styles from './styles.less';
import Sec from '../Sec';
import Record from '../Record';
import { FormItem, DatePicker, TimePicker, Button, TextArea, PopTip, Select } from 'COMPONENT';
import Fold from '../Fold';
import loaders from '../loaders';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { getCommonErrorTips } from 'COMPONENT/utils/form';
import { loadCommonDict } from 'UTIL/dict';
import moment from 'moment';
import { DATE_FORMAT } from 'COMPONENT/utils/form';
import { Row, Col } from 'antd';
import { formatIsInvalidIntention, range } from './util';
import { CUS_LIST_LOST } from 'PAGE/consultant/FollowCustomer/dicts';
import ModalPhoneCallFollow from 'PAGE/consultant/ModalPhoneCallFollow';
import { errorsHandle } from 'UTIL/common';
import ModalCallDetail from 'PAGE/consultant/ModalCallDetail';
import { getMyCustomerFollowUpWrappers } from 'SERVICE/consultant';
import { checkAgentBeforeCall } from 'SRC/utils/callCenter';
import { CALL_LOGIN_STATUS_ABNORMAL } from 'CONST/callConfig';

const formCfg = {
  labelWidth: 110,
};

const initFollowInfo = {
  intention: null, // {string} 添加记录字段 意向度
  tag: null, // {string} 添加记录字段 客户标签
  invalidTag: null, // {string} 添加记录字段 无效标签
  content: null, // {string} 添加记录字段 跟进内容
  nextDate: null, // {string} 添加记录字段 下次跟进日期
  appointmentDate: null, // {string} 添加记录字段 预约上门日期
  appointmentTime: null, // {string} 添加记录字段 预约上门时间
};

class RecordsBlock extends React.Component {
  constructor(props) {
    super(props);

    this.loadMyFollowList = this.loadMyFollowList.bind(this);
    this.followByCall = this.followByCall.bind(this);
    this.checkAgentLoginStatus = this.checkAgentLoginStatus.bind(this);
    this.switchModalPhoneCallFollow = this.switchModalPhoneCallFollow.bind(this);
    this.switchModalCallDetail = this.switchModalCallDetail.bind(this);

    const { followList, stuId } = this.props;
    this.isListFollowed = !!followList;
    this.isNextDisabled = false;
    if (this.isListFollowed) {
      const currentIndex = followList.findIndex((ele) => ele.stuId === stuId);
      this.isNextDisabled = followList.length - 1 === currentIndex;
      this.nextStudent = this.isNextDisabled ? undefined : followList[currentIndex + 1];
    }

    this.state = {
      list: [], // {object[]} 我的跟进记录列表，[{ time, intention, tag, content }]
      editing: true, // {boolean} 添加记录开启状态
      checking: false, // {boolean} 校验开启状态

      ...initFollowInfo,

      intentionDict: [], // {object[]} 意向度字典
      cusTagDict: [], // {object[]} 客户标签字典(应该删除'其他'选项)
      invalidTagDict: [], // 无效标签字典

      saveTipsVis: false, // {boolean} 保存按钮提示气泡显示状态
      saveTips: null, // {string} 保存按钮提示信息

      otherRecords: [], // {object[]} 其他记录，[{ id, name, list }]
      otherExpand: [], // {string[]} 展开的其他咨询师记录

      modalPhoneCallFollowVisible: false,
      modalCallDetailVisible: false,
    };

    this.phoneCallFollowData = null;
    this.callDetail = null;

    this.errors = {};
  }

  initContent = async () => {
    await new Promise(this.loadMyFollowList);
    const [intentionDict, cusTagDict, invalidTagDict] = await Promise.all([
      loadCommonDict('CUSTOMER_SATISFICING'),
      loadCommonDict('CustomerSignType'),
      loadCommonDict('INVALID_SIGN'),
    ]);
    this.setState({
      intentionDict,
      cusTagDict: cusTagDict.filter((item) => item.val !== 'CUSTOMER_SIGN_TYPE_OTHER'),
      invalidTagDict,
    });
  };

  async loadMyFollowList(resolve) {
    let apiParams = {
      customerId: this.props.pageParams.customerId,
      studentId: this.props.pageParams.studentId,
    };
    const res = await getMyCustomerFollowUpWrappers(apiParams).catch(errorsHandle());

    if (res && res.data) {
      this.setState({
        list: res.data.myFollowingList || [],
        otherRecords: this.otherFollowingAdapter(res.data.otherFollowingList),
      });
    }
    resolve();
  }

  otherFollowingAdapter = (otherFollowingList) => {
    return Object.keys(otherFollowingList).map((key) => ({
      name: key,
      list: otherFollowingList[key],
    }));
  };

  cancel = () => {
    this.error = {};
    this.setState({
      intention: null,
      tag: null,
      content: null,
      nextDate: null,
      appointmentDate: null,
      appointmentTime: null,
      checking: false,
    });
  };

  onOk = async (followNext) => {
    const { id, stuId, deliverTarget, onLoadingStart, onLoadingEnd, onCloseTab } = this.props;
    const { checking, intention, tag, invalidTag, content, nextDate, appointmentDate, appointmentTime } = this.state;
    const errors = { ...this.errors };
    const isInvalidIntention = formatIsInvalidIntention(intention);

    if (!checking) {
      this.setState({ checking: true });
    }

    const errorTips = getCommonErrorTips(errors);
    if (errorTips) {
      this.showSaveTips(errorTips);
      return;
    }

    if (appointmentDate && !appointmentTime) {
      this.showSaveTips('填写了预约上门日期必须填写预约上门时间');
      return;
    }

    let params = {};
    let saveFunc = () => {};

    if (isInvalidIntention) {
      const details = { remark: content };
      params = { customerId: id, studentId: stuId, invalidSignId: invalidTag, ...details };
      saveFunc = loaders.saveInvalidFollowRecord;
    } else {
      const details = { intention, tag, content, nextDate, appointmentDate, appointmentTime };
      params = { id, stuId, ...details };
      saveFunc = loaders.saveFollowRecord;
    }

    onLoadingStart();

    const res = await saveFunc(params).catch((e) => {
      if (e.text) {
        this.showSaveTips(e.text);
      } else {
        showMessage('保存失败');
      }
      onLoadingEnd();
    });

    if (res) {
      showMessage('保存成功', 'success');

      // if (isInvalidIntention) {
      //   onCloseTab();

      //   this.props.dispatch({
      //     type: 'main/openPage',
      //     payload: {
      //       path: '/ConsultantFollowCustomer',
      //       params: {
      //         initCusList: CUS_LIST_LOST
      //       },
      //       refresh: true,
      //     }
      //   });

      //   return;
      // }

      this.setState({
        checking: false,
        ...initFollowInfo,
      });

      this.error = {};

      await new Promise(this.loadMyFollowList);

      onLoadingEnd();

      if (followNext) {
        this.followNext();
      }
    }
  };

  /**
   * 选择意向度
   * error.invalidTag: 无效标签
   *
   * @param val 意向度
   */
  onIntentionChange = (val) => {
    const { invalidTag } = this.state;
    let error = { ...this.errors };

    error.intention = '必填项';
    error.invalidTag = '必填项';

    if (!val) {
      delete error.invalidTag;
    } else {
      delete error.intention;

      if (formatIsInvalidIntention(val)) {
        invalidTag && delete error.invalidTag;
      } else {
        delete error.invalidTag;
      }
    }

    this.setState({ intention: val });
    this.errors = error;
  };

  errorChange = (key, error) => {
    this.errors[key] = error;
  };

  showSaveTips = (text) => {
    this.setState({ saveTipsVis: true, saveTips: text || '请正确填写信息' });
  };

  disableDateAfterToday = (value) => {
    if (!value) {
      return false;
    }
    return moment(value).valueOf() < moment(moment().format(DATE_FORMAT)).valueOf();
  };

  disabledHours = () => {
    return range(0, 6).concat([24]);
  };

  disabledMinutes = () => {
    return range(0, 60).filter((ele) => ele % 15 !== 0);
  };

  followNext = () => {
    if (this.nextStudent) {
      const { dispatch, pageParams, isManager, onCloseTab } = this.props;
      const params = {
        ...pageParams,
        id: this.nextStudent.id,
        stuId: this.nextStudent.stuId,
        deliverTarget: this.nextStudent.deliverTarget,
        customerId: this.nextStudent.customerId,
        studentId: this.nextStudent.studentId,
        studentName: this.nextStudent.studentName,
        phoneNum: this.nextStudent.phoneNum,
      };

      dispatch({
        type: 'main/openPage',
        payload: {
          path: '/CommonCusDetailPage',
          params,
          refresh: true,
          mirrorId: `ConsultantFollowCustomer/CusListSec/${params.type}/${params.id}/${params.stuId}/${
            params.detailEditable ? 'detail' : 'follow'
          }`,
        },
      });

      onCloseTab();
    }
  };

  async followByCall() {
    const agentLoginStatus = await new Promise(this.checkAgentLoginStatus);
    if (checkAgentBeforeCall(agentLoginStatus)) {
      this.switchModalPhoneCallFollow();
    }
  }

  checkAgentLoginStatus(resolve) {
    this.props.dispatch({
      type: 'callCenter/fetchToUpdAgentInfo',
      payload: {
        callback(agentObj) {
          resolve((agentObj || {}).agentLoginStatus || CALL_LOGIN_STATUS_ABNORMAL);
        },
      },
    });
  }

  switchModalPhoneCallFollow() {
    this.setState((prevState) => {
      this.phoneCallFollowData = prevState.modalPhoneCallFollowVisible
        ? null
        : {
            customerId: this.props.pageParams.customerId,
            studentId: this.props.pageParams.studentId,
            studentName: this.props.pageParams.studentName,
            phoneNum: this.props.pageParams.phoneNum,
            gradeName: this.props.pageParams.gradeName,
            schoolName: this.props.pageParams.schoolName,
            resEntranceName: this.props.pageParams.resEntranceName,
            customerName: this.props.pageParams.customerName,
          };
      return { modalPhoneCallFollowVisible: !prevState.modalPhoneCallFollowVisible };
    });
  }

  switchModalCallDetail(detail) {
    this.setState((prevState) => {
      this.callDetail = prevState.modalCallDetailVisible ? null : detail;
      return { modalCallDetailVisible: !prevState.modalCallDetailVisible };
    });
  }

  render() {
    const { editable } = this.props;
    const hasConsultWorkbrench = this.props.callCenterConfig.hasConsultWorkbrench;
    const alreadyBind = this.props.alreadyBind;
    const {
      list,
      otherRecords,
      editing,
      checking,
      intentionDict,
      cusTagDict,
      invalidTagDict,
      intention,
      tag,
      invalidTag,
      content,
      nextDate,
      appointmentDate,
      appointmentTime,
      saveTipsVis,
      saveTips,
    } = this.state;
    const isInvalidIntention = formatIsInvalidIntention(intention);

    return (
      <div className={styles.wrap}>
        <Sec
          title={
            <span>
              <span>我的跟进记录</span>
              {(this.props.callProps || {}).enabled && hasConsultWorkbrench && alreadyBind && (
                <Button className={styles.callButton} onClick={this.followByCall} type="primary" size="small">
                  外呼跟进
                </Button>
              )}
            </span>
          }
        >
          <Record
            list={list}
            intentionDict={intentionDict}
            handleCallDetail={this.switchModalCallDetail}
            callProps={this.props.callProps}
            page="cousultant"
          />
        </Sec>
        {otherRecords && otherRecords.length > 0 && (
          <Sec title="历史跟进记录">
            <div>
              <div className={styles.info}>
                共<span className={styles.infoHighlight}>{otherRecords.length}</span>
                份跟进记录
              </div>
              {otherRecords.map((rec, idx) => {
                return (
                  <Fold key={idx} title={`${rec.name}跟进记录`} defaultExpand>
                    <Record
                      list={rec.list}
                      intentionDict={intentionDict}
                      handleCallDetail={this.switchModalCallDetail}
                      callProps={this.props.callProps}
                    />
                  </Fold>
                );
              })}
            </div>
          </Sec>
        )}
        {editable && !editing && (
          <div style={{ textAlign: 'center', marginTop: 20 }}>
            {this.isListFollowed && (
              <Button
                type="primary"
                onClick={this.followNext}
                style={{ marginLeft: 20 }}
                disabled={this.isNextDisabled}
              >
                跟进下一个学生
              </Button>
            )}
          </div>
        )}
        {editing && (
          <div className={styles.editBlock}>
            <Row>
              <Col span="24">
                <FormItem {...formCfg} name="跟进内容" required>
                  <TextArea
                    rows={3}
                    wordCount
                    maxLength={1000}
                    value={content}
                    onChange={(e) => this.setState({ content: e.target.value })}
                    validator={{
                      rules: ['required'],
                      silent: !checking,
                      onErrorChange: (e) => this.errorChange('content', e),
                    }}
                  />
                </FormItem>
              </Col>
            </Row>
            <Row>
              <Col span="12">
                <FormItem {...formCfg} name="意向度" required>
                  <Select
                    dict={intentionDict}
                    value={intention}
                    onChange={this.onIntentionChange}
                    validator={{
                      rules: ['required'],
                      silent: !checking,
                      onErrorChange: (e) => this.errorChange('intention', e),
                    }}
                  />
                </FormItem>
              </Col>
              {!isInvalidIntention ? (
                <Col span="12">
                  <FormItem {...formCfg} name="客户意向产品">
                    <Select
                      dict={cusTagDict}
                      value={tag}
                      validator={null}
                      onChange={(val) => this.setState({ tag: val })}
                    />
                  </FormItem>
                </Col>
              ) : (
                <Col span="12">
                  <FormItem {...formCfg} name="无效标签" required>
                    <Select
                      dict={invalidTagDict}
                      value={invalidTag}
                      onChange={(val) => this.setState({ invalidTag: val })}
                      validator={{
                        rules: ['required'],
                        silent: !checking,
                        onErrorChange: (e) => this.errorChange('invalidTag', e),
                      }}
                    />
                  </FormItem>
                </Col>
              )}
            </Row>
            {!isInvalidIntention && (
              <Row>
                <Col span="12">
                  <FormItem {...formCfg} name="下次跟进日期">
                    <DatePicker
                      value={nextDate}
                      onChange={(val) => this.setState({ nextDate: val })}
                      className={styles.pickerInput}
                      disabledDate={this.disableDateAfterToday}
                      style={{ width: '100%' }}
                      validator={null}
                    />
                  </FormItem>
                </Col>
                <Col span="12">
                  <FormItem {...formCfg} name="预约上门日期">
                    <div style={{ width: '100%', display: 'flex' }}>
                      <DatePicker
                        value={appointmentDate}
                        onChange={(val) => this.setState({ appointmentDate: val })}
                        className={styles.pickerInput}
                        disabledDate={this.disableDateAfterToday}
                        style={{ width: '48%' }}
                      />
                      <TimePicker
                        value={appointmentTime}
                        onChange={(val) => this.setState({ appointmentTime: val })}
                        className={styles.pickerInput}
                        disabledHours={this.disabledHours}
                        disabledMinutes={this.disabledMinutes}
                        hideDisabledOptions={true}
                        style={{ width: '48%', marginLeft: '4%' }}
                      />
                    </div>
                  </FormItem>
                </Col>
              </Row>
            )}
            <div style={{ textAlign: 'center', marginTop: 20 }}>
              <PopTip
                visible={saveTipsVis}
                content={saveTips}
                onClose={() => this.setState({ saveTipsVis: false })}
                getPopupContainer={(ele) => ele.parentNode}
              >
                <Button type="primary" onClick={() => this.onOk()}>
                  保存
                </Button>
              </PopTip>
              <Button onClick={this.cancel} style={{ marginLeft: 20 }}>
                重置
              </Button>
              {this.isListFollowed && (
                <Button
                  type="primary"
                  onClick={() => this.onOk(true)}
                  style={{ marginLeft: 20 }}
                  disabled={this.isNextDisabled}
                >
                  保存并跟进下一个学生
                </Button>
              )}
            </div>
          </div>
        )}
        {this.state.modalPhoneCallFollowVisible && (
          <ModalPhoneCallFollow data={this.phoneCallFollowData} onClose={this.switchModalPhoneCallFollow} />
        )}
        {this.state.modalCallDetailVisible && (
          <ModalCallDetail
            data={this.callDetail}
            onClose={this.switchModalCallDetail}
            audioAuth={this.props.audioAuth}
          />
        )}
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    callCenterConfig: state.account.callCenterConfig,
    alreadyBind: state.callCenter.alreadyBind,
  };
};
export default connect(mapStateToProps, null, null, { withRef: true })(RecordsBlock);
