import {
  findCustomerStudentByCusStuId,
  getConsultorStudentFollowingList,
  findCustomerDynamicStatusByCustomerStudentId,
  updateCustomerStudent,
  saveConsultorFollowRecord,
  saveConsultorInvalidFollowRecord,
  getStudentDynamicsInCampus,
  getStudentFollowupRecordsInCampus,
} from 'SERVICE/consultant';
import { findStudentById } from 'SERVICE/student';
import {
  findEletronicAccFlowList,
  getStuEleAccFlowMsg,
  getStudentAccoutInfo,
  getElectronicAccountChangeLogByStudentId,
  getStudentMoneyChanges,
  findContractByStu,
  getContractById,
  getMiniClassByContractProductId,
  findTwoTeacherTwoByContractProductId,
  getMyFundList,
  getMyChargeList,
} from 'SERVICE/eleAcc';

const loadCusDetail = async (id, stuId, deliverTarget) => {
  const { data: d } = await findCustomerStudentByCusStuId({
    customerId: id,
    studentId: stuId,
    deliverTarget,
  });
  return {
    ...d,
    studentName: d.studentName,
    grade: d.gradeId,
    school: d.schoolId,
    schoolName: d.schoolName,
    address: d.address,
    studentContract: d.stuContact,
    birthDay: d.bothday,
    cusName: d.cusName,
    entrance: d.resEntranceId,
    source: d.cusType,
    sourceDetail: d.cusOrg,
    campus: d.blCampusId,
    cusContract: d.contact,
    dispatcher: d.lastDeliverName,
    follower: d.deliverTargetName,
    followStatus: d.dealStatus,
    followStatusName: d.dealStatusName,
    branch: d.brenchId,
    remark: d.remark,
    introducer: d.introducer,
    introductionType: d.introductionType,
    reactivated: d.reactivated,
    preEntranceId: d.preEntranceId,
    preEntranceName: d.preEntranceName,
    cusIdCardNo: d.cusIdCardNo,
    cusIdCardType: d.cusIdCardType,
    cusIdCardTypeName: d.cusIdCardTypeName,
    partialCoverCusIdCardNo: d.partialCoverCusIdCardNo,
  };
};

const loadRecords = async (id, stuId, deliverTarget) => {
  const { data: apiData } = await getConsultorStudentFollowingList({
    customerId: id,
    studentId: stuId,
    deliverTarget,
  });
  const { MyFollowingList, OtherFollowingList } = apiData;
  const data = MyFollowingList.map((d) => {
    return {
      time: !!d.createTime && d.createTime,
      intention: d.intentionOfTheCustomerId,
      tag: d.customerSignName,
      content: d.remark,
      deliverTargetName: d.createUserName,
      nextFollowTime: d.nextFollowTime,
      meetingTime: d.meetingTime,
      createUserName: d.createUserName,
    };
  });
  const otherRecords = Object.keys(OtherFollowingList).map((key) => {
    return {
      name: key,
      list: OtherFollowingList[key].map((d) => {
        return {
          time: !!d.createTime && d.createTime,
          intention: d.intentionOfTheCustomerId,
          tag: d.customerSignName,
          content: d.remark,
          deliverTargetName: d.createUserName,
          meetingTime: d.meetingTime,
          nextFollowTime: d.nextFollowTime,
          createUserName: d.createUserName,
        };
      }),
    };
  });
  return { list: data, otherRecords };
};

const loadRecordsManage = async (campusId, customerId, studentId) => {
  const res = await getStudentFollowupRecordsInCampus({
    campusId,
    customerId,
    studentId,
  });

  const list = res.data.map((ele) => {
    const consultantName = ele.consultantName;
    return {
      consultantName,
      followupRecords: (ele.followupRecords.length ? ele.followupRecords : []).map((d) => {
        return {
          time: !!d.createTime && d.createTime,
          consultantName: consultantName,
          intention: d.intentionOfTheCustomerId,
          tag: d.customerSignName,
          content: d.remark,
          deliverTargetName: d.createUserName,
          nextFollowTime: d.nextFollowTime,
          meetingTime: d.meetingTime,
        };
      }),
    };
  });
  return { list };
};

const loadIntentionDict = async () => {
  return [
    { val: '1', text: '新客户' },
    { val: '2', text: '有意向' },
    { val: '3', text: '低意向' },
    { val: '4', text: '高意向' },
    { val: '5', text: '无意向' },
  ];
};

const loadCusTagDict = async () => {
  return [
    { val: '1', text: '一对一体验课' },
    { val: '2', text: '小班体验课' },
    { val: '3', text: '个性化诊断' },
    { val: '4', text: '中考精英班' },
    { val: '5', text: '高考精英班' },
    { val: '6', text: '小升初精英班' },
    { val: '7', text: '10人课堂体验课' },
    { val: '8', text: '1对1' },
    { val: '9', text: '活动课（讲座、杯赛）' },
    { val: '10', text: '小班体验课' },
    { val: '11', text: '1对多' },
  ];
};

const loadCusNews = async (customerId, studentId, deliverTarget) => {
  const list = await findCustomerDynamicStatusByCustomerStudentId({
    customerId,
    studentId,
    deliverTarget,
  });

  return list.map((d) => {
    return { time: d.occourTime, content: d.description };
  });
};

const loadCusNewsForManager = async (campusId, customerId, studentId) => {
  const res = await getStudentDynamicsInCampus({
    campusId,
    customerId,
    studentId,
  });

  return res.data.map((d) => {
    return { time: d.occourTime, content: d.description };
  });
};

const saveCusDetail = async ({
  id,
  studentName,
  grade,
  school,
  address,
  studentContract,
  birthDay,
  cusName,
  source,
  sourceDetail,
  remark,
  stuId,
  cusIdCardNo,
  cusIdCardType,
  studentFatherPhone,
  studentMotherPhone,
}) => {
  const params = {
    cusId: id,
    studentName: studentName,
    gradeId: grade,
    schoolId: school,
    address: address,
    stuContact: studentContract,
    bothday: birthDay,
    cusName: cusName,
    cusType: source,
    cusOrg: sourceDetail,
    remark: remark,
    studentId: stuId,
    cusIdCardNo: cusIdCardNo,
    cusIdCardType,
    studentFatherPhone,
    studentMotherPhone,
  };
  const res = await updateCustomerStudent(params);
  return res;
};

/**
 * 保存跟进记录
 *
 * @return {Promise<boolean>}
 */
const saveFollowRecord = async (param = {}) => {
  const { id, stuId, intention, tag, content, nextDate, appointmentDate, appointmentTime } = param;
  const params = {
    appointmentType: 'FOLLOW_UP',
    customerId: id,
    followStuId: stuId,
    satisficing: intention,
    customerSign: tag,
    remark: content,
    nextFollowTime: nextDate,
    meetingTime: appointmentDate && appointmentTime ? `${appointmentDate} ${appointmentTime}:00` : null,
  };
  await saveConsultorFollowRecord(params);
  return true;
};

/**
 * 保存跟进记录（意向度选择-“无效客户”时调用）
 *
 * @return {Promise<void>}
 */
function saveInvalidFollowRecord(params) {
  return saveConsultorInvalidFollowRecord(params);
}

const getStudentById = (stuId) => {
  return findStudentById({ id: stuId });
};

const getEletronicAccFlowList = async (params) => {
  const data = await findEletronicAccFlowList(params);
  if (data.resultCode !== 0) {
    throw new Error(data.resultMessage);
  }
  return data;
};

const getStuEleAccFlow = async (params) => {
  const res = await getStuEleAccFlowMsg(params);
  if (res.resultCode !== 0) {
    throw new Error(res.resultMessage);
  }
  return res.data;
};

export default {
  loadCusDetail,
  loadRecords,
  loadIntentionDict,
  loadCusTagDict,
  loadCusNews,
  saveCusDetail,
  saveFollowRecord,
  saveInvalidFollowRecord,
  loadCusNewsForManager,
  loadRecordsManage,
  getStudentById,
  getEletronicAccFlowList,
  getStuEleAccFlow,
  getStudentAccoutInfo,
  getElectronicAccountChangeLogByStudentId,
  getStudentMoneyChanges,
  findContractByStu,
  getContractById,
  getMiniClassByContractProductId,
  findTwoTeacherTwoByContractProductId,
  getMyFundList,
  getMyChargeList,
};
