import { getDataDictList } from 'SERVICE/system';
import { getDistinctUserGroupByCampus, searchBranchBank } from 'SERVICE/common';

export const loadSourceDetailDict = async (source) => {
  const data = await getDataDictList({
    category: 'CUS_ORG',
    'parentDataDict.id': source,
  });
  const dict = data.rows.map((d) => {
    return { val: d.id, text: d.name };
  });
  return dict;
};

export const loadConsultantDict = async (params) => {
  const api = await getDistinctUserGroupByCampus({ ...params, showBelong: 1, job: ['ZXS', 'ZXZG'] });
  return api.map((d) => {
    return { val: d.userId, text: d.userName };
  });
  /*  return [
      { val: '1', text: '李咨询' },
      { val: '2', text: '刘咨询' },
      { val: '3', text: '张咨询' },
    ]*/
};

/**
 * @desc 银行卡支行搜索数据处理
 */
export function searchBankBranchLoader(name) {
  return new Promise(async (resolve, reject) => {
    try {
      const { resultCode, data } = await searchBranchBank({
        name,
        pageNo: 0,
        pageSize: 500,
        isFromAts: true /** 获取ATS数据源 */,
      });
      resolve(
        resultCode === 0 && Array.isArray(data.branchList)
          ? data.branchList.map(({ name, branchCode, code }) => ({ label: name, value: branchCode || code }))
          : [],
      );
    } catch (e) {
      reject(e.text || '加载数据错误');
    }
  });
}
