import React from 'react';
import styles from './styles.less';
import List from '../FreeHeaderHeightList';
import { TIME_RANGE_WEEK, TIME_RANGE_MONTH, TIME_RANGE_QUARTER, timeRangeDict, cusStatusDict } from '../dicts';
import loaders from '../loaders';
import Tabs from 'COMPONENT/Tabs';
import DatePicker from 'COMPONENT/DatePicker';
import { getDictTextByVal } from 'UTIL/dict';
import moment from 'moment';
import SortButton from 'COMPONENT/SortButton';
import Section from 'COMPONENT/Section';

const { Tab } = Tabs;
const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;
const initSorts = {
  total: null,
  pull: null,
  direct: null,
  callIn: null,
  net: null,
  recommend: null,
  tmk: null,
  callOut: null,
};
class FollowRadioSec extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 加载状态
      activeTabKey: TIME_RANGE_WEEK, // {string} 当前激活状态的页签ID
      startDate: null, // {string} 开始日期搜索条件
      endDate: null, // {string} 结束日期搜索条件
      list: [], // {object[]} 列表数据，
      /* [{ name, total, totalAll, pull, pullAll, direct, directAll, callIn, callInAll,
       net, netAll, recommend, recommendAll, tmk, tmkAll, callOut, callOutAll }]
       */
      sorts: initSorts, // {object} 排序按钮状态
    };
  }

  componentDidMount() {
    this.initContent();
  }

  initContent = async () => {
    const { activeTabKey } = this.state;
    this.setState({ loading: true });
    const list = await loaders.loadFollowRadio({ timeRange: activeTabKey });
    this.setState({ loading: false, list });
  };

  getCellContent = (record, key) => {
    const val = record[key];
    const all = record[`${key}All`];
    const percent = Math.round((val / all) * 100);
    return (
      <div className={styles.cell}>
        <div>{`${val}/${all}`}</div>
        <div className={styles.percent}>{`${percent}%`}</div>
      </div>
    );
  };

  onTabChange = async (key) => {
    const { sorts } = this.state;
    this.setState({ loading: true });
    const list = await loaders.loadFollowRadio({ timeRange: key, sorts });
    this.setState({ loading: false, list, activeTabKey: key, startDate: null, endDate: null });
  };

  startDateChange = (val) => {
    const { endDate, sorts } = this.state;
    this.setState({ loading: true });
    loaders.loadFollowRadio({ startDate: val, endDate, sorts }).then((list) => {
      this.setState({ loading: false, list, startDate: val, activeTabKey: '' });
    });
  };

  endDateChange = (val) => {
    const { startDate, sorts } = this.state;
    this.setState({ loading: true });
    loaders.loadFollowRadio({ startDate, endDate: val, sorts }).then((list) => {
      this.setState({ loading: false, list, endDate: val, activeTabKey: '' });
    });
  };

  getDisabledStartDate = (startDate) => {
    const { endDate } = this.state;
    if (!startDate || !endDate) {
      return false;
    }
    return moment(startDate).valueOf() > moment(endDate).valueOf();
  };

  getDisabledEndDate = (endDate) => {
    const { startDate } = this.state;
    if (!endDate || !startDate) {
      return false;
    }
    return moment(endDate).valueOf() < moment(startDate).valueOf();
  };

  sortChange = (type, val) => {
    const { activeTabKey, startDate, endDate } = this.state;
    const sorts = { ...initSorts, [type]: val };
    this.setState({ loading: true });
    loaders.loadFollowRadio({ timeRange: activeTabKey, startDate, endDate, sorts }).then((list) => {
      this.setState({ loading: false, list, sorts });
    });
  };

  getListTitle = (name, type) => {
    const { sorts } = this.state;
    return (
      <div>
        <div className={styles.titleLeft}>
          <div className={styles.titleMain}>{name}</div>
          <div className={styles.titleInfo}>{`跟进量/资源量`}</div>
        </div>
        <div className={styles.titleRight}>
          <SortButton value={sorts[type]} onChange={(value) => this.sortChange(type, value)}>
            {name}
          </SortButton>
        </div>
      </div>
    );
  };

  render() {
    const { activeTabKey, startDate, endDate, list, loading } = this.state;
    const columns = [
      { title: '学习中心咨询师', dataIndex: 'name' },
      {
        title: this.getListTitle('客户总量', 'total'),
        width: COL_WIDTH_L,
        dataIndex: 'total',
        render: (text, record) => this.getCellContent(record, 'total'),
      },
      {
        title: this.getListTitle('拉访', 'pull'),
        width: COL_WIDTH_L,
        dataIndex: 'pull',
        render: (text, record) => this.getCellContent(record, 'pull'),
      },
      {
        title: this.getListTitle('直访', 'direct'),
        width: COL_WIDTH_L,
        dataIndex: 'direct',
        render: (text, record) => this.getCellContent(record, 'direct'),
      },
      {
        title: this.getListTitle('呼入', 'callIn'),
        width: COL_WIDTH_L,
        dataIndex: 'callIn',
        render: (text, record) => this.getCellContent(record, 'callIn'),
      },
      {
        title: this.getListTitle('网络', 'net'),
        width: COL_WIDTH_L,
        dataIndex: 'net',
        render: (text, record) => this.getCellContent(record, 'net'),
      },
      {
        title: this.getListTitle('转介绍', 'recommend'),
        width: COL_WIDTH_L,
        dataIndex: 'recommend',
        render: (text, record) => this.getCellContent(record, 'recommend'),
      },
      {
        title: this.getListTitle('TMK', 'tmk'),
        width: COL_WIDTH_L,
        dataIndex: 'tmk',
        render: (text, record) => this.getCellContent(record, 'tmk'),
      },
      {
        title: this.getListTitle('呼出', 'callOut'),
        width: COL_WIDTH_L,
        dataIndex: 'callOut',
        render: (text, record) => this.getCellContent(record, 'callOut'),
      },
    ];
    return (
      <Section title="客户跟进率" loading={loading}>
        <div className={styles.wrap}>
          <Tabs onChange={this.onTabChange} navAlign="center" activeKey={activeTabKey} style={{ marginTop: -10 }}>
            <Tab title={getDictTextByVal(timeRangeDict, TIME_RANGE_WEEK)} value={TIME_RANGE_WEEK} />
            <Tab title={getDictTextByVal(timeRangeDict, TIME_RANGE_MONTH)} value={TIME_RANGE_MONTH} />
            <Tab title={getDictTextByVal(timeRangeDict, TIME_RANGE_QUARTER)} value={TIME_RANGE_QUARTER} />
          </Tabs>
          <div className={styles.toolBar}>
            <DatePicker
              value={startDate}
              onChange={this.startDateChange}
              disabledDate={this.getDisabledStartDate}
              className={styles.input}
            />
            <DatePicker
              value={endDate}
              onChange={this.endDateChange}
              popupPlacement="bottomRight"
              disabledDate={this.getDisabledEndDate}
              className={styles.input}
            />
          </div>
          <div className={styles.list}>
            <List dataSource={list} columns={columns} />
          </div>
        </div>
      </Section>
    );
  }
}
export default FollowRadioSec;
