import React from 'react';
import styles from './styles.less';
import {
  TIME_RANGE_TODAY,
  TIME_RANGE_WEEK,
  TIME_RANGE_MONTH,
  TIME_RANGE_QUARTER,
  timeRange2Dict,
} from 'PAGE/consultant/common/dicts';
import { loadRankingData } from 'PAGE/consultant/common/loaders';
import Tabs from 'COMPONENT/Tabs';
import { getDictTextByVal } from 'UTIL/dict';
import moment from 'moment';
import AchievementBarChart from 'PAGE/consultant/common/AchievementChart';
import AchievementRankingItem from 'PAGE/consultant/common/AchievementChart/RankingItem';
import Section from 'COMPONENT/Section';
import TextLink from 'COMPONENT/TextLink';

const { Tab } = Tabs;

class RankingSec extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 加载状态
      activeTabKey: TIME_RANGE_TODAY, // {string} 当前激活状态的页签ID
      data: [], // {object[]} 图表数据，[{ userId, name, campusName, num, asc, desc, rank }]
    };
  }

  componentDidMount() {
    this.initContent();
  }

  initContent = async () => {
    const { activeTabKey } = this.state;
    this.setState({ loading: true });
    const data = await loadRankingData({ timeRange: activeTabKey });
    this.setState({ loading: false, data });
  };

  onTabChange = async (key) => {
    this.setState({ loading: true });
    const data = await loadRankingData({ timeRange: key });
    this.setState({ loading: false, data, activeTabKey: key });
  };

  scrollTo = (rank) => {
    const barRef = this.chartRef.getBarRef(rank);
    const barT = barRef.offsetTop;
    this.props.onScrollPage(barT + 470);
  };

  render() {
    const { userId } = this.props;
    const { activeTabKey, data, loading } = this.state;
    const myData = data.find((d) => d.isMe);

    return (
      <div className={styles.wrap}>
        <Section title="分公司业绩排名" loading={loading}>
          <Tabs onChange={this.onTabChange} navAlign="center" activeKey={activeTabKey} style={{ marginTop: -10 }}>
            <Tab title={getDictTextByVal(timeRange2Dict, TIME_RANGE_TODAY)} value={TIME_RANGE_TODAY} />
            <Tab title={getDictTextByVal(timeRange2Dict, TIME_RANGE_WEEK)} value={TIME_RANGE_WEEK} />
            <Tab title={getDictTextByVal(timeRange2Dict, TIME_RANGE_MONTH)} value={TIME_RANGE_MONTH} />
            <Tab title={getDictTextByVal(timeRange2Dict, TIME_RANGE_QUARTER)} value={TIME_RANGE_QUARTER} />
          </Tabs>
          <div className={styles.main}>
            <div className={styles.left}>
              <AchievementBarChart
                data={data.map((d) => {
                  return { ...d, showNum: d.isMe };
                })}
                currentUserId={userId}
                showNumCount={10}
                ref={(c) => (this.chartRef = c)}
              />
            </div>
            <div className={styles.right}>
              {myData && (
                <div>
                  <div className={styles.subTitle}>
                    <TextLink onClick={() => this.scrollTo(myData.rank)}>我的排名</TextLink>
                  </div>
                  <div className={styles.myRanking}>
                    <AchievementRankingItem data={myData || {}} />
                  </div>
                </div>
              )}
            </div>
          </div>
        </Section>
      </div>
    );
  }
}

export default RankingSec;
