import React from 'react';
import styles from './styles.less';
import loaders from '../loaders';
import echarts from 'ASSET/js/echarts.min';
import dicts from '../dicts';
import { cusStatusDict, CUS_STATUS_VISITED } from '../dicts';
import {
  TIME_RANGE_TODAY,
  TIME_RANGE_WEEK,
  TIME_RANGE_MONTH,
  TIME_RANGE_QUARTER,
  timeRangeDict,
  timeRange2Dict,
} from 'PAGE/consultant/common/dicts';
import List from 'COMPONENT/List';
import { loadCommonDict } from 'UTIL/dict';
import Tabs from 'COMPONENT/Tabs';
import { getDictTextByVal } from '../../../../utils/dict/index';
import classnames from 'classnames';
import Section from 'COMPONENT/Section';
import CampusSelector from '../../common/CampusSelector';

const { Tab } = Tabs;
const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;

class SaleFunnelSec extends React.Component {
  constructor(props) {
    super(props);
    let data = {};
    timeRangeDict.forEach((d) => {
      data[d.val] = { funnel: [], visited: [], listStatus: null };
    });
    this.state = {
      loading: false, // {boolean} 页面加载状态
      data, // {object} 所有数据，{ type: { funnel: [{ value, name }], visited: [{ stuName, grade, name, contract, id }] } }
      activeTabKey: TIME_RANGE_WEEK, // {string} 当前的标签页ID
      gradeDict: [], // {object[]} 年级字典
      activeFunnelItem: null, // {string} 漏斗图中当前激活的项目
      orgId: this.props.campusId || '',
      consultantId: this.props.consultantId || '',
      branchId: this.props.branchId || '',
    };
    this.funnelOption = {
      animation: false,
      series: [
        {
          name: '销售漏斗',
          type: 'funnel',
          left: 20,
          top: 0,
          bottom: 10,
          width: 210,
          minSize: 30,
          sort: 'none',
          gap: 4,
          label: {
            normal: { show: false },
            emphasis: { show: false },
          },
          data: [],
        },
      ],
    };
  }

  componentDidMount() {
    this.chart = echarts.init(this.chartContainerRef);
    this.chart.on('mouseover', (params) => this.setState({ activeFunnelItem: params.data.nameVal }));
    this.chart.on('mouseout', (params) => this.setState({ activeFunnelItem: null }));
    this.chart.on('click', this.funnelItemClick);
    this.initContent();
  }

  initContent = async () => {
    const { data, activeTabKey, branchId, orgId, consultantId } = this.state;
    const activeData = data[activeTabKey];
    this.setState({ loading: true });
    const [funnelInfo, gradeDict] = await Promise.all([
      loaders.loadSaleFunnel({ timeRange: activeTabKey, campusId: orgId, deliverTarget: consultantId, branchId }),
      loadCommonDict('STUDENT_GRADE'),
    ]);
    const { funnel, all } = funnelInfo;
    let listStatus = null;
    let visited = [];
    const firstItem = funnel.find((f) => f.value);
    if (firstItem) {
      listStatus = firstItem.nameVal;
      visited = await loaders.loadSaleCusList({
        timeRange: activeTabKey,
        cusStatus: listStatus,
        campusId: orgId,
        deliverTarget: consultantId,
        branchId,
      });
    }
    this.setState({
      loading: false,
      data: { ...data, [activeTabKey]: { funnel, all, visited, listStatus } },
      gradeDict,
    });
    this.refreshFunnel(funnel);
  };

  funnelItemClick = (params) => {
    const cusStatus = params.data.nameVal;
    const { data, activeTabKey, orgId, consultantId, branchId } = this.state;
    this.setState({ loading: true });
    loaders
      .loadSaleCusList({ timeRange: activeTabKey, cusStatus, campusId: orgId, deliverTarget: consultantId, branchId })
      .then((visited) => {
        this.setState({
          loading: false,
          data: { ...data, [activeTabKey]: { ...data[activeTabKey], visited, listStatus: cusStatus } },
        });
      });
  };

  refreshFunnel = (data) => {
    this.funnelOption.series[0].data = data.filter((d) => d.value);
    this.chart.setOption(this.funnelOption);
  };

  reloadTab = async (params) => {
    const { timeRange: key } = params;
    const { data } = this.state;
    this.setState({ loading: true });
    const { funnel, all } = await loaders.loadSaleFunnel(params);
    let listStatus = data[key].listStatus;
    let visited = [];
    if (!funnel.some((f) => f.nameVal === listStatus && f.value)) {
      const firstItem = funnel.find((f) => f.value);
      if (firstItem) {
        listStatus = firstItem.nameVal;
      } else {
        listStatus = null;
      }
    }
    if (listStatus) {
      visited = await loaders.loadSaleCusList({ ...params, cusStatus: listStatus });
    }
    this.setState({ loading: false, data: { ...data, [key]: { funnel, all, visited, listStatus } } });
    this.refreshFunnel(funnel);
  };

  onTabChange = (key) => {
    const { data, orgId, consultantId, branchId } = this.state;
    this.setState({ activeTabKey: key });
    this.reloadTab({ timeRange: key, campusId: orgId, deliverTarget: consultantId, branchId });
  };

  onSelectorChange = (orgId, consultantId, branchId) => {
    const { activeTabKey } = this.state;
    this.setState({ orgId, consultantId, branchId });
    this.reloadTab({ timeRange: activeTabKey, campusId: orgId, deliverTarget: consultantId, branchId });
  };

  goDetail = (record) => {
    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path: '/CommonCusDetailPage',
        params: record,
        refresh: true,
        mirrorId: `ConsultantDataBoard/SaleFunnelSec/${record.customerId}${record.studentId}`,
      },
    });
  };

  render() {
    const {
      showSearch,
      branchId,
      campusId,
      branchSelectorDisabled,
      campusSelectorDisabled,
      showBranchSelector,
      showCampusSelector,
      showConsultantSelector,
      consultantId,
      consultantSelectorDisabled,
      allowCampusSelectorClear,
    } = this.props;
    const { loading, data, activeTabKey, gradeDict, activeFunnelItem } = this.state;
    const activeData = data[activeTabKey];
    const list = activeData.visited;
    const funnelData = activeData.funnel;
    const funnelTotal = activeData.all;
    const hasData = data[activeTabKey].funnel.some((d) => d.value);
    const columns = [
      { title: '学生姓名', dataIndex: 'stuName' },
      { title: '学生年级', dataIndex: 'grade', dict: gradeDict },
      { title: '客户姓名', dataIndex: 'name' },
      { title: '联系方式', dataIndex: 'contract', width: COL_WIDTH_L },
      {
        title: '操作',
        key: 'action',
        fixed: list.length > 0 ? 'right' : null,
        getActionConfig: (text, record, index) => [{ text: '详情', onClick: () => this.goDetail(record) }],
      },
    ];
    return (
      <Section title="销售漏斗" loading={loading}>
        {showSearch && (
          <div>
            <CampusSelector
              branchSelectorDisabled={branchSelectorDisabled}
              campusSelectorDisabled={campusSelectorDisabled}
              branchId={branchId}
              campusId={campusId}
              showBranchSelector={showBranchSelector}
              showCampusSelector={showCampusSelector}
              showConsultantSelector={showConsultantSelector}
              consultantId={consultantId}
              consultantSelectorDisabled={consultantSelectorDisabled}
              allowCampusSelectorClear={allowCampusSelectorClear}
              onChange={this.onSelectorChange}
            />
          </div>
        )}
        <div className={styles.wrap}>
          <Tabs activeKey={activeTabKey} onChange={this.onTabChange} navAlign="center" style={{ marginTop: -10 }}>
            <Tab title={getDictTextByVal(timeRangeDict, TIME_RANGE_WEEK)} value={TIME_RANGE_WEEK} />
            <Tab title={getDictTextByVal(timeRangeDict, TIME_RANGE_MONTH)} value={TIME_RANGE_MONTH} />
            <Tab title={getDictTextByVal(timeRangeDict, TIME_RANGE_QUARTER)} value={TIME_RANGE_QUARTER} />
          </Tabs>
          <div className={styles.main} style={{ display: hasData ? 'block' : 'none' }}>
            <div className={styles.left}>
              <div ref={(c) => (this.chartContainerRef = c)} style={{ width: 280, height: 280 }} />
              <div className={styles.legend}>
                {funnelData.map((d, idx) => {
                  // const percent = funnelTotal ? Math.round(d.value / funnelTotal  * 100) : 0
                  const itemCls = {
                    [styles.legendItem]: 1,
                    [styles.legendItemActive]: d.nameVal === activeFunnelItem,
                  };
                  const dictItem = cusStatusDict.find((cd) => cd.text === d.name);
                  return (
                    <div key={idx} className={classnames(itemCls)}>
                      <span
                        className={styles.legendItemIcon}
                        style={dictItem ? { background: dictItem.color } : null}
                      />
                      <span className={styles.legendItemText + ' ' + styles.legendName}>{d.name}</span>
                      <span className={styles.legendItemText}>{`${d.value}个`}</span>
                      {/* <span className={styles.legendItemText + ' ' + styles.legendPercent}>{`${percent}%`}</span> */}
                    </div>
                  );
                })}
              </div>
            </div>
            <div className={styles.right}>
              <div className={styles.subTitle}>{getDictTextByVal(cusStatusDict, activeData.listStatus)}</div>
              <div className={styles.list}>
                <List dataSource={list} columns={columns} />
              </div>
            </div>
          </div>
          {!hasData && <div className={styles.tips}>暂无数据</div>}
        </div>
      </Section>
    );
  }
}
export default SaleFunnelSec;
