import React from 'react';
import { connect } from 'dva';
import moment from 'moment';
import { Page, SearchFilterForm, SearchFieldForm, Input, Select, List, ListAction } from 'COMPONENT';
import { momentToDate } from 'COMPONENT/utils/form';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { errorsHandle, loginUser } from 'UTIL/common';
import { createJxYearDict, jxQuarterDict } from 'CONST/dicts/jiaoXueDict';
import { getJxDictByType } from 'UTIL/dict';
import { getPaperList, getImportState } from 'SERVICE/entranceExam';
import { extraBranchDict } from '../config';
import ModalReportProgress from '../ModalReportProgress';

const setWidth = List.createSetWidth();
const jxYearDict = createJxYearDict();

class TestPaperManage extends React.Component {
  constructor(props) {
    super(props);

    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.initJxDict = this.initJxDict.bind(this);
    this.initSKey = this.initSKey.bind(this);
    this.getListParams = this.getListParams.bind(this);
    this.loadList = this.loadList.bind(this);
    this.search = this.search.bind(this);
    this.updatePage = this.updatePage.bind(this);
    this.changePage = this.changePage.bind(this);
    this.viewPrint = this.viewPrint.bind(this);
    this.inputScore = this.inputScore.bind(this);
    this.waitReport = this.waitReport.bind(this);
    this.pollFetchReport = this.pollFetchReport.bind(this);
    this.closeReportProgress = this.closeReportProgress.bind(this);
    this.openReport = this.openReport.bind(this);

    this.state = {
      title: '试卷管理',
      loading: false,
      searchKey: null,
      defaultSKey: {},
      list: [],
      total: 0,
      pageCurrent: 1,
      pageSize: 10,
      // 教学数据字典
      gradeDict: [],
      subjectDict: [],
      //
      reportProgressVisible: false,
      reportPercent: 0,
    };

    this.columns = [
      { title: '试卷名称', dataIndex: 'name', key: 'name', width: setWidth(20) },
      { title: '分公司', dataIndex: 'branchName', key: 'branchName', width: setWidth(5) },
      { title: '年份', dataIndex: 'year', key: 'year', width: setWidth(2) },
      { title: '季度', dataIndex: 'season', key: 'season', width: setWidth(2) },
      { title: '年级', dataIndex: 'grade', key: 'grade', width: setWidth(3) },
      { title: '科目', dataIndex: 'subject', key: 'subject', width: setWidth(2) },
      { title: '创建人', dataIndex: 'creator', key: 'creator', width: setWidth(4) },
      {
        title: '创建时间',
        dataIndex: 'createDttm',
        key: 'createDttm',
        width: setWidth(4),
        render: (text, record) => {
          return text ? momentToDate(moment(new Date(text))) : '';
        },
      },
      {
        title: '操作',
        dataIndex: 'id',
        key: 'actionPlus',
        width: setWidth(10),
        render: (text, record, index) => {
          let config = [];
          if (props.viewPrintAuth) {
            config.push({
              text: '预览/打印',
              onClick: () => {
                this.viewPrint(record);
              },
            });
          }
          if (props.inputScoreAuth) {
            config.push({
              text: '录入分数',
              onClick: () => {
                this.inputScore(record);
              },
            });
          }
          return <ListAction config={config} />;
        },
      },
    ];
  }

  componentDidMount() {
    this.init();
  }

  async init() {
    this.setState({ loading: true });
    await new Promise(this.initDict);
    await new Promise(this.initJxDict);
    await new Promise(this.initSKey);
    await new Promise(this.loadList);
    this.setState({ loading: false });
  }

  initDict(resolve) {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['branch'],
        callback() {
          resolve();
        },
      },
    });
  }

  async initJxDict(resolve) {
    const [gradeDict, subjectDict] = await Promise.all([getJxDictByType('GRADE'), getJxDictByType('SUBJECT')]);

    this.setState({ gradeDict, subjectDict });

    resolve();
  }

  initSKey(resolve) {
    const isGroup = loginUser.isGroupRole();
    let newSKey = {
      branchId: isGroup ? (this.props.branchDict || [{ val: null }])[0].val : loginUser.userData.brenchId,
      year: new Date().getFullYear().toString(),
    };
    this.setState(
      {
        searchKey: newSKey,
        defaultSKey: { ...newSKey },
      },
      () => {
        resolve();
      },
    );
  }

  getListParams() {
    let searchKey = {
      pageNum: this.state.pageCurrent,
      pageSize: this.state.pageSize,
    };

    for (let key in this.state.searchKey) {
      searchKey[key] = this.state.searchKey[key];
    }

    return searchKey;
  }

  async loadList(resolve) {
    const apiParams = this.getListParams();
    const rsp = await getPaperList(apiParams, true).catch(errorsHandle());
    if (rsp && rsp.resultCode === 0 && rsp.data) {
      this.setState({
        list: rsp.data.list || [],
        total: rsp.data.total,
      });
    }

    resolve();
  }

  search(params) {
    this.setState(
      {
        searchKey: { ...params },
      },
      () => {
        this.changePage({ current: 1, pageSize: this.state.pageSize });
      },
    );
  }

  updatePage(pageCurrent, pageSize) {
    return new Promise((resolve) => {
      this.setState(
        (prevState) => {
          let payload = {};
          if (pageSize && pageSize !== prevState.pageSize) {
            payload.pageCurrent = 1;
            payload.pageSize = pageSize;
          } else {
            payload.pageCurrent = pageCurrent;
          }
          return payload;
        },
        () => {
          resolve();
        },
      );
    });
  }

  async changePage({ current, pageSize }) {
    await this.updatePage(current, pageSize);
    this.setState({ loading: true });
    await new Promise(this.loadList);
    this.setState({ loading: false });
  }

  viewPrint(record) {
    let newPage = window.open('about:blank');
    setTimeout(() => {
      newPage.document.location.replace(record.previewUrl);
    }, 1);
  }

  inputScore(record) {
    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path: `/EntranceTestPaperManage/Add`,
        refresh: true,
        params: {
          paperId: record.id,
          title: record.name,
          branchId: record.branchId,
          onSaveSuccess: this.onInputScoreSuccess,
        },
      },
    });
  }

  onInputScoreSuccess = (examId) => {
    history.back();
    this.waitReport(examId);
    this.search(this.state.searchKey);
  };

  waitReport(examId) {
    if (!examId) {
      return showMessage('保存时未返回正确的考试ID，请手动打开报告管理查看考试报告', 'error');
    }
    this.setState({ reportProgressVisible: true });
    this.pollFetchReport(examId);
  }

  async pollFetchReport(examId) {
    const res = await getImportState({ examId }).catch(errorsHandle());
    if (res && res.resultCode === 0) {
      if (!res.data) {
        return this.closeReportProgress();
      }

      // 成功(5)
      if (res.data.importStatus === 5) {
        return this.setState({ reportPercent: 100 }, () => {
          const _self = this;
          setTimeout(function () {
            _self.openReport(res.data.reportUrl);
            _self.closeReportProgress();
          }, 1000);
        });
      }

      // 校验失败(2)、导入失败(4)
      if (res.data.importStatus === 2 || res.data.importStatus === 4) {
        this.closeReportProgress();
        return showMessage(res.data.errorMessage, 'error');
      }

      // 校验中(1)、导入中(3)
      this.setState({ reportPercent: res.data.progress });
      const _self = this;
      setTimeout(function () {
        _self.pollFetchReport(examId);
      }, 600);
    } else {
      this.closeReportProgress();
    }
  }

  closeReportProgress() {
    this.setState({
      reportProgressVisible: false,
      reportPercent: 0,
    });
  }

  openReport(url) {
    if (url) {
      this.viewPrint({ previewUrl: url });
    } else {
      showMessage('未返回报告地址，请手动打开报告管理查看考试报告', 'error');
    }
  }

  render() {
    return (
      <Page
        title={this.state.title}
        loading={this.state.loading || this.state.reportLoading}
        subHeader={
          <SearchFilterForm onSearch={this.search} truncationNum={4}>
            <SearchFieldForm name="name">
              <Input placeholder="试卷名称" />
            </SearchFieldForm>
            <SearchFieldForm name="branchId" initialValue={this.state.defaultSKey.branchId}>
              <Select placeholder="分公司" dict={this.props.branchDict} allowClear={false} />
            </SearchFieldForm>
            <SearchFieldForm name="year" initialValue={this.state.defaultSKey.year}>
              <Select placeholder="年份" dict={jxYearDict} />
            </SearchFieldForm>
            <SearchFieldForm name="seasonId">
              <Select placeholder="季度" dict={jxQuarterDict} />
            </SearchFieldForm>
            <SearchFieldForm name="grade">
              <Select placeholder="年级" dict={this.state.gradeDict} />
            </SearchFieldForm>
            <SearchFieldForm name="subject">
              <Select placeholder="科目" dict={this.state.subjectDict} />
            </SearchFieldForm>
          </SearchFilterForm>
        }
      >
        <List
          columns={this.columns}
          dataSource={this.state.list}
          pagination={{
            total: this.state.total,
            current: this.state.pageCurrent,
            pageSize: this.state.pageSize,
          }}
          onChange={this.changePage}
        />
        {this.state.reportProgressVisible && <ModalReportProgress percent={this.state.reportPercent} />}
      </Page>
    );
  }
}

let branchDictStorage = null;

const mapStateToProps = (state) => {
  if (state.dict.branch && !branchDictStorage) {
    branchDictStorage = extraBranchDict.concat(state.dict.branch);
  }
  return {
    branchDict: branchDictStorage,
    viewPrintAuth: state.account.btnAuths.indexOf('EETPM_VIEW_PRINT') > -1,
    inputScoreAuth: state.account.btnAuths.indexOf('EETPM_INPUT_SCORE') > -1,
  };
};

export default connect(mapStateToProps)(TestPaperManage);
