/**
 * 判断题目分数是否修改
 *
 * @param originList {array} 原始列表
 * @param changeList {array} 修改后的列表
 * @param idKey {string?} 唯一标识字段
 * @param compareKey {string?} 需要比较值的字段
 * @return {boolean}
 */
export function calcHasChangeList(originList = [], changeList = [], idKey = 'questionId', compareKey = 'score') {
  if (!originList && !changeList) {
    return false;
  }

  if (!originList || !changeList) {
    return true;
  }

  if (originList.length !== changeList.length) {
    return true;
  }

  let hasChange = false;

  // 防止题目顺序发生改变，用题目ID匹配
  for (let origin of originList) {
    const hitChange = changeList.find((change) => change[idKey] === origin[idKey]);

    if (!hitChange) {
      hasChange = true;
      break;
    }

    if (hitChange[compareKey] !== origin[compareKey]) {
      hasChange = true;
      break;
    }
  }

  return hasChange;
}

/**
 * 复制JSON数组，解决数组元素对象指针相同问题
 *
 * @param list
 * @return {Array}
 */
export function cloneJsonArray(list = []) {
  if (!list || list.length <= 0) {
    return [];
  }

  let result = [];

  list.forEach((item) => {
    result.push({ ...item });
  });

  return result;
}
