import React from 'react';
import { connect } from 'dva';
import styles from './index.less';
import { Page } from 'COMPONENT';
import StudentInfo from '../../component/StudentInfo';
import ScoreInfo from '../../component/ScoreInfo';
import PaperInfo from '../../component/PaperInfo';
import { getStudentPaperQuestion } from 'SERVICE/entranceExam';
import { formatQuestionList } from '../../component/ScoreInfo/util';
import { formatFileList } from '../../component/PaperInfo/util';
import { formatStudentInfo } from '../../component/StudentInfo/util';

/**
 * 查看分数
 *
 * @param props.examId 考试ID，从父页面传入(考试ID !== 试卷ID)
 * @param props.pageTitle 页面title，从父页面传入
 * @param props.studentId 学生ID，从父页面传入
 */
class ViewPage extends React.Component {
  static defaultProps = {
    examId: null,
    pageTitle: null,
    studentId: null,
  };

  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      studentInfo: null,
      questionList: null,
      fileList: null,
    };
  }

  componentWillMount() {
    this.initContent();
  }

  initContent = () => {
    this.loadPaperQuestion();
  };

  /**
   * 获取学生答卷信息
   */
  loadPaperQuestion = () => {
    const { examId, studentId } = this.props;

    this.toggleLoading();
    getStudentPaperQuestion({ examId, studentId })
      .then((response) => {
        const data = response.data || {};
        const { student, questions, picList } = data;

        const studentInfo = formatStudentInfo(student);
        const questionList = formatQuestionList(questions, false);
        const fileList = formatFileList(picList);

        this.toggleLoading();
        this.setState({ studentInfo, questionList, fileList });
      })
      .catch((e) => {
        this.toggleLoading();
        this.setState({
          studentInfo: null,
          questionList: [],
          fileList: [],
        });
      });
  };

  toggleLoading = () => {
    this.setState((prevState) => {
      return {
        loading: !prevState.loading,
      };
    });
  };

  render() {
    const { studentInfo, questionList, fileList } = this.state;

    return (
      <Page name={this.props.pageTitle} footer={false} loading={this.state.loading} route={this.props.route}>
        <section style={{ marginBottom: 50 }}>
          <h5 className={styles.blockTitle}>学生信息</h5>
          {studentInfo && <StudentInfo student={studentInfo} />}
        </section>

        <section style={{ marginBottom: 30 }}>
          <h5 className={styles.blockTitle}>答题信息</h5>
          {questionList && <ScoreInfo questionList={questionList} editable={false} />}
        </section>

        <section>
          <h5 className={styles.blockTitle}>学生答卷</h5>
          {fileList && <PaperInfo fileList={fileList} editable={false} />}
        </section>
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const pageParams = state.main.pageParams || {};
  const data = pageParams['/EntranceTestReportManage/View'] || {};
  const params = {
    examId: data.examId,
    pageTitle: data.title || '查看分数',
    studentId: data.studentId,
  };

  return {
    ...params,
  };
};

export default connect(mapStateToProps)(ViewPage);
