import React from 'react';
import { connect } from 'dva';
import { Page, SearchFilterForm, SearchFieldForm, Input, Select, List, ListAction } from 'COMPONENT';
import { momentToDate } from 'COMPONENT/utils/form';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import { DatePicker } from 'antd';
import moment from 'moment';
import { errorsHandle, loginUser } from 'UTIL/common';
import { getJxDictByType } from 'UTIL/dict';
import { getPaperReportList, deletePaperReport } from 'SERVICE/entranceExam';
import { dateFormat, monthUnit } from './config';
import { clone } from 'lodash';
import { extraBranchDict } from '../config';

const setWidth = List.createSetWidth();
const { RangePicker } = DatePicker;

class TestReportManage extends React.Component {
  constructor(props) {
    super(props);

    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.initJxDict = this.initJxDict.bind(this);
    this.initSKey = this.initSKey.bind(this);
    this.getListParams = this.getListParams.bind(this);
    this.loadList = this.loadList.bind(this);
    this.search = this.search.bind(this);
    this.updatePage = this.updatePage.bind(this);
    this.changePage = this.changePage.bind(this);
    this.viewPrint = this.viewPrint.bind(this);
    this.viewScore = this.viewScore.bind(this);
    this.invalidTest = this.invalidTest.bind(this);

    this.state = {
      title: '报告管理',
      loading: false,
      searchKey: null,
      defaultSKey: {},
      list: [],
      total: 0,
      pageCurrent: 1,
      pageSize: 10,
      // 教学数据字典
      gradeDict: [],
      subjectDict: [],
    };

    this.columns = [
      { title: '学生姓名', dataIndex: 'name', key: 'name', width: setWidth(4) },
      { title: '分公司', dataIndex: 'branchName', key: 'branchName', width: setWidth(5) },
      { title: '年份', dataIndex: 'year', key: 'year', width: setWidth(2) },
      { title: '季度', dataIndex: 'season', key: 'season', width: setWidth(2) },
      { title: '年级', dataIndex: 'grade', key: 'grade', width: setWidth(3) },
      { title: '科目', dataIndex: 'subject', key: 'subject', width: setWidth(2) },
      { title: '总得分', dataIndex: 'score', key: 'score', width: setWidth(3) },
      { title: '班级层次', dataIndex: 'classTypeName', key: 'classTypeName', width: setWidth(4) },
      {
        title: '创建时间',
        dataIndex: 'createTime',
        key: 'createTime',
        width: setWidth(4),
        render: (text, record) => {
          return text ? momentToDate(moment(new Date(text))) : '';
        },
      },
      {
        title: '操作',
        dataIndex: 'examId',
        key: 'actionPlus',
        width: setWidth(10),
        render: (text, record, index) => {
          let config = [];
          if (props.viewPrintAuth) {
            config.push({
              text: '预览/打印',
              onClick: () => {
                this.viewPrint(record);
              },
            });
          }
          if (props.viewScoreAuth) {
            config.push({
              text: '查看分数',
              onClick: () => {
                this.viewScore(text, record);
              },
            });
          }
          if (props.editScoreAuth) {
            config.push({
              text: '修改分数',
              onClick: () => {
                this.editScore(text, record);
              },
            });
          }
          if (props.invalidScoreAuth) {
            config.push({
              text: '无效',
              onClick: () => {
                this.invalidTest(text, record);
              },
            });
          }
          return <ListAction config={config} />;
        },
      },
    ];
  }

  componentDidMount() {
    this.init();
  }

  async init() {
    this.setState({ loading: true });
    await new Promise(this.initDict);
    await new Promise(this.initJxDict);
    await new Promise(this.initSKey);
    await new Promise(this.loadList);
    this.setState({ loading: false });
  }

  initDict(resolve) {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['branch', 'grade', 'subject'],
        callback() {
          resolve();
        },
      },
    });
  }

  async initJxDict(resolve) {
    const [gradeDict, subjectDict] = await Promise.all([getJxDictByType('GRADE'), getJxDictByType('SUBJECT')]);

    this.setState({ gradeDict, subjectDict });

    resolve();
  }

  initSKey(resolve) {
    const isGroup = loginUser.isGroupRole();
    const today = new Date();
    let newSKey = {
      branchId: isGroup ? (this.props.branchDict || [{ val: null }])[0].val : loginUser.userData.brenchId,
      date: [moment(today).startOf(monthUnit), moment(today).endOf(monthUnit)],
    };
    this.setState(
      {
        searchKey: this.searchKeyAdapter(newSKey),
        defaultSKey: { ...newSKey },
      },
      () => {
        resolve();
      },
    );
  }

  getListParams() {
    let searchKey = {
      pageNum: this.state.pageCurrent,
      pageSize: this.state.pageSize,
    };

    for (let key in this.state.searchKey) {
      searchKey[key] = this.state.searchKey[key];
    }

    return searchKey;
  }

  async loadList(resolve) {
    const apiParams = this.getListParams();
    const rsp = await getPaperReportList(apiParams, true).catch(errorsHandle());
    if (rsp && rsp.resultCode === 0 && rsp.data) {
      this.setState({
        list: rsp.data.list || [],
        total: rsp.data.total,
      });
    }

    resolve();
  }

  search(params) {
    this.setState(
      {
        searchKey: { ...this.searchKeyAdapter(params) },
      },
      () => {
        this.changePage({ current: 1, pageSize: this.state.pageSize });
      },
    );
  }

  searchKeyAdapter = (params) => {
    let sKey = clone(params) || {};
    if (Array.isArray(sKey.date) && sKey.date.length === 2) {
      sKey.startCreateDate = sKey.date[0].format(dateFormat);
      sKey.endCreateDate = sKey.date[1].format(dateFormat);
      delete sKey.date;
    }
    return sKey;
  };

  updatePage(pageCurrent, pageSize) {
    return new Promise((resolve) => {
      this.setState(
        (prevState) => {
          let payload = {};
          if (pageSize && pageSize !== prevState.pageSize) {
            payload.pageCurrent = 1;
            payload.pageSize = pageSize;
          } else {
            payload.pageCurrent = pageCurrent;
          }
          return payload;
        },
        () => {
          resolve();
        },
      );
    });
  }

  async changePage({ current, pageSize }) {
    await this.updatePage(current, pageSize);
    this.setState({ loading: true });
    await new Promise(this.loadList);
    this.setState({ loading: false });
  }

  viewPrint(record) {
    let newPage = window.open('about:blank');
    setTimeout(() => {
      newPage.document.location.replace(record.previewUrl);
    }, 1);
  }

  /**
   * 修改分数
   *
   * @param text 考试ID
   * @param record
   */
  editScore(text, record) {
    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path: `/EntranceTestReportManage/Edit`,
        refresh: true,
        params: {
          examId: text,
          title: record.examName,
          studentId: record.studentId,
          branchId: record.branchId,
          onSaveSuccess: this.onEditScoreSuccess,
        },
      },
    });
  }

  /**
   * 修改分数成功后回调，刷新列表
   */
  onEditScoreSuccess = () => {
    this.props.dispatch({
      type: 'main/openPage',
      payload: { path: `/EntranceTestReportManage` },
    });
    this.search(this.state.searchKey);
  };

  /**
   * 查看分数
   *
   * @param text 考试ID
   * @param record
   */
  viewScore(text, record) {
    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path: `/EntranceTestReportManage/View`,
        refresh: true,
        params: {
          examId: text,
          title: record.examName,
          studentId: record.studentId,
        },
      },
    });
  }

  /**
   * 无效
   *
   * @param text 考试ID
   * @param record
   * @return {Promise<void>}
   */
  invalidTest(text, record) {
    showConfirm('无效操作将会删除该报告，是否继续？', () => {
      const param = { id: text, studentId: record.studentId };

      deletePaperReport(param)
        .then((res) => {
          showMessage('操作成功！', 'success');
          this.changePage({ current: 1, pageSize: this.state.pageSize });
        })
        .catch((e) => {
          showMessage('操作失败，请重试！');
        });
    });
  }

  render() {
    return (
      <Page
        title={this.state.title}
        loading={this.state.loading}
        subHeader={
          <SearchFilterForm onSearch={this.search} truncationNum={3}>
            <SearchFieldForm name="branchId" initialValue={this.state.defaultSKey.branchId}>
              <Select placeholder="分公司" dict={this.props.branchDict} allowClear={false} />
            </SearchFieldForm>
            <SearchFieldForm name="date" initialValue={this.state.defaultSKey.date} span={2}>
              <RangePicker />
            </SearchFieldForm>
            <SearchFieldForm name="name">
              <Input placeholder="学生姓名" />
            </SearchFieldForm>
            <SearchFieldForm name="grade">
              <Select placeholder="年级" dict={this.state.gradeDict} />
            </SearchFieldForm>
            <SearchFieldForm name="subject">
              <Select placeholder="科目" dict={this.state.subjectDict} />
            </SearchFieldForm>
          </SearchFilterForm>
        }
      >
        <List
          columns={this.columns}
          dataSource={this.state.list}
          pagination={{
            total: this.state.total,
            current: this.state.pageCurrent,
            pageSize: this.state.pageSize,
          }}
          onChange={this.changePage}
        />
      </Page>
    );
  }
}

let branchDictStorage = null;

const mapStateToProps = (state) => {
  if (state.dict.branch && !branchDictStorage) {
    branchDictStorage = extraBranchDict.concat(state.dict.branch);
  }
  return {
    branchDict: branchDictStorage,
    gradeDict: state.dict.grade,
    subjectDict: state.dict.subject,
    viewPrintAuth: state.account.btnAuths.indexOf('EETRM_VIEW_PRINT') > -1,
    viewScoreAuth: state.account.btnAuths.indexOf('EETRM_VIEW_SCORE') > -1,
    editScoreAuth: state.account.btnAuths.indexOf('EETRM_EDIT_SCORE') > -1,
    invalidScoreAuth: state.account.btnAuths.indexOf('EETRM_INVALID_EXAM') > -1,
  };
};

export default connect(mapStateToProps)(TestReportManage);
