import React from 'react';
import styles from './index.less';
import UploadImg from 'SRC/BLoComponents/UploadImg';
import { formatApiFileList, cloneJsonArray } from './util';
import { MAX_FILE_LENGTH } from './constant';

const ossFile = { fileId: null, fileKey: null, fileName: null, fileType: null, size: null, src: null };

/**
 * 答卷信息
 *
 * @param props.editable {boolean} 是否允许编，默认true
 * @param props.fileList {array} 图片列表(需组装成OSS格式的图片文件)
 * @param props.onChange {function} 上传文件回调，回调参数为文件列表
 */
class PaperInfo extends React.Component {
  static defaultProps = {
    editable: true,
    fileList: [],
    onChange: (fileList) => {},
  };

  constructor(props) {
    super(props);

    this.state = {
      fileList: [],
    };
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化
   */
  initContent = () => {
    let fileList = cloneJsonArray(this.props.fileList);

    const canPushEmptyFile = this.props.editable && fileList.length < MAX_FILE_LENGTH;
    if (canPushEmptyFile) {
      fileList.push(ossFile);
    }

    this.setState({ fileList });
  };

  /**
   * 上传成功
   *
   * @param file {object} OSS图片文件
   * @param index {number} 文件列表索引
   */
  onUploadComplete = (file, index) => {
    let fileList = [...this.state.fileList];
    fileList[index] = { ...ossFile, ...file };

    const canPushEmptyFile = this.props.editable && fileList.length < MAX_FILE_LENGTH;
    if (canPushEmptyFile) {
      fileList.push(ossFile);
    }

    this.setState({ fileList });
    this.props.onChange(formatApiFileList(fileList));
  };

  /**
   * 删除图片
   *
   * @param fileId {string} 图片ID
   * @param index {number} 文件列表索引
   */
  onDeleteComplete = (fileId, index) => {
    let fileList = [...this.state.fileList];
    fileList.splice(index, 1);

    const hasEmptyFile = fileList.some((file) => file.src === null);
    const canPushEmptyFile = this.props.editable && !hasEmptyFile && fileList.length < MAX_FILE_LENGTH;
    if (canPushEmptyFile) {
      fileList.push(ossFile);
    }

    this.setState({ fileList });
    this.props.onChange(formatApiFileList(fileList));
  };

  render() {
    const fileList = this.state.fileList || [];

    return (
      <ul className={styles.fileWrap}>
        {fileList.map((item, index) => {
          const value = item.src ? item : null;

          return (
            <li key={index}>
              <UploadImg
                value={value}
                text="上传答卷"
                params={{ fileType: 0 }}
                size={160}
                viewOnly={!this.props.editable}
                uploadComplete={(file) => this.onUploadComplete(file, index)}
                deleteComplete={(fileId) => this.onDeleteComplete(fileId, index)}
              />
            </li>
          );
        })}
      </ul>
    );
  }
}

export default PaperInfo;
