/**
 * 格式化保存接口需要的文件列表
 *
 * @param fileList {array} OSS文件列表
 * @return {Array|*[]}
 */
export function formatApiFileList(fileList = []) {
  if (!fileList || fileList.length <= 0) {
    return [];
  }

  return fileList.filter((file) => !!file.src);
}

/**
 * 复制JSON数组，解决数组元素对象指针相同问题
 *
 * @param list
 * @return {Array}
 */
export function cloneJsonArray(list = []) {
  if (!list || list.length <= 0) {
    return [];
  }

  let result = [];
  list.forEach((item) => {
    result.push({ ...item });
  });

  return result;
}

/**
 * 将图片地址数组转为前端组件需要的OSS文件列表格式
 *
 * @param pictureList {<string>[]} ['https://xhboss-test.oss-cn-shenzhen.aliyuncs.com/abe89e95-6e10-4f80-b528-0d6f0c5100d2']
 * @return {{bucket: string, fileName: string, size: number, src: *, showProgress: boolean, fileKey: *, mimeType: string, percent: number, fileType: number, fileId: *, object: *}[]|Array}
 */
export function formatFileList(pictureList = []) {
  if (!pictureList) {
    return [];
  }

  return pictureList.map((src) => {
    const fileKey = src.slice(src.indexOf('/') + 1);

    return {
      bucket: '',
      fileId: fileKey,
      fileKey,
      fileName: '',
      fileType: 0,
      mimeType: 'image/jpeg',
      object: fileKey,
      percent: 100,
      showProgress: false,
      size: 102400,
      src,
    };
  });
}

/**
 * 将前端组件的答卷图片列表转换为API提交时需要的格式
 *
 * @param fileList {array} OSS格式的文件列表 [{fileId, fileKey, fileName, src, ...}]
 * @return {*[]}
 */
export function formatApiPictureList(fileList = []) {
  return fileList.map((file) => {
    return file.src;
  });
}
