/**
 * 求和,小数点保留N位
 *
 * @param arr {array} 求和数组
 * @param key {string} 求和字段
 * @param precision {number} 精度 默认整数
 * @return {(*|number)|number}
 */
export function calcTotal(arr = [], key = '', precision = 1) {
  if (!arr || arr.length <= 0 || !key) {
    return 0;
  }

  const total = arr.reduce((total, cur, index) => {
    return (total += Number(cur[key]) || 0);
  }, 0);

  return Math.floor(total * precision) / precision;
}

/**
 * 复制JSON数组，解决数组元素对象指针相同问题
 *
 * @param list
 * @return {Array}
 */
export function cloneJsonArray(list = []) {
  if (!list || list.length <= 0) {
    return [];
  }

  let result = [];
  list.forEach((item) => {
    result.push({ ...item });
  });

  return result;
}

/**
 * 将接口返回的题目信息转换为前端格式(主要将`score`转为`totalScore`字段)
 *
 * @param questionList {array} [{sort, questionId|id, titleName: '题目title', score: '题目总分数', stuScore: '学生得分数'}]
 * @param isInit {boolean} 是否为初始化 {true: 默认得分为满分, false: 得分为学生得分} 默认true
 * @return {{score: (*|number), questionId, questionTitle, totalScore: (*|number)}[]}
 */
export function formatQuestionList(questionList = [], isInit = true) {
  if (!questionList) {
    return [];
  }

  return questionList.map((question) => {
    const { sort, titleName } = question;
    const questionId = question.id || question.questionId || null;
    const totalScore = question.score || 0;
    const stuScore = isInit ? totalScore : question.stuScore || 0;

    return {
      questionId, // 题目ID
      sort, // 题目排序(非必须)
      questionTitle: titleName, // 题目名称
      score: stuScore, // (学生)得分数
      totalScore: totalScore, // 题目总分
    };
  });
}

/**
 * 将前端组件的题目列表转换为API提交时需要的格式
 *
 * @param questionList {array} 前端组件的题目列表 [{questionId, sort, questionTitle, score: '(学生)得分数', totalScore}]
 * @return {{score, questionId, titleName, sort}[]}
 */
export function formatApiQuestionList(questionList = []) {
  return questionList.map((question) => {
    const { questionId, sort, questionTitle, score } = question;

    return {
      questionId, // 题目ID
      sort, // 题目排序
      titleName: questionTitle, // 题目名称
      score, // (学生)得分数
    };
  });
}
