import React from 'react';
import moment from 'moment';
import styles from './index.less';
import { Modal, List, ListAction } from 'COMPONENT';
import { momentToDate } from 'COMPONENT/utils/form';
import { getPaperReportByStudentId } from 'SERVICE/entranceExam';

const setWidth = List.createSetWidth();

/**
 * 学生入学测报告弹窗
 *
 * @param props.studentId {string} 学生ID
 * @param props.onClose {function?} 关闭弹窗回调
 */
class StudentReport extends React.Component {
  static defaultProps = {
    studentId: null,
    onClose: () => {},
  };

  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      list: [],
    };

    this.columns = [
      {
        title: '序号',
        dataIndex: 'index',
        key: 'index',
        width: 20,
        render: (text, record, index) => {
          return index + 1;
        },
      },
      { title: '考试名', dataIndex: 'examName', key: 'examName', width: setWidth(5), lines: 2 },
      { title: '试卷名', dataIndex: 'paperName', key: 'paperName', width: setWidth(5), lines: 2 },
      {
        title: '创建时间',
        dataIndex: 'createTime',
        key: 'createTime',
        width: setWidth(3),
        render: (text, record) => {
          return text ? momentToDate(moment(new Date(text))) : '';
        },
      },
      {
        title: '操作',
        dataIndex: 'examId',
        key: 'action',
        width: setWidth(2),
        render: (text, record) => {
          const config = [
            {
              text: '预览/打印',
              onClick: () => {
                this.viewPrint(record);
              },
            },
          ];

          return <ListAction config={config} />;
        },
      },
    ];
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化数据
   */
  initContent = () => {
    const { studentId } = this.props;

    if (!studentId) {
      return;
    }

    this.toggleLoading();
    getPaperReportByStudentId({ studentId })
      .then((response) => {
        const list = (response && response.data) || [];

        this.toggleLoading();
        this.setState({ list });
      })
      .catch((e) => {
        this.toggleLoading();
      });
  };

  /**
   * 预览
   *
   * @param record
   */
  viewPrint(record) {
    let newPage = window.open('about:blank');
    setTimeout(() => {
      newPage.document.location.replace(record.previewUrl);
    }, 1);
  }

  toggleLoading = () => {
    this.setState((prevState) => {
      return {
        loading: !prevState.loading,
      };
    });
  };

  render() {
    return (
      <Modal
        className={styles.studentReportModal}
        width={640}
        title="入学测报告"
        cancelBtnShow={false}
        loading={this.state.loading}
        onOk={this.props.onClose}
        onClose={this.props.onClose}
      >
        <List columns={this.columns} dataSource={this.state.list} />
      </Modal>
    );
  }
}

export default StudentReport;
