import React from 'react';
import styles from './styles.less';
import Loading from 'COMPONENT/Loading';
import dicts from '../dicts';
import moment from 'moment';
import { Button, DatePicker, List, SearchBar, Input, Select, TextLink, Popover } from 'COMPONENT';
import { loadCommonDict, loadCommonDictNew, loadBranchDict, loadCampusDict, dictAddOptionTitle } from 'UTIL/dict';
import { Modal as AntdModal } from 'antd';
import loaders from '../loaders';
import { connect } from 'dva';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import TransferCampusModal from '../TransferCampusModal';
import RegisterToDoorModal from '../RegisterToDoorModal';
import DeductionCouponInfoModal from '../../../preferential/modals/DeductionCouponInfoModal';
import { loadConsultantDict } from 'PAGE/common/loaders';
import { xinghuoInstitution } from 'CONST/config';
import withTeachingReporterHOC from 'PAGE/teachingReporterHOC/teachingReporterHOC';
import PhoneCall from '../../components/PhoneCall';
import ModalPhoneCallFollow from '../../ModalPhoneCallFollow';
import { checkAgentBeforeCall } from 'UTIL/callCenter';
import { CALL_LOGIN_STATUS_ABNORMAL } from 'CONST/callConfig';
import classNames from 'classnames';
import { loginUser } from 'SRC/utils/common';
import { loadSourceDetailDict } from 'PAGE/common/loaders';

let TeachingReporter = null;

const { COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;
const couponUnusedDict = [
  { val: 'false', text: '请筛选抵扣券学生' },
  { val: 'true', text: '存在未使用抵扣券' },
];

function filterList(list) {
  return list.filter(
    (ele) => !ele.isSigned && !(ele.unFetch && (ele.entrance === 'ON_LINE' || ele.entrance === 'CALL_IN')),
  );
}

class CusList extends React.Component {
  constructor(props) {
    super(props);
    this.initTodoSKey = this.initTodoSKey.bind(this);
    this.onResolveTodo = this.onResolveTodo.bind(this);
    this.getSearchParams = this.getSearchParams.bind(this);
    this.updateObtain = this.updateObtain.bind(this);
    this.phoneCall = this.phoneCall.bind(this);
    this.checkAgentLoginStatus = this.checkAgentLoginStatus.bind(this);
    this.switchModalPhoneCallFollow = this.switchModalPhoneCallFollow.bind(this);
    this.showEmployeeTip = this.showEmployeeTip.bind(this);

    const { branchId, campusId, consultantId, isManager } = this.props;
    this.initListSearchKeys = {
      stuName: null,
      name: null,
      phone: null,
      school: null,
      grade: null,
      entrance: null,
      isVisit: null,
      intention: null,
      cusTag: null,
      couponUnused: 'false',
      newFollowTime: null,
      nextFollowTime: null,
      appointmentTime: null,
      newFollowTimeEnd: null,
      nextFollowTimeEnd: null,
      appointmentTimeEnd: null,
      branchId,
      campusId,
      consultantId,
      visitComeDateStart: null,
      visitComeDateEnd: null,
      createDateStart: null,
      createDateEnd: null,
      remindType: null,
      source: null,
      sourceDetail: null,
    };

    this.state = Object.assign(
      {
        studentId: null,

        loading: false, // {boolean} 内容加载状态
        searchExpand: false, // {boolean} 搜索栏展开状态

        couponUnused: 'false',
        stuName: null, // {string} 查询条件 学生姓名
        name: null, // {string} 查询条件 客户姓名
        phone: null, // {string} 查询条件 电话
        school: null, // {string} 查询条件 学校
        grade: null, // {string} 查询条件 年级
        entrance: null, // {string} 查询条件 资源入口
        isVisit: null, // {string} 查询条件 是否上门
        intention: null, // {string} 查询条件 意向度
        cusTag: null, // {string} 查询条件 客户标签
        newFollowTime: null, // {string} 查询条件 最新跟进时间（开始）
        nextFollowTime: null, // {string} 查询条件 下次跟进时间（开始）
        appointmentTime: null, // {string} 查询条件 预约上门时间（开始）
        newFollowTimeEnd: null, // {string} 查询条件 最新跟进时间（结束）
        nextFollowTimeEnd: null, // {string} 查询条件 下次跟进时间（结束）
        appointmentTimeEnd: null, // {string} 查询条件 预约上门时间（结束）
        source: null, // {string} 查询条件 资源来源
        sourceDetail: null, // {string} 查询条件  资源细分
        registerToDoRecords: [],

        visitComeDateStart: null,
        visitComeDateEnd: null,
        createDateStart: null,
        createDateEnd: null,
        remindType: null,

        gradeDict: [], // {[object]} 字典 年级
        entranceDict: [], // {[object]} 字典 资源入口
        isVisitDict: dicts.isVisitDict, // {[object]} 字典 是否上门
        intentionDict: [], // {[object]} 字典 意向度
        cusTagDict: [], // {[object]} 字典 客户标签

        list: [], // {Array} 列表数据，
        filteredList: [],
        /*
       { id, studentName, grade, schoolName, name, phone, entrance, isVisit, intention, cusTag,
       newFollowTime, nextFollowTime, appointmentTime, unFetch, isNew, isSigned, countDay }
      */
        listSearchKeys: this.initListSearchKeys, // {object} 列表数据的查询关键字
        totalRecords: 0, // {number} 总记录数
        currentPage: 1, // {number} 当前页码
        pageSize: 20, // {number} 每页记录数
        pageOrder: 'DESC', // {string} 页面排序
        pageOrderField: 'create_time', // {string} 排序字段

        transferCampusModalVis: false, // {boolean} 转校弹框显示状态
        registerToDoorModalVis: false, // 登记上门

        branchId: '',
        brenchDict: [],
        branchSelectorInnerDisabled: false,
        campusId: '',
        blCampusDict: [],
        campusSelectorInnerDisabled: false,
        consultantId: '',
        consultantDict: [],
        consultantSelectorInnerDisabled: false,
        sourceDict: [],
        sourceDetailDict: [],
        studentStatusDict: {},

        currentRecord: null,
        isReporter: false,

        modalPhoneCallFollowVisible: false,

        deductionCouponInfoModalVisible: false,
      },
      this.props.defaultSearchConditions ? this.props.defaultSearchConditions : {},
    );

    this.phoneCallFollowData = null;
    this.phoneCallFollowList = null;
  }

  componentDidMount() {
    this.initTodoSKey();
    GlobalEvent.on('CrmNotification:resolveFollowCustomer', this.onResolveTodo);
  }

  componentWillUnmount() {
    GlobalEvent.off('CrmNotification:resolveFollowCustomer', this.onResolveTodo);
    window.__refreshParentEntrance = null;
  }

  componentWillReceiveProps(nextProps) {
    const oldDefaultSearchConditions = this.props.defaultSearchConditions || {};
    const newDefaultSearchConditions = nextProps.defaultSearchConditions || {};
    // 不能直接比较两个对象，因此补充一个判断条件
    const hadDefaultSearchConditions =
      Object.keys(oldDefaultSearchConditions).length > 0 || Object.keys(newDefaultSearchConditions).length > 0;

    if (hadDefaultSearchConditions && oldDefaultSearchConditions !== newDefaultSearchConditions) {
      this.reset();
      Promise.resolve(this.setState(newDefaultSearchConditions));
      setTimeout(() => {
        this.search();
      }, 50);
    }
  }

  initTodoSKey() {
    const { isTodo, todoParams } = this.props.params || {};
    // 从系统通知跳转过来的，初始化筛选项值
    if (isTodo) {
      this.setState({ ...todoParams });
    }
  }

  onResolveTodo(todoParams) {
    this.setState(
      {
        ...this.initListSearchKeys,
        ...todoParams,
      },
      () => {
        this.search();
      },
    );
  }

  async initialBranch() {
    this.setState({
      branchSelectorInnerDisabled: true,
      campusSelectorInnerDisabled: true,
      consultantSelectorInnerDisabled: true,
    });
    let { branchId, campusId, showBranchSelector, showCampusSelector, showConsultantSelector } = this.props;
    if (this.props.defaultSearchConditions) {
      branchId = this.props.defaultSearchConditions.branchId || branchId;
      campusId = this.props.defaultSearchConditions.campusId || campusId;
    }
    const brenchDict = !showBranchSelector ? [] : await loadBranchDict();
    if (!branchId) {
      branchId = brenchDict[0] ? brenchDict[0]['val'] : '';
    }
    const blCampusDict = !showCampusSelector || !branchId ? [] : await loadCampusDict(branchId);
    const consultantDict = !showConsultantSelector || !campusId ? [] : await loadConsultantDict({ orgId: campusId });
    this.initListSearchKeys['branchId'] = branchId;
    this.initListSearchKeys['campusId'] = campusId;
    this.setState(
      Object.assign(
        {
          branchId,
          campusId,
          listSearchKeys: this.initListSearchKeys,
          brenchDict,
          blCampusDict,
          consultantDict,
          branchSelectorInnerDisabled: false,
          campusSelectorInnerDisabled: false,
          consultantSelectorInnerDisabled: false,
        },
        this.props.defaultSearchConditions ? this.props.defaultSearchConditions : {},
      ),
    );
  }

  loadCusList = (params) => {
    const { isManager } = this.props;
    return isManager ? loaders.loadCusListManage(params) : loaders.loadCusList(params);
  };

  initContent = async () => {
    await this.initialBranch();
    const [listData, gradeDict, entranceDict, intentionDict, cusTagDict, sourceDict, studentStatusList = []] =
      await Promise.all([
        this.loadCusList(
          Object.assign(
            this.getListParams(),
            this.getSearchParams(),
            this.props.defaultSearchConditions ? this.props.defaultSearchConditions : {},
          ),
        ),
        loadCommonDictNew('STUDENT_GRADE'),
        loadCommonDict('RES_ENTRANCE'),
        loadCommonDict('CUSTOMER_SATISFICING'),
        loadCommonDict('CustomerSignType'),
        loadCommonDict('RES_TYPE'),
        loadCommonDict('StudentStatus'),
      ]);
    const { list, totalRecords } = listData;
    const filteredList = filterList(list);
    this.setState({
      sourceDict,
      gradeDict,
      entranceDict,
      intentionDict,
      cusTagDict,
      list,
      totalRecords,
      filteredList,
      studentStatusDict: Object.fromEntries(studentStatusList.map((s) => [s.val, s.text])),
    });
  };

  getListParams = () => {
    const {
      listSearchKeys: {
        stuName,
        name,
        phone,
        school,
        grade,
        entrance,
        isVisit,
        intention,
        cusTag,
        newFollowTime,
        nextFollowTime,
        appointmentTime,
        newFollowTimeEnd,
        nextFollowTimeEnd,
        appointmentTimeEnd,
        branchId,
        campusId,
        consultantId,
        visitComeDateStart,
        visitComeDateEnd,
        createDateStart,
        createDateEnd,
      },
      currentPage,
      pageSize,
      pageOrder,
      pageOrderField,
    } = this.state;
    return {
      stuName,
      name,
      phone,
      school,
      grade,
      entrance,
      isVisit,
      intention,
      cusTag,
      newFollowTime,
      nextFollowTime,
      appointmentTime,
      newFollowTimeEnd,
      nextFollowTimeEnd,
      appointmentTimeEnd,
      branchId,
      campusId,
      consultantId,
      visitComeDateStart,
      visitComeDateEnd,
      createDateStart,
      createDateEnd,
      currentPage,
      pageSize,
      type: this.props.type,
      pageOrder,
      pageOrderField,
    };
  };

  getSearchParams() {
    let params = {
      stuName: this.state.stuName,
      name: this.state.name,
      phone: this.state.phone,
      school: this.state.school,
      grade: this.state.grade,
      entrance: this.state.entrance,
      isVisit: this.state.isVisit,
      intention: this.state.intention,
      cusTag: this.state.cusTag,
      newFollowTime: this.state.newFollowTime,
      nextFollowTime: this.state.nextFollowTime,
      appointmentTime: this.state.appointmentTime,
      newFollowTimeEnd: this.state.newFollowTimeEnd,
      nextFollowTimeEnd: this.state.nextFollowTimeEnd,
      appointmentTimeEnd: this.state.appointmentTimeEnd,
      branchId: this.state.branchId,
      campusId: this.state.campusId,
      consultantId: this.state.consultantId,
      visitComeDateStart: this.state.visitComeDateStart,
      visitComeDateEnd: this.state.visitComeDateEnd,
      createDateStart: this.state.createDateStart,
      createDateEnd: this.state.createDateEnd,
      source: this.state.source,
      sourceDetail: this.state.sourceDetail,
    };

    params.couponUnused = this.state.couponUnused === 'true'; // 是否存在未使用抵扣券
    if (this.state.resEntranceType) {
      params.resEntranceType = this.state.resEntranceType;
      this.setState({ resEntranceType: null });
    }
    if (this.state.remindType) {
      params.remindType = this.state.remindType;
      this.setState({ remindType: null });
    }

    return params;
  }

  search = () => {
    this.setState({ loading: true });
    let listSearchKeys = this.getSearchParams();
    this.loadCusList({
      ...this.getListParams(),
      ...listSearchKeys,
      currentPage: 1,
    }).then(({ list, totalRecords, page }) => {
      const filteredList = filterList(list);
      this.setState({
        loading: false,
        listSearchKeys,
        currentPage: 1,
        list,
        totalRecords,
        filteredList,
      });
    });
  };

  refreshCurrentPage = async () => {
    const {
      stuName,
      name,
      phone,
      school,
      grade,
      entrance,
      isVisit,
      intention,
      cusTag,
      newFollowTime,
      nextFollowTime,
      appointmentTime,
      newFollowTimeEnd,
      nextFollowTimeEnd,
      appointmentTimeEnd,
      branchId,
      campusId,
      consultantId,
      visitComeDateStart,
      visitComeDateEnd,
      createDateStart,
      createDateEnd,
    } = this.state;
    let { currentPage, pageSize } = this.state;
    const listSearchKeys = {
      stuName,
      name,
      phone,
      school,
      grade,
      entrance,
      isVisit,
      intention,
      cusTag,
      newFollowTime,
      nextFollowTime,
      appointmentTime,
      newFollowTimeEnd,
      nextFollowTimeEnd,
      appointmentTimeEnd,
      branchId,
      campusId,
      consultantId,
      visitComeDateStart,
      visitComeDateEnd,
      createDateStart,
      createDateEnd,
    };
    this.setState({ loading: true });
    const { list, totalRecords, page } = await this.loadCusList({
      ...this.getListParams(),
      ...listSearchKeys,
    });
    const filteredList = filterList(list);
    if (!filteredList.length) {
      currentPage = Math.ceil((totalRecords || 0) / pageSize) || 1;
      const { list, totalRecords, page } = await this.loadCusList({
        ...this.getListParams(),
        ...listSearchKeys,
        currentPage,
        pageSize,
      });
      const filteredList = filterList(list);
      this.setState({
        loading: false,
        listSearchKeys,
        currentPage,
        pageSize,
        list,
        totalRecords,
        filteredList,
      });
      return;
    }
    this.setState({
      loading: false,
      listSearchKeys,
      currentPage,
      pageSize,
      list,
      totalRecords,
      filteredList,
    });
  };

  reset = () => {
    this.setState(this.initListSearchKeys);
  };

  onPaginationChange = ({ current, pageSize }, filters, sorter) => {
    this.setState({ loading: true });
    const changed = {
      currentPage: this.state.pageSize !== pageSize ? 1 : current,
      pageSize,
    };
    const pageOrderField = sorter.field ? sorter.field : this.state.pageOrderField;
    const pageOrder = sorter.field ? (sorter.order === 'ascend' ? 'ASC' : 'DESC') : this.state.pageOrder;
    this.loadCusList({
      ...this.getListParams(),
      ...changed,
      pageOrder,
      pageOrderField,
    }).then(({ list, totalRecords }) => {
      const filteredList = filterList(list);
      this.setState({
        loading: false,
        ...changed,
        list,
        totalRecords,
        filteredList,
        pageOrder,
        pageOrderField,
      });
    });
  };

  sourceChange = (val) => {
    if (val) {
      this.setState({ loading: true });
      loadSourceDetailDict(val).then(
        (dict) => {
          this.setState({
            loading: false,
            source: val,
            sourceDetail: null,
            sourceDetailDict: dict,
          });
        },
        () => {
          this.setState({ loading: false });
        },
      );
    } else {
      this.setState({ source: null, sourceDetail: null, sourceDetailDict: [] });
    }
  };

  onRefresh = (params) => {
    this.search();
    this.goDetail(params);
  };

  goDetail = (params, record) => {
    const { dispatch } = this.props;

    params.type = this.props.type;
    params.onRefresh = this.onRefresh;

    if (record) {
      params.customerId = record.customerId;
      params.studentId = record.studentId;
      params.studentName = record.studentName;
      params.phoneNum = record.phoneNum;
      params.gradeName = record.gradeName;
      params.schoolName = record.schoolName;
      params.resEntranceName = record.resEntranceName;
      params.customerName = record.customerName;
    }

    dispatch({
      type: 'main/openPage',
      payload: {
        path: '/CommonCusDetailPage',
        params,
        refresh: true,
        mirrorId: `ConsultantFollowCustomer/CusListSec/${this.props.type}/${params.id}/${params.stuId}/${
          params.detailEditable ? 'detail' : 'follow'
        }`,
      },
    });
  };

  // 登记上门
  goRegisterToDoor = async (record) => {
    const res = await loaders.loadFollowingRecordNew({
      rows: 1,
      followStuId: record.studentId,
      createUserId: this.props.userId,
      appointmentType: 'APPOINTMENT',
      meetingTimeEnd: moment().format('YYYY-MM-DD'),
      meetingTimeStart: moment().format('YYYY-MM-DD'),
    });
    this.setState({
      currentRecord: record,
      registerToDoorModalVis: true,
      registerToDoRecords: res?.rows || [],
    });
  };

  signNew = async ({ id, stuId, studentName, grade, school, schoolName, deliverTarget, blCampusId, blCampusName }) => {
    // 非学习中心的，将其id值清空
    if (!/学习中心/.test(blCampusName) && loginUser.isXingHuoSym()) {
      blCampusId = null;
    }

    const res = await loaders.canSign({
      cusId: id,
      studentId: stuId,
      deliverTargetId: deliverTarget,
    });

    if (!res.data) {
      showMessage('该学生不能签单', 'error');
      return;
    }

    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path: `/signContractPro/${JSON.stringify({
          studentId: stuId,
          customerId: id,
          flag: true,
          signEntrance: 'CONSULTANT',
        })}`,
        refresh: true,
      },
    });
  };

  obtain = (id, stuId, index) => {
    this.setState({ loading: true });
    loaders.obtainCustomer({ id, stuId }).then(
      () => {
        showMessage('获取成功', 'success');
        this.updateObtain(index);
      },
      (e) => {
        showMessage(e.text || '获取失败');
        this.setState({ loading: false });
      },
    );
  };

  updateObtain(index) {
    let newList = this.state.list;
    newList[index].unFetch = false;
    this.setState({
      list: newList,
      loading: false,
    });
  }

  getDisabledNewFollowTimeStartDate = (startDate) => {
    const { newFollowTimeEnd: endDate } = this.state;

    // if (!startDate || !endDate) {
    //   return moment(startDate, "YYYY-MM-DD").day() !== 1;
    // }

    // return (
    //   moment(startDate, "YYYY-MM-DD").day() !== 1 ||
    //   moment(startDate).valueOf() > moment(endDate).valueOf()
    // );
    if (!startDate || !endDate) {
      return false;
    }

    return moment(startDate).valueOf() > moment(endDate).valueOf();
  };

  getDisabledNewFollowTimeEndDate = (endDate) => {
    const { newFollowTime: startDate } = this.state;

    // if (!endDate || !startDate) {
    //   return moment(endDate, "YYYY-MM-DD").day() !== 0
    // }

    // return (moment(endDate).valueOf() < moment(startDate).valueOf()) || moment(endDate, "YYYY-MM-DD").day() !== 0

    if (!endDate || !startDate) {
      return false;
    }

    return moment(endDate).valueOf() < moment(startDate).valueOf();
  };

  getDisabledNextFollowTimeStartDate = (startDate) => {
    const { nextFollowTimeEnd: endDate } = this.state;

    if (!startDate || !endDate) {
      return false;
    }

    return moment(startDate).valueOf() > moment(endDate).valueOf();
  };

  getDisabledNextFollowTimeEndDate = (endDate) => {
    const { nextFollowTime: startDate } = this.state;

    if (!endDate || !startDate) {
      return false;
    }

    return moment(endDate).valueOf() < moment(startDate).valueOf();
  };

  getDisabledAppointmentTimeStartDate = (startDate) => {
    const { appointmentTimeEnd: endDate } = this.state;

    if (!startDate || !endDate) {
      return false;
    }

    return moment(startDate).valueOf() > moment(endDate).valueOf();
  };

  getDisabledAppointmentTimeEndDate = (endDate) => {
    const { appointmentTime: startDate } = this.state;

    if (!endDate || !startDate) {
      return false;
    }

    return moment(endDate).valueOf() < moment(startDate).valueOf();
  };

  getDisabledVisitComeDateStartDate = (startDate) => {
    const { visitComeDateEnd: endDate } = this.state;

    if (!startDate || !endDate) {
      return false;
    }

    return moment(startDate).valueOf() > moment(endDate).valueOf();
  };

  getDisabledVisitComeDateEndDate = (endDate) => {
    const { visitComeDateStart: startDate } = this.state;

    if (!endDate || !startDate) {
      return false;
    }

    return moment(endDate).valueOf() < moment(startDate).valueOf();
  };

  getDisabledCreateDateStartDate = (startDate) => {
    const { createDateEnd: endDate } = this.state;

    if (!startDate || !endDate) {
      return false;
    }

    return moment(startDate).valueOf() > moment(endDate).valueOf();
  };

  getDisabledCreateDateEndDate = (endDate) => {
    const { createDateStart: startDate } = this.state;

    if (!endDate || !startDate) {
      return false;
    }

    return moment(endDate).valueOf() < moment(startDate).valueOf();
  };

  confirmLost = ({ id, stuId }) => {
    loaders.confirmLostCus({ id, stuId }).then(
      () => {
        showMessage('操作成功', 'success');
        this.search();
      },
      (e) => {
        showMessage(e.text || '操作失败');
        this.setState({ loading: false });
      },
    );
  };

  onBranchChange = async (branchId) => {
    const { showCampusSelector } = this.props;
    this.setState({
      campusSelectorInnerDisabled: true,
      consultantSelectorInnerDisabled: true,
    });
    const blCampusDict = !showCampusSelector || !branchId ? [] : await loadCampusDict(branchId);
    const consultantDict = [];
    this.setState({
      blCampusDict,
      consultantDict,
      campusSelectorInnerDisabled: false,
      consultantSelectorInnerDisabled: false,
    });
  };

  onCampusChange = async (campusId) => {
    const { showConsultantSelector } = this.props;
    this.setState({
      consultantSelectorInnerDisabled: true,
    });
    const consultantDict = !showConsultantSelector || !campusId ? [] : await loadConsultantDict({ orgId: campusId });
    this.setState({
      consultantDict,
      consultantSelectorInnerDisabled: false,
    });
  };

  // 关闭报告Modal
  closeReporterModal = () => {
    this.setState({ isReporter: false });
  };

  async phoneCall(record, index) {
    this.setState({ loading: true });
    const agentLoginStatus = await new Promise(this.checkAgentLoginStatus);
    this.setState({ loading: false });
    if (checkAgentBeforeCall(agentLoginStatus)) {
      this.phoneCallFollowList = this.state.list.slice(index);
      this.switchModalPhoneCallFollow();
    }
  }

  checkAgentLoginStatus(resolve) {
    this.props.dispatch({
      type: 'callCenter/fetchToUpdAgentInfo',
      payload: {
        callback(agentObj) {
          resolve((agentObj || {}).agentLoginStatus || CALL_LOGIN_STATUS_ABNORMAL);
        },
      },
    });
  }

  switchModalPhoneCallFollow() {
    this.setState((prevState) => {
      if (prevState.modalPhoneCallFollowVisible) {
        this.phoneCallFollowList = null;
      }
      return {
        modalPhoneCallFollowVisible: !prevState.modalPhoneCallFollowVisible,
      };
    });
  }

  showEmployeeTip = () => {
    const tip =
      '该学生的客户号码属于员工号码，根据集团要求，内部员工号码不允许签单，请使用非员工号码录入资源然后签单。';
    AntdModal.info({
      title: tip,
      content: '',
      className: 'homAntdModal',
    });
  };

  handleViewDeductionCouponInfo = (studentId) => {
    this.setState({
      studentId,
      deductionCouponInfoModalVisible: true,
    });
  };

  handleCloseDeductionCouponInfoModal = () => {
    this.setState({
      deductionCouponInfoModalVisible: false,
    });
  };

  generateDeductionCouponInfoModal() {
    const { studentId, deductionCouponInfoModalVisible } = this.state;
    const props = {
      studentId,
      visible: deductionCouponInfoModalVisible,
      onClose: this.handleCloseDeductionCouponInfoModal,
    };
    return props.visible && <DeductionCouponInfoModal {...props} />;
  }

  render() {
    const {
      showBranchSelector,
      branchSelectorDisabled,
      showCampusSelector,
      campusSelectorDisabled,
      showConsultantSelector,
      consultantSelectorDisabled,
      btnAuths,
    } = this.props;
    const hasConsultWorkbrench = this.props.callCenterConfig.hasConsultWorkbrench;
    const alreadyBind = this.props.alreadyBind;
    const {
      loading,
      searchExpand,
      name,
      stuName,
      phone,
      school,
      grade,
      entrance,
      isVisit,
      intention,
      cusTag,
      couponUnused,
      newFollowTime,
      nextFollowTime,
      appointmentTime,
      newFollowTimeEnd,
      nextFollowTimeEnd,
      appointmentTimeEnd,
      gradeDict,
      entranceDict,
      isVisitDict,
      intentionDict,
      cusTagDict,
      studentStatusDict,
      list,
      totalRecords,
      currentPage,
      pageSize,
      transferCampusModalVis,
      registerToDoorModalVis,
      currentRecord,
      visitComeDateStart,
      visitComeDateEnd,
      createDateStart,
      createDateEnd,
      branchId,
      brenchDict,
      campusId,
      blCampusDict,
      consultantId,
      source,
      sourceDetail,
      consultantDict,
      branchSelectorInnerDisabled,
      campusSelectorInnerDisabled,
      consultantSelectorInnerDisabled,
      filteredList,
      isReporter,
      sourceDict,
      sourceDetailDict,
    } = this.state;

    let columns = [
      {
        title: '学生姓名',
        dataIndex: 'studentName',
        key: 'studentName',
        width: COL_WIDTH_L,
        render: (text, record) => {
          const { isNew, isSigned, employee } = record;
          let flag = <div>&nbsp;</div>;

          if (isNew) {
            flag = <div className={styles.redDot} />;
          } else if (isSigned) {
            flag = <div className={styles.tag}>签</div>;
          }

          return (
            <div className={styles.stuNameWrap}>
              <div className={styles.flag}>{flag}</div>
              {text}
              {employee && (
                <div>
                  <span className={'employeeFlag'}>内部员工</span>
                </div>
              )}
            </div>
          );
        },
      },
      {
        title: '年级',
        dataIndex: 'gradeName',
        key: 'gradeName',
        width: COL_WIDTH_M,
      },
      {
        title: '学校级别',
        dataIndex: 'schoolLevelName',
        key: 'schoolLevelName',
        width: COL_WIDTH_M,
      },
      {
        title: '学生状态',
        dataIndex: 'status',
        key: 'status',
        width: COL_WIDTH_M,
        render: (status) => {
          return studentStatusDict[status] || '新生';
        },
      },
      {
        title: '资源入口',
        dataIndex: 'entrance',
        key: 'entrance',
        dict: entranceDict,
        width: 80,
      },
      {
        title: '最新跟进记录',
        dataIndex: 'lastRemark',
        key: 'lastRemark',
        width: 260,
        lines: 2,
      },
      {
        title: '最后跟进时间',
        dataIndex: 'newFollowTime',
        key: 'newFollowTime',
        width: 120,
        render: (text) => {
          return text ? text.slice(0, 10) : text;
        },
      },
      {
        title: '是否上门',
        dataIndex: 'isVisit',
        key: 'isVisit',
        dict: isVisitDict,
        width: 60,
      },
      {
        title: '上门日期',
        dataIndex: 'visitComeDate',
        key: 'visitComeDate',
        width: 100,
      },
      {
        title: '意向度',
        dataIndex: 'intention',
        key: 'intention',
        dict: intentionDict,
        width: 80,
      },
      {
        title: '客户意向产品',
        dataIndex: 'cusTag',
        key: 'cusTag',
        dict: cusTagDict,
        width: 80,
      },
      {
        title: '学生未跟进天数',
        dataIndex: 'countDay',
        key: 'countDay',
        width: 90,
        sorter: true,
      },
      {
        title: '咨询未跟进天数',
        dataIndex: 'noopDay',
        key: 'noopDay',
        width: 90,
        sorter: true,
      },
      {
        title: '创建日期',
        dataIndex: 'createDate',
        key: 'createDate',
        width: 120,
        sorter: true,
      },
      {
        title: '操作',
        width: COL_WIDTH_XL,
        key: 'action',
        fixed: 'right',
        render: (text, record, index) => {
          const {
            id,
            unFetch,
            stuId,
            waitingTime,
            isVisit,
            entrance,
            isSigned,
            campusId,
            deliverTarget,
            employee,
            couponUnused,
          } = record;
          const menus = [
            {
              text: '详情',
              onClick: () => this.goDetail({ id, stuId, detailEditable: true, deliverTarget }, record),
            },
            {
              text: '转校申请',
              onClick: () =>
                this.setState({
                  transferCampusModalVis: true,
                  currentRecord: record,
                }),
            },
          ];

          if (this.props.isManager) {
            return (
              <span>
                <TextLink
                  className="list-btn"
                  onClick={() =>
                    this.goDetail(
                      {
                        id,
                        stuId,
                        detailEditable: false,
                        followEditable: false,
                        isManager: true,
                        campusId,
                        deliverTarget,
                      },
                      record,
                    )
                  }
                >
                  查看详情
                </TextLink>
              </span>
            );
          }

          if (isSigned) {
            return (
              <Popover content="该客户已与其他咨询师签约，点击后进入流失列表" icon="info" maxWidth={180}>
                <TextLink className="list-btn" onClick={() => this.confirmLost(record)}>
                  已被签约
                </TextLink>
              </Popover>
            );
          }

          if (unFetch && (entrance === 'ON_LINE' || entrance === 'CALL_IN')) {
            return (
              <span className={styles.fetchWrap}>
                <Button type="primary" size="small" onClick={() => this.obtain(id, stuId, index)}>
                  获取
                </Button>
              </span>
            );
          }

          return (
            <span>
              <TextLink
                className={styles.textLink}
                onClick={() =>
                  this.goDetail(
                    {
                      id,
                      stuId,
                      detailEditable: true,
                      followEditable: true,
                      deliverTarget,
                      list: filteredList,
                    },
                    record,
                  )
                }
              >
                跟进
              </TextLink>
              {
                /** 已签约学生则不展示该功能入口 */
                !isSigned && (
                  <TextLink className={styles.textLink} onClick={() => this.goRegisterToDoor(record)}>
                    登记上门
                  </TextLink>
                )
              }
              {deliverTarget && (
                <TextLink
                  className={styles.textLink}
                  onClick={() =>
                    this.setState({
                      transferCampusModalVis: true,
                      currentRecord: record,
                    })
                  }
                >
                  转校申请
                </TextLink>
              )}
              {isVisit === dicts.IS_VISIT_YES && btnAuths.includes('FOLLOW_CUSTOMER_ADDCONTRACT') && (
                <TextLink
                  className={classNames({
                    [styles.textLink]: true,
                  })}
                  onClick={() => {
                    // 20221115版本去除内部员工无法签单限制
                    // 是否是内部员工
                    /*if (employee) {
                        this.showEmployeeTip();
                        return;
                      }*/
                    this.signNew(record);
                  }}
                >
                  签单
                </TextLink>
              )}
              {Index.getLoginUser().institution === xinghuoInstitution && (
                <TextLink
                  style={{ marginRight: '16px' }}
                  onClick={() => {
                    this.setState({ isReporter: true });
                    TeachingReporter = withTeachingReporterHOC(
                      record.stuId.slice(3),
                      this.closeReporterModal,
                      record.studentName,
                    );
                  }}
                >
                  报告
                </TextLink>
              )}
              {couponUnused && (
                <TextLink className={styles.textLink} onClick={() => this.handleViewDeductionCouponInfo(stuId)}>
                  抵扣券明细
                </TextLink>
              )}
            </span>
          );
        },
      },
    ];
    if (!this.props.isManager) {
      // 只在`跟进客户`页面插入客户电话字段
      const isEnable = !!(this.props.enabled && hasConsultWorkbrench && alreadyBind);

      columns.splice(4, 0, {
        title: '客户电话',
        dataIndex: 'phoneNum',
        key: 'phoneNum',
        width: 200,
        render: (text, record, index) => {
          return text ? (
            <PhoneCall
              phoneNum={text}
              onCall={() => {
                this.phoneCall(record, index);
              }}
              callVisible={isEnable}
            />
          ) : (
            '-'
          );
        },
      });
    }

    return (
      <div className={styles.wrap}>
        <div style={{ display: 'flex' }}>
          <SearchBar
            expand={searchExpand}
            onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
            onSearch={this.search}
            onReset={this.reset}
          >
            {showBranchSelector && (
              <Select
                placeholder="分公司"
                showSearch
                dict={brenchDict}
                value={branchId}
                onChange={(val) => {
                  this.setState({ branchId: val });
                  this.onBranchChange(val);
                }}
                disabled={branchSelectorDisabled || branchSelectorInnerDisabled}
                allowClear={false}
              />
            )}
            {showCampusSelector && (
              <Select
                placeholder="学习中心"
                showSearch
                dict={blCampusDict}
                value={campusId}
                onChange={(val) => {
                  this.setState({ campusId: val, consultantId: '' });
                  this.onCampusChange(val);
                }}
                disabled={campusSelectorDisabled || campusSelectorInnerDisabled}
              />
            )}
            {showConsultantSelector && (
              <Select
                placeholder="咨询师"
                showSearch
                dict={dictAddOptionTitle(consultantDict)}
                value={consultantId}
                onChange={(val) => this.setState({ consultantId: val })}
                disabled={consultantSelectorDisabled || consultantSelectorInnerDisabled}
              />
            )}
            <Input
              placeholder="学生姓名"
              value={stuName}
              onChange={(e) => this.setState({ stuName: e.target.value })}
            />
            <Input placeholder="电话" value={phone} onChange={(e) => this.setState({ phone: e.target.value })} />
            <Select
              dict={couponUnusedDict}
              value={couponUnused}
              onChange={(val) => this.setState({ couponUnused: val })}
            />
            <Select
              placeholder="年级"
              dict={gradeDict}
              value={grade}
              onChange={(val) => this.setState({ grade: val })}
            />
            <Select
              placeholder="资源入口"
              dict={entranceDict}
              value={entrance}
              onChange={(val) => this.setState({ entrance: val })}
            />
            <Select
              placeholder="是否上门"
              dict={isVisitDict}
              value={isVisit}
              onChange={(val) => this.setState({ isVisit: val })}
            />
            <Select
              placeholder="意向度"
              dict={intentionDict}
              value={intention}
              onChange={(val) => this.setState({ intention: val })}
            />
            <Select
              placeholder="客户标签"
              dict={cusTagDict}
              value={cusTag}
              onChange={(val) => this.setState({ cusTag: val })}
            />
            <DatePicker
              placeholder="最新跟进时间（起）"
              disabledDate={this.getDisabledNewFollowTimeStartDate}
              value={newFollowTime}
              onChange={(val) => this.setState({ newFollowTime: val })}
            />
            <DatePicker
              placeholder="最新跟进时间（止）"
              disabledDate={this.getDisabledNewFollowTimeEndDate}
              value={newFollowTimeEnd}
              onChange={(val) => this.setState({ newFollowTimeEnd: val })}
            />
            <DatePicker
              placeholder="下次跟进时间（起）"
              disabledDate={this.getDisabledNextFollowTimeStartDate}
              value={nextFollowTime}
              onChange={(val) => this.setState({ nextFollowTime: val })}
            />
            <DatePicker
              placeholder="下次跟进时间（止）"
              disabledDate={this.getDisabledNextFollowTimeEndDate}
              value={nextFollowTimeEnd}
              onChange={(val) => this.setState({ nextFollowTimeEnd: val })}
            />
            <DatePicker
              placeholder="预约上门时间（起）"
              disabledDate={this.getDisabledAppointmentTimeStartDate}
              value={appointmentTime}
              onChange={(val) => this.setState({ appointmentTime: val })}
            />
            <DatePicker
              placeholder="预约上门时间（止）"
              disabledDate={this.getDisabledAppointmentTimeEndDate}
              value={appointmentTimeEnd}
              onChange={(val) => this.setState({ appointmentTimeEnd: val })}
            />
            <DatePicker
              placeholder="上门日期（起）"
              disabledDate={this.getDisabledVisitComeDateStartDate}
              value={visitComeDateStart}
              onChange={(val) => this.setState({ visitComeDateStart: val })}
            />
            <DatePicker
              placeholder="上门日期（止）"
              disabledDate={this.getDisabledVisitComeDateEndDate}
              value={visitComeDateEnd}
              onChange={(val) => this.setState({ visitComeDateEnd: val })}
            />
            <DatePicker
              placeholder="创建日期（起）"
              disabledDate={this.getDisabledCreateDateStartDate}
              value={createDateStart}
              onChange={(val) => this.setState({ createDateStart: val })}
            />
            <DatePicker
              placeholder="创建日期（止）"
              disabledDate={this.getDisabledCreateDateEndDate}
              value={createDateEnd}
              onChange={(val) => this.setState({ createDateEnd: val })}
            />
            <Input placeholder="客户姓名" value={name} onChange={(e) => this.setState({ name: e.target.value })} />
            <Select placeholder="资源来源" dict={sourceDict} value={source} onChange={this.sourceChange} />
            <Select
              placeholder="来源细分"
              dict={sourceDetailDict}
              value={sourceDetail}
              onChange={(val) => this.setState({ sourceDetail: val })}
            />
            <Input placeholder="学校" value={school} onChange={(e) => this.setState({ school: e.target.value })} />
          </SearchBar>
          <Button style={{ marginLeft: '-20px' }} onClick={this.refreshCurrentPage}>
            刷新
          </Button>
        </div>
        <List
          dataSource={list}
          columns={columns}
          pagination={{
            current: currentPage,
            total: totalRecords,
            pageSize,
          }}
          onChange={this.onPaginationChange}
          bordered={true}
          className={styles.customList}
        />
        <Loading loading={loading} />
        {transferCampusModalVis && (
          <TransferCampusModal
            onClose={() => this.setState({ transferCampusModalVis: false })}
            followingRecord={currentRecord}
            onConfirm={this.search}
          />
        )}
        {registerToDoorModalVis && (
          <RegisterToDoorModal
            onClose={() => this.setState({ registerToDoorModalVis: false })}
            records={this.state.registerToDoRecords}
            currentRecord={this.state.currentRecord}
          />
        )}
        {isReporter && <TeachingReporter />}
        {this.state.modalPhoneCallFollowVisible && (
          <ModalPhoneCallFollow list={this.phoneCallFollowList} onClose={this.switchModalPhoneCallFollow} />
        )}
        {this.generateDeductionCouponInfoModal()}
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    activeTabId: state.main.activeTabId,
    btnAuths: state.account.btnAuths,
    userId: state.account.user.userId,
    params: state.main.pageParams['/ConsultantFollowCustomer'],
    enabled: state.callCenter.enabled,
    callCenterConfig: state.account.callCenterConfig,
    alreadyBind: state.callCenter.alreadyBind,
  };
};

export { CusList };
export default connect(mapStateToProps, null, null, { withRef: true })(CusList);
