import React from 'react';
import { connect } from 'dva';
import styles from './styles.less';
import CusList from '../CusList';
import CusSignedList from '../CusSignedList';
import CusLostList from '../CusLostList';
import { Button, Loading } from 'COMPONENT';
import { CUS_LIST_FOLLOW, CUS_LIST_SIGNED, CUS_LIST_LOST, CUS_TRANSFER, cusListDict } from '../dicts';
import AddCusModal from '../AddCusModal';
import { loadConsultantDict } from 'PAGE/common/loaders';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import TransferCampusList from '../TransferCampusList';
import loaders from '../loaders';
import { Tabs } from 'antd';
import sectionStyle from 'COMPONENT/Section/styles.less';

const TabPane = Tabs.TabPane;

class CusListSec extends React.Component {
  constructor(props) {
    super(props);

    const params = props.params || {};
    const { initCusList } = params;

    let cusLists = {};
    cusListDict.forEach((d) => {
      cusLists[d.val] = { ref: null, inited: false };
    });

    this.cusLists = cusLists; // 客户列表信息，{id: { ref, inited }}

    this.state = {
      loading: false,

      activeCusList: initCusList ? initCusList : CUS_LIST_FOLLOW, // 当前激活状态的客户列表
      addCusModalVis: false, // 录入客户弹框显示状态
      canAddCus: false,

      searchingState: {},
      defaultSearchConditions: props.params ? props.params.defaultSearchConditions : {},
    };
  }

  componentDidMount() {
    const { activeCusList } = this.state;
    const { brenchId } = this.props.user;
    this.setState({ loading: true });

    Promise.all([
      loaders.getCrmResourceVisibility({ orgId: brenchId, rid: 'saveCalloutCustomer' }),
      this.cusLists[activeCusList].ref.initContent(),
    ]).then(([canAddCus]) => {
      this.cusLists[activeCusList].inited = true;
      this.setState({ loading: false, canAddCus });
    });
  }

  componentWillReceiveProps(nextProps) {
    if (this.props.defaultSearchConditions !== nextProps.defaultSearchConditions) {
      setTimeout(() => {
        this.setState({ defaultSearchConditions: nextProps.defaultSearchConditions });
      }, 50);
    }
  }

  changeCusList = (type) => {
    if (this.cusLists[type] && this.cusLists[type].inited) {
      this.setState({ activeCusList: type });
    } else if (this.cusLists[type] && this.cusLists[type].ref) {
      this.setState({ loading: true });
      this.cusLists[type].ref.initContent().then(() => {
        this.cusLists[type].inited = true;
        this.setState({ loading: false, activeCusList: type });
      });
    }
  };

  getCusListToolBar = () => {
    const { canAddCus } = this.state;
    return (
      canAddCus && (
        <Button iconType="plus" onClick={() => this.setState({ addCusModalVis: true })}>
          录入客户
        </Button>
      )
    );
  };

  addCusSuccess = () => {
    const { activeCusList } = this.state;
    const ref = this.cusLists[activeCusList].ref;

    if (ref) {
      ref.search();
    }
  };

  render() {
    const { dispatch } = this.props;
    const { activeCusList, addCusModalVis, loading, defaultSearchConditions } = this.state;

    return (
      <div className="Section" style={sectionStyle}>
        <div className="Section-body custom-tabs">
          <Loading loading={loading} />
          <Tabs
            className={styles.customTabs}
            tabBarExtraContent={this.getCusListToolBar()}
            onChange={this.changeCusList}
            activeKey={activeCusList}
            tabBarStyle={{ border: 'none', margin: '19px 0 25px 0' }}
          >
            {cusListDict.map((c) => {
              let listComp;
              if (c.val === CUS_LIST_FOLLOW) {
                listComp = (
                  <CusList
                    ref={(comp) => {
                      if (comp) {
                        this.cusLists[c.val].ref = getWrappedInstance(comp);
                      }
                    }}
                    dispatch={dispatch}
                    changeCusList={this.changeCusList}
                    defaultSearchConditions={defaultSearchConditions}
                  />
                );
              } else if (c.val === CUS_LIST_SIGNED) {
                listComp = (
                  <CusSignedList
                    ref={(comp) => {
                      if (comp) {
                        this.cusLists[c.val].ref = getWrappedInstance(comp);
                      }
                    }}
                    dispatch={dispatch}
                    changeCusList={this.changeCusList}
                  />
                );
              } else if (c.val === CUS_LIST_LOST) {
                listComp = <CusLostList ref={(comp) => (this.cusLists[c.val].ref = comp)} dispatch={dispatch} />;
              } else if (c.val === CUS_TRANSFER) {
                listComp = <TransferCampusList ref={(comp) => (this.cusLists[c.val].ref = comp)} dispatch={dispatch} />;
              }
              return (
                <TabPane key={c.val} tab={c.text} forceRender={true}>
                  {listComp}
                </TabPane>
              );
            })}
          </Tabs>
        </div>
        {addCusModalVis && (
          <AddCusModal
            onSaveSuccess={this.addCusSuccess}
            onClose={() => this.setState({ addCusModalVis: false })}
            showResEntrance={true}
          />
        )}
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  const pathName = state.routing.locationBeforeTransitions.pathname || '/ConsultantFollowCustomer';
  const params = state.main.pageParams[pathName] || {};

  return {
    params: params,
    user: state.account.user,
  };
};

export default connect(mapStateToProps)(CusListSec);
