import React from 'react';
import styles from './styles.less';
import Loading from 'COMPONENT/Loading';
import dicts from '../dicts';
import DropdownMenu from 'COMPONENT/DropdownMenu';
import Icon from 'COMPONENT/Icon';
import Button from 'COMPONENT/Button';
import DatePicker from 'COMPONENT/DatePicker';
import List from 'COMPONENT/List';
import SearchBar from 'COMPONENT/SearchBar';
import Input from 'COMPONENT/Input';
import Select from 'COMPONENT/Select';
import TextLink from 'COMPONENT/TextLink';
import { loadCommonDict, loadBranchDict, loadCommonDictNew, loadCampusDict, dictAddOptionTitle } from 'UTIL/dict';
import loaders from '../loaders';
import utils from '../utils';
import { browserHistory } from 'dva/router';
import { OLDPAGEBASE, WELCOMEHTML } from 'CONST/config';
import showAlert from 'COMPONENT/utils/globalTips/showAlert';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { loadSourceDetailDict, loadConsultantDict } from 'PAGE/common/loaders';
import jslog from 'SRC/jslog';

const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;

class CusLostList extends React.Component {
  constructor(props) {
    super(props);

    const { branchId, campusId, consultantId, isManager } = this.props;
    this.initListSearchKeys = {
      stuName: null,
      grade: null,
      school: null,
      name: null,
      phone: null,
      entrance: null,
      source: null,
      sourceDetail: null,
      lostTime: null,
      branchId,
      campusId,
      consultantId,
    };
    this.state = {
      loading: false, // {boolean} 内容加载状态
      searchExpand: false, // {boolean} 搜索栏展开状态

      stuName: null, // {string} 查询条件 学生姓名
      grade: null, // {string} 查询条件 年级
      school: null, // {string} 查询条件 学校
      name: null, // {string} 查询条件 客户姓名
      phone: null, // {string} 查询条件 电话
      entrance: null, // {string} 查询条件 资源入口
      source: null, // {string} 查询条件 资源来源
      sourceDetail: null, // {string} 查询条件 资源细分
      lostTime: null, // {string} 查询条件 流失时间

      gradeDict: [], // {[object]} 字典 年级
      entranceDict: [], // {[object]} 字典 资源入口
      sourceDict: [], // {[object]} 字典 资源来源
      sourceDetailDict: [], // {[object]} 字典 资源细分

      list: [], // {Array} 列表数据，
      /*
       { id, stuName, grade, school, name, phone, entrance, source, sourceDetail, lostTime }
      */
      listSearchKeys: this.initListSearchKeys, // {object} 列表数据的查询关键字
      totalRecords: 0, // {number} 总记录数
      currentPage: 1, // {number} 当前页码
      pageSize: isManager ? 20 : 10, // {number} 每页记录数

      branchId: '',
      brenchDict: [],
      branchSelectorInnerDisabled: false,
      campusId: '',
      blCampusDict: [],
      campusSelectorInnerDisabled: false,
      consultantId: '',
      consultantDict: [],
      consultantSelectorInnerDisabled: false,
    };
  }

  async initialBranch() {
    this.setState({
      branchSelectorInnerDisabled: true,
      campusSelectorInnerDisabled: true,
      consultantSelectorInnerDisabled: true,
    });
    let { branchId, campusId, showBranchSelector, showCampusSelector, showConsultantSelector } = this.props;
    const brenchDict = !showBranchSelector ? [] : await loadBranchDict();
    if (!branchId) {
      branchId = brenchDict[0] ? brenchDict[0]['val'] : '';
    }
    const blCampusDict = !showCampusSelector || !branchId ? [] : await loadCampusDict(branchId);
    const consultantDict = !showConsultantSelector || !campusId ? [] : await loadConsultantDict({ orgId: campusId });
    this.initListSearchKeys['branchId'] = branchId;
    this.initListSearchKeys['campusId'] = campusId;
    this.setState({
      branchId,
      campusId,
      listSearchKeys: this.initListSearchKeys,
      brenchDict,
      blCampusDict,
      consultantDict,
      branchSelectorInnerDisabled: false,
      campusSelectorInnerDisabled: false,
      consultantSelectorInnerDisabled: false,
    });
  }

  loadCusLostList = (params) => {
    const { isManager } = this.props;
    return isManager ? loaders.loadCusLostListManage(params) : loaders.loadCusLostList(params);
  };

  initContent = async () => {
    await this.initialBranch();
    const [listData, gradeDict, entranceDict, sourceDict] = await Promise.all([
      this.loadCusLostList(this.getListParams()),
      loadCommonDictNew('STUDENT_GRADE'),
      loadCommonDict('RES_ENTRANCE'),
      loadCommonDict('RES_TYPE'),
    ]);
    const { list, totalRecords } = listData;
    this.setState({ gradeDict, entranceDict, sourceDict, list, totalRecords });
  };

  getListParams = () => {
    const {
      listSearchKeys: {
        id,
        stuName,
        grade,
        school,
        name,
        phone,
        entrance,
        source,
        sourceDetail,
        lostTime,
        branchId,
        campusId,
        consultantId,
      },
      currentPage,
      pageSize,
    } = this.state;
    return {
      id,
      stuName,
      grade,
      school,
      name,
      phone,
      entrance,
      source,
      sourceDetail,
      lostTime,
      branchId,
      campusId,
      consultantId,
      currentPage,
      pageSize,
    };
  };

  search = () => {
    const {
      stuName,
      grade,
      school,
      name,
      phone,
      entrance,
      source,
      sourceDetail,
      lostTime,
      branchId,
      campusId,
      consultantId,
    } = this.state;
    const listSearchKeys = {
      stuName,
      grade,
      school,
      name,
      phone,
      entrance,
      source,
      sourceDetail,
      lostTime,
      branchId,
      campusId,
      consultantId,
    };
    this.setState({ loading: true });
    this.loadCusLostList({ ...this.getListParams(), ...listSearchKeys, currentPage: 1 }).then(
      ({ list, totalRecords, page }) => {
        jslog.report('QUERY', {
          function_name: '咨询师工作台>跟进客户管理（新版）>流失客户>查询',
          page: page,
          total: totalRecords,
          size: list.length,
        });
        this.setState({ loading: false, listSearchKeys, currentPage: 1, list, totalRecords });
      },
    );
  };

  reset = () => {
    this.setState(this.initListSearchKeys);
  };

  onPaginationChange = ({ current, pageSize }) => {
    this.setState({ loading: true });
    const changed = {
      currentPage: this.state.pageSize !== pageSize ? 1 : current,
      pageSize,
    };
    this.setState({ loading: true });
    this.loadCusLostList({ ...this.getListParams(), ...changed }).then(({ list, totalRecords }) => {
      this.setState({ loading: false, ...changed, list, totalRecords });
    });
  };

  reFollow = ({ id, stuId }) => {
    this.setState({ loading: true });
    loaders.reObtainCustomer({ id, stuId }).then(
      () => {
        showMessage('获取成功', 'success');
        this.search();
      },
      (e) => {
        showMessage(e.text || '获取失败');
        this.setState({ loading: false });
      },
    );
  };

  sourceChange = (val) => {
    if (val) {
      this.setState({ loading: true });
      loadSourceDetailDict(val).then(
        (dict) => {
          this.setState({ loading: false, source: val, sourceDetail: null, sourceDetailDict: dict });
        },
        () => {
          this.setState({ loading: false });
        },
      );
    } else {
      this.setState({ source: null, sourceDetail: null, sourceDetailDict: [] });
    }
  };

  onBranchChange = async (branchId) => {
    const { showCampusSelector } = this.props;
    this.setState({
      campusSelectorInnerDisabled: true,
      consultantSelectorInnerDisabled: true,
    });
    const blCampusDict = !showCampusSelector || !branchId ? [] : await loadCampusDict(branchId);
    const consultantDict = [];
    this.setState({
      blCampusDict,
      consultantDict,
      campusSelectorInnerDisabled: false,
      consultantSelectorInnerDisabled: false,
    });
  };

  onCampusChange = async (campusId) => {
    const { showConsultantSelector } = this.props;
    this.setState({
      consultantSelectorInnerDisabled: true,
    });
    const consultantDict = !showConsultantSelector || !campusId ? [] : await loadConsultantDict({ orgId: campusId });
    this.setState({
      consultantDict,
      consultantSelectorInnerDisabled: false,
    });
  };

  render() {
    const {
      showBranchSelector,
      branchSelectorDisabled,
      showCampusSelector,
      campusSelectorDisabled,
      showConsultantSelector,
      consultantSelectorDisabled,
    } = this.props;
    const {
      loading,
      searchExpand,
      stuName,
      grade,
      school,
      name,
      phone,
      entrance,
      source,
      sourceDetail,
      lostTime,
      gradeDict,
      entranceDict,
      sourceDict,
      sourceDetailDict,
      list,
      totalRecords,
      currentPage,
      pageSize,
      branchId,
      brenchDict,
      campusId,
      blCampusDict,
      consultantId,
      consultantDict,
      branchSelectorInnerDisabled,
      campusSelectorInnerDisabled,
      consultantSelectorInnerDisabled,
    } = this.state;

    const columns = [
      { title: '学生姓名', dataIndex: 'stuName', width: COL_WIDTH_L },
      { title: '年级', dataIndex: 'grade', dict: gradeDict },
      { title: '学校', width: COL_WIDTH_L, dataIndex: 'school' },
      { title: '客户姓名', dataIndex: 'name' },
      { title: '联系电话', width: COL_WIDTH_L, dataIndex: 'phone' },
      { title: '资源入口', dataIndex: 'entrance', dict: entranceDict },
      { title: '资源来源', dataIndex: 'source' },
      { title: '来源细分', dataIndex: 'sourceDetail' },
      { title: '流失时间', dataIndex: 'lostTime', width: COL_WIDTH_XL },
      { title: '流失原因', dataIndex: 'lostReason', width: COL_WIDTH_L },
      /*      { title: '操作', fixed: 'right', key: 'action', getActionConfig: (text, record, index) =>
              {
                return [{ text: '重新跟进', onClick: () => this.reFollow(record) }]
              }
            },*/
    ];
    return (
      <div className={styles.wrap}>
        <SearchBar
          expand={searchExpand}
          onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
          onSearch={this.search}
          onReset={this.reset}
        >
          {showBranchSelector && (
            <Select
              placeholder="分公司"
              showSearch
              allowClear={false}
              dict={brenchDict}
              value={branchId}
              onChange={(val) => {
                this.setState({ branchId: val });
                this.onBranchChange(val);
              }}
              disabled={branchSelectorDisabled || branchSelectorInnerDisabled}
            />
          )}
          {showCampusSelector && (
            <Select
              placeholder="学习中心"
              showSearch
              allowClear={false}
              dict={blCampusDict}
              value={campusId}
              onChange={(val) => {
                this.setState({ campusId: val, consultantId: '' });
                this.onCampusChange(val);
              }}
              disabled={campusSelectorDisabled || campusSelectorInnerDisabled}
            />
          )}
          {showConsultantSelector && (
            <Select
              placeholder="咨询师"
              showSearch
              dict={dictAddOptionTitle(consultantDict)}
              value={consultantId}
              onChange={(val) => this.setState({ consultantId: val })}
              disabled={consultantSelectorDisabled || consultantSelectorInnerDisabled}
            />
          )}
          <Input placeholder="学生姓名" value={stuName} onChange={(e) => this.setState({ stuName: e.target.value })} />
          <Select placeholder="年级" dict={gradeDict} value={grade} onChange={(val) => this.setState({ grade: val })} />
          <Input placeholder="学校" value={school} onChange={(e) => this.setState({ school: e.target.value })} />
          <Input placeholder="客户姓名" value={name} onChange={(e) => this.setState({ name: e.target.value })} />
          <Input placeholder="联系电话" value={phone} onChange={(e) => this.setState({ phone: e.target.value })} />
          <Select
            placeholder="资源入口"
            dict={entranceDict}
            value={entrance}
            onChange={(val) => this.setState({ entrance: val })}
          />
          <Select placeholder="资源来源" dict={sourceDict} value={source} onChange={this.sourceChange} />
          <Select
            placeholder="来源细分"
            dict={sourceDetailDict}
            value={sourceDetail}
            onChange={(val) => this.setState({ sourceDetail: val })}
          />
          <DatePicker placeholder="流失时间" value={lostTime} onChange={(val) => this.setState({ lostTime: val })} />
        </SearchBar>
        <List
          dataSource={list}
          columns={columns}
          pagination={{
            current: currentPage,
            total: totalRecords,
            pageSize,
          }}
          onChange={this.onPaginationChange}
        />
        <Loading loading={loading} />
      </div>
    );
  }
}
export default CusLostList;
