import React from 'react';
import styles from './styles.less';
import Loading from 'COMPONENT/Loading';
import DatePicker from 'COMPONENT/DatePicker';
import List from 'COMPONENT/List';
import SearchBar from 'COMPONENT/SearchBar';
import Input from 'COMPONENT/Input';
import Select from 'COMPONENT/Select';
import TextLink from 'COMPONENT/TextLink';
import { loadCommonDict, loadCommonDictNew, loadBranchDict, loadCampusDict, dictAddOptionTitle } from 'UTIL/dict';
import loaders from '../loaders';
import { connect } from 'dva/';
import moment from 'moment';
import { loadConsultantDict } from 'PAGE/common/loaders';
import jslog from 'SRC/jslog';
import { CUS_LIST_LOST } from '../dicts';
import { Modal as AntdModal } from 'antd';
import { xinghuoInstitution } from 'CONST/config';
import { getSwitchConfigsByCodesAndCampusIds } from 'SERVICE/common';
import withTeachingReporterHOC from 'PAGE/teachingReporterHOC/teachingReporterHOC';
import classNames from 'classnames';
import { loadSourceDetailDict } from 'SRC/pages/common/loaders';

let TeachingReporter = null;

const { COL_WIDTH_L, COL_WIDTH_XL } = List;

class CusSignedList extends React.Component {
  constructor(props) {
    super(props);
    this.showEmployeeTip = this.showEmployeeTip.bind(this);

    const { branchId, campusId, consultantId, isManager } = this.props;
    this.initListSearchKeys = {
      stuName: null,
      grade: null,
      school: null,
      name: null,
      phone: null,
      entrance: null,
      studyManager: null,
      studentStatus: null,
      signTime: null,
      signTimeEnd: null,
      branchId,
      campusId,
      consultantId,
      source: null,
      sourceDetail: null,
    };

    this.state = {
      loading: false, // {boolean} 内容加载状态
      searchExpand: false, // {boolean} 搜索栏展开状态

      stuName: null, // {string} 查询条件 学生姓名
      grade: null, // {string} 查询条件 年级
      school: null, // {string} 查询条件 学校
      name: null, // {string} 查询条件 客户姓名
      phone: null, // {string} 查询条件 电话
      entrance: null, // {string} 查询条件 资源入口
      studyManager: null, // {string} 查询条件 学管师
      studentStatus: null, // {string} 查询条件 学生状态
      signTime: null, // {string} 查询条件 签单时间（开始）
      signTimeEnd: null, // {string} 查询条件 签单时间（结束）
      source: null, // {string} 查询条件 资源来源
      sourceDetail: null, // {string} 查询条件 资源细分

      gradeDict: [], // {[object]} 字典 年级
      entranceDict: [], // {[object]} 字典 资源入口
      studentStatusDict: [], // {[object]} 字典 学生状态
      sourceDict: [], // {[object]} 字典 资源来源
      sourceDetailDict: [], // {[object]} 字典 资源细分

      list: [], // {Array} 列表数据，
      /*
       { id, stuName, gradeName, schoolName, name, phone, entrance, studyManager, studentStatus }
      */
      listSearchKeys: this.initListSearchKeys, // {object} 列表数据的查询关键字
      totalRecords: 0, // {number} 总记录数
      currentPage: 1, // {number} 当前页码
      pageSize: isManager ? 20 : 10, // {number} 每页记录数

      branchId: '',
      brenchDict: [],
      branchSelectorInnerDisabled: false,
      campusId: '',
      blCampusDict: [],
      campusSelectorInnerDisabled: false,
      consultantId: '',
      consultantDict: [],
      consultantSelectorInnerDisabled: false,
      isReporter: false, // 是否打开入学测报告
    };
  }

  async initialBranch() {
    this.setState({
      branchSelectorInnerDisabled: true,
      campusSelectorInnerDisabled: true,
      consultantSelectorInnerDisabled: true,
    });
    let { branchId, campusId, showBranchSelector, showCampusSelector, showConsultantSelector } = this.props;
    const brenchDict = !showBranchSelector ? [] : await loadBranchDict();
    if (!branchId) {
      branchId = brenchDict[0] ? brenchDict[0]['val'] : '';
    }
    const blCampusDict = !showCampusSelector || !branchId ? [] : await loadCampusDict(branchId);
    const consultantDict = !showConsultantSelector || !campusId ? [] : await loadConsultantDict({ orgId: campusId });
    this.initListSearchKeys['branchId'] = branchId;
    this.initListSearchKeys['campusId'] = campusId;
    this.setState({
      branchId,
      campusId,
      listSearchKeys: this.initListSearchKeys,
      brenchDict,
      blCampusDict,
      consultantDict,
      branchSelectorInnerDisabled: false,
      campusSelectorInnerDisabled: false,
      consultantSelectorInnerDisabled: false,
    });
  }

  loadCusSignedList = (params) => {
    const { isManager } = this.props;
    return isManager ? loaders.loadCusSignedListManage(params) : loaders.loadCusSignedList(params);
  };

  initContent = async () => {
    await this.initialBranch();
    const [listData, gradeDict, entranceDict, studentStatusDict, sourceDict] = await Promise.all([
      this.loadCusSignedList(this.getListParams()),
      loadCommonDictNew('STUDENT_GRADE'),
      loadCommonDict('RES_ENTRANCE'),
      loadCommonDict('StudentStatus'),
      loadCommonDict('RES_TYPE'),
    ]);
    const { list, totalRecords } = listData;
    this.setState({ gradeDict, entranceDict, studentStatusDict, list, totalRecords, sourceDict });
  };

  getListParams = () => {
    const {
      listSearchKeys: {
        stuName,
        grade,
        school,
        name,
        phone,
        entrance,
        studyManager,
        studentStatus,
        signTime,
        signTimeEnd,
        branchId,
        campusId,
        consultantId,
        source,
        sourceDetail,
      },
      currentPage,
      pageSize,
    } = this.state;
    return {
      stuName,
      grade,
      school,
      name,
      phone,
      entrance,
      studyManager,
      studentStatus,
      signTime,
      signTimeEnd,
      branchId,
      campusId,
      consultantId,
      currentPage,
      pageSize,
      source,
      sourceDetail,
    };
  };

  search = () => {
    const {
      stuName,
      grade,
      school,
      name,
      phone,
      entrance,
      studyManager,
      studentStatus,
      signTime,
      signTimeEnd,
      branchId,
      campusId,
      consultantId,
      source,
      sourceDetail,
    } = this.state;
    const listSearchKeys = {
      stuName,
      grade,
      school,
      name,
      phone,
      entrance,
      studyManager,
      studentStatus,
      signTime,
      signTimeEnd,
      branchId,
      campusId,
      consultantId,
      source,
      sourceDetail,
    };
    this.setState({ loading: true });
    this.loadCusSignedList({ ...this.getListParams(), ...listSearchKeys, currentPage: 1 }).then(
      ({ list, totalRecords, page }) => {
        this.setState({ loading: false, listSearchKeys, currentPage: 1, list, totalRecords });
      },
    );
  };

  reset = () => {
    this.setState(this.initListSearchKeys);
  };

  onPaginationChange = ({ current, pageSize }) => {
    this.setState({ loading: true });
    const changed = {
      currentPage: this.state.pageSize !== pageSize ? 1 : current,
      pageSize,
    };
    this.setState({ loading: true });
    this.loadCusSignedList({ ...this.getListParams(), ...changed }).then(({ list, totalRecords }) => {
      this.setState({ loading: false, ...changed, list, totalRecords });
    });
  };

  signNew = async (record) => {
    const { id, stuId } = record;
    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path: `/signContractPro/${JSON.stringify({
          studentId: stuId,
          customerId: id,
          flag: true,
          signEntrance: 'CONSULTANT',
        })}`,
        refresh: true,
      },
    });
  };

  getDisabledSignTimeStartDate = (startDate) => {
    const { signTimeEnd: endDate } = this.state;
    if (!startDate || !endDate) {
      return false;
    }
    return moment(startDate).valueOf() > moment(endDate).valueOf();
  };

  getDisabledSignTimeEndDate = (endDate) => {
    const { signTime: startDate } = this.state;
    if (!endDate || !startDate) {
      return false;
    }
    return moment(endDate).valueOf() < moment(startDate).valueOf();
  };

  onBranchChange = async (branchId) => {
    const { showCampusSelector } = this.props;
    this.setState({
      campusSelectorInnerDisabled: true,
      consultantSelectorInnerDisabled: true,
    });
    const blCampusDict = !showCampusSelector || !branchId ? [] : await loadCampusDict(branchId);
    const consultantDict = [];
    this.setState({
      blCampusDict,
      consultantDict,
      campusSelectorInnerDisabled: false,
      consultantSelectorInnerDisabled: false,
    });
  };

  onCampusChange = async (campusId) => {
    const { showConsultantSelector } = this.props;
    this.setState({
      consultantSelectorInnerDisabled: true,
    });
    const consultantDict = !showConsultantSelector || !campusId ? [] : await loadConsultantDict({ orgId: campusId });
    this.setState({
      consultantDict,
      consultantSelectorInnerDisabled: false,
    });
  };

  // 关闭报告Modal
  closeReporterModal = () => {
    this.setState({ isReporter: false });
  };

  showEmployeeTip = () => {
    const tip =
      '该学生的客户号码属于员工号码，根据集团要求，内部员工号码不允许签单，请使用非员工号码录入资源然后签单。';
    AntdModal.info({
      title: tip,
      content: '',
      className: 'homAntdModal',
    });
  };

  sourceChange = (val) => {
    if (val) {
      this.setState({ loading: true });
      loadSourceDetailDict(val).then(
        (dict) => {
          this.setState({ loading: false, source: val, sourceDetail: null, sourceDetailDict: dict });
        },
        () => {
          this.setState({ loading: false });
        },
      );
    } else {
      this.setState({ source: null, sourceDetail: null, sourceDetailDict: [] });
    }
  };

  render() {
    const {
      showBranchSelector,
      branchSelectorDisabled,
      showCampusSelector,
      campusSelectorDisabled,
      showConsultantSelector,
      consultantSelectorDisabled,
      btnAuths,
    } = this.props;
    const {
      loading,
      searchExpand,
      stuName,
      grade,
      school,
      name,
      phone,
      entrance,
      studyManager,
      studentStatus,
      signTime,
      signTimeEnd,
      gradeDict,
      entranceDict,
      studentStatusDict,
      list,
      totalRecords,
      currentPage,
      pageSize,
      branchId,
      brenchDict,
      campusId,
      blCampusDict,
      consultantId,
      consultantDict,
      branchSelectorInnerDisabled,
      campusSelectorInnerDisabled,
      consultantSelectorInnerDisabled,
      isReporter,
    } = this.state;
    //id, stuName, gradeName, schoolName, name, phone, entrance, studyManager, studentStatus
    const columns = [
      {
        title: '学生姓名',
        dataIndex: 'stuName',
        width: COL_WIDTH_L,
        render: (text, record) => {
          const { employee } = record;

          return (
            <div className={styles.stuNameWrap}>
              {text}
              {employee && (
                <div>
                  <span className={'employeeFlag'}>内部员工</span>
                </div>
              )}
            </div>
          );
        },
      },
      { title: '年级', dataIndex: 'gradeName' },
      { title: '学校', width: COL_WIDTH_L, dataIndex: 'schoolName' },
      { title: '客户姓名', dataIndex: 'name' },
      { title: '联系电话', width: COL_WIDTH_L, dataIndex: 'phone' },
      { title: '资源入口', dataIndex: 'entrance', dict: entranceDict },
      { title: '学管师', dataIndex: 'studyManager' },
      { title: '学生状态', dataIndex: 'studentStatus', dict: studentStatusDict },
      { title: '签单时间', dataIndex: 'signTime', width: COL_WIDTH_XL },
      {
        title: '操作',
        key: 'action',
        render: (text, record, index) => {
          if (this.props.isManager) {
            return <span>{'-'}</span>;
          }
          return (
            <div>
              <span>
                {record.canSign !== undefined &&
                  record.canSign &&
                  btnAuths.includes('FOLLOW_CUSTOMER_SIGN_ADDCONTRACT') && (
                    <TextLink
                      className={classNames({
                        [styles.textLink]: true,
                      })}
                      onClick={() => {
                        // 20221115版本去除内部员工无法签单限制
                        // 是否是内部员工
                        /*if (record.employee) {
                        this.showEmployeeTip()
                        return
                      }*/

                        this.signNew(record);
                      }}
                    >
                      签单
                    </TextLink>
                  )}
              </span>
              <span>
                {Index.getLoginUser().institution === xinghuoInstitution && (
                  <TextLink
                    style={{ marginRight: '16px' }}
                    onClick={() => {
                      this.setState({ isReporter: true });
                      TeachingReporter = withTeachingReporterHOC(
                        record.stuId.slice(3),
                        this.closeReporterModal,
                        record.stuName,
                      );
                    }}
                  >
                    报告
                  </TextLink>
                )}
              </span>
            </div>
          );
        },
      },
    ];
    return (
      <div className={styles.wrap}>
        <SearchBar
          expand={searchExpand}
          onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
          onSearch={this.search}
          onReset={this.reset}
        >
          {showBranchSelector && (
            <Select
              placeholder="分公司"
              showSearch
              allowClear={false}
              dict={brenchDict}
              value={branchId}
              onChange={(val) => {
                this.setState({ branchId: val });
                this.onBranchChange(val);
              }}
              disabled={branchSelectorDisabled || branchSelectorInnerDisabled}
            />
          )}
          {showCampusSelector && (
            <Select
              placeholder="学习中心"
              showSearch
              allowClear={false}
              dict={blCampusDict}
              value={campusId}
              onChange={(val) => {
                this.setState({ campusId: val, consultantId: '' });
                this.onCampusChange(val);
              }}
              disabled={campusSelectorDisabled || campusSelectorInnerDisabled}
            />
          )}
          {showConsultantSelector && (
            <Select
              placeholder="咨询师"
              showSearch
              dict={dictAddOptionTitle(consultantDict)}
              value={consultantId}
              onChange={(val) => this.setState({ consultantId: val })}
              disabled={consultantSelectorDisabled || consultantSelectorInnerDisabled}
            />
          )}
          <Input placeholder="学生姓名" value={stuName} onChange={(e) => this.setState({ stuName: e.target.value })} />
          <Select placeholder="年级" dict={gradeDict} value={grade} onChange={(val) => this.setState({ grade: val })} />
          <Input placeholder="学校" value={school} onChange={(e) => this.setState({ school: e.target.value })} />
          <Input placeholder="客户姓名" value={name} onChange={(e) => this.setState({ name: e.target.value })} />
          <Input placeholder="联系电话" value={phone} onChange={(e) => this.setState({ phone: e.target.value })} />
          <Select
            placeholder="资源入口"
            dict={entranceDict}
            value={entrance}
            onChange={(val) => this.setState({ entrance: val })}
          />
          <Input
            placeholder="学管师"
            value={studyManager}
            onChange={(e) => this.setState({ studyManager: e.target.value })}
          />
          <Select
            placeholder="学生状态"
            dict={studentStatusDict}
            value={studentStatus}
            onChange={(val) => this.setState({ studentStatus: val })}
          />
          <DatePicker
            placeholder="签单时间（起）"
            disabledDate={this.getDisabledSignTimeStartDate}
            value={signTime}
            onChange={(val) => this.setState({ signTime: val })}
          />
          <DatePicker
            placeholder="签单时间（止）"
            disabledDate={this.getDisabledSignTimeEndDate}
            value={signTimeEnd}
            onChange={(val) => this.setState({ signTimeEnd: val })}
          />
          <Select
            placeholder="资源来源"
            dict={this.state.sourceDict}
            value={this.state.source}
            onChange={this.sourceChange}
          />
          <Select
            placeholder="来源细分"
            dict={this.state.sourceDetailDict}
            value={this.state.sourceDetail}
            onChange={(val) => this.setState({ sourceDetail: val })}
          />
        </SearchBar>
        <List
          dataSource={list}
          columns={columns}
          pagination={{
            current: currentPage,
            total: totalRecords,
            pageSize,
          }}
          onChange={this.onPaginationChange}
        />
        <Loading loading={loading} />
        {isReporter && <TeachingReporter />}
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    activeTabId: state.main.activeTabId,
    btnAuths: state.account.btnAuths,
  };
};

export default connect(mapStateToProps, null, null, { withRef: true })(CusSignedList);
