import React from 'react';
import styles from './styles.less';
import DatePicker from 'COMPONENT/DatePicker';
import { getDictTextByVal, loadCommonDict } from 'UTIL/dict';
import moment from 'moment';
import { DATE_FORMAT, TIME_FORMAT } from 'COMPONENT/utils/form';
import loaders from '../loaders';
import Section from 'COMPONENT/Section';
import { connect } from 'dva';
import jslog from 'SRC/jslog';
import Loading from 'COMPONENT/Loading';
import TextLink from 'COMPONENT/TextLink';
import { CUS_LIST_FOLLOW } from '../dicts';

const startOfWeek = moment().startOf('week').format(DATE_FORMAT);
const endOfWeek = moment().endOf('week').format(DATE_FORMAT);

class VisitCusContent extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 加载状态
      startDate: startOfWeek, // {string} 开始日期
      endDate: endOfWeek, // {string} 结束日期
      gradeDict: [], // {object[]} 年级字典
      doneCount: 0, // {number} 已跟进数量
      totalCount: 0, // {number} 总数量
      list: [], // {object[]} 列表数据，
      // { studentName, grade, schoolName, name, phone, remark }
    };
  }

  componentDidMount() {
    this.initContent();
  }

  initContent = async () => {
    const { userId } = this.props.user;
    const { startDate, endDate } = this.state;
    this.setState({ loading: true });
    const [list] = await Promise.all([loaders.loadPhoneCusList({ startDate, endDate, consultantId: userId })]);
    this.setState({ loading: false, list });
  };

  getDisabledStartDate = (startDate) => {
    const { endDate } = this.state;
    if (!startDate || !endDate) {
      return false;
    }
    return moment(startDate).valueOf() > moment(endDate).valueOf();
  };

  getDisabledEndDate = (endDate) => {
    const { startDate } = this.state;
    if (!endDate || !startDate) {
      return false;
    }
    return moment(endDate).valueOf() < moment(startDate).valueOf();
  };

  startDateChange = (val) => {
    const { userId } = this.props.user;
    const { endDate } = this.state;
    this.setState({ loading: true });
    jslog.report('click', { function_name: '咨询师工作台>跟进客户管理（新版）>预约跟进电话客户日期筛选' });
    loaders.loadPhoneCusList({ startDate: val, endDate, consultantId: userId }).then(
      (list) => {
        this.setState({ loading: false, startDate: val, list });
      },
      () => {
        this.setState({ loading: false });
      },
    );
  };

  endDateChange = (val) => {
    const { userId } = this.props.user;
    const { startDate } = this.state;
    this.setState({ loading: true });
    jslog.report('click', { function_name: '咨询师工作台>跟进客户管理（新版）>预约跟进电话客户日期筛选' });
    loaders.loadPhoneCusList({ startDate, endDate: val, consultantId: userId }).then(
      (list) => {
        this.setState({ loading: false, endDate: val, list });
      },
      () => {
        this.setState({ loading: false });
      },
    );
  };

  getField = (val) => {
    if (val) {
      return <span className={styles.field}>{val}</span>;
    }
    return null;
  };

  search = async () => {
    const { userId } = this.props.user;
    const { startDate, endDate } = this.state;
    this.setState({ loading: true });
    const list = await loaders.loadPhoneCusList({ startDate, endDate, consultantId: userId });
    this.setState({ loading: false, list });
  };

  defaultSearch = (item, startDate, endDate) => {
    const params = {
      [item.startDate]: startDate,
      [item.endDate]: endDate,
    };
    if (item.resEntranceId) {
      params['entrance'] = item.resEntranceId;
    }

    this.props.onChange({
      defaultSearchConditions: params,
      initCusList: CUS_LIST_FOLLOW,
    });
  };

  render() {
    const { loading, startDate, endDate, list } = this.state;
    return (
      <div className={styles.section}>
        <Loading loading={loading} />
        <div className={styles.sectionHeader}>
          <div className={styles.sectionTitle}>预约跟进电话客户</div>
        </div>
        <div className={styles.sectionBody}>
          <div className={styles.wrap}>
            <div className={styles.header}>
              <DatePicker
                placeholder="开始日期"
                value={startDate}
                onChange={this.startDateChange}
                disabledDate={this.getDisabledStartDate}
                className={styles.input}
              />
              <DatePicker
                placeholder="结束日期"
                value={endDate}
                onChange={this.endDateChange}
                popupPlacement="bottomRight"
                disabledDate={this.getDisabledEndDate}
                className={styles.input}
              />
            </div>
            <div className={styles.list}>
              {list.length > 0 ? (
                list.map((item, idx) => {
                  return (
                    <div key={item.id} className={styles.item}>
                      <span className={styles.itemLeft}>{item.text}</span>
                      <span className={styles.itemRight}>
                        <TextLink onClick={() => this.defaultSearch(item, startDate, endDate)}>{item.val}</TextLink>
                      </span>
                    </div>
                  );
                })
              ) : (
                <div className={styles.tips}>暂无记录</div>
              )}
            </div>
          </div>
        </div>
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    user: state.account.user,
  };
};
export default connect(mapStateToProps)(VisitCusContent);
