import React from 'react';
import { message } from 'antd';
import { Modal, List, Button } from 'COMPONENT';
import classnames from 'classnames';
import styles from './styles.less';
import moment from 'moment';
import loaders from '../loaders';

const modalCls = {
  [styles.wrap]: 1,
  [styles.modal]: 1,
};

export default class RegisterToDoorModal extends React.Component {
  state = {
    loading: false,
  };

  onOk = async () => {
    const { records, currentRecord } = this.props;

    this.setState({ loading: true });
    try {
      const res = await (records?.length
        ? loaders.postSignForCustomerComeOn({
            visitId: records[0].id,
            customerId: records[0].customerId,
          })
        : loaders.postSignForCustomerComeOnWithoutAppoint({
            customerId: currentRecord.cusId,
            studentId: currentRecord.studentId,
          }));

      this.setState({ loading: false });
      if (res.resultCode === 0) {
        this.props.onClose();
        message.success('登记上门操作成功');
      } else {
        message.success('登记上门操作失败，请稍后重试');
      }
    } catch (e) {
      this.setState({ loading: false });
      message.success('登记上门操作失败，请稍后重试');
    }
  };

  render() {
    const { onClose, records, currentRecord } = this.props;
    const modalProps = {
      onClose,
      width: 750,
      title: '登记上门',
      okText: '确认上门',
      cancelText: '取消',
      loading: this.state.loading,
      className: classnames(modalCls),
      footer: (
        <div>
          <Button
            type="primary"
            onClick={this.onOk}
            disabled={records?.length ? !!records[0].meetingConfirmTime : false}
          >
            确认上门
          </Button>
          <Button onClick={onClose}>取消</Button>
        </div>
      ),
    };
    const columns = [
      { title: '预约上门时间', dataIndex: 'meetingTime', key: 'meetingTime', width: 160 },
      { title: '预约人', dataIndex: 'createUserName', key: 'createUserName', width: 100 },
      { title: '提交时间', dataIndex: 'createTime', key: 'createTime', width: 160 },
      {
        title: '状态',
        dataIndex: 'status',
        key: 'status',
        render: (_, record) => (record.meetingConfirmTime ? '已登记' : '未登记'),
      },
    ];

    console.log(currentRecord);

    return (
      <Modal {...modalProps}>
        <div className={styles.main}>
          {records?.length ? (
            <div>
              <p
                style={{ marginBottom: 10 }}
              >{`您之前预约了学生【${records[0].followStuName}】于今日上门，点击登记上门就可以将以下所有预约记录更新为【已登记】`}</p>
              <List columns={columns} dataSource={records} pagination={false} />
            </div>
          ) : (
            <div>
              <p>{`本次操作将会更新学生【${currentRecord?.studentName}】为已上门状态，上门时间为【${moment().format(
                'YYYY-MM-DD HH:mm:ss',
              )}】，请确认`}</p>
            </div>
          )}
        </div>
      </Modal>
    );
  }
}
