import React from 'react';
import DatePicker from 'COMPONENT/DatePicker';
import List from 'COMPONENT/List';
import SearchBar from 'COMPONENT/SearchBar';
import Input from 'COMPONENT/Input';
import Select from 'COMPONENT/Select';
import loaders from '../loaders';
import { loadCommonDict, loadAllBranchCampusDict } from 'UTIL/dict';
import moment from 'moment';
import { auditStatusObj, receiveStatusObj } from 'PAGE/customer/TransferCampusAudit/dict';
// import { loadSourceDetailDict } from "PAGE/common/loaders";

class TransferCampusList extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      searchExpand: false,
      listLoading: true,
      dataSource: [],
      total: 0,
      current: 1,
      pageSize: 10,
      searchConditions: {},

      studentName: null,
      studentContact: null,
      startTime: null,
      endTime: null,
      receiveCampusId: null,
      resEntrance: null,

      resEntranceDict: [],
      campusDict: [],
    };
  }

  onSearch = async (val) => {
    const page = 1;
    this.setState({ listLoading: true });
    const {
      studentName,
      studentContact,
      startTime,
      endTime,
      receiveCampusId,
      resEntrance,
      // cusType,
      // cusOrg,
    } = this.state;
    const params = {
      startTime: startTime ? moment(startTime).startOf('day').format('YYYY-MM-DD HH:mm:ss') : null,
      endTime: endTime ? moment(endTime).endOf('day').format('YYYY-MM-DD HH:mm:ss') : null,
      receiveCampusId,
      resEntrance,
      // cusType,
      // cusOrg,
      studentName: studentName ? studentName.trim() : null,
      studentContact: studentContact ? studentContact.trim() : null,
    };
    const res = await loaders.loadTransferApplyingList({
      ...params,
      pageSize: this.state.pageSize,
      pageNo: page,
    });
    this.setState({
      searchConditions: params,
      current: page,
      total: res.rowCount || 0,
      dataSource: res.datas || [],
      listLoading: false,
    });
    return;
  };

  onChange = async ({ current = this.state.current, pageSize = this.state.pageSize }) => {
    this.setState({ listLoading: true });
    const res = await loaders.loadTransferApplyingList({
      ...this.state.searchConditions,
      pageSize: pageSize,
      pageNo: current,
    });
    this.setState({
      current: res.pageNo,
      pageSize: res.pageSize,
      total: res.rowCount,
      dataSource: res.datas,
      listLoading: false,
    });
    return;
  };

  onReset = () => {
    this.setState({
      studentName: null,
      studentContact: null,
      startTime: null,
      endTime: null,
      receiveCampusId: null,
      resEntrance: null,
      cusType: null,
      cusOrg: null,
      sourceDetailDict: [],
    });
  };

  initContent = () => {
    return this.init();
  };

  init = async () => {
    this.setState({ listLoading: true });
    const promises = [loadAllBranchCampusDict(), loadCommonDict('RES_ENTRANCE'), this.onSearch()];
    const [campusDict, resEntranceDict] = await Promise.all(promises);
    this.setState({
      resEntranceDict: resEntranceDict,
      campusDict: campusDict,
      listLoading: false,
    });
  };

  getDisabledStartDate = (startDate) => {
    const { endDate } = this.state;
    if (!startDate || !endDate) {
      return false;
    }
    return moment(startDate).valueOf() > moment(endDate).valueOf();
  };

  getDisabledEndDate = (endDate) => {
    const { startDate } = this.state;
    if (!endDate || !startDate) {
      return false;
    }
    return moment(endDate).valueOf() < moment(startDate).valueOf();
  };

  render() {
    const columns = [
      { title: '学生姓名', dataIndex: 'studentName' },
      { title: '联系方式', dataIndex: 'studentContact' },
      { title: '资源入口', dataIndex: 'resEntranceName' },
      { title: '申请转入学习中心', dataIndex: 'receiveCampusName' },
      { title: '申请转校时间', dataIndex: 'applyTime' },
      {
        title: '审核状态',
        dataIndex: 'auditStatus',
        render: (text, record, i) => {
          const { auditFailReason } = record;
          return (
            <div>
              <div>
                <span>{auditStatusObj['' + text]}</span>
              </div>
              {+text === 3 && (
                <div>
                  <span>（{auditFailReason}）</span>
                </div>
              )}
            </div>
          );
        },
      },
      { title: '转校原因', dataIndex: 'applyReason' },
      {
        title: '转校情况',
        dataIndex: 'receiveStatus',
        render: (text, record, i) => {
          const { receiveFailReason, auditStatus } = record;
          if (+auditStatus !== 1) {
            return '';
          }
          return (
            <div>
              <div>
                <span>{receiveStatusObj['' + text]}</span>
              </div>
              {text === 3 && (
                <div>
                  <span>（{receiveFailReason}）</span>
                </div>
              )}
            </div>
          );
        },
      },
    ];

    const {
      searchExpand,
      dataSource,
      total,
      current,
      pageSize,
      listLoading,

      studentName,
      studentContact,
      startTime,
      endTime,
      receiveCampusId,
      resEntrance,

      campusDict,
      resEntranceDict,
      // cusType,
      // cusOrg,
    } = this.state;

    return (
      <div className="flexWrap">
        <SearchBar
          expand={searchExpand}
          onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
          onSearch={this.onSearch}
          onReset={this.onReset}
        >
          <Input
            placeholder="学生姓名"
            value={studentName}
            onChange={(e) => this.setState({ studentName: e.target.value })}
          />
          <Input
            placeholder="联系方式"
            value={studentContact}
            onChange={(e) => this.setState({ studentContact: e.target.value })}
          />
          <DatePicker
            placeholder="申请开始时间（起）"
            disabledDate={this.getDisabledStartDate}
            value={startTime}
            onChange={(val) => this.setState({ startTime: val })}
          />
          <DatePicker
            placeholder="申请结束时间（止）"
            disabledDate={this.getDisabledEndDate}
            value={endTime}
            onChange={(val) => this.setState({ endTime: val })}
          />
          <Select
            showSearch
            placeholder="请选择学习中心"
            dict={campusDict}
            value={receiveCampusId}
            onChange={(val) => this.setState({ receiveCampusId: val })}
          />
          <Select
            placeholder="资源入口"
            dict={resEntranceDict}
            value={resEntrance}
            onChange={(val) => this.setState({ resEntrance: val })}
          />
          {/* <Select
            placeholder="资源来源"
            dict={this.state.resTypeDict}
            value={cusType}
            onChange={(val) => {
              this.setState({ cusType: val });
              this.setState({ listLoading: true });
              if (val) {
                loadSourceDetailDict(val).then((dict) => {
                  this.setState({
                    sourceDetailDict: dict,
                  });
                  this.setState({ listLoading: false });
                });
              } else {
                this.setState({
                  sourceDetailDict: [],
                });
              }
            }}
          />
          <Select
            placeholder="资源细分"
            value={cusOrg}
            onChange={(val) => this.setState({ cusOrg: val })}
            dict={this.state.sourceDetailDict}
          /> */}
        </SearchBar>
        <List
          dataSource={dataSource}
          columns={columns}
          pagination={{
            current,
            total,
            pageSize,
          }}
          loading={listLoading}
          onChange={this.onChange}
        />
      </div>
    );
  }
}

export default TransferCampusList;
