import { getDataDictList } from 'SERVICE/system';
import {
  getConsultorNextFollowing,
  getConsultorFollowingList,
  getConsultorSignUpList,
  getConsultorLostList,
  obtainCustomerInternet,
  obtainCustomerAgain,
  updateCustomerStudentStatus,
  getFollowupCustomersForOrgManager,
  getSignedCustomersForOrgManager,
  getLostCustomersForOrgManager,
  submitStudentTransferApply,
  getStudentTransferApplyDetailInfo,
  listStudentTransferApplyBySelf,
  listStudentTransferApplyForCampusAudit,
  crmResourceVisibility,
  canSign,
  getFollowingRecordNew,
  signForCustomerComeOn,
  signForCustomerComeOnWithoutAppoint,
} from 'SERVICE/consultant';
import { CUS_LIST_FOLLOW, CUS_LIST_SIGNED, CUS_LIST_LOST, IS_VISIT_YES, IS_VISIT_NO } from './dicts';

const postSignForCustomerComeOn = (data) => {
  return signForCustomerComeOn(data);
};

const postSignForCustomerComeOnWithoutAppoint = (data) => {
  return signForCustomerComeOnWithoutAppoint(data);
};

const loadFollowingRecordNew = async (params) => {
  try {
    const res = await getFollowingRecordNew(params);
    console.log(res);
    return res;
  } catch (e) {
    return { resultCode: -1 };
  }
};

const loadPhoneCusList = async ({ startDate, endDate, branchId, campusId, consultantId }) => {
  try {
    const { data } = await getConsultorNextFollowing({
      dateStart: startDate,
      dateEnd: endDate,
      branchId,
      campusId,
      consultantId,
    });
    const list = [
      { id: 'stayFollow', text: '预约跟进', val: 0, startDate: 'nextFollowTime', endDate: 'nextFollowTimeEnd' },
      {
        id: 'directVisit',
        text: '直访',
        val: 0,
        resEntranceId: 'DIRECT_VISIT',
        startDate: 'createDateStart',
        endDate: 'createDateEnd',
      },
      { id: 'appointment', text: '预约上门', val: 0, startDate: 'appointmentTime', endDate: 'appointmentTimeEnd' },
      {
        id: 'callIn',
        text: '呼入',
        val: 0,
        resEntranceId: 'CALL_IN',
        startDate: 'createDateStart',
        endDate: 'createDateEnd',
      },
      { id: 'visited', text: '已上门', val: 0, startDate: 'visitComeDateStart', endDate: 'visitComeDateEnd' },
      {
        id: 'onLine',
        text: '网络',
        val: 0,
        resEntranceId: 'ON_LINE',
        startDate: 'createDateStart',
        endDate: 'createDateEnd',
      },
      { id: 'followed', text: '已跟进', val: 0, startDate: 'newFollowTime', endDate: 'newFollowTimeEnd' },
    ];

    return list.map((ele) => {
      return Object.assign(ele, {
        val: data[ele.id],
      });
    });
  } catch (e) {
    console.log(e);
  }
};

const loadPhoneCusListManage = async ({ startDate, endDate, branchId, campusId, consultantId }) => {
  try {
    const { data } = await getConsultorNextFollowing({
      dateStart: startDate,
      dateEnd: endDate,
      branchId,
      campusId,
      consultantId,
    });
    const list = [
      { id: 'stayFollow', text: '预约跟进', val: 0, startDate: 'nextFollowTime', endDate: 'nextFollowTimeEnd' },
      { id: 'appointment', text: '预约上门', val: 0, startDate: 'appointmentTime', endDate: 'appointmentTimeEnd' },
      { id: 'visited', text: '已上门', val: 0, startDate: 'visitComeDateStart', endDate: 'visitComeDateEnd' },
      { id: 'followed', text: '已跟进', val: 0, startDate: 'newFollowTime', endDate: 'newFollowTimeEnd' },
      {
        id: 'directVisit',
        text: '直访',
        val: 0,
        resEntranceId: 'DIRECT_VISIT',
        startDate: 'createDateStart',
        endDate: 'createDateEnd',
      },
      {
        id: 'callIn',
        text: '呼入',
        val: 0,
        resEntranceId: 'CALL_IN',
        startDate: 'createDateStart',
        endDate: 'createDateEnd',
      },
      {
        id: 'onLine',
        text: '网络',
        val: 0,
        resEntranceId: 'ON_LINE',
        startDate: 'createDateStart',
        endDate: 'createDateEnd',
      },
    ];

    return list.map((ele) => {
      return Object.assign(ele, {
        val: data[ele.id],
      });
    });
  } catch (e) {
    console.log(e);
  }
};

const loadCusList = async ({
  couponUnused,
  stuName,
  name,
  phone,
  school,
  grade,
  entrance,
  isVisit,
  intention,
  cusTag,
  newFollowTime,
  nextFollowTime,
  appointmentTime,
  newFollowTimeEnd,
  nextFollowTimeEnd,
  appointmentTimeEnd,
  visitComeDateStart,
  visitComeDateEnd,
  createDateStart,
  createDateEnd,
  currentPage,
  pageSize,
  pageOrder,
  pageOrderField,
  resEntranceType,
  remindType,
  source,
  sourceDetail,
}) => {
  const params = {
    couponUnused,
    page: currentPage,
    rows: pageSize,
    studentName: stuName,
    customerName: name,
    contact: phone,
    schoolName: school,
    gradeId: grade,
    resEntranceId: entrance,
    isAppointCome: isVisit === IS_VISIT_YES ? 1 : isVisit === IS_VISIT_NO ? 0 : null,
    intentionOfTheCustomerId: intention,
    customerSign: cusTag,
    recentlyTimeStart: newFollowTime,
    recentlyTimeEnd: newFollowTimeEnd,
    nextFollowTimeStart: nextFollowTime,
    nextFollowTimeEnd: nextFollowTimeEnd,
    meetingTimeStart: appointmentTime,
    meetingTimeEnd: appointmentTimeEnd,
    visitComeDateStart,
    visitComeDateEnd,
    createDateStart,
    createDateEnd,
    cusType: source,
    cusOrg: sourceDetail,
    resEntranceType: resEntranceType || null,
    remindType: remindType || null,
    sord: pageOrder,
    sidx: pageOrderField,
  };

  const { records, rows, page } = await getConsultorFollowingList(params);
  const data = rows || [];
  const totalRecords = records || 0;
  const list = data.map((d) => {
    return {
      ...d,
      id: d.cusId,
      stuId: d.studentId,
      studentName: d.studentName,
      grade: d.gradeId,
      gradeName: d.gradeName,
      school: d.schoolId,
      schoolName: d.schoolName,
      name: d.cusName,
      phone: d.contact,
      entrance: d.resEntranceId,
      isVisit: d.isAppointCome,
      intention: d.intentionOfTheCustomerId,
      cusTag: d.customerSign,
      newFollowTime: d.lastFollowUpTime,
      nextFollowTime: getDateByDateTime(d.nextFollowupDate),
      appointmentTime: d.appointmentDate,
      isNew: d.isNew,
      isSigned: d.customerStatus === 'FLOW_AWAY',
      unFetch: d.dealStatus === 'STAY_FOLLOW',
      waitingTime: d.outTime,
      countDay: d.countDay,
      deliverTarget: d.deliverTargetId,
      createDate: d.createDate,
      visitComeDate: d.visitComeDate,
      lastRemark: d.lastRemark,
      noopDay: d.noopDay,
      blCampusId: d.blCampusId,
      studentId: d.studentId,
      customerId: d.cusId,
      customerName: d.cusName,
      phoneNum: d.contact,
      employee: d.employee,
    };
  });
  return { list, totalRecords, page };
};

const loadCusListManage = async ({
  couponUnused,
  stuName,
  name,
  phone,
  school,
  grade,
  entrance,
  isVisit,
  intention,
  cusTag,
  newFollowTime,
  nextFollowTime,
  appointmentTime,
  newFollowTimeEnd,
  nextFollowTimeEnd,
  appointmentTimeEnd,
  visitComeDateStart,
  visitComeDateEnd,
  createDateStart,
  createDateEnd,
  branchId,
  campusId,
  consultantId,
  currentPage,
  pageSize,
  pageOrder,
  pageOrderField,
  source,
  sourceDetail,
}) => {
  const params = {
    couponUnused,
    page: currentPage,
    rows: pageSize,
    studentName: stuName,
    customerName: name,
    contact: phone,
    schoolName: school,
    gradeId: grade,
    resEntranceId: entrance,
    isAppointCome: isVisit === IS_VISIT_YES ? 1 : isVisit === IS_VISIT_NO ? 0 : null,
    intentionOfTheCustomerId: intention,
    customerSign: cusTag,
    recentlyTimeStart: newFollowTime,
    recentlyTimeEnd: newFollowTimeEnd,
    nextFollowTimeStart: nextFollowTime,
    nextFollowTimeEnd: nextFollowTimeEnd,
    meetingTimeStart: appointmentTime,
    meetingTimeEnd: appointmentTimeEnd,
    branchId,
    campusId,
    deliverTarget: consultantId,
    visitComeDateStart,
    visitComeDateEnd,
    createDateStart,
    createDateEnd,
    sord: pageOrder,
    sidx: pageOrderField,
    cusType: source,
    cusOrg: sourceDetail,
  };

  const { records, rows } = await getFollowupCustomersForOrgManager(params);
  const data = rows || [];
  const totalRecords = records || 0;
  const list = data.map((d) => {
    return {
      id: d.cusId,
      stuId: d.studentId,
      studentName: d.studentName,
      grade: d.gradeId,
      gradeName: d.gradeName,
      school: d.schoolId,
      schoolName: d.schoolName,
      name: d.cusName,
      phone: d.contact,
      entrance: d.resEntranceId,
      isVisit: d.isAppointCome,
      intention: d.intentionOfTheCustomerId,
      cusTag: d.customerSign,
      newFollowTime: d.lastFollowUpTime,
      nextFollowTime: getDateByDateTime(d.nextFollowupDate),
      appointmentTime: d.appointmentDate,
      isNew: d.isNew,
      isSigned: d.customerStatus === 'FLOW_AWAY',
      unFetch: d.dealStatus === 'STAY_FOLLOW',
      waitingTime: d.outTime,
      countDay: d.countDay,
      campusId: d.blCampusId,
      deliverTarget: d.deliverTargetId,
      createDate: d.createDate,
      visitComeDate: d.visitComeDate,
      lastRemark: d.lastRemark,
      noopDay: d.noopDay,
    };
  });

  return { list, totalRecords };
};

const loadCusSignedList = async ({
  stuName,
  grade,
  school,
  name,
  phone,
  entrance,
  studyManager,
  studentStatus,
  signTime,
  signTimeEnd,
  currentPage,
  pageSize,
}) => {
  const params = {
    page: currentPage,
    rows: pageSize,
    studentName: stuName,
    gradeId: grade,
    schoolName: school,
    cusName: name,
    contact: phone,
    resEntranceId: entrance,
    studentStatusName: studyManager,
    studentStatus: studentStatus,
    signTimeStart: signTime,
    signTimeEnd: signTimeEnd,
  };

  const { records, rows, page } = await getConsultorSignUpList(params);
  const data = rows || [];
  const totalRecords = records || 0;
  const list = data.map((d) => {
    return {
      id: d.cusId,
      stuId: d.studentId,
      stuName: d.studentName,
      gradeName: d.gradeName,
      schoolName: d.schoolName,
      name: d.cusName,
      phone: d.contact,
      entrance: d.resEntranceId,
      studyManager: d.studyManegerName,
      studentStatus: d.studentStatus,
      days: d.days,
      signTime: d.signTime,
      isSignContract: d.isSignContract,
      canSign: d.canSign,
      blCampusId: d.blCampusId,
      employee: d.employee,
    };
  });

  return { list, totalRecords, page };
};

const loadCusSignedListManage = async ({
  stuName,
  grade,
  school,
  name,
  phone,
  entrance,
  studyManager,
  studentStatus,
  signTime,
  signTimeEnd,
  currentPage,
  pageSize,
  branchId,
  campusId,
  consultantId,
  source,
  sourceDetail,
}) => {
  const params = {
    page: currentPage,
    rows: pageSize,
    studentName: stuName,
    gradeId: grade,
    schoolName: school,
    cusName: name,
    contact: phone,
    resEntranceId: entrance,
    studentStatusName: studyManager,
    studentStatus: studentStatus,
    signTimeStart: signTime,
    signTimeEnd: signTimeEnd,
    brenchId: branchId,
    blCampusId: campusId,
    deliverTarget: consultantId,
    cusType: source,
    cusOrg: sourceDetail,
  };

  const { records, rows } = await getSignedCustomersForOrgManager(params);
  const data = rows || [];
  const totalRecords = records || 0;
  const list = data.map((d) => {
    return {
      id: d.cusId,
      stuId: d.studentId,
      stuName: d.studentName,
      gradeName: d.gradeName,
      schoolName: d.schoolName,
      name: d.cusName,
      phone: d.contact,
      entrance: d.resEntranceId,
      studyManager: d.studyManegerName,
      studentStatus: d.studentStatus,
      days: d.days,
      signTime: d.signTime,
      isSignContract: d.isSignContract,
      blCampusId: d.blCampusId,
    };
  });

  return { list, totalRecords };
};

const loadCusLostList = async ({
  stuName,
  grade,
  school,
  name,
  phone,
  entrance,
  source,
  sourceDetail,
  lostTime,
  currentPage,
  pageSize,
}) => {
  const params = {
    page: currentPage,
    rows: pageSize,
    studentName: stuName,
    gradeId: grade,
    schoolName: school,
    cusName: name,
    contact: phone,
    resEntranceId: entrance,
    cusType: source,
    cusOrg: sourceDetail,
    lostCustomerTimeStart: lostTime,
    lostCustomerTimeEnd: lostTime,
  };

  const { records, rows, page } = await getConsultorLostList(params);
  const data = rows || [];
  const totalRecords = records || 0;
  const list = data.map((d) => {
    return {
      id: d.cusId,
      stuId: d.studentId,
      stuName: d.studentName,
      grade: d.gradeId,
      school: d.schoolName,
      name: d.cusName,
      phone: d.contact,
      entrance: d.resEntranceId,
      source: d.cusTypeName,
      sourceDetail: d.cusOrgName,
      lostTime: d.lostCustomerTime,
      lostReason: d.lostCustomerReason,
      blCampusId: d.blCampusId,
    };
  });

  return { list, totalRecords, page };
};

const loadCusLostListManage = async ({
  stuName,
  grade,
  school,
  name,
  phone,
  entrance,
  source,
  sourceDetail,
  lostTime,
  branchId,
  campusId,
  consultantId,
  currentPage,
  pageSize,
}) => {
  const params = {
    page: currentPage,
    rows: pageSize,
    studentName: stuName,
    gradeId: grade,
    schoolName: school,
    cusName: name,
    contact: phone,
    resEntranceId: entrance,
    cusType: source,
    cusOrg: sourceDetail,
    lostCustomerTimeStart: lostTime,
    lostCustomerTimeEnd: lostTime,
    brenchId: branchId,
    blCampusId: campusId,
    deliverTarget: consultantId,
  };

  const { records, rows } = await getLostCustomersForOrgManager(params);
  const data = rows || [];
  const totalRecords = records || 0;
  const list = data.map((d) => {
    return {
      id: d.cusId,
      stuId: d.studentId,
      stuName: d.studentName,
      grade: d.gradeId,
      school: d.schoolName,
      name: d.cusName,
      phone: d.contact,
      entrance: d.resEntranceId,
      source: d.cusTypeName,
      sourceDetail: d.cusOrgName,
      lostTime: d.lostCustomerTime,
      lostReason: d.lostCustomerReason,
      blCampusId: d.blCampusId,
    };
  });

  return { list, totalRecords };
};

const getDateByDateTime = (dateTime) => {
  if (!dateTime) {
    return null;
  }
  return dateTime.slice(0, 10);
};

const loadTransToCampusDict = async () => {
  const dict = [{ val: '1', text: 'xx学习中心' }];
  return dict;
};

const saveTransferCampus = async (params) => {
  await submitStudentTransferApply(params);
  return true;
};

const obtainCustomer = async ({ id, stuId }) => {
  await obtainCustomerInternet({ customerId: id, studentId: stuId });
  return true;
};

const reObtainCustomer = async ({ id, stuId }) => {
  await obtainCustomerAgain({ customerId: id, studentId: stuId });
  return true;
};

const confirmLostCus = async ({ id, stuId }) => {
  await updateCustomerStudentStatus({ cusId: id, studentId: stuId, customerStatus: 'INVALID' }, true);
  return true;
};

const loadTransferApplyingList = async (params) => {
  const data = await listStudentTransferApplyBySelf(params);
  if (data.resultCode !== 0) {
    throw new Error(data.resultMessage);
  }
  return data;
};

const loadTransferOutAuditList = async (params) => {
  const data = await listStudentTransferApplyForCampusAudit(params);
  if (data.resultCode !== 0) {
    throw new Error(data.resultMessage);
  }
  return data;
};

const getCrmResourceVisibility = async (params) => {
  const res = await crmResourceVisibility(params);
  if (res.resultCode !== 0) {
    throw new Error(res.resultMessage);
  }
  return !!!res.data.hidden;
};

export default {
  loadCusList,
  loadCusSignedList,
  loadCusLostList,
  loadPhoneCusList,
  loadTransToCampusDict,
  saveTransferCampus,
  obtainCustomer,
  reObtainCustomer,
  confirmLostCus,
  loadCusListManage,
  loadCusSignedListManage,
  loadCusLostListManage,
  loadTransferApplyingList,
  loadTransferOutAuditList,
  getCrmResourceVisibility,
  loadPhoneCusListManage,
  loadFollowingRecordNew,
  postSignForCustomerComeOn,
  postSignForCustomerComeOnWithoutAppoint,
  canSign,
};
