import React from 'react';
import { connect } from 'dva';
import styles from './styles.less';
import Section from 'COMPONENT/Section';
import CusList from './CusListManage';
import CusSignedList from '../FollowCustomer/CusSignedList';
import CusLostList from '../FollowCustomer/CusLostList';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import DropdownMenu from 'COMPONENT/DropdownMenu';
import {
  CUS_LIST_FOLLOW,
  CUS_LIST_SIGNED,
  CUS_LIST_LOST,
  cusListManageDict as cusListDict,
} from '../FollowCustomer/dicts';
import Page from 'COMPONENT/Page';

class FollowCustomerManage extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 加载状态

      activeCusList: CUS_LIST_FOLLOW, // {boolean} 当前激活状态的客户列表
      searchingState: {},
      isManager: true,
      defaultSearchConditions: this.props.params ? this.props.params.defaultSearchConditions : {},
    };
    this.cusLists = {}; // {object} 客户列表信息，{ id: { ref, inited } }
    cusListDict.forEach((d) => {
      this.cusLists[d.val] = { ref: null, inited: false };
    });
  }

  componentDidMount() {
    this.setState({ loading: true });

    let { brenchId, campusId, organizationType } = this.props;
    const searchingState = {
      GROUNP: {
        branchId: '',
        showBranchSelector: true,
        branchSelectorDisabled: false,
        showCampusSelector: true,
        showConsultantSelector: true,
      },
      BRENCH: {
        branchId: brenchId,
        showBranchSelector: true,
        branchSelectorDisabled: true,
        showCampusSelector: true,
        showConsultantSelector: true,
      },
      CAMPUS: {
        branchId: brenchId,
        campusId: campusId,
        showBranchSelector: true,
        branchSelectorDisabled: true,
        showCampusSelector: true,
        campusSelectorDisabled: true,
        showConsultantSelector: true,
      },
    };
    const currentSearchState = searchingState[organizationType];

    this.setState({ searchingState: currentSearchState || {} });
    setTimeout(() => {
      Promise.all([this.cusLists[CUS_LIST_FOLLOW].ref.initContent()]).then(() => {
        this.cusLists[CUS_LIST_FOLLOW].inited = true;
        this.setState({ loading: false });
      });
    }, 0);
  }

  getCusListSecTitle = () => {
    const { activeCusList } = this.state;
    const menus = cusListDict.map((c) => {
      return { text: c.text, onClick: () => this.changeCusList(c.val), hide: c.val === activeCusList };
    });
    return (
      <div className={styles.cusListDropdownMenu}>
        <DropdownMenu menus={menus} text={cusListDict.find((cd) => cd.val === activeCusList).text} hasArrow />
      </div>
    );
  };

  changeCusList = (type) => {
    if (this.cusLists[type].inited) {
      this.setState({ activeCusList: type });
    } else {
      this.setState({ loading: true });
      this.cusLists[type].ref.initContent().then(() => {
        this.cusLists[type].inited = true;
        this.setState({ loading: false, activeCusList: type });
      });
    }
  };

  render() {
    const { dispatch } = this.props;
    const { activeCusList, loading, searchingState, isManager, defaultSearchConditions } = this.state;
    return (
      <Page route={this.props.route} title={false} className={styles.page}>
        <Section title={this.getCusListSecTitle()} loading={loading}>
          {cusListDict.map((c) => {
            let listComp;
            if (c.val === CUS_LIST_FOLLOW) {
              listComp = (
                <CusList
                  ref={(comp) => {
                    if (comp) {
                      this.cusLists[c.val].ref = getWrappedInstance(comp);
                    }
                  }}
                  dispatch={dispatch}
                  {...searchingState}
                  isManager={isManager}
                  defaultSearchConditions={defaultSearchConditions}
                />
              );
            } else if (c.val === CUS_LIST_SIGNED) {
              listComp = (
                <CusSignedList
                  ref={(comp) => {
                    if (comp) {
                      this.cusLists[c.val].ref = getWrappedInstance(comp);
                    }
                  }}
                  dispatch={dispatch}
                  {...searchingState}
                  isManager={isManager}
                />
              );
            } else if (c.val === CUS_LIST_LOST) {
              listComp = (
                <CusLostList
                  ref={(comp) => (this.cusLists[c.val].ref = comp)}
                  dispatch={dispatch}
                  {...searchingState}
                  isManager={isManager}
                />
              );
            }
            return (
              <div key={c.val} style={{ display: c.val === activeCusList ? 'block' : 'none' }}>
                {listComp}
              </div>
            );
          })}
        </Section>
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    userId: state.account.user.userId,
    organizationType: state.account.user.organizationType,
    brenchId: state.account.user.brenchId,
    campusId: state.account.user.campusId,
    params: state.main.pageParams['/ConsultantFollowCustomerManage'],
  };
};
export default connect(mapStateToProps)(FollowCustomerManage);
