import React from 'react';
import styles from './styles.less';
import { List, DatePicker, Select, Section, SearchBar, Input, ListAction } from 'COMPONENT';
import { CUS_LIST_LOST } from '../dicts';
import loaders from '../loaders';
import moment from 'moment';
import { dictAddOptionTitle, loadCommonDict } from '../../../../utils/dict/index';
import { DATE_FORMAT } from 'COMPONENT/utils/form';
import { loadConsultantDict } from 'PAGE/common/loaders';
import { loadBranchDict, loadCampusDict } from 'UTIL/dict';
import { connect } from 'dva';
import { getSwitchConfigsByCodesAndCampusIds } from 'SERVICE/common';
import { loadSourceDetailDict } from 'PAGE/common/loaders';
import ModalCallDetail from 'PAGE/consultant/ModalCallDetail';

const today = moment().format(DATE_FORMAT);

class CampusFollowSec extends React.Component {
  constructor(props) {
    super(props);

    const { branchId, campusId, userId } = this.props;

    this.initListSearchKeys = {
      consultant: null, // {string} 查询条件，咨询师
      stuName: null, // {string} 查询条件，学生姓名
      phone: null, // {string} 查询条件，电话
      grade: null, // {string} 查询条件，年级
      school: null, // {string} 查询条件，学校
      startDate: today, // {string} 查询条件，开始日期
      endDate: today, // {string} 查询条件，结束日期
      branchId: branchId ? branchId : '',
      campusId: campusId ? campusId : '',
      resEntranceId: null,
      cusType: null,
      cusOrg: null,
      sourceDetailDict: [],
    };

    if (userId) {
      this.initListSearchKeys.consultant = userId;
    }

    this.state = {
      loading: false, // {boolean} 加载状态
      searchExpand: false, // {boolean} 搜索栏展开状态

      ...this.initListSearchKeys,

      consultantDict: [], // {object[]} 字典，咨询师
      gradeDict: [], // {object[]} 字典，年级

      list: [], // {object[]} 列表数据，[{ stuName, grade, schoolName, name, contract, time, remark }]
      listSearchKeys: this.initListSearchKeys, // {object} 列表数据的查询关键字
      count: 0, // {number} 跟进数量
      countTotal: 0, // {number} 跟进数量，总数

      branchId: branchId ? branchId : '',
      campusId: campusId ? campusId : '',
      brenchDict: [],
      blCampusDict: [],
      resEntranceDict: [],

      modalCallDetailVisible: false,
    };
    this.audioAuth = props.btnAuths.indexOf('CALL_AUTH_PLAY_AUDIO') > -1;
    this.callDetail = null;
  }

  componentDidMount() {
    this.initContent();
  }

  initContent = async () => {
    const { userId, branchId, campusId } = this.props;
    const { startDate, endDate } = this.state;
    this.setState({ loading: true });

    const brenchDict = await loadBranchDict();
    const blCampusDict = !branchId ? [] : await loadCampusDict(branchId);
    const consultantDict = await loadConsultantDict(branchId && campusId ? { orgId: campusId } : '');
    const consultant = !!this.state.consultant
      ? this.state.consultant
      : consultantDict[0]
      ? consultantDict[0].val
      : undefined;
    if (consultantDict.length === 0) {
      this.setState({
        loading: false,
      });
      return;
    }
    const [listData, gradeDict, resEntranceDict, resTypeDict] = await Promise.all([
      loaders.loadCampusFollowData({ ...this.getListParams(), consultant }),
      loadCommonDict('STUDENT_GRADE'),
      loadCommonDict('RES_ENTRANCE'),
      loadCommonDict('RES_TYPE'),
    ]);
    const { list, count, countTotal } = listData;
    this.setState({
      loading: false,
      consultant,
      consultantDict,
      gradeDict,
      list,
      count,
      countTotal,
      brenchDict,
      blCampusDict,
      campusId,
      resEntranceDict,
      resTypeDict,
    });
  };

  getBlCampusDict = async (branchId, campusId) => {
    this.setState({ loading: true });
    const blCampusDict = !branchId ? [] : await loadCampusDict(branchId);
    campusId = campusId || blCampusDict[0].val || '';
    const consultantDict = !campusId ? [] : await loadConsultantDict({ orgId: campusId });
    this.setState({
      blCampusDict,
      campusId,
      consultantDict,
      consultant: '',
      loading: false,
    });
  };

  onCampusChange = async (campusId) => {
    this.setState({ loading: true });
    const consultantDict = !campusId ? [] : await loadConsultantDict({ orgId: campusId });
    this.setState({ consultantDict, consultant: '', loading: false });
  };

  getListParams = () => {
    const {
      listSearchKeys: {
        consultant,
        stuName,
        phone,
        grade,
        school,
        startDate,
        endDate,
        branchId,
        campusId,
        resEntranceId,
      },
    } = this.state;

    return {
      consultant,
      stuName,
      phone,
      grade,
      school,
      startDate,
      endDate: this.getNextDate(endDate),
      branchId,
      campusId,
      resEntranceId,
    };
  };

  search = () => {
    const {
      consultant,
      stuName,
      phone,
      grade,
      school,
      startDate,
      endDate,
      branchId,
      campusId,
      resEntranceId,
      cusType,
      cusOrg,
    } = this.state;
    const listSearchKeys = {
      consultant,
      stuName,
      phone,
      grade,
      school,
      startDate,
      endDate: this.getNextDate(endDate),
      branchId,
      campusId,
      resEntranceId,
      cusType,
      cusOrg,
    };
    this.setState({ loading: true });
    loaders.loadCampusFollowData({ ...this.getListParams(), ...listSearchKeys }).then(({ list, count, countTotal }) => {
      this.setState({
        loading: false,
        listSearchKeys,
        list,
        count,
        countTotal,
      });
    });
  };

  reset = () => {
    this.setState(this.initListSearchKeys);
  };

  getNextDate = (date) => {
    const nextEndDate = moment(date || new Date())
      .add(1, 'days')
      .format(DATE_FORMAT);
    return nextEndDate;
  };

  getDisabledStartDate = (startDate) => {
    const { endDate } = this.state;
    if (!startDate || !endDate) {
      return false;
    }
    return moment(startDate).valueOf() > moment(endDate).valueOf();
  };

  getDisabledEndDate = (endDate) => {
    const { startDate } = this.state;
    if (!endDate || !startDate) {
      return false;
    }
    return moment(endDate).valueOf() < moment(startDate).valueOf();
  };

  onRefresh = (params) => {
    this.search();
    this.goDetail(params);
  };

  goDetail = (params) => {
    const { dispatch } = this.props;
    params = {
      ...params,
      type: this.props.type,
      onRefresh: this.onRefresh,
      customerId: params.id,
      studentId: params.stuId,
    };
    dispatch({
      type: 'main/openPage',
      payload: {
        path: '/CommonCusDetailPage',
        params,
        refresh: true,
        mirrorId: `ConsultantFollowCustomer/CusListSec/${this.props.type}/${params.id}/${params.stuId}/${
          params.detailEditable ? 'detail' : 'follow'
        }`,
      },
    });
  };

  signNew = async ({ customerId, studentId }) => {
    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path: `/signContractPro/${JSON.stringify({
          studentId,
          customerId,
          flag: true,
          signEntrance: 'CONSULTANT',
        })}`,
        refresh: true,
      },
    });
  };

  switchModalCallDetail = (detail) => {
    this.setState((prevState) => {
      this.callDetail = prevState.modalCallDetailVisible ? null : detail;
      return { modalCallDetailVisible: !prevState.modalCallDetailVisible };
    });
  };

  render() {
    const { userId, branchSelectorDisabled, campusSelectorDisabled, btnAuths } = this.props;
    const hasConsultWorkbrench = this.props.callCenterConfig.hasConsultWorkbrench;
    const alreadyBind = this.props.alreadyBind;

    const {
      consultant,
      stuName,
      phone,
      grade,
      school,
      startDate,
      endDate,
      list,
      count,
      countTotal,
      consultantDict,
      gradeDict,
      loading,
      searchExpand,
      branchId,
      campusId,
      brenchDict,
      blCampusDict,
      resEntranceDict,
      resEntranceId,
      cusType,
      cusOrg,
    } = this.state;

    const columns = !!userId
      ? [
          { title: '跟进人', dataIndex: 'createUsername', width: 80 },
          { title: '学生姓名', dataIndex: 'stuName', width: 80 },
          { title: '年级', dataIndex: 'grade', width: 80 },
          { title: '学校', dataIndex: 'schoolName', width: 80 },
          { title: '客户电话', dataIndex: 'contract', width: 125 },
          { title: '意向度', dataIndex: 'intentionName', width: 80 },
          { title: '标签', dataIndex: 'customerSignName', width: 80 },
          { title: '跟进记录', dataIndex: 'remark', width: 155 },
          { title: '跟进时间', dataIndex: 'time', width: 150 },
          { title: '预约上门日期', dataIndex: 'meetingTime', width: 125 },
          { title: '下次跟进时间', dataIndex: 'nextTime', width: 125 },
          {
            title: '操作',
            key: 'action',
            width: 125,
            render: (text, record, index) => {
              let config = [];
              // 当前业务只存在一条通话记录
              const callRecord = (record.callRecordVos || [])[0];
              hasConsultWorkbrench &&
                config.push({
                  text: '通话详情',
                  onClick: () => {
                    this.switchModalCallDetail(callRecord);
                  },
                  disabled: !callRecord,
                });

              if (record.customerStatus === 'VALID' && record.dealStatus === 'FOLLOWING') {
                config.push({
                  text: '跟进',
                  onClick: () => {
                    this.goDetail({
                      id: record.customerId,
                      stuId: record.studentId,
                      detailEditable: true,
                      followEditable: true,
                      deliverTarget: record.deliverTarget,
                      customerId: record.customerId,
                      studentId: record.studentId,
                      studentName: record.studentName,
                      phoneNum: record.phoneNum,
                    });
                  },
                });
                // 新旧接口的isAppointCome类型不统一(number, string)
                if (record.isAppointCome == 1 && this.props.btnAuths.includes('FOLLOW_CUSTOMER_ADDCONTRACT')) {
                  config.push({
                    text: '签单',
                    onClick: () => {
                      this.signNew(record);
                    },
                  });
                }
              }

              return <ListAction config={config} direction="column" />;
            },
          },
        ]
      : [
          { title: '跟进人', dataIndex: 'createUsername', width: 80 },
          { title: '学生姓名', dataIndex: 'stuName', width: 80 },
          { title: '年级', dataIndex: 'grade', width: 80 },
          { title: '学校', dataIndex: 'schoolName', width: 80 },
          { title: '电话', dataIndex: 'contract', width: 125 },
          { title: '意向度', dataIndex: 'intentionName', width: 80 },
          { title: '标签', dataIndex: 'customerSignName', width: 80 },
          { title: '跟进记录', dataIndex: 'remark', width: 280 },
          { title: '跟进时间', dataIndex: 'time', width: 150 },
          { title: '预约上门日期', dataIndex: 'meetingTime', width: 125 },
          { title: '下次跟进时间', dataIndex: 'nextTime', width: 125 },
          {
            title: '操作',
            key: 'action',
            width: 125,
            render: (text, record, index) => {
              let config = [];
              // 当前业务只存在一条通话记录
              const callRecord = (record.callRecordVos || [])[0];
              hasConsultWorkbrench &&
                config.push({
                  text: '通话详情',
                  onClick: () => {
                    this.switchModalCallDetail(callRecord);
                  },
                  disabled: !callRecord,
                });
              return <ListAction config={config} direction="column" />;
            },
          },
        ];

    return (
      <Section title="学习中心跟进情况" loading={loading}>
        <div>
          <div className="flexWrap">
            {!!userId ? (
              <SearchBar
                expand={searchExpand}
                onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
                onSearch={this.search}
                onReset={this.reset}
              >
                <DatePicker
                  value={startDate}
                  onChange={(val) => this.setState({ startDate: val })}
                  placeholder="跟进日期（起）"
                  disabledDate={this.getDisabledStartDate}
                />
                <DatePicker
                  value={endDate}
                  onChange={(val) => this.setState({ endDate: val })}
                  placeholder="跟进日期（止）"
                  disabledDate={this.getDisabledEndDate}
                />
                <Input
                  placeholder="学生姓名"
                  value={stuName}
                  onChange={(e) => this.setState({ stuName: e.target.value })}
                />
                <Input placeholder="电话" value={phone} onChange={(e) => this.setState({ phone: e.target.value })} />
                <Select
                  placeholder="年级"
                  dict={gradeDict}
                  value={grade}
                  onChange={(val) => this.setState({ grade: val })}
                />
                <Input placeholder="学校" value={school} onChange={(e) => this.setState({ school: e.target.value })} />
                <Select
                  placeholder="资源入口"
                  dict={resEntranceDict}
                  value={resEntranceId}
                  onChange={(val) => this.setState({ resEntranceId: val })}
                />
                <Select
                  placeholder="资源来源"
                  dict={this.state.resTypeDict}
                  value={cusType}
                  onChange={(val) => {
                    this.setState({ cusType: val });
                    this.setState({ cusOrg: null });
                    if (val) {
                      loadSourceDetailDict(val).then((dict) => {
                        this.setState({
                          sourceDetailDict: dict,
                        });
                      });
                    } else {
                      this.setState({
                        sourceDetailDict: [],
                      });
                    }
                  }}
                />
                <Select
                  placeholder="资源细分"
                  value={cusOrg}
                  onChange={(val) => this.setState({ cusOrg: val })}
                  dict={this.state.sourceDetailDict}
                />
              </SearchBar>
            ) : (
              <SearchBar
                expand={searchExpand}
                onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
                onSearch={this.search}
                onReset={this.reset}
              >
                <Select
                  placeholder="分公司"
                  showSearch
                  allowClear={false}
                  dict={brenchDict}
                  value={branchId}
                  onChange={(val) => {
                    this.setState({ branchId: val });
                    this.getBlCampusDict(val);
                  }}
                  disabled={branchSelectorDisabled}
                />
                <Select
                  placeholder="学习中心"
                  showSearch
                  allowClear={false}
                  dict={blCampusDict}
                  value={campusId}
                  onChange={(val) => {
                    this.setState({ campusId: val });
                    this.onCampusChange(val);
                  }}
                  disabled={campusSelectorDisabled}
                />
                <Select
                  placeholder="咨询师"
                  showSearch
                  dict={dictAddOptionTitle(consultantDict)}
                  value={consultant}
                  onChange={(val) => this.setState({ consultant: val })}
                />
                <DatePicker
                  value={startDate}
                  onChange={(val) => this.setState({ startDate: val })}
                  placeholder="跟进日期（起）"
                  disabledDate={this.getDisabledStartDate}
                />
                <DatePicker
                  value={endDate}
                  onChange={(val) => this.setState({ endDate: val })}
                  placeholder="跟进日期（止）"
                  disabledDate={this.getDisabledEndDate}
                />
                <Input
                  placeholder="学生姓名"
                  value={stuName}
                  onChange={(e) => this.setState({ stuName: e.target.value })}
                />
                <Input placeholder="电话" value={phone} onChange={(e) => this.setState({ phone: e.target.value })} />
                <Select
                  placeholder="年级"
                  dict={gradeDict}
                  value={grade}
                  onChange={(val) => this.setState({ grade: val })}
                />
                <Input placeholder="学校" value={school} onChange={(e) => this.setState({ school: e.target.value })} />
                <Select
                  placeholder="资源入口"
                  dict={resEntranceDict}
                  value={resEntranceId}
                  onChange={(val) => this.setState({ resEntranceId: val })}
                />
                <Select
                  placeholder="资源来源"
                  dict={this.state.resTypeDict}
                  value={cusType}
                  onChange={(val) => {
                    this.setState({ cusType: val });
                    this.setState({ cusOrg: null });
                    if (val) {
                      loadSourceDetailDict(val).then((dict) => {
                        this.setState({
                          sourceDetailDict: dict,
                        });
                      });
                    } else {
                      this.setState({
                        sourceDetailDict: [],
                      });
                    }
                  }}
                />
                <Select
                  placeholder="资源细分"
                  value={cusOrg}
                  onChange={(val) => this.setState({ cusOrg: val })}
                  dict={this.state.sourceDetailDict}
                />
              </SearchBar>
            )}

            <div className="flexInnerFit">
              <div className={styles.list}>
                <List dataSource={list} columns={columns} />
                <div className={styles.tips}>
                  <span className={styles.count}>{`${count}`}</span>
                  <span style={{ paddingLeft: '10px' }}>跟进学生：</span>
                  <span className={styles.count}>{`${countTotal}`}</span>
                  <span>跟进记录：</span>
                </div>
              </div>
            </div>
          </div>
        </div>
        {this.state.modalCallDetailVisible && (
          <ModalCallDetail data={this.callDetail} onClose={this.switchModalCallDetail} audioAuth={this.audioAuth} />
        )}
      </Section>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    activeTabId: state.main.activeTabId,
    btnAuths: state.account.btnAuths,
    callCenterConfig: state.account.callCenterConfig,
    alreadyBind: state.callCenter.alreadyBind,
  };
};

export default connect(mapStateToProps, null, null, { withRef: true })(CampusFollowSec);
