import React from 'react';
import styles from './styles.less';
import {
  TIME_RANGE_TODAY,
  TIME_RANGE_WEEK,
  TIME_RANGE_MONTH,
  TIME_RANGE_QUARTER,
  timeRange2Dict,
} from 'PAGE/consultant/common/dicts';
import { loadRankingData } from 'PAGE/consultant/common/loaders';
import Tabs from 'COMPONENT/Tabs';
import { getDictTextByVal } from 'UTIL/dict';
import moment from 'moment';
import AchievementBarChart from 'PAGE/consultant/common/AchievementChart';
import AchievementRankingItem from 'PAGE/consultant/common/AchievementChart/RankingItem';
import Section from 'COMPONENT/Section';
import CampusSelector from '../../common/CampusSelector';

const { Tab } = Tabs;

class RankingSec extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 加载状态
      activeTabKey: TIME_RANGE_TODAY, // {string} 当前激活状态的页签ID
      data: [], // {object[]} 图表数据，[{ userId, name, campusName, num, asc, desc, rank }]
      orgId: this.props.showSearch ? this.props.campusId || '' : '',
    };
  }

  componentDidMount() {
    this.initContent();
  }

  initContent = async () => {
    const { activeTabKey, orgId } = this.state;
    this.setState({ loading: true });
    const data = await loadRankingData({ timeRange: activeTabKey, orgId });
    this.setState({ loading: false, data });
  };

  onChange = async (key, orgId) => {
    this.setState({ loading: true });
    const data = await loadRankingData({ timeRange: key, orgId });
    this.setState({ loading: false, data, activeTabKey: key, orgId });
  };

  onTabChange = (key) => {
    const { orgId } = this.state;
    return this.onChange(key, orgId);
  };

  onSelectorChange = (orgId) => {
    const { activeTabKey: key } = this.state;
    return this.onChange(key, orgId);
  };

  render() {
    const {
      showSearch,
      branchId,
      campusId,
      branchSelectorDisabled,
      campusSelectorDisabled,
      showBranchSelector,
      showCampusSelector,
      showConsultantSelector,
      consultantId,
      consultantSelectorDisabled,
    } = this.props;
    const { activeTabKey, data, loading } = this.state;

    return (
      <div className={styles.wrap}>
        <Section title="分公司业绩排名" loading={loading}>
          {showSearch && (
            <div>
              <CampusSelector
                branchSelectorDisabled={branchSelectorDisabled}
                campusSelectorDisabled={campusSelectorDisabled}
                branchId={branchId}
                campusId={campusId}
                showBranchSelector={showBranchSelector}
                showCampusSelector={showCampusSelector}
                showConsultantSelector={showConsultantSelector}
                consultantId={consultantId}
                consultantSelectorDisabled={consultantSelectorDisabled}
                onChange={this.onSelectorChange}
              />
            </div>
          )}
          <div className={styles.wrap}>
            <Tabs onChange={this.onTabChange} navAlign="center" activeKey={activeTabKey} style={{ marginTop: -10 }}>
              <Tab title={getDictTextByVal(timeRange2Dict, TIME_RANGE_TODAY)} value={TIME_RANGE_TODAY} />
              <Tab title={getDictTextByVal(timeRange2Dict, TIME_RANGE_WEEK)} value={TIME_RANGE_WEEK} />
              <Tab title={getDictTextByVal(timeRange2Dict, TIME_RANGE_MONTH)} value={TIME_RANGE_MONTH} />
              <Tab title={getDictTextByVal(timeRange2Dict, TIME_RANGE_QUARTER)} value={TIME_RANGE_QUARTER} />
            </Tabs>
            <div className={styles.main}>
              <div className={styles.left}>
                <AchievementBarChart
                  data={data.map((d) => {
                    return { ...d, showNum: d.isMe || d.isMyCampus };
                  })}
                  showNumCount={10}
                />
              </div>
              <div className={styles.right}>
                <div className={styles.subTitle}>本学习中心咨询师</div>
                <div className={styles.ranks}>
                  {data
                    .filter((da) => da.isMyCampus)
                    .map((d, idx) => {
                      return (
                        <div key={idx} className={styles.rank}>
                          <div className={styles.consultantName}>{d.name}</div>
                          <AchievementRankingItem data={d} />
                        </div>
                      );
                    })}
                </div>
              </div>
            </div>
          </div>
        </Section>
      </div>
    );
  }
}

export default RankingSec;
