import React from 'react';
import styles from './styles.less';
import loaders from '../loaders';
import followCustomerLoaders from '../../FollowCustomer/loaders';
import Section from 'COMPONENT/Section';
import VisitCusSchedule from 'PAGE/consultant/common/VisitCusSchedule';
import WeekSelect from 'PAGE/consultant/common/WeekSelect';
import VisitRadioBlock from '../VisitRadioBlock';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { getWeekRange } from 'PAGE/consultant/common/VisitCusSchedule/utils';
import moment from 'moment';
import CampusSelector from '../../common/CampusSelector';
import { connect } from 'dva';

const { startDate: initStartDate, endDate: initEndDate } = getWeekRange(moment());
/*const initStartDate = '2018-03-19'
const initEndDate = '2018-03-25'*/
const blankVar = '';

class VisitCusSec extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 内容加载状态
      startDate: initStartDate, // {string} 开始日期
      endDate: initEndDate, // {string} 结束日期
      list: [], // {object[]} 列表数据，
      // {status, date, startTime, endTime, name, contract, studentName, gradeName, schoolName, headerInfo, title, infos, moreInfos}
      visitRadioData: [], // 上门率数据，[{ name, visited, appointment }]
      orgId: this.props.showSearch ? this.props.campusId || '' : '',
      branchId: this.props.showSearch ? this.props.branchId || '' : '',
      consultantId: this.props.showSearch ? this.props.consultantId || '' : '',
    };
  }

  componentDidMount() {
    this.initContent();
  }

  initContent = async () => {
    const { startDate, endDate, orgId, branchId, consultantId } = this.state;
    this.setState({ loading: true });
    const { list, visitRadioData } = await this.loadData({ startDate, endDate, orgId, branchId, consultantId });
    this.setState({ loading: false, list, visitRadioData });
  };

  loadData = async ({ startDate, endDate, orgId, consultantId, branchId }) => {
    const [{ list }, visitRadioData] = await Promise.all([
      loaders.loadVisitCusInfo({ startDate, endDate, orgId, consultantId }),
      followCustomerLoaders.loadPhoneCusListManage({ startDate, endDate, campusId: orgId, consultantId, branchId }),
    ]);
    return { list, visitRadioData };
  };

  goWeek = ({ startDate, endDate, orgId, consultantId, branchId }) => {
    this.setState({ loading: true });
    this.loadData({ startDate, endDate, orgId, consultantId, branchId }).then(
      ({ list, visitRadioData }) => {
        this.setState({ loading: false, startDate, endDate, orgId, consultantId, branchId, list, visitRadioData });
      },
      (e) => {
        this.setState({ loading: false });
        showMessage(e.text || '预约上门客户数据加载失败');
      },
    );
  };

  onWeekChange = ({ startDate, endDate }) => {
    const { orgId, consultantId } = this.state;
    this.goWeek({ startDate, endDate, orgId, consultantId });
  };

  onSelectorChange = (orgId, consultantId, branchId) => {
    const { startDate, endDate } = this.state;
    this.goWeek({ startDate, endDate, orgId, consultantId, branchId });
  };

  search = () => {
    this.initContent();
  };

  onVisitRadioClick = (item) => {
    const { startDate, endDate } = this.state;
    const params = {
      [item.startDate]: startDate,
      [item.endDate]: endDate,
      campusId: this.state.orgId,
      branchId: this.state.branchId,
      consultantId: this.state.consultantId,
    };
    if (item.resEntranceId) {
      params['entrance'] = item.resEntranceId;
    }

    const { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: {
        path: '/ConsultantFollowCustomerManage',
        params: {
          defaultSearchConditions: params,
        },
        refresh: true,
      },
    });
  };

  render() {
    const {
      showSearch,
      branchId,
      campusId,
      branchSelectorDisabled,
      campusSelectorDisabled,
      showBranchSelector,
      showCampusSelector,
      showConsultantSelector,
      consultantId,
      consultantSelectorDisabled,
    } = this.props;
    const { loading, startDate, endDate, list, visitRadioData } = this.state;
    return (
      <Section title="学习中心预约上门情况" loading={loading}>
        {showSearch && (
          <div>
            <CampusSelector
              branchSelectorDisabled={branchSelectorDisabled}
              campusSelectorDisabled={campusSelectorDisabled}
              branchId={branchId}
              campusId={campusId}
              showBranchSelector={showBranchSelector}
              showCampusSelector={showCampusSelector}
              showConsultantSelector={showConsultantSelector}
              consultantId={consultantId}
              consultantSelectorDisabled={consultantSelectorDisabled}
              onChange={this.onSelectorChange}
            />
          </div>
        )}
        <div className={styles.wrap}>
          <div className={styles.toolbar}>
            <WeekSelect value={{ startDate, endDate }} onChange={this.onWeekChange} />
          </div>
          <div style={{ flex: 1, display: 'flex' }}>
            <div className={styles.left}>
              {showSearch ? (
                <VisitCusSchedule
                  startDate={startDate}
                  endDate={endDate}
                  list={list}
                  onRefresh={this.search}
                  isManager={true}
                />
              ) : (
                <VisitCusSchedule
                  startDate={startDate}
                  endDate={endDate}
                  list={list}
                  onRefresh={this.search}
                  isManager={true}
                />
              )}
            </div>
            <div className={styles.vrContainer}>
              <div className={styles.vr}></div>
            </div>
            <div className={styles.right}>
              <VisitRadioBlock data={visitRadioData} onClick={this.onVisitRadioClick} />
            </div>
          </div>
        </div>
      </Section>
    );
  }
}

export default connect()(VisitCusSec);
