import React from 'react';
import Page from 'COMPONENT/Page';
import styles from './styles.less';
import VisitCusSec from './VisitCusSec';
import CampusFollowSec from './CampusFollowSec';
import CallRecordsSec from '../common/CallRecordsSec';
import RankingSec from './RankingSec';
import Loading from 'COMPONENT/Loading';
import { checkHasCampus } from 'PAGE/consultant/common/loaders';
import { connect } from 'dva';
import SaleFunnelSec from '../DataBoard/SaleFunnelSec';
import { loadBranchDict, loadCampusDict } from 'UTIL/dict';

class MasterDataBoard extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false, // {boolean} 页面加载状态
      canShow: null, // {boolean} 页面是否有内容可显示
      errorTips: null, // {boolean} 页面无内容时的提示
      searchingState: {},
    };
  }

  initSearchState = async () => {
    const { brenchId, campusId, organizationType } = this.props;
    this.setState({ loading: true });
    const searchingState = {
      GROUNP: {
        showSearch: true,
        branchSelectorDisabled: false,
        campusSelectorDisabled: false,
        showConsultantSelector: true,
      },
      BRENCH: {
        showSearch: true,
        branchId: brenchId,
        branchSelectorDisabled: true,
        campusSelectorDisabled: false,
        showConsultantSelector: true,
      },
      CAMPUS: {
        showSearch: true,
        branchId: brenchId,
        campusId: campusId,
        branchSelectorDisabled: true,
        campusSelectorDisabled: true,
        showConsultantSelector: true,
      },
    };
    const currentSearchState = searchingState[organizationType];

    if (!!!currentSearchState.branchId) {
      const brenchDict = await loadBranchDict();
      currentSearchState.branchId = brenchDict[0] ? brenchDict[0].val : '';
    }
    if (!!currentSearchState.branchId && !!!currentSearchState.campusId) {
      const campusDict = await loadCampusDict(currentSearchState.branchId);
      currentSearchState.campusId = campusDict[0] ? campusDict[0].val : '';
    }

    this.setState({ searchingState: currentSearchState, canShow: true });
    this.setState({ loading: false });
  };

  componentDidMount() {
    // this.setState({ loading: true })
    // checkHasCampus().then(({ canShow, errorTips }) => {
    //   this.setState({ loading: false, canShow, errorTips })
    // }, e => {
    //   this.setState({ loading: false, canShow: false, errorTips: e.text || '数据加载错误' })
    // })
    this.initSearchState();
  }

  render() {
    const { dispatch } = this.props;
    const hasConsultWorkbrench = this.props.callCenterConfig.hasConsultWorkbrench;
    const alreadyBind = this.props.alreadyBind;
    const { loading, canShow, errorTips, searchingState } = this.state;

    const callRecordsSecProps = {
      branchSelectorVisible: true,
      campusSelectorVisible: true,
      branchSelectorDisabled: searchingState.branchSelectorDisabled,
      campusSelectorDisabled: searchingState.campusSelectorDisabled,
      branchId: searchingState.branchId,
      campusId: searchingState.campusId,
    };

    return (
      <Page loading={loading} route={this.props.route} title={false} className={canShow ? styles.page : null}>
        {canShow && (
          <div>
            <div className={styles.sec}>
              <SaleFunnelSec
                {...searchingState}
                allowCampusSelectorClear={true}
                showConsultantSelector={true}
                dispatch={this.props.dispatch}
              />
            </div>
            <div className={styles.sec}>
              <VisitCusSec {...searchingState} />
            </div>
            <div className={styles.sec}>
              <CampusFollowSec {...searchingState} dispatch={dispatch} />
            </div>
            <div className={styles.sec}>{hasConsultWorkbrench && <CallRecordsSec {...callRecordsSecProps} />}</div>
            <div className={styles.sec}>
              <RankingSec />
            </div>
          </div>
        )}
        {errorTips && <div className={styles.tips}>{errorTips}</div>}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    userId: state.account.user.userId,
    organizationType: state.account.user.organizationType,
    organizationId: state.account.user.organizationId,
    brenchId: state.account.user.brenchId,
    campusId: state.account.user.campusId,
    callCenterConfig: state.account.callCenterConfig,
    alreadyBind: state.callCenter.alreadyBind,
  };
};
export default connect(mapStateToProps)(MasterDataBoard);
