import { TIME_RANGE_WEEK, TIME_RANGE_MONTH, TIME_RANGE_QUARTER, timeRangeDict, cusStatusDict } from './dicts';
import {
  // getCampusCustomerFollowUpInfo,
  getCampusCustomerAppointmentInfo,
  getCustomerFollowUpWrappers,
} from 'SERVICE/consultant';
import commonDicts from 'PAGE/consultant/common/dicts';
import { getDictTextByVal } from 'UTIL/dict';
import moment from 'moment';

const loadCampusFollowData = async ({
  consultant,
  stuName,
  phone,
  grade,
  school,
  startDate,
  endDate,
  campusId,
  resEntranceId,
  cusType,
  cusOrg,
}) => {
  const api = await getCustomerFollowUpWrappers({
    consultantId: consultant,
    studentName: stuName,
    contact: phone,
    gradeId: grade,
    schoolName: school,
    startDate,
    endDate,
    orgId: campusId,
    resEntranceId,
    cusType,
    cusOrg,
  });
  const data = api.data || {};
  const info = data.infos || [];
  const list = info.map((d) => {
    return {
      stuName: d.studentName || '',
      grade: d.gradeName || '',
      schoolName: d.schoolName || '',
      name: d.customerName || '',
      contract: d.contact || '',
      time: d.followTime,
      nextTime: d.nextFollowTime,
      remark: d.remark,
      createUsername: d.createUserName,
      customerSignName: d.customerSignName,
      intentionName: d.intentionName,
      meetingTime: d.meetingTime,
      studentId: d.studentId,
      customerId: d.customerId,
      gradeId: d.gradeId,
      schoolId: d.schoolId,
      customerStatus: d.customerStatus,
      dealStatus: d.dealStatus,
      isAppointCome: d.isAppointCome,
      deliverTarget: d.deliverTarget,
      blCampusId: d.blCampusId,
      callRecordVos: d.callRecordVos,
      studentName: d.studentName,
      phoneNum: `${d.contact}`.replace(/(\w{3})\w*(\w{4})/, '$1****$2'),
    };
  });
  return {
    list,
    count: data.follow || 0,
    countTotal: data.tatol || info.length || 0,
  };
};

const loadVisitCusInfo = async ({ startDate, endDate, orgId, consultantId }) => {
  const api = await getCampusCustomerAppointmentInfo({
    startDate,
    endDate,
    orgId,
    consultantId,
  });
  const cusList = api.data.appointments
    .filter((da) => !!da.meetingTime && da.meetingTime.split(' ').length > 1)
    .map((d) => {
      const timeArry = d.meetingTime.split(' ');
      const date = timeArry[0];
      const startTime = timeArry[1].slice(0, 5);

      const endMeetingTime = moment(d.meetingTime).add(1, 'hours').format('YYYY-MM-DD HH:mm:ss');
      const endTimeArry = endMeetingTime.split(' ');
      const endDate = endTimeArry[0];
      const endTime = endTimeArry[1].slice(0, 5);

      return {
        id: d.id,
        status: d.isVisit ? commonDicts.VISIT_STATUS_VISITED : commonDicts.VISIT_STATUS_UNVISIT,
        date,
        startTime,
        endTime,
        name: d.customerName,
        contract: d.cusMobile,
        studentName: d.studentName,
        gradeName: d.gradeName,
        schoolName: d.schoolName,
        cusId: d.customerId,
        stuId: d.studentId,
        createUserName: d.createUserName,
        createUserId: d.createUserId,
        isOverDay: date !== endDate,
      };
    });

  const cardList = cusList.map((c) => {
    const { startTime, name, createUserName, status, contract, studentName, gradeName, schoolName } = c;
    const moreInfos = [
      {
        label: '上门状态',
        content: getDictTextByVal(commonDicts.visitStatusDict, status),
      },
      { label: '上门时间', content: startTime },
      { label: '预约人', content: createUserName },
      { label: '客户电话', content: contract },
      { label: '学生姓名', content: studentName },
      { label: '学生年级', content: gradeName },
      { label: '学生学校', content: schoolName },
    ];
    return {
      ...c,
      headerInfo: startTime,
      title: name,
      infos: [],
      moreInfos,
    };
  });
  return { list: cardList };
};

export default {
  loadCampusFollowData,
  loadVisitCusInfo,
};
