import React from 'react';
import PropTypes from 'prop-types';
import style from './styles.module.less';
import { Modal } from 'COMPONENT';
import { Row, Col, Form } from 'antd';
import { callFlagCfg, callDirectionCfg, callResultCfg, callEndCfg } from 'CONST/callConfig';
import { durationToTime } from '../utils';
import moment from 'moment';

const FormItem = Form.Item;

const labelColSpan = 4;
const wrapperColSpan = 24 - labelColSpan;

const momentFormat = 'YYYY-MM-DD HH:ss:mm';

const ModalCallDetail = (props) => {
  return (
    <Modal title="通话详情" className={style.modalCallDetail} onClose={props.onClose} okBtnShow={false}>
      <Row>
        <Col span={12}>
          <FormItem
            label="通话ID"
            colon={false}
            labelCol={{ span: labelColSpan }}
            wrapperCol={{ span: wrapperColSpan }}
          >
            {props.data.callId}
          </FormItem>
        </Col>
        <Col span={12}>
          {props.audioAuth && (
            <FormItem
              label="通话录音"
              colon={false}
              labelCol={{ span: labelColSpan }}
              wrapperCol={{ span: wrapperColSpan }}
            >
              {props.data.bossRecordUrl || props.data.recordUrl ? (
                <audio className={style.audio} controls>
                  <source src={props.data.bossRecordUrl || props.data.recordUrl} type="audio/wav" />
                </audio>
              ) : (
                '-'
              )}
            </FormItem>
          )}
        </Col>
      </Row>
      <Row>
        <Col span={12}>
          <FormItem
            label="客户姓名"
            colon={false}
            labelCol={{ span: labelColSpan }}
            wrapperCol={{ span: wrapperColSpan }}
          >
            {props.data.customerName}
          </FormItem>
        </Col>
        <Col span={12}>
          <FormItem
            label="学生姓名"
            colon={false}
            labelCol={{ span: labelColSpan }}
            wrapperCol={{ span: wrapperColSpan }}
          >
            {props.data.studentName}
          </FormItem>
        </Col>
        <Col span={12}>
          <FormItem
            label="所属学习中心"
            colon={false}
            labelCol={{ span: labelColSpan }}
            wrapperCol={{ span: wrapperColSpan }}
          >
            {props.data.organizationName}
          </FormItem>
        </Col>
        <Col span={12}>
          <FormItem
            label="坐席名称"
            colon={false}
            labelCol={{ span: labelColSpan }}
            wrapperCol={{ span: wrapperColSpan }}
          >
            {props.data.agentName}
          </FormItem>
        </Col>
        <Col span={12}>
          <FormItem
            label="主叫号码"
            colon={false}
            labelCol={{ span: labelColSpan }}
            wrapperCol={{ span: wrapperColSpan }}
          >
            {props.data.caller}
          </FormItem>
        </Col>
        <Col span={12}>
          <FormItem
            label="被叫号码"
            colon={false}
            labelCol={{ span: labelColSpan }}
            wrapperCol={{ span: wrapperColSpan }}
          >
            {props.data.callee}
          </FormItem>
        </Col>
        <Col span={12}>
          <FormItem
            label="接听状态"
            colon={false}
            labelCol={{ span: labelColSpan }}
            wrapperCol={{ span: wrapperColSpan }}
          >
            {callFlagCfg[props.data.callFlag]}
          </FormItem>
        </Col>
        <Col span={12}>
          <FormItem
            label="呼叫类型"
            colon={false}
            labelCol={{ span: labelColSpan }}
            wrapperCol={{ span: wrapperColSpan }}
          >
            {callDirectionCfg[props.data.callDirection]}
          </FormItem>
        </Col>
        <Col span={12}>
          <FormItem
            label="通话结果"
            colon={false}
            labelCol={{ span: labelColSpan }}
            wrapperCol={{ span: wrapperColSpan }}
          >
            {callResultCfg[props.data.callResult]}
          </FormItem>
        </Col>
        <Col span={12}>
          <FormItem
            label="挂断方"
            colon={false}
            labelCol={{ span: labelColSpan }}
            wrapperCol={{ span: wrapperColSpan }}
          >
            {callEndCfg[props.data.ender]}
          </FormItem>
        </Col>
        <Col span={12}>
          <FormItem
            label="呼叫时间"
            colon={false}
            labelCol={{ span: labelColSpan }}
            wrapperCol={{ span: wrapperColSpan }}
          >
            {moment(props.data.createTime).format(momentFormat)}
          </FormItem>
        </Col>
        <Col span={12}>
          <FormItem
            label="通话时长"
            colon={false}
            labelCol={{ span: labelColSpan }}
            wrapperCol={{ span: wrapperColSpan }}
          >
            {durationToTime(props.data.duration)}
          </FormItem>
        </Col>
      </Row>
    </Modal>
  );
};

ModalCallDetail.propTypes = {
  onClose: PropTypes.func.isRequired,
  data: PropTypes.shape({
    callId: PropTypes.string,
    recordUrl: PropTypes.string,
    bossRecordUrl: PropTypes.string,
    customerName: PropTypes.string,
    studentName: PropTypes.string,
    organizationName: PropTypes.string,
    agentName: PropTypes.string,
    caller: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
    callee: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
    callFlag: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
    callDirection: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
    callResult: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
    ender: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
    createTime: PropTypes.number,
    duration: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  }).isRequired,
  audioAuth: PropTypes.bool,
};

ModalCallDetail.defaultProps = {
  audioAuth: false,
};

export default ModalCallDetail;
