import React from 'react';
import PropTypes from 'prop-types';
import { Form, TextArea, Select } from 'COMPONENT';
import { DatePicker } from 'antd';
import { dateFormat, momentFormat, INVALID_CUSTOMER, FOLLOW_UP } from './config';
import { runFunc } from 'UTIL/common';

const FormItem = Form.Item;

const labelColSpan = 5;
const wrapperColSpan = 24 - labelColSpan;

class FollowUpForm extends React.Component {
  constructor(props) {
    super(props);

    this.handleFieldsChange = this.handleFieldsChange.bind(this);
    this.handleSubmit = this.handleSubmit.bind(this);
    this.submit = this.submit.bind(this);

    this.state = {
      isInvalid: false,
    };

    this.formRef = null;
  }

  static propTypes = {
    dictGroup: PropTypes.shape({
      customerIntentionDict: PropTypes.array,
      customerSignTypeDict: PropTypes.array,
      customerInvalidSignDict: PropTypes.array,
    }).isRequired,
    onSubmit: PropTypes.func.isRequired,
    // 是否为外呼资源，外呼资源不显示预约上门及下次跟进日期， 默认为false
    isOutCallResource: PropTypes.bool,
  };

  static defaultProps = {
    dictGroup: {},
    isOutCallResource: false,
  };

  handleFieldsChange(fields) {
    if (fields.satisficing) {
      this.setState({ isInvalid: fields.satisficing.value === INVALID_CUSTOMER });
    }
  }

  handleSubmit(...arg) {
    runFunc(this.formRef.submitFields, ...arg);
  }

  submit(values, ...arg) {
    let apiParams = { ...values };
    if (this.state.isInvalid) {
      delete apiParams.satisficing;
    } else {
      apiParams.appointmentType = FOLLOW_UP;

      if (apiParams.nextFollowTime) {
        apiParams.nextFollowTime = apiParams.nextFollowTime.format(dateFormat);
      }
      if (apiParams.meetingTime) {
        apiParams.meetingTime = apiParams.meetingTime.format(momentFormat);
      }
    }

    runFunc(this.props.onSubmit, apiParams, ...arg);
  }

  reset() {
    this.formRef.resetFields();
  }

  render() {
    const { isOutCallResource } = this.props;
    const isNotInvalid = !this.state.isInvalid;

    return (
      <Form ref={(c) => (this.formRef = c)} onSubmit={this.submit} onFieldsChange={this.handleFieldsChange}>
        <FormItem
          name="remark"
          rules={[{ required: true, message: '跟进内容不能为空' }]}
          formItemProps={{
            label: '跟进内容',
            colon: false,
            labelCol: { span: labelColSpan },
            wrapperCol: { span: wrapperColSpan },
          }}
        >
          <TextArea rows={3} wordCount maxLength={1000} />
        </FormItem>

        <FormItem
          name="satisficing"
          rules={[{ required: true, message: '意向度不能为空' }]}
          formItemProps={{
            label: '意向度',
            colon: false,
            labelCol: { span: labelColSpan },
            wrapperCol: { span: wrapperColSpan },
          }}
        >
          <Select placeholder="请选择" dict={this.props.dictGroup.customerIntentionDict} />
        </FormItem>

        {isNotInvalid && (
          <FormItem
            name="customerSign"
            formItemProps={{
              label: '客户意向产品',
              colon: false,
              labelCol: { span: labelColSpan },
              wrapperCol: { span: wrapperColSpan },
            }}
          >
            <Select placeholder="请选择" dict={this.props.dictGroup.customerSignTypeDict} />
          </FormItem>
        )}

        {!isOutCallResource && isNotInvalid && (
          <FormItem
            name="nextFollowTime"
            formItemProps={{
              label: '下次跟进日期',
              colon: false,
              labelCol: { span: labelColSpan },
              wrapperCol: { span: wrapperColSpan },
            }}
          >
            <DatePicker placeholder="请选择" format={dateFormat} />
          </FormItem>
        )}

        {!isOutCallResource && isNotInvalid && (
          <FormItem
            name="meetingTime"
            formItemProps={{
              label: '预约上门日期',
              colon: false,
              labelCol: { span: labelColSpan },
              wrapperCol: { span: wrapperColSpan },
            }}
          >
            <DatePicker placeholder="请选择" format={momentFormat} showTime />
          </FormItem>
        )}

        {this.state.isInvalid && (
          <FormItem
            name="invalidSignId"
            rules={[{ required: true, message: '无效标签不能为空' }]}
            formItemProps={{
              label: '无效标签',
              colon: false,
              labelCol: { span: labelColSpan },
              wrapperCol: { span: wrapperColSpan },
            }}
          >
            <Select placeholder="请选择" dict={this.props.dictGroup.customerInvalidSignDict} />
          </FormItem>
        )}
      </Form>
    );
  }
}

export default FollowUpForm;
