import React from 'react';
import { connect } from 'dva';
import PropTypes from 'prop-types';
import classNames from 'classnames';
import style from './styles.module.less';
import { Modal, List, Loading, Button, Title } from 'COMPONENT';
import { Layout, Row, Col } from 'antd';
import CallOutSider from 'BLOC/CallOutSider';
import FollowUpForm from './FollowUpForm';
import { errorsHandle, runFunc } from 'UTIL/common';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import { clone } from 'lodash';
import { FOLLOW_UP } from './FollowUpForm/config';
import { pageMyCurrentCustomerFollowUps } from 'SERVICE/consultant';
import { checkAgentBeforeCall } from 'UTIL/callCenter';
import { CALL_LOGIN_STATUS_CALLING, CALL_WAY_WEB } from 'CONST/callConfig';
import { saveCustomerFollowUp, invalidCustomerResource } from 'SERVICE/consultant';
import { saveFollowUpData, saveInvalidFollowUpData } from 'SERVICE/outcallWorkbench';

const { Sider, Content } = Layout;
const setWidth = List.createSetWidth();

class ModalPhoneCallFollow extends React.Component {
  constructor(props) {
    super(props);

    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.initList = this.initList.bind(this);
    this.checkAgentLoginStatus = this.checkAgentLoginStatus.bind(this);
    this.onClose = this.onClose.bind(this);
    this.validateCallStatus = this.validateCallStatus.bind(this);
    this.save = this.save.bind(this);
    this.saveAndFollowNext = this.saveAndFollowNext.bind(this);
    this.submitForm = this.submitForm.bind(this);
    this.resetForm = this.resetForm.bind(this);
    this.followNext = this.followNext.bind(this);

    const hadFollowList = Array.isArray(props.list) && props.list.length > 0;

    this.followList = hadFollowList ? clone(props.list) : [];
    this.tag = this.followList.length > 0 ? this.followList.shift() : props.data;

    this.state = {
      loading: false,
      contentLoading: false,
      followHistoryList: [],
      hadNoNext: this.followList.length === 0,
      nextVisible: hadFollowList,
    };

    this.isSaveFinish = false;
    this.followHistoryColumns = [
      { title: '跟进时间', dataIndex: 'createTime', key: 'createTime', width: setWidth(4) },
      { title: '跟进内容', dataIndex: 'remark', key: 'remark', width: setWidth(8) },
      {
        title: '意向度',
        dataIndex: 'intentionOfTheCustomerName',
        key: 'intentionOfTheCustomerName',
        width: setWidth(4),
      },
      { title: '客户意向产品', dataIndex: 'customerSignName', key: 'customerSignName', width: setWidth(6) },
    ];
    this.followUpFormRef = null;
    this.callOutRef = null;
  }

  static propTypes = {
    // 跟进单个学生时
    data: PropTypes.shape({
      studentId: PropTypes.string,
      customerId: PropTypes.string,
      studentName: PropTypes.string,
      phoneNum: PropTypes.string,
    }),
    // 跟进多个学生时
    list: PropTypes.arrayOf(
      PropTypes.shape({
        studentId: PropTypes.string,
        customerId: PropTypes.string,
        studentName: PropTypes.string,
        phoneNum: PropTypes.string,
      }),
    ),
    onClose: PropTypes.func.isRequired,
    // 是否外呼资源
    isOutCallResource: PropTypes.bool,
  };

  static defaultProps = {
    data: {},
    isOutCallResource: false,
  };

  static getNextFollowStudent(followList) {
    if (followList.length > 0) {
      let stuName = followList[0].studentName;
      return stuName ? `(${stuName})` : '';
    }
    return '';
  }

  componentDidMount() {
    this.init();
  }

  async init() {
    this.isSaveFinish = false;
    this.setState({ contentLoading: true });
    await new Promise(this.initDict);
    await new Promise(this.initList);
    this.setState({ contentLoading: false });
  }

  initDict(resolve) {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['customerIntention', 'customerSignType', 'customerInvalidSign'],
        callback() {
          resolve();
        },
      },
    });
  }

  async initList(resolve) {
    const apiParams = {
      customerId: this.tag.customerId,
      studentId: this.tag.studentId,
      pageSize: 3,
    };
    const res = await pageMyCurrentCustomerFollowUps(apiParams, true).catch(errorsHandle());
    let list = [];
    if (res && res.data) {
      list = res.data.list || [];
    }
    this.setState({ followHistoryList: list });
    resolve();
  }

  checkAgentLoginStatus(resolve) {
    return this.callOutRef.updateCallStatus(resolve);
  }

  async onClose() {
    let isCallEnd = this.callOutRef.isCallEnd();
    if (!isCallEnd) {
      this.setState({ loading: true });
      const agentLoginStatus = await new Promise(this.checkAgentLoginStatus);
      this.setState({ loading: false });
      isCallEnd = agentLoginStatus !== CALL_LOGIN_STATUS_CALLING;
    }
    if (isCallEnd) {
      if (!this.isSaveFinish) {
        return showConfirm('跟进记录还未提交，是否确认关闭？', this.props.onClose);
      }
      return this.props.onClose();
    }
    showConfirm('通话尚在进行中，是否确认关闭？', () => {
      if (this.props.callWay === CALL_WAY_WEB) {
        this.callOutRef.handUp(this.props.onClose);
      } else {
        this.props.onClose();
      }
    });
  }

  async validateCallStatus(callback) {
    if (this.callOutRef.isCallEnd()) {
      this.followUpFormRef.handleSubmit(callback);
    } else {
      this.setState({ loading: true });
      const agentLoginStatus = await new Promise(this.checkAgentLoginStatus);
      this.setState({ loading: false });
      if (checkAgentBeforeCall(agentLoginStatus, false)) {
        this.followUpFormRef.handleSubmit(callback);
      } else {
        showMessage('通话尚在进行中！', 'error');
      }
    }
  }

  save() {
    this.validateCallStatus(() => {
      this.props.onClose();
    });
  }

  saveAndFollowNext() {
    this.validateCallStatus(() => {
      this.resetForm();
      this.followNext();
    });
  }

  async submitForm(apiParams, callback) {
    this.setState({ contentLoading: true });
    const { isOutCallResource } = this.props;
    let fetch =
      apiParams.appointmentType === FOLLOW_UP
        ? isOutCallResource
          ? saveFollowUpData
          : saveCustomerFollowUp
        : isOutCallResource
        ? saveInvalidFollowUpData
        : invalidCustomerResource;
    const params = {
      ...apiParams,
      followStuId: this.tag.studentId,
      customerId: this.tag.customerId,
      callId: this.callOutRef.getCallId(),
    };
    if (isOutCallResource) {
      params.resourceType = 1; /** 外呼资源池需增加资源类型参数 */
    }
    const res = await fetch(params).catch(errorsHandle());

    if (res && res.resultCode === 0) {
      this.isSaveFinish = true;
      showMessage('跟进记录保存成功！', 'success');
    }

    this.setState({ contentLoading: false });
    if (res && res.resultCode === 0) {
      runFunc(callback);
    }
  }

  resetForm() {
    this.followUpFormRef.reset();
  }

  followNext() {
    if (this.followList.length > 0) {
      this.tag = this.followList.shift();
      this.setState({ hadNoNext: this.followList.length === 0 });
      this.init();
      this.callOutRef.init();
    }
  }

  render() {
    const btnDisabled = this.state.contentLoading;
    return (
      <Modal
        className={style.ModalPhoneCallFollow}
        title="外呼跟进"
        loading={this.state.loading}
        loadingTip="校验坐席状态中..."
        onCancel={this.onClose}
        onOk={this.save}
        widthSize="L"
        footer={
          <div>
            <Button type="default" onClick={this.resetForm} disabled={btnDisabled}>
              重置
            </Button>
            <Button type="primary" onClick={this.save} disabled={btnDisabled}>
              保存
            </Button>
            {this.state.nextVisible && (
              <Button type="primary" onClick={this.saveAndFollowNext} disabled={this.state.hadNoNext || btnDisabled}>
                保存并呼叫下一个学生{ModalPhoneCallFollow.getNextFollowStudent(this.followList)}
              </Button>
            )}
          </div>
        }
      >
        <Layout className={style.Layout} style={{ height: 590 }}>
          <Sider className={style.Sider} width={300}>
            <div className={classNames(style.contentWrapper, style.callOutSiderWrapper)}>
              <CallOutSider
                ref={(c) => (this.callOutRef = c)}
                phoneNum={this.tag.phoneNum}
                name={this.tag.studentName}
                customerId={this.tag.customerId}
                studentId={this.tag.studentId}
                agentId={this.props.agentId}
                agentLoginStatus={this.props.agentLoginStatus}
                callWay={this.props.callWay}
                dispatch={this.props.dispatch}
              />
            </div>
          </Sider>
          <Content className={style.Content}>
            <div className={style.contentWrapper}>
              <Row style={{ marginTop: 12 }}>
                <Col span={5}>
                  <div className={style.label}>学生姓名</div>
                </Col>
                <Col span={5}>
                  <div className={style.content}>{this.tag.studentName || '-'}</div>
                </Col>
                <Col span={3}>
                  <div className={style.label}>年级</div>
                </Col>
                <Col span={11}>
                  <div className={style.content}>{this.tag.gradeName || '-'}</div>
                </Col>
              </Row>

              <Row>
                <Col span={5}>
                  <div className={style.label}>资源入口</div>
                </Col>
                <Col span={5}>
                  <div className={style.content}>{this.tag.resEntranceName || '-'}</div>
                </Col>
                <Col span={3}>
                  <div className={style.label}>学校</div>
                </Col>
                <Col span={11}>
                  <div className={style.content}>{this.tag.schoolName || '-'}</div>
                </Col>
              </Row>

              <Title>跟进记录</Title>
              <div className={style.formWrapper}>
                <FollowUpForm
                  /** 是否为外呼资源 */
                  isOutCallResource={this.props.isOutCallResource}
                  ref={(c) => (this.followUpFormRef = c)}
                  dictGroup={this.props.dictGroup}
                  onSubmit={this.submitForm}
                />
              </div>

              <Title>过往跟进记录</Title>
              <List
                className={style.list}
                columns={this.followHistoryColumns}
                dataSource={this.state.followHistoryList}
              />
            </div>
            <Loading loading={this.state.contentLoading} />
          </Content>
        </Layout>
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    dictGroup: {
      customerIntentionDict: state.dict.customerIntention,
      customerSignTypeDict: state.dict.customerSignType,
      customerInvalidSignDict: state.dict.customerInvalidSign,
    },
    agentId: state.callCenter.agentId,
    agentLoginStatus: state.callCenter.agentLoginStatus,
    callWay: state.callCenter.callWay,
  };
};

export default connect(mapStateToProps)(ModalPhoneCallFollow);
