import React from 'react';
import { connect } from 'dva';
import styles from './styles.less';
import { Tabs } from 'antd';
import Pool from './Pool';
import loaders from './loaders';
import Page from 'COMPONENT/Page';

const TabPane = Tabs.TabPane;

class ResourcePools extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 页面加载状态
      poolDict: [], // {object[]} 资源池字典
    };
    this.pools = {}; // {object} 资源池列表信息，{ id: { ref, inited } }
  }

  componentWillMount() {
    this.setState({ loading: true });
    loaders.loadPoolDict().then((poolDict) => {
      poolDict.forEach((d) => {
        this.pools[d.val] = { ref: null, inited: false };
      });
      this.setState({ loading: false, poolDict });
    });
  }

  saveRef = (comp, key) => {
    this.pools[key].ref = comp;
    if (!this.pools[key].inited) {
      comp.initContent().then(() => {
        this.pools[key].inited = true;
      });
    }
  };

  render() {
    const { dispatch } = this.props;
    const { loading, poolDict } = this.state;
    return (
      <Page fitWindowHeight={true} route={this.props.route} loading={loading} className={styles.page}>
        <Tabs animated={false}>
          {poolDict.map((d, idx) => {
            return (
              <TabPane tab={d.text} key={d.val}>
                <Pool
                  ref={(comp) => this.saveRef(comp, d.val)}
                  poolId={d.val}
                  dispatch={dispatch}
                  initImmediately={idx === 0}
                />
              </TabPane>
            );
          })}
        </Tabs>
      </Page>
    );
  }
}

export default connect()(ResourcePools);
