import React from 'react';
import { DatePicker, Select, SearchBar, Input } from 'COMPONENT';
import moment from 'moment';
import { loadBranchDict, loadCampusDict } from 'UTIL/dict';
import {
  CALL_FLAG_MISSED,
  CALL_FLAG_ANSWERED,
  CALL_DIRECT_ANSWER,
  CALL_DIRECT_CALL,
  callFlagCfg,
  callDirectionCfg,
} from 'CONST/callConfig';

const callFlagDict = [
  {
    text: callFlagCfg[CALL_FLAG_MISSED],
    val: `${CALL_FLAG_MISSED}`,
  },
  {
    text: callFlagCfg[CALL_FLAG_ANSWERED],
    val: `${CALL_FLAG_ANSWERED}`,
  },
];

const callDirectionDict = [
  {
    text: callDirectionCfg[CALL_DIRECT_ANSWER],
    val: `${CALL_DIRECT_ANSWER}`,
  },
  {
    text: callDirectionCfg[CALL_DIRECT_CALL],
    val: `${CALL_DIRECT_CALL}`,
  },
];

class CallRecordsSearchBar extends React.Component {
  constructor(props) {
    super(props);

    const { initListSearchKeys } = props;

    this.state = {
      searchExpand: false,
      branchDict: [],
      blCampusDict: [],
      ...initListSearchKeys,
    };
  }

  componentDidMount() {
    this.initCompontent();
  }

  initCompontent = async () => {
    const { branchId } = this.state;

    const branchDict = await loadBranchDict();
    const blCampusDict = !branchId ? [] : await loadCampusDict(branchId);

    this.setState({
      branchDict,
      blCampusDict,
    });
  };

  getBlCampusDict = async (branchId, campusId) => {
    const blCampusDict = !branchId ? [] : await loadCampusDict(branchId);
    campusId = campusId || blCampusDict[0].val || '';

    this.setState({
      blCampusDict,
      campusId,
    });
  };

  getDisabledStartDate = (startCreateTime) => {
    const { endCreateTime } = this.state;
    if (!startCreateTime || !endCreateTime) {
      return false;
    }
    return moment(startCreateTime).valueOf() > moment(endCreateTime).valueOf();
  };

  getDisabledEndDate = (endCreateTime) => {
    const { startCreateTime } = this.state;
    if (!endCreateTime || !startCreateTime) {
      return false;
    }
    return moment(endCreateTime).valueOf() < moment(startCreateTime).valueOf();
  };

  reset = () => {
    this.setState({ ...this.props.initListSearchKeys });
  };

  search = () => {
    const {
      branchId,
      campusId,
      studentName,
      caller,
      callee,
      startCreateTime,
      endCreateTime,
      callFlag,
      callDirection,
      customerName,
    } = this.state;

    this.props.onSearch({
      branchId,
      campusId,
      studentName,
      caller,
      callee,
      startCreateTime: moment(startCreateTime).valueOf(),
      // 获取当天最后一秒
      endCreateTime: moment(endCreateTime).add(1, 'days').valueOf() - 1000,
      callFlag,
      callDirection,
      customerName,
    });
  };

  render() {
    const {
      branchSelectorVisible,
      campusSelectorVisible,
      customerInputVisible,
      branchSelectorDisabled,
      campusSelectorDisabled,
    } = this.props;

    const {
      branchDict,
      blCampusDict,
      searchExpand,
      branchId,
      campusId,
      studentName,
      caller,
      callee,
      startCreateTime,
      endCreateTime,
      callFlag,
      callDirection,
      customerName,
    } = this.state;

    return (
      <SearchBar
        expand={searchExpand}
        onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
        onSearch={this.search}
        onReset={this.reset}
      >
        {branchSelectorVisible && (
          <Select
            placeholder="分公司"
            showSearch
            allowClear={false}
            dict={branchDict}
            value={branchId}
            onChange={(val) => {
              this.setState({ branchId: val });
              this.getBlCampusDict(val);
            }}
            disabled={branchSelectorDisabled}
          />
        )}
        {campusSelectorVisible && (
          <Select
            placeholder="学习中心"
            showSearch
            allowClear={false}
            dict={blCampusDict}
            value={campusId}
            onChange={(val) => {
              this.setState({ campusId: val });
            }}
            disabled={campusSelectorDisabled}
          />
        )}
        <DatePicker
          placeholder="呼叫时间（起）"
          value={startCreateTime}
          onChange={(val) => this.setState({ startCreateTime: val })}
          disabledDate={this.getDisabledStartDate}
        />
        <DatePicker
          placeholder="呼叫时间（止）"
          value={endCreateTime}
          onChange={(val) => this.setState({ endCreateTime: val })}
          disabledDate={this.getDisabledEndDate}
        />
        {customerInputVisible && (
          <Input
            placeholder="客户姓名"
            value={customerName}
            onChange={(e) => this.setState({ customerName: e.target.value })}
          />
        )}
        <Input
          placeholder="学生姓名"
          value={studentName}
          onChange={(e) => this.setState({ studentName: e.target.value })}
        />
        <Input placeholder="主叫号码" value={caller} onChange={(e) => this.setState({ caller: e.target.value })} />
        <Input placeholder="被叫号码" value={callee} onChange={(e) => this.setState({ callee: e.target.value })} />
        <Select
          placeholder="接听状态"
          dict={callFlagDict}
          value={callFlag}
          onChange={(val) => this.setState({ callFlag: val })}
        />
        <Select
          placeholder="呼叫类型"
          dict={callDirectionDict}
          value={callDirection}
          onChange={(val) => {
            this.setState({ callDirection: val });
          }}
        />
      </SearchBar>
    );
  }
}

export default CallRecordsSearchBar;
