import React from 'react';
import { Section } from 'COMPONENT';
import moment from 'moment';
import CallRecord from 'PAGE/consultant/CallRecord';
import CallRecordSearchBar from './SearchBar';
import { connect } from 'dva';
import { DATE_FORMAT } from 'COMPONENT/utils/form';

const today = moment().format(DATE_FORMAT);

class CallRecordsSec extends React.Component {
  constructor(props) {
    super(props);

    const { branchId, campusId, userId, btnAuths, branchSelectorVisible, campusSelectorVisible } = props;

    this.initListSearchKeys = {
      startCreateTime: today, // 开始创建时间
      endCreateTime: today, // 结束创建时间
      studentName: '', // 学生名
      customerName: '', // 客户名
      caller: '', // 主叫号码
      callee: '', // 被叫号码
      callDirection: '', // 呼叫类型，0（呼入），1（呼出）
      callFlag: '', // 接听标识0：未接听，1：已接听
      branchId: branchId && branchSelectorVisible ? branchId : null,
      campusId: campusId && campusSelectorVisible ? campusId : null,
    };

    this.callRecordDefaultSKey = {
      startCreateTime: moment(today).valueOf(),
      // 获取当天最后一秒
      endCreateTime: moment(today).add(1, 'days').valueOf() - 1000,
      userId,
      organizationId: campusId || branchId,
    };
    this.audioAuth = btnAuths && btnAuths.indexOf('CALL_AUTH_PLAY_AUDIO') > -1;

    this.state = {
      callRecordKeys: this.callRecordDefaultSKey,
    };
  }

  searchCallRecord = (callRecordKeys) => {
    const organizationId = callRecordKeys.campusId || callRecordKeys.branchId;
    delete callRecordKeys.branchId;
    delete callRecordKeys.campusId;

    this.setState(
      {
        callRecordKeys: {
          ...callRecordKeys,
          userId: this.props.userId,
          organizationId,
        },
      },
      () => {
        this.callRecordRef && this.callRecordRef.search();
      },
    );
  };

  render() {
    const { callRecordKeys } = this.state;

    const {
      branchSelectorVisible,
      campusSelectorVisible,
      customerInputVisible,
      branchSelectorDisabled,
      campusSelectorDisabled,
    } = this.props;
    const searchBarProps = {
      branchSelectorVisible,
      campusSelectorVisible,
      customerInputVisible,
      branchSelectorDisabled,
      campusSelectorDisabled,
      initListSearchKeys: this.initListSearchKeys,
    };

    return (
      <Section title="通话记录">
        <div className="flexWrap">
          <CallRecordSearchBar {...searchBarProps} onSearch={this.searchCallRecord} />
        </div>
        {/* 列表 */}
        <CallRecord
          pagination
          searchKeys={callRecordKeys}
          audioAuth={this.audioAuth}
          ref={(ref) => (this.callRecordRef = ref)}
        />
      </Section>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    btnAuths: state.account.btnAuths,
  };
};

export default connect(mapStateToProps, null, null, { withRef: true })(CallRecordsSec);
