import React from 'react';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import { loadBranchDict, loadCampusDict } from 'UTIL/dict';
import { loadConsultantDict } from 'PAGE/common/loaders';
import { Row, Col } from 'antd';

class CampusSelector extends React.Component {
  constructor(props) {
    super(props);

    const {
      branchId,
      campusId,
      branchSelectorDisabled,
      campusSelectorDisabled,
      showBranchSelector,
      showCampusSelector,
      showConsultantSelector,
      consultantId,
      consultantSelectorDisabled,
      allowBranchSelectorClear,
      allowCampusSelectorClear,
      allowConsultantSelectorClear,
    } = this.props;

    this.state = {
      branchId: branchId ? branchId : '',
      campusId: campusId ? campusId : '',
      brenchDict: [],
      blCampusDict: [],
      branchSelectorDisabled: branchSelectorDisabled !== undefined ? branchSelectorDisabled : !!branchId,
      campusSelectorDisabled: campusSelectorDisabled !== undefined ? campusSelectorDisabled : !!campusId,
      branchSelectorInnerDisabled: true,
      campusSelectorInnerDisabled: true,

      showBranchSelector: showBranchSelector !== undefined ? showBranchSelector : true,
      showCampusSelector: showCampusSelector !== undefined ? showCampusSelector : true,
      showConsultantSelector: showConsultantSelector !== undefined ? showConsultantSelector : true,

      consultantId: consultantId ? consultantId : '',
      consultantDict: [],
      consultantSelectorDisabled:
        consultantSelectorDisabled !== undefined ? consultantSelectorDisabled : !!consultantId,
      consultantSelectorInnerDisabled: true,
      allowBranchSelectorClear: allowBranchSelectorClear ? allowBranchSelectorClear : false,
      allowCampusSelectorClear: allowCampusSelectorClear ? allowCampusSelectorClear : false,
      allowConsultantSelectorClear: allowConsultantSelectorClear ? allowConsultantSelectorClear : true,
    };
  }

  onBranchChange = async (branchId) => {
    const { showCampusSelector } = this.state;
    this.setState({
      campusSelectorInnerDisabled: true,
      consultantSelectorInnerDisabled: true,
    });
    const blCampusDict = !showCampusSelector || !branchId ? [] : await loadCampusDict(branchId);
    const consultantDict = [];
    this.setState({
      blCampusDict,
      consultantDict,
      campusSelectorInnerDisabled: false,
      consultantSelectorInnerDisabled: false,
    });
  };

  onCampusChange = async (campusId) => {
    const { showConsultantSelector } = this.state;
    this.setState({
      consultantSelectorInnerDisabled: true,
    });
    const consultantDict = !showConsultantSelector || !campusId ? [] : await loadConsultantDict({ orgId: campusId });
    this.setState({
      consultantDict,
      consultantSelectorInnerDisabled: false,
    });
    this.onChange({ orgId: campusId, consultantId: '' });
  };

  onConsultantChange = (consultantId) => {
    this.onChange({ consultantId });
  };

  onChange = (params) => {
    const args = Object.assign(
      {
        orgId: this.state.campusId,
        consultantId: this.state.consultantId,
        branchId: this.state.branchId,
      },
      params,
    );
    if (this.props.onChange) {
      this.props.onChange(args.orgId, args.consultantId, args.branchId);
    }
  };

  componentDidMount() {
    this.initial();
  }

  async initial() {
    this.setState({
      branchSelectorInnerDisabled: true,
      campusSelectorInnerDisabled: true,
      consultantSelectorInnerDisabled: true,
    });
    const { branchId, campusId, consultantId, showBranchSelector, showCampusSelector, showConsultantSelector } =
      this.state;
    const brenchDict = !showBranchSelector ? [] : await loadBranchDict();
    const blCampusDict = !showCampusSelector || !branchId ? [] : await loadCampusDict(branchId);
    const consultantDict = !showConsultantSelector || !campusId ? [] : await loadConsultantDict({ orgId: campusId });
    this.setState({
      brenchDict,
      blCampusDict,
      consultantDict,
      campusId: campusId || '',
      consultantId: consultantId || '',
      branchSelectorInnerDisabled: false,
      campusSelectorInnerDisabled: false,
      consultantSelectorInnerDisabled: false,
    });
  }

  render() {
    const {
      branchId,
      campusId,
      brenchDict,
      blCampusDict,
      branchSelectorDisabled,
      campusSelectorDisabled,
      branchSelectorInnerDisabled,
      campusSelectorInnerDisabled,
      showBranchSelector,
      showCampusSelector,
      showConsultantSelector,
      consultantId,
      consultantDict,
      consultantSelectorDisabled,
      consultantSelectorInnerDisabled,
      allowBranchSelectorClear,
      allowCampusSelectorClear,
      allowConsultantSelectorClear,
      consultantIds,
    } = this.state;

    return (
      <Row gutter={16}>
        {showBranchSelector && (
          <Col span={3}>
            <Select
              placeholder="分公司"
              showSearch
              allowClear={allowBranchSelectorClear}
              dict={brenchDict}
              value={branchId}
              onChange={(val) => {
                this.setState({ branchId: val });
                this.onBranchChange(val);
              }}
              disabled={branchSelectorDisabled || branchSelectorInnerDisabled}
            />
          </Col>
        )}
        {showCampusSelector && (
          <Col span={3}>
            <Select
              placeholder="学习中心"
              showSearch
              allowClear={allowCampusSelectorClear}
              dict={blCampusDict}
              value={campusId}
              onChange={(val) => {
                this.setState({ campusId: val, consultantId: '' });
                this.onCampusChange(val);
              }}
              disabled={campusSelectorDisabled || campusSelectorInnerDisabled}
            />
          </Col>
        )}
        {showConsultantSelector && (
          <Col span={3}>
            <Select
              placeholder="咨询师"
              showSearch
              allowClear={allowConsultantSelectorClear}
              dict={consultantDict}
              value={consultantId}
              onChange={(val) => {
                this.setState({ consultantId: val });
                this.onConsultantChange(val);
              }}
              disabled={consultantSelectorDisabled || consultantSelectorInnerDisabled}
            />
          </Col>
        )}
      </Row>
    );
  }
}

export default CampusSelector;
