import React from 'react';
import ReactDOM from 'react-dom';
import { connect } from 'dva';
import classnames from 'classnames';
import styles from './styles.less';
import utils from '../utils';
import moment from 'moment';
import { DATE_FORMAT } from 'COMPONENT/utils/form';
import MoreInfoPop from '../MoreInfoPop';
import { visitStatusColors } from '../../utils';
import TextLink from 'COMPONENT/TextLink';

class Card extends React.Component {
  state = {
    mode: null,
  };

  componentDidMount() {
    this.$wrap = ReactDOM.findDOMNode(this.wrapRef);
    this.updateMode();
    window.addEventListener('resize', this.updateMode, false);
  }

  componentWillReceiveProps(nextProps) {
    if (nextProps.expandDate !== this.expandDate) {
      if (nextProps.expand && !this.props.expand) {
        this.setState({ mode: 'default' });
      } else {
        setTimeout(() => {
          this.updateMode();
        }, 500);
      }
    }
  }

  componentWillUnmount() {
    window.removeEventListener('resize', this.updateMode, false);
  }

  updateMode = () => {
    const w = this.$wrap.offsetWidth;
    this.setState({ mode: w < 0 ? 'small' : 'default' });
  };

  getContent = (lines) => {
    const {
      data: { headerInfo, title, infos, cusId, stuId, createUserId, createUserName, name },
      onCardClick,
      isManager,
    } = this.props;
    const cls = {
      [styles.content]: 1,
      [styles.contentDefault]: 1,
    };
    const onClick =
      onCardClick !== undefined
        ? onCardClick
        : () =>
            this.goDetail({
              id: cusId,
              stuId,
              detailEditable: !isManager,
              followEditable: !isManager,
              deliverTarget: createUserId,
            });

    return (
      <div className={classnames(cls)}>
        <div className={styles.headerInfo}>{name}</div>
        {lines >= 2 && (
          <div className={styles.title}>
            {onClick ? <TextLink onClick={onClick}>{createUserName}</TextLink> : <span>{createUserName}</span>}
          </div>
        )}
        {lines >= 3 && (
          <div className={styles.infos}>
            {infos.slice(0, lines - 2).map((info, idx) => {
              return (
                <div key={idx} className={styles.info}>
                  {info}
                </div>
              );
            })}
          </div>
        )}
      </div>
    );
  };

  getContentSmall = (lines) => {
    const {
      data: { title, createUserName },
    } = this.props;
    const cls = {
      [styles.content]: 1,
      [styles.contentSmall]: 1,
    };
    const titleCls = {
      [styles.title]: 1,
      [styles[`titleLines${lines - 1}`]]: 1,
    };
    return (
      <div className={classnames(cls)}>
        <div className={classnames(titleCls)}>{createUserName}</div>
      </div>
    );
  };

  onRefresh = (params) => {
    const { onRefresh } = this.props;
    onRefresh();
    this.goDetail(params);
  };

  goDetail = (params) => {
    const { dispatch } = this.props;
    params = {
      ...params,
      customerId: this.props.data.cusId,
      studentId: this.props.data.stuId,
      studentName: this.props.data.studentName,
      phoneNum: this.props.data.contract.replace(/(\w{3})\w*(\w{4})/, '$1****$2'),
      onRefresh: this.onRefresh,
    };
    dispatch({
      type: 'main/openPage',
      payload: {
        path: '/CommonCusDetailPage',
        params,
        refresh: true,
        mirrorId: `consultant/common/VisitCusSchedule/${params.id}/${params.stuId}/${
          params.detailEditable ? 'detail' : 'follow'
        }`,
      },
    });
  };

  render() {
    const {
      data: {
        isOverDay,
        date,
        startTime,
        endTime,
        status,
        type,
        moreInfos,
        title,
        classId,
        cusId,
        stuId,
        createUserId,
      },
      nodeStartTime,
      onActiveTimeChange,
      $panel,
      isManager,
    } = this.props;
    const { mode } = this.state;
    const colors = visitStatusColors[status];
    const top = utils.getTimePx(nodeStartTime, startTime);
    const height = utils.getTimePx(startTime, endTime);
    const lines = Math.floor((height - 6) / 19);
    return (
      <MoreInfoPop
        data={moreInfos}
        onClick={() =>
          this.goDetail({
            id: cusId,
            stuId,
            detailEditable: !isManager,
            followEditable: !isManager,
            deliverTarget: createUserId,
          })
        }
      >
        <div
          className={styles.wrap}
          style={{ borderColor: colors.primary, top, height }}
          onMouseEnter={() => onActiveTimeChange({ start: startTime, end: endTime })}
          onMouseLeave={() => onActiveTimeChange(null)}
          ref={(c) => (this.wrapRef = c)}
        >
          {mode ? (mode === 'small' ? this.getContentSmall(lines) : this.getContent(lines)) : null}
        </div>
      </MoreInfoPop>
    );
  }
}

export default connect()(Card);
