import dicts from './dicts';
import { getConsultorCustomerFolowup, getBranchPerformanceRank } from 'SERVICE/consultant';
import moment from 'moment';
import { DATE_FORMAT, TIME_FORMAT } from 'COMPONENT/utils/form';
import { getDictTextByVal } from '../../../utils/dict/index';
import { judgeCampusPower } from 'SERVICE/consultant';

const loadVisitCusList = async ({ startDate, endDate }) => {
  const { data } = await getConsultorCustomerFolowup({ dateStart: startDate, dateEnd: endDate });
  const cusList = data
    .filter((da) => !!da.meetingTime && da.meetingTime.split(' ').length > 1)
    .map((d) => {
      const timeArry = d.meetingTime.split(' ');
      const date = timeArry[0];
      const startTime = timeArry[1].slice(0, 5);

      const endMeetingTime = moment(d.meetingTime).add(1, 'hours').format('YYYY-MM-DD HH:mm:ss');
      const endTimeArry = endMeetingTime.split(' ');
      const endDate = endTimeArry[0];
      const endTime = endTimeArry[1].slice(0, 5);

      return {
        id: d.id,
        status: d.isVisit ? dicts.VISIT_STATUS_VISITED : dicts.VISIT_STATUS_UNVISIT,
        date,
        startTime,
        endTime,
        name: d.customerName,
        contract: d.cusMobile,
        studentName: d.followStuName,
        gradeName: d.gradeName,
        schoolName: d.schoolName,
        cusId: d.customerId,
        stuId: d.followStuId,
        isVisit: d.isVisit,
        isOverDay: date !== endDate,
        createUserName: d.createUserName,
      };
    });
  /*
      const cusList = [
        { id: '1', status: dicts.VISIT_STATUS_UNVISIT, date: '2018-03-19', startTime: '10:30', endTime: '11:30',
          name: '王女士', contract: '133449999944', studentName: '季同学', gradeName: '三年级', schoolName: '培正中学',
        },
        { id: '11', status: dicts.VISIT_STATUS_UNVISIT, date: '2018-03-19', startTime: '10:30', endTime: '11:30',
          name: '王女士', contract: '133449999944', studentName: '季同学', gradeName: '三年级', schoolName: '培正中学',
        },
        { id: '12', status: dicts.VISIT_STATUS_UNVISIT, date: '2018-03-19', startTime: '10:30', endTime: '11:30',
          name: '王女士', contract: '133449999944', studentName: '季同学', gradeName: '三年级', schoolName: '培正中学',
        },
        { id: '13', status: dicts.VISIT_STATUS_UNVISIT, date: '2018-03-19', startTime: '10:30', endTime: '11:30',
          name: '王女士', contract: '133449999944', studentName: '季同学', gradeName: '三年级', schoolName: '培正中学',
        },
        { id: '2', status: dicts.VISIT_STATUS_VISITED, date: '2018-03-21', startTime: '14:00', endTime: '15:00',
          name: '李先生', contract: '133449999944', studentName: '季同学', gradeName: '三年级', schoolName: '培正中学',
        },
        { id: '1', status: dicts.VISIT_STATUS_UNVISIT, date: '2018-04-28', startTime: '09:30', endTime: '10:30',
          name: '王女士', contract: '133449999944', studentName: '季同学', gradeName: '三年级', schoolName: '培正中学',
        },
        { id: '4', status: dicts.VISIT_STATUS_VISITED, date: '2018-04-29', startTime: '16:00', endTime: '18:00',
          name: '李先生', contract: '133449999944', studentName: '季同学', gradeName: '三年级', schoolName: '培正中学',
        },
      ]*/
  const cardList = cusList.map((c) => {
    const { startTime, name, status, contract, studentName, gradeName, schoolName, createUserName } = c;
    const moreInfos = [
      { label: '上门状态', content: getDictTextByVal(dicts.visitStatusDict, status) },
      { label: '上门时间', content: startTime },
      { label: '预约人', content: createUserName },
      { label: '客户电话', content: contract },
      { label: '学生姓名', content: studentName },
      { label: '学生年级', content: gradeName },
      { label: '学生学校', content: schoolName },
    ];
    return {
      ...c,
      headerInfo: startTime,
      title: name,
      infos: [],
      moreInfos,
    };
  });
  return cardList;
};

const loadRankingData = async ({ timeRange, orgId }) => {
  const api = await getBranchPerformanceRank({
    roleCode: ['CONSULTOR', 'CONSULTOR_DIRECTOR'],
    timeType: timeRange,
    orgId,
  });
  const list = api.data.map((d) => {
    return {
      userId: d.userId,
      name: d.userName,
      campusName: d.orgName,
      num: d.amount,
      asc: d.upDown > 0 ? d.upDown : null,
      desc: d.upDown < 0 ? -d.upDown : null,
      isMe: d.isMe === '1',
      isMyCampus: d.isMyCampus === '1',
      rank: d.randId,
    };
  });
  return list;
  /*  let data = [
      { userId: '1', name: '李咨询', campusName: '香洲学习中心', num: 13680, asc: 12, desc: null },
      { userId: '2', name: '李咨询', campusName: '香洲学习中心', num: 13333, asc: null, desc: 3 },
      { userId: '3', name: '李咨询', campusName: '香洲珠江学习中心香洲珠江学习中心', num: 13680, asc: null, desc: null },
      { userId: '4', name: '李咨询', campusName: '香洲学习中心', num: 12680, asc: 12, desc: null },
      { userId: '5', name: '李咨询', campusName: '香洲学习中心', num: 12580, asc: 12, desc: null },
      { userId: '6', name: '李咨询', campusName: '香洲学习中心', num: 11111, asc: 12, desc: null },
      { userId: '7', name: '李咨询', campusName: '香洲学习中心', num: 9888, asc: 12, desc: null },
      { userId: '8', name: '李咨询', campusName: '香洲学习中心', num: 9888, asc: 12, desc: null },
      { userId: '9', name: '李咨询', campusName: '香洲学习中心', num: 9888, asc: 12, desc: null },
      { userId: '10', name: '李咨询', campusName: '香洲学习中心', num: 9888, asc: 12, desc: null },
      { userId: '11', name: '李咨询', campusName: '香洲学习中心', num: 9888, asc: 12, desc: null },
      { userId: '112233', name: '李咨询', campusName: '香洲学习中心', num: 9888, asc: 12, desc: null },
      { userId: '13', name: '李咨询', campusName: '香洲学习中心', num: 9888, asc: 12, desc: null },
      { userId: '14', name: '李咨询', campusName: '香洲学习中心', num: 9888, asc: 12, desc: null },
      { userId: '15', name: '李咨询', campusName: '香洲学习中心', num: 9888, asc: 12, desc: null },
      { userId: '16', name: '李咨询', campusName: '香洲学习中心', num: 9888, asc: 12, desc: null },
      { userId: '17', name: '李咨询', campusName: '香洲学习中心', num: 9888, asc: 12, desc: null },
      { userId: '18', name: '李咨询', campusName: '香洲学习中心', num: 9888, asc: 12, desc: null },
      { userId: '19', name: '李咨询', campusName: '香洲学习中心', num: 9888, asc: 12, desc: null },
    ]
    return data.map((d, idx) => {
      return { ...d, rank: idx + 1 }
    })*/
};

const checkHasCampus = async () => {
  /*  await new Promise(res => {
      setTimeout(() => {res()}, 2000)
    })*/
  const { data, resultCode, resultMessage } = await judgeCampusPower({}, true);
  if (resultCode === 0) {
    return { canShow: true };
  } else {
    return { canShow: false, errorTips: resultMessage || '没有所属学习中心' };
  }
};

export default {
  loadVisitCusList,
  loadRankingData,
  checkHasCampus,
};
