import React, { Component } from 'react';
import { connect } from 'dva';
import List from 'COMPONENT/List';
import Page from 'COMPONENT/Page';
import Button from 'COMPONENT/Button';
import IndexSearch from '../components/IndexSearch';
import DetailModal from './modals/DetailModal';
import ModalSelectClass from '../ModalSelectClass';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { clone } from 'lodash';
import { getDictTextByVal, getDictValByText } from 'UTIL/dict';
import * as Loader from '../loaders';
import './style.less';
import style from './style.less';
import { buildClassTypeDict } from 'MODEL/dict/utils';
import { productTypeAllow, SMALL_CLASS_VAL, TWO_TEACHER_VAL } from '../constants/config';
import { twoTeacherInstitution } from 'CONST/config';
import * as utils from '../utils';
import { getDefaultQuarter } from '../utils';
import { getTrueCampusArray } from '../utils';
import { buildBlCampus } from '../utils';

const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L } = List;

const classPrefix = 'ContinueIndex';
// 筛选项初始值
const defaultSKey = {
  className: '',
  branchId: null,
  blCampusId: null,
  productTypeId: null,
  yearId: null,
  quarterId: getDefaultQuarter(),
  phaseId: null,
  gradeId: null,
  subjectId: null,
  classTypeId: null,
  courseSeries: null,
  classLevel: null,
  mainTeacherId: null,
  assistTeacherId: null,
  associatedStatus: null,
};

class ContinueIndex extends Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: true,
      //列表数据
      listLoading: false,
      rows: [],
      total: 0,
      page: 1,
      pageSize: 20,
      selectedRecord: {},
      showDetailModal: false,
      showSelectClassModal: false,
      //搜索条件
      ...defaultSKey,
      // 列表选中项
      selectedRowKeys: [],
      // 关联范围
      originalClassQuantity: 0,
      associatedQuantity: 0,
    };
  }

  componentWillMount() {
    this.init();
  }

  init = async () => {
    this.setState({ loading: true });
    await new Promise(this.initDict);
    await new Promise(this.initSKeyWithCheck);
    await new Promise(this.initList);
    this.setState({ loading: false });
  };

  initDict = (resolve) => {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: [
          'branch',
          'branchCampus',
          'branchAllCampus',
          'productTypeTree',
          'year',
          'quarter',
          'phase',
          'weekDay',
          'grade',
          'subject',
          'twoTcMainTc',
          'twoTcAuxTc',
          'smallClassMainTc',
          'smallClassAuxTc',
          'courseSeries',
        ],
        callback() {
          resolve();
        },
      },
    });
  };

  initSKeyWithCheck = (resolve) => {
    if (this.props.branchDict && this.props.branchAllCampusDict && this.props.yearDict && this.props.productTypeTree) {
      this.initSKey(resolve);
    } else {
      const _self = this;
      setTimeout(() => {
        _self.initSKeyWithCheck(resolve);
      }, 100);
    }
  };

  initSKey = (resolve) => {
    let payload = {};
    const { brenchId, campusId, institution } = this.props.user;
    const isGroup = brenchId === campusId;
    payload.branchId = isGroup ? this.props.branchDict[0].val : brenchId;
    payload.blCampusId = null;
    payload.productTypeId = institution === twoTeacherInstitution ? TWO_TEACHER_VAL : SMALL_CLASS_VAL;
    payload.yearId = getDictValByText(this.props.yearDict, new Date().getFullYear().toString()) || null;
    this.setState(
      {
        ...payload,
        sKeyStorage: clone(payload),
      },
      () => {
        resolve();
      },
    );
  };

  initList = (resolve, reject) => {
    this.loadListData()
      .then((listData) => {
        let { rows, records, page, originalClassQuantity, associatedQuantity } = listData;
        this.setState(() => ({
          rows,
          total: records,
          page,
          originalClassQuantity,
          associatedQuantity,
        }));
        resolve();
      })
      .catch((err) => {
        showMessage(err.text, 'error');
        reject();
      });
  };

  loadListData = async (params) => {
    const {
      className,
      branchId,
      blCampusId,
      productTypeId,
      yearId,
      quarterId,
      phaseId,
      gradeId,
      subjectId,
      page,
      pageSize,
      classTypeId,
      mainTeacherId,
      assistTeacherId,
      associatedStatus,
      classLevel,
      courseSeries,
      total,
    } = this.state;
    const apiParams = {
      className: className.trim(),
      branchId,
      blCampusId: (blCampusId && blCampusId.join(',')) || null,
      productType: productTypeId,
      yearId,
      quarterId,
      phaseId: (phaseId && phaseId.join(',')) || null,
      gradeId: (gradeId && gradeId.join(',')) || null,
      subjectId: (subjectId && subjectId.join(',')) || null,
      classTypeId: classTypeId || null,
      classLevel: (classLevel && classLevel.join(',')) || null,
      courseSeriesId: courseSeries,
      mainTeacherName: mainTeacherId,
      assistTeacherName: assistTeacherId,
      page,
      rows: pageSize,
      getTotal: false,
      ...params,
    };
    associatedStatus && (apiParams.associatedStatus = Number(associatedStatus));
    const fetchList = [Loader.getLinkDetailRange(apiParams), Loader.getIndexLinkList(apiParams)];
    if (page === 1) {
      fetchList.push(Loader.getIndexLinkList({ ...apiParams, getTotal: true }));
    }
    let [linkRange, detail, extraDetail] = await Promise.all(fetchList);
    detail = {
      ...detail,
      records: extraDetail ? extraDetail.records : total,
      page,
      ...linkRange.data,
    };
    return detail;
  };

  loadList = async () => {
    this.setState({ listLoading: true, selectedRowKeys: [] });
    let { rows, records, page, originalClassQuantity, associatedQuantity } = await this.loadListData();
    this.setState({
      listLoading: false,
      rows,
      total: records,
      page,
      originalClassQuantity,
      associatedQuantity,
    });
  };

  updateSKey = (...params) => {
    this.setState(...params);
  };

  // 模态框手动关联
  onSubmitLinkByHand = async (seriesClassIdArray) => {
    if (!(Array.isArray(seriesClassIdArray) && seriesClassIdArray.length)) {
      showMessage('请选择续报班级！', 'error');
      return false;
    }
    const { classId, productTypeId } = this.state.selectedRecord;
    const seriesClassId = seriesClassIdArray.map((classDetail) => classDetail.classId).join(',');
    try {
      await Loader.indexHandLink({
        originalClassId: classId,
        seriesClassId,
        productType: productTypeId,
      });
      showMessage('手动关联成功', 'success');
      this.setState({ showSelectClassModal: false, selectedRecord: {} });
      this.search();
    } catch (e) {
      showMessage(`关联失败:${e.text}`, 'error');
      this.setState({ showSelectClassModal: false, selectedRecord: {} });
    }
  };
  onReset = () => {
    this.setState({
      ...defaultSKey,
      ...this.state.sKeyStorage,
    });
  };

  onSearch = () => {
    this.search();
  };

  search = async () => {
    this.setState({ page: 1 }, () => {
      this.loadList();
    });
  };
  // 切换页面
  changePage = async ({ current, pageSize }) => {
    this.setState(
      {
        pageSize,
        page: current,
      },
      () => {
        this.loadList();
      },
    );
  };

  // 导出excel
  exportExcel = async () => {
    const { transformArrayToString } = utils;
    const {
      productTypeId,
      branchId,
      blCampusId,
      yearId,
      quarterId,
      phaseId,
      className,
      associatedStatus,
      gradeId,
      subjectId,
      classTypeId,
      mainTeacherId,
      assistTeacherId,
      classLevel,
      courseSeries,
    } = this.state;
    const { branchDict, branchAllCampusDict: campusDict, yearDict, quarterDict, gradeDict } = this.props;
    const fetchParams = {
      className: className.trim(),
      classLevel: transformArrayToString(classLevel),
      courseSeriesId: courseSeries,
      mainTeacherName: mainTeacherId,
      assistTeacherName: assistTeacherId,
      branchId,
      blCampusId: transformArrayToString(blCampusId),
      productType: productTypeId,
      yearId,
      quarterId,
      phaseId: transformArrayToString(phaseId),
      gradeId: transformArrayToString(gradeId),
      subjectId: transformArrayToString(subjectId),
      classTypeId: classTypeId || null,
      associatedStatus,
    };
    // 分公司名称 + 学习中心名称 + 年份 + 季度 + 年级 + 文件名
    let _branchName = getDictTextByVal(branchDict, branchId);
    let _campusName = '';
    if (blCampusId && blCampusId.length > 0) {
      if (blCampusId.length === 1) {
        _campusName = getDictTextByVal(campusDict, blCampusId[0]);
      } else {
        _campusName = blCampusId.map((item) => getDictTextByVal(campusDict, item)).join('/');
      }
    }
    let _yearName = getDictTextByVal(yearDict, yearId);
    let _quarterName = getDictTextByVal(quarterDict, quarterId);
    let _gradeName = '';
    if (gradeId && gradeId.length > 0) {
      if (gradeId.length === 1) {
        _gradeName = getDictTextByVal(gradeDict, gradeId[0]);
      } else {
        _gradeName = gradeId.map((item) => getDictTextByVal(gradeDict, item)).join('/');
      }
    }
    let fileName = `${_branchName}${_campusName}${_yearName}${_quarterName}${_gradeName}续报关联`;

    this.setState({ loading: true });
    try {
      await utils.exportExcel({
        apiUrl: 'ClassRenewController/exportOriginalClass.do',
        data: fetchParams,
        fileName: fileName,
      });
      this.setState({ loading: false });
      showMessage('导出成功！', 'success');
    } catch (err) {
      this.setState({ loading: false });
      showMessage(err.resultMessage || '导出失败', 'error');
    }
  };

  openPage = (page) => {
    const pagePathMap = {
      linkStudent: 'LinkStd',
      linkClass: 'linkClass',
    };
    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path: `/continueRegManage/${pagePathMap[page]}`,
        refresh: true,
      },
    });
  };

  buildColumns = () => {
    const isTwoTeacher = this.state.productTypeId === TWO_TEACHER_VAL;
    const mainTeacherNameTitle = isTwoTeacher ? '主讲老师' : '班主任';
    const assistTeacherNameTitle = isTwoTeacher ? '辅导老师' : '任课老师';
    const { productTypeTree, yearDict, subjectDict, gradeDict } = this.props;
    return [
      {
        title: '班级名称',
        key: 'className',
        dataIndex: 'className',
        width: COL_WIDTH_L,
        fixed: 'left',
        render: (text) => (
          <div title={text} className={style.ContinueRegWrapText}>
            {text}
          </div>
        ),
      },
      {
        title: '科目',
        key: 'subjectId',
        dataIndex: 'subjectId',
        width: COL_WIDTH_S,
        render: (text) => getDictTextByVal(subjectDict, text),
      },
      {
        title: '年级',
        key: 'gradeId',
        dataIndex: 'gradeId',
        width: COL_WIDTH_S,
        render: (text) => getDictTextByVal(gradeDict, text),
      },
      {
        title: '学习中心',
        key: 'blCampusName',
        dataIndex: 'blCampusName',
        width: COL_WIDTH_M,
        render: (text) => text || '暂无',
      },
      {
        title: '年份',
        key: 'yearId',
        dataIndex: 'yearId',
        width: COL_WIDTH_M,
        render: (text) => getDictTextByVal(yearDict, text),
      },
      {
        title: '期数/星期数',
        key: 'phaseName',
        dataIndex: 'phaseName',
        width: COL_WIDTH_M,
        render: (text) => text || '暂无',
      },
      {
        title: '班级类型',
        key: 'classTypeName',
        dataIndex: 'classTypeName',
        width: COL_WIDTH_M,
        render: (text) => text || '暂无',
      },
      {
        title: '班级层次',
        key: 'classLevelName',
        dataIndex: 'classLevelName',
        width: COL_WIDTH_M,
        render: (text) => text || '暂无',
      },
      {
        title: mainTeacherNameTitle,
        key: 'mainTeacherName',
        dataIndex: 'mainTeacherName',
        width: COL_WIDTH_L,
        render: (data) => data || '其他',
      },
      {
        title: assistTeacherNameTitle,
        key: 'assistTeacherName',
        dataIndex: 'assistTeacherName',
        width: COL_WIDTH_M,
        render: (data) => data || '无',
      },
      {
        title: '上课时间',
        key: 'classTime',
        dataIndex: 'classTime',
        width: COL_WIDTH_L,
        render: (text, record) => {
          const { classStartDate, classStartTime } = record;
          if (!classStartDate && !classStartTime) return '暂无';
          return `${classStartDate} ${classStartTime}`;
        },
      },
      { title: '关联班级数量', key: 'associatedQuantity', dataIndex: 'associatedQuantity', width: COL_WIDTH_M },
      {
        title: '操作',
        key: 'action',
        fixed: 'right',
        width: COL_WIDTH_M,
        getActionConfig: (text, record, index) => [
          { text: '详情', onClick: () => this.setState({ selectedRecord: record, showDetailModal: true }) },
          { text: '关联', onClick: () => this.setState({ selectedRecord: record, showSelectClassModal: true }) },
        ],
      },
    ];
  };

  static getIndexSearchProps(state) {
    return {
      className: state.className,
      branchId: state.branchId,
      blCampusId: state.blCampusId,
      productTypeId: state.productTypeId,
      yearId: state.yearId,
      quarterId: state.quarterId,
      phaseId: state.phaseId,
      gradeId: state.gradeId,
      subjectId: state.subjectId,
      classTypeId: state.classTypeId,
      classLevel: state.classLevel,
      mainTeacherId: state.mainTeacherId,
      assistTeacherId: state.assistTeacherId,
      associatedStatus: state.associatedStatus,
      courseSeries: state.courseSeries,
    };
  }

  render() {
    const {
      loading,
      listLoading,
      rows,
      page,
      total,
      pageSize,
      showDetailModal,
      selectedRecord,
      showSelectClassModal,
      originalClassQuantity,
      associatedQuantity,
    } = this.state;

    const { classId, yearId, quarterId, gradeId, subjectId, blCampusId } = selectedRecord || {};
    const initKey = {
      originalClassId: classId,
      yearId,
      quarterId,
      gradeId,
      subjectId,
      blCampusId,
    };
    const linkPercent = originalClassQuantity ? ((associatedQuantity / originalClassQuantity) * 100).toFixed(1) : 0;
    const { btnAuths } = this.props;
    return (
      <Page
        fitWindowHeight={true}
        loading={loading}
        route={this.props.route}
        subHeader={
          <div className={style.listDetail}>
            总班级数量：<span>{originalClassQuantity}</span>个， 已关联班级数量<span>{associatedQuantity}</span>
            个，关联完成率<span>{linkPercent}%</span>
          </div>
        }
        headerToolBar={
          <div className={style[`${classPrefix}-header-toolbar-btns`]}>
            {btnAuths.includes('REG_CLASS_BTN') && (
              <Button type="primary" onClick={() => this.openPage('linkClass')}>
                班续
              </Button>
            )}
            {btnAuths.includes('REG_STUDENT_BTN') && (
              <Button type="primary" onClick={() => this.openPage('linkStudent')}>
                人续
              </Button>
            )}
            {btnAuths.includes('EXPORT_REG_BTN') && <Button onClick={this.exportExcel}>导出excel</Button>}
            <IndexSearch
              loading={loading}
              updateSearchKey={this.updateSKey}
              onSearch={this.onSearch}
              onReset={this.onReset}
              {...ContinueIndex.getIndexSearchProps(this.state)}
            />
          </div>
        }
      >
        <List
          columns={this.buildColumns()}
          loading={listLoading}
          dataSource={rows || []}
          pagination={{
            current: page,
            pageSize,
            total,
          }}
          onChange={this.changePage}
        />
        {showDetailModal && (
          <DetailModal
            selectedRecord={selectedRecord}
            onClose={() => this.setState({ showDetailModal: false, selectedRecord: {} })}
          />
        )}
        {showSelectClassModal && selectedRecord && (
          <ModalSelectClass
            initSKey={initKey}
            title="选择关联的班级"
            onClose={() => this.setState({ showSelectClassModal: false, selectedRecord: {} })}
            onSubmit={this.onSubmitLinkByHand}
          />
        )}
      </Page>
    );
  }
}

let productTypeTreeStorage = null;
let quarterDictStorage = null;

const mapStateToProps = (state) => {
  // 过滤与产品类型相关字典
  if (!productTypeTreeStorage && state.dict.productTypeTree) {
    productTypeTreeStorage = state.dict.productTypeTree.filter((item) => productTypeAllow.includes(item.val));
  }
  // 过滤季度字典中的全年选项
  if (!quarterDictStorage && state.dict.quarter) {
    quarterDictStorage = state.dict.quarter.filter((item) => item.val !== 'DAT0000000245');
  }
  return {
    btnAuths: state.account.btnAuths,
    user: state.account.user,
    // 数据字典
    branchDict: state.dict.branch,
    campusDict: state.dict.campus,
    branchAllCampusDict: state.dict.branchAllCampus,
    productTypeTree: productTypeTreeStorage,
    yearDict: state.dict.year,
    quarterDict: quarterDictStorage,
    phaseDict: state.dict.phase,
    courseSeriesDict: state.dict.courseSeries,
    weekDayDict: state.dict.weekDay,
    gradeDict: state.dict.grade,
    subjectDict: state.dict.subject,
    twoTcMainTcDict: state.dict.twoTcMainTc,
    twoTcAuxTcDict: state.dict.twoTcAuxTc,
    smallClassMainTcDict: state.dict.smallClassMainTc,
    smallClassAuxTcDict: state.dict.smallClassAuxTc,
  };
};

export default connect(mapStateToProps)(ContinueIndex);
