import React from 'react';
import Modal from 'COMPONENT/Modal';
import Block from 'COMPONENT/Block';
import List from 'COMPONENT/List';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { TWO_TEACHER_VAL } from '../../constants/config';
import * as loaders from '../../loaders';
import styles from './styles.less';
import style from '../style.less';

const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;

export default class DetailModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false,
      data: [],
      pageNo: 1,
      pageSize: 10,
      total: null,
    };
  }

  delete = async (record) => {
    const { originalClassId, seriesClassId } = record;
    showConfirm(`确认删除当前续报班关联关系？`, async () => {
      try {
        await loaders.deleteIndexClassLink({
          originalClassId,
          seriesClassId,
        });
        showMessage('删除成功', 'success');
        this.initContent();
      } catch (e) {
        showMessage(e.resultMessage, 'error');
      }
    });
  };
  buildColumns = () => {
    const { productTypeId } = this.props.selectedRecord;
    const isTwoTeacher = productTypeId === TWO_TEACHER_VAL;
    const mainTeacherNameTitle = isTwoTeacher ? '主讲老师' : '班主任';
    const assistTeacherNameTitle = isTwoTeacher ? '辅导老师' : '任课老师';
    return [
      {
        title: '班级名称',
        key: 'className',
        dataIndex: 'className',
        width: COL_WIDTH_XL,
        render: (text) => <div>{text}</div>,
      },
      { title: '学习中心', key: 'blCampusName', dataIndex: 'blCampusName', width: COL_WIDTH_M },
      {
        title: mainTeacherNameTitle,
        key: 'mainTeacherName',
        dataIndex: 'mainTeacherName',
        width: COL_WIDTH_M,
        render: (data) => data || '其他',
      },
      {
        title: assistTeacherNameTitle,
        key: 'assistTeacherName',
        dataIndex: 'assistTeacherName',
        width: COL_WIDTH_M,
        render: (data) => data || '无',
      },
      { title: '班级层次', key: 'classLevelName', dataIndex: 'classLevelName', width: COL_WIDTH_S },
      { title: '期数/星期数', key: 'phaseOrWeek', dataIndex: 'phaseName', width: COL_WIDTH_M },
      { title: '上课时间', key: 'yearName', dataIndex: 'classStartTime', width: COL_WIDTH_M },
      {
        title: '操作',
        key: 'action',
        width: COL_WIDTH_M,
        getActionConfig: (text, record) => [{ text: '删除', onClick: () => this.delete(record) }],
      },
    ];
  };

  componentDidMount() {
    this.initContent();
  }

  initContent = async () => {
    this.setState({ loading: true });
    await this.loadList();
    this.setState({ loading: false });
  };

  loadListData = async () => {
    const { pageNo, pageSize, total } = this.state;
    const { selectedRecord } = this.props;
    const apiParams = {
      originalClassId: selectedRecord.classId,
      page: pageNo,
      rows: pageSize,
      getTotal: false,
    };

    const fetchList = [loaders.getIndexClassSeriesDetail(apiParams)];
    if (pageNo === 1) {
      fetchList.push(loaders.getIndexClassSeriesDetail({ ...apiParams, getTotal: true }));
    }
    let [detail, extraDetail] = await Promise.all(fetchList);
    detail = {
      ...detail,
      rowCount: extraDetail ? extraDetail.records : total,
      pageNo,
    };
    return detail;
  };

  updateState = (changedState) => {
    const newState = {
      ...this.state,
      ...changedState,
    };
    return new Promise((resolve) => {
      this.setState(newState, () => {
        resolve();
      });
    });
  };

  pageChange = async ({ current, pageSize }) => {
    await this.updateState({ loading: true, pageSize, pageNo: current });
    const list = await this.loadListData();
    this.setState({
      loading: false,
      data: list.rows,
      total: list.rowCount,
      pageNo: list.pageNo,
    });
  };

  loadList = async () => {
    try {
      this.setState({ loading: true });
      let { rows, pageNo, rowCount } = await this.loadListData();
      this.setState({
        loading: false,
        data: rows,
        total: rowCount,
        page: pageNo,
      });
    } catch (e) {
      showMessage(e.text, 'error');
      this.setState({ loading: false });
    }
  };

  render() {
    const { loading, data, pageNo, pageSize, total } = this.state;
    const { onClose } = this.props;
    return (
      <Modal title="详情" width={1300} cancelText="关闭" onCancel={onClose} okBtnShow={false} loading={loading}>
        <div className={styles.regIndexDetailModal}>
          <Block title="关联详情">
            <List
              columns={this.buildColumns()}
              dataSource={data || []}
              pagination={{
                current: pageNo,
                pageSize,
                total: total,
              }}
              onChange={this.pageChange}
            />
          </Block>
        </div>
      </Modal>
    );
  }
}
