import React from 'react';
import Checkbox from 'COMPONENT/Checkbox';
import Select from 'COMPONENT/Select';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import styles from '../styles.less';

const startTimeDict = [
  { text: '严格匹配，精确到时分匹配', val: '0' },
  { text: '范围匹配（上午06:00-12:00；下午12:00-17:00；晚上17:00-22:00）', val: '1' },
];
export default class CanSelectRule extends React.Component {
  constructor(props) {
    super(props);
    const { selectRuleCampus, selectRuleClassLevel } = this.props;
    this.state = {
      checkboxCampus: true,
      checkboxClassLevel: true,
      checkboxMainTeacher: false,
      checkboxAssistTeacher: false,
      checkboxStartTime: false,
      selectCampus: selectRuleCampus,
      selectClassLevel: selectRuleClassLevel,
      selectStartTime: '0',
    };
  }

  componentWillReceiveProps(nextProps) {
    const { isCampusSingle, isClassLevelSingle, selectRuleCampus, selectRuleClassLevel } = nextProps;
    const paramsState = {};
    if (!isCampusSingle) {
      paramsState.checkboxCampus = true;
    }
    if (!isClassLevelSingle) {
      paramsState.checkboxClassLevel = true;
    }
    if (selectRuleClassLevel !== this.props.selectRuleClassLevel) {
      paramsState.selectClassLevel = selectRuleClassLevel;
    }
    if (selectRuleCampus !== this.props.selectRuleCampus) {
      paramsState.selectCampus = selectRuleCampus;
    }
    if (Object.keys(paramsState).length) {
      this.setState(paramsState);
    }
  }

  updateValue = (key, val) => {
    this.setState({ [key]: val });
  };

  // 通用checkbox处理事件
  handleNormalCheckbox = (type, e) => {
    this.setState({
      [type]: e.target.checked,
    });
  };
  // 学习中心
  handleCampusCheckbox = (e) => {
    const { isCampusSingle } = this.props;
    const val = e.target.checked;
    // 不匹配并且学习中心不是单选
    if (!val && !isCampusSingle) {
      showMessage('请选择单个学习中心', 'error');
      return false;
    }
    this.setState({
      checkboxCampus: val,
    });
  };

  //班级层次
  handleClassLevelCheckbox = (e) => {
    const { isClassLevelSingle } = this.props;
    const val = e.target.checked;
    // 不匹配并且班级层次不是单选
    if (!val && !isClassLevelSingle) {
      showMessage('请选择单个班级层次', 'error');
      return false;
    }
    this.setState({
      checkboxClassLevel: val,
    });
  };

  // 处理向外派发的数据
  getSelectRuleState = () => {
    const {
      checkboxCampus,
      checkboxClassLevel,
      checkboxMainTeacher,
      checkboxAssistTeacher,
      checkboxStartTime,
      selectCampus,
      selectClassLevel,
      selectStartTime,
    } = this.state;
    const returnObj = {
      isMainTeacher: checkboxMainTeacher ? 1 : 0,
      isAssistTeacher: checkboxAssistTeacher ? 1 : 0,
    };
    if (checkboxStartTime) {
      returnObj.startClassTimeCode = selectStartTime;
    }
    if (!checkboxCampus && selectCampus) {
      returnObj.blCampusId = selectCampus;
    }
    if (!checkboxClassLevel && selectClassLevel) {
      returnObj.classLevel = selectClassLevel;
    }
    return returnObj;
  };
  render() {
    const { mainTeacherNameTitle, assistTeacherNameTitle, campusDict, classLevelDict } = this.props;
    const {
      checkboxCampus,
      checkboxClassLevel,
      checkboxMainTeacher,
      selectStartTime,
      checkboxAssistTeacher,
      checkboxStartTime,
      selectCampus,
      selectClassLevel,
    } = this.state;
    return (
      <div className={styles.canSelectRuleWrapper}>
        <Checkbox onChange={this.handleCampusCheckbox} checked={checkboxCampus}>
          学习中心
        </Checkbox>
        {!checkboxCampus && (
          <Select
            placeholder="学习中心"
            dict={campusDict}
            value={selectCampus}
            onChange={(val) => this.updateValue('selectCampus', val)}
            allowClear={false}
            showSearch
          />
        )}
        <Checkbox onChange={this.handleClassLevelCheckbox} checked={checkboxClassLevel}>
          班级层次
        </Checkbox>
        {!checkboxClassLevel && (
          <Select
            allowClear={false}
            placeholder="班级层次"
            dict={classLevelDict}
            value={selectClassLevel}
            onChange={(val) => this.updateValue('selectClassLevel', val)}
            showSearch={false}
          />
        )}
        <Checkbox onChange={(e) => this.handleNormalCheckbox('checkboxMainTeacher', e)} checked={checkboxMainTeacher}>
          {mainTeacherNameTitle}
        </Checkbox>
        <Checkbox
          onChange={(e) => this.handleNormalCheckbox('checkboxAssistTeacher', e)}
          checked={checkboxAssistTeacher}
        >
          {assistTeacherNameTitle}
        </Checkbox>
        <Checkbox onChange={(e) => this.handleNormalCheckbox('checkboxStartTime', e)} checked={checkboxStartTime}>
          上课开始时间
        </Checkbox>
        {checkboxStartTime && (
          <Select
            allowClear={false}
            placeholder="上课开始时间"
            dict={startTimeDict}
            value={selectStartTime}
            onChange={(val) => this.updateValue('selectStartTime', val)}
            showSearch={false}
          />
        )}
      </div>
    );
  }
}
