import React from 'react';
import { Switch } from 'antd';
import MultiSelect from 'COMPONENT/MultiSelect';
import Select from 'COMPONENT/Select';
import Popover from 'COMPONENT/Popover';
import Icon from 'COMPONENT/Icon';
import List from 'COMPONENT/List';
import { quarterMap, quarterText, quarterToDict } from '../../constants/config';
import styles from '../styles.less';

const { COL_WIDTH_M } = List;
const { winter, spring, summer, autumn } = quarterMap;
const quarterToArrayMap = {
  [winter]: [spring],
  [spring]: [summer, autumn],
  [summer]: [autumn],
  [autumn]: [winter, spring],
};
export default class ExtraRule extends React.Component {
  constructor(props) {
    super(props);
    const ruleSubjectArray = [
      { grade: null, subject: null },
      { grade: null, subject: null },
      {
        grade: null,
        subject: null,
      },
    ];
    this.state = {
      phaseOrWeekVal: false,
      subjectVal: false,
      oneToManyVal: true,
      // 增开科目
      ruleSubjectArray,
    };
  }

  //用于下面buildPhaseOrWeekColumns方法内用于截断或者补长currentChangeState的方法
  balanceCurrentState = (selectQuarterId, dictLength) => {
    let returnChangeState;
    const currentChangeState = this.state[selectQuarterId];
    if (currentChangeState) {
      // 如果当前state的长度大于等于字典的长度
      if (currentChangeState.length >= dictLength) {
        // 截断
        returnChangeState = currentChangeState.slice(0, dictLength);
      } else {
        // 小于的话进行补长
        const extraLengthArray = Array.from(new Array(dictLength - currentChangeState.length)).map(() => []);
        returnChangeState = currentChangeState.concat(extraLengthArray);
      }
    } else {
      // 没有初始化过直接初始化二维数组
      returnChangeState = Array.from(new Array(dictLength)).map(() => []);
    }
    return returnChangeState;
  };

  buildPhaseOrWeekColumns = () => {
    const { quarterId } = this.props;

    // 第一个列表的字典数据 用于下面生成二维数组
    const firstDict = this.props[quarterToDict[quarterId]];

    // 需要生成多少个下拉的列表
    const hasSelectArray = quarterToArrayMap[quarterId];
    // 对生成的列表进行select的组件填充
    const selectColumnsArray = hasSelectArray.map((selectQuarterId, index) => {
      const title = quarterText[selectQuarterId];
      const selectDict = this.props[quarterToDict[selectQuarterId]];
      // 获取值得state 初次的时候没有声明在state上 只有在下拉框切换才会setState 初始化给个默认的二维空数组

      // 还会有这么一种情况当前值得数组长度小于第一列表字典的长度 这样子会造成超出长度的下拉框筛选不到
      // 比如字典长度是7 所以有7个下拉 但是this.state[selectQuarterId]只有上一次的4个
      const currentChangeState = this.balanceCurrentState(selectQuarterId, firstDict.length);

      const handleChange = (val, selectIndex) => {
        const newSelectQuarterIdArray = currentChangeState.map((state, stateIndex) => {
          return stateIndex === selectIndex ? val || [] : state;
        });
        this.setState({
          [selectQuarterId]: newSelectQuarterIdArray,
        });
      };
      return {
        title,
        key: selectQuarterId,
        dataIndex: selectQuarterId,
        width: COL_WIDTH_M,
        render: (text, record, selectIndex) => (
          <MultiSelect
            allowClear={false}
            placeholder="请选择"
            dict={selectDict}
            value={currentChangeState[selectIndex]}
            onChange={(val) => handleChange(val, selectIndex)}
            showSearch={false}
          />
        ),
      };
    });
    return [
      { title: quarterText[quarterId], key: 'text', dataIndex: 'text', width: COL_WIDTH_M },
      ...selectColumnsArray,
    ];
  };
  buildPhaseOrWeekDataSources = () => {
    const { quarterId } = this.props;
    // 第一行的字典数据
    return this.props[quarterToDict[quarterId]];
  };

  // 更新增开科目
  updateRuleSubjectSelect = (type, val, index) => {
    const newRuleSubjectArray = this.state.ruleSubjectArray.map((ruleSubject, ruleIndex) =>
      index === ruleIndex ? { ...ruleSubject, [type]: val } : ruleSubject,
    );
    this.setState({
      ruleSubjectArray: newRuleSubjectArray,
    });
  };

  getExtraRuleState = () => {
    const { quarterId } = this.props;

    // 用于下面进行截取 因为有可能会出现长度大于的情况
    const firstLabelDict = this.props[quarterToDict[quarterId]];
    const { phaseOrWeekVal, subjectVal, oneToManyVal, ruleSubjectArray } = this.state;

    const returnExtraRule = {
      phaseRule: null,
      addSubjectRules: null,
      isOneOnMany: oneToManyVal ? 1 : 0,
    };
    if (subjectVal) {
      returnExtraRule.addSubjectRules = ruleSubjectArray
        .filter(({ grade, subject }) => grade && subject)
        .map(({ grade, subject }) => {
          return {
            gradeId: grade,
            subjectId: subject,
          };
        });
    }
    // 处理期数星期数数据
    const currentQuarterArray = quarterToArrayMap[quarterId].map((quarterId) => this.state[quarterId]);
    // 填写了季度
    const isFillQuarter = currentQuarterArray.every((item) => Array.isArray(item));
    if (phaseOrWeekVal && isFillQuarter) {
      const quarter = {
        originalId: quarterId,
        seriesIds: quarterToArrayMap[quarterId],
      };
      // 当前真正填写的期数星期数
      const phases = firstLabelDict.map((dict, index) => {
        return {
          originalId: dict.val,
          seriesIds: currentQuarterArray.map((quarterArray) => quarterArray[index].join(',')),
        };
      });
      returnExtraRule.phaseRule = {
        quarter,
        phases,
      };
    }
    return returnExtraRule;
  };

  render() {
    const { quarterId, gradeDict, subjectDict } = this.props;
    const { phaseOrWeekVal, subjectVal, ruleSubjectArray, oneToManyVal } = this.state;
    const quarterLabelArray = quarterToArrayMap[quarterId];
    const ListLength = ((quarterLabelArray ? quarterLabelArray.length : 0) + 1) * 200;
    return (
      <div className={styles.extraRuleWrapper}>
        <div className={styles.extraRuleItem}>
          <div className={styles.extraSwitchWrapper}>
            <Switch checked={phaseOrWeekVal} onChange={(checked) => this.setState({ phaseOrWeekVal: checked })} />
            <span className={styles.extraRuleSwitch}>
              <span>期数/星期数</span>
              <Popover content="满足春秋季星期数匹配寒暑假期数的课程，如可以实现秋季星期六匹配寒假第一期的课程">
                <Icon type="question_circle" size={12} style={{ marginLeft: 4 }} />
              </Popover>
            </span>
          </div>
          {quarterId && phaseOrWeekVal && (
            <div style={{ width: ListLength }}>
              <List columns={this.buildPhaseOrWeekColumns()} dataSource={this.buildPhaseOrWeekDataSources()} />
            </div>
          )}
        </div>
        <div className={styles.extraRuleItem}>
          <div className={styles.extraSwitchWrapper}>
            <Switch checked={subjectVal} onChange={(checked) => this.setState({ subjectVal: checked })} />
            <span className={styles.extraRuleSwitch}>
              <span>增开科目</span>
              <Popover content="满足升年级增科的关联。如设置了初二物理，则初一的原班可以关联到初二的物理班，而其他的年级无法关联到初二的物理班">
                <Icon type="question_circle" size={12} style={{ marginLeft: 4 }} />
              </Popover>
            </span>
          </div>
          {subjectVal && (
            <div className={styles.ruleSubject}>
              {ruleSubjectArray.map((ruleSubject, index) => (
                <div key={index} className={styles.ruleSubjectSelectWrapper}>
                  <Select
                    placeholder="请选择年级"
                    dict={gradeDict}
                    value={ruleSubject.grade}
                    onChange={(val) => this.updateRuleSubjectSelect('grade', val, index)}
                    allowClear={false}
                  />
                  <Select
                    placeholder="请选择科目"
                    dict={subjectDict}
                    value={ruleSubject.subject}
                    onChange={(val) => this.updateRuleSubjectSelect('subject', val, index)}
                    allowClear={false}
                  />
                </div>
              ))}
            </div>
          )}
        </div>
        <div className={styles.extraRuleItem}>
          <div className={styles.extraSwitchWrapper}>
            <Switch checked={oneToManyVal} onChange={(checked) => this.setState({ oneToManyVal: checked })} />
            <span className={styles.extraRuleSwitch}>
              <span>允许一对多关联</span>
              <Popover content="开启：允许一个原班关联多个续报班；关闭：只允许一个原班关联一个续报班，超过多个提示关联冲突">
                <Icon type="question_circle" size={12} style={{ marginLeft: 4 }} />
              </Popover>
            </span>
          </div>
        </div>
      </div>
    );
  }
}
