import React from 'react';
import { connect } from 'dva';
import { Tabs } from 'antd';
import Page from 'COMPONENT/Page';
import Block from 'COMPONENT/Block';
import Checkbox from 'COMPONENT/Checkbox';
import TextLink from 'COMPONENT/TextLink';
import Popover from 'COMPONENT/Popover';
import Icon from 'COMPONENT/Icon';
import Button from 'COMPONENT/Button';
import Modal from 'COMPONENT/Modal';
import ImportExcel from '../components/ImportExcel';
import ClassSearchBar from '../components/ClassSearchBar';
import RuleDetailModal from './modals/RuleDetailModal';
import CanSelectRule from './components/CanSelectRule';
import ExtraRule from './components/ExtraRule';
import styles from './styles.less';
import { getDictValByText } from 'UTIL/dict';
import { buildBranchCampusDict, buildClassTypeDict, buildClassLevelDict } from 'MODEL/dict/utils';
import { TWO_TEACHER_VAL, SMALL_CLASS_VAL, productTypeAllow } from '../constants/config';
import { twoTeacherInstitution } from 'CONST/config';
import * as loaders from '../loaders';
import { getDefaultQuarter, transformArrayToString } from '../utils';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';

// 表单初始值
const defaultSKey = {
  branchId: null,
  blCampusId: null,
  productTypeId: null,
  yearId: null,
  quarterId: getDefaultQuarter(),
  phaseId: null,
  gradeId: null,
  subjectId: null,
  classTypeId: null,
  classLevel: null,
  mainTeacherId: null,
  assistTeacherId: null,
  associatedStatus: null,
  courseSeries: null,
  isConflict: null,
};

const defaultRuleOptions = [
  { value: 'year', label: '年份' },
  { value: 'quarter', label: '季度' },
  { value: 'grade', label: '年级' },
  { value: 'subject', label: '科目' },
];

const TabPane = Tabs.TabPane;
const CheckboxGroup = Checkbox.Group;

// 班续的导入接口地址
const importExcelUrl = '/ClassRenewController/sumitImportClassRenew.do';

class LinkClass extends React.Component {
  constructor(props) {
    super(props);
    const defaultRuleValue = defaultRuleOptions.map((rule) => rule.value);
    this.state = {
      loading: false,
      defaultRuleValue,
      showRuleModal: false,
      //搜索表单值
      ...defaultSKey,
      totalLink: 0,
      unLink: 0,
    };
  }

  componentWillMount() {
    this.init();
  }

  init = async () => {
    this.setState({ loading: true });
    try {
      await new Promise(this.initDict);
      await new Promise(this.initSKeyWithCheck);
      await new Promise(this.initList);
    } catch (e) {
      console.error(e);
    }
    this.setState({ loading: false });
  };
  initDict = (resolve) => {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: [
          'branch',
          'campus',
          'branchCampus',
          'branchAllCampus',
          'productTypeTree',
          'year',
          'quarter',
          'phase',
          'weekDay',
          'grade',
          'subject',
          'twoTcMainTc',
          'twoTcAuxTc',
          'smallClassMainTc',
          'smallClassAuxTc',
          'courseSeries',
        ],
        callback() {
          resolve();
        },
      },
    });
  };

  search = () => {
    this.setState({
      loading: true,
    });
    this.loadListData()
      .then(({ data }) => {
        this.setState(() => ({
          totalLink: data.originalClassQuantity,
          unLink: data.originalClassQuantity - data.associatedQuantity,
          loading: false,
        }));
      })
      .catch((err) => {
        this.setState({
          loading: true,
        });
        showMessage(err.text, 'error');
      });
  };
  loadListData = () => {
    const params = this.getClassSearchProps(this.state);
    params.productType = params.productTypeId;
    delete params.productTypeId;
    params.assistTeacherName = params.assistTeacherId;
    params.mainTeacherName = params.mainTeacherId;
    params.courseSeriesId = params.courseSeries;
    delete params.assistTeacherId;
    delete params.mainTeacherId;
    delete params.courseSeries;
    // 把数组拼装成字符转
    const apiParams = Object.keys(params).reduce((obj, cur) => {
      obj[cur] = Array.isArray(params[cur]) ? transformArrayToString(params[cur]) : params[cur];
      return obj;
    }, {});
    return loaders.getLinkDetailRange(apiParams);
  };
  initList = (resolve, reject) => {
    this.loadListData()
      .then(({ data }) => {
        this.setState(() => ({
          totalLink: data.originalClassQuantity,
          unLink: data.originalClassQuantity - data.associatedQuantity,
        }));
        resolve();
      })
      .catch((err) => {
        showMessage(err.text, 'error');
        reject();
      });
  };

  initSKeyWithCheck = (resolve) => {
    if (this.props.branchDict && this.props.branchCampusDict && this.props.yearDict && this.props.productTypeTree) {
      this.initSKey(resolve);
    } else {
      const _self = this;
      setTimeout(() => {
        _self.initSKeyWithCheck(resolve);
      }, 100);
    }
  };

  initSKey = (resolve) => {
    let payload = {};
    const { brenchId, campusId, institution } = this.props.user;
    const isGroup = brenchId === campusId;
    payload.branchId = isGroup ? this.props.branchDict[0].val : brenchId;
    payload.blCampusId = null;
    payload.productTypeId = institution === twoTeacherInstitution ? TWO_TEACHER_VAL : SMALL_CLASS_VAL;
    payload.yearId = getDictValByText(this.props.yearDict, new Date().getFullYear().toString()) || null;
    this.setState(
      {
        ...payload,
      },
      () => {
        resolve();
      },
    );
  };

  updateSKey = (...params) => {
    this.setState(...params);
  };

  // 获取班续导入列表记录
  getImportClassListFn = (params) => {
    return loaders.getImportClassList(params);
  };
  // 下载未匹配列表导入
  getImportClassUnMatchListFn = (id) => {
    return loaders.getUnMatchClassImport({ id });
  };
  getClassSearchProps = (state) => {
    return {
      branchId: state.branchId,
      blCampusId: state.blCampusId,
      productTypeId: state.productTypeId,
      yearId: state.yearId,
      quarterId: state.quarterId,
      phaseId: state.phaseId,
      gradeId: state.gradeId,
      subjectId: state.subjectId,
      classTypeId: state.classTypeId,
      classLevel: state.classLevel,
      assistTeacherId: state.assistTeacherId,
      associatedStatus: state.associatedStatus,
      courseSeries: state.courseSeries,
    };
  };

  onSubmit = async () => {
    this.setState({ loading: true });

    const {
      branchId,
      blCampusId,
      productTypeId,
      yearId,
      quarterId,
      gradeId,
      subjectId,
      classTypeId,
      classLevel,
      mainTeacherId,
      assistTeacherId,
      associatedStatus,
    } = this.state;

    let range = {
      branchId,
      blCampusId: transformArrayToString(blCampusId),
      yearId,
      quarterId,
      gradeId: transformArrayToString(gradeId),
      subjectId: transformArrayToString(subjectId),
      classTypeId,
      classLevel: transformArrayToString(classLevel),
      mainTeacherName: mainTeacherId,
      assistTeacherName: assistTeacherId,
      productType: productTypeId,
      associatedStatus,
    };
    const selectRuleData = this._selectRuleRef.getSelectRuleState();
    const extraRuleState = this._extraRuleRef.getExtraRuleState();
    const apiParams = {
      range,
      rule: {
        ...selectRuleData,
        ...extraRuleState,
      },
    };
    showConfirm(`确认提交吗？`, async () => {
      try {
        const data = await loaders.submitClassLink(apiParams);
        this.setState({ loading: false });
        Modal.confirm({
          content: '可以在平移关联记录页面查看进度',
          onOk: () => {
            this.props.dispatch({
              type: 'main/openPage',
              payload: {
                path: '/LinkClassRecord',
                params: {},
                refresh: true,
              },
            });
          },
          iconType: 'check-circle-o',
          okText: '查看进度',
          cancelText: '取消',
        });
      } catch (e) {
        showMessage(e.text, 'error');
        this.setState({ loading: false });
      }
    });
  };
  render() {
    const { productTypeTree, weekDayDict, phaseDict, gradeDict, subjectDict, branchCampusDict, aliPath } = this.props;
    const {
      loading,
      defaultRuleValue,
      showRuleModal,
      productTypeId,
      classTypeId,
      branchId,
      blCampusId,
      classLevel,
      quarterId,
      totalLink,
      unLink,
    } = this.state;
    const isTwoTeacher = productTypeId === TWO_TEACHER_VAL;
    const mainTeacherNameTitle = isTwoTeacher ? '主讲老师' : '班主任';
    const assistTeacherNameTitle = isTwoTeacher ? '辅导老师' : '任课老师';
    const selectRuleClassLevelDict = buildClassLevelDict(productTypeTree, productTypeId, classTypeId) || [];
    const selectRuleCampusDict = buildBranchCampusDict(branchId ? [branchId] : null, branchCampusDict);
    // 学习中心是否单选
    const isCampusSingle = Array.isArray(blCampusId) && blCampusId.length === 1;
    const isClassLevelSingle = Array.isArray(classLevel) && classLevel.length === 1;

    const selectCampus = Array.isArray(blCampusId) ? blCampusId[0] : '';
    const selectClassLevel = Array.isArray(classLevel) ? classLevel[0] : '';
    const downFileDemo = {
      href: `${aliPath}%E5%AF%BC%E5%85%A5%E7%8F%AD%E7%BB%AD%E6%A8%A1%E6%9D%BF.xlsx`,
      download: '导入班续模板.xlsx',
    };

    return (
      <Page
        className={styles.linkClassPage}
        name="班续"
        loading={loading}
        route={this.props.route}
        ref={(c) => (this._page = c)}
      >
        <Tabs type="card">
          <TabPane tab="平移班续" key="1">
            <Block title="选择要平移的班级范围">
              <ClassSearchBar
                onSearch={this.search}
                updateSearchKey={this.updateSKey}
                {...this.getClassSearchProps(this.state)}
              />
              <div className={styles.listDetail}>
                总班级数量：<span>{totalLink}</span>个，其中未关联量<span>{unLink}</span>个。
              </div>
            </Block>
            <Block title="关联匹配规则">
              <div className={styles.linkTitle}>
                <span className={styles.linkBold}>默认规则</span>
                （点击查看<TextLink onClick={() => this.setState({ showRuleModal: true })}>匹配规则</TextLink>）
              </div>
              <CheckboxGroup options={defaultRuleOptions} value={defaultRuleValue} disabled={true} />
              <div className={styles.linkTitle}>
                <span className={styles.linkBold}>可选规则</span>
                <Popover content="请选择以下匹配字段，勾选后将严格匹配，如勾选班级层次，则同一层次的班级才会匹配，即提高班匹配到提高班">
                  <Icon type="question_circle" size={12} style={{ marginLeft: 4 }} />
                </Popover>
              </div>

              <CanSelectRule
                ref={(c) => (this._selectRuleRef = c)}
                mainTeacherNameTitle={mainTeacherNameTitle}
                assistTeacherNameTitle={assistTeacherNameTitle}
                campusDict={selectRuleCampusDict}
                classLevelDict={selectRuleClassLevelDict}
                selectRuleCampus={selectCampus}
                selectRuleClassLevel={selectClassLevel}
                isCampusSingle={isCampusSingle}
                isClassLevelSingle={isClassLevelSingle}
              />
              <div className={styles.linkTitle}>
                <span className={styles.linkBold}>附加规则</span>
              </div>
              <ExtraRule
                ref={(c) => (this._extraRuleRef = c)}
                phaseDict={phaseDict}
                weekDayDict={weekDayDict}
                gradeDict={gradeDict}
                subjectDict={subjectDict}
                quarterId={quarterId}
              />
            </Block>
            <div className={styles.linkClassButton}>
              <Button type="primary" onClick={this.onSubmit}>
                提交关联
              </Button>
            </div>
          </TabPane>
          <TabPane tab="导入班续" key="2">
            <ImportExcel
              importType="class"
              title="导入班续"
              demoFile={downFileDemo}
              importExcelUrl={importExcelUrl}
              getList={this.getImportClassListFn}
              getUnMatchList={this.getImportClassUnMatchListFn}
              onChangeLoading={({ loading }) => this.setState({ loading })}
            />
          </TabPane>
        </Tabs>
        {showRuleModal && <RuleDetailModal onClose={() => this.setState({ showRuleModal: false })} />}
      </Page>
    );
  }
}

let productTypeTreeStorage = null;
let quarterDictStorage = null;

const mapStateToProps = (state) => {
  // 过滤与产品类型相关字典
  if (!productTypeTreeStorage && state.dict.productTypeTree) {
    productTypeTreeStorage = state.dict.productTypeTree.filter((item) => productTypeAllow.includes(item.val));
  }
  // 过滤季度字典中的全年选项
  if (!quarterDictStorage && state.dict.quarter) {
    quarterDictStorage = state.dict.quarter.filter((item) => item.val !== 'DAT0000000245');
  }
  const branchDict = (state.dict.branchAllCampus || []).map(({ label, val }) => ({ text: label, val }));

  return {
    btnAuths: state.account.btnAuths,
    user: state.account.user,
    aliPath: state.account.user.aliPath,
    // 数据字典
    branchDict,
    campusDict: state.dict.campus,
    branchCampusDict: state.dict.branchCampus,
    branchAllCampusDict: state.dict.branchAllCampus,
    productTypeTree: productTypeTreeStorage,
    yearDict: state.dict.year,
    quarterDict: quarterDictStorage,
    phaseDict: state.dict.phase,
    weekDayDict: state.dict.weekDay,
    gradeDict: state.dict.grade,
    subjectDict: state.dict.subject,
    twoTcMainTcDict: state.dict.twoTcMainTc,
    twoTcAuxTcDict: state.dict.twoTcAuxTc,
    smallClassMainTcDict: state.dict.smallClassMainTc,
    smallClassAuxTcDict: state.dict.smallClassAuxTc,
    courseSeriesDict: state.dict.courseSeries,
  };
};

export default connect(mapStateToProps)(LinkClass);
