import React from 'react';
import { Tabs } from 'antd';
import Page from 'COMPONENT/Page';
import SearchBar from 'COMPONENT/SearchBar';
import Input from 'COMPONENT/Input';
import List from 'COMPONENT/List';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import ImportExcel from '../components/ImportExcel';
import StdDetailModal from './modals/StdDetailModal';
import { SYSTEM_MARK } from 'CONST/config';
import { buildBranchCampusDict } from 'MODEL/dict/utils';
import { connect } from 'dva';
import * as loaders from '../loaders';
import { TWO_TEACHER_VAL, SMALL_CLASS_VAL, productTypeAllow } from '../constants/config';
import showMessage from '../../../../components/utils/globalTips/showMessage';
import ModalSelectClass from '../ModalSelectClass';
import { transformArrayToString, getDefaultQuarter, buildBlCampus } from '../utils';
import { getDictTextByVal, getDictValByText } from 'UTIL/dict';
const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;
const TabPane = Tabs.TabPane;

const linkStatus = {
  add: 1,
  delete: 0,
};
// 人续的导入接口地址
const importExcelUrl = '/OriginalClassController/controllinitRecord.do';
class LinkStudent extends React.Component {
  constructor(props) {
    super(props);
    const productTypeId = SYSTEM_MARK === 'TWO_TEACHER' ? TWO_TEACHER_VAL : SMALL_CLASS_VAL;
    this.state = {
      listLoading: false,
      loading: false,
      studentId: null,
      phone: null,
      name: null,
      branchId: null,
      blCampusId: [],
      studentStatus: null,
      selectedRecord: {},
      showDetailModal: false,
      showSelectClassModal: false,
      productTypeId,
      rows: [],
      pageNo: 1,
      pageSize: 20,
      total: null,
    };
  }

  componentDidMount() {
    this.initData();
  }

  initData = async () => {
    this.setState({ loading: true });
    await this.initDict();
    const { branchDict } = this.props;
    const { brenchId, campusId } = this.props.user;
    if (Array.isArray(branchDict)) {
      const isGroup = brenchId === campusId;
      const defaultBranchId = isGroup ? this.props.branchDict[0].val : brenchId;
      await this.updateState({
        branchId: defaultBranchId,
      });
    }
    await this.loadList();
    this.setState({ loading: false });
  };
  updateState = (changedState) => {
    const newState = {
      ...this.state,
      ...changedState,
    };
    return new Promise((resolve) => {
      this.setState(newState, () => {
        resolve();
      });
    });
  };

  pageChange = async ({ current, pageSize }) => {
    await this.updateState({ loading: true, pageSize, pageNo: current });
    const list = await this.loadListData();
    this.updateState({
      loading: false,
      rows: list.data,
      total: list.rowCount,
      pageNo: list.pageNo,
    });
  };

  // 人续获取导入列表
  getImportClassListFn = (params) => {
    return loaders.getImportStudentList(params);
  };
  loadListData = async () => {
    const { studentId, phone, name, productTypeId, branchId, blCampusId, studentStatus, pageNo, pageSize, total } =
      this.state;
    const apiParams = {
      studentId: studentId,
      customerContact: phone,
      studentName: name,
      productType: productTypeId,
      studentStatus,
      branchId,
      blCampusIds: transformArrayToString(blCampusId),
      page: pageNo,
      rows: pageSize,
      getTotal: false,
    };
    const fetchList = [loaders.getLinkStudentList(apiParams)];
    if (pageNo === 1) {
      fetchList.push(loaders.getLinkStudentList({ ...apiParams, getTotal: true }));
    }
    let [detail, extraDetail] = await Promise.all(fetchList);
    detail = {
      ...detail,
      rowCount: extraDetail ? extraDetail.rowCount : total,
      pageNo,
    };
    return detail;
  };
  loadList = async () => {
    try {
      this.setState({ loading: true });
      let { data, pageNo, rowCount } = await this.loadListData();
      this.setState({
        loading: false,
        rows: data,
        total: rowCount,
        page: pageNo,
      });
    } catch (e) {
      showMessage(e.text, 'error');
      this.setState({ loading: false });
    }
  };

  reset = () => {
    const { branchDict } = this.props;
    this.setState({
      studentId: null,
      phone: null,
      name: null,
      studentStatus: null,
      blCampusId: [],
      branchId: branchDict[0].val,
    });
  };

  search = () => {
    this.setState({ pageNo: 1 }, () => {
      this.loadList();
    });
  };

  initDict = () => {
    return new Promise((resolve, reject) => {
      this.props.dispatch({
        type: 'dict/initDict',
        payload: {
          list: ['branch', 'year', 'grade', 'branchCampus', 'branchAllCampus', 'studentStatus'],
          callback() {
            resolve();
          },
        },
      });
    });
  };

  // 弹框的确认续课
  recommendTeacher = async (seriesClassIdArray) => {
    if (!(Array.isArray(seriesClassIdArray) && seriesClassIdArray.length)) {
      showMessage('请选择荐课班级！', 'error');
      return false;
    }
    const classIds = seriesClassIdArray.map((classDetail) => classDetail.classId).join(',');
    const { productTypeId, selectedRecord = {} } = this.state;
    const { studentId } = selectedRecord;
    try {
      // 新增状态为1
      await loaders.saveLinkStudent({
        studentId,
        productType: productTypeId,
        classIds,
        status: linkStatus.add,
      });
      this.search();
      this.setState({ showSelectClassModal: false, selectedRecord: {} });
      showMessage('关联成功', 'success');
    } catch (e) {
      showMessage(e.text, 'error');
      this.setState({ showSelectClassModal: false, selectedRecord: {} });
    }
  };
  updateBranch = (val) => {
    const { blCampusId } = this.state;
    const { branchCampusDictOrigin } = this.props;
    let payload = { branchId: val };
    if (val) {
      let filterBranchCampusDict = buildBranchCampusDict([val], branchCampusDictOrigin);
      let filterCampusDict = filterBranchCampusDict.map((item) => (item ? item.options : []));
      let filterValArr = filterCampusDict.reduce((arr, current) => arr.concat(current), []).map((item) => item.val);
      payload.blCampusId = [];
    }
    this.setState(payload);
  };

  // 表头
  columns = [
    { title: '学生名字', dataIndex: 'studentName', width: COL_WIDTH_M },
    { title: '学籍学习中心', dataIndex: 'blCampusName', width: COL_WIDTH_M },
    { title: '在读学校', dataIndex: 'schoolName', width: COL_WIDTH_M },
    { title: '年级', dataIndex: 'gradeName', width: COL_WIDTH_S },
    { title: '客户号码', dataIndex: 'customerContact', width: COL_WIDTH_M },
    { title: '语文已关联', dataIndex: 'chinese', width: COL_WIDTH_M },
    { title: '数学已关联', dataIndex: 'math', width: COL_WIDTH_M },
    { title: '英语已关联', dataIndex: 'english', width: COL_WIDTH_M },
    { title: '物理已关联', dataIndex: 'physics', width: COL_WIDTH_M },
    { title: '化学已关联', dataIndex: 'chemistry', width: COL_WIDTH_M },
    { title: '其他已关联', dataIndex: 'other', width: COL_WIDTH_M },
    {
      title: '操作',
      key: 'action',
      width: COL_WIDTH_M,
      fixed: 'right',
      getActionConfig: (text, record) => {
        const { btnAuths } = this.props;
        const btns = [
          { text: '详情', onClick: () => this.setState({ selectedRecord: record, showDetailModal: true }) },
        ];
        btns.push({
          text: '荐课',
          onClick: () => this.setState({ selectedRecord: record, showSelectClassModal: true }),
        });

        if (btnAuths.indexOf('test') > -1) {
          btns.push({
            text: '荐课',
            onClick: () => this.setState({ selectedRecord: record, showSelectClassModal: true }),
          });
        }
        return btns;
      },
    },
  ];
  render() {
    const {
      loading,
      studentId,
      phone,
      name,
      listLoading,
      rows,
      pageNo,
      showSelectClassModal,
      pageSize,
      total,
      showDetailModal,
      selectedRecord,
      productTypeId,
      branchId,
      blCampusId,
      studentStatus,
    } = this.state;
    const { branchDict, branchCampusDictOrigin, aliPath, studentStatusDict, yearDict, gradeDict } = this.props;
    const downFileDemo = {
      href: `${aliPath}%E5%AD%A6%E7%94%9F%E8%8D%90%E8%AF%BE%E5%AF%BC%E5%85%A5%E6%A8%A1%E6%9D%BF.xlsx`,
      download: '学生荐课导入模板.xlsx',
    };
    const initKey = {
      yearId: getDictValByText(yearDict, new Date().getFullYear().toString()) || null,
      quarterId: getDefaultQuarter(),
      gradeId: selectedRecord ? getDictValByText(gradeDict, selectedRecord.gradeName) : null,
    };
    return (
      <Page name="人续" loading={loading} route={this.props.route} ref={(c) => (this._page = c)}>
        <Tabs type="card">
          <TabPane tab="学生荐课" key="1">
            <div style={{ paddingRight: '29px' }}>
              <SearchBar onSearch={this.search} onReset={this.reset}>
                <Input
                  placeholder="搜索学生名称"
                  value={name}
                  onChange={(e) => this.setState({ name: e.target.value })}
                />
                <Input
                  placeholder="搜索客户号码"
                  value={phone}
                  onChange={(e) => this.setState({ phone: e.target.value })}
                />
                <Input
                  placeholder="搜索学生编号"
                  value={studentId}
                  onChange={(e) => this.setState({ studentId: e.target.value })}
                />
                <Select
                  placeholder="请选择分公司"
                  dict={branchDict}
                  value={branchId}
                  showSearch
                  allowClear={false}
                  onChange={this.updateBranch}
                />
                <MultiSelect
                  placeholder="请选择学习中心"
                  dict={buildBranchCampusDict(branchId ? [branchId] : null, branchCampusDictOrigin)}
                  value={blCampusId}
                  showSearch
                  allowClear={true}
                  onChange={(blCampusId) => this.setState({ blCampusId: blCampusId || [] })}
                />
                <Select
                  placeholder="请选择学生状态"
                  dict={studentStatusDict}
                  value={studentStatus}
                  showSearch
                  allowClear={true}
                  onChange={(studentStatus) => this.setState({ studentStatus })}
                />
              </SearchBar>
            </div>
            <List
              columns={this.columns}
              loading={listLoading}
              dataSource={rows || []}
              pagination={{
                current: pageNo,
                pageSize,
                total: total,
              }}
              onChange={this.pageChange}
            />
            {showDetailModal && (
              <StdDetailModal
                productTypeId={productTypeId}
                onClose={() => this.setState({ showDetailModal: false, selectedRecord: {} })}
                selectRecord={selectedRecord}
              />
            )}
            {showSelectClassModal && selectedRecord && (
              <ModalSelectClass
                initSKey={initKey}
                title="学生荐课"
                onClose={() => this.setState({ showSelectClassModal: false, selectedRecord: {} })}
                onSubmit={this.recommendTeacher}
              />
            )}
          </TabPane>
          <TabPane tab="导入荐课" key="2">
            <ImportExcel
              importType="student"
              title="导入荐课"
              demoFile={downFileDemo}
              importExcelUrl={importExcelUrl}
              getList={this.getImportClassListFn}
              onChangeLoading={({ loading }) => this.setState({ loading })}
            />
          </TabPane>
        </Tabs>
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const branchDict = (state.dict.branchAllCampus || []).map(({ label, val }) => ({ text: label, val }));

  return {
    btnAuths: state.account.btnAuths,
    user: state.account.user,
    aliPath: state.account.user.aliPath,
    // 数据字典
    branchDict,
    yearDict: state.dict.year,
    gradeDict: state.dict.grade,
    studentStatusDict: state.dict.studentStatus,
    branchCampusDictOrigin: state.dict.branchCampus,
  };
};

export default connect(mapStateToProps)(LinkStudent);
