import React from 'react';
import Modal from 'COMPONENT/Modal';
import List from 'COMPONENT/List';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import { TWO_TEACHER_VAL } from '../../constants/config';
import { SYSTEM_MARK } from 'CONST/config';
import * as loaders from '../../loaders';
import styles from './styles.less';

const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;

export default class StdDetailModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false,
      data: [],
    };
  }

  buildColumns = () => {
    const isTwoTeacher = SYSTEM_MARK === TWO_TEACHER_VAL;
    const mainTeacherNameTitle = isTwoTeacher ? '主讲老师' : '班主任';
    const assistTeacherNameTitle = isTwoTeacher ? '辅导老师' : '任课老师';
    return [
      { title: '班级名称', key: 'className', dataIndex: 'className', width: COL_WIDTH_XL },
      { title: '学习中心', key: 'blCampusName', dataIndex: 'blCampusName', width: COL_WIDTH_M },
      {
        title: mainTeacherNameTitle,
        key: 'mainTeacherName',
        dataIndex: 'mainTeacherName',
        width: COL_WIDTH_L,
        render: (data) => data || '其他',
      },
      {
        title: assistTeacherNameTitle,
        key: 'teacherName',
        dataIndex: 'teacherName',
        width: COL_WIDTH_M,
        render: (data) => data || '无',
      },
      { title: '班级层次', key: 'classLevelName', dataIndex: 'classLevelName', width: COL_WIDTH_M },
      { title: '期数/星期数', key: 'phaseName', dataIndex: 'phaseName', width: COL_WIDTH_M },
      { title: '上课时间', key: 'classTime', dataIndex: 'classTime', width: COL_WIDTH_M },
      { title: '关联来源', key: 'source', dataIndex: 'source', width: COL_WIDTH_S },
      {
        title: '是否删除',
        key: 'status',
        dataIndex: 'status',
        width: COL_WIDTH_S,
        render: (text) => (+text === 1 ? '否' : '是'),
      },
      {
        title: '操作',
        key: 'action',
        width: COL_WIDTH_M,
        getActionConfig: (text, record, index) => {
          const isDelete = Number(record.status) === 0;
          return [
            {
              text: <span style={{ color: isDelete ? '#6d67ff' : 'red' }}>{isDelete ? '恢复' : '删除'}</span>,
              onClick: () => this.changeStatus(record, isDelete),
            },
          ];
        },
      },
    ];
  };

  // 删除
  changeStatus = (record, isDelete) => {
    const { productTypeId } = this.props;
    const { studentId, classId } = record;
    const statusText = isDelete ? '确认恢复当前该班关联记录？' : '确认删除当前该班关联记录？';
    showConfirm(`${statusText}？`, async () => {
      try {
        await loaders.saveLinkStudent({
          productType: productTypeId,
          studentId,
          classIds: classId,
          status: isDelete ? 1 : 0,
        });
        showMessage(`${isDelete ? '恢复' : '删除'}成功`);
        this.initContent();
      } catch (e) {}
    });
  };

  componentDidMount() {
    this.initContent();
  }

  initContent = async () => {
    const { selectRecord, productTypeId } = this.props;
    const { studentId } = selectRecord;
    this.setState({
      loading: true,
    });
    const detail = await loaders.getLinkStudentInfo({
      studentId,
      productType: productTypeId,
    });
    this.setState({ loading: false, data: detail.data });
  };

  render() {
    const { loading, data } = this.state;
    const { onClose, selectRecord } = this.props;
    return (
      <Modal title="详情" width={1200} cancelText="关闭" onCancel={onClose} okBtnShow={false} loading={loading}>
        <div className={styles.regStudentDetailModal}>
          <List columns={this.buildColumns()} dataSource={data} />
        </div>
      </Modal>
    );
  }
}
