import React from 'react';
import PropTypes from 'prop-types';
import { connect } from 'dva';
import style from './styles.module.less';
import { Row, Col } from 'antd';
import Select from 'COMPONENT/Select';
import Input from 'COMPONENT/Input';
import Button from 'COMPONENT/Button';
import Search2 from 'COMPONENT/Search2';
import Loading from 'COMPONENT/Loading';
import { winter, spring, summer, autumn, SMALL_CLASS, productTypeAllow, mainTcTextCfg, auxTcTextCfg } from '../config';
import { getRelationsDict, getRelationPayload } from 'MODEL/dict/utils/index';
import { productType, productSeries, classType, classLevel } from 'MODEL/dict/utils/config';
import { SYSTEM_MARK } from '../../../../../constants/config';
import { runFunc } from 'UTIL/common';
import { clone } from 'lodash';

const SearchSkeleton = (props) => {
  return <div className={style.SearchSkeleton} />;
};

const emptySKey = {
  className: null,
  mainTeacherId: null,
  assistTeacherId: null,
  blCampusId: null,
  productType: null,
  courseSeriesId: [],
  classTypeId: [],
  classLevel: [],
  gradeId: [],
  subjectId: [],
  yearId: null,
  quarterId: [],
  phaseId: null,
  productGroupId: null,
};
Object.freeze(emptySKey);

class Search extends React.Component {
  constructor(props) {
    super(props);
    this.getSearchCfg = this.getSearchCfg.bind(this);
    this.onSearchChange = this.onSearchChange.bind(this);
    this.submit = this.submit.bind(this);
    this.resetSKey = this.resetSKey.bind(this);
    this.state = {
      loading: false,
      ...emptySKey,
    };
    this.isInit = false;
    this.defaultSKey = null;
    // 产品类型关联关系type、key对应关系配置
    this.productTypeTreeMap = {
      [productType]: 'productType',
      [productSeries]: 'courseSeriesId',
      [classType]: 'classTypeId',
      [classLevel]: 'classLevel',
    };
  }

  static PropTypes = {
    initSKey: PropTypes.object,
    onSearch: PropTypes.func.isRequired,
    isAlready: PropTypes.bool,
  };

  componentDidUpdate(prevProps, prevState, snapshot) {
    if (!this.isInit && !prevProps.initSKey && this.props.initSKey) {
      this.isInit = true;
      this.defaultSKey = this.props.initSKey;
      this.setState({ ...this.props.initSKey });
    }
  }

  getSearchCfg() {
    const isTwoTeacher = SYSTEM_MARK === 'TWO_TEACHER';
    const filterType = isTwoTeacher ? 'SMALL_CLASS' : 'TWO_TEACHER';
    let productTypeTree = this.props.productTypeTree;
    productTypeTree = (productTypeTree || []).filter((productType) => productType.val !== filterType);
    const courseSeriesDict = getRelationsDict(productSeries)(productTypeTree, this.state.productType);
    const classTypeDict = getRelationsDict(classType)(productTypeTree, this.state.productType);
    const classLevelDict = getRelationsDict(classLevel)(
      productTypeTree,
      this.state.productType,
      this.state.classTypeId,
    );

    const phaseDict = (this.props.weekDayDict || []).concat(this.props.phaseDict || []);

    /**
     * 学生推课的年份、季度非必填，所以hadOptionAll需改成动态的
     * 班续时才会有classId，因此用classId来判断来源，虽然不好，但暂时这样处理
     * */
    const isFormLinkStd = !!(this.props.initSKey || {}).classId;

    return [
      {
        label: '产品类型',
        dict: productTypeTree,
        field: 'productType',
        value: this.state.productType,
        hadOptionAll: false,
      },
      {
        label: '产品系列',
        dict: courseSeriesDict,
        field: 'courseSeriesId',
        value: this.state.courseSeriesId,
        type: 'multi',
      },
      { label: '科目', dict: this.props.subjectDict, field: 'subjectId', value: this.state.subjectId, type: 'multi' },
      { label: '班级类型', dict: classTypeDict, field: 'classTypeId', value: this.state.classTypeId, type: 'multi' },
      { label: '班级层次', dict: classLevelDict, field: 'classLevel', value: this.state.classLevel, type: 'multi' },
      { label: '年级', dict: this.props.gradeDict, field: 'gradeId', value: this.state.gradeId, type: 'multi' },
      {
        label: '产品季度',
        dict: this.props.quarterDict,
        field: 'quarterId',
        value: this.state.quarterId,
        type: 'multi',
        hadOptionAll: isFormLinkStd,
      },
      {
        label: '产品年份',
        dict: this.props.yearDict,
        field: 'yearId',
        value: this.state.yearId,
        hadOptionAll: isFormLinkStd,
      },
      { label: '产品组', dict: this.props.productGroupDict, field: 'productGroupId', value: this.state.productGroupId },
      { label: '期数', dict: phaseDict, field: 'phaseId', value: this.state.phaseId },
    ];
  }

  onSearchChange(params) {
    this.setState(
      (prevState) => {
        let payload = {
          className: prevState.className,
          mainTeacherId: prevState.mainTeacherId,
          assistTeacherId: prevState.assistTeacherId,
          blCampusId: prevState.blCampusId,
          ...params,
        };

        if (prevState.classTypeId !== params.classTypeId) {
          payload = {
            ...payload,
            ...getRelationPayload('classTypeId', params.classTypeId, {
              type: classType,
              dictTree: this.props.productTypeTree,
              state: this.props,
              map: this.productTypeTreeMap,
            }),
            classLevel: [],
          };
        } else if (prevState.productType !== params.productType) {
          payload = {
            ...payload,
            mainTeacherId: null,
            assistTeacherId: null,
            ...getRelationPayload('productType', params.productType, {
              type: productType,
              dictTree: this.props.productTypeTree,
              state: this.state,
              map: this.productTypeTreeMap,
            }),
          };
        }

        return payload;
      },
      () => {
        this.submit();
      },
    );
  }

  submit() {
    let params = clone(this.state);
    delete params.loading;
    runFunc(this.props.onSearch, params);
  }

  resetSKey() {
    let payload = {
      ...emptySKey,
      ...(this.defaultSKey || {}),
    };
    this.setState(payload, () => {
      runFunc(this.props.onSearch, payload);
    });
  }

  render() {
    return (
      <div className={style.Search}>
        <div className={style.searchBar}>
          <Row className={style.searchBarKey}>
            <Col span={6}>
              <Input
                placeholder="班级名称"
                value={this.state.className}
                onChange={(e) => {
                  this.setState({ className: e.target.value });
                }}
              />
            </Col>
            <Col span={6}>
              <Input
                placeholder={mainTcTextCfg[this.state.productType]}
                value={this.state.mainTeacherId}
                onChange={(e) => {
                  this.setState({ mainTeacherId: e.target.value });
                }}
              />
            </Col>
            <Col span={6}>
              <Input
                placeholder={auxTcTextCfg[this.state.productType]}
                value={this.state.assistTeacherId}
                onChange={(e) => {
                  this.setState({ assistTeacherId: e.target.value });
                }}
              />
            </Col>
            <Col span={6}>
              <Select
                placeholder="班级学习中心"
                dict={this.props.campusDict}
                value={this.state.blCampusId}
                onChange={(val) => {
                  this.setState({ blCampusId: val });
                }}
                showSearch
              />
            </Col>
          </Row>
          <div className={style.searchBarBtn}>
            <Button type="primary" onClick={this.submit}>
              查询
            </Button>
            <Button onClick={this.resetSKey}>重置</Button>
          </div>
        </div>
        {this.props.isAlready ? (
          <Search2
            style={{ marginTop: 20 }}
            config={this.getSearchCfg()}
            onChange={this.onSearchChange}
            rowNum={5}
            defaultExpand
          />
        ) : (
          <SearchSkeleton />
        )}
        <Loading className={style.loading} loading={this.state.loading} />
      </div>
    );
  }
}

let productTypeTreeStorage = null;
let quarterDictStorage = null;

const mapStateToProps = (state) => {
  if (!productTypeTreeStorage && state.dict.productTypeTree) {
    productTypeTreeStorage = state.dict.productTypeTree.filter((item) => productTypeAllow.includes(item.val));
  }
  if (!quarterDictStorage && state.dict.quarter) {
    quarterDictStorage = state.dict.quarter.filter((item) => item.text !== '全年');
  }

  return {
    twoTcMainTcDict: state.dict.twoTcMainTc,
    twoTcAuxTcDict: state.dict.twoTcAuxTc,
    smallClassMainTcDict: state.dict.smallClassMainTc,
    smallClassAuxTcDict: state.dict.smallClassAuxTc,
    campusDict: state.dict.branchCampus,
    productTypeTree: productTypeTreeStorage || [],
    gradeDict: state.dict.grade || [],
    subjectDict: state.dict.subject || [],
    yearDict: state.dict.year || [],
    quarterDict: quarterDictStorage || [],
    phaseDict: state.dict.phase || [],
    weekDayDict: state.dict.weekDay || [],
    productGroupDict: state.dict.productGroup || [],
  };
};

export default connect(mapStateToProps)(Search);
