import React, { PropTypes } from 'react';
import style from './styles.module.less';
import { Selected } from 'COMPONENT/ShoppingCart';
import { runFunc } from 'UTIL/common';
import { getTimeRange } from '../utils';

const MoreInfo = (props) => {
  return (
    <div className={style.MoreInfo}>
      <div className={style.MoreInfoLabel}>{props.label}</div>
      <div className={style.MoreInfoText}>
        <p>{props.text || '-'}</p>
      </div>
    </div>
  );
};
MoreInfo.propTypes = {
  label: PropTypes.string.isRequired,
  text: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
};

const Content = (props) => {
  return (
    <div className={style.Content} style={{ maxHeight: props.isShowMore ? 244 : 52 }}>
      <div className={style.title}>
        <span className={style.titleType}>{props.productTypeName}</span>
        <span>{props.className}</span>
      </div>
      <MoreInfo label="班级类型" text={props.classTypeName} />
      <MoreInfo label="班级层次" text={props.classLevelName} />
      <MoreInfo label="班级学习中心" text={props.blCampusName} />
      <MoreInfo
        label="已上课时"
        text={props.consumeCourseHours ? `${props.consumeCourseHours}/${props.totalCourseHours}` : ''}
      />
      <MoreInfo label="老师" text={`${props.mainTeacherName || '暂无'}/${props.assistTeacherName || '暂无'}`} />
      <MoreInfo label="上课日期" text={props.startClassDate} />
      <MoreInfo label="上课时间" text={getTimeRange(props.startClassTime, props.classLength)} />
      <MoreInfo label="截止日期" text={props.endDate} />
    </div>
  );
};

const SelectedCard = (props) => {
  return (
    <Selected
      className={style.SelectedCard}
      onDelete={() => {
        runFunc(props.onDelete, props.index);
      }}
    >
      <Content {...props} />
    </Selected>
  );
};

export default SelectedCard;
