import React from 'react';
import PropTypes from 'prop-types';
import style from './styles.module.less';
import { ToBoSelected } from 'COMPONENT/ShoppingCart';
import { runFunc } from 'UTIL/common';
import { getTimeRange } from '../utils';

const ClassInfo = (props) => {
  return (
    <span className={style.ClassInfo} style={{ display: props.text ? 'inline-block' : 'none' }}>
      {props.text}
    </span>
  );
};

const MoreInfo = (props) => {
  return (
    <span className={style.MoreInfo} style={{ display: props.text ? 'inline-block' : 'none' }}>
      <span>{props.label}：</span>
      <span>{props.text}</span>
    </span>
  );
};
MoreInfo.propTypes = {
  label: PropTypes.string.isRequired,
  text: PropTypes.string,
};

const Price = (props) => {
  return (
    <div className={style.Price}>
      <span>¥</span>
      <span>{props.num}</span>
    </div>
  );
};

const Content = (props) => {
  const teacherLabel = props.isTwoTeacher ? '主讲老师/辅导老师' : '班主任/任课老师';

  return (
    <div className={style.Content}>
      <div className={style.mainContent}>
        <div className={style.title}>
          <span className={style.titleType}>{props.productTypeName}</span>
          <span>{props.className}</span>
        </div>
        <div className={style.body}>
          {props.classId ? (
            [
              <MoreInfo label="班级类型" text={props.classTypeName || '暂无'} key={`${props.index}_classTypeName`} />,
              <MoreInfo label="班级层次" text={props.classLevelName || '暂无'} key={`${props.index}_classLevel`} />,
              <MoreInfo
                label={teacherLabel}
                text={`${props.mainTeacherName || '暂无'}/${props.assistTeacherName || '暂无'}`}
                key={`${props.index}_teacherName`}
              />,
              <MoreInfo
                label="已上课时"
                text={`${props.consumeCourseHours || '-'}/${props.totalCourseHours || '-'}`}
                key={`${props.index}_courseTime`}
              />,
            ]
          ) : (
            <ClassInfo text="暂无课程安排" />
          )}
        </div>
        <div className={style.footer}>
          <MoreInfo label="截止日期" text={props.endDate} />
          {props.classId && [
            <MoreInfo label="学习中心" text={props.blCampusName} key={`${props.index}_campusName`} />,
            <MoreInfo
              label="已报人数"
              text={`${props.peopleQuantity || '-'}/${props.maxQuantity || '-'}`}
              key={`${props.index}_regPeople`}
            />,
            <MoreInfo label="下次上课日期" text={props.nextClassDate} key={`${props.index}_nextTime`} />,
            <MoreInfo
              label="上课时间"
              text={getTimeRange(props.startClassTime, props.classLength)}
              key={`${props.index}_classTimeText`}
            />,
          ]}
        </div>
      </div>
      <div className={style.subContent}>{props.price && <Price num={props.price} />}</div>
    </div>
  );
};

const ToBoSelectedCard = (props) => {
  return (
    <ToBoSelected
      className={style.ToBoSelectedCard}
      onClick={() => {
        runFunc(props.onSelect, props.index);
      }}
    >
      <Content {...props} />
    </ToBoSelected>
  );
};

export default ToBoSelectedCard;
