import { SMALL_CLASS, TWO_TEACHER, spring, autumn, updateQuarterCfg, updateGradeCfg } from './config';
import { loginUser } from 'UTIL/common';
import { getDictTextByVal, getDictValByText } from 'UTIL/dict';
import moment from 'moment';

function checkStr(val) {
  return val && typeof val === 'string';
}

/**
 * 将数据带过来的筛选项初始值根据规则转换为最终的初始值
 *
 * @param SKey {Object}
 * @param yearDict {Array}
 * @param gradeDict {Array}
 *
 * @return {Object}
 * */
function sKeyAdapter(SKey, yearDict, gradeDict) {
  let payload = {
    productType: loginUser.isTwoTeacherSym() ? TWO_TEACHER : SMALL_CLASS,
    ...(SKey || {}),
  };

  if (checkStr(SKey.quarterId)) {
    payload.quarterId = updateQuarterCfg[SKey.quarterId];
  } else {
    delete payload.quarterId;
  }

  if (checkStr(SKey.yearId)) {
    let tagYearId = SKey.yearId;

    if (SKey.quarterId && SKey.quarterId === autumn) {
      let yearText = getDictTextByVal(yearDict, SKey.yearId);
      if (yearText) {
        yearText = (Number(yearText) + 1).toString();
      }
      let newYearId = getDictValByText(yearDict, yearText);
      if (newYearId) {
        tagYearId = newYearId;
      }
    }

    payload.yearId = tagYearId;
  } else {
    delete payload.yearId;
  }

  if (checkStr(SKey.gradeId)) {
    let tagGradeId = SKey.gradeId;

    if (SKey.quarterId && SKey.quarterId === spring) {
      let gradeText = getDictTextByVal(gradeDict, SKey.gradeId);
      if (gradeText) {
        gradeText = updateGradeCfg[gradeText];

        if (gradeText) {
          let newGradeId = getDictValByText(gradeDict, gradeText);
          if (newGradeId) {
            tagGradeId = newGradeId;
          }
        }
      }
    }

    payload.gradeId = [tagGradeId];
  } else {
    delete payload.gradeId;
  }

  if (checkStr(SKey.subjectId)) {
    payload.subjectId = [SKey.subjectId];
  } else {
    delete payload.subjectId;
  }

  return payload;
}

/**
 * 通过开始上课时间及总课时长，获取上课时间范围
 *
 * @param startTime {string} '14:00'
 * @param timeThrough {number} 90
 *
 * @return {string} '14:00~15:30'
 * */
function getTimeRange(startTime, timeThrough) {
  return `${startTime}~${moment(startTime, 'HH:mm').add(timeThrough, 'm').format('HH:mm')}`;
}

export { sKeyAdapter, getTimeRange };
