import React from 'react';
import { connect } from 'dva';
import SearchBar from 'COMPONENT/SearchBar';
import Input from 'COMPONENT/Input';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import { clone } from 'lodash';
import { buildBranchCampusDict, buildClassTypeDict, buildClassLevelDict } from 'MODEL/dict/utils';
import { TWO_TEACHER_VAL, productTypeAllow, linkDict, isWinterOrSummerVacation } from '../../constants/config';

class IndexSearch extends React.Component {
  constructor(props) {
    super(props);
    this.updateValue = this.updateValue.bind(this);
    this.updateBranch = this.updateBranch.bind(this);
    this.updateProductType = this.updateProductType.bind(this);
    this.updateClassType = this.updateClassType.bind(this);
    this.updateQuarter = this.updateQuarter.bind(this);
    this.state = {
      expand: true,
    };
  }

  updateValue(type, val) {
    console.log(type, val);
    this.props.updateSearchKey({ [type]: val });
  }

  updateBranch(val) {
    let payload = { branchId: val };
    let sKey_campus = clone(this.props.blCampusId || []);
    if (val) {
      let filterBranchCampusDict = buildBranchCampusDict([val], this.props.branchCampusDictOrigin);
      let filterCampusDict = filterBranchCampusDict.map((item) => (item ? item.options : []));
      let filterValArr = filterCampusDict.reduce((arr, current) => arr.concat(current), []).map((item) => item.val);
      sKey_campus = sKey_campus.filter((val) => filterValArr.includes(val));
      payload.blCampusId = sKey_campus;
    }
    this.props.updateSearchKey(payload);
  }

  updateProductType(val) {
    const {
      classTypeId,
      productTypeTree,
      classLevel,
      mainTeacherId,
      assistTeacherId,
      twoTcMainTcDict,
      smallClassMainTcDict,
      twoTcAuxTcDict,
      smallClassAuxTcDict,
    } = this.props;
    let payload = { productTypeId: val };
    if (val) {
      if (classTypeId) {
        const _classTypeDict = buildClassTypeDict(productTypeTree, val) || [];
        // 是否需要清空班级类型
        if (_classTypeDict.findIndex((item) => item.val === classTypeId) === -1) {
          payload.classTypeId = null;
        }
      }
      if (classLevel) {
        const _classLevelDict = buildClassLevelDict(productTypeTree, val, classTypeId) || [];
        // 是否需要清空班级层次
        let classLevelVal = this.props.classLevel.filter((item) => _classLevelDict.includes(item));
        payload.classLevel = classLevelVal.length > 0 ? classLevelVal : null;
      }
      if (mainTeacherId || assistTeacherId) {
        const isTwoTeacher = val === TWO_TEACHER_VAL;
        const mainTcDict = isTwoTeacher ? twoTcMainTcDict : smallClassMainTcDict;
        const auxTcDict = isTwoTeacher ? twoTcAuxTcDict : smallClassAuxTcDict;
        if (mainTeacherId && mainTcDict.findIndex((item) => item.val === mainTeacherId) === -1) {
          payload.mainTeacherId = null;
        }
        if (assistTeacherId && auxTcDict.findIndex((item) => item.val === assistTeacherId) === -1) {
          payload.assistTeacherId = null;
        }
      }
    }
    this.props.updateSearchKey(payload);
  }

  updateClassType(val) {
    let payload = { classTypeId: val };
    if (val) {
      const { productTypeId, classLevel, productTypeTree } = this.props;
      if (classLevel) {
        const _classLevelDict = buildClassLevelDict(productTypeTree, productTypeId, val) || [];
        // 是否需要清空班级层次
        let classLevelVal = classLevel.filter((item) => _classLevelDict.includes(item));
        payload.classLevel = classLevelVal.length > 0 ? classLevelVal : null;
      }
    }
    this.props.updateSearchKey(payload);
  }

  updateQuarter(val) {
    this.props.updateSearchKey((prevState) => {
      let payload = { quarterId: val };
      if (isWinterOrSummerVacation(prevState.quarterId) !== isWinterOrSummerVacation(val)) {
        payload.phaseId = null;
      }
      return payload;
    });
  }

  render() {
    const { expand } = this.state;
    const {
      branchDict,
      branchId,
      productTypeId,
      branchCampusDictOrigin,
      blCampusId,
      productTypeTree,
      yearDict,
      yearId,
      quarterDict,
      quarterId,
      phaseDict,
      weekDayDict,
      phaseId,
      gradeDict,
      gradeId,
      subjectDict,
      subjectId,
      classTypeId,
      classLevel,
      twoTcMainTcDict,
      smallClassMainTcDict,
      mainTeacherId,
      twoTcAuxTcDict,
      smallClassAuxTcDict,
      assistTeacherId,
      associatedStatus,
      onSearch,
      courseSeriesDict,
      courseSeries,
    } = this.props;
    const isTwoTeacher = productTypeId === TWO_TEACHER_VAL;
    return (
      <div>
        <SearchBar
          showResetButton={false}
          onSearch={onSearch}
          expand={expand}
          onToggleExpand={(expand) => this.setState({ expand })}
        >
          <Select
            placeholder="分公司"
            dict={branchDict}
            value={branchId}
            onChange={this.updateBranch}
            allowClear={false}
            showSearch
          />
          <MultiSelect
            placeholder="学习中心"
            dict={buildBranchCampusDict(branchId ? [branchId] : null, branchCampusDictOrigin)}
            value={blCampusId}
            onChange={(val) => this.updateValue('blCampusId', val)}
            allowClear={false}
            showSearch
          />
          <Select
            placeholder="产品类型"
            dict={productTypeTree}
            value={productTypeId}
            onChange={this.updateProductType}
            allowClear={false}
          />
          <Select
            placeholder="年份"
            dict={yearDict}
            value={yearId}
            onChange={(val) => this.updateValue('yearId', val)}
            allowClear={false}
          />
          <Select
            placeholder="产品季度"
            dict={quarterDict}
            value={quarterId}
            onChange={this.updateQuarter}
            allowClear={false}
          />
          {/*<MultiSelect*/}
          {/*  placeholder="期/星期"*/}
          {/*  dict={isWinterOrSummerVacation(quarterId) ? phaseDict : weekDayDict}*/}
          {/*  value={phaseId}*/}
          {/*  onChange={val => this.updateValue('phaseId', val)}*/}
          {/*  showSearch={false}/>*/}
          <MultiSelect
            placeholder="年级"
            dict={gradeDict}
            value={gradeId}
            onChange={(val) => this.updateValue('gradeId', val)}
            showSearch={false}
          />
          <MultiSelect
            placeholder="科目"
            dict={subjectDict}
            value={subjectId}
            onChange={(val) => this.updateValue('subjectId', val)}
            showSearch={false}
          />
          <Select
            placeholder="班级类型"
            dict={buildClassTypeDict(productTypeTree, productTypeId)}
            value={classTypeId}
            onChange={this.updateClassType}
          />
          <MultiSelect
            placeholder="班级层次"
            dict={buildClassLevelDict(productTypeTree, productTypeId, classTypeId)}
            value={classLevel}
            onChange={(val) => this.updateValue('classLevel', val)}
            showSearch={false}
          />
          <MultiSelect
            placeholder="产品系列"
            dict={courseSeriesDict}
            value={courseSeries}
            onChange={(val) => this.updateValue('courseSeries', val)}
            showSearch
            allowClear={false}
          />
          {/*<Select*/}
          {/*  placeholder={isTwoTeacher ? '主讲老师' : '班主任'}*/}
          {/*  dict={isTwoTeacher ? twoTcMainTcDict : smallClassMainTcDict}*/}
          {/*  value={mainTeacherId}*/}
          {/*  onChange={val => this.updateValue('mainTeacherId', val)}*/}
          {/*  searchPlaceholder="请输入老师名字搜索"*/}
          {/*  showSearch/>*/}
          <Input
            placeholder={isTwoTeacher ? '辅导老师' : '任课老师'}
            value={assistTeacherId}
            onChange={(e) => this.updateValue('assistTeacherId', e.target.value)}
          />
          <Select
            placeholder="是否关联"
            dict={linkDict}
            value={associatedStatus}
            onChange={(val) => this.updateValue('associatedStatus', val)}
          />
        </SearchBar>
      </div>
    );
  }
}

let productTypeTreeStorage = null;
let quarterDictStorage = null;

const mapStateToProps = (state) => {
  // 过滤与产品类型相关字典
  if (!productTypeTreeStorage && state.dict.productTypeTree) {
    productTypeTreeStorage = state.dict.productTypeTree.filter((item) => productTypeAllow.includes(item.val));
  }
  // 过滤季度字典中的全年选项
  if (!quarterDictStorage && state.dict.quarter) {
    quarterDictStorage = state.dict.quarter.filter((item) => item.val !== 'DAT0000000245');
  }
  const branchDict = (state.dict.branchAllCampus || []).map(({ label, val }) => ({ text: label, val }));

  return {
    branchDict,
    branchCampusDictOrigin: state.dict.branchCampus,
    productTypeTree: productTypeTreeStorage,
    yearDict: state.dict.year,
    quarterDict: quarterDictStorage,
    phaseDict: state.dict.phase,
    weekDayDict: state.dict.weekDay,
    gradeDict: state.dict.grade,
    subjectDict: state.dict.subject,
    twoTcMainTcDict: state.dict.twoTcMainTc,
    twoTcAuxTcDict: state.dict.twoTcAuxTc,
    smallClassMainTcDict: state.dict.smallClassMainTc,
    smallClassAuxTcDict: state.dict.smallClassAuxTc,
    courseSeriesDict: state.dict.courseSeries,
  };
};

export default connect(mapStateToProps)(IndexSearch);
