import React from 'react';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import Block from 'COMPONENT/Block';
import Button from 'COMPONENT/Button';
import List from 'COMPONENT/List';
import Modal from 'COMPONENT/Modal';
import styles from './style.less';
import { getAliyunOSSKeyWithoutCallBack, initRecord } from 'SERVICE/course';
import moment from 'moment';

const DATEFORMAT = 'YYYY-MM-DD HH:mm';

const successAlert = Modal.success;

/**
 * 导出模板
 */
const exportDemo = async ({ href, download } = {}) => {
  const link = document.createElement('a');
  const body = document.querySelector('body');
  link.href = href;
  console.log(link.href);
  link.download = download;
  // fix Firefox
  link.style.display = 'none';
  body.appendChild(link);
  link.click();
  body.removeChild(link);
};

import utils from './utils';
import * as loaders from '../../loaders';

const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_XL } = List;

export default class ImportExcel extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      // 列表数据
      listLoading: false,
      rows: [],
      currentPage: 1,
      pageSize: 20,
      totalRecords: 0,
    };
    // 防止多次点击
    this.limitClickTag = 1;
  }
  static defaultProps = {
    onChangeLoading: () => {},
  };
  columns = [
    { title: '文件名称', dataIndex: 'fileName', width: COL_WIDTH_XL },
    { title: '成功数量', dataIndex: 'successNum', width: COL_WIDTH_S },
    { title: '重复数量', dataIndex: 'repeatNum', width: COL_WIDTH_S },
    {
      title: '未匹配数量',
      dataIndex: 'failNum',
      width: COL_WIDTH_S,
      render: (text, record) => {
        if (Number(text) && Number(text) > 0) {
          return (
            <div>
              <span>{text}</span>
              <span
                style={{ color: '#6d67ff', marginLeft: '5px', cursor: 'pointer' }}
                onClick={() => this.exportNotMatch(record)}
              >
                导出
              </span>
            </div>
          );
        }
        return text;
      },
    },
    { title: '上传者', dataIndex: 'createUser', width: COL_WIDTH_M },
    {
      title: '上传时间',
      dataIndex: 'createTime',
      width: COL_WIDTH_M,
      render: (text, record) => moment(record.createTime).format(DATEFORMAT),
    },
  ];

  componentDidMount() {
    this.loadListData();
  }

  loadListData = async () => {
    const { currentPage, pageSize, totalRecords } = this.state;
    const { getList } = this.props;
    this.updateState({ listLoading: true });
    const apiParams = {
      page: currentPage,
      rows: pageSize,
      getTotal: false,
    };
    const fetchList = [getList(apiParams)];
    if (currentPage === 1) {
      fetchList.push(getList({ ...apiParams, getTotal: true }));
    }
    let [detail, extraDetail] = await Promise.all(fetchList);
    detail = {
      ...detail,
      rowCount: extraDetail ? extraDetail.rowCount : totalRecords,
      pageNo: pageSize,
    };
    this.updateState({
      listLoading: false,
      rows: detail.data,
      totalRecords: detail.rowCount,
    });
  };
  /**
   * 导入关联关系
   */
  importLink = async (event) => {
    const { onChangeLoading, importType } = this.props;
    const category = importType === 'student' ? 'CLASS_RENEW_STUDENT_IMPORT' : 'CLASS_RENEW_IMPORT';
    let file = event.target.files[0];
    const name = file.name;
    const idx = name.lastIndexOf('.');
    const nameExtend = name.slice(idx + 1);
    if (nameExtend.toLowerCase() !== 'xlsx' && nameExtend.toLowerCase() !== 'xls') {
      return showMessage('文件格式不正确');
    }
    const { OSSAccessKeyId, policy, signature, host, Expires, callback } = await getAliyunOSSKeyWithoutCallBack();
    const time = moment().format('YYYYMMDDHHmmss');

    let formData = new FormData();
    formData.append('OSSAccessKeyId', OSSAccessKeyId);
    formData.append('policy', policy);
    formData.append('Signature', signature);
    formData.append('success_action_status', '200');
    formData.append('key', time + file.name);
    formData.append('file', file);

    let xhr = new XMLHttpRequest();
    xhr.open('POST', host);
    xhr.upload.onloadstart = (file) => {
      onChangeLoading({ loading: true });
    };
    xhr.onload = async () => {
      if (xhr.status === 200 && xhr.readyState === 4) {
        try {
          const res = await initRecord({ filePath: time + file.name, category });
          const { resultCode } = res;
          if (resultCode === 0) {
            successAlert({
              content: (
                <div>
                  <p className={styles.mainInfo}>导入成功</p>
                  <p className={styles.secInfo}>请核对未匹配的数据进行导出修改</p>
                </div>
              ),
            });
            await this.updateState({ currentPage: 1 });
            this.loadListData();
          } else {
            showMessage('导入失败,请重新导入!');
            onChangeLoading({ loading: false });
          }
          onChangeLoading({ loading: false });
        } catch (e) {
          showMessage(e.text, 'error');
          onChangeLoading({ loading: false });
          return false;
        }
      } else {
        showMessage('系统错误');
        onChangeLoading({ loading: false });
      }
    };
    xhr.onerror = (e) => {
      showMessage('上传失败');
      this.props.onChangeLoading({ loading: false });
    };
    xhr.send(formData);
  };

  /**
   * 下载模板
   * @returns {Promise<number>}
   */
  downloadDemo = async () => {
    const { href, download } = this.props.demoFile;
    if (this.limitClickTag === 1) {
      this.limitClickTag = 2;
      try {
        exportDemo({
          href,
          download,
        });
      } catch (e) {
        showMessage(e.text);
        return (this.limitClickTag = 1);
      }
      this.limitClickTag = 1;
    }
  };

  /*
   *  导出未匹配列表
   * */
  exportNotMatch = async (record) => {
    const { resultUrl, fileName } = record;
    const href = resultUrl;
    const download = `${fileName}-未匹配列表`;
    exportDemo({ href, download });
  };

  /**
   * 导出excel
   * */
  async exportExcel(fetchUrl, fileOriginName, record) {
    this.props.onChangeLoading({ loading: true });
    const fetchParams = {
      importId: record.id,
    };
    let fileName = `${fileOriginName}`;
    const _self = this;
    utils.exportExcel({
      apiUrl: fetchUrl,
      data: fetchParams,
      fileName: fileName,
      success: () => {
        _self.props.onChangeLoading({ loading: false });
        showMessage('导出成功！', 'success');
      },
      fail: (err) => {
        _self.props.onChangeLoading({ loading: false });
        showMessage(err.text, 'error');
      },
    });
  }

  /**
   * 翻页
   */
  pageChange = async ({ current, pageSize } = {}) => {
    await this.updateState({ pageSize, currentPage: current });
    this.loadListData();
  };

  /**
   * 更新状态
   * @param  {object} changedState 改变的状态
   * @return {Promise}
   */
  updateState = (changedState) => {
    const newState = {
      ...this.state,
      ...changedState,
    };
    return new Promise((resolve) => {
      this.setState(newState, () => {
        resolve();
      });
    });
  };

  render() {
    const { listLoading, rows, currentPage, pageSize, totalRecords } = this.state;

    const { title } = this.props;
    return (
      <div className={styles.importExcel}>
        <Block title={title}>
          <Button style={{ marginRight: '1em', position: 'relative' }}>
            导入Excel
            <input
              type="file"
              className={styles.importBtn}
              onClick={(event) => {
                event.target.value = null;
              }}
              onChange={this.importLink}
            />
          </Button>
          <Button onClick={this.downloadDemo}>模板下载</Button>
        </Block>
        <Block title="导入记录">
          <List
            columns={this.columns}
            loading={listLoading}
            dataSource={rows || []}
            pagination={{
              current: currentPage,
              pageSize,
              total: totalRecords,
              pageSizeOptions: ['10', '20', '50'],
            }}
            onChange={this.pageChange}
          />
        </Block>
      </div>
    );
  }
}
