import React from 'react';
import { connect } from 'dva';
import SearchButton from 'COMPONENT/SearchButton';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import Input from 'COMPONENT/Input';
import { clone } from 'lodash';
import { buildBranchCampusDict, buildClassTypeDict, buildClassLevelDict } from 'MODEL/dict/utils';
import { TWO_TEACHER_VAL, productTypeAllow, linkDict, isWinterOrSummerVacation } from '../../constants/config';

class IndexSearch extends React.Component {
  constructor(props) {
    super(props);
    this.updateValue = this.updateValue.bind(this);
    this.updateBranch = this.updateBranch.bind(this);
    this.updateProductType = this.updateProductType.bind(this);
    this.updateClassType = this.updateClassType.bind(this);
    this.updateQuarter = this.updateQuarter.bind(this);
  }

  updateValue(type, val) {
    this.props.updateSearchKey({ [type]: val });
  }

  updateBranch(val) {
    let payload = { branchId: val };
    let sKey_campus = clone(this.props.blCampusId || []);
    if (val) {
      let filterBranchCampusDict = buildBranchCampusDict([val], this.props.campusDict);
      let filterCampusDict = filterBranchCampusDict.map((item) => (item ? item.options : []));
      let filterValArr = filterCampusDict.reduce((arr, current) => arr.concat(current), []).map((item) => item.val);
      sKey_campus = sKey_campus.filter((val) => filterValArr.includes(val));
      payload.blCampusId = sKey_campus;
    }
    this.props.updateSearchKey(payload);
  }

  updateProductType(val) {
    let payload = { productTypeId: val };
    if (val) {
      if (this.props.classTypeId) {
        const _classTypeDict = buildClassTypeDict(this.props.productTypeTree, val) || [];
        // 是否需要清空班级类型
        if (_classTypeDict.findIndex((item) => item.val === this.props.classTypeId) === -1) {
          payload.classTypeId = null;
        }
      }
      if (this.props.classLevel) {
        const _classLevelDict = buildClassLevelDict(this.props.productTypeTree, val, this.props.classTypeId) || [];
        // 是否需要清空班级层次
        let classLevelVal = this.props.classLevel.filter((item) => _classLevelDict.includes(item));
        payload.classLevel = classLevelVal.length > 0 ? classLevelVal : null;
      }
      if (this.props.mainTeacherId || this.props.assistTeacherId) {
        const isTwoTeacher = val === TWO_TEACHER_VAL;
        const mainTcDict = isTwoTeacher ? this.props.twoTcMainTcDict : this.props.smallClassMainTcDict;
        const auxTcDict = isTwoTeacher ? this.props.twoTcAuxTcDict : this.props.smallClassAuxTcDict;
        if (this.props.mainTeacherId && mainTcDict.findIndex((item) => item.val === this.props.mainTeacherId) === -1) {
          payload.mainTeacherId = null;
        }
        if (
          this.props.assistTeacherId &&
          auxTcDict.findIndex((item) => item.val === this.props.assistTeacherId) === -1
        ) {
          payload.assistTeacherId = null;
        }
      }
    }
    this.props.updateSearchKey(payload);
  }

  updateClassType(val) {
    let payload = { classTypeId: val };
    if (val) {
      const { productTypeId, classLevel, productTypeTree } = this.props;
      if (classLevel) {
        const _classLevelDict = buildClassLevelDict(productTypeTree, productTypeId, val) || [];
        // 是否需要清空班级层次
        let classLevelVal = classLevel.filter((item) => _classLevelDict.includes(item));
        payload.classLevel = classLevelVal.length > 0 ? classLevelVal : null;
      }
    }
    this.props.updateSearchKey(payload);
  }

  updateQuarter(val) {
    this.props.updateSearchKey((prevState) => {
      let payload = { quarterId: val };
      if (isWinterOrSummerVacation(prevState.quarterId) !== isWinterOrSummerVacation(val)) {
        payload.phaseId = null;
      }
      return payload;
    });
  }

  render() {
    const isTwoTeacher = this.props.productTypeId === TWO_TEACHER_VAL;

    const branchDict = (this.props.campusDict || []).map(({ label, val }) => ({ text: label, val }));
    return (
      <SearchButton searchBtnLoading={this.props.loading} onSearch={this.props.onSearch} onReset={this.props.onReset}>
        <Input
          placeholder="班级名称"
          value={this.props.className}
          onChange={(ele) => this.updateValue('className', ele.target.value)}
        />
        <Select
          placeholder="分公司"
          dict={branchDict}
          value={this.props.branchId}
          onChange={this.updateBranch}
          allowClear={false}
          showSearch
        />
        <MultiSelect
          placeholder="所属学习中心"
          dict={buildBranchCampusDict(this.props.branchId ? [this.props.branchId] : null, this.props.campusDict)}
          value={this.props.blCampusId}
          onChange={(val) => this.updateValue('blCampusId', val)}
          allowClear={false}
          showSearch
        />
        <Select
          placeholder="产品类型"
          dict={this.props.productTypeTree}
          value={this.props.productTypeId}
          onChange={this.updateProductType}
          allowClear={false}
        />
        <Select
          placeholder="产品系列"
          dict={this.props.courseSeriesDict}
          value={this.props.courseSeries}
          onChange={(val) => this.updateValue('courseSeries', val)}
          showSearch
          allowClear={true}
        />
        <Select
          placeholder="年份"
          dict={this.props.yearDict}
          value={this.props.yearId}
          onChange={(val) => this.updateValue('yearId', val)}
          allowClear={false}
        />
        <Select
          placeholder="产品季度"
          dict={this.props.quarterDict}
          value={this.props.quarterId}
          onChange={this.updateQuarter}
          allowClear={false}
        />
        <MultiSelect
          placeholder="期/星期"
          dict={isWinterOrSummerVacation(this.props.quarterId) ? this.props.phaseDict : this.props.weekDayDict}
          value={this.props.phaseId}
          onChange={(val) => this.updateValue('phaseId', val)}
          showSearch={false}
        />
        <MultiSelect
          placeholder="年级"
          dict={this.props.gradeDict}
          value={this.props.gradeId}
          onChange={(val) => this.updateValue('gradeId', val)}
          showSearch={false}
        />
        <MultiSelect
          placeholder="科目"
          dict={this.props.subjectDict}
          value={this.props.subjectId}
          onChange={(val) => this.updateValue('subjectId', val)}
          showSearch={false}
        />
        <Select
          placeholder="班级类型"
          dict={buildClassTypeDict(this.props.productTypeTree, this.props.productTypeId)}
          value={this.props.classTypeId}
          onChange={this.updateClassType}
        />
        <MultiSelect
          placeholder="班级层次"
          dict={buildClassLevelDict(this.props.productTypeTree, this.props.productTypeId, this.props.classTypeId)}
          value={this.props.classLevel}
          onChange={(val) => this.updateValue('classLevel', val)}
          showSearch={false}
        />
        <Input
          placeholder={isTwoTeacher ? '主讲老师' : '班主任'}
          value={this.props.mainTeacherId}
          onChange={(e) => this.updateValue('mainTeacherId', e.target.value)}
        />
        <Input
          placeholder={isTwoTeacher ? '辅导老师' : '任课老师'}
          value={this.props.assistTeacherId}
          onChange={(e) => this.updateValue('assistTeacherId', e.target.value)}
        />
        <Select
          placeholder="是否关联"
          dict={linkDict}
          value={this.props.associatedStatus}
          onChange={(val) => this.updateValue('associatedStatus', val)}
        />
      </SearchButton>
    );
  }
}

let productTypeTreeStorage = null;
let quarterDictStorage = null;

const mapStateToProps = (state) => {
  // 过滤与产品类型相关字典
  if (!productTypeTreeStorage && state.dict.productTypeTree) {
    productTypeTreeStorage = state.dict.productTypeTree.filter((item) => productTypeAllow.includes(item.val));
  }
  // 过滤季度字典中的全年选项
  if (!quarterDictStorage && state.dict.quarter) {
    quarterDictStorage = state.dict.quarter.filter((item) => item.val !== 'DAT0000000245');
  }
  return {
    branchDict: state.dict.branch,
    campusDict: state.dict.branchCampus,
    branchCampusDictOrigin: state.dict.branchAllCampus,
    productTypeTree: productTypeTreeStorage,
    yearDict: state.dict.year,
    quarterDict: quarterDictStorage,
    phaseDict: state.dict.phase,
    courseSeriesDict: state.dict.courseSeries,
    weekDayDict: state.dict.weekDay,
    gradeDict: state.dict.grade,
    subjectDict: state.dict.subject,
    twoTcMainTcDict: state.dict.twoTcMainTc,
    twoTcAuxTcDict: state.dict.twoTcAuxTc,
    smallClassMainTcDict: state.dict.smallClassMainTc,
    smallClassAuxTcDict: state.dict.smallClassAuxTc,
  };
};

export default connect(mapStateToProps)(IndexSearch);
