import { urlBase } from 'CONST/config';
import { quarterMap } from '../constants/config';
export function transformArrayToString(arr = [], splitStr = ',') {
  if (!arr || !Array.isArray(arr) || arr.length <= 0) {
    return null;
  }

  const trueArr = arr.filter((value) => {
    return !!value;
  });

  return trueArr.join(splitStr) || null;
}

const makeQueryString = (params) => {
  let queryString = '';
  if (params && typeof params === 'object') {
    Object.keys(params).forEach((key, i) => {
      const value = params[key];
      if (i > 0) {
        queryString += '&';
      }
      queryString += `${key}=${value === null || value == undefined ? '' : encodeURIComponent(value)}`;
    });
  } else if (typeof params === 'string') {
    queryString = params;
  }
  return queryString;
};

/**
 * 导出Excel
 */
export function exportExcel(params) {
  const { apiUrl, data, fileName, success, fail } = params;
  const paramsString = makeQueryString(data);
  return new Promise((resolve, reject) => {
    let request = new XMLHttpRequest();
    request.open('get', urlBase + apiUrl + (paramsString ? '?' + paramsString : ''), true);
    request.responseType = 'blob';
    request.setRequestHeader('Content-type', 'application/json');
    request.onload = function () {
      if (this.status === 200) {
        let blob = this.response;
        let reader = new FileReader();
        reader.readAsDataURL(blob);
        reader.onload = function (event) {
          resolve();
          let a = document.createElement('a');
          a.href = window.URL.createObjectURL(request.response);
          a.download = fileName + '.xls';
          document.body.appendChild(a);
          a.click();
          a.remove();
          if (success) {
            success();
          }
        };
      } else {
        let reader = new FileReader();
        reader.addEventListener('loadend', (event) => {
          let text = event.srcElement.result;
          reject(JSON.parse(text));
        });
        reader.readAsText(this.response);
      }
    };
    request.onerror = (error) => {
      reject(error);
      if (fail) {
        fail(error);
      }
    };
    request.send(JSON.stringify(data));
  });
}

export function getDefaultQuarter() {
  const month = new Date().getMonth() + 1;
  if (month === 1) {
    return quarterMap.winter;
  } else if (2 <= month <= 6) {
    return quarterMap.spring;
  } else if (7 <= month <= 8) {
    return quarterMap.summer;
  } else {
    return quarterMap.autumn;
  }
}

export function getTrueCampusArray(arr) {
  if (!Array.isArray(arr)) return [];
  return arr.reduce((arr, cur) => {
    const optionsId = Array.isArray(cur.options) ? cur.options.map((option) => option.val) : [];
    return arr.concat(optionsId);
  }, []);
}

export function buildBlCampus(blCampus, campusDict) {
  return Array.isArray(blCampus) && blCampus.length ? blCampus : campusDict;
}
