import React from 'react';
import { connect } from 'dva';
import style from './styles.module.less';
import Modal from 'COMPONENT/Modal';
import List from 'COMPONENT/List';
import Button from 'COMPONENT/Button';
import { Radio } from 'antd';
import {
  successLink,
  failLink,
  conflictLink,
  linkStatueCfg,
  linkTypeCfg,
  completeStatus,
  revokeStatus,
} from '../config';
import { calcIndexNumber, renderMultiLine } from '../utils';
import { queryTranslateRecordDetailList, undoTranslateRecordDetail } from 'SERVICE/course';
import { errorsHandle } from 'UTIL/common';
import ListAction from 'COMPONENT/ListAction';
import exportFile from 'UTIL/exportFile';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

const RadioGroup = Radio.Group;

const setWidth = List.createSetWidth();

class ModalDetail extends React.Component {
  constructor(props) {
    super(props);
    this.init = this.init.bind(this);
    this.loadList = this.loadList.bind(this);
    this.listAdepter = this.listAdepter.bind(this);
    this.updatePage = this.updatePage.bind(this);
    this.changePage = this.changePage.bind(this);
    this.onListTypeChange = this.onListTypeChange.bind(this);
    this.exportExcel = this.exportExcel.bind(this);
    this.state = {
      title: '查看',
      loading: false,
      listType: successLink,
      list: [],
      total: 0,
      pageCurrent: 1,
      pageSize: 10,
    };
    this.successCol = [
      { title: '序号', dataIndex: 'indexNum', key: 'indexNum', width: setWidth(2) },
      { title: '原学习中心', dataIndex: 'originalBlCampusName', key: 'originalBlCampusName', width: setWidth(4) },
      { title: '原班名称', dataIndex: 'originalClassName', key: 'originalClassName', width: setWidth(6) },
      { title: '关联班学习中心', dataIndex: 'seriesBlCampusName', key: 'seriesBlCampusName', width: setWidth(6) },
      { title: '关联班名称', dataIndex: 'seriesClassName', key: 'seriesClassName', width: setWidth(6) },
      { title: '完成状态', dataIndex: 'statusText', key: 'statusText', width: setWidth(4) },
      {
        title: '操作',
        dataIndex: 'id',
        key: 'actionPlus',
        width: setWidth(3),
        render: (text, record, index) => (
          <ListAction
            config={[
              {
                text: '撤回',
                onClick: () => {
                  this.revoke(text, index);
                },
                needConfirm: true,
                confirmTips: '你确定要撤回吗？',
                disabled: record.completeStatus.toString() !== completeStatus,
              },
            ]}
          />
        ),
      },
    ];
    this.failCol = [
      { title: '序号', dataIndex: 'indexNum', key: 'indexNum', width: setWidth(2) },
      { title: '原学习中心', dataIndex: 'originalBlCampusName', key: 'originalBlCampusName', width: setWidth(4) },
      { title: '原班名称', dataIndex: 'originalClassName', key: 'originalClassName', width: setWidth(6) },
    ];
    this.conflictCol = [
      { title: '序号', dataIndex: 'indexNum', key: 'indexNum', width: setWidth(2) },
      { title: '原学习中心', dataIndex: 'originalBlCampusName', key: 'originalBlCampusName', width: setWidth(4) },
      { title: '原班名称', dataIndex: 'originalClassName', key: 'originalClassName', width: setWidth(6) },
      {
        title: '已关联续报班',
        dataIndex: 'associatedList',
        key: 'associatedList',
        width: setWidth(8),
        render: renderMultiLine,
      },
      {
        title: '冲突名单',
        dataIndex: 'conflictList',
        key: 'conflictList',
        width: setWidth(8),
        render: renderMultiLine,
      },
    ];
    this.columnsCfg = {
      [successLink]: this.successCol,
      [failLink]: this.failCol,
      [conflictLink]: this.conflictCol,
    };
  }

  componentDidMount() {
    this.init();
  }

  async init() {
    this.setState({ loading: true });
    await new Promise(this.loadList);
    this.setState({ loading: false });
  }

  async loadList(resolve) {
    let apiParams = {
      recordId: this.props.recordId,
      associatedStatus: this.state.listType,
      page: this.state.pageCurrent,
      rows: this.state.pageSize,
      getTotal: false,
    };

    const isFirstPage = apiParams.page === 1;

    let fetchList = [];

    if (isFirstPage) {
      fetchList.unshift(queryTranslateRecordDetailList({ ...apiParams, getTotal: true }, true));
    }
    fetchList.unshift(queryTranslateRecordDetailList(apiParams, true));

    const rspAll = await Promise.all(fetchList).catch(errorsHandle());
    if (rspAll) {
      const [rspList, rspTotal] = rspAll;
      let payload = { list: this.listAdepter(rspList.rows) || [] };
      if (rspTotal) {
        payload.total = rspTotal.records;
      }
      this.setState(payload);
    }

    resolve();
  }

  listAdepter(list) {
    return list.map((item, index) => ({
      ...item,
      indexNum: calcIndexNumber(index, this.state.pageCurrent, this.state.pageSize),
      statusText: linkStatueCfg[item.completeStatus.toString()] || '-',
    }));
  }

  updatePage(pageCurrent, pageSize) {
    return new Promise((resolve) => {
      this.setState(
        (prevState) => {
          let payload = {};
          if (pageSize && pageSize !== prevState.pageSize) {
            payload.pageCurrent = 1;
            payload.pageSize = pageSize;
          } else {
            payload.pageCurrent = pageCurrent;
          }
          return payload;
        },
        () => {
          resolve();
        },
      );
    });
  }

  async changePage({ current, pageSize }) {
    await this.updatePage(current, pageSize);
    this.setState({ loading: true });
    await new Promise(this.loadList);
    this.setState({ loading: false });
  }

  onListTypeChange(e) {
    this.setState(
      {
        listType: e.target.value,
        list: [],
        total: 0,
        pageCurrent: 1,
      },
      () => {
        this.init();
      },
    );
  }

  async revoke(id, index) {
    this.setState({ loading: true });
    const rsp = await undoTranslateRecordDetail({ detailId: id }, true).catch(errorsHandle());
    if (rsp && rsp.resultCode === 0) {
      showMessage('撤回成功！', 'success');
      this.setState((prevState) => {
        prevState.list[index].completeStatus = revokeStatus;
        prevState.list[index].statusText = linkStatueCfg[revokeStatus];
        return { list: prevState.list, loading: false };
      });
    } else {
      this.setState({ loading: false });
    }
  }

  exportExcel() {
    this.setState({ loading: true });
    exportFile
      .get({
        url: '/eduboss/ClassRenewController/exportTranslateRecordDetail.do',
        params: {
          recordId: this.props.recordId,
          associatedStatus: this.state.listType,
        },
        fileName: `平移班续-${linkTypeCfg[this.state.listType]}-${new Date().getTime()}`,
        fileExtension: 'xls',
      })
      .then(() => {
        this.setState({ loading: false });
      })
      .catch(() => {
        this.setState({ loading: false });
      });
  }

  render() {
    return (
      <Modal
        title={this.state.title}
        loading={this.state.loading}
        onClose={this.props.onClose}
        widthSize="L"
        okBtnShow={false}
      >
        <div className={style.toolBar}>
          <RadioGroup onChange={this.onListTypeChange} value={this.state.listType}>
            <Radio value={successLink}>{linkTypeCfg[successLink]}</Radio>
            <Radio value={failLink}>{linkTypeCfg[failLink]}</Radio>
            <Radio value={conflictLink}>{linkTypeCfg[conflictLink]}</Radio>
          </RadioGroup>
          {this.state.listType === failLink && (
            <span className={style.failTips}>关联失败：指该原班按照规则找不到续报班或者关联后进行撤回</span>
          )}
          <Button type="primary" onClick={this.exportExcel}>
            导出
          </Button>
        </div>
        <List
          dataSource={this.state.list}
          columns={this.columnsCfg[this.state.listType]}
          pagination={{
            total: this.state.total,
            current: this.state.pageCurrent,
            pageSize: this.state.pageSize,
          }}
          onChange={this.changePage}
        />
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  return {};
};

export default connect(mapStateToProps)(ModalDetail);
