import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import List from 'COMPONENT/List';
import ListAction from 'COMPONENT/ListAction';
import { queryTranslateRecordList, getTranslateRecord, submitUndoTranslateRecord } from 'SERVICE/course';
import { runFunc, errorsHandle } from 'UTIL/common';
import { linkStatueCfg, dateFormat, goingStatus, completeStatus, revokeStatus, revokingStatus } from './config';
import { calcIndexNumber, numFilter } from './utils';
import moment from 'moment';
import ModalDetail from './ModalDetail';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

const setWidth = List.createSetWidth();

class LinkClassRecord extends React.Component {
  constructor(props) {
    super(props);
    this.init = this.init.bind(this);
    this.loadList = this.loadList.bind(this);
    this.listItemAdepter = this.listItemAdepter.bind(this);
    this.updatePage = this.updatePage.bind(this);
    this.changePage = this.changePage.bind(this);
    this.openDetail = this.openDetail.bind(this);
    this.refreshRecord = this.refreshRecord.bind(this);
    this.revoke = this.revoke.bind(this);
    this.switchModalVisible = this.switchModalVisible.bind(this);
    this.state = {
      name: '平移班续记录',
      loading: false,
      list: [],
      total: 0,
      pageCurrent: 1,
      pageSize: 10,
      modalVisible: false,
      recordId: null,
    };
    this.columns = [
      { title: '序号', dataIndex: 'indexNum', key: 'indexNum', width: setWidth(2) },
      { title: '学习中心', dataIndex: 'blcampusName', key: 'blcampusName', width: setWidth(4) },
      { title: '提交原班', dataIndex: 'originalClassQuantity', key: 'originalClassQuantity', width: setWidth(4) },
      { title: '关联成功', dataIndex: 'successQuantity', key: 'successQuantity', width: setWidth(4) },
      { title: '关联失败', dataIndex: 'failQuantity', key: 'failQuantity', width: setWidth(4) },
      { title: '关联冲突', dataIndex: 'conflictQuantity', key: 'conflictQuantity', width: setWidth(4) },
      { title: '前面排队', dataIndex: 'queueQuantity', key: 'queueQuantity', width: setWidth(4) },
      { title: '完成状态', dataIndex: 'statusText', key: 'statusText', width: setWidth(4) },
      { title: '操作人员', dataIndex: 'createUser', key: 'createUser', width: setWidth(4) },
      { title: '操作时间', dataIndex: 'createTimeText', key: 'createTimeText', width: setWidth(7) },
      {
        title: '操作',
        dataIndex: 'id',
        key: 'actionPlus',
        width: setWidth(4),
        render: (text, record, index) => {
          let cfg = [
            {
              text: '查看',
              onClick: () => {
                this.openDetail(text);
              },
            },
          ];

          if (record.completeStatus === goingStatus) {
            cfg.push({
              text: '刷新',
              onClick: () => {
                this.refreshRecord(text, index);
              },
            });
          }
          if (record.completeStatus === completeStatus) {
            cfg.push({
              text: '撤回',
              onClick: () => {
                this.revoke(text, index);
              },
              needConfirm: true,
              confirmTips: '你确定要撤回吗？',
            });
          }

          return <ListAction config={cfg} />;
        },
      },
    ];
  }

  componentDidMount() {
    this.init();
  }

  async init() {
    this.setState({ loading: true });
    await new Promise(this.loadList);
    this.setState({ loading: false });
  }

  async loadList(resolve) {
    let apiParams = {
      page: this.state.pageCurrent,
      rows: this.state.pageSize,
      getTotal: false,
    };

    const isFirstPage = apiParams.page === 1;

    let fetchList = [];

    if (isFirstPage) {
      fetchList.unshift(queryTranslateRecordList({ ...apiParams, getTotal: true }, true));
    }
    fetchList.unshift(queryTranslateRecordList(apiParams, true));

    const rspAll = await Promise.all(fetchList).catch(errorsHandle());
    if (rspAll) {
      const [rspList, rspTotal] = rspAll;
      let payload = { list: (rspList.rows || []).map(this.listItemAdepter) };
      if (rspTotal) {
        payload.total = rspTotal.records;
      }
      this.setState(payload);
    }

    resolve();
  }

  listItemAdepter(item, index) {
    return {
      ...item,
      successQuantity: numFilter(item.successQuantity),
      failQuantity: numFilter(item.failQuantity),
      conflictQuantity: numFilter(item.conflictQuantity),
      queueQuantity: numFilter(item.queueQuantity),
      indexNum: calcIndexNumber(index, this.state.pageCurrent, this.state.pageSize),
      statusText: linkStatueCfg[item.completeStatus.toString()] || '-',
      createTimeText: moment(item.createTime).format(dateFormat),
    };
  }

  updatePage(pageCurrent, pageSize) {
    return new Promise((resolve) => {
      this.setState(
        (prevState) => {
          let payload = {};
          if (pageSize && pageSize !== prevState.pageSize) {
            payload.pageCurrent = 1;
            payload.pageSize = pageSize;
          } else {
            payload.pageCurrent = pageCurrent;
          }
          return payload;
        },
        () => {
          resolve();
        },
      );
    });
  }

  async changePage({ current, pageSize }) {
    await this.updatePage(current, pageSize);
    this.setState({ loading: true });
    await new Promise(this.loadList);
    this.setState({ loading: false });
  }

  openDetail(id) {
    this.recordId = id;
    this.switchModalVisible();
  }

  async refreshRecord(id, index) {
    this.setState({ loading: true });
    const rsp = await getTranslateRecord({ recordId: id }, true).catch(errorsHandle());
    if (rsp) {
      this.setState((prevState) => {
        prevState.list[index] = this.listItemAdepter(rsp.data, index);
        return { list: prevState.list };
      });
    }
    this.setState({ loading: false });
  }

  async revoke(id, index) {
    this.setState({ loading: true });
    const rsp = await submitUndoTranslateRecord({ recordId: id }, true).catch(errorsHandle());
    if (rsp && rsp.resultCode === 0) {
      showMessage('提交撤回成功', 'success');
      this.setState((prevState) => {
        prevState.list[index].completeStatus = revokingStatus;
        prevState.list[index].statusText = linkStatueCfg[revokingStatus];
        return { list: prevState.list };
      });
    }
    this.setState({ loading: false });
  }

  switchModalVisible() {
    this.setState(
      (prevState) => ({ modalVisible: !prevState.modalVisible }),
      () => {
        if (!this.state.modalVisible) {
          this.recordId = null;
        }
      },
    );
  }

  render() {
    return (
      <Page name={this.state.name} route={this.props.route} fitWindowHeight>
        <List
          loading={this.state.loading}
          columns={this.columns}
          dataSource={this.state.list}
          pagination={{
            total: this.state.total,
            current: this.state.pageCurrent,
            pageSize: this.state.pageSize,
          }}
          onChange={this.changePage}
        />
        {this.state.modalVisible && <ModalDetail recordId={this.recordId} onClose={this.switchModalVisible} />}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {};
};

export default connect(mapStateToProps)(LinkClassRecord);
