import React from 'react';
import PropTypes from 'prop-types';
import InfoBlock from './InfoBlock';
import InfoItem from './InfoItem';
import { List, InputNumber } from 'COMPONENT';
import style from './styles.module.less';
import { runFunc, loginUser } from 'UTIL/common';
import moment from 'moment';
import NP from 'number-precision';

const momentFormat = 'YYYY-MM-DD HH:mm:ss';
const momentDateFormat = 'YYYY-MM-DD';

const { InfoGroup } = InfoItem;

const setWidth = List.createSetWidth();

class ContractCard extends React.Component {
  constructor(props) {
    super(props);

    this.onChange = this.onChange.bind(this);
    this.getColumns = this.getColumns.bind(this);
    this.isExBoss = loginUser.isExBoss();

    this.columns = [
      { title: '产品名称', dataIndex: 'productName', key: 'productName', width: setWidth(6) },
      { title: '课程名称', dataIndex: 'className', key: 'className', width: setWidth(6) },
      {
        title: '小时数',
        dataIndex: 'courseHours',
        key: 'courseHours',
        width: setWidth(3),
        render: (val, row) => {
          return row.courseNatureHours;
        },
      },
      { title: '产品金额', dataIndex: 'totalAmount', key: 'totalAmount', width: setWidth(4) },
      { title: '优惠金额', dataIndex: 'promotionAmount', key: 'promotionAmount', width: setWidth(4) },
      { title: '已收金额', dataIndex: 'paidAmount', key: 'paidAmount', width: setWidth(4) },
      { title: '待收金额', dataIndex: 'pendingAmount', key: 'pendingAmount', width: setWidth(4) },
    ];
  }

  static PropTypes = {
    contractId: PropTypes.string,
    blCampusId: PropTypes.string,
    blCampusName: PropTypes.string,
    contractGradeName: PropTypes.string,
    totalAmount: PropTypes.number,
    signStaffName: PropTypes.string,
    signDate: PropTypes.string,
    canPaidAmount: PropTypes.number,
    availableAmount: PropTypes.number,
    contractStatus: PropTypes.string,
    paidStatus: PropTypes.string,
    contractProductArr: PropTypes.arrayOf(
      PropTypes.shape({
        contractProductId: PropTypes.string,
        productName: PropTypes.string,
        classId: PropTypes.string,
        className: PropTypes.string,
        courseHours: PropTypes.number,
        totalAmount: PropTypes.number,
        promotionAmount: PropTypes.number,
        paidAmount: PropTypes.number,
        pendingAmount: PropTypes.number,
        amount: PropTypes.number,
      }),
    ),
    segmentChargeConfigSnapshot: PropTypes.object,
    onChange: PropTypes.func,
    inputDisabled: PropTypes.bool,
    inputVisible: PropTypes.bool,
    segmentChargeItems: PropTypes.arrayOf(
      PropTypes.shape({
        segNo: PropTypes.number,
        natureHours: PropTypes.number,
        totalAmount: PropTypes.number,
        promotionAmount: PropTypes.number,
        realAmount: PropTypes.number,
      }),
    ),
  };

  static defaultProps = {
    inputDisabled: false,
    inputVisible: true,
  };

  onChange(value, record, index) {
    let valArr = this.props.contractProductArr.map((item) => item.amount || 0);
    if (valArr[index] < value) {
      valArr[index] = value;
      if (valArr.reduce((accumulate, current) => NP.plus(accumulate, current), 0) > this.props.canPaidAmount) {
        valArr[index] = 0;
        value = NP.minus(this.props.canPaidAmount, ...valArr);
      }
    }
    runFunc(this.props.onChange, {
      amount: value,
      contractId: this.props.contractId,
      contractProductId: record.contractProductId,
    });
  }

  getColumns() {
    const { props } = this;

    if (props.inputVisible) {
      let payDisabled = false;
      if (
        // 有配置分段
        props.segmentChargeConfigSnapshot &&
        // 是上段
        ContractCard.isBeforeTagDate(props.segmentChargeConfigSnapshot.downStartTime) &&
        // 可收金额为0
        props.canPaidAmount === 0
      ) {
        payDisabled = true;
      }

      return this.columns.concat({
        title: '本次收款',
        dataIndex: 'amount',
        key: 'amount',
        width: setWidth(7),
        render: (text, record, index) => {
          return (
            <InputNumber
              value={text}
              onChange={(val) => this.onChange(val, record, index)}
              min={0}
              max={record.pendingAmount}
              precision={2}
              disabled={props.inputDisabled || props.availableAmount > 0 || payDisabled || !record.pendingAmount}
            />
          );
        },
      });
    }

    return this.columns;
  }

  static isBeforeTagDate(timestamp) {
    const today = moment().format(momentDateFormat);
    const tagDay = moment(timestamp).format(momentDateFormat);
    return moment(today).isBefore(tagDay);
  }

  render() {
    const { props } = this;
    const { segmentChargeItems } = props;
    return (
      <InfoBlock
        isSegment={segmentChargeItems != null}
        title={
          <div>
            {segmentChargeItems && segmentChargeItems.length > 1 ? (
              <div style={{ color: 'red' }}>
                <div>
                  合同共{NP.plus(segmentChargeItems[0]['natureHours'], segmentChargeItems[1]['natureHours'])}小时，
                  产品金额{NP.plus(segmentChargeItems[0]['totalAmount'], segmentChargeItems[1]['totalAmount'])}元，
                  总优惠{NP.plus(segmentChargeItems[0]['promotionAmount'], segmentChargeItems[1]['promotionAmount'])}
                  元， 应收{NP.plus(segmentChargeItems[0]['realAmount'], segmentChargeItems[1]['realAmount'])}元
                </div>
                <div>
                  阶段1：共{segmentChargeItems[0]['natureHours']}小时， 产品金额{segmentChargeItems[0]['totalAmount']}
                  元， 优惠金额{segmentChargeItems[0]['promotionAmount']}元， 应收{segmentChargeItems[0]['realAmount']}
                  元
                </div>
                <div>
                  阶段2：共{segmentChargeItems[1]['natureHours']}小时， 产品金额{segmentChargeItems[1]['totalAmount']}
                  元， 优惠金额{segmentChargeItems[1]['promotionAmount']}元， 应收尾款
                  {segmentChargeItems[1]['realAmount']}元
                </div>
              </div>
            ) : (
              ''
            )}
            <div>合同编号：{this.props.contractId || '-'}</div>
          </div>
        }
        toolbar={
          <div className={style.toolbarWrap}>
            <InfoGroup className={style.infoGroup}>
              <InfoItem label="合同学习中心">{this.props.blCampusName}</InfoItem>
              <InfoItem label="合同年级">{this.props.contractGradeName}</InfoItem>
              <InfoItem label="合同金额">{this.props.totalAmount}</InfoItem>
              <InfoItem label="签约人">{this.props.signStaffName}</InfoItem>
              {!this.isExBoss && (
                <InfoItem label="合同签订日期">
                  {this.props.signDate ? moment(this.props.signDate).format(momentFormat) : '-'}
                </InfoItem>
              )}
              {this.props.availableAmount > 0 && (
                <InfoItem>
                  <span className={style.danger}>本合同有待分配资金，请分配完资金才可收款</span>
                </InfoItem>
              )}
            </InfoGroup>
          </div>
        }
      >
        <div className={style.listWrap}>
          <List columns={this.getColumns()} dataSource={this.props.contractProductArr || []} />
        </div>
      </InfoBlock>
    );
  }
}

export default ContractCard;
