import React from 'react';
import styles from './styles.module.less';
import PropTypes from 'prop-types';
import NP from 'number-precision';
import { numFormatToMoney } from 'UTIL/common';

const Statistics = (props) => {
  const paidAmount = NP.plus(props.depositAmount, props.paidAmount);

  return (
    <div className={styles.statistics}>
      <div className={styles.item}>
        <span className={styles.label}>合计</span>
        <span className={styles.content}>{numFormatToMoney(props.totalAmount)}</span>
      </div>
      <div className={styles.item}>
        <span className={styles.label}>已收合计</span>
        <span className={styles.content}>{numFormatToMoney(paidAmount)}</span>
      </div>
      <div className={styles.item}>
        <span className={styles.label}>优惠合计</span>
        <span className={styles.content}>{numFormatToMoney(props.promotionAmount)}</span>
      </div>
      <div className={styles.item}>
        <span className={styles.label}>待收合计</span>
        <span className={styles.content}>
          {numFormatToMoney(NP.minus(props.totalAmount, paidAmount, props.promotionAmount))}
        </span>
      </div>
      {props.transactionVisible && (
        <div className={styles.item}>
          <span className={styles.label}>本次支付</span>
          <span className={styles.content}>
            {numFormatToMoney(NP.minus(props.transactionAmount, props.depositAmount))}
          </span>
        </div>
      )}
    </div>
  );
};

Statistics.propTypes = {
  totalAmount: PropTypes.number.isRequired,
  promotionAmount: PropTypes.number.isRequired,
  paidAmount: PropTypes.number.isRequired,
  transactionAmount: PropTypes.number,
  depositAmount: PropTypes.number,
  transactionVisible: PropTypes.bool,
};

Statistics.defaultProps = {
  transactionAmount: 0,
  depositAmount: 0,
  transactionVisible: true,
};

export default Statistics;
