import React from 'react';
import PropTypes from 'prop-types';
import ConfirmByShareWay from 'BLOC/ConfirmByShareWay';
import { getSmsTemplate, sendContractConfirmSms } from 'SERVICE/contract';
import { getFrontEndConfig } from 'SERVICE/common';
import { errorsHandle, runFunc, desensitizationPhoneNumber } from 'UTIL/common';
import showMessage from 'SRC/components/utils/globalTips/showMessage';

class ModalOnlineConfirmWay extends React.Component {
  constructor(props) {
    super(props);

    this.init = this.init.bind(this);
    this.initData = this.initData.bind(this);
    this.sendMessage = this.sendMessage.bind(this);

    const contractInfo = props.contractInfo || {};

    this.state = {
      loading: true,
      url: undefined,
      template: undefined,
      messageInfo: [
        { text: `发送号码：${desensitizationPhoneNumber(contractInfo.contractNumber) || '-'}` },
        { text: `家长姓名：${contractInfo.customerName || '-'}` },
      ],
      posterInfo: [
        { text: `学生：${contractInfo.studentName || '-'}` },
        { text: `学习中心：${contractInfo.campusName || '-'}` },
      ],
    };
  }

  static propTypes = {
    title: PropTypes.string,
    onCancel: PropTypes.func.isRequired,
    contractInfo: PropTypes.shape({
      // 批次id
      id: PropTypes.string.isRequired,
      contractIds: PropTypes.array,
      contractNumber: PropTypes.string,
      customerName: PropTypes.string,
      studentName: PropTypes.string,
      signTime: PropTypes.string,
      campusName: PropTypes.string,
    }).isRequired,
    radioPoster: PropTypes.bool,
    radioMessage: PropTypes.bool,
    radioLink: PropTypes.bool,
  };

  static defaultProps = {
    title: '确认信息',
    radioPoster: true,
    radioMessage: true,
    radioLink: true,
  };

  componentDidMount() {
    this.init();
  }

  async init() {
    await new Promise(this.initData);
    this.setState({ loading: false });
  }

  async initData(resolve) {
    try {
      let fetchList = [
        getFrontEndConfig({
          key: 'history-confirm-h5-url',
          module: 'contract',
        }),
        getSmsTemplate({
          templateCodeTag: 'SMS_CONTRACT_CONFIRM_HISCONTRACT',
        }),
      ];

      const [resUrl, resTemplate] = await Promise.all(fetchList);

      this.setState(
        {
          url: resUrl.data.replace('#{batchId}', this.props.contractInfo.id),
          template: resTemplate.data.templateContent,
        },
        resolve,
      );
    } catch (e) {
      errorsHandle(e);
      resolve();
    }
  }

  async sendMessage() {
    const apiParams = {
      batchId: this.props.contractInfo.id,
    };
    const res = await sendContractConfirmSms(apiParams).catch(errorsHandle());
    if (res && res.data) {
      showMessage('发送成功！', 'success');
      runFunc(this.props.onCancel);
    }
  }

  render() {
    return (
      <ConfirmByShareWay
        title="确认信息"
        loading={this.state.loading}
        onCancel={this.props.onCancel}
        poster={{
          infos: this.state.posterInfo,
          url: this.state.url,
          tips: '请扫码确认相关信息',
        }}
        message={{
          infos: this.state.messageInfo,
          template: this.state.template,
          onSend: this.sendMessage,
        }}
        link={{
          url: this.state.url,
        }}
      />
    );
  }
}

export default ModalOnlineConfirmWay;
