import React from 'react';
import { connect } from 'dva';
import { Modal, Select, FormItem } from 'COMPONENT';
import { storeName } from '../store/config';
import { debounce } from 'lodash';
import { getUserSimple, batchDistributeDeliverSs } from 'SERVICE/contract';
import { errorsHandling } from '../store/utils';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

class ModalChangeClassApply extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false,
      deliverDict: [],
      deliverId: null,
    };

    this.onSearchDeliver = this.onSearchDeliver.bind(this);
    this.doSearchDeliver = this.doSearchDeliver.bind(this);
    this.onDistribute = this.onDistribute.bind(this);
  }

  onSearchDeliver(val, key) {
    debounce(this.doSearchDeliver, 300)(val, key);
  }

  async doSearchDeliver(val) {
    const dict = await getUserSimple({ term: val });
    this.setState({
      deliverDict: dict.map((item) => ({
        text: `${item.username}（${item.employeeNo}）`,
        val: item.userId,
        title: item.username,
      })),
    });
  }

  async onDistribute() {
    const apiParams = {
      studentIds: this.props.selectedList.map((v) => v.studentId),
      deliverTagert: this.state.deliverId,
    };
    const res = await batchDistributeDeliverSs(apiParams, true).catch(errorsHandling());
    if (res.resultCode === 0) {
      showMessage('操作成功', 'success');
      this.props.onClose();
      this.props.onSuc();
      this.props.dispatch({
        type: `${storeName}/updateState`,
        payload: {
          selectedRowKeys: [],
          selectedList: [],
        },
      });
    }
  }

  render() {
    return (
      <Modal
        title="分配跟进人"
        loading={this.state.loading}
        width={600}
        onCancel={this.props.onClose}
        onOk={this.onDistribute}
        okText="分配"
        cancelBtnShow={false}
      >
        <p style={{ paddingBottom: 20 }}>
          已选择<span>{this.props.selectedRowKeys.length}</span>个学生
        </p>
        <FormItem name="选择跟进人" labelWidth={80} style={{ paddingLeft: 0 }}>
          <Select
            placeholder="选择跟进人"
            dict={this.state.deliverDict}
            value={this.state.deliverId}
            onChange={(val) => this.setState({ deliverId: val })}
            showSearch={true}
            noOptionTips="没有找到"
            searchPlaceholder="输入名称搜索"
            onSearchKeyChange={(val) => this.onSearchDeliver(val)}
          />
        </FormItem>
      </Modal>
    );
  }
}
const mapStateToProps = (state) => {
  return {
    selectedRowKeys: state[storeName].selectedRowKeys,
    selectedList: state[storeName].selectedList,
  };
};
export default connect(mapStateToProps)(ModalChangeClassApply);
