/**
 * Created by Haolin on 2019/2/18.
 */
import React from 'react';
import { connect } from 'dva';
import { SearchBar, Select, Input } from 'COMPONENT';
import { storeName, confirmStatusDict, deliverStatusDict } from '../store/config';
import { buildBranchCampusDict } from 'MODEL/dict/utils';

class Search extends React.Component {
  constructor(props) {
    super(props);

    this.search = this.search.bind(this);
    this.reset = this.reset.bind(this);
    this.updateKeyword = this.updateKeyword.bind(this);
  }
  componentWillUnmount() {
    this.reset();
  }

  search() {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        pageCurrent: 1,
      },
    });
    this.props.dispatch({
      type: `${storeName}/loadList`,
      payload: {},
    });
  }
  reset() {
    this.props.dispatch({
      type: `${storeName}/resetKeyword`,
    });
  }
  updateKeyword(type, val) {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: { [type]: val },
    });
  }

  render() {
    const {
      dictBranch,
      branchCampusDict,
      gradeDict,
      sKey_branch,
      sKey_campus,
      sKey_grade,
      sKey_clientName,
      sKey_contact,
      sKey_stdName,
      sKey_deliverTargetName,
      sKey_signStaffName,
      sKey_confirmStatus,
      sKey_hasDeliver,
    } = this.props;

    return (
      <SearchBar onSearch={this.search} onReset={this.reset}>
        <Select
          placeholder="分公司"
          dict={dictBranch}
          value={sKey_branch}
          onChange={(val) => {
            this.updateKeyword('sKey_branch', val);
          }}
        />
        <Select
          placeholder="学习中心"
          dict={branchCampusDict}
          value={sKey_campus}
          onChange={(val) => {
            this.updateKeyword('sKey_campus', val);
          }}
        />
        <Select
          placeholder="年级"
          dict={gradeDict}
          value={sKey_grade}
          onChange={(val) => this.updateKeyword('sKey_grade', val)}
        />
        <Input
          placeholder="客户姓名"
          value={sKey_clientName}
          maxLength={10}
          onChange={(e) => this.updateKeyword('sKey_clientName', e.target.value)}
        />
        <Input
          placeholder="客户号码"
          value={sKey_contact}
          onChange={(e) => this.updateKeyword('sKey_contact', e.target.value)}
        />
        <Input
          placeholder="学生姓名"
          value={sKey_stdName}
          maxLength={10}
          onChange={(e) => this.updateKeyword('sKey_stdName', e.target.value)}
        />
        <Input
          placeholder="跟进人"
          value={sKey_deliverTargetName}
          maxLength={10}
          onChange={(e) => this.updateKeyword('sKey_deliverTargetName', e.target.value)}
        />
        <Input
          placeholder="签单人"
          value={sKey_signStaffName}
          maxLength={10}
          onChange={(e) => this.updateKeyword('sKey_signStaffName', e.target.value)}
        />
        <Select
          placeholder="确认状态"
          dict={confirmStatusDict}
          value={sKey_confirmStatus}
          onChange={(val) => this.updateKeyword('sKey_confirmStatus', val)}
        />
        <Select
          placeholder="分配情况"
          dict={deliverStatusDict}
          value={sKey_hasDeliver}
          onChange={(val) => this.updateKeyword('sKey_hasDeliver', val)}
        />
      </SearchBar>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    // 数据字典
    dictBranch: state.dict.branch,
    branchCampusDict: buildBranchCampusDict([state[storeName].sKey_branch], state.dict.branchCampus),
    gradeDict: state.dict.grade,

    sKey_branch: state[storeName].sKey_branch,
    sKey_campus: state[storeName].sKey_campus,
    sKey_grade: state[storeName].sKey_grade,
    sKey_clientName: state[storeName].sKey_clientName,
    sKey_contact: state[storeName].sKey_contact,
    sKey_stdName: state[storeName].sKey_stdName,
    sKey_deliverTargetName: state[storeName].sKey_deliverTargetName,
    sKey_signStaffName: state[storeName].sKey_signStaffName,
    sKey_confirmStatus: state[storeName].sKey_confirmStatus,
    sKey_hasDeliver: state[storeName].sKey_hasDeliver,
  };
};

export default connect(mapStateToProps)(Search);
