import React from 'react';
import { connect } from 'dva';
import { Page, List, Button, Modal } from 'COMPONENT';
import { storeName } from './store/config';
import Search from './Search';
import './style.less';
import ModalDistributeDeliver from './ModalDistributeDeliver';
import ModalOnlineConfirmWay from '../ModalOnlineConfirmWay';
import { clone } from 'lodash';

const { COL_WIDTH_M, COL_WIDTH_XL } = List;
class ConfirmHistory extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false,
      isShowDistributeModal: false,
      showConfirmDailog: false,
      isShowSendMsgModal: false,
    };

    this.init = this.init.bind(this);
    this.loadData = this.loadData.bind(this);
    this.refreshData = this.refreshData.bind(this);
    this.initDict = this.initDict.bind(this);
    this.updatePage = this.updatePage.bind(this);
    this.changePage = this.changePage.bind(this);
    this.confirmContract = this.confirmContract.bind(this);
    this.tableRowClick = this.tableRowClick.bind(this);
    this.selectRows = this.selectRows.bind(this);
    this.onSelectAll = this.onSelectAll.bind(this);
    this.distribute = this.distribute.bind(this);
    this.onCloseDistributeModal = this.onCloseDistributeModal.bind(this);
    this.onCloseConfirmDialog = this.onCloseConfirmDialog.bind(this);
    this.onSureBatchSend = this.onSureBatchSend.bind(this);

    this.columns = [
      { title: '分校', dataIndex: 'brenchName', key: 'brenchName', width: COL_WIDTH_M },
      { title: '学习中心', dataIndex: 'blCampusName', key: 'blCampusName', width: COL_WIDTH_M },
      { title: '学生ID', dataIndex: 'studentId', key: 'studentId', width: COL_WIDTH_XL },
      { title: '学生姓名', dataIndex: 'studentName', key: 'studentName', width: COL_WIDTH_XL },
      { title: '年级', dataIndex: 'gradeName', key: 'gradeName', width: COL_WIDTH_M },
      { title: '客户姓名', dataIndex: 'customerName', key: 'customerName', width: COL_WIDTH_M },
      { title: '家长联系电话', dataIndex: 'customerContact', key: 'customerContact', width: COL_WIDTH_M },
      { title: '合同编号', dataIndex: 'contractIds', key: 'contractIds', width: COL_WIDTH_XL },
      { title: '辅导老师/老师', dataIndex: 'teacherNames', key: 'teacherNames', width: COL_WIDTH_XL },
      { title: '签单人', dataIndex: 'signStaffNames', key: 'signStaffNames', width: COL_WIDTH_XL },
      { title: '跟进人', dataIndex: 'deliverTargetName', key: 'deliverTargetName', width: COL_WIDTH_XL },
      { title: '确认状态', dataIndex: 'confirmStatusName', key: 'confirmStatusName', width: COL_WIDTH_M },
      {
        title: '操作',
        key: 'action',
        width: 140,
        fixed: 'right',
        getActionConfig: (text, record, index) => {
          let config = [];
          if (this.props.userId === record.deliverTargetId) {
            config = [
              {
                text: '确认历史合同',
                onClick: () => {
                  this.confirmContract(record);
                },
              },
            ];
          }
          return config;
        },
      },
    ];

    this.contractInfo = null;
  }

  componentDidMount() {
    this.init();
  }

  componentWillUnmount() {
    this.destroy();
  }

  // 初始化
  async init() {
    this.setState({ loading: true });
    await new Promise(this.initDict);
    await new Promise(this.loadData);
    this.setState({ loading: false });
  }

  // 初始化
  async refreshData() {
    this.setState({ loading: true });
    await new Promise(this.loadData);
    this.setState({ loading: false });
  }

  initDict(resolve) {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['branch', 'branchCampus', 'grade'],
        callback() {
          resolve();
        },
      },
    });
  }

  loadData(resolve) {
    this.props.dispatch({
      type: `${storeName}/loadList`,
      payload: {
        callback() {
          resolve();
        },
      },
    });
  }

  updatePage(pageCurrent, pageSize) {
    let payload = {};
    if (pageSize && pageSize !== this.props.pageSize) {
      payload.pageCurrent = 1;
      payload.pageSize = pageSize;
    } else {
      payload.pageCurrent = pageCurrent;
    }
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload,
    });
  }

  changePage({ current, pageSize }) {
    this.updatePage(current, pageSize);
    this.refreshData();
  }

  // 确认历史合同
  confirmContract(row) {
    const { studentId, blCampusName, studentName, customerName, customerContact, contractIds } = row;
    this.props.dispatch({
      type: `${storeName}/confirmHistoryContract`,
      payload: {
        studentId,
        callback: (res) => {
          if (res && res.data) {
            this.contractInfo = {
              campusName: blCampusName,
              type: 'confirm',
              studentName,
              customerName,
              contractNumber: customerContact,
              contractIds: contractIds.split(','),
              id: res.data,
            };
            this.setState({ showConfirmDailog: true });
          }
        },
      },
    });
  }

  onSureBatchSend() {
    this.props.dispatch({
      type: `${storeName}/batchSendSms`,
      payload: {
        callback: () => {
          this.setState({ isShowSendMsgModal: false });
        },
      },
    });
  }

  // 打开分配跟进人弹窗
  distribute() {
    this.setState({
      isShowDistributeModal: true,
    });
  }

  tableRowClick(record, index) {
    const selected = !this.props.selectedRowKeys.includes(index);
    const _selectedRowKeys = clone(this.props.selectedRowKeys);
    if (selected) {
      _selectedRowKeys.push(index);
    } else {
      _selectedRowKeys.filter((item) => item !== index);
    }
    const selectedRows = _selectedRowKeys.map((index) => clone(this.props.list[index]));
    this.selectRows(record, selected, selectedRows);
  }

  onSelectAll(selected, selectedRows, changeRows) {
    let selectedRowKeys = [];
    if (selected) {
      selectedRowKeys = selectedRows.map((item) => item.key);
    }
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        selectedRowKeys,
        selectedList: selectedRows,
      },
    });
  }

  selectRows(record, selected, selectedRows) {
    this.props.dispatch({
      type: `${storeName}/selectRowForChange`,
      payload: {
        selected,
        record,
        index: record.key,
      },
    });
  }

  onCloseDistributeModal() {
    this.setState({ isShowDistributeModal: false });
  }

  onCloseConfirmDialog() {
    this.refreshData();
    this.setState({ showConfirmDailog: false });
  }

  destroy() {
    this.props.dispatch({
      type: `${storeName}/resetPage`,
    });
  }

  render() {
    const btnDisabled = this.props.selectedRowKeys.length === 0;
    return (
      <Page
        name="确认历史合同"
        loading={this.state.loading}
        route={this.props.route}
        fitWindowHeight
        subHeader={
          <div className="confirm-history-search-bar">
            <Search />
            <Button
              type="primary"
              className="send-btn"
              onClick={() => this.setState({ isShowSendMsgModal: true })}
              disabled={btnDisabled}
            >
              批量发送短信
            </Button>
            {this.props.roleCode.indexOf('CAMPUS_OPERATION_DIRECTOR') !== -1 && (
              <Button type="primary" className="distribute-btn" onClick={this.distribute} disabled={btnDisabled}>
                分配跟进人
              </Button>
            )}
          </div>
        }
      >
        <List
          dataSource={this.props.list}
          columns={this.columns}
          rowSelection={{
            selectedRowKeys: this.props.selectedRowKeys,
            onSelect: this.selectRows,
            onSelectAll: this.onSelectAll,
          }}
          pagination={{
            total: this.props.total,
            current: this.props.pageCurrent,
            pageSize: this.props.pageSize,
          }}
          onChange={this.changePage}
          onRowClick={this.tableRowClick}
        />
        {this.state.isShowDistributeModal && (
          <ModalDistributeDeliver onClose={this.onCloseDistributeModal} onSuc={this.refreshData} />
        )}
        {this.state.showConfirmDailog && (
          <ModalOnlineConfirmWay
            contractInfo={this.contractInfo}
            onCancel={this.onCloseConfirmDialog}
            radioLink={false}
          />
        )}
        {this.state.isShowSendMsgModal && (
          <Modal
            title="发送短信"
            width={380}
            onCancel={() => this.setState({ isShowSendMsgModal: false })}
            onOk={this.onSureBatchSend}
            okText="是"
            cancelText="否"
          >
            <p style={{ paddingBottom: 5 }}>
              已选择<span>{this.props.selectedRowKeys.length}</span>个学生
            </p>
            <p>即将发送确认历史合同的短信至所选学生的家长，是否继续</p>
          </Modal>
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    list: state[storeName].list,
    total: state[storeName].listTotal,
    pageSize: state[storeName].pageSize,
    pageCurrent: state[storeName].pageCurrent,
    selectedRowKeys: state[storeName].selectedRowKeys,
    roleCode: state.account.user.roleCode,
    userId: state.account.user.userId,
  };
};

export default connect(mapStateToProps)(ConfirmHistory);
