import { storeName } from './config';
import { clone } from 'lodash';
import { getHistoryContractListSs, confirmHistoryContract, confirmHistoryContractBatchSendSms } from 'SERVICE/contract';
import { errorsHandling, runFunc } from './utils';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

const status = {
  0: false,
  1: true,
};

const defaultState = {
  sKey: {
    sKey_branch: null,
    sKey_campus: null,
    sKey_grade: null,
    sKey_clientName: '',
    sKey_contact: '',
    sKey_stdName: '',
    sKey_deliverTargetName: '',
    sKey_signStaffName: '',
    sKey_confirmStatus: null,
    sKey_hasDeliver: null,
  },
  list: {
    // 列表数据
    list: [],
    listTotal: 0,
    pageSize: 20,
    pageCurrent: 1,
    selectedRowKeys: [],
    selectedList: [],
  },
};

Object.freeze(defaultState);

export default {
  namespace: storeName,
  state: {
    ...clone(defaultState.sKey),
    ...clone(defaultState.list),
  },
  reducers: {
    updateState(state, action) {
      const { payload } = action;
      return {
        ...state,
        ...payload,
      };
    },
    updateList(state, { payload: { list, total } }) {
      return {
        ...state,
        list,
        listTotal: total,
        selectedRowKeys: [],
      };
    },
    // 更新已选列表key
    updateSelectedRowKeys(state, { payload: { selected, index } }) {
      let selectedRowKeys = clone(state.selectedRowKeys);
      if (selected) {
        selectedRowKeys.unshift(index);
      } else {
        selectedRowKeys = selectedRowKeys.filter((item) => item !== index);
      }
      return {
        ...state,
        selectedRowKeys,
      };
    },
    // 更新已选列表
    updateSelectedList(state, { payload: { selected, record } }) {
      let selectedList = clone(state.selectedList);
      if (selected) {
        let std = clone(record);
        delete std.key;
        selectedList.unshift(std);
      } else {
        selectedList = selectedList.filter((item) => item.studentId !== record.studentId);
      }
      return {
        ...state,
        selectedList,
      };
    },
  },
  effects: {
    *resetKeyword(action, { put }) {
      yield put({
        type: 'updateState',
        payload: {
          ...clone(defaultState.sKey),
        },
      });
    },

    *loadList({ payload: { callback } }, { select, put }) {
      const apiParams = yield select((state) => {
        const store = state[storeName];
        return {
          branchId: store.sKey_branch,
          campusId: store.sKey_campus,
          gradeId: store.sKey_grade,
          customerName: store.sKey_clientName,
          customerContact: store.sKey_contact,
          studentName: store.sKey_stdName,
          deliverTargetName: store.sKey_deliverTargetName,
          signStaffName: store.sKey_signStaffName,
          confirmStatus: store.sKey_confirmStatus,
          hasDeliver: store.sKey_hasDeliver ? status[store.sKey_hasDeliver] : null,
          page: store.pageCurrent,
          rows: store.pageSize,
        };
      });
      const rsp = yield getHistoryContractListSs(apiParams, true).catch(errorsHandling());
      if (rsp && rsp.resultCode === 0) {
        yield put({
          type: 'updateList',
          payload: {
            list: rsp.rows,
            total: rsp.records,
          },
        });
        yield put({
          type: 'updateState',
          payload: {
            selectedRowKeys: [],
            selectedList: [],
          },
        });
      }
      runFunc(callback);
    },

    *selectRowForChange({ payload: { selected, record, index } }, { select, put }) {
      yield put({
        type: 'updateSelectedRowKeys',
        payload: { selected, index },
      });
      yield put({
        type: 'updateSelectedList',
        payload: { selected, record },
      });
    },

    *confirmHistoryContract({ payload: { callback, studentId } }) {
      const apiParams = {
        studentId,
      };
      const rsp = yield confirmHistoryContract(apiParams, true).catch(errorsHandling());
      runFunc(callback, rsp);
    },

    *batchSendSms({ payload: { callback } }, { select, put }) {
      const apiParams = yield select((state) => ({
        studentIds: state[storeName].selectedList.map((v) => v.studentId),
      }));
      const rsp = yield confirmHistoryContractBatchSendSms(apiParams, true).catch(errorsHandling());
      if (rsp.resultCode === 0) {
        yield put({
          type: 'updateState',
          payload: {
            selectedList: [],
            selectedRowKeys: [],
          },
        });
        showMessage('操作成功', 'success');
      }
      runFunc(callback);
    },

    *resetPage(action, { put }) {
      yield put({
        type: 'updateState',
        payload: {
          ...clone(defaultState.sKey),
          ...clone(defaultState.list),
        },
      });
    },
  },
};
