import React from 'react';
import { browserHistory } from 'dva/router';
import { Page, Input, Select, List, TextLink, SearchFilter, SearchField, Modal, Popover, Button } from 'COMPONENT';
import ModalOnlineConfirmWay from '../ModalOnlineConfirmWay/index';
import { getUserByRoldCodesSelection } from 'SERVICE/common';
import { formateDict } from 'UTIL/dict';
import {
  loadCommonDict,
  loadBranchCampusDict, // 学习中心字典
  loadBranchDict,
  dealCampus, // 分公司
} from 'UTIL/dict/index';
import { OLDPAGEBASE } from 'CONST/config';
import {
  confirmHistoryContractList,
  confirmHistoryContract,
  confirmHistoryContractBatchSendSms,
} from 'SERVICE/contract';
import { message } from 'antd';
import { loginUser } from 'UTIL/common';

const StatusDict = [
  { val: '1', text: '待确认' },
  { val: '2', text: '确认中' },
];

const initSearchParams = {
  campusId: '', // 学习中心id
  gradeId: '', // 年级id
  customerName: '', // 客户姓名
  studentName: '', // 学生姓名
  customerContact: '', // 客户手机号
  studyManagerId: '', // 学管师id
  homeroomTeacherId: '', // 班主任id
  branchId: '', // 分公司id
  confirmStatus: '', // 确认状态
};

export default class ConfirmHistory extends React.Component {
  loginUser = loginUser;

  state = {
    loading: false,
    // 列表数据
    dataList: [],
    // 查询参数
    searchParams: initSearchParams,
    // 分公司
    branchList: [],
    // 学习中心
    campusList: [],
    // 班主任
    monitors: [],
    // 学管师
    teacherManagers: [],
    // 年级字典
    gradeDict: [],
    pagination: {
      total: 0,
      current: 1,
      pageSize: 20,
    },
    showConfirmDailog: false,
    rowSelection: {
      onChange: (selectedRowKeys) => {
        const { rowSelection } = this.state;
        rowSelection.selectedRowKeys = selectedRowKeys;
        this.setState({ rowSelection });
      },
      selectedRowKeys: [],
    },
  };

  columns = [
    { title: '学生姓名', dataIndex: 'studentName', key: 'studentName', width: 150 },
    { title: '客户姓名', dataIndex: 'customerName', key: 'customerName', width: 150 },
    {
      title: '客户号码',
      dataIndex: 'customerContact',
      key: 'customerContact',
      width: 120,
      render: function (text) {
        return text ? `${text.slice(0, 3)}****${text.slice(7, 11)}` : '';
      },
    },
    { title: '年级', dataIndex: 'gradeName', key: 'gradeName', width: 120 },
    { title: '分公司', dataIndex: 'branchName', key: 'branchName', width: 120 },
    { title: '学习中心', dataIndex: 'campusName', key: 'campusName', width: 120 },
    { title: '学管师', dataIndex: 'studyManagerName', key: 'studyManagerName', width: 120 },
    { title: '学生班主任', dataIndex: 'homeroomTeacherName', key: 'homeroomTeacherName', width: 120 },
    {
      title: '待确认历史合同数',
      width: 132,
      render: function (record) {
        return record.unConfirmContractIds && Array.isArray(record.unConfirmContractIds) ? (
          record.unConfirmContractIds.length ? (
            <Popover
              trigger="click"
              content={record.unConfirmContractIds.map((item) => {
                return (
                  <p key={item} style={{ lineHeight: '24px' }}>
                    合同编号：{item}
                  </p>
                );
              })}
            >
              <TextLink>{record.unConfirmContractIds.length}</TextLink>
            </Popover>
          ) : (
            <TextLink>0</TextLink>
          )
        ) : null;
      },
    },
    {
      title: '确认中历史合同数',
      width: 132,
      render: function (record) {
        return record.confirmingContractIds && Array.isArray(record.confirmingContractIds) ? (
          record.confirmingContractIds.length ? (
            <Popover
              trigger="click"
              content={record.confirmingContractIds.map((item) => {
                return (
                  <p key={item} style={{ lineHeight: '24px' }}>
                    合同编号：{item}
                  </p>
                );
              })}
            >
              <TextLink>{record.confirmingContractIds.length}</TextLink>
            </Popover>
          ) : (
            <TextLink>0</TextLink>
          )
        ) : null;
      },
    },
    {
      title: '操作',
      key: 'action',
      fixed: 'right',
      width: 150,
      getActionConfig: (text, record) => {
        return [
          {
            text: '确认历史合同',
            onClick: this.handleConfirmContract.bind(this, record),
          },
          {
            text: '详情',
            onClick: function () {
              window.sessionStorage.setItem(
                'student_list:startup_params',
                JSON.stringify({
                  studentName: record.studentName,
                  actionType: 'SEARCH_BY_STUDENT_NAME',
                  from: window.location.pathname + window.location.search + window.location.hash,
                  timestamp: Date.now(),
                }),
              );
              browserHistory.push({
                pathname: OLDPAGEBASE,
                query: {
                  src: 'function/student/studentList.html',
                  title: '学生列表',
                },
              });
            },
          },
        ];
      },
    },
  ];

  contractInfo = null;

  isHomeTeacher = false;

  isStudyTeacher = false;

  constructor(props) {
    super(props);
    if (this.loginUser.userData) {
      const { roleCode } = this.loginUser.userData;
      // 角色列表
      const roles = roleCode.split(',');
      // 班主任角色
      this.isHomeTeacher = roles.includes('HOMEROOM_TEACHER');
      // 学管师
      this.isStudyTeacher = roles.includes('STUDY_MANAGER') || roles.includes('STUDY_MANAGER_HEAD');
    }
  }

  async componentDidMount() {
    try {
      this.setState({ loading: true });
      const { searchParams } = this.state;
      const [branchList, gradeDict, monitorsDict, teacherManagersDict] = await Promise.all([
        loadBranchDict(),
        loadCommonDict('STUDENT_GRADE'),
        getUserByRoldCodesSelection({ roleCode: 'HOMEROOM_TEACHER' }),
        getUserByRoldCodesSelection({ roleCode: 'STUDY_MANAGER, STUDY_MANAGER_HEAD' }),
      ]);
      const monitors = formateDict(monitorsDict);
      const teacherManagers = formateDict(teacherManagersDict);
      const fillParams = {};
      // 筛选默认
      if (this.isStudyTeacher) {
        teacherManagers.forEach((item) => {
          if (item.val === this.loginUser.userData.userId) {
            fillParams.studyManagerId = item.val;
          }
        });
      } else if (this.isHomeTeacher) {
        monitors.forEach((item) => {
          if (item.val === this.loginUser.userData.userId) {
            fillParams.homeroomTeacherId = item.val;
          }
        });
      }
      this.setState(
        {
          branchList,
          gradeDict,
          monitors,
          teacherManagers,
          searchParams: { ...searchParams, ...fillParams },
        },
        () => {
          this.getHistoryContractList();
        },
      );
    } finally {
      this.setState({ loading: false });
    }
  }

  getHistoryContractList = async () => {
    this.setState({ loading: true });
    const { searchParams, pagination } = this.state;
    try {
      const res = await confirmHistoryContractList({
        ...searchParams,
        page: pagination.current,
        rows: pagination.pageSize,
      });
      if (res.resultCode == 0) {
        this.setState({
          dataList: Array.isArray(res.data) ? res.data : [],
          pagination: { ...pagination, total: res.rowCount },
        });
      }
    } finally {
      this.setState({ loading: false });
    }
  };

  handleBranchIdChange = async (branchId) => {
    const campus = await loadBranchCampusDict();
    const { searchParams } = this.state;
    const newParams = { ...searchParams, branchId };
    this.setState({ campusList: branchId ? dealCampus(campus, branchId) : [], searchParams: newParams });
  };

  handleSearchParamChange = (key) => {
    return function binder(e) {
      const { searchParams } = this.state;
      const update = {};
      update[key] = e ? (e.target ? e.target.value : e) : '';
      this.setState({ searchParams: { ...searchParams, ...update } });
    };
  };

  handleSearch = () => {
    const { pagination } = this.state;
    this.setState({ pagination: { ...pagination, current: 1 } }, this.getHistoryContractList);
  };

  handleReset = () => {
    const { pagination } = this.state;
    this.setState(
      { searchParams: initSearchParams, pagination: { ...pagination, current: 1 } },
      this.getHistoryContractList,
    );
  };

  handlePageChange = (pagination) => {
    const { pageSize, current } = pagination;
    this.setState({ pagination: { ...this.state.pagination, pageSize, current } }, this.getHistoryContractList);
  };

  // 批量发送
  handleBatchSendSmsCode = () => {
    Modal.confirm({
      title: '提示',
      content: '即将发送确认历史合同的短信至所选学生的家长，是否继续？',
      onOk: async () => {
        try {
          const { rowSelection } = this.state;
          const res = await confirmHistoryContractBatchSendSms({ studentIds: rowSelection.selectedRowKeys });
          if (res.resultCode === 0) {
            rowSelection.selectedRowKeys = [];
            this.setState({ rowSelection });
            message.success('操作成功');
            // 刷新页面
            this.getHistoryContractList();
          }
        } catch (e) {
          throw e;
        }
      },
    });
  };

  // 确认合同
  handleConfirmContract = async (rowData) => {
    const { campusName, confirmingContractIds, studentName, customerName, customerContact, studentId } = rowData;
    try {
      this.setState({ loading: true });
      const res = await confirmHistoryContract({ studentId });
      if (res.data && res.resultCode == 0) {
        const contractInfo = {
          campusName,
          type: 'confirm',
          studentName,
          customerName,
          contractNumber: customerContact,
          contractIds: confirmingContractIds,
          id: res.data,
        };
        this.contractInfo = contractInfo;
        this.setState({ showConfirmDailog: true });
      }
    } finally {
      this.setState({ loading: false });
    }
  };

  handleCloseConfirmDialog = () => {
    this.contractInfo = null;
    this.setState({ showConfirmDailog: false });
  };

  render() {
    const {
      loading,
      searchParams,
      dataList,
      pagination,
      branchList,
      campusList,
      monitors,
      teacherManagers,
      gradeDict,
      showConfirmDailog,
      rowSelection,
    } = this.state;

    return (
      <Page
        name="确认历史合同"
        fitWindowHeight
        loadingTip="正在加载数据"
        loading={loading}
        route={this.props.route}
        headerToolBar={
          <Button type="primary" disabled={!rowSelection.selectedRowKeys.length} onClick={this.handleBatchSendSmsCode}>
            批量发送短信
          </Button>
        }
        subHeader={
          <SearchFilter truncationNum={9} onSearch={this.handleSearch} onReset={this.handleReset}>
            <SearchField>
              <Input
                value={searchParams.studentName}
                placeholder="学生姓名"
                allowClear
                maxLength={50}
                onChange={this.handleSearchParamChange('studentName').bind(this)}
              />
            </SearchField>
            <SearchField>
              <Select
                value={searchParams.gradeId}
                placeholder="年级"
                showSearch={true}
                noOptionTips="暂无数据"
                dict={gradeDict}
                onChange={this.handleSearchParamChange('gradeId').bind(this)}
              />
            </SearchField>
            <SearchField>
              <Input
                value={searchParams.customerContact}
                placeholder="客户号码"
                allowClear
                maxLength={50}
                onChange={this.handleSearchParamChange('customerContact').bind(this)}
              />
            </SearchField>
            <SearchField>
              <Input
                value={searchParams.customerName}
                placeholder="客户姓名"
                allowClear
                maxLength={50}
                onChange={this.handleSearchParamChange('customerName').bind(this)}
              />
            </SearchField>
            <SearchField>
              <Select
                value={searchParams.confirmStatus}
                showSearch={true}
                placeholder="确认状态"
                dict={StatusDict}
                noOptionTips="暂无数据"
                onChange={this.handleSearchParamChange('confirmStatus').bind(this)}
              />
            </SearchField>
            <SearchField>
              <Select
                value={searchParams.branchId}
                placeholder="分公司"
                showSearch={true}
                noOptionTips="暂无数据"
                dict={branchList}
                onChange={this.handleBranchIdChange}
              />
            </SearchField>
            <SearchField>
              <Select
                value={searchParams.campusId}
                placeholder="学习中心"
                showSearch={true}
                noOptionTips="暂无数据"
                dict={campusList}
                onChange={this.handleSearchParamChange('campusId').bind(this)}
              />
            </SearchField>
            <SearchField>
              <Select
                value={searchParams.studyManagerId}
                placeholder="学管师"
                showSearch={true}
                noOptionTips="暂无数据"
                dict={teacherManagers}
                onChange={this.handleSearchParamChange('studyManagerId').bind(this)}
              />
            </SearchField>
            <SearchField>
              <Select
                value={searchParams.homeroomTeacherId}
                placeholder="学生班主任"
                showSearch={true}
                noOptionTips="暂无数据"
                dict={monitors}
                onChange={this.handleSearchParamChange('homeroomTeacherId').bind(this)}
              />
            </SearchField>
          </SearchFilter>
        }
      >
        <List
          rowKey="studentId"
          rowSelection={rowSelection}
          dataSource={dataList}
          columns={this.columns}
          pagination={pagination}
          onChange={this.handlePageChange}
        />
        {showConfirmDailog && (
          <ModalOnlineConfirmWay contractInfo={this.contractInfo} onCancel={this.handleCloseConfirmDialog} />
        )}
      </Page>
    );
  }
}
