import React, { PropTypes } from 'react';
import SearchBar from 'COMPONENT/SearchBar';
import Select from 'COMPONENT/Select';
import Input from 'COMPONENT/Input';

const getSearchInitValue = () => {
  return {
    willExpireDays: undefined,
    customerContact: '',
    customerName: '',
    studentName: '',
  };
};

class Search extends React.Component {
  static propTypes = {
    onSearch: PropTypes.func.isRequired,
  };

  constructor(props) {
    super(props);
    this.state = {
      overdueDayList: [],
      searchParams: getSearchInitValue(),
    };
  }

  componentDidMount() {
    this.fetchOverdueDayList();
  }

  fetchOverdueDayList = () => {
    const overdueDayList = [
      { val: '4', text: '大于3天' },
      { val: '3', text: '2天' },
      { val: '2', text: '1天' },
      { val: '1', text: '1天内' },
    ];

    this.setState({
      overdueDayList,
    });
  };

  setSearchParams = (key, value) => {
    this.setState({
      searchParams: {
        ...this.state.searchParams,
        [key]: value,
      },
    });
  };

  resetSearchParams = () => {
    this.setState(
      {
        searchParams: getSearchInitValue(),
      },
      () => this.props.onSearch(this.state.searchParams),
    );
  };

  render() {
    const {
      searchParams,
      overdueDayList,
      searchParams: { willExpireDays, customerContact, customerName, studentName },
    } = this.state;

    return (
      <SearchBar onSearch={() => this.props.onSearch(searchParams)} onReset={this.resetSearchParams}>
        <Select
          placeholder="距超期天数"
          dict={overdueDayList}
          value={willExpireDays}
          onChange={(value) => {
            this.setSearchParams('willExpireDays', value);
          }}
        />
        <Input
          placeholder="客户号码"
          value={customerContact}
          onChange={(e) => {
            this.setSearchParams('customerContact', e.target.value);
          }}
        />
        <Input
          placeholder="客户姓名"
          value={customerName}
          onChange={(e) => {
            this.setSearchParams('customerName', e.target.value);
          }}
        />
        <Input
          placeholder="学生姓名"
          value={studentName}
          onChange={(e) => {
            this.setSearchParams('studentName', e.target.value);
          }}
        />
      </SearchBar>
    );
  }
}

export default Search;
