/*
 * @Author:zhengrunlin
 * @Date: 2020-10-30 10:57:09
 * @LastEditTime: 2020-11-05 15:37:45
 * @LastEditors: zhengrunlin
 * @Description: 合同超期预警常量
 * @FilePath: \boss-web-front\src\pages\contract\ContractEnsureAlarm\constants.js
 */
import moment from 'moment';
import { List } from 'COMPONENT';

export const ON_LINE = 'ON_LINE';
export const WAITING_CONFIRM = 1;
export const IN_CONFIRMING = 2;

const { COL_WIDTH_M } = List;
export const contractStatus = {
  1: '待确认',
  2: '确认中',
  4: '已确认',
  16: '无需确认',
};

export const contractChannel = {
  ON_LINE: '线上',
  OFF_LINE: '线下',
  NONE: '无',
};

export const h5ModeMap = {
  CONFIRM: 'onlyConfirm',
  PAY_CONFIRM: 'all',
};

const formatTime = (time) => {
  return time ? moment(time).format('YYYY-MM-DD HH:mm:ss') : '';
};

export const ALARM_TABLE_COLUMS = [
  {
    title: '合同编号',
    dataIndex: 'contractId',
    key: 'contractId',
    width: COL_WIDTH_M,
  },
  {
    title: '客户号码',
    dataIndex: 'customerContact',
    key: 'customerContact',
    width: COL_WIDTH_M,
  },
  {
    title: '客户姓名',
    dataIndex: 'customerName',
    key: 'customerName',
    width: COL_WIDTH_M,
  },
  {
    title: '学生姓名',
    dataIndex: 'studentName',
    key: 'studentName',
    width: COL_WIDTH_M,
  },
  {
    title: '合同学习中心',
    dataIndex: 'blCampusName',
    key: 'blCampusName',
    width: COL_WIDTH_M,
  },
  {
    title: '距超期天数',
    dataIndex: 'willExpireDays',
    key: 'willExpireDays',
    width: COL_WIDTH_M,
    render(text) {
      const expireDayMap = {
        1: <span style={{ color: 'red' }}>1天内</span>,
        2: <span style={{ color: 'red' }}>1天</span>,
        3: '2天',
      };

      return expireDayMap[text] || '大于3天';
    },
  },
  {
    title: '开始计算时间',
    dataIndex: 'confirmStartTime',
    key: 'confirmStartTime',
    width: COL_WIDTH_M,
    render: formatTime,
  },
  {
    title: '确认截止时间',
    dataIndex: 'confirmEndTime',
    key: 'confirmEndTime',
    width: COL_WIDTH_M,
    render: formatTime,
  },
  {
    title: '发起确认时间',
    dataIndex: 'confirmSubmitTime',
    key: 'confirmSubmitTime',
    width: COL_WIDTH_M,
    render: formatTime,
  },
];
