/*
 * @LastEditors: zhengrunlin
 */
import React from 'react';
import { Popover } from 'antd';
import { Page, List, QrCode, TextLink } from 'COMPONENT';
import Search from './Search';
import { ALARM_TABLE_COLUMS, ON_LINE, WAITING_CONFIRM, IN_CONFIRMING, h5ModeMap } from './constants';
import { isString } from 'UTIL/lang/data-type';
import { getContractEnsureAlarm } from 'SERVICE/contractEnsureAlarm';
import ModalConfirmContract from '../modals/ConfirmContract/index';
import ConfirmContractInformationModal from '../modals/ConfirmContractInformationModal';
import { h5ContractComfirmUrl } from 'CONST/otherSystemConfig';
import { loginUser } from 'UTIL/common';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

const EXPIRE_DAY_KEY = 'willExpireDays';

class ContractEnsureAlarm extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      tableData: [],
      pagination: {
        total: 0,
        current: 1,
        pageSize: 10,
      },
      searchParams: {},
      showEnsureDialog: false,
      showShareDialog: false,
      currentRow: null,
    };

    // 确认合同后展示各种类型的url用
    this.id = null;
    this.type = null;
  }

  componentDidMount() {
    this.fetchAlarmList();
  }

  onPageChange = (data) => {
    let pagination = this.state.pagination;
    let pageParams = Object.keys(pagination).reduce((pre, cur) => {
      pre[cur] = data[cur];
      return pre;
    }, {});

    this.setState(
      {
        pagination: pageParams,
      },
      this.fetchAlarmList,
    );
  };

  onSearch = (searchParams) => {
    this.setState(
      {
        searchParams,
      },
      this.fetchAlarmList,
    );
  };

  /**
   * @description: 请求超时告警列表
   */
  fetchAlarmList = async () => {
    const { current: page, pageSize: rows } = this.state.pagination;
    let { searchParams } = this.state;

    // 输入框字段为空时不传该参数
    // 超期时间需要传number类型
    searchParams = Object.keys(searchParams).reduce((pre, cur) => {
      if (this.formatValue(searchParams[cur])) {
        pre[cur] = cur === EXPIRE_DAY_KEY ? Number(searchParams[cur]) : searchParams[cur];
      }
      return pre;
    }, {});

    let queryParams = {
      ...searchParams,
      page,
      rows,
    };

    try {
      this.setRequestLoading(true);
      const res = await getContractEnsureAlarm(queryParams);
      const { datas, rowCount } = res;

      this.setState({
        tableData: datas || [],
        pagination: {
          ...this.state.pagination,
          total: rowCount,
        },
      });
    } catch (e) {
      showMessage(e.text);
      this.setState({
        tableData: [],
        pagination: {
          ...this.state.pagination,
          total: 0,
        },
      });
    } finally {
      this.setRequestLoading(false);
    }
  };

  formatValue = (value) => {
    if (isString(value)) {
      return value.trim();
    }
    return value;
  };

  /**
   * @description 设置列表请求Loading状态
   * @param  {boolean} loading
   * @return {void}
   */
  setRequestLoading = (loading) => {
    this.setState({
      loading,
    });
  };

  /**
   * @description:  弹框显示隐藏切换
   * @param {string} key 需要操作的弹框
   * @param {boolean} flag true/false
   * @return {void}
   */
  toggleDialog = (key, flag) => {
    const dialogMap = {
      share: 'showShareDialog',
      ensure: 'showEnsureDialog',
    };

    this.setState({
      [dialogMap[key]]: flag,
    });
  };

  /**
   * @description: 列表确认合同按钮
   * @param {Object} data 当前操作的列表行数据
   */
  ensureContract = (data) => {
    this.setState({
      currentRow: Object.assign({}, data),
    });
    this.toggleDialog('ensure', true);
  };

  /**
   * @description: 列表分享按钮按钮
   * @param {Object} data 当前操作的列表行数据
   */
  shareContract = (data) => {
    this.id = data.batchId;
    this.type = 'confirm';
    this.setState({
      currentRow: Object.assign({}, data),
    });
    this.toggleDialog('share', true);
  };

  renderOperationColumn = (text, row) => {
    const renderEnsure = <TextLink onClick={() => this.ensureContract(row)}>确认信息</TextLink>;

    const renderShare = (
      <Popover
        content={
          <QrCode
            id={`contractEnsureAlarm-${row.contractId}`}
            text={this.generateShareUrl(row.batchId, row.confirmMode)}
            width={120}
            height={120}
          />
        }
        trigger="click"
      >
        <TextLink>分享</TextLink>
      </Popover>
    );

    // 待确认展示确认合同按钮
    if (row.confirmStatus === WAITING_CONFIRM) {
      return renderEnsure;
    } else if (
      // 确认中且是线上，展示分享按钮
      row.confirmStatus == IN_CONFIRMING &&
      row.confirmChannel === ON_LINE
    ) {
      return renderShare;
    }
  };

  /**
   * @description: 确认合同成功回调
   * @param {string} id 批次号id
   * @param {string} type 确认类型，仅确认合同confirm/确认合同并支付confirmPay
   */
  handleConfirmContract = (id, type) => {
    this.id = id;
    this.type = ['pay', 'confirm'].includes(type) ? type : 'confirmPay';

    this.setState({
      showEnsureDialog: false,
      showShareDialog: true,
      radioMessage: this.type === 'confirm',
    });

    this.fetchAlarmList();
  };

  /**
   * @description: 生成合同分享地址
   * @param {string} id 批次号id
   * @param {string} type 确认类型，仅确认合同confirm/确认合同并支付confirmPay
   * @return {string} url分享地址
   */
  generateShareUrl(id, type) {
    const system = loginUser.isXingHuoSym() ? 'xh' : 'ss';
    const mode = h5ModeMap[type];

    return `${h5ContractComfirmUrl}/${system}/${id}/order/list?mode=${mode}`;
  }

  render() {
    const { route } = this.props;
    const { loading, tableData, pagination, showEnsureDialog, showShareDialog, currentRow } = this.state;

    return (
      <Page
        name={route.name}
        route={route}
        fitWindowHeight={true}
        subHeader={
          <div>
            <Search onSearch={this.onSearch} />
            <List
              dataSource={tableData}
              loading={loading}
              columns={ALARM_TABLE_COLUMS.concat({
                title: '操作',
                dataIndex: 'operation',
                width: List.COL_WIDTH_L,
                render: this.renderOperationColumn,
              })}
              pagination={pagination}
              onChange={this.onPageChange}
            />
          </div>
        }
      >
        {showEnsureDialog && (
          <ModalConfirmContract
            contractIds={currentRow.contractId}
            onCancel={() => this.toggleDialog('ensure', false)}
            onSuccess={this.handleConfirmContract}
          />
        )}

        {showShareDialog && (
          <ConfirmContractInformationModal
            radioMessage={this.type === 'confirm'}
            contractInfo={{
              id: this.id,
              type: this.type,
              contractIds: [currentRow.contractId],
              campusName: currentRow.blCampusName,
              contractNumber: currentRow.customerContact,
              customerName: currentRow.customerName,
              studentName: currentRow.studentName,
              signTime: currentRow.contractSignTime,
            }}
            onCancel={() => this.toggleDialog('share', false)}
          />
        )}
      </Page>
    );
  }
}

export default ContractEnsureAlarm;
