/**
 * Created by Haolin on 2019/2/18.
 */
import React from 'react';
import { connect } from 'dva';
import { storeName } from '../store/constants.js';
import SearchBar from 'COMPONENT/SearchBar';
import DatePicker from 'COMPONENT/DatePicker';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import Input from 'COMPONENT/Input';
import { clone } from 'lodash';
import { buildBranchDict, buildBranchCampusDict } from 'MODEL/dict/utils';

class Search extends React.Component {
  constructor(props) {
    super(props);
    // 合同产品数字典
    this.contractPrdNumDict = [
      {
        text: '单产品',
        val: '1',
      },
      {
        text: '多产品',
        val: '2',
      },
      {
        text: '单产品+多产品',
        val: '3',
      },
    ];
    this.search = this.search.bind(this);
    this.reset = this.reset.bind(this);
    this.updateKeyword = this.updateKeyword.bind(this);
    this.updateBranch = this.updateBranch.bind(this);
  }
  static classPrefix = 'Search';
  componentWillUnmount() {
    this.reset();
  }

  search() {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        pageCurrent: 1,
      },
    });
    this.props.dispatch({
      type: `${storeName}/loadList`,
      payload: {},
    });
  }
  reset() {
    this.props.dispatch({
      type: `${storeName}/resetSKey`,
    });
  }
  updateKeyword({ type, val }) {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: { [type]: val },
    });
  }

  updateBranch(val) {
    let payload = { sKey_branch: val };
    let sKey_campus = clone(this.props.sKey_campus || []);
    if (val && val.length > 0) {
      let filterBranchCampusDict = buildBranchCampusDict(val, this.props.branchCampusDictOrigin);
      let filterCampusDict = filterBranchCampusDict.map((item) => (item ? item.options : []));
      let filterValArr = filterCampusDict.reduce((arr, current) => arr.concat(current), []).map((item) => item.val);
      sKey_campus = sKey_campus.filter((val) => filterValArr.includes(val));
      payload.sKey_campus = sKey_campus;
    }
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload,
    });
  }

  render() {
    const {
      className,
      productTypeDict,
      productCharacterDict,
      productSeriesDict,
      gradeDict,
      yearDict,
      contractTypeDict,
      branchDict,
      branchCampusDict,
      subjectDict,
      quarterDict,
      resEntranceDict,
      paidStatueDict,
      sKey_beginDate,
      sKey_endDate,
      sKey_productType,
      sKey_productCharacter,
      sKey_productSeries,
      sKey_productGrade,
      sKey_productYear,
      sKey_productName,
      sKey_productNumType,
      sKey_contractType,
      sKey_branch,
      sKey_campus,
      sKey_stdGrade,
      sKey_subject,
      sKey_quarter,
      sKey_resourceEntrance,
      sKey_payStatus,
      sKey_contractId,
      sKey_stdName,
      sKey_signPerson,
      sKey_className,
    } = this.props;

    return (
      <SearchBar className={className} onSearch={this.search} onReset={this.reset}>
        <DatePicker
          placeholder="开始日期"
          value={sKey_beginDate}
          onChange={(val) => {
            this.updateKeyword({ type: 'sKey_beginDate', val: val.trim() });
          }}
        />
        <DatePicker
          placeholder="结束日期"
          value={sKey_endDate}
          onChange={(val) => {
            this.updateKeyword({ type: 'sKey_endDate', val: val.trim() });
          }}
        />
        <Select
          placeholder="产品类型"
          dict={productTypeDict}
          value={sKey_productType}
          onChange={(val) => {
            this.updateKeyword({ type: 'sKey_productType', val });
          }}
        />
        <Select
          placeholder="产品性质"
          dict={productCharacterDict}
          value={sKey_productCharacter}
          onChange={(val) => {
            this.updateKeyword({ type: 'sKey_productCharacter', val });
          }}
        />
        <Select
          placeholder="产品系列"
          dict={productSeriesDict}
          value={sKey_productSeries}
          showSearch={true}
          onChange={(val) => {
            this.updateKeyword({ type: 'sKey_productSeries', val });
          }}
        />
        <Select
          placeholder="产品年级"
          dict={gradeDict}
          value={sKey_productGrade}
          showSearch={true}
          onChange={(val) => {
            this.updateKeyword({ type: 'sKey_productGrade', val });
          }}
        />
        <Select
          placeholder="产品年份"
          dict={yearDict}
          value={sKey_productYear}
          onChange={(val) => {
            this.updateKeyword({ type: 'sKey_productYear', val });
          }}
        />
        <Input
          placeholder="产品名称"
          value={sKey_productName}
          onChange={(ele) => {
            this.updateKeyword({ type: 'sKey_productName', val: ele.target.value });
          }}
        />
        <Select
          placeholder="合同产品数"
          dict={this.contractPrdNumDict}
          value={sKey_productNumType}
          onChange={(val) => {
            this.updateKeyword({ type: 'sKey_productNumType', val });
          }}
        />
        <Select
          placeholder="合同类型"
          dict={contractTypeDict}
          value={sKey_contractType}
          onChange={(val) => {
            this.updateKeyword({ type: 'sKey_contractType', val });
          }}
        />
        <MultiSelect
          placeholder="合同分公司"
          dict={branchDict}
          value={sKey_branch}
          showSearch={true}
          onChange={this.updateBranch}
        />
        <MultiSelect
          placeholder="合同学习中心"
          dict={branchCampusDict}
          value={sKey_campus}
          showSearch={true}
          onChange={(val) => {
            this.updateKeyword({ type: 'sKey_campus', val });
          }}
        />
        <Select
          placeholder="学生年级"
          dict={gradeDict}
          value={sKey_stdGrade}
          showSearch={true}
          onChange={(val) => {
            this.updateKeyword({ type: 'sKey_stdGrade', val });
          }}
        />
        <Select
          placeholder="科目"
          dict={subjectDict}
          value={sKey_subject}
          showSearch={true}
          onChange={(val) => {
            this.updateKeyword({ type: 'sKey_subject', val });
          }}
        />
        <Select
          placeholder="季度"
          dict={quarterDict}
          value={sKey_quarter}
          onChange={(val) => {
            this.updateKeyword({ type: 'sKey_quarter', val });
          }}
        />
        <Select
          placeholder="资源入口"
          dict={resEntranceDict}
          value={sKey_resourceEntrance}
          onChange={(val) => {
            this.updateKeyword({ type: 'sKey_resourceEntrance', val });
          }}
        />
        <MultiSelect
          placeholder="支付状态"
          dict={paidStatueDict}
          value={sKey_payStatus}
          onChange={(val) => {
            this.updateKeyword({ type: 'sKey_payStatus', val });
          }}
        />
        <Input
          placeholder="合同编号"
          value={sKey_contractId}
          onChange={(ele) => {
            this.updateKeyword({ type: 'sKey_contractId', val: ele.target.value });
          }}
        />
        <Input
          placeholder="学生姓名或编号"
          value={sKey_stdName}
          onChange={(ele) => {
            this.updateKeyword({ type: 'sKey_stdName', val: ele.target.value });
          }}
        />
        <Input
          placeholder="签单人姓名"
          value={sKey_signPerson}
          onChange={(ele) => {
            this.updateKeyword({ type: 'sKey_signPerson', val: ele.target.value });
          }}
        />
        <Input
          placeholder="班级名称"
          value={sKey_className}
          onChange={(ele) => {
            this.updateKeyword({ type: 'sKey_className', val: ele.target.value });
          }}
        />
      </SearchBar>
    );
  }
}

const mapStateToProps = (state) => {
  let branchCampusDict = buildBranchCampusDict(state[storeName].sKey_branch, state.dict.branchCampus);
  return {
    // 字典
    productTypeDict: state.dict.productType,
    productCharacterDict: state.dict.productCharacter,
    productSeriesDict: state.dict.courseSeries,
    gradeDict: state.dict.grade,
    yearDict: state.dict.year,
    contractTypeDict: state.dict.contractType,
    branchDict: state.dict.branch,
    branchCampusDict,
    branchCampusDictOrigin: state.dict.branchCampus,
    subjectDict: state.dict.subject,
    quarterDict: state.dict.quarter,
    resEntranceDict: state.dict.resEntrance,
    paidStatueDict: state.dict.paidStatue,
    // 数据
    sKey_beginDate: state[storeName].sKey_beginDate,
    sKey_endDate: state[storeName].sKey_endDate,
    sKey_productType: state[storeName].sKey_productType,
    sKey_productCharacter: state[storeName].sKey_productCharacter,
    sKey_productSeries: state[storeName].sKey_productSeries,
    sKey_productGrade: state[storeName].sKey_productGrade,
    sKey_productYear: state[storeName].sKey_productYear,
    sKey_productName: state[storeName].sKey_productName,
    sKey_productNumType: state[storeName].sKey_productNumType,
    sKey_contractType: state[storeName].sKey_contractType,
    sKey_branch: state[storeName].sKey_branch,
    sKey_campus: state[storeName].sKey_campus,
    sKey_stdGrade: state[storeName].sKey_stdGrade,
    sKey_subject: state[storeName].sKey_subject,
    sKey_quarter: state[storeName].sKey_quarter,
    sKey_resourceEntrance: state[storeName].sKey_resourceEntrance,
    sKey_payStatus: state[storeName].sKey_payStatus,
    sKey_contractId: state[storeName].sKey_contractId,
    sKey_stdName: state[storeName].sKey_stdName,
    sKey_signPerson: state[storeName].sKey_signPerson,
    sKey_className: state[storeName].sKey_className,
  };
};

export default connect(mapStateToProps)(Search);
