/**
 * Created by Haolin on 2019/2/18.
 */
import React, { PropTypes } from 'react';
import { connect } from 'dva';
import style from './index.less';
import { storeName, MAX_EXPORT_NUMBER } from './store/constants.js';
import { Page, List, Icon, Popover, Button } from 'COMPONENT';
import { loginUser } from 'UTIL/common';
import Search from './Search';
import LinkBtn from './components/LinkBtn';
import ContractPrdModal from './ContractPrdModal';

const { COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;
const FilterDropDown = (props) => {
  const prefix = 'FilterDropDown';
  return (
    <Popover overlayClassName={style[`${prefix}-pop`]} content={props.text}>
      <Icon className={style[`${prefix}-icon`]} type="question_circle" size={12} />
    </Popover>
  );
};
FilterDropDown.protoTypes = {
  text: PropTypes.string.isRequired,
};
const Empty = () => {
  return <span className="Empty"> </span>;
};
const ArrayList = (props) => {
  const prefix = 'ArrayList';
  return (
    <div className={style[prefix]} style={props.width ? { width: props.width } : null}>
      {props.data.map((item, index) => (
        <div className={props.width ? style[`${prefix}-item`] : ''} key={index} title={item}>
          {item}
        </div>
      ))}
    </div>
  );
};
ArrayList.protoTypes = {
  data: PropTypes.array.isRequired,
  width: PropTypes.number,
};
const InfoWrap = (props) => {
  return <div className={style['InfoWrap']}>{props.children}</div>;
};
const Info = (props) => {
  const prefix = 'Info';
  return (
    <div className={style[prefix]}>
      <span className={style[`${prefix}-label`]}>{props.label}</span>
      <span className={style[`${prefix}-text`]}>{props.text || '-'}</span>
      {props.text && props.unit && <span className={style[`${prefix}-unit`]}>{props.unit}</span>}
    </div>
  );
};
Info.protoTypes = {
  label: PropTypes.string.isRequired,
  text: PropTypes.string,
  unit: PropTypes.string,
};

class ContractListMgmt extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      name: '',
      loading: false,
      contractPrdModal: false,
    };
    this.contractIdForModal = null;
    this.dataForModal = null;
    this.columns = [
      {
        title: '产品名称',
        dataIndex: 'productName',
        key: 'productName',
        width: COL_WIDTH_XL,
        render: (text, record, index) => <ArrayList data={text} width={180} />,
      },
      {
        title: '应收金额',
        dataIndex: 'totalPay',
        key: 'totalPay',
        width: COL_WIDTH_M,
        render: (text, record, index) => <ArrayList data={text} />,
      },
      {
        title: '已收金额',
        dataIndex: 'paidAmount',
        key: 'paidAmount',
        width: COL_WIDTH_M,
        render: (text, record, index) => <ArrayList data={text} />,
      },
      {
        title: '支付状态',
        dataIndex: 'payStatus',
        key: 'payStatus',
        width: COL_WIDTH_M,
        render: (text, record, index) => <ArrayList data={text} />,
      },
      {
        title: '合同编号',
        dataIndex: 'contractId',
        key: 'contractId',
        width: COL_WIDTH_L,
        filterDropdownVisible: false,
        filterIcon: <FilterDropDown text="点击合同编号查看该合同下的所有产品" />,
        filterDropdown: <Empty />,
        render: (text, record, index) => <LinkBtn text={text} data={record} onClick={this.toggleContractPrdModal} />,
      },
      { title: '学生编号', dataIndex: 'studentId', key: 'studentId', width: COL_WIDTH_L },
      { title: '学生姓名', dataIndex: 'studentName', key: 'studentName', width: COL_WIDTH_M },
      { title: '学校', dataIndex: 'schoolName', key: 'schoolName', width: COL_WIDTH_L },
      { title: '合同类型', dataIndex: 'contractType', key: 'contractType', width: COL_WIDTH_M },
      { title: '合同性质', dataIndex: 'contractCharacter', key: 'contractCharacter', width: COL_WIDTH_M },
      { title: '签约时间', dataIndex: 'signDate', key: 'signDate', width: COL_WIDTH_M },
      { title: '资源入口', dataIndex: 'resourceEntrance', key: 'resourceEntrance', width: COL_WIDTH_M },
      { title: '签单人', dataIndex: 'signPerson', key: 'signPerson', width: COL_WIDTH_M },
      { title: '合同学习中心', dataIndex: 'campus', key: 'campus', width: COL_WIDTH_M },
    ];
    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.initMain = this.initMain.bind(this);
    this.loadList = this.loadList.bind(this);
    this.changePage = this.changePage.bind(this);
    this.exportExcel = this.exportExcel.bind(this);
    this.toggleContractPrdModal = this.toggleContractPrdModal.bind(this);
    this.beforeDestroy = this.beforeDestroy.bind(this);
  }
  static classPrefix = 'ContractListMgmt';
  componentDidMount() {
    this.init();
  }
  componentWillUnmount() {
    this.beforeDestroy();
  }
  // 初始化
  async init() {
    this.setState({ loading: true });
    await this.initDict();
    await this.initMain();
    this.setState({ loading: false });
  }
  initDict() {
    const { dispatch } = this.props;
    return new Promise((resolve, reject) => {
      dispatch({
        type: 'dict/initDict',
        payload: {
          list: [
            'productType',
            'productCharacter',
            'courseSeries',
            'grade',
            'year',
            'contractType',
            'branch',
            'branchCampus',
            'subject',
            'quarter',
            'resEntrance',
            'paidStatue',
          ],
          callback() {
            resolve();
          },
        },
      });
    });
  }
  initMain() {
    const { dispatch } = this.props;
    return new Promise((resolve, reject) => {
      dispatch({
        type: `${storeName}/initMain`,
        payload: {
          callback() {
            resolve();
          },
        },
      });
    });
  }
  // 加载列表数据
  loadList() {
    this.props.dispatch({
      type: `${storeName}/loadList`,
      payload: {},
    });
  }
  // 翻页
  changePage({ current, pageSize }) {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        pageCurrent: current,
        pageSize,
      },
    });
    this.loadList();
  }
  // 导出
  exportExcel() {
    this.setState({ loading: true });
    const _self = this;
    this.props.dispatch({
      type: `${storeName}/exportExcel`,
      payload: {
        callback() {
          _self.setState({ loading: false });
        },
      },
    });
  }
  // 切换显示合同产品弹窗
  toggleContractPrdModal(record) {
    if (record) {
      this.contractIdForModal = record.contractId;
      this.dataForModal = record;
    }
    this.setState(
      (prevState) => ({
        contractPrdModal: !prevState.contractPrdModal,
      }),
      () => {
        if (!this.state.contractPrdModal) {
          this.contractIdForModal = null;
          this.dataForModal = null;
        }
      },
    );
  }
  // 组件注销前销毁用到的store数据
  beforeDestroy() {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        list: [],
        total: 0,
        pageCurrent: 1,
        pageSize: 10,
        listLoading: false,
        contractNum: null,
        productNum: null,
        stdNum: null,
        amount: null,
      },
    });
  }

  render() {
    const {
      route,
      list,
      total,
      pageCurrent,
      pageSize,
      listLoading,
      contractNum,
      productNum,
      stdNum,
      amount,
      exportAuth,
    } = this.props;
    const { name, loading, contractPrdModal } = this.state;
    const prefix = ContractListMgmt.classPrefix;
    const maxExportNumber = loginUser.isTwoTeacherSym() ? MAX_EXPORT_NUMBER.twoTeacher : MAX_EXPORT_NUMBER.xinghuo;

    return (
      <Page
        name={name}
        className={style[prefix]}
        loading={loading}
        route={route}
        fitWindowHeight={true}
        subHeader={
          <div className={style[`${prefix}-header`]}>
            <div className={style[`${prefix}-search-wrap`]}>
              <Search className={style[`${prefix}-search`]} />
              <Popover placement="bottomRight" content={`1次最多可导出${maxExportNumber}条数据`}>
                <Button
                  className={style[`${prefix}-exportBtn`]}
                  type="primary"
                  onClick={this.exportExcel}
                  disabled={exportAuth}
                >
                  导出
                </Button>
              </Popover>
            </div>
            <InfoWrap>
              <Info label="合同个数" text={contractNum} />
              <Info label="产品个数" text={productNum} />
              <Info label="学生人数" text={stdNum} />
              <Info label="应收金额合计" text={amount} unit="元" />
            </InfoWrap>
          </div>
        }
      >
        <List
          dataSource={list}
          columns={this.columns}
          loading={!loading && listLoading}
          pagination={{
            total,
            current: pageCurrent,
            pageSize,
          }}
          onChange={this.changePage}
          locale={{ filterTitle: '' }}
        />
        {contractPrdModal && (
          <ContractPrdModal
            id={this.contractIdForModal}
            data={this.dataForModal}
            onCancel={this.toggleContractPrdModal}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    list: state[storeName].list,
    total: state[storeName].total,
    pageCurrent: state[storeName].pageCurrent,
    pageSize: state[storeName].pageSize,
    listLoading: state[storeName].listLoading,
    contractNum: state[storeName].contractNum,
    productNum: state[storeName].productNum,
    stdNum: state[storeName].stdNum,
    amount: state[storeName].amount,
    exportAuth: state.account.btnAuths.indexOf('BTN_EXPORT_CONTRACT_PRODUCT') === -1,
  };
};

export default connect(mapStateToProps)(ContractListMgmt);
