/**
 * Created by Haolin<haolinhom@gmail.com> on 2019/2/18.
 */
import { storeName } from './constants.js';
import { getContractInfoList, getContractStaticInfo } from 'SERVICE/contract';
import { exportContractListMgmtExcel } from 'SERVICE/exportFile';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import moment from 'moment';
import { clone } from 'lodash';

const _fn = {
  // 初始化筛选栏默认值
  initSKey(state) {
    if (state[storeName].defaultSKey) {
      return state[storeName].defaultSKey;
    } else {
      const { brenchId, campusId } = state.account.user;
      const isGroup = brenchId === campusId;
      let sKey_branch = isGroup ? [state.dict.branch[0].val] : [brenchId];
      let sKey_campus = isGroup
        ? state.dict.branchCampus.find((item) => item.val === sKey_branch[0]).options.map((item) => item.val)
        : [campusId];
      let today = new Date();
      return {
        sKey_beginDate: moment(today).startOf('month').format('YYYY-MM-DD'),
        sKey_endDate: moment(today).format('YYYY-MM-DD'),
        sKey_branch,
        sKey_campus,
      };
    }
  },
  // 构建列表请求参数
  buildLoadListParams(state) {
    return {
      page: state[storeName].pageCurrent,
      rows: state[storeName].pageSize,
      signStartDate: state[storeName].sKey_beginDate,
      signEndDate: state[storeName].sKey_endDate,
      productType: state[storeName].sKey_productType,
      productCharacter: state[storeName].sKey_productCharacter,
      productCourseSeries: state[storeName].sKey_productSeries,
      productGrade: state[storeName].sKey_productGrade,
      productVersion: state[storeName].sKey_productYear,
      productName: state[storeName].sKey_productName,
      contractProductCountType: state[storeName].sKey_productNumType
        ? Number(state[storeName].sKey_productNumType)
        : null,
      contractType: state[storeName].sKey_contractType,
      contractBranch: state[storeName].sKey_branch,
      contractCampus: state[storeName].sKey_campus,
      studentGrade: state[storeName].sKey_stdGrade,
      productSubject: state[storeName].sKey_subject,
      productQuarter: state[storeName].sKey_quarter,
      resEntrance: state[storeName].sKey_resourceEntrance,
      paidStatus: state[storeName].sKey_payStatus,
      contractId: state[storeName].sKey_contractId,
      studentName: state[storeName].sKey_stdName,
      signStaffName: state[storeName].sKey_signPerson,
      className: state[storeName].sKey_className,
      // contractCharacter: null,
      // productId: null,
    };
  },
  /**
   * 将数组内的对象按照对象的key值进行分组
   * @param {array} arr [required]
   * @param {string} keyName [required]
   * */
  sortByKey(arr, keyName) {
    let keyNameStorage = {};
    arr.forEach((item) => {
      if (keyNameStorage[item[keyName]]) {
        keyNameStorage[item[keyName]].push(item);
      } else {
        keyNameStorage[item[keyName]] = [item];
      }
    });
    return Object.values(keyNameStorage);
  },
  // 获某日期两个月前的1号
  twoMonthsAgo(date) {
    let y = date.getFullYear();
    let m = date.getMonth() + 1;
    if (m < 3) {
      y = y - 1;
      m = 10 + m;
    } else {
      m = m - 2;
    }
    if (m < 10) {
      m = `0${m}`;
    }
    return `${y}-${m}-01`;
  },
  // 构建合同性质字段(将新签/续费合并为常规)
  buildContractCharacter(id) {
    let ret;
    switch (id) {
      case 'YINLIU':
        ret = '引流';
        break;
      case 'NEWSIGN':
      case 'RESIGN':
        ret = '常规';
        break;
      default:
        ret = '无';
    }
    return ret;
  },
};

export default {
  namespace: storeName,
  state: {
    // 筛选项默认值
    defaultSKey: null,
    // 筛选条件汇总
    sKeys: null,
    // 时间范围
    sKey_beginDate: null,
    sKey_endDate: null,
    // 产品类型
    sKey_productType: null,
    // 产品性质(引流、常规)
    sKey_productCharacter: null,
    // 产品系列
    sKey_productSeries: null,
    // 产品年级
    sKey_productGrade: null,
    // 产品年份
    sKey_productYear: null,
    // 产品名称
    sKey_productName: '',
    // 合同产品数
    sKey_productNumType: null,
    // 合同类型
    sKey_contractType: null,
    // 合同分公司
    sKey_branch: null,
    // 合同学习中心
    sKey_campus: null,
    // 学生年级
    sKey_stdGrade: null,
    // 科目
    sKey_subject: null,
    // 季度
    sKey_quarter: null,
    // 资源入口
    sKey_resourceEntrance: null,
    // 支付状态
    sKey_payStatus: null,
    // 合同编号
    sKey_contractId: '',
    // 学生姓名
    sKey_stdName: '',
    // 签单人姓名
    sKey_signPerson: '',
    // 班级名称
    sKey_className: '',
    // 列表数据
    list: [],
    total: 0,
    pageCurrent: 1,
    pageSize: 20,
    listLoading: false,
    // 统计数据
    contractNum: null,
    productNum: null,
    stdNum: null,
    amount: null,
  },
  reducers: {
    // 更新state
    updateState(state, { payload }) {
      return {
        ...state,
        ...payload,
      };
    },
    // 更新列表
    updateListData(state, { payload: { list, total, info } }) {
      // 转换后端返回的列表数据
      let contractList = _fn.sortByKey(list, 'contractId').map((items) => {
        let _data = {
          productName: [],
          contractId: null,
          studentName: null,
          schoolName: null,
          studentId: null,
          totalPay: [],
          paidAmount: [],
          payStatus: [],
          contractType: null,
          contractCharacter: null,
          signDate: null,
          resourceEntrance: null,
          signPerson: null,
          campus: null,
        };
        items.forEach((item, index) => {
          _data.productName.push(item.productName);
          _data.totalPay.push(item.realAmount);
          _data.paidAmount.push(item.paidAmount);
          _data.payStatus.push(item.paidStatusName);
          if (index === 0) {
            _data.contractId = item.contractId;
            _data.studentId = item.studentId;
            _data.studentName = item.studentName;
            _data.schoolName = item.schoolName;
            _data.contractType = item.contractTypeName;
            _data.contractCharacter = _fn.buildContractCharacter(item.contractCharacter);
            _data.signDate = item.signDate;
            _data.resourceEntrance = item.resEntranceName;
            _data.signPerson = item.signStaffName;
            _data.campus = item.contractCampusName;
          }
        });
        return _data;
      });
      let payload = {
        ...state,
        list: contractList,
      };
      if (typeof total === 'number') payload.total = total;
      if (info === 'noData') {
        payload.contractNum = null;
        payload.productNum = null;
        payload.stdNum = null;
        payload.amount = null;
      } else if (info) {
        payload.contractNum = info.contractCount ? info.contractCount.toString() : '0';
        payload.productNum = info.contractProductCount ? info.contractProductCount.toString() : '0';
        payload.stdNum = info.studentCount ? info.studentCount.toString() : '0';
        payload.amount = info.totalRealAmount ? info.totalRealAmount.toString() : '0';
      }
      return payload;
    },
  },
  effects: {
    // 初始化页面数据
    *initMain({ payload }, { select, put }) {
      // 初始化筛选栏
      const initData = yield select(_fn.initSKey);
      yield put({
        type: 'updateState',
        payload: {
          ...initData,
          defaultSKey: initData,
        },
      });
      // 加载数据
      // yield put({
      //   type: 'loadList',
      //   payload,
      // });
      if (payload.callback) payload.callback();
    },
    // 加载列表数据
    *loadList({ payload: { callback, handleErr } }, { select, put }) {
      yield put({
        type: 'updateState',
        payload: { listLoading: true },
      });
      const apiParams = yield select(_fn.buildLoadListParams);
      let fetchList = [];
      // 获取数据
      let apiParamsForList = clone(apiParams);
      apiParamsForList.getTotal = false;
      fetchList.push(
        getContractInfoList(apiParamsForList, true).catch((err) => {
          showMessage(err.text, 'error');
        }),
      );
      if (apiParams.page === 1) {
        // 获取总条数
        let apiParamsForTotal = clone(apiParams);
        apiParamsForTotal.getTotal = true;
        fetchList.push(
          getContractInfoList(apiParamsForTotal, true).catch((err) => {
            showMessage(err.text, 'error');
          }),
        );
        fetchList.push(
          getContractStaticInfo(apiParams, true).catch((err) => {
            showMessage(err.text, 'error');
          }),
        );
      }
      const [rspList, rspTotal, rspInfo] = yield Promise.all(fetchList).catch((err) => {
        showMessage(err.text, 'error');
        if (handleErr) handleErr();
      });
      if (rspList && rspList.resultCode === 0) {
        let payload = {
          list: rspList.rows,
          total: null,
          info: null,
        };
        if (apiParams.page === 1) {
          if (rspTotal) payload.total = rspTotal.records;
          payload.info = rspInfo ? rspInfo.data : 'noData';
        }
        yield put({
          type: 'updateListData',
          payload,
        });
      }
      yield put({
        type: 'updateState',
        payload: {
          listLoading: false,
          sKeys: apiParams,
        },
      });
      if (callback) callback();
    },
    // 重置筛选栏
    *resetSKey(action, { select, put }) {
      const defaultSKey = yield select((state) => state[storeName].defaultSKey);
      yield put({
        type: 'updateState',
        payload: {
          sKey_beginDate: null,
          sKey_endDate: null,
          sKey_productType: null,
          sKey_productCharacter: null,
          sKey_productSeries: null,
          sKey_productGrade: null,
          sKey_productYear: null,
          sKey_productName: '',
          sKey_productNumType: null,
          sKey_contractType: null,
          sKey_campus: null,
          sKey_stdGrade: null,
          sKey_subject: null,
          sKey_quarter: null,
          sKey_resourceEntrance: null,
          sKey_payStatus: [],
          sKey_contractId: '',
          sKey_stdName: '',
          sKey_signPerson: '',
          sKey_className: '',
          ...defaultSKey,
        },
      });
    },
    // 导出excel(添加导出任务至导出中心)
    *exportExcel({ payload: { callback } }, { select, put }) {
      const apiParams = yield select(_fn.buildLoadListParams);
      delete apiParams.page;
      delete apiParams.rows;
      const rsp = yield exportContractListMgmtExcel(apiParams, true).catch((err) => {
        showMessage(err.text, 'error');
      });
      if (rsp && rsp.resultCode === 0) {
        console.log('export success!');
        // 通知(导出)消息中心
        yield put({ type: 'main/exportNotice' });
      }
      if (callback) callback();
    },
  },
};
