import React from 'react';
import { connect } from 'dva';
import { Page, SearchFilterForm, SearchFieldForm, Select, Button, List } from 'COMPONENT';
import { DatePicker } from 'antd';
import PerformanceOwnerSelect from './BLoComponents/PerformanceOwnerSelect';
import moment from 'moment';
import { USER_USER } from './BLoComponents/PerformanceOwnerSelect/config';
import { pageRefundIncomeRecords, exportPageRefundIncomeRecords } from 'SERVICE/contract';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { errorsHandle } from 'UTIL/common';
import file from 'UTIL/file';

const dateFormat = 'YYYY-MM-DD HH:mm:ss';
const dayFormat = 'YYYY-MM-DD';
const setWidth = List.createSetWidth();
const errorsHandler = errorsHandle();

class DeductPerformance extends React.Component {
  constructor(props) {
    super(props);

    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.initData = this.initData.bind(this);
    this.fetchList = this.fetchList.bind(this);
    this.loadList = this.loadList.bind(this);
    this.onSearch = this.onSearch.bind(this);
    this.onPageChange = this.onPageChange.bind(this);
    this.onFieldsChange = this.onFieldsChange.bind(this);
    this.onExport = this.onExport.bind(this);
    this.getDisabledDate = this.getDisabledDate.bind(this);

    this.columns = [
      { title: '分公司', dataIndex: 'branchName', width: setWidth(5) },
      { title: '学习中心', dataIndex: 'campusName', width: setWidth(5) },
      { title: '退费ID', dataIndex: 'studentReturnFeeId', width: setWidth(4) },
      { title: '退费合同', dataIndex: 'contractId', width: setWidth(5) },
      { title: '退费产品', dataIndex: 'productName', width: setWidth(5) },
      { title: '个性化退费金额', dataIndex: 'refundAmount', width: setWidth(7) },
      {
        title: '退费日期',
        dataIndex: 'refundTime',
        width: setWidth(6),
        render: (text) =>
          moment(text)
            .format(dateFormat)
            .split(' ')
            .map((item, index) => <p key={index}>{item}</p>),
      },
      { title: '收款ID', dataIndex: 'fundsId', width: setWidth(4) },
      {
        title: '业绩归属',
        dataIndex: 'subDistributeType',
        width: setWidth(5),
        render: (text, record) => (text === USER_USER ? record.bonusStaffName : record.bonusOrgName),
      },
      { title: '收款金额', dataIndex: 'fundsAmount', width: setWidth(4) },
      {
        title: '收款日期',
        dataIndex: 'fundsTime',
        width: setWidth(6),
        render: (text) =>
          moment(text)
            .format(dateFormat)
            .split(' ')
            .map((item, index) => <p key={index}>{item}</p>),
      },
      { title: '分配业绩金额', dataIndex: 'incomeAmount', width: setWidth(6) },
      { title: '退费扣减业绩金额', dataIndex: 'refundIncomeAmount', width: setWidth(8) },
      { title: '收款月份员工获得的总业绩', dataIndex: 'staffMonthAmount', width: setWidth(12) },
    ];
    this.searchRef = null;

    this.state = {
      loading: false,
      searchData: {},
      pageNum: 1,
      pageSize: 20,
      list: [],
      total: 0,
      personDict: [],
      branchStorage: null,
      rangeInitial: null,
      rangeDisabledDate: null,
    };
  }

  componentDidMount() {
    this.init();
  }

  async init() {
    const { state } = this;
    this.setState({ loading: true });
    await new Promise(this.initDict);
    const sKeys = await new Promise(this.initData);
    await this.fetchList({
      ...sKeys,
      pageNum: state.pageNum,
      pageSize: state.pageSize,
    });
    this.setState({ loading: false });
  }

  initDict(resolve) {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['branchCampusTree'],
        callback: resolve,
      },
    });
  }

  initData(resolve) {
    const beginDate = moment(`${moment(this.props.onlineTime).subtract(29, 'day').format(dayFormat)} 00:00:00`);
    const endDate = moment(`${moment(this.props.onlineTime).format(dayFormat)} 23:59:59`);
    const beforeNinetyDay = moment(`${moment(this.props.onlineTime).subtract(89, 'day').format(dayFormat)} 00:00:00`);
    const searchData = {
      refundStartTime: beginDate.format('x'),
      refundEndTime: endDate.format('x'),
    };
    this.setState(
      {
        rangeInitial: [beginDate, endDate],
        rangeDisabledDate: [+beforeNinetyDay.format('x'), +endDate.format('x')],
        searchData,
      },
      () => {
        resolve(searchData);
      },
    );
  }

  async fetchList(apiParams) {
    const res = await pageRefundIncomeRecords(apiParams).catch(errorsHandler);
    if (res && res.data) {
      this.setState({
        list: res.data.list,
        total: res.data.total,
      });
    }
  }

  async loadList(apiParams) {
    this.setState({ loading: true });
    await this.fetchList(apiParams);
    this.setState({ loading: false });
  }

  onSearch(data) {
    let apiParams;
    const searchData = this.searchDataAdapter(data);
    this.setState(
      (state) => {
        apiParams = this.listParamsAdapter(searchData, 1, state.pageSize);
        return {
          searchData,
          pageNum: 1,
        };
      },
      () => {
        this.loadList(apiParams);
      },
    );
  }

  onPageChange({ current: pageNum, pageSize }) {
    let apiParams;
    this.setState(
      (state) => {
        pageNum = state.pageSize === pageSize ? pageNum : 1;
        apiParams = this.listParamsAdapter(state.searchData, pageNum, pageSize);
        return {
          pageNum,
          pageSize,
        };
      },
      () => {
        this.loadList(apiParams);
      },
    );
  }

  onFieldsChange(fields) {
    if (fields.hasOwnProperty('branchId')) {
      let clearCampus = false;
      this.setState(
        (state) => {
          clearCampus = state.branchStorage && fields.branchId.value;

          return {
            branchStorage: fields.branchId.value,
          };
        },
        () => {
          if (clearCampus) {
            this.searchRef.setFieldsValue({ campusId: null });
          }
        },
      );
    }
  }

  async onExport() {
    if (this.state.total > 5000) {
      return showMessage('当前查询数据已超过5000条，无法导出，请缩小查询范围并重新点击查询后再导出', 'error');
    }
    this.setState({ loading: true });
    const res = await exportPageRefundIncomeRecords({
      ...this.state.searchData,
      pageSize: this.state.total,
    }).catch(errorsHandler);
    if (res && res.data) {
      file.downloadDirect(res.data, `退费扣款明细_${Date.now()}`);
    }
    this.setState({ loading: false });
  }

  getDisabledDate(current) {
    if (current) {
      current = +current.format('x');
      return this.state.rangeDisabledDate[0] > current || current > this.state.rangeDisabledDate[1];
    }
    return true;
  }

  searchDataAdapter = (searchData) => {
    let data = {
      ...searchData,
      bonusStaffId: null,
      bonusOrgId: null,
      refundStartTime: null,
      refundEndTime: null,
    };
    delete data.bonus;
    delete data.range;
    if (Array.isArray(searchData.range) && searchData.range.length > 0) {
      data.refundStartTime = searchData.range[0].format('x');
      data.refundEndTime = searchData.range[1].format('x');
    }
    if (Array.isArray(searchData.bonus) && searchData.bonus.length) {
      if (searchData.bonus[0] === USER_USER) {
        data.bonusStaffId = searchData.bonus[1];
      } else {
        data.bonusOrgId = searchData.bonus[1];
      }
    }
    return data;
  };

  listParamsAdapter = (searchData, pageNum, pageSize) => {
    return {
      ...searchData,
      pageNum,
      pageSize,
    };
  };

  getCampusDict = (branch, tree) => {
    if (!tree) {
      return [];
    }

    if (branch) {
      const tag = tree.find((item) => item.val === branch);
      if (tag) {
        return tag.campus;
      }
    } else {
      return tree.map((item) => ({
        label: item.text,
        options: item.campus,
        val: item.val,
      }));
    }

    return [];
  };

  render() {
    const { state, props } = this;
    const { branchCampusTree, brenchId, isSuperAdmin } = props;
    // 非超管只显示主组织
    const usableCampusList =
      !isSuperAdmin && brenchId ? branchCampusTree.filter((b) => b.val === brenchId) : branchCampusTree;

    return (
      <Page
        route={props.route}
        loading={state.loading}
        headerToolBar={
          props.exportAuth && (
            <Button onClick={this.onExport} type="primary" disabled={this.state.total === 0}>
              导出
            </Button>
          )
        }
        subHeader={
          <SearchFilterForm
            ref={(c) => (this.searchRef = c)}
            truncationNum={5}
            onSearch={this.onSearch}
            onFieldsChange={this.onFieldsChange}
          >
            <SearchFieldForm name="branchId">
              <Select placeholder="分公司" dict={usableCampusList} showSearch />
            </SearchFieldForm>
            <SearchFieldForm name="campusId">
              <Select
                placeholder="学习中心"
                dict={this.getCampusDict(state.branchStorage, usableCampusList)}
                showSearch
              />
            </SearchFieldForm>
            <SearchFieldForm name="bonus" span={2}>
              <PerformanceOwnerSelect />
            </SearchFieldForm>
            <SearchFieldForm name="range" span={2} initialValue={state.rangeInitial} required>
              <DatePicker.RangePicker
                placeholder={['退费日期(起)', '退费日期(止)']}
                format={dayFormat}
                disabledDate={this.getDisabledDate}
                allowClear={false}
                style={{ width: '100%' }}
              />
            </SearchFieldForm>
          </SearchFilterForm>
        }
        fitWindowHeight
      >
        <List
          columns={this.columns}
          dataSource={state.list}
          pagination={{
            total: state.total,
            current: state.pageNum,
            pageSize: state.pageSize,
          }}
          onChange={this.onPageChange}
        />
      </Page>
    );
  }
}

const mapStoreToProps = (store) => {
  const { user } = store.account;
  const { roleCode, onlineTime, brenchId } = user;
  const isSuperAdmin = roleCode && roleCode.indexOf('SUPER_ADMIN') >= 0;

  return {
    branchCampusTree: store.dict.branchCampusTree || [],
    exportAuth: store.account.btnAuths.indexOf('DEDUCT_PERFORMANCE_EXPORT') > -1,
    onlineTime,
    brenchId,
    isSuperAdmin,
  };
};

export default connect(mapStoreToProps)(DeductPerformance);
