import React from 'react';
import styles from './index.less';
import PromotionTypeText from './PromotionTypeText';
import PromotionAllText from './PromotionAllText';
import { splitStartClassDate } from '../../util';

/**
 * 产品信息
 *
 * @param props.data {array} 产品列表
 * @return {*}
 * @constructor
 */
const ProductInfo = (props) => {
  const data = props.data || [];

  return (
    <table className={styles.infoTable}>
      <thead>
        <tr>
          <td>产品类型</td>
          <td>产品名称</td>
          <td>班级</td>
          <td>上课时间</td>
          <td>老师</td>
          <td>小时单价</td>
          <td>小时数</td>
          <td>产品总额</td>
          <td>产品优惠</td>
          <td>优惠券</td>
          <td>优惠活动</td>
          <td>优惠金额</td>
          <td>应收金额</td>
        </tr>
      </thead>
      <tbody>
        {data.map((item, index) => {
          const { promotionInfoVos = [], promotionActivityVos = [], promotionCouponVos = [] } = item;

          return (
            <tr key={index}>
              <td>{item.productTypeName}</td>
              <td>{item.productName}</td>
              <td>{item.className || '-'}</td>
              <td>{splitStartClassDate(item.startClassDate)}</td>
              <td>{item.mainTeacherName || '-'}</td>
              <td>{item.naturePrice}</td>
              <td>{item.totalCourseNatureHours}</td>
              <td>{item.totalAmount}</td>
              <td>
                <PromotionTypeText promotionList={promotionInfoVos} />
              </td>
              <td>
                <PromotionAllText promotionList={promotionCouponVos} />
              </td>
              <td>
                <PromotionTypeText promotionList={promotionActivityVos} />
              </td>
              <td>{item.promotionAmount}</td>
              <td>{item.realAmount}</td>
            </tr>
          );
        })}
      </tbody>
    </table>
  );
};

export default ProductInfo;
