import React from 'react';
import styles from './index.less';
import SelectedCouponList from '../SelectedCouponList';
import { formatRedStyle } from '../../util';

/**
 * 优惠信息
 *
 * @param props.data {Object} 合同详情
 * @param props.promotionList {Array} 优惠列表
 * @param props.differentKeys {string} 值不相同需要标红的字段
 * @return {*}
 * @constructor
 */
const PromotionInfo = (props) => {
  const { data, promotionList, differentKeys } = props;
  const hasPromotion = promotionList && promotionList.length > 0;

  /**
   * 标红样式
   *
   * @param key
   * @return {{}|{color: string}}
   */
  const formatStyle = (key) => {
    return formatRedStyle(key, differentKeys);
  };

  return (
    <div className={styles.wrap}>
      <div className={styles.left}>
        <span className={styles.title}>已使用优惠券：</span>
        <span className={styles.promotionList}>
          {hasPromotion ? <SelectedCouponList truncationNum={1} promotionList={promotionList} /> : '暂无'}
        </span>
      </div>
      <div className={styles.right}>
        <span className={styles.mr_15}>
          <span className={styles.title}>合同款总计：</span>
          <span style={formatStyle('totalAmount')}>{data.totalAmount}</span>
        </span>
        <span className={styles.mr_15}>
          <span className={styles.title}>优惠总计：</span>
          <span style={formatStyle('promotionAmount')}>{data.promotionAmount}</span>
        </span>
        <span className={styles.mr_15}>
          <span className={styles.title}>合同应收款总计：</span>
          <span style={formatStyle('realAmount')}>{data.realAmount}</span>
        </span>
        <span>
          <span className={styles.title}>合同已收款总计：</span>
          <span style={formatStyle('paidAmount')}>{data.paidAmount}</span>
        </span>
      </div>
    </div>
  );
};

export default PromotionInfo;
