import React from 'react';
import { formatContractTypeName, formatRedStyle, formatPhoneNumber } from '../../util';
import styles from './index.less';

/**
 * 学员信息
 *
 * @param props.data {object} 学生数据
 * @param props.differentKeys {string} 值不相同需要标红的字段
 * @return {*}
 * @constructor
 */
const StudentInfo = (props) => {
  const { data, differentKeys } = props;

  /**
   * 标红样式
   *
   * @param key
   * @return {{}|{color: string}}
   */
  const formatStyle = (key) => {
    return formatRedStyle(key, differentKeys);
  };

  return (
    <table className={styles.infoTable}>
      <thead>
        <tr>
          <td>学生姓名</td>
          <td>客户姓名</td>
          <td>客户手机号码</td>
          <td>学生年级</td>
          <td>合同学习中心</td>
          <td>学校</td>
          <td>合同类型</td>
          <td>操作人</td>
        </tr>
      </thead>
      <tbody>
        <tr>
          <td style={formatStyle('studentName')}>{data.studentName}</td>
          <td style={formatStyle('customerName')}>{data.customerName}</td>
          <td style={formatStyle('customerContact')}>{formatPhoneNumber(data.customerContact)}</td>
          <td style={formatStyle('studentGradeName')}>{data.studentGradeName}</td>
          <td style={formatStyle('blCampusName')}>{data.blCampusName}</td>
          <td style={formatStyle('schoolName')}>{data.schoolName}</td>
          <td style={formatStyle('contractTypeValue')}>{formatContractTypeName(data.contractTypeValue)}</td>
          <td style={formatStyle('operateUserName')}>{data.operateUserName}</td>
        </tr>
      </tbody>
    </table>
  );
};

export default StudentInfo;
