import React from 'react';
import { Page, Block, NoData } from 'COMPONENT';
import styles from './index.less';
import * as loader from './loader';
import * as util from './util';
import StudentInfo from './component/StudentInfo';
import ProductInfo from './component/ProductInfo';
import PromotionInfo from './component/PromotionInfo';

/**
 * 合同修改详情
 *
 * @param params.prevId {string} 修改前快照ID
 * @param params.currentId {string} 修改后快照ID
 */
class EditContractDetail extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      preData: null, // 修改前数据
      nextData: null, // 修改后数据
      differentKeys: null, // 值不相同需要标红的字段
    };
  }

  componentDidMount() {
    console.log('#:', this.props);
    this.initContent();
  }

  /**
   * 初始化快照数据
   *
   * @return {Promise<void>}
   */
  initContent = async () => {
    this.toggleLoading();

    // TODO: 支持只传一个id并且只渲染一个，不传时页面直接报错
    const { currentId: nextSnapshotId, prevId: preSnapshotId } = this.props.routeParams;

    const [preData, nextData] = await Promise.all([
      loader.getContractSnapshot({ id: preSnapshotId }),
      loader.getContractSnapshot({ id: nextSnapshotId }),
    ]).catch(() => this.toggleLoading());
    const differentKeys = util.collectDifferentKey(preData, nextData);

    this.toggleLoading();
    this.setState({ preData, nextData, differentKeys });
  };

  toggleLoading = () => {
    this.setState((prevState) => {
      return {
        loading: !prevState.loading,
      };
    });
  };

  render() {
    const { preData, nextData, differentKeys } = this.state;

    return (
      <Page title={false} route={this.props.route} loading={this.state.loading} className={styles.pageWrap}>
        <Block
          title={
            <span>
              <span style={{ color: 'red' }}>修改前</span>
              <span>-合同详情</span>
            </span>
          }
        >
          {preData ? (
            <section>
              <h5 className={styles.title}>学员信息</h5>
              <StudentInfo data={preData} differentKeys={differentKeys} />

              <h5 className={styles.title + ' ' + styles.mt_15}>报读课程产品</h5>
              <ProductInfo data={preData.contractProductVos} />

              <PromotionInfo data={preData} promotionList={preData.promotionCouponVos} differentKeys={differentKeys} />
            </section>
          ) : (
            <NoData />
          )}
        </Block>
        <Block
          title={
            <span>
              <span style={{ color: 'red' }}>修改后</span>
              <span>-合同详情</span>
            </span>
          }
        >
          {nextData ? (
            <section>
              <h5 className={styles.title}>学员信息</h5>
              <StudentInfo data={nextData} differentKeys={differentKeys} />

              <h5 className={styles.title + ' ' + styles.mt_15}>报读课程产品</h5>
              <ProductInfo data={nextData.contractProductVos} />

              <PromotionInfo
                data={nextData}
                promotionList={nextData.promotionCouponVos}
                differentKeys={differentKeys}
              />
            </section>
          ) : (
            <NoData />
          )}
        </Block>
      </Page>
    );
  }
}

export default EditContractDetail;
