import React from 'react';
import { connect } from 'dva';
import { storeName } from './store/config';
import Page from 'COMPONENT/Page';
import Button from 'COMPONENT/Button';
import Search from './Search';
import List from 'COMPONENT/List';

const setWidth = List.createSetWidth();

class ComponentName extends React.Component {
  constructor(props) {
    super(props);
    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.initDefault = this.initDefault.bind(this);
    this.loadData = this.loadData.bind(this);
    this.updatePage = this.updatePage.bind(this);
    this.changePage = this.changePage.bind(this);
    this.search = this.search.bind(this);
    this.reset = this.reset.bind(this);
    this.exportExcel = this.exportExcel.bind(this);
    this.beforeDestroy = this.beforeDestroy.bind(this);
    this.state = {
      name: '流量营销流水',
      loading: false,
    };
    this.columns = [
      { title: '交易日期', dataIndex: 'paymentDateText', key: 'paymentDateText', width: setWidth(5.5) },
      { title: '学生姓名', dataIndex: 'studentName', key: 'studentName', width: setWidth(4) },
      { title: '学生ID', dataIndex: 'studentId', key: 'studentId', width: setWidth(4) },
      { title: '分公司', dataIndex: 'orderBranchName', key: 'orderBranchName', width: setWidth(4) },
      { title: '学习中心', dataIndex: 'orderCampusName', key: 'orderCampusName', width: setWidth(4) },
      { title: '类型', dataIndex: 'financeType', key: 'financeType', width: setWidth(4) },
      { title: '支付方式', dataIndex: 'channel', key: 'channel', width: setWidth(4) },
      { title: '支付单号', dataIndex: 'orderNoInChannel', key: 'orderNoInChannel', width: setWidth(6.5) },
      { title: '总额', dataIndex: 'totalAmount', key: 'totalAmount', width: setWidth(4) },
      { title: '总额分成', dataIndex: 'incomeAmount', key: 'incomeAmount', width: setWidth(4) },
      { title: '订单号', dataIndex: 'orderNum', key: 'orderNum', width: setWidth(6) },
      { title: '活动类型', dataIndex: 'activityType', key: 'activityType', width: setWidth(4) },
      { title: '活动名称', dataIndex: 'activityName', key: 'activityName', width: setWidth(4) },
    ];
  }

  componentDidMount() {
    this.init();
  }

  componentWillUnmount() {
    this.beforeDestroy();
  }

  async init() {
    this.setState({ loading: true });
    await new Promise(this.initDict);
    await new Promise(this.initDefault);
    await new Promise(this.loadData);
    this.setState({ loading: false });
  }

  initDict(resolve) {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['allBranchCampusTree', 'insightFinanceType', 'insightActivityType'],
        callback() {
          resolve();
        },
      },
    });
  }

  initDefault(resolve) {
    this.props.dispatch({
      type: `${storeName}/initSKey`,
      payload: {
        callback() {
          resolve();
        },
      },
    });
  }

  loadData(resolve) {
    this.props.dispatch({
      type: `${storeName}/loadData`,
      payload: {
        callback() {
          resolve();
        },
      },
    });
  }

  updatePage(pageCurrent, pageSize) {
    let payload = {};
    if (pageSize && pageSize !== this.props.pageSize) {
      payload.pageCurrent = 1;
      payload.pageSize = pageSize;
    } else {
      payload.pageCurrent = pageCurrent;
    }
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload,
    });
  }

  async changePage({ current, pageSize }) {
    this.updatePage(current, pageSize);
    this.setState({ loading: true });
    await new Promise(this.loadData);
    this.setState({ loading: false });
  }

  search() {
    this.changePage({
      current: 1,
      pageSize: this.props.pageSize,
    });
  }

  reset() {
    this.props.dispatch({
      type: `${storeName}/resetSKey`,
    });
  }

  exportExcel() {
    this.setState({ loading: true });
    const _self = this;
    this.props.dispatch({
      type: `${storeName}/exportExcel`,
      payload: {
        callback() {
          _self.setState({ loading: false });
        },
      },
    });
  }

  beforeDestroy() {
    this.props.dispatch({
      type: `${storeName}/resetPage`,
    });
  }

  render() {
    return (
      <Page
        name={this.state.name}
        loading={this.state.loading}
        route={this.props.route}
        fitWindowHeight
        headerToolBar={
          this.props.exportBtnAuth && (
            <Button onClick={this.exportExcel} type="default">
              导出excel
            </Button>
          )
        }
        subHeader={<Search onSearch={this.search} onReset={this.reset} loading={this.state.loading} />}
      >
        <List
          columns={this.columns}
          dataSource={this.props.list}
          pagination={{
            total: this.props.total,
            current: this.props.pageCurrent,
            pageSize: this.props.pageSize,
          }}
          onChange={this.changePage}
        />
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    exportBtnAuth: state.account.btnAuths.includes('BTN_EXPORT_FLOW_MARKETING'),
    list: state[storeName].list,
    total: state[storeName].total,
    pageSize: state[storeName].pageSize,
    pageCurrent: state[storeName].pageCurrent,
  };
};

export default connect(mapStateToProps)(ComponentName);
