import { storeName, dateFormat, WEB_CHART_PAY } from './config';
import { runFunc, errorsHandle, getFetchList } from 'UTIL/common';
import { clone } from 'lodash';
import moment from 'moment';
import { getPaymentRecordList, exportInsightPaymentRecordListExcel } from 'SERVICE/contract';

const defaultState = {
  sKey: {
    sKey_momentRange: [null, null],
    sKey_dateRange: [null, null],
    sKey_studentName: null,
    sKey_orderNoInChannel: null,
    sKey_branch: null,
    sKey_campus: null,
    sKey_financeType: null,
    sKey_channel: null,
    sKey_activityType: null,
  },
  list: {
    list: [],
    total: 0,
    pageSize: 10,
    pageCurrent: 1,
  },
};
Object.freeze(defaultState);

// for loadData/exportExcel
function apiParamsAdapter(state) {
  return {
    startTime: state.sKey_dateRange[0],
    endTime: state.sKey_dateRange[1],
    studentName: state.sKey_studentName,
    orderNoInChannel: state.sKey_orderNoInChannel,
    orderBranchId: state.sKey_branch,
    orderCampusId: state.sKey_campus,
    financeType: state.sKey_financeType,
    channel: state.sKey_channel,
    activityType: state.sKey_activityType,
  };
}

export default {
  namespace: storeName,
  state: {
    ...clone(defaultState.sKey),
    ...clone(defaultState.list),
    // 筛选条件初始化缓存
    sKeyStorage: null,
  },
  reducers: {
    updateState(state, { payload }) {
      return {
        ...state,
        ...payload,
      };
    },
  },
  effects: {
    *initSKey({ payload: { callback } }, { select, put }) {
      let initData = yield select((state) => {
        if (state[storeName].sKeyStorage) {
          return state[storeName].sKeyStorage;
        } else {
          const { serverDate } = state.account.user;
          let today = new Date(serverDate);
          let startMoment = moment(today).startOf('month');
          let endMoment = moment(today);
          return {
            sKey_momentRange: [startMoment, endMoment],
            sKey_dateRange: [startMoment.format(dateFormat), endMoment.format(dateFormat)],
            sKey_channel: WEB_CHART_PAY,
          };
        }
      });
      yield put({
        type: 'updateState',
        payload: {
          ...initData,
          sKeyStorage: initData,
        },
      });
      runFunc(callback);
    },
    *loadData({ payload: { callback } }, { select, put }) {
      let apiParams = yield select((state) => apiParamsAdapter(state[storeName]));
      apiParams = yield select((state) => ({
        ...apiParams,
        page: state[storeName].pageCurrent,
        rows: state[storeName].pageSize,
      }));

      let fetchList = getFetchList(getPaymentRecordList, apiParams, 'page');
      let ret = yield Promise.all(fetchList).catch(errorsHandle());

      if (ret) {
        const [rspList, rspTotal] = ret;
        let payload = {};
        if (rspList) {
          payload.list = rspList.rows.map((item) => ({
            ...item,
            paymentDateText: moment(item.paymentDate).format(dateFormat),
          }));
        }
        if (rspTotal) {
          payload.total = rspTotal.records;
        }
        yield put({
          type: 'updateState',
          payload,
        });
      }

      runFunc(callback);
    },
    *resetSKey(action, { select, put }) {
      const sKeyStorage = yield select((state) => state[storeName].sKeyStorage);
      yield put({
        type: 'updateState',
        payload: {
          ...clone(defaultState.sKey),
          ...sKeyStorage,
        },
      });
    },
    *resetPage(action, { put }) {
      yield put({
        type: 'updateState',
        payload: {
          ...clone(defaultState.sKey),
          ...clone(defaultState.list),
        },
      });
    },
    *exportExcel({ payload: { callback } }, { select, put }) {
      let apiParams = yield select((state) => apiParamsAdapter(state[storeName]));
      const rsp = yield exportInsightPaymentRecordListExcel(apiParams, true).catch(errorsHandle());
      if (rsp && rsp.resultCode === 0) {
        yield put({
          type: 'main/exportNotice',
        });
      }
      runFunc(callback);
    },
  },
};
