import React from 'react';
import styles from './styles.module.less';
import PerformanceHeader from '../common/performanceHeader';
import SchoolAllocation from '../common/schoolAllocation';
import CommissionAllocation from '../common/commissionAllocation';
import AllocationDynamic from '../common/allocationDynamic';
import { Tabs, Collapse } from 'antd';
import PropTypes from 'prop-types';
import { getIncomeDistributionByFundsChangeIdApi } from '../loaders';
import showAlert from 'COMPONENT/utils/globalTips/showAlert';

const TabPane = Tabs.TabPane;
const Panel = Collapse.Panel;

// 业绩分配
class PerformanceAllocation extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      allocationData: {},
      initSchoolData: [],
      initCommissionData: [],
      schoolAllocationData: [],
      commissionAllocationData: [],
    };
  }

  static propTypes = {
    fundsChangeId: PropTypes.string,
    bonusType: PropTypes.string,
    modalType: PropTypes.string,
    onRef: PropTypes.func,
  };

  initContent = async () => {
    let data = await getIncomeDistributionByFundsChangeIdApi({
      fundsChangeId: this.props.fundsChangeId,
      bonusType: this.props.bonusType,
    });
    let tempSchoolAllocationData = [];
    let tempCommissionAllocationData = [];
    data.incomeDistributionVos.forEach((value) => {
      if (value.baseBonusType === 'CAMPUS') {
        tempSchoolAllocationData.push(value);
      } else {
        tempCommissionAllocationData.push(value);
      }
    });
    this.props.onChange(tempSchoolAllocationData.concat(tempCommissionAllocationData));
    this.setState({
      allocationData: {
        fundsChangeId: data.fundsChangeId,
        contractId: data.contractId,
        studentId: data.studentId,
        studentName: data.studentName,
        contractBlCampusId: data.contractBlCampusId,
        contractBlCampusName: data.contractBlCampusName,
        type: data.type,
        typeName: data.typeName,
        transactionAmount: data.transactionAmount,
      },
      initSchoolData: tempSchoolAllocationData,
      initCommissionData: tempCommissionAllocationData,
      schoolAllocationData: tempSchoolAllocationData,
      commissionAllocationData: tempCommissionAllocationData,
    });
  };

  // 各金额总额小于等于调整金额
  onChangeSchoolValue = (val) => {
    this.setState({
      schoolAllocationData: val,
    });
    let tempArr = val.concat(this.state.commissionAllocationData);
    this.props.onChange(tempArr);
  };

  // 各金额总额小于等于调整金额
  onChangeCommissionValue = (val) => {
    this.setState({
      commissionAllocationData: val,
    });
    let tempArr = val.concat(this.state.schoolAllocationData);
    this.props.onChange(tempArr);
  };

  // 提交前的校验
  checkValid = () => {
    const { schoolAllocationData, commissionAllocationData, allocationData } = this.state;
    let status = true;
    if (schoolAllocationData.length) {
      let tempMoneyAll = 0;
      for (let a of schoolAllocationData) {
        if (!a.baseBonusType || !a.subBonusType || !a.distributeBizId) {
          showAlert('请全部填完内容');
          return false;
        } else if (a.amount <= 0) {
          showAlert('填写的金额一定要正数');
          return false;
        }
        tempMoneyAll += Number(a.amount);
      }
      if (tempMoneyAll > allocationData.transactionAmount) {
        showAlert('总金额不能大于调整金额');
        return false;
      }
    }
    if (commissionAllocationData.length) {
      let tempMoneyAll = 0;
      for (let a of commissionAllocationData) {
        if (!a.baseBonusType || !a.subBonusType || !a.distributeBizId) {
          showAlert('请全部填完内容');
          return false;
        } else if (a.amount <= 0) {
          showAlert('填写的金额一定要正数');
          return false;
        }
        tempMoneyAll += Number(a.amount);
      }
      if (tempMoneyAll > allocationData.transactionAmount) {
        showAlert('总金额不能大于调整金额');
        return false;
      }
    }
    return status;
  };

  componentDidMount() {
    this.initContent();
    this.props.onRef(this);
  }

  render() {
    const { fundsChangeId, bonusType, modalType } = this.props;
    const { allocationData, initSchoolData, initCommissionData } = this.state;

    return (
      <div className={styles.performanceAllocation}>
        <PerformanceHeader
          studentId={allocationData.studentId}
          studentName={allocationData.studentName}
          contractId={allocationData.contractId}
          contractBlCampusName={allocationData.contractBlCampusName}
          typeName={allocationData.typeName}
          transactionAmount={allocationData.transactionAmount}
        />
        <div className={styles.allocation}>
          <Tabs defaultActiveKey="1">
            <TabPane tab="学习中心业绩" key="1">
              <SchoolAllocation
                modalType={modalType}
                fundsChangeId={fundsChangeId}
                bonusType={bonusType}
                initSchoolData={initSchoolData}
                onChange={(val) => this.onChangeSchoolValue(val)}
              />
            </TabPane>
          </Tabs>
          <Tabs defaultActiveKey="1">
            <TabPane tab="提成人业绩" key="1">
              <CommissionAllocation
                modalType={modalType}
                fundsChangeId={fundsChangeId}
                bonusType={bonusType}
                initCommissionData={initCommissionData}
                onChange={(val) => this.onChangeCommissionValue(val)}
              />
            </TabPane>
          </Tabs>
        </div>
        <div className={styles.dynamic}>
          <Collapse>
            <Panel header="业绩分配动态" key="1">
              <AllocationDynamic modalType={modalType} bonusType={bonusType} fundsChangeId={fundsChangeId} />
            </Panel>
          </Collapse>
        </div>
      </div>
    );
  }
}

export default PerformanceAllocation;
