import React from 'react';
import styles from './styles.module.less';
import PropTypes from 'prop-types';
import { List } from 'COMPONENT';
import { pageIncomeDistributionRecordsApi } from '../loaders';
import { timeTrim } from './common';

// 业绩动态分配
class AllocationDynamic extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      dynamicData: [],
      pageSize: 20,
      pageNum: 1,
      total: 0,
    };

    this.columns = [
      {
        title: '业绩类型',
        dataIndex: 'bonusType',
        render: (text) => {
          return <div>{text === 'NORMAL' ? '收费' : '退费'}</div>;
        },
      },
      {
        title: '业绩维度',
        dataIndex: 'baseBonusType',
        render: (text) => {
          return <div>{text === 'CAMPUS' ? '学习中心业绩' : '提成人业绩'}</div>;
        },
      },
      {
        title: '产品类型',
        key: 'productType',
        render: () => {
          return <div>{this.props.modalType === '1' ? '班课转个性化' : '个性化转班课'}</div>;
        },
      },
      {
        title: '业绩归属类型',
        dataIndex: 'subBonusTypeName',
      },
      {
        title: '业绩归属',
        dataIndex: 'distributeBizName',
      },
      {
        title: '业绩金额',
        dataIndex: 'amount',
      },
      {
        title: '操作类型',
        dataIndex: 'operationName',
      },
      {
        title: '收款日期',
        dataIndex: 'fundsCreateTime',
      },
      {
        title: '创建日期',
        dataIndex: 'createTime',
        render: (current) => {
          return timeTrim(current);
        },
      },
      {
        title: '操作人',
        dataIndex: 'createUserName',
      },
    ];
  }

  static propTypes = {
    fundsChangeId: PropTypes.string,
    bonusType: PropTypes.string,
    modalType: PropTypes.string,
  };

  getDataFunc = async (pageSize, pageNum) => {
    let data = await pageIncomeDistributionRecordsApi({
      fundsChangeId: this.props.fundsChangeId,
      bonusType: this.props.bonusType,
      pageSize: pageSize || this.state.pageSize,
      pageNum: pageNum || this.state.pageNum,
    });
    this.setState({
      dynamicData: data.list,
      total: data.total,
    });
  };

  initContent = async () => {
    this.getDataFunc();
  };

  componentDidMount() {
    this.initContent();
  }

  render() {
    const { dynamicData, pageNum, total, pageSize } = this.state;
    return (
      <div className={styles.allocationDynamic}>
        <List
          bordered
          dataSource={dynamicData}
          columns={this.columns}
          pagination={{
            current: pageNum,
            total,
            pageSize,
          }}
          onChange={({ current, pageSize }) => {
            this.getDataFunc(pageSize, current);
            this.setState({ pageNum: current, pageSize });
          }}
        />
      </div>
    );
  }
}

export default AllocationDynamic;
