import React from 'react';
import styles from './styles.module.less';
import { Table } from 'antd';
import PropTypes from 'prop-types';
import { connect } from 'dva';
import { Select, Input } from 'COMPONENT';
import { createUid } from './common';
import { getUserForPerformance } from 'SERVICE/contract';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { debounce, cloneDeep } from 'lodash';
import _ from 'lodash';

class CommissionAllocation extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      columns: [],
      dataSource: [],
    };

    this.selectAllocationFrom = [
      {
        val: 'USER_USER',
        text: '个人',
      },
      {
        val: 'USER_CAMPUS',
        text: '学习中心团队',
      },
      {
        val: 'USER_BRANCH',
        text: '分公司团队',
      },
    ];
    this.dictVersion = 0; //搜索计数

    this.performanceColumns = [
      {
        title: '业绩维度',
        dataIndex: 'baseBonusType',
        render: () => {
          return <div>提成人业绩</div>;
        },
      },
      {
        title: '业绩归属类型',
        dataIndex: 'subBonusType',
        render: (text, record) => {
          return (
            <Select
              dict={this.selectAllocationFrom}
              value={record.subBonusType}
              placeholder="业绩归属类型"
              onChange={(e) => this.onchangeValue('subBonusType', e, record.id)}
            />
          );
        },
      },
      {
        title: '业绩归属',
        dataIndex: 'distributeBizId',
        render: (text, record, index) => {
          if (record.subBonusType === 'USER_USER') {
            if (!(this.state['personDict-' + index] instanceof Array)) {
              this.onSearchUser(record.distributeBizName, index, true);
            }
            return (
              <Select
                dict={this.state['personDict-' + index]}
                value={record.distributeBizId}
                placeholder="个人"
                onSearchKeyChange={(e) => this.performanceChange(e, index)}
                onChange={(e) => this.onchangeValue('distributeBizId', e, record.id)}
                showSearch={true}
              />
            );
          } else if (record.subBonusType) {
            return (
              <Select
                dict={record.subBonusType === 'USER_CAMPUS' ? this.props.campusDict : this.props.branchDict}
                value={record.distributeBizId}
                placeholder={record.subBonusType === 'USER_CAMPUS' ? '学习中心' : '分公司'}
                onChange={(e) => this.onchangeValue('distributeBizId', e, record.id)}
                showSearch={true}
              />
            );
          }
          return <Select disabled={true} />;
        },
      },
      {
        title: '学习中心业绩',
        dataIndex: 'amount',
        render: (text, record) => {
          return (
            <Input
              type="text"
              value={record.amount}
              placeholder="请输入学习中心业绩"
              onChange={(e) => this.onchangeInputValue('amount', e.target.value, record.id)}
              onBlur={(e) => this.onBlurInputValue('amount', e.target.value, record.id)}
            />
          );
        },
      },
      {
        title: '操作',
        dataIndex: 'operation',
        render: (text, record) => {
          return (
            <div>
              {this.state.dataSource.length > 1 ? (
                <a className={styles.deleteBtn} onClick={() => this.onDelete(record.id)}>
                  删除
                </a>
              ) : null}
              <a onClick={() => this.onAdd(record.id)}>增加</a>
            </div>
          );
        },
      },
    ];
    this.dutyColumns = [
      {
        title: '业绩归属类型',
        dataIndex: 'subBonusType',
        render: (text, record) => {
          return (
            <Select
              dict={this.selectAllocationFrom}
              value={record.subBonusType}
              placeholder="业绩归属类型"
              onChange={(e) => this.onchangeValue('subBonusType', e, record.id)}
            />
          );
        },
      },
      {
        title: '责任归属',
        dataIndex: 'distributeBizId',
        render: (text, record, index) => {
          if (record.subBonusType === 'USER_USER') {
            if (!(this.state['personDict-' + index] instanceof Array)) {
              this.onSearchUser(record.distributeBizName, index, true);
            }
            return (
              <Select
                dict={this.state['personDict-' + index]}
                value={record.distributeBizId}
                placeholder="个人"
                onSearchKeyChange={(e) => this.performanceChange(e, index)}
                onChange={(e) => this.onchangeValue('distributeBizId', e, record.id)}
                showSearch={true}
              />
            );
          } else if (record.subBonusType) {
            return (
              <Select
                dict={record.subBonusType === 'USER_CAMPUS' ? this.props.campusDict : this.props.branchDict}
                value={record.distributeBizId}
                placeholder={record.subBonusType === 'USER_CAMPUS' ? '学习中心' : '分公司'}
                onChange={(e) => this.onchangeValue('distributeBizId', e, record.id)}
                showSearch={true}
              />
            );
          }
          return <Select disabled={true} />;
        },
      },
      {
        title: '学习中心业绩',
        dataIndex: 'amount',
        render: (text, record) => {
          return (
            <Input
              type="text"
              value={record.amount}
              placeholder="请输入学习中心业绩"
              onChange={(e) => this.onchangeInputValue('amount', e.target.value, record.id)}
              onBlur={(e) => this.onBlurInputValue('amount', e.target.value, record.id)}
            />
          );
        },
      },
      {
        title: '操作',
        dataIndex: 'operation',
        render: (text, record) => {
          return (
            <div>
              {this.state.dataSource.length > 1 ? (
                <a className={styles.deleteBtn} onClick={() => this.onDelete(record.id)}>
                  删除
                </a>
              ) : null}
              <a onClick={() => this.onAdd(record.id)}>增加</a>
            </div>
          );
        },
      },
    ];
  }

  static propTypes = {
    fundsChangeId: PropTypes.string,
    bonusType: PropTypes.string,
    modalType: PropTypes.string,
    initCommissionData: PropTypes.array,
  };

  initDict = () => {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['allBranchCampusTree'],
      },
    });
  };

  onAdd = () => {
    const { dataSource } = this.state;
    const randomKey = createUid(8);
    const newData = {
      id: randomKey,
      baseBonusType: 'USER',
      subBonusType: '',
      distributeBizId: '',
      amount: '',
    };
    this.props.onChange([...dataSource, newData]);
    this.setState({
      dataSource: [...dataSource, newData],
    });
  };

  onDelete = (id) => {
    const dataSource = [...this.state.dataSource];
    let tempData = dataSource.filter((item) => item.id !== id);
    this.props.onChange([...tempData]);
    this.setState({ dataSource: tempData });
  };

  // 数字校验
  onchangeInputValue = (key, value, id) => {
    const reg = /^(0|[1-9][0-9]*)(\.[0-9]{0,2})?$/;
    if ((!isNaN(value) && reg.test(value)) || value === '') {
      this.onchangeValue(key, value, id);
    }
  };
  onBlurInputValue = (key, value, id) => {
    const reg = /^(0|[1-9][0-9]*)(\.[0-9]{1,2})?$/;
    if (!reg.test(value)) {
      this.onchangeValue(key, Number(value), id);
    }
  };

  // 输入框的值修改
  onchangeValue = (key, value, id) => {
    let tempArr = cloneDeep(this.state.dataSource);
    for (let a of tempArr) {
      if (a.id === id) {
        a[key] = value;
        if (key === 'subBonusType') {
          a.distributeBizId = '';
        }
      }
    }
    this.props.onChange(tempArr);
    this.setState({ dataSource: tempArr });
  };

  performanceChange = (value, index) => {
    debounce(this.onSearchUser, 300)(value, index);
  };

  /**
   * 搜索用户
   * @param term
   * @param index
   * @param {boolean?} type 为true解决重复调用问题
   * @returns {Promise<void>}
   */
  onSearchUser = async (term, index, type) => {
    if (term) {
      if (type && this.state['personDict-' + index]) {
        return false;
      }
      const dictVersion = ++this.dictVersion;
      try {
        const dict = await getUserForPerformance({ term });
        if (this.dictVersion === dictVersion) {
          //每一行的用户下拉框一个dict，用 index 后缀区分
          this.setState({
            ['personDict-' + index]: dict.map((item) => ({ text: item.label, val: item.value, title: item.label })),
          });
        }
      } catch (e) {
        showMessage(e.text);
      }
    } else {
      this.dictVersion++;
      this.setState({ ['personDict-' + index]: [] });
    }
  };

  componentDidUpdate(prevProps) {
    if (prevProps.initCommissionData !== this.props.initCommissionData) {
      let tempArr = _.cloneDeep(this.props.initCommissionData);
      const randomKey = createUid(8);
      this.setState({
        dataSource: tempArr.length
          ? tempArr
          : [
              {
                id: randomKey,
                baseBonusType: 'USER',
                subBonusType: '',
                distributeBizId: '',
                amount: '',
              },
            ],
      });
    }
  }

  componentDidMount() {
    this.initDict();
    if (this.props.modalType === '1') {
      this.setState({
        columns: this.performanceColumns,
      });
    } else {
      this.setState({
        columns: this.dutyColumns,
      });
    }
  }

  render() {
    const { dataSource, columns } = this.state;

    return (
      <div className={styles.commissionAllocation}>
        <Table bordered dataSource={dataSource} columns={columns} pagination={false} />
      </div>
    );
  }
}

let campusDict = null;

const mapStateToProps = (state) => {
  if (state.dict.allBranchCampusTree && !campusDict) {
    campusDict = [];
    state.dict.allBranchCampusTree.forEach((value) => {
      campusDict.push(...value.campus);
    });
  }

  return {
    branchDict: state.dict.allBranchCampusTree,
    campusDict: campusDict,
  };
};

export default connect(mapStateToProps)(CommissionAllocation);
