import React from 'react';
import styles from './styles.module.less';
import { Table } from 'antd';
import PropTypes from 'prop-types';
import { connect } from 'dva';
import { Select, Input } from 'COMPONENT';
import { createUid } from './common';
import _ from 'lodash';

class SchoolAllocation extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      columns: [],
      dataSource: [],
    };

    this.performanceColumns = [
      {
        title: '业绩维度',
        dataIndex: 'baseBonusType',
        render: () => {
          return <div>学习中心业绩</div>;
        },
      },
      {
        title: '业绩归属类型',
        dataIndex: 'subBonusType',
        render: () => {
          return <div>学习中心</div>;
        },
      },
      {
        title: '业绩归属',
        dataIndex: 'distributeBizId',
        render: (text, record) => {
          return (
            <Select
              dict={this.props.campusDict}
              value={record.distributeBizId}
              placeholder="学习中心"
              onChange={(e) => this.onchangeValue('distributeBizId', e, record.id)}
              showSearch={true}
            />
          );
        },
      },
      {
        title: '学习中心业绩',
        dataIndex: 'amount',
        render: (text, record) => {
          return (
            <Input
              type="text"
              value={record.amount}
              placeholder="请输入学习中心业绩"
              onChange={(e) => this.onchangeInputValue('amount', e.target.value, record.id)}
              onBlur={(e) => this.onBlurInputValue('amount', e.target.value, record.id)}
            />
          );
        },
      },
      {
        title: '操作',
        dataIndex: 'operation',
        render: (text, record) => {
          return (
            <div>
              {this.state.dataSource.length > 1 ? (
                <a className={styles.deleteBtn} onClick={() => this.onDelete(record.id)}>
                  删除
                </a>
              ) : null}
              <a onClick={() => this.onAdd(record.id)}>增加</a>
            </div>
          );
        },
      },
    ];
    this.dutyColumns = [
      {
        title: '责任学习中心',
        dataIndex: 'distributeBizId',
        render: (text, record) => {
          return (
            <Select
              dict={this.props.campusDict}
              value={record.distributeBizId}
              placeholder="学习中心"
              onChange={(e) => this.onchangeValue('distributeBizId', e, record.id)}
              showSearch={true}
            />
          );
        },
      },
      {
        title: '责任金额',
        dataIndex: 'amount',
        render: (text, record) => {
          return (
            <Input
              type="text"
              value={record.amount}
              placeholder="请输入学习中心业绩"
              onChange={(e) => this.onchangeInputValue('amount', e.target.value, record.id)}
              onBlur={(e) => this.onBlurInputValue('amount', e.target.value, record.id)}
            />
          );
        },
      },
      {
        title: '操作',
        dataIndex: 'operation',
        render: (text, record) => {
          return (
            <div>
              {this.state.dataSource.length > 1 ? (
                <a className={styles.deleteBtn} onClick={() => this.onDelete(record.id)}>
                  删除
                </a>
              ) : null}
              <a onClick={() => this.onAdd(record.id)}>增加</a>
            </div>
          );
        },
      },
    ];
  }

  static propTypes = {
    fundsChangeId: PropTypes.string,
    bonusType: PropTypes.string,
    modalType: PropTypes.string,
    initSchoolData: PropTypes.array,
  };

  // 新增行
  onAdd = () => {
    const { dataSource } = this.state;
    const randomKey = createUid(8);
    const newData = {
      id: randomKey,
      baseBonusType: 'CAMPUS',
      subBonusType: 'CAMPUS_CAMPUS',
      distributeBizId: '',
      amount: '',
    };
    this.props.onChange([...dataSource, newData]);
    this.setState({
      dataSource: [...dataSource, newData],
    });
  };

  // 删除行
  onDelete = (id) => {
    const dataSource = [...this.state.dataSource];
    let tempData = dataSource.filter((item) => item.id !== id);
    this.props.onChange([...tempData]);
    this.setState({ dataSource: tempData });
  };

  // 初始化
  initDict = () => {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['allBranchCampusTree'],
      },
    });
  };

  // 数字校验
  onchangeInputValue = (key, value, id) => {
    const reg = /^(0|[1-9][0-9]*)(\.[0-9]{0,2})?$/;
    if ((!isNaN(value) && reg.test(value)) || value === '') {
      this.onchangeValue(key, value, id);
    }
  };
  onBlurInputValue = (key, value, id) => {
    const reg = /^(0|[1-9][0-9]*)(\.[0-9]{1,2})?$/;
    if (!reg.test(value)) {
      this.onchangeValue(key, Number(value), id);
    }
  };

  // 输入框的值修改
  onchangeValue = (key, value, id) => {
    let tempArr = _.cloneDeep(this.state.dataSource);
    for (let a of tempArr) {
      if (a.id === id) {
        a[key] = value;
      }
    }
    this.props.onChange(tempArr);
    this.setState({ dataSource: tempArr });
  };

  componentDidUpdate(prevProps) {
    if (prevProps.initSchoolData !== this.props.initSchoolData) {
      let tempArr = _.cloneDeep(this.props.initSchoolData);
      const randomKey = createUid(8);
      this.setState({
        dataSource: tempArr.length
          ? tempArr
          : [
              {
                id: randomKey,
                baseBonusType: 'CAMPUS',
                subBonusType: 'CAMPUS_CAMPUS',
                distributeBizId: '',
                amount: '',
              },
            ],
      });
    }
  }

  componentDidMount() {
    this.initDict();
    if (this.props.modalType === '1') {
      this.setState({
        columns: this.performanceColumns,
      });
    } else {
      this.setState({
        columns: this.dutyColumns,
      });
    }
  }

  render() {
    const { dataSource, columns } = this.state;
    return (
      <div className={styles.schoolAllocation}>
        <Table bordered dataSource={dataSource} columns={columns} pagination={false} />
      </div>
    );
  }
}

let campusDict = null;

const mapStateToProps = (state) => {
  if (state.dict.allBranchCampusTree && !campusDict) {
    campusDict = [];
    state.dict.allBranchCampusTree.forEach((value) => {
      campusDict.push(...value.campus);
    });
  }

  return {
    branchDict: state.dict.allBranchCampusTree,
    campusDict: campusDict,
  };
};

export default connect(mapStateToProps)(SchoolAllocation);
