import React from 'react';
import { DatePicker } from 'antd';
import { Page, Wrapper, SearchBar, Select, Input, List, Modal, SearchFilter, SearchField } from 'COMPONENT';
import { loadBranchDict, loadBranchCampusDict, dealCampus } from 'UTIL/dict';
import { pageTransferRecordsApi, saveIncomeDistributionRecordApi } from './loaders';
import DutyAllocation from './DutyAllocation';
import PerformanceAllocation from './PerformanceAllocation';
import showAlert from 'COMPONENT/utils/globalTips/showAlert';
import { timeTrim } from './common/common';
import moment from 'moment';
import { connect } from 'dva';
import { loginUser } from 'UTIL/common';
import style from '../modals/ContractModifyRecord/styles.module.less';

const { RangePicker } = DatePicker;
const dateFormat = 'YYYY/MM/DD';

/**
 * @param isBegin 时分秒是否是零时零分零秒
 * @param year
 * @param month
 * @param day
 * @description 获取某个日期的时间戳
 * **/
function getLastMonth(isBegin, year, month, day) {
  let time = new Date();
  let currTime = isBegin ? '00:00:00' : '23:59:59';
  let timeArr = [year || time.getFullYear(), month || time.getMonth() + 1, day || time.getDate()];
  return new Date(`${timeArr[0]}-${noTenAddZero(timeArr[1])}-${noTenAddZero(timeArr[2])}T${currTime}`);
}

// 不逢十补零
function noTenAddZero(val) {
  return val > 9 ? val : `0${val}`;
}

class FundTransferRecord extends React.Component {
  constructor(props) {
    super(props);
    let tempState = {
      loading: false,
      dataSource: [],
      branchDict: [],
      campusDict: [],
      modalType: '1', // 弹框类型，1是个性化转班课，2是班课转个性化
      pageNum: 1,
      total: 0,
      pageSize: 20,
      visible: false,
      tableData: [],
      fundsChangeId: '',
    };

    let time = new Date();
    let year = time.getFullYear();
    let month = time.getMonth() + 1;
    if (month === 1) {
      year -= 1;
      month = 12;
    } else {
      month -= 1;
    }

    // 下拉框默认值
    this.defaultSelectValue = {
      beginCreateTime: getLastMonth(true, year, month, 1),
      endCreateTime: getLastMonth(false),
      defaultBeginCreateTime: getLastMonth(true, year, month, 1),
      defaultEndCreateTime: getLastMonth(false),
      contractId: '',
      studentName: '',
      branchId: loginUser.isGroupRole() ? '' : props.branchId,
      contractBlCampusId: loginUser.isCampusRole() ? props.contractBlCampusId : '',
      type: '',
    };
    this.columns = [
      { title: '学生编号', dataIndex: 'studentId' },
      { title: '学生姓名', dataIndex: 'studentName' },
      { title: '年级', dataIndex: 'gradeName' },
      { title: '学生学习中心', dataIndex: 'studentBlCampusName' },
      { title: '合同编号', dataIndex: 'contractId' },
      { title: '签单人', dataIndex: 'signStaffName' },
      { title: '收款人', dataIndex: 'fundsUserName' },
      { title: '合同学习中心', dataIndex: 'contractBlCampusName' },
      { title: '调转类型', dataIndex: 'typeName' },
      { title: '调转金额', dataIndex: 'transactionAmount' },
      {
        title: '收款时间',
        dataIndex: 'fundsCreateTime',
        render: (current) => {
          return timeTrim(current);
        },
      },
      {
        title: '操作',
        dataIndex: 'type',
        render: (current, row) => {
          const { fundsChangeId, type } = row;
          return (
            <div>
              {type === 'PERSONALIZED_TO_CLASS' && window.Index.authFuncBtnTags.indexOf('DUTY_ALLOCATION') >= 0 && (
                <span
                  onClick={() => {
                    this.showContractModal(fundsChangeId, '2');
                  }}
                  style={{ cursor: 'pointer', color: '#6D67FF' }}
                >
                  责任分配
                </span>
              )}
              {type === 'CLASS_TO_PERSONALIZED' &&
                window.Index.authFuncBtnTags.indexOf('PERFORMANCE_ALLOCATION') >= 0 && (
                  <span
                    onClick={() => {
                      this.showContractModal(fundsChangeId, '1');
                    }}
                    style={{ cursor: 'pointer', color: '#6D67FF' }}
                  >
                    业绩分配
                  </span>
                )}
            </div>
          );
        },
      },
    ];
    this.sonComponentFunc = () => {};
    this.state = {
      ...this.defaultSelectValue,
      ...tempState,
    };
  }

  async loadDict() {
    const [branchDict, allCampusDict] = await Promise.all([loadBranchDict(), loadBranchCampusDict()]);
    let campusDict = dealCampus(allCampusDict, this.state.branchId);
    this.setState({
      branchDict,
      allCampusDict,
      campusDict,
    });
  }

  // 搜索
  onSearch = async (otherParams) => {
    const { contractBlCampusId, beginCreateTime, endCreateTime, type, contractId, branchId, studentName, pageSize } =
      this.state;
    let params = {
      contractBlCampusId: contractBlCampusId || '',
      beginCreateTime,
      endCreateTime,
      type,
      contractId,
      studentName,
      contractBlBranchId: branchId || '',
      pageSize,
      pageNum: 1,
    };
    let tempParams = Object.assign(params, otherParams);
    this.setState({ pageNum: params.pageNum });
    await this.getListData(tempParams);
  };

  // 重置
  onReset = async () => {
    let campusDict = dealCampus(this.state.allCampusDict, this.defaultSelectValue.branchId);
    this.setState({
      ...this.defaultSelectValue,
      campusDict,
      pageSize: 20,
      pageNum: 1,
    });
    await this.getListData({
      contractBlCampusId: this.defaultSelectValue.contractBlCampusId || '',
      beginCreateTime: this.defaultSelectValue.beginCreateTime,
      endCreateTime: this.defaultSelectValue.endCreateTime,
      type: this.defaultSelectValue.type,
      contractId: this.defaultSelectValue.contractId,
      studentName: this.defaultSelectValue.studentName,
      contractBlBranchId: this.defaultSelectValue.branchId || '',
      pageSize: 20,
      pageNum: 1,
    });
  };

  // 日期修改，antd本身的问题，选择当天会返回两个一样的时间戳
  timeOnchange = (date) => {
    if (date.length) {
      let tempStartTime = date[0]._d,
        tempEndTime = date[1]._d;
      if (tempStartTime === tempEndTime) {
        let tempTime = date[0]._d;
        tempStartTime = new Date(
          `${tempTime.getFullYear()}-${noTenAddZero(tempTime.getMonth() + 1)}-${noTenAddZero(
            tempTime.getDate(),
          )}T00:00:00`,
        );
        tempEndTime = new Date(
          `${tempTime.getFullYear()}-${noTenAddZero(tempTime.getMonth() + 1)}-${noTenAddZero(
            tempTime.getDate(),
          )}T23:59:59`,
        );
      }
      this.setState({
        beginCreateTime: tempStartTime,
        endCreateTime: tempEndTime,
      });
    } else {
      this.setState({
        beginCreateTime: null,
        endCreateTime: null,
      });
    }
  };

  // 筛选框修改
  wrapInputOnchange = (key, value) => {
    this.setState({ [key]: value });
  };

  // 分学习中心级联器
  onchangeCampus = (branchId) => {
    const { allCampusDict } = this.state;
    const campusDict = dealCampus(allCampusDict, branchId);
    this.setState({ branchId, campusDict, contractBlCampusId: '' });
  };

  // 初始化
  initContent = async () => {
    this.setState({ loading: true });
    await this.loadDict();
    await this.onSearch();
    this.setState({ loading: false });
  };

  performanceAllocationFunc = (ref) => {
    this.sonComponentFunc = ref;
  };

  dutyAllocationFunc = (ref) => {
    this.sonComponentFunc = ref;
  };

  // 保存
  saveFunc = async () => {
    let status = this.sonComponentFunc.checkValid();
    if (status) {
      let data = await saveIncomeDistributionRecordApi({
        fundsChangeId: this.state.fundsChangeId,
        bonusType: this.state.modalType === '1' ? 'NORMAL' : 'FEEDBACK_REFUND',
        incomeDistributionRecordDtos: this.state.tableData,
      });
      if (data.data) {
        this.closeModal();
      } else {
        showAlert(data.resultMessage);
      }
    }
  };

  // 关闭弹窗
  closeModal = () => {
    this.setState({ visible: false });
  };

  /**
   * @param {Object?} params 参数
   * @description 获取列表数据
   * **/
  getListData = async (params) => {
    let data = await pageTransferRecordsApi(params || { pageSize: 20, pageNum: 1 });
    this.setState({
      dataSource: data.list,
      total: data.total,
    });
  };

  showContractModal = (fundsChangeId, modalType) => {
    this.setState({
      modalType,
      fundsChangeId,
      visible: true,
    });
  };

  onChangeValue = (val) => {
    this.setState({
      tableData: val,
    });
  };

  componentDidMount() {
    this.initContent();
  }

  render() {
    const {
      loading,
      contractBlCampusId,
      type,
      contractId,
      studentName,
      branchId,
      branchDict,
      campusDict,
      dataSource,
      pageNum,
      total,
      pageSize,
      modalType,
      visible,
      fundsChangeId,
      defaultBeginCreateTime,
      defaultEndCreateTime,
      beginCreateTime,
      endCreateTime,
    } = this.state;
    const typeDict = [
      { val: '', text: '调转类型' },
      { val: 'PERSONALIZED_TO_CLASS', text: '个性化转班课' },
      { val: 'CLASS_TO_PERSONALIZED', text: '班课转个性化' },
    ];
    return (
      <Page
        loading={loading}
        title="资金调转记录"
        subHeader={
          <SearchFilter onSearch={this.onSearch} onReset={this.onReset} truncationNum={5}>
            <SearchField span={2}>
              <RangePicker
                size="large"
                defaultValue={[moment(defaultBeginCreateTime, dateFormat), moment(defaultEndCreateTime, dateFormat)]}
                value={[
                  beginCreateTime ? moment(beginCreateTime, dateFormat) : null,
                  endCreateTime ? moment(endCreateTime, dateFormat) : null,
                ]}
                onChange={this.timeOnchange}
              />
            </SearchField>
            <SearchField span={1}>
              <Input
                value={contractId}
                placeholder="合同编号"
                onChange={(e) => this.wrapInputOnchange('contractId', e.target.value)}
              />
            </SearchField>
            <SearchField span={1}>
              <Input
                value={studentName}
                placeholder="学生姓名"
                onChange={(e) => this.wrapInputOnchange('studentName', e.target.value)}
              />
            </SearchField>
            <SearchField span={1}>
              <Select
                dict={branchDict}
                value={branchId}
                placeholder="分公司"
                onChange={this.onchangeCampus}
                showSearch={true}
              />
            </SearchField>
            <SearchField span={1}>
              <Select
                dict={campusDict}
                value={contractBlCampusId}
                placeholder="学习中心"
                onChange={(e) => this.wrapInputOnchange('contractBlCampusId', e)}
                showSearch={true}
              />
            </SearchField>
            <SearchField span={1}>
              <Select
                dict={typeDict}
                value={type}
                placeholder="调转类型"
                onChange={(e) => this.wrapInputOnchange('type', e)}
              />
            </SearchField>
          </SearchFilter>
        }
      >
        <List
          dataSource={dataSource}
          pagination={{
            current: pageNum,
            total,
            pageSize,
          }}
          columns={this.columns}
          onChange={({ current, pageSize }) => {
            this.onSearch({ pageNum: current, pageSize });
            this.setState({ pageNum: current, pageSize });
          }}
        />
        {visible && (
          <Modal
            title={modalType === '1' ? '业绩分配' : '责任分配'}
            okText="保存"
            // loading={this.state.loading}
            onCancel={this.closeModal}
            onOk={this.saveFunc}
            widthSize="L"
            toggle={true}
            visible={visible}
          >
            {modalType === '1' ? (
              <PerformanceAllocation
                onRef={(ref) => this.performanceAllocationFunc(ref)}
                fundsChangeId={fundsChangeId}
                modalType={modalType}
                bonusType="NORMAL"
                onChange={(val) => this.onChangeValue(val)}
              />
            ) : (
              <DutyAllocation
                onRef={(ref) => this.dutyAllocationFunc(ref)}
                fundsChangeId={fundsChangeId}
                modalType={modalType}
                bonusType="FEEDBACK_REFUND"
                onChange={(val) => this.onChangeValue(val)}
              />
            )}
          </Modal>
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    contractBlCampusId: state.account.user.campusId,
    branchId: state.account.user.brenchId,
  };
};

export default connect(mapStateToProps)(FundTransferRecord);
