/**
 * Created by Haolin on 2019/1/10.
 */
import React, { PropTypes } from 'react';
import { connect } from 'dva';
import style from './index.less';
import shoppingCartPng from '../../../assets/png/shoppingCart.png';
import moment from 'moment';

const ClassInfo = (props) => {
  const prefix = 'ClassInfo';
  let display = {
    display: Boolean(props.text) ? 'inline-block' : 'none',
  };
  return (
    <span className={style[prefix]} style={display}>
      {props.text}
    </span>
  );
};
const MoreInfo = (props) => {
  const prefix = 'MoreInfo';
  let display = {
    display: Boolean(props.text) ? 'inline-block' : 'none',
  };
  return (
    <span className={style[prefix]} style={display}>
      <span>{props.label}：</span>
      <span>{props.text}</span>
    </span>
  );
};
MoreInfo.propTypes = {
  label: PropTypes.string.isRequired,
  text: PropTypes.string,
};
const Price = (props) => {
  const prefix = 'Price';
  return (
    <div className={style[prefix]}>
      <span>¥</span>
      <span>{props.num}</span>
    </div>
  );
};
const AddBtn = (props) => {
  const prefix = 'AddBtn';
  return (
    <div className={style[prefix]} onClick={props.onClick}>
      <div className={style[`${prefix}-body`]}>
        <img className={style[`${prefix}-icon`]} src={shoppingCartPng} />
      </div>
    </div>
  );
};
AddBtn.propTypes = {
  onClick: PropTypes.func.isRequired,
};

class ProductCard extends React.Component {
  constructor(props) {
    super(props);
  }
  static classPrefix = 'ProductCard';
  componentDidMount() {}
  componentWillUnmount() {}

  render() {
    const {
      data: {
        productName,
        productTypeName,
        validEndDate,
        className,
        classId,
        classTypeName,
        teacherName,
        price,
        consumeHour,
        totalHour,
        campusName,
        peopleNum,
        peopleQuantity,
        nextTime,
        classTimeRange,
        classLevel,
      },
      index,
      onBtnClick,
    } = this.props;
    const prefix = ProductCard.classPrefix;

    return (
      <div className={style[prefix]}>
        <div className={style[`${prefix}-left`]}>
          <div className={style[`${prefix}-header`]}>
            <div className={style[`${prefix}-title`]}>
              <span className={style[`${prefix}-type`]}>{productTypeName}</span>
              <span>{productName}</span>
            </div>
          </div>
          <div className={style[`${prefix}-body`]}>
            {classId ? (
              [
                <ClassInfo text={className ? '班级名称: ' + className : undefined} key={`${index}_className`} />,
                <ClassInfo text={classTypeName} key={`${index}_classTypeName`} />,
                <ClassInfo text={classLevel} key={`${index}_classLevel`} />,
                <ClassInfo text={teacherName} key={`${index}_teacherName`} />,
                <ClassInfo text={`已上课时 ${consumeHour || '-'}/${totalHour || '-'}`} key={`${index}_courseTime`} />,
              ]
            ) : (
              <ClassInfo text="暂无课程安排" />
            )}
          </div>
          <div className={style[`${prefix}-footer`]}>
            <MoreInfo label="截止日期" text={validEndDate} />
            {
              /**
               * 有课程id的时候才会有这些字段
               */
              classId && [
                <MoreInfo label="学习中心" text={campusName} key={`${index}_campusName`} />,
                <MoreInfo
                  label="已报人数"
                  text={`${peopleNum || '-'}/${peopleQuantity || '-'}`}
                  key={`${index}_regPeople`}
                />,
                <MoreInfo label="下次上课日期" text={nextTime} key={`${index}_nextTime`} />,
                <MoreInfo label="上课时间" text={classTimeRange} key={`${index}_classTimeText`} />,
              ]
            }
          </div>
        </div>
        <div className={style[`${prefix}-right`]}>
          <div className={style[`${prefix}-num-wrap`]}>
            <Price num={price} />
          </div>
          <div className={style[`${prefix}-btn-wrap`]}>
            <AddBtn
              onClick={() => {
                onBtnClick(index);
              }}
            />
          </div>
        </div>
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  return {};
};

export default connect(mapStateToProps)(ProductCard);
