/**
 * Created by Haolin on 2019/1/11.
 */
import React, { PropTypes } from 'react';
import { connect } from 'dva';
import { storeName } from '../../store/constant.js';
import style from './index.less';
import { Row, Col } from 'antd';
import Select from 'COMPONENT/Select';
import Input from 'COMPONENT/Input';
import Button from 'COMPONENT/Button';
import Search2 from 'COMPONENT/Search2';
import { ECS_CLASS, OTHERS, o2mTypeDict } from 'CONST/dicts/product';
import { productType, productGroup } from 'MODEL/dict/utils/config';
import { getRelationsDict, getRelationPayloadPlus } from 'MODEL/dict/utils/index';
import { loginUser } from 'UTIL/common';

//精英班可显示的产品类型
const ECS_PRO_ARR = [ECS_CLASS, OTHERS];

class Search extends React.Component {
  constructor(props) {
    super(props);
    this.search = this.search.bind(this);
    this.reset = this.reset.bind(this);
    this.updateKeyword = this.updateKeyword.bind(this);
    this.onSearchChange = this.onSearchChange.bind(this);
    this.destroy = this.destroy.bind(this);

    // 产品类型关联关系type、key对应关系配置
    this.productTypeTreeMap = {
      [productType]: 'sKey_productTypeId',
      [productGroup]: 'sKey_productGroup',
    };
  }
  static classPrefix = 'Search';
  static propTypes = {
    handleSearch: PropTypes.func,
  };
  componentWillUnmount() {
    this.destroy();
  }
  // 检索
  search() {
    this.props.handleSearch();
  }
  // 重置检索条件
  reset() {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        sKey_productName: '',
        sKey_className: '',
        sKey_classTeacherId: null,
        sKey_campus: null,
        sKey_productTypeId: null,
        sKey_classTypeId: null,
        sKey_productSeriesId: null,
        sKey_yearId: null,
        sKey_quarterId: null,
        sKey_gradeId: null,
        sKey_productGroup: null,
        sKey_subject: null,
        sKey_phaseId: null,
        sKey_oneOnManyType: null,
      },
    });
  }
  // 更新检索条件
  updateKeyword(params) {
    const { type, val } = params;
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: { [type]: val },
    });
  }
  // 更改筛选项
  onSearchChange(params) {
    this.props.toggleLoading();

    const { sKey_productTypeId, sKey_classTypeId } = this.props;

    //和之前的产品类型不一样，则改变了，要重置班级类型，产品系列
    const isProductTypeChanged = sKey_productTypeId !== params.sKey_productTypeId;
    let sKey_classLevelId = params.sKey_classLevelId;

    if (isProductTypeChanged || sKey_classTypeId !== params.sKey_classTypeId) {
      sKey_classLevelId = null;
    }

    let payload = {};
    if (loginUser.isTwoTeacherSym() && isProductTypeChanged) {
      payload = getRelationPayloadPlus('sKey_productTypeId', params.sKey_productTypeId, {
        type: productType,
        dictTree: this.props.productTypeTree,
        state: this.props,
        map: this.productTypeTreeMap,
      });
    }

    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        ...params,
        ...payload,
        sKey_classTypeId: isProductTypeChanged ? null : params.sKey_classTypeId,
        sKey_productSeriesId: isProductTypeChanged ? null : params.sKey_productSeriesId,
        sKey_classLevelId,
        sKey_oneOnManyType: isProductTypeChanged ? null : params.sKey_oneOnManyType,
        current: 1, //重置为第一页
      },
    });
    this.props.dispatch({
      type: `${storeName}/loadProductList`,
      payload: {
        callback: this.props.toggleLoading,
      },
    });
  }
  // 注销前销毁
  destroy() {
    this.reset();
  }

  genSearch2Config() {
    if (loginUser.isTwoTeacherSym()) {
      return this.genSearch2ConfigByTwoTeacher();
    } else {
      return this.genSearch2ConfigByXingHuo();
    }
  }

  genSearch2ConfigByXingHuo = () => {
    const {
      productTypeDict,
      classTypeDict,
      courseSeriesDict,
      yearDict,
      quarterDict,
      gradeDict,
      productGroupDict,
      subjectDict,
      phaseDict,
      sKey_productTypeId,
      sKey_classTypeId,
      sKey_productSeriesId,
      sKey_yearId,
      sKey_quarterId,
      sKey_gradeId,
      sKey_productGroup,
      sKey_subject,
      sKey_phaseId,
      classLevelDict,
      sKey_classLevelId,
      sKey_oneOnManyType,
    } = this.props;
    if (sKey_productTypeId === 'ONE_ON_MANY') {
      return [
        { label: '产品类型', dict: productTypeDict, field: 'sKey_productTypeId', value: sKey_productTypeId },
        { label: '产品系列', dict: courseSeriesDict, field: 'sKey_productSeriesId', value: sKey_productSeriesId },
        { label: '一对多类型', dict: o2mTypeDict, field: 'sKey_oneOnManyType', value: sKey_oneOnManyType },
        { label: '班级层次', dict: classLevelDict, field: 'sKey_classLevelId', value: sKey_classLevelId },
        { label: '年级', dict: gradeDict, field: 'sKey_gradeId', value: sKey_gradeId },
        { label: '产品季度', dict: quarterDict, field: 'sKey_quarterId', value: sKey_quarterId },
        { label: '产品年份', dict: yearDict, field: 'sKey_yearId', value: sKey_yearId },
        { label: '科目', dict: subjectDict, field: 'sKey_subject', value: sKey_subject },
        { label: '产品组', dict: productGroupDict, field: 'sKey_productGroup', value: sKey_productGroup },
        { label: '期数', dict: phaseDict, field: 'sKey_phaseId', value: sKey_phaseId },
      ];
    }
    return [
      { label: '产品类型', dict: productTypeDict, field: 'sKey_productTypeId', value: sKey_productTypeId },
      { label: '产品系列', dict: courseSeriesDict, field: 'sKey_productSeriesId', value: sKey_productSeriesId },
      { label: '班级类型', dict: classTypeDict, field: 'sKey_classTypeId', value: sKey_classTypeId },
      { label: '班级层次', dict: classLevelDict, field: 'sKey_classLevelId', value: sKey_classLevelId },
      { label: '年级', dict: gradeDict, field: 'sKey_gradeId', value: sKey_gradeId },
      { label: '产品季度', dict: quarterDict, field: 'sKey_quarterId', value: sKey_quarterId },
      { label: '产品年份', dict: yearDict, field: 'sKey_yearId', value: sKey_yearId },
      { label: '科目', dict: subjectDict, field: 'sKey_subject', value: sKey_subject },
      { label: '产品组', dict: productGroupDict, field: 'sKey_productGroup', value: sKey_productGroup },
      { label: '期数', dict: phaseDict, field: 'sKey_phaseId', value: sKey_phaseId },
    ];
  };

  genSearch2ConfigByTwoTeacher = () => {
    const {
      productTypeDict,
      classTypeDict,
      courseSeriesDict,
      yearDict,
      quarterDict,
      gradeDict,
      productTypeTree,
      subjectDict,
      phaseDict,
      sKey_productTypeId,
      sKey_classTypeId,
      sKey_productSeriesId,
      sKey_yearId,
      sKey_quarterId,
      sKey_gradeId,
      sKey_productGroup,
      sKey_subject,
      sKey_phaseId,
      classLevelDict,
      sKey_classLevelId,
      sKey_oneOnManyType,
    } = this.props;

    const ssProductGroupDict = getRelationsDict(productGroup)(productTypeTree, sKey_productTypeId);

    const search1 = [
      { label: '产品类型', dict: productTypeDict, field: 'sKey_productTypeId', value: sKey_productTypeId },
      { label: '产品组', dict: ssProductGroupDict, field: 'sKey_productGroup', value: sKey_productGroup },
    ];
    const search2 =
      sKey_productTypeId === 'ONE_ON_MANY'
        ? [{ label: '一对多类型', dict: o2mTypeDict, field: 'sKey_oneOnManyType', value: sKey_oneOnManyType }]
        : [];
    const search3 = [
      { label: '产品系列', dict: courseSeriesDict, field: 'sKey_productSeriesId', value: sKey_productSeriesId },
      { label: '班级类型', dict: classTypeDict, field: 'sKey_classTypeId', value: sKey_classTypeId },
      { label: '班级层次', dict: classLevelDict, field: 'sKey_classLevelId', value: sKey_classLevelId },
      { label: '年级', dict: gradeDict, field: 'sKey_gradeId', value: sKey_gradeId },
      { label: '产品季度', dict: quarterDict, field: 'sKey_quarterId', value: sKey_quarterId },
      { label: '产品年份', dict: yearDict, field: 'sKey_yearId', value: sKey_yearId },
      { label: '科目', dict: subjectDict, field: 'sKey_subject', value: sKey_subject },
      { label: '期数', dict: phaseDict, field: 'sKey_phaseId', value: sKey_phaseId },
    ];

    return search1.concat(search2, search3);
  };

  render() {
    const { campusDict, teacherDict, sKey_productName, sKey_className, sKey_classTeacherId, sKey_campus, isEcs } =
      this.props;
    const prefix = Search.classPrefix;

    //筛选项配置
    const Search2Config = this.genSearch2Config();

    return (
      <div className={style[prefix]}>
        <div className={style[`${prefix}-bar`]}>
          <Row className={style[`${prefix}-key-wrap`]}>
            <Col className={style[`${prefix}-key-wrap-item`]} span={6}>
              <Input
                placeholder="产品课程名称"
                value={sKey_productName}
                onChange={(e) => {
                  this.updateKeyword({ type: 'sKey_productName', val: e.target.value });
                }}
              />
            </Col>
            <Col className={style[`${prefix}-key-wrap-item`]} span={6}>
              <Input
                placeholder="班级名称"
                value={sKey_className}
                onChange={(e) => {
                  this.updateKeyword({ type: 'sKey_className', val: e.target.value });
                }}
              />
            </Col>
            <Col className={style[`${prefix}-key-wrap-item`]} span={6}>
              <Select
                placeholder="班级老师"
                dict={teacherDict}
                showSearch
                value={sKey_classTeacherId}
                onChange={(val) => {
                  this.updateKeyword({ type: 'sKey_classTeacherId', val });
                }}
              />
            </Col>
            <Col className={style[`${prefix}-key-wrap-item`]} span={6}>
              <Select
                placeholder="班级学习中心"
                dict={campusDict}
                showSearch
                value={sKey_campus}
                onChange={(val) => {
                  this.updateKeyword({ type: 'sKey_campus', val });
                }}
              />
            </Col>
          </Row>
          <div className={style[`${prefix}-btn-wrap`]}>
            <Button type="primary" onClick={this.search}>
              查询
            </Button>
            <Button type="default" onClick={this.reset}>
              重置
            </Button>
          </div>
        </div>
        {/* 更多筛选项 */}
        <Search2
          style={{ marginTop: 20 }}
          config={Search2Config}
          onChange={this.onSearchChange}
          rowNum={6}
          defaultExpand
        />
      </div>
    );
  }
}

function buildClassLevelDict(dictTree, productType, classType) {
  let filterDict;
  if (classType) {
    filterDict = dictTree.map((item) => item.classTypes).reduce((acc, cur) => acc.concat(cur));
    filterDict = filterDict.find((item) => item.id === classType);
    filterDict = filterDict ? filterDict.classLevels : [];
  } else if (productType) {
    filterDict = dictTree.find((item) => item.id === productType);
    filterDict = filterDict ? filterDict.classTypes : [];
    if (filterDict.length > 0) {
      filterDict = filterDict.filter((item) => item.classLevels.length > 0);
      filterDict = filterDict.map((item) => item.classLevels);
      if (filterDict.length > 0) {
        filterDict = filterDict.reduce((acc, cur) => acc.concat(cur));
      }
    }
  } else {
    filterDict = dictTree.map((item) => item.classTypes);
    if (filterDict.length > 0) {
      filterDict = filterDict.reduce((acc, cur) => acc.concat(cur));
      filterDict = filterDict.filter((item) => item.classLevels.length > 0);
      filterDict = filterDict.map((item) => item.classLevels);
      if (filterDict.length > 0) {
        filterDict = filterDict.reduce((acc, cur) => acc.concat(cur));
      }
    }
  }
  return filterDict.map((item) => ({
    val: item.id,
    text: item.name,
  }));
}

let yearDictStorage = null;

const mapStateToProps = (state) => {
  const { year, quarter, grade, productTypeTree, productGroup, subject, phase, weekDay } = state.dict;
  const {
    sKey_productName,
    sKey_className,
    sKey_classTeacherId,
    sKey_campus,
    sKey_productTypeId,
    sKey_classTypeId,
    sKey_productSeriesId,
    sKey_yearId,
    sKey_quarterId,
    sKey_gradeId,
    sKey_productGroup,
    sKey_subject,
    sKey_classLevelId,
    sKey_phaseId,
    teacherDict,
    isEcs,
    campusDict,
    productDicts,
    productTypes,
    sKey_oneOnManyType,
  } = state[storeName];

  // 过滤无用年份
  if (!yearDictStorage) {
    const { serverDate } = state.account.user;
    yearDictStorage = year ? year.filter((item) => Number(item.text) >= new Date(serverDate).getFullYear() - 1) : null;
  }

  //根据当前环境拥有的产品类型过滤
  let filterProduct = productDicts.filter((item) => ~productTypes.indexOf(item.id));
  const classTypeDict = sKey_productTypeId
    ? //如果已选产品类型，找到该产品类型下的 班级类型
      filterProduct
        .find((item) => item.id === sKey_productTypeId)
        .classTypes.map((item) => ({ text: item.name, val: item.id }))
    : //如果没有，则把所有产品类型下的 班级类型 聚合成一个新数组
      filterProduct.reduce(
        (acc, cur) => [...acc, ...cur.classTypes.map((item) => ({ text: item.name, val: item.id }))],
        [],
      );
  const courseSeriesDict = sKey_productTypeId
    ? //如果已选产品类型，找到该产品类型下的 产品系列
      filterProduct
        .find((item) => item.id === sKey_productTypeId)
        .courseSeries.map((item) => ({ text: item.name, val: item.id }))
    : //如果没有，则把所有产品类型下的 产品系列 聚合成一个新数组
      filterProduct.reduce(
        (acc, cur) => [...acc, ...cur.courseSeries.map((item) => ({ text: item.name, val: item.id }))],
        [],
      );
  // 根据产品类型/班级类型切换班级层次字典
  const classLevelDict = productDicts ? buildClassLevelDict(productDicts, sKey_productTypeId, sKey_classTypeId) : [];

  return {
    campusDict,
    teacherDict,
    productTypeTree,
    productTypeDict: filterProduct.map((item) => ({ text: item.name, val: item.id })), //产品类型
    classTypeDict, //班级类型
    courseSeriesDict, //产品系列
    yearDict: yearDictStorage || [],
    quarterDict: quarter || [],
    gradeDict: grade || [],
    productGroupDict: productGroup || [],
    subjectDict: subject || [],
    phaseDict: weekDay && phase ? weekDay.concat(phase) : [], //期数，拼接星期
    sKey_productName,
    sKey_className,
    sKey_classTeacherId,
    sKey_campus,
    sKey_productTypeId,
    sKey_classTypeId,
    sKey_productSeriesId,
    sKey_yearId,
    sKey_quarterId,
    sKey_gradeId,
    sKey_productGroup,
    sKey_subject,
    sKey_phaseId,
    classLevelDict,
    sKey_classLevelId,
    isEcs,
    sKey_oneOnManyType,
  };
};

export default connect(mapStateToProps)(Search);
