import React from 'react';
import { connect } from 'dva';
import { storeName } from '../store/constant.js';
import Section from '../components/Section';
import ProductDesk from './ProductDesk';
import Search from './Search';
import Loading from 'COMPONENT/Loading';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import ModalSelectCourseTime from 'BLOC/ModalSelectCourseTime';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import Wrapper from 'COMPONENT/Wrapper';

const TWO_TEACHER_VAL = 'TWO_TEACHER';
const SMALL_CLASS_VAL = 'SMALL_CLASS';

const SelectCourseDateTips = () => {
  return <span style={{ color: 'red' }}>注：补报历史课程最多只能报读今天之前15天内的课程</span>;
};

class AddProduct extends React.Component {
  constructor(props) {
    super(props);
    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.initPrdList = this.initPrdList.bind(this);
    this.loadData = this.loadData.bind(this);
    this.changePage = this.changePage.bind(this);
    this.search = this.search.bind(this);
    this.addProduct = this.addProduct.bind(this);
    this.addPrdAction = this.addPrdAction.bind(this);
    this.beforeDestroy = this.beforeDestroy.bind(this);
    this.toggleLoading = this.toggleLoading.bind(this);
    this.switchModalSelectCourseTime = this.switchModalSelectCourseTime.bind(this);
    this.addPrdWithCourseTime = this.addPrdWithCourseTime.bind(this);
    this.addPrdWithStartTime = this.addPrdWithStartTime.bind(this);
    this.state = {
      name: '选择报读课程',
      loading: false,
      deskLoading: false,
      modalVisible: false,
    };
    this.activeClassId = '';
    this.activeProductType = '';
    this.activeIndex = null;
  }
  static classPrefix = 'AddProduct';
  componentDidMount() {
    this.init();
  }
  componentWillUnmount() {
    this.beforeDestroy();
  }
  // 初始化
  async init() {
    this.props.modalLoading(true);
    await this.initDict();
    await this.initDefault();
    await this.initPrdList();
    this.props.modalLoading(false);
  }
  initDict() {
    const { dispatch } = this.props;
    dispatch({
      type: `${storeName}/getTeachers`,
    });
    dispatch({
      type: `${storeName}/getCampuses`,
    });
    const commonDicts = new Promise((resolve, reject) => {
      dispatch({
        type: 'dict/initDict',
        payload: {
          list: ['year', 'quarter', 'grade', 'productTypeTree', 'productGroup', 'subject', 'phase', 'weekDay'],
          callback() {
            resolve();
          },
          handleErr() {
            reject();
          },
        },
      });
    });
    const productDicts = new Promise((resolve, reject) => {
      dispatch({
        type: `${storeName}/getProductDicts`,
        payload: {
          callback() {
            resolve();
          },
          handleErr() {
            reject();
          },
        },
      });
    });
    return Promise.all([commonDicts, productDicts]);
  }
  initDefault() {
    const { dispatch } = this.props;
    return new Promise((resolve) => {
      dispatch({
        type: `${storeName}/initDefault`,
        payload: {
          callback() {
            resolve();
          },
        },
      });
    });
  }
  initPrdList() {
    const { dispatch, current, pageSize } = this.props;
    return new Promise((resolve, reject) => {
      dispatch({
        type: `${storeName}/loadProductList`,
        payload: {
          pageNo: current,
          pageSize,
          callback() {
            resolve();
          },
          handleErr() {
            reject();
          },
        },
      });
    });
  }
  // 加载列表数据
  loadData() {
    this.setState({ loading: true });
    const _self = this;
    this.props.dispatch({
      type: `${storeName}/loadProductList`,
      payload: {
        callback() {
          _self.setState({ loading: false });
        },
      },
    });
  }
  // 翻页
  changePage(current, pageSize) {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        current,
        pageSize,
      },
    });
    this.loadData();
  }
  // 搜索
  search() {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        current: 1,
      },
    });
    this.loadData();
  }
  // 添加产品到购物车中
  addProduct(index) {
    if (this.props.sKey_gradeId) {
      if (this.props.list[index].gradeId !== this.props.sKey_gradeId) {
        return showConfirm('学生在读年级与选择的产品年级不一致，请确认是否跨年级上课', () => {
          this.addPrdAction(this.props.list[index], index);
        });
      }
    }
    this.addPrdAction(this.props.list[index], index);
  }
  addPrdAction(item, index) {
    if ([TWO_TEACHER_VAL, SMALL_CLASS_VAL].includes(item.product.category)) {
      // 双师/小班需先选择上课日期(班级已排课，如果未排课则使用班级开始上课时间)
      let isTwoTc = item.product.category === TWO_TEACHER_VAL;
      let activeClass = isTwoTc ? item.twoTeacherClassTwo : item.miniClass;
      if (activeClass) {
        this.activeClassId = isTwoTc ? activeClass.id.toString() : activeClass.miniClassId;
      } else {
        return showMessage('所选产品没有班级信息，无法添加！', 'error');
      }
      this.activeProductType = item.product.category;
      this.activeIndex = index;
      this.switchModalSelectCourseTime();
    } else {
      // 直接添加
      this.props.dispatch({
        type: `${storeName}/addProductToCart`,
        payload: { index },
      });
    }
  }
  // 组件注销前销毁相关store
  beforeDestroy() {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        originProductList: [],
        selectedProductStorage: [],
        productList: [],
        productTotal: 0,
        current: 1,
        selectedProductList: [],
      },
    });
  }
  // 供子组件触发当前Loading
  toggleLoading() {
    this.setState({
      loading: !this.state.loading,
    });
  }
  // 切换选择首次上课时间弹窗显示
  switchModalSelectCourseTime() {
    this.setState(
      (prevState) => ({ modalVisible: !prevState.modalVisible }),
      () => {
        if (!this.state.modalVisible) {
          this.activeClassId = '';
          this.activeProductType = '';
          this.activeIndex = null;
        }
      },
    );
  }
  // 带课程上课时间添加产品
  addPrdWithCourseTime(item, index, list) {
    this.props.dispatch({
      type: `${storeName}/addProductWithDate`,
      payload: {
        index: this.activeIndex,
        date: item.courseDate,
        time: item.courseBeginTime,
        realClassHours: list
          .filter((course, courseIndex) => courseIndex >= index)
          .reduce((total, current) => total + current._origin.courseHours, 0),
      },
    });
    this.switchModalSelectCourseTime();
  }
  // 带开始上课时间添加产品
  addPrdWithStartTime() {
    let classData = this.props.list[this.activeIndex];
    this.props.dispatch({
      type: `${storeName}/addProductWithDate`,
      payload: {
        index: this.activeIndex,
        date: classData.classCreateDate,
        time: null,
        realClassHours: classData.totalHour,
      },
    });
    this.switchModalSelectCourseTime();
  }

  getValidCourseDate = (list) => {
    let result = [];
    for (let i = 0; i < list.length; i++) {
      if (!list[i].disabled) {
        result = [i];
        break;
      }
    }
    return result;
  };

  render() {
    const { list, total, selectedTotal, current, pageSize } = this.props;
    const { name, loading, deskLoading } = this.state;
    const prefix = AddProduct.classPrefix;

    return (
      <Section
        className={prefix}
        title={<span style={{ fontSize: 18 }}>{name}</span>}
        paddingSize={30}
        bodyStyle={{
          height: 'calc(100% - 58px)',
          overflowY: 'auto',
        }}
      >
        <Loading loading={loading} />
        <Wrapper header={<Search handleSearch={this.search} toggleLoading={this.toggleLoading} />}>
          <ProductDesk
            dataSource={list}
            loading={!loading && deskLoading}
            pagination={{
              current,
              pageSize,
              total,
            }}
            onChange={this.changePage}
            onBtnClick={this.addProduct}
          />
          {this.state.modalVisible && (
            <ModalSelectCourseTime
              title="选择首次上课时间"
              classId={this.activeClassId}
              productType={this.activeProductType}
              onCancel={this.switchModalSelectCourseTime}
              onConfirm={this.addPrdWithCourseTime}
              noDataDirectCallback={this.addPrdWithStartTime}
              beforeNowDisabled={true}
              beforeNowDays={14}
              beforeNowMaxCourse={Infinity}
              defaultSelect={this.getValidCourseDate}
              footerTipsRender={SelectCourseDateTips}
            />
          )}
        </Wrapper>
      </Section>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    list: state[storeName].productList,
    total: state[storeName].productTotal,
    selectedTotal: state[storeName].productList.length,
    selectedProductList: state[storeName].selectedProductList,
    current: state[storeName].current,
    pageSize: state[storeName].pageSize,
    productType: state[storeName].sKey_productTypeId,
  };
};

export default connect(mapStateToProps)(AddProduct);
