import React, { PropTypes } from 'react';
import { connect } from 'dva';
import style from './index.less';
import ShowMore from '../../../components/ShowMore';
import moment from 'moment';

const MoreInfo = (props) => {
  const prefix = 'MoreInfo';
  return (
    <div className={style[prefix]}>
      <div className={style[`${prefix}-label`]}>{props.label}</div>
      <div className={style[`${prefix}-text`]}>
        <p>{props.text || '-'}</p>
      </div>
    </div>
  );
};
MoreInfo.propTypes = {
  label: PropTypes.string.isRequired,
  text: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
};
const DelBtn = (props) => {
  const prefix = 'DelBtn';
  return (
    <div className={style[prefix]} onClick={props.onClick} title="删除">
      {' '}
    </div>
  );
};
DelBtn.propTypes = {
  onClick: PropTypes.func.isRequired,
};

class SelectedProductCard extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      showMore: false,
    };
    this.showMoreInfo = this.showMoreInfo.bind(this);
  }
  static classPrefix = 'SelectedProductCard';
  componentDidMount() {}
  componentWillUnmount() {}

  showMoreInfo() {
    this.setState((prevState) => ({
      showMore: !prevState.showMore,
    }));
  }

  render() {
    const {
      data: {
        productName,
        productTypeName,
        validEndDate,
        className,
        classTypeName,
        classLevel,
        teacherName,
        price,
        courseNatureHours,
        totalCourseNatureHours,
        campusName,
        otherOfCategoryId,
        classTimeRange,
        firstSchoolTime,
      },
      onBtnClick,
    } = this.props;
    const { showMore } = this.state;

    const prefix = SelectedProductCard.classPrefix;

    return (
      <div className={style[prefix]}>
        <div className={style[`${prefix}-header`]}>
          <div className={style[`${prefix}-title`]}>
            <span className={style[`${prefix}-type`]}>{productTypeName}</span>
            <span>{productName}</span>
          </div>
          {/*精英班关联产品不允许单独删除*/}
          {otherOfCategoryId !== 'ECS_CLASS_FOR_OTHER' && <DelBtn onClick={onBtnClick} />}
        </div>
        <div className={style[`${prefix}-body`]} style={{ maxHeight: showMore ? 300 : 24 }}>
          <MoreInfo label="班级名称" text={className} />
          <MoreInfo label="班级类型" text={classTypeName} key="班级类型" />
          <MoreInfo label="班级层次" text={classLevel} key="班级层次" />
          <MoreInfo label="班级学习中心" text={campusName} key="班级学习中心" />
          <MoreInfo label="单价" text={price} key="单价" />
          <MoreInfo
            label="已上小时"
            text={courseNatureHours && totalCourseNatureHours ? `${courseNatureHours}/${totalCourseNatureHours}` : ''}
            key="已上课时"
          />
          <MoreInfo label="老师" text={teacherName} key="老师" />
          <MoreInfo label="上课日期" text={firstSchoolTime} key="上课日期" />
          <MoreInfo label="上课时间" text={classTimeRange} key="上课时间" />
          <MoreInfo label="截止日期" text={validEndDate} key="截止日期" />
        </div>
        <div className={style[`${prefix}-footer`]}>
          <div className={style[`${prefix}-ShowMore-wrap`]}>
            <ShowMore text="展开更多" activeText="收起更多" activeColor="#aaa" onClick={this.showMoreInfo} />
          </div>
        </div>
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  return {};
};

export default connect(mapStateToProps)(SelectedProductCard);
