/**
 * Created by Haolin on 2019/1/9.
 */
import React from 'react';
import { connect } from 'dva';
import style from './index.less';
import { storeName } from './store/constant.js';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import Modal from 'COMPONENT/Modal';
import AddProduct from './AddProduct';
import SelectedProduct from './SelectedProduct';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

const MainWrap = (props) => {
  const { children, side } = props;
  const prefix = 'MainWrap';
  return (
    <div className={style[prefix]} style={{ height: window.innerHeight - 104 }}>
      <div className={style[`${prefix}-main`]}>{children}</div>
      <div className={style[`${prefix}-side`]}>{side}</div>
    </div>
  );
};

class ModalAddProduct extends React.Component {
  constructor(props) {
    super(props);
    this.modalLoading = this.modalLoading.bind(this);
    this.toggleModal = this.toggleModal.bind(this);
    this.cancelSelectTime = this.cancelSelectTime.bind(this);
    this.addNewSelected = this.addNewSelected.bind(this);
    this.addSubProduct = this.addSubProduct.bind(this);
    this.relatedDel = this.relatedDel.bind(this);
    this.state = {
      name: '',
      loading: false,
    };
    this.modalRef = null;
    //用已选产品数据初始化购物车
    props.dispatch({
      type: `${storeName}/initShopCar`,
      payload: {
        selectedProductStorage: props.oldPageParams.selectedProductStorage,
      },
    });
    //默认年级、学习中心
    props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        sKey_gradeId: props.oldPageParams.sKey_gradeId,
        sKey_campus: props.oldPageParams.sKey_campus,
        isEcs: props.oldPageParams.isEcs,
        temp_productTypeId: props.oldPageParams.sKey_productTypeId,
      },
    });
  }
  static classPrefix = 'ModalAddProduct';
  //旧页面信息提示
  static showOldPageMsg(message) {
    showMessage(message);
  }

  componentDidMount() {
    global.GlobalEmitter.create('contractAddProduct').listen('toggleModal', this.toggleModal);
    global.GlobalEmitter.create('contractAddProduct').listen('cancelSelectTime', this.cancelSelectTime);
    global.GlobalEmitter.create('contractAddProduct').listen('addNewSelected', this.addNewSelected);
    global.GlobalEmitter.create('contractAddProduct').listen('addSubProduct', this.addSubProduct);
    global.GlobalEmitter.create('contractAddProduct').listen('relatedDel', this.relatedDel);
    global.GlobalEmitter.create('contractAddProduct').listen('showMessage', ModalAddProduct.showOldPageMsg);
  }
  componentWillUnmount() {
    global.GlobalEmitter.create('contractAddProduct').remove('toggleModal', this.toggleModal);
    global.GlobalEmitter.create('contractAddProduct').remove('cancelSelectTime', this.cancelSelectTime);
    global.GlobalEmitter.create('contractAddProduct').remove('addNewSelected', this.addNewSelected);
    global.GlobalEmitter.create('contractAddProduct').remove('addSubProduct', this.addSubProduct);
    global.GlobalEmitter.create('contractAddProduct').remove('relatedDel', this.relatedDel);
    global.GlobalEmitter.create('contractAddProduct').remove('showMessage', ModalAddProduct.showOldPageMsg);
  }
  //显示隐藏模态框(旧页面添加产品有二次弹框时需要)
  toggleModal(params) {
    const { dispatch, modalVisible, selectedProductList } = this.props;
    dispatch({
      type: `${storeName}/updateState`,
      payload: {
        modalVisible: !modalVisible,
      },
    });
    //如果同时有更新数据
    if (params) {
      dispatch({
        type: `${storeName}/updateState`,
        payload: {
          selectedProductList: selectedProductList.map((item, index) => {
            //最后一个加上nextTime字段
            if (index === selectedProductList.length - 1) {
              return { ...item, nextTime: params.firstSchoolTime };
            }
            return item;
          }),
        },
      });
    }
  }
  //旧页面取消选择下次上课时间，要把原本添加到购物车的数据删除掉
  cancelSelectTime() {
    const { dispatch, modalVisible, selectedProductStorage, selectedProductList } = this.props;
    dispatch({
      type: `${storeName}/updateState`,
      payload: {
        modalVisible: !modalVisible,
        selectedProductStorage: selectedProductStorage.slice(0, selectedProductStorage.length - 1),
        selectedProductList: selectedProductList.slice(0, selectedProductList.length - 1),
      },
    });
  }
  //旧页面添加成功后通知新页面添加产品
  addNewSelected(tempIndex, tempId, date) {
    this.props.dispatch({
      type: `${storeName}/addNewSelected`,
      payload: {
        tempId,
        index: tempIndex,
        date,
      },
    });
  }
  //旧页面添加附属其他合同产品
  addSubProduct(subProduct) {
    this.props.dispatch({
      type: `${storeName}/addSubProduct`,
      payload: {
        subProduct,
      },
    });
  }
  //旧页面关联删除时，新页面同时删除
  relatedDel(data) {
    this.props.dispatch({
      type: `${storeName}/relatedDel`,
      payload: {
        data,
      },
    });
  }
  // 更新模态框加载状态
  modalLoading(params) {
    this.setState({ loading: params });
  }
  //
  onOk = () => {
    this.props.onClose();
  };

  render() {
    const {
      oldPageParams: { sKey_gradeId },
    } = this.props;
    const prefix = ModalAddProduct.classPrefix;
    return (
      <Modal
        className={style[prefix]}
        title={this.state.name}
        closable={true}
        cancelBtnShow={false}
        loading={this.state.loading}
        toggle={true}
        visible={this.props.modalVisible}
        onOk={this.onOk}
        onCancel={this.props.onClose}
        ref={(ref) => (this.modalRef = getWrappedInstance(ref))}
        widthSize="XL"
        themeConfig={{
          bgc: 'gray',
          footerTopShadow: true,
        }}
      >
        <MainWrap side={<SelectedProduct modalLoading={this.modalLoading} />}>
          <AddProduct modalLoading={this.modalLoading} sKey_gradeId={sKey_gradeId} />
        </MainWrap>
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    selectedProductStorage: state[storeName].selectedProductStorage,
    selectedProductList: state[storeName].selectedProductList,
    modalVisible: state[storeName].modalVisible,
  };
};

export default connect(mapStateToProps)(ModalAddProduct);
