/**
 * Created by Haolin<haolinhom@gmail.com> on 2019/1/9.
 */
import { storeName } from './constant.js';
import { clone } from 'lodash';
import { findProduct, getCurrentBranchTeacher } from 'SERVICE/contract';
import { getAllOrganizationForSelection, getProductTree, getProductTypes } from 'SERVICE/common';
import {
  ONE_ON_ONE_COURSE,
  SMALL_CLASS,
  ECS_CLASS,
  OTHERS,
  ONE_ON_MANY,
  TWO_TEACHER,
  LECTURE,
  LIVE,
} from 'CONST/dicts/product';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { getDictValByText, getBLoYearId } from 'UTIL/dict';
import moment from 'moment';

/**
 * 根据后端接口要求处理请求数据
 * @param before
 */
const filterRequestParam = (before) => {
  const {
    sKey_productName,
    sKey_className,
    sKey_classTeacherId,
    sKey_campus,
    isEcs,
    sKey_productTypeId,
    sKey_classTypeId,
    sKey_productSeriesId,
    sKey_yearId,
    sKey_phaseId,
    sKey_quarterId,
    sKey_gradeId,
    sKey_productGroup,
    sKey_subject,
    current,
    pageSize,
    sKey_classLevelId,
    sKey_oneOnManyType,
  } = before;
  const ret = {
    page: current,
    rows: pageSize,
  };
  //课程名称的字段名，不知道是什么东西，后端逻辑依赖此，所以先保持和旧代码一致
  ret['filter_LIKE_lectureClass.lectureName'] = sKey_className;
  ret['filter_LIKE_miniClass.name'] = sKey_className;
  ret['filter_LIKE_twoTeacherClassTwo.name'] = sKey_className;
  ret['filter_LIKE_product.name'] = sKey_productName; //产品名称
  ret.isEcsProductCategory = sKey_productTypeId === ECS_CLASS ? 'isEcs' : 'isNotEcs';
  ret['miniClass.blCampusId'] = sKey_campus; //学习中心id
  ret['product.category'] = sKey_productTypeId; //产品类型
  ret['product.classTypeId'] = sKey_classTypeId; //班级类型
  ret['product.courseSeriesId'] = sKey_productSeriesId; //产品系列
  ret['product.gradeId'] = sKey_gradeId; //年级
  ret['product.productGroupId'] = sKey_productGroup; //产品组
  ret['product.productQuarterId'] = sKey_quarterId; //产品季度
  ret['product.productVersionId'] = sKey_yearId; //产品年份
  ret['product.subjectId'] = sKey_subject; //科目
  ret['product.oneOnManyType'] = sKey_oneOnManyType; //一对多
  ret.phaseId = sKey_phaseId; //期数
  ret.teacherId = sKey_classTeacherId; //老师
  ret.isEcsProductCategory = isEcs ? 'isEcs' : 'isNotEcs'; //是否精英班
  ret.classLevel = sKey_classLevelId;
  return ret;
};
/**
 * 根据新页面数据结构处理数据
 * @param before
 * @returns {{}}
 */
const filterResponse = (before) => {
  //有id则为合同产品
  if (before.id) {
    return formatContractProduct(before);
  } else {
    return formatProductListItem(before);
  }
};
//格式化选择产品列表的数据结构
const formatProductListItem = (before) => {
  //产品的基础数据，不包含班级数据
  const basicData = before.product || before.productVo;
  //处理后的数据
  let ret = {
    ...before,
    productName: basicData.name,
    productTypeId: basicData.category,
    productTypeName: basicData.categoryName,
    productId: basicData.id,
    gradeId: basicData.gradeId,
    startDate: basicData.validStartDate,
    validEndDate: basicData.validEndDate,
    classTypeName: basicData.classTypeName,
    price: basicData.price,
    nextTime: null,
    firstSchoolTime: before.firstSchoolTime || null,
    otherOfCategoryId: basicData.otherOfCategoryId,
    //下面这部分数据一般是undefined，要从班级数据取
    className: basicData.className,
    campusName: basicData.campusName,
    teacherName: basicData.teacherName,
    peopleNum: basicData.peopleNum,
    peopleQuantity: basicData.peopleQuantity,
    consumeHour: basicData.consumeHour,
    totalHour: basicData.totalHour,
    classTime: basicData.classTime,
    classTimeLength: basicData.classTimeLength,
    classLevel: before.classLevelName,
  };
  //返回数据有很多时候为null，先过滤一遍
  const excludeNull = {
    lectureClass: before.lectureClass || {},
    miniClass: before.miniClass || {},
    twoTeacherClassTwo: before.twoTeacherClassTwo || {},
  };

  switch (basicData.category) {
    //讲座
    case LECTURE: {
      const lt = excludeNull.lectureClass;
      ret = {
        ...ret,
        nextTime: lt.startDate || basicData.firstSchoolTime,
        className: lt.lectureName,
        classId: lt.lectureId,
        peopleNum: lt.nowMembers,
        peopleQuantity: lt.members,
      };
      break;
    }
    //精英班
    case ECS_CLASS: {
      ret = {
        ...ret,
        totalClassHours: basicData.miniClassTotalhours,
        //要处理miniClassTotalhours字段 这个是总课时
        totalHour: Number(basicData.miniClassTotalhours),
      };
      break;
    }
    //小班
    case SMALL_CLASS: {
      const sc = excludeNull.miniClass;
      ret = {
        ...ret,
        nextTime: sc.nextMiniClassTime || sc.startDate,
        totalHour: Number(sc.totalClassHours),
        className: sc.name,
        classId: sc.miniClassId,
        consumeHour: sc.consume,
        peopleQuantity: sc.peopleQuantity,
        peopleNum: sc.miniClassPeopleNum,
        teacherName: sc.teacherName,
        teacherId: sc.teacherId,
        campusName: sc.blCampusName,
        classCreateDate: sc.startDate,
        classTime: sc.classTime,
        classTimeLength: sc.classTimeLength,
        classTimeRange: getClassTimeRange(sc),
      };
      break;
    }
    //双师
    case TWO_TEACHER: {
      const tt = excludeNull.twoTeacherClassTwo;
      ret = {
        ...ret,
        nextTime: tt.nextTwoTeacherClassTime || tt.startCourseDate,
        totalHour: Number(tt.totalHours),
        className: tt.name,
        classId: tt.twoTeacherClassId,
        consumeHour: tt.consume,
        peopleQuantity: tt.peopleQuantity,
        peopleNum: tt.twoTeacherClassPeopleNum,
        teacherName: tt.teacherName,
        teacherId: tt.teacherId,
        campusName: tt.blCampusName,
        classCreateDate: tt.startCourseDate,
        twoTeacherCreateContractId: tt.id, //辅班
        classTimeRange: getClassTimeRange(tt),
      };
      break;
    }
  }
  return ret;
};
//格式化修改合同时合同产品的数据结构
const formatContractProduct = (before) => {
  return {
    productTypeName: before.productTypeName,
    productName: before.productionName,
    className: before.miniClassName || before.twoClassName || before.lectureName || before.escClassName,
    classTypeName: before.productVo.classTypeName,
    campusName: before.productVo.organizationName,
    price: before.price,
    consumeHour: 0,
    totalHour: 0,
    teacherName: before.classTeacherName,
    nextTime: before.nextTime,
    firstSchoolTime: before.firstSchoolTime,
    classTime: before.classTime,
    classTimeLength: before.productVo.classTimeLength,
    validEndDate: before.productVo.validEndDate,
    otherOfCategoryId: before.productVo.otherOfCategoryId,
    classLevel: before.classLevelName,
  };
};

function getClassTimeRange({ classTime, classTimeLength, everyCourseClassNum }, prdType) {
  if (classTime) {
    if (classTimeLength && everyCourseClassNum) {
      return `${classTime}~${moment(classTime, 'HH:mm')
        .add(classTimeLength * everyCourseClassNum, 'm')
        .format('HH:mm')}`;
    }
    return classTime;
  }
  return null;
}

export default {
  namespace: storeName,
  state: {
    // 临时数据
    temp_productTypeId: null,

    isEcs: false, //是否精英班(星火)
    sKey_productName: '',
    sKey_className: '',
    sKey_classTeacherId: null,
    sKey_campus: null,
    sKey_productTypeId: null,
    sKey_classTypeId: null,
    sKey_productSeriesId: null,
    sKey_yearId: null,
    sKey_quarterId: null,
    sKey_gradeId: null,
    sKey_productGroup: null,
    sKey_subject: null,
    sKey_classLevelId: null,
    // 产品列表源数据
    originProductList: [],
    // 已选产品源数据存储
    selectedProductStorage: [],
    // 产品列表(展示用)
    productList: [],
    // 当前条件产品总数
    productTotal: 0,
    // 已选产品列表(展示用)
    selectedProductList: [],
    //老师字典
    teacherDict: [],
    //学习中心字典
    campusDict: [],
    //产品类型，产品系列，班级类型树形数据
    productDicts: [],
    //当前页
    current: 1,
    pageSize: 10,
    modalVisible: true, //模态框显示隐藏
    productTypes: [], //该环境下拥有的产品类型
  },
  //为了兼容旧接口，所有请求和响应都要先处理数据。购物车数据需要保留一份旧页面所需的数据结构(和旧页面一样带有tempId)
  reducers: {
    updateState(state, { payload }) {
      return {
        ...state,
        ...payload,
      };
    },
    //初始化购物车：修改合同时打开弹框，或者已选产品之后再次打开弹框
    initShopCar(state, { payload: { selectedProductStorage } }) {
      const newOrigin = clone(selectedProductStorage);
      const filterList = newOrigin.map((item) => filterResponse(item));
      return {
        ...state,
        selectedProductList: filterList,
        selectedProductStorage: newOrigin,
      };
    },
    //新模态框添加已选产品数据
    addNewSelected(state, { payload: { index, tempId, date } }) {
      //新页面数据
      let selectedList = clone(state.selectedProductList);
      if (date) {
        state.productList[index].firstSchoolTime = date;
        state.originProductList[index].firstSchoolTime = date;
      }
      selectedList.push({ ...state.productList[index], tempId });
      //旧页面数据
      let originSelected = clone(state.selectedProductStorage);
      originSelected.push({ ...state.originProductList[index], tempId });
      return {
        ...state,
        selectedProductList: selectedList,
        selectedProductStorage: originSelected,
      };
    },
    //添加附属的其他合同产品到购物车
    addSubProduct(state, { payload: { subProduct } }) {
      //新页面数据
      let selectedList = clone(state.selectedProductList);
      selectedList.push(formatProductListItem(subProduct));
      //旧页面数据
      let originSelected = clone(state.selectedProductStorage);
      originSelected.push(subProduct);
      return {
        ...state,
        selectedProductList: selectedList,
        selectedProductStorage: originSelected,
      };
    },
    //新模态框删除已选产品数据
    delNewSelected(state, { payload: { rowId } }) {
      //通过旧页面数据的tempId和id，找出index
      const index = state.selectedProductStorage.findIndex((row) => row.tempId === rowId || row.id === rowId);

      //如果有，则旧页面通知relatedDel的时候没有删除，需要删除
      if (index > -1) {
        //新页面数据
        let selectedList = clone(state.selectedProductList);
        selectedList.splice(index, 1);
        //旧页面数据
        let originSelected = clone(state.selectedProductStorage);
        originSelected.splice(index, 1);

        return {
          ...state,
          selectedProductList: selectedList,
          selectedProductStorage: originSelected,
        };
      } else {
        return state;
      }
    },
    // 更新产品列表数据
    updateProductList(state, { payload: { list, total } }) {
      const newList = clone(list);
      //处理成新页面的数据结构
      const filterList = newList.map((item) => filterResponse(item));
      return {
        ...state,
        originProductList: list,
        productList: filterList,
        productTotal: total,
      };
    },
    //关联删除
    relatedDel(state, { payload: { data } }) {
      const rowId = data.id || data.tempId;
      //通过旧页面数据的tempId和id，找出index
      const index = state.selectedProductStorage.findIndex((row) => row.tempId === rowId || row.id === rowId);
      //如果有，则旧页面通知relatedDel的时候没有删除，需要删除
      if (index > -1) {
        //新页面数据
        let selectedList = clone(state.selectedProductList);
        selectedList.splice(index, 1);
        //旧页面数据
        let originSelected = clone(state.selectedProductStorage);
        originSelected.splice(index, 1);

        return {
          ...state,
          selectedProductList: selectedList,
          selectedProductStorage: originSelected,
        };
      } else {
        return state;
      }
    },
  },
  effects: {
    // 加载产品列表(源数据)
    *loadProductList({ payload: { callback, handleErr } }, { put, select }) {
      //处理前的筛选数据
      let localParams = yield select((state) => state[storeName]);
      //处理后的筛选数据
      const fitlerParams = filterRequestParam(localParams);
      const rsp = yield findProduct(fitlerParams, true).catch((err) => {
        showMessage(err.text, 'error');
        if (handleErr) handleErr();
      });
      if (rsp && rsp.resultCode === 0) {
        yield put({
          type: 'updateProductList',
          payload: { list: rsp.rows, total: rsp.records },
        });
      }
      if (callback) callback();
    },
    //获取老师字典
    *getTeachers({ payload }, { put, call }) {
      const response = yield call(getCurrentBranchTeacher);
      yield put({
        type: 'updateState',
        payload: {
          teacherDict: response.teachers.map((item) => ({
            text: item.teacherName,
            val: item.teacherId,
            title: item.teacherName,
          })),
        },
      });
    },
    //获取学习中心字典
    *getCampuses({ payload }, { put, call }) {
      const response = yield call(getAllOrganizationForSelection, { orgType: 'CAMPUS' });
      yield put({
        type: 'updateState',
        payload: {
          campusDict: Object.keys(response.value).map((item) => {
            return { text: response.value[item], val: item, title: response.value[item] };
          }),
        },
      });
    },
    //获取产品相关字典
    *getProductDicts({ payload: { callback, handleErr } }, { put, select }) {
      const isEcs = yield select((state) => state[storeName].isEcs); //是否精英班
      const [response, response2] = yield Promise.all([
        getProductTree(),
        getProductTypes({ isEcsProductCategory: isEcs ? 'isEcs' : 'isNotEcs' }), //获取该环境下拥有的产品类型
      ]).catch(() => {
        handleErr();
      });
      callback();
      yield put({
        type: 'updateState',
        payload: {
          productDicts: response.data,
          productTypes: response2.map((item) => item.productType),
        },
      });
    },
    //加载完字典后，初始化默认值
    *initDefault({ payload: { callback } }, { put, select }) {
      const yearDict = yield select((state) => state.dict.year);
      const serverDate = yield select((state) => state.account.user.serverDate);
      const isEcs = yield select((state) => state[storeName].isEcs);
      const sKey_productTypeId = yield select((state) => state[storeName].temp_productTypeId);
      yield put({
        type: 'updateState',
        payload: {
          //默认当前年份
          sKey_yearId: getBLoYearId(yearDict, new Date(serverDate)),
          sKey_productTypeId: isEcs ? 'ECS_CLASS' : sKey_productTypeId,
          temp_productTypeId: null,
        },
      });
      if (callback) callback();
    },
    // 添加产品到购物车
    *addProductToCart({ payload: { index } }, { put, select }) {
      const state = yield select((state) => state[storeName]);
      const tempId = '@temp' + +new Date();
      //老页面先新增，成功再通知新页面新增
      global.GlobalEmitter.create('contractAddProduct').trigger('addProduct', {
        ...state.originProductList[index],
        tempId,
        tempIndex: index,
      });
    },
    /**
     * 添加产品到购物车(带开始上课时间)
     *
     * date 课程开始日期
     * time 课程开始时间(用于兼容一天多讲)
     * realClassHours 实际需要报名的总课时数(用于兼容一天多讲)
     * */
    *addProductWithDate({ payload: { index, date, time, realClassHours } }, { put, select }) {
      const state = yield select((state) => state[storeName]);
      const tempId = '@temp' + +new Date();
      let originPrdList = clone(state.originProductList);
      originPrdList[index].firstSchoolTime = date;
      yield put({
        type: 'updateState',
        payload: {
          originProductList: originPrdList,
        },
      });
      //老页面先新增，成功再通知新页面新增
      global.GlobalEmitter.create('contractAddProduct').trigger(
        'addProductWithDate',
        { ...originPrdList[index], tempId, tempIndex: index },
        date,
        time,
        realClassHours,
      );
    },
    // 从购物车删除产品
    *delProductFromCart({ payload: { index } }, { put, select }) {
      const state = yield select((state) => state[storeName]);
      const item = state.selectedProductStorage[index];
      //老页面也要删除
      global.GlobalEmitter.create('contractAddProduct').trigger('delProduct', item);
      yield put({
        type: 'delNewSelected',
        payload: { rowId: item.id || item.tempId },
      });
    },
  },
};
