/**
 * Created by luoming on 2018/3/3
 */
import React from 'react';
import { connect } from 'dva';
import moment from 'moment';
import styles from './index.less';
import { Page, List, SearchBar, Select, Input, Button, PopConfirm } from 'COMPONENT';
import ShowMessage from 'COMPONENT/utils/globalTips/showMessage';
import AddModal from './viewModal/newAdd';
import EditModal from './viewModal/edit';
import PassWordEdit from './viewModal/passwordEdit';
import { loadCommonDict, loadBranchDict, loadCampusDict } from 'UTIL/dict';
import { getPosList, disabledPos, enabledPos } from 'SERVICE/posEnd';

const { COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;
const values = function (array, key) {
  const res = [];

  (array || []).forEach(function (item) {
    res.push(item[key]);
  });

  return res;
};

class Pos extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false,
      add: false,
      edit: false,
      currentPage: 1,
      totalRecords: 0,
      pageSize: 10,
      list: [],
      modalData: {},
    };
  }

  componentDidMount() {
    this.setState({ loading: true });
    Promise.all([
      loadCommonDict('POS_MACHINE_TYPE'),
      loadCommonDict('POS_TYPE'),
      loadCommonDict('PosMachineStatus'),
      loadBranchDict(),
      this.loadData(),
    ]).then(([typeDict, ditchDict, statusDict, branchDict, data]) => {
      let { records, rows } = data;
      this.setState({
        typeDict,
        ditchDict,
        statusDict,
        branchDict,
        totalRecords: records,
        list: rows,
        loading: false,
      });
    });
  }

  loadData = async (searchKey) => {
    let { name, ditchId, number, branchId, campusId, statusId, /*typeId*/ currentPage, pageSize } = this.state;
    if (searchKey) {
      currentPage = searchKey.currentPage;
      pageSize = searchKey.pageSize;
    }
    let apiParams = {
      posName: name,
      posTypeId: ditchId,
      posNumber: number,
      brenchId: branchId,
      campusId: campusId,
      status: statusId,
      // typeId:typeId,

      _search: false,
      rows: pageSize,
      page: currentPage,
      sidx: 'CREATE_TIME',
      sord: 'desc',
    };
    return await getPosList(apiParams);
  };
  loadList = async (searchKey) => {
    this.setState({ loading: true });
    let { records, rows } = await this.loadData(searchKey);
    this.setState({
      totalRecords: records,
      list: rows,
      loading: false,
      ...searchKey,
    });
  };
  search = async () => {
    let { pageSize } = this.state;
    this.setState({ loading: true, currentPage: 1 });
    let { records, rows } = await this.loadData({ pageSize, currentPage: 1 });
    this.setState({
      totalRecords: records,
      list: rows,
      loading: false,
    });
  };
  reset = () => {
    this.setState({
      name: null,
      ditchId: null,
      number: null,
      branchId: null,
      campusId: null,
      statusId: null,
      typeId: null,
    });
  };

  edit = (record) => {
    let { posNumber } = record;
    this.setState({ edit: true, posNumber });
  };

  setPassWord = (record) => {
    let { posNumber, posMachineId } = record;
    this.setState({ setPassWord: true, posNumber, posMachineId });
  };

  changeBranchData = async (id) => {
    let { branchDict } = this.state;
    let branchArray = branchDict.filter((v) => {
      return v.val === id;
    });
    if (id) {
      let campusDict = await loadCampusDict(id);
      campusDict.unshift(branchArray[0]);
      this.setState({ campusDict, branchId: id, campusId: null });
    } else {
      this.setState({ campusDict: [], branchId: id, campusId: null });
    }
  };
  adjustPos = async (record, type) => {
    this.setState({ loading: true });
    let { posNumber } = record;
    let info;
    if (type === 'disabled') {
      info = '禁用成功！';
      await disabledPos({ posNumber });
    } else {
      info = '激活成功！';
      await enabledPos({ posNumber });
    }
    let { records, rows } = await this.loadData();
    this.setState({ totalRecords: records, list: rows, loading: false });
    ShowMessage(info, 'success');
  };
  onSuccess = () => {
    this.setState({ add: false, setPassWord: false });
    this.search();
  };
  editSuccess = async () => {
    let { records, rows } = await this.loadData();
    this.setState({
      totalRecords: records,
      list: rows,
    });
  };
  render() {
    let {
      loading,
      add,
      edit,
      setPassWord,
      searchExpand,
      name,
      ditchDict,
      ditchId,
      number,
      branchDict,
      branchId,
      campusId,
      campusDict,
      statusDict,
      statusId,
      typeDict,
      /*typeId*/ currentPage,
      totalRecords,
      pageSize,
      list,
      posNumber,
      posMachineId,
    } = this.state;
    let columns = [
      { title: '终端（POS）名称', dataIndex: 'posName', width: COL_WIDTH_L, fixed: 'left' },
      { title: '终端渠道', dataIndex: 'posTypeName', width: COL_WIDTH_L, fixed: 'left' },
      {
        title: '终端（POS）编号',
        dataIndex: 'posNumber',
        width: COL_WIDTH_L,
        render: (text, record) => {
          const isUnionPayPOS = record.posTypeId === 'DAT_unionPay_pos';
          return (
            <span>
              {isUnionPayPOS
                ? `${record.posNumber} [${values(record.posMachineInfoList, 'terminalNo').join(',\n')}]`
                : record.posNumber}
            </span>
          );
        },
      },
      { title: '所属分公司', dataIndex: 'brenchName', width: COL_WIDTH_M },
      { title: '所属学习中心', dataIndex: 'campusName', width: COL_WIDTH_L },
      {
        title: '使用日期',
        width: COL_WIDTH_XL,
        render: (text, record) => {
          let { endTime, startTime } = record;
          let startDate = '';
          let endDate = '';
          if (endTime) {
            endDate = moment(endTime).format('YYYY-MM-DD');
          }
          if (startTime) {
            startDate = moment(startTime).format('YYYY-MM-DD');
          }
          if (!startDate && !endDate) {
            return <span />;
          } else {
            return (
              <span>
                {startDate}至{endDate}
              </span>
            );
          }
        },
      },
      { title: '添加时间', dataIndex: 'createTime', width: COL_WIDTH_XL },
      { title: '添加人', dataIndex: 'createUserName', width: COL_WIDTH_M },
      { title: '状态', dataIndex: 'statusName', width: COL_WIDTH_M },
      {
        title: '操作',
        key: 'action',
        width: COL_WIDTH_L,
        getActionConfig: (text, record, index) => {
          return [{ text: '详情', onClick: () => this.edit(record, index) }];
        },
      },
    ];

    return (
      <Page
        fitWindowHeight={true}
        loading={loading}
        footer={false}
        route={this.props.route}
        className={styles.OTOCourseCheck}
        ref={(c) => (this.pageRef = c)}
        subHeader={
          <div className={styles.searchBar}>
            <SearchBar
              expand={searchExpand}
              onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
              onSearch={this.search}
              onReset={this.reset}
            >
              <Input
                placeholder="请输入终端名称"
                value={name}
                onChange={(e) => this.setState({ name: e.target.value })}
              />
              <Select
                placeholder="终端渠道"
                showSearch={true}
                dict={ditchDict}
                value={ditchId}
                onChange={(val) => this.setState({ ditchId: val })}
              />
              <Input
                placeholder="请输入终端号"
                value={number}
                onChange={(e) => this.setState({ number: e.target.value })}
              />
              <Select
                placeholder="所属分公司"
                showSearch={true}
                dict={branchDict}
                value={branchId}
                onChange={(val) => this.changeBranchData(val)}
              />
              <Select
                placeholder="所属学习中心"
                showSearch={true}
                dict={campusDict}
                value={campusId}
                onChange={(val) => this.setState({ campusId: val })}
              />
              <Select
                placeholder="状态"
                showSearch={true}
                dict={statusDict}
                value={statusId}
                onChange={(val) => this.setState({ statusId: val })}
              />
            </SearchBar>
          </div>
        }
        // headerToolBar={
        //   <Button className={styles.add} onClick={() => this.setState({add: true})}>新增</Button>
        // }
      >
        <List
          dataSource={list}
          columns={columns}
          loading={false}
          pagination={{
            current: currentPage,
            total: totalRecords,
            pageSize,
          }}
          onChange={({ current, pageSize }) => this.loadList({ currentPage: current, pageSize })}
        />

        {add && (
          <AddModal
            ditchDict={ditchDict}
            branchDict={branchDict}
            typeDict={typeDict}
            onSuccess={this.onSuccess}
            onClose={() => this.setState({ add: false })}
          />
        )}
        {edit && (
          <EditModal
            posNumber={posNumber}
            ditchDict={ditchDict}
            typeDict={typeDict}
            branchDict={branchDict}
            editSuccess={this.editSuccess}
            onClose={() => this.setState({ edit: false })}
          />
        )}
        {setPassWord && (
          <PassWordEdit
            posNumber={posNumber}
            posMachineId={posMachineId}
            onSuccess={this.onSuccess}
            onClose={() => this.setState({ setPassWord: false })}
          />
        )}
      </Page>
    );
  }
}

let mapStateToPropps = () => {
  return {};
};

export default connect(mapStateToPropps)(Pos);
