/**
 * Created by luoming on 2018/3/8
 */
import React from 'react';
import { Modal, List, FormItem, Button, PopConfirm } from 'COMPONENT';
import { Row, Col } from 'antd';
import moment from 'moment';
import styles from './index.less';
import edit from './edit.less';
import ShowMessage from 'COMPONENT/utils/globalTips/showMessage';
import Modify from './modify';
import TernimalCode from './terminalCode';
import { findPosByNumber, getPosListByNumber, handlePos } from 'SERVICE/posEnd';

const { COL_WIDTH_M } = List;
const COL_WIDTH_L = 178;
const values = function (array, key) {
  const res = [];

  array.forEach(function (item) {
    res.push(item[key]);
  });

  return res;
};

export default class NewAdd extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      data: {},
      list: [],
      currentPage: 1,
      totalRecords: 0,
      pageSize: 10,
      modify: false,
    };
  }

  componentDidMount() {
    this.loadData();
  }

  shouldComponentUpdate(nextProps, nextState) {
    return nextState !== this.state;
  }

  loadData = () => {
    let { posNumber } = this.props;
    this.setState({ loading: true });
    Promise.all([findPosByNumber({ posNumber }), this.getPosList()]).then(([data, subData]) => {
      let { records, rows, startTime, endTime, brenchName, campusName } = subData;
      this.setState({
        data,
        totalRecords: records,
        list: rows,
        loading: false,
        startTime,
        endTime,
        brenchName,
        campusName,
      });
    });
  };

  getPosList = async (searchKey) => {
    let { posNumber } = this.props;
    let { currentPage, pageSize } = this.state;
    if (searchKey) {
      currentPage = searchKey['currentPage'];
      pageSize = searchKey['pageSize'];
    }
    let apiParams = {
      posNumber,
      _search: false,
      rows: pageSize,
      page: currentPage,
      sidx: 'id',
      sord: 'desc',
    };
    let { records, rows } = await getPosListByNumber(apiParams);
    return { records, rows };
  };
  loadList = async (searchKey) => {
    this.setState({ loading: true, ...searchKey });
    let { records, rows, startTime, endTime, brenchName, campusName } = await this.getPosList(searchKey);
    this.setState({ totalRecords: records, list: rows, loading: false, startTime, endTime, brenchName, campusName });
  };
  deletePos = async (record) => {
    this.setState({ loading: true });
    let { id } = record;
    let { currentPage, pageSize } = this.state;
    let apiParams = { oper: 'del', id };
    await handlePos(apiParams);
    ShowMessage('删除成功', 'success');
    let { records, rows, startTime, endTime, brenchName, campusName } = await this.getPosList({
      currentPage,
      pageSize,
    });
    this.setState({ totalRecords: records, list: rows, loading: false, startTime, endTime, brenchName, campusName });
  };
  edit = async (record) => {
    this.setState({ modify: true, record });
  };
  onSuccess = async () => {
    let { records, rows, startTime, endTime, brenchName, campusName } = await this.getPosList();
    this.setState({ totalRecords: records, list: rows, modify: false, startTime, endTime, brenchName, campusName });
    this.props.editSuccess();
  };

  addTerminalCode = () => {
    this.setState({
      editTerminalCode: true,
    });
  };

  render() {
    let {
      loading,
      data,
      list,
      currentPage,
      totalRecords,
      pageSize,
      modify,
      record,
      startTime,
      endTime,
      brenchName,
      campusName,
      editTerminalCode,
      posTerminalCode,
    } = this.state;
    let { onClose, ditchDict, branchDict, typeDict } = this.props;
    let posMachineInfoList = [];
    const isUnionPayPos = data['posTypeId'] === 'DAT_unionPay_pos';
    if (isUnionPayPos) {
      posMachineInfoList = values(data['posMachineInfoList'], 'terminalNo');
    }

    let columns = [
      { title: '终端名称', dataIndex: 'posName', width: COL_WIDTH_M },
      { title: '终端渠道', dataIndex: 'posTypeName', width: COL_WIDTH_M },
      { title: '终端编号', dataIndex: 'posNumber', width: COL_WIDTH_M },
      { title: '所属公司', dataIndex: 'brenchName', width: COL_WIDTH_M },
      { title: '所属学习中心', dataIndex: 'campusName', width: COL_WIDTH_M },
      {
        title: '使用日期',
        width: COL_WIDTH_L,
        render: (text, record) => {
          let { endTime, startTime } = record;
          let startDate = '';
          let endDate = '';
          if (endTime) {
            endDate = moment(endTime).format('YYYY-MM-DD');
          }
          if (startTime) {
            startDate = moment(startTime).format('YYYY-MM-DD');
          }
          if (!startDate && !endDate) {
            return <span />;
          } else {
            return (
              <span>
                {startDate}至{endDate}
              </span>
            );
          }
        },
      },
      {
        title: '状态',
        width: COL_WIDTH_M,
        render: (text, record) => {
          let { startTime, endTime } = record;
          let currentTime = moment().format('YYYY-MM-DD hh:mm:ss');
          if (moment(currentTime).isBetween(startTime, endTime)) {
            return <span>正在使用</span>;
          } else if (moment(currentTime).isBefore(startTime)) {
            return <span>预计使用</span>;
          } else {
            return <span>已过期</span>;
          }
        },
      },
      { title: '操作人', dataIndex: 'createUserName', width: COL_WIDTH_M },
      { title: '操作时间', dataIndex: 'modifyTime', width: COL_WIDTH_L },
    ];

    return (
      <Modal
        loading={loading}
        onCancel={onClose}
        onOk={this.onOk}
        title="编辑终端（POS）"
        width={1000}
        closable={true}
        okBtnShow={false}
        cancelBtnShow={false}
        className={styles.modal}
        ref={(e) => (this.$modal = e)}
      >
        <div className={edit.top}>
          <div className={edit.title}>详情</div>
          <Row gutter={75}>
            <Col span={12}>
              <FormItem name="终端名称">
                <span>{data['posName']}</span>
              </FormItem>
            </Col>
            {isUnionPayPos && (
              <Col span={12}>
                <FormItem name={'机身序列号(S/N)'}>
                  <span>{data['serialNumber']}</span>
                </FormItem>
              </Col>
            )}
            <Col span={12}>
              <FormItem name={isUnionPayPos ? '登录帐号' : '终端编号'}>
                <span>{data['posNumber']}</span>
              </FormItem>
            </Col>
            {isUnionPayPos && (
              <Col span={12}>
                <FormItem name={'终端编号'}>
                  <span>{posMachineInfoList.join(',\n')}</span>
                </FormItem>
              </Col>
            )}
            <Col span={12}>
              <FormItem name="所属公司">
                <span>{data['brenchName']}</span>
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="所属学习中心">
                <span>{data['campusName']}</span>
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="状态">
                <span>{data['statusName']}</span>
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="使用日期">
                <span>
                  {data['startTime']}
                  {data['startTime'] ? '至' : ''}
                  {data['endTime']}
                </span>
              </FormItem>
            </Col>
          </Row>
        </div>
        <div className={edit.sub + ' popElement'}>
          <div className={edit.wrap}>
            <div className={edit.title}>使用时间</div>
          </div>
          <List
            dataSource={list}
            columns={columns}
            loading={false}
            pagination={{
              current: currentPage,
              total: totalRecords,
              pageSize,
            }}
            onChange={({ current, pageSize }) => this.loadList({ currentPage: current, pageSize })}
          />
        </div>
        {modify && (
          <Modify
            data={data}
            ditchDict={ditchDict}
            branchDict={branchDict}
            typeDict={typeDict}
            record={record}
            onSuccess={this.onSuccess}
            onClose={() => this.setState({ modify: false })}
          />
        )}
        {editTerminalCode && (
          <TernimalCode
            posTerminalCode={posTerminalCode}
            posNumber={data.posNumber}
            saveposTerminalCodeDirectly={true}
            onSuccess={() => {
              this.setState({ editTerminalCode: false });
              this.loadData();
            }}
            onClose={() => {
              this.setState({ editTerminalCode: false });
            }}
          />
        )}
      </Modal>
    );
  }
}
