/**
 * Created by luoming on 2018/3/8
 */
import React from 'react';
import { Modal, FormItem, Select, DatePicker, TimePicker } from 'COMPONENT';
import { Row, Col } from 'antd';
import styles from './index.less';
import localStyles from './modify.less';
import { loadCampusDict } from 'UTIL/dict';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import ShowMessage from 'COMPONENT/utils/globalTips/showMessage';
import { handlePos } from 'SERVICE/posEnd';
import moment from 'moment';

export default class NewAdd extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      posTypeId: '',
      style: '',
      branchId: '',
      campusId: '',
      startDate: '',
      startTime: '00:00',
      endDate: '',
      endTime: '23:59',
      ditchDict: [],
      styleDict: [],
      branchDict: [],
      campusDict: [],
    };
  }

  async componentDidMount() {
    let { record, branchDict } = this.props;

    if (record) {
      let { posTypeId, brenchId, campusId, startTime, endTime /*typeId*/ } = record;
      let branchArray = branchDict.filter((v) => {
        return v.val === brenchId;
      });
      let startDate = moment(startTime).format('YYYY-MM-DD'),
        timeStart = moment(startTime).format('HH:mm'),
        endDate = moment(endTime).format('YYYY-MM-DD'),
        timeEnd = moment(endTime).format('HH:mm');
      let campusDict = await loadCampusDict(brenchId);
      campusDict.unshift(branchArray[0]);

      this.setState({
        posTypeId,
        // typeId,
        branchId: brenchId,
        campusId,
        startDate,
        startTime: timeStart,
        endDate,
        endTime: timeEnd,
        campusDict,
      });
      // } else {
      //   // 新增终端类型默认A类型
      //   this.setState({
      //     typeId: 'POS_MACHINE_TYPE_A',
      //   });
    }
  }

  onOk = () => {
    let {
      onSuccess,
      record,
      data: { posNumber, posName, posPassword, serialNumber },
    } = this.props;
    let { posTypeId, /*typeId*/ branchId, campusId, startDate, startTime, endDate, endTime } = this.state;
    let error = (text) => getWrappedInstance(this.$modal).showOkTips(text);
    if (!posTypeId) {
      error('请选择终端渠道！');
      return;
    }
    // if (!typeId) {
    //   error('请选择终端类型！');
    //   return;
    // }
    if (!campusId) {
      error('请选择所属分公司！');
      return;
    }
    if (!branchId) {
      error('请选择所属学习中心！');
      return;
    }
    if (!startDate) {
      error('请选择开始日期！');
      return;
    }
    if (!startTime) {
      error('请选择开始时间！');
      return;
    }
    if (!endDate) {
      error('请选择结束日期！');
      return;
    }
    if (!endTime) {
      error('请选择结束时间！');
      return;
    }
    this.setState({ loading: true });
    let id = 0;
    if (record) {
      id = record['id'];
    }
    let apiParams = {
      id,
      posName,
      posNumber,
      brenchId: branchId,
      campusId,
      // typeId,
      startDate,
      endDate,
      posPassword,
      serialNumber,
      startDateTime: startTime,
      endDateTime: endTime,
      posTypeId,
    };
    handlePos(apiParams)
      .then(() => {
        this.setState({ loading: false });
        ShowMessage('保存成功', 'success');
        onSuccess();
      })
      .catch(() => {
        this.setState({ loading: false });
      });
  };
  changeBranchData = async (branchId) => {
    let { branchDict } = this.props;
    let branchArray = branchDict.filter((v) => {
      return v.val === branchId;
    });
    this.setState({ branchId, campusId: null });
    if (branchId) {
      let campusDict = await loadCampusDict(branchId);
      campusDict.unshift(branchArray[0]);
      this.setState({ campusDict });
    } else {
      this.setState({ campusDict: [] });
    }
  };
  getDisabledStartDate = (startDate) => {
    let { endDate } = this.state;
    if (!startDate || !endDate) {
      return false;
    }
    return moment(startDate).valueOf() > moment(endDate).valueOf();
  };
  getDisabledEndDate = (endDate) => {
    const { startDate } = this.state;
    if (!endDate || !startDate) {
      return false;
    }
    return moment(endDate).valueOf() < moment(startDate).valueOf();
  };

  render() {
    let { loading, posTypeId, branchId, campusId, startDate, startTime, endDate, endTime, campusDict } = this.state;
    let {
      onClose,
      data: { posName, posNumber },
      ditchDict,
      branchDict,
      record,
    } = this.props;
    return (
      <Modal
        loading={loading}
        onCancel={onClose}
        onOk={this.onOk}
        title={(record ? '修改' : '新增') + '终端（POS）'}
        width={690}
        okBtnShow={true}
        cancelBtnShow={true}
        okText="保存"
        className={styles.modal}
        ref={(e) => (this.$modal = e)}
      >
        <Row gutter={75} className={localStyles.item}>
          <Col span={24}>
            <FormItem name="终端编号" required={true}>
              <span>{posNumber}</span>
            </FormItem>
          </Col>
          <Col span={24}>
            <FormItem name="终端名称" required={true}>
              <span>{posName}</span>
            </FormItem>
          </Col>
          <Col span={24}>
            <FormItem name="终端渠道" required={true}>
              <Select
                dict={ditchDict}
                value={posTypeId}
                placeholder="请选择终端渠道"
                onChange={(val) => this.setState({ posTypeId: val })}
              />
            </FormItem>
          </Col>
          <Col span={24}>
            <FormItem name="所属分公司" required={true}>
              <Select
                dict={branchDict}
                value={branchId}
                placeholder="请选择所属分公司"
                showSearch={true}
                onChange={(val) => this.changeBranchData(val)}
              />
            </FormItem>
          </Col>
          <Col span={24}>
            <FormItem name="所属学习中心" required={true}>
              <Select
                dict={campusDict}
                value={campusId}
                placeholder="请选择所属学习中心"
                showSearch={true}
                onChange={(val) => this.setState({ campusId: val })}
              />
            </FormItem>
          </Col>
          <Col span={24}>
            <FormItem name="使用时间" required={true}>
              <Row gutter={10}>
                <Col span={6}>
                  <DatePicker
                    placeholder="开始日期"
                    value={startDate}
                    onChange={(val) => this.setState({ startDate: val })}
                    disabledDate={(val) => this.getDisabledStartDate(val)}
                  />
                </Col>
                <Col span={5}>
                  <TimePicker value={startTime} onChange={(val) => this.setState({ startTime: val })} />
                </Col>
                <Col span={2} style={{ textAlign: 'center' }}>
                  至
                </Col>
                <Col span={6}>
                  <DatePicker
                    placeholder="结束日期"
                    value={endDate}
                    onChange={(val) => this.setState({ endDate: val })}
                    disabledDate={(val) => this.getDisabledEndDate(val)}
                  />
                </Col>
                <Col span={5}>
                  <TimePicker value={endTime} onChange={(val) => this.setState({ endTime: val })} />
                </Col>
              </Row>
            </FormItem>
          </Col>
        </Row>
      </Modal>
    );
  }
}
