/**
 * Created by luoming on 2018/3/3
 */
import React from 'react';
import { Modal, FormItem, Select, Input, Button, DatePicker } from 'COMPONENT';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import ShowMessage from 'COMPONENT/utils/globalTips/showMessage';
import { Row, Col } from 'antd';
import styles from './index.less';
import { loadCampusDict } from 'UTIL/dict';
import { savePos, savePosMachineInfos } from 'SERVICE/posEnd';
import moment from 'moment';
import md5 from 'js-md5';
import TerminalCode from './terminalCode';
import { struct } from 'cubec';

const _trim = struct.string('trim');

export default class NewAdd extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false,
      posNumber: null,
      posName: null,
      posTypeId: null,
      // typeId: 'POS_MACHINE_TYPE_A', // 终端类型默认A类型
      branchId: null,
      date: null,
      campusDict: [],
      campusId: null,
    };
  }

  onOk = async () => {
    let {
      posNumber,
      posName,
      posTypeId,
      /*typeId*/ branchId,
      campusId,
      endDate,
      startDate,
      posPassword,
      serialNumber,
      posTerminalCode,
    } = this.state;
    let error = (text) => getWrappedInstance(this.$modal).showOkTips(text);

    if (!posNumber) {
      error('请填写终端编号！');
      return;
    }

    if (!posName) {
      error('请填写终端名称！');
      return;
    }

    // 如果是银联pos机
    if (
      posTypeId === 'DAT_unionPay_pos' &&
      (typeof posPassword !== 'string' || posPassword.length !== 8 || !/^[0-9]*$/.test(posPassword))
    ) {
      error('请输入8位有效数字的银联POS机密码！');
      return;
    }

    if (posTypeId === 'DAT_unionPay_pos' && (typeof serialNumber !== 'string' || serialNumber.length < 4)) {
      error('请输入有效的POS机机身序列号-S/N码！');
      return;
    }

    if (posTypeId === 'DAT_unionPay_pos' && (!posTerminalCode || posTerminalCode.length < 1)) {
      error('请至少添加一个与当前POS机对应的终端号');
      return;
    }

    if (!posTypeId) {
      error('请选择终端渠道！');
      return;
    }
    // if (!typeId) {
    //   error('请选择终端类型！');
    //   return;
    // }
    if (!branchId) {
      error('请选择所属分公司！');
      return;
    }
    if (!campusId) {
      error('请选择所属学习中心！');
      return;
    }
    if (!endDate) {
      error('请选择终止时间！');
      return;
    }
    if (!startDate) {
      error('请选择开始时间！');
      return;
    }

    let apiParams = {
      posNumber,
      posName,
      posTypeId,
      posPassword: posPassword ? md5(posPassword) : '',
      // typeId,
      brenchId: branchId,
      campusId,
      startDate,
      endDate,
      serialNumber,
      startDateTime: '00:00',
      endDateTime: '23:59',
    };

    await savePos(apiParams);

    // 新银联POS机需要额外存储多个终端号
    if (posTypeId === 'DAT_unionPay_pos' && posTerminalCode && posTerminalCode.length) {
      let posTerminalCodeJSON = [];
      let posTerminalCodeData = posTerminalCode.map(function (v) {
        return v;
      });
      if (posTerminalCodeData.length) {
        for (let i = 0; i < posTerminalCodeData.length; i++) {
          if (posTerminalCodeData[i]) {
            posTerminalCodeJSON.push({
              posNumber,
              terminalNo: posTerminalCodeData[i],
            });
          }
        }

        await savePosMachineInfos(posTerminalCodeJSON);
      }
    }

    ShowMessage('添加成功！', 'success');
    this.props.onSuccess();
  };

  changeBranchData = async (id) => {
    let { branchDict } = this.props;
    let branchArray = branchDict.filter((v) => {
      return v.val === id;
    });
    this.setState({ branchId: id, campusId: null, loading: true });
    if (id) {
      let campusDict = await loadCampusDict(id);
      campusDict.unshift(branchArray[0]);
      this.setState({ campusDict, loading: false });
    } else {
      this.setState({ campusDict: [], loading: false });
    }
  };

  getDisabledStartDate = (startDate) => {
    let { endDate } = this.state;
    if (!startDate || !endDate) {
      return false;
    }
    return moment(startDate).valueOf() > moment(endDate).valueOf();
  };
  getDisabledEndDate = (endDate) => {
    const { startDate } = this.state;
    if (!endDate || !startDate) {
      return false;
    }
    return moment(endDate).valueOf() < moment(startDate).valueOf();
  };

  render() {
    let {
      loading,
      posNumber,
      posName,
      posTypeId,
      branchId,
      campusDict,
      campusId,
      startDate,
      endDate,
      posPassword,
      serialNumber,
      addTerminalCode,
      posTerminalCode,
    } = this.state;
    let { onClose, ditchDict, branchDict } = this.props;

    return (
      <Modal
        loading={loading}
        onCancel={onClose}
        onOk={this.onOk}
        title="添加新终端（POS）"
        width={820}
        okText="保存"
        okBtnShow={true}
        cancelBtnShow={true}
        className={styles.modal}
        ref={(e) => (this.$modal = e)}
      >
        <Row gutter={75}>
          <Col span={12}>
            <FormItem name="终端渠道" required={true}>
              <Select
                dict={ditchDict}
                value={posTypeId}
                placeholder="请选择终端渠道"
                onChange={(val) => this.setState({ posTypeId: val })}
              />
            </FormItem>
          </Col>
          <Col span={12}>
            <FormItem name="终端名称" required={true}>
              <Input
                placeholder="请输入终端名称"
                value={posName}
                onChange={(e) => this.setState({ posName: e.target.value })}
              />
            </FormItem>
          </Col>
          {posTypeId === 'DAT_unionPay_pos' && [
            <Col span={12} key={'unionPay1'}>
              <FormItem name="机身序列号" required={true}>
                <Input
                  placeholder="请输入POS机的出厂S/N码"
                  value={serialNumber}
                  onChange={(e) =>
                    this.setState({ serialNumber: _trim(e.target.value || ''), posNumber: _trim(e.target.value || '') })
                  }
                />
              </FormItem>
            </Col>,
            <Col span={12} key={'unionPay2'}>
              <FormItem name="设置密码" required={true}>
                <Input
                  placeholder="请设数字类型的POS机密码"
                  type={'password'}
                  value={posPassword}
                  maxLength={8}
                  onChange={(e) => this.setState({ posPassword: _trim(e.target.value || '') })}
                />
              </FormItem>
            </Col>,
          ]}
          <Col span={12}>
            <FormItem name={posTypeId === 'DAT_unionPay_pos' ? '登录帐号' : '终端编号'} required={true}>
              <Input
                placeholder="请输入终端编号"
                value={posNumber}
                onChange={(e) => this.setState({ posNumber: _trim(e.target.value || '') })}
              />
            </FormItem>
          </Col>
          <Col span={12}>
            <FormItem name="所属分公司" required={true}>
              <Select
                dict={branchDict}
                value={branchId}
                placeholder="请选择所属分公司"
                showSearch={true}
                onChange={(val) => this.changeBranchData(val)}
              />
            </FormItem>
          </Col>
          <Col span={12}>
            <FormItem name="所属学习中心" required={true}>
              <Select
                dict={campusDict}
                value={campusId}
                placeholder="请选择所属学习中心"
                showSearch={true}
                onChange={(val) => this.setState({ campusId: val })}
              />
            </FormItem>
          </Col>
          <Col span={12}>
            <FormItem name="起始时间" required={true}>
              <DatePicker
                value={startDate}
                onChange={(val) => this.setState({ startDate: val })}
                disabledDate={(val) => this.getDisabledStartDate(val)}
              />
            </FormItem>
          </Col>
          <Col span={12}>
            <FormItem name="终止时间" required={true}>
              <DatePicker
                value={endDate}
                onChange={(val) => this.setState({ endDate: val })}
                disabledDate={(val) => this.getDisabledEndDate(val)}
              />
            </FormItem>
          </Col>
          {posTypeId === 'DAT_unionPay_pos' && (
            <Col span={24}>
              <Button
                onClick={() => {
                  this.setState({ addTerminalCode: true });
                }}
              >
                +添加终端号
              </Button>
            </Col>
          )}
          {addTerminalCode && (
            <TerminalCode
              posTerminalCode={posTerminalCode}
              onSuccess={(posTerminalCode) => {
                this.setState({ addTerminalCode: false, posTerminalCode });
              }}
              onClose={() => {
                this.setState({ addTerminalCode: false });
              }}
            />
          )}
        </Row>
      </Modal>
    );
  }
}
