import React from 'react';
import Modal from 'COMPONENT/Modal';
import FormItem from 'COMPONENT/FormItem';
import Input from 'COMPONENT/Input';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import ShowMessage from 'COMPONENT/utils/globalTips/showMessage';
import { Row, Col } from 'antd';
import styles from './index.less';
import { setPosPassWord } from 'SERVICE/posEnd';
import md5 from 'js-md5';

export default class PassWordEdit extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false,
      posPassword: '',
    };
  }

  onOk = async () => {
    let { posPassword } = this.state;
    let { posNumber, posMachineId } = this.props;
    let error = (text) => getWrappedInstance(this.$modal).showOkTips(text);

    // 如果是银联pos机
    if (typeof posPassword !== 'string' || posPassword.length !== 8 || !/^[0-9]*$/.test(posPassword)) {
      error('请输入8位有效数字的银联POS机密码！');
      return;
    }

    let apiParams = {
      posNumber,
      id: posMachineId,
      posPassword: md5(posPassword),
    };

    await setPosPassWord(apiParams);

    ShowMessage('修改成功！', 'success');
    this.props.onSuccess();
  };

  render() {
    let { loading, posPassword } = this.state;
    let { onClose } = this.props;

    return (
      <Modal
        loading={loading}
        onCancel={onClose}
        onOk={this.onOk}
        title="设置密码（POS）"
        width={520}
        okText="保存"
        okBtnShow={true}
        cancelBtnShow={true}
        className={styles.modal}
        ref={(e) => (this.$modal = e)}
      >
        <Row gutter={75}>
          <Col span={24}>
            <FormItem name="设置新密码" required={true}>
              <Input
                type={'password'}
                maxLength={8}
                placeholder="请输入8位纯数字密码"
                value={posPassword}
                onChange={(e) => this.setState({ posPassword: e.target.value })}
              />
            </FormItem>
          </Col>
        </Row>
      </Modal>
    );
  }
}
