/**
 * Created by luoming on 2018/3/8
 */
import React from 'react';
import { Modal, FormItem, Input, Button } from 'COMPONENT';
import { Row, Col } from 'antd';
import styles from './index.less';
import localStyles from './modify.less';
import { findAllPosMachineInfoByPosNumber, savePosMachineInfos } from 'SERVICE/posEnd';
import ShowMessage from 'COMPONENT/utils/globalTips/showMessage';
import { struct } from 'cubec';

const _values = struct.values();
const _cool = struct.cool();

const makeArray = function (len) {
  const source = new Array(len);

  for (let i = source.length; i--; ) {
    source[i] = undefined;
  }

  return source;
};

const fliterEmpty = function (array) {
  return array.filter(_cool);
};

const checkUniqueValue = function (array) {
  let check = false;

  for (let i = 0; i < array.length; i++) {
    let val = array[i];

    if (array.lastIndexOf(val) > i) {
      check = true;
      break;
    }
  }

  return check;
};

export default class TerminalCode extends React.Component {
  constructor(props) {
    super(props);

    const { posTerminalCode } = this.props;

    this.state = {
      posTerminalCode: posTerminalCode || [],
    };
  }

  async componentDidMount() {
    if (this.props.saveposTerminalCodeDirectly) {
      this.setState({ loading: true });

      const { posNumber } = this.props;
      const res = await findAllPosMachineInfoByPosNumber({ posNumber });
      const posTerminalCode = _values(res || [], 'terminalNo');

      this.setState({ loading: false, posTerminalCode, prevposTerminalCode: posTerminalCode.slice() });
    }
  }

  onOk = async () => {
    const { posTerminalCode } = this.state;

    if (this.props.saveposTerminalCodeDirectly) {
      const { posNumber } = this.props;
      const { prevposTerminalCode } = this.state;
      const len = prevposTerminalCode ? prevposTerminalCode.length : 0;

      if (checkUniqueValue(posTerminalCode)) {
        return ShowMessage('请勿添加重复的终端号!');
      }

      let currentposTerminalCode = fliterEmpty(posTerminalCode.slice(len));
      currentposTerminalCode = currentposTerminalCode.map(function (terminalNo) {
        return {
          posNumber,
          terminalNo,
        };
      });

      if (currentposTerminalCode.length) {
        await savePosMachineInfos(currentposTerminalCode);

        ShowMessage('修改成功!', 'success');
      }
    }

    this.props.onSuccess(posTerminalCode);
  };

  render() {
    const { onClose, saveposTerminalCodeDirectly } = this.props;
    const { loading, record, posTerminalCode, prevposTerminalCode } = this.state;
    const maps =
      posTerminalCode.length > 1 ? posTerminalCode : posTerminalCode.concat(makeArray(2 - posTerminalCode.length));

    return (
      <Modal
        loading={loading}
        onCancel={onClose}
        onOk={this.onOk}
        title={(record ? '修改' : '新增') + '终端号'}
        width={690}
        okBtnShow={true}
        cancelBtnShow={true}
        okText={saveposTerminalCodeDirectly ? '保存' : '确定'}
        className={styles.modal}
        ref={(e) => (this.$modal = e)}
      >
        <Row gutter={75} className={localStyles.item}>
          <Col span={24}>
            <div style={{ color: 'red', marginBottom: '20px' }}>
              请在POS机的 [设置->终端信息->子应用信息]
              中查看每种方式对应的终端号，[银行卡收款]对应刷卡支付方式，[POS通]对应支付宝微信等第三方支付功能
            </div>
          </Col>
          {maps.map((val, index) => (
            <Col span={24} key={index}>
              <FormItem name={`终端号${index}`} required={!index && !saveposTerminalCodeDirectly}>
                {saveposTerminalCodeDirectly &&
                prevposTerminalCode &&
                val &&
                prevposTerminalCode.indexOf(val) === index ? (
                  <div>{val}</div>
                ) : (
                  <Input
                    type={'text'}
                    placeholder={'请输入纯数字类型的终端号'}
                    value={val}
                    maxLength={18}
                    onChange={(e) => {
                      // 去除空格
                      maps[index] = (e.target.value || '').replace(/[^0-9]/g, '');
                      this.setState({ posTerminalCode: maps });
                    }}
                  />
                )}
              </FormItem>
            </Col>
          ))}
          {maps.length < 10 && (
            <Col span={24}>
              <Button
                onClick={() => {
                  maps[maps.length] = undefined;
                  this.setState({ posTerminalCode: maps });
                }}
              >
                +添加行
              </Button>
            </Col>
          )}
        </Row>
      </Modal>
    );
  }
}
