import React from 'react';
import Wrapper from 'COMPONENT/Wrapper';
import SearchBar from 'COMPONENT/SearchBar';
import Select from 'COMPONENT/Select';
import Input from 'COMPONENT/Input';
import DatePicker from 'COMPONENT/DatePicker';
import Page from 'COMPONENT/Page';
import List from 'COMPONENT/List';
import { connect } from 'dva';
import { loadBranchDict, loadBranchCampusDict, dealCampus } from 'UTIL/dict';
import Tabs from 'COMPONENT/Tabs';
import { loadCommonDict } from 'UTIL/dict';
import moment from 'moment';
import { loginUser } from 'UTIL/common';
import { remindByProductGradeErrListApi } from './loaders';
import OldModal from 'COMPONENT/OldModal';

class ProductGradeError extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      allCampusDict: [],
      branchDict: [],
      campusDict: [],
      branchId: '',
      campusId: '',
      studentName: '',
      customerName: '',
      customerPhone: '',
      isViewContract: false,
      currentContractId: '', // 需要打开旧版modal的合同id

      total: 0,
      dataSource: [],
      current: 1,
      pageSize: 10,
    };

    this.onBranchIdChange = this.onBranchIdChange.bind(this);
    this.onCampusIdChange = this.onCampusIdChange.bind(this);
    this.onChangeStudentName = this.onChangeStudentName.bind(this);
    this.onChangeCustomerName = this.onChangeCustomerName.bind(this);
    this.onChangeCustomerPhone = this.onChangeCustomerPhone.bind(this);
    this.onReset = this.onReset.bind(this);
    this.onSearch = this.onSearch.bind(this);
    this.showContractModal = this.showContractModal.bind(this);
    this.closeOldModal = this.closeOldModal.bind(this);

    this.columns = [
      { title: '合同编号', dataIndex: 'contractId' },
      { title: '学生姓名', dataIndex: 'studentName' },
      { title: '客户姓名', dataIndex: 'customerName' },
      { title: '联系方式', dataIndex: 'customerPhone' },
      { title: '合同分公司', dataIndex: 'branchName' },
      { title: '合同学习中心', dataIndex: 'campusName' },
      { title: '产品名称', dataIndex: 'productName' },
      { title: '产品年级', dataIndex: 'productGradeName' },
      { title: '学生年级', dataIndex: 'studentGradeName' },
      { title: '签约日期', dataIndex: 'signingTime' },
      {
        title: '操作',
        dataIndex: '',
        render: (current, row) => {
          const { contractId } = row;
          return (
            <span
              onClick={() => {
                this.showContractModal(contractId);
              }}
              style={{ cursor: 'pointer', color: '#6D67FF' }}
            >
              查看合同
            </span>
          );
        },
      },
    ];
  }

  componentDidMount() {
    this.initContent();
  }

  async initContent() {
    this.setState({ loading: true });
    await this.loadDict();
    await this.onSearch();
    this.setState({ loading: false });
  }

  async loadDict() {
    const [branchDict, allCampusDict] = await Promise.all([loadBranchDict(), loadBranchCampusDict()]);
    // let campusDict = this.props.params === undefined ? '' : this.props.params.itemType

    let campusDict = dealCampus(allCampusDict, branchDict[0].val);
    this.setState({
      branchDict,
      allCampusDict,
      campusDict,
      branchId: this.props.params !== undefined ? '' : branchDict[0].val,
      campusId: this.props.params !== undefined ? '' : campusDict[0].val,
    });
  }

  onBranchIdChange(branchId) {
    const { allCampusDict } = this.state;
    const campusDict = dealCampus(allCampusDict, branchId);
    this.setState({ branchId, campusDict, campusId: '', current: 1 });
  }

  onCampusIdChange(campusId) {
    this.setState({ campusId: campusId, current: 1 });
  }

  onChangeStudentName(event) {
    this.setState({ studentName: event.currentTarget.value, current: 1 });
  }

  onChangeCustomerPhone(event) {
    this.setState({ customerPhone: event.currentTarget.value, current: 1 });
  }

  onChangeCustomerName(event) {
    this.setState({ customerName: event.currentTarget.value, current: 1 });
  }

  onReset() {
    const { allCampusDict, branchDict } = this.state;
    const campusDict = dealCampus(allCampusDict, branchDict[0].val);
    this.setState({
      campusDict,
      branchId: this.props.params !== undefined ? '' : branchDict[0].val,
      campusId: this.props.params !== undefined ? '' : campusDict[0].val,
      studentName: '',
      customerName: '',
      customerPhone: '',
      signingTimeStart: '',
      signingTimeEnd: '',
    });
  }

  async loadData(isGetTotal) {
    this.setState({ loading: true });
    const { branchId, campusId, studentName, customerName, customerPhone, current: page, pageSize: rows } = this.state;
    try {
      const { total, dataSource, getTotal } = await remindByProductGradeErrListApi({
        branchId,
        campusId,
        studentName,
        customerName,
        customerPhone,
        page,
        rows,
        getTotal: isGetTotal,
      });
      if (getTotal === false) {
        this.setState({ dataSource, loading: false });
      } else {
        this.setState({ total, loading: false });
      }
    } catch (error) {
      this.setState({ loading: false });
    }
  }

  showContractModal(contractId) {
    this.setState({ currentContractId: contractId, isViewContract: true });
  }

  async onSearch() {
    await this.loadData(true);
    await this.loadData(false);
  }

  closeOldModal() {
    this.setState({ isViewContract: false });
  }

  render() {
    const {
      loading,
      branchDict,
      campusDict,
      branchId,
      campusId,
      studentName,
      customerName,
      customerPhone,
      signingTimeStart,
      signingTimeEnd,
      total,
      current,
      pageSize,
      dataSource,
      currentContractId,
      isViewContract,
    } = this.state;

    return (
      <Page loading={loading} title="产品年级错误明细页">
        <Wrapper
          header={
            <SearchBar onSearch={this.onSearch} onReset={this.onReset}>
              <Select
                dict={branchDict}
                value={branchId}
                placeholder="合同分公司"
                onChange={this.onBranchIdChange}
                showSearch={true}
              />
              <Select
                dict={campusDict}
                value={campusId}
                placeholder="合同学习中心"
                onChange={this.onCampusIdChange}
                showSearch={true}
              />
              <Input value={studentName} placeholder="学生姓名" onChange={this.onChangeStudentName} />
              <Input value={customerName} placeholder="客户姓名" onChange={this.onChangeCustomerName} />
              <Input value={customerPhone} placeholder="联系方式" onChange={this.onChangeCustomerPhone} />
            </SearchBar>
          }
        ></Wrapper>
        <List
          dataSource={dataSource}
          pagination={{
            current,
            total,
            pageSize,
          }}
          columns={this.columns}
          onChange={({ current, pageSize }) => {
            console.log(current, pageSize);
            this.setState({ current, pageSize }, () => {
              this.loadData(false);
            });
          }}
        />
        {isViewContract && (
          <OldModal
            modalUrl="function/contract/contractDetailModal.html"
            params={{ contractId: currentContractId }}
            onClose={this.closeOldModal}
            css="#checkContractAssignClass{display: none;}"
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const pathname = state.routing.locationBeforeTransitions.pathname;
  return {
    params: state.main.pageParams[pathname],
  };
};

export default connect(mapStateToProps, null)(ProductGradeError);
