import React from 'react';
import { connect } from 'dva';
import moment from 'moment';
import { Modal, List } from 'COMPONENT';
import { payStatusDict } from './config';
import FooterButton from './FooterButton';
import { countdownText, tipsStyle } from './util';

const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L } = List;
const dateFormat = 'YYYY-MM-DD HH:mm:ss';

/**
 * 上一次的支付记录弹窗
 *
 * @author wuyihao
 * @props.lastSharePayment {object} 上一次支付记录数据
 */
class ModalLastSharePayment extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      countdown: 10,
    };
    this.rebuildInterval = null;

    this.columns = [
      {
        title: '生成时间',
        dataIndex: 'createTime',
        key: 'createTime',
        width: COL_WIDTH_L,
        render: (text) => {
          return (text && moment(text).format(dateFormat)) || '';
        },
      },
      { title: '分享金额', dataIndex: 'payment', key: 'payment', width: COL_WIDTH_M },
      { title: '分享状态', dataIndex: 'status', key: 'status', width: COL_WIDTH_S, dict: payStatusDict },
      { title: '分享人', dataIndex: 'createUserName', key: 'createUserName', width: COL_WIDTH_M },
    ];
  }

  componentDidMount() {
    this.initContent();
  }

  componentWillUnmount() {
    this.resetInterval();
  }

  initContent = () => {
    this.rebuildInterval = setInterval(() => {
      if (this.state.countdown > 0) {
        this.setState((prevState) => ({ countdown: prevState.countdown - 1 }));
      } else {
        clearInterval(this.rebuildInterval);
      }
    }, 1000);
  };

  resetInterval = () => {
    if (this.rebuildInterval) {
      clearInterval(this.rebuildInterval);
    }
  };

  render() {
    const { lastSharePayment } = this.props;
    const { countdown } = this.state;

    const list = lastSharePayment ? [lastSharePayment] : [];

    return (
      <Modal
        title={
          <div>
            提示<span style={tipsStyle}>(当前已存在分享二维码，是否取消后重新生成?)</span>
          </div>
        }
        loading={this.state.loading}
        onCancel={this.props.onClose}
        okBtnShow={false}
        widthSize="M"
        footer={
          <FooterButton
            rebuildBtnText={`取消并重新生成二维码${countdownText(countdown)}`}
            disabled={this.props.loading || countdown > 0}
            onDetail={this.props.onDetail}
            onRebuild={this.props.onRebuild}
          />
        }
      >
        <List dataSource={list} columns={this.columns} />
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  return {};
};

export default connect(mapStateToProps)(ModalLastSharePayment);
