import React from 'react';
import style from './index.less';
import Modal from 'COMPONENT/Modal';
import Button from 'COMPONENT/Button';
import Icon from 'COMPONENT/Icon';
import { storeName } from '../store/constant';

const classPrefix = 'ModalRebuildQrCode';

const countdownText = (val) => (val === 0 ? '' : `(${val})`);

const ModalFooter = (props) => {
  return (
    <div className="ModalFooter">
      <Button type="default" onClick={props.onRebuild} disabled={props.rebuildDisabled || props.disabled}>
        {props.rebuildBtnText || '重新生成二维码'}
      </Button>
      <Button type="primary" onClick={props.onDetail} disabled={props.disabled}>
        查看已有二维码
      </Button>
    </div>
  );
};

class ModalRebuildQrCode extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      name: '提示',
      countdown: 10,
    };
    this.rebuildInterval = null;
  }
  componentDidMount() {
    const _self = this;
    this.rebuildInterval = setInterval(() => {
      if (this.state.countdown > 0) {
        _self.setState((prevState) => ({ countdown: prevState.countdown - 1 }));
      } else {
        clearInterval(_self.rebuildInterval);
      }
    }, 1000);
  }
  componentWillUnmount() {
    if (this.rebuildInterval) {
      clearInterval(this.rebuildInterval);
    }
  }

  render() {
    return (
      <Modal
        title={this.state.name}
        loading={this.props.loading}
        onClose={this.props.onClose}
        widthSize="S"
        footer={
          <ModalFooter
            disabled={this.props.loading}
            onDetail={this.props.onDetail}
            onRebuild={this.props.onRebuild}
            rebuildBtnText={`重新生成二维码${countdownText(this.state.countdown)}`}
            rebuildDisabled={this.state.countdown > 0}
          />
        }
      >
        <div className={style[`${classPrefix}-row`]}>
          <Icon type="warning" style={{ marginRight: 10 }} size={24} color="#FFB444" />
          <span className={style[`${classPrefix}-tips`]}>前一笔订单正在支付中，是否重新生成二维码？</span>
        </div>
        <div style={{ marginBottom: 10 }}>
          <span className={style[`${classPrefix}-danger`]}>
            注意：如果“重新生成二维码”，即使之前的二维码支付成功，这笔收款也不会生成收款记录，需要联系财务进行人工退款，请谨慎操作！
          </span>
        </div>
      </Modal>
    );
  }
}

export default ModalRebuildQrCode;
