/**
 * Created by Haolin on 2019/5/24.
 */
import React from 'react';
import { connect } from 'dva';
import style from './index.less';
import { storeName } from '../store/constant';
import { buildSharePaymentList, buildPaymentParams, errorsHandling } from '../store/utils';
import { Modal, Button, Radio, List, InputNumber } from 'COMPONENT';
import ModalLastSharePayment from '../ModalLastSharePayment';
import { createSharePayment, selectLastContractSharePayment, cancelContractSharePayment } from 'SERVICE/contract';

const { COL_WIDTH_S, COL_WIDTH_L } = List;
const classPrefix = 'ModalSharePayment';

const ModalFooter = (props) => {
  return (
    <div className={style[`${classPrefix}-footer`]}>
      <Button type="default" onClick={props.onLinkClick} disabled={props.disabled}>
        支付完成订单
      </Button>
      <Button type="primary" onClick={props.onConfirm} disabled={props.confirmDisabled || props.disabled}>
        生成二维码
      </Button>
    </div>
  );
};

const Content = (props) => {
  const styleObj = props.height ? { height: props.height } : null;
  return (
    <div className={style[`${classPrefix}-Content`]} style={styleObj}>
      <div className={style[`${classPrefix}-Content-label`]}>{props.label}</div>
      <div className={style[`${classPrefix}-Content-body`]}>{props.children}</div>
    </div>
  );
};

const TextInfo = (props) => {
  return (
    <div className={style[`${classPrefix}-TextInfo`]}>
      <div className={style[`${classPrefix}-TextInfo-title`]}>{props.title}</div>
      <div className={style[`${classPrefix}-TextInfo-body`]}>{props.children}</div>
    </div>
  );
};

const TextInfoItem = (props) => {
  return (
    <div className={style[`${classPrefix}-TextInfo-item`]}>
      <span className={style[`${classPrefix}-TextInfo-label`]}>{props.label}</span>
      <span>{props.text}</span>
    </div>
  );
};

const PayTypeRadios = (props) => {
  return (
    <div className={style[`${classPrefix}-PayTypeRadios`]}>
      <Radio checked={props.val} onChange={props.onChange}>
        全款
      </Radio>
      <Radio checked={!props.val} onChange={props.onChange}>
        部分收款
      </Radio>
    </div>
  );
};

const TotalAmount = (props) => {
  return (
    <div className={style[`${classPrefix}-TotalAmount`]}>
      <div>合计：</div>
      <div className={style[`${classPrefix}-TotalAmount-money`]}>
        <span>￥</span>
        <span className={style[`${classPrefix}-TotalAmount-num`]}>{props.amount}</span>
      </div>
    </div>
  );
};

function buildColumns(updateFn, disabled) {
  return [
    { title: '产品名称', dataIndex: 'productName', key: 'productName', width: COL_WIDTH_L },
    { title: '课程名称', dataIndex: 'className', key: 'className', width: COL_WIDTH_L },
    { title: '课时数', dataIndex: 'totalCourseTime', key: 'totalCourseTime', width: COL_WIDTH_S },
    { title: '产品金额', dataIndex: 'productAmount', key: 'productAmount', width: COL_WIDTH_S },
    { title: '优惠金额', dataIndex: 'promotionAmount', key: 'promotionAmount', width: COL_WIDTH_S },
    { title: '已收金额', dataIndex: 'paidAmount', key: 'paidAmount', width: COL_WIDTH_S },
    { title: '待收金额', dataIndex: 'notPayAmount', key: 'notPayAmount', width: COL_WIDTH_S },
    {
      title: '本次收款',
      dataIndex: 'amount',
      key: 'amount',
      width: COL_WIDTH_L,
      render: (text, record, index) => (
        <InputNumber
          min={0}
          max={record.notPayAmount}
          value={text}
          onChange={(val) => {
            updateFn(val, index);
          }}
          disabled={disabled}
        />
      ),
    },
  ];
}

class ModalSharePayment extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      name: '分享支付',
      loading: false,
      rebuildLoading: false, // 重新生成二维码loading
      lastSharePayment: null, // 上一次分享支付
      visible_ModalLastSharePayment: false, // 显示上一次支付记录弹窗
    };
    // 分享支付二维码id
    this.sharePaymentId = null;
  }

  componentDidMount() {
    this.init();
  }
  componentWillUnmount() {
    this.beforeDestroy();
  }

  init = async () => {
    const contractId = this.props.oldPageParams.contractId;
    this.setState({
      loading: true,
      name: `分享支付 (${contractId})`,
    });
    await new Promise(this.initContractInfo(contractId));
    this.setState({ loading: false });
  };

  initContractInfo = (contractId) => {
    return (resolve) => {
      this.props.dispatch({
        type: `${storeName}/initContractInfo`,
        payload: {
          contractId,
          callback() {
            resolve();
          },
        },
      });
    };
  };

  updatePayType = () => {
    this.props.dispatch({
      type: `${storeName}/updatePayType`,
      payload: !this.props.payType,
    });
  };

  updateRealPay = (val, index) => {
    this.props.dispatch({
      type: `${storeName}/updatePrdRealPay`,
      payload: { val, index },
    });
  };

  /**
   * 点击生成二维码
   * 1、检查是否有上一条支付记录
   *  a. 若存在支付中的支付记录，则弹窗提示，用户手动取消或打开该二维码进行支付
   *  b. 若不存在则生成二维码
   *
   * @return {Promise<void>}
   */
  sharePayment = async () => {
    this.setState({ loading: true });
    this.sharePaymentId = null;

    const lastSharePayment = await new Promise(this.checkLastContractSharePayment);
    if (lastSharePayment) {
      this.sharePaymentId = lastSharePayment.contractSharePaymentId;
      this.switchModalLastSharePayment();
    } else {
      this.sharePaymentId = await new Promise(this.createSharePayment);
      if (this.sharePaymentId) {
        this.openShareQrCode();
      }
    }

    this.setState({ loading: false });
    this.updateLastSharePayment(lastSharePayment);
  };

  /**
   * 更新上一次支付记录
   */
  updateLastSharePayment = (lastSharePayment = null) => {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: { lastSharePayment },
    });
  };

  /**
   * 检查是否有上一条支付记录
   *
   * @param resolve
   * @return {Promise<void>}
   */
  checkLastContractSharePayment = async (resolve) => {
    const params = { contractId: this.props.contractId };
    const response = await selectLastContractSharePayment(params, true).catch(errorsHandling());

    if (response && response.data) {
      resolve(response.data);
    } else {
      resolve(null);
    }
  };

  /**
   * 取消分享支付
   */
  cancelSharePayment = async (resolve) => {
    const param = { contractSharePaymentId: this.sharePaymentId };
    const response = await cancelContractSharePayment(param, true).catch(errorsHandling());

    if (response && response.data) {
      resolve(true);
    } else {
      resolve(false);
    }
  };

  /**
   * 创建分享支付
   *
   * @param resolve
   * @return {Promise<void>}
   */
  createSharePayment = async (resolve) => {
    let apiParams = buildPaymentParams(
      this.props.contractId,
      this.props.contractPrdList,
      this.props.contractPrdAmountList,
    );
    const rsp = await createSharePayment(apiParams, true).catch(errorsHandling());
    if (rsp && rsp.resultCode === 0) {
      resolve(rsp.data.sharePaymentId);
    } else {
      resolve(null);
    }
  };

  /**
   * 重新生成二维码
   * tips: 需先取消分享支付再生成
   *
   * @return {Promise<void>}
   */
  reCreateSharePayment = async () => {
    this.setState({ rebuildLoading: true });

    const hasCancel = await new Promise(this.cancelSharePayment);
    if (hasCancel) {
      this.updateLastSharePayment(null);
      this.sharePaymentId = await new Promise(this.createSharePayment);
      this.setState({ rebuildLoading: false });
      this.switchModalLastSharePayment();
      this.openShareQrCode();
    } else {
      this.setState({ rebuildLoading: false });
    }
  };

  openShareQrCode = () => {
    this.props.onQrCode(this.sharePaymentId);
  };

  showPayedOrder = () => {
    this.props.onPaidOrder();
  };

  /**
   * 打开上一此分享支付记录弹窗
   */
  switchModalLastSharePayment = () => {
    this.setState((prevState) => ({ visible_ModalLastSharePayment: !prevState.visible_ModalLastSharePayment }));
  };

  beforeDestroy = () => {
    this.props.dispatch({ type: `${storeName}/resetModalSharePayment` });
  };

  /**
   * 关闭弹窗的时候，回传一个是否支付完成的参数`paid`
   * 是否支付完成通过订单列表来判断
   */
  onClose = () => {
    const { onClose, orderList } = this.props;
    const paid = orderList && orderList.length > 0;

    onClose(paid);
  };

  render() {
    const { lastSharePayment } = this.props;

    return (
      <div>
        <Modal
          className={style[classPrefix]}
          title={this.state.name}
          loading={this.state.loading}
          onCancel={this.onClose}
          widthSize="L"
          themeConfig={{ footerTopShadow: true }}
          footer={
            <ModalFooter
              disabled={this.state.loading}
              onConfirm={this.sharePayment}
              confirmDisabled={this.props.realTotalAmount <= 0}
              onLinkClick={this.showPayedOrder}
            />
          }
        >
          <TextInfo title={<Content label="学生姓名">{this.props.studentName}</Content>}>
            <TextInfoItem label="学习中心" text={this.props.campusName} />
            <TextInfoItem label="年级" text={this.props.gradeName} />
            <TextInfoItem label="合同金额" text={this.props.totalAmount} />
            <TextInfoItem label="签约人" text={this.props.signByWho} />
            <TextInfoItem label="合同签订日期" text={this.props.signTime} />
          </TextInfo>
          <Content label="收款方式" height={58}>
            <PayTypeRadios val={this.props.payType} onChange={this.updatePayType} />
          </Content>
          <List dataSource={this.props.list} columns={buildColumns(this.updateRealPay, this.props.payType)} />
          <TotalAmount amount={this.props.realTotalAmount} />
        </Modal>
        {this.state.visible_ModalLastSharePayment && (
          <ModalLastSharePayment
            loading={this.state.rebuildLoading}
            lastSharePayment={lastSharePayment}
            onClose={this.switchModalLastSharePayment}
            onDetail={this.openShareQrCode}
            onRebuild={this.reCreateSharePayment}
          />
        )}
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  const contractPrdList = state[storeName].contractPrdList;
  const contractPrdAmountList = state[storeName].contractPrdAmountList;

  return {
    contractId: state[storeName].contractId,
    payType: state[storeName].payType,
    studentName: state[storeName].studentName,
    campusName: state[storeName].campusName,
    gradeName: state[storeName].gradeName,
    totalAmount: state[storeName].totalAmount,
    signByWho: state[storeName].signByWho,
    signTime: state[storeName].signTime,
    contractPrdList,
    contractPrdAmountList,
    list: buildSharePaymentList(contractPrdList, contractPrdAmountList),
    realTotalAmount: state[storeName].realTotalAmount,
    orderList: state[storeName].orderList,
    lastSharePayment: state[storeName].lastSharePayment,
  };
};

export default connect(mapStateToProps)(ModalSharePayment);
