/**
 * Created by Haolin on 2019/5/27.
 */
import React from 'react';
import { connect } from 'dva';
import { storeName } from './store/constant';
import ModalSharePayment from './ModalSharePayment';
import ModalPaidOrder from './ModalPaidOrder';
import ShareQRCode from 'COMPONENT/ShareQRCode';
import { pre, prd, peiYouPre, peiYou, twoTeacherPre, twoTeacher } from 'CONST/buildEnv';
import { SYSTEM_MARK } from 'CONST/config';
import style from './index.less';

// 分享二维码上显示部分合同信息
const SomeInfo = (props) => {
  const classPrefix = 'SomeInfo';
  return (
    <div className={style[classPrefix]}>
      {props.data.map((item, index) => (
        <span key={index}>
          {item.label}：{item.value}
        </span>
      ))}
    </div>
  );
};

class SharePayment extends React.Component {
  constructor(props) {
    super(props);
    this.init = this.init.bind(this);
    this.checkPayType = this.checkPayType.bind(this);
    this.switchModalPaidOrder = this.switchModalPaidOrder.bind(this);
    this.switchShareQrCode = this.switchShareQrCode.bind(this);
    this.buildShareUrl = this.buildShareUrl.bind(this);
    this.state = {
      visible_ModalSharePayment: false,
      visible_ModalPaidOrder: false,
      visible_ShareQrCode: false,
      shareType: 'new',
    };
    this.shareUrl = '';
    this.qrCodeTipStyle = { fontSize: 16 };
  }
  componentDidMount() {
    this.init();
  }

  async init() {
    // 不再限制AB类型
    this.setState({ visible_ModalSharePayment: true });
  }

  checkPayType(resolve, reject) {
    this.props.dispatch({
      type: `${storeName}/checkPayType`,
      payload: {
        contractId: this.props.oldPageParams.contractId,
        actionPayType: this.props.oldPageParams.actionPayType,
        callback(resultBool) {
          resolve(resultBool);
        },
      },
    });
  }

  switchModalPaidOrder() {
    this.setState((prevState) => ({
      visible_ModalPaidOrder: !prevState.visible_ModalPaidOrder,
    }));
  }

  /**
   * 生成二维码
   *
   * @param qrId 分享支付二维码ID
   * @param shareType new: 生成新的二维码, detail: 查看已支付二维码详情
   */
  switchShareQrCode(qrId, shareType = 'new') {
    this.shareUrl = this.buildShareUrl(qrId);

    if (shareType === 'detail') {
      // 更新详情的shareCodeDetail数据
      const codeDetail = this.props.list.find((listItem) => listItem.sharePaymentId === qrId) || {};
      this.shareCodeDetail = this.formatShareCodeDetail(this.props.contractInfo, codeDetail.paidAmount);
    }

    this.setState(
      (prevState) => ({
        visible_ShareQrCode: !prevState.visible_ShareQrCode,
        shareType,
      }),
      () => {
        if (!this.state.visible_ShareQrCode) {
          this.shareUrl = '';
          this.shareCodeDetail = null;
        }
      },
    );
  }

  // 构建分享链接
  buildShareUrl(qrId) {
    let baseUrl = 'https://cmall-test.xiao100.com/share-payment/cntr_payment_sharing';
    if (process.env) {
      if ([pre, peiYouPre, twoTeacherPre].includes(process.env.BUILD_ENV)) {
        baseUrl = 'https://cmall-pre.xiao100.com/share-payment/cntr_payment_sharing';
      } else if ([prd, peiYou, twoTeacher].includes(process.env.BUILD_ENV)) {
        baseUrl = 'https://cmall.xiao100.com/share-payment/cntr_payment_sharing';
      }
    }
    return `${baseUrl}?sys=${SYSTEM_MARK}&id=${this.props.contractId}&qr=${qrId}`;
  }

  /**
   * 格式化分享支付二维码金额等信息
   *
   * @param contractInfo {array} 分享信息？
   * @param paidAmount {number}
   */
  formatShareCodeDetail = (contractInfo = [], paidAmount = 0) => {
    return contractInfo.map((item) => {
      if (item.label === '支付金额') {
        return {
          ...item,
          value: paidAmount,
        };
      }
      return { ...item };
    });
  };

  render() {
    const { lastSharePayment } = this.props;
    const { shareType } = this.state;

    // 如果是新生成 那么拿原始的数据 不然就拿详情数据
    let shareTitleData = shareType === 'new' ? this.props.contractInfo : this.shareCodeDetail;
    if (lastSharePayment) {
      shareTitleData = this.formatShareCodeDetail(shareTitleData, lastSharePayment.payment);
    }

    const tipRender = (
      <div className={style.tipsWrapper}>
        <div className={style.emptyBlock} />
        <p>如果生成新的二维码，旧的二维码链接将失效！</p>
        <p>请使用微信或支付宝APP扫描二维码再分享发给家长。</p>
      </div>
    );

    return (
      <div>
        {this.state.visible_ModalSharePayment && (
          <ModalSharePayment
            oldPageParams={this.props.oldPageParams}
            onClose={this.props.onClose}
            onPaidOrder={this.switchModalPaidOrder}
            onQrCode={this.switchShareQrCode}
          />
        )}
        {this.state.visible_ModalPaidOrder && (
          <ModalPaidOrder onCancel={this.switchModalPaidOrder} onQrCode={this.switchShareQrCode} />
        )}
        <ShareQRCode
          title="分享支付"
          shareTitle={<SomeInfo data={shareTitleData} />}
          shareText={this.shareUrl}
          visible={this.state.visible_ShareQrCode}
          onCancel={this.switchShareQrCode}
          tips={tipRender}
          tipsStyle={this.qrCodeTipStyle}
          fingerprintVisible={false}
          showShareBtn={false}
          showHDBtn={false}
        />
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    contractId: state[storeName].contractId,
    list: state[storeName].orderList,
    contractInfo: [
      {
        label: '学生姓名',
        value: state[storeName].studentName,
      },
      {
        label: '合同编号',
        value: state[storeName].contractId,
      },
      {
        label: '支付金额',
        value: state[storeName].realTotalAmount,
      },
    ],
    lastSharePayment: state[storeName].lastSharePayment,
  };
};

export default connect(mapStateToProps)(SharePayment);
