/**
 * Created by Haolin<haolinhom@gmail.com> on 2019/5/24.
 */
import { storeName, needCheckPayTypeSys, payTypeA } from './constant';
import { getContractForPaymentById, findInternetPayByContractId, getFundsPayType } from 'SERVICE/contract';
import { errorsHandling } from './utils';
import { clone } from 'lodash';
import NP from 'number-precision';
import { runFunc } from 'UTIL/common';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

export default {
  namespace: storeName,
  state: {
    contractId: null,
    /**
     * 分享支付弹窗
     * */
    payType: true, // 收款方式(true: 全款 false: 部分收款)
    studentName: null,
    campusName: null,
    gradeName: null,
    totalAmount: null,
    signByWho: null,
    signTime: null,
    contractPrdList: [],
    contractInfoOrigin: null,
    contractPrdAmountList: [],
    realTotalAmount: 0,
    prdAmountStorage: null,
    lastSharePayment: null, // 上一次分享支付
    /**
     * 支付完成订单
     * */
    orderList: [],
  },
  reducers: {
    updateState(state, { payload }) {
      return {
        ...state,
        ...payload,
      };
    },
    updateContractInfo(state, { payload: { contractId, contractInfo } }) {
      const contractInfoOrigin = clone(contractInfo);
      let contractPrdList = contractInfo.contractProductVos.map((item) => ({
        id: item.id,
        productId: item.productId,
        productName: item.productionName,
        className: item.className || '-',
        totalCourseTime: item.quantity,
        productAmount: item.payment,
        promotionAmount: item.promotionAmount,
        paidAmount: item.paidAmount,
        notPayAmount: item.realAmount,
        amount: item.realAmount,
      }));
      let contractPrdAmountList = [];
      let total = 0;
      contractPrdList.forEach((item) => {
        contractPrdAmountList.push(item.amount);
        total = NP.plus(total, item.amount);
      });
      return {
        ...state,
        contractId,
        studentName: contractInfo.stuName,
        campusName: contractInfo.blCampusName,
        gradeName: contractInfo.contractGrade,
        totalAmount: contractInfo.totalAmount,
        signByWho: contractInfo.signByWho,
        signTime: contractInfo.signTime,
        contractPrdList,
        contractInfoOrigin,
        contractPrdAmountList,
        realTotalAmount: total,
        prdAmountStorage: {
          list: contractPrdAmountList.slice(),
          total,
        },
      };
    },
    updatePayType(state, { payload }) {
      let _payload = { payType: payload };
      if (payload) {
        _payload.contractPrdAmountList = state.prdAmountStorage.list.slice();
        _payload.realTotalAmount = state.prdAmountStorage.total;
      } else {
        _payload.contractPrdAmountList = state.contractPrdAmountList.map(() => 0);
        _payload.realTotalAmount = 0;
      }
      return {
        ...state,
        ..._payload,
      };
    },
    updatePrdRealPay(state, { payload: { val, index } }) {
      let prdAmountList = clone(state.contractPrdAmountList);
      prdAmountList[index] = val || 0;
      return {
        ...state,
        contractPrdAmountList: prdAmountList,
        realTotalAmount: prdAmountList.reduce((total, current) => NP.plus(total, current)),
      };
    },
    resetModalSharePayment(state) {
      return {
        ...state,
        contractId: null,
        payType: true,
        studentName: null,
        campusName: null,
        gradeName: null,
        totalAmount: null,
        signByWho: null,
        signTime: null,
        contractPrdList: [],
        contractInfoOrigin: null,
        contractPrdAmountList: [],
        realTotalAmount: 0,
        prdAmountListStorage: null,
      };
    },
    updateOrderInfo(state, { payload }) {
      let orderList = payload.map((item, index) => ({
        number: index + 1,
        sharePaymentId: item.payInfo,
        fundPerson: item.creatUserName,
        paidTime: item.finishTime,
        paidAmount: item.amount,
      }));
      return {
        ...state,
        orderList,
      };
    },
    resetModalPaidOrder(state) {
      return {
        ...state,
        orderList: [],
      };
    },
    resetModalRebuildQrCode(state) {
      return {
        ...state,
        sharePaymentId: null,
      };
    },
  },
  effects: {
    *initContractInfo({ payload: { contractId, callback } }, { select, put }) {
      const rspData = yield getContractForPaymentById({ contractId }, true).catch(errorsHandling());
      if (rspData) {
        yield put({
          type: 'updateContractInfo',
          payload: {
            contractId,
            contractInfo: rspData,
          },
        });
      }
      if (callback) callback();
    },
    *initOrderInfo({ payload: { callback } }, { select, put }) {
      const contractId = yield select((state) => state[storeName].contractId);
      const apiParams = { contractId };
      const rsp = yield findInternetPayByContractId(apiParams, true).catch(errorsHandling());
      if (rsp && rsp.resultCode === 0) {
        yield put({
          type: 'updateOrderInfo',
          payload: rsp.data || [],
        });
      }
      if (callback) callback();
    },
    /**
     * 校验收款类型
     *
     * 1.分享支付AB 无需校验合同收款类型(星火、双师、培优)
     * 2.分享支付A只能用于合同收款类型A的(星火、双师)
     * 3.分享支付B只能用于合同收款类型B的(星火、双师)
     *
     * @param contractId {string} 合同id(eduBoss传入)
     * @param actionPayType {string} 按钮支付方式(eduBoss传入)
     * @param callback {function} 回调
     *
     * @param select {function}
     * @param put {function}
     * */
    *checkPayType({ payload: { contractId, actionPayType, callback } }, { select, put }) {
      let notContinue = false;

      if (actionPayType !== 'AB') {
        const rsp = yield getFundsPayType({ contractId }, true).catch(errorsHandling());

        if (rsp && rsp.resultCode === 0) {
          notContinue = rsp.data === '' ? false : rsp.data !== actionPayType;
        }

        if (notContinue) {
          showMessage('请使用同类型收款方式收款！', 'error');
        }
      }
      runFunc(callback, notContinue);
    },
  },
};
