import React from 'react';
import SearchBar from 'COMPONENT/SearchBar';
import Select from 'COMPONENT/Select';
import Input from 'COMPONENT/Input';
import DatePicker from 'COMPONENT/DatePicker';
import Wrapper from 'COMPONENT/Wrapper';
import List from 'COMPONENT/List';
import { dealCampus } from 'UTIL/dict';
import moment from 'moment';
import { getNotMatchFundListApi } from './loaders';

class UnmatchedDataTab extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      branchDict: [], // 分公司字典
      campusDict: [], // 学习中心字典

      startDate: moment().startOf('month').format('YYYY-MM-DD'), // 支付日期起
      endDate: moment().format('YYYY-MM-DD'), // 支付日期止
      branchIdArr: '', // 分公司ID,单选,状态存 string 类型,接口传 array 类型
      campusIdArr: '', // 学习中心ID,单选,状态存 string 类型,接口传 array 类型
      contractId: '', // 合同编号
      studentName: '', // 学生姓名
      chargeUserName: '', // 收款人
      listData: [], // 列表数据

      pagination: {
        current: 1,
        pageSize: 20,
        total: 0,
        pageSizeOptions: ['20', '50', '100'],
      }, // 列表分页设置
    };

    this.getDisabledEndDate = this.getDisabledEndDate.bind(this);
    this.getDisabledStartDate = this.getDisabledStartDate.bind(this);
    this.onBranchIdChange = this.onBranchIdChange.bind(this);
    this.onCampusIdChange = this.onCampusIdChange.bind(this);
    this.onContractIdChange = this.onContractIdChange.bind(this);
    this.onStudentNameChange = this.onStudentNameChange.bind(this);
    this.onChargeUserNameChange = this.onChargeUserNameChange.bind(this);
    this.onSearch = this.onSearch.bind(this);
    this.onReset = this.onReset.bind(this);
    this.clickOnSearch = this.clickOnSearch.bind(this);

    this.columns = [
      { title: '合同编号', dataIndex: 'contractId' },
      { title: '客户姓名', dataIndex: 'customerName' },
      { title: '学生姓名', dataIndex: 'studentName' },
      { title: '所属分公司', dataIndex: 'branchName' },
      { title: '所属学习中心', dataIndex: 'campusName' },
      { title: '年级', dataIndex: 'studentGradeName' },
      { title: '签订人', dataIndex: 'contractSignStaffName' },
      { title: '收款日期', dataIndex: 'receiptDate' },
      { title: '支付方式', dataIndex: 'channel' },
      { title: '支付参考号', dataIndex: 'posId' },
      { title: '收款人', dataIndex: 'chargeUserName' },
      { title: '创建日期', dataIndex: 'transactionDate' },
    ];
  }

  componentDidUpdate(prevProps) {
    if (
      this.props.campusDict.length !== 0 &&
      this.props.branchDict.length !== 0 &&
      prevProps.campusDict.length === 0 &&
      prevProps.branchDict.length === 0
    ) {
      this.onSearch();
    }
  }

  /**
   * 获取要禁用的开始日期值
   * @param  {string} startDate 过滤前的日期值
   * @return {boolean} 是否要禁用
   */
  getDisabledStartDate(startDate) {
    const { endDate } = this.state;
    if (!startDate || !endDate) {
      return false;
    }
    return moment(startDate).valueOf() > moment(endDate).valueOf();
  }

  /**
   * 获取要禁用的结束日期值
   * @param  {string} endDate   过滤前的日期值
   * @return {boolean}      是否要禁用
   */
  getDisabledEndDate(endDate) {
    const { startDate } = this.state;
    if (!endDate || !startDate) {
      return false;
    }
    return moment(endDate).valueOf() < moment(startDate).valueOf();
  }

  onBranchIdChange(branchId) {
    const { allCampusDict } = this.props;
    const campusDict = dealCampus(allCampusDict, branchId);
    this.setState({ branchIdArr: branchId, campusDict, campusIdArr: '' });
  }

  onCampusIdChange(campusId) {
    this.setState({ campusIdArr: campusId });
  }

  onContractIdChange(event) {
    this.setState({ contractId: event.currentTarget.value });
  }

  onStudentNameChange(event) {
    this.setState({ studentName: event.currentTarget.value });
  }

  onChargeUserNameChange(event) {
    this.setState({ chargeUserName: event.currentTarget.value });
  }

  clickOnSearch() {
    this.setState({ pagination: Object.assign({}, this.state.pagination, { current: 1 }) }, () => {
      this.onSearch();
    });
  }

  async onSearch() {
    const { changeLoadingStatus } = this.props;
    let {
      startDate,
      endDate,
      branchIdArr,
      campusIdArr,
      contractId,
      studentName,
      chargeUserName,
      pagination: { current: page, pageSize: rows },
    } = this.state;
    const params = {
      startDate,
      endDate,
      branchIdArr: (branchIdArr && [branchIdArr]) || null,
      campusIdArr: (campusIdArr && [campusIdArr]) || null,
      contractId,
      studentName,
      chargeUserName,
      page,
      rows,
    };
    changeLoadingStatus(true);
    try {
      let { listData, total } = await getNotMatchFundListApi(params);
      this.setState({ listData, pagination: Object.assign({}, this.state.pagination, { total }) });
      changeLoadingStatus(false);
    } catch (err) {
      changeLoadingStatus(false);
    }
    return;
  }

  onReset() {
    this.setState({
      startDate: moment().startOf('month').format('YYYY-MM-DD'), // 支付日期起
      endDate: moment().format('YYYY-MM-DD'), // 支付日期止
      branchIdArr: '', // 分公司ID,单选,状态存 string 类型,接口传 array 类型
      campusIdArr: '', // 学习中心ID,单选,状态存 string 类型,接口传 array 类型
      contractId: '', // 合同编号
      studentName: '', // 学生姓名
      chargeUserName: '', // 收款人
    });
  }

  render() {
    const {
      startDate,
      endDate,
      branchIdArr,
      campusIdArr,
      contractId,
      chargeUserName,
      studentName,
      listData,
      pagination,
      campusDict,
    } = this.state;
    const { branchDict } = this.props;
    return (
      <Wrapper
        header={
          <SearchBar onSearch={this.clickOnSearch} onReset={this.onReset}>
            <DatePicker
              placeholder="支付日期起"
              value={startDate}
              onChange={(val) => this.setState({ startDate: val })}
              disabledDate={this.getDisabledStartDate}
            />
            <DatePicker
              placeholder="支付日期止"
              value={endDate}
              onChange={(val) => this.setState({ endDate: val })}
              disabledDate={this.getDisabledEndDate}
            />
            <Input value={contractId} placeholder="合同编号" onChange={this.onContractIdChange} />
            <Input value={studentName} placeholder="学生姓名" onChange={this.onStudentNameChange} />
            <Select
              dict={branchDict}
              value={branchIdArr}
              placeholder="所属分公司"
              onChange={this.onBranchIdChange}
              showSearch={true}
            />
            <Select
              dict={campusDict}
              value={campusIdArr}
              placeholder="所属学习中心"
              onChange={this.onCampusIdChange}
              showSearch={true}
            />
            <Input value={chargeUserName} placeholder="收款人" onChange={this.onChargeUserNameChange} />
          </SearchBar>
        }
      >
        <List
          dataSource={listData}
          columns={this.columns}
          pagination={pagination}
          onChange={(pagination) => {
            this.setState(
              {
                pagination: Object.assign({}, this.state.pagination, {
                  current: pagination.current,
                  pageSize: pagination.pageSize,
                }),
              },
              () => {
                this.onSearch();
              },
            );
          }}
        />
      </Wrapper>
    );
  }
}

export default UnmatchedDataTab;
