import React from 'react';
import SearchBar from 'COMPONENT/SearchBar';
import Select from 'COMPONENT/Select';
import Input from 'COMPONENT/Input';
import DatePicker from 'COMPONENT/DatePicker';
import Wrapper from 'COMPONENT/Wrapper';
import List from 'COMPONENT/List';
import moment from 'moment';
import { getPosPayDataMatchErrorListApi, updateFundHistoryPosApi } from './loaders';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { dealCampus } from 'UTIL/dict';

class WrongFundHistoryTab extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      startDate: moment().startOf('month').format('YYYY-MM-DD'), // 支付日期起
      endDate: moment().format('YYYY-MM-DD'), // 支付日期止
      branchIdArr: '', // 分公司ID,单选,状态存 string 类型,接口传 array 类型
      campusIdArr: '', // 学习中心ID,单选,状态存 string 类型,接口传 array 类型
      contractId: '', // 合同编号
      errType: this.props.errType || '', // 收款记录错误类型
      chargeUserName: '', // 收款人
      listData: [], // 列表数据
      campusDict: [],

      pagination: {
        current: 1,
        pageSize: 20,
        total: 0,
        pageSizeOptions: ['20', '50', '100'],
      }, // 列表分页设置
    };

    this.getDisabledEndDate = this.getDisabledEndDate.bind(this);
    this.getDisabledStartDate = this.getDisabledStartDate.bind(this);
    this.onBranchIdChange = this.onBranchIdChange.bind(this);
    this.onCampusIdChange = this.onCampusIdChange.bind(this);
    this.onContractIdChange = this.onContractIdChange.bind(this);
    this.onErrTypeChange = this.onErrTypeChange.bind(this);
    this.onChargeUserNameChange = this.onChargeUserNameChange.bind(this);
    this.onSearch = this.onSearch.bind(this);
    this.updateFundHistoryPos = this.updateFundHistoryPos.bind(this);
    this.onReset = this.onReset.bind(this);
    this.clickOnSearch = this.clickOnSearch.bind(this);

    this.columns = [
      { title: '合同编号', dataIndex: 'contractId', width: 130 },
      { title: '所属分公司', dataIndex: 'branchName', width: 113 },
      { title: '所属学习中心', dataIndex: 'campusName', width: 113 },
      { title: '终端号（银行）', dataIndex: 'bankPosNumber', width: 128 },
      {
        title: '终端号（系统）',
        dataIndex: 'sysPosNumber',
        width: 128,
        render: (value, row) => {
          if (row.posNumberErr) {
            return <span style={{ color: '#f00' }}>{value}</span>;
          } else {
            return value;
          }
        },
      },
      {
        title: '支付参考号（银行）',
        dataIndex: 'bankPosId',
        width: 139,
        render: (value) => {
          if (!value) {
            return <span>系统记录无对应参考号</span>;
          } else {
            return value;
          }
        },
      },
      {
        title: '支付参考号（系统）',
        dataIndex: 'sysPosId',
        width: 139,
        render: (value, row) => {
          if (!value) {
            return <span style={{ color: '#f00' }}>系统记录无对应参考号</span>;
          } else if (row.posIdErr) {
            return <span style={{ color: '#f00' }}>{value}</span>;
          } else {
            return value;
          }
        },
      },
      { title: '支付金额（银行）', dataIndex: 'bankAmount', width: 130 },
      {
        title: '支付金额（系统）',
        dataIndex: 'sysAmount',
        width: 130,
        render: (value, row) => {
          if (row.amountErr) {
            return <span style={{ color: '#f00' }}>{value}</span>;
          } else {
            return value;
          }
        },
      },
      { title: '支付日期（银行）', dataIndex: 'bankDate', width: 130 },
      {
        title: '支付日期（系统）',
        dataIndex: 'sysDate',
        width: 130,
        render: (value, row) => {
          if (row.dateErr) {
            return <span style={{ color: '#f00' }}>{value}</span>;
          } else {
            return value;
          }
        },
      },
      { title: '收款人', dataIndex: 'chargeUserName' },
      {
        title: '错误类型',
        dataIndex: 'errType',
        render: (value, row) => {
          return (
            <div>
              {row.posIdErr && <p>支付参考号错误</p>}
              {row.dateErr && <p>支付日期错误</p>}
              {row.posNumberErr && <p>支付终端号错误</p>}
              {row.amountErr && <p>支付金额错误</p>}
            </div>
          );
        },
        width: 114,
      },
      // { title: '操作', render:(value, row)=>{
      //   if(row.dateErr === false){
      //     return (<a onClick={()=>{this.updateFundHistoryPos(row)}}>一键修复</a>)
      //   }
      // } },
    ];
  }

  componentDidUpdate(prevProps) {
    if (
      this.props.campusDict.length !== 0 &&
      this.props.branchDict.length !== 0 &&
      prevProps.campusDict.length === 0 &&
      prevProps.branchDict.length === 0
    ) {
      this.onSearch();
    }
  }

  /**
   * 获取要禁用的开始日期值
   * @param  {string} startDate 过滤前的日期值
   * @return {boolean} 是否要禁用
   */
  getDisabledStartDate(startDate) {
    const { endDate } = this.state;
    if (!startDate || !endDate) {
      return false;
    }
    return moment(startDate).valueOf() > moment(endDate).valueOf();
  }

  /**
   * 获取要禁用的结束日期值
   * @param  {string} endDate   过滤前的日期值
   * @return {boolean}      是否要禁用
   */
  getDisabledEndDate(endDate) {
    const { startDate } = this.state;
    if (!endDate || !startDate) {
      return false;
    }
    return moment(endDate).valueOf() < moment(startDate).valueOf();
  }

  onBranchIdChange(branchId) {
    const { allCampusDict } = this.props;
    const campusDict = dealCampus(allCampusDict, branchId);
    this.setState({ branchIdArr: branchId, campusDict, campusIdArr: '' });
  }

  onCampusIdChange(campusId) {
    this.setState({ campusIdArr: campusId });
  }

  onContractIdChange(event) {
    this.setState({ contractId: event.currentTarget.value });
  }

  onErrTypeChange(errType) {
    this.setState({ errType });
  }

  onChargeUserNameChange(event) {
    this.setState({ chargeUserName: event.currentTarget.value });
  }

  async onSearch() {
    const { changeLoadingStatus } = this.props;
    let {
      startDate,
      endDate,
      branchIdArr,
      campusIdArr,
      contractId,
      errType,
      chargeUserName,
      pagination: { current: page, pageSize: rows },
    } = this.state;
    changeLoadingStatus(true);
    const params = {
      startDate,
      endDate,
      branchIdArr: (branchIdArr && [branchIdArr]) || null,
      campusIdArr: (campusIdArr && [campusIdArr]) || null,
      contractId,
      errType,
      chargeUserName,
      page,
      rows,
    };
    try {
      let { listData, total } = await getPosPayDataMatchErrorListApi(params);
      this.setState({ listData, pagination: Object.assign({}, this.state.pagination, { total }) });
      changeLoadingStatus(false);
    } catch (err) {
      changeLoadingStatus(false);
    }
  }

  clickOnSearch() {
    this.setState({ pagination: Object.assign({}, this.state.pagination, { current: 1 }) }, () => {
      this.onSearch();
    });
  }

  onReset() {
    this.setState({
      startDate: moment().startOf('month').format('YYYY-MM-DD'),
      endDate: moment().format('YYYY-MM-DD'),
      branchIdArr: '',
      campusIdArr: '',
      contractId: '',
      errType: this.props.errType || '',
      chargeUserName: '',
      listData: [],
    });
  }

  async updateFundHistoryPos(row) {
    const {
      fundsChangeHistoryId: id,
      bankPosId: POSid,
      bankPosNumber: posNumber,
      bankDate: posReceiptDate,
      channel,
      fundBlCampusId: blCampusId,
    } = row;
    try {
      await updateFundHistoryPosApi({ id, POSid, posNumber, posReceiptDate, channel, blCampusId });
      showMessage('数据修复成功!', 'success');
    } catch (err) {
      showMessage(err.text);
    }
  }

  render() {
    const {
      startDate,
      endDate,
      branchIdArr,
      campusIdArr,
      contractId,
      errType,
      chargeUserName,
      listData,
      pagination,
      campusDict,
    } = this.state;
    const { branchDict, errTypeDict } = this.props;
    return (
      <Wrapper
        header={
          <SearchBar onSearch={this.clickOnSearch} onReset={this.onReset}>
            <DatePicker
              placeholder="支付日期起"
              value={startDate}
              onChange={(val) => this.setState({ startDate: val })}
              disabledDate={this.getDisabledStartDate}
            />
            <DatePicker
              placeholder="支付日期止"
              value={endDate}
              onChange={(val) => this.setState({ endDate: val })}
              disabledDate={this.getDisabledEndDate}
            />
            <Select
              dict={branchDict}
              value={branchIdArr}
              placeholder="所属分公司"
              onChange={this.onBranchIdChange}
              showSearch={true}
            />
            <Select
              dict={campusDict}
              value={campusIdArr}
              placeholder="所属学习中心"
              onChange={this.onCampusIdChange}
              showSearch={true}
            />
            <Input value={contractId} placeholder="合同编号" onChange={this.onContractIdChange} />
            <Select dict={errTypeDict} value={errType} placeholder="收款记录错误类型" onChange={this.onErrTypeChange} />
            <Input value={chargeUserName} placeholder="收款人" onChange={this.onChargeUserNameChange} />
          </SearchBar>
        }
      >
        <List
          dataSource={listData}
          columns={this.columns}
          pagination={pagination}
          onChange={(pagination) => {
            this.setState(
              {
                pagination: Object.assign({}, this.state.pagination, {
                  current: pagination.current,
                  pageSize: pagination.pageSize,
                }),
              },
              () => {
                this.onSearch();
              },
            );
          }}
        />
      </Wrapper>
    );
  }
}

export default WrongFundHistoryTab;
