import React from 'react';
import Page from 'COMPONENT/Page';
import { connect } from 'dva';
import { loadBranchDict, loadBranchCampusDict, dealCampus } from 'UTIL/dict';
import Tabs from 'COMPONENT/Tabs';
import WrongFundHistoryTab from './WrongFundHistoryTab';
import UnmatchedDataTab from './UnmatchedDataTab';
import { loadCommonDict } from 'UTIL/dict';

const Tab = Tabs.Tab;

class WrongFundHistory extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: true,
      branchDict: [], // 分公司字典
      campusDict: [], // 学习中心字典
      errTypeDict: [], // 错误收款类型字典
      allCampusDict: [], // 用户学习中心按分公司分组
    };

    this.initContent = this.initContent.bind(this);
    this.loadDict = this.loadDict.bind(this);
    this.changeLoadingStatus = this.changeLoadingStatus.bind(this);
  }

  componentDidMount() {
    this.initContent();
  }

  async initContent() {
    await this.loadDict();
  }

  async loadDict() {
    const [branchDict, allCampusDict, errTypeDict] = await Promise.all([
      loadBranchDict(),
      loadBranchCampusDict(),
      loadCommonDict('DONATE_REMIND_ITEM_TYPE', 'DONATE_REMIND_CONTRACT_FUNDS'),
    ]);
    let campusDict = dealCampus(allCampusDict, branchDict[0].val);
    this.setState({ branchDict, allCampusDict, campusDict, errTypeDict });
  }

  changeLoadingStatus(status) {
    this.setState({ loading: status });
  }

  render() {
    const { branchDict, campusDict, errTypeDict, loading, allCampusDict } = this.state;
    return (
      <Page loading={loading} name="错误收款记录">
        <Tabs type="button">
          <Tab title="错误收款记录">
            <WrongFundHistoryTab
              campusDict={campusDict}
              branchDict={branchDict}
              errTypeDict={errTypeDict}
              changeLoadingStatus={this.changeLoadingStatus}
              errType={this.props.params === undefined ? '' : this.props.params.itemType}
              allCampusDict={allCampusDict}
            />
          </Tab>
          <Tab title="未匹配数据">
            <UnmatchedDataTab
              campusDict={campusDict}
              branchDict={branchDict}
              changeLoadingStatus={this.changeLoadingStatus}
              allCampusDict={allCampusDict}
            />
          </Tab>
        </Tabs>
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const pathname = state.routing.locationBeforeTransitions.pathname;
  return {
    params: state.main.pageParams[pathname],
  };
};

export default connect(mapStateToProps, null)(WrongFundHistory);
