import React from 'react';
import { connect } from 'dva';
import { browserHistory } from 'dva/router';
import Button from 'COMPONENT/Button';
import ProductTypeTag from '../ProductTypeTag';
import Icon from 'COMPONENT/Icon';
import IconButton from 'COMPONENT/IconButton';
import Badage from 'COMPONENT/Badage';
import styles from './styles.less';
import { formatCurrency } from 'COMPONENT/utils/basic/num';
import classnames from 'classnames';
import dicts from '../dicts';
import CountInput from '../CountInput';

class Cart extends React.Component {
  state = {
    expand: false,
  };

  componentWillReceiveProps(nextProps) {
    if (this.props.activeTabId !== nextProps.activeTabId) {
      this.setState({ expand: false });
    }
  }

  countChange = (id, count) => {
    const cartList = this.props.cartList.map((d) => {
      if (d.id === id) {
        return { ...d, count };
      }
      return d;
    });
    this.props.onUpdateParams({ cartList });
  };

  remove = (id) => {
    const cartList = this.props.cartList.filter((d) => d.id !== id);
    this.props.onUpdateParams({ cartList });
  };

  goCheckout = () => {
    this.setState({ expand: false });
    this.props.onUpdateParams({ status: dicts.SHOP_STATUS_CHECK });
    if (this.props.activeTabId !== '/AddContractList/Shop') {
      browserHistory.push('/AddContractList/Shop');
    }
  };

  render() {
    const { cartList } = this.props;
    const { expand } = this.state;
    const cls = {
      [styles.wrap]: 1,
      [styles.wrapExpand]: expand,
    };
    return (
      <div className={classnames(cls)}>
        {expand && <div className={styles.mask} onClick={() => this.setState({ expand: false })} />}
        <div className={styles.panel}>
          <div className={styles.content}>
            <div className={styles.header}>
              <div className={styles.col1}>优惠信息</div>
              <div className={styles.col2}>数量</div>
              <div className={styles.col3}>价格</div>
            </div>
            <div className={styles.body}>
              {cartList.map((d) => {
                const { id, type, name, count, price } = d;
                return (
                  <div key={id} className={styles.row}>
                    <div className={styles.col1} style={{ display: 'flex' }}>
                      <div className={styles.tag}>
                        <ProductTypeTag type={type} />
                      </div>
                      <div className={styles.name}>{name}</div>
                    </div>
                    <div className={styles.col2}>
                      {type === dicts.PRODUCT_TYPE_MINI ||
                      type === dicts.PRODUCT_TYPE_TWO ||
                      type === dicts.PRODUCT_TYPE_PROMISE ? (
                        <div className={styles.countText}>{count}</div>
                      ) : (
                        <CountInput value={count} onChange={(val) => this.countChange(id, val)} />
                      )}
                    </div>
                    <div className={styles.col3 + ' ' + styles.price}>{formatCurrency(price * count, '￥')}</div>
                    <div className={styles.close}>
                      <IconButton type={'cross'} onClick={() => this.remove(id)} size={12} />
                    </div>
                  </div>
                );
              })}
            </div>
            <div className={styles.footer}>
              <div className={styles.info}>
                <div className={styles.infoLeft}>{`已选${cartList.length}件`}</div>
                <div className={styles.infoRight}>
                  {formatCurrency(
                    cartList.map((d) => d.price * d.count).reduce((a, b) => a + b, 0),
                    '￥',
                  )}
                </div>
              </div>
              <div className={styles.footerBtn}>
                <Button type={'primary'} onClick={this.goCheckout} disabled={cartList.length === 0}>
                  去购物车结算
                </Button>
              </div>
            </div>
          </div>
        </div>
        <div className={styles.toolbar} onClick={() => this.setState({ expand: !expand })}>
          <Icon className={styles.toolbarIcon} type={'star'} />
          <div className={styles.toolbarText}>购物车</div>
          <div className={styles.toolbarBadage}>
            <Badage>{cartList.length}</Badage>
          </div>
        </div>
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    activeTabId: state.main.activeTabId,
  };
};

export default connect(mapStateToProps)(Cart);
