import React from 'react';
import styles from './styles.less';
import classnames from 'classnames';
import { tr, td1, td2, td3, td4, td5 } from '../styles.less';
import { formatCurrency } from '../../../../../components/utils/basic/num';
import TextLink from '../../../../../components/TextLink/index';
import InputNumber from 'COMPONENT/InputNumber';
import { removeClass } from 'COMPONENT/utils/dom';

class FeeItem extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      countValue: props.count,
      initCountValue: props.count,
    };
  }

  componentWillReceiveProps(nextProps) {
    if (this.props.count !== nextProps.count) {
      this.setState({ countValue: nextProps.count, initCountValue: nextProps.count });
    }
  }

  onCountBlur = () => {
    if (this.state.countValue !== this.state.initCountValue) {
      this.props.onCountChange(this.state.countValue);
    }
    this.setState({ initCountValue: this.state.countValue });
  };

  onCountKeyDown = (e) => {
    if (e.keyCode === 13) {
      document.activeElement.blur();
    }
  };

  render() {
    const { title, price, total, reduces, onCountChange, onDelete } = this.props;
    const { countValue } = this.state;
    const cls = {
      [styles.wrap]: 1,
    };
    return (
      <div className={classnames(cls)}>
        <div className={tr}>
          <div className={td1}>{title}</div>
          <div className={td2}>
            <div className={styles.price}>{formatCurrency(price, '￥')}</div>
          </div>
          <div className={td3}>
            {!!onCountChange ? (
              <InputNumber
                value={countValue}
                onChange={(val) => this.setState({ countValue: val || 1 })}
                onBlur={this.onCountBlur}
                onKeyDown={this.onCountKeyDown}
                min={1}
                precision={0}
              />
            ) : (
              <div className={styles.countText}>{countValue}</div>
            )}
          </div>
          <div className={td4}>
            <div className={styles.totalWrap}>
              <div className={styles.total}>{formatCurrency(total, '￥')}</div>
              <div className={styles.reduces}>
                {!!reduces &&
                  reduces.map((r, rIdx) => {
                    return (
                      <div key={rIdx} className={styles.reduce}>
                        <span>{r.name ? r.name + '：' : null}</span>
                        <span>{`-${formatCurrency(r.amount, '￥')}`}</span>
                      </div>
                    );
                  })}
              </div>
            </div>
          </div>
          <div className={td5}>
            {!!onDelete && (
              <TextLink onClick={onDelete} type={'secondary'}>
                删除
              </TextLink>
            )}
          </div>
        </div>
      </div>
    );
  }
}

export default FeeItem;
