import React from 'react';
import { connect } from 'dva';
import Popover from 'COMPONENT/Popover';
import Radio from 'COMPONENT/Radio';
import LockableInput from '../../LockableInput';
import utils from '../../utils';
import loaders from '../../loaders';
import PromotionModeTag from '../../PromotionModeTag';
import styles from './styles.less';
import Icon from 'COMPONENT/Icon';
import TextLink from 'COMPONENT/TextLink';

class SelectPromotionPop extends React.Component {
  componentDidUpdate(prevProps, prevState) {
    if (this.codeInputRef && !this.props.visible && prevProps.visible) {
      this.codeInputRef.closeEdit();
    }
  }

  toMakeDiscount = (promotionId, promotionTitle) => {
    const { dispatch, onUpdateParams, types } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: {
        path: '/AddContractMakeDiscount',
        params: { onUpdateParams, types, promotionId, promotionTitle },
        refresh: true,
        mirrorId: '' + new Date().getTime(),
      },
    });
  };

  render() {
    const {
      children,
      promotionModes,
      getPopupContainer,
      code,
      visible,
      selectedPromotion,
      onPromotionChange,
      onCodeChange,
      onClose,
    } = this.props;
    const content = (
      <div className={styles.wrap}>
        <div className={styles.selectNone + ' ' + styles.row}>
          <div className={styles.col1}>&nbsp;</div>
          <div className={styles.col2}>
            <div className={styles.itemWrap}>
              <div className={styles.radio + ' ' + styles.item}>
                <Radio checked={selectedPromotion === -1} onChange={() => onPromotionChange(-1)} />
              </div>
              <div className={styles.item}>不使用任何优惠</div>
            </div>
          </div>
        </div>
        <div>
          {promotionModes.map((p, i) => {
            const { mode, promotions } = p;
            return (
              <div key={i} className={styles.promotions}>
                <div className={styles.row}>
                  <div className={styles.col1}>
                    <PromotionModeTag mode={mode} />
                  </div>
                  <div className={styles.col2}>
                    {promotions.map((pro) => {
                      return (
                        <div key={pro.id} className={styles.itemWrap}>
                          <div className={styles.radio + ' ' + styles.item}>
                            <Radio checked={selectedPromotion === pro.id} onChange={() => onPromotionChange(pro.id)} />
                          </div>
                          <div className={styles.title + ' ' + styles.item}>{pro.title}</div>
                          <div className={styles.content + ' ' + styles.item}>{pro.content}</div>
                          <div className={styles.collectBtn + ' ' + styles.item}>
                            <TextLink onClick={() => this.toMakeDiscount(pro.id, pro.title)}>
                              <span>去凑单</span>
                              <Icon type="right" size={10} />
                            </TextLink>
                          </div>
                        </div>
                      );
                    })}
                  </div>
                </div>
              </div>
            );
          })}
        </div>
        {/*
        <div className={styles.row} style={{ paddingTop: 5 }}>
          <div className={styles.col1}>
            <div className={styles.codeTag}>促销码</div>
          </div>
          <div className={styles.col2}>
            <div style={{ position: 'relative', top: -5 }}>
              <LockableInput size={'small'} value={code} onChange={onCodeChange} ref={c => this.codeInputRef = c} />
            </div>
          </div>
        </div>*/}
      </div>
    );

    return (
      <Popover
        content={content}
        trigger="click"
        visible={visible}
        onClose={onClose}
        getPopupContainer={getPopupContainer}
      >
        {children}
      </Popover>
    );
  }
}

export default connect()(SelectPromotionPop);
