import React from 'react';
import ReactDOM from 'react-dom';
import { connect } from 'dva';
import styles from './styles.less';
import StudentInfo from '../StudentInfo';
import { getDictTextByVal } from '../../../../utils/dict/index';
import dicts from '../dicts';
import utils from '../utils';
import loaders from '../loaders';
import TextLink from 'COMPONENT/TextLink';
import Block from 'COMPONENT/Block';
import PromotionModeTag from '../PromotionModeTag/index';
import ProductTypeTag from '../ProductTypeTag/index';
import FeeItem from './FeeItem';
import GiftModeTag from '../GiftModeTag';
import Tabs from 'COMPONENT/Tabs';
import Ticket from '../Ticket';
import LockableInput from '../LockableInput';
import FormItem from 'COMPONENT/FormItem';
import IconButton from 'COMPONENT/IconButton';
import SelectPromotionPop from './SelectPromotionPop';
import { formatCurrency } from '../../../../components/utils/basic/num';
import SummaryItem from './SummaryItem';
import Popover from 'COMPONENT/Popover';

const Tab = Tabs.Tab;

class Checkout extends React.Component {
  state = {
    visPromotionPop: null, // number 当前打开的更换促销弹框序号
    tickets: null, // {[object]} 所有可使用的优惠券，[{ id, name, main, secondary, info }]
    selectPromotionPopVis: {}, // {object} 每个产品的选择优惠弹出气泡显示状态；key为产品ID，value为显示状态
    $wrap: null, // {domElement} 组件根元素，用于弹出气泡定位
  };

  componentDidMount() {
    this.setState({ $wrap: ReactDOM.findDOMNode(this.wrapRef) });
  }

  componentWillReceiveProps(nextProps) {
    const { onUpdateParams, cartList, activeTabId, activePageId, status } = nextProps;
    const currentNotCheckout =
      this.props.activeTabId !== '/AddContractList' ||
      this.props.activePageId !== '/AddContractList/Shop' ||
      this.props.status !== dicts.SHOP_STATUS_CHECK;
    const nextIsCheckout =
      activeTabId === '/AddContractList' &&
      activePageId === '/AddContractList/Shop' &&
      status === dicts.SHOP_STATUS_CHECK;
    if (currentNotCheckout && nextIsCheckout) {
      let calls = [loaders.loadCheckoutInfo(cartList)];
      if (!this.state.tickets) {
        calls.push(loaders.loadTickets());
      }
      onUpdateParams({ loading: true });
      Promise.all(calls).then(([checkoutInfo, tickets]) => {
        if (tickets) {
          this.setState({ tickets });
        }
        onUpdateParams({
          loading: false,
          checkoutInfo,
          cartList: this.updateCartListSelectedPromotion(cartList, checkoutInfo),
        });
      });
    }
  }

  updateCartListSelectedPromotion = (cartList, checkoutInfo) => {
    let productPromotionMap = {};
    if (checkoutInfo && checkoutInfo.promotionGroupList) {
      checkoutInfo.promotionGroupList.forEach((g) => {
        if (!!g.promotion) {
          g.products.forEach((p) => {
            productPromotionMap[p.id] = g.promotion.id;
          });
        }
      });
    }
    return cartList.map((c) => {
      return { ...c, selectedPromotion: productPromotionMap[c.id] || null };
    });
  };

  getToolBar = () => {
    return (
      <TextLink
        onClick={() => {
          this.props.onUpdateParams({ status: dicts.SHOP_STATUS_SELECT });
        }}
      >
        添加报读产品
      </TextLink>
    );
  };

  ticketClick = (id) => {
    this.calculateCheckoutInfo({ checkedTicket: this.props.checkedTicket === id ? null : id });
  };

  codeChange = (val) => {
    this.calculateCheckoutInfo({ code: val });
  };

  onCountChange = (id, val) => {
    const newCartList = this.props.cartList.map((c) => {
      if (c.id === id) {
        return { ...c, count: val };
      }
      return c;
    });
    this.calculateCheckoutInfo({ cartList: newCartList });
  };

  onDelete = (id) => {
    this.calculateCheckoutInfo({ cartList: this.props.cartList.filter((c) => c.id !== id) });
  };

  onAdditionProductDelete = (id, additionId) => {
    const newCartList = this.props.cartList.map((c) => {
      if (c.id === id) {
        return { ...c, additionProducts: c.additionProducts.filter((a) => a.id !== additionId) };
      }
      return c;
    });
    this.calculateCheckoutInfo({ cartList: newCartList });
  };

  calculateCheckoutInfo = (changed) => {
    const { onUpdateParams, cartList, checkedTicket, code } = this.props;
    const newProps = { cartList, checkedTicket, code, ...changed };
    onUpdateParams({ loading: true });
    loaders.loadCheckoutInfo(newProps.cartList).then((checkoutInfo) => {
      onUpdateParams({
        loading: false,
        checkoutInfo,
        ...newProps,
        cartList: this.updateCartListSelectedPromotion(newProps.cartList, checkoutInfo),
      });
    });
  };

  groupHasProductDetail = (group) => {
    const { products } = group;
    if (!products) {
      return false;
    }
    return products.some((p) => this.props.cartList.some((c) => c.id === p.id));
  };

  promotionChange = (id, val) => {
    const { onUpdateParams, cartList } = this.props;
    const newCartList = cartList.map((c) => {
      if (c.id === id) {
        return { ...c, selectedPromotion: val };
      }
      return c;
    });
    onUpdateParams({ loading: true, cartList: newCartList });
    loaders.loadCheckoutInfo(newCartList).then(
      (checkoutInfo) => {
        onUpdateParams({
          loading: false,
          checkoutInfo,
          cartList: this.updateCartListSelectedPromotion(newCartList, checkoutInfo),
        });
        this.setSelectPromotionPopVis(id, false);
      },
      () => {
        onUpdateParams({ loading: false, cartList });
      },
    );
  };

  codeChangeForProduct = (id, val) => {
    const { onUpdateParams, cartList } = this.props;
    const newCartList = cartList.map((c) => {
      if (c.id === id) {
        return { ...c, code: val };
      }
      return c;
    });
    onUpdateParams({ loading: true });
    loaders.loadCheckoutInfo(newCartList).then(
      (checkoutInfo) => {
        onUpdateParams({
          loading: false,
          checkoutInfo,
          cartList: this.updateCartListSelectedPromotion(newCartList, checkoutInfo),
        });
        this.setSelectPromotionPopVis(id, false);
      },
      () => {
        onUpdateParams({ loading: false });
      },
    );
  };

  setSelectPromotionPopVis = (id, visible) => {
    this.setState({
      selectPromotionPopVis: { ...this.state.selectPromotionPopVis, [id]: visible },
    });
  };

  render() {
    const { types, user, stuInfo, courseInfo, onUpdateParams, checkoutInfo, loading, checkedTicket, code, cartList } =
      this.props;
    const { visPromotionPop, tickets, selectPromotionPopVis, $wrap } = this.state;
    const typeName = types
      .map((t) => {
        return getDictTextByVal(dicts.productTypeDict, t);
      })
      .join('/');
    const { promotionGroupList, totalAmount, discountAmount, discountDetail } = checkoutInfo || {};
    return (
      <div className={styles.wrap} ref={(c) => (this.wrapRef = c)}>
        <StudentInfo user={user} stuInfo={stuInfo} courseInfo={courseInfo} onUpdateParams={onUpdateParams} />
        {!!checkoutInfo && (
          <div>
            <Block title={`${typeName}报读课程产品`} headerToolBar={this.getToolBar()}>
              <div className={styles.table}>
                <div className={styles.tHeader}>
                  <div className={styles.tr}>
                    <div className={styles.td1}>产品信息</div>
                    <div className={styles.td2}>单价</div>
                    <div className={styles.td3}>数量</div>
                    <div className={styles.td4}>小计</div>
                    <div className={styles.td5}>操作</div>
                  </div>
                </div>
                <div className={styles.tBody}>
                  {promotionGroupList
                    .filter((group) => this.groupHasProductDetail(group))
                    .map((d, i) => {
                      const { promotion, products } = d;
                      return (
                        <div key={i} className={styles.group}>
                          {!!promotion && promotion.id === -1 && (
                            <div className={styles.groupHeader}>
                              <div style={{ paddingLeft: 20 }}>不使用任何优惠</div>
                            </div>
                          )}
                          {!!promotion && promotion.id !== -1 && (
                            <div className={styles.groupHeader}>
                              <div className={styles.promotionWrap}>
                                <div className={styles.promotion}>
                                  <div className={styles.promotionTag + ' ' + styles.promotionItem}>
                                    <PromotionModeTag mode={promotion.mode} corner />
                                  </div>
                                  <div className={styles.promotionTitle + ' ' + styles.promotionItem}>
                                    {promotion.title}
                                  </div>
                                  <div className={styles.promotionContent + ' ' + styles.promotionItem}>
                                    {promotion.content}
                                  </div>
                                </div>
                              </div>
                            </div>
                          )}
                          <div className={styles.products}>
                            {products
                              .filter((p) => cartList.some((c) => c.id === p.id))
                              .map((product) => {
                                const { reduces, gifts } = product;
                                const {
                                  id,
                                  type,
                                  name,
                                  price,
                                  count,
                                  additionProducts,
                                  infos,
                                  promotionInfo,
                                  selectedPromotion,
                                  code,
                                } = cartList.find((c) => c.id === product.id);

                                return (
                                  <div key={id} className={styles.product}>
                                    <FeeItem
                                      title={
                                        <div className={styles.productTitleWrap}>
                                          <div className={styles.productTag}>
                                            <ProductTypeTag type={type} />
                                          </div>
                                          <div className={styles.productTitle}>
                                            <div className={styles.productName}>{name}</div>
                                            {infos && (
                                              <div className={styles.productInfos}>
                                                {infos.map((info, i) => {
                                                  return (
                                                    <div key={i} className={styles.productInfo}>
                                                      {info}
                                                    </div>
                                                  );
                                                })}
                                              </div>
                                            )}
                                          </div>
                                          {!!promotionInfo &&
                                            promotionInfo.some((pi) => pi.type === dicts.PROMOTION_TYPE_ACTIVITY) && (
                                              <div className={styles.productBtn}>
                                                <SelectPromotionPop
                                                  visible={!!selectPromotionPopVis[id]}
                                                  promotionModes={
                                                    promotionInfo.find(
                                                      (pi) => pi.type === dicts.PROMOTION_TYPE_ACTIVITY,
                                                    ).modes
                                                  }
                                                  selectedPromotion={selectedPromotion || null}
                                                  code={code || null}
                                                  getPopupContainer={() => $wrap}
                                                  onPromotionChange={(val) => this.promotionChange(id, val)}
                                                  onCodeChange={(val) => this.codeChangeForProduct(id, val)}
                                                  onClose={() => this.setSelectPromotionPopVis(id, false)}
                                                  onUpdateParams={onUpdateParams}
                                                  types={types}
                                                >
                                                  <TextLink
                                                    onClick={() =>
                                                      this.setSelectPromotionPopVis(id, !selectPromotionPopVis[id])
                                                    }
                                                  >
                                                    更换促销
                                                  </TextLink>
                                                </SelectPromotionPop>
                                              </div>
                                            )}
                                        </div>
                                      }
                                      price={price}
                                      count={count}
                                      total={price * count}
                                      reduces={reduces}
                                      onCountChange={
                                        type === dicts.PRODUCT_TYPE_PROMISE ||
                                        type === dicts.PRODUCT_TYPE_MINI ||
                                        type === dicts.PRODUCT_TYPE_TWO
                                          ? null
                                          : (val) => this.onCountChange(id, val)
                                      }
                                      onDelete={() => this.onDelete(id)}
                                    />
                                    {!!additionProducts &&
                                      additionProducts.map((ad) => {
                                        return (
                                          <FeeItem
                                            key={ad.id}
                                            title={
                                              <div className={styles.productTitleWrap + ' ' + styles.level2}>
                                                <div className={styles.productTitle}>
                                                  <div className={styles.productName}>{ad.name}</div>
                                                </div>
                                              </div>
                                            }
                                            price={ad.price}
                                            count={ad.count}
                                            total={ad.price * ad.count}
                                            onDelete={() => this.onAdditionProductDelete(id, ad.id)}
                                          />
                                        );
                                      })}
                                    {!!gifts &&
                                      gifts.map((g, gIdx) => {
                                        return (
                                          <FeeItem
                                            key={gIdx}
                                            title={
                                              <div className={styles.productTitleWrap}>
                                                <div className={styles.productTag}>
                                                  <GiftModeTag mode={g.mode} />
                                                </div>
                                                <div className={styles.productTitle}>
                                                  <div className={styles.productName}>{g.name}</div>
                                                </div>
                                              </div>
                                            }
                                            price={g.price}
                                            count={g.count}
                                            total={g.price * g.count}
                                            reduces={g.reduces}
                                          />
                                        );
                                      })}
                                  </div>
                                );
                              })}
                          </div>
                        </div>
                      );
                    })}
                </div>
              </div>
              {/*
              <Tabs>
                <Tab title="优惠券">
                  <div style={{overflow: 'hidden'}}>
                    {
                      tickets.map((t, idx) => {
                        const checked = checkedTicket === t.id
                        return (
                          <div key={t.id} className={styles.ticket}>
                            <Ticket data={t} checked={checked} onClick={() => this.ticketClick(t.id)}/>
                          </div>
                        )
                      })
                    }
                  </div>
                </Tab>
                <Tab title="促销码">
                  <LockableInput value={code} onChange={this.codeChange}/>
                </Tab>
              </Tabs>*/}
            </Block>
            <div className={styles.summary}>
              <div className={styles.summaryDetails}>
                <SummaryItem
                  name={
                    <span className={styles.summaryDetailName}>
                      <span className={styles.productCount}>{cartList ? cartList.length : 0}</span>
                      <span>件商品，总商品金额：</span>
                    </span>
                  }
                >
                  {formatCurrency(totalAmount, '￥')}
                </SummaryItem>
                <SummaryItem
                  name={
                    <span className={styles.summaryDetailName}>
                      {!!discountDetail && discountDetail.length > 0 && (
                        <Popover
                          content={
                            <div className={styles.discountPop}>
                              <div className={styles.discountPopTitle}>已使用优惠</div>
                              <div className={styles.discountPopContent}>
                                {discountDetail.map((d, i) => (
                                  <div key={i}>{d}</div>
                                ))}
                              </div>
                            </div>
                          }
                        >
                          <IconButton className={styles.discountDetailIcon} type={'question_mc'} />
                        </Popover>
                      )}
                      <span>优惠：</span>
                    </span>
                  }
                >
                  {`-${formatCurrency(discountAmount, '￥')}`}
                </SummaryItem>
              </div>
              <div className={styles.summaryFinal}>
                <SummaryItem name={'应付：'}>
                  <div className={styles.finalAmount}>{formatCurrency(totalAmount - discountAmount, '￥')}</div>
                </SummaryItem>
              </div>
            </div>
          </div>
        )}
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  const { status, checkoutInfo, cartList, loading, checkedTicket, code } =
    state.main.pageParams['/AddContractList/Shop'];
  return {
    status,
    checkoutInfo,
    cartList,
    loading,
    checkedTicket,
    code,
    activePageId: state.main.tabpanes.find((t) => t.id === '/AddContractList').activePageId,
    activeTabId: state.main.activeTabId,
  };
};

export default connect(mapStateToProps)(Checkout);
