import React from 'react';
import Modal from 'COMPONENT/Modal';
import styles from './styles.less';
import loaders from '../loaders';
import FormItem from 'COMPONENT/FormItem';
import { COURSE_STATUS_START, COURSE_STATUS_NOT_START, courseStatusDict } from '../dicts';
import dicts from '../dicts';
import { getDictTextByVal } from '../../../../utils/dict/index';
import classnames from 'classnames';
import ProductTypeTag from '../ProductTypeTag';
import NoData from '../NoData';

const formCfg = {
  labelWidth: 80,
};

class CourseModal extends React.Component {
  componentDidMount() {
    const { cusId, stuId, courseInfo, onUpdateParams } = this.props;
    if (!courseInfo) {
      loaders.loadStuCourseList({ cusId, stuId }).then((list) => {
        onUpdateParams({ courseInfo: list });
      });
    }
  }

  render() {
    const { onClose, courseInfo } = this.props;
    const modalCls = {
      [styles.wrap]: 1,
      [styles.modal]: 1,
    };
    const list = courseInfo || [];
    return (
      <Modal
        className={classnames(modalCls)}
        loading={!courseInfo}
        title="学生在读课程"
        width={688}
        footer={false}
        onClose={onClose}
        noLine
        closable
      >
        <div className={styles.main}>
          {list &&
            list.length > 0 &&
            list.map((li, i) => {
              const { type, name, grade, subject, teacher, address, time, status, endDate } = li;
              return (
                <div key={i} className={styles.row}>
                  <div className={styles.header}>
                    <div className={styles.tag}>
                      <ProductTypeTag type={type} />
                    </div>
                    <div className={styles.title}>{name}</div>
                    <div
                      className={
                        styles.status + ' ' + (status === COURSE_STATUS_NOT_START ? styles.statusNotStart : '')
                      }
                    >
                      {getDictTextByVal(courseStatusDict, status)}
                    </div>
                  </div>
                  <div className={styles.body}>
                    <FormItem {...formCfg} name={'老师'} className={styles.formItem}>
                      {teacher}
                    </FormItem>
                    <FormItem {...formCfg} name={'科目'} className={styles.formItem}>
                      {subject}
                    </FormItem>
                    <FormItem {...formCfg} name={'年级'} className={styles.formItem}>
                      {grade}
                    </FormItem>
                    <FormItem {...formCfg} name={'地点'} className={styles.formItem} style={{ clear: 'both' }}>
                      {address}
                    </FormItem>
                    <FormItem {...formCfg} name={'时间'} className={styles.formItem} style={{ clear: 'both' }}>
                      {time}
                    </FormItem>
                    <FormItem {...formCfg} name={'结课日期'} className={styles.formItem} style={{ clear: 'both' }}>
                      {endDate}
                    </FormItem>
                  </div>
                </div>
              );
            })}
          {(!list || list.length === 0) && <NoData />}
        </div>
      </Modal>
    );
  }
}

export default CourseModal;
