import React from 'react';
import { connect } from 'dva';
import Input from 'COMPONENT/Input';
import Select from 'COMPONENT/Select';
import TextLink from 'COMPONENT/TextLink';
import Block from 'COMPONENT/Block';
import FormItem from 'COMPONENT/FormItem';
import { Row, Col } from 'antd';
import { getDictTextByVal, loadCommonDict, loadBranchCampusDict } from 'UTIL/dict';
import { loadList } from '../loaders';
import {
  LIST_TYPE_MY,
  LIST_TYPE_OTHER,
  listTypeDict,
  CONTRACT_TYPE_NEW,
  CONTRACT_TYPE_AGAIN,
  contractTypeDict,
} from '../dicts';
import dicts from '../dicts';
import Tabs from 'COMPONENT/Tabs';
import { FIELD_FILLER, checkFieldEmpty, getFieldWithFiller, getCommonErrorTips } from 'COMPONENT/utils/form';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import PopConfirm from 'COMPONENT/PopConfirm';
import PopTip from 'COMPONENT/PopTip';
import List from 'COMPONENT/List';
import Page from 'COMPONENT/Page';
import Button from 'COMPONENT/Button';
import loaders from '../loaders';
import styles from './styles.less';
import SelectSchoolModal from 'PAGE/common/SelectSchoolModal';
import { sexDict } from 'PAGE/common/dicts';

const { Tab } = Tabs;
const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;
let defaultLists = {};
listTypeDict.forEach((d) => {
  defaultLists[d.val] = {
    list: [], // {object[]} 列表数据
    selected: [], // {number} 选中的列表序号
  };
});
const defaultState = {
  loading: false, // {boolean} 页面加载状态

  cusId: null, // {string} 客户ID
  stuId: null, // {string} 学生ID
  cusName: null, // {string} 客户姓名
  contract: null, // {string} 联系方式
  campus: null, // {string} 学习中心
  stuName: null, // {string} 学生姓名
  grade: null, // {string} 年级
  school: null, // {string} 学校ID
  schoolName: null, // {string} 学校名称
  contractType: CONTRACT_TYPE_NEW, // {string} 合同类型

  hasCheckDuplicate: false, // {boolean} 是否已检测重名学生
  activeTabKey: null, // {string} 激活的标签页ID
  lists: defaultLists, // {object} 列表数据集合，{key: {list, selected}}

  checking: false, // {boolean} 校验开启状态
  okTipsVis: false, // {boolean} 保存按钮提示气泡显示状态
  okTips: false, // {string} 保存按钮提示信息
  okConfirmVis: false, // {boolean} 保存按钮确认气泡显示状态

  selectSchoolModalVis: false, // {boolean} 选择学校弹框显示状态
};
const formCfg = {
  labelWidth: 80,
  style: { padding: '0 30px', marginBottom: 15 },
};
const clearFix = {
  style: { clear: 'both' },
};
class CreateStudent extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      campusDict: null, // {object[]} 字典，学习中心
      gradeDict: null, // {object[]} 字典，年级
      ...defaultState,
    };
    this.errors = {};
  }

  componentDidMount() {
    this.setState({ loading: true });
    Promise.all([loadBranchCampusDict(), loadCommonDict('STUDENT_GRADE')]).then(([campusDict, gradeDict]) => {
      this.setState({ loading: false, campusDict, gradeDict });
    });
  }

  componentWillReceiveProps(nextProps) {
    if (
      this.props.activePageId === '/AddContractList/Shop' &&
      nextProps.activePageId === '/AddContractList/CreateStudent'
    ) {
      this.props.dispatch({
        type: 'main/openPage',
        payload: {
          path: '/AddContractList',
        },
      });
    }
  }

  reSign = ({ cusId, stuId, cusName, contract, campus, stuName, grade, schoolName, contractType }) => {
    const { dispatch } = this.props;
    const { gradeDict, campusDict } = this.state;
    dispatch({
      type: 'main/openPage',
      payload: {
        path: '/AddContractList/Shop',
        params: {
          status: dicts.SHOP_STATUS_SELECT_SET,
          stuInfo: {
            cusId,
            stuId,
            contractType,
            cusName,
            contract,
            campusName: getDictTextByVal(campusDict, campus),
            stuName,
            gradeName: getDictTextByVal(gradeDict, grade),
            schoolName,
          },
        },
        refresh: true,
      },
    });
  };

  onOk = () => {
    const { checking, hasCheckDuplicate, contract, stuName } = this.state;
    const selectedStuInfo = this.getSelectedStuInfo();
    if (selectedStuInfo) {
      this.reSign(selectedStuInfo);
    } else {
      if (!checking) {
        this.setState({ checking: true });
      }
      const errorTips = getCommonErrorTips(this.errors);
      if (errorTips) {
        this.showOkTips(errorTips);
        return;
      }
      if (!hasCheckDuplicate) {
        this.setState({ loading: true });
        loaders.checkDuplicateStudent({ stuName, cusPhone: contract }).then(
          (res) => {
            if (res) {
              loadList({ stuName, cusPhone: contract, currentPage: 1, pageSize: 9999 }).then(
                (lists) => {
                  let activeTabKey;
                  let newLists = {};
                  Object.keys(lists).forEach((key) => {
                    if (!activeTabKey && lists[key].list.length > 0) {
                      activeTabKey = key;
                    }
                    newLists[key] = { list: lists[key].list, selected: [] };
                  });
                  if (activeTabKey) {
                    this.setState({
                      loading: false,
                      hasCheckDuplicate: true,
                      activeTabKey,
                      lists: newLists,
                      okConfirmVis: true,
                    });
                  } else {
                    this.save();
                  }
                },
                (e) => {
                  showMessage(e.text || '创建失败');
                  this.setState({ loading: false });
                },
              );
            } else {
              this.save();
            }
          },
          (e) => {
            showMessage(e.text || '创建失败');
            this.setState({ loading: false });
          },
        );
      } else {
        this.save();
      }
    }
  };

  save = () => {
    const { dispatch } = this.props;
    const {
      cusId,
      stuId,
      cusName,
      contract,
      campus,
      stuName,
      grade,
      school,
      schoolName,
      contractType,
      gradeDict,
      campusDict,
    } = this.state;
    this.setState({ loading: true });
    loaders.saveStudent({ cusId, stuId, cusName, contract, campus, stuName, grade, school }).then(
      ({ cusId, stuId }) => {
        this.setState({ loading: false });
        showMessage('创建成功', 'success');
        dispatch({
          type: 'main/openPage',
          payload: {
            path: '/AddContractList/Shop',
            params: {
              status: dicts.SHOP_STATUS_SELECT_SET,
              stuInfo: {
                cusId,
                stuId,
                contractType: CONTRACT_TYPE_NEW,
                cusName,
                contract,
                campusName: getDictTextByVal(campusDict, campus),
                stuName,
                gradeName: getDictTextByVal(gradeDict, grade),
                schoolName,
              },
            },
            refresh: true,
          },
        });
      },
      (e) => {
        showMessage(e.text || '创建失败');
        this.setState({ loading: false });
      },
    );
  };

  getToolBar = () => {
    return <TextLink onClick={this.reset}>重置</TextLink>;
  };

  reset = () => {
    this.setState(defaultState);
    this.errors = {};
  };

  onTabChange = (key) => {
    this.setState({ activeTabKey: key });
  };

  onRowSelectionChange = (keys, type) => {
    const { lists } = this.state;
    const { list, selected } = lists[type];
    let newSelected = [];
    if (keys.length > 0) {
      newSelected = [keys.find((k) => selected.indexOf(k) < 0)];
    }
    let newLists = {};
    Object.keys(lists).forEach((key) => {
      const obj = lists[key];
      if (key === type) {
        newLists[key] = { ...obj, selected: newSelected };
      } else {
        newLists[key] = { ...obj, ...(newSelected.length > 0 ? { selected: [] } : {}) };
      }
    });
    this.setState({ lists: newLists });
  };

  onCancel = () => {
    const { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: { path: '/AddContractList' },
    });
  };

  onErrorChange = (key, e) => {
    this.errors[key] = e;
  };

  showOkTips = (text) => {
    this.setState({ okTips: text, okTipsVis: true });
  };

  onOkConfirmVisiChange = (visible) => {
    if (!visible) {
      this.setState({ okConfirmVis: visible });
    }
  };
  onOkTipsVisiChange = (visible) => {
    if (!visible) {
      this.setState({ okTipsVis: visible });
    }
  };

  getPageFooter = () => {
    const { loading, okConfirmVis, okTips, okTipsVis } = this.state;
    return (
      <div style={{ overflow: 'hidden' }}>
        <div className="Page-btn-wrap">
          <Button onClick={this.onCancel}>取消</Button>
        </div>
        <div className="Page-btn-wrap">
          <PopConfirm
            visible={okConfirmVis}
            content="系统中发现已有同名学生，您可以选择合并学生或直接保存"
            okBtnText="合并学生"
            cancelBtnText="直接保存"
            onOk={() => this.setState({ okConfirmVis: false })}
            onCancel={() => {
              this.setState({ okConfirmVis: false });
              this.save();
            }}
            onVisibleChange={this.onOkConfirmVisiChange}
          >
            <PopTip content={okTips} visible={okTipsVis} onVisibleChange={this.onOkTipsVisiChange}>
              <Button type="primary" onClick={this.onOk} disabled={loading}>
                提交
              </Button>
            </PopTip>
          </PopConfirm>
        </div>
      </div>
    );
  };

  getSelectedStuInfo = () => {
    const { lists } = this.state;
    let info = null;
    Object.keys(lists).forEach((key) => {
      const { list, selected } = lists[key];
      if (selected.length > 0) {
        const d = list[selected[0]];
        info = {
          cusId: d.cusId,
          stuId: d.stuId,
          cusName: d.cusName,
          contract: d.phone,
          campus: d.campus,
          stuName: d.stuName,
          grade: d.grade,
          schoolName: d.stuSchool,
          contractType: CONTRACT_TYPE_AGAIN,
        };
        return false;
      }
    });
    return info;
  };

  clearDuplicateInfo = () => {
    this.setState({ hasCheckDuplicate: false, activeTabKey: null, lists: defaultLists });
  };

  render() {
    const { loading, lists, checking, gradeDict, campusDict, activeTabKey, hasCheckDuplicate, selectSchoolModalVis } =
      this.state;
    const columns = [
      { title: '客户姓名', dataIndex: 'cusName' },
      { title: '电话号码', dataIndex: 'phone', width: COL_WIDTH_L },
      { title: '学生编号', dataIndex: 'stuNo', width: 150 },
      { title: '学生姓名', dataIndex: 'stuName' },
      { title: '性别', dataIndex: 'sex', dict: sexDict },
      { title: '学生学校', dataIndex: 'stuSchool', width: COL_WIDTH_L },
      { title: '年级', dataIndex: 'grade', dict: gradeDict },
      { title: '学管师', dataIndex: 'studyManager' },
      { title: '学习中心', dataIndex: 'campusName' },
    ];
    const selectedStuInfo = this.getSelectedStuInfo();
    const { cusName, contract, campus, stuName, grade, schoolName, contractType } = selectedStuInfo
      ? selectedStuInfo
      : this.state;
    return (
      <Page loading={loading} route={this.props.route} footer={this.getPageFooter()} className={styles.page}>
        <Block title="学生信息" headerToolBar={this.getToolBar()}>
          <Row style={{ margin: '0 -30px 0 -5px' }}>
            <Col span={8}>
              <FormItem {...formCfg} name="客户姓名" required={!selectedStuInfo}>
                {selectedStuInfo ? (
                  <div>{getFieldWithFiller(cusName)}</div>
                ) : (
                  <Input
                    value={cusName}
                    onChange={(e) => this.setState({ cusName: e.target.value })}
                    validator={{
                      onErrorChange: (e) => this.onErrorChange('cusName', e),
                      silent: !checking,
                    }}
                  />
                )}
              </FormItem>
            </Col>
            <Col span={8}>
              <FormItem {...formCfg} name="联系方式" required={!selectedStuInfo}>
                {selectedStuInfo ? (
                  <div>{getFieldWithFiller(contract)}</div>
                ) : (
                  <Input
                    value={contract}
                    onChange={(e) => {
                      this.setState({ contract: e.target.value });
                      this.clearDuplicateInfo();
                    }}
                    validator={{
                      rules: ['required', 'phone'],
                      onErrorChange: (e) => this.onErrorChange('contract', e),
                      silent: !checking,
                    }}
                  />
                )}
              </FormItem>
            </Col>
            <Col span={8}>
              <FormItem {...formCfg} name="学习中心" required={!selectedStuInfo}>
                {selectedStuInfo ? (
                  <div>{getDictTextByVal(campusDict, campus, true)}</div>
                ) : (
                  <Select
                    dict={campusDict}
                    value={campus}
                    onChange={(val) => this.setState({ campus: val })}
                    validator={{
                      onErrorChange: (e) => this.onErrorChange('campus', e),
                      silent: !checking,
                    }}
                  />
                )}
              </FormItem>
            </Col>
            <Col span={8} {...clearFix}>
              <FormItem {...formCfg} name="学生姓名" required={!selectedStuInfo}>
                {selectedStuInfo ? (
                  <div>{getFieldWithFiller(stuName)}</div>
                ) : (
                  <Input
                    value={stuName}
                    onChange={(e) => {
                      this.setState({ stuName: e.target.value });
                      this.clearDuplicateInfo();
                    }}
                    validator={{
                      onErrorChange: (e) => this.onErrorChange('stuName', e),
                      silent: !checking,
                    }}
                  />
                )}
              </FormItem>
            </Col>
            <Col span={8}>
              <FormItem {...formCfg} name="年级" required={!selectedStuInfo}>
                {selectedStuInfo ? (
                  <div>{getDictTextByVal(gradeDict, grade, true)}</div>
                ) : (
                  <Select
                    dict={gradeDict}
                    value={grade}
                    onChange={(val) => this.setState({ grade: val })}
                    validator={{
                      onErrorChange: (e) => this.onErrorChange('grade', e),
                      silent: !checking,
                    }}
                  />
                )}
              </FormItem>
            </Col>
            <Col span={8}>
              <FormItem {...formCfg} name="学校">
                {selectedStuInfo ? (
                  <div>{getFieldWithFiller(schoolName)}</div>
                ) : (
                  <Input readOnly value={schoolName} onClick={() => this.setState({ selectSchoolModalVis: true })} />
                )}
              </FormItem>
            </Col>
            <Col span={8} {...clearFix}>
              <FormItem {...formCfg} name="合同类型">
                {getDictTextByVal(contractTypeDict, contractType, true)}
              </FormItem>
            </Col>
          </Row>
        </Block>
        {hasCheckDuplicate && activeTabKey && (
          <Block title="合并学生">
            <div className={styles.tips}>
              系统中发现已有同名学生：
              <span className={styles.tipsName}>{stuName}</span>
              ，请选择合并学生
            </div>
            <Tabs onChange={this.onTabChange} activeKey={activeTabKey}>
              {Object.keys(lists).map((key) => {
                const d = lists[key];
                return (
                  <Tab key={key} title={getDictTextByVal(listTypeDict, key)} value={key}>
                    <List
                      columns={columns}
                      dataSource={d.list}
                      rowSelection={{
                        selectedRowKeys: d.selected,
                        onChange: (params) => this.onRowSelectionChange(params, key),
                      }}
                    />
                  </Tab>
                );
              })}
            </Tabs>
          </Block>
        )}
        {selectSchoolModalVis && (
          <SelectSchoolModal
            onConfirm={(school, schoolName) => this.setState({ selectSchoolModalVis: false, school, schoolName })}
            onClose={() => this.setState({ selectSchoolModalVis: false })}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    activePageId: state.main.tabpanes.find((t) => t.id === '/AddContractList').activePageId,
  };
};

export default connect(mapStateToProps)(CreateStudent);
