import React from 'react';
import styles from './styles.less';
import Input from 'COMPONENT/Input';
import Button from 'COMPONENT/Button';
import IconButton from 'COMPONENT/IconButton';
import classnames from 'classnames';

class LockableInput extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      inputValue: props.value || null,
      editing: false,
    };
  }

  componentWillReceiveProps(nextProps) {
    if (this.props.value !== nextProps.value) {
      this.setState({ inputValue: nextProps.value || null, editing: false });
    }
  }

  confirm = () => {
    if (this.props.value === this.state.inputValue) {
      this.setState({ editing: false });
    } else {
      this.props.onChange(this.state.inputValue);
    }
  };

  closeEdit = () => {
    this.setState({ inputValue: this.props.value, editing: false });
  };

  render() {
    const { value, size } = this.props;
    const { inputValue, editing } = this.state;
    const cls = {
      [styles.wrap]: 1,
      [styles.wrapSmall]: size === 'small',
    };
    return (
      <div className={classnames(cls)}>
        {editing ? (
          <div className={styles.row}>
            <Input
              size={size}
              value={inputValue}
              onChange={(e) => this.setState({ inputValue: e.target.value })}
              className={styles.input + ' ' + styles.item}
            />
            <Button size={size} type={'primary'} onClick={this.confirm} className={styles.btn + ' ' + styles.item}>
              确定
            </Button>
          </div>
        ) : !!value ? (
          <div className={styles.row}>
            <div className={styles.valueText + ' ' + styles.item}>{value}</div>
            <IconButton
              size={size === 'small' ? 12 : 14}
              className={styles.icon + ' ' + styles.item}
              type={'edit_mc'}
              onClick={() => this.setState({ editing: true })}
            />
          </div>
        ) : (
          <div>
            <Button size={size} iconType={'plus'} onClick={() => this.setState({ editing: true })}>
              添加
            </Button>
          </div>
        )}
      </div>
    );
  }
}

export default LockableInput;
