import React from 'react';
import { connect } from 'dva';
import styles from './styles.less';
import Icon from 'COMPONENT/Icon';
import FormItem from 'COMPONENT/FormItem';
import TextLink from 'COMPONENT/TextLink';
import dicts from '../../dicts';
import PromotionModeTag from '../../PromotionModeTag';

class PromotionInfo extends React.Component {
  toMakeDiscount = (promotionId, promotionTitle) => {
    const { dispatch, onUpdateParams, types } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: {
        path: '/AddContractMakeDiscount',
        params: { onUpdateParams, types, promotionId, promotionTitle },
        refresh: true,
        mirrorId: '' + new Date().getTime(),
      },
    });
  };

  render() {
    const { data } = this.props;
    if (!data) {
      return null;
    }
    return (
      <div className={styles.wrap}>
        {data.map((d, i) => {
          const { name, canCollect, modes } = d;
          return (
            <FormItem key={i} name={name} labelWidth={80} style={{ marginBottom: 0, padding: 0 }}>
              {modes.map((t, idx) => {
                const { mode, promotions } = t;
                return (
                  <div key={mode} className={styles.mode}>
                    <div className={styles.modeLeft}>
                      <div className={styles.modeTag}>
                        <PromotionModeTag mode={mode} />
                      </div>
                    </div>
                    <div className={styles.modeRight}>
                      {promotions.map((p, idx) => {
                        return (
                          <div key={idx} className={styles.promotion}>
                            <div className={styles.promotionTitle}>{p.title}</div>
                            <div className={styles.promotionContent}>
                              {p.content}
                              {canCollect && (
                                <TextLink
                                  className={styles.collectBtn}
                                  onClick={() => this.toMakeDiscount(p.id, p.title)}
                                >
                                  <span>去凑单</span>
                                  <Icon type="right" size={10} />
                                </TextLink>
                              )}
                            </div>
                          </div>
                        );
                      })}
                    </div>
                  </div>
                );
              })}
            </FormItem>
          );
        })}
      </div>
    );
  }
}

export default connect()(PromotionInfo);
