import React from 'react';
import classnames from 'classnames';
import styles from './styles.less';
import Modal from 'COMPONENT/Modal';
import Radio from 'COMPONENT/Radio';
import IconButton from 'COMPONENT/IconButton';
import FormItems from '../../FormItems';
import SelectClassHours from '../SelectClassHours';
import NoData from '../../NoData';

class SelectAssistClassModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      assistClassId: props.assistClassId,
      firstClassDate: props.firstClassDate,
      count: props.count,
      selectHoursPanelExpand: false,
    };
  }

  selectClass = (id) => {
    const { firstClassDate, count } = this.props.assistClassList.find((d) => d.id === id);
    console.log(this.props.assistClassList.find((d) => d.id === id));
    this.setState({ assistClassId: id, firstClassDate, count, selectHoursPanelExpand: false });
  };

  render() {
    const { assistClassList, onClose, onConfirm } = this.props;
    const { assistClassId, firstClassDate, count, selectHoursPanelExpand } = this.state;
    const modalCls = {
      [styles.wrap]: 1,
      [styles.modal]: 1,
    };
    return (
      <Modal
        className={classnames(modalCls)}
        title="指定辅班选课"
        width={1040}
        onOk={() => onConfirm(assistClassId, firstClassDate, count)}
        onCancel={onClose}
        noLine
      >
        <div className={styles.main}>
          {!!assistClassList &&
            assistClassList.length > 0 &&
            assistClassList.map((d) => {
              const { id, name, teacher, num, numTotal, time, address, classHoursList } = d;
              const rowData1 = [
                { name: '辅导老师', content: teacher },
                {
                  name: '已上课时',
                  content: (
                    <span>
                      <span>{classHoursList.filter((d) => d.isDone).length}</span>
                      <span>/</span>
                      <span className={styles.textSecondary}>{classHoursList.length}</span>
                    </span>
                  ),
                },
                {
                  name: '剩余名额',
                  content: (
                    <span>
                      <span>{numTotal - num}</span>
                      <span>/</span>
                      <span className={styles.textSecondary}>{numTotal}</span>
                    </span>
                  ),
                },
                { name: '地点', content: address },
              ];
              const rowData2 = [
                { name: '上课时间', content: time },
                {
                  name: '首次上课日期',
                  content: (
                    <div>
                      {firstClassDate}
                      <IconButton
                        type={'edit_mc_h'}
                        hoverType={'edit_mc_h'}
                        style={{ marginLeft: 10, position: 'relative', top: 1 }}
                        onClick={() => this.setState({ selectHoursPanelExpand: !selectHoursPanelExpand })}
                      />
                    </div>
                  ),
                  hide: assistClassId !== id,
                },
              ];
              const isFull = num === numTotal;
              return (
                <div key={id} className={styles.row}>
                  <div className={styles.rowHeader}>
                    <Radio checked={assistClassId === id} onChange={() => this.selectClass(id)} disabled={isFull} />
                    <div className={styles.rowName}>{name}</div>
                    {isFull && <div className={styles.warningText}>已满额</div>}
                  </div>
                  <div className={styles.rowBody}>
                    <FormItems data={rowData1} />
                    <FormItems data={rowData2} />
                  </div>
                  {assistClassId === id && selectHoursPanelExpand && (
                    <div className={styles.panel}>
                      <div className={styles.panelTitle}>选择首次上课日期</div>
                      <SelectClassHours
                        classHoursList={classHoursList}
                        firstClassDate={firstClassDate}
                        count={count}
                        onSelect={(firstClassDate, count) => this.setState({ firstClassDate, count })}
                      />
                    </div>
                  )}
                </div>
              );
            })}
          {(!assistClassList || assistClassList.length === 0) && <NoData />}
        </div>
      </Modal>
    );
  }
}

export default SelectAssistClassModal;
