import React from 'react';
import SelectClassHours from '../SelectClassHours';
import classnames from 'classnames';
import styles from './styles.less';
import Modal from 'COMPONENT/Modal';
import loaders from '../../loaders';
import NoData from '../../NoData';

class SelectClassHoursModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false,
      classHoursList: props.classHoursList || [],
      firstClassDate: props.firstClassDate,
      count: props.count,
    };
  }

  componentDidMount() {
    if (!this.props.classHoursList) {
      this.setState({ loading: true });
      loaders.loadClassHoursList(this.props.productType, this.props.classId).then(
        (classHoursList) => {
          this.setState({ loading: false, classHoursList });
        },
        () => {
          this.setState({ loading: false });
        },
      );
    }
  }

  render() {
    const { onClose, onConfirm } = this.props;
    const { loading, classHoursList, firstClassDate, count } = this.state;
    const modalCls = {
      [styles.wrap]: 1,
      [styles.modal]: 1,
    };
    return (
      <Modal
        loading={loading}
        className={classnames(modalCls)}
        title="选择首次上课日期"
        width={980}
        onOk={() => onConfirm(firstClassDate, count, classHoursList)}
        onCancel={onClose}
        noLine
      >
        <div className={styles.main}>
          {!!classHoursList && classHoursList.length > 0 ? (
            <SelectClassHours
              classHoursList={classHoursList}
              firstClassDate={firstClassDate}
              count={count}
              onSelect={(firstClassDate, count) => this.setState({ firstClassDate, count })}
            />
          ) : (
            <NoData />
          )}
        </div>
      </Modal>
    );
  }
}

export default SelectClassHoursModal;
