import React from 'react';
import Cart from '../Cart';
import styles from './styles.less';
import dicts from '../dicts';
import loaders from '../loaders';
import Search from 'COMPONENT/Search';
import { getDictTextByVal } from 'UTIL/dict/index';
import List from 'COMPONENT/List';
import TextLink from 'COMPONENT/TextLink';
import { formatCurrency } from 'COMPONENT/utils/basic/num';
import InputNumber from 'COMPONENT/InputNumber';
import FormItem from 'COMPONENT/FormItem';
import Icon from 'COMPONENT/Icon';
import PromotionInfo from './PromotionInfo';
import { loadCommonDict } from 'UTIL/dict';
import CourseInfoLink from '../CourseInfoLink';
import ProductTypeTag from '../ProductTypeTag';
import ProductInfoBodyMini from './ProductInfoBodyMini';
import ProductInfoBodyTwo from './ProductInfoBodyTwo';
import SelectClassHoursModal from './SelectClassHoursModal';
import FormItems from '../FormItems';
import classnames from 'classnames';
import showMessage from '../../../../components/utils/globalTips/showMessage';

const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;
const productTypeFieldName = 'productType';
const headerFormItemCfg = {
  labelWidth: 40,
  style: { marginBottom: 0, padding: 0 },
};
class SelectProductMain extends React.Component {
  constructor(props) {
    super(props);
    const { types } = props;
    this.productTypeDict = [];
    let conditionValues = {};
    types.forEach((type) => {
      const typeInfo = dicts.productTypeDict.find((d) => d.val === type);
      this.productTypeDict.push({ val: type, text: typeInfo.text });
      typeInfo.conditions.forEach((c) => {
        conditionValues[c.field] = null;
      });
    });
    this.defaultSelectedType = this.productTypeDict[0].val;
    conditionValues[productTypeFieldName] = this.defaultSelectedType;
    const inputValue = '';
    this.state = {
      conditionValues, // {object} 筛选条件值
      inputValue, // {string} 搜索框值
      dynDicts: {}, // {object} 动态获取的字典集合
      list: [], // {Array} 列表数据
      listSearchKeys: { ...conditionValues, inputValue }, // {object} 列表数据的查询关键字
      totalRecords: 0, // {number} 总记录数
      currentPage: 1, // {number} 当前页码
      pageSize: 10, // {number} 每页记录数
      expandedRowKeys: [], // {number[]} 列表展开的行
      selectClassHoursModalVis: false, // {boolean} 小班或双师选择课时弹框显示状态
      currentRecord: null, // {object} 当前操作的行数据
    };
  }

  componentDidMount = () => {
    const { onUpdateParams } = this.props;
    const { listSearchKeys, currentPage, pageSize } = this.state;
    onUpdateParams({ loading: true });
    Promise.all([
      loaders.loadCampusDict(),
      loadCommonDict('STUDENT_GRADE'),
      loadCommonDict('SUBJECT'),
      loadCommonDict('COURSE_SERIES'),
      loadCommonDict('PRODUCT_GROUP'),
      loadCommonDict('PRODUCT_QUARTER'),
      loadCommonDict('SMALL_CLASS_PHASE'),
      loadCommonDict('CLASS_TYPE', 'SMALL_CLASS'),
      loadCommonDict('PRODUCT_FEE_TYPE'),
      loadCommonDict('CLASS_TYPE', 'TWO_TEACHER'),
      loadCommonDict('CLASS_TYPE', 'ECS_CLASS'),
      loadCommonDict('PRODUCT_CLASS_MODAL'),
      loaders.loadProductList(this.setProductTypeSearchKey({ ...listSearchKeys, currentPage, pageSize })),
    ]).then(
      ([
        campusDict,
        gradeDict,
        subjectDict,
        productSeriesDict,
        productGroupDict,
        quarterDict,
        phaseDict,
        levelDict,
        feeTypeDict,
        twoClassTypeDict,
        promiseClassTypeDict,
        miniClassTypeDict,
        listData,
      ]) => {
        const dynDicts = {
          campusDict,
          gradeDict,
          subjectDict,
          productSeriesDict,
          productGroupDict,
          quarterDict,
          phaseDict,
          levelDict,
          otmClassTypeDict: dicts.otmClassTypeDict,
          miniClassTypeDict,
          twoClassTypeDict,
          promiseClassTypeDict,
          feeTypeDict,
        };
        const { list, totalRecords } = listData;
        this.setState({ dynDicts, list, totalRecords });
        onUpdateParams({ loading: false });
      },
    );
  };

  getConditions = () => {
    const { types } = this.props;
    const { dynDicts, conditionValues } = this.state;
    const selectedType = conditionValues[productTypeFieldName];
    let conditions = [];
    types.forEach((type) => {
      const typeInfo = dicts.productTypeDict.find((d) => d.val === type);
      if (!selectedType || selectedType === type) {
        typeInfo.conditions.forEach((c) => {
          if (!conditions.some((co) => co === c)) {
            conditions.push(c);
          }
        });
      }
    });
    conditions = conditions.map((c) => {
      return {
        title: c.title,
        field: c.field,
        isMultiSelect: c.isMultiSelect,
        hideUnlimited: c.hideUnlimited,
        dict: dynDicts && dynDicts[c.dictName] ? dynDicts[c.dictName] : [],
      };
    });
    conditions = [
      {
        title: '产品类型',
        field: productTypeFieldName,
        isMultiSelect: false,
        hideUnlimited: true,
        dict: this.productTypeDict,
      },
      ...conditions,
    ];
    if (conditions.length > 6) {
      const showConditions = conditions.slice(0, 5);
      const moreConditions = conditions.slice(5);
      let moreTitle = [];
      let moreDict = {};
      let moreIsMultiSelect = {};
      let moreHideUnlimited = {};
      moreConditions.forEach((c) => {
        moreTitle.push({ val: c.field, text: c.title });
        moreDict[c.field] = c.dict;
        moreIsMultiSelect[c.field] = c.isMultiSelect;
        moreHideUnlimited[c.field] = c.hideUnlimited;
      });
      conditions = [
        ...showConditions,
        {
          type: 'moreSelect',
          title: moreTitle,
          dict: moreDict,
          isMultiSelect: moreIsMultiSelect,
          hideUnlimited: moreHideUnlimited,
        },
      ];
    }
    return conditions;
  };

  getPlaceHolder = () => {
    const { conditionValues } = this.state;
    const productType = conditionValues[productTypeFieldName];
    let searchNames = '产品名称';
    if (productType === dicts.PRODUCT_TYPE_MINI) {
      searchNames = '产品/老师名称';
    } else if (productType === dicts.PRODUCT_TYPE_TWO) {
      searchNames = '产品/班级/老师名称';
    }
    return `请输入${searchNames}搜索`;
  };

  setExpand = (idx, expand) => {
    const { expandedRowKeys: expandedRowKeysState } = this.state;
    let expandedRowKeys;
    if (expand) {
      expandedRowKeys = [...expandedRowKeysState, idx];
    } else {
      expandedRowKeys = expandedRowKeysState.filter((key) => key !== idx);
    }
    this.setState({ expandedRowKeys });
  };

  countChange = (idx, val) => {
    this.setState({
      list: this.state.list.map((d, i) => {
        if (i === idx) {
          return { ...d, count: val };
        }
        return d;
      }),
    });
  };

  onAddToCart = (record) => {
    const { id, productId, classId, type, name, count, price, promotionList, promotionInfo, infos, additionProducts } =
      record;
    if (type === dicts.PRODUCT_TYPE_MINI || type === dicts.PRODUCT_TYPE_TWO) {
      this.setState({ selectClassHoursModalVis: true, currentRecord: record });
    } else {
      this.addToCard({
        id,
        productId,
        classId,
        type,
        name,
        count,
        price,
        promotionList,
        promotionInfo,
        infos,
        additionProducts,
      });
    }
  };

  addToCard = (data) => {
    this.props.onUpdateParams({
      cartList: [...this.props.cartList, data],
    });
  };

  onPaginationChange = ({ current, pageSize }) => {
    const { listSearchKeys } = this.state;
    const pageInfo = { currentPage: current, pageSize };
    this.props.onUpdateParams({ loading: true });
    loaders
      .loadProductList(this.setProductTypeSearchKey({ ...listSearchKeys, ...pageInfo }))
      .then(({ list, totalRecords }) => {
        this.setState({ ...pageInfo, list, totalRecords, expandedRowKeys: [] });
        this.props.onUpdateParams({ loading: false });
      });
  };

  search = (searchKeys) => {
    const { inputValue, ...conditionValues } = searchKeys;
    const currentFileds = [
      ...dicts.productTypeDict
        .find((d) => d.val === conditionValues[productTypeFieldName])
        .conditions.map((c) => c.field),
      productTypeFieldName,
    ];
    let newConditionValues = {};
    Object.keys(this.state.conditionValues).forEach((key) => {
      newConditionValues[key] = currentFileds.indexOf(key) >= 0 ? conditionValues[key] : null;
    });
    const { pageSize } = this.state;
    this.props.onUpdateParams({ loading: true });
    loaders
      .loadProductList(this.setProductTypeSearchKey({ inputValue, ...newConditionValues, pageSize, currentPage: 1 }))
      .then(
        ({ list, totalRecords }) => {
          this.setState({
            inputValue,
            conditionValues: newConditionValues,
            listSearchKeys: { inputValue, ...newConditionValues },
            currentPage: 1,
            list,
            totalRecords,
            expandedRowKeys: [],
          });
          this.props.onUpdateParams({ loading: false });
        },
        (e) => {
          showMessage(e.text || '获取数据错误');
          this.props.onUpdateParams({ loading: false });
        },
      );
  };

  setProductTypeSearchKey = (params) => {
    const { types, promotionId } = this.props;
    let otherTypes = [];
    types.forEach((t) => {
      const otherType = dicts.productTypeDict.find((p) => p.val === t).otherType;
      if (!!otherType) {
        otherTypes.push(otherType);
      }
    });
    return { ...params, otherTypes: otherTypes.length > 0 ? otherTypes : null, promotionId: promotionId || null };
  };

  cartInfoChange = (id, changed) => {
    this.props.onUpdateParams({
      cartList: this.props.cartList.map((d) => {
        if (d.id === id) {
          return { ...d, ...changed };
        }
        return d;
      }),
    });
  };

  getProductInfoCell = (text, record, index) => {
    const { expandedRowKeys } = this.state;
    const { type, name, promotionInfo } = record;
    const expanded = expandedRowKeys.indexOf(index) >= 0;
    return (
      <div className={styles.productInfo}>
        <div className={styles.productInfoHeader}>
          <div className={styles.productInfoTag}>
            <ProductTypeTag type={type} />
          </div>
          <div className={styles.productInfoName}>{name}</div>
          {!!promotionInfo && promotionInfo.length > 0 && (
            <TextLink className={styles.productInfoBtn} onClick={() => this.setExpand(index, !expanded)}>
              {expanded ? '收起优惠' : '查看优惠'}
            </TextLink>
          )}
        </div>
      </div>
    );
  };

  getProductInfoBody = (record, index) => {
    const { cartList } = this.props;
    const { type, name, id, teacher, grade } = record;
    const itemInCart = cartList.find((d) => d.id === id);
    let infoBody;
    if (type === dicts.PRODUCT_TYPE_MINI) {
      infoBody = (
        <ProductInfoBodyMini
          data={record}
          itemInCart={itemInCart}
          cartInfo={itemInCart ? { firstClassDate: itemInCart.firstClassDate, count: itemInCart.count } : {}}
          onCartInfoChange={(changed) => this.cartInfoChange(id, changed)}
        />
      );
    } else if (type === dicts.PRODUCT_TYPE_TWO) {
      infoBody = (
        <ProductInfoBodyTwo
          data={record}
          itemInCart={itemInCart}
          cartInfo={
            itemInCart
              ? {
                  dispatchType: itemInCart.dispatchType,
                  assistClassId: itemInCart.assistClassId,
                  firstClassDate: itemInCart.firstClassDate,
                  count: itemInCart.count,
                }
              : {}
          }
          onCartInfoChange={(changed) => this.cartInfoChange(id, changed)}
        />
      );
    }
    return infoBody;
  };

  expandedRowRender = (record, idx) => {
    const { onUpdateParams, types } = this.props;
    const { expandedRowKeys } = this.state;
    const discountCls = {
      [styles.promotionInfo]: 1,
      [styles.promotionInfoHide]: expandedRowKeys.indexOf(idx) < 0,
    };
    return (
      <div className={styles.expandWrap}>
        <div className={styles.productInfoBody}>{this.getProductInfoBody(record, idx)}</div>
        <div className={classnames(discountCls)}>
          <PromotionInfo data={record.promotionInfo} onUpdateParams={onUpdateParams} types={types} />
        </div>
      </div>
    );
  };

  render() {
    const { cartList, onUpdateParams } = this.props;
    const {
      conditionValues,
      inputValue,
      list,
      totalRecords,
      currentPage,
      pageSize,
      expandedRowKeys,
      selectClassHoursModalVis,
      currentRecord,
    } = this.state;
    const conditions = this.getConditions();
    const columns = [
      { title: '优惠信息', dataIndex: 'productInfo', width: COL_WIDTH_L, render: this.getProductInfoCell },
      {
        title: '单价',
        dataIndex: 'price',
        width: COL_WIDTH_L,
        render: (text, record, index) => {
          return <div className={styles.price}>{formatCurrency(text, '￥')}</div>;
        },
      },
      {
        title: '数量',
        dataIndex: 'count',
        width: COL_WIDTH_L,
        render: (text, record, index) => {
          const { count, id, type } = record;
          const itemInCart = cartList.find((d) => d.id === id);
          if (
            !!itemInCart ||
            type === dicts.PRODUCT_TYPE_MINI ||
            type === dicts.PRODUCT_TYPE_TWO ||
            type === dicts.PRODUCT_TYPE_PROMISE
          ) {
            return <div className={styles.countText}>{!!itemInCart ? itemInCart.count : count}</div>;
          }
          return <InputNumber value={count} onChange={(val) => this.countChange(index, val)} min={1} precision={0} />;
        },
      },
      {
        title: '操作',
        dataIndex: 'action',
        render: (text, record, index) => {
          const { id } = record;
          const itemInCart = cartList.find((d) => d.id === id);
          const iconComp = <Icon type="star" className={styles.cartIcon} size={12} />;
          if (!!itemInCart) {
            return (
              <div className={styles.addCartBtn + ' ' + styles.added}>
                {iconComp}
                <span>已加入</span>
              </div>
            );
          }
          return (
            <div className={styles.addCartBtn}>
              <TextLink onClick={() => this.onAddToCart(record)}>
                {iconComp}
                <span>加入购物车</span>
              </TextLink>
            </div>
          );
        },
      },
    ];
    return (
      <div className={styles.wrap}>
        <Search
          conditions={conditions}
          selectedDatas={{ inputValue, ...conditionValues }}
          onInitFinish={() => {}}
          onSearch={this.search}
          onInputChange={(inputValue) => this.setState({ inputValue })}
          isShowResult={false}
          conditionWidth={150}
          placeholder={this.getPlaceHolder()}
        />
        <List
          dataSource={list}
          columns={columns}
          pagination={{
            current: currentPage,
            total: totalRecords,
            pageSize,
          }}
          onChange={this.onPaginationChange}
          expandedRowKeys={list.map((d, i) => i)}
          expandedRowRender={this.expandedRowRender}
        />
        <Cart cartList={cartList} onUpdateParams={onUpdateParams} />
        {selectClassHoursModalVis && (
          <SelectClassHoursModal
            productType={currentRecord.type}
            classId={currentRecord.classId}
            firstClassDate={currentRecord.firstClassDate}
            count={currentRecord.count}
            onConfirm={(firstClassDate, count, classHoursList) => {
              const {
                id,
                productId,
                classId,
                type,
                name,
                price,
                promotionList,
                promotionInfo,
                infos,
                additionProducts,
              } = currentRecord;
              this.addToCard({
                id,
                productId,
                classId,
                type,
                name,
                count,
                price,
                promotionList,
                promotionInfo,
                infos,
                additionProducts,
                firstClassDate,
              });
              this.setState({
                selectClassHoursModalVis: false,
                list: this.state.list.map((li) => {
                  if (li.id === id) {
                    return { ...li, classHoursList };
                  }
                  return li;
                }),
              });
            }}
            onClose={() => this.setState({ selectClassHoursModalVis: false })}
          />
        )}
      </div>
    );
  }
}

export default SelectProductMain;
