import React from 'react';
import Block from 'COMPONENT/Block';
import FormItem from 'COMPONENT/FormItem';
import { Row, Col } from 'antd';
import moment from 'moment';
import { FIELD_FILLER, checkFieldEmpty, getFieldWithFiller, getCommonErrorTips } from 'COMPONENT/utils/form';
import { getDictTextByVal } from 'UTIL/dict';
import { CONTRACT_TYPE_AGAIN, contractTypeDict } from '../dicts';
import CourseInfoLink from '../CourseInfoLink';
import styles from './styles.less';

const formCfg = {
  labelWidth: 80,
  style: { padding: '0 30px', marginBottom: 15 },
};

class StudentInfo extends React.Component {
  getToolBar = () => {
    const {
      stuInfo,
      stuInfo: { contractType },
      courseInfo,
      onUpdateParams,
    } = this.props;
    return contractType === CONTRACT_TYPE_AGAIN ? (
      <CourseInfoLink stuInfo={stuInfo} courseInfo={courseInfo} onUpdateParams={onUpdateParams} />
    ) : null;
  };

  render() {
    const {
      user,
      stuInfo: { contractType, cusName, contract, campusName, stuName, gradeName, schoolName },
    } = this.props;
    return (
      <div className={styles.wrap}>
        <Block title="学生信息" headerToolBar={this.getToolBar()}>
          <div className={styles.tips}>
            <span>签约人：</span>
            <span>{user.name}</span>
            <span style={{ marginLeft: 20 }}>签约日期：</span>
            <span>{moment().format('YYYY.MM.DD')}</span>
          </div>
          <Row style={{ margin: '20px -30px 0 -30px' }}>
            <Col span={8}>
              <FormItem {...formCfg} name="客户姓名">
                {getFieldWithFiller(cusName)}
              </FormItem>
            </Col>
            <Col span={8}>
              <FormItem {...formCfg} name="联系方式">
                {getFieldWithFiller(contract)}
              </FormItem>
            </Col>
            <Col span={8}>
              <FormItem {...formCfg} name="学习中心">
                {getFieldWithFiller(campusName)}
              </FormItem>
            </Col>
            <Col span={8}>
              <FormItem {...formCfg} name="学生姓名">
                {getFieldWithFiller(stuName)}
              </FormItem>
            </Col>
            <Col span={8}>
              <FormItem {...formCfg} name="年级">
                {getFieldWithFiller(gradeName)}
              </FormItem>
            </Col>
            <Col span={8}>
              <FormItem {...formCfg} name="学校">
                {getFieldWithFiller(schoolName)}
              </FormItem>
            </Col>
            <Col span={8}>
              <FormItem {...formCfg} name="合同类型">
                {getDictTextByVal(contractTypeDict, contractType, true)}
              </FormItem>
            </Col>
          </Row>
        </Block>
      </div>
    );
  }
}

export default StudentInfo;
