import { LIST_TYPE_MY, LIST_TYPE_OTHER, COURSE_STATUS_START, COURSE_STATUS_NOT_START, courseStatusDict } from './dicts';
import { getCampusByLoginUser } from 'SERVICE/common';
import { SEX_MALE, SEX_FEMALE } from 'PAGE/common/dicts';
import dicts from './dicts';
import utils from './utils';
import { getDictTextByVal } from '../../../utils/dict/index';
import {
  getStudentInfo,
  getCrossBranchStudentInfo,
  getContractPromotions,
  saveContractInfo,
  saveCusAndStuInfoInContract,
  checkStudentByNameAndPhone,
  getProductForContractSelect,
  getMiniClassCoursesById,
  getTwoTeacherClassTwoInfoById,
  getStudentMiniClassInfo,
  getClassProduct,
  getTwoTeacherClassProduct,
} from 'SERVICE/contract';

const loadList = async (params) => {
  const { type, ...listParams } = params;
  if (type === LIST_TYPE_MY) {
    return await loadListMy(listParams);
  } else if (type === LIST_TYPE_OTHER) {
    return await loadListOther(listParams);
  } else {
    const [listMy, listOther] = await Promise.all([loadListMy(listParams), loadListOther(listParams)]);
    return {
      [LIST_TYPE_MY]: listMy,
      [LIST_TYPE_OTHER]: listOther,
    };
  }
};

const loadListMy = async ({ stuName, cusPhone, cusName, currentPage, pageSize }) => {
  const apiParams = {
    studentName: stuName,
    cusContact: cusPhone,
    customerName: cusName,
    pageNo: currentPage - 1,
    pageSize: pageSize,
  };
  const apiData = await getStudentInfo(apiParams);
  const list = (apiData && apiData.datas ? apiData.datas : []).map((d) => {
    return {
      cusId: d.cusId,
      stuId: d.stuId,
      cusName: d.cusName,
      stuNo: d.stuId,
      stuName: d.stuName,
      phone: d.cusContact,
      sex: d.sex === '1' ? SEX_FEMALE : d.sex === '0' ? SEX_MALE : null,
      stuSchool: d.schoolName,
      grade: d.gradeId,
      gradeName: d.gradeName,
      studyManager: d.studyManager,
      campus: d.blCampusId,
      campusName: d.blCampusName,
    };
  });
  return { list, totalRecords: apiData && apiData.rowCount ? apiData.rowCount : 0 };
  /*
  return {
    list: [
      { cusId: '1', stuId: 'stu1', cusName: '王女士', stuNo: 'X315', stuName: '天天', phone: '13444444444',
        sex: SEX_MALE, stuSchool: '培正中学', gradeName: '高一', studyManager: '刘主任', campusName: '莞城学习中心'  },
      { cusId: '2', stuId: 'stu1', cusName: '王女士2', stuNo: 'X315', stuName: '天天2', phone: '13444444444',
        sex: SEX_MALE, stuSchool: '培正中学', gradeName: '高一', studyManager: '刘主任', campusName: '莞城学习中心'  },
      { cusId: '1', stuId: 'stu1', cusName: '王女士', stuNo: 'X315', stuName: '天天', phone: '13444444444',
        sex: SEX_MALE, stuSchool: '培正中学', gradeName: '高一', studyManager: '刘主任', campusName: '莞城学习中心'  },
      { cusId: '1', stuId: 'stu1', cusName: '王女士', stuNo: 'X315', stuName: '天天', phone: '13444444444',
        sex: SEX_MALE, stuSchool: '培正中学', gradeName: '高一', studyManager: '刘主任', campusName: '莞城学习中心'  },
      { cusId: '1', stuId: 'stu1', cusName: '王女士', stuNo: 'X315', stuName: '天天', phone: '13444444444',
        sex: SEX_MALE, stuSchool: '培正中学', gradeName: '高一', studyManager: '刘主任', campusName: '莞城学习中心'  },
      { cusId: '1', stuId: 'stu1', cusName: '王女士', stuNo: 'X315', stuName: '天天', phone: '13444444444',
        sex: SEX_MALE, stuSchool: '培正中学', gradeName: '高一', studyManager: '刘主任', campusName: '莞城学习中心'  },
      { cusId: '1', stuId: 'stu1', cusName: '王女士', stuNo: 'X315', stuName: '天天', phone: '13444444444',
        sex: SEX_MALE, stuSchool: '培正中学', gradeName: '高一', studyManager: '刘主任', campusName: '莞城学习中心'  },
      { cusId: '1', stuId: 'stu1', cusName: '王女士', stuNo: 'X315', stuName: '天天', phone: '13444444444',
        sex: SEX_MALE, stuSchool: '培正中学', gradeName: '高一', studyManager: '刘主任', campusName: '莞城学习中心'  },
      { cusId: '1', stuId: 'stu1', cusName: '王女士', stuNo: 'X315', stuName: '天天', phone: '13444444444',
        sex: SEX_MALE, stuSchool: '培正中学', gradeName: '高一', studyManager: '刘主任', campusName: '莞城学习中心'  },
      { cusId: '1', stuId: 'stu1', cusName: '王女士', stuNo: 'X315', stuName: '天天', phone: '13444444444',
        sex: SEX_MALE, stuSchool: '培正中学', gradeName: '高一', studyManager: '刘主任', campusName: '莞城学习中心'  },
      { cusId: '1', stuId: 'stu1', cusName: '王女士', stuNo: 'X315', stuName: '天天', phone: '13444444444',
        sex: SEX_MALE, stuSchool: '培正中学', gradeName: '高一', studyManager: '刘主任', campusName: '莞城学习中心'  },
      { cusId: '1', stuId: 'stu1', cusName: '王女士', stuNo: 'X315', stuName: '天天', phone: '13444444444',
        sex: SEX_MALE, stuSchool: '培正中学', gradeName: '高一', studyManager: '刘主任', campusName: '莞城学习中心'  },
      { cusId: '1', stuId: 'stu1', cusName: '王女士', stuNo: 'X315', stuName: '天天', phone: '13444444444',
        sex: SEX_MALE, stuSchool: '培正中学', gradeName: '高一', studyManager: '刘主任', campusName: '莞城学习中心'  },
      { cusId: '1', stuId: 'stu1', cusName: '王女士', stuNo: 'X315', stuName: '天天', phone: '13444444444',
        sex: SEX_MALE, stuSchool: '培正中学', gradeName: '高一', studyManager: '刘主任', campusName: '莞城学习中心'  },
      { cusId: '1', stuId: 'stu1', cusName: '王女士', stuNo: 'X315', stuName: '天天', phone: '13444444444',
        sex: SEX_MALE, stuSchool: '培正中学', gradeName: '高一', studyManager: '刘主任', campusName: '莞城学习中心'  },
      { cusId: '1', stuId: 'stu1', cusName: '王女士', stuNo: 'X315', stuName: '天天', phone: '13444444444',
        sex: SEX_MALE, stuSchool: '培正中学', gradeName: '高一', studyManager: '刘主任', campusName: '莞城学习中心'  },
      { cusId: '1', stuId: 'stu1', cusName: '王女士', stuNo: 'X315', stuName: '天天', phone: '13444444444',
        sex: SEX_MALE, stuSchool: '培正中学', gradeName: '高一', studyManager: '刘主任', campusName: '莞城学习中心'  },
      { cusId: '1', stuId: 'stu1', cusName: '王女士', stuNo: 'X315', stuName: '天天', phone: '13444444444',
        sex: SEX_MALE, stuSchool: '培正中学', gradeName: '高一', studyManager: '刘主任', campusName: '莞城学习中心'  },
      { cusId: '1', stuId: 'stu1', cusName: '王女士', stuNo: 'X315', stuName: '天天', phone: '13444444444',
        sex: SEX_MALE, stuSchool: '培正中学', gradeName: '高一', studyManager: '刘主任', campusName: '莞城学习中心'  },
      { cusId: '1', stuId: 'stu1', cusName: '王女士', stuNo: 'X315', stuName: '天天', phone: '13444444444',
        sex: SEX_MALE, stuSchool: '培正中学', gradeName: '高一', studyManager: '刘主任', campusName: '莞城学习中心'  },
      { cusId: '1', stuId: 'stu1', cusName: '王女士', stuNo: 'X315', stuName: '天天', phone: '13444444444',
        sex: SEX_MALE, stuSchool: '培正中学', gradeName: '高一', studyManager: '刘主任', campusName: '莞城学习中心'  },
      { cusId: '1', stuId: 'stu1', cusName: '王女士', stuNo: 'X315', stuName: '天天', phone: '13444444444',
        sex: SEX_MALE, stuSchool: '培正中学', gradeName: '高一', studyManager: '刘主任', campusName: '莞城学习中心'  },
    ],
    totalRecords: 21,
  }*/
};

const loadListOther = async ({ stuName, cusPhone, cusName, currentPage, pageSize }) => {
  /*
  await new Promise((res) => {
    setTimeout(() => {res()}, 2000)
  })*/

  const apiParams = {
    studentName: stuName,
    cusContact: cusPhone,
    customerName: cusName,
    pageNo: currentPage - 1,
    pageSize: pageSize,
  };
  const apiData = await getCrossBranchStudentInfo(apiParams);
  const list = (apiData && apiData.datas ? apiData.datas : []).map((d) => {
    return {
      cusId: d.cusId,
      stuId: d.stuId,
      cusName: d.cusName,
      stuNo: d.stuId,
      stuName: d.stuName,
      phone: d.cusContact,
      sex: d.sex === '1' ? SEX_FEMALE : d.sex === '0' ? SEX_MALE : null,
      stuSchool: d.schoolName,
      grade: d.gradeId,
      gradeName: d.gradeName,
      studyManager: d.studyManager,
      campus: d.blCampusId,
      campusName: d.blCampusName,
    };
  });
  return { list, totalRecords: apiData && apiData.rowCount ? apiData.rowCount : 0 };
  /*
  return {
    list: [
      { cusId: '1', stuId: 'stu1', cusName: '李先正', stuNo: 'X315', stuName: '天天', phone: '13444444444',
        sex: SEX_MALE, stuSchool: '培正中学', gradeName: '高一', studyManager: '刘主任', campusName: '莞城学习中心'  },
    ],
    totalRecords: 1,
  }*/
};

const checkDuplicateStudent = async ({ stuName, cusPhone }) => {
  try {
    return await checkStudentByNameAndPhone({ cusContact: cusPhone, studentName: stuName });
  } catch (e) {
    console.log(e);
  }
  //return true
};

const loadStuCourseList = async ({ cusId, stuId }) => {
  const apiData = await getStudentMiniClassInfo({ studentId: stuId });
  const list = apiData
    ? apiData.map((d) => {
        return {
          type: dicts.PRODUCT_TYPE_MINI,
          name: d.miniClassName,
          grade: d.gradeName,
          subject: d.subjectName,
          teacher: d.teacherName,
          address: `${d.blCampusName || ''} ${d.classRoom || ''}`,
          time: `${d.startTime || '-'}~${d.endTime || '-'} ${d.courseTime}`,
          status:
            d.status === 'PENDDING_START'
              ? COURSE_STATUS_NOT_START
              : d.status === 'STARTED' || d.status === 'CONPELETE'
              ? COURSE_STATUS_START
              : null,
          endDate: d.endTime,
        };
      })
    : [];
  return list;
  /*
  await new Promise(res => { setTimeout(() => { res() }, 2000) })
  const list = [
    { type: dicts.PRODUCT_TYPE_PROMISE, name: '2017年湛江夏季高三数学提高收费班', grade: '高三', subject: '英语', teacher: '王老师',
      address: '柠溪学习中心 201教室', time: '2017.07.01～2017.07.15 星期六08:00～10:00', status: COURSE_STATUS_START, endDate: '2018-02-22' },
    { type: dicts.PRODUCT_TYPE_OTO, name: '2017年湛江夏季高三数学提高收费班', grade: '高三', subject: '英语', teacher: '王老师',
      address: '柠溪学习中心 201教室', time: '2017.07.01～2017.07.15 星期六08:00～10:00', status: COURSE_STATUS_START, endDate: '2018-02-22' },
    { type: dicts.PRODUCT_TYPE_OTM, name: '2017年湛江夏季高三数学提高收费班', grade: '高三', subject: '英语', teacher: '王老师',
      address: '柠溪学习中心 201教室', time: '2017.07.01～2017.07.15 星期六08:00～10:00', status: COURSE_STATUS_NOT_START, endDate: '2018-02-22' },
  ]
  return list*/
};

const saveStudent = async ({ cusId, stuId, cusName, contract, campus, stuName, grade, school }) => {
  const params = {
    cusId,
    stuId,
    cusName: cusName,
    cusPhone: contract,
    blCampusId: campus,
    stuName: stuName,
    gradeId: grade,
    schoolId: school,
  };
  const apiData = await saveCusAndStuInfoInContract(params);
  return { cusId: apiData.cusId, stuId: apiData.stuId };

  //return { cusId: '1', stuId: '1' }
};

const getPromotionGroupList = (promotionList) => {
  let typeGroupList = [];
  promotionList.forEach((p) => {
    const promotion = { id: p.id, title: p.title, content: p.content };
    let typeGroup = typeGroupList.find((tg) => tg.type === p.type);
    if (!!typeGroup) {
      let modeGroupList = typeGroup.modes;
      let modeGroup = modeGroupList.find((mg) => mg.mode === p.mode);
      if (!!modeGroup) {
        modeGroup.promotions.push(promotion);
      } else {
        modeGroupList.push({ mode: p.mode, promotions: [promotion] });
      }
    } else {
      typeGroupList.push({
        type: p.type,
        name: p.name,
        canCollect: p.canCollect,
        modes: [{ mode: p.mode, promotions: [promotion] }],
      });
    }
  });
  return typeGroupList;
};

const getProductListApiParams = (params) => {
  const apiFieldsMap = {
    productType: 'productType',
    campus: 'blCampusId',
    grade: 'gradeId',
    subject: 'subject',
    productSeries: 'courseSeries',
    productGroup: 'productGroupId',
    quarter: 'seasonId',
    phase: 'phase',
    level: 'classTypeId',
    otmClassType: 'oneOnManyTypes',
    miniClassType: 'classModalId',
    twoClassType: 'classTypeId',
    promiseClassType: 'classTypeId',
    feeType: 'otherCategory',
    level: 'classTypeId',
  };
  const fields = dicts.productTypeDict.find((d) => d.val === params.productType).conditions.map((c) => c.field);
  let apiParams = {
    productType: params.productType,
    productName: params.inputValue,
    promotionId: params.promotionId,
    pageNo: params.currentPage - 1,
    pageSize: params.pageSize,
  };
  fields.forEach((f) => {
    apiParams[apiFieldsMap[f]] = params[f];
  });
  if (params.productType === dicts.PRODUCT_TYPE_OTHER && params.otherTypes) {
    apiParams['otherOfCategory'] = params.otherTypes.join(',');
  }
  return apiParams;
};

const getProductClassId = (productId, classId) => {
  return `${productId}${classId || ''}`;
};

const loadProductList = async (params) => {
  console.log(params);
  //productType, inputValue, campus, grade, subject, productSeries, productGroup, quarter, phase, level, otmClassType, miniClassType, twoClassType, promiseClassType, feeType, otherTypes, currentPage, pageSize

  const { productType } = params;
  const apiParams = getProductListApiParams(params);
  let apiData;
  if (productType === dicts.PRODUCT_TYPE_MINI) {
    apiData = await getClassProduct(apiParams);
  } else if (productType === dicts.PRODUCT_TYPE_TWO) {
    apiData = await getTwoTeacherClassProduct(apiParams);
  } else {
    apiData = await getProductForContractSelect(apiParams);
  }
  const datas = apiData && apiData.datas ? apiData.datas : [];
  const list = datas.map((d) => {
    const apiPromotionInfo = d.promotionInfo || [];
    const promotionList = apiPromotionInfo.map((p) => {
      const type = p.promotionKindId;
      const typeInfo = dicts.promotionTypeDict.find((di) => di.val === type);
      const ruleNames = p.ruleVos
        ? p.ruleVos
            .filter((ru) => !!ru.ruleName)
            .map((r) => r.ruleName)
            .join(',')
        : '';
      const rules = p.ruleVos
        ? p.ruleVos.map((r) => {
            return { id: r.id, content: r.ruleName };
          })
        : [];
      return {
        type: p.promotionKindId,
        name: typeInfo.text,
        canCollect: typeInfo.canCollect,
        mode: p.promotionLittleTypeId,
        id: p.id,
        title: p.promotionName,
        content: `${p.promotionTypeName ? p.promotionTypeName + '：' : ''}${ruleNames}`,
        littleModeName: p.promotionTypeName,
        rules,
      };
    });
    let infos = [];
    if (d.experienceLessonName) {
      infos.push(d.experienceLessonName);
    }
    const additionProducts = d.subProducts
      ? d.subProducts.map((s) => {
          return {
            id: s.productId,
            productId: s.productId,
            name: s.productName,
            price: s.price,
            count: s.totalHours,
          };
        })
      : [];
    let item = {
      id: getProductClassId(d.productId, d.classId),
      productId: d.productId,
      classId: d.classId,
      name: d.productName,
      type: productType,
      price: d.price,
      count: 1,
      infos,
      additionProducts,
      promotionList,
      promotionInfo: getPromotionGroupList(promotionList),
    };
    if (productType === dicts.PRODUCT_TYPE_TWO || productType === dicts.PRODUCT_TYPE_MINI) {
      item = {
        ...item,
        count: null,
        teacher: d.teacherName,
        num: d.classPeopleNum,
        numTotal: d.peopleNum,
        hours: d.consumeHours || 0,
        hoursTotal: d.totalHours || 0,
        time: `${d.startDate || '-'}~${d.endDate || '-'} ${d.classTime}`,
        address: `${d.campusName ? d.campusName + ' ' : ''}${d.roomName || ''}`,
        firstClassDate: null,
      };
      if (productType === dicts.PRODUCT_TYPE_TWO) {
        item = {
          ...item,
          mainTeacher: d.mainTeacherName,
          className: d.className,
        };
      }
    }
    return item;
  });
  return { list, totalRecords: apiData && apiData.rowCount ? apiData.rowCount : 0 };
  /*
  const productType = params.productType
  const promotionList = [
    { id: 'give_1', title: '暑期满赠优惠', content: '赠课：满200课时赠送5课时', type: dicts.PROMOTION_TYPE_ACTIVITY,
      name: getDictTextByVal(dicts.promotionTypeDict, dicts.PROMOTION_TYPE_ACTIVITY), canCollect: true, mode: dicts.PROMOTION_MODE_GIVE,
      littleModeName: '赠课', rules: [{ id: '1_1', content: '满200课时赠送5课时' }],
    },
    { id: 'reduce_1', title: '暑期满减优惠', content: '阶梯满减：满1000元减100元，满2000减250元，满3000减400元', type: dicts.PROMOTION_TYPE_ACTIVITY,
      name: getDictTextByVal(dicts.promotionTypeDict, dicts.PROMOTION_TYPE_ACTIVITY), canCollect: true, mode: dicts.PROMOTION_MODE_REDUCE,
      littleModeName: '阶梯满减', rules: [{ id: '2_1', content: '满1000元减100元' }, { id: '2_2', content: '满2000减250元' }, { id: '2_3', content: '满3000减400元' }],
    },
    { id: 'reduce_2', title: '暑期每满减优惠', content: '每满减：每满3000减300', type: dicts.PROMOTION_TYPE_ACTIVITY,
      name: getDictTextByVal(dicts.promotionTypeDict, dicts.PROMOTION_TYPE_ACTIVITY), canCollect: true, mode: dicts.PROMOTION_MODE_REDUCE,
      littleModeName: '每满减', rules: [{ id: '3_1', content: '每满3000减300' }],
    },
    { id: 'discount_1', title: '2科连报打8折', content: '课时折扣：满2个产品折80%', type: dicts.PROMOTION_TYPE_ACTIVITY,
      name: getDictTextByVal(dicts.promotionTypeDict, dicts.PROMOTION_TYPE_ACTIVITY), canCollect: true, mode: dicts.PROMOTION_MODE_DISCOUNT,
      littleModeName: '课时折扣', rules: [{ id: '4_1', content: '满2个产品折80%' }],
    },
    { id: '1', title: '满减券', content: '满减：满1000元减60元', type: dicts.PROMOTION_TYPE_TICKET,
      name: getDictTextByVal(dicts.promotionTypeDict, dicts.PROMOTION_TYPE_TICKET), canCollect: true, mode: dicts.PROMOTION_MODE_REDUCE_TICKET,
      littleModeName: '满减', rules: [{ id: '5_1', content: '满1000元减60元' }],
    },
  ]
  const promotionInfo = getPromotionGroupList(promotionList)
  console.log(promotionInfo)
  const listAll = [
    { id: 'oto_1', name: '2017年夏季高三一对一收费班1', type: dicts.PRODUCT_TYPE_OTO, price: 300, count: 1, promotionList, promotionInfo,
      infos: ['【赠品】体验3节 3×50'],
      additionProducts: [
        { id: '1', name: '【其他】一对一综合服务费', price: 100, count: 1 },
      ],
    },
    { id: 'otm_1', name: '2017年夏季高三一对多收费班1', type: dicts.PRODUCT_TYPE_OTM, price: 200, count: 1, promotionList, promotionInfo },
    { id: 'otm_2', name: '2017年夏季高三一对多收费班2', type: dicts.PRODUCT_TYPE_OTM, price: 200, count: 1, promotionList, promotionInfo },
    { id: 'otm_3', name: '2017年夏季高三一对多收费班3', type: dicts.PRODUCT_TYPE_OTM, price: 200, count: 1, promotionList, promotionInfo },
    { id: 'otm_4', name: '2017年夏季高三一对多收费班4', type: dicts.PRODUCT_TYPE_OTM, price: 200, count: 1, promotionList, promotionInfo },
    { id: 'otm_5', name: '2017年夏季高三一对多收费班5', type: dicts.PRODUCT_TYPE_OTM, price: 200, count: 1, promotionList, promotionInfo },
    { id: 'otm_6', name: '2017年夏季高三一对多收费班6', type: dicts.PRODUCT_TYPE_OTM, price: 200, count: 1, promotionList, promotionInfo },
    { id: 'otm_7', name: '2017年夏季高三一对多收费班7', type: dicts.PRODUCT_TYPE_OTM, price: 200, count: 1, promotionList: null, promotionInfo: null },
    { id: 'mini_1', name: '2017年夏季高三英语小班', type: dicts.PRODUCT_TYPE_MINI, price: 300, promotionList, promotionInfo,
     teacher: '胡成', num: 7, numTotal: 12, time: '2017.07.01～2017.07.15  星期六  08:00～10:00',
     address: '柠溪学习中心 201教室', firstClassDate: null, count: null, classId: 'class_mini_1', hours: 2, totalHours: 12,
    },
    { id: 'two_1', name: '2017年夏季高三英语双师1', type: dicts.PRODUCT_TYPE_TWO, price: 300, promotionList, promotionInfo,
      mainTeacher: '胡成', firstClassDate: null, count: null,
      classId: 'class_two_1', className: '2017年夏季高三英语双师辅班', teacher: '张三', num: 7, numTotal: 12, time: '2017.07.01～2017.07.15  星期六  08:00～10:00',
      address: '柠溪学习中心 201教室', hours: 2, totalHours: 12,
    },
    { id: 'two_2', name: '2017年夏季高三英语双师2', type: dicts.PRODUCT_TYPE_TWO, price: 300, promotionList, promotionInfo,
      mainTeacher: '胡成', firstClassDate: null, count: null,
      classId: 'class_two_2', className: '2017年夏季高三英语双师辅班', teacher: '张三', num: 7, numTotal: 12, time: '2017.07.01～2017.07.15  星期六  08:00～10:00',
      address: '柠溪学习中心 201教室', hours: 2, totalHours: 12,
    },
    { id: 'lectrue_1', name: '2017年夏季高三英语讲座', type: dicts.PRODUCT_TYPE_LECTRUE, price: 300, count: 1, promotionList, promotionInfo,
      teacher: '胡成', grade: '高三',
    },
  ]
  const list = listAll.filter(d => productType === d.type)
  return { list, totalRecords: 2 }*/
};

const loadTickets = async () => {
  return [
    { id: '1', name: '七月份优惠券', main: '￥100', secondary: '满20课时', info: '有效期至2017.06.15' },
    { id: '2', name: '八月份优惠券', main: '￥100', secondary: '满2000元', info: '有效期至2017.06.15' },
  ];
};

const loadCampusDict = async () => {
  const data = await getCampusByLoginUser({}, true);
  let dict = [];
  data.forEach((group) => {
    const campuses = group && group.length > 1 ? group.slice(1) : [];
    campuses.forEach((d) => {
      dict.push({ val: d.id, text: d.name });
    });
  });
  return dict;
};

const loadCheckoutInfo = async (cartList) => {
  try {
    let apiParams = [];
    cartList.forEach((c) => {
      const { productId, classId, count, price, additionProducts, selectedPromotion } = c;
      apiParams.push({
        productId,
        classId: classId || null,
        promotionId: selectedPromotion || null,
        quantity: count,
        realAmount: price * count,
      });
      if (additionProducts) {
        additionProducts.forEach((a) => {
          apiParams.push({
            productId: a.productId,
            quantity: a.count,
            realAmount: a.price * a.count,
          });
        });
      }
    });
    console.log(JSON.stringify(apiParams));
    const apiData = await getContractPromotions(apiParams);
    let totalAmount = 0;
    let discountAmount = 0;
    let discountDetail = [];
    let promotionGroupList = [];
    if (apiData && apiData.data) {
      apiData.data.forEach((d) => {
        const { promotionId, productId, classId, realAmount, promotionAmount, promotionHours, ruleId } = d;
        const pDetail = cartList.find((c) => c.id === getProductClassId(productId, classId));
        if (pDetail) {
          const { price, count, additionProducts, name, promotionList } = pDetail;
          totalAmount += price * count;
          discountAmount += promotionAmount;
          if (additionProducts && additionProducts.length > 0) {
            additionProducts.forEach((a) => {
              totalAmount += a.price * a.count;
            });
          }
          let gifts = [];
          if (promotionHours) {
            const total = price * promotionHours;
            totalAmount += total;
            discountAmount += total;
            gifts = [
              {
                mode: dicts.GIFT_MODE_COURSE,
                name,
                price,
                count: promotionHours,
                total,
                reduces: [{ name: '活动优惠', amount: total }],
              },
            ];
          }
          const product = {
            id: getProductClassId(productId, classId),
            productId,
            classId,
            reduces: promotionAmount ? [{ name: '活动优惠', amount: promotionAmount }] : [],
            gifts,
          };
          let group = promotionGroupList.find((p) => !!p.promotion && p.promotion.id === promotionId);
          if (!!group) {
            group.products.push(product);
          } else {
            let promotion = null;
            if (promotionId === -1) {
              promotion = { id: -1 };
            } else if (promotionId) {
              const promotionDetail = promotionList.find((p) => p.id === promotionId);
              const { title, content, mode, littleModeName, rules } = promotionDetail;
              promotion = { id: promotionId, mode, title, content };
              discountDetail.push(content);
            }
            promotionGroupList.push({ promotion, products: [product] });
          }
        }
      });
    }
    console.log({ promotionGroupList, totalAmount, discountAmount, discountDetail });
    return { promotionGroupList, totalAmount, discountAmount, discountDetail };
  } catch (e) {
    console.log(e);
    throw e;
  }
  /*
  const data = [
    { promotion: { id: 'give_1', mode: dicts.PROMOTION_MODE_GIVE, title: '暑假满赠优惠', content: '赠课：满200课时赠送5课时' },
      products: [
        { id: 'oto_1',
          reduces: [
            { name: '促销码优惠', amount: 100.15 },
          ],
          gifts: [
            { mode: dicts.GIFT_MODE_COURSE, name: '2017年夏季高三一对一收费班1', price: 300, count: 5, total: 1500,
              reduces: [
                { name: '活动优惠', amount: 1500 },
              ],
            },
          ],
        },
        { id: 'otm_1',
          reduces: [
            { name: '促销码优惠', amount: 100.15 },
          ],
        },
        { id: 'otm_2',
          reduces: [
            { name: '促销码优惠', amount: 100.15 },
          ],
        },
      ],
    },
    { promotion: { id: 'reduce_1', mode: dicts.PROMOTION_MODE_REDUCE, title: '暑假满减优惠', content: '阶梯满减：满1000元减100元' },
      products: [
        { id: 'otm_3',
          reduces: [
            { name: '活动优惠', amount: 100 },
            { name: '促销码优惠', amount: 100.15 },
          ],
        },
      ],
    },
    { promotion: { id: 'discount_1', mode: dicts.PROMOTION_MODE_DISCOUNT, title: '2科连报打8折', content: '课时折扣：满2个产品折80%' },
      products: [
        { id: 'otm_4',
          reduces: [
            { name: '活动优惠', amount: 40 },
          ],
        },
        { id: 'otm_5',
          reduces: [
            { name: '活动优惠', amount: 40 },
          ],
        },
      ],
    },
    { promotion: null,
      products: [
        { id: 'otm_6' },
      ],
    },
    { promotion: null,
      products: [
        { id: 'otm_7' },
      ],
    },
  ]

  const discountDetail = [
    '满1000元减100元', '课时折扣：满2个产品折80%', '课时折扣：满2个产品折80%', '满1000元减100元'
  ]
  await new Promise((res) => {
    setTimeout(() => {res()}, 1000)
  })
  return { promotionGroupList: data, totalAmount: 14110.15, discountAmount: 1110.15, discountDetail }*/
};

const loadClassHoursList = async (productType, classId) => {
  console.log(productType, classId);
  const apiParams = { classId };
  let apiData;
  if (productType === dicts.PRODUCT_TYPE_TWO) {
    apiData = await getTwoTeacherClassTwoInfoById(apiParams);
  } else {
    apiData = await getMiniClassCoursesById(apiParams);
  }
  return apiData && apiData.data && apiData.data.courseList
    ? apiData.data.courseList.map((c) => {
        return { date: c.courseDate, isDone: c.courseStatus !== 'NEW' };
      })
    : [];
  /*
  return [
    { date: '2017.06.16', isDone: true },
    { date: '2017.06.28', isDone: true },
    { date: '2017.06.30' },
    { date: '2017.07.02' },
    { date: '2017.07.14' },
    { date: '2017.07.18' },
    { date: '2017.07.20' },
    { date: '2017.07.22' },
    { date: '2017.07.24' },
    { date: '2017.07.26' },
    { date: '2017.07.28' },
    { date: '2017.07.30' },
  ]*/
};

const saveContract = async (params) => {
  console.log(params);
  try {
    const { cartList, contractType, cusId, stuId } = params;
    let cartInfos = [];
    cartList.forEach((c) => {
      cartInfos.push({
        classId: c.classId,
        price: c.price,
        productId: c.productId,
        promotionId: c.selectedPromotion,
        quantity: c.count,
        realAmount: c.price * c.count,
        startDate: c.firstClassDate,
      });
      if (c.additionProducts) {
        c.additionProducts.forEach((a) => {
          cartInfos.push({
            price: a.price,
            productId: a.productId,
            quantity: a.count,
            realAmount: a.price * a.count,
          });
        });
      }
    });
    const apiParams = {
      cartInfos,
      contractType,
      customerId: cusId,
      studentId: stuId,
    };
    await saveContractInfo(apiParams);
    return true;
  } catch (e) {
    console.log(e);
  }
};

export default {
  loadList,
  loadStuCourseList,
  saveStudent,
  loadProductList,
  loadTickets,
  loadCampusDict,
  loadClassHoursList,
  loadCheckoutInfo,
  checkDuplicateStudent,
  saveContract,
};
