import React from 'react';

import { DatePicker } from 'antd';
import Wrapper from 'COMPONENT/Wrapper';
import Select from 'COMPONENT/Select';
import Input from 'COMPONENT/Input';
import Page from 'COMPONENT/Page';
import List from 'COMPONENT/List';
import Button from 'COMPONENT/Button';
import { connect } from 'dva';

import * as broadcastCooperateBankStatementApi from 'SERVICE/broadcastCooperateBankStatement';

import showMessage from 'SRC/components/utils/globalTips/showMessage';
import { errorsHandle } from 'SRC/utils/common';
import moment from 'moment';

import styles from './styles.module.less';

const { RangePicker } = DatePicker;
const branchDict = [
  {
    val: '000002',
    text: '东莞分公司',
  },
];
const typeDict = [
  { val: 'RECEIPT', text: '收款' },
  { val: 'RETURN_FEE', text: '退款' },
];
const viewTypeDict = typeDict;
const Utils = {
  downloadFile(downFileUrl, fileName) {
    const a = document.createElement('a');

    console.log(fileName);
    a.href = downFileUrl;
    a.setAttribute('download', fileName);
    a.style.display = 'none';

    document.body.appendChild(a);
    a.addEventListener('click', () => {
      a.parentNode.removeChild(a);
    });
    a.click();
  },
};
class broadcastCooperateBankStatement extends React.Component {
  state = {
    loading: false,
    branchDict: branchDict,
    campusDict: [],

    type: '',
    posId: '',
    blCampusId: '',
    studentName: '',
    studentPhone: '',
    branchId: '000002',
    currentDate: [moment().startOf('month'), moment().endOf('day')],

    total: 0,
    pageNum: 1,
    pageSize: 30,
    dataSource: [],
  };

  constructor(props) {
    super(props);

    this.columns = [
      {
        title: '交易日期',
        dataIndex: 'paidTime',
        width: 160,
        render: (value) => (value ? moment(value).format('YYYY-MM-DD') : '-'),
      },
      { title: '合同编号', dataIndex: 'contractId', width: 160 },
      { title: '学生电话', dataIndex: 'studentPhone', width: 100 },
      { title: '学生姓名', dataIndex: 'studentName', width: 100 },
      { title: '分公司', dataIndex: 'branchName', width: 150 },
      { title: '学习中心', dataIndex: 'blCampusName', width: 150 },
      { title: '合作平台', dataIndex: 'payMerchantsName', width: 100 },
      {
        title: '类型',
        dataIndex: 'type',
        render: (value) => {
          return viewTypeDict.find(({ val }) => val == value).text || '-';
        },
        width: 100,
      },
      { title: '支付参考号', dataIndex: 'posId', width: 100 },
      { title: '金额（元）', dataIndex: 'paidAmount', width: 100 },
    ];
  }

  componentDidMount() {
    this.initContent();
  }

  async initContent() {
    this.loadDict();
  }

  loadDict = async () => {
    this.handleBranchIdChange('000002');
  };

  loadData = async () => {
    this.setState({ loading: true });

    const { type, pageNum, pageSize, branchId, posId, blCampusId, studentName, currentDate, studentPhone } = this.state;
    const endTime = currentDate[1].valueOf();
    const startTime = currentDate[0].valueOf();
    const result = await broadcastCooperateBankStatementApi
      .getHostFinanceReport({
        type,
        endTime,
        pageNum,
        pageSize,
        branchId,
        startTime,
        posId,
        blCampusId,
        studentName,
        studentPhone,
      })
      .catch(errorsHandle());

    if (!result || !result.data.list.length) {
      this.setState({ total: 0, dataSource: [], loading: false });
    } else {
      this.setState({ total: result.data.total, dataSource: result.data.list, loading: false });
    }
  };

  handleBranchIdChange = async (branchId) => {
    let campusDict = [];

    if (!branchId) {
      return this.setState({ branchId, campusDict, blCampusId: '', pageNum: 1 });
    }

    const result = await broadcastCooperateBankStatementApi
      .findOrganizationByParentId({ parentId: branchId, orgType: 'CAMPUS' })
      .catch(errorsHandle());
    if (result.resultCode === 0 && result.data && result.data.length) {
      campusDict = [...campusDict, ...result.data.map(({ name, id }) => ({ val: id, text: name }))];
    }

    this.setState({ branchId, campusDict, blCampusId: '', pageNum: 1 });
  };

  handleBlCampusIdChange = (blCampusId) => {
    this.setState({ blCampusId, pageNum: 1 });
  };

  handleTypeChange = (type) => {
    this.setState({ type, pageNum: 1 });
  };

  handleChangeDate = (currentDate) => {
    this.setState({ currentDate });
  };

  handleChangeStudentPhone = (event) => {
    this.setState({ studentPhone: event.currentTarget.value, pageNum: 1 });
  };

  handleChangeStudentName = (event) => {
    this.setState({ studentName: event.currentTarget.value, pageNum: 1 });
  };

  handleChangeStudentPosId = (event) => {
    this.setState({ posId: event.currentTarget.value, pageNum: 1 });
  };

  handleChangePage = ({ current, pageSize }) => {
    this.setState({ pageNum: current, pageSize }, this.loadData);
  };

  handleReset = () => {
    this.setState({
      type: '',
      posId: '',
      blCampusId: '',
      studentName: '',
      studentPhone: '',
      branchId: '000002',
      currentDate: [moment().startOf('month'), moment().endOf('day')],
    });
  };

  handleExport = async () => {
    const { type, total, pageSize, branchId, posId, blCampusId, studentName, currentDate, studentPhone } = this.state;

    if (total <= 0) {
      return showMessage('当前查询数据为空');
    }

    this.setState({ loading: true });
    const endTime = currentDate[1].valueOf();
    const startTime = currentDate[0].valueOf();
    const result = await broadcastCooperateBankStatementApi
      .exportHostFinanceReport({
        type,
        endTime,
        branchId,
        startTime,
        posId,
        blCampusId,
        studentName,
        studentPhone,
        pageNum: 1,
        pageSize: 5000,
      })
      .catch(errorsHandle());

    if (result && result.resultCode === 0 && result.data) {
      Utils.downloadFile(result.data, result.data.split(/\//).pop());
    }
    this.setState({ loading: false });
  };

  handleSearch = async () => {
    await this.loadData();
  };

  generateExportBtn() {
    const props = {
      type: 'primary',
      onClick: this.handleExport,
    };
    return <Button {...props}>导出Excel</Button>;
  }

  generateDate() {
    const props = {
      size: 'large',
      allowClear: false,
      style: { width: 300 },
      format: 'YYYY-MM-DD',
      value: this.state.currentDate,
      onChange: this.handleChangeDate,
      placeholder: ['开始时间', '结束时间'],
    };
    return <RangePicker {...props} />;
  }

  generateSearchBox() {
    const { type, branchId, blCampusId, posId, campusDict, branchDict, studentName, studentPhone } = this.state;

    return (
      <div className={styles.searchWrapper}>
        <div className={styles.searchFields}>
          {this.generateDate()}
          <Select
            disabled={true}
            dict={branchDict}
            value={branchId}
            placeholder="分公司"
            style={{ width: 160 }}
            onChange={this.handleBranchIdChange}
            showSearch={true}
          />
          <Select
            dict={campusDict}
            value={blCampusId}
            placeholder="学习中心"
            style={{ width: 160 }}
            onChange={this.handleBlCampusIdChange}
            showSearch={true}
          />
          <Select
            dict={typeDict}
            value={type}
            placeholder="类型"
            style={{ width: 100 }}
            onChange={this.handleTypeChange}
            showSearch={false}
          />
          <Input value={studentPhone} placeholder="学生电话" onChange={this.handleChangeStudentPhone} />
          <Input value={studentName} placeholder="学生姓名" onChange={this.handleChangeStudentName} />
          <Input value={posId} placeholder="支付参考号" onChange={this.handleChangeStudentPosId} />
          {this.generateSearchBtn()}
          {this.generateResetBtn()}
          {this.generateExportBtn()}
        </div>
      </div>
    );
  }

  generateSearchBtn() {
    const props = {
      type: 'primary',
      onClick: this.handleSearch,
      style: { marginRight: '10px' },
    };
    return <Button {...props}>查询</Button>;
  }

  generateResetBtn() {
    const props = {
      onClick: this.handleReset,
      style: { marginRight: '10px' },
    };
    return <Button {...props}>重置</Button>;
  }

  render() {
    const { total, loading, pageNum, pageSize, dataSource } = this.state;

    return (
      <Page
        title="广电合作流水"
        loading={loading}
        route={this.props.route}
        fitWindowHeight={true}
        className={styles.broadcastCooperateWrapper}
      >
        <Wrapper header={this.generateSearchBox()}>
          <List
            columns={this.columns}
            dataSource={dataSource}
            locale={{ emptyText: '请查询数据' }}
            onChange={this.handleChangePage}
            pagination={{
              current: pageNum,
              total,
              pageSize,
              defaultPageSize: 30,
              pageSizeOptions: ['10', '20', '30', '40', '50'],
            }}
          />
        </Wrapper>
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const pathname = state.routing.locationBeforeTransitions.pathname;
  return {
    params: state.main.pageParams[pathname],
  };
};

export default connect(mapStateToProps, null)(broadcastCooperateBankStatement);
