import React from 'react';

import { Modal } from 'COMPONENT';
import PlatformForm from '../PlatformForm';
import HistoryList from '../HistoryList';
import showMessage from 'SRC/components/utils/globalTips/showMessage';
import { errorsHandle } from 'SRC/utils/common';
import * as educationManagementPlatformApi from 'SERVICE/educationManagementPlatform';

export default class EducationPlatformModal extends React.Component {
  state = {
    loading: false,
    campusDict: [],
  };

  constructor(props) {
    super(props);
    this.formRef = null;
  }

  componentDidMount() {}

  trimValues(values = {}) {
    const { posName, posNumber, remark, platformName } = values || {};
    return {
      ...(values || {}),
      remark: (remark || '').trim(),
      posName: (posName || '').trim(),
      posNumber: (posNumber || '').trim(),
      platformName: (platformName || '').trim(),
    };
  }

  // 提交表单
  handleSubmit = () => {
    this.formRef.validateFields((err, values) => {
      if (err) {
        return;
      }
      const { modelMode } = this.props;

      if (modelMode === 'add') {
        this.handleAdd(values);
      } else if (modelMode === 'edit') {
        this.handleEdit(values);
      }
    });
  };

  handleAdd = async (values) => {
    this.handleSetLoading(true);

    values = this.trimValues(values);
    const result = await educationManagementPlatformApi
      .addEducationPlatform({
        ...values,
      })
      .catch(errorsHandle());
    this.handleSetLoading(false);
    if (!result || result.resultCode !== 0) {
      return;
    }
    showMessage('新增教育平台信息成功', 'success');
    this.props.onAddSuccess();
  };

  handleEdit = async (values) => {
    this.handleSetLoading(true);

    values = this.trimValues(values);
    const result = await educationManagementPlatformApi
      .updateEducationPlatformInfo({
        ...values,
        id: this.props.initialValues.id,
      })
      .catch(errorsHandle());

    this.handleSetLoading(false);
    if (!result || result.resultCode !== 0) {
      return;
    }
    showMessage('更新教育平台信息成功', 'success');
    this.props.onAddSuccess();
  };

  handleSetLoading = (loading) => {
    this.setState({ loading });
  };

  generateForm() {
    const { companyDict, initialValues, modelMode } = this.props;
    const props = {
      initialValues,
      mode: modelMode,
      dicts: { companyDict },
      ref: (c) => (this.formRef = c),
    };

    return <PlatformForm {...props} />;
  }

  generateHistoryList() {
    const props = {
      id: this.props.initialValues.id,
    };

    return this.props.modelMode === 'view' && <HistoryList {...props} />;
  }

  // 弹框
  generateModal() {
    const title = { add: '新增教育平台', edit: '修改教育平台', view: '详情' };
    const props = {
      widthSize: 'l',
      okBtnText: '保存',
      onOk: this.handleSubmit,
      visible: this.props.visible,
      loading: this.state.loading,
      onCancel: this.props.onCancel,
      title: title[this.props.modelMode],
      footer: this.props.modelMode === 'view' ? null : undefined,
    };

    return (
      <Modal {...props}>
        {this.props.modelMode === 'view' && (
          <h3 style={{ fontSize: '14px', fontWeight: 'bold', margin: '10px 0' }}>平台信息</h3>
        )}
        {this.generateForm()}
        {this.generateHistoryList()}
      </Modal>
    );
  }

  render() {
    return this.generateModal();
  }
}
