import React from 'react';

import { List } from 'COMPONENT';

import moment from 'moment';
import { errorsHandle } from 'SRC/utils/common';
import * as educationManagementPlatformApi from 'SERVICE/educationManagementPlatform';

import styles from './styles.module.less';

export default class HistoryList extends React.Component {
  state = {
    list: [],
    total: 0,
    pageSize: 10,
    currentPage: 1,
  };

  constructor(props) {
    super(props);
  }

  componentDidMount() {
    this.init();
  }

  init() {
    this.loadData();
  }

  async loadData() {
    const { pageSize, currentPage: pageNum } = this.state;
    const result = await educationManagementPlatformApi
      .getPageEducationPlatformOperationLogList({
        pageNum,
        pageSize,
        id: this.props.id,
      })
      .catch(errorsHandle());

    if (!result || result.resultCode !== 0) {
      this.setState({ total: 0, list: [] });
      return;
    }

    const { list, total } = result.data;
    this.setState({ list, total });
  }

  handlePaginationChange = (pageInfo) => {
    const { pageSize, current: currentPage } = pageInfo;
    this.setState(
      {
        pageSize,
        currentPage,
      },
      () => {
        this.loadData();
      },
    );
  };

  getColumns() {
    return [
      {
        title: '时间',
        width: 100,
        dataIndex: 'createTime',
        render: (value) => moment(value).format('YYYY-MM-DD HH:mm:ss'),
      },
      { title: '操作详情', dataIndex: 'operationContent' },
      { title: '备注', width: 150, dataIndex: 'remark' },
      { title: '操作人', width: 80, dataIndex: 'createUserName' },
    ];
  }

  generateTitle() {
    return <h3 style={{ fontSize: '14px', fontWeight: 'bold', margin: '10px 0' }}>操作流水</h3>;
  }

  // 列表
  generateList() {
    const { list, total, pageSize, currentPage } = this.state;

    return (
      <List
        dataSource={list}
        columns={this.getColumns()}
        onChange={this.handlePaginationChange}
        pagination={{
          pageSize,
          total: total,
          current: currentPage,
          pageSizeOptions: ['10', '20', '30', '40', '50'],
        }}
      />
    );
  }

  render() {
    return (
      <div className={styles.historyList}>
        {this.generateTitle()}
        {this.generateList()}
      </div>
    );
  }
}
