import React from 'react';

import { Row, Col, Form } from 'antd';

import { Input, Select, TextArea } from 'COMPONENT';

import { selectOrgByIdAndTypeOption } from 'SERVICE/common';

import _ from 'lodash';

const { Item: FormItem } = Form;

const formItemProps = {
  labelCol: { span: 6 },
  wrapperCol: { span: 16 },
};

class PlatformForm extends React.Component {
  state = {
    campusDict: [],
  };

  constructor(props) {
    super(props);
  }

  componentDidMount() {
    if (this.props.mode === 'add') {
      return;
    }
    this.loadCampusDict(this.props.initialValues.branchId);
  }

  get isDisabled() {
    const { mode } = this.props;
    return mode === 'view';
  }

  // 加载学习中心
  async loadCampusDict(orgId, callback) {
    this.setState({ campusDict: [] });

    const result = await selectOrgByIdAndTypeOption({
      orgId,
      orgType: 'CAMPUS',
    });

    if (!result || result.resultCode !== 0 || !result.value) {
      return;
    }
    const campusDict = _.map(result.value, (text, val) => ({ val, text })).filter(({ val }) => val);
    this.setState({ campusDict: [{ val: '', text: '请选择' }, ...campusDict] }, callback);
  }

  // 选择所属公司
  handleSelectCompany = (orgId) => {
    this.loadCampusDict(orgId);
    // 学习中心置空
    this.props.form.setFieldsValue({
      blCampusId: null,
    });
  };

  // 表单
  generateForm() {
    return (
      <Form>
        <Row>
          <Col>{this.generatePlatformFormItem()}</Col>
        </Row>
        <Row>
          <Col span={12}>{this.generateTerminalNameFormItem()}</Col>
          <Col span={12}>{this.generateTerminalNumFormItem()}</Col>
        </Row>
        <Row>
          <Col span={12}>{this.generateCompanyFormItem()}</Col>
          <Col span={12}>{this.generateCampusFormItem()}</Col>
        </Row>
        <Row>
          <Col>{this.generateRemarkFormItem()}</Col>
        </Row>
      </Form>
    );
  }

  // 平台名称
  generatePlatformFormItem() {
    const itemProps = {
      colon: false,
      label: '平台名称',
      key: 'platformName',
      labelCol: { span: 3 },
      wrapperCol: { span: 8 },
    };
    const { getFieldDecorator } = this.props.form;

    return (
      <FormItem {...itemProps}>
        {getFieldDecorator('platformName', {
          rules: [{ required: true, message: '平台名称不能为空' }],
          initialValue: this.props.initialValues.platformName,
        })(<Input maxLength="200" disabled={this.isDisabled} placeholder="请输入" />)}
      </FormItem>
    );
  }

  // 终端名称
  generateTerminalNameFormItem() {
    const itemProps = {
      colon: false,
      label: '终端名称',
      key: 'posName',
      ...formItemProps,
    };
    const { getFieldDecorator } = this.props.form;

    return (
      <FormItem {...itemProps}>
        {getFieldDecorator('posName', {
          rules: [{ required: true, message: '终端名称不能为空' }],
          initialValue: this.props.initialValues.posName,
        })(<Input maxLength="200" disabled={this.isDisabled} placeholder="请输入" />)}
      </FormItem>
    );
  }

  // 终端编号
  generateTerminalNumFormItem() {
    const itemProps = {
      colon: false,
      label: '终端编号',
      key: 'posNumber',
      ...formItemProps,
    };
    const { getFieldDecorator } = this.props.form;

    return (
      <FormItem {...itemProps}>
        {getFieldDecorator('posNumber', {
          rules: [{ required: true, message: '终端编号不能为空' }],
          initialValue: this.props.initialValues.posNumber,
        })(<Input maxLength="64" disabled={this.isDisabled} placeholder="请输入" />)}
      </FormItem>
    );
  }

  // 所属公司
  generateCompanyFormItem() {
    const itemProps = {
      colon: false,
      key: 'branchId',
      label: '所属公司',
      ...formItemProps,
    };
    const { getFieldDecorator } = this.props.form;

    return (
      <FormItem {...itemProps}>
        {getFieldDecorator('branchId', {
          initialValue: this.props.initialValues.branchId,
          rules: [{ required: true, message: '所属公司不能为空' }],
        })(
          <Select
            showSearch
            placeholder="请选择"
            onChange={this.handleSelectCompany}
            dict={this.props.dicts.companyDict}
            disabled={this.isDisabled}
          />,
        )}
      </FormItem>
    );
  }

  // 所属学习中心
  generateCampusFormItem() {
    const itemProps = {
      colon: false,
      key: 'blCampusId',
      label: '所属学习中心',
      ...formItemProps,
    };
    const { getFieldDecorator } = this.props.form;

    return (
      <FormItem {...itemProps}>
        {getFieldDecorator('blCampusId', {
          initialValue: this.props.initialValues.blCampusId,
          rules: [{ required: true, message: '所属学习中心' }],
        })(<Select showSearch placeholder="请选择" dict={this.state.campusDict} disabled={this.isDisabled} />)}
      </FormItem>
    );
  }

  // 备注
  generateRemarkFormItem() {
    const itemProps = {
      key: 'remark',
      colon: false,
      label: '备注',
      labelCol: { span: 3 },
      wrapperCol: { span: 20 },
    };
    const { getFieldDecorator } = this.props.form;

    return (
      this.props.mode !== 'view' && (
        <FormItem {...itemProps}>
          {getFieldDecorator('remark')(<TextArea rows={3} maxLength={500} placeholder="请输入500字以内文字" />)}
        </FormItem>
      )
    );
  }

  render() {
    return this.generateForm();
  }
}

export default Form.create()(PlatformForm);
