import React from 'react';

import { DatePicker } from 'antd';
import { List, Modal, Button } from 'COMPONENT';

import moment from 'moment';
import FileUtil from '../../../../../utils/file';
import { errorsHandle } from 'SRC/utils/common';
import * as educationPlatformPaymentApi from 'SERVICE/educationPlatformPayment';

import styles from './style.module.less';

const { RangePicker } = DatePicker;
export default class EducationPlatformModal extends React.Component {
  state = {
    list: [],
    total: 0,
    pageSize: 30,
    currentPage: 1,
    currentDate: [moment(new Date()).subtract(1, 'months').startOf('month'), moment()],
  };

  constructor(props) {
    super(props);
  }

  componentDidMount() {
    this.loadData();
  }

  // 加载数据
  async loadData() {
    const { currentDate, pageSize, currentPage: pageNum } = this.state;
    const [createTimeBegin, createTimeEnd] = currentDate;

    console.log(new Date(createTimeBegin._d.getTime()), new Date(createTimeEnd._d.getTime()));
    const result = await educationPlatformPaymentApi
      .exportEduPayPageData({
        pageNum,
        pageSize,
        createTimeEnd: createTimeEnd._d.getTime(),
        createTimeBegin: createTimeBegin._d.getTime(),
      })
      .catch(errorsHandle());

    if (!result || result.resultCode !== 0 || !result.data.list.length) {
      this.setState({ list: [], total: 0 });
      return;
    }
    const {
      data: { total, list },
    } = result;
    this.setState({ total, list });
  }

  getColumns() {
    return [
      {
        title: '导入时间',
        width: 120,
        dataIndex: 'createTime',
        render: (value) => (value ? moment(value).format('YYYY-MM-DD HH:mm:ss') : '-'),
      },
      { title: '导入人', width: 100, dataIndex: 'createUserName' },
      { title: '导入数量', width: 100, dataIndex: 'totalNum' },
      { title: '成功数量', width: 100, dataIndex: 'successNum' },
      { title: '失败数量', width: 100, dataIndex: 'failNum' },
      {
        title: '操作',
        fixed: 'right',
        key: 'action',
        getActionConfig: (text, record, index) => {
          return [{ text: '导出Excel', onClick: () => this.handleExport(record) }];
        },
      },
    ];
  }

  // 导出Excel
  handleExport = (record) => {
    const { resultName } = record;
    FileUtil.downloadFile(resultName);
  };

  // 查询
  handleSearch = () => {
    this.setState(
      {
        list: [],
        total: 0,
        currentPage: 1,
      },
      () => {
        this.loadData();
      },
    );
  };

  // 重置
  handleReset = () => {
    this.setState(
      {
        list: [],
        total: 0,
        currentPage: 1,
        currentDate: [moment(new Date()).subtract(1, 'months').startOf('month'), moment()],
      },
      () => {
        this.loadData();
      },
    );
  };

  // 分页处理
  handlePaginationChange = (pageInfo) => {
    const { pageSize, current: currentPage } = pageInfo;
    this.setState(
      {
        pageSize,
        currentPage,
      },
      () => {
        this.loadData();
      },
    );
  };

  // 选择日期
  handleChangeDate = (currentDate) => {
    this.setState({ currentDate });
  };

  // 弹框
  generateModal() {
    const props = {
      footer: null,
      widthSize: 'l',
      title: '导入结果',
      visible: this.props.visible,
      onCancel: this.props.onCancel,
    };

    return (
      <Modal {...props}>
        {this.generateSearchBox()}
        {this.generateList()}
      </Modal>
    );
  }

  // 导出列表
  generateList() {
    const { list, total, pageSize, currentPage } = this.state;

    return (
      <div className={styles.exportResultList}>
        <List
          dataSource={list}
          columns={this.getColumns()}
          pagination={{
            total,
            pageSize,
            current: currentPage,
            pageSizeOptions: ['10', '20', '30', '40', '50'],
          }}
          onChange={this.handlePaginationChange}
        />
      </div>
    );
  }

  // 搜索框
  generateSearchBox() {
    return (
      <div className={styles.searchBoxWrapper}>
        {this.generateDate()}
        {this.generateSearchBtn()}
        {this.generateResetBtn()}
      </div>
    );
  }

  // 搜索按钮
  generateSearchBtn() {
    return (
      <Button style={{ marginLeft: '10px' }} type="primary" onClick={this.handleSearch}>
        查询
      </Button>
    );
  }

  // 重置按钮
  generateResetBtn() {
    return (
      <Button style={{ marginLeft: '10px' }} onClick={this.handleReset}>
        重置
      </Button>
    );
  }

  generateDate() {
    const props = {
      size: 'large',
      allowClear: false,
      style: { width: 300 },
      format: 'YYYY-MM-DD',
      value: this.state.currentDate,
      onChange: this.handleChangeDate,
      placeholder: ['开始时间', '结束时间'],
    };
    return <RangePicker {...props} />;
  }

  render() {
    return this.generateModal();
  }
}
