import React from 'react';

import { List, Select, Button } from 'COMPONENT';
import AddConfRelModal from '../AddConfRelModal';

import { getSelectOption } from 'SERVICE/common';
import * as Api from 'SERVICE/industrialRelConfig';
import { errorsHandle } from 'SRC/utils/common';
import ShowConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import showMessage from 'SRC/components/utils/globalTips/showMessage';
import moment from 'moment';

import styles from './style.module.less';
import emptyDataImg from '../../../../../assets/imgs/industrial_rel_config_empty_data.png';

const payDict = [
  { val: 'A', text: 'A公司' },
  { val: 'B', text: 'B公司' },
  { val: 'C', text: 'C公司' },
  { val: 'D', text: 'D公司' },
  { val: 'E', text: 'E公司' },
  { val: 'F', text: 'F公司' },
  { val: 'G', text: 'G公司' },
  { val: 'H', text: 'H公司' },
];
const courseTypeDict = [
  { val: 'SMALL_CLASS', text: '小班' },
  { val: 'ONE_ON_ONE_COURSE', text: '一对一' },
  { val: 'ONE_ON_MANY', text: '一对多' },
  { val: 'PERSONALIZED', text: '其他' },
];
const courseTypeDictForModal = [
  { val: 'SMALL_CLASS', text: '小班' },
  { val: 'ONE_ON_ONE_COURSE', text: '一对一' },
  { val: 'ONE_ON_MANY', text: '一对多' },
  { val: 'PERSONALIZED', text: '其他（精英班/讲座/其他）' },
];

const Utils = {
  map2Array(map) {
    const array = [];

    for (const key in map) {
      if (key && Object.prototype.hasOwnProperty.call(map, key)) {
        array.push({ val: key, text: map[key] });
      }
    }

    return array;
  },

  downloadFile(downFileUrl, fileName) {
    const a = document.createElement('a');

    console.log(fileName);
    a.href = downFileUrl;
    a.setAttribute('download', fileName);
    a.style.display = 'none';

    document.body.appendChild(a);
    a.addEventListener('click', () => {
      a.parentNode.removeChild(a);
    });
    a.click();
  },
};

export default class DetailPanel extends React.Component {
  constructor(props) {
    super(props);
    this.searchRef = null;
    this.state = {
      list: [],
      total: 0,
      pageSize: 30,
      currentPage: 1,
      addModalVisible: false,
      selectedRowKeys: [],

      classTypeDict: [],
      studentGradeDict: [],
      courseSeriesDict: [],
      productGroupDict: [],

      gradeId: null,
      category: null,
      classTypeId: null,
      courseSeriesId: null,
      productGroupId: null,
      contractPayCategory: null,
    };
  }

  componentWillReceiveProps(nextProps) {
    if (nextProps.campusId && nextProps.campusId !== this.props.campusId) {
      this.setState({
        list: [],
        total: 0,
        pageSize: 30,
        currentPage: 1,
        selectedRowKeys: [],
      });
      this.loadData(nextProps.campusId);
    }
  }

  componentDidMount() {
    this.init();
  }

  init() {
    Promise.all([getSelectOption({ parentId: '', selectOptionCategory: 'STUDENT_GRADE' })])
      .then((res) => {
        const [studentGradeDict] = res.map(({ value }) => Utils.map2Array(value));
        this.setState({ studentGradeDict: (studentGradeDict || []).filter(({ text }) => text !== '不限') });
      })
      .catch(() => {
        this.setState({ studentGradeDict: [] });
      });
  }

  loadDictByCourseType(courseType) {
    if (!courseType) {
      return this.clearDict();
    }
    Promise.all([
      getSelectOption({ parentId: courseType, selectOptionCategory: 'PRODUCT_GROUP' }),
      getSelectOption({ parentId: courseType, selectOptionCategory: 'COURSE_SERIES' }),
      getSelectOption({ parentId: courseType, selectOptionCategory: 'CLASS_TYPE' }),
    ])
      .then((res) => {
        const [productGroupDict, courseSeriesDict, classTypeDict] = res.map(({ value }) => Utils.map2Array(value));
        this.setState({
          classTypeDict,
          courseSeriesDict,
          productGroupDict: courseType === 'SMALL_CLASS' ? [] : productGroupDict,
        });
      })
      .catch(() => {
        this.clearDict();
      });
  }

  clearDict() {
    this.setState({
      classTypeDict: [],
      courseSeriesDict: [],
      productGroupDict: [],
    });
  }

  async loadData(campusId = this.props.campusId) {
    const {
      gradeId,
      category,
      pageSize,
      classTypeId,
      productGroupId,
      courseSeriesId,
      contractPayCategory,
      currentPage: pageNum,
    } = this.state;
    this.props.onSetLoading(true);
    const {
      data: { list, total },
    } = await Api.getPagePayCategorys({
      pageNum,
      gradeId,
      campusId,
      category,
      pageSize,
      classTypeId,
      courseSeriesId,
      productGroupId,
      contractPayCategory,
    });
    this.setState({
      total,
      selectedRowKeys: [],
      list: list.map((item) => ({ ...item, key: item.id })),
    });
    this.props.onSetLoading(false);
  }

  handleDelete = (id) => {
    ShowConfirm('确定要删除吗', () => {
      this.handleSubmitDelete([id]);
    });
  };

  handleBatchDelete = () => {
    ShowConfirm('确定要删除吗', () => {
      this.handleSubmitDelete(this.state.selectedRowKeys);
    });
  };

  handleSubmitDelete = async (ids) => {
    this.props.onSetLoading(true);
    const result = await Api.deletePayCategorysByIds({ ids }).catch(errorsHandle());
    if (result && result.resultCode === 0) {
      this.loadData();
      showMessage('删除成功', 'success');
    }
    this.props.onSetLoading(false);
  };

  handleAddSuccess = () => {
    this.loadData();
    this.setState({ addModalVisible: false });
  };

  handleAddRelConfig = () => {
    this.setState({ addModalVisible: true });
  };

  handleExport = async () => {
    if (!this.state.total) {
      return showMessage('当前查询数据为空');
    }
    const { total, gradeId, category, classTypeId, courseSeriesId, contractPayCategory } = this.state;
    this.props.onSetLoading(true);
    const { data, resultCode } = await Api.exportPayCategorys({
      gradeId,
      category,
      pageNum: 1,
      classTypeId,
      courseSeriesId,
      pageSize: total,
      contractPayCategory,
      campusId: this.props.campusId,
    }).catch(errorsHandle());
    this.props.onSetLoading(false);
    if (resultCode !== 0 || !data) {
      return showMessage('导出数据失败，请稍后重试');
    }
    Utils.downloadFile(data, data.split(/\//).pop());
  };

  handleCancelModal = () => {
    this.setState({ addModalVisible: false });
  };

  handleSelectCategory = (category) => {
    this.setState({
      category,
      classTypeId: null,
      courseSeriesId: null,
      productGroupId: null,
    });
    this.loadDictByCourseType(category);
  };

  handleSelectGradeId = (gradeId) => {
    this.setState({
      gradeId,
    });
  };

  handleSelectCourseSeriesId = (courseSeriesId) => {
    this.setState({
      courseSeriesId,
    });
  };

  handleSelectClassTypeId = (classTypeId) => {
    this.setState({
      classTypeId,
    });
  };

  handleSelectProductGroupId = (productGroupId) => {
    this.setState({
      productGroupId,
    });
  };

  handleSelectContractPayCategory = (contractPayCategory) => {
    this.setState({
      contractPayCategory,
    });
  };

  handlePaginationChange = (pageInfo) => {
    const { pageSize, current: currentPage } = pageInfo;
    this.setState(
      {
        pageSize,
        currentPage,
      },
      () => {
        this.loadData();
      },
    );
  };

  handleSearch = () => {
    this.setState(
      {
        currentPage: 1,
      },
      () => {
        this.loadData();
      },
    );
  };

  handleReset = () => {
    this.setState(
      {
        gradeId: null,
        category: null,
        classTypeId: null,
        courseSeriesId: null,
        productGroupId: null,
        contractPayCategory: null,
      },
      () => {
        this.clearDict();
        this.handleSearch();
      },
    );
  };

  handleRowSelectChange = (selectedRowKeys) => {
    this.setState({
      selectedRowKeys,
    });
  };

  getColumns() {
    return [
      { title: '学习中心', width: 150, dataIndex: 'campusName', render: () => this.props.campusName },
      {
        title: '课程类型',
        width: 100,
        dataIndex: 'category',
        render: (value) => (courseTypeDict.find(({ val }) => val === value) || {}).text || '/',
      },
      { title: '年级', width: 100, dataIndex: 'gradeName' },
      { title: '产品系列', dataIndex: 'courseSeriesName', render: (value) => value || '/' },
      { title: '产品班级类型', dataIndex: 'classTypeName', render: (value) => value || '/' },
      { title: '产品组', dataIndex: 'productGroupName', render: (value) => value || '/' },
      {
        title: '支付模块',
        dataIndex: 'contractPayCategory',
        render: (value) => (payDict.find(({ val }) => val === value) || {}).text || '/',
      },
      {
        title: '创建时间',
        width: 140,
        dataIndex: 'createTime',
        render: (value) => (value ? moment(value).format('YYYY-MM-DD HH:mm:ss') : '-'),
      },
      {
        title: '操作',
        width: 50,
        fixed: 'right',
        key: 'action',
        getActionConfig: (text, record, index) => {
          const { key } = record;
          return [{ text: '删除', onClick: () => this.handleDelete(key) }];
        },
      },
    ];
  }

  generateSearchBox() {
    const {
      gradeId,
      category,
      classTypeId,
      courseSeriesId,
      productGroupId,
      contractPayCategory,
      classTypeDict,
      productGroupDict,
      studentGradeDict,
      courseSeriesDict,
    } = this.state;

    return (
      <div className={styles.searchWrapper}>
        <div className={styles.searchFields}>
          <Select
            placeholder="课程类型"
            dict={courseTypeDict}
            value={category}
            onChange={this.handleSelectCategory}
            showSearch
          />
          <Select
            value={gradeId}
            placeholder="年级"
            dict={studentGradeDict}
            onChange={this.handleSelectGradeId}
            showSearch
          />
          <Select
            value={courseSeriesId}
            placeholder="产品系列"
            dict={courseSeriesDict}
            onChange={this.handleSelectCourseSeriesId}
            showSearch
          />
          <Select
            value={classTypeId}
            placeholder="产品班级类型"
            dict={classTypeDict}
            onChange={this.handleSelectClassTypeId}
            showSearch
          />
          <Select
            value={productGroupId}
            placeholder="产品组"
            dict={productGroupDict}
            onChange={this.handleSelectProductGroupId}
            showSearch
          />
          <Select
            dict={payDict}
            placeholder="支付模块"
            value={contractPayCategory}
            onChange={this.handleSelectContractPayCategory}
            showSearch
          />
        </div>
      </div>
    );
  }

  generateToolbar() {
    const { selectedRowKeys } = this.state;

    return (
      <div className={styles.searchToolbar}>
        <Button style={{ marginRight: '10px' }} type="primary" onClick={this.handleSearch}>
          查询
        </Button>
        <Button style={{ marginRight: '10px' }} onClick={this.handleReset}>
          重置
        </Button>
        <Button
          type="primary"
          style={{ marginRight: '10px' }}
          disabled={!selectedRowKeys.length}
          onClick={this.handleBatchDelete}
        >
          批量删除
        </Button>
        <Button type="primary" style={{ marginRight: '10px' }} onClick={this.handleAddRelConfig}>
          新增关联
        </Button>
        <Button type="primary" onClick={this.handleExport}>
          导出Excel
        </Button>
      </div>
    );
  }

  generateTip() {
    return (
      <p className={styles.tip}>
        注：①其他课程类型包含精英班/讲座/其他等 ②未设置工商主体关联的合同默认使用对应A公司商户号收款
      </p>
    );
  }

  generateList() {
    const { list, total, pageSize, currentPage, selectedRowKeys } = this.state;

    return (
      <List
        dataSource={list}
        columns={this.getColumns()}
        pagination={{
          pageSize,
          total: total,
          current: currentPage,
          pageSizeOptions: ['10', '20', '30', '40', '50'],
        }}
        onChange={this.handlePaginationChange}
        rowSelection={{
          selectedRowKeys,
          onChange: this.handleRowSelectChange,
        }}
      />
    );
  }

  generateModal() {
    const { campusId, campusName } = this.props;
    const { classTypeDict, studentGradeDict, courseSeriesDict } = this.state;
    const modalProps = {
      dicts: {
        payDict,
        classTypeDict,
        courseTypeDict: courseTypeDictForModal,
        courseSeriesDict,
        studentGradeDict,
      },
      campusId,
      campusName,
      onCancel: this.handleCancelModal,
      onAddSuccess: this.handleAddSuccess,
    };
    return this.state.addModalVisible && <AddConfRelModal {...modalProps} />;
  }

  generatePathTip() {
    return <div className={styles.pathTip}>当前位置：{this.props.path || '星火教育'}</div>;
  }

  render() {
    return this.props.campusId ? (
      <div>
        {this.generatePathTip()}
        {this.generateSearchBox()}
        {this.generateToolbar()}
        {this.generateTip()}
        {this.generateList()}
        {this.generateModal()}
      </div>
    ) : (
      <div>
        {this.generatePathTip()}
        <div className={styles.emptyDataImg}>
          <img src={emptyDataImg} />
          <p>请选择学习中心</p>
        </div>
      </div>
    );
  }
}
