import React from 'react';
import { findD } from 'react-dom';
import { Tree } from 'antd';
import styles from './style.module.less';

import * as Api from 'SERVICE/industrialRelConfig';

const { TreeNode } = Tree;

export default class TreeSelector extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      treeNodes: [],
      selectedKeys: [],
      expandedKeys: [],
    };
  }

  componentDidMount() {
    this.initData();
  }

  handleSelect = (keys, e) => {
    const { expandedKeys } = this.state;
    const { title, parentId, isTemp, isLoaded, eventKey: branchId } = e.node.props;
    const getExpandedKeys = () => {
      return !!expandedKeys.find((key) => key === branchId)
        ? expandedKeys.filter((key) => key !== branchId)
        : [...expandedKeys, branchId];
    };

    if (parentId === -1) {
      !isLoaded && this.getCampusTreeNods(branchId);
      this.setState({ expandedKeys: getExpandedKeys(), selectedKeys: [branchId] });
      this.props.onSelectCampus('', title, ['星火集团', title].join(' >> '));
    } else if (!isTemp) {
      this.setState({ selectedKeys: [branchId] });
      this.props.onSelectCampus(
        branchId.startsWith('empty') ? '' : branchId,
        title,
        ['星火集团', this.state.treeNodes.find(({ id }) => id === parentId).name, title].join(' >> '),
      );
    }
  };

  handleExpand = (expandedKeys, nodeInfo) => {
    const {
      expanded,
      node: {
        props: { title, parentId, isLoaded, eventKey: branchId },
      },
    } = nodeInfo;

    if (expanded && parentId === -1 && !isLoaded) {
      this.getCampusTreeNods(branchId);
    }

    this.setState({ selectedKeys: [branchId] });
    this.setState({ expandedKeys });

    this.props.onSelectCampus('', title, ['星火集团', title].join(' >> '));
  };

  async getCampusTreeNods(branchId) {
    this.props.onSetLoading(true);
    const { data } = await Api.listUserAllCampusByResourceIdAndBranchId({
      branchId,
      resourceId: 'industrialRelConfig',
    });
    this.setState({
      treeNodes: this.state.treeNodes.map((treeNode, index) => {
        if (treeNode.id === branchId) {
          return {
            ...treeNode,
            children: data.length ? data : [{ id: `empty_${branchId}_${index}`, name: '无学习中心' }],
            isLoaded: true,
          };
        } else {
          return { ...treeNode };
        }
      }),
    });
    this.props.onSetLoading(false);
  }

  async initData() {
    const { data: treeNodes } = await Api.listUserAllBranchByResourceId({
      resourceId: 'industrialRelConfig',
    });
    this.setState({
      treeNodes: treeNodes.map((node) => ({
        ...node,
        children: [{ id: `_${node.id}`, parentId: node.id, name: '加载中...', isTemp: false }],
      })),
    });
    if (treeNodes.length === 1) {
      const branchId = treeNodes[0].id;
      this.getCampusTreeNods(branchId);
      this.setState({ expandedKeys: [branchId] });
    }
  }

  generateTree() {
    const { treeNodes, selectedKeys, expandedKeys } = this.state;
    const treeProps = {
      selectedKeys,
      expandedKeys,
      showIcon: false,
      checkable: false,
      onSelect: this.handleSelect,
      onExpand: this.handleExpand,
    };

    return (
      !!treeNodes.length && (
        <Tree {...treeProps}>
          {treeNodes.map((treeNode) => {
            const { id, name, children, isLoaded } = treeNode;
            return (
              <TreeNode key={id} title={name} parentId={-1} isLoaded={isLoaded}>
                {!!children &&
                  !!children.length &&
                  children.map((child) => {
                    return (
                      <TreeNode
                        key={child.id}
                        title={child.name}
                        parentId={id}
                        isTemp={child.isTemp || false}
                      ></TreeNode>
                    );
                  })}
              </TreeNode>
            );
          })}
        </Tree>
      )
    );
  }

  render() {
    return (
      <div className={styles.treeSelector}>
        <div className={styles.treeWrapper}>
          <p className={styles.groupName}>星火集团</p>
          <div className={styles.treeContent}>{this.generateTree()}</div>
        </div>
      </div>
    );
  }
}
