import React from 'react';
import Modal from 'COMPONENT/Modal';
import Button from 'COMPONENT/Button';
import List from 'COMPONENT/List';
import Select from 'COMPONENT/Select';
import InputNumber from 'COMPONENT/InputNumber';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import { loadCommonDict, getDictTextByVal } from 'UTIL/dict';
import { loadContractPrintDetail, initContractPrintDetail, saveContractPrintDetail } from 'SERVICE/contract';
import { ONE_ON_ONE_COURSE, ONE_ON_MANY, GIFT_COURSES, GIFT_COURSES_BY_RATE } from 'CONST/dicts/product';
import NP from 'number-precision';

import styles from './styles.less';
import ProductCard from './ProductCard';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

const { COL_WIDTH_S, COL_WIDTH_XL } = List;

const GIFT = 'GIFT'; //赠送
const GIFT_COURSES_ARR = [GIFT_COURSES, GIFT_COURSES_BY_RATE]; //赠送课时，赠送比例课时，缩单后产生
const REAL_COURSES = [GIFT, ONE_ON_MANY, ONE_ON_ONE_COURSE]; //需要分配的产品，暂时只有赠送，一对多，一对一

/**
 *
 * 需求：
 * 赠送分为两种情况：1. 未缩单。类型printArrangeType为GIFT
 *  2.已缩单。类型为GIFT_COURSES或GIFT_COURSES_BY_RATE
 *
 *
 *
 *
 */
export default class Index extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: true,
      contractTotalMoney: 0, //合同总金额
      totalArrangeNum: 0, //分配课时
      unAssignList: [], //未分配课时列表
      assignList: [], //已分配课时列表
      arrangeTypeDict: [],
      invalidArr: [], //已分配列表校验结果
      validating: false, //是否校验
    };
    this.originAssignList = [];
  }

  async componentDidMount() {
    const { contractId } = this.props.oldPageParams;
    //产品类型数据字典
    loadCommonDict('PrintArrangeType').then((arrangeTypeDict) => {
      this.setState({
        arrangeTypeDict,
      });
    });

    this.setState({ loading: true });
    try {
      await initContractPrintDetail({ contractId });
      let [detail, subjectDict] = await Promise.all([
        loadContractPrintDetail({ contractId }),
        loadCommonDict('SUBJECT'),
      ]);

      /**
       * 数据转换
       */
      subjectDict = subjectDict.map((dict) => {
        dict.title = dict.text;
        return dict;
      });
      const { data } = detail;
      //把科目从文字转换为id
      const assignList = data.notNeedArrangeList.map((assign) => {
        if (REAL_COURSES.indexOf(assign.printArrangeType) === -1) {
          return assign;
        } else {
          const target = subjectDict.find((dict) => dict.text === assign.subject);
          return {
            ...assign,
            subject: target ? target.val : null,
          };
        }
      });

      this.setState({
        loading: false,
        unAssignList: data.needArrangeList,
        assignList: assignList,
        totalArrangeNum: data.totalArrangeNum,
        giftCourseNum: data.giftCourseNum,
        contractTotalMoney: data.contractTotalMoney,
        subjectDict,
      });
      //保留原始数据
      this.originAssignList = assignList.map((item) => {
        return { ...item };
      });
    } catch (error) {
      showMessage(error.text);
    }
  }

  //分配课时，实际就是将上面列表的某一行复制一份添加到下面的列表
  assignClass = (unAssign) => {
    const { editType } = this.props.oldPageParams;
    if (unAssign.arrangeNum <= 0 || editType === 'check') return;
    const arrangeNum = unAssign.arrangeNum >= 45 ? 45 : unAssign.arrangeNum;
    const newAssign = {
      printArrangeType: unAssign.printArrangeType,
      arrangeTargetId: unAssign.id,
      arrangeTotalId: unAssign.arrangeTotalId,
      grade: unAssign.grade,
      arrangeName: unAssign.arrangeName,
      productId: unAssign.productId,
      arrangeNum,
      arrangeWeekNum: 3,
      arrangeMoneyNum: unAssign.printArrangeType === GIFT ? 0 : NP.times(arrangeNum, unAssign.price),
      price: unAssign.price,
      subject: null,
    };

    this.setState({
      unAssignList: this.state.unAssignList.map((row) => {
        if (row.id === unAssign.id) return { ...row, arrangeNum: NP.minus(row.arrangeNum, arrangeNum) };
        return row;
      }),
      assignList: [newAssign].concat(this.state.assignList),
    });
  };

  //更新已分配列表行数据
  updateRow = (keyValue, index, callback) => {
    this.setState(
      {
        assignList: this.state.assignList.map(function (row, idx) {
          if (idx !== index) return row;
          const newRow = { ...row };
          Object.keys(keyValue).forEach((key) => {
            newRow[key] = keyValue[key];
          });
          return newRow;
        }),
      },
      callback,
    );
  };

  //渲染课程类型/收费项目
  renderArrangeType = (text, row) => {
    const { arrangeTypeDict } = this.state;
    const dict = arrangeTypeDict.find((dict) => row.printArrangeType === dict.val);
    if (dict) {
      return dict.text;
    } else {
      return '-';
    }
  };

  //渲染科目
  renderSubject = (text, row, index) => {
    const { subjectDict, assignList } = this.state;
    const { printArrangeType } = row;

    //如果是查看或者是其它类型的产品，直接返回文字
    if (REAL_COURSES.indexOf(printArrangeType) === -1 || this.props.oldPageParams.editType === 'check') {
      return getDictTextByVal(subjectDict, text) || '-';
    }

    //已选的subjects
    let subjects;

    //如果是赠送
    if (printArrangeType === GIFT) {
      subjects = assignList.map((item) => {
        if (item.subject !== row.subject) {
          return item.subject;
        }
      });
    } else {
      //如果是一对一、一对多
      subjects = assignList.map((item) => {
        if (item.productId === row.productId && item.subject !== row.subject) {
          return item.subject;
        }
      });
    }

    //禁用掉已选的科目
    const filterDict = subjectDict.map((dict) => {
      if (subjects.indexOf(dict.val) === -1) {
        return { ...dict, disabled: false };
      } else {
        return { ...dict, disabled: true };
      }
    });

    return <Select dict={filterDict} value={row.subject} onChange={(subject) => this.updateRow({ subject }, index)} />;
  };

  //更新未分配课时的数据
  updateUnssign = (originArrageNum, id) => {
    const { unAssignList, assignList } = this.state;

    const newUnAssignList = unAssignList.map((row) => {
      if (row.id === id) {
        return {
          ...row,
          //根据原始数据和已分配的课时，计算出新的未分配课时
          arrangeNum: NP.minus(
            originArrageNum,
            assignList.reduce((acc, cur) => {
              if (cur.arrangeTargetId === row.id) return NP.plus(acc, cur.arrangeNum);
              return acc;
            }, 0),
          ),
        };
      } else {
        return row;
      }
    });
    this.setState({
      unAssignList: newUnAssignList,
    });
  };

  //修改课时前要校验，同时要更新未分配课时
  onArrangeNumChange = (arrangeNum, row, index) => {
    const { price, printArrangeType } = row;
    const { assignList, unAssignList } = this.state;
    // arrangeNum = Number(arrangeNum)
    //计算出变化后的总分配课时
    const toValue = assignList.reduce((acc, cur, idx) => {
      if (cur.arrangeTargetId === row.arrangeTargetId) {
        if (idx !== index) return NP.plus(acc, cur.arrangeNum);
        return NP.plus(acc, arrangeNum);
      } else {
        return acc;
      }
    }, 0);
    //原始总课时
    const originArrageNum = unAssignList.find((origin) => origin.id === row.arrangeTargetId).originArrangeNum;
    //小于或等于原始总课时才可以设置
    if (toValue <= originArrageNum && arrangeNum <= NP.times(row.arrangeWeekNum, 15)) {
      const toMoneyNum = printArrangeType === GIFT ? 0 : NP.times(arrangeNum, price);
      this.updateRow({ arrangeNum, arrangeMoneyNum: toMoneyNum }, index, () => {
        this.updateUnssign(originArrageNum, row.arrangeTargetId);
      });
    }
  };

  //渲染分配课时
  renderArrangeNum = (text, row, index) => {
    const { printArrangeType, arrangeNum } = row;
    if (
      REAL_COURSES.indexOf(printArrangeType) >= 0 && //要分配的类型
      this.props.oldPageParams.editType !== 'check'
    ) {
      return (
        <InputNumber
          min={0}
          value={arrangeNum}
          onChange={(arrangeNum) => this.onArrangeNumChange(arrangeNum, row, index)}
        />
      );
    } else {
      return text != undefined ? text : '-';
    }
  };

  //渲染每周课时
  renderArrangeWeekNum = (text, row, index) => {
    const { arrangeWeekNum, printArrangeType, arrangeNum } = row;

    if (REAL_COURSES.indexOf(printArrangeType) >= 0 && this.props.oldPageParams.editType !== 'check') {
      return (
        <InputNumber
          step={3}
          precision={0}
          min={3}
          value={arrangeWeekNum}
          onChange={(arrangeWeekNum) => {
            if (arrangeWeekNum % 3 === 0) {
              this.updateRow({ arrangeWeekNum }, index, () => {
                //触发一次课时数的变化，确保不会超出最大值
                const maxArrangeNum = arrangeWeekNum * 15; //该周课时对应的最大课时数
                if (arrangeNum > maxArrangeNum) {
                  this.onArrangeNumChange(maxArrangeNum, row, index);
                }
              });
            }
          }}
        />
      );
    } else {
      return text != undefined ? text : '-';
    }
  };

  //渲染金额
  renderMoneyNum = (text, row, index) => {
    const { printArrangeType } = row;
    if (printArrangeType === GIFT) {
      //赠送没有金额
      return '-';
    } else {
      //其他：为空则显示 -
      return text != undefined ? text : '-';
    }
  };

  onDel = (row, index) => {
    this.setState(
      {
        assignList: this.state.assignList.filter((item, idx) => index !== idx),
      },
      () => {
        this.updateUnssign(
          this.state.unAssignList.find((origin) => origin.id === row.arrangeTargetId).originArrangeNum,
          row.arrangeTargetId,
        );
      },
    );
  };

  /**
   * 获取未分配课时(包括赠送)
   */
  getUnAssignTimes = () => {
    const { unAssignList } = this.state;
    return unAssignList.reduce((acc, product) => NP.plus(acc, product.arrangeNum || 0), 0);
  };

  /**
   * 获取已分配金额(包括赠送)
   * @returns {*}
   */
  getAssignMoney = () => {
    const { assignList } = this.state;
    return assignList.reduce((acc, item) => NP.plus(acc, item.arrangeMoneyNum || 0), 0);
  };

  /**
   * 分别计算未分配总课时和赠送课时
   * @returns {{unAssignTotal: number, unAssignGift: number}}
   */
  caculateUnassignTimes = () => {
    const { unAssignList } = this.state;
    const typeArr = [...GIFT_COURSES_ARR, GIFT];

    let unAssignTotal = 0,
      unAssignGift = 0;
    unAssignList.forEach((row) => {
      if (typeArr.indexOf(row.printArrangeType) >= 0) {
        unAssignGift = NP.plus(unAssignGift, Number(row.arrangeNum) || 0);
      } else {
        unAssignTotal = NP.plus(unAssignTotal, Number(row.arrangeNum) || 0);
      }
    });
    return { unAssignTotal, unAssignGift };
  };

  /**
   * 分别计算已分配总课时和赠送课时
   * @returns {{assignGift: number, assignTotal: number}}
   */
  caculateAssignTimes = () => {
    const { assignList } = this.state;
    const typeArr = [...GIFT_COURSES_ARR, GIFT];

    let assignTotal = 0,
      assignGift = 0;
    assignList.forEach((row) => {
      if (typeArr.indexOf(row.printArrangeType) >= 0) {
        assignGift = NP.plus(assignGift, Number(row.arrangeNum) || 0);
      } else {
        assignTotal = NP.plus(assignTotal, Number(row.arrangeNum) || 0);
      }
    });
    return { assignTotal, assignGift };
  };

  /**
   * 渲染合同总课时
   */
  renderContractTotal = () => {
    const { totalArrangeNum, giftCourseNum } = this.state; //合同总的
    let gift = giftCourseNum;
    if (!gift) {
      //如果合同总的没有赠送，则尝试从已分配的取
      gift = this.caculateAssignTimes().assignGift;
    }
    return totalArrangeNum + (gift ? ' + ' + gift : '');
  };

  //关闭弹框
  onClose = () => {
    const { editType } = this.props.oldPageParams;
    const { assignList } = this.state;
    if (
      editType !== 'check' &&
      (JSON.stringify(assignList) !== JSON.stringify(this.originAssignList) || this.getUnAssignTimes() > 0)
    ) {
      showConfirm('已修改分配或存在未分配课时,确定关闭？', () => {
        this.props.onClose();
      });
    } else {
      this.props.onClose();
    }
  };

  //保存按钮
  save = async () => {
    const { unAssignList, assignList, contractTotalMoney, totalArrangeNum, subjectDict } = this.state;
    //已分配的总课时和赠送课时
    const { assignTotal, assignGift } = this.caculateAssignTimes(assignList);
    //已分配的总金额
    const assignMoney = this.getAssignMoney();
    //未分配课时
    const unAssignClass = this.getUnAssignTimes();

    try {
      if (assignTotal > totalArrangeNum) throw new Error('超课时分配，请重新分配!');
      if (unAssignClass > 0) throw new Error('课时未分配完全，请分配完再保存!');
      if (assignMoney !== contractTotalMoney) throw new Error('金额有误，请检查!');
    } catch (error) {
      return showMessage(error.message);
    }

    let invalidArr = []; //校验错误信息列表
    const arrangeVos = [];
    //重新过滤为后端需要的数据格式
    assignList.forEach((item) => {
      //只需要一对一、一对多、赠送类型的数据
      if (REAL_COURSES.indexOf(item.printArrangeType) >= 0) {
        let errMsg;
        if (!item.subject) errMsg = '请选择科目';
        if (!item.arrangeNum) errMsg = '请正确分配课时';
        if (!item.arrangeWeekNum) errMsg = '请正确分配课时/周';
        if (errMsg) {
          invalidArr.push(errMsg);
        }
        arrangeVos.push({
          id: item.id,
          arrangeTargetId: item.arrangeTargetId,
          subject: getDictTextByVal(subjectDict, item.subject),
          arrangeWeekNum: item.arrangeWeekNum,
          arrangeNum: item.arrangeNum,
          arrangeMoneyNum: item.arrangeMoneyNum,
          arrangeTotalId: item.arrangeTotalId,
        });
      }
    });
    const requestParams = {
      contractId: this.props.oldPageParams.contractId,
      notArrangedHours: unAssignClass,
      totalArrangeMoney: assignMoney,
      totalArrangeNum: assignTotal,
      giftCourseNum: assignGift,
      arrangeVos: arrangeVos,
      notArrangeVos: unAssignList.map((item) => ({
        arrangeNum: item.arrangeNum,
        id: item.id,
      })),
    };

    if (invalidArr[0]) {
      return showMessage(invalidArr[0]);
    }

    this.setState({ loading: true });
    try {
      const response = await saveContractPrintDetail(requestParams);
      this.setState({ loading: false });
      this.props.onClose();
    } catch (error) {
      showMessage(error.text);
      this.setState({ loading: false });
    }
  };

  columns = [
    { dataIndex: 'printArrangeType', title: '课程类型', render: this.renderArrangeType },
    { dataIndex: 'arrangeName', title: '课程名称', width: COL_WIDTH_XL },
    { dataIndex: 'grade', title: '年级' },
    { dataIndex: 'subject', title: '科目', render: this.renderSubject },
    { dataIndex: 'arrangeNum', title: '课时数', render: this.renderArrangeNum },
    { dataIndex: 'arrangeWeekNum', title: '课时/周', render: this.renderArrangeWeekNum },
    { dataIndex: 'arrangeMoneyNum', title: '金额', render: this.renderMoneyNum },
    {
      key: 'action',
      title: '操作',
      width: COL_WIDTH_S,
      getActionConfig: (text, row, index) => {
        if (REAL_COURSES.indexOf(row.printArrangeType) >= 0 && this.props.oldPageParams.editType !== 'check') {
          return [
            {
              text: '删除',
              onClick: () => this.onDel(row, index),
            },
          ];
        } else {
          return [];
        }
      },
    },
  ];

  render() {
    const { assignList, unAssignList, contractTotalMoney, arrangeTypeDict, loading } = this.state;
    const { editType } = this.props.oldPageParams;

    const { assignTotal, assignGift } = this.caculateAssignTimes();
    const { unAssignTotal, unAssignGift } = this.caculateUnassignTimes();

    return (
      <Modal
        title="分配课时"
        loading={loading}
        width={1046}
        onClose={this.props.onClose}
        className={styles.customModal}
        closable={false}
        footer={
          editType !== 'check' && (
            <Button type="primary" onClick={this.save}>
              保存
            </Button>
          )
        }
      >
        <button aria-label="Close" className={`${styles.closeBtn} ant-modal-close`} onClick={this.onClose}>
          <span className="ant-modal-close-x" />
        </button>
        {/*合同总计*/}
        <div className={styles.totalBlock}>
          <p className={styles.marginRight}>
            <label className={styles.label}>总课时:&nbsp;&nbsp;</label>
            <span className={styles.value}>{this.renderContractTotal()}</span>
          </p>
          <p className={styles.marginRight}>
            <label className={styles.label}>应收金额:&nbsp;&nbsp;</label>
            <span className={styles.value}>{contractTotalMoney}</span>
          </p>
          <p className={styles.marginRight}>
            <label className={styles.label}>未分配总课时:&nbsp;&nbsp;</label>
            <span className={styles.value}>{unAssignTotal + (unAssignGift ? ' + ' + unAssignGift : '')}</span>
          </p>
        </div>

        {/*未分配课时*/}
        <div className={styles.unsignBlock}>
          <p className={styles.blockTitle}>未分配课时</p>
          <div className={styles.products}>
            {unAssignList.length > 0 ? (
              unAssignList.map((product) => {
                return (
                  <ProductCard
                    disabled={editType === 'check'}
                    product={product}
                    arrangeTypeDict={arrangeTypeDict}
                    key={product.id}
                    onClick={() => this.assignClass(product)}
                  />
                );
              })
            ) : (
              <div style={{ width: '100%', textAlign: 'center' }}>暂无数据</div>
            )}
          </div>
        </div>

        {/*报读课程*/}
        <div>
          <p className={styles.blockTitle} style={{ marginBottom: 12 }}>
            报读课程
          </p>
          <List dataSource={assignList} columns={this.columns} />
          <div className={styles.totalAssign}>
            <p className={`${styles.marginRight} ${styles.label}`}>合计</p>
            <p className={styles.marginRight}>
              <span className={styles.label}>总课时&nbsp;&nbsp;&nbsp;</span>
              <span className={styles.value}>{assignTotal + (assignGift === 0 ? '' : ' + ' + assignGift)}</span>
            </p>
            <p>
              <span className={styles.label}>金额&nbsp;&nbsp;&nbsp;</span>
              <span className={styles.value} style={{ color: '#FF5B5B' }}>
                {this.getAssignMoney()}
              </span>
            </p>
          </div>
        </div>

        {assignList.some((row) => [...GIFT_COURSES_ARR, GIFT].indexOf(row.printArrangeType) >= 0) && (
          <span className={styles.mark}>备注：赠送课时可根据当下实际需求变更为其他科目。</span>
        )}
      </Modal>
    );
  }
}
