import React from 'react';
import { Modal } from 'COMPONENT';
import moment from 'moment';
import { batchFinishContract } from 'SERVICE/contract';
import downloadFile from 'COMPONENT/utils/file/downloadFile';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import WarnIcon from './assets/warning.png';

import styles from './styles.module.less';
export default class BatchCloseContractModal extends React.Component {
  state = {
    resultInfo: {},
    resultModalVisible: false,
  };

  constructor(props) {
    super(props);
  }

  // 确认批量关闭合同
  onConfirmCloseContract = async () => {
    try {
      const { data: resultInfo } = await batchFinishContract({
        contacts: this.props.contacts,
      });
      this.setState({ resultInfo });
      GlobalEvent.emit('REFRESH_CONTRACT_LIST'); // 刷新合同列表页
      this.setState({
        resultModalVisible: true,
      });
    } catch (e) {
      showMessage('批量关闭合同失败');
      this.setState({ resultInfo: {} });
    }
  };

  onCloseConfirmModal = () => {
    this.props.onCancel();
  };

  onCloseResultModal = () => {
    this.setState({ resultModalVisible: false });
    this.props.onCancel();
  };

  onDownloadFile = () => {
    if (!this.state.resultInfo?.resultFilePath) {
      return;
    }
    downloadFile(
      this.state.resultInfo?.resultFilePath,
      `批量结束合同失败明细_${moment().format('YYYY-MM-DD_hh_mm_ss')}.xls`,
    );
  };

  render() {
    return (
      <div>
        <Modal
          title="批量结束合同"
          cancelText="关闭"
          width="430"
          okText="确认结束合同"
          onOk={this.onConfirmCloseContract}
          onCancel={this.onCloseConfirmModal}
        >
          <div className={styles.resultModalContent}>
            <div className={styles.icon}>
              <img src={WarnIcon} />
            </div>
            <div className={styles.content}>
              结束合同操作不可逆，请问是否要将本次选中的
              <span className={styles.resultSuccess}>{this.props?.contacts?.length || 0}</span>份合同全部结束合同
            </div>
          </div>
        </Modal>
        {this.state.resultModalVisible && (
          <Modal
            title="批量结束合同结果"
            cancelText="关闭"
            width="430"
            okBtnShow={false}
            onOk={this.onConfirmClose}
            onCancel={this.onCloseResultModal}
          >
            <div className={styles.resultModalContent}>
              <div className={styles.icon}>
                <img src={WarnIcon} />
              </div>
              <div className={styles.content}>
                <p>
                  本次共选中{this.state.resultInfo?.total || 0}份合同，其中成功结束合同
                  <span className={styles.resultSuccess}>{this.state.resultInfo?.success || 0}</span>份，失败
                  <span className={styles.resultFail}>{this.state.resultInfo?.fail || 0}</span>份
                </p>
                <p>具体失败原因请查看以下结果Excel：</p>
                <a onClick={this.onDownloadFile}>批量结束合同失败明细</a>
              </div>
            </div>
          </Modal>
        )}
      </div>
    );
  }
}
