import React from 'react';
import PropTypes from 'prop-types';
import style from './styles.module.less';
import { Modal, Button, List } from 'COMPONENT';
import InfoBlock from './components/InfoBlock';
import { runFunc } from 'UTIL/common';

const setWidth = List.createSetWidth();

class BeforeConfirm extends React.Component {
  constructor(props) {
    super(props);

    this.handleNormalConfirm = this.handleNormalConfirm.bind(this);
    this.handleOtherConfirm = this.handleOtherConfirm.bind(this);
    this.hideNormalBtn = this.hideNormalBtn.bind(this);
    this.hideOtherBtn = this.hideOtherBtn.bind(this);

    this.state = {
      normalBtnVisible: true,
      otherBtnVisible: true,
    };

    this.columns = [
      { title: '合同编号', dataIndex: 'contractId', key: 'contractId', width: setWidth(6) },
      { title: '合同学习中心', dataIndex: 'contractSchoolName', key: 'contractSchoolName', width: setWidth(6) },
      { title: '合同年级', dataIndex: 'contractGradeName', key: 'contractGradeName', width: setWidth(4) },
      { title: '合同金额', dataIndex: 'contractAmount', key: 'contractAmount', width: setWidth(4) },
      { title: '签约人', dataIndex: 'signedUserName', key: 'signedUserName', width: setWidth(4) },
      { title: '合同签订日期', dataIndex: 'signedDateTie', key: 'signedDateTie', width: setWidth(8) },
    ];
  }

  static propTypes = {
    onCancel: PropTypes.func.isRequired,
    normalList: PropTypes.arrayOf(
      PropTypes.shape({
        contractId: PropTypes.string,
        contractSchoolName: PropTypes.string,
        contractGradeName: PropTypes.string,
        contractAmount: PropTypes.string,
        signedUserName: PropTypes.string,
        signedDateTie: PropTypes.string,
      }),
    ),
    otherList: PropTypes.arrayOf(
      PropTypes.shape({
        contractId: PropTypes.string,
        contractSchoolName: PropTypes.string,
        contractGradeName: PropTypes.string,
        contractAmount: PropTypes.string,
        signedUserName: PropTypes.string,
        signedDateTie: PropTypes.string,
      }),
    ),
    onNormalConfirm: PropTypes.func,
    onOtherConfirm: PropTypes.func,
  };

  static defaultProps = {
    normalList: [],
    otherList: [],
  };

  handleNormalConfirm() {
    runFunc(
      this.props.onNormalConfirm,
      // contractIds
      this.props.normalList.map((item) => item.contractId),
    );
  }

  handleOtherConfirm() {
    runFunc(
      this.props.onOtherConfirm,
      // contractIds
      this.props.otherList.map((item) => item.contractId),
    );
  }

  hideNormalBtn() {
    this.setState({ normalBtnVisible: false });
  }

  hideOtherBtn() {
    this.setState({ otherBtnVisible: false });
  }

  render() {
    return (
      <Modal title="确认信息" onCancel={this.props.onCancel} okBtnShow={false} widthSize="L">
        <InfoBlock
          title="常规合同"
          toolbar={
            <div className={style.toolBarWrap}>
              {this.state.normalBtnVisible && (
                <Button
                  onClick={this.handleNormalConfirm}
                  type="primary"
                  size="small"
                  disabled={this.props.normalList.length === 0}
                >
                  批量确认信息
                </Button>
              )}
            </div>
          }
          style={{ marginBottom: 16 }}
        >
          <List dataSource={this.props.normalList} columns={this.columns} />
        </InfoBlock>

        <InfoBlock
          title="引流合同/纯资料费/纯讲座/第三方付费合同"
          toolbar={
            <div className={style.toolBarWrap}>
              {this.state.otherBtnVisible && (
                <Button
                  onClick={this.handleOtherConfirm}
                  type="primary"
                  size="small"
                  disabled={this.props.otherList.length === 0}
                >
                  批量确认信息
                </Button>
              )}
            </div>
          }
        >
          <List dataSource={this.props.otherList} columns={this.columns} />
        </InfoBlock>
      </Modal>
    );
  }
}

export default BeforeConfirm;
