import React from 'react';
import PropTypes from 'prop-types';
import ModalConfirmContract from '../ConfirmContract';
import ModalShareConfirm from '../ConfirmContractInformationModal';
import SegmentDiscount from '../SegmentDiscount'; // 分段收费设置确认
import { runFunc } from 'UTIL/common';

class Confirm extends React.Component {
  constructor(props) {
    super(props);

    this.close = this.close.bind(this);
    this.handleConfirmContract = this.handleConfirmContract.bind(this);
    this.initModal = this.initModal.bind(this);

    this.state = {
      confirmContractVisible: false,
      shareConfirmVisible: false,
      radioMessage: true,
      chargeInfo: null,
    };

    this.refreshTrigger = false;

    this.id = null;
    this.type = null;
  }

  initModal() {
    this.setState({
      confirmContractVisible: true,
    });
  }
  static propTypes = {
    contractIds: PropTypes.arrayOf(PropTypes.string).isRequired,
    onCancel: PropTypes.func.isRequired,
    campusName: PropTypes.string.isRequired,
    customerName: PropTypes.string.isRequired,
    studentName: PropTypes.string.isRequired,
    studentId: PropTypes.string.isRequired,
    contractNumber: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
    signTime: PropTypes.string,
    isEcs: PropTypes.bool,
    useDeposit: PropTypes.bool,
  };

  static defaultProps = {
    isEcs: false,
    useDeposit: true,
  };

  close() {
    runFunc(this.props.onCancel);
    if (this.refreshTrigger) {
      GlobalEvent.emit('REFRESH_CONTRACT_LIST');
    }
  }

  handleConfirmContract(id, type) {
    this.refreshTrigger = true;
    this.id = id;
    this.type = ['pay', 'confirm'].includes(type) ? type : 'confirmPay';
    this.setState({
      confirmContractVisible: false,
      shareConfirmVisible: true,
      radioMessage: this.type === 'confirm',
    });
  }

  render() {
    return (
      <div>
        {/* 分段收费优惠设置 */}
        <SegmentDiscount onSuccess={this.initModal} contractIds={this.props.contractIds} onCancel={this.close} />
        {this.state.confirmContractVisible && (
          <ModalConfirmContract
            contractIds={this.props.contractIds}
            studentId={this.props.studentId}
            useDeposit={this.props.useDeposit}
            onCancel={this.close}
            onSuccess={this.handleConfirmContract}
          />
        )}

        {this.state.shareConfirmVisible && (
          <ModalShareConfirm
            title="确认信息"
            contractInfo={{
              id: this.id,
              type: this.type,
              contractIds: this.props.contractIds,
              campusName: this.props.campusName,
              contractNumber: this.props.contractNumber,
              customerName: this.props.customerName,
              studentName: this.props.studentName,
              signTime: this.props.signTime,
              isEcs: this.props.isEcs,
            }}
            onCancel={this.close}
            radioMessage={this.state.radioMessage}
          />
        )}
      </div>
    );
  }
}

export default Confirm;
