import React from 'react';
import PropTypes from 'prop-types';
import style from './styles.module.less';
import { Modal, List } from 'COMPONENT';
import { Row, Col } from 'antd';
import { errorsHandle, runFunc } from 'UTIL/common';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { batchConfirmActivityContract, sendSmsForActivityContract } from 'SERVICE/contract';

const setWidth = List.createSetWidth();

class ConfirmAgreement extends React.Component {
  constructor(props) {
    super(props);

    this.init = this.init.bind(this);
    this.loadData = this.loadData.bind(this);
    this.submit = this.submit.bind(this);

    this.state = {
      loading: false,
      customerName: null,
      phone: null,
      smsContent: null,
      sendRecord: [],
      okBtnDisabled: true,
    };

    this.columns = [
      { title: '发送时间', dataIndex: 'sendDateTime', key: 'sendDateTime', width: setWidth(8) },
      { title: '操作人', dataIndex: 'operator', key: 'operator', width: setWidth(8) },
      { title: '发送号码', dataIndex: 'phone', key: 'phone', width: setWidth(8) },
    ];
  }

  static propTypes = {
    contractIds: PropTypes.arrayOf(PropTypes.string).isRequired,
    onCancel: PropTypes.func.isRequired,
    onSuccess: PropTypes.func.isRequired,
  };

  componentDidMount() {
    this.init();
  }

  async init() {
    this.setState({ loading: true });
    await new Promise(this.loadData);
    this.setState({ loading: false });
  }

  async loadData(resolve) {
    const apiParams = {
      contractIdList: this.props.contractIds,
    };

    const rsp = await batchConfirmActivityContract(apiParams).catch(errorsHandle());
    if (rsp && rsp.resultCode === 0) {
      return this.setState(
        {
          ...rsp.data,
          okBtnDisabled: false,
        },
        () => {
          resolve();
        },
      );
    }

    resolve();
  }

  async submit() {
    this.setState({ loading: true });

    const apiParams = {
      contractIdList: this.props.contractIds,
    };
    const rsp = await sendSmsForActivityContract(apiParams).catch(errorsHandle());
    if (rsp && rsp.resultCode === 0 && rsp.data) {
      showMessage('短信发送成功', 'success');
      runFunc(this.props.onSuccess);
    } else {
      showMessage('短信发送失败', 'error');
      this.setState({ loading: false });
    }
  }

  render() {
    return (
      <Modal
        title="确认信息"
        loading={this.state.loading}
        okBtnDisabled={this.state.okBtnDisabled}
        onOk={this.submit}
        onCancel={this.props.onCancel}
        okText="发送短信"
        widthSize="M"
      >
        <Row className={style.row}>
          <Col span={3}>客户姓名：</Col>
          <Col span={21}>{this.state.customerName || '-'}</Col>
        </Row>
        <Row className={style.row}>
          <Col span={3}>发送号码：</Col>
          <Col span={21}>{this.state.phone || '-'}</Col>
        </Row>
        <Row className={style.row}>
          <Col span={3}>短信内容：</Col>
          <Col span={21}>{this.state.smsContent || '-'}</Col>
        </Row>

        <div className={style.subTitle}>短信发送历史</div>

        <List dataSource={this.state.sendRecord} columns={this.columns} />
      </Modal>
    );
  }
}

export default ConfirmAgreement;
