import React from 'react';
import PropTypes from 'prop-types';
import ConfirmByShareWay from 'BLOC/ConfirmByShareWay';
import { getSmsTemplate, sendContractConfirmSmsMessage } from 'SERVICE/contract';
import { errorsHandle, runFunc, desensitizationPhoneNumber } from 'UTIL/common';
import showMessage from 'SRC/components/utils/globalTips/showMessage';
import { h5ContractComfirmUrl } from 'CONST/otherSystemConfig';

const { radioPoster, radioMessage, radioLink } = ConfirmByShareWay;

const PAY = 'pay';
const CONFIRM = 'confirm';
const CONFIRM_PAY = 'confirmPay';

const h5ModeMap = {
  [PAY]: 'onlyPay',
  [CONFIRM]: 'onlyConfirm',
  [CONFIRM_PAY]: 'all',
};

const qrCodeTipMap = {
  [PAY]: '请长按二维码进行支付',
  [CONFIRM]: '请扫码确认相关信息',
  [CONFIRM_PAY]: '请扫码确认相关信息并支付',
};

class ModalOnlineConfirmWay extends React.Component {
  constructor(props) {
    super(props);

    this.init = this.init.bind(this);
    this.initData = this.initData.bind(this);
    this.sendMessage = this.sendMessage.bind(this);

    const {
      id,
      type = CONFIRM_PAY,
      contractNumber,
      customerName,
      studentName,
      signTime,
      campusName,
      isEcs,
    } = props.contractInfo;

    let radios = [];
    if (props.radioPoster) {
      radios.push(radioPoster);
    }
    if (props.radioMessage) {
      radios.push(radioMessage);
    }
    if (props.radioLink) {
      radios.push(radioLink);
    }
    if (radios.length === 0) {
      radios = [radioPoster, radioMessage, radioLink];
    }

    let routerPath = '/order/list';
    if ((type === CONFIRM || type === CONFIRM_PAY) && isEcs) {
      routerPath = '/signing/info-checking';
    }

    this.state = {
      loading: true,
      url: `${h5ContractComfirmUrl}/xh/${id}${routerPath}?mode=${h5ModeMap[type]}`,
      template: undefined,
      messageInfo: [
        { text: `发送号码：${desensitizationPhoneNumber(contractNumber)}` },
        { text: `家长姓名：${customerName}` },
      ],
      posterInfo: [
        { text: `学生：${studentName}` },
        { text: `下单时间：${signTime}` },
        { text: `学习中心：${campusName}` },
      ],
      radios,
      tips: qrCodeTipMap[type],
    };
  }

  static propTypes = {
    title: PropTypes.string,
    onCancel: PropTypes.func.isRequired,
    contractInfo: PropTypes.shape({
      // 批次id
      id: PropTypes.string.isRequired,
      contractIds: PropTypes.array,
      contractNumber: PropTypes.string,
      customerName: PropTypes.string,
      studentName: PropTypes.string,
      signTime: PropTypes.string,
      campusName: PropTypes.string,
      // pay 支付 confirm 确认 confirmPay 确认支付
      type: PropTypes.oneOf(['pay', 'confirm', 'confirmPay']),
      // 是否精英班(多个合同时，一律为false)
      isEcs: PropTypes.bool,
    }).isRequired,
    radioPoster: PropTypes.bool,
    radioMessage: PropTypes.bool,
    radioLink: PropTypes.bool,
  };

  static defaultProps = {
    title: '确认信息',
    radioPoster: true,
    radioMessage: true,
    radioLink: false,
  };

  componentDidMount() {
    this.init();
  }

  async init() {
    await new Promise(this.initData);
    this.setState({ loading: false });
  }

  async initData(resolve) {
    const res = await getSmsTemplate({ templateCodeTag: 'SMS_CONTRACT_CONFIRM_CONTRACT' }).catch(errorsHandle());
    if (res) {
      this.setState({ template: res.data.templateContent }, resolve);
    } else {
      resolve();
    }
  }

  async sendMessage() {
    const apiParams = {
      contractIds: this.props.contractInfo.contractIds,
      h5Url: this.state.url,
    };
    const res = await sendContractConfirmSmsMessage(apiParams).catch(errorsHandle());
    if (res && res.data) {
      showMessage('发送成功！', 'success');
      runFunc(this.props.onCancel);
    }
  }

  render() {
    return (
      <ConfirmByShareWay
        title={this.props.title}
        loading={this.state.loading}
        onCancel={this.props.onCancel}
        radios={this.state.radios}
        link={{
          url: this.state.url,
        }}
        message={{
          infos: this.state.messageInfo,
          template: this.state.template,
          onSend: this.sendMessage,
        }}
        poster={{
          infos: this.state.posterInfo,
          url: this.state.url,
          tips: this.state.tips,
        }}
      />
    );
  }
}

export default ModalOnlineConfirmWay;
