import React from 'react';
import Modal from 'COMPONENT/Modal';
import Button from 'COMPONENT/Button';
import { Form, Input } from 'antd';
import { sendSmsCode, confirmSavePayer } from 'SERVICE/contract';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import styles from './styles.module.less';

const FormItem = Form.Item;

const title = '确认付款人信息';

const formItemLayout = {
  labelCol: { span: 8 },
  wrapperCol: { span: 16 },
};

class ConfirmPayerInfoModal extends React.Component {
  constructor(props) {
    super(props);
    this.coutDown = this.coutDown.bind(this);
    this.sendVerifyCode = this.sendVerifyCode.bind(this);
    this.checkConfirmBtn = this.checkConfirmBtn.bind(this);
    this.cardID15To18 = this.cardID15To18.bind(this);
    this.confirmPayer = this.confirmPayer.bind(this);
    this.openNewWindow = this.openNewWindow.bind(this);
    this.close = this.close.bind(this);

    this.state = {
      // 姓名
      payerName: this.props.payerName,
      // 电话
      payerPhone: this.props.payerPhone,
      // 身份证号
      payerIdCardNo: this.props.payerIdCardNo,
      // 验证码
      verifyCode: '',
      // 确认按钮
      confirmBtnDisabled: true,
      count: 0,
    };

    this.timer = null;
  }

  componentDidMount() {}

  componentWillMount() {
    clearTimeout(this.timer);
  }

  coutDown(count) {
    this.timer = setTimeout(() => {
      if (count > 0) {
        this.setState({ count: count - 1 });
        this.coutDown(count - 1);
      }
    }, 1000);
  }

  async sendVerifyCode() {
    const messageType = 'CONTRACT_CONFIRM';
    const res = await sendSmsCode({ phone: this.state.payerPhone, type: messageType });

    this.setState({ count: 60 });
    this.coutDown(60);
  }

  formatPhone(phone) {
    return phone ? String(phone).replace(/(\d{3})(\d{4})(\d{4})/, '$1****$3') : '';
  }

  // 确认按钮高亮条件：姓名已填写，验证码已填写，身份证号已填写（满足身份证号通用填写规则）
  checkConfirmBtn() {
    const { payerPhone, payerIdCardNo, payerName, verifyCode } = this.state;

    if (payerPhone && payerIdCardNo && payerName && verifyCode && this.checkCardNo(payerIdCardNo)) {
      return true;
    }

    return false;
  }

  cardID15To18(cardID) {
    /*每位加权因子*/
    let powers = [7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2];

    /*第18位校检码*/
    let parityBit = ['1', '0', 'X', '9', '8', '7', '6', '5', '4', '3', '2'];

    //将15位的号码转换位17位
    let cardIDcardID17 = cardID.substring(0, 6) + '19' + cardID.substring(6);

    /*加权 */
    let power = 0;
    for (let i = 0; i < 17; i++) {
      power += parseInt(cardIDcardID17.charAt(i), 10) * parseInt(powers[i]);
    }
    /*取模*/
    let mod = power % 11;

    return cardIDcardID17 + parityBit[mod];
  }

  // 参考: https://juejin.im/post/6844903575877861390
  checkCode(val) {
    let p = /^[1-9]\d{5}(18|19|20)\d{2}((0[1-9])|(1[0-2]))(([0-2][1-9])|10|20|30|31)\d{3}[0-9Xx]$/;
    let factor = [7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2];
    let parity = [1, 0, 'X', 9, 8, 7, 6, 5, 4, 3, 2];
    let code = val.substring(17);
    if (p.test(val)) {
      let sum = 0;
      for (let i = 0; i < 17; i++) {
        sum += val[i] * factor[i];
      }
      if (parity[sum % 11] == code.toUpperCase()) {
        return true;
      }
    }
    return false;
  }

  checkDate(val) {
    let pattern = /^(18|19|20)\d{2}((0[1-9])|(1[0-2]))(([0-2][1-9])|10|20|30|31)$/;
    if (pattern.test(val)) {
      let year = val.substring(0, 4);
      let month = val.substring(4, 6);
      let date = val.substring(6, 8);
      let date2 = new Date(year + '-' + month + '-' + date);
      if (date2 && date2.getMonth() == parseInt(month) - 1) {
        return true;
      }
    }

    return false;
  }

  checkProv(val) {
    var pattern = /^[1-9][0-9]/;
    var provs = {
      11: '北京',
      12: '天津',
      13: '河北',
      14: '山西',
      15: '内蒙古',
      21: '辽宁',
      22: '吉林',
      23: '黑龙江 ',
      31: '上海',
      32: '江苏',
      33: '浙江',
      34: '安徽',
      35: '福建',
      36: '江西',
      37: '山东',
      41: '河南',
      42: '湖北 ',
      43: '湖南',
      44: '广东',
      45: '广西',
      46: '海南',
      50: '重庆',
      51: '四川',
      52: '贵州',
      53: '云南',
      54: '西藏 ',
      61: '陕西',
      62: '甘肃',
      63: '青海',
      64: '宁夏',
      65: '新疆',
      71: '台湾',
      81: '香港',
      82: '澳门',
    };
    if (pattern.test(val)) {
      if (provs[val]) {
        return true;
      }
    }
    return false;
  }

  checkCardNo(val) {
    if (val.length == 15) {
      val = this.cardID15To18(val);
    }

    if (this.checkCode(val)) {
      var date = val.substring(6, 14);
      if (this.checkDate(date)) {
        if (this.checkProv(val.substring(0, 2))) {
          return true;
        }
      }
    }
    return false;
  }

  async confirmPayer() {
    const params = {
      contractId: this.props.contractId,
      phone: this.state.payerPhone,
      name: this.state.payerName,
      verifyCode: this.state.verifyCode,
      cardNo: this.state.payerIdCardNo,
    };

    const res = await confirmSavePayer(params);

    showMessage('操作成功！', 'success');
    this.close();
    setTimeout(() => {
      this.openNewWindow();
    }, 1000);
  }

  openNewWindow() {
    var newWindow = window.parent.open('about:blank');
    newWindow.location.href = this.props.href;
  }

  close() {
    clearTimeout(this.timer);
    this.setState({ count: 0 }, () => {
      this.props.onCancel();
    });
  }

  render() {
    const { loading } = this.state;

    return (
      <Modal
        title={title}
        width={500}
        loading={loading}
        onCancel={this.close}
        footer={
          <div className={styles.footer}>
            <Button type="default" onClick={this.close}>
              关闭
            </Button>
            <Button type="primary" disabled={!this.checkConfirmBtn()} onClick={this.confirmPayer}>
              确认
            </Button>
          </div>
        }
      >
        <Form className={styles.contractPayerInfoForm}>
          <FormItem {...formItemLayout} label="付款人姓名">
            <Input value={this.state.payerName} onChange={(e) => this.setState({ payerName: e.target.value })} />
          </FormItem>
          <FormItem {...formItemLayout} label="联系电话">
            <div>{this.formatPhone(this.state.payerPhone)}&nbsp;</div>
            {
              <div className={styles.verify}>
                <div className={styles.verifyInput}>
                  <Input
                    size="large"
                    value={this.state.verifyCode}
                    onChange={(e) => this.setState({ verifyCode: e.target.value })}
                  />
                </div>
                <Button type="primary" size="middle" disabled={this.state.count > 0} onClick={this.sendVerifyCode}>
                  {this.state.count > 0 ? `重新发送(${this.state.count}s)` : `发送验证码`}
                </Button>
              </div>
            }
          </FormItem>
          <FormItem {...formItemLayout} label="身份证号">
            <Input
              value={this.state.payerIdCardNo}
              onChange={(e) => this.setState({ payerIdCardNo: e.target.value })}
            />
          </FormItem>
        </Form>
      </Modal>
    );
  }
}

export default ConfirmPayerInfoModal;
