import React from 'react';
import PropTypes from 'prop-types';
import { Modal, List, TextLink, QrCode } from 'COMPONENT';
import { Popover } from 'antd';
import style from './styles.module.less';
import { sourceDict, confirmChannelDict } from 'CONST/dicts/contract';
import { getContractModifyList, getElectronicContractPath } from 'SERVICE/contract';
import { getTemplateCode } from 'SERVICE/print';
import { errorsHandle, loginUser } from 'UTIL/common';
import moment from 'moment';
import { h5ContractComfirmUrl } from 'CONST/otherSystemConfig';
import { pdfViewerHref } from 'CONST/config';
import PreviewImgs from 'COMPONENT/PreviewImgs';
import file from 'UTIL/file';

const setWidth = List.createSetWidth();

const momentFormat = 'YYYY-MM-DD HH:mm:ss';

const ON_LINE = 'ON_LINE';

class ContractModifyRecord extends React.Component {
  constructor(props) {
    super(props);

    this.init = this.init.bind(this);
    this.loadData = this.loadData.bind(this);
    this.previewSnapshot = this.previewSnapshot.bind(this);
    this.getColumns = this.getColumns.bind(this);

    this.state = {
      loading: false,
      list: [],
      total: 0,
      photoModal: false,
      arrayList: [],
      fileFullPath: [],
      currImage: '',
      imgShow: false,
    };

    this.columns = [
      { title: '操作类型', dataIndex: 'source', key: 'source', dict: sourceDict, width: setWidth(4) },
      { title: '操作人', dataIndex: 'createUserName', key: 'createUserName', width: setWidth(4) },
      {
        title: '修改时间',
        dataIndex: 'contractUpdateTime',
        key: 'contractUpdateTime',
        width: setWidth(8),
        render: (text) => (text ? moment(text).format(momentFormat) : ''),
      },
      { title: '确认信息状态', dataIndex: 'confirmStatusName', key: 'confirmStatusName', width: setWidth(4) },
      { title: '确认渠道', dataIndex: 'confirmChannelName', key: 'confirmChannelName', width: setWidth(4) },
      {
        title: '确认时间',
        dataIndex: 'confirmTime',
        key: 'confirmTime',
        width: setWidth(8),
        render: (text) => (text ? moment(text).format(momentFormat) : ''),
      },
    ];

    this.baseUrl = `${h5ContractComfirmUrl}/${loginUser.isXingHuoSym() ? 'xh' : 'ss'}`;
  }

  static propTypes = {
    contractId: PropTypes.string.isRequired,
    onCancel: PropTypes.func.isRequired,
    characterText: PropTypes.string,
    fileList: PropTypes.object,
  };

  componentDidMount() {
    this.init();
  }

  async init() {
    this.setState({ loading: true });
    await new Promise(this.loadData);
    this.setState({ loading: false });
  }

  async loadData(resolve) {
    const res = await getContractModifyList({ contractId: this.props.contractId }).catch(errorsHandle());

    if (res && res.resultCode === 0) {
      this.setState({
        list: res.data,
        total: res.data.length,
      });
    }

    resolve();
  }

  previewSnapshot(prevId, currentId) {
    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path: `EditContractDetail/${currentId}/${prevId}`,
      },
    });
  }

  showUploadedPhotos = async (fileList) => {
    let fileFullPath = fileList.map((value) => {
      return value.fileFullPath;
    });
    this.setState({ photoModal: true, arrayList: fileFullPath, fileFullPath: fileList });
  };

  getColumns() {
    const action = {
      title: '操作',
      dataIndex: 'batchId',
      key: 'actionPlus',
      width: setWidth(6),
      render: (text, record, index) => {
        return (
          <div className={style.listAction}>
            {index < this.state.total - 1 && (
              <TextLink
                onClick={(e) => {
                  e.preventDefault();
                  e.stopPropagation();
                  this.previewSnapshot(
                    this.state.list[index + 1].contractSnapshotRecordId,
                    record.contractSnapshotRecordId,
                  );
                }}
              >
                查看详情
              </TextLink>
            )}

            {record.confirmChannel === 'OFF_LINE' && record.fileList.length && (
              <TextLink
                onClick={(e) => {
                  e.preventDefault();
                  e.stopPropagation();
                  this.showUploadedPhotos(record.fileList);
                }}
              >
                查看
              </TextLink>
            )}

            {index === 0 && record.confirmChannel === ON_LINE && [2, 4].includes(record.confirmStatus) && text && (
              <Popover
                content={
                  <QrCode
                    id={`confirmContract-${index}`}
                    text={`${this.baseUrl}/${text}/order/list?mode=${
                      record.confirmMode === 'PAY_CONFIRM' ? 'all' : 'onlyConfirm'
                    }`}
                    width={120}
                    height={120}
                  />
                }
                trigger="click"
              >
                <TextLink>分享</TextLink>
              </Popover>
            )}

            {index === 0 &&
              record.confirmChannel === ON_LINE &&
              record.confirmStatus === 4 &&
              record.electronicContractAddress && (
                <TextLink
                  onClick={() => {
                    this.printPdf(record.electronicContractAddress);
                  }}
                >
                  打印
                </TextLink>
              )}
          </div>
        );
      },
    };
    return this.columns.concat(action);
  }

  closePhotoModal = () => {
    this.setState({ photoModal: false });
  };

  previewPhoto = (path) => {
    this.setState({
      currImage: path,
      imgShow: true,
    });
  };

  printPdf = (path) => {
    var newWindow = window.parent.open('about:blank');
    // newWindow.location.href = pdfViewerHref.replace('${URL}', encodeURIComponent(path));

    newWindow.location.href = `${location.origin}/printFile?url=${encodeURIComponent(path)}`;
  };

  render() {
    const { arrayList, photoModal, currImage, imgShow } = this.state;

    return (
      <div>
        <Modal
          title={
            <div>
              <span>合同修改记录</span>
              <span>（</span>
              <span className={style.subTitle}>
                <span className={style.subTitleItem}>合同编号：{this.props.contractId}</span>

                {this.props.characterText && <span className={style.subTitleItem}>{this.props.characterText}合同</span>}
              </span>
              <span>）</span>
            </div>
          }
          loading={this.state.loading}
          onCancel={this.props.onCancel}
          okBtnShow={false}
          widthSize="L"
        >
          <List columns={this.getColumns()} dataSource={this.state.list} />
        </Modal>
        <Modal
          title="已上传的合同照片"
          centered
          cancelText="关闭"
          okBtnShow={false}
          toggle={true}
          visible={photoModal}
          onCancel={() => this.closePhotoModal()}
        >
          <div className={style.imagePanel}>
            {arrayList.map((value, key) => {
              return <img src={value} onClick={() => this.previewPhoto(value)} key={key} />;
            })}
          </div>
        </Modal>
        <Modal
          width={800}
          okBtnShow={false}
          cancelBtnShow={false}
          closable={true}
          toggle={true}
          visible={imgShow}
          onClose={() => this.setState({ imgShow: false })}
        >
          <PreviewImgs list={arrayList} activeSrc={currImage} />
        </Modal>
      </div>
    );
  }
}

export default ContractModifyRecord;
