import React from 'react';
import Select from 'COMPONENT/Select';
import Button from 'COMPONENT/Button';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { Row, Form, Input, Checkbox, Radio } from 'antd';
import { DATE_FORMAT } from 'COMPONENT/utils/form';
import DatePicker from 'COMPONENT/DatePicker';
import moment from 'moment';
import _ from 'lodash';
import { uuid } from 'COMPONENT/utils/basic/str';
import classNames from 'classnames';
import {
  getTransactionUUID,
  getPosNumSelectOptionByCampusId,
  sendBatchPayDetailInfo,
  updateAndCheckBossBatchPayInfo,
  getFundsInfoByPosId,
} from 'SERVICE/contract';
import { getStudentAccoutInfo, getStudentAccDetailInfo } from 'SERVICE/eleAcc';
import { getSelectOption } from 'SERVICE/scoreManage';
import { getDataDictList } from 'SERVICE/system';
import {
  defaultPaymentDict,
  paidStatus,
  visibleMap,
  valideConfig,
  channelMap,
  payStatusMap,
  POS,
  EDU_PLATFORM,
  BANK_TRANSFER,
  THIRD_PARTY_PAYMENT,
  ELECTRONIC_ACCOUNT,
  ELECTRONIC_ACCOUNT_A,
  QRCODE_PAY,
  defaultChannel,
  scanPaymentDict,
} from './config';
import styles from './styles.module.less';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';

const { TextArea } = Input;
const FormItem = Form.Item;
const CheckboxGroup = Checkbox.Group;

const today = moment().format(DATE_FORMAT);

const formItemLayout = {
  labelCol: { span: 8 },
  wrapperCol: { span: 16 },
  labelAlign: 'left',
};

class ContractPaymentForm extends React.Component {
  constructor(props) {
    super(props);
    this.handleChannelChange = this.handleChannelChange.bind(this);
    this.checkPosId = this.checkPosId.bind(this);
    this.getTransactionParams = this.getTransactionParams.bind(this);
    this.generatePaymentQrCode = this.generatePaymentQrCode.bind(this);
    this.fetchPaymentQrCode = this.fetchPaymentQrCode.bind(this);
    this.handleElectronicCheckChange = this.handleElectronicCheckChange.bind(this);

    this.state = {
      // 提交唯一id
      posNumberDict: [],
      transactionUUID: null,
      // 电子账户
      electronicAccount: 0,
      canUseAmount: 0,
      frozenAmount: 0,
      accList: [],
      // 日期
      posReceiptDate: today,
      // 订单编码
      reqsn: null,
      // 校验数据
      checkPosData: [],
      // 电子账户已选项
      electronicSelected: [],
      // 是否共用参考号
      posFlag: 0,
      // 第三方平台数据字典
      thirdPartyPaymentDict: [],
      // 第三方平台id
      platformId: '',
      paymentDict: this.paymentDictValues,
      // 支付方式
      channel: this.defaultChannel, // 合同收款类型为教育平台时，点击【录入线下收款】，支付方式默认【教育平台】
      ...visibleMap,
    };

    this.pollTimer = null;
    this.contractProductPaymentList = null;
  }

  componentDidMount() {
    this.initComponent();
  }

  componentWillUnmount() {
    clearInterval(this.pollTimer);
  }

  get paymentDictValues() {
    const { paidStatus, contractFundTypeName } = this.props;
    const defaultValues = [...defaultPaymentDict, ...scanPaymentDict];
    if (contractFundTypeName === '教育平台') {
      return [defaultValues.find(({ val }) => val === EDU_PLATFORM)];
    } else {
      // 未付款的付款方式可以所有，否则去掉教育平台
      return paidStatus === 'UNPAY' ? defaultValues : defaultValues.filter(({ val }) => val !== EDU_PLATFORM);
    }
  }

  get defaultChannel() {
    return this.props.contractFundTypeName === '教育平台' ? EDU_PLATFORM : defaultChannel;
  }

  initComponent() {
    this.getTransactionUUID();
    // 获取电子账户支付方式字典
    this.getECFlowPayTypeDictObj();
    // 加载第三方平台数据字典
    this.loadThirdPartyPayment();
    // 获取默认的渠道之外的渠道
    // this.getCampusFundsPayTypes();
    if (this.props.contractFundTypeName === '教育平台') {
      this.setEduPlatformMachineOption();
      this.handleChannelChange(EDU_PLATFORM);
    } else {
      // pos机终端选项
      this.setPOSMachineOption();
      // 默认触发POS选项
      this.handleChannelChange(POS);
    }
  }

  /**
   * 加载第三方平台数据字典
   * @memberof ContractPaymentForm
   */
  async loadThirdPartyPayment() {
    const result = await getDataDictList({
      category: 'THIRD_PARTY_PAYMENT',
    });
    const thirdPartyPaymentDict = (result.resultCode == 0 && result.rows.length ? result.rows : []).map(
      ({ value, name }) => ({ val: value, text: name }),
    );
    thirdPartyPaymentDict.unshift({ val: '', text: '请选择' });
    this.setState({ thirdPartyPaymentDict });
  }

  async getTransactionUUID() {
    const transactionUUID = await getTransactionUUID();
    this.setState({ transactionUUID });
  }

  async getECFlowPayTypeDictObj() {
    let res = await getSelectOption({ selectOptionCategory: 'ECFlowPayType' });
    this.setState({ ECFlowPayTypeDictObj: res.value });
  }

  async handleChannelChange(channel) {
    let payStatus = payStatusMap.SUCCESS;

    if (channelMap[channel] === POS || channelMap[channel] === EDU_PLATFORM) {
      this.loadMachineOption(channelMap[channel]);
      this.setState({
        ...visibleMap,
        posVisible: true,
      });
    } else if (channelMap[channel] === BANK_TRANSFER) {
      this.setState({
        ...visibleMap,
        collectVisible: true,
      });
    } else if (channelMap[channel] === QRCODE_PAY) {
      this.setState({
        ...visibleMap,
        scanningChannelVisible: true,
      });

      payStatus = payStatus.Unpaid;
    } else if (channelMap[channel] === THIRD_PARTY_PAYMENT) {
      // 选择第三方平台
      this.setState({
        ...visibleMap,
        platformVisible: true,
      });
    } else if (channelMap[channel] === ELECTRONIC_ACCOUNT) {
      this.setState({
        ...visibleMap,
        electronicAccountVisble: true,
      });

      this.getStudentAccountInfo();
      this.getStudentAccDetailInfo();
    } else {
      this.setState({
        ...visibleMap,
      });
    }

    clearInterval(this.pollTimer);
    this.setState({ channel, payStatus, intenetAgainVisible: false, errorMsgInfo: '' });
    this.props.updatePayStatus(payStatus);
    this.props.updateChannel(channel);
  }

  handleChangePlatformId = (platformId) => {
    this.setState({
      platformId,
      platformName: platformId
        ? (this.state.thirdPartyPaymentDict.find(({ val }) => val == platformId) || {}).text
        : '',
    });
  };

  async getStudentAccountInfo() {
    const { studentId } = this.props;

    const res = await getStudentAccoutInfo({ studentId });
    const electronicAccount = res.electronicAccount || 0;
    const frozenAmount = res.frozenAmount || 0;
    this.setState({
      electronicAccount: electronicAccount.toFixed(2),
      canUseAmount: (electronicAccount - frozenAmount).toFixed(2),
      frozenAmount: frozenAmount.toFixed(2),
    });
  }

  async loadMachineOption(payType) {
    const { blCampusId: campusId } = this.props;

    if (!campusId) return;

    let params;

    if (payType === POS) {
      params = { campusId };
    } else if (payType === EDU_PLATFORM) {
      params = { campusId, eduPlatForm: true };
    } else {
      return;
    }

    const result = await getPosNumSelectOptionByCampusId(params);
    const posNumberDict = _.map(result.value || {}, (text, val) => ({ val, text })).filter(({ val }) => val !== '');
    this.setState({
      posNumberDict: [{ val: '', text: '请选择' }, ...posNumberDict],
    });
  }

  /**
   * 付款方式为POS设置“收款终端号”
   */
  async setPOSMachineOption() {
    this.loadMachineOption(POS);
  }

  /**
   * 付款方式为教育平台设置“收款终端号”
   */
  async setEduPlatformMachineOption() {
    this.loadMachineOption(EDU_PLATFORM);
  }

  // 根据学生   获取账户金额信息
  async getStudentAccDetailInfo() {
    const { studentId } = this.props;
    const { ECFlowPayTypeDictObj: dictObj } = this.state;
    const res = await getStudentAccDetailInfo({ studentId });

    let data = res.data || {};
    let accList = [];
    for (let key in data) {
      accList.push({
        uuid: uuid(),
        payType: key,
        amount: data[key],
        payTypeText: dictObj[key],
      });

      this.setState({ accList });
    }
  }

  generatePaymentQrCode() {
    if (!this.checkTheMoney()) return;

    let apiParams = {
      transactionAmount: this.props.transactionAmount,
      payType: this.state.channel,
      title: '收款',
      remark: this.state.remark,
      contractProductPaymentList: this.props.contractProductPaymentList || [],
      studentId: this.props.studentId,
      depositIds: this.props.depositIds,
    };

    if (this.contractProductPaymentList) {
      apiParams.contractProductPaymentList = this.contractProductPaymentList;
      apiParams.transactionAmount = this.props.offsetAmount;
      apiParams.depositIds = [];
    }

    if (apiParams.depositIds.length > 0) {
      return showConfirm(
        `使用微信支付或支付宝支付，在生成付款二维码时就会将选中的定金券（金额共 ${this.props.depositAmount} 元）进行核销，请问是否确定要生成支付二维码。（定金券核销后不可恢复）`,
        () => {
          this.fetchPaymentQrCode(apiParams);
        },
      );
    }

    this.fetchPaymentQrCode(apiParams);
  }

  // 生成二维码
  async fetchPaymentQrCode(apiParams) {
    const res = await sendBatchPayDetailInfo(apiParams);
    const { data } = res;

    this.contractProductPaymentList = data.contractProductPaymentList;

    this.setState({
      // 收款二维码(图片链接地址)
      qrCodeUrl: data.payInfo,
      // 支付中心-订单号
      reqsn: data.reqsn,
      isQrcodePaying: true,
    });
    this.updatePayStatus(payStatusMap.PENDING);

    this.pollTimer = setInterval(() => {
      this.pollUpdateAndCheckBossBatchPayInfo(data.reqsn);
    }, 3000);
  }

  async pollUpdateAndCheckBossBatchPayInfo(reqsn) {
    const res = await updateAndCheckBossBatchPayInfo({ reqsn });
    if (!res.data) {
      clearInterval(this.pollTimer);
      return;
    }
    const status = res.data;

    // 扫码成功
    if (status === paidStatus.PAID) {
      showMessage('支付成功！', 'success');
      this.updatePayStatus(payStatusMap.SUCCESS);
      clearInterval(this.pollTimer);
    } else if (status === paidStatus.CANCEL || status === paidStatus.INVALID) {
      // 取消支付
      showMessage('支付取消！', 'success');
      this.updatePayStatus(payStatusMap.Unpaid);
      this.setState({ errorMsgInfo: res.resultMessage, intenetAgainVisible: true, isQrcodePaying: false });
      clearInterval(this.pollTimer);
    } else if (status === paidStatus.UNPAID_TIMEOUT) {
      // 扫码失败， 超时
      this.updatePayStatus(payStatusMap.FAIL);
      this.setState({ errorMsgInfo: res.resultMessage, intenetAgainVisible: true, isQrcodePaying: false });
      showMessage(res.resultMessage);
      clearInterval(this.pollTimer);
    } else {
      this.updatePayStatus(payStatusMap.PENDING);
    }
  }

  updatePayStatus(payStatus) {
    this.setState({ payStatus });

    this.props.updatePayStatus && this.props.updatePayStatus(payStatus);
  }

  checkTheMoney() {
    const { transactionAmount, pendingAmount } = this.props;

    if (transactionAmount <= 0) {
      showMessage('实际支付必须大于0！');
      return false;
    }

    if (transactionAmount > pendingAmount) {
      showMessage('不能输入过大的金额！');
      return false;
    }

    return true;
  }

  validForm(checkConfig = {}) {
    if (checkConfig.posNumber && !this.state.posNumber) {
      showMessage('请选择终端号！');
      return;
    }

    if (checkConfig.posId && !this.state.posId) {
      showMessage('请输入支付参考号！');
      return;
    }

    if (checkConfig.posReceiptDate && !this.state.posReceiptDate) {
      showMessage('请输入支付时间！');
      return;
    }

    if (checkConfig.platformName && !this.state.platformName) {
      showMessage('请选择平台名称！');
      return;
    }

    if (checkConfig.thirdPartyOrderNo && !this.state.thirdPartyOrderNo) {
      showMessage('请输入订单号！');
      return;
    }

    if (checkConfig.thirdPartyProductName && !this.state.thirdPartyProductName) {
      showMessage('请输入产品名称！');
      return;
    }

    if (checkConfig.collectCardNo && !/^\d{1,25}$/.test(this.state.collectCardNo)) {
      showMessage('请输入有效的收款账号（1 ~ 25 位数字）');
      return;
    }

    if (checkConfig.collector && !this.state.collector) {
      showMessage('请输入收款人！');
      return;
    }

    return true;
  }

  async getTransactionParams() {
    const { channel } = this.state;
    let posNumber = this.state.posNumber;
    let posMachineType = '';
    let params = {
      transactionUuid: this.state.transactionUUID,
      transactionAmount: this.props.transactionAmount,
      channel: this.state.channel,
      remark: this.state.remark,
    };

    // 验证表单内容
    if (!this.validForm(valideConfig[channel]) || !this.checkTheMoney()) return;

    // POS
    if (channelMap[channel] === POS) {
      if (posNumber) {
        let posNumAndType = posNumber.split('-');
        posNumber = posNumAndType[0];
        posMachineType = posNumAndType[1];
      }
    }

    // 校验电子账户
    if (channelMap[channel] === ELECTRONIC_ACCOUNT) {
      let totalSelectedAmount = 0;
      let ecPayAmounts = [];
      let ecPayTypes = [];

      this.state.electronicSelected.forEach((item) => {
        totalSelectedAmount += item.amount;
        ecPayTypes.push(item.payType);
        ecPayAmounts.push(item.amount);
      });

      if (!this.checkElectronicSelected(totalSelectedAmount)) return;
      params.ecPayAmounts = ecPayAmounts.join(',');
      params.ecPayTypes = ecPayTypes.join(',');
    }

    if (channelMap[channel] === POS) {
      params = {
        ...params,
        posId: this.state.posId,
        posReceiptDate: this.state.posReceiptDate,
        posNumber: posNumber,
        posMachineTypeId: posMachineType,
        posFlag: 1,
      };
    } else if (channelMap[channel] === EDU_PLATFORM) {
      // 教育平台
      params = {
        ...params,
        posNumber,
        posId: this.state.posId,
        posReceiptDate: this.state.posReceiptDate,
      };
    } else if (channelMap[channel] === THIRD_PARTY_PAYMENT) {
      // 第三方平台
      params = {
        ...params,
        platformId: this.state.platformId,
        platformName: this.state.platformName,
        thirdPartyOrderNo: this.state.thirdPartyOrderNo,
        thirdPartyProductName: this.state.thirdPartyProductName,
      };
    } else if (channelMap[channel] === BANK_TRANSFER) {
      params = {
        ...params,
        collectCardNo: this.state.collectCardNo,
        collector: this.state.collector,
      };
    }

    if (channelMap[channel] === ELECTRONIC_ACCOUNT_A) {
      params.posMachineTypeId = 'POS_MACHINE_TYPE_A';
    }

    return params;
  }

  checkElectronicSelected(totalSelectedAmount) {
    if (!totalSelectedAmount) {
      showMessage('请勾选需要消耗的产品！');
      return false;
    }

    if (this.props.transactionAmount > totalSelectedAmount) {
      showMessage('勾选产品的金额必须大于等于实际支付金额！');
      return false;
    }

    return true;
  }

  // 已废弃不需校验
  async checkPosId() {
    const { blCampusId } = this.props;
    let { posId, posNumber } = this.state;
    posNumber && (posNumber = posNumber.split('-')[0]);

    if (posNumber && posId) {
      const res = await getFundsInfoByPosId({ blCampusId, posId, posNumber });
      if (res && res.data && res.data.length > 0) {
        this.setState({ checkPosData: res.data });
      } else {
        this.resetPosCheck();
      }
    } else {
      this.resetPosCheck();
    }
  }

  resetPosCheck() {
    this.setState({ checkPosData: [], posFlag: 0 });
  }

  // 只能输入数字 去空格之后判断是否为纯数字即可赋值
  handleNumberChange = (val) => {
    const value = val.replace(/\s/g, '');

    const reg = /^[0-9]*$/;
    if (reg.test(value) || value === '') {
      this.setState({
        posId: value,
      });
    }
  };

  formatNumber = (val) => {
    let value = val || '';
    value = value.replace(/\s/g, '').replace(/(\d{4})(?=\d)/g, '$1 ');
    return value;
  };

  handleElectronicCheckChange(checkedValues) {
    let selectedList = this.state.accList.filter((item) => checkedValues.includes(item.uuid));
    this.setState({ electronicSelected: selectedList });
  }

  destroyIframe() {
    const iframe = document.querySelector('#contractPaymentIframe');

    iframe && iframe.remove();
  }

  render() {
    const { offsetAmount, isBroadcastCooperate } = this.props;
    const {
      paymentDict,
      platformId,
      channel,
      posNumber,
      posId,
      posReceiptDate,
      posNumberDict,
      thirdPartyOrderNo,
      thirdPartyProductName,
      remark,
      posVisible,
      platformVisible,
      collectVisible,
      electronicAccountVisble,
      scanningChannelVisible,
      electronicAccount,
      canUseAmount,
      frozenAmount,
      collectCardNo,
      collector,
      isQrcodePaying,
      thirdPartyPaymentDict,
      qrCodeUrl,
      posFlag,
    } = this.state;

    return (
      <div className={styles.contractPayment}>
        {!isQrcodePaying && (
          <Form className={styles.contractPaymentForm}>
            <FormItem {...formItemLayout} label="支付方式" required>
              <Select
                value={channel}
                allowClear={false}
                placeholder="支付方式"
                onChange={this.handleChannelChange}
                dict={(paymentDict || []).filter((item) =>
                  isBroadcastCooperate ? item.val !== 'ELECTRONIC_ACCOUNT' : true,
                )}
              />
              {/**广电项目不可选电子账户*/}
            </FormItem>
            {platformVisible && (
              <Row>
                <FormItem {...formItemLayout} label="平台名称" required>
                  <Select
                    value={platformId}
                    allowClear={false}
                    placeholder="请选择"
                    dict={thirdPartyPaymentDict}
                    onChange={this.handleChangePlatformId}
                  />
                </FormItem>
                <FormItem {...formItemLayout} label="订单号" required>
                  <Input
                    value={thirdPartyOrderNo}
                    onChange={(e) => this.setState({ thirdPartyOrderNo: e.target.value })}
                  />
                </FormItem>
                <FormItem {...formItemLayout} label="产品名称" required>
                  <Input
                    value={thirdPartyProductName}
                    onChange={(e) => this.setState({ thirdPartyProductName: e.target.value })}
                  />
                </FormItem>
              </Row>
            )}
            <FormItem {...formItemLayout} label="实际支付">
              <span>¥{offsetAmount}</span>
            </FormItem>
            {posVisible && (
              <Row>
                <FormItem {...formItemLayout} label="收款终端号" required>
                  <Select
                    allowClear={false}
                    placeholder="请选择终端号"
                    dict={posNumberDict}
                    value={posNumber}
                    onChange={(posNumber) => {
                      this.setState({ posNumber });
                    }}
                  />
                </FormItem>
                <FormItem {...formItemLayout} label="支付参考号" required>
                  <Input
                    value={posId}
                    onChange={(e) => this.setState({ posId: e.target.value.replace(/\s/g, '') })}
                    maxLength="64"
                  />
                </FormItem>
                {this.state.checkPosData.length > 0 && (
                  <div className={styles.posCheck}>
                    <div className={styles.danger}>该参考号已被其他合同占用，是否共用参考号？</div>
                    <Radio.Group onChange={(e) => this.setState({ posFlag: e.target.value })} value={posFlag}>
                      <Radio value={0}>否</Radio>
                      <Radio value={1}>是</Radio>
                    </Radio.Group>
                  </div>
                )}
                {this.state.checkPosData.map((item, index) => (
                  <div className={classNames(styles.posCheckList, styles.danger)} key={index}>
                    <div className={styles.posCheckItem}>
                      <span>{item.studentName}</span>
                      <span>{item.contractId}</span>
                      <span>{item.paidMoney}</span>
                    </div>
                  </div>
                ))}
                <FormItem {...formItemLayout} label="支付时间">
                  <DatePicker
                    value={posReceiptDate}
                    size={'small'}
                    onChange={(val) => this.setState({ posReceiptDate: val })}
                  />
                </FormItem>
              </Row>
            )}
            {collectVisible && (
              <Row>
                <FormItem {...formItemLayout} label="收款账号" required>
                  <Input
                    maxLength="25"
                    value={collectCardNo}
                    onChange={(e) => this.setState({ collectCardNo: e.target.value })}
                  />
                </FormItem>
                <FormItem {...formItemLayout} label="收款人" required>
                  <Input value={collector} onChange={(e) => this.setState({ collector: e.target.value })} />
                </FormItem>
              </Row>
            )}
            {electronicAccountVisble && (
              <div>
                <FormItem {...formItemLayout} label="电子账户余额">
                  <div>{electronicAccount}元</div>
                  <div>
                    (可用：{canUseAmount} 元， 冻结 {frozenAmount}元)
                  </div>
                </FormItem>
                <div className={styles.electronicAccountList}>
                  <CheckboxGroup onChange={this.handleElectronicCheckChange}>
                    {this.state.accList.map((item, index) => (
                      <div key={index} className={styles.electronicAccountItem}>
                        <Checkbox value={item.uuid}>{item.payTypeText}</Checkbox>
                        <span className={styles.electronicAccountText}>可用余额：¥{item.amount}</span>
                      </div>
                    ))}
                  </CheckboxGroup>
                  {this.state.accList.length === 0 && <div className={styles.danger}>暂无消耗产品数据</div>}
                </div>
              </div>
            )}

            <FormItem {...formItemLayout} label="备注">
              <TextArea
                value={remark}
                onChange={(e) => this.setState({ remark: e.target.value })}
                rows={4}
                maxLength="1000"
              />
            </FormItem>

            {electronicAccountVisble && (
              <div className={styles.electronTips}>
                <p>电子账户使用说明：</p>
                <p style={{ color: 'red' }}>小班产品使用非小班电子账户余额会分配责任金额</p>
                <p style={{ color: 'red' }}>非小班产品使用小班电子账户余额会分配业绩金额</p>
              </div>
            )}

            {posVisible && (
              <div className={styles.danger}>
                注意：线上一笔收款需要对应线下一笔POS机收款，如果有多次刷卡，请在线上多次操作收款，如有例外情况请联系财务；
              </div>
            )}
            <div>
              <span id="errorMsgInfo" style={{ color: 'red' }}>
                {this.state.errorMsgInfo}
              </span>
            </div>

            {scanningChannelVisible && (
              <FormItem {...formItemLayout} label="请选择扫码方式">
                <div style={{ textAlign: 'center' }}>
                  {!this.state.intenetAgainVisible && (
                    <div className={styles.intenetCode} onClick={this.generatePaymentQrCode} />
                  )}
                  {this.state.intenetAgainVisible && <Button onClick={this.generatePaymentQrCode}>重新生成</Button>}
                </div>
              </FormItem>
            )}
          </Form>
        )}
        {isQrcodePaying && (
          <div className={styles.qrCodePay}>
            <div className={styles.transactionAmountLabel}>实际支付： ¥{offsetAmount}</div>
            <iframe id="contractPaymentIframe" src={qrCodeUrl} width="280" height="280" />
          </div>
        )}
      </div>
    );
  }
}

export default ContractPaymentForm;
