import React from 'react';
import PropTypes from 'prop-types';
import ContractPaymentModal from '../../ContractPaymentModal';
import ConfirmContractInformationModal from '../../ConfirmContractInformationModal';
import { runFunc } from 'SRC/utils/common';

class Modal extends React.Component {
  constructor(props) {
    super(props);

    this.close = this.close.bind(this);
    this.handlePayment = this.handlePayment.bind(this);
    this.closeContractInfo = this.closeContractInfo.bind(this);

    this.state = {
      receivePaymentVisible: true,
      contractInformationModalVisible: false,
      modalTitle: '确认信息',
    };

    this.id = null;
    this.type = null;
  }

  static propTypes = {
    onCancel: PropTypes.func.isRequired,
    contractIds: PropTypes.arrayOf(PropTypes.string).isRequired,
    campusId: PropTypes.string.isRequired,
    studentId: PropTypes.string.isRequired,
    campusName: PropTypes.string.isRequired,
    customerName: PropTypes.string.isRequired,
    studentName: PropTypes.string.isRequired,
    contractNumber: PropTypes.oneOfType([PropTypes.string, PropTypes.number]).isRequired,
    signTime: PropTypes.string,
  };

  close() {
    runFunc(this.props.onCancel);
  }

  handlePayment(id, type) {
    this.id = id;
    this.type = ['pay', 'confirm'].includes(type) ? type : 'confirmPay';

    this.setState({
      receivePaymentVisible: false,
      contractInformationModalVisible: true,
    });
  }

  closeContractInfo() {
    this.setState({ contractInformationModalVisible: false }, () => {
      this.close();
      GlobalEvent.emit('REFRESH_CONTRACT_LIST');
    });
  }

  render() {
    return (
      <div>
        {/* 录入线下收款 */}
        {this.state.receivePaymentVisible && (
          <ContractPaymentModal
            contractIds={this.props.contractIds}
            blCampusId={this.props.campusId}
            studentId={this.props.studentId}
            onCancel={this.close}
            confirm={this.handlePayment}
          />
        )}

        {/* 分享H5链接 */}
        {this.state.contractInformationModalVisible && (
          <ConfirmContractInformationModal
            title={this.state.modalTitle}
            contractInfo={{
              id: this.id,
              type: this.type,
              contractIds: this.props.contractIds,
              campusName: this.props.campusName,
              contractNumber: this.props.contractNumber,
              customerName: this.props.customerName,
              studentName: this.props.studentName,
              signTime: this.props.signTime,
            }}
            onCancel={this.closeContractInfo}
          />
        )}
      </div>
    );
  }
}

export default Modal;
