import React from 'react';
import PropTypes from 'prop-types';
import { Modal, Button, PopoverWrap } from 'COMPONENT';
import { Modal as AntdModal } from 'antd';
import ContractPaymentForm from './ContractPaymentForm';
import ContractProductPayment from '../../BLoComponets/ContractProductPayment';
import {
  getContractByIds,
  batchSaveFundOfContractAndConfirm,
  batchConfirmingContract,
  batchSaveFundOfContract,
  batchDepositUsed,
  getSegmentChargeItemByContractIds, // 根据合同id集合获取分段收费明细数据
} from 'SERVICE/contract';
import { errorsHandle, runFunc } from 'UTIL/common';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { EDU_PLATFORM, payStatusMap, channelMap, QRCODE_PAY, THIRD_PARTY_PAYMENT } from './ContractPaymentForm/config';
import { invalidFilter } from '../../BLoComponets/ContractProductPayment/utils';
import NP from 'number-precision';
import UseDeposit from '../../BLoComponets/UseDeposit';
import styles from './styles.module.less';
import Statistics from '../../BLoComponets/ContractProductPayment/Statistics';

const confirm = AntdModal.confirm;

class ContractPaymentModal extends React.Component {
  constructor(props) {
    super(props);
    this.saveCollectionInformation = this.saveCollectionInformation.bind(this);
    this.confirmAndPay = this.confirmAndPay.bind(this);
    this.saveDeposit = this.saveDeposit.bind(this);
    this.saveDepositThenConfirm = this.saveDepositThenConfirm.bind(this);
    this.onPaymentChange = this.onPaymentChange.bind(this);
    this.calcTotalAmount = this.calcTotalAmount.bind(this);
    this.loadData = this.loadData.bind(this);
    this.getParams = this.getParams.bind(this);
    this.cancel = this.cancel.bind(this);
    this.close = this.close.bind(this);
    this.onDepositChange = this.onDepositChange.bind(this);

    this.state = {
      loading: false,
      // 默认可以保存
      payStatus: payStatusMap.SUCCESS,
      // 实收
      transactionAmount: 0,
      // 应收
      pendingAmount: 0,
      // 合同列表
      list: [],
      // 合同支付信息
      contractProductPaymentList: [],
      // 是否需要确认
      isNeedConfirm: false,
      // 本次支付的合同里是否存在待分配资金不为0的合同, 有则置灰 (暂时放开)
      hasUnableAmount: false,
      // 付款方式
      channel: '',

      maxDepositVal: 0,
      depositAmount: 0,
      depositSelected: [],
      // 应收 - 定金券金额
      offsetAmount: 0,
      // 统计值
      totalStatistics: 0,
      paidStatistics: 0,
      promotionStatistics: 0,
      // 是否广电合作项目
      isBroadcastCooperate: false,

      contractProductPaymentKey: 1,
    };

    this.formRef = null;
    this.sharePaymentRef = null;
    this.useDepositRef = null;
  }

  componentDidMount() {
    this.init();
  }

  async init() {
    this.setState({ loading: true });
    await new Promise(this.loadData);
    this.setState({ loading: false });
  }

  handleReload = () => {
    return new Promise(this.loadData);
  };

  async loadData(resolve) {
    const res = await getContractByIds({ contractIds: this.props.contractIds }).catch(errorsHandle());
    if (res && res.data) {
      const list = invalidFilter(res.data || []);

      let totalAmount = 0;
      list.forEach((item) => {
        item.contractProductArr.forEach((prd) => {
          prd.amount = prd.pendingAmount;
          totalAmount = NP.plus(totalAmount, prd.pendingAmount);
        });
      });
      const total = this.calcTotalAmount(list);

      // 如果所有合同均已确认则不展示该按钮
      let isNeedConfirm = list.some((item) => [1, 2].includes(item.confirmStatus));
      // 多个合同时，如果存在精英班合同的，不可进行确认
      if (
        list.length > 1 &&
        list.some((item) => item.contractProductArr.some((subItem) => subItem.productType === 'ECS_CLASS'))
      ) {
        isNeedConfirm = false;
      }

      // 判断是否存在待分配资金不为0的合同
      const hasUnableAmount = this.judgeUnableAmount(list);

      // 获取分段收费相关设置
      const response = await getSegmentChargeItemByContractIds({ contractIds: this.props.contractIds }).catch(
        errorsHandle(),
      );
      if (response && response.data) {
        const tempContractIds = Object.keys(response.data);
        list.forEach((item) => {
          if (tempContractIds.indexOf(item.contractId) > -1) {
            item.segmentChargeItems = response['data'][item.contractId];
          }
        });
      }

      const isBroadcastCooperate = !!(list || []).filter(
        (contract) => !!(contract.contractTagsList || []).filter((tag) => tag.value == 1).length,
      ).length;
      this.setState(
        {
          list,
          isNeedConfirm,
          hasUnableAmount,
          isBroadcastCooperate,
          offsetAmount: total.transactionAmount,
          transactionAmount: total.transactionAmount,
          pendingAmount: total.pendingAmount,
          contractProductPaymentKey: this.state.contractProductPaymentKey + 1,
          ...this.calcStatistics(list),
        },
        () => {
          this.sharePaymentRef.refreshValue();
          this.setState({
            contractProductPaymentList: this.sharePaymentRef.getValue(),
          });
        },
      );
    }
    resolve();
  }

  calcStatistics = (list) => {
    let result = {
      totalStatistics: 0,
      paidStatistics: 0,
      promotionStatistics: 0,
    };

    list.forEach((contract) => {
      contract.contractProductArr.forEach((item) => {
        result.totalStatistics = NP.plus(result.totalStatistics, item.totalAmount);
        result.paidStatistics = NP.plus(result.paidStatistics, item.paidAmount);
        result.promotionStatistics = NP.plus(result.promotionStatistics, item.promotionAmount);
      });
    });

    return result;
  };

  onPaymentChange(list, totalAmount) {
    const total = this.calcTotalAmount(list);
    const contractProductPaymentList = this.sharePaymentRef.getValue();
    const hasUnableAmount = this.judgeUnableAmount(list);

    this.setState(
      {
        hasUnableAmount,
        list,
        contractProductPaymentList,
        offsetAmount: NP.minus(totalAmount, this.state.depositAmount),
        transactionAmount: total.transactionAmount,
        pendingAmount: total.pendingAmount,
        maxDepositVal: totalAmount,
      },
      () => {
        if (totalAmount === 0) {
          this.useDepositRef && this.useDepositRef.reset();
        }
      },
    );
  }

  calcTotalAmount(list) {
    let total = {
      transactionAmount: 0,
      pendingAmount: 0,
    };

    list.forEach((contractItem) => {
      (contractItem.contractProductArr || []).forEach((productItem) => {
        total.transactionAmount = NP.plus(total.transactionAmount, productItem.amount || 0);
        total.pendingAmount = NP.plus(total.pendingAmount, productItem.pendingAmount || 0);
      });
    });

    return total;
  }

  // 1、在支付时需要判断本次支付的合同里是否存在待分配资金不为0的合同，
  // 如果有则“保存收款信息”和“保存并确认合同”的按钮置灰，且在对应合同下方提示“本合同存在待分配资金，请分配后再进行支付”
  judgeUnableAmount = (list) => {
    return list.some((item) => item.availableAmount > 0);
  };

  async getParams() {
    if (!this.sharePaymentRef.validate()) {
      showMessage('本次收款金额不能为0！');
      return;
    }

    // 待优化 将校验和getParams 分开
    const getTransactionParams = this.formRef && this.formRef.getTransactionParams;
    const formParams = await getTransactionParams();
    const contractProductPaymentList = this.sharePaymentRef.getValue();
    if (!formParams) return;

    return {
      ...formParams,
      contractProductPaymentList,
      studentId: this.props.studentId,
      depositIds: this.state.depositSelected,
    };
  }

  // 保存收款信息
  async saveCollectionInformation() {
    const params = await this.getParams();
    if (!params) return;

    // 如果是二维码则已经支付所以无需重复保存
    if (channelMap[params.channel] !== QRCODE_PAY) {
      const res = await batchSaveFundOfContract(params);
    }

    // 必须先销毁iframe，防止支付中心会弹出alert
    this.formRef.destroyIframe();
    this.close();

    showMessage('保存成功！', 'success');
  }

  // 保存并确认
  async confirmAndPay() {
    const params = await this.getParams();
    if (!params) return;

    let res = {};
    if (channelMap[params.channel] === QRCODE_PAY) {
      res = await batchConfirmingContract({ contractIds: this.props.contractIds });
    } else {
      res = await batchSaveFundOfContractAndConfirm(params);
    }

    // 必须先销毁iframe，防止支付中心会弹出alert
    this.formRef.destroyIframe();
    runFunc(this.props.confirm, res.data, 'confirm');
  }

  async saveDeposit() {
    this.setState({ loading: true });

    let apiParams = {
      needConfirm: false,
      studentId: this.props.studentId,
      depositIds: this.state.depositSelected,
      transactionAmount: this.sharePaymentRef.getTotalAmount(),
      contractProductPaymentList: this.sharePaymentRef.getValue(),
    };

    const res = await batchDepositUsed(apiParams).catch(errorsHandle());

    if (res) {
      this.formRef.destroyIframe();
      this.close();
      showMessage('保存成功！', 'success');
    } else {
      this.setState({ loading: false });
    }
  }

  async saveDepositThenConfirm() {
    this.setState({ loading: true });

    let apiParams = {
      needConfirm: true,
      studentId: this.props.studentId,
      depositIds: this.state.depositSelected,
      transactionAmount: this.sharePaymentRef.getTotalAmount(),
      contractProductPaymentList: this.sharePaymentRef.getValue(),
    };

    const res = await batchDepositUsed(apiParams).catch(errorsHandle());

    if (res && res.data) {
      // 必须先销毁iframe，防止支付中心会弹出alert
      this.formRef.destroyIframe();
      runFunc(this.props.confirm, res.data.confirmBatchId, 'confirm');
    } else {
      this.setState({ loading: false });
    }
  }

  cancel() {
    const that = this;
    confirm({
      title: '',
      width: 520,
      content: (
        <div>
          <p style={{ textAlign: 'center', marginBottom: 0 }}>是否关闭窗口</p>
          <p>如已发起微信/支付宝收款，关闭窗口后收款码仍有效（共30分钟）</p>
        </div>
      ),
      onOk() {
        that.props.onCancel();
      },
      onCancel() {},
    });
  }

  close() {
    this.props.onCancel();
  }

  onDepositChange(depositSelected, depositAmount) {
    this.setState({
      offsetAmount: NP.minus(this.state.maxDepositVal, depositAmount),
      depositAmount,
      depositSelected,
    });
  }

  // 是否显示支付后确认按钮
  needShowConfirmAfterPayBtn() {
    const channel = channelMap[this.state.channel];
    return (
      !this.props.isMixing && this.state.isNeedConfirm && channel !== THIRD_PARTY_PAYMENT && channel !== EDU_PLATFORM
    );
  }

  render() {
    const { props, state } = this;
    const inputDisabled =
      channelMap[state.channel] === QRCODE_PAY &&
      (state.payStatus === payStatusMap.PENDING || state.payStatus === payStatusMap.SUCCESS);
    const isOffsetAmount = !!state.depositAmount && state.offsetAmount === 0;
    const popoverVisible =
      !!state.maxDepositVal &&
      this.useDepositRef &&
      this.useDepositRef.getDepositList().length > 0 &&
      state.depositAmount === 0;

    return (
      <Modal
        title="合同付款"
        widthSize="XL"
        loading={state.loading}
        onCancel={this.cancel}
        footer={
          <div className={styles.flexWrap}>
            <div className={styles.flexItemStart}>
              <Statistics
                totalAmount={state.totalStatistics}
                paidAmount={state.paidStatistics}
                promotionAmount={state.promotionStatistics}
                transactionAmount={state.transactionAmount}
                depositAmount={state.depositAmount}
                transactionVisible={false}
              />
            </div>

            <div className={styles.flexItemEnd}>
              <Button onClick={this.cancel}>取消</Button>

              <PopoverWrap
                visible={popoverVisible}
                content="该学生名下有未使用定金券，请确认本次支付是否要使用？"
                type="warning"
                overlayStyle={{ width: 220 }}
              >
                <Button
                  onClick={isOffsetAmount ? this.saveDeposit : this.saveCollectionInformation}
                  disabled={state.payStatus !== payStatusMap.SUCCESS}
                  type="primary"
                >
                  保存收款信息
                </Button>
              </PopoverWrap>

              {this.needShowConfirmAfterPayBtn() && (
                <PopoverWrap
                  visible={popoverVisible}
                  content="该学生名下有未使用定金券，请确认本次支付是否要使用？"
                  type="warning"
                  overlayStyle={{ width: 220 }}
                >
                  <Button
                    onClick={isOffsetAmount ? this.saveDepositThenConfirm : this.confirmAndPay}
                    disabled={state.payStatus !== payStatusMap.SUCCESS}
                    type="primary"
                  >
                    支付后确认信息
                  </Button>
                </PopoverWrap>
              )}
            </div>
          </div>
        }
      >
        <div className={styles.modalContent}>
          <div className={styles.contractList}>
            <ContractProductPayment
              list={state.list}
              disabled={inputDisabled}
              radioPartVisible={false}
              autoAssignVisible={true}
              onReload={this.handleReload}
              onChange={this.onPaymentChange}
              bodyClassName={styles.maxHeight}
              ref={(ref) => (this.sharePaymentRef = ref)}
              key={this.state.contractProductPaymentKey}
              hasPayAllBtn={(state.list || []).length === 1}
            />
            {props.isMixing && (
              <p className={styles.tip} style={{ marginTop: 12 }}>
                本次所选合同包含部分非常规合同，故仅能进行批量支付，无法进行“确认信息并支付”的操作。
              </p>
            )}
          </div>

          <div className={styles.contractFrom}>
            {props.useDeposit && (
              <div className={styles.useDepositWrap} style={{ display: inputDisabled ? 'none' : 'block' }}>
                <UseDeposit
                  studentId={props.studentId}
                  ref={(c) => (this.useDepositRef = c)}
                  amount={state.maxDepositVal}
                  onChange={this.onDepositChange}
                  direction="column"
                  labelSlot={<span className={styles.depositLabel}>已预付定金</span>}
                  disabledTips="收款金额必需大于0"
                />
              </div>
            )}

            {isOffsetAmount && (
              <p className={styles.tip}>
                本次使用定金券后，合同剩余待付金额为0，无须进行后续支付，请直接保存收款信息即可。
              </p>
            )}

            <div style={{ display: isOffsetAmount ? 'none' : 'block' }}>
              <ContractPaymentForm
                {...props}
                ref={(ref) => (this.formRef = ref)}
                contractId={props.contractIds}
                isBroadcastCooperate={state.isBroadcastCooperate}
                depositAmount={state.depositAmount}
                offsetAmount={state.offsetAmount}
                transactionAmount={state.transactionAmount}
                pendingAmount={state.pendingAmount}
                paidStatus={this.props.paidStatus}
                contractFundTypeName={this.props.contractFundTypeName}
                updatePayStatus={(payStatus) => this.setState({ payStatus })}
                updateChannel={(channel) => this.setState({ channel })}
                contractProductPaymentList={state.contractProductPaymentList}
                depositIds={this.state.depositSelected}
              />
            </div>
          </div>
        </div>
      </Modal>
    );
  }

  static propTypes = {
    contractIds: PropTypes.arrayOf(PropTypes.string).isRequired,
    blCampusId: PropTypes.string.isRequired,
    studentId: PropTypes.string.isRequired,
    onCancel: PropTypes.func.isRequired,
    confirm: PropTypes.func,
    isMixing: PropTypes.bool,
    useDeposit: PropTypes.bool,
  };

  static defaultProps = {
    isMixing: false,
    useDeposit: false,
  };
}

export default ContractPaymentModal;
