import React from 'react';
import { Icon } from 'antd';
import { List, Modal } from 'COMPONENT';

import * as narrowContractApi from 'SERVICE/narrowContract';

import { errorsHandle } from 'SRC/utils/common';
import showMessage from 'SRC/components/utils/globalTips/showMessage';

import style from './styles.module.less';

export default class NarrowContractModal extends React.Component {
  state = {
    list: [],
    listLoading: false,
    modalLoading: false,
    selectedRowKeys: [],
    confirmModalVisible: false,
  };

  constructor(props) {
    super(props);
    this.columns = [
      { title: '产品名称', dataIndex: 'productionName', key: 'productionName', width: 240 },
      { title: '小班名称', dataIndex: 'miniClassName', key: 'miniClassName', width: 260 },
      { title: '购买小时', dataIndex: 'natureQuantity', key: 'natureQuantity', width: 80 },
      { title: '单价', dataIndex: 'naturePrice', key: 'naturePrice', width: 80 },
      { title: '合同产品总金额', dataIndex: 'totalAmount', key: 'totalAmount', width: 120 },
      { title: '优惠金额', dataIndex: 'promotionAmount', key: 'promotionAmount', width: 80 },
      { title: '应收金额', dataIndex: 'realAmount', key: 'realAmount', width: 100 },
      { title: '已分配资金', dataIndex: 'paidAmount', key: 'paidAmount', width: 100 },
      {
        title: '缩单后总金额',
        dataIndex: 'narrowedNatureTotalAmount',
        key: 'narrowedNatureTotalAmount',
        width: 120,
        render: (text, record) => (record.canNarrow ? text : ''),
      },
      {
        title: '缩单后优惠金额',
        dataIndex: 'narrowedNaturePromotionAmount',
        key: 'narrowedNaturePromotionAmount',
        width: 120,
        render: (text, record) => (record.canNarrow ? text : ''),
      },
      {
        title: '缩单后应收金额',
        dataIndex: 'narrowedRealAmount',
        key: 'narrowedRealAmount',
        width: 120,
        render: (text, record) => (record.canNarrow ? text : ''),
      },
      {
        title: '缩单后购买小时',
        dataIndex: 'narrowedNatureQuantity',
        key: 'narrowedNatureQuantity',
        width: 120,
        render: (text, record) => (record.canNarrow ? text : ''),
      },
    ];
  }

  componentDidMount() {
    this.init();
  }

  get selectedProdNames() {
    return this.state.list
      .filter((item) => {
        return this.state.selectedRowKeys.includes(item.id);
      })
      .filter((item) => {
        return (item.productionName || '').trim();
      })
      .map((item) => {
        return item.productionName;
      })
      .join('、');
  }

  init() {
    this.loadData();
  }

  async loadData() {
    this.setState({ list: [], selectedRowKeys: [] });
    this.handleSetListLoading(true);

    const { contractId } = this.props.oldPageParams;
    const result = await narrowContractApi
      .getNarrowPreviewInfo({
        contractId,
      })
      .catch(errorsHandle());

    this.handleSetListLoading(false);
    if (!result || result.resultCode !== 0) {
      return;
    }

    const {
      data: { hasNoAuditFunds, hasNoArrangeFunds, contractProductVos },
    } = result;
    this.hasNoAuditFunds = hasNoAuditFunds;
    this.hasNoArrangeFunds = hasNoArrangeFunds;
    this.setState({
      list: (contractProductVos || []).map((item) => ({ ...item, key: item.id })),
    });
  }

  handleSubmitMainModal = () => {
    if (!this.state.selectedRowKeys.length) {
      return showMessage('请先选择需要缩单的合同产品', 'warning');
    }
    if (this.hasNoAuditFunds) {
      return showMessage('合同名下存在未审核收款，请联系财务审核通过后再进行缩单', 'warning');
    }
    if (this.hasNoArrangeFunds) {
      return showMessage('合同名下存在待分配资金，请先前往合同列表完成资金分配后再操作缩单', 'warning');
    }
    this.setState({ confirmModalVisible: true });
  };

  handleCancelMainModal = () => {
    this.props.onClose();
    GlobalEvent.emit('REFRESH_CONTRACT_LIST'); // 刷新合同列表页
  };

  handleSetListLoading = (listLoading) => {
    this.setState({ listLoading });
  };

  handleSetModalLoading = (modalLoading) => {
    this.setState({ modalLoading });
  };

  handleCancelConfirmModal = () => {
    this.setState({ confirmModalVisible: false });
  };

  handleSubmitConfirmModal = async () => {
    this.handleSetModalLoading(true);
    const { contractId } = this.props.oldPageParams;
    const result = await narrowContractApi
      .narrowContractProduct({
        contractId,
        contractProductIds: this.state.selectedRowKeys,
      })
      .catch(errorsHandle());

    this.handleSetModalLoading(false);
    if (!result || result.resultCode !== 0) {
      return;
    }

    showMessage('合同产品缩单成功', 'success');
    // 操作成功关闭弹框
    this.setState(
      {
        confirmModalVisible: false,
      },
      () => {
        this.handleCancelMainModal();
      },
    );
  };

  // 主体弹框
  generateMainModal() {
    const props = {
      widthSize: 'xl',
      okText: '点击缩单',
      title: '合同产品缩单',
      visible: this.props.visible,
      onOk: this.handleSubmitMainModal,
      onCancel: this.handleCancelMainModal,
    };

    return (
      <Modal {...props}>
        {this.generateList()}
        {this.generateTips()}
      </Modal>
    );
  }

  // 二次确认弹框
  generateConfirmModal() {
    const props = {
      widthSize: 's',
      okText: '确认缩单',
      title: '缩单产品确认',
      loading: this.state.modalLoading,
      onOk: this.handleSubmitConfirmModal,
      onCancel: this.handleCancelConfirmModal,
    };

    return (
      this.state.confirmModalVisible && (
        <Modal {...props}>
          <div className={style.confirmTextWrapper}>
            <Icon type="exclamation-circle" style={{ color: '#ffbf00', fontSize: 22, marginRight: 5 }} />
            请问是否确认对合同<strong>{this.props.oldPageParams.contractId}</strong>里的
            <strong>{this.selectedProdNames}</strong>进行缩单操作，缩单后合同金额将相应变更，该操作不可逆，请谨慎操作。
          </div>
        </Modal>
      )
    );
  }

  // 列表
  generateList() {
    const props = {
      columns: this.columns,
      dataSource: this.state.list,
      loading: this.state.listLoading,
      rowSelection: {
        onChange: (selectedRowKeys) => {
          this.setState({ selectedRowKeys });
        },
        getCheckboxProps: (record) => {
          return {
            disabled: !record.canNarrow,
          };
        },
      },
    };
    return (
      <div className={style.listWrapper}>
        <List {...props} />
      </div>
    );
  }

  generateTips() {
    return (
      <div className={style.tipWrapper}>
        <p>注：</p>
        <ul>
          <li>1、缩单后合同产品只能按原价上课，不再享受优惠，故缩单后学生可上小时数为（已分配资金/合同产品单价）</li>
          <li>2、该合同名下所有收款需全部审核通过才可进行缩单操作 </li>
          <li>3、缩单后该合同产品的资金不可提取</li>
          <li>4、缩单后可能购买小时出现小数，建议将小数点后的小时退回电子账户</li>
          <li>5、缩单操作不可逆，请谨慎操作</li>
        </ul>
      </div>
    );
  }

  render() {
    return (
      <div>
        {this.generateMainModal()}
        {this.generateConfirmModal()}
      </div>
    );
  }
}
