import React from 'react';
import UploadFileModal from './UploadFileModal';
import SelectApprovalModal from './SelectApprovalModal';
import {
  getLastOfflineContractConfirmRecord,
  submitContractConfirmAudit,
  getContractConfirmAuditUserMsg,
} from 'SERVICE/contract';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { runFunc } from 'SRC/utils/common';
import styles from './styles.module.less';

class OfflineContractConfirm extends React.Component {
  constructor(props) {
    super(props);
    this.handleUploadFileConfirm = this.handleUploadFileConfirm.bind(this);
    this.handleSelectApprovalConfirm = this.handleSelectApprovalConfirm.bind(this);
    this.getLastOfflineContractConfirmRecord = this.getLastOfflineContractConfirmRecord.bind(this);
    this.handleSelectUserChange = this.handleSelectUserChange.bind(this);
    this.handleFileChange = this.handleFileChange.bind(this);
    this.close = this.close.bind(this);

    this.state = {
      contractFiles: [],
      nodeId: null,
      selectedUser: null,
      approvalUsers: [],
      uploadFileModalVisible: true,
      selectApprovalModalVisible: false,
    };
  }

  componentDidMount() {
    this.getLastOfflineContractConfirmRecord();
  }

  // 获取附件
  async getLastOfflineContractConfirmRecord() {
    const res = await getLastOfflineContractConfirmRecord({ contractId: this.props.contractId });
    if (!res.data) return;

    const { fileList } = res.data;
    if (fileList && fileList.length) {
      let contractFiles = fileList.filter((r) => !!r.fileFullPath).map((r) => ({ ...r, src: r.fileFullPath }));
      this.setState({ contractFiles });
    }
  }

  // 附件合同确认, 审批人只有一个的时候直接提交操作成功
  async handleUploadFileConfirm() {
    const fileLength = this.state.contractFiles.length;
    if (fileLength < 2) {
      return showMessage('请上传2张合同照片！', 'warning');
    }
    const res = await getContractConfirmAuditUserMsg({
      contractId: this.props.contractId,
    });
    const { approvalUsers, nodeId } = res.data;

    // 只有一个审核人的时候 默认选中
    if (approvalUsers.length === 1) {
      this.setState({ approvalUsers, nodeId, selectApprovalModalVisible: true, selectedUser: approvalUsers[0] });
    } else {
      this.setState({ approvalUsers, nodeId, selectApprovalModalVisible: true });
    }
  }

  async handleSelectApprovalConfirm() {
    try {
      const fileList = this.state.contractFiles.map((r) => {
        let fileName;
        if (r.fileName) {
          fileName = r.fileName.substr(0, r.fileName.lastIndexOf('.'));
        }
        return {
          filePath: r.fileKey || r.filePath,
          fileSize: r.size || r.fileSize,
          realFileName: fileName || r.realFileName,
          type: 101, // 文件类型 101:合同正面 102:合同反面
        };
      });
      await submitContractConfirmAudit({
        contractId: this.props.contractId,
        fileList,
        nodeId: this.state.nodeId,
        auditUserId: this.state.selectedUser && this.state.selectedUser.userId,
      });
      showMessage('操作成功！', 'success');
      this.close();
    } catch (e) {
      showMessage(`${e.text || '操作失败！'}`, 'error');
    }
  }

  handleFileChange(contractFiles) {
    this.setState({ contractFiles });
  }

  handleSelectUserChange(value) {
    const selectedUser = this.state.approvalUsers.find((item) => item.userId === value);
    this.setState({ selectedUser });
  }

  close() {
    runFunc(this.props.onCancel);
  }

  render() {
    const { uploadFileModalVisible, selectApprovalModalVisible } = this.state;

    return (
      <div className={styles.offlineContractConfirm}>
        {uploadFileModalVisible && (
          <UploadFileModal
            onConfirm={this.handleUploadFileConfirm}
            onFileChange={this.handleFileChange}
            onCancel={this.close}
            contractFiles={this.state.contractFiles}
          />
        )}
        {selectApprovalModalVisible && (
          <SelectApprovalModal
            approvalUsers={this.state.approvalUsers}
            selectedUser={this.state.selectedUser}
            onChange={this.handleSelectUserChange}
            onCancel={() => this.setState({ selectApprovalModalVisible: false })}
            onConfirm={this.handleSelectApprovalConfirm}
          />
        )}
      </div>
    );
  }
}

export default OfflineContractConfirm;
