import React from 'react';
import PropTypes from 'prop-types';
import { Modal, List, TextLink, QrCode } from 'COMPONENT';
import { Popover } from 'antd';
import { getBatchSharePaymentRecordByContractId } from 'SERVICE/contract';
import { errorsHandle, loginUser } from 'UTIL/common';
import { h5ContractComfirmUrl } from 'CONST/otherSystemConfig';

const setWidth = List.createSetWidth();

class ModalPaidOrder extends React.Component {
  constructor(props) {
    super(props);

    this.init = this.init.bind(this);
    this.loadData = this.loadData.bind(this);

    this.state = {
      loading: false,
      list: [],
    };

    this.baseUrl = `${h5ContractComfirmUrl}/${loginUser.isXingHuoSym() ? 'xh' : 'ss'}`;

    this.columns = [
      {
        title: '序号',
        dataIndex: '_number',
        key: '_number',
        width: setWidth(2),
        render: (text, record, index) => index + 1,
      },
      { title: '收款人', dataIndex: 'creatUserName', key: 'creatUserName', width: setWidth(4) },
      { title: '支付时间', dataIndex: 'paidTime', key: 'paidTime', width: setWidth(8) },
      { title: '支付金额', dataIndex: 'amount', key: 'amount', width: setWidth(4) },
      {
        title: '操作',
        dataIndex: 'batchPayId',
        key: 'batchPayId',
        width: setWidth(2),
        render: (text, record, index) => (
          <div>
            {text && (
              <Popover
                content={
                  <QrCode
                    id={`payQrCode-${index}`}
                    text={`${this.baseUrl}/${text}/order/list?mode=onlyPay`}
                    width={120}
                    height={120}
                  />
                }
                trigger="click"
              >
                <TextLink>详情</TextLink>
              </Popover>
            )}
          </div>
        ),
      },
    ];
  }

  static propTypes = {
    contractId: PropTypes.string.isRequired,
    onCancel: PropTypes.func.isRequired,
  };

  componentDidMount() {
    this.init();
  }

  async init() {
    this.setState({ loading: true });
    await new Promise(this.loadData);
    this.setState({ loading: false });
  }

  async loadData(resolve) {
    const apiParams = {
      contractId: this.props.contractId,
      paidStatus: 'PAID',
    };

    const res = await getBatchSharePaymentRecordByContractId(apiParams).catch(errorsHandle());

    if (res && res.resultCode === 0) {
      this.setState({ list: res.data || [] }, resolve);
    } else {
      resolve();
    }
  }

  render() {
    return (
      <Modal
        title="支付完成订单"
        loading={this.state.loading}
        onCancel={this.props.onCancel}
        okBtnShow={false}
        widthSize="M"
      >
        <List dataSource={this.state.list} columns={this.columns} />
      </Modal>
    );
  }
}

export default ModalPaidOrder;
