import React from 'react';
import PropTypes from 'prop-types';
import ModalSelectPayType from '../SelectPayType';
import ContractPaymentModal from '../ContractPaymentModal';
import ModalSharePayment from '../SharePayment';
import ConfirmContractInformationModal from '../ConfirmContractInformationModal';
import { RECEIVE, SHARE } from '../../signContractPro/config';
import SegmentDiscount from '../SegmentDiscount'; // 分段收费设置确认
import { runFunc } from 'SRC/utils/common';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { Loading } from 'COMPONENT';

/**
 * payCode
 * */
const SELECT_PAY_TYPE = 'SELECT_PAY_TYPE';
const OFF_LINE_PAYMENT = 'OFF_LINE_PAYMENT';
const SHARE_PAYMENT = 'SHARE_PAYMENT';

class Pay extends React.Component {
  constructor(props) {
    super(props);
    this.init = this.init.bind(this);
    this.initModal = this.initModal.bind(this);
    this.handleSwitchPayType = this.handleSwitchPayType.bind(this);
    this.close = this.close.bind(this);
    this.handlePayment = this.handlePayment.bind(this);
    this.closePayment = this.closePayment.bind(this);
    this.closeContractInfo = this.closeContractInfo.bind(this);

    this.state = {
      loading: false,
      selectPayTypeVisible: false,
      receivePaymentVisible: false,
      sharePaymentVisible: false,
      contractInformationModalVisible: false,
      modalTitle: '确认信息',
      radioMessage: true,
    };

    this.isMixing = props.isEcs && props.contractIds.length > 1;
    this.isConfirmH5 = !this.isMixing;

    this.id = null;
    this.type = null;
  }

  static propTypes = {
    onCancel: PropTypes.func.isRequired,
    contractIds: PropTypes.arrayOf(PropTypes.string).isRequired,
    campusId: PropTypes.string.isRequired,
    studentId: PropTypes.string.isRequired,
    campusName: PropTypes.string.isRequired,
    customerName: PropTypes.string.isRequired,
    studentName: PropTypes.string.isRequired,

    contractNumber: PropTypes.oneOfType([PropTypes.string, PropTypes.number]).isRequired,
    signTime: PropTypes.string,
    payCode: PropTypes.string,
    isEcs: PropTypes.bool,
    useDeposit: PropTypes.bool,
  };

  static defaultProps = {
    payCode: SELECT_PAY_TYPE,
    isEcs: false,
    useDeposit: true,
  };

  componentDidMount() {
    // this.init();
  }

  async init() {
    this.setState({ loading: true });
    this.initModal();
    this.setState({ loading: false });
  }

  initModal() {
    this.setState((prevState) => {
      switch (this.props.payCode) {
        case OFF_LINE_PAYMENT:
          prevState.receivePaymentVisible = true;
          break;
        case SHARE_PAYMENT:
          prevState.sharePaymentVisible = true;
          prevState.modalTitle = '分享支付';
          break;
        default:
          prevState.selectPayTypeVisible = true;
      }
      return prevState;
    });
  }

  handleSwitchPayType(type) {
    let payload = { selectPayTypeVisible: false };

    if (type === RECEIVE) {
      payload.receivePaymentVisible = true;
      payload.modalTitle = '确认信息';
    } else if (type === SHARE) {
      if (this.props.contractFundTypeName === '教育平台') {
        return showMessage('教育平台收款合同不能使用分享支付', 'error');
      }
      payload.sharePaymentVisible = true;
      payload.modalTitle = '分享支付';
    }

    this.setState(payload);
  }

  close() {
    runFunc(this.props.onCancel);
  }

  handlePayment(id, type) {
    this.id = id;
    this.type = ['pay', 'confirm'].includes(type) ? type : 'confirmPay';
    this.setState({
      receivePaymentVisible: false,
      sharePaymentVisible: false,
      contractInformationModalVisible: true,
      radioMessage: this.type === 'confirm',
    });
  }

  closePayment() {
    this.setState(
      {
        receivePaymentVisible: false,
        sharePaymentVisible: false,
      },
      () => {
        this.close();
        GlobalEvent.emit('REFRESH_CONTRACT_LIST');
      },
    );
  }

  closeContractInfo() {
    this.setState({ contractInformationModalVisible: false }, () => {
      this.close();
      GlobalEvent.emit('REFRESH_CONTRACT_LIST');
    });
  }

  render() {
    return (
      <div>
        <Loading loading={this.state.loading} />
        {/* 分段收费优惠设置 */}
        <SegmentDiscount contractIds={this.props.contractIds} onSuccess={this.init} onCancel={this.closeContractInfo} />

        {/* 选择支付方式 */}
        {this.state.selectPayTypeVisible && (
          <ModalSelectPayType
            campusId={this.props.campusId}
            onCancel={this.close}
            contractFundTypeName={this.props.contractFundTypeName}
            switchPayType={this.handleSwitchPayType}
          />
        )}

        {/* 录入线下收款 */}
        {this.state.receivePaymentVisible && (
          <ContractPaymentModal
            isMixing={this.isMixing}
            contractIds={this.props.contractIds}
            blCampusId={this.props.campusId}
            studentId={this.props.studentId}
            useDeposit={this.props.useDeposit}
            onCancel={this.closePayment}
            confirm={this.handlePayment}
            paidStatus={this.props.paidStatus}
            contractFundTypeName={this.props.contractFundTypeName}
          />
        )}

        {/* 分享支付 */}
        {this.state.sharePaymentVisible && (
          <ModalSharePayment
            isConfirmH5={this.isConfirmH5}
            isMixing={this.isMixing}
            contractIds={this.props.contractIds}
            studentId={this.props.studentId}
            useDeposit={this.props.useDeposit}
            onCancel={this.closePayment}
            onSuccess={this.handlePayment}
          />
        )}

        {/* 分享H5链接 */}
        {this.state.contractInformationModalVisible && (
          <ConfirmContractInformationModal
            title={this.state.modalTitle}
            contractInfo={{
              id: this.id,
              type: this.type,
              contractIds: this.props.contractIds,
              campusName: this.props.campusName,
              contractNumber: this.props.contractNumber,
              customerName: this.props.customerName,
              studentName: this.props.studentName,
              signTime: this.props.signTime,
              isEcs: this.props.isEcs,
            }}
            onCancel={this.closeContractInfo}
            radioMessage={this.state.radioMessage}
          />
        )}
      </div>
    );
  }
}

export default Pay;
