import React from 'react';
import PropTypes from 'prop-types';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import ModalConfirmPayerInfo from './ModalConfirmPayerInfo';
import ModalOrintSetting from './ModalPrintSetting';
import ModalGuidePrice from './ModalGuidePrice';
import * as loaders from './loader';
import { CONFIRM_STATUS, CONFIRM_CHANNEL } from './constant';
import { CATEGORY_B } from 'CONST/dicts/product';

/**
 * 打印合同处理逻辑弹窗
 *
 * @param props.contractId {string} 合同ID
 * @param props.blCampusId {string} 合同学习中心ID
 */
class Index extends React.Component {
  static PropTypes = {
    contractId: PropTypes.string.isRequired,
    blCampusId: PropTypes.string.isRequired,
    ecsContract: PropTypes.bool,
    onClose: PropTypes.func.isRequired,
  };

  static defaultProps = {
    contractId: null,
    blCampusId: null,
    ecsContract: false,
    onClose() {},
  };

  constructor(props) {
    super(props);

    this.state = {
      modalConfirmPayerInfoShow: false, // 显示确认付款人信息弹窗
      modalSelectTemplateShow: false, // 显示选择合同模版弹窗
      modalPrintSettingShow: false, // 显示打印设置
      templateCode: null, // 合同模版code
      switchFlagResult: null, // 开关信息
      contractInfo: null, // 合同打印信息
      printSettingResult: null, // 合同确认信息
      modalGuidePriceVisible: false,
    };

    this.hasNatureGuidePrice = false;
    this.contactId = props;
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化
   * 1、该合同是否线上已完成确认
   * 2、判断合同学习中心是否配置合同模版
   * 3、判断是否配置确认合同开关
   * 4、判断是否已经确认合同
   * 5、判断是否配置选择国家模版开关
   *
   * @return {Promise<void>}
   */
  initContent = async () => {
    const { contractId, blCampusId } = this.props;

    if (!contractId || !blCampusId) {
      showMessage('缺少参数', 'error');
      this.props.onClose();
      return;
    }

    const [templateData, switchFlagResult, printSettingResult, isHadGuideNaturePrice] = await Promise.all([
      loaders.loadContractTemplateCode(contractId),
      loaders.loadSwitchConfigFlag(blCampusId),
      loaders.loadConfirmInfoByContractId(contractId),
      loaders.loadIsContractHadGuideNaturePrice(contractId),
    ]);

    this.hasNatureGuidePrice = isHadGuideNaturePrice;

    const { templateCode, contractPayCategory } = templateData;

    this.setState({ templateCode, switchFlagResult, printSettingResult });
    // const { isConfirmContract } = switchFlagResult;
    if (!printSettingResult) {
      showMessage('抱歉，该合同暂无合同确认信息', 'error');
      this.props.onClose();
      return;
    }

    // 判断是否配置模版
    if (!templateCode) {
      showMessage('抱歉，您所在的学习中心暂无合同模板', 'error');
      this.props.onClose();
      return;
    }

    if (contractPayCategory === CATEGORY_B) {
      this.openServiceBook(contractId, templateCode);
      this.props.onClose();
      return;
    } else if (isHadGuideNaturePrice)
      if (
        printSettingResult.confirmStatus === CONFIRM_STATUS.CONFIRMED &&
        printSettingResult.confirmChannel === CONFIRM_CHANNEL.ON_LINE &&
        printSettingResult.electronicContractRecordId &&
        !this.props.ecsContract
      ) {
        // 是否已经确认线上合同(非精英班)
        if (printSettingResult.fullElectronicContractAddress) {
          let newWindow = window.open('about:blank');

          // newWindow.location.href = pdfViewerHref.replace('${URL}', encodeURIComponent(printSettingResult.fullElectronicContractAddress));
          newWindow.location.href = `${location.origin}/printFile?url=${encodeURIComponent(
            printSettingResult.fullElectronicContractAddress,
          )}`;
        } else {
          showMessage('抱歉，暂无合同pdf链接', 'error');
        }
        this.props.onClose();

        return;
      }

    // 直接打印
    this.onConfirmPayerInfoOk();
  };

  /**
   * 确认收款人信息OK
   */
  onConfirmPayerInfoOk = () => {
    const { isSystemPrintCode } = this.state.switchFlagResult;
    if (isSystemPrintCode) {
      this.setState({ modalPrintSettingShow: true });
    } else {
      this.choosePrintGuidePriceContract(this.state.templateCode);
    }
  };

  /**
   * 关闭确认收款人信息弹窗
   */
  onConfirmPayerInfoClose = () => {
    this.toggleModalConfirmPayerInfo();
    this.props.onClose();
  };

  /**
   * 显示确认付款人信息弹窗
   */
  toggleModalConfirmPayerInfo = () => {
    this.setState((prevState) => {
      return {
        modalConfirmPayerInfoShow: !prevState.modalConfirmPayerInfoShow,
      };
    });
  };

  choosePrintGuidePriceContract = (templateCode, isAllPage = 0) => {
    if (this.hasNatureGuidePrice) {
      this.setState({ modalGuidePriceVisible: true });
    } else {
      this.openPrintPage(templateCode, isAllPage, 0);
      this.props.onClose();
    }
  };

  /**
   * 打开打印页面
   *
   * @param templateCode {string} 合同模版code
   * @param isAllPage=0 {string} 是否全页打印, 0是否，1是真
   * @param guideType {string|number} 1课程费合同，2服务费合同
   */
  openPrintPage = (templateCode, isAllPage = 0, guideType) => {
    const { contractId } = this.props;
    const path = `/print/contract/${contractId}/${templateCode}?tempType=1&isAllPage=${isAllPage}&guideType=${guideType}`;

    let newWindow = window.parent.open('about:blank');
    newWindow.location.href = path;
  };

  // 打印设置页面点击打印
  printFunc = (setting) => {
    this.setState({ modalPrintSettingShow: false });
    this.choosePrintGuidePriceContract(this.state.templateCode, setting);
  };

  // 关闭打印设置页面
  closePrintSetting = () => {
    this.setState({ modalPrintSettingShow: false });
    this.props.onClose();
  };

  // (B类合同)服务书
  openServiceBook = (contractId, templateCode) => {
    const path = `/print/servicebook/${contractId}/${templateCode}`;
    let newWindow = window.parent.open('about:blank');
    newWindow.location.href = path;
  };

  render() {
    const { contractInfo, templateCode, modalGuidePriceVisible } = this.state;
    const { contractId } = this.props;

    return (
      <div>
        {this.state.modalConfirmPayerInfoShow && contractInfo && (
          <ModalConfirmPayerInfo
            contractId={this.props.contractId}
            payerName={contractInfo.confirmCustomerName || contractInfo.payerName}
            payerPhone={contractInfo.confirmCustomerContact || contractInfo.payerPhone}
            payerIdCardNo={contractInfo.payerIdCardNo}
            customerIdCardType={contractInfo.customerIdCardType}
            customerConfirmStatus={contractInfo.customerConfirmStatus}
            onOk={this.onConfirmPayerInfoOk}
            onCancel={this.onConfirmPayerInfoClose}
          />
        )}
        {this.state.modalPrintSettingShow && (
          <ModalOrintSetting onOk={this.printFunc} onCancel={this.closePrintSetting} />
        )}

        {modalGuidePriceVisible && (
          <ModalGuidePrice contractId={contractId} templateCode={templateCode} onClose={this.props.onClose} />
        )}
      </div>
    );
  }
}

export default Index;
