import {
  getTemplateCodeNew,
  getCampusContractPrint,
  getConfirmInfoByContractId,
  getContractPrintFlagForGuideNaturePrice,
} from 'SERVICE/print';
import { getSwitchConfigsByCodesAndCampusIds } from 'SERVICE/common';

/**
 * 根据合同ID获取合同模版code
 *
 * @param contractId {string} 合同ID
 * @return {Promise<*>}
 */
export async function loadContractTemplateCode(contractId) {
  try {
    const response = await getTemplateCodeNew({ contractId, flag: 1 });
    const { data } = response || {};

    return data || {};
  } catch (e) {
    return {};
  }
}

export async function loadIsContractHadGuideNaturePrice(contractId) {
  try {
    const response = await getContractPrintFlagForGuideNaturePrice({ contractId, flag: 1 });
    return response.data.hasNatureGuidePrice || false;
  } catch (e) {
    return {};
  }
}

/**
 * 获取是否配置确认合同开关
 * 获取是否配置选择合同模版开关
 *
 * @param blCampusId {string} 合同学习中心
 * @return {Promise<{}|{isConfirmContract: boolean, isSelectContractTemplate: boolean}>}
 */
export async function loadSwitchConfigFlag(blCampusId) {
  try {
    const selectContractTemplateCode = 'SC_CONTRACT_PRINT_COUNTRY';
    const systemPrintCode = 'SC_CONTRACT_PRINT_GROUP_PAGE';
    const tempCodes = [selectContractTemplateCode, systemPrintCode];
    const codes = tempCodes.join(',');
    const params = { codes, campusIds: blCampusId, flag: true };
    const response = await getSwitchConfigsByCodesAndCampusIds(params);
    const data = response ? response.data : [];

    let isConfirmContract = true;
    let isSelectContractTemplate = false;
    let isSystemPrintCode = false;

    data.forEach((item) => {
      const { itemCode, switchFlag } = item;

      if (itemCode === selectContractTemplateCode && switchFlag === true) {
        isSelectContractTemplate = true;
      }

      if (itemCode === systemPrintCode && switchFlag === true) {
        isSystemPrintCode = true;
      }
    });

    return {
      isConfirmContract,
      isSelectContractTemplate,
      isSystemPrintCode,
    };
  } catch (e) {
    return {
      isConfirmContract: true,
      isSelectContractTemplate: false,
      isSystemPrintCode: false,
    };
  }
}

/**
 * 获取打印合同信息
 *
 * @param contractId {string} 合同ID
 * @param templateCode {string} 合同模版code
 * @return {Promise<>}
 */
export async function loadContractPrintInfo(contractId, templateCode) {
  try {
    const response = await getCampusContractPrint({ contractId, templateCode });
    const { data } = response || {};

    return data || null;
  } catch (e) {
    return null;
  }
}

// 目前只有星火使用，根据合同id获取合同信息
export async function loadConfirmInfoByContractId(contractId) {
  try {
    const response = await getConfirmInfoByContractId({ contractId });
    const { data } = response || {};
    return data || null;
  } catch (e) {
    return null;
  }
}
