import React from 'react';
import { Modal, List, InputNumber } from 'COMPONENT';
import { message } from 'antd';
import PropTypes from 'prop-types';
import { runFunc } from 'SRC/utils/common';
import * as np from 'number-precision';
import { getContractSegmentChargeInfo, updateContractSegmentChargeConfigSnapshot } from 'SERVICE/contract';

class SegmentDiscount extends React.Component {
  static propTypes = {
    onCancel: PropTypes.func,
    onSuccess: PropTypes.func.isRequired,
    contractIds: PropTypes.arrayOf(PropTypes.string).isRequired,
  };
  static defaultProps = {
    chargeInfo: null,
  };
  constructor(props) {
    super(props);
    this.saveConfirm = this.saveConfirm.bind(this);
    this.changePromotionAmount = this.changePromotionAmount.bind(this);

    this.state = {
      segmentVisible: false,
      dataList: [],
      segmentInfo: null,
      unChargeCount: 0, // 为确认合同数
      promotionAmountList: [{ upPromotionAmount1: 0 }],
    };
    this.columns = [
      { title: '班级名称', width: 135, dataIndex: 'miniClassName' },
      { title: '产品金额', width: 100, dataIndex: 'totalAmount' },
      { title: '产品总优惠金额', width: 80, dataIndex: 'promotionAmount' },
      {
        title: '阶段1收费明细',
        key: 'promotion1',
        children: [
          { title: '产品金额', width: 80, dataIndex: 'segFirstTotalAmount' },
          { title: '优惠上限', width: 80, key: 'upPromotionAmountLimit', dataIndex: 'upPromotionAmountLimit' },
          {
            title: '优惠金额',
            width: 100,
            key: 'upPromotionAmount1',
            render: (text, row) => {
              return (
                <InputNumber
                  /** 阶段二产品金额大于0，为避免阶段二应收金额小于0，控制阶段一优惠下限 **/
                  min={
                    row.segSecondTotalAmount > 0 && row.promotionAmount - row.segSecondTotalAmount > 0
                      ? row.promotionAmount - row.segSecondTotalAmount
                      : 0
                  }
                  max={
                    row.segSecondTotalAmount > 0 &&
                    row.promotionAmount - row.segSecondTotalAmount > 0 &&
                    row.upPromotionAmountLimit < row.promotionAmount - row.segSecondTotalAmount
                      ? row.promotionAmount - row.segSecondTotalAmount
                      : row.upPromotionAmountLimit
                  }
                  defaultValue={row.upPromotionAmountLimit1 || 0}
                  precision={2}
                  onChange={(v) => {
                    this.changePromotionAmount(v, row.snapshotId);
                  }}
                  disabled={!row.canModify}
                />
              );
            },
          },
          {
            title: '应收金额',
            width: 80,
            key: 'segFirstRealAmount',
            render: (amount, row) => {
              const index = this.state.promotionAmountList.findIndex((item) => item.innerSnapshotId == row.snapshotId);
              const num = np.minus(
                row.upPromotionAmountLimit,
                this.state.promotionAmountList[index]['upPromotionAmount1'],
              ); // 新优惠与上限的偏差
              return np.plus(row.segFirstRealAmount, num);
            },
          },
        ],
      },
      {
        title: '阶段2收费明细',
        key: 'promotion2',
        children: [
          { title: '产品金额', width: 80, dataIndex: 'segSecondTotalAmount' },
          {
            title: '优惠金额',
            width: 80,
            key: 'segSecondPromotionAmount',
            render: (sAmount, row) => {
              const index = this.state.promotionAmountList.findIndex((item) => item.innerSnapshotId == row.snapshotId);
              return np.minus(row.promotionAmount, this.state.promotionAmountList[index]['upPromotionAmount1']);
            },
          },
          {
            title: '应收金额',
            width: 80,
            key: 'segSecondRealAmount',
            render: (sRealAmount, row) => {
              const index = this.state.promotionAmountList.findIndex((item) => item.innerSnapshotId == row.snapshotId);
              const newPromotion = np.minus(
                row.promotionAmount,
                this.state.promotionAmountList[index]['upPromotionAmount1'],
              ); // 阶段二优惠金额
              if (row.segSecondRealAmount <= 0 || np.minus(row.segSecondTotalAmount, newPromotion) <= 0)
                return '阶段2无需收款';
              return np.minus(row.segSecondTotalAmount, newPromotion);
            },
          },
        ],
      },
    ];
  }

  async componentDidMount() {
    const { promotionAmountList } = this.state;
    let unChargeCount = 0;
    const { resultCode, data } = await getContractSegmentChargeInfo({ contractIds: this.props.contractIds });
    let show = false;
    const showData = [];
    if (resultCode == 0 && data) {
      // 设置了分段收费 判断是否存在为确认 confirmStatus == 2 表示已经确认过
      promotionAmountList.splice(0, 1);
      data.forEach((item) => {
        if (item.confirmStatus != 2) {
          show = true;
          unChargeCount += 1;
          item.contractProductSegmentChargeInfoVos.forEach((i, index) => {
            if (
              i.segSecondTotalAmount > 0 &&
              i.promotionAmount - i.segSecondTotalAmount > 0 &&
              i.upPromotionAmountLimit < i.promotionAmount - i.segSecondTotalAmount
            ) {
              i.upPromotionAmountLimit1 = np.minus(i.promotionAmount, i.segSecondTotalAmount);
            } else {
              i.upPromotionAmountLimit1 = i.upPromotionAmountLimit;
            }
            promotionAmountList.push({
              upPromotionAmount1: i.upPromotionAmountLimit1,
              snapshotId: item.snapshotId, // 合同快照
              innerSnapshotId: i.snapshotId, // 合同产品分段收费快照id
            });
          });
          showData.push(item);
        }
      });
      this.setState({
        segmentVisible: show,
        segmentInfo: showData,
        unChargeCount,
        promotionAmountList,
      });
      if (!show) {
        // 不存在未确认的，跳过
        runFunc(this.props.onSuccess);
      }
    } else {
      runFunc(this.props.onSuccess);
    }
  }

  changePromotionAmount(amount, snapshotId) {
    const { promotionAmountList } = this.state;
    const index = promotionAmountList.findIndex((item) => item.innerSnapshotId == snapshotId);
    promotionAmountList[index]['upPromotionAmount1'] = amount;
    this.setState({
      promotionAmountList,
    });
  }

  async saveConfirm() {
    const { promotionAmountList } = this.state;
    const promotionData = {},
      params = [];
    let status = true;
    promotionAmountList.forEach((item) => {
      if (item.upPromotionAmount1 === undefined) {
        status = false;
      }
      if (promotionData[item.snapshotId]) {
        promotionData[item.snapshotId]['cpSegmentSnapshotUpdateDtos']
          ? promotionData[item.snapshotId]['cpSegmentSnapshotUpdateDtos'].push({
              snapshotId: item.innerSnapshotId,
              upPromotionAmountLimit: item.upPromotionAmount1,
            })
          : (promotionData[item.snapshotId]['cpSegmentSnapshotUpdateDtos'] = [
              { snapshotId: item.innerSnapshotId, upPromotionAmountLimit: item.upPromotionAmount1 },
            ]);
      } else {
        promotionData[item.snapshotId] = {
          cpSegmentSnapshotUpdateDtos: [
            { snapshotId: item.innerSnapshotId, upPromotionAmountLimit: item.upPromotionAmount1 },
          ],
        };
      }
    });
    Object.keys(promotionData).forEach((snapshotId) => {
      const temp = {};
      temp.snapshotId = parseInt(snapshotId);
      temp.cpSegmentSnapshotUpdateDtos = promotionData[snapshotId]['cpSegmentSnapshotUpdateDtos'];
      params.push(temp);
    });
    if (!status) {
      return message.error('请填写优惠金额！');
    }
    const { resultCode } = await updateContractSegmentChargeConfigSnapshot(params);
    if (resultCode == 0) {
      message.success('保存成功！');
      this.setState({
        segmentVisible: false,
      });
      runFunc(this.props.onSuccess);
    }
  }

  render() {
    const { onCancel } = this.props;
    const { segmentInfo, segmentVisible, unChargeCount } = this.state;
    const ele =
      unChargeCount == 1 ? (
        <span>
          （本次购买的班课合同命中了【{segmentInfo[0]['name']}
          】，请确认每个班级在阶段1和阶段2分别可享受的优惠金额是否正确，
        </span>
      ) : (
        <span>
          （请确认每个班级在阶段1和阶段2分别可享受的优惠金额是否正确，如不修改则以系统默认填写的优惠金额为准，
        </span>
      );
    return (
      <div>
        {segmentVisible && (
          <Modal
            title="分段优惠设置"
            cancelBtnShow={false}
            closable={false}
            onOk={this.saveConfirm}
            onCancel={onCancel}
          >
            <div>
              {ele}
              <span style={{ color: 'red' }}>各阶段优惠金额确认后不可更改</span>！）
            </div>
            {segmentInfo &&
              segmentInfo.map((segment) => {
                return (
                  <div key={segment.contractId}>
                    <div>合同编号：{segment.contractId}</div>
                    <List dataSource={segment.contractProductSegmentChargeInfoVos} columns={this.columns} bordered />
                  </div>
                );
              })}
          </Modal>
        )}
      </div>
    );
  }
}
export default SegmentDiscount;
