import React from 'react';
import PropTypes from 'prop-types';
import ModalSelectPayType from '../';
import ContractPaymentModal from '../../ContractPaymentModal';
import ModalSharePayment from '../../SharePayment';
import ConfirmContractInformationModal from '../../ConfirmContractInformationModal';
import { RECEIVE, SHARE } from '../../../signContractPro/config';
import { runFunc } from 'SRC/utils/common';

class Modal extends React.Component {
  constructor(props) {
    super(props);

    this.handleSwitchPayType = this.handleSwitchPayType.bind(this);
    this.close = this.close.bind(this);
    this.handlePayment = this.handlePayment.bind(this);
    this.closePayment = this.closePayment.bind(this);
    this.closeContractInfo = this.closeContractInfo.bind(this);

    this.state = {
      selectPayTypeVisible: true,
      receivePaymentVisible: false,
      sharePaymentVisible: false,
      contractInformationModalVisible: false,
      modalTitle: '确认信息',
      radioMessage: true,
    };

    this.id = null;
    this.type = null;
  }

  static propTypes = {
    onCancel: PropTypes.func.isRequired,
    contractIds: PropTypes.arrayOf(PropTypes.string).isRequired,
    campusId: PropTypes.string.isRequired,
    studentId: PropTypes.string.isRequired,
    campusName: PropTypes.string.isRequired,
    customerName: PropTypes.string.isRequired,
    studentName: PropTypes.string.isRequired,
    contractNumber: PropTypes.oneOfType([PropTypes.string, PropTypes.number]).isRequired,
    signTime: PropTypes.string,
    isEcs: PropTypes.bool,
    useDeposit: PropTypes.bool,
  };

  static defaultProps = {
    isEcs: false,
    useDeposit: false,
  };

  handleSwitchPayType(type) {
    let payload = { selectPayTypeVisible: false };

    if (type === RECEIVE) {
      payload.receivePaymentVisible = true;
      payload.modalTitle = '确认信息';
    } else if (type === SHARE) {
      payload.sharePaymentVisible = true;
      payload.modalTitle = '分享支付';
    }

    this.setState(payload);
  }

  close() {
    runFunc(this.props.onCancel);
  }

  handlePayment(id, type) {
    this.id = id;
    this.type = ['pay', 'confirm'].includes(type) ? type : 'confirmPay';

    this.setState({
      receivePaymentVisible: false,
      sharePaymentVisible: false,
      contractInformationModalVisible: true,
      radioMessage: this.type === 'confirm',
    });
  }

  closePayment() {
    this.setState(
      {
        receivePaymentVisible: false,
        sharePaymentVisible: false,
      },
      () => {
        this.close();
        GlobalEvent.emit('REFRESH_CONTRACT_LIST');
      },
    );
  }

  closeContractInfo() {
    this.setState({ contractInformationModalVisible: false }, () => {
      this.close();
      GlobalEvent.emit('REFRESH_CONTRACT_LIST');
    });
  }

  render() {
    return (
      <div>
        {/* 选择支付方式 */}
        {this.state.selectPayTypeVisible && (
          <ModalSelectPayType
            campusId={this.props.campusId}
            onCancel={this.close}
            switchPayType={this.handleSwitchPayType}
          />
        )}

        {/* 录入线下收款 */}
        {this.state.receivePaymentVisible && (
          <ContractPaymentModal
            contractIds={this.props.contractIds}
            blCampusId={this.props.campusId}
            studentId={this.props.studentId}
            useDeposit={this.props.useDeposit}
            onCancel={this.closePayment}
            confirm={this.handlePayment}
          />
        )}

        {/* 分享支付 */}
        {this.state.sharePaymentVisible && (
          <ModalSharePayment
            contractIds={this.props.contractIds}
            studentId={this.props.studentId}
            useDeposit={this.props.useDeposit}
            onCancel={this.closePayment}
            onSuccess={this.handlePayment}
          />
        )}

        {/* 分享H5链接 */}
        {this.state.contractInformationModalVisible && (
          <ConfirmContractInformationModal
            title={this.state.modalTitle}
            contractInfo={{
              id: this.id,
              type: this.type,
              contractIds: this.props.contractIds,
              campusName: this.props.campusName,
              contractNumber: this.props.contractNumber,
              customerName: this.props.customerName,
              studentName: this.props.studentName,
              signTime: this.props.signTime,
              isEcs: this.props.isEcs,
            }}
            onCancel={this.closeContractInfo}
            radioMessage={this.state.radioMessage}
          />
        )}
      </div>
    );
  }
}

export default Modal;
