import React from 'react';
import PropTypes from 'prop-types';
import ModalSharePayment from '../';
import ModalShareConfirm from '../../ConfirmContractInformationModal';
import { runFunc } from 'UTIL/common';

class Modal extends React.Component {
  constructor(props) {
    super(props);

    this.close = this.close.bind(this);
    this.handleSharePayment = this.handleSharePayment.bind(this);
    this.closeConfirm = this.closeConfirm.bind(this);

    this.state = {
      sharePaymentVisible: true,
      shareConfirmVisible: false,
      radioMessage: true,
    };

    this.id = null;
    this.type = null;
  }

  static propTypes = {
    contractIds: PropTypes.arrayOf(PropTypes.string).isRequired,
    onCancel: PropTypes.func.isRequired,
    campusName: PropTypes.string.isRequired,
    customerName: PropTypes.string.isRequired,
    studentName: PropTypes.string.isRequired,
    contractNumber: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
    signTime: PropTypes.string,
  };

  close() {
    runFunc(this.props.onCancel);
  }

  handleSharePayment(id, type) {
    this.id = id;
    this.type = ['pay', 'confirm'].includes(type) ? type : 'confirmPay';
    this.setState({
      sharePaymentVisible: false,
      shareConfirmVisible: true,
      radioMessage: this.type === 'confirm',
    });
  }

  closeConfirm() {
    this.setState({ shareConfirmVisible: false }, () => {
      this.close();
      GlobalEvent.emit('REFRESH_CONTRACT_LIST');
    });
  }

  render() {
    return (
      <div>
        {this.state.sharePaymentVisible && (
          <ModalSharePayment
            contractIds={this.props.contractIds}
            onCancel={this.close}
            onSuccess={this.handleSharePayment}
          />
        )}

        {this.state.shareConfirmVisible && (
          <ModalShareConfirm
            title="分享支付"
            contractInfo={{
              id: this.id,
              type: this.type,
              contractIds: this.props.contractIds,
              campusName: this.props.campusName,
              contractNumber: this.props.contractNumber,
              customerName: this.props.customerName,
              studentName: this.props.studentName,
              signTime: this.props.signTime,
            }}
            onCancel={this.closeConfirm}
            radioMessage={this.state.radioMessage}
          />
        )}
      </div>
    );
  }
}

export default Modal;
