import React from 'react';
import PropTypes from 'prop-types';
import { Modal, Button, PopoverWrap } from 'COMPONENT';
import ContractProductPayment from '../../BLoComponets/ContractProductPayment';
import { invalidFilter } from '../../BLoComponets/ContractProductPayment/utils';
import {
  getContractByIds,
  batchConfirmingContractAndSharePay,
  createBatchSharePayment,
  batchDepositUsed,
  getSegmentChargeItemByContractIds,
} from 'SERVICE/contract';
import { errorsHandle, runFunc } from 'UTIL/common';
import style from './styles.module.less';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import ModalPaidOrder from '../PaidOrder';
import NP from 'number-precision';
import UseDeposit from '../../BLoComponets/UseDeposit';
import Statistics from '../../BLoComponets/ContractProductPayment/Statistics';

class SharePayment extends React.Component {
  static propTypes = {
    contractIds: PropTypes.arrayOf(PropTypes.string).isRequired,
    studentId: PropTypes.string.isRequired,
    onCancel: PropTypes.func.isRequired,
    onSuccess: PropTypes.func,
    isConfirmH5: PropTypes.bool,
    isMixing: PropTypes.bool,
    useDeposit: PropTypes.bool,
  };

  static defaultProps = {
    isConfirmH5: true,
    isMixing: false,
    useDeposit: false,
  };

  constructor(props) {
    super(props);

    this.init = this.init.bind(this);
    this.loadData = this.loadData.bind(this);
    this.onPaymentChange = this.onPaymentChange.bind(this);
    this.switchPaidOrderVisible = this.switchPaidOrderVisible.bind(this);
    this.sharePay = this.sharePay.bind(this);
    this.confirmAndPay = this.confirmAndPay.bind(this);
    this.saveDeposit = this.saveDeposit.bind(this);
    this.saveDepositThenConfirm = this.saveDepositThenConfirm.bind(this);
    this.onDepositChange = this.onDepositChange.bind(this);

    this.state = {
      loading: false,
      list: [],
      isNeedConfirm: false,
      paidOrderVisible: false,
      //
      transactionAmount: 0,
      depositAmount: 0,
      depositSelected: [],
      // 统计值
      totalStatistics: 0,
      paidStatistics: 0,
      promotionStatistics: 0,

      contractProductPaymentKey: 1,
    };

    this.paymentRef = null;
    this.useDepositRef = null;
  }

  componentDidMount() {
    this.init();
  }

  handleReload = () => {
    return new Promise(this.loadData);
  };

  async init() {
    this.setState({ loading: true });
    await new Promise(this.loadData);
    this.setState({ loading: false });
  }

  async loadData(resolve) {
    const res = await getContractByIds({ contractIds: this.props.contractIds }).catch(errorsHandle());
    if (res && res.data) {
      const list = invalidFilter(res.data);

      let totalAmount = 0;
      list.forEach((item) => {
        item.contractProductArr.forEach((item) => {
          item.amount = item.pendingAmount;
          totalAmount = NP.plus(totalAmount, item.pendingAmount);
        });
      });

      // 如果所有合同均已确认则不展示该按钮
      let isNeedConfirm = list.some((item) => [1, 2].includes(item.confirmStatus));
      // 多个合同时，如果存在精英班合同的，不可进行确认
      if (
        list.length > 1 &&
        list.some((item) => item.contractProductArr.some((subItem) => subItem.productType === 'ECS_CLASS'))
      ) {
        isNeedConfirm = false;
      }

      // 获取分段收费相关设置
      const response = await getSegmentChargeItemByContractIds({ contractIds: this.props.contractIds }).catch(
        errorsHandle(),
      );
      if (response && response.data) {
        const tempContractIds = Object.keys(response.data);
        list.forEach((item) => {
          if (tempContractIds.indexOf(item.contractId) > -1) {
            item.segmentChargeItems = response['data'][item.contractId];
          }
        });
      }
      this.setState(
        {
          list,
          isNeedConfirm,
          contractProductPaymentKey: this.state.contractProductPaymentKey + 1,
          ...this.calcStatistics(list),
        },
        () => {
          this.paymentRef.refreshValue();
        },
      );
    }
    resolve();
  }

  calcStatistics = (list) => {
    let result = {
      totalStatistics: 0,
      paidStatistics: 0,
      promotionStatistics: 0,
    };

    list.forEach((contract) => {
      contract.contractProductArr.forEach((item) => {
        result.totalStatistics = NP.plus(result.totalStatistics, item.totalAmount);
        result.paidStatistics = NP.plus(result.paidStatistics, item.paidAmount);
        result.promotionStatistics = NP.plus(result.promotionStatistics, item.promotionAmount);
      });
    });

    return result;
  };

  onPaymentChange(list, total) {
    this.setState(
      {
        list,
        transactionAmount: total,
      },
      () => {
        if (total === 0) {
          this.useDepositRef && this.useDepositRef.reset();
        }
      },
    );
  }

  switchPaidOrderVisible() {
    this.setState((prevState) => ({ paidOrderVisible: !prevState.paidOrderVisible }));
  }

  async sharePay() {
    if (this.paymentRef.validate()) {
      this.setState({ loading: true });

      let apiParams = {
        studentId: this.props.studentId,
        depositIds: this.state.depositSelected,
        transactionAmount: this.paymentRef.getTotalAmount(),
        contractProductPaymentList: this.paymentRef.getValue(),
      };

      const res = await createBatchSharePayment(apiParams).catch(errorsHandle());

      if (res && res.data) {
        return runFunc(this.props.onSuccess, res.data, 'pay');
      }

      this.setState({ loading: false });
    } else {
      showMessage('本次收款金额不能为0！');
    }
  }

  async confirmAndPay() {
    if (this.paymentRef.validate()) {
      this.setState({ loading: true });

      let apiParams = {
        studentId: this.props.studentId,
        depositIds: this.state.depositSelected,
        transactionAmount: this.paymentRef.getTotalAmount(),
        contractProductPaymentList: this.paymentRef.getValue(),
      };

      const res = await batchConfirmingContractAndSharePay(apiParams).catch(errorsHandle());

      if (res && res.data) {
        return runFunc(this.props.onSuccess, res.data, 'confirmPay');
      }

      this.setState({ loading: false });
    } else {
      showMessage('本次收款金额不能为0！');
    }
  }

  async saveDeposit() {
    this.setState({ loading: true });

    let apiParams = {
      needConfirm: false,
      studentId: this.props.studentId,
      depositIds: this.state.depositSelected,
      transactionAmount: this.paymentRef.getTotalAmount(),
      contractProductPaymentList: this.paymentRef.getValue(),
    };

    const res = await batchDepositUsed(apiParams).catch(errorsHandle());

    if (res) {
      showMessage('保存成功', 'success');
      return runFunc(this.props.onCancel);
    } else {
      this.setState({ loading: false });
    }
  }

  async saveDepositThenConfirm() {
    this.setState({ loading: true });

    let apiParams = {
      needConfirm: true,
      studentId: this.props.studentId,
      depositIds: this.state.depositSelected,
      transactionAmount: this.paymentRef.getTotalAmount(),
      contractProductPaymentList: this.paymentRef.getValue(),
    };

    const res = await batchDepositUsed(apiParams).catch(errorsHandle());

    if (res && res.data) {
      return runFunc(this.props.onSuccess, res.data.confirmBatchId, 'confirm');
    }

    this.setState({ loading: false });
  }

  onDepositChange(depositSelected, depositAmount) {
    this.setState({
      depositAmount,
      depositSelected,
    });
  }

  render() {
    const { props, state } = this;
    const isOffsetAmount = state.depositAmount && NP.minus(state.transactionAmount, state.depositAmount) === 0;
    const popoverVisible =
      !!state.transactionAmount &&
      this.useDepositRef &&
      this.useDepositRef.getDepositList().length > 0 &&
      state.depositAmount === 0;

    return (
      <div>
        <Modal
          title="分享支付"
          loading={state.loading}
          widthSize="XL"
          onCancel={props.onCancel}
          footer={
            <div className={style.flexWrap}>
              <div className={style.flexItemStart}>
                <Statistics
                  totalAmount={state.totalStatistics}
                  paidAmount={state.paidStatistics}
                  promotionAmount={state.promotionStatistics}
                  transactionAmount={state.transactionAmount}
                  depositAmount={state.depositAmount}
                />
              </div>

              <div className={style.flexItemEnd}>
                {props.contractIds.length === 1 && <Button onClick={this.switchPaidOrderVisible}>支付完成订单</Button>}

                <PopoverWrap
                  visible={popoverVisible}
                  content="该学生名下有未使用定金券，请确认本次支付是否要使用？"
                  type="warning"
                  overlayStyle={{ width: 220 }}
                >
                  <Button onClick={isOffsetAmount ? this.saveDeposit : this.sharePay} type="primary">
                    {isOffsetAmount ? '保存支付信息' : '分享支付'}
                  </Button>
                </PopoverWrap>

                {props.isConfirmH5 && state.isNeedConfirm && (
                  <PopoverWrap
                    visible={popoverVisible}
                    content="该学生名下有未使用定金券，请确认本次支付是否要使用？"
                    type="warning"
                    overlayStyle={{ width: 220 }}
                  >
                    <Button onClick={isOffsetAmount ? this.saveDepositThenConfirm : this.confirmAndPay} type="primary">
                      {isOffsetAmount ? '保存支付并确认信息' : '确认信息并支付'}
                    </Button>
                  </PopoverWrap>
                )}
              </div>
            </div>
          }
        >
          <ContractProductPayment
            list={state.list}
            radioVisible={true}
            autoAssignVisible={true}
            onReload={this.handleReload}
            onChange={this.onPaymentChange}
            ref={(c) => (this.paymentRef = c)}
            bodyClassName={style.maxHeight}
            className={style.contractProductPaymentTop}
            key={this.state.contractProductPaymentKey}
            hasPayAllBtn={(state.list || []).length === 1}
            headerTailSlot={state.depositSelected.length > 0 && <span>【本次收款金额】已包含定金券金额</span>}
            subSlot={
              props.useDeposit ? (
                <UseDeposit
                  studentId={props.studentId}
                  ref={(c) => (this.useDepositRef = c)}
                  amount={state.transactionAmount}
                  onChange={this.onDepositChange}
                  labelSlot={<span className={style.depositLabel}>已预付定金</span>}
                  selectWidth={260}
                  size="xSmall"
                  disabledTips="收款金额必需大于0"
                />
              ) : undefined
            }
          />
          {state.isNeedConfirm && props.isConfirmH5 ? (
            <p className={style.tip}>家长打开分享支付链接后，需先确认信息，然后才能进行支付。</p>
          ) : props.isMixing ? (
            <p className={style.tip}>
              本次所选合同包含部分非常规合同，故仅能进行批量支付，无法进行“确认信息并支付”的操作。
            </p>
          ) : (
            <p className={style.tip}>请在完成分享支付操作后，再回到合同列表点击“确认信息”按钮，发送短信向家长确认。</p>
          )}
        </Modal>

        {state.paidOrderVisible && (
          <ModalPaidOrder contractId={props.contractIds[0]} onCancel={this.switchPaidOrderVisible} />
        )}
      </div>
    );
  }
}

export default SharePayment;
