import React from 'react';
import { PreviewImgs, Modal } from 'COMPONENT';
import styles from './index.less';

/*
  imgArr: 数组 图片
  widthArr: [一张图时图片的宽度， 2或者4张图时图片的宽度，3、5、n张图时图片的宽度]
*/
class ImageContainer extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      previewImg: false, // 图片预览
      previewSrc: '',
    };
  }
  /**
   * 预览
   */
  onPreview = (src) => {
    this.setState({
      previewSrc: src,
      previewImg: true,
    });
  };
  render() {
    const { imgArr, widthArr, center } = this.props;
    const { previewImg, previewSrc } = this.state;
    const imgLen = imgArr.length;
    let empty = 0;
    let imgWidth = 200;

    if (imgLen === 1) {
      imgWidth = widthArr[0];
    } else if (imgLen === 2 || imgLen === 4) {
      imgWidth = widthArr[1];
    } else {
      imgWidth = widthArr[2];
      const emptyLen = imgLen % 3;
      empty = emptyLen === 0 ? emptyLen : 3 - emptyLen;
    }

    const imgList = imgArr.map((v, i) => {
      return (
        <img
          src={v}
          style={{ width: imgWidth, height: imgWidth, margin: 5 }}
          key={'img' + i}
          onClick={() => this.onPreview(v)}
        />
      );
    });
    const emptyElm = [];
    for (var i = 0; i < empty; i++) {
      emptyElm.push(
        <div className={styles.emptyElm} style={{ width: imgWidth, height: imgWidth }} key={'empty' + i}></div>,
      );
    }
    return (
      <div className={center ? styles.imageContainerCenter : styles.imageContainer}>
        {imgList}
        {emptyElm}
        {previewImg && (
          <Modal
            width={800}
            okBtnShow={false}
            cancelBtnShow={false}
            closable={true}
            onClose={() => this.setState({ previewImg: false })}
          >
            <PreviewImgs list={imgArr} activeSrc={previewSrc} showThumbnail={false} />
          </Modal>
        )}
      </div>
    );
  }
}

export default ImageContainer;
