import React from 'react';
import Modal from 'COMPONENT/Modal';
import Button from 'COMPONENT/Button';
import ImageContainer from './ImageContainer';
import styles from './styles.module.less';
import { getAttendancePhotoOrCertInfo } from 'SERVICE/course';
import { errorsHandle, runFunc } from 'UTIL/common';

const PHOTO = 'PHOTO';
const CERT = 'CERT';
const attendanceType = {
  [PHOTO]: '考勤拍照',
  [CERT]: '考勤凭证',
};
class ShowCourseImageModal extends React.Component {
  constructor(props) {
    super(props);
    this.getDetail = this.getDetail.bind(this);
    this.close = this.close.bind(this);

    this.state = {
      // 姓名
      type: this.props.type,
      detail: {},
      photos: [],
      certificate: {},
    };
  }

  componentDidMount() {
    this.getDetail();
  }

  componentWillMount() {}

  async getDetail() {
    this.setState({ loading: true });
    const apiParams = {
      classCourseType: this.props.classCourseType,
      classId: this.props.classId,
      courseId: this.props.courseId,
    };
    const res = await getAttendancePhotoOrCertInfo(apiParams).catch(errorsHandle());
    if (res && res.data) {
      this.setState({
        detail: res.data,
        certificate: res.data.certificate,
        photos: res.data.photos,
        loading: false,
      });
    }
  }

  close() {
    this.props.onCancel();
  }

  render() {
    const { detail, photos, certificate, loading } = this.state;
    let imgArr = [];
    if (this.props.attendanceType === PHOTO) {
      imgArr = photos.map((v) => {
        return v.path;
      });
    } else {
      imgArr = certificate.paths;
    }
    return (
      <Modal
        title={attendanceType[this.props.attendanceType]}
        width={500}
        loading={loading}
        onCancel={this.close}
        footer={
          <div className={styles.footer}>
            <Button type="default" onClick={this.close}>
              关闭
            </Button>
          </div>
        }
      >
        {this.props.attendanceType === PHOTO ? (
          <div>
            <p>提交时间：{detail.attendanceTime}</p>
            <p>
              课程名称：{detail.className}第{detail.courseNum}讲
            </p>
            <p>提交人：{detail.attendanceUserName}</p>
            <div className={styles.takePictureImg}>
              {imgArr && imgArr.length && <ImageContainer imgArr={imgArr} widthArr={[250, 150, 130]} center={true} />}
            </div>
          </div>
        ) : (
          <div>
            <div className={styles.item}>
              <p className={styles.title}>图片</p>
              {imgArr && imgArr.length && <ImageContainer imgArr={imgArr} widthArr={[250, 150, 130]} />}
            </div>
            <div className={styles.item}>
              <p className={styles.title}>说明</p>
              <p>{certificate.overdueReason}</p>
            </div>
          </div>
        )}
      </Modal>
    );
  }
}

export default ShowCourseImageModal;
