import React from 'react';
import { connect } from 'dva';
import { noop } from 'lodash';
import { Modal, Form, Input, Select, Icon } from 'COMPONENT';
import { Popover } from 'antd';
import { runFunc } from 'SRC/utils/common';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { getContractInfoById, updateQuarterType } from 'SERVICE/contract';
import { getQuarterTypeDict } from 'UTIL/dict';
import styles from './styles.module.less';
import tipsBase64 from './tipsBase64';
import { SUMMER, AUTUMN } from './config';

const { Item } = Form;

class SmallClassAlterQuarter extends React.Component {
  constructor(props) {
    super(props);
    this.init = this.init.bind(this);
    this.loadContractInfo = this.loadContractInfo.bind(this);
    this.handleAlterQuarter = this.handleAlterQuarter.bind(this);
    this.close = this.close.bind(this);
    this.handleFieldsChange = this.handleFieldsChange.bind(this);

    this.formRef = null;
    this.state = {
      loading: false,
      studentName: '',
      contractId: props.contractId,
      blCampusName: '',
      year: null,
      quarterType: null,
      isRenderTips: false,
    };

    this.quarterTypeStorage = null;
    this.singYear = 0;
    this.signMount = 0;
  }

  componentDidMount() {
    this.init();
  }

  async init() {
    const dict = await getQuarterTypeDict({ contractId: this.state.contractId });
    this.setState(
      {
        dict,
        isRenderTips: true,
      },
      () => {
        this.props.dispatch({
          type: 'dict/initDict',
          payload: {
            list: ['campus'],
            callback: this.loadContractInfo,
          },
        });
      },
    );
  }

  handleAlterQuarter() {
    this.formRef.validateFields(async (error) => {
      if (!error) {
        const { contractId, quarterType } = this.formRef.getFieldsValue();
        try {
          this.setState({ loading: true });
          const data = await updateQuarterType({ contractId, quarterType });
          if (data.resultCode === 0) {
            showMessage('修改成功', 'success');
            GlobalEvent.emit('REFRESH_CONTRACT_LIST');
            this.close();
          }
        } finally {
          this.setState({ loading: false });
        }
      }
    });
  }

  close() {
    runFunc(this.props.onCancel);
  }

  async loadContractInfo() {
    try {
      this.setState({ loading: true });
      const { data, resultCode } = await getContractInfoById({ contractId: this.props.contractId });
      if (resultCode === 0) {
        const _signDate = data.signDate.split('-').map(Number);

        this.quarterTypeStorage = data.quarterType;
        this.singYear = _signDate[0];
        this.signMount = _signDate[1];

        this.setState({
          studentName: data.studentName,
          blCampusName: data.blCampusName,
          year: this.yearAdapter(this.quarterTypeStorage, this.singYear, this.signMount),
          quarterType: data.quarterType,
        });
      }
    } finally {
      this.setState({ loading: false });
    }
  }

  yearAdapter = (quarterType, year, mount) => {
    switch (quarterType) {
      case SUMMER:
        return mount < 3 ? year - 1 : year;
      case AUTUMN:
        return mount < 8 ? year - 1 : year;
      default:
        return year;
    }
  };

  handleFieldsChange(fields) {
    if (fields.quarterType && fields.quarterType.value !== this.quarterTypeStorage) {
      this.quarterTypeStorage = fields.quarterType.value;
      this.formRef.setFieldsValue({
        year: this.yearAdapter(this.quarterTypeStorage, this.singYear, this.signMount),
      });
    }
  }

  render() {
    const { state } = this;
    return (
      <Modal
        title="小班合同签单季度修改"
        width={500}
        okText="保存"
        cancelText="取消"
        loading={state.loading}
        maskClosable={false}
        onCancel={this.close}
        onOk={this.handleAlterQuarter}
      >
        <Form ref={(c) => (this.formRef = c)} onFieldsChange={this.handleFieldsChange} onSubmit={noop}>
          <Item
            name="studentName"
            initialValue={state.studentName}
            rules={[{ required: true, message: '学生姓名必填' }]}
            formItemProps={{
              label: '学生姓名：',
              colon: false,
              labelCol: { span: 5 },
              wrapperCol: { span: 18 },
            }}
          >
            <Input readOnly />
          </Item>
          <Item
            name="contractId"
            initialValue={state.contractId}
            rules={[{ required: true, message: '合同编号必填' }]}
            formItemProps={{
              label: '合同编号：',
              colon: false,
              labelCol: { span: 5 },
              wrapperCol: { span: 18 },
            }}
          >
            <Input readOnly />
          </Item>
          <Item
            name="blCampusName"
            initialValue={state.blCampusName}
            rules={[{ required: true, message: '合同学习中心必填' }]}
            formItemProps={{
              label: '合同学习中心：',
              colon: false,
              labelCol: { span: 5 },
              wrapperCol: { span: 18 },
            }}
          >
            <Input readOnly />
          </Item>

          <Item
            name="year"
            initialValue={state.year}
            formItemProps={{
              label: '签单年份：',
              colon: false,
              labelCol: { span: 5 },
              wrapperCol: { span: 18 },
            }}
            slot={
              <Popover placement="topRight" content={<img src={tipsBase64} alt="签单年份计算公式" />}>
                <Icon className={styles.tipIcon} type="warning" size={20} />
              </Popover>
            }
          >
            <Input readOnly disabled />
          </Item>

          <Item
            name="quarterType"
            initialValue={state.quarterType}
            rules={[{ required: true, message: '请选择签单季度' }]}
            formItemProps={{
              label: '签单季度：',
              colon: false,
              labelCol: { span: 5 },
              wrapperCol: { span: 18 },
            }}
          >
            <Select dict={this.state.dict} />
          </Item>
        </Form>
      </Modal>
    );
  }
}

const DecorateModal = connect((state) => {
  return {
    campusDict: state.dict.campus,
  };
})(SmallClassAlterQuarter);

export default DecorateModal;
