import React from 'react';
import { connect } from 'dva';
import { browserHistory } from 'dva/router';
import { Modal, List, Select, InputNumber, TextLink } from 'COMPONENT';
import { Tabs, Switch } from 'antd';
import NP from 'number-precision';
import { debounce, clone, difference } from 'lodash';
import Header from './Header';
import HistoryTab from './HistoryTab';
import styles from './styles.less';
import { OTHERS } from 'CONST/dicts/product';
import { USER_USER, USER_CAMPUS, USER_BRANCH, CAMPUS_CAMPUS, subBonusDict } from './common';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { loginUser } from 'UTIL/common';
import { formatProductTypeName } from './util';

const TabPane = Tabs.TabPane;
const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L } = List;
const _dictContractType = [
  {
    text: '新签',
    val: 'NEW_CONTRACT',
  },
  {
    text: '续费',
    val: 'RE_CONTRACT',
  },
];
let _dictContractTypeObj = {};
_dictContractType.forEach((value) => {
  _dictContractTypeObj[value.val] = value.text;
});

const listContractTabPaneId = 'function/contract/listContract.html'; //redux上的合同列表tab的id

class Content extends React.Component {
  constructor(props) {
    super(props);

    this.renderContractType = this.renderContractType.bind(this);
    this.switchPersonFilter = this.switchPersonFilter.bind(this);

    this.state = {
      isFilterPerson: true,
    };

    this.isTwoTeacher = loginUser.isTwoTeacherSym();
  }

  getList = (isCampus) => {
    const { campusList, personalList } = this.props;
    if (isCampus) {
      return campusList;
    } else {
      return personalList;
    }
  };
  //渲染业绩归属类型
  renderBaseBonus = (text, row, index, isCampus) => {
    const { canUpdate } = this.props;

    if (isCampus) {
      return '学习中心';
    } else {
      if (canUpdate) {
        return (
          <Select
            dict={subBonusDict}
            value={row.subDistributeType}
            onChange={(subDistributeType) =>
              this.props.onUpdateRowById(
                {
                  subDistributeType,
                  distributeStaffId: null,
                  distributeStaffName: null,
                  distributeOrgName: null,
                  distributeOrgId: null,
                },
                row.id,
                isCampus,
              )
            }
            disabled={true}
          />
        );
      } else {
        return subBonusDict.find((dict) => dict.val === row.subDistributeType).text;
      }
    }
  };
  /**
   * 搜索
   */
  search = (key, index) => {
    debounce(this.props.onSearchUser, 300)(key, index);
  };
  //渲染学习中心业绩归属或个人业绩归属
  renderSubBonus = (text, row, index, isCampus) => {
    const { branchDict, campusDict, canUpdate } = this.props;
    const { subDistributeType, distributeOrgName, distributeStaffName, distributeStaffId } = row;
    //不能分配则直接显示名字
    if (!canUpdate) {
      if (subDistributeType === USER_USER) {
        return distributeStaffName;
      } else {
        return distributeOrgName;
      }
    }
    let dict, key;
    //不同的业绩归属类型对应不同的字典和归属
    switch (subDistributeType) {
      case USER_USER: {
        dict = this.props['personDict-' + index];
        dict = dict || [];
        //拼接上已选的
        if (distributeStaffId) {
          dict = [...dict, { val: distributeStaffId, text: distributeStaffName, title: distributeStaffName }];
        }
        key = 'distributeStaffId';
        break;
      }
      case USER_CAMPUS: {
        dict = campusDict;
        key = 'distributeOrgId';
        break;
      }
      case USER_BRANCH: {
        dict = branchDict;
        key = 'distributeOrgId';
        break;
      }
      case CAMPUS_CAMPUS: {
        dict = campusDict;
        key = 'distributeOrgId';
      }
    }
    return (
      <Select
        value={row[key]}
        dict={dict}
        onSearchKeyChange={subDistributeType === USER_USER ? (key) => this.search(key, index) : null}
        showSearch
        noOptionTips="没有找到"
        searchPlaceholder="输入名字搜索"
        onChange={(type) => this.props.onUpdateRowById({ [key]: type }, row.id, isCampus)}
      />
    );
  };
  /**
   * 业绩金额输入变化事件
   * @param amount
   * @param row
   * @param index
   * @param isCampus
   */
  onAmountChange = (amount, row, index, isCampus) => {
    const list = this.getList(isCampus);

    //计算出即将变化后的总业绩金额
    const toValue = list.reduce((acc, cur, idx) => {
      if (cur.contractProductId === row.contractProductId) {
        if (idx !== index) return NP.plus(acc, cur.amount);
        return NP.plus(acc, amount);
      } else {
        return acc;
      }
    }, 0);
    //小于或等于原始总金额才可以设置
    if (toValue <= row.totalAmount) {
      this.props.onUpdateRow({ amount }, index, isCampus);
    }
  };
  //渲染金额
  renderAmount = (text, row, index, isCampus) => {
    const { canUpdate } = this.props;
    if (!canUpdate) return text;
    return (
      <InputNumber
        disabled={true}
        value={text}
        min={0}
        onChange={(amount) => this.onAmountChange(amount, row, index, isCampus)}
      />
    );
  };
  /**
   * 业绩科次输入变化事件
   * @param amountRate
   * @param row
   * @param index
   * @param isCampus
   */
  onAssignChange = (amountRate, row, index, isCampus) => {
    const list = this.getList(isCampus);

    //计算出即将变化后的总业绩科次
    const toValue = list.reduce((acc, cur, idx) => {
      if (cur.contractProductId === row.contractProductId) {
        if (idx !== index) return NP.plus(acc, cur.amountRate);
        return NP.plus(acc, amountRate);
      } else {
        return acc;
      }
    }, 0);
    //小于或等于原始总科次才可以设置
    if (toValue <= row.totalAmountRate) {
      this.props.onUpdateRow({ amountRate }, index, isCampus);
    }
  };
  //渲染科次
  renderAssign = (text, row, index, isCampus) => {
    const { canUpdate } = this.props;
    if (!canUpdate || row.productType === OTHERS) return <span style={{ fontSize: 14 }}>{text}</span>;
    let newText = text.toString().slice(0, 4) - 0;
    return (
      <InputNumber
        disabled={true}
        value={newText}
        min={0}
        precision={2}
        step={0.0001}
        max={Math.min(1, row.totalAmountRate)}
        onChange={(amountRate) => this.onAssignChange(amountRate, row, index, isCampus)}
      />
    );
  };

  onTabChange = (tabKey) => {
    if (tabKey === '2' && this.props.historyList.length === 0) {
      this.props.onHistoryListChange({});
    }
  };
  /**
   * 打开分配金额弹框
   */
  openAssignMoney = () => {
    //先关闭弹框再跳转
    this.props.onClose(() => {
      //要先转码，否则OldPage解析路由会把 contractId 参数弄丢
      browserHistory.push(
        '/oldPage?src=function/contract/listContract.html?' +
          encodeURIComponent('contractId=' + this.props.fundDetail.contractId),
      );

      //如果已存在合同列表tab，要刷新tab才会打开弹框，因为旧页面绑定的是页面onload事件
      if (this.props.hasListContract) {
        this.props.dispatch({
          type: 'main/refreshTab',
          payload: {
            id: listContractTabPaneId,
          },
        });
      }
    });
  };

  reduceByProductId = (list, key) => {
    const keysMap = {
      amount: ['amount', 'totalAmount'],
      rate: ['amountRate', 'totalAmountRate'],
    };
    const afterProducts = []; //合同产品去重数组
    return list.reduce((acc, cur) => {
      let ret = acc;
      //同一合同产品的total要加一次
      if (afterProducts.indexOf(cur.contractProductId) === -1) {
        afterProducts.push(cur.contractProductId);
        ret = NP.plus(acc, cur[keysMap[key][1]]);
      }
      //减去当前分配行的
      return NP.minus(ret, cur[keysMap[key][0]]);
    }, 0);
  };

  /**
   * @param {Function=} sucCb 满足条件的回调函数
   * @param {Function=} failCb 不满足条件的回调函数
   * @description 权限校验函数
   * @return {*} 没有回调函数的时候返回一个布尔值，true为校验成功；有回调函数的时候返回回调函数返回值
   * **/
  authCheck = (sucCb, failCb) => {
    // TWO_TEACHER_PRINCIPAL   分校校长
    // CAMPUS_OPERATION_DIRECTOR  学习中心负责人
    // BRANCH_HEAD_OF_ACADEMIC_AFFAIR 分校教务负责人
    const getRoleCode = this.props.roleCode ? this.props.roleCode.split(',') : [];
    const initRoleArr = ['TWO_TEACHER_PRINCIPAL', 'CAMPUS_OPERATION_DIRECTOR', 'BRANCH_HEAD_OF_ACADEMIC_AFFAIR'];
    let status = difference(getRoleCode, initRoleArr).length !== getRoleCode.length;
    let defaultCallback = () => {};
    if (!sucCb && !failCb) {
      return status;
    }
    return status ? (sucCb || defaultCallback)() : (failCb || defaultCallback)();
  };

  onOk = () => {
    const { personalList, campusList } = this.props;
    const currStatus = this.authCheck();
    // 权限限制非指定用户不能修改业绩归属类型、个人业绩归属和类型，目前只需个人，学习中心暂无要求
    try {
      personalList.forEach((item) => {
        if (currStatus) {
          if (!item.distributeStaffId && !item.distributeOrgId) throw new Error('个人业绩归属不能为空');
        }
      });
      campusList.forEach((item) => {
        if (!item.distributeOrgId) throw new Error('学习中心业绩归属不能为空');
      });

      this.props.onSave();
    } catch (e) {
      showMessage(e.message);
    }
  };
  //获取表头
  getColumns = (type) => {
    const isCampus = type === 'campus';
    const productTypeNameWidth = this.isTwoTeacher ? COL_WIDTH_M : COL_WIDTH_S;

    /**
     * 兼容`其他类型`隐藏(业绩归属类型/个人业绩归属/类型)
     * */
    const columns = [
      {
        dataIndex: 'productTypeName',
        title: '类型',
        width: productTypeNameWidth,
        render: (text, row, index) => {
          return formatProductTypeName(row, this.isTwoTeacher);
        },
      },
      { dataIndex: 'productCharacterName', title: '产品性质', width: 80 },
      { dataIndex: 'productName', title: '产品名称', width: COL_WIDTH_L },
      { dataIndex: 'subjectName', title: '科目', width: COL_WIDTH_S },
      { dataIndex: 'gradeName', title: '年级', width: COL_WIDTH_S },
      {
        dataIndex: 'baseDistributeType',
        title: '业绩归属类型',
        width: 105,
        render: (text, row, index) => {
          return this.authCheck(
            () => this.renderBaseBonus(text, row, index, isCampus),
            () => row.subDistributeTypeName,
          );
        },
      },
      {
        dataIndex: 'subDistributeType',
        title: isCampus ? '学习中心业绩归属' : '个人业绩归属',
        width: 150,
        render: (text, row, index) => {
          return this.authCheck(
            () => this.renderSubBonus(text, row, index, isCampus),
            () => row.distributeStaffName,
          );
        },
      },
      {
        dataIndex: 'amount',
        title: '业绩金额',
        width: 100,
        render: (text, row, index) => this.renderAmount(text, row, index, isCampus),
      },
      {
        dataIndex: 'amountRate',
        title: '业绩科次',
        width: 100,
        render: (text, row, index) => this.renderAssign(text, row, index, isCampus),
      },
      {
        key: 'action',
        title: '操作',
        width: 95,
        getActionConfig: (text, row, index) => {
          const { canUpdate } = this.props;
          const ret = [];
          if (canUpdate) {
            ret.push({ text: '添加', disabled: true, onClick: () => this.props.onAddRow(row, index, isCampus) });
          }
          if (row.ISADD) {
            ret.push({ text: '删除', onClick: () => this.props.onDelRow(index, isCampus) });
          }
          return ret;
        },
      },
    ];

    if (type === 'personal') {
      columns.splice(-1, 0, {
        dataIndex: 'contractType',
        title: '类型',
        width: 100,
        render: (text, row, index) => {
          return this.authCheck(
            () => this.renderContractType(text, row, index, isCampus),
            () => _dictContractTypeObj[text],
          );
        },
      });
    }

    return columns;
  };

  // 更新-类型(合同类型)
  updateContractType(val, row, index, isCampus) {
    let list = clone(this.props.personalList);
    const prdId = row.contractProductId;
    list = list.map((item) => {
      return item.contractProductId === prdId
        ? {
            ...item,
            contractType: val,
          }
        : item;
    });
    this.props.onUpdateList(list, isCampus);
  }
  // 渲染列表项-类型(合同类型)
  renderContractType(text, row, index, isCampus) {
    return (
      <Select
        placeholder="类型"
        dict={_dictContractType}
        value={text}
        onChange={(val) => {
          this.updateContractType(val, row, index, isCampus);
        }}
        allowClear={false}
      />
    );
  }

  // 切换个人业绩分配数据过滤开关
  switchPersonFilter() {
    this.setState((prevState) => ({
      isFilterPerson: !prevState.isFilterPerson,
    }));
  }

  static filterPersonList(list) {
    return list.filter((item) => item.productType !== 'OTHERS');
  }

  render() {
    const {
      campusList = [],
      personalList = [],
      historyList = [],
      fundDetail = {},
      canUpdate,
      historyTotal,
    } = this.props;
    return (
      <Modal
        widthSize="XL"
        title="科次分配"
        okBtnShow={canUpdate}
        loading={this.props.loading}
        className={styles.customModal}
        onOk={this.onOk}
        okText="提交"
        closable
        onClose={this.props.onClose}
        footerTipText={
          <span>
            合同产品的业绩分配额度同时受收款分配资金影响，如需要调整或查看，请点击{' '}
            <TextLink onClick={this.openAssignMoney}>合同资金分配</TextLink> 。
          </span>
        }
      >
        <Header fundDetail={fundDetail} />

        {/*业绩分配*/}
        <Tabs defaultActiveKey="1" onChange={this.onTabChange}>
          <TabPane tab="业绩分配" key="1">
            {/* 20191114 双师系统 隐藏`学习中心业绩分配` */}
            <div style={{ display: 'none' }}>
              <div className={styles.listHeader}>
                <p className={styles.headerTitle}>学习中心业绩分配</p>
                <p>
                  <span className={styles.headerPice}>
                    <label>可分配业绩金额:&nbsp;&nbsp;</label>
                    <span>{this.reduceByProductId(campusList, 'amount')}</span>
                  </span>
                  <span className={styles.headerPice}>
                    <label>可分配科次:&nbsp;&nbsp;</label>
                    <span>{this.reduceByProductId(campusList, 'rate')}</span>
                  </span>
                </p>
              </div>
              <List style={{ marginBottom: 30 }} dataSource={campusList} columns={this.getColumns('campus')} />
            </div>

            <div className={styles.listHeader}>
              <p className={styles.headerTitle}>
                <span>个人业绩分配</span>
                <span className={styles.headerTitleSwitchLabel}>其他类型</span>
                <Switch
                  checkedChildren="隐藏"
                  unCheckedChildren="显示"
                  checked={this.state.isFilterPerson}
                  onChange={this.switchPersonFilter}
                />
              </p>
              <p>
                <span className={styles.headerPice}>
                  <label>可分配业绩金额:&nbsp;&nbsp;</label>
                  <span>{this.reduceByProductId(personalList, 'amount')}</span>
                </span>
                <span className={styles.headerPice}>
                  <label>可分配科次:&nbsp;&nbsp;</label>
                  <span>{this.reduceByProductId(personalList, 'rate')}</span>
                </span>
              </p>
            </div>
            <List
              dataSource={this.state.isFilterPerson ? Content.filterPersonList(personalList) : personalList}
              columns={this.getColumns('personal')}
            />
          </TabPane>

          <TabPane tab="业绩分配记录" key="2">
            <HistoryTab
              historyList={historyList}
              historyTotal={historyTotal}
              onHistoryListChange={this.props.onHistoryListChange}
            />
          </TabPane>
        </Tabs>
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    roleCode: state.account.user.roleCode,
    hasListContract: state.main.tabpanes.some((pane) => pane.id === listContractTabPaneId),
  };
};

export default connect(mapStateToProps, null)(Content);
