import React from 'react';
import List from 'COMPONENT/List';
import { USER_USER } from './common';
import styles from './styles.less';
import { loginUser } from 'UTIL/common';
import { formatProductTypeName } from './util';

const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L } = List;

export default class extends React.Component {
  state = {
    page: 1,
    rows: 10,
  };

  isTwoTeacher = loginUser.isTwoTeacherSym();

  /**
   * 翻页
   */
  pageChange = ({ current, pageSize }) => {
    const newPagination = { page: current, rows: pageSize };

    this.props.onHistoryListChange({ ...newPagination });
    this.setState({
      ...newPagination,
    });
  };

  renderDistributeName = (text, row, index) => {
    if (text === USER_USER) {
      return row.distributeStaffName;
    } else {
      return row.distributeOrgName;
    }
  };

  formatColumn = () => {
    const productTypeNameWidth = this.isTwoTeacher ? COL_WIDTH_M : COL_WIDTH_S;

    return [
      {
        dataIndex: 'productTypeName',
        title: '类型',
        width: productTypeNameWidth,
        render: (text, row, index) => {
          return formatProductTypeName(row, this.isTwoTeacher);
        },
      },
      { dataIndex: 'productCharacterName', title: '产品性质', width: 80 },
      { dataIndex: 'productName', title: '产品名称', width: COL_WIDTH_L },
      { dataIndex: 'subDistributeTypeName', title: '业绩归属类型', width: 105 },
      { dataIndex: 'subDistributeType', title: '业绩归属', width: 80, render: this.renderDistributeName },
      { dataIndex: 'amount', title: '业绩金额', width: 80 },
      { dataIndex: 'amountRate', title: '业绩科次', width: 80 },
      { dataIndex: 'operationName', title: '操作类型', width: 80 },
      { dataIndex: 'createTime', title: '创建日期', width: 100 },
      { dataIndex: 'createUserName', title: '操作人', width: COL_WIDTH_S },
    ];
  };

  render() {
    const { historyList, historyTotal } = this.props;
    const { page, rows } = this.state;

    return (
      <div className={styles.customListHeight}>
        <List
          dataSource={historyList}
          columns={this.formatColumn()}
          pagination={{
            current: page,
            pageSize: rows,
            total: historyTotal,
          }}
          onChange={this.pageChange}
        />
      </div>
    );
  }
}
