import React from 'react';
import Content from './Content';
import {
  getUserForPerformance,
  getDistributionList,
  saveDistributionList,
  getDistributionHistoryList,
  canUpdateIncomeDistribution,
} from 'SERVICE/contract';
import { loadCommonDict, loadBranchCampusDict, loadBranchDict } from 'UTIL/dict';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

import { USERTYPE, CAMPUSTYPE } from './common';

export default class PageWrapper extends React.Component {
  constructor(props) {
    super(props);
    this.updateRowById = this.updateRowById.bind(this);
    this.updateList = this.updateList.bind(this);
    this.state = {
      //从旧页面列表过来的数据
      fundDetail: {
        // fundId: props.oldPageParams.fundId,
        contractId: props.oldPageParams.contractId,
        contractTotalMoney: props.oldPageParams.contractTotalMoney,
        contractCampus: props.oldPageParams.contractCampus,
        // payType: props.oldPageParams.payType,
        // payMoney: props.oldPageParams.payMoney,
        stuName: props.oldPageParams.stuName,
        stuId: props.oldPageParams.stuId,
      },
      canUpdate: false, //是否可以分配科次
      campusList: [],
      personalList: [],
      historyList: [],
      historyTotal: 0,
      branchDict: [],
      campusDict: [],
      personDict: [],
      loading: false,
    };
    this.dictVersion = 0; //搜索计数
  }

  async componentDidMount() {
    const { contractId } = this.state.fundDetail;
    this.setState({ loading: true });
    try {
      const [distributionData, branchDict, campusDict, canUpdate] = await Promise.all([
        getDistributionList({ page: 1, rows: 999, contractId }),
        loadBranchDict(),
        loadBranchCampusDict(),
        canUpdateIncomeDistribution({ contractId }),
      ]);

      //把数据拆分成学习中心绩效列表和个人绩效列表
      let campusList = [];
      let personalList = [];
      distributionData.rows.forEach((item) => {
        if (item.baseDistributeType === USERTYPE) {
          personalList.push(item);
        } else {
          campusList.push(item);
        }
      });
      // 将2种list中的数据按contractProductId分类
      personalList = PageWrapper.arrSortByItemKey(personalList, 'contractProductId');
      campusList = PageWrapper.arrSortByItemKey(campusList, 'contractProductId');

      this.setState({
        canUpdate,
        campusList,
        personalList,
        branchDict,
        campusDict: campusDict.map((group) => {
          group.options = group.options.map((dict) => ({ ...dict, title: dict.text }));
          return group;
        }),
        loading: false,
      });
    } catch (err) {
      this.setState({ loading: false });
      console.log(err);
      showMessage(err.text || err.message);
    }
  }
  // 将数组中的对象按指定key名分类 [{key: val}, {key: val}, ...]
  static arrSortByItemKey(arr, keyName) {
    let _storage = {};
    arr.forEach((item) => {
      if (_storage[item[keyName]]) {
        _storage[item[keyName]].push(item);
      } else {
        _storage[item[keyName]] = [item];
      }
    });
    return Object.values(_storage).reduce((prevVal, currentVal) => prevVal.concat(currentVal), []);
  }

  /**
   * 更新列表行
   * isCampus为true，更新campusList，为false则更新personalList
   * @param obj
   * @param index
   * @param isCampus
   */
  onUpdateRow = (obj, index, isCampus) => {
    const listName = isCampus ? 'campusList' : 'personalList';
    this.setState({
      [listName]: this.state[listName].map((record, idx) => {
        if (index !== idx) return record;
        return {
          ...record,
          ...obj,
        };
      }),
    });
  };

  updateRowById(obj, id, isCampus) {
    const listName = isCampus ? 'campusList' : 'personalList';
    this.setState({
      [listName]: this.state[listName].map((record, idx) => {
        if (id !== record.id) return record;
        return {
          ...record,
          ...obj,
        };
      }),
    });
  }

  // 更新列表
  updateList(list, isCampus) {
    const listName = isCampus ? 'campusList' : 'personalList';
    this.setState({ [listName]: list });
  }

  /**
   * 增加列表行
   * @param row
   * @param isCampus
   */
  onAddRow = (row, index, isCampus) => {
    const listName = isCampus ? 'campusList' : 'personalList';
    const list = this.state[listName];

    const newRow = {
      ISADD: true,
      productType: row.productType,
      productTypeName: row.productTypeName,
      bonusType: row.bonusType,
      contractProductId: row.contractProductId,
      productCharacter: row.productCharacter,
      productCharacterName: row.productCharacterName,
      productId: row.productId,
      productName: row.productName,
      subjectName: row.subjectName,
      gradeName: row.gradeName,
      baseDistributeType: row.baseDistributeType,
      subDistributeType: row.subDistributeType,
      amount: 0,
      amountRate: 0,
      totalAmount: row.totalAmount,
      totalAmountRate: row.totalAmountRate,
      distributeOrgId: null,
      distributeOrgName: null,
      distributeStaffId: null,
      distributeStaffName: null,
      contractType: row.contractType,
    };
    this.setState({
      [listName]: list.slice(0, index + 1).concat(newRow, list.slice(index + 1)),
    });
  };

  /**
   * 删除列表行
   * @param index
   * @param isCampus
   */
  onDelRow = (index, isCampus) => {
    const listName = isCampus ? 'campusList' : 'personalList';
    this.setState({
      [listName]: this.state[listName].filter((row, idx) => index !== idx),
    });
  };

  /**
   * 搜索用户
   * @param term
   * @returns {Promise<void>}
   */
  onSearchUser = async (term, index) => {
    if (term) {
      const dictVersion = ++this.dictVersion;
      try {
        const dict = await getUserForPerformance({ term });
        if (this.dictVersion === dictVersion) {
          //每一行的用户下拉框一个dict，用 index 后缀区分
          this.setState({
            ['personDict-' + index]: dict.map((item) => ({ text: item.label, val: item.value, title: item.label })),
          });
        }
      } catch (e) {
        showMessage(e.text);
      }
    } else {
      this.dictVersion++;
      this.setState({ ['personDict-' + index]: [] });
    }
  };

  onHistoryListChange = async ({ page = 1, rows = 10 }) => {
    const { contractId } = this.state.fundDetail;
    this.setState({ loading: true });
    try {
      const response = await getDistributionHistoryList({ page, rows, contractId });
      this.setState({
        historyList: response.rows,
        loading: false,
        historyTotal: response.records,
      });
    } catch (e) {
      this.setState({ loading: false });
      showMessage(e.message || e.text);
    }
  };

  onSave = async () => {
    const { fundDetail, personalList, campusList } = this.state;
    const requestParams = {
      contractId: fundDetail.contractId,
      incomeDistributionList: [
        ...personalList.map((person) => {
          const newPerson = { ...person };
          if (Number(newPerson.id) < 0) {
            //去掉小于 0 的id(和后端的约定)
            delete newPerson.id;
          }
          return newPerson;
        }),
        ...campusList.map((campus) => {
          const newCampus = { ...campus };
          if (Number(newCampus.id) < 0) {
            delete newCampus.id;
          }
          return newCampus;
        }),
      ],
    };
    this.setState({ loading: true });
    try {
      await saveDistributionList(requestParams);
      this.setState({ loading: false });
      showMessage('保存成功！', 'success');
      this.props.onClose();
    } catch (e) {
      console.log(e);
      this.setState({ loading: false });
      showMessage(e.message || e.text, 'error');
    }
  };

  render() {
    return (
      <Content
        {...this.state}
        onClose={this.props.onClose}
        onUpdateRow={this.onUpdateRow}
        onUpdateRowById={this.updateRowById}
        onUpdateList={this.updateList}
        onAddRow={this.onAddRow}
        onDelRow={this.onDelRow}
        onSearchUser={this.onSearchUser}
        onHistoryListChange={this.onHistoryListChange}
        onSave={this.onSave}
      />
    );
  }
}
