import React from 'react';
import Modal from 'COMPONENT/Modal';
import { Row, Col, Table, message } from 'antd';
import styles from './styles.less';
import { getWithDetailByStudentReturnFeeId } from 'SERVICE/withdrawInfo';

class WithdrawInfo extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      withdrawMessage: {},
      withdrawReason: {},

      withdrawCertificate: [],

      reasonColumns: [],
      reasonData: [],

      attrSchoolColumns: [],
      attrSchoolData: [],
      attrTypeColumns: [],
      attrTypeData: [],
      oneToOneColumns: [],
      oneToOneData: [],

      classSchoolColumns: [],
      classSchoolData: [],
      personColumns: [],
      personData: [],
    };
  }

  componentDidMount() {
    const { studentReturnFeeId } = this.props;
    this.getData(studentReturnFeeId);
  }

  initReasonColumn = () => {
    let temp = [
      {
        title: '商户号',
        dataIndex: 'merchantNo',
        key: 'merchantNo',
      },
      {
        title: '支付方式',
        dataIndex: 'channelName',
        key: 'channelName',
      },
      {
        title: '支付参考号',
        dataIndex: 'posId',
        key: 'posId',
      },
      {
        title: '本次成功退费金额',
        key: 'refundAmount',
        dataIndex: 'refundAmount',
      },
    ];
    this.setState({
      reasonColumns: temp,
    });
  };

  initDutyColumn = () => {
    let temp1 = [
      {
        title: '责任学习中心',
        dataIndex: 'refundDutyName',
        key: 'refundDutyName',
      },
      {
        title: '责任业绩',
        dataIndex: 'refundDutyAmount',
        key: 'refundDutyAmount',
      },
    ];

    let temp2 = [
      {
        title: '责任归属类型',
        dataIndex: 'bonusDistributeTypeName',
        key: 'bonusDistributeTypeName',
      },
      {
        title: '责任归属',
        key: 'refundDutyName',
        dataIndex: 'refundDutyName',
      },
      {
        title: '责任归属业绩',
        dataIndex: 'refundDutyAmount',
        key: 'refundDutyAmount',
      },
    ];

    let temp3 = [
      {
        title: '1对1科目',
        dataIndex: 'refundSubjectName',
        key: 'refundSubjectName',
      },
      {
        title: '1对1科任老师',
        key: 'refundDutyTeacherName',
        dataIndex: 'refundDutyTeacherName',
      },
    ];

    this.setState({
      attrSchoolColumns: temp1,
      attrTypeColumns: temp2,
      oneToOneColumns: temp3,
    });
  };

  initClassColumn = () => {
    let temp1 = [
      {
        title: '责任学习中心',
        dataIndex: 'distributeOrganizationName',
        key: 'distributeOrganizationName',
      },
      {
        title: '责任科次',
        dataIndex: 'amountRate',
        key: 'amountRate',
      },
    ];
    let temp2 = [
      {
        title: '责任个人',
        dataIndex: 'distributeUserName',
        key: 'distributeUserName',
      },
      {
        title: '责任科次',
        key: 'amountRate',
        dataIndex: 'amountRate',
      },
    ];
    this.setState({
      classSchoolColumns: temp1,
      personColumns: temp2,
    });
  };

  getData = async (id) => {
    try {
      const data = await getWithDetailByStudentReturnFeeId({ studentReturnFeeId: id });
      let tempWithdrawMessage = {},
        tempWithdrawReason = {},
        tempReasonData = [],
        tempAttrSchoolData = [],
        tempAttrTypeData = [],
        tempOneToOneData = [],
        tempClassSchoolData = [],
        tempPersonData = [],
        tempWithdrawCertificate = [];
      if (data && data.data) {
        tempWithdrawMessage = {
          campusName: data.data.campusName,
          studentName: data.data.studentName,
          returnNormalAmount: data.data.returnNormalAmount,
          returnAmount: data.data.returnAmount,
          returnSpecialAmount: data.data.returnSpecialAmount,
          offlineRefundAmount: data.data.offlineRefundAmount,
        };
        tempWithdrawReason = {
          returnReason: data.data.returnReason,
          returnType: data.data.returnType,
          accountName: data.data.accountName,
          account: data.data.account,
        };
        tempWithdrawCertificate = data.data.refundEvidenceVos;
        tempReasonData = data.data.backTrackDetailVos.filter((value) => value.refundAmount > 0);
        tempAttrSchoolData = data.data.refundDutyCampuses;
        tempAttrTypeData = data.data.refundDutyPersons;
        tempOneToOneData = data.data.refundDutyTeachers;
        if (data.data.refundDutyVos && data.data.refundDutyVos.length) {
          data.data.refundDutyVos.forEach((value) => {
            value.distributeType === 'USER' ? tempPersonData.push(value) : tempClassSchoolData.push(value);
          });
        }
      }
      this.initReasonColumn();
      this.initDutyColumn();
      this.initClassColumn();
      this.setState({
        withdrawMessage: tempWithdrawMessage,
        withdrawReason: tempWithdrawReason,
        withdrawCertificate: tempWithdrawCertificate,
        reasonData: tempReasonData,
        attrSchoolData: tempAttrSchoolData,
        attrTypeData: tempAttrTypeData,
        oneToOneData: tempOneToOneData,
        classSchoolData: tempClassSchoolData,
        personData: tempPersonData,
      });
    } catch (e) {
      message.error(e);
    }
  };

  render() {
    const {
      withdrawMessage,
      withdrawReason,
      withdrawCertificate,
      reasonColumns,
      reasonData,
      attrSchoolColumns,
      attrSchoolData,
      attrTypeColumns,
      attrTypeData,
      oneToOneColumns,
      oneToOneData,
      classSchoolColumns,
      classSchoolData,
      personColumns,
      personData,
    } = this.state;
    const { onCancel, onClose } = this.props;
    return (
      <Modal
        title="退费详情"
        centered
        visible={true}
        toggle={true}
        cancelBtnShow={false}
        onCancel={onCancel}
        onOk={onClose}
      >
        <div className={styles.content}>
          <Row>
            <Col span={8}>
              <span>退费学习中心：{withdrawMessage.campusName}</span>
            </Col>
            <Col span={8}>
              <span>退费学生：{withdrawMessage.studentName}</span>
            </Col>
            <Col span={8}>
              <span>线下特殊退费：{withdrawMessage.offlineRefundAmount}元</span>
            </Col>
          </Row>
          <Row>
            <Col span={8}>
              <span>正常退费：{withdrawMessage.returnNormalAmount}元</span>
            </Col>
            <Col span={8}>
              <span>超额退费：{withdrawMessage.returnSpecialAmount}元</span>
            </Col>
            <Col span={8}>
              <span>退费总额：{withdrawMessage.returnAmount}元</span>
            </Col>
          </Row>
        </div>
        <div className={styles.content}>
          <div className={styles.contentTitle}>退费原因</div>
          <div className={styles.contentPanel}>
            <Row>
              <Col span={24}>退费原因：{withdrawReason.returnReason}</Col>
            </Row>
            <Row>
              <Col span={24}>详细原因：{withdrawReason.returnType}</Col>
            </Row>
            <Row>
              <Col span={24}>
                <div>
                  <span>退费凭证：</span>
                  {withdrawCertificate && withdrawCertificate.length > 0 ? (
                    withdrawCertificate.map((value, key) => {
                      return (
                        <a className={styles.fileStyle} key={key} href={value.evidencePath} target="_blank">
                          {value.fileName}
                        </a>
                      );
                    })
                  ) : (
                    <span>暂无</span>
                  )}
                </div>
              </Col>
            </Row>
            <Row>
              <Col span={12}>账户名：{withdrawReason.accountName}</Col>
              <Col span={12}>账号：{withdrawReason.account}</Col>
            </Row>
            <div className={styles.tablePanel}>
              <Table pagination={false} columns={reasonColumns} dataSource={reasonData} />
            </div>
          </div>
        </div>
        <div className={styles.content}>
          <div className={styles.contentTitle}>退费责任归属</div>
          <div className={styles.contentPanel}>
            <Row>
              <Col span={7}>
                <Table pagination={false} columns={attrSchoolColumns} dataSource={attrSchoolData} />
              </Col>
              <Col span={8} offset={1}>
                <Table pagination={false} columns={attrTypeColumns} dataSource={attrTypeData} />
              </Col>
              <Col span={7} offset={1}>
                <Table pagination={false} columns={oneToOneColumns} dataSource={oneToOneData} />
              </Col>
            </Row>
          </div>
        </div>
        <div className={styles.content}>
          <div className={styles.contentTitle}>退费责任科次</div>
          <div className={styles.contentPanel}>
            <Row>
              <Col span={11}>
                <Table pagination={false} columns={classSchoolColumns} dataSource={classSchoolData} />
              </Col>
              <Col span={11} offset={2}>
                <Table pagination={false} columns={personColumns} dataSource={personData} />
              </Col>
            </Row>
          </div>
        </div>
      </Modal>
    );
  }
}

export default WithdrawInfo;
