import React from 'react';
import PropTypes from 'prop-types';
import { connect } from 'dva';
import { browserHistory } from 'dva/router';
import { TextLink } from 'COMPONENT';
import styles from './styles.less';

// global store 上合同列表 tab 的id
const listContractTabPaneId = 'function/contract/listContract.html';

class ModalFooter extends React.Component {
  /**
   * 打开分配金额弹框
   */
  openAssignMoney = () => {
    const id = this.props.info.contractId;

    //先关闭弹框再跳转
    this.props.onLinkClick(() => {
      //要先转码，否则OldPage解析路由会把 contractId 参数弄丢
      browserHistory.push(`/oldPage?src=${listContractTabPaneId}?${encodeURIComponent('contractId=' + id)}`);

      //如果已存在合同列表tab，要刷新tab才会打开弹框，因为旧页面绑定的是页面onload事件
      if (this.props.hasListContract) {
        this.props.dispatch({
          type: 'main/refreshTab',
          payload: {
            id: listContractTabPaneId,
          },
        });
      }
    });
  };

  render() {
    return (
      <div className={styles.footer}>
        <p>合同的科次业绩必须同时满足以下条件方可进行科次业绩分配</p>
        {/* <p>1. 合同必须经过家长确认</p> */}
        <p>1. 合同产品没有发起退费</p>
        <p>
          2. 合同产品的待分配资金为0（如需调整或查看，请点击
          <TextLink onClick={this.openAssignMoney}>合同资金分配</TextLink>）
        </p>
      </div>
    );
  }
}

ModalFooter.propTypes = {
  info: PropTypes.object.isRequired,
  onLinkClick: PropTypes.func.isRequired,
};

const mapStateToProps = (state) => {
  return {
    hasListContract: state.main.tabpanes.some((pane) => pane.id === listContractTabPaneId),
  };
};

export default connect(mapStateToProps, null)(ModalFooter);
