import React from 'react';
import PropTypes from 'prop-types';
import { Tabs } from 'antd';
import { Modal } from 'COMPONENT';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

import styles from './styles.less';

const TabPane = Tabs.TabPane;

const tabMap = {
  OPS_AREA: {
    key: '1',
    label: '业绩分配',
  },
  OPS_HISTORY: {
    key: '2',
    label: '业绩分配记录',
  },
};

class ModalWrapper extends React.Component {
  constructor(props) {
    super(props);

    this.opsAreaInstance = void 0;
  }

  onOk = () => {
    if (!this.opsAreaInstance) {
      return;
    }

    const { isValid, result, errMsg } = this.opsAreaInstance.getCurrentResult();

    if (isValid) {
      this.props.onSave(result);
    } else {
      showMessage(errMsg);
    }
  };

  render() {
    const {
      loading,
      opsEditable,
      onClose,
      header: Header,
      footer: Footer,
      opsHistory: OpsHistory,
      renderOpsArea,
    } = this.props;

    const { OPS_AREA: opsAreaTab, OPS_HISTORY: opsHistTab } = tabMap;

    return (
      <Modal
        className={styles.customModal}
        widthSize="XL"
        title="科次分配"
        loading={loading}
        okBtnShow={opsEditable}
        onOk={this.onOk}
        okText="提交"
        closable
        onClose={onClose}
      >
        {Header}
        <Tabs defaultActiveKey={opsAreaTab.key}>
          <TabPane tab={opsAreaTab.label} key={opsAreaTab.key}>
            {renderOpsArea({
              refCb: (item) => (this.opsAreaInstance = item),
            })}
            {Footer}
          </TabPane>
          <TabPane tab={opsHistTab.label} key={opsHistTab.key}>
            {OpsHistory}
          </TabPane>
        </Tabs>
      </Modal>
    );
  }
}

ModalWrapper.propTypes = {
  loading: PropTypes.bool.isRequired,
  opsEditable: PropTypes.bool.isRequired,
  onClose: PropTypes.func.isRequired,
  onSave: PropTypes.func.isRequired,
  header: PropTypes.element.isRequired,
  footer: PropTypes.element.isRequired,
  opsHistory: PropTypes.element.isRequired,
  renderOpsArea: PropTypes.func.isRequired,
};

export default ModalWrapper;
