import React from 'react';
import PropTypes from 'prop-types';
import List from 'COMPONENT/List';
import { operationMap } from './common';
import styles from './styles.less';

const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L } = List;

class OpsHistory extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      pageNum: 1,
      pageSize: 10,
    };
  }

  /**
   * 翻页
   */
  onPageChange = ({ current, pageSize }) => {
    const newPagination = {
      pageNum: current,
      pageSize,
    };

    this.props.onPageChange({ ...newPagination });

    this.setState({
      ...newPagination,
    });
  };

  /**
   * 渲染科次业绩
   */
  renderAmountRate = (text, row, index) => {
    const num = Number(text);
    const oprType = row.operation;
    const { radix = 1 } = operationMap[oprType];

    return num * radix;
  };

  /**
   * 渲染操作类型
   */
  renderOperationName = (text, row, index) => {
    const { label = '' } = operationMap[text];

    return label;
  };

  formatColumn = () => {
    return [
      {
        dataIndex: 'productName',
        title: '产品名称',
        width: COL_WIDTH_L,
      },
      {
        dataIndex: 'productCharacterName',
        title: '产品性质',
        width: 80,
      },
      {
        dataIndex: 'subDistributeTypeName',
        title: '业绩归属类型',
        width: 105,
      },
      {
        dataIndex: 'distributeBizName',
        title: '业绩归属',
        width: 80,
      },
      {
        dataIndex: 'amountRate',
        title: '科次业绩',
        width: 80,
        render: this.renderAmountRate,
      },
      {
        dataIndex: 'operation',
        title: '操作类型',
        width: 80,
        render: this.renderOperationName,
      },
      {
        dataIndex: 'createTime',
        title: '创建日期',
        width: 100,
        render: (date) => {
          const dateNum = Number(date);

          return isNaN(dateNum) ? '-' : new Date(date).toLocaleString('zh', { hour12: false });
        },
      },
      {
        dataIndex: 'createUserName',
        title: '操作人',
        width: COL_WIDTH_S,
        render: (text) => {
          return text || '系统操作';
        },
      },
    ];
  };

  render() {
    const { list, totalNum } = this.props;
    const { pageNum, pageSize } = this.state;

    return (
      <div className={styles.customListHeight}>
        <List
          dataSource={list}
          columns={this.formatColumn()}
          pagination={{
            current: pageNum,
            pageSize,
            total: totalNum,
          }}
          onChange={this.onPageChange}
        />
      </div>
    );
  }
}

OpsHistory.propTypes = {
  list: PropTypes.array.isRequired,
  totalNum: PropTypes.number.isRequired,
  onPageChange: PropTypes.func.isRequired,
};

export default OpsHistory;
