import React from 'react';
import Modal from 'COMPONENT/Modal';
import loaders from '../loaders';
import styles from './index.less';
import Icon from 'COMPONENT/Icon';

class ContractProductDistributionInfoRow extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      showInfo: false,
    };
  }

  /**
   * 打开科次业绩分配统计
   */
  openInfo = () => {
    this.setState({ showInfo: !this.state.showInfo });
  };

  render() {
    const { contractProductDistributionInfo, columns } = this.props;
    const { distributionInfo } = contractProductDistributionInfo;
    const {
      campusPendingAmount,
      campusPendingAmountRate,
      incomeDistributionList,
      staffPendingAmount,
      staffPendingAmountRate,
    } = distributionInfo;
    const { showInfo } = this.state;
    return (
      <div>
        <div className={styles.row}>
          {columns.map((column, index) => {
            if (index === 0) {
              if (showInfo === true) {
                return <Icon type="triangle_down" style={{ width: column.width + 'px' }} onClick={this.openInfo} />;
              } else {
                return <Icon type="triangle_up" style={{ width: column.width + 'px' }} onClick={this.openInfo} />;
              }
            }
            return (
              <div style={{ width: column.width + 'px' }} className={styles.column}>
                {contractProductDistributionInfo[column.dataIndex]}
              </div>
            );
          })}
        </div>
        {Object.keys(distributionInfo).length !== 0 && showInfo === true && (
          <div className={styles.tables}>
            <div className={styles.unallocated}>
              <div className={styles.unallocatedText}>待分配</div>
              <table>
                <tr>
                  <th>归属类型</th>
                  <th>待分配金额</th>
                  <th>待分配科次</th>
                </tr>
                <tr>
                  <td>学习中心</td>
                  <td>{campusPendingAmount}</td>
                  <td>{campusPendingAmountRate}</td>
                </tr>
                <tr>
                  <td>个人</td>
                  <td>{staffPendingAmount}</td>
                  <td>{staffPendingAmountRate}</td>
                </tr>
              </table>
            </div>
            <div className={styles.assigned}>
              <div className={styles.assignedText}>已分配</div>
              <table>
                <tr>
                  <th>归属类型</th>
                  <th>归属</th>
                  <th>已分配金额</th>
                  <th>已分配科次</th>
                </tr>
                {incomeDistributionList.map((ncomeDistribution) => {
                  const { baseDistributeType, distributeOrgName, distributeStaffName, amount, amountRate } =
                    ncomeDistribution;
                  return (
                    <tr>
                      <td>{baseDistributeType === 'CAMPUS' ? '学习中心' : '个人'}</td>
                      <td>{baseDistributeType === 'CAMPUS' ? distributeOrgName : distributeStaffName}</td>
                      <td>{amount}</td>
                      <td>{amountRate}</td>
                    </tr>
                  );
                })}
              </table>
            </div>
          </div>
        )}
        {Object.keys(distributionInfo).length === 0 && showInfo === true && (
          <div className={styles.tip}>该产品未分配资金，请收款后在【合同列表】中进行资金分配！</div>
        )}
      </div>
    );
  }
}

class ProductDetail extends React.Component {
  constructor(props) {
    super(props);

    this.state = {};
    this.columns = [
      { title: '', dataIndex: '', width: 38 },
      { title: '产品类型', dataIndex: 'productTypeName', width: 93 },
      { title: '产品性质', dataIndex: 'productCharacterName', width: 93 },
      { title: '产品名称', dataIndex: 'productName', width: 205 },
      { title: '合同编号', dataIndex: 'contractId', width: 140 },
      { title: '合同类型', dataIndex: 'contractTypeName', width: 95 },
      { title: '合同日期', dataIndex: 'contractDate', width: 115 },
      { title: '应收金额', dataIndex: 'contractAmount', width: 81 },
      { title: '已分配资金', dataIndex: 'paidAmount', width: 81 },
      { title: '签订人', dataIndex: 'signStaffName', width: 66 },
    ];
  }

  componentDidMount() {}

  render() {
    const { contractProductDistributionInfoList, closeDetailModal } = this.props;
    return (
      <Modal title="产品详情" width={1140} onCancel={closeDetailModal} onOk={closeDetailModal}>
        <div className={styles.header + ' ' + styles.row}>
          {this.columns.map((column) => {
            return (
              <div style={{ width: column.width + 'px' }} className={styles.column}>
                {column.title}
              </div>
            );
          })}
        </div>
        {contractProductDistributionInfoList.map((contractProductDistributionInfo) => {
          return (
            <ContractProductDistributionInfoRow
              contractProductDistributionInfo={contractProductDistributionInfo}
              columns={this.columns}
            />
          );
        })}
      </Modal>
    );
  }
}

export default ProductDetail;
