import React from 'react';
import style from './styles.module.less';
import PropTypes from 'prop-types';
import toolbarStyle from '../components/toolbar/styles.module.less';
import classNames from 'classnames';
import { Button } from 'antd';
import TipText from '../components/TipText';
import SelectedCouponList from '../components/SelectedCouponList';
import { numFormatToMoney } from '../utils';

/**
 * footer
 *
 * @constructor
 */
const FooterBar = (props) => {
  return (
    <div className={classNames(style.FooterBar, toolbarStyle.toolbar)}>
      <div className={style.FooterBarAction}>
        {props.canUseCoupon && (
          <div className={style.FooterBarActionWrap}>
            <Button
              className={style.FooterBarCouponBtn}
              type="primary"
              size="large"
              onClick={props.onSelectCouponClick}
              disabled={props.couponDisabled}
            >
              选择优惠券
            </Button>
            {props.couponList && props.couponList.length > 0 ? (
              <SelectedCouponList couponList={props.couponList} />
            ) : (
              <div className={style.FooterBarNoCoupon}>暂未使用优惠券</div>
            )}
          </div>
        )}
      </div>

      <div className={style.FooterBarInfoGroups}>
        <div className={style.FooterBarInfo}>
          <span className={style.FooterBarInfoLabel}>合同款合计：</span>
          <span className={style.FooterBarInfoContent}>￥{numFormatToMoney(props.totalAmount)}</span>
        </div>
        <div className={style.FooterBarInfo}>
          <span className={style.FooterBarInfoLabel}>优惠合计：</span>
          <span className={style.FooterBarInfoContent}>￥{numFormatToMoney(props.promotionAmount)}</span>
        </div>
        <div className={style.FooterBarInfo}>
          <span className={style.FooterBarInfoLabel}>
            <b>合同应收款合计：</b>
          </span>
          <span className={style.FooterBarInfoContent}>
            <TipText fontSize={20} fontWeight="bold">
              ￥{numFormatToMoney(props.realAmount)}
            </TipText>
          </span>
        </div>
      </div>

      <div className={style.FooterBarBtnGroups}>
        {props.canSaveAndAssign && (
          <button onClick={props.onSaveAndAssign} className={classNames(style.FooterBarBtn, style.color3)}>
            <span>保存并分配课时</span>
          </button>
        )}
        <button onClick={props.onSave} className={classNames(style.FooterBarBtn, style.color2)}>
          <span>保存并跳转</span>
          <span>合同列表</span>
        </button>
        {props.realAmount > 0 && (
          <button onClick={props.onSaveAndPay} className={classNames(style.FooterBarBtn, style.color1)}>
            <span>生成合同并支付</span>
          </button>
        )}
      </div>
    </div>
  );
};

FooterBar.propTypes = {
  canUseCoupon: PropTypes.bool.isRequired,
  couponList: PropTypes.array.isRequired,
  couponDisabled: PropTypes.bool,
  totalAmount: PropTypes.number.isRequired,
  promotionAmount: PropTypes.number.isRequired,
  realAmount: PropTypes.number.isRequired,
  onSelectCouponClick: PropTypes.func.isRequired,
  canSaveAndAssign: PropTypes.bool.isRequired,
  onSave: PropTypes.func.isRequired,
  onSaveAndPay: PropTypes.func.isRequired,
  onSaveAndAssign: PropTypes.func.isRequired,
};

FooterBar.defaultProps = {
  couponDisabled: true,
};

export default FooterBar;
