import React from 'react';
import { clone } from 'lodash';
import { connect } from 'dva';
import { Row, Col } from 'antd';
import PropTypes from 'prop-types';
import { runFunc, loginUser } from 'UTIL/common';
import style from './styles.module.less';
import { o2mTypeDict } from 'CONST/dicts/product';
import { storeName } from '../../store/storeCfg';
import { Button, Select, Search2, Loading, Input } from 'COMPONENT';
import { getRelationsDict, getRelationPayload } from 'MODEL/dict/utils/index';
import { productType, productSeries, classType, classLevel } from 'MODEL/dict/utils/config';
import {
  SMALL_CLASS,
  ECS_CLASS,
  productTypeAllow,
  OTHERS,
  ONE_ON_ONE_COURSE,
  LECTURE,
  TWO_TEACHER,
  ONE_ON_MANY,
} from '../config';

// 空的搜索条件
const emptySKey = {
  productName: null,
  className: null,
  assistTeacherId: null,
  campusId: null,
  courseSeriesId: null,
  classTypeId: null,
  classLevelId: null,
  gradeId: null,
  subjectId: null,
  productVersionId: null,
  quarterId: [],
  phaseId: null,
  productGroupId: null,
  teacherId: null,
  otmType: null,
  characterId: null,
  otherOfCategoryId: null,
  otherCategoryId: null,
};
Object.freeze(emptySKey);

class Search extends React.Component {
  static PropTypes = {
    initSKey: PropTypes.object,
    onSearch: PropTypes.func.isRequired,
    isAlready: PropTypes.bool,
  };

  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      ...emptySKey,
    };
    this.isInit = false;
    this.defaultSKey = null;
    this.isTwoTeacher = loginUser.isTwoTeacherSym();

    // 产品类型关联关系type、key对应关系配置
    this.productTypeTreeMap = {
      [productType]: 'productType',
      [productSeries]: 'courseSeriesId',
      [classType]: 'classTypeId',
      [classLevel]: 'classLevel',
    };
  }

  componentDidUpdate(prevProps) {
    if (!this.isInit && !prevProps.initSKey && this.props.initSKey) {
      const payload = this.formatSKey();

      this.isInit = true;
      this.defaultSKey = payload;
      this.setState({ ...payload });
    }
  }

  // 平铺项数据
  getSearchCfg = () => {
    if (this.isTwoTeacher) {
      return this.formatTwoTeacherCfg();
    }

    return this.formatXingHuoCfg();
  };

  /**
   * 星火配置
   * @return {*[]|Array}
   */
  formatXingHuoCfg = () => {
    const {
      productTypeTree,
      selectProductType,
      weekDayDict,
      phaseDict,
      yearDict,
      quarterDict,
      subjectDict,
      gradeDict,
      productCharacter,
      otherCategoryDict,
      otherOfCategoryDict,
      isEcs,
    } = this.props;

    const courseSeriesDict = getRelationsDict(productSeries)(productTypeTree, selectProductType);
    const classTypeDict = getRelationsDict(classType)(productTypeTree, selectProductType);
    const classLevelDict = getRelationsDict(classLevel)(productTypeTree, selectProductType, this.state.classTypeId);

    const _phaseDict = (weekDayDict || []).concat(phaseDict || []);

    // 小班过滤幼儿园和毕业, 以及默认值是幼儿园和小班是需要变为全部
    let _gradeDict = gradeDict;
    if (selectProductType === SMALL_CLASS) {
      _gradeDict = gradeDict.filter(({ text }) => !['幼儿园', '毕业'].includes(text));
    }

    // 精英班没有 幼儿园
    if (selectProductType === ECS_CLASS) {
      _gradeDict = gradeDict.filter(({ text }) => !['幼儿园'].includes(text));
    }

    if ([SMALL_CLASS, TWO_TEACHER].includes(selectProductType)) {
      return [
        { label: '班级类型', dict: classTypeDict, field: 'classTypeId', value: this.state.classTypeId },
        { label: '产品系列', dict: courseSeriesDict, field: 'courseSeriesId', value: this.state.courseSeriesId },
        { label: '产品性质', dict: productCharacter, field: 'characterId', value: this.state.characterId },
        {
          label: '班级层次',
          dict: classLevelDict,
          field: 'classLevelId',
          value: this.state.classLevelId,
          showMoreButtons: true,
        },
        { label: '科目', dict: subjectDict, field: 'subjectId', value: this.state.subjectId, showMoreButtons: true },
        { label: '年级', dict: _gradeDict, field: 'gradeId', value: this.state.gradeId },
        { label: '产品季度', dict: quarterDict, field: 'quarterId', value: this.state.quarterId, type: 'multi' },
        { label: '产品年份', dict: yearDict, field: 'productVersionId', value: this.state.productVersionId },
        { label: '期数', dict: _phaseDict, field: 'phaseId', value: this.state.phaseId },
      ];
    }

    if ([ONE_ON_ONE_COURSE].includes(selectProductType)) {
      return [
        { label: '产品系列', dict: courseSeriesDict, field: 'courseSeriesId', value: this.state.courseSeriesId },
        { label: '产品性质', dict: productCharacter, field: 'characterId', value: this.state.characterId },
        { label: '年级', dict: _gradeDict, field: 'gradeId', value: this.state.gradeId, showMoreButtons: true },
        { label: '产品年份', dict: yearDict, field: 'productVersionId', value: this.state.productVersionId },
      ];
    }

    if ([ONE_ON_MANY].includes(selectProductType)) {
      return [
        { label: '产品系列', dict: courseSeriesDict, field: 'courseSeriesId', value: this.state.courseSeriesId },
        { label: '产品性质', dict: productCharacter, field: 'characterId', value: this.state.characterId },
        { label: '一对多类型', dict: o2mTypeDict, field: 'otmType', value: this.state.otmType },
        { label: '年级', dict: _gradeDict, field: 'gradeId', value: this.state.gradeId, showMoreButtons: true },
        { label: '产品年份', dict: yearDict, field: 'productVersionId', value: this.state.productVersionId },
      ];
    }

    if ([LECTURE].includes(selectProductType)) {
      return [
        { label: '年级', dict: _gradeDict, field: 'gradeId', value: this.state.gradeId, showMoreButtons: true },
        { label: '产品年份', dict: yearDict, field: 'productVersionId', value: this.state.productVersionId },
      ];
    }

    if ([OTHERS].includes(selectProductType)) {
      let _otherOfCategoryDict = otherOfCategoryDict;

      // if (isEcs) {
      //   _otherOfCategoryDict = otherOfCategoryDict.filter((item) => ['精英班', '其他'].includes(item.text));
      // }

      if (loginUser.isPeiYouSym()) {
        _otherOfCategoryDict = otherOfCategoryDict.filter((item) => ['小班', '讲座', '其他'].includes(item.text));
      }

      const mainConfigListForOthers = [
        { label: '产品年份', dict: yearDict, field: 'productVersionId', value: this.state.productVersionId },
        { label: '费用类型', dict: otherCategoryDict, field: 'otherCategoryId', value: this.state.otherCategoryId },
      ];

      return isEcs
        ? mainConfigListForOthers
        : mainConfigListForOthers.concat([
            {
              label: '所属产品类型',
              dict: _otherOfCategoryDict,
              field: 'otherOfCategoryId',
              value: this.state.otherOfCategoryId,
            },
          ]);
    }

    if ([ECS_CLASS].includes(selectProductType)) {
      return [
        { label: '年级', dict: _gradeDict, field: 'gradeId', value: this.state.gradeId, showMoreButtons: true },
        { label: '产品年份', dict: yearDict, field: 'productVersionId', value: this.state.productVersionId },
        { label: '产品季度', dict: quarterDict, field: 'quarterId', value: this.state.quarterId, type: 'multi' },
      ];
    }

    return [];
  };

  /**
   * 双师配置
   * @return {*[]|Array}
   */
  formatTwoTeacherCfg = () => {
    const {
      productTypeTree,
      selectProductType,
      weekDayDict,
      phaseDict,
      yearDict,
      quarterDict,
      subjectDict,
      gradeDict,
      productCharacter,
      otherCategoryDict,
      otherOfCategoryDict,
      smallClassProductGroupDict,
      isEcs,
    } = this.props;

    const courseSeriesDict = getRelationsDict(productSeries)(productTypeTree, selectProductType);
    const classTypeDict = getRelationsDict(classType)(productTypeTree, selectProductType);
    const classLevelDict = getRelationsDict(classLevel)(productTypeTree, selectProductType, this.state.classTypeId);

    const _phaseDict = (weekDayDict || []).concat(phaseDict || []);

    // 小班过滤幼儿园和毕业, 以及默认值是幼儿园和小班是需要变为全部
    let _gradeDict = gradeDict;
    if (selectProductType === SMALL_CLASS) {
      _gradeDict = gradeDict.filter(({ text }) => !['幼儿园', '毕业'].includes(text));
    }

    // 精英班没有 幼儿园
    if (selectProductType === ECS_CLASS) {
      _gradeDict = gradeDict.filter(({ text }) => !['幼儿园'].includes(text));
    }

    if ([TWO_TEACHER].includes(selectProductType)) {
      return [
        { label: '科目', dict: subjectDict, field: 'subjectId', value: this.state.subjectId, showMoreButtons: true },
        { label: '年级', dict: _gradeDict, field: 'gradeId', value: this.state.gradeId },
        { label: '产品季度', dict: quarterDict, field: 'quarterId', value: this.state.quarterId, type: 'multi' },
        { label: '产品年份', dict: yearDict, field: 'productVersionId', value: this.state.productVersionId },
        { label: '产品系列', dict: courseSeriesDict, field: 'courseSeriesId', value: this.state.courseSeriesId },
        { label: '产品性质', dict: productCharacter, field: 'characterId', value: this.state.characterId },
        { label: '班级类型', dict: classTypeDict, field: 'classTypeId', value: this.state.classTypeId },
        {
          label: '班级层次',
          dict: classLevelDict,
          field: 'classLevelId',
          value: this.state.classLevelId,
          showMoreButtons: true,
        },
        { label: '期数', dict: _phaseDict, field: 'phaseId', value: this.state.phaseId },
      ];
    }

    if ([SMALL_CLASS].includes(selectProductType)) {
      return [
        {
          label: '产品组',
          dict: smallClassProductGroupDict,
          field: 'productGroupId',
          value: this.state.productGroupId,
        },
        { label: '科目', dict: subjectDict, field: 'subjectId', value: this.state.subjectId, showMoreButtons: true },
        { label: '年级', dict: _gradeDict, field: 'gradeId', value: this.state.gradeId },
        { label: '产品季度', dict: quarterDict, field: 'quarterId', value: this.state.quarterId, type: 'multi' },
        { label: '产品年份', dict: yearDict, field: 'productVersionId', value: this.state.productVersionId },
        { label: '产品系列', dict: courseSeriesDict, field: 'courseSeriesId', value: this.state.courseSeriesId },
        { label: '产品性质', dict: productCharacter, field: 'characterId', value: this.state.characterId },
        { label: '班级类型', dict: classTypeDict, field: 'classTypeId', value: this.state.classTypeId },
        {
          label: '班级层次',
          dict: classLevelDict,
          field: 'classLevelId',
          value: this.state.classLevelId,
          showMoreButtons: true,
        },
        { label: '期数', dict: _phaseDict, field: 'phaseId', value: this.state.phaseId },
      ];
    }

    if ([OTHERS].includes(selectProductType)) {
      let _otherOfCategoryDict = otherOfCategoryDict.filter((item) =>
        ['小班', '双师', '讲座', '其他'].includes(item.text),
      );

      return [
        { label: '产品年份', dict: yearDict, field: 'productVersionId', value: this.state.productVersionId },
        { label: '费用类型', dict: otherCategoryDict, field: 'otherCategoryId', value: this.state.otherCategoryId },
        {
          label: '所属产品类型',
          dict: _otherOfCategoryDict,
          field: 'otherOfCategoryId',
          value: this.state.otherOfCategoryId,
        },
      ];
    }

    if ([LECTURE].includes(selectProductType)) {
      return [
        { label: '年级', dict: _gradeDict, field: 'gradeId', value: this.state.gradeId, showMoreButtons: true },
        { label: '产品年份', dict: yearDict, field: 'productVersionId', value: this.state.productVersionId },
      ];
    }

    return [];
  };

  /**
   * 默认显示行数
   * @return {number}
   */
  formatSearchRowNum = () => {
    if (!this.isTwoTeacher) {
      return 7;
    }

    return [SMALL_CLASS].includes(this.props.selectProductType) ? 5 : 4;
  };

  onSearchChange = (params) => {
    this.setState(
      (prevState) => {
        let payload = {
          className: prevState.className,
          assistTeacherId: prevState.assistTeacherId,
          blCampusId: prevState.blCampusId,
          ...params,
        };

        if (prevState.classTypeId !== params.classTypeId) {
          payload = {
            ...payload,
            ...getRelationPayload('classTypeId', params.classTypeId, {
              type: classType,
              dictTree: this.props.productTypeTree,
              state: this.props,
              map: this.productTypeTreeMap,
            }),
          };
        } else if (prevState.productType !== params.productType) {
          payload = {
            ...payload,
            assistTeacherId: null,
            ...getRelationPayload('productType', params.productType, {
              type: productType,
              dictTree: this.props.productTypeTree,
              state: this.state,
              map: this.productTypeTreeMap,
            }),
          };
        }

        return payload;
      },
      () => {
        this.submit();
      },
    );
  };

  submit = () => {
    const { onSearch } = this.props;
    let params = clone(this.state);

    delete params.loading;
    delete params.searchVal;
    delete params.productTypeId;

    runFunc(onSearch, {
      ...params,
    });
  };

  resetSKey = () => {
    const payload = this.formatSKey();

    this.setState(payload, () => {
      runFunc(this.props.onSearch, payload);
    });
  };

  /**
   * 格式化最新的搜索条件
   * tips：因componentDidUpdate缺陷，必须这样做
   */
  formatSKey = () => {
    return {
      ...emptySKey,
      ...(this.defaultSKey || {}),
      ...(this.props.initSKey || {}),
    };
  };

  // 根据类型返回不同的搜索框
  getSearchDom() {
    const { selectProductType, campusDict, teacherDict } = this.props;

    // 细粒化查询表单节点
    const { productName, className, teacherId, campusId } = this.state;

    const productNameDom = (
      <Input
        placeholder="产品名称"
        value={productName}
        onChange={(e) => this.setState({ productName: e.target.value })}
      />
    );
    const classNameDom = (
      <Input placeholder="班级名称" value={className} onChange={(e) => this.setState({ className: e.target.value })} />
    );
    const mainTeacherNameDom = (
      <Select
        placeholder="班级老师"
        dict={teacherDict}
        showSearch
        value={teacherId}
        onChange={(val) => {
          this.setState({ teacherId: val });
        }}
      />
    );
    const blCampusIdDom = (
      <Select
        placeholder="班级学习中心"
        dict={campusDict}
        showSearch
        value={campusId}
        onChange={(val) => {
          this.setState({ campusId: val });
        }}
      />
    );

    const dom0 = (
      <Row className={style.searchBarKey}>
        <Col span={6}>{productNameDom}</Col>
        <Col span={6}>{classNameDom}</Col>
        <Col span={6}>{mainTeacherNameDom}</Col>
        <Col span={6}>{blCampusIdDom}</Col>
      </Row>
    );

    const dom1 = (
      <Row className={style.searchBarKey}>
        <Col span={6}>{productNameDom}</Col>
      </Row>
    );

    // 根据不同系统 与不同 产品类型进行拼凑查询表单
    switch (selectProductType) {
      case SMALL_CLASS:
      case TWO_TEACHER:
        return dom0;
      case ONE_ON_MANY:
      case ONE_ON_ONE_COURSE:
      case OTHERS:
      case ECS_CLASS:
        return dom1;
      case LECTURE:
        return (
          <Row className={style.searchBarKey}>
            <Col span={6}>{productNameDom}</Col>
            <Col span={6}>{blCampusIdDom}</Col>
          </Row>
        );
      default:
        return null;
    }
  }

  render() {
    return (
      <div className={style.Search}>
        <div className={style.searchBar}>
          {this.getSearchDom()}
          <div className={style.searchBarBtn}>
            <Button type="primary" onClick={this.submit}>
              查询
            </Button>
            <Button onClick={this.resetSKey}>重置</Button>
          </div>
        </div>
        <Search2
          key={this.props.selectProductType}
          style={{ marginTop: 20 }}
          config={this.getSearchCfg()}
          onChange={this.onSearchChange}
          rowNum={this.formatSearchRowNum()}
          defaultExpand={false}
        />
        <Loading className={style.loading} loading={this.state.loading} />
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  const { dict, account, signContract } = state;
  const {
    year,
    grade = [],
    subject = [],
    twoTcMainTc,
    twoTcAuxTc,
    smallClassMainTc,
    smallClassAuxTc,
    phase = [],
    weekDay = [],
    quarter = [],
    productGroup = [],
    branchAllCampus = [],
    productCharacter = [],
    other_category,
    other_of_category,
  } = dict;

  let productTypeTreeStorage = null;
  if (!productTypeTreeStorage && dict.productTypeTree) {
    productTypeTreeStorage = dict.productTypeTree.filter(
      (item) => productTypeAllow.includes(item.val) || item.val === ECS_CLASS,
    );
  }

  const { serverDate } = account.user;
  const yearDict = year ? year.filter((item) => Number(item.text) >= new Date(serverDate).getFullYear() - 1) : [];
  const smallClassProductGroupDict = productGroup ? productGroup['SMALL_CLASS'] : null;

  return {
    twoTcMainTcDict: twoTcMainTc,
    twoTcAuxTcDict: twoTcAuxTc,
    smallClassMainTcDict: smallClassMainTc,
    smallClassAuxTcDict: smallClassAuxTc,
    campusDict: branchAllCampus,
    productTypeTree: productTypeTreeStorage || [],
    gradeDict: grade,
    subjectDict: subject,
    yearDict: yearDict,
    quarterDict: quarter,
    phaseDict: phase,
    weekDayDict: weekDay,
    serverDate,
    productCharacter,
    otherCategoryDict: other_category || [],
    otherOfCategoryDict: other_of_category || [],
    teacherDict: state[storeName].teacherDict,
    productGroupDict: productGroup,
    smallClassProductGroupDict,
    grade: signContract.grade || {},
    isEcs: signContract.isEcs,
    gradeId: signContract.gradeId,
  };
};

export default connect(mapStateToProps, null, null, { withRef: true })(Search);
