import React, { PropTypes } from 'react';
import { Tooltip } from 'antd';
import style from './styles.module.less';
import { Selected } from 'COMPONENT/ShoppingCart';
import { runFunc } from 'UTIL/common';
import { getTimeRange } from '../utils';
import { productTypeAllow, TWO_TEACHER } from '../config';

const MoreInfo = (props) => {
  return (
    <div className={style.MoreInfo}>
      <div className={style.MoreInfoLabel}>{props.label}</div>
      <div className={style.MoreInfoText}>
        <p>{props.text || '-'}</p>
      </div>
    </div>
  );
};
MoreInfo.propTypes = {
  label: PropTypes.string.isRequired,
  text: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
};

const Content = (props) => {
  const {
    productName,
    productTypeName,
    classTypeName,
    classLevelName,
    courseSeriesName,
    className,
    assistTeacherName,
    mainTeacherName,
    consumeCourseHours,
    totalCourseHours,
    validEndDate,
    startClassTime,
    classLength,
    price,
    productType,
    startClassDate,
    courseHours,
    _courseDate,
    campusName,
  } = props;

  const isShow = productTypeAllow.includes(productType);
  const teacherText =
    productType === TWO_TEACHER
      ? `${mainTeacherName || '暂无'}/${assistTeacherName || '暂无'}`
      : assistTeacherName || '暂无';
  return (
    <div className={style.Content} style={{ maxHeight: props.isShowMore ? 500 : 122 }}>
      <div className={style.title}>
        <Tooltip title="产品类型">
          {productTypeName && <span className={style.titleType}>{productTypeName}</span>}
        </Tooltip>
        <Tooltip title="产品系列">
          {isShow && courseSeriesName && <span className={style.titleType}>{courseSeriesName}</span>}
        </Tooltip>
        <Tooltip title="班级类型">
          {isShow && classTypeName && <span className={style.titleType2}>{classTypeName}</span>}
        </Tooltip>
        <Tooltip title="班级层次">
          {isShow && classLevelName && <span className={style.titleType3}>{classLevelName}</span>}
        </Tooltip>
      </div>
      <span>{productName}</span>
      <MoreInfo label="班级名称" text={className} />
      <MoreInfo label="班级类型" text={classTypeName} />
      <MoreInfo label="班级学习中心" text={campusName} />
      <MoreInfo label="单价" text={`¥${price}`} />
      <MoreInfo label="已上课时" text={`${consumeCourseHours || '-'}/${totalCourseHours || '-'}`} />
      <MoreInfo label="老师" text={teacherText} />
      <MoreInfo label="上课日期" text={_courseDate || startClassDate} />
      <MoreInfo label="上课时间" text={!startClassTime ? '暂无' : getTimeRange(startClassTime, classLength)} />
      <MoreInfo label="截止日期" text={validEndDate} />
    </div>
  );
};

const SelectedCard = (props) => {
  const { deleteBtnVisible, onDelete, index } = props;
  return (
    <Selected
      className={style.SelectedCard}
      deleteBtnVisible={deleteBtnVisible}
      onDelete={() => {
        runFunc(onDelete, index);
      }}
    >
      <Content {...props} />
    </Selected>
  );
};

export default SelectedCard;
