import React from 'react';
import PropTypes from 'prop-types';
import style from './styles.module.less';
import { ToBoSelected } from 'COMPONENT/ShoppingCart';
import { runFunc } from 'UTIL/common';
import { getTimeRange } from '../utils';
import { Tooltip } from 'antd';
import { SMALL_CLASS, productTypeAllow, TWO_TEACHER } from '../config';

const ClassInfo = (props) => {
  return (
    <span className={style.ClassInfo} style={{ display: props.text ? 'inline-block' : 'none' }}>
      {props.text}
    </span>
  );
};

const MoreInfo = (props) => {
  return (
    <span className={style.MoreInfo} style={{ display: props.text ? 'inline-block' : 'none' }}>
      <span>{props.label}：</span>
      <span>{props.text}</span>
    </span>
  );
};
MoreInfo.propTypes = {
  label: PropTypes.string.isRequired,
  text: PropTypes.string,
};

const Price = (props) => {
  return (
    <div className={style.Price}>
      <span>¥</span>
      <span>{props.num}</span>
    </div>
  );
};

const Content = (props) => {
  const {
    productName,
    className,
    productTypeName,
    classTypeName,
    classLevelName,
    courseSeriesName,
    nextCourseDate,
    index,
    assistTeacherName,
    mainTeacherName,
    consumeCourseHours,
    totalCourseHours,
    validEndDate,
    peopleQuantity,
    maxPeopleQuantity,
    startClassTime,
    classLength,
    price,
    productType,
    campusName,
  } = props;

  const title = {
    [SMALL_CLASS]: className,
    [TWO_TEACHER]: className,
  };

  const isShow = productTypeAllow.includes(productType);
  const teacherText =
    productType === TWO_TEACHER
      ? `${mainTeacherName || '暂无'}/${assistTeacherName || '暂无'}`
      : assistTeacherName || '暂无';
  return (
    <div className={style.Content}>
      <div className={style.mainContent}>
        <div className={style.title}>
          <Tooltip title="产品类型">
            {productTypeName && <span className={style.titleType}>{productTypeName}</span>}
          </Tooltip>
          <Tooltip title="产品系列">
            {isShow && courseSeriesName && <span className={style.titleType}>{courseSeriesName}</span>}
          </Tooltip>
          <Tooltip title="班级类型">
            {isShow && classTypeName && <span className={style.titleType2}>{classTypeName}</span>}
          </Tooltip>
          <Tooltip title="班级层次">
            {isShow && classLevelName && <span className={style.titleType3}>{classLevelName}</span>}
          </Tooltip>
          <span>{title[productType] || productName}</span>
        </div>
        <div className={style.body}>
          {props.classId ? (
            [
              <MoreInfo label="老师" text={teacherText} key={`${index}_teacherName`} />,
              <MoreInfo label="下次上课日期" text={nextCourseDate} key={`${index}_nextTime`} />,
            ]
          ) : (
            <ClassInfo text="暂无课程安排" />
          )}
        </div>
        <div className={style.footer}>
          <MoreInfo label="截止日期" text={validEndDate} />
          {props.classId && [
            <MoreInfo label="学习中心" text={campusName} key={`${index}_campusName`} />,
            <MoreInfo
              label="已上课时"
              text={`${consumeCourseHours || '-'}/${totalCourseHours || '-'}`}
              key={`${index}_courseTime`}
            />,
            <MoreInfo
              label="已报人数"
              text={`${peopleQuantity || '-'}/${maxPeopleQuantity || '-'}`}
              key={`${index}_regPeople`}
            />,
            <MoreInfo
              label="上课时间"
              text={getTimeRange(startClassTime, classLength)}
              key={`${index}_classTimeText`}
            />,
          ]}
        </div>
      </div>
      <div className={style.subContent}>{price && <Price num={price} />}</div>
    </div>
  );
};

const ToBoSelectedCard = (props) => {
  return (
    <ToBoSelected
      className={style.ToBoSelectedCard}
      onClick={() => {
        runFunc(props.onSelect, props.index);
      }}
    >
      <Content {...props} />
    </ToBoSelected>
  );
};

export default ToBoSelectedCard;
