import React from 'react';
import { Tabs, Row, Col } from 'antd';
import PropTypes from 'prop-types';
import { connect } from 'dva';
import Search from './Search';
import { getNextYear, getQuarterId, getQuarterIdByTwoTeacher, calcIsDrainageProduct } from './utils';
import { Deck, Modal, Loading, NoDataPlus } from 'COMPONENT';
import { getBLoYearId } from 'UTIL/dict';
import style from './styles.module.less';
import classCardRender from './ClassCardRender';
import selectedCardRender from './SelectedCardRender';
import SearchResult from './SearchResult';
import { storeName } from '../store/storeCfg';
import ModalSelectCourseTime from 'BLOC/ModalSelectCourseTime';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import showAlert from 'COMPONENT/utils/globalTips/showAlert';
import { findMiniClassWillBuyHour, findClassWillBuyHour, selectProductForContract } from 'SERVICE/contract';
import {
  systemProduct,
  productTypeAllow,
  SMALL_CLASS,
  ECS_CLASS,
  systemProductTab,
  OTHERS,
  quantityResetType,
  NoAcquiescenceType,
  TWO_TEACHER,
  LECTURE,
} from './config';
import { checkSamePrd } from '../store/storeUtils';
import { loginUser } from 'UTIL/common';

const { TabPane } = Tabs;

const SelectCourseDateTips = () => {
  return <span style={{ color: 'red' }}>注：补报历史课程最多只能报读今天之前15天内的课程</span>;
};

class ModalAddProduct extends React.Component {
  constructor(props) {
    super(props);

    const { defaultValue = {} } = props;
    const { productTypeId } = defaultValue || {};

    this.initValue = {
      page: 1,
      rows: 5,
      searchKey: {}, // 查询的值
    };
    this.state = {
      ...this.initValue,
      loading: false,
      initSKey: null,
      searchKey: null, // 筛选项
      selectProduct: {}, // 当前选择当产品
      modalVisible: false, // 课程时间选择
      selectProductType: productTypeId, // 已选产品类型

      productList: [], // 产品列表
      productListTotal: 0, // 产品总数
    };
    this.isTwoTeacher = loginUser.isTwoTeacherSym();
  }

  static propTypes = {
    defaultValue: PropTypes.object, // 默认值，包括产品类型等
    title: PropTypes.string, // 弹窗标题
    initSKey: PropTypes.object, // 需初始化的筛选项
    onClose: PropTypes.func.isRequired, // 关闭回调事件
  };

  static defaultProps = {
    title: null,
    initSKey: null,
    visible: true,
    productTypeId: undefined,
  };

  componentDidMount() {
    this.initContent();
  }

  initContent = async () => {
    await new Promise(this.initDict);
    await new Promise(this.initTeacher);
    await new Promise(this.initSearchKey);
    this.loadProductList(this.state.initSKey);
  };

  /**
   * 请求产品类型枚举
   */
  initDict = (resolve) => {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: [
          'productTypeTree',
          'grade',
          'quarter',
          'subject',
          'year',
          'phase',
          'weekDay',
          'branchAllCampus',
          'productCharacter',
          'other_category',
          'other_of_category',
          { name: 'productGroup', params: 'SMALL_CLASS' },
        ],
        callback: () => {
          resolve();
        },
      },
    });
  };
  /**
   * 请求老师枚举
   */
  initTeacher = (resolve) => {
    this.props.dispatch({
      type: `${storeName}/getTeachers`,
      payload: {
        callback: () => {
          resolve();
        },
      },
    });
  };

  /**
   * 初始化搜索条件
   */
  initSearchKey = (resolve) => {
    const { yearDict, serverDate, grade = {}, blCampusId, isEcs } = this.props;
    const { selectProductType } = this.state;
    const isSmallORTwoTeacher = [SMALL_CLASS, TWO_TEACHER].includes(selectProductType);

    // 类型为其它时不需要年级默认值
    let gradeId = NoAcquiescenceType.includes(selectProductType) ? null : this.props.gradeId;
    let productVersionId = getBLoYearId(yearDict, new Date(serverDate));
    let quarterId = [];

    // 小班与双师要进行升班处理
    if (isSmallORTwoTeacher) {
      gradeId = grade.gradeId || gradeId;
      quarterId = getQuarterId(serverDate);
      productVersionId = getNextYear(yearDict, serverDate) || productVersionId;

      if (this.isTwoTeacher) {
        quarterId = getQuarterIdByTwoTeacher(serverDate);
      }
    }

    // 小班时年级为幼儿园和毕业都改完全部
    if (
      selectProductType === SMALL_CLASS &&
      gradeId === grade.gradeId &&
      ['幼儿园', '毕业'].includes(grade.gradeName)
    ) {
      gradeId = null;
    }

    // 精英班时年级为幼儿园和毕业都改完全部
    if (selectProductType === ECS_CLASS && ['幼儿园', '毕业'].includes(grade.gradeName)) {
      gradeId = null;
    }

    // 其他类型 所属产品类型默认值为其他
    const otherOfCategoryId = !isEcs && OTHERS === selectProductType ? 'DAT0000000388' : null;

    this.setState(
      {
        initSKey: {
          gradeId,
          productVersionId,
          quarterId,
          otherOfCategoryId,
          campusId: blCampusId,
        },
      },
      () => {
        resolve();
      },
    );
  };

  // 根据查询条件查询产品
  loadProductList = async (params = {}) => {
    const { type } = this.getSystemProduct();
    const { isEcs, yearDict } = this.props;
    const { page, rows, searchKey } = this.state;

    const _params = {
      page,
      rows,
      isEcs,
      ...searchKey,
      ...params,
      productType: type,
    };

    if (type && yearDict.length > 0) {
      this.setState({ loading: true });

      const response = await selectProductForContract(_params).catch(() => {
        this.setState({ loading: false, productList: [], productListTotal: 0 });
      });
      const { datas, rowCount } = response || {};

      this.setState({ loading: false, productList: datas || [], productListTotal: rowCount || 0 });
    }
  };

  // 查询搜索
  handleSearch = (val) => {
    this.setState({ ...this.initValue, searchKey: val });
    this.loadProductList({ ...this.initValue, ...val });
  };

  // 选项卡切换回调
  handleTabsChange = (val) => {
    this.setState({ selectProductType: val, ...this.initValue }, async () => {
      await new Promise(this.initSearchKey);
      this.searchRef.getWrappedInstance().resetSKey();
    });
  };

  // 产品列表分页
  handlePaginationChange = (page, rows) => {
    this.setState({ page, rows });
    this.loadProductList({ page, rows, ...(this.state.searchKey || this.state.initSKey) });
  };

  /**
   * @description 小班、双师要先选择课程
   * @param index 列表索引
   * @memberof ModalAddProduct
   */
  checkSelect = (index) => {
    const { productList } = this.state;
    const { contractId, selectedList = [], grade } = this.props;
    const { type } = this.getSystemProduct();
    const product = productList[index];
    const { gradeId } = product;

    // 双师系统限制混签
    if (loginUser.isTwoTeacherSym()) {
      const isNewContract = !contractId;
      // 过滤已选产品列表里的关联产品
      const filterSelectedList = selectedList.filter((item) => {
        return !item._relateProductType;
      });
      // 即将混签
      const willMixing = filterSelectedList.some((item) => {
        return calcIsDrainageProduct(product.productCharacterName) !== calcIsDrainageProduct(item.productCharacterName);
      });

      // 添加新合同: 引流产品与常规产品不允许混签
      if (isNewContract && willMixing) {
        showAlert('引流活动不能与常规课程进行混签，请分别签订合同！');
        return;
      }

      // 修改合同: 判断当前合同是否已经混签,已经混签则允许混签,否则不允许混签
      const hasMixing = filterSelectedList.some((item) => {
        return (
          calcIsDrainageProduct(selectedList[0].productCharacterName) !==
          calcIsDrainageProduct(item.productCharacterName)
        );
      });
      if (!hasMixing && willMixing) {
        showAlert('引流活动不能与常规课程进行混签，请分别签订合同！');
        return;
      }
    }

    // 限制小班/双师不能重复; 讲座如果选班也限制不能重复，选产品则不限制
    const shouldLimit = [SMALL_CLASS, TWO_TEACHER].includes(type) || ([LECTURE].includes(type) && product.classId);
    if (shouldLimit) {
      for (let i = 0, len = selectedList.length; i < len; i += 1) {
        if (checkSamePrd(product) === checkSamePrd(selectedList[i])) {
          showMessage('购物车已存在此产品');
          return;
        }
      }
    }

    if (gradeId && this.props.gradeId !== gradeId) {
      // 小班与双师要进行升班判断,升班的情况下高一个年级不会提醒，拿后台返回的判断即可
      if (!([SMALL_CLASS, TWO_TEACHER].includes(type) && grade.gradeId === gradeId)) {
        return showConfirm('学生在读年级与选择的产品年级不一致，请确认是否跨年级上课', () => {
          if (productTypeAllow.includes(type) && product.classId) {
            this.setState({ modalVisible: true, selectProduct: product });
            return;
          }
          this.handleSelect(product);
        });
      }
    }

    if (productTypeAllow.includes(type) && product.classId) {
      this.setState({ modalVisible: true, selectProduct: product });
      return;
    }

    this.handleSelect(product);
  };

  // 产品选择回调
  handleSelect = (product) => {
    const { quantity, totalCourseHours } = product;
    const { selectProductType } = this.state;

    let _quantity = quantity || totalCourseHours;
    // 数量的默认逻辑
    if (quantityResetType.includes(selectProductType)) {
      _quantity = !!_quantity ? _quantity : 1;
    }
    product.quantity = _quantity;
    this.props.dispatch({
      type: 'signContract/updateSelectProduct',
      payload: { type: 'ADD', data: product, selectProductType },
    });
  };

  // 已选产品 - 删除
  handleDelete = (index) => {
    const { selectedList } = this.props;
    const prd = selectedList[index];
    let list = selectedList;
    // 是否包含子产品
    prd.subProducts ? (list = selectedList.filter((item) => prd._relation !== item._relation)) : list.splice(index, 1);
    this.props.dispatch({
      type: 'signContract/updateSelectProduct',
      payload: {
        type: 'DEL',
        list,
        record: prd,
        index,
      },
    });
  };

  // 获取产品列表和类型
  getSystemProduct = () => {
    const { props, state } = this;
    const { isEcs, institution } = props;
    const { selectProductType } = state;
    const systemProductType = (isEcs ? systemProduct[ECS_CLASS] : systemProduct[institution]) || [];
    const system_product = (isEcs ? systemProductTab[ECS_CLASS] : systemProductTab[institution]) || [];
    return { type: selectProductType || systemProductType[0], system_product };
  };

  /**
   * @description 切换选择首次上课时间弹窗显示
   * @memberof ModalAddProduct
   */
  switchModalSelectCourseTime = () => {
    this.setState(() => ({ modalVisible: false }));
  };

  /**
   * @description 带课程上课时间添加产品
   * @memberof ModalAddProduct
   */
  addPrdWithCourseTime = async (item, index, list) => {
    const { selectProduct } = this.state;
    const { courseBeginTime, courseDate, _origin = {} } = item;
    const { type } = this.getSystemProduct();
    const { startClassDate, startClassTime, totalCourseHours = 0, classId } = selectProduct;
    const realClassHours = list
      .filter((course, courseIndex) => courseIndex >= index)
      .reduce((total, current) => total + current._origin.courseHours, 0);
    const firstTime = `${courseDate} ${courseBeginTime}`;
    const productFirstTime = `${startClassDate} ${startClassTime}`;
    let _quantity = totalCourseHours;

    if (new Date(firstTime).getTime() > new Date(productFirstTime).getTime() && totalCourseHours !== realClassHours) {
      if (type === SMALL_CLASS) {
        _quantity = await findMiniClassWillBuyHour({ startDate: firstTime, miniClassId: classId });
      } else {
        // 这里的班级id要取双师的主班id
        _quantity = await findClassWillBuyHour({ startDate: firstTime, classId: _origin.twoTeacherClassId });
      }
    }

    const val = {
      ...selectProduct,
      courseHours: _origin.courseNum,
      startClassTime: courseBeginTime,
      _courseDate: courseDate,
      quantity: _quantity,
    };

    this.handleSelect(val);
    this.switchModalSelectCourseTime();
  };

  /**
   * @description 带开始上课时间添加产品
   * @memberof ModalAddProduct
   */
  addPrdWithStartTime = () => {
    const { selectProduct } = this.state;
    const val = {
      _date: selectProduct.startClassTime,
      _time: null,
      _realClassHours: selectProduct.totalCourseHours,
      ...selectProduct,
    };
    this.handleSelect(val);
    this.switchModalSelectCourseTime();
  };

  getValidCourseDate = (list, today) => {
    let result = [];
    for (let i = 0; i < list.length; i++) {
      if (!list[i].disabled) {
        result = [i];
        break;
      }
    }
    return result;
  };

  render() {
    const { isEcs, selectedList, onClose, visible, title, institution, calcLoading, ...reset } = this.props;
    const { rows, page, selectProduct, loading, productList, productListTotal } = this.state;
    const { type, system_product } = this.getSystemProduct();
    const { classId } = selectProduct;

    return (
      <Modal
        toggle
        title={title}
        visible={visible}
        widthSize="xl"
        okText="确认"
        loadingTip="优惠金额计算中..."
        okBtnShow={false}
        cancelText="关闭"
        onClose={onClose}
        loading={calcLoading}
        wrapClassName={style.product_modal}
        {...reset}
      >
        <Row gutter={20}>
          <Col className={style.left_content} span={18}>
            <Tabs activeKey={type} className={style.header_tabs} onChange={this.handleTabsChange}>
              {system_product.map(({ val, text }) => (
                <TabPane tab={`新增${text}`} key={val} />
              ))}
            </Tabs>
            <div>
              <Search
                selectProductType={type}
                initSKey={this.state.initSKey}
                isEcs={isEcs}
                institution={institution}
                onSearch={this.handleSearch}
                isAlready
                ref={(ref) => (this.searchRef = ref)}
              />
              <SearchResult number={productListTotal} />
            </div>
            {productList.length === 0 && <Loading loading={loading} />}
            {productList.length > 0 ? (
              <Deck
                loading={loading}
                dataSource={productList}
                render={classCardRender({
                  onClick: this.checkSelect,
                })}
                defaultValue={{ showQuickJumper: false }}
                pagination={{
                  total: productListTotal,
                  current: page,
                  size: rows,
                  showSizeChanger: true,
                  pageSizeOptions: ['5', '10', '20', '50'],
                  onChange: this.handlePaginationChange,
                }}
                className={style.ToBeSelectCard}
              />
            ) : (
              <NoDataPlus />
            )}
          </Col>
          <Col span={6} className={style.right_content}>
            <span className={style.select_title}>已选产品</span>
            {selectedList.length > 0 ? (
              <Deck
                dataSource={selectedList}
                render={selectedCardRender({
                  onDelete: this.handleDelete,
                })}
                className={style.SelectedCard}
              />
            ) : (
              <NoDataPlus className={style.nodata} />
            )}
          </Col>
          {this.state.modalVisible && classId && (
            <ModalSelectCourseTime
              title="选择首次上课时间"
              classId={classId}
              productType={type}
              onCancel={this.switchModalSelectCourseTime}
              onConfirm={this.addPrdWithCourseTime}
              noDataDirectCallback={this.addPrdWithStartTime}
              beforeNowDisabled={true}
              beforeNowDays={14}
              beforeNowMaxCourse={Infinity}
              defaultSelect={this.getValidCourseDate}
              footerTipsRender={SelectCourseDateTips}
            />
          )}
        </Row>
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  const {
    account,
    dict: { year },
    signContract,
  } = state;
  const { contractId, isEcs, selectedList, blCampusId, grade = {}, gradeId, calcLoading } = signContract;
  let { institution } = account ? account.user : {};
  const { serverDate } = account.user;
  const yearDict = year ? year.filter((item) => Number(item.text) >= new Date(serverDate).getFullYear() - 1) : [];

  return {
    contractId,
    isEcs,
    blCampusId,
    institution,
    selectedList,
    serverDate,
    yearDict,
    grade: grade || {},
    calcLoading,
    gradeId,
    productTypeTree: state.dict.productTypeTree || [],
  };
};

export default connect(mapStateToProps)(ModalAddProduct);
