import { QUARTER_SPRING, QUARTER_SUMMER, QUARTER_AUTUMN, QUARTER_WINTER } from './config';
import { getBLoYearId } from 'UTIL/dict';
import moment from 'moment';

/**
 * 通过开始上课时间及总课时长，获取上课时间范围
 *
 * @param startTime {string} '14:00'
 * @param timeThrough {number} 90
 *
 * @return {string} '14:00~15:30'
 * */
function getTimeRange(startTime, timeThrough) {
  return startTime ? `${startTime}~${moment(startTime, 'HH:mm').add(timeThrough, 'm').format('HH:mm')}` : '-';
}

/**
 * 双师、小班在11月1日至12月31日年份 + 1
 * @param {*} yearDict
 * @param {*} date
 * @param initValue
 */
function getNextYear(yearDict, date, initValue) {
  const _date = new Date(date);
  const year = _date.getFullYear();

  if (_date >= new Date(`${year}-11-01`) && _date <= new Date(`${year}-12-31`)) {
    return initValue !== undefined ? initValue : getBLoYearId(yearDict, new Date(`${year + 1}-1-01`));
  }

  return undefined;
}

/**
 * 双师小班获取默认值
 * 配置写死，优化：后端添加接口返回各日期
 * @param {*} date
 */
function getQuarterId(date) {
  const _date = new Date(date);
  const year = _date.getFullYear();

  if (_date >= new Date(`${year}-2-01`) && _date <= new Date(`${year}-04-30`)) {
    return [QUARTER_SPRING];
  }

  if (_date >= new Date(`${year}-5-01`) && _date <= new Date(`${year}-08-15`)) {
    return [QUARTER_SUMMER, QUARTER_AUTUMN];
  }

  if (_date >= new Date(`${year}-8-16`) && _date <= new Date(`${year}-10-30`)) {
    return [QUARTER_AUTUMN];
  }

  return [QUARTER_AUTUMN, QUARTER_WINTER];
}

/**
 * 双师系统-季度默认值计算
 *
 * @param serverDate 服务器时间
 */
function getQuarterIdByTwoTeacher(serverDate) {
  if (!serverDate) {
    return null;
  }

  const date = new Date(serverDate);
  const month = date.getMonth() + 1;
  const spring = [1, 2, 3];
  const summer = [4, 5, 6];
  const autumn = [7, 8, 9, 10];
  const winter = [11, 12];
  let quarterId = null;

  if (spring.includes(month)) {
    quarterId = QUARTER_SPRING;
  } else if (summer.includes(month)) {
    quarterId = QUARTER_SUMMER;
  } else if (autumn.includes(month)) {
    quarterId = QUARTER_AUTUMN;
  } else if (winter.includes(month)) {
    quarterId = QUARTER_WINTER;
  }

  return quarterId ? [quarterId] : null;
}

/**
 * 判断是否为引流产品
 *
 * @param productCharacterName {string} 产品性质名称
 * @return {*|boolean}
 */
function calcIsDrainageProduct(productCharacterName) {
  return productCharacterName && /引流/.test(productCharacterName);
}

export { getTimeRange, getNextYear, getQuarterId, getQuarterIdByTwoTeacher, calcIsDrainageProduct };
