import React from 'react';
import { connect } from 'dva';
import PropTypes from 'prop-types';
import { storeName } from '../store/storeCfg';
import style from './styles.module.less';
import { Table } from 'antd';
import Quarter from '../components/Quarter';
import TableSubject from '../ProductTableSS/TableSubject';
import TableProduct from '../ProductTableSS/TableProduct';
import TableClass from '../ProductTableSS/TableClass';
import { ListAction, Modal } from 'COMPONENT';
import { createSetWidth } from 'COMPONENT/List/utils';

const setWidth = createSetWidth();

class ModalChangeLinkPrd extends React.Component {
  constructor(props) {
    super(props);

    this.columns = [
      {
        title: '季度',
        dataIndex: 'quarterId',
        key: 'quarterId',
        width: setWidth(2),
        render: (text, record, index) => {
          return text ? <Quarter id={text} isTwoTeacher={true} /> : '-';
        },
      },
      {
        title: '科目',
        dataIndex: 'subjectName',
        key: 'subjectName',
        width: setWidth(4),
        render: (text, record, index) => {
          return <TableSubject type={record.productTypeName} name={text || '-'} />;
        },
      },
      {
        title: '产品名称',
        key: 'productInfo',
        width: setWidth(6),
        render: (text, record, index) => {
          return <TableProduct type={record.productTypeName} name={record.productName} />;
        },
      },
      {
        title: '班级',
        dataIndex: 'classInfo',
        key: 'classInfo',
        width: setWidth(10),
        render: (text, record, index) => {
          return <TableClass data={record} today={this.props.timeText} showOthers={true} />;
        },
      },
      { title: '上课开始时间', dataIndex: 'startClassTime', key: 'startClassTime', width: setWidth(6) },
      { title: '老师', dataIndex: 'assistTeacherName', key: 'assistTeacherName', width: setWidth(4) },
      { title: '单价', dataIndex: 'price', key: 'price', width: setWidth(4) },
      { title: '状态', dataIndex: 'status', key: 'status', width: setWidth(2) },
      {
        title: '操作',
        key: 'actions',
        width: setWidth(2),
        render: (text, record, index) => (
          <ListAction
            config={[
              {
                text: '更换',
                onClick: () => {
                  this.changeProduct(index);
                },
                disabled: this.props.data.productId === record.productId && this.props.data.classId === record.classId,
              },
            ]}
          />
        ),
      },
    ];
  }

  static propTypes = {
    data: PropTypes.object.isRequired,
    index: PropTypes.number.isRequired,
    type: PropTypes.string.isRequired,
    onCancel: PropTypes.func.isRequired,
  };

  changeProduct = (subIndex) => {
    const _self = this;
    this.props.dispatch({
      type: `${storeName}/${this.props.type}`,
      payload: {
        index: this.props.index,
        subIndex,
        callback(isNeedClose) {
          if (isNeedClose) {
            _self.props.onCancel();
          }
        },
      },
    });
  };

  render() {
    return (
      <Modal
        className={style.ModalChangeLinkPrd}
        title="更换班级"
        widthSize="XL"
        footer={null}
        onCancel={this.props.onCancel}
      >
        <Table dataSource={this.props.data._list} columns={this.columns} pagination={false} bordered />
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    timeText: state[storeName].timeText,
  };
};

export default connect(mapStateToProps)(ModalChangeLinkPrd);
