import React from 'react';
import PropTypes from 'prop-types';
import { Modal, List } from 'COMPONENT';
import { Tabs } from 'antd';
import { runFunc } from 'UTIL/common';

const TabPane = Tabs.TabPane;
const setWidth = List.createSetWidth();

class ModalSameStudent extends React.Component {
  constructor(props) {
    super(props);
    this.onConfirm = this.onConfirm.bind(this);
    this.columns = [
      { title: '分公司', dataIndex: 'brenchName', key: 'brenchName', width: setWidth(5) },
      { title: '学习中心', dataIndex: 'blCampusName', key: 'blCampusName', width: setWidth(5) },
      { title: '客户姓名', dataIndex: 'cusName', key: 'cusName', width: setWidth(4) },
      { title: '客户电话', dataIndex: 'contact', key: 'contact', width: setWidth(6) },
      { title: '学生编号', dataIndex: 'id', key: 'id', width: setWidth(6) },
      { title: '学生姓名', dataIndex: 'name', key: 'name', width: setWidth(4) },
      { title: '性别', dataIndex: 'sex', key: 'sex', width: setWidth(2) },
      { title: '学校', dataIndex: 'schoolName', key: 'schoolName', width: setWidth(5) },
      { title: '年级', dataIndex: 'gradeName', key: 'gradeName', width: setWidth(3) },
      { title: '所属学管', dataIndex: 'studyManegerName', key: 'studyManegerName', width: setWidth(4) },
    ];
  }

  static propTypes = {
    studentName: PropTypes.string.isRequired,
    sameList: PropTypes.array.isRequired,
    diffList: PropTypes.array.isRequired,
    onCancel: PropTypes.func.isRequired,
    onConfirm: PropTypes.func.isRequired,
    callback: PropTypes.func,
  };

  onConfirm() {
    runFunc(this.props.onConfirm, this.props.callback);
  }

  render() {
    return (
      <Modal
        title={`系统中发现已有同名的学生：${this.props.studentName}，详情如下：`}
        widthSize="L"
        onCancel={this.props.onCancel}
        onOk={this.onConfirm}
        cancelBtnShow={false}
      >
        <Tabs defaultActiveKey="same">
          <TabPane tab="本分公司" key="same">
            <List columns={this.columns} dataSource={this.props.sameList} bordered />
          </TabPane>
          <TabPane tab="跨分公司" key="diff">
            <List columns={this.columns} dataSource={this.props.diffList} bordered />
          </TabPane>
        </Tabs>
        <div>
          <p>
            请核实本次签单学生与系统已存在学生<span>是否同一人</span>，避免产生重复学籍。
          </p>
          <p>如果是同一学生，请前往学生列表给该学生续费。</p>
          <p>如果不是同一学生，可直接点击下方“继续保存合同” 。</p>
        </div>
      </Modal>
    );
  }
}

export default ModalSameStudent;
