import React from 'react';
import { Tooltip } from 'antd';
import styles from '../index.less';
import { COUPON_RULE_VO_KEY, COUPON_PROMOTION_TYPE_KEY } from '../../config';
import SingleDetail from './SingleDetail';
import { formatPromotionRuleResult, formatRuleEntireText } from '../util';

const ruleVoKey = COUPON_RULE_VO_KEY;
const promotionTypeKey = COUPON_PROMOTION_TYPE_KEY;

/**
 * 显示文字
 *
 * @param promotionType
 * @param rule
 * @return {string}
 */
function formatResultText(promotionType, rule) {
  const result = formatPromotionRuleResult({ promotionType, rule });

  return formatRuleEntireText(result);
}

/**
 * tooltip 规则显示
 *
 * @param props
 * @return {*}
 * @constructor
 */
const TooltipRuleList = (props) => {
  const { data } = props;
  const ruleVos = data[ruleVoKey] || [];
  const promotionType = data[promotionTypeKey];

  return (
    <ul className={styles.top_left_tooltip}>
      {ruleVos.map((item, index) => {
        return (
          <li key={index}>
            <span className={styles.top_left_text}>{formatResultText(promotionType, item)}</span>
          </li>
        );
      })}
    </ul>
  );
};

/**
 * 多个规则显示
 * 1、该优惠券使用规则已确定是多个
 * 2、仍然需要判断选中产品数量是否命中的情况
 * 3、如果未找到则显示全部 (正常情况下一定能找到一个最优的优惠规则，符合条件且排序后的最后一个规则)
 *
 * @param props
 * @return {*}
 * @constructor
 */
const MultipleDetail = (props) => {
  const { data, productList, onlyShowSingleRule } = props;

  const promotionType = data[promotionTypeKey];
  const ruleVos = data[ruleVoKey] || [];
  const productLength = productList.length;
  let hitRule = null;

  if (onlyShowSingleRule && productLength > 0) {
    // 先过滤掉产品数量已经不符合条件的规则，再排序
    const filterRules = ruleVos
      .filter((rule) => rule.promotionRule <= productLength)
      .sort((a, b) => a.promotionRule - b.promotionRule);

    // 如果过滤后的规则有多个则需要找到优惠金额最多的一张 (直接按价格升序)
    if (filterRules.length > 1) {
      filterRules.sort((a, b) => a.promotion - b.promotion);
    }

    hitRule = filterRules[filterRules.length - 1];
  }

  return hitRule ? (
    <SingleDetail {...props} hitRule={hitRule} />
  ) : (
    <Tooltip placement="topLeft" title={<TooltipRuleList {...props} />}>
      <ul className={styles.top_left_multiple}>
        {ruleVos.map((item, index) => {
          if (index < 2) {
            return (
              <li key={index}>
                <span className={styles.top_left_text}>{formatResultText(promotionType, item)}</span>
              </li>
            );
          }

          return '';
        })}

        {ruleVos.length > 2 && (
          <li>
            <span className={styles.top_left_text}>更多...</span>
          </li>
        )}
      </ul>
    </Tooltip>
  );
};

export default MultipleDetail;
