import React from 'react';
import classNames from 'classnames';
import { Checkbox, Tooltip } from 'antd';
import styles from './index.less';
import { formatSelectedStyle, renderActivityDate, formatContinuityTypeName } from './util';
import LeftDetail from './LeftDetail';
import { superPositionCfg } from '../config';

/**
 * 单个优惠, 共3种展示样式：
 * 1、未选产品且优惠券使用规则只有一条
 * 2、未选产品且优惠券使用规则有多条（显示多个标签）
 * 3、选择产品且优惠券使用规则只有一条
 * 4、选择产品且优惠券规则有多条（只显示命中的规则, 效果同3）
 *
 * @param props.data 优惠券数据
 * @param props.selected 是否选中
 * @param props.disabled 是否禁用
 * @param props.productList 已选产品列表
 * @constructor
 */
const CouponCard = (props) => {
  const { data, onClick } = props;
  const { selected, disabled, superPosition, promotionName, continuityType, branchName } = data;

  const branchNameList = (typeof branchName === 'string' && branchName.split(',')) || [];
  const branchLen = branchNameList.length;

  /**
   * 点击优惠券
   */
  const onCouponClick = () => {
    if (disabled) {
      return;
    }

    onClick(data);
  };

  return (
    <section className={classNames([styles.coupon, formatSelectedStyle(selected, disabled)])} onClick={onCouponClick}>
      <div className={styles.coupon_top}>
        <LeftDetail {...props} onlyShowSingleRule={false} />

        <div className={styles.top_right}>
          <h5 className={styles.top_right_title}>
            <Tooltip placement="topLeft" title={promotionName}>
              <span className={styles.right_title_desc}>{promotionName}</span>
            </Tooltip>
            <span className={styles.right_title_checkbox}>
              <Checkbox checked={selected} disabled={disabled} />
            </span>
          </h5>
          <p className={styles.top_right_time}>有效期：{renderActivityDate(data, '至')}</p>
        </div>
      </div>

      <div className={styles.coupon_bottom}>
        <span className={styles.Label}>{formatContinuityTypeName(continuityType)}</span>
        <span className={styles.Label}>{superPositionCfg[superPosition]}</span>
        {branchLen > 1 ? (
          <Tooltip placement="topLeft" title={branchName}>
            <span className={styles.Label}>{branchNameList[0]}...</span>
          </Tooltip>
        ) : (
          <span className={styles.Label}>{branchName}</span>
        )}
      </div>

      <div className={styles.coupon_triangle} data-type="left" />
      <div className={styles.coupon_triangle} data-type="right" />
    </section>
  );
};

export default CouponCard;
