import React from 'react';
import styles from './index.less';
import { formatPayText } from './util';

/**
 * 支付卡片
 *
 * @param props.sharePayTypeList {array} 分享支付支付方式类型
 * @param props.showPayTypeList {boolean} 是否显示分享支付支付方式类型
 * @param props.buttonType {string} 按钮支付方式, receive：合同收款,share：分享支付
 * @param props.onClick {function} 点击事件，回调参数为payType
 * @constructor
 */
const PayCard = (props) => {
  const { buttonType, onClick } = props;

  const payText = formatPayText(buttonType);
  const canDoClick = true;

  const onCardClick = () => {
    if (!canDoClick) {
      return;
    }

    if (onClick) {
      onClick(buttonType);
    }
  };

  return (
    <section className={styles.payCard} data-type={buttonType} data-disabled={!canDoClick} onClick={onCardClick}>
      <div className={styles.icon_wrap}>{/*icon*/}</div>

      <div className={styles.desc_wrap}>
        <h5 className={styles.desc_title}>
          <span style={{ marginRight: 10 }}>{payText.title}</span>
        </h5>
        <p className={styles.desc_detail}>{payText.detail}</p>
      </div>
    </section>
  );
};

export default PayCard;
