import React, { PropTypes } from 'react';
import { Modal, OldModal } from 'COMPONENT';
import styles from './index.less';
import PayCard from './PayCard';
import { loadModal } from './util';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

/**
 * 选择支付方式
 *
 * @param props.campusId {string} 合同学习中心ID（用于判断是否配置分享支付）
 * @param props.contractId {string} 合同ID
 * @param props.onCancel {function} 点击关闭
 */
class Index extends React.Component {
  static propTypes = {
    campusId: PropTypes.string,
    contractId: PropTypes.string,
    onCancel: PropTypes.func,
  };

  static defaultProps = {
    campusId: null,
    contractId: null,
    onCancel: () => {},
  };

  constructor(props) {
    super(props);

    this.state = {
      showContractPayModal: false, // 显示合同收款弹窗

      SharePaymentModal: null, // 分享支付弹窗
      showSharePaymentModal: false, // 显示分享支付弹窗
      sharePaymentModalParams: {}, // 分享支付弹窗参数
    };
  }

  componentDidMount() {
    this.initEventListener();
    this.initModal();
  }

  componentWillUnmount() {
    this.removeEventListener();
  }

  /**
   * 初始化自定义监听事件
   * 监听老boss弹窗触发的关闭弹窗事件
   */
  initEventListener = () => {
    window.addEventListener('propToReactCloseModal', this.onContractPayModalClose, false);
  };

  removeEventListener = () => {
    window.removeEventListener('propToReactCloseModal', this.onContractPayModalClose, false);
  };

  /**
   * 初始化弹窗
   */
  initModal = () => {
    Promise.all([loadModal()])
      .then(([comp]) => {
        const { contractId } = this.props;
        const oldPageParams = {
          contractId,
          actionPayType: 'AB',
        };

        this.setState({
          SharePaymentModal: comp,
          sharePaymentModalParams: oldPageParams,
        });
      })
      .catch((e) => {});
  };

  /**
   * 点击合同收款
   */
  onReceiveClick = () => {
    this.setState({ showContractPayModal: true });
  };

  /**
   * 关闭合同收款
   */
  onContractPayModalClose = () => {
    this.setState({ showContractPayModal: false });
    this.props.onCancel();
  };

  /**
   * 点击分享支付
   */
  onShareClick = () => {
    if (this.props.contractFundTypeName === '教育平台') {
      return showMessage('教育平台收款合同不能使用分享支付', 'error');
    }
    this.setState({
      showSharePaymentModal: true,
    });
  };

  /**
   * 关闭分享支付弹窗
   *
   * @param paid {boolean} 是否支付完成(支付状态通过订单列表判断), true: 支付完成, false: 未支付/支付失败
   */
  onSharePaymentModalClose = (paid) => {
    this.setState({ showSharePaymentModal: false });
    this.props.onCancel();
  };

  render() {
    const { contractId } = this.props;
    const { showContractPayModal, SharePaymentModal, showSharePaymentModal, sharePaymentModalParams } = this.state;

    return (
      <Modal
        title="请选择付款方式"
        cancelText="放弃支付"
        okBtnShow={false}
        okBtnDisabled={false}
        loading={false}
        onCancel={this.props.onCancel}
        className={styles.payTypeModal}
      >
        <article className={styles.container}>
          <PayCard
            buttonType="receive"
            onClick={this.onReceiveClick}
            contractFundTypeName={this.props.contractFundTypeName}
          />

          <PayCard buttonType="share" onClick={this.onShareClick} />
        </article>

        {showContractPayModal && (
          <OldModal
            modalUrl="function/modals/contractTransactionModal.html"
            params={{ contractId: contractId }}
            onClose={this.onContractPayModalClose}
            contractFundTypeName={this.props.contractFundTypeName}
          />
        )}

        {SharePaymentModal && showSharePaymentModal && (
          <SharePaymentModal oldPageParams={sharePaymentModalParams} onClose={this.onSharePaymentModalClose} />
        )}
      </Modal>
    );
  }
}

export default Index;
