import React from 'react';
import ReactDOM from 'react-dom';
import { connect } from 'dva';
import style from './styles.module.less';
import { storeName, TAB_TRIGGER } from '../store/storeCfg';
import { Tabs } from 'antd';
import List from 'COMPONENT/List';
import InputNumber from 'COMPONENT/InputNumber';
import MultiSelect from 'COMPONENT/MultiSelect';
import Popover from 'COMPONENT/Popover';
import Loading from 'COMPONENT/Loading';
import Quarter from '../components/Quarter';
import TableProduct from './TableProduct';
import TableClass from './TableClass';
import ListAction from 'COMPONENT/ListAction';
import ModalChangeLinkPrd from '../ModalChangeLinkPrd';
import { focusResize } from '../utils';
import { SMALL_CLASS, TWO_TEACHER, UNVALID } from '../config';
import { runFunc, isNumberPlus } from 'UTIL/common';
import { debounce, floor } from 'lodash';
import { logStoreName } from '../alilogStore/config';

const TabPane = Tabs.TabPane;
const setWidth = List.createSetWidth();

class ProductTable extends React.Component {
  constructor(props) {
    super(props);
    this.setTableFixedWidth = this.setTableFixedWidth.bind(this);
    this.getSelectedColumns = this.getSelectedColumns.bind(this);
    this.switchModalVisible = this.switchModalVisible.bind(this);
    this.onLinkSelect = this.onLinkSelect.bind(this);
    this.onLinkSelectAll = this.onLinkSelectAll.bind(this);
    this.deleteSelectedPrd = this.deleteSelectedPrd.bind(this);
    this.numberParser = this.numberParser.bind(this);
    this.updatePrdQuantity = this.updatePrdQuantity.bind(this);
    this.updatePrdPromotion = this.updatePrdPromotion.bind(this);
    this.calcPrdPromotion = this.calcPrdPromotion.bind(this);
    this.clearPromotion = this.clearPromotion.bind(this);
    this.switchContractPrdInvalid = this.switchContractPrdInvalid.bind(this);

    this.handleProductTabChange = this.handleProductTabChange.bind(this);
    // 埋点上报日志
    this.logProductTabChange = this.logProductTabChange.bind(this);
    this.logLinkProductChecked = this.logLinkProductChecked.bind(this);

    this.debounceCalc = debounce(this.calcPrdPromotion, 600);
    this.state = {
      modalVisible: false,
      changeProductData: null,
      changeProductIndex: null,
    };
    this.columns = [
      {
        title: '季度',
        dataIndex: 'quarterId',
        key: 'quarterId',
        width: setWidth(4),
        render: (text, record, index) => {
          return text ? <Quarter id={text} /> : '-';
        },
      },
      {
        title: '科目',
        dataIndex: 'subjectName',
        key: 'subjectName',
        width: setWidth(3),
        render: (text, record, index) => {
          return text || '-';
        },
      },
      {
        title: '产品名称',
        key: 'productInfo',
        width: setWidth(6),
        render: (text, record, index) => {
          return <TableProduct type={record.productTypeName} name={record.productName || ''} />;
        },
      },
      {
        title: '班级',
        key: 'classInfo',
        width: setWidth(8),
        render: (text, record, index) => {
          return record.classId ? <TableClass data={record} today={this.props.timeText} /> : '-';
        },
      },
      {
        title: '上课开始时间',
        key: 'startClassDateAndTime',
        width: setWidth(7),
        render: (text, record, index) => {
          return record.classId ? (
            <div>
              <p>{record._courseDate || record.startClassDate}</p>
              <p>{record.startClassTime}</p>
            </div>
          ) : (
            '-'
          );
        },
      },
      {
        title: '老师',
        key: 'teacherName',
        width: setWidth(4),
        render: (text, record, index) => {
          return (
            ([SMALL_CLASS, TWO_TEACHER].includes(record.productType)
              ? record.assistTeacherName
              : record.mainTeacherName) || '-'
          );
        },
      },
      { title: '单价', dataIndex: 'price', key: 'price', width: setWidth(4) },
    ];
    this.linkColumns = this.columns.concat([
      { title: '状态', dataIndex: 'status', key: 'status', width: setWidth(2) },
      {
        title: '操作',
        key: 'actions',
        width: setWidth(6),
        render: (text, record, index) => {
          let subTableLen = record._list.length;
          if (subTableLen > 0) {
            return (
              <ListAction
                config={[
                  {
                    text: `更改班级(${subTableLen})`,
                    onClick: () => {
                      this.switchModalVisible(record, index, 'changeLinkPrd');
                    },
                  },
                ]}
                direction="column"
              />
            );
          }
          return null;
        },
      },
    ]);
    this.selectedColumns = this.columns.concat([
      {
        title: '数量',
        dataIndex: 'quantity',
        key: 'quantity',
        width: setWidth(4),
        render: (text, record, index) => {
          return (
            <InputNumber
              value={text}
              parser={(val) => this.numberParser(val, index)}
              onChange={(val) => this.updatePrdQuantity(val, index)}
              precision={2}
              min={0}
              disabled={record._editQuantityDisabled || (record._quantityDisabled && Boolean(record._hadInitQuantity))}
            />
          );
        },
      },
      { title: '产品总额', dataIndex: 'payment', key: 'payment', width: setWidth(5) },
      {
        title: '优惠',
        key: 'usedPromotion',
        width: 160,
        render: (text, record, index) => {
          if (Array.isArray(record._activePromotion) && record._activePromotion.length > 0) {
            return record._activePromotion.map((item) => <p key={item.id}>{item.name}</p>);
          } else {
            let dict = null;
            const disabled = !record.promotions || record.promotions.length === 0;
            if (!disabled) {
              dict =
                record.promotions[0].id === '-1'
                  ? this.props.productPromotionDict
                  : record.promotions.map((item) => ({ text: item.name, val: item.id }));
            }

            let multiSelect = (
              <MultiSelect
                placeholder="请选择可用优惠"
                dict={dict}
                value={record._prdPromotionVal}
                onChange={(val) => this.updatePrdPromotion(val, index)}
                disabled={record._editPrdPromotionDisabled || !record.quantity}
                selectAll={false}
                noOptionTips="没有可用优惠"
                showSearch
              />
            );

            return record._warningBeforeEditPrdPromotion ? (
              <Popover
                content="修改原合同产品优惠将重置优惠款"
                placement="top"
                overlayClassName={style.warningOverlay}
                key={record.contractProductId}
              >
                <div>{multiSelect}</div>
              </Popover>
            ) : (
              multiSelect
            );
          }
        },
      },
      { title: '优惠款', dataIndex: 'promotionAmount', key: 'promotionAmount', width: setWidth(4) },
      { title: '应收款', dataIndex: 'realAmount', key: 'realAmount', width: setWidth(4) },

      { title: '状态', dataIndex: 'status', key: 'status', width: setWidth(3) },
      {
        title: '操作',
        key: 'actions',
        width: setWidth(6),
        fixed: 'right',
        render: (text, record, index) => {
          let cfg = [];

          if (!record._btnDeleteDisabled) {
            cfg.push({
              text: '删除',
              onClick: () => {
                this.deleteSelectedPrd(record, index);
              },
            });
          } else if (this.props.isEdit && record.contractProductId && !record._btnInvalidDisabled) {
            cfg.push({
              text: record._contractPrdStatus === UNVALID ? '撤销' : '取消',
              onClick: () => {
                this.switchContractPrdInvalid(record, index);
              },
            });
          }

          if (record._btnClearPromotionVisible) {
            cfg.push({
              text: '清除优惠',
              onClick: () => {
                this.clearPromotion(record, index);
              },
              needConfirm: true,
              confirmTips: '确认清除优惠吗？',
            });
          }

          if (record.hasOwnProperty('_list')) {
            let subTableLen = record._list.length;
            if (subTableLen > 0) {
              cfg.unshift({
                text: `更改班级(${subTableLen})`,
                onClick: () => {
                  this.switchModalVisible(record, index, 'changeSelectedPrd');
                },
              });
            }
          }

          return <ListAction config={cfg} direction="column" />;
        },
      },
    ]);

    this.SelectedListRef = null;
  }

  componentDidMount() {
    // FIXME 在已选产品列表时，固定操作列无法更新宽度
    window.addEventListener('resize', this.setTableFixedWidth, false);
  }

  componentWillUnmount() {
    window.removeEventListener('resize', this.setTableFixedWidth, false);
  }

  // 补偿表格固定操作宽度列表(rc-table bug)
  setTableFixedWidth() {
    const listNode = ReactDOM.findDOMNode(this.SelectedListRef);
    let fixedCol = listNode.querySelector('.ant-table-fixed-right');
    if (fixedCol) {
      const tableWrap = listNode.querySelector('.ant-table-scroll');
      const thNodes = tableWrap.querySelector('.ant-table-header').querySelectorAll('th');
      const actionWidth = thNodes[thNodes.length - 1].offsetWidth;
      fixedCol.style['min-width'] = `${actionWidth + 7}px`;
      fixedCol.style['max-width'] = `${actionWidth + 7}px`;
    }
  }

  getSelectedColumns(columns) {
    let isInsetCol = columns.findIndex((item) => item.key === 'promotionConsumeAmount') > -1;
    if (this.props.isEdit && !isInsetCol) {
      columns.splice(-2, 0, {
        title: '已扣优惠',
        dataIndex: 'promotionConsumeAmount',
        key: 'promotionConsumeAmount',
        width: setWidth(5),
      });
    } else if (!this.props.isEdit && isInsetCol) {
      columns.splice(-3, 1);
    }
    return columns;
  }

  switchModalVisible(record, index, type) {
    this.setState((prevState) => {
      let payload = { modalVisible: !prevState.modalVisible };
      if (payload.modalVisible) {
        payload.changeProductData = record;
        payload.changeProductIndex = index;
        payload.changeProductType = type;
      } else {
        payload.changeProductData = null;
        payload.changeProductIndex = null;
        payload.changeProductType = null;
      }
      return payload;
    });
  }

  onLinkSelect(record, selected, selectedRows) {
    this.props.dispatch({
      type: `${storeName}/synLinkPrdToSelected`,
      payload: {
        selected,
        updateRows: [record],
      },
    });

    this.props.dispatch({
      type: `${storeName}/updateLinkSelect`,
      payload: { record, selected, selectedRows },
    });

    this.logLinkProductChecked({
      selected,
      checkboxType: 'SINGLE',
    });
  }

  onLinkSelectAll(selected, selectedRows, changeRows) {
    this.props.dispatch({
      type: `${storeName}/synLinkPrdToSelected`,
      payload: {
        selected,
        updateRows: changeRows,
      },
    });

    this.props.dispatch({
      type: `${storeName}/updateLinkSelectAll`,
      payload: { selected, selectedRows, changeRows },
    });

    this.logLinkProductChecked({
      selected,
      checkboxType: 'ALL',
    });
  }

  logLinkProductChecked(payload) {
    const aliasMap = {
      SINGLE: {
        value: 'SINGLE_RENEW_PRODUCT_CHECKBOX',
        label: '单个续报产品',
      },
      ALL: {
        value: 'ALL_RENEW_PRODUCT_CHECKBOX',
        label: '所有续报产品',
      },
    };
    const { selected, checkboxType } = payload;
    const { value, label } = aliasMap[checkboxType];

    this.props.dispatch({
      type: `${logStoreName}/logBoxCheck`,
      payload: {
        checked: selected,
        value,
        label,
      },
    });
  }

  deleteSelectedPrd(record, index) {
    this.props.dispatch({
      type: `${storeName}/deleteSelectedPrd`,
      payload: { record, index },
    });
  }

  // 数量只能输入数字
  numberParser(val, index) {
    if (val === '') {
      return '';
    }

    if (isNumberPlus(val)) {
      if (typeof val === 'string' && val[val.length - 1] === '.') {
        return val;
      }
      return floor(val, 2);
    }

    if (Array.isArray(this.props.selectedList) && this.props.selectedList.length > 0) {
      return this.props.selectedList[index].quantity || '';
    }

    return '';
  }

  updatePrdQuantity(val, index) {
    if (val === this.props.selectedList[index].quantity) {
      return;
    }
    const _self = this;
    this.props.dispatch({
      type: `${storeName}/changePrdQuantity`,
      payload: {
        val,
        index,
        callback(payload) {
          runFunc(_self.debounceCalc, payload);
        },
      },
    });
  }

  calcPrdPromotion(payload) {
    this.props.dispatch({
      type: `${storeName}/calcPrdPromotion`,
      payload,
    });
  }

  updatePrdPromotion(val, index) {
    this.props.dispatch({
      type: `${storeName}/changePrdPromotion`,
      payload: { val, index },
    });
  }

  clearPromotion(record, index) {
    this.props.dispatch({
      type: `${storeName}/clearPromotion`,
      payload: { record, index },
    });
  }

  switchContractPrdInvalid(record, index) {
    this.props.dispatch({
      type: `${storeName}/switchContractPrdInvalid`,
      payload: { record, index },
    });
  }

  handleProductTabChange(activeKey) {
    this.logProductTabChange(activeKey);
  }

  logProductTabChange(activeKey) {
    const hashmap = {
      1: { value: 'RENEW_CLASS_TAB', label: '续报班级' },
      2: { value: 'SELECTED_PRODUCT_TAB', label: '已选产品' },
    };
    const { value, label } = hashmap[activeKey];

    this.props.dispatch({
      type: `${logStoreName}/logTabChange`,
      payload: { value, label },
    });
  }

  render() {
    const ListSelected = (
      <List
        ref={(c) => (this.SelectedListRef = c)}
        dataSource={this.props.selectedList}
        columns={this.getSelectedColumns(this.selectedColumns)}
        bordered
      />
    );

    let ProductTable = ListSelected;

    if (this.props.funcAuth.includes(TAB_TRIGGER)) {
      ProductTable = (
        <Tabs defaultActiveKey="2" onTabClick={focusResize} onChange={this.handleProductTabChange}>
          <TabPane tab="续报班级" key="1">
            <List
              dataSource={this.props.linkList}
              columns={this.linkColumns}
              rowSelection={{
                selectedRowKeys: this.props.linkRowKeys,
                onSelect: this.onLinkSelect,
                onSelectAll: this.onLinkSelectAll,
              }}
              bordered
            />
          </TabPane>
          <TabPane tab={`已选产品${this.props.count ? `(${this.props.count})` : ''}`} key="2">
            {ListSelected}
          </TabPane>
        </Tabs>
      );
    }

    return (
      <div id="ProductTable" className={style.ProductTable}>
        {ProductTable}
        <Loading loading={this.props.calcLoading} tip="优惠金额计算中..." />
        {this.state.modalVisible && (
          <ModalChangeLinkPrd
            data={this.state.changeProductData}
            index={this.state.changeProductIndex}
            type={this.state.changeProductType}
            onCancel={this.switchModalVisible}
          />
        )}
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    timeText: state[storeName].timeText,
    isEdit: state[storeName].isEdit,
    funcAuth: state[storeName].funcAuth,
    isEcs: state[storeName].isEcs,
    calcLoading: state[storeName].calcLoading,
    linkExpandKeys: state[storeName].linkExpandKeys,
    linkRowKeys: state[storeName].linkRowKeys,
    linkList: state[storeName].linkList,
    selectedList: state[storeName].selectedList,
    count: state[storeName].selectedList.length,
    productPromotionDict: state[storeName].productPromotionDict,
  };
};

export default connect(mapStateToProps)(ProductTable);
