import React from 'react';
import style from './styles.module.less';
import PropTypes from 'prop-types';
import TipText from '../../components/TipText';

/**
 * 渲染班级
 *
 * @param props.showOthers {boolean} 显示其他信息如：开课日期、已上课时等
 * @return {*}
 * @constructor
 */
const TableClass = (props) => {
  const { showOthers } = props;

  return (
    <span className={style.TableClass}>
      <span className={style.TableClassName}>{props.data.className}</span>
      {showOthers && [
        <span className={style.TableClassInfo} key={1}>
          开课日期：
          {props.today > props.data.startClassDate ? (
            <TipText style={{ color: '#666' }}>{props.data.startClassDate || '-'}</TipText>
          ) : (
            props.data.startClassDate || '-'
          )}
        </span>,
        <span className={style.TableClassInfo} key={2}>
          已上课时：{`${props.data.consumeCourseHours || '-'}/${props.data.totalCourseHours || '-'}`}
        </span>,
      ]}
      <span className={style.TableClassInfo}>
        已报：{`${props.data.peopleQuantity || '-'}/${props.data.maxPeopleQuantity || '-'}`}
      </span>
    </span>
  );
};

TableClass.propTypes = {
  data: PropTypes.shape({
    className: PropTypes.string,
    startClassDate: PropTypes.string,
    consumeCourseHours: PropTypes.oneOfType([PropTypes.number, PropTypes.string]),
    totalCourseHours: PropTypes.oneOfType([PropTypes.number, PropTypes.string]),
    peopleQuantity: PropTypes.oneOfType([PropTypes.number, PropTypes.string]),
    maxPeopleQuantity: PropTypes.oneOfType([PropTypes.number, PropTypes.string]),
  }),
};

export default TableClass;
