import React from 'react';
import { Tooltip } from 'antd';
import {
  formatPromotionRuleResultByRule,
  formatPromotionRuleResult,
} from 'PAGE/contract/signContract/ModalSelectCoupon/CouponCard/util';
import style from './styles.module.less';
import { NEW_CONTRACT } from '../config';
import { takeOutPromotionProperties } from '../store/storeUtils';

// 是否显示优惠规则名称
const SHOW_RULE_NAME = false;

// 合同类型对应的优惠方案属性ID, {PROMOTION_PROPERTIES_ID_NEW: 新签, PROMOTION_PROPERTIES_ID_RE: 续费}
const PROMOTION_PROPERTIES_ID_NEW = 'DAT000000000801';
const PROMOTION_PROPERTIES_ID_RE = 'DAT000000000802';

/**
 * 格式化规则显示名称
 *
 * @param ruleDetail {object} 优惠规则
 * @return {*}
 */
function formatRuleName(ruleDetail) {
  const ruleResult = formatPromotionRuleResultByRule(ruleDetail);
  const { resultText, resultNum, resultUnit } = ruleResult;

  if (resultUnit === '折') {
    return resultNum + resultUnit;
  }

  return resultText + resultNum;
}

/**
 * 格式化规则显示名称
 * @param promotionType {string} 优惠券类型
 * @param rule {object} 优惠规则
 * @return {*}
 */
function formatRuleNameByPromotionType(promotionType, rule) {
  const ruleResult = formatPromotionRuleResult({ promotionType, rule });
  const { resultText, resultNum, resultUnit } = ruleResult;

  if (resultUnit.indexOf('折') > -1) {
    return resultNum + resultUnit;
  }

  return resultText + resultNum;
}

/**
 * 格式化优惠名称
 *
 * @param promotionList
 * @return {[]}
 */
export function renderPromotionName(promotionList = []) {
  return promotionList.map((item) => {
    let promotionName = item.name || '';

    let ruleName = '';
    if (item.ruleDetail) {
      ruleName = formatRuleNameByPromotionType(item.type, item.ruleDetail);
    }

    return [
      <Tooltip placement="topLeft" title={item.name}>
        <p>{promotionName}</p>
      </Tooltip>,
      ruleName && SHOW_RULE_NAME ? (
        <div>
          <Tooltip placement="topLeft" title={ruleName}>
            <span className={style.ruleName}>{ruleName}</span>
          </Tooltip>
        </div>
      ) : null,
    ];
  });
}

/**
 * 根据关联的classId找到关联的父产品信息
 * tips: 关注 /store/storeUtils.js 的 addProductWithSub() 方法
 *
 * @param selectedProductList 已选的所有产品
 * @param record 每一行数据
 * @return {null|*}
 */
export function findHitParentProduct(selectedProductList = [], record) {
  if (!selectedProductList || selectedProductList.length <= 0 || !record) {
    return null;
  }

  const hitProduct = selectedProductList.find((item) => {
    return item.classId === record._relateClassId;
  });

  return hitProduct;
}

/**
 * 根据合同类型过滤合同产品的优惠券
 * tips: 新签合同-过滤掉续费优惠券，续费合同-过滤掉新签优惠券
 *
 * @param promotionList 优惠列表
 * @param contractType 合同类型
 */
export function filterPromotionByContractType(promotionList, contractType) {
  if (!promotionList) {
    return [];
  }

  const filterProperties = contractType === NEW_CONTRACT ? PROMOTION_PROPERTIES_ID_RE : PROMOTION_PROPERTIES_ID_NEW;

  return promotionList.filter(function (item) {
    return takeOutPromotionProperties(item) !== filterProperties;
  });
}
