import React from 'react';
import { connect } from 'dva';
import { storeName } from '../store/storeCfg';
import InfoBlock from '../components/InfoBlock';
import InfoForm from '../components/InfoForm';
import TipText from '../components/TipText';
import Required from '../components/Required';
import { Input, Select, InputSelect } from 'COMPONENT';
import { contractCustomerSearch, findCustomerById } from 'SERVICE/contract';
import { errorsHandle, loginUser } from 'UTIL/common';
import ModalSelectSchool from 'SRC/BLoComponents/ModalSelectSchool';
import { Icon } from 'antd';
import style from './style.modules.less';
import ModalChangeStuName from 'SRC/BLoComponents/ModalChangeStuName';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { getBranchByCampus } from '../utils';
import ModalInfo from '../components/ModalInfo';
import { RE_CONTRACT } from '../config';
import { getSelectOption } from 'SERVICE/common';

const InfoItem = InfoForm.InfoItem;

class StudentInfo extends React.Component {
  constructor(props) {
    super(props);
    this.updateProps = this.updateProps.bind(this);
    this.findCustomerByPhone = this.findCustomerByPhone.bind(this);
    this.handlePhoneInput = this.handlePhoneInput.bind(this);
    this.handleCustomerSelect = this.handleCustomerSelect.bind(this);
    this.findStudentByCustomerId = this.findStudentByCustomerId.bind(this);
    this.handleStudentNameInput = this.handleStudentNameInput.bind(this);
    this.handleStudentSelect = this.handleStudentSelect.bind(this);
    this.handleSchoolSelected = this.handleSchoolSelected.bind(this);
    this.updateRemark = this.updateRemark.bind(this);
    this.switchChangedNameVisible = this.switchChangedNameVisible.bind(this);
    this.updateStudentName = this.updateStudentName.bind(this);
    this.handleMergeStudent = this.handleMergeStudent.bind(this);
    this.closeMergeStudentModal = this.closeMergeStudentModal.bind(this);
    this.switchSchoolSelectVisible = this.switchSchoolSelectVisible.bind(this);
    this.state = {
      // 客户
      customerSelectVisible: false,
      customerTempList: null,
      customerNameDisable: false,
      // 选择学校
      schoolSelectVisible: false,
      //
      studentSelectVisible: false,
      studentTempList: null,
      // 更改学生姓名
      changedNameVisible: false,
      //
      mergeStudentVisible: false,
      mergeStudentInfo: {},
      identityTypeList: [],
    };
    // 待选学生
    this.customerStudentList = [];
    this.isTwoTeacher = loginUser.isTwoTeacherSym();
    //
    this.mergeStudentInfo = null;
  }

  componentDidMount() {
    this.fetchIdentityType();
  }

  fetchIdentityType = async () => {
    try {
      const res = await getSelectOption({ selectOptionCategory: 'CERTIFICATE_TYPE' });
      const identityType = res.value || {};
      const identityTypeList = Object.keys(identityType)
        .filter((key) => key)
        .map((key) => {
          return {
            val: key,
            text: identityType[key],
          };
        });
      this.setState({
        identityTypeList,
      });
    } catch (e) {
      showMessage('获取证件类型失败', 'error');
    }
  };

  updateProps(payload) {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload,
    });
  }

  async findCustomerByPhone(phoneNum) {
    const rsp = await contractCustomerSearch({ customerContact: phoneNum }, true).catch(errorsHandle());
    if (rsp) {
      let customerDict = rsp.data.map((item) => ({
        text: `${item.customerName}-${item.customerContact}`,
        val: item.customerId,
      }));
      this.setState({
        customerSelectVisible: true,
        customerTempList: customerDict,
      });
    }
  }

  handlePhoneInput(e) {
    let val = e.target.value;
    let numVal = Number(val);
    if (typeof numVal === 'number' && !isNaN(numVal)) {
      this.setState({
        customerSelectVisible: false,
        customerTempList: null,
        customerNameDisable: false,
      });
      this.customerStudentList = [];
      if (val.length > 9) {
        this.findCustomerByPhone(val);
      }
      this.updateProps({
        customerContact: val,
        customerId: null,
      });
    }
  }

  handleCustomerSelect(data) {
    let [customerName, customerContact] = data.text.split('-');
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        customerContact,
        customerName,
        customerId: data.val,
      },
    });
    this.setState({ customerNameDisable: true });
    this.findStudentByCustomerId(data.val);
  }

  async findStudentByCustomerId(id) {
    const rsp = await findCustomerById({ id }, true).catch(errorsHandle());
    this.customerStudentList = rsp.studentVos;
  }

  handleStudentNameInput(e) {
    let val = e.target.value;
    let studentDict = this.customerStudentList
      .filter((item) => item.name.indexOf(val) > -1)
      .map((item) => ({
        text: item.name,
        val: item.id,
      }));
    if (studentDict.length > 0) {
      this.setState({
        studentSelectVisible: true,
        studentTempList: studentDict,
      });
    }
    this.updateProps({
      studentId: null,
      studentName: val,
    });
  }

  handleStudentSelect(data) {
    let tag = this.customerStudentList.find((item) => item.id === data.val);
    if (tag) {
      this.props.dispatch({
        type: `${storeName}/updateState`,
        payload: {
          studentId: tag.id,
          studentName: tag.name,
          gradeId: tag.gradeId,
          schoolId: tag.schoolId,
          schoolName: tag.schoolName,
        },
      });
      this.props.dispatch({
        type: `${storeName}/initContractType`,
        payload: {
          studentId: tag.id,
          flag: false,
        },
      });
    }
  }

  handleSchoolSelected(data) {
    const { id, name, remarks, provinceId, cityId } = data;
    const isOther = id === '-1' || !id;
    this.updateProps({
      schoolId: isOther ? null : id,
      schoolName: isOther ? remarks : name,
      virtualSchool: isOther
        ? {
            provinceId: provinceId,
            cityId: cityId,
            regionId: '',
            schoolName: remarks,
          }
        : null,
    });

    this.setState({ schoolSelectVisible: false });
  }

  updateRemark(e) {
    const val = e.target.value;
    if (val.length < 512) {
      this.updateProps({ remark: val });
    }
  }

  switchChangedNameVisible() {
    this.setState((prevState) => ({ changedNameVisible: !prevState.changedNameVisible }));
  }

  updateStudentName(val) {
    this.updateProps({ studentName: val });
  }

  handleMergeStudent(mergeStudentInfo) {
    this.setState({
      mergeStudentVisible: true,
      mergeStudentInfo,
      changedNameVisible: false,
    });
  }

  closeMergeStudentModal() {
    const { mergeStudentInfo } = this.state;
    this.setState(
      {
        mergeStudentVisible: false,
      },
      () => {
        if (mergeStudentInfo.mergeSuccess && window.__refreshParentEntrance) {
          window.__refreshParentEntrance({ stuId: mergeStudentInfo.newStudentId }, !mergeStudentInfo.lost);
        }
        this.setState({ mergeStudentInfo: {} });
      },
    );
  }

  switchSchoolSelectVisible() {
    if (!this.props.blCampusId) {
      return showMessage('请先选择合同学习中心');
    }
    this.setState((prevState) => ({ schoolSelectVisible: !prevState.schoolSelectVisible }));
  }

  render() {
    return (
      <InfoBlock title={this.props.title || '学员信息'} toolbar={this.props.toolbar}>
        <InfoForm>
          <InfoItem label={<Required>客户手机</Required>} span={6}>
            <InputSelect
              placeholder="家长手机号"
              value={this.props.customerContact}
              onChange={this.handlePhoneInput}
              showSelect={this.state.customerSelectVisible}
              list={this.state.customerTempList}
              onSelect={this.handleCustomerSelect}
              disabled={this.props.disabledEditInfo}
            />
          </InfoItem>
          <InfoItem label={<Required>客户姓名</Required>} span={6}>
            <Input
              placeholder="家长姓名"
              value={this.props.customerName}
              onChange={(e) => this.updateProps({ customerName: e.target.value })}
              disabled={this.props.disabledEditInfo || this.state.customerNameDisable}
            />
          </InfoItem>
          <InfoItem label={<Required>学生姓名</Required>} span={6}>
            <InputSelect
              placeholder="学生姓名"
              value={this.props.studentName}
              onChange={this.handleStudentNameInput}
              showSelect={this.state.studentSelectVisible}
              list={this.state.studentTempList}
              onSelect={this.handleStudentSelect}
              disabled={this.props.disabledEditInfo}
            />
            {this.props.canEditStdName && (
              <div className={style.changeStdName} onClick={this.switchChangedNameVisible} title="更换姓名">
                <Icon type="edit" />
              </div>
            )}
          </InfoItem>
          <InfoItem label={<Required>年级</Required>} span={6}>
            <Select
              dict={this.props.gradeDict}
              value={this.props.gradeId}
              onChange={(val) => this.updateProps({ gradeId: val })}
              // 兼容学生年级空的情况
              disabled={this.props.disabledEditGrade && this.props.disabledEditInfo && Boolean(this.props.gradeId)}
            />
          </InfoItem>
          <InfoItem label={<Required>合同学习中心</Required>} span={6}>
            <Select
              dict={this.props.branchCampusDict}
              value={this.props.blCampusId}
              onChange={(val) =>
                this.updateProps({
                  blCampusId: val,
                  contractBranchId: getBranchByCampus(val, this.props.branchCampusDict),
                })
              }
              allowClear={false}
              showSearch
            />
          </InfoItem>
          <InfoItem label={this.isTwoTeacher ? '学校' : <Required>学校</Required>} span={6}>
            <Input readOnly={true} value={this.props.schoolName} onClick={this.switchSchoolSelectVisible} />
          </InfoItem>
          <InfoItem label="证件类型" span={6}>
            <Select
              value={this.props.customerIdCardType}
              placeholder="请选择证件类型"
              dict={this.state.identityTypeList}
              onChange={(val) =>
                this.updateProps({
                  customerIdCardType: val,
                })
              }
            />
          </InfoItem>
          <InfoItem label="证件号码" span={6}>
            <Input
              value={this.props.customerIdCardNo}
              onChange={(e) => this.updateProps({ customerIdCardNo: e.target.value })}
            />
          </InfoItem>
          <InfoItem label="备注" span={6}>
            <Input value={this.props.remark} onChange={this.updateRemark} />
          </InfoItem>
          {!this.isTwoTeacher && (
            <TipText style={{ marginLeft: 70, marginRight: 16 }}>
              建议对报读一对一产品进行科目课时分配备注，避免遗忘。注意： 学生合同需同步购买产品所对应的1对1综合服务费、
              小班资料费、双师资料费或者精英班VIP服务费！
            </TipText>
          )}
        </InfoForm>

        {this.state.schoolSelectVisible && (
          <ModalSelectSchool
            blCampusId={this.props.blCampusId}
            schoolId={this.props.schoolId}
            onOk={this.handleSchoolSelected}
            onCancel={this.switchSchoolSelectVisible}
          />
        )}

        {this.state.changedNameVisible && (
          <ModalChangeStuName
            cusId={this.props.customerId}
            studentId={this.props.studentId}
            studentName={this.props.studentName}
            onCancel={this.switchChangedNameVisible}
            onOk={this.updateStudentName}
            onMerge={this.handleMergeStudent}
          />
        )}

        {this.state.mergeStudentVisible && (
          <ModalInfo title="结果反馈" onCancel={this.closeMergeStudentModal} isAutoClose>
            {this.state.mergeStudentInfo.mergeMessage}
          </ModalInfo>
        )}
      </InfoBlock>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    branchCampusDict: state.dict.branchAllCampus,
    gradeDict: state.dict.grade,
    customerContact: state[storeName].customerContact,
    customerId: state[storeName].customerId,
    customerName: state[storeName].customerName,
    studentId: state[storeName].studentId,
    studentName: state[storeName].studentName,
    gradeId: state[storeName].gradeId,
    schoolId: state[storeName].schoolId,
    schoolName: state[storeName].schoolName,
    blCampusId: state[storeName].blCampusId,
    remark: state[storeName].remark,
    customerIdCardNo: state[storeName].customerIdCardNo,
    customerIdCardType: state[storeName].customerIdCardType,
    disabledEditInfo: state[storeName].disabledEditInfo,
    canEditStdName: state[storeName].canEditStdName,
    // 学生默认有年级的 且 是续签合同的 不允许修改年级
    disabledEditGrade: state[storeName].disabledEditGrade && state[storeName].contractType === RE_CONTRACT,
  };
};

export default connect(mapStateToProps)(StudentInfo);
