import React from 'react';
import PropTypes from 'prop-types';
import { Modal } from 'COMPONENT';
import { runFunc } from 'COMPONENT/utils/common';

class ModalInfo extends React.Component {
  constructor(props) {
    super(props);
    this.autoClose = this.autoClose.bind(this);
    this.state = {
      count: 0,
    };
    this.autoCloseInterval = null;
  }

  componentDidMount() {
    if (this.props.isAutoClose) {
      const _self = this;
      this.autoCloseInterval = setInterval(() => {
        if (_self.state.count < 5) {
          _self.setState((prevState) => ({ count: prevState.count + 1 }));
        } else {
          _self.autoClose();
        }
      }, 1000);
    }
  }

  componentWillUnmount() {
    if (this.autoCloseInterval) {
      clearInterval(this.autoCloseInterval);
    }
  }

  static PropTypes = {
    title: PropTypes.string.isRequired,
    onCancel: PropTypes.string.isRequired,
    isAutoClose: PropTypes.bool,
    autoCloseTime: PropTypes.number,
  };

  static defaultProps = {
    isAutoClose: false,
    autoCloseTime: 5,
  };

  autoClose() {
    runFunc(this.props.onCancel);
  }

  render() {
    const { count } = this.state;
    const { title, isAutoClose, autoCloseTime, children, ...restProps } = this.props;
    return (
      <Modal
        title={`${title}${isAutoClose ? `(当前窗口 ${autoCloseTime - count} 秒后自动关闭)` : ''}`}
        cancelBtnShow={false}
        okBtnShow={false}
        widthSize="M"
        {...restProps}
      >
        {children}
      </Modal>
    );
  }
}

export default ModalInfo;
