import React from 'react';
import PropTypes from 'prop-types';
import { winter, spring, summer, autumn } from '../../config';
import style from './styles.module.less';

const colorCfg = {
  [winter]: '#56AEFF',
  [spring]: '#52C41A',
  [summer]: '#F5222D',
  [autumn]: '#FA541C',
  default: '#6D67FF',
};

const colorCfgSS = {
  [winter]: '#F5222D',
  [spring]: '#52C41A',
  [summer]: '#56AEFF',
  [autumn]: '#FFBB22',
  default: '#6D67FF',
};

const textCfg = {
  [winter]: '寒',
  [spring]: '春',
  [summer]: '暑',
  [autumn]: '秋',
  default: '全年',
};

/**
 * 季度展示
 *
 * @param props.isTwoTeacher 是否为双师
 * @return {*}
 * @constructor
 */
const Quarter = (props) => {
  const { isTwoTeacher } = props;
  const colorConfig = isTwoTeacher ? colorCfgSS : colorCfg;

  return (
    <span className={style.Quarter} style={{ backgroundColor: colorConfig[props.id] || colorConfig.default }}>
      {textCfg[props.id] || textCfg.default}
    </span>
  );
};

Quarter.propTypes = {
  id: PropTypes.string,
};

export default Quarter;
