import React from 'react';
import { connect } from 'dva';
import classNames from 'classnames';
import style from './styles.module.less';
import toolbarStyle from './components/toolbar/styles.module.less';
import Page from './components/Page';
import FooterBar from './FooterBar';
import TipText from './components/TipText';
import Text from './components/Text';
import InfoBlock from './components/InfoBlock';
import ModalSelectCoupon from './ModalSelectCoupon';
import ModalSelectPayType from './ModalSelectPayType';
import ModalConfirmAndMergePay from '../modals/SelectPayType/Modal';
import StudentInfo from './StudentInfo';
import ProductTable from './ProductTable';
import ProductTableSS from './ProductTableSS';
import { Switch, Radio, Button } from 'antd';
import {
  calcAuto,
  calcManual,
  ONE_ON_ONE_COURSE,
  ONE_ON_MANY,
  SMALL_CLASS,
  TWO_TEACHER,
  ECS_CLASS,
  OTHERS,
  ALL,
} from './config';
import { storeName, ECS_SWITCH, AUTO_CALC_PROMOTION, CHOOSE_COUPON, XH_BTN, SS_BTN, PY_BTN } from './store/storeCfg';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import ModalAddProduct from './ModalAddProduct';
import { runFunc, loginUser, errorsHandle } from 'UTIL/common';
import { browserHistory } from 'react-router';
import { OLDPAGEBASE } from 'CONST/config';
import AssignClassModal from '../modals/AssignClassModal';
import { jsonParseObjAttrByKey } from './utils';
import { filterValidProductList } from './ModalSelectCoupon/util';
import { focusResize } from './utils';
import { logStoreName } from './alilogStore/config';
import { getDictTextByVal } from 'UTIL/dict';
import countdown from 'COMPONENT/Modal/countdown';
import { getContractCouponDepositSaveFundTag, getContractInfoById } from 'SERVICE/contract';

const RadioButton = Radio.Button;
const RadioGroup = Radio.Group;

class SignContract extends React.Component {
  constructor(props) {
    super(props);
    this.contractFundTypeName = ''; // 合同收款类型

    this.restore = this.restore.bind(this);
    this.init = this.init.bind(this);
    this.initFuncAuth = this.initFuncAuth.bind(this);
    this.initDict = this.initDict.bind(this);
    this.initInfo = this.initInfo.bind(this);
    this.switchIsEcs = this.switchIsEcs.bind(this);
    this.switchAutoCalc = this.switchAutoCalc.bind(this);
    this.addContractProduct = this.addContractProduct.bind(this);
    this.closeModal = this.closeModal.bind(this);
    this.switchCouponModalVisible = this.switchCouponModalVisible.bind(this);
    this.selectCoupon = this.selectCoupon.bind(this);
    this.validateContract = this.validateContract.bind(this);
    this.checkSameStudent = this.checkSameStudent.bind(this);
    this.saveContract = this.saveContract.bind(this);
    this.handleSameStudent = this.handleSameStudent.bind(this);
    this.save = this.save.bind(this);
    this.saveEdit = this.saveEdit.bind(this);
    this.directSave = this.directSave.bind(this);
    this.saveAndGoToList = this.saveAndGoToList.bind(this);
    this.saveAndPay = this.saveAndPay.bind(this);
    this.saveAndAssign = this.saveAndAssign.bind(this);
    this.goToContractList = this.goToContractList.bind(this);
    this.getOnPayment = this.getOnPayment.bind(this);
    this.assignCourseHours = this.assignCourseHours.bind(this);
    this.closeAssignCourseHours = this.closeAssignCourseHours.bind(this);
    this.openPayTypeModal = this.openPayTypeModal.bind(this);
    this.closePayTypeModal = this.closePayTypeModal.bind(this);
    this.switchSameStudentVisible = this.switchSameStudentVisible.bind(this);

    //埋点上报日志
    this.initLogTrace = this.initLogTrace.bind(this);
    this.logSwitchPromotionType = this.logSwitchPromotionType.bind(this);
    this.logAddingProduct = this.logAddingProduct.bind(this);
    this.logSavingCntr = this.logSavingCntr.bind(this);
    this.logSubmitCntr = this.logSubmitCntr.bind(this);

    this.state = {
      loading: false,
      modalVisible: false,
      modalInitSKey: null,
      couponModalVisible: false,
      payTypeModalVisible: false, // 打开选择收款方式弹窗
      confirmAndMergePayModalVisible: false, // 打开确认合同并支付弹窗
      assignClassVisible: false,
      contractId: null,
      sameStudentVisible: false,
      sameStudentProps: null,
    };
    this.isTwoTeacher = loginUser.isTwoTeacherSym();

    this.contractId = null;
  }

  componentDidMount() {
    this.init();
  }

  componentWillUnmount() {
    this.restore();
  }

  restore() {
    this.props.dispatch({ type: `${storeName}/restore` });
    this.props.dispatch({ type: `${logStoreName}/reset` });
  }

  async init() {
    this.setState({ loading: true });
    this.restore();
    await new Promise(this.initFuncAuth);
    await new Promise(this.initDict);
    await new Promise(this.initInfo(jsonParseObjAttrByKey(this.props.routeParams, 'params') || {}));
    focusResize();
    this.initLogTrace();
    this.setState({ loading: false });
  }

  initLogTrace() {
    this.props.dispatch({ type: `${logStoreName}/initTrace` });
  }

  initFuncAuth(resolve) {
    this.props.dispatch({
      type: `${storeName}/initFuncAuth`,
      payload: {
        callback() {
          resolve();
        },
      },
    });
  }

  initDict(resolve) {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['branchAllCampus', 'grade'],
        callback() {
          resolve();
        },
      },
    });
  }

  initInfo(payload = {}) {
    return (resolve) => {
      this.props.dispatch({
        type: `${storeName}/initInfo`,
        payload: {
          ...payload,
          callback() {
            resolve();
          },
        },
      });
    };
  }

  switchIsEcs(checked) {
    if (this.props.selectedList.length > 0) {
      showConfirm(
        <div style={{ textAlign: 'center' }}>
          <p>{`${checked ? '非' : ''}精英班已有选购产品`}</p>
          <p>{`若切换到${!checked ? '非' : ''}精英班则会清空产品购物车并且重算优惠，是否确认切换？`}</p>
        </div>,
        () => {
          this.props.dispatch({
            type: `${storeName}/switchIsEcsClearSelected`,
            payload: {
              checked,
            },
          });
        },
      );
    } else {
      this.props.dispatch({
        type: `${storeName}/switchIsEcs`,
        payload: { checked },
      });
    }
  }

  switchAutoCalc(e) {
    this.props.dispatch({
      type: `${storeName}/switchAutoCalc`,
      payload: { value: e.target.value === calcAuto },
    });

    this.logSwitchPromotionType(e.target.value);
  }

  logSwitchPromotionType(type) {
    const hashmap = {
      [calcAuto]: { value: 'PROMOTION_AUTO', label: '自动计算优惠' },
      [calcManual]: { value: 'PROMOTION_MANUAL', label: '手动计算优惠' },
    };

    const { value, label } = hashmap[type] || {};

    if (value) {
      this.props.dispatch({
        type: `${logStoreName}/logTabChange`,
        payload: {
          value,
          label,
        },
      });
    }
  }

  addContractProduct(e) {
    let payload = {
      modalVisible: true,
      modalInitSKey: {
        productTypeId: e.target.dataset.type,
        campusId: this.props.blCampusId || null,
      },
    };

    if (payload.modalInitSKey.productTypeId === ALL) {
      delete payload.modalInitSKey;
    }

    this.setState(payload);

    this.logAddingProduct({
      targetType: e.target.dataset.type,
    });
  }

  logAddingProduct(payload) {
    const labelMap = {
      [ONE_ON_ONE_COURSE]: '新增1对1',
      [ONE_ON_MANY]: '新增1对多',
      [SMALL_CLASS]: '新增小班',
      [ECS_CLASS]: '新增精英班',
      [OTHERS]: '新增其他',
    };

    const targetType = payload.targetType;

    if (labelMap[targetType]) {
      this.props.dispatch({
        type: `${logStoreName}/logBtnClick`,
        payload: {
          value: targetType,
          label: labelMap[targetType],
        },
      });
    }
  }

  closeModal() {
    this.setState({
      modalVisible: false,
      modalInitSKey: null,
    });
    this.props.dispatch({ type: `${storeName}/calcCoursePromotion` });
  }

  switchCouponModalVisible() {
    this.setState((prevState) => ({ couponModalVisible: !prevState.couponModalVisible }));
  }

  selectCoupon(list) {
    this.props.dispatch({
      type: `${storeName}/selectCoupon`,
      payload: { list },
    });
    this.switchCouponModalVisible();
  }

  validateContract(resolve) {
    this.props.dispatch({
      type: `${storeName}/validateContract`,
      payload: {
        callback(canSave) {
          resolve(canSave);
        },
      },
    });
  }

  checkSameStudent(resolve) {
    this.props.dispatch({
      type: `${storeName}/checkSameStudent`,
      payload: {
        callback(options) {
          resolve(options);
        },
      },
    });
  }

  saveContract(resolve) {
    this.props.dispatch({
      type: `${storeName}/submit`,
      payload: {
        callback(contractId) {
          resolve(contractId);
        },
      },
    });
  }

  handleSameStudent(options) {
    if (options.type === 'modal') {
      this.switchSameStudentVisible(options);
    }
  }

  async save(callback) {
    this.setState({ loading: true });
    const canSave = await new Promise(this.validateContract);
    if (canSave) {
      // 继续保存
      const contractId = await new Promise(this.saveContract);
      if (contractId) {
        this.contractId = contractId;
        this.logSubmitCntr(contractId);
        runFunc(callback, contractId);
      }
    }
    this.setState({ loading: false });
  }

  async saveEdit(callback) {
    this.setState({ loading: true });
    const canSave = await new Promise(this.validateContract);
    if (canSave) {
      showConfirm(
        '修改合同产品，可能导致已分配业绩与修改后的资金不匹配而清空业绩，如需要调整业绩，请移步收款记录手动分配',
        async () => {
          const contractId = await new Promise(this.saveContract);
          if (contractId) {
            this.contractId = contractId;
            runFunc(callback, contractId);
          }
          this && this.setState && this.setState({ loading: false });
        },
        () => {
          this.setState({ loading: false });
        },
      );
    } else {
      this.setState({ loading: false });
    }
  }

  async directSave(callback) {
    this.setState({ loading: true });
    const contractId = await new Promise(this.saveContract);
    if (contractId) {
      runFunc(callback, contractId);
    }
    this.setState({ loading: false });
  }

  saveAndGoToList() {
    if (this.props.isEdit) {
      this.saveEdit(this.goToContractList, true);
    } else {
      this.save(this.goToContractList, true);
      this.logSavingCntr('SAVE_AND_TO_CNTR_LIST');
    }
  }

  /**
   * 生成合同并支付
   */
  saveAndPay() {
    if (this.props.isEdit) {
      this.saveEdit(this.getOnPayment);
    } else {
      this.save(this.getOnPayment);
      this.logSavingCntr('SAVE_AND_PAY');
    }
  }

  saveAndAssign() {
    if (this.props.isEdit) {
      this.saveEdit(this.assignCourseHours);
    } else {
      this.save(this.assignCourseHours);
      this.logSavingCntr('SAVE_AND_ASSIGN');
    }
  }

  logSavingCntr(type) {
    const aliasMap = {
      SAVE_AND_TO_CNTR_LIST: '保存并跳转合同列表',
      SAVE_AND_PAY: '保存并支付',
      SAVE_AND_ASSIGN: '保存并分配课时',
    };

    this.props.dispatch({
      type: `${logStoreName}/logBtnClick`,
      payload: {
        value: type,
        label: aliasMap[type],
      },
    });
  }

  logSubmitCntr(contractId) {
    this.props.dispatch({
      type: `${logStoreName}/logCntrSubmit`,
      payload: {
        contractId,
      },
    });
  }

  /**
   * 跳转合同列表
   * @param {string?} contractId 合同id
   */
  goToContractList(contractId) {
    contractId = contractId || this.contractId;

    let params = {
      contractId,
      startDate: null,
      endDate: null,
      stuName: this.props.studentName,
    };

    window.sessionStorage.setItem(
      'contr_list:startup_params',
      JSON.stringify({
        ...params,
        actionType: 'INIT_SEARCH_KEY',
        from: window.location.pathname + window.location.search + window.location.hash,
        timestamp: Date.now(),
      }),
    );

    setTimeout(function () {
      browserHistory.push({
        pathname: OLDPAGEBASE,
        query: {
          src: `function/contract/listContract.html?timestamp=${Date.now()}`,
        },
      });
    });

    this.props.dispatch({
      type: 'main/removeTab',
      payload: { id: this.props.activeTabId },
    });
  }

  async getOnPayment(contractId) {
    if (this.props.isEdit) {
      const res = await getContractInfoById({ contractId }).catch(errorsHandle());
      if (res && res.resultCode === 0 && res.data.paidStatus === 'PAID') {
        this.contractFundTypeName = res.data.contractFundTypeName; // 合同收款类型
        return countdown({
          title: '温馨提示',
          content: '由于您修改合同后，合同待付金额为0，无须支付，将直接跳转到合同列表',
          buttonText: '直接跳转',
          maxCount: 3,
          onOk: this.goToContractList,
          onCancel: this.goToContractList,
        });
      } else {
        this.contractFundTypeName = '';
      }
    }

    this.openPayTypeModal(contractId);
  }

  assignCourseHours(contractId) {
    this.setState({
      assignClassVisible: true,
      contractId,
    });
  }

  closeAssignCourseHours() {
    this.setState(
      {
        assignClassVisible: false,
        contractId: null,
      },
      () => {
        this.goToContractList();
      },
    );
  }

  /**
   * 打开选择支付方式弹窗
   *
   * @param contractId {string} 合同ID
   */
  async openPayTypeModal(contractId) {
    let payload = { payTypeModalVisible: true, contractId };

    this.setState({ ...payload });
  }

  /**
   * 关闭选择支付方式弹窗
   */
  closePayTypeModal() {
    this.setState(
      {
        payTypeModalVisible: false,
        contractId: null,
      },
      () => {
        this.goToContractList();
      },
    );
  }

  /**
   * 关闭确认合同并支付弹窗
   */
  closeConfirmAndMergePayModal = () => {
    this.setState(
      {
        confirmAndMergePayModalVisible: false,
        contractId: null,
      },
      () => {
        this.goToContractList();
      },
    );
  };

  switchSameStudentVisible(sameStudentProps) {
    this.setState((prevState) => ({
      sameStudentVisible: !prevState.sameStudentVisible,
      sameStudentProps: sameStudentProps || null,
    }));
  }

  render() {
    const toolbar = (
      <div className={classNames(toolbarStyle.toolbar, style.infoToolBar)}>
        {this.props.funcAuth.includes(ECS_SWITCH) && (
          <div className={toolbarStyle.toolbarLeft}>
            <span className={style.infoToolBarSwitch}>
              <TipText>是否精英班合同</TipText>
            </span>
            <Switch
              checked={this.props.isEcs}
              onChange={this.switchIsEcs}
              checkedChildren="是"
              unCheckedChildren="否"
              disabled={this.props.isEdit}
            />
          </div>
        )}
        <div className={toolbarStyle.toolbarRight}>
          <Text label="签约人">{this.props.signPersonName}</Text>
          <Text label="签约日期">{this.props.signDate}</Text>
          <Text label="合同类型">
            <TipText fontSize={16} fontWeight="bold">
              {this.props.contractTypeName || '-'}
            </TipText>
          </Text>
        </div>
      </div>
    );

    return (
      <Page
        className={style.SignContract}
        loading={this.state.loading}
        footer={
          <FooterBar
            canUseCoupon={this.props.funcAuth.includes(CHOOSE_COUPON)}
            couponDisabled={this.props.isEdit || !this.props.studentId || !this.props.contractBranchId}
            couponList={this.props.couponList}
            totalAmount={this.props.totalAmount}
            promotionAmount={this.props.promotionAmount}
            realAmount={this.props.realAmount}
            onSelectCouponClick={this.switchCouponModalVisible}
            canSaveAndAssign={this.props.hadOneToOneOrMorePrd}
            onSave={this.saveAndGoToList}
            onSaveAndPay={this.saveAndPay}
            onSaveAndAssign={this.saveAndAssign}
          />
        }
      >
        {this.isTwoTeacher ? (
          <StudentInfo title="基本信息" toolbar={toolbar} />
        ) : (
          [
            <InfoBlock title="合同信息" toolbar={toolbar} style={{ marginBottom: 12 }} key="contractInfo" />,
            <StudentInfo key="studentInfo" />,
          ]
        )}

        <div className={classNames(style.midBar, toolbarStyle.toolbar)}>
          <div className={style.title}>报读课程产品</div>
          <div className={classNames(toolbarStyle.toolbarRight, toolbarStyle.toolbar)}>
            <div>
              {this.props.funcAuth.includes(AUTO_CALC_PROMOTION) && (
                <RadioGroup
                  defaultValue={calcAuto}
                  value={this.props.isAutoCalc ? calcAuto : calcManual}
                  onChange={this.switchAutoCalc}
                  size="large"
                >
                  <RadioButton value={calcAuto} disabled={this.props.isEdit}>
                    自动计算优惠
                  </RadioButton>
                  <RadioButton value={calcManual}>手动计算优惠</RadioButton>
                </RadioGroup>
              )}
            </div>
            <div className={toolbarStyle.toolbarRight}>
              {this.props.funcAuth.includes(XH_BTN) &&
                (this.props.isEcs
                  ? [
                      <Button
                        onClick={this.addContractProduct}
                        type="primary"
                        size="large"
                        data-type={ECS_CLASS}
                        key="XH_BTN_5"
                      >
                        新增精英班
                      </Button>,
                      <Button
                        onClick={this.addContractProduct}
                        type="primary"
                        size="large"
                        data-type={OTHERS}
                        key="XH_BTN_4"
                      >
                        新增其他产品
                      </Button>,
                    ]
                  : [
                      <Button
                        onClick={this.addContractProduct}
                        type="primary"
                        size="large"
                        data-type={ONE_ON_ONE_COURSE}
                        key="XH_BTN_1"
                      >
                        新增1对1
                      </Button>,
                      <Button
                        onClick={this.addContractProduct}
                        type="primary"
                        size="large"
                        data-type={ONE_ON_MANY}
                        key="XH_BTN_2"
                      >
                        新增1对多
                      </Button>,
                      <Button
                        onClick={this.addContractProduct}
                        type="primary"
                        size="large"
                        data-type={SMALL_CLASS}
                        key="XH_BTN_3"
                      >
                        新增小班
                      </Button>,
                      <Button
                        onClick={this.addContractProduct}
                        type="primary"
                        size="large"
                        data-type={OTHERS}
                        key="XH_BTN_4"
                      >
                        新增其他产品
                      </Button>,
                    ])}
              {this.props.funcAuth.includes(SS_BTN) && (
                <Button onClick={this.addContractProduct} type="primary" size="large" data-type={TWO_TEACHER}>
                  新增报读产品
                </Button>
              )}
              {this.props.funcAuth.includes(PY_BTN) && (
                <Button onClick={this.addContractProduct} type="primary" size="large" data-type={SMALL_CLASS}>
                  新增报读产品
                </Button>
              )}
            </div>
          </div>
        </div>

        {this.isTwoTeacher ? <ProductTableSS /> : <ProductTable />}

        {
          // 选择报读产品
          this.state.modalVisible && (
            <ModalAddProduct defaultValue={this.state.modalInitSKey} onClose={this.closeModal} />
          )
        }

        {
          // 选择优惠券
          this.state.couponModalVisible && (
            <ModalSelectCoupon
              limit={-1}
              multiple={true}
              branchId={this.props.contractBranchId}
              studentId={this.props.studentId}
              couponList={this.props.couponList}
              productList={filterValidProductList(this.props.selectedList)}
              realAmount={this.props.realAmount}
              onCancel={this.switchCouponModalVisible}
              onOk={this.selectCoupon}
            />
          )
        }

        {
          // 选择支付方式
          this.state.payTypeModalVisible && (
            <ModalSelectPayType
              contractId={this.state.contractId}
              campusId={this.props.blCampusId}
              onCancel={this.closePayTypeModal}
              contractFundTypeName={this.contractFundTypeName}
            />
          )
        }

        {
          // 确认合同并支付
          this.state.confirmAndMergePayModalVisible && (
            <ModalConfirmAndMergePay
              contractIds={[this.state.contractId]}
              campusId={this.props.blCampusId}
              campusName={this.props.blCampusName}
              studentId={this.props.studentId}
              studentName={this.props.studentName}
              customerName={this.props.customerName}
              contractNumber={this.props.customerContact}
              signTime={this.props.signDate}
              onCancel={this.closeConfirmAndMergePayModal}
            />
          )
        }

        {
          // 分配课时
          this.state.assignClassVisible && (
            <AssignClassModal
              oldPageParams={{
                contractId: this.state.contractId,
                editType: null,
              }}
              onClose={this.closeAssignCourseHours}
            />
          )
        }
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    studentId: state[storeName].studentId,
    studentName: state[storeName].studentName,
    studentBranchId: state[storeName].studentBranchId,
    studentCampusId: state[storeName].studentCampusId,
    blCampusId: state[storeName].blCampusId,
    blCampusName: state[storeName].blCampusId
      ? getDictTextByVal(state.dict.branchAllCampus, state[storeName].blCampusId)
      : null,
    customerName: state[storeName].customerName,
    customerContact: state[storeName].customerContact,
    contractBranchId: state[storeName].contractBranchId,
    isEdit: state[storeName].isEdit,
    funcAuth: state[storeName].funcAuth,
    hadOneToOneOrMorePrd: state[storeName].hadOneToOneOrMorePrd,
    isEcs: state[storeName].isEcs,
    isAutoCalc: state[storeName].isAutoCalc,
    contractTypeName: state[storeName].contractTypeName,
    signPersonName: state[storeName].signPersonName,
    signDate: state[storeName].signDate,
    totalAmount: state[storeName].totalAmount,
    promotionAmount: state[storeName].promotionAmount,
    realAmount: state[storeName].realAmount,
    selectedList: state[storeName].selectedList,
    couponList: state[storeName].couponList,
    activeTabId: state.main.activeTabId,
  };
};

export default connect(mapStateToProps)(SignContract);
