import {
  storeName,
  ECS_SWITCH,
  TAB_TRIGGER,
  AUTO_CALC_PROMOTION,
  CHOOSE_COUPON,
  XH_BTN,
  SS_BTN,
  PY_BTN,
} from './storeCfg';
import { clone } from 'lodash';
import NP from 'number-precision';
import { runFunc, loginUser, errorsHandle, validIdentityCardByType } from 'UTIL/common';
import { parseUrl } from 'UTIL/url';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { isMoreThanZero, clickPrdTableTab, centToYuan, yuanToCent, getBranchByCampus } from '../utils';
import {
  contractTypeCfg,
  SMALL_CLASS,
  TWO_TEACHER,
  calcCouponIdCfg,
  ONE_ON_ONE_COURSE,
  ONE_ON_MANY,
  ECS_CLASS,
  OTHERS,
  UNVALID,
  LECTURE,
  activeUseCaseCfg,
  OTHER_CATEGORY_TYPE_MARKETING,
} from '../config';
import {
  getNewSignContractType,
  getContractStudentInfo,
  getCurrentBranchTeacher,
  queryLinkProductInfo,
  calculatePromotion,
  getProductActivePromotionList,
  calculatePrdPromotion,
  saveContractInfoNew,
  getStudentGradeInContinuedCycle,
  getContractInfoById,
  editContractInfo,
  checkCustomerStudentName,
  getStudentListByName,
  calculateCoupon,
} from 'SERVICE/contract';
import moment from 'moment';
import request from 'UTIL/http/fetch';
import {
  checkSamePrd,
  setPrdRelation,
  getPrdRelation,
  addProductWithSub,
  batchAddProductWithSub,
  contractInfoAdepter,
  takeOutPromotionProperties,
} from './storeUtils';

const defaultState = {
  // 服务器时间戳
  timeStamp: null,
  // 服务器时间YYYY-MM-DD
  timeText: '',
  //
  isEdit: false,
  funcAuth: [],
  isEcs: false,
  isAutoCalc: true,
  calcLoading: false,
  disabledEditInfo: false,
  hadOneToOneOrMorePrd: false,
  canEditStdName: false,
  //
  transactionUuid: null,
  contractId: null,
  contractType: null,
  contractTypeName: null,
  signPerson: null,
  signPersonName: null,
  signDate: null,
  //
  studentBranchId: null,
  studentCampusId: null,
  studentType: null,
  // 合同分公司
  contractBranchId: null,
  //
  customerContact: null,
  customerId: null,
  customerName: null,
  studentId: null,
  studentName: null,
  gradeId: null,
  schoolId: null,
  schoolName: null,
  virtualSchool: null,
  blCampusId: null,
  customerIdCardNo: null,
  customerIdCardType: null,
  remark: null,
  transferIntroduceId: null,
  totalAmount: 0,
  promotionAmount: 0,
  realAmount: 0,
  promotionCoupon: [],
  // 关联产品列表
  linkExpandKeys: [],
  linkRowKeys: [],
  linkList: [],
  selectedList: [],
  // 优惠券
  couponList: [],
  // 上次优惠计算返回结果
  lastTimeCalResult: null,
  // 已删合同产品(key: 产品id + 班级id || '', val: 合同产品id)
  removeContractPrdStorage: {},
  grade: {}, // 学生年级
  teacherDict: [], // 教师字典
};
Object.freeze(defaultState);

export default {
  namespace: storeName,
  state: {
    ...defaultState,
    defaultFuncAuth: null,
    productPromotionDict: null,
  },
  reducers: {
    updateState(state, { payload }) {
      return {
        ...state,
        ...payload,
      };
    },
    // 更新linkRowKeys
    updateLinkSelect(state, { payload: { record, selected, selectedRows } }) {
      let linkRowKeys = clone(state.linkRowKeys);
      if (selected) {
        linkRowKeys.push(record.key);
      } else {
        linkRowKeys = linkRowKeys.filter((item) => item !== record.key);
      }
      return {
        ...state,
        linkRowKeys,
      };
    },
    updateLinkSelectAll(state, { payload: { selected, selectedRows, changeRows } }) {
      let linkRowKeys = clone(state.linkRowKeys);
      if (selected) {
        linkRowKeys = selectedRows.map((item) => item.key);
      } else {
        linkRowKeys = [];
      }
      return {
        ...state,
        linkRowKeys,
      };
    },
    // 更换关联产品
    updateLinkPrd(state, { payload: { index, subIndex } }) {
      let list = clone(state.linkList);

      list[index] = {
        ...list[index],
        ...list[index]._list[subIndex],
        quantity: null,
      };

      return {
        ...state,
        linkList: list,
      };
    },
    // 更换已选产品
    updateSelectedPrd(state, { payload: { index, subIndex } }) {
      let list = clone(state.selectedList);
      let active = list[index];
      let activeRelation = getPrdRelation(active);
      let tag = active._list[subIndex];

      if (tag.subProducts && tag.subProducts.length > 0) {
        let [removeItem] = list.splice(index, 1);
        tag = {
          ...removeItem,
          ...tag,
        };
        list.splice(index, 0, ...addProductWithSub(tag));
      } else {
        list[index] = {
          ...list[index],
          ...tag,
          _relation: setPrdRelation(),
        };
      }
      list[index].quantity = null;
      delete list[index]._activePromotion;
      delete list[index]._discountInfo;
      delete list[index]._prdPromotion;
      delete list[index]._prdPromotionVal;
      delete list[index]._prdPromotionAmount;

      // 移除原产品的子产品
      if (active.subProducts && active.subProducts.length > 0) {
        list = list.filter((item) => item._relation !== activeRelation);
      }

      return {
        ...state,
        selectedList: list,
      };
    },
    // 切换计算优惠loading
    switchCalcLoading(state) {
      return {
        ...state,
        calcLoading: !state.calcLoading,
      };
    },
    // 切换功能权限
    switchFuncAuth(state) {
      let payload = { funcAuth: clone(state.defaultFuncAuth) };
      if (state.isEcs) {
        payload.funcAuth = payload.funcAuth.filter(
          (item) => [TAB_TRIGGER, AUTO_CALC_PROMOTION, CHOOSE_COUPON].indexOf(item) === -1,
        );
        payload.isAutoCalc = false;
      } else if (payload.funcAuth.includes(AUTO_CALC_PROMOTION)) {
        payload.isAutoCalc = true;
      }
      return {
        ...state,
        ...payload,
      };
    },
    // 重置页面
    restore(state) {
      return {
        ...state,
        ...defaultState,
      };
    },
  },
  effects: {
    // 初始化页面功能权限
    *initFuncAuth({ payload: { callback } }, { select, put }) {
      let payload = {
        funcAuth: [ECS_SWITCH, TAB_TRIGGER, AUTO_CALC_PROMOTION, CHOOSE_COUPON, XH_BTN],
        isEcs: false,
        isAutoCalc: true,
      };
      if (loginUser.isTwoTeacherSym()) {
        payload.funcAuth = [TAB_TRIGGER, AUTO_CALC_PROMOTION, CHOOSE_COUPON, SS_BTN];
      } else if (loginUser.isPeiYouSym()) {
        payload.funcAuth = [TAB_TRIGGER, AUTO_CALC_PROMOTION, CHOOSE_COUPON, PY_BTN];
      }
      // 存储一个默认功能配置
      payload.defaultFuncAuth = clone(payload.funcAuth);

      yield put({
        type: 'updateState',
        payload,
      });

      runFunc(callback);
    },

    /**
     * 初始化页面所需信息
     *
     * @param studentId {string?} 学生id
     * @param customerId {string?} 客户id
     * @param blCampusId {string?} 合同学习中心id
     * @param contractId {string?} 合同id
     * @param transferIntroduceId {string?} 转介绍id
     * @param flag {boolean?} 是否从咨询师工作台跳转过来
     * @param contractType {string?} 合同类型
     * @param callback {function?}
     * @param select
     * @param put
     * */
    *initInfo(
      { payload: { contractId, studentId, customerId, blCampusId, transferIntroduceId, flag, contractType, callback } },
      { select, put },
    ) {
      let payload = yield select((state) => ({
        signPerson: state.account.user.userId,
        signPersonName: state.account.user.name,
        blCampusId: loginUser.isCampusRole() ? state.account.user.campusId : null,
        contractBranchId: loginUser.isCampusRole() ? state.account.user.brenchId : null,
        signDate: moment(new Date()).format('YYYY-MM-DD'),
        studentId,
        customerId,
        contractId,
        transferIntroduceId,
        flag,
        isEdit: false,
      }));
      // 入参携带学习中心id优先级高于用户角色学习中心
      if (blCampusId) {
        const dict = yield select((state) => state.dict.branchAllCampus);
        payload.blCampusId = blCampusId;
        payload.contractBranchId = getBranchByCampus(blCampusId, dict);
      }

      if (contractId) {
        // 获取合同详情
        const rspContract = yield getContractInfoById({ contractId }).catch(errorsHandle());
        if (rspContract) {
          const funcAuth = yield select((state) => state[storeName].funcAuth.filter((item) => item !== TAB_TRIGGER));
          payload = {
            ...payload,
            ...contractInfoAdepter(rspContract.data),
            isAutoCalc: false,
            isEdit: true,
            funcAuth,
          };
        }
      } else if (studentId) {
        // 获取合同类型
        yield put({
          type: 'initContractType',
          payload: { studentId, flag, contractType },
        });
      }

      // 获取产品优惠
      yield put({ type: 'getProductPromotionDict' });

      if (payload.studentId) {
        // 获取学生信息
        let fetchList = [
          getContractStudentInfo(
            {
              studentId: payload.studentId,
              customerId: payload.customerId,
            },
            true,
          ),
        ];
        // 获取续报班级(修改合同时无需请求)
        if (!payload.isEdit) {
          fetchList.push(
            queryLinkProductInfo(
              {
                studentId: payload.studentId,
              },
              true,
            ),
          );
        }
        const [rspStudentInfo, rspLinkProduct] = yield Promise.all(fetchList).catch(errorsHandle());

        if (rspStudentInfo) {
          // 获取学生默认年级
          yield put({
            type: 'getStudentGradeInContinuedCycle',
            payload: {
              studentId: payload.studentId,
              gradeId: rspStudentInfo.data.gradeId,
            },
          });

          payload.customerContact = rspStudentInfo.data.customerContact;
          payload.customerId = rspStudentInfo.data.customerId;
          payload.customerName = rspStudentInfo.data.customerName;
          payload.customerIdCardNo = rspStudentInfo.data.customerIdCardNo || null;
          payload.customerIdCardType = rspStudentInfo.data.customerIdCardType || null;
          payload.studentId = rspStudentInfo.data.studentId;
          payload.studentName = rspStudentInfo.data.studentName;
          payload.gradeId = rspStudentInfo.data.gradeId;
          payload.schoolId = payload.schoolId || rspStudentInfo.data.schoolId;
          payload.schoolName = payload.schoolName || rspStudentInfo.data.schoolName;
          payload.studentBranchId = rspStudentInfo.data.studentBranchId;
          payload.studentCampusId = rspStudentInfo.data.studentCampusId;
          payload.studentType = rspStudentInfo.data.studentType;
          payload.disabledEditInfo = true;
          payload.disabledEditGrade = Boolean(rspStudentInfo.data.gradeId);
          // 星火、培优从咨询师页面跳转过来的非正式学生
          if (flag && payload.studentType !== 'ENROLLED' && !loginUser.isTwoTeacherSym()) {
            payload.canEditStdName = true;
          }
        }
        if (rspLinkProduct && rspLinkProduct.data) {
          payload.linkList = rspLinkProduct.data.map((itemList) => {
            let obj = {
              ...itemList[0],
              _list: [],
              _isLink: true,
            };
            if (itemList.length > 1) {
              obj._list = itemList;
            }
            return obj;
          });
          if (payload.linkList.length > 0 && !payload.isEdit) {
            clickPrdTableTab(0);
          }
        }
      }

      yield put({
        type: 'updateState',
        payload,
      });

      if (payload.isEcs) {
        yield put({
          type: 'switchIsEcs',
          payload: { checked: true },
        });
      }

      if (payload.isEdit) {
        yield put({ type: 'checkCanSaveAndAssign' });
      }

      runFunc(callback);
    },

    // 初始化合同类型
    *initContractType({ payload: { studentId, flag = false, contractType } }, { put }) {
      let payload = {};

      if (contractType) {
        payload = {
          contractType,
          contractTypeName: contractTypeCfg[contractType],
        };
      } else {
        const rsp = yield getNewSignContractType({ studentId, flag }, true).catch(errorsHandle());
        payload = {
          contractType: rsp.data,
          contractTypeName: contractTypeCfg[rsp.data],
        };
      }

      yield put({
        type: 'updateState',
        payload,
      });
    },

    // 获取学生默认年级(用于产品选择购物车) + 服务器时间戳
    *getStudentGradeInContinuedCycle({ payload: { studentId, gradeId } }, { put }) {
      const rsp = yield getStudentGradeInContinuedCycle({ studentId, gradeId }).catch(errorsHandle());
      if (rsp) {
        yield put({
          type: 'updateState',
          payload: {
            grade: rsp.data,
            // 公用数据
            timeStamp: rsp.timeStamp,
            timeText: moment(rsp.timeStamp).format('YYYY-MM-DD'),
          },
        });
      }
    },

    // 获取所有产品优惠
    *getProductPromotionDict(actions, { select, put }) {
      const productPromotionDict = yield select((state) => state[storeName].productPromotionDict);
      if (!productPromotionDict) {
        const organizationId = yield select((state) => state.account.user.brenchId);
        const rsp = yield getProductActivePromotionList({ organizationId }).catch(errorsHandle());
        let productPromotionDict = [];

        rsp.forEach((item) => {
          if (item.isActive === 'Y') {
            productPromotionDict.push({
              text: item.name,
              val: item.id,
              properties: takeOutPromotionProperties(item),
            });
          }
        });

        yield put({
          type: 'updateState',
          payload: {
            productPromotionDict,
          },
        });
      }
    },

    // 关联产品-更换产品
    *changeLinkPrd({ payload: { index, subIndex, callback } }, { select, put }) {
      const isCanChange = yield select((state) => {
        let selectedList = state[storeName].selectedList;
        let tag = state[storeName].linkList[index]._list[subIndex];
        let tagCheck = checkSamePrd(tag);
        if (selectedList.findIndex((item) => checkSamePrd(item) === tagCheck) > -1) {
          showMessage('无法更换，已存在相同产品！', 'error');
          return false;
        }
        return true;
      });

      if (isCanChange) {
        let tagIndex = yield select((state) => {
          const tagList = state[storeName].selectedList;
          if (tagList.length === 0) {
            return -1;
          }
          const active = state[storeName].linkList[index];
          const activeCheck = checkSamePrd(active);
          return tagList.findIndex((item) => checkSamePrd(item) === activeCheck);
        });

        yield put({
          type: 'updateLinkPrd',
          payload: { index, subIndex },
        });

        if (tagIndex > -1) {
          yield put({
            type: 'updateSelectedPrd',
            payload: {
              index: tagIndex,
              subIndex,
            },
          });
        }

        yield put({ type: 'calcPrdNumAndPayment' });
      }

      runFunc(callback, isCanChange);
    },

    // 已选产品-更换产品
    *changeSelectedPrd({ payload: { index, subIndex, callback } }, { select, put }) {
      const isCanChange = yield select((state) => {
        let selectedList = state[storeName].selectedList;
        let tag = state[storeName].selectedList[index]._list[subIndex];
        let tagCheck = checkSamePrd(tag);
        if (selectedList.findIndex((item) => checkSamePrd(item) === tagCheck) > -1) {
          showMessage('无法更换，已存在相同产品！', 'error');
          return false;
        }
        return true;
      });

      if (isCanChange) {
        let tagIndex = yield select((state) => {
          const tagList = state[storeName].linkList;
          if (tagList.length === 0) {
            return -1;
          }
          const active = state[storeName].selectedList[index];
          const activeCheck = checkSamePrd(active);
          return tagList.findIndex((item) => checkSamePrd(item) === activeCheck);
        });

        yield put({
          type: 'updateSelectedPrd',
          payload: { index, subIndex },
        });

        if (tagIndex > -1) {
          yield put({
            type: 'updateLinkPrd',
            payload: {
              index: tagIndex,
              subIndex,
            },
          });
        }

        yield put({ type: 'calcPrdNumAndPayment' });
      }

      runFunc(callback, isCanChange);
    },

    // 关联产品-选择或反选时同步到已选产品列表
    *synLinkPrdToSelected({ payload: { selected, updateRows } }, { select, put }) {
      let selectedList = yield select((state) => clone(state[storeName].selectedList));

      if (selected) {
        const linkList = yield select((state) => state[storeName].linkList);
        let newSelected = updateRows.map((item) => item.key).map((index) => clone(linkList[index]));
        selectedList = selectedList.concat(batchAddProductWithSub(newSelected));
      } else {
        // 收集删除的产品
        let removeList = [];
        updateRows.forEach((row) => {
          const index = selectedList.findIndex(
            (item) => item.productId === row.productId && item.classId === row.classId,
          );
          if (index > -1) {
            // 删除主产品
            let [removeItem] = selectedList.splice(index, 1);
            if (removeItem) {
              removeList.push(removeItem);
            }
          }
        });
        // 删除子产品
        removeList.forEach((removeItem) => {
          if (removeItem.subProducts && removeItem.subProducts.length > 0) {
            selectedList = selectedList.filter((item) => item._relation !== getPrdRelation(removeItem));
          }
        });
      }

      yield put({
        type: 'updateState',
        payload: { selectedList },
      });
      if (selectedList.length === 0) {
        yield put({ type: 'calcContractAmount' });
      } else {
        yield put({ type: 'calcPrdNumAndPayment' });
      }
      yield put({ type: 'checkCanSaveAndAssign' });
    },

    // 删除已选产品
    *deleteSelectedPrd({ payload: { record, index } }, { select, put }) {
      let selectedList = yield select((state) => clone(state[storeName].selectedList));
      // 删除选中产品
      const [removeItem] = selectedList.splice(index, 1);
      // 删除其子产品
      if (removeItem && removeItem.subProducts && removeItem.subProducts.length > 0) {
        selectedList = selectedList.filter((item) => item._relation !== getPrdRelation(removeItem));
      }
      yield put({
        type: 'updateState',
        payload: { selectedList },
      });
      yield put({
        type: 'synUnSelectedLinkRows',
        payload: { record, index },
      });
      if (selectedList.length > 0) {
        yield put({ type: 'calcCoursePromotion' });
      } else {
        yield put({ type: 'clearCoupon' });
        yield put({ type: 'calcContractAmount' });
      }
      yield put({ type: 'checkCanSaveAndAssign' });

      if (removeItem.contractProductId) {
        const isEdit = yield select((state) => state[storeName].isEdit);
        // 修改合同时，缓存已删的合同产品(只针对有班级id的小班、双师、讲座产品)
        if (isEdit && [SMALL_CLASS, TWO_TEACHER, LECTURE].includes(removeItem.productType) && removeItem.classId) {
          let removeContractPrdStorage = yield select((state) => clone(state[storeName].removeContractPrdStorage));
          removeContractPrdStorage[`${removeItem.productId}${removeItem.classId || ''}`] = removeItem.contractProductId;
          yield put({
            type: 'updateState',
            payload: { removeContractPrdStorage },
          });
        }
      }
    },

    // 删除产品时同步取消已勾选的关联产品
    *synUnSelectedLinkRows({ payload: { record } }, { select, put }) {
      if (record.hasOwnProperty('_list')) {
        const linkRowKeys = yield select((state) => {
          let index = state[storeName].linkList.findIndex(
            (item) => item.productId === record.productId && item.classId === record.classId,
          );
          if (index > -1) {
            return clone(state[storeName].linkRowKeys).filter((rowKey) => rowKey !== index);
          }
        });
        yield put({
          type: 'updateState',
          payload: { linkRowKeys },
        });
      }
    },

    // 购物车选择新的产品如果是关联产品的，需要在关联产品内同步勾选
    *synSelectedLinkRows({ payload: { record, index } }, { select, put }) {
      const linkList = yield select((state) => clone(state[storeName].linkList));
      const linkIndex = linkList.findIndex(
        (item) => item.productId === record.productId && item.classId === record.classId,
      );
      if (linkIndex > -1) {
        const payload = yield select((state) => {
          let selectedList = clone(state[storeName].selectedList);
          selectedList.splice(index, 1, clone({ ...linkList[linkIndex], ...record }));
          let linkRowKeys = clone(state[storeName].linkRowKeys);
          linkRowKeys.push(linkIndex);
          return {
            selectedList,
            linkRowKeys,
          };
        });
        yield put({
          type: 'updateState',
          payload,
        });
      }
    },

    // 改变产品数量
    *changePrdQuantity({ payload: { val, index, callback } }, { select, put }) {
      const selectedList = yield select((state) => {
        let selectedList = clone(state[storeName].selectedList);
        selectedList[index].quantity = val || 0;
        selectedList[index].payment = NP.times(val || 0, selectedList[index].price);
        return selectedList;
      });

      yield put({
        type: 'updateState',
        payload: { selectedList },
      });

      // 已选产品优惠的情况
      if (selectedList[index]._prdPromotionVal && selectedList[index]._prdPromotionVal.length > 0) {
        // 计算产品优惠放到回调执行(带防抖)
        runFunc(callback, {
          val: selectedList[index]._prdPromotionVal,
          index,
        });
      } else {
        yield put({
          type: 'calcPrdNumAndPayment',
          payload: { noNeedCalcPromotion: true },
        });
      }
    },

    // 改变产品优惠
    *changePrdPromotion({ payload: { val, index } }, { select, put }) {
      // 存在产品优惠->调接口计算; 不存在产品优惠->清空
      if (Array.isArray(val)) {
        yield put({
          type: 'calcPrdPromotion',
          payload: { val, index },
        });
      } else {
        yield put({
          type: 'clearPrdPromotion',
          payload: { val, index },
        });
      }
    },

    // 计算产品优惠(产品本身关联的优惠)
    *calcPrdPromotion({ payload: { val, index } }, { select, put }) {
      yield put({ type: 'switchCalcLoading' });

      const tagProduct = yield select((state) => state[storeName].selectedList[index]);

      let apiParams = {
        promotionIds: val.join(','),
        price: tagProduct.price,
        totalCourseCount: tagProduct.quantity,
        totalAmount: NP.times(tagProduct.price, tagProduct.quantity),
      };

      const rsp = yield calculatePrdPromotion(apiParams, true).catch(errorsHandle());

      if (rsp && rsp.promotionAmount && rsp.promotionVos && rsp.promotionVos.length) {
        const selectedList = yield select((state) => {
          let selectedList = clone(state[storeName].selectedList);
          selectedList[index]._prdPromotion = rsp.promotionVos;
          selectedList[index]._prdPromotionAmount = rsp.promotionAmount;

          if (rsp.totalAmount > selectedList[index].payment) {
            // 赠送课时的情况下，总额会增加
            selectedList[index]._paymentWithPrdPromotion = rsp.totalAmount;
          }
          // 由于存在赠送课时这种会增加产品总额的情况，所以在这里也更新payment字段
          selectedList[index].payment = rsp.totalAmount;
          // 优惠券金额 或者 已扣优惠
          const otherPromotionAmount =
            (selectedList[index]._discountInfo || {})._couponDiscount ||
            selectedList[index].promotionConsumeAmount ||
            0;

          selectedList[index].promotionAmount = NP.plus(rsp.promotionAmount, otherPromotionAmount);
          // 如果优惠金额还包含其他优惠，则应收金额应使用总金额-优惠金额，不能使用接口返回数据
          selectedList[index].realAmount =
            otherPromotionAmount > 0
              ? NP.minus(selectedList[index].payment, selectedList[index].promotionAmount)
              : rsp.realPayAmount;
          // 应收金额需减去已扣优惠
          selectedList[index].realAmount = NP.minus(
            selectedList[index].realAmount,
            selectedList[index].promotionConsumeAmount || 0,
          );
          // 更新选择的优惠(计算接口报错时，不更新)
          selectedList[index]._prdPromotionVal = val;
          // 重算优惠时删除`清除优惠`及`优惠变更提示`辅助值
          delete selectedList[index]._btnClearPromotionVisible;
          delete selectedList[index]._warningBeforeEditPrdPromotion;
          return selectedList;
        });

        yield put({
          type: 'updateState',
          payload: { selectedList },
        });
      }

      const needCalcCoupon = yield select((state) => {
        return state[storeName].couponList.length > 0 && !state[storeName].isEdit;
      });
      // 已选优惠券的情况需重新计算优惠券
      if (needCalcCoupon) {
        yield put({
          type: 'calcCouponPromotion',
          payload: { initLoading: false },
        });
      } else {
        yield put({ type: 'switchCalcLoading' });
        yield put({ type: 'calcContractAmount' });
      }
    },

    // 清空产品优惠(产品本身关联的优惠)
    *clearPrdPromotion({ payload: { val, index } }, { select, put }) {
      yield put({ type: 'switchCalcLoading' });

      const selectedList = yield select((state) => {
        let selectedList = clone(state[storeName].selectedList);

        selectedList[index]._prdPromotionVal = val;
        selectedList[index].payment = NP.times(selectedList[index].price, selectedList[index].quantity);
        // 去掉产品优惠后，还可以存在 优惠券金额 或者 已扣优惠
        selectedList[index].promotionAmount = selectedList[index]._discountInfo
          ? selectedList[index]._discountInfo._couponDiscount
          : selectedList[index].promotionConsumeAmount || 0;
        selectedList[index].realAmount = NP.minus(
          selectedList[index].payment,
          selectedList[index].promotionAmount || 0,
        );
        delete selectedList[index]._prdPromotion;
        delete selectedList[index]._prdPromotionAmount;
        delete selectedList[index]._paymentWithPrdPromotion;
        // 重算优惠时删除`清除优惠`及`优惠变更提示`辅助值
        delete selectedList[index]._btnClearPromotionVisible;
        delete selectedList[index]._warningBeforeEditPrdPromotion;
        return selectedList;
      });

      yield put({
        type: 'updateState',
        payload: { selectedList },
      });

      const needCalcCoupon = yield select((state) => {
        return state[storeName].couponList.length > 0 && !state[storeName].isEdit;
      });
      // 已选优惠券的情况需重新计算优惠券
      if (needCalcCoupon) {
        yield put({
          type: 'calcCouponPromotion',
          payload: { initLoading: false },
        });
      } else {
        yield put({ type: 'switchCalcLoading' });
        yield put({ type: 'calcContractAmount' });
      }
    },

    // 选择优惠券
    *selectCoupon({ payload: { list } }, { select, put }) {
      yield put({
        type: 'updateState',
        payload: { couponList: list },
      });
      const isAutoCalc = yield select((state) => state[storeName].isAutoCalc);
      // 手动优惠 ，没有优惠券
      if (list.length === 0) {
        yield put({ type: 'clearCouponPromotion' });
      } else if (isAutoCalc) {
        yield put({ type: 'calcCoursePromotion' });
      } else {
        yield put({
          type: 'calcCouponPromotion',
          payload: { initLoading: true },
        });
      }
    },

    // 清空优惠券
    *clearCoupon(actions, { select, put }) {
      const isEdit = yield select((state) => state[storeName].isEdit);
      const couponList = yield select((state) => state[storeName].couponList);
      if (couponList.length > 0 && !isEdit) {
        yield put({
          type: 'updateState',
          payload: { couponList: [] },
        });
        showMessage('当前已选优惠券已不满足使用条件', 'error');
      }
    },

    // 计算产品数量及产品总额
    *calcPrdNumAndPayment({ payload }, { select, put }) {
      const noNeedCalcPromotion = (payload && payload.noNeedCalcPromotion) || false;
      const selectedList = yield select((state) => {
        let list = clone(state[storeName].selectedList);
        list.forEach((item) => {
          const quantity = item.quantity || item.totalCourseHours || null;
          item.quantity = quantity;
          // 产品类型为"其他",并且费用类型为"市场",则产品数量可编辑
          if (item.productType === OTHERS && item.otherCategoryId === OTHER_CATEGORY_TYPE_MARKETING) {
            item._quantityDisabled = false;
          } else {
            item._quantityDisabled = [SMALL_CLASS, TWO_TEACHER, OTHERS, ECS_CLASS].includes(item.productType);
          }
          item._hadInitQuantity =
            (typeof item._hadInitQuantity === 'number' && item._hadInitQuantity) || (Boolean(quantity) ? 1 : 0);
          if (item._paymentWithPrdPromotion) {
            item.payment = item._paymentWithPrdPromotion;
          } else {
            item.payment = isMoreThanZero((quantity && Number(quantity)) || 0) ? NP.times(quantity, item.price) : null;
          }
          if (noNeedCalcPromotion) {
            item.realAmount = item.payment ? NP.minus(item.payment, item.promotionAmount || 0) : null;
          }
        });
        return list;
      });

      yield put({
        type: 'updateState',
        payload: { selectedList },
      });
      if (noNeedCalcPromotion) {
        yield put({ type: 'calcContractAmount' });
      } else {
        yield put({ type: 'calcCoursePromotion' });
      }
    },

    // 计算报读优惠(小班、双师的优惠活动、优惠券)
    *calcCoursePromotion(actions, { select, put }) {
      const isNotEdit = yield select((state) => !state[storeName].isEdit);
      if (isNotEdit) {
        const isAutoCalc = yield select((state) => state[storeName].isAutoCalc);
        if (isAutoCalc) {
          // 活动计算完如果存在已选优惠券则继续计算优惠券
          yield put({ type: 'calcActivePromotion' });
        } else {
          yield put({
            type: 'calcCouponPromotion',
            payload: { initLoading: true },
          });
        }
      } else {
        yield put({ type: 'calcContractAmount' });
      }
    },
    // 计算优惠活动(优惠券)
    *calcActivePromotion(actions, { select, put }) {
      let selectedList = yield select((state) => {
        let newSelected = clone(state[storeName].selectedList);
        return newSelected.map((item) => ({
          ...item,
          // 没有优惠值的才需要初始化值
          promotionAmount: item._activePromotion || item._prdPromotion ? item.promotionAmount : 0,
          realAmount: item._activePromotion || item._prdPromotion ? item.realAmount : item.payment,
        }));
      });

      const hadCoupon = yield select((state) => state[storeName].couponList.length > 0);

      let courses = selectedList
        .filter((item) => [SMALL_CLASS, TWO_TEACHER].includes(item.productType) && item.classId)
        .map((item) => ({
          courseId: calcCouponIdCfg[item.productType](item.classId),
          discount: yuanToCent(item._prdPromotionAmount || 0),
        }));

      let courseIds = courses.map((item) => item.courseId).join(',');
      // 自动时只判断有无产品，手动时还需判断有无优惠券
      if (courses.length > 0) {
        yield put({ type: 'switchCalcLoading' });

        const studentId = yield select((state) => state[storeName].studentId);
        const contractType = yield select((state) => state[storeName].contractType);
        const branchId = yield select((state) => state[storeName].contractBranchId);

        let rsp = yield calculatePromotion(
          {
            courseIds,
            studentId,
            useCase: activeUseCaseCfg[contractType],
            couponId: null,
            calculateCoupon: false,
            // 用于匹配是否使用新的优惠活动计算规则(不传或传空默认使用旧规则)
            branchId,
          },
          true,
        ).catch(errorsHandle());

        if (rsp && rsp.data) {
          const courses = rsp.data.courses.map((course) => ({
            ...course,
            productId: course.productNo,
            classId: course.id.replace('XB-', '').replace('SS-', ''),
            // 优惠活动优惠金额
            _activeDiscount: centToYuan(course.productDiscount),
          }));

          let rewritePrdPromotion = [];

          courses.forEach((course) => {
            const selectedIndex = selectedList.findIndex(
              (item) => item.productId === course.productId && item.classId === course.classId,
            );
            if (selectedIndex > -1) {
              const hadUsedActivePromotion = course.promotionNo && course.promotionName && course.promotionRule;
              const hadUsedPrdPromotion = !!selectedList[selectedIndex]._prdPromotion;

              if (hadUsedActivePromotion) {
                const _promotionNoList = course.promotionNo.split(',');
                const _promotionNameList = course.promotionName.split(',');
                const _promotionRuleList = course.promotionRule.split(',');
                selectedList[selectedIndex]._activePromotion = _promotionNoList.map((id, index) => {
                  // 处理优惠规则（目前仅双师有`promotionActivitys`字段）
                  let ruleDetail = {};
                  let promotionType = null;
                  const { promotionActivitys = [] } = course;
                  const active = promotionActivitys.find((activity) => id === activity.id);
                  if (active && active.rule) {
                    promotionType = active.promotionType;
                    ruleDetail = active.rule;
                  }

                  return {
                    id,
                    name: _promotionNameList[index],
                    rule: _promotionRuleList[index],
                    type: promotionType,
                    ruleDetail,
                  };
                });
              } else {
                selectedList[selectedIndex]._activePromotion = null;
              }
              // 缓存整块计算结果(优惠券、优惠活动分摊到产品里面的金额)
              selectedList[selectedIndex]._discountInfo = {
                ...(selectedList[selectedIndex]._discountInfo || {}),
                ...course,
              };
              // 优惠券优惠金额
              const _couponDiscount = Number(selectedList[selectedIndex]._discountInfo._couponDiscount || 0);

              if (hadUsedActivePromotion) {
                // 优惠券+优惠活动
                selectedList[selectedIndex].promotionAmount = NP.plus(course._activeDiscount, _couponDiscount);
                if (hadUsedPrdPromotion) {
                  delete selectedList[selectedIndex]._prdPromotion;
                  delete selectedList[selectedIndex]._prdPromotionAmount;
                  delete selectedList[selectedIndex]._paymentWithPrdPromotion;
                  selectedList[selectedIndex]._prdPromotionVal = null;
                  rewritePrdPromotion.push(selectedList[selectedIndex].productName);
                }
              } else {
                // 优惠券+产品优惠
                selectedList[selectedIndex].promotionAmount = NP.plus(
                  _couponDiscount,
                  hadUsedPrdPromotion ? selectedList[selectedIndex]._prdPromotionAmount : 0,
                );
              }

              selectedList[selectedIndex].realAmount = NP.minus(
                selectedList[selectedIndex].payment,
                selectedList[selectedIndex].promotionAmount,
              );
            }
          });

          if (rewritePrdPromotion.length > 0) {
            // TODO: 提示产品优惠被覆盖
            console.log('产品优惠被覆盖: ', rewritePrdPromotion);
          }
        }

        if (!hadCoupon) {
          yield put({ type: 'switchCalcLoading' });
        }
      }

      yield put({
        type: 'updateState',
        payload: { selectedList },
      });

      if (hadCoupon) {
        yield put({
          type: 'calcCouponPromotion',
          payload: { initLoading: false },
        });
      } else {
        yield put({ type: 'calcContractAmount' });
      }
    },
    // 计算优惠券
    *calcCouponPromotion({ payload: { initLoading } }, { select, put }) {
      let selectedList = yield select((state) => {
        let newSelected = clone(state[storeName].selectedList);
        return newSelected.map((item) => ({
          ...item,
          // 没有优惠值的才需要初始化值
          promotionAmount: item._activePromotion || item._prdPromotion ? item.promotionAmount : 0,
          realAmount: item._activePromotion || item._prdPromotion ? item.realAmount : item.payment,
        }));
      });

      let couponList = yield select((state) => clone(state[storeName].couponList));

      let bossStudentId = yield select((state) => state[storeName].studentId);

      let lastTimeCalResult = yield select((state) => state[storeName].lastTimeCalResult);

      let couponIds = couponList.map((item) => item.couponId);

      let courses = selectedList
        .filter((item) => [SMALL_CLASS, TWO_TEACHER].includes(item.productType) && item.classId)
        .map((item) => ({
          classId: item.classId,
          productType: item.productType,
          discount: yuanToCent(
            (item._discountInfo && item._discountInfo._activeDiscount) || item._prdPromotionAmount || 0,
          ),
        }));

      if (couponIds.length > 0 && courses.length > 0) {
        if (initLoading) {
          yield put({ type: 'switchCalcLoading' });
        }

        const rsp = yield calculateCoupon(
          {
            couponIds,
            courses,
            bossStudentId,
            // FIXME: 可叠加券与不可叠加券互相切换时，lastTimeCalResult会导致返回data为null，所以暂时不传
            lastTimeCalResult: null,
            type: 'COUPON_UPDATE',
            autoCoupon: false,
            autoPromotion: false,
            // 由于该接口后端无法把过期时间降低到1s内，无法满足当前需要。
            // 但这块限制只针对同参数请求，故在这里增加一个时间戳来保证每次请求参数都不相同
            timestamp: new Date().getTime(),
          },
          true,
        ).catch(errorsHandle());

        if (rsp && rsp.resultCode === 0 && rsp.data) {
          lastTimeCalResult = rsp.data;

          if (Array.isArray(rsp.data.promotionSendCoupons) && rsp.data.promotionSendCoupons.length > 0) {
            // 缓存优惠券
            const rspCouponIdList = rsp.data.promotionSendCoupons.map((item) => item.id);
            // 过滤掉没有命中的优惠券
            couponList = couponList.filter((coupon) => rspCouponIdList.includes(`${coupon.couponId}`));
            rsp.data.promotionSendCoupons.forEach((item) => {
              let index = couponList.findIndex((coupon) => `${coupon.couponId}` === `${item.id}`);
              if (index > -1) {
                couponList[index]._couponId = item.id;
                couponList[index]._couponName = couponList[index].promotionName || item.name;
                couponList[index]._ruleId = item.ruleId;
                couponList[index]._couponDiscount = centToYuan(item.discount);
              }
            });
          } else {
            couponList = [];
          }

          const courses = rsp.data.courses.map((course) => ({
            ...course,
            // 优惠券优惠金额
            _couponDiscount: centToYuan(course.couponDiscountAmount),
          }));

          courses.forEach((course) => {
            const selectedIndex = selectedList.findIndex(
              (item) => item.productId === course.productId && item.classId === course.classId,
            );
            if (selectedIndex > -1) {
              // 缓存整块计算结果(优惠券、优惠活动分摊到产品里面的金额)
              selectedList[selectedIndex]._discountInfo = {
                ...(selectedList[selectedIndex]._discountInfo || {}),
                ...course,
              };
              // 优惠券优惠金额
              const _activeDiscount = Number(selectedList[selectedIndex]._discountInfo._activeDiscount || 0);
              const hadUsedActivePromotion = (selectedList[selectedIndex]._activePromotion || []).length > 0;
              const hadUsedPrdPromotion = !!selectedList[selectedIndex]._prdPromotion;

              if (hadUsedActivePromotion) {
                // 优惠券+优惠活动
                selectedList[selectedIndex].promotionAmount = NP.plus(course._couponDiscount, _activeDiscount);
              } else {
                // 优惠券+产品优惠
                selectedList[selectedIndex].promotionAmount = NP.plus(
                  course._couponDiscount,
                  hadUsedPrdPromotion ? selectedList[selectedIndex]._prdPromotionAmount || 0 : 0,
                );
              }

              selectedList[selectedIndex].realAmount = NP.minus(
                selectedList[selectedIndex].payment,
                selectedList[selectedIndex].promotionAmount,
              );
            }
          });
        } else {
          lastTimeCalResult = null;
        }

        yield put({ type: 'switchCalcLoading' });
      }

      yield put({
        type: 'updateState',
        payload: {
          selectedList,
          couponList,
          lastTimeCalResult,
        },
      });

      yield put({ type: 'calcContractAmount' });
    },

    // (手动优惠)清空优惠券优惠金额
    *clearCouponPromotion(actions, { select, put }) {
      const selectedList = yield select((state) => {
        let list = clone(state[storeName].selectedList);
        let isAutoCalc = state[storeName].isAutoCalc;
        list.forEach((item) => {
          if (isAutoCalc) {
            if (item._discountInfo) {
              item._discountInfo._couponDiscount = 0;
              item.promotionAmount = NP.plus(item._discountInfo._activeDiscount || 0, item._prdPromotionAmount || 0);
            }
          } else {
            delete item._activePromotion;
            delete item._discountInfo;
            item.promotionAmount = item._prdPromotionAmount || 0;
          }
          item.realAmount = NP.minus(item.payment, item.promotionAmount);
        });
        return list;
      });

      yield put({
        type: 'updateState',
        payload: { selectedList },
      });

      yield put({ type: 'calcContractAmount' });
    },

    // 计算统计金额
    *calcContractAmount(actions, { select, put }) {
      const selectedList = yield select((state) => state[storeName].selectedList);

      let totalAmount = 0;
      let promotionAmount = 0;
      let realAmount = 0;

      selectedList.forEach((item) => {
        // 合同产品状态无效的不需统计
        if (item._contractPrdStatus !== UNVALID) {
          totalAmount = NP.plus(totalAmount, item.payment || 0);
          promotionAmount = NP.plus(promotionAmount, item.promotionAmount || 0);
          realAmount = NP.plus(realAmount, item.realAmount || 0);
        }
      });

      yield put({
        type: 'updateState',
        payload: { totalAmount, promotionAmount, realAmount },
      });
    },

    *updateSelectProduct({ payload }, { put }) {
      if (payload.type === 'ADD') {
        yield put({ type: 'cartAddProduct', payload });
      } else if (payload.type === 'DEL') {
        yield put({ type: 'cartDelProduct', payload });
      }
    },

    // 购物车添加产品
    *cartAddProduct({ payload: { data } }, { select, put }) {
      let selectedList = yield select((state) => clone(state[storeName].selectedList));

      const newPrdIndex = selectedList.length;

      let addList = addProductWithSub(data);
      selectedList = selectedList.concat(addList);

      yield put({
        type: 'updateState',
        payload: { selectedList },
      });
      yield put({
        type: 'synSelectedLinkRows',
        payload: {
          record: addList[0],
          index: newPrdIndex,
        },
      });
      yield put({
        type: 'calcPrdNumAndPayment',
        payload: { noNeedCalcPromotion: true },
      });
      yield put({ type: 'checkCanSaveAndAssign' });
    },

    // 购物车删除产品
    *cartDelProduct({ payload: { list, record } }, { select, put }) {
      yield put({
        type: 'updateState',
        payload: { selectedList: clone(list) },
      });
      yield put({
        type: 'synUnSelectedLinkRows',
        payload: { record },
      });
      if (list.length === 0) {
        yield put({ type: 'clearCoupon' });
      }
      yield put({ type: 'calcContractAmount' });
      yield put({ type: 'checkCanSaveAndAssign' });
    },

    // 获取老师字典
    *getTeachers({ payload: { callback } }, { put, call }) {
      const response = yield call(getCurrentBranchTeacher);

      yield put({
        type: 'updateState',
        payload: {
          teacherDict: response.teachers.map((item) => ({
            text: item.teacherName,
            val: item.teacherId,
            title: item.teacherName,
          })),
        },
      });

      if (callback) callback();
    },

    // 切换精英班
    *switchIsEcs({ payload: { checked } }, { select, put }) {
      yield put({
        type: 'updateState',
        payload: { isEcs: checked },
      });
      yield put({ type: 'switchFuncAuth' });
    },
    *switchIsEcsClearSelected({ payload: { checked, callback } }, { select, put }) {
      yield put({
        type: 'updateState',
        payload: {
          isEcs: checked,
          linkExpandKeys: [],
          linkRowKeys: [],
          selectedList: [],
          couponList: [],
        },
      });
      yield put({ type: 'switchFuncAuth' });
      yield put({ type: 'calcContractAmount' });
      showMessage(`${!checked ? '非' : ''}精英班切换成功`, 'success');
      clickPrdTableTab(1);
    },

    // 切换自动/手动计算优惠
    *switchAutoCalc({ payload: { value } }, { select, put }) {
      yield put({
        type: 'updateState',
        payload: { isAutoCalc: value },
      });
      // 手动时，清空已有优惠活动
      if (!value) {
        const selectedList = yield select((state) => {
          let selectedList = clone(state[storeName].selectedList);
          selectedList.forEach((item) => {
            // 只清空优惠活动，不清空优惠券
            if (item._activePromotion) {
              item.promotionAmount = item._discountInfo._couponDiscount || 0;
              item.realAmount = NP.minus(item.payment, item.promotionAmount);
              delete item._discountInfo._activeDiscount;
              delete item._activePromotion;
            }
          });
          return selectedList;
        });
        yield put({
          type: 'updateState',
          payload: { selectedList },
        });
      }
      yield put({ type: 'calcCoursePromotion' });
    },

    // 是否能保存并分配课时
    *checkCanSaveAndAssign(actions, { select, put }) {
      const isEcs = yield select((state) => state[storeName].isEcs);
      let payload = { hadOneToOneOrMorePrd: false };
      if (!isEcs) {
        const selectedList = yield select((state) => state[storeName].selectedList);
        if (selectedList.some((item) => [ONE_ON_ONE_COURSE, ONE_ON_MANY].includes(item.productType))) {
          payload.hadOneToOneOrMorePrd = true;
        }
      }
      yield put({
        type: 'updateState',
        payload,
      });
    },

    // 校验合同
    *validateContract({ payload: { callback } }, { select }) {
      const validateResult = yield select((state) => {
        const _state = state[storeName];
        // 学生信息
        if (!_state.customerContact) {
          return '客户手机不能为空！';
        }
        if (!_state.customerName) {
          return '客户姓名不能为空！';
        }
        if (!_state.studentName) {
          return '学生姓名不能为空！';
        }
        if (!_state.gradeId) {
          return '年级不能为空！';
        }
        if (!_state.schoolName && !loginUser.isTwoTeacherSym()) {
          // 双师的学校非必填
          return '学校不能为空！';
        }
        if (!_state.blCampusId) {
          return '合同学习中心不能为空！';
        }

        let validIdentityData = validIdentityCardByType(_state.customerIdCardType, _state.customerIdCardNo);
        if (!validIdentityData.valid) {
          return '证件格式有误，请检查证件类型及号码';
        }

        // 产品信息
        if (_state.selectedList.length === 0) {
          return '请先选择报读课程产品！';
        }
        if (_state.selectedList.some((item) => !item.quantity)) {
          return '合同产品数量必须大于0并且不能为空！';
        }
        if (_state.selectedList.some((item) => item.realAmount < 0)) {
          return '单个合同产品的应付金额不可小于零！';
        }
        // 收款信息
        if (_state.totalAmount <= 0) {
          return '合同总款必须大于0！';
        }

        return null;
      });

      if (validateResult) {
        showMessage(validateResult, 'error');
      }

      runFunc(callback, !validateResult);
    },

    // 提交合同
    *submit({ payload: { saveType, callback } }, { select, put }) {
      let noTransactionUuid = yield select((state) => !state[storeName].transactionUuid);

      if (noTransactionUuid) {
        const rspTransactionUuid = yield request(`${location.origin}/eduboss/CommonAction/getTransactionUUID.do`, {
          method: 'GET',
        });
        yield put({
          type: 'updateState',
          payload: { transactionUuid: rspTransactionUuid.data },
        });
      }

      // 缓存已使用的已删
      let _removeContractPrdStorage = yield select((state) => clone(state[storeName].removeContractPrdStorage));

      let apiParams = yield select((state) => {
        const _state = state[storeName];

        let apiParams = {
          transactionUuid: _state.transactionUuid,
          contractId: _state.contractId || null,
          contractType: _state.contractType,
          ecsContract: _state.isEcs,
          customerId: _state.customerId,
          customerName: _state.customerName,
          customerContact: _state.customerContact,
          studentId: _state.studentId,
          studentName: _state.studentName,
          blCampusId: _state.blCampusId,
          // 待审核学校的schoolId传空
          schoolId: _state.virtualSchool ? null : _state.schoolId,
          virtualSchool: _state.virtualSchool,
          contractGradeId: _state.gradeId,
          remark: _state.remark,
          transferIntroduceId: _state.transferIntroduceId || null,
          totalAmount: _state.totalAmount || 0,
          promotionAmount: _state.promotionAmount || 0,
          realAmount: _state.realAmount || 0,
          promotionCouponArr: [],
          contractProductArr: [],
          isAutoCalc: _state.isAutoCalc,
          customerIdCardType: _state.customerIdCardType,
          customerIdCardNo: _state.customerIdCardNo,
        };

        apiParams.promotionCouponArr = _state.couponList.map((item) => ({
          promotionCouponName: item._couponName,
          promotionCouponId: item._couponId,
          promotionCouponRuleId: item._ruleId,
          promotionCouponAmount: item._couponDiscount,
          promotionNature: item.promotionNature,
        }));

        apiParams.contractProductArr = _state.selectedList.map((item) => {
          let contractProduct = {
            contractProductId: item.contractProductId || null,
            productType: item.productType,
            productId: item.productId,
            productVersionId: item.productVersionId || null,
            classId: item.classId || null,
            gradeId: item.gradeId || null,
            classTypeId: item.classTypeId,
            continueClassId: item.continueClassId || null,
            extendClassId: item.extendClassId || null,
            courseHours: item.quantity,
            subjectId: item.subjectId || null,
            startClassDate: item._courseDate || item.startClassDate,
            startClassTime: item.startClassTime,
            promotionArr: (item._prdPromotion || []).map((item) => ({
              promotionId: item.id,
              promotionName: item.name,
            })),
            // 自动计算优惠才需要传优惠活动
            promotionActivityArr:
              _state.isEdit || _state.isAutoCalc
                ? (item._activePromotion || []).map((item) => ({
                    promotionActivityName: item.name,
                    promotionActivityId: item.id,
                    promotionActivityRuleId: item.rule,
                  }))
                : [],
            totalAmount: item.payment,
            promotionAmount: item.promotionAmount,
            realPayAmount: item.realAmount || 0,
            // 子产品需带父产品班级id(小班才会用到)
            relateClassId: item._relateClassId || null,
            relateProductType: item._relateProductType || null,
            // 合同产品状态是无效的才传(取消操作)
            status: item._contractPrdStatus === UNVALID ? UNVALID : null,
          };
          // 修改合同时匹配是否存在已删合同产品
          if (
            _state.isEdit &&
            _removeContractPrdStorage.hasOwnProperty(`${contractProduct.productId}${contractProduct.classId || ''}`)
          ) {
            contractProduct.contractProductId =
              _removeContractPrdStorage[`${contractProduct.productId}${contractProduct.classId || ''}`];
            // 缓存的已删合同产品只允许使用一次，防止产生重复的错误数据
            delete _removeContractPrdStorage[`${contractProduct.productId}${contractProduct.classId || ''}`];
          }
          return contractProduct;
        });
        return apiParams;
      });

      let rsp;
      if (apiParams.contractId) {
        // 修改
        rsp = yield editContractInfo(apiParams, true).catch(errorsHandle());
      } else {
        // 新签
        rsp = yield saveContractInfoNew(apiParams, true).catch(errorsHandle());
      }

      let contractId = null;

      if (rsp) {
        contractId = rsp.data;
        showMessage('合同保存成功！', 'success');
        yield put({
          type: 'updateState',
          payload: { removeContractPrdStorage: _removeContractPrdStorage },
        });
      }

      runFunc(callback, contractId);
    },

    // (修改合同)清除优惠
    *clearPromotion({ payload: { index } }, { select, put }) {
      const selectedList = yield select((state) => {
        let list = clone(state[storeName].selectedList);
        let tag = list[index];
        tag.promotionAmount = tag.promotionConsumeAmount || 0;
        tag.realAmount = NP.minus(tag.payment, tag.promotionAmount);
        tag._paymentWithPrdPromotion = 0;
        tag._btnClearPromotionVisible = false;
        tag._warningBeforeEditPrdPromotion = false;
        delete tag._prdPromotionVal;
        delete tag._prdPromotion;
        delete tag._activePromotion;
        return list;
      });
      yield put({
        type: 'updateState',
        payload: { selectedList },
      });

      yield put({ type: 'calcContractAmount' });
    },

    // 无效/撤销无效 合同产品
    *switchContractPrdInvalid({ payload: { record, index } }, { select, put }) {
      let selectedList = yield select((state) => {
        let list = clone(state[storeName].selectedList);
        list[index]._contractPrdStatus = record._contractPrdStatus === UNVALID ? record._statusCache : UNVALID;
        return list;
      });

      yield put({
        type: 'updateState',
        payload: { selectedList },
      });

      yield put({ type: 'calcContractAmount' });
    },

    // 新增合同时，校验是否存在同名学生
    *checkSameStudent({ payload: { callback } }, { select, put }) {
      const { customerId, studentId } = yield select((state) => ({
        customerId: state[storeName].customerId,
        studentId: state[storeName].studentId,
      }));

      if (studentId) {
        const options = null;
        runFunc(callback, options);
      } else if (customerId) {
        yield put({
          type: 'checkSameStudentByCustomer',
          payload: { callback },
        });
      } else {
        yield put({
          type: 'checkSameStudentByName',
          payload: { callback },
        });
      }
    },

    // 新增合同时，没有学生id的需要查询是否存在同名学生
    *checkSameStudentByCustomer({ payload: { callback } }, { select }) {
      const apiParams = yield select((state) => ({
        cusId: state[storeName].customerId,
        stuId: state[storeName].studentId,
        stuName: state[storeName].studentName,
      }));

      const rsp = yield checkCustomerStudentName(apiParams, true).catch(errorsHandle());
      const options = rsp ? null : {};

      runFunc(callback, options);
    },

    // 新增合同时，没有客户id的学生需要查询是否存在同名学生
    *checkSameStudentByName({ payload: { callback } }, { select }) {
      const apiParams = yield select((state) => ({
        studentId: state[storeName].studentId,
        studentName: state[storeName].studentName,
        conCampusId: state[storeName].blCampusId,
      }));

      let fetchList = [];
      for (let i = 1; i < 3; i++) {
        fetchList.push(getStudentListByName({ ...apiParams, brenchType: i }, true));
      }
      const [rspSame, rspDiff] = yield Promise.all(fetchList).catch(errorsHandle());

      let options = null;

      if (Array.isArray(rspSame) && rspSame.length > 0) {
        options = {
          type: 'modal',
          studentName: apiParams.studentName,
          sameList: rspSame,
          diffList: [],
        };
      }
      if (Array.isArray(rspDiff) && rspDiff.length > 0) {
        options = {
          type: 'modal',
          studentName: apiParams.studentName,
          sameList: [],
          ...(options || {}),
          diffList: rspDiff,
        };
      }

      runFunc(callback, options);
    },
  },
};
