import React from 'react';
import { clone } from 'lodash';
import { connect } from 'dva';
import { Row, Col } from 'antd';
import PropTypes from 'prop-types';
import { runFunc, loginUser } from 'UTIL/common';
import style from './styles.module.less';
import { o2mTypeDict } from 'CONST/dicts/product';
import { storeName } from '../../store/storeCfg';
import { Button, Select, Search2, Loading, Input } from 'COMPONENT';
import { getRelationsDict, getRelationPayload } from 'MODEL/dict/utils/index';
import { productType, productSeries, classType, classLevel } from 'MODEL/dict/utils/config';
import {
  SMALL_CLASS,
  ECS_CLASS,
  productTypeAllow,
  OTHERS,
  ONE_ON_ONE_COURSE,
  LECTURE,
  ONE_ON_MANY,
  characterTextCfg,
  otherOfCategoryDictCfg,
  SPECIAL_PRODUCT_TYPE,
  CLASS_TYPE_BROADCAST_COOPERATE,
  PRODUCT_GROUP_BROADCAST_COOPERATE,
} from '../config';

// 空的搜索条件
const emptySKey = {
  productName: null,
  className: null,
  assistTeacherId: null,
  campusId: null,
  courseSeriesId: null,
  classTypeId: null,
  classLevelId: null,
  gradeId: null,
  subjectId: null,
  productVersionId: null,
  quarterId: [],
  phaseId: null,
  productGroupId: null,
  teacherId: null,
  otmType: null,
  characterId: null,
  otherOfCategoryId: null,
  otherCategoryId: null,
  ecsClassType: null,
  contractPayCategory: null,
};
Object.freeze(emptySKey);

class Search extends React.Component {
  static PropTypes = {
    initSKey: PropTypes.object,
    onSearch: PropTypes.func.isRequired,
    isAlready: PropTypes.bool,
  };

  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      ...emptySKey,
    };
    this.isInit = false;
    this.defaultSKey = null;
    this.isTwoTeacher = loginUser.isTwoTeacherSym();

    // 产品类型关联关系type、key对应关系配置
    this.productTypeTreeMap = {
      [productType]: 'productType',
      [productSeries]: 'courseSeriesId',
      [classType]: 'classTypeId',
      [classLevel]: 'classLevel',
    };
  }

  componentDidUpdate(prevProps) {
    if (!this.isInit && !prevProps.initSKey && this.props.initSKey) {
      const payload = this.formatSKey();

      this.isInit = true;
      this.defaultSKey = payload;
      this.setState({ ...payload });
    }
  }

  // 平铺项数据
  getSearchCfg = () => {
    return this.formatXingHuoCfg();
  };

  /**
   * 星火配置
   * @return {*[]|Array}
   */
  formatXingHuoCfg = () => {
    const {
      isEcs,
      yearDict,
      gradeDict,
      phaseDict,
      quarterDict,
      subjectDict,
      weekDayDict,
      productTypeTree,
      ecsClassTypeDict,
      selectProductType,
      productCharacter,
      otherCategoryDict,
      otherOfCategoryDict,
      oneOnManyProductGroupDict,
      smallClassProductGroupDict,
    } = this.props;

    const courseSeriesDict = getRelationsDict(productSeries)(productTypeTree, selectProductType);
    const classTypeDict = getRelationsDict(classType)(productTypeTree, selectProductType);
    const classLevelDict = getRelationsDict(classLevel)(productTypeTree, selectProductType, this.state.classTypeId);
    const productCharacterDict = this.props.isEdit
      ? (productCharacter || []).filter((opt) => opt.text === characterTextCfg[this.props.contractCharacter])
      : productCharacter;
    const _phaseDict = (weekDayDict || []).concat(phaseDict || []);

    this.classTypeDict = classTypeDict;
    this.courseSeriesDict = courseSeriesDict;
    this.oneOnManyProductGroupDict = oneOnManyProductGroupDict;

    // 小班过滤幼儿园和毕业, 以及默认值是幼儿园和小班是需要变为全部
    let _gradeDict = gradeDict;
    if (selectProductType === SMALL_CLASS) {
      _gradeDict = gradeDict.filter(({ text }) => !['幼儿园', '毕业'].includes(text));
    }

    // 精英班没有 幼儿园
    if (selectProductType === ECS_CLASS) {
      _gradeDict = gradeDict.filter(({ text }) => !['幼儿园'].includes(text));
    }

    const characterOption = {
      label: '产品性质',
      dict: productCharacterDict,
      field: 'characterId',
      value: this.state.characterId,
      hadOptionAll: !this.props.isEdit,
    };

    // 小班
    if (SMALL_CLASS === selectProductType) {
      return [
        {
          label: '班级类型',
          dict: this.formatClassTypeDict(classTypeDict),
          field: 'classTypeId',
          value: this.state.classTypeId,
          hadOptionAll: this.props.isEdit ? !this.props.isBroadcastCooperate : true,
        },
        {
          label: '产品系列',
          dict: this.formatCourseSeriesDict(courseSeriesDict),
          field: 'courseSeriesId',
          value: this.state.courseSeriesId,
          type: 'multi',
          hadOptionAll: this.props.isEdit ? false : true,
        },
        { label: '班级层次', dict: classLevelDict, field: 'classLevelId', value: this.state.classLevelId },
        { label: '科目', dict: subjectDict, field: 'subjectId', value: this.state.subjectId },
        { label: '产品季度', dict: quarterDict, field: 'quarterId', value: this.state.quarterId, type: 'multi' },
        characterOption,
        { label: '年级', dict: _gradeDict, field: 'gradeId', value: this.state.gradeId },
        { label: '产品年份', dict: yearDict, field: 'productVersionId', value: this.state.productVersionId },
        { label: '期数', dict: _phaseDict, field: 'phaseId', value: this.state.phaseId },
        {
          label: '产品组',
          dict: smallClassProductGroupDict,
          field: 'productGroupId',
          value: this.state.productGroupId,
        },
      ];
    }

    // 一对一
    if (ONE_ON_ONE_COURSE === selectProductType) {
      return [
        { label: '产品系列', dict: courseSeriesDict, field: 'courseSeriesId', value: this.state.courseSeriesId },
        characterOption,
        { label: '年级', dict: _gradeDict, field: 'gradeId', value: this.state.gradeId },
        { label: '产品年份', dict: yearDict, field: 'productVersionId', value: this.state.productVersionId },
      ];
    }

    // 一对多
    if (ONE_ON_MANY === selectProductType) {
      return [
        {
          label: '产品系列',
          dict: this.formatCourseSeriesDict(courseSeriesDict),
          field: 'courseSeriesId',
          value: this.state.courseSeriesId,
          type: 'multi',
          hadOptionAll: this.props.isEdit ? false : true,
        },
        characterOption,
        {
          label: '产品组',
          dict: this.formatProductGroupDict(oneOnManyProductGroupDict),
          field: 'productGroupId',
          value: this.state.productGroupId,
          type: 'multi',
          hadOptionAll: this.props.isEdit ? false : true,
        },
        { label: '一对多类型', dict: o2mTypeDict, field: 'otmType', value: this.state.otmType },
        { label: '年级', dict: _gradeDict, field: 'gradeId', value: this.state.gradeId },
        { label: '产品年份', dict: yearDict, field: 'productVersionId', value: this.state.productVersionId },
      ];
    }

    // 讲座
    if (LECTURE === selectProductType) {
      return [
        { label: '年级', dict: _gradeDict, field: 'gradeId', value: this.state.gradeId },
        characterOption,
        { label: '产品年份', dict: yearDict, field: 'productVersionId', value: this.state.productVersionId },
      ];
    }

    // 其他产品
    if (OTHERS === selectProductType) {
      let _otherOfCategoryDict = otherOfCategoryDict;

      const mainConfigListForOthers = [
        { label: '产品年份', dict: yearDict, field: 'productVersionId', value: this.state.productVersionId },
        { label: '费用类型', dict: otherCategoryDict, field: 'otherCategoryId', value: this.state.otherCategoryId },
      ];

      return isEcs
        ? mainConfigListForOthers
        : mainConfigListForOthers.concat([
            characterOption,
            {
              label: '所属产品类型',
              dict:
                this.props.isEdit && this.props.contractProductTag !== OTHERS
                  ? otherOfCategoryDictCfg[this.props.contractProductTag]
                  : _otherOfCategoryDict,
              field: 'otherOfCategoryId',
              value: this.state.otherOfCategoryId,
              hadOptionAll: !this.props.isEdit,
            },
          ]);
    }

    // 精英班
    if (ECS_CLASS === selectProductType) {
      return [
        { label: '年级', dict: _gradeDict, field: 'gradeId', value: this.state.gradeId },
        { label: '产品年份', dict: yearDict, field: 'productVersionId', value: this.state.productVersionId },
        { label: '产品季度', dict: quarterDict, field: 'quarterId', value: this.state.quarterId, type: 'multi' },
        { label: '产品类别', dict: ecsClassTypeDict, field: 'ecsClassType', value: this.state.ecsClassType },
      ];
    }

    return [];
  };

  /**
   * 默认显示行数
   * @return {number}
   */
  formatSearchRowNum = () => {
    return SMALL_CLASS === this.props.selectProductType ? 5 : 10;
  };

  formatCourseSeriesDict = (courseSeriesDict) => {
    const { isEdit, isQualityContract } = this.props;
    return isEdit
      ? (courseSeriesDict || []).map((item) => ({
          ...item,
          disabled: true,
          visible: isQualityContract ? item.text === '素质' : item.text !== '素质',
        }))
      : courseSeriesDict;
  };

  formatClassTypeDict = (classTypeDict) => {
    const { isEdit, isBroadcastCooperate } = this.props;
    return isEdit
      ? (classTypeDict || []).map((item) => ({
          ...item,
          disabled: false,
          visible: isBroadcastCooperate ? item.val === CLASS_TYPE_BROADCAST_COOPERATE : true,
        }))
      : classTypeDict;
  };

  formatProductGroupDict = (productGroup = []) => {
    const { isEdit, isBroadcastCooperate } = this.props;
    return isEdit
      ? (productGroup || []).map((item) => ({
          ...item,
          disabled: true,
          visible: isBroadcastCooperate
            ? item.val === PRODUCT_GROUP_BROADCAST_COOPERATE
            : item.val !== PRODUCT_GROUP_BROADCAST_COOPERATE,
        }))
      : productGroup;
  };

  onSearchChange = (params) => {
    this.setState(
      (prevState) => {
        let payload = {
          className: prevState.className,
          assistTeacherId: prevState.assistTeacherId,
          blCampusId: prevState.blCampusId,
          ...params,
        };

        if (prevState.classTypeId !== params.classTypeId) {
          payload = {
            ...payload,
            ...getRelationPayload('classTypeId', params.classTypeId, {
              type: classType,
              dictTree: this.props.productTypeTree,
              state: this.props,
              map: this.productTypeTreeMap,
            }),
          };
        } else if (prevState.productType !== params.productType) {
          payload = {
            ...payload,
            assistTeacherId: null,
            ...getRelationPayload('productType', params.productType, {
              type: productType,
              dictTree: this.props.productTypeTree,
              state: this.state,
              map: this.productTypeTreeMap,
            }),
          };
        }

        return payload;
      },
      () => {
        this.submit();
      },
    );
  };

  submit = () => {
    const { onSearch } = this.props;
    let params = clone(this.state);

    delete params.loading;
    delete params.searchVal;
    delete params.productTypeId;

    runFunc(onSearch, {
      ...params,
    });
  };

  resetSKey = () => {
    const { selectProductType } = this.props;
    const payload = SPECIAL_PRODUCT_TYPE[selectProductType] ? this.formatSpecialKey() : this.formatSKey();
    this.setState(payload, () => runFunc(this.props.onSearch, payload));
  };

  formatSpecialKey = () => {
    let courseSeriesId;
    let classTypeId;
    let productGroupId;
    const payload = this.formatSKey();
    const { isEdit, isQualityContract, selectedList = [], isBroadcastCooperate, selectProductType } = this.props;

    if (isEdit) {
      courseSeriesId = this.getCourseSeriesId(isQualityContract);
      classTypeId = this.getClassTypeId(isBroadcastCooperate);
      productGroupId = selectProductType === ONE_ON_MANY ? this.getProductGroupId(isBroadcastCooperate) : '';
    } else {
      const specialSelectedList = selectedList.filter(({ productType }) => productType === selectProductType); // 小班合同
      if (specialSelectedList.length) {
        courseSeriesId = this.getCourseSeriesId(
          !!specialSelectedList.filter(({ courseSeriesName }) => courseSeriesName === '素质').length,
        );
        classTypeId = this.getClassTypeId(
          !!specialSelectedList.filter(
            ({ courseSeriesName, classTypeId }) =>
              courseSeriesName === '素质' && classTypeId === CLASS_TYPE_BROADCAST_COOPERATE,
          ).length,
        );
        productGroupId =
          selectProductType === ONE_ON_MANY
            ? this.getProductGroupId(
                !!specialSelectedList.filter(
                  ({ courseSeriesName, productGroupId }) =>
                    courseSeriesName === '素质' && productGroupId === PRODUCT_GROUP_BROADCAST_COOPERATE,
                ).length,
              )
            : '';
      } else {
        classTypeId = '';
        courseSeriesId = [];
        productGroupId = selectProductType === ONE_ON_MANY ? [] : '';
      }
    }

    payload.classTypeId = classTypeId;
    payload.courseSeriesId = courseSeriesId;
    payload.productGroupId = productGroupId;

    return payload;
  };

  resetCourseSeriesId = (isReset, isQuality, isBroadcastCooperate, isTabChange = false) => {
    const { courseSeriesId } = this.state;
    const { selectProductType } = this.props;
    const isShouldReset =
      SPECIAL_PRODUCT_TYPE[selectProductType] && (!courseSeriesId || !courseSeriesId.length || isBroadcastCooperate);

    if (isShouldReset) {
      // 小班并且产品系列为全部时起效
      const courseSeriesIds = isReset ? [] : this.getCourseSeriesId(isQuality);
      const classTypeId =
        selectProductType === SMALL_CLASS ? (isReset ? '' : this.getClassTypeId(isBroadcastCooperate)) : '';
      const productGroupId =
        selectProductType === ONE_ON_MANY ? (isReset ? [] : this.getProductGroupId(isBroadcastCooperate)) : '';
      this.setState({ ...this.state, classTypeId, courseSeriesId: courseSeriesIds, productGroupId }, () =>
        this.submit(),
      );
    }

    if (!isShouldReset && isTabChange) {
      this.resetSKey();
    }
  };

  getCourseSeriesId = (isQuality) => {
    return isQuality
      ? this.courseSeriesDict
          .filter(({ text }) => {
            return text === '素质';
          })
          .map(({ val }) => val)
      : this.courseSeriesDict
          .filter(({ text }) => {
            return text !== '素质';
          })
          .map(({ val }) => val);
  };

  getClassTypeId = (isBroadcastCooperate) => {
    return isBroadcastCooperate ? CLASS_TYPE_BROADCAST_COOPERATE : '';
  };

  getProductGroupId = (isBroadcastCooperate) => {
    return isBroadcastCooperate
      ? this.oneOnManyProductGroupDict
          .filter(({ val }) => {
            return val === PRODUCT_GROUP_BROADCAST_COOPERATE;
          })
          .map(({ val }) => val)
      : this.oneOnManyProductGroupDict
          .filter(({ val }) => {
            return val !== PRODUCT_GROUP_BROADCAST_COOPERATE;
          })
          .map(({ val }) => val);
  };

  /**
   * 格式化最新的搜索条件
   * tips：因componentDidUpdate缺陷，必须这样做
   */
  formatSKey = () => {
    return {
      ...emptySKey,
      ...(this.defaultSKey || {}),
      ...(this.props.initSKey || {}),
    };
  };

  // 根据类型返回不同的搜索框
  getSearchDom() {
    const { selectProductType, campusDict, teacherDict } = this.props;

    // 细粒化查询表单节点
    const { productName, className, teacherId, campusId } = this.state;

    const productNameDom = (
      <Input
        placeholder="产品名称"
        value={productName}
        onChange={(e) => this.setState({ productName: e.target.value })}
      />
    );
    const classNameDom = (
      <Input placeholder="班级名称" value={className} onChange={(e) => this.setState({ className: e.target.value })} />
    );
    const mainTeacherNameDom = (
      <Select
        placeholder="班级老师"
        dict={teacherDict}
        showSearch
        value={teacherId}
        onChange={(val) => {
          this.setState({ teacherId: val });
        }}
      />
    );
    const blCampusIdDom = (
      <Select
        placeholder="班级学习中心"
        dict={campusDict}
        showSearch
        value={campusId}
        onChange={(val) => {
          this.setState({ campusId: val });
        }}
      />
    );

    const dom0 = (
      <Row className={style.searchBarKey}>
        <Col span={6}>{productNameDom}</Col>
        <Col span={6}>{classNameDom}</Col>
        <Col span={6}>{mainTeacherNameDom}</Col>
        <Col span={6}>{blCampusIdDom}</Col>
      </Row>
    );

    const dom1 = (
      <Row className={style.searchBarKey}>
        <Col span={6}>{productNameDom}</Col>
      </Row>
    );

    // 根据不同系统 与不同 产品类型进行拼凑查询表单
    switch (selectProductType) {
      case SMALL_CLASS:
        return dom0;
      case ONE_ON_MANY:
      case ONE_ON_ONE_COURSE:
      case OTHERS:
      case ECS_CLASS:
        return dom1;
      case LECTURE:
        return (
          <Row className={style.searchBarKey}>
            <Col span={6}>{productNameDom}</Col>
            <Col span={6}>{blCampusIdDom}</Col>
          </Row>
        );
      default:
        return null;
    }
  }

  render() {
    return (
      <div className={style.Search}>
        <div className={style.searchBar}>
          {this.getSearchDom()}
          <div className={style.searchBarBtn}>
            <Button type="primary" onClick={this.submit}>
              查询
            </Button>
            <Button onClick={this.resetSKey}>重置</Button>
          </div>
        </div>
        <Search2
          defaultExpand={false}
          style={{ marginTop: 20 }}
          config={this.getSearchCfg()}
          onChange={this.onSearchChange}
          rowNum={this.formatSearchRowNum()}
          key={this.props.selectProductType}
        />
        <Loading className={style.loading} loading={this.state.loading} />
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  const { dict, account } = state;
  const {
    year,
    grade = [],
    subject = [],
    twoTcMainTc,
    twoTcAuxTc,
    smallClassMainTc,
    smallClassAuxTc,
    phase = [],
    weekDay = [],
    quarter = [],
    productGroup = [],
    branchAllCampus = [],
    productCharacter = [],
    other_category,
    other_of_category,
  } = dict;

  let productTypeTreeStorage = null;
  if (!productTypeTreeStorage && dict.productTypeTree) {
    productTypeTreeStorage = dict.productTypeTree.filter(
      (item) => productTypeAllow.includes(item.val) || item.val === ECS_CLASS,
    );
  }

  const { serverDate } = account.user;
  const yearDict = year ? year.filter((item) => Number(item.text) >= new Date(serverDate).getFullYear() - 1) : [];
  const smallClassProductGroupDict = productGroup ? productGroup[SMALL_CLASS] : null;
  const oneOnManyProductGroupDict = productGroup ? productGroup[ONE_ON_MANY] : null;

  return {
    isEdit: state[storeName].isEdit,
    contractCharacter: state[storeName].contractCharacter,
    contractProductTag: state[storeName].contractProductTag,

    twoTcMainTcDict: twoTcMainTc,
    twoTcAuxTcDict: twoTcAuxTc,
    smallClassMainTcDict: smallClassMainTc,
    smallClassAuxTcDict: smallClassAuxTc,
    campusDict: branchAllCampus,
    productTypeTree: productTypeTreeStorage || [],
    gradeDict: grade || [],
    subjectDict: subject,
    yearDict: yearDict,
    quarterDict: quarter,
    phaseDict: phase,
    weekDayDict: weekDay,
    ecsClassTypeDict: state.dict.ecsClassType,
    serverDate,
    productCharacter,
    otherCategoryDict: other_category || [],
    otherOfCategoryDict: other_of_category || [],
    teacherDict: state[storeName].teacherDict,
    productGroupDict: productGroup,
    smallClassProductGroupDict,
    oneOnManyProductGroupDict,
    grade: state[storeName].grade || {},
    isEcs: state[storeName].isEcs,
    gradeId: state[storeName].gradeId,
  };
};

export default connect(mapStateToProps, null, null, { withRef: true })(Search);
