import React from 'react';
import PropTypes from 'prop-types';
import style from './styles.module.less';
import { ToBoSelected } from 'COMPONENT/ShoppingCart';
import { runFunc } from 'UTIL/common';
import { getTimeRange } from '../utils';
import { Tooltip } from 'antd';
import { LECTURE, ONE_ON_MANY, SMALL_CLASS, productTypeAllow, ECS_CLASS, ONE_ON_ONE_COURSE, OTHERS } from '../config';
import SeasonSubjectMark from '../../components/SeasonSubjectMark';

// 显示产品组的产品类型
const SHOW_PRODUCT_GROUP_TYPE = {
  [SMALL_CLASS]: 1, // 班课
  [ONE_ON_MANY]: 1, // 1对多
};

const ClassInfo = (props) => {
  return (
    <span className={style.ClassInfo} style={{ display: props.text ? 'inline-block' : 'none' }}>
      {props.text}
    </span>
  );
};

const MoreInfo = (props) => {
  return (
    <span className={style.MoreInfo} style={{ display: props.text ? 'inline-block' : 'none' }}>
      <span>{props.label}：</span>
      <span>{props.text}</span>
    </span>
  );
};
MoreInfo.propTypes = {
  label: PropTypes.string.isRequired,
  text: PropTypes.string,
};

const Price = (props) => {
  return (
    <div className={style.Price}>
      <span>¥</span>
      <span>{props.num}</span>
    </div>
  );
};

const hasValue = (val) => val !== undefined && val !== null;

const formatInfo = (val) => (hasValue(val) ? val : '-');

const Content = (props) => {
  const isShow = productTypeAllow.includes(props.productType);

  return (
    <div className={style.Content}>
      <div className={style.mainContent}>
        <div className={style.header}>
          {/* 续报tab才显示 */}
          {props.isRenew && <SeasonSubjectMark {...props} />}
          <span>
            {props.productType === LECTURE
              ? props.className || props.productName
              : props.productType === SMALL_CLASS
              ? props.className
              : props.productName}
          </span>
        </div>

        <div className={style.body}>
          {props.productTypeName && (
            <Tooltip title="产品类型">
              <span className={style.titleType}>{props.productTypeName}</span>
            </Tooltip>
          )}
          {props.productType === OTHERS && props.otherCategoryName && (
            <Tooltip title="费用类型">
              <span className={style.titleType}>{props.otherCategoryName}</span>
            </Tooltip>
          )}
          {props.productType === ONE_ON_ONE_COURSE && props.productGroupName && (
            <Tooltip title="产品组">
              <span className={style.titleType}>{props.productGroupName}</span>
            </Tooltip>
          )}
          {isShow && props.courseSeriesName && (
            <Tooltip title="产品系列">
              <span className={style.titleType}>{props.courseSeriesName}</span>
            </Tooltip>
          )}
          {isShow && props.classTypeName && (
            <Tooltip title="班级类型">
              <span className={style.titleTypeLeft}>{props.classTypeName}</span>
            </Tooltip>
          )}
          {isShow && props.classTypeName && props.classLevelName && <span className={style.titleTypeMid}>、</span>}
          {isShow && props.classLevelName && (
            <Tooltip title="班级层次">
              <span className={style.titleTypeRight}>{props.classLevelName}</span>
            </Tooltip>
          )}
          {props.productType === ECS_CLASS && props.ecsClassTypeName && (
            <Tooltip title="产品类别">
              <span className={style.titleType}>{props.ecsClassTypeName}</span>
            </Tooltip>
          )}
          {SHOW_PRODUCT_GROUP_TYPE[props.productType] && props.productGroupName && (
            <Tooltip title="产品组">
              <span className={style.titleType}>{props.productGroupName}</span>
            </Tooltip>
          )}
        </div>

        <div className={style.footer}>
          {props.classId ? (
            [
              <MoreInfo
                label="老师"
                text={props.assistTeacherName || props.mainTeacherName || '暂无'}
                key={`${props.index}_teacherName`}
              />,
              <MoreInfo label="下次上课日期" text={props.nextCourseDate} key={`${props.index}_nextTime`} />,
            ]
          ) : (
            <ClassInfo text="暂无课程安排" />
          )}
          <MoreInfo label="截止日期" text={props.validEndDate} />
          {props.classId && [
            <MoreInfo label="学习中心" text={props.campusName} key={`${props.index}_campusName`} />,
            <MoreInfo
              label="已上小时"
              text={`${formatInfo(props.consumeCourseNatureHours)}/${formatInfo(props.totalCourseNatureHours)}`}
              key={`${props.index}_courseTime`}
            />,
            <MoreInfo
              label="已报人数"
              text={`${formatInfo(props.peopleQuantity)}/${formatInfo(props.maxPeopleQuantity)}`}
              key={`${props.index}_regPeople`}
            />,
            <MoreInfo
              label="上课时间"
              text={getTimeRange(props.startClassTime, props.classLength)}
              key={`${props.index}_classTimeText`}
            />,
          ]}
        </div>
      </div>

      <div className={style.subContent}>
        {props.price && <Price num={/(LECTURE)|(OTHERS)/.test(props.productType) ? props.price : props.naturePrice} />}
      </div>
    </div>
  );
};

const ToBoSelectedCard = (props) => {
  return (
    <ToBoSelected
      className={style.ToBoSelectedCard}
      onClick={() => {
        runFunc(props.onSelect, props.index);
      }}
    >
      <Content {...props} />
    </ToBoSelected>
  );
};

export default ToBoSelectedCard;
