import React from 'react';
import classNames from 'classnames';
import { Tabs, Modal } from 'antd';
import PropTypes from 'prop-types';
import { connect } from 'dva';
import Search from './Search';
import { getQuarterId } from './utils';
import { Deck, Loading, NoDataPlus } from 'COMPONENT';
import { getBLoYearId, getDictValByText } from 'UTIL/dict';
import style from './styles.module.less';
import classCardRender from './ClassCardRender';
import SearchResult from './SearchResult';
import { storeName } from '../store/storeCfg';
import ModalSelectCourseTime from 'BLOC/ModalSelectCourseTime';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import {
  findMiniClassWillBuyHour,
  findClassWillBuyHour,
  selectProductForContract,
  queryProductInfoNew,
} from 'SERVICE/contract';
import {
  systemProduct,
  productTypeAllow,
  SMALL_CLASS,
  ECS_CLASS,
  ONE_ON_MANY,
  ONE_ON_ONE_COURSE,
  systemProductTab,
  quantityResetType,
  NoAcquiescenceType,
  LECTURE,
  OTHERS,
  WARNING_MESSAGE,
  characterTextCfg,
  SPECIAL_PRODUCT_TYPE,
  otherOfCategoryValCfg,
  CLASS_TYPE_BROADCAST_COOPERATE,
  PRODUCT_GROUP_BROADCAST_COOPERATE,
  RENEWAL_CLASS,
} from './config';
import { checkSamePrd } from '../store/storeUtils';
import { runFunc, errorsHandle } from 'UTIL/common';
import { contractGroupMap } from '../config';
import { ADD_PRODUCT } from '../store/storeCfg';

const { TabPane } = Tabs;
const errorsHandler = errorsHandle();

class AddProduct extends React.Component {
  constructor(props) {
    super(props);

    this.switchLoading = this.switchLoading.bind(this);
    this.loadRenewalClass = this.loadRenewalClass.bind(this);

    let { productTypeId } = props.defaultValue || {};
    if (!productTypeId) {
      productTypeId = (props.isEcs ? systemProductTab[ECS_CLASS] : systemProductTab.default)[0].val;
    }
    if (props.isEdit) {
      productTypeId = props.contractProductTag;
    }

    this.initValue = {
      page: 1,
      rows: 5,
      searchKey: {}, // 查询的值
    };
    this.state = {
      ...this.initValue,
      initSKey: null,
      searchKey: null, // 筛选项
      selectProduct: {}, // 当前选择当产品
      modalVisible: false, // 课程时间选择
      selectProductType: productTypeId, // 已选产品类型

      productList: [], // 产品列表
      productListTotal: 0, // 产品总数
    };
  }

  static propTypes = {
    defaultValue: PropTypes.object, // 默认值，包括产品类型等
    title: PropTypes.string, // 弹窗标题
    initSKey: PropTypes.object, // 需初始化的筛选项
    updateLoading: PropTypes.func,
  };

  static defaultProps = {
    title: null,
    initSKey: null,
    visible: true,
    productTypeId: undefined,
  };

  static clickTab(index) {
    setTimeout(function () {
      let selectedPrdTab = document.querySelector('#add-product-tabs-wrap').querySelectorAll('.ant-tabs-tab')[index];
      if (selectedPrdTab) {
        selectedPrdTab.click();
      }
    });
  }

  componentDidMount() {
    this.initContent();
  }

  componentDidUpdate(prevProps) {
    if (prevProps.isEcs !== this.props.isEcs) {
      const productType = this.getSystemProduct().system_product[0].val;
      this.handleTabsChange(productType);
    }
  }

  initContent = async () => {
    await new Promise(this.initDict);
    await new Promise(this.initTeacher);
    await new Promise(this.initSearchKey);

    if (this.state.selectProductType === RENEWAL_CLASS) {
      this.loadRenewalClass();
    } else {
      this.loadProductList(this.state.initSKey);
    }
  };

  /**
   * 请求产品类型枚举
   */
  initDict = (resolve) => {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: [
          'productTypeTree',
          'grade',
          'quarter',
          'subject',
          'year',
          'phase',
          'weekDay',
          'branchAllCampus',
          'productCharacter',
          'other_category',
          'other_of_category',
          { name: 'productGroup', params: SMALL_CLASS },
          { name: 'productGroup', params: ONE_ON_MANY },
        ],
        callback: () => {
          resolve();
        },
      },
    });
  };
  /**
   * 请求老师枚举
   */
  initTeacher = (resolve) => {
    this.props.dispatch({
      type: `${storeName}/getTeachers`,
      payload: {
        callback: () => {
          resolve();
        },
      },
    });
  };

  /**
   * 初始化搜索条件
   */
  initSearchKey = (resolve) => {
    const { selectProductType } = this.state;
    const { yearDict, serverDate, grade = {}, blCampusId, isEcs } = this.props;

    let quarterId = [];
    let productGroupId = '';
    let courseSeriesId = null;
    // 类型为其它时不需要年级默认值
    let gradeId = NoAcquiescenceType.includes(selectProductType) ? null : this.props.gradeId;
    let productVersionId = getBLoYearId(yearDict, new Date(serverDate), {
      mount: 9,
      goUp: selectProductType !== ONE_ON_ONE_COURSE,
    });

    // 小班与双师要进行升班处理
    if (SMALL_CLASS === selectProductType) {
      courseSeriesId = [];
      gradeId = grade.gradeId || gradeId;
      quarterId = getQuarterId(serverDate);
    } else if (ONE_ON_MANY === selectProductType) {
      // 一对一
      courseSeriesId = [];
      productGroupId = [];
    }

    // 小班时年级为幼儿园和毕业都改完全部
    if (
      selectProductType === SMALL_CLASS &&
      gradeId === grade.gradeId &&
      ['幼儿园', '毕业'].includes(grade.gradeName)
    ) {
      gradeId = null;
    }

    // 精英班时年级为幼儿园和毕业都改完全部
    if (selectProductType === ECS_CLASS && ['幼儿园', '毕业'].includes(grade.gradeName)) {
      gradeId = null;
    }

    // 其他类型 所属产品类型默认值为其他
    const otherOfCategoryId = !isEcs && OTHERS === selectProductType ? 'DAT0000000388' : null;

    const characterId = this.props.isEdit
      ? getDictValByText(this.props.productCharacter, characterTextCfg[this.props.contractCharacter])
      : null;

    this.setState(
      {
        initSKey: {
          gradeId,
          quarterId,
          characterId,
          productGroupId,
          courseSeriesId,
          productVersionId,
          campusId: blCampusId,
          otherOfCategoryId: this.props.isEdit
            ? otherOfCategoryValCfg[this.props.contractProductTag]
            : otherOfCategoryId,
        },
      },
      () => {
        resolve();
      },
    );
  };

  // 根据查询条件查询产品
  loadProductList = async (params = {}) => {
    const { type } = this.getSystemProduct();
    const { isEcs, yearDict } = this.props;
    const { page, rows, searchKey, selectProductType } = this.state;

    const _params = {
      page,
      rows,
      isEcs,
      ...searchKey,
      ...params,
      productType: type,
      studentId: this.props.studentId, // 学生id用于查询该学生的续报班
    };
    delete _params.searchKey;

    // 小班、一对多产品系列支持多选，其他还是单选
    if (SPECIAL_PRODUCT_TYPE[selectProductType]) {
      _params.courseSeriesIds = params.courseSeriesId;
      delete _params.courseSeriesId;
    }

    if (selectProductType === ONE_ON_MANY) {
      _params.productGroupIds = params.productGroupId;
      delete _params.productGroupId;
    }

    if (type && yearDict.length > 0) {
      this.switchLoading(true);

      const response = await selectProductForContract(_params).catch((e) => {
        this.setState({
          productList: [],
          productListTotal: 0,
        });
        this.switchLoading();
      });
      let { datas, rowCount } = response || {};

      if (Array.isArray(datas) && datas.length > 0) {
        datas.forEach((item) => {
          item._groupType = contractGroupMap[item.productType];
        });
      }

      this.setState({
        productList: datas || [],
        productListTotal: rowCount || 0,
      });

      this.switchLoading(false);
    }
  };

  async loadRenewalClass(params = {}, autoChangeTab = false) {
    this.switchLoading(true);
    let apiParams = {
      ...params,
      studentId: this.props.studentId,
    };
    const res = await queryProductInfoNew(apiParams).catch(errorsHandler);
    let productList = [];
    let productListTotal = 0;
    if (res && Array.isArray(res.data)) {
      productList = res.data.reduce((arr, cur) => arr.concat(cur), []);
      productList.forEach((item) => {
        item._groupType = contractGroupMap[item.productType];
        item.renewTag = 'RENEW';
      });
      productListTotal = productList.length;
    }
    this.setState(
      {
        productList,
        productListTotal,
      },
      () => {
        if (productListTotal === 0 && autoChangeTab) {
          AddProduct.clickTab(1);
        }
      },
    );
    this.switchLoading(false);
  }

  // 查询搜索
  handleSearch = (val) => {
    if (this.state.selectProductType === RENEWAL_CLASS) {
      return this.loadRenewalClass();
    }
    this.setState({ ...this.initValue, searchKey: val });
    this.loadProductList({ ...this.initValue, ...val });
  };

  handleToTabsChange = (val) => {
    this.handleTabsChange(val, false);
  };

  // 选项卡切换回调
  handleTabsChange = (val, isInit = true) => {
    // 初始化或者不是小班或者不是一对多
    if (isInit === true || !SPECIAL_PRODUCT_TYPE[val]) {
      this.setState({ selectProductType: val, ...this.initValue }, async () => {
        await new Promise(this.initSearchKey);
        this.searchRef.getWrappedInstance().resetSKey();
      });
    } else {
      const { selectedList, isEdit } = this.props;
      this.setState(
        {
          selectProductType: val,
          ...this.initValue,
        },
        async () => {
          await new Promise(this.initSearchKey);
          const searchRefInstance = this.searchRef.getWrappedInstance();
          if (isEdit) {
            const { isQualityContract, isBroadcastCooperate } = this.props;
            searchRefInstance.resetCourseSeriesId(false, isQualityContract, isBroadcastCooperate, true);
          } else {
            const specialSelectedList = selectedList.filter(({ productType }) => productType === val); // 小班、一对多合同
            if (specialSelectedList.length) {
              const isQualityContract = !!specialSelectedList.filter(
                ({ courseSeriesName }) => courseSeriesName === '素质',
              ).length; // 素质类
              // 广电合同 小班产品：产品系列为素质+班级类型为合作项目 1对多产品：产品系列为素质+产品组为合作项目
              let isBroadcastCooperate = false;
              if (val === SMALL_CLASS) {
                isBroadcastCooperate = !!specialSelectedList.filter(
                  ({ classTypeId, courseSeriesName }) =>
                    courseSeriesName === '素质' && classTypeId === CLASS_TYPE_BROADCAST_COOPERATE,
                ).length; // 素质类合作项目
              } else if (val === ONE_ON_MANY) {
                isBroadcastCooperate = !!specialSelectedList.filter(
                  ({ productGroupId, courseSeriesName }) =>
                    courseSeriesName === '素质' && productGroupId === PRODUCT_GROUP_BROADCAST_COOPERATE,
                ).length; // 素质类合作项目
              }
              searchRefInstance.resetCourseSeriesId(false, isQualityContract, isBroadcastCooperate, true);
            } else {
              searchRefInstance.resetSKey();
            }
          }
        },
      );
    }
  };

  // 产品列表分页
  handlePaginationChange = (page, rows) => {
    this.setState({ page, rows });
    if (this.state.selectProductType === RENEWAL_CLASS) {
      return this.loadRenewalClass();
    }
    this.loadProductList({ page, rows, ...(this.state.searchKey || this.state.initSKey) });
  };

  initSearchValueWhenEditMode = (isQuality, isBroadcastCooperate) => {
    this.searchRef && this.searchRef.getWrappedInstance().resetCourseSeriesId(false, isQuality, isBroadcastCooperate);
  };

  // 是否广电合同
  checkItemIsBroadcastCooperate = (productType, courseSeriesName, classTypeId, productGroupId) => {
    return (
      SPECIAL_PRODUCT_TYPE[productType] &&
      courseSeriesName === '素质' &&
      ((productType === SMALL_CLASS && classTypeId === CLASS_TYPE_BROADCAST_COOPERATE) ||
        (productType === ONE_ON_MANY && productGroupId === PRODUCT_GROUP_BROADCAST_COOPERATE))
    );
  };

  // 判断小班、一对多素质类
  checkSelectValidate = (type, selectedList, courseSeriesName, classTypeId, productGroupId) => {
    // 如果已购课清单为空，则直接返回
    if (!selectedList.length) {
      return true;
    }

    let warningMessage = '';
    const specialSelectedList = selectedList.filter(({ productType }) => SPECIAL_PRODUCT_TYPE[productType]);
    const firstSelectedItem = specialSelectedList[0] || {}; // 默认值为空对象
    const selectedClassTypeId = firstSelectedItem.classTypeId;
    const selectedProductType = firstSelectedItem.productType; // 产品类型
    const selectedProductGroupId = firstSelectedItem.productGroupId; // 产品组
    const selectedCourseSeriesName = firstSelectedItem.courseSeriesName;
    const currentItemIsBroadcastCooperate = this.checkItemIsBroadcastCooperate(
      type,
      courseSeriesName,
      classTypeId,
      productGroupId,
    ); // 选购产品是广电合同
    const selectedItemIsBroadcastCooperate = this.checkItemIsBroadcastCooperate(
      selectedProductType,
      selectedCourseSeriesName,
      selectedClassTypeId,
      selectedProductGroupId,
    ); // 已购产品是广电合同

    if (selectedItemIsBroadcastCooperate) {
      // 已购清单是广电合同，则选购产品类型必须和已购产品类型相同或者产品类型为其他产品
      if (type !== OTHERS && (type !== selectedProductType || !currentItemIsBroadcastCooperate)) {
        warningMessage = '广电合同不可以混签';
      }
    } else if (currentItemIsBroadcastCooperate) {
      // 选购产品是广电合同，则已购产品类型为其他产品
      if (selectedList.filter(({ productType }) => productType === OTHERS).length !== selectedList.length) {
        warningMessage = '广电合同不可以混签';
      }
    } else {
      // 其他情况则判断素质合同的逻辑了
      if (!SPECIAL_PRODUCT_TYPE[type]) {
        return true;
      }
      if (selectedCourseSeriesName !== '素质' && courseSeriesName === '素质') {
        warningMessage = WARNING_MESSAGE[0][type];
      } else if (selectedCourseSeriesName === '素质' && courseSeriesName !== '素质') {
        warningMessage = WARNING_MESSAGE[1][type];
      }
    }

    if (warningMessage) {
      Modal.warning({
        title: '温馨提示',
        okText: '我知道了',
        content: warningMessage,
        wrapClassName: 'addProductWarningModal',
      });
      return false;
    }

    return true;
  };

  /**
   * @description 小班、双师要先选择课程
   * @param index 列表索引
   * @memberof AddProduct
   */
  checkSelect = (index) => {
    const { productList } = this.state;
    const { selectedList = [], grade } = this.props;
    const { type } = this.getSystemProduct();
    const product = productList[index];
    const { gradeId, classTypeId, courseSeriesName, productGroupId } = product;

    if (!this.checkSelectValidate(type, selectedList, courseSeriesName, classTypeId, productGroupId)) {
      return;
    }

    // 限制小班/双师不能重复; 讲座如果选班也限制不能重复，选产品则不限制
    const shouldLimit = SMALL_CLASS === type || (LECTURE === type && product.classId);
    if (shouldLimit) {
      for (let i = 0, len = selectedList.length; i < len; i += 1) {
        if (checkSamePrd(product) === checkSamePrd(selectedList[i])) {
          showMessage('购物车已存在此产品');
          return;
        }
      }
    }

    if (gradeId && this.props.gradeId !== gradeId) {
      // 小班与双师要进行升班判断,升班的情况下高一个年级不会提醒，拿后台返回的判断即可
      if (!(SMALL_CLASS === type && grade.gradeId === gradeId)) {
        return showConfirm('学生在读年级与选择的产品年级不一致，请确认是否跨年级上课', () => {
          if (productTypeAllow.includes(type) && product.classId) {
            this.setState({ modalVisible: true, selectProduct: product });
            return;
          }
          this.handleSelect(product);
        });
      }
    }

    if (productTypeAllow.includes(type) && product.classId) {
      this.setState({ modalVisible: true, selectProduct: product });
      return;
    }

    this.handleSelect(product);
  };

  // 产品选择回调
  handleSelect = (product) => {
    const { quantity, totalCourseHours, courseSeriesName, classTypeId, productGroupId } = product;
    const { selectProductType } = this.state;

    let _quantity = quantity || totalCourseHours;
    // 数量的默认逻辑
    if (quantityResetType.includes(selectProductType)) {
      _quantity = !!_quantity ? _quantity : 1;
    }
    product.quantity = _quantity;
    this.props.dispatch({
      type: `${storeName}/updateSelectProduct`,
      payload: { type: 'ADD', data: product, selectProductType },
    });

    const isQuality = courseSeriesName === '素质';
    const isBroadcastCooperate = this.checkItemIsBroadcastCooperate(
      selectProductType,
      courseSeriesName,
      classTypeId,
      productGroupId,
    );
    this.searchRef.getWrappedInstance().resetCourseSeriesId(false, isQuality, isBroadcastCooperate);
  };

  // 获取产品列表和类型
  getSystemProduct = () => {
    let sysPrd = {
      type: null,
      system_product: [],
    };
    if (this.props.isEdit) {
      sysPrd.type = this.props.contractProductTag;
      sysPrd.system_product = systemProductTab[this.props.contractProductTag] || [];
    } else if (this.props.isEcs) {
      sysPrd.type = ECS_CLASS;
      sysPrd.system_product = systemProductTab[ECS_CLASS];
    } else {
      sysPrd.type = systemProduct.default;
      sysPrd.system_product = systemProductTab.default;
    }

    if (this.state.selectProductType) {
      sysPrd.type = this.state.selectProductType;
    }

    return sysPrd;
  };

  /**
   * @description 切换选择首次上课时间弹窗显示
   * @memberof AddProduct
   */
  switchModalSelectCourseTime = () => {
    this.setState(() => ({ modalVisible: false }));
  };

  /**
   * @description 带课程上课时间添加产品
   * @memberof AddProduct
   */
  addPrdWithCourseTime = async (item) => {
    const { selectProduct } = this.state;
    const { courseBeginTime, courseDate, _origin = {} } = item;
    const { type } = this.getSystemProduct();
    const { totalCourseHours = 0, classId } = selectProduct;
    const firstTime = `${courseDate} ${courseBeginTime}`;
    let _quantity = totalCourseHours;

    if (type === SMALL_CLASS) {
      _quantity = await findMiniClassWillBuyHour({ startDate: firstTime, miniClassId: classId });
    } else {
      // 这里的班级id要取双师的主班id
      _quantity = await findClassWillBuyHour({ startDate: firstTime, classId: _origin.twoTeacherClassId });
    }

    const val = {
      ...selectProduct,
      courseHours: _origin.courseNum,
      startClassTime: courseBeginTime,
      _courseDate: courseDate,
      quantity: _quantity,
    };

    this.handleSelect(val);
    this.switchModalSelectCourseTime();
  };

  /**
   * @description 带开始上课时间添加产品
   * @memberof AddProduct
   */
  addPrdWithStartTime = () => {
    const { selectProduct } = this.state;
    const val = {
      _date: selectProduct.startClassTime,
      _time: null,
      _realClassHours: selectProduct.totalCourseHours,
      ...selectProduct,
    };
    this.handleSelect(val);
    this.switchModalSelectCourseTime();
  };

  switchLoading(loading) {
    if (this.props.stepType === ADD_PRODUCT) {
      runFunc(this.props.updateLoading, loading);
    }
  }

  getValidCourseDate = (list, today) => {
    let result = [];
    for (let i = 0; i < list.length; i++) {
      if (!list[i].disabled) {
        result = [i];
        break;
      }
    }
    return result;
  };

  render() {
    const { isEcs, institution } = this.props;
    const { rows, page, selectProduct, productList, productListTotal } = this.state;
    const { type, system_product } = this.getSystemProduct();
    const { classId } = selectProduct;
    const isRenewal = type === RENEWAL_CLASS;

    return (
      <div className={style.product_modal}>
        <Loading loading={this.props.calcLoading} tip="优惠金额计算中..." />

        <div id="add-product-tabs-wrap">
          <Tabs activeKey={type} onChange={this.handleToTabsChange}>
            {system_product.map(({ val, text }) => (
              <TabPane tab={`${text}`} key={val} />
            ))}
          </Tabs>
          <div style={{ display: isRenewal ? 'none' : 'block' }}>
            <Search
              isAlready
              isEcs={isEcs}
              selectProductType={type}
              isEdit={this.props.isEdit}
              institution={institution}
              onSearch={this.handleSearch}
              initSKey={this.state.initSKey}
              ref={(ref) => (this.searchRef = ref)}
              selectedList={this.props.selectedList}
              isBroadcastCooperate={this.props.isBroadcastCooperate}
              isQualityContract={this.props.isQualityContract}
            />
            <SearchResult number={productListTotal} />
          </div>

          {productList.length > 0 ? (
            <Deck
              dataSource={productList}
              render={classCardRender({
                onClick: this.checkSelect,
                isRenew: isRenewal,
              })}
              defaultValue={{ showQuickJumper: false }}
              pagination={
                isRenewal
                  ? undefined
                  : {
                      total: productListTotal,
                      current: page,
                      size: rows,
                      showSizeChanger: true,
                      pageSizeOptions: ['5', '10', '20', '50'],
                      onChange: this.handlePaginationChange,
                    }
              }
              className={classNames({ [style.renewalDeck]: isRenewal })}
            />
          ) : (
            <NoDataPlus textClassName={style.noDataText} />
          )}
        </div>

        {this.state.modalVisible && classId && (
          <ModalSelectCourseTime
            title="选择首次上课时间"
            classId={classId}
            productType={type}
            onCancel={this.switchModalSelectCourseTime}
            onConfirm={this.addPrdWithCourseTime}
            noDataDirectCallback={this.addPrdWithStartTime}
            defaultSelect={[0]}
          />
        )}
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  const serverDate = state.account.user.serverDate;

  return {
    contractCharacter: state[storeName].contractCharacter,
    contractProductTag: state[storeName].contractProductTag,
    isAlreadyInit: state[storeName].isAlreadyInit,
    isEcs: state[storeName].isEcs,
    isEdit: state[storeName].isEdit,
    stepType: state[storeName].stepType,
    limitProduct: state[storeName].limitProduct,
    blCampusId: state[storeName].blCampusId,
    studentId: state[storeName].studentId,
    selectedList: state[storeName].selectedList,
    grade: state[storeName].grade || {},
    calcLoading: state[storeName].calcLoading,
    gradeId: state[storeName].gradeId,
    productTypeTree: state.dict.productTypeTree || [],
    productCharacter: state.dict.productCharacter || [],
    yearDict: (state.dict.year || []).filter((item) => Number(item.text) >= new Date(serverDate).getFullYear() - 1),
    serverDate,
    institution: state.account.user.institution,
  };
};

export default connect(mapStateToProps, null, null, { withRef: true })(AddProduct);
