import {
  SMALL_CLASS,
  TWO_TEACHER,
  updateQuarterCfg,
  updateGradeCfg,
  QUARTER_SPRING,
  QUARTER_SUMMER,
  QUARTER_AUTUMN,
  QUARTER_WINTER,
} from './config';
import { loginUser } from 'UTIL/common';
import { getDictTextByVal, getDictValByText, getBLoYearId } from 'UTIL/dict';
import moment from 'moment';

function checkStr(val) {
  return val && typeof val === 'string';
}

/**
 * 将数据带过来的筛选项初始值根据规则转换为最终的初始值
 *
 * @param SKey {Object}
 * @param yearDict {Array}
 * @param gradeDict {Array}
 *
 * @return {Object}
 * */
function sKeyAdapter(SKey, yearDict, gradeDict) {
  let payload = {
    productType: loginUser.isTwoTeacherSym() ? TWO_TEACHER : SMALL_CLASS,
    ...(SKey || {}),
  };

  if (checkStr(SKey.quarterId)) {
    payload.quarterId = updateQuarterCfg[SKey.quarterId];
  } else {
    delete payload.quarterId;
  }

  if (checkStr(SKey.yearId)) {
    let tagYearId = SKey.yearId;

    if (SKey.quarterId && SKey.quarterId === QUARTER_AUTUMN) {
      let yearText = getDictTextByVal(yearDict, SKey.yearId);
      if (yearText) {
        yearText = (Number(yearText) + 1).toString();
      }
      let newYearId = getDictValByText(yearDict, yearText);
      if (newYearId) {
        tagYearId = newYearId;
      }
    }

    payload.yearId = tagYearId;
  } else {
    delete payload.yearId;
  }

  if (checkStr(SKey.gradeId)) {
    let tagGradeId = SKey.gradeId;

    if (SKey.quarterId && SKey.quarterId === QUARTER_SPRING) {
      let gradeText = getDictTextByVal(gradeDict, SKey.gradeId);
      if (gradeText) {
        gradeText = updateGradeCfg[gradeText];

        if (gradeText) {
          let newGradeId = getDictValByText(gradeDict, gradeText);
          if (newGradeId) {
            tagGradeId = newGradeId;
          }
        }
      }
    }

    payload.gradeId = [tagGradeId];
  } else {
    delete payload.gradeId;
  }

  if (checkStr(SKey.subjectId)) {
    payload.subjectId = [SKey.subjectId];
  } else {
    delete payload.subjectId;
  }

  return payload;
}

/**
 * 通过开始上课时间及总课时长，获取上课时间范围
 *
 * @param startTime {string} '14:00'
 * @param timeThrough {number} 90
 *
 * @return {string} '14:00~15:30'
 * */
function getTimeRange(startTime, timeThrough) {
  return startTime ? `${startTime}~${moment(startTime, 'HH:mm').add(timeThrough, 'm').format('HH:mm')}` : '-';
}

/**
 * 双师、小班在11月1日至12月31日年份 + 1
 * @param {*} yearDict
 * @param {*} date
 * @param initValue
 */
function getNextYear(yearDict, date, initValue) {
  const _date = new Date(date);
  const year = _date.getFullYear();

  if (_date >= new Date(`${year}-11-01`) && _date <= new Date(`${year}-12-31`)) {
    return initValue !== undefined ? initValue : getBLoYearId(yearDict, new Date(`${year + 1}-1-01`));
  }

  return undefined;
}

/**
 * 双师小班获取默认值
 * 配置写死，优化：后端添加接口返回各日期
 * @param {*} date
 */
function getQuarterId(date) {
  const _date = new Date(date);
  const year = _date.getFullYear();

  if (_date >= new Date(`${year}-2-01`) && _date <= new Date(`${year}-04-30`)) {
    return [QUARTER_SPRING];
  }

  if (_date >= new Date(`${year}-5-01`) && _date <= new Date(`${year}-08-15`)) {
    return [QUARTER_SUMMER, QUARTER_AUTUMN];
  }

  if (_date >= new Date(`${year}-8-16`) && _date <= new Date(`${year}-10-30`)) {
    return [QUARTER_AUTUMN];
  }

  return [QUARTER_AUTUMN, QUARTER_WINTER];
}

/**
 * 双师系统-季度默认值计算
 *
 * @param serverDate 服务器时间
 */
function getQuarterIdByTwoTeacher(serverDate) {
  if (!serverDate) {
    return null;
  }

  const date = new Date(serverDate);
  const month = date.getMonth() + 1;
  const spring = [1, 2, 3];
  const summer = [4, 5, 6];
  const autumn = [7, 8, 9, 10];
  const winter = [11, 12];
  let quarterId = null;
  if (spring.includes(month)) {
    quarterId = QUARTER_SPRING;
  } else if (summer.includes(month)) {
    quarterId = QUARTER_SUMMER;
  } else if (autumn.includes(month)) {
    quarterId = QUARTER_AUTUMN;
  } else if (winter.includes(month)) {
    quarterId = QUARTER_WINTER;
  }

  return quarterId ? [quarterId] : null;
}

export { sKeyAdapter, getTimeRange, getNextYear, getQuarterId, getQuarterIdByTwoTeacher };
