import React from 'react';
import classNames from 'classnames';
import style from './styles.module.less';
import PropTypes from 'prop-types';
import TipText from '../components/TipText';
import { numFormatToMoney } from '../utils';
import { ADD_PRODUCT, CONFIRM_PRODUCT, stepTypeList } from '../store/storeCfg';
import { Button, TextLink } from 'COMPONENT';

const lastStepType = stepTypeList[stepTypeList.length - 1];

const stepTypeCfg = {
  [ADD_PRODUCT]: {
    prev: null,
    next: {
      text: '下一步',
      value: CONFIRM_PRODUCT,
    },
  },
  [CONFIRM_PRODUCT]: {
    prev: {
      text: '上一步',
      value: ADD_PRODUCT,
    },
    next: null,
  },
  null: {},
  undefined: {},
};

const Info = (props) => {
  return (
    <div className={style.info}>
      {props.label && <div className={style.infoLabel}>{props.label}：</div>}
      <div className={style.infoContent}>{props.children}</div>
    </div>
  );
};

const FooterBar = (props) => {
  return (
    <div className={style.FooterBar}>
      {props.amountVisible && (
        <div className={style.FooterBarInfoGroups}>
          <Info label="合计">￥{numFormatToMoney(props.totalAmount)}</Info>
          <Info label="已收合计">￥{numFormatToMoney(props.paidAmount)}</Info>
          <Info label="优惠合计">
            ￥{numFormatToMoney(props.promotionAmount)}
            {/* 修改合同时，详情接口无法返回所有必要数据 */}
            <TextLink
              className={classNames(style.textLink, props.promotionAmount === 0 || props.isEdit ? style.hidden : '')}
              onClick={props.onPromotionDetail}
            >
              明细
            </TextLink>
          </Info>
          <Info label="应收合计">
            <TipText fontWeight="bold">￥{numFormatToMoney(props.realAmount)}</TipText>
          </Info>
        </div>
      )}

      <div className={style.FooterBarBtnGroups}>
        {!props.isOnlyEditDiscount && stepTypeCfg[props.stepType].prev && (
          <Button onClick={props.onStepChange} data-step-type={stepTypeCfg[props.stepType].prev.value} type="default">
            {stepTypeCfg[props.stepType].prev.text}
          </Button>
        )}
        {!props.isOnlyEditDiscount && stepTypeCfg[props.stepType].next && (
          <Button onClick={props.onStepChange} data-step-type={stepTypeCfg[props.stepType].next.value} type="primary">
            {stepTypeCfg[props.stepType].next.text}
          </Button>
        )}
        {props.stepType === lastStepType && (
          <Button onClick={props.onSave} type="primary">
            保存到合同列表
          </Button>
        )}
        {props.stepType === lastStepType && (
          <Button onClick={props.onSaveAndPay} type="primary">
            生成合同并支付
          </Button>
        )}
      </div>
    </div>
  );
};

FooterBar.propTypes = {
  stepType: PropTypes.oneOf(stepTypeList),
  totalAmount: PropTypes.number.isRequired,
  promotionAmount: PropTypes.number.isRequired,
  realAmount: PropTypes.number.isRequired,
  paidAmount: PropTypes.number,
  onPromotionDetail: PropTypes.func,
  onSave: PropTypes.func.isRequired,
  onSaveAndPay: PropTypes.func.isRequired,
  onStepChange: PropTypes.func.isRequired,
  amountVisible: PropTypes.bool,
  isEdit: PropTypes.bool,
};

FooterBar.defaultProps = {
  amountVisible: true,
  isEdit: false,
};

export default FooterBar;
