import React from 'react';
import { Tooltip } from 'antd';
import styles from '../index.less';
import { formatPromotionRuleResult } from '../util';
import { formatRuleFinalTextDeposit } from './utils';
import { COUPON_RULE_VO_KEY, COUPON_PROMOTION_TYPE_KEY } from '../../config';

const ruleVoKey = COUPON_RULE_VO_KEY;
const promotionTypeKey = COUPON_PROMOTION_TYPE_KEY;

/**
 * 单个规则显示
 *
 * @param props.data 优惠券
 * @param props.productList
 * @param props.hitRule 命中的规则（如果有）
 * @return {*}
 * @constructor
 */
const SingleDetail = (props) => {
  const { data } = props;

  const rulesVos = data[ruleVoKey] || [];
  const hitRule = props.hitRule || rulesVos[0];
  const result = formatPromotionRuleResult({ promotionType: data[promotionTypeKey], rule: hitRule });
  const desc = formatRuleFinalTextDeposit(data[promotionTypeKey], hitRule, data);

  return (
    <div className={styles.top_left_single}>
      <Tooltip placement="top" title={`¥ ${result.resultNum}`}>
        <h5 className={styles.top_left_money}>¥ {result.resultNum}</h5>
      </Tooltip>
      <Tooltip placement="top" title={desc}>
        <p className={styles.top_left_desc}>{desc}</p>
      </Tooltip>
    </div>
  );
};

export default SingleDetail;
