import React from 'react';
import moment from 'moment';
import styles from './index.less';
import { getDictTextByVal } from 'UTIL/dict';
import {
  promotionNatureCfg,
  conditionUnitCfg,
  promotionsPrefixCfg,
  promotionsTypeCfg,
  promotionsUnitCfg,
  needRuleUnitList,
  continuityDict,
} from './config';
import { dateFormatter } from '../config';

/**
 * 选中背景样式
 *
 * @param selected
 * @param disabled
 * @return {string}
 */
export function formatSelectedStyle(selected, disabled) {
  let selectedStyle = '';

  if (disabled) {
    selectedStyle = styles.disabled;
  } else if (selected) {
    selectedStyle = styles.selected;
  }

  return selectedStyle;
}

/**
 * 格式化优惠券类型文案
 *
 * @param promotionNature {string} 优惠券类型
 * @return {*|string}
 */
export function formatPromotionNatureName(promotionNature) {
  return promotionNatureCfg[promotionNature] || '';
}

/**
 * 格式化连报条件
 */
export function formatContinuityTypeName(continuityType) {
  if (!continuityType) {
    return '';
  }

  return getDictTextByVal(continuityDict, continuityType);
}

/**
 * 根据化优惠券数据格式化单个优惠规则描述
 *
 * @param params.promotionType 优惠券类型
 * @param params.continuityType 连报条件
 * @param params.rule 需要格式化的优惠规则
 */
export function formatPromotionRuleResult(params) {
  const { promotionType, rule } = params;

  if (!promotionType || !rule) {
    const defPreFixText = '';
    const defPrefixNum = '';
    const defPrefixUnit = '';
    const defResultText = '';
    const defResultNum = '';
    const defResultUnit = '';

    return {
      prefixText: defPreFixText,
      prefixNum: defPrefixNum,
      prefixUnit: defPrefixUnit,
      resultText: defResultText,
      resultNum: defResultNum,
      resultUnit: defResultUnit,
    };
  }

  const prefixText = promotionsPrefixCfg[promotionType];
  const prefixUnit = conditionUnitCfg[promotionType];
  const resultText = promotionsTypeCfg[promotionType];
  const resultUnit = promotionsUnitCfg[promotionType];

  let _prefixUnit;
  if (needRuleUnitList.includes(promotionType)) {
    _prefixUnit = prefixUnit[rule.ruleUnit];
  } else {
    _prefixUnit = prefixUnit;
  }
  if (typeof _prefixUnit !== 'string') {
    _prefixUnit = '';
  }

  return {
    prefixText: prefixText, // 满
    prefixNum: rule.promotionRule, // N (件)
    prefixUnit: _prefixUnit, // 件产品 (单位)
    resultText: resultText, // 减
    resultNum: rule.promotion, // N (元)
    resultUnit: resultUnit, // 元 (单位)
  };
}

/**
 * 将上面的优惠规则拼接为完整字符串
 *
 * @param ruleResult 格式化后的单个规则
 * @return {string} `满 N 件产品 减 N 元`
 */
export function formatRuleEntireText(ruleResult) {
  const { prefixText, prefixNum, prefixUnit, resultText, resultNum, resultUnit } = ruleResult;

  return prefixText + prefixNum + prefixUnit + resultText + resultNum + resultUnit;
}

/**
 * 提供一个格式化通用最终优惠规则显示条件文字（不包含抵扣券）
 *
 * @param promotionType {string} 优惠类型
 * @param rule {Object} 优惠规则
 * @return {string}
 */
export function formatRuleFinalText(promotionType, rule) {
  const result = formatPromotionRuleResult({ promotionType, rule });

  return formatRuleEntireText(result);
}

/**
 * 活动时间
 *
 * @param record
 * @param splitStr
 */
export function renderActivityDate(record, splitStr = '-') {
  const { startDate, endDate } = record;

  return (
    <span>
      {moment(startDate).format(dateFormatter)} {splitStr} {moment(endDate).format(dateFormatter)}
    </span>
  );
}
