import React from 'react';
import { Row, Col } from 'antd';
import CouponCard from '../CouponCard';
import styles from './index.less';

/**
 * 优惠券列表(根据是否可用分类)
 *
 * @param props.list {Array} 排好序的优惠券列表
 * @param props.productList {Array} 已选产品列表
 * @param props.onSelect {function} 选中优惠券
 */
const CouponList = (props) => {
  const { list, productList, onSelect } = props;
  const usableList = list.filter((item) => item.suitable);
  const unUsableList = list.filter((item) => !item.suitable);

  return (
    <div>
      <p className={styles.classify_title}>可用券</p>
      <Row gutter={20}>
        {usableList.map((coupon, index) => {
          return (
            <Col key={index} span={12}>
              <CouponCard data={coupon} productList={productList} onClick={onSelect} />
            </Col>
          );
        })}
      </Row>
      <p className={styles.classify_title}>不可用券</p>
      <Row gutter={20}>
        {unUsableList.map((coupon, index) => {
          return (
            <Col key={index} span={12}>
              <CouponCard data={coupon} productList={productList} onClick={onSelect} />
            </Col>
          );
        })}
      </Row>
    </div>
  );
};

export default CouponList;
