import NP from 'number-precision';
import { COUPON_ID_KEY } from '../config';
import { judgeIsPersonalizeCourse } from './judges';

const idKey = COUPON_ID_KEY;

/**
 * 优惠列表后端最终请求参数
 *
 * @param params 表单参数
 * @return {*}
 */
export function formatCouponListApiParam(params = {}) {
  const param = { ...params };

  param.current = params.pageNo;
  param.size = params.pageSize;

  delete param.pageNo;
  delete param.pageSize;

  return param;
}

/**
 * 计算选中产品中，可使用优惠的产品(小班、双师)的实际支付金额总金额（除去产品优惠、活动优惠）
 *
 * @param productList {array} 选中产品，需过滤不是小班双师的产品
 * @return {number}
 */
export function formatBDRealAmount(productList = []) {
  if (!productList) {
    return 0;
  }

  return productList.reduce((total, currentItem) => {
    const discountInfo = currentItem._discountInfo || {};
    const discountAmount = discountInfo._couponDiscount || 0;

    return NP.plus(total, currentItem.realAmount, discountAmount);
  }, 0);
}

/**
 * 提取批量查询的优惠券列表中的可用ID
 *
 * @param response {*} {data: {suitablePromotion: [可用优惠券], unSuitablePromotion: [不可用优惠券]}}
 * @return {array<string>}
 */
export function formatMultipleMatchCouponIds({ response = {} }) {
  if (!response) {
    return [];
  }

  const data = response.data || {};
  const suitableList = data.suitablePromotion || [];

  return suitableList.map((item) => item[idKey]);
}

/**
 * 格式化classInfo接口请求参数
 *
 * @param productList
 * @param useCase {String<'ALL'|'RE'|'NEW'>}
 * @return {{classId: *, useCase: *, productId: *, num: *, productType: *}[]}
 */
export function formatClassInfoApiParam(productList = [], useCase) {
  return (productList || []).map((item) => {
    const ruleNum = formatRuleNumAndRuleUnit(item);

    return {
      classId: item.classId,
      productId: item.productId,
      productType: item.productType,
      useCase,
      ...ruleNum,
    };
  });
}

/**
 * 格式化优惠金额与单位
 *
 * @param product {Object} 产品信息
 * @return {{ruleUnit: string, num: number}}
 */
export function formatRuleNumAndRuleUnit(product) {
  const isPersonalizeCourse = judgeIsPersonalizeCourse(product.productType);
  const ruleUnit = isPersonalizeCourse ? 'NATURE_MINUTES' : null;
  const num = isPersonalizeCourse ? product.quantityMinutes || 0 : product.quantity;

  return {
    num,
    ruleUnit,
  };
}
