import React, { PropTypes } from 'react';
import { Tooltip } from 'antd';
import style from './styles.module.less';
import Selected from '../Selected';
import { runFunc } from 'UTIL/common';
import { getTimeRange } from '../../AddProduct/utils';
import {
  productTypeAllow,
  ECS_CLASS,
  SMALL_CLASS,
  ONE_ON_MANY,
  ONE_ON_ONE_COURSE,
  OTHERS,
} from '../../AddProduct/config';

// 显示产品组的产品类型
const SHOW_PRODUCT_GROUP_TYPE = {
  [SMALL_CLASS]: 1, // 班课
  [ONE_ON_MANY]: 1, // 1对多
};

const MoreInfo = (props) => {
  return (
    <div className={style.MoreInfo}>
      <div className={style.MoreInfoLabel}>{props.label}</div>
      <div className={style.MoreInfoText}>
        <p>{props.text || '-'}</p>
      </div>
    </div>
  );
};
MoreInfo.propTypes = {
  label: PropTypes.string.isRequired,
  text: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
};

const hasValue = (val) => val !== undefined && val !== null;

const formatInfo = (val, symbol = '-') => (hasValue(val) ? val : symbol);

const Content = (props) => {
  const isShow = productTypeAllow.includes(props.productType);

  return (
    <div className={style.Content} style={{ maxHeight: props.isShowMore ? 500 : 122 }}>
      <div>
        {props.productTypeName && (
          <Tooltip title="产品类型">
            <span className={style.titleType}>{props.productTypeName}</span>
          </Tooltip>
        )}
        {props.productType === OTHERS && props.otherCategoryName && (
          <Tooltip title="费用类型">
            <span className={style.titleType}>{props.otherCategoryName}</span>
          </Tooltip>
        )}
        {props.productType === ONE_ON_ONE_COURSE && props.productGroupName && (
          <Tooltip title="产品组">
            <span className={style.titleType}>{props.productGroupName}</span>
          </Tooltip>
        )}
        {isShow && props.courseSeriesName && (
          <Tooltip title="产品系列">
            <span className={style.titleType}>{props.courseSeriesName}</span>
          </Tooltip>
        )}
        {isShow && props.classTypeName && (
          <Tooltip title="班级类型">
            <span className={style.titleTypeLeft}>{props.classTypeName}</span>
          </Tooltip>
        )}
        {isShow && props.classTypeName && props.classLevelName && <span className={style.titleTypeMid}>、</span>}
        {isShow && props.classLevelName && (
          <Tooltip title="班级层次">
            <span className={style.titleTypeRight}>{props.classLevelName}</span>
          </Tooltip>
        )}
        {props.productType === ECS_CLASS && props.ecsClassTypeName && (
          <Tooltip title="产品类别">
            <span className={style.titleType}>{props.ecsClassTypeName}</span>
          </Tooltip>
        )}
        {SHOW_PRODUCT_GROUP_TYPE[props.productType] && props.productGroupName && (
          <Tooltip title="产品组">
            <span className={style.titleType}>{props.productGroupName}</span>
          </Tooltip>
        )}
      </div>

      <div className={style.title}>
        <span>{props.productName}</span>
      </div>

      <MoreInfo label="班级名称" text={props.className} />
      <MoreInfo label="班级类型" text={props.classTypeName} />
      <MoreInfo label="班级学习中心" text={props.campusName} />
      <MoreInfo
        label="单价"
        text={`¥${/(LECTURE)|(OTHERS)/.test(props.productType) ? props.price : props.naturePrice || '-'}`}
      />
      <MoreInfo
        label="已上小时"
        text={`${props.consumeCourseNatureHours || '-'}/${props.totalCourseNatureHours || '-'}`}
      />
      <MoreInfo label="老师" text={props.assistTeacherName || props.mainTeacherName || '暂无'} />
      <MoreInfo label="上课日期" text={props._courseDate || props.startClassDate} />
      <MoreInfo
        label="上课时间"
        text={!props.startClassTime ? '暂无' : getTimeRange(props.startClassTime, props.classLength)}
      />
      <MoreInfo label="截止日期" text={props.validEndDate} />
    </div>
  );
};

const SelectedCard = (props) => {
  const { deleteBtnVisible, onDelete, index } = props;
  return (
    <Selected
      className={style.SelectedCard}
      deleteBtnVisible={deleteBtnVisible}
      onDelete={() => {
        runFunc(onDelete, index);
      }}
    >
      <Content {...props} />
    </Selected>
  );
};

export default SelectedCard;
