import React from 'react';
import { connect } from 'dva';
import styles from './styles.module.less';
import { storeName } from '../store/storeCfg';
import { Popconfirm, Icon } from 'antd';
import { Deck, NoDataPlus, TextLink } from 'COMPONENT';
import selectedCardRender from './SelectedCardRender';

class Order extends React.Component {
  constructor(props) {
    super(props);
  }

  handleDelete = (index) => {
    const { selectedList } = this.props;
    const prd = selectedList[index];
    let list = selectedList;
    // 是否包含子产品
    prd.subProducts ? (list = selectedList.filter((item) => prd._relation !== item._relation)) : list.splice(index, 1);
    this.props.dispatch({
      type: `${storeName}/updateSelectProduct`,
      payload: {
        type: 'DEL',
        list,
        record: prd,
        index,
      },
    });
  };

  clearAll = () => {
    this.props.dispatch({
      type: `${storeName}/cartDelProduct`,
      payload: { list: [] },
    });
  };

  render() {
    const { props } = this;
    const len = props.selectedList.length;
    const hadData = len > 0;

    return (
      <div className={styles.order}>
        <div className={styles.header}>
          <div className={styles.title}>购课单{hadData && ` (${len})`}</div>
          <div className={styles.tool}>
            <Popconfirm title="确认清空吗？" onConfirm={this.clearAll} placement="bottomRight">
              <TextLink fontSize={12} disabled={len === 0}>
                <Icon type="delete" />
                <span className={styles.textBtn}>清空</span>
              </TextLink>
            </Popconfirm>
          </div>
        </div>

        <div className={styles.body}>
          {hadData ? (
            <Deck
              dataSource={props.selectedList}
              render={selectedCardRender({
                onDelete: this.handleDelete,
              })}
            />
          ) : (
            <NoDataPlus
              className={styles.noData}
              textClassName={styles.noDataText}
              text="还没有选购课程哟，快去添加吧~"
            />
          )}
        </div>
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    selectedList: state[storeName].selectedList,
  };
};

export default connect(mapStateToProps)(Order);
