import React from 'react';
import PropTypes from 'prop-types';
import styles from './styles.module.less';
import { Popover } from 'antd';
import { InputNumber } from 'COMPONENT';
import { ONE_ON_MANY, ONE_ON_ONE_COURSE, OTHERS } from '../../config';
import { runFunc } from 'UTIL/common';

import { NoCourseHoursProductType } from '../../AddProduct/config';
import NP from 'number-precision';

class BloInputMinutes extends React.Component {
  constructor(props) {
    super(props);

    this.getMaxTips = this.getMaxTips.bind(this);
    this.showMaxTips = this.showMaxTips.bind(this);
    this.onChange = this.onChange.bind(this);
    this.onChangeHours = this.onChangeHours.bind(this);
    this.onChangeMinutes = this.onChangeMinutes.bind(this);

    this.isOneToOneOrOneToMoreBlo =
      [ONE_ON_ONE_COURSE, ONE_ON_MANY].includes(props.groupType) && props.isYinLiu && props.productType !== OTHERS;

    this.isCourseHoursType = !NoCourseHoursProductType.includes(props.productType);
    this.max = this.props.max || Infinity;

    this.courseNatureMinutes = this.props.courseNatureMinutes || 0;

    if (this.isOneToOneOrOneToMoreBlo) {
      this.max = NP.times(props.classTimeLength, 18);
      if (props.isEdit && props.courseNatureMinutes > this.max) {
        this.max = Infinity;
      }
    }

    this.state = {
      tipsVisible: false,
      hours: Math.floor(NP.divide(this.courseNatureMinutes, 60)),
      minutes: Math.floor(this.courseNatureMinutes % 60),
    };
  }

  static propTypes = {
    productType: PropTypes.string,
    classTimeLength: PropTypes.number,
    groupType: PropTypes.string,
    isYinLiu: PropTypes.bool,
    isEdit: PropTypes.bool,
    courseNatureMinutes: PropTypes.number,
  };

  getMaxTips() {
    if (this.isOneToOneOrOneToMoreBlo) {
      const hours = Math.floor(NP.divide(this.max, 60)),
        minutes = Math.floor(this.max % 60);
      return `引流1对1、引流1对多不可超过${hours}小时${minutes}分钟`;
    }

    return '';
  }

  showMaxTips() {
    this.setState(
      {
        tipsVisible: true,
      },
      () => {
        const _self = this;
        setTimeout(function () {
          _self.setState({
            tipsVisible: false,
          });
        }, 3000);
      },
    );
  }

  onChange(courseNatureMinutes) {
    /*
      if (this.isOneToOneOrOneToMoreBlo && courseNatureMinutes > this.max) {
        courseNatureMinutes = this.max;
        this.setState({
          hours: Math.floor( courseNatureMinutes/60 ),
          minutes: Math.floor(courseNatureMinutes % 60)
        }, () => {
          // 价格的计算使用？？？？？
          // 应收：课时单价 * 课时数（保留两位小数后的值）- 优惠    （入库要用这个最终课时数（保留2位小数）与课时单价的值，不要使用中间没有四舍五入的课时数 * 课时单价）
          let courseHours = NP.divide(courseNatureMinutes, this.props.classTimeLength).toFixed(2);
          console.debug('单课时长: ', this.props.classTimeLength, '课时数: ', courseHours, ' 时长：', this.state.hours, this.state.minutes, ' 分钟数：', courseNatureMinutes);
          runFunc(this.props.onChange, courseHours, courseNatureMinutes);
        });
        this.showMaxTips();
        return;
      }
    */
    // 价格的计算使用？？？？？
    // 应收：课时单价 * 课时数（保留两位小数后的值）- 优惠    （入库要用这个最终课时数（保留2位小数）与课时单价的值，不要使用中间没有四舍五入的课时数 * 课时单价）
    let courseHours = NP.divide(courseNatureMinutes, this.props.classTimeLength).toFixed(2);
    console.debug(
      '单课时长: ',
      this.props.classTimeLength,
      '课时数: ',
      courseHours,
      ' 时长：',
      this.state.hours,
      this.state.minutes,
      ' 分钟数：',
      courseNatureMinutes,
    );
    runFunc(this.props.onChange, courseHours, courseNatureMinutes);
  }

  onChangeHours(hours) {
    hours = parseInt(hours || 0);
    this.setState(
      {
        hours: hours,
      },
      () => {
        this.courseNatureMinutes = Math.floor(parseInt(this.state.minutes || 0) + hours * 60);
        this.onChange(this.courseNatureMinutes);
      },
    );
  }

  onChangeMinutes(minutes) {
    minutes = parseInt(minutes || 0);
    this.setState(
      {
        minutes: minutes,
      },
      () => {
        this.courseNatureMinutes = Math.floor(minutes + parseInt(this.state.hours || 0) * 60);
        this.onChange(this.courseNatureMinutes);
      },
    );
  }

  render() {
    return (
      <Popover
        visible={this.state.tipsVisible}
        placement="top"
        overlayClassName={styles.warningOverlay}
        content={this.getMaxTips()}
      >
        <div className={styles.courseTimeInputWrap}>
          <InputNumber
            {...this.props}
            value={this.state.hours}
            onChange={this.onChangeHours}
            size="small"
            step={1}
            min={0}
            precision={0}
            placeholder={'0'}
            formatter={(value) => `${value}`}
            className={styles.courseTimeInput}
          />
          <span className={styles.courseTimeLabel}>小时</span>

          <InputNumber
            {...this.props}
            value={this.state.minutes}
            max={59}
            min={0}
            step={1}
            precision={0}
            onChange={this.onChangeMinutes}
            size="small"
            placeholder={'0'}
            formatter={(value) => `${value}`}
            className={styles.courseTimeInput}
          />
          <span className={styles.courseTimeLabel}>分钟</span>
        </div>
      </Popover>
    );
  }
}

export default BloInputMinutes;
