import React from 'react';
import { connect } from 'dva';
import PropTypes from 'prop-types';
import classNames from 'classnames';
import { storeName } from '../../store/storeCfg';
import styles from './styles.module.less';
import { getCurrentQuarterByOrganizationId } from 'SERVICE/contract';
import { getQuarterTypeDict } from 'UTIL/dict';
import { errorsHandle } from 'UTIL/common';
import { Loading, Select } from 'COMPONENT';

const errorsHandler = errorsHandle();

class SignQuarter extends React.Component {
  static propTypes = {
    groupItem: PropTypes.object.isRequired,
    blCampusId: PropTypes.string,
    className: PropTypes.string,
    style: PropTypes.object,
    disabled: PropTypes.bool,
  };

  static defaultProps = {
    disabled: false,
  };

  constructor(props) {
    super(props);

    this.init = this.init.bind(this);
    this.loadQuarterTypeDict = this.loadQuarterTypeDict.bind(this);
    this.loadQuarterType = this.loadQuarterType.bind(this);
    this.updateValue = this.updateValue.bind(this);

    this.state = {
      loading: false,
      dict: [],
    };

    this.groupType = props.groupItem.groupType;
  }

  componentDidMount() {
    this.init();
  }

  componentDidUpdate(prevProps, prevState) {
    if (prevProps.blCampusId !== this.props.blCampusId) {
      this.loadQuarterType();
    }
  }

  init() {
    this.loadQuarterTypeDict();
    this.loadQuarterType();
  }

  async loadQuarterTypeDict() {
    const dict = await getQuarterTypeDict();
    this.setState({
      dict,
    });
  }

  async loadQuarterType() {
    if (this.props.isEdit) {
      return;
    }

    this.setState({ loading: true });

    let apiParams = {
      organizationId: this.props.blCampusId,
    };
    const res = await getCurrentQuarterByOrganizationId(apiParams).catch(errorsHandler);
    if (res && res.resultCode === 0) {
      this.updateValue(res.data.quarterType);
    }

    this.setState({ loading: false });
  }

  updateValue(val) {
    this.props.dispatch({
      type: `${storeName}/updateProductGroup`,
      payload: {
        productGroupItem: {
          ...this.props.groupItem,
          quarterType: val,
        },
      },
    });
  }

  render() {
    const { props, state } = this;

    return (
      <div className={classNames(styles.signQuarter, props.className)} style={props.style}>
        <div>签单季度：</div>
        <div className={styles.content}>
          <Select
            allowClear={false}
            dict={state.dict}
            disabled={props.disabled}
            onChange={this.updateValue}
            value={props.groupItem.quarterType}
          />
        </div>

        <Loading loading={state.loading} />
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    blCampusId: state[storeName].blCampusId,
    isEdit: state[storeName].isEdit,
  };
};

export default connect(mapStateToProps)(SignQuarter);
