import React from 'react';
import style from './styles.module.less';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import { Tooltip, Popconfirm } from 'antd';
import { MultiSelect, Popover, Card } from 'COMPONENT';
import { Button } from 'antd';
import BloInputNumber from '../BloInputNumber';
import TipText from '../../components/TipText';
import Text from '../../components/Text';
import SeasonSubjectMark from '../../components/SeasonSubjectMark';
import { NoCourseHoursProductType, productTypeAllow } from '../../AddProduct/config';
import {
  UNVALID,
  SMALL_CLASS,
  ECS_CLASS,
  ONE_ON_MANY,
  ONE_ON_ONE_COURSE,
  OTHERS,
  OTHER_CATEGORY_TYPE_MARKETING,
} from '../../config';
import invalidStamp from 'ASSET/imgs/invalid_stamp.svg';
import { numFormatToMoney } from '../../utils';
import BloInputMinutes from '../BloInputMinutes';
import BloCourseTimeLabel from '../BloCourseTimeLabel';

// 显示产品组的产品类型
const SHOW_PRODUCT_GROUP_TYPE = {
  [SMALL_CLASS]: 1, // 班课
  [ONE_ON_MANY]: 1, // 1对多
};

const MoreInfo = (props) => {
  return (
    <div className={style.MoreInfo}>
      <span className={style.MoreInfoLabel}>{props.label}:</span>
      <span className={style.MoreInfoText}>{props.text || '-'}</span>
    </div>
  );
};

const isDisableProductPromotion = (record, props) => {
  return props.isOnlyEditDiscount && (record.isProductNarrow || record._statusCache !== 'NORMAL'); // 缩单或退回电子账号
};

const PromotionCol = (props) => {
  const { data: record, productPromotionDict = [], isEdit } = props;

  let dict = [];
  const disabled = !record.promotions || record.promotions.length === 0;
  if (!disabled) {
    dict =
      record.promotions[0].id === '-1'
        ? productPromotionDict || []
        : record.promotions.map((item) => ({ text: item.name, val: item.id }));
  }

  const disableProductPromotion = isDisableProductPromotion(record, props); // 缩单或退回电子账号

  const multiSelect = (
    <MultiSelect
      showSearch
      dict={dict}
      selectAll={false}
      noOptionTips="没有可用优惠"
      placeholder="请选择可用优惠"
      value={record._prdPromotionVal}
      onChange={props.updatePrdPromotion}
      style={{ width: '100%', textAlign: 'left', marginTop: 20 }}
      disabled={record._editPrdPromotionDisabled || !record.quantity || disableProductPromotion}
    />
  );

  return record._warningBeforeEditPrdPromotion && !disableProductPromotion ? (
    <Popover
      placement="top"
      key={record.contractProductId}
      overlayClassName={style.warningOverlay}
      content={
        isEdit
          ? '修改原合同产品优惠将重置优惠款'
          : '本合同已经享受了【优惠活动】，继续选择【产品优惠】后会清空本合同名下已享受的优惠活动'
      }
    >
      <div>{multiSelect}</div>
    </Popover>
  ) : (
    multiSelect
  );
};

const DeleteBtn = (props) => {
  if (props.needConfirm) {
    return (
      <Popconfirm title="删除产品将重新推荐优惠券，是否继续？" placement="left" onConfirm={props.onDelete}>
        <Button className={style.operationBtn} size="small">
          删除
        </Button>
      </Popconfirm>
    );
  } else {
    return (
      <Button onClick={props.onDelete} className={style.operationBtn} size="small">
        删除
      </Button>
    );
  }
};

const TableProductCard = (props) => {
  const { data } = props;
  const isShow = productTypeAllow.includes(data.productType);
  const isCourseHoursType = !NoCourseHoursProductType.includes(data.productType);
  const isShowCancel = props.isEdit && data.contractProductId && !data._btnInvalidDisabled;
  const isInvalid = data._contractPrdStatus === UNVALID;
  // 小班使用优惠券删除需要提示
  const needDeleteConfirm = !props.isEdit && data.productType === SMALL_CLASS && props.group.couponList.length > 0;
  const bloInputNumberProps =
    data.productType === OTHERS && data.otherCategoryId === OTHER_CATEGORY_TYPE_MARKETING
      ? { precision: 0, step: 1, min: 1, max: 999 }
      : {};

  let customStartClassDate = data.classStartDate || '-';
  if (props.timeText > data.classStartDate && !isInvalid) {
    customStartClassDate = <TipText fontSize={12}>{customStartClassDate}</TipText>;
  }

  return (
    <Card className={classnames(style.tableProductCard, { [style.invalid]: isInvalid })}>
      <div className={style.header}>
        <div className={style.headerTitle}>
          {data.renewTag === 'RENEW' && <SeasonSubjectMark {...data} invalid={isInvalid} />}
          <span className={style.title}>{data.productName}</span>
        </div>

        <div className={style.headerTools}>
          {!props.isOnlyEditDiscount && isShowCancel && (
            <Button onClick={props.switchContractPrdInvalid} className={style.operationBtn} size="small">
              {isInvalid ? '撤销' : '取消'}
            </Button>
          )}
          {data._btnClearPromotionVisible && !isDisableProductPromotion(data, props) && (
            <Button onClick={props.clearPromotion} className={style.operationBtn} size="small">
              清空优惠
            </Button>
          )}
          {!props.isOnlyEditDiscount && !data._btnDeleteDisabled && (
            <DeleteBtn needConfirm={needDeleteConfirm} onDelete={props.delete} />
          )}
        </div>
      </div>

      <div className={style.body}>
        <div className={style.main}>
          <div className={style.typeWarp}>
            {data.renewTag === 'RENEW' && <span className={style.renewal}>续报班级</span>}
            {data.productTypeName && (
              <Tooltip title="产品类型">
                <span className={style.titleType}>{data.productTypeName}</span>
              </Tooltip>
            )}
            {data.productType === OTHERS && data.otherCategoryName && (
              <Tooltip title="费用类型">
                <span className={style.titleType}>{data.otherCategoryName}</span>
              </Tooltip>
            )}
            {data.productType === ONE_ON_ONE_COURSE && data.productGroupName && (
              <Tooltip title="产品组">
                <span className={style.titleType}>{data.productGroupName}</span>
              </Tooltip>
            )}
            {isShow && data.courseSeriesName && (
              <Tooltip title="产品系列">
                <span className={style.titleType}>{data.courseSeriesName}</span>
              </Tooltip>
            )}
            {isShow && data.classTypeName && (
              <Tooltip title="班级类型">
                <span className={style.titleTypeLeft}>{data.classTypeName}</span>
              </Tooltip>
            )}
            {isShow && data.classTypeName && data.classLevelName && <span className={style.titleTypeMid}>、</span>}
            {isShow && data.classLevelName && (
              <Tooltip title="班级层次">
                <span className={style.titleTypeRight}>{data.classLevelName}</span>
              </Tooltip>
            )}
            {data.productType === ECS_CLASS && data.ecsClassTypeName && (
              <Tooltip title="产品类别">
                <span className={style.titleType}>{data.ecsClassTypeName}</span>
              </Tooltip>
            )}
            {SHOW_PRODUCT_GROUP_TYPE[data.productType] && data.productGroupName && (
              <Tooltip title="产品组">
                <span className={style.titleType}>{data.productGroupName}</span>
              </Tooltip>
            )}
          </div>

          <div className={style.infoWrap}>
            {data.className && <MoreInfo label="班级名称" text={data.className} />}
            {data.subjectName && <MoreInfo label="科目" text={data.subjectName} />}
            {data.assistTeacherName && <MoreInfo label="老师" text={data.assistTeacherName} />}
            {data.maxPeopleQuantity > 0 && (
              <MoreInfo label="已报名" text={`${data.peopleQuantity || '-'}/${data.maxPeopleQuantity}`} />
            )}
            {!!data.totalCourseNatureHours && (
              <MoreInfo
                label="已上小时"
                text={`${data.consumeCourseNatureHours || '-'}/${data.totalCourseNatureHours}`}
              />
            )}
            {data.classStartDate && <MoreInfo label="开课日期" text={customStartClassDate} />}
            {data.startClassTime && (
              <MoreInfo
                label="上课开始时间"
                text={`${data._courseDate || data.startClassDate}  ${data.startClassTime}`}
              />
            )}
          </div>

          <div className={style.promotionWrap}>
            {(data._activePromotion || []).map((item) => (
              <span className={style.promotion} key={item.id}>
                {item.name}
              </span>
            ))}
          </div>
        </div>

        <div className={style.middle}>
          {props.data && <PromotionCol {...props} />}
          {(data._prdPromotion || []).length > 0 && (
            <div className={classnames(style.promotionWrap, style.productPromotion)}>
              <Popover
                content={
                  <div className={style.productPromotionPopover}>
                    {data._prdPromotion.map((item) => (
                      <p key={item.id}>{item.name}</p>
                    ))}
                  </div>
                }
                overlayClassName={style.productPromotionOverlay}
                placement="top"
              >
                <span>
                  {data._prdPromotion.map((item) => (
                    <span className={style.promotion} key={item.id}>
                      {item.name}
                    </span>
                  ))}
                </span>
              </Popover>
            </div>
          )}
          {isDisableProductPromotion(data, props) && (
            <p style={{ marginTop: 7, color: '#FF6166', fontSize: 12, lineHeight: '17px' }}>
              合同产品缩单或退回电子账户或已结束，均不可修改优惠，均不可修改优惠
            </p>
          )}
        </div>

        <div className={style.tail}>
          <Text label="单价" className={style.tailInfo}>
            <span className={style.tailContent}>¥{isCourseHoursType ? data.naturePrice : data.price}</span>
          </Text>
          <Text label={isCourseHoursType ? '时长' : '数量'} className={style.tailInfo}>
            {data._editQuantityDisabled || (data._quantityDisabled && Boolean(data._hadInitQuantity)) ? (
              <span className={style.tailContent}>
                {isCourseHoursType ? (
                  // todo 这里需要重构一下。小班课时长不可编辑，但是后台字段不足，需要确定一下各种类型的课的字段
                  <BloCourseTimeLabel data={data} />
                ) : (
                  data.quantity
                )}
              </span>
            ) : (
              <div className={classnames(style.bLoInputWrap, { [style.quantityValidate]: !data.quantity })}>
                {isCourseHoursType ? (
                  //需要转化分钟数换算的课
                  <BloInputMinutes
                    size={'middle'}
                    courseNatureMinutes={data.courseNatureMinutes}
                    onChange={props.updatePrdQuantity}
                    classTimeLength={data.classTimeLength}
                    productType={data.productType}
                    groupType={props.groupType}
                    isYinLiu={props.isYinLiu}
                    isEdit={props.isEdit}
                  />
                ) : (
                  // 按数量计算价格的课
                  <BloInputNumber
                    size={'middle'}
                    value={data.quantity}
                    onChange={props.updatePrdQuantity}
                    productType={data.productType}
                    groupType={props.groupType}
                    isYinLiu={props.isYinLiu}
                    isEdit={props.isEdit}
                    {...bloInputNumberProps}
                  />
                )}
              </div>
            )}
          </Text>
          <Text label="优惠" className={style.tailInfo}>
            <span className={style.tailContent}>¥{numFormatToMoney(data.promotionAmount || 0)}</span>
          </Text>
          {/*{data.paidAmount > 0 && (
            <Text label="已收" className={style.tailInfo}>
              <span className={style.tailContent}>¥{numFormatToMoney(data.paidAmount || 0)}</span>
            </Text>
          )}*/}
          <Text label="应收" className={style.tailInfo}>
            <span className={style.tailContent}>¥{numFormatToMoney(data.realAmount || 0)}</span>
          </Text>
          {props.isEdit && (
            <Text label="已扣优惠" className={style.tailInfo}>
              <span className={style.tailContent}>¥{numFormatToMoney(data.promotionConsumeAmount || 0)}</span>
            </Text>
          )}
        </div>
      </div>

      {isInvalid && <img className={style.invalidStamp} src={invalidStamp} alt="无效" />}
    </Card>
  );
};

TableProductCard.propTypes = {
  data: PropTypes.object.isRequired,
  delete: PropTypes.func,
  clearPromotion: PropTypes.func,
  updatePrdQuantity: PropTypes.func,
};

export default TableProductCard;
