import React from 'react';
import { connect } from 'dva';
import classNames from 'classnames';
import style from './styles.module.less';
import {
  calcAuto,
  calcManual,
  yinliu,
  useCouponGroupTypes,
  NEW_CONTRACT,
  RE_CONTRACT,
  useCouponProductTypes,
  CLASS_COURSE,
} from '../config';
import { storeName, CHOOSE_COUPON } from '../store/storeCfg';
import { isClass } from '../store/storeUtils';
import { logStoreName } from '../alilogStore/config';
import { Loading, NoDataPlus } from 'COMPONENT';
import TableCard from './TableCard';
import { runFunc } from 'UTIL/common';
import { debounce, clone } from 'lodash';
import ModalSelectCoupon from '../ModalSelectCoupon';
import { Modal, TextLink } from 'COMPONENT';
import { Icon, Button, Input as AntInput } from 'antd';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import TipText from '../components/TipText';
import { numFormatToMoney } from '../utils';
import ModalCouponDetail from '../ModalCouponDetail';
import SignQuarter from './SignQuarter';

const { TextArea } = AntInput;

class ProductTable extends React.Component {
  constructor(props) {
    super(props);
    this.deleteSelectedPrd = this.deleteSelectedPrd.bind(this);
    this.deletePrdErrHandle = this.deletePrdErrHandle.bind(this);
    this.deletePrdErrHandle = this.deletePrdErrHandle.bind(this);
    this.calcCoupon = this.calcCoupon.bind(this);
    this.updatePrdPromotion = this.updatePrdPromotion.bind(this);
    this.calcPrdPromotion = this.calcPrdPromotion.bind(this);
    this.clearPromotion = this.clearPromotion.bind(this);
    this.switchContractPrdInvalid = this.switchContractPrdInvalid.bind(this);
    this.switchCouponModalVisible = this.switchCouponModalVisible.bind(this);
    this.selectCoupon = this.selectCoupon.bind(this);
    this.logSwitchPromotionType = this.logSwitchPromotionType.bind(this);
    this.remarkChange = this.remarkChange.bind(this);
    this.handleUpdatePrdPromotion = this.handleUpdatePrdPromotion.bind(this);
    this.switchGroupAutoCalc = this.switchGroupAutoCalc.bind(this);
    this.switchCouponDetailModal = this.switchCouponDetailModal.bind(this);

    this.debounceCalc = debounce(this.calcPrdPromotion, 600);
    this.state = {
      couponModalVisible: false,
      currentGroupItem: [],
      confirmModalVisible: false,
      couponDetailVisible: false,
    };
  }

  switchCouponModalVisible(groupItem) {
    let currentGroupItem = clone(groupItem);
    this.setState((prevState) => {
      prevState.couponModalVisible = !prevState.couponModalVisible;
      prevState.currentGroupItem = [];
      if (prevState.couponModalVisible) {
        currentGroupItem.data = currentGroupItem.data.filter(
          (item) => useCouponProductTypes.includes(item.productType) && item.quantity > 0,
        );
        prevState.currentGroupItem = currentGroupItem;
      }
      return prevState;
    });
  }

  remarkChange(productGroupItem, remark) {
    this.props.dispatch({
      type: `${storeName}/updateProductGroup`,
      payload: {
        productGroupItem: {
          ...productGroupItem,
          remark,
        },
      },
    });
  }

  logSwitchPromotionType(type) {
    const hashmap = {
      [calcAuto]: { value: 'PROMOTION_AUTO', label: '自动计算优惠' },
      [calcManual]: { value: 'PROMOTION_MANUAL', label: '手动计算优惠' },
    };

    const { value, label } = hashmap[type] || {};

    if (value) {
      this.props.dispatch({
        type: `${logStoreName}/logTabChange`,
        payload: {
          value,
          label,
        },
      });
    }
  }

  selectCoupon(list, currentGroupItem) {
    this.props.dispatch({
      type: `${storeName}/selectCoupon`,
      payload: { list, groupItem: currentGroupItem },
    });

    this.switchCouponModalVisible(currentGroupItem);
  }

  deleteSelectedPrd(record, groupItem) {
    const _self = this;

    this.props.dispatch({
      type: `${storeName}/deleteSelectedPrd`,
      payload: {
        record,
        groupItem,
        errHandle: _self.deletePrdErrHandle,
      },
    });
  }

  deletePrdErrHandle(newGroupItem, hasCoupon) {
    showConfirm(
      hasCoupon ? '合同产品发生变化，抵扣券已被清除，请重新选择' : '合同产品发生变化，是否重新自动选择抵扣券？',
      () => {
        this.calcCoupon(newGroupItem);
        this.switchCouponModalVisible(newGroupItem);
      },
      () => this.calcCoupon(newGroupItem),
    );
  }

  calcCoupon(newGroupItem) {
    this.props.dispatch({
      type: `${storeName}/calcCouponPromotion`,
      payload: {
        initLoading: true,
        groupItem: newGroupItem,
      },
    });
  }

  /**
   * 修改产品数量/课时
   *
   * @param val {number} 课时或数量
   * @param record {Object}
   * @param groupItem {Array} 产品组
   * @param quantityMinutes {number} 课时分钟，保存分钟数
   */
  handleUpdatePrdQuantity = (val, record, groupItem, quantityMinutes) => {
    if (val === record.quantity) {
      return;
    }

    const _self = this;
    this.props.dispatch({
      type: `${storeName}/changePrdQuantity`,
      payload: {
        val,
        record,
        groupItem,
        quantityMinutes,
        callback(payload) {
          runFunc(_self.debounceCalc, payload);
        },
      },
    });
  };

  calcPrdPromotion(payload) {
    this.props.dispatch({
      type: `${storeName}/calcPrdPromotion`,
      payload,
    });
  }

  updatePrdPromotion(val, record, groupItem) {
    this.props.dispatch({
      type: `${storeName}/changePrdPromotion`,
      payload: { val, record, groupItem },
    });
  }

  clearPromotion(record) {
    this.props.dispatch({
      type: `${storeName}/clearPromotion`,
      payload: { record },
    });
  }

  // 清除合同下所有产品的活动优惠
  clearActivityCoupon(productGroupItem) {
    this.clearActivityCouponGroupItem = productGroupItem;
    this.setState({ confirmModalVisible: true });
  }

  switchContractPrdInvalid(record) {
    this.props.dispatch({
      type: `${storeName}/switchContractPrdInvalid`,
      payload: { record },
    });
  }

  /*
   * 如果后续需要兼容优惠活动与产品优惠混合使用，可改为直接调用 this.updatePrdPromotion
   * */
  async handleUpdatePrdPromotion(val, record, groupItem) {
    if (this.props.isEdit || groupItem.groupType !== CLASS_COURSE) {
      return this.updatePrdPromotion(val, record, groupItem);
    }

    const prdPromotionCheck = {};
    groupItem.data.forEach((item) => {
      prdPromotionCheck[`${item.productId}_${item.classId}`] = Array.isArray(item._prdPromotionVal);
    });
    prdPromotionCheck[`${record.productId}_${record.classId}`] = Array.isArray(val);

    const isAutoCalc = Object.values(prdPromotionCheck).every((item) => item === false);

    if (groupItem.isAutoCalc !== isAutoCalc) {
      groupItem = await new Promise(this.switchGroupAutoCalc(groupItem));
    }

    this.updatePrdPromotion(val, record, groupItem);
  }

  switchGroupAutoCalc(groupItem) {
    return (resolve) => {
      this.props.dispatch({
        type: `${storeName}/switchAutoCalc`,
        payload: {
          groupItem,
          callback(newGroupItem) {
            resolve(newGroupItem);
          },
        },
      });
    };
  }

  switchCouponDetailModal(groupItem) {
    this.setState((prevState) => {
      prevState.couponDetailVisible = !prevState.couponDetailVisible;
      if (groupItem) {
        prevState.couponDetail = groupItem.couponList || null;
      }
      return prevState;
    });
  }

  handleClearActivityCoupon = () => {
    this.handleCloseConfirmModal();
    this.props.dispatch({
      type: `${storeName}/clearActivityCoupon`,
      payload: { groupItem: this.clearActivityCouponGroupItem },
    });
  };

  handleCloseConfirmModal = () => {
    this.setState({ confirmModalVisible: false });
  };

  // 二次确认弹框
  generateConfirmModal() {
    const props = {
      widthSize: 's',
      title: '温馨提示',
      onOk: this.handleClearActivityCoupon,
      onCancel: this.handleCloseConfirmModal,
    };

    return (
      this.state.confirmModalVisible && (
        <Modal {...props}>
          <div className={style.confirmTextWrapper}>
            <Icon
              type="exclamation-circle"
              style={{ color: '#ffbf00', fontSize: 22, marginRight: 5, position: 'relative', top: 2 }}
            />
            请确认是否要清空本次选购所有产品命中的活动优惠
          </div>
        </Modal>
      )
    );
  }

  render() {
    const { isEdit, studentId, contractBranchId, realAmount } = this.props;
    const { currentGroupItem } = this.state;

    return (
      <div className={style.productTable}>
        {this.props.groupList.map((productGroupItem) => {
          return (
            <div className={style.productGroupItem} key={productGroupItem.id}>
              <div className={style.productHeader}>
                <div className={style.productHeaderTitle}>{productGroupItem.title}合同</div>

                {isClass(productGroupItem) && (
                  <div className={style.productHeaderQuarter}>
                    <SignQuarter groupItem={productGroupItem} disabled={isEdit} />
                  </div>
                )}

                <div className={style.productStatistics}>
                  <span className={style.totalItem}>总额合计：¥{numFormatToMoney(productGroupItem.totalAmount)}</span>
                  <span className={style.totalItem}>
                    已收合计：¥{numFormatToMoney(productGroupItem.paidAmount || 0)}
                  </span>
                  <span className={style.totalItem}>
                    优惠合计：¥{numFormatToMoney(productGroupItem.promotionAmount)}
                  </span>
                  <span className={style.totalItem} style={{ marginRight: 0 }}>
                    应收合计：<TipText fontWeight="bold">¥{numFormatToMoney(productGroupItem.realAmount)}</TipText>
                  </span>
                </div>
              </div>

              {(this.props.isEdit
                ? (productGroupItem.couponList || []).length > 0
                : this.props.chooseCouponAuth && useCouponGroupTypes.includes(productGroupItem.groupType)) && (
                /*&&
                  productGroupItem.productCharacterName !== yinliu 20220704需求若引流合同满足条件也允许选择优惠券*/
                <div className={style.productSubHeader}>
                  {(productGroupItem.couponList || []).length > 0 ? (
                    <span>
                      <span className={style.productSubHeaderText}>
                        已使用 {productGroupItem.couponList.length} 张优惠券
                      </span>
                      {this.props.isEdit ? (
                        <TextLink
                          onClick={() => {
                            this.switchCouponDetailModal(productGroupItem);
                          }}
                          className={style.productSubHeaderLink}
                        >
                          查看
                        </TextLink>
                      ) : (
                        <span className={style.productSubHeaderText}>
                          ，合计优惠 {numFormatToMoney(productGroupItem.couponAmount)} 元
                        </span>
                      )}
                    </span>
                  ) : (
                    productGroupItem.couponToBeUsed && <TipText>有可用优惠券</TipText>
                  )}
                  <Button
                    className={classNames(style.clearActivityCouponBtn, { [style.hidden]: this.props.isEdit })}
                    onClick={() => this.clearActivityCoupon(productGroupItem)}
                  >
                    清空全部活动优惠
                  </Button>
                  <Button
                    className={classNames(style.selectCouponBtn, { [style.hidden]: this.props.isEdit })}
                    type={'primary'}
                    onClick={() => this.switchCouponModalVisible(productGroupItem)}
                    disabled={isEdit || !studentId || !contractBranchId}
                  >
                    选择优惠券
                  </Button>
                </div>
              )}

              <div className={style.productBody}>
                {(productGroupItem.data || []).map((productItem) => (
                  <TableCard
                    key={productItem._uid}
                    group={productGroupItem}
                    data={productItem}
                    delete={() => {
                      this.deleteSelectedPrd(productItem, productGroupItem);
                    }}
                    clearPromotion={() => {
                      this.clearPromotion(productItem);
                    }}
                    switchContractPrdInvalid={() => {
                      this.switchContractPrdInvalid(productItem);
                    }}
                    updatePrdQuantity={(quantity, quantityMinutes) => {
                      this.handleUpdatePrdQuantity(quantity, productItem, productGroupItem, quantityMinutes);
                    }}
                    updatePrdPromotion={(val) => {
                      this.handleUpdatePrdPromotion(val, productItem, productGroupItem);
                    }}
                    groupType={productGroupItem.groupType}
                    isYinLiu={productGroupItem.productCharacterName === yinliu}
                    isEdit={isEdit}
                    {...this.props}
                  />
                ))}
              </div>

              <div className={style.productFooter}>
                <div className={style.remarkLabel}>备注</div>
                <TextArea
                  className={style.remark}
                  value={productGroupItem.remark}
                  autosize={{ minRows: 2, maxRows: 3 }}
                  disabled={this.props.isOnlyEditDiscount}
                  placeholder={this.props.isOnlyEditDiscount ? '' : '请输入'}
                  onChange={(e) => this.remarkChange(productGroupItem, e.target.value)}
                />
              </div>
            </div>
          );
        })}

        {this.props.groupList.length === 0 && (
          <NoDataPlus textClassName={style.noDataText} text="还没有选购课程哟，快去添加吧~" />
        )}

        <Loading loading={this.props.calcLoading} tip="优惠金额计算中..." />

        {this.state.couponModalVisible && (
          <ModalSelectCoupon
            limit={-1}
            multiple={true}
            branchId={contractBranchId}
            studentId={studentId}
            customerContact={this.props.confirmCustomerContact}
            contactList={this.props.customerContacts}
            couponList={currentGroupItem.couponList || []}
            productList={currentGroupItem.data}
            realAmount={realAmount}
            onCancel={this.switchCouponModalVisible}
            onOk={(list) => this.selectCoupon(list, currentGroupItem)}
            groupType={currentGroupItem.groupType}
            isYinLiu={currentGroupItem._isYinLiu}
            useCase={this.props.useCase}
          />
        )}

        {this.state.couponDetailVisible && (
          <ModalCouponDetail data={this.state.couponDetail} onCancel={this.switchCouponDetailModal} />
        )}

        {this.generateConfirmModal()}
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    isEdit: state[storeName].isEdit,
    funcAuth: state[storeName].funcAuth,
    chooseCouponAuth: state[storeName].funcAuth.indexOf(CHOOSE_COUPON) > -1,
    isEcs: state[storeName].isEcs,
    calcLoading: state[storeName].calcLoading,
    selectedList: state[storeName].selectedList,
    groupList: state[storeName].groupList,
    count: state[storeName].selectedList.length,
    contractBranchId: state[storeName].contractBranchId,
    realAmount: state[storeName].realAmount,
    studentId: state[storeName].studentId,
    timeText: state[storeName].timeText,
    productPromotionDict: state[storeName].productPromotionDict,
    customerContacts: state[storeName].customerContacts,
    confirmCustomerContact: state[storeName].confirmCustomerContact,
    useCase: {
      [NEW_CONTRACT]: 'NEW',
      [RE_CONTRACT]: 'RE',
    }[state[storeName].contractType],
  };
};

export default connect(mapStateToProps)(ProductTable);
