import React from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';
import styles from './styles.module.less';
import { Modal } from 'COMPONENT';
import TipText from '../components/TipText';
import { numFormatToMoney } from '../utils';
import { ONE_ON_ONE_COURSE, SMALL_CLASS, ONE_ON_MANY, LECTURE, OTHERS, ECS_CLASS } from '../config';
import NP from 'number-precision';

class PromotionDetail extends React.Component {
  static propTypes = {
    data: PropTypes.array,
    onCancel: PropTypes.func.isRequired,
  };

  static defaultProps = {
    data: [],
  };

  static reducePrdPromotion(data, accumulator) {
    data = data.filter((item) => !!item._prdPromotion);

    const sequence = [ECS_CLASS, ONE_ON_ONE_COURSE, SMALL_CLASS, ONE_ON_MANY, LECTURE, OTHERS];

    const sequenceMap = {
      [ECS_CLASS]: '【精英班】',
      [ONE_ON_ONE_COURSE]: '【1对1】',
      [SMALL_CLASS]: '【班课】',
      [ONE_ON_MANY]: '【1对多】',
      [LECTURE]: '【讲座】',
      [OTHERS]: '【其他】',
    };

    sequence.forEach((key) => {
      let total = 0;
      data.forEach((item) => {
        if (item.productType === key) {
          total = NP.plus(total, item._prdPromotionAmount);
        }
      });
      if (total > 0) {
        accumulator.push({
          label: `${sequenceMap[key]}产品优惠`,
          value: total,
        });
      }
    });

    return accumulator;
  }

  static reduceActivePromotion(data, accumulator) {
    data = data.filter((item) => Array.isArray(item._activePromotion) && item._activePromotion.length > 0);

    if (data.length > 0) {
      accumulator.push({
        label: '优惠活动优惠',
        value: data.reduce((accum, cur) => NP.plus(accum, cur._discountInfo._activeDiscount), 0),
      });
    }

    return accumulator;
  }

  static reduceCoupon(data, accumulator) {
    data = data.filter(
      (item) =>
        item._discountInfo &&
        Array.isArray(item._discountInfo.promotionSendCoupons) &&
        item._discountInfo.promotionSendCoupons.length > 0,
    );

    let temp = {};

    data.forEach((item) => {
      item._discountInfo.promotionSendCoupons.forEach((coupon) => {
        if (!temp[`${coupon._productTypeName}${coupon._couponTypeName}`]) {
          temp[`${coupon._productTypeName}${coupon._couponTypeName}`] = {
            productType: coupon._productTypeName,
            couponType: coupon._couponTypeName,
            total: 0,
          };
        }
        temp[`${coupon._productTypeName}${coupon._couponTypeName}`].total = NP.plus(
          temp[`${coupon._productTypeName}${coupon._couponTypeName}`].total,
          coupon._couponPromotionAmount,
        );
      });
    });

    Object.keys(temp).forEach((key) => {
      accumulator.push({
        label: `【${temp[key].productType}】${temp[key].couponType}优惠`,
        value: temp[key].total,
      });
    });

    return accumulator;
  }

  constructor(props) {
    super(props);

    let details = [];

    details = PromotionDetail.reducePrdPromotion(props.data, details);
    details = PromotionDetail.reduceActivePromotion(props.data, details);
    details = PromotionDetail.reduceCoupon(props.data, details);

    this.state = {
      details,
      total: details.reduce((total, current) => NP.plus(total, current.value), 0),
    };
  }

  render() {
    const { props, state } = this;

    return (
      <Modal
        className={styles.promotionDetail}
        title="优惠明细"
        widthSize="S"
        onCancel={props.onCancel}
        cancelText="关闭"
        okBtnShow={false}
      >
        {state.details.map((item, index) => (
          <div className={styles.detail} key={index}>
            <span className={styles.label}>{item.label}</span>
            <span className={styles.content}>￥{numFormatToMoney(item.value)}</span>
          </div>
        ))}

        {state.total > 0 && (
          <div className={classNames(styles.detail, styles.total)}>
            <span className={styles.label}>优惠合计</span>
            <span className={styles.content}>
              <TipText fontWeight="bold">￥{numFormatToMoney(state.total)}</TipText>
            </span>
          </div>
        )}
      </Modal>
    );
  }
}

export default PromotionDetail;
