import React from 'react';
import { connect } from 'dva';
import { storeName } from '../store/storeCfg';
import InfoBlock from '../components/InfoBlock';
import InfoForm from '../components/InfoForm';
import Required from '../components/Required';
import { Input, Select, InputSelect } from 'COMPONENT';
import { contractCustomerSearch, findCustomerById } from 'SERVICE/contract';
import { errorsHandle, loginUser } from 'UTIL/common';
import ModalSelectSchool from 'SRC/BLoComponents/ModalSelectSchool';
import { Icon } from 'antd';
import style from './style.modules.less';
import ModalChangeStuName from 'SRC/BLoComponents/ModalChangeStuName';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { getBranchByCampus } from '../utils';
import ModalInfo from '../components/ModalInfo';
import { certificateTypeMap, CT_IDENTITY_CARD } from '../config';

const InfoItem = InfoForm.InfoItem;

// 默认证件类型字典
const defaultCertificateTypeDict = [
  {
    text: certificateTypeMap[CT_IDENTITY_CARD],
    val: CT_IDENTITY_CARD,
  },
];

class StudentInfo extends React.Component {
  static defaultProps = {
    itemSpan: 6,
  };

  constructor(props) {
    super(props);

    this.updateProps = this.updateProps.bind(this);
    this.findCustomerByPhone = this.findCustomerByPhone.bind(this);
    this.handlePhoneInput = this.handlePhoneInput.bind(this);
    this.handleCustomerSelect = this.handleCustomerSelect.bind(this);
    this.findStudentByCustomerId = this.findStudentByCustomerId.bind(this);
    this.handleStudentNameInput = this.handleStudentNameInput.bind(this);
    this.handleStudentSelect = this.handleStudentSelect.bind(this);
    this.handleSchoolSelected = this.handleSchoolSelected.bind(this);
    this.switchChangedNameVisible = this.switchChangedNameVisible.bind(this);
    this.updateStudentName = this.updateStudentName.bind(this);
    this.handleMergeStudent = this.handleMergeStudent.bind(this);
    this.closeMergeStudentModal = this.closeMergeStudentModal.bind(this);
    this.switchSchoolSelectVisible = this.switchSchoolSelectVisible.bind(this);
    this.handleIdentityErr = this.handleIdentityErr.bind(this);

    this.state = {
      // 客户
      customerSelectVisible: false,
      customerTempList: null,
      customerNameDisable: false,
      // 选择学校
      schoolSelectVisible: false,
      //
      studentSelectVisible: false,
      studentTempList: null,
      // 更改学生姓名
      changedNameVisible: false,
      //
      mergeStudentVisible: false,
      mergeStudentInfo: {},
    };
    // 待选学生
    this.customerStudentList = [];
    this.isTwoTeacher = loginUser.isTwoTeacherSym();
    //
    this.mergeStudentInfo = null;
  }

  updateProps(payload) {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload,
    });
    if (payload.contractBranchId) {
      this.props.dispatch({
        type: `${storeName}/batchCalcCoursePromotion`,
      });
    }
  }

  async findCustomerByPhone(phoneNum) {
    const rsp = await contractCustomerSearch({ customerContact: phoneNum }, true).catch(errorsHandle());
    if (rsp) {
      let customerDict = rsp.data.map((item) => ({
        text: `${item.customerName}-${item.customerContact}`,
        val: item.customerId,
      }));
      this.setState({
        customerSelectVisible: true,
        customerTempList: customerDict,
      });
    }
  }

  handlePhoneInput(e) {
    let val = e.target.value;
    let numVal = Number(val);
    if (typeof numVal === 'number' && !isNaN(numVal)) {
      this.setState({
        customerSelectVisible: false,
        customerTempList: null,
        customerNameDisable: false,
      });
      this.customerStudentList = [];
      if (val.length > 9) {
        this.findCustomerByPhone(val);
      }
      this.updateProps({
        customerContact: val,
        customerId: null,
      });
    }
  }

  handleCustomerSelect(data) {
    let [customerName, customerContact] = data.text.split('-');
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        customerContact,
        customerName,
        customerId: data.val,
      },
    });
    this.setState({ customerNameDisable: true });
    this.findStudentByCustomerId(data.val);
  }

  async findStudentByCustomerId(id) {
    const rsp = await findCustomerById({ id }, true).catch(errorsHandle());
    this.customerStudentList = rsp.studentVos;
  }

  handleStudentNameInput(e) {
    let val = e.target.value;
    let studentDict = this.customerStudentList
      .filter((item) => item.name.indexOf(val) > -1)
      .map((item) => ({
        text: item.name,
        val: item.id,
      }));
    if (studentDict.length > 0) {
      this.setState({
        studentSelectVisible: true,
        studentTempList: studentDict,
      });
    }
    this.updateProps({
      studentId: null,
      studentName: val,
    });
  }

  handleStudentSelect(data) {
    let tag = this.customerStudentList.find((item) => item.id === data.val);
    if (tag) {
      this.props.dispatch({
        type: `${storeName}/updateState`,
        payload: {
          studentId: tag.id,
          studentName: tag.name,
          gradeId: tag.gradeId,
          schoolId: tag.schoolId,
          schoolName: tag.schoolName,
        },
      });
      this.props.dispatch({
        type: `${storeName}/initContractType`,
        payload: {
          studentId: tag.id,
          flag: false,
        },
      });
    }
  }

  handleSchoolSelected(data) {
    const { id, name, remarks, provinceId, cityId } = data;
    const isOther = id === '-1' || !id;
    this.updateProps({
      schoolId: isOther ? null : id,
      schoolName: isOther ? remarks : name,
      virtualSchool: isOther
        ? {
            provinceId: provinceId,
            cityId: cityId,
            regionId: '',
            schoolName: remarks,
          }
        : null,
    });

    this.setState({ schoolSelectVisible: false });
  }

  switchChangedNameVisible() {
    this.setState((prevState) => ({ changedNameVisible: !prevState.changedNameVisible }));
  }

  updateStudentName(val) {
    this.updateProps({ studentName: val });
  }

  handleMergeStudent(mergeStudentInfo) {
    this.setState({
      mergeStudentVisible: true,
      mergeStudentInfo,
      changedNameVisible: false,
    });
  }

  closeMergeStudentModal() {
    const { mergeStudentInfo } = this.state;
    this.setState(
      {
        mergeStudentVisible: false,
      },
      () => {
        if (mergeStudentInfo.mergeSuccess && window.__refreshParentEntrance) {
          window.__refreshParentEntrance({ stuId: mergeStudentInfo.newStudentId }, !mergeStudentInfo.lost);
        }
        this.setState({ mergeStudentInfo: {} });
      },
    );
  }

  switchSchoolSelectVisible() {
    if (!this.props.blCampusId) {
      return showMessage('请先选择合同学习中心');
    }
    this.setState((prevState) => ({ schoolSelectVisible: !prevState.schoolSelectVisible }));
  }

  handleIdentityErr(err) {
    this.props.dispatch({
      type: `${storeName}/updateErrors`,
      payload: {
        identity: !!err,
      },
    });
  }

  render() {
    return (
      <InfoBlock
        title={
          <span>
            <span className={style.titleName}>学员信息</span>
            {this.props.confirmCustomerContact && (
              <span className={style.titleTips}>
                <span>当前合同信息确认家长为（</span>
                <span className={style.titleTipsInfo}>{this.props.confirmCustomerContact}</span>
                <span>），如需更换请前往关联家长设置</span>
              </span>
            )}
          </span>
        }
        toolbar={this.props.toolbar}
      >
        <InfoForm className={style.studentInfoForm}>
          <InfoItem label={<Required>客户手机</Required>} span={this.props.itemSpan}>
            <InputSelect
              placeholder="家长手机号"
              value={this.props.customerContact}
              onChange={this.handlePhoneInput}
              showSelect={this.state.customerSelectVisible}
              list={this.state.customerTempList}
              onSelect={this.handleCustomerSelect}
              disabled={this.props.isOnlyEditDiscount || this.props.disabledEditInfo}
            />
          </InfoItem>
          <InfoItem label={<Required>客户姓名</Required>} span={this.props.itemSpan}>
            <Input
              placeholder="家长姓名"
              value={this.props.customerName}
              onChange={(e) => this.updateProps({ customerName: e.target.value })}
              disabled={this.props.isOnlyEditDiscount || this.props.disabledEditInfo || this.state.customerNameDisable}
            />
          </InfoItem>
          <InfoItem label={<Required>学生姓名</Required>} span={this.props.itemSpan}>
            <InputSelect
              placeholder="学生姓名"
              value={this.props.studentName}
              list={this.state.studentTempList}
              onSelect={this.handleStudentSelect}
              onChange={this.handleStudentNameInput}
              showSelect={this.state.studentSelectVisible}
              disabled={this.props.isOnlyEditDiscount || this.props.disabledEditInfo}
            />
            {this.props.canEditStdName && (
              <div className={style.changeStdName} onClick={this.switchChangedNameVisible} title="更换姓名">
                <Icon type="edit" />
              </div>
            )}
          </InfoItem>
          <InfoItem label={<Required>年级</Required>} span={this.props.itemSpan}>
            <Select
              dict={this.props.gradeDict}
              value={this.props.gradeId}
              onChange={(val) => this.updateProps({ gradeId: val })}
              // 兼容学生年级空的情况
              disabled={
                this.props.isOnlyEditDiscount ||
                (this.props.disabledEditGrade && this.props.disabledEditInfo && Boolean(this.props.gradeId))
              }
            />
          </InfoItem>
          <InfoItem label={<Required>合同学习中心</Required>} span={this.props.itemSpan}>
            <Select
              showSearch
              allowClear={false}
              value={this.props.blCampusId}
              disabled={this.props.isOnlyEditDiscount}
              dict={this.props.branchCampusDict}
              onChange={(val) =>
                this.updateProps({
                  blCampusId: val,
                  contractBranchId: getBranchByCampus(val, this.props.branchCampusDict),
                })
              }
            />
          </InfoItem>
          <InfoItem label={this.isTwoTeacher ? '学校' : <Required>学校</Required>} span={this.props.itemSpan}>
            <Input
              readOnly={true}
              value={this.props.schoolName}
              onClick={this.switchSchoolSelectVisible}
              disabled={this.props.isOnlyEditDiscount}
            />
          </InfoItem>
          <InfoItem label="证件类型" span={this.props.itemSpan}>
            <Select
              placeholder="请选择证件类型"
              value={this.props.customerIdCardType}
              onChange={(val) => this.updateProps({ customerIdCardType: val })}
              disabled={this.props.isOnlyEditDiscount || this.props._CertificateTypeDisabled}
              dict={this.props._lockCertificateType ? defaultCertificateTypeDict : this.props.certificateTypeDict}
            />
          </InfoItem>
          <InfoItem label="证件号码" span={this.props.itemSpan}>
            <Input
              value={this.props.customerIdCardNo}
              onChange={(e) => this.updateProps({ customerIdCardNo: e.target.value })}
              disabled={this.props.isOnlyEditDiscount || this.props._CertificateNumDisabled}
            />
          </InfoItem>
        </InfoForm>

        {this.state.schoolSelectVisible && (
          <ModalSelectSchool
            blCampusId={this.props.blCampusId}
            schoolId={this.props.schoolId}
            onOk={this.handleSchoolSelected}
            onCancel={this.switchSchoolSelectVisible}
          />
        )}

        {this.state.changedNameVisible && (
          <ModalChangeStuName
            cusId={this.props.customerId}
            studentId={this.props.studentId}
            studentName={this.props.studentName}
            onCancel={this.switchChangedNameVisible}
            onOk={this.updateStudentName}
            onMerge={this.handleMergeStudent}
          />
        )}

        {this.state.mergeStudentVisible && (
          <ModalInfo title="结果反馈" onCancel={this.closeMergeStudentModal} isAutoClose>
            {this.state.mergeStudentInfo.mergeMessage}
          </ModalInfo>
        )}
      </InfoBlock>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    identityErr: state[storeName].errors.identity,
    branchCampusDict: state.dict.branchAllCampus,
    // gradeDict: state.dict.grade,
    certificateTypeDict: state.dict.certificateType,
    confirmCustomerContact: state[storeName].confirmCustomerContact,
    customerContact: state[storeName].customerContact,
    customerId: state[storeName].customerId,
    customerName: state[storeName].customerName,
    studentId: state[storeName].studentId,
    studentName: state[storeName].studentName,
    gradeId: state[storeName].gradeId,
    schoolId: state[storeName].schoolId,
    schoolName: state[storeName].schoolName,
    blCampusId: state[storeName].blCampusId,
    customerIdCardNo: state[storeName].customerIdCardNo,
    customerIdCardType: state[storeName].customerIdCardType,
    _lockCertificateType: state[storeName]._lockCertificateType,
    _CertificateTypeDisabled: state[storeName]._CertificateTypeDisabled,
    _CertificateNumDisabled: state[storeName]._CertificateNumDisabled,
    disabledEditInfo: state[storeName].disabledEditInfo,
    canEditStdName: state[storeName].canEditStdName,
    disabledEditGrade: state[storeName].disabledEditGrade,
  };
};

export default connect(mapStateToProps)(StudentInfo);
