import { runFunc, loginUser } from 'UTIL/common';
import { AliLogTracker, logTracker } from 'UTIL/aliLog';
import { logStoreName, bizStoreName } from './config';

/**
 * 创建初始状态
 */
const createDefaultState = () => {
  return {
    traceId: '',
  };
};

export default {
  namespace: logStoreName,
  state: createDefaultState(),
  reducers: {
    /**
     * 重置 state
     * @param {Object} state
     */
    reset(state) {
      return createDefaultState();
    },
    /**
     * 更新跟踪id
     * @param {Object} state
     * @param {Object} param1
     * @param {Object} param1.payload
     */
    updateTraceId(state, { payload }) {
      return {
        ...state,
        traceId: payload.traceId,
      };
    },
  },
  effects: {
    /**
     * 初始化跟踪id
     * @param {Object} actions
     * @param {Object} param1
     * @param {Object} param1.put
     */
    *initTrace(actions, { put }) {
      const traceId = AliLogTracker.createTraceId();

      yield put({
        type: 'updateTraceId',
        payload: {
          traceId,
        },
      });

      yield put({
        type: 'logTrace',
        payload: {
          event_type: 'view',
        },
      });
    },
    *logTrace({ payload }, { select, put }) {
      const record = payload || {};
      const traceId = yield select((state) => state[logStoreName].traceId);
      const { studentId, customerId, cntrStatus } = yield select((state) => {
        const { studentId, customerId, isEdit } = state[bizStoreName];

        return {
          studentId,
          customerId,
          cntrStatus: isEdit ? 'EDITING' : 'CREATIING',
        };
      });
      const userId = yield select((state) => state.account.user.userId);

      if (traceId) {
        yield logTracker.log({
          ...record,
          topic: 'new_sign_contract',
          cntr_status: cntrStatus,
          stud_id: studentId,
          cust_id: customerId,
          user_id: userId,
          client_time: Date.now(),
          trace_id: traceId,
        });
      }
    },
    *logTabChange({ payload }, { put }) {
      yield put({
        type: 'logTrace',
        payload: {
          event_type: 'change',
          event_target: payload.value,
          event_target_alias: payload.label,
        },
      });
    },
    *logBtnClick({ payload }, { put }) {
      yield put({
        type: 'logTrace',
        payload: {
          event_type: 'click',
          event_target: payload.value,
          event_target_alias: payload.label,
        },
      });
    },
    *logBoxCheck({ payload }, { put }) {
      const eventType = payload.checked ? 'checked' : 'unchecked';

      yield put({
        type: 'logTrace',
        payload: {
          event_type: eventType,
          event_target: payload.value,
          event_target_alias: payload.label,
        },
      });
    },
    *logCntrSubmit({ payload }, { put }) {
      yield put({
        type: 'logTrace',
        payload: {
          event_type: 'submit',
          cntr_id: payload.contractId,
        },
      });
    },
  },
};
