import React from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';
import styles from './styles.module.less';
import { WINTER, SPRING, SUMMER, AUTUMN } from 'CONST/dicts/common';

function getNameText(text) {
  return text.slice(0, 1);
}

const colorMap = {
  [WINTER]: styles.winter,
  [SPRING]: styles.spring,
  [SUMMER]: styles.summer,
  [AUTUMN]: styles.autumn,
};

const SeasonSubjectMark = (props) => {
  return (
    <div className={classNames(styles.mark, colorMap[props.quarterId] || '', { [styles.invalid]: props.invalid })}>
      {props.quarterName && (
        <span className={styles.season} title={props.quarterName}>
          {getNameText(props.quarterName)}
        </span>
      )}
      {props.subjectName && (
        <span className={styles.subject} title={props.subjectName}>
          {getNameText(props.subjectName)}
        </span>
      )}
    </div>
  );
};

SeasonSubjectMark.propTypes = {
  quarterName: PropTypes.string,
  subjectName: PropTypes.string,
  invalid: PropTypes.bool,
};

SeasonSubjectMark.defaultProps = {
  invalid: false,
};

export default SeasonSubjectMark;
