import React from 'react';
import { connect } from 'dva';
import { Icon } from 'antd';
import { Modal, Button } from 'COMPONENT';
import classNames from 'classnames';
import style from './styles.module.less';
import toolbarStyle from './components/toolbar/styles.module.less';
import Page from './components/Page';
import FooterBar from './FooterBar';
import TipText from './components/TipText';
import Text from './components/Text';
import InfoBlock from './components/InfoBlock';
import FoldPanel from './components/FoldPanel';
import Pay from '../modals/Pay';
import ProductTable from './ProductTable';
import StudentInfo from './StudentInfo';
import { Switch } from 'antd';
import { PAID } from './config';
import { storeName, ECS_SWITCH, ADD_PRODUCT, CONFIRM_PRODUCT } from './store/storeCfg';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import AddProduct from './AddProduct';
import Order from './Order';
import { runFunc, loginUser, errorsHandle } from 'UTIL/common';
import { browserHistory } from 'react-router';
import { OLDPAGEBASE } from 'CONST/config';
import { jsonParseObjAttrByKey } from './utils';
import { logStoreName } from './alilogStore/config';
import { getDictTextByVal, loadCommonDictNew } from 'UTIL/dict';
import { SPECIAL_PRODUCT_TYPE } from './config';
import { getContractInfoById, getContractCouponDepositSaveFundTag } from 'SERVICE/contract';
import countdown from 'COMPONENT/Modal/countdown';
import PromotionDetail from './PromotionDetail';

class SignContract extends React.Component {
  constructor(props) {
    super(props);

    this.contractFundTypeName = ''; // 合同收款类型
    this.params = jsonParseObjAttrByKey(this.props.routeParams, 'params') || {};

    this.restore = this.restore.bind(this);
    this.init = this.init.bind(this);
    this.initFuncAuth = this.initFuncAuth.bind(this);
    this.initDict = this.initDict.bind(this);
    this.initInfo = this.initInfo.bind(this);
    this.switchIsEcs = this.switchIsEcs.bind(this);
    this.switchCouponModalVisible = this.switchCouponModalVisible.bind(this);
    this.selectCoupon = this.selectCoupon.bind(this);
    this.validateContract = this.validateContract.bind(this);
    this.checkSameStudent = this.checkSameStudent.bind(this);
    this.saveContract = this.saveContract.bind(this);
    this.handleSameStudent = this.handleSameStudent.bind(this);
    this.save = this.save.bind(this);
    this.saveEdit = this.saveEdit.bind(this);
    this.directSave = this.directSave.bind(this);
    this.saveAndGoToList = this.saveAndGoToList.bind(this);
    this.saveAndPay = this.saveAndPay.bind(this);
    this.goToContractList = this.goToContractList.bind(this);
    this.getOnPayment = this.getOnPayment.bind(this);
    this.openSelectPayType = this.openSelectPayType.bind(this);
    this.closeSelectPayType = this.closeSelectPayType.bind(this);
    this.switchSameStudentVisible = this.switchSameStudentVisible.bind(this);
    this.onStepChange = this.onStepChange.bind(this);
    this.updateLoading = this.updateLoading.bind(this);
    this.switchPromotionDetail = this.switchPromotionDetail.bind(this);

    //埋点上报日志
    this.initLogTrace = this.initLogTrace.bind(this);
    this.logSavingCntr = this.logSavingCntr.bind(this);
    this.logSubmitCntr = this.logSubmitCntr.bind(this);
    this.batchLogSubmitContr = this.batchLogSubmitContr.bind(this);

    this.state = {
      loading: false,
      gradeDict: [],
      modalVisible: false,
      modalInitSKey: null,
      isConfirmVisible: true,
      couponModalVisible: false,
      payTypeModalVisible: false,
      contractIds: null,
      isQualityContract: false,
      sameStudentVisible: false,
      sameStudentProps: null,
      isBroadcastCooperate: false,
    };
    this.isTwoTeacher = loginUser.isTwoTeacherSym();
    this.isExBoss = loginUser.isExBoss();
    this.contractIds = [];

    this.scrollTagId = 'scroll-tag';
    this.scrollTagDom = null;
  }

  componentDidMount() {
    this.init();
  }

  componentWillUnmount() {
    this.restore();
  }

  restore() {
    this.props.dispatch({ type: `${storeName}/restore` });
    this.props.dispatch({ type: `${logStoreName}/reset` });
  }

  async init() {
    this.setState({ loading: true });
    this.restore();

    await new Promise(this.initFuncAuth);
    await new Promise(this.initDict);
    await new Promise(this.initInfo(this.params));

    this.initLogTrace();
    this.setState({ loading: false });
  }

  initFuncAuth(resolve) {
    this.props.dispatch({
      type: `${storeName}/initFuncAuth`,
      payload: {
        callback() {
          resolve();
        },
      },
    });
  }

  initDict(resolve) {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['branchAllCampus', 'grade', 'certificateType', 'ecsClassType'],
        callback() {
          resolve();
        },
      },
    });
    loadCommonDictNew('STUDENT_GRADE').then((gradeDict) => {
      this.setState({ gradeDict });
    });
  }

  initInfo(payload = {}) {
    return (resolve) => {
      this.props.dispatch({
        type: `${storeName}/initInfo`,
        payload: {
          ...payload,
          callback() {
            resolve();
          },
        },
      });
    };
  }

  switchIsEcs(checked) {
    if (this.props.selectedList.length > 0) {
      showConfirm(
        <div style={{ textAlign: 'center' }}>
          <p>{`${checked ? '非' : ''}精英班已有选购产品`}</p>
          <p>{`若切换到${!checked ? '非' : ''}精英班则会清空产品购物车并且重算优惠，是否确认切换？`}</p>
        </div>,
        () => {
          this.props.dispatch({
            type: `${storeName}/switchIsEcsClearSelected`,
            payload: {
              checked,
            },
          });
        },
      );
    } else {
      this.props.dispatch({
        type: `${storeName}/switchIsEcs`,
        payload: { checked },
      });
    }
  }

  switchCouponModalVisible() {
    this.setState((prevState) => ({ couponModalVisible: !prevState.couponModalVisible }));
  }

  selectCoupon(list) {
    this.props.dispatch({
      type: `${storeName}/selectCoupon`,
      payload: { list },
    });
    this.switchCouponModalVisible();
  }

  validateContract(resolve) {
    this.props.dispatch({
      type: `${storeName}/validateContract`,
      payload: {
        callback(canSave) {
          resolve(canSave);
        },
      },
    });
  }

  // 是否只可编辑优惠券
  get isOnlyEditDiscount() {
    return this.params.isEditDiscount === '1';
  }

  checkSameStudent(resolve) {
    this.props.dispatch({
      type: `${storeName}/checkSameStudent`,
      payload: {
        callback(options) {
          resolve(options);
        },
      },
    });
  }

  saveContract(resolve) {
    this.props.dispatch({
      type: `${storeName}/submit`,
      payload: {
        callback(contractId) {
          resolve(contractId);
        },
      },
    });
  }

  handleSameStudent(options) {
    if (options.type === 'modal') {
      this.switchSameStudentVisible(options);
    }
  }

  async save(callback) {
    this.setState({ loading: true });
    const canSave = await new Promise(this.validateContract);
    if (canSave) {
      // 继续保存
      const contractIds = await new Promise(this.saveContract);
      if (contractIds) {
        this.contractIds = contractIds;
        this.batchLogSubmitContr(contractIds);
        runFunc(callback, contractIds);
      }
    }
    this.setState({ loading: false });
  }

  async saveEdit(callback) {
    this.setState({ loading: true });
    const canSave = await new Promise(this.validateContract);
    if (canSave) {
      showConfirm(
        '修改合同产品，可能导致已分配业绩与修改后的资金不匹配而清空业绩，如需要调整业绩，请移步收款记录手动分配',
        async () => {
          const contractIds = await new Promise(this.saveContract);
          if (contractIds) {
            this.contractIds = contractIds;
            runFunc(callback, contractIds);
          }
          this && this.setState && this.setState({ loading: false });
        },
        () => {
          this.setState({ loading: false });
        },
      );
    } else {
      this.setState({ loading: false });
    }
  }

  async directSave(callback) {
    this.setState({ loading: true });
    const contractId = await new Promise(this.saveContract);
    if (contractId) {
      runFunc(callback, contractId);
    }
    this.setState({ loading: false });
  }

  saveAndGoToList() {
    if (this.props.isEdit) {
      this.saveEdit(this.goToContractList, true);
    } else {
      this.save(this.goToContractList, true);
      this.logSavingCntr('SAVE_AND_TO_CNTR_LIST');
    }
  }

  saveAndPay() {
    if (this.props.isEdit) {
      this.saveEdit(this.getOnPayment);
    } else {
      this.save(this.getOnPayment);
      this.logSavingCntr('SAVE_AND_PAY');
    }
  }

  goToContractList(contractIdsArgs) {
    const contractIds = contractIdsArgs || this.contractIds;

    let params = {
      contractIds,
      startDate: this.props.signDate,
      endDate: null,
      stuName: this.props.studentName,
    };
    if (contractIds.length === 1) {
      params = {
        contractId: contractIds[0],
        startDate: null,
        endDate: null,
      };
    }

    window.sessionStorage.setItem(
      'contr_list:startup_params',
      JSON.stringify({
        ...params,
        actionType: 'INIT_SEARCH_KEY',
        from: window.location.pathname + window.location.search + window.location.hash,
        timestamp: Date.now(),
      }),
    );

    setTimeout(function () {
      browserHistory.push({
        pathname: OLDPAGEBASE,
        query: {
          src: `function/contract/listContract.html?timestamp=${Date.now()}`,
        },
      });
    });

    this.props.dispatch({
      type: 'main/removeTab',
      payload: { id: this.props.activeTabId },
    });
  }

  async getOnPayment(contractIds) {
    const res = await getContractInfoById({ contractId: contractIds[0] }).catch(errorsHandle());
    if (res && res.resultCode === 0) {
      this.paidStatus = res.data.paidStatus;
      this.contractFundTypeName = res.data.contractFundTypeName; // 合同收款类型
      if (this.props.isEdit && res.data.paidStatus === PAID) {
        return countdown({
          title: '温馨提示',
          content: '由于您修改合同后，合同待付金额为0，无须支付，将直接跳转到合同列表',
          buttonText: '直接跳转',
          maxCount: 3,
          onOk: this.goToContractList,
          onCancel: this.goToContractList,
        });
      }
    } else {
      this.paidStatus = null;
      this.contractFundTypeName = '';
    }

    const rsp = await getContractCouponDepositSaveFundTag({ contractIds }).catch(errorsHandle());
    if (rsp && rsp.resultCode === 0 && !rsp.data) {
      return countdown({
        title: '温馨提示',
        content: '合同使用的抵扣券正在生成收款记录，请稍后再发起收款',
        buttonText: '跳转合同列表',
        maxCount: 3,
        onOk: this.goToContractList,
        onCancel: this.goToContractList,
      });
    }

    this.openSelectPayType(contractIds);
  }

  openSelectPayType(contractIds) {
    this.setState({
      payTypeModalVisible: true,
      contractIds,
    });
  }

  closeSelectPayType() {
    this.setState(
      {
        payTypeModalVisible: false,
        contractIds: null,
      },
      () => {
        this.goToContractList();
      },
    );
  }

  switchSameStudentVisible(sameStudentProps) {
    this.setState((prevState) => ({
      sameStudentVisible: !prevState.sameStudentVisible,
      sameStudentProps: sameStudentProps || null,
    }));
  }

  onStepChange(e) {
    const stepType = e.target.dataset.stepType;
    const { isEdit, contractProductTag } = this.props;

    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: { stepType },
    });

    if (stepType === CONFIRM_PRODUCT && !this.props.isEdit) {
      this.props.dispatch({
        type: `${storeName}/batchCalcCoursePromotion`,
      });

      this.props.dispatch({
        type: `${storeName}/fetchCustomerDeposit`,
      });
    }

    // 编辑小班、一对多合同
    if (stepType === ADD_PRODUCT && isEdit && SPECIAL_PRODUCT_TYPE[contractProductTag]) {
      // 编辑合同是否是素质类合同
      const { isQualityContract, isBroadcastCooperate } = this.props;
      this.setState(
        {
          isQualityContract,
          isBroadcastCooperate, // 合作项目素质类
        },
        () => {
          setTimeout(() => {
            const { isQualityContract, isBroadcastCooperate } = this.state;
            this.addProductRef &&
              this.addProductRef
                .getWrappedInstance()
                .initSearchValueWhenEditMode(isQualityContract, isBroadcastCooperate);
          }, 1000);
        },
      );
    }

    // 滚动到顶部
    if (!this.scrollTagDom) {
      this.scrollTagDom = document.getElementById(this.scrollTagId);
    }
    this.scrollTagDom.scrollTop = 0;
  }

  updateLoading(loading) {
    this.setState({ loading });
  }

  switchPromotionDetail() {
    this.setState((prevState) => ({ promotionDetailVisible: !prevState.promotionDetailVisible }));
  }

  initLogTrace() {
    this.props.dispatch({ type: `${logStoreName}/initTrace` });
  }

  logSavingCntr(type) {
    const aliasMap = {
      SAVE_AND_TO_CNTR_LIST: '保存并跳转合同列表',
      SAVE_AND_PAY: '保存并支付',
      SAVE_AND_ASSIGN: '保存并分配课时',
    };

    this.props.dispatch({
      type: `${logStoreName}/logBtnClick`,
      payload: {
        value: type,
        label: aliasMap[type],
      },
    });
  }

  logSubmitCntr(contractId) {
    this.props.dispatch({
      type: `${logStoreName}/logCntrSubmit`,
      payload: {
        contractId,
      },
    });
  }

  batchLogSubmitContr(contractIds) {
    contractIds.forEach((contractId) => {
      this.logSubmitCntr(contractId);
    });
  }

  handleCloseConfirmModal = () => {
    this.setState({ isConfirmVisible: false });
  };

  render() {
    const isConfirmProductStep = this.props.stepType === CONFIRM_PRODUCT;

    return (
      <Page
        className={style.SignContract}
        loading={this.state.loading}
        footer={
          <FooterBar
            stepType={this.props.stepType}
            totalAmount={this.props.totalAmount}
            promotionAmount={this.props.promotionAmount}
            realAmount={this.props.realAmount}
            paidAmount={this.props.paidAmount}
            onPromotionDetail={this.switchPromotionDetail}
            amountVisible={isConfirmProductStep}
            onSave={this.saveAndGoToList}
            onSaveAndPay={this.saveAndPay}
            onStepChange={this.onStepChange}
            isOnlyEditDiscount={this.isOnlyEditDiscount}
            isEdit={this.props.isEdit}
          />
        }
      >
        <FoldPanel
          fold={isConfirmProductStep}
          foldWidth={296}
          main={
            <div id={this.scrollTagId} className={style.mainWrap}>
              <div style={{ marginTop: 11 }}>
                <InfoBlock
                  toolbar={
                    <div className={classNames(toolbarStyle.toolbar, style.infoToolBar)}>
                      <div className={toolbarStyle.toolbarLeft}>
                        <Text label="合同类型">
                          <TipText fontWeight="bold">{this.props.contractTypeName || '-'}</TipText>
                        </Text>
                        <Text label="签约人">{this.props.signPersonName}</Text>
                        {!this.isExBoss && <Text label="签约日期">{this.props.signDate}</Text>}
                      </div>
                      {this.props.funcAuth.includes(ECS_SWITCH) && (
                        <div className={toolbarStyle.toolbarRight}>
                          <span className={style.infoToolBarSwitch}>是否精英班合同</span>
                          <Switch
                            checked={this.props.isEcs}
                            onChange={this.switchIsEcs}
                            checkedChildren="是"
                            unCheckedChildren="否"
                            disabled={this.isOnlyEditDiscount || this.props.isEdit}
                          />
                        </div>
                      )}
                    </div>
                  }
                  style={{ borderBottom: '1px solid #f0f0f0' }}
                  key="contractInfo"
                />
              </div>
              {/*学员信息*/}
              <StudentInfo
                key="studentInfo"
                itemSpan={isConfirmProductStep ? 6 : 8}
                isOnlyEditDiscount={this.isOnlyEditDiscount}
                gradeDict={this.state.gradeDict}
              />
              {this.props.isAlreadyInit && (
                <div className={style.productWrap}>
                  {/**选购课程*/}
                  <div style={{ display: this.props.stepType === ADD_PRODUCT ? 'block' : 'none' }}>
                    <InfoBlock title="选购课程" key="addProduct">
                      <AddProduct
                        updateLoading={this.updateLoading}
                        defaultValue={this.state.modalInitSKey}
                        ref={(ref) => (this.addProductRef = ref)}
                        isBroadcastCooperate={this.state.isBroadcastCooperate}
                        isQualityContract={this.state.isQualityContract}
                      />
                    </InfoBlock>
                  </div>
                  {/**合同明细*/}
                  <div style={{ display: isConfirmProductStep ? 'block' : 'none' }}>
                    <InfoBlock
                      title="合同明细"
                      toolbar={
                        <div className={style.infoWrapper}>
                          <span
                            className={classNames(toolbarStyle.toolbar, style.infoToolBar)}
                            style={{ alignItems: 'center', marginLeft: '10px' }}
                          >
                            共{this.props.groupList.length}个合同
                          </span>
                          {this.props.hasDeposit && (
                            <span className={style.depositTips}>（学生名下存在未使用定金券，请在【收款】时使用）</span>
                          )}
                        </div>
                      }
                      key="productTable"
                    >
                      <ProductTable isOnlyEditDiscount={this.isOnlyEditDiscount} />
                    </InfoBlock>
                  </div>
                </div>
              )}
            </div>
          }
          sub={
            <div className={style.subWrap}>
              <Order />
            </div>
          }
        />

        {this.state.promotionDetailVisible && (
          <PromotionDetail data={this.props.selectedList} onCancel={this.switchPromotionDetail} />
        )}

        {this.state.payTypeModalVisible && (
          <Pay
            campusId={this.props.blCampusId}
            onCancel={this.closeSelectPayType}
            contractIds={this.state.contractIds}
            studentId={this.props.studentId}
            campusName={this.props.campusName}
            customerName={this.props.customerName}
            studentName={this.props.studentName}
            contractNumber={this.props.customerContact}
            signTime={this.props.signDate}
            paidStatus={this.paidStatus}
            contractFundTypeName={this.contractFundTypeName}
            isEcs={this.props.isEcs}
          />
        )}
        {this.state.isConfirmVisible && this.isOnlyEditDiscount && (
          <Modal
            width={439}
            title="温馨提示"
            onCancel={this.handleCloseConfirmModal}
            footer={
              <Button type="primary" onClick={this.handleCloseConfirmModal}>
                已知悉
              </Button>
            }
          >
            <div style={{ marginBottom: 30, marginLeft: -10 }}>
              <Icon type="exclamation-circle" style={{ fontSize: 18, color: '#ffbf00', marginRight: 4 }} />
              <span style={{ color: '#666666' }}>本次修改合同产品优惠，将清空已享受优惠，请谨慎操作！</span>
            </div>
          </Modal>
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    isAlreadyInit: state[storeName].isAlreadyInit,
    stepType: state[storeName].stepType,
    studentId: state[storeName].studentId,
    studentBranchId: state[storeName].studentBranchId,
    studentCampusId: state[storeName].studentCampusId,
    blCampusId: state[storeName].blCampusId,
    contractBranchId: state[storeName].contractBranchId,
    isEdit: state[storeName].isEdit,
    funcAuth: state[storeName].funcAuth,
    hadOneToOneOrMorePrd: state[storeName].hadOneToOneOrMorePrd,
    isEcs: state[storeName].isEcs,
    contractTypeName: state[storeName].contractTypeName,
    signPersonName: state[storeName].signPersonName,
    signDate: state[storeName].signDate,
    totalAmount: state[storeName].totalAmount,
    promotionAmount: state[storeName].promotionAmount,
    realAmount: state[storeName].realAmount,
    paidAmount: state[storeName].paidAmount,
    selectedList: state[storeName].selectedList,
    groupList: state[storeName].groupList,
    couponList: state[storeName].couponList,
    activeTabId: state.main.activeTabId,
    isQualityContract: state[storeName].isQualityContract,
    isBroadcastCooperate: state[storeName].isBroadcastCooperate,
    customerContact: state[storeName].customerContact,
    customerName: state[storeName].customerName,
    studentName: state[storeName].studentName,
    contractProductTag: state[storeName].contractProductTag,
    campusName: state[storeName].blCampusId
      ? getDictTextByVal(state.dict.branchAllCampus, state[storeName].blCampusId)
      : null,
    hasDeposit: state[storeName].hasDeposit,
  };
};

export default connect(mapStateToProps)(SignContract);
