import {
  storeName,
  ECS_SWITCH,
  TAB_TRIGGER,
  AUTO_CALC_PROMOTION,
  CHOOSE_COUPON,
  XH_BTN,
  ADD_PRODUCT,
  CONFIRM_PRODUCT,
} from './storeCfg';
import { clone } from 'lodash';
import NP from 'number-precision';
import { runFunc, loginUser, errorsHandle, validIdentityCardByType } from 'UTIL/common';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { centToYuan, yuanToCent, getBranchByCampus } from '../utils';
import {
  contractTypeCfg,
  SMALL_CLASS,
  TWO_TEACHER,
  ONE_ON_ONE_COURSE,
  ONE_ON_MANY,
  ECS_CLASS,
  OTHERS,
  UNVALID,
  LECTURE,
  CLASS_COURSE,
  activeUseCaseCfg,
  // yinliu,
  productTypeNameCfg,
  useCouponGroupTypes,
  useCouponProductTypes,
  NEW_CONTRACT,
  RE_CONTRACT,
  CT_IDENTITY_CARD,
  OTHER_CATEGORY_TYPE_MARKETING,
} from '../config';
import {
  getNewSignContractType,
  getContractStudentInfo,
  getCurrentBranchTeacher,
  calculatePromotion,
  getProductActivePromotionList,
  calculatePrdPromotion,
  batchSaveContractInfo,
  batchEditContractInfo,
  getStudentGradeInContinuedCycle,
  getContractInfoById,
  calculateCoupon,
  getStudentIfHasContract,
  getSuitablePromotion,
} from 'SERVICE/contract';
import moment from 'moment';
import request from 'UTIL/http/fetch';
import {
  getPrdRelation,
  addProductWithSub,
  addClassIdByUid,
  contractInfoAdepter,
  takeOutPromotionProperties,
  getGroupList,
  getLimitProduct,
  calcItemPayment,
  calcItemPromotionAmount,
  calcItemRealAmount,
  calcItemPaidAmount,
  removePrdPromotion,
  removeActivePromotion,
  calcGroupTotal,
  getCouponTypeName,
  isNormalClassCourse,
  isNormalOneToN,
  isNormalCanUseCoupon,
  isClass,
} from './storeUtils';
import { formatClassInfoApiParam, formatRuleNumAndRuleUnit } from '../ModalSelectCoupon/util';
import { getDepositCouponsByStudent } from 'SERVICE/contract';
import { CATEGORY_A, contractPayCategoryDict } from 'CONST/dicts/product';

const errorsHandler = errorsHandle();

const defaultState = {
  // 服务器时间戳
  timeStamp: null,
  // 服务器时间YYYY-MM-DD
  timeText: '',
  //
  isAlreadyInit: false,
  isEdit: false,
  limitProduct: null,
  funcAuth: [],
  isEcs: false,
  calcLoading: false,
  disabledEditInfo: false,
  hadOneToOneOrMorePrd: false,
  canEditStdName: false,
  // 操作步骤(添加产品、确认产品明细)
  stepType: null,
  // 合同产品性质(辅助修改合同时产品筛选项相关逻辑)
  contractCharacter: null,
  // 合同产品类型(辅助修改合同时产品筛选项相关逻辑)
  contractProductTag: null,
  // 产品分账(辅助判断是否可以添加合同产品)
  contractPayCategory: null,
  //
  transactionUuid: null,
  contractId: null,
  contractType: null,
  contractTypeName: null,
  signPerson: null,
  signPersonName: null,
  signDate: null,
  //
  studentBranchId: null,
  studentCampusId: null,
  studentType: null,
  // 合同分公司
  contractBranchId: null,
  //
  confirmCustomerContact: null,
  customerContact: null,
  customerId: null,
  customerName: null,
  studentId: null,
  studentName: null,
  gradeId: null,
  schoolId: null,
  schoolName: null,
  virtualSchool: null,
  blCampusId: null,
  customerIdCardNo: null,
  customerIdCardType: null,
  remark: null,
  transferIntroduceId: null,
  totalAmount: 0,
  promotionAmount: 0,
  realAmount: 0,
  promotionCoupon: [],
  //
  errors: {},
  // 关联产品列表
  linkExpandKeys: [],
  linkRowKeys: [],
  linkList: [],
  // 已选产品列表
  selectedList: [],
  // 合同分组列表
  groupList: [],
  // 优惠券
  couponList: [],
  // 上次优惠计算返回结果
  lastTimeCalResult: null,
  // 已删合同产品(key: 产品id + 班级id || '', val: 合同产品id)
  removeContractPrdStorage: {},
  grade: {}, // 学生年级
  // 签单入口
  signEntrance: 'NONE',
  // 定金券
  depositList: [],
  hasDeposit: false,
};
Object.freeze(defaultState);

export default {
  namespace: storeName,
  state: {
    ...defaultState,
    defaultFuncAuth: null,
    productPromotionDict: null,
    teacherDict: null,
  },
  reducers: {
    updateState(state, { payload }) {
      return {
        ...state,
        ...payload,
      };
    },
    // 计算优惠loading
    updateCalcLoading(state, { payload }) {
      return {
        ...state,
        calcLoading: payload,
      };
    },
    // 切换功能权限
    switchFuncAuth(state) {
      let payload = { funcAuth: clone(state.defaultFuncAuth) };
      if (state.isEcs) {
        payload.funcAuth = payload.funcAuth.filter(
          (item) => [TAB_TRIGGER, AUTO_CALC_PROMOTION, CHOOSE_COUPON].indexOf(item) === -1,
        );
      }
      return {
        ...state,
        ...payload,
      };
    },
    // 更新已选列表及对应的分组
    updateSelected(state, { payload }) {
      return {
        ...state,
        selectedList: payload.selectedList,
        groupList: getGroupList(payload.selectedList, state.groupList, state.isEdit),
      };
    },
    // 更新分组信息
    updateGroupList(state, { payload }) {
      return {
        ...state,
        groupList: payload.groupList,
      };
    },
    // 更新分组子项
    updateProductGroup(state, { payload }) {
      let index = state.groupList.findIndex((item) => item._uid === payload.productGroupItem._uid);
      const groupList = clone(state.groupList);
      if (index > -1) {
        groupList[index] = payload.productGroupItem;
      }

      return {
        ...state,
        groupList,
      };
    },
    // 重置页面
    restore(state) {
      return {
        ...state,
        ...defaultState,
      };
    },

    updateErrors(state, { payload }) {
      return {
        ...state,
        errors: {
          ...state.errors,
          ...payload,
        },
      };
    },
  },
  effects: {
    // 初始化页面功能权限
    *initFuncAuth({ payload: { callback } }, { select, put }) {
      let payload = {
        funcAuth: [ECS_SWITCH, TAB_TRIGGER, AUTO_CALC_PROMOTION, CHOOSE_COUPON, XH_BTN],
        isEcs: false,
      };
      // 存储一个默认功能配置
      payload.defaultFuncAuth = clone(payload.funcAuth);

      yield put({
        type: 'updateState',
        payload,
      });

      runFunc(callback);
    },

    /**
     * 初始化页面所需信息
     *
     * @param studentId {string?} 学生id
     * @param customerId {string?} 客户id
     * @param blCampusId {string?} 合同学习中心id
     * @param contractId {string?} 合同id
     * @param transferIntroduceId {string?} 转介绍id
     * @param flag {boolean?} 是否从咨询师工作台跳转过来
     * @param contractType {string?} 合同类型
     * @param callback {function?}
     * @param select
     * @param put
     * */
    *initInfo(
      {
        payload: {
          contractId,
          studentId,
          customerId,
          blCampusId,
          transferIntroduceId,
          flag,
          signEntrance,
          contractType,
          callback,
        },
      },
      { select, put },
    ) {
      let payload = yield select((state) => {
        const timeStamp = state.account.user.timeStamp;
        return {
          isAlreadyInit: true,
          signPerson: state.account.user.userId,
          signPersonName: state.account.user.name,
          blCampusId: loginUser.isCampusRole() ? state.account.user.campusId : null,
          contractBranchId: loginUser.isCampusRole() ? state.account.user.brenchId : null,
          signDate: moment(timeStamp || new Date()).format('YYYY-MM-DD'),
          studentId,
          customerId,
          contractId,
          transferIntroduceId,
          flag,
          signEntrance,
          isEdit: false,
          stepType: ADD_PRODUCT,
        };
      });
      if (signEntrance) {
        payload.signEntrance = signEntrance;
      }
      // 入参携带学习中心id优先级高于用户角色学习中心
      if (blCampusId) {
        const dict = yield select((state) => state.dict.branchAllCampus);
        payload.blCampusId = blCampusId;
        payload.contractBranchId = getBranchByCampus(blCampusId, dict);
      }

      if (contractId) {
        // 获取合同详情
        const rspContract = yield getContractInfoById({ contractId }).catch(errorsHandle());
        if (rspContract) {
          const funcAuth = yield select((state) => state[storeName].funcAuth.filter((item) => item !== TAB_TRIGGER));
          payload = {
            ...payload,
            ...contractInfoAdepter(rspContract.data),
            contractCharacter: rspContract.data.contractCharacter,
            contractProductTag: rspContract.data.contractProductTag,
            limitProduct: getLimitProduct(rspContract.data.contractProductArr),
            isEdit: true,
            stepType: CONFIRM_PRODUCT,
            funcAuth,
          };
        }
      } else if (studentId) {
        // 获取合同类型
        yield put({
          type: 'initContractType',
          payload: { studentId, flag, contractType },
        });
      }

      // 获取产品优惠
      yield put({ type: 'getProductPromotionDict' });

      if (payload.studentId) {
        let fetchList = [
          // 学生信息
          getContractStudentInfo(
            {
              studentId: payload.studentId,
              customerId: payload.customerId,
            },
            true,
          ),
          // 学生是否存在其他常规合同
          getStudentIfHasContract({
            studentId: payload.studentId,
          }),
        ];

        const [rspStudentInfo, rspHasContract] = yield Promise.all(fetchList).catch(errorsHandle());

        if (rspStudentInfo) {
          // 获取学生默认年级
          payload.customerContact = rspStudentInfo.data.customerContact;
          yield put({
            type: 'getStudentGradeInContinuedCycle',
            payload: {
              studentId: payload.studentId,
              gradeId: rspStudentInfo.data.gradeId,
            },
          });

          // `customerContacts`学生所关联的所有家长的手机号
          payload.customerContacts = rspStudentInfo.data.customerContacts;
          payload.confirmCustomerContact =
            rspStudentInfo.data.confirmCustomerContact || rspStudentInfo.data.customerContact;
          payload.customerId = rspStudentInfo.data.customerId;
          payload.customerName = rspStudentInfo.data.customerName;
          payload.customerIdCardNo = rspStudentInfo.data.customerIdCardNo || null;
          payload.customerIdCardType = rspStudentInfo.data.customerIdCardType;
          payload._lockCertificateType = !payload.customerIdCardNo;
          payload._CertificateTypeDisabled = Boolean(payload.customerIdCardNo);
          payload._CertificateNumDisabled =
            payload.customerIdCardType !== CT_IDENTITY_CARD && Boolean(payload.customerIdCardNo);
          payload.studentId = rspStudentInfo.data.studentId;
          payload.studentName = rspStudentInfo.data.studentName;
          payload.gradeId = rspStudentInfo.data.gradeId;
          payload.schoolId = payload.schoolId || rspStudentInfo.data.schoolId;
          payload.schoolName = payload.schoolName || rspStudentInfo.data.schoolName;
          payload.studentBranchId = rspStudentInfo.data.studentBranchId;
          payload.studentCampusId = rspStudentInfo.data.studentCampusId;
          payload.studentType = rspStudentInfo.data.studentType;
          payload.disabledEditInfo = true;
          // 星火、培优从咨询师页面跳转过来的非正式学生
          if (flag && payload.studentType !== 'ENROLLED') {
            payload.canEditStdName = true;
          }
        }
        if (rspHasContract) {
          payload.disabledEditGrade = rspHasContract.data.hasNormalContract;
        }
      }

      yield put({
        type: 'updateState',
        payload,
      });

      if (payload.isEcs) {
        yield put({
          type: 'switchIsEcs',
          payload: { checked: true },
        });
      }

      runFunc(callback);
    },

    // 初始化合同类型
    *initContractType({ payload: { studentId, flag = false, contractType } }, { put }) {
      let payload = {};

      if (contractType) {
        payload = {
          contractType,
          contractTypeName: contractTypeCfg[contractType],
        };
      } else {
        const rsp = yield getNewSignContractType({ studentId, flag }, true).catch(errorsHandle());
        payload = {
          contractType: rsp.data,
          contractTypeName: contractTypeCfg[rsp.data],
        };
      }

      yield put({
        type: 'updateState',
        payload,
      });
    },

    // 获取学生默认年级(用于产品选择购物车) + 服务器时间戳
    *getStudentGradeInContinuedCycle({ payload: { studentId, gradeId } }, { put }) {
      const rsp = yield getStudentGradeInContinuedCycle({ studentId, gradeId }).catch(errorsHandle());
      if (rsp) {
        yield put({
          type: 'updateState',
          payload: {
            grade: rsp.data,
            // 公用数据
            timeStamp: rsp.timeStamp,
            timeText: moment(rsp.timeStamp).format('YYYY-MM-DD'),
          },
        });
      }
    },

    // 获取所有产品优惠
    *getProductPromotionDict(actions, { select, put }) {
      const productPromotionDict = yield select((state) => state[storeName].productPromotionDict);
      if (!productPromotionDict) {
        const organizationId = yield select((state) => state.account.user.brenchId);
        const rsp = yield getProductActivePromotionList({ organizationId }).catch(errorsHandle());
        let productPromotionDict = [];

        rsp.forEach((item) => {
          if (item.isActive === 'Y') {
            productPromotionDict.push({
              text: item.name,
              val: item.id,
              properties: takeOutPromotionProperties(item),
            });
          }
        });

        yield put({
          type: 'updateState',
          payload: {
            productPromotionDict,
          },
        });
      }
    },

    // 删除已选产品
    *deleteSelectedPrd({ payload: { record, groupItem, errHandle } }, { select, put }) {
      let selectedList = yield select((state) => clone(state[storeName].selectedList));
      const index = selectedList.findIndex((item) => item._uid === record._uid);

      if (index === -1) {
        return;
      }

      // 删除选中产品
      const [removeItem] = selectedList.splice(index, 1);
      // 删除其子产品
      if (removeItem && removeItem.subProducts && removeItem.subProducts.length > 0) {
        selectedList = selectedList.filter((item) => item._relation !== getPrdRelation(removeItem));
      }

      yield put({
        type: 'updateSelected',
        payload: { selectedList },
      });

      // 处理异步操作
      const newGroupItem = yield select((state) =>
        clone(state[storeName].groupList.find((item) => item._uid === groupItem._uid)),
      );

      if (newGroupItem && isNormalCanUseCoupon(newGroupItem)) {
        yield put({
          type: 'calcCoursePromotion',
          payload: {
            groupItem: newGroupItem,
            errHandle,
          },
        });
      }

      yield put({
        type: 'calcContractAmount',
        payload: { groupItem },
      });

      if (removeItem.contractProductId) {
        const isEdit = yield select((state) => state[storeName].isEdit);
        // 修改合同时，缓存已删的合同产品(只针对有班级id的小班、双师、讲座产品)
        if (isEdit && [SMALL_CLASS, TWO_TEACHER, LECTURE].includes(removeItem.productType) && removeItem.classId) {
          let removeContractPrdStorage = yield select((state) => clone(state[storeName].removeContractPrdStorage));
          removeContractPrdStorage[`${removeItem.productId}${removeItem.classId || ''}`] = removeItem.contractProductId;
          yield put({
            type: 'updateState',
            payload: { removeContractPrdStorage },
          });
        }
      }
    },

    // 改变产品数量
    *changePrdQuantity({ payload: { val, record, groupItem, quantityMinutes, callback } }, { select, put }) {
      let index = -1;
      const selectedList = yield select((state) => {
        let selectedList = clone(state[storeName].selectedList);
        index = selectedList.findIndex((item) => item._uid === record._uid);
        selectedList[index].quantity = val;
        selectedList[index].quantityMinutes = quantityMinutes;
        selectedList[index].payment = calcItemPayment(selectedList[index]);
        selectedList[index].realAmount = calcItemRealAmount(selectedList[index]);

        return selectedList;
      });

      yield put({
        type: 'updateSelected',
        payload: { selectedList },
      });

      const newGroupItem = yield select((state) =>
        state[storeName].groupList.find((item) => item._uid === groupItem._uid),
      );
      if (!newGroupItem) {
        return;
      }

      // 已选产品优惠的情况
      if (selectedList[index]._prdPromotionVal && selectedList[index]._prdPromotionVal.length > 0) {
        // 计算产品优惠放到回调执行(带防抖)
        runFunc(callback, {
          index,
          groupItem: newGroupItem,
          recordUid: record._uid,
          val: selectedList[index]._prdPromotionVal,
        });
      } else {
        // 能修改产品数量的类型内，只有常规1对1、1对多可用个性化抵扣券
        const noNeedCalcPromotion =
          useCouponProductTypes.indexOf(selectedList[index].productType) ===
          -1; /* || groupItem._isYinLiu 20220704 引流合同可以选择产品优惠券*/
        yield put({
          type: 'calcPrdNumAndPayment',
          payload: {
            noNeedCalcPromotion,
            groupItem: newGroupItem,
            recordUid: record._uid,
          },
        });
      }
    },

    // 改变产品优惠
    *changePrdPromotion({ payload: { val, record, groupItem } }, { select, put }) {
      let selectedList = yield select((state) => clone(state[storeName].selectedList));
      const index = selectedList.findIndex((item) => item._uid === record._uid);

      if (Array.isArray(val)) {
        // 存在产品优惠->调接口计算
        yield put({
          type: 'calcPrdPromotion',
          payload: { val, index, groupItem },
        });
      } else {
        // 不存在产品优惠->清空
        yield put({
          type: 'clearPrdPromotion',
          payload: { val, index, groupItem, recordUid: record._uid },
        });
      }
    },

    // 计算产品优惠(产品本身关联的优惠)
    *calcPrdPromotion({ payload: { val, index, groupItem } }, { select, put }) {
      yield put({ type: 'updateCalcLoading', payload: true });
      const isEdit = yield select((state) => state[storeName].isEdit);
      const tagProduct = yield select((state) => state[storeName].selectedList[index]);

      let apiParams = {
        promotionIds: val.join(','),
        price: tagProduct.price,
        pricePerHour: tagProduct.naturePrice,
        totalCourseCount: tagProduct.quantity,
        totalAmount: NP.times(tagProduct.price, tagProduct.quantity),
      };

      const rsp = yield calculatePrdPromotion(apiParams, true).catch(errorsHandle());

      if (rsp && rsp.promotionAmount && rsp.promotionVos && rsp.promotionVos.length) {
        const selectedList = yield select((state) => {
          let selectedList = clone(state[storeName].selectedList);
          const originPayment = NP.times(selectedList[index].price, selectedList[index].quantity);
          // 更新选择的优惠(计算接口报错时，不更新)
          selectedList[index]._prdPromotionVal = val;

          selectedList[index]._prdPromotion = rsp.promotionVos;
          selectedList[index]._prdPromotionAmount = rsp.promotionAmount;
          if (rsp.totalAmount > originPayment) {
            // 赠送课时的情况下，总额会增加
            selectedList[index]._paymentAddition = NP.minus(rsp.totalAmount, originPayment);
          } else {
            delete selectedList[index]._paymentAddition;
          }

          // 清空优惠活动
          removeActivePromotion(selectedList[index]);
          // 修改合同-重算优惠时删除`清除优惠`及`优惠变更提示`辅助值
          if (isEdit) {
            selectedList[index]._promotionAmount = 0;
            delete selectedList[index]._btnClearPromotionVisible;
          }

          selectedList[index].payment = calcItemPayment(selectedList[index]);
          selectedList[index].promotionAmount = calcItemPromotionAmount(selectedList[index]);
          selectedList[index].realAmount = calcItemRealAmount(selectedList[index]);

          return selectedList;
        });

        groupItem.data.forEach((course) => {
          const selectedIndex = selectedList.findIndex(
            (item) => item.productId === course.productId && item.classId === course.classId,
          );
          if (selectedIndex > -1) {
            delete selectedList[selectedIndex]._warningBeforeEditPrdPromotion;
          }
        });

        yield put({
          type: 'updateSelected',
          payload: { selectedList },
        });
      }

      yield put({ type: 'updateCalcLoading', payload: false });

      if (isEdit) {
        yield put({ type: 'calcContractAmount' });
      } else {
        const newGroupItem = yield select((state) =>
          state[storeName].groupList.find((item) => item._uid === groupItem._uid),
        );
        if (newGroupItem) {
          yield put({
            type: 'calcCoursePromotion',
            payload: {
              groupItem: newGroupItem,
            },
          });
        }
      }
    },

    // 清空产品优惠(产品本身关联的优惠)
    *clearPrdPromotion({ payload: { val, index, groupItem, recordUid } }, { select, put }) {
      const isEdit = yield select((state) => state[storeName].isEdit);
      const selectedList = yield select((state) => {
        let selectedList = clone(state[storeName].selectedList);

        removePrdPromotion(selectedList[index]);
        // 重算优惠时删除`清除优惠`及`优惠变更提示`辅助值
        if (isEdit) {
          selectedList[index]._promotionAmount = 0;
          delete selectedList[index]._btnClearPromotionVisible;
          delete selectedList[index]._warningBeforeEditPrdPromotion;
        }

        selectedList[index].payment = calcItemPayment(selectedList[index]);
        selectedList[index].promotionAmount = calcItemPromotionAmount(selectedList[index]);
        selectedList[index].realAmount = calcItemRealAmount(selectedList[index]);

        return selectedList;
      });

      yield put({
        type: 'updateSelected',
        payload: { selectedList },
      });

      if (isEdit) {
        yield put({ type: 'calcContractAmount' });
      } else {
        yield put({
          type: 'calcCoursePromotion',
          payload: {
            recordUid,
            groupItem,
          },
        });
      }
    },

    // 选择优惠券
    *selectCoupon({ payload: { list, groupItem } }, { select, put }) {
      let groupList = yield select((state) => clone(state[storeName].groupList));
      let productGroupItem = groupList.find((item) => item._uid === groupItem._uid);
      const isAutoCalc = productGroupItem.isAutoCalc;

      yield put({
        type: 'updateProductGroup',
        payload: {
          productGroupItem: {
            ...productGroupItem,
            couponList: list,
          },
        },
      });

      const newGroupItem = yield select((state) =>
        state[storeName].groupList.find((item) => item._uid === groupItem._uid),
      );
      if (newGroupItem) {
        // 手动优惠 ，没有优惠券
        if (list.length === 0) {
          yield put({
            type: 'clearCouponPromotion',
            payload: {
              groupItem: newGroupItem,
            },
          });
        } else if (isAutoCalc) {
          yield put({
            type: 'calcCoursePromotion',
            payload: {
              groupItem: newGroupItem,
            },
          });
        } else {
          yield put({
            type: 'calcCouponPromotion',
            payload: {
              groupItem: newGroupItem,
            },
          });
        }
      }
    },

    // 计算产品数量及产品总额
    *calcPrdNumAndPayment({ payload: { recordUid, groupItem, noNeedCalcPromotion = false } }, { select, put }) {
      const selectedList = yield select((state) => {
        let list = clone(state[storeName].selectedList);
        list.forEach((item) => {
          const quantity = item.quantity || item.totalCourseHours || null;
          item.quantity = quantity;

          // 如果产品类型为"其他"，并且费用类型为"市场"，则产品数量可编辑
          if (item.productType === OTHERS && item.otherCategoryId === OTHER_CATEGORY_TYPE_MARKETING) {
            item._quantityDisabled = false;
          } else {
            item._quantityDisabled = [SMALL_CLASS, OTHERS, ECS_CLASS].includes(item.productType);
          }

          item._hadInitQuantity =
            (typeof item._hadInitQuantity === 'number' && item._hadInitQuantity) || (Boolean(quantity) ? 1 : 0);
          item.payment = calcItemPayment(item);
          if (noNeedCalcPromotion) {
            item.realAmount = calcItemRealAmount(item);
          }
        });
        return list;
      });

      yield put({
        type: 'updateSelected',
        payload: { selectedList },
      });

      if (noNeedCalcPromotion) {
        yield put({
          type: 'calcContractAmount',
          payload: { groupItem },
        });
      } else {
        yield put({
          type: 'calcCoursePromotion',
          payload: {
            recordUid,
            groupItem,
          },
        });
      }
    },

    // 批量计算报读优惠
    *batchCalcCoursePromotion(actions, { select, put }) {
      const groupList = yield select((state) => state[storeName].groupList);
      for (let i = 0; i < groupList.length; i++) {
        yield put({
          type: 'groupHadToBeUseCoupon',
          payload: { groupItem: groupList[i] },
        });

        if (isNormalClassCourse(groupList[i])) {
          yield put({
            type: 'calcCoursePromotion',
            payload: { groupItem: groupList[i] },
          });
        } else if (isNormalOneToN(groupList[i])) {
          yield put({
            type: 'calcCouponPromotion',
            payload: { groupItem: groupList[i] },
          });
        }
      }
    },

    // 计算报读优惠(小班、双师的优惠活动、优惠券)
    *calcCoursePromotion({ payload: { recordUid, groupItem, errHandle } }, { select, put }) {
      const isEdit = yield select((state) => state[storeName].isEdit);
      if (isEdit) {
        yield put({
          type: 'calcContractAmount',
          payload: { groupItem, recordUid },
        });
      } else if (groupItem.isAutoCalc && isNormalClassCourse(groupItem)) {
        // 活动计算完如果存在已选优惠券则继续计算优惠券
        yield put({
          type: 'calcActivePromotion',
          payload: {
            groupItem,
            errHandle,
          },
        });
      } else if (groupItem.isAutoCalc && isNormalOneToN(groupItem)) {
        yield put({
          type: 'calcActivePromotion',
          payload: {
            recordUid,
            groupItem,
            errHandle,
          },
        });
      } else {
        yield put({
          type: 'calcCouponPromotion',
          payload: {
            groupItem,
            errHandle,
          },
        });
      }
    },

    /**
     * 计算优惠活动(优惠券)
     * 常规班课才有活动
     * */
    *calcActivePromotion({ payload: { recordUid, groupItem, errHandle } }, { select, put }) {
      if (
        ![ONE_ON_ONE_COURSE, ONE_ON_MANY, CLASS_COURSE].includes(groupItem.groupType) ||
        (isNormalOneToN(groupItem) && recordUid == null) /* || groupItem._isYinLiu*/
      ) {
        const hadCoupon = groupItem.couponList.length > 0;
        if (hadCoupon) {
          return yield put({
            type: 'calcCouponPromotion',
            payload: {
              groupItem: groupItem,
              errHandle,
            },
          });
        } else {
          return yield put({
            type: 'calcContractAmount',
            payload: {
              groupItem: groupItem,
            },
          });
        }
      }

      let selectedList = yield select((state) =>
        clone(state[storeName].selectedList).map((item) => {
          item.payment = calcItemPayment(item);
          item.paidAmount = calcItemPaidAmount(item);
          item.promotionAmount = calcItemPromotionAmount(item);
          item.realAmount = calcItemRealAmount(item);
          return item;
        }),
      );
      const contractType = yield select((state) => state[storeName].contractType);
      const hadCoupon = groupItem.couponList.length > 0;

      let courses = null;
      if (isNormalOneToN(groupItem)) {
        courses = [groupItem.data.find((item) => item._uid === recordUid)].map((item) => ({
          classId: item.productId,
          productId: item.productId,
          productType: item.productType,
          useCase: activeUseCaseCfg[contractType],
          price: yuanToCent(Number(item.price)),
          num: item.quantity,
        }));
      } else {
        courses = groupItem.data
          .filter((item) => item.productType === SMALL_CLASS && item.classId && !item._prdPromotion)
          .map((item) => ({
            classId: item.classId,
            productId: item.productId,
            productType: item.productType,
            useCase: activeUseCaseCfg[contractType],
            price: yuanToCent(Number(item.price)),
            // num与ruleUnit
            ...formatRuleNumAndRuleUnit(item),
          }));
      }

      // 自动时只判断有无产品，手动时还需判断有无优惠券
      if (courses.length > 0) {
        yield put({ type: 'updateCalcLoading', payload: true });

        let rsp = yield calculatePromotion(
          {
            courses,
            timestamp: Date.now(),
          },
          true,
        ).catch(errorsHandle());

        if (rsp && rsp.data) {
          const courses = rsp.data.courses.map((course) => ({
            ...course,
            // 优惠活动优惠金额
            _activeDiscount: centToYuan(course.activityDiscountAmount),
          }));

          let hadUseActivePromotion = false;

          courses.forEach((course) => {
            const selectedIndex = isNormalOneToN(groupItem)
              ? selectedList.findIndex((item) => item._uid === recordUid)
              : selectedList.findIndex(
                  (item) => item.productId === course.productId && item.classId === course.classId,
                );
            if (selectedIndex > -1) {
              const hadUsedActivePromotion = course.promotionActivitys.length > 0;
              /** 20220704 引流合同可以选择产品优惠券 */
              /*const selectProductItem = selectedList[selectedIndex];
              const isYinliu = selectProductItem.productCharacterName === yinliu;*/
              // 引流不能使用优惠活动
              const refreshData = () => {
                const item = selectedList[selectedIndex];
                item.payment = calcItemPayment(item);
                item.paidAmount = calcItemPaidAmount(item);
                item.promotionAmount = calcItemPromotionAmount(item);
                item.realAmount = calcItemRealAmount(item);
              };
              if (hadUsedActivePromotion /* && !isYinliu*/) {
                selectedList[selectedIndex]._activePromotion = course.promotionActivitys.map((promotion) => ({
                  id: promotion.id,
                  name: promotion.name,
                  rule: promotion.ruleId,
                }));
                if (isNormalOneToN(groupItem)) {
                  if (course.promotionHours) {
                    selectedList[selectedIndex]._paymentAddition = NP.times(
                      selectedList[selectedIndex].price,
                      course.promotionHours,
                    );
                  } else {
                    delete selectedList[selectedIndex]._paymentAddition;
                  }
                  refreshData();
                }
              } else {
                hadUseActivePromotion = false;
                selectedList[selectedIndex]._activePromotion = null;
                if (isNormalOneToN(groupItem)) {
                  delete selectedList[selectedIndex]._paymentAddition;
                  refreshData();
                }
              }
              // 缓存整块计算结果(优惠券、优惠活动分摊到产品里面的金额)
              selectedList[selectedIndex]._discountInfo = {
                ...(selectedList[selectedIndex]._discountInfo || {}),
                ...course,
              };

              selectedList[selectedIndex].promotionAmount = calcItemPromotionAmount(selectedList[selectedIndex]);
              selectedList[selectedIndex].realAmount = calcItemRealAmount(selectedList[selectedIndex]);
            }
          });

          groupItem.data.forEach((course) => {
            const selectedIndex = selectedList.findIndex(
              (item) => item.productId === course.productId && item.classId === course.classId,
            );
            if (selectedIndex > -1) {
              selectedList[selectedIndex]._warningBeforeEditPrdPromotion = hadUseActivePromotion;
            }
          });
        }

        if (!hadCoupon) {
          yield put({ type: 'updateCalcLoading', payload: false });
        }
      }

      yield put({
        type: 'updateSelected',
        payload: { selectedList },
      });

      const newGroupItem = yield select((state) =>
        clone(state[storeName].groupList.find((item) => item._uid === groupItem._uid)),
      );

      if (hadCoupon) {
        yield put({
          type: 'calcCouponPromotion',
          payload: {
            groupItem: newGroupItem || groupItem,
            errHandle,
          },
        });
      } else {
        yield put({
          type: 'calcContractAmount',
          payload: {
            groupItem: newGroupItem || groupItem,
          },
        });
      }
    },

    /**
     * 计算优惠券
     * 常规班课、常规个性化1对1可用券
     * */
    *calcCouponPromotion({ payload: { groupItem, errHandle } }, { select, put }) {
      /**20220704 引流合同可以选择产品优惠券 */
      if (useCouponGroupTypes.indexOf(groupItem.groupType) === -1 /* || groupItem._isYinLiu*/) {
        return yield put({
          type: 'calcContractAmount',
          payload: { groupItem },
        });
      }

      let groupList = yield select((state) => clone(state[storeName].groupList));
      let productGroupItem = groupList.find((item) => item._uid === groupItem._uid);

      let selectedList = yield select((state) =>
        clone(state[storeName].selectedList).map((item) => {
          item.payment = calcItemPayment(item);
          item.paidAmount = calcItemPaidAmount(item);
          item.promotionAmount = calcItemPromotionAmount(item);
          item.realAmount = calcItemRealAmount(item);
          return item;
        }),
      );

      let couponList = productGroupItem.couponList;

      const bossStudentId = yield select((state) => state[storeName].studentId);
      // 用于查询学生所有关联家长名下的券
      const contactList = yield select((state) => state[storeName].customerContacts);

      const branchId = yield select((state) => state[storeName].contractBranchId);

      let lastTimeCalResult = yield select((state) => state[storeName].lastTimeCalResult);

      let couponIds = couponList.map((item) => item.couponId);

      let courses = productGroupItem.data
        .filter((item) => {
          switch (item.productType) {
            case SMALL_CLASS:
              return !!item.classId;
            case ONE_ON_ONE_COURSE:
            case ONE_ON_MANY:
              return true;
            default:
              return false;
          }
        })
        .map((item) => {
          const ruleNum = formatRuleNumAndRuleUnit(item);

          return {
            classId: item.classId,
            productId: item.productId,
            productType: item.productType,
            price: yuanToCent(Number(item.price)),
            discount: yuanToCent(
              (item._discountInfo && item._discountInfo._activeDiscount) || item._prdPromotionAmount || 0,
            ),
            ...ruleNum,
          };
        });

      if (couponIds.length > 0 && courses.length > 0) {
        yield put({ type: 'updateCalcLoading', payload: true });

        // 抵扣券是否已不符合使用条件
        let deductionInvalid = false;

        let rsp = yield calculateCoupon(
          {
            couponIds,
            courses,
            bossStudentId,
            contactList,
            branchId,
            // 可叠加券与不可叠加券互相切换时，lastTimeCalResult会导致返回data为null，所以暂时不传
            lastTimeCalResult: null,
            type: 'COUPON_UPDATE',
            autoCoupon: false,
            autoPromotion: false,
            // 由于该接口后端无法把过期时间降低到1s内，无法满足当前需要。
            // 但这块限制只针对同参数请求，故在这里增加一个时间戳来保证每次请求参数都不相同
            timestamp: new Date().getTime(),
          },
          true,
        ).catch(
          errorsHandle((err) => {
            // 5000011: 抵扣券超出使用张数
            if (err.resultCode === 5000011) {
              deductionInvalid = true;
            }
          }),
        );

        if (rsp && rsp.resultCode === 0 && rsp.data) {
          lastTimeCalResult = rsp.data;

          if (Array.isArray(rsp.data.promotionSendCoupons) && rsp.data.promotionSendCoupons.length > 0) {
            // 缓存优惠券
            const rspCouponIdList = rsp.data.promotionSendCoupons.map((item) => item.id);
            // 过滤掉没有命中的优惠券
            couponList = couponList.filter((coupon) => rspCouponIdList.includes(`${coupon.couponId}`));

            rsp.data.promotionSendCoupons.forEach((item) => {
              let index = couponList.findIndex((coupon) => `${coupon.couponId}` === `${item.id}`);
              if (index > -1) {
                couponList[index]._couponId = item.id;
                couponList[index]._couponName = couponList[index].promotionName || item.name;
                couponList[index]._ruleId = item.ruleId;
                couponList[index]._couponDiscount = centToYuan(item.discount);
                // 已付金额(抵扣券)
                couponList[index]._couponAlreadyPayAmount = centToYuan(item.alreadyPayAmount);
                // 订单号
                couponList[index]._couponOrderNoInChannel = item.orderNoInChannel;
              }
            });
          } else {
            couponList = [];
          }

          courses = rsp.data.courses.map((course) => ({
            ...course,
            // 优惠券优惠金额
            _couponDiscount: centToYuan(course.couponDiscountAmount),
            // 产品所用券的已收金额
            _couponAlreadyPayAmount: centToYuan(
              (course.promotionSendCoupons || []).reduce(
                (total, current) => NP.plus(total, current.alreadyPayAmount),
                0,
              ),
            ),
            promotionSendCoupons: course.promotionSendCoupons.map((item) => {
              const index = couponList.findIndex((coupon) => coupon._couponId === item.id);
              if (index > -1) {
                item._productTypeName = productTypeNameCfg[course.productType];
                item._couponTypeName = getCouponTypeName(couponList[index]);
                item._couponPromotionAmount = centToYuan(item.discount);
              }
              return item;
            }),
          }));

          courses.forEach((course) => {
            const selectedIndex = selectedList.findIndex(
              (item) => item.productId === course.productId && item.classId === course.classId,
            );
            if (selectedIndex > -1) {
              // 缓存整块计算结果(优惠券、优惠活动分摊到产品里面的金额)
              selectedList[selectedIndex]._discountInfo = {
                ...(selectedList[selectedIndex]._discountInfo || {}),
                ...course,
              };
              selectedList[selectedIndex].paidAmount = calcItemPaidAmount(selectedList[selectedIndex]);
              selectedList[selectedIndex].promotionAmount = calcItemPromotionAmount(selectedList[selectedIndex]);
              selectedList[selectedIndex].realAmount = calcItemRealAmount(selectedList[selectedIndex]);
            }
          });
        } else {
          lastTimeCalResult = null;

          if (deductionInvalid) {
            // 过滤掉抵扣券
            couponList = couponList.filter((item) => item.promotionNature === 'COUPON_NORMAL');

            const newPrdGroupItem = {
              ...clone(productGroupItem),
              couponList,
              couponAmount: couponList.reduce((amount, current) => NP.plus(amount, current._couponDiscount), 0),
            };

            yield put({
              type: 'updateProductGroup',
              payload: {
                productGroupItem: newPrdGroupItem,
              },
            });

            const hasCoupon = newPrdGroupItem.couponList.length > 0;

            runFunc(errHandle, newPrdGroupItem, hasCoupon);
          }
        }

        yield put({ type: 'updateCalcLoading', payload: false });
      }

      yield put({
        type: 'updateSelected',
        payload: { selectedList },
      });

      yield put({
        type: 'updateState',
        payload: {
          lastTimeCalResult,
        },
      });

      let newGroupItem = yield select((state) =>
        state[storeName].groupList.find((item) => item._uid === groupItem._uid),
      );

      if (newGroupItem) {
        newGroupItem = {
          ...newGroupItem,
          couponList,
          couponAmount: couponList.reduce((amount, current) => NP.plus(amount, current._couponDiscount), 0),
        };

        yield put({
          type: 'updateProductGroup',
          payload: {
            productGroupItem: newGroupItem,
          },
        });
      }

      yield put({
        type: 'calcContractAmount',
        payload: { groupItem: newGroupItem },
      });
    },

    // (手动优惠)清空优惠券优惠金额
    *clearCouponPromotion({ payload: { groupItem } }, { select, put }) {
      const groupSelectedList = groupItem.data;

      const selectedList = yield select((state) => {
        let list = clone(state[storeName].selectedList);
        let isAutoCalc = groupItem.isAutoCalc;
        list.forEach((item) => {
          const selectItem = groupSelectedList.find((groupListItem) => groupListItem._uid === item._uid);
          if (!selectItem) return;

          if (isAutoCalc && item._discountInfo) {
            item._discountInfo._couponDiscount = 0;
            item._discountInfo._couponAlreadyPayAmount = 0;
          } else {
            delete item._activePromotion;
            delete item._discountInfo;
          }

          item.paidAmount = calcItemPaidAmount(item);
          item.promotionAmount = calcItemPromotionAmount(item);
          item.realAmount = calcItemRealAmount(item);
        });
        return list;
      });

      yield put({
        type: 'updateSelected',
        payload: { selectedList },
      });

      yield put({
        type: 'calcContractAmount',
        payload: { groupItem },
      });
    },

    // (手动优惠)清空优惠券优惠金额
    *clearSingleMathActivityCoupon({ payload: { groupItem, recordUid } }, { select, put }) {
      const selectedList = yield select((state) => {
        return clone(state[storeName].selectedList).map((item) => {
          if (
            !!groupItem?.data?.find((data) => data._uid === recordUid) &&
            item._uid === recordUid &&
            !!item._activePromotion
          ) {
            removeActivePromotion(item); // 移除活动优惠
            delete item._paymentAddition;
            item.promotionAmount = 0;
            item.payment = calcItemPayment(item);
            item.paidAmount = calcItemPaidAmount(item);
            item.realAmount = calcItemRealAmount(item);
          }
          return { ...item };
        });
      });
      yield put({
        type: 'updateSelected',
        payload: { selectedList },
      });
      yield put({ type: 'calcContractAmount' });
    },

    // 计算统计金额
    *calcContractAmount(actions, { select, put }) {
      if (actions?.payload?.groupItem && actions?.payload?.recordUid) {
        yield put({
          type: 'clearSingleMathActivityCoupon',
          payload: { groupItem: actions?.payload?.groupItem, recordUid: actions?.payload?.recordUid },
        });
      } else {
        const selectedList = yield select((state) => state[storeName].selectedList);
        const payload = calcGroupTotal(selectedList);
        yield put({
          type: 'updateState',
          payload,
        });
        if (actions && actions.payload && actions.payload.groupItem) {
          yield put({
            type: 'groupHadToBeUseCoupon',
            payload: { groupItem: actions.payload.groupItem },
          });
        }
      }
    },

    *updateSelectProduct({ payload }, { put }) {
      if (payload.type === 'ADD') {
        yield put({ type: 'cartAddProduct', payload });
      } else if (payload.type === 'DEL') {
        yield put({ type: 'cartDelProduct', payload });
      }
    },

    // 购物车添加产品
    *cartAddProduct({ payload: { data } }, { select, put }) {
      let selectedList = yield select((state) => clone(state[storeName].selectedList));
      let contractPayCategory = yield select((state) => state[storeName].contractPayCategory);

      /*
       * contractPayCategory 与产品的 contractPayCategory 一致时才可添加产品
       * 其他类型的产品可以任意 contractPayCategory 一起添加
       * */
      if (contractPayCategory && data.productType !== OTHERS) {
        if (contractPayCategory !== (data.contractPayCategory || CATEGORY_A)) {
          return showMessage('A类产品与B类产品不能同时购买', 'error');
        }
      } else if (data.productType !== OTHERS) {
        contractPayCategory = data.contractPayCategory || CATEGORY_A;
      }
      yield put({
        type: 'updateState',
        payload: {
          contractPayCategory,
        },
      });

      // 1对1、1对多(用于优惠券计算)
      data = addClassIdByUid(clone(data));

      let addList = addProductWithSub(data);
      selectedList = selectedList.concat(addList);

      yield put({
        type: 'updateSelected',
        payload: { selectedList },
      });

      // 遍历所有班课计算
      let groupList = yield select((state) => clone(state[storeName].groupList));
      for (let i = 0; i < groupList.length; i++) {
        yield put({
          type: 'calcPrdNumAndPayment',
          payload: {
            noNeedCalcPromotion: true,
            groupItem: groupList[i],
          },
        });
      }
    },

    // 购物车删除产品
    *cartDelProduct({ payload: { list } }, { select, put }) {
      yield put({
        type: 'updateSelected',
        payload: { selectedList: clone(list) },
      });

      /*
       * 当不存在非`其他`的产品时，需重置 contractPayCategory
       * */
      const isResetPayCategory = yield select(
        (state) => state[storeName].selectedList.filter((item) => item.productType !== OTHERS).length === 0,
      );

      if (isResetPayCategory) {
        yield put({
          type: 'updateState',
          payload: {
            contractPayCategory: null,
          },
        });
      }

      yield put({ type: 'calcContractAmount' });
    },

    // 获取老师字典
    *getTeachers({ payload: { callback } }, { select, put, call }) {
      const isNeedFetchTeacherDict = yield select((state) => {
        return !Array.isArray(state[storeName].teacherDict) || state[storeName].teacherDict.length === 0;
      });

      if (isNeedFetchTeacherDict) {
        const response = yield call(getCurrentBranchTeacher);

        yield put({
          type: 'updateState',
          payload: {
            teacherDict: response.teachers.map((item) => ({
              text: item.teacherName,
              val: item.teacherId,
              title: item.teacherName,
            })),
          },
        });
      }

      if (callback) callback();
    },

    // 切换精英班
    *switchIsEcs({ payload: { checked } }, { select, put }) {
      yield put({
        type: 'updateState',
        payload: { isEcs: checked },
      });
      yield put({ type: 'switchFuncAuth' });
    },

    *switchIsEcsClearSelected({ payload: { checked, callback } }, { select, put }) {
      yield put({
        type: 'updateState',
        payload: {
          isEcs: checked,
          selectedList: [],
          groupList: [],
          couponList: [],
        },
      });
      yield put({ type: 'switchFuncAuth' });
      yield put({ type: 'calcContractAmount' });
      showMessage(`${!checked ? '非' : ''}精英班切换成功`, 'success');
    },

    // 切换自动/手动计算优惠
    *switchAutoCalc({ payload: { groupItem, callback } }, { select, put }) {
      const isAutoCalc = !groupItem.isAutoCalc;

      let newGroupItem = yield select((state) =>
        state[storeName].groupList.find((item) => item._uid === groupItem._uid),
      );
      if (newGroupItem) {
        newGroupItem.isAutoCalc = isAutoCalc;
        yield put({
          type: 'updateProductGroup',
          payload: { productGroupItem: newGroupItem },
        });
      }

      // 手动时，清空已有优惠活动
      if (!isAutoCalc) {
        const groupSelectedList = groupItem.data;
        const selectedList = yield select((state) => {
          let selectedList = clone(state[storeName].selectedList);
          selectedList.forEach((item) => {
            const selectItem = groupSelectedList.find((groupListItem) => groupListItem._uid === item._uid);
            // 只清空优惠活动，不清空优惠券
            if (item._activePromotion && selectItem) {
              removeActivePromotion(item);
              item.promotionAmount = calcItemPromotionAmount(item);
              item.realAmount = calcItemRealAmount(item);
            }
          });
          return selectedList;
        });

        yield put({
          type: 'updateSelected',
          payload: { selectedList },
        });
      }

      newGroupItem = yield select((state) => state[storeName].groupList.find((item) => item._uid === groupItem._uid));

      runFunc(callback, newGroupItem);
    },

    // 校验合同
    *validateContract({ payload: { callback } }, { select }) {
      const validateResult = yield select((state) => {
        const _state = state[storeName];
        // 学生信息
        if (!_state.customerContact) {
          return '客户手机不能为空！';
        }
        if (!_state.customerName) {
          return '客户姓名不能为空！';
        }
        if (!_state.studentName) {
          return '学生姓名不能为空！';
        }
        if (!_state.gradeId) {
          return '年级不能为空！';
        }
        if (!_state.schoolName) {
          return '学校不能为空！';
        }
        if (!_state.blCampusId) {
          return '合同学习中心不能为空！';
        }

        let validIdentityData = validIdentityCardByType(_state.customerIdCardType, _state.customerIdCardNo);
        if (!validIdentityData.valid) {
          return '证件格式有误，请检查证件类型及号码';
        }

        // 产品信息
        if (_state.selectedList.length === 0) {
          return '请先选择报读课程产品！';
        }
        if (_state.selectedList.some((item) => !item.quantity)) {
          return '合同产品数量必须大于0并且不能为空！';
        }
        if (_state.selectedList.some((item) => item.realAmount < 0)) {
          return '单个合同产品的应付金额不可小于零！';
        }
        // 小班、精英班签单季度必填
        if (_state.groupList.some((groupItem) => isClass(groupItem) && !groupItem.quarterType)) {
          return _state.isEcs ? '精英班合同的签单季度不能为空！' : '班课合同的签单季度不能为空！';
        }
        // 收款信息
        if (_state.totalAmount <= 0) {
          return '合同总款必须大于0！';
        }

        return null;
      });

      if (validateResult) {
        showMessage(validateResult, 'error');
      }

      runFunc(callback, !validateResult);
    },

    // 提交合同
    *submit({ payload: { saveType, callback } }, { select, put }) {
      const rspTransactionUuid = yield request(`${location.origin}/eduboss/CommonAction/getTransactionUUID.do`, {
        method: 'GET',
      });
      const transactionUuid = rspTransactionUuid.data;
      yield put({
        type: 'updateState',
        payload: { transactionUuid },
      });

      // 缓存已使用的已删
      let _removeContractPrdStorage = yield select((state) => clone(state[storeName].removeContractPrdStorage));

      let contractId = yield select((state) => state[storeName].contractId) || null;

      let apiParams = yield select((state) => {
        const _state = state[storeName];

        let apiParams = {
          transactionUuid,
          contractType: _state.contractType,
          ecsContract: _state.isEcs,
          customerId: _state.customerId,
          customerName: _state.customerName,
          customerContact: _state.customerContact,
          studentId: _state.studentId,
          studentName: _state.studentName,
          blCampusId: _state.blCampusId,
          // 待审核学校的schoolId传空
          schoolId: _state.virtualSchool ? null : _state.schoolId,
          virtualSchool: _state.virtualSchool,
          contractGradeId: _state.gradeId,
          transferIntroduceId: _state.transferIntroduceId || null,
          contractVoList: [],
          customerIdCardType: _state.customerIdCardType,
          customerIdCardNo: _state.customerIdCardNo,
        };
        if (!_state.isEdit) {
          apiParams.signEntrance = _state.signEntrance;
        }

        apiParams.contractVoList = _state.groupList.map((group) => {
          let contractVo = {
            contractId,
            promotionCouponArr: [],
            contractProductArr: [],
            remark: group.remark,
            totalAmount: group.totalAmount || 0,
            promotionAmount: group.promotionAmount || 0,
            // 暂不用group.realAmount字段，realAmount`计划应收款` = `总额` - `优惠`
            realAmount: NP.minus(group.totalAmount, group.promotionAmount || 0) || 0,
            isAutoCalc: group.isAutoCalc,
          };
          if (isClass(group)) {
            contractVo.quarterType = group.quarterType || null;
          }

          /** 20220704 引流合同可以选择产品优惠券 */
          if (useCouponGroupTypes.includes(group.groupType) /* && !group._isYinLiu*/) {
            contractVo.promotionCouponArr = (group.couponList || []).map((item) => ({
              promotionCouponId: item._couponId,
              promotionCouponName: item._couponName,
              promotionCouponRuleId: item._ruleId,
              promotionCouponAmount: item._couponDiscount,
              alreadyPayAmount: item._couponAlreadyPayAmount,
              payOrderNum: item._couponOrderNoInChannel,
              promotionNature: item.promotionNature,
            }));
          }

          contractVo.contractProductArr = group.data.map((item) => {
            let contractProduct = {
              contractProductId: item.contractProductId || null,
              productType: item.productType,
              productId: item.productId,
              productVersionId: item.productVersionId || null,
              classId: item.classId || null,
              gradeId: item.gradeId || null,
              classTypeId: item.classTypeId,
              continueClassId: item.continueClassId || null,
              extendClassId: item.extendClassId || null,
              courseHours: item.quantity,
              subjectId: item.subjectId || null,
              startClassDate: item._courseDate || item.startClassDate,
              startClassTime: item.startClassTime,
              promotionArr: (item._prdPromotion || []).map((item) => ({
                promotionId: item.id,
                promotionName: item.name,
              })),
              // 自动计算优惠才需要传优惠活动
              promotionActivityArr:
                _state.isEdit || group.isAutoCalc
                  ? (item._activePromotion || []).map((item) => ({
                      promotionActivityName: item.name,
                      promotionActivityId: item.id,
                      promotionActivityRuleId: item.rule,
                    }))
                  : [],
              totalAmount: item.payment,
              promotionAmount: item.promotionAmount || 0,
              // 暂不用item.realAmount字段，后端的realPayAmount字段表示`计划应收款` = `总额` - `优惠`
              realPayAmount: NP.minus(item.payment, item.promotionAmount || 0) || 0,
              // 子产品需带父产品班级id(小班才会用到)
              relateClassId: item._relateClassId || null,
              relateProductType: item._relateProductType || null,
              // 合同产品状态是无效的才传(取消操作)
              status: item._contractPrdStatus === UNVALID ? UNVALID : null,
            };
            // 修改合同时匹配是否存在已删合同产品
            if (
              _state.isEdit &&
              _removeContractPrdStorage.hasOwnProperty(`${contractProduct.productId}${contractProduct.classId || ''}`)
            ) {
              contractProduct.contractProductId =
                _removeContractPrdStorage[`${contractProduct.productId}${contractProduct.classId || ''}`];
              // 缓存的已删合同产品只允许使用一次，防止产生重复的错误数据
              delete _removeContractPrdStorage[`${contractProduct.productId}${contractProduct.classId || ''}`];
            }
            return contractProduct;
          });

          return contractVo;
        });

        return apiParams;
      });

      let rsp;
      if (contractId) {
        // 修改
        rsp = yield batchEditContractInfo(apiParams, true).catch(errorsHandle());
      } else {
        // 新签
        rsp = yield batchSaveContractInfo(apiParams, true).catch(errorsHandle());
      }

      let contractIds = null;

      if (rsp) {
        contractIds = rsp.data;
        showMessage('合同保存成功！', 'success');
        yield put({
          type: 'updateState',
          payload: { removeContractPrdStorage: _removeContractPrdStorage },
        });
      }

      runFunc(callback, contractIds);
    },

    // (修改合同)清除优惠
    *clearPromotion({ payload: { record } }, { select, put }) {
      const selectedList = yield select((state) => {
        return clone(state[storeName].selectedList).map((tag) => {
          if (tag._uid === record._uid) {
            tag.promotionAmount = tag.promotionConsumeAmount || 0;
            tag._promotionAmount = tag.promotionAmount;
            tag.realAmount = NP.minus(tag.payment, tag.promotionAmount);
            tag._btnClearPromotionVisible = false;
            tag._warningBeforeEditPrdPromotion = false;
            removePrdPromotion(tag); // 移除产品优惠
            removeActivePromotion(tag); // 移除活动优惠
            delete tag._paymentAddition;
            tag.payment = calcItemPayment(tag);
            tag.paidAmount = calcItemPaidAmount(tag);
            tag.promotionAmount = calcItemPromotionAmount(tag);
            tag.realAmount = calcItemRealAmount(tag);
            return { ...tag };
          } else {
            return { ...tag };
          }
        });
      });
      yield put({
        type: 'updateSelected',
        payload: { selectedList },
      });
      yield put({ type: 'calcContractAmount' });
    },

    // 清除合同下所有产品的活动优惠
    *clearActivityCoupon({ payload: { groupItem } }, { select, put }) {
      const selectedList = yield select((state) => {
        return clone(state[storeName].selectedList).map((item) => {
          if (!!groupItem?.data?.find((data) => data._uid === item._uid) && !!item._activePromotion) {
            console.log(item);
            removeActivePromotion(item); // 移除活动优惠
            delete item._paymentAddition;
            item.payment = calcItemPayment(item);
            item.paidAmount = calcItemPaidAmount(item);
            item.promotionAmount = calcItemPromotionAmount(item);
            item.realAmount = calcItemRealAmount(item);
          }
          return { ...item };
        });
      });
      yield put({
        type: 'updateSelected',
        payload: { selectedList },
      });
      yield put({ type: 'calcContractAmount' });
    },

    // 无效/撤销无效 合同产品
    *switchContractPrdInvalid({ payload: { record } }, { select, put }) {
      let index = -1;

      let selectedList = yield select((state) => {
        let list = clone(state[storeName].selectedList);
        index = list.findIndex((item) => item._uid === record._uid);
        list[index]._contractPrdStatus = record._contractPrdStatus === UNVALID ? record._statusCache : UNVALID;
        return list;
      });

      yield put({
        type: 'updateSelected',
        payload: { selectedList },
      });

      yield put({ type: 'calcContractAmount' });
    },

    // 合同组是否存在待使用优惠券
    *groupHadToBeUseCoupon({ payload: { groupItem } }, { select, put }) {
      let couponToBeUsed = false;

      const isNeedUpdate = yield select(
        (state) =>
          !state[storeName].isEdit &&
          isNormalCanUseCoupon(groupItem) &&
          (state[storeName].couponList || []).length === 0 &&
          state[storeName].contractBranchId,
      );

      if (isNeedUpdate) {
        const apiParams = yield select((state) => {
          const _list = groupItem.data.filter(
            (item) => useCouponProductTypes.includes(item.productType) && item.quantity > 0,
          );
          const useCase = {
            [NEW_CONTRACT]: 'NEW',
            [RE_CONTRACT]: 'RE',
          }[state[storeName].contractType];

          const studentId = state[storeName].studentId;
          const contactList = state[storeName].customerContacts;
          const branchId = state[storeName].contractBranchId;
          const amount = _list.reduce((total, current) => NP.plus(total, current.realAmount), 0);
          const classInfos = formatClassInfoApiParam(_list, useCase);

          return {
            studentId,
            branchId,
            amount,
            classInfos,
            contactList,
            timestamp: new Date().getTime(),
          };
        });

        if (apiParams.amount > 0) {
          const res = yield getSuitablePromotion(apiParams).catch(errorsHandler);

          if (res && res.data) {
            couponToBeUsed = res.data.suitablePromotion.length > 0;
          }
        }
      }

      const newGroupItem = yield select((state) =>
        state[storeName].groupList.find((item) => item._uid === groupItem._uid),
      );
      if (newGroupItem) {
        newGroupItem.couponToBeUsed = couponToBeUsed;
        yield put({
          type: 'updateProductGroup',
          payload: {
            productGroupItem: newGroupItem,
          },
        });
      }
    },

    // 获取客户名下的定金券
    *fetchCustomerDeposit(action, { select, put }) {
      const apiParams = yield select((state) => ({ studentId: state[storeName].studentId }));
      const res = yield getDepositCouponsByStudent(apiParams).catch(errorsHandler);
      if (res && res.data) {
        yield put({
          type: 'updateState',
          payload: {
            depositList: res.data.depositList || [],
            hasDeposit: (res.data.depositList || []).length > 0,
          },
        });
      }
    },
  },
};
