import React from 'react';

import { Form } from 'antd';
import { Modal, TextArea } from 'COMPONENT';
import { errorsHandle } from 'SRC/utils/common';
import showMessage from 'SRC/components/utils/globalTips/showMessage';

import * as thirdPlatformAuditListApi from 'SERVICE/thirdPlatformAuditList';

const { Item: FormItem } = Form;

const formItemProps = {
  labelCol: { span: 6 },
  wrapperCol: { span: 16 },
};

class ConfirmMatchModal extends React.Component {
  state = {
    loading: false,
  };

  constructor(props) {
    super(props);
  }

  componentDidMount() {}

  handleConfirmMatch = () => {
    this.props.form.validateFields(async (err, values) => {
      if (err) {
        return;
      }
      this.setState({ loading: true });
      const result = await thirdPlatformAuditListApi
        .manualMatchThirdPayData({
          ...values,
          id: this.props.id,
        })
        .catch(errorsHandle());
      this.setState({ loading: false });
      if (!result || result.resultCode !== 0) {
        return;
      }
      this.props.onConfirmMatchSuccess();
      showMessage('人工匹配成功', 'success');
    });
  };

  generateForm() {
    return <Form>{this.generateRemarkFormItem()}</Form>;
  }

  generateRemarkFormItem() {
    const itemProps = {
      colon: false,
      label: '备注',
      key: 'remark',
      ...formItemProps,
    };
    const { getFieldDecorator } = this.props.form;
    return <FormItem {...itemProps}>{getFieldDecorator('remark')(<TextArea rows={3} maxLength={200} />)}</FormItem>;
  }

  generateModal() {
    const props = {
      visible: true,
      widthSize: 'm',
      okBtnText: '保存',
      loading: this.state.loading,
      onCancel: this.props.onCancel,
      onOk: this.handleConfirmMatch,
      title: (
        <div>
          确定将该数据更改为<span style={{ color: 'red' }}>人工匹配</span>状态
        </div>
      ),
    };

    return <Modal {...props}>{this.generateForm()}</Modal>;
  }

  render() {
    return this.generateModal();
  }
}

export default Form.create()(ConfirmMatchModal);
