import React from 'react';

import { Modal, Upload, Button } from 'COMPONENT';

import { errorsHandle } from 'SRC/utils/common';
import FileUtil from '../../../../../utils/file';
import showMessage from 'SRC/components/utils/globalTips/showMessage';
import * as thirdPlatformAuditListApi from 'SERVICE/thirdPlatformAuditList';

import { OSS_BASE_PREFIX } from 'CONST/ossConfig';

import styles from './style.module.less';

const EXCEL_URL = `${OSS_BASE_PREFIX}/template/%E7%AC%AC%E4%B8%89%E6%96%B9%E5%B9%B3%E5%8F%B0%E6%94%AF%E4%BB%98%E6%95%B0%E6%8D%AE%E5%AF%BC%E5%85%A5%E6%A8%A1%E6%9D%BF.xlsx`;

export default class ImportExcelModal extends React.Component {
  state = {
    fileList: [],
    loading: false,
  };

  constructor(props) {
    super(props);
  }

  handleSubmit = async () => {
    const { fileList } = this.state;
    if (!fileList.length) {
      showMessage('请选择要导入的文件', 'error');
      return;
    }
    this.setState({ loading: true });
    console.log(fileList);
    const { name, fileKey: url } = fileList[0];
    const result = await thirdPlatformAuditListApi.importThirdPayData({ url, name }).catch(errorsHandle());
    this.setState({ loading: false });
    if (!result || result.resultCode !== 0) {
      return;
    }
    showMessage('导入第三方平台支付数据成功', 'success');
    this.props.onImportSuccess();
  };

  handleUploadChange = (file) => {
    file.id = file.fileKey;
    this.setState({ fileList: [file] });
  };

  handleBeforeUpload = (fileList) => {
    if (fileList.filter((file) => !~['xls', 'xlsx'].indexOf(file.name.split('.').pop())).length) {
      showMessage('文件格式有误，请重新上传', 'error');
      return false;
    }
    fileList.forEach((file) => (file.fileKey += `.${file.name.split('.').pop()}`));
    return fileList;
  };

  handleReset = () => {
    this.setState({ fileList: [] });
  };

  handleDownloadTemplate = () => {
    FileUtil.downloadFile(EXCEL_URL);
  };

  generateModal() {
    const props = {
      visible: true,
      widthSize: 'l',
      okBtnText: '保存',
      onOk: this.handleSubmit,
      loading: this.state.loading,
      onCancel: this.props.onCancel,
      title: '第三方平台支付数据导入',
    };

    return (
      <Modal {...props}>
        {this.generateTemplateFormItem()}
        {this.generateUploadFormItem()}
      </Modal>
    );
  }

  generateTemplateFormItem() {
    return (
      <div className={styles.downloadTemplate}>
        <p className={styles.downloadTip}>
          <strong>第一步：</strong>下载第三方平台收款数据导入模板，批量填写数据并导入。
        </p>
        <p className={styles.downloadBtn}>
          <Button type="primary" onClick={this.handleDownloadTemplate}>
            下载模板
          </Button>
        </p>
      </div>
    );
  }

  generateUploadFormItem() {
    const uploadProps = {
      multiple: false,
      onChange: this.handleUploadChange,
      beforeUpload: this.handleBeforeUpload,
      limit: { type: 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet' },
    };
    const fileName = this.state.fileList.length ? this.state.fileList[0].name : null;
    return (
      <div className={styles.importExcelWrapper}>
        <p className={styles.uploadTip}>
          <strong>第二步：</strong>上传填写好的第三方平台支付数据表。
        </p>
        <div className={styles.uploadWrapper}>
          <div>
            <label className={styles.uploadLabel}>资源上传：</label>
            <span className={styles.uploadDesc}>{fileName || '仅支持Excel文件'}</span>
          </div>
          <div>
            <Upload {...uploadProps}>
              <Button type="primary" className={styles.viewFileBtn}>
                浏览
              </Button>
            </Upload>
          </div>
          <div className={styles.resetBtnWrapper}>
            <Button className={styles.resetBtn} onClick={this.handleReset}>
              重置
            </Button>
          </div>
        </div>
      </div>
    );
  }

  render() {
    return this.generateModal();
  }
}
