import React from 'react';
import { DatePicker } from 'antd';
import { Page, SearchBar, Input, MultiSelect, Select, List, Modal } from 'COMPONENT';
import styles from './index.less';
import { loadBranchCampusDict } from 'UTIL/dict/index';
import { getUserForPerformance } from 'SERVICE/contract';
import { message, Row, Col } from 'antd';
import { debounce } from 'lodash';
import moment from 'moment';
import {
  miniClassBackApprovalList,
  minClassStopAudit,
  minClassBackApprovalDetail,
  miniClassAprrovalList,
} from 'SERVICE/course';

const { RangePicker } = DatePicker;

const formateDate = (timeStamp) => {
  if (!timeStamp) return '';
  return moment(timeStamp).format('YYYY-MM-DD HH:mm:ss');
};

const initSearchParams = {
  oaInstanceId: '',
  studentName: '',
  className: '',
  applicantId: '',
  campusIds: [],
  auditStatus: '',
  timeRange: [moment().startOf('day'), moment().endOf('day')],
};

const approvalStatusList = [
  {
    val: 'AUDITING',
    text: '审批中',
  },
  {
    val: 'AUDIT_STOP',
    text: '已终止',
  },
  {
    val: 'AUDIT_COMPLETED',
    text: '已通过',
  },
  {
    val: 'AUDIT_REJECT',
    text: '不通过',
  },
];

const CancelStatus = {
  1: '回班审批',
  2: '回班成功',
  3: '回班失败',
  1: '回班审批',
  2: '回班成功',
  3: '回班失败',
};

export default class BackClassApprovalRecords extends React.Component {
  state = {
    loading: true,
    showDetailModal: false,
    modalLoading: false,
    searchParams: initSearchParams,
    // 发起人列表
    userLists: [],
    campusList: [],
    dataList: [],
    approvalDetail: {},
    approvalList: [],
    pagination: {
      total: 0,
      current: 1,
      pageSize: 20,
    },
  };

  columns = [
    { title: '审批编号', dataIndex: 'id', key: 'id', width: 150 },
    {
      title: 'OA编号',
      dataIndex: 'oaInstanceId',
      key: 'oaInstanceId',
      width: 150,
      render: (text) => text || '-',
    },
    { title: '学生姓名', dataIndex: 'studentName', key: 'stuName', width: 120 },
    { title: '所属分公司', dataIndex: 'branchName', key: 'branchName', width: 120 },
    { title: '学习中心名称', dataIndex: 'campusName', key: 'campusName', width: 120 },
    { title: '班级名称', dataIndex: 'className', key: 'className', width: 150 },
    { title: '年级', dataIndex: 'gradeName', key: 'gradeName', width: 120 },
    { title: '班级类型', dataIndex: 'classTypeName', key: 'classTypeName', width: 120 },
    { title: '班级层次', dataIndex: 'classLevelName', key: 'classLevelName', width: 120 },
    { title: '发起人', dataIndex: 'applicantName', key: 'applicantName', width: 120 },
    {
      title: '当前处理人',
      dataIndex: 'auditUserName',
      key: 'auditUserName',
      width: 120,
      render: (text, record) => {
        if (!record.oaInstanceId) {
          return '-';
        }
        return text;
      },
    },
    {
      title: '审批状态',
      dataIndex: 'auditStatus',
      key: 'auditStatus',
      width: 120,
      render: function (text, record) {
        if (!record.oaInstanceId) {
          return '-';
        }
        return {
          AUDIT_STOP: '已终止',
          AUDITING: '审核中',
          AUDIT_COMPLETED: '已通过',
          AUDIT_REJECT: '不通过',
        }[text];
      },
    },
    {
      title: '回班状态',
      dataIndex: 'cancelStatus',
      key: 'cancelStatus',
      width: 120,
      render: function (text) {
        return CancelStatus[text];
      },
    },
    {
      title: '预退时间',
      dataIndex: 'prepareDropTime',
      key: 'prepareDropTime',
      width: 150,
      render: function (text) {
        return formateDate(text);
      },
    },
    {
      title: '发起时间',
      dataIndex: 'applicantTime',
      key: 'applicantTime',
      width: 150,
      render: function (text) {
        return formateDate(text);
      },
    },
    {
      title: '操作',
      key: 'action',
      width: 150,
      getActionConfig: (text, record) => [
        {
          text: '详情',
          onClick: () => this.handleShowDetailModal(record.id),
        },
        {
          text: '终止审批',
          hide: record.auditStatus !== 'AUDITING',
          onClick: () => this.handleStopAudit(record.id),
        },
      ],
    },
  ];

  constructor(props) {
    super(props);
    this.loadBasicData = this.loadBasicData.bind(this);
    this.loadDataList = this.loadDataList.bind(this);
    this.handleSearch = this.handleSearch.bind(this);
    this.handleReset = this.handleReset.bind(this);
    this.handlePageChange = this.handlePageChange.bind(this);
    this.handleCloseModal = this.handleCloseModal.bind(this);
    this.handleShowDetailModal = this.handleShowDetailModal.bind(this);
    this.renderApprovalInfo = this.renderApprovalInfo.bind(this);
    this.handleStopAudit = this.handleStopAudit.bind(this);
    this.onSearchUser = debounce(this.handleSearchUser, 300).bind(this);
  }

  componentDidMount() {
    this.loadBasicData();
    this.loadDataList();
  }

  async loadBasicData() {
    try {
      const campusList = await loadBranchCampusDict();
      this.setState({ campusList });
    } catch (e) {
      message.warning('获取基础数据失败');
    }
  }

  async handleSearchUser(val) {
    if (val && val.length > 1) {
      try {
        const dict = await getUserForPerformance({ term: val });
        this.setState({
          userLists: dict.map((item) => ({ text: item.label, val: item.value, title: item.label })),
        });
      } catch (e) {}
    }
  }

  async loadDataList() {
    try {
      this.setState({ loading: true });
      const { pagination, searchParams } = this.state;
      const { timeRange, ...params } = searchParams;
      const time = {};
      if (timeRange) {
        time.applicantTimeStart = +new Date(timeRange[0]);
        time.applicantTimeEnd = +new Date(timeRange[1]);
      }
      const { data } = await miniClassBackApprovalList({
        pageNum: pagination.current,
        pageSize: pagination.pageSize,
        ...params,
        ...time,
      });
      const { list, total } = data;
      this.setState({ dataList: list, pagination: { ...pagination, total } });
    } finally {
      this.setState({ loading: false });
    }
  }

  handleSearchParamChange(key) {
    return function binder(e) {
      const { searchParams } = this.state;
      const update = {};
      update[key] = e ? (e.target ? e.target.value : e) : key === 'campusIds' ? [] : '';
      this.setState({ searchParams: { ...searchParams, ...update } });
    };
  }

  handleSearch() {
    this.loadDataList();
  }

  handleReset() {
    this.setState({ searchParams: initSearchParams }, () => this.loadDataList());
  }

  handlePageChange(pageParams) {
    const { pageSize, current } = pageParams;
    const { pagination } = this.state;
    this.setState({ pagination: { ...pagination, pageSize, current } }, () => {
      this.loadDataList();
    });
  }

  async handleShowDetailModal(flowId) {
    this.setState({ showDetailModal: true, modalLoading: true }, async () => {
      if (this.state.showDetailModal) {
        const { resultCode, data, resultMessage } = await minClassBackApprovalDetail({ flowId });
        // 获取审批流程列表
        const {
          resultCode: resultCode2,
          data: data2,
          resultMessage: resultMessage2,
        } = await miniClassAprrovalList({ flowId, type: 6 });
        if (resultCode === 0) {
          this.setState({ approvalDetail: data });
        } else {
          this.setState({ modalLoading: false });
          message.warning(resultMessage || '获取详情失败');
        }
        if (resultCode2 === 0) {
          this.setState({ approvalList: data2 });
        } else {
          message.warning(resultMessage2 || '获取OA审批记录失败');
        }
        this.setState({ modalLoading: false });
      }
    });
  }

  handleCloseModal() {
    this.setState({ showDetailModal: false, modalLoading: false });
  }

  // 终止审批
  async handleStopAudit(flowId) {
    Modal.confirm({
      title: '提示',
      content: '确定终止当前审批？',
      onOk: async () => {
        const { resultCode, resultMessage } = await minClassStopAudit({ flowId });
        if (resultCode === 0) {
          message.success('操作成功');
          this.loadDataList();
        } else {
          message.warning(resultMessage || '操作失败');
          return Promise.reject();
        }
      },
    });
  }

  renderApprovalInfo() {
    const columns = [
      { title: '处理人', dataIndex: 'userName', key: 'userName', width: 80 },
      { title: '岗位', dataIndex: 'roleName', key: 'roleName', width: 100 },
      { title: '审批节点', dataIndex: 'title', key: 'title', width: 100 },
      { title: '操作类型', dataIndex: 'operateStatusName', key: 'operateStatusName', width: 80 },
      {
        title: '操作时间',
        dataIndex: 'operateTime',
        key: 'operateTime',
        width: 150,
        render: function (text) {
          return formateDate(text);
        },
      },
      { title: '批注', dataIndex: 'notes', key: 'notes', width: 80 },
    ];
    return (
      <div className={styles.back_class_approval_info}>
        <p className={styles.back_class_info_title}>OA审批记录</p>
        <List dataSource={this.state.approvalList} columns={columns} />
      </div>
    );
  }

  render() {
    const {
      loading,
      searchParams,
      campusList,
      dataList,
      pagination,
      showDetailModal,
      modalLoading,
      userLists,
      approvalDetail,
    } = this.state;

    return (
      <Page
        name="预退班回班审批记录"
        fitWindowHeight
        loadingTip="正在加载数据"
        route={this.props.route}
        loading={loading}
        subHeader={
          <SearchBar onSearch={this.handleSearch} onReset={this.handleReset}>
            <Input
              value={searchParams.oaInstanceId}
              placeholder="OA编号"
              allowClear
              maxLength={50}
              onChange={this.handleSearchParamChange('oaInstanceId').bind(this)}
            />
            <Input
              value={searchParams.studentName}
              placeholder="学生姓名"
              allowClear
              maxLength={50}
              onChange={this.handleSearchParamChange('studentName').bind(this)}
            />
            <Input
              value={searchParams.className}
              placeholder="班级名称"
              allowClear
              maxLength={50}
              onChange={this.handleSearchParamChange('className').bind(this)}
            />
            <Select
              value={searchParams.applicantId}
              placeholder="发起人"
              allowClear
              dict={userLists}
              showSearch={true}
              searchPlaceholder="输入名字搜索"
              onSearchKeyChange={this.onSearchUser}
              onChange={this.handleSearchParamChange('applicantId').bind(this)}
            />
            <MultiSelect
              value={searchParams.campusIds}
              placeholder="请选择学习中心"
              showSearch={true}
              noOptionTips="暂无数据"
              dict={campusList}
              onChange={this.handleSearchParamChange('campusIds').bind(this)}
            />
            <Select
              value={searchParams.auditStatus}
              placeholder="请选择审批状态"
              showSearch={true}
              noOptionTips="暂无数据"
              dict={approvalStatusList}
              onChange={this.handleSearchParamChange('auditStatus').bind(this)}
            />
            <RangePicker
              value={searchParams.timeRange}
              showTime
              format="YYYY-MM-DD HH:mm:ss"
              placeholder="选择时间"
              size="large"
              allowClear
              onChange={this.handleSearchParamChange('timeRange').bind(this)}
            />
          </SearchBar>
        }
      >
        <List dataSource={dataList} columns={this.columns} pagination={pagination} onChange={this.handlePageChange} />
        {showDetailModal && (
          <Modal
            title="预退回班审批详情"
            maskClosable={false}
            loading={modalLoading}
            footer={false}
            className={styles.back_class_modal}
            onClose={this.handleCloseModal}
          >
            <div className={styles.back_class_modal}>
              {/** 基础信息 */}
              <div className={styles.back_class_basic_info}>
                <Row className={styles.back_class_info_row} justify="space-around">
                  <Col span="8">
                    <label className={styles.back_class_info_label}>审批编号：</label>
                    <span className={styles.back_class_info_text}>{approvalDetail.id}</span>
                  </Col>
                  <Col span="8">
                    <label className={styles.back_class_info_label}>OA编号：</label>
                    <span className={styles.back_class_info_text}>{approvalDetail.oaInstanceId}</span>
                  </Col>
                  <Col span="8">
                    <label className={styles.back_class_info_label}>发起时间：</label>
                    <span className={styles.back_class_info_text}>{formateDate(approvalDetail.applicantTime)}</span>
                  </Col>
                </Row>
                <Row className={styles.back_class_info_row} justify="space-around">
                  <Col span="8">
                    <label className={styles.back_class_info_label}>学生姓名：</label>
                    <span className={styles.back_class_info_text}>{approvalDetail.studentName}</span>
                  </Col>
                  <Col span="8">
                    <label className={styles.back_class_info_label}>发起人：</label>
                    <span className={styles.back_class_info_text}>{approvalDetail.applicantName}</span>
                  </Col>
                  <Col span="8">
                    <label className={styles.back_class_info_label}>预退时间：</label>
                    <span className={styles.back_class_info_text}>{formateDate(approvalDetail.prepareDropTime)}</span>
                  </Col>
                </Row>
              </div>
              {/** 班级信息 */}
              <div className={styles.back_class_class_info}>
                <p className={styles.back_class_info_title}>班级信息</p>
                <Row className={styles.back_class_info_row} justify="space-around">
                  <Col span="7" className={styles.back_class_info_item}>
                    <label className={styles.back_class_info_label}>学习中心名称：</label>
                    <span className={styles.back_class_info_text}>{approvalDetail.campusName}</span>
                  </Col>
                  <Col span="7" className={styles.back_class_info_item}>
                    <label className={styles.back_class_info_label}>产品名称：</label>
                    <span title={approvalDetail.produceName} className={styles.back_class_info_text}>
                      {approvalDetail.produceName}
                    </span>
                  </Col>
                  <Col span="10" className={styles.back_class_info_item}>
                    <label className={styles.back_class_info_label}>班级名称：</label>
                    <span title={approvalDetail.className} className={styles.back_class_info_text}>
                      {approvalDetail.className}
                    </span>
                  </Col>
                </Row>
                <Row className={styles.back_class_info_row} justify="space-around">
                  <Col span="7" className={styles.back_class_info_item}>
                    <label className={styles.back_class_info_label}>所属分公司：</label>
                    <span className={styles.back_class_info_text}>{approvalDetail.branchName}</span>
                  </Col>
                  <Col span="7" className={styles.back_class_info_item}>
                    <label className={styles.back_class_info_label}>班级类型：</label>
                    <span className={styles.back_class_info_text}>{approvalDetail.classTypeName}</span>
                  </Col>
                  <Col span="10" className={styles.back_class_info_item}>
                    <label className={styles.back_class_info_label}>班级层次：</label>
                    <span className={styles.back_class_info_text}>{approvalDetail.classLevelName}</span>
                  </Col>
                </Row>
                <Row className={styles.row}>
                  <Col span="7" className={styles.back_class_info_item}>
                    <label className={styles.back_class_info_label}>学生剩余课时：</label>
                    <span className={styles.back_class_info_text}>{approvalDetail.remainCourseHours}</span>
                  </Col>
                  <Col span="7" className={styles.back_class_info_item}>
                    <label className={styles.back_class_info_label}>年级：</label>
                    <span className={styles.back_class_info_text}>{approvalDetail.gradeName}</span>
                  </Col>
                </Row>
              </div>
              {/** 审批记录 */}
              <div className={styles.back_class_approval_info}>{this.renderApprovalInfo()}</div>
            </div>
          </Modal>
        )}
      </Page>
    );
  }
}
