import React from 'react';
import Page from 'COMPONENT/Page';
import List from 'COMPONENT/List';
import Loaders from './loaders';

class BatchBuildClassRecords extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      dataSource: [],
      current: 1,
      pageSize: 10,
      total: 0,
    };

    this.columns = [
      { title: '导入时间', dataIndex: 'createTime' },
      { title: '操作人', dataIndex: 'createUserName' },
      { title: '导入总条数', dataIndex: 'total' },
      { title: '成功条数', dataIndex: 'sucess' },
      { title: '失败条数', dataIndex: 'fail' },
      {
        title: '导入文件下载',
        dataIndex: 'sourceUrl',
        render: (sourceUrl, row) => {
          return (
            <a
              href={BatchBuildClassRecords.fileUrlAdapter(sourceUrl, row.sourceName)}
              download={row.sourceName}
              style={{ color: ' #6D67FF', cursor: 'pointer' }}
            >
              {`${row.sourceName}(点击下载)`}
            </a>
          );
        },
      },
      {
        title: '导入结果下载',
        dataIndex: 'resultUrl',
        render: (resultUrl, row) => {
          if (row.importResult === 0) {
            return <p>导入失败</p>;
          } else if (row.importStatus === 1) {
            return <p>导入中......</p>;
          } else if (row.importStatus === 0) {
            return <p>待导入</p>;
          }
          return (
            <a
              href={BatchBuildClassRecords.fileUrlAdapter(resultUrl, row.resultName)}
              download={row.resultName}
              style={{ color: ' #6D67FF', cursor: 'pointer' }}
            >
              {`${row.resultName}(点击下载)`}
            </a>
          );
        },
      },
    ];
  }

  static fileUrlAdapter(fileUrl, fileName) {
    return `${fileUrl.replace(fileName, '')}${encodeURIComponent(fileName)}`;
  }

  componentDidMount() {
    this.loadData();
  }

  loadData = async () => {
    const { current: pageNum, pageSize } = this.state;
    this.setState({ loading: true });
    try {
      const { dataSource, total } = await Loaders.getListRecords({ pageNum, pageSize, category: 'BATCH_ADD_CLASS' });
      this.setState({ dataSource, total });
    } catch (error) {
      this.setState({ loading: false });
    }
    this.setState({ loading: false });
  };

  render() {
    const { loading, dataSource, current, pageSize, total } = this.state;
    return (
      <Page loading={loading} title="小班批量导入记录">
        <List
          dataSource={dataSource}
          columns={this.columns}
          pagination={{
            current,
            pageSize,
            total,
          }}
          onChange={({ current, pageSize }) => {
            this.setState({ current, pageSize }, () => {
              this.loadData();
            });
          }}
        />
      </Page>
    );
  }
}

export default BatchBuildClassRecords;
