import React from 'react';
import { connect } from 'dva';
import { Row, Col, Switch } from 'antd';
import { Block, FormItem, Input, Select } from 'COMPONENT';
import { getCourseModalList } from 'SERVICE/course';
import { getSmallClassTeacher } from 'SERVICE/common';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

const formItemLayout = {
  labelWidth: '120px',
  style: {
    padding: 0,
    marginBottom: '20px',
  },
};
const unStartStatus = 'PENDING_START';

const defaultSmallInfo = () => {
  return {
    productName: '',
    gradeName: '',
    subjectName: '',
    phaseName: '',
    productVersionName: '',
    personValue: '',
    peopleQuantity: '',
    classTime: '',
    startDate: '',
    totalClassHours: '',
    classTimeLength: '',
    classLevelName: '',
    branchId: '',
  };
};

// 格式化老师数据，按学习中心或分公司分组进行划分
function formatTeacherArray(arr, result = []) {
  const copyArr = JSON.parse(JSON.stringify(arr));

  copyArr.forEach((item) => {
    if (item.teachers && item.teachers.length > 0) {
      let teacherOptions = item.teachers.map((t) => {
        t.text = t.userName;
        t.val = t.userId;
        return t;
      });
      result.push({
        label: item.orgName,
        val: item.orgId,
        options: teacherOptions,
      });
    }

    if (item.children && item.children.length > 0) {
      formatTeacherArray(item.children, result);
    }
  });

  return result;
}

class Form extends React.Component {
  constructor(props) {
    super(props);
    const defaultCompany = props.COMPANY && props.COMPANY.length ? props.COMPANY[0].val : '';

    this.state = {
      form: {
        name: '',
        courseModalId: '',
        teacherId: '',
        needDeliveryInfo: false,
      },
      company: defaultCompany,
      smallClassInfo: defaultSmallInfo(),
      courseTemplate: [],
      teacherList: [],
    };

    if (defaultCompany) {
      props.onBranchChange && props.onBranchChange(defaultCompany);
    }
  }

  componentDidMount() {
    this.getSelectableTeacher(this.props.branchId || this.state.company);
  }

  componentWillReceiveProps(nextProps) {
    const { selectedSmallClass, formValue, branchId } = nextProps;
    const { onBranchChange } = this.props;

    // 根据选中小班数据来展示大班数据
    if (selectedSmallClass != this.props.selectedSmallClass) {
      if (selectedSmallClass.length) {
        let smallClassData = selectedSmallClass[0];
        let totalPeopleData = selectedSmallClass.reduce(
          (pre, cur) => {
            pre.current += cur.miniClassPeopleNum;
            pre.total += cur.peopleQuantity;

            return pre;
          },
          { current: 0, total: 0 },
        );
        let personValue = `${totalPeopleData.current}/${totalPeopleData.total}`;

        this.setState(
          {
            smallClassInfo: { ...smallClassData, personValue },
          },
          () => {
            let techNum = smallClassData.totalClassHours / smallClassData.everyCourseClassNum;

            if (techNum != Math.floor(techNum)) {
              showMessage('课程讲次不为整数,无法选择模板');
            } else {
              this.getCourseTemplate({
                branchId: branchId || this.state.company,
                productYearId: smallClassData.productVersionId,
                productSeasonId: smallClassData.productQuarterId,
                coursePhaseId: smallClassData.phaseId,
                techNum: techNum,
              });
            }
          },
        );
      } else {
        this.setState({
          smallClassInfo: defaultSmallInfo(),
          courseTemplate: [],
        });
      }
    }

    // 已保存表单值回显
    if (formValue && formValue != this.props.formValue) {
      this.setState(
        {
          form: {
            name: formValue.className,
            courseModalId: String(formValue.courseModalId),
            teacherId: formValue.teacherId,
            needDeliveryInfo: Boolean(formValue.needDeliveryInfo),
          },
          company: formValue.blBranchId,
        },
        () => {
          onBranchChange && onBranchChange(formValue.blBranchId);
        },
      );
    }
  }

  // 获取小班老师
  getSelectableTeacher = (branchId) => {
    getSmallClassTeacher({ brenchId: branchId }).then((res) => {
      let teacherList = res || [];
      let campusMap = teacherList.reduce((pre, cur) => {
        if (!pre[cur.campusName]) {
          pre[cur.campusName] = {
            label: cur.campusName,
            val: cur.campusId,
            options: [],
          };
        }
        pre[cur.campusName].options.push({
          text: cur.name,
          val: cur.userId,
        });

        return pre;
      }, {});

      let teacherGroup = Object.keys(campusMap).map((k) => campusMap[k]);

      this.setState({
        teacherList: teacherGroup,
      });
    });
  };

  // 根据年份及分公司获取课程模板
  getCourseTemplate = (params) => {
    getCourseModalList(params).then((res) => {
      let data = res || [];
      data = data.map((t) => {
        return {
          text: t.modalName,
          val: String(t.id),
        };
      });
      this.setState({
        courseTemplate: data,
      });
    });
  };

  getFormValue = () => {
    return {
      ...this.state.form,
      needDeliveryInfo: Number(this.state.form.needDeliveryInfo),
    };
  };

  // 设置单个表单值
  setFormValue = (key, value) => {
    this.setState({
      form: {
        ...this.state.form,
        [key]: value,
      },
    });
  };

  render() {
    const { COMPANY, isDetail, formValue, selectedSmallClass } = this.props;
    const classStarted = formValue && formValue.status !== unStartStatus;
    const hasSmallClass = selectedSmallClass && selectedSmallClass.length;
    const { courseTemplate, teacherList, company } = this.state;
    const { name, teacherId, courseModalId, needDeliveryInfo } = this.state.form;

    const {
      productName,
      gradeName,
      subjectName,
      phaseName,
      productVersionName,
      classStartTime,
      classEndTime,
      startDate,
      totalClassHours,
      classTimeLength,
      classLevelName,
      personValue,
    } = this.state.smallClassInfo;

    const onClassTime = classStartTime ? `${classStartTime} - ${classEndTime}` : '';

    return (
      <Block>
        <Row gutter={20}>
          <Col span={6}>
            <FormItem {...formItemLayout} name="本地网课名称：">
              <Input
                value={name}
                placeholder="请输入本地网课名称"
                disabled={isDetail || classStarted}
                onChange={(e) => this.setFormValue('name', e.target.value)}
              />
            </FormItem>
          </Col>
          <Col span={6}>
            <FormItem {...formItemLayout} name="产品名称：">
              <Input value={productName} disabled />
            </FormItem>
          </Col>
          <Col span={6}>
            <FormItem {...formItemLayout} name="产品年份：">
              <Input value={productVersionName} disabled />
            </FormItem>
          </Col>
          <Col span={6}>
            <FormItem {...formItemLayout} name="年级：">
              <Input value={gradeName} disabled />
            </FormItem>
          </Col>
        </Row>
        <Row gutter={20}>
          <Col span={6}>
            <FormItem {...formItemLayout} name="科目：">
              <Input value={subjectName} disabled />
            </FormItem>
          </Col>
          <Col span={6}>
            <FormItem {...formItemLayout} name="期数：">
              <Input value={phaseName} disabled />
            </FormItem>
          </Col>
          <Col span={6}>
            <FormItem {...formItemLayout} name="上课日期：">
              <Input value={startDate} disabled />
            </FormItem>
          </Col>
          <Col span={6}>
            <FormItem {...formItemLayout} name="上课时间：">
              <Input value={onClassTime} disabled />
            </FormItem>
          </Col>
        </Row>
        <Row gutter={20}>
          <Col span={6}>
            <FormItem {...formItemLayout} name="总课时数：">
              <Input value={totalClassHours} disabled />
            </FormItem>
          </Col>
          <Col span={6}>
            <FormItem {...formItemLayout} name="课时时长：">
              <Input value={classTimeLength} disabled />
            </FormItem>
          </Col>
          <Col span={6}>
            <FormItem {...formItemLayout} name="所属分公司">
              <Select
                showSearch
                dict={COMPANY}
                value={company}
                allowClear={false}
                disabled={!!this.props.formValue}
                onChange={(value) => {
                  this.setState({ company: value });
                  this.getSelectableTeacher(value);
                  this.props.onBranchChange(value);
                }}
              />
            </FormItem>
          </Col>
          <Col span={6}>
            <FormItem {...formItemLayout} name="班级层次">
              <Input value={classLevelName} disabled />
            </FormItem>
          </Col>
        </Row>
        <Row gutter={20}>
          <Col span={6}>
            <FormItem {...formItemLayout} name="招生人数：">
              <Input value={personValue} disabled />
            </FormItem>
          </Col>
          <Col span={6}>
            <FormItem {...formItemLayout} name="主讲老师：">
              <Select
                showSearch
                dict={teacherList}
                value={teacherId}
                disabled={isDetail || classStarted}
                onChange={(val) => this.setFormValue('teacherId', val)}
              />
            </FormItem>
          </Col>
          <Col span={6}>
            <FormItem {...formItemLayout} name="课程模板">
              <Select
                dict={courseTemplate}
                value={courseModalId}
                disabled={!!this.props.formValue}
                noOptionTips={hasSmallClass ? '所选小班无匹配课程模板' : '关联小班后才能选择课程模板'}
                searchNotFoundTips={hasSmallClass ? '所选小班无匹配课程模板' : '关联小班后才能选择课程模板'}
                onChange={(val) => this.setFormValue('courseModalId', val)}
              />
            </FormItem>
          </Col>
          <Col span={6}>
            <FormItem {...formItemLayout} name="强制填写收件信息">
              <Switch
                checkedChildren="是"
                checked={needDeliveryInfo}
                disabled={isDetail || classStarted}
                defaultChecked={false}
                unCheckedChildren="否"
                onChange={(val) => this.setFormValue('needDeliveryInfo', val)}
              />
            </FormItem>
          </Col>
        </Row>
      </Block>
    );
  }
}

export default connect(
  (state) => {
    return {
      COMPANY: state.dict['branch'],
    };
  },
  null,
  null,
  { withRef: true },
)(Form);
