import React from 'react';
import NP from 'number-precision';
import Block from 'COMPONENT/Block';
import Modal from 'COMPONENT/Modal';
import { Row, Col } from 'antd';
import Select from 'COMPONENT/Select';
import TimePicker from 'COMPONENT/TimePicker';
import Calendar from './Cal';
import moment from 'moment';
import styles from './index.less';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import InputNumber from 'COMPONENT/InputNumber';
import { getSmallClassTeacher } from 'SERVICE/common';
import { getBisClassDetail } from 'SERVICE/bigClass';

class AddNewCourse extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      teacherDict: null,
      date: [],
      courses: [], // 课表
      courseTime: [null], // 开始时间, 默认有一个
      classDetail: {
        notArrangeCourseHours: 0,
      },
    };

    this.init = this.init.bind(this);
    this.updateCurriculum = this.updateCurriculum.bind(this);
    this.updateTeacherId = this.updateTeacherId.bind(this);
    this.pushCourses = this.pushCourses.bind(this);
    this.spliceCourses = this.spliceCourses.bind(this);
  }

  componentDidMount() {
    this.init();
  }

  /**
   * 设置时间-更新已选日期
   * */
  updateDate = (newDate) => {
    let { date } = this.state;
    if (date.length === 0) {
      date.push(newDate);
    } else {
      for (let i = 0; i < date.length; i++) {
        if (moment(newDate).isBefore(date[i]) || moment(newDate).isSame(date[i])) {
          date.splice(i, 0, newDate);
          break;
        } else if (moment(newDate).isAfter(date[date.length - 1])) {
          date.push(newDate);
          break;
        }
      }
    }
    this.setState({ date });
    this.updateCurriculum(newDate);
  };

  pushCourses(courses, date, time, courseTime) {
    const { teacherId } = this.state;
    for (let i = 0; i < time; i++) {
      courses.push({
        courseDate: date,
        teacherId,
        courseTime: courseTime[i],
      });
    }
  }

  spliceCourses(courses, date, time, courseTime, index) {
    const { teacherId } = this.state;
    for (let i = 0; i < time; i++) {
      courses.splice(index, 0, {
        courseDate: date,
        teacherId,
        courseTime: courseTime[i],
      });
    }
  }

  /**
   * 更新课表
   */
  updateCurriculum(date) {
    let { courseTime, courses } = this.state;
    if (courses.length === 0) {
      this.pushCourses(courses, date, courseTime.length, courseTime);
    } else {
      for (let i = 0; i < courses.length; i++) {
        if (moment(date).isBefore(courses[i].courseDate) || moment(date).isSame(courses[i].courseDate)) {
          this.spliceCourses(courses, date, courseTime.length, courseTime, i);
          break;
        } else if (moment(date).isAfter(courses[courses.length - 1].courseDate)) {
          this.pushCourses(courses, date, courseTime.length, courseTime);
          break;
        }
      }
    }
    this.setState({ courses });
  }

  // 获取小班老师
  getSelectableTeacher = (brenchId) => {
    return getSmallClassTeacher({ brenchId }).then((res) => {
      let teacherList = res || [];
      let campusMap = teacherList.reduce((pre, cur) => {
        if (!pre[cur.campusName]) {
          pre[cur.campusName] = {
            label: cur.campusName,
            val: cur.campusId,
            options: [],
          };
        }
        pre[cur.campusName].options.push({
          text: cur.name,
          val: cur.userId,
        });

        return pre;
      }, {});

      let teacherGroup = Object.keys(campusMap).map((k) => campusMap[k]);

      this.setState({
        teacherDict: teacherGroup,
      });
    });
  };

  // 获取班级信息
  getClassInfo = (classId) => {
    return getBisClassDetail({ onlineClassId: classId }).then((res) => {
      let classDetail = res.data || {};
      this.setState({
        classDetail,
        teacherId: classDetail.teacherId,
      });
    });
  };

  async init() {
    const { branchId, classId } = this.props;

    this.setState({ loading: true });
    try {
      await this.getSelectableTeacher(branchId);
      await this.getClassInfo(classId);
    } catch (e) {
      this.setState({ loading: false });
    }
    this.setState({ loading: false });
  }

  // 更新老师id
  updateTeacherId(teacherId) {
    const { teacherId: oldTeacherId } = this.state;
    const { courses } = this.state;
    this.setState({
      teacherId,
    });
    let newCourses = courses.map((course) => {
      if (oldTeacherId === course.teacherId) {
        return Object.assign({}, course, { teacherId });
      } else {
        return course;
      }
    });
    this.setState({ courses: newCourses });
  }

  onBeforeSubmit = () => {
    let { classDetail, courses } = this.state;
    let { everyCourseClassNum } = classDetail;

    let coursesList = courses.map((course) => {
      return {
        courseDate: course.courseDate,
        courseStartTime: course.courseTime,
        teacherId: course.teacherId,
        courseHours: everyCourseClassNum,
        onlineClassId: this.props.classId,
      };
    });

    if (!coursesList.length) {
      return showMessage('请填写课程');
    }

    if (
      !coursesList.every((course) => {
        return !!(course.courseHours && course.teacherId && course.courseStartTime);
      })
    ) {
      return showMessage('请填写完整开课时间，老师以及每讲课时');
    }
    this.props.onSubmit(coursesList);
  };

  render() {
    const { loading, teacherDict, date, courses, courseTime, classDetail, teacherId } = this.state;
    const { startDate } = classDetail;

    return (
      <Modal
        title="新增课程"
        onCancel={this.props.onClose}
        onOk={this.onBeforeSubmit}
        loading={loading}
        footerTipText={`已有讲数: ${courses.length}讲`}
      >
        <Block title="基础信息">
          <Row className={styles.infoRow}>
            班级名称
            <span className={styles.text}>{classDetail.productName}</span>
          </Row>
          <Row className={styles.infoRow}>
            <Col span={8}>
              <p>
                总课时数
                <span className={styles.text}>{classDetail.totalClassHours}</span>
              </p>
            </Col>
            <Col span={8}>
              <p>
                未排课时
                <span className={styles.text}>
                  {NP.minus(
                    classDetail.notArrangeCourseHours,
                    NP.times(classDetail.everyCourseClassNum || 0, courses.length),
                  )}
                </span>
              </p>
            </Col>
            <Col span={8}>
              <p>
                课时时长
                <span className={styles.text}>{`${classDetail.classTimeLength}分钟`}</span>
              </p>
            </Col>
          </Row>
          <Row className={styles.infoRow}>
            <Col span={8}>
              <div className={styles.label}>
                授课老师<span className={styles.red}>*</span>
                <Select
                  className={styles.select}
                  value={teacherId}
                  dict={teacherDict}
                  allowClear={false}
                  onChange={this.updateTeacherId}
                  showSearch={true}
                />
              </div>
            </Col>
            <Col span={8}>
              <div className={styles.label}>
                每讲课时<span className={styles.red}>*</span>
                <InputNumber
                  className={styles.select}
                  value={classDetail.everyCourseClassNum}
                  onChange={(e) => {
                    this.setState((preState) => {
                      return {
                        classDetail: {
                          ...preState.classDetail,
                          everyCourseClassNum: e,
                        },
                      };
                    });
                  }}
                  step={0.5}
                  min={0.01}
                />
                <p className={styles.tips}>
                  (1讲=
                  {((classDetail.everyCourseClassNum * (classDetail.classTimeLength || 0)) / 60).toFixed(2)}
                  小时)
                </p>
              </div>
            </Col>
          </Row>
        </Block>
        <Block title="设置开始时间" style={{ marginTop: '20px' }}>
          {courseTime.map((time, index) => {
            return (
              <Row className={styles.startTimeInput} key={index}>
                <Col style={{ paddingRight: '14px', color: '#999' }}>开始时间</Col>
                <Col>
                  <TimePicker
                    className={styles.TimePicker}
                    value={time}
                    disabledHours={() => {
                      return Array(6)
                        .fill(null)
                        .map((item, i) => {
                          return i;
                        });
                    }}
                    disabledMinutes={(h) => {
                      if (h <= 6) {
                        return [0];
                      }
                      return [];
                    }}
                    onChange={(value) => {
                      let newCourses = courses.map((course) => {
                        if (time === course.courseTime) {
                          return Object.assign({}, course, {
                            courseTime: value,
                          });
                        } else {
                          return course;
                        }
                      });
                      this.setState({ courses: newCourses });
                      let newStartTime = courseTime.map((curTime, i) => {
                        if (i === index) {
                          return value;
                        } else {
                          return curTime;
                        }
                      });
                      this.setState({ courseTime: newStartTime });
                    }}
                  />
                </Col>
                <Col style={{ paddingLeft: '14px' }}>
                  {index === 0 ? (
                    <div>
                      <span
                        onClick={() => {
                          let { courseTime } = this.state;
                          courseTime.push(null);
                          this.setState({ courseTime });
                        }}
                      >
                        新增
                      </span>
                    </div>
                  ) : (
                    <div>
                      <span
                        onClick={() => {
                          let { courseTime } = this.state;
                          courseTime.push(null);
                          this.setState({ courseTime });
                        }}
                      >
                        新增
                      </span>
                      <span
                        onClick={() => {
                          let { courseTime } = this.state;
                          courseTime.splice(index, 1);
                          this.setState({ courseTime });
                        }}
                      >
                        删除
                      </span>
                    </div>
                  )}
                </Col>
              </Row>
            );
          })}
          <Calendar date={date} updateDate={this.updateDate} startDate={startDate} />
          <Row className={styles.setTimeRowHeader}>
            <Col span={3} className={styles.setTimeRowHeaderText}>
              日期
            </Col>
            <Col span={4} className={styles.setTimeRowHeaderText}>
              开始时间<span>*</span>
            </Col>
            <Col span={4} className={styles.setTimeRowHeaderText}>
              老师<span>*</span>
            </Col>
            <Col span={3} className={styles.setTimeRowHeaderText} style={{ paddingLeft: '30px' }}>
              操作
            </Col>
          </Row>
          {courses.map((course, index) => {
            const { courseDate, teacherId, courseTime } = course;
            return (
              <Row key={index} className={styles.setTimeRow}>
                <Col span={3} className={styles.setTimeRowLabel}>
                  {courseDate}
                </Col>
                <Col span={4} className={styles.setTimeRowLabel}>
                  <TimePicker
                    value={courseTime}
                    className={styles.setTimeRowPicker}
                    onChange={(courseTime) => {
                      let newCourses = courses.map((course, i) => {
                        if (index === i) {
                          return Object.assign({}, course, { courseTime });
                        } else {
                          return course;
                        }
                      });
                      this.setState({ courses: newCourses });
                    }}
                    disabledHours={() => {
                      return Array(6)
                        .fill(null)
                        .map((item, i) => {
                          return i;
                        });
                    }}
                    disabledMinutes={(h) => {
                      if (h <= 6) {
                        return [0];
                      }
                      return [];
                    }}
                  />
                </Col>
                <Col span={4} className={styles.setTimeRowLabel}>
                  <Select
                    className={styles.setTimeRowPicker}
                    dict={teacherDict}
                    value={teacherId}
                    onChange={(teacherId) => {
                      let newCourses = courses.map((course, i) => {
                        if (index === i) {
                          return Object.assign({}, course, { teacherId });
                        } else {
                          return course;
                        }
                      });
                      this.setState({ courses: newCourses });
                    }}
                  />
                </Col>
                <Col span={3} className={styles.setTimeRowLabel}>
                  <span
                    className={styles.delete}
                    onClick={() => {
                      let { date, courses } = this.state;
                      courses = courses.filter((course, i) => {
                        return i !== index;
                      });
                      date = date.filter((d, i) => {
                        return i !== index;
                      });
                      this.setState({ courses, date });
                    }}
                  >
                    删除
                  </span>
                </Col>
              </Row>
            );
          })}
        </Block>
      </Modal>
    );
  }
}

export default AddNewCourse;
