import React from 'react';
import { connect } from 'dva';
import { Modal, Card, Checkbox, Pagination, Cascader } from 'antd';
import { SearchBar, Input, Select, Button, Loading, NoData } from 'COMPONENT';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { getSmallClassCourse } from 'SERVICE/bigClass';
import styles from './styles.module.less';
import { loadCampusDict } from 'UTIL/dict';

const getSearchInitValue = () => {
  return {
    branchId: '',
    campusId: '',
    subjectId: '',
    phaseId: null,
    classLevelId: '',
    gradeId: '',
    className: '',
    productName: '',
  };
};

class AddSmallClassModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      searchExpand: false,
      searchParams: getSearchInitValue(),
      smallClassList: [],
      total: 0,
      currentPage: 1,
      loading: false,
      isSelectAll: false,
    };
  }
  componentDidMount() {
    this.initDictionary();
    this.onSearch();
  }

  componentWillReceiveProps(nextprops) {
    const { visible, currentSelectBranch } = nextprops;

    if (visible === true && visible !== this.props.visible) {
      this.setState({
        smallClassList: [],
        total: 0,
        isSelectAll: false,
      });
      this.resetSearchParams();

      if (currentSelectBranch) {
        this.fetchCampusByBranch(currentSelectBranch);
      }
    }
  }

  initDictionary() {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['branch', 'grade', 'classLevelAll', 'phase', 'subject', 'weekDay', 'quarter'],
      },
    });
  }

  // 根据分公司获取学习中心
  fetchCampusByBranch = (branchId) => {
    loadCampusDict(branchId).then((res) => {
      this.setState({
        CAMPUS: res || [],
      });
    });
  };

  // 选择小班保存
  onSubmit = () => {
    if (this.state.smallClassList.length == 0) {
      return showMessage('请选择小班');
    }
    const selectData = this.state.smallClassList.filter((item) => item.isSelected);
    this.props.onConfirmSelectSmallClass(selectData);
  };

  setSearchParams = (key, value) => {
    this.setState({
      searchParams: {
        ...this.state.searchParams,
        [key]: value,
      },
    });
  };

  // 搜索小班列表
  onSearch = () => {
    let params = {
      ...this.state.searchParams,
      branchId: this.props.currentSelectBranch,
      pageNum: this.state.currentPage,
      pageSize: 6,
    };
    params.phaseId = params.phaseId ? params.phaseId[1] : '';
    let hasSelectedData = this.props.hasSelectedData;

    this.setState({
      loading: true,
    });

    getSmallClassCourse(params)
      .then((res) => {
        let data = res.data.list || [];

        data = data.map((v) => {
          let checkIsSelected = hasSelectedData.find((item) => item.miniClassId === v.miniClassId);

          if (checkIsSelected) {
            v.isSelected = true;
            v.hasSave = true;
          }
          return v;
        });

        this.setState({
          smallClassList: data,
          total: res.data.total,
          isSelectAll: data.every((v) => v.isSelected),
        });
      })
      .finally(() => {
        this.setState({
          loading: false,
        });
      });
  };

  onPageChange = (e) => {
    this.setState(
      {
        currentPage: e,
      },
      this.onSearch,
    );
  };

  // 是否选中某个小班
  onCheckChange = (e, data) => {
    let checked = e.target.checked;
    let dataList = this.state.smallClassList.slice(0);
    let item = dataList.find((v) => v.miniClassId === data.miniClassId);
    item.isSelected = checked;

    this.setState({
      smallClassList: dataList,
      isSelectAll: dataList.every((v) => v.isSelected),
    });
  };

  // 全选/反选
  onSelectAllChange = (e) => {
    let checked = e.target.checked;
    let dataList = this.state.smallClassList.map((v) => {
      return {
        ...v,
        isSelected: !v.hasSave ? checked : v.isSelected,
      };
    });

    this.setState({
      isSelectAll: checked,
      smallClassList: dataList,
    });
  };

  resetSearchParams = () => {
    this.setState(
      {
        searchParams: getSearchInitValue(),
        currentPage: 1,
      },
      this.onSearch,
    );
  };

  formatSelectOption = (data) => {
    return data.map((v) => {
      return {
        value: v.val,
        label: v.text,
      };
    });
  };

  render() {
    const { searchExpand, searchParams, loading, smallClassList, total, currentPage, isSelectAll, CAMPUS } = this.state;

    const { STUDENT_GRADE, COMPANY, SUBJECT, LEVEL, PHASE, WEEKDAY, QUARTER, visible, currentSelectBranch } =
      this.props;

    const { campusId, subjectId, phaseId, classLevelId, gradeId, className, productName } = searchParams;

    const renderClassList = () => {
      return smallClassList.map((item) => {
        return (
          <div key={item.miniClassId} className={styles.cardItemWrap}>
            <Card>
              <label className={styles.cardItem}>
                <Checkbox
                  checked={item.isSelected}
                  disabled={item.hasSave}
                  onChange={(e) => {
                    this.onCheckChange(e, item);
                  }}
                  className={styles.cardItemSelect}
                ></Checkbox>
                <div className={styles.cardItemContent}>
                  <h4 title={item.miniClassName}>{item.miniClassName}</h4>
                  <p title={item.productName}>产品：{item.productName}</p>
                  <p>上课日期：{item.startDate}</p>
                  <p>
                    上课时间：{item.classStartTime} - {item.classEndTime}
                  </p>
                  <p>老师：{item.teacherName}</p>
                  <p>班主任：{item.studyManagerName}</p>
                  <p>教室：{item.classroomName}</p>
                  <p>
                    已上讲课/总讲数：{item.attendNum}/{item.allNum}
                  </p>
                  <p>
                    招生人数：{item.miniClassPeopleNum}/{item.peopleQuantity}
                  </p>
                  <p>班级层次：{item.classLevelName}</p>
                </div>
              </label>
            </Card>
          </div>
        );
      });
    };

    const phaseList = PHASE || [];
    const weekdayList = WEEKDAY || [];
    const quarterList = QUARTER || [];

    // 寒暑显示期数，其余显示星期
    const phaseSelect = quarterList.reduce((pre, cur) => {
      if (/(DAT0000000231)|(DAT0000000233)|(DAT0000000245)/.test(cur.val)) {
        pre.push({
          value: cur.val,
          label: cur.text,
          children: this.formatSelectOption(weekdayList),
        });
      } else {
        pre.push({
          value: cur.val,
          label: cur.text,
          children: this.formatSelectOption(phaseList),
        });
      }
      return pre;
    }, []);

    return (
      <Modal
        title="关联小班"
        wrapClassName={styles.addSmallClassDialog}
        width={1200}
        visible={visible}
        onCancel={this.props.onCancel}
        footer={
          <div>
            <Button type="primary" onClick={this.onSubmit}>
              确定
            </Button>
            <Button onClick={this.props.onCancel}>取消</Button>
          </div>
        }
      >
        <Loading loading={loading} />
        <SearchBar
          expand={searchExpand}
          onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
          onSearch={() => {
            this.setState(
              {
                currentPage: 1,
              },
              this.onSearch,
            );
          }}
          onReset={this.resetSearchParams}
        >
          <Select placeholder="所属分公司" dict={COMPANY} value={currentSelectBranch} disabled />
          <Select
            placeholder="所属分学习中心"
            dict={CAMPUS}
            value={campusId}
            onChange={(value) => {
              this.setSearchParams('campusId', value);
            }}
          />
          <Input
            placeholder="产品名称"
            value={productName}
            onChange={(e) => {
              this.setSearchParams('productName', e.target.value);
            }}
          />

          <Input
            placeholder="班级名称"
            value={className}
            onChange={(e) => {
              this.setSearchParams('className', e.target.value);
            }}
          />

          <Select
            placeholder="年级"
            dict={STUDENT_GRADE}
            value={gradeId}
            onChange={(value) => {
              this.setSearchParams('gradeId', value);
            }}
          />

          <Select
            placeholder="科目"
            dict={SUBJECT}
            value={subjectId}
            onChange={(value) => {
              this.setSearchParams('subjectId', value);
            }}
          />

          <Select
            placeholder="班级层次"
            dict={LEVEL}
            value={classLevelId}
            onChange={(value) => {
              this.setSearchParams('classLevelId', value);
            }}
          />

          <Cascader
            size="large"
            value={phaseId}
            options={phaseSelect}
            onChange={(value) => {
              this.setSearchParams('phaseId', value || null);
            }}
            placeholder="选择期数"
          />
        </SearchBar>

        {smallClassList.length ? (
          <Checkbox style={{ margin: '10px' }} checked={isSelectAll} onChange={this.onSelectAllChange}>
            全选
          </Checkbox>
        ) : null}

        <div className={styles.cardContainer}>
          {smallClassList.length ? renderClassList() : <NoData style={{ flex: 1 }} />}
        </div>

        {smallClassList.length
          ? [
              <p className={styles.tips} key="tip">
                *
                关联多个小班的信息（产品名称、科目、期数、班级层次、开始日期，上课时间，已上讲数）必须全部一致且课程模板都为空。关联的小班必须已绑定好老师、教室、班主任；
                绑定后，所有小班的课程信息不可修改。
              </p>,
              <Pagination
                key="page"
                current={currentPage}
                pageSize={6}
                total={total}
                showQuickJumper
                onChange={this.onPageChange}
              />,
            ]
          : null}
      </Modal>
    );
  }
}

export default connect((state) => {
  return {
    STUDENT_GRADE: state.dict['grade'],
    COMPANY: state.dict['branch'],
    SUBJECT: state.dict['subject'],
    LEVEL: state.dict['classLevelAll'],
    PHASE: state.dict['phase'],
    WEEKDAY: state.dict['weekDay'],
    QUARTER: state.dict['quarter'],
  };
})(AddSmallClassModal);
