import React from 'react';
import { connect } from 'dva';
import { Page, Button, Loading } from 'COMPONENT';
import { message } from 'antd';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import styles from './styles.module.less';

import SmallClassList from './smallClassList';
import ClassInfoForm from '../common/form';

import { createBisClass, editBisClass, getBisClassDetail } from 'SERVICE/bigClass';

class BigClassDetail extends React.Component {
  constructor(props) {
    super(props);

    const { pageParams } = props;

    this.state = {
      selectedSmallClass: [],
      classDetail: null,
      loading: false,
      currentSelectBranch: '',
    };

    this.pageParams = pageParams[`/${props.route.path}`];
    this.formRef = null;
  }

  componentDidMount() {
    this.initDict();

    if (this.pageParams.classId) {
      this.getDetail();
    }
  }

  initDict = () => {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['provinceCityDict'],
      },
    });
  };

  getDetail = () => {
    this.setState({
      loading: true,
    });
    getBisClassDetail({ onlineClassId: this.pageParams.classId })
      .then((res) => {
        this.setState({
          classDetail: res.data,
        });
      })
      .finally(() => {
        this.setState({
          loading: false,
        });
      });
  };

  // 大班保存
  save = () => {
    let formValue = getWrappedInstance(this.formRef).getFormValue();
    let miniClassIds = this.state.selectedSmallClass.map((v) => v.miniClassId);
    let params = {
      ...formValue,
      miniClassIds: miniClassIds,
      courseModalId: Number(formValue.courseModalId),
      name: formValue.name.trim(),
    };

    const errorTipsMap = {
      name(value) {
        if (!value.length) {
          return '请填写课程名称';
        }
        if (value.length > 50) {
          return '课程名称不超过50个字符';
        }
      },
      courseModalId(value) {
        if (!value) {
          return '请选择对应的课程模板';
        }
      },
      teacherId(value) {
        if (!value) {
          return '请选择对应的主讲老师';
        }
      },
      miniClassIds(value) {
        if (!value || value.length === 0) {
          return '请关联至少一个小班课信息';
        }
      },
    };

    const errList = Object.keys(params).reduce((arr, key) => {
      if (errorTipsMap[key]) {
        let errorValue = errorTipsMap[key](params[key]);

        errorValue && arr.push(errorValue);
      }
      return arr;
    }, []);

    if (errList.length > 0) {
      let errListContent = errList.map((e, i) => {
        return (
          <p style={{ textAlign: 'left' }} key={i}>
            {e}
          </p>
        );
      });

      let errorTip = (
        <div
          style={{
            display: 'inline-block',
            verticalAlign: 'middle',
            marginLeft: '10px',
          }}
        >
          {errListContent}
        </div>
      );

      return message.error(errorTip);
    }

    let saveRequest = this.pageParams.classId
      ? editBisClass(Object.assign({ id: this.pageParams.classId }, params))
      : createBisClass(params);

    saveRequest
      .then(() => {
        showMessage('保存成功', 'success');
        this.props.dispatch({
          type: 'main/openPage',
          payload: {
            path: '/LocalNetworkClass',
            refresh: false,
            params: {
              refreshBySelf: true,
            },
          },
        });
      })
      .catch((e) => {
        showMessage(e.text || '保存失败');
      });
  };

  render() {
    const { selectedSmallClass, classDetail, loading, currentSelectBranch } = this.state;
    const { route } = this.props;

    return (
      <Page name={route.name} route={route}>
        <Loading loading={loading} />

        <ClassInfoForm
          selectedSmallClass={selectedSmallClass}
          formValue={classDetail}
          branchId={this.pageParams.branchId}
          onBranchChange={(branch) => {
            this.setState({
              currentSelectBranch: branch,
            });
          }}
          ref={(e) => {
            this.formRef = e;
          }}
        />

        <SmallClassList
          classId={this.pageParams.classId}
          currentSelectBranch={currentSelectBranch}
          onSelectSmallClassChange={(selected) => {
            this.setState({
              selectedSmallClass: selected,
            });
          }}
        />
        <div className={styles.saveBtn}>
          <Button type="primary" onClick={this.save}>
            保存
          </Button>
        </div>
      </Page>
    );
  }
}

export default connect((state) => {
  return {
    pageParams: state.main.pageParams || {},
  };
})(BigClassDetail);
