import React from 'react';
import { Table } from 'antd';
import { List, Button, Section, TextLink, Modal } from 'COMPONENT';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import AddSmallClassModal from './addSmallClassModal';
import columns from '../common/smallClassColumn';

import { getSmallClassCourseByOnlineId } from 'SERVICE/bigClass';

const { COL_WIDTH_L } = List;

class SmallClassList extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      data: [],
      selectDialogVisible: false,
    };
  }

  componentDidMount() {
    if (this.props.classId) {
      this.fetchSavedSmallClass(this.props.classId);
    }
  }

  // 获取接口已保存关联的小班
  fetchSavedSmallClass = (classId) => {
    getSmallClassCourseByOnlineId({ onlineClassId: classId }).then((res) => {
      this.setSmallClassList(res.data.list);
    });
  };

  onDelete = (row) => {
    Modal.confirm({
      title: '提示',
      content: '确定要删除小班吗？',
      onOk: async () => {
        let data = this.state.data.filter((v) => v.miniClassId !== row.miniClassId);
        this.setSmallClassList(data);
      },
    });
  };

  // 弹框选择的小班，保存前进行检测
  onSaveSmallClass = (selected) => {
    let data = this.state.data;
    let addData = selected.filter((v) => !data.find((item) => item.miniClassId === v.miniClassId));
    let allSelected = data.concat(addData);
    let compareItem = allSelected[0];
    let compareKeys = [
      'productName',
      'subjectName',
      'phaseName',
      'classLevelName',
      'startDate',
      'classTime',
      'attendNum',
    ];

    // 以上字段所选须一致
    let checkNoConflict = allSelected.every((item) => {
      return compareKeys.every((key) => {
        return item[key] === compareItem[key];
      });
    });

    // 需都绑定老师、班主任、教室
    let isAllClassBindTeacher = allSelected.every((item) => {
      return item.studyManagerId && item.teacherId && item.classroomId;
    });

    if (!checkNoConflict) {
      return showMessage(
        '关联多个小班的信息（产品名称、科目、期数、班级层次、开始日期，上课时间，  已上讲数）必须全部一致，请检查无误后再提交',
      );
    }

    if (!isAllClassBindTeacher) {
      return showMessage('关联的小班必须都已绑定好老师、教室、班主任');
    }

    this.setState({
      data: allSelected,
      selectDialogVisible: false,
    });
    this.setSmallClassList(allSelected);
  };

  // 添加或删除小班
  setSmallClassList = (data) => {
    this.setState(
      {
        data,
      },
      () => {
        this.props.onSelectSmallClassChange(data);
      },
    );
  };

  render() {
    const { data, selectDialogVisible } = this.state;
    const { currentSelectBranch } = this.props;
    const onDelete = this.onDelete;

    const classColumns = columns.concat({
      title: '操作',
      dataIndex: 'operation',
      key: 'operation',
      rowKey: 'operation',
      width: COL_WIDTH_L,
      render(text, row) {
        return <TextLink onClick={() => onDelete(row)}>删除</TextLink>;
      },
    });

    return (
      <Section
        title="关联小班"
        style={{ marginTop: '20px' }}
        toolBar={
          <Button
            type="primary"
            icon="plus"
            onClick={() => {
              if (!currentSelectBranch) {
                return showMessage('请选择所属分公司');
              }
              this.setState({ selectDialogVisible: true });
            }}
          >
            新增关联小班
          </Button>
        }
      >
        <div className="List">
          <Table
            style={{ marginBottom: '20px' }}
            dataSource={data}
            total={data.length}
            columns={classColumns}
            bordered={true}
            fixedHeader={true}
            scroll={{ y: 350 }}
          />
        </div>

        <AddSmallClassModal
          hasSelectedData={data}
          visible={selectDialogVisible}
          currentSelectBranch={currentSelectBranch}
          onConfirmSelectSmallClass={(selected) => {
            this.onSaveSmallClass(selected);
          }}
          onCancel={() => {
            this.setState({
              selectDialogVisible: false,
            });
          }}
        />
      </Section>
    );
  }
}

export default SmallClassList;
