import React from 'react';
import moment from 'moment';
import { List } from 'COMPONENT';
import { getBisClassOperation } from 'SERVICE/bigClass';

const { COL_WIDTH_M, COL_WIDTH_L } = List;
const columns = [
  {
    title: '序号',
    dataIndex: 'index',
    key: 'index',
    width: COL_WIDTH_M,
    render(text, row, index) {
      return index + 1;
    },
  },
  {
    title: '操作时间',
    dataIndex: 'createTime',
    key: 'createTime',
    width: COL_WIDTH_L,
    render(text) {
      return moment(text).format('YYYY-MM-DD HH:mm:ss');
    },
  },
  {
    title: '操作人',
    dataIndex: 'createUserName',
    key: 'createUserName',
    width: COL_WIDTH_M,
  },
  {
    title: '操作类型',
    dataIndex: 'operationTypeName',
    key: 'operationTypeName',
    width: COL_WIDTH_L,
  },
  {
    title: '操作内容',
    dataIndex: 'operationContent',
    key: 'operationContent',
    width: COL_WIDTH_L,
  },
];
class SmallClassList extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      data: [],
      currentPage: 1,
      total: 0,
      pageSize: 10,
    };
  }

  componentDidMount() {
    this.fetchChangeRecord();
  }

  fetchChangeRecord = () => {
    getBisClassOperation({
      onlineClassId: this.props.onlineClassId,
      pageSize: this.state.pageSize,
      pageNum: this.state.currentPage,
    }).then((res) => {
      this.setState({
        data: res.data.list,
        total: res.data.total,
      });
    });
  };

  onPaginationChange = (pageData) => {
    this.setState(
      {
        currentPage: pageData.current,
        pageSize: pageData.pageSize,
      },
      this.fetchChangeRecord,
    );
  };

  render() {
    const { data, currentPage, pageSize, total } = this.state;

    return (
      <List
        dataSource={data}
        columns={columns}
        pagination={{
          current: currentPage,
          total: total,
          pageSize: pageSize,
        }}
        onChange={this.onPaginationChange}
        bordered={true}
      />
    );
  }
}

export default SmallClassList;
