import React from 'react';
import { connect } from 'dva';
import { Page, Loading } from 'COMPONENT';

import SmallClassList from './smallClassList';
import ClassInfoForm from '../common/form';

import { getBisClassDetail } from 'SERVICE/bigClass';

class BigClassDetail extends React.Component {
  constructor(props) {
    super(props);

    const { pageParams } = props;

    this.state = {
      selectedSmallClass: [],
      classDetail: null,
      loading: false,
    };

    this.pageParams = pageParams[`/${props.route.path}`];
    this.formRef = null;
  }

  componentDidMount() {
    this.initDict();
    this.getDetail();
  }

  initDict = () => {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['provinceCityDict', 'branchAllCampus'],
      },
    });
  };

  getDetail = () => {
    this.setState({
      loading: true,
    });
    getBisClassDetail({ onlineClassId: this.pageParams.classId })
      .then((res) => {
        this.setState({
          classDetail: res.data,
        });
      })
      .finally(() => {
        this.setState({
          loading: false,
        });
      });
  };

  render() {
    const { selectedSmallClass, classDetail, loading } = this.state;
    const { route } = this.props;

    return (
      <Page name={route.name} route={route}>
        <Loading loading={loading} />

        <ClassInfoForm
          selectedSmallClass={selectedSmallClass}
          formValue={classDetail}
          branchId={this.pageParams.branchId}
          isDetail={true}
          ref={(e) => {
            this.formRef = e;
          }}
        />

        <SmallClassList
          classId={this.pageParams.classId}
          onSelectSmallClassChange={(selected) => {
            this.setState({
              selectedSmallClass: selected,
            });
          }}
        />
      </Page>
    );
  }
}

export default connect((state) => {
  return {
    pageParams: state.main.pageParams || {},
  };
})(BigClassDetail);
