import React from 'react';
import { connect } from 'dva';
import { List, TextLink } from 'COMPONENT';
import Loading from 'COMPONENT/Loading';
import Search from './search';
import columns from './column';
import AddressDialog from './addressDialog';

import styles from './styles.module.less';
import { getStudentReceiveInfoList, getStudentReceiveInfoById } from 'SERVICE/bigClass';
class ReceiveInfo extends React.Component {
  constructor(props) {
    super(props);

    this.state = Object.assign({
      loading: false, // {boolean} 内容加载状态
      list: [], // {Array} 列表数据，
      totalRecords: 0, // {number} 总记录数
      currentPage: 1, // {number} 当前页码
      pageSize: 10, // {number} 每页记录数
      searchParams: {},
      showAddressDialog: false,
    });
  }

  componentDidMount() {
    this.fetchReceiverList();
  }

  fetchReceiverList() {
    this.setRequestLoading(true);

    let searchParams = this.state.searchParams;
    let pagination = {
      pageSize: this.state.pageSize,
      pageNum: this.state.currentPage,
    };

    let params = {
      ...searchParams,
      ...pagination,
      onlineClassId: this.props.onlineClassId,
    };

    getStudentReceiveInfoList(params)
      .then((res) => {
        this.setState({
          list: res.data.list,
          totalRecords: res.data.total,
          loading: false,
        });
      })
      .catch(() => {
        this.setRequestLoading(false);
      });
  }

  onSearch = (searchParams) => {
    this.setState(
      {
        searchParams,
      },
      this.fetchReceiverList,
    );
  };

  onPaginationChange = (pageData) => {
    this.setState(
      {
        currentPage: pageData.current,
        pageSize: pageData.pageSize,
      },
      this.fetchReceiverList,
    );
  };

  onEdit = (row) => {
    this.setState({
      loading: true,
    });
    getStudentReceiveInfoById({
      contractProductId: row.contractProductId,
    })
      .then((res) => {
        this.setState({
          showAddressDialog: true,
          currentRow: res.data,
        });
      })
      .finally(() => {
        this.setState({
          loading: false,
        });
      });
  };

  setRequestLoading = (loading) => {
    this.setState({
      loading,
    });
  };

  render() {
    const { loading, list, totalRecords, currentPage, pageSize, showAddressDialog, currentRow } = this.state;

    const { hasEditPermisison } = this.props;

    // 菜单有分配编辑权限才出现修改按钮
    const tableColumn = hasEditPermisison
      ? columns.concat({
          title: '操作',
          dataIndex: 'operation',
          key: 'operation',
          fixed: 'right',
          width: 150,
          render: (text, row) => {
            return (
              <div>
                <TextLink onClick={() => this.onEdit(row)}>修改</TextLink>
              </div>
            );
          },
        })
      : columns;

    return (
      <div className={styles.wrap}>
        <Search
          onSearch={this.onSearch}
          isManagePage={this.props.isManagePage}
          onlineClassId={this.props.onlineClassId}
        />

        <List
          dataSource={list}
          columns={tableColumn}
          pagination={{
            current: currentPage,
            total: totalRecords,
            pageSize,
          }}
          onChange={this.onPaginationChange}
          bordered={true}
          className={styles.customList}
        />

        <Loading loading={loading} />
        <AddressDialog
          visible={showAddressDialog}
          data={currentRow}
          onSuccess={() => {
            this.setState({
              showAddressDialog: false,
            });
            this.fetchReceiverList();
          }}
          onCancel={() => {
            this.setState({
              showAddressDialog: false,
            });
          }}
        />
      </div>
    );
  }
}

function mapStateToProps(state) {
  const { btnAuths } = state.account;
  const hasEditPermisison = btnAuths.indexOf('LOCAL_NETWORK_CLASS_EDIT') > -1;

  return {
    hasEditPermisison,
  };
}

export default connect(mapStateToProps)(ReceiveInfo);
