import React from 'react';
import { connect } from 'dva';
import { Input, Select } from 'COMPONENT';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import SearchBar from '../SearchBar';
import { exportReceiverTable } from 'SERVICE/bigClass';
import { getRegionList, getAreaList } from 'SERVICE/common';

let cityMap = {};
const getSearchInitValue = () => {
  return {
    miniClassName: '',
    classId: '',
    gradeId: '',
    subject: '',
    status: '',
    provinceId: '',
    cityId: '',
    areaId: '',
    studentId: '',
    studentName: '',
    recipient: '',
    recipientPhone: '',
    contractCampusId: '',
    recentlyCampusId: '',
  };
};

class Search extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false, // {boolean} 内容加载状态
      searchExpand: false, // {boolean} 搜索栏展开状态
      searchParams: getSearchInitValue(),
      provinceList: [],
      cityList: [],
      areaList: [],
    };
  }

  componentDidMount() {
    this.fetchProvince();
  }

  // 获取省份
  fetchProvince = () => {
    getRegionList().then((res) => {
      const provinceList = res.map((p) => {
        return {
          val: p.id,
          text: p.name,
        };
      });

      // 生成省份map用于获取city
      cityMap = res.reduce((pre, cur) => {
        pre[cur.id] = cur.children;
        return pre;
      }, {});

      this.setState({
        provinceList: provinceList,
      });
    });
  };

  // 获取地区
  getAreaListByCity = (value) => {
    getAreaList({ cityId: value }).then((res) => {
      this.setState({
        areaList: (res || []).map((area) => {
          return {
            val: area.id,
            text: area.name,
          };
        }),
      });
    });
  };

  formatParams = (params) => {
    return Object.keys(params).reduce((obj, key) => {
      if (params[key]) {
        if (/(provinceId)|(cityId)/.test(key)) {
          obj[key] = Number(params[key]);
        } else {
          obj[key] = params[key];
        }
      }

      return obj;
    }, {});
  };

  setSearchParams = (key, value) => {
    this.setState({
      searchParams: {
        ...this.state.searchParams,
        [key]: value,
      },
    });
  };

  resetSearchParams = () => {
    this.setState({
      searchParams: getSearchInitValue(),
    });
  };

  onExport = () => {
    const permission = this.props.hasExportPermisison;

    if (!permission) {
      return showMessage('您暂无导出报表权限');
    }

    this.setState({
      loading: true,
    });
    exportReceiverTable({
      onlineClassId: this.props.onlineClassId,
    })
      .then(() => {
        this.props.dispatch({ type: 'main/exportNotice' });
      })
      .catch((e) => {
        showMessage(e.text);
      })
      .finally(() => {
        this.setState({
          loading: false,
        });
      });
  };

  render() {
    const { searchExpand, searchParams, provinceList, cityList, areaList, loading } = this.state;
    const { CAMPUS } = this.props;
    const {
      miniClassName,
      studentId,
      studentName,
      recipient,
      recipientPhone,
      contractCampusId,
      recentlyCampusId,
      provinceId,
      cityId,
      areaId,
    } = searchParams;

    return (
      <SearchBar
        expand={searchExpand}
        onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
        onSearch={() => this.props.onSearch(this.formatParams(searchParams))}
        onExport={this.onExport}
        exportLoading={loading}
        onReset={this.resetSearchParams}
      >
        <Input
          placeholder="小班名称"
          value={miniClassName}
          onChange={(e) => {
            this.setSearchParams('miniClassName', e.target.value);
          }}
        />

        <Input
          placeholder="学生编号"
          value={studentId}
          onChange={(e) => {
            this.setSearchParams('studentId', e.target.value);
          }}
        />

        <Input
          placeholder="学生姓名"
          value={studentName}
          onChange={(e) => {
            this.setSearchParams('studentName', e.target.value);
          }}
        />

        <Select
          showSearch
          placeholder="所属学习中心"
          dict={CAMPUS}
          value={contractCampusId}
          onChange={(value) => {
            this.setSearchParams('contractCampusId', value);
          }}
        />

        <Input
          placeholder="收件人电话"
          value={recipientPhone}
          onChange={(e) => {
            this.setSearchParams('recipientPhone', e.target.value);
          }}
        />

        <Input
          placeholder="收件人"
          value={recipient}
          onChange={(e) => {
            this.setSearchParams('recipient', e.target.value);
          }}
        />

        <Select
          showSearch
          placeholder="距离最近分公司学习中心"
          dict={CAMPUS}
          value={recentlyCampusId}
          onChange={(value) => {
            this.setSearchParams('recentlyCampusId', value);
          }}
        />

        <Select
          placeholder="省"
          dict={provinceList}
          value={provinceId}
          onChange={(value) => {
            this.setSearchParams('provinceId', value);
            this.setState({
              areaList: [],
              cityList: (cityMap[value] || []).map((c) => {
                return {
                  text: c.name,
                  val: c.id,
                };
              }),
            });
          }}
        />

        <Select
          placeholder="市"
          dict={cityList}
          value={cityId}
          onChange={(value) => {
            this.setSearchParams('cityId', value);
            value && this.getAreaListByCity(value);
          }}
        />

        <Select
          placeholder="区"
          dict={areaList}
          value={areaId}
          onChange={(value) => {
            this.setSearchParams('areaId', value);
          }}
        />
      </SearchBar>
    );
  }
}

export default connect((state) => {
  const { btnAuths } = state.account;
  const hasExportPermisison = btnAuths.indexOf('LOCAL_NETWORK_CLASS_EXPORT') > -1;

  return {
    CAMPUS: state.dict['branchAllCampus'],
    PROVINCE: state.dict['provinceCityDict'],
    hasExportPermisison,
  };
})(Search);
