import React from 'react';
import { Table, Tabs } from 'antd';
import { Section } from 'COMPONENT';
import ChangeRecord from './changeRecord/index';
import ReceiveInfo from './receiveInfo/index';
import columns from '../common/smallClassColumn';
import styles from './styles.module.less';

import { getSmallClassCourseByOnlineId } from 'SERVICE/bigClass';

const TabPane = Tabs.TabPane;
const sectionTitle = {
  smallClassList: (
    <span>
      关联小班
      <span className={styles.smallClassTip}>
        （关联小班的相关页面如考勤、课表等，请到“小班课程管理”菜单下的相关页面查看与操作）
      </span>
    </span>
  ),
  changeRecord: '更改记录',
  receiveInfo: '收件信息',
};

class SmallClassList extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      data: [],
      selectDialogVisible: false,
      currentTab: 'smallClassList',
    };
  }

  componentDidMount() {
    this.fetchSavedSmallClass(this.props.classId);
  }

  // 获取接口已保存关联的小班
  fetchSavedSmallClass = (classId) => {
    getSmallClassCourseByOnlineId({ onlineClassId: classId }).then((res) => {
      this.setSmallClassList(res.data.list);
    });
  };

  // 添加或删除小班
  setSmallClassList = (data) => {
    this.setState(
      {
        data,
      },
      () => {
        this.props.onSelectSmallClassChange(data);
      },
    );
  };

  // 切换详情tab
  toggleTab = (tab) => {
    this.setState({
      currentTab: tab,
    });
  };

  render() {
    const { data, currentTab } = this.state;
    const { classId } = this.props;

    return (
      <Section title={sectionTitle[currentTab]} style={{ marginTop: '20px' }}>
        <Tabs activeKey={currentTab} onChange={this.toggleTab}>
          <TabPane tab="关联小班" key="smallClassList">
            <div className="List">
              <Table
                style={{ marginBottom: '20px' }}
                dataSource={data}
                total={data.length}
                columns={columns}
                bordered={true}
                fixedHeader={true}
                scroll={{ y: 350 }}
              />
            </div>
          </TabPane>
          <TabPane tab="更改记录" key="changeRecord">
            <ChangeRecord onlineClassId={classId} />
          </TabPane>
          <TabPane tab="收件信息" key="receiveInfo">
            <ReceiveInfo onlineClassId={classId} />
          </TabPane>
        </Tabs>
      </Section>
    );
  }
}

export default SmallClassList;
