import React from 'react';
import { connect } from 'dva';
import { SearchBar, Input, Select } from 'COMPONENT';
import { DatePicker } from 'antd';
const { RangePicker } = DatePicker;

const getSearchInitValue = () => {
  return {
    className: '',
    classId: '',
    gradeId: '',
    blBranchId: '',
    subjectId: '',
    searchTime: null,
    status: '',
  };
};

class Search extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false, // {boolean} 内容加载状态
      searchExpand: false, // {boolean} 搜索栏展开状态
      searchParams: getSearchInitValue(true),
    };
  }

  componentDidMount() {
    this.initDictionary();

    const defaultCompany = this.props.COMPANY && this.props.COMPANY[0];
    if (defaultCompany) {
      this.setSearchParams('blBranchId', defaultCompany.val, () => {
        this.props.onSearch(this.formatParams(this.state.searchParams));
      });
    }

    const els = document.querySelectorAll('.shouldResizeWidth');

    [...els].forEach((el) => {
      if (el.parentNode.classList.contains('SearchBar-control')) {
        el.parentNode.style.width = '33.33334%';
      }
    });
  }

  componentWillReceiveProps(nextProps) {
    if (nextProps.COMPANY && nextProps.COMPANY.length > 0 && !this.state.searchParams.blBranchId) {
      this.setSearchParams('blBranchId', nextProps.COMPANY[0].val, () => {
        this.props.onSearch(this.formatParams(this.state.searchParams));
      });
    }
  }

  initDictionary() {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['branch', 'grade', 'subject'],
      },
    });
  }

  formatTime = (momentTime) => {
    return Number(momentTime.format('x'));
  };

  formatParams = (params) => {
    return Object.keys(params).reduce((obj, key) => {
      if (params[key]) {
        switch (key) {
          case 'searchTime':
            if (params[key].length > 0) {
              obj.startDate = params[key][0].format('YYYY-MM-DD');
              obj.endDate = params[key][1].format('YYYY-MM-DD');
            }
            break;
          default:
            obj[key] = params[key];
        }
      }

      return obj;
    }, {});
  };

  setSearchParams = (key, value, cb = function () {}) => {
    this.setState(
      {
        searchParams: {
          ...this.state.searchParams,
          [key]: value,
        },
      },
      cb,
    );
  };

  resetSearchParams = () => {
    const blBranchId = this.state.searchParams.blBranchId;

    this.setState(
      {
        searchParams: { ...getSearchInitValue(), blBranchId },
      },
      () => this.props.onSearch(this.formatParams(this.state.searchParams)),
    );
  };

  render() {
    const { searchExpand, searchParams } = this.state;
    const { STUDENT_GRADE, COMPANY, SUBJECT } = this.props;
    const { className, classId, subjectId, gradeId, blBranchId, searchTime, status } = searchParams;

    return (
      <SearchBar
        expand={searchExpand}
        onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
        onSearch={() => this.props.onSearch(this.formatParams(searchParams))}
        onReset={this.resetSearchParams}
      >
        <Input
          placeholder="本地网课名称"
          value={className}
          onChange={(e) => {
            this.setSearchParams('className', e.target.value);
          }}
        />

        <Input
          placeholder="本地网课ID"
          value={classId}
          onChange={(e) => {
            this.setSearchParams('classId', e.target.value);
          }}
        />

        <Select
          showSearch
          placeholder="年级"
          dict={STUDENT_GRADE}
          value={gradeId}
          onChange={(value) => {
            this.setSearchParams('gradeId', value);
          }}
        />

        <Select
          showSearch
          placeholder="科目"
          dict={SUBJECT}
          value={subjectId}
          onChange={(value) => {
            this.setSearchParams('subjectId', value);
          }}
        />

        <Select
          placeholder="本地网课状态"
          dict={[
            { val: 'PENDING_START', text: '未上课' },
            { val: 'STARTED', text: '上课中' },
            { val: 'COMPLETE', text: '已完成' },
          ]}
          value={status}
          onChange={(value) => {
            this.setSearchParams('status', value);
          }}
        />

        <Select
          showSearch
          placeholder="所属分公司"
          allowClear={false}
          dict={COMPANY}
          value={blBranchId}
          onChange={(value) => {
            this.setSearchParams('blBranchId', value);
          }}
        />

        <RangePicker
          className="shouldResizeWidth"
          size="large"
          value={searchTime}
          placeholder={['开始时间', '结束时间']}
          onChange={(value) => {
            this.setSearchParams('searchTime', value);
          }}
        />
      </SearchBar>
    );
  }
}

export default connect((state) => {
  return {
    STUDENT_GRADE: state.dict['grade'],
    COMPANY: state.dict['branch'],
    SUBJECT: state.dict['subject'],
  };
})(Search);
