import React from 'react';
import { Icon, Progress, Pagination } from 'antd';
import compStyle from './compStyle.less';
import './style.less';
import notDataImg from 'ASSET/imgs/notDataDef.png';
import moment from 'moment';

const TWO_TEACHER_VAL = 'TWO_TEACHER';
const SMALL_CLASS_VAL = 'SMALL_CLASS';
const RegProgress = (props) => {
  const classPrefix = 'RegProgress';
  let percent = (props.real / props.total) * 100;
  if (percent > 100) {
    percent = 100;
  }
  let colorStr = '';
  if (percent < 50) {
    colorStr = 'success';
  } else if (percent < 90) {
    colorStr = 'info';
  } else {
    colorStr = 'warning';
  }

  return (
    <div className={compStyle[classPrefix]} style={{ margin: '18px 0 1px' }}>
      <p className={compStyle[`${classPrefix}-row`]}>
        <span className={compStyle[`${classPrefix}-label`]}>报名情况：</span>
        <span className={compStyle[`${classPrefix}-content`]}>{`${props.real}/${props.total}`}</span>
      </p>
      <div className={`${classPrefix}-${colorStr}`}>
        <Progress percent={percent} strokeWidth={5} showInfo={false} />
      </div>
    </div>
  );
};
const CourseCard = (props) => {
  const classPrefix = 'CourseCard';
  let checkStr = props.selected ? '-check' : '-no-check';

  const { classTime, classTimeLength, everyCourseClassNum } = props.data;
  const endTime = moment()
    .hour(classTime.slice(0, 2) - 0)
    .minute(classTime.slice(3, 5) - 0)
    .add(classTimeLength * everyCourseClassNum, 'minutes')
    .format('HH:mm');
  const courseTime = `${classTime}-${endTime}`;

  return (
    <div className={compStyle[classPrefix]} onClick={props.onClick}>
      <div className={compStyle[`${classPrefix}-wrap`]}>
        <p className={compStyle[`${classPrefix}-title`]} style={{ marginTop: '6px' }}>
          <span title={props.data.className}>{props.data.className}</span>
        </p>
        <p className={compStyle[`${classPrefix}-vice-title`]} style={{ marginTop: '5px' }}>
          <span>产品：</span>
          <span title={props.data.productName}>{props.data.productName}</span>
        </p>
        <p className={compStyle[`${classPrefix}-content`]} style={{ marginTop: '16px' }}>
          <Icon className={compStyle[`${classPrefix}-icon`]} type="user" />
          <span>授课老师：{props.data.teachers}</span>
        </p>
        <p className={compStyle[`${classPrefix}-content`]} style={{ marginTop: '6px' }}>
          <Icon className={compStyle[`${classPrefix}-icon`]} type="clock-circle-o" />
          <span>上课日期：{props.data.courseDate}</span>
        </p>
        <p className={compStyle[`${classPrefix}-content`]} style={{ marginTop: '6px' }}>
          <Icon className={compStyle[`${classPrefix}-icon`]} type="clock-circle-o" />
          <span>上课时间：{courseTime}</span>
        </p>
        <RegProgress real={props.data.realNum} total={props.data.totalNum} />
        <Icon className={compStyle[`${classPrefix}${checkStr}`]} type="check-circle" />
      </div>
    </div>
  );
};
const NoData = (props) => {
  const classPrefix = 'NoData';

  return (
    <div>
      <img className={compStyle[`${classPrefix}-img`]} src={notDataImg} alt="暂无数据" />
      <p className={compStyle[`${classPrefix}-text`]}>{props.text || '暂无数据'}</p>
    </div>
  );
};

class CourseDesk extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      list: [],
      size: 15,
    };
    this.selectCourse = this.selectCourse.bind(this);
  }
  static classPrefix = 'CourseDesk';
  static showTotal(total, range) {
    return (
      <span>
        <span style={{ marginRight: 15 }}>15条/页</span>共 {total} 条记录
      </span>
    );
  }

  selectCourse(index) {
    const { onChange, selectedRowKeys } = this.props.rowSelection;
    let checkIndex = selectedRowKeys.findIndex((item) => item === index);
    if (checkIndex !== -1) {
      onChange([]);
    } else {
      onChange([index]);
    }
  }

  render() {
    const {
      dataSource,
      rowSelection: { selectedRowKeys },
      pagination: { current, total, pageSize },
      onChange,
      oaRecord,
    } = this.props;
    const { size } = this.state;
    const hadData = dataSource.length > 0;

    return (
      <div className={compStyle[CourseDesk.classPrefix]}>
        {hadData ? (
          <div>
            <div className={compStyle[`${CourseDesk.classPrefix}-wrap`]}>
              {dataSource.map((item, index) => (
                <CourseCard
                  data={item}
                  selected={selectedRowKeys.findIndex((item) => item === index) !== -1}
                  onClick={() => {
                    this.selectCourse(index);
                  }}
                  key={index}
                />
              ))}
            </div>
            <div className={compStyle[`${CourseDesk.classPrefix}-Pagination`]}>
              {oaRecord && oaRecord.id && (
                <div className={compStyle[`${CourseDesk.classPrefix}-b-left`]}>
                  <p>转班申请记录：{oaRecord.oaInstanceId}</p>
                  <p>审批结果：{oaRecord.auditStatusName}</p>
                  <p>
                    转入班：{oaRecord.changeNewClassVo.blCampusName}——{oaRecord.changeNewClassVo.name}
                  </p>
                </div>
              )}
              <Pagination
                defaultCurrent={1}
                current={current}
                pageSize={pageSize || size}
                total={total}
                showQuickJumper={true}
                showTotal={CourseDesk.showTotal}
                onChange={onChange}
              />
            </div>
          </div>
        ) : (
          <NoData />
        )}
      </div>
    );
  }
}

export default CourseDesk;
