import React from 'react';
import { connect } from 'dva';
import SearchBar from 'COMPONENT/SearchBar';
import Select from 'COMPONENT/Select';
import Input from 'COMPONENT/Input';
import { storeName, priceStatusDict, TWO_TEACHER_VAL, SMALL_CLASS_VAL } from '../../store/config';
import {
  buildBranchDict,
  buildBranchCampusDict,
  buildProductSeriesDict,
  buildClassTypeDict,
  buildClassLevelDict,
  buildAuxTcDict,
} from 'MODEL/dict/utils';
import { loginUser } from 'UTIL/common';
import { PROJECT_NAME, twoTeacherInstitution, xinghuoInstitution } from 'CONST/config';

class Search extends React.Component {
  constructor(props) {
    super(props);
    this.updateProps = this.updateProps.bind(this);
    this.updateValue = this.updateValue.bind(this);
    this.updateBranch = this.updateBranch.bind(this);
    this.updateCampus = this.updateCampus.bind(this);
    this.updateClassType = this.updateClassType.bind(this);
    this.isTwoTeacherSym = loginUser.isTwoTeacherSym();
  }

  updateProps(params) {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: params,
    });
  }

  updateValue(type, val) {
    this.updateProps({ [type]: val });
  }

  updateBranch(val) {
    let payload = { sKey_modalBranch: val };
    if (val) {
      let needResetCampus = false;
      if (this.props.sKey_modalCampus) {
        let filterBranchCampusDict = buildBranchCampusDict([val], this.props.branchCampusDictOrigin);
        let filterCampusDict = filterBranchCampusDict.map((item) => (item ? item.options : []));
        let filterValArr = filterCampusDict.reduce((arr, current) => arr.concat(current), []).map((item) => item.val);
        if (filterValArr.indexOf(this.props.sKey_modalCampus) === -1) {
          payload.sKey_modalCampus = null;
          needResetCampus = true;
        }
      }
      if (this.props.sKey_modalTeacher) {
        let sKey_modalCampus = needResetCampus ? null : this.props.sKey_modalCampus;
        let auxTcDict = buildAuxTcDict(val, sKey_modalCampus, this.props.auxTeacherDictOrigin);
        if (auxTcDict) {
          let filterTeacherDict = auxTcDict.map((item) => (item ? item.options : []));
          let filterValArr = filterTeacherDict
            .reduce((arr, current) => arr.concat(current), [])
            .map((item) => item.val);
          if (filterValArr.indexOf(this.props.sKey_modalTeacher) === -1) {
            payload.sKey_modalTeacher = null;
          }
        }
      }
    }
    this.updateProps(payload);
  }

  updateCampus(val) {
    let payload = { sKey_modalCampus: val };
    if (val && this.props.sKey_modalTeacher) {
      let auxTcDict = buildAuxTcDict(this.props.sKey_modalBranch, val, this.props.auxTeacherDictOrigin);
      if (auxTcDict) {
        let filterTeacherDict = auxTcDict.map((item) => (item ? item.options : []));
        let filterValArr = filterTeacherDict.reduce((arr, current) => arr.concat(current), []).map((item) => item.val);
        if (filterValArr.indexOf(this.props.sKey_modalTeacher) === -1) {
          payload.sKey_modalTeacher = null;
        }
      }
    }
    this.updateProps(payload);
  }

  updateClassType(val) {
    let payload = { sKey_modalClassType: val };
    if (val) {
      const { sKey_prdType, sKey_modalClassLevel, productTypeTree } = this.props;
      if (sKey_modalClassLevel) {
        const _classLevelDict = buildClassLevelDict(productTypeTree, sKey_prdType, val) || [];
        // 是否需要清空班级层次
        if (_classLevelDict.findIndex((item) => item.val === sKey_modalClassLevel) === -1) {
          payload.sKey_modalClassLevel = null;
        }
      }
    }
    this.updateProps(payload);
  }

  render() {
    return (
      <SearchBar
        className={`${this.props.classPrefix}-search`}
        onSearch={this.props.onSearch}
        onReset={this.props.onReset}
        expand
      >
        <Input
          placeholder="班级名称"
          value={this.props.sKey_modalClassName}
          onChange={(e) => this.updateValue('sKey_modalClassName', e.target.value)}
        />
        <Select
          placeholder="学习中心"
          dict={this.props.branchCampusDict}
          value={this.props.sKey_modalCampus}
          onChange={this.updateCampus}
          showSearch
        />
        <Select
          placeholder="年级"
          dict={this.props.gradeDict}
          value={this.props.sKey_modalGrade}
          onChange={(val) => this.updateValue('sKey_modalGrade', val)}
          allowClear={false}
          disabled={true}
        />
        <Select
          placeholder="班级类型"
          dict={this.props.classTypeDict}
          value={this.props.sKey_modalClassType}
          onChange={this.updateClassType}
          allowClear={false}
        />
        <Select
          placeholder="班级层次"
          dict={this.props.classLevelDict}
          value={this.props.sKey_modalClassLevel}
          onChange={(val) => this.updateValue('sKey_modalClassLevel', val)}
        />
        <Select
          placeholder="科目"
          dict={this.props.subjectDict}
          value={this.props.sKey_modalSubject}
          onChange={(val) => this.updateValue('sKey_modalSubject', val)}
          disabled={this.isTwoTeacherSym}
        />
        <Select
          placeholder="任课老师"
          dict={this.props.auxTeacherDict}
          value={this.props.sKey_modalTeacher}
          onChange={(val) => this.updateValue('sKey_modalTeacher', val)}
          showSearch
        />
        {/* <Select
          placeholder="请选择"
          dict={priceStatusDict}
          value={this.props.sKey_modal_priceStatus}
          onChange={val => this.updateValue('sKey_modal_priceStatus', val)}
          allowClear={false}
          disabled={true}/> */}
        <Select
          placeholder="产品性质"
          dict={this.props.productCharacterDict}
          value={this.props.sKey_modalProductCharacter}
          onChange={(val) => this.updateValue('sKey_modalProductCharacter', val)}
          disabled={true}
        />
      </SearchBar>
    );
  }
}

const mapStateToProps = (state) => {
  const { institution } = state.account.user;
  let sKey_prdType = state[storeName].sKey_prdType;
  let productTypeTree = state.dict.productTypeTree;
  // 按学习中心过滤双师辅导老师/小班任课老师字典
  let auxTeacherDictOrigin =
    sKey_prdType === TWO_TEACHER_VAL ? state.dict.branchTwoTcAuxTc : state.dict.authBranchSmallClassAuxTc;
  let auxTeacherDict = buildAuxTcDict(
    state[storeName].sKey_modalBranch,
    state[storeName].sKey_modalCampus,
    auxTeacherDictOrigin,
  );
  return {
    // 字典
    branchDict: state.dict.branch,
    // branchDict: institution === twoTeacherInstitution ? state.dict.branch : state[storeName].branchDict,
    // branchCampusDict: institution === twoTeacherInstitution ? buildBranchCampusDict([state[storeName].sKey_modalBranch], state.dict.branchCampus) : buildBranchCampusDict([state[storeName].sKey_modalBranch], state[storeName].twoTeacherBranchCampusDict),
    branchCampusDict: buildBranchCampusDict([state[storeName].sKey_modalBranch], state.dict.branchCampus),
    branchCampusDictOrigin: state.dict.branchCampus,
    productTypeTree,
    classTypeDict: buildClassTypeDict(productTypeTree, sKey_prdType),
    classLevelDict: buildClassLevelDict(productTypeTree, sKey_prdType, state[storeName].sKey_modalClassType),
    gradeDict: state.dict.grade,
    subjectDict: state.dict.subject,
    auxTeacherDict,
    auxTeacherDictOrigin,
    productCharacterDict: state.dict.productCharacter,
    // 筛选项值
    sKey_modalBranch: state[storeName].sKey_modalBranch,
    sKey_modalCampus: state[storeName].sKey_modalCampus,
    sKey_modalYear: state[storeName].sKey_modalYear,
    sKey_modalQuarter: state[storeName].sKey_modalQuarter,
    sKey_modalGrade: state[storeName].sKey_modalGrade,
    sKey_modalClassType: state[storeName].sKey_modalClassType,
    sKey_modalClassName: state[storeName].sKey_modalClassName,
    sKey_modalSubject: state[storeName].sKey_modalSubject,
    sKey_modalClassLevel: state[storeName].sKey_modalClassLevel,
    sKey_modalTeacher: state[storeName].sKey_modalTeacher,
    sKey_modal_priceStatus: state[storeName].sKey_modal_priceStatus,
    sKey_modalProductCharacter: state[storeName].sKey_modalProductCharacter,
    // 其他
    sKey_prdType,
  };
};

export default connect(mapStateToProps)(Search);
