import React from 'react';
import { connect } from 'dva';
import Modal from 'COMPONENT/Modal';
import PopTip from 'COMPONENT/PopTip';
import Button from 'COMPONENT/Button';
import SearchSame from './Search/same.js';
import SearchDiff from './Search/diff.js';
import CourseDesk from './CourseDesk';
import ModalSelectCourseTime from 'BLOC/ModalSelectCourseTime';
import './style.less';
import { storeName, SAME_VAL } from '../store/config';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import NP from 'number-precision';

const ModalFooterBtn = (props) => {
  const { content, popTipVisible, onClose, onOk, onCancel, confirmBtnDisabled, confirmBtnText, isCenter } = props;
  return (
    <div style={{ display: 'flex', justifyContent: isCenter ? 'center' : 'flex-end' }}>
      <PopTip icon="error" content={content} visible={popTipVisible} onClose={onClose} maxWidth={200}>
        <Button type="primary" onClick={onOk} disabled={confirmBtnDisabled}>
          {confirmBtnText}
        </Button>
      </PopTip>
      <Button onClick={onCancel}>取消</Button>
    </div>
  );
};

const SelectedPriceTotal = (totalAmount) => {
  return (selectedKeys, list) => {
    return (
      <div style={{ textAlign: 'left' }}>
        <div>课时费用总计{totalAmount}元</div>
        <div>
          <span style={{ color: 'red' }}>注：补报历史课程最多只能报读今天之前15天内的最近两节课程</span>
        </div>
      </div>
    );
  };
};

class ModalChangeClass extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      name: '转班',
      loading: false,
      btnTipContent: '该班剩余名额不足，请重新选择班级',
      toggle_btnTip: false,
      totalAmount: 0,
      showConfirmTip: false,
      confirmTip: '选中学生剩余资金不足，请确认是否转班',
      showTipModal: false,
    };
    this.selectedClassId = null;
    this.init = this.init.bind(this);
    this.loadData = this.loadData.bind(this);
    this.getChangeClassOaRecord = this.getChangeClassOaRecord.bind(this);
    this.updatePage = this.updatePage.bind(this);
    this.changePage = this.changePage.bind(this);
    this.search = this.search.bind(this);
    this.reset = this.reset.bind(this);
    this.updateSelectedRowKeys = this.updateSelectedRowKeys.bind(this);
    this.onConfirm = this.onConfirm.bind(this);
    this.closeTips = this.closeTips.bind(this);
    this.switchModalSelectCourseTime = this.switchModalSelectCourseTime.bind(this);
    this.onSelectedCourseChange = this.onSelectedCourseChange.bind(this);
    this.confirmCourseTime = this.confirmCourseTime.bind(this);
    this.noCourseTime = this.noCourseTime.bind(this);
    this.submit = this.submit.bind(this);
    this.submitChange = this.submitChange.bind(this);
    this.destroy = this.destroy.bind(this);
    this.initModalCampus = this.initModalCampus.bind(this);
    this.closeApplyOa = this.closeApplyOa.bind(this);
    this.onApplyOa = this.onApplyOa.bind(this);
    this.getFlag = this.getFlag.bind(this);
  }

  static classPrefix = 'ModalChangeClass';

  componentDidMount() {
    this.init();
  }

  componentWillUnmount() {
    this.destroy();
  }

  async init() {
    this.setState({ loading: true });
    await new Promise(this.initModalCampus);
    await new Promise(this.loadData);
    await new Promise(this.getChangeClassOaRecord);
    this.setState({ loading: false });
  }

  loadData(resolve) {
    this.props.dispatch({
      type: `${storeName}/loadClassList`,
      payload: {
        callback() {
          resolve();
        },
      },
    });
  }

  initModalCampus(resolve) {
    this.props.dispatch({
      type: `${storeName}/initModalCampus`,
      payload: {
        callback() {
          resolve();
        },
      },
    });
  }

  // 获取OA申请记录
  getChangeClassOaRecord(resolve) {
    this.props.dispatch({
      type: `${storeName}/getChangeClassOaRecord`,
      payload: {
        callback() {
          resolve && resolve();
        },
      },
    });
  }

  updatePage(pageCurrent, pageSize) {
    let payload = {};
    if (pageSize && pageSize !== this.props.pageSize) {
      payload.classPageCurrent = 1;
      payload.classPageSize = pageSize;
    } else {
      payload.classPageCurrent = pageCurrent;
    }
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload,
    });
  }

  async changePage(current, pageSize) {
    this.updatePage(current, pageSize);
    this.setState({ loading: true });
    await new Promise(this.loadData);
    this.setState({ loading: false });
  }

  search() {
    this.getChangeClassOaRecord();
    this.changePage(1, this.props.pageSize);
  }

  reset() {
    this.props.dispatch({
      type: `${storeName}/resetModalKeyword`,
    });
  }

  updateSelectedRowKeys(keys) {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: { classSelectedRowKeys: keys },
    });
  }

  onConfirm() {
    this.props.dispatch({
      type: `${storeName}/getFlagsOa`,
      payload: {
        callback: (flag) => {
          if (flag === 0) {
            const index = this.props.selectedRowKeys[0];
            const selectedClass = this.props.list[index];
            this.selectedClassId = selectedClass.id;
            this.switchModalSelectCourseTime();
          } else if (flag === 1) {
            this.setState({
              showTipModal: true,
            });
          } else if (flag === 2) {
            this.onApplyOa();
          }
        },
      },
    });
  }

  getFlag() {
    getFlagsOaByClassIds();
  }

  closeApplyOa() {
    this.setState({ showTipModal: false });
  }

  onApplyOa() {
    this.closeApplyOa();
    this.props.dispatch({
      type: `${storeName}/openApplyModal`,
    });
  }

  switchModalSelectCourseTime() {
    this.setState(
      (prevState) => ({ modalVisible: !prevState.modalVisible }),
      () => {
        if (!this.state.modalVisible) {
          this.selectedClassId = null;
          this.setState({
            totalAmount: 0,
            showConfirmTip: false,
          });
        }
      },
    );
  }

  onSelectedCourseChange(selectedKeys, list) {
    let totalAmount = 0;
    if (selectedKeys.length > 0) {
      totalAmount = selectedKeys.reduce((total, current) => NP.plus(total, list[current]._origin.courseTotalPrice), 0);
    }
    // let showConfirmTip = false;

    let selectClassIndex = this.props.selectedRowKeys[0];
    let productInfo = this.props.productInfo || this.props.modalProduct;
    if (productInfo.id === this.props.list[selectClassIndex].productId) {
      // 同产品通过已选课程总课时与学生剩余课时数比较
      let totalHour = selectedKeys.reduce((total, current) => NP.plus(total, list[current]._origin.courseHours), 0);
      // showConfirmTip = this.props.modalStudentList.some(std => totalHour > std.cpRemainingHour);
    } else {
      // 非同产品通过已选课程总价与学生剩余金额比较
      // showConfirmTip = this.props.modalStudentList.some(std => totalAmount > std.cpRemainingAmount);
    }

    this.setState({
      totalAmount,
      // showConfirmTip,
    });
  }

  confirmCourseTime(item, index, list) {
    this.switchModalSelectCourseTime();
    this.submit(item);
  }

  noCourseTime() {
    this.switchModalSelectCourseTime();
    showConfirm('确认是否进行转班？', this.submit);
  }

  async submit(courseList) {
    this.setState({ loading: true });
    const isSuccess = await new Promise(this.submitChange(courseList));
    if (isSuccess) {
      this.props.onFinish();
    } else {
      this.setState({ loading: false });
    }
  }

  submitChange(courseList) {
    return (resolve) => {
      this.props.dispatch({
        type: `${storeName}/changeClass`,
        payload: {
          courseList,
          callback(params) {
            resolve(params);
          },
        },
      });
    };
  }

  closeTips() {
    this.setState({ toggle_btnTip: false });
  }

  destroy() {
    this.props.dispatch({ type: `${storeName}/resetModalKeyword` });
  }

  render() {
    return (
      <Modal
        title={this.state.name}
        className={`${ModalChangeClass.classPrefix}`}
        loading={this.state.loading}
        width={1100}
        onCancel={this.props.onCancel}
        footer={
          <ModalFooterBtn
            content={this.state.btnTipContent}
            popTipVisible={this.state.toggle_btnTip}
            onClose={this.closeTips}
            onOk={this.onConfirm}
            onCancel={this.props.onCancel}
            confirmBtnText="确定"
            confirmBtnDisabled={this.props.selectedRowKeys.length !== 1}
          />
        }
      >
        {this.props.changeType === SAME_VAL ? (
          <SearchSame classPrefix={`${ModalChangeClass.classPrefix}-bar`} onSearch={this.search} onReset={this.reset} />
        ) : (
          <SearchDiff classPrefix={`${ModalChangeClass.classPrefix}-bar`} onSearch={this.search} onReset={this.reset} />
        )}
        <CourseDesk
          dataSource={this.props.list}
          oaRecord={this.props.oaRecord}
          rowSelection={{
            selectedRowKeys: this.props.selectedRowKeys,
            onChange: this.updateSelectedRowKeys,
          }}
          pagination={{
            total: this.props.total,
            current: this.props.pageCurrent,
            pageSize: this.props.pageSize,
          }}
          onChange={this.changePage}
        />
        {this.state.modalVisible && (
          <ModalSelectCourseTime
            oldClassId={this.props.modalStudentList[0].classId}
            studentId={this.props.modalStudentList[0].studentId}
            classId={this.selectedClassId}
            productType={this.props.productType}
            noDataDirectCallback={this.noCourseTime}
            onCancel={this.switchModalSelectCourseTime}
            onConfirm={this.confirmCourseTime}
            onChange={this.onSelectedCourseChange}
            showConfirmTip={this.state.showConfirmTip}
            confirmTip={this.state.confirmTip}
            footerTipsRender={SelectedPriceTotal(this.state.totalAmount)}
            autoSelectAfter
            noSelectedDisabled
            beforeNowDisabled={true}
            beforeNowDays={14}
            modalType="changeClass"
            beforeNowMaxCourse={2}
          />
        )}
        {this.state.showTipModal && (
          <Modal
            title="温馨提示"
            className="tipsModal"
            loading={this.state.loading}
            width={350}
            cancelBtnShow={this.props.modalStudentList.length === 1}
            onOk={this.onApplyOa}
            onCancel={this.closeApplyOa}
            confirmBtnText={this.props.modalStudentList.length > 1 ? '确定' : '申请'}
          >
            <p style={{ marginLeft: 20, marginRight: 20 }}>
              {this.props.modalStudentList.length > 1
                ? '跨产品或跨学习中心转班，只能对单个学生进行操作！'
                : '当前为跨产品（或跨学习中心或跨产品跨学习中心）转班，需要走审批流程后再操作！'}
            </p>
          </Modal>
        )}
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    productType: state[storeName].sKey_prdType,
    changeType: state[storeName].changeType,
    modalProduct: state[storeName].modalProduct,
    modalStudentList: state[storeName].modalStudentList,
    list: state[storeName].classList,
    total: state[storeName].classListTotal,
    pageSize: state[storeName].classPageSize,
    pageCurrent: state[storeName].classPageCurrent,
    selectedRowKeys: state[storeName].classSelectedRowKeys,
    productInfo: state[storeName].productInfo,
    oaRecord: state[storeName].oaRecord,
  };
};

export default connect(mapStateToProps)(ModalChangeClass);
