import React from 'react';
import { connect } from 'dva';
import { Modal, List, Button, Select, FormItem } from 'COMPONENT';
import { Row, Col } from 'antd';
import { buildBranchCampusDict } from 'MODEL/dict/utils';
import { storeName, SMALL_CLASS_VAL } from '../store/config';
import styles from './style.less';
import { debounce } from 'lodash';

class ModalChangeClassApply extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false,
      showTipModal: false,
      newClassId: null,
      productDict: [],
      classDict: [],
      campus: null,
      productName: null,
    };

    this.init = this.init.bind(this);
    this.reloadData = this.reloadData.bind(this);
    this.onApply = this.onApply.bind(this);
    this.onCloseTipModal = this.onCloseTipModal.bind(this);
    this.updateClass = this.updateClass.bind(this);
    this.getClassList = this.getClassList.bind(this);
    this.getChangeClassOaRecord = this.getChangeClassOaRecord.bind(this);

    this.columns = [
      { title: '处理人', dataIndex: 'employeeName', key: 'employeeName', width: 60 },
      { title: '岗位', dataIndex: 'stationName', key: 'stationName', width: 60 },
      { title: '部门', dataIndex: 'orgName', key: 'orgName', width: 60 },
      { title: '审批结果', dataIndex: 'operateStatusName', key: 'operateStatusName', width: 60 },
      { title: '操作时间', dataIndex: 'operateTimeStr', key: 'operateTimeStr', width: 80 },
      {
        title: '备注',
        dataIndex: 'notes',
        key: 'notes',
        width: 100,
        render(text) {
          return text === '' || !text ? '-' : text;
        },
      },
    ];
  }

  componentDidMount() {
    this.init();
  }

  init() {
    const { list, classSelectedRowKeys } = this.props;
    if (list.length && classSelectedRowKeys.length) {
      this.setState({
        campus: list[classSelectedRowKeys[0]].blCampusId,
        newClassId: list[classSelectedRowKeys[0]].id,
        classDict: list.map((item) => ({
          productName: item.productName,
          text: item.className,
          val: item.id,
        })),
        productDict: list.map((item) => ({
          text: item.productName,
          val: item.id,
        })),
      });
    }
    this.reloadData();
  }

  onApply() {
    const { oaRecord } = this.props;
    if (oaRecord && oaRecord.oaInstanceId) {
      // 有OA记录
      if (oaRecord.auditStatus === 0) {
        // 审核中，撤销申请
        this.abort();
      } else {
        // 重新申请
        this.setState({
          showTipModal: true,
        });
      }
    } else {
      // 没有OA记录，提交申请
      this.submit();
    }
  }

  submit(reSubmit) {
    this.props.dispatch({
      type: `${storeName}/submitChangeClassOa`,
      payload: {
        newClassId: reSubmit ? this.props.oaRecord.changeNewClassVo.id : this.state.newClassId,
        reSubmit,
        callback: () => {
          this.state.showTipModal && this.onCloseTipModal();
          this.reloadData();
        },
      },
    });
  }

  async reloadData() {
    this.setState({ loading: true });
    await new Promise(this.getChangeClassOaRecord);
    this.setState({ loading: false });
  }

  // 获取OA申请记录
  getChangeClassOaRecord(resolve) {
    this.props.dispatch({
      type: `${storeName}/getChangeClassOaRecord`,
      payload: {
        callback() {
          resolve && resolve();
        },
      },
    });
  }

  abort() {
    this.props.dispatch({
      type: `${storeName}/abortChangeClassOa`,
      payload: {
        callback: () => {
          this.reloadData();
        },
      },
    });
  }

  onCloseTipModal() {
    this.setState({
      showTipModal: false,
    });
  }

  /**
   * 搜索
   *
   * @param val {string}
   * @param key {string} 保存字段
   */
  onSearchClass = (val, key) => {
    debounce(this.doSearchClass, 300)(val, key);
  };

  /**
   * 执行搜索
   * tips: 每一行的用户下拉框一个dict，用 index 后缀区分
   *
   * @param val
   * @param index
   * @param key
   */
  doSearchClass = async (val) => {
    await new Promise(
      this.getClassList({
        isInputSearch: true,
        classPageSize: 99,
        sKey_modalClassName: val,
        sKey_modalCampus: this.state.campus,
      }),
    );
    this.setState({
      classDict: this.props.list.map((item) => ({
        productName: item.productName,
        text: item.className,
        val: String(item.id),
      })),
      productDict: this.props.list.map((item) => ({
        text: item.productName,
        val: String(item.id),
      })),
    });
  };

  getClassList(params) {
    return (resolve) => {
      this.props.dispatch({
        type: `${storeName}/loadClassList`,
        payload: {
          ...params,
          callback() {
            resolve();
          },
        },
      });
    };
  }

  updateClass(val) {
    this.setState({
      newClassId: val,
    });
  }

  onSelectCampus(val) {
    this.setState({
      campus: val,
      newClassId: null,
      classDict: [],
      productDict: [],
    });
    this.updateCampus(val);
  }

  updateCampus(val) {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        sKey_modalCampus: val,
      },
    });
  }

  render() {
    let { branchCampusDict, oaRecord, modalStudent } = this.props;
    const { campus, newClassId } = this.state;
    const hasApplyRecord = !!oaRecord.oaInstanceId; // 是否有转班记录
    const isApplying = oaRecord.auditStatus === 0; // 转班审核中
    return (
      <Modal
        title="跨学习中心转班申请"
        loading={this.state.loading}
        width={1100}
        onCancel={this.props.onCancel}
        onOk={this.onApply}
        okText="提交申请"
        okBtnShow={!hasApplyRecord}
        cancelBtnShow={false}
      >
        <Row>
          <Col span={6} className={styles.primaryColor}>
            学生名字：{modalStudent.studentName}
          </Col>
          <Col span={6} className={styles.primaryColor}>
            学生年级：{modalStudent.gradeName}
          </Col>
        </Row>
        <div className={styles.transferOutWrap}>
          <p className={styles.itemTitle}>转出班</p>
          <Row>
            <Col span={8}>
              <p>
                学习中心名称：<span className={styles.primaryColor}>{modalStudent.campusName}</span>
              </p>
            </Col>
            <Col span={8}>
              <p>
                产品名称：<span className={styles.primaryColor}>{modalStudent.productName}</span>
              </p>
            </Col>
            <Col span={8}>
              <p>
                班级名称：<span className={styles.primaryColor}>{modalStudent.className}</span>
              </p>
            </Col>
          </Row>
        </div>
        <div className={styles.transferInWrap}>
          <p className={styles.itemTitle}>转入班</p>
          <div>
            {hasApplyRecord ? (
              <Row>
                <Col span={8}>
                  <p>
                    学习中心名称：<span className={styles.primaryColor}>{oaRecord.changeNewClassVo.blCampusName}</span>
                  </p>
                </Col>
                <Col span={8}>
                  <p>
                    产品名称：<span className={styles.primaryColor}>{oaRecord.changeNewClassVo.productName}</span>
                  </p>
                </Col>
                <Col span={8}>
                  <p>
                    班级名称：<span className={styles.primaryColor}>{oaRecord.changeNewClassVo.name}</span>
                  </p>
                </Col>
              </Row>
            ) : (
              <Row gutter={10}>
                <Col span={8}>
                  <FormItem name="学习中心名称" labelWidth={80} style={{ paddingLeft: 0 }}>
                    <Select
                      placeholder="学习中心名称"
                      dict={branchCampusDict}
                      value={campus}
                      onChange={(val) => this.onSelectCampus(val)}
                    />
                  </FormItem>
                </Col>
                <Col span={8}>
                  <FormItem name="班级名称" labelWidth={80} style={{ paddingLeft: 0 }}>
                    <Select
                      placeholder="班级名称"
                      dict={this.state.classDict}
                      value={newClassId}
                      onChange={(val) => this.updateClass(val)}
                      showSearch={true}
                      noOptionTips="没有找到"
                      searchPlaceholder="输入名称搜索"
                      onSearchKeyChange={(val) => this.onSearchClass(val)}
                      disabled={!campus}
                    />
                  </FormItem>
                </Col>
                <Col span={8}>
                  <FormItem name="产品名称" labelWidth={80} style={{ paddingLeft: 0 }}>
                    <Select
                      placeholder="产品名称"
                      dict={this.state.productDict}
                      value={newClassId}
                      disabled={!!newClassId}
                    />
                  </FormItem>
                </Col>
              </Row>
            )}
          </div>
        </div>

        {hasApplyRecord && (
          <div className={styles.primaryColor}>
            <div className={styles.cancelBtn}>
              <Button type="primary" onClick={this.onApply}>
                {isApplying ? '撤销申请' : '重新申请'}
              </Button>
            </div>
            <div>
              <div className={styles.number}>
                <span>OA单号：{oaRecord.oaInstanceId}</span>
                <span className={styles.statusText}>{oaRecord.auditStatusName}</span>
              </div>
              <List dataSource={oaRecord.workflowOaRecordVoList} columns={this.columns} />
            </div>
          </div>
        )}
        {this.state.showTipModal && (
          <Modal
            title="提示"
            loading={this.state.loading}
            width={350}
            onCancel={this.onCloseTipModal}
            okText="确定"
            onOk={() => this.submit(true)}
          >
            <p>重新发起流程，已通过的转班申请将会失效</p>
          </Modal>
        )}
      </Modal>
    );
  }
}
const mapStateToProps = (state) => {
  return {
    branchCampusDict: buildBranchCampusDict([state[storeName].sKey_modalBranch], state.dict.branchCampus),
    oaRecord: state[storeName].oaRecord,
    modalStudent: state[storeName].modalStudentList[0],
    list: state[storeName].classList,
    classSelectedRowKeys: state[storeName].classSelectedRowKeys,
  };
};
export default connect(mapStateToProps)(ModalChangeClassApply);
