import React from 'react';
import Modal from 'COMPONENT/Modal';
import FormItem from 'COMPONENT/FormItem';
import Select from 'COMPONENT/Select';
import Button from 'COMPONENT/Button';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import { storeName } from '../../store/config';
import { connect } from 'dva';

const ModalFooterBtn = (props) => {
  const { onOk, onCancel, confirmBtnDisabled } = props;
  return (
    <div style={{ display: 'flex', justifyContent: 'flex-end' }}>
      <Button type="primary" onClick={onOk} disabled={confirmBtnDisabled}>
        确定
      </Button>
      <Button onClick={onCancel}>取消</Button>
    </div>
  );
};

class ReasonModal extends React.Component {
  constructor(props) {
    super(props);
  }

  state = {
    detail: null,
    loading: false,
  };

  updateProps = (params) => {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: params,
    });
  };

  updateValue = (type, val) => {
    this.updateProps({ [type]: val ? val : '' });
  };

  /**
   * 请求提交数据
   */
  commit = () => {
    this.props.onFinish();
    this.closeModal();
  };

  closeModal = () => {
    this.setState({ loading: false });
    this.props.onCancel();
    this.props.dispatch({ type: `${storeName}/closeReasonModal` });
  };

  render() {
    const { loading } = this.state;
    let submitDisable = true;
    if (this.props.auditUserDict) {
      submitDisable = this.props.changeReason === '' || this.props.auditUserId === '';
    } else {
      submitDisable = this.props.changeReason === '';
    }

    return (
      <Modal
        loading={loading}
        okText="提交"
        ref={(c) => (this._modal = getWrappedInstance(c))}
        onClose={this.closeModal}
        width={650}
        footer={<ModalFooterBtn onOk={this.commit} onCancel={this.closeModal} confirmBtnDisabled={submitDisable} />}
      >
        <div style={{ paddingTop: '50px' }}>
          <FormItem name="转班原因" required={true}>
            <Select
              placeholder="转班原因"
              dict={this.props.changeClassReasonDict}
              value={this.props.changeReason}
              onChange={(val) => this.updateValue('changeReason', val)}
            />
          </FormItem>
          {this.props.auditUserDict && (
            <FormItem name="审批人" required={true}>
              <Select
                placeholder="审批人"
                dict={this.props.auditUserDict || []}
                value={this.props.auditUserId}
                onChange={(val) => this.updateValue('auditUserId', val)}
              />
            </FormItem>
          )}
        </div>
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    auditUserId: state[storeName].auditUserId,
    changeReason: state[storeName].changeReason,
    changeClassReasonDict: state.dict.changeClassReasonDict,
    auditUserDict: state[storeName].auditUserDict,
  };
};

export default connect(mapStateToProps)(ReasonModal);
