import React from 'react';
import { connect } from 'dva';
import SearchBar from 'COMPONENT/SearchBar';
import Select from 'COMPONENT/Select';
import Input from 'COMPONENT/Input';
import { storeName, TWO_TEACHER_VAL, SMALL_CLASS_VAL } from '../../store/config';
import { buildBranchCampusDict, buildClassLevelDict, buildAuxTcDict } from 'MODEL/dict/utils';
import { loginUser } from 'UTIL/common';
import { xinghuoInstitution } from 'CONST/config';

class Search extends React.Component {
  constructor(props) {
    super(props);
    this.updateProps = this.updateProps.bind(this);
    this.updateValue = this.updateValue.bind(this);
    this.updateCampus = this.updateCampus.bind(this);
    this.isTwoTeacherSym = loginUser.isTwoTeacherSym();
  }

  updateProps(params) {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: params,
    });
  }

  updateValue(type, val) {
    this.updateProps({ [type]: val });
  }

  updateCampus(val) {
    let payload = { sKey_modalCampus: val };
    if (val && this.props.sKey_modalTeacher) {
      let filterAuxTeacherDict = buildAuxTcDict(this.props.sKey_modalBranch, val, this.props.auxTeacherDictOrigin);
      if (filterAuxTeacherDict) {
        let filterTeacherDict = filterAuxTeacherDict.map((item) => (item ? item.options : []));
        let filterValArr = filterTeacherDict.reduce((arr, current) => arr.concat(current), []).map((item) => item.val);
        if (filterValArr.indexOf(this.props.sKey_modalTeacher) === -1) {
          payload.sKey_modalTeacher = null;
        }
      }
    }
    if (this.props.isXHSmallClass === true) {
      const dict = this.props.twoTeacherBranchCampusDict;
      dict.forEach((value) => {
        if (
          value.options.some((campus) => {
            return campus.val === val;
          })
        ) {
          payload.sKey_modalBranch = value.val;
        }
      });
    }
    this.updateProps(payload);
  }

  render() {
    return (
      <SearchBar
        className={`${this.props.classPrefix}-search`}
        onSearch={this.props.onSearch}
        onReset={this.props.onReset}
      >
        <Input
          placeholder="班级名称"
          value={this.props.sKey_modalClassName}
          onChange={(e) => this.updateValue('sKey_modalClassName', e.target.value)}
        />
        <Select
          placeholder="学习中心"
          dict={this.props.branchCampusDict}
          value={this.props.sKey_modalCampus}
          onChange={this.updateCampus}
          disabled={this.props.studentList.length > 1}
          showSearch
        />
        <Select
          placeholder="班级层次"
          dict={this.props.classLevelDict}
          value={this.props.sKey_modalClassLevel}
          onChange={(val) => this.updateValue('sKey_modalClassLevel', val)}
        />
        <Select
          placeholder="科目"
          dict={this.props.subjectDict}
          value={this.props.sKey_modalSubject}
          onChange={(val) => this.updateValue('sKey_modalSubject', val)}
        />
        <Select
          placeholder="任课老师"
          dict={this.props.auxTeacherDict}
          value={this.props.sKey_modalTeacher}
          onChange={(val) => this.updateValue('sKey_modalTeacher', val)}
          showSearch
        />
        {this.props.isXHSmallClass && (
          <Select
            placeholder="产品组"
            dict={this.props.productGroupDict}
            value={this.props.sKey_modalProductGroup}
            onChange={(val) => this.updateValue('sKey_modalProductGroup', val)}
          />
        )}
      </SearchBar>
    );
  }
}

const mapStateToProps = (state) => {
  const { institution } = state.account.user;
  const isXHSmallClass = institution === xinghuoInstitution && state[storeName].sKey_prdType === SMALL_CLASS_VAL;
  // 按学习中心过滤双师辅导老师/小班任课老师字典
  let auxTeacherDictOrigin =
    state[storeName].sKey_prdType === TWO_TEACHER_VAL
      ? state.dict.branchTwoTcAuxTc
      : state.dict.authBranchSmallClassAuxTc;
  let auxTeacherDict = buildAuxTcDict(
    state[storeName].sKey_modalBranch,
    state[storeName].sKey_modalCampus,
    auxTeacherDictOrigin,
  );
  return {
    twoTeacherBranchCampusDict: state[storeName].twoTeacherBranchCampusDict,
    branchCampusDict: isXHSmallClass
      ? state[storeName].twoTeacherBranchCampusDict
      : buildBranchCampusDict([state[storeName].sKey_modalBranch], state.dict.branchCampus),
    classLevelDict: buildClassLevelDict(
      state.dict.productTypeTree,
      state[storeName].sKey_prdType,
      state[storeName].sKey_modalClassType,
    ),
    subjectDict: state.dict.subject,
    auxTeacherDict,
    auxTeacherDictOrigin,
    productGroupDict: state.dict.productGroup ? state.dict.productGroup['SMALL_CLASS'] : null,
    sKey_modalClassName: state[storeName].sKey_modalClassName,
    sKey_modalCampus: state[storeName].sKey_modalCampus,
    sKey_modalClassLevel: state[storeName].sKey_modalClassLevel,
    sKey_modalSubject: state[storeName].sKey_modalSubject,
    sKey_modalTeacher: state[storeName].sKey_modalTeacher,
    isXHSmallClass,
    selectedRowKeys: state[storeName].selectedRowKeys,
    sKey_modalProductGroup: state[storeName].sKey_modalProductGroup,
    studentList: state[storeName].studentList,
  };
};

export default connect(mapStateToProps)(Search);
