import React from 'react';
import { connect } from 'dva';
import Modal from 'COMPONENT/Modal';
import PopTip from 'COMPONENT/PopTip';
import Button from 'COMPONENT/Button';
import SearchSame from './Search/same.js';
import SearchDiff from './Search/diff.js';
import CourseDesk from './CourseDesk';
import ModalSelectCourseTime from 'BLOC/ModalSelectCourseTime';
import styles from './styles.module.less';
import { storeName, SAME_VAL } from '../store/config';
import NP from 'number-precision';
import ReasonModal from './ReasonModal';

const ModalFooterBtn = (props) => {
  const { content, popTipVisible, onClose, onOk, onCancel, confirmBtnDisabled } = props;
  return (
    <div style={{ display: 'flex', justifyContent: 'flex-end' }}>
      <PopTip icon="error" content={content} visible={popTipVisible} onClose={onClose} maxWidth={200}>
        <Button type="primary" onClick={onOk} disabled={confirmBtnDisabled}>
          确定
        </Button>
      </PopTip>
      <Button onClick={onCancel}>取消</Button>
    </div>
  );
};

const SelectedPriceTotal = (totalAmount) => {
  return (selectedKeys, list) => {
    return (
      <div style={{ textAlign: 'left' }}>
        <div>课时费用总计{totalAmount}元</div>
      </div>
    );
  };
};

class ModalChangeClass extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      name: '转班',
      // loading: false,
      btnTipContent: '该班剩余名额不足，请重新选择班级',
      toggle_btnTip: false,
      totalAmount: 0,
      showConfirmTip: false,
      confirmTip: '选中学生剩余资金不足，请确认是否转班',
      selectedCourseList: [],
    };
    this.selectedClassId = null;
    this.init = this.init.bind(this);
    this.loadData = this.loadData.bind(this);
    this.updatePage = this.updatePage.bind(this);
    this.changePage = this.changePage.bind(this);
    this.search = this.search.bind(this);
    this.reset = this.reset.bind(this);
    this.updateSelectedRowKeys = this.updateSelectedRowKeys.bind(this);
    this.onConfirm = this.onConfirm.bind(this);
    this.closeTips = this.closeTips.bind(this);
    this.switchModalSelectCourseTime = this.switchModalSelectCourseTime.bind(this);
    this.onSelectedCourseChange = this.onSelectedCourseChange.bind(this);
    this.confirmCourseTime = this.confirmCourseTime.bind(this);
    // this.noCourseTime = this.noCourseTime.bind(this);
    this.submit = this.submit.bind(this);
    this.submitChange = this.submitChange.bind(this);
    this.destroy = this.destroy.bind(this);
    this.initModalCampus = this.initModalCampus.bind(this);
    this.confirmReasonAuditor = this.confirmReasonAuditor.bind(this);
    this.openReasonModal = this.openReasonModal.bind(this);
    this.showLoading = this.showLoading.bind(this);
    this.hideLoading = this.hideLoading.bind(this);
  }

  static classPrefix = 'ModalChangeClass';

  componentDidMount() {
    this.init();
  }

  componentWillUnmount() {
    this.destroy();
  }

  async init() {
    this.showLoading();
    // await new Promise(this.initModalCampus)
    await new Promise(this.loadData);
    this.hideLoading();
  }

  loadData(resolve) {
    this.showLoading();
    this.props.dispatch({
      type: `${storeName}/loadClassList`,
      payload: {
        callback() {
          resolve();
        },
      },
    });
    this.hideLoading();
  }

  initModalCampus(resolve) {
    this.props.dispatch({
      type: `${storeName}/initModalCampus`,
      payload: {
        callback() {
          resolve();
        },
      },
    });
  }

  updatePage(pageCurrent, pageSize) {
    let payload = {};
    if (pageSize && pageSize !== this.props.pageSize) {
      payload.classPageCurrent = 1;
      payload.classPageSize = pageSize;
    } else {
      payload.classPageCurrent = pageCurrent;
    }
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload,
    });
  }

  async changePage(current, pageSize) {
    this.updatePage(current, pageSize);
    this.showLoading();
    await new Promise(this.loadData);
    this.hideLoading();
  }

  search() {
    this.changePage(1, this.props.pageSize);
  }

  reset() {
    this.props.dispatch({
      type: `${storeName}/resetModalKeyword`,
    });
  }

  updateSelectedRowKeys(keys) {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        classSelectedRowKeys: keys,
        selectedCourseList: [],
      },
    });
  }

  onConfirm() {
    let index = this.props.selectedRowKeys[0];
    let selectedClass = this.props.list[index];
    if (selectedClass.maxNum - selectedClass.realNum >= this.props.modalStudentList.length) {
      this.selectedClassId = selectedClass.id;
      this.switchModalSelectCourseTime();
    } else {
      this.setState({ toggle_btnTip: true });
    }
  }

  switchModalSelectCourseTime() {
    this.setState(
      (prevState) => ({ modalVisible: !prevState.modalVisible }),
      () => {
        if (!this.state.modalVisible) {
          this.selectedClassId = null;
          this.setState({
            totalAmount: 0,
            showConfirmTip: false,
          });
        }
      },
    );
  }

  onSelectedCourseChange(selectedKeys, list) {
    let totalAmount = 0;
    if (selectedKeys.length > 0) {
      totalAmount = selectedKeys.reduce((total, current) => NP.plus(total, list[current]._origin.courseTotalPrice), 0);
    }
    let showConfirmTip = false;

    // 同产品、非同产品 都通过已选课程总课时与学生剩余课时数比较
    let totalHour = selectedKeys.reduce((total, current) => NP.plus(total, list[current]._origin.courseHours), 0);
    showConfirmTip = this.props.modalStudentList.some((std) => totalHour > std.cpRemainingHour);

    this.setState({
      totalAmount,
      showConfirmTip,
    });
  }

  confirmCourseTime(item, index, list) {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: { selectedCourseList: item },
    });
    this.openReasonModal();
  }

  showLoading() {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        changeClassDataLoading: true,
      },
    });
  }

  hideLoading() {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        changeClassDataLoading: false,
      },
    });
    this.setState({
      modalVisible: false,
    });
  }

  openReasonModal() {
    new Promise((resolve) => {
      this.props.dispatch({
        type: `${storeName}/openReasonModal`,
        payload: {
          resolve,
        },
      });
    }).finally(() => {
      this.hideLoading();
    });
  }

  confirmReasonAuditor() {
    this.submit();
    this.switchModalSelectCourseTime();
  }

  async submit() {
    this.showLoading();
    const isSuccess = await new Promise(this.submitChange());
    if (isSuccess) {
      this.props.onFinish();
    }
    this.hideLoading();
  }

  submitChange() {
    return (resolve) => {
      this.props.dispatch({
        type: `${storeName}/changeClass`,
        payload: {
          callback(params) {
            resolve(params);
          },
        },
      });
    };
  }

  closeTips() {
    this.setState({ toggle_btnTip: false });
  }

  destroy() {
    this.props.dispatch({ type: `${storeName}/resetModalKeyword` });
  }

  render() {
    return (
      <Modal
        title={this.state.name}
        className={`${ModalChangeClass.classPrefix}`}
        loading={this.props.changeClassDataLoading}
        width={1100}
        onCancel={this.props.onCancel}
        footer={
          <div className={styles.footer}>
            <div className={styles.footerLeft}>
              <span>注：产品组为自习/在线A/在线B的小班，只能在相同产品组间进行转班和调课</span>
            </div>
            <div className={styles.footerRight}>
              <ModalFooterBtn
                content={this.state.btnTipContent}
                popTipVisible={this.state.toggle_btnTip}
                onClose={this.closeTips}
                onOk={this.onConfirm}
                onCancel={this.props.onCancel}
                confirmBtnDisabled={this.props.selectedRowKeys.length !== 1}
              />
            </div>
          </div>
        }
      >
        {this.props.changeType === SAME_VAL ? (
          <SearchSame classPrefix={`${ModalChangeClass.classPrefix}-bar`} onSearch={this.search} onReset={this.reset} />
        ) : (
          <SearchDiff classPrefix={`${ModalChangeClass.classPrefix}-bar`} onSearch={this.search} onReset={this.reset} />
        )}
        <CourseDesk
          productType={this.props.productType}
          dataSource={this.props.list}
          rowSelection={{
            selectedRowKeys: this.props.selectedRowKeys,
            onChange: this.updateSelectedRowKeys,
          }}
          pagination={{
            total: this.props.total,
            current: this.props.pageCurrent,
            pageSize: this.props.pageSize,
          }}
          onChange={this.changePage}
        />
        {this.state.modalVisible && (
          <ModalSelectCourseTime
            classId={this.selectedClassId}
            productType={this.props.productType}
            noDataDirectCallback={this.openReasonModal}
            onCancel={this.switchModalSelectCourseTime}
            onConfirm={this.confirmCourseTime}
            onChange={this.onSelectedCourseChange}
            showConfirmTip={this.state.showConfirmTip}
            confirmTip={this.state.confirmTip}
            footerTipsRender={SelectedPriceTotal(this.state.totalAmount)}
            autoSelectAfter
            noSelectedDisabled
          />
        )}
        {this.props.reaSonModalTrigger && (
          <ReasonModal onFinish={this.confirmReasonAuditor} onCancel={this.hideLoading} />
        )}
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    productType: state[storeName].sKey_prdType,
    changeType: state[storeName].changeType,
    modalProduct: state[storeName].modalProduct,
    modalStudentList: state[storeName].modalStudentList,
    list: state[storeName].classList,
    total: state[storeName].classListTotal,
    pageSize: state[storeName].classPageSize,
    pageCurrent: state[storeName].classPageCurrent,
    selectedRowKeys: state[storeName].classSelectedRowKeys,
    productInfo: state[storeName].productInfo,
    reaSonModalTrigger: state[storeName].reaSonModalTrigger,
    isBatchChangeClass: state[storeName].selectedRowKeys.length > 1,
    changeClassDataLoading: state[storeName].changeClassDataLoading,
  };
};

export default connect(mapStateToProps)(ModalChangeClass);
