import React from 'react';
import { connect } from 'dva';
import { SearchButton, Select, Input } from 'COMPONENT';
import { storeName, productTypeAllow, TWO_TEACHER_VAL, SMALL_CLASS_VAL } from '../store/config';
import {
  buildBranchCampusDict,
  buildProductSeriesDict,
  buildClassTypeDict,
  buildClassLevelDict,
  buildAuxTcDict,
} from 'MODEL/dict/utils';
import { loginUser } from 'UTIL/common';

class Search extends React.Component {
  constructor(props) {
    super(props);

    this.updateProps = this.updateProps.bind(this);
    this.updateValue = this.updateValue.bind(this);
    this.updateBranch = this.updateBranch.bind(this);
    this.updateCampus = this.updateCampus.bind(this);
    this.updateProductType = this.updateProductType.bind(this);
    this.updateClassType = this.updateClassType.bind(this);
  }

  updateProps(params) {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: params,
    });
  }

  updateValue(type, val) {
    this.updateProps({ [type]: val });
  }

  updateBranch(val) {
    let payload = { sKey_branch: val };

    if (val) {
      let needResetCampus = false;
      if (this.props.sKey_campus) {
        let filterBranchCampusDict = buildBranchCampusDict([val], this.props.branchCampusDictOrigin);
        let filterCampusDict = filterBranchCampusDict.map((item) => (item ? item.options : []));
        let filterValArr = filterCampusDict.reduce((arr, current) => arr.concat(current), []).map((item) => item.val);
        if (filterValArr.indexOf(this.props.sKey_campus) === -1) {
          payload.sKey_campus = null;
          needResetCampus = true;
        }
      }
      if (this.props.sKey_modalTeacher) {
        let sKey_campus = needResetCampus ? null : this.props.sKey_campus;
        let auxTcDict = buildAuxTcDict(val, sKey_campus, this.props.auxTeacherDictOrigin);
        if (auxTcDict) {
          let filterTeacherDict = auxTcDict.map((item) => (item ? item.options : []));
          let filterValArr = filterTeacherDict
            .reduce((arr, current) => arr.concat(current), [])
            .map((item) => item.val);
          if (filterValArr.indexOf(this.props.sKey_modalTeacher) === -1) {
            payload.sKey_modalTeacher = null;
          }
        }
      }
    }

    this.updateProps(payload);
  }

  updateCampus(val) {
    let payload = { sKey_campus: val };

    if (val && this.props.sKey_teacher) {
      let auxTcDict = buildAuxTcDict(this.props.sKey_branch, val, this.props.auxTeacherDictOrigin);
      if (auxTcDict) {
        let filterTeacherDict = auxTcDict.map((item) => (item ? item.options : []));
        let filterValArr = filterTeacherDict.reduce((arr, current) => arr.concat(current), []).map((item) => item.val);
        if (filterValArr.indexOf(this.props.sKey_teacher) === -1) {
          payload.sKey_teacher = null;
        }
      }
    }

    this.updateProps(payload);
  }

  /**
   * 选择产品类型
   * 双师系统：产品类型为`小班`时，产品组可选，否则，产品组不可选
   *
   * @param val
   */
  updateProductType(val) {
    let payload = { sKey_prdType: val };

    if (val) {
      const { sKey_courseSeries, sKey_classType, sKey_classLevel, productTypeTree, sKey_teacher } = this.props;

      if (sKey_courseSeries) {
        const _productSeriesDict = buildProductSeriesDict(productTypeTree, val) || [];
        // 是否需要清空产品系列
        if (_productSeriesDict.findIndex((item) => item.val === sKey_courseSeries) === -1) {
          payload.sKey_courseSeries = null;
        }
      }

      let needResetClassType = false;
      if (sKey_classType) {
        const _classTypeDict = buildClassTypeDict(productTypeTree, val) || [];
        // 是否需要清空班级类型
        if (_classTypeDict.findIndex((item) => item.val === sKey_classType) === -1) {
          payload.sKey_classType = null;
          needResetClassType = true;
        }
      }

      if (needResetClassType && sKey_classLevel) {
        // 是否需要清空班级层次
        payload.sKey_classLevel = null;
      }

      if (sKey_teacher) {
        payload.sKey_teacher = null;
      }
    }

    if (val !== SMALL_CLASS_VAL) {
      payload.sKey_prdGroup = null;
    }

    this.updateProps(payload);
  }

  updateClassType(val) {
    let payload = { sKey_classType: val };

    if (val && this.props.sKey_classLevel) {
      const { sKey_prdType, sKey_classLevel, productTypeTree } = this.props;
      if (sKey_classLevel) {
        const _classLevelDict = buildClassLevelDict(productTypeTree, sKey_prdType, val) || [];
        // 是否需要清空班级层次
        if (_classLevelDict.findIndex((item) => item.val === sKey_classLevel) === -1) {
          payload.sKey_classLevel = null;
        }
      }
    }

    this.updateProps(payload);
  }

  render() {
    return (
      <SearchButton
        searchBtnLoading={this.props.searchBtnLoading}
        onSearch={this.props.onSearch}
        onReset={this.props.onReset}
      >
        <Input
          placeholder="班级名称"
          value={this.props.sKey_ClassName}
          onChange={(e) => this.updateValue('sKey_ClassName', e.target.value)}
        />
        <Input
          placeholder="学生姓名"
          value={this.props.sKey_stdName}
          maxLength={10}
          onChange={(e) => this.updateValue('sKey_stdName', e.target.value)}
        />
        <Select
          placeholder="分公司"
          dict={this.props.branchDict}
          value={this.props.sKey_branch}
          onChange={this.updateBranch}
          allowClear={false}
          showSearch
        />
        <Select
          placeholder="学习中心"
          dict={this.props.branchCampusDict}
          value={this.props.sKey_campus}
          onChange={this.updateCampus}
          showSearch
        />
        <Select
          placeholder="产品类型"
          dict={this.props.productTypeTree}
          value={this.props.sKey_prdType}
          onChange={this.updateProductType}
          allowClear={false}
        />
        <Select
          placeholder="产品组"
          dict={this.props.productGroupDict}
          value={this.props.sKey_prdGroup}
          onChange={(val) => this.updateValue('sKey_prdGroup', val)}
          disabled={this.props.sKey_prdType !== SMALL_CLASS_VAL}
          allowClear={true}
        />
        <Select
          placeholder="产品年份"
          dict={this.props.yearDict}
          value={this.props.sKey_year}
          onChange={(val) => this.updateValue('sKey_year', val)}
        />
        <Select
          placeholder="产品季度"
          dict={this.props.quarterDict}
          value={this.props.sKey_quarter}
          onChange={(val) => this.updateValue('sKey_quarter', val)}
        />
        <Select
          placeholder="产品性质"
          dict={this.props.productCharacterDict}
          value={this.props.sKey_productCharacter}
          onChange={(val) => this.updateValue('sKey_productCharacter', val)}
        />
        <Select
          placeholder="班级类型"
          dict={this.props.classTypeDict}
          value={this.props.sKey_classType}
          onChange={this.updateClassType}
        />
        <Select
          placeholder="班级层次"
          dict={this.props.classLevelDict}
          value={this.props.sKey_classLevel}
          onChange={(val) => this.updateValue('sKey_classLevel', val)}
        />
        <Select
          placeholder="产品系列"
          dict={this.props.productSeriesDict}
          value={this.props.sKey_courseSeries}
          onChange={(val) => this.updateValue('sKey_courseSeries', val)}
        />
        <Select
          placeholder="年级"
          dict={this.props.gradeDict}
          value={this.props.sKey_grade}
          onChange={(val) => this.updateValue('sKey_grade', val)}
        />
        <Select
          placeholder="科目"
          dict={this.props.subjectDict}
          value={this.props.sKey_subject}
          onChange={(val) => this.updateValue('sKey_subject', val)}
        />
        <Select
          placeholder="任课老师"
          dict={this.props.auxTeacherDict}
          value={this.props.sKey_teacher}
          onChange={(val) => this.updateValue('sKey_teacher', val)}
          showSearch
        />
      </SearchButton>
    );
  }
}

let productTypeTreeStorage = null;

const mapStateToProps = (state) => {
  // 过滤与产品类型相关字典
  if (!productTypeTreeStorage && state.dict.productTypeTree) {
    productTypeTreeStorage = state.dict.productTypeTree.filter((item) => productTypeAllow.includes(item.val));
  }
  let sKey_prdType = state[storeName].sKey_prdType;
  // 按学习中心过滤双师辅导老师/小班任课老师字典
  let auxTeacherDictOrigin =
    sKey_prdType === TWO_TEACHER_VAL ? state.dict.branchTwoTcAuxTc : state.dict.branchSmallClassAuxTc;
  let auxTeacherDict = buildAuxTcDict(state[storeName].sKey_branch, state[storeName].sKey_campus, auxTeacherDictOrigin);

  return {
    // 数据字典
    branchDict: state.dict.branch,
    branchCampusDict: buildBranchCampusDict([state[storeName].sKey_branch], state.dict.branchCampus),
    branchCampusDictOrigin: state.dict.branchCampus,
    productTypeTree: productTypeTreeStorage,
    productSeriesDict: buildProductSeriesDict(productTypeTreeStorage, sKey_prdType),
    classTypeDict: buildClassTypeDict(productTypeTreeStorage, sKey_prdType),
    classLevelDict: buildClassLevelDict(productTypeTreeStorage, sKey_prdType, state[storeName].sKey_classType),
    yearDict: state.dict.year,
    quarterDict: state.dict.quarter,
    gradeDict: state.dict.grade,
    subjectDict: state.dict.subject,
    auxTeacherDict,
    auxTeacherDictOrigin,
    productCharacterDict: state.dict.productCharacter,
    productGroupDict: state.dict.productGroup ? state.dict.productGroup['SMALL_CLASS'] : null,
    // 筛选项值
    sKey_stdName: state[storeName].sKey_stdName,
    sKey_branch: state[storeName].sKey_branch,
    sKey_campus: state[storeName].sKey_campus,
    sKey_prdType: sKey_prdType,
    sKey_prdGroup: state[storeName].sKey_prdGroup,
    sKey_classType: state[storeName].sKey_classType,
    sKey_classLevel: state[storeName].sKey_classLevel,
    sKey_year: state[storeName].sKey_year,
    sKey_quarter: state[storeName].sKey_quarter,
    sKey_grade: state[storeName].sKey_grade,
    sKey_subject: state[storeName].sKey_subject,
    sKey_teacher: state[storeName].sKey_teacher,
    sKey_productCharacter: state[storeName].sKey_productCharacter,
    sKey_courseSeries: state[storeName].sKey_courseSeries,
    sKey_ClassName: state[storeName].sKey_ClassName,
  };
};

export default connect(mapStateToProps)(Search);
