import React from 'react';
import { connect } from 'dva';
import { Page, List, Button, TempStorage } from 'COMPONENT';
import Search from './Search';
import ModalChangeClass from './ModalChangeClass';
import { storeName, SAME_VAL, DIFF_VAL } from './store/config';
import style from './index.less';
import { clone } from 'lodash';
import { xinghuoInstitution } from 'CONST/config';

const { COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;

const classPrefix = 'ChangeClass';

class ChangeClass extends React.Component {
  constructor(props) {
    super(props);

    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.initDefault = this.initDefault.bind(this);
    this.initParams = this.initParams.bind(this);
    this.loadData = this.loadData.bind(this);
    this.refreshData = this.refreshData.bind(this);
    this.updatePage = this.updatePage.bind(this);
    this.changePage = this.changePage.bind(this);
    this.search = this.search.bind(this);
    this.reset = this.reset.bind(this);
    this.selectStudents = this.selectStudents.bind(this);
    this.changeClass = this.changeClass.bind(this);
    this.batchChangeClass = this.batchChangeClass.bind(this);
    this.closeModalChangeClass = this.closeModalChangeClass.bind(this);
    this.changeFinish = this.changeFinish.bind(this);
    this.destroy = this.destroy.bind(this);
    this.deleteSelectedStd = this.deleteSelectedStd.bind(this);
    this.tableRowClick = this.tableRowClick.bind(this);
    this.cleanAllStudent = this.cleanAllStudent.bind(this);

    this.state = {
      name: '转班',
      loading: false,
    };

    this.columns = [
      { title: '学生名称', dataIndex: 'studentName', key: 'studentName', width: COL_WIDTH_M, fixed: 'left' },
      { title: '学生编号', dataIndex: 'studentId', key: 'studentId', width: COL_WIDTH_L, fixed: 'left' },
      { title: '所属分公司', dataIndex: 'branchName', key: 'branchName', width: COL_WIDTH_M },
      { title: '所属学习中心', dataIndex: 'campusName', key: 'campusName', width: COL_WIDTH_M },
      { title: '产品名称', dataIndex: 'productName', key: 'productName', width: COL_WIDTH_XL },
      { title: '班级名称', dataIndex: 'className', key: 'className', width: COL_WIDTH_XL },
      { title: '产品组', dataIndex: 'productGroupName', key: 'productGroupName', width: COL_WIDTH_M },
      { title: '班级层次', dataIndex: 'classLevelName', key: 'classLevelName', width: COL_WIDTH_M },
      { title: '任课老师', dataIndex: 'teacher', key: 'teacher', width: COL_WIDTH_M },
      { title: '开始日期', dataIndex: 'beginDate', key: 'beginDate', width: COL_WIDTH_M },
      { title: '上课时间', dataIndex: 'courseTime', key: 'courseTime', width: COL_WIDTH_M },
      {
        title: '操作',
        key: 'action',
        width: COL_WIDTH_M,
        fixed: 'right',
        getActionConfig: (text, record, index) => [
          {
            menuText: '转班',
            menus: [
              {
                text: '同产品转班',
                onClick: () => {
                  this.changeClass(SAME_VAL, record, index);
                },
                disabled: props.btnAuthChangeBySame,
              },
              {
                text: '跨产品转班',
                onClick: () => {
                  this.changeClass(DIFF_VAL, record, index);
                },
                disabled: props.btnAuthChangeByDiff,
              },
            ],
          },
        ],
      },
    ];
  }

  componentDidMount() {
    this.init();
  }

  componentWillUnmount() {
    this.destroy();
  }

  // 初始化
  async init() {
    this.setState({ loading: true });
    const { dispatch, pageParams } = this.props;
    await new Promise(this.initDict);
    if (!pageParams || !pageParams.studentId) {
      await new Promise(this.initDefault);
    } else {
      // 从小班学生详情页跳转
      await new Promise(this.initParams);
    }
    await new Promise(this.loadData);

    if (this.props.institution === xinghuoInstitution) {
      // 双师批量转班多个分公司权限获取学习中心
      await dispatch({
        type: `${storeName}/getBranchAndChildCampusForClassChangeApi`,
        payload: {},
      });
    }

    this.setState({ loading: false });
  }

  initDict(resolve) {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: [
          'branch',
          'branchCampus',
          'productTypeTree',
          'year',
          'quarter',
          'productCharacter',
          { name: 'productGroup', params: 'SMALL_CLASS' },
          'grade',
          'subject',
          'branchTwoTcAuxTc',
          'branchSmallClassAuxTc',
          'authBranchSmallClassAuxTc',
          'changeClassReasonDict',
        ],
        callback() {
          resolve();
        },
      },
    });
  }

  initDefault(resolve) {
    this.props.dispatch({
      type: `${storeName}/initSKey`,
      payload: {
        callback() {
          resolve();
        },
      },
    });
  }

  initParams(resolve) {
    this.props.dispatch({
      type: `${storeName}/initParamsSKey`,
      payload: {
        params: this.props.pageParams,
        callback: () => {
          this.props.dispatch({
            type: 'main/setPageParams',
            payload: {
              path: '/changeClass',
              params: null,
            },
          });
          resolve();
        },
      },
    });
  }

  loadData(resolve) {
    this.props.dispatch({
      type: `${storeName}/loadData`,
      payload: {
        callback() {
          resolve();
        },
      },
    });
  }

  async refreshData() {
    this.setState({ loading: true });
    await new Promise(this.loadData);
    this.setState({ loading: false });
  }

  updatePage(pageCurrent, pageSize) {
    let payload = {};
    if (pageSize && pageSize !== this.props.pageSize) {
      payload.pageCurrent = 1;
      payload.pageSize = pageSize;
    } else {
      payload.pageCurrent = pageCurrent;
    }
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload,
    });
  }

  changePage({ current, pageSize }) {
    this.updatePage(current, pageSize);
    this.refreshData();
  }

  search() {
    this.changePage({
      current: 1,
      pageSize: this.props.pageSize,
    });
  }

  reset() {
    this.props.dispatch({
      type: `${storeName}/resetKeyword`,
    });
  }

  selectStudents(record, selected, selectedRows) {
    this.props.dispatch({
      type: `${storeName}/selectStdForChange`,
      payload: {
        selected,
        record,
        index: record.key,
        selectedRows,
      },
    });
  }

  changeClass(changeType, record, index) {
    this.props.dispatch({
      type: `${storeName}/singleStdChange`,
      payload: {
        changeType,
        record,
        index,
      },
    });
  }

  batchChangeClass(changeType) {
    this.props.dispatch({
      type: `${storeName}/batchChangeClass`,
      payload: { changeType },
    });
  }

  closeModalChangeClass() {
    this.props.dispatch({ type: `${storeName}/closeModal` });
  }

  changeFinish() {
    const _self = this;
    this.props.dispatch({
      type: `${storeName}/changeFinish`,
      payload: {
        callback() {
          _self.refreshData();
        },
      },
    });
  }

  destroy() {
    this.props.dispatch({
      type: `${storeName}/resetPage`,
    });
  }

  deleteSelectedStd(student, index) {
    this.props.dispatch({
      type: `${storeName}/deleteStudent`,
      payload: {
        student,
        index,
      },
    });
  }

  tableRowClick(record, index, event) {
    let selected = !this.props.selectedRowKeys.includes(index);
    let _selectedRowKeys = clone(this.props.selectedRowKeys);
    if (selected) {
      _selectedRowKeys.push(index);
    } else {
      _selectedRowKeys.filter((item) => item !== index);
    }
    let selectedRows = _selectedRowKeys.map((index) => clone(this.props.list[index]));
    this.selectStudents(record, selected, selectedRows);
  }

  cleanAllStudent() {
    this.props.dispatch({
      type: `${storeName}/cleanAllStudent`,
      payload: {},
    });
  }

  render() {
    const noStudentSelected = this.props.studentList.length === 0;

    return (
      <Page
        name={this.state.name}
        className={style[classPrefix]}
        fitWindowHeight={true}
        loading={this.state.loading}
        route={this.props.route}
        headerToolBar={
          <div>
            <Button onClick={this.cleanAllStudent} disabled={noStudentSelected} style={{ marginRight: 20 }}>
              清空已选
            </Button>
            <Button
              onClick={() => {
                this.batchChangeClass(SAME_VAL);
              }}
              disabled={this.props.studentList.length <= 1}
              style={{ width: 100, marginRight: 20 }}
              type="primary"
            >
              批量转班
            </Button>
            <Search searchBtnLoading={this.state.loading} onSearch={this.search} onReset={this.reset} />
          </div>
        }
        subHeader={
          <TempStorage
            className={style[`${classPrefix}-studentList`]}
            label="已选学生"
            isLabelCount
            source={this.props.studentList}
            optionTextKey="studentName"
            onDelete={this.deleteSelectedStd}
          />
        }
      >
        <List
          dataSource={this.props.list}
          columns={this.columns}
          rowSelection={{
            selectedRowKeys: this.props.selectedRowKeys,
            onSelect: this.selectStudents,
          }}
          pagination={{
            total: this.props.total,
            current: this.props.pageCurrent,
            pageSize: this.props.pageSize,
          }}
          onChange={this.changePage}
          onRowClick={this.tableRowClick}
        />

        {this.props.modalTrigger && (
          <ModalChangeClass onFinish={this.changeFinish} onCancel={this.closeModalChangeClass} />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const btnAuth = state.account.btnAuths || '';
  const { institution } = state.account.user;

  return {
    pageParams: state.main.pageParams['/changeClass'],
    institution,
    btnAuthChangeBySame: btnAuth.indexOf('BTN_CHANGECLASS_SAME_PRODUCT') === -1,
    btnAuthChangeByDiff: btnAuth.indexOf('BTN_CHANGECLASS_ALL_PRODUCT') === -1,
    list: state[storeName].list,
    total: state[storeName].listTotal,
    pageSize: state[storeName].pageSize,
    pageCurrent: state[storeName].pageCurrent,
    selectedRowKeys: state[storeName].selectedRowKeys,
    studentList: state[storeName].studentList,
    modalTrigger: state[storeName].modalTrigger,
  };
};

export default connect(mapStateToProps)(ChangeClass);
