import { storeName, TWO_TEACHER_VAL, SMALL_CLASS_VAL, SAME_VAL, DIFF_VAL } from './config';
import { getDictValByText } from 'UTIL/dict';
import {
  findTwoTeacherClassTwoList,
  getClassChangeMiniClassListXH,
  getClassChangeStudentList,
  studentChangeClassXH,
  getBranchAndChildCampusForClassChange,
  listNodeAuditUsersForChangeClass,
  checkBeforeStudentChangeClass,
  changeClassWithFlow,
} from 'SERVICE/course';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { twoTeacherInstitution, xinghuoInstitution } from 'CONST/config';
import { clone } from 'lodash';
import { errorsHandling, runFunc, buildTeacher } from './utils';

const defaultState = {
  sKey: {
    sKey_ClassName: null,
    sKey_stdName: '',
    sKey_stdId: null, // 通过路由传参
    sKey_branch: null,
    sKey_campus: null,
    sKey_prdType: null,
    sKey_prdGroup: null,
    sKey_classType: null,
    sKey_classLevel: null,
    sKey_courseSeries: null,
    sKey_productCharacter: null,
    sKey_year: null,
    sKey_quarter: null,
    sKey_grade: null,
    sKey_subject: null,
    sKey_teacher: null,
  },
  list: {
    // 列表数据
    list: [],
    listTotal: 0,
    pageSize: 20,
    pageCurrent: 1,
    selectedRowKeys: [],
  },
  changeClassData: {
    changeType: null,
    productInfo: null,
    studentList: [],
    changeClassDataLoading: false,
  },
  changeModalSKey: {
    sKey_modalBranch: null,
    sKey_modalCampus: null,
    sKey_modalYear: null,
    sKey_modalQuarter: null,
    sKey_modalGrade: null,
    sKey_modalClassType: null,
    sKey_modalClassName: '',
    sKey_modalSubject: null,
    sKey_modalClassLevel: null,
    sKey_modalTeacher: null,
    sKey_modal_totalHour: null,
    sKey_modal_price: null,
    sKey_modal_priceStatus: 'same',
    sKey_modalProductGroup: null,
  },
  changeModal: {
    modalTrigger: false,
    modalProduct: null,
    modalStudentList: [],
    classList: [],
    classListTotal: 0,
    classPageSize: 15,
    classPageCurrent: 1,
    classSelectedRowKeys: [],
    modalSKeyStorage: null,
  },
  reaSonModal: {
    selectedCourseList: [],
    reaSonModalTrigger: false,
    auditUserId: '',
    changeReason: '',
    reasonModalStorage: null,
    auditUserDict: null,
    nodeId: '',
  },
};
Object.freeze(defaultState);

export default {
  namespace: storeName,
  state: {
    // index
    ...clone(defaultState.sKey),
    ...clone(defaultState.list),
    ...clone(defaultState.changeClassData),
    branchDict: [],
    twoTeacherBranchCampusDict: [],
    sKeyStorage: null,
    // ModalChangeClass
    ...clone(defaultState.changeModalSKey),
    ...clone(defaultState.changeModal),
    ...clone(defaultState.reaSonModal),
  },
  reducers: {
    updateState(state, action) {
      const { payload } = action;
      return {
        ...state,
        ...payload,
      };
    },

    updateList(state, { payload: { list, total } }) {
      let stdList = list.map((item) => ({
        _origin: item,
        id: `${item.studentId}_${item.classId}`,
        studentName: item.studentName,
        studentId: item.studentId,
        branchId: item.branchId,
        branchName: item.branchName,
        campusId: item.blCampusId,
        campusName: item.blCampusName,
        productId: item.productId,
        productName: item.productName,
        classId: item.classId,
        className: item.className,
        classLevelName: item.classLevelName,
        teacher: item.teacherName || '其他',
        beginDate: item.startDate,
        courseTime: item.courseTime,
        yearId: item.productVersionId,
        quarterId: item.productQuarterId,
        gradeId: item.gradeId,
        classTypeId: item.classTypeId,
        totalHour: item.classTotalHours,
        price: item.price,
        cpRemainingAmount: item.cpRemainingAmount,
        subjectId: item.subjectId,
        cpRemainingHour: item.cpRemainingHour,
        productGroupName: item.productGroupName,
      }));
      let selectedRowKeys = [];
      // 有已选学生的话要初始化`selectedRowKeys`
      if (state.studentList.length > 0) {
        stdList.forEach((std, index) => {
          // 同一个学生只能有一个小班/双师辅班
          if (state.studentList.find((item) => item.id === std.id)) {
            selectedRowKeys.push(index);
          }
        });
      }
      return {
        ...state,
        list: stdList,
        listTotal: total,
        selectedRowKeys,
      };
    },

    updateSelectedRowKeys(state, { payload: { selected, index } }) {
      let selectedRowKeys = clone(state.selectedRowKeys);
      if (selected) {
        selectedRowKeys.unshift(index);
      } else {
        selectedRowKeys = selectedRowKeys.filter((item) => item !== index);
      }
      return {
        ...state,
        selectedRowKeys,
      };
    },

    updateStudentList(state, { payload: { selected, record } }) {
      let studentList = clone(state.studentList);
      if (selected) {
        let std = clone(record);
        delete std.key;
        studentList.unshift(std);
      } else {
        studentList = studentList.filter((item) => item.id !== record.id);
      }
      return {
        ...state,
        studentList,
      };
    },

    updateProduct(state, { payload: { keyName, data } }) {
      return {
        ...state,
        [keyName]: {
          id: data.productId,
          name: data.productName,
          branchId: data.branchId,
          campusId: data.campusId,
          subjectId: data.subjectId,
          yearId: data.yearId,
          quarterId: data.quarterId,
          gradeId: data.gradeId,
          /**
           * 双师辅班的classType在辅班表里由单独字段，可以与对应产品的classType不一样
           * 而可转班列表的查询接口的classType是以产品的classType来查询的，故在双师辅班列表里新增一个prdClassTypeId来表示
           * */
          classTypeId: data._origin.prdClassTypeId || data.classTypeId,
          totalHour: data.totalHour,
          price: data.price,
        },
      };
    },

    updateModalInitData(state, { payload: { changeType, record } }) {
      let modalSKeyStorage = {
        sKey_modalBranch: record.branchId,
        sKey_modalCampus: record.campusId,
        sKey_modalYear: record.yearId,
        sKey_modalQuarter: record.quarterId,
        sKey_modalGrade: record.gradeId,
        sKey_modalClassType: record.classTypeId,
        sKey_modal_totalHour: record.totalHour,
        sKey_modal_price: record.price,
        sKey_modalSubject: record.subjectId,
      };
      return {
        ...state,
        changeType,
        ...clone(modalSKeyStorage),
        modalSKeyStorage,
      };
    },

    updateTwoTcClassList(state, { payload: { list, total, isTwoTeacherSys } }) {
      const classList = list.map((item) => {
        let allowedExcess = item.allowedExcess || 0;
        return {
          id: item.id,
          className: item.name,
          productName: item.productName,
          teachers: buildTeacher(item.teacherName, item.mainTeacherName),
          courseDate: item.startCourseDate,
          realNum: item.realPeople,
          totalNum: item.peopleQuantity + (isTwoTeacherSys ? 0 : allowedExcess),
          classTime: item.classTime,
          everyCourseClassNum: item.everyCourseClassNum,
          classTimeLength: item.classTimeLength,
          maxNum: item.peopleQuantity + allowedExcess,
          productId: item.productId,
        };
      });
      return {
        ...state,
        classList,
        classListTotal: total,
        classSelectedRowKeys: [],
      };
    },

    updateSmallClassList(state, { payload: { list, total, isTwoTeacherSys } }) {
      const classList = list.map((item) => {
        let allowedExcess = item.allowedExcess || 0;
        return {
          id: item.miniClassId,
          className: item.name,
          productName: item.productName,
          teachers: buildTeacher(item.studyManegerName, item.teacherName),
          courseDate: item.startDate,
          realNum: item.miniClassPeopleNum,
          totalNum: item.peopleQuantity + (isTwoTeacherSys ? 0 : allowedExcess),
          classTime: item.classTime,
          everyCourseClassNum: item.everyCourseClassNum,
          classTimeLength: item.classTimeLength,
          maxNum: item.peopleQuantity + allowedExcess,
          productId: item.productId,
          productGroupName: item.productGroupName,
        };
      });
      return {
        ...state,
        classList,
        classListTotal: total,
        classSelectedRowKeys: [],
      };
    },
  },
  effects: {
    // index
    *initSKey({ payload: { callback } }, { select, put }) {
      let initData = yield select((state) => {
        if (state[storeName].sKeyStorage) {
          return state[storeName].sKeyStorage;
        } else {
          let payload = {};
          const { brenchId, campusId, institution } = state.account.user;
          const isGroup = brenchId === campusId;
          payload.sKey_branch = isGroup ? state.dict.branch[0].val : brenchId;
          payload.sKey_campus = isGroup
            ? state.dict.branchCampus.find((item) => item.val === payload.sKey_branch).options[0].val
            : campusId;
          payload.sKey_prdType = institution === twoTeacherInstitution ? TWO_TEACHER_VAL : SMALL_CLASS_VAL;
          const nowYear = new Date().getFullYear().toString();
          payload.sKey_year = getDictValByText(state.dict.year, nowYear) || null;
          return payload;
        }
      });
      yield put({
        type: 'updateState',
        payload: {
          ...initData,
          sKeyStorage: initData,
        },
      });
      runFunc(callback);
    },

    // init with params
    *initParamsSKey({ payload: { params, callback } }, { select, put }) {
      let initData = yield select((state) => {
        const { studentId, studentName } = params;
        const payload = {};
        payload.sKey_stdId = studentId;
        payload.sKey_stdName = studentName;
        payload.sKey_prdType = SMALL_CLASS_VAL;
        return payload;
      });
      yield put({
        type: 'updateState',
        payload: {
          ...initData,
          sKeyStorage: null,
        },
      });
      runFunc(callback);
    },

    *loadData({ payload: { callback } }, { select, put }) {
      const apiParams = yield select((state) => ({
        studentId: state[storeName].sKey_stdId || null,
        studentName: state[storeName].sKey_stdName || null,
        branchId: state[storeName].sKey_branch || null,
        blCampusId: state[storeName].sKey_campus || null,
        productType: state[storeName].sKey_prdType,
        productGroupId: state[storeName].sKey_prdGroup,
        classTypeId: state[storeName].sKey_classType || null,
        classLevel: state[storeName].sKey_classLevel || null,
        productVersionId: state[storeName].sKey_year || null,
        productQuarterId: state[storeName].sKey_quarter || null,
        gradeId: state[storeName].sKey_grade || null,
        subjectId: state[storeName].sKey_subject || null,
        teacherId: state[storeName].sKey_teacher || null,
        productCharacter: state[storeName].sKey_productCharacter,
        courseSeriesId: state[storeName].sKey_courseSeries,
        className: state[storeName].sKey_ClassName || null,
        page: state[storeName].pageCurrent,
        rows: state[storeName].pageSize,
      }));
      const rsp = yield getClassChangeStudentList(apiParams, true).catch(errorsHandling());
      if (rsp && rsp.resultCode === 0) {
        yield put({
          type: 'updateList',
          payload: {
            list: rsp.rows,
            total: rsp.records,
          },
        });
      }
      runFunc(callback);
    },

    *resetKeyword(action, { put, select }) {
      const sKeyStorage = yield select((state) => state[storeName].sKeyStorage);
      yield put({
        type: 'updateState',
        payload: {
          ...clone(defaultState.sKey),
          ...sKeyStorage,
        },
      });
    },

    *resetPage(action, { put, select }) {
      yield put({
        type: 'updateState',
        payload: {
          ...clone(defaultState.sKey),
          ...clone(defaultState.list),
          ...clone(defaultState.changeClassData),
        },
      });
    },

    *resetAllModalData(action, { put }) {
      yield put({
        type: 'updateState',
        payload: {
          ...clone(defaultState.changeClassData),
          ...clone(defaultState.changeModalSKey),
          ...clone(defaultState.changeModal),
          ...clone(defaultState.reaSonModal),
          selectedRowKeys: [],
        },
      });
    },

    *deleteStudent({ payload: { student, index } }, { select, put }) {
      let payload = {};
      payload.selectedRowKeys = yield select((state) => {
        let list = state[storeName].list;
        let selectedRowKeys = clone(state[storeName].selectedRowKeys);
        let selectedList = selectedRowKeys.map((index) => list[index]);
        let checkIndex = selectedList.findIndex((item) => item.id === student.id);
        if (checkIndex > -1) {
          selectedRowKeys.splice(checkIndex, 1);
        }
        return selectedRowKeys;
      });
      payload.studentList = yield select((state) => clone(state[storeName].studentList));
      payload.studentList.splice(index, 1);
      if (payload.studentList.length === 0) {
        payload.productInfo = null;
      }
      yield put({
        type: 'updateState',
        payload,
      });
    },

    *cleanAllStudent({ payload: { callback } }, { put }) {
      yield put({
        type: 'updateState',
        payload: {
          selectedRowKeys: [],
          productInfo: null,
          studentList: [],
        },
      });
      runFunc(callback);
    },

    *selectStdForChange({ payload: { selected, record, index, selectedRows } }, { select, put }) {
      let { productInfo, studentList } = yield select((state) => ({
        productInfo: state[storeName].productInfo,
        studentList: state[storeName].studentList,
      }));
      if (selected) {
        if (productInfo && productInfo.id !== record.productId) {
          // 校验产品是否一致
          return showMessage('请选择报读同产品、同学习中心的学生！', 'warning');
        } else if (productInfo && productInfo.campusId !== record.campusId) {
          // 校验学习中心是否一致
          return showMessage('请选择报读同产品、同学习中心的学生！', 'warning');
        } else {
          // 初始化产品信息
          yield put({
            type: 'updateProduct',
            payload: {
              keyName: 'productInfo',
              data: record,
            },
          });
        }
        // 校验是否已存在该学生
        if (studentList.length > 0 && studentList.find((item) => item.studentId === record.studentId)) {
          return showMessage('转班时不能选择相同学生！', 'warning');
        }
      } else if (studentList.length === 1) {
        // 当取消选择的学生是最后一个时，清空产品信息
        yield put({
          type: 'updateState',
          payload: { productInfo: null },
        });
      }
      yield put({
        type: 'updateSelectedRowKeys',
        payload: { selected, index },
      });
      yield put({
        type: 'updateStudentList',
        payload: { selected, record },
      });
    },

    *singleStdChange({ payload: { changeType, record, index } }, { select, put }) {
      yield put({
        type: 'resetAllModalData',
      });

      yield put({
        type: 'updateProduct',
        payload: {
          keyName: 'modalProduct',
          data: record,
        },
      });
      yield put({
        type: 'updateState',
        payload: {
          modalStudentList: [record],
        },
      });
      yield put({
        type: 'updateModalInitData',
        payload: {
          changeType,
          record,
        },
      });
      yield put({ type: 'openModal' });
    },

    *batchChangeClass({ payload: { changeType } }, { select, put }) {
      let { productInfo, studentList } = yield select((state) => ({
        productInfo: state[storeName].productInfo,
        studentList: state[storeName].studentList,
      }));
      // let subjectId = null;
      // // 双师的时候校验所有已选数据是否相同科目
      // if (loginUser.isTwoTeacherSym()) {
      //   const subjectSet = new Set(studentList.map(std => std.subjectId));
      //   if (subjectSet.size > 1) {
      //     return showMessage('请选择相同科目的学生！', 'error');
      //   } else {
      //     subjectId = Array.from(subjectSet)[0] || null;
      //   }
      // }
      yield put({
        type: 'updateState',
        payload: {
          modalProduct: clone(productInfo),
          modalStudentList: clone(studentList),
        },
      });
      yield put({
        type: 'updateModalInitData',
        payload: {
          changeType,
          record: {
            ...productInfo,
            // subjectId,
          },
        },
      });
      yield put({ type: 'openModal' });
    },

    *openModal(action, { put }) {
      yield put({
        type: 'updateState',
        payload: { modalTrigger: true },
      });
    },
    *openReasonModal({ payload: { resolve } }, { select, put }) {
      const apiParams = yield select(selectedCoursesSelector);
      const selectedRowKeys = yield select((state) => state[storeName].selectedRowKeys);
      try {
        if (selectedRowKeys.length <= 1) {
          const checkRes = yield checkBeforeStudentChangeClass(apiParams).catch(errorsHandling());
          if (!checkRes) {
            yield put({
              type: 'updateState',
              payload: { changeClassDataLoading: false },
            });
          } else {
            if (!checkRes.data) {
              const {
                data: { approvalUsers, nodeId },
              } = yield listNodeAuditUsersForChangeClass(apiParams).catch(errorsHandling());
              const auditUserDict = approvalUsers.map((user) => ({ text: user.userName, val: user.userId }));
              yield put({
                type: 'updateState',
                payload: {
                  auditUserDict,
                  nodeId,
                },
              });
            }

            yield put({
              type: 'updateState',
              payload: {
                reaSonModalTrigger: true,
                changeClassDataLoading: false,
              },
            });
          }
        } else {
          yield put({
            type: 'updateState',
            payload: {
              reaSonModalTrigger: true,
              changeClassDataLoading: false,
            },
          });
        }
      } catch (err) {
        // do nothing
      } finally {
        resolve();
      }
    },
    *closeModal(action, { put }) {
      yield put({
        type: 'updateState',
        payload: clone(defaultState.changeModal),
      });
    },
    *closeReasonModal(action, { put }) {
      yield put({
        type: 'updateState',
        payload: clone(defaultState.reaSonModal),
      });
    },
    // Modal
    *loadClassList({ payload }, { select, put }) {
      let isTwoTeacher = yield select((state) => state[storeName].sKey_prdType === TWO_TEACHER_VAL);
      let effectName = isTwoTeacher ? 'loadTwoTcClassList' : 'loadSmallClassList';
      yield put({
        type: effectName,
        payload,
      });
    },
    *initModalCampus({ payload: { callback } }, { select, put }) {
      const { campusId, branchId } = yield select((state) => {
        if (state[storeName].selectedRowKeys.length > 1) {
          return {
            campusId: state[storeName].sKey_modalCampus,
            branchId: state[storeName].sKey_modalBranch,
          };
        }
        const { institution } = state.account.user;
        const isXHSmallClass = institution === xinghuoInstitution && state[storeName].sKey_prdType === SMALL_CLASS_VAL;
        return {
          campusId: isXHSmallClass
            ? state[storeName].twoTeacherBranchCampusDict[0].options[0].val
            : state[storeName].sKey_modalCampus,
          branchId: isXHSmallClass ? state[storeName].branchDict[0].val : state[storeName].sKey_modalBranch,
        };
      });
      yield put({
        type: 'updateState',
        payload: { sKey_modalCampus: campusId, sKey_modalBranch: branchId },
      });
      runFunc(callback);
    },
    *loadTwoTcClassList({ payload: { callback } }, { select, put }) {
      let apiParams = yield select((state) => {
        let store = state[storeName];
        let isChangeDiffSamePrice = store.changeType === DIFF_VAL && store.sKey_modal_priceStatus === SAME_VAL;
        let params = {
          name: store.sKey_modalClassName || null,
          branchId: store.sKey_modalBranch,
          blCampusId: store.sKey_modalCampus,
          classTypeId: store.sKey_modalClassType,
          productVersion: store.sKey_modalYear,
          productQuarter: store.sKey_modalQuarter,
          gradeId: store.sKey_modalGrade,
          classLevel: store.sKey_modalClassLevel || null,
          subject: store.sKey_modalSubject || null,
          teacherId: store.sKey_modalTeacher || null,
          totalHours: isChangeDiffSamePrice ? store.sKey_modal_totalHour : null,
          price: isChangeDiffSamePrice ? store.sKey_modal_price : null,
          page: store.classPageCurrent,
          rows: store.classPageSize,
          useTo: 'changeClass',
        };
        if (store.changeType === SAME_VAL) {
          params.productId = store.modalProduct.id;
        }
        return params;
      });
      const rsp = yield findTwoTeacherClassTwoList(apiParams, true).catch(errorsHandling());
      if (rsp) {
        const isTwoTeacherSys = yield select((state) => state.account.user.institution === twoTeacherInstitution);
        yield put({
          type: 'updateTwoTcClassList',
          payload: {
            list: rsp.rows,
            total: rsp.records,
            isTwoTeacherSys,
          },
        });
      }
      runFunc(callback);
    },

    *loadSmallClassList({ payload: { callback } }, { select, put }) {
      let apiParams = yield select((state) => {
        let store = state[storeName];
        let isChangeDiffSamePrice = store.changeType === DIFF_VAL && store.sKey_modal_priceStatus === SAME_VAL;
        let params = {
          name: store.sKey_modalClassName,
          branchId: store.sKey_modalBranch,
          blCampusId: store.sKey_modalCampus,
          classTypeId: store.sKey_modalClassType,
          productVersionId: store.sKey_modalYear,
          productQuarterId: store.sKey_modalQuarter,
          gradeId: store.sKey_modalGrade,
          classLevel: store.sKey_modalClassLevel,
          subjectId: store.sKey_modalSubject,
          teacherId: store.sKey_modalTeacher,
          productGroupId: store.sKey_modalProductGroup,
          totalClassHours: isChangeDiffSamePrice ? store.sKey_modal_totalHour : null,
          unitPrice: isChangeDiffSamePrice ? store.sKey_modal_price : null,
          page: store.classPageCurrent,
          rows: store.classPageSize,
        };
        if (store.changeType === SAME_VAL) {
          params.productId = store.modalProduct.id;
        }
        return params;
      });
      const rsp = yield getClassChangeMiniClassListXH(apiParams, true).catch(errorsHandling());
      if (rsp) {
        const isTwoTeacherSys = yield select((state) => state.account.user.institution === twoTeacherInstitution);
        yield put({
          type: 'updateSmallClassList',
          payload: {
            list: rsp.rows,
            total: rsp.records,
            isTwoTeacherSys,
          },
        });
      }
      runFunc(callback);
    },

    *resetModalKeyword(action, { select, put }) {
      let modalSKeyStorage = yield select((state) => state[storeName].modalSKeyStorage);
      yield put({
        type: 'updateState',
        payload: {
          ...clone(defaultState.changeModalSKey),
          ...clone(modalSKeyStorage),
        },
      });
    },

    *resetAuditUserDict(action, { select, put }) {
      yield put({
        type: 'updateState',
        payload: {
          auditUserDict: null,
        },
      });
    },

    *changeClass({ payload: { callback } }, { select, put }) {
      const auditUserDict = yield select((state) => state[storeName].auditUserDict);
      const effectName = auditUserDict ? 'changeClassWithFlow' : 'normalChangeClass';
      yield put({
        type: effectName,
        payload: { callback },
      });
    },

    *changeClassWithFlow({ payload: { callback } }, { select, put }) {
      const apiParams = yield select(selectedCoursesSelector);
      const rsp = yield changeClassWithFlow(apiParams).catch(errorsHandling());

      let params = false;
      if (rsp && rsp.resultCode === 0) {
        params = true;
        showMessage('学生转班申请提交成功，请等待审批！', 'success');
      }
      runFunc(callback, params);

      yield put({
        type: 'resetAuditUserDict',
      });
    },
    *normalChangeClass({ payload: { callback } }, { select, put }) {
      let apiParams = yield select((state) => {
        const courseList = state[storeName].selectedCourseList;
        const firstCourse = courseList && courseList[0];

        return {
          productType: state[storeName].sKey_prdType,
          studentChangeClassList: state[storeName].modalStudentList.map((std) => {
            let list = state[storeName].classList;
            let selectedRowKeys = state[storeName].classSelectedRowKeys;
            let courseIdKey = state[storeName].sKey_prdType === TWO_TEACHER_VAL ? 'courseId' : 'miniClassCourseId';
            return {
              changeReason: state[storeName].changeReason,
              studentId: std.studentId,
              oldClassId: std.classId,
              newClassId: list[selectedRowKeys[0]].id.toString(),
              courseIds: (courseList && courseList.map((course) => course[courseIdKey])) || null,
              classDate: (firstCourse && firstCourse.courseDate) || list[selectedRowKeys[0]].courseDate,
            };
          }),
        };
      });
      const rsp = yield studentChangeClassXH(apiParams, true).catch(errorsHandling());
      let params = false;
      if (rsp && rsp.resultCode === 0) {
        params = true;
        showMessage('学生转班成功！', 'success');
      }
      runFunc(callback, params);
    },

    *changeFinish({ payload: { callback } }, { put }) {
      yield put({
        type: 'closeModal',
      });
      yield put({
        type: 'updateState',
        payload: {
          ...clone(defaultState.changeClassData),
        },
      });
      runFunc(callback);
    },
    *getBranchAndChildCampusForClassChangeApi({ payload: { callback } }, { put, select }) {
      const rsp = yield getBranchAndChildCampusForClassChange().catch(errorsHandling());
      yield put({
        type: 'updateState',
        payload: {
          twoTeacherBranchCampusDict: rsp.data,
          branchDict: rsp.data.map((branch) => {
            return {
              text: branch.label,
              val: branch.val,
            };
          }),
        },
      });
      runFunc(callback);
    },
  },
};

function selectedCoursesSelector(state) {
  const courseList = state[storeName].selectedCourseList;
  const firstCourse = courseList && courseList[0];
  const firstStudent = state[storeName].modalStudentList[0];

  let list = state[storeName].classList;
  let selectedRowKeys = state[storeName].classSelectedRowKeys;
  let courseIdKey = state[storeName].sKey_prdType === TWO_TEACHER_VAL ? 'courseId' : 'miniClassCourseId';

  return {
    productType: state[storeName].sKey_prdType,
    studentId: firstStudent.studentId,
    oldClassId: firstStudent.classId,
    newClassId: list[selectedRowKeys[0]].id.toString(),
    courseIds: (courseList && courseList.map((course) => course[courseIdKey])) || null,
    firstSchoolDate: (firstCourse && firstCourse.courseDate) || list[selectedRowKeys[0]].courseDate,
    changeReason: state[storeName].changeReason,
    nodeId: state[storeName].nodeId,
    auditUserId: state[storeName].auditUserId,
  };
}
