import React from 'react';
import Modal from 'COMPONENT/Modal';
import { storeName } from '../store/config';
import { connect } from 'dva';
import style from './detail-modal.less';
import classNames from 'classnames';
import WorkflowTable from './WorkflowTable';
import MessageRecordTable from './MessageRecordTable';

function DetailModal(props) {
  function close() {
    props.dispatch({
      type: `${storeName}/closeDetailModal`,
    });
  }

  const { detailModalData } = props;

  const headInfos = [
    {
      key: 1,
      label: '审批编号',
      text: detailModalData.id,
    },
    {
      key: 2,
      label: 'OA编号',
      text: detailModalData.oaInstanceId,
    },
    {
      key: 3,
      label: '转班类型',
      text: detailModalData.changeTypeName,
    },
    {
      key: 4,
      label: '学生姓名',
      text: detailModalData.studentName,
    },
    {
      key: 5,
      label: '发起人',
      text: detailModalData.applicantUserName,
    },
    {
      key: 6,
      label: '发起时间',
      text: detailModalData.applicantTime,
    },
    {
      key: 9,
      label: '家长手机',
      text: detailModalData.customerContact,
    },
  ];

  const oldClassInfos = [
    {
      key: 1,
      label: '学习中心名称',
      text: detailModalData.oldCampusName,
    },
    {
      key: 2,
      label: '产品名称',
      text: detailModalData.oldProductName,
    },
    {
      key: 3,
      label: '班级名称',
      text: detailModalData.oldClassName,
    },
    {
      key: 4,
      label: '班级类型',
      text: detailModalData.oldClassTypeName,
    },
    {
      key: 5,
      label: '班级层次',
      text: detailModalData.oldClassLevelName,
    },
    {
      key: 6,
      label: '年级',
      text: detailModalData.oldGradeName,
    },
    {
      key: 7,
      label: '学生剩余小时',
      text: detailModalData.remainCourseNatureHours,
    },
    {
      key: 8,
      label: '产品组',
      text: detailModalData.oldProductGroupName,
    },
  ];

  const newClassInfos = [
    {
      key: 1,
      label: '学习中心名称',
      text: detailModalData.newCampusName,
    },
    {
      key: 2,
      label: '产品名称',
      text: detailModalData.newProductName,
    },
    {
      key: 3,
      label: '班级名称',
      text: detailModalData.newClassName,
    },
    {
      key: 4,
      label: '班级类型',
      text: detailModalData.newClassTypeName,
    },
    {
      key: 5,
      label: '班级层次',
      text: detailModalData.newClassLevelName,
    },
    {
      key: 6,
      label: '年级',
      text: detailModalData.newGradeName,
    },
    {
      key: 7,
      label: '学生剩余小时',
      text: detailModalData.changeCourseNatureHours || '-',
    },
    {
      key: 8,
      label: '产品组',
      text: detailModalData.newProductGroupName || '-',
    },
  ];

  return (
    <Modal
      loading={false}
      onClose={close}
      title="转班审批详情"
      width={1070}
      okBtnShow={false}
      cancelBtnShow={false}
      maskClosable={true}
    >
      <div className={style.flexWrap}>
        {headInfos.map((info) => {
          return (
            <div className={style.item} key={info.key}>
              <span className={style.label}>{info.label}: </span>
              <span className={style.text}>{info.text}</span>
            </div>
          );
        })}
      </div>
      <div className={style.block}>
        <div className={style.title}>转出班</div>
        <div className={style.flexWrap}>
          {oldClassInfos.map((info) => {
            return (
              <div className={style.item} key={info.key}>
                <span className={style.label}>{info.label}: </span>
                <span className={style.text}>{info.text}</span>
              </div>
            );
          })}
        </div>
      </div>
      <div className={style.block}>
        <div className={style.title}>转入班</div>
        <div className={style.flexWrap}>
          {newClassInfos.map((info) => {
            return (
              <div className={style.item} key={info.key}>
                <span className={style.label}>{info.label}: </span>
                <span className={style.text}>{info.text}</span>
              </div>
            );
          })}
        </div>
      </div>
      <div className={style.tables}>
        <div className={style.title}>OA审批记录</div>
        <div className={style.table}>
          <WorkflowTable />
        </div>
      </div>
    </Modal>
  );
}

function mapStateToProps(state) {
  return {
    detailModalData: state[storeName].detailModalData,
  };
}

export default connect(mapStateToProps)(DetailModal);
