import React from 'react';
import { connect } from 'dva';
import { Select, Input, SearchBar, DatePicker } from 'COMPONENT';
import { storeName } from '../store/config';
// import style from './search.less';
import { Row, Col } from 'antd';
import moment from 'moment';
import { changeTypeDict, operationTypeDict, auditStatusDict, changeStatusDict, isConfirmDict } from '../dicts';
import { getUserForPerformance } from 'SERVICE/contract';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { debounce } from 'lodash';

// prefer pure function rather than class component
function Search(props) {
  function updateProps(params) {
    props.dispatch({
      type: `${storeName}/updateState`,
      payload: params,
    });
  }

  function updateValue(type, val) {
    updateProps({ [type]: val });
  }

  /**
   * 获取要禁用的开始日期值
   * @param  {string} startDate 过滤前的日期值
   * @return {boolean} 是否要禁用
   */
  function getDisabledStartDate(startDate) {
    const { sKey_endTime } = props;
    if (!startDate || !props.sKey_endTime) {
      return false;
    }
    return moment(startDate).valueOf() > moment(sKey_endTime).valueOf();
  }

  /**
   * 获取要禁用的结束日期值
   * @param  {string} endDate   过滤前的日期值
   * @return {boolean}      是否要禁用
   */
  function getDisabledEndDate(endDate) {
    const { sKey_startTime } = props;
    if (!endDate || !sKey_startTime) {
      return false;
    }
    return moment(endDate).valueOf() < moment(sKey_startTime).valueOf();
  }

  function onReset() {
    props.dispatch({
      type: `${storeName}/resetPage`,
    });
    props.onSearch();
  }

  function batchSendMessage() {}

  /**
   * 搜索用户
   * @param term
   * @returns {Promise<void>}
   */
  async function onSearchUser(term, index) {
    if (term) {
      try {
        const dict = await getUserForPerformance({ term });
        props.dispatch({
          type: `${storeName}/updateState`,
          payload: {
            applicantDict: dict.map((item) => ({ text: item.label, val: item.value, title: item.label })),
          },
        });
      } catch (e) {
        showMessage(e.text);
      }
    } else {
      props.dispatch({
        type: `${storeName}/updateState`,
        payload: {
          applicantDict: [],
        },
      });
    }
  }

  /**
   * 搜索
   */
  function searchUserDebounce(key, index) {
    debounce(onSearchUser, 300)(key, index);
  }

  return (
    <div>
      <Row gutter={16}>
        <Col span={24}>
          <SearchBar onToggleExpand={props.onToggleExpand} onSearch={props.onSearch} onReset={onReset}>
            <Input
              placeholder="OA编号"
              value={props.sKey_oaInstanceId}
              onChange={(e) => updateValue('sKey_oaInstanceId', e.target.value)}
              allowClear={true}
            />
            <Input
              placeholder="学生姓名或编号"
              value={props.sKey_studentName}
              onChange={(e) => updateValue('sKey_studentName', e.target.value)}
              allowClear={true}
            />
            {/*<Input*/}
            {/*  placeholder="发起人"*/}
            {/*  value={props.sKey_applicantId}*/}
            {/*  onChange={e => updateValue('sKey_applicantId', e.target.value)}*/}
            {/*  allowClear={true}*/}
            {/*/>*/}
            <Select
              placeholder="发起人"
              value={props.sKey_applicantId}
              dict={props.applicantDict}
              onSearchKeyChange={(key) => searchUserDebounce(key, 0)}
              showSearch
              noOptionTips="没有找到"
              searchPlaceholder="输入名字搜索"
              onChange={(val) => updateValue('sKey_applicantId', val)}
            />
            <Select
              placeholder="转班类型"
              dict={changeTypeDict}
              value={props.sKey_changeType}
              onChange={(val) => updateValue('sKey_changeType', val)}
              allowClear={true}
              showSearch
            />
            <Select
              placeholder="业务类型"
              dict={operationTypeDict}
              value={props.sKey_operationType}
              onChange={(val) => updateValue('sKey_operationType', val)}
              allowClear={true}
              showSearch
            />
            <Select
              placeholder="转出学习中心"
              dict={props.campusDict}
              value={props.sKey_oldCampusId}
              onChange={(val) => updateValue('sKey_oldCampusId', val)}
              allowClear={true}
              showSearch
            />
            <Select
              placeholder="转入学习中心"
              dict={props.campusDict}
              value={props.sKey_newCampusId}
              onChange={(val) => updateValue('sKey_newCampusId', val)}
              allowClear={true}
              showSearch
            />
            <Select
              placeholder="审批状态"
              dict={auditStatusDict}
              value={props.sKey_auditStatus}
              onChange={(val) => updateValue('sKey_auditStatus', val)}
              allowClear={true}
              showSearch
            />
            <Select
              placeholder="转班状态"
              dict={changeStatusDict}
              value={props.sKey_changeStatus}
              onChange={(val) => updateValue('sKey_changeStatus', val)}
              allowClear={true}
              showSearch
            />
            <div style={{ width: 360 }}>
              <Row gutter={20}>
                <Col span={12}>
                  <DatePicker
                    placeholder="开始日期"
                    value={props.sKey_startTime}
                    onChange={(val) => updateValue('sKey_startTime', val)}
                    disabledDate={getDisabledStartDate}
                    allowClear={true}
                    showTime
                  />
                </Col>
                <Col span={12}>
                  <DatePicker
                    placeholder="结束日期"
                    value={props.sKey_endTime}
                    onChange={(val) => updateValue('sKey_endTime', val)}
                    disabledDate={getDisabledEndDate}
                    allowClear={true}
                    showTime
                  />
                </Col>
              </Row>
            </div>
          </SearchBar>
        </Col>
        {/*<Col span={2}>*/}
        {/*  <Button*/}
        {/*    onClick={ batchSendMessage}*/}
        {/*    disabled={props.selectedRows.length <= 1}*/}
        {/*    type="primary"*/}
        {/*  >批量发送提醒</Button>*/}
        {/*</Col>*/}
      </Row>
    </div>
  );
}

const mapStateToProps = (state) => {
  return {
    // 筛选项值
    sKey_oaInstanceId: state[storeName].sKey_oaInstanceId,
    sKey_studentName: state[storeName].sKey_studentName,
    sKey_applicantId: state[storeName].sKey_applicantId,
    sKey_changeType: state[storeName].sKey_changeType,
    sKey_operationType: state[storeName].sKey_operationType,
    sKey_oldCampusId: state[storeName].sKey_oldCampusId,
    sKey_newCampusId: state[storeName].sKey_newCampusId,
    sKey_auditStatus: state[storeName].sKey_auditStatus,
    sKey_changeStatus: state[storeName].sKey_changeStatus,
    sKey_startTime: state[storeName].sKey_startTime,
    sKey_endTime: state[storeName].sKey_endTime,
    selectedRows: state[storeName].selectedRows,
    campusDict: state.dict.campus,
    applicantDict: state[storeName].applicantDict,
  };
};

export default connect(mapStateToProps)(Search);
