/**
 * 转班审批记录
 */
import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import Search from './Search';
import DetailModal from './DetailModal';
import MessageModal from './MessageModal';
import {
  storeName,
  StopChangerClassByFlowId,
  SendMessageForChangerClass,
  DoChangerClassByFlowId,
  ShowDetailModal,
} from './store/config';
import { confirmMap } from './dicts';
import { Table } from 'antd';
import { TextLink, List } from 'COMPONENT';

const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_XL, COL_WIDTH_L } = List;
const COL_WIDTH_XXL = COL_WIDTH_XL + 50;

class ChangeClassFlowRecords extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      detailModalVisible: false,
      scrollY: 0,
    };
  }

  componentDidMount() {
    this.initSearchDefault();
    this.calculateScrollY();
    this.loadList();
    this.initDict();
  }

  calculateScrollY = () => {
    const searchDomRect = document.querySelector('.Page-header').getClientRects()[0];
    // 搜索+分页高度
    let y = searchDomRect.top + searchDomRect.height + 200;
    this.setState({
      scrollY: `calc(100vh - ${y}px)`,
    });
  };

  onToggleExpand = () => {
    setTimeout(() => {
      this.calculateScrollY();
    }, 0);
  };

  initDict = () => {
    new Promise((resolve) => {
      this.props.dispatch({
        type: 'dict/initDict',
        payload: {
          list: ['campus'],
          resolve,
        },
      });
    });
  };

  initSearchDefault = () => {
    this.props.dispatch({
      type: `${storeName}/initSearchDefault`,
    });
  };

  showLoading = () => {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        listLoading: true,
      },
    });
  };

  hideLoading = () => {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        listLoading: false,
      },
    });
  };

  loadList = async () => {
    this.showLoading();
    try {
      await new Promise((resolve, reject) => {
        this.props.dispatch({
          type: `${storeName}/loadData`,
          payload: {
            resolve,
            reject,
          },
        });
      });
    } finally {
      this.hideLoading();
    }
  };

  updatePagination = (pageCurrent, pageSize) => {
    let payload = {};
    if (pageSize && pageSize !== this.props.pageSize) {
      payload.pageCurrent = 1;
      payload.pageSize = pageSize;
    } else {
      payload.pageCurrent = pageCurrent;
    }
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload,
    });

    this.loadList();
  };

  onSelectRow = (selectedRow, selected, selectedRows) => {
    // const canSelect = selectedRows.every(row => {
    //   const { auditStatus, changeStatus, isSendMessage, isConfirm } = row;
    //   return canSend(auditStatus, changeStatus, isSendMessage, isConfirm);
    // });

    // if(!canSelect) {
    //   showMessage('请选择审批已通过; 转班成功;家长未确认的记录！');
    //   return ;
    // }

    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        selectedRows,
        selectedRow,
      },
    });
  };

  onRowBtnClick = (messageModalType, selectedRow) => {
    if (ShowDetailModal === messageModalType) {
      this.props.dispatch({
        type: `${storeName}/openDetailModal`,
        payload: {
          selectedRow,
        },
      });
      return;
    }
    this.props.dispatch({
      type: `${storeName}/openMessageModal`,
      payload: {
        messageModalType,
        selectedRow,
      },
    });
  };

  render() {
    let { records, listLoading, rowCount, pageCurrent, pageSize, messageModalVisible, detailModalVisible } = this.props;
    let { loading } = this.state;

    const columns = [
      { title: '编号', dataIndex: 'id', key: 'id', width: COL_WIDTH_S },
      { title: 'OA编号', dataIndex: 'oaInstanceId', width: COL_WIDTH_XL },
      { title: '学生编号', dataIndex: 'studentId', width: COL_WIDTH_L },
      { title: '学生姓名', dataIndex: 'studentName', width: COL_WIDTH_M },
      { title: '业务类型', dataIndex: 'operationTypeName', width: COL_WIDTH_M },
      { title: '转班类型', dataIndex: 'changeTypeName', width: COL_WIDTH_M },
      {
        title: '转出班',
        children: [
          { title: '学习中心名称', dataIndex: 'oldCampusName', width: COL_WIDTH_M },
          { title: '产品名称', dataIndex: 'oldProductName', width: COL_WIDTH_M },
          { title: '班级名称', dataIndex: 'oldClassName', width: COL_WIDTH_XXL },
          { title: '产品组', dataIndex: 'oldProductGroupName', width: COL_WIDTH_M },
          { title: '学生剩余小时', dataIndex: 'remainCourseNatureHours', width: COL_WIDTH_M },
        ],
      },
      {
        title: '转入班',
        children: [
          { title: '学习中心名称', dataIndex: 'newCampusName', width: COL_WIDTH_M },
          { title: '产品名称', dataIndex: 'newProductName', width: COL_WIDTH_M },
          { title: '班级名称', dataIndex: 'newClassName', width: COL_WIDTH_XXL },
          { title: '产品组', dataIndex: 'newProductGroupName', width: COL_WIDTH_M },
          { title: '学生申请转入小时', dataIndex: 'changeCourseNatureHours', width: COL_WIDTH_XXL },
        ],
      },
      { title: '发起时间', dataIndex: 'applicantTime', width: COL_WIDTH_XL },
      { title: '发起人', dataIndex: 'applicantUserName', width: COL_WIDTH_M },
      { title: '审批状态', dataIndex: 'auditStatusName', width: COL_WIDTH_M },
      { title: '转班状态', dataIndex: 'changeStatusName', width: COL_WIDTH_M },
      { title: '转班状态备注', dataIndex: 'changeStatusRemark', width: COL_WIDTH_M },
      {
        title: '超期天数',
        dataIndex: 'overdueDays',
        width: COL_WIDTH_M,
        render: (day, record, index) => {
          if (record.changeStatus !== 'CHANGE_SUCCESS') {
            return '-------';
          }
          return <span>{day > 1 ? `${day}天` : '1天内'}</span>;
        },
      },
      {
        title: '操作',
        key: 'action',
        width: COL_WIDTH_XL,
        fixed: 'right',
        render: (text, record, index) => {
          const { auditStatus, changeStatus, isSendMessage, isConfirm } = record;

          const stopChangButton =
            (auditStatus === 'AUDITING' && changeStatus === 'AUDITING' && isSendMessage === 0 && isConfirm === 0) ||
            (auditStatus === 'AUDIT_COMPLETED' &&
              changeStatus === 'CHANGE_EXCEPTION' &&
              isSendMessage === 0 &&
              isConfirm === 0);

          const sendButton = canSend(auditStatus, changeStatus, isSendMessage, isConfirm);

          const reDoButton =
            auditStatus === 'AUDIT_COMPLETED' &&
            changeStatus === 'CHANGE_EXCEPTION' &&
            isSendMessage === 0 &&
            isConfirm === 0;
          const { btnAuth_stop, btnAuth_send, btnAuth_redo } = this.props;
          const actions = [
            { text: '详情', onClick: () => this.onRowBtnClick(ShowDetailModal, record, index, name) },
            btnAuth_stop &&
              stopChangButton && {
                text: '终止转班',
                onClick: () => this.onRowBtnClick(StopChangerClassByFlowId, record, index, name),
              },
            // 隐藏转班成功的发送提醒
            // btnAuth_send && sendButton && { text: '发送提醒', onClick: () => this.onRowBtnClick(SendMessageForChangerClass, record, index, name) },
            btnAuth_redo &&
              reDoButton && {
                text: '重新转入',
                onClick: () => this.onRowBtnClick(DoChangerClassByFlowId, record, index, name),
              },
          ].filter((button) => button);

          return (
            <span>
              {actions.map((action, i) => {
                return (
                  <TextLink key={i} className="list-btn" onClick={action.onClick} disabled={action.disabled}>
                    {action.text}
                  </TextLink>
                );
              })}
            </span>
          );
        },
      },
    ];

    return (
      <Page
        fitWindowHeight={true}
        loading={loading}
        route={this.props.route}
        subHeader={<Search onSearch={this.loadList} onToggleExpand={this.onToggleExpand} />}
      >
        <Table
          rowKey="id"
          dataSource={records}
          columns={columns}
          loading={!loading && listLoading}
          rowSelection={{
            selectedRowKeys: this.props.selectedRowKeys,
            onSelect: this.onSelectRow,
          }}
          pagination={{
            showQuickJumper: true,
            pageSizeOptions: ['10', '20', '50'],
            showSizeChanger: true,
            showTotal: (total) => `共 ${total} 条记录`,
            current: pageCurrent,
            total: rowCount,
            pageSize,
          }}
          scroll={{
            x: 3100 - COL_WIDTH_XL,
            y: this.state.scrollY,
          }}
          onChange={({ current, pageSize }) => this.updatePagination(current, pageSize)}
        />
        {detailModalVisible && <DetailModal onClose={() => this.setState({ showDetailModal: false })} />}
        {messageModalVisible && <MessageModal onOk={this.loadList} />}
      </Page>
    );
  }
}

function mapStateToProps(state) {
  const btnAuth = state.account.btnAuths || '';

  return {
    records: state[storeName].records,
    rowCount: state[storeName].rowCount,
    pageSize: state[storeName].pageSize,
    pageCurrent: state[storeName].pageCurrent,
    selectedRowKeys: state[storeName].selectedRows.map((row) => row.id),
    btnAuth_stop: btnAuth.indexOf('ChangeClassFlowRecords_Stop') !== -1,
    btnAuth_send: btnAuth.indexOf('ChangeClassFlowRecords_Send') !== -1,
    btnAuth_redo: btnAuth.indexOf('ChangeClassFlowRecords_Redo') !== -1,
    listLoading: state[storeName].listLoading,
    messageModalVisible: state[storeName].messageModalVisible,
    detailModalVisible: state[storeName].detailModalVisible,
  };
}

export default connect(mapStateToProps)(ChangeClassFlowRecords);

function canSend(auditStatus, changeStatus, isSendMessage, isConfirm) {
  return (
    (auditStatus === 'AUDIT_COMPLETED' &&
      changeStatus === 'CHANGE_SUCCESS' &&
      isSendMessage === 0 &&
      isConfirm === 0) ||
    (auditStatus === 'AUDIT_COMPLETED' && changeStatus === 'CHANGE_SUCCESS' && isSendMessage === 1 && isConfirm === 0)
  );
}
