import React from 'react';
import classNames from 'classnames';
import styles from './styles.module.less';
import { Button, TextLink } from 'COMPONENT';

export default function ProductCard(props) {
  const { product, auths } = props;

  const ProductActions = () => {
    const isNormalProduct = product.status === 'NORMAL'; // 合同状态正常
    const noClass = !product.miniClassId; // 未在班
    const isNormalClass = product.miniClassStudentStatus === 'NORMAL'; // 在班
    const { action } = props;

    if (!isNormalProduct) {
      if (['ENDED', 'CLOSE_PRODUCT'].includes(product.status) && auths.dropCourse) {
        return (
          <Button size="xSmall" onClick={() => action({ type: 'DROP_COURSE', data: product })}>
            退课
          </Button>
        );
      }
      return null;
    }

    if (noClass)
      return (
        <div className={styles.footerRight}>
          {auths.register && (
            <Button type="primary" size="xSmall" onClick={() => action({ type: 'REGISTER', data: product })}>
              报班
            </Button>
          )}
        </div>
      );
    if (product.miniClassStudentStatus === 'PREPARE_DROP')
      return (
        <div className={styles.footerRight}>
          {auths.cancelPreWithdraw && (
            <Button type="primary" size="xSmall" onClick={() => action({ type: 'CANCEL_PREWITHDRAW', data: product })}>
              取消预退
            </Button>
          )}
        </div>
      );
    if (!isNormalClass) return null;
    return (
      <div className={styles.footerRight}>
        {auths.switch && (
          <Button type="primary" size="xSmall" onClick={() => action({ type: 'SWITCH', data: { product } })}>
            调课
          </Button>
        )}
        {auths.transfer && (
          <Button type="primary" size="xSmall" onClick={() => action({ type: 'TRANSFER', data: product })}>
            转班
          </Button>
        )}
        {auths.preWithdraw && (
          <Button type="primary" size="xSmall" onClick={() => action({ type: 'PREWITHDRAW', data: product })}>
            预退
          </Button>
        )}
        {auths.withdraw && (
          <Button size="xSmall" onClick={() => action({ type: 'WITHDRAW', data: product })}>
            退班
          </Button>
        )}
      </div>
    );
  };

  return (
    <div className={styles.productCard}>
      <div className={styles.header}>
        <div className={classNames([styles.status, styles[product.status]])}>
          <span>{product.statusName}</span>
          <span>{product.quarterName}</span>
        </div>
        <span className={styles.title}>{product.productName}</span>
      </div>

      <div className={styles.progress}>
        <div>
          已上<span>{product.consumedNatureHours}</span>小时
        </div>
        <div>
          剩余<span>{product.remainingNatureHour}</span>小时
        </div>
      </div>

      <div className={styles.body}>
        <div>
          <span>合同编号：</span>
          <span>{product.contractId}</span>
        </div>
        <div>
          <span>班级名称：</span>
          <span>
            {product.miniClassName}
            {product.miniClassTeacherName ? `(${product.miniClassTeacherName})` : ''}
          </span>
        </div>
        <div>
          <span>班级学习中心：</span>
          <span>{product.miniClassCampusName}</span>
        </div>
      </div>

      <div className={styles.footer}>
        <TextLink className={styles.footerLeft} onClick={() => props.onDetail(product)}>
          查看详情
        </TextLink>
        <ProductActions />
      </div>
    </div>
  );
}
