import React from 'react';
import styles from './styles.module.less';
import { connect } from 'dva';
import { Page, SearchFilterForm, SearchFieldForm, Select, Deck, NoDataPlus, Loading } from 'COMPONENT';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import SearchStudentAutoComplete, { parseStudent } from 'BLOC/SearchStudentAutoComplete';
import ProductCard from './ProductCard';
import { findStudentById } from 'SERVICE/student';
import { listValidSmallClassWithProductByStudentId } from 'SERVICE/contract';
import ProductDetailModal from './ProductDetailModal';
import ChangeCourse from 'PAGE/course/miniClassManage/ScheduleManage/ChangeCourse';
import { loadCampusDict } from 'PAGE/course/miniClassManage/ScheduleManage/loaders';
import PreWithdrawModal from 'PAGE/course/modals/prePareClass';
import CancelPreWithdrawModal from 'PAGE/course/modals/CancelPrePareClass';
import WithdrawModal from 'PAGE/course/modals/DropClass';
import SelectCourseModal from './SelectCourseModal';

const STATUS_DICT = [
  {
    text: '全部',
    val: '',
  },
  {
    text: '正常',
    val: 'NORMAL',
  },
  {
    text: '结束',
    val: 'ENDED',
  },
  {
    text: '结课',
    val: 'CLOSE_PRODUCT',
  },
];

const defaultSearchParams = {
  yearId: null,
  quarterId: null,
  studentId: null,
};

/* 产品卡片 */
function productCardRender(config) {
  return (product, index) => {
    return (
      <ProductCard
        index={index}
        key={index}
        product={product}
        auths={config.auths}
        action={config.action}
        onDetail={config.onDetail}
      />
    );
  };
}

/* 学生信息 */
function StudentOverview(props) {
  return (
    <section className={styles.studentOverview}>
      <div>
        <span>学生姓名</span>
        <span>{props.studentName || '--'}</span>
      </div>
      <div>
        <span>学生编号</span>
        <span>{props.studentId || '--'}</span>
      </div>
      <div>
        <span>主学籍</span>
        <span>{props.campusName || '--'}</span>
      </div>
    </section>
  );
}

class CourseStudentAdjustment extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false,
      productList: [],
      currentPage: 1,
      size: 10,
      total: 0,
      searchParams: { ...defaultSearchParams },
      studentId: '',
      studentName: '',
      campusName: '', // 主校区
      selectedProduct: null, // 选中的产品
      isShowDetail: false, // 显示详情
      isShowPreWithdraw: false, // 显示预退班
      isShowSwitch: false, // 显示调课
      isShowCancelPreWithdraw: false, // 显示取消预退班
      isShowWithdraw: false, // 显示退班
      isShowDropCourse: false, // 显示退课
      changeCourseMiniClassId: null, // 调课id
      teacherDict: null,
      campusDict: null,
    };
  }

  componentDidMount() {
    this.init();
  }

  async init() {
    this.setState({ loading: true });
    await new Promise(this.initDict);
    this.setState({ loading: false });
  }

  // 初始化筛选项
  initDict = (resolve) => {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['year', 'quarter'],
        callback() {
          resolve();
        },
      },
    });
  };

  // 完整的产品列表
  productList = [];
  getProductsPage = (page = this.state.currentPage, size = this.state.size) => {
    const start = (page - 1) * size;
    const end = start + size;
    return this.productList.slice(start, end);
  };

  search = (params) => {
    // valid
    if (!params.studentId) {
      return showMessage('请输入需要查询的学生');
    }
    const student = parseStudent(params.studentId);
    if (!student || !student.studentId) {
      return showMessage('请输入正确的学生');
    }
    const studentId = student.studentId;
    this.setState(
      {
        searchParams: params,
      },
      () => {
        this.getStudentInfo(studentId);
        this.searchProducts(studentId);
      },
    );
  };

  /* 查询报读产品 */
  searchProducts = async (studentId) => {
    this.setState({ loading: true });
    const { searchParams } = this.state;
    try {
      const res = await listValidSmallClassWithProductByStudentId({ ...searchParams, studentId });
      const productList = res.data || [];
      this.productList = productList;
      this.setState({
        total: productList.length,
        currentPage: 1,
        productList: this.getProductsPage(1),
      });
    } catch (e) {
      console.log('searchProducts e', e);
    } finally {
      this.setState({ loading: false });
    }
  };

  searchError = (err) => {
    console.log('onSearchFail', err);
  };

  reset = () => {
    this.setState({
      searchParams: { ...defaultSearchParams },
    });
  };

  setPage = (currentPage, size) => {
    this.setState({ loading: true });
    this.setState((prevState) => {
      const payload = {};
      if (size && size !== prevState.size) {
        payload.currentPage = 1;
        payload.size = size;
      } else {
        payload.currentPage = currentPage;
      }
      return {
        ...payload,
        loading: false,
        productList: this.getProductsPage(payload.currentPage, payload.size),
      };
    });
  };

  /* 查询学生信息 */
  getStudentInfo = async (studentId) => {
    const res = (await findStudentById({ id: studentId })) || {};
    this.setState({
      studentId,
      studentName: res.name,
      campusName: res.blCampusName,
    });
  };

  productAction = (params) => {
    switch (params.type) {
      case 'REGISTER':
        this.register(params.data);
        break;
      case 'SWITCH':
        this.changeCourse(params.data);
        break;
      case 'TRANSFER':
        this.transfer(params.data);
        break;
      case 'PREWITHDRAW':
        this.preWithdraw(params.data);
        break;
      case 'CANCEL_PREWITHDRAW':
        this.cancelPreWithdraw(params.data);
        break;
      case 'WITHDRAW':
        this.withdraw(params.data);
        break;
      case 'DROP_COURSE':
        this.dropCourse(params.data);
        break;
      default:
        console.error(`Error: invalid product actionType(${params.type})!`);
    }
  };

  /* 查看产品详情 */
  detail = (selectedProduct) => {
    this.setState({
      isShowDetail: true,
      selectedProduct,
    });
  };

  /* 关闭产品详情 */
  closeDetail = () => {
    this.setState({
      isShowDetail: false,
      selectedProduct: null,
    });
  };

  /* 报班 */
  register = (product) => {
    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path: '/registeredClass',
        refresh: true,
        params: {
          studentId: this.state.studentId,
          studentName: this.state.studentName,
        },
      },
    });
  };

  /* 获取校区相关选项 */
  async initCampusDicts() {
    this.setState({ loading: true });
    const campusDict = await Promise.resolve(loadCampusDict({ orgType: 'CAMPUS' }));
    this.setState({
      loading: false,
      campusDict,
    });
  }

  /* 调课 */
  changeCourse = async ({ product, miniClassId }) => {
    await this.initCampusDicts();
    this.setState({
      selectedProduct: product,
      changeCourseMiniClassId: miniClassId || product.miniClassId,
      isShowSwitch: true,
    });
  };

  /* 转班 */
  transfer = (product) => {
    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path: '/changeClass',
        refresh: true,
        params: {
          studentId: this.state.studentId,
          studentName: this.state.studentName,
        },
      },
    });
  };

  /* 预退 */
  preWithdraw = (product) => {
    this.setState({
      selectedProduct: product,
      isShowPreWithdraw: true,
    });
  };

  /* 取消预退 */
  cancelPreWithdraw = (product) => {
    this.setState({
      selectedProduct: product,
      isShowCancelPreWithdraw: true,
    });
  };

  /* 退班 */
  withdraw = (product) => {
    this.setState({
      selectedProduct: product,
      isShowWithdraw: true,
    });
  };

  /* 退课 */
  dropCourse = (product) => {
    this.setState({
      selectedProduct: product,
      isShowDropCourse: true,
    });
  };

  /* 刷新数据 */
  refresh = () => {
    this.closeDetail();
    setTimeout(() => {
      this.searchProducts(this.state.studentId);
    });
  };

  /* 产品详情弹窗 */
  renderProductDetail = () => {
    const { isShowDetail, selectedProduct } = this.state;
    if (!isShowDetail || !selectedProduct) return null;
    return (
      <ProductDetailModal
        product={selectedProduct}
        auths={this.props.btnAuthed}
        onClose={this.closeDetail}
        action={this.productAction}
      />
    );
  };

  /* 调课 */
  renderSwitchModal = () => {
    const { isShowSwitch, selectedProduct } = this.state;
    if (!isShowSwitch || !selectedProduct) return null;
    const { studentId, changeCourseMiniClassId, campusDict, teacherDict } = this.state;
    return (
      <ChangeCourse
        studentId={studentId}
        studentIds={[]}
        miniClassId={changeCourseMiniClassId}
        campusDict={campusDict}
        teacherDict={teacherDict}
        closeModal={() => {
          this.setState({
            isShowSwitch: false,
          });
        }}
        search={() => {
          this.refresh();
        }}
      />
    );
  };

  /* 预退班 */
  renderPreWithdrawModal = () => {
    const { isShowPreWithdraw, selectedProduct, studentId } = this.state;
    if (!isShowPreWithdraw || !selectedProduct) return null;
    return (
      <PreWithdrawModal
        studentId={studentId}
        miniClassId={selectedProduct.miniClassId}
        onClose={() => {
          this.setState({
            isShowPreWithdraw: false,
          });
        }}
        onOK={() => {
          this.refresh();
        }}
      />
    );
  };

  /* 取消预退班 */
  renderCancelPreWithdrawModal = () => {
    const { isShowCancelPreWithdraw, selectedProduct, studentId } = this.state;
    if (!isShowCancelPreWithdraw || !selectedProduct) return null;
    return (
      <CancelPreWithdrawModal
        studentId={studentId}
        miniClassId={selectedProduct.miniClassId}
        onClose={() => {
          this.setState({
            isShowCancelPreWithdraw: false,
          });
        }}
        onOK={() => {
          this.refresh();
        }}
      />
    );
  };

  /* 退班 */
  renderWithdrawModal = () => {
    const { isShowWithdraw, selectedProduct, studentId } = this.state;
    if (!isShowWithdraw || !selectedProduct) return null;
    return (
      <WithdrawModal
        studentId={studentId}
        miniClassId={selectedProduct.miniClassId}
        onClose={() => {
          this.setState({
            isShowWithdraw: false,
          });
        }}
        onOK={() => {
          this.refresh();
        }}
      />
    );
  };

  /* 退课 */
  renderDropCourseModal = () => {
    const { isShowDropCourse, selectedProduct } = this.state;
    if (!isShowDropCourse || !selectedProduct) return null;
    return (
      <SelectCourseModal
        title="选择所退课程"
        productId={selectedProduct.id}
        multiSelect={true}
        onCancel={() => {
          this.setState({ isShowDropCourse: false });
        }}
        onConfirm={() => {
          this.setState({ isShowDropCourse: false });
        }}
        noDataDirectCallback={() => {}}
      />
    );
  };

  render() {
    const { searchParams } = this.state;
    const hasSearch = !!searchParams.studentId;
    const hasResult = this.state.total > 0;
    return (
      <div className={styles.page}>
        <div className={styles.header}>
          <SearchFilterForm truncationNum={4} onSearch={this.search} onSearchFail={this.searchError}>
            <SearchFieldForm name="studentId" rules={[{ required: true, message: '学生姓名不能为空' }]}>
              <SearchStudentAutoComplete />
            </SearchFieldForm>
            <SearchFieldForm name="yearId">
              <Select placeholder="年份" dict={this.props.yearDict} />
            </SearchFieldForm>
            <SearchFieldForm name="quarterId">
              <Select placeholder="季度" dict={this.props.quarterDict} />
            </SearchFieldForm>
            <SearchFieldForm name="status">
              <Select placeholder="合同产品状态" dict={STATUS_DICT} />
            </SearchFieldForm>
          </SearchFilterForm>
        </div>
        {hasSearch ? (
          <Page
            name="学生信息"
            fitWindowHeight={true}
            subHeader={
              <StudentOverview
                studentName={this.state.studentName}
                studentId={this.state.studentId}
                campusName={this.state.campusName}
              />
            }
          >
            {hasResult ? (
              <Deck
                fixedBodyHeight={true}
                dataSource={this.state.productList}
                render={productCardRender({
                  auths: this.props.btnAuthed,
                  action: this.productAction,
                  onDetail: this.detail,
                })}
                pagination={{
                  total: this.state.total,
                  current: this.state.currentPage,
                  size: this.state.size,
                  onChange: this.setPage,
                }}
                bodyClassName={styles.productListBody}
              />
            ) : (
              <NoDataPlus key="noResult" className={styles.noData} />
            )}
            {this.renderProductDetail()}
            {this.renderSwitchModal()}
            {this.renderPreWithdrawModal()}
            {this.renderCancelPreWithdrawModal()}
            {this.renderWithdrawModal()}
            {this.renderDropCourseModal()}
          </Page>
        ) : (
          <div className={styles.empty}>
            <NoDataPlus key="noSearch" className={styles.noData} text="当前内容为空,请输入信息" />
          </div>
        )}
        <Loading loading={this.state.loading} tip="正在查询" />
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  const btnAuths = state.account.btnAuths || [];
  const hasAuth = (auth) => btnAuths.includes(auth);
  return {
    yearDict: state.dict.year,
    quarterDict: state.dict.quarter,
    // 操作权限
    btnAuthed: {
      register: hasAuth('MINI_CALSS_REGISTER'),
      switch: hasAuth('MINI_CLASS_SWITCH'),
      transfer: hasAuth('MINI_CLASS_TRANSFER'),
      preWithdraw: hasAuth('MINI_CLASS_PREWITHDRAW'),
      withdraw: hasAuth('MINI_CLASS_WITHDRAW'),
      dropCourse: hasAuth('DROP_MINI_CLASS_COURSE'),
      cancelPreWithdraw: hasAuth('MINI_CLASS_PREWITHDRAW_CANCEL'),
    },
  };
};

export default connect(mapStateToProps)(CourseStudentAdjustment);
