import React from 'react';
import ReactDOM from 'react-dom';
import classnames from 'classnames';
import styles from './styles.less';
import utils from '../utils';
import moment from 'moment';
import { DATE_FORMAT } from 'COMPONENT/utils/form';
import MoreInfoPop from '../MoreInfoPop';

class Node extends React.Component {
  state = {
    mode: null,
  };

  componentDidMount() {
    this.$wrap = ReactDOM.findDOMNode(this.wrapRef);
    this.updateMode();
    window.addEventListener('resize', this.updateMode, false);
  }

  componentWillReceiveProps(nextProps) {
    if (nextProps.expandDate !== this.expandDate) {
      if (nextProps.expand && !this.props.expand) {
        this.setState({ mode: 'default' });
      } else {
        setTimeout(() => {
          this.updateMode();
        }, 500);
      }
    }
  }

  componentWillUnmount() {
    window.removeEventListener('resize', this.updateMode, false);
  }

  updateMode = () => {
    const w = this.$wrap.offsetWidth;
    this.setState({ mode: w < 55 ? 'small' : 'default' });
  };

  getContent = (lines) => {
    const {
      data: { headerInfo, title, infos },
    } = this.props;
    const cls = {
      [styles.content]: 1,
      [styles.contentDefault]: 1,
    };
    return (
      <div className={classnames(cls)}>
        <div className={styles.headerInfo}>{headerInfo}</div>
        {lines >= 2 && <div className={styles.title}>{title}</div>}
        {lines >= 3 && (
          <div className={styles.infos}>
            {infos.slice(0, lines - 2).map((info, idx) => {
              return (
                <div key={idx} className={styles.info}>
                  {info}
                </div>
              );
            })}
          </div>
        )}
      </div>
    );
  };

  getContentSmall = (lines) => {
    const {
      data: { title },
    } = this.props;
    const cls = {
      [styles.content]: 1,
      [styles.contentSmall]: 1,
    };
    const titleCls = {
      [styles.title]: 1,
      [styles[`titleLines${lines - 1}`]]: 1,
    };
    return (
      <div className={classnames(cls)}>
        <div className={classnames(titleCls)}>{title}</div>
      </div>
    );
  };

  render() {
    const {
      data: { date, startTime, endTime, status, type, moreInfos, title, classId },
      nodeStartTime,
      onActiveTimeChange,
      $panel,
      onCardClick,
    } = this.props;
    const { mode } = this.state;
    const colors = utils.statusColors[status];
    const top = utils.getTimePx(nodeStartTime, startTime);
    const height = utils.getTimePx(startTime, endTime);
    const lines = Math.floor((height - 6) / 19);
    return (
      <MoreInfoPop data={moreInfos} $panel={$panel} title={title}>
        <div
          className={styles.wrap}
          style={{ borderColor: colors.primary, top, height }}
          onMouseEnter={() => onActiveTimeChange({ start: startTime, end: endTime })}
          onMouseLeave={() => onActiveTimeChange(null)}
          onClick={() => onCardClick(this.props.data)}
          ref={(c) => (this.wrapRef = c)}
        >
          <div className={styles.tag} style={{ color: colors.primary, background: colors.bg }}>
            {utils.productTypeTagMap[type]}
          </div>
          {mode ? (mode === 'small' ? this.getContentSmall(lines) : this.getContent(lines)) : null}
        </div>
      </MoreInfoPop>
    );
  }
}

export default Node;
