import React from 'react';
import { Row, Col } from 'antd';
import Modal from 'COMPONENT/Modal';
import FormItem from 'COMPONENT/FormItem';
import styles from './styles.less';
import loaders from '../loaders';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import getWrappedInstance from 'UTIL/getWrappedInstance';

class DetailModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 内容加载状态
      title: null, // {string} 弹框标题
      className: null, // {string} 课程名称
      infos: [], // {object[]} 课程信息，格式为[{ label, content }]
      conflicts: [], // {object[]} 冲突信息，格式为[{ title: '老师冲突', list: ['xxx冲突', 'yyy冲突'] }]
    };
  }

  componentDidMount() {
    const { classId, productClassId, ownerType } = this.props;
    this.setState({ loading: true });
    loaders.loadDetail({ classId, productClassId, ownerType }).then(
      (data) => {
        this.setState({ loading: false, ...data });
      },
      () => {
        this.setState({ loading: false });
        showMessage('数据加载失败');
      },
    );
  }

  render() {
    const { onClose } = this.props;
    const { loading, title, className, infos, conflicts } = this.state;
    const FormItemConfig = {
      labelWidth: 100,
      style: { marginBottom: 0, padding: '0 15px' },
    };
    return (
      <Modal
        loading={loading}
        footer={null}
        onClose={onClose}
        title={title}
        width={708}
        noLine
        className={styles.modal + ' ' + styles.xModal}
        closable
        ref={(c) => (this.modal = getWrappedInstance(c))}
      >
        {className && <div className={styles.className}>{className}</div>}
        <Row gutter={10}>
          {infos.map((info, idx) => {
            return (
              <Col key={idx} span={12}>
                <FormItem {...FormItemConfig} name={info.label}>
                  {info.content}
                  {info.secondaryContent && <span className={styles.secondaryContent}>{info.secondaryContent}</span>}
                </FormItem>
              </Col>
            );
          })}
        </Row>
        <div className={styles.conflicts}>
          {conflicts.map((con, idx) => {
            return (
              <div key={idx} className={styles.conflictSec}>
                <div className={styles.conflictTitle}>{con.title}</div>
                <div className={styles.conflictList}>
                  {con.list.map((item, i) => {
                    return (
                      <FormItem {...FormItemConfig} name="冲突原因" key={i} className={styles.conflictItem}>
                        {item}
                      </FormItem>
                    );
                  })}
                </div>
              </div>
            );
          })}
        </div>
      </Modal>
    );
  }
}

export default DetailModal;
