import React from 'react';
import moment from 'moment';
import styles from './styles.less';
import ScheduleHeader from '../ScheduleHeader';
import SchedulePanel from '../SchedulePanel';
import { DATE_FORMAT, TIME_FORMAT } from 'COMPONENT/utils/form';
import getWindowSize from 'UTIL/getWindowSize';

/*
 * cols: { date, width }
 * */
const getCols = (startDate, endDate, expandDate) => {
  let cols = [];
  for (
    let date = startDate;
    moment(date).valueOf() <= moment(endDate).valueOf();
    date = moment(date).add(1, 'days').format(DATE_FORMAT)
  ) {
    let width = 1;
    if (expandDate && date === expandDate) {
      width = 8;
    }
    cols.push({ date, width });
  }
  return cols;
};

class Schedule extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      activeDate: null,
      currentTime: moment().format(TIME_FORMAT),
      activeStartTime: null,
      activeEndTime: null,
      expandDate: null,
      winW: 0,
    };
    this.timer = setInterval(() => {
      this.updateCurrentTime();
    }, 60 * 1000);
  }

  componentDidMount() {
    this.updateWinW();
    window.addEventListener('resize', this.updateWinW, false);
  }
  componentWillUnmount() {
    window.removeEventListener('resize', this.updateWinW, false);
    clearInterval(this.timer);
    this.timer = null;
  }

  updateCurrentTime = () => {
    this.setState({ currentTime: moment().format(TIME_FORMAT) });
  };

  updateWinW = () => {
    this.setState({ winW: getWindowSize().width });
  };

  onExpandChange = (date, expand) => {
    this.setState({ expandDate: expand ? date : null });
  };

  onColActive = (date, active) => {
    this.setState({ activeDate: active ? date : null });
  };

  onActiveTimeChange = (time) => {
    this.setState({ activeStartTime: time ? time.start : null, activeEndTime: time ? time.end : null });
  };

  render() {
    const { startDate, endDate, list, pageType, onCardClick } = this.props;
    let tempList = list || [];
    const { expandDate, activeDate, currentTime, activeStartTime, activeEndTime, winW } = this.state;
    const cols = getCols(startDate, endDate, expandDate);

    return (
      <div className={styles.wrap}>
        <ScheduleHeader cols={cols} activeDate={activeDate} />
        <div className={styles.body} ref={(c) => (this.bodyRef = c)}>
          {tempList.map((panel) => {
            return (
              <SchedulePanel
                key={panel.id}
                cols={cols}
                data={panel}
                activeDate={activeDate}
                activeStartTime={activeStartTime}
                activeEndTime={activeEndTime}
                winW={winW}
                expandDate={expandDate}
                onExpandChange={this.onExpandChange}
                onColActive={this.onColActive}
                onActiveTimeChange={this.onActiveTimeChange}
                currentTime={currentTime}
                onCardClick={onCardClick}
              />
            );
          })}
        </div>
      </div>
    );
  }
}

export default Schedule;
