import moment from 'moment';
import { DATE_FORMAT, TIME_FORMAT } from 'COMPONENT/utils/form';
import dicts from './dicts';
import styleVars from 'COMPONENT/theme/variables';

const getWeekByDate = (date) => {
  const day = moment(date, DATE_FORMAT).day();
  return day === 0 ? 6 : day - 1;
};

const getWeekRange = (mom) => {
  let day = mom.day();
  day = day === 0 ? 6 : day - 1;
  const startDate = mom.add(-day, 'days').format(DATE_FORMAT);
  const endDate = moment(startDate, DATE_FORMAT).add(6, 'days').format(DATE_FORMAT);
  return { startDate, endDate };
};

const addDays = (date, days) => {
  return moment(date, DATE_FORMAT).add(days, 'days').format(DATE_FORMAT);
};

const monthNames = ['一月', '二月', '三月', '四月', '五月', '六月', '七月', '八月', '九月', '十月', '十一月', '十二月'];
const weekNames = ['周一', '周二', '周三', '周四', '周五', '周六', '周日'];

const minuteH = 0.8;

const statusColors = {
  [dicts.CLASS_STATUS_NOT_START]: { primary: styleVars.infoColor, bg: styleVars.infoColorBg },
  [dicts.CLASS_STATUS_CHECKED]: { primary: styleVars.successColor, bg: styleVars.successColorBg },
  [dicts.CLASS_STATUS_PAYED]: { primary: styleVars.warningColor, bg: styleVars.warningColorBg },
  [dicts.CLASS_STATUS_CONFLICT]: { primary: styleVars.highlightColor, bg: styleVars.highlightColorBg },
  [dicts.CLASS_STATUS_LEAVE]: { primary: styleVars.primaryColor, bg: styleVars.primaryColorBg },
  [dicts.CLASS_STATUS_CANCEL]: { primary: styleVars.fontColorPrimary, bg: styleVars.fontColorTips },
};

const panelStartTime = '06:00';
const panelEndTime = '23:00';

const getPanelTimes = () => {
  let times = [];
  for (let i = panelStartTime; ; i = moment(i, TIME_FORMAT).add(1, 'hours').format(TIME_FORMAT)) {
    times.push(i.slice(0, 5));
    if (moment(i, TIME_FORMAT).valueOf() === moment(panelEndTime, TIME_FORMAT).valueOf()) {
      break;
    }
  }
  return times;
};

const isConflict = (start1, end1, start2, end2) => {
  return (
    (moment(start1, TIME_FORMAT).valueOf() < moment(end2, TIME_FORMAT).valueOf() &&
      moment(end1, TIME_FORMAT).valueOf() > moment(start2, TIME_FORMAT).valueOf()) ||
    (moment(start2, TIME_FORMAT).valueOf() < moment(end1, TIME_FORMAT).valueOf() &&
      moment(end2, TIME_FORMAT).valueOf() > moment(start1, TIME_FORMAT).valueOf())
  );
};

const getNodeList = (cardList) => {
  let nodeList = [];
  cardList.forEach((card) => {
    const { startTime, endTime } = card;
    let conflictNodes = nodeList.filter((node) => node && isConflict(startTime, endTime, node.startTime, node.endTime));
    if (conflictNodes.length === 0) {
      nodeList.push({ startTime, endTime, cardList: [card] });
    } else {
      let newStartTime = startTime;
      let newEndTime = endTime;
      let newCardList = [];
      conflictNodes.forEach((node) => {
        newStartTime = moment(node.startTime, TIME_FORMAT).isBefore(moment(newStartTime, TIME_FORMAT))
          ? node.startTime
          : newStartTime;
        newEndTime = moment(node.endTime, TIME_FORMAT).isAfter(moment(newEndTime, TIME_FORMAT))
          ? node.endTime
          : newEndTime;
        newCardList = [...newCardList, ...node.cardList];
        node['delete'] = true;
      });
      nodeList = nodeList.filter((node) => node.delete !== true);
      nodeList.push({ startTime: newStartTime, endTime: newEndTime, cardList: [...newCardList, card] });
    }
  });
  return nodeList;
};

const getTimePx = (start, end) => {
  return moment(end, TIME_FORMAT).diff(moment(start, TIME_FORMAT), 'minutes') * minuteH;
};

const nodeMaxCardCountMap = [
  { winSize: 'small', nodeSize: 'small', count: 2 },
  { winSize: 'small', nodeSize: 'normal', count: 4 },
  { winSize: 'small', nodeSize: 'large', count: 9999 },
  { winSize: 'normal', nodeSize: 'small', count: 3 },
  { winSize: 'normal', nodeSize: 'normal', count: 6 },
  { winSize: 'normal', nodeSize: 'large', count: 9999 },
];

const getWinSize = (winW) => {
  return winW < 1920 ? 'small' : 'normal';
};

const getNodeMaxCardCount = (winW, nodeSize) => {
  return nodeMaxCardCountMap.find((m) => m.winSize === getWinSize(winW) && m.nodeSize === nodeSize).count;
};

const productTypeTagMap = {
  [dicts.PRODUCT_TYPE_ECS]: '精',
  [dicts.PRODUCT_TYPE_LECTURE]: '讲',
  [dicts.PRODUCT_TYPE_ONE]: '一',
  [dicts.PRODUCT_TYPE_OTM]: '多',
  [dicts.PRODUCT_TYPE_SMALL]: '班',
  [dicts.PRODUCT_TYPE_TWO]: '双',
  [dicts.PRODUCT_TYPE_TWO_ASSIST]: '双',
};

const panelsMaxCount = 10;

const getDotStr = (arry, split) => {
  const a = arry ? arry.filter((a) => !!a) : [];
  return a.join(split === null || split === undefined ? '·' : split);
};

export default {
  getWeekByDate,
  monthNames,
  weekNames,
  minuteH,
  statusColors,
  getPanelTimes,
  panelStartTime,
  panelEndTime,
  getTimePx,
  getNodeList,
  nodeMaxCardCountMap,
  getNodeMaxCardCount,
  productTypeTagMap,
  getWeekRange,
  addDays,
  panelsMaxCount,
  getDotStr,
};
