import React from 'react';
import Modal from 'COMPONENT/Modal';
import { saveClassinKey } from 'SERVICE/product';
import { Input, message } from 'antd';

class ClassinResModal extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      sid: '',
      secret: '',
    };
  }

  /**
   * 取消
   */
  onCancel = () => {
    this.setState({ status: 1 });
    this.props.onClose();
  };

  /**
   *
   * @returns 保存
   */
  save = async () => {
    const { sid, secret } = this.state;
    if (sid === '' || secret === '') {
      return message.error('sid 或者 secret 不能为空');
    }
    const Params = {
      campusId: this.props.campusId,
      sid,
      secret,
    };
    const apiData = await saveClassinKey(Params);
    if (apiData.resultCode !== 0) {
      throw new Error(apiData.resultMessage);
    } else {
      message.success('保存成功');
    }
    // 刷新列表
    this.props.loadList();
    this.props.onClose();
  };

  getWidthSize = () => {
    const { status } = this.state;

    return status === 2 ? 'M' : '';
  };

  /**
   *
   * @returns 渲染body
   */
  renderBody = () => {
    const { sid, secret } = this.state;
    return (
      <div>
        <div>
          <p>请按照以下文档的指引前往classin注册学习中心，注册完学习中心后返回本页面填写SID和SECRET。</p>
          <p>【腾讯文档】新学习中心注册classin操作指引</p>
          <p>
            <a href="https://docs.qq.com/doc/DR0tjZkVwY2hLcHFZ" target="_blank">
              https://docs.qq.com/doc/DR0tjZkVwY2hLcHFZ
            </a>
          </p>

          <Input
            addonBefore="SID"
            value={sid}
            onChange={(e) => {
              this.setState({ sid: e.target.value });
            }}
            style={{ marginTop: '10px', marginBottom: '10px' }}
          />
          <Input
            addonBefore="SECRET"
            defaultValue=""
            value={secret}
            onChange={(e) => {
              this.setState({ secret: e.target.value });
            }}
          />
        </div>
      </div>
    );
  };

  render() {
    return (
      <Modal
        loading={this.state.loading}
        title={'注册Classin'}
        widthSize={this.getWidthSize()}
        width="466"
        onCancel={this.onCancel}
        onOk={this.save}
        okText="保存"
        cancelText="取消"
      >
        {this.renderBody()}
      </Modal>
    );
  }
}

export default ClassinResModal;
