import React from 'react';
import { connect } from 'dva';
import { Page, List, SearchFilter, Button, Select, SearchField } from 'COMPONENT';
import * as Api from 'SERVICE/preferential';
import { getPageConfig, getUserBranchList } from 'SERVICE/product';
import { buildBranchCampusDict } from 'MODEL/dict/utils';
import ClassinResModal from './ClassinModal/ClassinResModal';
import ClassinSynModal from './ClassinModal/ClassinSynModal';
import ClassinSynMutileModal from './ClassinModal/ClassinSynMutileModal';
import ClearModal from './ClassinModal/ClearModal';
import { message } from 'antd';

const { COL_WIDTH_M, COL_WIDTH_L } = List;

class ClassinSetting extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 内容加载状态
      branchDict: null, // {[object]} 字典 所属分公司
      campusDict: [], // 学习中心字典
      resStatusDict: [
        {
          val: 0,
          text: '未注册',
        },
        {
          val: 1,
          text: '已注册',
        },
      ], //
      searchParams: {
        branchId: null, // {string} 查询条件 所属分公司
        campusId: null, // 学习中心选择ID
        registerStatus: null, //  注册状态
      },

      list: [], // {Array} 列表数据
      listLoading: false, // {false} 列表数据加载状态
      listSearchKeys: null, // {object} 列表数据的查询关键字
      totalRecords: 0, // {number} 总记录数
      currentPage: 1, // {number} 当前页码
      pageSize: 20, // {number} 每页记录数

      campusIdIn: '', // 选择的注册classin
      campusIds: [], // 校区ID ,String

      crows: [],
      ClassinResModalVisible: false, // 注册Classin弹窗
      ClassinSynModalVisible: false, // 设置Classin同步弹窗
      ClassinSynMutileModalVisible: false, // 批量设置Clssin同步弹窗
      ClearModalVisible: false, //  确认清除弹窗

      autoSyncOto: false,
      autoSyncOtm: false,
      autoSyncMiniclass: false,
    };

    this.onSelect = this.onSelect.bind(this);
    this.onSelectAll = this.onSelectAll.bind(this);
    this.loadList = this.loadList.bind(this);
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化内容
   */
  initContent = async () => {
    const { resultCode, data } = await getUserBranchList({ resourceId: 'ClassinSet' });
    if (resultCode === 0) {
      this.setState({ branchDict: data.map((r) => ({ val: r.id, text: r.name })) });
    }
  };

  listUserAllCampusByResourceIdAndBranchId = async (branchId) => {
    const response = await Api.listUserAllCampusByResourceIdAndBranchId({ resourceId: 'ClassinSet', branchId });
    if (response && response.resultCode === 0 && response.data && response.data.length) {
      this.setState(() => {
        return {
          campusDict: response.data.map((ele) => {
            return {
              text: ele.name,
              val: ele.id,
            };
          }),
          searchParams: {
            branchId,
          },
        };
      });
    } else {
      this.setState(() => {
        return {
          campusDict: [],
          searchParams: {
            branchId,
          },
        };
      });
    }
  };

  /**
   * 加载列表数据，并转换格式
   * @param  {object} changedParams 要更新的列表数据参数，包括查询条件、页码状态
   */
  loadList = async (changedParams) => {
    const { currentPage, pageSize, searchParams } = this.state;
    const apiParams = {
      pageSize: changedParams ? changedParams.pageSize : pageSize,
      campusId: searchParams.campusId,
      registerStatus: searchParams.registerStatus,
      pageNum: changedParams ? changedParams.current : currentPage,
      branchId: searchParams.branchId,
    };
    try {
      this.setState({ listLoading: true });
      const apiData = await getPageConfig(apiParams);
      const { pageNum, list, total } = apiData.data;
      this.setState({
        listLoading: false,
        list: list,
        totalRecords: total,
        currentPage: pageNum,
        pageSize: apiData.data.pageSize,
        selectedRowKeys: [],
      });
    } catch (error) {
      console.log(error);
      this.setState({
        listLoading: false,
      });
    } finally {
      this.setState({
        listLoading: false,
      });
    }
  };

  /**
   * 查询
   */
  search = () => {
    const { branchId } = this.state.searchParams;
    branchId ? this.loadList({ pageSize: 20, current: 1 }) : message.error('分公司不能为空');
  };

  /**
   * 重置查询条件
   */
  reset = () => {
    this.setState({
      searchParams: {
        branchId: null, // {string} 查询条件 所属分公司
        campusId: null, // 学习中心选择ID
        registerStatus: null, //  注册状态
      },
    });
  };

  // 表格选择
  onSelect(record, selected) {
    this.setState((pre) => {
      const { campusIds, crows } = pre;
      if (selected) {
        crows.push(record);
        campusIds.push(record.campusId);
      } else {
        const idx = campusIds.findIndex((item) => item === record.campusId);
        if (idx > -1) {
          campusIds.splice(idx, 1);
        }
        const sdx = crows.findIndex((idm) => idm.key === record.key);
        if (sdx > -1) {
          crows.splice(sdx, 1);
        }
      }
      return { campusIds, crows };
    });
  }

  // 全选
  onSelectAll(selected, rows, changeRows) {
    const ArrayList = [];
    changeRows.forEach((e) => {
      ArrayList.push(e.campusId);
    });
    this.setState((pre) => {
      let { campusIds, crows } = pre;
      if (selected) {
        crows = crows.concat(changeRows);
        campusIds = campusIds.concat(ArrayList);
      } else {
        ArrayList.forEach((row) => {
          const idx = campusIds.findIndex((item) => item === row);
          if (idx > -1) {
            campusIds.splice(idx, 1);
          }
        });

        changeRows.forEach((row) => {
          const idx = crows.findIndex((item) => item.key === row.key);
          if (idx > -1) {
            crows.splice(idx, 1);
          }
        });
      }
      return { campusIds, crows };
    });
  }

  // 批量按钮
  mutilSync = () => {
    const { campusIds, crows } = this.state;
    console.log(crows.findIndex((item) => item.registerStatus === 0));
    if (crows.length > 0 && crows.findIndex((item) => item.registerStatus === 0) > -1) {
      return message.error('选项中含有未注册Classin的学习中心，请重新勾选！');
    }
    campusIds.length === 0
      ? message.error('至少选择1个学习中心!')
      : this.setState({ ClassinSynMutileModalVisible: true });
  };

  onSelectChange = (selectedRowKeys) => {
    this.setState({ selectedRowKeys });
  };

  // 清除按钮
  clearAll = () => {
    const { campusIds, crows } = this.state;
    if (crows.length > 0 && crows.findIndex((item) => item.registerStatus === 0) > -1) {
      return message.error('选项中含有未注册Classin的学习中心，请重新勾选！');
    }
    campusIds.length === 0 ? message.error('至少选择1个学习中心!') : this.setState({ ClearModalVisible: true });
  };

  // 设置同步
  batchSetUpAutoSyncConfig = (configObject) => {
    const { campusId, autoSyncOto, autoSyncOtm, autoSyncMiniclass } = configObject;
    const campusIds = [];
    campusIds.push(campusId);
    this.setState({
      campusIds,
      ClassinSynModalVisible: true,
      autoSyncOto,
      autoSyncOtm,
      autoSyncMiniclass,
    });
  };

  render() {
    const {
      campusIds,
      loading,
      searchParams,
      ClassinResModalVisible,
      ClassinSynModalVisible,
      ClassinSynMutileModalVisible,
      selectedRowKeys,
      ClearModalVisible,
      list,
      totalRecords,
      currentPage,
      pageSize,
      listLoading,
      campusDict,
      branchDict,
      resStatusDict,
      campusIdIn,
      autoSyncOto,
      autoSyncOtm,
      autoSyncMiniclass,
    } = this.state;
    const columns = [
      { title: '分公司', dataIndex: 'branchName', width: COL_WIDTH_M },
      { title: '学习中心', dataIndex: 'campusName', width: COL_WIDTH_L },
      {
        title: '学习中心注册状态',
        dataIndex: 'registerStatus',
        width: COL_WIDTH_L,
        render: (text, record) => {
          return text === 0 ? '未注册' : '已注册';
        },
      },
      {
        title: '1对1课程',
        dataIndex: 'autoSyncOto',
        width: COL_WIDTH_M,
        render: (text, record) => {
          return text ? '已开通自动同步' : record.registerStatus === 0 ? '请先注册classin' : '未开通';
        },
      },
      {
        title: '1对多课程',
        dataIndex: 'autoSyncOtm',
        width: COL_WIDTH_M,
        render: (text, record) => {
          return text ? '已开通自动同步' : record.registerStatus === 0 ? '请先注册classin' : '未开通';
        },
      },
      {
        title: '小班课程',
        dataIndex: 'autoSyncMiniclass',
        width: COL_WIDTH_M,
        render: (text, record) => {
          return text ? '已开通自动同步' : record.registerStatus === 0 ? '请先注册classin' : '未开通';
        },
      },
      {
        title: '操作',
        key: 'action',
        width: COL_WIDTH_M,
        render: (text, record) => {
          return record.registerStatus === 0 ? (
            <Button
              disabled={!this.props.btnAuth_resSet}
              onClick={() => this.setState({ campusIdIn: record.campusId, ClassinResModalVisible: true })}
            >
              注册classin
            </Button>
          ) : (
            <Button disabled={!this.props.btnAuth_syncSet} onClick={() => this.batchSetUpAutoSyncConfig(record)}>
              设置同步
            </Button>
          );
        },
      },
    ];

    // 表格选择
    const rowSelection = {
      selectedRowKeys,
      onSelect: this.onSelect,
      onSelectAll: this.onSelectAll,
      onChange: this.onSelectChange,
    };
    return (
      <Page
        fitWindowHeight={true}
        loading={loading}
        route={this.props.route}
        subHeader={
          <SearchFilter truncationNum={4} onSearch={this.search} onReset={this.reset}>
            <SearchField>
              <Select
                value={searchParams.branchId}
                placeholder="请选择分公司"
                allowClear
                showSearch
                dict={branchDict}
                onChange={this.listUserAllCampusByResourceIdAndBranchId}
              />
            </SearchField>
            <SearchField>
              <Select
                value={searchParams.campusId}
                placeholder="请选择学习中心"
                allowClear
                showSearch
                dict={campusDict}
                onChange={(val) => this.setState({ searchParams: { ...searchParams, campusId: val } })}
              />
            </SearchField>
            <SearchField>
              <Select
                value={searchParams.registerStatus}
                placeholder="学习中心注册状态"
                allowClear
                dict={resStatusDict}
                onChange={(val) => this.setState({ searchParams: { ...searchParams, registerStatus: val } })}
              />
            </SearchField>
          </SearchFilter>
        }
        headerToolBar={
          <div
            style={{
              height: '100%',
              display: 'flex',
              alignItems: 'center',
              justifyContent: 'space-between',
              float: 'right',
            }}
          >
            <Button disabled={!this.props.btnAuth_syncSet} type="primary" onClick={() => this.mutilSync()}>
              批量设置同步
            </Button>
            <Button
              disabled={!this.props.btnAuth_syncSet}
              type="primary"
              style={{ marginLeft: '20px' }}
              onClick={() => {
                this.clearAll();
              }}
            >
              一键清除已同步课程
            </Button>
          </div>
        }
      >
        <List
          dataSource={list}
          columns={columns}
          loading={!loading && listLoading}
          pagination={{
            current: currentPage,
            total: totalRecords,
            pageSize,
          }}
          rowSelection={rowSelection}
          onChange={({ current, pageSize }) => {
            this.setState({ pageSize: pageSize, currentPage: current });
            this.loadList({ current, pageSize });
          }}
        />

        {
          /**
           * 注册Classin
           */
          ClassinResModalVisible && (
            <ClassinResModal
              campusId={campusIdIn}
              loadList={this.loadList}
              onClose={() => {
                this.setState({ ClassinResModalVisible: false, campusIds: [], selectedRowKeys: [], crows: [] });
              }}
            />
          )
        }
        {
          /**
           * 设置Classin同步
           */
          ClassinSynModalVisible && (
            <ClassinSynModal
              loadList={this.loadList}
              autoSyncOto={autoSyncOto}
              autoSyncOtm={autoSyncOtm}
              autoSyncMiniclass={autoSyncMiniclass}
              campusIds={campusIds}
              onClose={() => {
                this.setState({ ClassinSynModalVisible: false, campusIds: [], selectedRowKeys: [], crows: [] });
              }}
            />
          )
        }
        {
          /**
           * 批量设置Classin同步
           */
          ClassinSynMutileModalVisible && (
            <ClassinSynMutileModal
              loadList={this.loadList}
              campusIds={campusIds}
              onClose={() => {
                this.setState({ ClassinSynMutileModalVisible: false, campusIds: [], selectedRowKeys: [], crows: [] });
              }}
            />
          )
        }
        {
          /**
           * 取消提交
           */
          ClearModalVisible && (
            <ClearModal
              loadList={this.loadList}
              campusIds={campusIds}
              onClose={() => {
                this.setState({ ClearModalVisible: false, campusIds: [], selectedRowKeys: [], crows: [] });
              }}
            />
          )
        }
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const btnAuth = state.account.btnAuths;
  const btnAuth_syncSet = btnAuth.indexOf('ARTICLE_PERSONAL_INFO_SYNC_SET') > -1;
  const btnAuth_resSet = btnAuth.indexOf('ARTICLE_PERSONAL_INFO_RES_SET') > -1;

  return {
    btnAuth_syncSet,
    btnAuth_resSet,
  };
};

export default connect(mapStateToProps)(ClassinSetting);
