/**
 * Created by Haolin on 2019/2/28.
 */
import React, { PropTypes } from 'react';
import { connect } from 'dva';
import style from './index.less';
import Modal from 'COMPONENT/Modal';
import FormItem from 'COMPONENT/FormItem';
import Checkbox from 'COMPONENT/Checkbox';

class ModalUpdateClass extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      name: '课程更新',
      loading: false,
      showSmallClass: true,
      showTwoTeacher: true,
      checkSmallClass: false,
      checkTwoTeacher: false,
      tips: '更新规则如下：原来A班有四讲课程，第一讲2月7日（已考勤），第二讲2月14日（有学生调出课程），第三讲2月21日（有学生调入课程），第四讲2月28日（未考勤），现在把A班的课程模板改为3月7日/3月14日/3月21日/3月28日，则只删除2月28日的课程，并创建3月28日的课程，即不调整已经考勤和存在学生调课的课程。',
    };
    this.init = this.init.bind(this);
    this.updateCheckbox = this.updateCheckbox.bind(this);
    this.confirm = this.confirm.bind(this);
  }
  static classPrefix = 'ModalUpdateClass';
  static propTypes = {
    smallClassNum: PropTypes.number.isRequired,
    twoTeacherNum: PropTypes.number.isRequired,
    onCancel: PropTypes.func.isRequired,
    onConfirm: PropTypes.func.isRequired,
  };
  componentDidMount() {
    this.init();
  }

  init() {
    const { smallClassNum, twoTeacherNum } = this.props;
    let checkSmallClass = smallClassNum > 0;
    let checkTwoTeacher = twoTeacherNum > 0;
    this.setState({
      showSmallClass: checkSmallClass,
      showTwoTeacher: checkTwoTeacher,
      checkSmallClass,
      checkTwoTeacher,
    });
  }
  updateCheckbox({ type, val }) {
    this.setState({ [type]: val });
  }
  confirm() {
    let params = {};
    if (this.state.checkSmallClass) params.smallClass = true;
    if (this.state.checkTwoTeacher) params.twoTeacher = true;
    this.props.onConfirm(Object.keys(params).length > 0 ? params : null);
  }

  render() {
    const { smallClassNum, twoTeacherNum, onCancel } = this.props;
    const { name, loading, showSmallClass, showTwoTeacher, checkSmallClass, checkTwoTeacher, tips } = this.state;
    const prefix = ModalUpdateClass.classPrefix;
    let _smallClassNum = checkSmallClass ? smallClassNum : 0;
    let _twoTeacherNum = checkTwoTeacher ? twoTeacherNum : 0;

    return (
      <Modal
        className={style[prefix]}
        title={name}
        closable={true}
        loading={loading}
        onOk={this.confirm}
        onCancel={onCancel}
        widthSize="S"
      >
        <FormItem name="产品类型" labelWidth={100} required>
          {showSmallClass && (
            <Checkbox
              checked={checkSmallClass}
              onChange={(e) => {
                this.updateCheckbox({ type: 'checkSmallClass', val: e.target.checked });
              }}
            >
              小班
            </Checkbox>
          )}
          {showTwoTeacher && (
            <Checkbox
              checked={checkTwoTeacher}
              onChange={(e) => {
                this.updateCheckbox({ type: 'checkTwoTeacher', val: e.target.checked });
              }}
            >
              双师
            </Checkbox>
          )}
        </FormItem>
        <p className={style[`${prefix}-prompt`]}>
          本次更新班级数量为
          <span className={style[`${prefix}-important`]}>{_smallClassNum + _twoTeacherNum}</span>
          个，请于1小时后查看更新情况。
        </p>
        <p className={style[`${prefix}-prompt`]}>{tips}</p>
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  return {};
};

export default connect(mapStateToProps)(ModalUpdateClass);
