/**
 * 月历组件
 * @property {number} year 年份
 * @property {number} month 月份 0 ~ 11
 * @property {[string]} selectedDates 选中的日期，YYYY-MM-DD
 * @property {[string]} extraSelectedDates 超额选中的日期，YYYY-MM-DD
 * @property {function(string)} onSelectDate 选中日期时的回调
 * @property {function(number, number)} onSelectWeekDay 选中星期时的回调
 */
import React from 'react';
import ReactDOM from 'react-dom';
import moment from 'moment';
import { Calendar } from 'antd';
import { DATE_FORMAT } from 'COMPONENT/utils/form';
import styles from './styles.less';

/*
{
  "reason":"Success",
  "result":{
    "data":{
      "avoid":"开市.入宅.安床.动土.安葬.",
      "animalsYear":"鸡",
      "weekday":"星期五",
      "suit":"祭祀.进人口.纳财.纳畜.牧养.捕捉.余事勿取.",
      "lunarYear":"丁酉年",
      "lunar":"闰六月初六",
      "year-month":"2017-7",
      "date":"2017-7-28"
    }
  },
  "error_code":0
}

{
  "reason":"Success",
  "result":{
    "data":{
      "holiday":"[{\"name\":\"清明节\",\"festival\":\"2017-4-4\",\"desc\":\"4月2日至4日放假调休，共3天。4月1日（星期六）上班。\",\"list\":[{\"date\":\"2017-4-1\",\"status\":\"2\"},{\"date\":\"2017-4-2\",\"status\":\"1\"},{\"date\":\"2017-4-3\",\"status\":\"1\"},{\"date\":\"2017-4-4\",\"status\":\"1\"}],\"list#num#\":4},{\"name\":\"劳动节\",\"festival\":\"2017-5-1\",\"desc\":\"5月1日放假，与周末连休。\",\"list\":[{\"date\":\"2017-4-29\",\"status\":\"1\"},{\"date\":\"2017-4-30\",\"status\":\"1\"},{\"date\":\"2017-5-1\",\"status\":\"1\"}],\"list#num#\":3},{\"name\":\"端午节\",\"festival\":\"2017-5-30\",\"desc\":\"5月28日至30日放假调休，共3天。5月27日（星期六）上班。\",\"list\":[{\"date\":\"2017-5-27\",\"status\":\"2\"},{\"date\":\"2017-5-28\",\"status\":\"1\"},{\"date\":\"2017-5-29\",\"status\":\"1\"},{\"date\":\"2017-5-30\",\"status\":\"1\"}],\"list#num#\":4}]",
      "year":"2017",
      "year-month":"2017-5"
    }
  },
  "error_code":0
}

{
  "reason":"Success",
  "result":{
    "data":{
      "holidaylist":"[{\"name\":\"元旦\",\"startday\":\"2017-1-1\"},{\"name\":\"除夕\",\"startday\":\"2017-1-27\"},{\"name\":\"春节\",\"startday\":\"2017-1-28\"},{\"name\":\"清明节\",\"startday\":\"2017-4-4\"},{\"name\":\"劳动节\",\"startday\":\"2017-5-1\"},{\"name\":\"端午节\",\"startday\":\"2017-5-30\"},{\"name\":\"国庆节\",\"startday\":\"2017-10-1\"},{\"name\":\"中秋节\",\"startday\":\"2017-10-4\"}]",
      "year":"2017"
    }
  },
  "error_code":0
}
 */

const dateInfoList = {
  /*'2017-01-01': { lunarMonth: '一月', lunarDate: '二八', holiday: '2' },
  '2017-01-02': { lunarMonth: '一月', lunarDate: '二九', holiday: '1' },
  '2017-01-03': { lunarMonth: '一月', lunarDate: '三十', holiday: '1' },
  '2017-01-04': { lunarMonth: '闰一月', lunarDate: '初一', festival: '春节', holiday: '1' },
  '2017-01-05': { lunarMonth: '闰一月', lunarDate: '初二' },
  '2017-01-06': { lunarMonth: '闰一月', lunarDate: '初三' },*/
};

class MonthCalendar extends React.Component {
  dateHideStarted = false;

  dateFullCellRender = (m) => {
    const { year, month, selectedDates, extraSelectedDates } = this.props;
    const date = m.date();
    const day = m.day();
    const mStr = m.format(DATE_FORMAT);
    const isDisabled = m.month() !== month;
    const isSelected = selectedDates.find((d) => d === mStr) && !isDisabled;
    const isExtraSelected = extraSelectedDates.find((d) => d === mStr) && !isDisabled;
    let isHide = false;
    if (m.year() > year || (m.year() === year && m.month() > month)) {
      if (date < 20 && day === 1) {
        this.dateHideStarted = true;
      }
    } else {
      this.dateHideStarted = false;
    }
    if (this.dateHideStarted) {
      isHide = true;
    }
    const { lunarMonth, lunarDate, holiday, festival } = dateInfoList[mStr] || {};

    return (
      <div
        className={
          styles.cell +
          (day === 0 || day === 6 ? ' ' + styles.weekendCell : '') +
          (isSelected ? ' ' + (holiday === '1' ? styles.yellowCell : styles.greenCell) : '') +
          (isExtraSelected ? ' ' + styles.redCell : '') +
          (isDisabled ? ' ' + styles.disabledCell : '') +
          (isHide ? ' ' + styles.hideCell : '')
        }
      >
        <div>
          <div className={styles.cellLeftTop}>{date}</div>
          <div className={styles.cellRightTop}>{festival || (lunarDate === '初一' ? lunarMonth : lunarDate)}</div>
          {holiday && <div className={holiday === '1' ? styles.holiday : styles.work}></div>}
        </div>
      </div>
    );
  };

  selectDate = (m) => {
    const { month, onSelectDate } = this.props;
    if (m.month() !== month) {
      return;
    }
    onSelectDate(m.format(DATE_FORMAT));
  };

  disabledDate = (m) => {
    const { month } = this.props;
    if (m.month() !== month) {
      return true;
    }
  };

  render() {
    const { year, month, selectedDates, extraSelectedDates, onSelectDate, onSelectWeekDay } = this.props;
    return (
      <div className={styles.wrap}>
        <div className={styles.title}>{`${year}年${month + 1}月`}</div>
        <table className={styles.weekBar}>
          <tbody>
            <tr>
              <td onClick={() => onSelectWeekDay(1)}>周一</td>
              <td onClick={() => onSelectWeekDay(2)}>周二</td>
              <td onClick={() => onSelectWeekDay(3)}>周三</td>
              <td onClick={() => onSelectWeekDay(4)}>周四</td>
              <td onClick={() => onSelectWeekDay(5)}>周五</td>
              <td onClick={() => onSelectWeekDay(6)}>周六</td>
              <td onClick={() => onSelectWeekDay(0)}>周日</td>
            </tr>
          </tbody>
        </table>
        <Calendar
          value={moment([year, month, 1])}
          dateFullCellRender={this.dateFullCellRender}
          onSelect={this.selectDate}
          disabledDate={this.disabledDate}
        />
      </div>
    );
  }
}

export default MonthCalendar;
