import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import List from 'COMPONENT/List';
import SearchBar from 'COMPONENT/SearchBar';
import Select from 'COMPONENT/Select';
import Input from 'COMPONENT/Input';
import { buildBranchCampusDict } from 'MODEL/dict/utils';
import { storeName } from './store/config';
import moment from 'moment';

const typeDict = [
  { val: 'DONATE_REMIND_ITEM_OOO_TEA_OATT', text: '老师已上课考勤逾期' },
  { val: 'DONATE_REMIND_ITEM_OOO_TEA_NATT', text: '老师已上课未考勤' },
  { val: 'DONATE_REMIND_ITEM_OOO_TEA_TWAIT', text: '老师当天待考勤' },
  { val: 'DONATE_REMIND_ITEM_OOO_STY_OADU', text: '学管逾期确认' },
  { val: 'DONATE_REMIND_ITEM_OOO_STY_NADU', text: '学管未确认' },
  { val: 'DONATE_REMIND_ITEM_OOO_STY_TWAIT', text: '学管当天待确认' },
  { val: 'DONATE_REMIND_ITEM_OOO_TMA_OADU', text: '教务逾期扣费' },
  { val: 'DONATE_REMIND_ITEM_OOO_TMA_NADU', text: '教务未扣费' },
  { val: 'DONATE_REMIND_ITEM_OOO_TMA_TWAIT', text: '教务当天待扣费' },
];

const defaultState = {
  branchId: '',
  campusId: '',
  teacherName: '',
  studyManagerName: '',
  teachingManagerName: '',
};

class ScheduleAttendanceReminder extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      ...defaultState,
      loading: false,
      type: this.props.params === undefined ? '' : this.props.params.itemType,

      pagination: {
        current: 1,
        pageSize: 20,
        total: 0,
        pageSizeOptions: ['20', '50', '100'],
      }, // 列表分页
    };

    this.init = this.init.bind(this);
    this.onSearch = this.onSearch.bind(this);
    this.loadDict = this.loadDict.bind(this);
    this.onReset = this.onReset.bind(this);
    this.columns = [
      { title: '学生姓名', dataIndex: 'studentName', key: 'studentName' },
      { title: '家长姓名', dataIndex: 'customerName', key: 'customerName' },
      { title: '所属分公司', dataIndex: 'branchName', key: 'branchName' },
      { title: '所属学习中心', dataIndex: 'campusName', key: 'campusName' },
      { title: '联系方式', dataIndex: 'customerContact', key: 'customerContact' },
      { title: '老师', dataIndex: 'teacherName', key: 'teacherName' },
      { title: '课程日期', dataIndex: 'courseDate', key: 'courseDate' },
      {
        title: '考勤日期',
        dataIndex: 'firstAttendentTime',
        key: 'firstAttendentTime',
        render: (val) => {
          return val === null ? '' : moment(val).format('YYYY-MM-DD');
        },
      },
      { title: '学管师', dataIndex: 'studyManagerName', key: 'studyManagerName' },
      {
        title: '学管确认日期',
        dataIndex: 'studyManagerAuditTime',
        key: 'studyManagerAuditTime',
        render: (val) => {
          return val === null ? '' : moment(val).format('YYYY-MM-DD');
        },
      },
      { title: '教务', dataIndex: 'teachingManagerName', key: 'teachingManagerName' },
      {
        title: '教务扣费日期',
        dataIndex: 'teachingManagerAuditTime',
        key: 'teachingManagerAuditTime',
        render: (val) => {
          return val === null ? '' : moment(val).format('YYYY-MM-DD');
        },
      },
      {
        title: '乐捐提醒类型',
        dataIndex: 'teacherOverdueAtt',
        key: 'teacherOverdueAtt',
        width: 124,
        render: (val, row) => {
          const {
            teacherOverdueAtt,
            teacherNotAtt,
            teacherTodayWaitingAtt,
            studyManagerOverdueAudit,
            studyManagerNotAudit,
            studyManagerTodayWaitingAudit,
            teachingManagerOverdueAudit,
            teachingManagerNotAudit,
            teachingManagerTodayWaitingAudit,
          } = row;
          return (
            <div>
              {teacherOverdueAtt && <p>老师已上课考勤逾期</p>}
              {teacherNotAtt && <p>老师已上课未考勤</p>}
              {teacherTodayWaitingAtt && <p>老师当天待考勤数据</p>}
              {studyManagerOverdueAudit && <p>学管逾期确认数据</p>}
              {studyManagerNotAudit && <p>学管未确认数据</p>}
              {studyManagerTodayWaitingAudit && <p>学管当天待确认数据</p>}
              {teachingManagerOverdueAudit && <p>教务逾期扣费数据</p>}
              {teachingManagerNotAudit && <p>教务未扣费数据</p>}
              {teachingManagerTodayWaitingAudit && <p>教务当天待扣费数据</p>}
            </div>
          );
        },
      },
    ];
  }

  componentDidMount() {
    this.init();
  }

  async init() {
    this.setState({ loading: true });
    await new Promise(this.loadDict);
    const { branchDict, branchCampusDict } = this.props;
    const branchId = this.props.params === undefined ? branchDict[0].val : '';
    const campusId =
      this.props.params === undefined ? buildBranchCampusDict(branchId, branchCampusDict).options[0].val : '';
    this.setState({ branchId, campusId });
    await this.onSearch();
    this.setState({ loading: false });
  }

  loadDict(resolve, reject) {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['branch', 'branchCampus'],
        callback() {
          resolve();
        },
      },
    });
  }

  async onSearch() {
    this.setState({ loading: true });
    const {
      branchId,
      campusId,
      teacherName,
      studyManagerName,
      teachingManagerName,
      type,
      pagination: { current: page, pageSize: rows },
    } = this.state;
    await new Promise((resolve) => {
      this.props.dispatch({
        type: `${storeName}/loadData`,
        payload: {
          params: {
            branchId,
            campusId,
            teacherName,
            studyManagerName,
            teachingManagerName,
            type,
            page,
            rows,
          },
          callback() {
            resolve();
          },
        },
      });
    });
    this.setState({ loading: false });
  }

  onReset() {
    const { branchDict, branchCampusDict } = this.props;
    const branchId = this.props.params === undefined ? branchDict[0].val : '';
    const campusId =
      this.props.params === undefined ? buildBranchCampusDict(branchId, branchCampusDict).options[0].val : '';
    this.setState({
      ...defaultState,
      branchId,
      campusId,
      type: '',
    });
  }

  render() {
    const { dataSource, total, branchDict, branchCampusDict } = this.props;
    const { loading, pagination, branchId, campusId, teacherName, studyManagerName, teachingManagerName, type } =
      this.state;
    return (
      <Page
        name={this.props.route.name}
        loading={loading}
        subHeader={
          <SearchBar onSearch={this.onSearch} onReset={this.onReset}>
            <Select
              placeholder="分公司"
              value={branchId}
              dict={branchDict}
              onChange={(branchId) =>
                this.setState({
                  branchId,
                  pagination: Object.assign({}, this.state.pagination, { current: 1 }),
                  campusId: '',
                })
              }
              showSearch={true}
            />
            <Select
              placeholder="学习中心"
              value={campusId}
              dict={branchId === '' ? [] : buildBranchCampusDict(branchId, branchCampusDict)}
              onChange={(campusId) => {
                this.setState({ campusId, pagination: Object.assign({}, this.state.pagination, { current: 1 }) });
              }}
              showSearch={true}
            />
            <Input
              placeholder="老师姓名"
              value={teacherName}
              onChange={(event) =>
                this.setState({
                  teacherName: event.currentTarget.value,
                  pagination: Object.assign({}, this.state.pagination, { current: 1 }),
                })
              }
            />
            <Input
              placeholder="学管师姓名"
              value={studyManagerName}
              onChange={(event) =>
                this.setState({
                  studyManagerName: event.currentTarget.value,
                  pagination: Object.assign({}, this.state.pagination, { current: 1 }),
                })
              }
            />
            <Input
              placeholder="教务姓名"
              value={teachingManagerName}
              onChange={(event) =>
                this.setState({
                  teachingManagerName: event.currentTarget.value,
                  pagination: Object.assign({}, this.state.pagination, { current: 1 }),
                })
              }
            />
            <Select
              placeholder="乐捐提醒类型"
              value={type}
              dict={typeDict}
              onChange={(type) =>
                this.setState({ type, pagination: Object.assign({}, this.state.pagination, { current: 1 }) })
              }
            />
          </SearchBar>
        }
      >
        <List
          columns={this.columns}
          dataSource={dataSource}
          pagination={Object.assign({}, pagination, { total })}
          onChange={({ current, pageSize }) => {
            this.setState({ pagination: Object.assign({}, this.state.pagination, { current, pageSize }) }, () => {
              this.onSearch();
            });
          }}
        />
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const pathname = state.routing.locationBeforeTransitions.pathname;
  const { branch, branchCampus } = state.dict;
  const { dataSource, total } = state[storeName];
  return {
    branchDict: branch,
    branchCampusDict: branchCampus,
    params: state.main.pageParams[pathname],
    dataSource,
    total,
  };
};

export default connect(mapStateToProps)(ScheduleAttendanceReminder);
