/**
 * 合同信息表格
 * @property {object} data 内容数据，格式如下： 
   { 
    no: string  合同编号 
    actualMoney: number  实收总额
    feeItems: Array  收费项目
      [{ 
          name: string  名称
          money: number  总金额
          discount: number  优惠金额
      }] 
    deductedBaseMoney: number  已扣课时费
    deductedVipMoney: number  已扣VIP服务费
    }
 */
import React from 'react';
import { Row, Col } from 'antd';
import FeeItem from './FeeItem';
import { formatMoney } from './common';
import styles from './styles.less';

const ContractInfo = ({ data }) => {
  const { no, actualMoney, deductedBaseMoney, deductedVipMoney } = data;
  const feeItems = data.feeItems || [];
  let total = 0;
  let discountTotal = 0;
  feeItems.forEach((feeItem) => {
    const { money, discount } = feeItem;
    total += money;
    discountTotal += discount;
  });
  const receivable = total - discountTotal;
  const unReceive = receivable - actualMoney;

  return (
    <table className={styles.tableView}>
      <tbody>
        <tr>
          <th>合同编号</th>
          <th>合同总额</th>
          <th>优惠金额</th>
          <th>应收金额</th>
          <th>实收金额</th>
          <th>待收款</th>
          {/* <th className={styles.colorCell}>已扣课时费</th>
          <th className={styles.colorCell}>已扣VIP服务费</th> */}
        </tr>
        <tr>
          <td>{no}</td>
          <td>
            <div>{formatMoney(total)}</div>
            <div>
              {feeItems.map((c, i) => (
                <FeeItem key={i} name={c.name} money={c.money} />
              ))}
            </div>
          </td>
          <td>
            <div>{formatMoney(discountTotal)}</div>
            <div>
              {feeItems.map((c, i) => (
                <FeeItem key={i} type="discount" name={c.name} money={c.discount} />
              ))}
            </div>
          </td>
          <td>{formatMoney(receivable)}</td>
          <td>{formatMoney(actualMoney)}</td>
          <td>{formatMoney(unReceive)}</td>
          {/* <td className={styles.colorCell}>{formatMoney(deductedBaseMoney)}</td>
          <td className={styles.colorCell}>{formatMoney(deductedVipMoney)}</td> */}
        </tr>
      </tbody>
    </table>
  );
};

export default ContractInfo;
