/**
 * 合同信息表格
 * @property {object} data 内容数据，格式如下：
   {
    total: number  总课时数
    distributed: number  已分配课时数
    unDistributed: number  未分配课时数
    entered: number  已报小班课时数
    consumed: number  已上课时数
   }
 * @property {boolean} canPlanCourse 是否可以跳转到课程规划详情
   @property {function} onViewDetail 查看详情事件的回调
 */
import React from 'react';
import styles from './styles.less';
import TextLink from 'COMPONENT/TextLink';

const CourseHoursInfo = ({ data, canPlanCourse, onViewDetail }) => {
  const { total, distributed, unDistributed, entered, consumed } = data;

  return (
    <table className={styles.tableView}>
      <tbody>
        <tr>
          <th>总小时数</th>
          <th>已分配小时数</th>
          <th>未分配小时数</th>
          <th>已报小班小时数</th>
          <th>已上小时数</th>
          <th>操作</th>
        </tr>
        <tr>
          <td>{total}</td>
          <td>{distributed}</td>
          <td>{unDistributed}</td>
          <td>{entered}</td>
          <td>{consumed}</td>
          <td>
            {canPlanCourse ? <TextLink onClick={onViewDetail}>详情</TextLink> : <TextLink disabled>详情</TextLink>}
          </td>
        </tr>
      </tbody>
    </table>
  );
};

export default CourseHoursInfo;
