/**
 * 课程信息模态框
 * @property {string} miniClassId 小班ID
 * @property {string} studentId 学生ID
 */
import React from 'react';
import { Row, Col, Button } from 'antd';
import Modal from 'COMPONENT/Modal';
import List from 'COMPONENT/List';
import { attendanceDict } from 'CONST/dicts/course';
import { getStudentMiniClassAttendent } from 'SERVICE/course';
import styles from './styles.less';

const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;

class ModalCourseInfo extends React.Component {
  constructor(props) {
    console.log(props);
    super(props);
    this.state = {
      loading: false, // {boolean} 内容加载状态
      list: [], // {Array} 列表数据
    };
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化内容
   */
  initContent = () => {
    const { miniClassId, studentId } = this.props;
    this.setState({ loading: true });
    Promise.all([this.loadCourseInfoList({ miniClassId, studentId })]).then(([list]) => {
      this.setState({ loading: false, list });
    });
  };

  /**
   * 加载课程信息列表
   * @param  {string} miniClassId 小班ID
   * @param  {string} studentId 学生ID
   */
  loadCourseInfoList = async ({ miniClassId, studentId }) => {
    const apiData = await getStudentMiniClassAttendent({ miniClassId, studentId });
    const data = apiData.map((row) => {
      return {
        className: row.name,
        courseNo: '第' + row.courseNum + '讲',
        date: row.courseDate,
        time: row.startTime + ' - ' + row.endTime,
        teacher: row.teacherName,
        attendance: row.attendentStatus,
        started: row.attendentStatus !== 'NEW',
      };
    });
    /*const data = [
      {
        className: 'xx1',
        courseNo: '第1讲',
        date: '2017-05-06',
        time: '10:00 - 11:30',
        teacher: 'xxx',
        attendance: '迟到',
        started: true,
      },
      {
        className: 'xx1',
        courseNo: '第2讲',
        date: '2017-05-06',
        time: '10:00 - 11:30',
        teacher: 'xxx',
        attendance: null,
        started: false,
      },
    ]*/
    return data;
  };

  /**
   * 获取行的类名
   * @param  {object} record 行数据
   * @return {string}
   */
  getRowClassName = (record) => {
    if (record.started) {
      return styles.started;
    } else {
      return '';
    }
  };

  render() {
    const { onClose } = this.props;
    const { loading, list } = this.state;
    const columns = [
      { title: '小班名称', dataIndex: 'className', width: COL_WIDTH_L },
      { title: '课程序号', dataIndex: 'courseNo' },
      { title: '上课日期', dataIndex: 'date' },
      { title: '上课时间', dataIndex: 'time' },
      { title: '老师', dataIndex: 'teacher' },
      { title: '考勤状态', dataIndex: 'attendance', dict: attendanceDict },
    ];
    return (
      <Modal loading={loading} onCancel={onClose} width={960} title="课程信息" okBtnShow={false} cancelText="关闭">
        <List
          dataSource={list}
          columns={columns}
          loading={loading}
          pagination={false}
          rowClassName={this.getRowClassName}
        />
      </Modal>
    );
  }
}

export default ModalCourseInfo;
