import React from 'react';
import List from 'COMPONENT/List';
import Loading from 'COMPONENT/Loading';
import { findMiniClassByPromiseSubjectId, getMiniClassCourseDateInfo } from 'SERVICE/course';
import styles from '../styles.less';

class Main extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      list: [],
      total: 0,
      currentPage: 1,
      pageSize: 20,
      listLoading: false,
      expandedRowKeys: [], //展开的行
    };
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化内容
   */
  initContent = () => {
    this.loadList();
  };

  paginationChange = (pagination) => {
    this.loadList({
      currentPage: pagination.current,
      pageSize: pagination.pageSize,
    });
  };

  loadList = (params = {}) => {
    this.setState({ listLoading: true, expandedRowKeys: [] });
    const { queryStrings } = this.props;
    const { currentPage, pageSize } = this.state;
    const newParams = { currentPage, pageSize, ...params };
    const { currentPage: newCurrentPage, pageSize: newPageSize, ...otherParams } = newParams;
    findMiniClassByPromiseSubjectId({
      ...queryStrings,
      page: newCurrentPage,
      rows: newPageSize,
      ...otherParams,
    }).then(({ rows, records }) => {
      this.setState({
        list: rows,
        total: records,
        currentPage: newCurrentPage,
        pageSize: newPageSize,
        listLoading: false,
      });
    });
  };

  /**
   * 课程信息展现面板
   * @param {object} record 行记录
   * @return {ReactNode}
   */
  getCourseInfo = (record) => {
    const { classStartDate, courseList, startDate } = record;
    return (
      <div className={styles.coursePanel}>
        <div className={styles.classStartDate}>开始上课日期为：{startDate}</div>
        <div className={styles.courseList}>
          {courseList && courseList.length > 0 ? (
            courseList.map((c, i) => {
              return (
                <div
                  key={i}
                  className={styles.courseItem + ' ' + (c.payed ? styles.payed : '')}
                >{`${c.no}（${c.date}）`}</div>
              );
            })
          ) : (
            <div className="empty-tips" style={{ padding: 10 }}>
              暂无课程数据
            </div>
          )}
          {!courseList && <Loading loading={true} />}
        </div>
      </div>
    );
  };

  /**
   * 展开按钮点击时的处理
   * @param  {boolean} expanded 展开状态
   * @param  {object} record   行记录
   */
  expandClick = (expanded, record) => {
    if (expanded) {
      const { key, miniClassId } = record;
      const { list } = this.state;
      const idx = parseInt(key);
      if (!list[idx].courseList) {
        getMiniClassCourseDateInfo({ miniClassId }).then((data) => {
          const courseList = data.map((d, i) => {
            return {
              no: '第' + d.courseNum + '讲',
              date: d.courseDate,
              payed: d.courseStatus === 'CHARGED',
              hours: d.courseHours || 0,
            };
          });
          this.setState({
            list: list.map((d, i) => {
              if (i === idx) {
                return { ...d, courseList };
              } else {
                return d;
              }
            }),
          });
        });
      }
    }
  };

  render() {
    const { columns } = this.props;
    const { list, total, currentPage, pageSize, listLoading, expandedRowKeys } = this.state;
    return (
      <List
        dataSource={list}
        columns={columns}
        loading={listLoading}
        pagination={{
          current: currentPage,
          total,
          pageSize,
        }}
        onChange={this.paginationChange}
        expandedRowRender={this.getCourseInfo}
        onExpand={this.expandClick}
        expandedRowKeys={expandedRowKeys}
        onExpandedRowsChange={(val) => this.setState({ expandedRowKeys: val })}
      />
    );
  }
}

export default Main;
