/**
 * 选择小班模态框
 */
import React from 'react';
import ReactDOM from 'react-dom';
import { Row, Col, Button, Icon } from 'antd';
import Modal from 'COMPONENT/Modal';
import Select from 'COMPONENT/Select';
import Input from 'COMPONENT/Input';
import { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } from 'COMPONENT/List/config';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import showAlert from 'COMPONENT/utils/globalTips/showAlert';
import { getCampusByLoginUserNew } from 'SERVICE/common';
import { findMiniClassByPromiseSubjectId, findMiniClassTeacherForSelect } from 'SERVICE/course';
import { courseStatusDict } from 'CONST/dicts/course';
import DataList from './DataList';
import styles from './styles.less';
import TextLink from 'COMPONENT/TextLink';

const campusGroupDictFormater = (groups) => {
  const dict = groups.map((group) => {
    const [branch, ...campuses] = group;
    return {
      label: branch.name,
      options: campuses.map((campus) => {
        return { val: campus.id, text: campus.name };
      }),
    };
  });
  return dict;
};

class ModalSelectClass extends React.Component {
  constructor(props) {
    super(props);
    this.teacherGroups = []; //按首字母分组的老师姓名，格式为：[{ label: 'ALL', data: [{ id, name }] }]
    this.initState = {
      visible: false, //弹框可见状态
      promiseSubjectId: null, //小班科目ID
      className: '', //查询条件：小班名称
      teacherId: '', //查询条件：老师ID
      campusId: '', //查询条件：班级学习中心ID
      activeLetterBtn: 'ALL', //当前激活的首字母组按钮
      teacherName: '', //老师姓名关键字
      expand: false, //老师关键字搜索栏是否展开
      expandBtnShow: false, //老师栏是否需要显示更多按钮
      loading: false, //模态框加载状态
      campusGroupDict: [], //班级学习中心字典
    };
    this.state = { ...this.initState };
    this.targetVersionId = null;
    this.targetQuarterId = null;
  }

  componentWillMount() {
    const { dispatch } = this.props;
    //加载字典
    this.loadCampusDict();
    //加载初始化数据
    this.setState({ loading: true });
    this.loadTeachers().then(() => {
      this.setState({ loading: false });
    });
  }

  /**
   * 加载学习中心字典
   */
  loadCampusDict = async () => {
    const data = await getCampusByLoginUserNew();
    const dict = campusGroupDictFormater(data);
    this.setState({ campusGroupDict: dict });
    return true;
  };

  /**
   * 加载小班正常老师，并构建成前端所需格式
   * @return {Promise}
   */
  loadTeachers = async () => {
    const data = await findMiniClassTeacherForSelect();
    let all = [];
    let ag = [];
    let hn = [];
    let ot = [];
    let uz = [];
    data.forEach((d) => {
      if (d) {
        const { userId: id, name, account } = d;
        if (id && name) {
          const firstLetter = account.slice(0, 1).toUpperCase();
          const option = { id, name };
          all.push(option);
          if (firstLetter >= 'A' && firstLetter <= 'G') {
            ag.push(option);
          } else if (firstLetter >= 'H' && firstLetter <= 'N') {
            hn.push(option);
          } else if (firstLetter >= 'O' && firstLetter <= 'T') {
            ot.push(option);
          } else if (firstLetter >= 'U' && firstLetter <= 'Z') {
            uz.push(option);
          }
        }
      }
    });
    this.teacherGroups = [
      { label: 'ALL', data: all },
      { label: 'A-G', data: ag },
      { label: 'H-N', data: hn },
      { label: 'O-T', data: ot },
      { label: 'U-Z', data: uz },
    ];
    /*[{ label: 'ALL', data: [
      { id: '1', name: '打生' }, { id: '1', name: '打快点爱迪生' }, { id: '1', name: '打快点生' }, { id: '1', name: '打快点爱迪生' },
      { id: '1', name: '打快点爱迪生' }, { id: '1', name: '打快点爱迪生' }, { id: '1', name: '打快点爱迪生' }, { id: '1', name: '迪生' },
      { id: '1', name: '快迪生' }, { id: '1', name: '打爱迪生' }, { id: '1', name: '打快爱迪生' }, { id: '1', name: '打快点爱迪生' },
      { id: '1', name: '打快点爱迪生' }, { id: '1', name: '打快点爱迪生' }, { id: '1', name: '打点爱迪生' }, { id: '1', name: '打快点爱迪生' },
      { id: '1', name: '打快迪生' }, { id: '1', name: '打快点爱迪生' }, { id: '1', name: '打快点爱迪生' }, { id: '1', name: '打快点爱迪生' },
      { id: '1', name: '打快爱迪生' }, { id: '1', name: '打快点生' }, { id: '1', name: '打点爱迪生' }, { id: '1', name: '打快点迪生' },
      { id: '1', name: '爱迪生' }, { id: '1', name: '打快爱迪生' }, { id: '1', name: '打点爱迪生' }, { id: '1', name: '打快点迪生' },
      { id: '1', name: '快点爱迪生' }, { id: '1', name: '打快点爱迪生' }, { id: '1', name: '打点爱迪生' }, { id: '1', name: '打迪生' },
      { id: '1', name: '打生' }, { id: '1', name: '打快点爱迪生' }, { id: '1', name: '打快点生' }, { id: '1', name: '打快点爱迪生' },
      { id: '1', name: '打快点爱迪生' }, { id: '1', name: '打快点爱迪生' }, { id: '1', name: '打快点爱迪生' }, { id: '1', name: '迪生' },
      { id: '1', name: '快迪生' }, { id: '1', name: '打爱迪生' }, { id: '1', name: '打快爱迪生' }, { id: '1', name: '打快点爱迪生' },
      { id: '1', name: '打快点爱迪生' }, { id: '1', name: '打快点爱迪生' }, { id: '1', name: '打点爱迪生' }, { id: '1', name: '打快点爱迪生' },
      { id: '1', name: '打快迪生' }, { id: '1', name: '打快点爱迪生' }, { id: '1', name: '打快点爱迪生' }, { id: '1', name: '打快点爱迪生' },
      { id: '1', name: '打快爱迪生' }, { id: '1', name: '打快点生' }, { id: '1', name: '打点爱迪生' }, { id: '1', name: '打快点迪生' },
      { id: '1', name: '爱迪生' }, { id: '1', name: '打快爱迪生' }, { id: '1', name: '打点爱迪生' }, { id: '1', name: '打快点迪生' },
      { id: '1', name: '快点爱迪生' }, { id: '1', name: '打快点爱迪生' }, { id: '1', name: '打点爱迪生' }, { id: '1', name: '打迪生' },
    ] }]*/
    this.initExpandBtnShow();
    return true;
  };

  /**
   * 初始化是否显示更多老师按钮，超过三行则显示
   */
  initExpandBtnShow = () => {
    setTimeout(() => {
      if (this._teachersWrap) {
        const $teachersWrap = ReactDOM.findDOMNode(this._teachersWrap);
        if ($teachersWrap) {
          const h = $teachersWrap.offsetHeight;
          if (h > 19 * 3) {
            this.setState({ expandBtnShow: true });
          }
        }
      }
    }, 500);
  };

  /**
   * 关闭弹框
   */
  close = () => {
    const { expandBtnShow } = this.state;
    this.setState({ ...this.initState, expandBtnShow });
  };

  /**
   * 打开弹框
   * @param {string} promiseSubjectId 小班科目ID
   * @param {string} leftHours 剩余课时
   * @param {string} defaultCampus 默认选中的学习中心
   * @param {function} onConfirm 选中后的回调
   * @param {boolean?} isTransfer 是否是转班
   * @param {string} yearId 产品年份 (20190312迭代)
   * @param {boolean?} quarterId 季度 (20190312迭代)
   */
  open = (promiseSubjectId, leftHours, defaultCampus, onConfirm, isTransfer, yearId, quarterId) => {
    this.onConfirm = onConfirm;
    this.leftHours = leftHours;
    this.isTransfer = isTransfer;
    this.targetVersionId = yearId;
    this.targetQuarterId = quarterId;
    this.setState({ promiseSubjectId, campusId: defaultCampus, visible: true });
    this.initContent();
  };

  /**
   * 初始化弹框内容
   */
  initContent = () => {
    const { teacherGroups, loadTeachers } = this;
    const { expandBtnShow } = this.state;
    this.loadCampusDict();
    if (!teacherGroups) {
      this.setState({ loading: true });
      loadTeachers().then(() => {
        this.setState({ loading: false });
      });
    } else {
      if (!expandBtnShow) {
        this.initExpandBtnShow();
      }
    }
    this.search();
  };

  /**
   * 确定选中
   */
  select = (record) => {
    const { onConfirm, close } = this;
    const { subjects } = this.state;
    const { courseList } = record;
    let leftHours = this.leftHours;
    let payedCount = 0;
    if (courseList && leftHours) {
      for (let i = courseList.length - 1; i >= 0; i--) {
        const course = courseList[i];
        if (course.payed) {
          payedCount++;
        }
        leftHours -= course.hours;
        if (leftHours <= 0) {
          break;
        }
      }
    }
    if (payedCount > 0) {
      showConfirm('您报入的小班有部分课程已扣费，学生报入后已扣费课程需另行补课，是否报入？', () => {
        showAlert('您报入的小班有' + payedCount + '节课已扣费，请前往补考勤扣费', () => {
          onConfirm(record);
          close();
        });
      });
    } else {
      onConfirm(record);
      close();
    }
  };

  /**
   * 搜索
   */
  search = () => {
    const { _dataList } = this;
    if (_dataList) {
      _dataList.loadList({ currentPage: 1 });
    }
  };

  /**
   * 切换老师搜索栏的展开状态
   */
  toggleExpand = () => {
    const { expand } = this.state;
    this.setState({ expand: !expand });
  };

  /**
   * 选中老师
   * @param  {string} teacherId 老师ID
   */
  teacherBtnClick = (id) => {
    const { teacherId } = this.state;
    if (teacherId !== id) {
      this.setState({ teacherId: id }, this.search);
    }
  };

  /**
   * 学习中心值变化
   * @param  {string} val 学习中心ID
   */
  campusChange = (val) => {
    this.setState({ campusId: val }, this.search);
  };

  render() {
    const { close, teacherGroups, isTransfer } = this;
    const {
      className,
      teacherId,
      campusId,
      activeLetterBtn,
      teacherName,
      expand,
      loading,
      expandBtnShow,
      visible,
      promiseSubjectId,
      campusGroupDict,
    } = this.state;
    const modalOpts = {
      title: '选择报读小班',
      okBtnShow: false,
      onCancel: close,
      cancelText: '关闭',
      width: 1120,
      loading,
      visible,
      toggle: true,
    };
    const columns = [
      { title: '班级名称', dataIndex: 'name', width: COL_WIDTH_L },
      { title: '班级学习中心', dataIndex: 'blCampusName' },
      { title: '年级', dataIndex: 'gradeName', width: COL_WIDTH_S },
      { title: '科目', dataIndex: 'subjectName', width: COL_WIDTH_S },
      { title: '老师', dataIndex: 'teacherName' },
      { title: '已上小时', dataIndex: 'consumeNatureHour' },
      { title: '下次上课日期', dataIndex: 'startDate' },
      { title: '上课时间', dataIndex: 'classTime' },
      { title: '截止日期', dataIndex: 'endDate' },
      {
        title: '操作',
        key: 'action',
        width: COL_WIDTH_S,
        render: (text, record, index) => {
          return (
            <TextLink className="list-btn" onClick={() => this.select(record)}>
              选择
            </TextLink>
          );
        },
      },
    ];

    const activeTeacherGroup = teacherGroups.find((group) => group.label === activeLetterBtn);

    return (
      <Modal {...modalOpts}>
        <div style={{ padding: 10, margin: '-15px 0 10px 0' }}>
          <Row gutter={30} className={styles.searchRow} style={{ margin: '0 -10px' }}>
            <Col span={3} className={styles.searchTitle}>
              小班名称：
            </Col>
            <Col span={21}>
              <Input
                value={className}
                onPressEnter={this.search}
                onChange={(e) => this.setState({ className: e.target.value })}
                style={{ width: 200, display: 'inline-block' }}
              />
              <Button size="large" type="primary" style={{ marginLeft: 10 }} onClick={this.search}>
                查询
              </Button>
              <Button size="large" style={{ marginLeft: 10 }} onClick={() => this.setState({ className: '' })}>
                重置
              </Button>
            </Col>
          </Row>
          <Row gutter={30} className={styles.searchRow} style={{ margin: '0 -10px' }}>
            <Col span={3} className={styles.searchTitle}>
              老师：
            </Col>
            <Col span={21}>
              <Row gutter={10}>
                <Col span={12}>
                  {teacherGroups.map((group, i) => {
                    const { label } = group;
                    return (
                      <a
                        key={i}
                        className={
                          styles.searchKeyBtn + (label === activeLetterBtn ? ' ' + styles.searchKeyBtnActive : '')
                        }
                        onClick={() => this.setState({ activeLetterBtn: label })}
                      >
                        {label}
                      </a>
                    );
                  })}
                </Col>
                <Col span={8} style={{ textAlign: 'right', paddingRight: 15 }}>
                  <Input
                    size="small"
                    placeholder="请输入老师姓名搜索"
                    style={{ width: 200 }}
                    value={teacherName}
                    onChange={(e) => this.setState({ teacherName: e.target.value })}
                  />
                </Col>
                <Col span={4}>
                  {expandBtnShow && (
                    <a onClick={this.toggleExpand}>
                      <span>{expand ? '收起' : '展开'}</span>
                      <span>{expand ? <Icon type="up" /> : <Icon type="down" />}</span>
                    </a>
                  )}
                </Col>
              </Row>
              <div className={expand ? '' : styles.wrapCollapse} style={{ overflow: 'hidden' }}>
                <div ref={(c) => (this._teachersWrap = c)}>
                  <a
                    className={styles.searchKeyBtn + (teacherId ? '' : ' ' + styles.searchKeyBtnActive)}
                    onClick={() => this.teacherBtnClick('')}
                  >
                    全部
                  </a>
                  {activeTeacherGroup &&
                    activeTeacherGroup.data
                      .filter((teacher) => !teacherName || (teacherName && teacher.name.indexOf(teacherName) >= 0))
                      .map((teacher, i) => {
                        const { id } = teacher;
                        return (
                          <a
                            key={i}
                            className={styles.searchKeyBtn + (id === teacherId ? ' ' + styles.searchKeyBtnActive : '')}
                            onClick={() => this.teacherBtnClick(id)}
                          >
                            {teacher.name}
                          </a>
                        );
                      })}
                </div>
              </div>
            </Col>
          </Row>
          <Row gutter={30} className={styles.searchRow} style={{ margin: '0 -10px' }}>
            <Col span={3} className={styles.searchTitle}>
              班级学习中心：
            </Col>
            <Col span={21}>
              <Select
                allowClear={false}
                showSearch={true}
                dict={campusGroupDict}
                value={campusId}
                onChange={(val) => this.campusChange(val)}
                style={{ width: 200 }}
              />
            </Col>
          </Row>
        </div>
        {promiseSubjectId && (
          <div className={styles.listWrap}>
            <DataList
              columns={columns}
              queryStrings={{
                promiseSubjectId,
                name: className,
                teacherId,
                organizationId: campusId,
                targetVersionId: this.targetVersionId,
                targetQuarterId: this.targetQuarterId,
              }}
              ref={(c) => (this._dataList = c)}
            />
          </div>
        )}
      </Modal>
    );
  }
}

export default ModalSelectClass;
