/**
 * 一个科目卡片展现
 * @property {object} data  数据结构如下：
 * { id, name: '语文', hours: 48, classId: '123', className: '2017暑假初三精英课堂语文1班', campus: '柠溪学习中心',
    startTime: '2017-05-01 10:00', teacher: '高山', totalHours: 48, consume: 3, totalConsume: 15, status: '上课中',
  }
 * @property {boolean?} short 没有班级信息时，是否矮显示
 * @property {boolean?} signUpClass  区别“报班管理”和“科目规划”
 * @property {boolean?} payed  是否缴费
 * @property {function} onShowCourseInfo  查看课程信息的回调
 */
import React from 'react';
import { connect } from 'dva';
import { Form, InputNumber, Icon, Row, Col } from 'antd';
import { isPInt } from 'COMPONENT/utils/basic/num';
import Popover from 'COMPONENT/Popover';
import styles from './styles.less';
import IconButton from '../../../../components/IconButton/index';
import styVars from 'COMPONENT/theme/variables';

const FormItem = Form.Item;

const formItemLayout = {
  labelCol: { span: 6 },
  wrapperCol: { span: 14 },
};

const ClassInfoItem = ({ label, children }) => {
  return (
    <div className="ant-row ant-form-item" style={{ marginBottom: 0, color: '#999' }}>
      <div className="ant-col-6 ant-form-item-label" style={{ lineHeight: '22px' }}>
        <label style={{ color: '#999' }}>{label}</label>
      </div>
      <div className="ant-col-14 ant-form-item-control-wrapper">
        <div className="ant-form-item-control" style={{ lineHeight: '22px' }}>
          <span className="ant-form-text" style={{ paddingLeft: 5 }}>
            {children}
          </span>
        </div>
      </div>
    </div>
  );
};

class Subject extends React.Component {
  render() {
    const {
      payed,
      signUpClass,
      short,
      onHourChange,
      onDeleteSubject,
      onSelectClass,
      onDeleteClass,
      onShowCourseInfo,
      onAddSubject,
      checking,
      stuPromiseClassStatus,
      data: {
        id,
        hours,
        promiseSubjectId,
        name,
        classId,
        className,
        campus,
        startTime,
        teacher,
        totalHours,
        consume,
        totalConsume,
        status,
        statusCode,
        consumeCourseHours,
      },
    } = this.props;
    const leftHours = hours - totalConsume;
    /*    let hoursInput = <div>
                        <FormItem
                          label=""
                          validateStatus={isPInt(hours) ? null : 'error'}
                          help={isPInt(hours) ? null : '课时数只能是大于1的整数'}
                        >
                          <InputNumber min={0} value={hours || 0} onChange={onHourChange}
                            disabled={signUpClass || classId ? true : false}
                          />
                        </FormItem>
                      </div>*/
    const cannotDel = classId && statusCode !== 'PENDDING_START';
    let delBtn = (
      <div
        className={styles.delBtn + (cannotDel ? ' ' + styles.disabledDelBtn : '')}
        onClick={cannotDel ? null : () => onDeleteSubject()}
      >
        <Icon type="close" />
      </div>
    );
    const canAddClass = payed && totalConsume < hours && stuPromiseClassStatus === '1'; /*&& totalConsume === 0*/
    let addClassBtn = (
      <span
        className={styles.addClassBtn + (canAddClass ? '' : ' ' + styles.addClassBtnDisabled)}
        onClick={() => onSelectClass(promiseSubjectId, leftHours)}
      >
        <Icon type="plus" style={{ marginRight: 5, verticalAlign: 'middle' }} />
        <span style={{ fontSize: 14, verticalAlign: 'middle' }}>添加小班</span>
      </span>
    );
    if (signUpClass) {
      /*      hoursInput = <Popover content="请在科目规划中修改可分配课时" icon="info" maxWidth={180}>
                      {hoursInput}
                    </Popover>*/
      if (!canAddClass) {
        let canNotAddClassTips;
        if (!payed) {
          canNotAddClassTips = '当前季度未缴费，暂不能报读小班';
        } else if (totalConsume >= hours) {
          canNotAddClassTips = '当前科目课时已上完，不能添加小班';
        } else if (stuPromiseClassStatus !== '1') {
          canNotAddClassTips = '学生精英班状态不为进行中，不能添加小班';
        } /*else if (totalConsume !== 0) {
          canNotAddClassTips = '已上课时非0，不能添加小班'
        }*/
        addClassBtn = (
          <Popover content={canNotAddClassTips} icon="info" maxWidth={180}>
            {addClassBtn}
          </Popover>
        );
      }
    } else {
      if (classId) {
        /*        hoursInput = <Popover content={
                        statusCode === 'PENDDING_START' ? '删除已当前科目已报的小班才能修改课时数' : '当前科目所报小班已上课，不能修改课时数'
                      } icon="info" maxWidth={180}>
                        {hoursInput}
                      </Popover>*/
        delBtn = (
          <Popover
            content={
              statusCode === 'PENDDING_START' ? '删除科目，将同时删除已添加的小班' : '当前科目所报小班已上课，不能删除'
            }
            icon={statusCode === 'PENDDING_START' ? 'warning' : 'info'}
            maxWidth={180}
          >
            {delBtn}
          </Popover>
        );
      }
    }
    const hoursInput = (
      <Popover content="如需修改课时，请联系分公司精英班负责人修改精英班产品课时" icon="info" maxWidth={180}>
        <div>
          <FormItem label="">
            <InputNumber value={hours || 0} disabled />
          </FormItem>
        </div>
      </Popover>
    );
    let headerContent;
    if (signUpClass) {
      headerContent = name ? name : '未选择科目';
    } else {
      headerContent = (
        <span>
          {name ? name : <span style={checking ? { color: styVars.highlightColor } : {}}>请选择科目</span>}
          {!(totalConsume > 0 || classId) && stuPromiseClassStatus === '1' && (
            <IconButton
              type="edit_mc"
              onClick={onAddSubject}
              className={styles.editIcon}
              disabled={consumeCourseHours > 0}
            />
          )}
        </span>
      );
    }
    return (
      <div
        className={styles.wrap + (signUpClass && !classId && canAddClass ? ' ' + styles.wrapInvaild : '')}
        style={{ height: !classId && short && !signUpClass ? 140 : 355 }}
      >
        <div className={styles.title}>
          {headerContent}
          {/*!signUpClass && delBtn*/}
        </div>
        <div style={{ padding: '5px 15px' }}>
          <div>
            <div style={{ height: 33, lineHeight: '33px' }}>可分配小时（已上小时：{totalConsume}）</div>
            <div>{hoursInput}</div>
          </div>
          {classId && (
            <div style={{ borderTop: '1px dotted #ddd' }}>
              <Row>
                <Col
                  span={18}
                  style={{ fontWeight: 'bold', padding: '15px 0 10px 0' }}
                  className="text-ellipsis"
                  title={className}
                >
                  {className}
                </Col>
                <Col span={6} style={{ textAlign: 'right', lineHeight: '45px' }}>
                  {signUpClass && totalConsume === 0 && (
                    <span>
                      <a className={styles.classActBtn} onClick={() => onSelectClass(promiseSubjectId, leftHours)}>
                        重选
                      </a>
                      <a className={styles.classActBtn} onClick={() => onDeleteClass(id)}>
                        删除
                      </a>
                    </span>
                  )}
                  {signUpClass && totalConsume !== 0 && (
                    <a className={styles.classActBtn} onClick={() => onSelectClass(promiseSubjectId, leftHours, true)}>
                      转班
                    </a>
                  )}
                </Col>
              </Row>
              <div>
                <ClassInfoItem {...formItemLayout} label="班级学习中心">
                  {campus}
                </ClassInfoItem>
                <ClassInfoItem {...formItemLayout} label="开始时间">
                  {startTime}
                </ClassInfoItem>
                <ClassInfoItem {...formItemLayout} label="老师">
                  {teacher}
                </ClassInfoItem>
                <ClassInfoItem {...formItemLayout} label="总小时">
                  {totalHours}
                </ClassInfoItem>
                <ClassInfoItem {...formItemLayout} label="已上小时">
                  {consume}
                </ClassInfoItem>
                <ClassInfoItem {...formItemLayout} label="课程状态">
                  {status}
                </ClassInfoItem>
                <ClassInfoItem {...formItemLayout} label="课程信息">
                  <a onClick={() => onShowCourseInfo(classId)}>查看</a>
                </ClassInfoItem>
              </div>
            </div>
          )}
          {!classId && signUpClass && (
            <div className={styles.addClassBtnWrap}>{name ? addClassBtn : '未选择科目不能报班'}</div>
          )}
        </div>
      </div>
    );
  }
}
export default connect()(Subject);
