import React from 'react';
import { connect } from 'dva';
import { Row, Col } from 'antd';
import Modal from 'COMPONENT/Modal';
import Search from 'COMPONENT/Search';
import Button from 'COMPONENT/Button';
import Input from 'COMPONENT/Input';
import PopTip from 'COMPONENT/PopTip';
import FormItem from 'COMPONENT/FormItem';
import MultiSelect from 'COMPONENT/MultiSelect';
import List from 'COMPONENT/List';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { loadCommonDict, getDictTextByVal } from 'UTIL/dict';
import { getAuxClassList, addStudentSeriesClassInfo, getHandleProgress } from 'SERVICE/course';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import './style.less';
import { throttle, clone } from 'lodash';
import Search2 from 'COMPONENT/Search2';

const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L } = List;
const TWO_TEACHER_VAL = 'TWO_TEACHER';
const SMALL_CLASS_VAL = 'SMALL_CLASS';

const SearchBlock = (props) => {
  const {
    className,
    conditions,
    selectedData,
    handleSearch,
    placeholder,
    dict: { mainTcDict, auxTcDict },
    handleReset,
    handleSelect,
    productType,
    data: { selectedMainTeacher, selectedAuxTeacher, RealClassName, enrollQualification },
  } = props;
  const isTwoTeacher = productType === TWO_TEACHER_VAL;
  const mainTcPlaceholder = isTwoTeacher ? '主讲老师' : '班主任';
  const auxTcPlaceholder = isTwoTeacher ? '辅导老师' : '任课老师';
  return (
    <div className={className}>
      <Row className="search-block-header">
        {/* 暂时隐藏“生成报读资格”-20190730迭代 */}
        {/*<Col span={24}>*/}
        {/*<FormItem className="search-block-form-item" name="生成报读资格" labelWidth={120}>*/}
        {/*<Switch defaultChecked={enrollQualification} onChange={(val) => {handleSelect({ type: 'enrollQualification', val })}}/>*/}
        {/*<span style={{ marginLeft: 16 }}>（开启：根据关联关系给原班学生自动增加相应的报读资格）</span>*/}
        {/*</FormItem>*/}
        {/*</Col>*/}
        <Col span={12}>
          <FormItem className="search-block-form-item" name="班级名称" labelWidth={120}>
            <Input
              placeholder="请输入要查询的班级名称"
              value={RealClassName}
              onChange={(ele) => props.updateClassName(ele.target.value)}
            />
          </FormItem>
        </Col>
      </Row>
      <Search2 style={{ marginBottom: '20px' }} config={conditions} onChange={handleSearch} mode="dark" rowNum={6} />
      <div style={{ width: '100%', height: 66 }}> </div>
      <Row className="search-block-row">
        <Col span={12}>
          <FormItem name={mainTcPlaceholder} labelWidth={110}>
            <MultiSelect
              placeholder={mainTcPlaceholder}
              dict={mainTcDict}
              value={selectedMainTeacher}
              dropdownStyle={{ textAlign: 'left' }}
              onChange={(val) => {
                handleSelect({ type: 'selectedMainTeacher', val });
              }}
            />
          </FormItem>
        </Col>
        <Col span={12}>
          <FormItem name={auxTcPlaceholder} labelWidth={110}>
            <MultiSelect
              placeholder={auxTcPlaceholder}
              dict={auxTcDict}
              value={selectedAuxTeacher}
              dropdownStyle={{ textAlign: 'left' }}
              onChange={(val) => {
                handleSelect({ type: 'selectedAuxTeacher', val });
              }}
            />
          </FormItem>
        </Col>
      </Row>
    </div>
  );
};
const ModalFooterBtn = (props) => {
  const { content, popTipVisible, onClose, onOk, onCancel, okText, onSubmitDisabled } = props;
  return (
    <div style={{ display: 'flex', justifyContent: 'flex-end' }}>
      <Button onClick={onCancel} disabled={onSubmitDisabled}>
        取消
      </Button>
      <PopTip icon="error" content={content} visible={popTipVisible} onClose={onClose} maxWidth={200}>
        <Button type="primary" onClick={onOk} disabled={onSubmitDisabled}>
          {okText || '确定'}
        </Button>
      </PopTip>
    </div>
  );
};

function diffSet(arr) {
  return Array.from(new Set(arr));
}

function arrayToString(arr) {
  return Array.isArray(arr) ? arr.join(',') : null;
}

class AddAuxClassModal extends React.Component {
  constructor(props) {
    super(props);
    this.init = this.init.bind(this);
    this.initSearchKey = this.initSearchKey.bind(this);
    this.loadData = this.loadData.bind(this);
    this.updatePage = this.updatePage.bind(this);
    this.changePage = this.changePage.bind(this);
    this.search = this.search.bind(this);
    this.updateSearch = this.updateSearch.bind(this);
    this.updateSearch2 = this.updateSearch2.bind(this);
    this.updateClassName = this.updateClassName.bind(this);
    this.reset = this.reset.bind(this);
    this.searchAfterUpdateClassName = throttle(this.search, 2000, { leading: false });
    this.state = {
      name: '选择续报班',
      loading: false,
      error: null,
      submitNow: false,
      //字典
      versionDict: [],
      quarterDict: [],
      gradeDict: [],
      subjectDict: [],
      classTypeDict: [],
      classLevelDict: [],
      branchDict: [],
      mainTcDict: [],
      auxTcDict: [],
      weekDayDict: [],
      phaseDict: [],
      campusDict: [],
      campusAllDict: [],
      // 筛选条件
      branchId: null,
      productTypeId: null,
      classId: null,
      selectedVersion: null,
      selectedQuarter: null,
      selectedGrade: null,
      selectedSubject: null,
      selectedClassType: null,
      selectedCampus: null,
      selectedMainTeacher: null,
      selectedAuxTeacher: null,
      // 隐藏的条件
      originalClassTypeId: null,
      originalClassLevel: null,
      originalMainTeacherId: null,
      originalAssistTeacherId: null,
      // 是否生成报读资格
      enrollQualification: false,
      // 班级名称
      RealClassName: null,
      // 续班名
      keyword: null,
      // 原班名
      className: null,
      // 列表
      selectedRowKeys: [],
      listLoading: false,
      auxClassList: [],
      pageSize: 10,
      pageCurrent: 1,
      pageTotal: 0,
      // 列表是否显示选择框
      showRowSelected: false,
    };
    this.columns = [
      { title: '续报班名称', dataIndex: 'auxClassName', key: 'auxClassName', width: COL_WIDTH_L, fixed: 'left' },
      { title: '原班名称', dataIndex: 'originClassName', key: 'originClassName', width: COL_WIDTH_L, fixed: 'left' },
      { title: '年份', dataIndex: 'year', key: 'year', width: COL_WIDTH_M },
      { title: '季度', dataIndex: 'quarter', key: 'quarter', width: COL_WIDTH_S },
      {
        title: '期数/星期数',
        dataIndex: this.props.initData.productTypeId === 'SMALL_CLASS' ? 'phaseName' : 'phase',
        key: this.props.initData.productTypeId === 'SMALL_CLASS' ? 'phaseName' : 'phase',
        width: COL_WIDTH_L,
      },
      { title: '年级', dataIndex: 'grade', key: 'grade', width: COL_WIDTH_S },
      { title: '科目', dataIndex: 'subject', key: 'subject', width: COL_WIDTH_S },
      { title: '班级层次', dataIndex: 'classLevelName', key: 'classLevelName', width: COL_WIDTH_M },
      { title: '所属学习中心', dataIndex: 'campus', key: 'campus', width: COL_WIDTH_M },
      { title: '主讲老师', dataIndex: 'mainTc', key: 'mainTc', width: 90 },
      { title: '辅导老师', dataIndex: 'auxTc', key: 'auxTc', width: 90 },
    ];
    this.hadLink = false;
  }
  componentDidMount() {
    this.init();
  }

  async init() {
    this.setState({ loading: true });
    await new Promise(this.initSearchKey);
    await new Promise(this.loadData);
    this.setState({ loading: false });
  }

  initSearchKey(resolve) {
    const { initData } = this.props;
    this.setState(
      {
        ...this.props.dict,
        classId: initData.classId,
        productTypeId: initData.productTypeId,
        branchId: initData.branchId,
        className: initData.className,
        selectedVersion: clone(initData.version),
        selectedQuarter: clone(initData.quarter),
        selectedGrade: clone(initData.grade),
        selectedSubject: clone(initData.subject),
        selectedClassType:
          (Array.isArray(initData.classType) && arrayToString(initData.classType)) || initData.classType,
        selectedClassLevel: clone(initData.classLevel),
        selectedCampus: clone(initData.campus),
        // 隐藏的条件
        originalClassTypeId: Array.isArray(initData.classType) && arrayToString(initData.classType),
        originalClassLevel: arrayToString(initData.classLevel),
        // 20190730迭代 - 不限制老师
        // selectedMainTeacher: initData.mainTeacher,
        // selectedAuxTeacher: initData.auxTeacher,
        // originalMainTeacherId: arrayToString(initData.mainTeacher),
        // originalAssistTeacherId: arrayToString(initData.auxTeacher),
      },
      () => {
        resolve();
      },
    );
  }

  loadData(resolve) {
    this.setState({ selectedRowKeys: [] });
    const {
      productTypeId,
      classId,
      pageSize,
      pageCurrent,
      branchId,
      className,
      selectedVersion,
      selectedQuarter,
      selectedGrade,
      selectedSubject,
      selectedClassType,
      selectedClassLevel,
      selectedCampus,
      selectedMainTeacher,
      selectedAuxTeacher,
      versionDict,
      quarterDict,
      gradeDict,
      subjectDict,
      classTypeDict,
      campusAllDict,
      originalClassTypeId,
      originalClassLevel,
      originalMainTeacherId,
      originalAssistTeacherId,
    } = this.state;
    const apiParams = {
      branchId,
      originalCourseId: arrayToString(classId),
      blCampusId: arrayToString(selectedCampus),
      className,
      seriseName: this.state.RealClassName,
      classTypeId: selectedClassType,
      classLevel: arrayToString(selectedClassLevel),
      gradeId: arrayToString(selectedGrade),
      mainTeacherId: arrayToString(selectedMainTeacher),
      assistTeacherId: arrayToString(selectedAuxTeacher),
      productTypeId: productTypeId,
      quarterId: arrayToString(selectedQuarter),
      subjectId: arrayToString(selectedSubject),
      yearId: arrayToString(selectedVersion),
      page: pageCurrent,
      rows: pageSize,
      originalClassTypeId,
      originalClassLevel,
      originalMainTeacherId,
      originalAssistTeacherId,
    };
    getAuxClassList(apiParams, true).then(
      (data) => {
        const { datas, rowCount } = data;
        const list = datas.map((item) => {
          const {
            seriesPhase,
            seriesYearId,
            seriesClassTypeId,
            seriesGradeId,
            seriesName,
            seriesQuarterId,
            name,
            seriesSubjectId,
            seriesBlCampusId,
            seriesMainTeacherName,
            seriesAssistTeacherName,
            originalClassId,
            seriesClassId,
            classLevelName,
            phase,
            phaseName,
          } = item;
          return {
            auxClassName: seriesName,
            originClassName: name,
            year: getDictTextByVal(versionDict, seriesYearId),
            quarter: getDictTextByVal(quarterDict, seriesQuarterId),
            phase,
            phaseName,
            grade: getDictTextByVal(gradeDict, seriesGradeId),
            subject: getDictTextByVal(subjectDict, seriesSubjectId),
            classType: getDictTextByVal(classTypeDict, seriesClassTypeId),
            campus: getDictTextByVal(campusAllDict, seriesBlCampusId),
            mainTc: seriesMainTeacherName,
            auxTc: seriesAssistTeacherName || '无',
            classTypeId: seriesClassTypeId,
            classLevelName: classLevelName,
            gradeId: seriesGradeId,
            productTypeId: productTypeId,
            quarterId: seriesQuarterId,
            subjectId: seriesSubjectId,
            yearId: seriesYearId,
            seriesClassId,
            originalClassId,
          };
        });
        this.setState(
          {
            auxClassList: list,
            pageTotal: rowCount,
          },
          () => {
            if (resolve) resolve();
          },
        );
      },
      (err) => {
        if (resolve) resolve();
        showMessage(err.text, 'error');
      },
    );
  }

  updatePage(pageCurrent, pageSize, callback) {
    let payload = {};
    if (pageSize && pageSize !== this.state.pageSize) {
      payload.pageCurrent = 1;
      payload.pageSize = pageSize;
    } else {
      payload.pageCurrent = pageCurrent;
    }
    this.setState(payload, () => {
      if (callback) callback();
    });
  }

  async changePage({ current, pageSize }) {
    await new Promise((resolve) => {
      this.updatePage(current, pageSize, resolve);
    });
    this.setState({ loading: true });
    await new Promise(this.loadData);
    this.setState({ loading: false });
  }

  search() {
    this.changePage({
      current: 1,
      pageSize: this.state.pageSize,
    });
  }

  updateSearch(params) {
    this.setState(
      {
        selectedVersion: params.version,
        selectedQuarter: params.quarter,
        selectedGrade: params.grade,
        selectedSubject: params.subject,
        selectedClassType: params.classType,
        selectedClassLevel: params.classLevel,
        selectedCampus: params.campus,
      },
      () => {
        this.search();
      },
    );
  }

  updateSearch2({ type, val }) {
    this.setState({ [type]: val }, () => {
      // 报读资格选项`enrollQualification`
      if (type !== 'enrollQualification') {
        this.search();
      }
    });
  }

  updateClassName(val) {
    this.setState({ RealClassName: val }, () => {
      this.searchAfterUpdateClassName();
    });
  }

  reset() {
    this.setState({
      RealClassName: null,
      selectedVersion: null,
      selectedQuarter: null,
      selectedGrade: null,
      selectedSubject: null,
      selectedClassType: null,
      selectedBranch: null,
      selectedMainTeacher: null,
      selectedAuxTeacher: null,
      keyword: null,
      selectedRowKeys: [],
    });
  }

  submit = async () => {
    this.setState({ loading: true, submitNow: true });
    const {
      selectedRowKeys,
      auxClassList,
      productTypeId,
      branchId,
      classId,
      selectedClassType,
      selectedQuarter,
      selectedVersion,
      enrollQualification,
      RealClassName,
      selectedCampus,
      selectedGrade,
      selectedSubject,
      selectedMainTeacher,
      selectedAuxTeacher,
      originalClassTypeId,
      originalMainTeacherId,
      originalAssistTeacherId,
      selectedClassLevel,
    } = this.state;
    if (auxClassList.length === 0) {
      this.setState({ loading: false, submitNow: false });
      return showConfirm('无可关联班级，请重新选择筛选条件。');
    }
    let apiParams;
    const hadSelected = selectedRowKeys.length > 0;
    if (hadSelected) {
      let classTypeIdArr = [];
      let quarterIdArr = [];
      let yearIdArr = [];
      selectedRowKeys.forEach((item) => {
        const { classTypeId, quarterId, yearId } = auxClassList[item];
        classTypeIdArr.push(classTypeId);
        quarterIdArr.push(quarterId);
        yearIdArr.push(yearId);
      });
      apiParams = {
        branchId: branchId,
        classTypeId,
        productTypeId,
        quarterId: arrayToString(diffSet(quarterIdArr)),
        yearId: arrayToString(diffSet(yearIdArr)),
        seriesClassInfos: [],
        originalClassTypeId,
        originalMainTeacherId,
        originalAssistTeacherId,
        addNoExamqualification: enrollQualification ? '1' : '0',
      };
      apiParams.seriesClassInfos = selectedRowKeys.map((item) => {
        const { originalClassId, seriesClassId, classTypeId, gradeId, quarterId, subjectId, yearId } =
          auxClassList[item];
        return {
          classTypeId,
          gradeId,
          originalCourseId: originalClassId,
          originalSeriesId: seriesClassId,
          quarterId,
          subjectId,
          yearId,
        };
      });
    } else {
      apiParams = {
        branchId: branchId,
        classTypeId: selectedClassType,
        productTypeId,
        quarterId: arrayToString(selectedQuarter),
        yearId: arrayToString(selectedVersion),
        seriesClassInfos: [],
        // 模糊关联
        className: this.props.initData.className,
        seriseName: RealClassName,
        blCampusId: arrayToString(selectedCampus),
        gradeId: arrayToString(selectedGrade),
        subjectId: arrayToString(selectedSubject),
        mainTeacherId: arrayToString(selectedMainTeacher),
        classLevel: arrayToString(selectedClassLevel),
        assistTeacherId: arrayToString(selectedAuxTeacher),
        originalCourseId: arrayToString(classId),
        originalClassTypeId,
        originalMainTeacherId,
        originalAssistTeacherId,
        addNoExamqualification: enrollQualification ? '1' : '0',
      };
    }
    addStudentSeriesClassInfo(apiParams, true).then(
      (rsp) => {
        // 关联已选
        if (hadSelected) {
          this.setState({ loading: false, submitNow: false });
          this.hadLink = true;
          showMessage('已选班级关联成功！', 'success');
        }
      },
      (err) => {
        showMessage(err.text, 'error');
        this.setState({ loading: false });
      },
    );
    // 关联全部
    if (!hadSelected) {
      this.setState({ loading: false, submitNow: false });
      this.props.handleSubmit();
    }
  };

  closeModal = () => {
    const { handleCancel, handleRefreshList } = this.props;
    if (this.hadLink) {
      handleRefreshList();
    }
    handleCancel();
  };

  render() {
    const {
      initData: { productTypeId },
      handleCancel,
    } = this.props;
    const {
      name,
      loading,
      error,
      btnTipContent,
      toggle_btnTip,
      versionDict,
      quarterDict,
      gradeDict,
      subjectDict,
      classTypeDict,
      campusDict,
      mainTcDict,
      auxTcDict,
      selectedVersion,
      selectedQuarter,
      selectedGrade,
      selectedSubject,
      keyword,
      RealClassName,
      enrollQualification,
      selectedClassType,
      selectedCampus,
      selectedMainTeacher,
      selectedAuxTeacher,
      showRowSelected,
      listLoading,
      auxClassList,
      pageSize,
      pageCurrent,
      pageTotal,
      selectedRowKeys,
      submitNow,
      classLevelDict,
      selectedClassLevel,
    } = this.state;

    const selectedData = {
      version: selectedVersion,
      quarter: selectedQuarter,
      grade: selectedGrade,
      subject: selectedSubject,
      classType: selectedClassType,
      classLevel: selectedClassLevel,
      campus: selectedCampus,
      inputValue: keyword,
    };
    const conditions = [
      { label: '年份', type: 'multi', field: 'version', dict: versionDict, value: selectedVersion },
      { label: '季度', type: 'multi', field: 'quarter', dict: quarterDict, value: selectedQuarter },
      { label: '年级', type: 'multi', field: 'grade', dict: gradeDict, value: selectedGrade },
      { label: '科目', type: 'multi', field: 'subject', dict: subjectDict, value: selectedSubject },
      { label: '班级层次', type: 'multi', field: 'classLevel', dict: classLevelDict, value: selectedClassLevel },
      { label: '学习中心', type: 'multi', field: 'campus', dict: campusDict, value: selectedCampus },
    ];
    const dict = { mainTcDict, auxTcDict };
    const searchData = { selectedMainTeacher, selectedAuxTeacher, RealClassName, enrollQualification };
    const selected = selectedRowKeys.length > 0;

    return (
      <Modal
        title={name}
        className="re-enroll-aux-class-modal"
        loading={loading}
        error={error}
        width={1100}
        onCancel={handleCancel}
        footer={
          <ModalFooterBtn
            content={btnTipContent}
            popTipVisible={toggle_btnTip}
            onClose={() => {
              this.setState({ toggle_btnTip: false });
            }}
            okText={selected ? '关联已选' : '关联全部'}
            onOk={this.submit}
            onCancel={this.closeModal}
            onSubmitDisabled={submitNow}
          />
        }
      >
        <SearchBlock
          className="re-enroll-aux-class-search"
          productType={productTypeId}
          conditions={conditions}
          selectedData={selectedData}
          dict={dict}
          data={searchData}
          handleSearch={this.updateSearch}
          handleReset={this.reset}
          handleSelect={this.updateSearch2}
          updateClassName={this.updateClassName}
          placeholder="请输入班课名称关键字"
        />
        <List
          dataSource={auxClassList}
          columns={this.columns}
          loading={!loading && listLoading}
          pagination={{
            current: pageCurrent,
            total: pageTotal,
            pageSize,
          }}
          onChange={this.changePage}
          rowSelection={
            showRowSelected
              ? {
                  selectedRowKeys,
                  onChange: (keys) => {
                    this.setState({ selectedRowKeys: keys });
                  },
                }
              : null
          }
        />
      </Modal>
    );
  }
}

export default connect()(AddAuxClassModal);
