/**
 * Created by Haolin on 2018/9/19.
 */
import React from 'react';
import { connect } from 'dva';
import Modal from 'COMPONENT/Modal';
import Button from 'COMPONENT/Button';
import PopTip from 'COMPONENT/PopTip';
import List from 'COMPONENT/List';
import Popover from 'COMPONENT/Popover';
import Icon from 'COMPONENT/Icon';
import className from 'classnames';
import style from './style.less';
import { cloneDeep } from 'lodash';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { getConflictStudentSeriesList, resolveConflictStudentSeries } from 'SERVICE/course';

const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;

const ModalFooterBtn = (props) => {
  const { content, popTipVisible, onClose, onOk, onCancel, okText, onOkDisabled } = props;
  return (
    <div style={{ display: 'flex', justifyContent: 'flex-end' }}>
      <Button onClick={onCancel}>取消</Button>
      <PopTip icon="error" content={content} visible={popTipVisible} onClose={onClose} maxWidth={200}>
        <Button type="primary" onClick={onOk} disabled={!onOkDisabled}>
          {okText ? okText : '确定'}
        </Button>
      </PopTip>
    </div>
  );
};
const PopSell = (props) => {
  const { type, record, index, selected, handleClick } = props;
  const label = record[type][0];
  return (
    <div className={style['re-enroll-pop-sell']}>
      <div className={style['re-enroll-pop-sell-label']}>{label}</div>
      <div className={style['re-enroll-pop-sell-action']}>
        {record[type].length > 1 && (
          <Popover
            overlayClassName="re-enroll-pop-sell-pop"
            content={
              <ul>
                {record[type].map((item, itemIndex) => {
                  return PopSellItem({
                    text: item,
                    fatherIndex: index,
                    index: itemIndex,
                    handleClick: handleClick,
                    selected: selected ? selected[itemIndex] : false,
                  });
                })}
              </ul>
            }
          >
            <a>更多</a>
          </Popover>
        )}
      </div>
    </div>
  );
};
const PopSellItem = (props) => {
  const { text, fatherIndex, index, selected, handleClick } = props;
  return (
    <li
      className={className(
        style['re-enroll-pop-sell-item'],
        handleClick ? style['pointer'] : '',
        selected ? style['active'] : '',
      )}
      key={index}
      onClick={() => {
        if (handleClick) {
          handleClick({ fatherIndex, index });
        }
      }}
    >
      {text}
    </li>
  );
};

class AdjustConflictModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      name: '调整冲突关联',
      loading: false,
      error: null,
      btnTipContent: null,
      toggle_btnTip: false,
      //
      listLoading: false,
      list: [],
      pageCurrent: 1,
      pageTotal: 0,
      pageSize: 10,
      selectedRowKeys: [],
      recommendSelected: [],
    };
    this.classId = [];
  }
  componentWillMount() {
    this.init();
  }
  componentDidMount() {}
  componentWillUnmount() {}

  /**
   * 初始化
   * */
  init = () => {
    this.loadData(null);
  };
  /**
   * 加载列表数据
   * */
  loadData = async (params) => {
    await this.updateState({ listLoading: true, ...params });
    this.updateState({ listLoading: true, selectedRowKeys: [] });
    const { pageCurrent, pageSize } = this.state;
    const {
      initData: {
        className,
        branchId,
        blCampusId,
        associatedStatus,
        seriseName,
        productTypeId,
        yearId,
        quarterId,
        phaseId,
        isConflict,
        gradeId,
        subjectId,
        classTypeId,
        classLevel,
        mainTeacherId,
        assistTeacherId,
      },
    } = this.props;
    let page, rows;
    if (params) {
      page = params.pageCurrent;
      rows = params.pageSize;
      this.updateState({
        pageCurrent: page,
        pageSize: rows,
      });
    } else {
      page = pageCurrent;
      rows = pageSize;
    }
    const apiParams = {
      assistTeacherId,
      blCampusId: (blCampusId && blCampusId.join(',')) || null,
      classTypeId: classTypeId || null,
      classLevel: (classLevel && classLevel.join(',')) || null,
      courseWeek: (phaseId && phaseId.join(',')) || null,
      gradeId: (gradeId && gradeId.join(',')) || null,
      mainTeacherId,
      // originalCourseId,
      productTypeId,
      quarterId,
      subjectId: (subjectId && subjectId.join(',')) || null,
      yearId,
      page,
      rows,
    };
    getConflictStudentSeriesList(apiParams, true).then(
      (data) => {
        const { rowCount, datas } = data;
        const list = datas.map((item) => {
          const {
            studentId,
            originalClassId,
            recommendClassTypeId,
            seriesIds,
            recommendSeriesClassId,
            studentName,
            originalClassName,
            recommendClassTypeName,
            seriesClassNames,
            recommendSeriesClassName,
            productTypeId,
            seriesClassIds,
            recommendClassLevelName,
          } = item;
          return {
            stdName: studentName,
            className: originalClassName,
            qualifications: recommendClassLevelName || '无',
            classLinked: this.auxBuildArr(seriesClassNames),
            classRecommend: this.auxBuildArr(recommendSeriesClassName),
            qualificationId: recommendClassTypeId,
            // classLinkedIdArr: this.auxBuildArr(seriesIds),
            classRecommendIdArr: this.auxBuildArr(recommendSeriesClassId),
            studentId,
            originalClassId,
            productTypeId,
            seriesClassIds,
            seriesIds,
          };
        });
        // 默认全选推荐班级
        const recommendSelected = list.map((item) => {
          return item.classRecommendIdArr.map(() => true);
        });
        this.updateState({
          listLoading: false,
          list,
          recommendSelected,
          pageTotal: rowCount,
        });
      },
      (err) => {
        this.updateState({ listLoading: false });
        showMessage(err.text, 'error');
      },
    );
    // mock
    // const _self = this;
    // setTimeout(() => {
    //   const mock = [
    //     {
    //       index: 1,
    //       stdName: '靓仔伟1',
    //       studentId: 'stdxxxx01',
    //       className: '2018暑假一年级英语提高班',
    //       qualifications: '2018秋季二年级英语提高班',
    //       classLinked: [
    //         '2018秋季二年级英语尖子班1',
    //         '2018秋季二年级英语尖子班2',
    //         '2018秋季二年级英语尖子班3',
    //       ],
    //       classRecommend: [
    //         '2018秋季二年级英语提高班1',
    //         '2018秋季二年级英语提高班2',
    //         '2018秋季二年级英语提高班3',
    //       ],
    //       recommendData: [
    //         {
    //           classId: 'xxxxxxxx01',
    //           className: '2018秋季二年级英语提高班1',
    //         },
    //         {
    //           classId: 'xxxxxxxx02',
    //           className: '2018秋季二年级英语提高班2',
    //         },
    //         {
    //           classId: 'xxxxxxxx03',
    //           className: '2018秋季二年级英语提高班3',
    //         },
    //       ],
    //     },
    //     {
    //       index: 2,
    //       stdName: '靓仔伟2',
    //       studentId: 'stdxxxx02',
    //       className: '2018暑假一年级英语提高班',
    //       qualifications: '2018秋季二年级英语提高班',
    //       classLinked: [
    //         '2018秋季二年级英语尖子班1',
    //         '2018秋季二年级英语尖子班2',
    //         '2018秋季二年级英语尖子班3',
    //       ],
    //       classRecommend: [
    //         '2018秋季二年级英语提高班1',
    //         '2018秋季二年级英语提高班2',
    //         '2018秋季二年级英语提高班3',
    //       ],
    //       recommendData: [
    //         {
    //           classId: 'xxxxxxxx01',
    //           className: '2018秋季二年级英语提高班1',
    //         },
    //         {
    //           classId: 'xxxxxxxx02',
    //           className: '2018秋季二年级英语提高班2',
    //         },
    //         {
    //           classId: 'xxxxxxxx03',
    //           className: '2018秋季二年级英语提高班3',
    //         },
    //       ],
    //     },
    //     {
    //       index: 3,
    //       stdName: '靓仔伟3',
    //       studentId: 'stdxxxx03',
    //       className: '2018暑假一年级英语提高班',
    //       qualifications: '2018秋季二年级英语提高班',
    //       classLinked: [
    //         '2018秋季二年级英语尖子班1',
    //         '2018秋季二年级英语尖子班2',
    //         '2018秋季二年级英语尖子班3',
    //       ],
    //       classRecommend: [
    //         '2018秋季二年级英语提高班1',
    //         '2018秋季二年级英语提高班2',
    //         '2018秋季二年级英语提高班3',
    //       ],
    //       recommendData: [
    //         {
    //           classId: 'xxxxxxxx01',
    //           className: '2018秋季二年级英语提高班1',
    //         },
    //         {
    //           classId: 'xxxxxxxx02',
    //           className: '2018秋季二年级英语提高班2',
    //         },
    //         {
    //           classId: 'xxxxxxxx03',
    //           className: '2018秋季二年级英语提高班3',
    //         },
    //       ],
    //     },
    //   ];
    //   const recommendSelected = [
    //     [ true, true, true ],
    //     [ true, true, true ],
    //     [ true, true, true ],
    //   ];
    //   _self.updateState({
    //     listLoading: false,
    //     list: mock,
    //     pageTotal: mock.length,
    //     pageCurrent: 1,
    //     recommendSelected,
    //   });
    // },1000);
  };
  auxBuildArr = (params) => {
    return params ? params.split(',') : [];
  };
  /**
   * 手动选择要关联哪些班级
   * */
  chooseClass = (params) => {
    const { fatherIndex, index } = params;
    this.setState((prevState) => {
      let selected = cloneDeep(prevState.recommendSelected);
      selected[fatherIndex][index] = !selected[fatherIndex][index];
      return {
        recommendSelected: selected,
      };
    });
  };
  /**
   * 提交按钮
   * */
  submit = () => {
    this.updateState({ loading: true });
    function StdToClass(params) {
      const { studentId, originalClassId, productTypeId, seriesClassIds, seriesIds } = params;
      return new Object({
        originalClassId,
        productTypeId,
        recommendSeriesClassId: [],
        seriesClassId: seriesClassIds,
        seriesId: seriesIds,
        studentId,
      });
    }
    const { list, selectedRowKeys, recommendSelected } = this.state;
    let apiParams = [];
    selectedRowKeys.forEach((item, index) => {
      if (recommendSelected[item].length > 0) {
        let itemParams = new StdToClass(list[item]);
        recommendSelected[item].forEach((sItem, sIndex) => {
          if (sItem) {
            itemParams.recommendSeriesClassId.push(list[item].classRecommendIdArr[sIndex]);
          }
        });
        itemParams.recommendSeriesClassId = itemParams.recommendSeriesClassId.join(',');
        apiParams.push(itemParams);
      }
    });
    resolveConflictStudentSeries(apiParams, true).then(
      (data) => {
        showMessage('调整已选冲突关联成功！', 'success');
        this.updateState({ loading: false });
        this.loadData();
      },
      (err) => {
        showMessage(err.text, 'error');
        this.updateState({ loading: false });
      },
    );
  };
  /**
   * 更新state
   * */
  updateState = (newState) => {
    this.setState((prevState, props) => {
      return {
        ...newState,
      };
    });
  };
  render() {
    const { handleCancel } = this.props;
    const {
      name,
      loading,
      error,
      btnTipContent,
      toggle_btnTip,
      listLoading,
      list,
      pageCurrent,
      pageTotal,
      pageSize,
      selectedRowKeys,
      recommendSelected,
    } = this.state;

    const columns = [
      { title: '学生姓名', dataIndex: 'stdName', key: 'stdName', width: COL_WIDTH_M },
      { title: '班级名称', dataIndex: 'className', key: 'className', width: COL_WIDTH_L },
      { title: '报读资格', dataIndex: 'qualifications', key: 'qualifications', width: COL_WIDTH_M },
      {
        title: '已关联班级',
        dataIndex: 'classLinked',
        key: 'classLinked',
        width: COL_WIDTH_XL,
        render: (text, record, index) => {
          return PopSell({ type: 'classLinked', text, record, index });
        },
      },
      {
        title: '推荐关联班级',
        dataIndex: 'classRecommend',
        key: 'classRecommend',
        width: COL_WIDTH_XL,
        render: (text, record, index) => {
          return PopSell({
            type: 'classRecommend',
            text,
            record,
            index,
            selected: recommendSelected[index],
            handleClick: this.chooseClass,
          });
        },
        filterDropdownVisible: false,
        filterIcon: (
          <Popover overlayClassName="list-title-popover" content="可以手动选择关联班级，默认全选">
            <Icon className="list-title-icon" type="question_circle" size={12} />
          </Popover>
        ),
        filterDropdown: <span className="empty-content"> </span>,
      },
    ];
    const canSubmit = selectedRowKeys.length > 0;

    return (
      <Modal
        title={name}
        className="re-enroll-adjust-conflict-modal"
        loading={loading}
        error={error}
        width={1100}
        onCancel={handleCancel}
        footer={
          <ModalFooterBtn
            content={btnTipContent}
            popTipVisible={toggle_btnTip}
            onClose={() => this.updateState({ toggle_btnTip: false })}
            okText="立即替换"
            onOk={this.submit}
            onCancel={handleCancel}
            onOkDisabled={canSubmit}
          />
        }
      >
        <List
          dataSource={list}
          columns={columns}
          loading={!loading && listLoading}
          pagination={{
            current: pageCurrent,
            total: pageTotal,
            pageSize,
          }}
          onChange={({ current, pageSize }) => {
            this.loadData({ pageCurrent: current, pageSize: pageSize });
          }}
          rowSelection={{
            selectedRowKeys,
            onChange: (keys) => {
              this.updateState({ selectedRowKeys: keys });
            },
          }}
          locale={{ filterTitle: '' }}
        />
      </Modal>
    );
  }
}

// const mapStateToProps = (state) => {
//   return {
//     store: {}
//   }
// };

export default connect()(AdjustConflictModal);
