/**
 * Created by Haolin on 2018/9/18.
 */
import React from 'react';
import { connect } from 'dva';
import Modal from 'COMPONENT/Modal';
import { Progress } from 'antd';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { getHandleProgress, getCancelProgress } from 'SERVICE/course';

const ACTIVE = 'active';
const SUCCESS = 'success';
const FAIL = 'exception';

class LinkProgress extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false,
      error: null,
      percent: 0,
      progressStatus: 'active',
      onRunning: false,
    };
    this.intervFunc = null;
  }
  componentWillReceiveProps(nextProps) {
    this.beginRunning(nextProps);
  }
  componentWillUnmount() {
    if (this.state.onRunning) {
      this.clearProgressInterval();
    }
  }

  /**
   * 初始化
   * */
  init = (params) => {
    if (params) {
      this.updateState({
        percent: 0,
        progressStatus: 'active',
        onRunning: false,
      });
    } else {
      const { percent, progressStatus } = this.state;
      if ((percent === 100 && progressStatus === SUCCESS) || progressStatus === FAIL) {
        this.updateState({
          percent: 0,
          progressStatus: ACTIVE,
          onRunning: false,
        });
      }
    }
  };
  /**
   * 开始获取进度
   * */
  beginRunning = (nextProps) => {
    if (nextProps.begin && !this.state.onRunning) {
      this.updateState({
        onRunning: true,
      });
      this.intervFunc = setInterval(this.onProgress, 1000);
      this.switchTabClosable(true);
    }
  };
  /**
   * 监听进度
   * */
  onProgress = () => {
    const { type } = this.props;
    // 根据type请求不同的操作进度
    if (type === 'link') {
      this.onLinkProgress();
    } else {
      this.onUnlinkProgress();
    }
  };
  onLinkProgress = () => {
    getHandleProgress().then(
      (data) => {
        // console.log('批量关联进度: ', data);
        let percent = data.data || 0;
        this.handleProgress(percent);
      },
      (err) => {
        this.handleError();
        showMessage(err.text, 'error');
      },
    );
  };
  onUnlinkProgress = () => {
    getCancelProgress().then(
      (data) => {
        // console.log('取消关联进度: ', data);
        let percent = data.data || 0;
        this.handleProgress(percent);
      },
      (err) => {
        this.handleError();
        showMessage(err.text, 'error');
      },
    );
  };
  handleProgress = (params) => {
    if (params < 100) {
      this.updateState({ percent: params });
    } else {
      if (this.intervFunc) {
        this.clearProgressInterval(this.intervFunc);
        this.updateState({
          percent: 100,
          onRunning: false,
          progressStatus: SUCCESS,
        });
        this.switchTabClosable(false);
        const { visible, handleSuccess } = this.props;
        if (!visible) {
          this.init(true);
        }
        handleSuccess(SUCCESS);
      }
    }
  };

  /**
   * 清除定时器
   * */
  clearProgressInterval = () => {
    clearInterval(this.intervFunc);
    this.intervFunc = null;
  };
  /**
   * 切换显示-标签页关闭按钮
   * */
  switchTabClosable = (params) => {
    const { dispatch } = this.props;
    dispatch({
      type: 'main/updateTab',
      payload: {
        id: '/reEnroll',
        changed: { tabClosable: params },
      },
    });
  };
  /**
   * 关闭弹窗
   * */
  closeModal = () => {
    const { handleCancel } = this.props;
    this.init();
    handleCancel();
  };
  /**
   * 获取关联进度-错误
   * */
  handleError = () => {
    this.clearProgressInterval(this.intervFunc);
    this.updateState({
      onRunning: false,
      progressStatus: FAIL,
    });
    this.switchTabClosable(false);
    const { visible, handleSuccess } = this.props;
    if (!visible) {
      this.init(true);
    }
    handleSuccess(FAIL);
  };

  /**
   * 更新state
   * */
  updateState = (newState) => {
    this.setState((prevState, props) => {
      return {
        ...newState,
      };
    });
  };

  render() {
    const { visible, type } = this.props;
    const { loading, error, percent, progressStatus } = this.state;

    let title = '';
    let btnText = '';
    switch (progressStatus) {
      case SUCCESS:
        title = type === 'link' ? '关联成功' : '取消成功';
        btnText = '关闭';
        break;
      case FAIL:
        title = type === 'link' ? '关联失败' : '取消失败';
        btnText = '关闭';
        break;
      default:
        title = `正在${type === 'unlink' ? '取消' : ''}关联中...`;
        btnText = '后台运行';
    }

    return (
      <Modal
        title={title}
        className="re-enroll-link-progress-modal"
        loading={loading}
        error={error}
        width={380}
        toggle={true}
        visible={visible}
        okBtnShow={false}
        cancelText={btnText}
        onCancel={this.closeModal}
      >
        <Progress percent={percent} strokeWidth={8} status={progressStatus} />
      </Modal>
    );
  }
}

export default connect()(LinkProgress);
