/**
 * Created by Haolin on 2018/9/19.
 */
import React from 'react';
import { connect } from 'dva';
import Modal from 'COMPONENT/Modal';
import List from 'COMPONENT/List';
import { getLinkFailClassList } from 'SERVICE/course';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;

class FailCountModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      name: '失败数量',
      loading: false,
      error: null,
      // 列表
      dataId: null,
      dataPrdType: null,
      listLoading: false,
      list: [],
      pageCurrent: 1,
      pageTotal: 0,
      pageSize: 10,
    };
  }
  componentWillMount() {
    this.init();
  }
  componentDidMount() {}
  componentWillUnmount() {}

  /**
   * 初始化
   * */
  init = async () => {
    const {
      initData: { id, productTypeId },
    } = this.props;
    await this.setState({
      dataId: id,
      dataPrdType: productTypeId,
    });
    this.loadData(null);
  };
  /**
   * 加载列表数据
   * */
  loadData = (params, paramId, paramsType) => {
    this.updateState({ listLoading: true });
    const { dataId, dataPrdType, pageCurrent, pageSize } = this.state;
    let page, rows;
    if (params) {
      page = params.pageCurrent;
      rows = params.pageSize;
      this.updateState({
        pageCurrent: page,
        pageSize: rows,
      });
    } else {
      page = pageCurrent;
      rows = pageSize;
    }
    const apiParams = {
      associatedStatus: 2,
      recordId: dataId,
      productTypeId: dataPrdType,
      page,
      rows,
    };
    getLinkFailClassList(apiParams, true).then(
      (data) => {
        const { datas, rowCount } = data;
        const list = datas.map((item, index) => {
          return {
            ...item,
            index: index + 1 + Number(((page - 1) * rows).toFixed(0)),
          };
        });
        this.updateState({
          listLoading: false,
          list,
          pageTotal: rowCount,
        });
      },
      (err) => {
        showMessage(err.text, 'error');
        this.updateState({ listLoading: false });
      },
    );
  };
  /**
   * 手动关联
   * */
  link = (params) => {
    const { dispatch } = this.props;
    const _self = this;
    console.log(params);
    dispatch({
      type: 'main/openPage',
      payload: {
        path: '/reEnroll/Relevance',
        params: {
          ...params,
          refreshIndex: async () => {
            console.log('手动关联回调', _self.state.name);
            // await _self.updateState({ listLoading: true });
            // await _self.loadData();
          },
        },
        refresh: true,
      },
    });
  };
  /**
   * 更新state
   * */
  updateState = (newState) => {
    this.setState((prevState, props) => {
      return {
        ...newState,
      };
    });
  };
  render() {
    const { handleCancel } = this.props;
    const { name, loading, error, listLoading, list, pageCurrent, pageTotal, pageSize } = this.state;
    const columns = [
      { title: '序号', dataIndex: 'index', key: 'index', width: COL_WIDTH_XS },
      { title: '班级名称', dataIndex: 'className', key: 'className', width: COL_WIDTH_XL },
      {
        title: '操作',
        dataIndex: 'action',
        key: 'action',
        width: COL_WIDTH_S,
        getActionConfig: (text, record, index) => {
          return [
            {
              text: '手动关联',
              onClick: () => {
                this.link({ ...record });
              },
            },
          ];
        },
      },
    ];

    return (
      <Modal
        title={name}
        className="re-enroll-fail-count-modal"
        loading={loading}
        error={error}
        width={600}
        closable={true}
        footer={null}
        onCancel={handleCancel}
      >
        <List
          dataSource={list}
          columns={columns}
          loading={!loading && listLoading}
          pagination={{
            current: pageCurrent,
            total: pageTotal,
            pageSize,
          }}
          onChange={({ current, pageSize }) => this.loadData({ pageCurrent: current, pageSize: pageSize })}
        />
      </Modal>
    );
  }
}

export default connect()(FailCountModal);
