/**
 * Created by Haolin on 2018/9/19.
 */
import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import SearchBar from 'COMPONENT/SearchBar';
import Select from 'COMPONENT/Select';
import List from 'COMPONENT/List';
import Popover from 'COMPONENT/Popover';
import FailCountModal from './FailCountModal';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { loadCommonDict, loadBranchDict, getDictTextByVal, getDictValByText } from 'UTIL/dict';
import { getRenewClassRecordList } from 'SERVICE/course';
import moment from 'moment';

const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;
const TWO_TEACHER_VAL = 'TWO_TEACHER';
const SMALL_CLASS_VAL = 'SMALL_CLASS';
/**
 * 搜索栏
 * */
const Search = (props) => {
  const {
    keywords: { branch, productType, year, quarter },
    dict: { branchDict, productTypeDict, yearDict, quarterDict },
    handleSearch,
    handleReset,
    handleKeyword,
  } = props;
  return (
    <SearchBar className="re-enroll-search" onSearch={handleSearch} onReset={handleReset}>
      <Select
        placeholder="分公司"
        dict={branchDict}
        value={branch}
        onChange={(val) => {
          handleKeyword({ type: 'branch', val });
        }}
      />
      <Select
        placeholder="产品类型"
        dict={productTypeDict}
        value={productType}
        onChange={(val) => {
          handleKeyword({ type: 'productType', val });
        }}
      />
      <Select
        placeholder="产品年份"
        dict={yearDict}
        value={year}
        onChange={(val) => {
          handleKeyword({ type: 'year', val });
        }}
      />
      <Select
        placeholder="产品季度"
        dict={quarterDict}
        value={quarter}
        onChange={(val) => {
          handleKeyword({ type: 'quarter', val });
        }}
      />
    </SearchBar>
  );
};
/**
 * 失败数量单元格渲染
 * */
const FailCountSell = (props) => {
  const { text, record, handleClick } = props;
  const check = Number(text) > 0;
  return (
    <div>
      {check ? (
        <Popover content={'点击查看失败详情'}>
          {/*<a onClick={() => { handleClick(record) }}>{text}</a>*/}
          <span>{text}</span>
        </Popover>
      ) : (
        <div>{text}</div>
      )}
    </div>
  );
};

class LinkRecord extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      name: '批量关联记录',
      loading: false,
      // 搜索栏
      branchDict: null,
      productTypeDict: [
        { text: '双师', val: TWO_TEACHER_VAL },
        { text: '小班', val: SMALL_CLASS_VAL },
      ],
      yearDict: null,
      quarterDict: null,
      branch: null,
      productType: null,
      year: null,
      quarter: null,
      // 列表
      listLoading: false,
      list: [],
      pageCurrent: 1,
      pageTotal: 0,
      pageSize: 10,
      // 失败数量弹窗
      toggleFailCountModal: false,
    };
    this.failId = null;
    this.failPrdType = null;
  }
  componentWillMount() {
    this.init();
  }
  componentDidMount() {}
  componentWillUnmount() {}

  /**
   * 初始化
   * */
  init = async () => {
    await this.updateState({ loading: true });
    const [branchDict, yearDict, quarterDict] = await Promise.all([
      loadBranchDict(),
      loadCommonDict('PRODUCT_VERSION'),
      loadCommonDict('PRODUCT_QUARTER'),
    ]);
    // (暂时不用)初始化分公司/产品类型/年份
    // const { uBranchId } = this.props;
    // let isGroup = uBranchId===uCampusId;
    // let branchVal = isGroup?branchDict[0].val:uBranchId;
    await this.updateState({
      loading: false,
      branchDict,
      yearDict,
      quarterDict,
    });
    this.loadData(null);
  };
  /**
   * 更新搜索栏数据
   * */
  updateKeyword = (params) => {
    const { type, val } = params;
    this.updateState({ [type]: val });
  };
  /**
   * 搜索
   * */
  search = () => {
    const { pageSize } = this.state;
    this.loadData({ pageCurrent: 1, pageSize });
  };
  /**
   * 重置
   * */
  reset = () => {
    this.updateState({
      branch: null,
      classType: null,
      year: null,
      quarter: null,
    });
  };
  /**
   * 加载列表数据
   * */
  loadData = (params) => {
    this.updateState({ listLoading: true });
    const {
      branch,
      productType,
      year,
      quarter,
      pageCurrent,
      pageSize,
      branchDict,
      productTypeDict,
      yearDict,
      quarterDict,
    } = this.state;
    let page, rows;
    if (params) {
      page = params.pageCurrent;
      rows = params.pageSize;
      this.updateState({
        pageCurrent: page,
        pageSize: rows,
      });
    } else {
      page = pageCurrent;
      rows = pageSize;
    }
    const apiParams = {
      branchId: branch,
      yearId: year,
      quarterId: quarter,
      productTypeId: productType,
      page,
      rows,
    };
    getRenewClassRecordList(apiParams, true).then(
      (data) => {
        let list = data.rows.map((item, index) => {
          const { id, branchId, productTypeId, yearId, quarterId, createUserName, createTime, total, fail, success } =
            item;
          return {
            id,
            index: index + 1 + Number(((page - 1) * rows).toFixed(0)),
            branch: branchId ? getDictTextByVal(branchDict, branchId) : null,
            prdType: productTypeId ? getDictTextByVal(productTypeDict, productTypeId) : null,
            prdYear: yearId ? this.analysisArgument(yearDict, yearId) : null,
            prdQuarter: quarterId ? this.analysisArgument(quarterDict, quarterId) : null,
            operator: createUserName,
            operatingTime: createTime ? moment(createTime).format('YYYY-MM-DD HH:mm:ss') : null,
            totalCount: total || 0,
            successCount: success || 0,
            failCount: fail || 0,
            productTypeId,
          };
        });
        this.updateState({
          listLoading: false,
          list,
          pageTotal: data.records,
          pageCurrent: data.page,
        });
      },
      (err) => {
        showMessage(err.text, 'error');
        this.updateState({ listLoading: false });
      },
    );
  };
  analysisArgument = (dict, params) => {
    const paramsList = params.split(',');
    const quarterList = paramsList.map((item) => getDictTextByVal(dict, item));
    return quarterList.join('，');
  };
  /**
   * 失败数量弹窗
   * */
  openFailDetail = (params) => {
    console.log(params);
    this.failId = params.id;
    this.failPrdType = params.productTypeId;
    this.setState({
      toggleFailCountModal: true,
    });
  };
  closeFailDetail = () => {
    this.failId = null;
    this.failPrdType = null;
    this.setState({
      toggleFailCountModal: false,
    });
  };
  /**
   * 更新state
   * */
  updateState = (newState) => {
    this.setState((prevState, props) => {
      return {
        ...newState,
      };
    });
  };
  /**
   * 列表配置
   * */
  columns = [
    { title: '序号', dataIndex: 'index', key: 'index', width: COL_WIDTH_S },
    { title: '分公司', dataIndex: 'branch', key: 'branch', width: COL_WIDTH_L },
    { title: '产品类型', dataIndex: 'prdType', key: 'prdType', width: COL_WIDTH_M },
    { title: '产品年份', dataIndex: 'prdYear', key: 'prdYear', width: COL_WIDTH_M },
    { title: '产品季度', dataIndex: 'prdQuarter', key: 'prdQuarter', width: COL_WIDTH_M },
    { title: '操作人员', dataIndex: 'operator', key: 'operator', width: COL_WIDTH_M },
    { title: '操作时间', dataIndex: 'operatingTime', key: 'operatingTime', width: COL_WIDTH_L },
    { title: '总班数', dataIndex: 'totalCount', key: 'totalCount', width: COL_WIDTH_M },
    { title: '成功数量', dataIndex: 'successCount', key: 'successCount', width: COL_WIDTH_M },
    {
      title: '失败数量',
      dataIndex: 'failCount',
      key: 'failCount',
      width: COL_WIDTH_M,
      render: (text, record) => {
        return FailCountSell({ text, record, handleClick: this.openFailDetail });
      },
    },
  ];
  render() {
    const { route } = this.props;
    const {
      name,
      loading,
      branchDict,
      productTypeDict,
      yearDict,
      quarterDict,
      branch,
      productType,
      year,
      quarter,
      listLoading,
      list,
      pageCurrent,
      pageTotal,
      pageSize,
      toggleFailCountModal,
    } = this.state;

    const dict = { branchDict, productTypeDict, yearDict, quarterDict };
    const keywords = { branch, productType, year, quarter };

    return (
      <Page
        name={name}
        fitWindowHeight={true}
        loading={loading}
        route={route}
        subHeader={
          <Search
            keywords={keywords}
            dict={dict}
            handleKeyword={this.updateKeyword}
            handleSearch={this.search}
            handleReset={this.reset}
          />
        }
      >
        <List
          dataSource={list}
          columns={this.columns}
          loading={!loading && listLoading}
          pagination={{
            current: pageCurrent,
            total: pageTotal,
            pageSize,
          }}
          onChange={({ current, pageSize }) => this.loadData({ pageCurrent: current, pageSize: pageSize })}
        />
        {toggleFailCountModal && (
          <FailCountModal
            initData={{
              id: this.failId,
              productTypeId: this.failPrdType,
            }}
            handleCancel={this.closeFailDetail}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const { brenchId } = state.account.user;
  return {
    uBranchId: brenchId || null,
  };
};

export default connect()(LinkRecord);
