import React, { Component } from 'react';
import ReactDom from 'react-dom';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import Block from 'COMPONENT/Block';
import List from 'COMPONENT/List';
import TextLink from 'COMPONENT/TextLink';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import moment from 'moment';
import Input from 'COMPONENT/Input';
import AddClassModal from '../AddClassModal';
import { loadCommonDict } from 'UTIL/dict';
import Loader from '../loader';
import styles from './styles.less';
import ClassTag from './component/ClassTag';
import { formatHasStudent } from './util';

const { COL_WIDTH_S, COL_WIDTH_M } = List;

const colorArr = [
  ['#F8B62A14', '#F8B62A'],
  ['#6D67FF14', '#6D67FF'],
  ['#4A90E214', '#4A90E2'],
  ['#FF606E14', '#FF606E'],
  ['#F8B62A26', '#F8B62A'],
  ['#6D67FF26', '#6D67FF'],
  ['#4A90E226', '#4A90E2'],
  ['#FF606E26', '#FF606E'],
  ['#F8B62A33', '#F8B62A'],
  ['#6D67FF33', '#6D67FF'],
  ['#4A90E233', '#4A90E2'],
  ['#FF606E33', '#FF606E'],
];
let classTypeDict = [];

class Relevance extends Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: true,
      studentInfo: '',
      listLoading: false,
      showAddClassModal: false,
      seriesClassList: [], // 保存续报班列表
      students: [], // 学生列表
      editStuId: null, // 新增续报的学生ID
      selectedNextClass: null, // 选中的续报班级
    };

    this.idMapColor = {};
    this.colorCur = 0;

    this.beforeBack = this.beforeBack.bind(this);
    this.updateState = this.updateState.bind(this);

    this.initContent = this.initContent.bind(this);
    this.loadListData = this.loadListData.bind(this);
    this.getOriginClassName = this.getOriginClassName.bind(this);
    this.renderAllClasses = this.renderAllClasses.bind(this);
    this.addColor = this.addColor.bind(this);
    this.handleDelClass = this.handleDelClass.bind(this);
    this.filterStudentList = this.filterStudentList.bind(this);

    this.onShowAddClassModal = this.onShowAddClassModal.bind(this);
    this.onHideAddClassModal = this.onHideAddClassModal.bind(this);
    this.onSelectNextClass = this.onSelectNextClass.bind(this);
    this.handleHasStudentSelected = this.handleHasStudentSelected.bind(this);
    this.handleNoStudentSelected = this.handleNoStudentSelected.bind(this);
    this.onFinalCommit = this.onFinalCommit.bind(this);
    this.formatApiParam = this.formatApiParam.bind(this);
    this.formatHasStudentParam = this.formatHasStudentParam.bind(this);
    this.formNoStudentParam = this.formNoStudentParam.bind(this);
    this.isUnsave = this.isUnsave.bind(this);
  }

  componentWillMount() {
    this.initContent();
  }

  /**
   * 给返回按钮绑定事件，返回前刷新首页列表的数据
   */
  componentDidMount() {
    this.$page = ReactDom.findDOMNode(this._page);
    const $backBtns = this.$page.querySelectorAll('a[href="/reEnroll"]');
    $backBtns.forEach(($btn) => {
      $btn.onclick = this.beforeBack;
    });
  }

  beforeBack() {
    if (this.props.params.refreshIndex) {
      this.props.params.refreshIndex();
    }
  }

  /**
   * 更新状态异步
   * @param  {object} changedState 改变的状态
   * @return {Promise}
   */
  updateState(changedState) {
    const newState = {
      ...this.state,
      ...changedState,
    };

    return new Promise((resolve) => {
      this.setState(newState, () => {
        resolve();
      });
    });
  }

  /**
   * 初始化数据，将选中的续报班级置空
   */
  async initContent() {
    await this.updateState({ loading: true });

    const { classId, productTypeId } = this.props.params;
    let students = await this.loadListData({ originalClassId: classId, productTypeId });
    loadCommonDict('CLASS_TYPE').then((dict) => (classTypeDict = dict));

    this.originData = JSON.stringify(students);
    this.setState({ students, selectedNextClass: null, loading: false });
  }

  /**
   * 获取原班信息
   * @param params
   */
  async loadListData(params) {
    const apiParams = { ...params };

    const students = await Loader.loadStudents(apiParams);
    let seriesClassInfo = [];

    // 重新生成课程颜色
    students.forEach((student) => {
      student.seriesClassInfo &&
        student.seriesClassInfo.forEach((classItem) => {
          if (!(classItem.classId in this.idMapColor)) {
            this.addColor(classItem.classId);
            seriesClassInfo.push(classItem);
          }
        });
    });

    await this.setState({ seriesClassList: seriesClassInfo });

    return students;
  }

  /**
   * 获取原班课程名称
   */
  getOriginClassName() {
    const {
      className,
      blCampusName,
      mainTeacherName,
      assistTeacherName,
      classStartDate,
      classEndDate,
      classStartTime,
      classLength,
    } = this.props.params;

    return (
      className +
      ' / ' +
      blCampusName +
      ' / ' +
      (mainTeacherName || '') +
      ' ' +
      (assistTeacherName || '') +
      ' / ' +
      classStartDate +
      (classEndDate ? ' ~ ' + classEndDate : '') +
      ' / ' +
      classStartTime +
      ' ~ ' +
      moment(classStartTime, 'HH:mm').add(classLength, 'm').format('HH:mm')
    );
  }

  /**
   * 根据学生的续班班获取改原班的所有续报班并按照添加时间排序
   */
  renderAllClasses() {
    const { students } = this.state;
    const classesMap = {};
    let seriesClassInfo = [];

    if (students && students.length > 0) {
      students.forEach((student) => {
        student.seriesClassInfo &&
          student.seriesClassInfo.forEach((classItem) => {
            const id = classItem.classId;

            if (!(id in classesMap)) {
              classesMap[id] = true;
              seriesClassInfo.push(classItem);
            }
          });
      });
    }

    seriesClassInfo.sort((classA, classB) => {
      if (classA.sortNum && classB.sortNum) {
        return Number(classA.sortNum) > Number(classB.sortNum);
      } else {
        return true;
      }
    });

    const ret = seriesClassInfo.map((classItem, index) => {
      return (
        <ClassTag key={index} idMapColor={this.idMapColor} classItem={classItem} handleDelClass={this.handleDelClass} />
      );
    });

    if (ret.length > 0) {
      return ret;
    } else {
      return <p style={{ color: '#ccc' }}>暂无续报班课</p>;
    }
  }

  /**
   * 添加颜色
   */
  addColor(id) {
    const color = colorArr[this.colorCur % 12];
    this.colorCur++;
    this.idMapColor[id] = color;
  }

  /**
   * 删除课程
   */
  async handleDelClass(id, stuId, seriesId) {
    const { classId: originalClassId } = this.props.params;

    await this.updateState({ loading: true });

    // 如果flag为true，则有seriesId，则是已保存
    if (seriesId) {
      if (this.isUnsave()) {
        showMessage('请先提交后再删除原有的续报班');
        this.setState({ loading: false });
        return false;
      }

      try {
        const methodName = stuId !== undefined ? 'del' : 'delAll';
        const json = { originalClassId, seriesClassId: id };

        stuId !== undefined && (json.seriesId = seriesId);
        const data = await Loader[methodName](json);

        if (data.resultCode === 0) {
          this.initContent();
        } else {
          this.setState({ loading: false });
        }
      } catch (e) {
        this.setState({ loading: false });
      }
    } else {
      const { students } = this.state;
      const newStudents = students.map((student) => {
        if (stuId !== undefined && student.studentId !== stuId) {
          return student;
        }

        // 如果没有stuId，则是在班课信息删除课程，所有学生都要删除。
        const newClasses =
          (student.seriesClassInfo &&
            student.seriesClassInfo.filter((classItem) => {
              return classItem.classId !== id;
            })) ||
          [];

        return {
          ...student,
          seriesClassInfo: newClasses,
        };
      });

      this.setState({
        loading: false,
        students: newStudents,
      });
    }
  }

  /**
   * 学生列表
   */
  filterStudentList() {
    let { students, studentInfo } = this.state;

    studentInfo = studentInfo.trim();

    return students.filter((student) => {
      const hasName = student.studentName && student.studentName.indexOf(studentInfo) > -1;
      const hasId = student.studentId && student.studentId.indexOf(studentInfo) > -1;
      return hasName || hasId;
    });
  }

  /**
   * 打开添加续报班模态框
   * @Attention 产品需求改为原班没有学生也可以新增续报
   */
  onShowAddClassModal(studentId = null) {
    this.setState({
      editStuId: studentId,
      selectedNextClass: null,
      showAddClassModal: true,
    });
  }

  /**
   * 关闭添加续报班模态框
   */
  onHideAddClassModal() {
    this.setState({
      selectedNextClass: null,
      showAddClassModal: false,
    });
  }

  /**
   * 选择续报班回调
   * 需要判断有无学生列表
   */
  async onSelectNextClass(records) {
    const { students } = this.state;
    const hasStudent = formatHasStudent(students);

    await this.updateState({ showAddClassModal: false, loading: true, selectedNextClass: records });

    if (hasStudent) {
      this.handleHasStudentSelected(records);
    } else {
      this.handleNoStudentSelected(records);
    }
  }

  /**
   * 学生列表不为空时
   * @param records 选中的续班
   */
  async handleHasStudentSelected(records) {
    const { editStuId, students } = this.state;

    let stdList = students.filter((item) => {
      return item.studentId && item.studentName;
    });

    const studentIdArr = editStuId ? [editStuId] : stdList.map((student) => student.studentId);

    // 计算出新的学生列表
    let newStus = null;

    // 批量学生课程去重，添加课程
    records.forEach((record) => {
      const originStus = newStus || stdList;
      // 处理学生数据
      newStus = originStus.map((student) => {
        // 如果是单个学生新增，其它学生直接返回，不添加
        if (editStuId && student.studentId !== editStuId) {
          return student;
        }

        const index =
          student.seriesClassInfo &&
          student.seriesClassInfo.findIndex((item) => {
            return item.classId === record.classId;
          });

        // 如果该学生已存在相同id的课程，则不添加
        if (index !== null && index > -1) {
          showMessage(`学生 ${student.studentName} 已添加相同的课程`);
          return student;
        }

        const newClass = {
          ...record,
          sortNum: Date.now(),
        };

        return {
          ...student,
          seriesClassInfo: student.seriesClassInfo ? student.seriesClassInfo.concat([newClass]) : [newClass],
        };
      });
    });

    // 批量提取课程数据用于检测冲突
    const requestJson = records.map((record) => {
      let {
        classTypeId,
        gradeId,
        productTypeId,
        quarterId,
        subjectId,
        yearId,
        classId: seriseClassId,
        classLevel,
      } = record;
      classLevel = classLevel === null ? '' : classLevel;

      return {
        classTypeId,
        gradeId,
        productTypeId,
        quarterId,
        subjectId,
        yearId,
        studentIdArr,
        seriseClassId,
        classLevel,
      };
    });

    // 批量检测是否冲突
    // const conflictes = await Loader.loadConflict(requestJson);
    // conflictes.forEach( conflict => {
    //   newStus.forEach( stu => {
    //     if (stu.studentId !== conflict.studentId) {
    //       return;
    //     }
    //
    //     stu.seriesClassInfo && stu.seriesClassInfo.forEach( classItem => {
    //       if (conflict.seriseClassId && classItem.classId !== conflict.seriseClassId ) {
    //         return;
    //       }
    //
    //       classItem.isConflict = 1;
    //       classItem.confilctMessage = conflict.conflictmsg;
    //     });
    //   });
    // });

    // 批量添加颜色
    records.forEach((record) => {
      if (!(record.classId in this.idMapColor)) {
        this.addColor(record.classId);
      }
    });

    // 关闭弹框
    this.setState({ students: newStus, loading: false });
  }

  /**
   * 学生列表为空时, 需要去掉重复的续报班
   */
  handleNoStudentSelected(records) {
    const { students } = this.state;
    let studentList = students && [...students];

    studentList.forEach((student) => {
      const seriesClassInfo = student.seriesClassInfo || [];
      let tempClassInfo = [...seriesClassInfo, ...records];

      // 去重
      let hash = {};
      tempClassInfo = tempClassInfo.reduce((item, next) => {
        if (!hash[next.classId]) {
          hash[next.classId] = true;
          item.push(next);
        }

        return item;
      }, []);

      student.seriesClassInfo = tempClassInfo;
    });

    // 批量添加颜色
    records.forEach((record) => {
      if (!(record.classId in this.idMapColor)) {
        this.addColor(record.classId);
      }
    });

    this.setState({ students: studentList, loading: false });
  }

  /**
   * 最终保存提交
   * 无学生关联调用批量接口，有学生关联调用单个关联接口
   */
  async onFinalCommit() {
    const { students } = this.state;
    const hasStudent = formatHasStudent(students);

    let saveApi = Loader.saveStudentSeriesClassInfo;
    if (hasStudent) {
      saveApi = Loader.save;
    }

    const apiParam = this.formatApiParam(hasStudent);

    await this.updateState({ loading: true });

    saveApi(apiParam).then(
      (result) => {
        showMessage('保存成功', 'success');
        this.initContent();
      },
      (e) => {
        showMessage(e.resultMsg || '保存失败，请重试');
        this.setState({ loading: false });
      },
    );
  }

  /**
   * 格式化请求参数（批量关联）
   * 两个入口, 没选学生-只关联班级
   * @param hasStudent 是否有学生
   */
  formatApiParam(hasStudent) {
    let param = {};

    if (hasStudent) {
      param = this.formatHasStudentParam();
    } else {
      param = this.formNoStudentParam();
    }

    return param;
  }

  /**
   * 格式化没有学生的参数
   * seriesClassInfos: [{originalCourseId: '原班id', originalSeriesId: '续班id'}]
   */
  formNoStudentParam() {
    const { classId, productTypeId } = this.props.params || {};
    const { students } = this.state;
    const student = (students && students[0]) || {};
    const studentClassInfo = student.seriesClassInfo || [];
    let classInfo = [];

    studentClassInfo.forEach((item) => {
      classInfo.push({
        originalCourseId: classId,
        originalSeriesId: item.classId,
      });
    });

    return {
      addNoExamqualification: '0', // 是否生成免考资格
      originalCourseId: classId, // 原班id
      productTypeId: productTypeId, // 产品类型
      seriesClassInfos: classInfo,
    };
  }

  /**
   * 格式化有学生列表的接口参数
   */
  formatHasStudentParam() {
    const { students } = this.state;

    let stdList = students.filter((item) => item.studentId && item.studentName);
    let associatedStatus;
    const associatedNum = stdList.reduce((acc, cur) => {
      if (cur.seriesClassInfo && cur.seriesClassInfo.length > 0) {
        return ++acc;
      } else {
        return acc;
      }
    }, 0);

    if (associatedNum === stdList.length) {
      associatedStatus = 3;
    } else if (associatedNum > 0) {
      associatedStatus = 2;
    } else {
      associatedStatus = 1;
    }

    stdList.forEach((student) => {
      student.seriesClassInfo &&
        student.seriesClassInfo.forEach((classItem) => {
          classItem.associatedStatus = associatedStatus;
        });
    });

    return stdList;
  }

  /**
   * 是否未保存
   * @return {boolean}
   */
  isUnsave() {
    const { students } = this.state;
    return this.originData !== JSON.stringify(students);
  }

  /**
   * 表头
   */
  columns = [
    {
      title: '学生',
      dataIndex: 'studentName',
      key: 'studentName',
      width: COL_WIDTH_M,
      render: (studentName, record) => {
        return studentName + '(' + record.studentId + ')';
      },
    },
    {
      title: '续报班课名称',
      dataIndex: 'seriesClassInfo',
      key: 'seriesClassInfo',
      width: 1000,
      render: (seriesClassInfo, record) => {
        if (seriesClassInfo && seriesClassInfo.length > 0) {
          return seriesClassInfo.map((classItem, index) => {
            const { confilctMessage } = classItem;

            return (
              <ClassTag
                key={index}
                idMapColor={this.idMapColor}
                classItem={classItem}
                handleDelClass={this.handleDelClass}
                record={record}
                confilctMessage={confilctMessage}
              />
            );
          });
        }
      },
    },
    {
      title: '操作',
      key: 'action',
      width: COL_WIDTH_S,
      fixed: 'right',
      getActionConfig: (text, record, index) => {
        const { studentId } = record;
        return [{ text: '新增续报', onClick: () => this.onShowAddClassModal(studentId) }];
      },
    },
  ];

  render() {
    const { loading, listLoading, studentInfo, showAddClassModal } = this.state;
    const { params, route } = this.props;
    const {
      className,
      blCampusName,
      mainTeacherName,
      assistTeacherName,
      classStartDate,
      classEndDate,
      classStartTime,
      classLength,
      branchId,
      productTypeId,
      mainTeacherId,
      assistTeacherId,
      blCampusId,
      phaseId,
      subjectId,
      classTypeId,
      classLevel,
      yearName,
      quarterName,
      yearId,
      quarterId,
      gradeId,
      classId: originalClassId,
    } = params;

    return (
      <Page
        loading={loading}
        route={route}
        onOk={this.onFinalCommit}
        footer={this.isUnsave()}
        ref={(c) => (this._page = c)}
      >
        <Block title="班课信息">
          <div>
            <p>原班课名称:</p>
            <p style={{ padding: '1em 0' }}>{this.getOriginClassName()}</p>
          </div>
          <hr />
          <div>
            <p style={{ padding: '1em 0' }}>
              <span>续报班课名称:</span>
              <TextLink className={styles.batchAdd} onClick={() => this.onShowAddClassModal(null)}>
                新增续报
              </TextLink>
            </p>
            <div>{this.renderAllClasses()}</div>
          </div>
        </Block>

        <Block title="学生信息">
          <div style={{ width: '200px', paddingBottom: '1em' }}>
            <Input
              value={studentInfo}
              onChange={(e) => {
                this.setState({ studentInfo: e.target.value });
              }}
              placeholder="学生姓名/学生ID"
            />
          </div>
          <List columns={this.columns} loading={listLoading} dataSource={this.filterStudentList()} />
        </Block>

        {showAddClassModal && (
          <AddClassModal
            onCancel={this.onHideAddClassModal}
            handleChose={this.onSelectNextClass}
            className={className}
            quarterName={quarterName}
            blCampusName={blCampusName}
            mainTeacherName={mainTeacherName}
            assistTeacherName={assistTeacherName}
            yearName={yearName}
            classStartDate={classStartDate}
            classEndDate={classEndDate}
            branchId={branchId}
            mainTeacherId={mainTeacherId}
            assistTeacherId={assistTeacherId}
            blCampusId={blCampusId}
            phaseId={phaseId}
            subjectId={subjectId}
            classTypeId={classTypeId}
            classLevel={classLevel}
            productTypeId={productTypeId}
            yearId={yearId}
            quarterId={quarterId}
            gradeId={gradeId}
            classStartTime={classStartTime}
            classLength={classLength}
            originalClassId={originalClassId}
            propsDict={this.props.propsDict}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const dict = state.dict || {};
  const data = state.main.pageParams;
  const pathName = state.routing.locationBeforeTransitions.pathname;
  let params = {};

  if (data[pathName]) {
    params = data[pathName];
  }

  return {
    params,
    propsDict: {
      twoTcMainTcDict: dict.twoTcMainTc,
      twoTcAuxTcDict: dict.twoTcAuxTc,
      smallClassMainTcDict: dict.smallClassMainTc,
      smallClassAuxTcDict: dict.smallClassAuxTc,
    },
  };
};

export default connect(mapStateToProps, null)(Relevance);
