import React from 'react';
import { Row, Col } from 'antd';
import FormItem from 'COMPONENT/FormItem';
import Select from 'COMPONENT/Select';
import Input from 'COMPONENT/Input';
import Button from 'COMPONENT/Button';
import Search2 from 'COMPONENT/Search2';
import style from './index.less';
import { transformArrayToString } from 'UTIL/common';

class SearchFeat extends React.Component {
  static defaultProps = {
    condition: {}, // 字典枚举集
    selectedData: {}, // 初始化选中的数据
    isSmallClass: false, // 是否为双师
    handleSearch: () => {}, // 搜索
    onRef: () => {}, // 暴露ref
  };

  constructor(props) {
    super(props);

    this.state = {
      className: '', // 课程名称
      yearId: null, // 年份
      quarterId: null, // 季度 []
      gradeId: null, // 年级
      subjectId: null, // 科目
      classLevel: null, // 班级层次
      blCampusId: null, // 学习中心 []
      mainTeacherId: null, // 主讲老师
      assistTeacherId: null, // 辅讲老师
    };

    this.initParam = {
      className: '',
    };

    this.initPage = this.initPage.bind(this);
    this.initState = this.initState.bind(this);
    this.formatSearch2Config = this.formatSearch2Config.bind(this);
    this.onSearchChange = this.onSearchChange.bind(this);
    this.onSelectChange = this.onSelectChange.bind(this);
    this.onSearch = this.onSearch.bind(this);
    this.onReset = this.onReset.bind(this);
    this.formatParam = this.formatParam.bind(this);
    this.renderTeacherSelect = this.renderTeacherSelect.bind(this);
  }

  componentDidMount() {
    this.props.onRef(this);
    this.initPage();
  }

  /**
   * 初始化弹窗
   */
  initPage() {
    const { selectedData } = this.props;
    const state = { ...selectedData };
    this.setState(state, () => {
      this.onSearch();
    });
  }

  /**
   * 初始化state选中状态
   * @param props
   */
  initState(props) {
    const { selectedData } = props || this.props;
    const state = { ...selectedData };
    this.setState(state);
  }

  /**
   * condition搜索条件改变
   */
  onSearchChange(params) {
    this.setState({ ...params }, () => {
      this.onSearch();
    });
  }

  /**
   * 下拉选项修改
   * @param key
   * @param value
   */
  onSelectChange(key, value) {
    let obj = {};
    obj[key] = value;
    this.setState(obj, () => {
      this.onSearch();
    });
  }

  /**
   * 搜索动作
   */
  onSearch() {
    const param = this.formatParam();
    this.props.handleSearch(param);
  }

  /**
   * 重置条件
   */
  onReset() {
    Promise.all([this.initState(), this.setState(this.initParam)]).then(([]) => {
      this.onSearch();
    });
  }

  /**
   * 渲染老师下拉选项
   * 小班只有授课老师下拉框
   */
  renderTeacherSelect() {
    const { condition, isSmallClass } = this.props;
    const { mainTeacherDict, subTeacherDict } = condition || {};
    const { mainTeacherId, assistTeacherId } = this.state;

    const labelWidth = 80;
    let mainTitle = '主讲老师';
    let subTitle = '辅导老师';

    if (isSmallClass) {
      mainTitle = '班主任';
      subTitle = '任课老师';
    }

    return (
      <Row>
        <Col span={8}>
          <FormItem name={mainTitle} size="small" labelWidth={labelWidth}>
            <Select
              value={mainTeacherId}
              dict={mainTeacherDict}
              onChange={(val) => {
                this.onSelectChange('mainTeacherId', val);
              }}
              showSearch={true}
              allowClear={true}
              searchPlaceholder="请输入老师名字搜索"
            />
          </FormItem>
        </Col>
        <Col span={8}>
          <FormItem name={subTitle} size="small" labelWidth={labelWidth}>
            <Select
              value={assistTeacherId}
              dict={subTeacherDict}
              onChange={(val) => {
                this.onSelectChange('assistTeacherId', val);
              }}
              showSearch={true}
              allowClear={true}
              searchPlaceholder="请输入老师名字搜索"
            />
          </FormItem>
        </Col>
      </Row>
    );
  }

  /**
   * 格式化search2的配置项
   */
  formatSearch2Config(props) {
    const { condition } = props || this.props;
    const { yearDict, quarterDict, gradeDict, subjectDict, classLevelDict, campusDict } = condition || {};
    const { yearId, quarterId, gradeId, subjectId, classLevel, blCampusId } = this.state;

    return [
      { label: '年份', dict: yearDict, field: 'yearId', value: yearId },
      { label: '季度', dict: quarterDict, field: 'quarterId', value: quarterId, type: 'multi' },
      { label: '年级', dict: gradeDict, field: 'gradeId', value: gradeId },
      { label: '科目', dict: subjectDict, field: 'subjectId', value: subjectId },
      { label: '班级层次', dict: classLevelDict, field: 'classLevel', value: classLevel },
      { label: '学习中心', dict: campusDict, field: 'blCampusId', value: blCampusId, type: 'multi' },
    ];
  }

  /**
   * 格式化API请求参数
   * 全部学习中心传所有学习中心的ID
   */
  formatParam() {
    const { condition } = this.props;
    const { campusDict } = condition || {};

    let param = { ...this.state };
    const { blCampusId } = param;
    let campusIdArr = [];

    if ((!blCampusId || blCampusId.length <= 0) && campusDict.length > 0) {
      campusIdArr = campusDict.map((dict) => {
        return dict.val;
      });
    } else {
      campusIdArr = blCampusId;
    }

    param.quarterId = transformArrayToString(param.quarterId);
    param.blCampusId = transformArrayToString(campusIdArr);

    return param;
  }

  render() {
    const { className } = this.state;
    const Search2Config = this.formatSearch2Config(this.props);

    return (
      <section className={style.search2Wrap}>
        <Row gutter={10} className={style.mb_10}>
          <Col span={8}>
            <Input
              placeholder="班课名称"
              value={className}
              onChange={(e) => {
                this.setState({ className: e.target.value });
              }}
            />
          </Col>
          <Col span={8}>
            <div className={style.search__btnWrap}>
              <Button type="primary" onClick={this.onSearch}>
                查询
              </Button>
              <Button type="default" onClick={this.onReset}>
                重置
              </Button>
            </div>
          </Col>
        </Row>

        <Search2 rowNum={6} config={Search2Config} onChange={this.onSearchChange} />

        <div className={style.mt_10}>{this.renderTeacherSelect()}</div>
      </section>
    );
  }
}

export default SearchFeat;
