import React, { Component } from 'react';
import { connect } from 'dva';
import List from 'COMPONENT/List';
import Page from 'COMPONENT/Page';
import IndexSearch from './IndexSearch';
import Button from 'COMPONENT/Button';
import AddAuxClassModal from './AddAuxClassModal';
import LinkProgress from './LinkProgress';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import ButtonWrap from '../../signUpOnline/StdntApptMgmt/ButtonWrap';
import utils from '../../signUpOnline/StdntApptMgmt/utils.js';
import { clone } from 'lodash';
import { getDictTextByVal, getDictValByText } from 'UTIL/dict';
import { plentyDeleteSeriesClass, getClassAssociatedStatus } from 'SERVICE/course';
import Loader from './loader';
import './style.less';
import style from './style.less';
import { buildBranchCampusDict, buildClassTypeDict, buildClassLevelDict } from 'MODEL/dict/utils';
import {
  TWO_TEACHER_VAL,
  SMALL_CLASS_VAL,
  productTypeAllow,
  isWinterOrSummerVacation,
  statusCfg,
} from './store/config';
import { twoTeacherInstitution } from 'CONST/config';
import { errorsHandle } from 'UTIL/common';

const { COL_WIDTH_M, COL_WIDTH_L } = List;
const campusTempDict = [{ text: '请先选择分公司', val: '', disabled: true }];
const _exportClassExcel = 'OriginalClassController/exportOriginalClassList.do';
const _exportStdExcel = 'OriginalClassController/exportOriginalClassStudentList.do';

const classPrefix = 'ReEnroll';
// 筛选项初始值
const defaultSKey = {
  className: '',
  branchId: null,
  blCampusId: null,
  productTypeId: null,
  yearId: null,
  quarterId: null,
  phaseId: null,
  gradeId: null,
  subjectId: null,
  classTypeId: null,
  classLevel: null,
  mainTeacherId: null,
  assistTeacherId: null,
  associatedStatus: null,
  isConflict: null,
};

class ReEnroll extends Component {
  constructor(props) {
    super(props);
    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.initSKeyWithCheck = this.initSKeyWithCheck.bind(this);
    this.initSKey = this.initSKey.bind(this);
    this.initList = this.initList.bind(this);
    this.loadListData = this.loadListData.bind(this);
    this.loadList = this.loadList.bind(this);
    this.loadListStatus = this.loadListStatus.bind(this);
    this.auxBuildRows = this.auxBuildRows.bind(this);
    this.auxSetPhaseOrWeek = this.auxSetPhaseOrWeek.bind(this);
    this.updateSKey = this.updateSKey.bind(this);
    this.onReset = this.onReset.bind(this);
    this.onSearch = this.onSearch.bind(this);
    this.search = this.search.bind(this);
    this.changePage = this.changePage.bind(this);
    this.exportExcel = this.exportExcel.bind(this);
    this.exportClassExcel = this.exportClassExcel.bind(this);
    this.exportStdExcel = this.exportStdExcel.bind(this);
    this.buildColumns = this.buildColumns.bind(this);
    this.state = {
      loading: true,
      //列表数据
      listLoading: false,
      rows: [],
      total: 0,
      page: 1,
      pageSize: 20,
      //搜索条件
      ...defaultSKey,
      // 列表选中项
      selectedRowKeys: [],
      selectRowClassId: [],
      selectRowVersion: [],
      selectRowQuarter: [],
      selectRowGrade: [],
      selectRowSubject: [],
      selectRowClassType: [],
      selectRowClassLevel: [],
      selectRowCampus: [],
      selectRowMainTeacher: [],
      selectRowAuxTeacher: [],
      // 批量关联弹窗
      toggleAuxClassModal: false,
      // 关联进度弹窗
      toggleLinkProgressModal: false,
      beginShowProgress: false,
      // 批量关联/取消关联 操作类型
      linkActType: null,
      // 调整冲突关联弹窗
      toggleAdjustConflictModal: false,
    };
    this.linkActType = null;
    this.auxClassModalInitData = null;
  }
  componentWillMount() {
    this.init();
  }

  async init() {
    this.setState({ loading: true });
    await new Promise(this.initDict);
    await new Promise(this.initSKeyWithCheck);
    await new Promise(this.initList);
    this.setState({ loading: false });
  }

  initDict(resolve) {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: [
          'branch',
          'campus',
          'branchCampus',
          'productTypeTree',
          'year',
          'quarter',
          'phase',
          'weekDay',
          'grade',
          'subject',
          'twoTcMainTc',
          'twoTcAuxTc',
          'smallClassMainTc',
          'smallClassAuxTc',
        ],
        callback() {
          resolve();
        },
      },
    });
  }

  initSKeyWithCheck(resolve) {
    if (this.props.branchDict && this.props.branchCampusDict && this.props.yearDict && this.props.productTypeTree) {
      this.initSKey(resolve);
    } else {
      const _self = this;
      setTimeout(() => {
        _self.initSKeyWithCheck(resolve);
      }, 100);
    }
  }

  initSKey(resolve) {
    let payload = {};
    const { brenchId, campusId, institution } = this.props.user;
    const isGroup = brenchId === campusId;
    payload.branchId = isGroup ? this.props.branchDict[0].val : brenchId;
    let branchCampusDict = buildBranchCampusDict([payload.branchId], this.props.branchCampusDict);
    let campusDict = branchCampusDict && branchCampusDict[0] && branchCampusDict[0].options;
    payload.blCampusId = isGroup ? (campusDict && campusDict[0] ? [campusDict[0].val] : null) : [campusId];
    payload.productTypeId = institution === twoTeacherInstitution ? TWO_TEACHER_VAL : SMALL_CLASS_VAL;
    payload.yearId = getDictValByText(this.props.yearDict, new Date().getFullYear().toString()) || null;
    payload.quarterId = this.props.quarterDict[0].val;
    let classTypeDict = buildClassTypeDict(this.props.productTypeTree, payload.productTypeId);
    payload.classTypeId = classTypeDict && classTypeDict[0] ? classTypeDict[0].val : null;
    this.setState(
      {
        ...payload,
        sKeyStorage: clone(payload),
      },
      () => {
        resolve();
      },
    );
  }

  initList(resolve, reject) {
    this.loadListData()
      .then((rsp) => {
        let listData = this.auxBuildRows(rsp);
        this.setState(
          () => ({
            rows: listData.rows,
            total: listData.records,
            page: listData.page,
          }),
          () => {
            this.loadListStatus();
            resolve();
          },
        );
      })
      .catch((err) => {
        showMessage(err.text, 'error');
        reject();
      });
  }

  loadListData(params) {
    const {
      className,
      branchId,
      blCampusId,
      productTypeId,
      yearId,
      quarterId,
      phaseId,
      gradeId,
      subjectId,
      page,
      pageSize,
      classTypeId,
      mainTeacherId,
      assistTeacherId,
      associatedStatus,
      isConflict,
      classLevel,
    } = this.state;
    const apiParams = {
      className: className.trim(),
      branchId,
      blCampusId: (blCampusId && blCampusId.join(',')) || null,
      productTypeId,
      yearId,
      quarterId,
      phaseId: (phaseId && phaseId.join(',')) || null,
      gradeId: (gradeId && gradeId.join(',')) || null,
      subjectId: (subjectId && subjectId.join(',')) || null,
      classTypeId: classTypeId || null,
      classLevel: (classLevel && classLevel.join(',')) || null,
      mainTeacherId,
      assistTeacherId,
      page,
      rows: pageSize,
      ...params,
    };
    isConflict && (apiParams.isConflict = Number(isConflict));
    associatedStatus && (apiParams.associatedStatus = Number(associatedStatus));
    return Loader.loadList(apiParams);
  }

  async loadList() {
    this.setState({ listLoading: true, selectedRowKeys: [] });
    let rsp = await this.loadListData();
    let listData = this.auxBuildRows(rsp);
    this.setState(
      {
        listLoading: false,
        rows: listData.rows,
        total: listData.records,
        page: listData.page,
      },
      () => {
        this.loadListStatus();
      },
    );
  }

  // 加载列表关联状态(小班)
  async loadListStatus() {
    if (this.state.productTypeId === SMALL_CLASS_VAL) {
      const listCopy = clone(this.state.rows);
      let classId = listCopy.reduce((total, current) => (total ? `${total},${current.classId}` : current.classId), '');
      const rspList = await getClassAssociatedStatus({ classId }).catch(errorsHandle());
      rspList.forEach((statusItem) => {
        let index = listCopy.findIndex((listItem) => listItem.classId === statusItem.classId);
        if (index > -1) {
          listCopy[index].associatedStatus = statusItem.associatedStatus;
          listCopy[index].statusText = statusCfg[statusItem.associatedStatus];
        }
      });
      this.setState({ rows: listCopy });
    }
  }

  auxBuildRows(origin) {
    let classTypeDict = buildClassTypeDict(this.props.productTypeTree, this.state.productTypeId);
    let data = clone(origin);
    let isTwoTeacher = this.state.productTypeId === TWO_TEACHER_VAL;
    data.rows = data.rows.map((item) => {
      let twoTeacherItem = {};
      if (isTwoTeacher) {
        let associatedStatus = ReEnroll.setColOriginalNumber(item.contractNumber, item.originalNumber);
        twoTeacherItem = {
          associatedStatus,
          statusText: statusCfg[associatedStatus],
        };
      }
      return {
        ...item,
        yearName: getDictTextByVal(this.props.yearDict, item.yearId),
        quarterName: getDictTextByVal(this.props.quarterDict, item.quarterId),
        subjectName: getDictTextByVal(this.props.subjectDict, item.subjectId),
        classTypeName: getDictTextByVal(classTypeDict, item.classTypeId),
        productTypeName: getDictTextByVal(this.props.productTypeTree, item.productTypeId),
        phaseOrWeek: this.auxSetPhaseOrWeek(item),
        ...twoTeacherItem,
      };
    });
    return data;
  }

  auxSetPhaseOrWeek(data) {
    if (data.productTypeId === SMALL_CLASS_VAL) {
      // 小班不分寒暑春秋
      return data.phaseName || '无';
    } else if (isWinterOrSummerVacation(data.quarterId)) {
      // 双师寒暑
      return data.courseWeekName || '无';
    } else {
      // 双师春秋
      return data.phaseId ? getDictTextByVal(this.props.phaseDict, data.phaseId) : '无';
    }
  }

  updateSKey(...params) {
    this.setState(...params);
  }

  onReset() {
    this.setState({
      ...defaultSKey,
      ...this.state.sKeyStorage,
    });
  }

  onSearch() {
    if (!this.state.blCampusId) {
      return showMessage('请至少选择一个学习中心！', 'error');
    }
    this.search();
  }

  async search() {
    this.setState({ page: 1 }, () => {
      this.loadList();
    });
  }

  async changePage({ current, pageSize }) {
    this.setState(
      {
        pageSize,
        page: current,
      },
      () => {
        this.loadList();
      },
    );
  }

  gotoRele = (params) => {
    const { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: {
        path: '/reEnroll/Relevance',
        params: {
          ...params,
          refreshIndex: this.loadList,
        },
        refresh: true,
      },
    });
  };
  // 更新已选中行集合
  updateRowSelected = (keys) => {
    const { rows } = this.state;
    let classIds = [];
    let versions = [];
    let quarters = [];
    let grades = [];
    let subjects = [];
    let classTypes = [];
    let classLevels = [];
    let campus = [];
    let mainTeachers = [];
    let auxTeachers = [];
    keys.forEach((item) => {
      let quarterName = getDictTextByVal(this.props.quarterDict, rows[item].quarterId);
      classIds.push(rows[item].classId);
      versions.push(this.auxBuildVersion({ yearId: rows[item].yearId, quarter: quarterName }));
      quarters = quarters.concat(this.auxBuildQuarter({ quarterId: rows[item].quarterId, quarter: quarterName }));
      grades.push(this.auxBuildGrade({ gradeId: rows[item].gradeId, quarter: quarterName }));
      subjects.push(rows[item].subjectId);
      classTypes.push(rows[item].classTypeId);
      classLevels.push(rows[item].classLevel);
      campus.push(rows[item].blCampusId);
      mainTeachers.push(rows[item].mainTeacherId);
      auxTeachers.push(rows[item].assistTeacherId);
    });
    versions = Array.from(new Set(versions));
    quarters = Array.from(new Set(quarters));
    grades = Array.from(new Set(grades));
    subjects = Array.from(new Set(subjects));
    classTypes = Array.from(new Set(classTypes));
    classLevels = Array.from(new Set(classLevels));
    campus = Array.from(new Set(campus));
    mainTeachers = Array.from(new Set(mainTeachers));
    auxTeachers = Array.from(new Set(auxTeachers));
    this.setState({
      selectedRowKeys: keys,
      selectRowClassId: classIds,
      selectRowVersion: versions,
      selectRowQuarter: quarters,
      selectRowGrade: grades,
      selectRowSubject: subjects,
      selectRowClassType: classTypes,
      selectRowClassLevel: classLevels,
      selectRowCampus: campus,
      selectRowMainTeacher: mainTeachers,
      selectRowAuxTeacher: auxTeachers,
    });
  };
  auxBuildVersion = (params) => {
    const { yearId, quarter } = params;
    return quarter === '秋季'
      ? getDictValByText(this.props.yearDict, (Number(getDictTextByVal(this.props.yearDict, yearId)) + 1).toString())
      : yearId;
  };
  auxBuildQuarter = (params) => {
    const { quarter } = params;
    let quarterArr = [];
    switch (quarter) {
      case '寒假':
        quarterArr = ['春季'];
        break;
      case '春季':
        quarterArr = ['暑假', '秋季'];
        break;
      case '暑假':
        quarterArr = ['秋季'];
        break;
      case '秋季':
        quarterArr = ['寒假', '春季'];
        break;
      default:
        quarterArr = [quarter];
    }
    return quarterArr.map((item) => getDictValByText(this.props.quarterDict, item));
  };
  auxBuildGrade = (params) => {
    const { gradeId, quarter } = params;
    let result = '';
    if (quarter === '春季') {
      let originGradeText = getDictTextByVal(this.props.gradeDict, gradeId);
      let gradeText = '';
      switch (originGradeText) {
        case '幼升小':
        case '幼儿园':
          gradeText = '一年级';
          break;
        case '一年级':
          gradeText = '二年级';
          break;
        case '二年级':
          gradeText = '三年级';
          break;
        case '三年级':
          gradeText = '四年级';
          break;
        case '四年级':
          gradeText = '五年级';
          break;
        case '五年级':
          gradeText = '六年级';
          break;
        case '六年级':
          gradeText = '初一';
          break;
        case '初一':
          gradeText = '初二';
          break;
        case '初二':
          gradeText = '初三';
          break;
        case '初三':
          gradeText = '高一';
          break;
        case '高一':
          gradeText = '高二';
          break;
        case '高二':
          gradeText = '高三';
          break;
        case '高三':
          gradeText = '毕业';
          break;
        default:
          gradeText = originGradeText;
      }
      result = getDictValByText(this.props.gradeDict, gradeText);
    } else {
      result = gradeId;
    }
    return result;
  };
  // 按钮事件-批量关联、取消关联、调整冲突关联
  btnAction = (params) => {
    const { actType } = params;
    const { beginShowProgress } = this.state;
    let linkActType = this.linkActType;
    switch (actType) {
      case 'link':
        this.buildAuxClassModalInitData();
        if (beginShowProgress) {
          showMessage(`有一个${linkActType === 'link' ? '批量' : '取消'}关联操作正在进行，请稍后操作`, 'warning');
        } else {
          this.setState({ toggleAuxClassModal: true });
          this.linkActType = 'link';
        }
        break;
      case 'unLink':
        if (beginShowProgress) {
          showMessage(`有一个${linkActType === 'link' ? '批量' : '取消'}关联操作正在进行，请稍后操作`, 'warning');
        } else {
          this.linkActType = 'unlink';
          this.unLinkClass();
        }
        break;
      case 'adjustConflict':
        this.setState({ toggleAdjustConflictModal: true });
        break;
      case 'importAssociate':
        this.importLinkType();
        break;
      default:
        console.warn(`btnAction-actType: ${actType}`);
    }
  };
  // 导入关联关系
  importLinkType = () => {
    const { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: {
        path: '/reEnroll/ImportLink',
        params: {},
        refresh: true,
      },
    });
  };
  // 取消关联
  unLinkClass = async () => {
    const { selectedRowKeys, rows } = this.state;

    if (selectedRowKeys.length === 0) {
      return showMessage('请先选择要取消关联的班级！', 'error');
    }

    let seriesClassInfos = selectedRowKeys
      .map(
        // associatedStatus === 1 是未关联
        (item) => (rows[item].associatedStatus > 1 ? { originalCourseId: rows[item].classId } : null),
      )
      .filter((item) => Boolean(item));

    if (seriesClassInfos.length === 0) {
      return showMessage('所选原班并没有关联任何续班，不需要取消关联！', 'error');
    }

    const rsp = await plentyDeleteSeriesClass({ seriesClassInfos }, true).catch(errorsHandle());

    if (rsp && rsp.resultCode === 0) {
      this.setState({
        toggleLinkProgressModal: true,
        beginShowProgress: true,
      });
    }
  };
  // 导出excel
  async exportExcel(fetchUrl, fileOriginName) {
    this.setState({ loading: true });
    const {
      productTypeId,
      branchId,
      blCampusId,
      yearId,
      quarterId,
      phaseId,
      gradeId,
      className,
      subjectId,
      classTypeId,
      mainTeacherId,
      assistTeacherId,
    } = this.state;
    const { branchDict, branchCampusDict: campusDict, yearDict, quarterDict, gradeDict } = this.props;
    const fetchParams = {
      className: className.trim(),
      branchId,
      blCampusId: (blCampusId && blCampusId.join(',')) || null,
      productTypeId,
      yearId,
      quarterId,
      phaseId: (phaseId && phaseId.join(',')) || null,
      gradeId: (gradeId && gradeId.join(',')) || null,
      subjectId: (subjectId && subjectId.join(',')) || null,
      classTypeId: classTypeId || null,
      mainTeacherId,
      assistTeacherId,
    };
    // 分公司名称 + 学习中心名称 + 年份 + 季度 + 年级 + 文件名
    let _branchName = getDictTextByVal(branchDict, branchId);
    let _campusName = '';
    if (blCampusId && blCampusId.length > 0) {
      if (blCampusId.length === 1) {
        _campusName = getDictTextByVal(campusDict, blCampusId[0]);
      } else {
        _campusName = blCampusId.map((item) => getDictTextByVal(campusDict, item)).join('/');
      }
    }
    let _yearName = getDictTextByVal(yearDict, yearId);
    let _quarterName = getDictTextByVal(quarterDict, quarterId);
    let _gradeName = '';
    if (gradeId && gradeId.length > 0) {
      if (gradeId.length === 1) {
        _gradeName = getDictTextByVal(gradeDict, gradeId[0]);
      } else {
        _gradeName = gradeId.map((item) => getDictTextByVal(gradeDict, item)).join('/');
      }
    }
    let fileName = `${_branchName}${_campusName}${_yearName}${_quarterName}${_gradeName}${fileOriginName}`;
    const _self = this;
    utils.exportExcel({
      apiUrl: fetchUrl,
      data: fetchParams,
      fileName: fileName,
      success: () => {
        _self.setState({ loading: false });
        showMessage('导出成功！', 'success');
      },
      fail: (err) => {
        _self.setState({ loading: false });
        showMessage(err.text, 'error');
      },
    });
  }
  // 按班级导出excel
  exportClassExcel() {
    this.exportExcel(_exportClassExcel, '关联班级表');
  }
  // 按学生导出excel
  exportStdExcel() {
    this.exportExcel(_exportStdExcel, '关联学生表');
  }
  // 关联回调
  afterLinkSubmit = () => {
    this.setState({
      toggleAuxClassModal: false,
      toggleLinkProgressModal: true,
      beginShowProgress: true,
    });
  };

  linkSuccess = async (params) => {
    let linkText = this.linkActType === 'link' ? '批量' : '取消';
    let msgText = params === 'success' ? `${linkText}关联成功！` : `${linkText}关联失败！`;
    showMessage(msgText, params);
    this.setState({ beginShowProgress: false });
    this.loadList();
  };

  buildAuxClassModalInitData = () => {
    const {
      branchId,
      blCampusId,
      productTypeId,
      yearId,
      quarterId,
      className,
      gradeId,
      subjectId,
      classTypeId,
      classLevel,
      mainTeacherId,
      assistTeacherId,
      selectedRowKeys,
      selectRowClassId,
      selectRowVersion,
      selectRowQuarter,
      selectRowGrade,
      selectRowSubject,
      selectRowClassType,
      selectRowClassLevel,
      selectRowCampus,
      selectRowMainTeacher,
      selectRowAuxTeacher,
    } = this.state;
    const { quarterDict } = this.props;
    const quarterName = getDictTextByVal(quarterDict, quarterId);
    this.auxClassModalInitData =
      selectedRowKeys.length > 0
        ? {
            className,
            branchId,
            productTypeId,
            classId: selectRowClassId,
            version: selectRowVersion,
            quarter: selectRowQuarter,
            grade: selectRowGrade,
            subject: selectRowSubject,
            classType: selectRowClassType,
            classLevel: selectRowClassLevel,
            campus: selectRowCampus,
            mainTeacher: selectRowMainTeacher,
            auxTeacher: selectRowAuxTeacher,
          }
        : {
            className,
            branchId,
            productTypeId,
            classId: [],
            version: [this.auxBuildVersion({ yearId, quarter: quarterName })],
            quarter: this.auxBuildQuarter({ quarterId, quarter: quarterName }),
            grade:
              (gradeId && gradeId.map((item) => this.auxBuildGrade({ gradeId: item, quarter: quarterName }))) || [],
            subject: (subjectId && [...subjectId]) || [],
            classType: classTypeId,
            classLevel: classLevel,
            campus: (blCampusId && [...blCampusId]) || [],
            mainTeacher: mainTeacherId ? [mainTeacherId] : [],
            auxTeacher: assistTeacherId ? [assistTeacherId] : [],
          };
  };

  static setColOriginalNumber(contractNum, originalNum) {
    const contractNumber = parseInt(contractNum, 10);
    const originalNumber = parseInt(originalNum, 10);
    if (!contractNumber) {
      return 1;
    } else if (contractNumber < originalNumber) {
      return 2;
    } else if (contractNumber === originalNumber) {
      return 3;
    }
  }

  buildColumns() {
    const isTwoTeacher = this.state.productTypeId === TWO_TEACHER_VAL;
    const mainTeacherNameTitle = isTwoTeacher ? '主讲老师' : '班主任';
    const assistTeacherNameTitle = isTwoTeacher ? '辅导老师' : '任课老师';
    return [
      { title: '班课名称', key: 'className', dataIndex: 'className', width: COL_WIDTH_M, fixed: 'left' },
      { title: '分公司', key: 'branchName', dataIndex: 'branchName', width: COL_WIDTH_M },
      { title: '所属学习中心', key: 'blCampusName', dataIndex: 'blCampusName', width: COL_WIDTH_M },
      { title: '产品类型', key: 'productTypeName', dataIndex: 'productTypeName', width: COL_WIDTH_M },
      { title: '年份', key: 'yearName', dataIndex: 'yearName', width: COL_WIDTH_M },
      { title: '季度', key: 'quarterName', dataIndex: 'quarterName', width: COL_WIDTH_M },
      { title: '期数/星期数', key: 'phaseOrWeek', dataIndex: 'phaseOrWeek', width: COL_WIDTH_M },
      { title: '科目', key: 'subjectName', dataIndex: 'subjectName', width: COL_WIDTH_M },
      { title: '班级层次', key: 'classLevelName', dataIndex: 'classLevelName', width: COL_WIDTH_M },
      {
        title: mainTeacherNameTitle,
        key: 'mainTeacherName',
        dataIndex: 'mainTeacherName',
        width: COL_WIDTH_L,
        render: (data) => data || '其他',
      },
      {
        title: assistTeacherNameTitle,
        key: 'assistTeacherName',
        dataIndex: 'assistTeacherName',
        width: COL_WIDTH_M,
        render: (data) => data || '无',
      },
      { title: '关联状态', key: 'statusText', dataIndex: 'statusText', width: COL_WIDTH_M },
      {
        title: '操作',
        key: 'action',
        fixed: 'right',
        width: COL_WIDTH_M,
        getActionConfig: (text, record, index) => [{ text: '关联续报', onClick: () => this.gotoRele({ ...record }) }],
      },
    ];
  }

  static getIndexSearchProps(state) {
    return {
      className: state.className,
      branchId: state.branchId,
      blCampusId: state.blCampusId,
      productTypeId: state.productTypeId,
      yearId: state.yearId,
      quarterId: state.quarterId,
      phaseId: state.phaseId,
      gradeId: state.gradeId,
      subjectId: state.subjectId,
      classTypeId: state.classTypeId,
      classLevel: state.classLevel,
      mainTeacherId: state.mainTeacherId,
      assistTeacherId: state.assistTeacherId,
      associatedStatus: state.associatedStatus,
      isConflict: state.isConflict,
    };
  }

  render() {
    const {
      loading,
      listLoading,
      rows,
      page,
      total,
      pageSize,
      branchId,
      productTypeId,
      classTypeId,
      selectedRowKeys,
      toggleAuxClassModal,
      toggleLinkProgressModal,
      beginShowProgress,
    } = this.state;

    const isTwoTeacher = productTypeId === TWO_TEACHER_VAL;

    const campusDictPlus = branchId
      ? this.props.branchCampusDict && this.props.branchCampusDict.filter((branch) => branch.val === branchId)
      : campusTempDict;

    const auxClassModalDict = {
      branchDict: this.props.branchDict,
      campusDict: (campusDictPlus && campusDictPlus[0] && campusDictPlus[0].options) || [],
      campusAllDict: this.props.campusDict,
      versionDict: this.props.yearDict,
      quarterDict: this.props.quarterDict,
      gradeDict: this.props.gradeDict,
      subjectDict: this.props.subjectDict,
      classTypeDict: this.props.classTypeDict,
      mainTcDict: isTwoTeacher ? this.props.twoTcMainTcDict : this.props.smallClassMainTcDict,
      auxTcDict: isTwoTeacher ? this.props.twoTcAuxTcDict : this.props.smallClassAuxTcDict,
      weekDayDict: this.props.weekDayDict,
      phaseDict: this.props.phaseDict,
      classLevelDict: buildClassLevelDict(this.props.productTypeTree, productTypeId, classTypeId),
    };

    const checkSelected = selectedRowKeys.length === 0;

    return (
      <Page
        fitWindowHeight={true}
        loading={loading}
        route={this.props.route}
        headerToolBar={
          <div className={style[`${classPrefix}-header-toolbar-btns`]}>
            <ButtonWrap
              btnText="导出Excel"
              popPlacement="bottom"
              actions={[
                { text: '以班级为单位', onClick: this.exportClassExcel },
                { text: '以学生为单位', onClick: this.exportStdExcel },
              ]}
            />
            {this.props.btnAuths.indexOf('IMPORT_LINK_BUTTON') > -1 && (
              <Button
                onClick={() => {
                  this.btnAction({ actType: 'importAssociate' });
                }}
              >
                导入关联
              </Button>
            )}
            <Button
              onClick={() => {
                this.btnAction({ actType: 'link' });
              }}
              type={checkSelected ? '' : 'primary'}
            >
              批量关联
            </Button>
            <Button
              onClick={() => {
                this.btnAction({ actType: 'unLink' });
              }}
              type={checkSelected ? '' : 'primary'}
              disabled={checkSelected}
            >
              取消关联
            </Button>
            <IndexSearch
              loading={loading}
              updateSearchKey={this.updateSKey}
              onSearch={this.onSearch}
              onReset={this.onReset}
              {...ReEnroll.getIndexSearchProps(this.state)}
            />
          </div>
        }
      >
        <List
          columns={this.buildColumns()}
          loading={listLoading}
          dataSource={rows || []}
          pagination={{
            current: page,
            pageSize,
            total,
          }}
          onChange={this.changePage}
          rowSelection={{
            selectedRowKeys,
            onChange: (keys) => {
              this.updateRowSelected(keys);
            },
          }}
        />
        {toggleAuxClassModal && (
          <AddAuxClassModal
            needLoadDict={false}
            dict={auxClassModalDict}
            initData={this.auxClassModalInitData}
            handleCancel={() => {
              this.setState({ toggleAuxClassModal: false });
            }}
            handleSubmit={this.afterLinkSubmit}
            handleRefreshList={this.loadList}
          />
        )}
        <LinkProgress
          type={this.linkActType}
          begin={beginShowProgress}
          visible={toggleLinkProgressModal}
          handleCancel={() => {
            this.setState({ toggleLinkProgressModal: false });
          }}
          handleSuccess={this.linkSuccess}
        />
        {beginShowProgress && !toggleLinkProgressModal && (
          <Button
            type="primary"
            style={{ position: 'absolute', left: 20, bottom: 48 }}
            onClick={() => {
              this.setState({ toggleLinkProgressModal: true });
            }}
            size={'small'}
          >{`查看${this.linkActType === 'link' ? '批量' : '取消'}关联进度`}</Button>
        )}
      </Page>
    );
  }
}

let productTypeTreeStorage = null;
let quarterDictStorage = null;

const mapStateToProps = (state) => {
  // 过滤与产品类型相关字典
  if (!productTypeTreeStorage && state.dict.productTypeTree) {
    productTypeTreeStorage = state.dict.productTypeTree.filter((item) => productTypeAllow.includes(item.val));
  }
  // 过滤季度字典中的全年选项
  if (!quarterDictStorage && state.dict.quarter) {
    quarterDictStorage = state.dict.quarter.filter((item) => item.val !== 'DAT0000000245');
  }
  return {
    btnAuths: state.account.btnAuths,
    user: state.account.user,
    // 数据字典
    branchDict: state.dict.branch,
    campusDict: state.dict.campus,
    branchCampusDict: state.dict.branchCampus,
    productTypeTree: productTypeTreeStorage,
    yearDict: state.dict.year,
    quarterDict: quarterDictStorage,
    phaseDict: state.dict.phase,
    weekDayDict: state.dict.weekDay,
    gradeDict: state.dict.grade,
    subjectDict: state.dict.subject,
    twoTcMainTcDict: state.dict.twoTcMainTc,
    twoTcAuxTcDict: state.dict.twoTcAuxTc,
    smallClassMainTcDict: state.dict.smallClassMainTc,
    smallClassAuxTcDict: state.dict.smallClassAuxTc,
  };
};

export default connect(mapStateToProps)(ReEnroll);
