import {
  loadListApi,
  getNextClassInfoApi,
  loadClassesApi,
  loadStudentsApi,
  loadTcApi,
  loadConflictApi,
  saveApi,
  delApi,
  delAllApi,
  loadMainTcApi,
  loadAssitTcApi,
  loadSmallClassTcDict,
  getImportLinkList,
  exportUnMatchList,
  addStudentSeriesClassInfo,
} from 'SERVICE/course';
import { getUserByRoldCodesSelection } from 'SERVICE/common';
import { uniqBy } from 'lodash';

/**
 * 获取列表数据
 */
const loadList = async (params) => {
  const data = await loadListApi(params);
  return data;
};
/**
 * 根据分公司和权限获取学习中心
 */
const loadCampus = async (params) => {
  return new Promise(function (resolve, reject) {
    setTimeout(function () {
      resolve([
        { text: '柠溪学习中心', val: '23423' },
        { text: '香洲学习中心', val: '4545' },
      ]);
    }, 1000);
  });
};
/**
 * 加载双师主班老师数据
 */
const loadMainTc = async (params) => {
  const data = await loadMainTcApi(params);
  const teachers = data.map((tc) => {
    return {
      text: tc.name,
      val: tc.userId,
    };
  });
  teachers.unshift({ text: '其他', val: 'other' });
  return teachers;
};
/**
 * 加载双师辅班老师数据
 */
const loadAssistTc = async (params) => {
  const data = await loadAssitTcApi(params);
  return data.map((tc) => {
    return {
      text: tc.name,
      val: tc.userId,
    };
  });
};
/**
 * 加载小班班主任数据(学管师)
 * */
const loadSmallMainTc = async (params) => {
  const userData = Index.getLoginUser();
  let fetchParams = {
    roleCode: 'STUDY_MANAGER,STUDY_MANAGER_HEAD',
    organizationId: userData.organizationId,
    monitoringUserId: userData.userId,
    monitoringUserName: userData.name,
  };
  const rsp = await getUserByRoldCodesSelection(fetchParams, true);
  let dictKey = Object.keys(rsp.value);
  let dict = [];
  dictKey.forEach((item) => {
    dict.push({
      text: rsp.value[item],
      val: item,
    });
  });
  return dict;
};
/**
 * 加载小班授课老师数据
 */
const loadSmallTc = async (params) => {
  const data = await loadSmallClassTcDict(params);
  const teachers = data.map((tc) => {
    return {
      text: tc.name,
      val: tc.userId,
    };
  });
  teachers.unshift({ text: '其他', val: 'other' });
  return uniqBy(teachers, 'val');
};

/**
 * 加载授课老师数据
 */
const loadTc = async (params) => {
  const data = await loadTcApi(params);
  return data;
};

/**
 * 根据原班Id加载学生数据
 */
const loadStudents = async (params) => {
  const data = await loadStudentsApi(params);
  return data.data;
};
/**
 * 根据原班年份，季度，年级 去获取 续班班课的 年份，季度，年级
 */
const getNextClassInfo = async (params) => {
  const data = await getNextClassInfoApi(params);
  return data.data;
};
/**
 * 获取续班列表
 */
const loadClasses = async (params) => {
  const json = {
    ...params,
    rows: params.pageSize,
  };
  const data = loadClassesApi(json);
  return data;
};
/**
 * 保存班课信息
 */
const save = async (params) => {
  const data = await saveApi(params);
  if (data.resultCode !== 0) {
    throw new Error(data.resultMessage);
  }
  return data;
};

/**
 * 批量保存班课信息
 * @param params
 */
const saveStudentSeriesClassInfo = async (params) => {
  const data = await addStudentSeriesClassInfo(params);

  if (data.resultCode !== 0) {
    throw new Error(data.resultMessage);
  }

  return data;
};

/**
 * 删除学生的班课信息
 */
const del = async (params) => {
  const data = await delApi(params);
  return data;
};
/**
 * 删除所有学生的班课信息
 */
const delAll = async (params) => {
  const data = await delAllApi(params);
  return data;
};
/**
 * 检查冲突
 */
const loadConflict = async (params) => {
  const data = await loadConflictApi(params);
  return data;
};

/**
 * 导出模板
 */
const exportDemo = async ({ href, download } = {}) => {
  const link = document.createElement('a');
  const body = document.querySelector('body');
  link.href = href;
  console.log(link.href);
  link.download = download;
  // fix Firefox
  link.style.display = 'none';
  body.appendChild(link);
  link.click();
  body.removeChild(link);
};

const getImportLink = async (params) => {
  const data = await getImportLinkList(params);
  if (data.resultCode !== 0) {
    throw new Error(data.resultMessage);
  }
  return data.data;
};
// 导出关联失败的导入记录
const getUnMatchList = async (params) => {
  const data = await exportUnMatchList(params);
  if (data.resultCode !== 0) {
    throw new Error(data.resultMessage);
  }
  return data;
};
export default {
  loadList,
  loadMainTc,
  loadAssistTc,
  loadTc,
  loadSmallMainTc,
  loadSmallTc,
  loadStudents,
  loadCampus,
  loadClasses,
  save,
  saveStudentSeriesClassInfo,
  del,
  delAll,
  getNextClassInfo,
  loadConflict,
  getImportLink,
  exportDemo,
  getUnMatchList,
};
