import { getDictValByText, getDictTextByVal } from 'UTIL/dict';

/**
 * 根据季度名称季度提升
 * @param params JSON {quarter}
 * @param quarterDict 季度字典
 * @return {*[]}
 */
export function auxBuildQuarter(params = {}, quarterDict) {
  const { quarter } = params;
  let quarterArr = [];

  switch (quarter) {
    case '寒假':
      quarterArr = ['春季'];
      break;
    case '春季':
      quarterArr = ['暑假', '秋季'];
      break;
    case '暑假':
      quarterArr = ['秋季'];
      break;
    case '秋季':
      quarterArr = ['寒假', '春季'];
      break;
    default:
      quarterArr = [quarter];
  }

  return quarterArr.map((item) => {
    return getDictValByText(quarterDict, item);
  });
}

/**
 * 根据年级和季度名称提升年级
 * @param params JSON {gradeId, quarter}
 * @param gradeDict
 * @return {*}
 */
export function auxBuildGrade(params = {}, gradeDict) {
  const { gradeId, quarter } = params;
  let result = '';

  if (quarter === '春季') {
    let originGradeText = getDictTextByVal(gradeDict, gradeId);
    let gradeText = '';

    switch (originGradeText) {
      case '幼升小':
      case '幼儿园':
        gradeText = '一年级';
        break;
      case '一年级':
        gradeText = '二年级';
        break;
      case '二年级':
        gradeText = '三年级';
        break;
      case '三年级':
        gradeText = '四年级';
        break;
      case '四年级':
        gradeText = '五年级';
        break;
      case '五年级':
        gradeText = '六年级';
        break;
      case '六年级':
        gradeText = '初一';
        break;
      case '初一':
        gradeText = '初二';
        break;
      case '初二':
        gradeText = '初三';
        break;
      case '初三':
        gradeText = '高一';
        break;
      case '高一':
        gradeText = '高二';
        break;
      case '高二':
        gradeText = '高三';
        break;
      case '高三':
        gradeText = '毕业';
        break;
      default:
        gradeText = originGradeText;
    }

    result = getDictValByText(gradeDict, gradeText);
  } else {
    result = gradeId;
  }

  return result;
}

/**
 * 根据年份和季度名称提升年份
 * @param params JSON {yearId, quarter}
 * @param yearDict 年份字典
 * @return {*}
 */
export function auxBuildVersion(params = {}, yearDict) {
  const { yearId, quarter } = params;
  const nextYear = getDictValByText(yearDict, (Number(getDictTextByVal(yearDict, yearId)) + 1).toString());

  return quarter === '秋季' ? nextYear : yearId;
}
