import React from 'react';
import { connect } from 'dva';
import SearchBar from 'COMPONENT/SearchBar';
import Select from 'COMPONENT/Select';
import Input from 'COMPONENT/Input';
import { storeName } from '../../store/config';
import { campus } from 'MODEL/dict/utils/config';
import { buildClassLevelDict } from 'MODEL/dict/utils';
import { getRelationsDictPlus } from 'MODEL/dict/utils/index';
import { loginUser } from 'UTIL/common';

class Search extends React.Component {
  constructor(props) {
    super(props);
    this.updateProps = this.updateProps.bind(this);
    this.updateValue = this.updateValue.bind(this);
  }

  updateProps(params) {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: params,
    });
  }

  updateValue(type, val) {
    this.updateProps({ [type]: val });
  }

  render() {
    return (
      <SearchBar
        className={`${this.props.classPrefix}-search`}
        onSearch={this.props.onSearch}
        onReset={this.props.onReset}
      >
        <Input
          placeholder="班级名称"
          value={this.props.sKey_className}
          onChange={(e) => this.updateValue('sKey_className', e.target.value)}
        />
        <Select
          placeholder="学习中心"
          dict={this.props.branchCampusDict}
          value={this.props.sKey_modalCampus}
          disabled={this.props.sKey_isDisabled}
          onChange={(val) => this.updateValue('sKey_modalCampus', val)}
        />
        <Select
          placeholder="年级"
          dict={this.props.gradeDict}
          value={this.props.sKey_grade}
          disabled={this.props.sKey_isDisabled}
          onChange={(val) => this.updateValue('sKey_grade', val)}
        />
        <Select
          placeholder="科目"
          dict={this.props.subjectDict}
          value={this.props.sKey_subject}
          disabled={this.props.sKey_isDisabled}
          onChange={(val) => this.updateValue('sKey_subject', val)}
        />
        <Select
          placeholder="班级层次"
          dict={this.props.classLevelDict}
          value={this.props.sKey_classLevel}
          onChange={(val) => this.updateValue('sKey_classLevel', val)}
        />
      </SearchBar>
    );
  }
}

let classLevelDictStorage = null;

const mapStateToProps = (state) => {
  const { user } = state.account;
  const isCampus = loginUser.isCampusRole();

  // 过滤与产品类型相关字典
  if (!classLevelDictStorage && state.dict.productTypeTree) {
    classLevelDictStorage = buildClassLevelDict(state.dict.productTypeTree, state[storeName].sKey_prdType, null);
  }

  let branchCampusDict = null;
  if (isCampus) {
    branchCampusDict = [
      {
        label: user.brenchName,
        options: [{ val: user.campusId, text: user.campusName }],
      },
    ];
  } else {
    branchCampusDict = getRelationsDictPlus(campus)(state.dict.branchCampusTree, null);
  }

  return {
    // 字典
    branchCampusDict,
    gradeDict: state.dict.grade,
    subjectDict: state.dict.subject,
    classLevelDict: classLevelDictStorage,
    // 筛选项值
    sKey_className: state[storeName].sKey_className,
    sKey_modalCampus: state[storeName].sKey_modalCampus,
    sKey_grade: state[storeName].sKey_grade,
    sKey_subject: state[storeName].sKey_subject,
    sKey_classLevel: state[storeName].sKey_classLevel,
    sKey_isDisabled: state[storeName].sKey_isDisabled,
  };
};

export default connect(mapStateToProps)(Search);
