import React from 'react';
import { connect } from 'dva';
import { SearchButton, Select, Input } from 'COMPONENT';
import { storeName, productTypeAllow, SMALL_CLASS_VAL } from '../store/config';
import { loginUser } from 'UTIL/common';
import { branch, campus } from 'MODEL/dict/utils/config';
import { getRelationsDictPlus, getRelationPayloadPlus } from 'MODEL/dict/utils/index';

class Search extends React.Component {
  constructor(props) {
    super(props);

    this.updateProps = this.updateProps.bind(this);
    this.updateValue = this.updateValue.bind(this);

    this.isTwoTeacher = loginUser.isTwoTeacherSym();

    // 分公司关联关系type、key对应关系配置
    this.branchTreeMap = {
      [branch]: 'sKey_branch',
      [campus]: 'sKey_campus',
    };
  }

  updateProps(params) {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: params,
    });
  }

  updateValue(type, val) {
    this.updateProps({ [type]: val });
  }

  /**
   * 选择分公司/分校
   *
   * @param val
   */
  onBranchChange = (val) => {
    const payload = getRelationPayloadPlus('sKey_branch', val, {
      type: branch,
      dictTree: this.props.branchCampusTree,
      state: this.props,
      map: this.branchTreeMap,
    });

    this.updateProps(payload);
  };

  /**
   * 选择产品类型
   *
   * @param val
   */
  onProductTypeChange = (val) => {
    let payload = { sKey_prdType: val };

    if (this.isTwoTeacher && val !== SMALL_CLASS_VAL) {
      payload.sKey_prdGroup = null;
    }

    this.updateProps(payload);
  };

  render() {
    return this.isTwoTeacher ? (
      <SearchButton
        searchBtnLoading={this.props.searchBtnLoading}
        onSearch={this.props.onSearch}
        onReset={this.props.onReset}
      >
        <Input
          placeholder="学生姓名"
          value={this.props.sKey_stdName}
          onChange={(e) => this.updateValue('sKey_stdName', e.target.value)}
          maxLength={10}
        />
        <Select
          placeholder="分校"
          dict={this.props.branchDict}
          value={this.props.sKey_branch}
          onChange={this.onBranchChange}
          showSearch
        />
        <Select
          placeholder="学习中心"
          dict={this.props.branchCampusDict}
          value={this.props.sKey_campus}
          onChange={(val) => this.updateValue('sKey_campus', val)}
          showSearch
        />
        <Select
          placeholder="产品年份"
          dict={this.props.yearDict}
          value={this.props.sKey_year}
          onChange={(val) => this.updateValue('sKey_year', val)}
        />
        <Select
          placeholder="产品季度"
          dict={this.props.quarterDict}
          value={this.props.sKey_quarter}
          onChange={(val) => this.updateValue('sKey_quarter', val)}
        />
        <Select
          placeholder="产品类型"
          dict={this.props.productTypeTree}
          value={this.props.sKey_prdType}
          onChange={this.onProductTypeChange}
          allowClear={false}
        />
        <Select
          placeholder="产品组"
          dict={this.props.productGroupDict}
          value={this.props.sKey_prdGroup}
          onChange={(val) => this.updateValue('sKey_prdGroup', val)}
          disabled={this.props.sKey_prdType !== SMALL_CLASS_VAL}
          allowClear={true}
        />
      </SearchButton>
    ) : (
      <SearchButton
        searchBtnLoading={this.props.searchBtnLoading}
        onSearch={this.props.onSearch}
        onReset={this.props.onReset}
      >
        <Input
          placeholder="学生姓名"
          value={this.props.sKey_stdName}
          onChange={(e) => this.updateValue('sKey_stdName', e.target.value)}
          maxLength={10}
        />
        <Select
          placeholder="学习中心"
          dict={this.props.branchCampusDict}
          value={this.props.sKey_campus}
          onChange={(val) => this.updateValue('sKey_campus', val)}
          showSearch
        />
        <Select
          placeholder="产品类型"
          dict={this.props.productTypeTree}
          value={this.props.sKey_prdType}
          onChange={(val) => this.updateValue('sKey_prdType', val)}
          allowClear={false}
        />
        <Select
          placeholder="产品年份"
          dict={this.props.yearDict}
          value={this.props.sKey_year}
          onChange={(val) => this.updateValue('sKey_year', val)}
        />
        <Select
          placeholder="产品季度"
          dict={this.props.quarterDict}
          value={this.props.sKey_quarter}
          onChange={(val) => this.updateValue('sKey_quarter', val)}
        />
      </SearchButton>
    );
  }
}

let productTypeTreeStorage = null;

const mapStateToProps = (state) => {
  const { productGroup } = state.dict;

  // 过滤与产品类型相关字典
  if (!productTypeTreeStorage && state.dict.productTypeTree) {
    productTypeTreeStorage = state.dict.productTypeTree.filter((item) => productTypeAllow.includes(item.val));
  }

  return {
    // 字典
    branchCampusTree: state.dict.branchCampusTree,
    branchDict: state.dict.branchCampusTree,
    branchCampusDict: getRelationsDictPlus(campus)(state.dict.branchCampusTree, state[storeName].sKey_branch),
    productTypeTree: productTypeTreeStorage,
    yearDict: state.dict.year,
    quarterDict: state.dict.quarter,
    productGroupDict: productGroup ? productGroup['SMALL_CLASS'] : null,
    // 筛选项值
    sKey_stdName: state[storeName].sKey_stdName,
    sKey_branch: state[storeName].sKey_branch,
    sKey_campus: state[storeName].sKey_campus,
    sKey_prdType: state[storeName].sKey_prdType,
    sKey_prdGroup: state[storeName].sKey_prdGroup,
    sKey_year: state[storeName].sKey_year,
    sKey_quarter: state[storeName].sKey_quarter,
  };
};

export default connect(mapStateToProps)(Search);
