import React from 'react';
import { connect } from 'dva';
import { Page, Button, List, TempStorage } from 'COMPONENT';
import Search from './Search';
import ModalRegClass from './ModalRegClass';
import { storeName } from './store/config';
import style from './index.less';
import { clone } from 'lodash';

const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;

const classPrefix = 'RegisteredClass';

class RegisteredClass extends React.Component {
  constructor(props) {
    super(props);

    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.initDefault = this.initDefault.bind(this);
    this.loadData = this.loadData.bind(this);
    this.refreshData = this.refreshData.bind(this);
    this.updatePage = this.updatePage.bind(this);
    this.changePage = this.changePage.bind(this);
    this.search = this.search.bind(this);
    this.reset = this.reset.bind(this);
    this.selectStudents = this.selectStudents.bind(this);
    this.regClass = this.regClass.bind(this);
    this.batchRegClass = this.batchRegClass.bind(this);
    this.closeModalRegClass = this.closeModalRegClass.bind(this);
    this.regFinish = this.regFinish.bind(this);
    this.exportExcel = this.exportExcel.bind(this);
    this.destroy = this.destroy.bind(this);
    this.deleteSelectedStd = this.deleteSelectedStd.bind(this);
    this.tableRowClick = this.tableRowClick.bind(this);
    this.cleanAllStudent = this.cleanAllStudent.bind(this);

    this.state = {
      name: '课程报名',
      loading: false,
    };
    this.columns = [
      { title: '学生名称', dataIndex: 'studentName', key: 'studentName', width: COL_WIDTH_M, fixed: 'left' },
      { title: '学生编号', dataIndex: 'studentId', key: 'studentId', width: 150 },
      { title: '在读学校', dataIndex: 'schoolName', key: 'schoolName', width: COL_WIDTH_L },
      { title: '学籍学习中心', dataIndex: 'studentBlCampusName', key: 'studentBlCampusName', width: COL_WIDTH_M },
      { title: '合同学习中心', dataIndex: 'campusName', key: 'campusName', width: COL_WIDTH_M },
      { title: '未报读产品', dataIndex: 'unRegProductName', key: 'unRegProductName', width: COL_WIDTH_XL },
      { title: '剩余实收课时', dataIndex: 'restCourseNum', key: 'restCourseNum', width: COL_WIDTH_M },
      { title: '剩余优惠课时', dataIndex: 'restOfferNum', key: 'restOfferNum', width: COL_WIDTH_M },
      { title: '支付状态', dataIndex: 'payStatus', key: 'payStatus', width: COL_WIDTH_M },
      { title: '最近退班时间', dataIndex: 'dropOutTime', key: 'dropOutTime', width: COL_WIDTH_XL },
      { title: '退班人', dataIndex: 'dropOutPerson', key: 'dropOutPerson', width: COL_WIDTH_M },
    ];

    if (props.btnAuth_regClass) {
      this.columns.push({
        title: '操作',
        key: 'action',
        width: COL_WIDTH_S,
        fixed: 'right',
        getActionConfig: (text, record, index) => [
          {
            text: '报班',
            disabled: !props.btnAuth_regClass,
            onClick: () => this.regClass(record, index),
          },
        ],
      });
    }
    this.btnStyle = { marginRight: 20 };
  }

  componentDidMount() {
    this.init();
  }

  componentWillUnmount() {
    this.destroy();
  }

  // 初始化
  async init() {
    this.setState({ loading: true });

    await new Promise(this.initDict);
    await new Promise(this.initDefault);
    await new Promise(this.loadData);

    this.setState({ loading: false });
  }

  initDict(resolve) {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: [
          'productTypeTree',
          'branchCampusTree',
          { name: 'productGroup', params: 'SMALL_CLASS' },
          'year',
          'quarter',
        ],
        callback() {
          resolve();
        },
      },
    });
  }

  initDefault(resolve) {
    this.props.dispatch({
      type: `${storeName}/initSKey`,
      payload: {
        callback() {
          resolve();
        },
      },
    });
  }

  loadData(resolve) {
    this.props.dispatch({
      type: `${storeName}/loadData`,
      payload: {
        callback() {
          resolve();
        },
      },
    });
  }

  async refreshData() {
    this.setState({ loading: true });
    await new Promise(this.loadData);
    this.setState({ loading: false });
  }

  updatePage(pageCurrent, pageSize) {
    let payload = {};
    if (pageSize && pageSize !== this.props.pageSize) {
      payload.pageCurrent = 1;
      payload.pageSize = pageSize;
    } else {
      payload.pageCurrent = pageCurrent;
    }
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload,
    });
  }

  changePage({ current, pageSize }) {
    this.updatePage(current, pageSize);
    this.refreshData();
  }

  search() {
    this.changePage({
      current: 1,
      pageSize: this.props.pageSize,
    });
  }

  reset() {
    this.props.dispatch({
      type: `${storeName}/resetKeyword`,
    });
  }

  selectStudents(record, selected, selectedRows) {
    this.props.dispatch({
      type: `${storeName}/selectStdForReg`,
      payload: {
        selected,
        record,
        index: record.key,
        selectedRows,
      },
    });
  }

  regClass(record, index) {
    this.props.dispatch({
      type: `${storeName}/singleStdReg`,
      payload: {
        record,
        index,
      },
    });
  }

  batchRegClass() {
    this.props.dispatch({ type: `${storeName}/batchRegClass` });
  }

  closeModalRegClass() {
    this.props.dispatch({ type: `${storeName}/closeModal` });
  }

  regFinish() {
    const _self = this;
    this.props.dispatch({
      type: `${storeName}/regFinish`,
      payload: {
        callback() {
          _self.refreshData();
        },
      },
    });
  }

  exportExcel() {
    this.setState({ loading: true });
    const _self = this;
    this.props.dispatch({
      type: `${storeName}/exportExcel`,
      payload: {
        callback() {
          _self.setState({ loading: false });
        },
      },
    });
  }

  destroy() {
    this.props.dispatch({
      type: `${storeName}/resetPage`,
    });
  }

  deleteSelectedStd(student, index) {
    this.props.dispatch({
      type: `${storeName}/deleteStudent`,
      payload: {
        student,
        index,
      },
    });
  }

  tableRowClick(record, index, event) {
    let selected = !this.props.selectedRowKeys.includes(index);
    let _selectedRowKeys = clone(this.props.selectedRowKeys);
    if (selected) {
      _selectedRowKeys.push(index);
    } else {
      _selectedRowKeys.filter((item) => item !== index);
    }
    let selectedRows = _selectedRowKeys.map((index) => clone(this.props.list[index]));
    this.selectStudents(record, selected, selectedRows);
  }

  cleanAllStudent() {
    this.props.dispatch({
      type: `${storeName}/cleanAllStudent`,
      payload: {},
    });
  }

  render() {
    const noStudentSelected = this.props.studentList.length === 0;

    return (
      <Page
        className={style[classPrefix]}
        name={this.state.name}
        fitWindowHeight={true}
        loading={this.state.loading}
        route={this.props.route}
        headerToolBar={
          <div>
            <Button onClick={this.cleanAllStudent} disabled={noStudentSelected} style={{ marginRight: 20 }}>
              清空已选
            </Button>
            {this.props.btnAuth_exportExcel && (
              <Button type="primary" onClick={this.exportExcel} style={this.btnStyle}>
                导出Excel
              </Button>
            )}
            {this.props.btnAuth_regClass && (
              <Button type="primary" onClick={this.batchRegClass} disabled={noStudentSelected} style={this.btnStyle}>
                批量报班
              </Button>
            )}
            <Search searchBtnLoading={this.state.loading} onSearch={this.search} onReset={this.reset} />
          </div>
        }
        subHeader={
          <TempStorage
            className={style[`${classPrefix}-studentList`]}
            label="已选学生"
            isLabelCount
            source={this.props.studentList}
            optionTextKey="studentName"
            onDelete={this.deleteSelectedStd}
          />
        }
      >
        <List
          dataSource={this.props.list}
          columns={this.columns}
          rowSelection={{
            selectedRowKeys: this.props.selectedRowKeys,
            onSelect: this.selectStudents,
          }}
          pagination={{
            total: this.props.total,
            current: this.props.pageCurrent,
            pageSize: this.props.pageSize,
          }}
          onChange={this.changePage}
          onRowClick={this.tableRowClick}
        />

        {this.props.modalTrigger && <ModalRegClass onFinish={this.regFinish} onCancel={this.closeModalRegClass} />}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    list: state[storeName].list,
    total: state[storeName].listTotal,
    pageSize: state[storeName].pageSize,
    pageCurrent: state[storeName].pageCurrent,
    selectedRowKeys: state[storeName].selectedRowKeys,
    studentList: state[storeName].studentList,
    modalTrigger: state[storeName].modalTrigger,
    // 权限
    btnAuth_regClass: state.account.btnAuths.indexOf('BTN_SIGN_UP_CLASS_REG') > 0,
    btnAuth_exportExcel: state.account.btnAuths.indexOf('BTN_SIGN_UP_CLASS_EXPORT') > 0,
  };
};

export default connect(mapStateToProps)(RegisteredClass);
