import React from 'react';
import { connect } from 'dva';
import Modal from 'COMPONENT/Modal';
import List from 'COMPONENT/List';
import PopTip from 'COMPONENT/PopTip';
import Button from 'COMPONENT/Button';
import Search from './Search';
import ModalSelectCourseTime from 'BLOC/ModalSelectCourseTime';
import moment from 'moment';
import './style.less';
import { storeName } from '../store/config';
import ReasonModal from './ReasonModal';
import { computedHourPriceByCourse } from 'UTIL/common';
const { COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;

const ModalFooterBtn = (props) => {
  const { content, popTipVisible, onClose, onOk, onCancel, confirmBtnDisabled } = props;
  return (
    <div style={{ display: 'flex', justifyContent: 'flex-end' }}>
      <PopTip icon="error" content={content} visible={popTipVisible} onClose={onClose} maxWidth={200}>
        <Button type="primary" onClick={onOk} disabled={confirmBtnDisabled}>
          确定
        </Button>
      </PopTip>
      <Button onClick={onCancel}>取消</Button>
    </div>
  );
};

const SelectedPriceTotal = (selectedKeys, list) => {
  let totalAmount = 0;
  if (selectedKeys.length > 0) {
    totalAmount = selectedKeys.reduce((total, current) => total + list[current]._origin.courseTotalPrice, 0);
  }
  return (
    <div style={{ textAlign: 'left' }}>
      <div>课时费用总计{totalAmount}元</div>
    </div>
  );
};

const classPrefix = 'ModalRegClass';

class ModalRegClass extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      name: '报班',
      // loading: false,
      btnTipContent: '该班剩余名额不足，请重新选择班级',
      toggle_btnTip: false,
    };
    this.columns = [
      { title: '班级名称', dataIndex: 'className', key: 'className', width: COL_WIDTH_XL },
      { title: '辅导老师', dataIndex: 'teacherName', key: 'teacherName', width: COL_WIDTH_M },
      { title: '学习中心', dataIndex: 'blCampusName', key: 'blCampusName', width: COL_WIDTH_M },
      { title: '开始上课日期', dataIndex: 'beginDate', key: 'beginDate', width: COL_WIDTH_L },
      { title: '报名情况', dataIndex: 'regInfo', key: 'regInfo', width: COL_WIDTH_M },
      {
        title: '小时单价',
        dataIndex: 'unitPrice',
        key: 'unitPrice',
        width: COL_WIDTH_M,
        render: (value, record) => {
          const { classTimeLength, unitPrice } = record;
          return computedHourPriceByCourse(unitPrice, classTimeLength);
        },
      },
      { title: '已上讲数', dataIndex: 'alreadyClassNum', key: 'alreadyClassNum', width: COL_WIDTH_M },
      { title: '未上讲数', dataIndex: 'willAttendClassNum', key: 'willAttendClassNum', width: COL_WIDTH_M },
      {
        title: '上课时间',
        dataIndex: 'classTime',
        key: 'classTime',
        width: COL_WIDTH_M,
        render: (value, record) => {
          const { classTime, classTimeLength, everyCourseClassNum } = record;
          const endTime = moment()
            .hour(classTime.slice(0, 2) - 0)
            .minute(classTime.slice(3, 5) - 0)
            .add(classTimeLength * everyCourseClassNum, 'minutes')
            .format('HH:mm');
          return `${classTime}-${endTime}`;
        },
      },
    ];
    this.selectedClassId = null;
    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.loadData = this.loadData.bind(this);
    this.updatePage = this.updatePage.bind(this);
    this.changePage = this.changePage.bind(this);
    this.search = this.search.bind(this);
    this.reset = this.reset.bind(this);
    this.updateSelectedRowKeys = this.updateSelectedRowKeys.bind(this);
    this.listRowClick = this.listRowClick.bind(this);
    this.onConfirm = this.onConfirm.bind(this);
    this.closeTips = this.closeTips.bind(this);
    this.switchModalSelectCourseTime = this.switchModalSelectCourseTime.bind(this);
    this.confirmCourseTime = this.confirmCourseTime.bind(this);
    // this.noCourseTime = this.noCourseTime.bind(this);
    this.submit = this.submit.bind(this);
    this.destroy = this.destroy.bind(this);
    this.confirmReasonAuditor = this.confirmReasonAuditor.bind(this);
    this.openReasonModal = this.openReasonModal.bind(this);
    this.showLoading = this.showLoading.bind(this);
    this.hideLoading = this.hideLoading.bind(this);
  }

  componentDidMount() {
    this.init();
  }

  componentWillUnmount() {
    this.destroy();
  }

  async init() {
    this.showLoading();
    await new Promise(this.initDict);
    await new Promise(this.loadData);
    this.hideLoading();
  }

  initDict(resolve) {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['grade', 'subject'],
        callback() {
          resolve();
        },
      },
    });
  }

  loadData(resolve) {
    this.showLoading();
    this.props.dispatch({
      type: `${storeName}/loadClassList`,
      payload: {
        callback() {
          resolve();
        },
      },
    });
    this.hideLoading();
  }

  confirmCourseTime(item, index, list) {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: { selectedCourseList: item },
    });
    this.openReasonModal();
  }

  showLoading() {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        changeClassDataLoading: true,
      },
    });
  }

  hideLoading() {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        changeClassDataLoading: false,
      },
    });
    this.setState({
      modalVisible: false,
    });
  }

  openReasonModal() {
    new Promise((resolve) => {
      this.props.dispatch({
        type: `${storeName}/openReasonModal`,
        payload: {
          resolve,
        },
      });
    }).finally(() => {
      this.hideLoading();
    });
  }

  confirmReasonAuditor() {
    this.submit();
    this.switchModalSelectCourseTime();
  }

  async submit() {
    this.showLoading();
    const isSuccess = await new Promise(this.submitChange());
    if (isSuccess) {
      this.props.onFinish();
    }
    this.hideLoading();
  }

  updatePage(pageCurrent, pageSize) {
    let payload = {};
    if (pageSize && pageSize !== this.props.pageSize) {
      payload.classPageCurrent = 1;
      payload.classPageSize = pageSize;
    } else {
      payload.classPageCurrent = pageCurrent;
    }
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload,
    });
  }

  async changePage({ current, pageSize }) {
    this.updatePage(current, pageSize);
    this.showLoading();
    await new Promise(this.loadData);
    this.hideLoading();
  }

  search() {
    this.changePage({
      current: 1,
      pageSize: this.props.pageSize,
    });
  }

  reset() {
    this.props.dispatch({
      type: `${storeName}/resetModalKeyword`,
    });
  }

  updateSelectedRowKeys(keys) {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: { classSelectedRowKeys: keys },
    });
  }

  listRowClick(record, index, event) {
    this.updateSelectedRowKeys([index]);
  }

  onConfirm() {
    let index = this.props.selectedRowKeys[0];
    let selectedClass = this.props.list[index];
    // 允许最大人数 = 计划招生人数 + 允许超额人数
    if (selectedClass.maxNum - selectedClass.registeredNum >= this.props.modalStudentList.length) {
      this.selectedClassId = selectedClass.classId;
      this.switchModalSelectCourseTime();
    } else {
      this.setState({ toggle_btnTip: true });
    }
  }

  closeTips() {
    this.setState({ toggle_btnTip: false });
  }

  switchModalSelectCourseTime() {
    this.setState(
      (prevState) => ({ modalVisible: !prevState.modalVisible }),
      () => {
        if (!this.state.modalVisible) {
          this.selectedClassId = null;
        }
      },
    );
  }

  submitChange() {
    return (resolve) => {
      this.props.dispatch({
        type: `${storeName}/regClass`,
        payload: {
          callback(params) {
            resolve(params);
          },
        },
      });
    };
  }

  destroy() {
    this.props.dispatch({ type: `${storeName}/resetModalKeyword` });
  }

  render() {
    const {
      restCourseNum /** 剩余实收课时 */,
      restOfferNum /** 剩余优惠课时 */,
      restNatureOfferNum,
      totalUnConsumedNatureHours,
    } = this.props.modalProduct;
    let sumCoursNums = 0;
    if (!isNaN(restCourseNum) && !isNaN(restOfferNum)) {
      sumCoursNums = restCourseNum + restOfferNum;
    }

    return (
      <Modal
        title={this.state.name}
        className={classPrefix}
        loading={this.props.changeClassDataLoading}
        width={1100}
        onCancel={this.props.onCancel}
        footer={
          <ModalFooterBtn
            content={this.state.btnTipContent}
            popTipVisible={this.state.toggle_btnTip}
            onClose={this.closeTips}
            onOk={this.onConfirm}
            onCancel={this.props.onCancel}
            confirmBtnDisabled={this.props.selectedRowKeys.length !== 1}
          />
        }
      >
        <Search classPrefix={`${classPrefix}-bar`} onSearch={this.search} onReset={this.reset} />
        <List
          dataSource={this.props.list}
          columns={this.columns}
          rowSelection={{
            type: 'radio',
            selectedRowKeys: this.props.selectedRowKeys,
            onChange: this.updateSelectedRowKeys,
          }}
          pagination={{
            total: this.props.total,
            current: this.props.pageCurrent,
            pageSize: this.props.pageSize,
          }}
          onChange={this.changePage}
          onRowClick={this.listRowClick}
        />
        {this.state.modalVisible && (
          <ModalSelectCourseTime
            classId={this.selectedClassId}
            productType={this.props.productType}
            noDataDirectCallback={this.openReasonModal}
            onCancel={this.switchModalSelectCourseTime}
            onConfirm={this.confirmCourseTime}
            footerTipsRender={SelectedPriceTotal}
            autoSelectAfter
            noSelectedDisabled
            availableCourseNumber={sumCoursNums}
            restNatureOfferNum={restNatureOfferNum}
            totalUnConsumedNatureHours={totalUnConsumedNatureHours}
          />
        )}
        {this.props.reaSonModalTrigger && (
          <ReasonModal onFinish={this.confirmReasonAuditor} onCancel={this.hideLoading} />
        )}
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    productType: state[storeName].sKey_prdType,
    modalProduct: state[storeName].modalProduct,
    modalStudentList: state[storeName].modalStudentList,
    list: state[storeName].classList,
    total: state[storeName].classListTotal,
    pageSize: state[storeName].classPageSize,
    pageCurrent: state[storeName].classPageCurrent,
    selectedRowKeys: state[storeName].classSelectedRowKeys,
    reaSonModalTrigger: state[storeName].reaSonModalTrigger,
    isBatchChangeClass: state[storeName].selectedRowKeys.length > 1,
    changeClassDataLoading: state[storeName].changeClassDataLoading,
  };
};

export default connect(mapStateToProps)(ModalRegClass);
