import { storeName, TWO_TEACHER_VAL, SMALL_CLASS_VAL, payStatusCfg } from './config';
import { getDictValByText, getDictOptionByVal } from 'UTIL/dict';
import {
  getDropClassStudentList,
  getClassTwoListForEnrollInClass,
  getMiniClassListForEnrollInClass,
  exportDropClassStudentList,
  signUpNewClassForTwoTeacher,
  signUpNewClassForMiniClass,
  checkBeforeStudentSignClass,
  listNodeAuditUsersForSignClass,
  signClassWithFlow,
  listWithSubjectAlias,
} from 'SERVICE/course';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { twoTeacherInstitution } from 'CONST/config';
import { runFunc, errorsHandling } from './utils';
import moment from 'moment';
import { clone, groupBy } from 'lodash';

const defaultState = {
  sKey: {
    sKey_stdName: '',
    sKey_stdId: null, // 通过路由传参
    sKey_branch: null,
    sKey_campus_type: null,
    sKey_campus: null,
    sKey_prdType: null,
    sKey_prdGroup: null,
    sKey_year: null,
    sKey_quarter: null,
    sKey_productName: null,
  },
  list: {
    // 列表数据
    list: [],
    listTotal: 0,
    pageSize: 20,
    pageCurrent: 1,
    selectedRowKeys: [],
  },
  regClassData: {
    unRegProductInfo: null,
    studentList: [],
  },
  regModalSKey: {
    sKey_className: '',
    sKey_modalCampus: null,
    sKey_grade: null,
    sKey_subject: null,
    sKey_classLevel: null,
  },
  regModal: {
    modalTrigger: false,
    modalProduct: null,
    modalStudentList: [],
    classList: [],
    classListTotal: 0,
    classPageSize: 10,
    classPageCurrent: 1,
    classSelectedRowKeys: [],
    modalSKeyStorage: null,
    changeClassDataLoading: false,
  },
  reaSonModal: {
    selectedCourseList: [],
    reaSonModalTrigger: false,
    auditUserId: '',
    changeReason: '',
    reasonModalStorage: null,
    auditUserDict: null,
  },
};
Object.freeze(defaultState);

export default {
  namespace: storeName,
  state: {
    // index
    ...clone(defaultState.sKey),
    ...clone(defaultState.list),
    ...clone(defaultState.regClassData),
    sKeyStorage: null,
    // modal
    ...clone(defaultState.regModalSKey),
    ...clone(defaultState.regModal),
    ...clone(defaultState.reaSonModal),
    branchCampusTypeDict: [],
  },
  reducers: {
    updateState(state, action) {
      const { payload } = action;
      return {
        ...state,
        ...payload,
      };
    },

    updateList(state, action) {
      const {
        payload: { list, total },
      } = action;
      let stdList = list.map((item) => ({
        id: `${item.studentName}_${item.contractProductId}`,
        studentName: item.studentName,
        studentId: item.studentId,
        schoolName: item.schoolName,
        campusId: item.blcampusId,
        campusName: item.blCampusName,
        mainHomeTeacherName: item.mainHomeTeacherName,
        studentBlCampusId: item.studentBlCampusId,
        studentBlCampusName: item.studentBlCampusName,
        unRegProduct: {
          id: item.productId,
          name: item.productName,
          branchId: item.branchId,
          campusId: item.blcampusId,
          yearId: item.yearId,
          quarterId: item.quarterId,
          classType: item.classTypeId,
          restCourseNum: item.remainingRealHours,
          restOfferNum: item.remainingPromotionHours,
          restNatureOfferNum: item.remainingRealNatureHours,
          totalUnConsumedNatureHours: item.totalUnConsumedNatureHours,
        },
        unRegProductId: item.productId,
        unRegProductName: item.productName,
        restCourseNum: item.remainingRealNatureHours,
        restOfferNum: item.remainingPromotionNatureHours,
        payStatus: payStatusCfg[item.paidStatus],
        dropOutTime: item.dropTime || '无',
        dropOutPerson: item.dropTeacherName || '无',
        contractId: item.contractId,
        contractProductId: item.contractProductId,
        _origin: item,
        oldBlcampusId: item.oldBlcampusId,
        oldGradeId: item.oldGradeId,
        oldSubjectId: item.oldSubjectId,
        otherCampusNames: item.otherCampusNames,
      }));
      let selectedRowKeys = [];
      // 有已选学生的话要初始化`selectedRowKeys`
      if (state.studentList.length > 0) {
        stdList.forEach((std, index) => {
          // 同一个学生的合同产品id是唯一的
          if (state.studentList.find((item) => item.id === std.id)) {
            selectedRowKeys.push(index);
          }
        });
      }
      return {
        ...state,
        list: stdList,
        listTotal: total,
        selectedRowKeys,
      };
    },

    updateSelectedRowKeys(state, { payload: { selected, index } }) {
      let selectedRowKeys = clone(state.selectedRowKeys);
      if (selected) {
        selectedRowKeys.unshift(index);
      } else {
        selectedRowKeys = selectedRowKeys.filter((item) => item !== index);
      }
      return {
        ...state,
        selectedRowKeys,
      };
    },

    updateStudentList(state, { payload: { selected, record } }) {
      let studentList = clone(state.studentList);
      if (selected) {
        let std = clone(record);
        delete std.key;
        studentList.unshift(std);
      } else {
        studentList = studentList.filter((item) => item.id !== record.id);
      }
      return {
        ...state,
        studentList,
      };
    },

    updateProduct(state, { payload: { keyName, data } }) {
      return {
        ...state,
        [keyName]: data.unRegProduct,
      };
    },

    updateModalInitData(state, { payload: { record } }) {
      let modalSKeyStorage = {
        // sKey_modalCampus: record.campusId
        sKey_modalCampus: record.oldBlcampusId,
        sKey_grade: record.oldGradeId,
        sKey_subject: record.oldSubjectId,
        sKey_isDisabled: !!record.oldBlcampusId,
      };
      return {
        ...state,
        ...clone(modalSKeyStorage),
        modalSKeyStorage,
      };
    },

    updateTwoTcClassList(state, { payload: { list, total, isTwoTeacherSys } }) {
      const classList = list.map((item) => {
        let allowedExcess = item.allowedExcess || 0;
        let totalNum = item.peopleQuantity + (isTwoTeacherSys ? 0 : allowedExcess);
        return {
          productId: item.productId,
          classId: item.classTwoId,
          className: item.classTwoName,
          teacherName: item.teacherName || '其他',
          beginDate: item.startDate,
          registeredNum: item.enrolledNum,
          totalNum,
          unitPrice: item.unitPrice,
          alreadyClassNum: item.chargedCourseNum,
          willAttendClassNum: item.unChargedCourseNum,
          regInfo: `${item.enrolledNum}/${totalNum}`,
          classTime: item.classTime,
          classTimeLength: item.classTimelength,
          everyCourseClassNum: item.everyCourseClassNum,
          maxNum: item.peopleQuantity + allowedExcess,
        };
      });
      return {
        ...state,
        classList,
        classListTotal: total,
      };
    },

    updateSmallClassList(state, { payload: { list, total, isTwoTeacherSys } }) {
      const classList = list.map((item) => {
        let allowedExcess = item.allowedExcess || 0;
        let totalNum = item.peopleQuantity + (isTwoTeacherSys ? 0 : allowedExcess);
        return {
          productId: item.productId,
          classId: item.classId,
          className: item.className,
          teacherName: item.teacherName || '其他',
          beginDate: item.startDate,
          registeredNum: item.enrolledNum,
          totalNum,
          unitPrice: item.unitPrice,
          alreadyClassNum: item.chargedCourseNum,
          willAttendClassNum: item.unChargedCourseNum,
          regInfo: `${item.enrolledNum}/${totalNum}`,
          classTime: item.classTime,
          classTimeLength: item.classTimelength,
          everyCourseClassNum: item.everyCourseClassNum,
          maxNum: item.peopleQuantity + allowedExcess,
          blCampusName: item.blCampusName,
        };
      });
      return {
        ...state,
        classList,
        classListTotal: total,
      };
    },
  },
  effects: {
    *resetAllModalData(action, { put }) {
      yield put({
        type: 'updateState',
        payload: {
          ...clone(defaultState.regClassData),
          ...clone(defaultState.regModalSKey),
          ...clone(defaultState.regModal),
          ...clone(defaultState.reaSonModal),
          selectedRowKeys: [],
        },
      });
    },
    *closeReasonModal(action, { put }) {
      yield put({
        type: 'updateState',
        payload: clone(defaultState.reaSonModal),
      });
    },

    // 默认参数
    *initSKey({ payload: { callback } }, { select, put }) {
      let initData = yield select((state) => {
        let payload = {};
        const { brenchId, campusId, institution } = state.account.user;
        const isGroup = brenchId === campusId;

        let campusOption = { val: null };
        if (state.dict.branchCampusTree && state.dict.branchCampusTree.length > 0) {
          const branchCampus = state.dict.branchCampusTree[0].campus;
          campusOption = branchCampus[0];
        }

        payload.sKey_campus = isGroup && campusOption ? campusOption.val : campusId;
        payload.sKey_prdType = institution === twoTeacherInstitution ? TWO_TEACHER_VAL : SMALL_CLASS_VAL;
        const nowYear = new Date().getFullYear().toString();
        payload.sKey_year = getDictValByText(state.dict.year, nowYear) || null;
        payload.sKey_campus_type = (getDictOptionByVal(state.dict.campusType, 'CONTRACT') || {}).val; // 默认选择合同学习中心
        return payload;
      });

      yield put({
        type: 'updateState',
        payload: {
          ...initData,
          sKeyStorage: initData,
        },
      });

      runFunc(callback);
    },

    /* 根据学生id初始化 */
    *initParamsSKey({ payload: { params, callback } }, { select, put }) {
      let initData = yield select((state) => {
        const payload = {};
        const { studentId, studentName } = params;
        payload.sKey_stdId = studentId;
        payload.sKey_stdName = studentName;
        payload.sKey_prdType = SMALL_CLASS_VAL;
        return payload;
      });
      yield put({
        type: 'updateState',
        payload: {
          ...initData,
          sKeyStorage: null,
        },
      });

      runFunc(callback);
    },

    *loadData({ payload: { callback } }, { select, put }) {
      const apiParams = yield select((state) => ({
        studentId: state[storeName].sKey_stdId || null,
        studentName: state[storeName].sKey_stdName || null,
        branchId: state[storeName].sKey_branch,
        campusIds: state[storeName].sKey_campus ? [state[storeName].sKey_campus] : null,
        campusType: state[storeName].sKey_campus_type,
        productTypeId: state[storeName].sKey_prdType,
        productGroupId: state[storeName].sKey_prdGroup,
        quarterId: state[storeName].sKey_quarter || null,
        yearId: state[storeName].sKey_year || null,
        productName: state[storeName].sKey_productName || null,
        pageNum: state[storeName].pageCurrent,
        pageSize: state[storeName].pageSize,
      }));
      const { data, resultCode } = yield getDropClassStudentList(apiParams, true).catch(errorsHandling());
      if (resultCode === 0) {
        const { list = [], total = 0 } = data;
        yield put({
          type: 'updateList',
          payload: {
            list: list,
            total: total,
          },
        });
      }
      runFunc(callback);
    },

    *resetKeyword(action, { put, select }) {
      const sKeyStorage = yield select((state) => state[storeName].sKeyStorage);
      yield put({
        type: 'updateState',
        payload: {
          ...clone(defaultState.sKey),
          ...sKeyStorage,
        },
      });
    },

    *resetPage(action, { put }) {
      yield put({
        type: 'updateState',
        payload: {
          sKeyStorage: null,
          ...clone(defaultState.sKey),
          ...clone(defaultState.list),
          ...clone(defaultState.regClassData),
        },
      });
    },

    *exportExcel({ payload: { callback } }, { select, put }) {
      const apiParams = yield select((state) => ({
        studentName: state[storeName].sKey_stdName || null,
        branchId: state[storeName].sKey_branch,
        campusId: state[storeName].sKey_campus,
        campusType: state[storeName].sKey_campus_type,
        productTypeId: state[storeName].sKey_prdType,
        productGroupId: state[storeName].sKey_prdGroup,
        quarterId: state[storeName].sKey_quarter || null,
        yearId: state[storeName].sKey_year || null,
      }));
      const total = yield select((state) => state[storeName].listTotal);
      if (total > 5000) {
        return showMessage('记录数大于5000条，请缩小范围查询，再点击导出', 'error');
      }
      const { resultCode, data, resultMessage } = yield exportDropClassStudentList(apiParams, true).catch(
        errorsHandling(),
      );
      if (resultCode === 0 /* && data*/) {
        // showMessage('导出成功', 'success');
        yield put({ type: 'main/exportNotice' });
      } else {
        showMessage(resultMessage || '导出失败', 'error');
      }
      runFunc(callback);
    },

    *deleteStudent({ payload: { student, index } }, { select, put }) {
      let payload = {};
      payload.selectedRowKeys = yield select((state) => {
        let list = state[storeName].list;
        let selectedRowKeys = clone(state[storeName].selectedRowKeys);
        let selectedList = selectedRowKeys.map((index) => list[index]);
        let checkIndex = selectedList.findIndex((item) => item.id === student.id);
        if (checkIndex > -1) {
          selectedRowKeys.splice(checkIndex, 1);
        }
        return selectedRowKeys;
      });
      payload.studentList = yield select((state) => clone(state[storeName].studentList));
      payload.studentList.splice(index, 1);
      if (payload.studentList.length === 0) {
        payload.unRegProductInfo = null;
      }
      yield put({
        type: 'updateState',
        payload,
      });
    },

    *cleanAllStudent({ payload: { callback } }, { put }) {
      yield put({
        type: 'updateState',
        payload: {
          selectedRowKeys: [],
          unRegProductInfo: null,
          studentList: [],
        },
      });
      runFunc(callback);
    },

    *selectStdForReg({ payload: { selected, record, index } }, { select, put }) {
      let { unRegProductInfo, studentList } = yield select((state) => ({
        unRegProductInfo: state[storeName].unRegProductInfo,
        studentList: state[storeName].studentList,
      }));
      if (selected) {
        const _record = record._origin;
        if (unRegProductInfo && unRegProductInfo.id !== record.unRegProductId) {
          // 校验产品是否一致
          return showMessage('请选择报读同产品、同学习中心的学生！', 'warning');
        } else if (unRegProductInfo && unRegProductInfo.classCampusId !== _record.classCampusId) {
          // 校验学习中心是否一致
          return showMessage('请选择报读同产品、同学习中心的学生！', 'warning');
        } else {
          // 初始化产品信息
          record.unRegProduct.classCampusId = _record.classCampusId;
          record.unRegProduct.classSubjectId = _record.classSubjectId;
          record.unRegProduct.oldBlcampusId = _record.classCampusId;
          record.unRegProduct.oldSubjectId = _record.classSubjectId;

          yield put({
            type: 'updateProduct',
            payload: {
              keyName: 'unRegProductInfo',
              data: record,
            },
          });
        }
      } else if (studentList.length === 1) {
        // 当取消选择的学生是最后一个时，清空产品信息
        yield put({
          type: 'updateState',
          payload: { unRegProductInfo: null },
        });
      }
      yield put({
        type: 'updateSelectedRowKeys',
        payload: { selected, index },
      });
      yield put({
        type: 'updateStudentList',
        payload: { selected, record },
      });
    },
    *batchSelectStdForReg({ payload: { selected, selectedRows, changeRows } }, { select, put }) {
      let { unRegProductInfo } = yield select((state) => ({
        unRegProductInfo: state[storeName].unRegProductInfo,
        studentList: state[storeName].studentList,
      }));
      if (selected) {
        if (
          Object.keys(groupBy(selectedRows, 'unRegProductId')).length !== 1 ||
          (unRegProductInfo && selectedRows.some((row) => row.unRegProductId !== unRegProductInfo.id))
        ) {
          return showMessage('请选择报读同产品、同学习中心的学生！', 'warning'); // 校验产品是否一致
        } else if (
          Object.keys(groupBy(selectedRows, (row) => row._origin.classCampusId)).length !== 1 ||
          (unRegProductInfo && selectedRows.some((row) => row._origin.classCampusId !== unRegProductInfo.classCampusId))
        ) {
          return showMessage('请选择报读同产品、同学习中心的学生！', 'warning'); // 校验学习中心是否一致
        } else {
          // 初始化产品信息
          for (let i = 0; i < selectedRows.length; i++) {
            const record = selectedRows[i];
            record.unRegProduct.classCampusId = record._origin.classCampusId;
            record.unRegProduct.classSubjectId = record._origin.classSubjectId;
            record.unRegProduct.oldBlcampusId = record._origin.classCampusId;
            record.unRegProduct.oldSubjectId = record._origin.classSubjectId;
            yield put({
              type: 'updateProduct',
              payload: {
                keyName: 'unRegProductInfo',
                data: record,
              },
            });
            yield put({
              type: 'updateSelectedRowKeys',
              payload: { selected, index: record.key },
            });
            yield put({
              type: 'updateStudentList',
              payload: { selected, record },
            });
          }
        }
      } else {
        yield put({
          type: 'updateState',
          payload: { unRegProductInfo: null },
        });
        for (let i = 0; i < changeRows.length; i++) {
          const record = changeRows[i];
          record.unRegProduct.classCampusId = record._origin.classCampusId;
          record.unRegProduct.classSubjectId = record._origin.classSubjectId;
          record.unRegProduct.oldBlcampusId = record._origin.classCampusId;
          record.unRegProduct.oldSubjectId = record._origin.classSubjectId;
          yield put({
            type: 'updateSelectedRowKeys',
            payload: { selected, index: record.key },
          });
          yield put({
            type: 'updateStudentList',
            payload: { selected, record },
          });
        }
      }
    },
    *singleStdReg({ payload: { record, index } }, { select, put }) {
      yield put({
        type: 'updateProduct',
        payload: {
          keyName: 'modalProduct',
          data: record,
        },
      });
      yield put({
        type: 'updateState',
        payload: {
          modalStudentList: [record],
        },
      });
      yield put({
        type: 'updateModalInitData',
        payload: {
          record: record,
        },
      });
      yield put({ type: 'openModal' });
    },
    *batchRegClass(action, { select, put }) {
      let { unRegProductInfo, studentList } = yield select((state) => ({
        unRegProductInfo: state[storeName].unRegProductInfo,
        studentList: state[storeName].studentList,
      }));
      yield put({
        type: 'updateState',
        payload: {
          modalProduct: clone(unRegProductInfo),
          modalStudentList: clone(studentList),
        },
      });
      yield put({
        type: 'updateModalInitData',
        payload: {
          record: unRegProductInfo,
        },
      });
      yield put({ type: 'openModal' });
    },

    *openModal(action, { put }) {
      yield put({
        type: 'updateState',
        payload: { modalTrigger: true },
      });
    },

    *closeModal(action, { put }) {
      yield put({
        type: 'updateState',
        payload: clone(defaultState.regModal),
      });
    },
    *openReasonModal({ payload: { resolve } }, { select, put }) {
      const apiParams = yield select(selectedCoursesSelector);
      const selectedRowKeys = yield select((state) => state[storeName].selectedRowKeys);
      try {
        if (selectedRowKeys.length <= 1) {
          const checkRes = yield checkBeforeStudentSignClass(apiParams).catch(errorsHandling());
          if (!checkRes) {
            yield put({
              type: 'updateState',
              payload: { changeClassDataLoading: false },
            });
          } else {
            if (!checkRes.data) {
              const {
                data: { approvalUsers, nodeId },
              } = yield listNodeAuditUsersForSignClass(apiParams).catch(errorsHandling());
              const auditUserDict = approvalUsers.map((user) => ({
                text: user.userName,
                val: user.userId,
              }));
              yield put({
                type: 'updateState',
                payload: {
                  auditUserDict,
                  nodeId,
                },
              });
            }
            yield put({
              type: 'updateState',
              payload: {
                reaSonModalTrigger: true,
                changeClassDataLoading: false,
              },
            });
          }
        } else {
          yield put({
            type: 'updateState',
            payload: {
              reaSonModalTrigger: true,
              changeClassDataLoading: false,
            },
          });
        }
      } catch (err) {
        // do nothing
      } finally {
        resolve();
      }
    },
    // modal
    *loadClassList({ payload }, { select, put }) {
      const apiParams = yield select((state) => ({
        productId: state[storeName].modalProduct.id,
        classTypeId: state[storeName].modalProduct.classType,
        className: state[storeName].sKey_className || null,
        blCampusId: state[storeName].sKey_modalCampus,
        gradeId: state[storeName].sKey_grade || null,
        subjectId: state[storeName].sKey_subject || null,
        classLevel: state[storeName].sKey_classLevel || null,
        page: state[storeName].classPageCurrent,
        rows: state[storeName].classPageSize,
      }));
      let isTwoTeacher = yield select((state) => state[storeName].sKey_prdType === TWO_TEACHER_VAL);
      let effectName = isTwoTeacher ? 'loadTwoTcClassList' : 'loadSmallClassList';
      yield put({
        type: effectName,
        payload: {
          ...payload,
          apiParams,
        },
      });
    },

    *loadTwoTcClassList({ payload: { apiParams, callback } }, { select, put }) {
      const rsp = yield getClassTwoListForEnrollInClass(apiParams, true).catch(errorsHandling());
      if (rsp) {
        const isTwoTeacherSys = yield select((state) => state.account.user.institution === twoTeacherInstitution);
        yield put({
          type: 'updateTwoTcClassList',
          payload: {
            list: rsp.rows,
            total: rsp.records,
            isTwoTeacherSys,
          },
        });
      }
      runFunc(callback);
    },

    *loadSmallClassList({ payload: { apiParams, callback } }, { select, put }) {
      const rsp = yield getMiniClassListForEnrollInClass(apiParams, true).catch(errorsHandling());
      if (rsp) {
        const isTwoTeacherSys = yield select((state) => state.account.user.institution === twoTeacherInstitution);
        yield put({
          type: 'updateSmallClassList',
          payload: {
            list: rsp.rows,
            total: rsp.records,
            isTwoTeacherSys,
          },
        });
      }
      runFunc(callback);
    },

    *resetModalKeyword(action, { select, put }) {
      let modalSKeyStorage = yield select((state) => state[storeName].modalSKeyStorage);
      yield put({
        type: 'updateState',
        payload: {
          ...clone(defaultState.regModalSKey),
          ...clone(modalSKeyStorage),
        },
      });
    },

    *resetAuditUserDict(action, { select, put }) {
      yield put({
        type: 'updateState',
        payload: {
          auditUserDict: null,
        },
      });
    },

    *regClass({ payload: { callback } }, { select, put }) {
      const { auditUserDict, courseList } = yield select((state) => {
        return {
          auditUserDict: state[storeName].auditUserDict,
          courseList: state[storeName].selectedCourseList,
        };
      });
      const effectName = auditUserDict ? 'signClassWithFlow' : 'normalRegClass';
      yield put({
        type: effectName,
        payload: { courseList, callback },
      });
    },

    *signClassWithFlow({ payload: { callback } }, { select, put }) {
      const apiParams = yield select(selectedCoursesSelector);
      const rsp = yield signClassWithFlow(apiParams).catch(errorsHandling());

      let params = false;
      if (rsp && rsp.resultCode === 0) {
        params = true;
        showMessage('报班成功！', 'success');
      }
      runFunc(callback, params);
      yield put({
        type: 'resetAuditUserDict',
      });
    },

    *normalRegClass({ payload }, { select, put }) {
      let isTwoTeacher = yield select((state) => state[storeName].sKey_prdType === TWO_TEACHER_VAL);
      let effectName = isTwoTeacher ? 'regTwoTcClass' : 'regSmallClass';
      yield put({
        type: effectName,
        payload,
      });
    },

    *regTwoTcClass({ payload: { courseList, callback } }, { select, put }) {
      const firstCourse = courseList && courseList[0];
      let apiParams = yield select((state) => {
        const studentList = state[storeName].modalStudentList;
        const classList = state[storeName].classList;
        const selectedRowKeys = state[storeName].classSelectedRowKeys;
        let studentIds = [];
        let contractIds = [];
        let contractProductIds = [];
        studentList.forEach((std) => {
          studentIds.push(std.studentId);
          contractIds.push(std.contractId);
          contractProductIds.push(std.contractProductId);
        });
        return {
          classTwoId: classList[selectedRowKeys[0]].classId,
          studentIds,
          contractIds,
          contractProductIds,
          courseIds: (courseList && courseList.map((course) => course.courseId)) || null,
          productId: state[storeName].modalProduct.id,
          firstSchoolTime: (firstCourse && firstCourse.courseDate) || classList[selectedRowKeys[0]].courseDate,
        };
      });
      const rsp = yield signUpNewClassForTwoTeacher(apiParams, true).catch(errorsHandling());
      let params = false;
      if (rsp && rsp.resultCode === 0) {
        params = true;
        showMessage('报班成功！', 'success');
      }
      runFunc(callback, params);
    },

    *regSmallClass({ payload: { courseList, callback } }, { select, put }) {
      let firstCourse = undefined;
      let _courseList = undefined;
      if (courseList) {
        _courseList = [...courseList];
        _courseList.sort((a, b) => {
          return moment(a.courseDate).valueOf() - moment(b.courseDate).valueOf();
        }); // 按时间排序，越小的时间在前面

        firstCourse = _courseList && _courseList[0];
      }

      let apiParams = yield select((state) => {
        const studentList = state[storeName].modalStudentList;
        const classList = state[storeName].classList;
        const selectedRowKeys = state[storeName].classSelectedRowKeys;
        let studentIds = [];
        let contractIds = [];
        let contractProductIds = [];
        studentList.forEach((std) => {
          studentIds.push(std.studentId);
          contractIds.push(std.contractId);
          contractProductIds.push(std.contractProductId);
        });
        return {
          changeReason: state[storeName].changeReason,
          classId: classList[selectedRowKeys[0]].classId,
          studentIds,
          contractIds,
          contractProductIds,
          courseIds: (_courseList && _courseList.map((course) => course.miniClassCourseId)) || null,
          productId: state[storeName].modalProduct.id,
          firstSchoolTime: (firstCourse && firstCourse.courseDate) || classList[selectedRowKeys[0]].courseDate,
        };
      });
      const rsp = yield signUpNewClassForMiniClass(apiParams, true).catch(errorsHandling());
      let params = false;
      if (rsp && rsp.resultCode === 0) {
        params = true;
        showMessage('报班成功！', 'success');
      }
      runFunc(callback, params);
    },

    *regFinish({ payload: { callback } }, { put }) {
      yield put({
        type: 'closeModal',
      });
      yield put({
        type: 'updateState',
        payload: {
          ...clone(defaultState.regClassData),
        },
      });
      runFunc(callback);
    },
    // 根据城市和年级来查科目
    *updateCourse({ payload: { callback } }, { select, put }) {
      const params = yield select((state) => {
        console.log(state[storeName]);
        return {
          gradeId: state[storeName].sKey_grade,
          orgId: state[storeName].sKey_modalCampus,
        };
      });

      if (!params.gradeId || !params.orgId) {
        return;
      }

      const listRes = yield listWithSubjectAlias(params, true);
      var subjectData = {};
      listRes.data.forEach((item) => {
        subjectData[item.id] = item.aliasName || item.name;
      });
      /*

      regModalSKey: {
    sKey_className: '',
    sKey_modalCampus: null,
    sKey_grade: null,
    sKey_subject: null,
    sKey_classLevel: null,
  },
      */
      let final = yield select((state) => {
        return {
          ...state[storeName],
          sKey_subject: subjectData,
        };
      });

      yield put({
        type: 'updateState',
        payload: {
          ...final,
        },
      });
    },
  },
};

function selectedCoursesSelector(state) {
  const courseList = state[storeName].selectedCourseList;
  const classList = state[storeName].classList;

  let firstCourse = undefined;
  let _courseList = undefined;
  if (courseList) {
    _courseList = [...courseList];
    _courseList.sort((a, b) => {
      return moment(a.courseDate).valueOf() - moment(b.courseDate).valueOf();
    }); // 按时间排序，越小的时间在前面

    firstCourse = _courseList && _courseList[0];
  }

  const firstStudent = state[storeName].modalStudentList[0];

  let list = state[storeName].classList;
  let selectedRowKeys = state[storeName].classSelectedRowKeys;

  return {
    productType: state[storeName].sKey_prdType,
    productId: state[storeName].modalProduct.id,
    classId: list[selectedRowKeys[0]].classId,
    firstSchoolTime: (firstCourse && firstCourse.courseDate) || classList[selectedRowKeys[0]].courseDate,
    studentId: firstStudent.studentId,
    contractId: firstStudent.contractId,
    contractProductId: firstStudent.contractProductId,
    courseIds: (_courseList && _courseList.map((course) => course.miniClassCourseId)) || null,
    changeReason: state[storeName].changeReason,
    auditUserId: state[storeName].auditUserId,
    nodeId: state[storeName].nodeId,
  };
}
