/**
 * 一个结点
 * @property {object} data 结点数据，结构如下：
 * {
    startTime  string  开始时间，'HH:mm'
    endTime  string  结束时间，'HH:mm'
    startPx  number 开始位置（px，相对于所在单元格的开始位置的距离）
    endPx  number 结束位置（px，相对于所在单元格的开始位置的距离）
    width  number  宽度（px）
    classId  string  小班ID
    className  string  小班名称
    classStatusName  string  小班状态名称
    createTime  string  小班创建时间
    classCampusName  string  小班所属学习中心名称
    totalCount  number  招生人数
    signedCount  number  已报人数
    classroomName  string  教室名称
    classroomVolume  number  教室容量
    teacherName  string  老师姓名
  }
 * @property {number} startPxToGroup 相对于结点组的开始位置
 * @property {string} color 颜色 [ red | green | gray ]
 * @property {boolean} isShort 是否缩略显示
 * @property {string} tabKey 当前标签页 [ classroom | teacher ]]
 * @property {string} activeClassId 当前激活状态的小班ID
 * @property {function(string)} onSelectNode 选中结点时的回调，参数为结点对应的小班ID
 * @property {boolean} hasAnimi 切换结点组的展开状态时是否有动画效果
 */
import React from 'react';
import moment from 'moment';
import styles from './styles.less';

class Node extends React.Component {
  render() {
    const {
      data: {
        startTime,
        endTime,
        width,
        classId,
        className,
        classStatusName,
        classCampusName,
        totalCount,
        signedCount,
        classroomName,
        classroomVolume,
        teacherName,
      },
      startPxToGroup,
      color: colorProp,
      isShort,
      tabKey,
      activeClassId,
      onSelectNode,
      hasAnimi,
    } = this.props;
    const timeRange = `${startTime} ~ ${endTime}`;
    const counts = `${signedCount} / ${totalCount}`;
    const color = colorProp ? colorProp : classStatusName === '未上课' ? 'green' : null;

    return (
      <div
        className={
          styles.wrap +
          ' ' +
          (color ? styles[color] : '') +
          (classId === activeClassId ? ' ' + styles.active : '') +
          (isShort ? ' ' + styles.short : '') +
          (hasAnimi ? ' ' + styles.animi : '')
        }
        style={{ left: startPxToGroup, width }}
        onClick={() => onSelectNode(classId)}
        title={className}
      >
        {tabKey === 'classroom' ? (
          <div>
            <div className={styles.info}>{className}</div>
            <div className={styles.info}>
              <span className="bossicon i-shijian"></span>
              {timeRange}
            </div>
            <div className={styles.info}>
              <span className="bossicon i-1"></span>
              {teacherName}
            </div>
            <div className={styles.info}>
              <span className="bossicon i-liangren"></span>
              {counts}
            </div>
          </div>
        ) : (
          <div>
            <div className={styles.info}>{className}</div>
            <div className={styles.info}>
              <span className="bossicon i-shijian"></span>
              {timeRange}
            </div>
            <div className={styles.info}>
              <span className={styles.infoCol}>
                <span className="bossicon i-fangzi"></span>
                {classroomName}
                {classroomVolume !== null && `（${classroomVolume}）`}
              </span>
              <span className={styles.infoCol}>
                <span className="bossicon i-liangren"></span>
                {counts}
              </span>
            </div>
            <div className={styles.info}>
              <span className="bossicon i-dingwei"></span>
              {classCampusName}
            </div>
          </div>
        )}
      </div>
    );
  }
}

export default Node;
