/**
 * 一个结点组
 * @property {object} data 结点组数据，结构如下：
 * { 
      id  number  结点组ID
      row  string  所在行，对应行字典的值
      col  string  所在列，对应列字典的值
      expand  boolean  展开状态
      nodes [string]  结点数组
    }
 * @property {boolean} hasAnimi 切换结点组的展开状态时是否有动画效果
 * @property {function} onToggleNodeGroupExpand 切换结点组展开状态时的回调
 * @property {string} tabKey 当前标签页 [ classroom | teacher ]]
 * @property {string} activeClassId 当前激活状态的小班ID
 * @property {function(string)} onSelectNode 选中结点时的回调，参数为结点对应的小班ID
 */
import React from 'react';
import { Icon } from 'antd';
import Node from '../Node';
import styles from './styles.less';

class NodeGroup extends React.Component {
  render() {
    const {
      data: { id, expand, nodes },
      onToggleNodeGroupExpand,
      ...props
    } = this.props;
    let groupStartPx = 1000000;
    let groupEndPx = -1;
    nodes.forEach((n) => {
      const { startPx, endPx } = n;
      groupStartPx = startPx < groupStartPx ? startPx : groupStartPx;
      groupEndPx = endPx > groupEndPx ? endPx : groupEndPx;
    });

    const visibleNodes = nodes.filter((n, i) => {
      if (!expand && i > 1) {
        return false;
      }
      return true;
    });

    const isMulti = nodes.length > 1;

    return (
      <div
        className={styles.wrap + (isMulti ? ' ' + styles.shadowBg : '')}
        style={{ left: groupStartPx, width: groupEndPx - groupStartPx }}
      >
        {visibleNodes.map((n, i) => {
          return (
            <Node
              key={id + n.classId}
              data={n}
              startPxToGroup={n.startPx - groupStartPx}
              color={isMulti ? 'red' : null}
              isShort={isMulti && !expand ? true : false}
              {...props}
            />
          );
        })}
        {isMulti && (
          <div className={styles.moreBtn} onClick={() => onToggleNodeGroupExpand(id)}>
            <span className={styles.expandIcon + (expand ? ' ' + styles.expand : '')}>
              <Icon type="down" />
            </span>
            <span>{expand ? '收起' : '查看更多'}</span>
          </div>
        )}
      </div>
    );
  }
}

export default NodeGroup;
