/**
 * 排课表的一行
 * @property {[object]} nodeGroups 当前行的结点组列表
 * @property {number} height 行高度（px）
 * @property {[object]} colDict 列字典
 * @property {string} tabKey 当前标签页 [ classroom | teacher ]]
 * @property {boolean} hasAnimi 切换结点组的展开状态时是否有动画效果
 * @property {function(number)} onToggleNodeGroupExpand 切换结点组展开状态时的回调，参数为结点组ID
 * @property {string} activeClassId 当前激活状态的小班ID
 * @property {function(string)} onSelectNode 选中结点时的回调，参数为结点对应的小班ID
 */
import React from 'react';
import NodeGroup from '../NodeGroup';
import { dayW } from '../common';
import styles from './styles.less';

class ScheduleBoardRow extends React.Component {
  render() {
    const { nodeGroups, height, colDict, hasAnimi, ...props } = this.props;

    return (
      <div className={styles.wrap + (hasAnimi ? ' ' + styles.animi : '')} style={{ height }}>
        {colDict.map((col, i) => {
          const cellNodeGroups = nodeGroups.filter((g) => g.col === col.val);
          return (
            <div key={col.val} className={styles.cell} style={{ width: dayW }}>
              {cellNodeGroups.map((g, j) => (
                <NodeGroup key={j} data={g} hasAnimi={hasAnimi} {...props} />
              ))}
            </div>
          );
        })}
      </div>
    );
  }
}

export default ScheduleBoardRow;
