import React from 'react';
import { connect } from 'dva';
import moment from 'moment';
import { Page, List, Button } from 'COMPONENT';
import { Modal } from 'antd';
import Search from './Search';
import { TABLE_COLUMS } from './constants';
import styles from './styles.module.less';
import { isString } from 'UTIL/lang/data-type';
import { getSmallClassChargeRecord, confirmResetAttendance } from 'SERVICE/course';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

class SmallClassChargeRecord extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false,
      pageParams: {},
      tableData: [],
      pagination: {
        total: 0,
        current: 1,
        pageSize: 10,
      },
      searchParams: {},
      visible: false,
      currentRowData: {},
    };
  }

  componentDidMount() {
    this.setState({ pageParams: this.getPageParams(this.props) });
  }

  componentWillReceiveProps(nextProps) {
    const nextPageParams = this.getPageParams(nextProps);
    const pageParams = this.getPageParams(this.props);
    if (
      nextPageParams.contractId !== pageParams.contractId ||
      nextPageParams.classCampusId !== pageParams.classCampusId
    ) {
      this.setState({ pageParams: nextPageParams });
    }
  }

  getPageParams = (props) => {
    return props.pageParams[`/${this.props.route.path}`];
  };

  onPageChange = (data) => {
    let pagination = this.state.pagination;
    let pageParams = Object.keys(pagination).reduce((pre, cur) => {
      pre[cur] = data[cur];
      return pre;
    }, {});

    this.setState(
      {
        pagination: pageParams,
      },
      this.fetchErrorList,
    );
  };

  onSearch = (searchParams) => {
    this.setState(
      {
        searchParams,
      },
      this.fetchErrorList,
    );
  };

  /**
   * @description: 请求审批列表
   */
  fetchErrorList = async () => {
    const { current: pageNum, pageSize } = this.state.pagination;
    let { searchParams } = this.state;

    searchParams = Object.keys(searchParams).reduce((pre, cur) => {
      if (this.formatValue(searchParams[cur])) {
        switch (cur) {
          case 'failTimeStart':
            pre[cur] = moment(searchParams[cur]).startOf('day').format('YYYY-MM-DD HH:mm:ss');
            break;
          case 'failTimeEnd':
            pre[cur] = moment(searchParams[cur]).endOf('day').format('YYYY-MM-DD HH:mm:ss');
            break;
          default:
            pre[cur] = searchParams[cur];
        }
      }
      return pre;
    }, {});

    let queryParams = {
      ...searchParams,
      pageSize,
      pageNum,
    };

    try {
      this.setRequestLoading(true);
      const res = await getSmallClassChargeRecord(queryParams);
      const { list, total } = res.data;

      this.setState({
        tableData: list || [],
        pagination: {
          ...this.state.pagination,
          total: total,
        },
      });
    } catch (e) {
      showMessage(e.text);
      this.setState({
        tableData: [],
        pagination: {
          ...this.state.pagination,
          total: 0,
        },
      });
    } finally {
      this.setRequestLoading(false);
    }
  };

  formatValue = (value) => {
    if (isString(value)) {
      return value.trim();
    }
    return value;
  };

  /**
   * @description 设置列表请求Loading状态
   * @param  {boolean} loading
   * @return {void}
   */
  setRequestLoading = (loading) => {
    this.setState({
      loading,
    });
  };

  resetAttendance = (row) => {
    this.setState({
      visible: true,
      currentRowData: { ...row },
    });
  };

  hideResetModal = () => {
    this.setState({
      visible: false,
    });
  };
  // 学生不是已考勤未结算状态，无法重置考勤。
  confirmResetModal = () => {
    return confirmResetAttendance({ id: this.state.currentRowData.id }).then(() => {
      showMessage('重置成功', 'success');
      this.fetchErrorList();
      this.setState({
        visible: false,
      });
    });
  };

  render() {
    const { route, hasResetPermisison } = this.props;
    const { loading, tableData, pagination, visible, currentRowData } = this.state;
    const resetAttendance = this.resetAttendance;

    const columns = hasResetPermisison
      ? TABLE_COLUMS.concat({
          title: '操作',
          dataIndex: 'operation',
          key: 'operation',
          width: 150,
          render: (text, row) => {
            return <Button onClick={() => resetAttendance(row)}>重置考勤</Button>;
          },
        })
      : TABLE_COLUMS;

    return (
      <Page
        name={route.name}
        route={route}
        subHeader={
          <div>
            <Search onSearch={this.onSearch} pageParams={this.state.pageParams} />
            <List
              dataSource={tableData}
              loading={loading}
              columns={columns}
              pagination={pagination}
              bordered={true}
              onChange={this.onPageChange}
            />
          </div>
        }
      >
        <Modal
          title="提示"
          visible={visible}
          onOk={this.confirmResetModal}
          onCancel={this.hideResetModal}
          okText="确认"
          cancelText="取消"
        >
          <p>
            是否确认重置{currentRowData.miniClassName} -（
            {currentRowData.courseDate} {currentRowData.courseStartTime}-{currentRowData.courseEndTime}）-
            {currentRowData.studentName}
            的考勤记录？
          </p>
          <div className={styles.tip}>
            <p>重要提醒：</p>
            <p>1.重置后该学生的该课节考勤状态将会被清除，如需扣费需重新提交考勤！</p>

            <p>2. 重置后允许该学生对该课节操作调课、转班、预退班、退班和退费！</p>
          </div>
        </Modal>
      </Page>
    );
  }
}

function mapStateToProps(state) {
  const { btnAuths } = state.account;
  const hasResetPermisison = btnAuths.indexOf('SMALL_CLASS_RESET_ATTENDANCE') > -1;

  return {
    hasResetPermisison,
    pageParams: state.main.pageParams,
  };
}

export default connect(mapStateToProps)(SmallClassChargeRecord);
