import React, { Component } from 'react';
import { Input, Button, Modal, Select, SearchFieldForm } from 'COMPONENT';
import SearchBox from 'SRC/BLoComponents/SearchBox';
import { Table, Form, InputNumber, Switch } from 'antd';
import { connect } from 'dva';
import { findOrganizationByParentId, getPageMiniClassForClassGroup } from 'SERVICE/smallClassCustomerTransfer';
import _ from 'lodash';
import { errorsHandle } from 'UTIL/common';
import { listByQueryNameAndRoleCode } from 'SERVICE/common';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import styles from './styles.module.less';

const errorsHandler = errorsHandle();

class AddClassCourseChangeGroupMiniClassModal extends Component {
  state = {
    list: [],
    blCampusDict: [],
    selectedRowKeys: [],

    total: 0,
    pageNum: 1,
    pageSize: 20,
    searchData: {},
    teacherId: undefined,

    loading: false,
    isSubmitLoading: false,
  };

  constructor(props) {
    super(props);
    this.initColumns();
  }

  componentDidMount() {
    this.init();
  }

  componentWillUnmount() {}

  init() {
    this.initDict();
    this.initData();
    this.initSearchFormValues();
  }

  initDict() {
    findOrganizationByParentId({
      orgType: 'CAMPUS',
      parentId: this.props.params.organizationId,
    })
      .then(({ data }) => {
        this.setState({ blCampusDict: (data || []).map(({ id, name }) => ({ val: id, text: name })) });
      })
      .catch(() => {
        this.setState({ blCampusDict: [] });
      });
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['gradeNew', 'subject', 'classType', 'classLevel', 'productGroup'],
      },
    });
  }

  initData() {
    this.onSearch({});
  }

  initSearchFormValues() {
    // const { params: { coursePrice } } = this.props;
    // if (coursePrice != null) {
    //   this.props.form.setFieldsValue({ coursePrice });
    // }
  }

  initColumns() {
    this.columns = [
      {
        width: 250,
        title: '小班名称',
        dataIndex: 'miniClassName',
      },
      {
        width: 130,
        title: '每讲费用',
        dataIndex: 'coursePrice',
      },
      {
        width: 130,
        title: '产品组',
        dataIndex: 'productGroupName',
      },
      {
        width: 130,
        title: '班级层次',
        dataIndex: 'classLevelName',
      },
      {
        width: 115,
        dataIndex: 'totalPeoples',
        title: (
          <div style={{ position: 'absolute', width: '64px', top: '3px', fontSize: '12px', lineHeight: '18px' }}>
            已报人数/招生人数
          </div>
        ),
        render: (value, { inClassPeoples }) => `${inClassPeoples || 0}/${value || 0}`,
      },
      {
        width: 130,
        title: '所属学校中心',
        dataIndex: 'blCampusName',
      },
      {
        width: 130,
        title: '年级',
        dataIndex: 'gradeName',
      },
      {
        width: 130,
        title: '科目',
        dataIndex: 'subjectName',
      },
      {
        width: 130,
        title: '老师',
        dataIndex: 'teacherName',
      },
      {
        width: 115,
        dataIndex: 'totalCourseNumber',
        title: (
          <div style={{ position: 'absolute', width: '64px', top: '3px', fontSize: '12px', lineHeight: '18px' }}>
            已上讲数/总讲数
          </div>
        ),
        render: (value, { attendanceCourseNumber }) => `${attendanceCourseNumber || 0}/${value || 0}`,
      },
      {
        width: 130,
        title: '上课日期',
        dataIndex: 'startDate',
      },
      {
        width: 160,
        title: '上课时间',
        dataIndex: 'courseStartTime',
        render: (_, { courseStartTime, courseEndTime }) =>
          [courseStartTime || '', courseEndTime || ''].filter((v) => !!v).join('-'),
      },
      {
        width: 130,
        title: '状态',
        dataIndex: 'status',
        render: (value) => {
          return value == null ? '-' : { STARTED: '上课中', CONPELETE: '已完成', PENDDING_START: '未上课' }[value];
        },
      },
      {
        width: 130,
        title: '上架状态',
        dataIndex: 'onShelves',
        render: (value) => {
          return value == null ? (
            '-'
          ) : (
            <div>
              <span
                style={{
                  top: '6px',
                  marginRight: '7px',
                  position: 'relative',
                  display: 'inline-block',
                  width: '6px',
                  height: '6px',
                  borderRadius: '3px',
                  background: `${value == 1 ? '#2ACF6D' : '#D9D9D9'}`,
                }}
              >
                &nbsp;
              </span>
              {{ 0: '下架', 1: '上架' }[value]}
            </div>
          );
        },
      },
      {
        width: 200,
        title: '关联规则',
        dataIndex: 'changeGroupName',
        render: (value) => value || '-',
      },
    ];
  }

  async fetchList(apiParams) {
    const {
      params: { yearId, quarterId, organizationId, unitPrice, everyCourseClassNum, coursePrice },
    } = this.props;
    const res = await getPageMiniClassForClassGroup({
      yearId,
      quarterId,
      unitPrice,
      coursePrice,
      everyCourseClassNum,
      ...apiParams,
      branchId: organizationId,
    }).catch(errorsHandler);
    if (res && res.data) {
      const {
        data: { list, total },
      } = res;
      this.setState({
        total: total,
        selectedRowKeys: [],
        list: list.map((item) => ({ ...item })),
      });
    } else {
      this.setState({
        list: [],
        total: 0,
        selectedRowKeys: [],
      });
    }
  }

  async loadList(apiParams) {
    this.setState({ loading: true });
    await this.fetchList(apiParams);
    this.setState({ loading: false });
  }

  // 搜索
  onSearch = () => {
    this.props.form.validateFields((_, searchData) => {
      this.setState(
        () => {
          return {
            searchData,
            pageNum: 1,
          };
        },
        () => {
          this.loadList({
            ...this.state.searchData,
            pageNum: this.state.pageNum,
            pageSize: this.state.pageSize,
          });
        },
      );
    });
  };

  // 重置
  onReset = () => {
    this.props.form.resetFields();
    this.setState({ searchData: {} }); // 清空搜索条件
  };

  // 分页
  onPageChange = ({ current: pageNum, pageSize }) => {
    this.setState(
      () => {
        return {
          pageNum,
          pageSize,
        };
      },
      () => {
        this.loadList({
          ...this.state.searchData,
          pageNum: this.state.pageNum,
          pageSize: this.state.pageSize,
        });
      },
    );
  };

  onSubmit = () => {
    const {
      state: { list, selectedRowKeys: miniClassIds },
      props: {
        params: { id: changeGroupId },
      },
    } = this;
    if (!miniClassIds.length) {
      return showMessage('请选择要添加的班级', 'error');
    }

    const selectedData = list.filter(({ miniClassId }) => miniClassIds.includes(miniClassId));
    const selectedDataGroupByUnitPrice = _.groupBy(selectedData, ({ unitPrice }) => unitPrice);
    const selectedDataGroupByCoursePrice = _.groupBy(selectedData, ({ coursePrice }) => coursePrice);
    const selectedDataGroupByEveryCourseClassNum = _.groupBy(
      selectedData,
      ({ everyCourseClassNum }) => everyCourseClassNum,
    );
    const selectedDataGroupByBranchId = _.groupBy(selectedData, ({ branchId }) => branchId);
    const selectedDataGroupByYearId = _.groupBy(selectedData, ({ yearId }) => yearId);
    const selectedDataGroupByQuarterId = _.groupBy(selectedData, ({ quarterId }) => quarterId);

    const isInvalid = [
      selectedDataGroupByBranchId,
      selectedDataGroupByYearId,
      selectedDataGroupByQuarterId,
      selectedDataGroupByCoursePrice,
      selectedDataGroupByUnitPrice,
      selectedDataGroupByEveryCourseClassNum,
    ]
      .map((groups) => {
        return Object.keys(groups).length;
      })
      .some((length) => {
        return length > 1;
      });

    if (isInvalid) {
      return showMessage('同组小班要满足相同分公司、相同年份、相同季度，相同每讲费用条件，请重新选择小班！', 'error');
    }

    this.setState({ isSubmitLoading: true });
    this.props
      .onSubmit({
        miniClassIds,
        changeGroupId,
      })
      .finally(() => {
        this.setState({ isSubmitLoading: false });
      });
  };

  onCancel = () => {
    this.props.onCancel();
  };

  generateSearchForm() {
    const { blCampusDict } = this.state;
    const { gradeDict, classLevelDict, classTypeDict, subjectDict, productGroupDict } = this.props;
    return (
      <div style={{ marginBottom: '18px' }}>
        <div style={{ display: 'flex', marginBottom: '8px' }}>
          <SearchFieldForm
            name="miniClassName"
            form={this.props.form}
            style={{ flex: 'none', width: '153px', paddingLeft: 0 }}
          >
            <Input placeholder="班级名称" maxLength={200} />
          </SearchFieldForm>
          <SearchFieldForm name="blCampusId" form={this.props.form} style={{ flex: 'none', width: '153px' }}>
            <Select showSearch dict={blCampusDict} placeholder="学习中心" />
          </SearchFieldForm>
          <SearchFieldForm name="gradeId" form={this.props.form} style={{ flex: 'none', width: '153px' }}>
            <Select showSearch dict={gradeDict} placeholder="年级" />
          </SearchFieldForm>
          <SearchFieldForm name="classType" form={this.props.form} style={{ flex: 'none', width: '153px' }}>
            <Select showSearch dict={classTypeDict} placeholder="班级类型" />
          </SearchFieldForm>
          <SearchFieldForm name="classLevel" form={this.props.form} style={{ flex: 'none', width: '153px' }}>
            <Select showSearch dict={classLevelDict} placeholder="班级层次" />
          </SearchFieldForm>
          <SearchFieldForm name="subjectId" form={this.props.form} style={{ flex: 'none', width: '153px' }}>
            <Select showSearch dict={subjectDict} placeholder="科目" />
          </SearchFieldForm>
        </div>
        <div style={{ display: 'flex', marginBottom: '8px' }}>
          <SearchFieldForm
            name="teacherId"
            form={this.props.form}
            style={{ flex: 'none', width: '153px', paddingLeft: 0 }}
          >
            <SearchBox
              mode="edit"
              size="large"
              isMultiple={false}
              paramName="queryName"
              placeholder="任课老师"
              isClearOptionsAfterSelect={true}
              getData={listByQueryNameAndRoleCode}
              extraParams={{ roleCode: 'TEATCHER', number: 100 }}
              onLoaded={(list) =>
                list.map(({ userId, name, employeeNo }) => ({
                  code: userId,
                  name: `${employeeNo ? `${name}（${employeeNo}）` : name}`,
                }))
              }
            />
          </SearchFieldForm>
          <SearchFieldForm name="productGroupId" form={this.props.form} style={{ flex: 'none', width: '153px' }}>
            <Select showSearch dict={productGroupDict} placeholder="产品组" />
          </SearchFieldForm>
          {this.props.params.coursePrice == null && (
            <SearchFieldForm name="coursePrice" form={this.props.form} style={{ flex: 'none', width: '153px' }}>
              <InputNumber min={0} max={10000} placeholder="每讲费用" />
            </SearchFieldForm>
          )}
          <SearchFieldForm name="filterType" form={this.props.form} style={{ flex: 'none', width: '200px' }}>
            <Select
              showSearch
              dict={[
                { val: '0', text: '关联' },
                { val: '1', text: '未关联' },
              ]}
              placeholder="班级是否关联自定义规则"
            />
          </SearchFieldForm>
          <Button
            type="primary"
            onClick={this.onSearch}
            style={{ marginRight: '10px', marginLeft: '8px', position: 'relative', top: '5px' }}
          >
            查询
          </Button>
          <Button
            onClick={this.onReset}
            style={{ marginRight: '10px', marginLeft: '5px', position: 'relative', top: '5px' }}
          >
            重置
          </Button>
        </div>
      </div>
    );
  }

  generateTable() {
    const { list, total, pageNum, pageSize, selectedRowKeys } = this.state;
    const rowSelection = {
      selectedRowKeys,
      onChange: (selectedRowKeys) => {
        this.setState({ selectedRowKeys });
      },
      getCheckboxProps: ({ changeGroupId }) => ({
        disabled: changeGroupId != null,
      }),
    };
    return (
      <Table
        bordered={false}
        dataSource={list}
        fixedHeader={false}
        columns={this.columns}
        rowSelection={rowSelection}
        onChange={this.onPageChange}
        rowKey={({ miniClassId }) => miniClassId}
        pagination={{
          total,
          pageSize,
          current: pageNum,
          showQuickJumper: true,
          showSizeChanger: true,
          defaultPageSize: 20,
          pageSizeOptions: ['20', '50', '100'],
          showTotal: (total) => `共 ${total} 条记录`,
        }}
        scroll={{ y: 300, x: this.columns.reduce((sum, item) => (sum += item.width), 0) }}
      />
    );
  }

  generateModal() {
    const props = {
      width: 960,
      title: '添加班级',
      onCancel: this.onCancel,
      loading: this.state.loading,
      wrapClassName: styles.addMiniClassModalWrapper,
      footer: (
        <div className={styles.submitButtonWrapper}>
          <Button type="primary" onClick={this.onSubmit}>
            添加班级到自定义规则
          </Button>
        </div>
      ),
    };
    return (
      <Modal {...props}>
        {this.generateSearchForm()}
        {this.generateTable()}
      </Modal>
    );
  }

  render() {
    return this.generateModal();
  }
}

const mapStoreToProps = (store) => {
  const {
    dict: {
      gradeNew: gradeDict,
      classType: classTypeDict,
      subject: subjectDict,
      classLevel: classLevelDict,
      productGroup: productGroupDict,
    },
  } = store;
  return {
    gradeDict,
    subjectDict,
    classTypeDict,
    classLevelDict,
    productGroupDict,
  };
};

export default connect(mapStoreToProps)(Form.create()(AddClassCourseChangeGroupMiniClassModal));
