import React from 'react';
import { Form, Row, Col } from 'antd';
import { Modal, Input, Select } from 'COMPONENT';
import { connect } from 'dva';

const { Item: FormItem } = Form;
const formItemCol = {
  colon: false,
  labelCol: { span: 5 },
  wrapperCol: { span: 18 },
};
class CreateCustomerTransferRuleModal extends React.Component {
  state = {
    loading: false,
    initialValues: {},
  };

  constructor(props) {
    super(props);
  }

  componentDidMount() {
    const {
      yearDict,
      branchDict,
      user: { brenchId, campusId },
    } = this.props;
    const isGroup = brenchId === campusId;
    const branchId = isGroup ? branchDict[0].val : brenchId;
    const yearId = yearDict.find(({ text }) => text == new Date().getFullYear()).val;
    this.setState({
      initialValues: { yearId, branchId },
    });
  }

  onSubmit = () => {
    this.props.form.validateFields((error, value) => {
      if (error) {
        return;
      }
      this.setState({ loading: true });
      this.props.onSubmit(value).finally(() => {
        this.setState({ loading: false });
      });
    });
  };

  onCancel = () => {
    this.props.onCancel();
  };

  render() {
    const {
      initialValues: { yearId, branchId },
    } = this.state;
    const {
      branchDict,
      yearDict,
      quarterDict,
      form: { getFieldDecorator },
    } = this.props;
    const props = {
      width: 680,
      title: '创建调转规则',
      onOk: this.onSubmit,
      onCancel: this.onCancel,
      loading: this.state.loading,
    };

    return (
      <Modal {...props}>
        <Row>
          <Col span={24}>
            <FormItem label="规则名称" {...formItemCol}>
              {getFieldDecorator('name', {
                rules: [
                  {
                    required: true,
                    message: '请填写规则名称',
                  },
                ],
              })(<Input placeholder="请输入" maxLength={32} />)}
            </FormItem>
          </Col>
          <Col span={24}>
            <FormItem label="归属分公司" {...formItemCol}>
              {getFieldDecorator('organizationId', {
                initialValue: branchId,
                rules: [
                  {
                    required: true,
                    message: '请选择归属分公司',
                  },
                ],
              })(<Select showSearch dict={branchDict} placeholder="请选择" />)}
            </FormItem>
          </Col>
          <Col span={24}>
            <FormItem label="年份" {...formItemCol}>
              {getFieldDecorator('yearId', {
                initialValue: yearId,
                rules: [
                  {
                    required: true,
                    message: '请选择年份',
                  },
                ],
              })(<Select showSearch dict={yearDict} placeholder="请选择" />)}
            </FormItem>
          </Col>
          <Col span={24}>
            <FormItem label="产品季度" {...formItemCol}>
              {getFieldDecorator('quarterId', {
                rules: [
                  {
                    required: true,
                    message: '请选择产品季度',
                  },
                ],
              })(<Select showSearch dict={quarterDict} placeholder="请选择" />)}
            </FormItem>
          </Col>
        </Row>
      </Modal>
    );
  }
}

const mapStoreToProps = (store) => {
  const {
    account: { user },
    dict: { branch: branchDict, year: yearDict, quarter: quarterDict },
  } = store;
  return {
    user,
    branchDict,
    quarterDict,
    yearDict: (yearDict || []).sort((a, b) => b.text - a.text),
  };
};

export default connect(mapStoreToProps)(Form.create()(CreateCustomerTransferRuleModal));
