import React, { Component } from 'react';
import { connect } from 'dva';
import _ from 'lodash';
import { browserHistory } from 'dva/router';
import { OLDPAGEBASE } from 'CONST/config';
import { List, Input, Select, Button, Modal } from 'COMPONENT';
import { Icon } from 'antd';
import InfoBlock from './InfoBlock';
import AddClassCourseChangeGroupMiniClassModal from '../AddClassCourseChangeGroupMiniClassModal';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

import { errorsHandle } from 'UTIL/common';

import {
  addClassCourseChangeGroupMiniClass,
  deleteClassCourseChangeGroupMiniClass,
  getClassCourseChangeGroupMiniClassList,
} from 'SERVICE/smallClassCustomerTransfer';

import styles from './styles.module.less';

const errorsHandler = errorsHandle();

class SmallClassCustomerTransferDetail extends Component {
  state = {
    list: [],

    total: 0,
    pageNum: 1,
    pageSize: 20,
    coursePrice: undefined,

    tableFixedHeight: 100,

    isDeleteLoading: false,

    isAddMiniClassVisible: false,
    isConfirmDeleteModalVisible: false,
  };

  constructor(props) {
    super(props);
    this.currentId = null;
    this.searchRef = null;
  }

  componentWillMount() {
    this.init();
    this.initColumns();
    this.initParams();
    // this.setTableFixedHeight();
    // window.addEventListener('resize', this.setTableFixedHeight, false);
  }

  componentWillUnmount() {
    // window.removeEventListener('resize', this.setTableFixedHeight, false);
  }

  init() {
    this.initDict();
    this.initData();
  }

  initColumns() {
    this.columns = [
      {
        width: 250,
        title: '小班名称',
        dataIndex: 'miniClassName',
      },
      {
        width: 130,
        title: '每讲费用',
        dataIndex: 'coursePrice',
      },
      {
        width: 130,
        title: '产品组',
        dataIndex: 'productGroupName',
      },
      {
        width: 130,
        title: '班级层次',
        dataIndex: 'classLevelName',
      },
      {
        width: 115,
        dataIndex: 'totalPeoples',
        title: (
          <div style={{ position: 'absolute', width: '64px', top: '6px', fontSize: '12px', lineHeight: '18px' }}>
            已报人数/招生人数
          </div>
        ),
        render: (value, { inClassPeoples }) => `${inClassPeoples || 0}/${value || 0}`,
      },
      {
        width: 130,
        title: '所属学校中心',
        dataIndex: 'blCampusName',
      },
      {
        width: 130,
        title: '年级',
        dataIndex: 'gradeName',
      },
      {
        width: 130,
        title: '科目',
        dataIndex: 'subjectName',
      },
      {
        width: 130,
        title: '老师',
        dataIndex: 'teacherName',
      },
      {
        width: 115,
        dataIndex: 'totalCourseNumber',
        title: (
          <div style={{ position: 'absolute', width: '64px', top: '6px', fontSize: '12px', lineHeight: '18px' }}>
            已上讲数/总讲数
          </div>
        ),
        render: (value, { attendanceCourseNumber }) => `${attendanceCourseNumber || 0}/${value || 0}`,
      },
      {
        width: 130,
        title: '上课日期',
        dataIndex: 'startDate',
      },
      {
        width: 160,
        title: '上课时间',
        dataIndex: 'courseStartTime',
        render: (_, { courseStartTime, courseEndTime }) =>
          [courseStartTime || '', courseEndTime || ''].filter((v) => !!v).join('-'),
      },
      {
        width: 130,
        title: '状态',
        dataIndex: 'status',
        render: (value) => {
          return value == null ? '-' : { STARTED: '上课中', CONPELETE: '已完成', PENDDING_START: '未上课' }[value];
        },
      },
      {
        width: 130,
        title: '上架状态',
        dataIndex: 'onShelves',
        render: (value) => {
          return value == null ? (
            '-'
          ) : (
            <div>
              <span
                style={{
                  top: '10px',
                  marginRight: '7px',
                  position: 'relative',
                  display: 'inline-block',
                  width: '6px',
                  height: '6px',
                  borderRadius: '3px',
                  background: `${value == 1 ? '#2ACF6D' : '#D9D9D9'}`,
                }}
              >
                &nbsp;
              </span>
              {{ 0: '下架', 1: '上架' }[value]}
            </div>
          );
        },
      },
      {
        width: 96,
        title: '操作',
        fixed: 'right',
        dataIndex: 'action',
        render: (_, record) => {
          return (
            <div>
              {this.props.btnAuth.includes('CLASS_CUST_TRAN_VIEW_DETAIL') && (
                <a style={{ marginRight: '10px' }} onClick={() => this.onViewDetail(record)}>
                  详情
                </a>
              )}
              {this.props.btnAuth.includes('CLASS_CUST_TRAN_DELETE_DETAIL') && (
                <a onClick={() => this.onDelete(record.miniClassId)}>删除</a>
              )}
            </div>
          );
        },
      },
    ];
  }

  initParams() {
    this.params = this.props.pageParams['/smallClassCustomerTransferDetail'];
  }

  initDict() {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['branch', 'year', 'quarter'],
      },
    });
  }

  initData() {
    this.setState(
      {
        pageNum: 1,
      },
      () => {
        const { pageSize, pageNum } = this.state;
        this.loadList({ pageNum, pageSize }, true);
      },
    );
  }

  reload() {
    const { pageSize, pageNum } = this.state;
    this.loadList({ pageNum, pageSize }, false);
  }

  async loadList(apiParams, isInit = false) {
    this.setState({ loading: true });
    await this.fetchList(apiParams, isInit);
    this.setState({ loading: false });
  }

  async fetchList(apiParams, isInit) {
    const res = await getClassCourseChangeGroupMiniClassList({
      changeGroupId: this.params.id,
      ...apiParams,
    }).catch(errorsHandler);
    if (res && res.data) {
      const {
        data: { total, list },
      } = res;
      this.setState({
        total: total,
        list: list.map((item, id) => ({ id, ...item })),
        coursePrice: isInit
          ? _.pick((list || [])[0] || {}, ['coursePrice', 'unitPrice', 'everyCourseClassNum'])
          : total
          ? this.state.coursePrice
          : undefined,
      });
    } else {
      this.setState({
        list: [],
        total: 0,
        coursePrice: undefined,
      });
    }
  }

  setTableFixedHeight = () => {
    this.setState({
      tableFixedHeight: Math.max(window.document.body.clientHeight - 388, 100),
    });
  };

  onViewDetail = ({ miniClassId }) => {
    // 路由切换
    browserHistory.push({
      pathname: OLDPAGEBASE,
      query: {
        title: '小班详情',
        src: `function/course/miniClassDetail.html?miniClassId=${miniClassId}`,
      },
    });
  };

  onDelete = (miniClassId) => {
    this.currentId = miniClassId;
    this.setState({ isConfirmDeleteModalVisible: true });
  };

  // 添加班级
  onAdd = () => {
    this.setState({ isAddMiniClassVisible: true });
  };

  onSubmitAddMiniClass = (values) => {
    return addClassCourseChangeGroupMiniClass(values)
      .then((result) => {
        if (result.resultCode === 0) {
          this.initData();
          showMessage('班级添加成功', 'success');
          setTimeout(() => this.setState({ isAddMiniClassVisible: false }), 100);
        } else {
          showMessage(result.resultMessage || '班级添加失败，请稍后重试', 'error');
        }
      })
      .catch(errorsHandler);
  };

  onCancelAddMiniClass = () => {
    this.setState({ isAddMiniClassVisible: false });
  };

  // 分页
  onPageChange = ({ current: pageNum, pageSize }) => {
    this.setState(
      () => {
        return {
          pageNum,
          pageSize,
        };
      },
      () => {
        this.loadList({
          ...this.state.searchData,
          pageNum: this.state.pageNum,
          pageSize: this.state.pageSize,
        });
      },
    );
  };

  handleDelete = () => {
    this.setState({ isDeleteLoading: true });
    deleteClassCourseChangeGroupMiniClass({
      miniClassIds: [this.currentId],
      changeGroupId: this.params.id,
    })
      .then((result) => {
        if (result.resultCode === 0) {
          this.reload();
          this.handleCancelDelete();
          showMessage('班级移除成功', 'success');
        } else {
          showMessage(result.resultMessage || '班级移除失败，请稍后重试', 'error');
        }
      })
      .catch(errorsHandler)
      .finally(() => {
        this.setState({ isDeleteLoading: false });
      });
  };

  handleCancelDelete = () => {
    this.setState({ isConfirmDeleteModalVisible: false });
  };

  generateMainContent() {
    const {
      state,
      props: { branchDict, yearDict, quarterDict },
    } = this;
    return (
      <div>
        <InfoBlock title="规则信息" style={{ marginBottom: '10px' }}>
          <div className={styles.ruleInfoWrapper}>
            <Input
              placeholder="调转规则名称"
              style={{ width: '220px' }}
              disabled
              value={decodeURIComponent(this.params.name)}
            />
            <Select
              dict={branchDict}
              placeholder="分公司"
              style={{ width: '154px' }}
              disabled
              value={this.params.organizationId}
            ></Select>
            <Select
              dict={yearDict}
              placeholder="年份"
              style={{ width: '154px' }}
              disabled
              value={this.params.yearId}
            ></Select>
            <Select
              dict={quarterDict}
              placeholder="产品季度"
              style={{ width: '154px' }}
              disabled
              value={this.params.quarterId}
            ></Select>
          </div>
        </InfoBlock>
        <InfoBlock
          title="班级列表"
          toolbar={
            this.props.btnAuth.includes('CLASS_CUST_TRAN_ADD_DETAIL') && (
              <Button
                icon="plus"
                type="primary"
                style={{ float: 'right', position: 'relative', top: '6px' }}
                onClick={this.onAdd}
              >
                添加班级
              </Button>
            )
          }
        >
          <List
            rowKey="id"
            dataSource={state.list}
            columns={this.columns}
            bordered={false}
            fixedHeader={true}
            onChange={this.onPageChange}
            defaultPagination={{
              defaultPageSize: 20,
              pageSizeOptions: ['20', '50', '100'],
            }}
            pagination={{
              total: state.total,
              current: state.pageNum,
              pageSize: state.pageSize,
              showQuickJumper: true,
              showSizeChanger: true,
              pageSizeOptions: ['20', '50', '100'],
              showTotal: (total) => `共 ${total} 条记录`,
            }}
          />
        </InfoBlock>
      </div>
    );
  }

  generateMiniClassModal() {
    const props = {
      params: { ...this.params, ...(this.state.coursePrice || {}) },
      onCancel: this.onCancelAddMiniClass,
      onSubmit: this.onSubmitAddMiniClass,
    };

    return this.state.isAddMiniClassVisible && <AddClassCourseChangeGroupMiniClassModal {...props} />;
  }

  // 二次确认弹框
  generateConfirmDeleteModal() {
    const props = {
      widthSize: 's',
      title: '温馨提示',
      onOk: this.handleDelete,
      loading: this.state.isDeleteLoading,
      onCancel: this.handleCancelDelete,
    };

    return (
      this.state.isConfirmDeleteModalVisible && (
        <Modal {...props}>
          <div className={styles.confirmTextWrapper}>
            <Icon
              type="exclamation-circle"
              style={{ color: '#ffbf00', fontSize: 22, marginRight: 5, position: 'relative', top: 2 }}
            />
            确认从该规则中移除班级？
          </div>
        </Modal>
      )
    );
  }

  render() {
    return (
      <div className={styles.smallClassCustomerTransferDetailWrapper}>
        {this.generateMainContent()}
        {this.generateMiniClassModal()}
        {this.generateConfirmDeleteModal()}
      </div>
    );
  }
}

const mapStoreToProps = (store) => {
  const {
    main: { pageParams },
    account: { btnAuths },
    dict: { branch: branchDict, year: yearDict, quarter: quarterDict },
  } = store;

  return {
    yearDict,
    pageParams,
    branchDict,
    quarterDict,
    btnAuth: btnAuths || [],
    yearDict: (yearDict || []).sort((a, b) => b.text - a.text),
  };
};

export default connect(mapStoreToProps)(SmallClassCustomerTransferDetail);
