import React from 'react';
import { connect } from 'dva';
import { Modal, List } from 'COMPONENT';
import { Row, Col } from 'antd';
import styles from './style.less';
import { storeName } from '../store/config';

class ModalChangeClassDetail extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false,
    };

    this.init = this.init.bind(this);
    this.getDetail = this.getDetail.bind(this);

    this.columns = [
      { title: '处理人', dataIndex: 'employeeName', key: 'employeeName', width: 60 },
      { title: '岗位', dataIndex: 'stationName', key: 'stationName', width: 60 },
      { title: '部门', dataIndex: 'orgName', key: 'orgName', width: 60 },
      { title: '操作类型', dataIndex: 'operateStatusName', key: 'operateStatusName', width: 60 },
      { title: '操作时间', dataIndex: 'operateTimeStr', key: 'operateTimeStr', width: 80 },
      {
        title: '备注',
        dataIndex: 'notes',
        key: 'notes',
        width: 100,
        render(text) {
          return text === '' || !text ? '-' : text;
        },
      },
    ];
  }

  componentDidMount() {
    this.init();
  }

  // 初始化
  async init() {
    await new Promise(this.getDetail);
    this.setState({ loading: false });
  }

  getDetail(resolve) {
    this.props.dispatch({
      type: `${storeName}/getChangeClassOaRecord`,
      payload: {
        id: this.props.id,
        callback() {
          resolve();
        },
      },
    });
  }

  render() {
    const { detail } = this.props;
    return (
      <Modal
        title="转班审批详情"
        loading={this.state.loading}
        width={1100}
        okBtnShow={false}
        cancelBtnShow={false}
        onCancel={this.props.onCancel}
      >
        <div className={styles.infoWrap}>
          <Row style={{ marginBottom: 10 }}>
            <Col span={6}>
              审批编号：<span className={styles.primaryColor}>{detail.id}</span>
            </Col>
            <Col span={6}>
              OA编号：<span className={styles.primaryColor}>{detail.oaInstanceId}</span>
            </Col>
            <Col span={6}>
              转班类型：<span className={styles.primaryColor}>{detail.changeTypeName}</span>
            </Col>
          </Row>
          <Row>
            <Col span={6}>
              学生名字：<span className={styles.primaryColor}>{detail.studentName}</span>
            </Col>
            <Col span={6}>
              发起人：<span className={styles.primaryColor}>{detail.createUserName}</span>
            </Col>
            <Col span={6}>
              发起时间：<span className={styles.primaryColor}>{detail.createTimeStr}</span>
            </Col>
          </Row>
        </div>
        <div className={styles.transferWrap}>
          <p className={styles.itemTitle}>转出班</p>
          <Row>
            <Col span={8}>
              <p>
                学习中心名称：<span className={styles.primaryColor}>{detail.changeOldClassVo.blCampusName}</span>
              </p>
            </Col>
            <Col span={8}>
              <p>
                产品名称：<span className={styles.primaryColor}>{detail.changeOldClassVo.productName}</span>
              </p>
            </Col>
            <Col span={8}>
              <p>
                班级名称：<span className={styles.primaryColor}>{detail.changeOldClassVo.name}</span>
              </p>
            </Col>
          </Row>
        </div>
        <div className={styles.transferWrap}>
          <p className={styles.itemTitle}>转入班</p>
          <Row>
            <Col span={8}>
              <p>
                学习中心名称：<span className={styles.primaryColor}>{detail.changeNewClassVo.blCampusName}</span>
              </p>
            </Col>
            <Col span={8}>
              <p>
                产品名称：<span className={styles.primaryColor}>{detail.changeNewClassVo.productName}</span>
              </p>
            </Col>
            <Col span={8}>
              <p>
                班级名称：<span className={styles.primaryColor}>{detail.changeNewClassVo.name}</span>
              </p>
            </Col>
          </Row>
        </div>
        <p className={styles.listTitle}>OA审批记录</p>
        <List dataSource={detail.workflowOaRecordVoList} columns={this.columns} />
      </Modal>
    );
  }
}
const mapStateToProps = (state) => {
  return {
    detail: state[storeName].modalRecord,
  };
};
export default connect(mapStateToProps)(ModalChangeClassDetail);
