import React from 'react';
import { connect } from 'dva';
import { Page, List } from 'COMPONENT';
import { storeName } from './store/config';
import ModalChangeClassDetail from './ModalChangeClassDetail';
import Search from './Search';
import { Table } from 'antd';
import TextLink from 'COMPONENT/TextLink';

const { COL_WIDTH_M, COL_WIDTH_XL } = List;

class TwoTeacherChangeClassRecord extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      modalShow: false,
      modalRecordId: null,
      scrollY: 0,
    };

    this.loadData = this.loadData.bind(this);
    this.search = this.search.bind(this);
    this.changePage = this.changePage.bind(this);
    this.reset = this.reset.bind(this);
    this.onShowDetail = this.onShowDetail.bind(this);
    this.abort = this.abort.bind(this);
    this.initSKey = this.initSKey.bind(this);
    this.updateProps = this.updateProps.bind(this);
    this.updateValue = this.updateValue.bind(this);
    this.onCloseModal = this.onCloseModal.bind(this);

    this.columns = [
      { title: 'OA编号', dataIndex: 'oaInstanceId', key: 'oaInstanceId', width: COL_WIDTH_XL },
      { title: '学生姓名', dataIndex: 'studentName', key: 'studentName', width: COL_WIDTH_XL },
      { title: '转班类型', dataIndex: 'changeTypeName', key: 'changeTypeName', width: COL_WIDTH_XL },
      {
        title: '转出班',
        children: [
          { title: '学习中心名称', dataIndex: 'changeOutCampusName', key: 'changeOutCampusName', width: COL_WIDTH_XL },
          { title: '产品名称', dataIndex: 'changeOutProductName', key: 'changeOutProductName', width: COL_WIDTH_XL },
          { title: '班级名称', dataIndex: 'changeOutClassName', key: 'changeOutClassName', width: COL_WIDTH_XL },
        ],
        key: 'changeOldClassVo',
      },
      {
        title: '转入班',
        children: [
          { title: '学习中心名称', dataIndex: 'changeInCampusName', key: 'changeInCampusName', width: COL_WIDTH_XL },
          { title: '产品名称', dataIndex: 'changeInProductName', key: 'changeInProductName', width: COL_WIDTH_XL },
          { title: '班级名称', dataIndex: 'changeInClassName', key: 'changeInClassName', width: COL_WIDTH_XL },
        ],
        key: 'changeNewClassVo',
      },
      { title: '发起时间', dataIndex: 'createTimeStr', key: 'createTimeStr', width: COL_WIDTH_XL },
      { title: '发起人', dataIndex: 'createUserName', key: 'createUserName', width: COL_WIDTH_M },
      {
        title: '当前处理人',
        dataIndex: 'currentApprovalUserName',
        key: 'currentApprovalUserName',
        width: COL_WIDTH_XL,
      },
      { title: '审批状态', dataIndex: 'auditStatusName', key: 'auditStatusName', width: COL_WIDTH_XL },
      { title: '转班状态', dataIndex: 'changeStatusName', key: 'changeStatusName', width: COL_WIDTH_XL },
      { title: '备注', dataIndex: 'remarks', key: 'remarks', width: COL_WIDTH_XL },
      {
        title: '操作',
        key: 'action',
        width: COL_WIDTH_XL,
        fixed: 'right',
        render: (text, record, index) => {
          let config = [
            { text: '审批详情', onClick: () => this.onShowDetail(record) },
            { text: '撤回审批', onClick: () => this.abort(record) },
          ];
          if (
            (record.auditStatus === 1 && record.changeStatus === 1) ||
            this.props.loginUserId !== record.createUserId
          ) {
            config.splice(1, 1);
          }
          return (
            <span>
              {config.map((action, i) => {
                return (
                  <TextLink key={i} className="list-btn" onClick={action.onClick}>
                    {action.text}
                  </TextLink>
                );
              })}
            </span>
          );
        },
      },
    ];
  }

  componentDidMount() {
    this.init();
    this.calculateScrollY();
  }

  componentWillUnmount() {
    this.destroy();
  }

  calculateScrollY = () => {
    const searchDomRect = document.querySelector('.Page-header').getClientRects()[0];
    // 搜索+分页高度
    let y = searchDomRect.top + searchDomRect.height + 200;
    this.setState({
      scrollY: `calc(100vh - ${y}px)`,
    });
  };

  // 初始化
  async init() {
    this.setState({ loading: true });
    await new Promise(this.initSKey);
    await new Promise(this.loadData);
    this.setState({ loading: false });
  }

  loadData(resolve) {
    this.props.dispatch({
      type: `${storeName}/loadData`,
      payload: {
        callback() {
          resolve();
        },
      },
    });
  }

  initSKey(resolve) {
    this.props.dispatch({
      type: `${storeName}/initSKey`,
      payload: {
        callback() {
          resolve();
        },
      },
    });
  }

  async refreshData() {
    this.setState({ loading: true });
    await new Promise(this.loadData);
    this.setState({ loading: false });
  }

  updatePage(pageCurrent, pageSize) {
    let payload = {};
    if (pageSize && pageSize !== this.props.pageSize) {
      payload.pageCurrent = 1;
      payload.pageSize = pageSize;
    } else {
      payload.pageCurrent = pageCurrent;
    }
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload,
    });
  }

  changePage({ current, pageSize }) {
    this.updatePage(current, pageSize);
    this.refreshData();
  }

  search() {
    this.changePage({
      current: 1,
      pageSize: this.props.pageSize,
    });
  }

  reset() {
    this.props.dispatch({
      type: `${storeName}/resetKeyword`,
    });
  }

  destroy() {
    this.props.dispatch({
      type: `${storeName}/resetPage`,
    });
  }

  onShowDetail(row) {
    this.setState({
      modalRecordId: row.id,
      modalShow: true,
    });
  }

  abort(row) {
    this.props.dispatch({
      type: `${storeName}/abortChangeClassOa`,
      payload: {
        oaInstanceId: row.oaInstanceId,
        callback: () => {
          this.refreshData();
        },
      },
    });
  }

  updateProps(params) {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: params,
    });
  }

  updateValue({ type, val }) {
    this.updateProps({ [type]: val });
  }

  onCloseModal() {
    this.setState({
      modalRecordId: null,
      modalShow: false,
    });
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        modalRecord: {
          changeOldClassVo: {},
          changeNewClassVo: {},
          workflowOaRecordVoList: [],
        },
      },
    });
  }

  render() {
    return (
      <Page
        name={this.state.name}
        fitWindowHeight={true}
        loading={this.state.loading}
        route={this.props.route}
        subHeader={
          <div>
            <Search handleKeyword={this.updateValue} handleSearch={this.search} handleReset={this.reset} />
          </div>
        }
        scroll={{
          x: 3100,
          y: this.state.scrollY,
        }}
      >
        <Table
          rowKey="id"
          dataSource={this.props.list}
          columns={this.columns}
          pagination={{
            showQuickJumper: true,
            pageSizeOptions: ['10', '20', '50'],
            showSizeChanger: true,
            showTotal: (total) => `共 ${this.props.total} 条记录`,
            current: this.props.pageCurrent,
            total: this.props.total,
            pageSize: this.props.pageSize,
          }}
          scroll={{
            x: 3100,
            y: this.state.scrollY,
          }}
          onChange={({ current, pageSize }) => this.changePage({ current, pageSize })}
        />
        {this.state.modalShow && <ModalChangeClassDetail id={this.state.modalRecordId} onCancel={this.onCloseModal} />}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    list: state[storeName].list,
    total: state[storeName].listTotal,
    pageSize: state[storeName].pageSize,
    pageCurrent: state[storeName].pageCurrent,
    loginUserId: state.account.user.userId,
  };
};

export default connect(mapStateToProps)(TwoTeacherChangeClassRecord);
