import { storeName, changStatusText } from './config';
import { getChangeClassOaRecordList, abortChangeClassOa, getClassChangeOaRecordByParams } from 'SERVICE/course';
import { clone } from 'lodash';
import { errorsHandling, runFunc } from './utils';
import moment from 'moment';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

const defaultState = {
  sKey: {
    sKey_startTime: null,
    sKey_endTime: null,
    sKey_oaInstanceId: null,
    sKey_studentName: null,
    sKey_changeType: null,
    sKey_auditStatus: null,
  },
  list: {
    // 列表数据
    list: [],
    listTotal: 0,
    pageSize: 20,
    pageCurrent: 1,
    selectedRowKeys: [],
  },
};
Object.freeze(defaultState);

export default {
  namespace: storeName,
  state: {
    // index
    ...clone(defaultState.sKey),
    ...clone(defaultState.list),
    modalRecord: {
      changeOldClassVo: {},
      changeNewClassVo: {},
      workflowOaRecordVoList: [],
    },
  },
  reducers: {
    updateState(state, action) {
      const { payload } = action;
      return {
        ...state,
        ...payload,
      };
    },

    updateList(state, { payload: { list, total } }) {
      const listArr = list.map((item) => {
        return {
          id: item.id,
          oaInstanceId: item.oaInstanceId,
          studentId: item.studentId,
          studentName: item.studentName,
          changeTypeName: item.changeTypeName,
          changeOutCampusName: item.changeOldClassVo.blCampusName,
          changeOutProductName: item.changeOldClassVo.productName,
          changeOutClassName: item.changeOldClassVo.name,
          changeInCampusName: item.changeNewClassVo.blCampusName,
          changeInProductName: item.changeNewClassVo.productName,
          changeInClassName: item.changeNewClassVo.name,
          createTimeStr: item.createTimeStr,
          createUserName: item.createUserName,
          createUserId: item.createUserId,
          currentApprovalUserName: item.currentApprovalUserName,
          auditStatus: item.auditStatus,
          auditStatusName: item.auditStatusName,
          changeStatus: item.changeStatus,
          changeStatusName: changStatusText[item.changeStatus] || '-',
          remarks: item.remarks ? item.remarks : '-',
        };
      });
      return {
        ...state,
        list: listArr,
        listTotal: total,
      };
    },
  },
  effects: {
    // 初始化筛选栏默认值
    *initSKey({ payload: { callback } }, { select, put }) {
      const today = new Date();
      yield put({
        type: 'updateState',
        payload: {
          sKey_startTime: moment(today).add(-1, 'days').format('YYYY-MM-DD'),
          sKey_endTime: moment(today).format('YYYY-MM-DD'),
        },
      });
      runFunc(callback);
    },
    *loadData({ payload: { callback } }, { select, put }) {
      const apiParams = yield select((state) => ({
        startTime: state[storeName].sKey_startTime || null,
        endTime: state[storeName].sKey_endTime || null,
        oaInstanceId: state[storeName].sKey_oaInstanceId || null,
        studentName: state[storeName].sKey_studentName || null,
        changeType: state[storeName].sKey_changeType || null,
        auditStatus: state[storeName].sKey_auditStatus || null,
        page: state[storeName].pageCurrent,
        rows: state[storeName].pageSize,
      }));
      const rsp = yield getChangeClassOaRecordList(apiParams, true).catch(errorsHandling());
      if (rsp && rsp.resultCode === 0) {
        yield put({
          type: 'updateList',
          payload: {
            list: rsp.rows,
            total: rsp.records,
          },
        });
      }
      runFunc(callback);
    },

    *resetKeyword(action, { put, select }) {
      const sKeyStorage = yield select((state) => state[storeName].sKeyStorage);
      yield put({
        type: 'updateState',
        payload: {
          ...clone(defaultState.sKey),
          ...sKeyStorage,
        },
      });
    },

    *resetPage(action, { put }) {
      yield put({
        type: 'updateState',
        payload: {
          ...clone(defaultState.sKey),
          ...clone(defaultState.list),
        },
      });
    },

    // 获取转班OA申请记录
    *getChangeClassOaRecord({ payload: { callback, id } }, { put, select }) {
      const apiParams = {
        id,
        flags: 1,
      };
      const rsp = yield getClassChangeOaRecordByParams(apiParams).catch(errorsHandling());
      if (rsp) {
        yield put({
          type: 'updateState',
          payload: {
            modalRecord: rsp.data,
          },
        });
      }
      runFunc(callback);
    },

    // 撤销转班OA申请
    *abortChangeClassOa({ payload: { callback, oaInstanceId } }, { put, select }) {
      const apiParams = { oaInstanceId };
      const rsp = yield abortChangeClassOa(apiParams).catch(errorsHandling());
      if (rsp && rsp.data.flags) {
        showMessage('撤销申请成功！', 'success');
      }
      runFunc(callback);
    },
  },
};
