/**
 * Created by Haolin on 2018/7/10.
 */
import React from 'react';
import { connect } from 'dva';
import { Row, Col, message } from 'antd';
import Page from 'COMPONENT/Page';
import Block from 'COMPONENT/Block';
import FormItem from 'COMPONENT/FormItem';
import ChooseProduct from '../ChooseProduct';
import MainClass from '../MainClass';
import AddLine from '../AddLine';
import { cloneDeep, debounce } from 'lodash';
import { getTeacherDict, submitAddMainClass } from 'SERVICE/course';
import { loadCommonDict, getDictTextByVal } from 'UTIL/dict';
import ModalAddProduct from '../ModalAddProduct';
import storeName from '../store/constant.js';
import './style.less';

// 产品信息展示
const productText = (val) => (val && val.text && val.text !== '' ? val.text : '自动匹配');
// 产品信息
const ProductInfo = (props) => {
  const {
    data: { version, quarter, gradeSegment, grade, classType, courseCount, courseTime },
    list,
    handleChooseProduct,
  } = props;
  return (
    <Block title="双师课堂产品信息" type="dark">
      <Row>
        <Col span={10}>
          <FormItem name="产品名称" labelWidth={80} required>
            <ChooseProduct list={list} handleClick={handleChooseProduct} />
          </FormItem>
        </Col>
        <Col span={8}>
          <FormItem name="产品年份" labelWidth={80}>
            {productText(version)}
          </FormItem>
        </Col>
        <Col span={6}>
          <FormItem name="季度" labelWidth={80}>
            {productText(quarter)}
          </FormItem>
        </Col>
      </Row>
      <Row>
        <Col span={10}>
          <FormItem name="年级段" labelWidth={80}>
            {productText(gradeSegment)}
          </FormItem>
        </Col>
        <Col span={8}>
          <FormItem name="年级" labelWidth={80}>
            {productText(grade)}
          </FormItem>
        </Col>
        <Col span={6}>
          <FormItem name="班级类型" labelWidth={80}>
            {productText(classType)}
          </FormItem>
        </Col>
      </Row>
      <Row>
        <Col span={10}>
          <FormItem name="总课时数" labelWidth={80}>
            {productText(courseCount)}
          </FormItem>
        </Col>
        <Col span={8}>
          <FormItem name="课时时长" labelWidth={80}>
            {productText(courseTime)}
          </FormItem>
        </Col>
      </Row>
    </Block>
  );
};
// 班级信息
const ClassInfo = (props) => {
  const {
    data,
    list,
    handleUpdate,
    handleUpdateLine,
    toggleCover,
    error,
    checkValidator,
    handleValidator,
    teacherDict,
    handleTeacherDict,
    prdChosenCount,
    lockSubject,
  } = props;
  return (
    <Block title="双师课堂主班信息" type="dark" className="hom-block-content-pd-0">
      <div className="hom-container">
        <MainClass
          data={data}
          list={list}
          prdChosenCount={prdChosenCount}
          toggleCover={toggleCover}
          handleUpdate={handleUpdate}
          handleUpdateLine={handleUpdateLine}
          teacherDict={teacherDict}
          handleTeacherDict={handleTeacherDict}
          error={error}
          checkValidator={checkValidator}
          handleValidator={handleValidator}
          lockSubject={lockSubject}
        />
      </div>
    </Block>
  );
};
// 校验提示
const checkTips = (props) => {
  const { empty, repeat } = props;
  return (
    <div>
      {empty && <p>您有信息未填写，请补充完整</p>}
      {repeat && <p>存在同名主班，请修改</p>}
    </div>
  );
};

class AddMainClass extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false,
      // 产品基础信息
      baseInfo: {
        version: null,
        quarter: null,
        gradeSegment: null,
        grade: null,
        classType: null,
        courseCount: null,
        courseTime: null,
      },
      // {Array} 课堂产品列表
      resultList: [],
      // 是否已选择产品
      prdTrigger: false,
      // 选择了几次产品
      prdTriggerCount: 0,
      // {Array} 主班列表
      mainClassList: [
        {
          // {String} 主班课堂名称 required
          className: null,
          // {String} 班级层次 required
          classLevel: null,
          // {String} 期数/星期数
          classPhase: null,
          // {String} 科目 required
          subject: null,
          // {String} 主讲老师 required
          teacher: null,
          // {String} 课程模板 required
          template: null,
          // {String} 开始日期 required
          beginDate: null,
          // {String} 上课时间 required
          classTime: null,
          // {String} 每讲课时数
          courseTime: 3,
          // {String} 计划招生人数
          recruitTotal: 1000,
          // {String} 直播间
          studio: null,
        },
      ],
      // {Boolean} 是否显示新增行数弹窗
      toggle_AddClassLine: false,
      // {Number} 行数
      addLineIndex: null,
      lineMax: 50,
      // 校验表单校验结果存储
      errors: [{}],
      // 主班列表表单是否有错误项
      checkValidator: true,
      // 字典_主讲老师
      mapTeacher: [[]],
      // 锁定科目变更
      lockSubject: false,
      // 选择产品
      toggle_addProduct: false,
    };
    // 节流-获取老师字典数据
    this.getTeacherDict = debounce(this.loadTeacherDict, 330);
    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.updateProduct = this.updateProduct.bind(this);
    this.buildSubmitParams = this.buildSubmitParams.bind(this);
    this.submit = this.submit.bind(this);
    this.switchModalAddPrd = this.switchModalAddPrd.bind(this);
    this.updateAddProduct = this.updateAddProduct.bind(this);
    this.beforeDestroy = this.beforeDestroy.bind(this);
  }
  componentDidMount() {
    this.init();
  }
  componentWillUnmount() {
    this.beforeDestroy();
  }
  // 初始化
  async init() {
    this.setState({ loading: true });
    await this.initDict();
    this.setState({ loading: false });
  }
  initDict() {
    return new Promise((resolve) => {
      this.props.dispatch({
        type: 'dict/initDict',
        payload: {
          list: ['subject', 'phase', 'weekDay', 'liveRoom'],
          callback() {
            resolve();
          },
        },
      });
    });
  }
  // 更新state
  updateState = (newState) => {
    this.setState((prevState, props) => {
      return {
        ...newState,
      };
    });
  };
  // 更新数据-产品列表及产品信息
  updateProduct({ resultList, productInfo, subjectSet }) {
    let check = resultList && resultList.length > 0;
    let lockSubject = subjectSet.length === 1 && subjectSet[0] !== null;
    let subjectName = lockSubject ? getDictTextByVal(this.props.dictSubject, subjectSet[0]) : '';
    let className = check ? `${productInfo.courseName}${subjectName}` : null;
    let mainClassList = [
      {
        className: className,
        classLevel: null,
        classLevelText: null,
        classPhase: null,
        classPhaseText: null,
        subject: lockSubject ? subjectSet[0] : null,
        subjectText: lockSubject ? subjectName : null,
        teacher: null,
        template: null,
        beginDate: null,
        classTime: null,
        courseTime: 3,
        recruitTotal: 1000,
        studio: null,
      },
    ];
    this.setState((prevState) => ({
      resultList: check ? resultList : [],
      baseInfo: productInfo,
      prdTrigger: check,
      prdTriggerCount: prevState.prdTriggerCount + 1,
      toggle_ChooseProduct: false,
      mainClassList: mainClassList,
      lockSubject,
      errors: check ? cloneDeep(prevState.errors) : [{}],
    }));
  }
  // 更新数据-班级列表
  updateClass = (params) => {
    const { mainClassList, mapTeacher, baseInfo } = this.state;
    const { type, val, index, data } = params;
    let list = mainClassList.slice(0);
    list[index][type] = val;
    switch (type) {
      case 'subject':
        if (list[index].teacher) {
          list[index].teacher = null;
          let dict = cloneDeep(mapTeacher);
          dict[index] = null;
          this.updateState({ mapTeacher: dict });
        }
        list[index][type + 'Text'] = data;
        list[index].className = `${baseInfo.courseName}${data || ''}${list[index].classLevelText || ''}${
          list[index].classPhaseText || ''
        }`;
        break;
      case 'classLevel':
        list[index][type + 'Text'] = data;
        list[index].className = `${baseInfo.courseName}${list[index].subjectText || ''}${data || ''}${
          list[index].classPhaseText || ''
        }`;
        break;
      case 'classPhase':
        list[index][type + 'Text'] = data;
        list[index].className = `${baseInfo.courseName}${list[index].subjectText || ''}${
          list[index].classLevelText || ''
        }${data || ''}`;
        break;
      case 'template':
        if (data) {
          list[index].beginDate = data.beginDate;
          list[index].classTime = data.classTime;
        }
        break;
      case 'courseTime':
        if (list[index].template) {
          list[index].template = null;
          list[index].beginDate = null;
          list[index].classTime = null;
        }
        break;
    }
    this.updateState({ mainClassList: list });
  };
  // 新增/删除-班级列表
  updateClassLine = (params) => {
    const { mainClassList, errors, mapTeacher } = this.state;
    const { type, index } = params;
    if (type === 'add') {
      this.updateState({ addLineIndex: index, toggle_AddClassLine: true });
    } else if (type === 'delete' && mainClassList.length > 1) {
      let list = mainClassList.slice(0);
      let err = cloneDeep(errors);
      let dict = cloneDeep(mapTeacher);
      list.splice(index, 1);
      err.splice(index, 1);
      dict.splice(index, 1);
      this.updateState({ mainClassList: list, errors: err, mapTeacher: dict });
    }
  };
  // 新增行数
  addClassLine = (params) => {
    const { mainClassList, addLineIndex, errors, mapTeacher } = this.state;
    let list = mainClassList.slice(0);
    let err = cloneDeep(errors);
    let dict = cloneDeep(mapTeacher);
    let dictItem = cloneDeep(dict[addLineIndex]);
    for (let i = 0; i < params; i++) {
      list.splice(addLineIndex + 1, 0, { ...list[addLineIndex] });
      err.splice(addLineIndex + 1, 0, { ...err[addLineIndex] });
      dict.splice(addLineIndex + 1, 0, dictItem);
    }
    this.updateState({
      mainClassList: list,
      errors: err,
      mapTeacher: dict,
      toggle_AddClassLine: false,
      addLineIndex: null,
    });
  };
  // 用于校验表单是否存在错误
  errorChange = (params) => {
    const { type, val, index } = params;
    this.setState((prevState) => {
      let err = cloneDeep(prevState.errors);
      err[index][type] = val;
      return { errors: err };
    });
  };
  // 加载主讲老师字典
  loadTeacherDict = async (params) => {
    const { name, gradeId, subjectId, index } = params;
    let apiParams = { name, gradeId, subjectId };
    const apiData = await getTeacherDict(apiParams, true);
    const { resultCode, data } = apiData;
    if (resultCode === 0) {
      const { mapTeacher } = this.state;
      let dictWrap = mapTeacher.slice(0);
      dictWrap[Number(index)] = data.map((item) => ({ val: item.userId, text: item.name }));
      this.updateState({ mapTeacher: dictWrap });
    } else {
      message.error('服务器错误，请联系系统管理员！');
    }
  };
  // 校验主班名称是否相同
  checkClassName = () => {
    const { mainClassList, errors } = this.state;
    let tempObj = {};
    mainClassList.forEach((item, index) => {
      const { className } = item;
      if (!tempObj[className]) {
        tempObj[className] = [];
      }
      tempObj[className].push(index);
    });
    let tempArr = Object.values(tempObj);
    let errorCopy = cloneDeep(errors);
    tempArr.forEach((item) => {
      let len = item.length;
      if (len === 1) {
        errorCopy[item[0]].className = null;
      } else {
        for (let i = 0; i < len; i++) {
          errorCopy[item[i]].className = '主班名称重复';
        }
      }
    });
    this.updateState({ errors: errorCopy });
  };
  // 校验数据
  checkBeforeSubmit = () => {
    const { errors } = this.state;
    let empty = false;
    let repeat = false;
    errors.forEach((item) => {
      let arr = Object.values(item);
      return arr.forEach((item) => {
        if (item) {
          if (item === '主班名称重复') {
            repeat = true;
          } else {
            empty = true;
          }
        }
      });
    });
    let check = !(empty || repeat);
    this.updateState({ checkValidator: check });
    if (empty || repeat) {
      this.pageRef.showOkTips(checkTips({ empty, repeat }));
    }
    return check;
  };
  // 构建批量新增主班参数
  buildSubmitParams() {
    const { baseInfo, resultList, mainClassList } = this.state;
    let branchVos = resultList.map((item) => ({
      brenchId: item.branchId,
      productId: item.productId,
    }));
    return mainClassList.map((item) => {
      // 判断是否春秋季
      let weekPhaseCheck = ['春季', '秋季'].includes(baseInfo.quarter.text);
      return {
        brenchVos: branchVos,
        // classId: null,//新增不传
        classTime: item.classTime,
        classTimeLength: baseInfo.courseTime.val,
        classTypeId: baseInfo.classType.val,
        courseModalId: item.template,
        courseWeek: weekPhaseCheck ? item.classPhase : null,
        everyCourseClassNum: item.courseTime,
        gradeId: baseInfo.grade.val,
        gradeSegmentId: baseInfo.gradeSegment.val,
        liveRoomId: item.studio,
        name: item.className,
        // oper: null,//新增不传
        peopleQuantity: item.recruitTotal || 0,
        phaseId: weekPhaseCheck ? null : item.classPhase,
        productQuarterId: baseInfo.quarter.val,
        productVersionId: baseInfo.version.val,
        startDate: item.beginDate,
        status: 'PENDDING_START',
        subjectId: item.subject,
        teacherId: item.teacher,
        totalClassHours: baseInfo.courseCount.val,
        unitPrice: null,
        classLevel: item.classLevel,
      };
    });
  }
  // 提交
  async submit() {
    this.setState({ loading: true });
    let needOpenPage = false;
    // 校验批量添加的主班里面是否有重复的班级名称
    await this.checkClassName();
    // 校验必填项及是否存在错误
    let checkVal = this.checkBeforeSubmit();
    if (checkVal) {
      let apiParams = this.buildSubmitParams();
      const rsp = await submitAddMainClass(apiParams, false).catch((err) => {});
      if (rsp && rsp.resultCode === 0) {
        let len = apiParams.length;
        message.success(`提交成功，新建 ${len} 个主班！`);
        needOpenPage = true;
      }
    }
    this.setState({ loading: false }, () => {
      if (needOpenPage) {
        this.props.dispatch({
          type: 'main/openPage',
          payload: {
            path: '/twoTeacherClassGroup',
            params: {},
            refresh: true,
          },
        });
      }
    });
  }
  // 切换产品选择弹窗显示
  switchModalAddPrd() {
    this.setState((prevState) => ({
      toggle_addProduct: !prevState.toggle_addProduct,
    }));
  }
  // 添加产品成功回调
  updateAddProduct() {
    // 转换数据格式
    let data = AddMainClass.prdListAdapter(this.props.prdList);
    this.updateProduct(data);
    this.switchModalAddPrd();
  }
  // 已选产品数据设配器(将store里的数据转成当前组件原有的数据格式)
  static prdListAdapter(prdList) {
    let firstPrd = prdList[0];
    let subjectSet = new Set(prdList.map((item) => item.subject.val));
    subjectSet = subjectSet.size > 0 ? [...subjectSet] : [null];
    return {
      resultList: prdList.map((item) => {
        let showPrdName = `${item.branch.text}-${item.product.text}`;
        let showPrdNameFull = `${showPrdName}（总课时数${item.totalHours.text}）`;
        return {
          branchId: item.branch.val,
          branchName: item.branch.text,
          productId: item.product.val,
          productInfo: {
            id: item.product.val,
            name: item.product.text,
            branchId: item.branch.val,
            gradeName: item.grade.text,
            gradeId: item.grade.val,
            totalHours: item.totalHours.val,
            classTypeName: item.classType.text,
            classTypeId: item.classType.val,
            subjectId: item.subject.val,
            yearId: item.year.val,
            seasonName: item.quarter.text,
            seasonId: item.quarter.val,
            courseSeriesName: item.courseSeries.text,
            yearName: item.year.text,
            classTimeLength: item.classTimeLength.val,
            gradeSegmentId: item.gradeSegment.val,
            gradeSegmentName: item.gradeSegment.text,
            price: item.price.val,
          },
          productNameFirst: item.product.text,
          productNameLast: item.classType.text,
          showText: showPrdName,
          showTextFull: showPrdNameFull,
        };
      }),
      productInfo: {
        version: {
          val: firstPrd.year.val,
          text: firstPrd.year.text,
        },
        quarter: firstPrd.quarter,
        gradeSegment: firstPrd.gradeSegment,
        grade: firstPrd.grade,
        classType: firstPrd.classType,
        courseCount: {
          val: firstPrd.totalHours.val,
          text: firstPrd.totalHours.text,
        },
        courseTime: {
          val: firstPrd.classTimeLength.val,
          text: firstPrd.classTimeLength.text,
        },
        courseName: `${firstPrd.year.text}${firstPrd.quarter.text}${firstPrd.grade.text}`,
      },
      subjectSet,
    };
  }
  // 组件注销前销毁数据
  beforeDestroy() {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        addMain_prdList: [],
      },
    });
  }

  render() {
    const {
      loading,
      baseInfo,
      resultList,
      prdTrigger,
      prdTriggerCount,
      mainClassList,
      mapTeacher,
      toggle_AddClassLine,
      checkValidator,
      errors,
      lineMax,
      lockSubject,
      toggle_addProduct,
    } = this.state;
    let rest = lineMax - mainClassList.length;

    return (
      <Page
        className="add-main-class"
        name="新增双师课堂主班"
        loading={loading}
        route={this.props.route}
        footer={true}
        onOk={this.submit}
        ref={(c) => (this.pageRef = c)}
      >
        <ProductInfo data={baseInfo} list={resultList} handleChooseProduct={this.switchModalAddPrd} />
        <ClassInfo
          data={baseInfo}
          list={mainClassList}
          prdChosenCount={prdTriggerCount}
          toggleCover={!prdTrigger}
          handleUpdate={this.updateClass}
          handleUpdateLine={this.updateClassLine}
          teacherDict={mapTeacher}
          handleTeacherDict={this.getTeacherDict}
          error={errors}
          checkValidator={checkValidator}
          handleValidator={this.errorChange}
          lockSubject={lockSubject}
        />
        {toggle_addProduct && (
          <ModalAddProduct type="addMain" onCancel={this.switchModalAddPrd} onConfirm={this.updateAddProduct} />
        )}
        <AddLine
          toggle={toggle_AddClassLine}
          max={rest}
          handleCancel={() => {
            this.updateState({ toggle_AddClassLine: false });
          }}
          handleConfirm={this.addClassLine}
        />
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    dictSubject: state.dict.subject,
    prdList: state[storeName].addMain_prdList,
  };
};

export default connect(mapStateToProps)(AddMainClass);
