/**
 * Created by Haolin on 2018/7/19.
 */
import React from 'react';
import { connect } from 'dva';
import { Row, Col } from 'antd';
import Block from 'COMPONENT/Block';
import FormItem from 'COMPONENT/FormItem';
import Input from 'COMPONENT/Input';
import InputNumber from 'COMPONENT/InputNumber';
import DatePicker from 'COMPONENT/DatePicker';
import TimePicker from 'COMPONENT/TimePicker';
import Select from 'COMPONENT/Select';
import TextArea from 'COMPONENT/TextArea';
import { getDictTextByVal } from 'UTIL/dict';
import './style.less';

class EditClassInfo extends React.Component {
  constructor(props) {
    super(props);
  }
  /**
   * 监听数据输入
   * */
  updateVal = (params) => {
    const { type, val } = params;
    const {
      handleUpdate,
      dict: { mapSubject, mapTeacher },
    } = this.props;
    let paramsData = {
      type,
      val,
      data: null,
    };
    if (val) {
      switch (type) {
        case 'subject':
          paramsData.data = getDictTextByVal(mapSubject, val);
          break;
        case 'courseTime':
        case 'recruitTotal':
          paramsData.val = Math.round(paramsData.val);
          break;
        default:
          paramsData.data = null;
      }
    }
    handleUpdate(paramsData);
  };

  render() {
    const {
      data: {
        className,
        classPhase,
        subject,
        teacher,
        template,
        beginDate,
        classTime,
        courseTime,
        recruitTotal,
        studio,
        classLevel,
        remarks,
      },
      dict: { mapPhase, mapSubject, mapLiveRoom, mapTeacher, mapTemplate, mapClassLevel },
      handleUpdate,
      error,
      checkValidator,
      handleValidator,
      handleSearch,
      isModal,
      courseCount,
    } = this.props;
    let mapTemp = mapTemplate && mapTemplate[courseTime.toString()] ? mapTemplate[courseTime.toString()] : null;

    return (
      <Block title="双师课堂主班信息" type="dark" className="edit-class-info">
        <Row>
          <Col span={12}>
            <FormItem name="双师主班名称" labelWidth={100} required>
              <Input
                placeholder="请输入双师主班名称"
                value={className}
                onChange={(e) => this.updateVal({ type: 'className', val: e.target.value })}
                validator={{
                  rules: ['required'],
                  silent: !Boolean(error['className']) || checkValidator,
                  showStatus: true,
                  showTips: false,
                  onErrorChange: (e) => handleValidator({ type: 'className', val: e }),
                }}
              />
            </FormItem>
          </Col>
          <Col span={12}>
            <FormItem name="期数/星期数" labelWidth={100}>
              <Select
                placeholder="请选择期数/星期数"
                dict={mapPhase}
                value={classPhase}
                onChange={(val) => this.updateVal({ type: 'classPhase', val: val })}
              />
            </FormItem>
          </Col>
        </Row>
        <Row>
          <Col span={12}>
            <FormItem name="科目" labelWidth={100} required>
              <Select
                placeholder="请选择科目"
                dict={mapSubject}
                value={subject}
                onChange={(val) => this.updateVal({ type: 'subject', val: val })}
                validator={{
                  rules: ['required'],
                  silent: !Boolean(error['subject']) || checkValidator,
                  showStatus: true,
                  showTips: false,
                  onErrorChange: (e) => handleValidator({ type: 'subject', val: e }),
                }}
              />
            </FormItem>
          </Col>
          <Col span={12}>
            <FormItem name="主讲老师" labelWidth={100} required>
              <Select
                placeholder="请选择主讲老师"
                searchPlaceholder="请输入老师名称"
                dict={mapTeacher}
                value={teacher}
                onChange={(val) => this.updateVal({ type: 'teacher', val: val })}
                validator={{
                  rules: ['required'],
                  silent: !Boolean(error['teacher']) || checkValidator,
                  showStatus: true,
                  showTips: false,
                  onErrorChange: (e) => handleValidator({ type: 'teacher', val: e }),
                }}
                onSearchKeyChange={(keyword) => {
                  handleSearch({ val: keyword });
                }}
                showSearch
              />
            </FormItem>
          </Col>
        </Row>
        <Row>
          <Col span={12}>
            <FormItem name="课程模板" labelWidth={100}>
              <Select
                placeholder="请选择课程模板"
                dict={mapTemp}
                value={template ? template.toString() : template}
                onChange={(val) => this.updateVal({ type: 'template', val: val })}
                dropdownStyle={{ width: 'auto !important' }}
              />
            </FormItem>
          </Col>
          <Col span={12}>
            <FormItem name="开始日期" labelWidth={100} required>
              <DatePicker
                placeholder="请选择开始日期"
                value={beginDate}
                onChange={(val) => this.updateVal({ type: 'beginDate', val: val })}
                validator={{
                  rules: ['required'],
                  silent: !Boolean(error['beginDate']) || checkValidator,
                  showStatus: true,
                  showTips: false,
                  onErrorChange: (e) => handleValidator({ type: 'beginDate', val: e }),
                }}
              />
            </FormItem>
          </Col>
        </Row>
        <Row>
          <Col span={12}>
            <FormItem name="上课时间" labelWidth={100} required>
              <TimePicker
                placeholder="请选上课时间"
                value={classTime}
                onChange={(val) => this.updateVal({ type: 'classTime', val: val })}
                validator={{
                  rules: ['required'],
                  silent: !Boolean(error['classTime']) || checkValidator,
                  showStatus: true,
                  showTips: false,
                  onErrorChange: (e) => handleValidator({ type: 'classTime', val: e }),
                }}
              />
            </FormItem>
          </Col>
          <Col span={12}>
            <FormItem name="每讲课时数" labelWidth={100} required>
              <InputNumber
                placeholder="请输入每讲课时数"
                value={courseTime}
                max={1000}
                min={0}
                onChange={(val) => this.updateVal({ type: 'courseTime', val: val })}
                validator={{
                  rules: [
                    (val) =>
                      (!val || val < 1 || (isModal === '未排课' && val > courseCount.val)) &&
                      '每讲课时数错误且不可大于总课时数',
                  ],
                  silent: !Boolean(error['courseTime']) || checkValidator,
                  showStatus: true,
                  showTips: true,
                  onErrorChange: (e) => handleValidator({ type: 'courseTime', val: e }),
                }}
              />
            </FormItem>
          </Col>
        </Row>
        <Row>
          <Col span={12}>
            <FormItem name="计划招生人数" labelWidth={100}>
              <InputNumber
                placeholder="请输入计划招生人数"
                value={recruitTotal}
                max={100000}
                min={0}
                onChange={(val) => this.updateVal({ type: 'recruitTotal', val: val })}
              />
            </FormItem>
          </Col>
          <Col span={12}>
            <FormItem name="直播间" labelWidth={100}>
              <Select
                placeholder="请输入直播间"
                dict={mapLiveRoom}
                value={studio}
                onChange={(val) => this.updateVal({ type: 'studio', val: val })}
              />
            </FormItem>
          </Col>
        </Row>
        <Row>
          <Col span={12}>
            <FormItem name="班级层次" labelWidth={100} required>
              <Select
                placeholder="班级层次"
                dict={mapClassLevel}
                value={classLevel}
                onChange={(val) => this.updateVal({ type: 'classLevel', val: val })}
                validator={{
                  rules: ['required'],
                  silent: !Boolean(error['classLevel']) || checkValidator,
                  showStatus: true,
                  showTips: false,
                  onErrorChange: (e) => handleValidator({ type: 'classLevel', val: e }),
                }}
              />
            </FormItem>
          </Col>
        </Row>
        <Row>
          <Col span={24}>
            <FormItem name="课程须知" labelWidth={100}>
              <TextArea
                placeholder="请输入课程须知"
                value={remarks}
                onChange={(e) => this.updateVal({ type: 'remarks', val: e.target.value })}
              />
            </FormItem>
          </Col>
        </Row>
      </Block>
    );
  }
}

const mapStateToProps = (state) => {
  return {};
};

export default connect()(EditClassInfo);
