/**
 * Created by Haolin on 2018/7/18.
 */
import React from 'react';
import { connect } from 'dva';
import { Row, Col, message } from 'antd';
import Modal from 'COMPONENT/Modal';
import Block from 'COMPONENT/Block';
import FormItem from 'COMPONENT/FormItem';
import ChooseProduct from '../ChooseProduct';
import EditClassInfo from './EditClassInfo';
import { loadCommonDict, loadBranchDict, loadCampusDict, getDictTextByVal, getDictValByText } from 'UTIL/dict';
import { getMainClassInfo, getCourseTemplate, getTeacherDict, editAndDeleteMainClass } from 'SERVICE/course';
import { findClassLevelByClassType } from 'SERVICE/common';
import './style.less';
import { cloneDeep, debounce } from 'lodash';
import ModalAddProduct from '../ModalAddProduct';
import storeName from '../store/constant.js';
//
const productText = (val) => (val && val.text && val.text !== '' ? val.text : '自动匹配');
//
const ProductInfo = (props) => {
  const {
    data: { version, quarter, gradeSegment, grade, classType, courseCount, courseTime },
    list,
    handleChooseProduct,
    noChoose,
  } = props;
  return (
    <Block title="双师课堂产品信息" type="dark" className="main-class-product-info">
      <Row>
        <Col span={12}>
          <FormItem name="产品名称" labelWidth={80} required>
            <ChooseProduct noChoose={noChoose} list={list} handleClick={handleChooseProduct} />
          </FormItem>
        </Col>
        <Col span={12}>
          <FormItem name="产品年份" labelWidth={80}>
            {productText(version)}
          </FormItem>
        </Col>
      </Row>
      <Row>
        <Col span={12}>
          <FormItem name="季度" labelWidth={80}>
            {productText(quarter)}
          </FormItem>
        </Col>
        <Col span={12}>
          <FormItem name="年级段" labelWidth={80}>
            {productText(gradeSegment)}
          </FormItem>
        </Col>
      </Row>
      <Row>
        <Col span={12}>
          <FormItem name="年级" labelWidth={80}>
            {productText(grade)}
          </FormItem>
        </Col>
        <Col span={12}>
          <FormItem name="班级类型" labelWidth={80}>
            {productText(classType)}
          </FormItem>
        </Col>
      </Row>
      <Row>
        <Col span={12}>
          <FormItem name="总课时数" labelWidth={80}>
            {productText(courseCount)}
          </FormItem>
        </Col>
        <Col span={12}>
          <FormItem name="课时时长" labelWidth={80}>
            {productText(courseTime)}
          </FormItem>
        </Col>
      </Row>
    </Block>
  );
};

class EditMainClass extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false,
      error: null,
      // 产品基础信息
      baseInfo: {
        version: null,
        quarter: null,
        gradeSegment: null,
        grade: null,
        classType: null,
        courseCount: null,
        courseTime: null,
      },
      // {Array} 课堂产品列表
      resultList: [],
      map: {
        // {Object} 字典_主讲老师
        mapTeacher: null,
        // {Object} 字典_课程模板，每讲课时数未key存储dict
        mapTemplate: null,
        mapClassLevel: null,
      },
      templateStorage: {}, //课程模板对应的数据
      classData: null,
      prdTrigger: false,
      prdTriggerCount: 0, //选择了几次产品
      errors: {},
      checkValidator: true,
      productList: [],
      chosenList: [],
      classInfo: {
        // {String} 主班课堂名称 required
        className: null,
        // {String} 期数/星期数
        classPhase: null,
        // {String} 科目 required
        subject: null,
        // {String} 主讲老师 required
        teacher: null,
        // {String} 课程模板 required
        template: null,
        // {String} 开始日期 required
        beginDate: null,
        // {String} 上课时间 required
        classTime: null,
        // {String} 每讲课时数
        courseTime: null,
        // {String} 计划招生人数
        recruitTotal: null,
        studio: null, // {String} 直播间
        classLevel: null,
        // {String} 课程须知
        remarks: null,
        teacherName: null,
      },
      // 选择产品弹窗显示状态
      toggle_addProduct: false,
    };
    /**
     * 节流-获取老师字典数据
     * */
    this.getTeacherDict = debounce(this.updateTeacherDict, 330);
    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.buildClassData = this.buildClassData.bind(this);
    this.initPrdListInStore = this.initPrdListInStore.bind(this);
    this.updateProduct = this.updateProduct.bind(this);
    this.switchModalAddPrd = this.switchModalAddPrd.bind(this);
    this.updateAddProduct = this.updateAddProduct.bind(this);
    this.beforeDestroy = this.beforeDestroy.bind(this);
  }
  componentDidMount() {
    this.init();
  }
  componentWillUnmount() {
    this.beforeDestroy();
  }
  // 初始化
  async init() {
    this.setState({ loading: true });
    await this.initDict();
    // 初始主班数据
    const classData = await getMainClassInfo({ id: this.props.id }, true);
    await this.buildClassData(classData);
    // 根据classData加载主讲老师及课程模板字典
    const [mapTeacher, mapTemplate] = await Promise.all([
      this.loadTeacherDict({
        teacherName: classData.teacherName,
        gradeId: classData.gradeId,
        subjectId: classData.subjectId,
      }),
      this.loadTemplateDict({
        productVersionId: classData.productVersionId,
        productQuarterId: classData.productQuarterId,
        totalClassHours: classData.totalClassHours,
        everyCourseClassNum: classData.everyCourseClassNum,
      }),
    ]);
    // 初始化班级层次数据字典
    const { data } = await findClassLevelByClassType({ classTypeId: classData.classTypeId }, true);
    const mapClassLevel = data.map((item) => ({
      text: item.NAME,
      val: item.ID,
    }));
    // 在store里初始化已选列表数据
    this.initPrdListInStore(classData);
    // 更新state
    this.setState({
      loading: false,
      map: { mapTeacher, mapTemplate, mapClassLevel },
    });
  }
  initDict() {
    return new Promise((resolve) => {
      this.props.dispatch({
        type: 'dict/initDict',
        payload: {
          list: ['quarter', 'phase', 'weekDay', 'subject', 'liveRoom'],
          callback() {
            resolve();
          },
        },
      });
    });
  }
  // 在store里初始化已选列表数据
  initPrdListInStore(classData) {
    let prdList = classData.brenchVos.map((item) => ({
      product: {
        text: item.productName,
        val: item.productId,
      },
      branch: {
        text: item.brenchName,
        val: item.brenchId,
      },
      year: {
        text: classData.productVersionName,
        val: classData.productVersionId,
      },
      quarter: {
        text: classData.productQuarterName,
        val: classData.productQuarterId,
      },
      character: {
        text: null,
        val: item.productCharacter,
      },
      classType: {
        text: classData.classTypeName,
        val: classData.classTypeId,
      },
      courseSeries: {
        text: item.courseSeries ? item.courseSeries.name : null,
        val: item.courseSeries ? item.courseSeries.id : null,
      },
      grade: {
        text: classData.gradeName,
        val: classData.gradeId,
      },
      subject: {
        text: classData.subjectName,
        val: classData.subjectId,
      },
      totalHours: {
        text: classData.totalClassHours,
        val: classData.totalClassHours,
      },
      classTimeLength: {
        text: classData.classTimeLength,
        val: classData.classTimeLength,
      },
      price: {
        text: item.price,
        val: item.price,
      },
      gradeSegment: {
        val: classData.gradeSegmentId,
        text: classData.gradeSegmentName,
      },
      viceClassNum: item.classNum,
    }));
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        editMain_originPrdList: prdList,
        editMain_prdList: prdList,
      },
    });
  }
  // 构建主班数据
  buildClassData(classData) {
    return new Promise((resolve) => {
      let phase = ['春季', '秋季'].includes(getDictTextByVal(this.props.dictQuarter, classData.productQuarterId))
        ? classData.courseWeekId
        : classData.phaseId;
      let baseInfo = {
        version: {
          val: classData.productVersionId,
          text: classData.productVersionName,
        },
        quarter: {
          val: classData.productQuarterId,
          text: classData.productQuarterName,
        },
        gradeSegment: {
          val: classData.gradeSegmentId,
          text: classData.gradeSegmentName,
        },
        grade: {
          val: classData.gradeId,
          text: classData.gradeName,
        },
        classType: {
          val: classData.classTypeId,
          text: classData.classTypeName,
        },
        courseCount: {
          val: classData.totalClassHours,
          text: classData.totalClassHours,
        },
        courseTime: {
          val: classData.classTimeLength,
          text: classData.classTimeLength,
        },
      };
      let resultList = classData.brenchVos.map((item) => {
        let showText = `${item.brenchName}-${item.productName}${classData.classTypeName}`;
        return {
          branchId: item.brenchId,
          branchName: item.brenchName,
          productId: item.productId,
          productInfo: null,
          productNameFirst: item.productName,
          productNameLast: classData.classTypeName,
          showText: showText,
          showTextFull: `${showText}（总课时数${classData.totalClassHours}）`,
          deleteBtn: item.classNum > 0,
        };
      });
      this.setState(
        {
          classData: classData,
          classInfo: {
            className: classData.name,
            classPhase: phase,
            subject: classData.subjectId,
            teacher: classData.teacherId,
            template: classData.courseModalId,
            beginDate: classData.startDate,
            classTime: classData.classTime,
            courseTime: classData.everyCourseClassNum,
            recruitTotal: classData.peopleQuantity,
            studio: classData.liveRoomId,
            classLevel: classData.classLevel,
            remarks: classData.remark,
          },
          baseInfo: baseInfo,
          resultList: resultList,
          prdTrigger: true,
        },
        () => {
          resolve();
        },
      );
    });
  }
  // 获取主讲老师字典
  loadTeacherDict = async (params) => {
    const { teacherName, gradeId, subjectId, type } = params;
    let apiData = {
      name: teacherName,
      gradeId,
      subjectId,
    };
    const { resultCode, data } = await getTeacherDict(apiData, true);
    let list = resultCode === 0 ? data : [];
    let result = list.map((item) => ({ val: item.userId, text: item.name }));
    if (type === 'update') {
      const { map } = this.state;
      let dict = cloneDeep(map);
      dict.mapTeacher = result;
      this.updateState({ map: dict });
    } else {
      return result;
    }
  };
  // 更新主讲老师字典数据
  updateTeacherDict = (params) => {
    const { val } = params;
    const {
      baseInfo: { grade },
      classInfo: { subject },
    } = this.state;
    this.loadTeacherDict({ teacherName: val, gradeId: grade.val, subjectId: subject, type: 'update' });
  };
  // 获取课程模板字典
  loadTemplateDict = async (params) => {
    let mapTemp = null;
    if (params.totalClassHours && params.everyCourseClassNum) {
      const { templateStorage, map } = this.state;
      let storage = cloneDeep(templateStorage);
      mapTemp = cloneDeep(map.mapTemplate);
      if (!mapTemp) {
        mapTemp = {};
      }
      let apiData = {
        productSeasonId: params.productQuarterId,
        productYearId: params.productVersionId,
        techNum: (params.totalClassHours / params.everyCourseClassNum).toFixed(0),
      };
      let list = await getCourseTemplate(apiData, true);
      let len = list.length;
      for (let i = 0; i < len; i++) {
        storage[list[i].id] = {
          beginDate: list[i].courseDate[0],
          classTime: list[i].classTime,
        };
      }
      this.updateState({ templateStorage: storage });
      mapTemp[params.everyCourseClassNum.toString()] = list.map((item) => ({
        val: item.id.toString(),
        text: item.modalName,
      }));
    }
    return mapTemp;
  };
  // 更新课程模板字典数据
  updateTemplateDict = (params) => {
    const { val } = params;
    const {
      baseInfo: { version, quarter, courseCount },
      map,
    } = this.state;
    Promise.all([
      this.loadTemplateDict({
        productVersionId: version.val,
        productQuarterId: quarter.val,
        totalClassHours: courseCount.val,
        everyCourseClassNum: val,
      }),
    ]).then(([templateDict]) => {
      this.updateState({ map: { ...map, mapTemplate: templateDict } });
    });
  };
  // 更新state
  updateState = (newState) => {
    this.setState((prevState, props) => {
      return {
        ...newState,
      };
    });
  };
  // 更新数据-产品列表及产品信息
  async updateProduct({ resultList, productInfo, subjectSet }) {
    this.setState({ loading: true });
    let check = resultList && resultList.length > 0;
    // 待更新的state
    let payload = {
      loading: false,
      resultList: check ? resultList : [],
      baseInfo: productInfo,
      prdTrigger: check,
      prdTriggerCount: this.state.prdTriggerCount + 1,
      toggle_ChooseProduct: false,
      map: this.state.map,
      // errors: check ? cloneDeep(this.state.errors) : [{}],
    };
    // 是否需要重置主班信息
    if (this.props.isNeedResetMainClass) {
      // 默认每讲课时数/报名人数限制
      const defaultCourseTime = 3;
      const defaultRecruitTotal = 1000;
      // 重置主班信息及老师字典
      payload.classInfo = {
        className: check ? resultList[0].productNameFirst + resultList[0].productNameLast : null,
        classPhase: null,
        subject: null,
        teacher: null,
        template: null,
        beginDate: null,
        classTime: null,
        courseTime: defaultCourseTime,
        recruitTotal: defaultRecruitTotal,
        studio: null,
        remarks: null,
      };
      payload.map.mapTeacher = null;
      // 更新课程模板字典/班级层次字典
      const [mapTemplate, classLevelRsp] = await Promise.all([
        this.loadTemplateDict({
          productVersionId: productInfo.version.val,
          productQuarterId: productInfo.quarter.val,
          totalClassHours: productInfo.courseCount.val,
          everyCourseClassNum: defaultCourseTime,
        }),
        findClassLevelByClassType({ classTypeId: productInfo.classType.val }, true),
      ]);
      payload.map.mapTemplate = mapTemplate || { [defaultCourseTime.toString()]: null };
      payload.map.mapClassLevel =
        classLevelRsp && classLevelRsp.data
          ? classLevelRsp.data.map((item) => ({
              text: item.NAME,
              val: item.ID,
            }))
          : null;
    }
    this.setState(payload);
  }
  // 更新数据-班级列表
  updateClass = (params) => {
    const { classInfo, resultList, templateStorage } = this.state;
    const { type, val, data } = params;
    let info = cloneDeep(classInfo);
    info[type] = val;
    switch (type) {
      case 'subject':
        if (info.teacher) {
          info.teacher = null;
          const { map } = this.state;
          let dict = cloneDeep(map);
          dict.mapTeacher = null;
          this.updateState({ map: dict });
        }
        let subData = resultList[0];
        let prdNameF = subData.productNameFirst;
        let prdNameL = subData.productNameLast;
        info.className = data ? prdNameF + data + prdNameL : prdNameF + prdNameL;
        break;
      case 'template':
        if (val) {
          info.beginDate = templateStorage[val].beginDate;
          info.classTime = templateStorage[val].classTime;
        }
        break;
      case 'courseTime':
        if (info.template) {
          info.template = null;
          info.beginDate = null;
          info.classTime = null;
        }
        //请求新的课程模板数据
        this.updateTemplateDict({ val });
        break;
    }
    this.updateState({ classInfo: info });
  };
  // 用于校验表单是否存在错误
  errorChange = (params) => {
    const { type, val } = params;
    this.setState((prevState) => {
      let err = cloneDeep(prevState.errors);
      err[type] = val;
      return { errors: err };
    });
  };
  // 校验数据
  checkBeforeSubmit = () => {
    const { errors } = this.state;
    let check = Object.values(errors).every((item) => {
      return !Boolean(item);
    });
    this.updateState({ checkValidator: check });
    return check;
  };
  // post主班数据
  postSubmit = async (params) => {
    this.updateState({ loading: true });
    const { handleConfirm } = this.props;
    editAndDeleteMainClass(params, true).then(
      (data) => {
        message.success('修改主班成功！');
        this.updateState({ loading: false });
        handleConfirm();
      },
      (err) => {
        message.error(err.text);
        this.updateState({ loading: false });
      },
    );
  };
  // 提交
  submit = () => {
    let checkVal = this.checkBeforeSubmit();
    if (checkVal) {
      const {
        baseInfo,
        baseInfo: { version, quarter, grade, gradeSegment, classType, courseCount },
        resultList,
        classInfo,
        classData,
        classInfo: {
          className,
          classPhase,
          subject,
          teacher,
          template,
          beginDate,
          classTime,
          courseTime,
          recruitTotal,
          studio,
          classLevel,
          remarks,
        },
      } = this.state;
      let branchVos = resultList.map((item) => ({ brenchId: item.branchId, productId: item.productId }));
      //根据季度选取期数或者星期数
      let weekPhaseCheck = ['春季', '秋季'].includes(quarter.text);
      let mapWeekPhase = weekPhaseCheck ? classPhase : null;
      let mapPhase = weekPhaseCheck ? null : classPhase;
      let apiParams = {
        brenchVos: branchVos,
        classId: classData.classId,
        classTime: classTime,
        classTimeLength: baseInfo.courseTime.val,
        classTypeId: classType.val,
        courseModalId: template,
        courseWeek: mapWeekPhase,
        everyCourseClassNum: courseTime,
        gradeId: grade.val,
        gradeSegmentId: gradeSegment.val,
        liveRoomId: studio,
        name: className,
        peopleQuantity: recruitTotal || 0,
        phaseId: mapPhase,
        productQuarterId: quarter.val,
        productVersionId: version.val,
        startDate: beginDate,
        status: classData.status,
        subjectId: subject,
        teacherId: teacher,
        totalClassHours: courseCount.val,
        remark: remarks,
        unitPrice: null,
        classLevel,
      };
      this.postSubmit(apiParams);
    }
  };
  // 切换产品选择弹窗显示
  switchModalAddPrd() {
    this.setState((prevState) => ({
      toggle_addProduct: !prevState.toggle_addProduct,
    }));
  }
  // 添加产品成功回调
  updateAddProduct() {
    // 转换数据格式
    let data = EditMainClass.prdListAdapter(this.props.prdList);
    this.updateProduct(data);
    this.switchModalAddPrd();
  }
  // 已选产品数据设配器(将store里的数据转成当前组件原有的数据格式)
  static prdListAdapter(prdList) {
    let firstPrd = prdList[0];
    return {
      resultList: prdList.map((item) => {
        let showPrdName = `${item.branch.text}-${item.product.text}${item.classType.text}`;
        let showPrdNameFull = `${showPrdName}（总课时数${item.totalHours.text}）`;
        return {
          branchId: item.branch.val,
          branchName: item.branch.text,
          productId: item.product.val,
          productInfo: {
            id: item.product.val,
            name: item.product.text,
            branchId: item.branch.val,
            gradeName: item.grade.text,
            gradeId: item.grade.val,
            totalHours: item.totalHours.val,
            classTypeName: item.classType.text,
            classTypeId: item.classType.val,
            subjectId: item.subject.val,
            yearId: item.year.val,
            seasonName: item.quarter.text,
            seasonId: item.quarter.val,
            courseSeriesName: item.courseSeries.text,
            yearName: item.year.text,
            classTimeLength: item.classTimeLength.val,
            gradeSegmentId: item.gradeSegment.val,
            gradeSegmentName: item.gradeSegment.text,
            price: item.price.val,
          },
          productNameFirst: item.product.text,
          productNameLast: item.classType.text,
          showText: showPrdName,
          showTextFull: showPrdNameFull,
        };
      }),
      productInfo: {
        version: {
          val: firstPrd.year.val,
          text: firstPrd.year.text,
        },
        quarter: firstPrd.quarter,
        gradeSegment: firstPrd.gradeSegment,
        grade: firstPrd.grade,
        classType: firstPrd.classType,
        courseCount: {
          val: firstPrd.totalHours.val,
          text: firstPrd.totalHours.text,
        },
        courseTime: {
          val: firstPrd.classTimeLength.val,
          text: firstPrd.classTimeLength.text,
        },
        courseName: `${firstPrd.year.text}${firstPrd.quarter.text}${firstPrd.grade.text}`,
      },
      subjectSet: null,
    };
  }
  // 组件注销前销毁数据
  beforeDestroy() {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        editMain_originPrdList: [],
        editMain_prdList: [],
        isNeedResetMainClass: false,
      },
    });
  }

  render() {
    const { handleCancel } = this.props;
    const {
      loading,
      error,
      classInfo,
      baseInfo,
      baseInfo: { quarter, courseCount },
      resultList,
      map: { mapTeacher, mapTemplate, mapClassLevel },
      checkValidator,
      errors,
      toggle_addProduct,
    } = this.state;
    let dict = {
      mapPhase: quarter && ['春季', '秋季'].includes(quarter.text) ? this.props.dictWeekDay : this.props.dictPhase,
      mapSubject: this.props.dictSubject,
      mapLiveRoom: this.props.dictLiveRoom,
      mapTeacher,
      mapTemplate,
      mapClassLevel,
    };
    let noChoosePrd = false;

    return (
      <Modal
        title="修改双师课堂主班"
        className="edit-main-class"
        loading={loading}
        error={error}
        width={1100}
        okText="保存"
        onOk={this.submit}
        onCancel={handleCancel}
        ref={(c) => (this.pageRef = c)}
      >
        <ProductInfo
          data={baseInfo}
          list={resultList}
          noChoose={noChoosePrd}
          handleChooseProduct={this.switchModalAddPrd}
        />
        <EditClassInfo
          data={classInfo}
          dict={dict}
          handleUpdate={this.updateClass}
          error={errors}
          checkValidator={checkValidator}
          handleValidator={this.errorChange}
          handleSearch={this.getTeacherDict}
          courseCount={courseCount}
          isModal={this.props.isModal}
        />
        {toggle_addProduct && (
          <ModalAddProduct type="editMain" onCancel={this.switchModalAddPrd} onConfirm={this.updateAddProduct} />
        )}
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    prdList: state[storeName].editMain_prdList,
    isNeedResetMainClass: state[storeName].isNeedResetMainClass,
    dictProductCharacter: state.dict.productCharacter,
    dictQuarter: state.dict.quarter,
    dictPhase: state.dict.phase,
    dictWeekDay: state.dict.weekDay,
    dictSubject: state.dict.subject,
    dictLiveRoom: state.dict.liveRoom,
  };
};

export default connect(mapStateToProps)(EditMainClass);
