/**
 * Created by Haolin on 2018/7/16.
 */
import React from 'react';
import { connect } from 'dva';
import { Row, Col, Spin } from 'antd';
import Input from 'COMPONENT/Input';
import InputNumber from 'COMPONENT/InputNumber';
import DatePicker from 'COMPONENT/DatePicker';
import TimePicker from 'COMPONENT/TimePicker';
import Select from 'COMPONENT/Select';
import { loadCommonDict, loadBranchDict, loadCampusDict, getDictTextByVal } from 'UTIL/dict';
import { getCourseTemplate, getTeacherDict } from 'SERVICE/course';
import { findClassLevelByClassType } from 'SERVICE/common';
import style from './style.less';
import './style.less';

// 遮罩层
const Cover = (props) => {
  const { visible, loading, handleClick } = props;
  let style = visible || loading ? { display: 'block' } : { display: 'none' };
  return (
    <div
      className="hom-Cover"
      style={style}
      onClick={() => {
        if (handleClick) handleClick();
      }}
    >
      {loading && <Spin />}
    </div>
  );
};
// 容器
const Wrapper = (props) => {
  const { children, className, coverVisible, coverLoading, style, coverClick } = props;
  let classNameStr = className ? 'hom-wrapper ' + className : 'hom-wrapper';
  let styleObj = style ? style : {};
  return (
    <div className={classNameStr} style={styleObj}>
      {children}
      <Cover visible={coverVisible} loading={coverLoading} handleClick={coverClick} />
    </div>
  );
};
// table head block
const ThBlock = (props) => {
  const { text, required } = props;
  return (
    <div className={style['hom-th-block']}>
      <span>{text}</span>
      {required && <span className="hom-required"> </span>}
    </div>
  );
};
// table data block
const TdBlock = (props) => {
  const { children } = props;
  return <div className={style['hom-main-class-td']}>{children}</div>;
};
// 主班行
const ClassBlock = (props) => {
  const {
    dict: { mapPhase, mapSubject, mapTeacher, mapTemplate, mapLiveRoom, mapClassLevel },
    data: {
      className,
      classLevel,
      classPhase,
      subject,
      teacher,
      template,
      beginDate,
      classTime,
      courseTime,
      recruitTotal,
      studio,
    },
    index,
    toggleDelete,
    handleUpdate,
    handleUpdateLine,
    error,
    checkValidator,
    handleValidator,
    handleSearch,
    lockSubject,
  } = props;
  let tempNum = courseTime || '';
  tempNum = typeof tempNum === 'string' ? tempNum : tempNum.toString();
  let teacherDict = mapTeacher[index] ? mapTeacher[index] : null;
  return (
    <Row type="flex" justify="center" align="middle" className={style['hom-main-class-row']}>
      <Col span={2}>
        <TdBlock>
          <Input
            placeholder="双师主班名称"
            value={className}
            onChange={(e) => handleUpdate({ type: 'className', val: e.target.value, index: index })}
            validator={{
              rules: ['required'],
              silent: !Boolean(error[index]['className']) || checkValidator,
              showStatus: true,
              showTips: false,
              onErrorChange: (e) => handleValidator({ type: 'className', val: e, index: index }),
            }}
            size="small"
          />
        </TdBlock>
      </Col>
      <Col span={2}>
        <TdBlock>
          <Select
            placeholder="班级层次"
            dict={mapClassLevel}
            value={classLevel}
            onChange={(val) => handleUpdate({ type: 'classLevel', val: val, index: index })}
            validator={{
              rules: ['required'],
              silent: !Boolean(error[index]['classLevel']) || checkValidator,
              showStatus: true,
              showTips: false,
              onErrorChange: (e) => handleValidator({ type: 'classLevel', val: e, index: index }),
            }}
            size="small"
          />
        </TdBlock>
      </Col>
      <Col span={2}>
        <TdBlock>
          <Select
            placeholder="期数/星期数"
            dict={mapPhase}
            value={classPhase}
            onChange={(val) => handleUpdate({ type: 'classPhase', val: val, index: index })}
            size="small"
          />
        </TdBlock>
      </Col>
      <Col span={2}>
        <TdBlock>
          <Select
            placeholder="科目"
            dict={mapSubject}
            value={subject}
            onChange={(val) => handleUpdate({ type: 'subject', val: val, index: index })}
            validator={{
              rules: ['required'],
              silent: !Boolean(error[index]['subject']) || checkValidator,
              showStatus: true,
              showTips: false,
              onErrorChange: (e) => handleValidator({ type: 'subject', val: e, index: index }),
            }}
            size="small"
            disabled={lockSubject}
          />
        </TdBlock>
      </Col>
      <Col span={2}>
        <TdBlock>
          <Select
            placeholder="主讲老师"
            searchPlaceholder="请输入老师名称"
            dict={teacherDict}
            value={teacher}
            onChange={(val) => handleUpdate({ type: 'teacher', val: val, index: index })}
            validator={{
              rules: ['required'],
              silent: !Boolean(error[index]['teacher']) || checkValidator,
              showStatus: true,
              showTips: false,
              onErrorChange: (e) => handleValidator({ type: 'teacher', val: e, index: index }),
            }}
            size="small"
            onSearchKeyChange={(keyword) => {
              handleSearch({ val: keyword, index: index });
            }}
            showSearch
          />
        </TdBlock>
      </Col>
      <Col span={2}>
        <TdBlock>
          <Select
            placeholder="课程模板"
            dropdownStyle={{ width: 'auto !important' }}
            dict={mapTemplate[tempNum]}
            value={template}
            onChange={(val) => handleUpdate({ type: 'template', val: val, index: index })}
            size="small"
          />
        </TdBlock>
      </Col>
      <Col span={2}>
        <TdBlock>
          <DatePicker
            placeholder="开始日期"
            value={beginDate}
            onChange={(val) => handleUpdate({ type: 'beginDate', val: val, index: index })}
            validator={{
              rules: ['required'],
              silent: !Boolean(error[index]['beginDate']) || checkValidator,
              showStatus: true,
              showTips: false,
              onErrorChange: (e) => handleValidator({ type: 'beginDate', val: e, index: index }),
            }}
            size="small"
          />
        </TdBlock>
      </Col>
      <Col span={2}>
        <TdBlock>
          <TimePicker
            placeholder="上课时间"
            value={classTime}
            onChange={(val) => handleUpdate({ type: 'classTime', val: val, index: index })}
            validator={{
              rules: ['required'],
              silent: !Boolean(error[index]['classTime']) || checkValidator,
              showStatus: true,
              showTips: false,
              onErrorChange: (e) => handleValidator({ type: 'classTime', val: e, index: index }),
            }}
            size="small"
          />
        </TdBlock>
      </Col>
      <Col span={2}>
        <TdBlock>
          <InputNumber
            placeholder="每讲课时数"
            value={courseTime}
            max={1000}
            min={0}
            onChange={(e) => handleUpdate({ type: 'courseTime', val: e, index: index })}
            validator={{
              rules: [(val) => (!val || val < 1) && '每讲课时数参数错误'],
              silent: !Boolean(error[index]['courseTime']) || checkValidator,
              showStatus: true,
              showTips: false,
              onErrorChange: (e) => handleValidator({ type: 'courseTime', val: e, index: index }),
            }}
            size="small"
          />
        </TdBlock>
      </Col>
      <Col span={2}>
        <TdBlock>
          <InputNumber
            placeholder="计划招生人数"
            value={recruitTotal}
            max={100000}
            min={0}
            onChange={(e) => handleUpdate({ type: 'recruitTotal', val: e, index: index })}
            size="small"
          />
        </TdBlock>
      </Col>
      <Col span={2}>
        <TdBlock>
          <Select
            placeholder="直播间"
            dict={mapLiveRoom}
            value={studio}
            onChange={(val) => handleUpdate({ type: 'studio', val: val, index: index })}
            size="small"
            showSearch
          />
        </TdBlock>
      </Col>
      <Col span={2}>
        <div className={style['hom-link-btn-row']}>
          <span className="hom-link-btn-info" onClick={(e) => handleUpdateLine({ type: 'add', index: index })}>
            新增
          </span>
          {toggleDelete && (
            <span className="hom-link-btn-danger" onClick={(e) => handleUpdateLine({ type: 'delete', index: index })}>
              删除
            </span>
          )}
        </div>
      </Col>
    </Row>
  );
};

class MainClass extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false,
      error: null,
      mapTeacher: {},
      // 字典_课程模板，每讲课时数未key存储dict
      mapTemplate: {},
      mapClassLevel: null,
      //课程模板对应的数据
      templateStorage: {},
      isInitCourseTemplate: false,
      initTemplateCount: 0,
    };
    this.updateVal = this.updateVal.bind(this);
  }
  componentWillReceiveProps(nextProps) {
    this.initCourseTemplate(nextProps);
    this.initClassLevel(nextProps);
  }
  // 当有产品选中后更新 课程模板 选项
  initCourseTemplate = (nextProps) => {
    const { data, prdChosenCount } = nextProps;
    const { initTemplateCount } = this.state;
    if (prdChosenCount > initTemplateCount) {
      this.updateState({ initTemplateCount: initTemplateCount + 1 });
      this.loadCourseTemplate({ type: 'init', val: 3, index: 0, initData: { data } });
    }
  };
  // 当有产品选中后更新 班级层次 选项
  initClassLevel = (nextProps) => {
    if (nextProps.data.classType && nextProps.data.classType.val) {
      findClassLevelByClassType({ classTypeId: nextProps.data.classType.val }, true).then((rsp) => {
        const dict = rsp.data.map((item) => ({
          text: item.NAME,
          val: item.ID,
        }));
        this.updateState({ mapClassLevel: dict });
      });
    }
  };
  /**
   * 加载课程模板
   * params:{type:"init"/"change",val,index}
   * */
  loadCourseTemplate = async (params) => {
    this.setState({ loading: true });
    const { type, val, index, initData } = params;
    const {
      data: { version, quarter, courseCount },
    } = type === 'init' && initData ? initData : this.props;
    const { templateStorage } = this.state;
    //总讲数
    let techNum = (courseCount.val / Number(val)).toFixed(0);
    let apiParams = {
      branchId: null,
      productSeasonId: quarter.val,
      productYearId: version.val,
      techNum: techNum,
    };
    let mapTemplate = {};
    let tempStorage = { ...templateStorage };
    const apiData = await getCourseTemplate(apiParams, true);
    let apiLen = apiData.length;
    mapTemplate[val.toString()] = apiData.map((item, index) => ({ val: item.id.toString(), text: item.modalName }));
    for (let i = 0; i < apiLen; i++) {
      tempStorage[apiData[i].id] = { beginDate: apiData[i].courseDate[0], classTime: apiData[i].classTime };
    }
    this.updateState({ loading: false, mapTemplate, templateStorage: tempStorage });
  };
  // 搜索主讲老师
  searchTeacher = (params) => {
    const { handleTeacherDict } = this.props;
    const { val, index } = params;
    const {
      data: { grade },
      list,
    } = this.props;
    let data = {
      name: val,
      gradeId: grade.val,
      subjectId: list[index].subject,
      campusId: null,
      index,
    };
    handleTeacherDict(data);
  };
  // 更新state
  updateState = (newState) => {
    this.setState((prevState, props) => {
      return {
        ...newState,
      };
    });
  };
  // 监听数据输入
  updateVal({ type, val, index }) {
    let paramsData = {
      type,
      val,
      index,
      data: null,
    };
    if (val) {
      switch (type) {
        case 'subject':
          paramsData.data = getDictTextByVal(this.props.dictSubject, val);
          break;
        case 'classLevel':
          paramsData.data = getDictTextByVal(this.state.mapClassLevel, val);
          break;
        case 'classPhase':
          let phase = getDictTextByVal(this.props.dictPhase, val);
          paramsData.data = phase || getDictTextByVal(this.props.dictWeekDay, val);
          break;
        case 'template':
          paramsData.data = this.state.templateStorage[val];
          break;
        case 'courseTime':
          this.loadCourseTemplate({ type: 'change', val, index });
          paramsData.val = Math.round(paramsData.val);
          break;
        case 'recruitTotal':
          paramsData.val = Math.round(paramsData.val);
          break;
        default:
          paramsData.data = null;
      }
    }
    this.props.handleUpdate(paramsData);
  }
  // 未选中产品时，不可操作提示
  notChooseModal = () => {};

  render() {
    const {
      data: { quarter },
      list,
      toggleCover,
      handleUpdateLine,
      error,
      checkValidator,
      handleValidator,
      teacherDict,
      lockSubject,
      dictWeekDay,
      dictPhase,
      dictSubject,
      dictLiveRoom,
    } = this.props;
    const { loading, mapTemplate, mapClassLevel } = this.state;
    let moreThanOne = list.length > 1;
    let map = {
      mapPhase: quarter && ['春季', '秋季'].includes(quarter.text) ? dictWeekDay : dictPhase,
      mapSubject: dictSubject,
      mapTeacher: teacherDict,
      mapLiveRoom: dictLiveRoom,
      mapTemplate,
      mapClassLevel,
    };

    return (
      <Wrapper
        className="hom-main-class-wrapper hom-scroll-bar"
        coverVisible={toggleCover}
        coverLoadding={loading}
        coverClick={this.notChooseModal}
      >
        <Row type="flex" justify="center" align="middle" className={style['hom-main-class-header']}>
          <Col span={2}>
            <ThBlock text="双师主班名称" required />
          </Col>
          <Col span={2}>
            <ThBlock text="班级层次" required />
          </Col>
          <Col span={2}>
            <ThBlock text="期数/星期数" />
          </Col>
          <Col span={2}>
            <ThBlock text="科目" required />
          </Col>
          <Col span={2}>
            <ThBlock text="主讲老师" required />
          </Col>
          <Col span={2}>
            <ThBlock text="课程模板" />
          </Col>
          <Col span={2}>
            <ThBlock text="开始日期" required />
          </Col>
          <Col span={2}>
            <ThBlock text="上课时间" required />
          </Col>
          <Col span={2}>
            <ThBlock text="每讲课时数" required />
          </Col>
          <Col span={2}>
            <ThBlock text="计划招生人数" />
          </Col>
          <Col span={2}>
            <ThBlock text="直播间" />
          </Col>
          <Col span={2}>
            <ThBlock text="操作" />
          </Col>
        </Row>
        {list.map((item, index) => (
          <ClassBlock
            dict={map}
            data={item}
            index={index}
            lockSubject={lockSubject}
            toggleDelete={moreThanOne}
            handleUpdate={this.updateVal}
            handleUpdateLine={handleUpdateLine}
            handleSearch={this.searchTeacher}
            error={error}
            checkValidator={checkValidator}
            handleValidator={handleValidator}
            key={index}
          />
        ))}
      </Wrapper>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    dictPhase: state.dict.phase,
    dictWeekDay: state.dict.weekDay,
    dictSubject: state.dict.subject,
    dictLiveRoom: state.dict.liveRoom,
    dictQuarter: state.dict.quarter,
  };
};

export default connect(mapStateToProps)(MainClass);
