/**
 * Created by Haolin on 2019/3/25.
 */
import React from 'react';
import { connect } from 'dva';
import style from './index.less';
import List from 'COMPONENT/List';
import { getCourseList } from 'SERVICE/course';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import Wrapper from 'COMPONENT/Wrapper';

const { COL_WIDTH_M, COL_WIDTH_L } = List;

class CourseList extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      name: '课程列表',
      loading: false,
      list: [],
      total: 0,
      current: 1,
      pageSize: 10,
    };
    this.columns = [
      { title: '课程序号', dataIndex: 'courseNum', key: 'courseNum', width: COL_WIDTH_M },
      { title: '课程名称', dataIndex: 'courseName', key: 'courseName', width: COL_WIDTH_L },
      { title: '上课日期', dataIndex: 'courseDate', key: 'courseDate', width: COL_WIDTH_L },
      { title: '上课时间', dataIndex: 'courseTime', key: 'courseTime', width: COL_WIDTH_M },
      { title: '课时数', dataIndex: 'courseHours', key: 'courseHours', width: COL_WIDTH_M },
      { title: '主班课程状态', dataIndex: 'courseStatus', key: 'courseStatus', width: COL_WIDTH_M },
    ];
    this.buildParams = this.buildParams.bind(this);
    this.getList = this.getList.bind(this);
    this.loadData = this.loadData.bind(this);
    this.pageChange = this.pageChange.bind(this);
  }
  static classPrefix = 'CourseList';
  componentDidMount() {
    this.loadData();
  }
  // 构建列表接口参数
  buildParams() {
    return {
      twoTeacherClassId: this.props.mainClassId,
      startDate: this.props.beginDate,
      endDate: this.props.endDate,
      page: this.state.current,
      rows: this.state.pageSize,
    };
  }
  // 请求列表接口
  getList(params) {
    return getCourseList(params, true).catch((err) => {
      showMessage(err.text, 'error');
    });
  }
  // 解析列表响应
  static resolveListRsp(rsp) {
    return rsp && rsp.resultCode === 0
      ? {
          list: CourseList.buildList(rsp.rows),
          total: rsp.records,
        }
      : {};
  }
  // 构建列表数据
  static buildList(list) {
    return list.map((item, index) => ({
      courseNum: `第 ${item.courseNum} 讲`,
      courseName: item.courseName || '暂无',
      courseDate: item.courseDate,
      courseTime: item.courseTime,
      courseHours: item.courseHours,
      courseStatus: item.courseStatusName,
    }));
  }
  // 加载数据
  async loadData() {
    this.setState({ loading: true });
    const rsp = await this.getList(this.buildParams());
    const payload = CourseList.resolveListRsp(rsp);
    this.setState({
      ...payload,
      loading: false,
    });
  }
  // 翻页
  pageChange({ current, pageSize }) {
    this.setState(
      {
        current,
        pageSize,
      },
      () => {
        this.loadData();
      },
    );
  }

  render() {
    const {} = this.props;
    const { loading, list, total, current, pageSize } = this.state;
    const prefix = CourseList.classPrefix;

    return (
      <Wrapper className={style[prefix]} loading={loading}>
        <List
          dataSource={list}
          columns={this.columns}
          pagination={{
            total,
            current,
            pageSize,
          }}
          onChange={this.pageChange}
        />
      </Wrapper>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    mainClassId: state.main.pageParams['/twoTeacherClassGroup/mainClassDetail'].classId,
    beginDate: state.main.pageParams['/twoTeacherClassGroup/mainClassDetail'].beginDate,
    endDate: state.main.pageParams['/twoTeacherClassGroup/mainClassDetail'].endDate,
  };
};

export default connect(mapStateToProps)(CourseList);
