/**
 * Created by Haolin on 2019/3/25.
 */
import React from 'react';
import { connect } from 'dva';
import style from './index.less';
import SearchBar from 'COMPONENT/SearchBar';
import Input from 'COMPONENT/Input';
import Select from 'COMPONENT/Select';
import List from 'COMPONENT/List';
import { findTwoTeacherClassTwoList } from 'SERVICE/course';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { OLDPAGEBASE } from 'CONST/config';
import { browserHistory } from 'react-router';
import Wrapper from 'COMPONENT/Wrapper';

const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L } = List;

class ViceClassList extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      name: '辅班列表',
      loading: false,
      viceClassName: null,
      branch: null,
      courseStatus: null,
      list: [],
      total: 0,
      current: 1,
      pageSize: 10,
    };
    this.columns = [
      { title: '序号', dataIndex: 'number', key: 'number', width: COL_WIDTH_S },
      { title: '辅班名称', dataIndex: 'name', key: 'name', width: COL_WIDTH_L },
      { title: '分公司', dataIndex: 'branchName', key: 'branchName', width: COL_WIDTH_M },
      { title: '学习中心', dataIndex: 'campusName', key: 'campusName', width: COL_WIDTH_M },
      { title: '辅导老师', dataIndex: 'teacherName', key: 'teacherName', width: COL_WIDTH_M },
      { title: '联系方式', dataIndex: 'teacherPhoneNum', key: 'teacherPhoneNum', width: COL_WIDTH_L },
      { title: '计划招生人数', dataIndex: 'recruitTotal', key: 'recruitTotal', width: COL_WIDTH_M },
      { title: '已报人数', dataIndex: 'actualTotal', key: 'actualTotal', width: COL_WIDTH_M },
      { title: '课程状态', dataIndex: 'courseState', key: 'courseState', width: COL_WIDTH_M },
      {
        title: '操作',
        key: 'action',
        fixed: 'right',
        getActionConfig: (text, record, index) => [
          {
            text: '详情',
            onClick: () => {
              this.goToDetail(record);
            },
          },
        ],
      },
    ];
    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.buildParams = this.buildParams.bind(this);
    this.getList = this.getList.bind(this);
    this.resolveListRsp = this.resolveListRsp.bind(this);
    this.buildList = this.buildList.bind(this);
    this.reset = this.reset.bind(this);
    this.search = this.search.bind(this);
    this.loadData = this.loadData.bind(this);
    this.pageChange = this.pageChange.bind(this);
    this.updateViceClassName = this.updateViceClassName.bind(this);
    this.updateBranch = this.updateBranch.bind(this);
    this.updateCourseState = this.updateCourseState.bind(this);
    this.goToDetail = this.goToDetail.bind(this);
  }
  static classPrefix = 'ViceClassList';
  componentDidMount() {
    this.init();
  }
  // 初始化
  async init() {
    this.setState({ loading: true });
    await this.initDict();
    const rsp = await this.getList(this.buildParams());
    const payload = this.resolveListRsp(rsp);
    this.setState({
      ...payload,
      loading: false,
    });
  }
  initDict() {
    return new Promise((resolve, reject) => {
      this.props.dispatch({
        type: 'dict/initDict',
        payload: {
          list: ['branch', 'courseStatus'],
          callback() {
            resolve();
          },
        },
      });
    });
  }
  // 构建列表接口参数
  buildParams() {
    return {
      twoTeacherClassId: this.props.mainClassId,
      name: this.state.viceClassName,
      branchId: this.state.branch,
      status: this.state.courseStatus,
      page: this.state.current,
      rows: this.state.pageSize,
    };
  }
  // 请求列表接口
  getList(params) {
    return findTwoTeacherClassTwoList(params, true).catch((err) => {
      showMessage(err.text, 'error');
    });
  }
  // 解析列表响应
  resolveListRsp(rsp) {
    return rsp && rsp.resultCode === 0
      ? {
          list: this.buildList(rsp.rows, this.state.current, this.state.pageSize),
          total: rsp.records,
        }
      : {};
  }
  // 构建列表数据
  buildList(list, page, size) {
    return list.map((item, index) => ({
      number: (page - 1) * size + index + 1,
      id: item.id,
      name: item.name,
      branchName: item.blBrenchName,
      campusName: item.blCampusName,
      teacherName: item.teacherName,
      teacherPhoneNum: item.teacherPhone,
      recruitTotal: item.peopleQuantity,
      actualTotal: item.realPeople,
      courseState: item.statusName,
    }));
  }
  // 重置
  reset() {
    this.setState({
      className: null,
      branch: null,
      courseStatus: null,
    });
  }
  // 检索
  search() {
    this.setState(
      {
        pageCurrent: 1,
      },
      () => {
        this.loadData();
      },
    );
  }
  // 加载数据
  async loadData() {
    this.setState({ loading: true });
    const rsp = await this.getList(this.buildParams());
    const payload = this.resolveListRsp(rsp);
    this.setState({
      ...payload,
      loading: false,
    });
  }
  // 翻页
  pageChange({ current, pageSize }) {
    this.setState(
      {
        current,
        pageSize,
      },
      () => {
        this.loadData();
      },
    );
  }
  // 更新辅班名称
  updateViceClassName(ele) {
    this.setState({ viceClassName: ele.target.value });
  }
  // 更新分公司
  updateBranch(val) {
    this.setState({ branch: val });
  }
  // 更新课程状态
  updateCourseState(val) {
    this.setState({ courseStatus: val });
  }
  // 辅班详情
  goToDetail(data) {
    let path = `function/twoteacher/twoTeacherClassDetail.html?classId=${this.props.mainClassId}&twoClassId=${data.id}`;
    let title = `双师详情（${data.name}）`;
    browserHistory.push({
      pathname: OLDPAGEBASE,
      query: { src: path, title },
    });
  }

  render() {
    const { dictBranch, dictCourseStatus } = this.props;
    const { loading, viceClassName, branch, courseStatus, list, total, current, pageSize } = this.state;
    const prefix = ViceClassList.classPrefix;

    return (
      <Wrapper
        className={style[prefix]}
        loading={loading}
        header={
          <SearchBar onSearch={this.search} onReset={this.reset}>
            <Input placeholder="辅班名称" value={viceClassName} onChange={this.updateViceClassName} />
            <Select placeholder="所属分公司" dict={dictBranch} value={branch} onChange={this.updateBranch} showSearch />
            <Select
              placeholder="课程状态"
              dict={dictCourseStatus}
              value={courseStatus}
              onChange={this.updateCourseState}
            />
          </SearchBar>
        }
      >
        <List
          dataSource={list}
          columns={this.columns}
          pagination={{
            total,
            current,
            pageSize,
          }}
          onChange={this.pageChange}
        />
      </Wrapper>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    mainClassId: state.main.pageParams['/twoTeacherClassGroup/mainClassDetail'].classId,
    dictBranch: state.dict.branch,
    dictCourseStatus: state.dict.courseStatus,
  };
};

export default connect(mapStateToProps)(ViceClassList);
