/**
 * Created by Haolin on 2018/7/19.
 */
import React from 'react';
import { connect } from 'dva';
import { Row, Col } from 'antd';
import Page from 'COMPONENT/Page';
import Block from 'COMPONENT/Block';
import FormItem from 'COMPONENT/FormItem';
import ShowProduct from './ShowProduct';
import { getMainClassInfo } from 'SERVICE/course';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import ViceClassInfo from './ViceClassInfo';
import './style.less';
// 返回不空值
const infoContent = (val) => (val && val !== '' ? val : '-');
// 主班详情
const MainClassInfo = (props) => {
  const {
    data: {
      className,
      productList,
      subject,
      teacher,
      classType,
      courseState,
      courseCount,
      courseDuration,
      courseTime,
      studio,
      recruitTotal,
      actualTotal,
      beginDate,
      classTime,
      classLevel,
      remarks,
    },
  } = props;
  return (
    <Block title="双师课堂主班信息" type="dark">
      <Row>
        <Col span={12}>
          <FormItem name="双师主班名称" labelWidth={100}>
            {infoContent(className)}
          </FormItem>
        </Col>
        <Col span={12}>
          <FormItem name="产品名称" labelWidth={100}>
            <ShowProduct list={productList || []} />
          </FormItem>
        </Col>
      </Row>
      <Row>
        <Col span={6}>
          <FormItem name="科目" labelWidth={100}>
            {infoContent(subject)}
          </FormItem>
        </Col>
        <Col span={6}>
          <FormItem name="主讲老师" labelWidth={100}>
            {infoContent(teacher)}
          </FormItem>
        </Col>
        <Col span={6}>
          <FormItem name="班级类型" labelWidth={100}>
            {infoContent(classType)}
          </FormItem>
        </Col>
        <Col span={6}>
          <FormItem name="课程状态" labelWidth={100}>
            {infoContent(courseState)}
          </FormItem>
        </Col>
      </Row>
      <Row>
        <Col span={6}>
          <FormItem name="总课时数" labelWidth={100}>
            {infoContent(courseCount)}
          </FormItem>
        </Col>
        <Col span={6}>
          <FormItem name="课时时长" labelWidth={100}>
            {infoContent(courseDuration)}
          </FormItem>
        </Col>
        <Col span={6}>
          <FormItem name="每讲课时数" labelWidth={100}>
            {infoContent(courseTime)}
          </FormItem>
        </Col>
        <Col span={6}>
          <FormItem name="直播间" labelWidth={100}>
            {infoContent(studio)}
          </FormItem>
        </Col>
      </Row>
      <Row>
        <Col span={6}>
          <FormItem name="计划招生人数" labelWidth={100}>
            {infoContent(recruitTotal)}
          </FormItem>
        </Col>
        <Col span={6}>
          <FormItem name="已报人数" labelWidth={100}>
            {actualTotal ? actualTotal : 0}
          </FormItem>
        </Col>
        <Col span={6}>
          <FormItem name="开始日期" labelWidth={100}>
            {infoContent(beginDate)}
          </FormItem>
        </Col>
        <Col span={6}>
          <FormItem name="上课时间" labelWidth={100}>
            {infoContent(classTime)}
          </FormItem>
        </Col>
      </Row>
      <Row>
        <Col span={6}>
          <FormItem name="班级层次" labelWidth={100}>
            {infoContent(classLevel)}
          </FormItem>
        </Col>
        <Col span={18}>
          <FormItem name="课程须知" labelWidth={100}>
            {infoContent(remarks)}
          </FormItem>
        </Col>
      </Row>
    </Block>
  );
};

class MainClassDetail extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false,
      mainClassInfo: {
        className: null,
        productList: null,
        subject: null,
        teacher: null,
        classType: null,
        courseState: null,
        courseCount: null,
        courseDuration: null,
        courseTime: null,
        studio: null,
        recruitTotal: null,
        actualTotal: null,
        beginDate: null,
        classTime: null,
        remarks: null,
      },
      // classData: null,
    };
    this.loadData = this.loadData.bind(this);
  }
  componentDidMount() {
    this.loadData();
  }
  // 加载数据
  async loadData() {
    this.setState({ loading: true });
    const rsp = await getMainClassInfo({ id: this.props.mainClassId }, true).catch((err) => {
      showMessage(err.text, 'error');
    });
    let payload = rsp ? MainClassDetail.buildData(rsp) : {};
    this.setState({
      ...payload,
      loading: false,
    });
  }
  // 构建列表数据
  static buildData(classData) {
    const {
      name,
      brenchVos,
      remark,
      subjectName,
      teacherName,
      classTypeName,
      statusName,
      totalClassHours,
      classTimeLength,
      everyCourseClassNum,
      liveRoomName,
      peopleQuantity,
      studentNums,
      startDate,
      classTime,
      classLevelName,
      classLevel,
    } = classData;
    let productList = brenchVos.map((item) => ({ name: `${item.brenchName}-${item.productName}` }));
    return {
      mainClassInfo: {
        className: name,
        productList: productList,
        subject: subjectName,
        teacher: teacherName,
        classType: classTypeName,
        courseState: statusName,
        courseCount: totalClassHours,
        courseDuration: classTimeLength,
        courseTime: everyCourseClassNum,
        studio: liveRoomName,
        recruitTotal: peopleQuantity,
        actualTotal: studentNums,
        beginDate: startDate,
        classTime: classTime,
        classLevel: classLevelName || classLevel,
        remarks: remark,
      },
      // classData: classData,
    };
  }

  render() {
    return (
      <Page
        className="main-class-detail"
        name="双师课堂主班详情"
        loading={this.state.loading}
        route={this.props.route}
        fitWindowHeight={false}
      >
        <MainClassInfo data={this.state.mainClassInfo} />
        <ViceClassInfo />
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    mainClassId: state.main.pageParams['/twoTeacherClassGroup/mainClassDetail'].classId,
  };
};

export default connect(mapStateToProps)(MainClassDetail);
