/**
 * Created by Haolin on 2019/3/19.
 */
import React, { PropTypes } from 'react';
import { connect } from 'dva';
import style from './index.less';
import { Pagination, Button } from 'antd';
import NoData from '../../../../../contract/ModalAddProduct/components/NoData';
import ProductCard from './ProductCard';
import Loading from 'COMPONENT/Loading';
import storeName from '../../../store/constant.js';

const DataTotal = (props) => {
  const { total } = props;
  const prefix = 'DataTotal';
  return (
    <div className={style[prefix]}>
      <span>
        共筛选到<span className={style[`${prefix}-highLight`]}>{total}</span>个结果
      </span>
      <Button className={style[`${prefix}-addAllBtn`]} type="primary" onClick={() => props.addAll()}>
        全部添加
      </Button>
    </div>
  );
};

class ProductDesk extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      _current: 1,
      _pageSize: 20,
    };
    this.pageSizeOptions = ['10', '20', '50'];
    this.pageChange = this.pageChange.bind(this);
    this.pageSizeChange = this.pageSizeChange.bind(this);
  }
  static classPrefix = 'ProductDesk';
  static defaultProps = {
    loading: false,
  };
  static propTypes = {
    dataSource: PropTypes.array.isRequired,
    loading: PropTypes.bool,
    pagination: PropTypes.object.isRequired,
    onChange: PropTypes.func.isRequired,
  };
  static showTotal(total, range) {
    return <span>共 {total} 条</span>;
  }
  componentDidMount() {}
  componentWillUnmount() {}
  // 翻页
  pageChange(current, pageSize) {
    const { pagination, onChange } = this.props;
    if (pagination && pagination.current && pagination.pageSize && onChange) {
      onChange(current, pageSize);
    }
    this.setState({
      _current: current,
      _pageSize: pageSize,
    });
  }
  // 每页数量切换
  pageSizeChange(current, pageSize) {
    const { pagination, onChange } = this.props;
    if (pagination && pagination.pageSize && onChange) {
      onChange(1, pageSize);
    }
    this.setState({
      _current: 1,
      _pageSize: pageSize,
    });
  }

  // 添加全部
  addAll = () => {
    this.props.dispatch({
      type: `${storeName}/selectAllProduct`,
      payload: {
        callback: () => {},
      },
    });
  };

  render() {
    const {
      dataSource,
      loading,
      pagination: { current, pageSize, total },
      onBtnClick,
      addPrdSelectedList,
    } = this.props;
    const { _current, _pageSize } = this.state;
    const prefix = ProductDesk.classPrefix;
    const checkLen = dataSource.length > 0;

    return (
      <div className={style[prefix]}>
        <Loading loading={loading} />
        <div className={style[`${prefix}-header`]}>
          <DataTotal total={total} addAll={this.addAll} />
        </div>
        <div className={style[`${prefix}-body`]}>
          {checkLen ? (
            dataSource.map((item, index) => (
              <ProductCard
                data={item}
                index={index}
                addPrdSelectedList={addPrdSelectedList}
                onBtnClick={() => {
                  onBtnClick(index);
                }}
                key={index}
              />
            ))
          ) : (
            <NoData />
          )}
        </div>
        <div className={style[`${prefix}-footer`]}>
          {checkLen && (
            <Pagination
              defaultCurrent={1}
              current={current || _current}
              pageSize={pageSize || _pageSize}
              total={total}
              showQuickJumper={false}
              showTotal={ProductDesk.showTotal}
              onChange={this.pageChange}
              pageSizeOptions={this.pageSizeOptions}
              showSizeChanger
              onShowSizeChange={this.pageSizeChange}
            />
          )}
        </div>
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    addPrdSelectedList: state[storeName].addPrd_selectedList,
  };
};

export default connect(mapStateToProps)(ProductDesk);
