/**
 * Created by Haolin on 2019/3/14.
 */
import React from 'react';
import { connect } from 'dva';
import style from './index.less';
import { Row, Col } from 'antd';
import InputNumber from 'COMPONENT/InputNumber';
import Button from 'COMPONENT/Button';
import Search2 from 'COMPONENT/Search2';
import storeName from '../../../store/constant.js';

const TWO_TEACHER_VAL = 'TWO_TEACHER';

class Search extends React.Component {
  constructor(props) {
    super(props);
    this.buildSearchConfig = this.buildSearchConfig.bind(this);
    this.search = this.search.bind(this);
    this.reset = this.reset.bind(this);
    this.updateCourseTimeTotal = this.updateCourseTimeTotal.bind(this);
    this.onSearchChange = this.onSearchChange.bind(this);
    this.beforeDestroy = this.beforeDestroy.bind(this);
  }
  static classPrefix = 'Search';
  componentWillUnmount() {
    this.beforeDestroy();
  }
  // 生成筛选器配置
  buildSearchConfig() {
    return [
      {
        label: '分公司',
        dict: this.props.dictBranch,
        field: 'sKey_addPrd_branch',
        value: this.props.sKey_addPrd_branch,
        type: 'multi',
      },
      { label: '年份', dict: this.props.dictYear, field: 'sKey_addPrd_year', value: this.props.sKey_addPrd_year },
      {
        label: '季度',
        dict: this.props.dictQuarter,
        field: 'sKey_addPrd_quarter',
        value: this.props.sKey_addPrd_quarter,
      },
      {
        label: '产品性质',
        dict: this.props.dictProductCharacter,
        field: 'sKey_addPrd_character',
        value: this.props.sKey_addPrd_character,
        type: 'multi',
      },
      {
        label: '班级类型',
        dict: this.props.dictClassType,
        field: 'sKey_addPrd_classType',
        value: this.props.sKey_addPrd_classType,
        type: 'multi',
      },
      {
        label: '产品系列',
        dict: this.props.dictCourseSeries,
        field: 'sKey_addPrd_courseSeries',
        value: this.props.sKey_addPrd_courseSeries,
        type: 'multi',
      },
      { label: '年级', dict: this.props.dictGrade, field: 'sKey_addPrd_grade', value: this.props.sKey_addPrd_grade },
      {
        label: '科目',
        dict: this.props.dictSubject,
        field: 'sKey_addPrd_subject',
        value: this.props.sKey_addPrd_subject,
      },
    ];
  }
  // 搜索
  search() {
    this.props.handleSearch();
  }
  // 重置
  reset() {
    this.props.dispatch({
      type: `${storeName}/initAddPrdSKey`,
      payload: {},
    });
  }
  // 更新总课时数
  updateCourseTimeTotal(val) {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: { sKey_addPrd_courseTimeTotal: val },
    });
  }
  // 监听筛选器选项值变化
  onSearchChange(sKeys) {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: sKeys,
    });
    this.search();
  }
  // 组件注销前
  beforeDestroy() {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        sKey_addPrd_courseTimeTotal: null,
        sKey_addPrd_branch: [],
        sKey_addPrd_year: null,
        sKey_addPrd_quarter: null,
        sKey_addPrd_character: [],
        sKey_addPrd_classType: [],
        sKey_addPrd_courseSeries: [],
        sKey_addPrd_grade: null,
        sKey_addPrd_subject: null,
      },
    });
  }

  render() {
    const prefix = Search.classPrefix;

    return (
      <div className={style[prefix]}>
        <Row className={style[`${prefix}-bar`]}>
          <Col span={6} style={{ paddingRight: 20 }}>
            <InputNumber
              placeholder="请输入总课时数"
              min={0}
              value={this.props.sKey_addPrd_courseTimeTotal}
              onChange={this.updateCourseTimeTotal}
            />
          </Col>
          <Col span={6}>
            <Button type="primary" onClick={this.search}>
              查询
            </Button>
            <Button type="default" onClick={this.reset}>
              重置
            </Button>
          </Col>
        </Row>
        <Search2 style={{ marginTop: 20 }} config={this.buildSearchConfig()} onChange={this.onSearchChange} />
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    dictBranch: state.dict.branch || [],
    dictYear: state.dict.year || [],
    dictQuarter: state.dict.quarter ? state.dict.quarter.filter((item) => item.text !== '全年') : [],
    dictProductCharacter: state.dict.productCharacter || [],
    dictClassType: state.dict.classType ? state.dict.classType[TWO_TEACHER_VAL] : [],
    dictCourseSeries: state.dict.courseSeriesSingle ? state.dict.courseSeriesSingle[TWO_TEACHER_VAL] : [],
    dictGrade: state.dict.grade || [],
    dictSubject: state.dict.subject || [],
    sKey_addPrd_courseTimeTotal: state[storeName].sKey_addPrd_courseTimeTotal,
    sKey_addPrd_branch: state[storeName].sKey_addPrd_branch,
    sKey_addPrd_year: state[storeName].sKey_addPrd_year,
    sKey_addPrd_quarter: state[storeName].sKey_addPrd_quarter,
    sKey_addPrd_character: state[storeName].sKey_addPrd_character,
    sKey_addPrd_classType: state[storeName].sKey_addPrd_classType,
    sKey_addPrd_courseSeries: state[storeName].sKey_addPrd_courseSeries,
    sKey_addPrd_grade: state[storeName].sKey_addPrd_grade,
    sKey_addPrd_subject: state[storeName].sKey_addPrd_subject,
  };
};

export default connect(mapStateToProps)(Search);
