/**
 * Created by Haolin on 2019/3/14.
 */
import React from 'react';
import { connect } from 'dva';
import style from './index.less';
import storeName from '../../store/constant.js';
import Section from '../../components/Section';
import Search from './Search';
import ProductDesk from './ProductDesk';

const TWO_TEACHER_VAL = 'TWO_TEACHER';

class AddProduct extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      name: '选择产品',
      loading: false,
      deskLoading: false,
    };
    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.initSKey = this.initSKey.bind(this);
    this.loadData = this.loadData.bind(this);
    this.loadList = this.loadList.bind(this);
    this.search = this.search.bind(this);
    this.changePage = this.changePage.bind(this);
    this.addProduct = this.addProduct.bind(this);
    this.beforeDestroy = this.beforeDestroy.bind(this);
  }
  static classPrefix = 'AddProduct';
  componentDidMount() {
    this.init();
  }
  componentWillUnmount() {
    this.beforeDestroy();
  }
  // 初始化
  async init() {
    this.setState({ loading: true });
    await this.initDict();
    await this.initSKey();
    await this.loadData();
    this.setState({ loading: false });
  }
  initDict() {
    return new Promise((resolve, reject) => {
      this.props.dispatch({
        type: 'dict/initDict',
        payload: {
          list: [
            'branch',
            'year',
            'quarter',
            'productCharacter',
            { name: 'classType', params: TWO_TEACHER_VAL },
            { name: 'courseSeriesSingle', params: TWO_TEACHER_VAL },
            'grade',
            'subject',
          ],
          callback() {
            resolve();
          },
        },
      });
    });
  }
  initSKey() {
    return new Promise((resolve, reject) => {
      this.props.dispatch({
        type: `${storeName}/initAddPrdSKey`,
        payload: {
          callback() {
            resolve();
          },
        },
      });
    });
  }
  // 加载数据
  loadData() {
    return new Promise((resolve, reject) => {
      this.props.dispatch({
        type: `${storeName}/loadProductList`,
        payload: {
          callback() {
            resolve();
          },
        },
      });
    });
  }
  // 加载列表
  async loadList() {
    this.setState({ loading: true });
    await this.loadData();
    this.setState({ loading: false });
  }
  // 筛选
  search() {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        addPrd_current: 1,
      },
    });
    this.loadList();
  }
  // 翻页
  changePage(current, pageSize) {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        addPrd_current: current,
        addPrd_pageSize: pageSize,
      },
    });
    this.loadList();
  }
  // 添加产品到购物车中
  addProduct(index) {
    this.props.dispatch({
      type: `${storeName}/selectProduct`,
      payload: { index },
    });
  }
  // 组件注销前销毁数据
  beforeDestroy() {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        addPrd_list: [],
        addPrd_total: 0,
        addPrd_current: 1,
        addPrd_pageSize: 10,
      },
    });
  }

  render() {
    const { list, current, pageSize, total } = this.props;
    const { name, loading, deskLoading } = this.state;
    const prefix = AddProduct.classPrefix;

    return (
      <Section
        className={style[prefix]}
        loading={loading}
        title={<span>{name}</span>}
        subHeader={<Search handleSearch={this.search} />}
        paddingSize={30}
        bodyStyle={{
          height: 'calc(100% - 10px)',
          overflowY: 'auto',
        }}
      >
        <ProductDesk
          dataSource={list}
          loading={!loading && deskLoading}
          pagination={{
            current,
            pageSize,
            total,
          }}
          onChange={this.changePage}
          onBtnClick={this.addProduct}
        />
      </Section>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    list: state[storeName].addPrd_list,
    current: state[storeName].addPrd_current,
    pageSize: state[storeName].addPrd_pageSize,
    total: state[storeName].addPrd_total,
  };
};

export default connect(mapStateToProps)(AddProduct);
