/**
 * Created by Haolin on 2019/3/14.
 */
import React, { PropTypes } from 'react';
import { connect } from 'dva';
import style from './index.less';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import Modal from 'COMPONENT/Modal';
import MainWrap from '../components/MainWrap';
import AddProduct from './AddProduct';
import SelectedProduct from './SelectedProduct';
import storeName from '../store/constant.js';

class ModalAddProduct extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      name: '',
      loading: false,
    };
    this.modalRef = null;
    this.init = this.init.bind(this);
    this.initSelectedList = this.initSelectedList.bind(this);
    this.synData = this.synData.bind(this);
    this.confirm = this.confirm.bind(this);
  }
  static propTypes = {
    // 由哪个页面调用的标识
    type: PropTypes.string.isRequired,
    onCancel: PropTypes.func.isRequired,
    onConfirm: PropTypes.func,
  };
  static classPrefix = 'ModalAddProduct';
  componentDidMount() {
    this.init();
  }
  // 初始化
  async init() {
    await this.initSelectedList();
  }
  initSelectedList() {
    return new Promise((resolve) => {
      this.props.dispatch({
        type: `${storeName}/initSelectedList`,
        payload: {
          type: this.props.type,
          callback() {
            resolve();
          },
        },
      });
    });
  }
  // 提交前同步已选数据
  synData() {
    return new Promise((resolve) => {
      this.props.dispatch({
        type: `${storeName}/confirmProduct`,
        payload: {
          type: this.props.type,
          callback() {
            resolve();
          },
        },
      });
    });
  }
  // 弹窗确定按钮
  async confirm() {
    if (this.props.list.length > 0) {
      await this.synData();
      this.props.onConfirm();
    } else {
      this.modalRef.showOkTips('请先选择产品！');
    }
  }

  render() {
    const { onCancel } = this.props;
    const { name, loading } = this.state;
    const prefix = ModalAddProduct.classPrefix;

    return (
      <Modal
        className={style[prefix]}
        title={name}
        loading={loading}
        onOk={this.confirm}
        onCancel={onCancel}
        ref={(ref) => {
          this.modalRef = getWrappedInstance(ref);
        }}
        widthSize="XL"
        themeConfig={{
          bgc: 'gray',
          footerTopShadow: true,
        }}
      >
        <MainWrap side={<SelectedProduct />}>
          <AddProduct />
        </MainWrap>
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    list: state[storeName].addPrd_selectedList,
  };
};

export default connect(mapStateToProps)(ModalAddProduct);
