/**
 * Created by Haolin on 2019/3/13.
 */
import React from 'react';
import { connect } from 'dva';
import style from './index.less';
import Modal from 'COMPONENT/Modal';
import Select from 'COMPONENT/Select';
import getWrappedInstance from 'UTIL/getWrappedInstance';

class ModalEditLiveRoom extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      name: '修改直播间',
      loading: false,
      liveRoomVal: null,
    };
    this.modalRefs = null;
    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.updateLiveRoom = this.updateLiveRoom.bind(this);
    this.confirm = this.confirm.bind(this);
  }
  static classPrefix = 'ModalEditLiveRoom';
  componentDidMount() {
    this.init();
  }
  componentWillUnmount() {}
  // 初始化
  async init() {
    this.setState({ loading: true });
    await this.initDict();
    this.setState({ loading: false });
  }
  initDict() {
    return new Promise((resolve, reject) => {
      this.props.dispatch({
        type: 'dict/initDict',
        payload: {
          list: ['liveRoom'],
          callback() {
            resolve();
          },
        },
      });
    });
  }
  // 更新直播间值
  updateLiveRoom(val) {
    this.setState({ liveRoomVal: val });
  }
  // 确定
  confirm() {
    if (this.state.liveRoomVal) {
      this.setState({ loading: true });
      const _self = this;
      this.props.onConfirm(this.state.liveRoomVal, () => {
        this.setState({ loading: false });
      });
    } else {
      this.modalRefs.showOkTips('请先选择直播间！');
    }
  }

  render() {
    const { onCancel, number, dictLiveRoom } = this.props;
    const { name, loading, liveRoomVal } = this.state;
    const prefix = ModalEditLiveRoom.classPrefix;

    return (
      <Modal
        className={style[prefix]}
        title={name}
        footerTipText="不影响已考勤的数据"
        loading={loading}
        widthSize="S"
        onCancel={onCancel}
        onOk={this.confirm}
        ref={(ref) => {
          this.modalRefs = getWrappedInstance(ref);
        }}
      >
        <div className={style[`${prefix}-row`]}>
          <span>
            将已选的<span className={style[`${prefix}-important`]}>{number}</span>个主班的直播间修改为：
          </span>
        </div>
        <div className={style[`${prefix}-row`]}>
          <Select
            placeholder="选择直播间"
            dict={dictLiveRoom}
            value={liveRoomVal}
            showSearch
            onChange={this.updateLiveRoom}
          />
        </div>
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    dictLiveRoom: state.dict.liveRoom,
  };
};

export default connect(mapStateToProps)(ModalEditLiveRoom);
