/**
 * Created by Haolin on 2019/3/12.
 */
import React from 'react';
import { connect } from 'dva';
import style from './index.less';
import Modal from 'COMPONENT/Modal';
import Select from 'COMPONENT/Select';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import { getTeacherDict } from 'SERVICE/course';
import { debounce } from 'lodash';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

class ModalEditMainTeacher extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      name: '修改主讲老师',
      loading: false,
      dictTeacher: null,
      mainTcVal: null,
    };
    this.modalRefs = null;
    this.updateMainTc = this.updateMainTc.bind(this);
    this.updateTeacherDict = this.updateTeacherDict.bind(this);
    this.handleMainTcSearch = debounce(this.updateTeacherDict, 330);
    this.confirm = this.confirm.bind(this);
  }
  static classPrefix = 'ModalEditMainTeacher';
  // 构建主讲老师字典
  static buildMainTcDict(list) {
    return list.map((item) => ({
      val: item.userId,
      text: item.name,
    }));
  }
  componentDidMount() {}
  componentWillUnmount() {}
  // 更新主讲老师值
  updateMainTc(val) {
    this.setState({ mainTcVal: val });
  }
  // 更新主讲老师字典数据
  updateTeacherDict(keyword) {
    this.setState({ loading: true });
    let apiParams = {
      name: keyword,
      // gradeId: null,
      // subjectId: null,
      ...this.props.params,
    };
    getTeacherDict(apiParams, true)
      .then((rsp) => {
        let dictTeacher = null;
        if (rsp && rsp.resultCode === 0) {
          dictTeacher = ModalEditMainTeacher.buildMainTcDict(rsp.data);
        }
        this.setState({
          loading: false,
          dictTeacher,
        });
      })
      .catch((err) => {
        showMessage(err.text, 'error');
        this.setState({ loading: false });
      });
  }
  // 确定
  confirm() {
    if (this.state.mainTcVal) {
      this.setState({ loading: true });
      const _self = this;
      this.props.onConfirm(this.state.mainTcVal, () => {
        this.setState({ loading: false });
      });
    } else {
      this.modalRefs.showOkTips('请先选择主讲老师！');
    }
  }

  render() {
    const { onCancel, number } = this.props;
    const { name, loading, dictTeacher, mainTcVal } = this.state;
    const prefix = ModalEditMainTeacher.classPrefix;

    return (
      <Modal
        className={style[prefix]}
        title={name}
        footerTipText="不影响已考勤的数据"
        loading={loading}
        widthSize="S"
        onCancel={onCancel}
        onOk={this.confirm}
        ref={(ref) => {
          this.modalRefs = getWrappedInstance(ref);
        }}
      >
        <div className={style[`${prefix}-row`]}>
          <span>
            将已选的<span className={style[`${prefix}-important`]}>{number}</span>个主班的主讲老师修改为：
          </span>
        </div>
        <div className={style[`${prefix}-row`]}>
          <Select
            placeholder="选择主讲老师"
            dict={dictTeacher}
            value={mainTcVal}
            onChange={this.updateMainTc}
            showSearch
            searchPlaceholder="请输入老师姓名进行搜索"
            onSearchKeyChange={this.handleMainTcSearch}
          />
        </div>
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  return {};
};

export default connect(mapStateToProps)(ModalEditMainTeacher);
