import { PropTypes } from 'react';
import Loading from 'COMPONENT/Loading';
import classNames from 'classnames';
import style from './index.less';
import Wrapper from 'COMPONENT/Wrapper';

// const Section = (props) => {
//   const { className, title, loading, paddingSize, bodyStyle } = props;
//   const prefix = 'Section';
//   return (
//     <div className={classNames(style[prefix], className)}>
//       <Loading loading={loading}/>
//       <div className={style[`${prefix}-header`]}>
//         <div className={style[`${prefix}-title`]}>{title}</div>
//       </div>
//       <div
//         className={style[`${prefix}-body`]}
//         style={{
//           paddingLeft: paddingSize,
//           paddingRight: paddingSize,
//           ...bodyStyle,
//         }}>
//         <Wrapper
//           header={props.subHeader}>
//           {props.children}
//         </Wrapper>
//       </div>
//     </div>
//   )
// };

const classPrefix = 'Section';

const Section = (props) => {
  return (
    <Wrapper
      className={classNames(style[classPrefix], props.className)}
      loading={props.loading}
      bodyFitHeight
      header={
        props.title && (
          <div className={style[`${classPrefix}-header`]}>
            <div className={style[`${classPrefix}-title`]}>{props.title}</div>
          </div>
        )
      }
    >
      <div
        className={style[`${classPrefix}-body`]}
        style={{
          paddingLeft: props.paddingSize,
          paddingRight: props.paddingSize,
          ...props.bodyStyle,
        }}
      >
        <Wrapper header={props.subHeader}>{props.children}</Wrapper>
      </div>
    </Wrapper>
  );
};

Section.protoTypes = {
  className: PropTypes.string,
  title: PropTypes.string,
  subHeader: PropTypes.oneOfType([PropTypes.node, PropTypes.string, PropTypes.number, PropTypes.array]),
  children: PropTypes.oneOfType([PropTypes.node, PropTypes.string, PropTypes.number, PropTypes.array]),
  loading: PropTypes.bool,
  paddingSize: PropTypes.number,
  bodyStyle: PropTypes.object,
};

Section.defaultProps = {
  className: '',
  paddingSize: 0,
  loading: false,
};

export default Section;
