/**
 * Created by Haolin on 2018/7/10.
 */
import React from 'react';
import { connect } from 'dva';
import { Modal, message } from 'antd';
import Page from 'COMPONENT/Page';
import Button from 'COMPONENT/Button';
import SearchBar from 'COMPONENT/SearchBar';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import Input from 'COMPONENT/Input';
import DatePicker from 'COMPONENT/DatePicker';
import List from 'COMPONENT/List';
import { debounce } from 'lodash';
import { getDictTextByVal } from 'UTIL/dict';
import {
  getMainClassList,
  editAndDeleteMainClass,
  getTeacherDict,
  batchUpdateTwoTeacherClassTeacher,
  batchUpdateTwoTeacherClassLiveRoom,
} from 'SERVICE/course';
import { findClassLevelByClassType } from 'SERVICE/common';
import { OLDPAGEBASE } from 'CONST/config';
import { browserHistory } from 'react-router';
import './style.less';
import style from './index.less';
import EditMainClass from './EditMainClass';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import exportFile from 'UTIL/exportFile';
import ButtonWrap from '../../signUpOnline/StdntApptMgmt/ButtonWrap';
import compStyle from '../ChangeClass/compStyle.less';
import ModalEditMainTeacher from './ModalEditMainTeacher';
import ModalEditLiveRoom from './ModalEditLiveRoom';
import PopContent from './components/PopContent';

const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;

const TWO_TEACHER_VAL = 'TWO_TEACHER';
const CONPELETE = 'CONPELETE';

// 是否上课字典
const isModalDict = [
  { val: '1', text: '已排课' },
  { val: '0', text: '未排课' },
];
// 辅班数量字典
const dictViceClassRange = [
  {
    val: '0',
    text: '0个',
  },
  {
    val: '1',
    text: '1~5个',
  },
  {
    val: '6',
    text: '大于5个',
  },
];
// 学生数量字典
const dictStdRange = [
  {
    val: '0',
    text: '0个',
  },
  {
    val: '1',
    text: '大于0个',
  },
];
// 搜索栏
const Search = (props) => {
  const {
    keywords: {
      beginDate,
      endDate,
      className,
      classType,
      branch,
      grade,
      subject,
      teacher,
      courseState,
      classPhase,
      productType,
      classLevel,
      year,
      quarter,
      isModal,
      productCharacter,
      viceClassRange,
      stdRange,
    },
    dict: {
      dictClassType,
      dictBranch,
      dictStudentGrade,
      dictSubject,
      dictCourseState,
      dictTeacher,
      dictPhaseWeekDay,
      dictProductType,
      dictClassLevel,
      dictYear,
      dictQuarter,
      isModalDict,
      dictProductCharacter,
      dictViceClassRange,
      dictStdRange,
    },
    handleSearch,
    handleReset,
    handleKeyword,
    handleTcDict,
  } = props;
  return (
    <SearchBar onSearch={handleSearch} onReset={handleReset}>
      <DatePicker
        placeholder="开始时间"
        value={beginDate}
        onChange={(val) => {
          handleKeyword({ type: 'beginDate', val });
        }}
      />
      <DatePicker
        placeholder="结束时间"
        value={endDate}
        onChange={(val) => {
          handleKeyword({ type: 'endDate', val });
        }}
      />
      <Input
        placeholder="双师课堂名称"
        value={className}
        onChange={(e) => {
          handleKeyword({ type: 'className', val: e.target.value });
        }}
      />
      <Select
        placeholder="所属分公司"
        dict={dictBranch}
        value={branch}
        showSearch
        onChange={(val) => {
          handleKeyword({ type: 'branch', val });
        }}
      />
      <Select
        placeholder="产品年份"
        dict={dictYear}
        value={year}
        onChange={(val) => {
          handleKeyword({ type: 'year', val });
        }}
      />
      <Select
        placeholder="产品季度"
        dict={dictQuarter}
        value={quarter}
        onChange={(val) => {
          handleKeyword({ type: 'quarter', val });
        }}
      />
      <Select
        placeholder="期数/星期数"
        dict={dictPhaseWeekDay}
        value={classPhase}
        onChange={(val) => {
          handleKeyword({ type: 'classPhase', val });
        }}
        disabled={!quarter}
      />
      <Select
        placeholder="产品性质"
        dict={dictProductCharacter}
        value={productCharacter}
        onChange={(val) => {
          handleKeyword({ type: 'productCharacter', val });
        }}
      />
      <Select
        placeholder="班级类型"
        dict={dictClassType}
        value={classType}
        onChange={(val) => {
          handleKeyword({ type: 'classType', val });
        }}
      />
      <Select
        placeholder="产品系列"
        dict={dictProductType}
        value={productType}
        onChange={(val) => {
          handleKeyword({ type: 'productType', val });
        }}
      />
      <MultiSelect
        placeholder="班级层次"
        dict={dictClassLevel}
        value={classLevel}
        onChange={(val) => {
          handleKeyword({ type: 'classLevel', val });
        }}
        disabled={!classType}
      />
      <Select
        placeholder="年级"
        dict={dictStudentGrade}
        value={grade}
        showSearch
        onChange={(val) => {
          handleKeyword({ type: 'grade', val });
        }}
      />
      <Select
        placeholder="科目"
        dict={dictSubject}
        value={subject}
        showSearch
        onChange={(val) => {
          handleKeyword({ type: 'subject', val });
        }}
      />
      <Select
        placeholder="主讲老师"
        searchPlaceholder="请输入老师名称"
        dict={dictTeacher}
        value={teacher}
        onChange={(val) => {
          handleKeyword({ type: 'teacher', val });
        }}
        onSearchKeyChange={(keyword) => {
          handleTcDict({ val: keyword });
        }}
        showSearch
      />
      <Select
        placeholder="课程状态"
        dict={dictCourseState}
        value={courseState}
        onChange={(val) => {
          handleKeyword({ type: 'courseState', val });
        }}
      />
      <Select
        placeholder="是否排课"
        dict={isModalDict}
        value={isModal}
        onChange={(val) => {
          handleKeyword({ type: 'isModal', val });
        }}
      />
      <Select
        placeholder="辅班数量"
        dict={dictViceClassRange}
        value={viceClassRange}
        onChange={(val) => {
          handleKeyword({ type: 'viceClassRange', val });
        }}
      />
      <Select
        placeholder="学生数量"
        dict={dictStdRange}
        value={stdRange}
        onChange={(val) => {
          handleKeyword({ type: 'stdRange', val });
        }}
      />
    </SearchBar>
  );
};

class TwoTeacherClass extends React.Component {
  constructor(props) {
    super(props);
    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.columns = this.columns.bind(this);
    this.buildFetchParams = this.buildFetchParams.bind(this);
    this.loadData = this.loadData.bind(this);
    this.buildListData = this.buildListData.bind(this);
    this.pageChange = this.pageChange.bind(this);
    this.buildPhaseDict = this.buildPhaseDict.bind(this);
    this.exportExcel = this.exportExcel.bind(this);
    this.updateSelectedRowKeys = this.updateSelectedRowKeys.bind(this);
    this.updateSelect = this.updateSelect.bind(this);
    this.updateSelectAll = this.updateSelectAll.bind(this);
    this.batchAction = this.batchAction.bind(this);
    this.switchModalEditMainTc = this.switchModalEditMainTc.bind(this);
    this.switchModalEditLiveRoom = this.switchModalEditLiveRoom.bind(this);
    this.submitEditMainTc = this.submitEditMainTc.bind(this);
    this.submitEditLiveRoom = this.submitEditLiveRoom.bind(this);
    // 节流-获取老师字典数据
    this.getTeacherDict = debounce(this.updateTeacherDict, 330);
    this.getClassLevelDict = debounce(this.updateClassLevelDict, 330);

    this.state = {
      loading: false,
      toggle_EditMainClass: false,
      // 字典 班级层次
      dictClassLevel: null,
      // 字典 主班老师
      dictTeacher: null,
      // 字典 期数/星期数(实际使用)
      dictPhaseWeekDay: null,
      keywords: {
        beginDate: null,
        endDate: null,
        className: null,
        classType: null,
        classLevel: null,
        branch: null,
        grade: null,
        subject: null,
        teacher: null,
        courseState: null,
        year: null,
        quarter: null,
        classPhase: null,
        productType: null,
        isModal: null,
        productCharacter: null,
        viceClassRange: null,
        stdRange: null,
      },
      listLoading: false,
      selectedRowKeys: [],
      // 主班列表
      classList: [],
      pageSize: 20,
      pageCurrent: 1,
      pageTotal: 0,
      // 编辑中的主班id
      editClassId: null,
      toggle_editMainTeacher: false,
      toggle_editLiveRoom: false,
      isModal: false, // 是否排课
    };
    // 批量修改主讲老师所需年级/科目
    this.batchEditTeacherParams = {
      gradeId: null,
      subjectId: null,
    };
    this.selectAll = false;
  }

  componentDidMount() {
    this.init();
  }

  static classPrefix = 'TwoTeacherClass';
  // 构建上课时间范围
  static buildMainClassTimeRange(beginTime, classTimeLength, everyCourseClassNum) {
    if (classTimeLength && everyCourseClassNum) {
      let beginTimestamp = new Date(`2019-01-01 ${beginTime}`).getTime();
      let rangeMillisecond = classTimeLength * everyCourseClassNum * 60000;
      let endDate = new Date(beginTimestamp + rangeMillisecond);
      let endHour = endDate.getHours();
      let endMin = endDate.getMinutes();
      if (endHour < 10) endHour = `0${endHour}`;
      if (endMin < 10) endMin = `0${endMin}`;
      return `${beginTime}-${endHour}:${endMin}`;
    } else if (beginTime) {
      return `${beginTime}-`;
    } else {
      return '-';
    }
  }
  // 初始化
  async init() {
    this.setState({ loading: true });
    await this.initDict();
    await this.loadData();
    this.setState({ loading: false });
  }

  initDict() {
    return new Promise((resolve, reject) => {
      this.props.dispatch({
        type: 'dict/initDict',
        payload: {
          list: [
            { name: 'classType', params: TWO_TEACHER_VAL },
            'branch',
            'grade',
            'subject',
            'phase',
            { name: 'courseSeriesSingle', params: TWO_TEACHER_VAL },
            'courseStatus',
            'weekDay',
            'year',
            'quarter',
            'productCharacter',
          ],
          callback() {
            resolve();
          },
        },
      });
    });
  }
  // 更新state
  updateState = (newState) => {
    this.setState((prevState, props) => {
      return {
        ...newState,
      };
    });
  };
  // 更新搜索关键词
  updateKeyword = ({ type, val }) => {
    let payload = {
      keywords: {
        ...this.state.keywords,
        [type]: val,
      },
    };
    if (type === 'quarter') {
      payload.keywords.classPhase = null;
      if (val) {
        payload.dictPhaseWeekDay = this.buildPhaseDict(val);
      }
    }
    if (type === 'classType') {
      payload.keywords.classLevel = null;
      if (val) {
        this.getClassLevelDict(val);
      }
    }
    this.setState(payload);
  };
  // 更新主讲老师字典数据
  updateTeacherDict = (params) => {
    const { val } = params;
    const {
      keywords: { grade, subject },
    } = this.state;
    this.loadTeacherDict({ teacherName: val, gradeId: grade, subjectId: subject, type: 'update' });
  };
  // 获取主讲老师字典
  loadTeacherDict = async (params) => {
    const { teacherName, gradeId, subjectId, type } = params;
    let apiData = {
      name: teacherName,
      gradeId,
      subjectId,
    };
    const { resultCode, data } = await getTeacherDict(apiData, true);
    let list = resultCode === 0 ? data : [];
    let result = list.map((item) => ({ val: item.userId, text: item.name }));
    if (type === 'update') {
      this.updateState({ dictTeacher: result });
    } else {
      return result;
    }
  };
  // 获取班级层次字典
  updateClassLevelDict = async (params, hadReturn) => {
    this.setState({ loading: true });
    const { resultCode, resultMessage, data } = await findClassLevelByClassType({ classTypeId: params }, true);
    if (resultCode === 0) {
      const dict = data.map((item) => ({
        text: item.NAME,
        val: item.ID,
      }));
      if (hadReturn) {
        return dict;
      } else {
        this.setState({
          loading: false,
          dictClassLevel: dict,
        });
      }
    } else {
      if (hadReturn) {
        return null;
      } else {
        showMessage(resultMessage);
        this.setState({ loading: false });
      }
    }
  };
  // 搜索主班列表
  search = () => {
    this.setState(
      {
        pageCurrent: 1,
      },
      () => {
        this.loadData();
      },
    );
  };
  // 重置搜索关键词
  reset = () => {
    this.setState({
      keywords: {
        beginDate: null,
        endDate: null,
        className: null,
        classType: null,
        classLevel: null,
        branch: null,
        grade: null,
        subject: null,
        teacher: null,
        courseState: null,
        year: null,
        quarter: null,
        classPhase: null,
        productType: null,
        isModal: null,
        productCharacter: null,
        viceClassRange: null,
        stdRange: null,
      },
    });
  };
  // 构建筛选栏参数
  buildFetchParams() {
    const {
      keywords: {
        beginDate,
        endDate,
        className,
        classType,
        branch,
        grade,
        subject,
        teacher,
        courseState,
        classPhase,
        productType,
        classLevel,
        year,
        quarter,
        isModal,
        productCharacter,
        viceClassRange,
        stdRange,
      },
      pageCurrent,
      pageSize,
    } = this.state;
    let mapWeekPhase = null;
    let mapPhase = null;
    const quarterText = getDictTextByVal(this.props.dictQuarter, quarter);
    if (quarterText === '春季' || quarterText === '秋季') {
      // 春秋季星期数
      mapWeekPhase = classPhase;
    } else {
      // 寒暑假期数
      mapPhase = classPhase;
    }
    return {
      startDate: beginDate,
      endDate,
      name: className,
      classTypeId: classType,
      branchId: branch,
      gradeId: grade,
      subjectId: subject,
      subject: subject,
      teacherId: teacher,
      statusId: courseState,
      courseWeek: mapWeekPhase,
      phaseId: mapPhase,
      productCourseSeriesId: productType,
      productVersion: year,
      productQuarter: quarter,
      classLevel,
      isModal: typeof isModal === 'string' && isModal !== '' ? Number(isModal) : null,
      sidx: '',
      sord: '',
      page: pageCurrent,
      rows: pageSize,
      productCharacter: productCharacter,
      classNumType: viceClassRange,
      studentNumType: stdRange,
    };
  }
  // 加载列表数据
  async loadData() {
    this.setState({
      listLoading: true,
      selectedRowKeys: [],
    });
    const apiParams = this.buildFetchParams();
    const rsp = await getMainClassList(apiParams, true).catch((err) => {
      showMessage(err.text, 'error');
    });
    let classList = [];
    let pageTotal = 0;
    if (rsp && rsp.resultCode === 0) {
      classList = this.buildListData(rsp.rows, apiParams.page, apiParams.rows);
      pageTotal = rsp.records;
    }
    this.setState({
      listLoading: false,
      classList,
      pageTotal,
    });
  }
  // 构建列表数据
  buildListData(list, current, pageSize) {
    return list.map((item, index) => {
      let phaseWeekday = null;
      if (item.productQuarterName) {
        phaseWeekday = ['春季', '秋季'].includes(item.productQuarterName) ? item.courseWeekName : item.phaseName;
      }
      return {
        number: index + 1 + (current - 1) * pageSize,
        id: item.classId,
        className: item.name,
        classLevel: item.classLevelName,
        branch: item.blCampusName.split(','),
        grade: item.gradeName,
        subject: item.subjectName,
        teacher: `${item.teacherName}${item.teacherEnableFlg === 1 ? '（已离职）' : ''}`,
        liveRoomName: item.liveRoomName || '无',
        courseCount: item.totalClassHours,
        beginDate: item.startDate,
        endDate: item.endDate,
        classTime: TwoTeacherClass.buildMainClassTimeRange(
          item.classTime,
          item.classTimeLength,
          item.everyCourseClassNum,
        ),
        isModal: typeof item.isModal === 'number' ? getDictTextByVal(isModalDict, item.isModal.toString()) : null,
        classCount: item.classNum,
        courseState: item.statusName,
        status: item.status,
        gradeId: item.gradeId,
        subjectId: item.subjectId,
        teacherId: item.teacherId,
        blCampusId: item.blCampusId,
        phaseWeekday,
        studentCount: item.studentNums || 0,
      };
    });
  }
  // 翻页
  pageChange({ current, pageSize }) {
    this.setState(
      {
        pageSize: pageSize,
        pageCurrent: current,
      },
      () => {
        this.loadData();
      },
    );
  }
  // 新增双师课堂主班
  goToAddMainClass = () => {
    let { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: {
        path: '/twoTeacherClassGroup/addMainClass',
        params: {},
        refresh: true,
      },
    });
  };
  //  列表更多操作
  moreAction = (params) => {
    const { type, record, index } = params;
    switch (type) {
      case 'detail':
        this.goToMainClassDetail(record);
        break;
      case 'course':
        this.goToCourse(record);
        break;
      case 'edit':
        this.openEdit(record.id, record.isModal);
        break;
      case 'delete':
        let modalTitle = '确认删除该主班课堂吗？';
        let modalContent = '双师课堂(' + record.className + ')';
        const that = this;
        Modal.confirm({
          title: modalTitle,
          content: modalContent,
          className: 'homAntdModal',
          onOk() {
            that.deleteMainClass(record.id);
          },
        });
        break;
    }
  };
  // 主班详情页
  goToMainClassDetail = (data) => {
    const { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: {
        path: '/twoTeacherClassGroup/mainClassDetail',
        params: {
          classId: data.id,
          beginDate: data.beginDate,
          endDate: data.endDate,
        },
        refresh: true,
      },
    });
  };
  // 课程详情
  goToCourse = (record) => {
    const { id, className, gradeId, subjectId, teacherId, blCampusId } = record;
    let path =
      'function/twoteacher/twoTeacherClassCourse.html?classId=' +
      id +
      '&name=' +
      className +
      '&gradeId=' +
      gradeId +
      '&subjectId=' +
      subjectId +
      '&teacherId=' +
      teacherId +
      '&blCampusId=' +
      blCampusId +
      '&type=group';
    let title = '双师课程(' + className + ')';
    browserHistory.push({
      pathname: OLDPAGEBASE,
      query: { src: path, title },
    });
  };
  // 修改主班
  openEdit = (id, isModal) => {
    this.setState((prevState) => ({
      toggle_EditMainClass: !prevState.toggle_EditMainClass,
      editClassId: id,
      isModal,
    }));
  };
  // 删除主班
  deleteMainClass = async (id) => {
    let apiParams = { classId: id, oper: 'del' };
    const apiData = await editAndDeleteMainClass(apiParams);
    if (apiData.resultCode === 0) {
      message.success('删除成功！');
      this.loadData();
    } else {
      message.error(apiData.resultMessage);
    }
  };
  // 取消按钮-修改主班弹窗
  editCancel = () => {
    this.setState({ toggle_EditMainClass: false, editClassId: null });
  };
  // 修改成功-修改主班弹窗
  editConfirm = () => {
    this.editCancel();
    this.loadData();
  };
  // 根据季度值切换期数/星期数字典
  buildPhaseDict(quarterVal) {
    const { dictQuarter, dictPhase, dictWeekDay } = this.props;
    let quarterText = getDictTextByVal(dictQuarter, quarterVal);
    if (quarterText === '暑假' || quarterText === '寒假') {
      return dictPhase;
    } else if (quarterText === '春季' || quarterText === '秋季') {
      return dictWeekDay;
    } else {
      return null;
    }
  }
  // 定义list列表参数
  columns(purviewCourse, purviewEdit, purviewDelete) {
    return [
      { title: '序号', dataIndex: 'number', key: 'number', width: COL_WIDTH_S, fixed: 'left' },
      { title: '双师课堂名称', dataIndex: 'className', key: 'className', width: COL_WIDTH_XL, fixed: 'left' },
      { title: '班级层次', dataIndex: 'classLevel', key: 'classLevel', width: COL_WIDTH_L },
      {
        title: '所属分公司',
        dataIndex: 'branch',
        key: 'branch',
        width: COL_WIDTH_L,
        render: (text, record, index) => <PopContent data={text} />,
      },
      { title: '期数/星期数', dataIndex: 'phaseWeekday', key: 'phaseWeekday', width: COL_WIDTH_L },
      { title: '年级', dataIndex: 'grade', key: 'grade', width: COL_WIDTH_M },
      { title: '科目', dataIndex: 'subject', key: 'subject', width: COL_WIDTH_M },
      { title: '主讲老师', dataIndex: 'teacher', key: 'teacher', width: COL_WIDTH_M },

      { title: '直播间', dataIndex: 'liveRoomName', key: 'liveRoomName', width: COL_WIDTH_L },

      { title: '课时数', dataIndex: 'courseCount', key: 'courseCount', width: COL_WIDTH_M },
      { title: '开始日期', dataIndex: 'beginDate', key: 'beginDate', width: COL_WIDTH_L },
      { title: '上课时间', dataIndex: 'classTime', key: 'classTime', width: COL_WIDTH_M },
      { title: '是否排课', dataIndex: 'isModal', key: 'isModal', width: COL_WIDTH_M },
      { title: '辅班数量', dataIndex: 'classCount', key: 'classCount', width: COL_WIDTH_M },
      { title: '学生数量', dataIndex: 'studentCount', key: 'studentCount', width: COL_WIDTH_M },
      { title: '课程状态', dataIndex: 'courseState', key: 'courseState', width: COL_WIDTH_M },
      {
        title: '操作',
        key: 'action',
        width: COL_WIDTH_L,
        fixed: 'right',
        getActionConfig: (text, record, index) => {
          return [
            {
              text: '课程',
              onClick: () => this.moreAction({ type: 'course', record, index }),
              disabled: purviewCourse,
            },
            {
              menus: [
                { text: '详情', onClick: () => this.moreAction({ type: 'detail', record, index }) },
                {
                  text: '修改',
                  onClick: () => this.moreAction({ type: 'edit', record, index }),
                  disabled: purviewEdit,
                },
                {
                  text: '删除',
                  onClick: () => this.moreAction({ type: 'delete', record, index }),
                  disabled: purviewDelete,
                },
              ],
            },
          ];
        },
      },
    ];
  }
  // 导出Excel
  exportExcel() {
    this.setState({ loading: true });
    const params = this.buildFetchParams();
    const _self = this;
    exportFile
      .get({
        url: '/eduboss/TwoTeacherClassController/ExportTwoTeacherClassExcel.do',
        params,
        fileName: '双师主班课程名单',
        fileExtension: 'xls',
      })
      .then((data) => {
        _self.setState({ loading: false });
      })
      .catch((err) => {
        _self.setState({ loading: false });
      });
  }

  // 更新选中的列表项状态
  updateSelectedRowKeys(keys) {
    this.setState({ selectedRowKeys: keys });
  }

  updateSelect(record, selected, selectedRows) {
    let canSelectMaxLen = this.state.classList.filter((item) => item.status !== CONPELETE).length;
    let selectedRowKeys = [];
    if (selected) {
      selectedRowKeys = selectedRows.filter((item) => item.status !== CONPELETE).map((item) => item.key);
      if (selectedRows.length === canSelectMaxLen) {
        this.selectAll = true;
      }
    } else {
      selectedRowKeys = selectedRows.map((item) => item.key);
      if (selectedRows.length + 1 === canSelectMaxLen) {
        this.selectAll = false;
      }
    }

    this.setState({ selectedRowKeys });
  }

  updateSelectAll(selected, selectedRows, changeRows) {
    this.selectAll = !this.selectAll;

    let selectedRowKeys = [];
    if (this.selectAll) {
      selectedRowKeys = selectedRows.filter((item) => item.status !== CONPELETE).map((item) => item.key);
    }

    this.setState({ selectedRowKeys });
  }

  // 批量操作
  batchAction({ actionType }) {
    if (actionType === 'mainTeacher') {
      const { selectedRowKeys, classList } = this.state;
      // 选择多个的时候判断是否相同年级/科目
      if (selectedRowKeys.length > 1) {
        let checkArr = selectedRowKeys.map((item) => [classList[item].gradeId, classList[item].subjectId]);
        if (!TwoTeacherClass.isArrSameVal(checkArr)) {
          showMessage('请选择相同年级与科目的班级！', 'error');
          return;
        }
      }
      this.batchEditTeacherParams = {
        gradeId: classList[selectedRowKeys[0]].gradeId,
        subjectId: classList[selectedRowKeys[0]].subjectId,
      };
      this.switchModalEditMainTc();
    } else {
      this.switchModalEditLiveRoom();
    }
  }
  // 判断2级嵌套数组的每一项值是否相等 [[xxx, yyy, ...], [xxx, yyy, ...], ...]
  static isArrSameVal(arr) {
    let result = true;
    let firstItem = arr.shift();
    firstItem.forEach((item, index) => {
      let tempRet = arr.some((cItem) => item !== cItem[index]);
      if (tempRet) result = false;
    });
    return result;
  }
  // 切换 批量修改主讲老师弹窗 显示状态
  switchModalEditMainTc() {
    this.setState(
      (prevState) => ({
        toggle_editMainTeacher: !prevState.toggle_editMainTeacher,
      }),
      () => {
        if (!this.state.toggle_editMainTeacher) {
          this.batchEditTeacherParams = {
            gradeId: null,
            subjectId: null,
          };
        }
      },
    );
  }
  // 切换 批量修改直播间弹窗 显示状态
  switchModalEditLiveRoom() {
    this.setState((prevState) => ({
      toggle_editLiveRoom: !prevState.toggle_editLiveRoom,
    }));
  }
  // 提交批量修改主讲老师
  async submitEditMainTc(id, failCallback) {
    const { selectedRowKeys, classList } = this.state;
    let apiParams = {
      classIds: selectedRowKeys.map((item) => classList[item].id),
      teacherId: id,
    };
    const rsp = await batchUpdateTwoTeacherClassTeacher(apiParams, true).catch((err) => {
      showMessage(err.text, 'error');
      if (failCallback) failCallback();
    });
    if (rsp && rsp.resultCode === 0) {
      showMessage('批量修改主讲老师成功！', 'success');
      this.switchModalEditMainTc();
      this.updateSelectedRowKeys([]);
      this.loadData();
    }
  }
  // 提交批量修改直播间
  async submitEditLiveRoom(id, failCallback) {
    const { selectedRowKeys, classList } = this.state;
    let apiParams = {
      classIds: selectedRowKeys.map((item) => classList[item].id),
      liveRoomId: id,
    };
    const rsp = await batchUpdateTwoTeacherClassLiveRoom(apiParams, true).catch((err) => {
      showMessage(err.text, 'error');
      if (failCallback) failCallback();
    });
    if (rsp && rsp.resultCode === 0) {
      showMessage('批量修改直播间成功！', 'success');
      this.switchModalEditLiveRoom();
      this.updateSelectedRowKeys([]);
      this.loadData();
    }
  }

  render() {
    const {
      route,
      btnPurviewAddMainClass,
      btnPurviewCourse,
      btnPurviewEdit,
      btnPurviewDelete,
      btnExportExcel,
      dictClassType,
      dictBranch,
      dictStudentGrade,
      dictSubject,
      dictCourseState,
      dictProductType,
      dictYear,
      dictQuarter,
      dictProductCharacter,
    } = this.props;
    const {
      loading,
      toggle_EditMainClass,
      keywords,
      listLoading,
      classList,
      pageSize,
      pageCurrent,
      pageTotal,
      dictTeacher,
      dictPhaseWeekDay,
      editClassId,
      dictClassLevel,
      selectedRowKeys,
      toggle_editMainTeacher,
      toggle_editLiveRoom,
      isModal,
    } = this.state;
    const dictObj = {
      dictClassType,
      dictBranch,
      dictStudentGrade,
      dictSubject,
      dictCourseState,
      dictTeacher,
      dictProductType,
      dictClassLevel,
      dictYear,
      dictQuarter,
      isModalDict,
      dictPhaseWeekDay,
      dictProductCharacter,
      dictViceClassRange,
      dictStdRange,
    };

    return (
      <Page
        name="双师管理（集团）"
        className={TwoTeacherClass.classPrefix}
        fitWindowHeight={true}
        loading={loading}
        route={route}
        subHeader={
          <div className="two-teacher-search-bar">
            <Search
              keywords={keywords}
              dict={dictObj}
              handleKeyword={this.updateKeyword}
              handleTcDict={this.getTeacherDict}
              handleSearch={this.search}
              handleReset={this.reset}
            />
            <Button
              type="primary"
              className="two-teacher-search-bar-btn"
              onClick={this.goToAddMainClass}
              disabled={btnPurviewAddMainClass}
            >
              新增主班
            </Button>
            <Button
              type="primary"
              className="two-teacher-export-btn"
              onClick={this.exportExcel}
              disabled={btnExportExcel}
            >
              导出Excel
            </Button>
            <div className="two-teacher-action-btn">
              <ButtonWrap
                btnText="批量修改"
                popPlacement="bottom"
                actions={[
                  {
                    text: '主讲老师',
                    onClick: () => {
                      this.batchAction({ actionType: 'mainTeacher' });
                    },
                  },
                  {
                    text: '直播间',
                    onClick: () => {
                      this.batchAction({ actionType: 'liveRoom' });
                    },
                  },
                ]}
                btnDisabled={selectedRowKeys.length === 0}
                btnType="default"
                btnStyle={{ width: '100%' }}
              />
            </div>
          </div>
        }
      >
        <List
          dataSource={classList}
          columns={this.columns(btnPurviewCourse, btnPurviewEdit, btnPurviewDelete)}
          loading={!loading && listLoading}
          rowSelection={{
            selectedRowKeys,
            // onChange: this.updateSelectedRowKeys,
            onSelect: this.updateSelect,
            onSelectAll: this.updateSelectAll,
          }}
          pagination={{
            current: pageCurrent,
            total: pageTotal,
            pageSize,
          }}
          onChange={this.pageChange}
        />
        {toggle_EditMainClass && (
          <EditMainClass
            id={editClassId}
            handleConfirm={this.editConfirm}
            handleCancel={this.editCancel}
            isModal={isModal}
          />
        )}
        {toggle_editMainTeacher && (
          <ModalEditMainTeacher
            params={this.batchEditTeacherParams}
            number={selectedRowKeys.length}
            onCancel={this.switchModalEditMainTc}
            onConfirm={this.submitEditMainTc}
          />
        )}
        {toggle_editLiveRoom && (
          <ModalEditLiveRoom
            number={selectedRowKeys.length}
            onCancel={this.switchModalEditLiveRoom}
            onConfirm={this.submitEditLiveRoom}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  let btnAuth = state.account.btnAuths;
  const btnPurviewAddMainClass = btnAuth.indexOf('addMainClassBtn') === -1;
  const btnPurviewCourse = btnAuth.indexOf('modifyCourseBtn') === -1;
  const btnPurviewEdit = btnAuth.indexOf('modifyMainClassBtn') === -1;
  const btnPurviewDelete = btnAuth.indexOf('deleteMainClassBtn') === -1;
  const btnExportExcel = btnAuth.indexOf('TWO_TEACHER_MAIN_EXCEL') === -1;
  return {
    btnAuth,
    btnPurviewAddMainClass,
    btnPurviewCourse,
    btnPurviewEdit,
    btnPurviewDelete,
    btnExportExcel,
    dictClassType: state.dict.classType ? state.dict.classType[TWO_TEACHER_VAL] : null,
    dictBranch: state.dict.branch,
    dictStudentGrade: state.dict.grade,
    dictSubject: state.dict.subject,
    dictCourseState: state.dict.courseStatus,
    dictProductType: state.dict.courseSeriesSingle ? state.dict.courseSeriesSingle[TWO_TEACHER_VAL] : null,
    dictYear: state.dict.year,
    dictQuarter: state.dict.quarter ? state.dict.quarter.filter((item) => item.text !== '全年') : null,
    dictPhase: state.dict.phase,
    dictWeekDay: state.dict.weekDay,
    dictProductCharacter: state.dict.productCharacter,
  };
};

export default connect(mapStateToProps)(TwoTeacherClass);
