/**
 * Created by Haolin<haolinhom@gmail.com> on 2019/3/13.
 */
import storeName from './constant';
import { getDictValByText } from 'UTIL/dict';
import { findTwoTeacherProductForChoose } from 'SERVICE/course';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { clone } from 'lodash';
// 模块内部方法
const _fn = {
  // 校验两对象下的对应key的value是否相同
  objsHadSameKeysVal(obj1, obj2, keyList) {
    return keyList.every((item) => {
      let keys = item.split('.');
      let checkObj1 = obj1;
      let checkObj2 = obj2;
      let key = null;
      do {
        key = keys.shift();
        checkObj1 = checkObj1[key];
        checkObj2 = checkObj2[key];
      } while (keys.length > 0 && checkObj1 && checkObj2);
      return checkObj1 === checkObj2;
    });
  },
};

export default {
  namespace: storeName,
  state: {
    /**
     * 新增主班(addMain)
     * */
    // 已选产品列表
    addMain_prdList: [],
    /**
     * 修改主班(editMain)
     * */
    // 已选产品列表
    editMain_originPrdList: [],
    editMain_prdList: [],
    isNeedResetMainClass: false,
    /**
     * 选择产品弹窗(addPrd)
     * */
    // 选择产品弹窗筛选项
    sKey_addPrd_courseTimeTotal: null,
    sKey_addPrd_branch: [],
    sKey_addPrd_year: null,
    sKey_addPrd_quarter: null,
    sKey_addPrd_character: [],
    sKey_addPrd_classType: [],
    sKey_addPrd_courseSeries: [],
    sKey_addPrd_grade: null,
    sKey_addPrd_subject: null,
    // 缓存-选择产品弹窗筛选项
    sKey_addPrd_initDate: null,
    // 产品列表
    addPrd_list: [],
    addPrd_total: 0,
    addPrd_current: 1,
    addPrd_pageSize: 20,
    // 已选产品列表
    addPrd_selectedList: [],
  },
  reducers: {
    // 更新state
    updateState(state, { payload }) {
      return {
        ...state,
        ...payload,
      };
    },
    // 更新待选产品列表
    updatePrdList(state, { payload: { list, total } }) {
      let addPrd_list = list.map((item) => ({
        product: {
          text: item.name,
          val: item.id,
        },
        branch: {
          text: item.branchName,
          val: item.branchIds,
        },
        year: {
          text: item.yearName,
          val: item.yearId,
        },
        quarter: {
          text: item.seasonName,
          val: item.seasonId,
        },
        character: {
          text: item.productCharacterName,
          val: item.productCharacter,
        },
        classType: {
          text: item.classTypeName,
          val: item.classTypeId,
        },
        courseSeries: {
          text: item.courseSeriesName,
          val: item.courseSeriesId,
        },
        grade: {
          text: item.gradeName,
          val: item.gradeId,
        },
        subject: {
          text: item.subjectName,
          val: item.subjectId,
        },
        totalHours: {
          text: item.totalHours,
          val: item.totalHours,
        },
        classTimeLength: {
          text: item.classTimeLength,
          val: item.classTimeLength,
        },
        price: {
          text: item.price,
          val: item.price,
        },
        gradeSegment: {
          text: item.gradeSegmentName,
          val: item.gradeSegmentId,
        },
        // 辅班数量，修改主班[editMain]时，当辅班数量 > 0 时不能删除已选产品，新增产品[addMain]时，默认值为0
        viceClassNum: 0,
      }));
      return {
        ...state,
        addPrd_list,
        addPrd_total: total,
      };
    },
    // 更新已选产品列表
    updateSelectedPrdList(state, { payload }) {
      let addPrd_selectedList = clone(state.addPrd_selectedList);
      addPrd_selectedList.push(clone(payload));
      return {
        ...state,
        addPrd_selectedList,
      };
    },
  },
  effects: {
    /**
     * 选择产品弹窗
     * */
    // 初始化筛选项
    *initAddPrdSKey({ payload: { callback } }, { select, put }) {
      const initData = yield select((state) => {
        return (
          state[storeName].sKey_addPrd_initDate || {
            sKey_addPrd_courseTimeTotal: null,
            sKey_addPrd_branch: [],
            sKey_addPrd_year: getDictValByText(state.dict.year, new Date().getFullYear().toString()),
            sKey_addPrd_quarter: null,
            sKey_addPrd_character: [],
            sKey_addPrd_classType: [],
            sKey_addPrd_courseSeries: [],
            sKey_addPrd_grade: null,
            sKey_addPrd_subject: null,
          }
        );
      });
      yield put({
        type: 'updateState',
        payload: {
          ...initData,
          sKey_addPrd_initDate: initData,
        },
      });
      if (callback) callback();
    },
    // 加载产品列表
    *loadProductList({ payload: { callback } }, { select, put }) {
      let apiParams = yield select((state) => ({
        totalHours: state[storeName].sKey_addPrd_courseTimeTotal,
        // 多选
        brenchIds: state[storeName].sKey_addPrd_branch.join(','),
        productVersionId: state[storeName].sKey_addPrd_year,
        seasonId: state[storeName].sKey_addPrd_quarter,
        // 多选
        productCharacter: state[storeName].sKey_addPrd_character.join(','),
        // 多选
        classTypeId: state[storeName].sKey_addPrd_classType.join(','),
        // 多选
        courseSeriesId: state[storeName].sKey_addPrd_courseSeries.join(','),
        gradeId: state[storeName].sKey_addPrd_grade,
        subjectIds: state[storeName].sKey_addPrd_subject,
        page: state[storeName].addPrd_current,
        rows: state[storeName].addPrd_pageSize,
      }));
      const rsp = yield findTwoTeacherProductForChoose(apiParams, true).catch((err) => {
        showMessage(err.text, 'error');
      });
      if (rsp && rsp.resultCode === 0) {
        yield put({
          type: 'updatePrdList',
          payload: {
            list: rsp.rows,
            total: rsp.records,
          },
        });
      }
      if (callback) callback();
    },
    // 初始化已选产品
    *initSelectedList({ payload: { type, callback } }, { select, put }) {
      const key = `${type}_prdList`;
      const list = yield select((state) => clone(state[storeName][key]));
      yield put({
        type: 'updateState',
        payload: { addPrd_selectedList: list },
      });
      if (callback) callback();
    },
    // 选择产品
    *selectProduct({ payload: { index, callback } }, { select, put }) {
      const prdList = yield select((state) => state[storeName].addPrd_list);
      const selectedList = yield select((state) => state[storeName].addPrd_selectedList);
      // 校验输入
      let checkResult = prdList[index];
      if (selectedList.length > 0) {
        // 1.校验prd是否在selectedList中(一个产品可以有多个分公司)
        if (
          selectedList.some(
            (item) => item.product.val === checkResult.product.val && item.branch.val === checkResult.branch.val,
          )
        ) {
          checkResult = '该产品已选，请勿重复选择！';
        }
        // 2.校验prd的branchId是否在selectedList中
        else if (selectedList.some((item) => item.branch.val === checkResult.branch.val)) {
          checkResult = '该分公司已选产品，无需重复添加！';
        }
        // 3.校验prd除branch的其他所有筛选条件是否与selectedList[0]相同
        // (科目不可作为判断依据。首先产品里的科目为可选项，其次建主班时科目是可以更改的)
        else if (
          !_fn.objsHadSameKeysVal(selectedList[0], checkResult, [
            'year.val',
            'quarter.val',
            'character.val',
            'classType.val',
            'courseSeries.val',
            'grade.val',
            'totalHours.val',
          ])
        ) {
          checkResult = '请选择年份、季度、产品性质、班级类型、产品系列、年级、总课时数相同的产品！';
        }
      }
      if (typeof checkResult === 'string') {
        showMessage(checkResult, 'error');
      } else {
        yield put({
          type: 'updateSelectedPrdList',
          payload: checkResult,
        });
      }
      if (callback) callback();
    },
    // 全选产品
    *selectAllProduct({ payload: { callback } }, { select, put }) {
      const prdList = yield select((state) => state[storeName].addPrd_list);
      const tempSelectedList = clone(yield select((state) => state[storeName].addPrd_selectedList));
      for (let index = 0; index < prdList.length; index++) {
        let selectedList = yield select((state) => state[storeName].addPrd_selectedList);
        // 校验输入
        let checkResult = prdList[index];
        if (selectedList.length > 0) {
          // 1.校验prd是否在selectedList中(一个产品可以有多个分公司)
          if (
            selectedList.some(
              (item) => item.product.val === checkResult.product.val && item.branch.val === checkResult.branch.val,
            )
          ) {
            checkResult = '该产品已选，请勿重复选择！';
          }
          // 2.校验prd的branchId是否在selectedList中
          else if (selectedList.some((item) => item.branch.val === checkResult.branch.val)) {
            checkResult = '该分公司已选产品，无需重复添加！';
          }
          // 3.校验prd除branch的其他所有筛选条件是否与selectedList[0]相同
          // (科目不可作为判断依据。首先产品里的科目为可选项，其次建主班时科目是可以更改的)
          else if (
            !_fn.objsHadSameKeysVal(selectedList[0], checkResult, [
              'year.val',
              'quarter.val',
              'character.val',
              'classType.val',
              'courseSeries.val',
              'grade.val',
              'totalHours.val',
            ])
          ) {
            checkResult = '请选择年份、季度、产品性质、班级类型、产品系列、年级、总课时数相同的产品！';
          }
        }
        if (typeof checkResult === 'string') {
          // showMessage(checkResult, 'error');
        } else {
          yield put({
            type: 'updateSelectedPrdList',
            payload: checkResult,
          });
        }
      }
      const finalSelectedList = clone(yield select((state) => state[storeName].addPrd_selectedList));
      if (tempSelectedList.length === finalSelectedList.length) {
        let status = 0;
        for (let a = 0; a < tempSelectedList.length; a++) {
          if (tempSelectedList[a].product.val === finalSelectedList[a].product.val) {
            status++;
          }
        }
        if (status === tempSelectedList.length) {
          showMessage('无可选产品', 'error');
        }
      }
      if (callback) callback();
    },
    // 删除产品
    *deleteProduct({ payload: { index, callback } }, { select, put }) {
      // 在组件[EditMainClass]内初始化数据时，会根据辅班数量初始化[editMain_prdList]中产品数据的viceClassNum，新增时viceClassNum默认为0
      let viceClassNum = yield select((state) => state[storeName].addPrd_selectedList[index].viceClassNum);
      if (viceClassNum === 0) {
        let selectedList = yield select((state) => clone(state[storeName].addPrd_selectedList));
        selectedList.splice(index, 1);
        yield put({
          type: 'updateState',
          payload: { addPrd_selectedList: selectedList },
        });
      } else {
        showMessage('不可删除主班下已建辅班的产品！', 'error');
      }
      if (callback) callback();
    },
    // 确认添加已选产品 (params type [string] 新增主班: addMain, 编辑主班: editMain)
    *confirmProduct({ payload: { type, callback } }, { select, put }) {
      const list = yield select((state) => clone(state[storeName].addPrd_selectedList));
      const key = `${type}_prdList`;
      let payload = { [key]: list };
      // 编辑主班时，当产品有更新时判断新的已选产品是否有全部更新，如果有则需重置主班信息
      if (type === 'editMain') {
        const oldPrdIdList = yield select((state) =>
          state[storeName].editMain_originPrdList.map((item) => item.product.val),
        );
        if (!list.some((item) => oldPrdIdList.includes(item.product.val))) {
          payload.isNeedResetMainClass = true;
        }
      }
      yield put({
        type: 'updateState',
        payload,
      });
      if (callback) callback();
    },
  },
};
