import React from 'react';
import Modal from 'COMPONENT/Modal';
import List from 'COMPONENT/List';
import Search from 'PAGE/course/ReEnroll/Search';
import { Row, Col } from 'antd';
import Select from 'COMPONENT/Select';
import styles from './index.less';

class SelectNewMiniClass extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      inputValue: '',
      campusId: null,
      classTypeIds: null,
      teacherId: '',
      loading: false,
    };
  }

  onSearch = async (selectedDatas) => {
    console.log('selectedDatas');
    console.log(selectedDatas);
    if (JSON.stringify(selectedDatas.campusId) === '[""]') {
      selectedDatas.campusId = null;
    }
    if (JSON.stringify(selectedDatas.classTypeIds) === '[""]') {
      selectedDatas.classTypeIds = null;
    }
    const { teacherId } = this.state;
    const { searchNewTwoTeacherClass } = this.props;
    let params = Object.assign(selectedDatas, { teacherId });
    this.setState({ loading: true });
    await searchNewTwoTeacherClass(params);
    this.setState(selectedDatas);
    this.setState({ loading: false });
  };

  reset = () => {
    this.setState({
      inputValue: '',
      campusId: null,
      classTypeIds: null,
      teacherId: '',
    });
  };

  render() {
    const { inputValue, campusId, classTypeIds, loading, teacherId } = this.state;
    const {
      changeTwoTeacherClassList,
      twoTeacherClassTypeDict,
      campusDict,
      searchNewTwoTeacherClass,
      closeSelectNewMiniClass,
      getNewTwoTeacherClass,
      teacherDict,
    } = this.props;
    const { dataSource, total, current } = changeTwoTeacherClassList;
    const columns = [
      { title: '班课名称', dataIndex: 'classTwoName', width: 148 },
      { title: '学习中心', dataIndex: 'campusName', width: 94 },
      { title: '老师', dataIndex: 'teacherName', width: 72 },
      { title: '开始上课时间', dataIndex: 'startDate', width: 105 },
      { title: '上课时间', dataIndex: 'classTime', width: 78 },
      { title: '计划招生数量', dataIndex: 'peopleQuantity' },
      // {title: '超额数量',dataIndex:'allowedExcess',width: 80},
      { title: '已招数量', dataIndex: 'realQuantity', width: 80 },
      { title: '班级类型', dataIndex: 'classTypeName', width: 80 },
      {
        title: '操作',
        width: 55,
        render: (currentValue, currentRow, index) => {
          return (
            <span
              className={styles.selectBtn}
              onClick={() => {
                getNewTwoTeacherClass(currentValue.classTwoId);
              }}
            >
              选择
            </span>
          );
        },
      },
    ];
    const pagination = {
      current,
      total,
    };
    const conditions = [
      { title: '授课老师', field: 'teacherId', dict: teacherDict, type: 'children' },
      {
        title: '所属学习中心',
        field: 'campusId',
        dict: campusDict,
        isMultiSelect: true,
      },
      {
        title: '班级类型',
        field: 'classTypeIds',
        dict: twoTeacherClassTypeDict,
        isMultiSelect: true,
      },
    ];
    const teacherSelect = [
      <div key="asdf" className="label">
        授课老师
      </div>,
      <div key="gaef" className="items">
        <Row style={{ marginTop: '10px' }}>
          <Col span={7}>
            <Select
              name="teacherId"
              value={teacherId}
              onChange={(teacherId) => {
                this.setState({ teacherId });
              }}
              searchPlaceholder="请输入老师名字搜索"
              dict={teacherDict}
              showSearch={true}
              allowClear={true}
            />
          </Col>
        </Row>
      </div>,
    ];
    const selectedDatas = {
      inputValue,
      teacherId,
      campusId,
      classTypeIds,
    };

    return (
      <Modal
        title="选择调入班"
        loading={loading}
        width="1097"
        okBtnShow={false}
        onClose={() => {
          closeSelectNewMiniClass();
        }}
      >
        <Search
          onInitFinish={() => {}}
          conditions={conditions}
          selectedDatas={selectedDatas}
          onSearch={(selectedDatas) => {
            this.onSearch(selectedDatas);
          }}
          isShowResult={false}
          onInputChange={(inputValue) => {
            this.setState({ inputValue });
          }}
          onReset={() => {
            this.reset();
          }}
        >
          <div className="clearfix">{teacherSelect}</div>
        </Search>
        <List
          dataSource={dataSource}
          columns={columns}
          pagination={pagination}
          onChange={(pagination) => {
            searchNewTwoTeacherClass(selectedDatas, pagination.current, pagination.pageSize);
          }}
        />
      </Modal>
    );
  }
}

export default SelectNewMiniClass;
