import React from 'react';
import { connect } from 'dva';
import Modal from 'COMPONENT/Modal';
import OldCourseNum from './OldCourseNum';
import NewClassInfo from './NewClassInfo';
import styles from './index.less';
import loaders from '../loaders';
import selectNewMiniClass from './SelectNewMiniClass';
import SelectNewMiniClass from './SelectNewMiniClass';
import moment from 'moment';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

class ChangeCourse extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      studentCourses: [], //原班课程
      selectedCourses: [], // 原班被选中课程
      changeTwoTeacherClassList: {}, // 学生调课可调入班列表
      isSelectNewMiniClass: false, // 是否显示调入班列表Modal
      twoTeacherClassTypeDict: {}, //小班类型字典
      loading: true,
      newCourseList: [], // 与原班课程相匹配的调入班课程
      newTwoTeacherClassInfo: {}, //调入班信息
      newTwoTeacherClassCourses: [], //调入班课程
    };
  }

  componentDidMount() {
    this.initContent();
  }

  initContent = async () => {
    const { classTwoId, studentId, courseNum, studentIds } = this.props;
    let studentCourses;
    if (studentIds.length === 0) {
      studentCourses = await loaders.loadStudentConflictCourse({ classTwoId, studentId });
    } else {
      studentCourses = await loaders.loadStudentConflictCourse({ classTwoId, studentId });
    }
    this.setState({ studentCourses, loading: false });
    studentCourses.forEach((course, index) => {
      if (course.courseNum === courseNum) {
        this.selectCourse(index);
      }
    });
  };

  // 原班课程选中
  selectCourse = (index) => {
    const { studentCourses } = this.state;
    studentCourses[index].selected = !studentCourses[index].selected;
    let selectedCourses = studentCourses.filter((course) => {
      return course.selected === true;
    });
    this.setState({
      studentCourses,
      selectedCourses,
    });
  };

  // 学生调课选择调入班
  selectNewMiniClass = async () => {
    const { classTwoId } = this.props;
    const { selectedCourses } = this.state;
    let courseIds = selectedCourses.map((course) => {
      return `${classTwoId}/${course.courseId}`;
    });
    let params = { courseIds, page: 1, rows: '10' };
    const [changeTwoTeacherClassList, twoTeacherClassTypeDict] = await Promise.all([
      loaders.loadChangeTwoTeacherClassList(params),
      loaders.loadSelectOption(),
    ]);
    this.setState({ changeTwoTeacherClassList, isSelectNewMiniClass: true, twoTeacherClassTypeDict });
  };

  // 搜索可调入小班
  searchNewTwoTeacherClass = async (selectedDatas, page = 1, rows = 10) => {
    const { campusId, classTypeIds, inputValue, teacherId } = selectedDatas;
    let searchParams = { campusId, classTypeIds, classTwoName: inputValue, teacherId };
    const { classTwoId } = this.props;
    const { selectedCourses } = this.state;
    let courseIds = selectedCourses.map((course) => {
      return `${classTwoId}/${course.courseId}`;
    });
    let params = { classTwoId, page, rows, sidx: 'createTime', sord: 'desc', ...searchParams, courseIds };
    params = Object.assign(params, selectedDatas);
    const changeTwoTeacherClassList = await loaders.loadChangeTwoTeacherClassList(params);
    this.setState({ changeTwoTeacherClassList });
  };

  // 关闭选择调入班模态框
  closeSelectNewMiniClass = () => {
    this.setState({ isSelectNewMiniClass: false });
  };

  // 获取调入班
  getNewTwoTeacherClass = async (classTwoId) => {
    const { studentCourses } = this.state;
    this.closeSelectNewMiniClass();
    this.setState({ loading: true });
    let newTwoTeacherClassCourses = await loaders.loadConflictCourseInfo({ classTwoId });
    let newTwoTeacherClassInfo = await loaders.loadConflictClassInfo({ classTwoId });
    this.juxtaposeCourses(studentCourses, newTwoTeacherClassCourses);
    this.setState({ loading: false, newTwoTeacherClassInfo: newTwoTeacherClassInfo[0], newTwoTeacherClassCourses });
  };

  // 匹配原班讲数和调入班讲数
  juxtaposeCourses = (studentCourses, newTwoTeacherClassCourses) => {
    const selectedCourses = studentCourses.filter((course) => {
      return course.selected === true;
    });
    let newCourseList = [];
    for (let i = 0; i < selectedCourses.length; i++) {
      for (let j = 0; j < newTwoTeacherClassCourses.length; j++) {
        if (selectedCourses[i].courseNum === newTwoTeacherClassCourses[j].courseNum) {
          newCourseList.push(newTwoTeacherClassCourses[j]);
        }
      }
    }
    newCourseList = newCourseList.map((course) => {
      return Object.assign(course, {
        courseEndTime: moment(course.courseDate)
          .add({
            hours: course.courseTime.slice(0, 2),
            minutes: parseInt(course.courseTime.slice(3)) + course.courseTimeLength,
          })
          .format()
          .slice(11, 16),
      });
    });
    this.setState({ newCourseList });
  };

  // 换课
  changeCourse = async (studentId, newCourseList, selectedCourses) => {
    const { newTwoTeacherClassInfo } = this.state;
    const { search, closeModal, studentIds, cancelMultiChangeCourses, classTwoId } = this.props;
    const { classTwoId: newClassTwoId } = newTwoTeacherClassInfo;
    let newCourseIds = newCourseList.map((course) => {
      return `${newClassTwoId}/${course.courseId}`;
    });
    let oldCourseIds = selectedCourses.map((course) => {
      return `${classTwoId}/${course.courseId}`;
    });
    let courseMap = [];
    for (let i = 0; i < oldCourseIds.length; i++) {
      courseMap.push({ newCourseId: newCourseIds[i], oldCourseId: oldCourseIds[i] });
    }
    let changeCourseVos =
      studentIds.length === 0
        ? [
            {
              studentId,
              courseMap,
            },
          ]
        : studentIds.map((studentId) => {
            return {
              studentId,
              courseMap,
            };
          });
    let params = {
      changeCourseVos,
    };
    let checkConflictParams = {
      studentId,
      courseIds: newCourseIds,
    };
    let response = await loaders.changeCourseApi(params);
    if (response.resultMessage === '调课成功') {
      let checkConflictResponse = await loaders.checkConflictAfterTtChangeCourseApi(checkConflictParams);
      if (checkConflictResponse.data === true) {
        showMessage(checkConflictResponse.resultMessage, 'success');
      } else {
        showMessage('调课成功', 'success');
      }
      closeModal();
      search();
    }
    cancelMultiChangeCourses();
  };

  render() {
    const {
      studentCourses,
      changeTwoTeacherClassList,
      isSelectNewMiniClass,
      twoTeacherClassTypeDict,
      loading,
      newCourseList,
      newTwoTeacherClassInfo,
      hasNewMiniClassInfo,
      newTwoTeacherClassCourses,
      selectedCourses,
    } = this.state;
    const { closeModal, campusDict, studentId, teacherDict } = this.props;
    return (
      <Modal
        title="调课"
        onCancel={closeModal}
        width="598"
        loading={loading}
        onOk={() => {
          this.changeCourse(studentId, newCourseList, selectedCourses);
        }}
      >
        <OldCourseNum
          studentCourses={studentCourses}
          changeTwoTeacherClassList={changeTwoTeacherClassList}
          selectCourse={this.selectCourse}
          newTwoTeacherClassCourses={newTwoTeacherClassCourses}
          juxtaposeCourses={this.juxtaposeCourses}
        />
        <NewClassInfo
          selectNewMiniClass={this.selectNewMiniClass}
          newCourseList={newCourseList}
          newTwoTeacherClassInfo={newTwoTeacherClassInfo}
          hasNewMiniClassInfo={hasNewMiniClassInfo}
        />
        {isSelectNewMiniClass && (
          <SelectNewMiniClass
            teacherDict={teacherDict}
            searchNewTwoTeacherClass={this.searchNewTwoTeacherClass}
            campusDict={campusDict}
            twoTeacherClassTypeDict={twoTeacherClassTypeDict}
            changeTwoTeacherClassList={changeTwoTeacherClassList}
            closeSelectNewMiniClass={this.closeSelectNewMiniClass}
            getNewTwoTeacherClass={this.getNewTwoTeacherClass}
          />
        )}
      </Modal>
    );
  }
}

export default connect()(ChangeCourse);
