import React from 'react';
import Modal from 'COMPONENT/Modal';
import List from 'COMPONENT/List';
import loaders from '../loaders';
import Select from 'COMPONENT/Select';
import DatePicker from 'COMPONENT/DatePicker';
import TimePicker from 'COMPONENT/TimePicker';
import styles from './index.less';
import moment from 'moment';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';

// const TeacherSelect = (twoClassTeacherDict, record, upDateRow,disable) => {
//   return (
//     <Select
//       disabled={disable}
//       showSearch={true}
//       allowClear={false}
//       dict={twoClassTeacherDict}
//       value={record.teacher.teacherId}
//       onChange={value => {
//         upDateRow("twoClassCourseDataSource", index, {
//           teacher: {
//             teacherName: twoClassTeacherDict.filter(teacher => {
//               return teacher.val === value;
//             })[0].text,
//             teacherId: value
//           }
//         });
//       }}
//     />
//   );
// }

class EditCourse extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      classTwoDataSource: [],
      twoClassCourseDataSource: [],
      title: '',
      error: null,
      border: styles.border,
      conflictCourseNum: [], // 原班冲突讲数
      twoClassTeacherDict: [], // 双师老师
      defaultTeacherId: '', // 辅班未更换前的老师
    };
  }

  componentDidMount() {
    this.initContent();
  }

  initContent = async () => {
    const { classTwoId, campusId } = this.props;
    this.setState({ loading: true });
    const [classTwoDataSource, twoClassCourseDataSource] = await Promise.all([
      loaders.loadConflictClassInfo({ classTwoId }),
      loaders.loadConflictCourseInfo({ classTwoId }),
    ]);
    const { subjectId, gradeId } = classTwoDataSource[0];
    const twoClassTeacherDict = await loaders.getTwoClassTeachersApi({ campusId, subjectId, gradeId });
    this.setState({
      loading: false,
      classTwoDataSource,
      twoClassCourseDataSource,
      title: classTwoDataSource[0].classTwoName,
      twoClassTeacherDict,
      defaultTeacherId: classTwoDataSource[0].teacher.teacherId,
    });
  };

  upDateRow = (dataSource, index, changed) => {
    const { courseNum } = this.props;
    const currentCourseNum =
      dataSource === 'twoClassCourseDataSource' ? this.state.twoClassCourseDataSource[index].courseNum : null;
    if (dataSource === 'twoClassCourseDataSource' && courseNum === currentCourseNum) {
      this.setState({ border: '' });
    }
    this.setState({
      [dataSource]: this.state[dataSource].map((row, i) => {
        if (index === i) {
          return { ...row, ...changed };
        }
        return row;
      }),
      border: '',
    });
  };

  // 检出修改过后的课程(未提交修改)
  getCoursesOfChange = (typeValue) => {
    const { twoClassCourseDataSource } = this.state;
    const { type, classTwoId, closeModal, search } = this.props;
    let courses = twoClassCourseDataSource
      .map((course) => {
        const { courseId, courseNum, courseTime, courseDate, courseTimeLength, courseStatus } = course;
        let hour = courseTime.slice(0, 2);
        let minute = courseTime.slice(3);
        let startTime = courseTime.replace(':', '-');
        // let endTime = moment(courseDate).set({'hour':hour,'minute': minute}).add(courseTimeLength,'minutes').format().slice(11,16).replace(':','-')
        return {
          // courseStatus,
          classTwoId,
          // courseId,
          // courseNum,
          // endTime,
          personOrRoomeId: course[typeValue][typeValue + 'Id'],
          // startDate: courseDate,
          // startTime,
          type,
        };
      })
      .filter((course) => {
        return course.courseStatus === 'NEW';
      });
    return courses;
  };

  onOk = async () => {
    const { twoClassCourseDataSource, classTwoDataSource, defaultTeacherId } = this.state;
    const { type, classTwoId, closeModal, search /* reloadTeacherDict*/ } = this.props;
    // let params = type === 'teacher' ? this.getCoursesOfChange('teacher'):this.getCoursesOfChange('classroom')
    let personOrRoomeId =
      type === 'teacher' ? classTwoDataSource[0].teacher.teacherId : classTwoDataSource[0].classroom.classroomId;
    // 未更改老师，直接关闭 modal
    if (defaultTeacherId === personOrRoomeId) {
      closeModal();
      return;
    }
    // params = JSON.parse(JSON.stringify(params))
    this.setState({ loading: true });
    let conflictCourseNum = await loaders.getEditConflictInfoApi({ type, personOrRoomeId, classTwoId });

    this.setState({ loading: false, conflictCourseNum, border: styles.border });
    if (conflictCourseNum.length === 0) {
      await loaders.editTeacherOrclassroomApi({ type, personOrRoomeId, classTwoId });
      closeModal();
      // if(type==='teacher'){
      //   await reloadTeacherDict(personOrRoomeId)
      // }
      search();
    } else {
      let tipString =
        type === 'teacher' ? '该老师仍存在排课冲突，是否继续修改？' : '该教室仍存在排课冲突，是否继续修改？';
      showConfirm(tipString, async () => {
        await loaders.editTeacherOrclassroomApi({ type, personOrRoomeId, classTwoId });
        closeModal();
        // if(type==='teacher'){
        //   await reloadTeacherDict(personOrRoomeId)
        // }
        search();
      });
    }
  };

  // 生成课程的时间选择器
  renderTimePicker = (record, index, conflictCourseNum, border, courseNum, disable) => {
    if (conflictCourseNum.includes(record.courseNum)) {
      return (
        <TimePicker
          disabled={true}
          className={border}
          value={record.courseTime}
          onChange={(value) => {
            this.upDateRow('twoClassCourseDataSource', index, {
              courseTime: value,
            });
          }}
        />
      );
    } else {
      return (
        <TimePicker
          disabled={true}
          value={record.courseTime}
          onChange={(value) => {
            this.upDateRow('twoClassCourseDataSource', index, {
              courseTime: value,
            });
          }}
        />
      );
    }
  };

  // 改变任课老师/教室
  changeClassTeacherOrClassRoom = (value, type) => {
    let { twoClassCourseDataSource, twoClassTeacherDict } = this.state;
    const { classroomDict } = this.props;
    if (type === 'teacher') {
      twoClassCourseDataSource = twoClassCourseDataSource.map((course) => {
        if (course.courseStatus === 'NEW') {
          return Object.assign(course, {
            teacher: {
              teacherName: twoClassTeacherDict.filter((teacher) => {
                return teacher.val === value;
              })[0].text,
              teacherId: value,
            },
          });
        } else {
          return course;
        }
      });
    } else {
      twoClassCourseDataSource = twoClassCourseDataSource.map((course) => {
        if (course.courseStatus === 'NEW') {
          return Object.assign(course, {
            classroom: {
              classroomName: classroomDict.filter((classroom) => {
                return classroom.val === value;
              })[0].text,
              classroomId: value,
            },
          });
        } else {
          return course;
        }
      });
    }

    this.setState(twoClassCourseDataSource);
  };

  render() {
    const {
      loading,
      twoClassTeacherDict,
      twoClassCourseDataSource,
      title,
      error,
      border,
      conflictCourseNum,
      classTwoDataSource,
    } = this.state;
    // let classTwoName = classTwoDataSource[0].classTwoName
    const { closeModal, type, classroomDict, courseNum } = this.props;

    let classTwoColumns = null;
    let hasNotNewCourse = twoClassCourseDataSource.filter((course) => {
      return course.courseStatus !== 'NEW';
    });
    if (type === 'teacher') {
      classTwoColumns = [
        {
          title: '任课老师',
          dataIndex: 'teacher',
          render: (text, record, index) => {
            let disabled = hasNotNewCourse.length === 0 ? false : true;
            return (
              <Select
                disabled={disabled}
                allowClear={false}
                showSearch={true}
                dict={twoClassTeacherDict}
                value={record.teacher.teacherId}
                onChange={(value) => {
                  this.changeClassTeacherOrClassRoom(value, type);
                  this.upDateRow('classTwoDataSource', index, {
                    teacher: {
                      teacherName: twoClassTeacherDict.filter((teacher) => {
                        return teacher.val === value;
                      })[0].text,
                      teacherId: value,
                    },
                  });
                }}
              />
            );
          },
        },
        { title: '教室', dataIndex: 'classroomName' },
        { title: '科目', dataIndex: 'subject' },
        { title: '年级', dataIndex: 'grade' },
        { title: '课时数/讲', dataIndex: 'everyCourseNum' },
        { title: '课时时长', dataIndex: 'classTimeLength', width: 100 },
      ];
    } else {
      classTwoColumns = [
        {
          title: '任课老师',
          dataIndex: 'teacherName',
        },
        {
          title: '教室',
          dataIndex: 'classroom',
          render: (text, record, index) => {
            let disabled = hasNotNewCourse.length === 0 ? false : true;
            return (
              <Select
                disabled={disabled}
                allowClear={false}
                showSearch={true}
                dict={classroomDict}
                value={record.classroom.classroomId}
                onChange={(value) => {
                  this.changeClassTeacherOrClassRoom(value, type);
                  this.upDateRow('classTwoDataSource', index, {
                    classroom: {
                      classroomName: classroomDict.filter((classroom) => {
                        return classroom.val === value;
                      })[0].text,
                      classroomId: value,
                    },
                  });
                }}
              />
            );
          },
        },
        { title: '科目', dataIndex: 'subject' },
        { title: '年级', dataIndex: 'grade' },
        { title: '课时数/讲', dataIndex: 'everyCourseNum' },
        { title: '课时时长', dataIndex: 'classTimeLength', width: 100 },
      ];
    }

    let miniCourseColumns = null;
    if (type === 'teacher') {
      miniCourseColumns = [
        { title: '序号', dataIndex: 'courseNum', width: 60 },
        {
          title: '老师',
          dataIndex: 'teacher',
          render: (text, record, index) => {
            let disable = record.courseStatus !== 'NEW' ? true : false;
            return (
              <Select
                disabled={true}
                showSearch={true}
                allowClear={false}
                dict={twoClassTeacherDict}
                value={record.teacher.teacherId}
                onChange={(value) => {
                  this.upDateRow('twoClassCourseDataSource', index, {
                    teacher: {
                      teacherName: twoClassTeacherDict.filter((teacher) => {
                        return teacher.val === value;
                      })[0].text,
                      teacherId: value,
                    },
                  });
                }}
              />
            );
          },
        },
        {
          title: '日期',
          dataIndex: 'courseDate',
          width: 180,
          render: (text, record, index) => {
            let disable = record.courseStatus !== 'NEW' ? true : false;
            return (
              <DatePicker
                disabled={true}
                value={record.courseDate}
                onChange={(value) => {
                  this.upDateRow('twoClassCourseDataSource', index, {
                    courseDate: value,
                  });
                }}
              />
            );
          },
        },
        {
          title: '时间',
          dataIndex: 'courseTime',
          render: (text, record, index) => {
            let disable = record.courseStatus !== 'NEW' ? true : false;
            return this.renderTimePicker(record, index, conflictCourseNum, border, courseNum, disable);
          },
        },
      ];
    } else {
      miniCourseColumns = [
        { title: '序号', dataIndex: 'courseNum', width: 60 },
        {
          title: '教室',
          dataIndex: 'classroom',
          render: (text, record, index) => {
            let disable = record.courseStatus !== 'NEW' ? true : false;
            return (
              <Select
                disabled={true}
                showSearch={true}
                allowClear={false}
                dict={classroomDict}
                value={record.classroom.classroomId}
                onChange={(value) => {
                  this.upDateRow('twoClassCourseDataSource', index, {
                    classroom: {
                      classroomName: classroomDict.filter((classroom) => {
                        return classroom.val === value;
                      })[0].text,
                      classroomId: value,
                    },
                  });
                }}
              />
            );
          },
        },
        {
          title: '日期',
          dataIndex: 'courseDate',
          render: (text, record, index) => {
            let disable = record.courseStatus !== 'NEW' ? true : false;
            return (
              <DatePicker
                disabled={true}
                allowClear={false}
                value={record.courseDate}
                onChange={(value) => {
                  this.upDateRow('twoClassCourseDataSource', index, {
                    courseDate: value,
                  });
                }}
              />
            );
          },
        },
        {
          title: '时间',
          dataIndex: 'courseTime',
          render: (text, record, index) => {
            let disable = record.courseStatus !== 'NEW' ? true : false;
            return this.renderTimePicker(record, index, conflictCourseNum, border, courseNum, disable);
          },
        },
      ];
    }

    return (
      <Modal title={title} width="759" error={error} loading={loading} onCancel={closeModal} onOk={this.onOk}>
        <div>
          <List dataSource={classTwoDataSource} columns={classTwoColumns} />
          <List dataSource={twoClassCourseDataSource} columns={miniCourseColumns} />
        </div>
      </Modal>
    );
  }
}

export default EditCourse;
