import React from 'react';
import { connect } from 'dva';
import styles from './index.less';
import moment from 'moment';

const courseStatusDict = {
  NEW: '未上课',
  CHARGED: '准时上课',
  TEACHER_ATTENDANCE: '已考勤',
};
const courseStatusDictOfStudent = {
  ABSENT: '缺勤',
  CONPELETE: '准时上课',
  LATE: '迟到',
  NEW: '未开课',
};
class Hovercard extends React.Component {
  constructor(props) {
    super(props);
    this.state = {};
  }

  // 判断是否有调课权限
  hasChangeCourseAuth = () => {
    const { btnAuths } = this.props;
    return btnAuths.includes('TWO_TEACHER_CHANGE_COURSE');
  };

  // 判断是否有编辑权限
  hasEditAuth = () => {
    const { btnAuths } = this.props;
    return btnAuths.includes('TWO_TEACHER_EDIT');
  };

  render() {
    const { node, type, editCourse, changeCourse } = this.props;
    let courseDate;
    if (Array.isArray(node) === false) {
      courseDate = node.courseDate;
    } else {
      courseDate = node[0].courseDate;
    }
    let weekDate = moment(courseDate).day();
    let style =
      weekDate === 0 || weekDate === 6 ? { top: '0', right: '0', zIndex: '1' } : { top: '0', left: '0', zIndex: '1' };

    if (Array.isArray(node) === false) {
      let {
        courseNum,
        courseStatus,
        courseTime,
        courseEndTime,
        classTwoName,
        blCampusName,
        classroom,
        teacherName,
        classTwoId,
      } = node;
      let courseStatusText =
        type !== 'student' ? courseStatusDict[courseStatus] : courseStatusDictOfStudent[courseStatus];
      style =
        moment(courseTime).format().slice(11, 13) >= 16 ? Object.assign(style, { bottom: '0', top: undefined }) : style;
      let year = moment(courseTime).year();
      let month = moment(courseTime).month() + 1;
      let day = moment(courseTime).format().slice(8, 10);
      courseTime = moment(courseTime).format().slice(11, 16);
      courseEndTime = moment(courseEndTime).format().slice(11, 16);
      let isNewDom =
        courseStatus === 'NEW' ? (
          type === 'student' ? (
            this.hasChangeCourseAuth() === true ? (
              <div
                className={styles.changeCourse}
                onClick={() => {
                  changeCourse(classTwoId, courseNum);
                }}
              >
                调课
              </div>
            ) : null
          ) : this.hasEditAuth() === true ? (
            <div
              className={styles.changeCourse}
              onClick={() => {
                editCourse(classTwoId, courseNum);
              }}
            >
              编辑
            </div>
          ) : null
        ) : (
          <div className={styles.courseStatus}>{courseStatusText}</div>
        );
      return (
        <div className={styles.hoverCard + ' ' + 'hoverCard'} style={style}>
          <div className={styles.header}>
            <div className={styles.time}>
              <div className={styles.date}>
                <div className={styles.day}>{day}</div>
                <div className={styles.yearMonth}>
                  <p>{month}月</p>
                  <p>{year}</p>
                </div>
              </div>
              <div className={styles.hour}>
                {courseTime}-{courseEndTime}
              </div>
            </div>
            {/* <div className={styles.courseStatus}>
              {courseStatusText}
            </div> */}
            {isNewDom}
          </div>
          <div className={styles.content}>
            <h1>
              {classTwoName} {courseNum}
            </h1>
            <dl>
              <dt>上课地点：</dt>
              <dd>
                {blCampusName}
                {`(${classroom})`}
              </dd>
              <dt>任课教师：</dt>
              <dd>{teacherName}</dd>
            </dl>
          </div>
        </div>
      );
    } else {
      return (
        <div
          className={styles.hoverCard + ' ' + 'hoverCard'}
          style={Object.assign(style, { padding: '23px 24px 27px 25px' })}
        >
          <h1 className={styles.title}>冲突详情</h1>
          {node.map((course, index) => {
            let {
              courseNum,
              courseStatus,
              courseTime,
              courseEndTime,
              classTwoName,
              courseDate,
              classTwoId,
              blCampusName,
              classroom,
            } = course;
            let courseStatusText = courseStatusDict[courseStatus];
            style =
              moment(courseTime).format().slice(11, 13) >= 16
                ? Object.assign(style, { bottom: '0', top: undefined })
                : style;
            courseTime = moment(courseTime).format().slice(11, 16);
            courseEndTime = moment(courseEndTime).format().slice(11, 16);
            return (
              <div className={styles.course} key={index}>
                <div className={styles.courseInfo}>
                  <div className={styles.classTwoName}>
                    {classTwoName} {courseNum}
                    {`(${blCampusName}${classroom})`}
                  </div>
                  {courseStatus === 'NEW' && type === 'student' && this.hasChangeCourseAuth() === true && (
                    <div
                      className={styles.changeCourse}
                      onClick={() => {
                        changeCourse(classTwoId, courseNum);
                      }}
                    >
                      调课
                    </div>
                  )}
                  {courseStatus === 'NEW' && type !== 'student' && this.hasEditAuth() === true && (
                    <div
                      className={styles.changeCourse}
                      onClick={() => {
                        editCourse(classTwoId, courseNum);
                      }}
                    >
                      编辑
                    </div>
                  )}
                  {courseStatus !== 'NEW' && type === 'student' && (
                    <div className={styles.courseStatus}>{courseStatusDictOfStudent[courseStatus]}</div>
                  )}
                  {courseStatus !== 'NEW' && type !== 'student' && (
                    <div className={styles.courseStatus}>{courseStatusText}</div>
                  )}
                </div>
                <div className={styles.courseTime}>
                  <span className={styles.courseDate}>{courseDate}</span>
                  <span className={styles.courseHour}>
                    {courseTime}-{courseEndTime}
                  </span>
                </div>
              </div>
            );
          })}
        </div>
      );
    }
  }
}

let mapStateToProps = (state) => {
  return {
    btnAuths: state.account.btnAuths,
  };
};
export default connect(mapStateToProps)(Hovercard);
