import React from 'react';
import utils from '../utils';
import styles from './index.less';
import moment from 'moment';
import Node from './Node';

class SchedulePanel extends React.Component {
  constructor(props) {
    super(props);
    this.state = {};
  }

  divideConflictCourses = (conflictCourses) => {
    if (conflictCourses.length <= 1) {
      return conflictCourses;
    }

    conflictCourses = conflictCourses.sort((a, b) =>
      a.courseTime !== b.courseTime ? a.courseTime - b.courseTime : a.courseEndTime - b.courseEndTime,
    );

    let result = [];
    let beginIndex = 0;

    for (let i = 1; i < conflictCourses.length; i += 1) {
      if (conflictCourses[i].courseTime > conflictCourses[i - 1].courseEndTime) {
        result.push(conflictCourses.slice(beginIndex, i));
        beginIndex = i;
      }
    }

    if (beginIndex !== conflictCourses.length) {
      result.push(conflictCourses.slice(beginIndex, conflictCourses.length));
    }

    return result;
  };

  // 每一天的课程分块
  getNoteList = (courses) => {
    let tempArray = [];
    for (let i = 0; i < courses.length; i++) {
      let array = [courses[i]];
      for (let j = 0; j < courses.length; j++) {
        if (courses[i].courseEndTime <= courses[j].courseTime) {
          continue;
        } else if (courses[i].courseTime >= courses[j].courseEndTime) {
          continue;
        } else {
          array.push(courses[j]);
        }
      }
      tempArray.push(array);
    }
    // 没有冲突的课程
    const noConconflictCourses = tempArray
      .filter((note) => {
        return note.length === 2;
      })
      .map((array) => {
        return array.pop();
      });
    // 存在冲突的课程
    const conconflictCourses = courses.filter((course) => {
      return noConconflictCourses.includes(course) === false;
    });
    // 将存在冲突的课程分块
    let noteList = this.divideConflictCourses(conconflictCourses).concat(noConconflictCourses);
    return noteList;
  };

  render() {
    const panelTimes = utils.getPanelTimes();
    const { courseSchedules, type, editCourse, changeCourse } = this.props; // courseSchedules 是一个包含7天课程的数组，7个项分别是7个数组
    const timeLungime = new Date('2018-07-17T22:00:00').getTime() - new Date('2018-07-17T06:00:00').getTime();
    return (
      <div className={styles.wrap} style={{ height: 'calc(100% - 96px)' }}>
        <div className={styles.main}>
          <div className={styles.timeLine}>
            {panelTimes.map((time, i) => {
              return (
                <div key={time} className={styles.time}>
                  {time}
                </div>
              );
            })}
          </div>
          <div className={styles.content}>
            {courseSchedules.map((oneDayCourses, index) => {
              let nodeList = this.getNoteList(oneDayCourses);
              return (
                <div className={styles.col} key={index}>
                  {nodeList.map((node, index) => {
                    if (Array.isArray(node) === true) {
                      let minCourseTime = moment(
                        Math.min(
                          ...node.map((course) => {
                            return course.courseTime;
                          }),
                        ),
                      );
                      let maxCourseEndTime = moment(
                        Math.max(
                          ...node.map((course) => {
                            return course.courseEndTime;
                          }),
                        ),
                      );
                      let height = (maxCourseEndTime.diff(minCourseTime) / timeLungime) * 568;
                      let top =
                        (minCourseTime.diff(
                          moment(maxCourseEndTime).set({ hour: 6, minute: 0, second: 0, millisecond: 0 }).valueOf(),
                        ) /
                          timeLungime) *
                        568;
                      return (
                        <Node
                          top={top}
                          height={height}
                          node={node}
                          key={index}
                          type={type}
                          editCourse={editCourse}
                          changeCourse={changeCourse}
                        ></Node>
                      );
                    } else {
                      let minCourseTime = node.courseTime;
                      let maxCourseEndTime = node.courseEndTime;
                      let height = ((maxCourseEndTime - minCourseTime) / timeLungime) * 568;
                      let top =
                        ((minCourseTime -
                          moment(maxCourseEndTime).set({ hour: 6, minute: 0, second: 0, millisecond: 0 }).valueOf()) /
                          timeLungime) *
                        568;
                      return (
                        <Node
                          top={top}
                          height={height}
                          node={node}
                          key={index}
                          type={type}
                          editCourse={editCourse}
                          changeCourse={changeCourse}
                        ></Node>
                      );
                    }
                  })}
                </div>
              );
            })}
          </div>
          <div className={styles.right}></div>
        </div>
      </div>
    );
  }
}

export default SchedulePanel;
