import React from 'react';
import { connect } from 'dva';
import styles from './index.less';
import loaders from './loaders';
import Page from 'COMPONENT/Page';
import SearchBar from 'COMPONENT/SearchBar';
import Button from 'COMPONENT/Button';
import DatePicker from 'COMPONENT/DatePicker';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import ConflictList from './ConflictList';
import ScheduleHeader from './ScheduleHeader';
import SchedulePanel from './SchedulePanel';
import moment from 'moment';
import { DATE_FORMAT, TIME_FORMAT } from 'COMPONENT/utils/form';
import ShowMessage from 'COMPONENT/utils/globalTips/showMessage';
import DefaultTip from './DefaultTip';
import EditCourse from './EditCourse';
import Change from './ChangeCourse';
import ChangeCourse from './ChangeCourse';
import DropdownMenu from 'COMPONENT/DropdownMenu';
// import { connect } from 'http2';

const initSearchKeys = {
  date: moment().format().slice(0, 10),
  campusId: '',
  type: null,
  classScheduleTypeValue: null,
  startDate: moment().startOf('week').format().slice(0, 10),
  endDate: moment().endOf('week').format().slice(0, 10),
  courseStatus: ['NEW', 'CHARGED', 'TEACHER_ATTENDANCE'],
  hasCourseSchedules: false,
  classScheduleTypeValueDict: null,
  hasConflictName: false,
  isNotData: false,
};
class ScheduleManage extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: true,
      campusDict: null,
      classScheduleTypeDict: null,
      filterConditionDict: null,
      classScheduleTypeValueDict: null,
      teacherDict: null,
      studentDict: null,
      classroomDict: null,
      date: moment().format().slice(0, 10),
      campusId: null, // {val:'',text:''},学习中心
      type: null,
      classScheduleTypeValue: null,
      courseStatus: ['NEW', 'CHARGED', 'TEACHER_ATTENDANCE'],
      title: null,
      list: [],
      startDate: moment().startOf('week').format().slice(0, 10),
      endDate: moment().endOf('week').format().slice(0, 10),
      courseSchedules: [],
      hasCourseSchedules: false,
      hasConflictName: false,
      dateArray: [], //一周7天的日期数组 如 2018-08-02
      current: null, //当前查询课表的归属者
      checking: false,
      errors: {},
      exportStatus: '导出冲突名单',
      isEditCourse: false,
      classTwoId: '',
      courseNum: '',
      isNotData: false,
      isChangeCourse: false, // 是否正在调课(学生)
      studentIds: [], // 为[]则非批量调课
    };
  }

  // errors = {}//搜索参数校验

  /**
   * 生命周期钩子
   */
  componentWillMount() {}
  componentDidMount() {
    this.initContent();
  }
  componentWillUnmount() {}

  initContent = async () => {
    const { dispatch, pageParams } = this.props;
    const campusDict = await Promise.resolve(loaders.loadCampusDict({ orgType: 'CAMPUS' }));
    const [classScheduleTypeDict, filterConditionDict] = await Promise.all([
      loaders.loadClassScheduleTypeDict(),
      loaders.loadFilterConditionDict(),
    ]);
    this.setState({
      campusDict,
      classScheduleTypeDict,
      filterConditionDict,
      loading: false,
    });
    if (pageParams === undefined || pageParams.isMultiChangeCourse === undefined) {
      this.setState({ isChangeCourse: false });
    } else {
      const { studentIds, classTwoId, isMultiChangeCourse } = pageParams;
      this.setState({ isChangeCourse: true, classTwoId, classScheduleTypeValue: studentIds[0], studentIds });
      dispatch({
        type: 'main/setPageParams',
        payload: {
          path: '/twoTeacherCourseManage',
          params: null,
        },
      });
    }
  };

  // 选中学习中心改变
  changeCampus = async (value) => {
    const { startDate, endDate, type } = this.state;
    this.setState({
      campusId: '',
      // type: null,
      classScheduleTypeValue: null,
      courseStatus: ['NEW', 'CHARGED', 'TEACHER_ATTENDANCE'],
      hasCourseSchedules: false,
      classScheduleTypeValueDict: null,
      hasConflictName: false,
      isNotData: false,
    });
    if (value) {
      this.setState({ loading: true, campusId: value });
      const [classroomDict, studentDict, teacherDict] = await Promise.all([
        loaders.loadClassroomDict({ campusId: value }),
        loaders.loadStudentDict({ blCampusId: value, endDate, startDate }),
        loaders.loadTeacherDict({ campusId: value, startDate, endDate }),
      ]);
      this.changeClassScheduleTypeValueDict(type, teacherDict, studentDict, classroomDict);
      this.setState({
        loading: false,
        classroomDict,
        studentDict,
        teacherDict,
      });
    } else {
      this.setState({ campusId: value });
    }
  };

  // 课表类型与课表类型值 级联
  onChangeclassScheduleTypeValue = (value) => {
    if (value === 'teacher') {
      this.setState({ classScheduleTypeValueDict: this.state.teacherDict });
    } else if (value === 'student') {
      this.setState({ classScheduleTypeValueDict: this.state.studentDict });
    } else if (value === 'classroom') {
      this.setState({ classScheduleTypeValueDict: this.state.classroomDict });
    } else {
      this.setState({ classScheduleTypeValueDict: null });
    }
    this.setState({ type: value, isNotData: false });
  };

  // 改变日期
  dateChange = async (value) => {
    const { campusId, type, classroomDict } = this.state;
    let startDate = moment(value, DATE_FORMAT).startOf('week').format().slice(0, 10);
    let endDate = moment(value, DATE_FORMAT).endOf('week').format().slice(0, 10);
    this.setState({ date: value, startDate, endDate });
    if (campusId) {
      this.setState({ loading: true });
      const [studentDict, teacherDict] = await Promise.all([
        loaders.loadStudentDict({ blCampusId: campusId, endDate, startDate }),
        loaders.loadTeacherDict({ campusId: campusId, startDate, endDate }),
      ]);
      this.changeClassScheduleTypeValueDict(type, teacherDict, studentDict, classroomDict);
      this.setState({ studentDict, teacherDict, loading: false });
    }
  };

  // 搜索冲突名单
  searchConflictNames = async () => {
    let { campusId, courseStatus, endDate, startDate, type } = this.state;
    courseStatus =
      courseStatus === null
        ? null
        : courseStatus
            .reduce((preValue, value) => {
              return preValue + value + ',';
            }, '')
            .slice(0, -1);
    let params = {
      campusId: campusId,
      courseStatus: courseStatus,
      endDate: endDate,
      startDate,
      type: type,
    };
    let conflictNames = await loaders.loadConflictName(params);
    if (!conflictNames) {
      this.setState({ hasConflictName: false, isNotData: true });
    } else {
      let { conflictList } = conflictNames;
      let title;
      switch (conflictNames.type) {
        case 'student':
          title = '学生冲突名单:';
          break;
        case 'teacher':
          title = '老师冲突名单:';
          break;
        case 'classroom':
          title = '教室冲突名单:';
          break;
        default:
          '教室冲突名单:';
          break;
      }
      this.setState({
        title: title,
        hasConflictName: true,
        list: conflictList,
      });
    }
  };

  // 冲突名单点击搜索
  onClickName = async (id) => {
    await this.setState({ classScheduleTypeValue: id, current: id });
    this.searchConflictCourse();
  };

  // // 辅班老师更换之后重新获取老师字典
  // reloadTeacherDict = async (teacherId) =>{
  //   const { campusId, startDate, endDate } = this.state
  //   const teacherDict = await loaders.loadTeacherDict({ campusId, startDate, endDate })
  //   this.setState({ teacherDict})
  // }

  // 搜索冲突课程
  searchConflictCourse = async () => {
    let { campusId, courseStatus, endDate, startDate, classScheduleTypeValue, type } = this.state;
    if (!classScheduleTypeValue) {
      this.setState({ hasCourseSchedules: false });
      return;
    }
    courseStatus =
      courseStatus === null
        ? courseStatus
        : courseStatus
            .reduce((preValue, value) => {
              return preValue + value + ',';
            }, '')
            .slice(0, -1);
    let params = {
      campusId,
      courseStatus,
      endDate,
      startDate,
      type,
      personOrRoomeId: classScheduleTypeValue,
    };
    this.setState({ loading: true });
    let data = await loaders.loadCoursesSchedules(params);
    let { courseSchedules, dateArray } = data;
    this.setState({ courseSchedules, hasCourseSchedules: true, dateArray, loading: false });
  };

  // 搜索回调
  search = async () => {
    const { checking, errors } = this.state;
    if (!checking) {
      this.setState({ checking: true });
    }
    if (Object.values(errors).find((v) => v)) {
      this.setState({ hasConflictName: false, hasCourseSchedules: false });
      return;
    }

    this.setState({ loading: true });
    await Promise.all([this.searchConflictNames(), this.searchConflictCourse()]);
    this.setState({ loading: false });
  };

  // 更改课表类型值字典classScheduleTypeValueDict
  changeClassScheduleTypeValueDict = (type, teacherDict, studentDict, classroomDict) => {
    switch (type) {
      case 'teacher':
        this.setState({ classScheduleTypeValueDict: teacherDict });
        break;
      case 'student':
        this.setState({ classScheduleTypeValueDict: studentDict });
        break;
      case 'classroom':
        this.setState({ classScheduleTypeValueDict: classroomDict });
        break;
    }
  };

  // 改变时间并搜索冲突课程
  dateChangeAndSearchConflictCourse = async (date) => {
    let { campusId, type, classroomDict } = this.state;
    let startDate = moment(date, DATE_FORMAT).startOf('week').format().slice(0, 10);
    let endDate = moment(date, DATE_FORMAT).endOf('week').format().slice(0, 10);
    if (campusId) {
      const [studentDict, teacherDict] = await Promise.all([
        loaders.loadStudentDict({ blCampusId: campusId, endDate, startDate }),
        loaders.loadTeacherDict({ campusId: campusId, startDate, endDate }),
      ]);
      this.changeClassScheduleTypeValueDict(type, teacherDict, studentDict, classroomDict);
      this.setState({ teacherDict, studentDict, date: date, startDate, endDate }, this.search);
    } else {
      this.setState({ date: date, startDate, endDate }, this.search);
    }
  };

  // 搜索上一周课程
  onClickPreWeek = () => {
    let { date } = this.state;
    this.dateChangeAndSearchConflictCourse(moment(date).subtract(7, 'days').format().slice(0, 10));
  };

  // 搜索下一周课程
  onClickNextWeek = () => {
    let { date } = this.state;
    this.dateChangeAndSearchConflictCourse(moment(date).add(7, 'days').format().slice(0, 10));
  };

  // 修改课表类型联动的值(老师、教室、学生)
  onChangeClassScheduleTypeValue = (value) => {
    let { classScheduleTypeValueDict } = this.state;
    let current = value === null ? null : value;
    this.setState({ classScheduleTypeValue: value, current });
  };
  // 重置
  reset = () => {
    this.setState(initSearchKeys);
    this.setState((preState) => {
      return {
        checking: false,
      };
    });
  };

  // 采集校验信息进 errors 对象
  errorChange = (key, error) => {
    this.setState((preState) => {
      const { errors } = preState;
      return {
        errors: {
          ...errors,
          [key]: error,
        },
      };
    });
  };

  // 导出冲突名单
  exportExcel = async () => {
    let { campusId, campusDict, courseStatus, endDate, startDate, type, list } = this.state;
    let yearName = moment(startDate).format().slice(0, 4);
    let campusName = campusDict.filter((campus) => {
      return campus.val === campusId;
    })[0].text;
    let title = type === 'teacher' ? '老师' : type === 'student' ? '学生' : '教室';
    let fileName = yearName + campusName + title + '冲突名单';
    let personOrRoomeId = list
      .reduce((preValue, person) => {
        return preValue + person.id + ',';
      }, '')
      .slice(0, -1);
    courseStatus = courseStatus
      .reduce((preValue, value) => {
        return preValue + value + ',';
      }, '')
      .slice(0, -1);
    let params = JSON.stringify({
      campusId,
      courseStatus,
      endDate,
      startDate,
      type,
      personOrRoomeId,
      yearName,
      campusName,
    });
    this.setState({ exportStatus: '名单导出中...' });
    loaders.exportExcel(params, fileName).then(
      () => {
        this.setState({ exportStatus: '导出冲突名单' });
      },
      (error) => {
        ShowMessage(error);
      },
    );
  };

  // 老师/教室编辑课程
  editCourse = (classTwoId, courseNum) => {
    this.setState({ isEditCourse: true, classTwoId, courseNum });
  };

  // 学生调课
  changeCourse = (classTwoId, courseNum) => {
    this.setState({ isChangeCourse: true, classTwoId, courseNum });
  };

  // 导出课程明细
  exportCourseInfoBycampusId = async () => {
    const { campusId, campusDict, courseStatus, endDate, startDate, type, errors, checking } = this.state;
    if (!checking) {
      this.setState({ checking: true });
    }
    if (Object.values(errors).find((v) => v)) {
      return;
    }
    const campusName = campusDict.filter((campus) => {
      return campus.val === campusId;
    })[0].text;
    let params = {
      campusId,
      campusName,
      courseStatus,
      endDate,
      startDate,
      type,
    };
    window.open(
      `/eduboss/TwoTeacherClassController/exportTwoTeacherCourseInfoBycampusId.do?campusId=${campusId}&campusName=${campusName}&courseStatus=${courseStatus}&endDate=${endDate}&startDate=${startDate}&type=${type}`,
    );
  };

  // 判断是否有导出冲突名单权限
  hasExportConflictAuth = () => {
    const { btnAuths } = this.props;
    return btnAuths.includes('TWO_TEACHER_EXPORT_EXCEL');
  };

  // 判断是否有导出课程明细权限
  hasExportCourseAuth = () => {
    const { btnAuths } = this.props;
    return btnAuths.includes('TWO_TEACHER_EXPORT_COURSES');
  };

  render() {
    const {} = this.props;
    const { title, list, loading } = this.state;
    const {
      date,
      type,
      campusDict,
      classScheduleTypeDict,
      filterConditionDict,
      classScheduleTypeValueDict,
      classScheduleTypeValue,
      campusId,
      courseSchedules,
      hasCourseSchedules,
      hasConflictName,
      courseStatus,
      dateArray,
      current,
      checking,
      errors,
      exportStatus,
      isEditCourse,
      classTwoId,
      teacherDict,
      classroomDict,
      courseNum,
      isNotData,
      isChangeCourse,
      studentIds,
    } = this.state;

    let menus = [];
    if (this.hasExportConflictAuth() === true) {
      menus.push({ text: exportStatus, disabled: !hasConflictName, onClick: this.exportExcel });
    }
    if (this.hasExportCourseAuth() === true) {
      menus.push({ text: '导出课程明细', onClick: this.exportCourseInfoBycampusId });
    }
    return (
      <Page title={0} fitWindowHeight={false} loading={loading} className={styles.page}>
        {/* <div className={styles.content}> */}
        {/* <div> */}
        <div className={styles.content + ' ' + styles.clearfix}>
          <div className={styles.left}>
            <SearchBar onSearch={this.search} onReset={this.reset}>
              <DatePicker
                placeholder="选择时间"
                onChange={this.dateChange}
                value={date}
                validator={{
                  rules: ['required'],
                  silent: !errors['time'] || !checking,
                  showTips: false,
                  showStatus: true,
                  onErrorChange: (e) => this.errorChange('time', e),
                }}
              />
              <Select
                placeholder="学习中心"
                dict={campusDict}
                onChange={this.changeCampus}
                value={campusId}
                showSearch={true}
                validator={{
                  rules: ['required'],
                  silent: !errors['campus'] || !checking,
                  showTips: false,
                  showStatus: true,
                  onErrorChange: (e) => this.errorChange('campus', e),
                }}
              />
              <Select
                placeholder="课表类型"
                dict={classScheduleTypeDict}
                onChange={this.onChangeclassScheduleTypeValue}
                value={type}
                validator={{
                  rules: ['required'],
                  silent: !Boolean(errors['type']) || !checking,
                  showTips: false,
                  showStatus: true,
                  onErrorChange: (e) => this.errorChange('type', e),
                }}
              />
              <Select
                placeholder="请选择"
                dict={classScheduleTypeValueDict}
                onChange={(value) => {
                  this.onChangeClassScheduleTypeValue(value);
                }}
                value={classScheduleTypeValue}
                showSearch={true}
                noOptionTips={'暂无数据'}
                validator={{
                  rules: ['required'],
                  silent: !errors['classScheduleTypeValue'] || !checking,
                  showTips: false,
                  showStatus: true,
                  onErrorChange: (e) => this.errorChange('classScheduleTypeValue', e),
                }}
              />
              <MultiSelect
                dict={filterConditionDict}
                onChange={(value) => {
                  // let courseStatus = value === null ? ["NEW", "CHARGED", "TEACHER_ATTENDANCE"] : value
                  let courseStatus = value;
                  this.setState({ courseStatus });
                }}
                showSearch={false}
                value={courseStatus}
              />
            </SearchBar>
          </div>
          <div>
            <DropdownMenu menus={menus} text="导出Excel" hasArrow={true} className={styles.right} />
            {/* <Button onClick={this.exportExcel}>{exportStatus}</Button> */}
          </div>
        </div>
        {/* </div> */}
        {hasConflictName && <ConflictList title={title} list={list} current={current} onClickName={this.onClickName} />}
        {hasCourseSchedules && (
          <div>
            <ScheduleHeader
              dateArray={dateArray}
              onClickPreWeek={this.onClickPreWeek}
              onClickNextWeek={this.onClickNextWeek}
              searchDate={date}
            />
            <SchedulePanel
              courseSchedules={courseSchedules}
              type={type}
              editCourse={this.editCourse}
              changeCourse={this.changeCourse}
            />
          </div>
        )}
        {!hasConflictName && campusId === null && <DefaultTip message={'请先选择学习中心'} />}
        {!hasConflictName && type === null && campusId !== null && <DefaultTip message={'请先选择课表类型'} />}
        {!hasConflictName && type !== null && campusId !== null && !isNotData && classScheduleTypeValue === null && (
          <DefaultTip message={'请先选择教室/老师/学生'} />
        )}
        {isNotData && !hasCourseSchedules && <DefaultTip message={'请更换条件重新查询'} />}
        {/* </div> */}

        {isEditCourse && (
          <EditCourse
            type={type}
            classTwoId={classTwoId}
            closeModal={() => this.setState({ isEditCourse: false })}
            teacherDict={teacherDict}
            classroomDict={classroomDict}
            courseNum={courseNum}
            search={this.search}
            campusId={campusId}
            // reloadTeacherDict={this.reloadTeacherDict}
          />
        )}
        {isChangeCourse && (
          <ChangeCourse
            cancelMultiChangeCourses={() => {
              this.setState({ studentIds: [] });
            }}
            studentIds={studentIds}
            teacherDict={teacherDict}
            closeModal={() => {
              this.setState({ isChangeCourse: false, loading: false, studentIds: [] });
            }}
            classTwoId={classTwoId}
            studentId={classScheduleTypeValue}
            campusDict={campusDict}
            search={this.search}
            courseNum={courseNum}
          ></ChangeCourse>
        )}
      </Page>
    );
  }
}

let mapStateToProps = (state) => {
  let account = state.account;
  let pageParams = state.main.pageParams['/twoTeacherCourseManage'];
  return {
    account: account,
    pageParams,
    btnAuths: state.account.btnAuths,
  };
};

export default connect(mapStateToProps)(ScheduleManage);
