import dicts from './dicts';
import {
  selectOrgByIdAndTypeOption,
  getClassroomByCampu,
  changetoGetConflictInfo,
  changeClassRoomOrTeacher,
} from 'SERVICE/common';
import {
  getTwoTeacherClassInfoBySelect,
  getStudentsByCampusId,
  getTwoTeacherBycampusId,
  exportTwoTeacherConflictPerson,
  exportTwoTeacherCourseInfoBycampusId,
  getTwoTeacherConflictName,
  getEditTwoTeacherClassCourseInfo,
  getEditTwoTeacherClassInfo,
  getEditConflictInfo,
  editTeacherOrclassroom,
  getTwoClassTeachers,
  getStudentConflictCourse,
  getStudentConflictCourseNew,
  getChangeMiniClassList,
  changeCourse,
  getSubSelectOptionBySuper,
  checkConflictAfterTtChangeCourse,
} from 'SERVICE/twoTeacher';
import moment from 'moment';
import { DATE_FORMAT, TIME_FORMAT } from 'COMPONENT/utils/form';
import { urlBase } from 'CONST/config';
import { formateDict, formatGroupDict } from 'UTIL/dict';
import { getSelectOption } from 'SERVICE/common';
// 课表类型
const classScheduleDict = [
  {
    val: 'teacher',
    text: '老师',
  },
  {
    val: 'student',
    text: '学生',
  },
  {
    val: 'classroom',
    text: '教室',
  },
];
// 筛选条件
const filterConditionDict = [
  {
    val: 'NEW',
    text: '未上课',
  },
  {
    val: 'TEACHER_ATTENDANCE',
    text: '已考勤',
  },
  {
    val: 'CHARGED',
    text: '已结算',
  },
];

const loadCampusDict = async (params) => {
  let data = await selectOrgByIdAndTypeOption(params);
  // let value = data.value
  // let vals = Object.keys(value)
  // let campusDict = vals.map((key)=>{
  //   return {
  //     val: key,
  //     text: value[key]
  //   }
  // })
  // campusDict.pop()
  // campusDict.unshift({val:'',text:'请选择'})
  // return campusDict
  return formateDict(data);
};

const loadClassScheduleTypeDict = async () => {
  return Promise.resolve(classScheduleDict);
};

const loadFilterConditionDict = async () => {
  return Promise.resolve(filterConditionDict);
};

const loadTeacherDict = async (params) => {
  let data = await getTwoTeacherBycampusId(params);
  let teachers = data.data;
  return teachers.map((teacher) => {
    return {
      val: teacher.teacherId,
      text: teacher.teacherName,
    };
  });
};

const loadStudentDict = async (params) => {
  let response = await getStudentsByCampusId(params);
  const { data } = response;
  let studentDict = data.map((student) => {
    return {
      val: student.studentId,
      text: student.studentName,
    };
  });
  return studentDict;
};

const loadClassroomDict = async (params) => {
  let data = await getClassroomByCampu(params);
  let classroomDict = data.map((classroom) => {
    return {
      val: classroom.value,
      text: classroom.name,
    };
  });
  return classroomDict;
};

const loadConflictName = async (params) => {
  let data = await getTwoTeacherConflictName(params);
  let nameList = data.data;
  if (nameList.length === 0) {
    return null;
  } else {
    if (params.type === 'student') {
      let conflictList = nameList.map((name) => {
        return { name: name.studentName, id: name.studentId };
      });
      return { type: nameList[0].type, conflictList };
    } else if (params.type === 'teacher') {
      let conflictList = nameList.map((name) => {
        return { name: name.teacherName, id: name.teacherId };
      });
      return { type: nameList[0].type, conflictList };
    } else {
      let conflictList = nameList.map((name) => {
        return { name: name.classroomName, id: name.classroomId };
      });
      return { type: nameList[0].type, conflictList };
    }
  }
};

const loadCoursesSchedules = async (params) => {
  let response = await getTwoTeacherClassInfoBySelect(params);
  let { data } = response;
  let dateMessage = response.resultMessage;
  let startDate = dateMessage.split(',')[0];
  let endDate = dateMessage.split(',')[1];
  let dateArray = [];
  for (
    let date = startDate;
    moment(date).valueOf() <= moment(endDate).valueOf();
    date = moment(date).add(1, 'days').format(DATE_FORMAT)
  ) {
    dateArray.push(date);
  }
  let courseSchedules = [];
  for (let i = 0; i < dateArray.length; i++) {
    let oneDayCourses = [];
    for (let j = 0; j < data.length; j++) {
      if (dateArray[i] === data[j].courseDate) {
        oneDayCourses.push(data[j]);
      }
    }
    courseSchedules.push(oneDayCourses);
  }
  console.log('------------------');
  console.log({ courseSchedules, dateArray });
  return { courseSchedules, dateArray };
};

const exportExcel = async (params, fileName) => {
  return new Promise((resolve, reject) => {
    let request = new XMLHttpRequest();
    request.open('POST', urlBase + 'TwoTeacherClassController/exportTwoTeacherConflictPerson.do', true);
    request.responseType = 'blob';
    request.setRequestHeader('Content-type', 'application/json');
    request.onload = function () {
      if (this.status === 200) {
        let blob = this.response;
        let reader = new FileReader();
        reader.readAsDataURL(blob);
        reader.onload = function (event) {
          resolve();
          let a = document.createElement('a');
          a.href = window.URL.createObjectURL(request.response);
          a.download = fileName + '.xls';
          document.body.appendChild(a);
          a.click();
          a.remove();
        };
      } else {
        let reader = new FileReader();
        reader.addEventListener('loadend', (event) => {
          let text = event.srcElement.result;
          reject(JSON.parse(text));
        });
        reader.readAsText(this.response);
      }
    };
    request.onerror = (error) => {
      reject(error);
    };
    request.send(JSON.parse(JSON.stringify(params)));
  });
};

const loadConflictClassInfo = async (params) => {
  let response = await getEditTwoTeacherClassInfo(params);
  let { data } = response;
  let classTwoDataSource = data.map((classTwoInfo) => {
    return {
      classTwoName: classTwoInfo.classTwoName,
      teacher: {
        teacherId: classTwoInfo.teacherId,
        teacherName: classTwoInfo.teacherName,
      },
      classroom: {
        classroomId: classTwoInfo.classroomId,
        classroomName: classTwoInfo.classroomName,
      },
      classTwoId: classTwoInfo.classTwoId,
      classroomName: classTwoInfo.classroomName,
      subject: classTwoInfo.subjectName,
      subjectId: classTwoInfo.subjectId,
      grade: classTwoInfo.gradeName,
      gradeId: classTwoInfo.gradeId,
      everyCourseNum: classTwoInfo.everyCourseNum,
      classTimeLength: classTwoInfo.classTimeLength,
      teacherName: classTwoInfo.teacherName,
    };
  });
  return classTwoDataSource;
};

const loadConflictCourseInfo = async (params) => {
  let response = await getEditTwoTeacherClassCourseInfo(params);
  let { data } = response;
  let miniCourseDataSource = data.map((miniCourseInfo) => {
    return {
      courseStatus: miniCourseInfo.courseStatus,
      courseDate: miniCourseInfo.courseDate,
      courseTimeLength: miniCourseInfo.courseTimeLength,
      courseId: miniCourseInfo.courseId,
      classroom: {
        classroomName: miniCourseInfo.classroomName,
        classroomId: miniCourseInfo.classroomId,
      },
      teacher: {
        teacherName: miniCourseInfo.teacherName,
        teacherId: miniCourseInfo.teacherId,
      },
      courseNum: miniCourseInfo.courseNum,
      courseDate: miniCourseInfo.courseDate,
      courseTime: miniCourseInfo.courseTime,
      courseTimeLength: miniCourseInfo.courseTimeLength,
    };
  });
  return miniCourseDataSource;
};

const loadChangetoGetConflictInfo = async (params) => {
  let response = await changetoGetConflictInfo(params);
  let { data } = response;
  return data;
};

const changeClassRoomOrTeacherApi = async (params) => {
  let response = await changeClassRoomOrTeacher(params);
};

const loadStudentConflictCourse = async (params) => {
  let response = await getStudentConflictCourse(params);
  return response.data.map((course) => {
    return Object.assign(course, { selected: false });
  });
};

const loadStudentConflictCourseNew = async (params) => {
  let response = await getStudentConflictCourse(params);
  return response.data.map((course) => {
    return Object.assign(course, { selected: false });
  });
};

const loadChangeTwoTeacherClassList = async (params) => {
  let response = await getChangeMiniClassList(params);
  console.log('ChangeMiniClassList');
  console.log(response);
  return { dataSource: response.rows, total: response.records, current: response.page };
};

const loadSelectOption = async () => {
  let response = await getSelectOption({ selectOptionCategory: 'CLASS_TYPE', parentId: 'TWO_TEACHER' });
  return formateDict(response);
};

const changeCourseApi = async (params) => {
  let response = await changeCourse(params);
  console.log('response');
  console.log(response);
  return response;
};

const checkConflictAfterTtChangeCourseApi = async (params) => {
  let response = await checkConflictAfterTtChangeCourse(params);
  return response;
};

const getEditConflictInfoApi = async (params) => {
  let response = await getEditConflictInfo(params);
  return response.data;
};

const editTeacherOrclassroomApi = async (params) => {
  let response = await editTeacherOrclassroom(params);
  return response.data;
};

const getTwoClassTeachersApi = async (params) => {
  let response = await getTwoClassTeachers(params);
  let { kuaxiaoqu, renke } = response;
  let kuaxiaoquTeachers = kuaxiaoqu.map((teacher) => {
    return {
      val: teacher.value,
      text: teacher.name,
    };
  });
  let renkeTeachers = renke.map((teacher) => {
    return {
      val: teacher.value,
      text: teacher.name,
    };
  });
  return kuaxiaoquTeachers.concat(renkeTeachers);
};

export default {
  loadCampusDict,
  loadClassScheduleTypeDict,
  loadFilterConditionDict,
  loadStudentDict,
  loadClassroomDict,
  loadTeacherDict,
  loadCoursesSchedules,
  loadConflictName,
  exportExcel,
  loadConflictClassInfo,
  loadConflictCourseInfo,
  loadChangetoGetConflictInfo,
  changeClassRoomOrTeacherApi,
  loadStudentConflictCourse,
  loadStudentConflictCourseNew,
  loadChangeTwoTeacherClassList,
  loadSelectOption,
  changeCourseApi,
  checkConflictAfterTtChangeCourseApi,
  getEditConflictInfoApi,
  editTeacherOrclassroomApi,
  getTwoClassTeachersApi,
};
