/**
 * Created by Haolin on 2019/5/16.
 */
import React from 'react';
import { connect } from 'dva';
import style from './index.less';
import { storeName } from '../store/constants';
import DatePicker from 'COMPONENT/DatePicker';
import Select from 'COMPONENT/Select';
import Button from 'COMPONENT/Button';
import { Switch, Icon } from 'antd';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

const classPrefix = 'IndexBar';

const ItemWrap = (props) => {
  let widthStyle = props.width ? { width: props.width } : null;
  return (
    <div className={style[`${classPrefix}-ItemWrap`]} style={widthStyle}>
      {props.children}
    </div>
  );
};

class IndexBar extends React.Component {
  constructor(props) {
    super(props);
    this.updateDate = this.updateDate.bind(this);
    this.updateMainTc = this.updateMainTc.bind(this);
    this.search = this.search.bind(this);
    this.reset = this.reset.bind(this);
    this.dateErrorChange = this.dateErrorChange.bind(this);
    this.updateSwitchFilter = this.updateSwitchFilter.bind(this);
    this.state = {
      dateError: null,
    };
  }

  updateDate(val) {
    this.props.dispatch({
      type: `${storeName}/updateDate`,
      payload: val,
    });
  }

  updateMainTc(val) {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        sKey_mainTc: val,
      },
    });
  }

  search() {
    if (this.state.dateError) {
      return showMessage('请选择日期！', 'error');
    }
    this.props.onSearch();
  }

  reset() {
    this.props.dispatch({
      type: `${storeName}/resetSKey`,
    });
  }

  dateErrorChange(err) {
    this.setState({ dateError: err });
  }

  updateSwitchFilter(checked) {
    this.props.dispatch({
      type: `${storeName}/filterCourseTable`,
      payload: { checked },
    });
  }

  render() {
    const { mainTcDict, sKey_date, sKey_mainTc, switchFilter } = this.props;
    return (
      <div className={style[classPrefix]}>
        <ItemWrap width={180}>
          <DatePicker
            value={sKey_date}
            onChange={this.updateDate}
            validator={{
              rules: ['required'],
              silent: !Boolean(this.state.dateError),
              showStatus: true,
              showTips: false,
              onErrorChange: this.dateErrorChange,
            }}
          />
        </ItemWrap>
        <ItemWrap width={180}>
          <Select dict={mainTcDict} value={sKey_mainTc} onChange={this.updateMainTc} showSearch />
        </ItemWrap>
        <ItemWrap>
          <Button type="primary" onClick={this.search}>
            查询
          </Button>
        </ItemWrap>
        <ItemWrap>
          <Button type="default" onClick={this.reset}>
            重置
          </Button>
        </ItemWrap>
        <ItemWrap>
          <Button type="primary" onClick={this.props.onExport} disabled={this.props.exportBtnDisabled}>
            导出
          </Button>
        </ItemWrap>
        <ItemWrap>
          <Switch
            checked={switchFilter}
            onChange={this.updateSwitchFilter}
            checkedChildren={<Icon type="check" />}
            unCheckedChildren={<Icon type="cross" />}
          />
          <span style={{ marginLeft: 4 }}>只显示有课程的信息</span>
        </ItemWrap>
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    mainTcDict: state.dict.allTwoTcMainTc,
    sKey_date: state[storeName].sKey_date,
    sKey_mainTc: state[storeName].sKey_mainTc,
    switchFilter: state[storeName].switchFilter,
  };
};

export default connect(mapStateToProps)(IndexBar);
