import React from 'react';
import ReactDOM from 'react-dom';
import { connect } from 'dva';
import style from './index.less';
import classNames from 'classnames';
import { Icon } from 'antd';
import { storeName } from '../../store/constants.js';

const classPrefix = 'CourseCard';
// 课程卡片主体内容
const CardMainNormal = (props) => {
  const statusClass = CourseCard.setClassByStatue(props.status);
  return (
    <div className={style.CardMainNormal}>
      <div className={style[`${classPrefix}-header`]}>
        <div className={classNames(style[`${classPrefix}-header-label`], style[statusClass])}>{props.statusName}</div>
        <div className={style[`${classPrefix}-header-content`]}>
          {props.beginTime}-{props.endTime}
        </div>
      </div>
      <div className={style[`${classPrefix}-body`]}>
        <div className={style[`${classPrefix}-body-content`]}>
          {props.current === '1' ? props.className : props.classTwoName}
        </div>
      </div>
      <div className={style[`${classPrefix}-footer`]}>
        {props.current !== '1' && (
          <div className={style[`${classPrefix}-footer-item`]}>
            <span>辅导老师：{props.teacherTwoName}</span>
          </div>
        )}
        <div className={style[`${classPrefix}-footer-item`]}>
          <span>课程老师：{props.teacherName}</span>
        </div>
      </div>
    </div>
  );
};

class CourseCard extends React.Component {
  constructor(props) {
    super(props);
    this.setCardHeight = this.setCardHeight.bind(this);
    this.resetCardHeight = this.resetCardHeight.bind(this);
    this.state = {
      hoverClassName: '',
      cardShowStatus: false, // 卡片是否手动设置
    };
    this.$cardContainer = null;
    this.$card = null;
    this.$cardMain = null;
    this.$cardHeader = null;
    this.cardOriginHeight = 0;
    this.headerOriginHeight = 0;
  }
  // 根据状态(未考勤、已考勤、课程冲突)来设定课程卡片的类名
  static setClassByStatue(status) {
    let ret;
    switch (status) {
      case 'NEW':
        ret = 'course-new';
        break;
      case 'CONFLICT':
        ret = 'course-conflict';
        break;
      default:
        ret = '';
    }
    return ret;
  }
  /**
   * 鼠标移入时设置卡片的高度及悬浮效果
   *
   * 数值 22 对应index.less内的.CourseCard-header的高度18与.CourseCard-body的上外边距4的和
   * 数值 20 对应index.less内的.CourseCard上下padding的和
   * */
  setCardHeight() {
    if (this.state.cardShowStatus) {
      return false;
    }
    this.cardOriginHeight = this.cardOriginHeight || this.$card.offsetHeight;
    let cardHeight = this.$cardMain.offsetHeight;
    if (this.props.status === 'CONFLICT') {
      this.headerOriginHeight = this.headerOriginHeight || this.$cardHeader.offsetHeight;
      this.$cardHeader.style.height = '0px';
      cardHeight = cardHeight - 22;
    }
    cardHeight = cardHeight + 20;
    if (this.$cardContainer.parentNode.offsetHeight < cardHeight) {
      this.$card.style.height = `${cardHeight}px`;
    }
    this.setState({ hoverClassName: 'card-hover' });
  }
  // 鼠标移出时重置卡片为初始高度
  resetCardHeight() {
    if (this.state.cardShowStatus) {
      return false;
    }
    this.$card.style.height = `${this.cardOriginHeight}px`;
    if (this.props.status === 'CONFLICT') {
      this.$cardHeader.style.height = `${this.headerOriginHeight}px`;
    }
    this.setState({ hoverClassName: '' });
  }
  /**
   * @param {Boolean} status 状态控制，true为展开，false为收缩，展开之后鼠标移入移出效果消失，收缩之后才会有
   * @description 手动控制卡片的悬浮效果
   * **/
  userSetCardHeight = (status) => {
    this.setState({
      cardShowStatus: status,
    });
  };

  render() {
    const statusClass = CourseCard.setClassByStatue(this.props.status);
    const isConflict = this.props.status === 'CONFLICT';
    const isHover = this.state.hoverClassName !== '';
    let { cardShowStatus } = this.state;

    return (
      <div
        className={classNames(style[`${classPrefix}-container`])}
        ref={(c) => (this.$cardContainer = this.$cardContainer || ReactDOM.findDOMNode(c))}
      >
        <div
          className={classNames(
            style[classPrefix],
            style[statusClass],
            isHover ? style[this.state.hoverClassName] : '',
          )}
          ref={(c) => (this.$card = this.$card || ReactDOM.findDOMNode(c))}
          onMouseEnter={this.setCardHeight}
          onMouseLeave={this.resetCardHeight}
        >
          <div className={style[`${classPrefix}-wrap`]}>
            <div
              className={style[`${classPrefix}-main`]}
              ref={(c) => (this.$cardMain = this.$cardMain || ReactDOM.findDOMNode(c))}
            >
              <div
                className={style[`${classPrefix}-header`]}
                ref={(c) => (this.$cardHeader = this.$cardHeader || ReactDOM.findDOMNode(c))}
              >
                <div className={classNames(style[`${classPrefix}-header-label`], style[statusClass])}>
                  {this.props.statusName}
                </div>
                <div className={style[`${classPrefix}-header-content`]}>
                  {this.props.beginTime}-{this.props.endTime}
                </div>
              </div>
              <div
                className={classNames(
                  style[`${classPrefix}-body`],
                  isConflict && isHover ? style[this.state.hoverClassName] : '',
                )}
              >
                <div className={style[`${classPrefix}-body-content`]}>
                  {isConflict ? (
                    <div className={style[`${classPrefix}-body-header`]}>
                      <span className={isHover ? style.warning : ''}>共有 {this.props.list.length} 门冲突课程</span>
                      {/*TODO: 这里收缩的时候标签不跟内容一起消失 */}
                      <span
                        onClick={() => this.userSetCardHeight(!cardShowStatus)}
                        className={style[`${classPrefix}-body-header-icon`]}
                      >
                        {cardShowStatus ? (
                          <Icon type="pushpin-o" style={{ color: 'blue' }} />
                        ) : (
                          <Icon type="pushpin-o" />
                        )}
                      </span>
                    </div>
                  ) : this.props.current === '1' ? (
                    this.props.className
                  ) : (
                    this.props.classTwoName
                  )}
                </div>
                <div
                  className={classNames(
                    style[`${classPrefix}-body-wrap`],
                    isHover ? style[this.state.hoverClassName] : '',
                  )}
                >
                  {isConflict &&
                    this.props.list.map((item, index) => (
                      <CardMainNormal {...item} current={this.props.current} key={index} />
                    ))}
                </div>
              </div>
              {!isConflict && (
                <div className={style[`${classPrefix}-footer`]}>
                  {this.props.current !== '1' && (
                    <div className={style[`${classPrefix}-footer-item`]}>
                      <span>辅导老师：{this.props.teacherTwoName}</span>
                    </div>
                  )}
                  <div className={style[`${classPrefix}-footer-item`]}>
                    <span>课程老师：{this.props.teacherName}</span>
                  </div>
                </div>
              )}
            </div>
          </div>
        </div>
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  let menuStatus = state[storeName].menuStatus;
  return {
    current: menuStatus ? '1' : '2',
  };
};

export default connect(mapStateToProps)(CourseCard);
