import React from 'react';
import ReactDOM from 'react-dom';
import PropTypes from 'prop-types';
import style from './index.less';
import {
  tableColumnCellWidth,
  tableColumnHeaderHeight,
  tableRowAsideWidth,
  tableRowMinCellHeight,
  tableDividingHeight,
} from '../config';

const classPrefix = 'TableContainer';
// 星期数行
const TableRow = (props) => {
  return (
    <div className={style[`${classPrefix}-rows-row`]}>
      <TableRowDividing text={props.name} date={props.date} />
      {props.row.map((item) => (
        <div
          className={style[`${classPrefix}-rows-cell`]}
          style={{ height: tableRowMinCellHeight }}
          key={`${props.id}-${item}`}
        >
          {item}
        </div>
      ))}
    </div>
  );
};
// 星期数分隔
const TableRowDividing = (props) => {
  const dividingStyle = {
    top: (tableRowMinCellHeight - tableDividingHeight - 20) / 2,
    height: tableDividingHeight,
    lineHeight: `${tableDividingHeight}px`,
  };
  return (
    <div className={style[`${classPrefix}-rows-dividing`]} style={dividingStyle}>
      <div className={style[`${classPrefix}-rows-dividing-text`]}>
        <ruby>
          {props.date}
          <rt className={style[`${classPrefix}-rows-dividing-rt`]}>{props.text}</rt>
        </ruby>
      </div>
    </div>
  );
};

class TableContainer extends React.Component {
  constructor(props) {
    super(props);
    this.handleScroll = this.handleScroll.bind(this);
  }
  static propTypes = {
    columns: PropTypes.array.isRequired,
    rows: PropTypes.array.isRequired,
  };
  componentDidMount() {
    this.$tableBodyMain.addEventListener('scroll', this.handleScroll, false);
  }
  componentWillUnmount() {
    this.$tableBodyMain.removeEventListener('scroll', this.handleScroll, false);
  }
  // 滚动监听事件
  handleScroll() {
    this.$columnScroller.style.left = `${-this.$tableBodyMain.scrollLeft}px`;
    this.$rowScroller.style.top = `${-this.$tableBodyMain.scrollTop}px`;
  }

  render() {
    const columnsStyle = {
      left: tableRowAsideWidth,
      height: tableColumnHeaderHeight,
    };
    const columnsWrapStyle = {
      width: this.props.columns.filter((item) => !item.disabled).length * tableColumnCellWidth,
      height: tableColumnHeaderHeight,
    };
    const columnsCellStyle = { maxWidth: tableColumnCellWidth };
    const rowsStyle = {
      top: tableColumnHeaderHeight,
      width: tableRowAsideWidth,
    };
    const rowsWrapStyle = {
      maxWidth: tableRowAsideWidth,
      minWidth: tableRowAsideWidth,
    };

    return (
      <div className={style[classPrefix]}>
        <div className={style[`${classPrefix}-columns`]} style={columnsStyle}>
          <div
            className={style[`${classPrefix}-columns-wrap`]}
            ref={(c) => (this.$columnScroller = ReactDOM.findDOMNode(c))}
            style={columnsWrapStyle}
          >
            {this.props.columns.map((item) =>
              item.disabled ? (
                ''
              ) : (
                <div className={style[`${classPrefix}-columns-cell`]} key={item.id} style={columnsCellStyle}>
                  <div className={style[`${classPrefix}-columns-text`]} title={item.name}>
                    {item.name}
                  </div>
                  {item.subName && <div className={style[`${classPrefix}-columns-text-sub`]}>{item.subName}</div>}
                </div>
              ),
            )}
          </div>
        </div>
        <div className={style[`${classPrefix}-rows`]} style={rowsStyle}>
          <div
            className={style[`${classPrefix}-rows-wrap`]}
            ref={(c) => (this.$rowScroller = ReactDOM.findDOMNode(c))}
            style={rowsWrapStyle}
          >
            {this.props.rows.map((item) => (item.disabled ? '' : <TableRow {...item} key={item.id} />))}
          </div>
        </div>
        <div
          className={style[`${classPrefix}-body`]}
          ref={(c) => (this.$tableBodyMain = ReactDOM.findDOMNode(c))}
          style={{ top: tableColumnHeaderHeight, left: tableRowAsideWidth }}
        >
          {this.props.children}
        </div>
      </div>
    );
  }
}

export default TableContainer;
