import React from 'react';
import PropTypes from 'prop-types';
import style from './index.less';
import { tableColumnCellWidth, tableRowMinCellHeight } from '../config';
import TimeRangeWrap from '../TimeRangeWrap';

const classPrefix = 'TableMain';
// 行内列
const TableMainCol = (props) => {
  const colStyle = {
    maxWidth: tableColumnCellWidth,
    padding: `${tableRowMinCellHeight - 10}px 0 10px`,
  };
  return (
    <div className={style[`${classPrefix}-col`]} style={colStyle}>
      <div className={style[`${classPrefix}-col-wrap`]}>
        {props.dataSource.length > 0 &&
          props.dataSource.map((item) => (
            <TimeRangeWrap
              renderMinHeight={props.renderMinHeight}
              rangeCfg={props.rangeCfg}
              range={item.range}
              key={item.id}
            >
              {props.render(item)}
            </TimeRangeWrap>
          ))}
      </div>
    </div>
  );
};
// 分隔线
const SeparationLine = (props) => {
  return (
    <div className={style[`${classPrefix}-separation`]} style={{ top: props.top }}>
      {/*<div className={style[`${classPrefix}-separation-circle`]}> </div>*/}
    </div>
  );
};
// 行
const TableMainRow = (props) => {
  const rowStyle = {
    width: props.columns.filter((item) => !item.disabled).length * tableColumnCellWidth,
    height: props.rowHeight,
  };
  return (
    <div className={style[`${classPrefix}-row`]} style={rowStyle}>
      {props.columns.map((item) =>
        item.disabled ? (
          ''
        ) : (
          <TableMainCol
            dataSource={props.dataSource[item.id] || []}
            render={props.render}
            renderMinHeight={props.renderMinHeight}
            colHeight={props.rowHeight}
            colWidth={tableColumnCellWidth}
            rangeCfg={props.rangeCfg}
            key={item.id}
          />
        ),
      )}
      {props.separationVisible && <SeparationLine top={(tableRowMinCellHeight - 22) / 2} />}
    </div>
  );
};

const TableMain = (props) => {
  return (
    <div className={style[classPrefix]}>
      {props.rows.map((item, index) =>
        item.disabled ? (
          ''
        ) : (
          <TableMainRow
            columns={props.columns}
            rowHeight={item.row.length * tableRowMinCellHeight}
            dataSource={props.dataSource[item.id] || {}}
            render={props.render}
            renderMinHeight={props.renderMinHeight}
            separationVisible={index !== 0}
            rangeCfg={item.row}
            key={item.id}
          />
        ),
      )}
    </div>
  );
};
TableMain.propTypes = {
  columns: PropTypes.array.isRequired,
  rows: PropTypes.array.isRequired,
  dataSource: PropTypes.object.isRequired,
  render: PropTypes.func.isRequired,
};

export default TableMain;
