/**
 * Created by Haolin<haolinhom@gmail.com> on 2019/5/14.
 */
import React from 'react';
import style from './index.less';
import { tableRowMinCellHeight as defaultHeight } from '../config';

const classPrefix = 'TimeRangeWrap';
// 获取时间段间隔
function getHourThrough(begin, end) {
  let beginTime = new Date(`2019-01-01 ${begin}`).getTime();
  let endTime = new Date(`2019-01-01 ${end}`).getTime();
  let time = beginTime < endTime ? endTime - beginTime : beginTime - endTime;
  return Number((time / 3600000).toFixed(1));
}
// 获取每个小时高度
function getOneHourHeight(rangeCfg, defaultHeight) {
  let num = rangeCfg.length - 1;
  let timeTotal = getHourThrough(rangeCfg[0], rangeCfg[num]);
  return Number(((defaultHeight * num) / timeTotal).toFixed(1));
}
// 检验起始时间是否在配置的时间段内，如果不是则重新设定一个默认值
function checkTime(timeStr, range) {
  let rangeFirst = range[0];
  let rangeLast = range[range.length - 1];
  let check = Number(timeStr.slice(0, 2));
  let firstTime = Number(rangeFirst.slice(0, 2));
  let lastTime = Number(rangeLast.slice(0, 2));
  if (check === 0 || check > lastTime) {
    return rangeLast;
  } else if (check < firstTime) {
    return rangeFirst;
  } else {
    return timeStr;
  }
}
// 设置TimeRangeWrap样式
function setWrapStyle(range, rangeCfg, minHeight) {
  let begin = checkTime(range[0], rangeCfg);
  let end = checkTime(range[1], rangeCfg);
  const oneHourHeight = getOneHourHeight(rangeCfg, defaultHeight);
  let height = Number((getHourThrough(begin, end) * oneHourHeight).toFixed(1));
  return {
    top: Number((getHourThrough(rangeCfg[0], begin) * oneHourHeight).toFixed(1)),
    height: minHeight && minHeight > height ? minHeight : height,
  };
}
//
const TimeRangeWrap = (props) => {
  return (
    <div className={style[classPrefix]} style={setWrapStyle(props.range, props.rangeCfg, props.renderMinHeight)}>
      {props.children}
    </div>
  );
};

export default TimeRangeWrap;
