import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import IndexBar from '../../IndexBar';
import SlaveSearchBar from '../SlaveSearchBar';
import CourseTable from '../CourseTable';
import style from './index.less';
import { Menu } from 'antd';
import { differenceBy } from 'lodash';
import { storeName } from '../../store/constants';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { buildDateConfig, buildTableHead, buildExportTableData, buildTableDOM, exportExcel } from '../../utils';

const classPrefix = 'TwoTeacherMasterSlaveCurriculum';

class TwoTeacherMasterSlaveCurriculum extends React.Component {
  constructor(props) {
    super(props);
    this.handleExport = this.handleExport.bind(this);
    this.state = {
      slaveClassRoomRow: [],
    };
  }

  componentWillUnmount() {
    this.reset();
  }

  componentDidMount() {
    let temp = document.querySelector('.Page-main-header');
    temp.style.display = 'none';
  }

  reset = () => {
    this.props.dispatch({
      type: `${storeName}/resetSKey`,
    });
  };

  /**
   * @param {Array} arr 目标数组
   * @description 返回两者数组里不同的元素函数
   * @return {Array} 返回两者数组里不同的元素
   * **/
  filterDifference = (arr) => {
    let filterColList = this.props.filterColList.map((value) => {
      return {
        val: value,
      };
    });
    return differenceBy(arr, filterColList, 'val');
  };

  // 导出课表为excel
  async handleExport() {
    let { childChangeParentState } = this.props;
    if (this.props.origin.length === 0) {
      return showMessage('当前课表没有数据，无法导出！', 'error');
    }
    childChangeParentState({ loading: true });
    await new Promise(this.exportExcel);
    childChangeParentState({ loading: false });
  }
  exportExcel = (resolve) => {
    let headerData = this.props.tableColumns.map((value) => {
      return {
        val: value.id,
        text: value.name,
      };
    });
    if (headerData.length) {
      headerData = this.filterDifference(headerData);
    }
    const dateCfg = buildDateConfig(this.props.beginDate);
    const tableHeadData = buildTableHead(headerData);
    const tableBodyData = buildExportTableData(this.props.origin, dateCfg, headerData, this.props.menuStatus);
    let titleStatus = this.props.current === '1' ? '主班' : '辅班';
    let title = `双师${titleStatus}课表(${this.props.beginDate}-${this.props.endDate})`;
    let table = buildTableDOM(tableHeadData, tableBodyData, title);
    exportExcel(table, title, () => {
      resolve();
    });
  };

  render() {
    const { current } = this.props;
    let {
      name,
      className,
      loading,
      route,
      exportBtnDisabled,
      tableRows,
      tableColumns,
      courseCardRender,
      handleClick,
      search,
    } = this.props;
    return (
      <div className={style.TwoTeacherMasterSlaveCurriculum}>
        <div className={style[`${classPrefix}-header`]}>
          <Menu onClick={handleClick} selectedKeys={[current]} mode="horizontal">
            <Menu.Item key="1">主班课表</Menu.Item>
            <Menu.Item key="2">辅班课表</Menu.Item>
          </Menu>
        </div>
        <div className={style[`${classPrefix}-contain`]}>
          <Page
            name={name}
            className={className}
            loading={loading}
            route={route}
            fitWindowHeight={true}
            subHeader={
              current === '1' ? (
                <IndexBar onSearch={search} onExport={this.handleExport} exportBtnDisabled={exportBtnDisabled} />
              ) : (
                <SlaveSearchBar onSearch={search} onExport={this.handleExport} exportBtnDisabled={exportBtnDisabled} />
              )
            }
          >
            <CourseTable
              dataSource={this.props.courseData}
              rowsConfig={tableRows}
              colsConfig={tableColumns}
              render={courseCardRender}
              renderMinHeight={40}
            />
          </Page>
        </div>
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  let menuStatus = state[storeName].menuStatus;
  let courseDataObjKeyName = menuStatus ? 'courseMaster' : 'courseSlave';
  let origin = menuStatus ? state[storeName].originCourseData : state[storeName].originCourseSlaveData || [];
  let courseData = state[storeName][courseDataObjKeyName].courseData || {};
  return {
    origin,
    menuStatus,
    liveRoomDict: state.dict.liveRoom,
    current: menuStatus ? '1' : '2',
    beginDate: state[storeName][courseDataObjKeyName].courseData_beginDate,
    endDate: state[storeName][courseDataObjKeyName].courseData_endDate,
    courseData: courseData,
    filterColList: menuStatus ? state[storeName].filterMasterColList : state[storeName].filterSlaveColList,
  };
};

export default connect(mapStateToProps)(TwoTeacherMasterSlaveCurriculum);
