/**
 * Created by Haolin on 2019/5/13.
 */
import React from 'react';
import { connect } from 'dva';
import CourseCard from './components/CourseCard';
import { storeName } from './store/constants';
import { Monday, Tuesday, Wednesday, Thursday, Friday, Saturday, Sunday, hourConfig as tableRowConfig } from './config';
import moment from 'moment';
import TwoTeacherMasterSlaveCurriculum from './components/TwoTeacherMasterSlaveCurriculum';
import { indexOf } from 'lodash';

// 课程卡片渲染函数
const courseCardRender = (props) => <CourseCard {...props} />;
// 双师主辅班课表
class TwoTeacherCurriculum extends React.Component {
  constructor(props) {
    super(props);
    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.initLiveRoom = this.initLiveRoom.bind(this);
    this.initDefault = this.initDefault.bind(this);
    this.initTableConfig = this.initTableConfig.bind(this);
    this.updateTableColsDayName = this.updateTableColsDayName.bind(this);
    this.updateTableRowColDisabled = this.updateTableRowColDisabled.bind(this);
    this.loadMasterData = this.loadMasterData.bind(this);
    this.loadSlaveDate = this.loadSlaveDate.bind(this);
    this.beforeDestroy = this.beforeDestroy.bind(this);
    this.state = {
      name: '双师主辅班课表',
      loading: false,
      tableRows: [
        {
          id: Monday.id,
          name: Monday.text,
          date: null,
          row: tableRowConfig,
          disabled: false,
        },
        {
          id: Tuesday.id,
          name: Tuesday.text,
          date: null,
          row: tableRowConfig,
          disabled: false,
        },
        {
          id: Wednesday.id,
          name: Wednesday.text,
          date: null,
          row: tableRowConfig,
          disabled: false,
        },
        {
          id: Thursday.id,
          name: Thursday.text,
          date: null,
          row: tableRowConfig,
          disabled: false,
        },
        {
          id: Friday.id,
          name: Friday.text,
          date: null,
          row: tableRowConfig,
          disabled: false,
        },
        {
          id: Saturday.id,
          name: Saturday.text,
          date: null,
          row: tableRowConfig,
          disabled: false,
        },
        {
          id: Sunday.id,
          name: Sunday.text,
          date: null,
          row: tableRowConfig,
          disabled: false,
        },
      ],
      current: '1',
      tableColumns: [],
      isDataInit: false, // 辅班数据是否首次加载
    };
  }
  static classPrefix = 'TwoTeacherCurriculum';
  componentDidMount() {
    this.init();
  }
  componentWillUnmount() {
    this.beforeDestroy();
  }
  componentWillReceiveProps(nextProps) {
    // 更新列表y轴表头的日期/星期数显示
    if (nextProps.beginDate !== this.props.beginDate) {
      this.setState({ tableRows: this.updateTableColsDayName(nextProps.beginDate) });
    }
    // 更新要过滤的行列
    if (nextProps.switchFilter !== this.props.switchFilter) {
      this.updateTableRowColDisabled(nextProps.switchFilter, nextProps.filterRowList, nextProps.filterColList);
    }
  }
  // 初始化
  async init() {
    this.setState({ loading: true });
    await new Promise(this.initDict);
    await new Promise(this.initCampus);
    await new Promise(this.initLiveRoom);
    await new Promise(this.initDefault);
    await new Promise(this.initTableConfig);
    await new Promise(this.loadMasterData);
    await this.setDefaultCampus(this.props.slave_school);
    this.setState({ loading: false });
  }

  // 切换菜单
  handleClick = async (e) => {
    this.setState({ current: e.key });
    await new Promise(this.turnOffSwitchFilter);
    if (e.key === '1') {
      this.changeState(true);
      await new Promise(this.initTableConfig);
    } else {
      this.changeState(false);
      if (!this.state.isDataInit) {
        this.setState({ isDataInit: true });
        this.search();
      } else {
        await new Promise(this.initSlaveClassRoomRow);
      }
    }
  };

  /**
   * @param {Boolean} val 状态值
   * @description 切换tab状态值函数
   * **/
  changeState = (val) => {
    this.props.dispatch({
      type: `${storeName}/changeState`,
      payload: {
        menuStatus: val,
      },
    });
  };

  /**
   * @param {String} campus 学习中心id
   * @description 设置默认的campus值
   * **/
  setDefaultCampus = (campus) => {
    this.props.dispatch({
      type: `${storeName}/setDefaultCampus`,
      payload: {
        slave_default_campus: campus,
      },
    });
  };

  initDict(resolve) {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['liveRoom', 'allTwoTcMainTc'],
        callback() {
          resolve();
        },
      },
    });
  }

  initCampus = (resolve) => {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['branchCampus', 'campus', { name: 'productGroup', params: 'SMALL_CLASS' }, 'year', 'quarter'],
        callback() {
          resolve();
        },
      },
    });
  };

  initLiveRoom(resolve) {
    this.props.dispatch({
      type: `${storeName}/initLiveRoom`,
      payload: {
        callback() {
          resolve();
        },
      },
    });
  }

  initDefault(resolve) {
    this.props.dispatch({
      type: `${storeName}/setDefaultSKey`,
      payload: {
        callback() {
          resolve();
        },
      },
    });
  }

  initTableConfig(resolve) {
    let tempArr =
      this.props.liveRoom.length > 0
        ? this.props.liveRoom
        : this.props.liveRoomDict.map((item) => ({
            id: item.val,
            name: item.text,
            subName: null,
            disabled: false,
          }));
    let tempId = tempArr.map((value) => value.id);
    if (indexOf(tempId, '-1') === -1) {
      tempArr.push({
        id: '-1',
        name: '无直播间',
        subName: null,
        disabled: false,
      });
    }
    this.setState(
      {
        tableColumns: tempArr,
      },
      () => {
        resolve();
      },
    );
  }

  initSlaveClassRoomRow = (resolve) => {
    let tempArr = this.props.slave_campus
      ? this.props.classRoomsDict[this.props.slave_campus]
      : this.props.classRoomsDict[this.props.slave_school];
    let tempId = tempArr.map((value) => value.val);
    if (indexOf(tempId, '-1') === -1) {
      tempArr.push({
        val: '-1',
        text: '无教室',
      });
    }
    this.setState(
      {
        tableColumns: tempArr.map((item) => ({
          id: item.val,
          name: item.text,
          subName: null,
          disabled: false,
        })),
      },
      () => {
        resolve();
      },
    );
  };

  // 搜索
  search = async () => {
    let { current } = this.state;
    // if (current === '2' && !this.props.slave_campus) {
    //   return showMessage('请选择学习中心！', 'error');
    // }
    this.childChangeParentState({ loading: true });
    await new Promise(this.turnOffSwitchFilter);
    if (current === '1') {
      await new Promise(this.loadMasterData);
    } else {
      await new Promise(this.loadSlaveDate);
      await new Promise(this.getClassRoom);
      await new Promise(this.initSlaveClassRoomRow);
    }
    this.childChangeParentState({ loading: false });
  };

  // 辅班横轴教室接口
  getClassRoom = (resolve) => {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: [{ name: 'classRoomsByCampusId', params: this.props.slave_campus || this.props.slave_school }],
        callback() {
          resolve();
        },
      },
    });
  };

  // 关闭空数据行列过滤
  turnOffSwitchFilter = (resolve) => {
    this.props.dispatch({
      type: `${storeName}/filterCourseTable`,
      payload: {
        checked: false,
        callback() {
          resolve();
        },
      },
    });
  };

  // 更新tableRows[n].date
  updateTableColsDayName(beginDate) {
    const oneDayTime = 86400000;
    let firstDayTime = new Date(beginDate).getTime();
    return this.state.tableRows.map((item, index) => ({
      ...item,
      date: moment(firstDayTime + oneDayTime * index).format('MM-DD'),
    }));
  }
  // 更新tableRows/tableColumns的disabled
  updateTableRowColDisabled(checked, rowList, colList) {
    if (checked) {
      this.setState((prevState) => ({
        tableRows: prevState.tableRows.map((item) => {
          return rowList.includes(item.id) ? { ...item, disabled: true } : item;
        }),
        tableColumns: prevState.tableColumns.map((item) => {
          return colList.includes(item.id) ? { ...item, disabled: true } : item;
        }),
      }));
    } else {
      this.setState((prevState) => ({
        tableRows: prevState.tableRows.map((item) => ({ ...item, disabled: false })),
        tableColumns: prevState.tableColumns.map((item) => ({ ...item, disabled: false })),
      }));
    }
  }
  // 加载主班数据
  loadMasterData(resolve) {
    this.props.dispatch({
      type: `${storeName}/loadMasterCourseList`,
      payload: {
        callback() {
          resolve();
        },
      },
    });
  }
  // 加载辅班数据
  loadSlaveDate(resolve) {
    this.props.dispatch({
      type: `${storeName}/loadSlaveCourseList`,
      payload: {
        campusId: this.props.slave_school,
        callback() {
          resolve();
        },
      },
    });
  }
  /**
   * @param {String, Object} key 字段名称；当key为对象时，表示多个键值对，可以修改多个值，此时value不必传
   * @param {*=} value 可选参数，字段的值
   * @description 子组件修改父组件state的值
   * **/
  childChangeParentState = (key, value = '') => {
    if (typeof key === 'string') {
      this.setState(key, value);
    } else if (key && key.constructor === Object) {
      this.setState(key);
    } else {
      console.error('key不是string类型，也不是object类型');
    }
  };

  // 组件销毁前调用
  beforeDestroy() {
    this.setState({
      current: 1,
    });
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        courseMaster: {
          courseData: null,
          courseData_beginDate: null,
          courseData_endDate: null,
          courseDate_teacher: null,
        },
        courseSlave: {
          courseData: null,
          courseData_beginDate: null,
          courseData_endDate: null,
          courseDate_campus: null,
        },
        originCourseData: null,
        originCourseSlaveData: null,
        menuStatus: true,
      },
    });
  }

  render() {
    let params = {
      name: this.state.name,
      className: TwoTeacherCurriculum.classPrefix,
      loading: this.state.loading,
      route: this.props.route,
      exportBtnDisabled: this.props.exportBtnDisabled,
      tableRows: this.state.tableRows,
      tableColumns: this.state.tableColumns,
      courseCardRender: courseCardRender,
      childChangeParentState: this.childChangeParentState,
      handleClick: this.handleClick,
      search: this.search,
    };
    return <TwoTeacherMasterSlaveCurriculum {...params}></TwoTeacherMasterSlaveCurriculum>;
  }
}

const mapStateToProps = (state) => {
  let btnAuth = state.account.btnAuths;
  let menuState = state[storeName].menuStatus;
  let courseDataObjKeyName = menuState ? 'courseMaster' : 'courseSlave';
  const btnExportAuth = btnAuth.indexOf('TWO_TEACHER_CURRICULUM_EXPORT') === -1;
  let origin = state[storeName].originCourseData || [];
  let slaveId = state.dict.branchCampus;
  if (slaveId) {
    slaveId = slaveId.find((value) => {
      if (value.options && value.options.length) return value;
    }).options[0].val;
  }
  return {
    slave_school: slaveId,
    liveRoomDict: state.dict.liveRoom,
    origin,
    beginDate: state[storeName][courseDataObjKeyName].courseData_beginDate,
    endDate: state[storeName][courseDataObjKeyName].courseData_endDate,
    exportBtnDisabled: btnExportAuth || origin.length === 0,
    switchFilter: state[storeName].switchFilter,
    filterRowList: menuState ? state[storeName].filterMasterRowList : state[storeName].filterSlaveRowList,
    filterColList: menuState ? state[storeName].filterMasterColList : state[storeName].filterSlaveColList,
    liveRoom: state[storeName].liveRoom,
    slave_campus: state[storeName].slave_campus,
    classRoomsDict: state.dict.classRoomsByCampusId,
  };
};

export default connect(mapStateToProps)(TwoTeacherCurriculum);
