/**
 * Created by Haolin<haolinhom@gmail.com> on 2019/5/13.
 */
import { storeName, LIVE_ROOM } from './constants.js';
import { Monday, Tuesday, Wednesday, Thursday, Friday, Saturday, Sunday, beginAndEndHour } from '../config';
import moment from 'moment';
import { getTwoTeacherClassCourseLiveRoom, getTwoTeacherClassController } from 'SERVICE/course';
import { getListDataDict } from 'SERVICE/common';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { clone } from 'lodash';
import { buildDateConfig } from '../utils';
import { runFunc, errorsHandle } from 'UTIL/common';

const _fn = {
  // 获取某天对应周的起始日期
  getDateWeekRange(dateStr) {
    if (dateStr) {
      let oneDayMs = 86400000;
      let date = new Date(dateStr);
      let timestamp = date.getTime();
      let day = date.getDay();
      if (day === 0) {
        day = 7;
      }
      let timestampRange = [timestamp - (day - 1) * oneDayMs, (7 - day) * oneDayMs + timestamp];
      return [moment(timestampRange[0]).format('YYYY-MM-DD'), moment(timestampRange[1]).format('YYYY-MM-DD')];
    } else {
      return [null, null];
    }
  },
  // 错误处理
  errorsHandling(errCallback) {
    return (err) => {
      showMessage(err, 'error');
      if (errCallback) errCallback();
    };
  },
  // 通过day获取星期数id
  getDayIdByDay: {
    0: Sunday.id,
    1: Monday.id,
    2: Tuesday.id,
    3: Wednesday.id,
    4: Thursday.id,
    5: Friday.id,
    6: Saturday.id,
  },
  // 过滤掉源数据里不在时间范围内的数据
  filterOutTimeRangeData(origin) {
    let willDel = [];
    origin.forEach((dayItem, index) => {
      dayItem.groupValues = dayItem.groupValues.filter((item) => {
        let begin = Number(item.courseStartTime.slice(0, 2));
        return beginAndEndHour[0] <= begin && begin <= beginAndEndHour[1];
      });
      if (dayItem.groupValues.length === 0) {
        willDel.push(index);
      }
    });
    willDel.forEach((item, index) => {
      origin.splice(item - index, 1);
    });
    return origin;
  },
  // 过滤冲突数据
  filterConflictPlus(list) {
    if (list.length > 1) {
      return _fn.compareListConflict([], list);
    } else {
      return list;
    }
  },
  // 比较list内的每一项是否有冲突
  compareListConflict(retList, list) {
    let compareResult;
    let compare = list.shift();
    let len = list.length;
    for (let i = 0; i < len; i++) {
      compareResult = _fn.compareConflict(compare, list[i]);
      if (compareResult) {
        list.splice(i, 1);
        list.unshift(compareResult);
        return _fn.compareListConflict(retList, list);
      }
    }
    retList.push(compare);
    if (list.length > 1) {
      return _fn.compareListConflict(retList, list);
    } else {
      return retList.concat(list);
    }
  },
  // 比较2课程是否有冲突
  compareConflict(first, second) {
    let firstRange = [
      new Date(`${first.courseDate} ${first.range[0]}`).getTime(),
      new Date(`${first.courseDate} ${first.range[1] === '00:00' ? '24:00' : first.range[1]}`).getTime(),
    ];
    let secondRange = [
      new Date(`${second.courseDate} ${second.range[0]}`).getTime(),
      new Date(`${second.courseDate} ${second.range[1] === '00:00' ? '24:00' : second.range[1]}`).getTime(),
    ];
    let compareList = firstRange.concat(secondRange).sort();
    if (
      (compareList[0] === firstRange[0] && compareList[1] === firstRange[1]) ||
      (compareList[2] === firstRange[0] && compareList[3] === firstRange[1])
    ) {
      return null;
    } else {
      let beginTime = moment(compareList[0]).format('HH:mm');
      let endTime = moment(compareList[3]).format('HH:mm');
      return {
        id: `${first.id}_${second.id}`,
        range: [beginTime, endTime],
        statusName: '冲突课程',
        status: 'CONFLICT',
        beginTime,
        endTime,
        liveRoomId: first.liveRoomId,
        courseDate: first.courseDate,
        list: first.status === 'CONFLICT' ? [...first.list, second] : [first, second],
      };
    }
  },
  // 过滤课表内星期数/直播间没有数据的行列
  filterNoDataList(origin, liveRoomList, weekDateList) {
    origin = _fn.filterOutTimeRangeData(origin);
    let _weekDateList = origin.map((item) => item.groupKey);
    let _liveRoomList = [];
    origin.forEach((dayItem) => {
      dayItem.groupValues.forEach((item) => {
        if (!_liveRoomList.includes(item.liveRoomId)) {
          _liveRoomList.push(item.liveRoomId);
        }
      });
    });
    return {
      row: weekDateList.filter((item) => !_weekDateList.includes(item.date)).map((item) => item.id),
      col: liveRoomList.filter((item) => !_liveRoomList.includes(item)),
    };
  },
  // 没有liveRoomId的课室标记为无教室，即-1
  filterNoLiveRoomId(list) {
    let temp = [];
    list.forEach((value) => {
      temp.push({
        groupKey: value.groupKey,
        groupValues: value.groupValues.map((value) => {
          let tempData = clone(value);
          if (value.liveRoomId === '') {
            tempData.liveRoomId = '-1';
          }
          return tempData;
        }),
      });
    });
    return temp;
  },
  /**
   * @param {Object} state state对象的值
   * @param {Boolean} menuStatus 切换开关的状态
   * @param {Boolean} isDefault 是否需要返回默认值
   * @description
   * @return {Object} 返回一个Default对象
   * **/
  getDefaultValue(state, menuStatus, isDefault) {
    let date = 'sKey_date',
      beginDate = 'sKey_beginDate',
      endDate = 'sKey_endDate',
      keyId = 'sKey_mainTc';
    if (!menuStatus) {
      date = 'slave_date';
      beginDate = 'slave_beginDate';
      endDate = 'slave_endDate';
      keyId = 'slave_campus';
    }
    let todayStr = moment(new Date()).format('YYYY-MM-DD');
    let weekRange = _fn.getDateWeekRange(todayStr);
    return {
      [date]: isDefault ? todayStr : state[date] || todayStr,
      [beginDate]: isDefault ? weekRange[0] : state[beginDate] || weekRange[0],
      [endDate]: isDefault ? weekRange[1] : state[endDate] || weekRange[1],
      [keyId]: isDefault ? null : state[keyId],
      switchFilter: isDefault ? false : state.switchFilter,
    };
  },
};

export default {
  namespace: storeName,
  state: {
    sKey_default: null,
    // 主班选择框数据
    sKey_date: null,
    sKey_beginDate: null,
    sKey_endDate: null,
    sKey_mainTc: null,
    // 辅班选择框数据
    slave_date: null,
    slave_beginDate: null,
    slave_endDate: null,
    slave_campus: null,
    slave_default_campus: null, // 默认的campus值
    // 主班课表数据
    courseMaster: {
      courseData: null,
      courseData_beginDate: null,
      courseData_endDate: null,
      courseDate_teacher: null,
    },
    // 辅班课表数据
    courseSlave: {
      courseData: null,
      courseData_beginDate: null,
      courseData_endDate: null,
      courseDate_campus: null,
    },
    // 接口返回的主班课表数据
    originCourseData: null,
    // 接口返回的辅班课表数据
    originCourseSlaveData: null,
    // 切换显示有课程的信息
    switchFilter: false,
    filterMasterRowList: [],
    filterMasterColList: [],
    filterSlaveRowList: [],
    filterSlaveColList: [],
    liveRoom: [],
    menuStatus: true, // tab页切换，true为主班，false为辅班
  },
  reducers: {
    // 切换主辅课表state
    changeState(state, { payload }) {
      let defaultSKey = _fn.getDefaultValue(state, payload.menuStatus, false);
      return {
        ...state,
        ...payload,
        sKey_default: defaultSKey,
      };
    },
    // 更新state
    updateState(state, { payload }) {
      return {
        ...state,
        ...payload,
      };
    },
    //
    updateLiveRoom(state, { payload: { list } }) {
      let liveRoom = list.map((item) => ({
        id: item.id,
        name: item.name,
        subName: item.data,
        disabled: false,
      }));
      return {
        ...state,
        liveRoom,
      };
    },
    // 更新日期筛选项及周起始日期
    updateDate(state, { payload }) {
      let beginDate = 'sKey_beginDate',
        endDate = 'sKey_endDate',
        date = 'sKey_date';
      if (!state.menuStatus) {
        beginDate = 'slave_beginDate';
        endDate = 'slave_endDate';
        date = 'slave_date';
      }
      let _payload = {
        ...state,
        [date]: payload,
      };
      if (payload) {
        let weekRange = _fn.getDateWeekRange(payload);
        _payload[beginDate] = weekRange[0];
        _payload[endDate] = weekRange[1];
      }
      return _payload;
    },
    // 更新双师主班课表数据
    updateMasterCourseList(state, { payload: { list, beginDate, endDate, teacher } }) {
      let courseData = null;
      // 过滤掉开始时间在配置时间的起始外的数据
      let originCourseData = _fn.filterOutTimeRangeData(list);
      originCourseData = _fn.filterNoLiveRoomId(originCourseData);
      let originList = clone(originCourseData);
      if (originList.length > 0) {
        courseData = {};
        let tempData = {};
        let dayList = [];
        originList.forEach((item) => {
          let dayId = _fn.getDayIdByDay[new Date(item.groupKey).getDay()];
          courseData[dayId] = {};
          tempData[dayId] = item.groupValues;
          dayList.push(dayId);
        });
        dayList.forEach((day) => {
          // 转数据格式
          let tempList = tempData[day].map((item) => ({
            id: `${item.courseStartTime}_${item.courseEntTime}_${item.classId}_${item.teacherId}`,
            range: [item.courseStartTime, item.courseEntTime],
            statusName: item.courseStatusName.indexOf('已考勤') === -1 ? item.courseStatusName : '已考勤',
            status: item.courseStatus,
            beginTime: item.courseStartTime,
            endTime: item.courseEntTime,
            className: item.className,
            teacherName: item.teacherName,
            teacherTwoName: item.teacherTwoName,
            liveRoomId: item.liveRoomId,
            courseDate: item.courseDate,
            _origin: item,
          }));
          // 按直播间分组
          let tempByLiveRoom = {};
          tempList.forEach((item) => {
            if (!tempByLiveRoom[item.liveRoomId]) {
              tempByLiveRoom[item.liveRoomId] = [];
            }
            tempByLiveRoom[item.liveRoomId].push(item);
          });
          // 判断各直播间是否存在冲突时间段
          tempList = Object.keys(tempByLiveRoom).map((item) => _fn.filterConflictPlus(tempByLiveRoom[item]));
          if (tempList.length > 0) {
            tempList = tempList.reduce((accumulator, currentValue) => accumulator.concat(currentValue));
          }
          // 生成最终数据
          tempList.forEach((item) => {
            if (!courseData[day][item.liveRoomId]) {
              courseData[day][item.liveRoomId] = [];
            }
            courseData[day][item.liveRoomId].push(item);
          });
        });
        tempData = null;
        dayList = null;
      }
      let courseMaster = {
        courseData: courseData,
        courseData_beginDate: beginDate,
        courseData_endDate: endDate,
        courseDate_teacher: teacher,
      };
      return {
        ...state,
        courseMaster,
        originCourseData,
      };
    },
    // 更新双师辅班课表数据
    updateSlaveCourseList(state, { payload: { list, beginDate, endDate, campus } }) {
      let courseData = null;
      // 过滤掉开始时间在配置时间的起始外的数据
      let originCourseSlaveData = _fn.filterOutTimeRangeData(list);
      originCourseSlaveData = _fn.filterNoLiveRoomId(originCourseSlaveData);
      let originList = clone(originCourseSlaveData);
      if (originList.length > 0) {
        courseData = {};
        let tempData = {};
        let dayList = [];
        originList.forEach((item) => {
          let dayId = _fn.getDayIdByDay[new Date(item.groupKey).getDay()];
          courseData[dayId] = {};
          tempData[dayId] = item.groupValues;
          dayList.push(dayId);
        });
        dayList.forEach((day) => {
          // 转数据格式
          let tempList = tempData[day].map((item) => ({
            id: `${item.courseStartTime}_${item.courseEntTime}_${item.classId}_${item.teacherTwoId}`,
            range: [item.courseStartTime, item.courseEntTime],
            statusName: item.courseStatusName.indexOf('已考勤') === -1 ? item.courseStatusName : '已考勤',
            status: item.courseStatus,
            beginTime: item.courseStartTime,
            endTime: item.courseEntTime,
            className: item.className,
            classTwoName: item.classTwoName,
            teacherName: item.teacherName,
            teacherTwoName: item.teacherTwoName,
            liveRoomId: item.liveRoomId,
            courseDate: item.courseDate,
            _origin: item,
            studentNumber: item.studentNumber,
          }));
          // 按直播间分组
          let tempByLiveRoom = {};
          tempList.forEach((item) => {
            if (!tempByLiveRoom[item.liveRoomId]) {
              tempByLiveRoom[item.liveRoomId] = [];
            }
            tempByLiveRoom[item.liveRoomId].push(item);
          });
          // 判断各直播间是否存在冲突时间段
          tempList = Object.keys(tempByLiveRoom).map((item) => _fn.filterConflictPlus(tempByLiveRoom[item]));
          if (tempList.length > 0) {
            tempList = tempList.reduce((accumulator, currentValue) => accumulator.concat(currentValue));
          }
          // 生成最终数据
          tempList.forEach((item) => {
            if (!courseData[day][item.liveRoomId]) {
              courseData[day][item.liveRoomId] = [];
            }
            courseData[day][item.liveRoomId].push(item);
          });
        });
        tempData = null;
        dayList = null;
      }
      let courseSlave = {
        courseData: courseData,
        courseData_beginDate: beginDate,
        courseData_endDate: endDate,
        courseDate_campus: campus,
      };
      return {
        ...state,
        courseSlave,
        originCourseSlaveData,
      };
    },
    // 设置campus默认值
    setDefaultCampus(state, { payload }) {
      return {
        ...state,
        ...payload,
      };
    },
  },
  effects: {
    *initLiveRoom({ payload: { callback } }, { put }) {
      const rsp = yield getListDataDict({ category: LIVE_ROOM }, true).catch(errorsHandle());
      if (rsp && rsp.resultCode === 0) {
        yield put({
          type: 'updateLiveRoom',
          payload: {
            list: rsp.data || [],
          },
        });
      }
      runFunc(callback);
    },
    // 初始化筛选项
    *setDefaultSKey({ payload: { callback } }, { select, put }) {
      const defaultSKey = yield select((state) => state[storeName].sKey_default);
      const slaveDefaultCampus = yield select((state) => state[storeName].slave_default_campus);
      if (defaultSKey) {
        yield put({
          type: 'updateState',
          payload: defaultSKey,
        });
      } else {
        let todayStr = moment(new Date()).format('YYYY-MM-DD');
        let weekRange = _fn.getDateWeekRange(todayStr);
        let defaultMaster = {
          sKey_date: todayStr,
          sKey_beginDate: weekRange[0],
          sKey_endDate: weekRange[1],
          sKey_mainTc: null,
          switchFilter: false,
        };
        let defaultSlave = {
          slave_date: todayStr,
          slave_beginDate: weekRange[0],
          slave_endDate: weekRange[1],
          slave_campus: slaveDefaultCampus,
          switchFilter: false,
        };
        yield put({
          type: 'updateState',
          payload: {
            ...defaultMaster,
            ...defaultSlave,
            sKey_default: defaultMaster,
          },
        });
      }
      if (callback) callback();
    },
    // 加载双师主班课表数据
    *loadMasterCourseList({ payload: { callback } }, { select, put }) {
      let apiParams = yield select((state) => ({
        startDate: state[storeName].sKey_beginDate,
        endDate: state[storeName].sKey_endDate,
        teacherId: state[storeName].sKey_mainTc,
      }));
      const rsp = yield getTwoTeacherClassCourseLiveRoom(apiParams, true).catch(_fn.errorsHandling());
      if (rsp && rsp.resultCode === 0) {
        yield put({
          type: 'updateMasterCourseList',
          payload: {
            list: rsp.data || [],
            beginDate: apiParams.startDate,
            endDate: apiParams.endDate,
            teacher: apiParams.teacherId,
          },
        });
      }
      if (callback) callback();
    },
    // 加载双师辅班课表数据
    *loadSlaveCourseList({ payload: { callback, campusId } }, { select, put }) {
      let apiParams = yield select((state) => ({
        startDate: state[storeName].slave_beginDate,
        endDate: state[storeName].slave_endDate,
        campusId: state[storeName].slave_campus || campusId,
      }));
      const rsp = yield getTwoTeacherClassController(apiParams, true).catch(_fn.errorsHandling());
      if (rsp && rsp.resultCode === 0) {
        yield put({
          type: 'updateSlaveCourseList',
          payload: {
            list: rsp.data || [],
            beginDate: apiParams.startDate,
            endDate: apiParams.endDate,
            campus: apiParams.campusId,
          },
        });
      }
      if (callback) callback();
    },
    // 重置筛选项
    *resetSKey(action, { select, put }) {
      const state = yield select((state) => state[storeName]);
      const menuStatus = yield select((state) => state[storeName].menuStatus);
      let defaultSKey = _fn.getDefaultValue(state, menuStatus, true);
      yield put({
        type: 'updateState',
        payload: defaultSKey,
      });
    },
    // 更新按天、直播间过滤没有数据的行列
    *filterCourseTable({ payload: { checked, callback } }, { select, put }) {
      let payload = { switchFilter: checked };
      const menuStatus = yield select((state) => state[storeName].menuStatus);
      const campus = yield select((state) => state[storeName].slave_campus || state[storeName].slave_default_campus);
      if (checked) {
        let courseObjKeyName = menuStatus ? 'courseMaster' : 'courseSlave';
        const [beginDate, origin, liveRoomList] = yield select((state) => [
          state[storeName][`${courseObjKeyName}`].courseData_beginDate,
          menuStatus ? state[storeName].originCourseData : state[storeName].originCourseSlaveData,
          menuStatus
            ? state.dict.liveRoom.map((item) => item.val)
            : (campus ? state.dict.classRoomsByCampusId[campus] : []).map((item) => item.val),
        ]);
        let filterResult = _fn.filterNoDataList(origin, liveRoomList, buildDateConfig(beginDate));
        if (menuStatus) {
          payload.filterMasterRowList = filterResult.row;
          payload.filterMasterColList = filterResult.col;
        } else {
          payload.filterSlaveRowList = filterResult.row;
          payload.filterSlaveColList = filterResult.col;
        }
      } else {
        if (menuStatus) {
          payload.filterMasterRowList = [];
          payload.filterMasterColList = [];
        } else {
          payload.filterSlaveRowList = [];
          payload.filterSlaveColList = [];
        }
      }
      yield put({
        type: 'updateState',
        payload,
      });
      if (callback) callback();
    },
  },
};
