/**
 * Created by Haolin<haolinhom@gmail.com> on 2019/5/20.
 */
import moment from 'moment';
import { Monday, Tuesday, Wednesday, Thursday, Friday, Saturday, Sunday, hourRanges, hourKeys } from '../config';
//
const getDayByIndex = {
  0: Monday,
  1: Tuesday,
  2: Wednesday,
  3: Thursday,
  4: Friday,
  5: Saturday,
  6: Sunday,
};
//
function buildDictObj(dict) {
  return function () {
    let obj = {};
    dict.forEach((item) => {
      obj[item.val] = '';
    });
    return new Object(obj);
  };
}
//
function hourKeyByBeginTime(begin) {
  let beginHour = Number(begin.slice(0, 2));
  return beginHour % 2 === 0 ? `${beginHour}${beginHour + 1}` : `${beginHour - 1}${beginHour}`;
}
// 构建
function buildDateConfig(beginDateStr) {
  const oneDayTime = 86400000;
  let timestamp = new Date(beginDateStr).getTime();
  let dateTimeList = [timestamp];
  for (let i = 1; i < 7; i++) {
    dateTimeList.push(timestamp + oneDayTime * i);
  }
  return dateTimeList.map((item, index) => ({
    date: moment(item).format('YYYY-MM-DD'),
    name: getDayByIndex[index].text,
    id: getDayByIndex[index].id,
  }));
}
//
function buildTableHead(dict) {
  const firstRow = ['日期', '上课日期'];
  return firstRow.concat(dict.map((item) => item.text));
}
//
function buildExportTableData(rspList, dateConfig, dict, menuStatus) {
  const createLiveRoom = buildDictObj(dict);
  let temp = {};
  dateConfig.forEach((dateCfg) => {
    temp[dateCfg.date] = {};
    hourKeys.forEach((hourRange, index) => {
      temp[dateCfg.date][hourRange] = {
        list: [],
        liveRoom: createLiveRoom(),
      };
      temp[dateCfg.date][hourRange].list = [];
      if (index === 0) {
        temp[dateCfg.date][hourRange].list.push(`${dateCfg.date}<br/>${dateCfg.name}`);
      }
      temp[dateCfg.date][hourRange].list.push(hourRanges[index]);
    });
  });
  rspList.forEach((dayObj) => {
    dayObj.groupValues.forEach((item) => {
      if (temp[dayObj.groupKey][hourKeyByBeginTime(item.courseStartTime)]) {
        if (temp[dayObj.groupKey][hourKeyByBeginTime(item.courseStartTime)].liveRoom[item.liveRoomId] !== '') {
          temp[dayObj.groupKey][hourKeyByBeginTime(item.courseStartTime)].liveRoom[item.liveRoomId] += '，<br/>';
        }
        temp[dayObj.groupKey][hourKeyByBeginTime(item.courseStartTime)].liveRoom[item.liveRoomId] += `${
          menuStatus ? item.className : item.classTwoName + `（${item.studentNumber}人）`
        }<br/>${item.courseStartTime}-${item.courseEntTime}<br/>课程老师：${item.teacherName}`;
        if (!menuStatus) {
          temp[dayObj.groupKey][hourKeyByBeginTime(item.courseStartTime)].liveRoom[
            item.liveRoomId
          ] += `<br/>辅导老师：${item.teacherTwoName}`;
        }
      }
    });
  });
  let tempList = Object.values(temp);
  return tempList.map((rangeObj) => {
    return Object.values(rangeObj).map((item) => item.list.concat(Object.values(item.liveRoom)));
  });
}
//
function buildTableDOM(headOrigin, bodyOrigin, tableHeadContent) {
  let head = `<tr>${headOrigin.map((item) => `<td>${item}</td>`).join('')}</tr>`;
  let body = bodyOrigin
    .map((dayItem) =>
      dayItem
        .map((timeItem, timeIndex, dayList) => {
          return `<tr>${timeItem
            .map((item, index) =>
              timeIndex === 0 && index === 0 ? `<td rowspan="${dayList.length}">${item}</td>` : `<td>${item}</td>`,
            )
            .join('')}</tr>`;
        })
        .join(''),
    )
    .join('');
  let tableHead = tableHeadContent
    ? `<thead><tr><th colspan="${headOrigin.length}">${tableHeadContent}</th></tr></thead>`
    : '';
  return `<html lang="zh-CN"><head><title>${
    tableHeadContent || '双师主辅班课表'
  }</title><style>table {border-collapse: collapse;}th, td {border: 1px solid #333;text-align: center;padding: 0 8px;}</style></head><body><table>${tableHead}<tbody>${head}${body}</tbody></table></body></html>`;
}
//
function exportExcel(data, fileName, callback) {
  var reader = new FileReader();
  reader.addEventListener(
    'load',
    function () {
      let a = document.createElement('a');
      a.href = reader.result;
      a.download = `${fileName}.xls`;
      document.body.appendChild(a);
      a.click();
      a.remove();
      if (callback) callback();
    },
    false,
  );
  if (data) {
    reader.readAsDataURL(new Blob([data], { type: 'application/vnd.ms-excel' }));
  }
}

export { buildDateConfig, hourKeyByBeginTime, buildTableHead, buildExportTableData, buildTableDOM, exportExcel };
