/**
 * Created by Haolin on 2019/1/3.
 */
import React from 'react';
import { connect } from 'dva';
import { Row, Col } from 'antd';
import { storeName } from '../../store/constant.js';
import Loading from 'COMPONENT/Loading';
import Input from 'COMPONENT/Input';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import DatePicker from 'COMPONENT/DatePicker';
import TextLink from 'COMPONENT/TextLink';
import Label from '../../components/Label';
import style from './style.less';

const ColChildWrap = (props) => {
  const classPrefix = 'ColChildWrap';
  return <div className={style[classPrefix]}>{props.children}</div>;
};

const LectureInfo = (props) => {
  const {
    dict,
    data: { name, grade, subject, mainTc, beginDate },
    index,
    onChange,
    error,
    checkValidator,
    handleValidator,
    action,
  } = props;
  const classPrefix = 'LectureInfo';
  return (
    <Row className={style[classPrefix]}>
      <Col span={8} className={style[`${classPrefix}-col`]}>
        <ColChildWrap>
          <Input
            placeholder="讲座名称"
            value={name}
            onChange={(ele) => {
              onChange({ type: 'name', val: ele.target.value, index });
            }}
            maxLength={30}
            validator={{
              rules: ['required'],
              silent: !Boolean(error[index]['name']) || checkValidator,
              showStatus: true,
              showTips: false,
              onErrorChange: (err) => {
                handleValidator({ type: 'name', val: err, index });
              },
            }}
          />
        </ColChildWrap>
      </Col>
      <Col span={4}>
        <ColChildWrap>
          <MultiSelect
            placeholder="年级"
            dict={dict.grade}
            value={grade}
            onChange={(val) => {
              onChange({ type: 'grade', val, index });
            }}
            showSearch={true}
            validator={{
              rules: ['required'],
              silent: !Boolean(error[index]['grade']) || checkValidator,
              showStatus: true,
              showTips: false,
              onErrorChange: (err) => {
                handleValidator({ type: 'grade', val: err, index });
              },
            }}
          />
        </ColChildWrap>
      </Col>
      <Col span={4}>
        <ColChildWrap>
          <MultiSelect
            placeholder="科目"
            dict={dict.subject}
            value={subject}
            onChange={(val) => {
              onChange({ type: 'subject', val, index });
            }}
            showSearch={true}
            validator={{
              rules: ['required'],
              silent: !Boolean(error[index]['subject']) || checkValidator,
              showStatus: true,
              showTips: false,
              onErrorChange: (err) => {
                handleValidator({ type: 'subject', val: err, index });
              },
            }}
          />
        </ColChildWrap>
      </Col>
      <Col span={4}>
        <ColChildWrap>
          <Select
            placeholder="主讲老师"
            dict={dict.mainTcDict}
            value={mainTc}
            onChange={(val) => {
              onChange({ type: 'mainTc', val, index });
            }}
            showSearch={true}
            validator={{
              rules: ['required'],
              silent: !Boolean(error[index]['mainTc']) || checkValidator,
              showStatus: true,
              showTips: false,
              onErrorChange: (err) => {
                handleValidator({ type: 'mainTc', val: err, index });
              },
            }}
          />
        </ColChildWrap>
      </Col>
      <Col span={4}>
        <ColChildWrap>
          <DatePicker
            placeholder="上课日期"
            value={beginDate}
            onChange={(val) => {
              onChange({ type: 'beginDate', val, index });
            }}
            validator={{
              rules: ['required'],
              silent: !Boolean(error[index]['beginDate']) || checkValidator,
              showStatus: true,
              showTips: false,
              onErrorChange: (err) => {
                handleValidator({ type: 'beginDate', val: err, index });
              },
            }}
          />
        </ColChildWrap>
      </Col>
      {/*<Col span={3}>*/}
      {/*<ColChildWrap>*/}
      {/*<TextLink fontSize={14} onClick={() => { action('type', index) }}>action</TextLink>*/}
      {/*</ColChildWrap>*/}
      {/*</Col>*/}
    </Row>
  );
};

class LectureInfoList extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false,
      error: [{}],
      checkValidator: true,
    };
    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.btnAction = this.btnAction.bind(this);
  }
  static classPrefix = 'LectureInfoList';
  componentDidMount() {
    this.init();
  }
  componentWillUnmount() {}

  async init() {
    this.setState({ loading: true });
    await this.initDict();
    this.setState({ loading: false });
  }
  initDict() {
    const { dispatch } = this.props;
    return new Promise((resolve, reject) => {
      dispatch({
        type: 'dict/initDict',
        payload: {
          list: ['grade', 'subject', 'allTwoTcMainTc'],
          callback() {
            resolve();
          },
        },
      });
    });
  }
  // 列表操作
  btnAction(type, index) {}

  render() {
    const { data, dict, handleUpdate, error, checkValidator, handleValidator } = this.props;
    const { loading } = this.state;
    const prefix = LectureInfoList.classPrefix;

    return (
      <div className={style[prefix]}>
        <Loading loading={loading} />
        <div>
          <Row>
            <Col span={8}>
              <ColChildWrap>
                <Label required>讲座名称</Label>
              </ColChildWrap>
            </Col>
            <Col span={4}>
              <ColChildWrap>
                <Label required>年级</Label>
              </ColChildWrap>
            </Col>
            <Col span={4}>
              <ColChildWrap>
                <Label required>科目</Label>
              </ColChildWrap>
            </Col>
            <Col span={4}>
              <ColChildWrap>
                <Label required>主讲老师</Label>
              </ColChildWrap>
            </Col>
            <Col span={4}>
              <ColChildWrap>
                <Label required>上课日期</Label>
              </ColChildWrap>
            </Col>
            {/*<Col span={3}>*/}
            {/*<ColChildWrap>*/}
            {/*<Label>操作</Label>*/}
            {/*</ColChildWrap>*/}
            {/*</Col>*/}
          </Row>
          {data.map((item, index) => (
            <LectureInfo
              dict={dict}
              data={item}
              index={index}
              onChange={handleUpdate}
              error={error}
              checkValidator={checkValidator}
              handleValidator={handleValidator}
              key={index}
            />
          ))}
        </div>
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  const { grade, subject, allTwoTcMainTc } = state.dict;
  return {
    dict: {
      grade,
      subject,
      mainTcDict: allTwoTcMainTc,
    },
  };
};

export default connect(mapStateToProps)(LectureInfoList);
