/**
 * Created by Haolin on 2018/12/26.
 */
import React from 'react';
import { connect } from 'dva';
import { Row, Col } from 'antd';
import Page from 'COMPONENT/Page';
import FormItem from 'COMPONENT/FormItem';
import Button from 'COMPONENT/Button';
import Section from '../components/Section';
import FlexRow from '../components/FlexRow';
import ListWithDropDownMenu from '../components/ListWithDropDownMenu';
import ModalSelectProduct from '../ModalSelectProduct';
import LectureInfoList from './LectureInfoList';
import { storeName } from '../store/constant.js';
import style from './style.less';

class AddLecture extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      name: '新增线上讲座',
      loading: false,
      toggle_selectPrd: false,
      // 由于store中state.addLecture_lectureList默认存在一项，故此处也需一个空{}
      error: [{}],
      checkValidator: true,
    };
    this.lectureActionType = 'add';
    this.switchSelectPrd = this.switchSelectPrd.bind(this);
    this.confirmSelectPrd = this.confirmSelectPrd.bind(this);
    this.updateLectureInfo = this.updateLectureInfo.bind(this);
    this.errorChange = this.errorChange.bind(this);
    this.checkBeforeSubmit = this.checkBeforeSubmit.bind(this);
    this.submit = this.submit.bind(this);
    this.goToMainPage = this.goToMainPage.bind(this);
    this.destroy = this.destroy.bind(this);
  }
  static classPrefix = 'AddLecture';
  componentDidMount() {}
  componentWillUnmount() {
    this.destroy();
  }

  switchSelectPrd() {
    this.setState((prevState) => ({
      toggle_selectPrd: !prevState.toggle_selectPrd,
    }));
  }
  confirmSelectPrd() {
    let type = this.lectureActionType;
    const _self = this;
    this.props.dispatch({
      type: `${storeName}/confirmProduct`,
      payload: {
        type,
        callback() {
          _self.switchSelectPrd();
        },
      },
    });
  }

  updateLectureInfo({ type, val, index }) {
    this.props.dispatch({
      type: `${storeName}/updateLectureInfo`,
      payload: { type, val, index },
    });
  }
  // 用于校验表单是否存在错误
  errorChange({ type, val, index }) {
    this.setState((prevState) => {
      let error = prevState.error;
      error[index][type] = val;
      return { error };
    });
  }
  // 校验输入
  checkBeforeSubmit() {
    const { error } = this.state;
    let result = true;
    error.forEach((err) => {
      let errResult = Object.values(err).every((item) => !Boolean(item));
      if (!errResult) {
        result = false;
      }
    });
    this.setState({ checkValidator: result });
    return result;
  }
  // 提交
  submit() {
    // 校验讲座产品，此处为兼容旧Page组件，防止他人组件库未更新影响页面逻辑，后期可以删掉
    if (!this.props.checkSelectPrd) {
      this.pageRef.showOkTips('请先选择讲座产品！');
      return;
    }
    // 校验讲座信息
    let check = this.checkBeforeSubmit();
    if (!check) {
      this.pageRef.showOkTips('您有信息未填写，请补充完整！');
      return;
    }
    this.setState({ loading: true });
    const _self = this;
    this.props.dispatch({
      type: `${storeName}/submitNewLecture`,
      payload: {
        callback() {
          _self.goToMainPage();
        },
      },
    });
  }
  // 跳转回讲座主列表页
  goToMainPage() {
    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path: '/twoTeacherLecture',
        params: {},
        refresh: true,
      },
    });
  }

  destroy() {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        addLecture_productInfo: {},
        addLecture_productList: [],
        addLecture_lectureList: [
          {
            name: '',
            grade: [],
            subject: [],
            teacher: null,
            beginDate: null,
          },
        ],
      },
    });
  }

  render() {
    const {
      route,
      productNameList,
      checkSelectPrd,
      productYear,
      productClassType,
      productPrice,
      productTotalHour,
      list,
    } = this.props;
    const { name, loading, toggle_selectPrd, error, checkValidator } = this.state;

    return (
      <Page
        name={name}
        className={style[AddLecture.classPrefix]}
        loading={loading}
        route={route}
        footer={true}
        okBtnDisabled={!checkSelectPrd}
        okBtnText="保存"
        onOk={this.submit}
        ref={(ref) => (this.pageRef = ref)}
        fitWindowHeight={true}
        subHeader={
          <Section title="讲座产品信息">
            <Row>
              <Col span={9}>
                <FormItem name="产品名称" labelWidth={80} required>
                  <FlexRow
                    fitWidth={70}
                    fitNode={
                      <Button type="primary" size="small" onClick={this.switchSelectPrd}>
                        选择产品
                      </Button>
                    }
                  >
                    {checkSelectPrd && <ListWithDropDownMenu data={productNameList} />}
                  </FlexRow>
                </FormItem>
              </Col>
              <Col span={9}>
                <FormItem name="产品年份" labelWidth={80}>
                  {productYear}
                </FormItem>
              </Col>
              <Col span={6}>
                <FormItem name="班级类型" labelWidth={80}>
                  {productClassType}
                </FormItem>
              </Col>
            </Row>
            <Row>
              <Col span={9}>
                <FormItem name="课时单价" labelWidth={80}>
                  {productPrice}
                </FormItem>
              </Col>
              <Col span={9}>
                <FormItem name="课时时长" labelWidth={80}>
                  {productTotalHour}
                </FormItem>
              </Col>
              <Col span={6}> </Col>
            </Row>
          </Section>
        }
      >
        <Section title="讲座课堂信息">
          {checkSelectPrd ? (
            <LectureInfoList
              data={list}
              handleUpdate={this.updateLectureInfo}
              handleSubmit={this.submit}
              error={error}
              checkValidator={checkValidator}
              handleValidator={this.errorChange}
            />
          ) : (
            '请先选择讲座产品'
          )}
        </Section>
        {toggle_selectPrd && (
          <ModalSelectProduct
            type={this.lectureActionType}
            needInit={checkSelectPrd}
            onCancel={this.switchSelectPrd}
            onConfirm={this.confirmSelectPrd}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const { addLecture_productInfo, addLecture_productList, addLecture_lectureList } = state[storeName];
  let checkSelectPrd = addLecture_productList.length > 0;
  let productNameList = checkSelectPrd ? addLecture_productList.map((item) => item.name) : [];
  let autoText = '自动匹配';
  let productYear = addLecture_productInfo.year ? addLecture_productInfo.year.text : autoText;
  let productClassType = addLecture_productInfo.classType ? addLecture_productInfo.classType.text : autoText;
  let productPrice = addLecture_productInfo.price ? addLecture_productInfo.price.text : autoText;
  let productTotalHour = addLecture_productInfo.totalHour ? addLecture_productInfo.totalHour.text : autoText;

  return {
    productInfo: addLecture_productInfo,
    productList: addLecture_productList,
    productNameList,
    list: addLecture_lectureList,
    checkSelectPrd,
    productYear,
    productClassType,
    productPrice,
    productTotalHour,
  };
};

export default connect(mapStateToProps)(AddLecture);
