/**
 * Created by Haolin on 2019/1/7.
 */
import React from 'react';
import { connect } from 'dva';
import { Row, Col } from 'antd';
import { storeName } from '../../store/constant.js';
import Loading from 'COMPONENT/Loading';
import Button from 'COMPONENT/Button';
import FormItem from 'COMPONENT/FormItem';

import ListWithDropDownMenu from '../../components/ListWithDropDownMenu';
import ListWithPop from '../../components/ListWithPop';
import style from './index.less';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import UploadImg from '../../../../course/modals/UploadImg';
import { saveLectureClassAttendanceFile, getLectureClassAttendanceFile } from 'SERVICE/course';

const InfoText = (props) => {
  const { text } = props;
  const prefix = 'InfoText';
  return <span className={style[`${prefix}-text`]}>{text || '-'}</span>;
};

class LectureDetail extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false,
      uploadRef: null,
      fileList: [],
      fileList2: null,
      previewSrc: '',
      showpreviewImgs: false,
    };
    this.init = this.init.bind(this);
    this.initDetail = this.initDetail.bind(this);
    this.destroy = this.destroy.bind(this);
  }
  static classPrefix = 'LectureDetail';
  componentDidMount() {
    this.init();
    getLectureClassAttendanceFile({ lectureClassId: this.props.id }).then((res) => {
      let fileList = res.data.attendanceFileRecords || [];
      fileList = fileList.map((file) => {
        return {
          ...file,
          iconSrc: file.filePath,
          src: file.filePath,
          'x:fileid': file.uploadFileRecordId,
        };
      });

      this.setState({
        fileList,
      });
      this.props.onFileChange(fileList);
    });
  }
  componentWillUnmount() {
    this.destroy();
  }

  async init() {
    this.setState({ loading: true });
    await this.initDetail();
    this.setState({ loading: false });
  }
  initDetail() {
    const { dispatch, id } = this.props;
    return new Promise((resolve, reject) => {
      dispatch({
        type: `${storeName}/getLectureDetail`,
        payload: {
          id,
          type: 'detail',
          callback() {
            resolve();
          },
          handleErr() {
            reject();
          },
        },
      });
    });
  }
  destroy() {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        lecture_detail: null,
      },
    });
  }
  uploadRequest = () => {
    let uploadFile = this.state.fileList.map((file) => {
      return {
        fileName: file.fileName || file['x:realname'],
        uploadFileRecordId: file.uploadFileRecordId || file['x:fileid'],
      };
    });

    saveLectureClassAttendanceFile({
      lectureClassId: this.props.id,
      attendanceFileRecords: uploadFile,
    })
      .then(() => {
        showMessage('保存成功', 'success');
        this.props.onFileChange(this.state.fileList);
      })
      .catch((e) => {
        showMessage(e.text);
      });
  };

  onFileUpload = (file) => {
    this.setState({ fileList: file });
  };

  render() {
    const {
      lectureName,
      productNameList,
      year,
      classType,
      totalHour,
      price,
      grade,
      subject,
      mainTc,
      beginDate,
      hasUploadPermission,
    } = this.props;

    const { loading, fileList } = this.state;
    const prefix = LectureDetail.classPrefix;

    return (
      <div className={style[prefix]}>
        <Loading loading={loading} />
        <Row>
          <Col span={12}>
            <FormItem name="讲座名称" labelWidth={80}>
              <InfoText text={lectureName} />
            </FormItem>
          </Col>
          <Col span={12}>
            <FormItem name="产品名称" labelWidth={80}>
              {productNameList.length > 0 ? (
                <ListWithDropDownMenu data={productNameList} />
              ) : (
                <InfoText text={productNameList[1]} />
              )}
            </FormItem>
          </Col>
        </Row>
        <Row>
          <Col span={6}>
            <FormItem name="产品年份" labelWidth={80}>
              <InfoText text={year} />
            </FormItem>
          </Col>
          <Col span={6}>
            <FormItem name="班级类型" labelWidth={80}>
              <InfoText text={classType} />
            </FormItem>
          </Col>
          <Col span={6}>
            <FormItem name="课时时长" labelWidth={80}>
              <InfoText text={totalHour} />
            </FormItem>
          </Col>
          <Col span={6}>
            <FormItem name="课时单价" labelWidth={80}>
              <InfoText text={price} />
            </FormItem>
          </Col>
        </Row>
        <Row>
          <Col span={6}>
            <FormItem name="年级" labelWidth={80}>
              <ListWithPop data={grade || []} />
            </FormItem>
          </Col>
          <Col span={6}>
            <FormItem name="科目" labelWidth={80}>
              <ListWithPop data={subject || []} />
            </FormItem>
          </Col>
          <Col span={6}>
            <FormItem name="主讲老师" labelWidth={80}>
              <InfoText text={mainTc} />
            </FormItem>
          </Col>
          <Col span={6}>
            <FormItem name="上课日期" labelWidth={80}>
              <InfoText text={beginDate} />
            </FormItem>
          </Col>
        </Row>
        <Row>
          <Col span={24}>
            <FormItem name="考勤照片" labelWidth={80}>
              <p className={style['uploadTip']}>需至少有1张考勤照片，才允许对该讲座中的学生进行考勤和扣费操作；</p>
              <p className={style['uploadTip']}>请拍摄讲座现场全部学生合照，照片一经保存不可删除，可保存多张照片。</p>
              <UploadImg
                value={fileList}
                text="点击上传"
                multiple
                hasDelete={false}
                max={99}
                size={100}
                uploadComplete={(val) => this.onFileUpload(val)}
                viewOnly={!hasUploadPermission}
              />

              <div className={style['fileListContainer']}>
                {hasUploadPermission && fileList.length ? (
                  <div>
                    <Button type="primary" onClick={this.uploadRequest}>
                      保存
                    </Button>{' '}
                  </div>
                ) : null}
              </div>
            </FormItem>
          </Col>
        </Row>
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  const { lecture_detail } = state[storeName];

  const btnAuth = state.account.btnAuths;
  const hasUploadPermission = btnAuth.indexOf('BTN_ONLINE_LECTURE_UPLOAD') > -1;

  const {
    lectureName,
    productList,
    year,
    classType,
    totalHour,
    price,
    grade,
    subject,
    mainTc,
    beginDate,
    lectureStatus,
  } = lecture_detail || {};
  let productNameList = (productList && productList.map((item) => item.productName)) || ['-'];

  return {
    lectureName,
    productNameList,
    year,
    classType,
    totalHour,
    price,
    grade,
    subject,
    mainTc,
    beginDate,
    lectureStatus,
    hasUploadPermission,
  };
};

export default connect(mapStateToProps)(LectureDetail);
