/**
 * Created by Haolin on 2018/1/7.
 */
import React, { PropTypes } from 'react';
import { connect } from 'dva';
import { storeName } from '../../../store/constant.js';
import SearchBar from 'COMPONENT/SearchBar';
import Input from 'COMPONENT/Input';
import Select from 'COMPONENT/Select';

class Search extends React.Component {
  constructor(props) {
    super(props);
    this.search = this.search.bind(this);
    this.reset = this.reset.bind(this);
    this.updateKeyword = this.updateKeyword.bind(this);
    this.destroy = this.destroy.bind(this);
  }
  static propTypes = {
    classPrefix: PropTypes.string,
    handleSearch: PropTypes.func,
  };
  componentWillUnmount() {
    this.destroy();
  }
  /**
   * 检索
   * */
  search() {
    this.props.handleSearch();
  }
  /**
   * 重置检索条件
   * */
  reset() {
    this.props.dispatch({
      type: `${storeName}/resetLectureStdKeyword`,
    });
  }
  /**
   * 更新检索条件
   * */
  updateKeyword(params) {
    const { type, val } = params;
    let payload = { [type]: val };
    let branchCampusDict = this.props.branchCampusDict.filter((item) => item.val === val);
    let campusDict =
      branchCampusDict && branchCampusDict[0] && branchCampusDict[0].options ? branchCampusDict[0].options : [];
    if (type === 'sKey_lecture_branch') {
      payload.sKey_lecture_campus = campusDict[0] ? campusDict[0].val : null;
    }
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload,
    });
  }
  /**
   * 注销前销毁
   * */
  destroy() {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        sKey_lecture_name: '',
        sKey_lecture_branch: null,
        sKey_lecture_campus: null,
        sKey_lecture_status: null,
        sKey_lecture_payStatus: null,
      },
    });
  }

  render() {
    const {
      className,
      branchDict,
      tcAttendanceDict,
      stdChargeStatus,
      branchCampusDict,
      sKey_lecture_name,
      sKey_lecture_branch,
      sKey_lecture_campus,
      sKey_lecture_status,
      sKey_lecture_payStatus,
    } = this.props;

    return (
      <div className={className}>
        <SearchBar onSearch={this.search} onReset={this.reset}>
          <Input
            placeholder="学生名称"
            value={sKey_lecture_name}
            onChange={(e) => {
              this.updateKeyword({ type: 'sKey_lecture_name', val: e.target.value });
            }}
          />
          <Select
            placeholder="分公司"
            dict={branchDict}
            value={sKey_lecture_branch}
            onChange={(val) => {
              this.updateKeyword({ type: 'sKey_lecture_branch', val });
            }}
            allowClear={false}
            showSearch
          />
          <Select
            placeholder="学习中心"
            dict={
              sKey_lecture_branch
                ? branchCampusDict && branchCampusDict.filter((item) => item.val === sKey_lecture_branch)
                : branchCampusDict
            }
            value={sKey_lecture_campus}
            onChange={(val) => {
              this.updateKeyword({ type: 'sKey_lecture_campus', val });
            }}
            allowClear={false}
            showSearch
          />
          <Select
            placeholder="是否考勤"
            dict={tcAttendanceDict}
            value={sKey_lecture_status}
            onChange={(val) => {
              this.updateKeyword({ type: 'sKey_lecture_status', val });
            }}
          />
          <Select
            placeholder="扣费状态"
            dict={stdChargeStatus}
            value={sKey_lecture_payStatus}
            onChange={(val) => {
              this.updateKeyword({ type: 'sKey_lecture_payStatus', val });
            }}
          />
        </SearchBar>
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  const { branch, branchCampus, lectureStdChargeStatus, baseStatus } = state.dict;
  const { sKey_lecture_name, sKey_lecture_branch, sKey_lecture_campus, sKey_lecture_status, sKey_lecture_payStatus } =
    state[storeName];
  return {
    branchDict: branch,
    branchCampusDict: branchCampus,
    stdChargeStatus: lectureStdChargeStatus,
    tcAttendanceDict: baseStatus,
    sKey_lecture_name,
    sKey_lecture_branch,
    sKey_lecture_campus,
    sKey_lecture_status,
    sKey_lecture_payStatus,
  };
};

export default connect(mapStateToProps)(Search);
