/**
 * Created by Haolin on 2019/1/7.
 */
import React from 'react';
import { connect } from 'dva';
import { Modal } from 'antd';
import { storeName } from '../../store/constant.js';
import style from './index.less';
import LinkBtnWithPop from '../../components/LinkBtnWithPop';
import List from 'COMPONENT/List';
import Button from 'COMPONENT/Button';
import Search from './Search';
import Wrapper from 'COMPONENT/Wrapper';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L } = List;
const confirm = Modal.confirm;
const BtnWrap = (props) => {
  const { node, nodeMarginRight, children } = props;
  const prefix = 'BtnWrap';
  return (
    <div className={style[prefix]}>
      <div style={{ marginRight: nodeMarginRight }}>{node}</div>
      <div className={style[`${prefix}-btnGroup`]}>{children}</div>
    </div>
  );
};
BtnWrap.defaultProps = {
  nodeMarginRight: 0,
};

class RegStdDetail extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false,
      listLoading: false,
      pageCurrent: 1,
      pageSize: 20,
      selectedRowKeys: [],
    };
    this.columns = [
      { title: '序号', dataIndex: 'number', key: 'number', width: COL_WIDTH_S },
      { title: '学生姓名', dataIndex: 'studentName', key: 'studentName', width: COL_WIDTH_M },
      { title: '学生编号', dataIndex: 'studentId', key: 'studentId', width: COL_WIDTH_L },
      { title: '分公司', dataIndex: 'branchName', key: 'branchName', width: COL_WIDTH_M },
      { title: '学习中心', dataIndex: 'campusName', key: 'campusName', width: COL_WIDTH_M },
      { title: '老师是否考勤', dataIndex: 'attendanceStatusName', key: 'attendanceStatusName', width: COL_WIDTH_M },
      { title: '扣费状态', dataIndex: 'chargeStatusName', key: 'chargeStatusName', width: COL_WIDTH_M },
      {
        title: '操作',
        key: 'actionPlus',
        width: COL_WIDTH_S,
        render: (text, record, index) => (
          <LinkBtnWithPop
            text="退班"
            title="确定将该学生退班吗？"
            confirm={() => {
              this.quitLecture(record);
            }}
            disabled={record.chargeStatus === 'CHARGED'}
          />
        ),
      },
    ];
    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.initDetail = this.initDetail.bind(this);
    this.loadList = this.loadList.bind(this);
    this.changePage = this.changePage.bind(this);
    this.search = this.search.bind(this);
    this.updateRowSelected = this.updateRowSelected.bind(this);
    this.quitLecture = this.quitLecture.bind(this);
    this.openAuditConfirmModal = this.openAuditConfirmModal.bind(this);
    this.openChargeConfirmModal = this.openChargeConfirmModal.bind(this);
    this.auditLectureStd = this.auditLectureStd.bind(this);
    this.chargeLectureStd = this.chargeLectureStd.bind(this);
    this.destroy = this.destroy.bind(this);
  }
  static classPrefix = 'RegStdDetail';
  static openConfirmModal(title, content, confirmFn, cancelFn) {
    confirm({
      title: title,
      content: content,
      okText: '确定',
      cancelText: '取消',
      iconType: '',
      onOk() {
        if (confirmFn) confirmFn();
      },
      onCancel() {
        if (cancelFn) cancelFn();
      },
    });
  }
  componentDidMount() {
    this.init();
  }
  componentWillUnmount() {
    this.destroy();
  }

  async init() {
    this.setState({ loading: true });
    await this.initDict();
    await this.initDetail();
    this.setState({ loading: false });
  }
  initDict() {
    const { dispatch } = this.props;
    return new Promise((resolve, reject) => {
      dispatch({
        type: 'dict/initDict',
        payload: {
          list: ['branch', 'branchCampus', 'lectureStdChargeStatus', 'baseStatus'],
          callback() {
            resolve();
          },
          handleErr() {
            reject();
          },
        },
      });
    });
  }
  initDetail() {
    const { dispatch, id } = this.props;
    let page = this.state.pageCurrent;
    let rows = this.state.pageSize;
    return new Promise((resolve, reject) => {
      dispatch({
        type: `${storeName}/initRegStdDetail`,
        payload: {
          id,
          page,
          rows,
          callback() {
            resolve();
          },
          handleErr() {
            reject();
          },
        },
      });
    });
  }

  loadList(current, size) {
    this.setState({ listLoading: true, selectedRowKeys: [] });
    const _self = this;
    const { dispatch, id } = this.props;
    dispatch({
      type: `${storeName}/loadLectureStdList`,
      payload: {
        id,
        page: current,
        rows: size,
        callback() {
          _self.setState({ listLoading: false });
        },
      },
    });
  }
  changePage({ current, pageSize }) {
    this.setState({
      pageCurrent: current,
      pageSize,
    });
    this.loadList(current, pageSize);
  }
  search() {
    this.loadList(1, this.state.pageSize);
  }
  // 选择学生
  updateRowSelected(keys) {
    this.setState({ selectedRowKeys: keys });
  }
  // 退讲座(退班)
  quitLecture(record) {
    this.setState({ loading: true });
    const _self = this;
    let current = this.state.pageCurrent;
    let size = this.state.pageSize;
    const { dispatch, id } = this.props;
    dispatch({
      type: `${storeName}/quitLecture`,
      payload: {
        lectureId: id,
        studentId: record.studentId,
        callback() {
          _self.setState({ loading: false });
          _self.loadList(current, size);
        },
      },
    });
  }
  // 打开确认考勤modal
  openAuditConfirmModal() {
    if (!this.props.fileList.length) {
      showMessage('请先上传并保存考勤照片', 'error');
      return;
    }
    let len = this.state.selectedRowKeys.length;
    let content = `一共选中${len}个学生，将进行考勤`;
    RegStdDetail.openConfirmModal('确认考勤', content, this.auditLectureStd);
  }
  // 打开确认扣费modal
  openChargeConfirmModal() {
    if (!this.props.fileList.length) {
      showMessage('请先上传并保存考勤照片', 'error');
      return;
    }
    let len = this.state.selectedRowKeys.length;
    let content = `一共选中${len}个学生，将进行扣费`;
    RegStdDetail.openConfirmModal('确认扣费', content, this.chargeLectureStd);
  }
  // 学生批量考勤
  auditLectureStd() {
    let selectKeys = this.state.selectedRowKeys;
    const _self = this;
    let current = this.state.pageCurrent;
    let size = this.state.pageSize;
    this.props.dispatch({
      type: `${storeName}/auditLectureStd`,
      payload: {
        selectKeys,
        callback() {
          _self.loadList(current, size);
        },
      },
    });
  }
  // 学生批量扣费
  chargeLectureStd() {
    let selectKeys = this.state.selectedRowKeys;
    const _self = this;
    let current = this.state.pageCurrent;
    let size = this.state.pageSize;
    this.props.dispatch({
      type: `${storeName}/chargeLectureStd`,
      payload: {
        selectKeys,
        callback() {
          _self.loadList(current, size);
        },
      },
    });
  }

  destroy() {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        lecture_stdList: [],
        lecture_stdTotal: 0,
      },
    });
  }

  render() {
    const { list, total, btnAuth_audit, btnAuth_charge } = this.props;
    const { loading, listLoading, pageCurrent, pageSize, selectedRowKeys } = this.state;
    const prefix = RegStdDetail.classPrefix;
    const checkSelectedStd = selectedRowKeys.length === 0;

    return (
      <Wrapper
        className={style[prefix]}
        loading={loading}
        bodyFitHeight
        header={
          <BtnWrap
            node={<Search className={style[`${prefix}-search`]} handleSearch={this.search} />}
            nodeMarginRight={10}
          >
            <Button type="primary" onClick={this.openAuditConfirmModal} disabled={btnAuth_audit || checkSelectedStd}>
              考勤
            </Button>
            <Button type="primary" onClick={this.openChargeConfirmModal} disabled={btnAuth_charge || checkSelectedStd}>
              扣费
            </Button>
          </BtnWrap>
        }
      >
        <List
          dataSource={list}
          columns={this.columns}
          loading={!loading && listLoading}
          pagination={{
            current: pageCurrent,
            pageSize,
            total,
          }}
          onChange={this.changePage}
          rowSelection={{
            selectedRowKeys,
            onChange: this.updateRowSelected,
          }}
        />
      </Wrapper>
    );
  }
}

const mapStateToProps = (state) => {
  const { lecture_stdList, lecture_stdTotal } = state[storeName];
  const btnAuth = state.account.btnAuths;
  let btnAuth_audit = btnAuth.indexOf('AUDIT_ONLINE_LECTURE') === -1;
  let btnAuth_charge = btnAuth.indexOf('CHARGE_ONLINE_LECTURE') === -1;
  return {
    list: lecture_stdList,
    total: lecture_stdTotal,
    btnAuth_audit,
    btnAuth_charge,
  };
};

export default connect(mapStateToProps)(RegStdDetail);
