/**
 * Created by Haolin on 2019/1/4.
 */
import React from 'react';
import { connect } from 'dva';
import { storeName } from '../store/constant.js';
import Modal from 'COMPONENT/Modal';
import Button from 'COMPONENT/Button';
import FormItem from 'COMPONENT/FormItem';
import Input from 'COMPONENT/Input';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import DatePicker from 'COMPONENT/DatePicker';
import FlexRow from '../components/FlexRow';
import ListWithDropDownMenu from '../components/ListWithDropDownMenu';
import ModalSelectProduct from '../ModalSelectProduct';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import style from './style.less';

class ModalEditLecture extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      name: '修改讲座',
      loading: false,
      toggle_selectPrd: false,
      error: {},
      checkValidator: true,
    };
    this.lectureActionType = 'edit';
    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.initDetail = this.initDetail.bind(this);
    this.switchSelectPrd = this.switchSelectPrd.bind(this);
    this.confirmSelectPrd = this.confirmSelectPrd.bind(this);
    this.updateState = this.updateState.bind(this);
    this.errorChange = this.errorChange.bind(this);
    this.checkBeforeSubmit = this.checkBeforeSubmit.bind(this);
    this.submit = this.submit.bind(this);
  }
  static classPrefix = 'ModalEditLecture';
  componentDidMount() {
    this.init();
  }
  componentWillUnmount() {
    this.destroy();
  }

  async init() {
    this.setState({ loading: true });
    await this.initDict();
    await this.initDetail();
    this.setState({ loading: false });
  }
  initDict() {
    const { dispatch } = this.props;
    return new Promise((resolve, reject) => {
      dispatch({
        type: 'dict/initDict',
        payload: {
          list: ['grade', 'subject', 'allTwoTcMainTc'],
          callback() {
            resolve();
          },
        },
      });
    });
  }
  initDetail() {
    const { dispatch, lectureId, stdNum } = this.props;
    return new Promise((resolve, reject) => {
      dispatch({
        type: `${storeName}/getLectureDetail`,
        payload: {
          id: lectureId,
          stdNum,
          type: 'edit',
          callback() {
            resolve();
          },
        },
      });
    });
  }

  switchSelectPrd() {
    this.setState((prevState) => ({
      toggle_selectPrd: !prevState.toggle_selectPrd,
    }));
  }
  confirmSelectPrd() {
    let type = this.lectureActionType;
    const _self = this;
    this.props.dispatch({
      type: `${storeName}/confirmProduct`,
      payload: {
        type,
        callback() {
          _self.switchSelectPrd();
        },
      },
    });
  }

  updateState(type, val) {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: { [type]: val },
    });
  }
  // 用于校验表单是否存在错误
  errorChange(type, val) {
    this.setState((prevState) => {
      let error = prevState.error;
      error[type] = val;
      return { error };
    });
  }
  // 校验输入
  checkBeforeSubmit() {
    let result = Object.values(this.state.error).every((item) => !Boolean(item));
    this.setState({ checkValidator: result });
    return result;
  }
  // 提交
  submit() {
    // 校验讲座信息
    const { checkSelectPrd, dispatch, lectureId, handleConfirm } = this.props;
    if (!checkSelectPrd) {
      this.modalRef.showOkTips('请选择讲座产品！');
      return;
    }
    let check = this.checkBeforeSubmit();
    if (!check) {
      this.modalRef.showOkTips('您有信息未填写，请补充完整！');
      return;
    }
    this.setState({ loading: true });
    dispatch({
      type: `${storeName}/submitEditLecture`,
      payload: {
        id: lectureId,
        callback() {
          handleConfirm();
        },
      },
    });
  }

  destroy() {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        editLecture_canEditPrd: false,
        editLecture_productInfo: {},
        editLecture_productList: [],
        editLecture_name: '',
        editLecture_grade: [],
        editLecture_subject: [],
        editLecture_teacher: null,
        editLecture_beginDate: null,
      },
    });
  }

  render() {
    const {
      onCancel,
      gradeDict,
      subjectDict,
      mainTcDict,
      checkSelectPrd,
      productNameList,
      editLecture_name,
      editLecture_grade,
      editLecture_subject,
      editLecture_canEditPrd,
      editLecture_teacher,
      editLecture_beginDate,
    } = this.props;
    const { name, loading, toggle_selectPrd, error, checkValidator } = this.state;

    return (
      <Modal
        className={style[ModalEditLecture.classPrefix]}
        title={name}
        width={560}
        loading={loading}
        onCancel={onCancel}
        onOk={this.submit}
        ref={(ref) => (this.modalRef = getWrappedInstance(ref))}
      >
        <div>
          <FormItem name="产品名称" labelWidth={100} required>
            <FlexRow
              fitWidth={70}
              fitNode={
                <Button type="primary" size="small" onClick={this.switchSelectPrd} disabled={!editLecture_canEditPrd}>
                  选择产品
                </Button>
              }
            >
              {checkSelectPrd && <ListWithDropDownMenu data={productNameList} />}
            </FlexRow>
          </FormItem>
          <FormItem name="讲座名称" labelWidth={100} required>
            <Input
              placeholder="讲座名称"
              value={editLecture_name}
              onChange={(ele) => {
                this.updateState('editLecture_name', ele.target.value);
              }}
              maxLength={30}
              validator={{
                rules: ['required'],
                silent: !Boolean(error['name']) || checkValidator,
                showStatus: true,
                showTips: false,
                onErrorChange: (err) => {
                  this.errorChange('name', err);
                },
              }}
            />
          </FormItem>
          <FormItem name="年级" labelWidth={100} required>
            <MultiSelect
              placeholder="年级"
              dict={gradeDict}
              value={editLecture_grade}
              onChange={(val) => {
                this.updateState('editLecture_grade', val);
              }}
              showSearch={true}
              validator={{
                rules: ['required'],
                silent: !Boolean(error['editLecture_grade']) || checkValidator,
                showStatus: true,
                showTips: false,
                onErrorChange: (err) => {
                  this.errorChange('editLecture_grade', err);
                },
              }}
            />
          </FormItem>
          <FormItem name="科目" labelWidth={100} required>
            <MultiSelect
              placeholder="科目"
              dict={subjectDict}
              value={editLecture_subject}
              onChange={(val) => {
                this.updateState('editLecture_subject', val);
              }}
              showSearch={true}
              validator={{
                rules: ['required'],
                silent: !Boolean(error['editLecture_subject']) || checkValidator,
                showStatus: true,
                showTips: false,
                onErrorChange: (err) => {
                  this.errorChange('editLecture_subject', err);
                },
              }}
            />
          </FormItem>
          <FormItem name="主讲老师" labelWidth={100} required>
            <Select
              placeholder="主讲老师"
              dict={mainTcDict}
              value={editLecture_teacher}
              onChange={(val) => {
                this.updateState('editLecture_teacher', val);
              }}
              showSearch={true}
              validator={{
                rules: ['required'],
                silent: !Boolean(error['editLecture_teacher']) || checkValidator,
                showStatus: true,
                showTips: false,
                onErrorChange: (err) => {
                  this.errorChange('editLecture_teacher', err);
                },
              }}
            />
          </FormItem>
          <FormItem name="上课日期" labelWidth={100} required>
            <DatePicker
              placeholder="上课日期"
              value={editLecture_beginDate}
              onChange={(val) => {
                this.updateState('editLecture_beginDate', val);
              }}
              validator={{
                rules: ['required'],
                silent: !Boolean(error['editLecture_beginDate']) || checkValidator,
                showStatus: true,
                showTips: false,
                onErrorChange: (err) => {
                  this.errorChange('editLecture_beginDate', err);
                },
              }}
            />
          </FormItem>
        </div>
        {toggle_selectPrd && (
          <ModalSelectProduct
            type={this.lectureActionType}
            needInit={checkSelectPrd}
            onCancel={this.switchSelectPrd}
            onConfirm={this.confirmSelectPrd}
          />
        )}
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  const { grade, subject, allTwoTcMainTc } = state.dict;
  const {
    editLecture_productList,
    editLecture_name,
    editLecture_grade,
    editLecture_canEditPrd,
    editLecture_subject,
    editLecture_teacher,
    editLecture_beginDate,
  } = state[storeName];
  let checkSelectPrd = editLecture_productList.length > 0;
  let productNameList = checkSelectPrd ? editLecture_productList.map((item) => item.name) : [];
  return {
    gradeDict: grade,
    subjectDict: subject,
    mainTcDict: allTwoTcMainTc,
    checkSelectPrd,
    productNameList,
    editLecture_name,
    editLecture_grade,
    editLecture_subject,
    editLecture_teacher,
    editLecture_beginDate,
    editLecture_canEditPrd,
  };
};

export default connect(mapStateToProps)(ModalEditLecture);
