/**
 * Created by Haolin on 2019/1/5.
 */
import React from 'react';
import { connect } from 'dva';
import { Row, Col } from 'antd';
import { storeName } from '../../store/constant.js';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import Button from 'COMPONENT/Button';
import style from './style.less';

const SearchBarPlus = (props) => {
  const { children, onSearch } = props;
  const prefix = 'SearchBarPlus';
  return (
    <Row type="flex" className={style[prefix]}>
      {children.length > 0 &&
        children.map((item, index) => (
          <Col className={style[`${prefix}-col`]} span={6} key={index}>
            {item}
          </Col>
        ))}
      <Col className={style[`${prefix}-col`]} span={6}>
        <Button type="primary" onClick={onSearch}>
          查询
        </Button>
      </Col>
    </Row>
  );
};

class Search extends React.Component {
  constructor(props) {
    super(props);
    this.search = this.search.bind(this);
    this.reset = this.reset.bind(this);
    this.updateKeyword = this.updateKeyword.bind(this);
    this.destroy = this.destroy.bind(this);
  }
  static classPrefix = 'Search';
  componentDidMount() {}
  componentWillUnmount() {}

  /**
   * 检索
   * */
  search() {
    this.props.handleSearch();
  }
  /**
   * 重置检索条件
   * */
  reset() {
    this.props.dispatch({
      type: `${storeName}/resetRegKeyword`,
    });
  }
  /**
   * 更新检索条件
   * */
  updateKeyword({ type, val }) {
    let payload = { [type]: val };
    let branchCampusDict = this.props.branchCampusDict.filter((item) => item.val === val);
    let campusDict =
      branchCampusDict && branchCampusDict[0] && branchCampusDict[0].options ? branchCampusDict[0].options : [];
    if (type === 'sKey_reg_branch') {
      payload.sKey_reg_campus = campusDict[0] ? campusDict[0].val : null;
    }
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload,
    });
  }
  /**
   * 注销前销毁
   * */
  destroy() {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        sKey_reg_branch: null,
        sKey_reg_campus: null,
        sKey_reg_grade: [],
      },
    });
  }

  render() {
    const { className, branchDict, branchCampusDict, gradeDict, sKey_reg_branch, sKey_reg_campus, sKey_reg_grade } =
      this.props;

    return (
      <div className={className}>
        <SearchBarPlus onSearch={this.search} onReset={this.reset}>
          <Select
            placeholder="分公司"
            dict={branchDict}
            value={sKey_reg_branch}
            onChange={(val) => {
              this.updateKeyword({ type: 'sKey_reg_branch', val });
            }}
            showSearch={true}
            allowClear={false}
          />
          <Select
            placeholder="学习中心"
            dict={
              sKey_reg_branch
                ? branchCampusDict && branchCampusDict.filter((item) => item.val === sKey_reg_branch)
                : branchCampusDict
            }
            value={sKey_reg_campus}
            onChange={(val) => {
              this.updateKeyword({ type: 'sKey_reg_campus', val });
            }}
            showSearch={true}
            allowClear={false}
          />
          <MultiSelect
            placeholder="年级"
            dict={gradeDict}
            value={sKey_reg_grade}
            onChange={(val) => {
              this.updateKeyword({ type: 'sKey_reg_grade', val });
            }}
            showSearch={true}
          />
        </SearchBarPlus>
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  const { branch, branchCampus, grade } = state.dict;
  const { sKey_reg_branch, sKey_reg_campus, sKey_reg_grade } = state[storeName];
  return {
    branchDict: branch,
    branchCampusDict: branchCampus,
    gradeDict: grade,
    sKey_reg_branch,
    sKey_reg_campus,
    sKey_reg_grade,
  };
};

export default connect(mapStateToProps)(Search);
