/**
 * Created by Haolin on 2019/1/5.
 */
import React from 'react';
import { connect } from 'dva';
import { storeName } from '../store/constant.js';
import Modal from 'COMPONENT/Modal';
import List from 'COMPONENT/List';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import Search from './Search';
// import style from './style.less';

const { COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;

class ModalReg extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      name: '报名',
      loading: false,
      listLoading: false,
      pageCurrent: 1,
      pageSize: 50,
      selectedRowKeys: [],
    };
    this.columns = [
      { title: '学生姓名', dataIndex: 'studentName', key: 'studentName', width: COL_WIDTH_M },
      { title: '年级', dataIndex: 'gradeName', key: 'gradeName', width: COL_WIDTH_M },
      { title: '产品名称', dataIndex: 'productName', key: 'productName', width: COL_WIDTH_XL },
      { title: '课时单价', dataIndex: 'price', key: 'price', width: COL_WIDTH_M },
    ];
    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.initModal = this.initModal.bind(this);
    this.loadList = this.loadList.bind(this);
    this.changePage = this.changePage.bind(this);
    this.search = this.search.bind(this);
    this.updateRowSelected = this.updateRowSelected.bind(this);
    this.submit = this.submit.bind(this);
    this.destroy = this.destroy.bind(this);
  }
  static classPrefix = 'ModalReg';
  componentDidMount() {
    this.init();
  }
  componentWillUnmount() {
    this.destroy();
  }

  async init() {
    let lectureName = this.props.lectureDate.name;
    this.setState((prevState) => ({
      loading: true,
      name: `${prevState.name} (讲座：${lectureName})`,
    }));
    await this.initDict();
    await this.initModal();
    this.setState({ loading: false });
  }
  initDict() {
    const { dispatch } = this.props;
    return new Promise((resolve, reject) => {
      dispatch({
        type: 'dict/initDict',
        payload: {
          list: ['branch', 'branchCampus', 'grade'],
          callback() {
            resolve();
          },
        },
      });
    });
  }
  initModal() {
    const { dispatch, lectureDate } = this.props;
    let page = this.state.pageCurrent;
    let rows = this.state.pageSize;
    return new Promise((resolve, reject) => {
      dispatch({
        type: `${storeName}/initModalReg`,
        payload: {
          id: lectureDate.id,
          page,
          rows,
          callback() {
            resolve();
          },
        },
      });
    });
  }
  // 加载列表
  loadList(current, size) {
    const { dispatch, lectureDate } = this.props;
    this.setState({ listLoading: true, selectedRowKeys: [] });
    const _self = this;
    dispatch({
      type: `${storeName}/loadStdList`,
      payload: {
        id: lectureDate.id,
        page: current,
        rows: size,
        callback() {
          _self.setState({ listLoading: false });
        },
      },
    });
  }
  // 翻页
  changePage({ current, pageSize }) {
    this.setState({
      pageCurrent: current,
      pageSize,
    });
    this.loadList(current, pageSize);
  }
  // 搜索
  search() {
    this.loadList(1, this.state.pageSize);
  }
  //
  updateRowSelected(keys) {
    this.setState({ selectedRowKeys: keys });
  }

  submit() {
    const { dispatch, lectureDate, handleConfirm } = this.props;
    dispatch({
      type: `${storeName}/stdRegLecture`,
      payload: {
        id: lectureDate.id,
        selectedKeys: this.state.selectedRowKeys,
        callback() {
          handleConfirm();
        },
      },
    });
  }

  destroy() {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        regStdList: [],
        regStdTotal: 0,
      },
    });
  }

  render() {
    const { onCancel, list, total } = this.props;
    const { name, loading, listLoading, pageCurrent, pageSize, selectedRowKeys } = this.state;
    const prefix = ModalReg.classPrefix;

    return (
      <Modal
        className={prefix}
        title={name}
        width={720}
        loading={loading}
        onCancel={onCancel}
        onOk={this.submit}
        okBtnDisabled={selectedRowKeys.length === 0}
        ref={(ref) => (this.modalRef = getWrappedInstance(ref))}
      >
        <Search className={`${prefix}-search`} handleSearch={this.search} />
        <List
          dataSource={list}
          columns={this.columns}
          loading={!loading && listLoading}
          pagination={{
            current: pageCurrent,
            pageSize,
            total,
          }}
          onChange={this.changePage}
          rowSelection={{
            selectedRowKeys,
            onChange: this.updateRowSelected,
          }}
        />
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  const { regStdList, regStdTotal } = state[storeName];
  return {
    list: regStdList,
    total: regStdTotal,
  };
};

export default connect(mapStateToProps)(ModalReg);
