/**
 * Created by Haolin on 2018/12/29.
 */
import React from 'react';
import { connect } from 'dva';
import { Row, Col } from 'antd';
import { storeName, LECTURE_VAL } from '../../store/constant.js';
import Select from 'COMPONENT/Select';
import FormItem from 'COMPONENT/FormItem';
import style from './style.less';

class SelectProduct extends React.Component {
  constructor(props) {
    super(props);
    this.updateSelectVal = this.updateSelectVal.bind(this);
    this.updateSelectDict = this.updateSelectDict.bind(this);
    this.selectYear = this.selectYear.bind(this);
    this.selectClassType = this.selectClassType.bind(this);
    this.selectPrice = this.selectPrice.bind(this);
    this.selectTotalHour = this.selectTotalHour.bind(this);
    this.selectBranch = this.selectBranch.bind(this);
    this.selectPrd = this.selectPrd.bind(this);
    this.destroy = this.destroy.bind(this);
  }
  static classPrefix = 'SelectProduct';
  componentDidMount() {}
  componentWillUnmount() {
    this.destroy();
  }

  updateSelectVal(payload) {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload,
    });
  }
  updateSelectDict(type) {
    const { handleLoading, dispatch } = this.props;
    handleLoading(true);
    dispatch({
      type: `${storeName}/loadSelectPrdDict`,
      payload: {
        type,
        callback() {
          handleLoading(false);
        },
      },
    });
  }
  selectYear(val) {
    this.updateSelectVal({
      sKey_selectPrd_year: val,
      sKey_selectPrd_price: null,
      sKey_selectPrd_totalHour: null,
    });
    this.updateSelectDict(1);
  }
  selectClassType(val) {
    this.updateSelectVal({
      sKey_selectPrd_classType: val,
      sKey_selectPrd_price: null,
      sKey_selectPrd_totalHour: null,
    });
    this.updateSelectDict(1);
  }
  selectPrice(val) {
    this.updateSelectVal({
      sKey_selectPrd_price: val,
      sKey_selectPrd_totalHour: null,
    });
    this.updateSelectDict(2);
  }
  selectTotalHour(val) {
    this.updateSelectVal({ sKey_selectPrd_totalHour: val });
  }
  selectBranch(val) {
    this.updateSelectVal({
      sKey_selectPrd_branch: val,
      sKey_selectPrd_Product: null,
    });
    if (val) {
      this.updateSelectDict(3);
      this.props.dispatch({
        type: `${storeName}/initProductVal`,
        payload: { branchId: val },
      });
    }
  }
  selectPrd(val) {
    this.updateSelectVal({ sKey_selectPrd_Product: val });
    this.props.dispatch({
      type: `${storeName}/selectProduct`,
      payload: { val },
    });
  }

  destroy() {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        sKey_selectPrd_year: null,
        sKey_selectPrd_classType: null,
        sKey_selectPrd_price: null,
        sKey_selectPrd_totalHour: null,
        sKey_selectPrd_branch: null,
        sKey_selectPrd_Product: null,
        dict_price: {},
        dict_totalHour: {},
        dict_product: {},
      },
    });
  }

  render() {
    const {
      yearDict,
      classTypeDict,
      branchDict,
      sKey_selectPrd_year,
      sKey_selectPrd_classType,
      sKey_selectPrd_price,
      sKey_selectPrd_totalHour,
      sKey_selectPrd_branch,
      sKey_selectPrd_Product,
      priceDict,
      totalHourDict,
      productDict,
      productList,
    } = this.props;
    const prefix = SelectProduct.classPrefix;
    const checkSelected = productList.length > 0;

    return (
      <div className={style[prefix]}>
        <Row>
          <Col span={6}>
            <Select
              placeholder="年份"
              dict={yearDict}
              value={sKey_selectPrd_year}
              allowClear={false}
              onChange={this.selectYear}
              disabled={checkSelected}
            />
          </Col>
          <Col span={6}>
            <Select
              placeholder="班级类型"
              dict={classTypeDict}
              value={sKey_selectPrd_classType}
              allowClear={false}
              onChange={this.selectClassType}
              disabled={checkSelected}
            />
          </Col>
          <Col span={6}>
            <Select
              placeholder="课时单价"
              dict={priceDict}
              value={sKey_selectPrd_price}
              onChange={this.selectPrice}
              disabled={checkSelected}
            />
          </Col>
          <Col span={6}>
            <Select
              placeholder="课时时长"
              dict={totalHourDict}
              value={sKey_selectPrd_totalHour}
              onChange={this.selectTotalHour}
              disabled={checkSelected || !sKey_selectPrd_price}
            />
          </Col>
        </Row>
        <Row>
          <Col span={3}>
            <div className="label-required">选择产品</div>
          </Col>
          <Col span={6}>
            <Select
              placeholder="分公司"
              dict={branchDict}
              value={sKey_selectPrd_branch}
              onChange={this.selectBranch}
              disabled={!(sKey_selectPrd_price && sKey_selectPrd_totalHour)}
              showSearch
            />
          </Col>
          <Col span={15}>
            <Select
              placeholder="讲座产品"
              dict={productDict}
              value={sKey_selectPrd_Product}
              onChange={this.selectPrd}
              disabled={!sKey_selectPrd_branch}
            />
          </Col>
        </Row>
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  const { year, classType, branch } = state.dict;
  const {
    sKey_selectPrd_year,
    sKey_selectPrd_classType,
    sKey_selectPrd_price,
    sKey_selectPrd_totalHour,
    sKey_selectPrd_branch,
    sKey_selectPrd_Product,
    dict_price,
    dict_totalHour,
    dict_product,
    addLecture_prdList,
  } = state[storeName];
  let classTypeDict = classType[LECTURE_VAL] ? classType[LECTURE_VAL] : null;
  let productDict =
    dict_product[
      `${sKey_selectPrd_year}${sKey_selectPrd_classType}${sKey_selectPrd_price}${sKey_selectPrd_totalHour}${sKey_selectPrd_branch}`
    ];
  let priceDict = dict_price[`${sKey_selectPrd_year}${sKey_selectPrd_classType}`];
  let totalHourDict = dict_totalHour[`${sKey_selectPrd_year}${sKey_selectPrd_classType}${sKey_selectPrd_price}`];

  return {
    yearDict: year,
    classTypeDict,
    branchDict: branch,
    sKey_selectPrd_year,
    sKey_selectPrd_classType,
    sKey_selectPrd_price,
    sKey_selectPrd_totalHour,
    sKey_selectPrd_branch,
    sKey_selectPrd_Product,
    priceDict: priceDict || null,
    totalHourDict: totalHourDict || null,
    productDict: productDict || null,
    productList: addLecture_prdList,
  };
};

export default connect(mapStateToProps)(SelectProduct);
