/**
 * Created by Haolin on 2018/12/29.
 */
import React from 'react';
import { connect } from 'dva';
import Modal from 'COMPONENT/Modal';
import SelectProduct from './SelectProduct';
import AllSelectedProduct from './AllSelectedProduct';
import { storeName } from '../store/constant.js';
import style from './style.less';

class ModalSelectProduct extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      name: '选择产品',
      loading: false,
    };
    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.initEdit = this.initEdit.bind(this);
    this.initNormal = this.initNormal.bind(this);
    this.modalLoading = this.modalLoading.bind(this);
    this.destroy = this.destroy.bind(this);
  }
  static classPrefix = 'ModalSelectProduct';
  componentDidMount() {
    this.init();
  }
  componentWillUnmount() {
    this.destroy();
  }

  async init() {
    this.setState({ loading: true });
    await this.initDict();
    if (this.props.needInit) {
      await this.initEdit();
    } else {
      await this.initNormal();
    }
    this.setState({ loading: false });
  }
  initDict() {
    const { dispatch } = this.props;
    return new Promise((resolve, reject) => {
      dispatch({
        type: 'dict/initDict',
        payload: {
          list: ['year', { name: 'classType', params: 'LECTURE' }, 'branch'],
          callback() {
            resolve();
          },
        },
      });
    });
  }
  initEdit() {
    const { dispatch, type } = this.props;
    return new Promise((resolve, reject) => {
      dispatch({
        type: `${storeName}/initModalSelectPrdEdit`,
        payload: {
          type,
          callback() {
            resolve();
          },
        },
      });
    });
  }
  initNormal() {
    const { dispatch } = this.props;
    return new Promise((resolve, reject) => {
      dispatch({
        type: `${storeName}/initModalSelectPrd`,
        payload: {
          callback() {
            resolve();
          },
        },
      });
    });
  }
  modalLoading(params) {
    this.setState({ loading: params });
  }

  destroy() {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        addLecture_prdList: [],
        addLecture_prdList_cache: {},
      },
    });
  }

  render() {
    const { onCancel, onConfirm, checkSelectedPrd } = this.props;
    const { name, loading } = this.state;

    return (
      <Modal
        className={style[ModalSelectProduct.classPrefix]}
        title={name}
        width={900}
        loading={loading}
        onCancel={onCancel}
        onOk={onConfirm}
        okBtnDisabled={!checkSelectedPrd}
      >
        <SelectProduct handleLoading={this.modalLoading} />
        <AllSelectedProduct />
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  const { addLecture_prdList } = state[storeName];
  let checkSelectedPrd = addLecture_prdList.length > 0;
  return {
    checkSelectedPrd,
  };
};

export default connect(mapStateToProps)(ModalSelectProduct);
