/**
 * Created by Haolin on 2018/12/26.
 */
import React, { PropTypes } from 'react';
import { connect } from 'dva';
import SearchBar from 'COMPONENT/SearchBar';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import Input from 'COMPONENT/Input';
import DatePicker from 'COMPONENT/DatePicker';
import { storeName } from '../store/constant.js';

class Search extends React.Component {
  constructor(props) {
    super(props);
    this.search = this.search.bind(this);
    this.reset = this.reset.bind(this);
    this.updateKeyword = this.updateKeyword.bind(this);
    this.destroy = this.destroy.bind(this);
  }
  static propTypes = {
    classPrefix: PropTypes.string,
    handleSearch: PropTypes.func,
  };
  componentWillUnmount() {
    this.destroy();
  }
  /**
   * 检索
   * */
  search() {
    this.props.handleSearch();
  }
  /**
   * 重置检索条件
   * */
  reset() {
    this.props.dispatch({
      type: `${storeName}/resetMainKeyword`,
    });
  }
  /**
   * 更新检索条件
   * */
  updateKeyword(params) {
    const { type, val } = params;
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: { [type]: val },
    });
  }
  /**
   * 注销前销毁
   * */
  destroy() {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        sKey_main_name: '',
        sKey_main_branch: [],
        sKey_main_year: null,
        sKey_main_grade: [],
        sKey_main_subject: [],
        sKey_main_classType: null,
        sKey_main_status: null,
        sKey_main_date: null,
      },
    });
  }

  render() {
    const {
      className,
      branchDict,
      yearDict,
      gradeDict,
      subjectDict,
      classTypeDict,
      mainTcDict,
      statusDict,
      sKey_main_name,
      sKey_main_branch,
      sKey_main_year,
      sKey_main_grade,
      sKey_main_mainTc,
      sKey_main_subject,
      sKey_main_classType,
      sKey_main_status,
      sKey_main_date,
    } = this.props;

    return (
      <div className={className}>
        <SearchBar onSearch={this.search} onReset={this.reset}>
          <Input
            placeholder="讲座名称"
            value={sKey_main_name}
            onChange={(e) => {
              this.updateKeyword({ type: 'sKey_main_name', val: e.target.value });
            }}
          />
          <MultiSelect
            placeholder="分公司"
            dict={branchDict}
            value={sKey_main_branch}
            showSearch={true}
            onChange={(val) => {
              this.updateKeyword({ type: 'sKey_main_branch', val });
            }}
          />
          <Select
            placeholder="年份"
            dict={yearDict}
            value={sKey_main_year}
            allowClear={false}
            onChange={(val) => {
              this.updateKeyword({ type: 'sKey_main_year', val });
            }}
          />
          <MultiSelect
            placeholder="年级"
            dict={gradeDict}
            value={sKey_main_grade}
            showSearch={true}
            onChange={(val) => {
              this.updateKeyword({ type: 'sKey_main_grade', val });
            }}
          />
          <MultiSelect
            placeholder="科目"
            dict={subjectDict}
            value={sKey_main_subject}
            showSearch={true}
            onChange={(val) => {
              this.updateKeyword({ type: 'sKey_main_subject', val });
            }}
          />
          <DatePicker
            placeholder="上课日期"
            value={sKey_main_date}
            onChange={(val) => {
              this.updateKeyword({ type: 'sKey_main_date', val });
            }}
          />
          <Select
            placeholder="讲座状态"
            dict={statusDict}
            value={sKey_main_status}
            onChange={(val) => {
              this.updateKeyword({ type: 'sKey_main_status', val });
            }}
          />
          <Select
            placeholder="班级类型"
            dict={classTypeDict}
            value={sKey_main_classType}
            allowClear={false}
            onChange={(val) => {
              this.updateKeyword({ type: 'sKey_main_classType', val });
            }}
          />
          <Select
            placeholder="主讲老师"
            dict={mainTcDict}
            value={sKey_main_mainTc}
            showSearch={true}
            onChange={(val) => {
              this.updateKeyword({ type: 'sKey_main_mainTc', val });
            }}
          />
        </SearchBar>
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  const { branch, year, grade, subject, classType, lectureStatus, allTwoTcMainTc } = state.dict;
  const {
    sKey_main_name,
    sKey_main_branch,
    sKey_main_year,
    sKey_main_grade,
    sKey_main_subject,
    sKey_main_classType,
    sKey_main_status,
    sKey_main_date,
    sKey_main_mainTc,
  } = state[storeName];
  return {
    branchDict: branch,
    yearDict: year,
    gradeDict: grade,
    subjectDict: subject,
    classTypeDict: classType ? classType['LECTURE'] : null,
    statusDict: lectureStatus,
    mainTcDict: allTwoTcMainTc,
    sKey_main_name,
    sKey_main_branch,
    sKey_main_year,
    sKey_main_grade,
    sKey_main_subject,
    sKey_main_classType,
    sKey_main_status,
    sKey_main_date,
    sKey_main_mainTc,
  };
};

export default connect(mapStateToProps)(Search);
