/**
 * Created by Haolin on 2018/12/26.
 */
import React from 'react';
import { connect } from 'dva';
import { Modal } from 'antd';
import { storeName } from './store/constant.js';
import style from './style.less';
import Page from 'COMPONENT/Page';
import Button from 'COMPONENT/Button';
import List from 'COMPONENT/List';
import Search from './Search';
import ListWithPop from './components/ListWithPop';
import ModalEditLecture from './ModalEditLecture';
import ModalReg from './ModalReg';

const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;
const confirm = Modal.confirm;

class TwoTeacherLecture extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      name: '线上讲座',
      loading: false,
      listLoading: false,
      pageCurrent: 1,
      pageSize: 20,
      toggleModalEditLecture: false,
      toggleModalReg: false,
    };
    this.columns = [
      { title: '序号', dataIndex: 'number', key: 'number', width: COL_WIDTH_S },
      { title: '讲座名称', dataIndex: 'lectureName', key: 'lectureName', width: COL_WIDTH_XL },
      {
        title: '分公司',
        dataIndex: 'branchName',
        key: 'branchName',
        width: COL_WIDTH_M,
        render: (text, record, index) => <ListWithPop data={record.branchName} />,
      },
      { title: '年份', dataIndex: 'yearName', key: 'yearName', width: COL_WIDTH_M },
      {
        title: '年级',
        dataIndex: 'gradeName',
        key: 'gradeName',
        width: COL_WIDTH_M,
        render: (text, record, index) => <ListWithPop data={record.gradeName} />,
      },
      {
        title: '科目',
        dataIndex: 'subjectName',
        key: 'subjectName',
        width: COL_WIDTH_M,
        render: (text, record, index) => <ListWithPop data={record.subjectName} />,
      },
      { title: '班级类型', dataIndex: 'classTypeName', key: 'classTypeName', width: COL_WIDTH_M },
      { title: '主讲老师', dataIndex: 'mainTeacherName', key: 'mainTeacherName', width: COL_WIDTH_M },
      { title: '上课日期', dataIndex: 'beginDate', key: 'beginDate', width: COL_WIDTH_L },
      { title: '报读人数', dataIndex: 'regCount', key: 'regCount', width: COL_WIDTH_M },
      { title: '讲座状态', dataIndex: 'statusName', key: 'statusName', width: COL_WIDTH_M },
      {
        title: '操作',
        key: 'action',
        width: 158,
        fixed: 'right',
        getActionConfig: (text, record, index) => {
          return [
            {
              text: '报名',
              onClick: () => {
                this.moreAction('reg', record, index);
              },
              disabled: props.btnAuth_reg,
            },
            {
              text: '详情',
              onClick: () => {
                this.moreAction('detail', record, index);
              },
              disabled: props.btnAuth_detail,
            },
            {
              menus: [
                {
                  text: '修改',
                  onClick: () => {
                    this.moreAction('edit', record, index);
                  },
                  disabled: props.btnAuth_add,
                },
                {
                  text: '删除',
                  onClick: () => {
                    this.moreAction('delete', record, index);
                  },
                  disabled: props.btnAuth_add,
                },
              ],
            },
          ];
        },
      },
    ];
    this.editLectureId = null;
    this.editLectureStdNum = 0;
    this.regLectureData = null;
    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.initMain = this.initMain.bind(this);
    this.loadList = this.loadList.bind(this);
    this.changePage = this.changePage.bind(this);
    this.search = this.search.bind(this);
    this.goTo = this.goTo.bind(this);
    this.addLecture = this.addLecture.bind(this);
    this.moreAction = this.moreAction.bind(this);
    this.switchModalEditLecture = this.switchModalEditLecture.bind(this);
    this.afterConfirmEdit = this.afterConfirmEdit.bind(this);
    this.switchModalReg = this.switchModalReg.bind(this);
    this.afterConfirmReg = this.afterConfirmReg.bind(this);
    this.openDeleteConfirmModal = this.openDeleteConfirmModal.bind(this);
    this.deleteLecture = this.deleteLecture.bind(this);
    this.destroy = this.destroy.bind(this);
  }
  static classPrefix = 'TwoTeacherLecture';
  static openConfirmModal(title, content, confirmFn, cancelFn) {
    confirm({
      title: title,
      content: content,
      okText: '确定',
      cancelText: '取消',
      iconType: '',
      onOk() {
        if (confirmFn) confirmFn();
      },
      onCancel() {
        if (cancelFn) cancelFn();
      },
    });
  }
  componentDidMount() {
    this.init();
  }
  componentWillUnmount() {
    this.destroy();
  }
  // 初始化
  async init() {
    this.setState({ loading: true });
    await this.initDict();
    await this.initMain();
    this.setState({ loading: false });
  }
  // 初始化页面所需字典数据
  initDict() {
    const { dispatch } = this.props;
    return new Promise((resolve, reject) => {
      dispatch({
        type: 'dict/initDict',
        payload: {
          list: [
            'branch',
            'year',
            'grade',
            'subject',
            'lectureStatus',
            { name: 'classType', params: 'LECTURE' },
            'allTwoTcMainTc',
          ],
          callback() {
            resolve();
          },
        },
      });
    });
  }
  // 初始化筛选栏默认值、加载列表数据
  initMain() {
    const { dispatch } = this.props;
    const { pageCurrent, pageSize } = this.state;
    return new Promise((resolve, reject) => {
      dispatch({
        type: `${storeName}/initMain`,
        payload: {
          page: pageCurrent,
          rows: pageSize,
          callback() {
            resolve();
          },
        },
      });
    });
  }
  // 加载列表
  loadList(current, size) {
    this.setState({ listLoading: true });
    const _self = this;
    this.props.dispatch({
      type: `${storeName}/loadMainList`,
      payload: {
        page: current,
        rows: size,
        callback() {
          _self.setState({ listLoading: false });
        },
      },
    });
  }
  // 翻页
  changePage({ current, pageSize }) {
    this.setState({
      pageCurrent: current,
      pageSize,
    });
    this.loadList(current, pageSize);
  }
  // 搜索
  search() {
    this.loadList(1, this.state.pageSize);
  }
  // 页面跳转
  goTo(path, params = null, refresh = true) {
    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path,
        params,
        refresh,
      },
    });
  }
  // 新增
  addLecture() {
    this.goTo('/twoTeacherLecture/addLecture');
  }
  // 列表操作
  moreAction(type, record, index) {
    switch (type) {
      case 'reg':
        this.switchModalReg({
          id: record.id,
          name: record.lectureName,
        });
        break;
      case 'detail':
        this.goTo('/twoTeacherLecture/detail', { lectureId: record.id });
        break;
      case 'edit':
        this.switchModalEditLecture(record.id, record.regCount);
        break;
      case 'delete':
        this.openDeleteConfirmModal(record.id);
        break;
      default:
        console.warn("moreAction's params [type] is wrong");
        return;
    }
  }
  // 切换修改讲座弹窗显示
  switchModalEditLecture(id, stdNum) {
    this.editLectureId = id || null;
    this.editLectureStdNum = stdNum || 0;
    this.setState((prevState) => ({
      toggleModalEditLecture: !prevState.toggleModalEditLecture,
    }));
  }
  // 提交修改讲座
  afterConfirmEdit() {
    this.switchModalEditLecture();
    this.loadList(this.state.pageCurrent, this.state.pageSize);
  }
  // 切换报名弹窗显示
  switchModalReg(data) {
    this.regLectureData = data || null;
    this.setState((prevState) => ({
      toggleModalReg: !prevState.toggleModalReg,
    }));
  }
  // 提交报名
  afterConfirmReg() {
    this.switchModalReg();
    this.loadList(this.state.pageCurrent, this.state.pageSize);
  }
  // 打开删除讲座确认modal
  openDeleteConfirmModal(id) {
    TwoTeacherLecture.openConfirmModal('确定删除该讲座吗？', '', () => {
      this.deleteLecture(id);
    });
  }
  // 删除讲座
  deleteLecture(id) {
    this.setState({ loading: true });
    const _self = this;
    let _current = this.state.pageCurrent;
    let size = this.state.pageSize;
    this.props.dispatch({
      type: `${storeName}/deleteLecture`,
      payload: {
        id,
        callback() {
          _self.setState({ loading: false });
          _self.loadList(_current, size);
        },
      },
    });
  }
  // 组件注销前销毁状态
  destroy() {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        mainList: [],
        mainTotal: 0,
      },
    });
  }

  render() {
    const { route, list, total, btnAuth_add } = this.props;
    const { name, loading, listLoading, pageCurrent, pageSize, toggleModalEditLecture, toggleModalReg } = this.state;
    const classPrefix = TwoTeacherLecture.classPrefix;

    return (
      <Page
        name={name}
        className={style[classPrefix]}
        loading={loading}
        route={route}
        fitWindowHeight={true}
        subHeader={
          <div style={{ position: 'relative' }}>
            <Search className={style[`${classPrefix}-search`]} handleSearch={this.search} />
            <Button
              className={style[`${classPrefix}-addBtn`]}
              type="primary"
              onClick={this.addLecture}
              disabled={btnAuth_add}
            >
              新增
            </Button>
          </div>
        }
      >
        <List
          dataSource={list}
          columns={this.columns}
          loading={!loading && listLoading}
          pagination={{
            current: pageCurrent,
            pageSize,
            total,
          }}
          onChange={this.changePage}
        />
        {toggleModalEditLecture && (
          <ModalEditLecture
            lectureId={this.editLectureId}
            stdNum={this.editLectureStdNum}
            onCancel={this.switchModalEditLecture}
            handleConfirm={this.afterConfirmEdit}
          />
        )}
        {toggleModalReg && (
          <ModalReg
            lectureDate={this.regLectureData}
            onCancel={this.switchModalReg}
            handleConfirm={this.afterConfirmReg}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const { mainList, mainTotal } = state[storeName];
  const btnAuth = state.account.btnAuths;
  let btnAuth_add = btnAuth.indexOf('ADD_ONLINE_LECTURE') === -1;
  let btnAuth_reg = btnAuth.indexOf('REG_ONLINE_LECTURE') === -1;
  let btnAuth_detail = btnAuth.indexOf('DETAIL_ONLINE_LECTURE') === -1;
  return {
    list: mainList,
    total: mainTotal,
    btnAuth_add,
    btnAuth_reg,
    btnAuth_detail,
  };
};

export default connect(mapStateToProps)(TwoTeacherLecture);
