/**
 * Created by Haolin<haolinhom@gmail.com> on 2018/12/26.
 */
import { storeName, LECTURE_VAL } from './constant.js';
import { getDictValByText, getDictTextByVal } from 'UTIL/dict';
import {
  lectureClassTwProductSelect,
  saveLectureClassTw,
  queryLectureClassTw,
  updateLectureClassTw,
  queryLectureDetailTw,
  queryLectureWantStudent,
  addStudentToLectureClass,
  delLectureClassTw,
  queryLectureStudentTw,
  removeStudentFromLecture,
  auditLectureClassStudent,
  chargeLectureClassStudent,
} from 'SERVICE/course';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { clone } from 'lodash';

const _fn = {
  // 初始化线上讲座检索条件
  initMainSKey(state) {
    const { branch, year, classType } = state.dict;
    return {
      sKey_main_branch: [branch[0].val],
      sKey_main_year: getDictValByText(year, new Date().getFullYear().toString()) || null,
      sKey_main_classType: classType[LECTURE_VAL][0].val,
    };
  },
  // 构建线上讲座请求参数
  mainListFetchParams(state) {
    const {
      sKey_main_name,
      sKey_main_branch,
      sKey_main_year,
      sKey_main_grade,
      sKey_main_subject,
      sKey_main_classType,
      sKey_main_status,
      sKey_main_date,
      sKey_main_mainTc,
    } = state[storeName];
    return {
      lectureName: sKey_main_name,
      branchId: sKey_main_branch ? sKey_main_branch.join(',') : null,
      yearId: sKey_main_year,
      gradeId: sKey_main_grade ? sKey_main_grade.join(',') : null,
      subjectId: sKey_main_subject ? sKey_main_subject.join(',') : null,
      classTypeId: sKey_main_classType,
      lectureStatus: sKey_main_status,
      lectureDate: sKey_main_date,
      mainTeacherId: sKey_main_mainTc,
    };
  },
  // 初始化选择讲座产品弹窗筛选条件
  initSelectPrdSKey(state) {
    const { year, classType } = state.dict;
    return {
      sKey_selectPrd_year: getDictValByText(year, new Date().getFullYear().toString()) || null,
      sKey_selectPrd_classType: classType[LECTURE_VAL][0].val,
    };
  },
  //
  buildProductDictKey(type, state) {
    const {
      sKey_selectPrd_year,
      sKey_selectPrd_classType,
      sKey_selectPrd_price,
      sKey_selectPrd_totalHour,
      sKey_selectPrd_branch,
    } = state;
    let result;
    switch (type) {
      case 2:
        result = `${sKey_selectPrd_year}${sKey_selectPrd_classType}${sKey_selectPrd_price}`;
        break;
      case 3:
        result = `${sKey_selectPrd_year}${sKey_selectPrd_classType}${sKey_selectPrd_price}${sKey_selectPrd_totalHour}${sKey_selectPrd_branch}`;
        break;
      default:
        result = `${sKey_selectPrd_year}${sKey_selectPrd_classType}`;
    }
    return result;
  },
  //
  checkIsNeedLoadDict(type, state) {
    let result;
    switch (type) {
      case 2:
        result = !state[storeName].dict_totalHour[_fn.buildProductDictKey(type, state[storeName])];
        break;
      case 3:
        result = !state[storeName].dict_product[_fn.buildProductDictKey(type, state[storeName])];
        break;
      default:
        result = !state[storeName].dict_price[_fn.buildProductDictKey(type, state[storeName])];
    }
    return result;
  },
  //
  buildSelectPrdDictParams(state) {
    return {
      productVersionId: state[storeName].sKey_selectPrd_year,
      classTypeId: state[storeName].sKey_selectPrd_classType,
      price: state[storeName].sKey_selectPrd_price,
      classTimeLength: state[storeName].sKey_selectPrd_totalHour,
      branchId: state[storeName].sKey_selectPrd_branch,
      // types 必传,value 1 or 2 or 3
      types: null,
    };
  },
  // 更新选择讲座产品弹窗相关字典数据
  updateSelectPrdDict(type, data, state) {
    switch (type) {
      case 2:
        return _fn.updateTotalHourDict(data, state);
        break;
      case 3:
        return _fn.updateProductDict(data, state);
        break;
      default:
        return _fn.updatePriceDict(data, state);
    }
  },
  updatePriceDict(data, state) {
    let dict =
      data && data.length > 0
        ? data.map((item) => {
            let price = item.price.toString();
            return {
              text: price,
              val: price,
            };
          })
        : null;
    return {
      dict_price: {
        ...state.dict_price,
        [_fn.buildProductDictKey(1, state)]: dict,
      },
    };
  },
  updateTotalHourDict(data, state) {
    let dict =
      data && data.length > 0
        ? data.map((item) => {
            let totalHour = item.classTimeLength.toString();
            return {
              text: totalHour,
              val: totalHour,
            };
          })
        : null;
    return {
      dict_totalHour: {
        ...state.dict_totalHour,
        [_fn.buildProductDictKey(2, state)]: dict,
      },
    };
  },
  updateProductDict(data, state) {
    let dict =
      data && data.length > 0
        ? data.map((item) => {
            let name = `${item.branchName} - ${item.productName}`;
            return {
              text: name,
              val: item.productId,
              data: {
                productName: item.productName,
                productId: item.productId,
                branchName: item.branchName,
                branchId: item.branchId,
                name,
              },
            };
          })
        : [];
    return {
      dict_product: {
        ...state.dict_product,
        [_fn.buildProductDictKey(3, state)]: dict,
      },
    };
  },
  // 获取产品字典里的data属性值
  getDictDataByVal(val, state) {
    const dictStorage = state[storeName].dict_product;
    const dictKey = _fn.buildProductDictKey(3, state[storeName]);
    const productDict = dictStorage[dictKey];
    let productValArr = productDict.map((item) => item.val);
    let index = productValArr.indexOf(val);
    return index === -1 ? null : productDict[index].data;
  },
  //
  deletePrdByVal(list, val) {
    let valArr = list.map((item) => item.productId);
    let index = valArr.indexOf(val);
    if (index !== -1) {
      list.splice(index, 1);
    }
    return list;
  },
  //
  buildProductInfo(state) {
    const { year, classType } = state.dict;
    const classTypeDict = classType[LECTURE_VAL];
    const { sKey_selectPrd_year, sKey_selectPrd_classType, sKey_selectPrd_price, sKey_selectPrd_totalHour } =
      state[storeName];
    return {
      year: {
        text: getDictTextByVal(year, sKey_selectPrd_year),
        val: sKey_selectPrd_year,
      },
      classType: {
        text: getDictTextByVal(classTypeDict, sKey_selectPrd_classType),
        val: sKey_selectPrd_classType,
      },
      price: {
        text: sKey_selectPrd_price,
        val: sKey_selectPrd_price,
      },
      totalHour: {
        text: sKey_selectPrd_totalHour,
        val: sKey_selectPrd_totalHour,
      },
    };
  },
  // 构建新增讲座保存参数
  buildNewLectureParams(state) {
    const { addLecture_productList, addLecture_lectureList } = state[storeName];
    let lectureClassTwBranch = addLecture_productList.map((item) => ({
      branchId: item.branchId,
      productId: item.productId,
    }));
    return addLecture_lectureList.map((item) => ({
      lectureName: item.name,
      gradeId: item.grade.join(','),
      subjectId: item.subject.join(','),
      mainTeacherId: item.mainTc,
      lectureDate: item.beginDate,
      lectureClassTwBranch,
    }));
  },
  // 构建修改讲座保存参数
  buildEditLectureParams(state, id) {
    const {
      editLecture_productList,
      editLecture_name,
      editLecture_grade,
      editLecture_subject,
      editLecture_teacher,
      editLecture_beginDate,
    } = state[storeName];
    let lectureClassTwBranch = editLecture_productList.map((item) => ({
      branchId: item.branchId,
      productId: item.productId,
    }));
    return {
      // id不在这里赋值
      lectureId: id,
      lectureName: editLecture_name,
      gradeId: editLecture_grade.join(','),
      subjectId: editLecture_subject.join(','),
      mainTeacherId: editLecture_teacher,
      lectureDate: editLecture_beginDate,
      lectureClassTwBranch,
    };
  },
  // 初始化报名弹窗筛选条件
  initModalRegSKey(state) {
    const { brenchId, campusId } = state.account.user;
    const { branch, branchCampus } = state.dict;
    const isGroup = brenchId === campusId;
    let sKey_reg_branch = isGroup ? branch[0].val : brenchId;
    let sKey_reg_campus = isGroup ? branchCampus.find((item) => item.val === sKey_reg_branch).options[0].val : campusId;
    return {
      sKey_reg_branch,
      sKey_reg_campus,
    };
  },
  // 构建报名弹窗学生列表参数
  buildGetStdListParams(state) {
    const { sKey_reg_branch, sKey_reg_campus, sKey_reg_grade } = state[storeName];
    return {
      branchId: sKey_reg_branch,
      campusId: sKey_reg_campus,
      gradeId: sKey_reg_grade.join(','),
      pageNo: null,
      pageSize: null,
    };
  },
  // 构建报名讲座提交参数
  buildStdRegLectureParams(state, id, selectedKeys) {
    const { regStdList } = state[storeName];
    let contractProductIdArr = [];
    let studentIdArr = [];
    selectedKeys.forEach((item) => {
      contractProductIdArr.push(regStdList[item].contractProductId);
      studentIdArr.push(regStdList[item].studentId);
    });
    return {
      lectureId: id,
      contractProductId: contractProductIdArr.join(','),
      studentId: studentIdArr.join(','),
    };
  },
  // 初始化讲座已报名学生(学生报名信息)筛选条件
  initRegStdDetailSKey(state) {
    const { brenchId, campusId } = state.account.user;
    const { branch, branchCampus } = state.dict;
    const isGroup = brenchId === campusId;
    let sKey_lecture_branch = isGroup ? branch[0].val : brenchId;
    let sKey_lecture_campus = isGroup
      ? branchCampus.find((item) => item.val === sKey_lecture_branch).options[0].val
      : campusId;
    return {
      sKey_lecture_branch,
      sKey_lecture_campus,
    };
  },
  // 构建请求讲座已报名学生列表参数
  buildLectureStdListParams(state, id, page, rows) {
    const { sKey_lecture_name, sKey_lecture_campus, sKey_lecture_status, sKey_lecture_payStatus } = state[storeName];
    return {
      lectureId: id,
      studentName: sKey_lecture_name,
      campusId: sKey_lecture_campus,
      hasTeacherAttedance: sKey_lecture_status,
      chargeStatus: sKey_lecture_payStatus,
      pageNo: page,
      pageSize: rows,
    };
  },
};
const _attendanceStatus = {
  TRUE: true,
  FALSE: false,
};

export default {
  namespace: storeName,
  state: {
    // 线上讲座(主页面)
    sKey_main_name: '',
    sKey_main_branch: [],
    sKey_main_year: null,
    sKey_main_grade: [],
    sKey_main_subject: [],
    sKey_main_classType: null,
    sKey_main_status: null,
    sKey_main_date: null,
    sKey_main_mainTc: null,
    mainList: [],
    mainTotal: 0,
    // 新增讲座
    addLecture_productInfo: {},
    addLecture_productList: [],
    addLecture_lectureList: [
      {
        name: '',
        grade: [],
        subject: [],
        mainTc: null,
        beginDate: null,
      },
    ],
    // 新增讲座->选择产品弹窗 or 主页面->修改弹窗->选择产品弹窗
    selectPrdType: null,
    addLecture_prdList: [],
    addLecture_prdList_cache: {},
    sKey_selectPrd_year: null,
    sKey_selectPrd_classType: null,
    sKey_selectPrd_price: null,
    sKey_selectPrd_totalHour: null,
    sKey_selectPrd_branch: null,
    sKey_selectPrd_Product: null,
    dict_price: {},
    dict_totalHour: {},
    dict_product: {},
    // 主页面->修改弹窗
    editLecture_canEditPrd: false,
    editLecture_productInfo: {},
    editLecture_productList: [],
    editLecture_name: '',
    editLecture_grade: [],
    editLecture_subject: [],
    editLecture_teacher: null,
    editLecture_beginDate: null,
    // 主页面->报名弹窗
    sKey_reg_branch: null,
    sKey_reg_campus: null,
    sKey_reg_grade: [],
    regStdList: [],
    regStdTotal: 0,
    // 讲座详情
    lecture_detail: null,
    sKey_lecture_name: '',
    sKey_lecture_branch: null,
    sKey_lecture_campus: null,
    sKey_lecture_status: null,
    sKey_lecture_payStatus: null,
    lecture_stdList: [],
    lecture_stdTotal: 0,
  },
  reducers: {
    // 更新state
    updateState(state, { payload }) {
      return {
        ...state,
        ...payload,
      };
    },
    // 更新线上讲座列表数据
    updateMainList(state, { payload: { list, total, page, rows } }) {
      let mainList = list.map((item, index) => ({
        ...item,
        id: item.lectureId,
        branchName: item.branchName.split(','),
        gradeName: item.gradeName.split(','),
        subjectName: item.subjectName.split(','),
        number: index + 1 + (page - 1) * rows,
        beginDate: item.lectureDate,
        regCount: item.nums,
        statusName: item.lectureStatus === 'PENDDING_START' ? '未上课' : '已完成',
      }));
      return {
        ...state,
        mainList,
        mainTotal: total,
      };
    },
    // 更新选择讲座产品弹窗相关字典数据
    updateSelectPrdDict(state, { payload: { type, data } }) {
      let dictData = _fn.updateSelectPrdDict(type, data, state);
      return {
        ...state,
        ...dictData,
      };
    },
    // 新增讲座时更新讲座课堂信息
    updateLectureInfo(state, { payload: { type, val, index } }) {
      let lectureList = state.addLecture_lectureList;
      lectureList[index][type] = val;
      return {
        ...state,
        addLecture_lectureList: lectureList,
      };
    },
    // 更新讲座详情(修改讲座弹窗)
    updateLectureDetailForEdit(state, { payload: { data, stdNum } }) {
      let _price = data.PRICE.toString();
      let _totalHour = data.classTimeLength.toString();
      let productInfo = {
        year: {
          text: data.yearName,
          val: data.yearId,
        },
        classType: {
          text: data.classTypeName,
          val: data.classTypeId,
        },
        price: {
          text: _price,
          val: _price,
        },
        totalHour: {
          text: _totalHour,
          val: _totalHour,
        },
      };
      let productList = data.product;
      productList.forEach((item) => {
        item.name = `${item.branchName} - ${item.productName}`;
      });
      return {
        ...state,
        editLecture_productInfo: productInfo,
        editLecture_productList: productList,
        editLecture_name: data.lectureName,
        editLecture_grade: data.gradeId.split(','),
        editLecture_subject: data.subjectId.split(','),
        editLecture_teacher: data.mainTeacherId,
        editLecture_beginDate: data.lectureDate,
        editLecture_canEditPrd: data.lectureStatus === 'PENDDING_START' && stdNum === 0,
      };
    },
    // 更新报名弹窗学生列表数据
    updateRegStdList(state, { payload: { list, total } }) {
      return {
        ...state,
        regStdList: list,
        regStdTotal: total,
      };
    },
    // 更新讲座详情(详情页)
    updateLectureDetail(state, { payload: { data } }) {
      let lecture_detail = {
        lectureName: data.lectureName,
        productList: data.product,
        year: data.yearName,
        classType: data.classTypeName,
        totalHour: data.classTimeLength,
        price: data.PRICE,
        grade: data.gradeName.split(','),
        subject: data.subjectName.split(','),
        mainTc: data.mainTeacherName,
        beginDate: data.lectureDate,
      };
      return {
        ...state,
        lecture_detail,
      };
    },
    // 更新讲座已报名学生(详情页)
    updateRegStdDetail(
      state,
      {
        payload: {
          list,
          total,
          page,
          rows,
          dict: { baseStatus, lectureStdChargeStatus },
        },
      },
    ) {
      let lecture_stdList = list.map((item, index) => ({
        ...item,
        number: index + 1 + (page - 1) * rows,
        attendanceStatus: _attendanceStatus[item.hasTeacherAttedance],
        attendanceStatusName: getDictTextByVal(baseStatus, item.hasTeacherAttedance),
        chargeStatusName: getDictTextByVal(lectureStdChargeStatus, item.chargeStatus),
      }));
      return {
        ...state,
        lecture_stdList,
        lecture_stdTotal: total,
      };
    },
  },
  effects: {
    // 初始化线上讲座页面
    *initMain({ payload }, { put, select }) {
      const initData = yield select(_fn.initMainSKey);
      yield put({
        type: 'updateState',
        payload: initData,
      });
      yield put({
        type: 'loadMainList',
        payload,
      });
    },
    // 加载线上讲座列表数据
    *loadMainList({ payload: { page, rows, callback, handleErr } }, { put, select }) {
      let fetchParams = yield select(_fn.mainListFetchParams);
      fetchParams.pageNo = page;
      fetchParams.pageSize = rows;
      let rsp = yield queryLectureClassTw(fetchParams, true).catch((err) => {
        showMessage(err.text, 'error');
        if (handleErr) handleErr();
      });
      if (rsp && rsp.resultCode === 0) {
        yield put({
          type: 'updateMainList',
          payload: { list: rsp.datas, total: rsp.rowCount, page, rows },
        });
      }
      if (callback) callback();
    },
    // 重置筛选栏
    *resetMainKeyword(action, { put, select }) {
      const defaultData = {
        sKey_main_name: '',
        sKey_main_branch: [],
        sKey_main_year: null,
        sKey_main_grade: [],
        sKey_main_subject: [],
        sKey_main_classType: null,
        sKey_main_status: null,
        sKey_main_date: null,
        sKey_main_mainTc: null,
      };
      const initData = yield select(_fn.initMainSKey);
      yield put({
        type: 'updateState',
        payload: {
          ...defaultData,
          ...initData,
        },
      });
    },
    // 初始化选择讲座产品弹窗筛选条件
    *initModalSelectPrd({ payload: { callback } }, { put, select }) {
      const initData = yield select(_fn.initSelectPrdSKey);
      yield put({
        type: 'updateState',
        payload: initData,
      });
      yield put({
        type: 'loadSelectPrdDict',
        payload: {
          callback,
        },
      });
      if (callback) callback();
    },
    // 加载选择讲座产品弹窗相关数据字典
    *loadSelectPrdDict({ payload: { type = 1, callback, handleErr } }, { put, select }) {
      // 判断是否需要重新加载该数据字典
      let check = yield select((state) => {
        return _fn.checkIsNeedLoadDict(type, state);
      });
      if (check) {
        let fetchParams = yield select(_fn.buildSelectPrdDictParams);
        fetchParams.types = type;
        const rsp = yield lectureClassTwProductSelect(fetchParams, true).catch((err) => {
          showMessage(err.text, 'error');
          if (handleErr) handleErr();
        });
        if (rsp && rsp.resultCode === 0) {
          yield put({
            type: 'updateSelectPrdDict',
            payload: {
              type,
              data: rsp.data,
            },
          });
        }
      }
      if (callback) callback();
    },
    // 通过下来列表 选择/去除选择 产品
    *selectProduct({ payload: { val, callback } }, { put, select }) {
      let { prdList, prdListCache, branchId } = yield select((state) => ({
        prdList: clone(state[storeName].addLecture_prdList),
        prdListCache: clone(state[storeName].addLecture_prdList_cache),
        branchId: clone(state[storeName].sKey_selectPrd_branch),
      }));
      if (val) {
        // 判断该分公司是否已有选择产品
        if (prdListCache[branchId]) {
          prdList = prdList.filter((item) => item.branchId !== branchId);
        }
        // 选择产品
        const prd = yield select((state) => _fn.getDictDataByVal(val, state));
        prdList.push(prd);
        prdListCache[branchId] = val;
      } else {
        // 去除选择的产品
        if (prdList.length === 1) {
          // 该去除的产品是最后一个已选产品时
          prdList = [];
          prdListCache = {};
        } else {
          prdList = _fn.deletePrdByVal(prdList, prdListCache[branchId]);
          delete prdListCache[branchId];
        }
      }
      yield put({
        type: 'updateState',
        payload: {
          addLecture_prdList: prdList,
          addLecture_prdList_cache: prdListCache,
        },
      });
      if (callback) callback();
    },
    // 如果切换的分公司前面已经有选中产品时，为 sKey_selectPrd_Product 赋值
    *initProductVal({ payload: { branchId, callback } }, { put, select }) {
      const prdListCache = yield select((state) => state[storeName].addLecture_prdList_cache);
      let prdVal = prdListCache[branchId];
      if (prdVal) {
        yield put({
          type: 'updateState',
          payload: { sKey_selectPrd_Product: prdVal },
        });
      }
      if (callback) callback();
    },
    // 通过已选产品列表 删除 已选产品
    *deleteProduct({ payload: { index, callback } }, { put, select }) {
      let { prdList, prdListCache, sKey_selectPrd_branch, sKey_selectPrd_Product } = yield select((state) => ({
        prdList: clone(state[storeName].addLecture_prdList),
        prdListCache: clone(state[storeName].addLecture_prdList_cache),
        sKey_selectPrd_branch: clone(state[storeName].sKey_selectPrd_branch),
        sKey_selectPrd_Product: clone(state[storeName].sKey_selectPrd_Product),
      }));
      if (prdList.length === 1) {
        prdList = [];
        prdListCache = {};
      } else {
        let { productId, branchId } = prdList[index];
        if (sKey_selectPrd_branch === branchId && sKey_selectPrd_Product === productId) {
          yield put({
            type: 'updateState',
            payload: { sKey_selectPrd_Product: null },
          });
        }
        prdList.splice(index, 1);
        delete prdListCache[branchId];
      }
      yield put({
        type: 'updateState',
        payload: {
          addLecture_prdList: prdList,
          addLecture_prdList_cache: prdListCache,
        },
      });
      if (callback) callback();
    },
    // 确认选好讲座产品
    *confirmProduct({ payload: { type, callback } }, { put, select }) {
      const productInfo = yield select(_fn.buildProductInfo);
      const productList = yield select((state) => state[storeName].addLecture_prdList);
      yield put({
        type: 'updateState',
        payload: {
          [`${type}Lecture_productInfo`]: clone(productInfo),
          [`${type}Lecture_productList`]: clone(productList),
        },
      });
      if (callback) callback();
    },
    // 编辑已选讲座产品时初始化
    *initModalSelectPrdEdit({ payload: { type, callback } }, { put, select }) {
      const prdInfo = yield select((state) => {
        const _prdInfo = state[storeName][`${type}Lecture_productInfo`];
        return {
          year: _prdInfo.year.val,
          classType: _prdInfo.classType.val,
          price: _prdInfo.price.val,
          totalHour: _prdInfo.totalHour.val,
        };
      });
      const { prdList, prdListCache } = yield select((state) => {
        const prdList = state[storeName][`${type}Lecture_productList`];
        const branchIdArr = prdList.map((item) => item.branchId);
        let prdListCache = {};
        branchIdArr.forEach((item, index) => {
          prdListCache[item] = prdList[index].productId;
        });
        return { prdList, prdListCache };
      });
      yield put({
        type: 'updateState',
        payload: {
          addLecture_prdList: prdList,
          addLecture_prdList_cache: prdListCache,
          sKey_selectPrd_year: prdInfo.year,
          sKey_selectPrd_classType: prdInfo.classType,
          sKey_selectPrd_price: prdInfo.price,
          sKey_selectPrd_totalHour: prdInfo.totalHour,
        },
      });
      yield put({
        type: 'loadSelectPrdDict',
        payload: { type: 1 },
      });
      yield put({
        type: 'loadSelectPrdDict',
        payload: { type: 2 },
      });
      if (callback) callback();
    },
    // 新增讲座产品
    *submitNewLecture({ payload: { callback, handleErr } }, { select }) {
      const fetchParams = yield select(_fn.buildNewLectureParams);
      const rsp = yield saveLectureClassTw(fetchParams, true).catch((err) => {
        showMessage(err.text, 'error');
        if (handleErr) handleErr();
      });
      if (rsp && rsp.resultCode === 0) {
        showMessage('新增讲座成功！', 'success');
      }
      if (callback) callback();
    },
    // 获取讲座详情(修改讲座 与 讲座详情 共用)
    *getLectureDetail({ payload: { id, stdNum, type, callback, handleErr } }, { put }) {
      const rsp = yield queryLectureDetailTw({ lectureId: id }, true).catch((err) => {
        showMessage(err.text, 'error');
        if (handleErr) handleErr();
      });
      if (rsp && rsp.resultCode === 0) {
        let updateType = type === 'edit' ? 'updateLectureDetailForEdit' : 'updateLectureDetail';
        yield put({
          type: updateType,
          payload: {
            data: rsp.data,
            stdNum: stdNum || 0,
          },
        });
      }
      if (callback) callback();
    },
    // 修改讲座
    *submitEditLecture({ payload: { id, callback, handleErr } }, { select }) {
      let fetchParams = yield select((state) => _fn.buildEditLectureParams(state, id));
      const rsp = yield updateLectureClassTw(fetchParams, true).catch((err) => {
        showMessage(err.text, 'error');
        if (handleErr) handleErr();
      });
      if (rsp && rsp.resultCode === 0) {
        showMessage('修改讲座成功！', 'success');
      }
      if (callback) callback();
    },
    // 删除讲座
    *deleteLecture({ payload: { id, callback, handleErr } }, effects) {
      const rsp = yield delLectureClassTw({ lectureId: id }, true).catch((err) => {
        showMessage(err.text, 'error');
        if (handleErr) handleErr();
      });
      if (rsp && rsp.resultCode === 0) {
        showMessage('删除讲座成功！', 'success');
      }
      if (callback) callback();
    },
    // 初始化报名弹窗
    *initModalReg({ payload: { id, page, rows, callback } }, { put, select }) {
      const sKey = yield select(_fn.initModalRegSKey);
      yield put({
        type: 'updateState',
        payload: sKey,
      });
      yield put({
        type: 'loadStdList',
        payload: { id, page, rows, callback },
      });
    },
    // 加载学生列表
    *loadStdList({ payload: { id, page, rows, callback, handleErr } }, { put, select }) {
      let fetchParams = yield select((state) => ({
        lectureId: id,
        branchId: state[storeName].sKey_reg_branch,
        campusId: state[storeName].sKey_reg_campus,
        gradeId: state[storeName].sKey_reg_grade,
        pageNo: page,
        pageSize: rows,
      }));
      const rsp = yield queryLectureWantStudent(fetchParams, true).catch((err) => {
        showMessage(err.text, 'error');
        if (handleErr) handleErr();
      });
      if (rsp && rsp.resultCode === 0) {
        yield put({
          type: 'updateRegStdList',
          payload: { list: rsp.datas, total: rsp.rowCount },
        });
      }
      if (callback) callback();
    },
    // 提交报名
    *stdRegLecture({ payload: { id, selectedKeys, callback, handleErr } }, { select }) {
      let fetchParams = yield select((state) => _fn.buildStdRegLectureParams(state, id, selectedKeys));
      const rsp = yield addStudentToLectureClass(fetchParams, true).catch((err) => {
        showMessage(err.text, 'error');
        if (handleErr) handleErr();
      });
      if (rsp && rsp.resultCode === 0) {
        showMessage('报名讲座成功！', 'success');
      }
      if (callback) callback();
    },
    // 初始化讲座已报名学生详情(属于讲座详情)
    *initRegStdDetail({ payload: { id, page, rows, callback } }, { put, select }) {
      const sKey = yield select(_fn.initRegStdDetailSKey);
      yield put({
        type: 'updateState',
        payload: sKey,
      });
      yield put({
        type: 'loadLectureStdList',
        payload: { id, page, rows, callback },
      });
    },
    // 加载讲座已报名学生列表
    *loadLectureStdList({ payload: { id, page, rows, callback, handleErr } }, { put, select }) {
      let fetchParams = yield select((state) => _fn.buildLectureStdListParams(state, id, page, rows));
      const rsp = yield queryLectureStudentTw(fetchParams, true).catch((err) => {
        showMessage(err.text, 'error');
        if (handleErr) handleErr();
      });
      if (rsp && rsp.resultCode === 0) {
        const dict = yield select((state) => state.dict);
        yield put({
          type: 'updateRegStdDetail',
          payload: { list: rsp.datas, total: rsp.rowCount, page, rows, dict },
        });
      }
      if (callback) callback();
    },
    // 重置讲座已报名学生列表检索栏
    *resetLectureStdKeyword(action, { put, select }) {
      const sKey = yield select(_fn.initRegStdDetailSKey);
      const defaultSKey = {
        sKey_lecture_name: '',
        sKey_lecture_branch: null,
        sKey_lecture_campus: null,
        sKey_lecture_status: null,
        sKey_lecture_payStatus: null,
        ...sKey,
      };
      yield put({
        type: 'updateState',
        payload: defaultSKey,
      });
    },
    // 退讲座(退班)
    *quitLecture({ payload: { lectureId, studentId, callback, handleErr } }, effects) {
      let rsp = yield removeStudentFromLecture({ lectureId, studentId }, true).catch((err) => {
        showMessage(err.text, 'error');
        if (handleErr) handleErr();
      });
      if (rsp && rsp.resultCode === 0) {
        showMessage('该学生退班成功！', 'success');
      }
      if (callback) callback();
    },
    // 讲座学生批量考勤
    *auditLectureStd({ payload: { selectKeys, callback, handleErr } }, { put, select }) {
      let checkAudit = yield select((state) =>
        selectKeys.some((item) => state[storeName].lecture_stdList[item].attendanceStatus),
      );
      if (checkAudit) {
        showMessage('请选择未上课的学生进行考勤！', 'error');
        return;
      }
      let idArr = yield select((state) => selectKeys.map((item) => state[storeName].lecture_stdList[item].id));
      let fetchParams = {
        auditStatus: 'CONPELETE',
        ids: idArr.join(','),
      };
      let rsp = yield auditLectureClassStudent(fetchParams, true).catch((err) => {
        showMessage(err.text, 'error');
        if (handleErr) handleErr();
      });
      if (rsp && rsp.resultCode === 0) {
        showMessage('考勤成功！', 'success');
      }
      if (callback) callback();
    },
    // 讲座学生批量扣费
    *chargeLectureStd({ payload: { selectKeys, callback, handleErr } }, { put, select }) {
      let checkAudit = yield select((state) =>
        selectKeys.some((item) => state[storeName].lecture_stdList[item].auditStatus === 'NEW'),
      );
      if (checkAudit) {
        showMessage('所选学生存在未考勤的，请先考勤再扣费！', 'error');
        return;
      }
      let checkCharge = yield select((state) =>
        selectKeys.some((item) => state[storeName].lecture_stdList[item].chargeStatus === 'CHARGED'),
      );
      if (checkCharge) {
        showMessage('请选择未扣费的学生进行扣费！', 'error');
        return;
      }
      let idArr = yield select((state) => selectKeys.map((item) => state[storeName].lecture_stdList[item].id));
      let fetchParams = {
        ids: idArr.join(','),
      };
      let rsp = yield chargeLectureClassStudent(fetchParams, true).catch((err) => {
        showMessage(err.text, 'error');
        if (handleErr) handleErr();
      });
      if (rsp && rsp.resultCode === 0) {
        showMessage('扣费成功！', 'success');
      }
      if (callback) callback();
    },
  },
};
