import React from 'react';

import { Modal, Spin, message } from 'antd';
import { TextArea, DatePicker, Form } from 'COMPONENT';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import UploadImg from '../UploadImg';

import * as attendanceMakeUpClassApi from 'SERVICE/attendanceMakeUpClass';

import _ from 'lodash';

import styles from './styles.module.less';

const labelColSpan = 5;
const wrapperColSpan = 24 - labelColSpan;

const FormItem = Form.Item;

export default class AttendanceModal extends React.Component {
  state = {
    isUploading: false,
    confirmLoading: false,
    supplementUrls: null,
  };

  constructor(props) {
    super(props);
  }

  componentDidMount() {
    const { students } = this.props;
    if (students.length === 1) {
      this.formRef?.setFieldsValue({
        supplementRemark: students[0]?.supplementRemark,
        supplementDate: students[0]?.supplementDate,
      });
      if (students[0]?.supplementUrls) {
        this.setState({
          supplementUrls: { src: students[0]?.supplementUrls },
        });
      }
    }
  }

  handleSubmit = () => {
    this.formRef.validateFields((error, values) => {
      if (this.state.isUploading) {
        return message.warn('图片上传中，请稍等再提交');
      }
      if (error) {
        return;
      }
      const { supplementUrls } = this.state;
      this.setState({ confirmLoading: true });
      attendanceMakeUpClassApi
        .saveAttendanceMinClass({
          supplementUrls: supplementUrls?.src,
          ..._.omit(values, ['student']),
          attendanceIds: this.props.students.map(({ attendanceId }) => attendanceId),
        })
        .then(() => {
          showMessage('补课登记成功', 'success');
          this.props.onSuccess(this.props.students?.length > 1);
        })
        .catch((e) => {
          showMessage(e?.text || '补课登记失败，请稍后重试');
        })
        .finally(() => {
          this.setState({ confirmLoading: false });
        });
    });
  };

  handleClose = () => {
    this.props.onClose();
  };

  // 监听上传回调
  handleUploadComplete = (supplementUrls) => {
    this.setState({ supplementUrls });
  };

  handleSetUploadStatus = (isUploading) => {
    this.setState({ isUploading });
  };

  // 监听删除回调
  handleDeleteComplete = () => {
    this.setState({ supplementUrls: null });
  };

  render() {
    const { confirmLoading, supplementUrls, isUploading } = this.state;
    const { students } = this.props;
    const modalProps = {
      width: 500,
      visible: true,
      confirmLoading,
      maskClosable: false,
      onOk: this.handleSubmit,
      onCancel: this.handleClose,
      title: students?.length > 1 ? '批量登记 ' : '补课登记',
    };

    return (
      <Modal {...modalProps}>
        <Spin spinning={isUploading} tip="图片上传中...">
          <Form ref={(c) => (this.formRef = c)} onSubmit={() => {}}>
            <FormItem
              name="student"
              formItemProps={{
                label: '学生',
                colon: false,
                labelCol: { span: labelColSpan },
                wrapperCol: { span: wrapperColSpan },
              }}
            >
              <div className={styles.students}>
                {students
                  ?.map(
                    ({ courseDate, studentName }) =>
                      `${courseDate ? `${studentName}(${courseDate})` : `${studentName}`}`,
                  )
                  .join('、')}
              </div>
            </FormItem>
            <FormItem
              name="supplementDate"
              rules={[{ required: true, message: '补课日期不能为空' }]}
              formItemProps={{
                label: '补课日期',
                colon: false,
                labelCol: { span: labelColSpan },
                wrapperCol: { span: wrapperColSpan },
              }}
            >
              <DatePicker size="large" />
            </FormItem>
            <FormItem
              name="supplementRemark"
              formItemProps={{
                label: '备注',
                colon: false,
                labelCol: { span: labelColSpan },
                wrapperCol: { span: wrapperColSpan },
              }}
            >
              <TextArea rows={3} wordCount maxLength={200} />
            </FormItem>
          </Form>
          {this.props.students?.length === 1 && (
            <div style={{ paddingLeft: '98px' }}>
              <UploadImg
                text="上传图片"
                value={supplementUrls}
                onSetUploadStatus={this.handleSetUploadStatus}
                uploadComplete={(val) => this.handleUploadComplete(val)}
                deleteComplete={(val) => this.handleDeleteComplete(val)}
                size={160}
              />
            </div>
          )}
        </Spin>
      </Modal>
    );
  }
}
