import React from 'react';
import './styles.less';
import Icon from 'COMPONENT/Icon';
import IconButton from 'COMPONENT/IconButton';
import Popover from 'COMPONENT/Popover';
import QrcodeFileUpload from 'COMPONENT/QrcodeFileUpload';
import Upload from 'COMPONENT/Upload';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { Modal as AntdModal } from 'antd';
import PreviewImgs from 'COMPONENT/PreviewImgs';
import Viewer from 'viewerjs';

const validateFileTypes = ['png', 'jpg', 'jpeg', 'bmp'];

class UploadImg extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      file: props.value || null,
      btnCover: false, // 切换显示 按钮层
      previewImg: false, // 图片预览,
    };
  }

  componentWillReceiveProps = (nextProps) => {
    console.log(nextProps);
    this.setState({ file: nextProps.value || null });
  };

  /**
   * 二维码传图回调
   *
   filePath:"阿里云oss"
   fileSize:"7467B"
   fileid:"UPL000000004645"
   realFileName:"mmexport1529651473907.jpg"
   type:""
   *
   * */
  get2dUploadImgInfo = (info) => {
    const { params, uploadComplete } = this.props;
    const len = info.length;
    if (len > 0) {
      const file = info[len - 1];
      if (file.filePath && file.filePath !== '') {
        let newFile = {
          fileId: file.fileid,
          fileName: file.realFileName,
          src: file.filePath,
          ...params,
          fileSize: file.fileSize,
        };
        this.setState({ file: newFile });
        uploadComplete(newFile);
      }
    }
  };

  /**
   * 图片上传时不断调用
   * 当上传成功时，可获取到全部file数据
   * 调用父组件传递进来的callback
   * */
  onUploadChange = (info) => {
    const { params, uploadComplete } = this.props;
    const file = info;
    // console.log("oldFile:",file);
    if (file.src && file.src !== '') {
      let newFile = {
        ...file,
        ...params,
        fileId: file['x:fileid'],
        fileName: file.name,
      };
      //
      if (!newFile.fileId && !newFile.fileName) {
        showMessage('图片上传失败，请联系管理人员！', 'error');
        return;
      }
      // console.table(newFile);
      this.setState({ file: newFile });
      uploadComplete(newFile);
      showMessage('图片上传成功！', 'success');
    }
  };

  /**
   * 图片上传前进行校验
   * 必需返回一个fileList [Array]
   * */
  checkLength = (files) => {
    // console.log("校验图片",files);
    if (files.length > 0) {
      let file = files[0];
      if (!validateFileTypes.some((item) => file.type.indexOf(item) !== -1)) {
        // console.log("图片格式不正确",file.type);
        showMessage('图片格式不正确！', 'warning');
        return [];
      }
      return files;
    }
  };

  /**
   * 按钮遮罩层
   * */
  btnCoverToggle = () => {
    this.setState((prevState, props) => ({
      btnCover: !prevState.btnCover,
    }));
  };

  /**
   * 设置插件大小
   * */
  setSize = (size, type) => {
    let result = {};
    switch (type) {
      case 1:
        result.width = size;
        result.height = size;
        break;
      case 2:
        result.width = size - 20;
        result.height = size - 20;
        break;
      case 3:
        let icon = parseInt(size / 5);
        result.width = icon;
        result.height = icon;
        break;
      default:
        result.width = 100;
        result.height = 100;
    }
    return result;
  };
  setIconSize = (size) => {
    return parseInt(size / 5);
  };

  /**
   * 预览
   * */
  previewToggle = (e) => {
    // this.setState((prevState, props) => ({
    //   previewImg: !prevState.previewImg,
    // }));
    const image = new Image();
    image.src = this.state?.file?.src;
    const viewer = new Viewer(image, {
      hidden: function () {
        viewer.destroy();
      },
    });
    viewer.show();
  };

  /**
   * 删除图片
   * */
  deleteImg = (e) => {
    const { deleteComplete } = this.props;
    const { file } = this.state;
    const that = this;

    AntdModal.confirm({
      title: '是否删除此图片？',
      content: '',
      okType: 'danger',
      onOk() {
        deleteComplete(file.fileId);
        that.setState({ file: null, btnCover: false });
      },
    });
    this.setState((prevState) => {
      return { btnCover: false };
    });
  };

  /**
   * 生成预览图样式
   * */
  setPreview = (img) => {
    return {
      backgroundImage: 'url(' + img + ')',
      backgroundSize: 'contain',
      backgroundPosition: 'center center',
      backgroundRepeat: 'no-repeat',
    };
  };

  render() {
    const { size, text = '上传', disabled = false, viewOnly = false } = this.props;
    const { file, previewImg } = this.state;

    let size_1 = this.setSize(size, 1);
    let size_2 = this.setSize(size, 2);
    let size_3 = this.setSize(size, 3);

    let iconSize = this.setIconSize(size);

    let imgSrc = file && file.src ? file.src : '';
    let imgList = [];
    imgList.push(imgSrc);
    // let imgStyle = this.setPreview(file&&file.src?file.src:"");

    return (
      <div className="UploadImg AttendanceUploadImg">
        <div className="hom-upload-img" style={size_1}>
          <div className="hom-upload-img-qr" style={size_3}>
            <Popover
              placement="topRight"
              content={
                <div>
                  <QrcodeFileUpload height={160} width={160} callback={this.get2dUploadImgInfo} multiple />
                  <div style={{ textAlign: 'center', marginTop: 10 }}>手机扫码上传图片</div>
                </div>
              }
            >
              <Icon type="2d_code_half_mc" size={iconSize} />
            </Popover>
          </div>
          <div className="hom-upload-img-picker">
            <Upload
              multiple={false}
              hasExtension={true}
              onChange={this.onUploadChange}
              onSetUploadStatus={this.props.onSetUploadStatus}
              beforeUpload={(files) => this.checkLength(files)}
            >
              <div className="hom-upload-img-picker-btn">
                <div>
                  <Icon type="plus" size={iconSize} />
                </div>
                <div>{text}</div>
              </div>
            </Upload>
          </div>
          {(disabled || file || viewOnly) && <div className="hom-upload-img-disabled"> </div>}
          {file && file.src !== '' && (
            <div
              className="hom-upload-img-view"
              style={size_2}
              // onMouseEnter={this.btnCoverToggle}
              // onMouseLeave={this.btnCoverToggle}
            >
              <div className="hom-upload-img-view-wrapper">
                <img className="hom-upload-img-view-img" src={file.src} />
              </div>

              <div className="hom-upload-img-cover">
                <div className="hom-upload-img-detail" onClick={(e) => this.previewToggle(e)} title="预览图片">
                  <Icon type="visible_mc" color="#000000" size={iconSize} />
                </div>
                {!viewOnly && (
                  <div className="hom-upload-img-delete" onClick={(e) => this.deleteImg(e)} title="删除图片">
                    <Icon type="delete_mc" color="#000000" size={iconSize} />
                  </div>
                )}
              </div>
            </div>
          )}
        </div>
        {previewImg && (
          <div className="hom-upload-img-preview">
            <div className="hom-upload-img-preview-wrapper">
              <PreviewImgs list={imgList} activeSrc={imgSrc} showThumbnail={false} />
            </div>
            <div className="hom-upload-img-preview-close" onClick={(e) => this.previewToggle(e)}>
              <Icon type="cross_circle" size={32} />
            </div>
          </div>
        )}
      </div>
    );
  }
}

export default UploadImg;
