import React from 'react';
import { browserHistory } from 'react-router';
import { OLDPAGEBASE } from 'CONST/config';
import { Page, List, TextLink, Button } from 'COMPONENT';
import { Modal, Table, DatePicker, TimePicker, Radio, Checkbox } from 'antd';
import Loading from 'COMPONENT/Loading';
import Search from './search';
import columns from './column';
import styles from './styles.module.less';
import {
  getAllMiniClassCourse,
  batchDeleteCourse,
  batchModifyCourse,
  updateTeachingTypeByIds,
  updateTeachingTypeByParams,
} from 'SERVICE/course';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import moment from 'moment';
import { teachingTypeEnum, teachingTypeDict } from 'CONST/dicts/course';
const RadioGroup = Radio.Group;

class BatchModifyCourse extends React.Component {
  constructor(props) {
    super(props);

    this.state = Object.assign({
      loading: false, // {boolean} 内容加载状态
      list: [], // {Array} 列表数据，
      totalRecords: 0, // {number} 总记录数
      currentPage: 1, // {number} 当前页码
      pageSize: 50, // {number} 每页记录数
      searchParams: {},
      selectedRowKeys: [],
      selectedRows: [],
      showModifyModal: false,
      showModifyTeachingTypeModal: false,
      modifyCourseList: [],
      confirmLoading: false,
      teachingType: teachingTypeEnum.FACE_TO_FACE,
      modifyTeachingTypeResult: {},
      modifyTimeResult: {},
      selectAllCheckType: false,
    });
  }

  fetchBigClassList = () => {
    let searchParams = this.state.searchParams;
    let { startDate, endDate, branchId } = searchParams;
    let pagination = {
      pageSize: this.state.pageSize,
      pageNum: this.state.currentPage,
    };

    let params = {
      ...searchParams,
      ...pagination,
    };

    if (!startDate || !endDate || !branchId) {
      return showMessage('开始时间，结束时间，所属分公司必填');
    }

    this.setRequestLoading(true);
    getAllMiniClassCourse(params)
      .then((res) => {
        let courseList = res.data.list || [];
        // 按同一个课程日期、同一个上课时间、同一个课程时长的课程，归集为一行。

        courseList = courseList.map((c) => {
          return {
            ...c,
            groupKey: `${c.courseDate}-${c.courseTime}-${c.courseEndTime}-${c.courseMinutes}`,
          };
        });

        this.setState({
          list: courseList,
          totalRecords: res.data.total,
          loading: false,
          selectedRowKeys: [],
          selectedRows: [],
        });
      })
      .catch((e) => {
        showMessage(e.text || '请求异常');
        this.setRequestLoading(false);
      });
  };

  onSearch = (searchParams) => {
    this.setState(
      {
        searchParams,
        currentPage: 1,
      },
      this.fetchBigClassList,
    );
  };
  toggleSelectAll = (e) => {
    let searchParams = this.state.searchParams;
    let { startDate, endDate, branchId } = searchParams;
    let pagination = {
      pageSize: this.state.pageSize,
      pageNum: this.state.currentPage,
    };
    let params = {
      ...searchParams,
      ...pagination,
    };

    if (!startDate || !endDate || !branchId) {
      return showMessage('查询项必须选中日期、分公司、学习中心');
    }
    this.setState(
      (state) => {
        console.log(state);
        return {
          ...state,
          selectAllCheckType: !state.selectAllCheckType,
        };
      },
      () => {
        // let {selectAllCheckType,list} = this.state
        // let keys =[];
        // let rows=[];
        // if(selectAllCheckType){
        //   // 全选勾选
        //   const _TempKeys = []
        //   const _TempRows = []
        //   list.map((i,idx)=>{
        //     _TempKeys.push(idx);
        //     _TempRows.push({
        //       ...i,
        //       id:idx
        //     })
        //   })
        //   keys = _TempKeys
        //   rows = _TempRows
        // }
        // // 设置table的勾选为全选|空
        // this.onRowSelectChange(keys, rows)
      },
    );
  };
  onPaginationChange = (pageData) => {
    this.setState(
      {
        currentPage: pageData.current,
        pageSize: pageData.pageSize,
      },
      this.fetchBigClassList,
    );
  };

  setRequestLoading = (loading) => {
    this.setState({
      loading,
    });
  };

  // 跳转小班详情
  toClassDetail = (row) => {
    let path = `function/course/miniClassDetail.html?miniClassId=${row.miniClassId}`;
    let title = `小班详情`;

    browserHistory.push({
      pathname: OLDPAGEBASE,
      query: { src: path, title },
    });
  };

  // 跳转小班课程详情
  toClassCourseDetail = (row) => {
    let path = `function/course/miniClassCourseSchedule.html?miniClassId=${row.miniClassId}&miniClassName=${row.miniClassName}&miniClassGrade=${row.grade}&miniClassSubject=${row.subject}&blCampusId=${row.blCampusId}&productGroupId=${row.produceId}&studyManegerId=${row.studyManegerId}`;
    let title = `小班课程详情`;

    browserHistory.push({
      pathname: OLDPAGEBASE,
      query: { src: path, title },
    });
  };

  onModify = () => {
    if (this.state.selectedRowKeys.length <= 0) {
      return showMessage('请先选择课程');
    }

    const modifyCourseList = this.state.selectedRows.map((r) => {
      let res = {
        ...r,
        disabled: false,
        newStartDate: moment(r.courseDate, 'YYYY-MM-DD'),
        newStartTime: moment(r.courseTime, 'HH:mm'),
      };
      // if(r.teachingType === teachingTypeEnum.LIVE || r.teachingType === teachingTypeEnum.RECORDED){
      //   res = {
      //     ...res,
      //     newStartDate:moment(moment().format( "YYYY-MM-DD")) ,
      //     newStartTime:moment(),
      //   }
      // }
      return res;
    });

    const groupMap = modifyCourseList.reduce((pre, cur) => {
      if (!pre[cur.groupKey]) {
        pre[cur.groupKey] = {
          key: cur.groupKey,
          courseList: [],
          courseDate: cur.courseDate,
          courseTime: cur.courseTime,
          courseEndTime: cur.courseEndTime,
          courseMinutes: cur.courseMinutes,
          courseHours: cur.courseHours,
          newStartDate: cur.newStartDate,
          newStartTime: cur.newStartTime,
          teachingType: cur.teachingType,
          disabled: cur.disabled,
        };
      }
      pre[cur.groupKey].courseList.push(cur);
      return pre;
    }, {});

    const groupList = Object.keys(groupMap).reduce((pre, cur) => {
      pre.push(groupMap[cur]);
      return pre;
    }, []);

    this.setState({
      showModifyModal: true,
      modifyCourseList: groupList,
    });
  };

  onDelete = () => {
    if (this.state.selectedRowKeys.length <= 0) {
      return showMessage('请先选择课程');
    }
    const courseIds = this.state.selectedRows.map((r) => r.miniClassCourseId);

    Modal.confirm({
      content: (
        <p>
          提示:是否确认删除所选排课？
          <br /> <br />
          删除后，如小班存在未排课小时，请及时排完！
        </p>
      ),
      okText: '确认',
      cancelText: '取消',
      onOk: () => {
        return new Promise((reslove) => {
          return batchDeleteCourse({ courseIds })
            .then((res) => {
              // Modal.success({
              //   content: (
              //     <p>
              //       <h3>提交成功</h3>
              //       <p style={{ textAlign: "left" }}>
              //         <p style={{ marginTop: "20px" }}>
              //           已选：{this.state.selectedRowKeys.length || 0}{" "}
              //           &nbsp;&nbsp;修改成功：{res.data.sucUpdate || 0}{" "}
              //           &nbsp;&nbsp;修改失败：{res.data.failUpdate || 0}
              //         </p>
              //         <p style={{ marginTop: "20px", color: "red" }}>
              //           课程状态为：未上课的课程才允许删除~
              //         </p>
              //       </p>
              //     </p>
              //   ),
              //   okText: "知道了",
              // });

              showMessage('删除成功', 'success');
              this.fetchBigClassList();
            })
            .catch((e) => {
              showMessage(e.text || '删除失败 ,课程状态为：未上课的课程才允许删除~');
            })
            .finally(() => {
              reslove();
            });
        });
      },
    });
  };

  onModifyTeachType = () => {
    const { selectedRows, selectAllCheckType, list } = this.state;
    if (selectAllCheckType) {
      // 如果全选搜索结果按钮已经勾选,调用另一个方法
      this.setState({
        teachingType: list[0].teachingType,
        showModifyTeachingTypeModal: true,
      });
      return;
    }

    if (selectedRows.length <= 0) {
      return showMessage('请先选择课程');
    }

    this.setState({
      teachingType: selectedRows[0].teachingType,
      showModifyTeachingTypeModal: true,
    });
  };

  onSaveModifyTeachingType = () => {
    const { teachingType, selectedRows = [], selectAllCheckType, searchParams, newTeachingType } = this.state;
    if (selectAllCheckType) {
      let parameters = {
        ...searchParams,
      };
      if (newTeachingType) {
        parameters = { ...parameters, newTeachingType };
      }
      updateTeachingTypeByParams(parameters)
        .then((res) => {
          this.fetchBigClassList();
          this.setState(
            {
              showModifyTeachingTypeModal: false,
            },
            () => {
              Modal.success({
                content: (
                  <p>
                    <h3>提交成功</h3>
                    <p style={{ textAlign: 'left' }}>
                      <p style={{ marginTop: '20px' }}>
                        已选：{res.data.allUpdate || 0} &nbsp;&nbsp;修改成功：{res.data.sucUpdate || 0}{' '}
                        &nbsp;&nbsp;修改失败：{res.data.failUpdate || 0}
                      </p>
                      <p style={{ marginTop: '20px', color: 'red' }}>
                        修改失败原因：①仅支持修改状态为【未上课】课程的上课方式；②本地网课上课方式不允许修改；
                      </p>
                    </p>
                  </p>
                ),
                okText: '知道了',
              });
            },
          );
        })
        .catch((e) => {
          showMessage(e.text || '操作失败');
        });
      return;
    }
    const miniClassCourseIds = selectedRows.map((r) => r.miniClassCourseId);

    updateTeachingTypeByIds({ teachingType: teachingType, miniClassCourseIds })
      .then((res) => {
        this.fetchBigClassList();

        this.setState(
          {
            modifyTeachingTypeResult: res.data,
            showModifyTeachingTypeModal: false,
          },
          () => {
            this.modifyTeachingTypeSuccess();
          },
        );
      })
      .catch((e) => {
        showMessage(e.text || '操作失败');
      });
  };

  modifyTeachingTypeSuccess = () => {
    const { modifyTeachingTypeResult } = this.state;

    Modal.success({
      content: (
        <p>
          <h3>提交成功</h3>
          <p style={{ textAlign: 'left' }}>
            <p style={{ marginTop: '20px' }}>
              已选：{modifyTeachingTypeResult.allUpdate || 0} &nbsp;&nbsp;修改成功：
              {modifyTeachingTypeResult.sucUpdate || 0} &nbsp;&nbsp;修改失败：{modifyTeachingTypeResult.failUpdate || 0}
            </p>
            <p style={{ marginTop: '20px', color: 'red' }}>
              修改失败原因：①仅支持修改状态为【未上课】课程的上课方式；②本地网课上课方式不允许修改；
            </p>
          </p>
        </p>
      ),
      okText: '知道了',
    });
  };

  validateCourseTime = (courses) => {
    for (let i = 0; i < courses.length; i++) {
      const courseItem = courses[i];
      const isBefore = moment(`${courseItem.courseDate} ${courseItem.courseTime}`).isBefore(new Date());
      if (
        !courseItem.disabled &&
        (courseItem.teachingType === teachingTypeEnum.LIVE || courseItem.teachingType === teachingTypeEnum.RECORDED) &&
        isBefore
      ) {
        return false;
      }
    }

    return true;
  };

  onSaveModifyCourseTime = () => {
    const modifyCourseList = this.state.modifyCourseList;
    const checkAllHaveValue = modifyCourseList.every((course) => {
      return course.newStartDate && course.newStartTime;
    });
    if (!checkAllHaveValue) {
      return showMessage('请填写上课日期以及开始时间');
    }

    const params = modifyCourseList.reduce((pre, cur) => {
      const courseIds = cur.courseList.map((c) => c.miniClassCourseId);

      pre.push({
        courseIds,
        courseDate: cur.newStartDate.format('YYYY-MM-DD'),
        courseTime: cur.newStartTime.format('HH:mm'),
        teachingType: cur.teachingType,
        disabled: cur.disabled,
      });

      return pre;
    }, []);

    if (!this.validateCourseTime(params)) {
      showMessage('“在线A”或“在线B”的课程上课时间不能选择历史时间~');
      return;
    }

    this.setState({
      confirmLoading: true,
    });
    batchModifyCourse({ miniClassCourseUpdateDtos: params })
      .then((res) => {
        this.fetchBigClassList();

        this.setState({ modifyTimeResult: res.data, showModifyModal: false }, () => {
          this.modifyCourseTimeSuccess();
        });
      })
      .catch((e) => {
        showMessage(e.text || '操作失败');
      })
      .finally(() => {
        this.setState({
          confirmLoading: false,
        });
      });
  };

  modifyCourseTimeSuccess = () => {
    const { modifyTimeResult } = this.state;

    Modal.success({
      content: (
        <p>
          <h3>提交成功</h3>
          <p style={{ textAlign: 'left' }}>
            <p style={{ marginTop: '20px' }}>
              已选：{modifyTimeResult.allUpdate || 0} &nbsp;&nbsp;修改成功：
              {modifyTimeResult.sucUpdate || 0} &nbsp;&nbsp;修改失败：
              {modifyTimeResult.failUpdate || 0}
            </p>
            <p style={{ marginTop: '20px', color: 'red' }}>上课方式为在线A和在线B时，不能把上课时间改为历史时间。</p>
          </p>
        </p>
      ),
      okText: '知道了',
    });
  };

  onRowSelectChange = (keys, rows) => {
    this.setState({
      selectedRowKeys: keys,
      selectedRows: rows,
    });
  };

  isDisabledRow = (row) => {
    /**
     * 【现逻辑】 ①当课程上课方式为为在线B/在线A时， 上课时间不允许修改为历史时间【即不得早于提交时间】；
     *  ②对于符合条件的课程修改成功，不符合条件课程修改失败；
     */
    return false;
    const isBefore = moment(`${row.courseDate} ${row.courseTime}`).isBefore(new Date());
    const isDisabledType = row.teachingType === teachingTypeEnum.LIVE || row.teachingType === teachingTypeEnum.RECORDED;

    return isBefore && isDisabledType;
  };

  render() {
    const { route } = this.props;
    const {
      loading,
      list,
      totalRecords,
      currentPage,
      pageSize,
      selectedRowKeys,
      showModifyModal,
      modifyCourseList,
      confirmLoading,
      showModifyTeachingTypeModal,
      selectAllCheckType,
    } = this.state;

    const modifyColumns = [
      {
        title: '原上课日期',
        dataIndex: 'onClassTime',
        key: 'onClassTime',
        width: '220px',
        render: (text, row) => {
          return (
            <span>
              {row.courseDate} {row.courseTime} : {row.courseEndTime}
              <span style={{ marginLeft: '10px' }}>——&gt;</span>
            </span>
          );
        },
      },
      {
        title: '上课日期',
        dataIndex: 'newStartDate',
        key: 'newStartDate',
        render: (text, row) => {
          return (
            <DatePicker
              size="large"
              disabled={row.disabled}
              value={row.newStartDate}
              placeholder={row.courseDate}
              onChange={(e) => {
                row.newStartDate = e;
                this.setState({
                  modifyCourseList: [...modifyCourseList],
                });
              }}
            />
          );
        },
      },
      {
        title: '开始时间',
        dataIndex: 'newStartTime',
        key: 'newStartTime',
        render: (text, row) => {
          return (
            <TimePicker
              size="large"
              format="HH:mm"
              value={row.newStartTime}
              disabled={row.disabled}
              placeholder={row.courseTime}
              onChange={(e) => {
                row.newStartTime = e;
                this.setState({
                  modifyCourseList: [...modifyCourseList],
                });
              }}
            />
          );
        },
      },
      {
        title: '本讲时长',
        dataIndex: 'courseLength',
        key: 'courseLength',
        width: '150px',
        render: (text, row) => {
          const totalMinutes = row.courseHours * row.courseMinutes;
          let hours = Math.floor(totalMinutes / 60);
          let minutes = parseFloat((totalMinutes % 60).toFixed(2));
          hours = hours > 0 ? `${hours}小时` : '';
          minutes = minutes > 0 ? `${minutes}分钟` : '';

          return hours + minutes;
        },
      },
      {
        title: '涉及课程数',
        dataIndex: 'courseCounts',
        key: 'courseCounts',
        width: '100px',
        render: (text, row) => {
          return `共${row.courseList && row.courseList.length}个课程`;
        },
      },
    ];

    return (
      <Page
        name={route.name}
        route={route}
        headerToolBar={
          <div>
            <Button style={{ marginRight: '20px' }} onClick={this.onModifyTeachType}>
              修改上课方式
            </Button>
            <Button disabled={selectAllCheckType} style={{ marginRight: '20px' }} onClick={this.onDelete}>
              批量删课
            </Button>
            <Button disabled={selectAllCheckType} onClick={this.onModify}>
              批量修改上课时间
            </Button>
          </div>
        }
        subHeader={
          <div className={styles.wrap}>
            <Search onSearch={this.onSearch} />
            <Checkbox checked={selectAllCheckType} style={{ marginBottom: '20px' }} onChange={this.toggleSelectAll}>
              全选搜索结果(共{totalRecords}条)
            </Checkbox>
            <List
              dataSource={list}
              columns={columns.concat({
                title: '操作',
                dataIndex: 'operation',
                key: 'operation',
                fixed: 'right',
                width: 100,
                render: (text, row) => {
                  return (
                    <div>
                      <TextLink onClick={() => this.toClassDetail(row)}>小班详情</TextLink>
                      <TextLink onClick={() => this.toClassCourseDetail(row)}>小班课程</TextLink>
                    </div>
                  );
                },
              })}
              pagination={{
                current: currentPage,
                total: totalRecords,
                pageSize,
                onChange: (e) => {
                  console.log({ e, state: this.state });
                },
              }}
              onChange={this.onPaginationChange}
              rowSelection={{
                selectedRowKeys,
                onChange: (keys, rows) => {
                  this.onRowSelectChange(keys, rows);
                },
                getCheckboxProps: (record) => ({
                  disabled: record.courseStatus !== 'NEW' || this.state.selectAllCheckType,
                }),
              }}
              bordered={true}
              className={styles.customList}
            />

            <Loading loading={loading} />
          </div>
        }
      >
        <Modal
          title="批量修改上课时间"
          okText="确认修改"
          visible={showModifyModal}
          confirmLoading={confirmLoading}
          width="800px"
          onCancel={() => {
            this.setState({
              showModifyModal: false,
            });
          }}
          onOk={this.onSaveModifyCourseTime}
        >
          <Table columns={modifyColumns} dataSource={modifyCourseList} pagination={false} />
          <p style={{ color: 'red', margin: '30px 12px 10px' }}>
            <p>调课后可能会造成排课冲突，为避免影响老师考勤，修改后请前往课表或冲突预警报表检查。</p>
            <p>课程为在线A或在线B时，不能把上课时间修改为历史时间。</p>
          </p>
        </Modal>
        <Modal
          title="修改上课方式"
          okText="确认"
          visible={showModifyTeachingTypeModal}
          width="600px"
          onCancel={() => {
            this.setState({
              showModifyTeachingTypeModal: false,
            });
          }}
          onOk={this.onSaveModifyTeachingType}
        >
          <RadioGroup
            style={{ margin: '30px 12px 0 12px' }}
            value={this.state.teachingType}
            onChange={(event) => {
              if (this.state.selectAllCheckType) {
                // 如果是勾选了全选的逻辑
                this.setState({ newTeachingType: event.target.value });
              }
              this.setState({ teachingType: event.target.value });
            }}
          >
            {teachingTypeDict.map((item) => (
              <Radio key={item.val} value={item.val}>
                {item.text}
              </Radio>
            ))}
          </RadioGroup>
          <p style={{ color: '#ccc', margin: '30px 12px 10px' }}>
            注：①仅支持修改状态为【未上课】课程的上课方式；②把在线A课改为面授课后，请到课程详情选择教室；③本地网课上课方式不支持修改
          </p>
        </Modal>
      </Page>
    );
  }
}

export default BatchModifyCourse;
