import React from 'react';
import { connect } from 'dva';
import Block from 'COMPONENT/Block';
import Modal from 'COMPONENT/Modal';
import { Row, Col } from 'antd';
import Select from 'COMPONENT/Select';
import Input from 'COMPONENT/Input';
import Loaders from '../ModifyCourse/loaders';
import TimePicker from 'COMPONENT/TimePicker';
// import Calendar from 'PAGE/signUpOnline/ExamSiteMgmt/ExamSiteModal/Cal';
import Calendar from './Cal';
import moment from 'moment';
import styles from './index.less';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import InputNumber from 'COMPONENT/InputNumber';
import { newGetClassroomByCampu } from 'SERVICE/common';
import * as NP from 'number-precision';
import { productGroupEnum, teachingTypeEnum, teachingTypeDict } from 'CONST/dicts/course';

class AddNewCourse extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      teacherDict: null,
      date: [],
      courses: [], // 课表
      courseTime: [null], // 开始时间, 默认有一个
      classRoomsDict: [],
      courseHours: 0, // 课时小时数
      courseMinutes: 0, // 课时分钟数
    };

    this.init = this.init.bind(this);
    this.initMiniClassId = this.initMiniClassId.bind(this);
    this.initDict = this.initDict.bind(this);
    this.updateCurriculum = this.updateCurriculum.bind(this);
    this.updateClassroomId = this.updateClassroomId.bind(this);
    this.updateTeacherId = this.updateTeacherId.bind(this);
    this.updateEveryCourseClassNum = this.updateEveryCourseClassNum.bind(this);
    this.batchAddMiniClassCourse = this.batchAddMiniClassCourse.bind(this);
    this.getMiniClassCourseTimeAnalyze = this.getMiniClassCourseTimeAnalyze.bind(this);
    this.pushCourses = this.pushCourses.bind(this);
    this.spliceCourses = this.spliceCourses.bind(this);
    this.getDefaultTeachingType = this.getDefaultTeachingType.bind(this);
  }

  componentDidMount() {
    this.init();
  }

  componentWillUnmount() {
    this.props.dispatch({
      type: 'addNewCourse/resetState',
    });
  }

  initMiniClassId() {
    const { dispatch } = this.props;
    dispatch({
      type: 'addNewCourse/updateMiniClassId',
      payload: {
        miniClassId: this.props.oldPageParams.miniClassId,
      },
    });
  }

  initMiniClassInfo() {
    return new Promise((resolve, reject) => {
      this.props.dispatch({
        type: 'addNewCourse/getMiniClass',
        payload: {
          callback: (classTime) => {
            this.setState({ courseTime: [classTime] });
            resolve();
          },
          miniClassId: this.props.oldPageParams.miniClassId,
        },
      });
    });
  }

  /**
   * 设置时间-更新已选日期
   * */
  updateDate = (newDate) => {
    let { date } = this.state;
    if (date.length === 0) {
      date.push(newDate);
    } else {
      for (let i = 0; i < date.length; i++) {
        if (moment(newDate).isBefore(date[i]) || moment(newDate).isSame(date[i])) {
          date.splice(i, 0, newDate);
          break;
        } else if (moment(newDate).isAfter(date[date.length - 1])) {
          date.push(newDate);
          break;
        }
      }
    }
    this.setState({ date });
    this.updateCurriculum(newDate);
  };

  // ①产品组的值为【本地网课/直播】时，默认值为直播；
  // ②产品组的值为【答疑】时，默认值为答疑；
  // ③产品组的值为【录播】时，默认值为录播；
  // ②产品组的值为空或者其它，默认值为面授；
  getDefaultTeachingType = () => {
    const { productGroupId } = this.props.oldPageParams;
    if (productGroupId === productGroupEnum.ONLINE_CLASS || productGroupId === productGroupEnum.LIVE_CLASS)
      return teachingTypeEnum.LIVE;
    if (productGroupId === productGroupEnum.FAQ_CLASS) return teachingTypeEnum.ANSWER;
    if (productGroupId === productGroupEnum.PRODUCT_GROUP_HL) return teachingTypeEnum.AI;
    return teachingTypeEnum.FACE_TO_FACE;
  };

  pushCourses(courses, date, time, courseTime) {
    console.log('测试');
    const { teacherId, classRoomId } = this.props;
    for (let i = 0; i < time; i++) {
      courses.push({
        courseDate: date,
        courseName: null,
        teacherId,
        classRoomId,
        courseTime: courseTime[i],
        teachingType: this.getDefaultTeachingType(),
      });
    }
  }

  spliceCourses(courses, date, time, courseTime, index) {
    const { teacherId, classRoomId } = this.props;
    for (let i = 0; i < time; i++) {
      courses.splice(index, 0, {
        courseDate: date,
        courseName: null,
        teacherId,
        classRoomId,
        courseTime: courseTime[i],
        teachingType: this.getDefaultTeachingType(),
      });
    }
  }

  /**
   * 更新课表
   */
  updateCurriculum(date) {
    let { courseTime, courses } = this.state;
    if (courses.length === 0) {
      this.pushCourses(courses, date, courseTime.length, courseTime);
    } else {
      for (let i = 0; i < courses.length; i++) {
        if (moment(date).isBefore(courses[i].courseDate) || moment(date).isSame(courses[i].courseDate)) {
          this.spliceCourses(courses, date, courseTime.length, courseTime, i);
          break;
        } else if (moment(date).isAfter(courses[courses.length - 1].courseDate)) {
          this.pushCourses(courses, date, courseTime.length, courseTime);
          break;
        }
      }
    }
    this.setState({ courses });
  }

  /**
   * 获取课时信息
   */
  getMiniClassCourseTimeAnalyze() {
    return new Promise((resolve, reject) => {
      this.props.dispatch({
        type: 'addNewCourse/getMiniClassCourseTimeAnalyzeApi',
        payload: {
          miniClassId: this.props.oldPageParams.miniClassId,
          callback() {
            resolve();
          },
        },
      });
    });
  }

  async initDict() {
    const { campusId, isOnline } = this.props.oldPageParams;

    try {
      const data = await newGetClassroomByCampu({ campusId, isOnline });
      const classroom = data.data.map((campus) => {
        return {
          text: campus.name,
          val: campus.value,
        };
      });
      this.setState({
        classRoomsDict: classroom,
      });
    } catch (e) {
      showMessage('获取教室失败');
    }
  }

  async init() {
    this.setState({ loading: true });
    try {
      await this.initMiniClassInfo();
      await this.initDict();
      await this.getMiniClassCourseTimeAnalyze();
      const { subjectId, gradeId } = this.props;
      const {
        /*isOnline, */ campusId /*, productVersionId: yearId, productQuarterId: quarterId, blBranchId, classLevelId*/,
      } = this.props.oldPageParams;
      let teacherDict = [];
      //if (isOnline === "true") {
      teacherDict = await Loaders.loadSmallTeacherApi({ subjectId, gradeId, campusId });
      //} else {
      //teacherDict = await Loaders.loadSmallClassQuarterTeacherList({ subjectId, gradeId, blCampusId: campusId, yearId, quarterId, blBranchId, classLevelId });
      //}
      teacherDict.unshift({ val: null, text: '其他' });
      this.setState({ teacherDict });
    } catch (e) {
      this.setState({ loading: false });
    }
    this.setState({ loading: false });
    const { everyCourseClassNatureMinute } = this.props;
    this.updateEveryCourseClassNum(parseInt(everyCourseClassNatureMinute / 60), everyCourseClassNatureMinute % 60);
  }

  // 更新老师id
  updateTeacherId(teacherId) {
    const { teacherId: oldTeacherId } = this.props;
    const { courses } = this.state;
    this.props.dispatch({
      type: 'addNewCourse/updateTeacherId',
      payload: {
        teacherId,
      },
    });
    let newCourses = courses.map((course, i) => {
      if (oldTeacherId === course.teacherId) {
        return Object.assign({}, course, { teacherId });
      } else {
        return course;
      }
    });
    this.setState({ courses: newCourses });
  }

  // 更新教室id
  updateClassroomId(classRoomId) {
    const { classRoomId: oldClassroomId } = this.props;
    const { courses } = this.state;
    this.props.dispatch({
      type: 'addNewCourse/updateClassroomId',
      payload: {
        classRoomId,
      },
    });
    let newCourses = courses.map((course, i) => {
      if (oldClassroomId === course.classRoomId) {
        return Object.assign({}, course, { classRoomId });
      } else {
        return course;
      }
    });
    this.setState({ courses: newCourses });
  }

  // 更新每讲课时
  updateEveryCourseClassNum(courseHours = 0, courseMinutes = 0) {
    this.setState({
      courseHours,
      courseMinutes,
    });
    let newMinutes = NP.plus(courseHours * 60, courseMinutes); // 分钟数
    let courseNum = (newMinutes / this.props.classTimeLength).toFixed(2); // 课时数
    this.props.dispatch({
      type: 'addNewCourse/updateEveryCourseClassNum',
      payload: {
        everyCourseClassNum: courseNum,
      },
    });
  }

  // 批量新增小班课程
  batchAddMiniClassCourse() {
    const { courses } = this.state;
    const { NOT_ARRANG_COURSE_HOURS, everyCourseClassNum } = this.props;
    const error = (1 / everyCourseClassNum).toFixed(2); // 允许误差范围
    if (courses.length * everyCourseClassNum > NOT_ARRANG_COURSE_HOURS) {
      showMessage('超过总小时，请删除某些课程!');
      return;
    } else if (
      courses.length * everyCourseClassNum < NOT_ARRANG_COURSE_HOURS &&
      (NOT_ARRANG_COURSE_HOURS - courses.length * everyCourseClassNum).toFixed(2) > error
    ) {
      showMessage('还有未排小时，请先新增课程!');
      return;
    }

    if (!this.validateCourseTime()) {
      showMessage('““在线A”或“在线B”的课程上课时间不能选择历史时间~');
      return;
    }

    this.setState({ loading: true });
    return new Promise((resolve, reject) => {
      this.props.dispatch({
        type: 'addNewCourse/addMiniClassCourse',
        payload: {
          callback: (status) => {
            if (status === true) {
              this.props.onClose();
              this.props.dispatch({
                type: 'main/refreshCurrentTab',
              });
            }
            resolve();
          },
          courses,
          miniClassId: this.props.oldPageParams.miniClassId,
        },
      });
      this.setState({ loading: false });
    });
  }

  // 本地网课不校验
  validateCourseTime = () => {
    const { productGroupId } = this.props.oldPageParams;
    if (productGroupId === productGroupEnum.ONLINE_CLASS) return true;

    const { courses } = this.state;

    for (let i = 0; i < courses.length; i++) {
      const courseItem = courses[i];
      const isBefore = moment(`${courseItem.courseDate} ${courseItem.courseTime}`).isBefore(new Date());
      if (
        (courseItem.teachingType === teachingTypeEnum.LIVE || courseItem.teachingType === teachingTypeEnum.RECORDED) &&
        isBefore
      ) {
        return false;
      }
    }

    return true;
  };

  hasValue = (val) => val !== undefined && val !== null;

  render() {
    const { loading, teacherDict, date, courses, courseTime, courseHours, courseMinutes } = this.state;
    const {
      name,
      classTimeLength,
      teacherId,
      classRoomId,
      everyCourseClassNum,
      NOT_ARRANG_COURSE_HOURS,
      TOTAL_CLASS_HOURS,
      startDate,
    } = this.props;
    const productGroupName = this.props.oldPageParams.productGroupName;
    const productGroupId = this.props.oldPageParams.productGroupId;

    return (
      <Modal
        title="新增课程"
        onCancel={this.props.onClose}
        loading={loading}
        onOk={this.batchAddMiniClassCourse}
        footerTipText={`已有讲数: ${courses.length}讲`}
        width={1200}
      >
        <Block title="基础信息">
          <Row className={styles.infoRow}>
            <Col span={12}>
              班级名称<span className={styles.text}>{name}</span>
            </Col>
            <Col span={12}>
              产品组<span className={styles.text}>{productGroupName}</span>
            </Col>
          </Row>
          <Row className={styles.infoRow}>
            <Col span={8}>
              <p>
                总小时数
                <span className={styles.text}>
                  {this.hasValue(TOTAL_CLASS_HOURS) ? formatCourseToHour(TOTAL_CLASS_HOURS, classTimeLength) : ''}
                </span>
              </p>
            </Col>
            <Col span={8}>
              <p>
                未排小时
                <span className={styles.text}>
                  {this.hasValue(NOT_ARRANG_COURSE_HOURS)
                    ? formatCourseToHour(NOT_ARRANG_COURSE_HOURS, classTimeLength)
                    : ''}
                </span>
              </p>
            </Col>
            {/* <Col span={8}>
              <p>
                课时时长<span className={styles.text}>{`${classTimeLength}分钟`}</span>
              </p>
            </Col> */}
          </Row>
          <Row className={styles.infoRow}>
            <Col span={8}>
              <div className={styles.label}>
                授课老师<span className={styles.red}>*</span>
                <Select
                  className={styles.select}
                  value={teacherId}
                  dict={teacherDict}
                  allowClear={false}
                  onChange={this.updateTeacherId}
                  showSearch={true}
                />
              </div>
            </Col>
            <Col span={8}>
              <div className={styles.label}>
                授课教室<span className={styles.red}>*</span>
                <Select
                  className={styles.select}
                  value={classRoomId}
                  dict={this.state.classRoomsDict}
                  allowClear={false}
                  onChange={this.updateClassroomId}
                />
              </div>
            </Col>
            <Col span={8}>
              <div className={styles.label}>
                <span className={styles.title}>时长</span>
                <span className={styles.red}>*</span>
                <InputNumber
                  className={styles.smallInput}
                  value={courseHours}
                  onChange={(val) => this.updateEveryCourseClassNum(val, courseMinutes)}
                  step={1}
                  min={0}
                  max={17}
                  prision={0}
                  defaultValue={2}
                />
                <span className={styles.title}>小时</span>
                <InputNumber
                  className={styles.smallInput}
                  value={courseMinutes}
                  onChange={(val) => this.updateEveryCourseClassNum(courseHours, val)}
                  step={1}
                  min={0}
                  max={59}
                  prision={0}
                  defaultValue={0}
                />
                <span className={styles.title}>分钟</span>
                {/* <p className={styles.tips}>(1讲={formatCourseToHour(1, classTimeLength)}小时)</p> */}
              </div>
            </Col>
          </Row>
        </Block>
        <Block title="设置开始时间" style={{ marginTop: '20px' }}>
          {courseTime.map((time, index) => {
            return (
              <Row className={styles.startTimeInput} key={index}>
                <Col style={{ paddingRight: '14px', color: '#999' }}>开始时间</Col>
                <Col>
                  <TimePicker
                    className={styles.TimePicker}
                    value={time}
                    onChange={(value) => {
                      let newCourses = courses.map((course) => {
                        if (time === course.courseTime) {
                          return Object.assign({}, course, { courseTime: value });
                        } else {
                          return course;
                        }
                      });
                      this.setState({ courses: newCourses });
                      let newStartTime = courseTime.map((curTime, i) => {
                        if (i === index) {
                          return value;
                        } else {
                          return curTime;
                        }
                      });
                      this.setState({ courseTime: newStartTime });
                    }}
                  />
                </Col>
                <Col style={{ paddingLeft: '14px' }}>
                  {index === 0 ? (
                    <div>
                      <span
                        onClick={() => {
                          let { courseTime } = this.state;
                          courseTime.push(null);
                          this.setState({ courseTime });
                        }}
                      >
                        新增
                      </span>
                    </div>
                  ) : (
                    <div>
                      <span
                        onClick={() => {
                          let { courseTime } = this.state;
                          courseTime.push(null);
                          this.setState({ courseTime });
                        }}
                      >
                        新增
                      </span>
                      <span
                        onClick={() => {
                          let { courseTime } = this.state;
                          courseTime.splice(index, 1);
                          this.setState({ courseTime });
                        }}
                      >
                        删除
                      </span>
                    </div>
                  )}
                </Col>
              </Row>
            );
          })}
          <Calendar date={date} updateDate={this.updateDate} startDate={startDate} />
          <Row className={styles.setTimeRowHeader}>
            <Col span={3} className={styles.setTimeRowHeaderText}>
              日期
            </Col>
            <Col span={4} className={styles.setTimeRowHeaderText}>
              课程名称
            </Col>
            <Col span={4} className={styles.setTimeRowHeaderText}>
              开始时间<span>*</span>
            </Col>
            <Col span={4} className={styles.setTimeRowHeaderText}>
              老师<span>*</span>
            </Col>
            <Col span={4} className={styles.setTimeRowHeaderText}>
              教室<span>*</span>
            </Col>
            <Col span={3} className={styles.setTimeRowHeaderText}>
              上课方式
            </Col>
            <Col span={2} className={styles.setTimeRowHeaderText} style={{ paddingLeft: '30px' }}>
              操作
            </Col>
          </Row>
          {courses.map((course, index) => {
            const { classRoomId, courseDate, courseName, teacherId, courseTime, teachingType } = course;
            return (
              <Row key={index} className={styles.setTimeRow}>
                <Col span={3} className={styles.setTimeRowLabel}>
                  {courseDate}
                </Col>
                <Col span={4} className={styles.setTimeRowLabel}>
                  <Input
                    className={styles.setTimeRowPicker}
                    value={courseName}
                    onChange={(e) => {
                      let courseName = e.currentTarget.value;
                      let newCourses = courses.map((course, i) => {
                        if (index === i) {
                          return Object.assign({}, course, { courseName });
                        } else {
                          return course;
                        }
                      });
                      this.setState({ courses: newCourses });
                    }}
                  />
                </Col>
                <Col span={4} className={styles.setTimeRowLabel}>
                  <TimePicker
                    value={courseTime}
                    className={styles.setTimeRowPicker}
                    onChange={(courseTime) => {
                      let newCourses = courses.map((course, i) => {
                        if (index === i) {
                          return Object.assign({}, course, { courseTime });
                        } else {
                          return course;
                        }
                      });
                      this.setState({ courses: newCourses });
                    }}
                  />
                </Col>
                <Col span={4} className={styles.setTimeRowLabel}>
                  <Select
                    className={styles.setTimeRowPicker}
                    dict={teacherDict}
                    value={teacherId}
                    onChange={(teacherId) => {
                      let newCourses = courses.map((course, i) => {
                        if (index === i) {
                          return Object.assign({}, course, { teacherId });
                        } else {
                          return course;
                        }
                      });
                      this.setState({ courses: newCourses });
                    }}
                  />
                </Col>
                <Col span={4} className={styles.setTimeRowLabel}>
                  <Select
                    className={styles.setTimeRowPicker}
                    dict={this.state.classRoomsDict}
                    value={classRoomId}
                    onChange={(classRoomId) => {
                      let newCourses = courses.map((course, i) => {
                        if (index === i) {
                          return Object.assign({}, course, { classRoomId });
                        } else {
                          return course;
                        }
                      });
                      this.setState({ courses: newCourses });
                    }}
                  />
                </Col>
                <Col span={3} className={styles.setTimeRowLabel}>
                  <Select
                    className={styles.setTimeRowPicker}
                    disabled={
                      productGroupId === productGroupEnum.ONLINE_CLASS ||
                      productGroupId === productGroupEnum.PRODUCT_GROUP_HL
                    }
                    dict={teachingTypeDict}
                    value={teachingType}
                    onChange={(teachingType) => {
                      let newCourses = courses.map((course, i) => {
                        if (index === i) {
                          return Object.assign({}, course, { teachingType });
                        } else {
                          return course;
                        }
                      });
                      this.setState({ courses: newCourses });
                    }}
                  />
                </Col>
                <Col span={2} className={styles.setTimeRowLabel}>
                  <span
                    className={styles.delete}
                    onClick={() => {
                      let { date, courses } = this.state;
                      courses = courses.filter((course, i) => {
                        return i !== index;
                      });
                      date = date.filter((d, i) => {
                        return i !== index;
                      });
                      this.setState({ courses, date });
                    }}
                  >
                    删除
                  </span>
                </Col>
              </Row>
            );
          })}
        </Block>
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  const {
    classRoomDict,
    teacherDict,
    name,
    alreadyTotalClassHours,
    arrangedHours,
    classTimeLength,
    teacherId,
    classRoomId,
    everyCourseClassNum,
    subjectId,
    campusId,
    gradeId,
    NOT_ARRANG_COURSE_HOURS,
    TOTAL_CLASS_HOURS,
    classTime,
    startDate,
    everyCourseClassNatureMinute,
  } = state.addNewCourse;
  return {
    classRoomDict,
    teacherDict,
    name,
    alreadyTotalClassHours,
    arrangedHours,
    classTimeLength,
    teacherId,
    classRoomId,
    everyCourseClassNum,
    subjectId,
    campusId,
    gradeId,
    everyCourseClassNatureMinute,
    classRoomsDict: state.dict.classRoomsByCampusId,
    NOT_ARRANG_COURSE_HOURS,
    TOTAL_CLASS_HOURS,
    classTime,
    startDate,
  };
};

export default connect(mapStateToProps)(AddNewCourse);
