import { findMiniClassById, batchAddMiniClassCourse, getMiniClassCourseTimeAnalyze } from 'SERVICE/course';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import Loaders from '../ModifyCourse/loaders';

export default {
  namespace: 'addNewCourse',
  state: {
    miniClassId: null, // 小班 id
    name: null,
    alreadyTotalClassHours: null,
    arrangedHours: null,
    classTimeLength: null,
    teacherId: null,
    classRoomId: null,
    everyCourseClassNum: null,
    campusId: null,
    subjectId: null,
    studyManegerId: null,

    teacherDict: null, // 老师字典
    classRoomDict: null, // 教室字典

    NOT_ARRANG_COURSE_HOURS: null, // 未排课时
    TOTAL_CLASS_HOURS: null, // 总课时

    startDate: null, // 开课时间
  },
  reducers: {
    // 获取小班id
    updateMiniClassId(state, { payload: { miniClassId } }) {
      return {
        ...state,
        miniClassId,
      };
    },
    // 获取小班信息
    getMiniClassInfo(state, { payload }) {
      return {
        ...state,
        ...payload,
      };
    },
    // 更新老师id
    updateTeacherId(state, { payload }) {
      return {
        ...state,
        ...payload,
      };
    },
    // 更新教室id
    updateClassroomId(state, { payload }) {
      return {
        ...state,
        ...payload,
      };
    },
    // 更新每讲课时
    updateEveryCourseClassNum(state, { payload }) {
      return {
        ...state,
        ...payload,
      };
    },
    getCourseTimeAnalyze(state, { payload }) {
      return {
        ...state,
        ...payload,
      };
    },
    resetState(state) {
      return {
        miniClassId: null, // 小班 id
        name: null,
        alreadyTotalClassHours: null,
        arrangedHours: null,
        classTimeLength: null,
        teacherId: null,
        classRoomId: null,
        everyCourseClassNum: null,
        campusId: null,
        subjectId: null,
        studyManegerId: null,

        teacherDict: null, // 老师字典
        classRoomDict: null, // 教室字典

        NOT_ARRANG_COURSE_HOURS: null, // 未排课时
        TOTAL_CLASS_HOURS: null, // 总课时
      };
    },
  },
  effects: {
    /**
     * 获取小班信息
     */
    *getMiniClass({ payload: { callback, miniClassId } }, { select, put, call }) {
      const rsp = yield findMiniClassById({ miniClassId }).catch((error) => {
        showMessage(error.text, 'error');
      });
      let courseTime = '';
      if (rsp) {
        const {
          name,
          alreadyTotalClassHours,
          arrangedHours,
          classTimeLength,
          teacherId,
          classroomId,
          everyCourseClassNum,
          blCampusId,
          subjectId,
          gradeId,
          studyManegerId,
          classTime,
          startDate,
          everyCourseClassNatureMinute,
        } = rsp;
        yield put({
          type: 'getMiniClassInfo',
          payload: {
            name,
            alreadyTotalClassHours,
            arrangedHours,
            classTimeLength,
            teacherId,
            classRoomId: classroomId,
            everyCourseClassNum,
            campusId: blCampusId,
            subjectId,
            gradeId,
            studyManegerId,
            classTime,
            startDate,
            everyCourseClassNatureMinute,
          },
        });
        courseTime = classTime;
      }
      if (callback) callback(courseTime);
    },
    *addMiniClassCourse({ payload: { callback, courses, miniClassId } }, { select, put, call }) {
      let apiParams = yield select((state) => {
        const { studyManegerId, subjectId, gradeId, name, classTimeLength, everyCourseClassNum } =
          state['addNewCourse'];
        return {
          miniClassCourses: courses.map((course) => {
            return Object.assign({}, course, {
              studyManegerId,
              subjectId,
              gradeId,
              miniClassId,
              miniClassName: name,
              courseMinutes: classTimeLength,
              courseHours: everyCourseClassNum,
            });
          }),
        };
      });
      const rsp = yield batchAddMiniClassCourse(apiParams, true).catch((error) => {
        showMessage(error.text, 'error');
      });
      let status = false;
      if (rsp && rsp.resultCode === 0) {
        showMessage('新增成功!', 'success');
        status = true;
      }
      if (callback) callback(status);
    },
    *getMiniClassCourseTimeAnalyzeApi({ payload: { callback, miniClassId } }, { select, put, call }) {
      let rsp = yield getMiniClassCourseTimeAnalyze({ miniClassId }).catch((error) => {
        showMessage(error.text, 'error');
      });
      if (rsp) {
        const { NOT_ARRANG_COURSE_HOURS, TOTAL_CLASS_HOURS } = rsp;
        yield put({
          type: 'getCourseTimeAnalyze',
          payload: {
            NOT_ARRANG_COURSE_HOURS,
            TOTAL_CLASS_HOURS,
          },
        });
      }
      if (callback) callback();
    },
  },
};
