import React from 'react';
import { connect } from 'dva';
import Modal from 'COMPONENT/Modal';
import { Row, Col } from 'antd';
import FormItem from 'COMPONENT/FormItem';
import Input from 'COMPONENT/Input';
import InputNumber from 'COMPONENT/InputNumber';
import Select from 'COMPONENT/Select';
import TimePicker from 'COMPONENT/TimePicker';
import DatePicker from 'COMPONENT/DatePicker';
import ArrangeTimeModal from '../ArrangeTimeModal';
import styles from './index.less';
import Loaders from './loaders';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import { newGetClassroomByCampu } from 'SERVICE/common';
import { getMiniClassCourseTimeAnalyze } from 'SERVICE/course';
import { formatHourToCourse, formatCourseToHour } from 'UTIL/common';
import moment from 'moment';
import { teachingTypeEnum, teachingTypeDict } from 'CONST/dicts/course';

class ModifyCourse extends React.Component {
  constructor(props) {
    super(props);
    const { courses = [], courseMinutes } = props.oldPageParams;
    const [first] = courses;
    this.state = {
      loading: false,
      isArrangeTimeModalVisible: false,
      courseIds: courses.map((course) => {
        return course.miniClassCourseId;
      }),
      courseName: first ? first.courseName : '',
      courseDate: first ? first.courseDate : '',
      courseTime: first ? first.courseTime : '',
      courseHour: first ? first.courseHours : '',
      courseMinutes: courseMinutes,
      classRoomId: first ? first.classroomId : '',
      teacherId: first ? first.teacherId : '',
      teachingType: first ? this.getValueByLabel(first.teachingType, teachingTypeDict) : teachingTypeEnum.FACE_TO_FACE,

      hours: 2,
      minutes: 0,
      teacherDict: null,
      classRoomsDict: [],
      arrangeTimeModalParams: {},
    };

    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.onCourseNameChange = this.onCourseNameChange.bind(this);
    this.updateState = this.updateState.bind(this);
    this.onCourseHourChange = this.onCourseHourChange.bind(this);
    this.onCourseMinutesChange = this.onCourseMinutesChange.bind(this);
    this.onCourseTimeChange = this.onCourseTimeChange.bind(this);
    this.onCourseDateChange = this.onCourseDateChange.bind(this);
    this.onClassRoomIdChange = this.onClassRoomIdChange.bind(this);
    this.onTeacherIdChange = this.onTeacherIdChange.bind(this);
    this.onTeacherTypeChange = this.onTeacherTypeChange.bind(this);
    this.onOk = this.onOk.bind(this);
  }

  componentDidMount() {
    this.init();
  }

  async init() {
    const { courseNatureMinutes } = this.props.oldPageParams;
    this.setState({
      loading: true,
      hours: parseInt(courseNatureMinutes / 60),
      minutes: parseInt(courseNatureMinutes % 60),
    });

    const {
      /*isOnline, */ subjectId,
      gradeId,
      campusId /*productVersionId: yearId, productQuarterId: quarterId, blBranchId, classLevelId*/,
    } = this.props.oldPageParams;
    try {
      let teacherDict = [];
      //if (isOnline === "true") {
      teacherDict = await Loaders.loadSmallTeacherApi({ subjectId, gradeId, campusId });
      //} else {
      // teacherDict = await Loaders.loadSmallClassQuarterTeacherList({ subjectId, gradeId, blCampusId: campusId, yearId, quarterId, blBranchId, classLevelId });
      //}
      teacherDict.unshift({ val: null, text: '其他' });
      this.setState({ teacherDict });
      await this.initDict();
      this.setState({ loading: false });
    } catch (e) {
      console.log(e);
      this.setState({ loading: false });
    }
  }

  getValueByLabel = (text, dict) => {
    const item = dict.find((item) => item.text === text);

    return item ? item.val : dict[0].val;
  };

  async initDict() {
    const { campusId, isOnline } = this.props.oldPageParams;

    try {
      const data = await newGetClassroomByCampu({ campusId, isOnline });
      const classroom = data.data.map((campus) => {
        return {
          text: campus.name,
          val: campus.value,
        };
      });
      this.setState({
        classRoomsDict: classroom,
      });
    } catch (e) {
      showMessage('获取教室失败');
    }
  }

  onCourseNameChange(e) {
    let courseName = e.currentTarget.value;
    this.setState({ courseName });
  }

  onCourseMinutesChange(e) {
    let courseMinutes = e.currentTarget.value;
    this.setState({ courseMinutes });
  }

  onCourseHourChange(hours = 0, minutes = 0) {
    let totalHours = hours + parseFloat((minutes / 60).toFixed(2));
    let courseHour = formatHourToCourse(totalHours, this.state.courseMinutes);
    this.setState({ courseHour, hours, minutes });
  }

  updateState(changedState) {
    const newState = {
      ...this.state,
      ...changedState,
    };
    this.setState(newState);
  }

  onCourseDateChange(courseDate) {
    this.setState({ courseDate });
  }

  onCourseTimeChange(courseTime) {
    this.setState({ courseTime });
  }

  onTeacherIdChange(teacherId) {
    this.setState({ teacherId });
  }

  onTeacherTypeChange(teachingType) {
    this.setState({ teachingType });
  }

  onClassRoomIdChange(classRoomId) {
    this.setState({ classRoomId });
  }

  async onOk() {
    const {
      courseIds,
      courseName,
      courseDate,
      courseTime,
      courseHour,
      courseMinutes,
      teacherId,
      classRoomId,
      hours,
      minutes,
      teachingType,
    } = this.state;
    const params = {
      courseIds,
      courseName,
      courseDate,
      courseTime,
      courseHour,
      courseMinutes,
      teacherId,
      classRoomId,
      teachingType,
    };
    let error = (text) => getWrappedInstance(this.$modal).showOkTips(text);
    if (!courseDate) {
      error('请选择上课日期！');
      return;
    }
    if (!courseTime) {
      error('请选择上课时间！');
      return;
    }
    if (hours == 0 && minutes == 0) {
      error('请填写每讲小时！');
      return;
    }
    if (!courseMinutes) {
      error('请填写课时时长！');
      return;
    }
    if (!classRoomId) {
      error('请选择教室！');
      return;
    }
    if (!this.validateCourseTime()) {
      showMessage('““在线A”或“在线B”的课程上课时间不能选择历史时间~');
      return;
    }

    this.setState({ loading: true });
    try {
      const totalHours = hours + parseFloat((minutes / 60).toFixed(2)); // 小时数
      params.courseHour = formatHourToCourse(totalHours, courseMinutes);
      let resultCode = await Loaders.batchUpdateMiniClassCourseApi(params);
      this.setState({ loading: false });
      if (resultCode === 0) {
        this.getMiniClassCourseTimeAnalyzeInfo();
        showMessage('修改成功！如需同步修改小班详情，请前往【小班管理 - 操作 - 修改】中修改。', 'success');
      } else {
        this.props.onClose();
      }
    } catch (error) {
      this.setState({ loading: false });
    }
  }

  validateCourseTime = () => {
    const { isOnline } = this.props.oldPageParams;
    if (isOnline == 'true') return true;

    const { courseDate, courseTime, teachingType } = this.state;

    const isBefore = moment(`${courseDate} ${courseTime}`).isBefore(new Date());
    if ((teachingType === teachingTypeEnum.LIVE || teachingType === teachingTypeEnum.RECORDED) && isBefore) {
      return false;
    }

    return true;
  };

  onCloseArrangeTimeModal = () => {
    this.setState({ isArrangeTimeModalVisible: false });
  };

  /**
   * 获取课时信息
   */
  getMiniClassCourseTimeAnalyzeInfo = () => {
    getMiniClassCourseTimeAnalyze({
      miniClassId: this.props.oldPageParams.miniClassId,
    })
      .then((result) => {
        if (!result) {
          return;
        }
        const {
          TOTAL_CLASS_HOURS /**总小时数*/,
          ARRANGED_COURSE_HOURS /**已排小时数*/,
          NOT_ARRANG_COURSE_HOURS /**未排小时数*/,
        } = result;
        if (ARRANGED_COURSE_HOURS !== TOTAL_CLASS_HOURS) {
          const { courseMinutes } = this.props.oldPageParams;
          this.setState({
            isArrangeTimeModalVisible: true,
            arrangeTimeModalParams: {
              TOTAL_CLASS_HOURS: formatCourseToHour(TOTAL_CLASS_HOURS, courseMinutes),
              ARRANGED_COURSE_HOURS: formatCourseToHour(ARRANGED_COURSE_HOURS, courseMinutes),
              NOT_ARRANG_COURSE_HOURS: formatCourseToHour(NOT_ARRANG_COURSE_HOURS, courseMinutes),
            },
          });
        } else {
          this.props.dispatch({ type: 'main/refreshCurrentTab' });
        }
      })
      .catch(({ text }) => {
        this.props.dispatch({ type: 'main/refreshCurrentTab' });
      });
  };

  render() {
    const {
      loading,
      courseName,
      courseDate,
      courseTime,
      classRoomId,
      teacherId,
      teachingType,
      teacherDict,
      courseIds,
      hours,
      minutes,
    } = this.state;
    const { courses, isOnline } = this.props.oldPageParams;

    return (
      <div>
        <Modal
          onCancel={this.props.onClose}
          title="修改课程"
          width={750}
          onOk={this.onOk}
          loading={loading}
          ref={(e) => (this.$modal = e)}
        >
          <Row>
            <FormItem name="课程名称" labelWidth={70}>
              <Input
                value={courseName}
                onChange={this.onCourseNameChange}
                disabled={courseIds.length === 1 ? false : true}
              />
            </FormItem>
          </Row>
          <Row>
            <Col span={12}>
              <FormItem required={true} name="上课日期" labelWidth={70}>
                <DatePicker
                  value={courseDate}
                  onChange={this.onCourseDateChange}
                  disabled={courseIds.length === 1 ? false : true}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem required={true} name="上课时间" labelWidth={70}>
                <TimePicker
                  value={courseTime}
                  onChange={this.onCourseTimeChange}
                  disabled={
                    Array.from(
                      new Set(
                        courses.map((course) => {
                          return course.courseTime;
                        }),
                      ),
                    ).length >= 2
                  }
                />
              </FormItem>
            </Col>
          </Row>
          <Row>
            <Col span={12}>
              <FormItem required={true} name="时长" labelWidth={70}>
                <InputNumber
                  value={hours}
                  onChange={(val) => this.onCourseHourChange(val, minutes)}
                  allowClear={false}
                  min={0}
                  max={17}
                  step={1}
                  precision={0}
                  defaultValue={2}
                  style={{ display: 'inline-block', width: '50px' }}
                />
                <span>小时</span>
                <InputNumber
                  value={minutes}
                  onChange={(val) => this.onCourseHourChange(hours, val)}
                  allowClear={false}
                  min={0}
                  max={59}
                  step={1}
                  precision={0}
                  defaultValue={0}
                  style={{ display: 'inline-block', width: '50px' }}
                />
                <span>分钟</span>
              </FormItem>
            </Col>
            {/* <Col span={12}>
              <FormItem
                required={true}
                name="课时时长"
                labelWidth={70}
              >
                <Input
                  value={courseMinutes}
                  onChange={this.onCourseMinutesChange}
                  allowClear={false}
                  disabled={true}
                />
              </FormItem>
            </Col> */}
            <Col span={12}>
              <FormItem required={true} name="教室" labelWidth={70}>
                <Select
                  value={classRoomId}
                  dict={this.state.classRoomsDict}
                  onChange={this.onClassRoomIdChange}
                  allowClear={false}
                />
              </FormItem>
            </Col>
          </Row>
          <Row>
            <Col span={12}>
              <FormItem required={true} name="任课老师" labelWidth={70}>
                <Select
                  value={teacherId}
                  dict={teacherDict}
                  onChange={this.onTeacherIdChange}
                  allowClear={false}
                  showSearch={true}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="上课方式" labelWidth={70} required={true}>
                <Select
                  disabled={isOnline == 'true'}
                  dict={teachingTypeDict}
                  value={teachingType}
                  onChange={this.onTeacherTypeChange}
                  allowClear={false}
                />
              </FormItem>
            </Col>
          </Row>
        </Modal>
        {this.state.isArrangeTimeModalVisible && (
          <ArrangeTimeModal oldPageParams={this.state.arrangeTimeModalParams}></ArrangeTimeModal>
        )}
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    // oldPageParams: state.oldPageParams
    classRoomsDict: state.dict.classRoomsByCampusId,
  };
};

export default connect(mapStateToProps)(ModifyCourse);
