import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import SearchBar from 'COMPONENT/SearchBar';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import List from 'COMPONENT/List';
const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;
import IconButton from 'COMPONENT/IconButton';
import Button from 'COMPONENT/Button';
import Modal from 'COMPONENT/Modal';
import FormItem from 'COMPONENT/FormItem';
import { Row, Col } from 'antd';
import moment from 'moment';
import styles from './index.less';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import Loaders from './loaders';
import {
  loadCommonDict,
  loadBranchDict,
  loadCampusDict,
  getDictTextByVal,
  getDictValByText,
  loadBranchCampusDict,
  loadClassLevelDict,
} from 'UTIL/dict';

const campusTempDict = [{ text: '请先选择分公司', val: '', disabled: true }];

class MiniClassReEnroll extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      branchId: null, // 分公司 id
      campusId: null, // 学习中心 id
      classTypeIds: null, // 班级类型 ids
      courseSeriesId: null, // 产品系列 id
      getTotal: null, //
      gradeId: null, // 年级 id
      page: null,
      productQuarterId: null, // 产品季度 id
      productVersionId: null, // 产品年份 id
      rows: null,
      subjectId: null, // 科目 id
      teacherId: null, // 老师 id
      sidx: null, // 排序字段,可选字段为 满班率 （full_class_rate）退费率（refund_rate） 下季续报率 （re_nq_rate）下下季续报率（re_nnq_rate）
      sord: null, // 排序, 顺序asc 逆序desc

      branchDict: null, // 分公司字典
      campusDict: null, // 学习中心字典
      classTypeDict: null, // 班级类型字典
      courseSeriesDict: null, // 产品系列字典
      gradeDict: null, // 年级字典
      productQuarterDict: null, // 产品季度字典
      productVersionDict: null, // 产品年份字典
      subjectDict: null, // 科目字典
      teacherDict: null, // 老师字典

      dataSource: [], // 列表数据
      pagination: {
        current: 1, // 默认第一页
        pageSize: 20, // 默认10条每页
        total: 0, // 默认总数据数为 0
      }, // 分页器设置

      fullClassRateIsAsc: true,
      refundRateIsAsc: true,
      reNqRateIsAsc: true,
      reNnqRateIsAsc: true,

      defaultValue: {
        // 重置默认参数
        branchId: null, // 分公司 id
        campusId: null, // 学习中心 id
        classTypeIds: null, // 班级类型 ids
        courseSeriesId: null, // 产品系列 id
        getTotal: null, //
        gradeId: null, // 年级 id
        page: null,
        productQuarterId: null, // 产品季度 id
        productVersionId: null, // 产品年份 id
        rows: null,
        subjectId: null, // 科目 id
        teacherId: null, // 老师 id
        sidx: null, // 排序字段,可选字段为 满班率 （full_class_rate）退费率（refund_rate） 下季续报率 （re_nq_rate）下下季续报率（re_nnq_rate）
        sord: null, // 排序, 顺序asc 逆序desc

        isSetSubjectLeader: false,
        subjectLeaders: [], // 学科负责人

        fullClassRate: null, //总满班率
        reNnqRate: null, // 总下下季度续报率
        reNqRate: null, // 总下季度续报率
      },
    };
  }

  componentDidMount() {
    this.initContent();
  }

  initContent = async () => {
    const [
      branchDict,
      campusDict,
      classTypeDict,
      courseSeriesDict,
      gradeDict,
      productQuarterDict,
      productVersionDict,
      subjectDict,
    ] = await Promise.all([
      loadBranchDict(),
      loadBranchCampusDict(),
      loadCommonDict('CLASS_TYPE'),
      loadCommonDict('COURSE_SERIES'),
      loadCommonDict('STUDENT_GRADE'),
      loadCommonDict('PRODUCT_QUARTER'),
      loadCommonDict('PRODUCT_VERSION'),
      loadCommonDict('SUBJECT'),
    ]);

    const teacherDict = await Loaders.getTeacherDictByBrench({
      roleCode: 'TEATCHER',
      organizationId: branchDict[0].val,
    });
    this.setState(
      {
        branchDict,
        campusDict,
        classTypeDict,
        courseSeriesDict,
        gradeDict,
        productQuarterDict,
        productVersionDict,
        subjectDict,
        branchId: branchDict[0].val,
        productVersionId: getDictValByText(productVersionDict, moment().year().toString()),
        teacherDict,
        defaultValue: Object.assign(this.state.defaultValue, {
          branchId: branchDict[0].val,
          productVersionId: getDictValByText(productVersionDict, moment().year().toString()),
        }),
        productQuarterId: productQuarterDict[0].val,
      },
      () => {
        this.search();
      },
    );
  };

  /**
   * 搜索
   */
  search = async () => {
    this.setState({ loading: true });
    const { dataSource, total, fullClassRate, reNnqRate, reNqRate } = await this.loadList();
    this.setState({
      dataSource,
      pagination: Object.assign(this.state.pagination, { total }),
      fullClassRate,
      reNnqRate,
      reNqRate,
    });
    this.setState({ loading: false });
  };

  /**
   * 获取列表数据
   */
  loadList = async () => {
    let {
      branchId,
      campusId,
      productVersionId,
      productQuarterId,
      courseSeriesId,
      classTypeIds,
      gradeId,
      subjectId,
      teacherId,
      sidx,
      sord,
      pagination,
    } = this.state;
    const params = {
      branchId,
      campusId: campusId === null ? campusId : campusId.join(','),
      productVersionId,
      productQuarterId,
      courseSeriesId,
      classTypeIds,
      gradeId: gradeId === null ? gradeId : gradeId.join(','),
      subjectId: subjectId === null ? subjectId : subjectId.join(','),
      teacherId,
      sidx,
      sord,
      page: pagination.current,
      rows: pagination.pageSize,
    };
    const [{ dataSource, total }, { fullClassRate, reNnqRate, reNqRate }] = await Promise.all([
      Loaders.searchMiniClassRenewPage(params),
      Loaders.findMiniClassRenewRateApi(params),
    ]);
    return {
      dataSource,
      total,
      fullClassRate,
      reNnqRate,
      reNqRate,
    };
  };

  /**
   * 重置
   */
  reset = () => {
    const { defaultValue } = this.state;
    this.setState({ ...defaultValue }, () => {
      this.search();
    });
  };

  /**
   * 获取学科负责人
   */
  getSubjectLeader = async () => {
    this.setState({ loading: true });
    const data = await Loaders.searchSubjectLeader();
    this.setState({ subjectLeaders: data });
    this.setState({ loading: false });
  };

  /**
   * 打开学科负责人Modal
   */
  openModal = () => {
    this.getSubjectLeader();
    this.setState({ isSetSubjectLeader: true });
  };

  /**
   *  添加学科负责人
   */
  addSubjectLeader = () => {
    let { subjectLeaders } = this.state;
    subjectLeaders.push({
      id: null,
      subjectId: null,
      subjectName: null,
      teacherId: null,
      teacherName: null,
    });
    this.setState({ subjectLeaders });
  };

  /**
   * 数字转化成百分比
   */
  toPercent = (number) => {
    if (number === '') {
      return '无';
    }
    let str = Number(number * 100).toFixed(1);
    str += '%';
    return str;
  };

  /**
   * 改变老师或者科目
   */
  changeTeacherOrSubject = ({ value, type, index }) => {
    let { subjectLeaders } = this.state;
    subjectLeaders[index][type] = value;
    this.setState({ subjectLeaders });
  };
  /*
   * 删除学科负责人
   */
  onDeleteLeader = async (id, index) => {
    let { subjectLeaders } = this.state;
    if (id === null) {
      subjectLeaders.splice(index, 1);
      this.setState({ subjectLeaders });
    } else {
      this.setState({ loading: true });
      const resultCode = await Loaders.deleteSubjectLeader({ id });
      if (resultCode === 0) {
        showMessage('删除成功', 'success');
        subjectLeaders.splice(index, 1);
        this.setState({ subjectLeaders });
      } else {
        showMessage('删除失败');
      }
      this.setState({ loading: false });
    }
  };

  /**
   * 修改学科负责人
   */
  modifySubjectLeader = async (params) => {
    this.setState({ loading: true });
    const resultCode = await Loaders.modifySubjectLeader(params);
    console.log(resultCode);
    if (resultCode === 0) {
      showMessage('修改成功', 'success');
    } else {
      showMessage('修改失败');
    }
    this.setState({ loading: false });
  };

  /**
   * 导出 excel
   */
  exportExcel = async () => {
    const {
      branchId,
      campusId,
      productVersionId,
      productQuarterId,
      courseSeriesId,
      classTypeIds,
      gradeId,
      subjectId,
      teacherId,
      sidx,
      sord,
    } = this.state;
    const params = {
      branchId,
      campusId: campusId === null ? campusId : campusId.join(','),
      productVersionId,
      productQuarterId,
      courseSeriesId,
      classTypeIds,
      gradeId: gradeId === null ? gradeId : gradeId.join(','),
      subjectId: subjectId === null ? subjectId : subjectId.join(','),
      teacherId,
      sidx,
      sord,
    };
    await Loaders.exportMiniClassRenewExcelApi(params).then(({ resultCode }) => {
      if (resultCode === 0) {
        this.props.dispatch({ type: 'main/exportNotice' });
      }
    });
  };

  /**
   * 权限判断
   */
  authDetect = (authString) => {
    const { btnAuths } = this.props;
    console.log(btnAuths.includes(authString));
    return btnAuths.includes(authString);
  };

  /**
   * 负责人参数判断
   */
  detectLeaders = (subjectLeaders) => {
    const detectTemp = subjectLeaders.map((subjectLeader) => {
      if (!subjectLeader.teacherId || !subjectLeader.subjectId) {
        return false;
      }
    });
    if (detectTemp.includes(false)) {
      return true;
    } else {
      return false;
    }
  };

  /**
   * 设置学科负责人
   */
  onOk = () => {
    const { subjectLeaders } = this.state;
    if (this.detectLeaders(subjectLeaders)) {
      showMessage('老师或学科不能为空');
      return;
    }
    this.modifySubjectLeader(subjectLeaders);
    this.setState({ isSetSubjectLeader: false });
  };
  render() {
    const {
      loading,
      branchDict,
      campusDict,
      classTypeDict,
      courseSeriesDict,
      gradeDict,
      productQuarterDict,
      productVersionDict,
      subjectDict,
      teacherDict,
      branchId,
      campusId,
      productVersionId,
      productQuarterId,
      courseSeriesId,
      classTypeIds,
      gradeId,
      subjectId,
      teacherId,
      sidx,
      sord,
      dataSource,
      pagination,
      fullClassRateIsAsc,
      refundRateIsAsc,
      reNqRateIsAsc,
      reNnqRateIsAsc,
      isSetSubjectLeader,
      subjectLeaders,
      fullClassRate,
      reNnqRate,
      reNqRate,
    } = this.state;

    // 学习中心字典根据选中分公司变化
    const campusDictPlus = branchId
      ? campusDict &&
        campusDict
          .filter((branch) => branchId.includes(branch.val))
          .reduce((pre, value) => {
            return pre.concat(value);
          }, [])
      : campusTempDict;

    const columns = [
      {
        title: '序号',
        dataIndex: '',
        render: (current, row, index) => {
          return index + 1;
        },
        width: COL_WIDTH_S,
      },
      { title: '分公司', dataIndex: 'branchName', width: COL_WIDTH_S },
      { title: '学习中心', dataIndex: 'blCampusName', width: COL_WIDTH_S },
      { title: '班级名称', dataIndex: 'name', width: COL_WIDTH_M },
      { title: '季度', dataIndex: 'productQuarterName', width: COL_WIDTH_S },
      { title: '年级', dataIndex: 'gradeName', width: COL_WIDTH_S },
      { title: '科目', dataIndex: 'subjectName', width: COL_WIDTH_S },
      { title: '老师', dataIndex: 'teacherName', width: COL_WIDTH_S },
      { title: '开课日期', dataIndex: 'startDate' },
      {
        title: '满班率',
        dataIndex: 'fullClassRate',
        width: COL_WIDTH_M,
        filterDropdown: <span></span>,
        filterIcon:
          fullClassRateIsAsc === true ? (
            <IconButton
              type="triangle_up"
              onClick={() => {
                this.setState({ fullClassRateIsAsc: false, sidx: 'full_class_rate', sord: 'desc' }, () => {
                  this.search();
                });
              }}
            />
          ) : (
            <IconButton
              type="triangle_down"
              onClick={() => {
                this.setState({ fullClassRateIsAsc: true, sidx: 'full_class_rate', sord: 'asc' }, () => {
                  this.search();
                });
              }}
            />
          ),
        render: (value) => {
          return this.toPercent(value);
        },
      },
      {
        title: '退费率',
        dataIndex: 'refundRate',
        width: COL_WIDTH_S,
        filterDropdown: <span></span>,
        filterIcon:
          refundRateIsAsc === true ? (
            <IconButton
              type="triangle_up"
              onClick={() => {
                this.setState({ refundRateIsAsc: false, sidx: 'refund_rate', sord: 'desc' }, () => {
                  this.search();
                });
              }}
            />
          ) : (
            <IconButton
              type="triangle_down"
              onClick={() => {
                this.setState({ refundRateIsAsc: true, sidx: 'refund_rate', sord: 'asc' }, () => {
                  this.search();
                });
              }}
            />
          ),
        render: (value) => {
          return this.toPercent(value);
        },
      },
      {
        title: '下季度续报率',
        dataIndex: 'reNqRate',
        width: COL_WIDTH_L,
        filterDropdown: <span></span>,
        filterIcon:
          reNqRateIsAsc === true ? (
            <IconButton
              type="triangle_up"
              onClick={() => {
                this.setState({ reNqRateIsAsc: false, sidx: 're_nq_rate', sord: 'desc' }, () => {
                  this.search();
                });
              }}
            />
          ) : (
            <IconButton
              type="triangle_down"
              onClick={() => {
                this.setState({ reNqRateIsAsc: true, sidx: 're_nq_rate', sord: 'asc' }, () => {
                  this.search();
                });
              }}
            />
          ),
        render: (value) => {
          return this.toPercent(value);
        },
      },
      {
        title: '下下季度续报率',
        dataIndex: 'reNnqRate',
        width: COL_WIDTH_L,
        filterDropdown: <span></span>,
        filterIcon:
          reNnqRateIsAsc === true ? (
            <IconButton
              type="triangle_up"
              onClick={() => {
                this.setState({ reNnqRateIsAsc: false, sidx: 're_nnq_rate', sord: 'desc' }, () => {
                  this.search();
                });
              }}
            />
          ) : (
            <IconButton
              type="triangle_down"
              onClick={() => {
                this.setState({ reNnqRateIsAsc: true, sidx: 're_nnq_rate', sord: 'asc' }, () => {
                  this.search();
                });
              }}
            />
          ),
        render: (value) => {
          return this.toPercent(value);
        },
      },
    ];

    return (
      <Page loading={loading} title={false} className={styles.page}>
        <div className={styles.header}>
          <div className={styles.searchBar}>
            <SearchBar onSearch={() => this.search()} onReset={this.reset} expand={true}>
              <Select
                dict={branchDict}
                value={branchId}
                placeholder="分公司"
                allowClear={false}
                onChange={(branchId) => {
                  this.setState({ branchId });
                }}
              />
              <MultiSelect
                dict={campusDictPlus}
                value={campusId}
                placeholder="学习中心"
                onChange={(campusId) => {
                  this.setState({ campusId });
                }}
              />
              <Select
                dict={productVersionDict}
                value={productVersionId}
                placeholder="产品年份"
                allowClear={false}
                onChange={(productVersionId) => {
                  this.setState({ productVersionId });
                }}
              />
              <Select
                dict={productQuarterDict}
                value={productQuarterId}
                placeholder="产品季度"
                onChange={(productQuarterId) => {
                  this.setState({ productQuarterId });
                }}
              />
              <Select
                dict={courseSeriesDict}
                value={courseSeriesId}
                placeholder="产品系列"
                onChange={(courseSeriesId) => {
                  this.setState({ courseSeriesId });
                }}
              />
              <MultiSelect
                dict={classTypeDict}
                value={classTypeIds}
                placeholder="班级类型"
                onChange={(classTypeIds) => {
                  this.setState({ classTypeIds });
                }}
              />
              <MultiSelect
                dict={gradeDict}
                value={gradeId}
                placeholder="年级"
                onChange={(gradeId) => {
                  this.setState({ gradeId });
                }}
              />
              <MultiSelect
                dict={subjectDict}
                value={subjectId}
                placeholder="科目"
                onChange={(subjectId) => {
                  this.setState({ subjectId });
                }}
              />
              <Select
                dict={teacherDict}
                value={teacherId}
                placeholder="老师"
                showSearch={true}
                onChange={(teacherId) => {
                  this.setState({ teacherId });
                }}
              />
            </SearchBar>
          </div>

          <div className={styles.buttons}>
            <Button onClick={this.openModal} disabled={!this.authDetect('MINICLASS_REENROLL_SETLEADERS_BU')}>
              设置学科负责人
            </Button>
            <Button
              className={styles.exportExcel}
              onClick={this.exportExcel}
              disabled={!this.authDetect('MINICLASS_REENROLL_BUTTON')}
            >
              导出 excel
            </Button>
          </div>
        </div>
        {dataSource.length !== 0 && (
          <div className={styles.statistics}>
            总满班率:{this.toPercent(fullClassRate)}，总下季度续报率:{this.toPercent(reNqRate)}，总下下季度续报率:
            {this.toPercent(reNnqRate)}
          </div>
        )}

        <List
          columns={columns}
          dataSource={dataSource}
          pagination={pagination}
          onChange={(paginationOptions) => {
            this.setState(
              {
                pagination: Object.assign(pagination, {
                  pageSize: paginationOptions.pageSize,
                  current: paginationOptions.current,
                }),
              },
              () => {
                this.search();
              },
            );
          }}
        />
        {isSetSubjectLeader && (
          <Modal
            loading={loading}
            title="设置"
            widthSize="M"
            onCancel={() => {
              this.setState({ isSetSubjectLeader: false });
            }}
            className={styles.modal}
            footerTipText="有设置的老师才能查看"
            onOk={this.onOk}
          >
            <Row style={{ paddingBottom: 20 }}>
              <Col span={20}>选择对应学科老师查看对应科目的续报数据</Col>
              <Col span={4} style={{ paddingLeft: 20 }}>
                <a onClick={this.addSubjectLeader}>添加</a>
              </Col>
            </Row>
            {subjectLeaders.map((subjectLeader, index) => {
              const { id, subjectId, teacherId } = subjectLeader;
              return (
                <Row>
                  <Col span={10}>
                    <FormItem name="老师" labelWidth={40}>
                      <Select
                        value={teacherId}
                        dict={teacherDict}
                        showSearch={true}
                        onChange={(value) => {
                          this.changeTeacherOrSubject({ type: 'teacherId', value, index });
                        }}
                      />
                    </FormItem>
                  </Col>
                  <Col span={10}>
                    <FormItem name="科目" labelWidth={40}>
                      <Select
                        value={subjectId}
                        dict={subjectDict}
                        showSearch={true}
                        onChange={(value) => {
                          this.changeTeacherOrSubject({ type: 'subjectId', value, index });
                        }}
                      />
                    </FormItem>
                  </Col>
                  <Col span={4}>
                    <span
                      style={{ paddingLeft: 20, lineHeight: '36px' }}
                      onClick={() => {
                        this.onDeleteLeader(id, index);
                      }}
                    >
                      <a>删除</a>
                    </span>
                  </Col>
                </Row>
              );
            })}
          </Modal>
        )}
      </Page>
    );
  }
}

let mapStateToProps = (state) => {
  let account = state.account;
  return {
    account: account,
    btnAuths: state.account.btnAuths,
  };
};

export default connect(mapStateToProps)(MiniClassReEnroll);
