import React from 'react';
import List from 'COMPONENT/List';
import SearchBar from 'COMPONENT/SearchBar';
import Select from 'COMPONENT/Select';
import Input from 'COMPONENT/Input';
import DatePicker from 'COMPONENT/DatePicker';
import Wrapper from 'COMPONENT/Wrapper';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import moment from 'moment';
import { dealCampus } from 'UTIL/dict';
import { remindByAttendanceListApi } from './loaders';
import { miniClassRemindByAttendanceListExport } from 'SERVICE/course';
import { connect } from 'dva';
import styles from './index.less';

class RemindByAttendanceList extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      branchId: '',
      campusId: '',
      teacherName: '',
      studyManegerName: '',
      homeroomTeacherName: '',
      courseDateStart: moment().startOf('month').format('YYYY-MM-DD'),
      courseDateEnd: moment().subtract(1, 'days').format('YYYY-MM-DD'),
      campusDict: [],

      remindByAttendanceListData: [], // 列表数据
      pagination: {
        current: 1,
        pageSize: 20,
        total: 0,
        pageSizeOptions: ['20', '50', '100'],
      }, // 列表分页设置
    };

    this.onBranchIdChange = this.onBranchIdChange.bind(this);
    this.onCampusIdChange = this.onCampusIdChange.bind(this);
    this.onHomeroomTeacherNameChange = this.onHomeroomTeacherNameChange.bind(this);
    this.onStudyManegerNameChange = this.onStudyManegerNameChange.bind(this);
    this.onSearch = this.onSearch.bind(this);
    this.onTeacherNameChange = this.onTeacherNameChange.bind(this);
    this.getDisabledCourseDateStart = this.getDisabledCourseDateStart.bind(this);
    this.getDisabledCourseDateEnd = this.getDisabledCourseDateEnd.bind(this);
    this.onReset = this.onReset.bind(this);
    this.clickOnSearch = this.clickOnSearch.bind(this);
    this.exportExecelData = this.exportExecelData.bind(this);

    this.columns = [
      { title: '学生姓名', dataIndex: 'studentName' },
      { title: '客户姓名', dataIndex: 'customerName' },
      { title: '联系方式', dataIndex: 'customerContact' },
      { title: '所属分公司', dataIndex: 'branchName' },
      { title: '所属学习中心', dataIndex: 'campusName' },
      { title: '班级名称', dataIndex: 'miniClassName' },
      { title: '班级班主任', dataIndex: 'studyManegerName' },
      { title: '学生班主任', dataIndex: 'homeroomTeacherName' },
      { title: '老师', dataIndex: 'teacherName' },
      {
        title: '讲数',
        dataIndex: 'courseNum',
        render: (value) => {
          return <span>{`第${value}讲`}</span>;
        },
      },
      { title: '上课日期', dataIndex: 'courseDate' },
      { title: '上课时间', dataIndex: 'courseTime' },
      {
        title: '乐捐提醒类型',
        dataIndex: 'isNotAtt',
        render: (value) => {
          if (value === true) {
            return <span>插班/冲销补考勤</span>;
          } else {
            return <span></span>;
          }
        },
      },
    ];
  }

  componentDidMount() {
    const self = this;
    GlobalEmitter.create('mini_class_attendance_reminder_export').listen('courseTime_export', function () {
      self.exportExecelData();
    });
  }

  componentWillUnmount() {
    GlobalEmitter.remove('mini_class_attendance_reminder_export');
  }

  componentWillReceiveProps(nextProps, nextState) {
    const { pageParams } = this.props;
    if (this.props.campusDict.length === 0 && nextProps.campusDict.length !== 0) {
      this.setState({ campusDict: nextProps.campusDict });
      if (pageParams === undefined) {
        this.setState({ branchId: nextProps.branchDict[0].val, campusId: nextProps.campusDict[0].val });
      }
    }
  }

  componentDidUpdate(prevProps) {
    if (
      this.props.campusDict.length !== 0 &&
      this.props.branchDict.length !== 0 &&
      prevProps.campusDict.length === 0 &&
      prevProps.branchDict.length === 0
    ) {
      this.onSearch();
    }
  }

  /**
   * 获取要禁用的开始日期值
   * @param  {string} courseDateStart 过滤前的日期值
   * @return {boolean} 是否要禁用
   */
  getDisabledCourseDateStart(courseDateStart) {
    const { courseDateEnd } = this.state;
    if (!courseDateStart || !courseDateEnd) {
      return false;
    }
    return moment(courseDateStart).valueOf() > moment(courseDateEnd).valueOf();
  }

  /**
   * 获取要禁用的结束日期值
   * @param  {string} courseDateEnd   过滤前的日期值
   * @return {boolean}      是否要禁用
   */
  getDisabledCourseDateEnd(courseDateEnd) {
    const { courseDateStart } = this.state;
    if (!courseDateEnd || !courseDateStart) {
      return false;
    }
    return moment(courseDateEnd).valueOf() < moment(courseDateStart).valueOf();
  }

  onBranchIdChange(branchId) {
    const { allCampusDict } = this.props;
    const campusDict = dealCampus(allCampusDict, branchId);
    this.setState({ branchId, campusDict, campusId: '' });
  }

  onCampusIdChange(campusId) {
    this.setState({ campusId });
  }

  onTeacherNameChange(e) {
    this.setState({ teacherName: e.currentTarget.value });
  }

  onStudyManegerNameChange(e) {
    this.setState({ studyManegerName: e.currentTarget.value });
  }

  onHomeroomTeacherNameChange(e) {
    this.setState({ homeroomTeacherName: e.currentTarget.value });
  }

  async onSearch() {
    const { changeLoadingStatus } = this.props;
    changeLoadingStatus(true);
    const {
      branchId,
      campusId,
      teacherName,
      studyManegerName,
      homeroomTeacherName,
      courseDateStart,
      courseDateEnd,
      pagination: { pageSize: rows, current: page },
    } = this.state;
    const params = {
      branchId,
      campusId,
      teacherName,
      studyManegerName,
      homeroomTeacherName,
      courseDateStart,
      courseDateEnd,
      rows,
      page,
    };
    try {
      let { total, remindByAttendanceListData } = await remindByAttendanceListApi(params);
      this.setState({ remindByAttendanceListData, pagination: Object.assign({}, this.state.pagination, { total }) });
      changeLoadingStatus(false);
    } catch (err) {
      showMessage(err.text);
      changeLoadingStatus(false);
    }
  }

  clickOnSearch() {
    this.setState({ pagination: Object.assign({}, this.state.pagination, { current: 1 }) }, () => {
      this.onSearch();
    });
  }

  async exportExecelData() {
    const { branchId, campusId, teacherName, studyManegerName, homeroomTeacherName, courseDateStart, courseDateEnd } =
      this.state;
    const params = {
      branchId,
      campusId,
      teacherName,
      studyManegerName,
      homeroomTeacherName,
      courseDateStart,
      courseDateEnd,
    };
    if (this.state.pagination.total > 5000) {
      return showMessage('记录数大于5000条，请缩小范围查询，再点击导出', 'error');
    }
    try {
      this.props.onExport(true);
      const { resultCode } = await miniClassRemindByAttendanceListExport(params);
      if (resultCode === 0) {
        this.props.dispatch({
          type: 'main/exportNotice',
        });
      } else {
        showMessage(resultMessage || '导出失败', 'error');
      }
    } finally {
      this.props.onExport(false);
    }
  }

  onReset() {
    const { allCampusDict } = this.props;
    this.setState({
      branchId: this.props.pageParams === undefined ? this.props.branchDict[0].val : '',
      campusDict: dealCampus(allCampusDict, this.props.branchDict[0].val),
      campusId:
        this.props.pageParams === undefined ? dealCampus(allCampusDict, this.props.branchDict[0].val)[0].val : '',
      teacherName: '',
      studyManegerName: '',
      homeroomTeacherName: '',
      courseDateStart: '',
      courseDateEnd: '',
      pagination: {
        current: 1,
        pageSize: 20,
        total: 0,
        pageSizeOptions: ['20', '50', '100'],
      },
    });
  }

  render() {
    const {
      branchId,
      campusId,
      teacherName,
      studyManegerName,
      homeroomTeacherName,
      courseDateStart,
      courseDateEnd,
      remindByAttendanceListData,
      pagination,
      campusDict,
    } = this.state;
    const { branchDict } = this.props;

    return (
      <div>
        <Wrapper className="remindByAttendance">
          <SearchBar onSearch={this.clickOnSearch} onReset={this.onReset} expand={true}>
            <Select
              dict={branchDict}
              placeholder="所属分公司"
              value={branchId}
              onChange={this.onBranchIdChange}
              showSearch={true}
            />
            <Select
              dict={campusDict}
              placeholder="所属学习中心"
              value={campusId}
              onChange={this.onCampusIdChange}
              showSearch={true}
            />
            <Input value={teacherName} placeholder="老师" onChange={this.onTeacherNameChange} />
            <Input value={studyManegerName} placeholder="班级班主任" onChange={this.onStudyManegerNameChange} />
            <Input value={homeroomTeacherName} placeholder="学生班主任" onChange={this.onHomeroomTeacherNameChange} />
            <DatePicker
              placeholder="上课日期起"
              value={courseDateStart}
              onChange={(val) => this.setState({ courseDateStart: val })}
              disabledDate={this.getDisabledCourseDateStart}
            />
            <DatePicker
              placeholder="上课日期止"
              value={courseDateEnd}
              onChange={(val) => this.setState({ courseDateEnd: val })}
              disabledDate={this.getDisabledCourseDateEnd}
            />
          </SearchBar>
        </Wrapper>
        <List
          dataSource={remindByAttendanceListData}
          columns={this.columns}
          pagination={pagination}
          onChange={(pagination) => {
            this.setState(
              {
                pagination: Object.assign({}, this.state.pagination, {
                  current: pagination.current,
                  pageSize: pagination.pageSize,
                }),
              },
              () => {
                this.onSearch();
              },
            );
          }}
        />
      </div>
    );
  }
}

export default connect(null, null)(RemindByAttendanceList);
