import React from 'react';
import Page from 'COMPONENT/Page';
import Tabs from 'COMPONENT/Tabs';
import Button from 'COMPONENT/Button';
import { connect } from 'dva';
const Tab = Tabs.Tab;
import { loadBranchDict, loadBranchCampusDict, dealCampus } from 'UTIL/dict';
import RemindByAttendanceList from './RemindByAttendanceList';
import RemindByCourseList from './RemindByCourseList';

class MiniClassScheduleAttendanceReminder extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: true,
      exporting: false,
      activeKey:
        this.props.params === undefined || this.props.params.itemType === 'DONATE_REMIND_ITEM_MC_NOT_ATT'
          ? 'courseTime'
          : 'course',
      allCampusDict: [], // 用户学习中心按分公司分组

      branchDict: [],
      campusDict: [],
    };

    this.initContent = this.initContent.bind(this);
    this.loadDict = this.loadDict.bind(this);
    this.changeLoadingStatus = this.changeLoadingStatus.bind(this);
    this.exportCourseData = this.exportCourseData.bind(this);
    this.toggleExportState = this.toggleExportState.bind(this);
  }

  componentDidMount() {
    this.initContent();
  }

  async initContent() {
    await this.loadDict();
  }

  async loadDict() {
    const [branchDict, allCampusDict] = await Promise.all([loadBranchDict(), loadBranchCampusDict()]);
    let campusDict = dealCampus(allCampusDict, branchDict[0].val);
    this.setState({ branchDict, allCampusDict, campusDict });
  }

  changeLoadingStatus(status) {
    this.setState({ loading: status });
  }

  exportCourseData() {
    const { activeKey } = this.state;
    GlobalEmitter.create('mini_class_attendance_reminder_export').trigger(
      activeKey === 'course' ? 'course_export' : 'courseTime_export',
    );
  }

  toggleExportState(state) {
    this.setState({ exporting: state });
  }

  render() {
    const { params } = this.props;
    const { loading, activeKey, branchDict, campusDict, allCampusDict, exporting } = this.state;
    return (
      <Page
        title="小班排课考勤提醒"
        loading={loading}
        headerToolBar={
          <Button loading={exporting} type="primary" onClick={this.exportCourseData}>
            导出
          </Button>
        }
      >
        <Tabs
          type="button"
          activeKey={activeKey}
          onChange={(activeKey) => {
            this.setState({ activeKey });
          }}
        >
          <Tab title="科次" value="courseTime">
            <RemindByAttendanceList
              branchDict={branchDict}
              campusDict={campusDict}
              changeLoadingStatus={this.changeLoadingStatus}
              allCampusDict={allCampusDict}
              pageParams={params}
              onExport={this.toggleExportState}
            />
          </Tab>
          <Tab title="课程" value="course">
            <RemindByCourseList
              branchDict={branchDict}
              campusDict={campusDict}
              changeLoadingStatus={this.changeLoadingStatus}
              allCampusDict={allCampusDict}
              pageParams={params}
              onExport={this.toggleExportState}
            />
          </Tab>
        </Tabs>
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const pathname = state.routing.locationBeforeTransitions.pathname;
  return {
    params: state.main.pageParams[pathname],
  };
};

export default connect(mapStateToProps, null)(MiniClassScheduleAttendanceReminder);
