import React from 'react';
import Modal from 'COMPONENT/Modal';
import List from 'COMPONENT/List';
import Search from 'PAGE/course/ReEnroll/Search';
import { Row, Col } from 'antd';
import Select from 'COMPONENT/Select';
import styles from './index.less';

class SelectNewMiniClass extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      inputValue: '',
      campusId: null,
      miniClassTypeId: null,
      teacherId: '',
      loading: false,
    };
  }

  onSearch = async (selectedDatas) => {
    const { teacherId } = this.state;
    const { searchNewMiniClass } = this.props;
    const params = Object.assign(selectedDatas, { teacherId });

    this.setState({ loading: true });
    await searchNewMiniClass(params);
    this.setState(selectedDatas);
    this.setState({ loading: false });
  };

  reset = () => {
    this.setState({
      inputValue: '',
      campusId: null,
      miniClassTypeId: null,
      teacherId: '',
    });
  };

  render() {
    const { inputValue, campusId, miniClassTypeId, loading, teacherId } = this.state;
    const {
      changeMiniClassList,
      miniClassTypeDict,
      campusDict,
      searchNewMiniClass,
      closeSelectNewMiniClass,
      getNewMiniClass,
      teacherDict,
    } = this.props;
    // 坑！！！！conditions组件会改变原始字典
    const conditionCampusDict = [...campusDict];
    const conditionMiniClassTypeDict = [...miniClassTypeDict];

    const { dataSource, total, current } = changeMiniClassList;
    const columns = [
      { title: '班课名称', dataIndex: 'name', key: 'name', width: 148 },
      { title: '学习中心', dataIndex: 'blCampusName', key: 'blCampusName', width: 94 },
      { title: '老师', dataIndex: 'teacherName', key: 'teacherName', width: 72 },
      { title: '开始上课时间', dataIndex: 'startDate', key: 'startDate', width: 105 },
      { title: '上课时间', dataIndex: 'classTime', key: 'classTime', width: 78 },
      { title: '计划招生数量', dataIndex: 'peopleQuantity', key: 'peopleQuantity' },
      { title: '已招数量', dataIndex: 'miniClassPeopleNum', width: 80, key: 'miniClassPeopleNum' },
      { title: '班级类型', dataIndex: 'miniClassTypeName', width: 80, key: 'miniClassTypeName' },
      {
        title: '操作',
        key: 'id',
        width: 55,
        render: (currentValue, currentRow, index) => {
          return (
            <span
              className={styles.selectBtn}
              onClick={() => {
                getNewMiniClass(currentValue.miniClassId);
              }}
            >
              选择
            </span>
          );
        },
      },
    ];
    const pagination = { current: current || 1, total: total || 0, pageSize: 10 };
    const conditions = [
      { title: '授课老师', field: 'teacherId', dict: teacherDict, type: 'children' },
      {
        title: '所属学习中心',
        field: 'campusId',
        dict: conditionCampusDict,
        isMultiSelect: true,
      },
      {
        title: '班级类型',
        field: 'miniClassTypeId',
        dict: conditionMiniClassTypeDict,
        isMultiSelect: true,
      },
    ];
    const teacherSelect = [
      <div key="asdf" className="label" style={{ width: 136 }}>
        授课老师
      </div>,
      <div key="gaef" className="items">
        <Row style={{ marginTop: '10px' }}>
          <Col span={7}>
            <Select
              name="teacherId"
              value={teacherId}
              onChange={(teacherId) => {
                this.setState({ teacherId });
              }}
              searchPlaceholder="请输入老师名字搜索"
              dict={teacherDict}
              showSearch={true}
              allowClear={true}
            />
          </Col>
        </Row>
      </div>,
    ];
    const selectedDatas = { inputValue, teacherId, campusId, miniClassTypeId };

    return (
      <Modal
        title="选择调入班"
        loading={loading}
        width="1097"
        okBtnShow={false}
        onClose={() => {
          closeSelectNewMiniClass();
        }}
      >
        <Search
          onInitFinish={() => {}}
          conditions={conditions}
          selectedDatas={selectedDatas}
          onSearch={(selectedDatas) => {
            this.onSearch(selectedDatas);
          }}
          isShowResult={false}
          onInputChange={(inputValue) => {
            this.setState({ inputValue });
          }}
          onReset={() => {
            this.reset();
          }}
          conditionWidth={136}
        >
          <div className="clearfix">{teacherSelect}</div>
        </Search>
        <List
          dataSource={dataSource}
          columns={columns}
          pagination={pagination}
          onChange={(changeParam) => {
            searchNewMiniClass(selectedDatas, changeParam.current, changeParam.pageSize);
          }}
        />
      </Modal>
    );
  }
}

export default SelectNewMiniClass;
