import React from 'react';
import { connect } from 'dva';
import Modal from 'COMPONENT/Modal';
import OldCourseNum from './OldCourseNum';
import NewClassInfo from './NewClassInfo';
import styles from './index.less';
import loaders from '../loaders';
import SelectNewMiniClass from './SelectNewMiniClass';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

class ChangeCourse extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      studentCourses: [], //原班课程
      selectedCourses: [], // 原班被选中课程
      changeMiniClassList: {}, // 学生调课可调入班列表
      isSelectNewMiniClass: false, // 是否显示调入班列表Modal
      miniClassTypeDict: {}, //小班类型字典
      loading: true,
      newMiniClassInfo: {}, //调入班信息
      newMiniClassCourses: [], //调入班课程
      selectedNewCourses: [], // 被选择调入班课程
      studentsInfo: [], // 被选中学生信息
    };
  }

  componentDidMount() {
    this.initContent();
  }

  initContent = async () => {
    const { miniClassId, studentId, courseNum, studentIds } = this.props;
    await this.getMiniClassDetailStudentList();
    let studentCourses;
    if (studentIds.length === 0) {
      studentCourses = await loaders.loadStudentConflictCourse({ miniClassId, studentId });
    } else {
      studentCourses = await loaders.loadStudentConflictCourseNew({ miniClassId, studentIds });
    }
    this.setState({ studentCourses, loading: false });
    studentCourses.forEach((course, index) => {
      if (course.courseNum === courseNum) {
        this.selectCourse(index);
      }
    });
  };

  // 获取学生课时单价和每讲课时数
  getMiniClassDetailStudentList = async () => {
    const { miniClassId, studentId, studentIds } = this.props;
    let studentsList = await loaders.getMiniClassDetailStudentListApi({ miniClassId });
    if (studentIds.length === 0) {
      let studentsInfo = studentsList.rows.filter((student) => {
        return student.studentId === studentId;
      });
      this.setState({ studentsInfo });
    } else {
      let studentsInfo = studentsList.rows.filter((student) => {
        return studentIds.join(',').indexOf(student.studentId) !== -1;
      });
      this.setState({ studentsInfo });
    }
  };

  // 原班课程选中
  selectCourse = (index) => {
    let { studentCourses, newMiniClassCourses } = this.state;
    studentCourses[index].selected = !studentCourses[index].selected;

    if (index < newMiniClassCourses.length) {
      newMiniClassCourses[index].selected = studentCourses[index].selected;
      if (
        newMiniClassCourses[index].selected === true &&
        (newMiniClassCourses[index].courseStatus === 'TEACHER_ATTENDANCE' ||
          newMiniClassCourses[index].courseStatus === 'CHARGED')
      ) {
        showMessage('该讲课已考勤，若调课成功后，请及时通知任课老师进行考勤');
      }
    }

    const selectedCourses = studentCourses.filter((course) => {
      return course.selected === true;
    });
    const selectedNewCourses = newMiniClassCourses.filter((course) => {
      return course.selected === true;
    });
    this.setState({
      studentCourses,
      selectedCourses,
      newMiniClassCourses,
      selectedNewCourses,
    });
  };

  // 选择调入班课程
  selectNewCourse = (index) => {
    let { studentCourses, newMiniClassCourses } = this.state;
    newMiniClassCourses[index].selected = !newMiniClassCourses[index].selected;
    if (
      newMiniClassCourses[index].selected === true &&
      (newMiniClassCourses[index].courseStatus === 'TEACHER_ATTENDANCE' ||
        newMiniClassCourses[index].courseStatus === 'CHARGED')
    ) {
      showMessage('该讲课已考勤，若调课成功后，请及时通知任课老师进行考勤');
    }
    let selectedNewCourses = newMiniClassCourses.filter((course) => {
      return course.selected === true;
    });
    this.setState({
      newMiniClassCourses,
      selectedNewCourses,
      studentCourses,
    });
  };

  // 学生调课选择调入班
  selectNewMiniClass = async () => {
    const { miniClassId } = this.props;
    let params = { miniClassId, page: 1, rows: '10', sidx: 'createTime', sord: 'desc' };
    const [changeMiniClassList, miniClassTypeDict] = await Promise.all([
      loaders.loadChangeMiniClassList(params),
      loaders.loadSelectOption(),
    ]);
    this.setState({ changeMiniClassList, isSelectNewMiniClass: true, miniClassTypeDict });
  };

  // 搜索可调入小班
  searchNewMiniClass = async (selectedDatas, page = 1, rows = 10) => {
    const { campusId, miniClassTypeId, inputValue, teacherId } = selectedDatas;
    let searchParams = { campusId, miniClassTypeId, className: inputValue, teacherId };
    const { miniClassId } = this.props;
    let params = { miniClassId, page, rows, sidx: 'createTime', sord: 'desc', ...searchParams };
    params = Object.assign(params, selectedDatas);
    const changeMiniClassList = await loaders.loadChangeMiniClassList(params);
    this.setState({ changeMiniClassList });
  };

  // 关闭选择调入班模态框
  closeSelectNewMiniClass = () => {
    this.setState({ isSelectNewMiniClass: false });
  };

  // 获取调入班
  getNewMiniClass = async (miniClassId) => {
    const { studentCourses } = this.state;
    this.closeSelectNewMiniClass();
    this.setState({ loading: true });
    let newMiniClassCourses = await loaders.loadConflictMiniCourseInfo({ miniClassId });
    let newMiniClassInfo = await loaders.loadConflictMiniClassInfo({ miniClassId });
    for (let i = 0; i < studentCourses.length; i++) {
      if (studentCourses[i].selected === true) {
        if (newMiniClassCourses[i] !== undefined) {
          if (
            newMiniClassCourses[i].courseStatus === 'TEACHER_ATTENDANCE' ||
            newMiniClassCourses[i].courseStatus === 'CHARGED'
          ) {
            showMessage('该讲课已考勤，若调课成功后，请及时通知任课老师进行考勤');
          }
          newMiniClassCourses[i].selected = true;
        }
      } else {
        if (newMiniClassCourses[i] !== undefined) {
          newMiniClassCourses[i].selected = false;
        }
      }
    }
    let selectedNewCourses = newMiniClassCourses.filter((course) => {
      return course.selected === true;
    });
    this.setState({ loading: false, newMiniClassInfo: newMiniClassInfo[0], newMiniClassCourses, selectedNewCourses });
  };

  // 换课
  changeCourse = async (studentId, selectedNewCourses, selectedCourses) => {
    const { newMiniClassCourses } = this.state;
    const { search, closeModal, studentIds, cancelMultiChangeCourses } = this.props;
    if (selectedNewCourses.length === 0) {
      showMessage('请选择您需要调入的课程!');
      return;
    }
    if (newMiniClassCourses.length === 0) {
      showMessage('调入班未排课, 请先排课!');
      return;
    }
    if (selectedCourses.length === 0) {
      showMessage('请选择原班的讲数!');
      return;
    }
    if (selectedNewCourses.length === 0) {
      showMessage('请选择调入班的讲数!');
      return;
    }
    if (selectedNewCourses.length !== selectedCourses.length) {
      showMessage('调出课程讲数与调入课程讲数不一致!');
      return;
    }
    let newCourseIds = selectedNewCourses.map((course) => {
      return course.courseId;
    });
    let oldCourseIds = selectedCourses.map((course) => {
      return course.courseId;
    });
    let courseMap = [];
    for (let i = 0; i < oldCourseIds.length; i++) {
      courseMap.push({ newCourseId: newCourseIds[i], oldCourseId: oldCourseIds[i] });
    }
    let changeCourseVos =
      studentIds.length === 0
        ? [
            {
              studentId,
              courseMap,
            },
          ]
        : studentIds.map((studentId) => {
            return {
              studentId,
              courseMap,
            };
          });
    let params = {
      changeCourseVos,
    };
    let checkConflictParams = {
      studentId,
      courseIds: newCourseIds,
    };
    let response = await loaders.changeCourseApi(params);
    if (response.resultMessage === '调课成功') {
      let checkConflictResponse = await loaders.checkConflictAfterChangeCourseApi(checkConflictParams);
      if (checkConflictResponse.data === true) {
        showMessage(checkConflictResponse.resultMessage, 'success');
      } else {
        showMessage('调课成功', 'success');
      }
      closeModal();
      search();
    }
    cancelMultiChangeCourses && cancelMultiChangeCourses();
  };

  render() {
    const {
      studentCourses,
      changeMiniClassList,
      isSelectNewMiniClass,
      miniClassTypeDict,
      loading,
      selectedNewCourses,
      newMiniClassInfo,
      hasNewMiniClassInfo,
      newMiniClassCourses,
      selectedCourses,
      studentsInfo,
    } = this.state;
    const { closeModal, campusDict, studentId, teacherDict } = this.props;

    return (
      <Modal
        title="调课"
        onCancel={closeModal}
        width="598"
        loading={loading}
        onOk={() => {
          this.changeCourse(studentId, selectedNewCourses, selectedCourses);
        }}
        footerTipText={
          <div className={styles.footerTips}>产品组为自习/在线A/在线B的小班，只能在相同产品组间进行转班和调课</div>
        }
      >
        {studentsInfo.map((studentInfo, index) => {
          const {
            studentName,
            miniClassTotalUnConsumedNatureHours,
            miniClassTotalUnConsumedHours,
            miniClassSurplusFinance,
            everyCourseClassNum,
            miniClassCourseHourPrice,
          } = studentInfo;
          const courseNum = Math.floor(miniClassTotalUnConsumedHours / everyCourseClassNum); // 可上课程讲数 = 学生剩余已购买未扣费小时 / 课程每讲小时
          const courseNum2 = Math.floor(miniClassSurplusFinance / (everyCourseClassNum * miniClassCourseHourPrice)); // 可扣课程 = 学生剩余小时 / 课程每讲小时

          return (
            <div key={index} className={styles.studentCourseInfo}>
              {studentName}({miniClassTotalUnConsumedNatureHours}小时) 可上{courseNum}讲课程；({miniClassSurplusFinance}
              元) 可扣{courseNum2}讲课程
            </div>
          );
        })}
        <OldCourseNum
          studentCourses={studentCourses}
          changeMiniClassList={changeMiniClassList}
          selectCourse={this.selectCourse}
          newMiniClassCourses={newMiniClassCourses}
          juxtaposeCourses={this.juxtaposeCourses}
        />
        <NewClassInfo
          selectNewMiniClass={this.selectNewMiniClass}
          newMiniClassInfo={newMiniClassInfo}
          hasNewMiniClassInfo={hasNewMiniClassInfo}
          newMiniClassCourses={newMiniClassCourses}
          selectNewCourse={this.selectNewCourse}
        />
        {isSelectNewMiniClass && (
          <SelectNewMiniClass
            teacherDict={teacherDict}
            searchNewMiniClass={this.searchNewMiniClass}
            campusDict={campusDict}
            miniClassTypeDict={miniClassTypeDict}
            changeMiniClassList={changeMiniClassList}
            closeSelectNewMiniClass={this.closeSelectNewMiniClass}
            getNewMiniClass={this.getNewMiniClass}
          />
        )}
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  return {};
};

export default connect(mapStateToProps)(ChangeCourse);
