import React from 'react';

import PropTypes from 'prop-types';
import styles from './index.less';

export default class RoomViewOfMiniClassCourseDetail extends React.Component {
  componentDidMount() {
    this.timer = null;
    this.setLocation();
  }

  componentWillUnmount() {
    this.clearTimer();
  }

  clearTimer() {
    if (!this.timer) {
      return;
    }
    window.clearTimeout(this.timer);
    this.timer = null;
  }

  setLocation() {
    this.clearTimer();
    this.timer = setTimeout(() => {
      const location = {
        top: 0,
        left: '-2px',
      };

      const parent = this.$wrapper.parentElement;
      const room = document.querySelector('.roomViewOfMiniClassCourseRoom');
      const viewPointScrollWidth = room.scrollWidth;
      const viewPointScrollHeight = room.scrollHeight;
      const locationX = parseFloat(parent.style.left);
      const locationY = parseFloat(parent.style.top);
      const clientWidth = this.$wrapper.clientWidth;
      const clientHeight = this.$wrapper.clientHeight;

      // 边界判断 X轴
      if (clientWidth + locationX > viewPointScrollWidth) {
        location.left = `-${clientWidth - 163}px`;
      }
      // 边界判断 Y轴
      if (clientHeight + locationY > viewPointScrollHeight) {
        location.top = `-${clientHeight - 69}px`;
      }

      this.$wrapper.style = `top: ${location.top}; left: ${location.left}`;
    }, 2000);
  }

  handleScroll = (e) => {
    e.stopPropagation();
  };

  handleMouseLeave = () => {
    if (!this.$wrapper) {
      return;
    }
    this.$wrapper.scrollTop = 0;
  };

  render() {
    const { list, isConflict } = this.props;
    const classNames = ['roomViewOfMiniClassCourseDetail', styles.wrapper, isConflict ? styles.conflict : ''].join(' ');

    return (
      <div
        ref={(el) => (this.$wrapper = el)}
        className={classNames}
        onScroll={this.handleScroll}
        onMouseLeave={this.handleMouseLeave}
      >
        <div className={styles.content}>
          {isConflict && <h3>冲突详情</h3>}
          <ul>
            {list.map((item) => {
              const {
                planNum,
                classroom,
                enrollNum,
                courseNum,
                courseTime,
                courseDate,
                teacherName,
                blCampusName,
                miniClassName,
                courseEndTime,
                _courseEndTime,
                miniClassCourseId,
              } = item;

              return (
                <li key={miniClassCourseId}>
                  <h4>
                    {miniClassName}
                    <span>{`第${courseNum}讲`}</span>
                  </h4>
                  <p>
                    上课时间：
                    <span>
                      {courseDate}
                      <i>
                        {courseTime}-{_courseEndTime || courseEndTime}
                      </i>
                    </span>
                  </p>
                  <p>
                    上课地点：
                    <span>
                      {blCampusName}（{classroom}）
                    </span>
                  </p>
                  <p>
                    任课老师：<span>{teacherName}</span>
                  </p>
                  <p>
                    学生人数：
                    <span>
                      {enrollNum}/{planNum}
                    </span>
                  </p>
                </li>
              );
            })}
          </ul>
        </div>
      </div>
    );
  }

  static propTypes = {
    list: PropTypes.array.isRequired,
    isConflict: PropTypes.bool.isRequired,
  };
}
