import React from 'react';
import { Icon } from 'antd';

import _ from 'lodash';
import moment from 'moment';
import Utils from './utils';
import { DATE_FORMAT } from 'COMPONENT/utils/form';

import DefaultTip from '../DefaultTip';
import RoomViewOfMiniClassCourseDetail from './RoomViewOfMiniClassCourseDetail';

import PropTypes from 'prop-types';
import styles from './index.less';

import iconWarnIcon from '../../../../../assets/imgs/icon_warn.png';
import iconPersonIcon from '../../../../../assets/imgs/icon_person.png';

const TIME_RANGE_COUNT = 9; // 时间刻度
const CELL_HEIGHT = 70;
const CELL_WIDTH = 164;

const WEEK_NAMES = ['周一', '周二', '周三', '周四', '周五', '周六', '周日'];
const TIMES = ['06:00', '08:00', '10:00', '12:00', '14:00', '16:00', '18:00', '20:00', '22:00'];

export default class RoomViewOfMiniClassCourseList extends React.Component {
  state = {
    roomLocations: [],
  };

  constructor(props) {
    super(props);
    this.requestAnimationFrameId = null;
  }

  componentWillMount() {
    this.flattenRoomData = this.getFlattenRoomData();
    this.formatRoomDataByDateAndRoomId = this.getFormatRoomDataByDateAndRoomId();
    this.realRoomList = this.getRealRoomList();
    this.roomHeaderList = this.getRoomHeaderList();
    this.datesOnCurrentWeek = this.getDatesOnCurrentWeek();
    this.datesOnCurrentWeekHasData = this.getDatesOnCurrentWeekHasData();
    this.isEmpty = this.getIsEmpty();
  }

  componentDidMount() {
    this.setRoomLocation();
  }

  componentWillUnmount() {
    this.cancelAnimationFrame();
  }

  getIsEmpty() {
    return !this.datesOnCurrentWeekHasData.length || !this.realRoomList.length;
  }

  getRealRoomList() {
    const { roomList, isCompressed } = this.props;
    return isCompressed
      ? roomList
          .filter(({ miniClassCourseList }) => !!miniClassCourseList.length)
          .map(({ id, classRoom }) => ({ id, name: classRoom }))
      : roomList.map(({ id, classRoom }) => ({ id, name: classRoom }));
  }

  getRoomHeaderList() {
    const headerList = [...this.realRoomList];
    return [
      ...headerList,
      ...Array.from({ length: this.props.roomList.length - this.realRoomList.length }).map((_, index) => ({
        id: `${new Date().getTime()}_${index}`,
        name: '',
      })),
    ];
  }

  getDatesOnCurrentWeek() {
    return Array.from({ length: 7 }).map((_, index) => ({
      weekName: WEEK_NAMES[index],
      date: moment(this.props.currentDate, DATE_FORMAT).startOf('week').add(index, 'day').format().slice(0, 10),
    }));
  }

  // 过滤出有数据的日期
  getDatesOnCurrentWeekHasData() {
    if (this.props.isCompressed) {
      return this.datesOnCurrentWeek.filter(({ date }) => {
        for (const key in this.formatRoomDataByDateAndRoomId) {
          if (Object.prototype.hasOwnProperty.call(this.formatRoomDataByDateAndRoomId, key) && key.includes(date)) {
            return true;
          }
        }
        return false;
      });
    } else {
      return this.datesOnCurrentWeek;
    }
  }

  get firstDateOnCurrentWeek() {
    return this.datesOnCurrentWeek[0];
  }

  get lastDateOnCurrentWeek() {
    return this.datesOnCurrentWeek[this.datesOnCurrentWeek.length - 1];
  }

  getFlattenRoomData() {
    const data = [];
    this.props.roomList.forEach(({ miniClassCourseList }) => {
      if (miniClassCourseList.length) {
        data.push(...miniClassCourseList);
      }
    });
    return data;
  }

  getFormatRoomDataByDateAndRoomId() {
    const data = {};

    // 用教室id+上课日期作为key存储数据
    this.flattenRoomData.forEach((room) => {
      const { courseDate, classroomId } = room;
      const key = `${classroomId}_${courseDate}`;
      if (!data[key]) {
        data[key] = [];
      }
      data[key].push({ ...room });
    });
    // 按开始时间排序
    _.forEach(data, (list, key) => {
      data[key] = _.sortBy(list, ({ courseTime }) => Utils.getTimeByMinutes(courseTime));
    });

    // 处理冲突
    _.forEach(data, (list) => {
      for (let i = 0; i < list.length; i++) {
        const source = list[i];
        for (let j = i + 1; j < list.length; ) {
          const target = list[j];
          const { courseTime: targetStartTime, courseEndTime: targetEndTime, miniClassCourseId } = target;

          // 不冲突直接跳过本次循环，继续下一个
          if (Utils.getTimeByMinutes(targetStartTime) >= Utils.getTimeByMinutes(source.courseEndTime)) {
            break;
          }

          if (!source._courseEndTime) {
            source._courseEndTime = source.courseEndTime;
          }
          source.courseEndTime =
            Utils.getTimeByMinutes(targetEndTime) > Utils.getTimeByMinutes(source.courseEndTime)
              ? targetEndTime
              : source.courseEndTime;
          if (!source.conflictList) {
            source.conflictList = [];
          }
          source.conflictList.push(miniClassCourseId); // 将冲突的id存储便于后续检索
          list.splice(j, 1); // 干掉
        }
      }
    });

    return data;
  }

  setRoomLocation() {
    const roomLocations = this.flattenRoomData
      .map((room) => {
        const { classroomId, courseDate, miniClassCourseId } = room;
        const match = this.formatRoomDataByDateAndRoomId[`${classroomId}_${courseDate}`].find(
          (item) => item.miniClassCourseId === miniClassCourseId,
        );
        if (!match) {
          return {
            ...room,
            isConflict: true,
          };
        }

        const locationX = _.findIndex(this.realRoomList, ({ id }) => id === classroomId) * CELL_WIDTH; // 计算横坐标
        const locationY =
          _.findIndex(this.datesOnCurrentWeekHasData, ({ date }) => date === match.courseDate) *
            TIME_RANGE_COUNT *
            CELL_HEIGHT +
          Utils.getHeightByMinutes(match.courseTime); // 计算纵坐标
        const totalCourseTime = Utils.getTimeByMinutes(match.courseEndTime) - Utils.getTimeByMinutes(match.courseTime);
        const height = (totalCourseTime * Utils.getHeightPerMinute() * 100000000 - 1 * 100000000) / 100000000; // 计算高度，乘与100000000是为了避免JS浮点计算错误
        return {
          ...match,
          height,
          locationX,
          locationY,
          isShowTeacherName: totalCourseTime >= 2 * 60, // 小于两个显示则不显示教师名称
        };
      })
      .filter(({ isConflict }) => !isConflict); // 过滤掉不冲突的数据
    this.setState({ roomLocations });
  }

  formatDate(dateTime) {
    return dateTime ? dateTime.date.split(/-/).slice(1).join('/') : '';
  }

  cancelAnimationFrame = () => {
    if (this.requestAnimationFrameId) {
      this.requestAnimationFrameId = null;
      window.cancelAnimationFrame(this.requestAnimationFrameId);
    }
  };

  formatTeacherName = (teacherName) => {
    return (teacherName || '').length > 4 ? `${teacherName.substr(0, 4)}...` : teacherName;
  };

  handleScroll = (e) => {
    e.persist();
    this.cancelAnimationFrame();
    this.requestAnimationFrameId = window.requestAnimationFrame(() => {
      this.headers.scrollLeft = e.target.scrollLeft;
      this.timeAreaWrapper.scrollTop = e.target.scrollTop;
    });
  };

  // 上一周
  handlePreWeek = () => {
    const preDate = moment(this.firstDateOnCurrentWeek.date).subtract(7, 'days').format(DATE_FORMAT).slice(0, 10);
    this.props.onChangeDate(preDate);
  };

  // 下一周
  handleNextWeek = () => {
    const nextDate = moment(this.lastDateOnCurrentWeek.date).add(1, 'days').format(DATE_FORMAT).slice(0, 10);
    this.props.onChangeDate(nextDate);
  };

  generateLeftWrapper() {
    return <div className={styles.leftWrapper}>{this.generateSideBar()}</div>;
  }

  generateSideBar() {
    return (
      <div className={styles.sideBar}>
        {this.generateSwitchWeekArea()}
        {this.generateDateRange()}
      </div>
    );
  }

  generateSwitchWeekArea() {
    return (
      <div className={[styles.header, styles.dateRange].join(' ')}>
        <Icon type="left" onClick={this.handlePreWeek} />
        {this.formatDate(this.firstDateOnCurrentWeek)}-{this.formatDate(this.lastDateOnCurrentWeek)}
        <Icon type="right" onClick={this.handleNextWeek} />
      </div>
    );
  }

  generateDateRange() {
    return (
      <ul className={styles.timeAreaWrapper} ref={(el) => (this.timeAreaWrapper = el)}>
        {this.datesOnCurrentWeekHasData.map(({ weekName, date }) => {
          return (
            <li key={date}>
              <div className={styles.timeAreaLeftWrapper}>
                <div className={styles.timeAreaDate}>
                  <p>{weekName}</p>
                  <p>{date.split(/-/).slice(1).join('/')}</p>
                </div>
              </div>
              <ul className={styles.timeAreaRightWrapper}>
                {TIMES.map((time) => (
                  <li key={time}>{time}</li>
                ))}
              </ul>
            </li>
          );
        })}
        <li className={styles.blankWeek}></li>
      </ul>
    );
  }

  generateRightWrapper() {
    return (
      <div className={styles.rightWrapper}>
        {this.generateHeaders()}
        {this.generateRoomArea()}
      </div>
    );
  }

  generateRoomArea() {
    return (
      <div
        className={[styles.rooms, 'roomViewOfMiniClassCourseRoom'].join(' ')}
        ref={(el) => (this.rooms = el)}
        onScroll={this.handleScroll}
      >
        {this.datesOnCurrentWeekHasData.map(({ date }) => {
          return TIMES.map((time) => {
            return (
              <div key={`${date}_${time}_blank`} className={styles.blankRoom}>
                <ul>
                  {this.roomHeaderList.map(({ id }) => (
                    <li key={`${date}_${time}_${id}_blank`} className={[styles.room].join(' ')}></li>
                  ))}
                </ul>
              </div>
            );
          });
        })}
        {this.state.roomLocations.map((room) => {
          const {
            height,
            planNum,
            locationX,
            locationY,
            enrollNum,
            courseTime,
            teacherName,
            courseEndTime,
            miniClassName,
            _courseEndTime,
            isShowTeacherName,
            conflictList = [],
            miniClassCourseId,
          } = room;
          const style = {
            height: `${height}px`,
            top: `${locationY}px`,
            left: `${locationX}px`,
          };
          const isConflict = !!conflictList.length;
          const list = [
            { ...room },
            ...this.flattenRoomData.filter(({ miniClassCourseId }) => conflictList.includes(miniClassCourseId)),
          ];
          return (
            <div
              key={miniClassCourseId}
              className={[styles.realRoom, isConflict ? styles.conflict : ''].join(' ')}
              style={style}
            >
              <p className={styles.classTime}>
                {courseTime}-{courseEndTime}
              </p>
              <p className={styles.className}>{miniClassName}</p>
              {isShowTeacherName ? (
                isConflict ? (
                  <p className={styles.conflictDesc}>
                    <img src={iconWarnIcon} />
                    &nbsp;<span>共有{conflictList.length + 1}门课程冲突</span>
                  </p>
                ) : (
                  <p className={styles.teacherName}>
                    <img src={iconPersonIcon} />
                    &nbsp;<span>{this.formatTeacherName(teacherName)}</span>&nbsp;
                    <span>
                      {enrollNum}/{planNum}
                    </span>
                  </p>
                )
              ) : null}
              <RoomViewOfMiniClassCourseDetail isConflict={isConflict} list={list} />
            </div>
          );
        })}
      </div>
    );
  }

  generateHeaders() {
    return (
      <ul className={styles.headers} ref={(el) => (this.headers = el)}>
        {this.roomHeaderList.map(({ name, id }) => {
          return (
            <li
              key={id}
              className={[styles.header].join(' ')}
              dangerouslySetInnerHTML={{ __html: name ? name : '&nbsp;' }}
            ></li>
          );
        })}
      </ul>
    );
  }

  render() {
    return this.isEmpty ? (
      <DefaultTip message={''} />
    ) : (
      <div className={styles.wrapper}>
        {this.generateLeftWrapper()}
        {this.generateRightWrapper()}
      </div>
    );
  }

  static propTypes = {
    roomList: PropTypes.array.isRequired,
    currentDate: PropTypes.string.isRequired,
    isCompressed: PropTypes.bool.isRequired,
    onChangeDate: PropTypes.func.isRequired,
  };
}
