import React from 'react';
import classNames from 'classnames';
import styles from './index.less';
import Icon from 'COMPONENT/Icon';
import HoverCard from '../Hovercard/index';
import { teachingTypeEnum } from 'CONST/dicts/course';

const teacherType = ['面授', '在线A', '在线B', '自习', 'AI录播课'];

class Node extends React.Component {
  constructor(props) {
    super(props);
    this.state = {};
  }

  // 获取时间冲突的课程
  getNoteList = (courses, type) => {
    let tempArray = [];
    for (let i = 0; i < courses.length; i++) {
      let array = [];
      for (let j = i; j < courses.length; j++) {
        if (courses[i].courseEndTime <= courses[j].courseTime) {
          continue;
        } else if (courses[i].courseTime >= courses[j].courseEndTime) {
          continue;
        } else {
          if (
            [teachingTypeEnum.FACE_TO_FACE, teachingTypeEnum.ANSWER, teachingTypeEnum.AI].includes(
              courses[j].teachingType,
            ) ||
            (courses[j].teachingType === teachingTypeEnum.LIVE && type === 'teacher')
          ) {
            array.push(courses[j]);
          }
        }
      }
      if (tempArray.length && array.length === tempArray[tempArray.length - 1].length) {
        let tempNode = tempArray[tempArray.length - 1];

        for (let k = 0; k < array.length; k++) {
          if (array[k].courseId !== tempNode[k].courseId) {
            tempArray.push(array.length > 1 ? array : []);
            break;
          }
        }
      } else {
        tempArray.push(array.length > 1 ? array : []);
      }
    }

    // 返回时间上冲突的课程
    return [...new Set(tempArray.flat())];
  };

  render() {
    const { node, top, height, type, editCourse, changeCourse, customCourseConflictNum } = this.props;
    console.log({ props: this.props, node });
    if (Array.isArray(node) === true) {
      const course = node[0];
      let conflictNums = node.length;
      let { courseTime, courseEndTime, courseName, miniClassName } = course;
      let courseTimes = node.map((course) => {
        return course.courseTime;
      });
      let courseEndTimes = node.map((course) => {
        return course.courseEndTime;
      });
      courseTime = new Date(Math.min(...courseTimes)).toTimeString().substr(0, 5);
      courseEndTime = new Date(Math.max(...courseEndTimes)).toTimeString().substr(0, 5);

      // 如果一个节点有两节课就不用走getNoteList函数了，getNoteList是父组件的方法，还是有些问题的
      let tempNode;

      let faceAndAnswer = 0;

      if (node.length > 2) {
        tempNode = this.getNoteList(node, type);
        faceAndAnswer = tempNode.length > 1 ? tempNode.length : 0;
      } else {
        tempNode = node;
        if (node.length <= 1) {
          faceAndAnswer = 0;
        } else {
          if (tempNode[0].courseEndTime <= tempNode[1].courseTime) {
          } else if (tempNode[0].courseTime >= tempNode[1].courseEndTime) {
          } else {
            if (
              [teachingTypeEnum.FACE_TO_FACE, teachingTypeEnum.ANSWER, teachingTypeEnum.AI].includes(
                tempNode[0].teachingType,
              ) ||
              (tempNode[0].teachingType === teachingTypeEnum.LIVE && type === 'teacher')
            ) {
              faceAndAnswer++;
            }
            if (
              [teachingTypeEnum.FACE_TO_FACE, teachingTypeEnum.ANSWER, teachingTypeEnum.AI].includes(
                tempNode[1].teachingType,
              ) ||
              (tempNode[1].teachingType === teachingTypeEnum.LIVE && type === 'teacher')
            ) {
              faceAndAnswer++;
            }
            faceAndAnswer = faceAndAnswer > 1 ? 2 : 0;
          }
        }
      }

      let showCourseNum = 0;
      if (faceAndAnswer > 1 || customCourseConflictNum >= 1) {
        showCourseNum = faceAndAnswer + customCourseConflictNum;
        showCourseNum = showCourseNum === 1 ? 2 : showCourseNum;
      }

      return (
        <div
          className={styles.node + ' ' + styles.conflictNode}
          style={{ top: top + 'px', height: height + 'px', 'border-color': showCourseNum ? '#FF5B5B' : '#6D67FF' }}
        >
          <div>
            <div className={styles.center}>
              <span className={styles.time} style={{ color: showCourseNum ? '#FF5B5B' : '#6D67FF' }}>
                {courseTime}
              </span>
              <span className={styles.time} style={{ color: showCourseNum ? '#FF5B5B' : '#6D67FF' }}>
                -
              </span>
              <span className={styles.time} style={{ color: showCourseNum ? '#FF5B5B' : '#6D67FF' }}>
                {courseEndTime}
              </span>
              <Icon type="information_cricle" style={{ marginLeft: '2px' }} />
            </div>
            <p className={styles.courseName + ' ' + styles.courseInfo}>{miniClassName}</p>
            <p className={styles.teachingType}>
              上课方式: {course.teachingType ? teacherType[course.teachingType - 1] : ''}
            </p>
          </div>
          <p className={styles.conflictMessage}>
            共有{showCourseNum ? showCourseNum : conflictNums}门{showCourseNum ? '课程冲突' : ''}
          </p>
          <div className={styles.hoverCard}>
            <HoverCard
              node={node}
              type={type}
              editCourse={editCourse}
              changeCourse={changeCourse}
              conflictNum={showCourseNum}
              customCourseConflictNum={customCourseConflictNum}
            />
          </div>
        </div>
      );
    } else {
      let { courseTime, courseEndTime, courseName, blCampusName, courseNum, miniClassName, teachingType } = node;
      courseTime = new Date(courseTime).toTimeString().substr(0, 5);
      courseEndTime = new Date(courseEndTime).toTimeString().substr(0, 5);
      return (
        <div
          className={classNames(styles.node, { [styles.conflictNode]: customCourseConflictNum > 0 })}
          style={{ top: top + 'px', height: height + 'px' }}
        >
          <div>
            <span className={styles.time}>{courseTime}</span>
            <span className={styles.time}>-</span>
            <span className={styles.time}>{courseEndTime}</span>
            <p className={styles.courseName + ' ' + styles.courseInfo}>{miniClassName}</p>
          </div>
          <p>
            <p className={styles.courseInfo}>
              {courseNum}({blCampusName})
            </p>
            <p className={styles.teachingType}>上课方式: {teachingType ? teacherType[teachingType - 1] : ''}</p>
          </p>
          {customCourseConflictNum > 0 && (
            <p className={styles.conflictMessage}>共有{1 + customCourseConflictNum}门课程冲突</p>
          )}
          <div className={styles.hoverCard}>
            <HoverCard
              node={node}
              type={type}
              editCourse={editCourse}
              changeCourse={changeCourse}
              customCourseConflictNum={customCourseConflictNum}
            />
          </div>
        </div>
      );
    }
  }
}

export default Node;
