import React from 'react';
import utils from '../utils';
import styles from './index.less';
import moment from 'moment';
import Node from './Node';

class SchedulePanel extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      customCourseArr: [],
    };
    this.timer = null;
  }

  divideConflictCourses = (conflictCourses) => {
    if (conflictCourses.length <= 1) {
      return conflictCourses;
    }

    conflictCourses = conflictCourses.sort((a, b) =>
      a.courseTime !== b.courseTime ? a.courseTime - b.courseTime : a.courseEndTime - b.courseEndTime,
    );

    let result = [];
    let beginIndex = 0;

    for (let i = 1; i < conflictCourses.length; i += 1) {
      if (conflictCourses[i].courseTime > conflictCourses[i - 1].courseEndTime) {
        result.push(conflictCourses.slice(beginIndex, i));
        beginIndex = i;
      }
    }

    if (beginIndex !== conflictCourses.length) {
      result.push(conflictCourses.slice(beginIndex, conflictCourses.length));
    }

    return result;
  };

  // 每一天的课程分块
  getNoteList = (courses) => {
    let tempArray = [];
    for (let i = 0; i < courses.length; i++) {
      let array = [courses[i]];
      for (let j = 0; j < courses.length; j++) {
        if (courses[i].courseEndTime <= courses[j].courseTime) {
          continue;
        } else if (courses[i].courseTime >= courses[j].courseEndTime) {
          continue;
        } else {
          array.push(courses[j]);
        }
      }
      tempArray.push(array);
    }
    // 没有冲突的课程
    const noConconflictCourses = tempArray
      .filter((note) => {
        return note.length === 2;
      })
      .map((array) => {
        return array.pop();
      });
    // 存在冲突的课程
    const conconflictCourses = courses.filter((course) => {
      return noConconflictCourses.includes(course) === false;
    });
    // 将存在冲突的课程分块
    let noteList = this.divideConflictCourses(conconflictCourses).concat(noConconflictCourses);
    return noteList;
  };

  // 拿到小班课程的时间段，去请求是否存在个性化课程冲突
  toCheckStudentCourseConflict = (props) => {
    if (props.checkStudentCourseConflict) {
      clearTimeout(this.timer);
      const { courseSchedules } = props;

      this.timer = setTimeout(() => {
        const nodeLists = courseSchedules.map((oneDayCourses) => {
          let nodeList = this.getNoteList(oneDayCourses);
          return nodeList;
        });

        const timeLists = nodeLists
          .map((nodeList) => {
            const timeList = nodeList.map((node) => {
              if (Array.isArray(node)) {
                let minCourseTime = moment(
                  Math.min(
                    ...node.map((course) => {
                      return course.courseTime;
                    }),
                  ),
                );
                let maxCourseEndTime = moment(
                  Math.max(
                    ...node.map((course) => {
                      return course.courseEndTime;
                    }),
                  ),
                );
                return {
                  startTime: minCourseTime.format('YYYY-MM-DD HH:mm'),
                  endTime: maxCourseEndTime.format('YYYY-MM-DD HH:mm'),
                };
              } else {
                let minCourseTime = moment(node.courseTime);
                let maxCourseEndTime = moment(node.courseEndTime);
                return {
                  startTime: minCourseTime.format('YYYY-MM-DD HH:mm'),
                  endTime: maxCourseEndTime.format('YYYY-MM-DD HH:mm'),
                };
              }
            });
            return timeList;
          })
          .filter((item) => {
            return item.length > 0;
          });

        props.checkStudentCourseConflict(timeLists, (data) => {
          if (data) {
            const { timeRanges } = data;
            if (timeRanges.length > 0) {
              const customCourseArr = timeRanges.filter((item) => item.otmCourseNum > 0 || item.otoCourseNum > 0); // 是否存在个性化冲突课程
              this.setState({
                customCourseArr: customCourseArr,
              });
            }
          } else {
            this.setState({
              customCourseArr: [],
            });
          }
        });
      }, 1500);
    }
  };

  // 对比 课表的一段时间范围内，是否存在个性化课程冲突。
  contrastCustomCourseData = (_startTime, _endTime) => {
    const { customCourseArr } = this.state;
    console.log({ customCourseArr });
    let sum = 0;
    if (customCourseArr.length > 0) {
      customCourseArr.forEach((item) => {
        let { startTime, endTime, otmCourseNum, otoCourseNum } = item;
        if (moment(_startTime).isSame(startTime) && moment(_endTime).isSame(endTime)) {
          console.log('小班与个性化课程冲突了!!');
          sum = otmCourseNum + otoCourseNum;
        }
      });
    }
    return sum;
  };

  componentWillReceiveProps(nextProps) {
    this.toCheckStudentCourseConflict(nextProps);
  }

  componentDidMount() {
    if (this.props.checkStudentCourseConflict) {
      this.toCheckStudentCourseConflict(this.props);
    }
  }

  render() {
    const panelTimes = utils.getPanelTimes();
    // courseSchedules 是一个包含7天课程的数组，7个项分别是7个数组
    const { courseSchedules, type, editCourse, changeCourse } = this.props;
    const timeLungime = new Date('2018-07-17T22:00:00').getTime() - new Date('2018-07-17T06:00:00').getTime();
    console.log('🚀 ~ file: index.js ~ line 152 ~ SchedulePanel ~ render ~ courseSchedules', courseSchedules);

    return (
      <div className={styles.wrap} style={{ height: 'calc(100% - 96px)' }}>
        <div className={styles.main}>
          <div className={styles.timeLine}>
            {panelTimes.map((time, i) => {
              return (
                <div key={time} className={styles.time}>
                  {time}
                </div>
              );
            })}
          </div>
          <div className={styles.content}>
            {courseSchedules.map((oneDayCourses, index) => {
              let nodeList = this.getNoteList(oneDayCourses);
              console.log('🚀 ~ file: index.js ~ line 172 ~ SchedulePanel ~ courseSchedules.map ~ nodeList', nodeList);

              return (
                <div className={styles.col} key={index}>
                  {nodeList.map((node, index) => {
                    console.log('🚀 ~ file: index.js ~ line 178 ~ SchedulePanel ~ nodeList.map ~ node', node);

                    if (Array.isArray(node) === true) {
                      let minCourseTime = moment(
                        Math.min(
                          ...node.map((course) => {
                            return course.courseTime;
                          }),
                        ),
                      );
                      let maxCourseEndTime = moment(
                        Math.max(
                          ...node.map((course) => {
                            return course.courseEndTime;
                          }),
                        ),
                      );
                      let height = (maxCourseEndTime.diff(minCourseTime) / timeLungime) * 568;
                      let top =
                        (minCourseTime.diff(
                          moment(maxCourseEndTime).set({ hour: 6, minute: 0, second: 0, millisecond: 0 }).valueOf(),
                        ) /
                          timeLungime) *
                        568;
                      const cfCourseNum = this.contrastCustomCourseData(minCourseTime, maxCourseEndTime);
                      return (
                        <Node
                          top={top}
                          height={height}
                          node={node}
                          key={index}
                          type={type}
                          editCourse={editCourse}
                          changeCourse={changeCourse}
                          customCourseConflictNum={cfCourseNum}
                        ></Node>
                      );
                    } else {
                      let minCourseTime = node.courseTime;
                      let maxCourseEndTime = node.courseEndTime;
                      let height = ((maxCourseEndTime - minCourseTime) / timeLungime) * 568;
                      let top =
                        ((minCourseTime -
                          moment(maxCourseEndTime).set({ hour: 6, minute: 0, second: 0, millisecond: 0 }).valueOf()) /
                          timeLungime) *
                        568;
                      const cfCourseNum = this.contrastCustomCourseData(minCourseTime, maxCourseEndTime);
                      return (
                        <Node
                          top={top}
                          height={height}
                          node={node}
                          key={index}
                          type={type}
                          editCourse={editCourse}
                          changeCourse={changeCourse}
                          customCourseConflictNum={cfCourseNum}
                        ></Node>
                      );
                    }
                  })}
                </div>
              );
            })}
          </div>
          <div className={styles.right} />
        </div>
      </div>
    );
  }
}

export default SchedulePanel;
