import React from 'react';
import { connect } from 'dva';
import { Tabs, Cascader } from 'antd';
import styles from './index.less';
import loaders from './loaders';
import Page from 'COMPONENT/Page';
import SearchBar from 'COMPONENT/SearchBar';
import DatePicker from 'COMPONENT/DatePicker';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import Validator from 'COMPONENT/Validator';
import ConflictList from './ConflictList';
import ScheduleHeader from './ScheduleHeader';
import SchedulePanel from './SchedulePanel';
import { Switch } from 'antd';
import RoomViewOfMiniClassCourseList from './RoomViewOfMiniClassCourseList';
import moment from 'moment';
import { DATE_FORMAT } from 'COMPONENT/utils/form';
import ShowMessage from 'COMPONENT/utils/globalTips/showMessage';
import DefaultTip from './DefaultTip';
import EditCourse from './EditCourse';
import ChangeCourse from './ChangeCourse';
import DropdownMenu from 'COMPONENT/DropdownMenu';
import { TabsNames, CustomerTypes } from './utils';
import { Button } from 'antd/lib/radio';
import { Button as RealButton } from 'antd';
import { errorsHandle } from 'SRC/utils/common';

import RoomViewOfMiniClassCourseListUtils from './RoomViewOfMiniClassCourseList/utils';

const initSearchKeys = {
  blCampusId: '',
  campusIds: null,
  type: null,
  date: moment().format().slice(0, 10),
  classScheduleTypeValue: null,
  startDate: moment().startOf('week').format().slice(0, 10),
  endDate: moment().endOf('week').format().slice(0, 10),
  courseStatus: ['NEW', 'CHARGED', 'TEACHER_ATTENDANCE'],
  hasCourseSchedules: false,
  classScheduleTypeValueDict: null,
  isNotData: false,
};

const cfInitSearchKeys = {
  cfDate: moment().format().slice(0, 10),
  cfScheduleDate: moment().format().slice(0, 10),
  cfBranchCampusIds: [], // 冲突查询 - [分公司id, 学习中心id]
  cfCampusIds: [], // 冲突查询 - 学习中心ids[]
  cfType: null,
  cfClassScheduleTypeValue: null,
  cfStartDate: moment().startOf('week').format().slice(0, 10),
  cfEndDate: moment().endOf('week').format().slice(0, 10),
  cfHasCourseSchedules: false,
  cfHasConflictName: false,
  cfListOp: {
    classroomManageVos: [],
    studentVos: [],
    teachers: [],
  },
  cfTempSearchType: null,
};

class ScheduleManage extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: true,
      campusDict: null,
      cfCampusDict: [],
      classScheduleTypeDict: null,
      filterConditionDict: null,
      classScheduleTypeValueDict: null,
      teacherDict: null,
      studentDict: null,
      classroomDict: null,
      cfTeacherDict: null,
      cfClassroomDict: null,
      cfStudentDict: null,

      currentTab: TabsNames.COURSE_SUMMARY,
      date: moment().format().slice(0, 10),
      cfDate: moment().format().slice(0, 10),
      cfScheduleDate: moment().format().slice(0, 10),
      cfBranchCampusIds: [], // 冲突查询 - [分公司id, 学习中心id]
      cfCampusIds: [], // 冲突查询 - 学习中心ids[]
      blCampusId: null, // 学习中心
      campusIds: null,

      cfType: null, // 课表类型 STUDENT|TEACHER|CLASSROOM
      type: null, // 课表类型 STUDENT|TEACHER|CLASSROOM
      classScheduleTypeValue: null, // 学生/老师/教师
      cfClassScheduleTypeValue: null,
      courseStatus: ['NEW', 'CHARGED', 'TEACHER_ATTENDANCE'], // 课程状态
      title: null,
      list: [],
      csDate: moment().format().slice(0, 10),
      currentCsDate: moment().format().slice(0, 10),
      startDate: moment().startOf('week').format().slice(0, 10),
      endDate: moment().endOf('week').format().slice(0, 10),
      cfStartDate: moment().startOf('week').format().slice(0, 10),
      cfEndDate: moment().endOf('week').format().slice(0, 10),
      cfScheduleStartDate: moment().startOf('week').format().slice(0, 10),
      cfScheduleEndDate: moment().endOf('week').format().slice(0, 10),

      isCompressed: true,

      isCourseSummaryExporting: false,

      courseSchedules: [],
      cfCourseSchedules: [],
      hasCourseSchedules: false,
      cfHasCourseSchedules: false,
      cfHasConflictName: false,
      dateArray: [], //一周7天的日期数组 如 2018-08-02
      cfDateArray: [], //一周7天的日期数组 如 2018-08-02
      current: null, //当前查询课表的归属者
      checking: false,
      cfChecking: false,
      csChecking: false,
      errors: {},
      cfErrors: {},
      csErrors: {},
      exportStatus: '导出冲突名单',
      isEditCourse: false,
      miniClassId: '',
      courseNum: '',
      isNotData: false,
      isChangeCourse: false, // 是否正在调课(学生)
      studentIds: [], // 为[]则非批量调课

      roomViewOfMiniClassCourseKey: 1,
      roomViewOfMiniClassCourseList: [], // 课表总览数据

      // 冲突列表
      cfListOp: {
        classroomManageVos: [],
        studentVos: [],
        teachers: [],
      },

      tempSearchType: null, // 误删！！每次点击查询都保存当前查询的类型（用于卡片展示的编辑按钮判断）
      cfTempSearchType: null,
    };

    this.onCampusChange = this.onCampusChange.bind(this);
  }

  componentDidMount() {
    this.initDict();
    this.initContent();
  }

  initDict = () => {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['grade'],
        callback: (a) => {
          setTimeout(() => this.setState({ gradeIds: this.props.gradeDict.map(({ val }) => val) }), 1000);
        },
      },
    });
  };

  isCourseSummaryTab = () => {
    return this.state.currentTab === TabsNames.COURSE_SUMMARY;
  };

  handleChangeCourseSummaryCompress = (isCompressed) => {
    this.setState({
      isCompressed,
      roomViewOfMiniClassCourseKey: this.state.roomViewOfMiniClassCourseKey + 1,
    });
  };

  isCourseTab = () => {
    return this.state.currentTab === TabsNames.COURSE;
  };

  isConflictTab = () => {
    return this.state.currentTab === TabsNames.CONFLICT;
  };

  initContent = async () => {
    const { dispatch, pageParams } = this.props;
    const campusDict = await Promise.resolve(loaders.loadCampusDict({ orgType: 'CAMPUS' }));
    const [classScheduleTypeDict, filterConditionDict] = await Promise.all([
      loaders.loadClassScheduleTypeDict(),
      loaders.loadFilterConditionDict(),
    ]);

    let branchCampusDict = (await loaders.getBranchCampusDict()) || [];

    const handleOptions2Children = (options = []) => {
      if (!Array.isArray(options)) return [];
      return options.map((option) => ({
        value: option.val,
        label: option.label || option.text,
        children:
          Array.isArray(option.options) && option.options.length > 0
            ? handleOptions2Children(option.options)
            : undefined,
      }));
    };

    this.setState({
      campusDict,
      cfCampusDict: handleOptions2Children(branchCampusDict),
      classScheduleTypeDict,
      filterConditionDict,
      loading: false,
    });
    if (pageParams === undefined || pageParams.isMultiChangeCourse === undefined) {
      this.setState({ isChangeCourse: false });
    } else {
      const { studentIds, miniClassId, isMultiChangeCourse } = pageParams;
      this.setState({ isChangeCourse: true, miniClassId, classScheduleTypeValue: studentIds[0], studentIds });
    }
  };

  /**
   * tab切换事件
   */
  handelTabsChange = (key) => {
    this.setState({
      currentTab: key,
    });
  };

  /**
   * 关闭换课弹窗
   */
  closeChangeCoursesModal = () => {
    this.setState({ isChangeCourse: false, loading: false, studentIds: [] });

    this.props.dispatch({
      type: 'main/setPageParams',
      payload: {
        path: '/MiniClassCourseManage',
        params: null,
      },
    });
  };

  /**
   * 选中学习中心改变
   * @param value
   */
  async onCampusChange(value) {
    const { startDate, endDate, type } = this.state;

    this.setState({
      campusIds: value,
      classScheduleTypeValue: null,
      courseStatus: ['NEW', 'CHARGED', 'TEACHER_ATTENDANCE'],
      hasCourseSchedules: false,
      classScheduleTypeValueDict: null,
      isNotData: false,
    });

    if (value) {
      this.setState({ loading: true });
      const [classroomDict, studentDict, teacherDict] = await Promise.all([
        loaders.loadClassroomDict({ campusIds: value }),
        loaders.loadStudentDict({ campusIds: value, endDate, startDate }),
        loaders.loadTeacherDict({ campusIds: value, startDate, endDate }),
      ]).catch((e) => {
        this.setState({ loading: false });
      });
      this.changeClassScheduleTypeValueDict(type, teacherDict, studentDict, classroomDict);
      this.setState({
        loading: false,
        classroomDict,
        studentDict,
        teacherDict,
      });
    }
  }

  // 课表类型与课表类型值 级联
  onChangeclassScheduleTypeValue = (value) => {
    if (value === 'teacher') {
      this.setState({ classScheduleTypeValueDict: this.state.teacherDict });
    } else if (value === 'student') {
      this.setState({ classScheduleTypeValueDict: this.state.studentDict });
    } else if (value === 'classroom') {
      this.setState({ classScheduleTypeValueDict: this.state.classroomDict });
    } else {
      this.setState({ classScheduleTypeValueDict: null });
    }
    this.setState({ type: value, isNotData: false });
  };

  csDateChange = (csDate) => {
    this.setState({ csDate });
  };

  handleChangeCsDate = (csDate) => {
    this.setState({ csDate }, () => {
      this.csSearch();
    });
  };

  onCsCampusChange = (blCampusId) => {
    this.setState({ blCampusId });
  };

  onCsGradeChange = (gradeIds) => {
    this.setState({ gradeIds });
  };

  // 改变日期
  dateChange = async (value) => {
    const { campusIds, type, classroomDict } = this.state;
    let startDate = moment(value, DATE_FORMAT).startOf('week').format().slice(0, 10);
    let endDate = moment(value, DATE_FORMAT).endOf('week').format().slice(0, 10);
    this.setState({ date: value, startDate, endDate });

    if (campusIds) {
      this.setState({ loading: true });

      const [studentDict, teacherDict] = await Promise.all([
        loaders.loadStudentDict({ campusIds, endDate, startDate }),
        loaders.loadTeacherDict({ campusIds, startDate, endDate }),
      ]).catch((e) => {
        this.setState({ loading: false });
      });

      this.changeClassScheduleTypeValueDict(type, teacherDict, studentDict, classroomDict);
      this.setState({ studentDict, teacherDict, loading: false });
    }
  };

  // 改变日期
  cfDateChange = async (value) => {
    if (!value) {
      this.setState({
        cfDate: value,
        cfStartDate: null,
        cfEndDate: null,
        cfHasConflictName: false,
        cfHasCourseSchedules: false,
      });
    } else {
      let startDate = moment(value, DATE_FORMAT).startOf('week').format().slice(0, 10);
      let endDate = moment(value, DATE_FORMAT).endOf('week').format().slice(0, 10);
      this.setState({
        cfDate: value,
        cfStartDate: startDate,
        cfEndDate: endDate,
        cfHasConflictName: false,
        cfHasCourseSchedules: false,
      });
      const { cfCampusIds } = this.state;
      if (cfCampusIds && cfCampusIds.length > 0) {
        const [cfClassroomDict, cfTeacherDict] = await Promise.all([
          loaders.loadClassroomDict({ campusIds: cfCampusIds }),
          loaders.loadTeacherDict({ campusIds: cfCampusIds, startDate: startDate, endDate: endDate }),
        ]).catch((e) => {
          console.log(e);
        });
        this.setState({
          cfClassroomDict,
          cfTeacherDict,
        });
      }
    }
  };

  /**
   * 获取冲突名单
   */
  searchConflictNames = async () => {
    const params = this.formatSearchParam();

    this.setState({ loading: true });
    const data = await loaders
      .checkMiniClassCourseConflictApi({
        ...params,
        checkTeacher: true,
        checkClassRoom: true,
        checkStudent: true,
      })
      .catch((e) => {
        this.setState({ loading: false });
      });

    this.setState({
      loading: false,
      cfHasConflictName: true,
      current: null,
      cfListOp: {
        classroomManageVos: data.classroomManageVos,
        studentVos: data.studentVos,
        teachers: data.teachers,
      },
    });
  };

  checkStudentCourseConflict = async (nodeLists, callback) => {
    const { cfType, cfClassScheduleTypeValue } = this.state;
    if (cfType === CustomerTypes.student && this.isConflictTab() && nodeLists.length > 0) {
      const data = await loaders.checkStudentCourseConflictApi({
        studentId: cfClassScheduleTypeValue,
        timeRanges: nodeLists.flat(),
      });
      callback && callback(data.data);
    } else {
      callback && callback(undefined);
    }
  };

  getConflictTitle = (type) => {
    let title;
    switch (type) {
      case CustomerTypes.student:
        title = '本周学生冲突名单:';
        break;
      case CustomerTypes.teacher:
        title = '本周老师冲突名单:';
        break;
      case CustomerTypes.classroom:
        title = '本周教室冲突名单:';
        break;
      default:
        title = '本周冲突名单:';
        break;
    }
    return title;
  };

  // 冲突名单点击搜索
  onClickName = async (id, customerType) => {
    await this.setState((prevState) => ({
      cfClassScheduleTypeValue: id,
      current: id,
      cfType: customerType,
      cfScheduleStartDate: prevState.cfStartDate,
      cfScheduleEndDate: prevState.cfEndDate,
      cfTempSearchType: customerType.toLocaleLowerCase(), // 与 tempSearchType 一样，用小写。 src/pages/course/miniClassManage/ScheduleManage/SchedulePanel/Hovercard/index.js
    }));
    this.searchConflictCourse();
  };

  /**
   * 获取冲突课程
   */
  searchConflictCourse = async () => {
    const { classScheduleTypeValue, cfClassScheduleTypeValue } = this.state;

    if (this.isCourseTab() && !classScheduleTypeValue) {
      this.setState({ hasCourseSchedules: false });
      return;
    } else if (this.isConflictTab() && !cfClassScheduleTypeValue) {
      this.setState({ cfHasCourseSchedules: false });
      return;
    }

    const params = this.formatSearchParam(this.isConflictTab());

    this.setState({ loading: true });
    const data = await loaders.loadCoursesSchedules(params).catch((e) => {
      this.setState({ loading: false });
    });

    const { courseSchedules, dateArray } = data;

    if (this.isCourseTab()) {
      this.setState({ courseSchedules, hasCourseSchedules: true, dateArray, loading: false });
    } else {
      this.setState({
        cfCourseSchedules: courseSchedules,
        cfHasCourseSchedules: true,
        cfDateArray: dateArray,
        loading: false,
      });
    }
  };

  /**
   * 搜索
   *
   * @return {Promise<void>}
   */
  search = async () => {
    const { checking, errors } = this.state;

    if (!checking) {
      this.setState({ checking: true });
    }

    if (Object.values(errors).find((v) => v)) {
      this.setState({ hasCourseSchedules: false });
      return;
    }

    const { type } = this.state;

    this.setState({ loading: true, tempSearchType: type });

    await Promise.all([this.searchConflictCourse()]).catch((e) => {
      this.setState({ loading: false });
    });

    this.setState({ loading: false });
  };

  /**
   * 冲突查询
   *
   * @return {Promise<void>}
   */
  cfSearch = async () => {
    const { cfChecking, cfErrors } = this.state;

    if (!cfChecking) {
      this.setState({ cfChecking: true });
    }

    if (Object.values(cfErrors).find((v) => v)) {
      this.setState({ cfHasConflictName: false, cfHasCourseSchedules: false });
      return;
    }

    this.setState({ cfHasCourseSchedules: false });
    this.setState({ loading: true });

    await Promise.all([
      this.searchConflictNames(),
      // this.searchConflictCourse()
    ]).catch((e) => {
      this.setState({ loading: false });
    });

    this.setState({ loading: false });
  };

  /**
   * 格式化搜索请求参数
   */
  formatSearchParam = (isCfSchedule = false) => {
    const {
      startDate,
      endDate,
      campusIds,
      type,
      csDate,
      gradeIds,
      blCampusId,
      classScheduleTypeValue,
      cfClassScheduleTypeValue,
      courseStatus,
      cfType,
      cfStartDate,
      cfEndDate,
      cfScheduleStartDate,
      cfScheduleEndDate,
      cfCampusIds,
    } = this.state;

    if (this.isCourseTab()) {
      return {
        startDate,
        endDate,
        campusIds,
        timeTableType: CustomerTypes[type],
        objId: classScheduleTypeValue,
        courseStatus,
      };
    } else if (this.isCourseSummaryTab()) {
      const courseDateStart = moment(csDate, DATE_FORMAT).startOf('week').format().slice(0, 10);
      const courseDateEnd = moment(csDate, DATE_FORMAT).endOf('week').format().slice(0, 10);
      return {
        gradeIds,
        blCampusId,
        courseDateEnd,
        courseDateStart,
      };
    } else {
      let _startDate = cfStartDate;
      let _endDate = cfEndDate;
      if (isCfSchedule) {
        // 冲突查询中 - 课表的 开始时间&结束时间，是独立的，与筛选框的日期没有关联（除了初始状态外）
        _startDate = cfScheduleStartDate;
        _endDate = cfScheduleEndDate;
      }
      return {
        startDate: _startDate,
        endDate: _endDate,
        campusIds: cfCampusIds,
        timeTableType: cfType,
        objId: cfClassScheduleTypeValue,
        courseStatus: ['NEW', 'CHARGED', 'TEACHER_ATTENDANCE'],
      };
    }
  };

  /**
   * 格式化导出报表文件名
   *
   * @param exportType {number} 1: 冲突名单，2：课表明细
   */
  formatExportFileName = (exportType = 1, customerType) => {
    const { campusIds, cfCampusIds, cfBranchCampusIds, cfCampusDict, campusDict, startDate, type } = this.state;

    let _type;
    let _campusIds;

    let isOnlyBranch = false;
    let campusNames = [];
    let campusName = '';

    if (this.isCourseTab()) {
      _type = CustomerTypes[type];
      _campusIds = campusIds;
    } else {
      _type = customerType;
      _campusIds = cfCampusIds;
      const [cf_branchId, cf_campusIds] = cfBranchCampusIds;
      // 只选择了分公司，导出文件的名称只显示分公司名称
      if (!cf_campusIds) {
        const branchName = cfCampusDict.find((item) => item.value === cf_branchId).label;
        campusName = branchName;
        isOnlyBranch = true;
      }
    }

    if (!isOnlyBranch) {
      campusDict.forEach((campus) => {
        if (_campusIds.indexOf(campus.val) > -1) {
          campusNames.push(campus.text);
        }
      });
      campusName = campusNames.join(',');
      if (campusName.length > 50) {
        campusName = campusName.successbstring(0, 50) + '...';
      }
    }

    const yearName = moment(startDate).format().substring(0, 4);
    const title = _type === 'TEACHER' ? '老师' : _type === 'STUDENT' ? '学生' : '教室';
    const fileName = yearName + campusName + title + (exportType === 1 ? '冲突名单' : '课表明细');

    return fileName;
  };

  // 更改课表类型值字典classScheduleTypeValueDict
  changeClassScheduleTypeValueDict = (type, teacherDict, studentDict, classroomDict) => {
    switch (type) {
      case 'teacher':
        this.setState({ classScheduleTypeValueDict: teacherDict });
        break;
      case 'student':
        this.setState({ classScheduleTypeValueDict: studentDict });
        break;
      case 'classroom':
        this.setState({ classScheduleTypeValueDict: classroomDict });
        break;
    }
  };

  // 改变时间并搜索冲突课程
  dateChangeAndSearchConflictCourse = async (clickType) => {
    let { currentTab, campusIds, type, classroomDict, date, cfScheduleDate } = this.state;

    // let _date = this.isCourseTab() ? date : cfScheduleDate;
    let _date = date;
    if (clickType === 'preWeek') {
      // 上一周
      _date = moment(_date).subtract(7, 'days').format().slice(0, 10);
    } else {
      // 下一周
      _date = moment(_date).add(7, 'days').format().slice(0, 10);
    }

    let startDate = moment(_date, DATE_FORMAT).startOf('week').format().slice(0, 10);
    let endDate = moment(_date, DATE_FORMAT).endOf('week').format().slice(0, 10);

    if (this.isCourseTab()) {
      if (campusIds) {
        const [studentDict, teacherDict] = await Promise.all([
          loaders.loadStudentDict({ campusIds, endDate, startDate }),
          loaders.loadTeacherDict({ campusIds, startDate, endDate }),
        ]).catch((e) => {
          this.setState({ loading: false });
        });
        this.changeClassScheduleTypeValueDict(type, teacherDict, studentDict, classroomDict);
        this.setState({ teacherDict, studentDict, date: _date, startDate, endDate }, this.search);
      } else {
        this.setState({ date: date, startDate, endDate }, this.search);
      }
    } else if (this.isConflictTab()) {
      this.setState(
        { cfScheduleDate: _date, cfScheduleStartDate: startDate, cfScheduleEndDate: endDate },
        this.searchConflictCourse,
      );
    }
  };

  // 搜索上一周课程
  onClickPreWeek = () => {
    this.dateChangeAndSearchConflictCourse('preWeek');
  };

  // 搜索下一周课程
  onClickNextWeek = () => {
    this.dateChangeAndSearchConflictCourse('nextWeek');
  };

  // 修改课表类型联动的值(老师、教室、学生)
  onChangeClassScheduleTypeValue = (value) => {
    this.setState({ classScheduleTypeValue: value });
  };

  // 冲突查询-学习中心选择
  cascaderOnchange = async (value) => {
    const { cfCampusDict, cfStartDate, cfEndDate } = this.state;
    const [_branchId, _campusId] = value || [];

    let _cfCampusIds = [_campusId];

    if (!_campusId) {
      // 如果只选择了分公司，找出该分公司下所有的 学习中心
      const branchCampus = cfCampusDict.find((item) => item.value === _branchId) || {};
      if (branchCampus.children) {
        _cfCampusIds = branchCampus.children.map((item) => item.value);
      }
    }

    this.setState({
      cfBranchCampusIds: value,
      cfCampusIds: _cfCampusIds || [],

      cfHasConflictName: false,
      cfHasCourseSchedules: false,
    });

    if (_cfCampusIds) {
      const [cfClassroomDict, cfTeacherDict] = await Promise.all([
        loaders.loadClassroomDict({ campusIds: _cfCampusIds }),
        loaders.loadTeacherDict({ campusIds: _cfCampusIds, startDate: cfStartDate, endDate: cfEndDate }),
      ]).catch((e) => {
        console.log(e);
      });
      this.setState({
        cfClassroomDict,
        cfTeacherDict,
      });
    }
  };

  // 重置
  reset = () => {
    this.setState(initSearchKeys);
    this.setState((preState) => {
      return {
        checking: false,
      };
    });
  };

  // 重置
  cfReset = () => {
    this.setState(cfInitSearchKeys);
    this.setState((preState) => {
      return {
        cfChecking: false,
      };
    });
  };

  // 采集校验信息进 errors 对象
  errorChange = (tabName, key, error) => {
    // 不同 tabName ，错误信息收集的字段不一样
    const errorKey = {
      [TabsNames.COURSE]: 'errors',
      [TabsNames.CONFLICT]: 'cfErrors',
      [TabsNames.COURSE_SUMMARY]: 'csErrors',
    }[tabName];

    this.setState((preState) => {
      const errorState = preState[errorKey];
      return {
        [errorKey]: {
          ...errorState,
          [key]: error,
        },
      };
    });
  };

  /**
   * 导出冲突名单
   *
   * @return {Promise<void>}
   */
  exportExcel = async (customerType = CustomerTypes.student) => {
    const loaderFeacher = {
      [CustomerTypes.student]: loaders.exportStudent,
      [CustomerTypes.teacher]: loaders.exportTeacher,
      [CustomerTypes.classroom]: loaders.exportClassroom,
    }[customerType];

    const param = this.formatSearchParam();
    // param.timeTableType = customerType;
    const fileName = this.formatExportFileName(1, customerType);
    console.log('🚀 ~ file: index.js ~ line 704 ~ ScheduleManage ~ exportExcel= ~ fileName', fileName);
    console.log('🚀 ~ file: index.js ~ line 703 ~ ScheduleManage ~ exportExcel= ~ param', param);
    loaderFeacher(param, fileName).then(
      (r) => {
        if (!r.data) {
          return ShowMessage('没有可导出的数据');
        }
        window.open(r.data);
      },
      (error) => {
        ShowMessage(error);
      },
    );
    return;
  };

  // 老师/教室编辑课程
  editCourse = (miniClassId, courseNum) => {
    this.setState({ isEditCourse: true, miniClassId, courseNum });
  };

  // 学生调课
  changeCourse = (miniClassId, courseNum) => {
    this.setState({ isChangeCourse: true, miniClassId, courseNum });
  };

  /**
   * 导出课程明细
   *
   * @return {Promise<void>}
   */
  exportCourseInfoBycampusId = async () => {
    const { errors, checking } = this.state;

    if (!checking) {
      this.setState({ checking: true });
    }

    if (Object.values(errors).find((v) => v)) {
      return;
    }

    const param = this.formatSearchParam();
    const fileName = this.formatExportFileName(2);

    loaders.exportCourseInfo(JSON.stringify(param), fileName).then(
      () => {},
      (error) => {
        ShowMessage(error);
      },
    );
  };

  handleExportCourseSummary = () => {
    const { csDate, campusDict, blCampusId } = this.state;
    const campusName = (campusDict.find(({ val }) => val === blCampusId) || {}).text || '';
    const courseDateStart = moment(csDate, DATE_FORMAT).startOf('week').format().slice(0, 10);
    const courseDateEnd = moment(csDate, DATE_FORMAT).endOf('week').format().slice(0, 10);
    const fileName = `${campusName}${courseDateStart}-${courseDateEnd}小班课表`;

    this.setState({ isCourseSummaryExporting: true });
    RoomViewOfMiniClassCourseListUtils.exportExcel(this.state.roomViewOfMiniClassCourseList, fileName)
      .then(
        () => {
          ShowMessage('导出Excel成功', 'success');
        },
        (error) => {
          ShowMessage(error);
        },
      )
      .finally(() => {
        this.setState({ isCourseSummaryExporting: false });
      });
  };

  // 判断是否有导出冲突名单权限
  hasExportConflictAuth = () => {
    const { btnAuths } = this.props;
    return btnAuths.includes('NEW_MINICLASS_EXPORT_EXCEL');
  };

  // 判断是否有导出课程明细权限
  hasExportCourseAuth = () => {
    const { btnAuths } = this.props;
    return btnAuths.includes('NEW_MINICLASS_EXPORT_COURSES');
  };

  getSelectLocationName = () => {
    const { cfCampusDict, cfBranchCampusIds } = this.state;
    if (cfBranchCampusIds.length !== 0) {
      const cfSelectedBranch = cfCampusDict.find((item) => item.value === cfBranchCampusIds[0]);
      const cfSelectedCampus =
        cfSelectedBranch.children && cfSelectedBranch.children.find((item) => item.value === cfBranchCampusIds[1]);
      return cfSelectedBranch.label + (cfSelectedCampus ? '/' + cfSelectedCampus.label : '');
    }
    return '~';
  };

  getCourseTabEmptyStatus = () => {
    const { campusIds, type, isNotData, classScheduleTypeValue, hasCourseSchedules } = this.state;
    let reactEl = null;
    if (this.isCourseTab()) {
      if (campusIds === null) {
        reactEl = <DefaultTip message={'请先选择学习中心'} />;
      }
      if (type === null && campusIds !== null) {
        reactEl = <DefaultTip message={'请先选择课表类型'} />;
      }
      if (type !== null && campusIds !== null && !isNotData && classScheduleTypeValue === null) {
        reactEl = <DefaultTip message={'请先选择教室/老师/学生'} />;
      }
      if (isNotData && !hasCourseSchedules) {
        reactEl = <DefaultTip message={'请更换条件重新查询'} />;
      }
    }
    return reactEl;
  };

  getCourseSummaryTabEmptyStatus = () => {
    const { /*blCampusId, gradeIds, */ roomViewOfMiniClassCourseList } = this.state;
    if (this.isCourseSummaryTab()) {
      /*if(blCampusId === null){
        return <DefaultTip message={"请先选择学习中心"} />;
      }
      if(blCampusId && (gradeIds == null || !gradeIds.length) ){
        return <DefaultTip message={"请先选择年级"} />;
      }*/
      if (!roomViewOfMiniClassCourseList.length) {
        return <DefaultTip message={''} />;
      }
    }
  };

  getConflictTabEmptyStatus = () => {
    const { cfBranchCampusIds, cfHasCourseSchedules, cfHasConflictName } = this.state;
    let reactEl = null;
    if (this.isConflictTab() && !cfHasCourseSchedules && !cfHasConflictName) {
      if (cfBranchCampusIds.length === 0) {
        reactEl = <DefaultTip message={'请先选择学习中心'} />;
      }
    }
    return reactEl;
  };

  csSearch = async () => {
    let roomViewOfMiniClassCourseList = [];
    const { csDate, csChecking, csErrors, roomViewOfMiniClassCourseKey } = this.state;
    if (!csChecking) {
      this.setState({ csChecking: true });
    }
    if (Object.values(csErrors).find((v) => v)) {
      return;
    }
    this.setState({ loading: true, currentCsDate: csDate });
    const result = await loaders.listRoomViewOfMiniClassCourseApi(this.formatSearchParam()).catch(errorsHandle());
    this.setState({ loading: false });
    if (!result || result.resultCode !== 0 || !result.data.length) {
      roomViewOfMiniClassCourseList = [];
    } else {
      roomViewOfMiniClassCourseList = result.data;
    }
    this.setState({
      roomViewOfMiniClassCourseList,
      roomViewOfMiniClassCourseKey: roomViewOfMiniClassCourseKey + 1,
    });
  };

  csReset = () => {
    this.setState({
      blCampusId: '',
      csChecking: false,
      gradeIds: this.props.gradeDict.map(({ val }) => val),
      csDate: moment().format().slice(0, 10),
    });
  };

  generateRoomViewOfMiniClassCourseList() {
    const {
      isCompressed,
      currentCsDate: currentDate,
      roomViewOfMiniClassCourseKey: key,
      roomViewOfMiniClassCourseList: roomList,
    } = this.state;
    const props = {
      key,
      roomList,
      currentDate,
      isCompressed,
      onChangeDate: this.handleChangeCsDate,
    };

    return this.isCourseSummaryTab() && !!roomList.length && <RoomViewOfMiniClassCourseList {...props} />;
  }

  render() {
    const { loading } = this.state;
    const {
      date,
      cfDate,
      csDate,
      type,
      cfType,
      campusDict,
      cfCampusDict,
      classScheduleTypeDict,
      cfTeacherDict,
      cfClassroomDict,
      filterConditionDict,
      classScheduleTypeValueDict,
      classScheduleTypeValue,
      cfClassScheduleTypeValue,
      cfBranchCampusIds,
      campusIds,
      blCampusId,
      courseSchedules,
      cfCourseSchedules,
      hasCourseSchedules,
      cfHasCourseSchedules,
      cfHasConflictName,
      courseStatus,
      dateArray,
      cfDateArray,
      current,
      checking,
      cfChecking,
      csChecking,
      errors,
      cfErrors,
      csErrors,
      exportStatus,
      isEditCourse,
      miniClassId,
      teacherDict,
      classroomDict,
      courseNum,
      isChangeCourse,
      studentIds,
      tempSearchType,
      cfTempSearchType,
      cfListOp,
    } = this.state;

    let menus = [];
    if (this.hasExportConflictAuth() === true) {
      menus.push({
        text: '学生冲突',
        disabled: !cfHasConflictName || cfBranchCampusIds.length === 0 || !cfDate,
        onClick: () => {
          this.exportExcel(CustomerTypes.student);
        },
      });
    }
    if (this.hasExportConflictAuth() === true) {
      menus.push({
        text: '老师冲突',
        disabled: !cfHasConflictName || cfBranchCampusIds.length === 0 || !cfDate,
        onClick: () => {
          this.exportExcel(CustomerTypes.teacher);
        },
      });
    }
    if (this.hasExportConflictAuth() === true) {
      menus.push({
        text: '教室冲突',
        disabled: !cfHasConflictName || cfBranchCampusIds.length === 0 || !cfDate,
        onClick: () => {
          this.exportExcel(CustomerTypes.classroom);
        },
      });
    }

    return (
      <Page title={0} fitWindowHeight={false} loading={loading} className={styles.page}>
        <Tabs defaultActiveKey="1" onChange={this.handelTabsChange} className={styles.tabs}>
          <Tabs.TabPane tab="课表总览" key={TabsNames.COURSE_SUMMARY}></Tabs.TabPane>
          <Tabs.TabPane tab="课表查询" key={TabsNames.COURSE}></Tabs.TabPane>
          <Tabs.TabPane tab="冲突查询" key={TabsNames.CONFLICT}></Tabs.TabPane>
        </Tabs>

        <div
          style={{ display: this.isCourseSummaryTab() ? 'block' : 'none' }}
          className={styles.content + ' ' + styles.clearfix + ' ' + styles.wrapper}
        >
          <div className={styles.left}>
            <SearchBar onSearch={this.csSearch} onReset={this.csReset}>
              <DatePicker
                placeholder="选择时间"
                onChange={this.csDateChange}
                value={csDate}
                validator={{
                  rules: ['required'],
                  showTips: false,
                  showStatus: true,
                  silent: !csErrors['time'] || !csChecking,
                  onErrorChange: (e) => this.errorChange(TabsNames.COURSE_SUMMARY, 'time', e),
                }}
              />
              <Select
                placeholder="学习中心"
                dict={campusDict}
                onChange={this.onCsCampusChange}
                value={blCampusId}
                showSearch={true}
                validator={{
                  rules: ['required'],
                  silent: !csErrors['blCampusId'] || !csChecking,
                  showTips: false,
                  showStatus: true,
                  onErrorChange: (e) => this.errorChange(TabsNames.COURSE_SUMMARY, 'blCampusId', e),
                }}
              />
              <MultiSelect
                placeholder="请选择年级"
                value={this.state.gradeIds}
                dict={this.props.gradeDict}
                selectAll={true}
                onChange={this.onCsGradeChange}
                showSearch={true}
                validator={{
                  rules: ['required'],
                  silent: !csErrors['gradeIds'] || !csChecking,
                  showTips: false,
                  showStatus: true,
                  onErrorChange: (e) => this.errorChange(TabsNames.COURSE_SUMMARY, 'gradeIds', e),
                }}
              />
            </SearchBar>
          </div>
          <div className={styles.extractButtons}>
            <RealButton onClick={this.handleExportCourseSummary} disabled={this.state.isCourseSummaryExporting}>
              导出Excel
            </RealButton>
          </div>
          <div className={styles.rightWrapper}>
            <div className={styles.switchWrapper}>
              <span style={{ marginRight: '4px' }}>只显示有课程的日期和教室</span>
              <Switch defaultChecked={true} onChange={this.handleChangeCourseSummaryCompress} />
            </div>
          </div>
        </div>

        {/* 课表查询 search bar */}
        <div
          style={{ display: this.isCourseTab() ? 'block' : 'none' }}
          className={styles.content + ' ' + styles.clearfix}
        >
          <div className={styles.left}>
            <SearchBar onSearch={this.search} onReset={this.reset}>
              <DatePicker
                placeholder="选择时间"
                onChange={this.dateChange}
                value={date}
                validator={{
                  rules: ['required'],
                  silent: !errors['time'] || !checking,
                  showTips: false,
                  showStatus: true,
                  onErrorChange: (e) => this.errorChange(TabsNames.COURSE, 'time', e),
                }}
              />
              <MultiSelect
                placeholder="学习中心"
                dict={campusDict}
                onChange={this.onCampusChange}
                value={campusIds}
                showSearch={true}
                validator={{
                  rules: ['required'],
                  silent: !errors['campusIds'] || !checking,
                  showTips: false,
                  showStatus: true,
                  onErrorChange: (e) => this.errorChange(TabsNames.COURSE, 'campusIds', e),
                }}
              />
              <Select
                placeholder="课表类型"
                dict={classScheduleTypeDict}
                onChange={this.onChangeclassScheduleTypeValue}
                value={type}
                validator={{
                  rules: ['required'],
                  silent: !Boolean(errors['type']) || !checking,
                  showTips: false,
                  showStatus: true,
                  onErrorChange: (e) => this.errorChange(TabsNames.COURSE, 'type', e),
                }}
              />
              <Select
                placeholder="请选择"
                dict={classScheduleTypeValueDict}
                onChange={(value) => {
                  this.onChangeClassScheduleTypeValue(value);
                }}
                value={classScheduleTypeValue}
                showSearch={true}
                noOptionTips={'暂无数据'}
                validator={{
                  rules: ['required'],
                  silent: !errors['classScheduleTypeValue'] || !checking,
                  showTips: false,
                  showStatus: true,
                  onErrorChange: (e) => this.errorChange(TabsNames.COURSE, 'classScheduleTypeValue', e),
                }}
              />
              <MultiSelect
                dict={filterConditionDict}
                onChange={(value) => {
                  this.setState({ courseStatus: value });
                }}
                showSearch={false}
                value={courseStatus}
              />
            </SearchBar>
          </div>
          <div>
            <Button
              className={styles.right}
              onClick={() => {
                this.exportCourseInfoBycampusId();
              }}
            >
              导出本周课程明细
            </Button>
          </div>
        </div>

        {/* 冲突查询search bar */}
        <div
          style={{ display: this.isConflictTab() ? 'block' : 'none' }}
          className={styles.content + ' ' + styles.clearfix}
        >
          <div className={styles.left}>
            <SearchBar
              searchText="一键查询本周冲突"
              onSearch={this.cfSearch}
              onReset={this.cfReset}
              searchBarClassName={styles.searchBar}
              searchBarControlsClassName={styles.searchBarControls}
            >
              <DatePicker
                placeholder="选择时间"
                onChange={this.cfDateChange}
                value={cfDate}
                validator={{
                  rules: ['required'],
                  silent: !cfErrors['cfTime'] || !cfChecking,
                  showTips: false,
                  showStatus: true,
                  onErrorChange: (e) => this.errorChange(TabsNames.CONFLICT, 'cfTime', e),
                }}
              />

              <Validator
                value={cfBranchCampusIds.length === 0 ? undefined : cfBranchCampusIds}
                rules={['required']}
                silent={!cfErrors['cfBranchCampusIds'] || !cfChecking}
                showTips={false}
                showStatus={true}
                onErrorChange={(e) => this.errorChange(TabsNames.CONFLICT, 'cfBranchCampusIds', e)}
              >
                <Cascader
                  size={'large'}
                  options={cfCampusDict}
                  allowClear
                  value={cfBranchCampusIds}
                  placeholder="请选择学习中心"
                  expandTrigger="hover"
                  changeOnSelect
                  displayRender={(label) => {
                    const [branchName, campusName] = label || [];
                    return campusName || branchName || '';
                  }}
                  onChange={(e) => {
                    this.cascaderOnchange(e);
                  }}
                  showSearch
                />
              </Validator>
            </SearchBar>
          </div>
          <div>
            <DropdownMenu menus={menus} text={exportStatus} hasArrow={true} className={styles.right} />
          </div>
        </div>

        {/* 冲突名单 */}
        {this.isConflictTab() && (
          <div>
            <div>
              <h3>冲突结果</h3>
              <div className={styles.mt6}>
                <span className={styles.fw}>时间范围：</span>
                {this.state.cfStartDate} ~ {this.state.cfEndDate}
              </div>
              <div className={styles.mt6}>
                <span className={styles.fw}>地点范围：</span>
                {this.getSelectLocationName()}
              </div>
            </div>
            {cfHasConflictName && [
              <ConflictList
                key={CustomerTypes.teacher}
                type={CustomerTypes.teacher}
                title={this.getConflictTitle(CustomerTypes.teacher)}
                list={cfListOp.teachers || []}
                current={current}
                onClickName={this.onClickName}
              />,
              <ConflictList
                key={CustomerTypes.student}
                type={CustomerTypes.student}
                title={this.getConflictTitle(CustomerTypes.student)}
                list={cfListOp.studentVos || []}
                current={current}
                onClickName={this.onClickName}
              />,
              <ConflictList
                key={CustomerTypes.classroom}
                type={CustomerTypes.classroom}
                title={this.getConflictTitle(CustomerTypes.classroom)}
                list={cfListOp.classroomManageVos || []}
                current={current}
                onClickName={this.onClickName}
              />,
            ]}
          </div>
        )}

        {/* 课表查询-课程表 */}
        {this.isCourseTab() && hasCourseSchedules && (
          <div>
            <ScheduleHeader
              dateArray={dateArray}
              onClickPreWeek={this.onClickPreWeek}
              onClickNextWeek={this.onClickNextWeek}
              searchDate={date}
            />
            <SchedulePanel
              courseSchedules={courseSchedules}
              type={tempSearchType}
              editCourse={this.editCourse}
              changeCourse={this.changeCourse}
            />
          </div>
        )}

        {this.generateRoomViewOfMiniClassCourseList()}

        {/* 冲突查询-课程表 */}
        {this.isConflictTab() && cfHasCourseSchedules && (
          <div>
            <ScheduleHeader
              dateArray={cfDateArray}
              onClickPreWeek={this.onClickPreWeek}
              onClickNextWeek={this.onClickNextWeek}
              searchDate={cfDate}
            />
            <SchedulePanel
              courseSchedules={cfCourseSchedules}
              type={cfTempSearchType}
              editCourse={this.editCourse}
              changeCourse={this.changeCourse}
              checkStudentCourseConflict={this.checkStudentCourseConflict}
            />
          </div>
        )}

        {this.getCourseSummaryTabEmptyStatus()}
        {this.getCourseTabEmptyStatus()}
        {this.getConflictTabEmptyStatus()}

        {isEditCourse && (
          <EditCourse
            type={this.isCourseTab() ? type : cfType && cfType.toLocaleLowerCase()}
            miniClassId={miniClassId}
            closeModal={() => this.setState({ isEditCourse: false })}
            teacherDict={this.isCourseTab() ? teacherDict : cfTeacherDict}
            classroomDict={this.isCourseTab() ? classroomDict : cfClassroomDict}
            courseNum={courseNum}
            search={this.search}
          />
        )}

        {isChangeCourse && (
          <ChangeCourse
            cancelMultiChangeCourses={() => {
              this.setState({ studentIds: [] });
            }}
            studentIds={studentIds}
            teacherDict={this.isCourseTab() ? teacherDict : cfTeacherDict}
            closeModal={this.closeChangeCoursesModal}
            miniClassId={miniClassId}
            studentId={this.isCourseTab() ? classScheduleTypeValue : cfClassScheduleTypeValue}
            campusDict={campusDict}
            search={this.search}
            courseNum={courseNum}
          />
        )}
      </Page>
    );
  }
}

let mapStateToProps = (state) => {
  let account = state.account;
  let pageParams = state.main.pageParams['/MiniClassCourseManage'];

  return {
    pageParams,
    account: account,
    gradeDict: state.dict.grade,
    btnAuths: state.account.btnAuths,
  };
};

export default connect(mapStateToProps)(ScheduleManage);
