import {
  selectOrgByIdAndTypeOption,
  getConflictMiniClassInfo,
  getConflictMiniCourseInfo,
  changetoGetConflictInfo,
  changeClassRoomOrTeacher,
  getStudentConflictCourse,
  getChangeMiniClassList,
  getSubSelectOptionBySuper,
  changeCourse,
  checkConflictAfterChangeCourse,
  exportCourseInfoBycampusId,
  getStudentConflictCourseNew,
} from 'SERVICE/common';
import {
  getMiniClassDetailStudentList,
  getStudentIdByCampusIdsParam,
  getTeacherIdByCampusIdsParam,
  getClassroomIdByCampusIdsParam,
  getMiniClassTimeTableConflictName,
  getMiniClassTimeTableInfo,
  checkMiniClassCourseConflict,
  checkStudentCourseConflict,
  fetchToexportClassroom,
  fetchToexportTeacher,
  fetchToexportStudent,
  listRoomViewOfMiniClassCourse,
} from 'SERVICE/course';
import { urlBase } from 'CONST/config';
import { formateDict, loadBranchCampusDict } from 'UTIL/dict';
import moment from 'moment';
import { DATE_FORMAT } from 'COMPONENT/utils/form';

// 课表类型
const classScheduleDict = [
  {
    val: 'teacher',
    text: '老师',
  },
  {
    val: 'student',
    text: '学生',
  },
  {
    val: 'classroom',
    text: '教室',
  },
];
// 筛选条件
const filterConditionDict = [
  {
    val: 'NEW',
    text: '未上课',
  },
  {
    val: 'TEACHER_ATTENDANCE',
    text: '已考勤',
  },
  {
    val: 'CHARGED',
    text: '已结算',
  },
];

export const loadCampusDict = async (params) => {
  let data = await selectOrgByIdAndTypeOption(params);
  return formateDict(data);
};

const getBranchCampusDict = async () => {
  let data = await loadBranchCampusDict();
  return data;
};

const loadClassScheduleTypeDict = async () => {
  return Promise.resolve(classScheduleDict);
};

const loadFilterConditionDict = async () => {
  return Promise.resolve(filterConditionDict);
};

// 老师字典
const loadTeacherDict = async (params) => {
  const response = await getTeacherIdByCampusIdsParam(params);
  const { data } = response;

  return formatGroupDict(data);
};

// 学生字典
const loadStudentDict = async (params) => {
  const response = await getStudentIdByCampusIdsParam(params);
  const { data } = response;

  return formatGroupDict(data);
};

// 教室字典
const loadClassroomDict = async (params) => {
  const response = await getClassroomIdByCampusIdsParam(params);
  const { data } = response;

  return formatGroupDict(data);
};

/**
 * 获取冲突学生
 *
 * @param params.startDate
 * @param params.endDate
 * @param params.campusIds
 * @param params.timeTableType {string} STUDENT|TEACHER|CLASSROOM
 * @param params.objId {string}
 * @param params.courseStatus {array}
 * @return {Promise<{type: string, conflictList: Array}>}
 */
const loadConflictName = async (params) => {
  const { timeTableType: type } = params;

  const response = await getMiniClassTimeTableConflictName(params);
  const data = response.data || [];

  if (data.length === 0) {
    return null;
  } else {
    const conflictList = data.map((item) => {
      return { name: item.objName, id: item.objId };
    });

    return { type, conflictList };
  }
};

/**
 * 冲突课程
 *
 * @param params.startDate
 * @param params.endDate
 * @param params.campusIds
 * @param params.timeTableType {string} STUDENT|TEACHER|CLASSROOM
 * @param params.objId {string}
 * @param params.courseStatus {array}
 * @return {Promise<{courseSchedules: Array, dateArray: Array}>}
 */
const loadCoursesSchedules = async (params) => {
  const response = await getMiniClassTimeTableInfo(params);
  const data = response ? response.data || [] : [];
  const { startDate, endDate } = params;

  // 先把date提取
  let resultDate = [];
  // 组装成二维数组 [[...], [...], ...]
  let resultCourse = [];

  for (
    let date = startDate;
    new Date(date) <= new Date(endDate);
    date = moment(date).add(1, 'days').format(DATE_FORMAT)
  ) {
    resultDate.push(date);
  }

  resultDate.sort((a, b) => new Date(a) - new Date(b));

  resultDate.forEach((date) => {
    const oneDayCourses = data.filter((item) => {
      return item.courseDate === date;
    });

    resultCourse.push(oneDayCourses);
  });

  return { courseSchedules: resultCourse, dateArray: resultDate };
};

/**
 * 导出冲突名单 (废弃)
 *
 * @param params 参数
 * @param fileName 文件名
 * @return {Promise<any>}
 */
const exportExcel = async (params, fileName) => {
  return doExport(params, fileName, 'CourseConflictController/exportMiniClassTimeTableConflictName.do');
};

/**
 * 导出学生冲突小班课节名单
 *
 * @param params 参数
 * @param fileName 文件名
 * @return {Promise<any>}
 */
const exportStudent = async (params, fileName) => {
  return fetchToexportStudent(params);
};

/**
 * 导出教室冲突小班课节名单
 *
 * @param params 参数
 * @param fileName 文件名
 * @return {Promise<any>}
 */
const exportClassroom = async (params, fileName) => {
  return fetchToexportClassroom(params);
};

/**
 * 导出老师冲突小班课节名单
 *
 * @param params 参数
 * @param fileName 文件名
 * @return {Promise<any>}
 */
const exportTeacher = async (params, fileName) => {
  return fetchToexportTeacher(params);
};

// eduboss/boss-xinghuo-api/course/courseConflict/listStudentConflictMiniClassCourse
// eduboss/boss-xinghuo-api/course/courseConflict/listClassroomConflictMiniClassCourse
// /eduboss/boss-xinghuo-api/course/courseConflict/listTeacherConflictMiniClassCourse
/**
 * 导出课表明细
 *
 * @param params 参数
 * @param fileName 文件名
 * @return {Promise<any>}
 */
const exportCourseInfo = async (params, fileName) => {
  return doExport(params, fileName, 'CourseConflictController/exportMiniClassTimeTableInfo.do');
};

/**
 * 执行导出
 *
 * @param params 参数
 * @param fileName 文件名
 * @param apiPath 导出API
 * @return {Promise<any>}
 */
function doExport(params, fileName, apiPath) {
  return new Promise((resolve, reject) => {
    let request = new XMLHttpRequest();

    request.open('POST', urlBase + apiPath, true);
    request.responseType = 'blob';
    request.setRequestHeader('Content-type', 'application/json');
    request.onload = function () {
      if (this.status === 200) {
        const blob = this.response;
        const reader = new FileReader();

        reader.readAsDataURL(blob);
        reader.onload = function (event) {
          resolve();

          let a = document.createElement('a');
          a.href = window.URL.createObjectURL(request.response);
          a.download = fileName + '.xls';
          document.body.appendChild(a);
          a.click();
          a.remove();
        };
      } else {
        const reader = new FileReader();

        reader.addEventListener('loadend', (event) => {
          const text = event.srcElement.result;
          reject(JSON.parse(text));
        });

        reader.readAsText(this.response);
      }
    };

    request.onerror = (error) => {
      reject(error);
    };

    request.send(JSON.parse(JSON.stringify(params)));
  });
}

const loadConflictMiniClassInfo = async (params) => {
  let response = await getConflictMiniClassInfo(params);
  let { data } = response;
  let miniClassDataSource = data.map((miniClassInfo) => {
    return {
      miniClassName: miniClassInfo.miniClassName,
      teacher: {
        teacherId: miniClassInfo.teacherId,
        teacherName: miniClassInfo.teacherName,
      },
      classroom: {
        classroomId: miniClassInfo.classroomId,
        classroomName: miniClassInfo.classroomName,
      },
      classroomName: miniClassInfo.classroomName,
      subject: miniClassInfo.subject,
      grade: miniClassInfo.grade,
      everyCourseNum: miniClassInfo.everyCourseNum,
      classTimeLength: miniClassInfo.classTimeLength,
      teacherName: miniClassInfo.teacherName,
      productGroupId: miniClassInfo.productGroupId,
    };
  });
  return miniClassDataSource;
};

const loadConflictMiniCourseInfo = async (params) => {
  let response = await getConflictMiniCourseInfo(params);
  let { data } = response;
  let miniCourseDataSource = data.map((miniCourseInfo) => {
    return {
      courseStatus: miniCourseInfo.courseStatus,
      courseId: miniCourseInfo.courseId,
      classroom: {
        classroomName: miniCourseInfo.classroomName,
        classroomId: miniCourseInfo.classroomId,
      },
      teacher: {
        teacherName: miniCourseInfo.teacherName,
        teacherId: miniCourseInfo.teacherId,
      },
      courseNum: miniCourseInfo.courseNum,
      courseDate: miniCourseInfo.courseDate,
      courseTime: miniCourseInfo.courseTime,
      courseTimeLength: miniCourseInfo.courseTimeLength,
      teachingType: miniCourseInfo.teachingType,
    };
  });
  return miniCourseDataSource;
};

const loadChangetoGetConflictInfo = async (params) => {
  let response = await changetoGetConflictInfo(params);
  let { data } = response;
  return data;
};

const changeClassRoomOrTeacherApi = async (params) => {
  let response = await changeClassRoomOrTeacher(params);
};

const loadStudentConflictCourse = async (params) => {
  let response = await getStudentConflictCourse(params);
  return response.data.map((course) => {
    return Object.assign(course, { selected: false });
  });
};

const loadStudentConflictCourseNew = async (params) => {
  let response = await getStudentConflictCourseNew(params);
  return response.data.map((course) => {
    return Object.assign(course, { selected: false });
  });
};

const loadChangeMiniClassList = async (params) => {
  let response = await getChangeMiniClassList(params);
  return { dataSource: response.rows, total: response.records, current: response.page };
};

const loadSelectOption = async () => {
  let response = await getSubSelectOptionBySuper({ category: 'CLASS_TYPE' });
  return formateDict(response);
};

const changeCourseApi = async (params) => {
  let response = await changeCourse(params);
  return response;
};

const checkConflictAfterChangeCourseApi = async (params) => {
  let response = await checkConflictAfterChangeCourse(params);
  return response;
};

const exportCourseInfoBycampusIdApi = async (params) => {
  let response = await exportCourseInfoBycampusId(params);
};

const getMiniClassDetailStudentListApi = async (params) => {
  let response = await getMiniClassDetailStudentList(params);
  return response;
};

const checkMiniClassCourseConflictApi = async (params) => {
  let response = await checkMiniClassCourseConflict(params);
  const data = response.data;
  data.classroomManageVos && data.classroomManageVos.forEach((item) => (item.name = item.classRoom));
  data.teachers && data.teachers.forEach((item) => (item.id = item.userId));
  return data;
};

const checkStudentCourseConflictApi = async (params) => {
  let response = await checkStudentCourseConflict(params);
  return response;
};

const listRoomViewOfMiniClassCourseApi = (params) => {
  return listRoomViewOfMiniClassCourse(params);
};

/**
 * 格式化新字典接口数据
 * @param data
 */
function formatGroupDict(data = []) {
  let dict = [];

  if (data) {
    data.forEach((item) => {
      const { campusName, children } = item;
      let option = [];

      if (children) {
        option = children.map((child) => {
          return {
            val: child.id,
            text: child.name,
          };
        });
      }

      dict.push({
        label: campusName,
        options: option,
      });
    });
  }

  return dict;
}

export default {
  loadCampusDict,
  getBranchCampusDict,
  loadClassScheduleTypeDict,
  loadFilterConditionDict,
  loadStudentDict,
  loadClassroomDict,
  loadTeacherDict,
  loadCoursesSchedules,
  loadConflictName,
  exportExcel,
  exportCourseInfo,
  loadConflictMiniClassInfo,
  loadConflictMiniCourseInfo,
  loadChangetoGetConflictInfo,
  changeClassRoomOrTeacherApi,
  loadStudentConflictCourse,
  loadStudentConflictCourseNew,
  loadChangeMiniClassList,
  loadSelectOption,
  changeCourseApi,
  checkConflictAfterChangeCourseApi,
  exportCourseInfoBycampusIdApi,
  getMiniClassDetailStudentListApi,
  checkMiniClassCourseConflictApi,
  checkStudentCourseConflictApi,
  listRoomViewOfMiniClassCourseApi,
  exportTeacher,
  exportClassroom,
  exportStudent,
};
