import React from 'react';
import Modal from 'COMPONENT/Modal';
import { connect } from 'dva';
import Button from 'COMPONENT/Button';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import TextLink from 'COMPONENT/TextLink';
import { OSS_BASE_PREFIX } from 'CONST/ossConfig';
import excelPng from './assets/excel.png';
import filePng from './assets/file.png';
import delPng from './assets/del.png';
import styles from './styles.less';
import { getAliyunOSSKeyWithoutCallBack, initRecord } from 'SERVICE/course';
import moment from 'moment';

const templateUrl = '/%E6%89%B9%E9%87%8F%E5%BB%BA%E7%8F%AD%E6%A8%A1%E6%9D%BF.xlsx';

class BatchBuildClass extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      //1：待上传，2：上传完成
      status: 1,
      files: [], // 上传文件
    };
  }

  getTitle = () => {
    const { status } = this.state;
    if (status === 1) {
      return '产品导入';
    } else {
      return '导入提醒';
    }
  };

  /**
   * 下载模板
   */
  downloadTemplate = () => {
    const link = document.createElement('a');
    const body = document.body;
    link.href = OSS_BASE_PREFIX + templateUrl;
    link.style.display = 'none';
    link.download = '批量建班模板.xls';
    body.appendChild(link);
    link.click();
    body.removeChild(link);
  };

  /**
   * 点击选择文件事件
   * @param event
   */
  pickFile = (event) => {
    const files = Array.prototype.map.call(event.target.files, (file) => ({
      file,
      name: file.name,
      size: file.size,
      type: file.type,
    }));
    const file = files[0] || {};
    if (file.size && file.size / 1024 / 1024 > 3) {
      showMessage('文件大小超过3M,请重新选择');
      return;
    }
    this.setState({ files });
  };

  /**
   * 删除已选文件
   */
  onDelete = () => {
    this.setState({
      files: [],
    });
  };

  /**
   * 渲染底部按钮
   * @returns {*}
   */
  renderFooter = () => {
    const { status, files } = this.state;
    if (status === 1) {
      return [
        <Button key="222" type="primary" disabled={files.length === 0} onClick={this.onUploadFile}>
          导入
        </Button>,
        <Button key="111" onClick={this.onCancel}>
          取消
        </Button>,
      ];
    } else {
      return <Button onClick={this.onCancel}>取消</Button>;
    }
  };

  /**
   * 渲染模板下载
   */
  renderDownloadTemplate = () => {
    return (
      <div>
        <p style={{ display: 'flex', margin: '12px 0 24px 0' }}>
          <img src={excelPng} width="22" height="22" alt="excel" />
          <TextLink style={{ marginLeft: 10 }} onClick={() => this.downloadTemplate()}>
            模板下载
          </TextLink>
        </p>
        <p className={styles.label}>上传文件</p>
      </div>
    );
  };

  /**
   * 渲染选择的文件或按钮
   */
  renderUpload = () => {
    const { files } = this.state;
    if (files.length > 0) {
      return (
        <div>
          {this.renderDownloadTemplate()}
          <div className={styles.fileArea}>
            {files.map((file) => {
              return (
                <div key={file.name} className={styles.file}>
                  <span className={styles.fileWrap}>
                    <img src={filePng} alt="文件" width="16" height="16" />
                    <span className={styles.fileName}>{file.name}</span>
                  </span>
                  <img
                    src={delPng}
                    width="16"
                    height="16"
                    onClick={this.onDelete}
                    className={styles.fileDel}
                    alt="删除"
                  />
                </div>
              );
            })}
          </div>
        </div>
      );
    } else {
      return (
        <div>
          {this.renderDownloadTemplate()}
          <div className={styles.pickFile}>
            选择文件
            <input
              type="file"
              accept="application/vnd.openxmlformats-officedocument.spreadsheetml.sheet, application/vnd.ms-excel"
              className={styles.uploadBtn}
              onClick={(e) => (e.target.value = null)}
              onChange={this.pickFile}
            />
          </div>
        </div>
      );
    }
  };

  renderUploadResult = () => {
    const { courseScheduleBIUrl } = this.props;

    return (
      <div>
        <div className={styles.resultCard}>
          <p className={styles.resultText}>数据正在导入，您可在「课程管理-小班管理-小班批量导入记录」中查看导入结果</p>
          <Button type="primary" onClick={this.openImportRecordPage}>
            查看导入结果
          </Button>
        </div>
        {courseScheduleBIUrl && (
          <div className={styles.resultCard}>
            <p className={styles.resultText}>可在「报表中心-智能排班-预警监控」中查看排班冲突</p>
            <Button type="primary" onClick={this.openCourseSchedulePage}>
              查看排班冲突
            </Button>
          </div>
        )}
      </div>
    );
  };

  openImportRecordPage = () => {
    const { dispatch } = this.props;

    this.props.onClose(() => {
      dispatch({
        type: 'main/openPage',
        payload: {
          path: '/BatchBuildClassRecords',
        },
      });
    });
  };

  openCourseSchedulePage = () => {
    const { courseScheduleBIUrl, courseScheduleBIName, dispatch } = this.props;

    const [, queryStr] = courseScheduleBIUrl.split('?path=');

    this.props.onClose(() => {
      dispatch({
        type: 'main/openPage',
        payload: {
          path: '/next-compact',
          query: {
            path: queryStr,
            title: courseScheduleBIName,
          },
          title: courseScheduleBIName,
        },
      });
    });
  };

  /**
   * 渲染模态框内容
   * @returns {*}
   */
  renderBody = () => {
    const { status } = this.state;

    if (status === 1) {
      return this.renderUpload();
    } else {
      return this.renderUploadResult();
    }
  };

  /**
   * 上传
   */
  onUploadFile = async () => {
    const { files } = this.state;
    const { OSSAccessKeyId, policy, signature, host, Expires, callback } = await getAliyunOSSKeyWithoutCallBack();
    let formData = new FormData();
    const time = moment().format('YYYYMMDDHHmmss');

    formData.append('OSSAccessKeyId', OSSAccessKeyId);
    formData.append('policy', policy);
    formData.append('Signature', signature);
    formData.append('success_action_status', '200');
    formData.append('key', time + files[0].file.name);
    files.forEach((file) => {
      formData.append('file', file.file);
    });
    const xhr = new XMLHttpRequest();
    xhr.open('POST', host);
    xhr.onload = async () => {
      if (xhr.status === 200 && xhr.readyState === 4) {
        const { status, resultCode } = await initRecord({
          filePath: time + files[0].file.name,
          category: 'BATCH_ADD_CLASS',
        }).catch(() => {
          this.setState({ loading: false });
        });
        if (resultCode === 0) {
          this.setState({ status: 2, files: [] });
        } else {
          showMessage('导入失败,请重新导入!');
          this.setState({ loading: false });
        }
        this.setState({ loading: false });
      } else {
        showMessage('系统错误');
        this.setState({ loading: false });
      }
    };
    xhr.onerror = () => {
      showMessage('网络错误!');
      this.setState({ loading: false });
    };
    xhr.send(formData);
    this.setState({ loading: true });
  };

  /**
   * 取消
   */
  onCancel = () => {
    this.setState({ status: 1 });
    this.props.onClose();
  };

  getWidthSize = () => {
    const { status } = this.state;

    return status === 2 ? 'M' : '';
  };

  render() {
    return (
      <Modal
        loading={this.state.loading}
        title={this.getTitle()}
        widthSize={this.getWidthSize()}
        width="446"
        onCancel={this.onCancel}
        footer={this.renderFooter()}
      >
        {this.renderBody()}
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  const { menus } = state.main;

  const [{ rurl: courseScheduleBIUrl = '', name: courseScheduleBIName = '' } = {}] = menus.filter(
    (item) => item.id === 'BI-rptXhBossTotalSchedule',
  );

  const { user = {} } = state.account || {};

  const isXinghuo = user && user.institution === 'xinghuo';

  return {
    courseScheduleBIUrl: isXinghuo ? courseScheduleBIUrl : '',
    courseScheduleBIName: isXinghuo ? courseScheduleBIName : '',
  };
};

export default connect(mapStateToProps)(BatchBuildClass);
