import React from 'react';
import showMessage from 'SRC/components/utils/globalTips/showMessage';
import { Table, Form } from 'antd';
import { Modal } from 'COMPONENT';
import { createSetWidth } from 'COMPONENT/List/utils';
import Select from 'COMPONENT/Select';
import { schoolType } from './dict';
import loaders from './loaders';

import { batchUpdateStudent } from 'SERVICE/course';

const setWidth = createSetWidth();

const FormItem = Form.Item;
const formItemLayout = {
  labelCol: {
    xs: { span: 24 },
    sm: { span: 5 },
  },
  wrapperCol: {
    xs: { span: 24 },
    sm: { span: 12 },
  },
};

class BatchModifySchool extends React.Component {
  static propTypes = {};
  constructor(props) {
    super(props);
    this.state = {
      loading: false,

      provinceGroupDict: [{}],
      provinceDict: [{}],
      cityDict: [
        {
          val: '440100',
          text: '广州市',
        },
      ],

      schoolName: '',
      organization: {},
      regionDict: [{ val: '440000', text: '广东省' }],
      schoolDict: [{ val: '13567', text: '香洲区第一小学（小学）' }],
      cityDisabled: false,
      schoolDisabled: false,
    };
    this.columns = [
      {
        title: '学生姓名',
        dataIndex: 'name',
        key: 'name',
        rowKey: 'name',
        width: setWidth(3),
        render: (text) => {
          return text || '-';
        },
      },
      {
        title: '分公司',
        dataIndex: 'brenchName',
        rowKey: 'brenchName',
        key: 'brenchName',
        width: setWidth(6),
        render: (text) => {
          return text || '-';
        },
      },
      {
        title: '校区',
        dataIndex: 'blCampusName',
        key: 'blCampusName',
        rowKey: 'schoolName',

        width: setWidth(6),
        render: (text) => {
          return text || '-';
        },
      },
      {
        title: '在读学校',
        dataIndex: 'schoolName',
        key: 'schoolName',
        rowKey: 'schoolName',

        width: setWidth(6),
        render: (text) => {
          return text || '-';
        },
      },
    ];
    // 表单引用
  }

  componentDidMount() {
    let dataSource = this.props.selectedRow;
    let { blCampusId } = dataSource[0];
    Promise.all([
      loaders.regionList(),
      loaders.organization({
        id: blCampusId,
      }),
    ])
      .then(([regionDict, organization]) => {
        this.setState({
          regionDict,
          organization,
        });
        this.props.form.setFieldsValue({
          province: organization.province.id,
          city: organization.city.id,
        });
        return this.getCityDict(organization.province.id).then(() => {
          return this.getSchoolDict(organization.city.id);
        });
      })
      .then(() => {
        this.setState({ loading: false });
      })
      .catch(() => {
        this.setState({ loading: false });
      });
  }

  getCityDict = async (params) => {
    this.setState({
      cityDisabled: true,
      schoolDisabled: true,
    });
    if (!params) {
      return this.setState({
        cityDict: [],
        schoolDict: [],
        cityDisabled: false,
        schoolDisabled: false,
      });
    }

    const res = await loaders.cityList({ provinceID: params });
    this.setState({
      cityDict: res,
      schoolDict: [],
      cityDisabled: false,
      schoolDisabled: false,
    });
  };

  getSchoolDict = async (blCampus, schoolType) => {
    if (!blCampus) {
      return this.setState({
        schoolDict: [],
      });
    }
    this.setState({
      schoolDisabled: true,
    });
    let params = { blCampus };
    if (schoolType) {
      params.schoolType = schoolType;
    }
    let res = [];
    try {
      res = await loaders.schoolList(params);
    } catch (error) {
      this.setState({ schoolDisabled: false });
    }
    const schoolDict = res.filter((ele) => ele.val);
    this.setState({ schoolDict: schoolDict, schoolDisabled: false });
  };
  /**
   * 判断值是否为数组
   *
   * @param value
   * @return {boolean}
   */
  calcIsArray = (value) => Object.prototype.toString.call(value) === '[object Array]';

  handleSubmit = () => {
    const { onClose, selectedRow } = this.props;
    const studentIds = selectedRow.map((i) => i.id);
    this.props.form.validateFields((err, values) => {
      if (err) {
        return;
      }
      batchUpdateStudent({
        studentIds,
        updateType: 'ATTEND_SCHOOL',
        schoolId: values.school.val,
      })
        .then(() => {
          showMessage('批量修改在读学校成功！', 'success');
        })
        .catch((er) => showMessage('批量修改在读学校失败！' + er))
        .finally(() => {
          try {
            var SEARCH_FORM_STUDENT_LIST = 'SEARCH_FORM_STUDENT_LIST';
            var START_SEARCH = 'START_SEARCH';
            global.GlobalEmitter.create(SEARCH_FORM_STUDENT_LIST).trigger(START_SEARCH);
          } catch (e) {
            console.warn(e);
          }
          onClose();
        });
    });
  };

  submitFail = () => {
    showMessage('请完善好所有信息后再提交！', 'error');
  };

  closePanel = () => {
    this.props.onCancel();
  };

  render() {
    let { cityDict, regionDict, schoolDict, cityDisabled, schoolDisabled } = this.state;
    let { getFieldDecorator, setFieldsValue, getFieldValue } = this.props.form;
    return (
      <Modal title="批量修改在读学校" width={880} loading={false} onCancel={this.closePanel} onOk={this.handleSubmit}>
        <br />
        <h2>已选学生</h2>
        <br />
        <Table dataSource={this.props.selectedRow} columns={this.columns} pagination={false} bordered />
        <br />
        <h2>选择在读学校</h2>
        <br />

        <Form onSubmit={this.submit} onSubmitFail={this.submitFail} onCancel={this.closePanel}>
          <FormItem
            name="province"
            rules={[{ required: true, message: '省份不能为空' }]}
            label="省份"
            colon={false}
            {...formItemLayout}
          >
            {getFieldDecorator('province', {
              rules: [
                {
                  required: true,
                  message: '请选择省份',
                },
              ],
            })(
              <Select
                showSearch
                dict={regionDict}
                onChange={(val) => {
                  this.getCityDict(val);
                  setFieldsValue({ province: val, city: '', school: '' });
                }}
              />,
            )}
          </FormItem>
          <FormItem
            name="city"
            rules={[{ required: true, message: '城市不能为空' }]}
            label="城市"
            colon={false}
            {...formItemLayout}
          >
            {getFieldDecorator('city', {
              rules: [
                {
                  required: true,
                  message: '请选择城市',
                },
              ],
            })(
              <Select
                dict={cityDict}
                showSearch
                disabled={cityDisabled}
                // value={city}
                onChange={(blCampus) => {
                  this.getSchoolDict(blCampus);
                  let schoolType = getFieldValue('schoolType');
                  if (schoolType) {
                    this.getSchoolDict(blCampus, schoolType);
                  } else {
                    this.getSchoolDict(blCampus);
                  }
                  setFieldsValue({ school: '' });
                }}
              />,
            )}
          </FormItem>

          <FormItem
            name="schoolType"
            initialValue={this.props.studentName}
            rules={[{ required: true, message: '学校级别不能为空' }]}
            label="学校级别"
            colon={false}
            {...formItemLayout}
          >
            {getFieldDecorator('schoolType', {
              rules: [
                {
                  required: true,
                  message: '请选择学校级别',
                },
              ],
            })(
              <Select
                dict={schoolType}
                placeholder="请选择学校级别"
                onChange={(schoolType) => {
                  let blCampus = getFieldValue('city');
                  if (blCampus) {
                    this.getSchoolDict(blCampus, schoolType);
                  }
                  setFieldsValue({ school: '' });
                }}
              />,
            )}
          </FormItem>
          <FormItem
            name="school"
            rules={[{ required: true, message: '就读学校不能为空' }]}
            label="就读学校"
            colon={false}
            {...formItemLayout}
          >
            {getFieldDecorator('school', {
              rules: [
                {
                  required: true,
                  message: '请选择学校',
                },
              ],
            })(<Select placeholder="请选择学校" showSearch dict={schoolDict} textInValue disabled={schoolDisabled} />)}
          </FormItem>
        </Form>
      </Modal>
    );
  }
}
const BatchModifySchoolForm = Form.create({})(BatchModifySchool);

export default BatchModifySchoolForm;
