import React from 'react';
import { Modal, Radio } from 'COMPONENT';
import { runFunc } from 'SRC/utils/common';
import { batchUpOnShelves, batchDownOnShelves, validateWhenDownOnShelves } from 'SERVICE/course';
import style from './style.less';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

const RadioGroup = Radio.Group;

export default class MiniClassBatchShelf extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false,
      messageDialog: false,
    };
    this.onOk = this.onOk.bind(this);
    this.onClose = this.onClose.bind(this);
  }

  onOk() {
    if (this.props.action === '1') {
      // 下架
      this.validateOnDownShelves();
    } else {
      // 上架
      this.upOnShelves();
    }
  }

  /** 上架 */
  async upOnShelves() {
    try {
      this.setState({ loading: true });
      const res = await batchUpOnShelves({ miniClassIds: this.props.classIds });
      if (res && res.resultCode === 0) {
        showMessage('上架成功', 'success');
        this.setState({ loading: false }, this.onClose);
      }
    } catch (e) {
      this.setState({ loading: false });
    }
  }

  /** 下架 */
  async downOnShelves() {
    try {
      this.setState({ loading: true });
      const res = await batchDownOnShelves({ miniClassIds: this.props.classIds });
      if (res && res.resultCode === 0) {
        showMessage('下架成功', 'success');
        this.setState({ loading: false }, this.onClose);
      }
    } catch (e) {
      this.setState({ loading: false });
    }
  }

  async validateOnDownShelves() {
    try {
      const self = this;
      this.setState({ loading: true });
      const res = await validateWhenDownOnShelves({ miniClassIds: this.props.classIds });
      if (res && res.resultCode === 0) {
        const {
          localOnlineClasses, // 本地网课的所有小班名字
          studentEnrollments, // 有学生报读的所有小班名字
          attendances, // 考勤课消的所有小班名字
          renewalClasses, // 所有关联到续报的小班名字
        } = res.data;
        if (localOnlineClasses.length || studentEnrollments.length || attendances.length) {
          // 无法下架
          const attendTip = this.formatTip(attendances, 1);
          const stuErollTip = this.formatTip(studentEnrollments, 2);
          const localTip = this.formatTip(localOnlineClasses, 3);
          Modal.info({
            title: '提示',
            width: '640px',
            content: (
              <div className={style.modal_tip_container}>
                <p>所选班级中：</p>
                {attendTip && <p>{attendTip}；</p>}
                {stuErollTip && <p>{stuErollTip}；</p>}
                {localTip && <p>{localTip}；</p>}
                <p>无法下架。</p>
              </div>
            ),
            onOk() {
              self.onClose();
            },
          });
        } else if (renewalClasses.length) {
          // 已有续报关联，能继续下架
          const tipText = renewalClasses.join('，');
          Modal.confirm({
            title: '提示',
            okText: '确定下架',
            width: '640px',
            content: <p className={style.modal_tip_container}>所选班级中：（{tipText}）已有续报关联，确定要下架吗？</p>,
            onCancel() {
              self.onClose();
            },
            onOk() {
              self.downOnShelves();
            },
          });
        } else {
          // 能下架
          this.downOnShelves();
        }
      }
    } finally {
      this.setState({ loading: false });
    }
  }

  formatTip(data, type) {
    if (!data.length) return '';
    let text;
    switch (type) {
      case 2:
        text = '有学生报读或转班/预退中：';
        break;
      case 1:
        text = '有考勤课消：';
        break;
      case 3:
        text = '有本地网课：';
        break;
    }
    return `${text}（${data.join('，')}）`;
  }

  onClose() {
    this.informIfameToRefreash();
    runFunc(this.props.onCancel);
  }

  informIfameToRefreash() {
    GlobalEmitter.create('small_class_batch_shelves').trigger('refresh');
  }

  componentWillUnmount() {
    GlobalEmitter.remove('small_class_batch_shelves');
  }

  render() {
    return (
      <Modal title="修改上下架状态" width={520} onCancel={this.onClose} onOk={this.onOk} loading={this.state.loading}>
        {/** 与传过来参数相反 */}
        <div className={style.selections}>
          <span className={style.label}>上下架状态：</span>
          <RadioGroup disabled defaultValue={this.props.action}>
            <Radio value="0">上架</Radio>
            <Radio value="1">下架</Radio>
          </RadioGroup>
        </div>
        <p className={style.tip_text}>
          注：下架班级，请务必到在线报读后台对班级做下架处理，否则可能有学生通过在线报读进入下架班级
        </p>
      </Modal>
    );
  }
}
