import React from 'react';
import PropType from 'prop-types';
import { Icon } from 'antd';
import classNames from 'classnames';
import style from './course-time-desk.less';

const CourseStatusTag = (props) => {
  const classPrefix = 'CourseStatusTag';
  return (
    <div
      className={classNames(
        style[classPrefix],
        props.isNormal ? '' : style.notNormal,
        props.insufficient ? style.insufficient : '',
      )}
    >
      <span>{props.insufficient ? '学生课时不足' : props.text}</span>
    </div>
  );
};

const DateShow = (props) => {
  const classPrefix = 'DateShow';
  return (
    <div className={style[classPrefix]}>
      <div className={style[`${classPrefix}-date-text`]}>{props.date}</div>
      <div className={style[`${classPrefix}-time-text`]}>{`${props.timeRange[0]}-${props.timeRange[1]}`}</div>
    </div>
  );
};

const Lesson = (props) => {
  const classPrefix = 'Lesson';
  return (
    <div className={style[classPrefix]}>
      <Icon type="book" />
      <span className={style[`${classPrefix}-text`]}>{props.data}</span>
    </div>
  );
};

const CourseTimeCard = (props) => {
  const {
    data: {
      name,
      courseStatusName,
      courseStatusBool,
      courseDateText,
      courseBeginTime,
      courseEndTime,
      insufficient = false,
    },
    onClick,
    beforeNowDisabled,
    selected,
  } = props;
  const classPrefix = 'CourseTimeCard';
  let checkStr = selected ? '-check' : '-no-check';
  return (
    <div
      className={classNames(style[classPrefix], beforeNowDisabled ? style.disabled : '')}
      onClick={beforeNowDisabled || insufficient ? null : onClick}
    >
      <div className={style[`${classPrefix}-wrap`]}>
        <CourseStatusTag text={courseStatusName} isNormal={courseStatusBool} insufficient={insufficient} />
        <DateShow date={courseDateText} timeRange={[courseBeginTime, courseEndTime]} />
        <Lesson data={name} />
        <Icon className={style[`${classPrefix}${checkStr}`]} type="check-circle" />
      </div>
    </div>
  );
};

class CourseTimeDesk extends React.Component {
  constructor(props) {
    super(props);

    this.updateSelectKeys = this.updateSelectKeys.bind(this);
  }
  static classPrefix = 'CourseTimeDesk';

  static propTypes = {
    dataSource: PropType.array.isRequired,
    selectedKeys: PropType.array.isRequired,
    onKeysChange: PropType.func.isRequired,
    multiSelect: PropType.bool.isRequired,
    autoSelectAfter: PropType.bool.isRequired,
    beforeNowDisabled: PropType.bool.isRequired,
  };

  // 更新已选项
  updateSelectKeys(index) {
    // 转班 不能修改
    if (this.props.modalType === 'changeClass') return;
    let origin = this.props.selectedKeys.slice();
    let checkIndex = origin.indexOf(index);
    if (this.props.autoSelectAfter) {
      // 自动选择之后的日期
      origin = [];
      let total = this.props.dataSource.length;
      if (index < total - 1) {
        for (let i = index; i < total; i++) {
          origin.push(i);
        }
      } else {
        origin = [index];
      }
    } else if (this.props.multiSelect) {
      // 多选
      if (checkIndex === -1) {
        origin.push(index);
        origin.sort();
      } else {
        origin.splice(checkIndex, 1);
      }
    } else {
      // 单选
      origin = checkIndex === -1 ? [index] : [];
    }
    this.props.onKeysChange(origin);
  }

  render() {
    const prefix = CourseTimeDesk.classPrefix;

    return (
      <div className={style[prefix]}>
        <div className={style[`${prefix}-wrap`]}>
          {this.props.dataSource.map((item, index) => (
            <CourseTimeCard
              data={item}
              index={index}
              beforeNowDisabled={item.disabled}
              onClick={() => {
                this.updateSelectKeys(index);
              }}
              selected={this.props.selectedKeys.includes(index)}
              key={index}
            />
          ))}
        </div>
      </div>
    );
  }
}

export default CourseTimeDesk;
