import React from 'react';
import PropTypes from 'prop-types';
import UpdateModal from './update-modal';
import { runFunc } from 'UTIL/common';

const BUMCT_MODAL = 'BUMCT_MODAL';
const BUMCT_MODAL_CONFIRM = 'BUMCT_MODAL_CONFIRM';
const BUMCT_MODAL_NO_DATA = 'BUMCT_MODAL_NO_DATA';

class Modal extends React.Component {
  constructor(props) {
    super(props);

    this.getDefaultDate = this.getDefaultDate.bind(this);
    this.handleNoData = this.handleNoData.bind(this);
    this.confirm = this.confirm.bind(this);
    this.footerTipsRender = this.footerTipsRender.bind(this);
  }

  static propTypes = {
    // 用于匹配触发事件的对象
    actionId: PropTypes.oneOfType([PropTypes.string, PropTypes.number]).isRequired,
    actionType: PropTypes.string,
    classId: PropTypes.oneOfType([PropTypes.string, PropTypes.number]).isRequired,
    productType: PropTypes.string.isRequired,
    onCancel: PropTypes.func.isRequired,
    title: PropTypes.string,
    // YYYY-MM-DD
    tagDate: PropTypes.string,
    data: PropTypes.any,
    beforeNowDisabled: PropTypes.bool,
    beforeNowDays: PropTypes.number,
    beforeNowMaxCourse: PropTypes.number,
    defaultSelectType: PropTypes.oneOf(['FIRST', 'ALL']),
    multiSelect: PropTypes.bool,
    footerTips: PropTypes.string,
    noSelectedCourseConfirmTip: PropTypes.string,
  };

  static defaultProps = {
    title: '选择课程时间',
    beforeNowDisabled: false,
    beforeNowDays: 0,
    beforeNowMaxCourse: 0,
    multiSelect: false,
    footerTips: undefined,
  };

  getDefaultDate(list, today, prevDay) {
    if (this.props.tagDate) {
      return this.getTagCourseDate(list, this.props.tagDate);
    } else if (this.props.defaultSelectType) {
      switch (this.props.defaultSelectType) {
        case 'FIRST':
          return this.getValidCourseDate(list);
        case 'ALL':
          if (this.props.multiSelect) {
            return this.getAllValidCourseDate(list);
          }
          return [];
        default:
          return [];
      }
    }

    return [];
  }

  getTagCourseDate = (list, tagDate) => {
    if (tagDate) {
      const index = list.findIndex((item) => item.courseDate === this.props.tagDate);
      if (index > -1) {
        return [index];
      }
    }

    return [];
  };

  getValidCourseDate = (list) => {
    let result = [];
    for (let i = 0; i < list.length; i++) {
      if (!list[i].disabled) {
        result = [i];
        break;
      }
    }
    return result;
  };

  getAllValidCourseDate = (list) => {
    let result = [];
    for (let i = 0; i < list.length; i++) {
      if (!list[i].disabled) {
        result.push(i);
      }
    }
    return result;
  };

  handleNoData() {
    global.GlobalEmitter.create(BUMCT_MODAL).trigger(BUMCT_MODAL_NO_DATA, {
      actionId: this.props.actionId,
      actionType: this.props.actionType,
    });
    runFunc(this.props.onCancel);
  }

  confirm(item) {
    global.GlobalEmitter.create(BUMCT_MODAL).trigger(BUMCT_MODAL_CONFIRM, {
      actionId: this.props.actionId,
      actionType: this.props.actionType,
      courseDate: item.courseDate,
      data: this.props.data,
    });
    runFunc(this.props.onCancel);
  }

  footerTipsRender() {
    return typeof this.props.footerTips === 'string' && <span style={{ color: 'red' }}>{this.props.footerTips}</span>;
  }

  render() {
    return (
      <UpdateModal
        noSelectedCourseConfirmTip={this.props.noSelectedCourseConfirmTip}
        title={this.props.title}
        classId={this.props.classId}
        students={this.props.students}
        productType={this.props.productType}
        onCancel={this.props.onCancel}
        onConfirm={this.confirm}
        defaultSelect={this.getDefaultDate}
        noDataDirectCallback={this.handleNoData}
        footerTipsRender={this.footerTipsRender}
      />
    );
  }
}

export default Modal;
