import React from 'react';
import Modal from 'COMPONENT/Modal';
import { connect } from 'dva';
import styles from './index.less';
import Loaders from './loaders';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { message } from 'antd';
import moment from 'moment';

const momentFormat = 'YYYY-MM-DD HH:mm';

class CancelPrePare extends React.Component {
  constructor(props) {
    super(props);
    this.isInOldPage = !!props.oldPageParams;

    this.state = {
      // 从旧页面过来的数据
      ...(this.isInOldPage
        ? {
            studentId: props.oldPageParams.studentId,
            miniClassId: props.oldPageParams.miniClassId,
          }
        : {
            studentId: props.studentId,
            miniClassId: props.miniClassId,
          }),

      loading: false,
      attendedCourse: [], // 学生已考勤课程
      attendingCourse: [], // 学生未考勤课程

      courseIds: [], // 被选中的课程
    };
  }

  componentDidMount() {
    const { miniClassId, studentId, courseIds } = this.state;
    this.setState({ loading: true });
    Loaders.getMiniClassCourseForPrepareDropApi({ miniClassId, studentId })
      .then(({ attendedCourse, attendingCourse, timeStamp }) => {
        attendingCourse.forEach((course) => {
          if (course.courseStatus === 'NEW' && !course.disabled) {
            courseIds.push(course.miniClassCourseId);
          }
        });

        this.setState({
          courseIds,
          attendedCourse,
          attendingCourse,
          loading: false,
        });
      })
      .catch((error) => {
        this.props.onClose();
      });
  }

  disabledAdapter = (list, today, tagDay) => {
    if (today < list[0].courseDay) {
      return list;
    }

    const len = list.length;

    let baseIndex = -1;
    if (today < list[len - 1].courseDay) {
      baseIndex = list.findIndex((item) => today < item.courseDay);
    } else {
      baseIndex = len;
    }

    let tagIndex = list.findIndex((item) => tagDay < item.courseDay);
    if (tagIndex === -1) {
      if (tagDay > list[len - 1].courseDay) {
        tagIndex = len;
      } else {
        tagIndex = 0;
      }
    }
    const maxCourse = Infinity;
    const end = baseIndex - tagIndex >= maxCourse ? baseIndex - maxCourse : tagIndex;
    for (let i = 0; i < end; i++) {
      list[i].disabled = true;
    }

    return list;
  };

  /**
   * 讲数转换
   */
  courseNumTranslate = (courseNum) => {
    return `第${courseNum}讲`;
  };

  /**
   * 课程点击
   */
  onClickCourse = (miniClassCourseId, courseIds) => {
    if (courseIds.includes(miniClassCourseId)) {
      let index = courseIds.indexOf(miniClassCourseId);
      courseIds.splice(index, 1);
      this.setState({ courseIds });
    } else {
      courseIds.push(miniClassCourseId);
      this.setState({ courseIds });
    }
  };

  /**
   * 取消预退班
   */
  cancelPrepareDropMiniClass = async () => {
    const { miniClassId, studentId, courseIds } = this.state;
    const params = { miniClassId, studentId, courseIds: courseIds.join(',') };
    const { resultCode } = await Loaders.cancelPrepareDropMiniClassApi(params);
    if (resultCode === 0) {
      showMessage('学生取消预退已提交完成', 'success');
      this.props.onClose();
      if (this.isInOldPage) {
        this.props.dispatch({
          type: 'main/refreshCurrentTab',
        });
      } else {
        this.props.onOK();
      }
    } else {
      this.props.onClose();
      showMessage('取消预退班失败');
    }
  };

  // 是否有选择课程
  checkSelectCourses = () => {
    // 有未考勤课且没选择课程需提醒
    if (this.state.attendingCourse.length && !this.state.courseIds.length) {
      return message.warning('请选择具体回班课程');
    }
    this.cancelPrepareDropMiniClass();
  };

  render() {
    const { loading, attendedCourse, attendingCourse, courseIds } = this.state;
    return (
      <Modal
        loading={loading}
        title="取消预退"
        footerTipText="学生将恢复在读状态"
        onCancel={this.props.onClose}
        width={520}
        onOk={this.checkSelectCourses}
      >
        <h2>学生已考勤</h2>
        <div className={styles.attend + ' ' + 'clearfix'}>
          {attendedCourse.length === 0 ? (
            <p className={styles.tip}>该学生无已考勤课程</p>
          ) : (
            attendedCourse.map((course) => (
              <div className={styles.attended} key={course.miniClassCourseId}>
                <p>
                  {course.courseDate}
                  {this.courseNumTranslate(course.courseNum)}
                </p>
              </div>
            ))
          )}
        </div>

        <h2>学生未考勤</h2>
        <div className={styles.attend + ' ' + 'clearfix'}>
          {attendingCourse.length === 0 ? (
            <p className={styles.tip}>该学生无未考勤课程</p>
          ) : (
            attendingCourse.map((course) => {
              const { miniClassCourseId, courseNum, courseDate, disabled } = course;

              return disabled ? (
                <div className={styles.attended} key={miniClassCourseId}>
                  <p>
                    {courseDate}
                    {this.courseNumTranslate(courseNum)}
                  </p>
                </div>
              ) : (
                <div
                  className={courseIds.includes(miniClassCourseId) ? styles.selected : ''}
                  onClick={() => {
                    this.onClickCourse(miniClassCourseId, courseIds);
                  }}
                  key={miniClassCourseId}
                >
                  <p>
                    {courseDate}
                    {this.courseNumTranslate(courseNum)}
                  </p>
                </div>
              );
            })
          )}
        </div>
        <div className={styles.bodyFooter}>
          <span className={styles.warningTips} />
        </div>
      </Modal>
    );
  }
}

export default connect()(CancelPrePare);
