import React from 'react';
import { connect } from 'dva/index';
import Modal from 'COMPONENT/Modal';
import Select from 'COMPONENT/Select';
import FormItem from 'COMPONENT/FormItem';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { loadBranchCampusDict } from 'UTIL/dict';
import {
  peiYouTest,
  peiYouPre,
  peiYou,
  twoTeacherTest,
  twoTeacherPre,
  twoTeacher,
  uat,
  pre,
  prd,
} from 'CONST/buildEnv';
import ShareQRCode from 'COMPONENT/ShareQRCode';
const formCfg = {
  labelWidth: 100,
  style: {
    padding: '0',
    width: '300px',
  },
};
const buildBaseUrl = () => {
  let result = '';
  if (process.env) {
    switch (process.env.BUILD_ENV) {
      case pre:
      case peiYouPre:
      case twoTeacherPre:
        result = 'https://cmall-pre.xiao100.com';
        break;
      case prd:
      case peiYou:
      case twoTeacher:
        result = 'https://cmall.xiao100.com';
        break;
      default:
        result = 'https://cmall-test.xiao100.com';
        break;
    }
    switch (process.env.BUILD_ENV) {
      case uat:
      case pre:
      case prd:
        result += '/student-schedule/xhboss/login?';
        break;
      case peiYouTest:
      case peiYouPre:
      case peiYou:
        result += '/student-schedule/pyboss/login?';
        break;
      case twoTeacherTest:
      case twoTeacherPre:
      case twoTeacher:
        result += '/student-schedule/ssboss/login?';
        break;
      default:
        result += '/student-schedule/xhboss/login?';
        break;
    }
  }
  return result;
};

class DownloadStudentSchedule extends React.Component {
  constructor(props) {
    super(props);

    const { user } = this.props;
    this.state = {
      loading: true,
      campusId: null,
      campusDict: [],
      qrcodeVisiable: false,
      qrcodeVisible: false,
      qrcodeUrl: '',
    };
  }

  confirm = async () => {
    const { campusId } = this.state;
    if (!campusId) {
      showMessage('请选择学习中心', 'error');
      return false;
    }
    this.setState({
      qrcodeVisiable: true,
    });
  };

  componentDidMount() {
    this.initContent();
  }

  initContent = async () => {
    this.setState({ loading: true });
    const campusDict = await loadBranchCampusDict();
    this.setState({
      campusDict: campusDict,
      loading: false,
    });
  };

  render() {
    const { onClose } = this.props;
    const { loading, campusId, campusDict, qrcodeVisiable, qrcodeUrl } = this.state;
    return (
      <Modal
        loading={loading}
        title="选择学习中心"
        onCancel={onClose}
        okText="生成二维码"
        cancelText="关闭"
        onOk={this.confirm}
        ref={(c) => (this.modal = getWrappedInstance(c))}
        width={500}
      >
        <div>
          <FormItem {...formCfg} name="选择学习中心" required>
            <Select
              placeholder="选择学习中心后生成二维码"
              showSearch
              dict={campusDict}
              value={campusId}
              onChange={(val) => this.setState({ campusId: val, qrcodeUrl: `${buildBaseUrl()}campusId=${val}` })}
            />
          </FormItem>
        </div>
        <ShareQRCode
          title="分享"
          shareTitle="扫码查询学生课程"
          shareText={qrcodeUrl}
          fileName="引流活动_二维码"
          visible={qrcodeVisiable}
          onCancel={() => {
            this.setState({ qrcodeVisiable: false });
          }}
        />
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    user: state.account.user,
  };
};
export default connect(mapStateToProps)(DownloadStudentSchedule);
