import React from 'react';
import Modal from 'COMPONENT/Modal';
import Icon from 'COMPONENT/Icon';
import styles from './index.less';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import {
  getMiniClassStuRemainFinAndHour,
  getMiniClassAttendedStudent,
  deleteStudentInMiniClasss,
} from 'SERVICE/course';

/* 小班退班 */
export default class CancelPrePare extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false,
      classInfo: {},
      studentList: [], // TODO: 未来兼容接入旧系统的批量退班
      attendedStudentNames: [], // 已考勤学生
    };
  }

  componentDidMount() {
    this.init();
  }

  async init() {
    this.setState({ loading: true });
    await this.getStudentInfo();
    await this.getAttendedStudent();
    this.setState({ loading: false });
  }

  async getStudentInfo() {
    const { miniClassId, studentId } = this.props;
    const res = await getMiniClassStuRemainFinAndHour({
      miniClassId,
      studentId,
    });
    this.setState({
      studentList: [
        {
          studentId,
          studentName: res.studentName,
          remainCourseHour: res.remainCourseHour,
          remainFinance: res.remainFinance,
        },
      ],
    });
  }

  async getAttendedStudent() {
    const { miniClassId, studentId } = this.props;
    const res = await getMiniClassAttendedStudent({
      smallClassId: miniClassId,
      studentIds: [studentId].join(','),
    });
    this.setState({
      attendedStudentNames: res.map((student) => student.name),
    });
  }

  handleDrop = async () => {
    const { miniClassId, studentId } = this.props;
    const res = await deleteStudentInMiniClasss({
      smallClassId: miniClassId,
      studentIds: [studentId].join(','),
    });
    if (res.resultCode == 0) {
      showMessage('退班成功', 'success');
      this.close();
      this.props.onOK();
    } else {
      showMessage('退班失败');
    }
  };

  close = () => {
    this.props.onClose();
  };

  render() {
    const { studentList, attendedStudentNames } = this.state;
    return (
      <Modal
        title="小班退班"
        okText="退班"
        loading={this.state.loading}
        onCancel={this.close}
        onOk={this.handleDrop}
        width={520}
      >
        <div className={styles.body}>
          <header className={styles.header}>
            <Icon className={styles.icon} type="warning" size={24} color="#FFB444" />
            <span>是否对以下学生做退班处理？</span>
          </header>
          <section className={styles.studentList}>
            {studentList.map((student, index) => (
              <div key={index} className={styles.student}>
                <span>学生：{student.studentName}</span>
                <span>剩余资金：{student.remainFinance}</span>
                <span>（{student.remainCourseHour}小时）</span>
              </div>
            ))}
          </section>
          {attendedStudentNames.length ? (
            <section className={styles.attended}>
              {attendedStudentNames.map((name) => (
                <span>{name}</span>
              ))}
              学生有已考勤课程尚未结算，操作退班或者转班后相对应考勤记录会被清除。
            </section>
          ) : null}
        </div>
      </Modal>
    );
  }
}
