import React from 'react';
import PropTypes from 'prop-types';
import moment from 'moment';
import { Table } from 'antd';
import { Modal } from 'COMPONENT';
import PreviewImgs from 'COMPONENT/PreviewImgs';

import { getEsignatureList } from 'SERVICE/course';
const statusMap = {
  COMPLETED: '已完成',
  UNCOMPLETED: '未完成',
};

class Appointment extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      data: [],
      currentPreviewImg: '',
      showPreview: false,
    };
  }

  static propTypes = {
    courseId: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
    classType: PropTypes.string,
  };

  componentDidMount() {
    const { courseId, classType } = this.props;

    this.setLoading(true);

    getEsignatureList({
      courseId,
      classType,
    })
      .then((res) => {
        this.setState({
          data: res.data || [],
        });
      })
      .finally(() => {
        this.setLoading(false);
      });
  }

  setLoading = (loading) => {
    this.setState({
      loading,
    });
  };

  preview = (img) => {
    this.setState({
      showPreview: true,
      currentPreviewImg: img,
    });
  };

  render() {
    const { onCancel } = this.props;
    const { loading, data, currentPreviewImg, showPreview } = this.state;
    const preview = this.preview;

    const columns = [
      {
        title: '学生姓名',
        dataIndex: 'studentName',
        key: 'studentName',
      },
      {
        title: '签名状态',
        dataIndex: 'status',
        key: 'status',
        render(text) {
          return statusMap[text] || '';
        },
      },
      {
        title: '签名时间',
        dataIndex: 'createTime',
        key: 'createTime',
        render(text) {
          return text ? moment(text, 'x').format('YYYY-MM-DD HH:mm:ss') : '';
        },
      },
      {
        title: '签名照片',
        dataIndex: 'picUrl',
        key: 'picUrl',
        width: 120,
        render(text) {
          return (
            <img
              onClick={() => preview(text)}
              style={{
                maxWidth: '100%',
                objectFit: 'scale-down',
              }}
              src={text}
            />
          );
        },
      },
    ];

    return (
      <Modal
        title="电子签名"
        cancelText="关闭"
        width={800}
        loading={loading}
        onCancel={onCancel}
        maskClosable={false}
        okBtnShow={false}
      >
        <Table columns={columns} dataSource={data} rowKey="studentId" />

        {showPreview && (
          <Modal
            width={800}
            okBtnShow={false}
            cancelBtnShow={false}
            closable={true}
            onClose={() => this.setState({ showPreview: false })}
          >
            <PreviewImgs list={[currentPreviewImg]} activeSrc={currentPreviewImg} showThumbnail={false} />
          </Modal>
        )}
      </Modal>
    );
  }
}

export default Appointment;
