import React from 'react';
import { Row, Col } from 'antd';
import Input from 'COMPONENT/Input';
import { Modal as AntdModal } from 'antd';
import Popover from 'COMPONENT/Popover';

function attributionType(type) {
  let result = '';
  switch (type) {
    case 'USER_USER':
      result = '个人';
      break;
    case 'USER_CAMPUS':
      result = '学习中心团队';
      break;
    case 'USER_BRANCH':
      result = '分公司团队';
      break;
  }
  return <span>{result}</span>;
}
function attributionName(params) {
  let key = '';
  switch (params.subBonusDistributeType) {
    case 'USER_USER':
      key = 'bonusStaffName';
      break;
    default:
      key = 'bonusOrgName';
  }
  let data = params[key];
  let flag = params.enableFlg === 1 ? '（无效）' : '';
  return (
    <span>
      {data}
      {flag}
    </span>
  );
}

const AttributionItem = (props) => {
  const { data, amounts, handle, index } = props;
  return (
    <Row>
      <Col span={4}>{attributionType(data.subBonusDistributeType)}</Col>
      <Col span={4}>{attributionName(data)}</Col>
      <Col span={6} className="hom-input-no-num">
        <Popover icon="info" maxWidth={180} content="请输入责任金额">
          <Input
            type="number"
            placeholder="请输入责任金额"
            value={amounts}
            onChange={(e) => {
              handle(e.target.value, index);
            }}
            style={{ lineHeight: '36px' }}
            disabled={
              data.subBonusDistributeType === 'USER_USER' && (!data.bonusStaffName || data.bonusStaffName === '')
            }
          />
        </Popover>
      </Col>
    </Row>
  );
};

class AttributionList extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      amountList: [],
      amountTotalSaver: 0,
      amountTotal: 0,
    };
  }

  componentDidMount() {
    this.init();
  }

  init = () => {
    // 默认 amount 为 0
    let result = [];
    let amounts = 0;

    if (this.props.data.length > 0) {
      result = this.props.data.map((item) => item.amount);
      amounts = this.props.data.reduce((total, current) => total + current.maxAmount, 0);
    }

    this.setState({
      amountList: result,
      amountTotalSaver: amounts,
      amountTotal: amounts,
    });
  };

  handleInput = (val, index) => {
    val = Number(val);
    if (val < 0) {
      val = Math.abs(val);
    }
    const { handleUpdate } = this.props;
    const { amountList, amountTotalSaver } = this.state;
    let arr = amountList.slice();
    arr[index] = val;
    let amounts = arr.reduce((prevItem, item) => Number(prevItem) + Number(item));
    if (amounts > amountTotalSaver) {
      AntdModal.info({
        title: '归属类型金额总额不能超过' + amountTotalSaver,
        content: '',
        className: 'homAntdModal',
      });
      return;
    }
    handleUpdate({ val, index });
    arr[index] = val;
    this.setState({
      amountList: arr,
      amountTotal: amounts,
    });
  };

  render() {
    const { data } = this.props;
    const { amountList } = this.state;

    let isNoData = data.length === 0;

    return (
      <div>
        <Row style={{ fontWeight: 'bold' }}>
          <Col span={4}>责任归属类型</Col>
          <Col span={4}>责任归属</Col>
          <Col span={6}>责任业绩</Col>
        </Row>
        {data.map((item, index) => (
          <AttributionItem
            data={item}
            amounts={amountList[index]}
            handle={this.handleInput}
            index={index}
            key={index}
          />
        ))}
        {isNoData && (
          <Row>
            <Col span={4}>无</Col>
            <Col span={4}>无</Col>
            <Col span={6}>无</Col>
          </Row>
        )}
      </div>
    );
  }
}

export default AttributionList;
