/**
 * Created by Haolin on 2018/7/7.
 */
import React from 'react';
import { connect } from 'dva';
import { Row, Col } from 'antd';
import Input from 'COMPONENT/Input';
import { Modal as AntdModal } from 'antd';
import Popover from 'COMPONENT/Popover';

function FeeBlock(props) {
  const { total, target } = props;
  let over = target - total;
  return (
    <div>
      {props.children}
      <div>
        合计：{total}，剩余{over}
      </div>
    </div>
  );
}

const CampusItem = (props) => {
  const { data, amounts, handle, index } = props;
  return (
    <Row>
      <Col span={4}>
        <span>{data.bonusOrgName}</span>
      </Col>
      <Col span={6} className="hom-input-no-num">
        <Popover icon="info" maxWidth={180} content="请输入责任金额">
          <Input
            type="number"
            placeholder="请输入责任金额"
            value={amounts}
            onChange={(e) => {
              handle(e.target.value, index);
            }}
            style={{ lineHeight: '36px' }}
          />
        </Popover>
      </Col>
    </Row>
  );
};

class CampusList extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      amountList: [],
      amountTotalSaver: 0,
      amountTotal: 0,
    };
  }

  componentWillMount() {
    this.initPage();
  }

  componentDidMount() {}

  componentWillUnmount() {}

  /**
   * 初始化 -> amountList,amountTotal
   * */
  initPage = () => {
    const { data } = this.props;
    let result = data.map((item) => item.amount);
    let amounts = data.length > 0 ? result.reduce((prevItem, item) => prevItem + item) : 0;
    this.updateState({ amountList: result, amountTotalSaver: amounts, amountTotal: amounts });
  };
  /**
   * 更新state
   * */
  updateState = (newState) => {
    this.setState((prevState, props) => {
      return {
        prevState,
        ...newState,
      };
    });
  };
  /**
   * 监听“学习中心责任”责任金额
   * */
  handleInput = (val, index) => {
    val = Number(val);
    if (val < 0) {
      val = Math.abs(val);
    }
    const { handleUpdate } = this.props;
    const { amountList, amountTotalSaver } = this.state;
    let arr = amountList.slice();
    arr[index] = val;
    let amounts = arr.reduce((prevItem, item) => prevItem + item);
    if (amounts > amountTotalSaver) {
      AntdModal.info({
        title: '责任金额总额不能超过' + amountTotalSaver,
        content: '',
        className: 'homAntdModal',
      });
      return;
    }
    handleUpdate({ val, index });
    // arr[index] = val!==0?val:"";
    arr[index] = val;
    this.updateState({ amountList: arr, amountTotal: amounts });
  };

  render() {
    const { data } = this.props;
    const { amountList, amountTotal, amountTotalSaver } = this.state;

    let isNoData = data.length === 0;

    return (
      <FeeBlock total={amountTotal} target={amountTotalSaver}>
        <Row style={{ fontWeight: 'bold' }}>
          <Col span={4}>责任学习中心</Col>
          <Col span={6}>责任业绩</Col>
        </Row>
        {data.map((item, index) => (
          <CampusItem data={item} amounts={amountList[index]} handle={this.handleInput} index={index} key={index} />
        ))}
        {isNoData && (
          <Row>
            <Col span={4}>无</Col>
            <Col span={6}>无</Col>
          </Row>
        )}
      </FeeBlock>
    );
  }
}

export default connect()(CampusList);
