/**
 * Created by Haolin on 2018/7/7.
 */
import React from 'react';
import { connect } from 'dva';
import { Row, Col } from 'antd';
import Input from 'COMPONENT/Input';
import { Modal as AntdModal } from 'antd';
import Popover from 'COMPONENT/Popover';

function FeeBlock(props) {
  const { total, target } = props;
  let over = (target - total).toFixed(2);
  return (
    <div>
      {props.children}
      <div>
        合计：{total}，剩余{over}
      </div>
    </div>
  );
}

const CampusItem = (props) => {
  const { data, amounts, handle, index } = props;
  return (
    <Row>
      <Col span={4}>
        <span>{data.bonusOrgName}</span>
      </Col>
      <Col span={6} className="hom-input-no-num">
        <Popover icon="info" maxWidth={180} content="请输入责任金额">
          <Input
            type="number"
            placeholder="请输入责任金额"
            value={amounts}
            onChange={(e) => {
              handle(e.target.value, index);
            }}
            style={{ lineHeight: '36px' }}
          />
        </Popover>
      </Col>
    </Row>
  );
};

class CampusList extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      amountTotal: 0,
    };
  }

  componentDidMount() {
    this.init();
  }

  componentDidUpdate(prevState) {
    if (prevState.refundTotalAmountActual !== this.props.refundTotalAmountActual) {
      this.init();
    }
  }

  init() {
    const { data } = this.props;
    let amounts = 0;
    data.forEach((item) => (amounts += Number(item.amount)));

    this.updateState({ amountTotal: amounts });
  }

  /**
   * 更新state
   * */
  updateState = (newState) => {
    this.setState((prevState, props) => {
      return {
        prevState,
        ...newState,
      };
    });
  };

  /**
   * 监听“学习中心责任”责任金额
   * */
  handleInput = (val, index) => {
    val = Number(val);
    if (val < 0) {
      val = Math.abs(val);
    }

    const { data, handleUpdate, refundTotalAmountActual } = this.props;
    let arr = [...data];
    arr[index].amount = val;
    let amounts = 0;
    arr.forEach((item) => (amounts += item.amount));

    if (amounts > refundTotalAmountActual) {
      AntdModal.info({
        title: '责任金额总额不能超过' + refundTotalAmountActual,
        content: '',
        className: 'homAntdModal',
      });
      return;
    }

    handleUpdate({ val, index });
    arr[index] = val;
    this.updateState({ amountTotal: amounts });
  };

  render() {
    const { data, refundTotalAmountActual } = this.props;
    const { amountTotal } = this.state;

    let isNoData = data.length === 0;

    return (
      <FeeBlock total={amountTotal} target={refundTotalAmountActual}>
        <Row style={{ fontWeight: 'bold' }}>
          <Col span={4}>责任学习中心</Col>
          <Col span={6}>责任业绩</Col>
        </Row>
        {data.map((item, index) => (
          <CampusItem data={item} amounts={item.amount} handle={this.handleInput} index={index} key={index} />
        ))}
        {isNoData && (
          <Row>
            <Col span={4}>无</Col>
            <Col span={6}>无</Col>
          </Row>
        )}
      </FeeBlock>
    );
  }
}

export default connect()(CampusList);
