/**
 * 精英班中途退费模态框
 * @property {number} type 操作类型 1: 退费申请  2、查看退费申请  3、退费审批  4、查看退费审批
 * @property {string} promiseStudentId 精英班学生ID
 * @property {string} contractProductId 合同产品ID
 * @property {function} onClose 模态框关闭事件触发时的回调
 * @property {function?} onSave 模态框保存成功后的回调
 */
import React from 'react';
import { connect } from 'dva';
import { debounce, noop } from 'lodash';
import { Row, Col, Form, AutoComplete } from 'antd';
import Radio from 'COMPONENT/Radio';
import Button from 'COMPONENT/Button';
import Modal from 'COMPONENT/Modal';
import Block from 'COMPONENT/Block';
import Popover from 'COMPONENT/Popover';
import TextArea from 'COMPONENT/TextArea';
import FormItem from 'COMPONENT/FormItem';
import Input from 'COMPONENT/Input';
import InputNum from 'COMPONENT/InputNumber';
import HomSection from './SectionCard';
import { getDictTextByVal, loadCommonDict } from 'UTIL/dict';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import {
  getEcsContractInfo,
  getEcsContractChargeInfo,
  findStudentInfoByPromiseStudentId,
  submitPromiseResult,
  getPromiseResultDetail,
  getEcsTargetEndContractChargeInfo,
  getOaPromiseClassTargetEndAuditUserList,
  getDynamicVoByPromiseStudentId,
} from 'SERVICE/course';
import StudentInfo from './StudentInfo';
import ContractInfo from '../PromiseFeeDetail/ContractInfo';
import FeeItem from '../PromiseFeeDetail/FeeItem';
import ApprovalInfo from '../common/ApprovalInfo';
import ApprovalSelect from '../common/ApprovalSelect';
import { formatMoney } from '../PromiseFeeDetail/common';
import styles from '../PromiseFeeDetail/styles.less';
import styles_ from './styles.less';
import { Modal as AntdModal } from 'antd';
import UploadImg from '../UploadImg';
import CampusList from './CampusList';
import AttributionList from './AttributionList';
import ShowLiability from './ShowLiability';
import calc from './utils/calc';
import { ecsProcessStep } from './utils';
import { runFunc, errorsHandle } from 'UTIL/common';
import { getBankCardInfo } from 'SERVICE/common';
import { searchBankBranchLoader } from 'PAGE/common/loaders';
import BankTooltip from 'BLOC/BankTooltip';

const RadioGroup = Radio.Group;
const Option = AutoComplete.Option;
const accountNumberID = 'RESULT_ACCOUNT_NUMBER_ID';

/**
 * 扣费明细表中，一个季度的一个扣费项目对应的单元格所显示的内容
 * @property {object} data 明细数据，格式如下：
 {
   regular: number 正价金额
   discount: number 优惠金额
   regularHours: number 正价课时数
   discountHours: number 优惠课时数
  }
 */
const FeeCell = ({ data }) => {
  const { regular, discount, regularHours, discountHours } = data || {};
  return (
    <div>
      <FeeItem name="正价扣费" money={regular} />
      {regularHours !== undefined && <div>{`（${regularHours}小时）`}</div>}
      <FeeItem type="discount" name="优惠扣费" money={discount} />
      {discountHours !== undefined && <div>{`（${discountHours}小时）`}</div>}
    </div>
  );
};
/**
 * 在antd的Button组件基础上添加气泡信息提示
 * @property {any} ...props antd的Button组件所有属性
 * @property {string?} info 鼠标悬停时的提示信息
 * @property {string?} error 错误信息
 * @property {string?} onErrorClose 错误信息关闭事件触发时的回调
 */
const ButtonWithPop = ({ info, error, onErrorClose, ...props }) => {
  let btn = <Button {...props} />;
  if (info) {
    btn = (
      <Popover icon="info" content={info} maxWidth={300}>
        {btn}
      </Popover>
    );
  } else if (error) {
    btn = (
      <Popover icon="error" content={error} maxWidth={300} trigger="click" visible={true} onClose={onErrorClose}>
        {btn}
      </Popover>
    );
  }
  btn = <span style={{ marginLeft: 8 }}>{btn}</span>;
  return btn;
};

class PromiseResultDetail extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 内容加载状态
      studentInfo: {}, // {object} 学生信息
      contractInfo: {}, // {object} 合同信息
      quarterFeeDetails: [], // {Array} 各季度扣费明细
      refundMode: '', // {string} 退费方式 [ RETURN_TO_ACCOUNT：退到电子账户 | RETURN_TO_CUSTOMER：直接退款 ]
      historyOpinions: [], // {Array} 历史审批意见
      opinion: '', // {string} 审批意见
      // 出纳审批意见
      cashierAuditRemark: '',
      commitApplyBtnError: null, // {string} 提交申请按钮的错误信息
      cancelRefundBtnError: null, // {string} 撤销退费按钮的错误信息
      approvalPassBtnError: null, // {string} 审批通过按钮的错误信息
      totals: {}, // {object} 根据原始数据计算得到的各种合计数据
      statusDict: null, // {[object]} 字典 学生精英班状态
      isBindVip: null, // {boolean} 是否绑定VIP服务费
      error: null, // {string} 数据加载失败的错误信息
      vipRealCostAmount: 0, // 实际已经扣取的vip费用
      divisionAmount: 0, // 目标失败时，退费 合计划归总额
      refundTotalAmount: 0, // 合计退款金额
      refundTotalAmountActual: '0', // 审核时，手动修改退款金额
      id: null, //目标id
      schoolFraction: '', // 学校分数线
      studentFraction: '', // 学生总分
      fraction: false, // {Boolean} 是否输入学校分数线与总分
      chooseResult: 0, // {Number} 选中目标结果 [ 1：成功 | 2：失败 ]
      autoChoose: 0, // 保存"填入分数后自动选择"的目标结果
      changeReason: '', // 手动修改目标结果原因
      refundReason: '', //退款到电子账号，退款说明
      fileList: [], // 图片列表，保存所有字段的图片
      file_0: null,
      file_1: null,
      file_2: null,
      file_3: null,
      file_4: null,
      cashierFile: null,
      chooseResultToggle: false, //是否可以选择目标结果,

      campusLiability: [], // {Array} 学习中心责任
      originCampusLiability: [], // 学习中心责任 原始数据
      campusTotal: 0,
      attributionLiability: [], // {Array} 归属类型
      attributionTotal: 0,
      returnBankAccount: null,
      returnBankAccountOwner: null,
      returnBankName: null,
      branchCode: '', // 退款支行号
      branchBankList: [], // 支行数据
      //流程节点ID
      nodeId: null,
      approvalSelectVisible: false,
      selectedApproval: '',
      approvalUsers: [],
      approvalDynamic: [],
    };

    this.step = 0;
    this.getBankInfo = this.getBankInfo.bind(this);
    this.onBranchBankSelect = this.onBranchBankSelect.bind(this);
    this.searchBankInfo = debounce(this.searchBankInfo, 300).bind(this);
    this.checkBankSelect = this.checkBankSelect.bind(this);
  }

  componentDidMount() {
    this.initContent();
  }

  initContent = () => {
    const { promiseStudentId, contractProductId, type } = this.props;
    this.setState({ loading: true });
    // 非提交页面加载审批动态非提交页面加载审批动态
    this.props.type !== 5 && this.loadApprovalDynamic(this.props.promiseStudentId);
    // 加载数据信息
    Promise.all([
      this.loadStudentInfo(promiseStudentId),
      this.loadContractInfo(contractProductId),
      this.loadFeeDetails(contractProductId),
      this.loadResultDetail(promiseStudentId),
      loadCommonDict('StudentPromiseStatus'),
    ]).then(
      ([studentInfo, contractInfo, feeDetails, resultDetail, statusDict]) => {
        const { isBindVip, quarterFeeDetails, vipRealCostAmount } = feeDetails;
        const {
          actualMoney,
          vipRealPayAmount,
          campusList: contractInfoCampus,
          userList: contractInfoUser,
        } = contractInfo;
        const { campusList: resultDetailCampus, userList: resultDetailUser } = resultDetail;
        const { promiseStudentStatus } = studentInfo;
        const courseRealPayAmount = Number(calc.sub(actualMoney, vipRealPayAmount));

        const _refundTotalAmountActual = Number(resultDetail.refundTotalAmountActual);
        const _refundTotalAmount = _refundTotalAmountActual > 0 ? _refundTotalAmountActual : courseRealPayAmount;

        //学习中心责任
        let campus = resultDetailCampus || contractInfoCampus || [];
        campus = campus.filter((item) => item.productType !== 'OTHERS');
        //归属责任
        let attribution = resultDetailUser || contractInfoUser || [];
        attribution = attribution.filter((item) => item.productType !== 'OTHERS');

        // type: 7 已结课，目标结果审批
        if (type === 7) {
          campus = campus.filter((item) => item.bonusType === 'NORMAL');

          // type: 7 时 初始化 amount 为 0
          attribution = attribution
            .filter((item) => item.bonusType === 'NORMAL')
            .map((item) => {
              return {
                ...item,
                amount: 0,
                maxAmount: item.amount,
              };
            });
        } else if (type >= 8) {
          campus = campus.filter((item) => item.bonusType === 'FEEDBACK_REFUND');
          attribution = attribution.filter((item) => item.bonusType === 'FEEDBACK_REFUND');
        }

        //统计责任金额总额
        let campusTotal = 0;
        let attributionTotal = 0;
        if (campus.length > 0) {
          campusTotal = campus.map((item) => item.amount).reduce((prevItem, item) => prevItem + item);
        }

        if (attribution.length > 0) {
          attributionTotal = attribution.reduce((total, current) => total + current.amount, 0);
        }

        attribution.forEach((item) => {
          item.maxAmount = item.maxAmount || item.amount || 0;
          item.amount = item.amount || 0;
        });

        // 计算当前流程步骤
        this.step = ecsProcessStep(this.props.courseStatus, this.props.auditStatus, resultDetail.chooseResult);

        let branchBankList = [];
        if (studentInfo.returnBankName && studentInfo.branchCode) {
          branchBankList = [{ label: studentInfo.returnBankName, branchCode: studentInfo.branchCode }];
        }

        this.setState({
          loading: false,
          studentInfo: { ...studentInfo, promiseStudentStatusName: getDictTextByVal(statusDict, promiseStudentStatus) },
          courseStatus: studentInfo.promiseStudentStatus,
          contractInfo,
          ...resultDetail,
          quarterFeeDetails,
          totals: this.compute(actualMoney, quarterFeeDetails),
          statusDict,
          isBindVip,
          vipRealCostAmount,
          courseRealPayAmount: courseRealPayAmount,
          vipRealIncome: vipRealPayAmount - vipRealCostAmount,
          refundTotalAmount: _refundTotalAmount,
          refundTotalAmountActual: _refundTotalAmount.toString(),
          chooseResultToggle: type === 5 || type === 7,
          campusLiability: campus,
          originCampusLiability: [...campus],
          campusTotal: campusTotal,
          attributionLiability: attribution,
          attributionTotal: attributionTotal,
          cashierFile: studentInfo.cashierFile,
          cashierAuditRemark: studentInfo.cashierAuditRemark,
          returnBankAccount: studentInfo.returnBankAccount,
          returnBankName: studentInfo.returnBankName,
          branchCode: studentInfo.branchCode,
          branchBankList,
          returnBankAccountOwner: studentInfo.returnBankAccountOwner,
        });
      },
      (e) => {
        const error = e.text || '数据加载出错';
        this.setState({ loading: false, error });
      },
    );
  };

  // 加载学生信息
  loadStudentInfo = async (promiseStudentId) => {
    const apiData = await findStudentInfoByPromiseStudentId({ promiseStudentId }, true);
    return {
      id: apiData.id,
      name: apiData.studentName,
      cusName: apiData.customerName,
      gradeName: apiData.gradeName,
      contact: apiData.customerContact,
      promiseStudentStatus: apiData.courseStatus,
      promiseClassName: apiData.promiseClassName,
      signCourseName: apiData.productName,
      refundMode: apiData.returnType,
      opinion: apiData.auditRemark,
      oldAuditRemark: apiData.oldAuditRemark,
      managerContact: apiData.sudyManagerContact,
      managerName: apiData.sudyManagerName,
      studentCampusName: apiData.studentCampusName,
      cashierFile: apiData.cashierAuditCertPath ? { src: apiData.cashierAuditCertPath } : null,
      cashierAuditRemark: apiData.cashierAuditRemark,
      returnBankAccount: apiData.returnBankAccount,
      returnBankName: apiData.returnBankName,
      branchCode: apiData.branchCode,
      returnBankAccountOwner: apiData.returnBankAccountOwner,
    };
  };

  // 载合同信息
  loadContractInfo = async (contractProductId) => {
    const { type } = this.props;
    const apiData = await getEcsContractInfo({ contractProductId }, true);
    const {
      contractId,
      courseAmount,
      courseProAmount,
      realPaidAmount,
      vipAmount,
      vipProAmount,
      courseName,
      consumeAmount,
      vipConsumeAmount,
      vipRealPayAmount,
      realPayAmount,
      after2018,
      availableAmount,
      paidStatus,
      coursePaidAmount,
      vipPaidAmount,
      courseRealPayAmount,
      campusList,
      userList,
    } = apiData;
    const contractAmount = courseAmount + vipAmount;
    const contractPaidAmount = coursePaidAmount + vipPaidAmount;

    let campus = campusList.map((item) => {
      delete item.id;
      return item;
    });

    // 责任归属类型（个人责任/团队责任）申请默认为0
    let attribution = userList.map((item) => {
      delete item.id;
      let amount = item.amount;
      type === 5 && (amount = 0);
      return {
        ...item,
        amount,
      };
    });

    return {
      no: contractId,
      actualMoney: realPaidAmount,
      feeItems: [
        { name: 'VIP服务费', money: vipRealPayAmount + vipProAmount, discount: vipProAmount },
        { name: courseName, money: realPayAmount + courseProAmount, discount: courseProAmount },
      ],
      deductedBaseMoney: consumeAmount,
      deductedVipMoney: vipConsumeAmount,
      amount: contractAmount,
      paidAmount: contractPaidAmount,
      vipRealPayAmount,
      after2018,
      availableAmount,
      paidStatus,
      campusList: campus,
      userList: attribution,
    };
  };

  // 加载扣费明细
  loadFeeDetails = async (contractProductId) => {
    /*
     0 chargeInfo  array[object] true  扣费信息
     1 realAmount  number  true  实收扣费
     1 promotionQuantity number  true  优惠课时
     1 realQuantity  number  true  实际课时
     1 seasonId  string  true  季节
     1 promotionAmount number  true  优惠扣费
     0 vipFee  array[object] true  vip服务费
     1 contractProductId string  true  合同产品ID
     1 realAmount  number  true  实际金额
     1 quantity  number  true  数量
     1 consumeAmount number  true  课消金额
     1 price number  true  单价
     1 consumeQuanity  number  true  课消课时
     1 paidAmount  number  true  支付金额
     1 promotionAmount number  true  优惠金额
     1 planAmount  number  true  计划金额
     1 status
     */
    const { courseStatus, courseResult } = this.props;
    const apiData =
      courseStatus === '0' && courseResult === '0'
        ? await getEcsTargetEndContractChargeInfo({ contractProductId }, true)
        : await getEcsContractChargeInfo({ contractProductId }, true);
    const { chargeInfo, result } = apiData;
    const defaultQuarter = {
      base: { regular: 0, discount: 0, regularHours: 0, discountHours: 0 },
      vip: { regular: 0, discount: 0 },
    };
    let summer = { ...defaultQuarter, id: 'DAT0000000232' };
    let autumn = { ...defaultQuarter, id: 'DAT0000000233' };
    let winter = { ...defaultQuarter, id: 'DAT0000000230' };
    let spring = { ...defaultQuarter, id: 'DAT0000000231' };
    chargeInfo.forEach((f) => {
      const {
        realAmount,
        promotionQuantity,
        realQuantity,
        seasonId,
        promotionAmount,
        realVipAmount,
        proVipAmount,
        realNatureHour,
        promotionNatureHour,
      } = f;
      const fee = {
        regular: realAmount,
        discount: promotionAmount,
        regularHours: realNatureHour,
        discountHours: promotionNatureHour,
      };
      const vipFee = {
        regular: realVipAmount,
        discount: proVipAmount,
      };
      if (seasonId === summer.id) {
        summer.base = fee;
        summer.vip = vipFee;
      } else if (seasonId === autumn.id) {
        autumn.base = fee;
        autumn.vip = vipFee;
      } else if (seasonId === winter.id) {
        winter.base = fee;
        winter.vip = vipFee;
      } else if (seasonId === spring.id) {
        spring.base = fee;
        spring.vip = vipFee;
      }
    });
    let summerVipCost = summer.vip.regular ? summer.vip.regular : 0;
    let autumnVipCost = autumn.vip.regular ? autumn.vip.regular : 0;
    let winterVipCost = winter.vip.regular ? winter.vip.regular : 0;
    let sspringVipCost = spring.vip.regular ? spring.vip.regular : 0;
    const vipRealCostAmount = summerVipCost + autumnVipCost + winterVipCost + sspringVipCost;
    return {
      isBindVip: result,
      quarterFeeDetails: [summer, autumn, winter, spring],
      vipRealCostAmount: vipRealCostAmount,
    };
  };

  // 加载目标结果详情
  loadResultDetail = async (promiseStudentId) => {
    if (this.props.type !== 5) {
      const apiData = await getPromiseResultDetail({ promiseStudentId }, true);
      const {
        id,
        promiseSchoolScore,
        studentTotalPoint,
        resultStatus,
        modifyReason,
        returnType,
        refundNote,
        promiseClassTargetEndFileVo,
        refundTotalAmountActual,
        auditRemark,
        campusList,
        userList,
      } = apiData;

      let keyArr = Object.keys(promiseClassTargetEndFileVo);
      let keyLen = keyArr.length;
      let list = [];
      let file_ = {};
      let type;
      let temp;
      let campus = null;
      let attribution = null;

      for (let i = 0; i < keyLen; i++) {
        type = keyArr[i];
        temp = promiseClassTargetEndFileVo[type];
        temp.src = temp.filePath;
        file_['file_' + type] = temp;
        list.push(temp);
      }

      if (Array.isArray(campusList)) {
        campus = campusList.map((item) => {
          delete item.id;
          return item;
        });
      }

      if (Array.isArray(userList)) {
        attribution = userList.map((item) => {
          delete item.id;
          return item;
        });
      }

      return {
        id: id,
        schoolFraction: promiseSchoolScore.toString(),
        studentFraction: studentTotalPoint.toString(),
        autoChoose: studentTotalPoint > promiseSchoolScore ? 1 : 2,
        fraction: true,
        chooseResult: resultStatus === 1 ? 1 : 2,
        changeReason: modifyReason && modifyReason !== '' ? modifyReason : null,
        refundMode: returnType,
        refundReason: refundNote,
        fileList: list, //...
        refundTotalAmountActual: refundTotalAmountActual ? refundTotalAmountActual : '0',
        opinion: auditRemark,
        ...file_,
        campusList: campus,
        userList: attribution,
      };
    } else {
      return { refundTotalAmountActual: '0' };
    }
  };

  // 加载审批动态
  loadApprovalDynamic = async (promiseStudentId) => {
    try {
      const data = await getDynamicVoByPromiseStudentId({ promiseStudentId }, true);
      this.setState({
        approvalDynamic: data || [],
      });
    } catch (error) {
      showMessage(error.text);
    }
  };

  // 计算金额
  compute = (actualMoney, quarterFeeDetails) => {
    let baseRegularTotal = 0;
    let baseDiscountTotal = 0;
    let vipRegularTotal = 0;
    let vipDiscountTotal = 0;
    let baseRegularHoursTotal = 0;
    let baseDiscountHoursTotal = 0;
    quarterFeeDetails.forEach((quarterFeeDetail) => {
      const { base, vip } = quarterFeeDetail;
      baseRegularTotal += base.regular;
      baseDiscountTotal += base.discount;
      baseRegularHoursTotal += base.regularHours;
      baseDiscountHoursTotal += base.discountHours;
      vipRegularTotal += vip.regular;
      vipDiscountTotal += vip.discount;
    });
    const baseTotal = baseRegularTotal + baseDiscountTotal;
    const baseHoursTotal = baseRegularHoursTotal + baseDiscountHoursTotal;
    const vipTotal = vipRegularTotal + vipDiscountTotal;
    const total = baseTotal + vipTotal;
    let refundMoney = actualMoney - baseRegularTotal - vipRegularTotal;
    refundMoney = refundMoney < 0 ? 0 : refundMoney;
    return {
      baseRegularTotal,
      baseDiscountTotal,
      baseRegularHoursTotal,
      baseDiscountHoursTotal,
      vipRegularTotal,
      vipDiscountTotal,
      baseTotal,
      baseHoursTotal,
      vipTotal,
      total,
      refundMoney,
    };
  };

  // 计算一个季度扣费总额
  computeQuarterTotal = (quarterFeeDetail) => {
    const { base, vip } = quarterFeeDetail;
    if (!base || !vip) {
      return null;
    }
    return base.regular + base.discount + vip.regular + vip.discount;
  };

  formatNum = (numStr, decimal = 1, positive = true) => {
    numStr = typeof numStr === 'string' ? numStr : numStr.toString();
    if (positive) {
      numStr = Math.abs(Number(numStr)).toString();
    }
    return numStr.replace(/([0-9]+\.[0-9]{1})[0-9]*/, '$1');
  };

  checkDecimal = (numStr) => {
    numStr = typeof numStr === 'string' ? numStr : numStr.toString();
    let result = '';
    let fIndex = numStr.indexOf('.');
    if (fIndex !== -1) {
      let lIndex = numStr.lastIndexOf('.');
      result = fIndex === lIndex ? numStr : numStr.slice(0, lIndex);
    } else {
      result = numStr;
    }
    return result;
  };

  formatFraction = (val) => {
    let numVal = Number(val);
    if (val !== '' && numVal && numVal !== 0) {
      let temp = this.checkDecimal(val);
      return temp.indexOf('.') === temp.length - 1 ? temp : this.formatNum(temp);
    } else {
      return '0';
    }
  };

  /**
   * 更新实际退费金额
   *
   * @param val {string|number}
   */
  updateRefundTotalAmount = (val) => {
    val = Number(val);
    if (val > this.state.refundTotalAmount) return;

    this.setState({ refundTotalAmountActual: val });
    this.calcCampusAmount(val);
  };

  // 如果手动修改了实际退费金额
  // 则默认各校责任=各学习中心业绩比例 * 实际退费金额（如收款业绩A学习中心600，B学习中心400，现在实际退费金额800，则A学习中心责任业绩=600/（600+400）*800=480，校B区责任业绩=400/（600+400）*800=320），注：如果除不尽，则向下取两位小数，学习中心责任总额不能超过实际退费金额
  calcCampusAmount = (refundTotalAmountActual) => {
    const { refundTotalAmount, originCampusLiability } = this.state;
    let campusArr = [...originCampusLiability];

    if (refundTotalAmount === 0) return;
    campusArr = campusArr.map((item) => {
      return {
        ...item,
        amount: ((refundTotalAmountActual / refundTotalAmount) * item.amount).toFixed(2),
      };
    });

    this.setState({ campusLiability: campusArr });
  };

  // 监听输入目标学校分数线与学生总分
  inputFraction = (val, type) => {
    val = this.formatFraction(val);
    let num = Number(val);
    let otherNum = 0;
    let result = 0;
    if (type === 'school') {
      this.setState({ schoolFraction: val });
      otherNum = Number(this.state.studentFraction);
      result = num <= otherNum ? 1 : 2;
    } else {
      this.setState({ studentFraction: val });
      otherNum = Number(this.state.schoolFraction);
      result = otherNum <= num ? 1 : 2;
    }
    let check = num > 0 && otherNum > 0;
    this.setState({ fraction: check });
    if (check) {
      this.setState({ chooseResult: result, autoChoose: result });
    }
  };

  // 选择目标结果
  chooseResultTag = (e) => {
    const { fraction, chooseResultToggle, schoolFraction, studentFraction } = this.state;
    if (chooseResultToggle) {
      if (fraction) {
        this.setState({ chooseResult: Number(e.target.dataset.value) });
      } else {
        let errorMsg =
          schoolFraction === '0' || studentFraction === '0'
            ? '录入的分数不能为0!'
            : '请先录入目标学校分数线和学生考试总分!';
        AntdModal.info({
          title: errorMsg,
          content: '',
          className: 'homAntdModal',
        });
      }
    }
  };

  disableClick = () => {
    AntdModal.info({
      title: '请先录入目标学校分数线和学生考试总分',
      content: '',
      className: 'homAntdModal',
    });
  };

  // 切换退款方式
  switchRefundType = (val) => {
    this.setState({
      refundMode: val,
    });
  };

  // 监听上传回调
  handleUploadComplete = (file) => {
    this.setState((prevState, props) => {
      let list = prevState.fileList.slice();
      list.push(file);
      return { fileList: list, ['file_' + file.fileType]: file };
    });
  };

  // 监听删除回调
  handleDeleteComplete = (id) => {
    this.setState((prevState, props) => {
      let list = prevState.fileList.slice();
      // 遍历
      let index = list.findIndex((item) => item.fileId === id);
      let type = list[index].fileType;
      list.splice(index, 1);
      return { fileList: list, ['file_' + type]: null };
    });
  };

  /**
   * 提交申请
   * 1. 查询审批人员，有多个人则进行选人审批操作
   * 2. 否者直接提交
   */
  submitApply = async () => {
    const validate = this.checkParams();
    if (!validate) return;

    let params = this.getCommitApplyParams();
    await this.getAuditUserList(params);

    if (this.state.approvalUsers.length > 1) {
      this.setState({
        approvalSelectVisible: true,
      });
      return;
    } else {
      const user = this.state.approvalUsers[0];
      params.auditUserId = user.val;
      params.nodeId = this.state.nodeId;
      this.submit(params);
    }
  };

  /**
   * 预申请：查询审核人
   */
  getAuditUserList = async (params) => {
    try {
      const res = await getOaPromiseClassTargetEndAuditUserList(params, true);
      const { approvalUsers = [], nodeId } = res.data;
      const users = approvalUsers.map((item) => {
        return {
          val: item.userId,
          text: item.userName,
        };
      });

      this.setState({
        approvalUsers: users,
        nodeId,
      });
    } catch (error) {
      this.setState({ loading: false });
      showMessage(error.text || error.message);
    }
  };

  checkParams = () => {
    if (!this.state.schoolFraction) {
      return this.showError('请输入目标学校分数线');
    }
    if (!this.state.studentFraction) {
      return this.showError('请输入学生考试总分');
    }
    if (this.state.chooseResult === 0) {
      return this.showError('请设置目标结果');
    }
    if (this.state.chooseResult === 2) {
      if (this.state.refundMode === 'RETURN_TO_CUSTOMER') {
        // if (!this.state.file_3) {
        //   return this.showError('请上传家长银行卡照片');
        // }
        // if (!this.state.file_4) {
        //   return this.showError('请上传家长身份证照片');
        // }
        // const checkAccountResult = this.checkAccount();
        // if (checkAccountResult) {
        //   return this.showError(checkAccountResult);
        // }
      } else if (!this.state.refundMode) {
        return this.showError('请选择退款方式');
      }
      // if (!this.state.file_1) {
      //   return this.showError('请上传退费申请表');
      // }
      if (!this.state.file_0) {
        return this.showError('请上传合同承诺学校凭证');
      }
      if (!this.state.file_2) {
        return this.showError('请上传录取通知书或成绩凭证');
      }
    }
    if (this.state.chooseResult !== this.state.autoChoose && !this.state.changeReason) {
      return this.showError('请输入申请人备注');
    }

    return true;
  };

  checkAccount = () => {
    const { returnBankAccount, returnBankAccountOwner, branchCode } = this.state;
    if (!returnBankAccount) {
      return '请输入退费账号';
    }
    if (!returnBankAccountOwner) {
      return '请输入退费账号名';
    }
    if (!branchCode) {
      return '请输入搜索选择银行卡支行信息';
    }

    return false;
  };

  /**
   * 获取提交参数
   * workflowType （流程类型 0：boss流程 1:OA类型）
   */
  getCommitApplyParams = () => {
    const isFailResult = this.state.chooseResult === 2;
    let apiParams = {
      id: null,
      workflowType: 1,
      courseStatus: '4',
      promiseStudentId: this.state.studentInfo.id,
      auditStatus: 'UNAUDIT',
      promiseClassTargetEndFile: [this.state.file_0, this.state.file_1, this.state.file_2].filter((el) => el),
      promiseSchoolScore: Number(this.state.schoolFraction),
      studentTotalPoint: Number(this.state.studentFraction),
      modifyReason: this.state.autoChoose === this.state.chooseResult ? null : this.state.changeReason,
      resultStatus: !isFailResult ? 1 : 0,
      returnType: isFailResult ? this.state.refundMode : null,
      refundNote: isFailResult && this.state.refundMode === 'RETURN_TO_ACCOUNT' ? this.state.refundReason : null,
      refundTotalAmountActual: isFailResult ? Number(this.state.refundTotalAmountActual) : 0,
      isFullClassRefund: this.state.courseRealPayAmount === Number(this.state.refundTotalAmountActual) ? 1 : 0,
      // 责任归属
      incomeDistributeVo: [],
    };

    if (isFailResult && this.state.refundMode === 'RETURN_TO_CUSTOMER') {
      [this.state.file_3, this.state.file_4].forEach((item) => {
        if (item) {
          apiParams.promiseClassTargetEndFile.push(item);
        }
      });
      apiParams.returnBankAccount = this.state.returnBankAccount;
      apiParams.returnBankAccountOwner = this.state.returnBankAccountOwner;
      apiParams.returnBankName = this.state.returnBankName;
      apiParams.branchCode = this.state.branchCode;
      apiParams.incomeDistributeVo = (this.state.campusLiability || []).concat(this.state.attributionLiability || []);
    }

    return apiParams;
  };

  showError = (errorMsg) => {
    AntdModal.info({
      title: errorMsg,
      content: '',
      className: 'homAntdModal',
    });
  };

  submit = async (aipParams) => {
    this.setState({ loading: true });
    const rsp = await submitPromiseResult(aipParams, true).catch(errorsHandle());
    this.setState({ loading: false });
    if (rsp && rsp.resultCode === 0) {
      showMessage('操作成功', 'success');
      runFunc(this.props.onSave);
    }
  };

  updateState = (newState) => {
    this.setState((prevState, props) => {
      return {
        ...newState,
      };
    });
  };

  // 更新学习中心责任数据
  updateCampus = (params) => {
    const { campusLiability } = this.state;
    let arr = campusLiability.slice();
    arr[params.index].amount = params.val;
    this.updateState({ campusLiability: arr });
  };

  // 更新归属责任数据
  updateAttribution = (params) => {
    const { attributionLiability } = this.state;
    let arr = attributionLiability.slice();
    arr[params.index].amount = params.val;
    this.updateState({ attributionLiability: arr });
  };

  handleCashierUpload = (file) => {
    this.setState({ cashierFile: file });
  };

  handleCashierDelete = (id) => {
    this.setState({ cashierFile: null });
  };

  // 只能输入数字 去空格之后判断是否为纯数字即可赋值
  // formatter={value => String(value).replace(/\s/g, '').replace(/(\d{4})(?=\d)/g, "$1 ") }
  handleAcountNumberChange = (val) => {
    const value = val.replace(/\s/g, '');
    const maxLength = 26;

    const reg = /^[0-9]*$/;
    if ((reg.test(value) || value === '') && value.length <= maxLength) {
      let payload = {};
      if (!value) {
        payload = { returnBankName: '', branchCode: '', branchBankList: [] };
      }
      this.setState({
        returnBankAccount: value,
        ...payload,
      });
    }
  };

  // parser={value => String(value).replace(/\s/g, '')}
  formatAccountNumber = (val) => {
    let value = val || '';
    value = value.replace(/\s/g, '').replace(/(\d{4})(?=\d)/g, '$1 ');
    return value;
  };

  copyAccount = () => {
    const accountNumberInput = document.querySelector(`#${accountNumberID}`);
    const disabled = accountNumberInput.disabled;
    disabled && (accountNumberInput.disabled = !disabled);

    accountNumberInput && accountNumberInput.select();
    if (document.execCommand('copy')) {
      document.execCommand('copy');
      showMessage('复制成功', 'success');
    } else {
      showMessage('当前浏览器不支持复制功能', 'error');
    }

    accountNumberInput.disabled = disabled;
  };

  approvalSelectClose = () => {
    this.setState({ approvalSelectVisible: false });
  };

  approvalSelectConfirm = (val) => {
    this.setState({ selectedApproval: val });
    this.approvalSelectClose();

    let params = this.getCommitApplyParams();
    // 获取审批人id
    params.auditUserId = val;
    params.nodeId = this.state.nodeId;
    this.submit(params);
  };

  async getBankInfo() {
    const { returnBankAccount } = this.state;
    if (returnBankAccount) {
      let bankCardNo = returnBankAccount.trim().replace(/\s/g, '');
      try {
        const { resultCode, data } = await getBankCardInfo({ bankCardNo, isFromAts: true /** 获取ATS数据源 */ });
        if (resultCode === 0) {
          const { openBranchCode, accountOpeningBranch } = data;
          let payload = {};
          if (openBranchCode && accountOpeningBranch) {
            payload = {
              returnBankName: accountOpeningBranch,
              branchCode: openBranchCode,
              branchBankList: [{ label: accountOpeningBranch, value: openBranchCode }],
            };
          }
          this.setState(payload);
        }
      } catch (e) {
        showMessage(e.resultMessage || '查询支行信息失败');
      }
    }
  }

  async searchBankInfo(query) {
    if (query.length >= 2) {
      try {
        const branchBankList = await searchBankBranchLoader(query);
        this.setState({ branchBankList });
      } catch (e) {
        showMessage(e, 'error');
      }
    }
  }

  onBranchBankSelect(label) {
    this.matchBank(label);
  }

  matchBank(text) {
    const { branchBankList } = this.state;
    let item;
    for (let i = 0, ii = branchBankList.length; i < ii; i++) {
      if (branchBankList[i].label === text) {
        item = branchBankList[i];
        break;
      }
    }
    this.setState({ branchCode: item ? item.value : '' });
  }

  checkBankSelect(e) {
    e.persist();
    let text = e.target.value.trim(/\s+/g, '');
    if (text) {
      this.matchBank(text);
    }
  }

  render() {
    const { onClose, type } = this.props;
    const {
      loading,
      studentInfo,
      contractInfo,
      quarterFeeDetails,
      refundMode,
      commitApplyBtnError,
      totals,
      isBindVip,
      error,
      schoolFraction,
      studentFraction,
      fraction,
      chooseResult,
      changeReason,
      autoChoose,
      vipRealCostAmount,
      vipRealIncome,
      courseRealPayAmount,
      refundTotalAmount,
      refundTotalAmountActual,
      refundReason,
      file_0,
      file_1,
      file_2,
      file_3,
      file_4,
      campusLiability,
      attributionLiability,
      branchBankList,
      returnBankAccount,
      returnBankAccountOwner,
      returnBankName,
      approvalDynamic,
      approvalSelectVisible,
      approvalUsers,
    } = this.state;
    const dataSource = branchBankList.map((item) => (
      <Option value={item.label} key={item.value}>
        {item.label}
      </Option>
    ));
    const { actualMoney } = contractInfo;
    const { baseTotal, baseHoursTotal, vipTotal, total } = totals;
    const summerFee = quarterFeeDetails.find((q) => q.id === 'DAT0000000232') || {};
    const autumnFee = quarterFeeDetails.find((q) => q.id === 'DAT0000000233') || {};
    const winterFee = quarterFeeDetails.find((q) => q.id === 'DAT0000000230') || {};
    const springFee = quarterFeeDetails.find((q) => q.id === 'DAT0000000231') || {};
    let btnInfo;
    if (type === 6) {
      btnInfo = '已提交申请';
    } else if (type === 8) {
      btnInfo = '已审批';
    } else if (type === 5 && !isBindVip) {
      btnInfo =
        '该精英班合同关联的vip服务费产品超过1种或未绑定VIP服务费，暂不能通过本页面申请退费，请联系分公司精英班负责人，让其对接集团精英班项目部进行退费';
    }

    // 目标成功或目标失败且退到电子账号，跳过了出纳审批，直接审批通过
    const isFailResult = chooseResult === 2;
    const isRefundAccount = refundMode === 'RETURN_TO_ACCOUNT'; // 退费到电子账户
    // 是否跳过出纳环节
    const isOverCashier = !isFailResult || (isFailResult && isRefundAccount);
    // 是否申请页面
    const isApply = type === 5;

    return (
      <div>
        <Modal
          loading={loading}
          onCancel={onClose}
          width={1100}
          error={error}
          className={'promiseResultDetailOA'}
          title={
            <span>
              <span>精英班扣费详情</span>
              <span style={{ color: 'red' }}>（合同完结）</span>
            </span>
          }
          footer={
            <div className={styles_.footerWrapper}>
              <span className={styles_.footerTip}>
                <span style={{ color: 'red' }}>*</span>
                审批通过退费金额将退至客户电子账户或退款，同时对该学生所有已报未完成的小班进行退班；审核不通过则该学生可恢复上课
              </span>
              {!isApply && (
                <Button size="large" onClick={onClose}>
                  关闭
                </Button>
              )}
              {isApply && (
                <ButtonWithPop
                  size="large"
                  type="primary"
                  onClick={this.submitApply}
                  disabled={loading || type === 6 || (type === 5 && !isBindVip)}
                  info={btnInfo}
                  error={commitApplyBtnError}
                  onErrorClose={() => this.setState({ commitApplyBtnError: null })}
                >
                  提交申请
                </ButtonWithPop>
              )}
            </div>
          }
        >
          <Block type="dark" title="学生信息" style={{ marginTop: -10 }}>
            <StudentInfo data={studentInfo} />
          </Block>
          <Block type="dark" title="合同信息">
            <ContractInfo data={contractInfo} />
          </Block>
          <Block type="dark" title="扣费明细">
            {
              <table className={styles.tableView}>
                <tbody>
                  <tr>
                    <th>扣费项目/季度</th>
                    <th>暑假</th>
                    <th>秋季</th>
                    <th>寒假</th>
                    <th>春季</th>
                    <th className={styles.bold}>小计</th>
                  </tr>
                  <tr>
                    <td className={styles.bold}>已扣款金额</td>
                    <td>
                      <FeeCell data={summerFee.base} />
                    </td>
                    <td>
                      <FeeCell data={autumnFee.base} />
                    </td>
                    <td>
                      <FeeCell data={winterFee.base} />
                    </td>
                    <td>
                      <FeeCell data={springFee.base} />
                    </td>
                    <td>{`${formatMoney(baseTotal)}（${baseHoursTotal}小时）`}</td>
                  </tr>
                  <tr>
                    <td className={styles.bold}>VIP服务费</td>
                    <td>
                      <FeeCell data={summerFee.vip} />
                    </td>
                    <td>
                      <FeeCell data={autumnFee.vip} />
                    </td>
                    <td>
                      <FeeCell data={winterFee.vip} />
                    </td>
                    <td>
                      <FeeCell data={springFee.vip} />
                    </td>
                    <td>{formatMoney(vipTotal)}</td>
                  </tr>
                  <tr>
                    <td className={styles.bold}>合计</td>
                    <td>{formatMoney(this.computeQuarterTotal(summerFee))}</td>
                    <td>{formatMoney(this.computeQuarterTotal(autumnFee))}</td>
                    <td>{formatMoney(this.computeQuarterTotal(winterFee))}</td>
                    <td>{formatMoney(this.computeQuarterTotal(springFee))}</td>
                    <td className={styles.bold}>{formatMoney(total)}</td>
                  </tr>
                </tbody>
              </table>
            }
          </Block>

          <Block type="dark" title="录入分数">
            <div>
              <Row>
                <Col span={12}>
                  <FormItem name="目标学校分数线" labelWidth={124} style={{ marginBottom: 0 }} required>
                    <InputNum
                      placeholder="请输入目标学校分数线"
                      min={0}
                      max={999}
                      value={schoolFraction}
                      onChange={(val, type = 'school') => this.inputFraction(val, type)}
                      disabled={type !== 5}
                    />
                  </FormItem>
                </Col>
                <Col span={12}>
                  <FormItem name="学生考试总分" labelWidth={124} style={{ marginBottom: 0 }} required>
                    <InputNum
                      placeholder="请输入学生考试总分"
                      min={0}
                      max={999}
                      value={studentFraction}
                      onChange={(val, type = 'student') => this.inputFraction(val, type)}
                      disabled={type !== 5}
                    />
                  </FormItem>
                </Col>
              </Row>
            </div>
          </Block>
          <Block type="dark" title="设置目标结果">
            {
              <div>
                <Row gutter={10}>
                  <Col span={12}>
                    <HomSection
                      title="成功"
                      cover={fraction && chooseResult === 1}
                      coverVal={1}
                      handleClick={(e) => this.chooseResultTag(e)}
                      className={styles_.colEvenTextRight + ' ' + styles_.rowDefaultH}
                    >
                      <div className={styles_.homRow}>
                        <div className={styles_.homCol}>合同总额：</div>
                        <div className={styles_.homCol}>{formatMoney(contractInfo.amount)}</div>
                      </div>
                      <div className={styles_.homRow}>
                        <div className={styles_.homCol}>合计扣取小时费总额：</div>
                        <div className={styles_.homCol}>{formatMoney(baseTotal)}</div>
                      </div>
                      <div className={styles_.homRow}>
                        <div className={styles_.homCol}>合计扣取VIP服务费总额：</div>
                        <div className={styles_.homCol}>{formatMoney(vipTotal)}</div>
                      </div>
                      <div className={styles_.homRow}>
                        <div className={styles_.homCol}>合计划归总额：</div>
                        <div className={styles_.homCol}>{formatMoney(contractInfo.amount - baseTotal - vipTotal)}</div>
                      </div>
                    </HomSection>
                  </Col>
                  <Col span={12}>
                    <HomSection
                      title="失败"
                      cover={fraction && chooseResult === 2}
                      coverVal={2}
                      handleClick={(e) => this.chooseResultTag(e)}
                    >
                      <div className={styles_.homRow}>
                        <div className={styles_.homCol}>合同实收金额：</div>
                        <div className={styles_.homCol}>{formatMoney(actualMoney)}</div>
                      </div>
                      <div className={styles_.homRow}>
                        <div className={styles_.homCol}>合计扣取正价VIP服务费总额：</div>
                        <div className={styles_.homCol}>{formatMoney(vipRealCostAmount)}</div>
                      </div>
                      <div className={styles_.homRow}>
                        <div className={styles_.homCol}>合计划归总额：</div>
                        <div className={styles_.homCol}>
                          {formatMoney(vipRealIncome + (courseRealPayAmount - Number(refundTotalAmountActual)))}
                        </div>
                      </div>
                      <div className={styles_.homRow}>
                        <div className={styles_.homCol + ' ' + styles_.homBoldBlue}>合计退费总额：</div>
                        <div className={styles_.homCol + ' ' + styles_.homBoldBlue}>
                          {formatMoney(refundTotalAmount)}
                        </div>
                      </div>
                      {(type === 5 || type === 7) && (
                        <div className={styles_.homRow}>
                          <div className={styles_.homCol + ' ' + styles_.homBoldBlue}>实际退费总额：</div>
                          <div className={styles_.homCol}>
                            <InputNum
                              precision={2}
                              placeholder="请输入实际退费金额"
                              value={refundTotalAmountActual}
                              max={refundTotalAmount}
                              onChange={(val) => this.updateRefundTotalAmount(val)}
                            />
                          </div>
                        </div>
                      )}
                      <div className={styles_.homBorderTop}>
                        <RadioGroup onChange={(e) => this.switchRefundType(e.target.value)} value={refundMode}>
                          <Radio
                            value="RETURN_TO_ACCOUNT"
                            disabled={!(type === 5 || type === 7)}
                            tip="部分退费、或想把钱退到电子账户先放着的时候采用"
                          >
                            退到电子账户
                          </Radio>
                          <Radio value="RETURN_TO_CUSTOMER" disabled={!(type === 5 || type === 7)} tip="全额退费时采用">
                            直接退款
                          </Radio>
                        </RadioGroup>
                        <div style={{ marginBottom: 12 }}>
                          {refundMode === 'RETURN_TO_ACCOUNT' && (
                            <TextArea
                              rows={2}
                              placeholder="如特殊退款，例如与家长协商退半款的，请在此处备注退款总额及原因"
                              value={refundReason}
                              onChange={(e) => this.setState({ refundReason: e.target.value })}
                              disabled={type !== 5}
                            />
                          )}
                          {refundMode === 'RETURN_TO_CUSTOMER' && (
                            <div className="clearfix" style={{ display: 'none' }}>
                              <div className={styles_['hom-upload-block']}>
                                <UploadImg
                                  value={file_3}
                                  text="选填"
                                  uploadComplete={(val) => this.handleUploadComplete(val)}
                                  deleteComplete={(val) => this.handleDeleteComplete(val)}
                                  params={{ fileType: 3 }}
                                  size={80}
                                  viewOnly={type === 6 || type === 8}
                                />
                                <div className={styles_['hom-upload-block-text']}>家长银行卡</div>
                              </div>
                              <div className={styles_['hom-upload-block']}>
                                <UploadImg
                                  value={file_4}
                                  text="选填"
                                  uploadComplete={(val) => this.handleUploadComplete(val)}
                                  deleteComplete={(val) => this.handleDeleteComplete(val)}
                                  params={{ fileType: 4 }}
                                  size={80}
                                  viewOnly={type === 6 || type === 8}
                                />
                                <div className={styles_['hom-upload-block-text']}>家长身份证</div>
                              </div>
                            </div>
                          )}
                        </div>
                        {refundMode === 'RETURN_TO_CUSTOMER' && (isApply || (!isApply && returnBankAccount)) && (
                          <Form className={styles.promiseFeeForm}>
                            <Row align="middle">
                              <Col span={24}>
                                <p className={styles.promiseFeeFormTip}>退回银行卡信息由家长操作确认时填写！</p>
                              </Col>
                            </Row>
                            <Row align="middle">
                              <Col span={20}>
                                <FormItem name="退费账号名" labelWidth={100}>
                                  <Input
                                    placeholder="退费账号名"
                                    value={returnBankAccountOwner}
                                    onChange={(e) => this.setState({ returnBankAccountOwner: e.target.value })}
                                    onBlur={() =>
                                      this.setState({
                                        returnBankAccountOwner: returnBankAccountOwner && returnBankAccountOwner.trim(),
                                      })
                                    }
                                    disabled={/*!isApply*/ true}
                                  />
                                </FormItem>
                              </Col>
                            </Row>
                            <Row align="middle">
                              <Col span={20}>
                                <FormItem name="退费账号" labelWidth={100}>
                                  <Input
                                    id={accountNumberID}
                                    placeholder="退费账号"
                                    value={this.formatAccountNumber(returnBankAccount)}
                                    onChange={(e) => this.handleAcountNumberChange(e.target.value)}
                                    onBlur={this.getBankInfo}
                                    disabled={/*!isApply*/ true}
                                  />
                                </FormItem>
                              </Col>
                              <Col span={4}>
                                <Button
                                  type="primary"
                                  size="small"
                                  disabled
                                  style={{ margin: '5px 0 5px 10px' }}
                                  onClick={this.copyAccount}
                                >
                                  复制
                                </Button>
                              </Col>
                            </Row>
                            <Row align="middle">
                              <Col span={20}>
                                <FormItem name="银行支行信息" labelWidth={100}>
                                  <AutoComplete
                                    value={returnBankName}
                                    dataSource={dataSource}
                                    onSelect={this.onBranchBankSelect}
                                    onSearch={this.searchBankInfo}
                                    onChange={(val) => this.setState({ returnBankName: val })}
                                    disabled={/*!isApply*/ true}
                                  >
                                    <Input
                                      size="large"
                                      onBlur={this.checkBankSelect}
                                      placeholder="请输入关键字搜索支行"
                                    />
                                  </AutoComplete>
                                </FormItem>
                              </Col>
                              <Col span={4} style={{ paddingLeft: 10 }}>
                                <BankTooltip placement="bottom" />
                              </Col>
                            </Row>
                          </Form>
                        )}
                      </div>
                    </HomSection>
                  </Col>
                </Row>
                {chooseResult !== autoChoose && (
                  <div className={styles_.homBorderTop}>
                    <Block title="申请人备注" className="homPageCourseModal">
                      <TextArea
                        rows={3}
                        placeholder="请输入目标结果修改原因"
                        value={changeReason}
                        onChange={(e) => this.setState({ changeReason: e.target.value })}
                        disabled={!(type === 5 || type === 7)}
                      />
                    </Block>
                  </div>
                )}
              </div>
            }
          </Block>
          <Block type="dark" title="上传凭证">
            <div className="hom-container">
              {!fraction && (
                <div className="hom-disable" onClick={this.disableClick}>
                  {' '}
                </div>
              )}
              <div className="clearfix">
                <div className={styles_['hom-upload-block']}>
                  <UploadImg
                    value={file_0}
                    text={chooseResult === 1 ? '选填' : '必填'}
                    uploadComplete={(val) => this.handleUploadComplete(val)}
                    deleteComplete={(val) => this.handleDeleteComplete(val)}
                    params={{ fileType: 0 }}
                    size={160}
                    viewOnly={type === 6 || type === 8}
                  />
                  <div className={styles_['hom-upload-block-text']}>合同承诺学校凭证</div>
                </div>
                <div className={styles_['hom-upload-block']}>
                  <UploadImg
                    value={file_2}
                    text={chooseResult === 1 ? '选填' : '必填'}
                    uploadComplete={(val) => this.handleUploadComplete(val)}
                    deleteComplete={(val) => this.handleDeleteComplete(val)}
                    params={{ fileType: 2 }}
                    size={160}
                    viewOnly={type === 6 || type === 8}
                  />
                  <div className={styles_['hom-upload-block-text']}>
                    {chooseResult === 1 ? '录取通知书' : '录取通知书或成绩凭证'}
                  </div>
                </div>
                <div className={styles_['hom-upload-block']}>
                  <UploadImg
                    value={file_1}
                    text={chooseResult === 1 ? '选填' : '选填'}
                    uploadComplete={(val) => this.handleUploadComplete(val)}
                    deleteComplete={(val) => this.handleDeleteComplete(val)}
                    params={{ fileType: 1 }}
                    size={160}
                    viewOnly={type === 6 || type === 8}
                  />
                  <div className={styles_['hom-upload-block-text']}>
                    {chooseResult === 1 ? '成绩凭证' : '退费申请表'}
                  </div>
                </div>
              </div>
            </div>
          </Block>
          {isApply && chooseResult === 2 && refundMode === 'RETURN_TO_CUSTOMER' && (
            <Block type="dark" title="退费责任归属">
              <div className="">
                <FormItem name="学习中心责任" labelWidth={124} style={{ padding: 0, marginBottom: 0 }} required>
                  <CampusList
                    data={campusLiability}
                    refundTotalAmountActual={refundTotalAmountActual}
                    handleUpdate={this.updateCampus}
                  />
                </FormItem>
                <div className={styles_.homBorderTop} style={{ paddingTop: '20px' }}>
                  <FormItem name="归属类型" labelWidth={124} style={{ padding: 0, marginBottom: 0 }}>
                    <AttributionList
                      data={attributionLiability}
                      refundTotalAmountActual={refundTotalAmountActual}
                      handleUpdate={this.updateAttribution}
                    />
                  </FormItem>
                </div>
              </div>
            </Block>
          )}
          {!isApply && chooseResult === 2 && refundMode === 'RETURN_TO_CUSTOMER' && (
            <Block type="dark" title="退费责任归属">
              <ShowLiability campus={campusLiability} attribution={attributionLiability} />
            </Block>
          )}
          {!isApply && approvalDynamic.length > 0 && (
            <Block title={<span>审批动态</span>}>
              <ApprovalInfo data={approvalDynamic} />
            </Block>
          )}
          {!isOverCashier && this.props.type > 8 && this.step >= 2 && (
            <Block type="dark" title={<span className={this.step === 2 && styles.required}>出纳上传凭证</span>}>
              <div className="hom-container">
                <div className="clearfix">
                  <div className={styles_['hom-upload-block']}>
                    <UploadImg
                      value={this.state.cashierFile}
                      text="必填"
                      uploadComplete={(val) => this.handleCashierUpload(val)}
                      deleteComplete={(val) => this.handleCashierDelete(val)}
                      size={160}
                      viewOnly={this.step > 2}
                    />
                    <div className={styles_['hom-upload-block-text']}>转账凭证</div>
                  </div>
                </div>
              </div>
            </Block>
          )}
        </Modal>
        {/* 选择审批人 */}
        <ApprovalSelect
          visible={approvalSelectVisible}
          dict={approvalUsers}
          onClose={this.approvalSelectClose}
          onConfirm={this.approvalSelectConfirm}
        />
      </div>
    );
  }
}

export default connect()(PromiseResultDetail);
