import React from 'react';
import styles from '../index.less';
import PropTypes from 'prop-types';
import { Table } from 'antd';
import Modal from 'COMPONENT/Modal';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { updateMiniClassCourseByClassIdsApi } from '../loaders';

class MultiPanel extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      loadingTip: null,
      selectKeys: [],
      rowSelection: {
        onChange: (selectedRowKeys, selectedRows) => {
          let tempKey = selectedRows.map((value) => {
            return value.miniClassId;
          });
          this.setState({
            selectKeys: tempKey,
          });
        },
        getCheckboxProps: (record) => {
          return {
            disabled: record.status === 'CONPELETE',
            defaultChecked: record.status !== 'CONPELETE',
          };
        },
      },
    };

    this.columns = [
      {
        title: '小班名称',
        dataIndex: 'name',
      },
      {
        title: '所属校区',
        dataIndex: 'blCampusName',
      },
      {
        title: '上课日期',
        dataIndex: 'startDate',
      },
      {
        title: '上课时间',
        dataIndex: 'classTime',
      },
      {
        title: '小班状态',
        dataIndex: 'statusName',
      },
      {
        title: '年级',
        dataIndex: 'gradeName',
      },
      {
        title: '科目',
        dataIndex: 'subjectName',
      },
      {
        title: '老师',
        dataIndex: 'teacherName',
      },
      {
        title: '班主任',
        dataIndex: 'studyManegerName',
      },
      {
        title: '教室',
        dataIndex: 'classroomName',
      },
    ];
  }

  static propTypes = {
    teacherName: PropTypes.string,
    classroomName: PropTypes.string,
    studyManagerName: PropTypes.string,
    selectedRow: PropTypes.array,
  };

  submit = async () => {
    if (this.state.selectKeys.length === 0) {
      return showMessage('请选择其中一个小班来提交');
    }
    let { teacherId, studyManagerId, classRoomId } = this.props;
    teacherId = teacherId === '' ? null : teacherId;
    studyManagerId = studyManagerId === '' ? null : studyManagerId;
    classRoomId = classRoomId === '' ? null : classRoomId;
    if (!teacherId && !studyManagerId && !classRoomId) {
      return showMessage('请选择老师或者班主任或者教室');
    }
    try {
      let params = {};
      if (teacherId) {
        params = { teacherId: teacherId };
      } else if (studyManagerId) {
        params = { studyManagerId: studyManagerId };
      } else if (classRoomId) {
        params = { classRoomId: classRoomId };
      }
      const data = await updateMiniClassCourseByClassIdsApi({
        ...params,
        miniClassIds: this.state.selectKeys,
      });
      if (data.data) {
        showMessage('同步成功！', 'success');
        this.props.onClose();
      } else {
        showMessage(data.resultMessage);
      }
    } catch (e) {
      showMessage(e.text);
    }
  };

  componentDidMount() {
    let tempKey = [];
    this.props.selectedRow.forEach((value) => {
      if (value.status !== 'CONPELETE') {
        tempKey.push(value.miniClassId);
      }
    });
    this.setState({
      selectKeys: tempKey,
    });
  }

  render() {
    let { loading, loadingTip, rowSelection } = this.state;
    let { onClose, teacherName, classRoomId, classroomName, studyManagerId, studyManagerName, selectedRow } =
      this.props;

    return (
      <Modal
        loading={loading}
        loadingTip={loadingTip}
        onCancel={onClose}
        okText="确认同步所选小班的全部未上课课程"
        cancelText="无需同步"
        title="修改成功"
        width={1200}
        onOk={() => this.submit()}
        className={styles.syncConfirm}
      >
        <div className={styles.syncConfirmPanel}>
          <div className={styles.panelTop}>
            {!studyManagerId && !classRoomId && (
              <p>
                <span>老师：</span>
                <span>{teacherName}</span>
              </p>
            )}
            {studyManagerId && (
              <p>
                <span>班主任：</span>
                <span>{studyManagerName}</span>
              </p>
            )}
            {classRoomId && (
              <p>
                <span>教室：</span>
                <span>{classroomName}</span>
              </p>
            )}
          </div>
          <div className={styles.panelTable}>
            <div className={styles.panelHeader}>
              <p>是否需要将本次修改同步至班内所有未上课课程？</p>
            </div>
            <Table rowKey="miniClassId" rowSelection={rowSelection} dataSource={selectedRow} columns={this.columns} />
          </div>
          <div className={styles.panelFooter}>
            <p>1. 点击确认同步后，所选班内所有未上课课程将同步本次修改；</p>

            <p>2. 支持同步至小班状态为未上课、上课中的小班的未上课课程，支持单独取消勾选本次不同步的小班。</p>
            {/* <p>3. 课程上课方式为“在线A”或“在线B”的课程，开课后将不会同步老师信息的修改。</p> */}
          </div>
        </div>
      </Modal>
    );
  }
}

export default MultiPanel;
