import React from 'react';
import Modal from 'COMPONENT/Modal';
import Select from 'COMPONENT/Select';
import { connect } from 'dva';
import { loadCommonDict } from 'UTIL/dict';
import styles from './index.less';
import Input from 'COMPONENT/Input';
import Loaders from './loaders';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

class PrePareClass extends React.Component {
  constructor(props) {
    super(props);
    this.isInOldPage = !!props.oldPageParams;

    this.state = {
      // 从旧页面过来的数据
      ...(this.isInOldPage
        ? {
            studentId: props.oldPageParams.studentId,
            miniClassId: props.oldPageParams.miniClassId,
          }
        : {
            studentId: props.studentId,
            miniClassId: props.miniClassId,
          }),

      prepareDropReasonDict: [],
      prepareDropReason: null,
      remark: null,

      checking: false,
      errors: {},
    };
  }

  componentDidMount() {
    loadCommonDict('PREPARE_DROP_REASON').then((response) => {
      console.log('response');
      console.log(response);
      this.setState({ prepareDropReasonDict: response });
    });
  }

  /**
   * 小班预退班
   */
  prepareDropMiniClass = async (params) => {
    console.log(params);
    const { resultCode } = await Loaders.prepareDropMiniClassApi(params);
    if (resultCode === 0) {
      showMessage('预退班成功', 'success');
      this.props.onClose();
      if (this.isInOldPage) {
        this.props.dispatch({
          type: 'main/refreshCurrentTab',
        });
      } else {
        this.props.onOK();
      }
    } else {
      showMessage('预退班失败');
    }
  };

  // 采集校验信息进 errors 对象
  errorChange = (key, error) => {
    this.setState((preState) => {
      const { errors } = preState;
      return {
        errors: {
          ...errors,
          [key]: error,
        },
      };
    });
  };

  render() {
    const { prepareDropReasonDict, prepareDropReason, remark, miniClassId, studentId, errors, checking } = this.state;
    const params = { prepareDropReasonDict, prepareDropReason, remark, miniClassId, studentId };
    return (
      <Modal
        title="预退班"
        footerTipText="学生状态将变为预退班"
        onCancel={this.props.onClose}
        width="520"
        onOk={() => {
          const { checking, errors } = this.state;
          if (!checking) {
            this.setState({ checking: true });
          }
          if (Object.values(errors).find((v) => v)) {
            return;
          }
          this.prepareDropMiniClass(params);
        }}
      >
        <Select
          dict={prepareDropReasonDict}
          style={{ width: '330px' }}
          value={prepareDropReason}
          onChange={(value) => {
            this.setState((preState) => {
              const { errors } = preState;
              return {
                prepareDropReason: value,
                errors: {
                  ...errors,
                  remark: null,
                },
              };
            });
          }}
          validator={{
            rules: ['required'],
            silent: !errors['reason'] || !checking,
            showTips: false,
            showStatus: true,
            onErrorChange: (e) => this.errorChange('reason', e),
          }}
        />
        {prepareDropReason === 'DAT000000000699' && (
          <Input
            placeholder="请输入预退原因"
            className={styles.input}
            value={remark}
            onChange={(e) => {
              this.setState({ remark: e.currentTarget.value });
            }}
            validator={{
              rules: ['required'],
              silent: !errors['remark'] || !checking,
              showTips: false,
              showStatus: true,
              onErrorChange: (e) => this.errorChange('remark', e),
            }}
            maxLength={50}
          />
        )}
        <p className={styles.tip}>
          预退班学生将退出该班课的所有未考勤的讲次，并非立即退班。如在15天内未取消预退，将自动退班。
        </p>
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  return {};
};
export default connect()(PrePareClass);
