import React from 'react';
import List from 'COMPONENT/List';
import SearchBar from 'COMPONENT/SearchBar';
import Select from 'COMPONENT/Select';
import Input from 'COMPONENT/Input';
import Wrapper from 'COMPONENT/Wrapper';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import moment from 'moment';
import { dealCampus } from 'UTIL/dict';
import { remindByAttendanceListApi } from './loaders';
import { connect } from 'dva';

const courseTimeItemType = [
  'DONATE_REMIND_ITEM_OTM_STY_OADU',
  'DONATE_REMIND_ITEM_OTM_STY_NADU',
  'DONATE_REMIND_ITEM_OTM_STY_TWAIT',
  'DONATE_REMIND_ITEM_OTM_TEA_NATS',
];

const typeDict = [
  { val: 'DONATE_REMIND_ITEM_OTM_STY_OADU', text: '学管逾期扣费' },
  { val: 'DONATE_REMIND_ITEM_OTM_STY_NADU', text: '学管未扣费' },
  { val: 'DONATE_REMIND_ITEM_OTM_STY_TWAIT', text: '学管当天待扣费' },
  { val: 'DONATE_REMIND_ITEM_OTM_TEA_NATS', text: '老师已上课未考勤学生' },
];

class RemindByAttendanceList extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      branchId: '',
      campusId: '',
      teacherName: '',
      studyManagerName: '',
      type:
        this.props.params === undefined || courseTimeItemType.includes(this.props.params.itemType) === false
          ? ''
          : this.props.params.itemType,
      campusDict: [],

      remindByAttendanceListData: [], // 列表数据
      pagination: {
        current: 1,
        pageSize: 20,
        total: 0,
        pageSizeOptions: ['20', '50', '100'],
      }, // 列表分页设置
    };

    this.onBranchIdChange = this.onBranchIdChange.bind(this);
    this.onCampusIdChange = this.onCampusIdChange.bind(this);
    this.onStudyManagerNameChange = this.onStudyManagerNameChange.bind(this);
    this.onSearch = this.onSearch.bind(this);
    this.onTeacherNameChange = this.onTeacherNameChange.bind(this);
    this.onReset = this.onReset.bind(this);
    this.clickOnSearch = this.clickOnSearch.bind(this);

    this.columns = [
      { title: '学生姓名', dataIndex: 'studentName', key: 'studentName' },
      { title: '家长姓名', dataIndex: 'customerName', key: 'customerName' },
      { title: '联系方式', dataIndex: 'customerContact', key: 'customerContact' },
      { title: '所属分公司', dataIndex: 'branchName', key: 'branchName' },
      { title: '所属学习中心', dataIndex: 'campusName', key: 'campusName' },
      { title: '课程名称', dataIndex: 'courseName', key: 'courseName' },
      { title: '课程日期', dataIndex: 'courseDate', key: 'courseDate' },
      { title: '老师', dataIndex: 'teacherName', key: 'teacherName' },
      {
        title: '考勤日期',
        dataIndex: 'firstAttendentTime',
        key: 'firstAttendentTime',
        render: (val) => {
          return val === null ? '未考勤' : moment(val).format('YYYY-MM-DD');
        },
      },
      { title: '学管师', dataIndex: 'studyManagerName', key: 'studyManagerName' },
      {
        title: '学管扣费日期',
        dataIndex: 'studyManagerChargeTime',
        key: 'studyManagerChargeTime',
        render: (val, row) => {
          if (row.studyManagerNotAudit || row.studyManagerTodayWaitingAudit) {
            return '';
          } else {
            return moment(val).format('YYYY-MM-DD');
          }
        },
      },
      {
        title: '乐捐提醒类型',
        dataIndex: 'studyManagerOverdueAudit',
        key: 'studyManagerOverdueAudit',
        render: (val, row) => {
          return (
            <div>
              {row.studyManagerOverdueAudit && <p>学管逾期扣费</p>}
              {row.studyManagerNotAudit && <p>学管未扣费</p>}
              {row.studyManagerTodayWaitingAudit && <p>学管当天待扣费</p>}
              {row.teacherNotAtt && <p>老师已上课未考勤（学生）</p>}
            </div>
          );
        },
      },
    ];
  }

  componentWillReceiveProps(nextProps, nextState) {
    if (this.props.campusDict.length === 0 && nextProps.campusDict.length !== 0 && this.props.params === undefined) {
      this.setState({
        campusDict: nextProps.campusDict,
        branchId: nextProps.branchDict[0].val,
        campusId: nextProps.campusDict[0].val,
      });
    }
  }

  componentDidUpdate(prevProps) {
    if (
      this.props.campusDict.length !== 0 &&
      this.props.branchDict.length !== 0 &&
      prevProps.campusDict.length === 0 &&
      prevProps.branchDict.length === 0
    ) {
      this.onSearch();
    }
  }

  onBranchIdChange(branchId) {
    const { allCampusDict } = this.props;
    const campusDict = dealCampus(allCampusDict, branchId);
    this.setState({ branchId, campusDict, campusId: '' });
  }

  onCampusIdChange(campusId) {
    this.setState({ campusId });
  }

  onTeacherNameChange(e) {
    this.setState({ teacherName: e.currentTarget.value });
  }

  onStudyManagerNameChange(e) {
    this.setState({ studyManagerName: e.currentTarget.value });
  }

  onHomeroomTeacherNameChange(e) {
    this.setState({ homeroomTeacherName: e.currentTarget.value });
  }

  async onSearch() {
    const { changeLoadingStatus } = this.props;
    changeLoadingStatus(true);
    const {
      branchId,
      campusId,
      teacherName,
      studyManagerName,
      type,
      pagination: { pageSize: rows, current: page },
    } = this.state;
    const params = { branchId, campusId, teacherName, studyManagerName, rows, page, type };
    try {
      let { total, remindByAttendanceListData } = await remindByAttendanceListApi(params);
      this.setState({ remindByAttendanceListData, pagination: Object.assign({}, this.state.pagination, { total }) });
      changeLoadingStatus(false);
    } catch (err) {
      showMessage(err.text);
      changeLoadingStatus(false);
    }
  }

  clickOnSearch() {
    this.setState({ pagination: Object.assign({}, this.state.pagination, { current: 1 }) }, () => {
      this.onSearch();
    });
  }

  onReset() {
    this.setState({
      branchId: '',
      campusId: '',
      teacherName: '',
      studyManagerName: '',
      type: '',
      pagination: {
        current: 1,
        pageSize: 20,
        total: 0,
        pageSizeOptions: ['20', '50', '100'],
      },
    });
  }

  render() {
    const {
      branchId,
      campusId,
      teacherName,
      studyManagerName,
      remindByAttendanceListData,
      pagination,
      campusDict,
      type,
    } = this.state;
    const { branchDict } = this.props;

    return (
      <div>
        <Wrapper className="remindByAttendance">
          <SearchBar onSearch={this.clickOnSearch} onReset={this.onReset} expand={true}>
            <Select
              dict={branchDict}
              placeholder="所属分公司"
              value={branchId}
              onChange={this.onBranchIdChange}
              showSearch={true}
            />
            <Select
              dict={campusDict}
              placeholder="所属学习中心"
              value={campusId}
              onChange={this.onCampusIdChange}
              showSearch={true}
            />
            <Input value={teacherName} placeholder="老师" onChange={this.onTeacherNameChange} />
            <Input value={studyManagerName} placeholder="学管师" onChange={this.onStudyManagerNameChange} />
            <Select
              placeholder="乐捐提醒类型"
              dict={typeDict}
              value={type}
              onChange={(type) => this.setState({ type })}
            />
          </SearchBar>
        </Wrapper>
        <List
          dataSource={remindByAttendanceListData}
          columns={this.columns}
          pagination={pagination}
          onChange={(pagination) => {
            this.setState(
              {
                pagination: Object.assign({}, this.state.pagination, {
                  current: pagination.current,
                  pageSize: pagination.pageSize,
                }),
              },
              () => {
                this.onSearch();
              },
            );
          }}
        />
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  const pathname = state.routing.locationBeforeTransitions.pathname;
  return {
    params: state.main.pageParams[pathname],
  };
};

export default connect(mapStateToProps, null)(RemindByAttendanceList);
