import React from 'react';
import SearchBar from 'COMPONENT/SearchBar';
import List from 'COMPONENT/List';
import Select from 'COMPONENT/Select';
import Input from 'COMPONENT/Input';
import Wrapper from 'COMPONENT/Wrapper';
import moment from 'moment';
import { dealCampus } from 'UTIL/dict';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { connect } from 'dva';
import { remindByCourseListApi } from './loaders';

const courseTimeItemType = [
  'DONATE_REMIND_ITEM_OTM_STY_OADU',
  'DONATE_REMIND_ITEM_OTM_STY_NADU',
  'DONATE_REMIND_ITEM_OTM_STY_TWAIT',
  'DONATE_REMIND_ITEM_OTM_TEA_NATS',
];

const remindTypeDict = [
  { val: 'DONATE_REMIND_ITEM_OTM_TEA_OATT', text: '老师已上课考勤逾期' },
  { val: 'DONATE_REMIND_ITEM_OTM_TEA_NATC', text: '老师已上课未考勤' },
  { val: 'DONATE_REMIND_ITEM_OTM_TEA_TWAIT', text: '老师当天待考勤' },
];

class RemindByAttendanceList extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      branchId: '',
      campusId: '',
      teacherName: '',
      type:
        this.props.params === undefined || courseTimeItemType.includes(this.props.params.itemType) === true
          ? ''
          : this.props.params.itemType,
      campusDict: [],

      remindByCourseListData: [], // 列表数据
      pagination: {
        current: 1,
        pageSize: 20,
        total: 0,
        pageSizeOptions: ['20', '50', '100'],
      }, // 列表分页设置
    };

    this.onBranchIdChange = this.onBranchIdChange.bind(this);
    this.onCampusIdChange = this.onCampusIdChange.bind(this);
    this.onRemindTypeChange = this.onRemindTypeChange.bind(this);
    this.onTeacherNameChange = this.onTeacherNameChange.bind(this);
    this.onSearch = this.onSearch.bind(this);
    this.onReset = this.onReset.bind(this);
    this.clickOnSearch = this.clickOnSearch.bind(this);

    this.columns = [
      { title: '课程名称', dataIndex: 'courseName', key: 'courseName' },
      { title: '所属分公司', dataIndex: 'branchName', key: 'branchName' },
      { title: '所属学习中心', dataIndex: 'campusName', key: 'campusName' },
      { title: '课程日期', dataIndex: 'courseDate', key: 'courseDate' },
      { title: '老师', dataIndex: 'teacherName', key: 'teacherName' },
      {
        title: '考勤日期',
        dataIndex: 'firstAttendentTime',
        key: 'firstAttendentTime',
        render: (val) => {
          return val === null ? '未考勤' : moment(val).format('YYYY-MM-DD');
        },
      },
      {
        title: '乐捐提醒类型',
        dataIndex: 'teacherOverdueAtt',
        key: 'teacherOverdueAtt',
        render: (val, row) => {
          return (
            <div>
              {row.teacherOverdueAtt && <p>老师已上课考勤逾期</p>}
              {row.teacherNotAtt && <p>老师已上课未考勤</p>}
              {row.teacherTodayWaitingAtt && <p>老师当天待考勤数据</p>}
            </div>
          );
        },
      },
    ];
  }
  componentWillReceiveProps(nextProps, nextState) {
    if (this.props.campusDict.length === 0 && nextProps.campusDict.length !== 0 && this.props.params === undefined) {
      this.setState({
        campusDict: nextProps.campusDict,
        branchId: nextProps.branchDict[0].val,
        campusId: nextProps.campusDict[0].val,
      });
    }
  }

  componentDidUpdate(prevProps) {
    if (
      this.props.campusDict.length !== 0 &&
      this.props.branchDict.length !== 0 &&
      prevProps.campusDict.length === 0 &&
      prevProps.branchDict.length === 0
    ) {
      this.onSearch();
    }
  }

  onBranchIdChange(branchId) {
    const { allCampusDict } = this.props;
    const campusDict = dealCampus(allCampusDict, branchId);
    this.setState({ branchId, campusDict, campusId: '' });
  }

  onCampusIdChange(campusId) {
    this.setState({ campusId });
  }

  onTeacherNameChange(e) {
    this.setState({ teacherName: e.currentTarget.value });
  }

  onRemindTypeChange(type) {
    this.setState({ type });
  }

  async onSearch() {
    const { changeLoadingStatus } = this.props;
    changeLoadingStatus(true);
    const {
      branchId,
      campusId,
      teacherName,
      type,
      pagination: { pageSize: rows, current: page },
    } = this.state;
    const params = { branchId, campusId, teacherName, type, rows, page };
    try {
      let { total, remindByCourseListData } = await remindByCourseListApi(params);
      this.setState({ remindByCourseListData, pagination: Object.assign({}, this.state.pagination, { total }) });
      changeLoadingStatus(false);
    } catch (err) {
      showMessage(err.text);
      changeLoadingStatus(false);
    }
  }

  clickOnSearch() {
    this.setState({ pagination: Object.assign({}, this.state.pagination, { current: 1 }) }, () => {
      this.onSearch();
    });
  }

  onReset() {
    this.setState({
      branchId: '',
      campusId: '',
      teacherName: '',
      type: '',
      pagination: {
        current: 1,
        pageSize: 20,
        total: 0,
        pageSizeOptions: ['20', '50', '100'],
      },
    });
  }

  render() {
    const { branchDict } = this.props;
    const { remindByCourseListData, branchId, campusId, teacherName, type, pagination, campusDict } = this.state;

    return (
      <div>
        <Wrapper className="remindByAttendance">
          <SearchBar onSearch={this.clickOnSearch} onReset={this.onReset} expand={true}>
            <Select
              dict={branchDict}
              placeholder="所属分公司"
              value={branchId}
              onChange={this.onBranchIdChange}
              showSearch={true}
            />
            <Select
              dict={campusDict}
              placeholder="所属学习中心"
              value={campusId}
              onChange={this.onCampusIdChange}
              showSearch={true}
            />
            <Input value={teacherName} placeholder="老师" onChange={this.onTeacherNameChange} />
            <Select dict={remindTypeDict} value={type} placeholder="乐捐提醒类型" onChange={this.onRemindTypeChange} />
          </SearchBar>
        </Wrapper>
        <List
          dataSource={remindByCourseListData}
          columns={this.columns}
          pagination={pagination}
          onChange={(pagination) => {
            this.setState(
              {
                pagination: Object.assign({}, this.state.pagination, {
                  current: pagination.current,
                  pageSize: pagination.pageSize,
                }),
              },
              () => {
                this.onSearch();
              },
            );
          }}
        />
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  const pathname = state.routing.locationBeforeTransitions.pathname;
  return {
    params: state.main.pageParams[pathname],
  };
};

export default connect(mapStateToProps, null)(RemindByAttendanceList);
