import React from 'react';
import Page from 'COMPONENT/Page';
import Tabs from 'COMPONENT/Tabs';
import { connect } from 'dva';
const Tab = Tabs.Tab;
import { loadBranchDict, loadBranchCampusDict, dealCampus } from 'UTIL/dict';
import RemindByAttendanceList from './RemindByAttendanceList';
import RemindByCourseList from './RemindByCourseList';

const courseTimeItemType = [
  'DONATE_REMIND_ITEM_OTM_STY_OADU',
  'DONATE_REMIND_ITEM_OTM_STY_NADU',
  'DONATE_REMIND_ITEM_OTM_STY_TWAIT',
  'DONATE_REMIND_ITEM_OTM_TEA_NATS',
];

class OneToMoreScheduleAttendanceReminder extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      activeKey:
        this.props.params === undefined || courseTimeItemType.includes(this.props.params.itemType) === true
          ? 'courseTime'
          : 'course',
      allCampusDict: [], // 用户学习中心按分公司分组

      branchDict: [],
      campusDict: [],
    };

    this.initContent = this.initContent.bind(this);
    this.loadDict = this.loadDict.bind(this);
    this.changeLoadingStatus = this.changeLoadingStatus.bind(this);
  }

  componentDidMount() {
    this.initContent();
  }

  async initContent() {
    await this.loadDict();
  }

  async loadDict() {
    this.setState({ loading: true });
    try {
      const [branchDict, allCampusDict] = await Promise.all([loadBranchDict(), loadBranchCampusDict()]);
      let campusDict = dealCampus(allCampusDict, branchDict[0].val);
      this.setState({ branchDict, allCampusDict, campusDict, loading: false });
    } catch (error) {
      this.setState({ loading: false });
    }
  }

  changeLoadingStatus(status) {
    this.setState({ loading: status });
  }

  render() {
    const {} = this.props;
    const { loading, activeKey, branchDict, campusDict, allCampusDict } = this.state;
    return (
      <Page name="一对多排课考勤提醒" loading={loading}>
        <Tabs
          type="button"
          activeKey={activeKey}
          onChange={(activeKey) => {
            this.setState({ activeKey });
          }}
        >
          <Tab title="科次" value="courseTime">
            <RemindByAttendanceList
              branchDict={branchDict}
              campusDict={campusDict}
              changeLoadingStatus={this.changeLoadingStatus}
              allCampusDict={allCampusDict}
            />
          </Tab>
          <Tab title="课程" value="course">
            <RemindByCourseList
              branchDict={branchDict}
              campusDict={campusDict}
              changeLoadingStatus={this.changeLoadingStatus}
              allCampusDict={allCampusDict}
            />
          </Tab>
        </Tabs>
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const pathname = state.routing.locationBeforeTransitions.pathname;
  return {
    params: state.main.pageParams[pathname],
  };
};

export default connect(mapStateToProps, null)(OneToMoreScheduleAttendanceReminder);
