import React from 'react';
import { connect } from 'dva';
import { List, Button, TextLink, Loading } from 'COMPONENT';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import TransferAuditedSearchBar from './TransferAuditedSearchBar';
import TransferInfoModal from './TransferInfoModal';
import * as loaders from './loaders';
import styles from './styles.less';
import classNames from 'classnames';
import { Tooltip, Modal as AntdModal } from 'antd';
import { getSwitchConfigsByCodesAndCampusIds } from 'SERVICE/common';

const { COL_WIDTH_M, COL_WIDTH_L } = List;

// 已签单
class TransferAudited extends React.Component {
  constructor(props) {
    super(props);
    this.exportExcel = this.exportExcel.bind(this);
    this.showEmployeeTip = this.showEmployeeTip.bind(this);
    this.state = {
      loading: true,
      defaultConditions: {},
      listLoading: false,
      dataSource: [],
      total: 0,
      current: 1,
      pageSize: 20,
      searchConditions: {},
      transferInfoModalVisible: false,
      transferInfoModalData: null,
      isModalAudited: false,
      modalEditable: false,
    };
    this.$searchBar = null;
  }

  init = async (val) => {
    const page = 1;
    const res = await loaders.getTransferAuditCustomerList({
      ...val,
      rows: this.state.pageSize,
      page: page,
    });
    this.setState({
      searchConditions: val,
      total: res.records,
      dataSource: res.rows,
    });
  };

  onSearch = async (val) => {
    const page = 1;
    this.setState({ listLoading: true });
    const res = await loaders.getTransferAuditCustomerList({
      ...val,
      rows: this.state.pageSize,
      page: page,
    });
    this.setState({
      searchConditions: val,
      total: res.records,
      dataSource: res.rows,
      listLoading: false,
    });
  };

  onChange = async ({ current = this.state.current, pageSize = this.state.pageSize }) => {
    this.setState({ listLoading: true });
    const res = await loaders.getTransferAuditCustomerList({
      ...this.state.searchConditions,
      rows: pageSize,
      page: current,
    });
    this.setState({
      current: current,
      pageSize: pageSize,
      total: res.records,
      dataSource: res.rows,
      listLoading: false,
    });
  };

  componentDidMount() {
    this.setState({ loading: true });
    Promise.all([this.init(this.state.defaultConditions)]).then(() => {
      this.setState({ loading: false });
    });
  }

  showTransferInfo = (data) => {
    this.setState({
      transferInfoModalVisible: true,
      transferInfoModalData: data,
      isModalAudited: false,
      modalEditable: false,
    });
  };

  /**
   * 跳转新的签单页面，不再判断学习中心开关
   * tips: 老生-提示:当前学生已到达4次课消，请联系辅导老师进行签单
   */
  signContract = async (params = {}) => {
    const { customerId, studentId, transferIntroduceId, blCampusId } = params;
    const query = {
      studentId,
      customerId,
      transferIntroduceId,
      blCampusId,
      flag: true,
      signEntrance: 'TRANSFER_CUSTOMER',
    };

    const judgeResponse = await loaders.judgeIsOldStuForSsBoss({ studentId });
    const isOldStudent = judgeResponse && judgeResponse.data;

    if (isOldStudent) {
      showMessage('当前学生长期常规课程课消已达到4次，请联系辅导老师进行签单', 'error');
      return;
    }

    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path: `/signContractPro/${JSON.stringify(query)}`,
        refresh: true,
      },
    });
  };

  columns = [
    { title: '家长姓名', key: 'customerName', dataIndex: 'customerName', width: COL_WIDTH_M },
    {
      title: '学生姓名',
      key: 'studentName',
      dataIndex: 'studentName',
      width: COL_WIDTH_M,
      render: (val, row) => {
        return (
          <div>
            {row.studentName}
            {row.employee && (
              <div>
                <span className={'employeeFlag'}>内部员工</span>
              </div>
            )}
          </div>
        );
      },
    },
    { title: '学生年级', key: 'gradeName', dataIndex: 'gradeName', width: COL_WIDTH_M },
    {
      title: '联系方式',
      key: 'contact',
      dataIndex: 'contact',
      width: COL_WIDTH_L,
      render: (val) => {
        const texts = !val ? [''] : val.split('/').filter((ele) => ele);
        return (
          <Tooltip title={texts.join('/')}>
            <span>{`${texts[0]}${texts.length > 1 ? '/…' : ''}`}</span>
          </Tooltip>
        );
      },
    },
    { title: '介绍类型', key: 'introductionTypeName', dataIndex: 'introductionTypeName', width: COL_WIDTH_M },
    { title: '意向学习中心', key: 'intentionCampusName', dataIndex: 'intentionCampusName', width: COL_WIDTH_L },
    { title: '负责老师', key: 'inChargeUsername', dataIndex: 'inChargeUsername', width: COL_WIDTH_M },
    { title: '备注', key: 'remark', dataIndex: 'remark', width: COL_WIDTH_M, lines: 2 },
    { title: '录入人', key: 'createUsername', dataIndex: 'createUsername', width: COL_WIDTH_M },
    { title: '操作时间', key: 'signedTime', dataIndex: 'signedTime', width: COL_WIDTH_L },
    { title: '已签合同', key: 'contractId', dataIndex: 'contractId', width: COL_WIDTH_L },
    {
      title: '操作',
      key: 'action',
      width: COL_WIDTH_L,
      fixed: 'right',
      render: (val, row) => {
        const { btnAuth } = this.props;
        const { employee } = row;
        const editParam = {
          customerId: row.customerId,
          studentId: row.studentId,
          studentName: row.studentName,
          gradeId: row.gradeId,
          schoolId: row.schoolId,
          schoolName: row.schoolName,
          transferIntroduceId: row.id,
          intentionCampusId: row.intentionCampusId,
          blCampusId: row.intentionCampusId,
        };

        return (
          <span>
            {btnAuth.indexOf('CRM_TRANSFER_INFO') > -1 && (
              <TextLink
                className="list-btn"
                onClick={() => {
                  this.showTransferInfo(row);
                }}
              >
                登记详情
              </TextLink>
            )}
            {btnAuth.indexOf('CRM_TRANSFER_SIGN') > -1 && (
              <TextLink
                className={classNames({
                  [styles.textLink]: true,
                })}
                onClick={() => {
                  // 20221115版本去除内部员工无法签单限制
                  // 是否是内部员工
                  /*if (employee) {
                      this.showEmployeeTip();
                      return;
                    }*/

                  this.signContract(editParam);
                }}
              >
                签单
              </TextLink>
            )}
          </span>
        );
      },
    },
  ];

  async exportExcel() {
    const params = this.$searchBar.getParams();
    const ret = await loaders.exportTransferAudited(params);
    if (ret) {
      this.props.dispatch({ type: 'main/exportNotice' });
    }
  }

  showEmployeeTip = () => {
    const tip =
      '该学生的客户号码属于员工号码，根据集团要求，内部员工号码不允许签单，请使用非员工号码录入资源然后签单。';
    AntdModal.info({
      title: tip,
      content: '',
      className: 'homAntdModal',
    });
  };

  render() {
    const { btnAuth } = this.props;
    const {
      loading,
      defaultConditions,
      dataSource,
      total,
      current,
      pageSize,
      listLoading,
      transferInfoModalVisible,
      transferInfoModalData,
      isModalAudited,
      modalEditable,
    } = this.state;

    return (
      <div>
        <Loading loading={loading} />
        <div className={styles.action}>
          <TransferAuditedSearchBar
            ref={(c) => (this.$searchBar = this.$searchBar || c)}
            onSearch={this.onSearch}
            defaultConditions={defaultConditions}
          />
          {btnAuth.indexOf('CRM_TRANSFER_AUDITED_EXPORT') > -1 && (
            <Button onClick={this.exportExcel} style={{ marginLeft: '10px' }}>
              导出
            </Button>
          )}
        </div>
        <List
          dataSource={dataSource}
          columns={this.columns}
          pagination={{
            current,
            total,
            pageSize,
          }}
          loading={listLoading}
          onChange={this.onChange}
        />
        {transferInfoModalVisible && (
          <TransferInfoModal
            data={transferInfoModalData}
            isAudited={isModalAudited}
            editable={modalEditable}
            onClose={() => {
              setTimeout(() => {
                this.setState({ transferInfoModalVisible: false });
              }, 50);
            }}
            onSubmit={this.onSubmit}
          />
        )}
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    btnAuth: state.account.btnAuths || [],
  };
};

export default connect(mapStateToProps, null)(TransferAudited);
