import React from 'react';
import { SearchButton, Select, Input } from 'COMPONENT';
import { AutoComplete } from 'antd';
import * as loaders from './loaders';
import styles from './styles.less';
import { DatePicker } from 'antd';
import Field from './components/Field';

const RangePicker = DatePicker.RangePicker;

const dateFormat = 'YYYY-MM-DD';

const cusOrgDict = [
  { text: '家长推荐', val: 'CUSTOMER_INTRODUCE' },
  { text: '学生推荐', val: 'STUDENT_INTRODUCE' },
  { text: '员工推荐', val: 'EMPLOYEE_INTRODUCE' },
];

class TransferAuditedSearchBar extends React.Component {
  constructor(props) {
    super(props);
    this.updateDateRange = this.updateDateRange.bind(this);
    this.getParams = this.getParams.bind(this);
    const { defaultConditions } = this.props;
    this.defaultSearchConditions = Object.assign(
      {
        customerName: null,
        studentName: null,
        contact: null,
        introductionTypeId: null,
        audited: null,
        auditStatus: null,
        inChargeUserId: null,
        momentRange: [null, null],
        dateRange: [null, null],
        startDate: null,
        endDate: null,
      },
      defaultConditions,
    );

    this.defaultDicts = {
      dataSource: [],
    };

    this.state = Object.assign(
      {
        searchExpand: false, // {boolean} 搜索栏展开状态
      },
      this.defaultSearchConditions,
      this.defaultDicts,
    );
    this.searchParams = {};
  }

  onSearch = () => {
    const searchConditions = Object.keys(this.defaultSearchConditions).reduce((obj, ele) => {
      if (this.state[ele]) {
        obj[ele] = this.state[ele];
      }
      return obj;
    }, {});
    if (searchConditions.audited) {
      searchConditions.audited = Boolean(Number(searchConditions.audited));
    }
    delete searchConditions.momentRange;
    delete searchConditions.dateRange;
    this.searchParams = searchConditions;
    this.props.onSearch(searchConditions);
  };

  onReset = () => {
    this.setState({
      ...this.defaultSearchConditions,
    });
  };

  handleSearch = async (value) => {
    const names = await loaders.findStaffNameByTerm({ term: value });
    this.setState({
      dataSource: (names || []).map((ele) => ({
        text: ele.label,
        value: ele.value,
      })),
    });
  };

  updateDateRange(moments, dates) {
    this.setState({
      momentRange: moments,
      dateRange: dates,
      startDate: dates ? dates[0] : null,
      endDate: dates ? dates[1] : null,
    });
  }

  getParams() {
    return this.searchParams;
  }

  render() {
    const { style } = this.props;
    const { customerName, studentName, contact, inChargeUserId, introductionTypeId, dataSource, momentRange } =
      this.state;
    return (
      <div style={style}>
        <SearchButton onSearch={this.onSearch} onReset={this.onReset} girdConfig={[2]}>
          <Field label="首次签单时间">
            <RangePicker
              value={momentRange}
              onChange={this.updateDateRange}
              placeholder={['开始时间', '结束时间']}
              format={dateFormat}
              showTime={false}
              size="large"
            />
          </Field>
          <Input
            placeholder="家长姓名"
            value={customerName}
            onChange={(e) => this.setState({ customerName: e.target.value })}
          />
          <Input
            placeholder="学生姓名"
            value={studentName}
            onChange={(e) => this.setState({ studentName: e.target.value })}
          />
          <Input placeholder="联系方式" value={contact} onChange={(e) => this.setState({ contact: e.target.value })} />
          <AutoComplete
            dataSource={dataSource}
            onSearch={this.handleSearch}
            placeholder="负责老师"
            onChange={(e) => this.setState({ inChargeUserId: e })}
            value={inChargeUserId}
            className={styles.autocomplete}
          />
          <Select
            placeholder="请选择介绍类型"
            showSearch
            dict={cusOrgDict}
            vlaue={introductionTypeId}
            onChange={(e) => this.setState({ introductionTypeId: e })}
          />
        </SearchButton>
      </div>
    );
  }
}

export default TransferAuditedSearchBar;
