import React from 'react';
import { SearchButton, Select, Input } from 'COMPONENT';
import { DatePicker } from 'antd';
import Field from './components/Field';

const RangePicker = DatePicker.RangePicker;
const dateFormat = 'YYYY-MM-DD';

const cusOrgDict = [
  { text: '家长推荐', val: 'CUSTOMER_INTRODUCE' },
  { text: '学生推荐', val: 'STUDENT_INTRODUCE' },
  { text: '员工推荐', val: 'EMPLOYEE_INTRODUCE' },
];

const auditTypeDict = [
  { val: '0', text: '未审核' },
  { val: '1', text: '已审核' },
];

const auditStatusDict = [
  { val: 'NOT_PASS', text: '不通过' },
  { val: 'PASS', text: '通过' },
];

class TransferAvailableSearchBar extends React.Component {
  constructor(props) {
    super(props);
    this.updateDateRange = this.updateDateRange.bind(this);
    this.getParams = this.getParams.bind(this);
    const { defaultConditions } = this.props;
    this.defaultSearchConditions = Object.assign(
      {
        customerName: null,
        studentName: null,
        contact: null,
        introductionTypeId: null,
        audited: null,
        auditStatus: null,
        momentRange: [null, null],
        dateRange: [null, null],
        startDate: null,
        endDate: null,
      },
      defaultConditions,
    );

    this.defaultDicts = {};

    this.state = Object.assign(
      {
        searchExpand: false, // {boolean} 搜索栏展开状态
      },
      this.defaultSearchConditions,
      this.defaultDicts,
    );
    this.searchParams = {};
  }

  onSearch = () => {
    const searchConditions = Object.keys(this.defaultSearchConditions).reduce((obj, ele) => {
      if (this.state[ele]) {
        obj[ele] = this.state[ele];
      }
      return obj;
    }, {});
    if (searchConditions.audited) {
      searchConditions.audited = Boolean(Number(searchConditions.audited));
    }
    delete searchConditions.momentRange;
    delete searchConditions.dateRange;
    this.searchParams = searchConditions;
    this.props.onSearch(this.getParams());
  };

  onReset = () => {
    this.setState({
      ...this.defaultSearchConditions,
    });
  };

  updateDateRange(moments, dates) {
    this.setState({
      momentRange: moments,
      dateRange: dates,
      startDate: dates ? dates[0] : null,
      endDate: dates ? dates[1] : null,
    });
  }

  getParams() {
    return this.searchParams;
  }

  render() {
    const { style } = this.props;
    const { customerName, studentName, contact, introductionTypeId, audited, auditStatus, momentRange } = this.state;

    return (
      <div style={style}>
        <SearchButton onSearch={this.onSearch} onReset={this.onReset} girdConfig={[2]}>
          <Field label="审核通过时间">
            <RangePicker
              value={momentRange}
              onChange={this.updateDateRange}
              placeholder={['开始时间', '结束时间']}
              format={dateFormat}
              showTime={false}
              size="large"
            />
          </Field>
          <Input
            placeholder="家长姓名"
            value={customerName}
            onChange={(e) => this.setState({ customerName: e.target.value })}
          />
          <Input
            placeholder="学生姓名"
            value={studentName}
            onChange={(e) => this.setState({ studentName: e.target.value })}
          />
          <Input placeholder="联系方式" value={contact} onChange={(e) => this.setState({ contact: e.target.value })} />
          <Select
            placeholder="介绍类型"
            showSearch
            dict={cusOrgDict}
            value={introductionTypeId}
            onChange={(val) => this.setState({ introductionTypeId: val })}
          />
          <Select
            placeholder="审核状态"
            showSearch
            dict={auditTypeDict}
            value={audited}
            onChange={(val) => this.setState({ audited: val })}
          />
          <Select
            placeholder="审核结果"
            showSearch
            dict={auditStatusDict}
            value={auditStatus}
            onChange={(val) => this.setState({ auditStatus: val })}
          />
        </SearchButton>
      </div>
    );
  }
}

export default TransferAvailableSearchBar;
