import React from 'react';
import { connect } from 'dva';
import { TextArea, Button, Select, Modal, FormItem, Icon } from 'COMPONENT';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { Form, Input, AutoComplete, Radio } from 'antd';
import { loadCommonDictByGroup, loadAllBranchCampusDict } from 'UTIL/dict';
import * as loaders from './loaders';
import SchoolModal from './SchoolModal';
import styles from './styles.less';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import { loginUser, isIdCard } from 'UTIL/common';
import { CERTIFICATE_TYPE_ID_CARD } from 'PAGE/SSCRM/common/util/dicts';

const RadioGroup = Radio.Group;

const FormItemNative = Form.Item;
const formItemCol = {
  labelWidth: '80px',
  className: styles.formItem,
};
const phoneRegExp = /^1\d{10}$/;

const cusOrgDict = [
  { text: '家长推荐', val: 'CUSTOMER_INTRODUCE' },
  { text: '学生推荐', val: 'STUDENT_INTRODUCE' },
  { text: '员工推荐', val: 'EMPLOYEE_INTRODUCE' },
];
// 身份证
const certificateTypeIdCard = CERTIFICATE_TYPE_ID_CARD;

class TransferInfoModal extends React.Component {
  constructor(props) {
    super(props);

    this.dicts = {
      studentGradeDict: [],
      campusDict: [],
      stationDict: [],
      certificateTypeDict: [], // 字典，证件类型
    };

    this.state = Object.assign(this.dicts, {
      loading: true,
      schoolModalVisible: false,
      dataSource: [],
      transferInfo: null,
      disabled: false,
      checkParams: {},
      showMessage: false,
      message: '',
      contactErr: '',
    });

    this.uuid = 1;
  }

  /**
   * 提交保存
   *
   * @param e
   */
  onFormSubmit = (e) => {
    const { isAudited, onSubmit } = this.props;
    const { transferInfo } = this.state;

    e.preventDefault();

    this.props.form.validateFieldsAndScroll((err, values) => {
      if (err) {
        let tips = '请完善信息再进行操作!';

        if (isAudited && (err.introducer || err.introducerContact)) {
          tips = '请补充介绍人信息再进行审核！';
        }

        return this.modal.showOkTips(tips);
      }

      const contacts = Object.keys(values)
        .filter((key) => key.indexOf('contact-') > -1)
        .map((ele) => values[ele] && values[ele].trim())
        .filter((value) => value && value.length === 11)
        .filter((value) => value.trim());
      values = Object.assign({}, values, { contacts });

      const params = Object.keys(values)
        .filter((key) => key.indexOf('contact-') < 0)
        .reduce((obj, ele) => {
          obj[ele] = values[ele] || '';
          if (typeof values[ele] === 'string') {
            obj[ele] = obj[ele].trim();
          }
          if (ele === 'studentName' || ele === 'customerName' || ele === 'introducer') {
            obj[ele] = obj[ele].replace(/\s/g, '');
          }
          return obj;
        }, {});

      if (!isAudited && (!transferInfo || !transferInfo.id)) {
        onSubmit(params);
      } else if (!isAudited && transferInfo && transferInfo.id) {
        onSubmit({
          ...params,
          id: transferInfo.id,
        });
      } else if (isAudited && transferInfo && transferInfo.id) {
        const { inChargeUserId, inChargeUserStation, auditStatus, auditRemark } = params;
        onSubmit({
          id: transferInfo.id,
          inChargeUserId,
          inChargeUserStation,
          presentAudit: {
            auditStatus,
            remark: auditRemark,
          },
        });
      }
    });
  };

  onSchoolModalCommit = (params) => {
    this.props.form.setFieldsValue({
      schoolName: params.schoolName,
      schoolId: params.schoolId,
    });
  };

  onClearSchoolClick = () => {
    this.props.form.setFieldsValue({
      schoolName: '',
      schoolId: '',
    });
  };

  removeContact = (k) => {
    const { form } = this.props;
    const contacts = form.getFieldValue('contacts');
    if (contacts.length <= 2) {
      return;
    }

    form.setFieldsValue({
      contacts: contacts.filter((key) => key !== k),
    });
  };

  addContact = () => {
    this.uuid++;
    const { form } = this.props;
    const contacts = form.getFieldValue('contacts');
    const nextContacts = contacts.concat(this.uuid);
    form.setFieldsValue({
      contacts: nextContacts,
    });
  };

  handleSearch = async (value) => {
    const names = await loaders.findStaffNameByTerm({ term: value });

    this.setState({
      dataSource: (names || []).map((ele) => ({
        text: ele.label,
        value: ele.value,
      })),
    });
  };

  contactsValidator = (rule, value, callback) => {
    const { getFieldValue } = this.props.form;
    const contacts = getFieldValue('contacts');
    if (value) {
      const repeatCount = contacts.map((ele) => getFieldValue('contact-' + ele)).filter((ele) => ele === value);
      if (repeatCount.length > 1) {
        callback('请不要重复填写联系方式');
      }
    }
    callback();
  };

  teacherValidator = (rule, value, callback) => {
    const { dataSource } = this.state;
    if (value) {
      const idx = dataSource.findIndex((ele) => ele.value === value);
      if (idx < 0) {
        callback('请点击选择下拉框内老师');
      }
    }
    callback();
  };

  schoolValidator = (rule, value, callback) => {
    const { getFieldValue } = this.props.form;
    const schoolId = getFieldValue('schoolId');

    if (value && !schoolId) {
      callback('请点击选择弹窗内学校');
    }
    callback();
  };

  /**
   * 检查手机号码重复
   *
   * @return {Promise<void|*>}
   */
  checkContacts = async () => {
    const { transferInfo } = this.state;

    if (transferInfo && transferInfo.id) {
      return;
    }

    const { getFieldValue } = this.props.form;
    const contactsField = getFieldValue('contacts');
    const studentName = (getFieldValue('studentName') || '').replace(/\s/g, '');
    const contacts = contactsField
      .map((ele) => getFieldValue('contact-' + ele))
      .filter((ele) => ele && ele.trim().length === 11)
      .map((value) => value.trim());

    if (contacts.length) {
      const { checkParams } = this.state;
      const { studentName: checkedStudent = undefined, contacts: checkedContacts = [] } = checkParams;
      if (
        checkedStudent === studentName &&
        checkedContacts.map((ele) => contacts.indexOf(ele) > -1).filter((ele) => !!ele).length === contacts.length
      ) {
        return;
      }
      this.setState({ disabled: true, contactErr: '' });
      try {
        const checkParams = {
          studentName,
          contacts,
        };
        this.setState({ checkParams });
        const result = await loaders.checkTransferCustomer({
          studentName,
          contacts,
        });

        if (Number(result.resultCode) === 0) {
          this.setState({ disabled: false });
          if (studentName) {
            this.props.form.setFields({
              studentName: {
                value: studentName,
                errors: [new Error(result.resultMessage)],
              },
            });
          } else {
            this.setState({ contactErr: result.resultMessage });
          }
          return;
        }

        if (Number(result.resultCode) === 1) {
          this.setState({ disabled: false, contactErr: '' });
          return this.setState({
            showMessage: true,
            message: result.resultMessage,
          });
        }

        if (Number(result.resultCode) === 2) {
          this.setState({ disabled: false, contactErr: '' });
          this.props.form.setFields({
            studentName: {
              value: studentName,
            },
          });
        }
      } catch (error) {
        this.setState({ disabled: false, contactErr: '' });
        return showMessage(error.message, 'error');
      }
    }
  };

  /**
   * 家长信息-手机号码输入
   *
   * @param val {string} 手机号
   * @param key {string} 表单字段
   */
  onContactChange = (val, key) => {
    const value = val ? val.trim() : null;
    this.props.form.setFieldsValue({ [key]: value });
  };

  checkModify = () => {
    const { isFieldsTouched } = this.props.form;
    const { onClose } = this.props;
    if (isFieldsTouched()) {
      return this.setState({ showConfirm: true });
    }
    return onClose();
  };

  /**
   * 伪造学校字典
   * 当已选资源学校或意向学校时，后端获取不到字典则需要伪造学校
   * @param campusDict 学习中心字典
   * @param data 登记信息
   * @return {*} {resCampusDict: 资源学校, intentionCampusDict: 意向学校}
   */
  forgeCampusDict = (campusDict, data) => {
    let resCampusDict = campusDict;
    let intentionCampusDict = campusDict;

    if (data) {
      const { resCampusId, resCampusName, intentionCampusId, intentionCampusName } = data;

      if (campusDict && campusDict.length > 0) {
        const hasResCampus = this.finValueInDict(campusDict, resCampusId);
        const hasIntentionCampus = this.finValueInDict(campusDict, intentionCampusId);

        if (!hasResCampus) {
          resCampusDict = campusDict.concat([
            {
              label: resCampusName,
              val: '',
              options: [{ text: resCampusName, val: resCampusId }],
            },
          ]);
        }

        if (!hasIntentionCampus) {
          intentionCampusDict = campusDict.concat([
            {
              label: intentionCampusName,
              val: '',
              options: [{ text: intentionCampusName, val: intentionCampusId }],
            },
          ]);
        }
      } else {
        resCampusDict = (resCampusId && [{ text: resCampusName, val: resCampusId }]) || [];
        intentionCampusDict = (intentionCampusId && [{ text: intentionCampusName, val: intentionCampusId }]) || [];
      }
    }

    return {
      resCampusDict,
      intentionCampusDict,
    };
  };

  /**
   * 查找字典中是否存在某个值
   * @param dict 字典
   * @param value 值
   * @return {boolean}
   */
  finValueInDict(dict, value) {
    if (!dict || !value) {
      return false;
    }

    let arr = [];

    dict.forEach((item) => {
      arr = arr.concat(item.options || []);
    });

    return arr.some((item) => item.val === value);
  }

  /**
   * 检查证件类型
   *
   * @param rule
   * @param value
   * @param callback
   */
  checkIdCardType = (rule, value, callback) => {
    const {
      form: { getFieldValue },
      isAudited,
      editable,
    } = this.props;
    const cusIdCardType = getFieldValue('customerIdCardType');
    const cusIdCardNo = getFieldValue('customerIdCardNo');

    // 兼容登记审核
    const shouldPass = isAudited || !editable;
    // 证件类型、证件号码不同时为空时
    const shouldReturn = cusIdCardNo && !cusIdCardType;

    if (shouldPass) {
      callback();
    } else if (shouldReturn) {
      callback('请选择证件类型！');
    } else {
      callback();
    }
  };

  /**
   * 检查证件号码
   *
   * @param rule
   * @param value
   * @param callback
   */
  checkIdNumber = (rule, value, callback) => {
    const {
      form: { getFieldValue },
      isAudited,
      editable,
    } = this.props;
    const cusIdCardType = getFieldValue('customerIdCardType');
    const cusIdCardNo = getFieldValue('customerIdCardNo');

    // 兼容登记审核
    const shouldPass = isAudited || !editable;
    // 证件类型、证件号码不同时为空时
    const shouldReturn = cusIdCardType && !cusIdCardNo;

    if (shouldPass) {
      callback();
    } else if (shouldReturn) {
      callback('请填写证件号码！');
    } else if (cusIdCardType) {
      // 再校验身份证号码
      if (cusIdCardType === certificateTypeIdCard && !isIdCard(cusIdCardNo)) {
        callback('客户身份证格式错误');
      } else if (!cusIdCardNo.trim()) {
        callback('客户证件号码错误');
      } else {
        callback();
      }
    } else {
      callback();
    }
  };

  /**
   * 身份证类型修改
   */
  onIdCardTypeChange = () => {
    this.props.form.validateFields(['customerIdCardNo', 'customerIdCardType'], { force: true });
  };

  /**
   * 证件号码修改
   */
  onIdNumberChange = () => {
    this.props.form.validateFields(['customerIdCardNo', 'customerIdCardType'], { force: true });
  };

  /**
   * 检查学生身份证
   *
   * @param rule
   * @param value
   * @param callback
   */
  checkIdNumberOnly = (rule, value, callback) => {
    if (!value || isIdCard(value)) {
      callback();
    } else {
      callback('身份证格式错误');
    }
  };

  componentDidMount() {
    const { brenchId, organizationType } = this.props.user;
    const { data, form } = this.props;
    const promises = [
      loadCommonDictByGroup('STUDENT_GRADE,TwoTeacherTicUserStation,CERTIFICATE_TYPE'),
      loadAllBranchCampusDict(),
    ];

    if (data && data.id) {
      promises.push(loaders.loadTransferInfo({ id: data.id }));
    }

    Promise.all(promises).then(([groupDict, campusDict, transferInfo]) => {
      if (organizationType !== 'GROUNP') {
        campusDict = brenchId ? campusDict.filter((branch) => branch.val === brenchId) : [];
      }

      this.setState({
        studentGradeDict: groupDict['STUDENT_GRADE'],
        stationDict: groupDict['TwoTeacherTicUserStation'],
        certificateTypeDict: groupDict['CERTIFICATE_TYPE'],
        campusDict,
        loading: false,
      });

      if (transferInfo) {
        this.uuid = transferInfo.contacts.length;
        const fieldContacts = Array(transferInfo.contacts.length)
          .fill(null)
          .map((ele, i) => i);
        const newFormValue = Object.assign({
          customerName: transferInfo.customerName || '',
          contacts: fieldContacts,
          remark: transferInfo.remark || '',
          studentName: transferInfo.studentName || '',
          gradeId: transferInfo.gradeId || '',
          schoolName: transferInfo.schoolName || '',
          schoolId: transferInfo.schoolId || '',
          resCampusId: transferInfo.resCampusId || '',
          intentionCampusId: transferInfo.intentionCampusId || '',
          introductionTypeId: transferInfo.introductionTypeId || '',
          inChargeUserId: transferInfo.inChargeUserId || '',
          inChargeUserStation: '' + transferInfo.inChargeUserStation || '',
          introducer: transferInfo.introducer || '',
          introducerContact: transferInfo.introducerContact || '',
          customerIdCardType: transferInfo.customerIdCardType || null,
          customerIdCardNo: transferInfo.customerIdCardNo,
          studentIdCardNo: transferInfo.studentIdCardNo,
        });

        this.setState({
          dataSource: [
            {
              text: transferInfo.inChargeUsername,
              value: transferInfo.inChargeUserId,
            },
          ],
          transferInfo,
        });
        form.setFieldsValue(newFormValue);
        setTimeout(() => {
          form.setFieldsValue(
            fieldContacts.reduce((obj, ele) => {
              obj[`contact-${ele}`] = transferInfo.contacts[ele] || '';
              return obj;
            }, {}),
          );
        }, 50);
      } else {
        const { name, userId } = this.props.user;
        this.setState({
          dataSource: [
            {
              text: name,
              value: userId,
            },
          ],
        });
        form.setFieldsValue({ inChargeUserId: userId });
      }
    });
  }

  render() {
    const { isAudited, editable } = this.props;
    const { organizationId } = this.props.user;
    const { getFieldDecorator, getFieldValue } = this.props.form;
    const {
      loading,
      studentGradeDict,
      campusDict,
      stationDict,
      schoolModalVisible,
      dataSource,
      disabled,
      showMessage,
      message,
      showConfirm,
      transferInfo,
      contactErr,
    } = this.state;
    const { resCampusDict, intentionCampusDict } = this.forgeCampusDict(campusDict, transferInfo);

    getFieldDecorator('contacts', { initialValue: [0, 1] });
    const contacts = getFieldValue('contacts');
    const ContactsFormItems = contacts.map((k, index) => {
      return (
        <FormItem
          key={k}
          {...formItemCol}
          name={`联系方式${index + 1}`}
          required={index === 0}
          className={styles.formItem + ' ' + styles.contactFormItem}
        >
          <FormItemNative>
            {getFieldDecorator(`contact-${k}`, {
              rules: [
                { transform: (val) => (val ? val.trim() : val) },
                { required: index === 0, message: '联系方式' },
                {
                  pattern: phoneRegExp,
                  message: '请填写正确的联系方式',
                },
                {
                  validator: this.contactsValidator,
                },
              ],
            })(
              <Input
                className={styles.item}
                onBlur={this.checkContacts}
                onChange={(e) => {
                  this.onContactChange(e.target.value, `contact-${k}`);
                }}
                disabled={isAudited || !editable || !!transferInfo}
              />,
            )}
            {!(isAudited || !editable) && index > 1 ? (
              <div className={styles.deleteBtn}>
                <Icon type="minus" size={16} disabled={contacts.length === 2} onClick={() => this.removeContact(k)} />
              </div>
            ) : null}
          </FormItemNative>
        </FormItem>
      );
    });

    return (
      <Modal
        loading={loading}
        title="转介绍登记"
        onOk={this.onFormSubmit}
        onClose={this.checkModify}
        okBtnShow={editable && !disabled}
        cancelText={editable ? '取消' : '关闭'}
        closable={true}
        widthSize="l"
        ref={(c) => (this.modal = getWrappedInstance(c))}
      >
        <Form className="Page-scroller" style={{ margin: '0' }}>
          <div className={styles.block}>
            <div className={styles.row}>
              <h3 className={styles.header}>家长信息</h3>
            </div>
            <div className={styles.row}>
              <FormItem {...formItemCol} name="家长姓名" required>
                <FormItemNative>
                  {getFieldDecorator('customerName', {
                    rules: [
                      { transform: (val) => (val ? val.replace(/\s/g, '') : val) },
                      { required: true, message: '请填写家长姓名' },
                      { max: 20, message: '客户姓名不能超过20字' },
                      { whitespace: true },
                    ],
                  })(<Input className={styles.item} disabled={isAudited || !editable} />)}
                </FormItemNative>
              </FormItem>
              <FormItem {...formItemCol} name="证件类型">
                <FormItemNative>
                  {getFieldDecorator('customerIdCardType', {
                    rules: [{ required: false, validator: this.checkIdCardType }],
                  })(
                    <Select
                      className={styles.item}
                      dict={this.state.certificateTypeDict}
                      disabled={isAudited || !editable}
                      onBlur={this.onIdCardTypeChange}
                    />,
                  )}
                </FormItemNative>
              </FormItem>
              <FormItem {...formItemCol} name="证件号码">
                <FormItemNative>
                  {getFieldDecorator('customerIdCardNo', {
                    rules: [{ required: false, validator: this.checkIdNumber }],
                  })(
                    <Input className={styles.item} disabled={isAudited || !editable} onBlur={this.onIdNumberChange} />,
                  )}
                </FormItemNative>
              </FormItem>
            </div>
            <div className={styles.row}>
              {ContactsFormItems}
              {!(isAudited || !editable || transferInfo) && (
                <Button
                  type="default"
                  onClick={this.addContact}
                  style={{ marginBottom: '14px', borderColor: '#6d67ff' }}
                >
                  <Icon type="plus" color="#6d67ff" /> 新增联系方式
                </Button>
              )}
              {contactErr && (
                <div
                  style={{ color: 'red', width: '100%', display: 'block', marginBottom: '20px', marginTop: '-15px' }}
                >
                  {contactErr}
                </div>
              )}
            </div>
            <div className={styles.row}>
              <FormItem {...formItemCol} style={{ width: '100%', margin: 0 }} name="备注">
                <FormItemNative>
                  {getFieldDecorator('remark', {
                    rules: [{ max: 50, message: '客户情况不能超过50字' }],
                  })(
                    <TextArea
                      maxLength={50}
                      wordCount={true}
                      rows={1}
                      style={{ width: '100%' }}
                      disabled={isAudited || !editable}
                    />,
                  )}
                </FormItemNative>
              </FormItem>
            </div>
          </div>
          <div className={styles.block}>
            <div className={styles.row}>
              <h3 className={styles.header}>学生信息</h3>
            </div>
            <div className={styles.row}>
              <FormItem {...formItemCol} name="学生姓名" required>
                <FormItemNative>
                  {getFieldDecorator('studentName', {
                    rules: [
                      { transform: (val) => (val ? val.replace(/\s/g, '') : val) },
                      { required: true, message: '请填写学生姓名' },
                      { max: 20, message: '学生姓名不能超过20字' },
                      { whitespace: true },
                    ],
                  })(<Input className={styles.item} onBlur={this.checkContacts} disabled={isAudited || !editable} />)}
                </FormItemNative>
              </FormItem>
              <FormItem {...formItemCol} name="学生年级" required>
                <FormItemNative>
                  {getFieldDecorator('gradeId', {
                    rules: [{ required: true, message: '请选择学生年级' }],
                  })(
                    <Select
                      placeholder="请选择学生年级"
                      showSearch
                      dict={studentGradeDict}
                      className={styles.item}
                      disabled={isAudited || !editable}
                    />,
                  )}
                </FormItemNative>
              </FormItem>
              <FormItem {...formItemCol} name="就读学校">
                <FormItemNative>
                  {getFieldDecorator('schoolName', {
                    rules: [{ validator: this.schoolValidator }],
                  })(
                    <Input
                      className={styles.item}
                      autoComplete="off"
                      onClick={() => {
                        this.setState({ schoolModalVisible: true });
                      }}
                      suffix={
                        !(isAudited || !editable) && getFieldValue('schoolId') ? (
                          <Icon type="cross_circle" style={{ color: '#6d67ff' }} onClick={this.onClearSchoolClick} />
                        ) : null
                      }
                      style={{ caretColor: 'transparent' }}
                      disabled={isAudited || !editable}
                    />,
                  )}
                </FormItemNative>
                <div style={{ display: 'none' }}>
                  <FormItemNative>
                    {getFieldDecorator(
                      'schoolId',
                      {},
                    )(<Input className={styles.item} disabled={isAudited || !editable} />)}
                  </FormItemNative>
                </div>
              </FormItem>
              <FormItem {...formItemCol} name="资源学习中心" required>
                <FormItemNative>
                  {getFieldDecorator('resCampusId', {
                    rules: [{ required: true, message: '请选择资源学习中心' }],
                  })(
                    <Select
                      placeholder="请选择资源学习中心"
                      showSearch
                      dict={resCampusDict}
                      className={styles.item}
                      disabled={isAudited || !editable}
                    />,
                  )}
                </FormItemNative>
              </FormItem>
              <FormItem {...formItemCol} name="意向学习中心" required>
                <FormItemNative>
                  {getFieldDecorator('intentionCampusId', {
                    rules: [{ required: true, message: '请选择意向学习中心' }],
                  })(
                    <Select
                      placeholder="请选择意向学习中心"
                      showSearch
                      dict={intentionCampusDict}
                      className={styles.item}
                      disabled={isAudited || !editable}
                    />,
                  )}
                </FormItemNative>
              </FormItem>
              {loginUser.isTwoTeacherSym() && (
                <FormItem {...formItemCol} name="学生身份证">
                  <FormItemNative>
                    {getFieldDecorator('studentIdCardNo', {
                      rules: [{ required: false, message: '身份证格式错误', validator: this.checkIdNumberOnly }],
                    })(<Input className={styles.item} disabled={isAudited || !editable} />)}
                  </FormItemNative>
                </FormItem>
              )}
            </div>
          </div>
          <div className={styles.block}>
            <div className={styles.row}>
              <h3 className={styles.header}>介绍人信息</h3>
            </div>
            <div className={styles.row}>
              <FormItem {...formItemCol} name="介绍类型" required>
                <FormItemNative>
                  {getFieldDecorator('introductionTypeId', {
                    rules: [{ required: true, message: '请选择介绍类型' }],
                  })(
                    <Select
                      placeholder="请选择介绍类型"
                      showSearch
                      dict={cusOrgDict}
                      className={styles.item}
                      disabled={isAudited || !editable}
                    />,
                  )}
                </FormItemNative>
              </FormItem>
              <FormItem {...formItemCol} name="负责老师" required>
                <FormItemNative>
                  {getFieldDecorator('inChargeUserId', {
                    rules: [{ required: true, message: '请选择负责老师' }, { validator: this.teacherValidator }],
                  })(
                    <AutoComplete
                      dataSource={dataSource}
                      onSearch={this.handleSearch}
                      placeholder="请选择负责老师"
                      disabled={!editable}
                    />,
                  )}
                </FormItemNative>
              </FormItem>
              <FormItem {...formItemCol} name="老师岗位" required>
                <FormItemNative>
                  {getFieldDecorator('inChargeUserStation', {
                    rules: [{ required: true, message: '请选择老师岗位' }],
                  })(
                    <Select
                      placeholder="请选择老师岗位"
                      showSearch
                      dict={stationDict}
                      className={styles.item}
                      disabled={!editable}
                    />,
                  )}
                </FormItemNative>
              </FormItem>
              <FormItem {...formItemCol} name="介绍人" required>
                <FormItemNative>
                  {getFieldDecorator('introducer', {
                    rules: [
                      { transform: (val) => (val ? val.replace(/\s/g, '') : val) },
                      { required: true, message: '请填写介绍人' },
                      { max: 20, message: '介绍人姓名不能超过20字' },
                      { whitespace: true },
                    ],
                  })(<Input className={styles.item} disabled={isAudited || !editable} />)}
                </FormItemNative>
              </FormItem>
              <FormItem {...formItemCol} name="联系方式" required>
                <FormItemNative>
                  {getFieldDecorator('introducerContact', {
                    rules: [
                      { transform: (val) => (val ? val.trim() : val) },
                      { required: true, message: '请填写联系方式' },
                      { pattern: phoneRegExp, message: '请填写正确的联系方式' },
                    ],
                  })(<Input className={styles.item} disabled={isAudited || !editable} />)}
                </FormItemNative>
              </FormItem>
            </div>
          </div>
          {isAudited && (
            <div>
              <div className={styles.hrContainer}>
                <div className={styles.hr} />
              </div>
              <div className={styles.row}>
                <h3 className={styles.header}>审核</h3>
                <FormItemNative style={{ marginBottom: 0, marginLeft: '46px' }}>
                  {getFieldDecorator('auditStatus', {
                    rules: [{ required: true, message: '请选择审核结果' }],
                  })(
                    <RadioGroup style={{ position: 'relative', top: '-5px' }}>
                      <Radio value="PASS">通过</Radio>
                      <Radio value="NOT_PASS">不通过</Radio>
                    </RadioGroup>,
                  )}
                </FormItemNative>
              </div>
              <div className={styles.row}>
                <FormItem {...formItemCol} style={{ width: '100%', margin: 0 }} name="审核备注">
                  <FormItemNative>
                    {getFieldDecorator('auditRemark', {
                      rules: [{ max: 30, message: '客户情况不能超过30字' }],
                    })(
                      <TextArea
                        maxLength={30}
                        wordCount={true}
                        rows={1}
                        style={{ width: '100%' }}
                        disabled={!editable}
                      />,
                    )}
                  </FormItemNative>
                </FormItem>
              </div>
            </div>
          )}
        </Form>
        {schoolModalVisible && (
          <SchoolModal
            orgId={organizationId}
            onCommit={this.onSchoolModalCommit}
            onClose={() => this.setState({ schoolModalVisible: false })}
          />
        )}
        {showMessage && (
          <Modal
            title="提示"
            onOk={() => this.setState({ showMessage: false, disabled: false })}
            onClose={() => {
              this.props.onClose();
            }}
            closable={true}
            widthSize="s"
          >
            <div style={{ display: 'flex' }}>
              <Icon
                type="warning"
                size={24}
                color="#ffb444"
                style={{ height: '24px', width: '24px', lineHeight: '24px', marginRight: '5px' }}
              />
              {message}
            </div>
          </Modal>
        )}
        {showConfirm && (
          <Modal
            title="提示"
            onOk={() => this.props.onClose()}
            onClose={() => {
              this.setState({ showConfirm: false });
            }}
            closable={true}
            widthSize="s"
          >
            <div style={{ display: 'flex' }}>
              <Icon
                type="warning"
                size={24}
                color="#ffb444"
                style={{ height: '24px', width: '24px', lineHeight: '24px', marginRight: '5px' }}
              />
              内容已修改，是否确认离开?
            </div>
          </Modal>
        )}
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    user: state.account.user,
  };
};
export default connect(mapStateToProps)(Form.create()(TransferInfoModal));
