import { findOrganizationById, getSchoolByBlCampus, getRegionList } from 'SERVICE/common';
import { getCustomerList, findStaffName, addCustomer, auditCustomer, updateCustomer } from 'SERVICE/crmBench';
import { getCitys } from 'SERVICE/customer';
import { judgeIsOldStuForSsBoss } from 'SERVICE/ssCRM';
import { exportTwoTeacherTicUnsignedList, exportTwoTeacherTicSignedList } from 'SERVICE/exportFile';
import fetch from 'dva/fetch';
import { urlBase } from 'CONST/config';
import { errorsHandle } from 'UTIL/common';

export const auditTransferCustomer = async (params) => {
  const data = await auditCustomer(params);
  if (data.resultCode !== 0) {
    throw new Error(data.resultMessage);
  }
  return data;
};

export const loadTransferInfo = async (params) => {
  const id = params.id;
  const raw = await fetch(`${urlBase}twoteacher/tic/customer/${id}.do?`, {
    credentials: 'same-origin',
    method: 'GET',
    headers: {
      'Content-Type': 'application/json;charset=utf-8',
    },
  });
  const res = await raw.json();
  if (res.resultCode !== 0) {
    throw new Error(res.resultMessage);
  }

  return res.data;
};

/**
 * 修改客户信息
 *
 * @param params
 * @return {Promise<*>}
 */
export const updateTransferCustomer = async (params) => {
  const data = await updateCustomer(params);
  if (data.resultCode !== 0) {
    throw new Error(data.resultMessage);
  }
  return data;
};

export const checkTransferCustomer = async (params) => {
  const raw = await fetch(`${urlBase}twoteacher/tic/customer/check.do`, {
    credentials: 'same-origin',
    method: 'POST',
    headers: {
      'Content-Type': 'application/json;charset=utf-8',
    },
    body: JSON.stringify(params),
  });
  const res = await raw.json();
  return res;
};

/**
 * 新增客户
 *
 * @param params
 * @return {Promise<*>}
 */
export const addTransferCustomer = async (params) => {
  const data = await addCustomer(params);
  if (data.resultCode !== 0) {
    throw new Error(data.resultMessage);
  }
  return data;
};

export const findStaffNameByTerm = (params) => {
  return findStaffName(params);
};

export const getTransferAvailabelCustomerList = async (params) => {
  const data = await getCustomerList(params);
  if (data.resultCode !== 0) {
    throw new Error(data.resultMessage);
  }
  return data;
};

export const getTransferAuditCustomerList = async (params) => {
  params['signed'] = 1;
  const data = await getCustomerList(params);
  if (data.resultCode !== 0) {
    throw new Error(data.resultMessage);
  }
  return data;
};

export const regionList = async () => {
  const res = await getRegionList();
  return res.map((ele) => {
    return { val: ele.id, text: ele.name };
  });
};

export const organization = async (params) => {
  return await findOrganizationById(params);
};

export const cityList = async (params) => {
  const res = await getCitys(params);
  return res.map((ele) => {
    return { val: ele.id, text: ele.name };
  });
};

export const schoolList = async (params) => {
  const res = await getSchoolByBlCampus(params);
  const value = res.value || {};
  return Object.keys(value).map((ele) => {
    return { val: ele, text: value[ele] };
  });
};

export const exportTransferAvailable = async (params) => {
  const rsp = await exportTwoTeacherTicUnsignedList(params, true).catch(errorsHandle());
  return rsp && rsp.resultCode === 0;
};

export const exportTransferAudited = async (params) => {
  const rsp = await exportTwoTeacherTicSignedList(params, true).catch(errorsHandle());
  return rsp && rsp.resultCode === 0;
};

export { judgeIsOldStuForSsBoss };
