import React from 'react';
import { connect } from 'dva';
import Modal from 'COMPONENT/Modal';
import { Form } from 'antd';
import Select from 'COMPONENT/Select';
import loaders from './loaders';

const FormItem = Form.Item;

class SchoolModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: true,
      province: '',
      city: '',
      schoolId: '',
      schoolName: '',
      organization: {},
      regionDict: [],
      cityDict: [],
      schoolDict: [],
    };
  }

  onCommit = async () => {
    const { onClose, onCommit } = this.props;
    onCommit({
      schoolId: this.state.schoolId,
      schoolName: this.state.schoolName,
    });
    onClose();
  };

  getCityDict = async (params) => {
    if (!params) {
      return this.setState({
        cityDict: [],
        city: '',
        schoolDict: [],
        schoolId: '',
        schoolName: '',
      });
    }
    const res = await loaders.cityList({ provinceID: params });
    this.setState({
      cityDict: res,
      city: '',
      schoolDict: [],
      schoolId: '',
      schoolName: '',
    });
  };

  getSchoolDict = async (params) => {
    if (!params) {
      return this.setState({
        schoolDict: [],
        schoolId: '',
        schoolName: '',
      });
    }
    const res = await loaders.schoolList({ blCampus: params });
    const schoolDict = res.filter((ele) => ele.val);
    this.setState({ schoolDict: schoolDict, schoolId: '', schoolName: '' });
  };

  componentDidMount() {
    this.setState({ loading: true });
    const {
      orgId,
      user: { campusId, brenchId },
    } = this.props;
    Promise.all([
      loaders.regionList(),
      loaders.organization({
        id: orgId || campusId || brenchId,
      }),
    ])
      .then(([regionDict, organization]) => {
        this.setState({
          regionDict,
          organization,
          province: organization.province.id,
        });
        return this.getCityDict(organization.province.id).then(() => {
          this.setState({ city: organization.city.id });
          return this.getSchoolDict(organization.city.id);
        });
      })
      .then(() => {
        this.setState({ loading: false });
      });
  }

  render() {
    const { onClose } = this.props;
    const { loading, province, city, schoolId, regionDict, cityDict, schoolDict } = this.state;
    return (
      <Modal loading={loading} title="学校选择" onCancel={onClose} okText="保存" onOk={this.onCommit}>
        <FormItem label="省份" labelCol={{ span: 5 }} wrapperCol={{ span: 12 }}>
          <Select
            placeholder="请选择省份"
            showSearch
            size="small"
            value={province}
            dict={regionDict}
            onChange={(val) => {
              this.getCityDict(val);
              this.setState({ province: val });
            }}
          />
        </FormItem>
        <FormItem label="城市" labelCol={{ span: 5 }} wrapperCol={{ span: 12 }}>
          <Select
            placeholder="请选择城市"
            showSearch
            size="small"
            value={city}
            dict={cityDict}
            onChange={(val) => {
              this.getSchoolDict(val);
              this.setState({ city: val });
            }}
          />
        </FormItem>
        <FormItem label="就读学校" labelCol={{ span: 5 }} wrapperCol={{ span: 12 }}>
          <Select
            placeholder="请选择学校"
            showSearch
            size="small"
            value={schoolId}
            dict={schoolDict}
            textInValue
            onChange={(value) => {
              if (value) {
                this.setState({
                  schoolId: value.val,
                  schoolName: value.text,
                });
              } else {
                this.setState({
                  schoolId: '',
                  schoolName: '',
                });
              }
            }}
          />
        </FormItem>
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    user: state.account.user,
  };
};
export default connect(mapStateToProps)(SchoolModal);
