import React from 'react';
import { connect } from 'dva';
import moment from 'moment';
import Page from 'COMPONENT/Page';
import Block from 'COMPONENT/Block';
import Button from 'COMPONENT/Button';
import TextArea from 'COMPONENT/TextArea';
import DatePicker from 'COMPONENT/DatePicker';
import Select from 'COMPONENT/Select';
import Section from 'COMPONENT/Section';
import TextLink from 'COMPONENT/TextLink';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { Form, Input, Checkbox, Row, Col, Tooltip } from 'antd';
import { loadCommonDictNew, loadAllBranchCampusDict, getEntranceDict } from 'UTIL/dict';
import { loadSourceDetailDict } from 'PAGE/common/loaders';
import loaders from './loaders';
import SchoolModal from './SchoolModal';
import Icon from 'COMPONENT/Icon';
import styles from './styles.less';
import { loginUser, isIdCard, validIdentityCardByType } from 'UTIL/common';
import { getSelectOption } from 'SERVICE/common';

const FormItem = Form.Item;
const CheckboxGroup = Checkbox.Group;
const PARENT_CLUB_DICT_VALUE = 'PARENT_CLUB';
const sexDict = [
  { val: '1', text: '男' },
  { val: '0', text: '女' },
];
const formItemCol = {
  labelCol: { span: 8 },
  wrapperCol: { span: 15 },
};
const linkFormCfg = {
  labelCol: { span: 5 },
  wrapperCol: { span: 12 },
  style: {
    padding: '0 10px',
    marginBottom: 10,
    lineHeight: '24px',
  },
};
const phoneRegExp = /^1\d{10}$/;

const resTypeDictGroups = {
  GROUNDPROMOTION: [
    { text: '市场推广', val: 'TEL_CONSULTATION' },
    { text: '渠道公关', val: 'GROUND_CALLING' },
    { text: '活动策划', val: 'OUT_CALLING' },
    { text: '户外广告投放', val: 'INTERNET_CALLING' },
  ],
  RES_ENTRANCE_MARKET: [{ text: '市场流量', val: 'RES_TYPE_MARKET' }],
  CALL_OUT: [{ text: '数据库数据', val: 'STRANGE_CALL' }],
  NEW_MEDIA: [{ text: '新媒体', val: 'b07e325eadf52ffc67d615ddc08e344d' }],
  ON_LINE_LOCAL: [{ text: '网络推广', val: 'INTERNET' }],
  CALL_IN: [
    { text: '短信', val: 'VISIT' },
    { text: '市场推广', val: 'TEL_CONSULTATION' },
    { text: '渠道公关', val: 'GROUND_CALLING' },
    { text: '活动策划', val: 'OUT_CALLING' },
    { text: '户外广告投放', val: 'INTERNET_CALLING' },
    { text: '转介绍', val: 'INTRODUCE' },
    { text: '新媒体', val: 'b07e325eadf52ffc67d615ddc08e344d' },
    { text: '网络推广', val: 'INTERNET' },
  ],
  PARENT_CLUB: [
    { text: '市场推广', val: 'TEL_CONSULTATION' },
    { text: '渠道公关', val: 'GROUND_CALLING' },
    { text: '活动策划', val: 'OUT_CALLING' },
    { text: '户外广告投放', val: 'INTERNET_CALLING' },
  ],
  [PARENT_CLUB_DICT_VALUE]: [{ text: '家长俱乐部', val: 'CLUB_OF_PARENT' }],
};

const getResTypeDictByEntrance = (entrance) => {
  if (entrance) {
    return (
      resTypeDictGroups[entrance] ||
      Object.values(resTypeDictGroups).reduce((result, current) => result.concat(current), [])
    );
  }
  return [];
};

class AddCustomer extends React.Component {
  constructor(props) {
    super(props);

    this.dicts = {
      followerDict: [],
      studentGradeDict: [],
      resTypeDict: [],
      cusOrgDict: [],
      entranceDict: [],
      identityTypeList: [],
    };

    this.state = Object.assign(this.dicts, {
      loading: true,
      disabled: false,
      organization: {},
      studentList: [],
      studentListSelectable: [],
      checkAll: false,
      newStudents: [0],
      oldStudents: [],
      schoolModalVisible: false,
      schoolModalData: {},
      oldStudentLoading: false,
    });

    this.schoolInput = [];
  }

  handleSubmit = (e) => {
    e.preventDefault();
    this.props.form.validateFieldsAndScroll((err, values) => {
      if (err) {
        return showMessage('保存失败：请确认必填输入项是否填写完毕', 'error');
      }
      let params = Object.assign({}, values);
      params = Object.keys(params).reduce((obj, ele) => {
        obj[ele] = typeof params[ele] === 'string' ? params[ele].trim() : params[ele];
        return obj;
      }, {});

      let validIdentityData = validIdentityCardByType(params.idCardType, params.idCardNo);
      if (!validIdentityData.valid) {
        return showMessage('证件格式有误，请检查证件类型及号码', 'error');
      }

      params['deliverTarget'] = params['deliverTarget'].substring(0, params['deliverTarget'].indexOf('-'));

      let studentImportVos = params['studentImportVos'] ? params['studentImportVos'] : [];
      studentImportVos = studentImportVos
        .filter((ele) => !!ele)
        .reduce((arr, ele, i) => {
          let tmp = Object.assign({}, ele);
          tmp = Object.keys(tmp).reduce((obj, ele) => {
            obj[ele] = typeof tmp[ele] === 'string' ? tmp[ele].trim() : tmp[ele];
            return obj;
          }, {});

          if (i === 0 && this.state.oldStudents.length === 0 && !tmp.name) {
            tmp.name = params.name;
            tmp.contact = params.contact;
          }

          if (tmp.name) {
            arr.push(tmp);
          }
          return arr;
        }, []);

      const studentTotal = [
        ...studentImportVos,
        ...this.state.studentList.map((ele) => {
          return {
            name: ele.studentName.trim(),
          };
        }),
      ];

      studentImportVos = [
        ...studentImportVos,
        ...this.state.oldStudents.map((ele) => {
          return {
            gradeId: ele.gradeId || '',
            name: ele.studentName.trim(),
            schoolId: ele.schoolId || '',
            id: ele.studentId,
          };
        }),
      ];

      if (studentImportVos.length <= 0) {
        return showMessage('保存失败：请确认学员档案是否填写完毕', 'error');
      }

      const repeatName = studentTotal.map((ele) => ele.name).filter((ele, index, arr) => arr.indexOf(ele) !== index);
      if (repeatName.length) {
        return showMessage(`保存失败：选中学员姓名重复（${repeatName.join('、')}）`, 'error');
      }

      if (studentImportVos.length) {
        let tempIndex = 0;
        for (let value of studentImportVos) {
          ++tempIndex;
          if (value.idCardNo && !isIdCard(value.idCardNo)) {
            break;
          }
        }
        let tempStatus = studentImportVos.every((value) => !value.idCardNo || isIdCard(value.idCardNo));
        if (!tempStatus) {
          return showMessage(`保存失败：学员${tempIndex}身份证格式错误`);
        }
      }

      this.saveCustomer({ ...params, studentImportVos: studentImportVos });
    });
  };

  saveCustomer = async (params) => {
    try {
      this.setState({ loading: true });
      const res = await loaders.saveOrUpdateCustomer({
        ...params,
        callType: 'CUSTOMER_MANAGER',
        workBenchType: 'CUSTOMER_MANAGER',
      });
      if (res.resultCode != 0) {
        throw new Error(res.resultMessage);
      }
      showMessage('保存成功', 'success');
      this.reset();
    } catch (e) {
      console.log(e);
      showMessage('保存失败：' + e.text, 'error');
    }
    this.setState({ loading: false });
  };

  getDisabledDate = (date) => {
    if (!date) {
      return false;
    }
    return moment(date).valueOf() > moment(new Date()).valueOf();
  };

  getCusTypeDict = (entranceId) => {
    if (!entranceId) {
      this.setState({ resTypeDict: [], cusOrgDict: [] });
      this.props.form.setFieldsValue({
        cusType: '',
        cusOrg: '',
      });
      return;
    }
    const resTypeDict = getResTypeDictByEntrance(entranceId);
    this.setState({ resTypeDict, cusOrgDict: [] });
    this.props.form.setFieldsValue({ cusType: '', cusOrg: '' });
  };

  getCusOrgDict = (resTypeId) => {
    if (!resTypeId) {
      this.setState({ cusOrgDict: [] });
      this.props.form.setFieldsValue({
        cusOrg: '',
      });
      return;
    }
    loadSourceDetailDict(resTypeId).then((cusOrgDict) => {
      this.setState({ cusOrgDict });
      this.props.form.setFieldsValue({ cusOrg: '' });
    });
  };

  getStudentsOnChange = (e) => this.getStudents(e.target.value);

  getStudents = async (val) => {
    if (phoneRegExp.test(val)) {
      this.setState({ loading: true, oldStudentLoading: true });
      const res = await loaders.studentList({ contact: val });
      const studentList = res.data.studentList || [];
      this.setState({
        studentList,
        studentListSelectable: studentList.filter((ele) => ele.canFollow),
        oldStudentLoading: false,
        loading: false,
      });
    } else {
      this.setState({
        studentList: [],
        studentListSelectable: [],
      });
    }
    this.setState({
      oldStudents: [],
    });
  };

  onCheckBoxChange = (val) => {
    this.setState({
      oldStudents: val,
      checkAll: val.length === this.state.studentListSelectable.length,
    });
  };

  increaseStudend = () => {
    const students = [...this.state.newStudents];
    students.push(students.length ? students[students.length - 1] + 1 : 0);
    this.setState({ newStudents: students });
  };

  decreaseStudend = (i) => {
    const students = [...this.state.newStudents];
    this.setState({
      newStudents: [...students.slice(0, i), ...students.slice(i + 1)],
    });
  };

  schoolModalCommit = (params) => {
    const index = this.state.schoolModalData.index;
    this.props.form.setFieldsValue({
      ['studentImportVos[' + index + '].schoolName']: params.schoolName,
      ['studentImportVos[' + index + '].schoolId']: params.schoolId,
    });
  };

  reset = () => {
    this.props.form.resetFields();
    this.props.form.setFieldsValue({ remark: '' });
    this.setState({
      newStudents: [0],
      oldStudents: [],
      studentList: [],
    });
  };

  emitEmpty = (index) => {
    return () => {
      this.props.form.setFieldsValue({
        ['studentImportVos[' + index + '].schoolName']: '',
        ['studentImportVos[' + index + '].schoolId']: '',
      });
    };
  };

  fetchIdentityType = async () => {
    try {
      const res = await getSelectOption({ selectOptionCategory: 'CERTIFICATE_TYPE' });
      const identityType = res.value || {};
      const identityTypeList =
        this.state.customerIdCardType && this.state.customerIdCardType !== 'CT_IDENTITY_CARD'
          ? Object.keys(identityType)
              .filter((key) => key)
              .map((key) => {
                return {
                  val: key,
                  text: identityType[key],
                };
              })
          : [{ text: '身份证', val: 'CT_IDENTITY_CARD' }];
      this.setState({
        identityTypeList,
      });
    } catch (e) {
      showMessage('获取证件类型失败', 'error');
    }
  };

  componentDidMount() {
    Promise.all([
      loadCommonDictNew('STUDENT_GRADE'),
      loadAllBranchCampusDict(),
      loaders.organization({
        id: this.props.user.campusId || this.props.user.brenchId,
      }),
      loaders.loadFollowerDict({
        showBelong: 'manage',
        job: ['ZXS', 'ZXZG'],
        monitoringUserId: this.props.user.userId,
        monitoringUserName: this.props.user.name,
        monitoringHtml: 'inputCustomer',
        monitoringTime: moment().format('YYYY-MM-DD HH:mm:ss'),
      }),
      getEntranceDict('RES_ENTRANCE', `CustomerAddCustomer,${this.props.user.brenchId}`),
    ]).then(([studentGradeDict, campusDict, organization, followerDict, entranceDict]) => {
      followerDict = followerDict.map((group) => {
        group.options = group.options.map((option) => {
          option.val = option.val + '-' + option.group + '-' + option.text;
          return option;
        });
        return group;
      });

      this.setState({
        studentGradeDict,
        campusDict,
        organization,
        followerDict,
        entranceDict,
        loading: false,
      });
    });

    this.fetchIdentityType();
  }

  getToolTipText = ({ status, finishDay, studentName, blCampusName }) => {
    return {
      CLASSING: `【学生：${studentName}/学籍：${blCampusName}/状态：上课中】不满足激活条件，请到【学生列表】操作签单`,
      STOP_CLASS: `【学生：${studentName}/学籍：${blCampusName}/状态：停课${
        finishDay || 0
      }天】不满足激活条件，请到【学生列表】操作签单`,
      FINISH_CLASS: `【学生：${studentName}/学籍：${blCampusName}/状态：结课${
        finishDay || 0
      }天】不满足激活条件，请到【学生列表】操作签单`,
    }[status];
  };

  render() {
    const { organizationId } = this.props.user;
    const { getFieldDecorator, getFieldValue } = this.props.form;
    const {
      loading,
      studentGradeDict,
      resTypeDict,
      followerDict,
      cusOrgDict,
      studentList,
      studentListSelectable,
      checkAll,
      newStudents,
      oldStudents,
      schoolModalVisible,
      oldStudentLoading,
      disabled,
    } = this.state;
    return (
      <Page loading={loading} title="客户录入">
        <Form onSubmit={this.handleSubmit}>
          <Block title="客户信息">
            <Row type="flex" justify="start">
              <Col span={8}>
                <FormItem label="客户姓名" {...formItemCol}>
                  {getFieldDecorator('name', {
                    rules: [{ required: true, message: '请填写客户姓名' }],
                  })(<Input />)}
                </FormItem>
              </Col>
              <Col span={8}>
                <FormItem label="联系方式" {...formItemCol}>
                  {getFieldDecorator('contact', {
                    rules: [
                      {
                        required: true,
                        pattern: phoneRegExp,
                        message: '请填写正确的联系方式',
                      },
                    ],
                  })(<Input onChange={this.getStudentsOnChange} />)}
                </FormItem>
              </Col>
            </Row>
            <Row type="flex" justify="start">
              <Col span={8}>
                <FormItem label="资源入口" {...formItemCol}>
                  {getFieldDecorator('resEntranceId', {
                    rules: [{ required: true, message: '资源入口' }],
                  })(
                    <Select
                      placeholder="资源入口"
                      dict={this.state.entranceDict}
                      onChange={(val) => {
                        this.getCusTypeDict(val);
                      }}
                    />,
                  )}
                </FormItem>
              </Col>
              <Col span={8}>
                <FormItem label="资源来源" {...formItemCol}>
                  {getFieldDecorator('cusType', {
                    rules: [{ required: true, message: '资源来源' }],
                  })(
                    <Select
                      placeholder="资源来源"
                      dict={resTypeDict}
                      onChange={(val) => {
                        this.getCusOrgDict(val);
                      }}
                    />,
                  )}
                </FormItem>
              </Col>
            </Row>
            <Row type="flex" justify="start">
              <Col span={8}>
                <FormItem label="来源细分" {...formItemCol}>
                  {getFieldDecorator('cusOrg', {
                    rules: [{ required: true, message: '来源细分' }],
                  })(<Select placeholder="来源细分" dict={cusOrgDict} />)}
                </FormItem>
              </Col>
              <Col span={8}>
                {' '}
                <FormItem label="分配跟进人" {...formItemCol}>
                  {getFieldDecorator('deliverTarget', {
                    rules: [{ required: true, message: '分配跟进人' }],
                  })(
                    <Select
                      placeholder="请选择跟进人"
                      showSearch
                      dict={followerDict}
                      filter={(searchKey, option) => {
                        return option.val.indexOf(searchKey) >= 0;
                      }}
                    />,
                  )}
                </FormItem>
              </Col>
            </Row>
            {loginUser.isXingHuoSym() && (
              <Row type="flex" justify="start">
                <Col span={8}>
                  <FormItem label="证件类型" {...formItemCol}>
                    {getFieldDecorator(
                      'idCardType',
                      {},
                    )(<Select placeholder="请选择证件类型" dict={this.state.identityTypeList} />)}
                  </FormItem>
                </Col>

                <Col span={8}>
                  <FormItem label="证件号码" {...formItemCol}>
                    {getFieldDecorator('idCardNo', {})(<Input />)}
                  </FormItem>
                </Col>
              </Row>
            )}
            <Row type="flex" justify="start">
              <Col span={8}>
                <FormItem label="备注" {...formItemCol}>
                  {getFieldDecorator('remark', {})(<TextArea />)}
                </FormItem>
              </Col>
            </Row>
          </Block>
          {studentList.length <= 0 ? (
            ''
          ) : (
            <Section
              loading={oldStudentLoading}
              title="关联学生"
              style={{ paddingBottom: '30px', marginBottom: '30px' }}
              toolBar={
                <Checkbox
                  checked={checkAll}
                  onChange={(e) => {
                    this.setState({
                      checkAll: !checkAll,
                      oldStudents: e.target.checked ? [...studentListSelectable] : [],
                    });
                  }}
                >
                  全部跟进
                </Checkbox>
              }
            >
              <div className={styles.tipInfo}>
                <span>同一学生，请勾选关联并保存即可</span>
              </div>
              {studentList.length <= 0 ? (
                <div style={{ height: '60px', textAlign: 'center' }}>无关联学生</div>
              ) : (
                <CheckboxGroup onChange={this.onCheckBoxChange} value={oldStudents}>
                  {studentList.map((ele) => {
                    const toolTipText = this.getToolTipText(ele);
                    const tag = ele.studentType === 'ENROLLED' && !ele.canFollow && '在读学生';
                    return (
                      <Row key={ele.studentId} style={{ marginLeft: 20 }}>
                        <Col span={1} className={styles.xsCheckboxCol}>
                          {ele.canFollow ? (
                            <Checkbox value={ele} />
                          ) : toolTipText ? (
                            <Tooltip placement="right" title={toolTipText} overlayClassName={styles.toolTipWrapper}>
                              <Checkbox disabled />
                            </Tooltip>
                          ) : (
                            <Checkbox disabled />
                          )}
                        </Col>
                        <Col span={6}>
                          <FormItem label="学生" {...linkFormCfg}>
                            <div
                              style={{
                                wordBreak: 'break-word',
                                display: 'flex',
                                justifyContent: 'center',
                                alignItems: 'baseline',
                              }}
                            >
                              <span style={{ wordBreak: 'break-word', flex: '1' }}>
                                {ele.studentName}
                                {ele.haveLabel && <i style={{ color: 'red' }}>*</i>}
                              </span>
                              {tag && (
                                <span
                                  style={{
                                    marginLeft: '10px',
                                    border: '1px #ff8080 solid',
                                    padding: '0 4px',
                                    borderRadius: '4px',
                                    color: '#ff8080',
                                    display: 'inline-block',
                                    lineHeight: '20px',
                                    fontSize: '12px',
                                  }}
                                >
                                  {tag}
                                </span>
                              )}
                            </div>
                          </FormItem>
                        </Col>
                        <Col span={6}>
                          <FormItem label="年级" {...linkFormCfg}>
                            {ele.gradeName}
                          </FormItem>
                        </Col>
                        <Col span={6}>
                          <FormItem label="学校" {...linkFormCfg}>
                            {ele.schoolName}
                          </FormItem>
                        </Col>
                      </Row>
                    );
                  })}
                </CheckboxGroup>
              )}
            </Section>
          )}
          <Section title="学员档案" style={{ paddingBottom: '30px', marginBottom: '30px' }}>
            {newStudents.length <= 0 ? (
              <div style={{ height: '60px', textAlign: 'center' }}>请新增学员</div>
            ) : (
              newStudents.map((ele, i) => {
                return (
                  <Section
                    title={'学员' + (i + 1)}
                    style={{
                      paddingBottom: '30px',
                      marginBottom: '30px',
                    }}
                    toolBar={<TextLink onClick={() => this.decreaseStudend(i)}>删除学员</TextLink>}
                    key={ele}
                  >
                    <Row type="flex" justify="start">
                      <Col span={8}>
                        <FormItem label="学员姓名" {...formItemCol}>
                          {getFieldDecorator('studentImportVos[' + ele + '].name', {})(<Input />)}
                        </FormItem>
                      </Col>
                      <Col span={8}>
                        <FormItem label="学员电话" {...formItemCol}>
                          {getFieldDecorator('studentImportVos[' + ele + '].contact', {
                            rules: [
                              {
                                pattern: phoneRegExp,
                                message: '请填写正确的联系方式',
                              },
                            ],
                          })(<Input />)}
                        </FormItem>
                      </Col>
                    </Row>
                    <Row type="flex" justify="start">
                      <Col span={8}>
                        <FormItem label="性别" {...formItemCol}>
                          {getFieldDecorator(
                            'studentImportVos[' + ele + '].sex',
                            {},
                          )(<Select placeholder="请选择" dict={sexDict} />)}
                        </FormItem>
                      </Col>
                      <Col span={8}>
                        <FormItem label="出生日期" {...formItemCol}>
                          {getFieldDecorator(
                            'studentImportVos[' + ele + '].bothday',
                            {},
                          )(<DatePicker size="small" placeholder="出生日期" disabledDate={this.getDisabledDate} />)}
                        </FormItem>
                      </Col>
                    </Row>
                    <Row type="flex" justify="start">
                      <Col span={8}>
                        <FormItem label="就读学校" {...formItemCol}>
                          {getFieldDecorator(
                            'studentImportVos[' + ele + '].schoolName',
                            {},
                          )(
                            <Input
                              autoComplete="off"
                              onClick={() => {
                                this.setState({
                                  schoolModalData: { index: ele },
                                  schoolModalVisible: true,
                                });
                              }}
                              suffix={
                                getFieldValue('studentImportVos[' + ele + '].schoolId') ? (
                                  <Icon
                                    type="cross_circle"
                                    style={{
                                      color: '#6d67ff',
                                    }}
                                    onClick={this.emitEmpty(ele)}
                                  />
                                ) : null
                              }
                              style={{ caretColor: 'transparent' }}
                            />,
                          )}
                        </FormItem>
                        <div style={{ display: 'none' }}>
                          <FormItem>
                            {getFieldDecorator('studentImportVos[' + ele + '].schoolId', {})(<Input />)}
                          </FormItem>
                        </div>
                      </Col>
                      <Col span={8}>
                        <FormItem label="就读年级" {...formItemCol}>
                          {getFieldDecorator(
                            'studentImportVos[' + ele + '].gradeId',
                            {},
                          )(<Select placeholder="请选择年级" showSearch dict={studentGradeDict} />)}
                        </FormItem>
                      </Col>
                    </Row>
                    <Row type="flex" justify="start">
                      <Col span={8}>
                        <FormItem label="父亲姓名" {...formItemCol}>
                          {getFieldDecorator('studentImportVos[' + ele + '].fatherName', {})(<Input />)}
                        </FormItem>
                      </Col>
                      <Col span={8}>
                        <FormItem label="父亲联系方式" {...formItemCol}>
                          {getFieldDecorator('studentImportVos[' + ele + '].fatherPhone', {
                            rules: [
                              {
                                pattern: phoneRegExp,
                                message: '请填写正确的联系方式',
                              },
                            ],
                          })(<Input />)}
                        </FormItem>
                      </Col>
                    </Row>
                    <Row type="flex" justify="start">
                      <Col span={8}>
                        <FormItem label="母亲姓名" {...formItemCol}>
                          {getFieldDecorator('studentImportVos[' + ele + '].motherName', {})(<Input />)}
                        </FormItem>
                      </Col>
                      <Col span={8}>
                        <FormItem label="母亲联系方式" {...formItemCol}>
                          {getFieldDecorator('studentImportVos[' + ele + '].notherPhone', {
                            rules: [
                              {
                                pattern: phoneRegExp,
                                message: '请填写正确的联系方式',
                              },
                            ],
                          })(<Input />)}
                        </FormItem>
                      </Col>
                    </Row>
                    <Row type="flex" justify="start">
                      <Col span={8}>
                        <FormItem label="学员位置" {...formItemCol}>
                          {getFieldDecorator(
                            'studentImportVos[' + ele + '].address',
                            {},
                          )(
                            <Input
                              placeholder="请按省、市、区/县（乡/镇 村）、街道、门牌号的格式填写"
                              title="请按省、市、区/县（乡/镇 村）、街道、门牌号的格式填写"
                            />,
                          )}
                        </FormItem>
                      </Col>
                      {loginUser.isXingHuoSym() && (
                        <Col span={8}>
                          <FormItem label="学生身份证" {...formItemCol}>
                            {getFieldDecorator('studentImportVos[' + ele + '].idCardNo', {})(<Input />)}
                          </FormItem>
                        </Col>
                      )}
                    </Row>
                  </Section>
                );
              })
            )}
          </Section>
          <Block>
            <Row type="flex" justify="start">
              <Col offset={22} span={2}>
                <FormItem wrapperCol={{ span: 12, offset: 5 }}>
                  <Button htmlType="button" onClick={this.increaseStudend}>
                    增加学员
                  </Button>
                </FormItem>
              </Col>
            </Row>
            <Row type="flex" justify="start">
              <Col offset={10} span={2}>
                <FormItem wrapperCol={{ span: 12, offset: 5 }}>
                  <Button type="primary" htmlType="submit" disabled={disabled}>
                    保存
                  </Button>
                </FormItem>
              </Col>
              <Col span={2}>
                <FormItem wrapperCol={{ span: 12, offset: 5 }}>
                  <Button htmlType="button" onClick={this.reset}>
                    重置
                  </Button>
                </FormItem>
              </Col>
            </Row>
          </Block>
        </Form>
        {schoolModalVisible && (
          <SchoolModal
            orgId={organizationId}
            onCommit={this.schoolModalCommit}
            onClose={() => this.setState({ schoolModalVisible: false })}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const data = state.main.pageParams['/CustomerAddCustomer'];
  return {
    user: state.account.user,
  };
};

export default connect(mapStateToProps)(Form.create()(AddCustomer));
