import { findOrganizationById, getSchoolByBlCampus, getRegionList, getReceptionistGroupByCampus } from 'SERVICE/common';
import {
  getCitys,
  getStudentListByCusContact,
  saveOrUpdateCustomer,
  getRelatedStudentsByCusId,
  saveFollowRecord,
} from 'SERVICE/customer';

const regionList = async () => {
  const res = await getRegionList();
  return res.map((ele) => {
    return { val: ele.id, text: ele.name };
  });
};

const organization = async (params) => {
  return await findOrganizationById(params);
};

const cityList = async (params) => {
  const res = await getCitys(params);
  return res.map((ele) => {
    return { val: ele.id, text: ele.name };
  });
};

const schoolList = async (params) => {
  const res = await getSchoolByBlCampus(params);
  const value = res.value || {};
  return Object.keys(value).map((ele) => {
    return { val: ele, text: value[ele] };
  });
};

const studentList = async (params) => {
  return await getStudentListByCusContact(params);
};

const relatedStudents = async (params) => {
  return await getRelatedStudentsByCusId(params);
};

const loadFollowerDict = async (params) => {
  const res = await getReceptionistGroupByCampus({ ...params });
  const dictObj = res.reduce((obj, ele) => {
    if (!obj[ele.orgId]) {
      obj[ele.orgId] = {
        label: ele.orgName,
        options: [],
      };
    }
    obj[ele.orgId].options.push({
      val: ele.userId,
      text: ele.userName,
      group: ele.orgName,
    });
    return obj;
  }, {});
  return Object.keys(dictObj).reduce((arr, ele) => {
    arr.push(dictObj[ele]);
    return arr;
  }, []);
};

export default {
  regionList,
  organization,
  cityList,
  schoolList,
  studentList,
  saveOrUpdateCustomer,
  relatedStudents,
  saveFollowRecord,
  loadFollowerDict,
};
