import React from 'react';
import { connect } from 'dva';
import PropTypes from 'prop-types';
import { Loading, Block, Button, TextArea, Select, Icon } from 'COMPONENT';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { Form, Input, Row, Col } from 'antd';
import { loadCommonDict, loadCommonDictNew, loadAllBranchCampusDict } from 'UTIL/dict';
import loaders from './loaders';
import SchoolModal from './SchoolModal';
import styles from './styles.less';
import debounce from 'lodash/debounce';
import { searchUser } from 'SERVICE/customer';
import { errorsHandle, runFunc, loginUser, isIdCard, validIdentityCardByType } from 'UTIL/common';
import { getSelectOption } from 'SERVICE/common';

const FormItem = Form.Item;
const formItemCol = {
  labelCol: { span: 6 },
  wrapperCol: { span: 16 },
};
const phoneRegExp = /^1\d{10}$/;

function extendFormItemCfg(origin, extend = {}) {
  for (let key in extend) {
    if (origin.hasOwnProperty(key)) {
      for (let subKey in extend[key]) {
        if (origin[key].hasOwnProperty(subKey)) {
          origin[key][subKey] = extend[key][subKey];
        }
      }
    }
  }
  return origin;
}

class AddTransferCustomer extends React.Component {
  constructor(props) {
    super(props);

    this.formItemCfg = extendFormItemCfg(
      {
        // 学生姓名
        pointialStuName: {
          initialValue: '',
          required: true,
          visible: true,
          disabled: false,
        },
        // 家长姓名
        name: {
          initialValue: '',
          required: true,
          visible: true,
          disabled: false,
        },
        // 联系方式
        contact: {
          initialValue: '',
          required: true,
          visible: true,
          disabled: false,
        },
        // 就读学校
        schoolName: {
          initialValue: '',
          required: true,
          visible: true,
          disabled: false,
        },
        // 学生年级
        pointialStuGrade: {
          initialValue: null,
          required: true,
          visible: true,
          disabled: false,
        },
        // 介绍类型
        cusOrg: {
          initialValue: null,
          required: true,
          visible: true,
          disabled: false,
        },
        // 意向学习中心
        intentionCampusId: {
          initialValue: null,
          required: true,
          visible: true,
          disabled: false,
        },
        // 客户情况
        remark: {
          initialValue: '',
          required: false,
          visible: true,
          disabled: false,
        },
        // 谁介绍
        introducer: {
          initialValue: '',
          required: true,
          visible: true,
          disabled: false,
        },
        // 联系方式
        introducerContact: {
          initialValue: '',
          required: true,
          visible: true,
          disabled: false,
        },
        // 奖金归属人
        bonusBelongId: {
          initialValue: null,
          required: true,
          visible: true,
          disabled: false,
        },
        // 被介绍人奖励
        customerBonusRemark: {
          initialValue: '',
          required: false,
          visible: true,
          disabled: false,
        },
        // 介绍人奖励
        introduceBonusRemark: {
          initialValue: '',
          required: false,
          visible: true,
          disabled: false,
        },
        // 归属员工奖励
        belongBonusRemark: {
          initialValue: '',
          required: false,
          visible: true,
          disabled: false,
        },
      },
      props.formItemCfg,
    );

    this.handleBonusPersonSearch = debounce(this.onBonusPersonSearch, 1000);

    this.dicts = {
      studentGradeDict: [],
      campusDict: [],
      cusOrgDict: [],
      bonusPersonDict: [],
    };

    this.state = Object.assign(this.dicts, {
      loading: true,
      disabled: false,
      schoolModalVisible: false,
      intentionCampusDisabled: false,
      identityTypeList: [],
    });

    this.defaultIntentionCampus = null;

    this.bonusPersonStorage = {};

    this.formRef = null;
  }

  static propTypes = {
    btnGroupVisible: PropTypes.bool,
    afterConfirm: PropTypes.func,
    /**
     * formItemCfg: {
     *   "keyName": {
     *     initialValue: PropTypes.any,
     *     required: PropTypes.bool,
     *     disabled: PropTypes.bool,
     *     visible: PropTypes.bool,
     *   },
     * }
     * */
    formItemCfg: PropTypes.object,
  };

  static defaultProps = {
    btnGroupVisible: true,
    formItemCfg: {},
  };

  save = async (params) => {
    this.setState({ loading: true });
    try {
      const res = await loaders.saveTransferCustomer({
        ...params,
      });
      if (res.resultCode != 0) {
        throw new Error(res.resultMessage);
      }
      showMessage('提交审核成功', 'success');

      if (this.props.afterConfirm) {
        // 仅用于Modal模式
        runFunc(this.props.afterConfirm);
      } else {
        this.onResetClick();
      }
    } catch (e) {
      console.log(e);
      showMessage('提交审核失败：' + e.text, 'error');
    }
    this.setState({ loading: false });
  };

  onFormSubmit = (e) => {
    e.preventDefault();
    this.props.form.validateFieldsAndScroll((err, values) => {
      if (err) {
        return showMessage('提交审核失败：请确认必填输入项是否填写完毕', 'error');
      }
      const params = Object.assign(
        {
          cusType: 'INTRODUCE',
        },
        values,
      );
      delete params.schoolName;
      params.pointialStuName = params.pointialStuName.trim();
      params.name = params.name.trim();
      params.introducer = params.introducer.trim();

      let validIdentityData = validIdentityCardByType(params.idCardType, params.idCardNo);
      if (!validIdentityData.valid) {
        return showMessage('证件格式有误，请检查证件类型及号码', 'error');
      }

      this.save(params);
    });
  };

  onSchoolModalCommit = (params) => {
    this.props.form.setFieldsValue({
      schoolName: params.schoolName,
      pointialStuSchoolId: params.schoolId,
    });
  };

  onResetClick = () => {
    this.props.form.resetFields();
  };

  onClearSchoolClick = () => {
    this.props.form.setFieldsValue({
      schoolName: '',
      pointialStuSchoolId: '',
    });
  };

  onBonusPersonSearch = async (val) => {
    if (val) {
      if (this.bonusPersonStorage[val]) {
        this.setState({ bonusPersonDict: this.bonusPersonStorage[val] }, this.updateBonusPerson);
      } else {
        this.fundBonusPerson(val);
      }
    }
  };

  fundBonusPerson = async (name) => {
    this.setState({ loading: true });
    const rsp = await searchUser({ name }).catch(errorsHandle());
    if (rsp && Array.isArray(rsp.data)) {
      const dict = rsp.data.map((item) => ({
        text: `${item.username}<${item.employeeNo}>`,
        subText: item.orgInfo,
        val: item.userId,
      }));
      this.bonusPersonStorage[name] = dict;
      this.setState({ bonusPersonDict: dict }, this.updateBonusPerson);
    }
    this.setState({ loading: false });
  };

  updateBonusPerson = () => {
    const bonusBelongId = this.props.form.getFieldValue('bonusBelongId');
    const bonusBelongIndex = this.state.bonusPersonDict.findIndex((item) => item.val === bonusBelongId);
    if (bonusBelongIndex === -1) {
      this.props.form.setFieldsValue({ bonusBelongId: null });
    }
  };

  fetchIdentityType = async () => {
    try {
      const res = await getSelectOption({ selectOptionCategory: 'CERTIFICATE_TYPE' });
      const identityType = res.value || {};
      const identityTypeList =
        this.state.customerIdCardType && this.state.customerIdCardType !== 'CT_IDENTITY_CARD'
          ? Object.keys(identityType)
              .filter((key) => key)
              .map((key) => {
                return {
                  val: key,
                  text: identityType[key],
                };
              })
          : [{ text: '身份证', val: 'CT_IDENTITY_CARD' }];
      this.setState({
        identityTypeList,
      });
    } catch (e) {
      showMessage('获取证件类型失败', 'error');
    }
  };

  componentDidMount() {
    const { brenchId, campusId, organizationType } = this.props.user;

    this.fetchIdentityType();

    Promise.all([
      loadCommonDictNew('STUDENT_GRADE'),
      loadAllBranchCampusDict(),
      loadCommonDict('CUS_ORG', 'INTRODUCE'),
    ]).then(([studentGradeDict, campusDict, cusOrgDict]) => {
      if (organizationType !== 'GROUNP') {
        campusDict = brenchId ? campusDict.filter((branch) => branch.val === brenchId) : [];
      }
      this.setState({
        studentGradeDict,
        campusDict,
        cusOrgDict,
        loading: false,
      });
      if (organizationType === 'CAMPUS') {
        this.defaultIntentionCampus = campusId;
        this.formItemCfg.intentionCampusId.initialValue = campusId;
      }
      this.setState({
        intentionCampusDisabled: !!this.defaultIntentionCampus,
      });
    });
  }

  render() {
    const { organizationId } = this.props.user;
    const { getFieldDecorator, getFieldValue } = this.props.form;
    const { loading, studentGradeDict, campusDict, cusOrgDict, schoolModalVisible, disabled } = this.state;
    const formItemCfg = this.formItemCfg;
    return (
      <div>
        <Loading loading={loading} />
        <div>
          <Form ref={(c) => (this.formRef = c)} onSubmit={this.onFormSubmit}>
            <Block title="被介绍人信息" key="basic">
              <Row type="flex" justify="start">
                <Col span={8}>
                  <FormItem label="学生姓名" {...formItemCol}>
                    {getFieldDecorator('pointialStuName', {
                      initialValue: formItemCfg.pointialStuName.initialValue,
                      rules: [
                        { transform: (val) => (val ? val.trim() : val) },
                        { required: formItemCfg.pointialStuName.required, message: '请填写学生姓名' },
                      ],
                    })(<Input disabled={formItemCfg.pointialStuName.disabled} />)}
                  </FormItem>
                </Col>
                <Col span={8}>
                  <FormItem label="家长姓名" {...formItemCol}>
                    {getFieldDecorator('name', {
                      initialValue: formItemCfg.name.initialValue,
                      rules: [
                        { transform: (val) => (val ? val.trim() : val) },
                        { required: formItemCfg.name.required, message: '请填写家长姓名' },
                      ],
                    })(<Input disabled={formItemCfg.name.disabled} />)}
                  </FormItem>
                </Col>
              </Row>
              <Row type="flex" justify="start">
                <Col span={8}>
                  <FormItem label="联系方式" {...formItemCol}>
                    {getFieldDecorator('contact', {
                      initialValue: formItemCfg.contact.initialValue,
                      rules: [
                        {
                          required: formItemCfg.contact.required,
                          message: '请填写联系方式',
                        },
                        {
                          pattern: phoneRegExp,
                          message: '请填写正确的联系方式',
                        },
                      ],
                    })(<Input disabled={formItemCfg.contact.disabled} />)}
                  </FormItem>
                </Col>
                <Col span={8}>
                  <FormItem label="就读学校" {...formItemCol}>
                    {getFieldDecorator('schoolName', {
                      initialValue: formItemCfg.schoolName.initialValue,
                      rules: [{ required: formItemCfg.schoolName.required, message: '请选择就读学校' }],
                    })(
                      <Input
                        autoComplete="off"
                        onClick={() => {
                          this.setState({
                            schoolModalVisible: true,
                          });
                        }}
                        suffix={
                          getFieldValue('pointialStuSchoolId') ? (
                            <Icon
                              type="cross_circle"
                              style={{
                                color: '#6d67ff',
                              }}
                              onClick={this.onClearSchoolClick}
                            />
                          ) : null
                        }
                        style={{ caretColor: 'transparent' }}
                        disabled={formItemCfg.schoolName.disabled}
                      />,
                    )}
                  </FormItem>
                  <div style={{ display: 'none' }}>
                    <FormItem>{getFieldDecorator('pointialStuSchoolId', {})(<Input />)}</FormItem>
                  </div>
                </Col>
              </Row>
              <Row type="flex" justify="start">
                <Col span={8}>
                  <FormItem label="学生年级" {...formItemCol}>
                    {getFieldDecorator('pointialStuGrade', {
                      initialValue: formItemCfg.pointialStuGrade.initialValue,
                      rules: [{ required: formItemCfg.pointialStuGrade.required, message: '请选择学生年级' }],
                    })(
                      <Select
                        placeholder="请选择学生年级"
                        showSearch
                        dict={studentGradeDict}
                        disabled={formItemCfg.pointialStuGrade.disabled}
                      />,
                    )}
                  </FormItem>
                </Col>
                <Col span={8}>
                  <FormItem label="介绍类型" {...formItemCol}>
                    {getFieldDecorator('cusOrg', {
                      initialValue: formItemCfg.cusOrg.initialValue,
                      rules: [{ required: formItemCfg.cusOrg.required, message: '请选择介绍类型' }],
                    })(
                      <Select
                        placeholder="请选择介绍类型"
                        showSearch
                        dict={cusOrgDict}
                        disabled={formItemCfg.cusOrg.disabled}
                      />,
                    )}
                  </FormItem>
                </Col>
              </Row>
              <Row type="flex" justify="start">
                <Col span={8}>
                  <FormItem label="意向学习中心" {...formItemCol}>
                    {getFieldDecorator('intentionCampusId', {
                      initialValue: formItemCfg.intentionCampusId.initialValue,
                      rules: [{ required: formItemCfg.intentionCampusId.required, message: '请选择意向学习中心' }],
                    })(
                      <Select
                        placeholder="请选择意向学习中心"
                        showSearch
                        dict={campusDict}
                        disabled={formItemCfg.intentionCampusId.disabled}
                      />,
                    )}
                  </FormItem>
                </Col>
              </Row>
              {loginUser.isXingHuoSym() && (
                <Row type="flex" justify="start">
                  <Col span={8}>
                    <FormItem label="证件类型" {...formItemCol}>
                      {getFieldDecorator(
                        'idCardType',
                        {},
                      )(<Select placeholder="请选择证件类型" dict={this.state.identityTypeList} />)}
                    </FormItem>
                  </Col>

                  <Col span={8}>
                    <FormItem label="证件号码" {...formItemCol}>
                      {getFieldDecorator('idCardNo', {})(<Input />)}
                    </FormItem>
                  </Col>
                </Row>
              )}
              <Row type="flex" justify="start">
                <Col span={16}>
                  <FormItem label="客户情况" labelCol={{ span: 3 }} wrapperCol={{ span: 20 }}>
                    {getFieldDecorator('remark', {
                      initialValue: formItemCfg.remark.initialValue,
                      rules: [
                        { required: formItemCfg.remark.required, message: '请输入客户情况' },
                        { max: 200, message: '客户情况不能超过200字' },
                      ],
                    })(<TextArea maxLength={200} wordCount={true} disabled={formItemCfg.remark.disabled} />)}
                  </FormItem>
                </Col>
              </Row>
            </Block>

            <hr className={styles.hr} />

            <Block title="介绍人信息" key="intro">
              <Row type="flex" justify="start">
                <Col span={8}>
                  <FormItem label="谁介绍" {...formItemCol}>
                    {getFieldDecorator('introducer', {
                      initialValue: formItemCfg.introducer.initialValue,
                      rules: [
                        { transform: (val) => (val ? val.trim() : val) },
                        { required: formItemCfg.introducer.required, message: '请填写介绍人姓名' },
                      ],
                    })(<Input disabled={formItemCfg.introducer.disabled} />)}
                  </FormItem>
                </Col>
                <Col span={8}>
                  <FormItem label="联系方式" {...formItemCol}>
                    {getFieldDecorator('introducerContact', {
                      initialValue: formItemCfg.introducerContact.initialValue,
                      rules: [
                        { required: formItemCfg.introducerContact.required, message: '请填写联系方式' },
                        {
                          pattern: phoneRegExp,
                          message: '请填写正确的联系方式',
                        },
                      ],
                    })(<Input disabled={formItemCfg.introducerContact.disabled} />)}
                  </FormItem>
                </Col>
              </Row>
            </Block>

            <Block
              title={
                <span>
                  <span>转介绍奖金归属</span>
                  <span className={styles.subTitle}>
                    (转介绍奖金原则上归属于登记转介绍的员工，如果您是帮助他人登记转介绍，则奖金归属人需选择对应员工)
                  </span>
                </span>
              }
              key="bonus"
            >
              <Row type="flex" justify="start">
                <Col span={8}>
                  <FormItem label="归属人" {...formItemCol}>
                    {getFieldDecorator('bonusBelongId', {
                      initialValue: formItemCfg.bonusBelongId.initialValue,
                      rules: [{ required: formItemCfg.bonusBelongId.required, message: '请选择转介绍奖金归属人' }],
                    })(
                      <Select
                        placeholder="请选择员工"
                        dict={this.state.bonusPersonDict}
                        showSearch
                        searchPlaceholder="请先输入员工名称查找"
                        searchNotFoundTips="没有找到该员工"
                        onSearchKeyChange={this.handleBonusPersonSearch}
                        disabled={formItemCfg.bonusBelongId.disabled}
                      />,
                    )}
                  </FormItem>
                </Col>
              </Row>
            </Block>

            <Block title="奖励内容备注" key="remark">
              <Row type="flex" justify="start">
                <Col span={16}>
                  <FormItem label="被介绍人奖励" labelCol={{ span: 4 }} wrapperCol={{ span: 20 }}>
                    {getFieldDecorator('customerBonusRemark', {
                      initialValue: formItemCfg.customerBonusRemark.initialValue,
                      rules: [
                        { transform: (val) => (val ? val.trim() : val) },
                        { required: formItemCfg.customerBonusRemark.required, message: '请填写被介绍人奖励' },
                      ],
                    })(<TextArea maxLength={50} wordCount={true} />)}
                  </FormItem>
                </Col>
                <Col span={16}>
                  <FormItem label="介绍人奖励" labelCol={{ span: 4 }} wrapperCol={{ span: 20 }}>
                    {getFieldDecorator('introduceBonusRemark', {
                      initialValue: formItemCfg.introduceBonusRemark.initialValue,
                      rules: [
                        { transform: (val) => (val ? val.trim() : val) },
                        { required: formItemCfg.introduceBonusRemark.required, message: '请填写介绍人奖励' },
                      ],
                    })(<TextArea maxLength={50} wordCount={true} />)}
                  </FormItem>
                </Col>
                <Col span={16}>
                  <FormItem label="归属员工奖励" labelCol={{ span: 4 }} wrapperCol={{ span: 20 }}>
                    {getFieldDecorator('belongBonusRemark', {
                      initialValue: formItemCfg.belongBonusRemark.initialValue,
                      rules: [
                        { transform: (val) => (val ? val.trim() : val) },
                        { required: formItemCfg.belongBonusRemark.required, message: '请填写归属员工奖励' },
                      ],
                    })(<TextArea maxLength={50} wordCount={true} />)}
                  </FormItem>
                </Col>
              </Row>
            </Block>

            <Row type="flex" justify="start" style={{ display: this.props.btnGroupVisible ? 'flex' : 'none' }}>
              <Col span={16}>
                <div
                  style={{
                    display: 'flex',
                    justifyContent: 'center',
                  }}
                >
                  <FormItem wrapperCol={{ span: 12, offset: 5 }}>
                    <Button type="primary" htmlType="submit" disabled={disabled} id="AddTransferCustomer-submit-btn">
                      提交审核
                    </Button>
                  </FormItem>
                  <FormItem
                    wrapperCol={{ span: 12, offset: 5 }}
                    style={{
                      marginLeft: '20px',
                    }}
                  >
                    <Button htmlType="button" onClick={this.onResetClick}>
                      重置
                    </Button>
                  </FormItem>
                </div>
              </Col>
            </Row>
          </Form>
        </div>
        {schoolModalVisible && (
          <SchoolModal
            orgId={organizationId}
            onCommit={this.onSchoolModalCommit}
            onClose={() => this.setState({ schoolModalVisible: false })}
          />
        )}
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  const data = state.main.pageParams['/CustomerAddTransferCustomer'];
  return {
    user: state.account.user,
  };
};

export default connect(mapStateToProps)(Form.create()(AddTransferCustomer));
