import React from 'react';
import { connect } from 'dva';
import Modal from 'COMPONENT/Modal';
import { Form } from 'antd';
import Select from 'COMPONENT/Select';
import loaders from './loaders';

const FormItem = Form.Item;

class SchoolModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: true,
      province: '',
      city: '',
      schoolId: '',
      schoolName: '',
      organization: {},
      regionDict: [],
      cityDict: [],
      schoolDict: [],
      provinceDisabled: false,
      cityDisabled: false,
      schoolDisabled: false,
    };
  }

  onCommit = async () => {
    const { onClose, onCommit } = this.props;
    onCommit({
      schoolId: this.state.schoolId,
      schoolName: this.state.schoolName,
    });
    onClose();
  };

  /**
   * 选择学校事件
   * tips: Select组件打包后有奇怪行为，e.text值变为数组: ['text', undefined]
   *
   * @param e
   */
  onSchoolChange = (e) => {
    let schoolId = '';
    let schoolName = '';

    if (e) {
      let val = e.val;
      let text = e.text;

      if (this.calcIsArray(val)) {
        val = val.filter((v) => !!v).join(',');
      }

      if (this.calcIsArray(text)) {
        text = text.filter((v) => !!v).join(',');
      }

      schoolId = val;
      schoolName = text;
    }

    this.setState({
      schoolId,
      schoolName,
    });
  };
  /**
   * 判断值是否为数组
   *
   * @param value
   * @return {boolean}
   */
  calcIsArray(value) {
    return Object.prototype.toString.call(value) === '[object Array]';
  }

  getCityDict = async (params) => {
    this.setState({
      cityDisabled: true,
      schoolDisabled: true,
    });
    if (!params) {
      return this.setState({
        cityDict: [],
        city: '',
        schoolDict: [],
        schoolId: '',
        schoolName: '',
        cityDisabled: false,
        schoolDisabled: false,
      });
    }
    const res = await loaders.cityList({ provinceID: params });
    this.setState({
      cityDict: res,
      city: '',
      schoolDict: [],
      schoolId: '',
      schoolName: '',
      cityDisabled: false,
      schoolDisabled: false,
    });
  };

  getSchoolDict = async (params) => {
    this.setState({
      schoolDisabled: true,
    });
    if (!params) {
      return this.setState({
        schoolDict: [],
        schoolId: '',
        schoolName: '',
        schoolDisabled: false,
      });
    }
    const res = await loaders.schoolList({ blCampus: params });
    const schoolDict = res.filter((ele) => ele.val);
    this.setState({ schoolDict: schoolDict, schoolId: '', schoolName: '', schoolDisabled: false });
  };

  componentDidMount() {
    this.setState({ loading: true });
    const {
      orgId,
      user: { campusId, brenchId },
    } = this.props;

    let apiParams = {
      id: orgId || campusId || brenchId,
    };
    if (orgId) {
      apiParams.f = '';
    }

    Promise.all([loaders.regionList(), loaders.organization(apiParams)])
      .then(([regionDict, organization]) => {
        this.setState({
          regionDict,
          organization,
          province: organization.province.id,
        });
        return this.getCityDict(organization.province.id).then(() => {
          this.setState({ city: organization.city.id });
          return this.getSchoolDict(organization.city.id);
        });
      })
      .then(() => {
        this.setState({ loading: false });
      });
  }

  render() {
    const { onClose } = this.props;
    const {
      loading,
      province,
      city,
      schoolId,
      regionDict,
      cityDict,
      schoolDict,
      provinceDisabled,
      cityDisabled,
      schoolDisabled,
    } = this.state;
    return (
      <Modal loading={loading} title="学校选择" onCancel={onClose} okText="保存" onOk={this.onCommit}>
        <FormItem label="省份" labelCol={{ span: 5 }} wrapperCol={{ span: 12 }}>
          <Select
            placeholder="请选择省份"
            showSearch
            value={province}
            dict={regionDict}
            onChange={(val) => {
              this.getCityDict(val);
              this.setState({ province: val });
            }}
            disabled={provinceDisabled}
          />
        </FormItem>
        <FormItem label="城市" labelCol={{ span: 5 }} wrapperCol={{ span: 12 }}>
          <Select
            placeholder="请选择城市"
            showSearch
            value={city}
            dict={cityDict}
            onChange={(val) => {
              this.getSchoolDict(val);
              this.setState({ city: val });
            }}
            disabled={cityDisabled}
          />
        </FormItem>
        <FormItem label="就读学校" labelCol={{ span: 5 }} wrapperCol={{ span: 12 }}>
          <Select
            placeholder="请选择学校"
            showSearch
            value={schoolId}
            dict={schoolDict}
            textInValue
            onChange={this.onSchoolChange}
            disabled={schoolDisabled}
          />
        </FormItem>
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    user: state.account.user,
  };
};
export default connect(mapStateToProps)(SchoolModal);
