/**
 * 无效资源池页面
 */

import React, { Component } from 'react';
import { connect } from 'dva';
import { message } from 'antd';
import { Page, List, Modal, Select } from 'COMPONENT';
import SearchPage from './search';
import { namespace, menuID, orgTypeConfig } from './store/constant';

const { COL_WIDTH_M, COL_WIDTH_S } = List;

class InvalidResourcePool extends Component {
  state = {
    pagination: {
      current: 1,
      pageSize: 50,
    },
    loading: false,
    searchData: {},
    selectedRowKeys: [],
    delModalVisible: false,
    delModalLoading: false,
    recoveryModalVisible: false,
    recoveryModalLoading: false,
    resultModalVisible: false,
    allocateCampusId: '', // 分配学习中心
  };

  columns = [
    { title: '学生姓名', dataIndex: 'studentName', key: 'studentName', width: COL_WIDTH_S },
    { title: '客户姓名', dataIndex: 'customerName', key: 'customerName', width: COL_WIDTH_S },
    { title: '客户手机号码', dataIndex: 'contact', key: 'contact', width: COL_WIDTH_M },
    { title: '资源入口', dataIndex: 'resEntranceName', key: 'resEntranceName', width: COL_WIDTH_S },
    { title: '所属分公司', dataIndex: 'brenchName', key: 'brenchName', width: COL_WIDTH_M },
    { title: '所属学习中心', dataIndex: 'campusName', key: 'campusName', width: COL_WIDTH_S },
    { title: '跟进人', dataIndex: 'deliverTargetName', key: 'deliverTargetName', width: COL_WIDTH_S },
    { title: '录入日期', dataIndex: 'entryDate', key: 'entryDate', width: COL_WIDTH_S },
    { title: '无效日期', dataIndex: 'invalidDate', key: 'invalidDate', width: COL_WIDTH_S },
    { title: '跟进记录', dataIndex: 'remark', key: 'remark', width: COL_WIDTH_S },
    { title: '操作删除人', dataIndex: 'operator', key: 'operator', width: COL_WIDTH_M },
    { title: '操作删除时间', dataIndex: 'operatorTime', key: 'operatorTime', width: COL_WIDTH_M },
    { title: '是否删除', dataIndex: 'deleted', key: 'deleted', width: COL_WIDTH_S, render: (t) => (t ? '是' : '否') },
  ];

  resultsColumns = [
    { title: '学生姓名', dataIndex: 'studentName', key: 'studentName', width: COL_WIDTH_M },
    { title: '客户姓名', dataIndex: 'customerName', key: 'customerName', width: COL_WIDTH_M },
    { title: '手机号码', dataIndex: 'contact', key: 'contact', width: COL_WIDTH_M },
    {
      title: '分配结果',
      dataIndex: 'businessErrorMessage',
      key: 'businessErrorMessage',
      width: COL_WIDTH_M,
      render: (t, r) => `分配${r.success ? '成功' : '失败'}(${t ? t : '无'})`,
    },
  ];

  componentDidMount() {
    this.initDict();
    this.getOptionalOrganization(orgTypeConfig.CAMPUS, null, true);
  }

  initDict = () => {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['invalid_sign', 'resEntrance'],
      },
    });
  };

  // 查询 无效资源列表
  queryInvalidResource = (params) => {
    const {
      pagination: { current, pageSize },
      searchData,
    } = this.state;
    const data = {
      page: current,
      rows: pageSize,
      ...searchData,
      ...params,
    };
    this.setState({ loading: true });
    this.props.dispatch({
      type: `${namespace}/queryInvalidResource`,
      payload: { data, callback: () => this.setState({ loading: false, selectedRowKeys: [] }) },
    });
  };

  /**
   * 查询可选组织
   * @param orgType 组织类型 可选：分公司 || 分校
   * @param parentId 分公司时父组织id
   * @param returnParent 是否返回父组织
   */
  getOptionalOrganization = (orgType, parentId, returnParent = false, callback) => {
    const params = {
      orgType,
      parentId,
      returnParent,
      resourceId: menuID,
    };
    this.props.dispatch({ type: `${namespace}/getOptionalOrganization`, payload: { params, callback } });
  };

  // 查询事件回调
  handleSearch = (searchData) => {
    this.setState({ searchData });
    this.queryInvalidResource(searchData);
  };

  // 分页切换
  changePage = ({ current, pageSize }) => {
    this.setState({ pagination: { current, pageSize } });
    this.queryInvalidResource({ page: current, rows: pageSize });
  };

  // 表格多选
  onSelectChange = (selectedRowKeys) => {
    this.setState({ selectedRowKeys });
  };

  // 彻底删除确认
  handleDelOk = () => {
    const { selectedRowKeys } = this.state;
    this.setState({ delModalLoading: true });
    const params = {
      ids: selectedRowKeys,
      resourceId: menuID,
    };
    this.props.dispatch({
      type: `${namespace}/deleteInvalidResource`,
      payload: {
        params,
        callback: () => {
          this.queryInvalidResource();
          this.setState({ delModalVisible: false, delModalLoading: false, selectedRowKeys: [] });
        },
      },
    });
  };

  // 搜索框彻底删除回调
  handleDel = () => {
    const { selectedRowKeys } = this.state;
    if (selectedRowKeys.length <= 0) {
      message.warning('请先选择无效资源');
      return;
    }
    this.setState({ delModalVisible: true });
  };

  // 搜索框彻底恢复回调
  handleRecovery = () => {
    const { selectedRowKeys } = this.state;
    if (selectedRowKeys.length <= 0) {
      message.warning('请先选择无效资源');
      return;
    }
    this.setState({ recoveryModalVisible: true });
  };

  // 彻底恢复 - 确认
  handleRecoveryOk = () => {
    const { selectedRowKeys, allocateCampusId } = this.state;
    this.setState({ recoveryModalLoading: true });
    const params = {
      ids: selectedRowKeys,
      resourceId: menuID,
      allocateCampusId,
    };
    this.props.dispatch({
      type: `${namespace}/recoverResource`,
      payload: {
        params,
        callback: () => {
          this.queryInvalidResource();
          this.setState({
            recoveryModalVisible: false,
            recoveryModalLoading: false,
            resultModalVisible: true,
            selectedRowKeys: [],
          });
        },
      },
    });
  };

  render() {
    const { route, total, list, distributionObjects, distributionResults } = this.props;
    const {
      pagination: { current, pageSize },
      loading,
      delModalVisible,
      delModalLoading,
      recoveryModalLoading,
      resultModalVisible,
      selectedRowKeys,
      recoveryModalVisible,
    } = this.state;

    const rowSelection = {
      selectedRowKeys,
      onChange: this.onSelectChange,
      getCheckboxProps: (record) => ({
        disabled: record.deleted,
      }),
    };

    return (
      <Page
        fitWindowHeight={true}
        route={route}
        subHeader={
          <SearchPage
            onSearch={this.handleSearch}
            onDelete={this.handleDel}
            onRecovery={this.handleRecovery}
            getOptionalOrganization={this.getOptionalOrganization}
          />
        }
      >
        <List
          dataSource={list}
          columns={this.columns}
          loading={loading}
          pagination={{
            current,
            pageSize,
            total,
            pageSizeOptions: ['50', '100'],
          }}
          rowKey={(record) => record.id}
          onChange={this.changePage}
          rowSelection={rowSelection}
        />
        {delModalVisible && (
          <Modal
            title="彻底删除"
            widthSize="s"
            onOk={this.handleDelOk}
            loading={delModalLoading}
            onCancel={() => {
              this.setState({ delModalVisible: false });
            }}
          >
            <span>本操作会将所选资源从无效资源中彻底删除，删除后不可恢复，请确认是否需要彻底删除</span>
          </Modal>
        )}
        {recoveryModalVisible && (
          <Modal
            title="请选择分配对象"
            widthSize="s"
            onOk={this.handleRecoveryOk}
            okText="确认恢复"
            loading={recoveryModalLoading}
            onCancel={() => {
              this.setState({ recoveryModalVisible: false });
            }}
          >
            <Select
              placeholder="请选择分配对象"
              dict={distributionObjects}
              showSearch={true}
              onChange={(val) => this.setState({ allocateCampusId: val })}
            />
          </Modal>
        )}
        {resultModalVisible && (
          <Modal
            title="分配结果"
            widthSize="m"
            okBtnShow={false}
            loading={recoveryModalLoading}
            cancelText="关闭"
            onCancel={() => {
              this.setState({ resultModalVisible: false });
            }}
          >
            <List dataSource={distributionResults} columns={this.resultsColumns} />
          </Modal>
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const { list, total, distributionObjects, distributionResults } = state[namespace];
  return {
    list,
    total,
    distributionResults,
    distributionObjects,
  };
};

export default connect(mapStateToProps)(InvalidResourcePool);
