import React from 'react';
import { Form, Row, Col, DatePicker } from 'antd';
import { connect } from 'dva';
import moment from 'moment';
import { Select, Button, Input } from 'COMPONENT';
import { is_delete, menuID, namespace, orgTypeConfig } from '../store/constant';
import style from './index.less';

const { Item } = Form;

class SearchPage extends React.Component {
  constructor(props) {
    super(props);
  }
  componentDidMount() {
    const { getOptionalOrganization } = this.props;
    getOptionalOrganization(orgTypeConfig.BRENCH, _, _, (val) => {
      if (val) {
        const { setFieldsValue } = this.props.form;
        setFieldsValue({ brenchId: val });
        this.handleSearch();
        getOptionalOrganization(orgTypeConfig.CAMPUS, val);
      }
    });
  }

  /**
   * 获取学习中心的下拉字典项
   * @memberof SearchPage
   */
  getCampus = (key) => {
    const { form, getOptionalOrganization } = this.props;
    const { setFieldsValue } = form;
    setFieldsValue({ campusId: undefined });
    key && getOptionalOrganization(orgTypeConfig.CAMPUS, key);
  };

  /**
   * 查询事件
   * @memberof SearchPage
   */
  handleSearch = () => {
    const { form, onSearch } = this.props;
    const values = form.getFieldsValue();
    const { invalidDateStart, invalidDateEnd, deleted } = values;
    const deletedConfig = {
      1: true,
      0: false,
    };
    onSearch &&
      onSearch({
        ...values,
        deleted: deletedConfig[deleted],
        invalidDateStart: invalidDateStart && invalidDateStart.format('YYYY-MM-DD'),
        invalidDateEnd: invalidDateEnd && invalidDateEnd.format('YYYY-MM-DD'),
        resourceId: menuID,
      });
  };

  /**
   * 查询重置
   * @memberof SearchPage
   */
  handleReset = () => {
    const { branchDict, form } = this.props;
    const { resetFields, setFieldsValue } = form;
    resetFields();
    branchDict[0] && setFieldsValue({ brenchId: branchDict[0].val });
  };

  render() {
    const {
      branchDict,
      invalid_sign,
      resEntrance,
      form: { getFieldDecorator, getFieldValue },
      campusDict,
      onRecovery,
      onDelete,
      btnAuths,
    } = this.props;
    const brenchId = getFieldValue('brenchId');
    return (
      <Form layout="inline">
        <Row className={style.pool_search} gutter={16}>
          <Col span={4}>
            <Item>
              {getFieldDecorator('invalidDateStart', {
                initialValue: moment(new Date()).subtract(1, 'months'),
              })(<DatePicker placeholder="无效日期(开始)" />)}
            </Item>
          </Col>
          <Col span={4}>
            <Item>
              {getFieldDecorator('invalidDateEnd', {
                initialValue: moment(new Date()),
              })(<DatePicker placeholder="无效日期(结束)" />)}
            </Item>
          </Col>
          <Col span={4}>
            <Item>
              {getFieldDecorator('brenchId')(
                <Select placeholder="请选择分公司" dict={branchDict} showSearch={true} onChange={this.getCampus} />,
              )}
            </Item>
          </Col>
          <Col span={4}>
            <Item>
              {getFieldDecorator('campusId')(
                <Select placeholder="请选择学习中心" dict={brenchId ? campusDict : []} showSearch={true} />,
              )}
            </Item>
          </Col>
          <Col span={4}>
            <Item>{getFieldDecorator('studentName')(<Input placeholder="学生姓名" />)}</Item>
          </Col>
          <Col span={4}>
            <Item>{getFieldDecorator('contact')(<Input placeholder="客户手机号码" />)}</Item>
          </Col>
        </Row>
        <Row className={style.pool_search} style={{ marginTop: '10px' }} gutter={16}>
          <Col span={4}>
            <Item>{getFieldDecorator('invalidSignId')(<Select placeholder="无效标签" dict={invalid_sign} />)}</Item>
          </Col>
          <Col span={4}>
            <Item>{getFieldDecorator('deleted')(<Select placeholder="是否删除" dict={is_delete} />)}</Item>
          </Col>
          <Col span={4}>
            <Item>{getFieldDecorator('resEntranceId')(<Select placeholder="资源入口" dict={resEntrance} />)}</Item>
          </Col>
          {/* <Col span={4}>
            <Item>
              {getFieldDecorator("cusType")(
                <Select
                  placeholder="资源来源"
                  dict={resType}
                  onChange={(val) => {
                    if (val) {
                      loadSourceDetailDict(val).then((dict) => {
                        this.setState({
                          sourceDetailDict: dict,
                        });
                      });
                    } else {
                      this.setState({
                        sourceDetailDict: [],
                      });
                    }
                  }}
                />
              )}
            </Item>
          </Col>
          <Col span={4}>
            <Item>
              {getFieldDecorator("cusOrg")(
                <Select
                  placeholder="资源细分"
                  dict={this.state.sourceDetailDict}
                />
              )}
            </Item>
          </Col> */}
        </Row>
        <Row className={style.pool_search} gutter={16} style={{ marginTop: '10px' }}>
          <Col className={style.search_button} span={16}>
            <Button type="primary" onClick={this.handleSearch}>
              查询
            </Button>
            <Button type="primary" onClick={this.handleReset}>
              重置
            </Button>
            <Button type="primary" onClick={onRecovery} disabled={!btnAuths.includes('pool_restore_resources')}>
              恢复资源
            </Button>
            <Button type="primary" onClick={onDelete} disabled={!btnAuths.includes('pool_delete_resources')}>
              彻底删除
            </Button>
          </Col>
        </Row>
      </Form>
    );
  }
}

const mapStateToProps = (state) => {
  const {
    account: { btnAuths = [] },
    dict: { invalid_sign, resEntrance },
  } = state;
  const { branchDict, campusDict } = state[namespace];

  return {
    btnAuths,
    branchDict,
    campusDict,
    invalid_sign,
    resEntrance,
  };
};

export default connect(mapStateToProps)(Form.create()(SearchPage));
