import { message } from 'antd';
import { runFunc } from 'UTIL/common';
import { namespace, orgTypeConfig } from './constant';
import {
  queryInvalidResource,
  deleteInvalidResource,
  getOptionalOrganization,
  recoverResource,
} from 'SERVICE/customer';
import { transformArrayToSelect } from 'UTIL/dict';

export default {
  namespace,
  state: {
    list: [], // 无效资源池列表
    total: 0,
    branchDict: [], // 分公司枚举
    campusDict: [], // 学习中心枚举
    distributionObjects: [], // 分配对象
    distributionResults: [], // 分配结果
  },
  reducers: {
    updateState(state, { payload }) {
      return {
        ...state,
        ...payload,
      };
    },
  },
  effects: {
    // 请求无效资源列表
    *queryInvalidResource({ payload }, { put, call }) {
      const { data, callback } = payload;
      const { resultCode, rows, records } = yield call(queryInvalidResource, data);
      if (resultCode === 0) {
        yield put({
          type: 'updateState',
          payload: { list: rows, total: records },
        });
      }
      runFunc(callback);
    },
    // 彻底删除
    *deleteInvalidResource({ payload }, { call }) {
      const { params, callback } = payload;
      const { resultCode } = yield call(deleteInvalidResource, params);
      if (resultCode === 0) {
        // TODO
        message.success('彻底删除成功');
      }
      runFunc(callback);
    },
    // 查询可选组织
    *getOptionalOrganization({ payload }, { put, call }) {
      const { params, callback } = payload;
      const { orgType, returnParent } = params;
      const { resultCode, data } = yield call(getOptionalOrganization, params);
      const format = { valName: 'orgId', textName: 'orgName', optionsName: 'childList' };
      if (resultCode === 0) {
        const list = transformArrayToSelect(data, format);
        if (orgType === orgTypeConfig.BRENCH) {
          yield put({
            type: 'updateState',
            payload: { branchDict: list },
          });
          runFunc(callback, list[0] ? list[0].val : null);
        } else if (orgType === orgTypeConfig.CAMPUS) {
          yield put({
            type: 'updateState',
            payload: returnParent ? { distributionObjects: list } : { campusDict: list },
          });
          runFunc(callback);
        }
      }
    },
    // 恢复无效资源
    *recoverResource({ payload }, { put, call }) {
      const { params, callback } = payload;
      const {
        resultCode,
        data: { results },
      } = yield call(recoverResource, params);
      if (resultCode === 0) {
        yield put({
          type: 'updateState',
          payload: { distributionResults: results },
        });
      }
      runFunc(callback);
    },
  },
};
