import React from 'react';
import Page from 'COMPONENT/Page';
import SearchBar from 'COMPONENT/SearchBar';
import Input from 'COMPONENT/Input';
import Button from 'COMPONENT/Button';
import List from 'COMPONENT/List';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import DatePicker from 'COMPONENT/DatePicker';
import { buildBranchCampusDict } from 'MODEL/dict/utils';
import { connect } from 'dva';
import * as loaders from './loaders';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { transformArrayToString } from 'UTIL/common';
import DistrbuteResourceModal from './modal/DistrbuteResourceModal';
import moment from 'moment';
import styles from './style.less';
import { clone } from 'lodash';
const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M } = List;

const linkStatus = {
  add: 1,
  delete: 0,
};
const dateFormat = 'YYYY-MM-DD';

class NetworkResourcePool extends React.Component {
  constructor(props) {
    super(props);
    const today = moment(new Date()).format(dateFormat);

    this.state = {
      listLoading: false,
      loading: false,
      phone: null,
      name: null,
      blCampusId: [],
      studentStatus: null,
      allocateStatus: 'UNALLOCATED',
      createDateStart: today,
      createDateEnd: today,
      followupBeginDate: null,
      followupEndDate: null,
      resourceModalData: [],
      distributeResourceModalVisible: false,
      selectedRowKeys: [],
      rows: [],
      pageNo: 1,
      pageSize: 20,
      total: null,
    };
  }

  componentDidMount() {
    this.initData();
  }

  initData = async () => {
    this.setState({ loading: true });
    await this.initDict();
    await this.loadList();
    this.setState({ loading: false });
  };
  updateState = (changedState) => {
    const newState = {
      ...this.state,
      ...changedState,
    };
    return new Promise((resolve) => {
      this.setState(newState, () => {
        resolve();
      });
    });
  };

  pageChange = async ({ current, pageSize }) => {
    await this.updateState({ loading: true, pageSize, pageNo: current });
    const list = await this.loadListData();
    this.updateState({
      loading: false,
      rows: list.data,
      total: list.rowCount,
      pageNo: list.pageNo,
    });
  };

  loadListData = async () => {
    const {
      phone,
      name,
      blCampusId,
      studentStatus,
      allocateStatus,
      pageNo,
      pageSize,
      total,
      createDateStart,
      createDateEnd,
      followupBeginDate,
      followupEndDate,
    } = this.state;
    const apiParams = {
      contact: phone,
      studentName: name,
      studentStatus,
      allocateStatus,
      intentionCampusId: transformArrayToString(blCampusId),
      page: pageNo,
      rows: pageSize,
      getTotal: false,
      entryTimeStart: createDateStart,
      entryTimeEnd: createDateEnd,
      allocateTimeStart: followupBeginDate,
      allocateTimeEnd: followupEndDate,
    };
    const fetchList = [loaders.getResourcePoolList(apiParams)];
    if (pageNo === 1) {
      fetchList.push(loaders.getResourcePoolList({ ...apiParams, getTotal: true }));
    }
    let [detail, extraDetail] = await Promise.all(fetchList);
    detail = {
      ...detail,
      data: detail.rows,
      rowCount: extraDetail ? extraDetail.records : total,
      pageNo,
    };
    return detail;
  };
  loadList = async () => {
    try {
      this.setState({ loading: true });
      let { data, pageNo, rowCount } = await this.loadListData();
      this.setState({
        loading: false,
        rows: data,
        total: rowCount,
        page: pageNo,
        selectedRowKeys: [],
      });
    } catch (e) {
      showMessage(e.text, 'error');
      this.setState({ loading: false });
    }
  };

  reset = () => {
    this.setState({
      studentId: null,
      phone: null,
      name: null,
      studentStatus: null,
      allocateStatus: 'UNALLOCATED',
      blCampusId: [],
      createDateStart: null,
      createDateEnd: null,
      followupBeginDate: null,
      followupEndDate: null,
    });
  };

  search = () => {
    this.setState({ pageNo: 1 }, () => {
      this.loadList();
    });
  };

  initDict = () => {
    return new Promise((resolve, reject) => {
      this.props.dispatch({
        type: 'dict/initDict',
        payload: {
          list: ['branchCampus', 'studentStatus', 'networkCustomerAllocateStatus'],
          callback() {
            resolve();
          },
        },
      });
    });
  };

  getDisabledStartDate = (startDate, type) => {
    const stateType = {
      follow: 'followupEndDate',
      create: 'createDateEnd',
    };
    const endDate = this.state[stateType[type]];
    if (!startDate || !endDate) {
      return false;
    }
    return moment(startDate).valueOf() > moment(endDate).valueOf();
  };

  getDisabledEndDate = (endDate, type) => {
    const stateType = {
      follow: 'followupBeginDate',
      create: 'createDateStart',
    };
    const startDate = this.state[stateType[type]];
    if (!endDate || !startDate) {
      return false;
    }
    return moment(endDate).valueOf() < moment(startDate).valueOf();
  };

  listRowClick = (record, index, event) => {
    const { selectedRowKeys } = this.state;
    let selected = !selectedRowKeys.includes(index);
    let _selectedRowKeys = clone(selectedRowKeys);
    if (selected) {
      _selectedRowKeys.push(index);
    } else {
      _selectedRowKeys.filter((item) => item !== index);
    }
    this.setState({
      selectedRowKeys: _selectedRowKeys,
    });
  };

  handleResourceDistribute = () => {
    const { rows, selectedRowKeys } = this.state;
    const resourceModalData = selectedRowKeys.map((rowIndex) => rows[rowIndex]);
    this.setState({
      distributeResourceModalVisible: true,
      resourceModalData,
    });
  };
  modalClose = (isReload) => {
    this.setState({ distributeResourceModalVisible: false, resourceModalData: [] });
    if (isReload) {
      this.loadList();
    }
  };
  downloadExcel = async () => {
    const {
      phone,
      name,
      blCampusId,
      allocateStatus,
      createDateStart,
      createDateEnd,
      followupBeginDate,
      followupEndDate,
    } = this.state;
    try {
      this.setState({ loading: true });
      await loaders.exportResourcePool({
        intentionCampusId: transformArrayToString(blCampusId),
        studentName: name,
        contact: phone,
        allocateStatus,
        entryTimeStart: createDateStart,
        entryTimeEnd: createDateEnd,
        allocateTimeStart: followupBeginDate,
        allocateTimeEnd: followupEndDate,
      });
      this.props.dispatch({ type: 'main/exportNotice' });
      this.setState({ loading: false });
    } catch (e) {
      showMessage(e.text, 'error');
    }
  };
  // 表头
  columns = [
    { title: '学生名字', dataIndex: 'studentName', width: COL_WIDTH_S },
    { title: '客户姓名', dataIndex: 'customerName', width: COL_WIDTH_S },
    { title: '客户手机号码', dataIndex: 'contact', width: COL_WIDTH_M },
    { title: '意向学习中心', dataIndex: 'intentionCampusName', width: COL_WIDTH_S },
    { title: '资源入口', dataIndex: 'resEntranceName', width: COL_WIDTH_M },
    { title: '创建时间', dataIndex: 'entryTime', width: COL_WIDTH_M },
    { title: '分配时间', dataIndex: 'allocateTime', width: COL_WIDTH_M },
    { title: '分配状态', dataIndex: 'allocateStatus', width: COL_WIDTH_M },
  ];

  render() {
    const {
      loading,
      phone,
      name,
      listLoading,
      rows,
      pageNo,
      pageSize,
      total,
      selectedRowKeys,
      showDistributeDetailModal,
      blCampusId,
      studentStatus,
      createDateStart,
      createDateEnd,
      followupBeginDate,
      followupEndDate,
      distributeResourceModalVisible,
      resourceModalData,
      allocateStatus,
    } = this.state;
    const { branchCampusDictOrigin, studentStatusDict, networkCustomerAllocateStatus, btnAuths } = this.props;
    return (
      <Page name="网络资源池" loading={loading} route={this.props.route} ref={(c) => (this._page = c)}>
        <div className={styles.networkResourcePool}>
          <SearchBar onSearch={this.search} onReset={this.reset} showExpandButton={false} expand={true}>
            <DatePicker
              placeholder="创建日期（开始）"
              value={createDateStart}
              onChange={(val) => this.setState({ createDateStart: val })}
              disabledDate={(val) => this.getDisabledStartDate(val, 'create')}
            />
            <DatePicker
              placeholder="创建日期（结束）"
              value={createDateEnd}
              onChange={(val) => this.setState({ createDateEnd: val })}
              disabledDate={(val) => this.getDisabledEndDate(val, 'create')}
            />
            <Input placeholder="学生姓名" value={name} onChange={(e) => this.setState({ name: e.target.value })} />
            <Input
              placeholder="客户手机号码"
              value={phone}
              onChange={(e) => this.setState({ phone: e.target.value })}
            />
            <MultiSelect
              placeholder="意向学习中心"
              dict={buildBranchCampusDict(null, branchCampusDictOrigin)}
              value={blCampusId}
              showSearch
              allowClear={true}
              onChange={(blCampusId) => this.setState({ blCampusId: blCampusId || [] })}
            />
            {/*<Select*/}
            {/*  placeholder="学生状态"*/}
            {/*  dict={studentStatusDict}*/}
            {/*  value={studentStatus}*/}
            {/*  showSearch*/}
            {/*  allowClear={true}*/}
            {/*  onChange={(studentStatus) => this.setState({ studentStatus })}/>*/}
            <Select
              placeholder="分配状态"
              dict={networkCustomerAllocateStatus}
              value={allocateStatus}
              showSearch
              allowClear={true}
              onChange={(allocateStatus) => this.setState({ allocateStatus })}
            />
            <DatePicker
              placeholder="分配日期（开始）"
              value={followupBeginDate}
              onChange={(val) => this.setState({ followupBeginDate: val })}
              disabledDate={(val) => this.getDisabledStartDate(val, 'follow')}
            />
            <DatePicker
              placeholder="分配日期（结束）"
              value={followupEndDate}
              onChange={(val) => this.setState({ followupEndDate: val })}
              disabledDate={(val) => this.getDisabledEndDate(val, 'follow')}
            />
          </SearchBar>
          <div className={styles.buttonWrapper}>
            {btnAuths.includes('BTN_POOL_DIST_BTN') && (
              <Button
                type="primary"
                onClick={this.handleResourceDistribute}
                disabled={!(Array.isArray(selectedRowKeys) && selectedRowKeys.length)}
              >
                资源分配
              </Button>
            )}
            {btnAuths.includes('BTN_POOL_STUD_DOWN') && (
              <Button type="primary" onClick={this.downloadExcel}>
                在读学生下载
              </Button>
            )}
          </div>
        </div>
        <List
          columns={this.columns}
          loading={listLoading}
          dataSource={rows || []}
          pagination={{
            current: pageNo,
            pageSize,
            total: total,
          }}
          onChange={this.pageChange}
          rowSelection={{
            selectedRowKeys,
            onChange: (keys) => this.setState({ selectedRowKeys: keys }),
          }}
          onRowClick={this.listRowClick}
        />
        {distributeResourceModalVisible && (
          <DistrbuteResourceModal data={resourceModalData} onClose={this.modalClose} />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  // 去掉分配成功的字典
  const networkCustomerAllocateStatus = (state.dict.networkCustomerAllocateStatus || []).filter(
    (status) => status.val !== 'SUCCESS',
  );
  return {
    btnAuths: state.account.btnAuths,
    user: state.account.user,
    aliPath: state.account.user.aliPath,
    // 数据字典
    studentStatusDict: state.dict.studentStatus,
    branchCampusDictOrigin: state.dict.branchCampus,
    networkCustomerAllocateStatus,
  };
};

export default connect(mapStateToProps)(NetworkResourcePool);
