import React from 'react';
import { Form, Row, Col, Checkbox, message } from 'antd';
import { connect } from 'dva';
import { Page, Block, Input, Select, TextArea, Button } from 'COMPONENT';
import { loadCommonDict, loadAllBranchCampusDict } from 'UTIL/dict';
import { getStudentListByCusContact, saveSignStudentTransferCustomer } from 'SERVICE/customer';
import styles from './styles.module.less';
import StuffSelect from './StuffSelect';
const FormItem = Form.Item;
const CheckboxGroup = Checkbox.Group;
const formItemCol = {
  labelCol: { span: 8 },
  wrapperCol: { span: 15 },
};
const linkFormCfg = {
  labelCol: { span: 5 },
  wrapperCol: { span: 19 },
  style: {
    padding: '0 10px',
    marginBottom: 10,
    lineHeight: '24px',
  },
};
const phoneRegExp = /^1\d{10}$/;
class AddOldStudent extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      cusOrgDict: [],
      loading: false,
      studentList: [],
      oldStudents: [],
      bonusBelongName: '',
      customer: null,
      campusDict: [],
      initialCampus: '',
    };
  }
  componentDidMount() {
    Promise.all([loadCommonDict('CUS_ORG', 'INTRODUCE'), loadAllBranchCampusDict(), this.loadCampus()]).then(
      ([cusOrgDict, campusDict]) => {
        const { brenchId, campusId, organizationType } = this.props.user;
        let defaultCampus = '';
        if (organizationType !== 'GROUNP') {
          campusDict = brenchId ? campusDict.filter((branch) => branch.val === brenchId) : [];
        }
        if (organizationType === 'CAMPUS') {
          defaultCampus = campusId; // 校区账号，默认本校区
        }
        this.setState({
          cusOrgDict: cusOrgDict,
          campusDict,
          initialCampus: defaultCampus,
        });
      },
    );
  }
  loadCampus = () => {
    if (!this.props.campus) {
      this.props.dispatch({
        type: 'dict/initDict',
        payload: {
          list: ['branch', 'branchAllCampus'],
        },
      });
    }
  };

  onCheckBoxChange = (val) => {
    this.setState(
      (preState) => {
        const preChecked = preState.oldStudents;
        val.forEach((item) => {
          if (preChecked.indexOf(item) > -1) {
            preChecked.splice(preChecked.indexOf(item), 1);
          } else {
            preChecked.push(item);
          }
        });
        return {
          oldStudents: val.length ? preChecked : [],
        };
      },
      () => {
        const student = this.state.studentList.filter((item) => item.studentId == this.state.oldStudents[0]);
        console.log(student);
        if (this.state.oldStudents.length === 0) return;
        if (student[0].studentType === 'POTENTIAL') {
          // 未签过单-POTENTIAL,签过单-ENROLLED
          message.warning('未签单学生不可以登记老生转介绍');
          this.setState({
            oldStudents: [],
          });
          return;
        } else if (!student[0].canRecyclable) {
          // 不符合激活条件
          message.warning('不满足老生激活条件');
          this.setState({
            oldStudents: [],
          });
          return;
        }
      },
    );
  };

  getStudentsOnChange = async (val) => {
    if (phoneRegExp.test(val)) {
      this.setState({ loading: true });
      const res = await getStudentListByCusContact({ contact: val });
      const studentList = res.data.studentList || [];
      if (studentList.length === 0) {
        message.warn('该客户系统不存在');
      }
      this.setState({
        studentList,
        loading: false,
        customer: res.data,
      });
    } else {
      this.setState({
        studentList: [],
        customer: null,
      });
    }
    this.setState({
      oldStudents: [],
    });
  };
  handleSubmit = (e) => {
    e.preventDefault();
    this.props.form.validateFieldsAndScroll(async (errors, form) => {
      if (!errors) {
        if (this.state.oldStudents.length === 0) {
          message.warn('请选择满足条件的学生提交申请');
          return;
        }
        const { resultCode, resultMessage } = await saveSignStudentTransferCustomer({
          ...form,
          bonusBelongId: form.bonusBelongId.val,
          studentId: this.state.oldStudents[0],
          customerId: this.state.customer.customerId,
        });
        if (resultCode === 0) {
          message.success('提交成功');
          const { dispatch } = this.props;
          dispatch({
            type: 'main/openPage',
            payload: {
              path: '/oldStudentRegister',
              refresh: true,
            },
          });
        } else {
          message.warning(resultMessage);
        }
      }
    });
  };
  onResetClick = () => {
    this.props.form.resetFields();
    this.setState({
      studentList: [],
      oldStudents: [],
    });
  };

  handleChange = (selected) => {
    this.props.form.setFieldsValue({
      bonusBelongId: selected ? selected.val : '',
    });
    this.setState({
      bonusBelongName: selected ? selected.text : '',
    });
  };
  render() {
    const { cusOrgDict, loading, studentList, oldStudents, campusDict, initialCampus } = this.state;
    const { getFieldDecorator } = this.props.form;
    return (
      <Page loading={loading} name="客户录入" route={this.props.route}>
        <Form onSubmit={this.handleSubmit}>
          <Block title="被激活老生信息">
            <Row type="flex" justify="start">
              <Col span={8}>
                <FormItem label="客户电话" {...formItemCol}>
                  {getFieldDecorator('customerId', {
                    rules: [
                      {
                        required: true,
                        pattern: phoneRegExp,
                        message: '请填写正确的联系方式',
                      },
                    ],
                  })(<Input onChange={(e) => this.getStudentsOnChange(e.target.value)} />)}
                </FormItem>
              </Col>
              <Col span={8}>
                <FormItem label="介绍类型" {...formItemCol}>
                  {getFieldDecorator('cusOrg', {
                    rules: [{ required: true, message: '请选择介绍类型' }],
                  })(<Select placeholder="请选择介绍类型" dict={cusOrgDict} />)}
                </FormItem>
              </Col>
            </Row>
            <Row type="flex" justify="start">
              <Col span={8}>
                <FormItem label="意向校区" {...formItemCol}>
                  {getFieldDecorator('intentionCampusId', {
                    initialValue: initialCampus,
                    rules: [{ required: true, message: '请选择意向校区' }],
                  })(<Select placeholder="意向校区" showSearch dict={campusDict} allowClear={true} />)}
                </FormItem>
              </Col>
            </Row>
            <Row type="flex" justify="start">
              <Col span={16}>
                <FormItem label="客户情况" labelCol={{ span: 4 }} wrapperCol={{ span: 20 }}>
                  {getFieldDecorator('remark', {
                    initialValue: '',
                    rules: [{ max: 50, message: '客户情况不能超过50字' }],
                  })(<TextArea maxLength={50} wordCount={true} />)}
                </FormItem>
              </Col>
            </Row>
          </Block>

          {studentList.length <= 0 ? (
            ''
          ) : (
            <Block title="关联学生" style={{ paddingBottom: '30px', marginBottom: '30px' }}>
              <Row type="flex" justify="start">
                <Col span={16}>
                  <div className={styles.tipInfo}>
                    <span>同一学生，请勾选关联并保存即可</span>
                  </div>
                </Col>
              </Row>
              <CheckboxGroup onChange={this.onCheckBoxChange} value={oldStudents}>
                {studentList.map((ele) => {
                  const tag = ele.studentType === 'ENROLLED' && !ele.canFollow && '在读学生';
                  return (
                    <Row key={ele.studentId} style={{ marginLeft: 20 }}>
                      <Col span={1}>
                        <Checkbox value={ele.studentId} checked={this.state.oldStudents.indexOf(ele.studentId) > -1} />
                      </Col>
                      <Col span={6}>
                        <FormItem label="学生" {...linkFormCfg}>
                          <div
                            style={{
                              wordBreak: 'break-word',
                              display: 'flex',
                              justifyContent: 'center',
                              alignItems: 'baseline',
                            }}
                          >
                            <span
                              style={{
                                wordBreak: 'break-word',
                                flex: '1',
                              }}
                            >
                              {ele.studentName}
                            </span>
                            {tag && (
                              <span
                                style={{
                                  marginLeft: '10px',
                                  border: '1px #ff8080 solid',
                                  padding: '0 4px',
                                  borderRadius: '4px',
                                  color: '#ff8080',
                                  display: 'inline-block',
                                  lineHeight: '20px',
                                  fontSize: '12px',
                                }}
                              >
                                {tag}
                              </span>
                            )}
                          </div>
                        </FormItem>
                      </Col>
                      <Col span={4}>
                        <FormItem label="年级" {...linkFormCfg}>
                          {ele.gradeName}
                        </FormItem>
                      </Col>
                      <Col span={6}>
                        <FormItem label="学校" {...linkFormCfg}>
                          {ele.schoolName}
                        </FormItem>
                      </Col>
                    </Row>
                  );
                })}
              </CheckboxGroup>
            </Block>
          )}

          <Block title="介绍人信息" key="intro">
            <Row type="flex" justify="start">
              <Col span={8}>
                <FormItem label="谁介绍" {...formItemCol}>
                  {getFieldDecorator('introducer', {
                    initialValue: '',
                    rules: [{ required: true, message: '请填写介绍人姓名' }],
                  })(<Input maxLength={10} />)}
                </FormItem>
              </Col>
              <Col span={8}>
                <FormItem label="联系方式" {...formItemCol}>
                  {getFieldDecorator('introducerContact', {
                    initialValue: '',
                    rules: [
                      { required: true, message: '请填写联系方式' },
                      {
                        pattern: phoneRegExp,
                        message: '请填写正确的联系方式',
                      },
                    ],
                  })(<Input />)}
                </FormItem>
              </Col>
            </Row>
          </Block>

          <Block
            title={
              <span>
                <span>奖金归属</span>
              </span>
            }
            key="bonus"
          >
            <Row type="flex" justify="start">
              <Col span={8}>
                <FormItem label="归属人" {...formItemCol}>
                  {getFieldDecorator('bonusBelongId', {
                    initialValue: '',
                    rules: [{ required: true, message: '请选择奖金归属人' }],
                  })(
                    <StuffSelect
                      onChange={this.handleChange}
                      initialValue={{
                        value:
                          typeof this.props.form.getFieldValue('bonusBelongId') == 'object'
                            ? this.props.form.getFieldValue('bonusBelongId').val
                            : this.props.form.getFieldValue('bonusBelongId'),
                        text: this.state.bonusBelongName,
                      }}
                    />,
                  )}
                </FormItem>
              </Col>
            </Row>
          </Block>

          <Block title="奖励内容备注" key="remark">
            <Row type="flex" justify="start">
              <Col span={16}>
                <FormItem label="被介绍人奖励" labelCol={{ span: 4 }} wrapperCol={{ span: 20 }}>
                  {getFieldDecorator('customerBonusRemark', {
                    rules: [{ transform: (val) => (val ? val.trim() : val) }],
                  })(<TextArea maxLength={50} wordCount={true} />)}
                </FormItem>
              </Col>
              <Col span={16}>
                <FormItem label="介绍人奖励" labelCol={{ span: 4 }} wrapperCol={{ span: 20 }}>
                  {getFieldDecorator('introduceBonusRemark', {
                    rules: [{ transform: (val) => (val ? val.trim() : val) }],
                  })(<TextArea maxLength={50} wordCount={true} />)}
                </FormItem>
              </Col>
              <Col span={16}>
                <FormItem label="归属员工奖励" labelCol={{ span: 4 }} wrapperCol={{ span: 20 }}>
                  {getFieldDecorator('belongBonusRemark', {
                    rules: [{ transform: (val) => (val ? val.trim() : val) }],
                  })(<TextArea maxLength={50} wordCount={true} />)}
                </FormItem>
              </Col>
            </Row>
            <Row type="flex" justify="start">
              <Col span={16}>
                <div
                  style={{
                    display: 'flex',
                    justifyContent: 'center',
                  }}
                >
                  <FormItem wrapperCol={{ span: 12, offset: 5 }}>
                    <Button type="primary" htmlType="submit">
                      提交审核
                    </Button>
                  </FormItem>
                  <FormItem
                    wrapperCol={{ span: 12, offset: 5 }}
                    style={{
                      marginLeft: '20px',
                    }}
                  >
                    <Button htmlType="button" onClick={this.onResetClick}>
                      重置
                    </Button>
                  </FormItem>
                </div>
              </Col>
            </Row>
          </Block>
        </Form>
      </Page>
    );
  }
}
const mapStateToProps = (state) => {
  return {
    user: state.account.user,
  };
};
export default connect(mapStateToProps)(Form.create()(AddOldStudent));
