import React from 'react';
import { SearchBar, Input, Select } from 'COMPONENT';
import { DatePicker } from 'antd';
import { runFunc } from 'UTIL/common';
import PropTypes from 'prop-types';
import { connect } from 'dva';
import { loadCommonDict, loadBranchCampusDict } from 'UTIL/dict';
import StuffSelect from './StuffSelect';
import moment from 'moment';
class Search extends React.Component {
  static propTypes = {
    onSearch: PropTypes.func.isRequired,
    filterCreateUser: PropTypes.bool, // 是否需要登记人过滤条件
  };
  static defaultProps = {
    filterCreateUser: true,
    hasDefaultDate: true,
  };
  constructor(props) {
    super(props);
    const now = new Date();
    const defaultDate = `${now.getFullYear()}-${now.getMonth() + 1}-${now.getDate()}`;
    this.state = {
      searchParams: {
        customerName: null,
        studentName: null,
        contact: null,
        introducer: null,
        beginCreateTime: moment(defaultDate + ' 00:00:00'),
        endCreateTime: moment(defaultDate + ' 23:59:59'),
        branchId: null,
        intentionCampusId: null,
        bonusBelongId: null,
        createUserId: null,
        introducerType: null,
      },
      cusOrgDict: null,
      bonusBelongName: '',
      createUserName: '',
      company: [],
      filterCampus: this.props.campus, // 校区
    };
    this.search = this.search.bind(this);
    this.reset = this.reset.bind(this);
    this.handleSelectBonus = this.handleSelectBonus.bind(this);
    this.handleSelectCreate = this.handleSelectCreate.bind(this);
    this.changeCompany = this.changeCompany.bind(this);
  }
  componentDidMount() {
    Promise.all([loadCommonDict('CUS_ORG', 'INTRODUCE'), loadBranchCampusDict()]).then(([cusOrgDict, campusDict]) => {
      const company = [];
      campusDict.forEach((item) => {
        item.text = item.label;
        company.push(item);
        delete item.options;
        delete item.label;
      });
      this.setState({
        cusOrgDict: cusOrgDict,
        company: company,
      });
    });
    if (!this.props.campus) {
      this.props.dispatch({
        type: 'dict/initDict',
        payload: {
          list: ['branch', 'branchAllCampus'],
          callback: () => {
            this.setState({
              filterCampus: this.props.campus,
            });
          },
        },
      });
    }
  }
  search() {
    runFunc(this.props.onSearch, this.state.searchParams);
  }
  reset() {
    const now = new Date();
    const defaultStartDate = `${now.getFullYear()}-${now.getMonth() + 1}-${now.getDate()}`;
    const obj = {
      ...this.state.searchParams,
    };
    Object.keys(obj).forEach((key) => (obj[key] = null));
    obj.beginCreateTime = moment(defaultStartDate + ' 00:00:00');
    obj.endCreateTime = moment(defaultStartDate + ' 23:59:59');
    this.setState({
      searchParams: obj,
    });
    this.changeCompany(null);
    runFunc(this.props.onSearch, obj);
  }
  handleSelectBonus(selected) {
    this.setState({
      searchParams: {
        ...this.state.searchParams,
        bonusBelongId: selected ? selected.val : '',
      },
      bonusBelongName: selected ? selected.text : '',
    });
  }
  handleSelectCreate(selected) {
    this.setState({
      searchParams: {
        ...this.state.searchParams,
        createUserId: selected ? selected.val : '',
      },
      createUserName: selected ? selected.text : '',
    });
  }
  // 选择分公司 -> 联动校区选项
  changeCompany(companyId) {
    const filterCampus = companyId ? this.props.campus.filter((item) => item.val === companyId) : this.props.campus;
    // 意向校区先选 -> 反选分公司
    const matchIndex =
      this.state.searchParams.intentionCampusId && companyId
        ? filterCampus[0].options.findIndex((item) => item.val === this.state.searchParams.intentionCampusId)
        : -1;
    const intentionCampusId = matchIndex > -1 ? this.state.searchParams.intentionCampusId : null;
    this.setState({
      searchParams: { ...this.state.searchParams, branchId: companyId, intentionCampusId },
      filterCampus: companyId ? this.props.campus.filter((item) => item.val === companyId) : this.props.campus,
    });
  }
  render() {
    const { cusOrgDict, searchParams, company, filterCampus } = this.state;
    const {
      customerName,
      studentName,
      contact,
      introducer,
      beginCreateTime,
      endCreateTime,
      branchId,
      intentionCampusId,
      bonusBelongId,
      createUserId,
      introducerType,
    } = searchParams;
    // const { campus } = this.props
    return (
      <SearchBar onSearch={this.search} onReset={this.reset}>
        <Input
          placeholder="客户姓名"
          value={customerName}
          onChange={(e) => this.setState({ searchParams: { ...searchParams, customerName: e.target.value } })}
        />
        <Input
          placeholder="学生姓名"
          value={studentName}
          onChange={(e) => this.setState({ searchParams: { ...searchParams, studentName: e.target.value } })}
        />
        <Input
          placeholder="客户电话"
          value={contact}
          onChange={(e) => this.setState({ searchParams: { ...searchParams, contact: e.target.value } })}
        />
        <Input
          placeholder="介绍人"
          value={introducer}
          onChange={(e) => this.setState({ searchParams: { ...searchParams, introducer: e.target.value } })}
        />
        <DatePicker
          placeholder="登记开始时间"
          value={beginCreateTime}
          onChange={(date, dateString) =>
            this.setState({ searchParams: { ...searchParams, beginCreateTime: moment(dateString + ' 00:00:00') } })
          }
          allowClear={!this.props.hasDefaultDate}
          size="large"
          format="YYYY-MM-DD"
        />
        <DatePicker
          placeholder="登记结束时间"
          value={endCreateTime}
          onChange={(date, dateString) =>
            this.setState({ searchParams: { ...searchParams, endCreateTime: moment(dateString + ' 23:59:59') } })
          }
          allowClear={!this.props.hasDefaultDate}
          size="large"
          format="YYYY-MM-DD"
        />
        <Select
          placeholder="所属分公司"
          showSearch
          dict={company}
          value={branchId}
          allowClear={true}
          onChange={(value) => this.changeCompany(value)}
        />
        <Select
          placeholder="意向校区"
          showSearch
          dict={filterCampus}
          value={intentionCampusId}
          allowClear={true}
          onChange={(value) => {
            this.setState({ searchParams: { ...searchParams, intentionCampusId: value } });
          }}
        />
        <StuffSelect
          placeholder="激励归属员工"
          onChange={this.handleSelectBonus}
          initialValue={{
            value: bonusBelongId,
            text: this.state.bonusBelongName,
          }}
        />
        {this.props.filterCreateUser && (
          <StuffSelect
            placeholder="登记人"
            onChange={this.handleSelectCreate}
            initialValue={{
              value: createUserId,
              text: this.state.createUserName,
            }}
          />
        )}
        <Select
          placeholder="介绍类型"
          dict={cusOrgDict}
          value={introducerType}
          onChange={(value) => {
            this.setState({ searchParams: { ...searchParams, introducerType: value } });
          }}
        />
      </SearchBar>
    );
  }
}
const mapStateToProps = (state) => {
  return {
    campus: state.dict['branchAllCampus'],
  };
};
export default connect(mapStateToProps)(Search);
