import React from 'react';
import PropTypes from 'prop-types';
import { Select } from 'COMPONENT';
import debounce from 'lodash/debounce';
import { searchUser } from 'SERVICE/customer';
import { errorsHandle } from 'UTIL/common';
import { runFunc } from 'UTIL/common';
export default class StuffSelect extends React.Component {
  static propTypes = {
    initialValue: PropTypes.shape({
      value: PropTypes.string,
      text: PropTypes.string,
    }),
    onChange: PropTypes.func.isRequired,
    placeholder: PropTypes.string,
    searchPlaceholder: PropTypes.string,
    searchNotFoundTips: PropTypes.string,
  };
  static defaultProps = {
    placeholder: '请选择员工',
    searchPlaceholder: '请先输入员工名称查找',
    searchNotFoundTips: '没有找到该员工',
    initialValue: {
      text: '',
      value: '',
    },
  };

  constructor(props) {
    super(props);
    this.state = {
      loading: false,
      bonusPersonDict: null,
      fieldValue: this.props.initialValue.val,
    };
    this.bonusPersonStorage = {};
    this.handleBonusPersonSearch = debounce(this.onBonusPersonSearch, 1000);
  }

  componentDidMount() {
    this.props.initialValue.text && this.fundBonusPerson(this.props.initialValue.text);
  }

  onBonusPersonSearch = async (val) => {
    if (val) {
      if (this.bonusPersonStorage[val]) {
        this.setState({ bonusPersonDict: this.bonusPersonStorage[val] }, this.updateBonusPerson);
      } else {
        this.fundBonusPerson(val);
      }
    }
  };

  fundBonusPerson = async (name) => {
    this.setState({ loading: true });
    const rsp = await searchUser({ name }).catch(errorsHandle());
    if (rsp && Array.isArray(rsp.data)) {
      const dict = rsp.data.map((item) => ({
        text: `${item.username}<${item.employeeNo}>`,
        subText: item.orgInfo,
        val: item.userId,
      }));
      this.bonusPersonStorage[name] = dict;
      this.setState({ bonusPersonDict: dict }, this.updateBonusPerson);
    }
    this.setState({ loading: false });
  };

  updateBonusPerson = () => {
    const bonusBelongId = this.state.fieldValue;
    const bonusBelongIndex = this.state.bonusPersonDict.findIndex((item) => item.val === bonusBelongId);
    if (bonusBelongIndex === -1) {
      this.setState({
        fieldValue: null,
      });
    }
  };
  handleChange = (selected) => {
    const bonusBelongIndex = this.state.bonusPersonDict.findIndex((item) => item.val === selected);
    runFunc(this.props.onChange, bonusBelongIndex > -1 ? this.state.bonusPersonDict[bonusBelongIndex] : '');
  };
  render() {
    return (
      <Select
        placeholder={this.props.placeholder}
        dict={this.state.bonusPersonDict}
        showSearch
        allowClear
        searchPlaceholder={this.props.searchPlaceholder}
        searchNotFoundTips={this.props.searchNotFoundTips}
        onSearchKeyChange={this.handleBonusPersonSearch}
        onChange={this.handleChange}
        value={this.props.initialValue.value}
      />
    );
  }
}
