import React from 'react';
import { connect } from 'dva';
import { Page, List, ListAction, Button } from 'COMPONENT';
import { pageMyTransfer } from 'SERVICE/customer';
import Search from './Search';
import { auditStatus } from './config';
import moment from 'moment';
const createSetWidth = List.createSetWidth();
class TableList extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      list: [],
      pageNum: 1,
      total: 0,
      pageSize: 10,
      searchParams: {
        beginCreateTime: moment(Date.now()).format('YYYY-MM-DD') + ' 00:00:00',
        endCreateTime: moment(Date.now()).format('YYYY-MM-DD') + ' 24:00:00',
      },
    };
    this.columns = [
      {
        title: '客户姓名',
        dataIndex: 'customerName',
        width: createSetWidth(5),
      },
      { title: '学生姓名', dataIndex: 'studentName', width: createSetWidth(5) },
      { title: '客户电话', dataIndex: 'contact', width: createSetWidth(7) },
      {
        title: '所属分公司',
        dataIndex: 'intentionBranchName',
        width: createSetWidth(11),
      },
      {
        title: '意向校区',
        dataIndex: 'intentionCampusName',
        width: createSetWidth(11),
      },
      { title: '介绍人', dataIndex: 'introducer', width: createSetWidth(5) },
      {
        title: '联系电话',
        dataIndex: 'introducerContact',
        width: createSetWidth(7),
      },
      {
        title: '奖金归属员工',
        dataIndex: 'bonusBelongName',
        width: createSetWidth(9),
      },
      {
        title: '跟进人',
        dataIndex: 'distributeTargetName',
        width: createSetWidth(5),
      },
      { title: '是否签单', dataIndex: 'signed', width: createSetWidth(5) },
      {
        title: '登记人',
        dataIndex: 'createUserName',
        width: createSetWidth(5),
      },
      { title: '登记时间', dataIndex: 'createTime', width: createSetWidth(11) },
      {
        title: '被介绍人奖励',
        dataIndex: 'customerBonusRemark',
        width: createSetWidth(11),
      },
      {
        title: '介绍人奖励',
        dataIndex: 'introduceBonusRemark',
        width: createSetWidth(11),
      },
      {
        title: '归属员工奖励',
        dataIndex: 'belongBonusRemark',
        width: createSetWidth(11),
      },
      {
        title: '审核状态',
        dataIndex: 'auditStatus',
        width: createSetWidth(5),
        render: (text) => {
          return auditStatus.filter((item) => item.text === text)[0]['value'];
        },
      },
      {
        title: '操作',
        dataIndex: 'id',
        fixed: 'right',
        width: createSetWidth(3),
        render: (id) => (
          <ListAction
            config={[
              {
                text: '详情',
                onClick: () => this.handleRowClick(id),
              },
            ]}
          />
        ),
      },
    ];

    this.handleRowClick = this.handleRowClick.bind(this);
    this.loadList = this.loadList.bind(this);
    this.registerStudent = this.registerStudent.bind(this);
    this.handleSearch = this.handleSearch.bind(this);
  }

  componentDidMount() {
    this.setState({ loading: true });
    this.loadList({ pageNum: 1, pageSize: 10 });
    this.setState({ loading: false });
  }

  handleRowClick(id) {
    const { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: {
        path: '/oldStudentRegister/detail',
        params: {
          id,
        },
        refresh: true,
      },
    });
  }
  async loadList({ pageNum, pageSize }) {
    const { resultCode, data } = await pageMyTransfer({
      ...this.state.searchParams,
      type: 2, // 老生转介绍
      pageSize,
      pageNum,
    });
    if (resultCode === 0) {
      this.setState({
        list: data.list,
        pageNum: data.pageNum,
        pageSize: data.pageSize,
        total: data.total,
      });
    }
  }
  handleSearch(params) {
    this.setState(
      {
        searchParams: params,
      },
      () => {
        this.loadList({ pageNum: 1, pageSize: 10 });
      },
    );
  }

  registerStudent() {
    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path: '/oldStudentRegister/add',
        refresh: true,
      },
    });
  }
  render() {
    const { pageNum, total, pageSize } = this.state;
    return (
      <Page
        fitWindowHeight
        loading={this.state.loading}
        route={this.props.route}
        name="老生激活登记记录"
        headerToolBar={
          this.props.hasBtn && (
            <Button type="primary" onClick={this.registerStudent}>
              老生激活登记
            </Button>
          )
        }
        subHeader={<Search onSearch={this.handleSearch} filterCreateUser={false} />}
      >
        <List
          scroll={{ x: true }}
          columns={this.columns}
          dataSource={this.state.list}
          pagination={{
            current: pageNum,
            total: total,
            pageSize: pageSize,
          }}
          onChange={({ current, pageSize }) => this.loadList({ pageNum: current, pageSize })}
        />
      </Page>
    );
  }
}
const mapStateToProps = (state) => {
  const btnAuths = state.account.btnAuths;
  return {
    hasBtn: btnAuths.indexOf('OLD_STUDENT_REGISTER_RULES') > -1,
  };
};
export default connect(mapStateToProps)(TableList);
