import React from 'react';
import { Page, Wrapper, Title, List } from 'COMPONENT';
import { Row, Col } from 'antd';
import { connect } from 'dva';
import { loadTransferCustomerResult, getAuditRecordById } from 'SERVICE/customer';
import { auditStatus as auditStatusDict } from './config';
import styles from './styles.module.less';
import { loadCommonDict } from 'UTIL/dict';
class RegisterDetail extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false,
      verifyList: [],
      detailData: {},
      cusOrgDict: [],
      id: props.params[`/${props.route.path}`]['id'],
    };
    this.verifyColumns = [
      { title: '日期', dataIndex: 'auditDate' },
      {
        title: '审核意见',
        dataIndex: 'auditStatusValue',
        render: (auditStatusValue) => {
          return auditStatusValue ? auditStatusDict.filter((item) => item.text === auditStatusValue)[0]['value'] : '';
        },
      },
      { title: '审核备注', dataIndex: 'auditRemark' },
      { title: '审核人', dataIndex: 'auditUserName' },
    ];
  }
  componentDidMount() {
    this.setState({ loading: true });
    Promise.all([
      loadTransferCustomerResult({ id: this.state.id }),
      getAuditRecordById({ parentId: this.state.id, type: 'transfer' }),
      loadCommonDict('CUS_ORG', 'INTRODUCE'),
    ]).then(([res1, res2, res3]) => {
      if (res1.resultCode === 0) {
        this.setState({ detailData: res1.result || [] });
      }
      if (res2.resultCode === 0) {
        this.setState({
          verifyList: res2.data || [],
        });
      }
      if (res3) {
        this.setState(
          {
            cusOrgDict: res3,
          },
          () => {
            // 处理介绍类型
            const { detailData } = this.state;
            detailData.cusOrgName = res3.filter((item) => item.val === detailData.cusOrg)
              ? res3.filter((item) => item.val === detailData.cusOrg)[0]['text']
              : '';
            this.setState({ detailData });
          },
        );
      }
      this.setState({ loading: false });
    });
  }
  render() {
    const { detailData } = this.state;
    const {
      customerName,
      studentName,
      contact,
      auditStatus,
      intentionCampusName,
      remark,
      introducer,
      introducerContact,
      bonusBelongName,
      customerBonusRemark,
      introduceBonusRemark,
      belongBonusRemark,
      studentSchoolName,
      studentGradeName,
      cusOrgName,
      bonusBelongOrgInfo,
    } = detailData;
    return (
      <Page loading={this.state.loading} name="老生激活登记详情" route={this.props.route}>
        <Wrapper bodyFitHeight autoScrollBar>
          <Title>被激活老生信息</Title>
          <Row className={styles.row}>
            <Col className={styles.column} span={2}>
              家长姓名
            </Col>
            <Col className={styles.column} span={2}>
              {customerName}
            </Col>
            <Col className={styles.column} span={2} offset={2}>
              学生姓名
            </Col>
            <Col className={styles.column} span={2}>
              {studentName}
            </Col>
          </Row>
          <Row className={styles.row}>
            <Col className={styles.column} span={2}>
              联系方式
            </Col>
            <Col className={styles.column} span={2}>
              {contact}
            </Col>
            <Col className={styles.column} span={2} offset={2}>
              就读学校
            </Col>
            <Col className={styles.column} span={2}>
              {studentSchoolName}
            </Col>
          </Row>
          <Row className={styles.row}>
            <Col className={styles.column} span={2}>
              学生年级
            </Col>
            <Col className={styles.column} span={2}>
              {studentGradeName}
            </Col>
            <Col className={styles.column} span={2} offset={2}>
              介绍类型
            </Col>
            <Col className={styles.column} span={2}>
              {cusOrgName}
            </Col>
          </Row>
          <Row className={styles.row}>
            <Col className={styles.column} span={2}>
              意向校区
            </Col>
            <Col className={styles.column} span={2}>
              {intentionCampusName}
            </Col>
            <Col className={styles.column} span={2} offset={2}>
              审核状态
            </Col>
            <Col className={styles.column} span={2}>
              {auditStatus ? auditStatusDict.filter((item) => item.text === auditStatus)[0]['value'] : ''}
            </Col>
          </Row>
          <Row className={styles.row}>
            <Col className={styles.column} span={2}>
              客户情况
            </Col>
            <Col className={styles.column} span={2}>
              {remark}
            </Col>
          </Row>

          <Title>介绍人信息</Title>
          <Row className={styles.row}>
            <Col className={styles.column} span={2}>
              谁介绍
            </Col>
            <Col className={styles.column} span={2}>
              {introducer}
            </Col>
            <Col className={styles.column} span={2} offset={2}>
              联系方式
            </Col>
            <Col className={styles.column} span={2}>
              {introducerContact}
            </Col>
          </Row>

          <Title>奖金归属</Title>
          <Row className={styles.row}>
            <Col className={styles.column} span={2}>
              归属人
            </Col>
            <Col className={styles.column} span={2}>
              {bonusBelongName}
            </Col>
            <Col className={styles.column} span={10}>
              {bonusBelongOrgInfo}
            </Col>
          </Row>

          <Title>奖励备注内容</Title>
          <Row className={styles.row}>
            <Col className={styles.column} span={2}>
              被介绍人奖励
            </Col>
            <Col className={styles.column} span={10}>
              {customerBonusRemark}
            </Col>
          </Row>
          <Row className={styles.row}>
            <Col className={styles.column} span={2}>
              介绍人奖励
            </Col>
            <Col className={styles.column} span={10}>
              {introduceBonusRemark}
            </Col>
          </Row>
          <Row className={styles.row}>
            <Col className={styles.column} span={2}>
              归属员工奖励
            </Col>
            <Col className={styles.column} span={10}>
              {belongBonusRemark}
            </Col>
          </Row>

          <Title>审核记录</Title>
          <Row className={styles.row}>
            <Col>
              <List columns={this.verifyColumns} dataSource={this.state.verifyList} />
            </Col>
          </Row>
        </Wrapper>
      </Page>
    );
  }
}
const mapStateToProps = (state) => {
  return {
    params: state.main.pageParams,
  };
};
export default connect(mapStateToProps)(RegisterDetail);
