import React from 'react';
import Modal from 'COMPONENT/Modal';
import styles from './styles.less';
import Select from 'COMPONENT/Select';
import FormItem from 'COMPONENT/FormItem';
import TextArea from 'COMPONENT/TextArea';
import Radio from 'COMPONENT/Radio';
import { loadConsultantDict } from 'PAGE/common/loaders';
import classnames from 'classnames';
import loaders from './loaders';

const formCfg = {
  labelWidth: 120,
  style: {},
};

class TransferInAuditInfoModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 等待状态
      receiveStatus: null,
      receiveReason: null,
      newDeliverTarget: null,
      deliverTargetDict: [],
    };
  }

  componentDidMount() {
    this.init();
  }

  init = async () => {
    this.setState({ loading: true });
    const { transferRecord } = this.props;
    const promises = [loadConsultantDict(), loaders.auditTransferInInfo({ id: transferRecord.id })];
    const [deliverTargetDict, transferInInfo] = await Promise.all(promises);
    const isReceive = +transferInInfo.data.receiveStatus === 1 || +transferInInfo.data.receiveStatus === 3;
    this.setState({
      deliverTargetDict,
      receiveStatus: isReceive,
      receiveReason: transferInInfo.data.receiveReason,
      newDeliverTarget: transferInInfo.data.newDeliverTarget,
      loading: false,
    });
  };

  render() {
    const { onClose } = this.props;
    const { loading, receiveStatus, receiveReason, newDeliverTarget, deliverTargetDict } = this.state;
    const modalCls = {
      [styles.wrap]: 1,
      [styles.modal]: 1,
    };
    return (
      <Modal
        className={classnames(modalCls)}
        loading={loading}
        title="审核转校申请"
        width={514}
        okBtnShow={false}
        cancelText="关闭"
        onClose={onClose}
      >
        <div>
          <FormItem name="审核意见">
            <Radio
              checked={receiveStatus === true}
              onChange={() => this.setState({ receiveStatus: true })}
              disabled={true}
            >
              通过
            </Radio>
            <Radio
              checked={receiveStatus === false}
              onChange={() => this.setState({ receiveStatus: false })}
              style={{ marginLeft: 20 }}
              disabled={true}
            >
              不通过
            </Radio>
          </FormItem>
          <FormItem {...formCfg} name="备注">
            <TextArea
              rows={4}
              value={receiveReason}
              onChange={(e) => this.setState({ receiveReason: e.target.value })}
              disabled={true}
            />
          </FormItem>
          {receiveStatus === true && (
            <FormItem {...formCfg} name="选择分配对象">
              <Select
                placeholder="请选择分配人"
                dict={deliverTargetDict}
                value={newDeliverTarget}
                onChange={(val) => this.setState({ newDeliverTarget: val })}
                disabled={true}
              />
            </FormItem>
          )}
        </div>
      </Modal>
    );
  }
}

export default TransferInAuditInfoModal;
