import React from 'react';
import Modal from 'COMPONENT/Modal';
import styles from './styles.less';
import Select from 'COMPONENT/Select';
import FormItem from 'COMPONENT/FormItem';
import TextArea from 'COMPONENT/TextArea';
import Radio from 'COMPONENT/Radio';
import { loadConsultantDict } from 'PAGE/common/loaders';
import classnames from 'classnames';
import loaders from './loaders';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { getCommonErrorTips } from 'COMPONENT/utils/form';
import moment from 'moment';

const formCfg = {
  labelWidth: 120,
  style: {},
};

const checkLen = (value) => {
  value = value || '';
  if (value.length > 200) {
    return `请勿超过200字`;
  }
  return null;
};
class TransferInAuditModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 等待状态
      receiveStatus: null,
      receiveReason: null,
      newDeliverTarget: null,
      deliverTargetDict: [],
      checking: false,
    };
    this.errors = {};
  }

  componentDidMount() {
    this.init();
  }

  init = async () => {
    this.setState({ loading: true });
    const promises = [loadConsultantDict()];
    const [deliverTargetDict] = await Promise.all(promises);
    this.setState({
      deliverTargetDict,
      loading: false,
    });
  };

  onOk = async () => {
    const { transferRecord, onConfirm, onClose, onError } = this.props;
    const { receiveStatus, receiveReason, newDeliverTarget, checking } = this.state;
    this.setState({ loading: true });
    try {
      if (!checking) {
        this.setState({ checking: true });
      }
      const errorTips = getCommonErrorTips(this.errors);
      if (errorTips) {
        this.modal.showOkTips(errorTips);
        this.setState({ loading: false });
        return;
      }
      if (receiveStatus === null) {
        this.modal.showOkTips('请补充审核意见');
        this.setState({ loading: false });
        return;
      }
      if (receiveStatus === true && !newDeliverTarget) {
        this.modal.showOkTips('请补充分配对象');
        this.setState({ loading: false });
        return;
      }
      const params = {
        id: transferRecord.id,
        newDeliverTarget: receiveStatus ? newDeliverTarget : null,
        receiveReason: receiveReason,
        receiveStatus: receiveStatus ? '1' : '2',
      };
      await loaders.auditTransferInApply(params);
      showMessage('操作成功', 'success');
      onConfirm({
        receiveStatus: receiveStatus ? 1 : 2,
        receiveTime: moment().format('YYYY-MM-DD HH:mm:ss'),
      });
      onClose();
    } catch (e) {
      console.log(e);
      if (+e.code === -1) {
        onError(e.text);
        onClose();
        return;
      }
      showMessage('操作失败:' + e.text, 'error');
      this.setState({ loading: false });
    }
  };

  errorChange = (key, error) => {
    this.errors[key] = error;
  };

  render() {
    const { onClose } = this.props;
    const { loading, receiveStatus, receiveReason, newDeliverTarget, deliverTargetDict, checking } = this.state;
    const modalCls = {
      [styles.wrap]: 1,
      [styles.modal]: 1,
    };
    return (
      <Modal
        className={classnames(modalCls)}
        loading={loading}
        title="审核转校申请"
        width={514}
        onOk={this.onOk}
        onClose={onClose}
        ref={(c) => (this.modal = getWrappedInstance(c))}
      >
        <div>
          <FormItem name="审核意见" required="true">
            <Radio checked={receiveStatus === true} onChange={() => this.setState({ receiveStatus: true })}>
              通过
            </Radio>
            <Radio
              checked={receiveStatus === false}
              onChange={() => this.setState({ receiveStatus: false })}
              style={{ marginLeft: 20 }}
            >
              不通过
            </Radio>
          </FormItem>
          <FormItem {...formCfg} name="备注">
            <TextArea
              placeholder="请输入"
              rows={4}
              value={receiveReason}
              onChange={(e) => this.setState({ receiveReason: e.target.value })}
              validator={{
                rules: [checkLen],
                silent: !checking,
                onErrorChange: (e) => this.errorChange('receiveReason', e),
              }}
              maxLength={200}
              wordCount={true}
            />
          </FormItem>
          {receiveStatus === true && (
            <FormItem {...formCfg} name="选择分配对象" required="true">
              <Select
                placeholder="请选择分配人"
                dict={deliverTargetDict}
                value={newDeliverTarget}
                onChange={(val) => this.setState({ newDeliverTarget: val })}
                showSearch
              />
            </FormItem>
          )}
        </div>
      </Modal>
    );
  }
}

export default TransferInAuditModal;
