import React from 'react';
import DatePicker from 'COMPONENT/DatePicker';
import List from 'COMPONENT/List';
import SearchBar from 'COMPONENT/SearchBar';
import Input from 'COMPONENT/Input';
import Select from 'COMPONENT/Select';
import TextLink from 'COMPONENT/TextLink';
import Modal from 'COMPONENT/Modal';
import loaders from './loaders';
import { loadCommonDict, loadAllBranchCampusDict } from 'UTIL/dict';
import moment from 'moment';
import { auditStatusDict, receiveStatusDict, receiveStatusObj, auditStatusObj } from './dict';
import TransferOutAuditModal from './TransferOutAuditModal';
import styles from './styles.less';

class TransferOutAudit extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      searchExpand: false,
      listLoading: true,
      dataSource: [],
      total: 0,
      current: 1,
      pageSize: 10,
      searchConditions: {},

      studentName: null,
      studentContact: null,
      startTime: null,
      endTime: null,
      receiveCampusId: null,
      resEntrance: null,

      resEntranceDict: [],
      campusDict: [],

      transferOutAuditModalVisible: false,
      transferRecord: null,
      auditErrModalVisible: false,
      auditErr: null,
    };
  }

  onSearch = async (val) => {
    const page = 1;
    this.setState({ listLoading: true });
    const { studentName, studentContact, startTime, endTime, receiveCampusId, resEntrance } = this.state;
    const params = {
      startTime: startTime ? moment(startTime).startOf('day').format('YYYY-MM-DD HH:mm:ss') : null,
      endTime: endTime ? moment(endTime).endOf('day').format('YYYY-MM-DD HH:mm:ss') : null,
      receiveCampusId,
      resEntrance,
      studentName: studentName ? studentName.trim() : null,
      studentContact: studentContact ? studentContact.trim() : null,
    };
    const res = await loaders.loadTransferOutAuditList({
      ...params,
      pageSize: this.state.pageSize,
      pageNo: page,
    });
    this.setState({
      searchConditions: params,
      current: page,
      total: res.rowCount,
      dataSource: res.datas,
      listLoading: false,
    });
    return;
  };

  onChange = async ({ current = this.state.current, pageSize = this.state.pageSize }) => {
    this.setState({ listLoading: true });
    const res = await loaders.loadTransferOutAuditList({
      ...this.state.searchConditions,
      pageSize: pageSize,
      pageNo: current,
    });
    this.setState({
      current: res.pageNo,
      pageSize: res.pageSize,
      total: res.rowCount,
      dataSource: res.datas,
      listLoading: false,
    });
    return;
  };

  onReset = () => {
    this.setState({
      studentName: null,
      studentContact: null,
      startTime: null,
      endTime: null,
      receiveCampusId: null,
      resEntrance: null,
    });
  };

  initContent = () => {
    return this.init();
  };

  init = async () => {
    this.setState({ listLoading: true });
    const promises = [loadAllBranchCampusDict(), loadCommonDict('RES_ENTRANCE'), this.onSearch()];
    const [campusDict, resEntranceDict] = await Promise.all(promises);
    this.setState({
      resEntranceDict: resEntranceDict,
      campusDict: campusDict,
      listLoading: false,
    });
  };

  getDisabledStartDate = (startDate) => {
    const { endDate } = this.state;
    if (!startDate || !endDate) {
      return false;
    }
    return moment(startDate).valueOf() > moment(endDate).valueOf();
  };

  getDisabledEndDate = (endDate) => {
    const { startDate } = this.state;
    if (!endDate || !startDate) {
      return false;
    }
    return moment(endDate).valueOf() < moment(startDate).valueOf();
  };

  updateRow = (raw, changed) => {
    this.setState({ listLoading: true });
    const idx = this.state.dataSource.findIndex((ele) => ele.id === raw.id);
    const dataSource = this.state.dataSource.map((row, i) => {
      if (i === idx) {
        return { ...row, ...changed };
      }
      return row;
    });
    this.setState({
      listLoading: false,
      dataSource,
    });
    return { ...raw, ...changed };
  };

  onError = async (errMsg) => {
    this.setState({
      listLoading: true,
      auditErrModalVisible: true,
      auditErr: errMsg,
    });
    const res = await loaders.loadTransferOutAuditList({
      ...this.state.searchConditions,
      pageSize: this.state.pageSize,
      pageNo: this.state.current,
    });
    this.setState({
      current: res.pageNo,
      pageSize: res.pageSize,
      total: res.rowCount,
      dataSource: res.datas,
      listLoading: false,
    });
  };

  render() {
    const columns = [
      { title: '学生姓名', dataIndex: 'studentName' },
      { title: '联系方式', dataIndex: 'studentContact' },
      { title: '资源入口', dataIndex: 'resEntranceName' },
      { title: '申请转入学习中心', dataIndex: 'receiveCampusName' },
      { title: '申请转校时间', dataIndex: 'applyTime' },
      {
        title: '审核状态',
        dataIndex: 'auditStatus',
        render: (text, record, i) => {
          const { auditFailReason } = record;
          return (
            <div>
              <div>
                <span>{auditStatusObj['' + text]}</span>
              </div>
              {+text === 3 && (
                <div>
                  <span>（{auditFailReason}）</span>
                </div>
              )}
            </div>
          );
        },
      },
      { title: '转校原因', dataIndex: 'applyReason' },
      {
        title: '转校情况',
        dataIndex: 'receiveStatus',
        render: (text, record, i) => {
          const { receiveFailReason, auditStatus } = record;
          if (+auditStatus !== 1) {
            return '';
          }
          return (
            <div>
              <div>
                <span>{receiveStatusObj['' + text]}</span>
              </div>
              {text === 3 && (
                <div>
                  <span>（{receiveFailReason}）</span>
                </div>
              )}
            </div>
          );
        },
      },
      {
        title: '操作',
        fixed: 'right',
        key: 'action',
        render: (text, record, index) => {
          return (
            <span>
              {+record.auditStatus === 0 && (
                <TextLink
                  className="list-btn"
                  onClick={() => this.setState({ transferOutAuditModalVisible: true, transferRecord: record })}
                >
                  审核
                </TextLink>
              )}
            </span>
          );
        },
      },
    ];

    const {
      searchExpand,
      dataSource,
      total,
      current,
      pageSize,
      listLoading,

      studentName,
      studentContact,
      startTime,
      endTime,
      receiveCampusId,
      resEntrance,

      campusDict,
      resEntranceDict,

      transferOutAuditModalVisible,
      transferRecord,
      auditErrModalVisible,
      auditErr,
    } = this.state;

    return (
      <div className="flexWrap">
        <SearchBar
          expand={searchExpand}
          onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
          onSearch={this.onSearch}
          onReset={this.onReset}
        >
          <Input
            placeholder="学生姓名"
            value={studentName}
            onChange={(e) => this.setState({ studentName: e.target.value })}
          />
          <Input
            placeholder="联系方式"
            value={studentContact}
            onChange={(e) => this.setState({ studentContact: e.target.value })}
          />
          <DatePicker
            placeholder="申请开始时间（起）"
            disabledDate={this.getDisabledStartDate}
            value={startTime}
            onChange={(val) => this.setState({ startTime: val })}
          />
          <DatePicker
            placeholder="申请结束时间（止）"
            disabledDate={this.getDisabledEndDate}
            value={endTime}
            onChange={(val) => this.setState({ endTime: val })}
          />
          <Select
            showSearch
            placeholder="请选择学习中心"
            dict={campusDict}
            value={receiveCampusId}
            onChange={(val) => this.setState({ receiveCampusId: val })}
          />
          <Select
            placeholder="资源入口"
            dict={resEntranceDict}
            value={resEntrance}
            onChange={(val) => this.setState({ resEntrance: val })}
          />
        </SearchBar>
        <List
          dataSource={dataSource}
          columns={columns}
          pagination={{
            current,
            total,
            pageSize,
          }}
          loading={listLoading}
          onChange={this.onChange}
        />
        {transferOutAuditModalVisible && (
          <TransferOutAuditModal
            transferRecord={transferRecord}
            onConfirm={(changed) => this.updateRow(transferRecord, changed)}
            onClose={() => this.setState({ transferOutAuditModalVisible: false })}
            onError={this.onError}
          />
        )}
        {auditErrModalVisible && (
          <Modal
            width={514}
            onOk={() => this.setState({ auditErrModalVisible: false })}
            cancelBtnShow={false}
            okText="知道了"
            className={styles.customModal}
          >
            <div style={{ padding: '40px 0 5px 0', textAlign: 'center' }}>{auditErr}</div>
          </Modal>
        )}
      </div>
    );
  }
}

export default TransferOutAudit;
