import React from 'react';
import Modal from 'COMPONENT/Modal';
import styles from './styles.less';
import FormItem from 'COMPONENT/FormItem';
import TextArea from 'COMPONENT/TextArea';
import Radio from 'COMPONENT/Radio';
import classnames from 'classnames';
import loaders from './loaders';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { getCommonErrorTips } from 'COMPONENT/utils/form';
import moment from 'moment';

const formCfg = {
  labelWidth: 80,
};

const checkLen = (value) => {
  value = value || '';
  if (value.length > 200) {
    return `请勿超过200字`;
  }
  return null;
};

class TransferOutAuditModal extends React.Component {
  constructor(props) {
    super(props);
    const { transferRecord } = this.props;
    this.state = {
      loading: false, // {boolean} 等待状态

      reason: transferRecord.applyReason, // {string} 转校理由

      auditStatus: null,

      checking: false,
    };
    this.errors = {};
  }

  onOk = async () => {
    const { transferRecord, onConfirm, onClose, onError } = this.props;
    const { checking, reason, auditStatus } = this.state;
    this.setState({ loading: true });
    try {
      if (!checking) {
        this.setState({ checking: true });
      }
      const errorTips = getCommonErrorTips(this.errors);
      if (errorTips) {
        this.modal.showOkTips(errorTips);
        this.setState({ loading: false });
        return;
      }
      if (auditStatus === null) {
        this.modal.showOkTips('请选择审核结果');
        this.setState({ loading: false });
        return;
      }
      if (!reason) {
        this.modal.showOkTips('请填写转校原因');
        this.setState({ loading: false });
        return;
      }
      const params = {
        applyReason: reason,
        auditStatus: auditStatus ? '1' : '2',
        id: transferRecord.id,
      };
      await loaders.auditTransferOutApply(params);
      showMessage('操作成功', 'success');
      onConfirm({
        applyReason: reason,
        auditStatus: auditStatus ? 1 : 2,
        auditTime: moment().format('YYYY-MM-DD HH:mm:ss'),
      });
      onClose();
    } catch (e) {
      console.log(e);
      if (+e.code === -1) {
        onError(e.text);
        onClose();
        return;
      }
      showMessage('操作失败:' + e.text, 'error');
      this.setState({ loading: false });
    }
  };

  errorChange = (key, error) => {
    this.errors[key] = error;
  };

  render() {
    const { onClose } = this.props;
    const { loading, reason, auditStatus, checking } = this.state;
    const modalCls = {
      [styles.wrap]: 1,
      [styles.modal]: 1,
    };
    return (
      <Modal
        className={classnames(modalCls)}
        loading={loading}
        title="审核转校申请"
        width={514}
        onOk={this.onOk}
        onClose={onClose}
        ref={(c) => (this.modal = getWrappedInstance(c))}
      >
        <div>
          <FormItem {...formCfg} name="转校原因">
            <TextArea
              placeholder="请输入"
              rows={4}
              value={reason}
              onChange={(e) => this.setState({ reason: e.target.value })}
              validator={{
                rules: [checkLen],
                silent: !checking,
                onErrorChange: (e) => this.errorChange('reason', e),
              }}
              maxLength={200}
              wordCount={true}
            />
          </FormItem>
          <FormItem name="审核意见" required="true">
            <Radio checked={auditStatus === true} onChange={() => this.setState({ auditStatus: true })}>
              通过
            </Radio>
            <Radio
              checked={auditStatus === false}
              onChange={() => this.setState({ auditStatus: false })}
              style={{ marginLeft: 20 }}
            >
              不通过
            </Radio>
          </FormItem>
        </div>
      </Modal>
    );
  }
}

export default TransferOutAuditModal;
