import React from 'react';
import styles from './styles.less';
import Section from 'COMPONENT/Section';
import DropdownMenu from 'COMPONENT/DropdownMenu';
import Page from 'COMPONENT/Page';
import TransferInAudit from './TransferInAudit';
import TransferOutAudit from './TransferOutAudit';

const TRANSFER_OUT = '0';
const TRANSFER_IN = '1';
const sectionListDict = [
  { val: TRANSFER_OUT, text: '转出申请' },
  { val: TRANSFER_IN, text: '转入申请' },
];

class TransferCampusAudit extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 加载状态

      activeCusList: TRANSFER_OUT, // {boolean} 当前激活状态的客户列表
      isManager: true,
    };
    this.sectionLists = {}; // {object} 客户列表信息，{ id: { ref, inited } }
    sectionListDict.forEach((d) => {
      this.sectionLists[d.val] = { ref: null, inited: false };
    });
  }

  componentDidMount() {
    this.setState({ loading: true });
    setTimeout(() => {
      Promise.all([this.sectionLists[TRANSFER_OUT].ref.initContent()]).then(() => {
        this.sectionLists[TRANSFER_OUT].inited = true;
        this.setState({ loading: false });
      });
    }, 0);
  }

  getCusListSecTitle = () => {
    const { activeCusList } = this.state;
    const menus = sectionListDict.map((c) => {
      return { text: c.text, onClick: () => this.changeCusList(c.val), hide: c.val === activeCusList };
    });
    return (
      <div className={styles.cusListDropdownMenu}>
        <DropdownMenu menus={menus} text={sectionListDict.find((cd) => cd.val === activeCusList).text} hasArrow />
      </div>
    );
  };

  changeCusList = (type) => {
    if (this.sectionLists[type].inited) {
      this.setState({ activeCusList: type });
    } else {
      this.setState({ loading: true });
      this.sectionLists[type].ref.initContent().then(() => {
        this.sectionLists[type].inited = true;
        this.setState({ loading: false, activeCusList: type });
      });
    }
  };

  render() {
    const { dispatch } = this.props;
    const { activeCusList, loading } = this.state;
    return (
      <Page route={this.props.route} title={false} className={styles.page}>
        <Section title={this.getCusListSecTitle()} loading={loading}>
          {sectionListDict.map((c) => {
            let listComp;
            if (c.val === TRANSFER_OUT) {
              listComp = <TransferOutAudit ref={(comp) => (this.sectionLists[c.val].ref = comp)} dispatch={dispatch} />;
            } else if (c.val === TRANSFER_IN) {
              listComp = <TransferInAudit ref={(comp) => (this.sectionLists[c.val].ref = comp)} dispatch={dispatch} />;
            }
            return (
              <div key={c.val} style={{ display: c.val === activeCusList ? 'block' : 'none' }}>
                {listComp}
              </div>
            );
          })}
        </Section>
      </Page>
    );
  }
}

export default TransferCampusAudit;
