import {
  listStudentTransferApplyForCampusAudit,
  listStudentTransferApplyForCampusReceive,
  getStudentTransferApplyDetailInfo,
} from 'SERVICE/consultant';
import request from 'UTIL/http/fetch';
import { urlBase } from 'CONST/config';

async function httpPost({ url, params, dataType }) {
  let options = {
    method: 'POST',
  };
  if (dataType === 'json') {
    options = {
      ...options,
      body: JSON.stringify(params),
      headers: {
        'Content-Type': 'application/json;charset=utf-8',
      },
    };
  } else {
    options = {
      ...options,
      body: makeQueryString(params),
      headers: {
        'Content-Type': 'application/x-www-form-urlencoded',
      },
    };
  }
  const res = await request(urlBase + url, options);
  return res;
}

const loadTransferOutAuditList = async (params) => {
  const data = await listStudentTransferApplyForCampusAudit(params);
  if (data.resultCode !== 0) {
    throw new Error(data.resultMessage);
  }
  return data;
};

const loadTransferInAuditList = async (params) => {
  const data = await listStudentTransferApplyForCampusReceive(params);
  if (data.resultCode !== 0) {
    throw new Error(data.resultMessage);
  }
  return data;
};

const auditTransferOutApply = async (params) => {
  const { data } = await httpPost({
    params,
    url: 'CustomerAction/v2/auditStudentTransferApply.do',
    method: 'POST',
    dataType: 'json',
  });
  console.log(data);
  if (data.resultCode !== 0) {
    throw {
      text: data.resultMessage,
      code: data.resultCode,
    };
  }
  return data;
};

const auditTransferInApply = async (params) => {
  const { data } = await httpPost({
    params,
    url: 'CustomerAction/v2/receiveStudentTransferApply.do',
    method: 'POST',
    dataType: 'json',
  });
  if (data.resultCode !== 0) {
    throw {
      text: data.resultMessage,
      code: data.resultCode,
    };
  }
  return data;
};

const auditTransferInInfo = async (params) => {
  const data = await getStudentTransferApplyDetailInfo(params);
  if (data.resultCode !== 0) {
    throw new Error(data.resultMessage);
  }
  return data;
};

export default {
  loadTransferOutAuditList,
  loadTransferInAuditList,
  auditTransferOutApply,
  auditTransferInApply,
  auditTransferInInfo,
};
