import React from 'react';
import { connect } from 'dva';
import PropTypes from 'prop-types';
import { Modal, Select, TextArea } from 'COMPONENT';
import { Form, Row, Col, Radio, message } from 'antd';
import { errorsHandle } from 'UTIL/common';
import debounce from 'lodash/debounce';
import { searchUser, auditTransferCustomer, getTransferTargetByCampus } from 'SERVICE/customer';
const FormItem = Form.Item;
const RadioGroup = Radio.Group;
const formItemCol = {
  labelCol: { span: 8 },
  wrapperCol: { span: 15 },
};
class VerifyModal extends React.Component {
  static propTypes = {
    onSuccess: PropTypes.func.isRequired,
    onCancel: PropTypes.func,
    id: PropTypes.string.isRequired,
  };
  constructor(props) {
    super(props);
    this.state = {
      bonusPersonDict: null,
      loading: false,
    };
  }

  componentDidMount() {
    this.initOptions();
  }

  initOptions = async () => {
    // 获取咨询师
    const data = await getTransferTargetByCampus({
      showBelong: 'transfer_introduce',
      'job[]': 'CONSULTOR',
      transferId: this.props.id,
    });
    if (data) {
      const users = [];
      let obj = {};
      data.forEach((item) => {
        item.text = item.userName;
        item.val = item.userId;
        if (obj.label === item.orgName) obj.options.push(item);
        else {
          obj = {};
          obj['label'] = item.orgName;
          obj.options = [item];
          users.push(obj);
        }
      });
      this.setState({
        bonusPersonDict: users,
      });
    }
  };

  handleSubmit = () => {
    this.props.form.validateFields(async (errors, form) => {
      if (!errors) {
        const { resultCode, resultMessage } = await auditTransferCustomer({
          id: this.props.id,
          ...form,
        });
        if (resultCode === 0 || resultCode === -2) {
          this.props.onSuccess();
        } else {
          message.warn(resultMessage);
        }
      }
    });
  };

  render() {
    const { getFieldDecorator, getFieldValue } = this.props.form;
    const { bonusPersonDict } = this.state;
    return (
      <Modal onOk={this.handleSubmit} onCancel={this.props.onCancel} title="老生激活登记审核" widthSize="S">
        <Form>
          <Row justify="start" type="flex">
            <Col span={24}>
              <FormItem label="审核意见" {...formItemCol}>
                {getFieldDecorator('auditStatus', {
                  initialValue: 'PASS',
                  rules: [{ required: true, message: '请选择审核意见' }],
                })(
                  <RadioGroup>
                    <Radio value="PASS">通过</Radio>
                    <Radio value="NOT_PASS">不通过</Radio>
                  </RadioGroup>,
                )}
              </FormItem>
            </Col>
          </Row>
          {getFieldValue('auditStatus') == 'PASS' ? (
            <Row justify="start" type="flex">
              <Col span={24}>
                <FormItem label="客户分配" {...formItemCol}>
                  {getFieldDecorator('deliverTarget', {
                    rules: [{ required: true, message: '请选择客户分配' }],
                  })(
                    <Select
                      placeholder="请选择"
                      dict={bonusPersonDict}
                      showSearch
                      searchPlaceholder="请先输入员工名称查找"
                      searchNotFoundTips="没有找到该员工"
                    />,
                  )}
                </FormItem>
              </Col>
            </Row>
          ) : (
            ''
          )}
          <Row justify="start" type="flex">
            <Col span={24}>
              <FormItem label="审核备注" {...formItemCol}>
                {getFieldDecorator('remark', {
                  rules: [{ transform: (val) => (val ? val.trim() : val) }],
                })(<TextArea maxLength={50} wordCount={true} />)}
              </FormItem>
            </Col>
          </Row>
        </Form>
      </Modal>
    );
  }
}

export default connect()(Form.create()(VerifyModal));
