import React from 'react';

import Wrapper from 'COMPONENT/Wrapper';
import Page from 'COMPONENT/Page';
import List from 'COMPONENT/List';

import moment from 'moment';

import { errorsHandle } from 'SRC/utils/common';

import * as emergencyStateApi from 'SERVICE/emergencyState';

import styles from './styles.module.less';

export default class EmergencyStateContractList extends React.Component {
  state = {
    loading: false,
    total: 0,
    pageNum: 1,
    pageSize: 30,
    dataSource: [],
    scrollY: 300,
  };

  constructor(props) {
    super(props);

    this.columns = [
      {
        title: '分公司',
        dataIndex: 'branchName',
        width: 180,
      },
      { title: '学生姓名', dataIndex: 'studentName', width: 160 },
      { title: '客户电话', dataIndex: 'phone', width: 100 },
      { title: '合同年级', dataIndex: 'grade', width: 80 },
      {
        title: '签约日期',
        dataIndex: 'signDate',
        width: 160,
      },
      { title: '产品类型', dataIndex: 'productType', width: 100 },
      { title: '产品名称', dataIndex: 'productName', width: 200 },
      { title: '产品单价(元)', dataIndex: 'productPrice', width: 100 },
      { title: '产品总时长（小时）', dataIndex: 'productHours', width: 160 },
      {
        title: '产品总金额',
        dataIndex: 'totalPrice',
        width: 160,
        render: (_, record) =>
          record.productPrice && record.productHours ? record.productPrice * record.productHours : 0,
      },
    ];
  }

  componentDidMount() {
    this.loadData();
    this.setState({ scrollY: Math.max(document.body.clientHeight - 298, 300) });
  }

  loadData = async () => {
    this.setState({ loading: true });

    const { pageNum, pageSize } = this.state;
    const result = await emergencyStateApi
      .getEmergencyStateContractList({
        pageNum,
        pageSize,
      })
      .catch(errorsHandle());

    if (!result || !result.data.list.length) {
      this.setState({ total: 0, dataSource: [], loading: false });
    } else {
      this.setState({ total: result.data.total, dataSource: result.data.list, loading: false });
    }
  };

  handleChangePage = ({ current, pageSize }) => {
    this.setState({ pageNum: current, pageSize }, this.loadData);
  };

  render() {
    const { total, loading, pageNum, pageSize, dataSource } = this.state;

    return (
      <div className={styles.wrapper}>
        <p style={{ height: '50px', lineHeight: '50px' }}>家长合同列表</p>
        <List
          loading={loading}
          noFixedHeight={true}
          columns={this.columns}
          dataSource={dataSource}
          scrollY={this.state.scrollY}
          locale={{ emptyText: '请查询数据' }}
          onChange={this.handleChangePage}
          pagination={{
            current: pageNum,
            total,
            pageSize,
            defaultPageSize: 30,
            pageSizeOptions: ['10', '20', '30', '40', '50'],
          }}
        />
      </div>
    );
  }
}
