import React from 'react';
import SearchBar from 'COMPONENT/SearchBar';
import DatePicker from 'COMPONENT/DatePicker';
import Select from 'COMPONENT/Select';
import Input from 'COMPONENT/Input';
import moment from 'moment';
import { loadCommonDict, loadBranchCampusDict } from 'UTIL/dict';

const sourceTypeDict = [
  { val: 'TRANSFER_IN', text: '转入' },
  { val: 'REFUND', text: '退回账户' },
  { val: 'RECHARGE', text: '充值' },
  { val: 'HISTORY_BALANCE', text: '历史余额' },
  { val: 'CONTRACT_CHARGE', text: '合同消耗' },
  { val: 'ACCOUNT_TRANSFER_SUCCESS', text: '转出' },
  { val: 'MOENY_ACC_REFUND_SUCCESS', text: '退款' },
  { val: 'PUT_MONEY_INCOME', text: '划归收入' },
  { val: 'EXPIRED_PROD_CLEAN', text: '过季产品清理' },
  { val: 'WASH_CHARGE', text: '冲销' },
  { val: 'MARKET_ACTIVITY_RECHARGE', text: '引流充值' },
];

class EleAccFlowSearchBar extends React.Component {
  constructor(props) {
    super(props);

    const { defaultConditions } = this.props;
    this.defaultSearchConditions = Object.assign(
      {
        searchExpand: null,
        startTime: null,
        endTime: null,
        campus: null,
        sourceType: null,
        productType: null,
        contractType: null,
        contractCharacter: null,
        productName: null,
        contractId: null,
        className: null,
        studentName: null,
        flowId: null,
      },
      defaultConditions,
    );

    this.defaultDicts = {
      campusDict: [],
      productTypeDict: [],
      contractTypeDict: [],
      contractCharacterDict: [],
    };

    this.state = Object.assign(
      {
        searchExpand: false, // {boolean} 搜索栏展开状态
      },
      this.defaultSearchConditions,
      this.defaultDicts,
    );
  }

  onSearch = () => {
    const searchConditions = Object.keys(this.defaultSearchConditions).reduce((obj, ele) => {
      if (this.state[ele]) {
        obj[ele] = this.state[ele];
      }
      return obj;
    }, {});
    this.props.onSearch(searchConditions);
  };

  onReset = () => {
    this.setState({
      ...this.defaultSearchConditions,
    });
  };

  getDisabledStartDate = (endDate) => {
    return (beginDate) => {
      if (!beginDate || !endDate) {
        return false;
      }
      return moment(beginDate).valueOf() > moment(endDate).valueOf();
    };
  };

  getDisabledEndDate = (beginDate) => {
    return (endDate) => {
      if (!beginDate || !endDate) {
        return false;
      }
      return moment(endDate).valueOf() < moment(beginDate).valueOf();
    };
  };

  componentDidMount() {
    Promise.all([
      loadBranchCampusDict(),
      loadCommonDict('PRODUCT_TYPE'),
      loadCommonDict('ContractType'),
      loadCommonDict('ContractCharacter'),
    ]).then(([campusDict, productTypeDict, contractTypeDict, contractCharacterDict]) => {
      this.setState({
        campusDict,
        productTypeDict,
        contractTypeDict,
        contractCharacterDict,
      });
    });
  }

  render() {
    const { style } = this.props;
    const {
      searchExpand,
      startTime,
      endTime,
      campus,
      sourceType,
      productType,
      contractType,
      contractCharacter,
      productName,
      contractId,
      className,
      studentName,
      flowId,
      campusDict,
      productTypeDict,
      contractTypeDict,
      contractCharacterDict,
    } = this.state;

    return (
      <div style={style}>
        <SearchBar
          expand={searchExpand}
          onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
          onSearch={this.onSearch}
          onReset={this.onReset}
        >
          <DatePicker
            placeholder="开始日期"
            value={startTime}
            onChange={(val) => this.setState({ startTime: val })}
            disabledDate={this.getDisabledStartDate(endTime)}
          />
          <DatePicker
            placeholder="结束日期"
            value={endTime}
            onChange={(val) => this.setState({ endTime: val })}
            disabledDate={this.getDisabledEndDate(startTime)}
          />
          <Select
            placeholder="选择学习中心"
            showSearch
            dict={campusDict}
            value={campus}
            onChange={(val) => this.setState({ campus: val })}
          />
          <Select
            placeholder="来源"
            showSearch
            dict={sourceTypeDict}
            value={sourceType}
            onChange={(val) => this.setState({ sourceType: val })}
          />
          <Select
            placeholder="产品类型"
            showSearch
            dict={productTypeDict}
            value={productType}
            onChange={(val) => this.setState({ productType: val })}
          />
          <Select
            placeholder="合同类型"
            showSearch
            dict={contractTypeDict}
            value={contractType}
            onChange={(val) => this.setState({ contractType: val })}
          />
          <Select
            placeholder="合同性质"
            showSearch
            dict={contractCharacterDict}
            value={contractCharacter}
            onChange={(val) => this.setState({ contractCharacter: val })}
          />
          <Input
            placeholder="产品名称"
            value={productName}
            onChange={(e) => this.setState({ productName: e.target.value })}
          />
          <Input
            placeholder="合同编号"
            value={contractId}
            onChange={(e) => this.setState({ contractId: e.target.value })}
          />
          <Input
            placeholder="班级名称"
            value={className}
            onChange={(e) => this.setState({ className: e.target.value })}
          />
          <Input
            placeholder="学生姓名"
            value={studentName}
            onChange={(e) => this.setState({ studentName: e.target.value })}
          />
          <Input placeholder="流水ID" value={flowId} onChange={(e) => this.setState({ flowId: e.target.value })} />
        </SearchBar>
      </div>
    );
  }
}

export default EleAccFlowSearchBar;
