import React from 'react';
import Modal from 'COMPONENT/Modal';
import FormItem from 'COMPONENT/FormItem';
import List from 'COMPONENT/List';
import { Row, Col } from 'antd';
import loaders from './loaders';
import styles from './styles.less';
import { loadCommonDict } from 'UTIL/dict';
import 'COMPONENT/Page/styles.less';

function moneyToUpper(num) {
  let strOutput = '';
  let strUnit = '仟佰拾亿仟佰拾万仟佰拾元角分';
  num += '00';
  const intPos = num.indexOf('.');
  if (intPos >= 0) num = num.substring(0, intPos) + num.substr(intPos + 1, 2);
  strUnit = strUnit.substr(strUnit.length - num.length);
  for (let i = 0; i < num.length; i++)
    strOutput += '零壹贰叁肆伍陆柒捌玖'.substr(num.substr(i, 1), 1) + strUnit.substr(i, 1);
  return strOutput
    .replace(/零角零分$/, '')
    .replace(/零[仟佰拾]/g, '零')
    .replace(/零{2,}/g, '零')
    .replace(/零([亿|万])/g, '$1')
    .replace(/零+元/, '元')
    .replace(/亿零{0,3}万/, '亿')
    .replace(/^元/, '零元');
}

const summaryCn = {
  totalAmount: '合同总费用(元)：',
  realPaidAmount: '实际付款(元)：',
  electronicPaidAmount: '电子账户付款(元)：',
  promotinAmount: '总优惠资金(元)：',
  planAmount: '待付款资金(元)：',
  oneOnOneRemainingAmount: '一对一剩余资金(元)：',
  oneOnOneRemainingHour: '一对一剩余课时：',
  miniRemainingAmount: '小班剩余资金(元)：',
  miniRemainingHour: '小班剩余课时：',
  otmRemainingAmount: '一对多剩余资金(元)：',
  otmRemainingHour: '一对多剩余课时：',
  ecsRemainingAmount: '精英班剩余资金(元)：',
  electronicAccount: '电子账户余额(元)：',
  lectureRemainingAmount: '讲座剩余资金(元)：',
  lectureRemainingHour: '讲座剩余课时：',
  remainingAmount: '剩余资金(元)：',
  twoTeacherRemainingAmount: '双师剩余资金(元)：',
  twoTeacherRemainingHour: '双师剩余课时：',
};

class EleAccInfoModal extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: true,
      info: {},
      summary: {},
      details: [],
      sourceTypeDict: [],
    };
  }

  componentDidMount() {
    this.setState({ loading: true });
    const { studentId } = this.props;
    if (!studentId) {
      return;
    }
    Promise.all([
      loaders.getStudentAccoutInfo({ studentId: studentId }),
      loaders.getStuEleAccFlow({ studentId: studentId }),
      loadCommonDict('ECFlowSourceType'),
    ]).then(([accoutInfo, eleAccFlow, sourceTypeDict]) => {
      const info = {
        electronicAccount: accoutInfo.electronicAccount || 0,
        frozenAmount: accoutInfo.frozenAmount || 0,
        electronicAccountUpper: moneyToUpper(accoutInfo.electronicAccount) || '',
      };
      const summary = {
        totalAmount: accoutInfo.totalAmount,
        realPaidAmount: accoutInfo.realPaidAmount,
        electronicPaidAmount: accoutInfo.electronicPaidAmount,
        promotinAmount: accoutInfo.promotinAmount || 0,
        planAmount: accoutInfo.planAmount,
        oneOnOneRemainingAmount: accoutInfo.oneOnOneRemainingAmount,
        oneOnOneRemainingHour: accoutInfo.oneOnOneRemainingHour,
        miniRemainingAmount: accoutInfo.miniRemainingAmount,
        miniRemainingHour: accoutInfo.miniRemainingHour || 0,
        otmRemainingAmount: accoutInfo.otmRemainingAmount,
        otmRemainingHour: accoutInfo.otmRemainingHour,
        ecsRemainingAmount: accoutInfo.ecsRemainingAmount || 0,
        electronicAccount: accoutInfo.electronicAccount || 0,
        lectureRemainingAmount: accoutInfo.lectureRemainingAmount || 0,
        lectureRemainingHour: accoutInfo.lectureRemainingHour || 0,
        remainingAmount: accoutInfo.remainingAmount,
        twoTeacherRemainingAmount: accoutInfo.twoTeacherRemainingAmount,
        twoTeacherRemainingHour: accoutInfo.twoTeacherRemainingHour,
      };
      this.setState({
        loading: false,
        info,
        summary,
        details: eleAccFlow,
        sourceTypeDict,
      });
    });
  }

  renderSummary = () => {
    const summaryList = Object.keys(summaryCn);
    const { summary } = this.state;
    return summaryList.map((ele) => {
      return (
        <Col key={ele} span={8} className={styles.customFormItem}>
          <FormItem name={summaryCn[ele] || ''}>{summary[ele] || ''}</FormItem>
        </Col>
      );
    });
  };

  commonRender = (val, row, i) => {
    return val || '-';
  };

  render() {
    const detailColumns = [
      { title: '合同编号', dataIndex: 'contractId', render: this.commonRender },
      { title: '产品名称', dataIndex: 'productName', render: this.commonRender },
      { title: '产品类型', dataIndex: 'productType', render: this.commonRender },
      { title: '来源', dataIndex: 'source', dict: this.state.sourceTypeDict },
      { title: '入账金额', dataIndex: 'incomeMoney' },
      { title: '已用金额', dataIndex: 'consumeMoney' },
      { title: '可用金额', dataIndex: 'resumeMoney' },
    ];

    const { onClose, studentName } = this.props;
    const { loading, info, details } = this.state;
    return (
      <Modal
        loading={loading}
        title={<h2 style={{ fontSize: '18px' }}>{`${studentName || ''}学生账务信息`}</h2>}
        onCancel={onClose}
        cancelText={'关闭'}
        okBtnShow={false}
        closable={true}
        widthSize="l"
        className={styles.customModal}
      >
        <Row key="info">
          <Col span={24}>
            <h3 className={styles.header}>账户信息</h3>
          </Col>
          <Col span={24} className={styles.customFormItem + ' ' + styles.info}>
            <div>
              <FormItem name="电子账户余额：">
                <span>{info.electronicAccount || 0}</span>元
                {info.frozenAmount ? (
                  <span style={{ marginLeft: '16px' }}>
                    （冻结： <span>{info.frozenAmount}</span>元）
                  </span>
                ) : (
                  ''
                )}
              </FormItem>
            </div>
            <div>
              <FormItem name="金额大写：">
                <span>{info.electronicAccountUpper || ''}</span>
              </FormItem>
            </div>
          </Col>
        </Row>
        <div className={styles.hr}></div>
        <Row key="summary">
          <Col span={24}>
            <h3 className={styles.header}>财务总览</h3>
          </Col>
          <Col span={24} className={styles.summary}>
            {this.renderSummary()}
          </Col>
        </Row>
        <div className={styles.hr}></div>
        <Row key="details">
          <Col span={24}>
            <h3 className={styles.header}>电子账户余额</h3>
          </Col>
          <Col span={24}>
            <div className="Page-scroller" style={{ margin: '0', height: '224px' }}>
              <List dataSource={details} columns={detailColumns} />
            </div>
          </Col>
        </Row>
      </Modal>
    );
  }
}
export default EleAccInfoModal;
