import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import List from 'COMPONENT/List';
import Button from 'COMPONENT/Button';
import EleAccFlowSearchBar from './EleAccFlowSearchBar';
import { Popover, Tooltip } from 'antd';
import loaders from './loaders';
import moment from 'moment';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import EleAccInfoModal from './EleAccInfoModal';
import styles from './styles.less';

const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;

const curMonthFitstDay = moment().startOf('month').format('YYYY-MM-DD');
const curDay = moment().format('YYYY-MM-DD');

const defaultSearchConditions = {
  startTime: curMonthFitstDay,
  endTime: curDay,
};

function dateRender(val, row, i) {
  return (val || '').slice(0, 16);
}

function commonRender(val, row, i) {
  return val || '-';
}

const sourceTypeIncomeColumns = [
  { title: '日期', dataIndex: 'fromCreateTime', render: dateRender, width: COL_WIDTH_L },
  { title: '学生编号', dataIndex: 'fromStudentId', width: COL_WIDTH_L },
  { title: '学生姓名', dataIndex: 'fromStudentName', width: COL_WIDTH_L },
  { title: '学生学习中心', dataIndex: 'fromStudentCampus', width: COL_WIDTH_L },
  { title: '转入金额', dataIndex: 'opreateMoney', width: COL_WIDTH_M },
];

const sourceTypeOutcomeColumns = [
  { title: '日期', dataIndex: 'fromCreateTime', render: dateRender, width: COL_WIDTH_L },
  { title: '学生编号', dataIndex: 'fromStudentId', width: COL_WIDTH_L },
  { title: '学生姓名', dataIndex: 'fromStudentName', width: COL_WIDTH_L },
  { title: '学生学习中心', dataIndex: 'fromStudentCampus', width: COL_WIDTH_L },
  { title: '转出金额', dataIndex: 'opreateMoney', width: COL_WIDTH_M },
];

const sourceTypeConsumeColumns = [
  { title: '日期', dataIndex: 'createTime', render: dateRender, width: COL_WIDTH_L },
  { title: '合同编号', dataIndex: 'chargeConId', render: commonRender, width: COL_WIDTH_L },
  { title: '合同学习中心', dataIndex: 'stuCampusName', render: commonRender, width: COL_WIDTH_L },
  { title: '消耗金额', dataIndex: 'opreateMoney', render: commonRender, width: COL_WIDTH_M },
];

const sourceTypeColumnsObj = {
  TRANSFER_IN: sourceTypeIncomeColumns,
  ACCOUNT_TRANSFER_SUCCESS: sourceTypeOutcomeColumns,
  CONTRACT_CHARGE: sourceTypeConsumeColumns,
};

class EleAccFlow extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: true,
      listLoading: false,
      dataSource: [],
      total: 0,
      current: 1,
      pageSize: 20,
      searchConditions: {},
      eleAccInfoModalVisible: false,
      eleAccInfoModalData: null,
    };
  }

  init = async (val) => {
    const page = 1;
    const res = await loaders.getEletronicAccFlowList({
      ...val,
      rows: this.state.pageSize,
      page: page,
    });
    this.setState({
      searchConditions: val,
      total: res.rowCount,
      dataSource: res.datas,
    });
    return;
  };

  onSearch = async (val) => {
    const page = 1;
    this.setState({ listLoading: true });
    const res = await loaders.getEletronicAccFlowList({
      ...val,
      rows: this.state.pageSize,
      page: page,
    });
    this.setState({
      searchConditions: val,
      total: res.rowCount,
      dataSource: res.datas,
      listLoading: false,
    });
    return;
  };

  onChange = async ({ current = this.state.current, pageSize = this.state.pageSize }) => {
    this.setState({ listLoading: true });
    const res = await loaders.getEletronicAccFlowList({
      ...this.state.searchConditions,
      rows: pageSize,
      page: current,
    });
    this.setState({
      current: current,
      pageSize: pageSize,
      total: res.rowCount,
      dataSource: res.datas,
      listLoading: false,
    });
    return;
  };

  componentDidMount() {
    this.setState({ loading: true });
    Promise.all([this.init(defaultSearchConditions)]).then(() => {
      this.setState({ loading: false });
    });
  }

  onExportClick = async () => {
    this.setState({ loading: true });
    try {
      await loaders.exportECFlowList(this.state.searchConditions);
      this.props.dispatch({
        type: 'main/exportNotice',
      });
    } catch (e) {
      console.log(e);
      showMessage('导出失败：' + e.text, 'error');
    }
    this.setState({ loading: false });
  };

  sourceTypeRender = (val, row, i) => {
    const { sourceTypeName, sourceTypeValue } = row;
    const columns = sourceTypeColumnsObj[sourceTypeValue];
    if (!columns) {
      return <span>{sourceTypeName}</span>;
    }
    const list = [row];
    const content = <List dataSource={list} columns={columns} />;
    return (
      <Popover
        content={content}
        title="流水详情"
        trigger="click"
        overlayClassName={styles.popoverlay}
        getPopupContainer={() => document.querySelector('div.ant-tabs-tabpane-active .Page-content')}
      >
        <span className={styles.btn}>{sourceTypeName}</span>
      </Popover>
    );
  };

  stuNameRender = (val, row, i) => {
    const { stuName } = row;
    return (
      <span
        onClick={() => {
          this.setState({
            eleAccInfoModalVisible: true,
            eleAccInfoModalData: { ...row },
          });
        }}
        className={styles.btn}
      >
        {stuName}
      </span>
    );
  };

  dotRender = (val, row, i) => {
    const { opreateMoney } = row;
    if (opreateMoney === undefined) {
      return '';
    }
    return <span className={styles.dot + ' ' + (opreateMoney >= 0 ? styles.green : styles.red)}></span>;
  };

  columns = [
    { title: '', key: 'dot', render: this.dotRender, width: 46 },
    { title: '流水ID', dataIndex: 'flowId', width: COL_WIDTH_M },
    { title: '学生姓名', dataIndex: 'stuName', render: this.stuNameRender, width: COL_WIDTH_M },
    { title: '学生编号', dataIndex: 'studentId', width: COL_WIDTH_M },
    { title: '学生学习中心', dataIndex: 'stuCampusName', width: COL_WIDTH_M },
    { title: '来源', dataIndex: 'sourceTypeName', render: this.sourceTypeRender, width: COL_WIDTH_M },
    { title: '金额', dataIndex: 'opreateMoney', width: COL_WIDTH_M },
    { title: '合同编号', dataIndex: 'contractId', render: commonRender, width: COL_WIDTH_M },
    { title: '合同类型', dataIndex: 'contractTypeName', render: commonRender, width: COL_WIDTH_M },
    { title: '产品名称', dataIndex: 'productName', render: commonRender, width: COL_WIDTH_M },
    { title: '产品类型', dataIndex: 'productTypeName', render: commonRender, width: COL_WIDTH_M },
    { title: '班级名称', dataIndex: 'className', render: commonRender, width: COL_WIDTH_M },
    { title: '操作人', dataIndex: 'opreateUserName', render: commonRender, width: COL_WIDTH_M },
    { title: '操作时间', dataIndex: 'createTime', render: dateRender, width: 140 },
  ];

  render() {
    const { btnAuth } = this.props;
    const { loading, dataSource, total, current, pageSize, listLoading, eleAccInfoModalVisible, eleAccInfoModalData } =
      this.state;

    return (
      <Page
        loading={loading}
        fitWindowHeight={true}
        route={this.props.route}
        subHeader={
          <div style={{ display: 'flex' }}>
            <EleAccFlowSearchBar defaultConditions={defaultSearchConditions} onSearch={this.onSearch} />
            {btnAuth.indexOf('EXPORT_ELE_ACC_FLOW') > -1 && (
              <Tooltip title="1次最多只能导出10000条数据">
                <Button onClick={this.onExportClick}>导出</Button>
              </Tooltip>
            )}
          </div>
        }
      >
        <List
          dataSource={dataSource}
          columns={this.columns}
          pagination={{
            current,
            total,
            pageSize,
          }}
          loading={listLoading}
          onChange={this.onChange}
        />

        {eleAccInfoModalVisible && (
          <EleAccInfoModal
            studentName={eleAccInfoModalData.stuName}
            studentId={eleAccInfoModalData.studentId}
            onClose={() => this.setState({ eleAccInfoModalVisible: false })}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    btnAuth: state.account.btnAuths || [],
  };
};

export default connect(mapStateToProps, null)(EleAccFlow);
