import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import SearchBar from 'COMPONENT/SearchBar';
import Select from 'COMPONENT/Select';
import DatePicker from 'COMPONENT/DatePicker';
import PictureWall from './component/right';
import ShowBig from './component/showBig';
import Button from 'COMPONENT/Button';
import IconButton from 'COMPONENT/IconButton';
import MultiSelect from 'COMPONENT/MultiSelect';
import SearchBox from 'SRC/BLoComponents/SearchBox';
import moment from 'moment';
import ShowConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import RefuseViewModal from './viewModal/index.js';
import styles from './index.less';
import { loadCommonDict } from 'UTIL/dict';

import {
  getCampusByLoginUser,
  getOTOList,
  checkAudit,
  checkAllAudit,
  getOneOnOneAutoRecogCount,
  getOneOnOneAuditSummary,
} from 'SERVICE/otoCourseCheck';
import { listByQueryNameAndRoleCode } from 'SERVICE/common';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

// 查看明细(默认配置)
let detailDict = [
  { text: '幼儿园', num: 0 },
  { text: '一年级', num: 0 },
  { text: '二年级', num: 0 },
  { text: '三年级', num: 0 },
  { text: '四年级', num: 0 },
  { text: '五年级', num: 0 },
  { text: '六年级', num: 0 },
  { text: '初一', num: 0 },
  { text: '初二', num: 0 },
  { text: '初三', num: 0 },
  { text: '高一', num: 0 },
  { text: '高二', num: 0 },
  { text: '高三', num: 0 },
  { text: '毕业', num: 0 },
];
let checkDict = [
  { val: 'MANUALLY_FILL_IN', text: '手动填写' },
  { val: 'AUTO_RECOG', text: '自动识别' },
];
let timeDict = [
  // {val: 'keshi', text: '课时'},
  { val: 'xiaoshi', text: '小时' },
];

class CourseCheck extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false,
      index: 0,
      count: 0,
      gradeDict: [],
      subjectDict: [],
      auditDict: [],
      campusDict: [],
      subjectList: [],
      list: [],
      startDate: '',
      endDate: '',
      checkType: '',
      timeType: 'xiaoshi',
      campusId: '',
      teacherId: '',
      auditStatus: '',
      columns: 4,
      unit: '小时',
    };
  }

  componentDidMount() {
    let { stateCampusId, redirectCampusId, teacherId, propsStartDate, propsEndDate, teacherName } = this.props;
    let campusId = stateCampusId === '000001' ? '' : stateCampusId;
    campusId = redirectCampusId ? redirectCampusId : campusId;
    let { startDate, endDate } = this.initStartAndEndDate();
    startDate = propsStartDate ? propsStartDate : startDate;
    endDate = propsEndDate ? propsEndDate : endDate;
    Promise.all([
      loadCommonDict('STUDENT_GRADE'),
      loadCommonDict('SUBJECT'),
      loadCommonDict('AuditStatus'),
      this.getCampus(),
      loadCommonDict('WASH_CAUSE'),
    ]).then(([gradeDict, subjectDict, auditDict, campusDict, washDict]) => {
      let subjectList = subjectDict.map((v, k) => {
        return v.val;
      });
      this.setState(
        {
          gradeDict,
          subjectDict,
          auditDict,
          campusDict,
          washDict,
          subjectList,
          startDate,
          endDate,
          campusId,
          teacherId,
          teacherName,
        },
        () => {
          if (redirectCampusId) {
            this.loadData();
          }
        },
      );
    });
  }

  componentWillReceiveProps(nextProps) {
    let { menuExpand } = nextProps;
    setTimeout(() => {
      this.calcWidth();
    }, 250);
  }

  /**
   * 获取学习中心列表
   * @returns {Promise.<Array|*>}
   */
  getCampus = async () => {
    let campusList = await getCampusByLoginUser();
    let dict = campusList.map((v, k) => {
      let options = [];
      let label = v[0].name;
      v.forEach((value, key) => {
        if (key !== 0) {
          options.push({ val: value.id, text: value.name });
        }
      });
      return { label: label, options: options };
    });
    return dict;
  };
  selectCampus = (val) => {
    this.setState({ campusId: val });
  };
  loadData = async () => {
    let { timeType } = this.state;
    // this.setState({index: 0, loading: true});
    let { rows, total = '', valid = '', detailDict = [] } = await this.search({});

    window.addEventListener('resize', this.calcWidth, false);
    let unit = timeType === 'keshi' ? '课时' : '小时';
    if (rows) {
      this.setState({ list: rows, unit, detailData: { total, valid, detailDict } }, () => {
        this.calcWidth();
      });
    }
    if (rows && rows.length > 0) {
      this.scrollTop(0);
    }
  };
  /**
   * 查询课时单列表
   * @param searchKey
   */
  search = async (searchKey) => {
    let { startDate, endDate, campusId, teacherId, auditStatus, subjectList, timeType, checkType } = this.state;
    if (!startDate) {
      showMessage('请选择开始时间！');
      return {};
    }
    if (!endDate) {
      showMessage('请选择结束时间！');
      return {};
    }
    if (!campusId || campusId === '000001') {
      showMessage('请选择学习中心！');
      return {};
    }
    if (subjectList === null) {
      showMessage('请至少选择一门科目！');
      return {};
    }
    let apiParams = {
      startDate, //开始时间
      endDate, //结束时间
      campusId, //学习中心id
      teacherId, //老师id
      auditStatus, //审核状态
      subject: subjectList.join(','), // 课程id
      courseAttenceType: checkType, //审批类型  手动 自动
      anshazhesuan: timeType, //类型，小时 课时
      courseStatus: 'CHARGED', //课程已结算，写死
      currentRoleId: 'financeAudit', //不知道啥意思，写死
      rows: 9999, //一页多少条数据,给个极大值，加载所有数据
      // page: currentPage || 1,//当前页码
    };

    this.setState({ index: 0, loading: true });
    let otherApiParams = { ...apiParams, rows: '', page: '' };
    let { data } = await getOneOnOneAuditSummary(otherApiParams);
    let { total, valid } = data;
    let keysArrays = Object.keys(data);
    detailDict.forEach((v, k) => {
      if (keysArrays.includes(v.text)) {
        v.num = data[v.text];
      } else {
        v.num = 0;
      }
    });
    let { rows } = await getOTOList(apiParams);
    let unit = timeType === 'keshi' ? '课时' : '小时';
    rows.forEach((v, k) => {
      v.completeUrl = this.props.aliPath + '/' + v.attendacePicName;
      if (timeType === 'keshi') {
        v.planHours = v.planHours + unit;
        v.realHours = v.realHours + unit;
      } else {
        let plan = (v.planHours * v.courseTimeLong) / 60;
        let real = (v.realHours * v.courseTimeLong) / 60;
        if (!Number.isInteger(plan)) {
          plan = plan.toFixed(2);
        }
        if (!Number.isInteger(real)) {
          real = real.toFixed(2);
        }
        v.planHours = plan + unit;
        v.realHours = real + unit;
      }
    });
    this.setState({ loading: false });
    return { rows, total, valid, detailDict };
  };
  /**
   * 重置查询条件
   */
  reset = () => {
    this.setState({
      // subjectList: [],
      // startDate: '',
      // endDate: '',
      // checkType: '',
      // timeType: '',
      campusId: '',
      teacherId: '',
      // auditStatus: ''
    });
  };

  getOneOnOneAuditSummary = async () => {
    let { startDate, endDate, campusId, teacherId, auditStatus, subjectList, timeType, checkType } = this.state;
    let apiParams = {
      startDate, //开始时间
      endDate, //结束时间
      campusId, //学习中心id
      teacherId, //老师id
      auditStatus, //审核状态
      subject: subjectList.join(','), // 课程id
      courseAttenceType: checkType, //审批类型  手动 自动
      anshazhesuan: timeType, //类型，小时 课时
      courseStatus: 'CHARGED', //课程已结算，写死
      currentRoleId: 'financeAudit', //不知道啥意思，写死
      rows: 9999, //一页多少条数据,给个极大值，加载所有数据
    };
    let { data } = await getOneOnOneAuditSummary(apiParams);
    let { total, valid } = data;
    detailDict.forEach((v, k) => {
      for (let item in data) {
        if (item === v.text) {
          v.num = data[item];
        }
      }
    });
    return { total, valid, detailDict };
  };

  /**
   * 获取 startDate和endDate
   * @returns {{startDate: string, endDate: *}}
   */
  initStartAndEndDate = () => {
    let endDate = moment().format('YYYY-MM-DD');
    let splitdate = endDate.split('-');
    splitdate[splitdate.length - 1] = '01';
    let startDate = splitdate.join('-');
    return { startDate, endDate };
  };

  /**
   * 点击旋转图片按钮
   */
  rotateImg = () => {
    let { count } = this.state;
    count += 1;
    let degCount = count % 4;
    this.$img.resetRatio();
    this.setState({ count, degCount });
  };
  /**
   * 自动识别全部通过
   * @returns {Promise.<void>}
   */
  confirmAllCheck = async () => {
    let { startDate, endDate, campusId, teacherId, auditStatus, subjectList, timeType, checkType, campusDict } =
      this.state;
    if (!campusId) {
      showMessage('请选择学习中心！');
      return;
    }
    let apiParams = {
      startDate, //开始时间
      endDate, //结束时间
      campusId, //学习中心id
      teacherId, //老师id
      auditStatus, //审核状态
      subject: subjectList.join(','), // 课程id
      courseAttenceType: 'AUTO_RECOG', //审批类型  手动 自动
      anshazhesuan: timeType, //类型，小时 课时
      courseStatus: 'CHARGED', //课程已结算，写死
      currentRoleId: 'financeAudit', //不知道啥意思，写死
    };

    let { data } = await getOneOnOneAutoRecogCount(apiParams);
    let name;
    for (let i of campusDict) {
      for (let k of i['options']) {
        if (k['val'] === campusId) {
          name = k['text'];
          break;
        }
      }
      if (name) {
        break;
      }
    }
    let content = (
      <div>
        {name}自动识别的一对一课时单（共 {data} 张）将全部审批通过，请确认是否继续？
      </div>
    );
    ShowConfirm(
      content,
      async () => {
        let params = {
          campusId,
          currentRoleId: 'financeAudit',
          endDate,
          startDate,
          subject: subjectList.join(','),
        };
        let data = await checkAllAudit(params);
        showMessage('操作成功！', 'success');
        let { rows } = await this.search({});
        this.setState({ list: rows });
      },
      () => {},
    );
  };
  /**
   * 上一张
   */
  previous = () => {
    let { list, index, columns } = this.state;
    if (index > 0) {
      index--;
      if (index % columns === columns - 1) {
        this.scrollTop(index, columns);
      }
      this.$img.resetRatio();
      this.setState({ index, count: 0, degCount: 0 });
    }
  };
  /**
   * 下一张
   */
  next = () => {
    let { list, index, columns } = this.state;
    if (index < list.length - 1) {
      index++;
      if (index % columns === 0) {
        this.scrollTop(index, columns);
      }
      this.$img.resetRatio();
      this.setState({ index, count: 0, degCount: 0 });
    }
  };
  /**
   * 通过课时单
   * @param index
   */
  checkAudit = () => {
    let { list, index, columns } = this.state;
    let { courseId } = list[index];
    let copyIndex = index;
    let copyList = JSON.parse(JSON.stringify(list));
    this.setState({ loading: true });
    checkAudit({ courseId, auditStatus: 'VALIDATE' }).then(async (res) => {
      showMessage('操作成功！', 'success');
      //跳到下一张未审批的课时单
      // let {rows} = await this.search({});
      let { total, valid, detailDict } = await this.getOneOnOneAuditSummary();
      // list[index]['auditStatus'] = 'VALIDATE';
      copyList[index]['auditStatus'] = 'VALIDATE';
      while (index < copyList.length && copyList[index]['auditStatus'] === 'VALIDATE') {
        index++;
      }
      if (index === copyList.length) {
        index = copyIndex;
      }
      if (index % columns === 0) {
        this.scrollTop(index, columns);
      }
      this.$img.resetRatio();
      // this.setState({index, list, loading: false, detailData: {total, valid, detailDict}});
      this.setState({ index, list: copyList, loading: false, detailData: { total, valid, detailDict } });
    });
  };

  /**
   * 拒绝课时单成功之后的回调
   */
  onSuccess = async () => {
    let { index } = this.state;
    let copyIndex = index;
    // index--;
    this.setState({ refuseVisiable: false });
    let { rows, total, valid, detailDict } = await this.search();
    while (index < rows.length && rows[index]['auditStatus'] === 'VALIDATE') {
      index++;
    }
    if (copyIndex === rows.length) {
      copyIndex -= 1;
    }
    if (index === rows.length) {
      index = copyIndex;
    }
    if (index % 4 === 0) {
      this.scrollTop(index);
    }
    this.$img.resetRatio();
    this.setState({ list: rows, index, detailData: { total, valid, detailDict } });
  };

  /**
   * 右边图片墙滚动至相应位置
   * @param index {number} 图片索引
   */
  scrollTop = (index, col = 4) => {
    let count = Math.floor(index / col);
    let $contains = document.querySelector('.scrollContains');
    let $item = document.querySelector('.scrollItem');
    let height = window.getComputedStyle($item, null).height;
    let marginBottom = window.getComputedStyle($item, null).marginBottom;
    let distance = parseFloat(height) + parseFloat(marginBottom);
    $contains.scrollTo(0, distance * count);
  };
  /**
   * 获取要禁用的开始日期值
   * @param  {string} startDate 过滤前的日期值
   * @return {boolean} 是否要禁用
   */
  getDisabledStartDate = (startDate) => {
    let { endDate } = this.state;
    if (!startDate || !endDate) {
      return false;
    }
    return moment(startDate).valueOf() > moment(endDate).valueOf();
  };

  /**
   * 获取要禁用的结束日期值
   * @param  {string} endDate   过滤前的日期值
   * @return {boolean}      是否要禁用
   */
  getDisabledEndDate = (endDate) => {
    const { startDate } = this.state;
    if (!endDate || !startDate) {
      return false;
    }
    return moment(endDate).valueOf() < moment(startDate).valueOf();
  };
  updateIndex = (index) => {
    this.$img.resetRatio();
    this.setState({ index, count: 0, degCount: 0 });
  };

  /**
   * 窗口改变时计算div宽度，调整缩略图列数
   */
  calcWidth = () => {
    let $flexInnerFit = document.querySelector('.flexInnerFit');
    let $right = document.querySelector('.calcRight');
    if ($right) {
      let $wrap = document.querySelector('.calcWidth');
      let W = $flexInnerFit.offsetWidth;
      let multiple = Math.floor((W - 560) / 240);
      $wrap.style.width = multiple * 240 + 6 + 'px';
      this.setState({ columns: multiple });
    }
  };

  componentWillUnmount() {
    window.removeEventListener('resize', this.calcWidth, false);
  }

  render() {
    let {
      loading,
      subjectDict,
      auditDict,
      campusDict,
      teacherDict,
      washDict,
      searchExpand,
      degCount = 0,
      refuseVisiable = false,
      showArrow = true,
      inArrow,
      auditStatus,
      subjectList,
      startDate,
      endDate,
      checkType,
      timeType,
      campusId,
      list,
      index,
      detailData,
    } = this.state;
    return (
      <Page
        loading={loading}
        footer={false}
        route={this.props.route}
        className={styles.OTOCourseCheck}
        fitWindowHeight={true}
        subHeader={
          <div className={styles.searchBar}>
            <Button className={styles.allcheck} onClick={this.confirmAllCheck}>
              自动识别全部通过
            </Button>
            <SearchBar
              expand={searchExpand}
              onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
              onSearch={this.loadData}
              onReset={this.reset}
            >
              <DatePicker
                placeholder="开始日期"
                value={startDate}
                onChange={(val) => this.setState({ startDate: val })}
                disabledDate={(val) => this.getDisabledStartDate(val)}
              />
              <DatePicker
                placeholder="结束日期"
                value={endDate}
                onChange={(val) => this.setState({ endDate: val })}
                disabledDate={(val) => this.getDisabledEndDate(val)}
              />
              <Select
                placeholder="学习中心"
                showSearch={true}
                dict={campusDict}
                value={campusId}
                onChange={(val) => this.selectCampus(val)}
              />
              <SearchBox
                mode="edit"
                size="large"
                isMultiple={false}
                paramName="queryName"
                value={this.state.teacherId || ''}
                placeholder="请选择老师"
                isClearOptionsAfterSelect={true}
                getData={listByQueryNameAndRoleCode}
                initOriginData={
                  this.props.teacherId ? [{ name: this.props.teacherName, userId: this.props.teacherId }] : []
                }
                extraParams={{ roleCode: 'TEATCHER', number: 100 }}
                onSelfChange={(value) => {
                  this.setState({ teacherId: (value || {}).key || '' });
                }}
                onLoaded={(list) =>
                  list.map(({ userId, name, employeeNo }) => ({
                    code: userId,
                    name: `${employeeNo ? `${name}（${employeeNo}）` : name}`,
                  }))
                }
              />
              <Select
                placeholder="审批状态"
                dict={auditDict}
                value={auditStatus}
                onChange={(val) => this.setState({ auditStatus: val })}
              />
              <Select
                placeholder="考勤类型"
                dict={checkDict}
                value={checkType}
                onChange={(val) => this.setState({ checkType: val })}
              />
              <MultiSelect
                placeholder="科目"
                dict={subjectDict}
                value={subjectList}
                onChange={(val) => this.setState({ subjectList: val })}
              />
              <Select
                placeholder="时间类型"
                dict={timeDict}
                value={timeType}
                onChange={(val) => this.setState({ timeType: val })}
              />
            </SearchBar>
          </div>
        }
      >
        <div className={styles.contains}>
          {list[0] && (
            <div className={styles.left}>
              <div className={styles.wrap}>
                <ShowBig
                  className={'rotate' + degCount}
                  src={list[index].completeUrl}
                  ref={(e) => (this.$img = e)}
                  inArrow={inArrow}
                  outArrow={(e) => this.setState({ inArrow: false })}
                />
                <span onMouseEnter={(e) => this.setState({ inArrow: true })} className={styles.leftwrap}>
                  <IconButton
                    type={'left'}
                    size={27}
                    color="#fff"
                    className={styles.leftarrow}
                    onClick={this.previous}
                  />
                </span>
                <span onMouseEnter={(e) => this.setState({ inArrow: true })} className={styles.rightwrap}>
                  <IconButton type={'right'} size={27} color="#fff" className={styles.rightarrow} onClick={this.next} />
                </span>

                <div className={styles.survey}>
                  <span>{list[index]['grade'] + list[index]['subject']}</span>
                  <IconButton type={'rotate_left'} size={24} color="#fff" onClick={this.rotateImg} />
                </div>
              </div>
              <div className={styles.content}>
                <div>
                  <span>日期</span>
                  <span>{list[index]['courseDate']}</span>
                </div>
                <div>
                  <span>时间</span>
                  <span>{list[index]['courseTime']}</span>
                </div>
                <div>
                  <span>老师</span>
                  <span className={styles.extrude}>{list[index]['teacherName']}</span>
                </div>
                <div>
                  <span>学生</span>
                  <span className={styles.extrude}>{list[index]['studentName']}</span>
                </div>
                <div>
                  <span>计划</span>
                  <span>{list[index]['planHours']}</span>
                </div>
                <div>
                  <span>实际</span>
                  <span className={styles.extrude}>{list[index]['realHours']}</span>
                </div>
                <div>
                  <span>状态</span>
                  <span>{list[index]['courseStatusName']}</span>
                </div>
                <div>
                  <span>类型</span>
                  <span>{list[index]['courseAttenceTypeName']}</span>
                </div>
              </div>
              <div className={styles.handle}>
                <Button onClick={(e) => this.setState({ refuseVisiable: true })}>拒绝</Button>
                <Button
                  type="primary"
                  onClick={this.checkAudit}
                  disabled={list[index]['auditStatus'] === 'VALIDATE' ? true : false}
                >
                  通过
                </Button>
              </div>
            </div>
          )}

          {list[0] && (
            <div className={styles.right + ' ' + 'calcRight'}>
              <PictureWall data={list} index={index} detailData={detailData} updateIndex={this.updateIndex} />
            </div>
          )}
        </div>

        {refuseVisiable && (
          <RefuseViewModal
            onClose={(e) => this.setState({ refuseVisiable: false })}
            onSuccess={this.onSuccess}
            courseId={list[index]['courseId']}
            washDict={washDict}
          />
        )}
      </Page>
    );
  }
}

function mapStateToProps(state) {
  let OTOCourseCheck = state.main.pageParams['/OTOCourseCheck'];
  return {
    stateCampusId: state.account.user.campusId,
    aliPath: state.account.user.aliPath,
    menuExpand: state.main.menuExpand,
    redirectCampusId: OTOCourseCheck ? OTOCourseCheck.campusId : null,
    teacherName: OTOCourseCheck ? OTOCourseCheck.teacherName : '',
    teacherId: OTOCourseCheck ? OTOCourseCheck.teacherId : '',
    propsStartDate: OTOCourseCheck ? OTOCourseCheck.startDate : null,
    propsEndDate: OTOCourseCheck ? OTOCourseCheck.endDate : null,
  };
}

export default connect(mapStateToProps)(CourseCheck);
