import React from 'react';
import Modal from 'COMPONENT/Modal';
import FormItem from 'COMPONENT/FormItem';
import TextArea from 'COMPONENT/TextArea';
import Radio from 'COMPONENT/Radio';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import Icon from 'COMPONENT/Icon';
import Input from 'COMPONENT/Input';
import styles from './index.less';
import getWrappedInstance from 'UTIL/getWrappedInstance';

import { invalidAudit } from 'SERVICE/otoCourseCheck';

const RadioButton = Radio.RadioButton;
const RadioGroup = Radio.Group;
export default class RefuseModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      oaFlowId: '财务驳回考勤',
    };
  }

  onOk = () => {
    let { details, reason, oaFlowId } = this.state;
    let { courseId, onSuccess } = this.props;
    const error = (text) => getWrappedInstance(this.$modal).showOkTips(text);

    if (!reason) {
      error('请选择拒绝原因！');
      return;
    }
    if (!oaFlowId) {
      error('请填写已审批通过的OA工单！');
      return;
    }
    if (!details) {
      error('请填写详细原因！');
      return;
    }
    this.setState({ loading: true });
    invalidAudit({ 'cause.id': reason, courseId, detailReason: details, oaFlowId })
      .then((res) => {
        showMessage('操作成功!', 'success');
        this.setState({ loading: false });
        onSuccess();
      })
      .catch(() => {
        this.setState({ loading: false });
      });
  };

  onChange = (e) => {
    this.setState({ reason: e.target.value });
  };

  render() {
    let { loading = false, details, reason, oaFlowId } = this.state;
    let { onClose, washDict } = this.props;
    return (
      <Modal
        loading={loading}
        onCancel={onClose}
        onOk={this.onOk}
        title=""
        width={500}
        okBtnShow={true}
        cancelBtnShow={true}
        className={styles.modal}
        ref={(e) => (this.$modal = e)}
      >
        <div className={styles.state}>
          <div className={styles.warning}>
            <Icon type={'warning'} size={42} color={'#ffb444'} />
          </div>
          <div>拒绝后，该课程将有如下影响：</div>
          <div>
            <div>1、该课程将执行扣费冲销操作</div>
            <div>2、该课程的扣费冲销款项将回退至学生的合同产品中</div>
            <div>3、该课程的状态会回退至老师已考勤</div>
          </div>
        </div>

        <div className={styles.refusecheck}>
          <div>如确认拒绝，请填写原因并提交</div>
          <FormItem name="拒绝原因" required={true}>
            <RadioGroup onChange={this.onChange} value={reason}>
              {washDict.map((v, k) => {
                return (
                  <RadioButton value={v.val} key={k}>
                    {v.text}
                  </RadioButton>
                );
              })}
            </RadioGroup>
          </FormItem>
          <FormItem name="详细原因" required={true}>
            <Input type="hidden" placeholder="" value={oaFlowId} />
            <TextArea
              rows={3}
              value={details}
              style={{ resize: 'none' }}
              onChange={(e) => this.setState({ details: e.target.value })}
            />
          </FormItem>
        </div>
      </Modal>
    );
  }
}
