import React from 'react';
import { connect } from 'dva';
import styles from './index.less';
import { DatePicker } from 'antd';
import { Page, SearchBar, Input, Select, MultiSelect, List, Button } from 'COMPONENT';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import { loadCommonDictByGroup, loadBranchCampusDict } from 'UTIL/dict';
import { dateToMoment } from 'COMPONENT/utils/form';
import NP from 'number-precision';
import * as dicts from './dicts';
import * as loaders from './loaders';
import * as utils from './utils';
import { computeTime } from 'PAGE/Dashboard/TopTenSS/TimePagination/util';

const RangePicker = DatePicker.RangePicker;
const { COL_WIDTH_M, COL_WIDTH_L } = List;

/**
 * 学习中心分账明细
 */
class Index extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false, // 页面loading
      downLoading: false, // 导出loading

      tempTime: null, // 临时变量-时间
      opTimeStart: null, // 操作时间-开始日期
      opTimeEnd: null, // 操作时间-结束日期
      opType: null, // 类型
      inCampusIds: null, // 收款学习中心
      campusType: null, // 临时变量-学习中心类型
      campusTypeCampusIds: null, // 学习中心类型学习中心ids
      contractProductCharacter: null, // 合同性质
      settlementStatus: null, // 结算情况
      contractId: null, // 合同编号
      studentName: null, // 学生姓名

      listCountData: {}, // 列表汇总数据
      list: [], // 列表数据
      pageNo: 1, // 当前页码
      pageSize: 20, // 每页记录数
      totalRecords: 0, // 总记录数
      selectedRows: [], // 选中行
      selectedRowKeys: [], // 选中行key

      branchCampusDict: [], // 学习中心字典
      productCharacterDict: [], // 产品性质字典
    };

    this.defaultState = {
      tempTime: null,
      opTimeStart: null,
      opTimeEnd: null,
      opType: null,
      inCampusIds: null,
      campusType: null,
      campusTypeCampusIds: null,
      contractProductCharacter: null,
      settlementStatus: null,
      contractId: null,
      studentName: null,
    };
  }

  componentDidMount() {
    this.initDict();
    this.initSearch();
  }

  initDict = async () => {
    const [branchCampusDict, groupDict] = await Promise.all([
      loadBranchCampusDict(),
      loadCommonDictByGroup('PRODUCT_CHARACTER'),
    ]);

    this.setState({
      branchCampusDict,
      productCharacterDict: groupDict['PRODUCT_CHARACTER'],
    });
  };

  initSearch = () => {
    const { startTimeText, endTimeText } = computeTime(3);
    const tempTime = [startTimeText, endTimeText];

    this.setState({ tempTime }, () => {
      this.loadList();
    });
  };

  /**
   * 获取列表
   *
   * @param changeParam
   */
  loadList = async (changeParam = {}) => {
    const params = this.formatListApiParams(changeParam);

    if (!params) {
      return;
    }

    this.toggleLoading();
    this.setState({ selectedRowKeys: [] });

    const { list, pageNo, pageSize, totalRecords } = await loaders.loadSplitAccountList(params);
    const listCountData = await loaders.loadSplitAccountListTotal(params);

    this.toggleLoading();
    this.setState({
      list,
      pageNo,
      pageSize,
      totalRecords,
      listCountData,
    });
  };

  /**
   * 格式化列表请求参数
   *
   * @param changeParam {Object}
   * @return {{}&{opTimeStart: *, opTimeEnd: *}|*}
   */
  formatListApiParams = (changeParam = {}) => {
    if (!this.state.tempTime) {
      showMessage('请先选择操作时间', 'error');
      return;
    }

    if (this.state.campusTypeCampusIds && this.state.campusTypeCampusIds.length > 0 && !this.state.campusType) {
      showMessage('请先选择学习中心类型再选择学习中心', 'error');
      return;
    }

    const {
      pageNo,
      pageSize,
      tempTime,
      opType,
      inCampusIds,
      contractProductCharacter,
      settlementStatus,
      contractId,
      studentName,
    } = this.state;
    const pageState = { pageNo, pageSize, ...changeParam };
    const campusParam = utils.formatCampusTypeCampusIds(this.state.campusType, this.state.campusTypeCampusIds);
    const pageParam = {
      page: pageState.pageNo,
      rows: pageState.pageSize,
    };

    return {
      opTimeStart: tempTime[0],
      opTimeEnd: tempTime[1],
      opType,
      inCampusIds,
      contractProductCharacter,
      settlementStatus,
      contractId,
      studentName,
      ...campusParam,
      ...pageParam,
    };
  };

  /**
   * 表格配置
   *
   * @return {*[]}
   */
  formatListColumns = () => {
    return [
      { title: '类型', dataIndex: 'opType', key: 'opType', width: COL_WIDTH_M, dict: dicts.OperateType },
      {
        title: '操作完成时间',
        dataIndex: 'opTime',
        key: 'opTime',
        width: COL_WIDTH_L,
        render: (text) => {
          return utils.formatTime(text);
        },
      },
      { title: '合同编号', dataIndex: 'contractId', key: 'contractId', width: COL_WIDTH_L },
      {
        title: '合同产品性质',
        dataIndex: 'contractProductCharacter',
        key: 'contractProductCharacter',
        width: COL_WIDTH_M,
        dict: this.state.productCharacterDict,
      },
      { title: '学生姓名', dataIndex: 'studentName', key: 'studentName', width: COL_WIDTH_M },
      { title: '产品名称', dataIndex: 'productName', key: 'productName', width: COL_WIDTH_L },
      { title: '班级名称', dataIndex: 'className', key: 'className', width: COL_WIDTH_L },
      { title: '合同学习中心', dataIndex: 'contractCampusName', key: 'contractCampusName', width: COL_WIDTH_L },
      { title: '收款学习中心', dataIndex: 'inCampusName', key: 'inCampusName', width: COL_WIDTH_L },
      { title: '扣费学习中心', dataIndex: 'deductionCampusName', key: 'deductionCampusName', width: COL_WIDTH_L },
      { title: '退费学习中心', dataIndex: 'refundCampusName', key: 'refundCampusName', width: COL_WIDTH_L },
      { title: '转入学习中心', dataIndex: 'intoStudentCampusName', key: 'intoStudentCampusName', width: COL_WIDTH_L },
      {
        title: '电子账户学习中心',
        dataIndex: 'electronicAccountCampusName',
        key: 'electronicAccountCampusName',
        width: COL_WIDTH_L,
      },
      { title: '结算金额', dataIndex: 'opAmount', key: 'opAmount', width: COL_WIDTH_M },
      {
        title: '已结算',
        dataIndex: 'settlementStatus',
        key: 'settlementStatus',
        width: 80,
        dict: dicts.SettlementStatusListDict,
      },
    ];
  };

  onSearch = () => {
    this.loadList({ pageNo: 1 });
  };

  onReset = () => {
    this.setState({ ...this.defaultState });
  };

  /**
   * 翻页
   *
   * @param current
   * @param pageSize
   */
  onPaginationChange = ({ current, pageSize }) => {
    const changed = {
      pageNo: this.state.pageSize !== pageSize ? 1 : current,
      pageSize,
    };

    this.loadList(changed);
  };

  /**
   * 选中行
   *
   * @param keys
   * @param rows
   */
  onRowSelectChange = (keys, rows) => {
    this.setState({
      selectedRowKeys: keys,
      selectedRows: rows,
    });
  };

  /**
   * 标记已结算
   */
  onMarkFlag = () => {
    const { selectedRows } = this.state;

    if (!selectedRows || selectedRows.length === 0) {
      showMessage('请选择分账明细', 'error');
      return;
    }

    const hasSettleStatusYes = selectedRows.find(
      (item) => String(item.settlementStatus) === dicts.SETTLEMENT_STATUS_YES,
    );
    if (hasSettleStatusYes) {
      showMessage('请选择未结算的分账明细', 'error');
      return;
    }

    this.doMarkFlag();
  };

  /**
   * 执行标记
   */
  doMarkFlag = () => {
    const { selectedRows } = this.state;
    const params = {
      detailIds: selectedRows.map((item) => item.detailId),
    };

    const opAmount = selectedRows.reduce((total, item) => {
      return NP.plus(total, item.opAmount);
    }, 0);
    const content = (
      <div>
        <p>已勾选的结算金额为{opAmount}元，确认是否标记为已结算？</p>
        <p style={{ color: 'red', fontSize: '14px' }}>注意：标记后无法修改，请确认无误后再操作！</p>
      </div>
    );

    showConfirm(content, () => {
      this.toggleLoading();
      loaders
        .updateSettleStatus(params)
        .then(() => {
          showMessage('操作成功', 'success');
          this.toggleLoading();
          this.loadList();
        })
        .catch(() => {
          this.toggleLoading();
        });
    });
  };

  /**
   * 导出excel
   */
  onDownloadExcel = () => {
    const params = this.formatListApiParams({});

    if (!params) {
      return;
    }

    this.setState({ downLoading: true });
    loaders
      .downLoadSplitAccountList(params)
      .then(() => {
        this.setState({ downLoading: false });
        showMessage('导出成功', 'success');
      })
      .catch(() => {
        this.setState({ downLoading: false });
        showMessage('导出失败，请重试！', 'error');
      });
  };

  /**
   * 显示/隐藏loading
   */
  toggleLoading = () => {
    this.setState((prevState) => {
      return {
        loading: !prevState.loading,
      };
    });
  };

  render() {
    const { listCountData } = this.state;

    return (
      <Page
        fitWindowHeight={true}
        route={this.props.route}
        loading={this.state.loading}
        className={styles.pageWrap}
        subHeader={[
          <SearchBar key={1} onSearch={this.onSearch} onReset={this.onReset}>
            <RangePicker
              placeholder={['日期(起)', '日期(止)']}
              format="YYYY-MM-DD"
              showTime={false}
              allowClear={true}
              size="large"
              value={
                this.state.tempTime ? [dateToMoment(this.state.tempTime[0]), dateToMoment(this.state.tempTime[1])] : []
              }
              onChange={(date, dateString) => {
                this.setState({ tempTime: dateString });
              }}
            />

            <Select
              placeholder="类型"
              dict={dicts.OperateTypeDict}
              value={this.state.opType}
              onChange={(val) => this.setState({ opType: val })}
            />

            <MultiSelect
              placeholder="收款学习中心"
              showSearch
              allowClear={true}
              dict={this.state.branchCampusDict}
              value={this.state.inCampusIds}
              onChange={(val) => {
                this.setState({ inCampusIds: val });
              }}
            />

            <Select
              placeholder="学习中心类型"
              dict={dicts.CampusType}
              value={this.state.campusType}
              onChange={(val) => this.setState({ campusType: val })}
            />
            <MultiSelect
              placeholder="请选择学习中心"
              showSearch
              allowClear={true}
              dict={this.state.branchCampusDict}
              value={this.state.campusTypeCampusIds}
              onChange={(val) => {
                this.setState({ campusTypeCampusIds: val });
              }}
              disabled={!this.state.campusType}
            />

            <Select
              placeholder="合同性质"
              showSearch
              allowClear={true}
              dict={this.state.productCharacterDict}
              value={this.state.contractProductCharacter}
              onChange={(val) => {
                this.setState({ contractProductCharacter: val });
              }}
            />
            <Select
              placeholder="结算情况"
              showSearch
              allowClear={true}
              dict={dicts.SettlementStatus}
              value={this.state.settlementStatus}
              onChange={(val) => {
                this.setState({ settlementStatus: val });
              }}
            />
            <Input
              placeholder="合同编号"
              value={this.state.contractId}
              onChange={(e) => this.setState({ contractId: e.target.value })}
            />
            <Input
              placeholder="学生姓名"
              value={this.state.studentName}
              onChange={(e) => this.setState({ studentName: e.target.value })}
            />
          </SearchBar>,
          <section key={2}>
            <span className={styles.countTitle}>跨学习中心扣费</span>
            <span className={styles.countMoney}>{listCountData.crossCampusDeductiontotalAmount || 0}元</span>
            <span className={styles.countTitle}>跨学习中心扣费冲销</span>
            <span className={styles.countMoney}>{listCountData.crossCampusWashchargeTotalAmount || 0}元</span>
            <span className={styles.countTitle}>跨学习中心退费</span>
            <span className={styles.countMoney}>{listCountData.crossCampusRefundtotalAmount || 0}元</span>
            <span className={styles.countTitle}>跨学习中心转账</span>
            <span className={styles.countMoney}>{listCountData.crossCampusTransferTotalAmount || 0}元</span>
            <span className={styles.countTitle}>跨学习中心电子消费</span>
            <span className={styles.countMoney}>
              {listCountData.crossCampusElectronicAccountConsumeTotalAmount || 0}元
            </span>
          </section>,
        ]}
        headerToolBar={[
          this.props.btnAuth_mark && (
            <Button key={1} onClick={this.onMarkFlag} style={{ marginRight: 15 }}>
              标记已结算
            </Button>
          ),
          this.props.btnAuth_download && (
            <Button key={2} onClick={this.onDownloadExcel} disabled={this.state.downLoading}>
              {this.state.downLoading ? '导出中...' : '导出excel'}
            </Button>
          ),
        ]}
      >
        <List
          dataSource={this.state.list}
          columns={this.formatListColumns()}
          pagination={{
            current: this.state.pageNo,
            total: this.state.totalRecords,
            pageSize: this.state.pageSize,
          }}
          rowSelection={{
            selectedRowKeys: this.state.selectedRowKeys,
            onChange: (keys, rows) => {
              this.onRowSelectChange(keys, rows);
            },
          }}
          onChange={this.onPaginationChange}
          bordered={true}
        />
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const btnAuth = state.account.btnAuths;
  const btnAuth_mark = btnAuth.indexOf('BTN_FINANCE_SPLIT_MARK') > -1;
  const btnAuth_download = btnAuth.indexOf('BTN_FINANCE_SPLIT_DOWNLOAD') > -1;

  return {
    btnAuth_mark,
    btnAuth_download,
  };
};

export default connect(mapStateToProps)(Index);
