import {
  getSplitAccountList,
  getSplitAccountTotal,
  exportSplitAccountList,
  updateAsSettled,
} from 'SERVICE/financeManage';
import NP from 'number-precision';

/**
 * 查询分账明细记录
 * tips: 金额单位为分，需转成元的单位
 *
 * @param params {Object}
 * @return {Promise<{}|*>}
 */
export async function loadSplitAccountList(params) {
  try {
    const { datas, rowCount } = await getSplitAccountList(params);
    const result = (datas || []).map((item) => {
      const opAmount = NP.divide(item.opAmount || 0, 100);

      return {
        ...item,
        opAmount,
      };
    });

    return {
      list: result,
      pageNo: params.page,
      pageSize: params.rows,
      totalRecords: rowCount,
    };
  } catch (e) {
    return {
      list: [],
      pageNo: params.page,
      pageSize: params.rows,
      totalRecords: 0,
    };
  }
}

/**
 * 统计分账明细
 * tips: 金额单位为分，需转成元的单位
 *
 * @param params
 * @return {Promise<{}|*>}
 */
export async function loadSplitAccountListTotal(params) {
  try {
    const { data } = await getSplitAccountTotal(params);
    const result = data || {};

    Object.keys(result).forEach((key) => {
      result[key] = NP.divide(result[key] || 0, 100);
    });

    return result;
  } catch (e) {
    return {};
  }
}

/**
 * 导出分账明细记录
 *
 * @return {Promise<void>}
 */
export function downLoadSplitAccountList(params) {
  return exportSplitAccountList(params);
}

/**
 * 更新为已结算
 *
 * @param params {Object} { detailIds: [] }
 * @return {Promise<void>}
 */
export function updateSettleStatus(params) {
  return updateAsSettled(params);
}
