import React from 'react';
import { connect } from 'dva';
import SearchBar from 'COMPONENT/SearchBar';
import Button from 'COMPONENT/Button';
import Select from 'COMPONENT/Select';
import Input from 'COMPONENT/Input';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { formatOrgTypeDict, formatQuarterDict, hasValDict, HAS_VAL_VALID } from '../../dicts';
import { getDictTextByVal } from 'UTIL/dict';
import { STORE_NAME } from '../../store/config';
import { SHOW_HAS_VAL_SELECTOR } from '../../config';

class CommonSearch extends React.Component {
  static defaultProps = {
    onRefs: () => {},
    planImportDisabled: true,
    planImportLimitDisabled: true,
    goalImportDisabled: true,
    goalImportLimitDisabled: true,
    onSearch: () => {},
    onShowImportModal: () => {},
  };

  constructor(props) {
    super(props);

    this.state = {
      year: null,
      orgType: 'BRENCH',
      quarterType: 'MONTH', // 时间类型
      branchName: '', // 分公司名称
      campusName: '', // 学习中心名称
      hasVal: HAS_VAL_VALID,
    };

    this.initSearchParam = {
      year: null,
      orgType: 'BRENCH',
      quarterType: 'MONTH',
      branchName: '',
      campusName: '',
      hasVal: HAS_VAL_VALID,
    };

    this.onRefs = this.onRefs.bind(this);
    this.onSearch = this.onSearch.bind(this);
    this.onReset = this.onReset.bind(this);
    this.onOrgTypeChange = this.onOrgTypeChange.bind(this);
    this.onShowImportModal = this.onShowImportModal.bind(this);
    this.getParam = this.getParam.bind(this);
  }

  componentDidMount() {
    this.onRefs();
  }

  onRefs() {
    this.props.onRefs(this);
  }

  onSearch() {
    const { year, orgType, quarterType } = this.state;

    if (!year || !orgType || !quarterType) {
      showMessage('请选择年份、组织类型及时间类型！');
      return false;
    }

    this.props.onSearch();
  }
  onReset() {
    this.setState(this.initSearchParam);
  }

  /**
   * 组织类型切换
   * @param val
   */
  onOrgTypeChange(val) {
    const { campusName } = this.state;
    let tempCampusName = val === 'CAMPUS' ? campusName : '';

    this.setState({
      orgType: val,
      campusName: tempCampusName,
    });
  }

  /**
   * 打开导入弹窗
   * @param importType 导入目标类型 确认目标/导入目标权限分开 用于下拉选择
   * @param canImportHistory 是否允许导入历史数据 默认false
   */
  onShowImportModal(importType, canImportHistory = false) {
    this.props.onShowImportModal(importType, canImportHistory);
  }

  /**
   * 获取搜索条件参数
   */
  getParam() {
    const { year, orgType, quarterType, branchName, campusName, hasVal } = this.state;
    const { dicts } = this.props;

    return {
      yearId: year, // 年份
      yearText: getDictTextByVal(dicts.yearDict, year), // 年份text
      organizationType: orgType, // 组织类型
      goalTimeType: quarterType, // 时间类型
      productTypeId: '', // 产品类型
      cashFlowType: '', // 预收类型
      classTypeId: '', // 班级类型
      gradeId: '', // 年级
      subjectId: '', // 科目
      branchName: branchName, // 分公司名称
      campusName: campusName, // 学习中心名称
      hasVal: eval(hasVal), // 有无数据
    };
  }

  render() {
    const { year, orgType, quarterType, branchName, campusName, hasVal } = this.state;
    const { dicts, planImportDisabled, planImportLimitDisabled, goalImportDisabled, goalImportLimitDisabled } =
      this.props;

    const showCampus = orgType === 'CAMPUS';
    const showPlanImport = !planImportDisabled;
    const showPlanImportLimit = !planImportLimitDisabled;
    const showGoalImport = !goalImportDisabled;
    const showGoalImportLimit = !goalImportLimitDisabled;

    return (
      <SearchBar onSearch={this.onSearch} onReset={this.onReset} showExpandButton={false} expand={true}>
        <Select
          placeholder="年份"
          dict={dicts.yearDict}
          value={year}
          onChange={(val) => {
            this.setState({ year: val });
          }}
        />

        <Select placeholder="组织类型" dict={formatOrgTypeDict(1)} value={orgType} onChange={this.onOrgTypeChange} />

        <Select
          placeholder="时间类型"
          dict={formatQuarterDict(4)}
          value={quarterType}
          onChange={(val) => {
            this.setState({ quarterType: val });
          }}
        />

        <Input
          placeholder="请输入分公司"
          value={branchName}
          onChange={(e) => this.setState({ branchName: e.target.value })}
        />

        {showCampus && (
          <Input
            placeholder="请输入学习中心"
            value={campusName}
            onChange={(e) => this.setState({ campusName: e.target.value })}
          />
        )}

        {SHOW_HAS_VAL_SELECTOR && (
          <Select
            placeholder="是否已上传数据"
            dict={hasValDict}
            value={hasVal}
            onChange={(val) => {
              this.setState({ hasVal: val });
            }}
            allowClear={false}
          />
        )}

        {showPlanImport ? (
          <Button
            style={{ width: '100%' }}
            type="primary"
            data-tips="无限制"
            onClick={(e) => {
              this.onShowImportModal('PLAN', true);
            }}
          >
            规划目标导入
          </Button>
        ) : (
          showPlanImportLimit && (
            <Button
              style={{ width: '100%' }}
              type="primary"
              data-tips="不可导入历史"
              onClick={(e) => {
                this.onShowImportModal('PLAN', false);
              }}
            >
              规划目标导入
            </Button>
          )
        )}

        {showGoalImport ? (
          <Button
            style={{ width: '100%', fontSize: '12px' }}
            type="primary"
            data-tips="无限制"
            onClick={(e) => {
              this.onShowImportModal('GOAL', true);
            }}
          >
            确认目标导入
          </Button>
        ) : (
          showGoalImportLimit && (
            <Button
              style={{ width: '100%', fontSize: '12px' }}
              type="primary"
              data-tips="不可导入历史"
              onClick={(e) => {
                this.onShowImportModal('GOAL', false);
              }}
            >
              确认目标导入
            </Button>
          )
        )}
      </SearchBar>
    );
  }
}

function mapStateToProps(state) {
  const { dicts } = state[STORE_NAME];

  return { dicts };
}

export default connect(mapStateToProps)(CommonSearch);
