import { Button, Menu, Dropdown, Icon } from 'antd';
import style from './index.less';

/**
 * 格式化按钮
 * @param props
 * @return {*|({size, name, type}&{})|*|({size, name, type}&{})}
 */
function formatAction(props) {
  const defaultAction = {
    name: 'Actions',
    size: 'large',
    type: 'primary',
  };

  return { ...defaultAction, ...props };
}

/**
 * 格式化菜单
 * @param props
 * @return {*}
 */
function formatMenu(props) {
  const menu = props.menu || [];

  return (
    <Menu
      onClick={(e) => {
        onMenuClick(e, props);
      }}
    >
      {menu.map((item, index) => {
        return (
          <Menu.Item key={index} disabled={!!item.disabled}>
            {item.name}
          </Menu.Item>
        );
      })}
    </Menu>
  );
}

/**
 * 点击菜单
 * @param e
 * @param props
 */
function onMenuClick(e, props) {
  const { key } = e;
  const { menu, handleMenuClick } = props;

  if (menu && handleMenuClick) {
    handleMenuClick(menu[Number(key)]);
  }
}

/**
 * 调用：
 * <DropdownButton name="确认目标导入" menu={[{name: '无限制导入', type: 1}, {name: '不可导入历史', type: 2}]} handleMenuClick={e => {console.log(e)}} />
 * @param props
 * @return {*}
 * @constructor
 */
const DropdownButton = (props) => {
  const action = formatAction(props);
  const menu = formatMenu(props);
  const { name, size, type } = action;

  return (
    <Dropdown overlay={menu}>
      <Button className={style.buttonWrap} size={size} type={type}>
        <span className={style.actionName}>{name}</span>
        <Icon type="down" style={{ marginLeft: 0 }} />
      </Button>
    </Dropdown>
  );
};

export default DropdownButton;
