import React from 'react';
import Modal from 'COMPONENT/Modal';
import TextLink from 'COMPONENT/TextLink';
import { Table } from 'antd';

export default class ImportResultModal extends React.Component {
  static defaultProps = {
    onClose: () => {},
    data: {
      totalNum: 0,
      successNum: 0,
      partlySuccessNum: 0,
      failNum: 0,
      resultUrl: '',
    },
  };

  constructor(props) {
    super(props);
  }

  /**
   * 表格列
   */
  formatColumn() {
    return [
      {
        title: '导入总数量',
        dataIndex: 'totalNum',
        key: 'totalNum',
      },
      {
        title: '成功数量',
        dataIndex: 'successNum',
        key: 'successNum',
      },
      {
        title: '部分成功数量',
        dataIndex: 'partlySuccessNum',
        key: 'partlySuccessNum',
      },
      {
        title: '失败数量',
        dataIndex: 'failNum',
        key: 'failNum',
      },
    ];
  }

  /**
   * 格式化导入结果
   * @param data 导入结果
   */
  formatData(data = {}) {
    let obj = {
      key: 0,
      totalNum: 0,
      successNum: 0,
      partlySuccessNum: 0,
      failNum: 0,
      resultUrl: '',
    };

    if (data) {
      obj = { ...obj, ...data };
    }

    return [obj];
  }

  /**
   * 导入结果下载
   * @param url
   */
  downloadTemplate(url = '') {
    if (!url) {
      return false;
    }

    const link = document.createElement('a');
    const body = document.body;

    link.href = url;
    link.style.display = 'none';
    link.download = '导入结果.xlsx';
    body.appendChild(link);
    link.click();
    body.removeChild(link);
  }

  render() {
    const { data } = this.props;
    const list = this.formatData(data);

    return (
      <Modal
        loading={false}
        onCancel={this.props.onClose}
        title="导入结果"
        width={400}
        cancelText="关闭"
        okBtnShow={false}
      >
        <Table columns={this.formatColumn()} dataSource={list} bordered pagination={false} />

        <div style={{ marginTop: 15 }}>
          <TextLink
            onClick={() => {
              this.downloadTemplate(data.resultUrl);
            }}
          >
            导入结果下载
          </TextLink>
        </div>
      </Modal>
    );
  }
}
