import React from 'react';
import { connect } from 'dva';
import Modal from 'COMPONENT/Modal';
import { Col, Row } from 'antd';
import FormItem from 'COMPONENT/FormItem';
import Select from 'COMPONENT/Select';
import ChooseFileButton from 'PAGE/personnelManage/TeacherIntroduction/ChooseFileButton';
import TextLink from 'COMPONENT/TextLink';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { urlBase } from 'CONST/config';
import { OSS_BASE_PREFIX } from 'CONST/ossConfig';
import { formatOrgTypeDict, formatImportTypeDict } from '../../dicts';
import { getDictTextByVal } from 'UTIL/dict';
import ImportResultModal from '../ImportResultModal';
import { STORE_NAME } from '../../store/config';

class ImportTemplateModal extends React.Component {
  static defaultProps = {
    onClose: () => {},
    pageType: 'CashFlow', // CashFlow: 预收, SmallClass: 小班科次, Course: 小时消耗, YearExamGoal: 全年中考目标人数
    pageImportType: '', // 导入类型 'PLAN'|'GOAL'
    canImportHistory: false, // 能否允许导入历史数据，true：能导入，false：不能导入
  };

  constructor(props) {
    super(props);

    this.downloadTemplate = this.downloadTemplate.bind(this);
    this.onImport = this.onImport.bind(this);
    this.doImport = this.doImport.bind(this);
    this.formatParam = this.formatParam.bind(this);
    this.onClose = this.onClose.bind(this);
    this.showLoading = this.showLoading.bind(this);
    this.hideLoading = this.hideLoading.bind(this);

    this.state = {
      isLoading: false,

      year: null, // 年份
      orgType: 'CAMPUS', // 组织类型
      importType: null, // 导入目标类型 'PLAN'|'GOAL'
      file: null, // 文件

      templateDict: [], // 模版列表
      templateList: null, // 当前类型模版列表

      importResult: null, // 导入结果
      showImportResultModal: false, // 显示导入结果弹窗
    };

    this.goalTypeCfg = {
      CashFlow: 'CASH_FLOW',
      SmallClass: 'MINI_CLASS_SUBJECT_COUNT',
      Course: 'COURSE_HOUR_CONSUME',
      YearExamGoal: 'PROMISE_CLASS_STUDENT_NUMBER',
      ActualIncome: 'REAL_RECEIVE',
      Personalise: 'SPECIAL_SIGN_MINI_CLASS_SUBJECT_COUNT',
    };

    this.titleCfg = {
      CashFlow: '预收',
      SmallClass: '小班科次',
      Course: '小时消耗',
      YearExamGoal: '全年中考目标人数',
      ActualIncome: '实收',
      Personalise: '个性化签班课科次',
    };

    this.templateCfg = {
      CashFlow: {
        BRENCH: { url: '/%E9%A2%84%E6%94%B6%E5%88%86%E5%85%AC%E5%8F%B8.xlsx' },
        CAMPUS: { url: '/%E9%A2%84%E6%94%B6%E6%A0%A1%E5%8C%BA.xlsx' },
      },
      SmallClass: {
        BRENCH: { url: '/%E5%B0%8F%E7%8F%AD%E7%A7%91%E6%AC%A1%E5%88%86%E5%85%AC%E5%8F%B8.xlsx' },
        CAMPUS: { url: '/%E5%B0%8F%E7%8F%AD%E7%A7%91%E6%AC%A1%E6%A0%A1%E5%8C%BA.xlsx' },
      },
      Course: {
        BRENCH: { url: '/%E8%AF%BE%E7%A8%8B%E5%B0%8F%E6%97%B6%E6%B6%88%E8%80%97%E5%88%86%E5%85%AC%E5%8F%B8.xlsx' },
        CAMPUS: { url: '/%E8%AF%BE%E7%A8%8B%E5%B0%8F%E6%97%B6%E6%B6%88%E8%80%97%E6%A0%A1%E5%8C%BA.xlsx' },
      },
      YearExamGoal: {
        BRENCH: {
          url: '/%E7%9B%AE%E6%A0%87%E7%8F%AD%E6%8B%9B%E7%94%9F%E4%BA%BA%E6%95%B0%E5%88%86%E5%85%AC%E5%8F%B8.xlsx',
        },
        CAMPUS: { url: '/%E7%9B%AE%E6%A0%87%E7%8F%AD%E6%8B%9B%E7%94%9F%E4%BA%BA%E6%95%B0%E6%A0%A1%E5%8C%BA.xlsx' },
      },
      ActualIncome: {
        BRENCH: { url: '/%E5%AE%9E%E6%94%B6%E5%88%86%E5%85%AC%E5%8F%B8.xlsx' },
        CAMPUS: { url: '/%E5%AE%9E%E6%94%B6%E6%A0%A1%E5%8C%BA.xlsx' },
      },
      Personalise: {
        BRENCH: {
          url: '/%E4%B8%AA%E6%80%A7%E5%8C%96%E7%AD%BE%E7%8F%AD%E8%AF%BE%E7%A7%91%E6%AC%A1%E5%88%86%E5%85%AC%E5%8F%B8.xlsx',
        },
        CAMPUS: {
          url: '/%E4%B8%AA%E6%80%A7%E5%8C%96%E7%AD%BE%E7%8F%AD%E8%AF%BE%E7%A7%91%E6%AC%A1%E6%A0%A1%E5%8C%BA.xlsx',
        },
      },
    };
  }

  /**
   * 下载模版
   */
  downloadTemplate() {
    const { pageType } = this.props;
    const { orgType, importType } = this.state;

    if (!orgType || !importType) {
      showMessage('请选组织类型与导入目标类型！');
      return false;
    }

    let template = this.templateCfg[pageType][orgType] || {};

    let title = this.titleCfg[pageType] || '';
    title = title + getDictTextByVal(formatOrgTypeDict(1), orgType);
    title = title + getDictTextByVal(formatImportTypeDict(''), importType);
    // `预收分公司确认目标导入模版.xls`
    template.text = title + '导入模板.xls';

    this.doDownload(template);
  }

  doDownload(template) {
    if (!template || !template.url) {
      return false;
    }

    const link = document.createElement('a');
    const body = document.body;

    link.href = OSS_BASE_PREFIX + template.url;
    link.style.display = 'none';
    link.download = template.text;
    body.appendChild(link);
    link.click();
    body.removeChild(link);
  }

  /**
   * 选择文件
   */
  onSelectFile(file) {
    this.setState({ file: file });
  }

  /**
   * 导入
   */
  onImport() {
    const param = this.formatParam();
    const { yearId, organizationType, importType, file } = param;

    if (!file) {
      showMessage('请选择文件');
      return false;
    }

    if (!yearId || !organizationType || !importType) {
      showMessage('请完整选择导入年份、组织类型及导入目标类型');
      return false;
    }

    this.doImport(param);
  }

  doImport(param) {
    this.showLoading();

    const formData = new FormData();
    for (let key in param) {
      formData.append(key, param[key]);
    }

    const xhr = new XMLHttpRequest();
    xhr.onload = () => {
      try {
        this.hideLoading();

        let result = {};
        try {
          result = JSON.parse(xhr.responseText);
        } catch (e) {}

        if (xhr.status === 200 && result.resultCode === 0) {
          const data = result.data;
          const { failNum, partlySuccessNum } = data;
          const hasFailure = Number(failNum || 0) > 0 || Number(partlySuccessNum || 0) > 0;

          if (hasFailure) {
            this.setState({
              showImportResultModal: true,
              importResult: data,
            });
          } else {
            showMessage('导入成功', 'success');
            this.onClose();
          }
        } else {
          showMessage(result.resultMessage);
        }
      } catch (e) {
        showMessage('系统错误');
        this.hideLoading();
      }
    };
    xhr.onerror = () => {
      showMessage('网络错误!');
      this.hideLoading();
    };
    xhr.open('POST', urlBase + '/MarketingGoalManagementController/importMarketingGoal.do');
    xhr.send(formData);
  }

  /**
   * 格式化请求参数
   */
  formatParam() {
    const { pageType, canImportHistory } = this.props;
    const { year, orgType, importType, file } = this.state;

    return {
      goalType: this.goalTypeCfg[pageType] || '',
      yearId: year,
      organizationType: orgType,
      importType: importType,
      canImportHistoryData: canImportHistory,
      file: file,
    };
  }

  /**
   * 关闭导入弹窗
   */
  onClose() {
    this.setState({ file: null });
    this.props.onClose();
  }

  showLoading() {
    this.setState({ isLoading: true });
  }

  hideLoading() {
    this.setState({ isLoading: false });
  }

  render() {
    const { isLoading, year, orgType, importType, file, importResult, showImportResultModal } = this.state;
    const { dicts, pageImportType } = this.props;
    const labelWidth = 100;

    let orgTypeDict = null;
    if (pageImportType.toLowerCase() === 'plan') {
      orgTypeDict = formatOrgTypeDict(1);
    } else {
      orgTypeDict = formatOrgTypeDict(3);
    }

    return (
      <Modal
        loading={isLoading}
        onCancel={this.onClose}
        title="目标导入"
        width={600}
        okText="导入"
        cancelText="关闭"
        okBtnShow={true}
        onOk={this.onImport}
        okBtnDisabled={isLoading}
      >
        <Row gutter={10}>
          <Col span={24}>
            <FormItem name="导入年份" labelWidth={labelWidth}>
              <Select
                placeholder="请选择"
                dict={dicts.yearDict}
                value={year}
                onChange={(val) => {
                  this.setState({ year: val });
                }}
              />
            </FormItem>

            <FormItem name="组织类型" labelWidth={labelWidth}>
              <Select
                placeholder="请选择"
                dict={orgTypeDict}
                value={orgType}
                onChange={(val) => {
                  this.setState({ orgType: val });
                }}
              />
            </FormItem>

            <FormItem name="导入目标类型" labelWidth={labelWidth}>
              <Select
                placeholder="请选择"
                dict={formatImportTypeDict(pageImportType)}
                value={importType}
                onChange={(val) => {
                  this.setState({ importType: val });
                }}
              />
            </FormItem>

            <FormItem name="" labelWidth={labelWidth}>
              <TextLink
                onClick={() => {
                  this.downloadTemplate();
                }}
              >
                下载模板
              </TextLink>
            </FormItem>

            <FormItem name="上传文件" labelWidth={labelWidth}>
              <ChooseFileButton
                key="upload"
                size="small"
                btnText="选择文件"
                accept="application/vnd.openxmlformats-officedocument.spreadsheetml.sheet,application/vnd.ms-excel"
                fileType={['excel']}
                onChange={(file) => this.onSelectFile(file)}
              />
              {file && <span style={{ marginLeft: '15px' }}>{file.name}</span>}
            </FormItem>
          </Col>
        </Row>

        {showImportResultModal && (
          <ImportResultModal
            data={importResult || {}}
            onClose={() => {
              this.setState(
                {
                  showImportResultModal: false,
                  importResult: null,
                },
                () => {
                  this.onClose();
                },
              );
            }}
          />
        )}
      </Modal>
    );
  }
}

function mapStateToProps(state) {
  const { dicts } = state[STORE_NAME];

  return { dicts };
}

export default connect(mapStateToProps)(ImportTemplateModal);
