import React from 'react';
import { Input, InputNumber, Form } from 'antd';
const FormItem = Form.Item;

/**
 * 表格输入/展示单元
 */
class Cell extends React.Component {
  constructor(props) {
    super(props);
  }

  /**
   * 渲染编辑状态下的输入框
   * 根据 column.inputType 判断显示哪个输入框
   * @param record 每一行数据
   * @param column 列配置
   * @param onChange
   */
  renderInput(record, column, onChange) {
    const { inputType } = column;
    const style = { margin: 0, padding: '0 8px', height: '24px' };

    if (inputType && inputType === 'number') {
      return <InputNumber style={style} onChange={(value) => onChange(value)} />;
    } else {
      return <Input style={style} onChange={(e) => onChange(e.target.value)} />;
    }
  }

  /**
   * 渲染展示单元
   * @param value
   * @param record
   * @param column
   */
  renderValue(value, record, column) {
    const { style } = column;
    let st = {};

    if (style) {
      if (Object.prototype.toString.call(style) === '[object Function]') {
        st = style(record);
      } else if (Object.prototype.toString.call(style) === '[object Object]') {
        st = style;
      }
    }

    return <span style={st}>{value}</span>;
  }

  /**
   * 判断能否编辑单个单元格
   * 根据 column.canBeEdit 判断单元格是否允许被编辑
   * @param record 每一行数据
   * @param column 列配置
   */
  isInput(record, column) {
    const { canBeEdit } = column;

    if (canBeEdit !== undefined) {
      return Object.prototype.toString.call(canBeEdit) === '[object Function]' ? canBeEdit(record) : !!canBeEdit;
    } else {
      return true;
    }
  }

  render() {
    const { form, editable, value, dataIndex, record, column, onChange } = this.props;
    const isInput = this.isInput(record, column);

    return (
      <div>
        {editable ? (
          isInput ? (
            <FormItem style={{ margin: 0 }}>
              {form.getFieldDecorator(dataIndex, {
                rules: [
                  {
                    required: false,
                    message: ' ',
                  },
                ],
                initialValue: value,
              })(this.renderInput(record, column, onChange))}
            </FormItem>
          ) : (
            this.renderValue(value, record, column)
          )
        ) : (
          this.renderValue(value, record, column)
        )}
      </div>
    );
  }
}

export default Cell;
