import React from 'react';
import ReactDOM from 'react-dom';
import { connect } from 'dva';
import PersonaliseSearch from '../../common/PersonaliseSearch';
import TableForm, { ColWidth, renderTableFooter } from '../../common/TableForm';
import ImportTemplateModal from '../../common/ImportTemplateModal';
import { getGoalList, updateGoalValue } from 'SERVICE/goalManage';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import {
  formatRecordToApiParam,
  formatRedStyle,
  renderRedText,
  formatCountRow,
  formatYearText,
  isHistoryQuarter,
  renderBranchText,
} from '../../util';
import TheWrapper from '../../common/TheWrapper';
import { focusResize } from 'UTIL/common';

const GOAL_TYPE = 'SPECIAL_SIGN_MINI_CLASS_SUBJECT_COUNT';
const PAGE_TYPE = 'Personalise';

const tableFooterId = 'personalise-table-footer';

class Personalise extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      searchParam: {}, // 搜索条件

      tableKey: '',
      listLoading: false,
      list: [], // 列表数据
      pageNo: 1, // 当前页码
      pageSize: 20, // 每页记录数
      totalRecords: 0, // 总记录数
      countRow: null, // 汇总行数据
      timeStamp: null, // 服务器时间戳

      showImportModal: false, // 显示导入弹窗
      importType: '', // 导入类型 'PLAN'|'GOAL'
      canImportHistory: false, // 能够导入历史数据
    };

    this.$ref = {
      searchRef: null,
    };

    this.tagRef = null;

    this.setTableFixedWidth = this.setTableFixedWidth.bind(this);
    this.onSearch = this.onSearch.bind(this);
    this.loadList = this.loadList.bind(this);
    this.formatParam = this.formatParam.bind(this);
    this.formatResponse = this.formatResponse.bind(this);
    this.setListData = this.setListData.bind(this);
    this.formatColumn = this.formatColumn.bind(this);
    this.formatQuarterColumn = this.formatQuarterColumn.bind(this);
    this.onSaveRow = this.onSaveRow.bind(this);
  }

  componentDidMount() {
    window.addEventListener('resize', this.setTableFixedWidth, false);
    focusResize();
  }

  // 补偿表格固定操作宽度列表(rc-table bug)
  setTableFixedWidth() {
    const listNode = ReactDOM.findDOMNode(this.tagRef);
    const fixedWrap = listNode.querySelector('.ant-table-fixed-left');
    const scrollWrap = listNode.querySelector('.ant-table-scroll');

    const thNodes = scrollWrap.querySelector('.ant-table-header').querySelectorAll('th');
    const thFixNodes = fixedWrap.querySelectorAll('th');

    const tdNodes = scrollWrap.querySelector('.ant-table-body').querySelectorAll('td');
    const tdFixNodes = fixedWrap.querySelectorAll('td');

    const hadData = tdNodes.length > 0;

    Array.prototype.forEach.call(thFixNodes, (item, index) => {
      const thWidth = thNodes[index].offsetWidth;
      item.style['min-width'] = `${thWidth}px`;
      item.style['max-width'] = `${thWidth}px`;

      if (hadData) {
        const tdWidth = tdNodes[index].offsetWidth;
        tdFixNodes[index].style['min-width'] = `${tdWidth}px`;
        tdFixNodes[index].style['max-width'] = `${tdWidth}px`;
      }
    });

    if (hadData) {
      const footerWrap = document.querySelector(`#${tableFooterId}`);
      const tdFooterScrollNodes = footerWrap.querySelector('.table-footer-scroll').querySelectorAll('td');
      const footerFixedNodes = footerWrap.querySelector('.table-footer-fixed');
      const tdFooterFixedNodes = footerFixedNodes.querySelectorAll('td');

      const fixedWidth = fixedWrap.offsetWidth;
      footerFixedNodes.style['min-width'] = `${fixedWidth}px`;
      footerFixedNodes.style['max-width'] = `${fixedWidth}px`;

      Array.prototype.forEach.call(tdNodes, (item, index) => {
        const tdWidth = item.offsetWidth;

        if (tdFooterScrollNodes[index]) {
          tdFooterScrollNodes[index].style['min-width'] = `${tdWidth}px`;
          tdFooterScrollNodes[index].style['max-width'] = `${tdWidth}px`;
        }

        if (tdFooterFixedNodes[index]) {
          tdFooterFixedNodes[index].style['min-width'] = `${tdWidth}px`;
          tdFooterFixedNodes[index].style['max-width'] = `${tdWidth}px`;
        }
      });
    }
  }

  onSearch() {
    this.loadList({ pageNo: 1 });
  }

  /**
   * 请求列表
   */
  loadList(changedParams = {}) {
    let param = this.formatParam(changedParams);

    this.setState({ listLoading: true });

    getGoalList(param)
      .then((response) => {
        let result = this.formatResponse(response);
        this.setListData(result, param);
      })
      .catch((e) => {
        let result = this.formatResponse({});
        this.setListData(result, param);
      });
  }

  /**
   * 格式化请求参数并保存搜素条件, 注意参数的分页参数
   * param.goalType 对应tab页面写死
   * @param changeParam
   */
  formatParam(changeParam = {}) {
    const { pageNo, pageSize } = this.state;
    const searchParam = this.$ref.searchRef.getParam();

    const param = {
      pageNo,
      pageSize,
      getTotal: true,
      ...searchParam,
      ...changeParam,
    };

    param.page = param.pageNo;
    param.rows = param.pageSize;
    param.goalType = GOAL_TYPE;

    this.setState({
      searchParam: { goalType: param.goalType, ...searchParam },
    });

    return param;
  }

  /**
   * 格式化列表数据,根据所选时间类型分类、汇总
   * 注意修改页面汇总的列位置，处于columns的哪一列
   */
  formatResponse(response = {}) {
    let data = response.data || [];
    let countRow = null;
    let countRowKey = 'operation';

    if (data.length > 0) {
      countRow = {};
      countRow[countRowKey] = '页面汇总';

      countRow = formatCountRow(data, countRow, countRowKey);
    }

    return {
      list: data,
      total: response.rowCount || 0,
      countRow: countRow,
      timeStamp: response.timeStamp || null,
    };
  }

  /**
   * 保存数据
   */
  setListData(result = {}, param = {}) {
    this.setState(
      {
        tableKey: new Date().getTime(),
        listLoading: false,
        list: result.list || [],
        totalRecords: result.total || 0,
        countRow: result.countRow,
        pageNo: param.pageNo,
        pageSize: param.pageSize,
        timeStamp: result.timeStamp,
      },
      focusResize,
    );
  }

  /**
   * 格式化表头, 季度才允许编辑(小班科次只有季度)
   */
  formatColumn() {
    const { btnAuth_edit, btnAuth_edit_limit } = this.props;
    const { searchParam } = this.state;
    const { yearText, organizationType } = searchParam;
    const showEditColumn = true;
    const editAuth = btnAuth_edit || btnAuth_edit_limit;
    const showCampusColumn = organizationType === 'CAMPUS';

    let leftColumn = this.formatLeftColumn(showEditColumn, editAuth, showCampusColumn);
    let centerColumn = this.formatQuarterColumn(yearText);
    const rightColumn = this.formatRightColumn();

    return leftColumn.concat(centerColumn, rightColumn);
  }

  /**
   * 左边表头
   * @param showEditColumn 是否显示操作列
   * @param editAuth 编辑按钮权限
   * @param showCampusColumn 是否显示学习中心列 仅当选择学习中心的时候显示
   */
  formatLeftColumn(showEditColumn = false, editAuth = false, showCampusColumn = false) {
    let column = [
      {
        title: '集团',
        dataIndex: 'groupName',
        width: ColWidth.WIDTH_XXS,
        fixed: 'left',
      },
      {
        title: '分公司',
        dataIndex: 'branchName',
        width: ColWidth.WIDTH_XXS,
        fixed: 'left',
        render: (text, record) => {
          return renderBranchText(text, record.hasVal);
        },
      },
    ];

    if (showCampusColumn) {
      column.push({
        title: '学习中心',
        dataIndex: 'campusName',
        width: ColWidth.WIDTH_XM,
        fixed: 'left',
      });
    }

    if (showEditColumn) {
      column.push({
        title: '操作',
        dataIndex: 'operation',
        width: 90,
        fixed: 'left',
        align: 'center',
        type: 'operation',
        showEdit: editAuth,
        canDoEdit: (record) => {
          return !!record.campusId;
        },
      });
    }

    return column;
  }

  /**
   * 季度表头
   * @param year 年份
   */
  formatQuarterColumn(year = '') {
    const { btnAuth_edit } = this.props;
    const { timeStamp } = this.state;
    const { lastYear, thisYear } = formatYearText(year);

    return [
      {
        title: `${lastYear}秋季`,
        children: [
          {
            title: '规划目标',
            dataIndex: 'autumnPlanValue',
            width: ColWidth.WIDTH_M,
          },
          {
            title: '确认目标',
            dataIndex: 'autumnGoalValue',
            width: ColWidth.WIDTH_M,
            type: 'input',
            inputType: 'number',
            canBeEdit: (record) => {
              if (btnAuth_edit) {
                return true;
              }

              return !isHistoryQuarter(timeStamp, parseInt(lastYear), 'AUTUMN');
            },
            style: (record) => {
              return formatRedStyle(record.autumnGoalValue, record.autumnPlanValue);
            },
          },
        ],
      },
      {
        title: `${thisYear}寒假`,
        children: [
          {
            title: '规划目标',
            dataIndex: 'winterPlanValue',
            width: ColWidth.WIDTH_M,
          },
          {
            title: '确认目标',
            dataIndex: 'winterGoalValue',
            width: ColWidth.WIDTH_M,
            type: 'input',
            inputType: 'number',
            canBeEdit: (record) => {
              if (btnAuth_edit) {
                return true;
              }

              return !isHistoryQuarter(timeStamp, parseInt(thisYear), 'WINTER');
            },
            style: (record) => {
              return formatRedStyle(record.winterGoalValue, record.winterPlanValue);
            },
          },
        ],
      },
      {
        title: `${thisYear}春季`,
        children: [
          {
            title: '规划目标',
            dataIndex: 'springPlanValue',
            width: ColWidth.WIDTH_M,
          },
          {
            title: '确认目标',
            dataIndex: 'springGoalValue',
            width: ColWidth.WIDTH_M,
            type: 'input',
            inputType: 'number',
            canBeEdit: (record) => {
              if (btnAuth_edit) {
                return true;
              }

              return !isHistoryQuarter(timeStamp, parseInt(thisYear), 'SPRING');
            },
            style: (record) => {
              return formatRedStyle(record.springGoalValue, record.springPlanValue);
            },
          },
        ],
      },
      {
        title: `${thisYear}暑假`,
        children: [
          {
            title: '规划目标',
            dataIndex: 'summerPlanValue',
            width: ColWidth.WIDTH_M,
          },
          {
            title: '确认目标',
            dataIndex: 'summerGoalValue',
            width: ColWidth.WIDTH_M,
            type: 'input',
            inputType: 'number',
            canBeEdit: (record) => {
              if (btnAuth_edit) {
                return true;
              }

              return !isHistoryQuarter(timeStamp, parseInt(thisYear), 'SUMMER');
            },
            style: (record) => {
              return formatRedStyle(record.summerGoalValue, record.summerPlanValue);
            },
          },
        ],
      },
    ];
  }

  /**
   * 右边汇总
   */
  formatRightColumn() {
    return [
      {
        title: '年度总目标',
        children: [
          {
            title: '规划目标',
            dataIndex: 'yearPlanValue',
            width: ColWidth.WIDTH_XXS,
          },
          {
            title: '确认目标',
            dataIndex: 'yearGoalValue',
            width: ColWidth.WIDTH_XXS,
            render: (text, record) => {
              return renderRedText(text, record.yearGoalValue, record.yearPlanValue);
            },
          },
        ],
      },
    ];
  }

  /**
   * 保存编辑
   */
  onSaveRow(rowData, callback) {
    const { btnAuth_edit } = this.props;
    const param = formatRecordToApiParam(rowData, GOAL_TYPE, btnAuth_edit);

    this.setState({ listLoading: true });

    updateGoalValue(param)
      .then((response) => {
        showMessage('保存成功！', 'success');
        callback.call();
        this.setState({ listLoading: false });
        this.loadList();
      })
      .catch((e) => {
        showMessage('保存失败，请重试！');
        this.setState({ listLoading: false });
      });
  }

  render() {
    const { btnAuth_plan_import, btnAuth_plan_import_limit, btnAuth_goal_import, btnAuth_goal_import_limit } =
      this.props;
    const {
      tableKey,
      list,
      listLoading,
      pageNo,
      pageSize,
      totalRecords,
      countRow,
      showImportModal,
      importType,
      canImportHistory,
    } = this.state;
    const columns = this.formatColumn();

    return (
      <TheWrapper>
        <section className="tabPanelWrap">
          <PersonaliseSearch
            planImportDisabled={!btnAuth_plan_import}
            planImportLimitDisabled={!btnAuth_plan_import_limit}
            goalImportDisabled={!btnAuth_goal_import}
            goalImportLimitDisabled={!btnAuth_goal_import_limit}
            onSearch={this.onSearch}
            onRefs={(ref) => {
              this.$ref.searchRef = ref;
            }}
            onShowImportModal={(type, history) => {
              this.setState({
                showImportModal: true,
                importType: type,
                canImportHistory: history,
              });
            }}
            {...this.props}
          />

          <TableForm
            ref={(c) => (this.tagRef = c)}
            key={tableKey}
            loading={listLoading}
            list={list}
            columns={columns}
            pagination={{
              current: pageNo,
              pageSize: pageSize,
              total: totalRecords,
            }}
            onPageChange={({ current, pageSize }) => this.loadList({ pageNo: current, pageSize })}
            onSaveRow={(rowData, callback) => {
              this.onSaveRow(rowData, callback);
            }}
            footer={() => renderTableFooter(columns, countRow, tableFooterId)}
          />

          {showImportModal && (
            <ImportTemplateModal
              pageType={PAGE_TYPE}
              pageImportType={importType}
              canImportHistory={canImportHistory}
              onClose={() => this.setState({ showImportModal: false })}
            />
          )}
        </section>
      </TheWrapper>
    );
  }
}

function mapStateToProps(state) {
  const { btnAuths } = state.account;

  const btnAuth_edit = btnAuths.indexOf('OPERATION_SSMCSC_EDIT_FREE') > -1;
  const btnAuth_edit_limit = btnAuths.indexOf('OPERATION_SSMCSC_EDIT') > -1;
  const btnAuth_plan_import = btnAuths.indexOf('OPERATION_SSMCSC_PLANIMPORT_FREE') > -1;
  const btnAuth_plan_import_limit = btnAuths.indexOf('OPERATION_SSMCSC_PLANIMPORT') > -1;
  const btnAuth_goal_import = btnAuths.indexOf('OPERATION_SSMCSC_GOALIMPORT_FREE') > -1;
  const btnAuth_goal_import_limit = btnAuths.indexOf('OPERATION_SSMCSC_GOALIMPORT') > -1;

  return {
    btnAuth_edit,
    btnAuth_edit_limit,
    btnAuth_plan_import,
    btnAuth_plan_import_limit,
    btnAuth_goal_import,
    btnAuth_goal_import_limit,
  };
}

export default connect(mapStateToProps)(Personalise);
