import { ONE_ON_ONE_COURSE, ONE_ON_MANY, SMALL_CLASS, SPECIAL_SIGN_MINI_CLASS } from 'CONST/dicts/product';

/**
 * 目标类型(目前写死在tab请求参数)
 */
export const goalTypeDict = [
  { val: 'CASH_FLOW', text: '预收' },
  { val: 'MINI_CLASS_SUBJECT_COUNT', text: '小班科次' },
  { val: 'COURSE_HOUR_CONSUME', text: '小时消耗' },
  { val: 'PROMISE_CLASS_STUDENT_NUMBER', text: '精英班招生人数' },
];

/**
 * 组织类型
 */
export function formatOrgTypeDict(type = 1) {
  let dict = [
    { val: 'BRENCH', text: '分公司' },
    { val: 'CAMPUS', text: '学习中心' },
  ];

  if (type === 2) {
    dict.unshift({ val: 'GROUNP', text: '集团' });
  }

  if (type === 3) {
    dict = [{ val: 'CAMPUS', text: '学习中心' }];
  }

  return dict;
}

/**
 * 产品类型
 * @param type 1：预收
 */
export function formatProductDict(type = 1) {
  if (type === 1) {
    return [
      { val: ONE_ON_ONE_COURSE, text: '一对一' },
      { val: ONE_ON_MANY, text: '一对多' },
      { val: SMALL_CLASS, text: '小班' },
      { val: SPECIAL_SIGN_MINI_CLASS, text: '个性化签班课' },
    ];
  }

  return [
    { val: ONE_ON_ONE_COURSE, text: '一对一' },
    { val: ONE_ON_MANY, text: '一对多' },
  ];
}

/**
 * 预收类型
 */
export const cashTypeDict = [
  { val: 'NEW', text: '新签' },
  { val: 'RENEW', text: '续费' },
];

/**
 * 格式化季度字典
 * @param type 1: 只有季度，2：季度+月份, 3: 季度+月份+年份
 */
export function formatQuarterDict(type = 1) {
  let dict = [{ val: 'QUARTER', text: '季度' }];

  if (type === 2) {
    dict.unshift({ val: 'MONTH', text: '月份' });
  } else if (type === 3) {
    dict = dict.concat([
      { val: 'MONTH', text: '月份' },
      { val: 'YEAR', text: '年份' },
    ]);
  } else if (type === 4) {
    dict = [{ val: 'MONTH', text: '月份' }];
  }

  return dict;
}

/**
 * 导入目标类型(按需)
 */
export function formatImportTypeDict(type = '') {
  let dict = [
    { val: 'PLAN', text: '规划目标' },
    { val: 'GOAL', text: '确认目标' },
  ];

  if (type && type.toLowerCase() === 'plan') {
    dict = [{ val: 'PLAN', text: '规划目标' }];
  } else if (type && type.toLowerCase() === 'goal') {
    dict = [{ val: 'GOAL', text: '确认目标' }];
  }

  return dict;
}

/**
 * 班级字典特殊处理,只需要公益班和收费班
 * @param dictArray 完整班级字典
 */
export function formatClassTypeDict(dictArray) {
  let dict = [];

  if (dictArray && dictArray.length > 0) {
    dictArray.forEach((item) => {
      if (item.text === '公益班' || item.text === '收费班') {
        dict.push(item);
      }
    });
  }

  return dict;
}

/**
 * 是否已上传数据,用eval(val)转换
 */
export const HAS_VAL_VALID = 'true';
export const HAS_VAL_INVALID = 'false';
export const hasValDict = [
  { val: HAS_VAL_VALID, text: '已上传数据' },
  { val: HAS_VAL_INVALID, text: '待上传数据' },
];
