import React from 'react';
import { connect } from 'dva';
import style from './index.less';
import { Tabs } from 'antd';
const TabPane = Tabs.TabPane;
import Page from 'COMPONENT/Page';
import { loadCommonDictByGroup } from 'UTIL/dict';
import { formatClassTypeDict } from './dicts';
import { STORE_NAME } from './store/config';

class OperationGoalStatement extends React.Component {
  constructor(props) {
    super(props);

    this.initPage = this.initPage.bind(this);
    this.onUpdateStore = this.onUpdateStore.bind(this);
  }

  componentDidMount() {
    this.initPage();
  }

  initPage() {
    const { dispatch } = this.props;

    dispatch({ type: `${STORE_NAME}/initTabs` });

    const cateGory = [
      { category: 'PRODUCT_VERSION', parentId: '' },
      { category: 'STUDENT_GRADE', parentId: '' },
      { category: 'SUBJECT', parentId: '' },
      { category: 'CLASS_TYPE', parentId: 'SMALL_CLASS' },
    ];
    Promise.all([loadCommonDictByGroup(cateGory)]).then(([groupDict]) => {
      const smallClassTypeDict = formatClassTypeDict(groupDict['CLASS_TYPE']);

      this.onUpdateStore('dicts', {
        yearDict: groupDict['PRODUCT_VERSION'],
        smallClassTypeDict: smallClassTypeDict,
        gradeDict: groupDict['STUDENT_GRADE'],
        subjectDict: groupDict['SUBJECT'],
      });
    });
  }

  /**
   * 更新store
   * @param key 键
   * @param value 值
   */
  onUpdateStore(key, value) {
    this.props.dispatch({
      type: `${STORE_NAME}/updateState`,
      payload: { [key]: value },
    });
  }

  render() {
    const { route, tabs, tab_key } = this.props;

    return (
      <Page title={false} name="营运目标统计" className={style.OperationGoalStatement} loading={false} route={route}>
        <Tabs
          activeKey={tab_key}
          onChange={(key) => {
            this.onUpdateStore('tab_key', key);
          }}
        >
          {tabs.map((tab) => {
            const TabComponent = tab.component;
            const { title, key } = tab;

            return (
              <TabPane tab={title} key={key}>
                <TabComponent paneKey={key} />
              </TabPane>
            );
          })}
        </Tabs>
      </Page>
    );
  }
}

function mapStateToProps(state) {
  const { tabs, tab_key } = state[STORE_NAME];

  return { tabs, tab_key };
}

export default connect(mapStateToProps)(OperationGoalStatement);
