import { clone } from 'lodash';
import { STORE_NAME } from './config';
import { defaultState } from './state';
import { tabArray } from '../config';

export default {
  namespace: STORE_NAME,
  state: {
    ...clone(defaultState),
  },
  reducers: {
    /**
     * @Attention 因为tab有多个，这里需要深度拷贝(深度为2)
     * @param state
     * @param payload
     */
    updateState(state, { payload }) {
      let newState = { ...state };

      if (payload) {
        for (let i in payload) {
          let item = payload[i];
          if (item && Object.prototype.toString.call(item) === '[object Object]') {
            newState[i] = { ...state[i], ...item };
          } else {
            newState[i] = item;
          }
        }
      }

      return {
        ...newState,
      };
    },
  },
  effects: {
    /**
     * 初始化用户权限tab
     * @param payload
     * @param put
     * @param select
     * @return {IterableIterator<*>}
     */
    *initTabs({ payload }, { put, select }) {
      let tab_key = '';
      const tabs = yield select((state) => {
        const { btnAuths } = state.account;
        let result = [];

        tabArray.forEach((item) => {
          if (btnAuths.includes(item.authId)) {
            result.push(item);
          }
        });

        return result;
      });

      if (tabs && tabs.length > 0) {
        tab_key = tabs[0].key;
      }

      yield put({
        type: 'updateState',
        payload: {
          tabs,
          tab_key,
        },
      });
    },
  },
};
