import React from 'react';
import { connect } from 'dva';
import SearchButton from 'COMPONENT/SearchButton';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import DatePicker from 'COMPONENT/DatePicker';
import { DATE_FORMAT } from 'COMPONENT/utils/form';
import moment from 'moment';
import {
  loadCommonDictByGroup,
  loadBranchDict,
  loadBranchCampusDict,
  dictAddOptionTitle,
  getOptionValueInFormatterDict,
} from 'UTIL/dict';
import { loadConsultantDict } from 'PAGE/common/loaders';
import { getLoginUserMainRole } from 'SERVICE/common';
import { isRealCounselor } from '../../config';

class HeadSearch extends React.Component {
  static defaultProps = {
    handleSearch: () => {},
    onRef: () => {},
  };

  constructor(props) {
    super(props);

    this.state = {
      branchId: null, // 分公司
      blCampusId: null, // 学习中心 默认不选中
      teacherId: null, // 咨询师
      fileType: null, // 文件类型
      status: null, // 状态
      startTime: null, // 开始时间
      endTime: null, // 结束时间

      teacherDisable: false, // 咨询师禁用
      campusDisable: false, // 学习中心禁用
      branchDisable: false, // 分公司禁用

      branchDict: null,
      branchCampusDict: null,
      teacherDict: null,
      typeDict: null,
      statusDict: null,
    };

    this.initParam = {
      branchId: null,
      blCampusId: null,
      teacherId: null,
      fileType: null,
      status: null,
      startTime: null,
      endTime: null,
    };

    this.initDict = this.initDict.bind(this);
    this.loadTeacherDict = this.loadTeacherDict.bind(this);
    this.onBranchChange = this.onBranchChange.bind(this);
    this.onCampusChange = this.onCampusChange.bind(this);
    this.onSearch = this.onSearch.bind(this);
    this.onReset = this.onReset.bind(this);
    this.getParam = this.getParam.bind(this);
    this.getDict = this.getDict.bind(this);
  }

  componentDidMount() {
    this.props.onRef(this);
    this.initDict();
  }

  /**
   * 初始化字典，根据用户所属组织初始化分公司、学习中心及咨询师
   * 除咨询师需用主组织角色判断，其他用登录用户信息的组织判断
   */
  initDict() {
    const commonCategory = 'CmsContentType,CmsContentStatus';

    Promise.all([
      loadBranchDict(),
      loadBranchCampusDict(),
      loadCommonDictByGroup(commonCategory),
      getLoginUserMainRole(),
    ]).then(([branchDict, branchCampusDict, groupDict, roleResult]) => {
      const { user } = this.props;
      const { isBlCampus, isBlBrench, brenchId, campusId, userId } = user;
      const mainRole = (roleResult && roleResult.data) || {};

      const isCounselor = isRealCounselor(mainRole.roleCode);
      const isCampus = isBlCampus;
      const isBranch = !isCampus && isBlBrench;
      const isGroup = !isCampus && !isBranch;

      let branchVal = null;
      let campusVal = null;
      let teacherVal = null;

      let teacherDisable = false;
      let campusDisable = false;
      let branchDisable = false;

      if (isCounselor) {
        branchVal = brenchId;
        campusVal = campusId;
        teacherVal = userId;
        teacherDisable = true;
        campusDisable = true;
        branchDisable = true;
      } else if (isCampus) {
        branchVal = brenchId;
        campusVal = campusId;
        campusDisable = true;
        branchDisable = true;
      } else if (isBranch) {
        branchVal = brenchId;
        branchDisable = true;
      } else if (isGroup) {
        branchVal = branchDict[0].val;
        campusVal = getOptionValueInFormatterDict(branchCampusDict, branchVal);
      }

      // 保存默认值
      const initParam = { ...this.initParam, branchId: branchVal, blCampusId: campusVal, teacherId: teacherVal };
      this.initParam = initParam;

      this.setState(
        {
          branchId: branchVal,
          blCampusId: campusVal,
          teacherId: teacherVal,
          teacherDisable,
          campusDisable,
          branchDisable,
          branchDict,
          branchCampusDict,
          typeDict: groupDict['CmsContentType'],
          statusDict: groupDict['CmsContentStatus'],
        },
        () => {
          this.loadTeacherDict();
          this.onSearch();
        },
      );
    });
  }

  /**
   * 获取咨询师字典
   * 跟分公司及学习中心有级联关系，orgId为branchId或campusId
   */
  loadTeacherDict() {
    const { branchId, blCampusId } = this.state;
    const orgId = blCampusId || branchId || '';

    loadConsultantDict({ orgId }).then((result) => {
      this.setState({ teacherDict: result });
    });
  }

  /**
   * 分公司选中,先清空学习中心ID
   * @param val
   */
  onBranchChange(val) {
    const { branchId } = this.state;

    if (branchId !== val) {
      this.setState(
        {
          branchId: val,
          blCampusId: null,
          teacherId: null,
        },
        () => {
          this.loadTeacherDict();
        },
      );
    }
  }

  /**
   * 学习中心改变
   * @param val
   */
  onCampusChange(val) {
    this.setState(
      {
        blCampusId: val,
        teacherId: null,
      },
      () => {
        this.loadTeacherDict();
      },
    );
  }

  /**
   * 搜索
   */
  onSearch() {
    this.props.handleSearch();
  }

  /**
   * 重置
   */
  onReset() {
    this.setState(this.initParam);
  }

  /**
   * 格式化请求参数
   */
  getParam() {
    const { branchId, blCampusId, teacherId, fileType, status, startTime, endTime } = this.state;

    return {
      relationUserId: '',
      branchId,
      blCampusId,
      createUser: teacherId,
      type: fileType,
      status,
      beginTime: startTime,
      endTime,
    };
  }

  /**
   * 获取部分字典
   */
  getDict() {
    const { typeDict, statusDict } = this.state;
    return { typeDict, statusDict };
  }

  /**
   * 时间禁用
   */
  disableDate(value) {
    if (!value) {
      return false;
    }

    return moment(value, DATE_FORMAT).valueOf() > moment(moment().format(DATE_FORMAT), DATE_FORMAT).valueOf();
  }

  render() {
    const {
      branchId,
      blCampusId,
      teacherId,
      fileType,
      status,
      startTime,
      endTime,
      branchDict,
      branchCampusDict,
      teacherDict,
      typeDict,
      statusDict,
      teacherDisable,
      campusDisable,
      branchDisable,
    } = this.state;

    return (
      <SearchButton girdConfig={[]} searchBtnLoading={false} onSearch={this.onSearch} onReset={this.onReset}>
        <Select
          placeholder="分公司"
          showSearch={true}
          value={branchId}
          dict={branchDict}
          onChange={this.onBranchChange}
          disabled={branchDisable}
        />
        {/*<MultiSelect
          placeholder="所属学习中心"
          value={blCampusId}
          dict={branchId ? branchCampusDict && branchCampusDict.filter(item => item.val === branchId) : branchCampusDict}
          onChange={val => this.setState({blCampusId: val})}/>*/}
        <Select
          placeholder="所属学习中心"
          showSearch={true}
          value={blCampusId}
          dict={
            branchId ? branchCampusDict && branchCampusDict.filter((item) => item.val === branchId) : branchCampusDict
          }
          onChange={this.onCampusChange}
          disabled={campusDisable}
        />
        <Select
          placeholder="咨询师"
          showSearch={true}
          value={teacherId}
          dict={dictAddOptionTitle(teacherDict)}
          onChange={(val) => {
            this.setState({ teacherId: val });
          }}
          disabled={teacherDisable}
        />
        <Select
          placeholder="文件类型"
          value={fileType}
          dict={typeDict}
          onChange={(val) => {
            this.setState({ fileType: val });
          }}
        />
        <Select
          placeholder="发布状态"
          value={status}
          dict={statusDict}
          onChange={(val) => {
            this.setState({ status: val });
          }}
        />
        <DatePicker
          value={startTime}
          onChange={(val) => this.setState({ startTime: val })}
          placeholder="创建时间(起)"
          disabledDate={this.disableDate}
        />
        <DatePicker
          value={endTime}
          onChange={(val) => this.setState({ endTime: val })}
          placeholder="创建时间(止)"
          disabledDate={this.disableDate}
        />
      </SearchButton>
    );
  }
}

function mapStateToProps(state) {
  const user = state.account.user || {};

  return {
    user,
  };
}

export default connect(mapStateToProps)(HeadSearch);
