import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import List from 'COMPONENT/List';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import moment from 'moment';
import HeadSearch from './component/HeadSearch';
import { getCmsContentList } from 'SERVICE/articleEdit';
import { publishContent } from 'SERVICE/ipadBackEnd';
import { STATUS_UNPUBLISH, STATUS_PUBLISHED } from '../common';

const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;

class ArticlePersonalInfo extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      listLoading: false,
      list: [], // 列表数据
      totalRecords: 0, // 总记录数
      pageNo: 1, // 当前页码
      pageSize: 50, // 每页记录数

      typeDict: null,
      statusDict: null,

      nowTime: null, // 记录请求的时间点
    };

    this.searchRef = null;

    this.onSearch = this.onSearch.bind(this);
    this.canSearch = this.canSearch.bind(this);
    this.loadList = this.loadList.bind(this);
    this.formatResponse = this.formatResponse.bind(this);
    this.formatApiParam = this.formatApiParam.bind(this);
    this.formatColumn = this.formatColumn.bind(this);
    this.onPageToDetail = this.onPageToDetail.bind(this);
    this.doPublish = this.doPublish.bind(this);
    this.onSearchRef = this.onSearchRef.bind(this);
  }

  /**
   * 搜索
   */
  onSearch() {
    const canDoSearch = this.canSearch();

    if (!canDoSearch) {
      showMessage('请求太频繁啦，请等待2秒后再请求！', 'error');
      return false;
    }

    this.loadList({ pageNo: 1 });
  }

  /**
   * 限制频繁请求
   * @return {boolean}
   */
  canSearch() {
    const { nowTime } = this.state;

    if (!nowTime) {
      return true;
    }

    const newTime = new Date().getTime();

    return newTime - nowTime >= 3000;
  }

  /**
   * 获取列表
   * @param changedParams
   */
  loadList(changedParams = {}) {
    if (this.listLoading) {
      return false;
    }

    if (this.searchRef) {
      const dicts = this.searchRef.getDict() || {};
      this.setState({
        typeDict: dicts.typeDict || null,
        statusDict: dicts.statusDict || null,
      });
    }

    this.setState({ listLoading: true });

    const param = this.formatApiParam(changedParams);
    let apiParam = { ...param };
    apiParam.pageNo = apiParam.pageNo - 1;

    getCmsContentList(apiParam)
      .then((result) => {
        this.formatResponse(param, result);
      })
      .catch((e) => {
        this.formatResponse(param, {});
      });
  }

  /**
   * 格式化请求数据
   * @param param
   * @param response
   */
  formatResponse(param, response = {}) {
    const data = response.datas || [];
    const rowCount = response.rowCount || 0;

    this.setState({
      listLoading: false,
      list: data,
      totalRecords: rowCount,
      pageNo: param.pageNo,
      pageSize: param.pageSize,
      nowTime: new Date().getTime(),
    });
  }

  /**
   * 格式化api请求参数
   * @param changeParam
   */
  formatApiParam(changeParam = {}) {
    const { pageNo, pageSize } = this.state;
    const tempParam = { pageNo, pageSize, getTotal: true };
    const searchParam = (this.searchRef && this.searchRef.getParam()) || {};

    return { ...tempParam, ...changeParam, ...searchParam };
  }

  /**
   * 格式化列表列
   */
  formatColumn() {
    const { typeDict, statusDict } = this.state;
    const { btnAuth_detail, btnAuth_edit } = this.props;

    return [
      {
        title: '序号',
        dataIndex: 'number',
        key: 'number',
        width: COL_WIDTH_S,
        render: (text, record, index) => {
          return index + 1;
        },
      },
      { title: '个人资料名称', dataIndex: 'title', key: 'title', width: COL_WIDTH_XL, lines: 2 },
      { title: '所属目录名称', dataIndex: 'menuName', key: 'menuName', width: COL_WIDTH_L, lines: 2 },
      { title: '上传者', dataIndex: 'createUserName', key: 'createUserName', width: COL_WIDTH_M },
      { title: '所属分公司', dataIndex: 'branchName', key: 'branchName', width: COL_WIDTH_L, lines: 2 },
      { title: '所属学习中心', dataIndex: 'blCampusName', key: 'blCampusName', width: COL_WIDTH_M },
      { title: '文件类型', dataIndex: 'type', key: 'type', width: COL_WIDTH_M, dict: typeDict },
      {
        title: '创建日期',
        dataIndex: 'createTime',
        key: 'createTime',
        width: COL_WIDTH_XL,
        render: (text, record) => {
          const { createTime } = record;
          return moment(createTime).format('YYYY-MM-DD HH:mm:ss');
        },
      },
      {
        title: '查看次数',
        dataIndex: 'visitNum',
        key: 'visitNum',
        width: COL_WIDTH_M,
        render: (text, record) => {
          return text || 0;
        },
      },
      { title: '发布状态', dataIndex: 'status', key: 'status', width: COL_WIDTH_M, dict: statusDict },
      {
        title: '操作',
        key: 'action',
        width: COL_WIDTH_XL,
        fixed: 'right',
        getActionConfig: (text, record, index) => {
          const { status } = record;

          let editText = <span style={{ color: 'red' }}>发布</span>;
          if (status === STATUS_PUBLISHED) {
            editText = '取消发布';
          }

          return [
            { text: '查看详情', onClick: () => this.onRowBtnClick('view', record), disabled: !btnAuth_detail },
            { text: editText, onClick: () => this.onRowBtnClick('edit', record), disabled: !btnAuth_edit },
          ];
        },
      },
    ];
  }

  /**
   * 操作
   * @param type
   * @param record
   */
  onRowBtnClick(type, record) {
    if (type === 'view') {
      this.onPageToDetail(record);
    } else {
      this.doPublish(record);
    }
  }

  /**
   * 查看详情
   * @param record
   */
  onPageToDetail(record) {
    const { id } = record;
    const { dispatch } = this.props;

    dispatch({
      type: 'main/openPage',
      payload: {
        path: '/ArticlePersonalInfo/Edit',
        params: {
          id,
          pageType: 'view',
        },
        refresh: true,
      },
    });
  }

  /**
   * 发布/取消发布
   */
  doPublish(record) {
    const { status, id } = record;
    let param = { id };
    let title = '';

    if (status === STATUS_PUBLISHED) {
      title = '取消发布';
      param.status = STATUS_UNPUBLISH;
    } else {
      title = '发布';
      param.status = STATUS_PUBLISHED;
    }

    showConfirm(`是否确定${title}？`, () => {
      publishContent([param], true)
        .then((result) => {
          showMessage(`${title}成功`, 'success');
          this.loadList();
        })
        .catch((e) => {
          showMessage(e.text || `${title}失败，请重试`);
        });
    });
  }

  onSearchRef(ref) {
    this.searchRef = ref;
  }

  render() {
    const { loading, listLoading, list, pageNo, totalRecords, pageSize } = this.state;
    const columns = this.formatColumn();

    return (
      <Page
        fitWindowHeight={true}
        loading={loading}
        route={this.props.route}
        headerToolBar={<HeadSearch handleSearch={this.onSearch} onRef={this.onSearchRef} />}
      >
        <List
          dataSource={list}
          columns={columns}
          loading={!loading && listLoading}
          pagination={{
            current: pageNo,
            total: totalRecords,
            pageSize,
            pageSizeOptions: ['20', '50', '100'],
          }}
          onChange={({ current, pageSize }) => this.loadList({ pageNo: current, pageSize })}
        />
      </Page>
    );
  }
}

function mapStateToProps(state) {
  const btnAuth = state.account.btnAuths;
  const btnAuth_edit = btnAuth.indexOf('ARTICLE_PERSONAL_INFO_EDIT') > -1;
  const btnAuth_detail = btnAuth.indexOf('ARTICLE_PERSONAL_INFO_DETAIL') > -1;

  return {
    btnAuth_edit,
    btnAuth_detail,
  };
}

export default connect(mapStateToProps)(ArticlePersonalInfo);
