/**
 * 发布
 * @property {ReactNode} children - 触发弹框的元素
 * @property {string} [type] - 类型，默认为发布，可选取消发布'cancel'
 * @property {function} onOk  点击确定按钮时的回调方法，仅预约发布的情况有参数date: string（预约发布日期）
 */
import React from 'react';
import ReactDOM from 'react-dom';
import classnames from 'classnames';
import { Popover } from 'antd';
import styles from './styles.less';
import Tabs from 'COMPONENT/Tabs';
import DatePicker from 'COMPONENT/DatePicker';
import TextLink from 'COMPONENT/TextLink';
import moment from 'moment';
import { DATE_FORMAT } from 'COMPONENT/utils/form';

const Tab = Tabs.Tab;
const initState = {
  visible: false,
  date: null,
  publishType: null, //发布类型，0：立即发布， 1：预约发布
};
class PublishPopConfirm extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      ...initState,
    };
  }

  reset = () => {
    this.setState(initState);
  };

  ok = () => {
    const { onOk } = this.props;
    const { publishType, date } = this.state;
    let param;
    if (publishType === 1) {
      if (!date) {
        return;
      }
      param = date;
    }
    this.reset();
    onOk(param);
  };

  getDisabledDate = (value) => {
    if (!value) {
      return false;
    }
    return moment(value, DATE_FORMAT).valueOf() <= moment(moment().format(DATE_FORMAT), DATE_FORMAT).valueOf();
  };

  render() {
    const { children, onOk, type, getPopupContainer } = this.props;
    const { visible, date } = this.state;
    const isCancel = type === 'cancel';
    const cls = {
      [styles.content]: 1,
      [styles.contentCancel]: isCancel,
    };
    const content = (
      <div className={classnames(cls)} ref={(c) => (this.$content = ReactDOM.findDOMNode(c))}>
        <div className={styles.body}>
          {isCancel ? (
            <div className={styles.contentText}>确定取消发布？</div>
          ) : (
            <Tabs
              className={styles.tabs}
              type="button"
              navAlign="center"
              onChange={(key) => {
                this.setState({ publishType: key });
              }}
            >
              <Tab title="立即发布" className={styles.contentText}>
                确定立即发布？
              </Tab>
              <Tab title="预约发布">
                <DatePicker
                  value={date}
                  onChange={(val) => this.setState({ date: val })}
                  getPopupContainer={() => this.$content}
                  disabledDate={this.getDisabledDate}
                />
              </Tab>
            </Tabs>
          )}
        </div>
        <div className={styles.footer}>
          <TextLink onClick={this.ok}>确定</TextLink>
          <TextLink type="secondary" style={{ marginLeft: 20 }} onClick={this.reset}>
            取消
          </TextLink>
        </div>
      </div>
    );

    return (
      <Popover
        content={content}
        trigger="click"
        visible={visible}
        onVisibleChange={(visible) => {
          if (!visible) {
            this.reset();
          } else {
            this.setState({ visible });
          }
        }}
        placement="topRight"
        getPopupContainer={getPopupContainer}
      >
        {children}
      </Popover>
    );
  }
}

export default PublishPopConfirm;
