/**
 * 设置可见范围模态框
 */
import React from 'react';
import classnames from 'classnames';
import Checkbox from 'COMPONENT/Checkbox';
import Modal from 'COMPONENT/Modal';
import styles from './styles.less';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { saveCmsCotentAuth } from 'SERVICE/ipadBackEnd';
import { TOP_ORG_ID } from 'PAGE/ipadBackEnd/articleEdit/common';
import { getCmsContentById } from 'SERVICE/ipadBackEnd';
import getWrappedInstance from 'UTIL/getWrappedInstance';

class VisibleRangeModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 等待状态
      initCheckedOrgIds: [], // {string[]} 初始状态的选中的组织机构ID
      checkedOrgIds: [], // {string[]} 选中的组织机构ID
    };
  }

  componentDidMount() {
    const { id, orgDict } = this.props;
    this.setState({ loading: true });
    getCmsContentById({ id }).then((detail) => {
      const { auths, allCanSee } = detail;
      let checkedOrgIds = [];
      if (allCanSee === 0) {
        checkedOrgIds = orgDict.map((d) => d.val);
      } else {
        checkedOrgIds = auths.map((a) => a.orgId);
      }
      this.setState({ loading: false, initCheckedOrgIds: checkedOrgIds, checkedOrgIds });
    });
  }

  onOk = () => {
    const { onSaveSuccess, onClose, id } = this.props;
    const { checkedOrgIds } = this.state;
    const params = {
      allCanSee: 1,
      id,
      auths: checkedOrgIds.map((c) => {
        return { cmsContentId: id, orgId: c };
      }),
    };
    this.setState({ loading: true });
    saveCmsCotentAuth(params, true).then(
      () => {
        showMessage('保存成功', 'success');
        this.setState({ loading: false });
        onSaveSuccess();
      },
      (e) => {
        if (e.text) {
          this.modal.showOkTips(e.text);
          this.setState({ loading: false });
        }
      },
    );
  };

  checkChange = (id, checked) => {
    const { checkedOrgIds } = this.state;
    let newCheckedOrgIds;
    if (checked) {
      newCheckedOrgIds = [...checkedOrgIds, id];
    } else {
      newCheckedOrgIds = checkedOrgIds.filter((c) => c !== id);
    }
    this.setState({ checkedOrgIds: newCheckedOrgIds });
  };

  checkAllChange = (check) => {
    const { orgDict } = this.props;
    const { checkedOrgIds } = this.state;
    const disabledChecked = checkedOrgIds.filter((c) => this.isDisabled(c));
    if (check) {
      this.setState({
        checkedOrgIds: [...disabledChecked, ...orgDict.filter((d) => !this.isDisabled(d.val)).map((dic) => dic.val)],
      });
    } else {
      this.setState({ checkedOrgIds: disabledChecked });
    }
  };

  isDisabled = (id) => {
    const { isBelongTop, unCheckable } = this.props;
    const { initCheckedOrgIds } = this.state;
    return (TOP_ORG_ID === id && isBelongTop) || (!unCheckable && initCheckedOrgIds.indexOf(id) >= 0);
  };

  render() {
    const { orgDict, onClose } = this.props;
    const { loading, checkedOrgIds } = this.state;
    const allChecked = !orgDict.some((d) => !this.isDisabled(d.val) && checkedOrgIds.indexOf(d.val) < 0);
    const titleComp = (
      <div className={styles.title}>
        <div className={styles.titleLeft}>可见范围设置</div>
        <div className={styles.titleRight}>
          <Checkbox
            checked={allChecked}
            onChange={() => this.checkAllChange(!allChecked)}
            disabled={!orgDict.some((d) => !this.isDisabled(d.val))}
          >
            全选
          </Checkbox>
        </div>
      </div>
    );
    return (
      <Modal
        className={styles.modalWrap}
        loading={loading}
        title={titleComp}
        width={564}
        onOk={this.onOk}
        onCancel={onClose}
        noLine
        ref={(c) => (this.modal = getWrappedInstance(c))}
      >
        <div className={styles.wrap}>
          {orgDict.map((c, i) => {
            const checked = checkedOrgIds.indexOf(c.val) >= 0;
            return (
              <div key={c.val} className={styles.cItem}>
                <Checkbox
                  checked={checked}
                  onChange={(e) => this.checkChange(c.val, e.target.checked)}
                  disabled={this.isDisabled(c.val)}
                >
                  {c.text}
                </Checkbox>
              </div>
            );
          })}
        </div>
      </Modal>
    );
  }
}

export default VisibleRangeModal;
