import React from 'react';
import ReactDOM from 'react-dom';
import classnames from 'classnames';
import { Switch, Tooltip } from 'antd';
import { connect } from 'dva';
import styles from './styles.less';
import DropdownMenu from 'COMPONENT/DropdownMenu';
import Icon from 'COMPONENT/Icon';
import TextLink from 'COMPONENT/TextLink';
import styleVars from 'COMPONENT/theme/variables';
import {
  publishStatusDict,
  STATUS_UNPUBLISH,
  STATUS_PUBLISHED,
  STATUS_TOPUBLISH,
  MAX_CAROUSEL_COUNT,
  MAX_CAROUSEL_TIPS,
  MODE_WORKBENCH,
  PERSONALINFO,
  BELONG_TYPE_PERSONAL,
  BELONG_TYPE_BRANCH,
  BELONG_TYPE_TOP,
} from '../../common';
import { getDictTextByVal } from 'UTIL/dict';
import PublishPopConfirm from './PublishPopConfirm';
import Checkbox from 'COMPONENT/Checkbox';
import PropTypes from 'prop-types';
import VisibleRangeModal from './VisibleRangeModal';
import CopyModal from './CopyModal';
import HistoryModal from './HistoryModal';
import showAlert from 'COMPONENT/utils/globalTips/showAlert';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import defImgSrc from '../def.png';
import {
  publishContent,
  publishFrontContent,
  deleteContent,
  pushContentToFront,
  cacelContentToFront,
} from 'SERVICE/ipadBackEnd';

const dropdownCfg = {
  color: styleVars.fontColor,
  activeColor: styleVars.primaryColor,
  disabledColor: styleVars.fontColorTips,
};

class ArticleCard extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      rangeModalVis: false,
      copyModalVis: false,
      historyModalVis: false,
      imgLoaded: false,
    };
  }

  componentDidMount() {
    ReactDOM.findDOMNode(this.imgRef).addEventListener('load', this.onImgLoaded, false);
  }

  componentWillUnmount() {
    ReactDOM.findDOMNode(this.imgRef).removeEventListener('load', this.onImgLoaded, false);
  }

  onImgLoaded = () => {
    this.setState({ imgLoaded: true });
  };

  switchPulish = (type, date) => {
    const { onLoadingStart, onLoadingEnd, data, onReload, columnType } = this.props;
    const { id } = data;
    if (columnType === 'carousel') {
      const params = {
        appointmentPublishTime: date ? `${date} 00:00:00` : null,
        id,
        status: type === 'cancel' ? STATUS_UNPUBLISH : date ? STATUS_TOPUBLISH : STATUS_PUBLISHED,
      };
      onLoadingStart();
      publishFrontContent(params, true).then(
        () => {
          onReload();
          onLoadingEnd();
        },
        (e) => {
          showMessage(e.text || '操作失败');
          onLoadingEnd();
        },
      );
    } else {
      const params = [
        {
          appointmentPublishTime: date ? `${date} 00:00:00` : null,
          id,
          status: type === 'cancel' ? STATUS_UNPUBLISH : date ? STATUS_TOPUBLISH : STATUS_PUBLISHED,
        },
      ];
      onLoadingStart();
      publishContent(params, true).then(
        () => {
          onReload();
          onLoadingEnd();
        },
        (e) => {
          showMessage(e.text || '操作失败');
          onLoadingEnd();
        },
      );
    }
  };

  delete = () => {
    const { onLoadingStart, onLoadingEnd, data, onReload } = this.props;
    showConfirm(`确定删除 ${data.title} 吗？`, () => {
      onLoadingStart();
      deleteContent({ id: data.id }, true).then(
        () => {
          showMessage('删除成功', 'success');
          onReload();
          onLoadingEnd();
        },
        (e) => {
          showMessage(e.text || '删除失败');
          onLoadingEnd();
        },
      );
    });
  };

  recommend = () => {
    const {
      onLoadingStart,
      onLoadingEnd,
      onReload,
      data: { id },
    } = this.props;
    onLoadingStart();
    pushContentToFront({ id }, true).then(
      () => {
        onReload();
        onLoadingEnd();
      },
      (e) => {
        showMessage(e.text || '操作失败');
        onLoadingEnd();
      },
    );
  };

  cancelRecommend = () => {
    const {
      onLoadingStart,
      onLoadingEnd,
      onReload,
      data: { id },
    } = this.props;
    onLoadingStart();
    cacelContentToFront({ id }).then(() => {
      onReload();
      onLoadingEnd();
    });
  };

  disable = (keys) => {
    const { type } = this.props;
    if (type === 'batch' || type === 'sort') {
      return true;
    }
    return keys ? keys.some((k) => k) : false;
  };

  hide = (keys) => {
    return keys ? keys.some((k) => k) : false;
  };

  /**
   * 禁用编辑 2018.10.10
   * @param orgType
   * @param articleUserId
   * @returns {boolean}
   */
  disableEdit = (orgType, articleUserId) => {
    const { userId, isTopOrg, isTopUser } = this.props;
    //个人类型，根据创建者Id判断
    if (orgType === BELONG_TYPE_PERSONAL) {
      return articleUserId !== userId;
      //分公司类型，创建者可以，同时集团切换后查询到也可以
    } else if (orgType === BELONG_TYPE_BRANCH) {
      return articleUserId !== userId && !isTopUser;
      //集团类型，集团账号切换至星火集团时才可以
    } else if (orgType === BELONG_TYPE_TOP) {
      return !isTopOrg;
    }
  };

  disableEditPersonalInfo = (orgType, articleUserId) => {
    const { btnAuths } = this.props;
    if (!btnAuths.includes('EDIT_PERSONAL_INFO')) {
      return true;
    }
    return this.disableEdit(orgType, articleUserId);
  };

  /**
   * 禁用删除 2018.10.10
   * @param orgType
   * @param articleUserId
   * @returns {boolean}
   */
  disableDel = (orgType, articleUserId) => {
    const { userId, isTopOrg } = this.props;
    //个人类型，根据创建者Id判断
    if (orgType === BELONG_TYPE_PERSONAL) {
      return articleUserId !== userId;
      //分公司类型，只有创建者可以
    } else if (orgType === BELONG_TYPE_BRANCH) {
      return articleUserId !== userId;
      //集团类型，集团账号切换至星火集团时才可以
    } else if (orgType === BELONG_TYPE_TOP) {
      return !isTopOrg;
    }
  };

  disableDelPersonalInfo = (orgType, articleUserId) => {
    const { btnAuths } = this.props;
    if (!btnAuths.includes('DEL_PERSONAL_INFO')) {
      return true;
    }
    return this.disableDel(orgType, articleUserId);
  };

  render() {
    const {
      data,
      checked,
      onCheckedChange,
      orgDict,
      menuTree,
      isTopUser,
      isTopOrg,
      orgId,
      onReload,
      recommendCount,
      onEdit,
      type,
      columnType,
      publishCount,
      mode,
      roleCode,
    } = this.props;
    const { rangeModalVis, copyModalVis, historyModalVis, imgLoaded } = this.state;
    const {
      id,
      title,
      publishTime,
      isAppointment,
      isRecommend,
      imgSrc,
      orgTypeName,
      readCount,
      isBelongTop,
      isTopArticleInBranch,
      publishStatus,
      appointmentPublishTime,
      orgType,
      createUser,
      publishUserName,
    } = data;

    const isBranchArticleInTopUser = isTopUser && !isBelongTop;

    const hideMoreVisible = this.hide([
      !isTopOrg /*publishStatus === STATUS_PUBLISHED ||publishStatus === STATUS_TOPUBLISH*/,
    ]);
    const hideMoreCopy = this.hide([]);
    const hideMoreHistory = true;
    const hideMoreDelete = this.disableDel(orgType, createUser);

    const disableMore = this.disable([
      columnType === 'indexArticle' && !isTopOrg,
      hideMoreVisible && hideMoreCopy && hideMoreHistory && hideMoreDelete,
    ]);
    const disablePublish = this.disable([isTopArticleInBranch]);
    const disableEdit = this.disableEdit(orgType, createUser);
    const disablePushIndex = this.disable([isTopArticleInBranch, isBranchArticleInTopUser]);

    const hidePushIndex = this.hide([!isTopOrg, columnType === 'carousel', mode === MODE_WORKBENCH]);

    const moreMenus = [
      { text: '可见范围', onClick: () => this.setState({ rangeModalVis: true }), hide: hideMoreVisible },
      { text: '复制图文', onClick: () => this.setState({ copyModalVis: true }), hide: hideMoreCopy },
      { text: '操作记录', onClick: () => this.setState({ historyModalVis: true }), hide: hideMoreHistory },
      //在个性化资料的时候，需要额外的判断权限按钮btnAuths
      {
        text: '删除',
        onClick: this.delete,
        hide: this.context.isNowPersonal() ? this.disableDelPersonalInfo(orgType, createUser) : hideMoreDelete,
      },
    ];
    const recommendFull = recommendCount >= 4444444444;
    const publishFull = columnType === 'carousel' && publishCount >= MAX_CAROUSEL_COUNT;
    const recommendContent = (
      <span className={styles.pushIndexItem}>
        <span>首页推荐</span>
        {recommendFull && <span className={styles.pushIndexTips}>{`满额4/4`}</span>}
      </span>
    );
    const recommendMenus = isRecommend
      ? [{ content: <span className={styles.pushIndexItem}>取消推荐</span>, onClick: this.cancelRecommend }]
      : [{ content: recommendContent, onClick: this.recommend, disabled: recommendFull }];
    const switchPublishType = publishStatus === STATUS_UNPUBLISH ? null : 'cancel';
    let switchBtn;
    const switchChecked = publishStatus !== STATUS_UNPUBLISH;
    if (disablePublish) {
      switchBtn = <Switch size="small" checked={switchChecked} disabled={disablePublish} />;
    } else if (publishFull && !switchChecked) {
      switchBtn = <Switch size="small" checked={switchChecked} onClick={() => showAlert(MAX_CAROUSEL_TIPS)} />;
    } else {
      switchBtn = (
        <PublishPopConfirm
          type={switchPublishType}
          onOk={(date) => this.switchPulish(switchPublishType, date)}
          getPopupContainer={() => this.$wrap}
        >
          <span>
            <Switch size="small" checked={switchChecked} />
          </span>
        </PublishPopConfirm>
      );
    }
    const cls = {
      [styles.wrap]: 1,
      [styles[`wrap-${type}`]]: type,
    };

    return (
      <div className={classnames(cls)} ref={(c) => (this.$wrap = ReactDOM.findDOMNode(c))}>
        {type === 'batch' && (
          <div className={styles.checkbox}>
            <Checkbox checked={checked} onChange={(e) => onCheckedChange(e.target.checked)} />
          </div>
        )}
        <div className={styles.header}>
          <Tooltip placement="top" title={title}>
            <div className={styles.headerLeft}>{title}</div>
          </Tooltip>

          <div className={styles.headerRight}>
            <DropdownMenu
              menus={moreMenus}
              iconType="more"
              {...dropdownCfg}
              disabled={disableMore}
              placement="bottomRight"
              getPopupContainer={() => this.$wrap}
            />
          </div>
        </div>
        <div className={styles.topBar}>
          {(publishTime || appointmentPublishTime) && (
            <div className={styles.topBarLeft}>
              <div>
                <Icon type="time_mc" className={styles.icon} />
                {isAppointment
                  ? appointmentPublishTime && appointmentPublishTime.slice(0, 10)
                  : publishTime && publishTime.slice(0, 10)}
                {isAppointment && '（预约发布）'}
              </div>
              <div style={{ marginLeft: '16px' }}>{publishUserName}</div>
            </div>
          )}
          <div className={styles.topBarRight}>
            <span style={{ display: 'inline-block', marginRight: 8 }}>
              {getDictTextByVal(publishStatusDict, publishStatus)}
            </span>
            {switchBtn}
          </div>
        </div>
        <div className={styles.imgWrap}>
          <div className={styles.imgWrapInner}>
            <img src={defImgSrc} ref={(c) => (this.defImgRef = c)} className={imgLoaded ? styles.defImgHide : null} />
            <img
              src={imgSrc}
              className={imgLoaded ? styles.faceImg : styles.faceImgHide}
              ref={(c) => (this.imgRef = c)}
            />
          </div>
          {isRecommend ? <div className={styles.recommendTag}>首页推荐</div> : null}
        </div>
        <div className={styles.bottomBar}>
          <div className={styles.bottomBarLeft}>
            <Icon type="city_mc" className={styles.icon} />
            {orgTypeName}
          </div>
          <div className={styles.bottomBarRight}>
            <Icon type="book_mc" className={styles.icon} />
            {readCount || 0}
          </div>
        </div>
        <div className={styles.footer}>
          {isTopArticleInBranch ? (
            <div style={{ paddingLeft: 15, lineHeight: '46px' }}>集团配置内容不可编辑</div>
          ) : (
            <div className={styles.btns}>
              <div className={styles.btn}>
                <TextLink
                  type="secondary"
                  onClick={onEdit}
                  disabled={
                    this.context.isNowPersonal() ? this.disableEditPersonalInfo(orgType, createUser) : disableEdit
                  }
                >
                  编辑
                </TextLink>
              </div>
              {!hidePushIndex && (
                <div className={styles.btn}>
                  <DropdownMenu
                    text="推荐设置"
                    hasArrow={true}
                    menus={recommendMenus}
                    {...dropdownCfg}
                    disabled={disablePushIndex}
                    placement="bottomCenter"
                  />
                </div>
              )}
            </div>
          )}
        </div>
        {rangeModalVis && (
          <VisibleRangeModal
            orgDict={orgDict}
            onSaveSuccess={() => {
              this.setState({ rangeModalVis: false });
              onReload();
            }}
            onClose={() => this.setState({ rangeModalVis: false })}
            isBelongTop={isBelongTop}
            isTopUser={isTopUser}
            id={id}
            unCheckable={
              publishStatus === STATUS_UNPUBLISH ||
              publishStatus === STATUS_PUBLISHED ||
              publishStatus === STATUS_TOPUBLISH
            }
          />
        )}
        {copyModalVis && (
          <CopyModal
            canSelectOrg={!!(isTopUser && !isBelongTop)}
            orgDict={orgDict}
            menuTree={menuTree}
            onSaveSuccess={() => {
              this.setState({ copyModalVis: false });
              onReload();
            }}
            onClose={() => this.setState({ copyModalVis: false })}
            ids={[id]}
            orgId={orgId}
            isTopUser={isTopUser}
            roleCode={roleCode}
          />
        )}
        {historyModalVis && <HistoryModal orgId={orgId} onClose={() => this.setState({ historyModalVis: false })} />}
      </div>
    );
  }
}

ArticleCard.contextTypes = {
  isNowPersonal: PropTypes.func,
};

const mapStateToProps = (state) => {
  return {
    btnAuths: state.account.btnAuths,
    roleCode: state.account.user.roleCode,
    userId: state.account.user.userId,
  };
};

export default connect(mapStateToProps)(ArticleCard);
