import React from 'react';
import { Row, Col } from 'antd';
import { connect } from 'dva';
import ArticleCard from '../ArticleCard';
import SortButton from 'COMPONENT/SortButton';
import Button from 'COMPONENT/Button';
import Icon from 'COMPONENT/Icon';
import styles from './styles.less';
import Search from 'COMPONENT/Search';
import Select from 'COMPONENT/Select';
import TreeSelect from 'COMPONENT/TreeSelect';
import BatchPage from '../BatchPage';
import PropTypes from 'prop-types';
import SortPage from '../SortPage';
import classNames from 'classnames';
import defImgSrc from '../def.png';
import {
  ARTICLE_TYPE_PICTURE,
  ARTICLE_TYPE_DOCUMENT,
  ARTICLE_TYPE_LINK,
  ARTICLE_TYPE_VIDEO,
  STATUS_UNPUBLISH,
  CHECK_CAMPUS_INFO,
  colStyle,
} from '../../common';
import { loadCampusDict } from 'UTIL/dict';
import { getConsultor } from 'SERVICE/ipadBackEnd';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

const SortButtonGroup = SortButton.Group;

const consultorTemp = [{ text: '请先选择学习中心', val: undefined, disabled: true }];

class ArticleManage extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      batchShow: false,
      sortShow: false,
      imagesIcon: 'images_mc',
      linkIcon: 'link_mc',
      documentsIcon: 'documents_mc',
      videoIcon: 'video_mc',
      campusDict: [],
      consultorDict: [],
    };
  }

  async componentDidMount() {
    const { brenchId, brenchName } = this.props;
    const campusDict = await loadCampusDict(this.props.brenchId);
    campusDict.forEach((campus) => (campus.parentVal = brenchId));
    //添加分公司选项
    campusDict.unshift({ val: brenchId, text: brenchName, parentVal: null });
    this.setState({ campusDict });
  }

  onBatchSuccess = () => {
    this.setState({ batchShow: false });
    this.props.onReload();
  };

  onBatchCancel = () => {
    this.setState({ batchShow: false });
  };

  onSortSuccess = () => {
    this.setState({ sortShow: false });
    this.props.onReload();
  };

  onSortCancel = () => {
    this.setState({ sortShow: false });
  };

  /**
   * 学习中心改变
   * @param value
   */
  onCampusChange = async (value) => {
    const { articleName, belongType, publishStatus, recommend, onSearch } = this.props;
    onSearch({
      campusId: value,
      consultorId: null,
      consultant: value ? null : this.props.consultantDict[0].val,
      articleName,
      belongType,
      publishStatus,
      recommend,
    });
    /**
     * 学习中心改变就获取对应的咨询师数据
     */
    if (value) {
      const response = await getConsultor({ campusId: value });
      const consultorDict = response.data.map((item) => ({
        val: item.userId,
        text: item.name,
      }));
      this.setState({ consultorDict });
    }
  };

  /**
   * 咨询师改变
   * @param value
   */
  onconsultorChange = (value) => {
    const { articleName, belongType, publishStatus, recommend, onSearch, campusId } = this.props;
    onSearch({
      consultorId: value,
      consultant: null,
      campusId,
      articleName,
      belongType,
      publishStatus,
      recommend,
    });
  };

  /**
   * 咨询师筛选项改变
   */
  onConsultChange = (consult) => {
    const { articleName, belongType, publishStatus, recommend, onSearch } = this.props;
    onSearch({
      consultant: consult.val,
      campusId: null,
      consultorId: null,
      articleName,
      belongType,
      publishStatus,
      recommend,
    });
  };

  handleSearch = (keys) => {
    const { onSearch, consultorId, consultant, campusId } = this.props;
    onSearch({
      consultorId,
      consultant,
      campusId,
      ...keys,
    });
  };

  /**
   * 渲染咨询师筛选项
   */
  renderConsultant = () => {
    const { consultantDict, consultant, campusId, consultorId } = this.props;
    const { campusDict, consultorDict } = this.state;

    return (
      <div className="Search-clearfix" key="consultant">
        <div className="Search-label">咨询师</div>
        <div className="Search-items">
          <div className="Search-clearfix" style={{ paddingBottom: 10 }}>
            {consultantDict.map((consult) => {
              return (
                <div
                  key={consult.val}
                  className={classNames('Search-item noClose', { 'filter-active': consult.val === consultant })}
                  onClick={() => this.onConsultChange(consult)}
                >
                  {consult.text}
                </div>
              );
            })}
            {/*下拉框*/}
            <div style={{ padding: '12px 7px 0', display: 'flex' }}>
              <div style={{ display: 'inline-block', width: '148px' }}>
                <TreeSelect
                  value={campusId}
                  onChange={this.onCampusChange}
                  dict={campusDict}
                  placeholder="请选择学习中心"
                  size="small"
                />
              </div>
              <div style={{ display: 'inline-block', width: '148px', marginLeft: '7px' }}>
                <Select
                  value={consultorId}
                  onChange={this.onconsultorChange}
                  dict={campusId ? consultorDict : consultorTemp}
                  placeholder="请选择咨询师"
                  size="small"
                />
              </div>
            </div>
          </div>
        </div>
      </div>
    );
  };

  handleEditArticle = (params) => {
    const { btnAuths, onEditArticle } = this.props;
    // 咨询师才可以编辑个性化资料
    if (this.context.isNowPersonal()) {
      if (!btnAuths.includes('ADD_PERSONAL_INFO')) {
        showMessage('该角色没有权限');
        return;
      }
    }
    onEditArticle(params);
  };

  hideSortBtn = () => {
    const { list = [], userId } = this.props;
    //个性化资料，只有在当前的资料都属于当前登录用户时，才允许顺序管理
    return this.context.isNowPersonal() && list.every((item) => item.createUser === userId);
  };

  render() {
    const {
      belongTypeDict,
      publishStatusDict,
      recommendDict,
      consultantDict,
      onInputChange,
      sort,
      onSortChange,
      onLoadingStart,
      onLoadingEnd,
      articleName,
      belongType,
      publishStatus,
      recommend,
      list: listProp,
      totalRecords,
      orgDict,
      menuTree,
      isTopUser,
      isTopOrg,
      onReload,
      orgId,
      columnType,
      onLoadListForSort,
      mode,
      isDefaultTopBelongType,
      colCount,
      roleCode,
      onEditArticle,
      isIndex,
      menuItem,
      btnAuths,
    } = this.props;
    const { batchShow, sortShow, imagesIcon, linkIcon, documentsIcon, videoIcon } = this.state;
    const selectedDatas = {
      inputValue: articleName,
      belongType,
      publishStatus,
      recommend,
    };

    let list = listProp || [];
    const recommendCount = list.filter((d) => d.isRecommend).length;
    const publishCount = list.filter((d) => d.publishStatus !== STATUS_UNPUBLISH).length;
    const colSpan = 24 / colCount;

    //Search组件配置
    let conditions = [];
    if (!isDefaultTopBelongType) {
      conditions.push({ title: '所属类型', field: 'belongType', dict: belongTypeDict });
    }
    conditions = [
      ...conditions,
      { title: '发布状态', field: 'publishStatus', dict: publishStatusDict },
      { title: '首页推荐', field: 'recommend', dict: recommendDict },
    ];
    //用户角色不是咨询师，并且是在个性化资料菜单下，则拥有咨询师筛选项
    if (btnAuths.includes(CHECK_CAMPUS_INFO) && this.context.isNowPersonal()) {
      conditions.push({
        title: '咨询师',
        field: 'consultant',
        dict: consultantDict,
        type: 'custom',
        render: this.renderConsultant,
      });
    }

    if (!menuItem) {
      showMessage('该用户角色或分公司没有浏览权限');
    }

    return (
      <div>
        <div style={{ display: !batchShow && !sortShow ? 'block' : 'none' }}>
          <div className={styles.searchBar}>
            <Search
              conditions={conditions}
              selectedDatas={selectedDatas}
              onInitFinish={() => {}}
              onSearch={this.handleSearch}
              result={totalRecords}
              isShowResult={false}
              onInputChange={onInputChange}
            />
          </div>
          {/*有选中菜单项才有浏览权限*/}
          {menuItem && [
            <div className={styles.toolbar} key="dsvqev">
              <div className={styles.leftTools}>
                <SortButtonGroup value={sort} onChange={onSortChange}>
                  <SortButton name="time">时间</SortButton>
                  <SortButton name="readCount">阅读量</SortButton>
                </SortButtonGroup>
              </div>
              <div className={styles.rightTools}>
                <Button onClick={() => this.setState({ batchShow: true })}>批量操作</Button>
                {this.hideSortBtn() ? null : (
                  <Button style={{ marginLeft: 20 }} onClick={() => this.setState({ sortShow: true })}>
                    顺序管理
                  </Button>
                )}
              </div>
            </div>,
            <Row style={{ margin: '-10px -10px 0 -10px' }} key="verv">
              <Col key="newCard" span={colSpan} style={colStyle}>
                <div className={styles.newCard}>
                  <div className={styles.newCardBg}>
                    <img src={defImgSrc} />
                  </div>
                  <div className={styles.newCardContent + ' vMiddleContainer'}>
                    <div className={styles.newCardIconTemp} />
                    <div
                      className={styles.newCardIcon + ' vMiddleItem'}
                      onMouseEnter={() => this.setState({ imagesIcon: 'images_mc_h' })}
                      onMouseLeave={() => this.setState({ imagesIcon: 'images_mc' })}
                      onClick={() => {
                        this.handleEditArticle({ columnType, type: ARTICLE_TYPE_PICTURE });
                      }}
                    >
                      <Icon type={imagesIcon} size={36} />
                      <div>图文</div>
                    </div>
                    <div
                      className={styles.newCardIcon + ' vMiddleItem'}
                      onMouseEnter={() => this.setState({ linkIcon: 'link_mc_h' })}
                      onMouseLeave={() => this.setState({ linkIcon: 'link_mc' })}
                      onClick={() => {
                        this.handleEditArticle({ columnType, type: ARTICLE_TYPE_LINK });
                      }}
                    >
                      <Icon type={linkIcon} size={36} />
                      <div>链接</div>
                    </div>
                    <div
                      className={styles.newCardIcon + ' vMiddleItem'}
                      onMouseEnter={() => this.setState({ documentsIcon: 'documents_mc_h' })}
                      onMouseLeave={() => this.setState({ documentsIcon: 'documents_mc' })}
                      onClick={() => {
                        this.handleEditArticle({ columnType, type: ARTICLE_TYPE_DOCUMENT });
                      }}
                    >
                      <Icon type={documentsIcon} size={36} />
                      <div>文档</div>
                    </div>

                    <div
                      className={styles.newCardIcon + ' vMiddleItem'}
                      onMouseEnter={() => this.setState({ videoIcon: 'video_mc_h' })}
                      onMouseLeave={() => this.setState({ videoIcon: 'video_mc' })}
                      onClick={() => {
                        this.handleEditArticle({ columnType, type: ARTICLE_TYPE_VIDEO });
                      }}
                    >
                      <Icon type={videoIcon} size={36} />
                      <div>视频</div>
                    </div>

                    <div className={styles.newCardIconTemp} />
                  </div>
                </div>
              </Col>
              {list.map((article) => {
                return (
                  <Col key={article.id} span={colSpan} style={colStyle}>
                    <ArticleCard
                      data={article}
                      isTopUser={isTopUser}
                      isTopOrg={isTopOrg}
                      orgDict={orgDict}
                      menuTree={menuTree}
                      onReload={onReload}
                      orgId={orgId}
                      onLoadingStart={onLoadingStart}
                      onLoadingEnd={onLoadingEnd}
                      recommendCount={recommendCount}
                      onEdit={() => onEditArticle({ id: article.id, columnType })}
                      columnType={columnType}
                      publishCount={publishCount}
                      mode={mode}
                      menuItem={menuItem}
                    />
                  </Col>
                );
              })}
            </Row>,
          ]}
        </div>
        {batchShow && (
          <BatchPage
            list={list.filter((li) => !li.isTopArticleInBranch)}
            onSaveSuccess={this.onBatchSuccess}
            onCancel={this.onBatchCancel}
            isTopUser={isTopUser}
            orgId={orgId}
            orgDict={orgDict}
            menuTree={menuTree}
            onLoadingStart={onLoadingStart}
            onLoadingEnd={onLoadingEnd}
            columnType={columnType}
            publishCount={publishCount}
            colSpan={colSpan}
            isIndex={isIndex}
            menuItem={menuItem}
          />
        )}
        {sortShow && (
          <SortPage
            mode={null}
            orgId={null}
            menuId={null}
            onSaveSuccess={this.onSortSuccess}
            onCancel={this.onSortCancel}
            onLoadingStart={onLoadingStart}
            onLoadingEnd={onLoadingEnd}
            onLoadList={onLoadListForSort}
            isTopUser={isTopUser}
            isTopOrg={isTopOrg}
            columnType={columnType}
            colCount={colCount}
            menuItem={menuItem}
          />
        )}
      </div>
    );
  }
}

ArticleManage.contextTypes = {
  isNowPersonal: PropTypes.func,
};

const mapStateToProps = (state) => {
  return {
    brenchId: state.account.user.brenchId,
    roleCode: state.account.user.roleCode,
    btnAuths: state.account.btnAuths,
    userId: state.account.user.userId,
    brenchName: state.account.user.brenchName,
  };
};

export default connect(mapStateToProps)(ArticleManage);
