/**
 * 从属于某个菜单的右边主要内容区
 */
import React from 'react';
import { connect } from 'dva';
import styles from './styles.less';
import Tabs from 'COMPONENT/Tabs';
import ArticleManage from '../ArticleManage';
import ViewManage from '../ViewManage';
import { getCmsContentList, getPersonalContentList } from 'SERVICE/ipadBackEnd';
import {
  STATUS_TOPUBLISH,
  getMenuDict,
  consultantDict,
  RECOMMEND_NO,
  RECOMMEND_YES,
  defaultListParams,
  BELONG_TYPE_TOP,
  BELONG_TYPE_BRANCH,
  MODE_BROWSER,
  BELONG_TYPE_CAMPUS,
  BELONG_TYPE_PERSONAL,
} from '../../common';
import PropTypes from 'prop-types';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { objectDeepEquals } from 'COMPONENT/utils/basic/obj';
import IndexArticleManage from '../IndexArticleManage';
import getWindowSize from 'UTIL/getWindowSize';

const Tab = Tabs.Tab;
const initActiveTab = 'articleList';
class MenuContent extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      totalRecords: 0, // {number} 列表总记录数
      activeTab: initActiveTab, //{string} 当前标签页key
      list: [], // {object[]} 图文管理、轮播图管理列表数据
      ...defaultListParams,
      indexList: [], // {object[]} 首页图文管理列表数据
      layoutType: this.getViewType(props), // {string} 平板展示方式
      articleListInited: false, // {boolean} 图文管理、轮播图管理数据初始化状态
      indexArticleListInited: false, // {boolean} 首页图文管理数据初始化状态
      winW: 0,
    };
  }

  componentDidMount() {
    this.updateWinW();
    window.addEventListener('resize', this.updateWinW, false);
  }

  componentWillReceiveProps(nextProps) {
    if (!objectDeepEquals(this.props, nextProps)) {
      if (this.props.isIndex !== nextProps.isIndex) {
        this.setState({ activeTab: 'articleList' });
      }
      const { activeTab } = this.state;
      let changed = { articleListInited: false, indexArticleListInited: false, ...defaultListParams };
      if (activeTab === 'articleList') {
        changed = { ...changed, ...defaultListParams, list: nextProps.contentData || [], articleListInited: true };
      } else if (activeTab === 'indexArticleList') {
        changed = { ...changed, indexList: nextProps.contentData || [], indexArticleListInited: true };
      }
      changed = { ...changed, layoutType: this.getViewType(nextProps) };
      this.setState(changed);
    }
  }

  componentWillUnmount() {
    window.removeEventListener('resize', this.updateWinW, false);
  }

  updateWinW = () => {
    this.setState({ winW: getWindowSize().width });
  };

  getColumnType = () => {
    return this.props.mode === MODE_BROWSER && this.props.isIndex && this.state.activeTab === 'articleList'
      ? 'carousel'
      : null;
  };

  isCarouselInBranch = () => {
    return this.getColumnType() === 'carousel' && !this.props.isTopOrg;
  };

  isDefaultTopBelongType = () => {
    return this.props.isTopUser && this.props.isTopOrg;
  };

  getViewType = ({ menuId, menuTree }) => {
    const menu = getMenuDict(menuTree).find((d) => {
      return d.val == menuId;
    });
    return menu ? menu.showContentNum + '' : null;
  };

  loadContentData = async (menuId) => {
    const { activeTab } = this.state;
    if (activeTab === 'articleList') {
      return await this.loadList({ ...this.getListParams(), menuId });
    } else if (activeTab === 'indexArticleList') {
      return await this.loadIndexList();
    }
    return true;
  };

  /**
 * 返回数据结构
 [
   {
    menuId: '1',
    menuName: '产品介绍',
    articles: [
      { ... }
    ]
  }
]
*/
  loadIndexList = async () => {
    const { orgId } = this.props;
    const data = await this.loadList({ orgId, recommend: RECOMMEND_YES });
    let list = [];
    data.forEach((d) => {
      const { fmenuId } = d;
      let idx;
      let group = list.find((li, i) => {
        if (li.menuId === fmenuId) {
          idx = i;
          return true;
        }
      });
      if (group) {
        let articles = [...group.articles, d];
        list[idx] = { ...list[idx], articles };
      } else {
        list.push({
          menuId: fmenuId,
          menuName: d.fmenuName,
          articles: [d],
        });
      }
    });
    return list;
  };

  getFindType = (belongType, campusId, consultorId, isTopOrg) => {
    //如果没有筛选权限，不需要findType参数
    if (!this.props.btnAuths.includes('CHECK_CAMPUS_INFO')) {
      return;
    }
    if (consultorId) {
      return BELONG_TYPE_PERSONAL;
    }
    if (campusId) {
      return BELONG_TYPE_CAMPUS;
    }
    if (!isTopOrg) {
      return BELONG_TYPE_BRANCH;
    }
  };

  loadList = async (params) => {
    const { mode, isTopOrg } = this.props;
    let apiParams = {
      brenchId: params.orgId,
      menuId: params.menuId,
      title: params.articleName || null,
      orgType: params.belongType ? params.belongType : this.isDefaultTopBelongType() ? BELONG_TYPE_TOP : null,
      findType: this.getFindType(params.belongType, params.campusId, params.consultorId, isTopOrg),
      status: params.publishStatus || null,
      isFront: params.recommend === RECOMMEND_YES ? 0 : params.recommend === RECOMMEND_NO ? 1 : null,
      orderType: params.sortType === 'desc' ? 'desc' : null,
      orderBy: params.sortName === 'time' ? 'publish_time' : params.sortName === 'readCount' ? 'visit_Num' : null,
      //个性化资料增加咨询师筛选项
      consultorId: params.consultorId || undefined,
      orgId: params.campusId || undefined,
    };
    let apiData;
    try {
      //个性化资料请求的接口不一样
      if (this.context.isNowPersonal()) {
        //如果是集团类型，数据太多了，先不查
        if (apiParams.findType === BELONG_TYPE_TOP) {
          apiData = [];
        } else {
          apiData = await getPersonalContentList(apiParams);
          if (apiData.resultCode !== 0) {
            throw new Error(apiData.resultMessage);
          }
          apiData = apiData.data;
        }
      } else {
        apiData = await getCmsContentList(apiParams);
      }
    } catch (e) {
      showMessage(e.text);
      return;
    }
    const data = apiData.map((d) => {
      const isBelongTop = d.orgType === BELONG_TYPE_TOP;
      return {
        id: d.id,
        title: d.title,
        publishTime: d.publishTime,
        isAppointment: d.status === STATUS_TOPUBLISH,
        isRecommend: d.isFront === 0,
        imgSrc: d.facePic, //https://imgsa.baidu.com/exp/pic/item/8605f5f81a4c510f73efaa146859252dd42aa552.jpg
        orgType: d.orgType,
        orgTypeName: d.orgTypeName,
        readCount: d.visitNum,
        isBelongTop,
        isTopArticleInBranch: isBelongTop && !isTopOrg,
        publishStatus: d.status,
        menuId: d.menuId,
        fmenuId: d.fmenuId,
        fmenuName: d.fmenuName,
        appointmentPublishTime: d.appointmentPublishTime,
        createUser: d.createUser,
        publishUserName: d.publishUserName,
      };
    });
    return data;
  };

  getListParams = () => {
    const { orgId, menuId } = this.props;
    const { articleName, belongType, publishStatus, recommend, currentPage, pageSize, sortName, sortType } = this.state;
    return {
      orgId,
      menuId,
      articleName,
      belongType,
      publishStatus,
      recommend,
      currentPage,
      pageSize,
      sortName,
      sortType,
    };
  };

  updateList = (changedParams = {}) => {
    const { onLoadingStart, onLoadingEnd } = this.props;
    this.setState(changedParams);
    onLoadingStart();
    this.loadList({ ...this.getListParams(), ...changedParams }).then((list) => {
      this.setState({ list, articleListInited: true });
      onLoadingEnd();
    });
  };

  updateIndexList = () => {
    const { onLoadingStart, onLoadingEnd } = this.props;
    onLoadingStart();
    this.loadIndexList().then((list) => {
      this.setState({ indexList: list, indexArticleListInited: true });
      onLoadingEnd();
    });
  };

  search = (keys) => {
    const { inputValue, ...otherKeys } = keys;
    const changedParams = { articleName: keys.inputValue, ...otherKeys, currentPage: 1 };
    this.updateList(changedParams);
  };

  onSortChange = (sort) => {
    const changedParams = { sortName: sort ? sort.name : null, sortType: sort ? sort.type : null };
    this.updateList(changedParams);
  };

  tabChange = (key) => {
    this.setState({ activeTab: key }, () => {
      const { articleListInited, indexArticleListInited } = this.state;
      if (key === 'articleList' && !articleListInited) {
        this.updateList();
      } else if (key === 'indexArticleList' && !indexArticleListInited) {
        this.updateIndexList();
      }
    });
  };

  openEditPage = (params, isIndexArticle) => {
    const { dispatch, route, menuId, isTopOrg, mode, orgId, getUserOrgType, menuItem } = this.props;
    const { isNowPersonal } = this.context;
    const payload = {
      path: `/${route.path}/Edit`,
      params: {
        ...params,
        orgId,
        useType: mode,
        menuId,
        menuItem,
        isNowPersonal,
        orgType: getUserOrgType(),
        onSaveSuccess: () => this.editSaveSuccess(isIndexArticle),
      },
      refresh: true,
    };
    dispatch({
      type: 'main/openPage',
      payload,
    });
  };

  editSaveSuccess = (isIndexArticle) => {
    const { dispatch, route } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: { path: `/${route.path}` },
    });
    if (isIndexArticle) {
      this.updateIndexList();
    } else {
      this.updateList();
    }
  };

  render() {
    const {
      orgId,
      orgDict,
      onLoadingStart,
      onLoadingEnd,
      menuTree,
      isTopUser,
      isTopOrg,
      belongTypeDict,
      publishStatusDict,
      recommendDict,
      isIndex,
      route,
      menuId,
      onReloadPage,
      mode,
      menuItem,
    } = this.props;
    const {
      sortName,
      sortType,
      articleName,
      belongType,
      publishStatus,
      recommend,
      list,
      indexList,
      articleListInited,
      indexArticleListInited,
      activeTab,
      layoutType,
      winW,
      consultant,
      campusId,
      consultorId,
    } = this.state;
    const colCount = winW < 1600 ? 2 : winW < 2200 ? 3 : 4;
    let tabs = [
      <Tab
        title={isIndex && mode === MODE_BROWSER ? '轮播图管理' : '图文管理'}
        className={styles.tab}
        key="articleList"
        value="articleList"
      >
        {articleListInited && (
          <ArticleManage
            belongTypeDict={belongTypeDict}
            publishStatusDict={publishStatusDict}
            recommendDict={recommendDict}
            consultantDict={consultantDict}
            consultant={consultant}
            campusId={campusId}
            consultorId={consultorId}
            articleName={articleName}
            belongType={belongType}
            publishStatus={publishStatus}
            recommend={recommend}
            list={list}
            sort={sortName ? { name: sortName, type: sortType } : null}
            onSearch={this.search}
            onInputChange={(value) => this.setState({ articleName: value })}
            onSortChange={this.onSortChange}
            onReload={this.updateList}
            onLoadingStart={onLoadingStart}
            onLoadingEnd={onLoadingEnd}
            orgDict={orgDict}
            menuTree={menuTree}
            isTopUser={isTopUser}
            isTopOrg={isTopOrg}
            orgId={orgId}
            columnType={this.getColumnType()}
            route={route}
            onLoadList={() => this.loadList({ orgId, menuId, ...this.getListParams() })}
            onLoadListForSort={() => this.loadList({ orgId, menuId })}
            menuId={menuId}
            menuItem={menuItem}
            mode={mode}
            isCarouselInBranch={this.isCarouselInBranch()}
            isDefaultTopBelongType={this.isDefaultTopBelongType()}
            colCount={colCount}
            onEditArticle={this.openEditPage}
            isIndex={isIndex}
          />
        )}
      </Tab>,
    ];

    if (isIndex && mode === MODE_BROWSER) {
      tabs.push(
        <Tab title="首页图文管理" className={styles.tab} key="indexArticleList" value="indexArticleList">
          {indexArticleListInited && (
            <IndexArticleManage
              onLoadingStart={onLoadingStart}
              orgDict={orgDict}
              menuTree={menuTree}
              isTopUser={isTopUser}
              isTopOrg={isTopOrg}
              orgId={orgId}
              list={indexList}
              onReload={this.updateIndexList}
              onLoadingEnd={onLoadingEnd}
              colCount={colCount}
              mode={mode}
              menuItem={menuItem}
              onEditArticle={(params) => this.openEditPage(params, true)}
            />
          )}
        </Tab>,
      );
    } else {
      tabs.push(
        <Tab title="平板展示" className={styles.tab} key="viewManage" value="viewManage">
          <ViewManage
            layoutType={layoutType}
            menuId={menuId}
            onReload={onReloadPage}
            onLoadingStart={onLoadingStart}
            menuItem={menuItem}
            onLoadingEnd={onLoadingEnd}
          />
        </Tab>,
      );
    }
    return (
      <div className={styles.wrap}>
        <Tabs navAlign="center" className={styles.tabs} activeKey={activeTab} onChange={this.tabChange}>
          {tabs}
        </Tabs>
      </div>
    );
  }
}

MenuContent.contextTypes = {
  isNowPersonal: PropTypes.func,
};

const mapStateToProps = (state) => {
  return {
    roleCode: state.account.user.roleCode,
    btnAuths: state.account.btnAuths,
  };
};

export default connect(mapStateToProps, null, null, { withRef: true })(MenuContent);
