import React from 'react';
import { Tree } from 'antd';
import { connect } from 'dva';
import styles from './styles.less';
import Icon from 'COMPONENT/Icon';
import IconButton from 'COMPONENT/IconButton';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import { getMenuDict } from '../../common';
import { objectDeepEquals } from 'COMPONENT/utils/basic/obj';
import { setShowMenuContent } from 'SERVICE/ipadBackEnd';
import classnames from 'classnames';

const TreeNode = Tree.TreeNode;

class MenuTree extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      data: [],
      expandedKeys: [],
      selectedKeys: [],
    };
  }

  componentWillReceiveProps(nextProps) {
    if (!objectDeepEquals(this.props.data, nextProps.data)) {
      const { data, selected: menuId } = nextProps;
      const selectedMenu = data.find((ele) => {
        return +ele.id === +menuId || (ele.children.length && !!ele.children.find((ele) => +ele.id === +menuId));
      });

      const expandedKeys = selectedMenu && !!!selectedMenu.visible ? ['' + selectedMenu.id] : [];
      const selectedKeys = menuId ? [menuId] : [];
      this.setState({ data, expandedKeys, selectedKeys });
    }
  }

  onSelect = (keys, e) => {
    const hasChild = !!e.node.props.children;
    const visible = !!e.node.props.visible;
    const canExpand = visible ? !visible : hasChild;
    if (canExpand) {
      const expandedKeys = keys.filter((ele) => this.state.expandedKeys.indexOf(ele) < 0);
      this.setState({ expandedKeys, selectedKeys: [] });
      return;
    } else if (hasChild) {
      this.setState({ expandedKeys: [] });
    }
    if (keys.length > 0) {
      this.setState({ selectedKeys: keys });
      this.props.onChange(keys[0]);
    }
  };

  onToggleVisible = (id) => {
    const { data } = this.state;
    let vis;
    let item;
    let parentItem;
    data.forEach((d, i) => {
      if (d.id === id) {
        item = d;
        vis = !d.visible;
      } else if (d.children) {
        d.children.forEach((c, j) => {
          if (c.id === id) {
            item = c;
            parentItem = d;
            vis = !c.visible;
          }
        });
      }
    });
    const parentId = parentItem && parentItem.id;
    let tips;
    const { children } = item;
    if (vis) {
      if (children && children.length > 0 && children.find((c) => c.visible)) {
        tips = '显示一级目录的图文将隐藏其下所有子目录的图文，确定吗？';
      }
      /*      if (parentItem && parentItem.visible) {
        tips = "显示二级目录的图文将隐藏其父目录的图文，确定吗？"
      }*/
    } else {
      if (children && children.length > 0 && children.find((c) => !c.visible)) {
        tips = '隐藏一级目录的图文将显示其下所有子目录的图文，确定吗？';
      }
    }
    if (tips) {
      showConfirm(tips, () => this.toggleVisible(item, vis, parentId));
    } else {
      this.toggleVisible(item, vis, parentId);
    }
  };

  toggleVisible = (item, vis, parentId) => {
    const { onLoadingStart, onLoadingEnd } = this.props;
    const { data } = this.state;
    let newData = data.map((d) => {
      const { id, children } = d;
      let newItem = { ...d };
      if (id === item.id) {
        newItem.visible = vis;
      }
      if (children && children.length > 0) {
        newItem.children = children.map((c) => {
          let newCItem = { ...c };
          if (c.id === item.id) {
            newCItem.visible = vis;
          }
          return newCItem;
        });
      }
      return newItem;
    });
    if (vis) {
      newData = newData.map((d) => {
        const { id, children } = d;
        let newItem = { ...d };
        if (id === parentId) {
          newItem.visible = false;
        }
        if (id === item.id && children && children.length > 0) {
          newItem.children = children.map((c) => {
            return { ...c, visible: false };
          });
        }
        return newItem;
      });
    } else {
      newData = newData.map((d) => {
        const { id, children } = d;
        let newItem = { ...d };
        if (id === parentId) {
          newItem.visible = true;
        }
        if (id === item.id && children && children.length > 0) {
          newItem.children = children.map((c) => {
            return { ...c, visible: true };
          });
        }
        return newItem;
      });
    }
    this.setState({ data: newData, selectedKeys: [] });
    onLoadingStart();
    setShowMenuContent({ id: item.id, status: vis ? 1 : 0 }, true).then(
      () => {
        showMessage('操作成功', 'success');
        onLoadingEnd();
      },
      () => {
        showMessage('操作失败');
        this.setState({ data });
        onLoadingEnd();
      },
    );
  };

  onEyeClick = (e, id) => {
    e.stopPropagation();
    this.onToggleVisible(id);
  };

  /**
   * 判断是否有切换显示内容或子目录的权限
   * @returns {boolean}
   */
  hasEye = () => {
    const { nowRouter, btnAuths } = this.props;
    //如果当前为工作台模式内容
    if (nowRouter === '/ArticleEditWorkbench') {
      return btnAuths.indexOf('TOGGLE_WORK_DIR') > -1;
      //如果当前为浏览模式内容
    } else if (nowRouter === '/ArticleEditBrowser') {
      return btnAuths.indexOf('TOGGLE_BROWSER_DIR') > -1;
    }
  };

  getNodeContent = (data, level) => {
    const { title, id, visible } = data;
    const isChildNode = level === 2;
    const eyeCls = {
      [styles.eye]: 1,
      [styles.eyeUnClickable]: isChildNode,
    };
    return (
      <div className={styles.content}>
        <div className={styles.title}>{title}</div>
        {this.hasEye() && (
          <div
            className={classnames(eyeCls)}
            onClick={isChildNode ? null : (e) => this.onEyeClick(e, id)}
            onMouseOver={(e) => {
              e.stopPropagation();
              e.preventDefault();
            }}
          >
            {isChildNode ? (
              <Icon type={visible ? 'visible_mc' : 'hide_mc'} />
            ) : (
              <IconButton type={visible ? 'visible_mc' : 'hide_mc'} />
            )}
          </div>
        )}
      </div>
    );
  };

  render() {
    const { selected } = this.props;
    const { data, testVisible, expandedKeys, selectedKeys } = this.state;

    return (
      <div className={styles.wrap}>
        <Tree selectedKeys={selectedKeys} onSelect={this.onSelect} expandedKeys={expandedKeys}>
          {data &&
            data.map((d) => {
              const { id, children, visible } = d;
              return (
                <TreeNode key={id} visible={visible} title={this.getNodeContent(d)}>
                  {children &&
                    children.length > 0 &&
                    children.map((c) => <TreeNode key={c.id} visible={c.visible} title={this.getNodeContent(c, 2)} />)}
                </TreeNode>
              );
            })}
        </Tree>
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    nowRouter: state.main.activeTabId,
    //nowRouter: routing.locationBeforeTransitions.activeTabId.pathname,
    btnAuths: state.account.btnAuths,
  };
};

export default connect(mapStateToProps)(MenuTree);
