/**
 * 顺序管理页面
 * @property {string} mode - 浏览模式或工作台模式，'browser'|'workbench'
 * @property {string} orgId - 机构ID
 * @property {string} menuId - 文章所属菜单ID
 * @property {function} onSaveSuccess - 保存成功时的回调
 * @property {function} onCancel - 取消时的回调
 */
import React from 'react';
import styles from './styles.less';
import Button from 'COMPONENT/Button';
import ArticleCard from '../ArticleCard';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import styleVars from 'COMPONENT/theme/variables';
import Reorder from 'react-reorder';
import defImgSrc from '../def.png';
import { saveCmsContentOrder } from 'SERVICE/ipadBackEnd';
import SortList from '../SortList';
import commonStyles from 'PAGE/ipadBackEnd/articleEdit/commonStyles.less';

class SortPage extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      list: [], //文章列表
    };
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化页面内容
   */
  initContent = () => {
    const { onLoadingStart, onLoadingEnd, onLoadList, isTopOrg } = this.props;
    onLoadingStart();
    onLoadList().then((list) => {
      this.setState({ list: list.filter((li) => !li.isTopArticleInBranch) });
      onLoadingEnd();
    });
  };

  sortChange = (event, item, index, newIndex, list) => {
    this.setState({ list });
  };

  ok = () => {
    const { onLoadingStart, onLoadingEnd, onSaveSuccess } = this.props;
    const { list } = this.state;
    const params = list.map((d, i) => {
      return { id: d.id, orderNum: i };
    });
    onLoadingStart();
    saveCmsContentOrder(params).then(
      () => {
        onLoadingEnd();
        showMessage('操作成功', 'success');
        onSaveSuccess();
      },
      (e) => {
        onLoadingEnd();
        showMessage(e.text || '操作失败');
      },
    );
  };

  render() {
    const { onCancel, isTopUser, columnType, colCount, menuItem } = this.props;
    const { list } = this.state;
    const noData = list.length === 0;
    return (
      <div className={styles.wrap}>
        <div className={styles.header}>
          <div className={styles.headerRight}>
            <Button type="primary" onClick={this.ok} disabled={noData}>
              确定
            </Button>
            <Button onClick={onCancel}>取消</Button>
          </div>
        </div>
        {noData ? (
          <div className={commonStyles.noDataTips}>没有可排序操作的图文!</div>
        ) : (
          <SortList
            colCount={colCount}
            menuItem={menuItem}
            isTopUser={isTopUser}
            columnType={columnType}
            list={list}
            onChange={(list) => this.setState({ list })}
          />
        )}
      </div>
    );
  }
}

export default SortPage;
