import React from 'react';
import { Row, Col } from 'antd';
import Radio from 'COMPONENT/Radio';
import img4 from '../layout4.png';
import img9 from '../layout9.png';
import { LAYOUT_TYPE_4, LAYOUT_TYPE_9, layoutTypeDict } from '../../common';
import { getDictTextByVal } from 'UTIL/dict';
import styles from './styles.less';
import { saveCmsMenuContentNum } from 'SERVICE/ipadBackEnd';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

class ViewManage extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      layoutType: null,
    };
  }

  componentWillReceiveProps(nextProps) {
    this.setState({ layoutType: nextProps.layoutType });
  }

  save = (type) => {
    const { onLoadingStart, onLoadingEnd, onReload, menuId } = this.props;
    onLoadingStart();
    saveCmsMenuContentNum({ id: parseInt(menuId), showContentNum: parseInt(type) }, true).then(
      () => {
        showMessage('操作成功', 'success');
        onLoadingEnd();
        onReload();
      },
      (e) => {
        showMessage(e.text || '操作失败');
        onLoadingEnd();
      },
    );
  };

  render() {
    const { layoutType } = this.state;
    return (
      <Row style={{ margin: '60px 0' }} gutter={30}>
        <Col span={12} className={styles.col + (layoutType === LAYOUT_TYPE_4 ? ' ' + styles.colActive : '')}>
          <div
            className={styles.imgWrap}
            onClick={layoutType === LAYOUT_TYPE_4 ? null : () => this.save(LAYOUT_TYPE_4)}
          >
            <img src={img4} />
          </div>
          <div className={styles.radioWrap}>
            <Radio
              value={LAYOUT_TYPE_4}
              checked={layoutType === LAYOUT_TYPE_4}
              onChange={() => this.save(LAYOUT_TYPE_4)}
            >
              {getDictTextByVal(layoutTypeDict, LAYOUT_TYPE_4)}
            </Radio>
          </div>
        </Col>
        <Col span={12} className={styles.col + (layoutType === LAYOUT_TYPE_9 ? ' ' + styles.colActive : '')}>
          <div
            className={styles.imgWrap}
            onClick={layoutType === LAYOUT_TYPE_9 ? null : () => this.save(LAYOUT_TYPE_9)}
          >
            <img src={img9} />
          </div>
          <div className={styles.radioWrap}>
            <Radio
              value={LAYOUT_TYPE_9}
              checked={layoutType === LAYOUT_TYPE_9}
              onChange={() => this.save(LAYOUT_TYPE_9)}
            >
              {getDictTextByVal(layoutTypeDict, LAYOUT_TYPE_9)}
            </Radio>
          </div>
        </Col>
      </Row>
    );
  }
}

export default ViewManage;
