import React from 'react';
import Button from 'COMPONENT/Button';
import styles from './index.less';
import { Upload, Progress, Icon as AntIcon } from 'antd';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import { getSseKey } from 'SERVICE/common';
import { uuid } from 'COMPONENT/utils/basic/str';
import styleVars from 'COMPONENT/theme/variables';
import moment from 'moment';
import IconButton from 'COMPONENT/IconButton';
import Icon from 'COMPONENT/Icon';
import xls from 'ASSET/imgs/xls.png';
import doc from 'ASSET/imgs/doc.png';
import ppt from 'ASSET/imgs/ppt.png';
import pdf from 'ASSET/imgs/pdf.png';
import TextLink from '../../../../../components/TextLink';

let mapObj = { xls, doc, ppt, pdf };

//支持的视频格式
const ALLOWTYPE = [{ type: 'video/mp4', name: 'mp4' }];
//支持的视频大小
const MAXSIZE = 2048;

class Video extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      disabled: false,
      progress: false,
      docList: [],
    };
  }

  beforeUpload = (file, fileList) => {
    if (file.type) {
      let type = file.type;
      if (!ALLOWTYPE.find((a) => a.type === type)) {
        showMessage(`只支持上传 mp4 格式`);
        return false;
      } else {
        let isLt50M = file.size / 1024 / 1024 < MAXSIZE;
        if (!isLt50M) {
          ShowMessage('文件大小超过2G');
          return false;
        }
        this.uploadDoc(file);
        return false;
      }
    }
    return false;
  };

  uploadDoc = async (file) => {
    // let {docList} = this.state;
    let splitArray = file.name.split('.');
    let type = splitArray[splitArray.length - 1];

    let { files, handleFiles } = this.props;
    let configKey = await getSseKey();
    let realname = /\.(?:png|jpg|jpeg)$/gi.test(file.name) ? file.name : file.name + '.' + file.type;
    let fileKey = uuid();
    let formData = new FormData();

    formData.append('x:realname', realname);
    formData.append('x:uploadtype', 'ipad');
    formData.append('x:uploadid', uuid());
    formData.append('key', fileKey + '.' + type);
    Object.keys(configKey).forEach((key, index) => {
      if (key != 'host') formData.append(key, configKey[key]);
    });
    formData.append('file', file);

    let xhr = new XMLHttpRequest();
    xhr.open('POST', configKey.host, true);
    xhr.upload.onloadstart = () => {
      this.setState({ disabled: true, progress: true });
    };
    xhr.upload.onprogress = (file) => {
      this.setState({ percent: (100 * file.loaded) / file.total });
    };
    xhr.onload = () => {
      let size = file.size / 1024 / 1024;
      let unit;
      if (size >= 1) {
        unit = 'M';
      } else {
        unit = 'K';
        size = size * 1024;
      }
      let aliSaveTime = moment().format('YYYY-MM-DD HH:mm');
      size = size.toFixed(2);
      handleFiles('add', {
        fileSize: size + unit,
        fileName: file.name,
        type,
        aliSaveTime,
        filePath: configKey.host + fileKey + '.' + type,
      });
      this.setState({ disabled: false, progress: false, percent: 0 });
    };
    xhr.send(formData);
    // let res = await fetch(configKey.host, {method: "POST", body: formData, mode: 'cors'});
  };
  deleteDoc = async (index) => {
    let { handleFiles } = this.props;
    showConfirm('该操作会删除已上传的视频，请谨慎操作', () => {
      handleFiles('delete', index);
    });
  };

  render() {
    let { disabled, progress, percent, docList } = this.state;
    let { files } = this.props;
    disabled = disabled === false ? files.length >= 1 : true;
    return (
      <div className={styles.docContains}>
        <div>
          <div className={styles.title}>上传视频</div>
          <div style={{ width: 650 }}>
            <Upload
              className={styles.hideAntUploadList}
              beforeUpload={(file, fileList) => this.beforeUpload(file, fileList)}
            >
              <Button disabled={disabled}>点击上传</Button>
            </Upload>
            <div className={styles.tips}>
              支持格式：mp4；且单个视频大小不能超过2G&nbsp;&nbsp;
              <AntIcon style={{ color: styleVars.primaryColor, fontSize: 20 }} type="question" />
              <TextLink
                onClick={() => {
                  window.open('/video_transform.pdf', '_blank');
                }}
              >
                视频转码指引
              </TextLink>
            </div>
            {progress && <Progress percent={percent.toFixed(0)} size="small" strokeWidth={4} className={'progress'} />}
            {files.map((v, k) => {
              return (
                <div key={k} className={styles.docList}>
                  {/*<span><img src={mapObj[(v.fileName.split('.')[1] || '').substr(0, 3)]} alt=""/></span>*/}
                  <Icon type="upload_video" size={36} style={{ marginRight: 10 }} />
                  <span>{v.fileName}</span>
                  <span>{v.fileSize}</span>
                  <span>{v.aliSaveTime}</span>
                  <span>
                    <IconButton type={'cross'} onClick={(e) => this.deleteDoc(k)} />
                  </span>
                </div>
              );
            })}
          </div>
        </div>
      </div>
    );
  }
}

export default Video;
