import React from 'react';
import Page from 'COMPONENT/Page';
import styles from './index.less';
import TitleCarousel from './components/titleCarousel';
import Link from './components/link';
import Doc from './components/doc';
import Video from './components/video';
import Label from './components/label';
import ImageText from './components/imageText';
import UploadCover from './viewModal/uploadCover';
import UploadCarousel from './viewModal/uploadCarousel';
import PreviewModal from './viewModal/previewModal';
import {
  ARTICLE_TYPE_PICTURE,
  ARTICLE_TYPE_DOCUMENT,
  ARTICLE_TYPE_LINK,
  ARTICLE_TYPE_VIDEO,
  PERSONALINFO,
} from '../common';
import { getTagList, getContentById, saveTags, saveContent, savePersonalContent } from 'SERVICE/articleEdit';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

class Edit extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      carouselVisible: null,
      coverVisible: null,
      previewVisible: null,
      facePic: null,
      type: null, //类型
      tagList: {},
      tags: [], //已选标签id
      tagsTypeObj: {},
      files: [],
    };
  }

  componentDidMount() {
    this.setState({ loading: true });
    Promise.all([this.getTagList(), this.getContentById()]).then(([tagList, contentData]) => {
      let { type, title, facePic, content, signList = [], files = [] } = contentData;
      let { tagsTypeObj } = this.state;
      let keys = Object.keys(tagList);
      Object.values(tagList).forEach((v, k) => {
        let array = [];
        v.forEach((value, key) => {
          signList.forEach((V, K) => {
            if (value.value === V.cmsSignId) {
              value.checked = true;
              array.push(V.cmsSignId);
            }
          });
        });
        tagsTypeObj[keys[k]] = array;
      });
      this.setState(
        {
          tagList, //标签种类
          contentData,
          files,
          type,
          facePic,
          title,
          content,
          signList,
          loading: false,
        },
        () => {
          if (type === 'PICTURE') {
            this.Ueditor.setContent(content);
            this.Ueditor.setTitle(title);
          }
        },
      );
    });
  }

  getTagList = async () => {
    let optionsArray = {};
    let apiParams = {
      type: '',
      useType: this.props.useType || 'VISITOR', //LOGIN_USER
    };
    let data = await getTagList(apiParams);
    data.forEach((v, k) => {
      if (!(v.typeName in optionsArray)) {
        optionsArray[v.typeName] = [];
        optionsArray[v.typeName].push({ label: v.value, value: v.id });
      } else {
        optionsArray[v.typeName].push({ label: v.value, value: v.id });
      }
    });
    return optionsArray;
  };
  updateTagList = (data) => {
    this.setState({ tagList: data });
  };
  getContentById = async () => {
    let { id } = this.props;
    id = id ? id : null;
    if (id) {
      let data = await getContentById({ id });
      return data;
    } else {
      return {};
    }
  };
  saveContent = () => {
    let { contentData, type, content, facePic, tags, tagsTypeObj, signList, files } = this.state;
    type = type ? type : this.props.type;
    let signIds = Object.values(tagsTypeObj).reduce((pre, current) => {
      return pre.concat(current);
    }, []);
    let title;
    const error = (text) => this._page.showOkTips(text);
    if (type === 'PICTURE') {
      title = this.Ueditor.getContent().title;
      content = this.Ueditor.getContent().content;
      let $overflow = document.querySelector('.editorOverflow');
      if ($overflow) {
        error('内容请保持在10000字以下!');
        return;
      }
    } else {
      title = this.state.title;
    }

    if (!title) {
      error('请输入标题!');
      return;
    }
    if (!facePic) {
      error('请选择封面图!');
      return;
    }
    if (!content) {
      if (type === ARTICLE_TYPE_PICTURE) {
        error('请输入内容!');
        return;
      } else if (type === ARTICLE_TYPE_LINK) {
        error('请输入链接!');
        return;
      }
    }
    if (signIds.length === 0) {
      error('请选择标签!');
      return;
    }
    let fileList = files.map((v, k) => {
      return {
        id: v.id,
        aliSaveTime: v.aliSaveTime,
        fileName: v.fileName,
        filePath: v.filePath,
        fileSize: v.fileSize,
      };
    });
    if (fileList.length === 0) {
      if (type === ARTICLE_TYPE_DOCUMENT) {
        error('请上传文档!');
        return;
      } else if (type === ARTICLE_TYPE_VIDEO) {
        error('请上传视频');
        return;
      }
    }
    let isFront = contentData.isFront;
    if (isFront === undefined) {
      isFront = 1;
    }

    let apiParams = {
      allCanSee: contentData.allCanSee,
      content,
      facePic, //封面图
      createUser: contentData.createUser, //作者也要加上
      files: fileList,
      id: contentData.id || null,
      isFront: isFront, //0 首页推荐，1非推荐
      menuId: contentData.menuId || this.props.menuId, //目录id
      orderNum: contentData.orderNum || null, //排序
      orgType: contentData.orgType || this.props.orgType, //GROUNP 集团，BRENCH 分公司,CAMPUS学习中心
      publishTime: contentData.publishTime || null, //发布时间
      remark: contentData.remark || null, //
      signIds, //标签id
      status: contentData.status || 'UN_PUBLISH', //UN_PUBLISH 未发布 TO_PUBLISH 预约发布  PUBLISH 已发布，
      title,
      type: contentData.type || this.props.type, // PICTURE  图文  DOCUMENT 文档  LINK 链接
      visitNum: contentData.visitNum || null,
    };
    this.setState({ loading: true });
    const method = this.props.isNowPersonal() ? savePersonalContent : saveContent;
    method(apiParams)
      .then((res) => {
        this.setState({ loading: false });
        showMessage('保存成功', 'success');
        //  回调函数 跳转页面
        this.props.onSaveSuccess();
      })
      .catch((res) => {
        this.setState({ loading: false });
      });
  };
  upDateSrc = (src) => {
    this.setState({ coverVisible: false, carouselVisible: false, facePic: src });
  };
  updateContent = (type, val, tagsType) => {
    if (tagsType) {
      let { tagsTypeObj } = this.state;
      tagsTypeObj[tagsType] = val;
      this.setState({});
    } else {
      this.setState({ [type]: val });
    }
  };
  handleFiles = (handle, value) => {
    let { files } = this.state;
    if (handle === 'add') {
      files.push(value);
    } else {
      files.splice(value, 1);
    }
    this.setState({ files });
  };
  /**
   * 效果预览
   */
  preview = () => {
    let { title, content } = this.Ueditor.getContent();
    this.setState({ previewVisible: true, title, content });
  };

  render() {
    let { carouselVisible, coverVisible, previewVisible, tagList, files, type, title, content, facePic, loading } =
      this.state;
    let { columnType, type: propsType, mode, route, orgId, useType, pageType } = this.props;
    type = type ? type : propsType;

    const showOkBtn = !pageType || pageType !== 'view';

    return (
      <Page
        loading={loading}
        route={this.props.route}
        className={styles.edit}
        footer={true}
        okBtnShow={showOkBtn}
        okBtnText="保存修改"
        onOk={this.saveContent}
        onCancel={this.preview}
        cancelBtnText="效果预览"
        ref={(e) => (this._page = e)}
      >
        <div className={styles.wrap}>
          <div style={{ marginBottom: 15 }}>
            {type === 'PICTURE' && (
              <ImageText
                carouselClick={(e) => this.setState({ carouselVisible: true })}
                coverClick={(e) => this.setState({ coverVisible: true })}
                ref={(e) => (this.Ueditor = e)}
                showType={columnType}
                title={title}
                content={content}
                facePic={facePic}
              />
            )}
            {type !== 'PICTURE' && (
              <TitleCarousel
                carouselClick={(e) => this.setState({ carouselVisible: true })}
                coverClick={(e) => this.setState({ coverVisible: true })}
                showType={columnType}
                title={title}
                updateTitle={this.updateContent}
                facePic={facePic}
              />
            )}
            {type === 'LINK' && <Link updateLink={this.updateContent} link={content} />}
            {type === 'DOCUMENT' && <Doc updateDoc={this.updateContent} files={files} handleFiles={this.handleFiles} />}
            {type === 'VIDEO' && (
              <Video updateVideo={this.updateContent} files={files} handleFiles={this.handleFiles} />
            )}
            <Label
              data={tagList}
              mode={mode}
              route={route}
              orgId={orgId}
              useType={useType}
              updateTagList={this.updateTagList}
              getTagList={this.getTagList}
              updateTags={this.updateContent}
            />
          </div>
        </div>
        {carouselVisible && (
          <UploadCarousel
            onClose={(e) => this.setState({ carouselVisible: false })}
            upDateSrc={(src) => this.upDateSrc(src)}
          />
        )}
        {coverVisible && (
          <UploadCover
            onClose={(e) => this.setState({ coverVisible: false })}
            upDateSrc={(src) => this.upDateSrc(src)}
          />
        )}
        {previewVisible && (
          <PreviewModal title={title} content={content} onClose={(e) => this.setState({ previewVisible: false })} />
        )}
      </Page>
    );
  }
}

export default Edit;
