import React from 'react';
import { Upload, Icon, Progress } from 'antd';
import ShowMessage from 'COMPONENT/utils/globalTips/showMessage';
import styles from './index.less';
import { getSseKey } from 'SERVICE/common';
import { uuid } from 'COMPONENT/utils/basic/str';

class PicturesWall extends React.Component {
  state = {
    previewVisible: false,
    previewImage: '',
    fileList: [],
    uploaded: false,
    percent: 0,
    showProgress: false,
  };

  componentDidMount() {
    let { srcProps, width, height } = this.props;
    if (srcProps) {
      this.setState({ uploaded: true });
    }
  }

  beforeUpload = (file, fileList) => {
    let isJPGOrPNG = file.type === 'image/jpeg' || file.type === 'image/png';

    let isLt500K = file.size / 1024 / 1024 < 0.5;

    if (!isJPGOrPNG) {
      ShowMessage('只能上传jpg/png文件');
      return false;
    }
    if (!isLt500K) {
      ShowMessage('图片大小超过500k');
      return false;
    }
    let reader = new FileReader();
    reader.addEventListener(
      'load',
      async () => {
        this.setState({ file }, async () => {
          let src = await this.uploadImg();
          // upDateSrc(src);
        });
      },
      false,
    );
    reader.readAsDataURL(file);
    return false;
  };

  uploadImg = async () => {
    let { upDateSrc } = this.props;
    let { file } = this.state;
    let configKey = await getSseKey();
    let realname = /\.(?:png|jpg|jpeg)$/gi.test(file.name) ? file.name : file.name + '.' + file.type;
    let fileKey = uuid();
    let formData = new FormData();

    formData.append('x:realname', realname);
    formData.append('x:uploadtype', 'ipad');
    formData.append('x:uploadid', uuid());
    formData.append('key', fileKey);
    Object.keys(configKey).forEach((key, index) => {
      if (key != 'host') formData.append(key, configKey[key]);
    });
    formData.append('file', file);

    let xhr = new XMLHttpRequest();
    xhr.open('POST', configKey.host, true);
    xhr.upload.onloadstart = (file) => {
      this.setState({ uploaded: true });
    };
    xhr.upload.onprogress = (file) => {
      this.setState({ showProgress: true, percent: (file.loaded * 100) / file.total });
    };
    xhr.onload = () => {
      let src = configKey.host + fileKey;
      upDateSrc(src);
      this.setState({ src, showProgress: false });
    };
    xhr.send(formData);
  };

  render() {
    let { fileList, src, uploaded, percent, showProgress } = this.state;
    let { srcProps, tips: Tips, iconSize } = this.props;
    src = srcProps ? srcProps : src;
    const uploadButton = (
      <div className={styles.wrap}>
        <div className={styles.iconWrap}>
          <Icon type="plus" style={{ fontSize: iconSize || 28 }} />
          {Tips && !uploaded && <Tips />}
        </div>
        {showProgress && (
          <Progress percent={percent} size="small" strokeWidth={4} showInfo={false} className={'progress'} />
        )}
        <div className={'uploadMask'}></div>
        <Icon type="upload" style={{ fontSize: iconSize || 28 }} />
        <img src={src} alt="" className={'img'} />
      </div>
    );
    return (
      <div>
        <Upload
          listType="picture-card"
          beforeUpload={(file, fileList) => this.beforeUpload(file, fileList)}
          className={styles.uploadImg + ' ' + (uploaded ? styles.uploaded : '')}
        >
          {uploadButton}
        </Upload>
      </div>
    );
  }
}

export default PicturesWall;
