/**
 * 复制图文
 */
import React from 'react';
import classnames from 'classnames';
import Checkbox from 'COMPONENT/Checkbox';
import Modal from 'COMPONENT/Modal';
import styles from './styles.less';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { getMenuDict, BELONG_TYPE_TOP, BELONG_TYPE_BRANCH, TOP_ORG_ID, PERSONALINFO } from '../../../common';
import { copyContent } from 'SERVICE/ipadBackEnd';
import getWrappedInstance from 'UTIL/getWrappedInstance';

class CopyModal extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false, // 等待状态
      checkedOrgIds: [], // 选中的机构ID集合
      checkedMenuIds: [], // 选中的菜单ID集合
    };

    this.orgCheckChange = this.orgCheckChange.bind(this);
    this.menuCheckChange = this.menuCheckChange.bind(this);
    this.onOk = this.onOk.bind(this);
  }

  orgCheckChange(id, checked) {
    const { checkedOrgIds } = this.state;
    let newCheckedOrgIds;

    if (checked) {
      newCheckedOrgIds = [...checkedOrgIds, id];
    } else {
      newCheckedOrgIds = checkedOrgIds.filter((c) => c !== id);
    }

    this.setState({ checkedOrgIds: newCheckedOrgIds });
  }

  menuCheckChange(id, checked) {
    const { checkedMenuIds } = this.state;
    let newCheckedMenuIds;

    if (checked) {
      newCheckedMenuIds = [...checkedMenuIds, id];
    } else {
      newCheckedMenuIds = checkedMenuIds.filter((c) => c !== id);
    }

    this.setState({ checkedMenuIds: newCheckedMenuIds });
  }

  onOk() {
    const { onSaveSuccess, ids, orgId, canSelectOrg, isTopUser } = this.props;
    const { checkedOrgIds, checkedMenuIds } = this.state;
    let error;

    if (checkedMenuIds.length === 0) {
      error = '请选择菜单';
    } else if (canSelectOrg && checkedOrgIds.length === 0) {
      error = '请选择组织机构';
    } else {
      if (checkedOrgIds.some((c) => c === TOP_ORG_ID) && checkedOrgIds.some((c) => c !== TOP_ORG_ID)) {
        error = '集团或分公司只能选一种类型';
      }
    }

    if (error) {
      this.modal.showOkTips(error);
      return;
    }

    let newType;
    // 如果是咨询师角色，则肯定为个人类型
    if (canSelectOrg) {
      newType = checkedOrgIds[0] === TOP_ORG_ID ? BELONG_TYPE_TOP : BELONG_TYPE_BRANCH;
    } else {
      newType = isTopUser ? BELONG_TYPE_TOP : BELONG_TYPE_BRANCH;
    }

    const params = {
      contentId: ids,
      newBrenchId: canSelectOrg ? checkedOrgIds : [orgId],
      newMenuId: checkedMenuIds,
      newType,
    };
    this.setState({ loading: true });
    copyContent(params, true)
      .then(() => {
        showMessage('保存成功', 'success');
        onSaveSuccess();
      })
      .catch((err) => {
        showMessage(err.text);
        this.setState({ loading: false });
      });
  }

  render() {
    const { canSelectOrg, orgDict, onClose, menuTree, isTopUser } = this.props;
    const { loading, checkedOrgIds, checkedMenuIds } = this.state;

    return (
      <Modal
        className={styles.modalWrap}
        loading={loading}
        title="复制图文"
        width={canSelectOrg ? 600 : 374}
        onOk={this.onOk}
        onCancel={onClose}
        noLine
        ref={(c) => (this.modal = getWrappedInstance(c))}
      >
        <div className={styles.wrap + (canSelectOrg ? ' ' + styles.two : '')}>
          {canSelectOrg && (
            <div className={styles.left}>
              {orgDict.map((p, i) => {
                return (
                  <div key={p.val} className={styles.pItem}>
                    <Checkbox
                      checked={checkedOrgIds.indexOf(p.val) >= 0}
                      onChange={(e) => this.orgCheckChange(p.val, e.target.checked)}
                    >
                      {p.text}
                    </Checkbox>
                  </div>
                );
              })}
            </div>
          )}
          <div className={styles.right}>
            {getMenuDict(
              menuTree.filter((menu) => {
                //集团过滤掉个性化资料菜单
                if (isTopUser) {
                  return menu.title !== PERSONALINFO;
                } else {
                  return true;
                }
              }),
            ).map((c, i) => {
              const cls = {
                [styles.cItem]: 1,
                [styles['cItemLevel' + c.level]]: !!c.level,
              };

              return (
                <div key={c.val} className={classnames(cls)}>
                  <Checkbox
                    checked={checkedMenuIds.indexOf(c.val) >= 0}
                    onChange={(e) => this.menuCheckChange(c.val, e.target.checked)}
                  >
                    {c.text}
                  </Checkbox>
                </div>
              );
            })}
          </div>
        </div>
      </Modal>
    );
  }
}

export default CopyModal;
