/**
 * 历史记录模态框
 */
import React from 'react';
import Modal from 'COMPONENT/Modal';
import styles from './styles.less';

class HistoryModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 等待状态
      data: [], // {object[]} 历史记录数据
    };
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化内容
   */
  initContent = () => {
    this.setState({ loading: true });
    Promise.all([this.loadData()]).then(([data]) => {
      this.setState({
        loading: false,
        data,
      });
    });
  };

  loadData = async () => {
    await new Promise((resolve) => {
      setTimeout(() => {
        resolve();
      }, 500);
    });
    const fakeData = [
      {
        year: '2018',
        records: [
          {
            month: '03',
            day: '11',
            detail: '修改可见范围，东莞分公司、佛山分公司、东莞分公司、佛山分公司、江门分公司可见',
            userName: '李晶晶',
          },
          { month: '02', day: '23', detail: '修改图文', userName: '黄管理' },
          {
            month: '02',
            day: '01',
            detail: '修改可见范围，东莞分公司、佛山分公司、江门分公司可见',
            userName: '李晶晶',
          },
        ],
      },
      {
        year: '2017',
        records: [
          {
            month: '11',
            day: '22',
            detail: '修改可见范围，东莞分公司、佛山分公司、江门分公司可见',
            userName: '李晶晶',
          },
          { month: '02', day: '23', detail: '修改图文', userName: '黄管理' },
        ],
      },
    ];
    return fakeData;
  };

  getYearBlock = (data) => {
    const { year, records } = data;
    return (
      <div key={year} className={styles.yearBlock}>
        <div className={styles.yearBlockTitle}>{year}</div>
        <div>{records.map(this.getRecordBlock)}</div>
      </div>
    );
  };

  getRecordBlock = (record, idx) => {
    const { month, day, detail, userName } = record;
    return (
      <div key={idx} className={styles.recordBlock}>
        <div className={styles.recordBlockLeft}>
          <div className={styles.day}>{day}</div>
          <div className={styles.month}>{`${month}月`}</div>
        </div>
        <div className={styles.recordBlockRight}>
          <div className={styles.detail}>{detail}</div>
          <div className={styles.userName}>{`操作人：${userName}`}</div>
        </div>
      </div>
    );
  };

  render() {
    const { onClose } = this.props;
    const { loading, data } = this.state;

    return (
      <Modal
        className={styles.modalWrap}
        loading={loading}
        title="状态记录"
        width={550}
        okBtnShow={false}
        cancelText="关闭"
        onCancel={onClose}
        noLine
        ref={(c) => (this.modal = c)}
      >
        <div className={styles.wrap}>{data.map(this.getYearBlock)}</div>
      </Modal>
    );
  }
}

export default HistoryModal;
