import React from 'react';
import Modal from 'COMPONENT/Modal';
import Checkbox from 'COMPONENT/Checkbox';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import { getMenuDict, PERSONALINFO } from '../../../common';
import { updateMenuId } from 'SERVICE/ipadBackEnd';
import styles from '../CopyModal/styles.less';
import classnames from 'classnames';

class TransformModal extends React.Component {
  static defaultProps = {
    menuItem: null, // 左侧选中的菜单
    ids: [], // 选中的ID
    onClose: () => {},
  };

  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      checkedOrgIds: [], // 选中的机构ID集合
      checkedMenuIds: [], // 选中的菜单ID集合
    };

    this.modal = null;

    this.formatMenuList = this.formatMenuList.bind(this);
    this.onSubmit = this.onSubmit.bind(this);
    this.onClose = this.onClose.bind(this);
  }

  /**
   * 筛选目录, 仅开放在个性化资料模块
   */
  formatMenuList() {
    const { menuTree } = this.props;

    const menu = menuTree.filter((menu) => {
      return menu.title === PERSONALINFO;
    });

    return getMenuDict(menu);
  }

  /**
   * 选中目录，单选并且只能选二级目录
   * @param id
   * @param checked
   */
  onMenuChecked(id, checked) {
    const { checkedMenuIds } = this.state;
    let newCheckedMenuIds;

    if (checked) {
      if (checkedMenuIds.length > 0) {
        showMessage('只能选择一个目录!', 'error');
        return false;
      }

      newCheckedMenuIds = [...checkedMenuIds, id];
    } else {
      newCheckedMenuIds = checkedMenuIds.filter((val) => val !== id);
    }

    this.setState({ checkedMenuIds: newCheckedMenuIds });
  }

  /**
   * 提交
   */
  onSubmit() {
    if (this.state.loading) {
      return false;
    }

    const { ids, onSaveSuccess } = this.props;
    const { checkedMenuIds } = this.state;

    if (!checkedMenuIds || checkedMenuIds.length <= 0) {
      showMessage('请选择目录', 'error');
      return false;
    }

    const param = {
      cmsContenIds: ids,
      menuId: checkedMenuIds[0],
    };

    this.setState({ loading: true });
    updateMenuId(param, true)
      .then(() => {
        showMessage('保存成功', 'success');
        onSaveSuccess();
      })
      .catch((e) => {
        this.setState({ loading: false });
        showMessage(e.text || '保存失败，请重试', 'error');
      });
  }

  /**
   * 关闭弹窗
   */
  onClose() {
    this.props.onClose();
  }

  render() {
    const { menuItem } = this.props;
    const { loading, checkedMenuIds } = this.state;
    const menu = this.formatMenuList();

    return (
      <Modal
        className={styles.modalWrap}
        loading={loading}
        title="转移图文"
        width={400}
        onOk={this.onSubmit}
        onCancel={this.onClose}
        noLine
        ref={(ref) => (this.modal = getWrappedInstance(ref))}
      >
        <section className={styles.wrap}>
          <div className={styles.right}>
            {menu.map((item, index) => {
              const cls = {
                [styles.cItem]: 1,
                [styles['cItemLevel' + item.level]]: !!item.level,
              };
              const isShow = !menuItem || menuItem.id !== item.val;

              return (
                isShow && (
                  <div key={index} className={classnames(cls)}>
                    <Checkbox
                      disabled={!item.level}
                      checked={checkedMenuIds.indexOf(item.val) >= 0}
                      onChange={(e) => this.onMenuChecked(item.val, e.target.checked)}
                    >
                      {item.text}
                    </Checkbox>
                  </div>
                )
              );
            })}
          </div>
        </section>
      </Modal>
    );
  }
}

export default TransformModal;
