import React from 'react';
import ReactDOM from 'react-dom';
import classnames from 'classnames';
import { Switch, Tooltip } from 'antd';
import { connect } from 'dva';
import styles from './styles.less';
import DropdownMenu from 'COMPONENT/DropdownMenu';
import Icon from 'COMPONENT/Icon';
import TextLink from 'COMPONENT/TextLink';
import styleVars from 'COMPONENT/theme/variables';
import {
  publishStatusDict,
  STATUS_UNPUBLISH,
  STATUS_PUBLISHED,
  STATUS_TOPUBLISH,
  MAX_CAROUSEL_COUNT,
  MAX_CAROUSEL_TIPS,
} from '../../common';
import { getDictTextByVal } from 'UTIL/dict';
import PublishPopConfirm from './PublishPopConfirm';
import Checkbox from 'COMPONENT/Checkbox';
import PropTypes from 'prop-types';
import CopyModal from './CopyModal';
import TransformModal from './TransformModal';
import showAlert from 'COMPONENT/utils/globalTips/showAlert';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import defImgSrc from '../def.png';
import {
  publishContent,
  publishFrontContent,
  deleteContent,
  pushContentToFront,
  cacelContentToFront,
} from 'SERVICE/ipadBackEnd';

const dropdownCfg = {
  color: styleVars.fontColor,
  activeColor: styleVars.primaryColor,
  disabledColor: styleVars.fontColorTips,
};

class ArticleCard extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      transformModalVis: false, // 转移弹窗
      copyModalVis: false, // 复制弹窗
      imgLoaded: false,
    };

    this.onImgLoaded = this.onImgLoaded.bind(this);
    this.switchPulish = this.switchPulish.bind(this);
    this.delete = this.delete.bind(this);
    this.recommend = this.recommend.bind(this);
    this.cancelRecommend = this.cancelRecommend.bind(this);
  }

  componentDidMount() {
    ReactDOM.findDOMNode(this.imgRef).addEventListener('load', this.onImgLoaded, false);
  }

  componentWillUnmount() {
    ReactDOM.findDOMNode(this.imgRef).removeEventListener('load', this.onImgLoaded, false);
  }

  onImgLoaded() {
    this.setState({ imgLoaded: true });
  }

  switchPulish(type, date) {
    const { onLoadingStart, onLoadingEnd, data, onReload, columnType } = this.props;
    const { id } = data;
    if (columnType === 'carousel') {
      const params = {
        appointmentPublishTime: date ? `${date} 00:00:00` : null,
        id,
        status: type === 'cancel' ? STATUS_UNPUBLISH : date ? STATUS_TOPUBLISH : STATUS_PUBLISHED,
      };
      onLoadingStart();
      publishFrontContent(params, true).then(
        () => {
          onReload();
          onLoadingEnd();
        },
        (e) => {
          showMessage(e.text || '操作失败');
          onLoadingEnd();
        },
      );
    } else {
      const params = [
        {
          appointmentPublishTime: date ? `${date} 00:00:00` : null,
          id,
          status: type === 'cancel' ? STATUS_UNPUBLISH : date ? STATUS_TOPUBLISH : STATUS_PUBLISHED,
        },
      ];
      onLoadingStart();
      publishContent(params, true).then(
        () => {
          onReload();
          onLoadingEnd();
        },
        (e) => {
          showMessage(e.text || '操作失败');
          onLoadingEnd();
        },
      );
    }
  }

  delete() {
    const { onLoadingStart, onLoadingEnd, data, onReload } = this.props;
    showConfirm(`确定删除 ${data.title} 吗？`, () => {
      onLoadingStart();
      deleteContent({ id: data.id }, true).then(
        () => {
          showMessage('删除成功', 'success');
          onReload();
          onLoadingEnd();
        },
        (e) => {
          showMessage(e.text || '删除失败');
          onLoadingEnd();
        },
      );
    });
  }

  recommend() {
    const {
      onLoadingStart,
      onLoadingEnd,
      onReload,
      data: { id },
    } = this.props;
    onLoadingStart();
    pushContentToFront({ id }, true).then(
      () => {
        onReload();
        onLoadingEnd();
      },
      (e) => {
        showMessage(e.text || '操作失败');
        onLoadingEnd();
      },
    );
  }

  cancelRecommend() {
    const {
      onLoadingStart,
      onLoadingEnd,
      onReload,
      data: { id },
    } = this.props;
    onLoadingStart();
    cacelContentToFront({ id }).then(() => {
      onReload();
      onLoadingEnd();
    });
  }

  render() {
    const {
      data,
      checked,
      onCheckedChange,
      orgDict,
      menuTree,
      menuItem,
      selectedMenu,
      isTopUser,
      orgId,
      onReload,
      onEdit,
      type,
      columnType,
      publishCount,
      roleCode,
    } = this.props;
    const { copyModalVis, transformModalVis, imgLoaded } = this.state;
    const {
      id,
      title,
      publishTime,
      isAppointment,
      isRecommend,
      imgSrc,
      orgTypeName,
      readCount,
      isBelongTop,
      publishStatus,
      appointmentPublishTime,
      publishUserName,
    } = data;

    const isPersonalMenu = selectedMenu && !!selectedMenu.isPersonalMenu;

    const moreMenus = [
      { text: '转移图文', onClick: () => this.setState({ transformModalVis: true }), disabled: !isPersonalMenu },
      { text: '复制图文', onClick: () => this.setState({ copyModalVis: true }) },
      { text: '删除', onClick: this.delete },
    ];
    const publishFull = columnType === 'carousel' && publishCount >= MAX_CAROUSEL_COUNT;
    const switchPublishType = publishStatus === STATUS_UNPUBLISH ? null : 'cancel';

    let switchBtn;
    const switchChecked = publishStatus !== STATUS_UNPUBLISH;
    if (publishFull && !switchChecked) {
      switchBtn = <Switch size="small" checked={switchChecked} onClick={() => showAlert(MAX_CAROUSEL_TIPS)} />;
    } else {
      switchBtn = (
        <PublishPopConfirm
          type={switchPublishType}
          onOk={(date) => this.switchPulish(switchPublishType, date)}
          getPopupContainer={() => this.$wrap}
        >
          <span>
            <Switch size="small" checked={switchChecked} />
          </span>
        </PublishPopConfirm>
      );
    }
    const cls = {
      [styles.wrap]: 1,
      [styles[`wrap-${type}`]]: type,
    };

    return (
      <section className={classnames(cls)} ref={(c) => (this.$wrap = ReactDOM.findDOMNode(c))}>
        {type === 'batch' && (
          <div className={styles.checkbox}>
            <Checkbox checked={checked} onChange={(e) => onCheckedChange(e.target.checked)} />
          </div>
        )}
        <div className={styles.header}>
          <Tooltip placement="top" title={title}>
            <div className={styles.headerLeft}>{title}</div>
          </Tooltip>

          <div className={styles.headerRight}>
            <DropdownMenu
              menus={moreMenus}
              iconType="more"
              {...dropdownCfg}
              placement="bottomRight"
              getPopupContainer={() => this.$wrap}
            />
          </div>
        </div>
        <div className={styles.topBar}>
          {(publishTime || appointmentPublishTime) && (
            <div className={styles.topBarLeft}>
              <div>
                <Icon type="time_mc" className={styles.icon} />
                {isAppointment
                  ? appointmentPublishTime && appointmentPublishTime.slice(0, 10)
                  : publishTime && publishTime.slice(0, 10)}
                {isAppointment && '（预约发布）'}
              </div>
              <div style={{ marginLeft: '16px' }}>{publishUserName}</div>
            </div>
          )}
          <div className={styles.topBarRight}>
            <span style={{ display: 'inline-block', marginRight: 8 }}>
              {getDictTextByVal(publishStatusDict, publishStatus)}
            </span>
            {switchBtn}
          </div>
        </div>
        <div className={styles.imgWrap}>
          <div className={styles.imgWrapInner}>
            <img src={defImgSrc} ref={(c) => (this.defImgRef = c)} className={imgLoaded ? styles.defImgHide : null} />
            <img
              src={imgSrc}
              className={imgLoaded ? styles.faceImg : styles.faceImgHide}
              ref={(c) => (this.imgRef = c)}
            />
          </div>
          {isRecommend ? <div className={styles.recommendTag}>首页推荐</div> : null}
        </div>
        <div className={styles.bottomBar}>
          <div className={styles.bottomBarLeft}>
            <Icon type="city_mc" className={styles.icon} />
            {orgTypeName}
          </div>
          <div className={styles.bottomBarRight}>
            <Icon type="book_mc" className={styles.icon} />
            {readCount || 0}
          </div>
        </div>
        <div className={styles.footer}>
          <div className={styles.btns}>
            <div className={styles.btn}>
              <TextLink type="secondary" onClick={onEdit}>
                编辑
              </TextLink>
            </div>
          </div>
        </div>

        {copyModalVis && (
          <CopyModal
            canSelectOrg={!!(isTopUser && !isBelongTop)}
            orgDict={orgDict}
            menuTree={menuTree}
            onSaveSuccess={() => {
              this.setState({ copyModalVis: false });
              onReload();
            }}
            onClose={() => this.setState({ copyModalVis: false })}
            ids={[id]}
            orgId={orgId}
            isTopUser={isTopUser}
            roleCode={roleCode}
          />
        )}

        {transformModalVis && (
          <TransformModal
            menuTree={menuTree}
            menuItem={menuItem}
            ids={[id]}
            onClose={() => this.setState({ transformModalVis: false })}
            onSaveSuccess={() => {
              this.setState({ copyModalVis: false });
              onReload();
            }}
          />
        )}
      </section>
    );
  }
}

ArticleCard.contextTypes = {
  isNowPersonal: PropTypes.func,
};

const mapStateToProps = (state) => {
  return {
    roleCode: state.account.user.roleCode,
    userId: state.account.user.userId,
  };
};

export default connect(mapStateToProps)(ArticleCard);
