import React from 'react';
import { Row, Col } from 'antd';
import ArticleCard from '../../ArticleCard';
import Button from 'COMPONENT/Button';
import styles from './styles.less';
import { saveCmsContentOrder } from 'SERVICE/ipadBackEnd';
import SortList from '../../SortList';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import commonStyles from 'PAGE/ipadBackEnd/articleEdit/commonStyles.less';

const columnType = 'indexArticle';

class OneList extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      sortShow: false,
      imagesIcon: 'images_mc',
      linkIcon: 'link_mc',
      documentsIcon: 'documents_mc',
      list: [],
    };
  }

  onSortSuccess = () => {
    this.setState({ sortShow: false });
    this.props.onReload();
  };

  onSortCancel = () => {
    this.setState({ sortShow: false, list: [] });
  };

  ok = () => {
    const { onLoadingStart, onLoadingEnd, onSaveSuccess } = this.props;
    const { list } = this.state;
    const params = list.map((d, i) => {
      return { id: d.id, orderNum: i };
    });
    onLoadingStart();
    saveCmsContentOrder(params).then(
      () => {
        onLoadingEnd();
        showMessage('操作成功', 'success');
        onSaveSuccess();
        this.setState({ sortShow: false, list: [] });
      },
      (e) => {
        onLoadingEnd();
        showMessage(e.text || '操作失败');
      },
    );
  };

  startSort = () => {
    this.setState({ sortShow: true, list: this.props.data.articles });
  };

  render() {
    const {
      onLoadingStart,
      onLoadingEnd,
      orgDict,
      menuTree,
      isTopUser,
      isTopOrg,
      orgId,
      data,
      onReload,
      colCount,
      mode,
      onEditArticle,
      menuItem,
    } = this.props;
    const { sortShow, list } = this.state;
    const listForSort = list.filter((li) => !li.isTopArticleInBranch);
    const noData = listForSort.length === 0;
    let sortContent;
    if (noData) {
      sortContent = <div className={commonStyles.noDataTips}>没有可排序操作的图文!</div>;
    } else {
      sortContent = (
        <SortList
          isTopUser={isTopUser}
          list={listForSort}
          onChange={(list) => this.setState({ list })}
          columnType={columnType}
          colCount={colCount}
        />
      );
    }
    let toolbar;
    if (sortShow) {
      toolbar = (
        <span>
          <Button type="primary" onClick={this.ok} disabled={noData}>
            确定
          </Button>
          <Button onClick={() => this.setState({ sortShow: false })}>取消</Button>
        </span>
      );
    } else {
      toolbar = <Button onClick={this.startSort}>顺序管理</Button>;
    }
    return (
      <div className={styles.wrap}>
        <div className={styles.header}>
          <div className={styles.title}>{data.menuName}</div>
          <div className={styles.rightTools}>{isTopOrg && toolbar}</div>
        </div>
        <div style={{ display: !sortShow ? 'block' : 'none' }}>
          <Row style={{ margin: '-10px -10px 0 -10px' }}>
            {data.articles &&
              data.articles.map((article) => {
                return (
                  <Col key={article.id} span={24 / colCount} style={{ padding: '10px 10px' }}>
                    <ArticleCard
                      data={article}
                      isTopUser={isTopUser}
                      isTopOrg={isTopOrg}
                      orgDict={orgDict}
                      menuTree={menuTree}
                      orgId={orgId}
                      onLoadingStart={onLoadingStart}
                      onLoadingEnd={onLoadingEnd}
                      onReload={onReload}
                      mode={mode}
                      columnType={columnType}
                      onEdit={() => onEditArticle({ id: article.id, columnType })}
                      menuItem={menuItem}
                    />
                  </Col>
                );
              })}
          </Row>
        </div>
        {sortShow && sortContent}
      </div>
    );
  }
}

export default OneList;
