import React from 'react';
import styles from './index.less';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { getSseKey } from 'SERVICE/common';
import { uuid } from 'COMPONENT/utils/basic/str';
import { Icon, Progress } from 'antd';

export default class SelfUpload extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      list: [],
    };
  }

  componentDidMount() {
    let { srcProps } = this.props;
    if (srcProps) {
      this.setState({ uploaded: true });
    }
  }

  componentWillReceiveProps(nextProps) {
    if (nextProps.srcProps === '' && nextProps.clear === true) {
      this.setState({ src: '', uploaded: false });
    }
  }

  triggerInput = () => {
    this.$input.click();
  };

  limitValidate(data, file) {
    let { size, type } = data;
    if (size && !(file.size / 1024 < size)) {
      showMessage(`文件大小超过${size}k`);
      return false;
    }
    if (type) {
      let limitType;
      type.split('/').forEach((v, k) => {
        if (file.type.includes(v)) {
          limitType = true;
        }
      });
      if (!limitType) {
        showMessage(`只能上传${type}文件`);
        return false;
      }
    }
    return true;
  }

  onChange = async (e) => {
    let list = [];
    let { beforeUpload } = this.props;
    let fileList = [...e.target.files];

    if (typeof beforeUpload === 'function') {
      let res = beforeUpload(fileList);
      if (res === false) {
        return false;
      } else if (typeof res === 'object') {
        res
          .then(async (resolve) => {
            if (!this.checkLimit(fileList)) {
              return false;
            }
            this.getKeyAndMapRequest(fileList, list);
          })
          .catch((reject) => {
            return false;
          });
      } else {
        if (!this.checkLimit(fileList)) {
          return false;
        }
        this.getKeyAndMapRequest(fileList, list);
      }
    } else {
      if (!this.checkLimit(fileList)) {
        return false;
      }
      this.getKeyAndMapRequest(fileList, list);
    }
  };

  checkLimit = (fileList) => {
    let { limit } = this.props;
    if (limit) {
      for (let item of fileList) {
        if (!this.limitValidate(limit, item)) {
          return false;
        }
      }
    }
    return true;
  };
  getKeyAndMapRequest = async (fileList, list) => {
    let configKey = await getSseKey();
    //  循环  uploadFile
    for (let file of fileList) {
      this.uploadFile(file, configKey, list);
    }
  };

  uploadFile = (file, configKey, list) => {
    let { upDateSrc } = this.props;
    let realname = /\.(?:png|jpg|jpeg)$/gi.test(file.name) ? file.name : file.name + '.' + file.type;
    let fileKey = uuid();
    let formData = new FormData();
    formData.append('x:realname', realname);
    formData.append('x:uploadtype', 'ipad');
    formData.append('x:uploadid', uuid());
    formData.append('key', fileKey);
    Object.keys(configKey).forEach((key, index) => {
      if (key != 'host') formData.append(key, configKey[key]);
    });
    formData.append('file', file);
    // let info = {};
    let xhr = new XMLHttpRequest();
    xhr.open('POST', configKey.host, true);
    xhr.upload.onloadstart = (file) => {
      // info = {...info, fileKey, showProgress: true, percent: 0};
      // this.update(info, list);

      this.setState({ showProgress: true, percent: 0 });
    };
    xhr.upload.onprogress = (file) => {
      // info = {...info, percent: file.loaded * 100 / file.total};
      // this.update(info, list);
      this.setState({ percent: (file.loaded * 100) / file.total });
    };
    xhr.onload = () => {
      // info = {...info, src: configKey.host + fileKey, showProgress: false};
      // this.update(info, list);
      upDateSrc(configKey.host + fileKey);
      this.setState({ src: configKey.host + fileKey, showProgress: false });
    };
    xhr.onerror = () => {
      // info['error'] = true;
      // this.update(info, list);
    };
    xhr.send(formData);
  };

  render() {
    let { multiple = false, srcProps } = this.props;
    let { showProgress, percent, src } = this.state;
    srcProps = src ? src : srcProps;
    let uploadButton = (
      <div className={styles.wrap}>
        {srcProps ? (
          <div className={styles.hasImages}>
            <img src={srcProps} alt="" />
            <span className={styles.mask}></span>
            <Icon type="upload" style={{ fontSize: 28 }} />
          </div>
        ) : (
          <div className={styles.noImages}>
            {showProgress ? <div></div> : <Icon type="plus" style={{ fontSize: 28 }} />}
          </div>
        )}
        {showProgress && (
          <Progress percent={percent} size="small" strokeWidth={5} showInfo={false} className={'progress'} />
        )}
      </div>
    );
    return (
      <div onClick={this.triggerInput}>
        <input
          type="file"
          multiple={multiple}
          ref={(e) => (this.$input = e)}
          onChange={this.onChange}
          className={styles.upload}
        />
        {uploadButton}
      </div>
    );
  }
}
