import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import CardList from 'COMPONENT/List/CardList';
import Button from 'COMPONENT/Button';
import Input from 'COMPONENT/Input';
import IconButton from 'COMPONENT/IconButton';
import MultiSelect from 'COMPONENT/MultiSelect';
import PopConfirm from 'COMPONENT/PopConfirm';
import TextLink from 'COMPONENT/TextLink';
import AddMenu from './viewModal/firstAddMenu';
import AddSecondMenu from './viewModal/secondAddMenu';
import IconManage from './viewModal/iconManage';
import Cell from './viewModal/cell';
import { Row, Col } from 'antd';
import styles from './index.less';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import {
  getMenuList,
  getCompanyList,
  changeMenuList,
  changeVisibleRange,
  deleteMenu,
  changeMenuOrder,
} from 'SERVICE/ipadFrameManage';

const [COL_WIDTH_S, COL_WIDTH_L, COL_WIDTH_XS] = [150, 300, 100];

class FrameManage extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      listLoading: false,
      addMenu: null,
      addSecondMenu: null,
      expandedRowKey: null,
      iconManage: null,
      editIndex: null,
      subEditIndex: null,
      list: [],
      subData: [],
      companyDict: [],
      allCanSee: null,
      subCompanyDict: [],
    };
  }

  componentDidMount() {
    //获取集团和分公司列表
    getCompanyList().then((res) => {
      let companyDict = [];
      if (res.length) {
        res.forEach((v, k) => {
          companyDict.push({ text: v.name, val: v.id });
        });
      }
      this.setState({ companyDict });
    });
    this.loadData();
  }

  /**
   * 加载数据
   * @param level {number} 一级目录和二级目录
   * @param parentId {number} 目录id
   * @returns {Promise.<void>}
   */
  loadData = async (level, parentId) => {
    let { brenchId } = this.props;
    if (brenchId === '000001') {
      brenchId = '';
    }
    let apiParams = {
      level: level || 1,
      name: null,
      orgId: brenchId,
      parentId: parentId || null,
      useType: 'VISITOR',
    };
    this.setState({ listLoading: true });
    let data = await getMenuList(apiParams);
    let list = data.map((v, k) => {
      return {
        id: v.id,
        allCanSee: v.allCanSee,
        auths: v.auths,
        createTime: v.createTime,
        createUser: v.createUser,
        grayIco: v.grayIco,
        ico: v.ico,
        level: v.level,
        modifyTime: v.modifyTime,
        modifyUser: v.modifyUser,
        name: v.name,
        orderNum: v.orderNum,
        orgId: v.orgId,
        orgName: v.orgName,
        parentId: v.parentId,
        showContentNum: v.showContentNum,
        status: v.status,
        useType: v.useType,
      };
    });
    let backupData = JSON.parse(JSON.stringify(list));
    if (level === 2) {
      this.setState({ listLoading: false, backupSubData: backupData, subData: list || [] });
      return list;
    } else {
      this.setState({ list, backupList: backupData, listLoading: false });
    }
  };

  /**
   *
   * @param expandedRow
   * 加载二级目录数据
   */
  onExpandedRowChange = async (expandedRow) => {
    let { list } = this.state;
    if (typeof expandedRow === 'number') {
      let { id } = list[expandedRow];
      (await this.loadData(2, id)) || [];
    }
    // this.setState({expandedRowKey: expandedRow, editIndex: null, subData, backupSubData});
    this.setState({ expandedRowKey: expandedRow, editIndex: null });
  };

  /**
   * 表格展开
   * @param record {object}
   */
  expandedRowRender = (record) => {
    let { subEditIndex, companyDict, subData, orderManage, subCompanyDict, expandedRowKey } = this.state;
    let canHandle = this.props.brenchId === '000001';
    subData.forEach((v, k) => {
      let { orgId, allCanSee } = v;

      if (orgId && typeof orgId === 'string') {
        v.orgId = orgId.split(';');
      } else if (orgId === null && allCanSee === 0) {
        orgId = [];
        companyDict.forEach((v, k) => {
          orgId.push(v.val);
        });
        v.orgId = orgId;
      } else if (orgId === '') {
        v.orgId = [];
      }
      v.truthName = v.orgName;
    });
    let copyCompanyDict = companyDict.filter((v, k) => {
      return subCompanyDict[expandedRowKey].includes(v.val);
    });
    return (
      <div>
        {subData.map((v, k) => {
          return (
            <Row gutter={0} key={k} style={{ display: 'flex', alignItems: 'center' }}>
              <Col span={2} className={styles.pad1215 + (subEditIndex === k ? ' editHeight' : '')}></Col>
              <Col span={6} className={styles.pad1215 + (subEditIndex === k ? ' editHeight' : '')}>
                {subEditIndex === k && (
                  <div
                    className={styles.count}
                    data-length={((v.name.length || 0) > 6 ? 6 : v.name.length || 0) + '/6'}
                  >
                    <Input
                      value={v.name}
                      onChange={(e) => this.subChangeData(e.target.value, 'name', k)}
                      maxLength="6"
                    />
                  </div>
                )}
                {subEditIndex !== k && <span>{v.name}</span>}
              </Col>
              <Col span={12} className={styles.pad1215 + (subEditIndex === k ? ' editHeight' : '')}>
                {subEditIndex === k && (
                  <MultiSelect
                    dict={copyCompanyDict}
                    value={v.orgId}
                    onChange={(val) => this.subChangeData(val, 'orgId', k)}
                  />
                )}
                {subEditIndex !== k && <Cell text={v.truthName} />}
              </Col>
              <Col span={4} className={styles.pad1215 + (subEditIndex === k ? ' editHeight' : '')}>
                {orderManage && k === 0 && (
                  <div>
                    <span className={styles.mr50 + ' line'}></span>
                    <IconButton type="move_down_mc" onClick={(e) => this.gradeChange(k, 'down', 2)} />
                  </div>
                )}
                {orderManage && k > 0 && k < subData.length - 1 && (
                  <div>
                    <IconButton
                      className={styles.mr50}
                      type="move_up_mc"
                      onClick={(e) => this.gradeChange(k, 'up', 2)}
                    />
                    <IconButton type="move_down_mc" onClick={(e) => this.gradeChange(k, 'down', 2)} />
                  </div>
                )}
                {orderManage && k === subData.length - 1 && subData.length > 1 && (
                  <div>
                    <IconButton
                      className={styles.mr50}
                      type="move_up_mc"
                      onClick={(e) => this.gradeChange(k, 'up', 2)}
                    />
                    <span className={'line'}></span>
                  </div>
                )}
                {subEditIndex !== k && !orderManage && canHandle && (
                  <div className="edit">
                    <TextLink className={styles.mr30} onClick={(e) => this.subEdit(k)}>
                      编辑
                    </TextLink>
                    <PopConfirm
                      content="确认删除该目录吗？"
                      okBtnText="是"
                      cancelBtnText="否"
                      onOk={(e) => this.deleteMenu(v.id)}
                    >
                      <TextLink>删除</TextLink>
                    </PopConfirm>
                  </div>
                )}
                {subEditIndex === k && !orderManage && (
                  <div>
                    <TextLink className={styles.mr30} onClick={(e) => this.subSaveData(k)}>
                      保存
                    </TextLink>
                    <TextLink onClick={(e) => this.subCancelEdit()}>取消</TextLink>
                  </div>
                )}
              </Col>
            </Row>
          );
        })}
      </div>
    );
  };
  /**
   * 点击二级目录编辑按钮
   * @param index {number} 当前所修改行的索引
   */
  subEdit = (index) => {
    let { subData, backupSubData } = this.state;
    let copyBackupSubData = JSON.parse(JSON.stringify(backupSubData));
    this.setState({ subData: copyBackupSubData });
    let copySubData = JSON.parse(JSON.stringify(subData));
    this.setState({ subEditIndex: index, copySubData });
  };
  /**
   * 取消编辑操作
   */
  subCancelEdit = () => {
    let { copySubData, backupSubData } = this.state;
    // this.setState({subData: copySubData, subEditIndex: null});
    this.setState({ subData: backupSubData, subEditIndex: null });
  };
  /**
   * 编辑二级目录
   * @param value
   * @param key {string} 修改的属性名 'name' | 'orgId'
   * @param index {number} 当前所修改行的索引
   */
  subChangeData = (value, key, index) => {
    let { subData } = this.state;
    subData[index][key] = key === 'name' ? value : value || [];
    this.setState({ subData });
  };
  subSaveData = (idx) => {
    this.saveData(idx, 2);
  };

  edit = (index) => {
    let { list, backupList } = this.state;
    //当编辑一个时 没有点击保存 又点击另外一个目录的编辑按钮  将原来的改动重置
    let copyBackupList = JSON.parse(JSON.stringify(backupList));
    this.setState({ list: copyBackupList });
    let copyList = JSON.parse(JSON.stringify(list));
    this.setState({ editIndex: index, copyList });
  };
  deleteMenu = (id) => {
    let apiParams = { id };
    deleteMenu(apiParams).then((res) => {
      showMessage('删除目录成功', 'success');
      let { editIndex, expandedRowKey, list } = this.state;
      if (editIndex) {
        this.loadData();
      } else {
        let parentId = list[expandedRowKey].id;
        this.loadData(2, parentId).then((res) => {});
      }
    });
  };
  cancelEdit = () => {
    let { backupList } = this.state;
    this.setState({ editIndex: null, list: backupList });
  };
  saveData = (idx, level) => {
    let { list, companyDict, subData } = this.state;
    let apiParams;
    let visibleParams = [];
    let saveParams = list[idx];
    let visibleRangeArray;
    if (level) {
      saveParams = subData[idx];
      delete saveParams['orgIdArray'];
      delete saveParams['truthName'];
      visibleRangeArray = subData[idx]['orgId'];
    } else {
      visibleRangeArray = list[idx]['orgId'];
    }
    let { id } = saveParams;
    if (!visibleRangeArray) {
      visibleRangeArray = [];
      companyDict.forEach((v, k) => {
        visibleRangeArray.push(v.val);
      });
    } else if (typeof visibleRangeArray === 'string') {
      visibleRangeArray = visibleRangeArray.split(';');
    }
    visibleRangeArray.forEach((v, k) => {
      visibleParams.push({ cmsMenuId: id, orgId: v });
    });
    apiParams = {
      ...saveParams,
      orgName: null,
      orgId: null,
      allCanSee: companyDict.length === visibleParams.length ? 0 : 1,
    };
    if (!apiParams.name) {
      showMessage('请输入目录名称！');
      return;
    }
    // if (visibleRangeArray.length === 0) {
    //   showMessage('请选择可见范围');
    //   return;
    // }
    this.setState({ listLoading: true });
    changeVisibleRange({ allCanSee: companyDict.length === visibleParams.length ? 0 : 1, auths: visibleParams, id });
    changeMenuList(apiParams)
      .then((res) => {
        showMessage('保存成功！', 'success');
        if (level) {
          let { expandedRowKey } = this.state;
          this.onExpandedRowChange(expandedRowKey);
          this.setState({ subEditIndex: null });
        } else {
          this.loadData();
          this.setState({ editIndex: null });
        }
      })
      .catch((res) => {
        this.setState({ listLoading: false });
      });
  };
  changeData = (value, key, index) => {
    let { list } = this.state;
    list[index][key] = key === 'name' ? value : value || [];
    this.setState({ list });
  };

  onExpand = () => {};
  /**
   * 添加一级目录
   */
  addMenu = () => {
    let { list } = this.state;
    if (list.length >= 6) {
      showMessage('最多只能创建6个一级目录！');
      return;
    }
    this.setState({ addMenu: true });
  };
  /**
   * 添加二级目录
   * @param parentId {number} 父级目录id
   * @param allCanSee {number} 是否全部可见 0 | 1;
   */
  addSecondMenu = (parentId, allCanSee) => {
    let { subData } = this.state;
    if (subData.length >= 8) {
      showMessage('最多只能创建8个二级目录！');
      return;
    }
    this.setState({ addSecondMenu: true, parentId, allCanSee });
  };
  /**
   * 添加二级目录成功后的刷新列表回调
   * @param id {number} 父目录id
   */
  secondMenuSuccess = async (id) => {
    let subData = await this.loadData(2, id);
    // let backupSubData = JSON.parse(JSON.stringify(subData));
    this.setState({ subData });
  };
  /**
   *保存目录顺序
   */
  saveOrder = () => {
    let { expandedRowKey, list, subData } = this.state;
    let apiParams = [];
    let data = expandedRowKey !== null ? subData : list;
    data.forEach((v, k) => {
      apiParams.push({ id: v.id, orderNum: k });
    });
    this.setState({ listLoading: true });
    changeMenuOrder(apiParams).then((res) => {
      showMessage('保存成功', 'success');
      if (expandedRowKey !== null) {
        this.onExpandedRowChange(expandedRowKey);
      } else {
        this.loadData();
      }
      this.setState({ orderManage: false, listLoading: false });
    });
  };

  /**
   * 调整目录顺序
   * @param idx {number} 当前行索引
   * @param handle {string} 'up' | 'down'
   * @param level {number | undefined} 目录层次 2 | undefined
   */
  gradeChange = (idx, handle, level) => {
    let data = level ? this.state.subData : this.state.list;
    let i = handle === 'down' ? idx + 1 : idx - 1;
    [data[idx], data[i]] = [data[i], data[idx]];
    if (level) {
      let $nodeList = document.querySelectorAll('.browsermode .ant-table-expanded-row .ant-row');
      $nodeList[idx].classList.add('subBg');
      setTimeout(() => {
        this.setState({ subData: data });
        $nodeList[idx].classList.remove('subBg');
        $nodeList[i].classList.add('subBg');
        setTimeout(() => {
          $nodeList[i].classList.remove('subBg');
        }, 600);
      }, 300);
    } else {
      let $nodeList = document.querySelectorAll('.browsermode .ant-table-tbody>.ant-table-row');
      let $tdList = $nodeList[idx].querySelectorAll('td');
      $tdList.forEach((v, k) => {
        v.classList.add('bg');
      });
      setTimeout(() => {
        this.setState({ list: data });
        $tdList.forEach((v, k) => {
          v.classList.remove('bg');
        });
        $nodeList[i].classList.add('bg');
        setTimeout(() => {
          $nodeList[i].classList.remove('bg');
        }, 600);
      }, 300);
    }
  };

  /**
   * 点击顺序管理按钮，备份一份数据，确保后续点击取消的时候找回原数据
   */
  orderManage = () => {
    let { expandedRowKey, list, subData } = this.state;
    let copyList, copySubData;
    copyList = JSON.parse(JSON.stringify(list));
    if (expandedRowKey !== null) {
      copySubData = JSON.parse(JSON.stringify(subData));
    }
    this.setState({ orderManage: true, copyList, copySubData });
  };
  /**
   * 点击取消按钮，撤销操作，找回原数据
   */
  cancelOrderManage = () => {
    let { copyList, copySubData } = this.state;
    this.setState({ orderManage: false, list: copyList, subData: copySubData });
  };

  render() {
    const {
      listLoading,
      expandedRowKey,
      list,
      companyDict,
      addMenu,
      addSecondMenu,
      orderManage,
      iconManage,
      parentId,
      subData,
      allCanSee,
      subCompanyDict,
    } = this.state;
    let canHandle = this.props.brenchId === '000001';
    let len = list.length;
    let sliceIdList = list.slice(1);
    //当只有一个目录时，让顺序管理按钮不可用
    let orderDisabled =
      expandedRowKey !== null ? (subData.length <= 1 ? true : false) : list.length <= 1 ? true : false;
    const columns = [
      {
        title: '菜单',
        dataIndex: 'name',
        width: COL_WIDTH_S,
        key: 'name',
        render: (text, record, idx) => {
          let { editIndex } = this.state;
          if (editIndex === idx) {
            return (
              <div className={styles.count} data-length={((text.length || 0) > 4 ? 4 : text.length || 0) + '/4'}>
                <Input value={text} onChange={(e) => this.changeData(e.target.value, 'name', idx)} maxLength="4" />
              </div>
            );
          } else {
            return <span>{text}</span>;
          }
        },
      },
      {
        title: '可见范围',
        dataIndex: 'orgName',
        width: COL_WIDTH_L,
        key: 'orgName',
        render: (text, record, idx) => {
          let { editIndex } = this.state;
          let { orgId, allCanSee } = record;
          if (orgId && typeof orgId === 'string') {
            orgId = orgId.split(';');
          } else if (orgId === null && allCanSee === 0) {
            orgId = [];
            companyDict.forEach((v, k) => {
              orgId.push(v.val);
            });
          } else if (orgId === '') {
            orgId = [];
          }
          subCompanyDict[idx] = orgId;
          if (editIndex === idx) {
            return (
              <MultiSelect dict={companyDict} value={orgId} onChange={(val) => this.changeData(val, 'orgId', idx)} />
            );
          } else {
            // if (text === '全部分公司') {
            //   text = '';
            //   companyDict.forEach((v, k) => {
            //     if (k === companyDict.length - 1) {
            //       text += v.text;
            //     } else {
            //       text += v.text + ';';
            //     }
            //
            //   })
            // }
            return <Cell text={text} />;
          }
        },
      },
      {
        title: '操作',
        dataIndex: 'handle',
        width: COL_WIDTH_XS,
        key: 'handle',
        render: (text, record, idx) => {
          let { editIndex } = this.state;
          let { parentId, id, allCanSee } = record;
          if (!orderManage) {
            if (expandedRowKey === null) {
              if (editIndex !== idx) {
                if (canHandle) {
                  return (
                    <div className={styles.handle}>
                      <TextLink onClick={(e) => this.edit(idx)}>编辑</TextLink>
                    </div>
                  );
                } else {
                  return <span></span>;
                }
              } else {
                return (
                  <div className={styles.handle}>
                    <TextLink onClick={(e) => this.saveData(idx)}>保存</TextLink>
                    <TextLink onClick={(e) => this.cancelEdit()}>取消</TextLink>
                  </div>
                );
              }
            } else if (expandedRowKey !== null && expandedRowKey === idx && canHandle) {
              // } else if (expandedRowKey !== null && expandedRowKey === idx) {
              return <TextLink onClick={(e) => this.addSecondMenu(id, allCanSee)}>添加二级目录</TextLink>;
            } else {
              return <span></span>;
            }
          } else {
            if (expandedRowKey === null && idx > 0) {
              if (idx === 1) {
                return (
                  <div>
                    <span className={styles.mr50 + ' line'}></span>
                    <IconButton type="move_down_mc" onClick={(e) => this.gradeChange(idx, 'down')} />
                  </div>
                );
              } else if (idx === len - 1) {
                return (
                  <div>
                    <IconButton
                      className={styles.mr50}
                      type="move_up_mc"
                      onClick={(e) => this.gradeChange(idx, 'up')}
                    />
                    <span className={'line'}></span>
                  </div>
                );
              } else {
                return (
                  <div>
                    <IconButton
                      className={styles.mr50}
                      type="move_up_mc"
                      onClick={(e) => this.gradeChange(idx, 'up')}
                    />
                    <IconButton type="move_down_mc" onClick={(e) => this.gradeChange(idx, 'down')} />
                  </div>
                );
              }
            }
          }
        },
      },
    ];
    return (
      <Page
        loading={false}
        route={this.props.route}
        className={styles.frameManage + ' ' + (!canHandle ? 'canhandle' : '') + ' browsermode'}
        fitWindowHeight
        subHeader={
          <div className={styles.wrap}>
            <div style={{ marginBottom: 15 }}>
              <Button onClick={this.addMenu} disabled={expandedRowKey !== null || !canHandle}>
                添加一级目录
              </Button>
              {/*// disabled={(expandedRowKey === null ? false : true)}>添加一级目录</Button>*/}
              {!orderManage && (
                <span>
                  <Button
                    type="primary"
                    className={styles.fr}
                    disabled={orderDisabled || !canHandle}
                    // disabled={orderDisabled}
                    onClick={(e) => this.orderManage()}
                  >
                    顺序管理
                  </Button>
                  <Button
                    type="primary"
                    className={styles.fr}
                    disabled={!canHandle}
                    onClick={(e) => this.setState({ iconManage: true })}
                  >
                    图标管理
                  </Button>
                </span>
              )}
              {orderManage && (
                <span>
                  <Button type="primary" className={styles.fr} onClick={(e) => this.cancelOrderManage()}>
                    取消
                  </Button>
                  <Button type="primary" className={styles.fr} onClick={(e) => this.saveOrder()}>
                    确认修改
                  </Button>
                </span>
              )}
            </div>
          </div>
        }
      >
        <CardList
          dataSource={list}
          columns={columns}
          loading={listLoading}
          expandedRowRender={(record) => this.expandedRowRender(record)}
          expandedRowKey={expandedRowKey}
          onExpandedRowChange={(expandedRows) => {
            this.onExpandedRowChange(expandedRows);
          }}
          detailIndent={false}
          onExpand={(expanded, record) => {
            this.onExpand(expanded, record);
          }}
        />
        {addMenu && <AddMenu onClose={(e) => this.setState({ addMenu: false })} success={this.loadData} />}
        {addSecondMenu && (
          <AddSecondMenu
            onClose={(e) => this.setState({ addSecondMenu: false })}
            success={(e) => this.secondMenuSuccess(parentId)}
            parentId={parentId}
            allCanSee={allCanSee}
          />
        )}
        {iconManage && (
          <IconManage
            onClose={(e) => this.setState({ iconManage: false })}
            menuIdList={list}
            sliceIdlist={sliceIdList}
          />
        )}
      </Page>
    );
  }
}

function mapStateToProps(state) {
  return {
    brenchId: state.account.user.brenchId,
  };
}

export default connect(mapStateToProps)(FrameManage);
