import React from 'react';
import Modal from 'COMPONENT/Modal';
import styles from './styles.less';
import FormItem from 'COMPONENT/FormItem';
import Input from 'COMPONENT/Input';
import Upload from '../Upload/index';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

import { changeMenuList } from 'SERVICE/ipadFrameManage';

class AddMenu extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      name: '',
      loading: false,
    };
  }

  onOk = async () => {
    let { name, srcNormal, srcSelect } = this.state;
    let { success, onClose, useType } = this.props;
    if (!name) {
      showMessage('请输入目录名称！');
      return;
    }
    if (!srcNormal) {
      showMessage('请上传正常状态图片！');
      return;
    }
    if (!srcSelect) {
      showMessage('请上传选中状态图片！');
      return;
    }
    let apiParams = {
      allCanSee: 0,
      createTime: null,
      createUser: null,
      grayIco: srcSelect,
      ico: srcNormal,
      id: null,
      level: 1,
      modifyTime: null,
      modifyUser: null,
      name,
      orderNum: null,
      parentId: null,
      showContentNum: 4,
      status: null,
      useType: useType ? useType : 'VISITOR',
    };
    this.setState({ loading: true });
    changeMenuList(apiParams)
      .then((res) => {
        showMessage('添加成功', 'success');
        this.setState({ loading: false });
        onClose();
        success();
      })
      .catch((res) => {
        this.setState({ loading: false });
      });
  };
  upDateSrc = (src, type) => {
    this.setState({ [type]: src });
  };
  onChange = (value) => {
    this.setState({ name: value });
  };

  render() {
    let { onClose } = this.props;
    let { name, loading } = this.state;
    return (
      <Modal
        loading={loading}
        onCancel={onClose}
        onOk={this.onOk}
        title="添加目录"
        width={452}
        okBtnShow={true}
        cancelText="取消"
        okText="保存"
        className={styles.modal}
      >
        <FormItem name={'目录名称'}>
          <div data-length={((name.length || 0) > 4 ? 4 : name.length || 0) + '/4'} className="count">
            <Input
              value={name}
              placeholder={'请输入'}
              maxLength={'4'}
              onChange={(e) => this.onChange(e.target.value)}
            />
          </div>
        </FormItem>
        <FormItem name={'上传图标'}>
          <div className={styles.tips}>只能上传jpg/ png文件，且不超过500kb</div>
          <div style={{ overflow: 'hidden' }} className={styles.clearfix}>
            <div style={{ float: 'left', textAlign: 'center', marginRight: 20 }}>
              <Upload ref={(e) => (this.uploadNormal = e)} upDateSrc={(src) => this.upDateSrc(src, 'srcNormal')} />
              <div className={styles.status}>正常状态</div>
            </div>
            <div style={{ float: 'left', textAlign: 'center' }}>
              <Upload ref={(e) => (this.uploadSelect = e)} upDateSrc={(src) => this.upDateSrc(src, 'srcSelect')} />
              <div className={styles.status}>选中状态</div>
            </div>
          </div>
        </FormItem>
      </Modal>
    );
  }
}

export default AddMenu;
