import React from 'react';
import Modal from 'COMPONENT/Modal';
import styles from './iconManage.less';

import Tabs, { Tab } from 'COMPONENT/Tabs';
import SecondIconManage from './secondIconManage';
import Item from './firstTabsItem';
import Preview from './preview';
import { getIconList, saveIconList, publishIconList } from 'SERVICE/ipadFrameManage';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

class IconManage extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      name: '',
      iconList: [],
      saveObject: {},
      loading: false,
      activeKey: 0,
      preview: false,
      tags: 0,
      previewList: null,
    };
  }

  async componentDidMount() {
    let data = await this.getIconList();
    this.setState({ iconList: data, loading: false });
  }

  getIconList = async () => {
    this.setState({ loading: true });
    let data = await getIconList({ menuLevel: 1, menuId: 0 });
    this.setState({ loading: false });
    return data;
  };
  onOk = () => {};

  activeChange = (activeKey) => {
    if (activeKey === 1) {
      this.setState({ activeKey, tags: 1 });
    } else {
      this.setState({ activeKey });
    }
  };

  updateIconList = async () => {
    let data = await this.getIconList();
    this.setState({ iconList: data });
  };
  /**
   * 一级目录图标预览
   * @param key
   */
  showPreview = (data) => {
    this.setState({ previewList: data, preview: true });
  };

  render() {
    let { iconList, loading, activeKey, preview, tags, previewList } = this.state;
    let { onClose, menuIdList } = this.props;
    // let idList=menuIdList.slice(1);
    return (
      <Modal
        loading={loading}
        onCancel={onClose}
        onOk={this.onOk}
        title="图标管理"
        width={960}
        okBtnShow={false}
        cancelBtnShow={false}
        closable={true}
        className={styles.modal}
      >
        <Tabs type="button" navAlign={'center'} activeKey={activeKey} onChange={this.activeChange}>
          <Tab title="一级图标">
            <Item
              data={iconList}
              showLoading={(e) => this.setState({ loading: true })}
              hideLoading={(e) => this.setState({ loading: false })}
              onSaveSuccess={this.updateIconList}
              showPreview={(data) => this.showPreview(data)}
            />
          </Tab>
          <Tab title="二级图标">
            <SecondIconManage
              showLoading={(e) => this.setState({ loading: true })}
              hideLoading={(e) => this.setState({ loading: false })}
              idList={this.props.sliceIdlist}
              // showPreview={data=>this.showSecondPreview(data)}
              tags={tags}
            />
          </Tab>
        </Tabs>

        {preview && <Preview previewList={previewList} onClose={(e) => this.setState({ preview: false })} />}
      </Modal>
    );
  }
}

export default IconManage;
