import React from 'react';
import Modal from 'COMPONENT/Modal';
import styles from './styles.less';
import FormItem from 'COMPONENT/FormItem';
import Input from 'COMPONENT/Input';
import Upload from '../Upload/index';

import { changeMenuList } from 'SERVICE/ipadFrameManage';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

class AddMenu extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      name: '',
      loading: false,
    };
  }

  onOk = async () => {
    let { name, src } = this.state;
    let { success, onClose, parentId, useType, allCanSee } = this.props;
    if (!name) {
      showMessage('请输入目录名称');
      return;
    }
    if (!src) {
      showMessage('请上传图标');
      return;
    }
    let apiParams = {
      allCanSee,
      createTime: null,
      createUser: null,
      grayIco: null,
      ico: src,
      id: null,
      level: 2,
      modifyTime: null,
      modifyUser: null,
      name,
      orderNum: null,
      parentId,
      showContentNum: 4,
      status: null,
      useType: useType ? useType : 'VISITOR',
    };
    this.setState({ loading: true });
    changeMenuList(apiParams)
      .then((res) => {
        this.setState({ loading: false });
        showMessage('添加成功', 'success');
        onClose();
        success();
      })
      .catch((res) => {
        this.setState({ loading: false });
      });
  };
  upDateSrc = (src) => {
    this.setState({ src });
  };
  onChange = (value) => {
    this.setState({ name: value });
  };

  render() {
    let { onClose } = this.props;
    let { name, loading } = this.state;
    return (
      <Modal
        loading={loading}
        onCancel={onClose}
        onOk={this.onOk}
        title="添加目录"
        width={452}
        okBtnShow={true}
        cancelText="取消"
        okText="保存"
        className={styles.modal}
      >
        <FormItem name={'目录名称'}>
          <div data-length={((name.length || 0) > 6 ? 6 : name.length || 0) + '/6'} className="count">
            <Input
              value={name}
              placeholder={'请输入'}
              maxLength={'6'}
              onChange={(e) => this.onChange(e.target.value)}
            />
          </div>
        </FormItem>
        <FormItem name={'上传图标'}>
          <Upload
            ref={(e) => (this.upload = e)}
            upDateSrc={(src) => this.upDateSrc(src)}
            limit={{ size: 500, type: 'jpeg/png' }}
          />
          <div className={styles.tips}>只能上传jpg/ png文件，且不超过500kb</div>
        </FormItem>
      </Modal>
    );
  }
}

export default AddMenu;
