import React from 'react';
import Upload from '../Upload';
import styles from './secondIconManage.less';

import Tabs, { Tab } from 'COMPONENT/Tabs';

import Button from 'COMPONENT/Button';
import { Switch } from 'antd';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

import Item from './secondTabsItem';

import { getIconList, saveIconList, getMenuList, publishIconList } from 'SERVICE/ipadFrameManage';

class SecondIconManage extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      name: '',
      iconList: [],
      iconListObj: {},
      saveObject: {},
      menuList: [],
      showLoading: this.props.showLoading,
      hideLoading: this.props.hideLoading,
    };
  }

  async componentWillReceiveProps(nextProps) {
    if (this.props.tags !== nextProps.tags) {
      let data = await this.getIconList(0);
      this.setState({ iconListObj: data });
    }
  }

  /**
   * 获取图标列表
   * @param index {number} 当前tabs页索引
   * @returns {Promise.<*>}
   */
  getIconList = async (index) => {
    let { idList } = this.props;
    let { iconListObj } = this.state;
    let id = idList[index]['id'];
    this.props.showLoading();
    let data = await getIconList({ menuLevel: 2, menuId: id });
    this.props.hideLoading();
    iconListObj[id] = data;
    return iconListObj;
  };

  /**
   * tabs页变化加载数据
   * @param activeKey {number} 当前索引
   * @returns {Promise.<void>}
   */
  tabsChange = async (activeKey) => {
    let { idList } = this.props;
    let { iconListObj } = this.state;
    let id = idList[activeKey]['id'];
    if (!(id in iconListObj)) {
      let data = await this.getIconList(activeKey);
      this.setState({ iconListObj: data, tabIndex: activeKey });
    } else {
      this.setState({ iconListObj, tabIndex: activeKey });
    }
  };

  /**
   * 保存成功之后更新图标列表
   * @param index
   * @returns {Promise.<void>}
   */
  updateIconList = async (index) => {
    let data = await this.getIconList(index);
    this.setState({ iconListObj: data });
  };
  changeMode = (val) => {
    let { idList } = this.props;
    let { iconListObj, tabIndex = 0 } = this.state;
    let id = idList[tabIndex]['id'];
    iconListObj[id][0].showType = val;
    this.setState({ iconListObj });
  };

  render() {
    let { idList } = this.props;
    let { iconListObj, tabIndex } = this.state;
    return (
      <Tabs className={styles.secondtabs} onChange={this.tabsChange}>
        {idList.map((v, k) => {
          return (
            <Tab key={k} title={v.name}>
              <Item
                data={iconListObj[v.id] || []}
                onSaveSuccess={(e) => this.updateIconList(tabIndex || 0)}
                changeMode={this.changeMode}
                showLoading={this.props.showLoading}
                hideLoading={this.props.hideLoading}
                showPreview={(k) => this.props.showPreview(iconListObj[v.id][k])}
              />
            </Tab>
          );
        })}
      </Tabs>
    );
  }
}

export default SecondIconManage;
