import React from 'react';
import Upload from '../Upload';
import styles from './secondTabsItem.less';

import Tabs, { Tab } from 'COMPONENT/Tabs';
import Radio from 'COMPONENT/Radio';
import Button from 'COMPONENT/Button';
import { Switch } from 'antd';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

const RadioGroup = Radio.Group;

import { saveIconList, publishIconList } from 'SERVICE/ipadFrameManage';

const mapOrder = {
  0: '一',
  1: '二',
  2: '三',
  3: '四',
  4: '五',
  5: '六',
  6: '七',
  7: '八',
  8: '九',
  9: '十',
  10: '十一',
  11: '十二',
  12: '十三',
  13: '十四',
  14: '十五',
};

class TabsItem extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      saveObject: {},
      tagsArray: [],
      // newAdd:{tag:false,clear:false}
    };
  }

  newAdd = { tag: false, clear: false };
  value = 1;

  shouldComponentUpdate(nextProps, nextState) {
    return nextProps.data !== this.props.data;
  }

  componentWillReceiveProps(nextProps) {
    let { data } = nextProps;
    if (data[0]) {
      this.value = data[0].showType === null ? 1 : data[0].showType;
      this.setState({});
    }
  }

  /**
   * 记录图片url地址
   * @param v {object} 第几套图标
   * @param V {object} 每套图标中单个目录信息对象
   * @param src {string} 阿里云图片地址
   * @param status {string} 选中还是未选中
   */
  upDateSrc = (v, V, src, k) => {
    let { saveObject, tagsArray } = this.state;
    let { id } = v;
    id = id ? id : 'newAdd';
    saveObject[id] = saveObject[id] ? saveObject[id] : [];
    let index = saveObject[id].findIndex((value, key) => {
      return V.menuName == value.menuName;
    });
    if (index === -1) {
      saveObject[id].push({ menuName: V.menuName, menuId: V.menuId, icoUrl: src });
    } else {
      saveObject[id][index]['icoUrl'] = src;
    }
    if (k === undefined) {
      this.newAdd['tag'] = true;
      this.newAdd['clear'] = false;
      this.forceUpdate();
    } else {
      tagsArray[k] = true;
      this.forceUpdate();
    }
  };

  /**
   * 图标管理保存
   * @param id
   */
  saveIconList = (id, k) => {
    let { saveObject, tagsArray } = this.state;
    let { data: propsData, onSaveSuccess } = this.props;
    let saveLists = saveObject[id] || [];
    let data = propsData.find((v, k) => {
      return v.id === id;
    });
    data = data ? data : propsData[0];
    let bindIcoVos = data.bindIcoVos.map((v, k) => {
      let obj = saveLists.find((V, K) => {
        return V.menuName == v.menuName;
      });
      let icoUrl;
      if (obj) {
        icoUrl = obj.icoUrl;
      }
      if (id === 'newAdd') {
        return {
          grayIco: null,
          icoId: null,
          icoUrl: icoUrl ? icoUrl : null,
          id: null,
          menuId: v.menuId,
        };
      } else {
        return {
          grayIco: v.grayIco,
          icoId: v.icoId,
          icoUrl: icoUrl ? icoUrl : v.icoUrl,
          id: v.id,
          menuId: v.menuId,
        };
      }
    });
    let apiParams = {
      bindIcoVos,
      id: id === 'newAdd' ? null : id,
      menuLevel: data.menuLevel,
      publishUser: data.publishUser,
      status: id === 'newAdd' ? '1' : data.status,
      menuId: data.menuId,
      showType: this.value, //0,图标模式，1,页签模式（默认）
    };
    this.props.showLoading();
    saveIconList(apiParams)
      .then(async (res) => {
        showMessage('保存成功', 'success');
        onSaveSuccess();
        tagsArray[k] = false;
        this.newAdd = { tag: false, clear: id === 'newAdd' ? true : false };
        // this.forceUpdate();
      })
      .catch((res) => {
        this.props.hideLoading();
      });
  };

  /**
   * 启用图标和取消启用
   * @param checked {Boolean} switch是否开启
   * @param id {number} 图标对应id
   */
  onChange = (checked, id) => {
    let { data: propsData, onSaveSuccess } = this.props;
    let status;
    let data = propsData.find((v, k) => {
      return v.id === id;
    });
    if (data) {
      let bindIcoVos = data.bindIcoVos;
      let len = bindIcoVos.length;
      for (let i = 0; i < len; i++) {
        if (!bindIcoVos[i].icoUrl) {
          showMessage('请将图标补充完整再启用!');
          return;
        }
      }
    }

    if (checked) {
      status = 0;
    } else {
      return;
      status = 1;
    }
    publishIconList({ id, menuLevel: 2 }).then((res) => {
      showMessage('启用成功', 'success');
      let copyPropsData = JSON.parse(JSON.stringify(propsData));
      onSaveSuccess();
    });
  };
  modeChange = (e) => {
    this.value = e.target.value;
    // this.forceUpdate(() => {
    // })
    let { data } = this.props;
    if (data[0]) {
      let selectDataArray = data.filter((v, k) => {
        return v.status === '0';
      });
      let selectData = selectDataArray[0];
      let apiParams = {
        bindIcoVos: selectData.bindIcoVos,
        id: selectData.id,
        menuLevel: selectData.menuLevel,
        publishUser: selectData.publishUser,
        status: selectData.status,
        menuId: selectData.menuId,
        showType: this.value, //0,图标模式，1,页签模式（默认）
      };
      saveIconList(apiParams).then((res) => {
        showMessage('模式切换成功', 'success');
        this.forceUpdate(() => {});
      });
    }
  };

  render() {
    let { tagsArray } = this.state;
    let { data } = this.props;
    return (
      <div>
        <div className={styles.mode}>
          <span>模式切换</span>
          <RadioGroup onChange={this.modeChange} value={this.value}>
            <Radio value={1}>页签模式</Radio>
            <Radio value={0}>图标模式</Radio>
          </RadioGroup>
        </div>
        {data.map((v, k) => {
          return (
            <div key={k} className={'item'}>
              <div className={styles.top}>
                <div className={styles.title}>第{mapOrder[k]}套图标</div>
                <Switch
                  checked={v.status === '0' ? true : false}
                  onChange={(e) => this.onChange(e, v.id)}
                  checkedChildren="启用"
                  unCheckedChildren="关闭"
                />
              </div>
              <div className={styles.content}>
                <div className={styles.normal}>
                  {v.bindIcoVos.map((V, K) => {
                    return (
                      <div className={styles.iconUpload} key={K}>
                        <Upload srcProps={V.icoUrl} upDateSrc={(src) => this.upDateSrc(v, V, src, k)} />
                        <span style={{ maxWidth: 90, whiteSpace: 'nowrap' }}>{V.menuName}</span>
                      </div>
                    );
                  })}
                </div>
              </div>
              <div className={styles.btn}>
                <Button
                  type="primary"
                  onClick={(e) => this.saveIconList(v.id, k)}
                  disabled={tagsArray[k] ? false : true}
                >
                  {tagsArray[k] ? '保存' : '已保存'}
                </Button>
              </div>
            </div>
          );
        })}
        <div className={'item'}>
          <div className={styles.top}>
            <div className={styles.title}>第{mapOrder[data.length]}套图标</div>
          </div>
          <div className={styles.content}>
            <div className={styles.normal}>
              {data[0] ? (
                data[0]['bindIcoVos'].map((V, K) => {
                  return (
                    <div className={styles.iconUpload} key={K}>
                      <Upload
                        srcProps={''}
                        clear={this.newAdd['clear']}
                        upDateSrc={(src) => this.upDateSrc({}, V, src)}
                      />
                      <span>{V.menuName}</span>
                    </div>
                  );
                })
              ) : (
                <div className={styles.noDataTips}>该目录下暂无二级目录，请创建时同时上传图标</div>
              )}
            </div>
          </div>
          {data[0] && (
            <div className={styles.btn}>
              <Button type="primary" disabled={!this.newAdd['tag']} onClick={(e) => this.saveIconList('newAdd')}>
                保存
              </Button>
            </div>
          )}
        </div>
      </div>
    );
  }
}

export default TabsItem;
