import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import CardList from 'COMPONENT/List/CardList';
import Button from 'COMPONENT/Button';
import Input from 'COMPONENT/Input';
import MultiSelect from 'COMPONENT/MultiSelect';
import IconButton from 'COMPONENT/IconButton';
import PopConfirm from 'COMPONENT/PopConfirm';
import TextLink from 'COMPONENT/TextLink';
import AddSecondMenu from '../BrowserMode/viewModal/secondAddMenu';
import IconManage from '../BrowserMode/viewModal/iconManage';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import Cell from '../BrowserMode/viewModal/cell';
import {
  getMenuList,
  getCompanyList,
  changeMenuList,
  changeVisibleRange,
  deleteMenu,
  changeMenuOrder,
} from 'SERVICE/ipadFrameManage';
import { Row, Col } from 'antd';
import styles from './index.less';
import { isEditDisable } from '../../config';

const [COL_WIDTH_S, COL_WIDTH_L, COL_WIDTH_XS] = [150, 300, 100];

class FrameManage extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      expandedRowKey: 0,
      companyDict: [], //城市字典
      list: [],
      subData: [],
      allCanSee: null,
    };
  }

  componentDidMount() {
    //获取集团分公司列表
    getCompanyList().then((res) => {
      let companyDict = [];
      if (res.length) {
        res.forEach((v, k) => {
          companyDict.push({ text: v.name, val: v.id });
        });
      }
      this.setState({ companyDict });
    });
    //加载列表数据
    this.loadData().then(async (res) => {
      let { parentId } = this.state;
      await this.loadData(2, parentId);
    });
  }

  /**
   * 加载数据
   * @param level{number} 一级目录和二级目录
   * @param parentId {number}目录id
   * @returns {Promise.<void>}
   */
  loadData = async (level, parentId) => {
    let { brenchId } = this.props;
    if (brenchId === '000001') {
      brenchId = '';
    }
    let apiParams = {
      level: level || 1,
      name: null,
      orgId: brenchId,
      parentId: parentId || null,
      useType: 'LOGIN_USER', //工作台模式
    };
    this.setState({ listLoading: true });
    let data = await getMenuList(apiParams);
    this.setState({ listLoading: false });
    let list = data.map((v, k) => {
      return {
        id: v.id,
        allCanSee: v.allCanSee,
        auths: v.auths,
        createTime: v.createTime,
        createUser: v.createUser,
        grayIco: v.grayIco,
        ico: v.ico,
        level: v.level,
        modifyTime: v.modifyTime,
        modifyUser: v.modifyUser,
        name: v.name,
        orderNum: v.orderNum,
        orgId: v.orgId,
        orgName: v.orgName,
        parentId: v.parentId,
        showContentNum: v.showContentNum,
        status: v.status,
        useType: v.useType,
      };
    });
    if (level === 2) {
      let backupSubData = JSON.parse(JSON.stringify(list));
      this.setState({ listLoading: false, backupSubData, subData: list || [] });
      return list;
    } else {
      this.setState({
        list,
        listLoading: false,
        parentId: list[0].id,
        allCanSee: list[0].allCanSee,
      });
    }
  };
  onExpand = () => {};
  /**
   * @param expandedRow{number} 展开索引
   */
  onExpandedRowChange = async (expandedRow) => {
    let { list } = this.state;
    if (typeof expandedRow === 'number') {
      let { id } = list[expandedRow];
      await this.loadData(2, id);
    }
    this.setState({ expandedRowKey: expandedRow, editIndex: null });
  };
  /**
   * 表格展开渲染内容
   * @param record {object}
   * @returns {XML}
   */
  expandedRowRender = (record) => {
    let { subEditIndex, companyDict, subData, orderManage } = this.state;
    let canHandle = this.props.brenchId === '000001';
    const { name } = record;
    const editDisable = isEditDisable(name);

    subData.forEach((v, k) => {
      let { orgId, allCanSee } = v;
      if (orgId && typeof orgId === 'string') {
        v.orgId = orgId.split(';');
      } else if (orgId === null && allCanSee === 0) {
        orgId = [];
        companyDict.forEach((v, k) => {
          orgId.push(v.val);
        });
        v.orgId = orgId;
      } else if (orgId === '') {
        v.orgId = [];
      }
      v.truthName = v.orgName;
    });

    return (
      <div>
        {subData.map((v, k) => {
          return (
            <Row gutter={0} key={k} style={{ display: 'flex', alignItems: 'center' }}>
              <Col span={2} className={styles.pad1215 + (subEditIndex === k ? ' editHeight' : '')} />
              <Col span={6} className={styles.pad1215 + (subEditIndex === k ? ' editHeight' : '')}>
                {subEditIndex === k && (
                  <div
                    className={styles.count}
                    data-length={((v.name.length || 0) > 6 ? 6 : v.name.length || 0) + '/6'}
                  >
                    <Input
                      value={v.name}
                      onChange={(e) => this.subChangeData(e.target.value, 'name', k)}
                      maxLength="6"
                    />
                  </div>
                )}
                {subEditIndex !== k && <span>{v.name}</span>}
              </Col>
              <Col span={12} className={styles.pad1215 + (subEditIndex === k ? ' editHeight' : '')}>
                {subEditIndex === k && (
                  <MultiSelect
                    dict={companyDict}
                    value={v.orgId}
                    onChange={(val) => this.subChangeData(val, 'orgId', k)}
                  />
                )}
                {subEditIndex !== k && <Cell text={v.truthName} />}
              </Col>
              <Col span={4} className={styles.pad1215 + (subEditIndex === k ? ' editHeight' : '')}>
                {orderManage && k === 0 && (
                  <div>
                    <span className={styles.mr50 + ' line'} />
                    <IconButton type="move_down_mc" onClick={(e) => this.gradeChange(k, 'down')} />
                  </div>
                )}
                {orderManage && k > 0 && k < subData.length - 1 && (
                  <div>
                    <IconButton className={styles.mr50} type="move_up_mc" onClick={(e) => this.gradeChange(k, 'up')} />
                    <IconButton type="move_down_mc" onClick={(e) => this.gradeChange(k, 'down')} />
                  </div>
                )}
                {orderManage && subData.length > 1 && k === subData.length - 1 && (
                  <div>
                    <IconButton className={styles.mr50} type="move_up_mc" onClick={(e) => this.gradeChange(k, 'up')} />
                    <span className={'line'} />
                  </div>
                )}
                {subEditIndex !== k && !orderManage && canHandle && (
                  <div className="edit">
                    <TextLink className={styles.mr30} onClick={(e) => this.subEdit(k)} disabled={editDisable}>
                      编辑
                    </TextLink>
                    <PopConfirm
                      content="确认删除该目录吗？"
                      okBtnText="是"
                      cancelBtnText="否"
                      onOk={(e) => this.deleteMenu(v.id)}
                    >
                      <TextLink disabled={editDisable}>删除</TextLink>
                    </PopConfirm>
                  </div>
                )}
                {subEditIndex === k && !orderManage && (
                  <div className="edit">
                    <TextLink className={styles.mr30} onClick={(e) => this.subSaveData(k)} disabled={editDisable}>
                      保存
                    </TextLink>
                    <TextLink onClick={(e) => this.subCancelEdit()}>取消</TextLink>
                  </div>
                )}
              </Col>
            </Row>
          );
        })}
      </div>
    );
  };
  /**
   * 添加二级目录
   * @param parentId {number} 父级目录id
   */
  addSecondMenu = (parentId, allCanSee) => {
    let { subData } = this.state;
    if (subData.length >= 8) {
      showMessage('最多只能创建8个目录！');
      return;
    }
    this.setState({ addSecondMenu: true, parentId, allCanSee });
  };
  /**
   * 删除二级目录
   * @param id {number} 当前目录id
   */
  deleteMenu = (id) => {
    let apiParams = { id };
    deleteMenu(apiParams).then((res) => {
      showMessage('删除目录成功', 'success');
      let { parentId } = this.state;
      this.loadData(2, parentId).then((res) => {
        this.setState({ subData: res || [] });
      });
    });
  };
  /**
   * 点击编辑目录按钮
   * @param index {number} 当前目录索引
   */
  subEdit = (index) => {
    let { subData, backupSubData } = this.state;
    let copyBackupSubData = JSON.parse(JSON.stringify(backupSubData));
    this.setState({ subData: copyBackupSubData });
    let copySubData = JSON.parse(JSON.stringify(subData));
    this.setState({ subEditIndex: index, copySubData });
  };
  /**
   * 编辑二级目录
   * @param value
   * @param key {string} 修改的属性名 'name' | 'orgId'
   * @param index {number} 当前所修改行的索引
   */
  subChangeData = (value, key, index) => {
    let { subData } = this.state;
    subData[index][key] = key === 'name' ? value : value || [];
    this.setState({ subData });
  };
  /**
   * 保存数据
   * @param idx {number}
   */
  subSaveData = (idx) => {
    let { list, companyDict, subData } = this.state;
    this.saveData(subData, idx, 2);
  };
  /**
   * 取消编辑操作
   */
  subCancelEdit = () => {
    let { backupSubData } = this.state;
    this.setState({ subData: backupSubData, subEditIndex: null });
  };
  /**
   * 添加二级目录成功之后的刷新列表回调
   * @param id {number} 目录id
   * @returns {Promise.<void>}
   */
  secondMenuSuccess = async (id) => {
    let subData = await this.loadData(2, id);
  };

  /**
   * 点击顺序管理按钮，备份一份数据，确保后续点击取消的时候找回原数据
   */
  orderManage = () => {
    let { subData } = this.state;
    let copySubData;
    copySubData = JSON.parse(JSON.stringify(subData));
    this.setState({ orderManage: true, copySubData });
  };
  /**
   * 点击取消按钮，撤销操作，找回原数据
   */
  cancelOrderManage = () => {
    let { copySubData } = this.state;
    this.setState({ orderManage: false, subData: copySubData });
  };

  /**
   * 调整目录顺序
   * @param idx {number} 当前行索引
   * @param handle {string} 'up' | 'down'
   */
  gradeChange = (idx, handle) => {
    let { subData: data } = this.state;
    let i = handle === 'down' ? idx + 1 : idx - 1;
    [data[idx], data[i]] = [data[i], data[idx]];
    let $nodeList = document.querySelectorAll('.workbenchMode .ant-table-expanded-row .ant-row');
    $nodeList[idx].classList.add('subBg');
    setTimeout(() => {
      this.setState({ subData: data });
      $nodeList[idx].classList.remove('subBg');
      $nodeList[i].classList.add('subBg');
      setTimeout(() => {
        $nodeList[i].classList.remove('subBg');
      }, 600);
    }, 300);
  };

  /**
   *保存目录顺序
   */
  saveOrder = () => {
    let { expandedRowKey, list, subData } = this.state;
    let apiParams = [];
    let data = expandedRowKey !== null ? subData : list;
    data.forEach((v, k) => {
      apiParams.push({ id: v.id, orderNum: k });
    });
    this.setState({ listLoading: true });
    changeMenuOrder(apiParams).then((res) => {
      showMessage('保存成功', 'success');
      this.onExpandedRowChange(expandedRowKey);
      this.setState({ orderManage: false, listLoading: false });
    });
  };

  edit = (index) => {
    let { list } = this.state;
    let copyList = JSON.parse(JSON.stringify(list));
    this.setState({ editIndex: index, copyList });
  };
  cancelEdit = () => {
    let { copyList } = this.state;
    this.setState({ editIndex: null, list: copyList });
  };
  changeData = (value, key, index) => {
    let { list } = this.state;
    list[index][key] = key === 'name' ? value : value || [];
    this.setState({ list });
  };

  saveData = (list, idx, level) => {
    let { companyDict, subData } = this.state;
    let apiParams;
    let visibleParams = [];
    let saveParams = list[idx];
    let visibleRangeArray = list[idx]['orgId'];
    if (level) {
      saveParams = subData[idx];
      delete saveParams['orgIdArray'];
      delete saveParams['truthName'];
      visibleRangeArray = subData[idx]['orgId'];
    }
    let { id } = saveParams;
    if (!visibleRangeArray) {
      visibleRangeArray = [];
      companyDict.forEach((v, k) => {
        visibleRangeArray.push(v.val);
      });
    } else if (typeof visibleRangeArray === 'string') {
      visibleRangeArray = visibleRangeArray.split(';');
    }
    visibleRangeArray.forEach((v, k) => {
      visibleParams.push({ cmsMenuId: id, orgId: v });
    });
    apiParams = {
      ...saveParams,
      orgName: null,
      orgId: null,
      allCanSee: companyDict.length === visibleParams.length ? 0 : 1,
    };
    if (!apiParams.name) {
      showMessage('请输入目录名称！');
      return;
    }

    this.setState({ listLoading: true });
    Promise.all([
      changeVisibleRange({ allCanSee: companyDict.length === visibleParams.length ? 0 : 1, auths: visibleParams, id }),
      changeMenuList(apiParams),
    ])
      .then(([rangeRes, res]) => {
        showMessage('保存成功！', 'success');
        if (level) {
          let { expandedRowKey } = this.state;
          this.onExpandedRowChange(expandedRowKey);
          this.setState({ subEditIndex: null });
        } else {
          this.loadData();
          this.setState({ editIndex: null });
        }
      })
      .catch((res) => {
        this.setState({ listLoading: false });
      });
  };

  render() {
    let {
      listLoading,
      expandedRowKey,
      orderManage,
      list,
      companyDict,
      addSecondMenu,
      iconManage,
      parentId,
      subData,
      allCanSee,
    } = this.state;
    //收起展开行或者子目录只有一个时，让顺序管理按钮不可用
    let orderDisabled = expandedRowKey !== null ? subData.length <= 1 : true;
    let canHandle = this.props.brenchId === '000001';
    const columns = [
      {
        title: '菜单',
        dataIndex: 'name',
        width: COL_WIDTH_S,
        key: 'name',
        render: (text, record, idx) => {
          let { editIndex } = this.state;
          if (editIndex === idx) {
            return <Input value={text} onChange={(e) => this.changeData(e.target.value, 'name', idx)} maxLength="4" />;
          } else {
            return <span>{text}</span>;
          }
        },
      },
      {
        title: '可见范围',
        dataIndex: 'orgName',
        width: COL_WIDTH_L,
        key: 'orgName',
        render: (text, record, idx) => {
          let { editIndex } = this.state;
          let { orgId, allCanSee } = record;
          if (orgId && typeof orgId === 'string') {
            orgId = orgId.split(';');
          } else if (orgId === null && allCanSee === 0) {
            orgId = [];
            companyDict.forEach((v, k) => {
              orgId.push(v.val);
            });
          } else if (orgId === '') {
            orgId = [];
          }
          if (editIndex === idx) {
            return (
              <MultiSelect dict={companyDict} value={orgId} onChange={(val) => this.changeData(val, 'orgId', idx)} />
            );
          } else {
            return <Cell text={text} />;
          }
        },
      },
      {
        title: '操作',
        dataIndex: 'handle',
        width: COL_WIDTH_XS,
        key: 'handle',
        render: (text, record, idx) => {
          const { editIndex } = this.state;
          const { id, allCanSee, name } = record;
          const editDisable = isEditDisable(name);

          if (expandedRowKey === null) {
            if (editIndex !== idx) {
              if (canHandle) {
                return (
                  <div className={styles.handle}>
                    <TextLink onClick={(e) => this.edit(idx)} disabled={editDisable}>
                      编辑
                    </TextLink>
                  </div>
                );
              } else {
                return '';
              }
            } else {
              return (
                <div className={styles.handle}>
                  <TextLink onClick={(e) => this.saveData(list, idx)} disabled={editDisable}>
                    保存
                  </TextLink>
                  <TextLink onClick={(e) => this.cancelEdit()}>取消</TextLink>
                </div>
              );
            }
          } else if (expandedRowKey !== null && expandedRowKey === idx && canHandle) {
            return (
              <TextLink onClick={(e) => this.addSecondMenu(id, allCanSee)} disabled={editDisable}>
                添加二级目录
              </TextLink>
            );
          }
        },
      },
    ];

    return (
      <Page
        loading={false}
        route={this.props.route}
        className={styles.frameManage + ' workbenchMode'}
        fitWindowHeight
        subHeader={
          <div className={styles.wrap}>
            <div style={{ marginBottom: 15 }}>
              {!orderManage && (
                <span>
                  <Button
                    type="primary"
                    className={styles.fr}
                    disabled={orderDisabled || !canHandle}
                    onClick={(e) => this.orderManage()}
                  >
                    顺序管理
                  </Button>
                </span>
              )}
              {orderManage && (
                <span>
                  <Button type="primary" className={styles.fr} onClick={(e) => this.cancelOrderManage()}>
                    取消
                  </Button>
                  <Button type="primary" className={styles.fr} onClick={(e) => this.saveOrder()}>
                    确认修改
                  </Button>
                </span>
              )}
            </div>
          </div>
        }
      >
        <CardList
          dataSource={list}
          columns={columns}
          loading={listLoading}
          expandedRowRender={(record) => this.expandedRowRender(record)}
          expandedRowKey={expandedRowKey}
          onExpandedRowChange={(expandedRows) => {
            this.onExpandedRowChange(expandedRows);
          }}
          detailIndent={false}
          onExpand={(expanded, record) => {
            this.onExpand(expanded, record);
          }}
        />
        {addSecondMenu && (
          <AddSecondMenu
            onClose={(e) => this.setState({ addSecondMenu: false })}
            success={(e) => this.secondMenuSuccess(parentId)}
            parentId={parentId}
            useType={'LOGIN_USER'}
            allCanSee={allCanSee}
          />
        )}
        {iconManage && <IconManage onClose={(e) => this.setState({ iconManage: false })} />}
      </Page>
    );
  }
}

function mapStateToProps(state) {
  return {
    brenchId: state.account.user.brenchId,
  };
}

export default connect(mapStateToProps)(FrameManage);
