import React from 'react';
import SearchBar from 'COMPONENT/SearchBar';
import Select from 'COMPONENT/Select';
import DatePicker from 'COMPONENT/DatePicker';
import moment from 'moment';

const Search = (props) => {
  const {
    valueMap: {
      createTimeStart,
      createTimeEnd,
      uploadCompleteTimeStart,
      uploadCompleteTimeEnd,
      branchId,
      blCampusId,
      consultantId,
      isBranchDisabled,
      isCampusDisabled,
      isConsultDisabled,
    },
    dict: { branchDict, campusDict, consultantDict },
    handleKeyword,
    getDisabledStartDate,
    getDisabledEndDate,
    ...resetProps
  } = props;
  return (
    <SearchBar className="class-search" {...resetProps}>
      {!isBranchDisabled && (
        <Select
          placeholder="请选择分公司"
          dict={branchDict}
          value={branchId}
          allowClear={false}
          showSearch={true}
          disabled={isBranchDisabled}
          onChange={(val) => handleKeyword({ type: 'branchId', val })}
        />
      )}
      {!isCampusDisabled && (
        <Select
          placeholder="请选择学习中心"
          dict={campusDict}
          value={blCampusId}
          allowClear={true}
          showSearch={true}
          disabled={isCampusDisabled}
          onChange={(val) => handleKeyword({ type: 'blCampusId', val })}
        />
      )}
      <Select
        placeholder="请选择咨询师"
        dict={consultantDict}
        value={consultantId}
        allowClear={true}
        showSearch={true}
        disabled={isConsultDisabled}
        onChange={(val) => handleKeyword({ type: 'consultantId', val })}
      />
      <DatePicker
        placeholder="录音创建时间（开始）"
        value={createTimeStart}
        disabledDate={(beginTime) => getDisabledStartDate(beginTime, 'createTimeEnd')}
        onChange={(val) => handleKeyword({ type: 'createTimeStart', val })}
      />
      <DatePicker
        placeholder="录音创建时间（结束）"
        value={createTimeEnd}
        disabledDate={(endTime) => getDisabledEndDate(endTime, 'createTimeStart')}
        onChange={(val) => handleKeyword({ type: 'createTimeEnd', val })}
      />

      <DatePicker
        placeholder="上传完成时间（开始）"
        value={uploadCompleteTimeStart}
        disabledDate={(beginTime) => getDisabledStartDate(beginTime, 'uploadCompleteTimeEnd')}
        onChange={(val) => handleKeyword({ type: 'uploadCompleteTimeStart', val })}
      />
      <DatePicker
        placeholder="上传结束时间（开始）"
        value={uploadCompleteTimeEnd}
        disabledDate={(endTime) => getDisabledEndDate(endTime, 'uploadCompleteTimeStart')}
        onChange={(val) => handleKeyword({ type: 'uploadCompleteTimeEnd', val })}
      />
    </SearchBar>
  );
};
export default Search;
