/**
 * Created by Haolin on 2018/11/28.
 */
import React, { PropTypes } from 'react';
import { Icon } from 'antd';
import compStyle from './compStyle.less';

const COURSE_STATUS_HAD_NOT = 'NEW';
const DateShow = (props) => {
  const { year, mount, day } = props;
  const classPrefix = 'DateShow';
  return (
    <div className={compStyle[classPrefix]}>
      <div className={compStyle[`${classPrefix}-top`]}>{`${mount}月${day}日`}</div>
      <div className={compStyle[`${classPrefix}-bottom`]}>{`${year}年`}</div>
    </div>
  );
};
const Lesson = (props) => {
  const { data } = props;
  const classPrefix = 'Lesson';
  return (
    <div className={compStyle[classPrefix]}>
      <Icon type="book" />
      <span className={compStyle[`${classPrefix}-text`]}>{data}</span>
    </div>
  );
};
const DateCard = (props) => {
  const {
    data: { name, year, mount, day, select },
    onClick,
  } = props;
  const classPrefix = 'DateCard';
  let checkStr = select ? '-check' : '-no-check';
  return (
    <div className={compStyle[classPrefix]} onClick={onClick}>
      <div className={compStyle[`${classPrefix}-wrap`]}>
        <DateShow year={year} mount={mount} day={day} />
        <Lesson data={name} />
        <Icon className={compStyle[`${classPrefix}${checkStr}`]} type="check-circle" />
      </div>
    </div>
  );
};

class DateDesk extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      list: [],
    };
    this.init = this.init.bind(this);
    this.selectDate = this.selectDate.bind(this);
    this.handleSelect = this.handleSelect.bind(this);
  }
  static propTypes = {
    data: PropTypes.array,
    onChange: PropTypes.func,
    defaultAllSelected: PropTypes.bool,
  };

  static defaultProps = {
    defaultAllSelected: false,
  };

  static classPrefix = 'DateDesk';
  // 选择时间策略
  static newSelect(list, index) {
    let len = list.length;
    for (let i = index; i < len; i++) {
      list[i].select = true;
    }
    return list;
  }
  static reSelect(list, index) {
    for (let i = index - 1; i >= 0; i--) {
      list[i].select = false;
    }
    return DateDesk.newSelect(list, index);
  }
  static clearSelect(list) {
    list.forEach((item) => (item.select = false));
    return list;
  }

  componentDidMount() {
    this.init();
  }

  init() {
    const { data, defaultAllSelected } = this.props;
    let list = data.map((item) => ({ ...item, select: defaultAllSelected }));
    this.setState({ list });

    // 如果默认是全选的话 需要人为触发一次onChange事件
    if (defaultAllSelected) {
      this.props.onChange(0);
    }
  }
  /**
   * 选择上课日
   * */
  selectDate(params) {
    const { index } = params;
    let originList = this.state.list.slice();
    let list = this.handleSelect(originList.slice(), index);
    this.setState({ list });
  }
  /**
   * 返回更新后的list，并将选中的日期index传回父组件
   * */
  handleSelect(list, index) {
    const { onChange } = this.props;
    let tag = list[index].select;
    if (tag) {
      // 当前tag为选中
      let prev = list[index - 1];
      if (prev && prev.select) {
        // tag前一个存在且选中
        onChange(index);
        return DateDesk.reSelect(list, index);
      } else {
        // tag前一个不存在或者未选中
        onChange(null);
        return DateDesk.clearSelect(list, index);
      }
    } else {
      // 当前tag为未选中
      onChange(index);
      return DateDesk.newSelect(list, index);
    }
  }

  render() {
    const { list } = this.state;

    return (
      <div className={compStyle[DateDesk.classPrefix]}>
        <div className={compStyle[`${DateDesk.classPrefix}-wrap`]}>
          {list.map((item, index) => {
            return (
              <DateCard
                data={item}
                index={index}
                onClick={() => {
                  this.selectDate({ ...item, index });
                }}
                key={index}
              />
            );
          })}
        </div>
      </div>
    );
  }
}

export default DateDesk;
