/**
 * Created by Haolin on 2018/11/28.
 */
import React from 'react';
import Modal from 'COMPONENT/Modal';
import DateDesk from './DateDesk';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import compStyle from './compStyle.less';
import './style.less';
import { getMiniClassCourseDetail, findCourseByClassTwoId } from 'SERVICE/course';

const NoData = (props) => {
  return (
    <div className={compStyle['DateDesk-no-data']}>
      <div className={compStyle['DateDesk-no-data-tips']}>{props.text}</div>
    </div>
  );
};

class ChooseBeginDateModal extends React.Component {
  constructor(props) {
    super(props);
    const { courseTimeList = [] } = this.props;

    const rows = this.buildCourseList(courseTimeList);

    this.state = {
      name: '选择上课时间',
      list: rows,
      selectIndex: null,
      noDataText: '正在加载...',
    };
    this.loadCourseList = this.loadCourseList.bind(this);
    this.buildCourseList = this.buildCourseList.bind(this);
    this.updateDateSelect = this.updateDateSelect.bind(this);
    this.confirm = this.confirm.bind(this);
  }
  static classPrefix = 'ChooseBeginDateModal';

  /**
   * 加载课程列表
   * */
  loadCourseList(classId) {
    const { product } = this.props;
    const params = {
      page: 1,
      rows: 100,
    };
    switch (product.category) {
      case 'SMALL_CLASS':
        return getMiniClassCourseDetail({ ...params, miniClassId: classId });
      case 'TWO_TEACHER':
        return findCourseByClassTwoId({ ...params, twoTeacherClassTwoId: classId });
      default:
        throw new Error(`invalid product type`);
    }
  }
  /**
   * 构建课程列表
   * */
  buildCourseList(list) {
    return list.map((item, index) => {
      const { courseId, courseDate, miniClassCourseId, courseStatus } = item;
      const dateArr = courseDate.split('-');
      return {
        courseId: courseId,
        miniClassCourseId: miniClassCourseId,
        courseDate,
        name: `第 ${index + 1} 讲`,
        year: Number(dateArr[0]),
        mount: Number(dateArr[1]),
        day: Number(dateArr[2]),
        courseStatus,
      };
    });
  }
  /**
   * 选择时间
   * */
  updateDateSelect(params) {
    this.setState(() => ({ selectIndex: params }));
  }
  /**
   * 确定按钮
   * */
  async confirm() {
    this.setState({ loading: true });
    const { onConfirm } = this.props;
    const { list, selectIndex } = this.state;
    if (selectIndex === null) {
      this.modal.showOkTips('请选择班课');
      this.setState({ loading: false });
      return;
    }
    await onConfirm({
      selectedCourse: list.slice(selectIndex),
    });
  }

  render() {
    const { onClose } = this.props;
    const { list, selectIndex, noDataText } = this.state;

    return (
      <Modal
        title="选择上课时间"
        className={`${ChooseBeginDateModal.classPrefix}`}
        width={660}
        onCancel={onClose}
        onOk={this.confirm}
        okBtnDisabled={selectIndex === null}
        ref={(c) => (this.modal = getWrappedInstance(c))}
      >
        {list.length > 0 ? (
          <DateDesk data={list} onChange={this.updateDateSelect} defaultAllSelected={true} />
        ) : (
          <NoData text={noDataText} />
        )}
      </Modal>
    );
  }
}

export default ChooseBeginDateModal;
