/**
 * Created by Haolin on 2019/4/9.
 */
import React from 'react';
import { connect } from 'dva';
import style from './index.less';
import Modal from 'COMPONENT/Modal';
import FormItem from 'COMPONENT/FormItem';
import Input from 'COMPONENT/Input';
import Select from 'COMPONENT/Select';
import { getSchoolByCampus } from 'UTIL/dict';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import { editStudentInfo } from 'SERVICE/marketActivityManage';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import loader from 'PAGE/marketActivityManage/MarketActivity/loader.js';
import styles from 'PAGE/marketActivityManage/MarketActivity/styles.less';

class ModalEditStdInfo extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      name: '修改',
      loading: false,
      dictSchool: null,
      studentName: '',
      studentPhone: '',
      grade: null,
      school: null,
      schoolName: '',
      validator: {},
      validateResult: true,
      studentHasOrder: false,
    };
    this._modal = null;
    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.initSchoolDict = this.initSchoolDict.bind(this);
    this.initDefaultVal = this.initDefaultVal.bind(this);
    this.checkStudentIsOld = this.checkStudentIsOld.bind(this);
    this.setModalRef = this.setModalRef.bind(this);
    this.updateStdName = this.updateStdName.bind(this);
    this.updateGrade = this.updateGrade.bind(this);
    this.updateSchool = this.updateSchool.bind(this);
    this.updateSchoolName = this.updateSchoolName.bind(this);
    this.handleValidator = this.handleValidator.bind(this);
    this.validateData = this.validateData.bind(this);
    this.buildSubmitParams = this.buildSubmitParams.bind(this);
    this.submit = this.submit.bind(this);
  }
  static classPrefix = 'ModalEditStdInfo';
  // 校验规则-不能超过n个字数
  static validateStrLen(len) {
    return function (val) {
      return val.length > len ? `不能超过${len}个字` : null;
    };
  }
  // 校验规则-手机号
  static validateLessStr(val) {
    if (!val) return;
    return /^1\d{10}$/.test(val) ? null : '请确认手机号码是否填写正确';
  }
  componentDidMount() {
    this.init();
  }
  componentWillUnmount() {}

  // todo: 获取当前学生在账号中心中新老生判断状态
  checkStudentIsOld() {
    loader
      .getStudentById(this.props.data.studentId)
      .then((res) => {
        if (res && res.studentType === 'ENROLLED') {
          this.setState({
            studentHasOrder: true,
          });
        }
      })
      .catch((err) => showMessage(err.text, 'error'));
  }

  // 初始化
  async init() {
    this.setState({ loading: true });
    await this.initDict();
    await this.initSchoolDict();
    await this.initDefaultVal();
    await this.checkStudentIsOld();
    this.setState({ loading: false });
  }
  initDict() {
    return new Promise((resolve) => {
      this.props.dispatch({
        type: 'dict/initDict',
        payload: {
          list: ['grade'],
          callback() {
            resolve();
          },
        },
      });
    });
  }
  initSchoolDict() {
    return new Promise((resolve) => {
      getSchoolByCampus(this.props.data.campusId).then((dictSchool) => {
        dictSchool.push({
          text: '其他',
          val: 'other',
        });
        this.setState({ dictSchool }, () => {
          resolve();
        });
      });
    });
  }
  initDefaultVal() {
    return new Promise((resolve) => {
      this.setState(
        {
          studentName: this.props.data.name,
          studentPhone: this.props.data.studentPhone,
          grade: this.props.data.gradeId,
          school: this.props.data.schoolId,
        },
        () => {
          resolve();
        },
      );
    });
  }
  // 设置弹窗ref
  setModalRef(ref) {
    if (this._modal) {
      return this._modal;
    } else {
      return (this._modal = getWrappedInstance(ref));
    }
  }
  // 更新值
  updateStdName(ele) {
    this.setState({ studentName: ele.target.value });
  }
  updateGrade(val) {
    this.setState({ grade: val });
  }
  updateSchool(val) {
    this.setState((prevState) => {
      let payload = { school: val };
      // 当原值是other时，需将校验结果里的学校名称去掉
      if (prevState.school === 'other' && prevState.school !== val) {
        payload.validator = prevState.validator;
        delete payload.validator.schoolName;
      }
      return payload;
    });
  }
  updateSchoolName(ele) {
    this.setState({ schoolName: ele.target.value });
  }
  // 处理表单校验值更新
  handleValidator({ type, val }) {
    this.setState((prevState) => ({
      validator: {
        ...prevState.validator,
        [type]: val,
      },
    }));
  }
  // 校验数据
  validateData(resolve) {
    this.setState(
      {
        validateResult: Object.values(this.state.validator).every((item) => !Boolean(item)),
      },
      () => {
        resolve();
      },
    );
  }
  // 构建提交参数
  buildSubmitParams() {
    let params = {
      orderNo: this.props.data.orderNo,
      customerPhone: this.props.data.phone,
      studentNo: this.props.data.id,
      bossStudentId: this.props.data.studentId,
      newStudentPhone: this.state.studentPhone,
      oldStudentName: this.props.data.name,
      studentName: this.state.studentName,
      gradeId: this.state.grade,
      schoolNo: this.state.school,
      schoolName: this.state.schoolName,
      isNewSchool: false,
    };
    if (
      this.props.data.name === params.studentName &&
      this.props.data.gradeId === params.gradeId &&
      this.props.data.studentPhone === params.studentPhone &&
      this.props.data.schoolId === params.schoolNo
    ) {
      params = null;
    } else {
      if (params.schoolNo === 'other') {
        delete params.schoolNo;
        params.isNewSchool = true;
      } else {
        delete params.schoolName;
      }
    }
    return params;
  }
  // 提交
  async submit() {
    await new Promise(this.validateData);
    if (this.state.validateResult) {
      let apiParams = this.buildSubmitParams();
      if (!apiParams) {
        this._modal.showOkTips('请先修改学生信息后再保存！');
        return;
      }
      try {
        const rsp = await editStudentInfo(apiParams, true);
        if (rsp && rsp.resultCode === 0) {
          showMessage('学生信息保存成功！', 'success');
          this.props.callback(apiParams);
        } else {
          showMessage('学生信息保存错误，请联系管理员！', 'error');
        }
      } catch (e) {
        showMessage(e.text || JSON.stringify(e), 'error');
      }
    }
  }

  render() {
    const { onCancel, dictGrade } = this.props;
    const {
      name,
      loading,
      dictSchool,
      studentName,
      studentPhone,
      grade,
      school,
      schoolName,
      validator,
      validateResult,
      studentHasOrder,
    } = this.state;
    const prefix = ModalEditStdInfo.classPrefix;
    return (
      <Modal
        width={520}
        title={this.props.name || name}
        className={style[prefix]}
        loading={loading}
        widthSize="S"
        ref={this.setModalRef}
        okText={this.props.okText || '保存'}
        onOk={this.submit}
        onCancel={onCancel}
        cancelBtnShow={this.props.okText ? false : true}
      >
        <FormItem name="学生名称" labelWidth={100}>
          <Input
            placeholder="请输入学生姓名(最多20字)"
            value={studentName}
            disabled={studentHasOrder}
            validator={{
              rules: ['required', ModalEditStdInfo.validateStrLen(20)],
              silent: !Boolean(validator.studentName) || validateResult,
              showStatus: true,
              showTips: true,
              onErrorChange: (err) => {
                this.handleValidator({ type: 'studentName', val: err });
              },
            }}
            onChange={this.updateStdName}
          />
        </FormItem>
        <FormItem name="学生手机号码" labelWidth={100}>
          <Input
            placeholder="请输入学生手机号码"
            value={studentPhone}
            validator={{
              rules: [ModalEditStdInfo.validateLessStr],
              silent: !Boolean(validator.studentPhone) || validateResult,
              showStatus: true,
              showTips: true,
              onErrorChange: (err) => {
                this.handleValidator({ type: 'studentPhone', val: err });
              },
            }}
            maxLength={11}
            onChange={(e) => this.setState({ studentPhone: e.target.value })}
          />
          <div className={styles.warningMSg}>新增学生手机号码字段，即客户信息中的学生手机号一致</div>
        </FormItem>
        <FormItem name="学生年级" labelWidth={100}>
          <Select
            placeholder="请选择学生年级"
            dict={dictGrade}
            value={grade}
            showSearch
            allowClear={false}
            onChange={this.updateGrade}
          />
        </FormItem>
        <FormItem name="在读学校" labelWidth={100}>
          <Select
            placeholder="请选择在读学校"
            dict={dictSchool}
            value={school}
            showSearch
            allowClear={false}
            disabled={schoolName !== ''}
            onChange={this.updateSchool}
          />
        </FormItem>
        {school === 'other' && (
          <FormItem name="备注" labelWidth={100}>
            <Input
              placeholder="请输入在读学校名称"
              value={schoolName}
              validator={{
                rules: ['required', ModalEditStdInfo.validateStrLen(20)],
                silent: !Boolean(validator.schoolName) || validateResult,
                showStatus: true,
                showTips: true,
                onErrorChange: (err) => {
                  this.handleValidator({ type: 'schoolName', val: err });
                },
              }}
              onChange={this.updateSchoolName}
            />
          </FormItem>
        )}
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    dictGrade: state.dict.grade,
  };
};

export default connect(mapStateToProps)(ModalEditStdInfo);
