import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import { Form, DatePicker, Row, Col } from 'antd';
import SearchBar from './NewSearchBar';
import moment from 'moment';
import Input from 'COMPONENT/Input';
import Select from 'COMPONENT/Select';
import { loadBranchCampusDict } from 'UTIL/dict';
import List from 'COMPONENT/List';
import { getVisitUser, getInviteDetail } from './loader';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import Button from 'COMPONENT/Button';
import styles from './index.less';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import utils from '../../common/utils';
import { loadCampusDict } from 'UTIL/dict';
import VisitClientModal from './modal/VisitClientModal';

const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;
const { RangePicker } = DatePicker;
const dateFormat = 'YYYY-MM-DD HH:mm';
// 学籍学习中心字典
const blCampusDict = [
  { text: '本学习中心', val: '1' },
  { text: '其他学习中心', val: '2' },
];
// 是否在读字典
const classStatusDict = [
  { text: '在读', val: '1' },
  { text: '非在读', val: '2' },
];
// 是否有转介绍
const isTransformDict = [
  { text: '有', val: '1' },
  { text: '无', val: '0' },
];
// 是否下单
const isPurchaseDict = [
  { text: '是', val: '1' },
  { text: '否', val: '0' },
];

class VisitClientPage extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      time: null,
      contactNo: null,
      studentName: null,
      blCampusId: null, // 学籍学习中心
      attendSchool: null, // 意向学习中心
      classStatus: null, //是否在读
      isCreateOrder: null, //是否下单
      isInviteSuccess: null, // 是否转介绍
      listData: [], // 列表数据

      campusDict: null, // 意向学习中心字典
      showVisitClientModal: false,
      selectedRecord: null,
      searchExpand: true,
    };

    // List 列配置
    this.columns = [
      {
        title: '序号',
        dataIndex: 'activityId',
        render: (value, row, index) => {
          return index + 1;
        },
        width: COL_WIDTH_S,
      },
      { title: '学生名称', dataIndex: 'studentName' },
      { title: '客户名称', dataIndex: 'customerName' },
      { title: '客户号码', dataIndex: 'contactNo' },
      { title: '意向学习中心', dataIndex: 'attendSchoolName' },
      { title: '学籍学习中心', dataIndex: 'blCampusName' },
      {
        title: '是否在读',
        dataIndex: 'classStatus',
        render: (value) => {
          if (value === '1') {
            return '在读';
          } else {
            return '非在读';
          }
        },
      },
      { title: '是否下单', dataIndex: 'isCreateOrder', render: (text) => (text === 1 ? '已下单' : '未下单') },
      { title: '转介绍人数', dataIndex: 'inviteCount' },
      {
        title: '学管师',
        dataIndex: 'teacherName',
        width: COL_WIDTH_M,
        render: (text, record) => {
          const productTypeId = this.props.productTypeId;
          const studyManagerName = productTypeId === 'SMALL_CLASS' ? record.teacherName : record.studyManegerName;
          if (!studyManagerName) {
            return '';
          }
          const filterTeacherName = Array.from(new Set(studyManagerName.split(',')));
          return filterTeacherName.join(',');
        },
      },
      { title: '年级', dataIndex: 'gradeName' },
      { title: '在读学校', dataIndex: 'originalSchoolName' },
      { title: '注册时间', dataIndex: 'registTime', width: COL_WIDTH_L },
      {
        title: '操作',
        key: 'action',
        width: COL_WIDTH_M,
        fixed: 'right',
        getActionConfig: (text, record, index) => {
          const { inviteCount } = record;
          return inviteCount > 0 ? [{ text: '转介绍详情', onClick: () => this.getInviteDetail(record) }] : [];
        },
      },
    ];
    // List 分页配置
    this.pagination = {
      current: 1,
      pageSize: 10,
      total: 0,
    };
  }

  componentDidMount() {
    this.init();
  }

  init = async () => {
    const branchId = this.props.branchId;
    let campusDict = await loadBranchCampusDict(branchId);
    campusDict = campusDict.filter((ele) => ele.val === branchId);
    this.search();
    this.setState({ campusDict });
  };

  oncontactNoChange = (event) => {
    this.setState({ contactNo: event.target.value === '' ? null : event.target.value });
  };

  onStudentNameChange = (event) => {
    this.setState({ studentName: event.target.value === '' ? null : event.target.value });
  };

  onAttendSchoolChange = (value) => {
    this.setState({ attendSchool: value });
  };

  onBlCampusChange = (value) => {
    this.setState({ blCampusId: value });
  };

  onClassStatusChange = (value) => {
    this.setState({ classStatus: value });
  };
  onPurchaseChange = (value) => {
    this.setState({ isCreateOrder: value });
  };
  onTransformChange = (value) => {
    this.setState({ isInviteSuccess: value });
  };

  parseTime = (time) => {
    let [beginTime, endTime] = time;
    return {
      beginTime: moment(beginTime).format('YYYY-MM-DD HH:mm:ss'),
      endTime: moment(endTime).format('YYYY-MM-DD HH:mm:ss'),
    };
  };

  onPaginationChange = (pagination) => {
    this.pagination = Object.assign({}, this.pagination, {
      pageSize: pagination.pageSize,
      current: pagination.current,
    });
    this.search(pagination.current);
  };

  search = async (pageNo = 1) => {
    this.setState({ loading: true });
    const { activityId } = this.props;
    const { time, attendSchool, classStatus, contactNo, blCampusId, studentName, isInviteSuccess, isCreateOrder } =
      this.state;
    let beginTime;
    let endTime;
    if (time === null) {
      beginTime = null;
      endTime = null;
    } else {
      let timer = this.parseTime(time);
      beginTime = timer.beginTime;
      endTime = timer.endTime;
    }
    const { pageSize } = this.pagination;
    const params = {
      activityId,
      attendSchool,
      beginTime,
      endTime,
      classStatus,
      contactNo,
      blCampusId,
      studentName,
      pageNo,
      pageSize,
      isInviteSuccess,
      isCreateOrder,
    };
    try {
      const { records, total } = await getVisitUser(params);
      this.pagination = Object.assign({}, this.pagination, { total });
      this.setState({ listData: records, loading: false });
    } catch (e) {
      this.setState({ loading: false });
    }
  };

  reset = () => {
    this.setState({
      time: null,
      contactNo: null,
      studentName: null,
      isInviteSuccess: null,
      isCreateOrder: null,
      blCampusId: null, // 学籍学习中心
      attendSchool: null, // 意向学习中心
      classStatus: null, //是否在读
    });
  };

  exportExcel = async () => {
    const { activityId, activityName } = this.props;
    const { time, attendSchool, classStatus, contactNo, blCampusId, studentName, isInviteSuccess, isCreateOrder } =
      this.state;
    let beginTime;
    let endTime;
    if (time === null) {
      beginTime = null;
      endTime = null;
    } else {
      let timer = this.parseTime(time);
      beginTime = timer.beginTime;
      endTime = timer.endTime;
    }
    const { current: pageNo, pageSize } = this.pagination;
    const params = {
      activityId,
      attendSchool,
      beginTime,
      endTime,
      classStatus,
      contactNo,
      blCampusId,
      studentName,
      pageNo,
      pageSize,
      isCreateOrder,
      isInviteSuccess,
    };
    try {
      await utils.exportExcel({
        apiUrl: 'MarketActivityController/exportVisitUser.do',
        data: params,
        fileName: `${activityName}访问客户列表`,
        success: () => {
          showMessage('导出成功！', 'success');
        },
        fail: (err) => {
          throw new Error(err);
        },
      });
    } catch (e) {
      showMessage('导出失败：' + e.resultMessage, 'error');
    }
  };

  // 判断是否有导出权限
  hasExportAuth = () => {
    const { btnAuths } = this.props;
    return btnAuths.includes('VISIT_CLIENT_EXPORT');
  };

  // 获取转介绍详情
  getInviteDetail = (selectedRecord) => {
    const { activityId } = this.props;
    this.setState({
      showVisitClientModal: true,
      selectedRecord,
    });
  };
  render() {
    const { route, activityId } = this.props;
    const {
      loading,
      time,
      contactNo,
      studentName,
      blCampusId,
      campusDict,
      attendSchool,
      classStatus,
      listData,
      showVisitClientModal,
      selectedRecord,
      isInviteSuccess,
      isCreateOrder,
      searchExpand,
    } = this.state;
    return (
      <Page route={route} loading={loading}>
        <SearchBar
          childWidth={[2.5, 1, 1, 1, 1, 1, 1, 1]}
          onSearch={this.search}
          onReset={this.reset}
          className={styles.searchBar}
          expand={searchExpand}
          onToggleExpand={(expand) => this.setState({ searchExpand: expand })}
        >
          <RangePicker
            size="large"
            style={{ width: '312px' }}
            format={dateFormat}
            value={time}
            showTime={{
              hideDisabledOptions: true,
              defaultValue: [moment('00:00:00', 'HH:mm:ss'), moment('23:59:59', 'HH:mm:ss')],
            }}
            placeholder={['开始时间', '结束时间']}
            onChange={(time) => {
              this.setState({ time: time.length === 0 ? null : time });
            }}
          />
          <Input placeholder="客户号码" value={contactNo} maxLength={11} onChange={this.oncontactNoChange} />
          <Input placeholder="学生名称" value={studentName} onChange={this.onStudentNameChange} />
          <Select
            placeholder="意向学习中心"
            dict={campusDict}
            value={attendSchool}
            onChange={this.onAttendSchoolChange}
          />
          <Select placeholder="学籍学习中心" dict={blCampusDict} value={blCampusId} onChange={this.onBlCampusChange} />
          <Select
            placeholder="是否在读"
            dict={classStatusDict}
            value={classStatus}
            onChange={this.onClassStatusChange}
          />
          <Select
            placeholder="是否有转介绍"
            dict={isTransformDict}
            value={isInviteSuccess}
            onChange={this.onTransformChange}
          />
          <Select placeholder="是否下单" dict={isPurchaseDict} value={isCreateOrder} onChange={this.onPurchaseChange} />
          <Button className={styles.button} onClick={this.exportExcel} disabled={!this.hasExportAuth()}>
            导出 Excel
          </Button>
        </SearchBar>
        <List
          dataSource={listData}
          columns={this.columns}
          onChange={this.onPaginationChange}
          pagination={this.pagination}
        />
        {showVisitClientModal && (
          <VisitClientModal
            record={selectedRecord}
            activityId={activityId}
            onClose={() => {
              this.setState({ showVisitClientModal: false, selectedRecord: null });
            }}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  let account = state.account;
  const { activityId, activityName, branchId, productTypeId } = state.main.pageParams.VisitClientPage;
  return {
    activityId,
    activityName,
    btnAuths: state.account.btnAuths,
    branchId,
    productTypeId,
  };
};

export default connect(mapStateToProps)(VisitClientPage);
