import React from 'react';
import Modal from 'COMPONENT/Modal';
import List from 'COMPONENT/List';
import { getInviteDetail } from '../loader';
import Styles from '../../styles.less';
const { COL_WIDTH_S, COL_WIDTH_M } = List;
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import moment from 'moment';

const dateFormat = 'YYYY-MM-DD HH:mm';

const columns = [
  { title: '姓名', dataIndex: 'studentName', width: COL_WIDTH_S },
  { title: '手机号码', dataIndex: 'phone', width: COL_WIDTH_M },
  {
    title: '下单时间',
    dataIndex: 'createOrderTime',
    width: COL_WIDTH_S,
    render: (text) => moment(text).format(dateFormat),
  },
];
export default class InviteClientModal extends React.Component {
  state = {
    loading: false,
    pageNo: 1,
    pageSize: 5,
    total: 0,
    inviteListData: [],
  };

  componentDidMount() {
    this.loadListData();
  }

  pageChange = async ({ current, pageSize }) => {
    await this.updateState({ pageSize, pageNo: current });
    this.loadListData();
  };

  loadListData = async () => {
    const { pageNo, pageSize } = this.state;
    const { record, activityId } = this.props;
    const { studentNo } = record;
    this.setState({
      loading: true,
    });

    try {
      const res = await getInviteDetail({
        activityId,
        pageSize,
        pageNo,
        inviteStudentNo: studentNo,
      });
      const { records, total, current } = res.data;
      const filterRecords = Array.isArray(records) ? records : [];
      this.setState({
        inviteListData: filterRecords,
        total,
      });
    } catch (e) {
      console.log(e);
      showMessage(e.text, 'error');
    }
    this.setState({
      loading: false,
    });
  };
  updateState = (changedState) => {
    const newState = {
      ...this.state,
      ...changedState,
    };
    return new Promise((resolve) => {
      this.setState(newState, () => {
        resolve();
      });
    });
  };

  render() {
    const { loading, inviteListData, pageNo, pageSize, total } = this.state;
    const { onClose } = this.props;

    return (
      <Modal title="转介绍详情" width={715} cancelText="关闭" onCancel={onClose} okBtnShow={false}>
        <div className={Styles.orderStatusModal}>
          <div className={Styles.orderStatusTop}>{`转介绍人数 ${total}`}</div>
          <List
            loading={loading}
            columns={columns}
            dataSource={inviteListData}
            pagination={{
              current: pageNo,
              pageSize,
              total: total,
              pageSizeOptions: ['5', '10', '20'],
            }}
            onChange={this.pageChange}
          />
        </div>
      </Modal>
    );
  }
}
