/**
 * Created by Haolin<haolinhom@gmail.com> on 2019/4/10.
 */
import React from 'react';
import style from './index.less';
import PropTypes from 'prop-types';
import InputNumber from 'COMPONENT/InputNumber';
import Validator from 'COMPONENT/Validator';

class NumRanger extends React.Component {
  constructor(props) {
    super(props);
    this.updateFirstNum = this.updateFirstNum.bind(this);
    this.updateLastNum = this.updateLastNum.bind(this);
  }
  static classPrefix = 'NumRanger';
  static propTypes = {
    value: PropTypes.array.isRequired,
    firstRanger: PropTypes.array,
    lastRanger: PropTypes.array,
    lastText: PropTypes.string,
    isInt: PropTypes.bool,
    disabled: PropTypes.bool,
  };
  static defaultProps = {
    lastText: '',
    isInt: false,
  };
  // 更新value
  updateFirstNum(value) {
    if (this.props.isInt) {
      value = NumRanger.setIntNum(value);
    }
    if (this.props.firstRanger) {
      value = NumRanger.setValInRanger(value, this.props.firstRanger);
    }
    this.props.onChange([value, this.props.value[1]]);
  }
  updateLastNum(value) {
    if (this.props.isInt) {
      value = NumRanger.setIntNum(value);
    }
    if (this.props.lastRanger) {
      value = NumRanger.setValInRanger(value, this.props.lastRanger);
    }
    this.props.onChange([this.props.value[0], value]);
  }
  // 将Number()后非NaN的值转换成整数
  static setIntNum(value) {
    let tempNum = Number(value);
    if (tempNum && value.toString().indexOf('.') !== -1) {
      return Number(tempNum.toFixed(0));
    }
    return value;
  }
  // 返回符合范围内的值
  static setValInRanger(val, ranger) {
    if (typeof ranger[0] === 'number') {
      val = ranger[0] <= val ? val : ranger[0];
    }
    if (typeof ranger[1] === 'number') {
      val = val <= ranger[1] ? val : ranger[1];
    }
    return val;
  }

  render() {
    const { value, lastText, validator, disabled } = this.props;
    const prefix = NumRanger.classPrefix;
    let numRanger = (
      <div className={style[prefix]}>
        <div className={style[`${prefix}-input`]}>
          <InputNumber disabled={disabled} value={value[0]} onChange={this.updateFirstNum} />
        </div>
        <div className={style[`${prefix}-midText`]}>
          <span>~</span>
        </div>
        <div className={style[`${prefix}-input`]}>
          <InputNumber disabled={disabled} value={value[1]} onChange={this.updateLastNum} />
        </div>
        {lastText !== '' && (
          <div className={style[`${prefix}-lastText`]}>
            <span>{lastText}</span>
          </div>
        )}
      </div>
    );
    if (validator) {
      numRanger = (
        <Validator value={value} {...validator}>
          {numRanger}
        </Validator>
      );
    }
    return numRanger;
  }
}

export default NumRanger;
