/**
 * Created by Haolin on 2019/4/9.
 */
import React from 'react';
import { connect } from 'dva';
import style from './index.less';
import Modal from 'COMPONENT/Modal';
// import FormItem from "COMPONENT/FormItem";
import { Form, DatePicker } from 'antd';
import Radio from 'COMPONENT/Radio';
import TextArea from 'COMPONENT/TextArea';
import NoData from 'COMPONENT/NoData';
import Upload from '../Upload';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import styles from './index.less';
import moment from 'moment';
import loader from '../../loader';

const remarkIntention = [
  { text: '低意向', val: 0 },
  { text: '有意向', val: 1 },
  { text: '高意向', val: 2 },
];

class OrderRemarkModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      name: '修改',
      loading: false,
      validator: {},
      validateResult: true,

      addFriends: 0, //微信好友是否已添加
      placementTestTime: '', //分班测试时间
      intention: '', // 家长意向度
      content: '', //跟进内容,
      fileList: [], //上传的图片
      remarkList: [], //跟进记录列表
      isDetailModalVisible: false, //是否显示记录详情弹窗
      remarkDetail: {}, //记录详情
    };
  }
  static classPrefix = 'OrderRemarkModal';

  componentDidMount() {
    const { externalOrderId } = this.props;
    if (externalOrderId) {
      this.getRemarkList(externalOrderId);
    }
  }
  componentWillUnmount() {}

  componentDidUpdate(prev) {
    const { externalOrderId } = this.props;
    if (externalOrderId !== prev.externalOrderId) {
      this.getRemarkList(externalOrderId);
    }
  }

  onUploadChange = (file) => {
    if (file && file.src) {
      // this.setState({ file: file });
      console.log(file);
    }
  };
  beforeUpload = (fileArray, fileList) => {
    const file = fileArray[0];
    const fileType = ['image/jpeg', 'image/png', 'image/jpg'];
    let isJPGOrPNG = fileType.indexOf(file.type) > -1;

    let isLt1m = file.size / 1024 < 300;

    if (!isJPGOrPNG) {
      showMessage('只能上传jpg,png,jpeg文件');
      return false;
    }
    if (!isLt1m) {
      showMessage('图片大小超过300k');
      return false;
    }
    return fileArray;
  };

  //获取跟进记录列表
  getRemarkList = (externalOrderId) => {
    if (!externalOrderId) return;
    const params = {
      size: 10000,
      current: 1,
      params: {
        externalOrderId,
      },
    };
    loader
      .getOrderRemarkList(params)
      .then((res) => {
        this.setState({ remarkList: res || [] });
      })
      .catch((err) => showMessage(err.text, 'error'));
  };

  //添加跟进记录
  addRemark = () => {
    const { externalOrderId, user } = this.props;
    const { addFriends, placementTestTime, intention, content, fileList, loading } = this.state;
    if (loading) {
      return;
    }
    if (!externalOrderId || !user) return;

    // v3.0版本调整为非必选
    // if (!placementTestTime) {
    //   return showMessage("可分班测日期不能为空", "error");
    // }

    this.setState({
      loading: true,
    });
    const params = {
      externalOrderId,
      bossUserId: user.userId,
      addFriends,
      placementTestTime,
      intention,
      content,
      fileList: fileList.map((file) => ({
        fileUrl: file.src,
        fileName: file.name,
      })),
    };
    loader
      .createOrderRemark(params)
      .then(() => {
        showMessage('添加跟进记录成功', 'success');
        this.setState({
          loading: false,
          addFriends: 0,
          placementTestTime: '',
          intention: '',
          content: '',
          fileList: [],
        });
        const { externalOrderId } = this.props;
        this.getRemarkList(externalOrderId);
      })
      .catch((err) => {
        showMessage(err.text, 'error');
        this.setState({
          loading: false,
        });
      });
  };

  //获取跟进记录详情
  getRemarkDetail = (remarkId) => {
    if (!remarkId) return;
    loader
      .getOrderRemarkDetail(remarkId)
      .then((res) => {
        this.setState({ remarkDetail: res || {} });
      })
      .catch((err) => showMessage(err.text, 'error'));
  };

  /**
   * 上传成功
   *
   * @param file {object} OSS图片文件
   * @param index {number} 文件列表索引
   */
  onUploadComplete = (file, index) => {
    let fileList = [...this.state.fileList];
    fileList[index] = { ...ossFile, ...file };

    const canPushEmptyFile = this.props.editable && fileList.length < this.props.maxFileLength;
    if (canPushEmptyFile) {
      fileList.push(ossFile);
    }

    // this.setState({ fileList });
  };

  render() {
    const { onCancel } = this.props;
    const {
      loading,

      addFriends,
      placementTestTime,
      intention,
      content,
      fileList,
      remarkList,
      isDetailModalVisible,
      remarkDetail,
    } = this.state;
    const prefix = OrderRemarkModal.classPrefix;
    return (
      <Modal
        width={1200}
        title="请填写跟进记录"
        className={style[prefix]}
        loading={loading}
        okText="保存"
        onOk={() => {
          this.addRemark();
        }}
        onCancel={() => {
          //重置数据
          this.setState({
            addFriends: 0,
            placementTestTime: '',
            intention: '',
            content: '',
            fileList: [],
          });
          onCancel();
        }}
      >
        <div className={styles.orderRemarkModal}>
          <div className={styles.formWrap}>
            <Form.Item label="微信好友是否已添加" required>
              <Radio.Group
                onChange={(e) => {
                  this.setState({
                    addFriends: e.target.value,
                  });
                }}
                value={addFriends}
              >
                <Radio value={1}>是</Radio>
                <Radio value={0}>否</Radio>
              </Radio.Group>
            </Form.Item>
            <Form.Item label="可分班测日期">
              <DatePicker
                className={styles.datePicker}
                onChange={(value) => {
                  this.setState({
                    placementTestTime: value ? `${value.format('YYYY-MM-DD')} 00:00:00` : '',
                  });
                }}
                value={placementTestTime ? moment(placementTestTime) : null}
              />
            </Form.Item>
            <Form.Item label="家长意向度">
              <Radio.Group
                onChange={(e) => {
                  this.setState({
                    intention: e.target.value,
                  });
                }}
                value={intention}
              >
                {remarkIntention.map((ite) => (
                  <Radio value={ite.val} key={ite.val}>
                    {ite.text}
                  </Radio>
                ))}
              </Radio.Group>
            </Form.Item>
            <Form.Item label="跟进内容">
              <TextArea
                className={styles.remarkContent}
                rows={3}
                value={content}
                maxLength={1000}
                wordCount
                onChange={(e) => {
                  this.setState({ content: e.target.value });
                }}
              />
            </Form.Item>
            <Form.Item label="跟进截图 (支持格式：jpg png jpeg，图片大小不超过300K)">
              <Upload
                value={fileList}
                text="上传图片"
                size={100}
                max={10}
                maxFileSize={300}
                multiple
                fileType={['png', 'jpg', 'jpeg']}
                uploadComplete={(newFileList) => {
                  console.log(newFileList);
                  this.setState({
                    fileList: newFileList,
                  });
                }}
                deleteComplete={(list) => {
                  this.setState({
                    fileList: list,
                  });
                }}
              />
            </Form.Item>
          </div>
          <div className={styles.remarkWrap}>
            <span className={styles.secondTitle}>跟进动态</span>
            {remarkList.map((obj, index) => (
              <div key={index} className={styles.remarkListItem}>
                <div className={styles.userWrap}>
                  <span className={styles.userName}>{obj.bossUserName}</span>
                  <span className={styles.remarkTime}>
                    {obj.updateTime}
                    <span
                      className={styles.checkDetailButton}
                      onClick={() => {
                        this.setState({
                          isDetailModalVisible: true,
                        });
                        //获取记录详情
                        this.getRemarkDetail(obj.remarkId);
                      }}
                    >
                      查看详情
                    </span>
                  </span>
                </div>
                <div className={styles.contentWrap}>
                  <span className={styles.userName}>备注</span>
                  <span
                    className={styles.remarkTime}
                    dangerouslySetInnerHTML={{
                      __html: (obj.content || '').replace(/\n/g, '<br/>').replace(/\s/g, '&nbsp;'),
                    }}
                  ></span>
                </div>
              </div>
            ))}
            {!remarkList.length && <NoData />}
          </div>
        </div>
        {isDetailModalVisible && (
          <Modal
            width={690}
            title="跟进记录详情"
            className={style[prefix]}
            okText="我知道了"
            cancelBtnShow={false}
            onOk={() => {
              this.setState({
                isDetailModalVisible: false,
                remarkDetail: {},
              });
            }}
            onCancel={() => {
              this.setState({
                isDetailModalVisible: false,
                remarkDetail: {},
              });
            }}
          >
            <span className={styles.secondTitle}>跟进信息</span>
            <div className={styles.detailTimeWrap}>
              <div className={styles.detailAddFriend}>
                <span>微信好友是否已添加</span>
                <span className={styles.detailAddFriendValue}>{remarkDetail.addFriends ? '是' : '否'}</span>
              </div>
              <div className={styles.detailAddFriend}>
                <span>可分班测日期</span>
                <span className={styles.detailAddFriendValue}>
                  {remarkDetail.placementTestTime && remarkDetail.placementTestTime.split(' ')[0]}
                </span>
              </div>
              <div className={styles.detailAddFriend}>
                <span>家长意向度</span>
                <span className={styles.detailAddFriendValue}>
                  {typeof remarkDetail.intention === 'number'
                    ? remarkIntention[remarkDetail.intention] && remarkIntention[remarkDetail.intention].text
                    : '暂无'}
                </span>
              </div>
            </div>
            <div className={styles.detailContentSection}>
              <p className={styles.detailSectionTitle}>跟进内容</p>
              <span
                className={styles.detailContent}
                dangerouslySetInnerHTML={{
                  __html: (remarkDetail.content || '').replace(/\n/g, '<br/>').replace(/\s/g, '&nbsp;'),
                }}
              ></span>
              {!remarkDetail.content && <span className={styles.detailAddFriendValue}>暂无</span>}
            </div>
            <div className={styles.detailImgSection}>
              <p className={styles.detailSectionTitle}>跟进截图</p>
              <div className={styles.detailImgWrap}>
                {(remarkDetail.fileList || []).map((file, index) => (
                  <img key={index} className={styles.detailImg} src={file.fileUrl} />
                ))}
                {(!remarkDetail.fileList || !remarkDetail.fileList.length) && (
                  <span className={styles.detailAddFriendValue}>暂无</span>
                )}
              </div>
            </div>
          </Modal>
        )}
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  return {};
};

export default connect(mapStateToProps)(OrderRemarkModal);
