import React from 'react';
import { connect } from 'dva';
import Button from 'COMPONENT/Button';
import Page from 'COMPONENT/Page';
import List from 'COMPONENT/List';
import Input from 'COMPONENT/Input';
import InputNumber from 'COMPONENT/InputNumber';
import FormItem from 'COMPONENT/FormItem';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import TextLink from 'COMPONENT/TextLink';
import TextArea from 'COMPONENT/TextArea';
import DropdownMenu from 'COMPONENT/DropdownMenu';
import CheckBox from 'COMPONENT/Checkbox';
import Popover from 'COMPONENT/Popover';
import Icon from 'COMPONENT/Icon';
import { Form, DatePicker, Row, Col, Switch } from 'antd';
import ProductModal from './modal/ProductModal';
import PeopleLimitModal from './modal/PeopleLimitModal';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { renderMultiLine } from '../common';
import { loadCommonDict, loadBranchDict, loadBranchCampusDict } from 'UTIL/dict';
import loader from './loader';
import moment from 'moment';
import { formCfg } from '../common';
import commonStyles from '../common/commonStyle.less';
import styles from './styles.less';
import NumRanger from './components/NumRanger';

const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L } = List;
const { RangePicker } = DatePicker;
const dateFormat = 'YYYY-MM-DD HH:mm';
const FormItemNative = Form.Item;
const productClassTypeToActivityOjb = {
  OTHERS: '22e136054e24a09093bf39348b5a24f8',
  ONE_ON_ONE_COURSE: '27a8bc8df744f6d04a01ecc6837af6de',
  LECTURE: '39cbf184238405128f3b3d5259554a6f',
  ONE_ON_MANY: '55fbeaf98761de46686a607ca540bd11',
  SMALL_CLASS: 'fe53122a5ceb46fb5340af911eb5e027',
  TWO_TEACHER: 'bc6d66557ea5b6967d81be5ff9c5cf32',
};
import { listAllQuestionnaires, selectAnswerCount } from 'SERVICE/marketActivityManage';

// 报名 退费模板
const tempType = {
  marketOrderSms: 'marketOrderSms',
  marketGrouponOrderSms: 'marketGrouponOrderSms',
  marketRefundSms: 'marketRefundSms',
};

let activityTypeObj = {};
let branchObj = {};

const checkLen = (value) => {
  value = value || '';
  if (value.length > 20) {
    return `不能超过30个字`;
  }
  return null;
};

const toFloat2 = (val) => {
  return Math.round(val * 100) / 100;
};

class MarketActivityDetail extends React.Component {
  constructor(props) {
    super(props);
    this.updateNumRanger = this.updateNumRanger.bind(this);
    this.handleValidator = this.handleValidator.bind(this);
    this.validateData = this.validateData.bind(this);
    this.loadQuestionnaireDict = this.loadQuestionnaireDict.bind(this);
    this.filterInvalidQuestionnaire = this.filterInvalidQuestionnaire.bind(this);
    this.updateQuestionnaire = this.updateQuestionnaire.bind(this);
    this.state = {
      loading: true,
      checking: false,
      activity: {},
      activityTypeId: null,
      activityProductList: [],
      productModalVisible: false,
      firstLimitNum: 1,
      lastLimitNum: 2,
      // 每单产品数是否可以输入 一对一 一对多不能输入 只能写死为1
      isNumInputDisabled: false,
      questionnaireId: null,
      // 是否不可更改更多信息
      questionDisabled: false,
      // 报名退费模板以及code
      marketOrderSmsTemp: null,
      marketOrderSmsCode: null,
      marketRefundSmsTemp: null,
      marketRefundSmsCode: null,
      // 拼团报名模板
      marketGrouponOrderSmsTemp: null,
      marketGrouponOrderSmsCode: null,
      // 是否拼团
      isFightGroup: false,
      // 拼团人数
      fightGroupNum: null,
      validator: {},
      validateResult: true,
      // 双师的分享文案
      twoTeacherShareMsg: null,
      // 是否开启凑团
      enableJoinGroupon: 1,
      //数据字典数据
      tempObject: {},
      branchDict: [],
      campusDict: [],
      promotionDict: [],
      yearDict: [],
      questionnaireDict: null,
    };
    this.detailObj = {};
    this.errors = {};
  }

  componentDidMount() {
    this.initContent();
  }

  initContent = async () => {
    this.setState({ loading: true });
    const { activity } = this.props.params;
    const { user } = this.props;
    const asyncCalls = [
      loadCommonDict('PRODUCT_VERSION'),
      loadBranchDict(),
      loadCommonDict('MARKET_ACTIVITY_TYPE'),
      loader.loadPromotionList({ organizationId: user.organizationId }),
      loader.loadMsgTemp(),
    ];
    if (activity) {
      asyncCalls.push(loader.loadDetail({ id: activity.id }));
    }
    const [yearDict, branchDict, activityTypeDict, promotionDict, msgTempDict, detailRes] = await Promise.all(
      asyncCalls,
    );
    let orgId = activity ? activity.branchId : branchDict[0] && branchDict[0].val;
    // 加载更多信息字典
    const questionnaireDict = await new Promise(this.loadQuestionnaireDict(orgId, activity ? null : 0));
    activityTypeObj = this.dictToObj(activityTypeDict);
    branchObj = this.dictToObj(branchDict);
    const tempObject = Object.keys(msgTempDict).reduce((obj, key) => {
      const childMap = msgTempDict[key];
      obj[key] = Object.keys(childMap).map((childKey) => ({ code: childKey, text: childMap[childKey] }));
      return obj;
    }, {});
    //默认模板数据
    const defaultTemp = {
      marketOrderSmsCode: tempObject[tempType.marketOrderSms][0].code,
      marketOrderSmsTemp: tempObject[tempType.marketOrderSms][0].text,
      marketRefundSmsCode: tempObject[tempType.marketRefundSms][0].code,
      marketRefundSmsTemp: tempObject[tempType.marketRefundSms][0].text,
      marketGrouponOrderSmsTemp: tempObject[tempType.marketGrouponOrderSms][0].text,
      marketGrouponOrderSmsCode: tempObject[tempType.marketGrouponOrderSms][0].code,
    };
    this.setState({
      yearDict,
      branchDict,
      promotionDict: promotionDict.filter((ele) => ele),
      tempObject,
      ...defaultTemp,
    });
    if (detailRes) {
      this.detailObj = detailRes.data.activityProductList.reduce((obj, ele) => {
        obj[ele.product.id] = ele;
        return obj;
      }, {});
      const { branchId, productId, beginTime, endTime, blCampusId, name, activityTypeId } = activity;
      const {
        orderSmsTemplate,
        refundSmsTemplate,
        grouponOrderSmsTemplate,
        enableGroupon,
        grouponPeopleNum,
        enableJoinGroupon,
      } = detailRes.data;
      // 是否拼团
      const isFightGroup = enableGroupon === 1;

      let editDefaultCode = {};
      let campusDict = await this.getCapmusDict(branchId);
      let activityProductList = (await loader.loadActivityProductList({ productIds: productId })) || [];
      activityProductList = await Promise.all(
        activityProductList.map((ele) => {
          const { promotionId, totalHour, pepoleLimit, inventoryList, campusOrderStudents, grouponPromotionId } =
            this.detailObj[ele.id];
          ele.selectedPromotionId = promotionId;
          ele.selectGroupPromotionId = grouponPromotionId;
          ele.totalCourseCount = ele.miniClassTotalhours ? ele.miniClassTotalhours : totalHour;
          ele.pepoleLimit = pepoleLimit;
          ele.inventoryList = inventoryList || [];
          ele.campusOrderStudents = campusOrderStudents || [];
          ele = this.calcRealPayAmount(ele);
          return ele;
        }),
      );
      this.props.form.setFieldsValue({
        range: [moment(beginTime), moment(endTime)],
        blCampusId: blCampusId.split(','),
        branchId: branchId,
        name: name,
      });
      // 编辑详情模板id
      if (orderSmsTemplate && refundSmsTemplate) {
        editDefaultCode = {
          marketOrderSmsCode: orderSmsTemplate,
          marketOrderSmsTemp: msgTempDict[tempType.marketOrderSms][orderSmsTemplate],
          marketRefundSmsCode: refundSmsTemplate,
          marketRefundSmsTemp: msgTempDict[tempType.marketRefundSms][refundSmsTemplate],
          marketGrouponOrderSmsCode: grouponOrderSmsTemplate,
          marketGrouponOrderSmsTemp: msgTempDict[tempType.marketGrouponOrderSms][grouponOrderSmsTemplate],
        };
      }
      let payload = {
        activity: activity,
        activityProductList: activityProductList,
        activityTypeId: activityTypeId,
        campusDict,
        loading: false,
        ...editDefaultCode,
      };
      // 双师的话有分享文案填写
      if (detailRes.data.grouponInviteTitle) {
        payload.twoTeacherShareMsg = detailRes.data.grouponInviteTitle;
      }
      if (isFightGroup) {
        payload.isFightGroup = isFightGroup;
        payload.fightGroupNum = grouponPeopleNum;
        payload.enableJoinGroupon = enableJoinGroupon;
      }
      if (detailRes.data.maxProductNum && detailRes.data.miniProductNum) {
        payload.firstLimitNum = detailRes.data.miniProductNum;
        payload.lastLimitNum = detailRes.data.maxProductNum;
      }
      if (detailRes.data.questionnaireId) {
        payload.questionnaireId = detailRes.data.questionnaireId.toString();
        payload.questionnaireDict = questionnaireDict.filter(
          (item) => payload.questionnaireId === item.val || item.status === 0,
        );
        const rsp = await selectAnswerCount({
          activityId: activity.id,
          id: payload.questionnaireId,
        }).catch((err) => showMessage(err.text, 'error'));
        if (rsp && rsp.resultCode === 0) {
          payload.questionDisabled = rsp.data;
        }
      } else {
        payload.questionnaireDict = questionnaireDict.filter((item) => item.status === 0);
      }
      this.setState(payload);
    } else {
      let campusDict = await this.getCapmusDict(branchDict[0] && branchDict[0].val);
      this.props.form.setFieldsValue({
        blCampusId: [campusDict[0] && campusDict[0].val],
        branchId: branchDict[0] && branchDict[0].val,
      });
      this.setState({
        questionnaireDict: questionnaireDict.filter((item) => item.status === 0),
        campusDict,
        loading: false,
      });
    }
  };

  loadQuestionnaireDict(orgId, status = null) {
    return (resolve, reject) => {
      listAllQuestionnaires({
        orgIds: [orgId],
        status,
        whetherPaging: false,
      })
        .then((rsp) => {
          resolve(
            rsp.rows.map((item) => ({
              text: item.name,
              val: item.id.toString(),
              status: item.status,
            })),
          );
        })
        .catch((err) => {
          showMessage(err.text, 'error');
          reject([]);
        });
    };
  }

  filterInvalidQuestionnaire(dict, exceptionVal) {
    return dict.filter((item) => exceptionVal === item.val || item.status === 0);
  }

  getCapmusDict = async (branchId) => {
    let campusDict = (await loadBranchCampusDict()) || [];
    if (branchId) {
      campusDict = campusDict.filter((ele) => ele.val === branchId);
    }
    campusDict = campusDict.reduce((arr, ele) => {
      return [...arr, ...ele.options];
    }, []);
    return campusDict;
  };

  onBranchChange = async (branchId) => {
    this.setState({ loading: true });
    const campusDict = !branchId ? [] : await this.getCapmusDict(branchId);
    const questionnaireDict = !branchId ? [] : await new Promise(this.loadQuestionnaireDict(branchId, 0));
    this.setState({
      campusDict,
      questionnaireDict,
      questionnaireId: null,
      loading: false,
    });
    this.props.form.setFieldsValue({
      blCampusId: null,
    });
  };

  dictToObj = (dict) => {
    return (dict || []).reduce((obj, ele) => {
      obj[ele.val] = ele.text;
      return obj;
    }, {});
  };

  updateProductList = async (productList) => {
    if (!productList.length) {
      this.setState({
        activityProductList: [],
        activityTypeId: null,
      });
      return false;
    }
    this.setState({ loading: true });
    const { activityProductList, promotionDict } = this.state;
    const activityProductListObj = (activityProductList || []).reduce((obj, ele) => {
      obj[ele.id] = ele;
      return obj;
    }, {});
    productList = await Promise.all(
      productList.map(async (ele) => {
        if (!activityProductListObj[ele.id]) {
          const promotionIds = ele.promotionId ? ele.promotionId.split(',') : [];
          ele['selectedPromotionId'] = +ele.promotionId === -1 ? promotionDict[0]['val'] : promotionIds[0];
          // 拼团优惠
          ele.selectGroupPromotionId = null;
          // 如果是小班 添加时的课时九尾小班的课时数 不然就是默认为1
          ele.totalCourseCount = ele.miniClassTotalhours ? ele.miniClassTotalhours : 1;
          // 添加人数限制参数 默认为-1即不限制
          ele.pepoleLimit = ele.pepoleLimit ? ele.pepoleLimit : -1;
          // 产品的学习中心限制人数 默认为空数组即共用
          ele.inventoryList = ele.inventoryList ? ele.inventoryList : [];
          // 产品的学习中心报名人数
          ele.campusOrderStudents = ele.campusOrderStudents ? ele.campusOrderStudents : [];
          return await this.calcRealPayAmount(ele);
        }
        return Object.assign({}, ele, activityProductListObj[ele.id]);
      }),
    );
    // 如果是一对一或者一对多的话 那么每单产品数都必须为1
    // 因为一对一一对多只每个年级只允许购买一个产品
    const limitOneProduct = ['ONE_ON_ONE_COURSE', 'ONE_ON_MANY'];
    let extraParams = {};
    if (limitOneProduct.indexOf(productList[0].category) > -1) {
      extraParams = {
        firstLimitNum: 1,
        lastLimitNum: 1,
        isNumInputDisabled: true,
      };
    } else {
      extraParams = {
        firstLimitNum: 1,
        lastLimitNum: 2,
        isNumInputDisabled: false,
      };
    }
    this.setState({
      activityProductList: productList,
      activityTypeId: productClassTypeToActivityOjb[productList[0].category],
      loading: false,
      ...extraParams,
    });
  };

  openActivityPage = () => {
    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path: '/MarketActivity',
        refresh: true,
      },
    });
  };
  //
  handleSubmit = async (e) => {
    const { editable } = this.props.params;
    e.preventDefault();
    this.setState({ loading: true });
    await this.validateData();
    const { checking, activity } = this.state;
    try {
      if (!checking) {
        this.setState({ checking: true });
      }
      if (Object.values(this.errors).filter((ele) => ele).length || !this.state.validator) {
        throw new Error('保存失败：请确认必填输入项是否填写完毕');
      }
      this.props.form.validateFieldsAndScroll(async (err, values) => {
        try {
          if (err) {
            throw new Error('保存失败：请确认必填输入项是否填写完毕');
          }
          const params = this.genActivityParams(values);
          await (activity.id ? this.updateActivity(params) : this.addActivity(params));
          this.setState({ loading: false });
          showMessage('保存成功', 'success');
          // 不存在id就是新增 保存完回到列表页
          if (!activity.id) {
            this.openActivityPage();
          }
        } catch (error) {
          this.setState({ loading: false });
          showMessage(error.message, 'error');
        }
      });
    } catch (error) {
      console.log(error);
      showMessage(error.message, 'error');
      this.setState({ loading: false });
    }
  };

  genActivityParams = (formValues) => {
    formValues = {
      ...formValues,
      beginTime: formValues.range[0].format(dateFormat),
      endTime: formValues.range[1].format(dateFormat),
    };
    delete formValues.range;
    const {
      activity,
      activityTypeId,
      activityProductList,
      firstLimitNum,
      lastLimitNum,
      marketOrderSmsCode,
      marketRefundSmsCode,
      marketGrouponOrderSmsCode,
      questionnaireId,
      isFightGroup,
      fightGroupNum,
      twoTeacherShareMsg,
      enableJoinGroupon,
    } = this.state;
    const rawDatas = Object.assign({}, activity, formValues, { activityTypeId });
    const params = {
      activityTypeId: rawDatas.activityTypeId,
      beginTime: rawDatas.beginTime,
      blCampusId: (rawDatas.blCampusId || []).join(','),
      branchId: rawDatas.branchId,
      endTime: rawDatas.endTime,
      name: rawDatas.name,
      activityProductList: [],
      productId: '',
      productTypeId: '',
      orderSmsTemplate: marketOrderSmsCode,
      refundSmsTemplate: marketRefundSmsCode,
      enableGroupon: isFightGroup ? 1 : 0,
      grouponPeopleNum: fightGroupNum,
      enableJoinGroupon: enableJoinGroupon || 0,
    };
    // 如果是拼团的话 要吧拼团模板加入 如果是普通的话则不需要
    if (isFightGroup) {
      params.grouponOrderSmsTemplate = marketGrouponOrderSmsCode;
    }
    if (twoTeacherShareMsg) {
      params.grouponInviteTitle = twoTeacherShareMsg;
    }
    // 拼团或者优惠金额都不能小于0
    const invalidProductList = activityProductList.filter(
      (ele) => ele.realPayAmountCalced < 0 || ele.groupPayAmountCalced < 0,
    );
    if (invalidProductList.length) {
      throw new Error(`当前产品：“${invalidProductList[0].name}”所选优惠方案大于产品金额，请重新选择优惠方案!`);
    }
    if (activity.id) {
      params['id'] = activity.id;
      params['createUserId'] = activity.createUserId;
    }
    params.productId = activityProductList
      .map((ele) => {
        const promotionIds = ele.promotionId ? ele.promotionId.split(',') : [];
        let activityProduct = {
          product: {
            id: ele.id,
          },
          productTypeId: ele.productCategoryId,
          promotionId: ele.selectedPromotionId !== undefined ? ele.selectedPromotionId : promotionIds[0],
          totalAmount: toFloat2((ele.realPayAmount || 0) * 100),
          totalDiscount: toFloat2((ele.promotionAmount || 0) * 100),
          totalHour: ele.category === 'SMALL_CLASS' ? void 0 : ele.totalCourseCount,
          pepoleLimit: ele.pepoleLimit,
          inventoryList: ele.inventoryList,
          campusOrderStudents: ele.campusOrderStudents,
        };
        // 如果拼团的话 需要加上拼团优惠 以及拼团人数
        if (isFightGroup) {
          if (!fightGroupNum || +fightGroupNum < 2) {
            throw new Error('拼团人数不能小于2');
          }
          activityProduct = {
            ...activityProduct,
            grouponAmount: toFloat2((ele.groupPayAmount || 0) * 100),
            grouponDiscount: toFloat2((ele.groupPromotionAmount || 0) * 100),
            grouponPromotionId: ele.selectGroupPromotionId !== undefined ? ele.selectGroupPromotionId : promotionIds[0],
          };
        }

        if (activity.id) {
          activityProduct['activityId'] = activity.id;
          activityProduct['id'] = this.detailObj[ele.id] ? this.detailObj[ele.id].id : '';
        }
        params.activityProductList.push(activityProduct);
        return ele.id;
      })
      .join(',');
    if (!activityProductList.length) {
      // 未选择产品
      throw new Error('请选择报读产品');
    }
    const _productTypeId = activityProductList[0].category;
    const shouldCheckProductNum = !(_productTypeId === 'LECTURE' || _productTypeId === 'OTHERS');
    // 校验所选产品中，各年级的产品数量大于等于每单产品数的最小值 其他和讲座不需要检验
    let checkPrdGradeList = MarketActivityDetail.arrSortByItemKey(activityProductList, 'gradeId');
    if (shouldCheckProductNum && checkPrdGradeList.some((gradeList) => gradeList.length < firstLimitNum)) {
      throw new Error('有年级达不到每单产品数的最小值，请继续添加产品！');
    }
    params.productTypeId = _productTypeId;
    // 每单产品数参数
    params.miniProductNum = firstLimitNum;
    params.maxProductNum = lastLimitNum;
    params.questionnaireId = questionnaireId ? Number(questionnaireId) : null;
    return params;
  };

  // 将数组中的对象按指定key名分类，返回一个新的数组[[xx], [yy], ...]
  static arrSortByItemKey(arr, keyName) {
    let _storage = {};
    arr.forEach((item) => {
      if (_storage[item[keyName]]) {
        _storage[item[keyName]].push(item);
      } else {
        _storage[item[keyName]] = [item];
      }
    });
    return Object.values(_storage);
  }

  addActivity = (params) => {
    return loader.add(params);
  };

  updateActivity = (params) => {
    return loader.edit(params);
  };

  updateRow = async (record, changed, isCalcAmount = true) => {
    this.setState({ loading: true });
    const activityProductList = await Promise.all(
      this.state.activityProductList.map(async (row, i) => {
        if (row.id === record.id) {
          // 修改人数限制不需要重新计算金额 修改课时数和优惠才需要
          return isCalcAmount ? await this.calcRealPayAmount({ ...row, ...changed }) : { ...row, ...changed };
        }
        return row;
      }),
    );
    this.setState({
      loading: false,
      activityProductList,
    });
  };

  removeRow = (index, record) => {
    const activityProductList = this.state.activityProductList.filter((row, i) => i !== index);
    if (!activityProductList.length) {
      this.setState({
        activityProductList,
        activityTypeId: null,
      });
    } else {
      this.setState({
        activityProductList,
      });
    }
  };

  errorChange = (key, error) => {
    this.errors[key] = error;
  };

  calcRealPayAmount = async (product) => {
    // 计算优惠金额 勾选了拼团的话 需要计算两个优惠金额
    let {
      price,
      promotionId,
      sonProduct,
      totalCourseCount,
      selectedPromotionId,
      selectGroupPromotionId,
      isCalcGroupAmount,
    } = product;
    const { isFightGroup } = this.state;
    const promotionIds = promotionId ? promotionId.split(',') : [];
    let sonProductTotalPrice = 0;
    if (sonProduct.length) {
      sonProductTotalPrice = sonProduct
        .map((product) => product.price)
        .reduce((totalAmount, price) => {
          return totalAmount + price;
        }, 0);
    }
    let returnParams = {};
    if (!selectedPromotionId) {
      returnParams = {
        realPayAmountCalced: 0,
        realPayAmount: toFloat2(price * totalCourseCount + sonProductTotalPrice),
        promotionAmount: 0,
      };
    } else {
      const res = await loader.calculatePromotion({
        price: price,
        totalCourseCount: totalCourseCount,
        totalAmount: toFloat2(price * totalCourseCount),
        promotionIds: selectedPromotionId || promotionIds[0],
      });
      returnParams = {
        realPayAmountCalced: res.realPayAmount,
        realPayAmount: toFloat2((res.realPayAmount >= 0 ? res.realPayAmount : 0) + sonProductTotalPrice),
        promotionAmount: toFloat2(res.realPayAmount >= 0 ? res.promotionAmount : price * totalCourseCount),
      };
    }
    // 拼团 初始化的新添加的产品需要计算金额 就是没有isCalcGroupAmount金额的也要初始化金额 这是一个标示位
    // 及时不是拼团也可以进行计算
    if (isFightGroup || !isCalcGroupAmount) {
      if (!selectGroupPromotionId) {
        returnParams = {
          ...returnParams,
          groupPayAmountCalced: 0,
          groupPayAmount: toFloat2(price * totalCourseCount + sonProductTotalPrice),
          groupPromotionAmount: 0,
        };
      } else {
        const res = await loader.calculatePromotion({
          price: price,
          totalCourseCount: totalCourseCount,
          totalAmount: toFloat2(price * totalCourseCount),
          promotionIds: selectGroupPromotionId || promotionIds[0],
        });
        returnParams = {
          ...returnParams,
          isCalcGroupAmount: true,
          groupPayAmountCalced: res.realPayAmount,
          groupPayAmount: toFloat2((res.realPayAmount >= 0 ? res.realPayAmount : 0) + sonProductTotalPrice),
          groupPromotionAmount: toFloat2(res.realPayAmount >= 0 ? res.promotionAmount : price * totalCourseCount),
        };
      }
    }
    return {
      ...product,
      ...returnParams,
    };
  };

  columns = [
    { title: '序号', dataIndex: 'index', width: 50, fixed: 'left', render: (text, record, index) => index + 1 },
    { title: '产品类型', dataIndex: 'productCategoryName', width: COL_WIDTH_S },
    { title: '年级', dataIndex: 'gradeName', width: COL_WIDTH_S },
    { title: '产品名称', dataIndex: 'name', width: COL_WIDTH_L },
    { title: '课时单价（元）', dataIndex: 'price', width: COL_WIDTH_S },
    {
      title: '关联其他产品',
      key: 'sonProduct',
      width: COL_WIDTH_L,
      render: (text, record, index) => {
        const { sonProduct } = record;
        const sonProductNameArrayString = sonProduct
          .map((product) => `${product.name}（${product.price}元）`)
          .join(',');
        return renderMultiLine(sonProductNameArrayString);
      },
    },
    {
      title: '课时数',
      dataIndex: 'miniClassTotalhours',
      width: COL_WIDTH_S,
      render: (text, record, index) => {
        const { miniClassTotalhours, totalCourseCount } = record;
        const { editable } = this.props.params;
        const { activity } = this.state;
        return miniClassTotalhours ? (
          miniClassTotalhours
        ) : (
          <InputNumber
            disabled={!editable || !!+activity.orderNum}
            placeholder="请输入课时数"
            value={totalCourseCount}
            onChange={(val) => this.updateRow(record, { totalCourseCount: val }, false)}
            onBlur={(e) => this.updateRow(record, { totalCourseCount: Math.floor(e.target.value) || 1 })}
            min={1}
            max={1000}
            step={1}
          />
        );
      },
    },
    {
      title: '活动模式',
      key: 'insightMode',
      render: (text, record, index) => {
        const { activity } = this.state;
        return +activity.insightMode === 1 ? '组合模式' : '普通模式';
      },
    },
    {
      title: '基本优惠',
      key: 'promotion',
      render: (text, record, index) => {
        const { editable } = this.props.params;
        const { activity } = this.state;
        let { promotionName, promotionId, selectedPromotionId } = record;
        // 组合模式 基本优惠 拼团优惠，支付置空
        if (+activity.insightMode === 1) {
          return '';
        }

        let dict = [{ text: '', val: '' }];
        if (+promotionId !== -1) {
          promotionName = promotionName ? promotionName.split(',') : [];
          promotionId = promotionId ? promotionId.split(',') : [];
          dict = !promotionName.length
            ? dict
            : promotionName.map((ele, i) => {
                return {
                  text: ele,
                  val: promotionId[i],
                };
              });
        } else {
          dict = this.state.promotionDict;
        }
        selectedPromotionId = selectedPromotionId !== undefined ? selectedPromotionId : dict[0] ? dict[0]['val'] : '';

        return (
          <Select
            disabled={!editable || !!+activity.orderNum}
            showSearch={true}
            allowClear={true}
            dict={dict}
            value={selectedPromotionId}
            placeholder={selectedPromotionId && '优惠已删除'}
            onChange={(val) => this.updateRow(record, { selectedPromotionId: val })}
          />
        );
      },
    },
    {
      title: '拼团优惠',
      key: 'groupPromotion',
      render: (text, record, index) => {
        const { editable } = this.props.params;
        const { activity, isFightGroup } = this.state;
        // 如果没拼团的话
        if (!isFightGroup) return null;
        let { promotionName, promotionId, selectGroupPromotionId } = record;
        // 组合模式 基本优惠 拼团优惠，支付置空
        if (+activity.insightMode === 1) {
          return null;
        }
        let dict = [{ text: '', val: '' }];
        if (+promotionId !== -1) {
          promotionName = promotionName ? promotionName.split(',') : [];
          promotionId = promotionId ? promotionId.split(',') : [];
          dict = !promotionName.length
            ? dict
            : promotionName.map((ele, i) => {
                return {
                  text: ele,
                  val: promotionId[i],
                };
              });
        } else {
          dict = this.state.promotionDict;
        }
        return (
          <Select
            disabled={!editable || !!+activity.orderNum}
            showSearch={true}
            allowClear={true}
            dict={dict}
            value={selectGroupPromotionId}
            placeholder={selectGroupPromotionId && '优惠已删除'}
            onChange={(val) => this.updateRow(record, { selectGroupPromotionId: val })}
          />
        );
      },
    },
    {
      title: '实付金额',
      dataIndex: 'realPayAmount',
      width: COL_WIDTH_M,
      render: (text, record) => {
        const { isFightGroup, activity } = this.state;
        const { realPayAmount, groupPayAmount } = record;
        // 组合模式 基本优惠 拼团优惠，支付置空
        if (+activity.insightMode === 1 && isFightGroup) {
          return null;
        }
        return isFightGroup ? `${realPayAmount}/${groupPayAmount}` : realPayAmount;
      },
    },
    {
      title: '人数限制',
      dataIndex: 'pepoleLimit',
      width: COL_WIDTH_S,
      render: (text, record, index) => {
        const { editable } = this.props.params;
        const { pepoleLimit, inventoryList } = record;
        //const hasSignPeople = (pepoleLimit - inventory) <= 0 ? 0 : (pepoleLimit - inventory)
        // 修改人数限制不需要重新计算金额 所以第三个参数false
        return (
          <span>
            {pepoleLimit === -1 ? '' : pepoleLimit}
            <TextLink
              className="list-btn"
              onClick={() => this.setState({ peopleLimitModal: true, selectedProductRecord: record })}
            >
              {editable ? (pepoleLimit === -1 ? '未设置' : ' 已设置') : '查看'}
            </TextLink>
          </span>
        );
      },
    },
    {
      title: '操作',
      key: 'action',
      fixed: 'right',
      render: (text, record, index) => {
        const { activity } = this.state;
        const { editable } = this.props.params;
        return !(editable && !+activity.orderNum) ? (
          ''
        ) : (
          <span>
            <TextLink className="list-btn" onClick={() => this.removeRow(index, record)}>
              删除
            </TextLink>
          </span>
        );
      },
    },
  ];

  static numRanger = [1, 1000];

  static setFirstNumRanger(lastNum, ranger) {
    return typeof lastNum === 'number' ? [ranger[0], lastNum] : [ranger[0], ranger[1]];
  }

  static setLastNumRanger(firstNum, ranger) {
    return typeof firstNum === 'number' ? [firstNum, ranger[1]] : [ranger[0], ranger[1]];
  }

  // 更新每单产品数
  updateNumRanger(valArr) {
    this.setState({
      firstLimitNum: valArr[0],
      lastLimitNum: valArr[1],
    });
  }

  //
  static validateNumRanger([firstLimitNum, lastLimitNum]) {
    let checkFirst = typeof firstLimitNum === 'number';
    let checkLast = typeof lastLimitNum === 'number';
    if (!checkFirst && !checkLast) {
      return '必填项';
    } else if (!checkFirst) {
      return '第一项不能为空';
    } else if (!checkLast) {
      return '第二项不能为空';
    } else {
      return null;
    }
  }

  // 处理表单校验值更新
  handleValidator({ type, val }) {
    this.setState((prevState) => ({
      validator: {
        ...prevState.validator,
        [type]: val,
      },
    }));
  }

  // 校验数据
  validateData() {
    return new Promise((resolve) => {
      this.setState(
        {
          validateResult: Object.values(this.state.validator).every((item) => !Boolean(item)),
        },
        () => {
          resolve();
        },
      );
    });
  }

  /*
   * 报名模板方法
   * */
  selectMsgTemp = (text, code, type) => {
    // 需要更新的两个变量键名 marketRefundSms 和 marketOrderSms
    const codeKey = type + 'Code';
    const TextKey = type + 'Temp';
    this.setState({
      [codeKey]: code,
      [TextKey]: text,
    });
  };

  buildMsgTempMenus = (menus, type) => {
    if (menus && menus.length) {
      return menus.map((item) => ({
        ...item,
        onClick: () => {
          this.selectMsgTemp(item.text, item.code, type);
        },
      }));
    }
    return [];
  };

  peopleLimitConfirm = (obj) => {
    const { selectedProductRecord } = this.state;
    this.updateRow(selectedProductRecord, obj);
    this.setState({
      peopleLimitModal: false,
    });
  };
  // 学习中心选项发生变化时 需要重置下面产品选的人数限制
  handleCampusChange = () => {
    const activityProductList = this.state.activityProductList.map((row) => {
      return {
        ...row,
        pepoleLimit: -1,
        inventoryList: [],
      };
    });
    this.setState({
      activityProductList,
    });
  };

  //
  updateQuestionnaire(val) {
    this.setState({ questionnaireId: val });
  }

  // 切换拼团是 不仅仅要修改拼团状态 而且报名模板的初始值和模板也要发生变化
  changeFightGroup = (e) => {
    const isFightGroup = e.target.checked;
    this.setState({
      isFightGroup,
      enableJoinGroupon: 1,
    });
  };

  render() {
    const {
      loading,
      branchDict,
      campusDict,
      activityProductList,
      activityTypeId,
      productModalVisible,
      tempObject,
      peopleLimitModal,
      activity,
      checking,
      yearDict,
      firstLimitNum,
      lastLimitNum,
      validator,
      validateResult,
      marketOrderSmsTemp,
      marketRefundSmsTemp,
      selectedProductRecord,
      questionnaireId,
      questionnaireDict,
      questionDisabled,
      fightGroupNum,
      isFightGroup,
      marketGrouponOrderSmsTemp,
      twoTeacherShareMsg,
      isNumInputDisabled,
      enableJoinGroupon,
    } = this.state;
    const {
      form: { getFieldDecorator, getFieldValue },
      params: { editable },
    } = this.props;
    const blCampusId = getFieldValue('blCampusId') || [];
    const selectCampusDict = campusDict.filter((campus) => blCampusId.includes(campus.val));
    // 没有选中拼团自动去除拼团优惠 并且修改实际支付标题
    const filterColumns = this.columns.map((column) => {
      if (column.dataIndex === 'realPayAmount') {
        column.title = isFightGroup ? '单独购买/拼团费用' : '实付金额';
      }
      return column;
    });
    // 是否编辑或者查看
    const isEditOrCheck = !!this.props.params.activity;

    return (
      <div className="custom-page">
        <Page loading={loading} route={this.props.route}>
          <Form name onSubmit={this.handleSubmit}>
            <div className={commonStyles.wrap}>
              <h2 className={styles.header}>活动信息</h2>
              <Row>
                <Col span={8}>
                  <FormItem labelWidth={100} name="活动名称" required>
                    <FormItemNative>
                      {getFieldDecorator('name', {
                        rules: [
                          {
                            required: true,
                            message: '请填写活动名称',
                          },
                        ],
                      })(
                        <Input
                          placeholder={'活动名称'}
                          disabled={!!!editable}
                          validator={{
                            rules: [checkLen],
                            silent: !checking,
                            onErrorChange: (e) => this.errorChange('name', e),
                          }}
                        />,
                      )}
                      <div className={styles.warningMSg}>该名称是给家长看到的活动名，例如：【4招赢中考】1对1专题课</div>
                    </FormItemNative>
                  </FormItem>
                </Col>
                <Col span={8}>
                  <FormItem labelWidth={100} name="活动时间" required>
                    <FormItemNative>
                      {getFieldDecorator('range', {
                        rules: [{ required: true, message: '请选择活动时间' }],
                      })(
                        <RangePicker
                          size="large"
                          disabled={!!!editable}
                          format={dateFormat}
                          showTime={{
                            hideDisabledOptions: true,
                            defaultValue: [moment('00:00:00', 'HH:mm:ss'), moment('23:59:59', 'HH:mm:ss')],
                          }}
                          placeholder={['开始时间', '结束时间']}
                        />,
                      )}
                    </FormItemNative>
                  </FormItem>
                </Col>
                <Col span={8}>
                  <FormItem labelWidth={100} name="分公司" required>
                    <FormItemNative>
                      {getFieldDecorator('branchId', {
                        rules: [{ required: true, message: '请选择分公司' }],
                      })(
                        <Select
                          placeholder="请选择分公司"
                          dict={branchDict}
                          onChange={(val) => {
                            this.onBranchChange(val);
                          }}
                          allowClear={false}
                          disabled={!!!editable || !!activityProductList.length}
                        />,
                      )}
                    </FormItemNative>
                  </FormItem>
                </Col>
              </Row>
              <Row>
                <Col span={8}>
                  <FormItem labelWidth={100} name="学习中心" required>
                    <FormItemNative>
                      {getFieldDecorator('blCampusId', {
                        rules: [{ required: true, message: '请选择学习中心' }],
                      })(
                        <MultiSelect
                          disabled={!!!editable}
                          placeholder="请选择学习中心"
                          dict={campusDict}
                          allowClear={false}
                          onChange={this.handleCampusChange}
                        />,
                      )}
                    </FormItemNative>
                  </FormItem>
                </Col>
                {
                  <Col span={8}>
                    <FormItem labelWidth={100} name="每单产品数" required>
                      <FormItemNative>
                        <NumRanger
                          value={[firstLimitNum, lastLimitNum]}
                          isInt
                          firstRanger={MarketActivityDetail.setFirstNumRanger(
                            lastLimitNum,
                            MarketActivityDetail.numRanger,
                          )}
                          lastRanger={MarketActivityDetail.setLastNumRanger(
                            firstLimitNum,
                            MarketActivityDetail.numRanger,
                          )}
                          validator={{
                            rules: [MarketActivityDetail.validateNumRanger],
                            silent: !Boolean(validator.NumRanger) || validateResult,
                            showStatus: true,
                            showTips: true,
                            onErrorChange: (err) => {
                              this.handleValidator({ type: 'NumRanger', val: err });
                            },
                          }}
                          disabled={!editable || isNumInputDisabled}
                          onChange={this.updateNumRanger}
                          lastText="个之间"
                        />
                      </FormItemNative>
                    </FormItem>
                  </Col>
                }
                <Col span={8}>
                  <FormItem labelWidth={100} {...formCfg} name="活动类型">
                    {activityTypeId ? activityTypeObj[activityTypeId] : ''}
                  </FormItem>
                </Col>
              </Row>
              <Row>
                <Col span={8}>
                  <FormItem labelWidth={100} name="更多信息">
                    <Select
                      placeholder="请选择更多信息"
                      dict={questionnaireDict}
                      value={questionnaireId}
                      onChange={this.updateQuestionnaire}
                      disabled={questionDisabled || !editable}
                    />
                  </FormItem>
                </Col>
                <Col span={8}>
                  <FormItem labelWidth={100} name="允许拼团" className={styles.groupWrapper}>
                    <CheckBox checked={isFightGroup} disabled={isEditOrCheck} onChange={this.changeFightGroup} />
                    <InputNumber
                      style={{ width: '50%' }}
                      disabled={isEditOrCheck || !isFightGroup}
                      placeholder="请输入成团人数"
                      value={fightGroupNum}
                      onChange={(value) => this.setState({ fightGroupNum: value })}
                      onBlur={(e) => this.setState({ fightGroupNum: Math.floor(e.target.value) || 2 })}
                      min={2}
                      max={1000}
                      step={1}
                    />
                    <div>人成团</div>
                  </FormItem>
                </Col>
                {isFightGroup && (
                  <Col span={8}>
                    <FormItem labelWidth={100} name="拼团邀请文案">
                      <Input
                        maxLength={15}
                        disabled={!editable}
                        placeholder="非必填，15个字"
                        value={twoTeacherShareMsg}
                        onChange={(e) => this.setState({ twoTeacherShareMsg: e.target.value })}
                      />
                      <div className={styles.warningMSg}>备注：该文案用于生成邀请二维码和转发链接时的提示语</div>
                    </FormItem>
                  </Col>
                )}
              </Row>
              {isFightGroup && (
                <Row>
                  <Col span={7}>
                    <FormItem labelWidth={100} name="是否开启凑团">
                      <Popover content="开启凑团后，家长可以在未成团列表中参与别人的团，未开启则必须由家长自己转发邀请才能拼团">
                        <Icon type="question_circle" size={12} style={{ marginLeft: -16, marginRight: 6 }} />
                      </Popover>
                      <Switch
                        disabled={!editable}
                        checkedChildren="开启"
                        unCheckedChildren="关闭"
                        checked={enableJoinGroupon === 1}
                        onChange={(enableJoinGroupon) =>
                          this.setState({ enableJoinGroupon: enableJoinGroupon ? 1 : 0 })
                        }
                      />
                    </FormItem>
                  </Col>
                </Row>
              )}
              <Row>
                <Col span={isFightGroup ? 7 : 10}>
                  <FormItem labelWidth={0} name={isFightGroup ? '单报模板' : '报名模板'} required>
                    <DropdownMenu
                      disabled={!!!editable}
                      text="请选择"
                      menus={this.buildMsgTempMenus(tempObject[tempType.marketOrderSms], tempType.marketOrderSms)}
                      hasArrow={true}
                    />
                    <TextArea placeholder="请点击上方选择框选择短信模板" value={marketOrderSmsTemp} rows={3} disabled />
                  </FormItem>
                </Col>
                <Col span={1} />
                {isFightGroup && (
                  <div>
                    <Col span={7}>
                      <FormItem labelWidth={0} name="拼团模板" required>
                        <DropdownMenu
                          disabled={!!!editable}
                          text="请选择"
                          menus={this.buildMsgTempMenus(
                            tempObject[tempType.marketGrouponOrderSms],
                            tempType.marketGrouponOrderSms,
                          )}
                          hasArrow={true}
                        />
                        <TextArea
                          placeholder="请点击上方选择框选择短信模板"
                          value={marketGrouponOrderSmsTemp}
                          rows={3}
                          disabled
                        />
                      </FormItem>
                    </Col>
                    <Col span={1} />
                  </div>
                )}
                <Col span={isFightGroup ? 7 : 10}>
                  <FormItem labelWidth={0} name="退费模板" required>
                    <DropdownMenu
                      disabled={!!!editable}
                      text="请选择"
                      menus={this.buildMsgTempMenus(tempObject[tempType.marketRefundSms], tempType.marketRefundSms)}
                      hasArrow={true}
                    />
                    <TextArea
                      placeholder="请点击上方选择框选择短信模板"
                      value={marketRefundSmsTemp}
                      rows={3}
                      disabled
                    />
                  </FormItem>
                </Col>
              </Row>
            </div>
            <div className={styles.hrContainer}>
              <div className={styles.hr}></div>
            </div>
            <div className={commonStyles.wrap} style={{ marginBottom: '40px' }}>
              <div className="flexWrap">
                <div className="flexInnerFit">
                  <h2 className={styles.header}>
                    报读产品
                    <span className={styles.notes}>（注：每个学生只能报读同一个年级的产品）</span>
                  </h2>
                  {!(editable && !+activity.orderNum) ? (
                    ''
                  ) : (
                    <Button
                      className={commonStyles.addQuli}
                      onClick={() => {
                        this.setState({ productModalVisible: true });
                      }}
                    >
                      选择产品
                    </Button>
                  )}
                  <List columns={filterColumns} dataSource={activityProductList} />
                </div>
              </div>
            </div>
            {editable && (
              <div className={styles.fixButton}>
                <Button type="primary" htmlType="submit" disabled={loading && !!!activityProductList.length}>
                  保存
                </Button>
              </div>
            )}
          </Form>
          {productModalVisible && (
            <ProductModal
              yearDict={yearDict}
              title={`选择产品(${branchObj[getFieldValue('branchId')] ? branchObj[getFieldValue('branchId')] : ''})`}
              onConfirm={this.updateProductList}
              branchId={getFieldValue('branchId')}
              productList={activityProductList}
              onClose={() => this.setState({ productModalVisible: false })}
            />
          )}
          {peopleLimitModal && (
            <PeopleLimitModal
              selectedProductRecord={selectedProductRecord}
              campusDict={selectCampusDict}
              editable={editable}
              onConfirm={this.peopleLimitConfirm}
              onClose={() => this.setState({ peopleLimitModal: false, selectedProductRecord: null })}
            />
          )}
        </Page>
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  let data = state.main.pageParams;
  let pathName = state.routing.locationBeforeTransitions.pathname;
  return {
    params: data[pathName] || {},
    user: state.account.user,
  };
};
export default connect(mapStateToProps)(Form.create()(MarketActivityDetail));
