import React from 'react';
import { connect } from 'dva';
import Button from 'COMPONENT/Button';
import Page from 'COMPONENT/Page';
import List from 'COMPONENT/List';
import SearchBar from '../components/NewSearchBar';
import Select from 'COMPONENT/Select';
import Input from 'COMPONENT/Input';
import showMessage from 'SRC/components/utils/globalTips/showMessage';
import showConfirm from 'SRC/components/utils/globalTips/showConfirm';
import { loadCommonDict, loadBranchDict, loadBranchCampusDict } from 'UTIL/dict';
import loader from './loader';
import { renderMultiLine } from '../common';
import moment from 'moment';
import ShareModal from './modal/ShareModal';
import commonStyles from '../common/commonStyle.less';
import style from './styles.less';
import { DatePicker, Icon } from 'antd';
import classnames from 'classnames';
const { RangePicker } = DatePicker;
const dateFormat = 'YYYY-MM-DD HH:mm';

const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;

let defaultBranch = null; //默认第一个学习中心
const statusDict = [
  { text: '未开始', val: '0' },
  { text: '进行中', val: '1' },
  { text: '已结束', val: '2' },
];

// 订单来源字典
const orderFromDict = [
  { text: '流量营销平台', val: '1' },
  { text: 'boss系统', val: '0' },
];

const orderModeDict = [
  { text: '组合模式', val: '1' },
  { text: '普通模式', val: '0' },
];

const statusMap = ['未开始', '进行中', '已结束'];
let branchObj = {};
let activityTypeObj = {};

class MarketActivity extends React.Component {
  constructor(props) {
    super(props);
    const startTimeDate = moment().subtract(1, 'months').startOf('month');
    const endTimeDate = moment();
    this.state = {
      loading: true, // boolean 页面加载状态
      searchExpand: false,
      shareActivity: null,
      shareModalVisible: false,
      //查询条件
      branchId: null,
      blCampusId: null,
      activityTypeId: null,
      name: null,
      time: [startTimeDate, endTimeDate],
      status: null,
      insightFlag: '',
      insightMode: '',
      phone: '',
      studentName: '',
      // real has search
      searchData: '',
      //列表数据
      listLoading: false,
      rows: [],
      pageNo: 1,
      pageSize: 10,
      total: 0,
      //数据字典数据
      branchDict: [],
      campusDict: [],
      activityTypeDict: [],
      // 分享的活动名称
      activityName: '',
      isShowTip: true, //是否显示顶部的提示
    };
  }

  componentDidMount() {
    this.initContent();
  }

  updateState = (changedState) => {
    const newState = {
      ...this.state,
      ...changedState,
    };
    return new Promise((resolve) => {
      this.setState(newState, () => {
        resolve();
      });
    });
  };

  parseTime = (time) => {
    let [beginTime, endTime] = time;
    return {
      beginTime: moment(beginTime).format('YYYY-MM-DD HH:mm:ss'),
      endTime: moment(endTime).format('YYYY-MM-DD HH:mm:ss'),
    };
  };

  loadListData = async (params) => {
    const {
      activityTypeId,
      blCampusId,
      branchId,
      name,
      time,
      pageNo,
      pageSize,
      status,
      insightFlag,
      insightMode,
      phone,
      studentName,
    } = this.state;

    // 开始和结束时间
    let beginTime;
    let endTime;
    if (time === null) {
      beginTime = null;
      endTime = null;
    } else {
      let timer = this.parseTime(time);
      beginTime = timer.beginTime;
      endTime = timer.endTime;
    }
    const apiParams = {
      current: pageNo,
      size: pageSize,
      params: {
        activityTypeId,
        blCampusId,
        branchId,
        name,
        beginTime,
        endTime,
        status,
        insightFlag,
        insightMode,
        phone,
        studentName,
        ...params,
      },
    };
    try {
      const list = await loader.loadList(apiParams);
      const insightIdArray = (list.datas || [])
        .filter((insightData) => insightData.insightFlag === 1)
        .map((insightData) => insightData.id);
      const bossIdArray = (list.datas || []).filter((data) => data.insightFlag !== 1).map((data) => data.id);

      const dataLoaderArray = [loader.getListDetail({ activityIds: bossIdArray.join(',') })];
      if (insightIdArray.length) {
        dataLoaderArray.push(loader.getInsightData({ activityIds: insightIdArray.join(',') }));
      }
      const [bossData, insightData] = await Promise.all(dataLoaderArray);
      if (bossIdArray.length) {
        const bossDataMap = bossData.data.reduce((obj, cur) => {
          obj[cur.activityId] = cur;
          return obj;
        }, {});
        list.datas = list.datas.map((data) => {
          if (bossDataMap[data.id]) {
            const { orderNum, registNum, inviteStudentNum, orderStudentNum, subjectNum, grouponEnd } =
              bossDataMap[data.id];
            return {
              ...data,
              orderNum,
              registNum,
              inviteStudentNum,
              orderStudentNum,
              subjectNum,
              grouponEnd,
            };
          }
          return data;
        });
      }

      if (insightIdArray.length) {
        const insightDataMap = insightData.data.reduce((obj, cur) => {
          obj[cur.activityId] = cur;
          return obj;
        }, {});
        list.datas = list.datas.map((data) => {
          const insightData = insightDataMap[data.id];
          if (insightData) {
            // 洞察的分享量boss和洞察的相加
            const shareNum = Number(data.shareNum) + Number(insightData.shareNum);
            return {
              ...data,
              ...insightData,
              shareNum,
            };
          }
          return data;
        });
      }
      this.setState({
        searchData: phone !== '' ? phone : studentName,
      });
      return list;
    } catch (e) {
      showMessage(e.text);
      return [];
    }
  };

  initContent = async () => {
    this.setState({ loading: true });
    const asyncCalls = [loadBranchDict(), loadBranchCampusDict(), loadCommonDict('MARKET_ACTIVITY_TYPE')];
    Promise.all(asyncCalls).then(([branchDict, campusDict, activityTypeDict]) => {
      branchObj = this.dictToObj(branchDict);
      activityTypeObj = this.dictToObj(activityTypeDict);
      //默认第一个分公司，当前年份
      defaultBranch = branchDict[0] ? branchDict[0].val : null;
      //获取列表数据
      this.loadListData({ branchId: defaultBranch }).then((list) => {
        this.setState({
          branchId: defaultBranch,
          loading: false,
          rows: list.datas,
          total: list.rowCount,
          pageNo: list.pageNo,
          branchDict,
          campusDict,
          activityTypeDict,
        });
      });
    });
  };

  dictToObj = (dict) => {
    return (dict || []).reduce((obj, ele) => {
      obj[ele.val] = ele.text;
      return obj;
    }, {});
  };

  search = async () => {
    await this.updateState({ listLoading: true, pageNo: 1 });
    const list = await this.loadListData();
    this.updateState({
      listLoading: false,
      rows: list.datas,
      total: list.rowCount,
      pageNo: list.pageNo,
    });
  };

  reset = async () => {
    const startTimeDate = moment().subtract(1, 'months').startOf('month');
    const endTimeDate = moment();
    this.updateState({
      activityTypeId: null,
      blCampusId: null,
      branchId: defaultBranch,
      name: null,
      status: null,
      insightFlag: '',
      insightMode: '',
      phone: '',
      studentName: '',
      time: [startTimeDate, endTimeDate],
    });
  };

  pageChange = async ({ current, pageSize }) => {
    await this.updateState({ listLoading: true, pageSize, pageNo: current });
    const list = await this.loadListData();
    this.updateState({
      listLoading: false,
      rows: list.datas,
      total: list.rowCount,
      pageNo: list.pageNo,
    });
  };

  openOrderPage = (activity) => {
    const { searchData } = this.state;
    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path: '/MarketActivityOrderList',
        params: {
          activity: {
            ...activity,
            branchName: branchObj[activity.branchId] || '',
          },
          searchData,
          branchId: this.state.branchId,
        },
        refresh: true,
      },
    });
  };

  openDetailPage = (activity, editable) => {
    // 是否洞察活动
    const isInsight = activity && activity.insightFlag === 1 && editable;
    if (isInsight) {
      showMessage('该活动由流量营销平台创建，暂不支持修改', 'error');
      return false;
    }
    const params = {
      type: 'main/openPage',
      payload: {
        path: '/MarketActivity/AddActivity',
        params: {
          activity,
          editable: activity ? !!editable : true,
        },
        refresh: true,
      },
    };
    if (activity) {
      params.payload.path = '/MarketActivity/' + (!!editable ? 'EditActivity' : 'MarketActivityDetail');
    }
    this.props.dispatch(params);
  };

  openNewClientPage = (record) => {
    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path: 'ClientPage',
        refresh: true,
        params: {
          activityId: record.id,
          activityName: record.name,
          branchId: this.state.branchId,
        },
      },
    });
  };

  openVisitClientPage = (record) => {
    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path: 'VisitClientPage',
        refresh: true,
        params: {
          activityId: record.id,
          activityName: record.name,
          branchId: this.state.branchId,
          productTypeId: record.productTypeId,
        },
      },
    });
  };

  share = (activity) => {
    this.setState({
      shareActivity: activity,
      shareModalVisible: true,
      activityName: activity.name,
    });
  };

  updateRow = (raw, changed) => {
    this.setState({ listLoading: true });
    const idx = this.state.rows.findIndex((ele) => ele.id === raw.id);
    const rows = this.state.rows.map((row, i) => {
      if (i === idx) {
        return { ...row, ...changed };
      }
      return row;
    });
    this.setState({
      listLoading: false,
      rows,
    });
    return { ...raw, ...changed };
  };

  onShareConfirm = () => {
    const shareActivity = this.updateRow(this.state.shareActivity, {
      shareNum: (this.state.shareActivity.shareNum || 0) + 1,
    });
    this.setState({
      shareActivity,
    });
  };

  renderStatus = (detail) => {
    detail = String(detail);
    return (
      <p key="asd" style={{ display: 'flex' }}>
        <b
          style={{
            fontSize: '25px',
            color: detail === '1' ? '#12ce66' : '#AAA',
            marginRight: '3px',
          }}
        >
          ·
        </b>
        {statusMap[detail]}
      </p>
    );
  };

  renderTime = (detail, rowData) => {
    const { beginTime, endTime } = rowData;
    return (
      <div className={commonStyles.timeWrapper}>
        <p>{moment(+beginTime).format('YYYY-MM-DD HH:mm:ss')}</p>
        <p> ~ </p>
        <p>{moment(+endTime).format('YYYY-MM-DD HH:mm:ss')}</p>
      </div>
    );
  };

  deleteRecords = (record) => {
    showConfirm(`确定删除（${record.name}）？`, () => {
      const param = {
        id: record.id,
        activityTypeId: record.activityTypeId,
        validFlag: 0,
      };
      this.updateState({
        loading: true,
      });
      loader.edit(param).then(
        () => {
          showMessage('删除成功！', 'success');
          this.search();
          this.updateState({
            loading: false,
          });
        },
        (err) => {
          showMessage(err.text, 'error');
          this.updateState({
            loading: false,
          });
        },
      );
    });
  };

  /**
   * 表头
   */
  columns = [
    {
      title: '序号',
      dataIndex: 'index',
      width: 70,
      fixed: 'left',
      render: (text, record, index) => index + 1,
    },
    { title: '活动名称', dataIndex: 'name', width: COL_WIDTH_L },
    {
      title: '分公司',
      dataIndex: 'branchId',
      width: COL_WIDTH_M,
      render: (text, record) => {
        const { branchId } = record;
        return branchObj[branchId] || '';
      },
    },
    {
      title: '学习中心',
      dataIndex: 'blCampusName',
      width: COL_WIDTH_M,
      render: renderMultiLine,
    },
    {
      title: '报读产品',
      dataIndex: 'productName',
      width: COL_WIDTH_XL,
      render: renderMultiLine,
    },
    {
      title: '活动类型',
      dataIndex: 'activityTypeId',
      width: 80,
      render: (text, record) => {
        const { activityTypeId } = record;
        return activityTypeObj[activityTypeId] || '';
      },
    },
    {
      title: '活动来源',
      dataIndex: 'insightFlag',
      width: 150,
      render: (text) => (+text === 1 ? '流量营销平台' : 'boss系统'),
    },
    {
      title: '活动模式',
      dataIndex: 'insightMode',
      width: 150,
      render: (text) => (+text === 1 ? '组合模式' : '普通模式'),
    },
    {
      title: '活动时间',
      dataIndex: 'beginTime',
      width: 150,
      render: this.renderTime,
    },
    {
      title: '状态',
      dataIndex: 'status',
      width: 90,
      render: this.renderStatus,
    },
    {
      title: '访问人数',
      dataIndex: 'registNum',
      width: 80,
      render: (text) => text || 0,
    },
    {
      title: '已报人数',
      dataIndex: 'orderStudentNum',
      width: 80,
      render: (text) => text || 0,
    },
    {
      title: '转介绍人数',
      dataIndex: 'inviteStudentNum',
      width: 100,
      render: (text) => text || 0,
    },
    // {
    //   title: "报读科次",
    //   dataIndex: "subjectNum",
    //   width: 80,
    //   render: (text) => text || 0,
    // },
    // { title: "订单量", dataIndex: "orderNum", width: COL_WIDTH_S },
    {
      title: '分享量',
      dataIndex: 'shareNum',
      width: COL_WIDTH_S,
      render: (text) => text || 0,
    },
    {
      title: '操作',
      key: 'action',
      width: COL_WIDTH_XL,
      fixed: 'right',
      getActionConfig: (text, record) => {
        const { btnAuth } = this.props;
        const btns = [];
        if (btnAuth.indexOf('MARKET_ACTIVITY_SHARE') > -1) {
          btns.push({
            text: '分享',
            onClick: () => {
              this.share(record);
            },
          });
        }
        if (btnAuth.indexOf('MARKET_ACTIVITY_MODIFY') > -1) {
          btns.push({
            text: '修改',
            onClick: () => {
              this.openDetailPage(record, true);
            },
          });
        }
        if (btnAuth.indexOf('MARKET_ACTIVITY_DETAIL') > -1) {
          btns.push({
            text: '详情',
            onClick: () => {
              this.openDetailPage(record);
            },
          });
        }
        /*
      if(btnAuth.indexOf('MARKET_ACTIVITY_CLIENT') > -1){
        btns.push(
          { text: '新注册', onClick: () => {this.openNewClientPage(record)} },
        );
      }
*/
        if (btnAuth.indexOf('VISIT_ACTIVITY_CLIENT') > -1) {
          btns.push({
            text: '访问客户',
            onClick: () => {
              this.openVisitClientPage(record);
            },
          });
        }
        if (btnAuth.indexOf('MARKET_ACTIVITY_ORDER') > -1) {
          btns.push({
            text: '订单',
            onClick: () => {
              this.openOrderPage(record);
            },
          });
        }
        // boss创建者的活动可以进行删除
        if (this.props.user.userId === record.createUserId && record.insightFlag === 0) {
          btns.push({
            text: '删除',
            onClick: () => {
              this.deleteRecords(record);
            },
          });
        }
        return btns;
      },
    },
  ];

  render() {
    const {
      loading,
      searchExpand,
      listLoading,
      rows,
      pageNo,
      pageSize,
      total,
      branchId,
      blCampusId,
      activityTypeId,
      name,
      time,
      status,
      branchDict,
      campusDict,
      activityTypeDict,
      shareActivity,
      insightFlag,
      insightMode,
      phone,
      studentName,
      shareModalVisible,
      activityName,
      isShowTip,
    } = this.state;
    const { route, btnAuth } = this.props;

    const isHideCreateButton = new Date().getTime() > new Date('2021-2-10 00:00:00').getTime();

    return (
      <Page
        loading={loading}
        route={route}
        fitWindowHeight={true}
        className={style.marketIndex}
        subHeader={
          <div className={commonStyles.searchBar} style={{ paddingRight: '0px' }}>
            {!isHideCreateButton && isShowTip && (
              <div className={style.warningTip}>
                <span className={style.warningIcon}>!</span>
                旧引流活动的新建入口将于2月10日永久下线，转移到流量营销平台， 需要创建引流活动的老师，请申请新的权限，
                <span
                  className={style.hideCreateButtonTipLink}
                  onClick={() => {
                    window.open('https://docs.qq.com/doc/DV3ByYk5oakJmd0VJ');
                  }}
                >
                  点击查看权限申请流程
                </span>
                <Icon
                  type="close"
                  className={style.closeIcon}
                  onClick={() => {
                    this.setState({ isShowTip: false });
                  }}
                />
              </div>
            )}
            {btnAuth.indexOf('MARKET_ACTIVITY_MODIFY') > -1 && !isHideCreateButton && (
              <Button
                className={classnames({
                  [style.createButton]: isShowTip && !isHideCreateButton,
                  [commonStyles.addQuli]: !(isShowTip && !isHideCreateButton),
                })}
                onClick={() => {
                  this.openDetailPage();
                }}
              >
                新增
              </Button>
            )}
            <SearchBar
              onSearch={this.search}
              onReset={this.reset}
              childWidth={[1, 2, 1, 1, 1, 1, 1, 1]}
              expand={searchExpand}
              onToggleExpand={(expand) => this.setState({ searchExpand: expand })}
            >
              <Input
                placeholder="搜索活动名称"
                value={name}
                onChange={(e) => this.setState({ name: e.target.value })}
              />
              <RangePicker
                style={{ width: '100%' }}
                size="large"
                format={dateFormat}
                showTime={{
                  hideDisabledOptions: true,
                  defaultValue: [moment('00:00:00', 'HH:mm:ss'), moment('23:59:59', 'HH:mm:ss')],
                }}
                placeholder={['开始时间', '结束时间']}
                value={time}
                onChange={(time) => {
                  this.setState({ time: time.length === 0 ? null : time });
                }}
              />
              <Select
                placeholder="分公司"
                dict={branchDict}
                value={branchId}
                allowClear={false}
                showSearch={true}
                onChange={(branchId) => this.setState({ branchId, blCampusId: null })}
              />
              <Select
                placeholder="学习中心"
                dict={branchId ? campusDict.filter((branch) => branch.val === branchId) : []}
                value={blCampusId}
                allowClear={true}
                showSearch={true}
                onChange={(blCampusId) => this.setState({ blCampusId })}
              />
              <Select
                placeholder="活动类型"
                dict={activityTypeDict}
                value={activityTypeId}
                allowClear={true}
                onChange={(activityTypeId) => this.setState({ activityTypeId })}
              />
              <Select
                placeholder="活动状态"
                dict={statusDict}
                value={status}
                allowClear={true}
                onChange={(status) => this.setState({ status })}
              />
              <Select
                placeholder="活动来源"
                dict={orderFromDict}
                value={insightFlag}
                onChange={(insightFlag) => this.setState({ insightFlag })}
              />
              <Select
                placeholder="活动模式"
                dict={orderModeDict}
                value={insightMode}
                onChange={(insightMode) => this.setState({ insightMode })}
              />
              <Input
                placeholder="搜索手机号码"
                value={phone}
                onChange={(e) => this.setState({ phone: e.target.value })}
              />
              <Input
                placeholder="搜索精准学生姓名"
                value={studentName}
                onChange={(e) => this.setState({ studentName: e.target.value })}
              />
            </SearchBar>
          </div>
        }
      >
        <List
          columns={this.columns}
          loading={listLoading}
          dataSource={rows || []}
          pagination={{
            current: pageNo,
            pageSize,
            total: total,
          }}
          onChange={this.pageChange}
        />
        {shareModalVisible && (
          <ShareModal
            activityName={activityName}
            shareActivity={shareActivity}
            onClose={() => {
              this.setState({ shareModalVisible: false, activityName: '' });
            }}
            onConfirm={this.onShareConfirm}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    user: state.account.user,
    btnAuth: state.account.btnAuths || [],
  };
};
export default connect(mapStateToProps)(MarketActivity);
