const timeStrToTimeStamp = (timeStr) => new Date(timeStr.replace(/-/g, '/')).getTime();
import { getReceptionistGroupByUserName, getDistinctUserGroupByCampus } from 'SERVICE/common';
import { loadConsultantDict } from 'PAGE/common/loaders';
import { findStudentById } from 'SERVICE/student';
import {
  listActivity,
  getOrderCount,
  findActivityDetail,
  getProductList,
  getProductByIds,
  calculatePromotion,
  addActivity,
  editActivity,
  getDataDict,
  updateShareNum,
  listPageOrder,
  dispatchOrder,
  cancelOrder,
  openOrder,
  exportActivityOrder,
  getProductActivePromotionList,
  getRechargeMoney,
  checkAllowAddStudent4MiniClass,
  checkAllowAddStudent4TwoTeacherClass,
  findCourseByClassTwoId,
  getMsgTemp,
  transferSchool,
  getOrderDetailByOrderNo,
  getInviteData,
  shareByInsight,
  getInsightActivityData,
  listOrderRemark,
  addOrderRemark,
  listOrderRemarkDetail,
} from 'SERVICE/marketActivityManage';
import { getMiniClassListForEnrollInClass, getClassTwoListForEnrollInClass } from 'SERVICE/course';
import fetch from 'dva/fetch';
import { urlBase } from 'CONST/config';

// 旧boss接口
const loadFollowerDict = async (params, orgParams) => {
  const res = await getReceptionistGroupByUserName({ ...params });
  const dictObj = res.reduce((obj, ele) => {
    if (!obj[ele.orgId]) {
      obj[ele.orgId] = {
        label: ele.orgName,
        options: [],
        val: ele.orgId,
      };
    }
    obj[ele.orgId].options.push({
      val: ele.orgId + '//' + ele.userId,
      text: ele.userName,
      group: ele.orgName,
    });
    // //这里有可能出现这么一个问题 一个身份含有两个学习中心身份 会存在第二个学习中心身份无法添加进入的问题
    // const userIdObj = {};
    // if(!userIdObj[ele.userId]){
    //   userIdObj[ele.userId] = true;
    // }
    return obj;
  }, {});
  if (orgParams) {
    // 数组
    if (Array.isArray(orgParams)) {
      return orgParams.map((orgId) => dictObj[orgId]);
    }
    // 字符串
    return [dictObj[orgParams]];
  }
  return Object.keys(dictObj).reduce((arr, ele) => {
    arr.push(dictObj[ele]);
    return arr;
  }, []);
};
const loadMarketerDict = async (params) => {
  const api = await getDistinctUserGroupByCampus({
    ...params,
    showBelong: 1,
    job: ['SCZY'],
  });
  return api.map((d) => {
    return { val: d.userId, text: d.userName };
  });
};

// 双师课程顾问字典
const loadCourseTeacherDict = async (params) => {
  const api = await getDistinctUserGroupByCampus({
    ...params,
    showBelong: 1,
    job: ['ZXS'],
    // isFilterAcademic: true, // 是否过滤教务角色，true-是，false-否，默认不过滤
  });
  return api.map((d) => {
    return { val: d.userId, text: d.userName };
  });
};
// 双师营运组负责人字典
const load2TeacherYYZFZRDict = async (params) => {
  const api = await getDistinctUserGroupByCampus({
    ...params,
    showBelong: 2,
    job: ['YYZFZR'],
  });
  return api.map((d) => {
    return {
      val: d.userId,
      text: d.userRole,
      name: d.userName,
      orgId: d.orgId,
    };
  });
};

const loadXQYYZRDict = async (params) => {
  try {
    const XQYYFZRData = await getDistinctUserGroupByCampus({
      ...params,
      showBelong: 2,
      job: ['XQYYFZR'],
    });
    if (XQYYFZRData.length) {
      return XQYYFZRData.map((d) => {
        return {
          val: d.userId,
          text: d.userRole,
          name: d.userName,
          orgId: d.orgId,
        };
      });
    }
    const XQZRData = await getDistinctUserGroupByCampus({
      ...params,
      showBelong: 2,
      job: ['XQZR'],
    });
    if (XQZRData.length) {
      return XQZRData.map((d) => {
        return {
          val: d.userId,
          text: d.userRole,
          name: d.userName,
          orgId: d.orgId,
        };
      });
    }
    const XQJXZRData = await getDistinctUserGroupByCampus({
      ...params,
      showBelong: 2,
      job: ['XQJXZR'],
    });
    if (XQJXZRData.length) {
      return XQJXZRData.map((d) => {
        return {
          val: d.userId,
          text: d.userRole,
          name: d.userName,
          orgId: d.orgId,
        };
      });
    }
  } catch (e) {
    return Promise.reject(e);
  }
};

const loadMiniClassList = async (params) => {
  const res = await getMiniClassListForEnrollInClass(params);
  if (res.resultCode !== 0) {
    throw { text: res.resultMessage };
  }
  return res;
};
const loadTwoTeacherClassList = async (params) => {
  const res = await getClassTwoListForEnrollInClass(params);
  if (res.resultCode !== 0) {
    throw { text: res.resultMessage };
  }
  return res;
};
const getConsultantDict = async (params) => {
  return await loadConsultantDict(params);
};

//获取分公司培优老师分享邀请人字典
const loadPeiYouTeacherDict = async (params) => {
  const api = await getDistinctUserGroupByCampus({ ...params, showBelong: 2 });
  return api.map((d) => {
    return {
      val: d.userId,
      text: d.userRole,
      name: d.userName,
      orgId: d.orgId,
    };
  });
};

//获取分公司培优引流项目退费分班字典（就是咨询师）
const loadPeiYouDistrbuteDict = async (params) => {
  const api = await getDistinctUserGroupByCampus({
    ...params,
    showBelong: 1,
    job: ['YLXMCY'],
  });
  return api.map((d) => {
    return {
      val: d.userId,
      text: d.userRole,
      name: d.userName,
      orgId: d.orgId,
    };
  });
};

//获取分公司培优引流项目负责人字典（退费时选择的人）
const loadPeiYouRefundPersonDict = async (params) => {
  const api = await getDistinctUserGroupByCampus({
    ...params,
    showBelong: 2,
    job: ['YLXMFZR'],
  });
  return api.map((d) => {
    return {
      val: d.userId,
      text: d.userRole,
      name: d.userName,
      orgId: d.orgId,
    };
  });
};

// 新报读接口---------------------
const loadList = async (params) => {
  const res = await listActivity(params);
  const { current, records, total } = res.data;
  if (res.code !== 2000000) {
    throw new Error(res.text);
  }
  return {
    datas: records.map((record) => ({
      ...record,
      beginTime: +record.beginTime,
      endTime: +record.endTime,
      createTime: +record.createTime,
      updateTime: +record.updateTime,
    })),
    rowCount: total,
    pageNo: current,
  };
  // return {
  //   datas: records.map(record => ({
  //     ...record,
  //     beginTime: timeStrToTimeStamp(record.beginTime),
  //     endTime: timeStrToTimeStamp(record.endTime),
  //     createTime: timeStrToTimeStamp(record.createTime),
  //     updateTime: timeStrToTimeStamp(record.updateTime),
  //   })),
  //   rowCount: total,
  //   pageNo: current
  // }
};
const getListDetail = async (params) => {
  const res = await getOrderCount(params);
  if (res.code !== 2000000) {
    throw new Error(res.msg);
  }
  return res;
};
const loadOrderList = async (params = {}) => {
  const { pageNo, pageSize, ...extraParams } = params;
  const res = await listPageOrder({
    current: pageNo,
    size: pageSize,
    params: extraParams,
  });
  if (res.code !== 2000000) {
    throw { text: res.msg };
  }
  return res.data;
};

// const loadDetail = async (params) => {
//   const res = await findActivityDetail(params)
//   if (res.code!==2000000) {
//     throw new Error(res.msg)
//   }
//   return res
// }

// // 添加活动
// const add = async (params) => {
//   const data = await addActivity(params)
//   if (data.code!==2000000) {
//     throw new Error(data.msg)
//   }
//   return data
// }
//
// // 编辑活动
// const edit = async (params) => {
//   const data = await editActivity(params)
//   if (data.code!==2000000) {
//     throw new Error(data.text)
//   }
//   return data
// }

// // 获取分享字典
// const getRoleDict = async (params) => {
//   const data = await getDataDict(params)
//   if (data.code!==2000000) {
//     throw new Error(data.msg)
//   }
//   return data
// }

// 未迁移的新报读接口
const loadDetail = async (params) => {
  const res = await findActivityDetail(params);
  if (res.resultCode !== 0) {
    throw new Error(res.resultMessage);
  }
  return res;
};

const getRoleDict = async (params) => {
  const data = await getDataDict(params);
  if (data.resultCode !== 0) {
    throw new Error(data.resultMessage);
  }
  return data;
};

// 添加活动
const add = async (params) => {
  const data = await addActivity(params);
  if (data.resultCode !== 0) {
    throw new Error(res.resultMessage);
  }
  return data;
};

// 编辑活动
const edit = async (params) => {
  const data = await editActivity(params);
  if (data.resultCode !== 0) {
    throw new Error(res.resultMessage);
  }
  return data;
};

const loadProductList = async (params) => {
  const res = await getProductList(params);
  if (res.resultCode !== 0) {
    throw new Error(res.resultMessage);
  }
  return res;
};

const loadActivityProductList = (params) => {
  return getProductByIds(params);
};

const share = async (params) => {
  const data = await updateShareNum(params);
  if (data.resultCode !== 0) {
    throw new Error(data.resultMessage);
  }
  return data;
};

const shareWithInsight = async (params) => {
  const data = await shareByInsight(params);
  if (data.resultCode !== 0) {
    throw new Error(data.resultMessage);
  }
  return data;
};
const getInsightData = async (params) => {
  const data = await getInsightActivityData(params);
  if (data.resultCode !== 0) {
    throw new Error(data.resultMessage);
  }
  return data;
};

const distrbute = async (params) => {
  const data = await dispatchOrder(params);
  if (data.resultCode !== 0) {
    throw { text: data.resultMessage };
  }
  return data;
};

const rejectOrder = async (params) => {
  const res = await cancelOrder(params);
  if (res.resultCode !== 0) {
    throw { text: res.resultMessage };
  }
  return res;
};

const auditOrder = async (params) => {
  const orderNo = params[0].orderNo;
  const useGrouponDiscount = params[0].isGroupPromotion;
  const purchaseAgreement = params[0].purchaseAgreement;
  let url = `${urlBase}MarketActivityController/storageOrder.do?orderNo=${orderNo}&useGrouponDiscount=${useGrouponDiscount}`;
  if (purchaseAgreement !== undefined) {
    url = `${url}&signedAgreement=${purchaseAgreement}`;
  }
  const res = await fetch(url, {
    credentials: 'same-origin',
    method: 'POST',
    body: JSON.stringify(params),
    headers: {
      'Content-Type': 'application/json;charset=utf-8',
    },
  });
  const data = await res.json();
  if (data.resultCode !== 0) {
    throw { text: data.resultMessage };
  }
  return data;
};

const reopenOrder = async (params) => {
  const res = await openOrder(params);
  if (res.resultCode !== 0) {
    throw { text: res.resultMessage };
  }
  return res;
};

const exportExcel = async (params) => {
  const res = await exportActivityOrder(params);
  return res;
};

const loadPromotionList = async (params) => {
  const api = await getProductActivePromotionList({ ...params });
  return api.map((d) => {
    return { val: d.id, text: d.name };
  });
};

const loadRechargeMoney = async (params) => {
  const res = await getRechargeMoney(params);
  if (res.resultCode !== 0) {
    throw { text: res.resultMessage };
  }
  return res;
};
const loadMsgTemp = async (params) => {
  const res = await getMsgTemp(params);
  if (res.resultCode !== 0) {
    throw { text: res.resultMessage };
  }
  return res.data;
};

const transSchool = async (params) => {
  const res = await transferSchool(params);
  if (res.resultCode !== 0) {
    throw { text: res.resultMessage };
  }
  return res;
};

const getOrderDetail = async (params) => {
  const data = await getOrderDetailByOrderNo(params);
  if (data.resultCode !== 0) {
    throw { text: data.resultMessage };
  }
  return data;
};

const getInviteDetail = async (params) => {
  const data = await getInviteData(params);
  if (data.resultCode !== 0) {
    throw new Error(data.resultMessage);
  }
  return data;
};

//获取引流订单的跟进记录
const getOrderRemarkList = async (params) => {
  const res = await listOrderRemark(params);
  if (res.code !== 2000000) {
    throw { text: res.msg };
  }
  return res.data;
};

//获取引流订单的跟进记录详情
const getOrderRemarkDetail = async (remarkId) => {
  if (!remarkId) return;
  const res = await listOrderRemarkDetail({ remarkId });
  if (res.code !== 2000000) {
    throw { text: res.msg };
  }
  return res.data;
};

//添加引流订单的跟进记录
const createOrderRemark = async (params) => {
  const res = await addOrderRemark(params);
  if (res.code !== 2000000) {
    throw { text: res.msg };
  }
  return res.data;
};

// 判断新老生 根据已有的旧逻辑
const getStudentById = (stuId) => {
  return findStudentById({ id: stuId });
};

export default {
  loadList,
  getListDetail,
  loadDetail,
  loadProductList,
  loadActivityProductList,
  calculatePromotion,
  add,
  edit,
  getRoleDict,
  share,
  shareWithInsight,
  loadFollowerDict,
  loadOrderList,
  distrbute,
  loadMiniClassList,
  loadTwoTeacherClassList,
  rejectOrder,
  reopenOrder,
  auditOrder,
  exportExcel,
  loadMarketerDict,
  loadPeiYouTeacherDict,
  loadPromotionList,
  loadRechargeMoney,
  loadXQYYZRDict,
  loadCourseTeacherDict,
  load2TeacherYYZFZRDict,
  checkAllowAddStudent4MiniClass,
  checkAllowAddStudent4TwoTeacherClass,
  findCourseByClassTwoId,
  loadMsgTemp,
  transSchool,
  getOrderDetail,
  getInviteDetail,
  getInsightData,
  loadPeiYouDistrbuteDict,
  loadPeiYouRefundPersonDict,
  getConsultantDict,
  getOrderRemarkList,
  getOrderRemarkDetail,
  createOrderRemark,
  getStudentById,
};
