import React from 'react';
import Modal from 'COMPONENT/Modal';
import FormItem from 'COMPONENT/FormItem';
import Select from 'COMPONENT/Select';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import loader from '../loader';
import { SYSTEM_MARK } from '../../../../constants/config';
import { roleCodeFeatureMap } from '../utilts';
const isTwoTeacher = SYSTEM_MARK === 'TWO_TEACHER';
const { SHOW_XGS_DICT, FOLLOW_UP_PERSON_LOADER } = roleCodeFeatureMap[SYSTEM_MARK];

class DistrbuteModal extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: true,
      userId: '',
      userDict: [],
      currentUserManagerDIct: [],
    };
  }

  onConfirm = async () => {
    this.setState({ loading: true });
    try {
      const { onClose, onConfirm, orders } = this.props;
      const { userId, userDict, currentUserManagerDIct } = this.state;
      if (!userId) {
        this._modal.showOkTips(`请选择跟进人`);
        return;
      }

      let followupUserId = null;
      let followupUserName = null;
      // 双师就是课程顾问
      if (!SHOW_XGS_DICT) {
        followupUserId = userId;
        followupUserName = userDict.filter((ele) => ele.val === userId)[0].name;
      } else {
        const [follweType, currentUserId] = userId.split('//');
        followupUserId = currentUserId;
        followupUserName = currentUserManagerDIct.filter((ele) => ele.val === currentUserId)[0].name;
      }
      await Promise.all(
        orders.map((ele) => {
          return loader.distrbute({
            followupUserId: [followupUserId],
            orderNo: ele.orderNo,
          });
        }),
      );
      onConfirm({
        followupUserId,
        followupUserName,
      });
      this.setState({ loading: false });
      onClose();
      showMessage('操作成功', 'success');
    } catch (e) {
      console.log(e);
      showMessage('操作失败：' + e.text, 'error');
      this.setState({ loading: false });
    }
  };

  async componentDidMount() {
    this.setState({ loading: true });
    const { orders, managerDict } = this.props;

    const { intentionCampusId } = orders[0];
    try {
      // 如果是双师和培养普 并不用拿学管师字典
      if (!SHOW_XGS_DICT) {
        const res = await loader[FOLLOW_UP_PERSON_LOADER]();
        this.setState({
          userDict: res,
          loading: false,
        });
      } else {
        const res = await loader[FOLLOW_UP_PERSON_LOADER]({
          orgId: intentionCampusId,
        });
        // 咨询师与学管师的合并
        const userManagerDict = [
          {
            label: '学管师',
            options: managerDict.map((manager) => ({ ...manager, group: '学管师', val: `manager//${manager.val}` })),
            val: 'manager',
          },
          {
            label: '咨询师',
            options: res.map((user) => ({ ...user, group: '咨询师', val: `user//${user.val}` })),
            val: 'user',
          },
        ];
        this.setState({
          userDict: userManagerDict,
          loading: false,
          currentUserManagerDIct: [...managerDict, ...res],
        });
      }
    } catch (e) {
      this.setState({ loading: false });
    }
  }

  render() {
    const { onClose, orders } = this.props;
    const { userId, userDict, loading } = this.state;

    return (
      <Modal
        loading={loading}
        title={orders.length > 1 ? '批量分配' : '分配'}
        onCancel={onClose}
        okText="保存"
        onOk={this.onConfirm}
        ref={(c) => (this._modal = getWrappedInstance(c))}
        width={430}
      >
        <div className="activityModalContainer">
          <FormItem name="请选择跟进人" required="true">
            <Select
              placeholder="分配跟进人"
              dict={userDict}
              value={userId}
              showSearch
              onChange={(val) => this.setState({ userId: val })}
            />
          </FormItem>
        </div>
      </Modal>
    );
  }
}

export default DistrbuteModal;
