import React from 'react';
import Modal from 'COMPONENT/Modal';
import List from 'COMPONENT/List';
import loader from '../loader';
import Styles from '../styles.less';
const { COL_WIDTH_S, COL_WIDTH_M } = List;
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

const columns = [
  {
    title: '姓名',
    dataIndex: 'invitedStudentName',
    width: COL_WIDTH_S,
    render: (text, record) => {
      const isLeader = record.isGrouponLeader === 1;
      return isLeader ? <span style={{ color: '#6d67ff' }}>{`(团长) ${text}`}</span> : text;
    },
  },
  { title: '手机号码', dataIndex: 'customerPhone', width: COL_WIDTH_M },
  { title: '资源来源', dataIndex: 'resType', width: COL_WIDTH_S },
  {
    title: '邀请者',
    dataIndex: 'inviteStudentName',
    width: COL_WIDTH_S,
    render: (text) => {
      return text || '---';
    },
  },
];
export default class FightGroupModal extends React.Component {
  state = {
    loading: false,
    fightGroupData: [],
  };
  componentDidMount() {
    this.initData();
  }
  initData = async () => {
    const { order } = this.props;
    const { orderNo } = order;
    this.setState({
      loading: true,
    });

    try {
      const res = await loader.getInviteDetail({
        orderNo,
        pageSize: 2000,
        pageNo: 1,
      });

      const records = Array.isArray(res.data.records) ? res.data.records : [];
      this.setState({
        fightGroupData: records.reverse(),
      });
    } catch (e) {
      console.log(e);
      showMessage(e.text, 'error');
    }
    this.setState({
      loading: false,
    });
  };

  render() {
    const { loading, fightGroupData } = this.state;
    const { onClose } = this.props;

    return (
      <Modal loading={loading} title="拼团记录" width={715} cancelText="关闭" onCancel={onClose} okBtnShow={false}>
        <div className={Styles.orderStatusModal}>
          <div className={Styles.orderStatusTop}>{`拼团人数${fightGroupData.length}`}</div>
          <List columns={columns} dataSource={fightGroupData} />
        </div>
      </Modal>
    );
  }
}
