import React from 'react';
import Modal from 'COMPONENT/Modal';
import Styles from '../styles.less';

const orderStatusMap = {
  order: [
    {
      label: '未支付',
      content: '家长提交订单，在30分钟内未完成支付。',
    },
    {
      label: '已支付',
      content: '家长提交订单，在30分钟内完成支付。',
    },
    {
      label: '退款申请中',
      content: '家长提交订单并完成支付，跟进人发起退款，但营主还未审核。',
    },
    {
      label: '退费中',
      content: '家长提交订单并完成支付，跟进人发起退款，营主审核通过，但微信还未到账（时间一般3分钟以内）。',
    },
    {
      label: '退费成功',
      content: '家长提交订单并完成支付，跟进人发起退款，营主审核通过，且微信已到账。',
    },
    {
      label: '退款失败',
      content: '家长提交订单并完成支付，跟进人发起退款，但营主审核不通过或者集团银行账户余额不足退不出钱。',
    },
  ],
  refund: [
    {
      label: '未审核',
      content: '跟进人未进行审核操作。需要由学习中心营运主任分配给相应的老师进行跟进，由跟进人进行审核。',
    },
    {
      label: '审核通过',
      content: '跟进人审核通过，并生成合同。',
    },
    {
      label: '审核未过',
      content: '跟进人审核不通过，对于已支付的订单需要发起退款。',
    },
    {
      label: '已报读',
      content:
        '一个学生只能报读同一个活动一次。学生有多个订单，若其中一个订单已支付，则其他订单状态为已报读，即已报读的订单作废，只有已支付的订单有效。',
    },
  ],
};
export default function OrderStatusModal({ onClose, orderModalType = 'order' }) {
  return (
    <Modal
      title={orderModalType === 'order' ? '订单状态说明' : '审核状态说明'}
      width={715}
      cancelText="关闭"
      onCancel={onClose}
      okBtnShow={false}
    >
      <div className={Styles.orderStatusModal}>
        {orderStatusMap[orderModalType].map((item, index) => (
          <div>
            <div className={Styles.label}>{`${index + 1}.${item.label}`}</div>
            <div className={Styles.content}>{item.content}</div>
          </div>
        ))}
      </div>
    </Modal>
  );
}
