import React from 'react';
import Modal from 'COMPONENT/Modal';
import FormItem from 'COMPONENT/FormItem';
import TextArea from 'COMPONENT/TextArea';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import Checkbox from 'COMPONENT/Checkbox';
import styles from '../styles.less';
import detail from '../detail';

const toFloat2 = (val) => {
  return Math.round(val * 100) / 100;
};
class RefundModal extends React.Component {
  state = {
    textInputValue: '',
    selectPersonIndex: 0,
    selectProductArray: this.props.order.productId.slice(),
  };
  onConfirmModal = () => {
    const { textInputValue, selectPersonIndex, selectProductArray } = this.state;
    if (!textInputValue) {
      showMessage('退款理由不能为空');
      return;
    }
    if (!selectProductArray.length) {
      showMessage('请勾选退款产品');
      return;
    }
    if (!this.props.refundXQYYZRArray[selectPersonIndex]) {
      showMessage('请勾选退款审批人');
      return;
    }
    this.props.onConfirm({
      refundReason: textInputValue,
      refundXQYYZR: this.props.refundXQYYZRArray[selectPersonIndex],
      selectProductArray,
    });
  };
  handleSelectProduct = (id, checked) => {
    const { selectProductArray } = this.state;
    let newCheckedArray;
    if (checked) {
      newCheckedArray = [...selectProductArray, id];
    } else {
      newCheckedArray = selectProductArray.filter((c) => c !== id);
    }
    this.setState({ selectProductArray: newCheckedArray });
  };

  render() {
    const { onClose, order, orderDetail, refundXQYYZRArray, activityProductMap } = this.props;
    const { textInputValue, selectPersonIndex, selectProductArray } = this.state;
    const { onConfirmModal } = this;

    const { orderItems, grouponLevel } = orderDetail;
    const { productId, productName, grouponStatus, insightMode } = order;
    // 取拼团优惠价格还是 原石价格
    const amountType = grouponStatus === 2 ? 'totalAmount' : 'grouponAmount';
    const productArray = productId.map((id, index) => ({ productId: id, productName: productName[index] }));
    let totalAmount = selectProductArray.reduce((amount, productId) => {
      return amount + activityProductMap[productId][amountType];
    }, 0);

    if (orderItems) {
      totalAmount = orderItems
        .filter((oItem) => selectProductArray.includes(oItem.relateProductNo))
        .reduce((prev, cur) => {
          let itemAppendixsCount = cur.itemAppendixs.reduce((initCount, item) => {
            return initCount + item.price * item.num;
          }, 0);
          return prev + (cur.price * cur.num - cur.discount) + itemAppendixsCount;
        }, 0);
    }

    totalAmount = toFloat2(totalAmount / 100);

    return (
      <Modal title="退款" onCancel={onClose} width={650} okText="确定" onOk={onConfirmModal}>
        <div>{`${order.studentName}(${order.customerPhone})`}</div>
        <FormItem name="勾选要退费的产品" required="true" className={styles.refundContainer}>
          <div>
            <div className={styles.refundCheckbox}>
              {productArray.map((product) => {
                const { productId, productName } = product;
                let productAmount = activityProductMap[productId][amountType];
                if (orderItems) {
                  let filterOrderItem = orderItems.find((oi) => oi.relateProductNo === productId);
                  if (filterOrderItem) {
                    let itemAppendixsCount = filterOrderItem.itemAppendixs.reduce((initCount, item) => {
                      return initCount + item.price * item.num;
                    }, 0);
                    productAmount =
                      filterOrderItem.price * filterOrderItem.num - filterOrderItem.discount + itemAppendixsCount;
                  }
                }
                productAmount = toFloat2(productAmount / 100);
                const checked = selectProductArray.includes(productId);
                return (
                  <Checkbox
                    key={productId}
                    checked={checked}
                    onChange={(e) => this.handleSelectProduct(productId, e.target.checked)}
                  >
                    {`${productName}  实收${productAmount}元`}
                  </Checkbox>
                );
              })}
            </div>
            <div>{`退款金额（${totalAmount}元）`}</div>
          </div>
        </FormItem>
        <FormItem name="退款" required="true" className={styles.refundContainer}>
          <TextArea
            maxLength={70}
            rows={3}
            placeholder="请输入退款理由"
            value={textInputValue}
            onChange={(e) => this.setState({ textInputValue: e.target.value })}
          />
          <span>
            <span style={{ color: 'red' }}>{`${textInputValue ? textInputValue.length : 0}`}</span>/70
          </span>
        </FormItem>
        <div>审核人</div>
        <div className={styles.refundPersonChoose}>
          {refundXQYYZRArray.map((person, index) => (
            <div
              className={(index === selectPersonIndex ? styles.selected : '') + ' ' + styles.selectItem}
              key={index}
              onClick={() => this.setState({ selectPersonIndex: index })}
            >
              {person.text}
            </div>
          ))}
        </div>
      </Modal>
    );
  }
}

export default RefundModal;
