import React from 'react';
import { connect } from 'dva/index';
import Modal from 'COMPONENT/Modal';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import FormItem from 'COMPONENT/FormItem';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { loadBranchCampusDict } from 'UTIL/dict';
import loader from '../loader';
import { formCfg } from '../../common';
import {
  peiYouTest,
  peiYouPre,
  peiYou,
  twoTeacherTest,
  twoTeacherPre,
  twoTeacher,
  uat,
  pre,
  prd,
} from 'CONST/buildEnv';
import ShareQRCode from 'COMPONENT/ShareQRCode';
import { SYSTEM_MARK } from '../../../../constants/config';
import { roleCodeFeatureMap } from '../utilts';
const isTwoTeacher = SYSTEM_MARK === 'TWO_TEACHER';
const { IS_SHARE_HAS_CUS_ORG, CAN_SELECT_SHARE_PERSON, SELECT_SHARE_PERSON_LOADER, SHARE_TITLE_NUM } =
  roleCodeFeatureMap[SYSTEM_MARK];
const objToDict = (obj) => {
  return Object.keys(obj)
    .filter((ele) => !!ele)
    .map((ele) => {
      return {
        text: obj[ele],
        val: ele,
      };
    });
};

const cusOrgObjToDictObj = (cusOrgObj) => {
  return Object.keys(cusOrgObj).reduce((obj, ele) => {
    obj[ele] = cusOrgObj[ele].map((ele) => {
      return {
        text: ele.name,
        val: ele.id,
      };
    });
    return obj;
  }, {});
};

const buildBaseUrl = () => {
  let result = '';
  if (process.env) {
    switch (process.env.BUILD_ENV) {
      case pre:
      case peiYouPre:
      case twoTeacherPre:
        result = 'https://cmall-pre.xiao100.com/sharing-sale/studentList?';
        break;
      case prd:
      case peiYou:
      case twoTeacher:
        result = 'https://cmall.xiao100.com/sharing-sale/studentList?';
        break;
      default:
        result = 'https://cmall-test.xiao100.com/sharing-sale/studentList?';
        break;
    }
    switch (process.env.BUILD_ENV) {
      case uat:
      case pre:
      case prd:
        result += 'sys=XINGHUO&';
        break;
      case peiYouTest:
      case peiYouPre:
      case peiYou:
        result += 'sys=ADVANCE&';
        break;
      case twoTeacherTest:
      case twoTeacherPre:
      case twoTeacher:
        result += 'sys=TWO_TEACHER&';
        break;
      default:
        result += 'sys=XINGHUO&';
        break;
    }
  }
  return result;
};

class ShareModal extends React.Component {
  constructor(props) {
    super(props);

    const { user, activityName } = this.props;
    this.state = {
      loading: true,
      shareNum: null,
      campusId: [],
      resEntranceId: null,
      cusOrgId: null,
      resTypeId: null,
      distributeUserId: null,
      inviteUserId: user.userId,

      resTypeDict: [],
      cusOrgDict: [],
      resEntranceDict: [],
      campusDict: [],
      marketerDict: [{ text: user.name, val: user.userId }],

      campusSelectorDisabled: false,
      resEntranceSelectorDisabled: false,
      cusOrgSelectorDisabled: false,
      resTypeSelectorDisabled: false,
      inviteUserSelectorDisabled: true,

      qrcodeVisiable: false,
      qrcodeVisible: false,
      qrcodeUrl: '',

      checking: false,
    };
    this.cusOrgDictObj = {};
    this.errors = {};
  }

  confirm = async () => {
    const { campusId, resEntranceId, cusOrgId, resTypeId, distributeUserId, inviteUserId, checking, shareNum } =
      this.state;
    const { shareActivity, onConfirm } = this.props;
    this.setState({ loading: true });
    try {
      if (!checking) {
        this.setState({ checking: true });
      }
      const errors = Object.values(this.errors).filter((ele) => ele);
      if (errors.length) {
        this.modal.showOkTips('保存失败：请确认必填输入项是否填写完毕');
        this.setState({
          loading: false,
        });
        return;
      }
      const isInsight = shareActivity.insightFlag === 1;

      let params = {
        resEntranceId,
        resType: resTypeId,
      };
      if (isInsight) {
        params = {
          ...params,
          marketActivityId: shareActivity.id,
          campus: campusId,
          distributeUserId,
          inviteUser: inviteUserId,
        };
      } else {
        params = {
          ...params,
          activityId: shareActivity.id,
          campus: campusId.join(','),
          distributeUserId,
          inviteUserId: inviteUserId,
        };
      }
      // 双师没有来源细分 星火有
      if (IS_SHARE_HAS_CUS_ORG) {
        params.cusOrg = cusOrgId;
      }
      let shareNumText = null;
      const res = await loader[isInsight ? 'shareWithInsight' : 'share'](params);
      let qrcodeUrl;
      // 使用洞察的二维码
      if (isInsight) {
        qrcodeUrl = res.data.url;
      } else {
        qrcodeUrl = buildBaseUrl();
        const { marketActivityShareId, shareNum } = res.data;
        qrcodeUrl += `activityId=${shareActivity.id}&shareCodeId=${marketActivityShareId}`;
        if (SHARE_TITLE_NUM) {
          shareNumText = `（${shareNum}）`;
        }
      }
      this.setState({
        qrcodeVisiable: true,
        qrcodeUrl,
        shareNum: shareNumText,
      });
      onConfirm();
    } catch (e) {
      console.log(e);
      showMessage(e.text || e.message, 'error');
    }
    this.setState({ loading: false });
  };

  componentDidMount() {
    try {
      this.initContent();
    } catch (e) {
      console.log(e);
    }
  }

  initContent = async () => {
    this.setState({ loading: true });
    const { shareActivity, user } = this.props;
    const asyncCalls = [loadBranchCampusDict(), loader.getRoleDict({ id: shareActivity.id })];
    // 如果不是双师 也就是星火的话如果角色是市场经理那么邀请人可以自己选
    if (CAN_SELECT_SHARE_PERSON && user.roleCode.indexOf(CAN_SELECT_SHARE_PERSON) > -1) {
      asyncCalls.push(loader[SELECT_SHARE_PERSON_LOADER]());
    }
    const [campusDict, res, marketerDict] = await Promise.all(asyncCalls);
    // 如果是双师的话 资料来源字典就不用处理了 跟资源入口级联
    const resTypeDict = !IS_SHARE_HAS_CUS_ORG ? res.data.resType || {} : objToDict(res.data.resType || {});
    const resEntranceDict = objToDict(res.data.resEntrance || {});
    this.cusOrgDictObj = cusOrgObjToDictObj(res.data.cusOrg || {});
    const resEntranceId = resEntranceDict.length === 1 ? resEntranceDict[0].val : '';
    if (marketerDict) {
      this.setState({
        marketerDict: marketerDict,
        inviteUserId: null,
        inviteUserSelectorDisabled: false,
      });
    }

    const campusIds = res.data.campus ? res.data.campus.split(',') : [];
    const newCampusDict = campusDict.map((ele) => {
      ele.options = ele.options.reduce((arr, ele) => {
        if (campusIds.indexOf(ele.val) > -1) {
          arr.push(ele);
        }
        return arr;
      }, []);
      return ele;
    });
    const resEntranceSelectorDisabled = !IS_SHARE_HAS_CUS_ORG
      ? false
      : resEntranceDict.length === 1 || resTypeDict.length === 2;
    this.setState({
      resTypeDict,
      resEntranceDict,
      campusDict: newCampusDict,
      distributeUserId: res.data.followupUserId,
      loading: false,

      campusId: campusIds[0] ? [campusIds[0]] : [],
      resEntranceId,
      campusSelectorDisabled: campusIds.length === 1,
      resEntranceSelectorDisabled,
    });
  };

  onResTypeChange = (resTypeId) => {
    const { resTypeDict, resEntranceId } = this.state;
    let newResEntranceId = resEntranceId;
    if (resTypeDict.length === 2) {
      switch (resTypeId) {
        case 'b07e325eadf52ffc67d615ddc08e344d':
          newResEntranceId = 'NEW_MEDIA';
          break;
        case 'TEL_CONSULTATION':
          newResEntranceId = 'GROUNDPROMOTION';
          break;
      }
    }
    const cusOrgDict = !resTypeId ? [] : this.cusOrgDictObj[resTypeId];
    this.setState({
      cusOrgDict,
      cusOrgId: '',
      resEntranceId: newResEntranceId,
    });
  };

  errorChange = (key, error) => {
    this.errors[key] = error;
  };

  render() {
    const { onClose, shareActivity } = this.props;
    const {
      loading,
      campusId,
      resEntranceId,
      cusOrgId,
      resTypeId,
      inviteUserId,
      resTypeDict,
      cusOrgDict,
      resEntranceDict,
      campusDict,
      marketerDict,
      shareNum,
      qrcodeVisiable,
      qrcodeUrl,
      campusSelectorDisabled,
      resEntranceSelectorDisabled,
      cusOrgSelectorDisabled,
      resTypeSelectorDisabled,
      inviteUserSelectorDisabled,
      checking,
    } = this.state;
    const isInsight = shareActivity.insightFlag === 1;

    //双师的资料来源字典需要根据第一个级联
    // 星火的是根据资料来源反推资源入口字典
    let trueResTypeDict = resTypeDict;
    if (!IS_SHARE_HAS_CUS_ORG) {
      trueResTypeDict = (resTypeDict[resEntranceId] || []).map((resType) => {
        const text = resType.value.indexOf('talk') > -1 ? '外呼' : resType.value;
        return { ...resType, text, val: resType.key };
      });
    }
    return (
      <Modal
        loading={loading}
        title="分享"
        onCancel={onClose}
        okText="生成二维码"
        cancelText="关闭"
        onOk={this.confirm}
        ref={(c) => (this.modal = getWrappedInstance(c))}
        width={650}
      >
        <div>
          <FormItem {...formCfg} name="资源入口" required>
            <Select
              placeholder="请选择"
              dict={resEntranceDict}
              value={resEntranceId}
              onChange={(val) =>
                this.setState(
                  !IS_SHARE_HAS_CUS_ORG
                    ? { resEntranceId: val, resTypeId: null, cusOrgId: null }
                    : { resEntranceId: val },
                )
              }
              disabled={resEntranceSelectorDisabled}
              validator={{
                rules: [{ required: '请选择资源入口' }],
                silent: !checking,
                onErrorChange: (e) => this.errorChange('resEntranceId', e),
              }}
            />
          </FormItem>
          <FormItem {...formCfg} name="资源来源" required>
            <Select
              placeholder="请选择"
              dict={trueResTypeDict}
              value={resTypeId}
              onChange={(val) => {
                this.setState(!IS_SHARE_HAS_CUS_ORG ? { resTypeId: val, cusOrgId: null } : { resTypeId: val });
                IS_SHARE_HAS_CUS_ORG && this.onResTypeChange(val);
              }}
              disabled={!IS_SHARE_HAS_CUS_ORG ? !resEntranceId : resTypeSelectorDisabled}
              validator={{
                rules: [{ required: '请选择资源来源' }],
                silent: !checking,
                onErrorChange: (e) => this.errorChange('resTypeId', e),
              }}
            />
          </FormItem>
          {
            /*双师没有来源细分*/
            IS_SHARE_HAS_CUS_ORG && (
              <FormItem {...formCfg} name="来源细分" required>
                <Select
                  placeholder="请选择"
                  dict={cusOrgDict}
                  value={cusOrgId}
                  onChange={(val) => this.setState({ cusOrgId: val })}
                  disabled={!IS_SHARE_HAS_CUS_ORG ? !resTypeId : cusOrgSelectorDisabled}
                  validator={{
                    rules: [{ required: '请选择来源细分' }],
                    silent: !checking,
                    onErrorChange: (e) => this.errorChange('cusOrgId', e),
                  }}
                />
              </FormItem>
            )
          }
          <FormItem {...formCfg} name="参加活动学习中心" required>
            <MultiSelect
              placeholder="请选择"
              showSearch
              dict={campusDict}
              value={campusId}
              onChange={(val) => this.setState({ campusId: val })}
              disabled={campusSelectorDisabled}
              validator={{
                rules: [{ required: '请选择参加活动学习中心' }],
                silent: !checking,
                onErrorChange: (e) => this.errorChange('campusId', e),
              }}
            />
          </FormItem>
          <FormItem {...formCfg} name="邀请人" required>
            <Select
              placeholder="请选择"
              showSearch
              dict={marketerDict}
              value={inviteUserId}
              onChange={(val) => this.setState({ inviteUserId: val })}
              disabled={inviteUserSelectorDisabled}
              validator={{
                rules: [{ required: '请选择邀请人' }],
                silent: !checking,
                onErrorChange: (e) => this.errorChange('inviteUserId', e),
              }}
            />
          </FormItem>
        </div>
        <ShareQRCode
          title="分享"
          shareTitle={this.props.activityName + (shareNum || '')}
          shareText={qrcodeUrl}
          shareBottom={isInsight ? '该二维码由流量营销平台提供' : ''}
          fileName="引流活动_二维码"
          visible={qrcodeVisiable}
          onCancel={() => {
            this.setState({ qrcodeVisiable: false });
          }}
        />
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    user: state.account.user,
  };
};
export default connect(mapStateToProps)(ShareModal);
