import React from 'react';
import { connect } from 'dva';
import Button from 'COMPONENT/Button';
import Page from 'COMPONENT/Page';
import List from 'COMPONENT/List';
import SearchBar from 'COMPONENT/SearchBar';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import Input from 'COMPONENT/Input';
import Icon from 'COMPONENT/Icon';
import Popover from 'COMPONENT/Popover';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { loadCommonDict, loadBranchDict, loadBranchCampusDict } from 'UTIL/dict';
import { getManagerDict } from 'SERVICE/followReport';
import loader from './loader';
import commonStyles from '../common/commonStyle.less';
import moment from 'moment';
import AuditModal from './modal/AuditModal';
import FailReasonModal from './modal/FailReasonModal';
import RefundModal from './modal/RefundModal';
import DistrbuteModal from './modal/DistrbuteModal';
import TransferSchoolModal from './modal/TransferSchoolModal';
import OrderStatusModal from './modal/OrderStatusModal';
import FightGroupModal from './modal/FightGroupModal';
import utils from '../common/utils';
import { applyRefund } from '../RefundDetail/loader';
import { renderMultiLine } from '../common';
import ModalEditStdInfo from './ModalEditStdInfo';
import ModalStdQuestionnaire from '../Questionnaire/ModalStdQuestionnaire';
import { SYSTEM_MARK } from '../../../constants/config';
import { roleCodeFeatureMap } from './utilts';
import ButtonWrap from '../../signUpOnline/StdntApptMgmt/ButtonWrap';
import { xinghuoInstitution } from 'CONST/config';
import withTeachingReporterHOC from 'PAGE/teachingReporterHOC/teachingReporterHOC';
import { debounce } from 'lodash';
import OrderRemarkModal from './components/OrderRemarkModal';
let TeachingReporter = null;

// 是否双师
const isTwoTeacher = SYSTEM_MARK === 'TWO_TEACHER';

const { DISTRIBUTE_ROLE_CODE, LIMIT_CAMPUS_DISTRIBUTE, REFUND_PERSON_LOADER_NAME } = roleCodeFeatureMap[SYSTEM_MARK];

const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L } = List;
const dateFormat = 'YYYY-MM-DD HH:mm';
// 分配字典
const distributeDict = [
  { text: '已分配', val: 'HAS_ALLOCATION' },
  { text: '未分配', val: 'NOT_ALLOCATION' },
];
const auditStatusDict = [
  { text: '待审核', val: '0' },
  { text: '审核通过', val: '1' },
  { text: '审核未过', val: '2' },
  { text: '已报读', val: '3' },
];
// 学籍学习中心字典
const studentStatusCampusDict = [
  { text: '本学习中心', val: '1' },
  { text: '其他学习中心', val: '2' },
];
// 是否在读字典
const attendSchoolDict = [
  { text: '在读', val: '1' },
  { text: '非在读', val: '2' },
];
const surrenderStatusMap = {
  dict: [
    { text: '申请退款中', val: '1' },
    { text: '退款中', val: '2' },
    { text: '退款成功', val: '3' },
    { text: '退款失败', val: '4' },
  ],
  increase(status) {
    if (Array.isArray(status)) {
      return status.map((item) => ({ ...item, val: String(+item.val + 10) }));
    }
    // 单一 "0" 返回 “10”
    return String(+status + 10);
  },
  // payStatus解析方法 传入的是数组
  decrease(statusArray) {
    // 没选择状态直接返回
    if (!statusArray.length) {
      return {};
    }
    const payStatus = [];
    const surrenderStatus = [];
    statusArray.forEach((status) => {
      if (+status >= 10) {
        // 退款状态
        surrenderStatus.push(Number(status - 10));
      } else {
        // 支付状态
        payStatus.push(status);
      }
    });
    return {
      payStatus: payStatus,
      surrenderStatus: surrenderStatus,
    };
  },
};
const payStatusDict = [
  { text: '未支付', val: '0' },
  { text: '已支付', val: '1' },
  ...surrenderStatusMap.increase(surrenderStatusMap.dict),
];
const auditStatusColorObj = {
  0: '#a4a0ff',
  1: '#d8d8d8',
  2: '#ff5f5f',
  3: '#d8d8d8',
};
//1：已成团；0：未成团；空 ：单独购买
const groupStatusDict = [
  { text: '未成团', val: 0 },
  { text: '已成团', val: 1 },
  { text: '单独购买', val: 2 },
];
function dictToObj(dict) {
  return (dict || []).reduce((obj, ele) => {
    obj[ele.val] = ele.text;
    return obj;
  }, {});
}
const groupStatusObj = dictToObj(groupStatusDict);
const auditStatusObj = dictToObj(auditStatusDict);
const payStatusObj = dictToObj(payStatusDict);

class MarketActivityOrder extends React.Component {
  constructor(props) {
    super(props);
    const { activity, searchData } = this.props.params;
    const { userOrgRoles } = this.props;
    // 获取星火 营运学习中心主任角色的组件架构id
    const XINGHUOCampusXQYYZROrg = userOrgRoles
      .filter((role) => role.roleId === 'ROL0000000077')
      .map((role) => role.organizationId);
    this.switchEditStdInfo = this.switchEditStdInfo.bind(this);
    this.afterSubmitEditStdInfo = this.afterSubmitEditStdInfo.bind(this);
    this.switchModalStdQuestionnaire = this.switchModalStdQuestionnaire.bind(this);
    const isFightGroup = activity.enableGroupon === 1;
    this.state = {
      loading: true, // boolean 页面加载状态
      searchExpand: true,
      shareActivity: null,
      shareModalVisible: false,
      handleRefundXQYYZRArray: [], //学习中心营运主任数组 用于点击退款时
      //查询条件
      name: searchData || null,
      followupUserName: null,
      intentionCampusId: null,
      gradeId: null,
      inviteUserId: '',
      payStatus: [],
      // payStatus 不仅代表着支付状态也代表着退款状态
      auditStatus: [],
      selectedRowKeys: [], //选中行的key的集合
      grouponStatus: null, // 拼团状态
      // 学生是否当前的学籍学习中心
      blCampusId: null,
      // 学生是否在读
      classStatus: null,
      // 是否已分配
      allocationFlag: null,
      //列表数据
      listLoading: false,
      //学管师
      studyManegerId: null,
      // 活动详情用于退款的产品金额计算
      activityProductMap: {},
      rows: [],
      pageNo: 1,
      pageSize: 20,
      total: null,
      // orderDetail 申请退款订单详情
      orderDetail: {},
      //数据字典数据
      groupStatusDict,
      campusDict: [],
      gradeDict: [],
      inviteUserDict: [],
      managerDict: [],
      // 转校学习中心字典
      transferSchoolCampusDict: [],
      auditModalVisible: false,
      productDict: [],
      selectedOrders: [],
      failReasonModalVisible: false,
      refundModalVisible: false,
      selectedOrder: null,
      distrbuteModalVisible: false,

      batchDistrbuteDisabled: true,
      // 修改学生信息弹窗显示状态
      editStdInfoVisible: false,
      // 转校弹窗
      transferSchoolModal: false,
      orderStatusModal: false,
      orderModalType: null,
      // 更多信息弹窗
      visibleStdQuestionnaire: false,
      // 拼团记录
      fightGroupModal: false,
      // 该活动订单是否拼团
      isFightGroup,
      // 给星火用的筛选出学习中心架构的营运学习中心主任
      XINGHUOCampusXQYYZROrg,
      isReporter: false, //是否打开入学测报告
      isClickAuditButton: false, //是否点击了审核按钮
      isShowOrderRemarkModal: false, //是否显示跟进记录弹窗
    };
    // 更多信息弹窗props
    this.studentNo = null;
    this.studentName = null;

    this.editStdData = null;

    this.columns = [
      {
        title: '序号',
        dataIndex: 'index',
        width: 60,
        fixed: 'left',
        render: (text, record, index) => index + 1,
      },
      { title: '学生名称', dataIndex: 'studentName', width: 100 },
      {
        title: '准考证号',
        dataIndex: 'bossStudentNo',
        width: COL_WIDTH_L,
        render: (text) => text || '暂无',
        filterDropdownVisible: false,
        filterIcon: (
          <Popover content="用于教学平台的阅卷系统使用">
            <Icon type="question_circle" size={12} style={{ marginLeft: 2 }} />
          </Popover>
        ),
        filterDropdown: <span className="empty-content"> </span>,
      },
      { title: '客户号码', dataIndex: 'customerPhone', width: 120 },
      { title: '学生手机号码', dataIndex: 'studentPhone', width: 120 },
      { title: '意向学习中心', dataIndex: 'intentionCampusName', width: 100 },
      { title: '学籍学习中心', dataIndex: 'blCampusName', width: COL_WIDTH_M },
      {
        title: '是否在读',
        dataIndex: 'classStatus',
        width: COL_WIDTH_M,
        render: (text) => {
          return Number(text) === 1 ? '在读' : '非在读';
        },
      },
      {
        title: '学管师',
        dataIndex: 'teacherName',
        width: COL_WIDTH_M,
        render: (text, record) => {
          const { productTypeId } = this.props.params.activity || {};
          const studyManagerName = productTypeId === 'SMALL_CLASS' ? record.teacherName : record.studyManegerName;
          if (!studyManagerName) {
            return '';
          }
          const filterTeacherName = Array.from(new Set(studyManagerName.split(',')));
          return filterTeacherName.join(',');
        },
      },
      { title: '跟进人', dataIndex: 'followupUserName', width: COL_WIDTH_M },
      { title: '邀请人', dataIndex: 'inviteUserName', width: COL_WIDTH_M },
      {
        title: '实付金额',
        dataIndex: 'totalAmount',
        width: COL_WIDTH_M,
        render: (text) => +text / 100,
      },
      {
        title: '订单状态',
        dataIndex: 'payStatus',
        width: COL_WIDTH_M,
        render: (text, record) => {
          let { payStatus, surrenderStatus } = record;
          surrenderStatus = String(+surrenderStatus + 10);
          return <span>{payStatusObj[surrenderStatus] ? payStatusObj[surrenderStatus] : payStatusObj[payStatus]}</span>;
        },
        filterDropdownVisible: false,
        filterIcon: (
          <Icon
            className="order-list-icon"
            type="question_circle"
            size={12}
            onMouseOver={() => this.setState({ orderStatusModal: true, orderModalType: 'order' })}
            onClick={() => this.setState({ orderStatusModal: true, orderModalType: 'order' })}
          />
        ),
        filterDropdown: <span className="empty-content"> </span>,
      },
      {
        title: '审核状态',
        dataIndex: 'auditStatus',
        width: COL_WIDTH_M,
        render: this.renderStatus,
        filterDropdownVisible: false,
        filterIcon: (
          <Icon
            className="order-list-icon"
            type="question_circle"
            size={12}
            onMouseOver={() =>
              this.setState({
                orderStatusModal: true,
                orderModalType: 'refund',
              })
            }
            onClick={() =>
              this.setState({
                orderStatusModal: true,
                orderModalType: 'refund',
              })
            }
          />
        ),
        filterDropdown: <span className="empty-content"> </span>,
      },
      { title: '年级', dataIndex: 'gradeName', width: COL_WIDTH_S },
      { title: '在读学校', dataIndex: 'schoolName', width: COL_WIDTH_M },
      { title: '订单号', dataIndex: 'orderNo', width: 150 },
      {
        title: '创建时间',
        dataIndex: 'modifyTime',
        width: COL_WIDTH_L,
        render: (text, record) => {
          return record.createTime ? moment(record.createTime).format(dateFormat) : '';
        },
      },
      {
        title: '产品名称',
        dataIndex: 'productName',
        key: 'productName',
        width: COL_WIDTH_M,
        render: renderMultiLine,
      },
      {
        title: '报课信息',
        dataIndex: 'extList',
        key: 'extList',
        width: COL_WIDTH_M,
        render: (_, row) => {
          let extList = row.extList || [];
          if (!extList.length) {
            return '-';
          }
          return (
            <div className={commonStyles.extListWrap}>
              {extList.map((obj, index) => (
                <span
                  className={commonStyles.extListItem}
                  key={`${obj.key}${index}`}
                >{`${obj.key}：${obj.value}`}</span>
              ))}
            </div>
          );
        },
      },
    ];
    if (isFightGroup) {
      this.columns.splice(11, 0, {
        title: '拼团状态',
        key: 'grouponStatus',
        width: COL_WIDTH_M,
        render: (text, record) => {
          const { payStatus, grouponStatus, isGroupon } = record;
          // 未支付并且活动订单是拼团的 那么返回未成团
          if (+payStatus === 0 && isGroupon !== 2) {
            return '未成团';
          }
          return groupStatusObj[grouponStatus] || '未成团';
        },
      });
    }
    if (
      [
        'fe53122a5ceb46fb5340af911eb5e027',
        '39cbf184238405128f3b3d5259554a6f',
        'bc6d66557ea5b6967d81be5ff9c5cf32',
      ].indexOf(activity.activityTypeId) > -1
    ) {
      this.columns.push({
        title: '报读班级',
        dataIndex: 'className',
        render: renderMultiLine,
        width: COL_WIDTH_M,
      });
    }
    this.columns.push({
      title: '操作',
      key: 'action',
      width: COL_WIDTH_M,
      fixed: 'right',
      getActionConfig: (text, record) => {
        const {
          followupUserId,
          distributeUserId,
          auditStatus,
          payStatus,
          surrenderStatus,
          cancelReason,
          intentionCampusId,
          grouponStatus,
        } = record;
        const { userId, roleCode, organizationId } = this.props.user;
        const { btnAuth, questionnaireBtnAuth, params } = this.props;
        const { XINGHUOCampusXQYYZROrg } = this.state;
        const isActivityEnabled = params.activity && params.activity.status === 1;
        let btn = [];

        btn.push({
          text: '更多信息',
          onClick: () => {
            this.switchModalStdQuestionnaire(record.studentNo, record.studentName);
          },
          disabled: questionnaireBtnAuth,
        });
        // 该订单的支付是否拼团
        const isGroupPaid = grouponStatus === 1 || grouponStatus === 0;
        if (isGroupPaid) {
          btn.push({
            text: '拼团记录',
            onClick: () => {
              this.setState({ fightGroupModal: true, selectedOrder: record });
            },
          });
        }
        if (Index.getLoginUser().institution === xinghuoInstitution) {
          btn.push({
            text: '报告',
            onClick: () => {
              console.log(record);
              this.setState({ isReporter: true });
              TeachingReporter = withTeachingReporterHOC(
                record.bossStudentNo,
                this.closeReporterModal,
                record.studentName,
              );
            },
          });
        }
        // 未审核 0 并且未申请过退款 0 就可以转校 不需要跟进
        if (+auditStatus === 0 && +surrenderStatus === 0 && btnAuth.indexOf('BTN_ACTIVITY_TRANSFER') > -1) {
          btn.unshift({
            text: '转校',
            onClick: (e) => {
              e.stopPropagation();
              this.setState({
                transferSchoolModal: true,
                selectedOrder: record,
              });
            },
          });
        }
        btn.unshift({
          text: '跟进记录',
          onClick: (e) => {
            e.stopPropagation();
            this.setState({
              isShowOrderRemarkModal: true,
              selectedOrder: record,
            });
          },
        });
        // 星火还只能规定为学习中心组织架构才可以
        // 双师不用规定

        //未审核

        // 如果是双师培优不限制必须要有意向学习中心的意向学习中心主任才可以进行审核
        const isXingHuoCanDistribute =
          !LIMIT_CAMPUS_DISTRIBUTE || XINGHUOCampusXQYYZROrg.indexOf(intentionCampusId) > -1;
        if (+auditStatus === 0) {
          if (!followupUserId) {
            // 未分配
            // 双师和培优只要有角色就可以进行分配
            return roleCode.indexOf(DISTRIBUTE_ROLE_CODE) > -1 && isXingHuoCanDistribute
              ? [
                  {
                    text: '分配',
                    onClick: (e) => {
                      e.stopPropagation();
                      this.setState({
                        selectedOrders: [record],
                        distrbuteModalVisible: true,
                      });
                    },
                  },
                  ...btn,
                ]
              : btn;
          }
          //已分配
          if (roleCode.indexOf(DISTRIBUTE_ROLE_CODE) > -1 && isXingHuoCanDistribute) {
            btn.push({
              text: '重新分配',
              onClick: (e) => {
                e.stopPropagation();
                this.setState({
                  selectedOrders: [record],
                  distrbuteModalVisible: true,
                });
              },
            });
          }
        }
        if (+auditStatus === 1 || +auditStatus === 3) {
          return btn;
        }
        if (cancelReason) {
          btn.push({
            text: '查看',
            onClick: (e) => {
              e.stopPropagation();
              this.setState({
                selectedOrder: record,
                failReasonModalVisible: true,
              });
            },
          });
        }
        if (followupUserId === userId) {
          // 审核未通过 并且不处于申请退款或者退款中状态
          if (+auditStatus === 2 && +surrenderStatus !== 2 && +surrenderStatus !== 1) {
            btn.unshift({
              text: '重审',
              onClick: (e) => {
                e.stopPropagation();
                this.onReopenClick(record);
              },
            });
          }
          // 只要处于未审核状态都可以进行修改或者审核 支付不支付 订单状态不进行限制
          if (!+auditStatus) {
            btn.unshift({
              text: '审核',
              onClick: (e) => {
                e.stopPropagation();
                this.editStdData = {
                  id: record.studentNo,
                  name: record.studentName,
                  gradeId: record.gradeId,
                  schoolId: record.schoolNo,
                  campusId: record.intentionCampusId,
                  phone: record.phone,
                  orderNo: record.orderNo,
                  studentPhone: record.realStudentPhone,
                  studentId: record.bossStudentId,
                  blCampusId: record.blCampusId,
                };
                this.setState({
                  isClickAuditButton: true,
                  selectedOrder: record,
                });
                this.switchEditStdInfo();
              },
            });
            if (btnAuth.indexOf('BTN_EDIT_ACTIVITY_STUD') > -1) {
              btn.unshift({
                text: '修改',
                onClick: (e) => {
                  e.stopPropagation();
                  this.editStdData = {
                    id: record.studentNo,
                    name: record.studentName,
                    gradeId: record.gradeId,
                    schoolId: record.schoolNo,
                    campusId: record.intentionCampusId,
                    phone: record.phone,
                    orderNo: record.orderNo,
                    studentPhone: record.realStudentPhone,
                    studentId: record.bossStudentId,
                    blCampusId: record.blCampusId,
                  };
                  this.switchEditStdInfo();
                },
              });
            }
          }
          if (+payStatus === 1 && +auditStatus === 2 && (+surrenderStatus === 0 || +surrenderStatus === 4)) {
            btn.unshift({
              text: '申请退款',
              onClick: (e) => {
                e.stopPropagation();
                this.handleRefundButton(record);
              },
            });
          }
        }
        return btn;
      },
    });
  }

  componentDidMount() {
    this.initContent();
  }

  loadListData = async (params) => {
    const {
      name,
      followupUserName,
      bossStudentNo,
      intentionCampusId,
      gradeId,
      inviteUserId,
      payStatus,
      auditStatus,
      classStatus,
      allocationFlag,
      blCampusId,
      studyManegerId,
      pageNo,
      pageSize,
      grouponStatus,
    } = this.state;
    const { activity } = this.props.params;
    // 如果大于10就是退款状态 不然就是支付状态
    const refundStatusOrPayStatus = surrenderStatusMap.decrease(payStatus);
    const currentAuditStatus = auditStatus.length ? auditStatus : null;
    let apiParams = {
      name: name || null,
      followupUserName: followupUserName || null,
      bossStudentNo: bossStudentNo || null,
      intentionCampusId: intentionCampusId || null,
      gradeId: gradeId || null,
      inviteUserId: inviteUserId.split('//')[1] || '',
      auditStatus: currentAuditStatus,
      classStatus,
      allocationFlag,
      blCampusId,
      studyManegerId,
      grouponStatus,
      pageNo,
      pageSize,
      activityId: [activity.id],
      ...refundStatusOrPayStatus,
      ...params,
    };
    apiParams['intentionCampusId'] = apiParams['intentionCampusId']
      ? [apiParams['intentionCampusId']]
      : this.defaultBlCampus;
    if (apiParams.name) {
      apiParams['orderNo'] = apiParams.name.length > 11 ? apiParams.name : null;
      apiParams['customerPhone'] = apiParams.name.length === 11 ? apiParams.name : null;
      apiParams['studentName'] = !+apiParams.name || apiParams.name.length < 11 ? apiParams.name : null;
    }
    delete apiParams.name;

    apiParams = Object.keys(apiParams).reduce((obj, ele) => {
      const value = apiParams[ele];
      if (value !== null && value !== '' && value !== void 0) {
        obj[ele] = apiParams[ele];
      }
      return obj;
    }, {});

    const res = await loader.loadOrderList(apiParams);
    if (!res) {
      return {
        records: [],
      };
    }
    if (res && res.records) {
      // 原来的productId没有剔除已经退款的产品id 这里进行剔除
      res.records.map((order) => {
        const { refundProductId = [], productId, productName, refundProductName = [] } = order;
        if (Array.isArray(refundProductId) && refundProductId.length) {
          // 未退款产品的id和名字
          order.productId = productId.filter((id) => !refundProductId.includes(id));
          order.productName = productName.filter((name) => !refundProductName.includes(name));
          order.originProductId = productId;
          order.originProductName = productName;
        }
        return order;
      });
    }
    return res;
  };

  // 获取邀请人列表 根据值搜索
  getFollowerDict = (key) => {
    debounce(this.searchDict, 600)(key);
  };

  searchDict = async (value) => {
    // 空字符串不查询 否则查出几万数据 不允许
    if (!/[^\s]/.test(value)) {
      return false;
    }
    try {
      const asyncCalls = [loader.loadFollowerDict({ showBelong: 'market', userName: value })];
      let [inviteUserDict] = await Promise.all(asyncCalls);
      inviteUserDict = inviteUserDict.filter((ele) => !!ele);
      console.log('inviteUserDict', inviteUserDict);
      this.setState({
        inviteUserDict,
      });
    } catch (e) {
      console.error(e);
    }
  };

  initContent = async () => {
    let activityProductMap = {};
    // 转校的学习中心数组
    let transferSchoolCampusDict = [];
    this.setState({ loading: true });
    const { activity, branchId } = this.props.params;
    try {
      const asyncCalls = [
        loadBranchCampusDict(),
        loadCommonDict('STUDENT_GRADE'),
        // loader.loadFollowerDict({showBelong: 'market'}),
        loader.loadActivityProductList({ productIds: activity.productId }),
        getManagerDict({ roleCode: 'STUDY_MANAGER,STUDY_MANAGER_HEAD' }),
        loader.loadDetail({ id: activity.id }),
      ];
      let [
        campusDict,
        gradeDict,
        // inviteUserDict,
        productList,
        managerObj,
        activityDetail,
      ] = await Promise.all(asyncCalls);
      //获取列表数据
      campusDict = campusDict.filter((ele) => ele.val === branchId);
      // inviteUserDict = inviteUserDict.filter(ele => !!ele)
      this.defaultBlCampus =
        campusDict[0] && campusDict[0]['options'] ? campusDict[0]['options'].map((ele) => ele.val) : null;

      //学管师数据处理
      let _managerObj = managerObj.value;
      let managerArray = Object.keys(_managerObj);
      let managerDict = managerArray.map((v) => {
        return { text: _managerObj[v], val: v };
      });

      const res = await this.loadListData();
      //下拉框帅选的数据
      const productDict = productList.map((ele) => {
        return {
          text: ele.name,
          val: ele.id,
          product: ele,
          gradeId: ele.gradeId,
        };
      });
      if (activityDetail) {
        const { activityProductList, campus } = activityDetail.data;
        activityProductMap = activityProductList.reduce((obj, ele) => {
          obj[ele.product.id] = ele;
          return obj;
        }, {});
        transferSchoolCampusDict = campus.map(({ name, id }) => ({
          text: name,
          val: id,
        }));
      }
      this.setState({
        campusDict,
        gradeDict,
        // inviteUserDict,
        productDict,
        managerDict,
        activityProductMap,
        transferSchoolCampusDict,
        rows: res.records,
        total: res.total,
      });
    } catch (e) {
      console.error(e);
      showMessage(e.text, 'error');
    }
    this.setState({ loading: false });
  };

  search = async () => {
    this.setState({ listLoading: true });
    try {
      const res = await this.loadListData({ pageNo: 1 });
      this.setState({
        rows: res.records,
        total: res.total,
        pageNo: 1,
        selectedRowKeys: [],
      });
    } catch (e) {
      showMessage(e.text, 'error');
    }
    this.setState({
      listLoading: false,
    });
  };

  reset = async () => {
    this.setState({
      name: null,
      followupUserName: null,
      bossStudentNo: null,
      intentionCampusId: null,
      gradeId: null,
      blCampusId: null,
      classStatus: null,
      allocationFlag: null,
      studyManegerId: null,
      inviteUserId: '',
      payStatus: [],
      surrenderStatus: [],
      auditStatus: [],
      grouponStatus: null,
    });
  };

  pageChange = async ({ current, pageSize }) => {
    this.setState({ listLoading: true });
    try {
      const res = await this.loadListData({ pageNo: current, pageSize });
      this.setState({
        listLoading: false,
        rows: res.records,
        total: res.total,
        pageNo: current,
        pageSize,
        selectedRowKeys: [],
      });
    } catch (e) {
      showMessage(e.text, 'error');
    }
    this.setState({
      listLoading: false,
    });
  };

  renderStatus = (detail, record) => {
    const { auditStatus } = record;
    const color = auditStatusColorObj['' + auditStatus];
    return (
      <p key="asd" style={{ display: 'flex' }}>
        {!color ? '' : <b style={{ fontSize: '25px', color: color, marginRight: '3px' }}>·</b>}
        {auditStatusObj[auditStatus] || ''}
      </p>
    );
  };

  onExportClick = async (hasPhoneNumber, showRemark) => {
    this.setState({ loading: true });
    const {
      campusDict,
      name,
      intentionCampusId,
      gradeId,
      inviteUserId,
      payStatus,
      auditStatus,
      classStatus,
      allocationFlag,
      blCampusId,
      studyManegerId,
      grouponStatus,
      followupUserName,
      bossStudentNo,
    } = this.state;
    const { activity } = this.props.params;
    try {
      const refundStatusOrPayStatus = surrenderStatusMap.decrease(payStatus);
      const currentAuditStatus = auditStatus.length ? auditStatus : null;
      const apiParams = {
        name,
        followupUserName,
        bossStudentNo,
        intentionCampusId,
        gradeId,
        inviteUserId: inviteUserId.split('//')[1],
        auditStatus: currentAuditStatus,
        classStatus,
        allocationFlag,
        blCampusId,
        studyManegerId,
        activityId: [activity.id],
        grouponStatus,
        showPhoneFlag: hasPhoneNumber ? 1 : 0,
        showRemark: typeof showRemark === 'boolean' ? showRemark : null,
        ...refundStatusOrPayStatus,
      };
      const totalCampusDict = campusDict.reduce((array, parentObj) => {
        const childArray = Array.isArray(parentObj.options) ? parentObj.options.map((item) => item.val) : [];
        return [...array, ...childArray];
      }, []);
      apiParams['intentionCampusId'] = apiParams['intentionCampusId']
        ? [apiParams['intentionCampusId']]
        : totalCampusDict;
      if (apiParams.name) {
        apiParams['orderNo'] = apiParams.name.length > 11 ? apiParams.name : null;
        apiParams['customerPhone'] = apiParams.name.length === 11 ? apiParams.name : null;
        apiParams['studentName'] = !+apiParams.name ? apiParams.name : null;
      }
      delete apiParams.name;
      // 文件名称
      let fileName = `${activity.branchName}_${activity.name}`;
      if (showRemark) {
        fileName = `${activity.name}_跟进记录`;
      }
      await utils.exportExcel({
        apiUrl: 'MarketActivityController/exportActivityOrder',
        data: apiParams,
        fileName,
        success: () => {
          showMessage('导出成功！', 'success');
        },
        fail: (err) => {
          throw new Error(err);
        },
      });
    } catch (e) {
      showMessage('导出失败：' + e.resultMessage, 'error');
    }
    this.setState({ loading: false });
  };

  updateRow = (raw, changed) => {
    this.setState({ listLoading: true });
    const idx = this.state.rows.findIndex((ele) => ele.orderNo === raw.orderNo);
    const rows = this.state.rows.map((row, i) => {
      if (i === idx) {
        return { ...row, ...changed };
      }
      return row;
    });
    this.setState({
      listLoading: false,
      rows,
    });
    return { ...raw, ...changed };
  };

  onRowSelected = (keys) => {
    const { rows, XINGHUOCampusXQYYZROrg } = this.state;
    const { roleCode } = this.props.user;
    // 星火还只能规定为学习中心组织架构才可以
    // 双师不用规定
    const sortedKeys = keys.filter((key) => {
      const row = rows[key];
      const isXingHuoCanDistribute =
        !LIMIT_CAMPUS_DISTRIBUTE || XINGHUOCampusXQYYZROrg.indexOf(row.intentionCampusId) > -1;
      return row.auditStatus === 0 && roleCode.indexOf(DISTRIBUTE_ROLE_CODE) > -1 && isXingHuoCanDistribute;
    });

    const batchDistrbuteDisabled = keys.length ? sortedKeys.length !== keys.length : true;
    const selectedOrders = !batchDistrbuteDisabled ? keys.map((ele) => rows[ele]) : [];
    this.setState({
      selectedRowKeys: keys,
      selectedOrders,
      batchDistrbuteDisabled,
    });
  };

  onDistrbuteConfirm = (change) => {
    const selectedOrders = this.state.selectedOrders.map((ele) => {
      return this.updateRow(ele, change);
    });

    this.setState({
      selectedOrders: [],
      selectedRowKeys: [],
    });
  };
  //退款modal点击确定
  onRefundConfirm = async ({ refundReason, refundXQYYZR, selectProductArray }) => {
    const { selectedOrder, pageNo, pageSize } = this.state;
    this.setState({ listLoading: true });
    try {
      await applyRefund({
        orderNo: selectedOrder.orderNo,
        refundReason,
        auditUserId: refundXQYYZR.val,
        refundProductId: selectProductArray,
      });
      await this.pageChange({
        current: pageNo,
        pageSize,
      });
      showMessage('操作成功', 'success');
    } catch (e) {
      showMessage('操作失败：' + e.text, 'error');
    }
    this.setState({
      listLoading: false,
      refundModalVisible: false,
      orderDetail: {},
    });
  };
  // 点击退款获取营运主任打开modal
  handleRefundButton = async (record) => {
    const { intentionCampusId } = record;
    const orgId = this.props.params.branchId;
    const canSelectOrgIdArray = [intentionCampusId, orgId];
    try {
      const orderDetail = await this.getOrderDetail(record);
      const getRefundPersonFn = loader[REFUND_PERSON_LOADER_NAME];
      const data = await getRefundPersonFn({ orgId });

      // // 退款审核人的数据只筛选 分公司架构就是orgId ===orgId 或者orgId===intentionCampusId 意向学习中心的
      const handleRefundArray = data.filter((person) => canSelectOrgIdArray.indexOf(person.orgId) > -1);

      this.setState({
        orderDetail,
        refundModalVisible: true,
        selectedOrder: record,
        handleRefundXQYYZRArray: handleRefundArray,
      });
    } catch (e) {
      showMessage('获取数据失败：' + e.text, 'error');
    }
  };

  // 转校确定逻辑
  onTransferConfirm = async ({ intentionCampusId, studentNo }) => {
    const { activity } = this.props.params;
    const { selectedOrder, pageNo, pageSize } = this.state;
    this.setState({ listLoading: true });
    try {
      await loader.transSchool({
        orderNo: selectedOrder.orderNo,
        intentionCampusId,
        activityId: activity.id,
        studentNo,
      });
      await this.pageChange({
        current: pageNo,
        pageSize,
      });
      showMessage('操作成功', 'success');
    } catch (e) {
      showMessage('操作失败：' + e.text, 'error');
    }
    this.setState({
      listLoading: false,
      transferSchoolModal: false,
    });
  };
  onAuditConfirm = async (data) => {
    this.setState({ listLoading: true });
    const { selectedOrder, pageSize, pageNo } = this.state;
    const { productTypeId } = this.props.params.activity;
    // 是否双师或者小班 这两个要报班

    const isSmallOTwoAudis = ['SMALL_CLASS', 'TWO_TEACHER'].indexOf(productTypeId) > -1;
    const isPaid = +selectedOrder.payStatus === 1 && +selectedOrder.surrenderStatus !== 3;
    try {
      //审核情况分两种 审核成功与审核失败
      if (!data.auditStatus) {
        // 审核失败
        await Promise.all(
          [selectedOrder].map(async (ele) => {
            await loader.rejectOrder({
              cancelReason: data.failReason,
              orderNo: ele.orderNo,
            });
          }),
        );
      } else {
        // 审核成功种 第一种 小班或者双师并且已支付 并且 不是退款成功状态的话 那么需要填写报班以及课时数据 其他的情况不用填写小班以及课时数据
        if (isSmallOTwoAudis && isPaid) {
          //小班或者双师审核成功 已支付状态
          await this.onCourseConfirm(data);
        } else {
          // 其余的不用填写状态
          const updateAuditParam = data.updateAuditArray.map((product) => ({
            productId: product.val,
            classId: '',
            firstSchoolTime: '',
            studentIds: [],
            contractIds: [],
            contractProductIds: [],
            courseIds: [],
            orderNo: selectedOrder.orderNo,
            isGroupPromotion: data.isGroupPromotion,
            purchaseAgreement: selectedOrder.purchaseAgreement,
          }));
          await loader.auditOrder(updateAuditParam);
        }
      }
      await this.pageChange({
        current: pageNo,
        pageSize,
      });
      showMessage('操作成功', 'success');
      this.setState({
        auditModalVisible: false,
        selectedOrder: null,
      });
    } catch (e) {
      console.log(e);
      this.setState({
        auditModalVisible: false,
      });
      showMessage('提交审核失败：' + e.text.replace('999-系统错误，请联系管理员:', ''), 'error');
    }
    this.setState({
      listLoading: false,
    });
  };

  onReopenClick = async (data) => {
    const { pageNo, pageSize } = this.state;
    this.setState({ listLoading: true });
    try {
      await loader.reopenOrder({
        orderNo: data.orderNo,
      });
      await this.pageChange({
        current: pageNo,
        pageSize,
      });
      this.setState({
        selectedOrder: data,
        auditModalVisible: true,
      });
    } catch (e) {
      showMessage('操作失败：' + e.text, 'error');
    }
    this.setState({
      listLoading: false,
    });
  };

  // 小班或者双师选中课时以及班级后的审核
  onCourseConfirm = async (auditData) => {
    //审核通过
    const { selectedOrder, pageNo, pageSize } = this.state;
    this.setState({
      listLoading: true,
    });
    const { updateAuditArray, isGroupPromotion } = auditData;
    // 分两种情况双师和小班
    const updateAuditParam = updateAuditArray.map((productItem) => {
      const { val, hasSelectedClass, hasSelectedCourse } = productItem;
      return {
        productId: val,
        // hasSelectedClass不存在就没有排课
        classId: hasSelectedClass ? hasSelectedClass.classId : null,
        firstSchoolTime: hasSelectedCourse[0] ? hasSelectedCourse[0].courseDate : '',
        studentIds: [],
        contractIds: [],
        contractProductIds: [],
        courseIds: hasSelectedCourse.map((ele) => ele.classCourseId),
        orderNo: selectedOrder.orderNo,
        isGroupPromotion,
        purchaseAgreement: selectedOrder.purchaseAgreement,
      };
    });
    await loader.auditOrder(updateAuditParam);
    this.setState({
      selectedOrder: null,
      auditModalVisible: false,
    });
    await this.pageChange({
      current: pageNo,
      pageSize,
    });
    this.setState({
      listLoading: false,
      auditModalVisible: false,
    });
  };

  listRowClick = (record, index) => {
    const selectedRowKeys = this.state.selectedRowKeys.slice();
    const currentSelectedRowKeys = selectedRowKeys.includes(index)
      ? selectedRowKeys.filter((rowIndex) => rowIndex !== index)
      : selectedRowKeys.concat(index).sort((a, b) => a - b);
    this.onRowSelected(currentSelectedRowKeys);
  };
  // 切换显示修改学生信息弹窗
  switchEditStdInfo() {
    this.setState(
      (prevState) => ({
        editStdInfoVisible: !prevState.editStdInfoVisible,
      }),
      () => {
        if (!this.state.editStdInfoVisible) this.editStdData = null;
      },
    );
  }
  // 提交修改学生信息回调
  async afterSubmitEditStdInfo() {
    this.switchEditStdInfo();
    if (this.state.isClickAuditButton) {
      this.setState({
        isClickAuditButton: false,
        auditModalVisible: true,
      });
      return;
    }
    this.setState({ listLoading: true });
    const res = await this.loadListData();
    this.setState({
      rows: res.records,
      total: res.total,
      selectedRowKeys: [],
      listLoading: false,
    });
  }
  // 切换更多信息弹窗显隐
  switchModalStdQuestionnaire(studentNo, studentName) {
    if (studentNo && studentName) {
      this.studentNo = studentNo;
      this.studentName = studentName;
    }
    this.setState(
      (prevState) => ({
        visibleStdQuestionnaire: !prevState.visibleStdQuestionnaire,
      }),
      () => {
        if (!this.state.visibleStdQuestionnaire) {
          this.studentNo = null;
          this.studentName = null;
        }
      },
    );
  }

  // 批量分配
  handleGroupDistribute = () => {
    const { selectedOrders } = this.state;
    // 批量分配只允许同一学习中心的批量分配
    const selectedCampus = Array.from(new Set(selectedOrders.map((order) => order.intentionCampusId)));
    if (selectedCampus.length > 1) {
      // 存在多个学习中心
      showMessage('操作失败：请选择同一学习中心的订单进行分配', 'error');
      return false;
    }
    this.setState({
      distrbuteModalVisible: true,
    });
  };

  // 关闭报告Modal
  closeReporterModal = () => {
    this.setState({ isReporter: false });
  };

  // 获取订单详情数据
  getOrderDetail = async (records) => {
    try {
      const res = await loader.getOrderDetail({
        orderNo: records.orderNo,
      });
      return res.data;
    } catch (e) {
      throw { text: e.text };
    }
  };

  render() {
    const {
      loading,
      searchExpand,
      listLoading,
      rows,
      pageNo,
      pageSize,
      total,
      selectedRowKeys,
      handleRefundXQYYZRArray,
      name,
      followupUserName,
      bossStudentNo,
      intentionCampusId,
      gradeId,
      inviteUserId,
      payStatus,
      auditStatus,
      campusDict,
      gradeDict,
      inviteUserDict,
      blCampusId,
      classStatus,
      auditModalVisible,
      studyManegerId,
      productDict,
      managerDict,
      selectedOrders,
      orderDetail,
      transferSchoolCampusDict,
      failReasonModalVisible,
      refundModalVisible,
      selectedOrder,
      distrbuteModalVisible,
      batchDistrbuteDisabled,
      editStdInfoVisible,
      transferSchoolModal,
      orderStatusModal,
      orderModalType,
      activityProductMap,
      fightGroupModal,
      visibleStdQuestionnaire,
      grouponStatus,
      groupStatusDict,
      isFightGroup,
      allocationFlag,
      isReporter,
      isClickAuditButton,

      isShowOrderRemarkModal,
    } = this.state;
    const {
      route,
      btnAuth,
      params: { branchId, activity },
      user,
    } = this.props;

    const isShowExportOrderBtn = btnAuth.indexOf('EXPORT_ACTIVITY_ORDER') > -1;
    const isShowBtnWithoutPhone = btnAuth.indexOf('EXPORT_ACTIVITY_NO_PHONE') > -1;
    const isShowBtnOrderRemark = btnAuth.indexOf('EXPORT_ACTIVITY_REMARK') > -1;
    const showButtonWrap = isShowExportOrderBtn || isShowBtnWithoutPhone || isShowBtnOrderRemark;
    const exportBtnArray = [];

    if (isShowExportOrderBtn) {
      exportBtnArray.push({
        text: '含手机号码导出',
        onClick: () => this.onExportClick(true),
      });
    }
    if (isShowBtnWithoutPhone) {
      exportBtnArray.push({
        text: '不含手机号码导出',
        onClick: () => this.onExportClick(false),
      });
    }

    if (isShowBtnOrderRemark) {
      // 是否展示跟进记录数据
      exportBtnArray.push({
        text: '跟进记录导出',
        onClick: () => this.onExportClick(false, true),
      });
    }

    return (
      <Page
        loading={loading}
        route={route}
        fitWindowHeight={true}
        subHeader={
          <div className={commonStyles.searchBar} style={{ paddingRight: '29px' }}>
            <div
              className={commonStyles.addShare}
              style={{
                display: 'flex',
                justifyContent: 'flex-start',
                width: 235,
              }}
            >
              <Button type="primary" onClick={this.handleGroupDistribute} disabled={batchDistrbuteDisabled}>
                批量分配
              </Button>
              {showButtonWrap && (
                <ButtonWrap
                  btnStyle={{ marginLeft: 15 }}
                  btnText="导出Excel"
                  popPlacement="bottom"
                  actions={exportBtnArray}
                />
              )}
            </div>
            <div>
              <SearchBar
                onSearch={this.search}
                onReset={this.reset}
                expand={searchExpand}
                onToggleExpand={(expand) => this.setState({ searchExpand: expand })}
              >
                <Input
                  placeholder="订单编号/学生名称/客户号码"
                  value={name}
                  onChange={(e) => this.setState({ name: e.target.value })}
                />
                <Input
                  placeholder="跟进人"
                  value={followupUserName}
                  onChange={(e) => this.setState({ followupUserName: e.target.value })}
                />
                <Input
                  placeholder="准考证号"
                  value={bossStudentNo}
                  onChange={(e) => this.setState({ bossStudentNo: e.target.value })}
                />
                <Select
                  placeholder="意向学习中心"
                  dict={campusDict}
                  value={intentionCampusId}
                  allowClear={true}
                  showSearch={true}
                  onChange={(intentionCampusId) => this.setState({ intentionCampusId })}
                />
                <Select
                  placeholder="学籍学习中心"
                  dict={studentStatusCampusDict}
                  value={blCampusId}
                  allowClear={true}
                  showSearch={true}
                  onChange={(blCampusId) => this.setState({ blCampusId })}
                />
                <Select
                  placeholder="是否在读"
                  dict={attendSchoolDict}
                  value={classStatus}
                  allowClear={true}
                  showSearch={true}
                  onChange={(classStatus) => this.setState({ classStatus })}
                />
                <Select
                  placeholder="学管师"
                  showSearch={true}
                  value={studyManegerId}
                  dict={managerDict}
                  allowClear={true}
                  onChange={(studyManegerId) => this.setState({ studyManegerId })}
                />
                <Select
                  placeholder="选择年级"
                  showSearch={true}
                  dict={gradeDict}
                  value={gradeId}
                  onChange={(gradeId) => this.setState({ gradeId })}
                />
                <Select
                  placeholder="邀请人"
                  searchNotFoundTips="该老师未分享引流活动"
                  showSearch={true}
                  value={inviteUserId}
                  dict={inviteUserDict}
                  allowClear={true}
                  onChange={(inviteUserId) =>
                    this.setState({
                      inviteUserId: inviteUserId ? inviteUserId : '',
                    })
                  }
                  onSearchKeyChange={this.getFollowerDict}
                />
                <MultiSelect
                  placeholder="订单状态"
                  dict={payStatusDict}
                  value={payStatus}
                  allowClear={true}
                  showSearch={true}
                  onChange={(payStatus) => this.setState({ payStatus: payStatus || [] })}
                />
                <MultiSelect
                  placeholder="审核状态"
                  dict={auditStatusDict}
                  value={auditStatus}
                  allowClear={true}
                  showSearch={true}
                  onChange={(auditStatus) => this.setState({ auditStatus: auditStatus || [] })}
                />
                <Select
                  placeholder="是否分配"
                  value={allocationFlag}
                  dict={distributeDict}
                  allowClear={true}
                  onChange={(allocationFlag) => this.setState({ allocationFlag })}
                />
                {isFightGroup && (
                  <Select
                    placeholder="拼团状态"
                    showSearch={true}
                    value={grouponStatus}
                    dict={groupStatusDict}
                    allowClear={true}
                    onChange={(grouponStatus) => this.setState({ grouponStatus })}
                  />
                )}
              </SearchBar>
            </div>
          </div>
        }
      >
        <List
          columns={this.columns}
          loading={listLoading}
          dataSource={rows}
          pagination={{
            current: pageNo,
            pageSize,
            total: total,
          }}
          onChange={this.pageChange}
          rowSelection={{
            selectedRowKeys,
            onChange: this.onRowSelected,
          }}
          onRowClick={this.listRowClick}
          className={commonStyles.activityOrderList}
        />
        {auditModalVisible && (
          <AuditModal
            productDict={productDict}
            order={selectedOrder}
            branchId={branchId}
            productTypeId={activity.productTypeId}
            onClose={() => {
              this.pageChange({
                current: pageNo,
                pageSize,
              })
                .then(() => {
                  this.setState({
                    selectedOrder: null,
                    auditModalVisible: false,
                  });
                })
                .catch(() => {
                  this.setState({
                    selectedOrder: null,
                    auditModalVisible: false,
                  });
                });
            }}
            onConfirm={this.onAuditConfirm}
          />
        )}
        {failReasonModalVisible && (
          <FailReasonModal order={selectedOrder} onClose={() => this.setState({ failReasonModalVisible: false })} />
        )}
        {distrbuteModalVisible && (
          <DistrbuteModal
            managerDict={managerDict}
            orders={selectedOrders}
            onClose={() => this.setState({ distrbuteModalVisible: false })}
            onConfirm={this.onDistrbuteConfirm}
          />
        )}
        {refundModalVisible && (
          <RefundModal
            activityProductMap={activityProductMap}
            order={selectedOrder}
            orderDetail={orderDetail}
            refundXQYYZRArray={handleRefundXQYYZRArray}
            onClose={() => this.setState({ refundModalVisible: false })}
            onConfirm={this.onRefundConfirm}
          />
        )}
        {editStdInfoVisible && (
          <ModalEditStdInfo
            name={isClickAuditButton ? '请核对学生基础信息（务必认真核对）' : ''}
            okText={isClickAuditButton ? '我已完善并核对学生信息，保存并进入下一步' : ''}
            isAudit={isClickAuditButton}
            data={this.editStdData}
            onCancel={() => {
              this.setState({
                isClickAuditButton: false,
              });
              this.switchEditStdInfo();
            }}
            callback={this.afterSubmitEditStdInfo}
          />
        )}
        {transferSchoolModal && (
          <TransferSchoolModal
            order={selectedOrder}
            campusDict={transferSchoolCampusDict}
            onClose={() => this.setState({ transferSchoolModal: false })}
            onConfirm={this.onTransferConfirm}
          />
        )}
        {orderStatusModal && (
          <OrderStatusModal
            onClose={() => this.setState({ orderStatusModal: false })}
            orderModalType={orderModalType}
          />
        )}
        {fightGroupModal && (
          <FightGroupModal onClose={() => this.setState({ fightGroupModal: false })} order={selectedOrder} />
        )}
        {visibleStdQuestionnaire && (
          <ModalStdQuestionnaire
            activityId={activity.id}
            studentNo={this.studentNo}
            studentName={this.studentName}
            onCancel={this.switchModalStdQuestionnaire}
          />
        )}
        {isReporter && <TeachingReporter />}
        {isShowOrderRemarkModal && (
          <OrderRemarkModal
            externalOrderId={selectedOrder.orderNo}
            user={user}
            onCancel={() => {
              this.setState({ isShowOrderRemarkModal: false });
            }}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  let data = state.main.pageParams;
  let pathName = state.routing.locationBeforeTransitions.pathname;
  return {
    params: data[pathName] || {},
    user: state.account.user,
    btnAuth: state.account.btnAuths || [],
    questionnaireBtnAuth: state.account.btnAuths.indexOf('ORDER_QUESTIONNAIRE_DETAIL') === -1,
    userOrgRoles: state.account.userRoleInfo.userOrgRoles,
  };
};
export default connect(mapStateToProps, null)(MarketActivityOrder);
