import React from 'react';
import { connect } from 'dva';
import Button from 'COMPONENT/Button';
import Page from 'COMPONENT/Page';
import List from 'COMPONENT/List';
import SearchBar from '../components/NewSearchBar';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import Input from 'COMPONENT/Input';
import Icon from 'COMPONENT/Icon';
import Popover from 'COMPONENT/Popover';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { getManagerDict } from 'SERVICE/followReport';
import loader from './loader';
import commonStyles from '../common/commonStyle.less';
import moment from 'moment';
import AuditModal from './modal/AuditModal';
import FailReasonModal from './modal/FailReasonModal';
import RefundModal from './modal/RefundModal';
import DistrbuteModal from './modal/DistrbuteModal';
import TransferSchoolModal from './modal/TransferSchoolModal';
import OrderStatusModal from './modal/OrderStatusModal';
import FightGroupModal from './modal/FightGroupModal';
import { applyRefund } from '../RefundDetail/loader';
import { renderMultiLine } from '../common';
import ModalEditStdInfo from './ModalEditStdInfo';
import ModalStdQuestionnaire from '../Questionnaire/ModalStdQuestionnaire';
import { SYSTEM_MARK } from '../../../constants/config';
import { roleCodeFeatureMap, formatIntentionCampusId } from './utilts';
import ButtonWrap from '../../signUpOnline/StdntApptMgmt/ButtonWrap';
import { xinghuoInstitution } from 'CONST/config';
import withTeachingReporterHOC from 'PAGE/teachingReporterHOC/teachingReporterHOC';
import OrderRemarkModal from './components/OrderRemarkModal';
import { debounce } from 'lodash';
import { DatePicker } from 'antd';
import { loadCommonDict, loadBranchCampusDict, loadBranchDict } from 'UTIL/dict';
const { RangePicker } = DatePicker;

let TeachingReporter = null;

const { REFUND_PERSON_LOADER_NAME } = roleCodeFeatureMap[SYSTEM_MARK];

const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L } = List;
const dateFormat = 'YYYY-MM-DD HH:mm';
// 分配字典
const distributeDict = [
  { text: '已分配', val: 'HAS_ALLOCATION' },
  { text: '未分配', val: 'NOT_ALLOCATION' },
];
const auditStatusDict = [
  { text: '待审核', val: '0' },
  { text: '审核通过', val: '1' },
  { text: '审核未过', val: '2' },
  { text: '已报读', val: '3' },
];
// 学籍校区字典
const studentStatusCampusDict = [
  { text: '本校区', val: '1' },
  { text: '其他校区', val: '2' },
];
// 是否在读字典
const attendSchoolDict = [
  { text: '在读', val: '1' },
  { text: '非在读', val: '2' },
];
const surrenderStatusMap = {
  dict: [
    { text: '申请退款中', val: '1' },
    { text: '退款中', val: '2' },
    { text: '退款成功', val: '3' },
    { text: '退款失败', val: '4' },
  ],
  increase(status) {
    if (Array.isArray(status)) {
      return status.map((item) => ({ ...item, val: String(+item.val + 10) }));
    }
    // 单一 "0" 返回 “10”
    return String(+status + 10);
  },
  // payStatus解析方法 传入的是数组
  decrease(statusArray) {
    // 没选择状态直接返回
    if (!statusArray.length) {
      return {};
    }
    const payStatus = [];
    const surrenderStatus = [];
    statusArray.forEach((status) => {
      if (+status >= 10) {
        // 退款状态
        surrenderStatus.push(Number(status - 10));
      } else {
        // 支付状态
        payStatus.push(status);
      }
    });
    return {
      payStatus: payStatus,
      surrenderStatus: surrenderStatus,
    };
  },
};
const payStatusDict = [
  { text: '未支付', val: '0' },
  { text: '已支付', val: '1' },
  ...surrenderStatusMap.increase(surrenderStatusMap.dict),
];
const auditStatusColorObj = {
  0: '#a4a0ff',
  1: '#d8d8d8',
  2: '#ff5f5f',
  3: '#d8d8d8',
};
//1：已成团；0：未成团；空 ：单独购买
const groupStatusDict = [
  { text: '未成团', val: '0' },
  { text: '已成团', val: '1' },
  { text: '单独购买', val: '2' },
];
function dictToObj(dict) {
  return (dict || []).reduce((obj, ele) => {
    obj[ele.val] = ele.text;
    return obj;
  }, {});
}
const groupStatusObj = dictToObj(groupStatusDict);
const auditStatusObj = dictToObj(auditStatusDict);
const payStatusObj = dictToObj(payStatusDict);

class MarketActivityOrder extends React.Component {
  branchObj = {};
  activityTypeObj = {};

  constructor(props) {
    super(props);
    const { activity, searchData, branchId } = this.props.params;

    this.switchEditStdInfo = this.switchEditStdInfo.bind(this);
    this.afterSubmitEditStdInfo = this.afterSubmitEditStdInfo.bind(this);
    this.switchModalStdQuestionnaire = this.switchModalStdQuestionnaire.bind(this);

    this.state = {
      loading: true, // boolean 页面加载状态
      searchExpand: true,
      shareActivity: null,
      shareModalVisible: false,
      handleRefundXQYYZRArray: [], //校区营运主任数组 用于点击退款时
      // 订单编号/学生名称/客户号码
      name: searchData || null,
      time: null,
      followupUserName: null,
      intentionCampusId: null,
      gradeId: null,
      inviteUserId: '',
      payStatus: [],
      // payStatus 不仅代表着支付状态也代表着退款状态
      auditStatus: [],
      selectedRowKeys: [], //选中行的key的集合
      grouponStatus: null, // 拼团状态
      // 学生是否当前的学籍校区
      blCampusId: null,
      // 学生是否在读
      classStatus: null,
      // 是否已分配
      allocationFlag: null,
      //列表数据
      listLoading: false,
      //学管师
      studyManegerId: null,
      // 活动详情用于退款的产品金额计算
      activityProductMap: {},
      rows: [],
      pageNo: 1,
      pageSize: 20,
      total: 0,
      // orderDetail 申请退款订单详情
      orderDetail: {},
      //数据字典数据
      groupStatusDict,
      campusDict: [],
      gradeDict: [],
      inviteUserDict: [],
      managerDict: [],
      // 转校校区字典
      transferSchoolCampusDict: [],
      auditModalVisible: false,
      productDict: [],
      selectedOrders: [],
      failReasonModalVisible: false,
      refundModalVisible: false,
      selectedOrder: null,
      distrbuteModalVisible: false,

      batchDistrbuteDisabled: true,
      // 修改学生信息弹窗显示状态
      editStdInfoVisible: false,
      // 转校弹窗
      transferSchoolModal: false,
      orderStatusModal: false,
      orderModalType: null,
      // 更多信息弹窗
      visibleStdQuestionnaire: false,
      // 拼团记录
      fightGroupModal: false,
      isReporter: false, //是否打开入学测报告
      isClickAuditButton: false, //是否点击了审核按钮
      isShowOrderRemarkModal: false, //是否显示跟进记录弹窗
      activityName: '', // 活动名称
      branchId: branchId || null, // 分公司
      activityTypeId: null, // 活动类型
      branchDict: [],
      // 完整校区字典
      blCampusDict: [],
      activityId: activity.id,
    };
    // 更多信息弹窗props
    this.studentNo = null;
    this.studentName = null;

    this.editStdData = null;

    this.columns = [
      {
        title: '序号',
        dataIndex: 'index',
        width: 60,
        fixed: 'left',
        render: (text, record, index) => index + 1,
      },
      { title: '学生名称', dataIndex: 'studentName', width: 100 },
      {
        title: '准考证号',
        dataIndex: 'bossStudentNo',
        width: COL_WIDTH_L,
        render: (text) => text || '暂无',
        filterDropdownVisible: false,
        filterIcon: (
          <Popover content="用于教学平台的阅卷系统使用">
            <Icon type="question_circle" size={12} style={{ marginLeft: 2 }} />
          </Popover>
        ),
        filterDropdown: <span className="empty-content"> </span>,
      },
      { title: '客户号码', dataIndex: 'customerPhone', width: 120 },
      { title: '学生手机号码', dataIndex: 'studentPhone', width: 120 },
      { title: '意向校区', dataIndex: 'intentionCampusName', width: 160 },
      { title: '学籍校区', dataIndex: 'blCampusName', width: COL_WIDTH_M },
      { title: '活动名称', dataIndex: 'name', width: 200 },
      {
        title: '分公司',
        dataIndex: 'branchId',
        width: COL_WIDTH_M,
        render: (text, record) => {
          const { branchId } = record;
          return this.branchObj[branchId] || '';
        },
      },
      {
        title: '活动类型',
        dataIndex: 'activityTypeId',
        width: COL_WIDTH_M,
        render: (text, record) => {
          const { activityTypeId } = record;
          return this.activityTypeObj[activityTypeId] || '';
        },
      },
      {
        title: '是否在读',
        dataIndex: 'classStatus',
        width: COL_WIDTH_M,
        render: (text) => {
          return Number(text) === 1 ? '在读' : '非在读';
        },
      },
      {
        title: '学管师',
        dataIndex: 'teacherName',
        width: COL_WIDTH_M,
        render: (text, record) => {
          const { productTypeId } = record;
          const studyManagerName = productTypeId === 'SMALL_CLASS' ? record.teacherName : record.studyManegerName;
          if (!studyManagerName) {
            return '';
          }
          const filterTeacherName = Array.from(new Set(studyManagerName.split(',')));
          return filterTeacherName.join(',');
        },
      },
      { title: '跟进人', dataIndex: 'followupUserName', width: COL_WIDTH_M },
      { title: '邀请人', dataIndex: 'inviteUserName', width: COL_WIDTH_M },
      {
        title: '拼团状态',
        key: 'grouponStatus',
        width: COL_WIDTH_M,
        render: (text, record) => {
          const { payStatus, grouponStatus, isGroupon } = record;
          if (isGroupon === 2) {
            // 不允许拼团
            return '-';
          }
          // 未支付并且活动订单是拼团的 那么返回未成团
          if (+payStatus === 0 && isGroupon !== 2) {
            return '未成团';
          }
          return groupStatusObj[String(grouponStatus)] || '未成团';
        },
      },
      {
        title: '实付金额',
        dataIndex: 'totalAmount',
        width: COL_WIDTH_M,
        render: (text) => +text / 100,
      },
      {
        title: '订单状态',
        dataIndex: 'payStatus',
        width: COL_WIDTH_M,
        render: (text, record) => {
          let { payStatus, surrenderStatus } = record;
          surrenderStatus = String(+surrenderStatus + 10);
          return <span>{payStatusObj[surrenderStatus] ? payStatusObj[surrenderStatus] : payStatusObj[payStatus]}</span>;
        },
        filterDropdownVisible: false,
        filterIcon: (
          <Icon
            className="order-list-icon"
            type="question_circle"
            size={12}
            onMouseOver={() => this.setState({ orderStatusModal: true, orderModalType: 'order' })}
            onClick={() => this.setState({ orderStatusModal: true, orderModalType: 'order' })}
          />
        ),
        filterDropdown: <span className="empty-content"> </span>,
      },
      {
        title: '审核状态',
        dataIndex: 'auditStatus',
        width: COL_WIDTH_M,
        render: this.renderStatus,
        filterDropdownVisible: false,
        filterIcon: (
          <Icon
            className="order-list-icon"
            type="question_circle"
            size={12}
            onMouseOver={() =>
              this.setState({
                orderStatusModal: true,
                orderModalType: 'refund',
              })
            }
            onClick={() =>
              this.setState({
                orderStatusModal: true,
                orderModalType: 'refund',
              })
            }
          />
        ),
        filterDropdown: <span className="empty-content"> </span>,
      },
      { title: '年级', dataIndex: 'gradeName', width: COL_WIDTH_S },
      { title: '在读学校', dataIndex: 'schoolName', width: 180 },
      { title: '订单号', dataIndex: 'orderNo', width: 180 },
      {
        title: '创建时间',
        dataIndex: 'modifyTime',
        width: COL_WIDTH_L,
        render: (text, record) => {
          return record.createTime ? moment(record.createTime).format(dateFormat) : '';
        },
      },
      {
        title: '产品名称',
        dataIndex: 'productName',
        key: 'productName',
        width: 250,
        render: renderMultiLine,
      },
      {
        title: '报课信息',
        dataIndex: 'extList',
        key: 'extList',
        width: 280,
        render: (_, row) => {
          let extList = row.extList || [];
          if (!extList.length) {
            return '-';
          }
          return (
            <div className={commonStyles.extListWrap}>
              {extList.map((obj, index) => (
                <span
                  className={commonStyles.extListItem}
                  key={`${obj.key}${index}`}
                >{`${obj.key}：${obj.value}`}</span>
              ))}
            </div>
          );
        },
      },
      {
        title: '报读班级',
        dataIndex: 'className',
        render: renderMultiLine,
        width: 200,
      },
    ];

    this.columns.push({
      title: '操作',
      key: 'action',
      width: COL_WIDTH_M,
      fixed: 'right',
      getActionConfig: (text, record) => {
        const { followupUserId, auditStatus, payStatus, surrenderStatus, cancelReason, grouponStatus } = record;
        const { userId } = this.props.user;
        const { btnAuth, questionnaireBtnAuth } = this.props;

        let btn = [];

        btn.push({
          text: '更多信息',
          onClick: () => {
            this.setState({ selectedOrder: record });

            this.switchModalStdQuestionnaire(record.studentNo, record.studentName);
          },
          disabled: questionnaireBtnAuth,
        });
        // 该订单的支付是否拼团
        const isGroupPaid = grouponStatus === 1 || grouponStatus === 0;
        if (isGroupPaid) {
          btn.push({
            text: '拼团记录',
            onClick: () => {
              this.setState({ fightGroupModal: true, selectedOrder: record });
            },
          });
        }
        if (Index.getLoginUser().institution === xinghuoInstitution) {
          btn.push({
            text: '报告',
            onClick: () => {
              this.setState({ isReporter: true, selectedOrder: record });
              TeachingReporter = withTeachingReporterHOC(
                record.bossStudentNo,
                this.closeReporterModal,
                record.studentName,
              );
            },
          });
        }
        // 未审核 0 并且未申请过退款 0 就可以转校 不需要跟进
        if (+auditStatus === 0 && +surrenderStatus === 0 && btnAuth.indexOf('BTN_ACTIVITY_TRANSFER') > -1) {
          btn.unshift({
            text: '转校',
            onClick: async (e) => {
              e.stopPropagation();
              await this.getActivityDicts(record);
              this.setState({
                transferSchoolModal: true,
                selectedOrder: record,
              });
            },
          });
        }
        btn.unshift({
          text: '跟进记录',
          onClick: (e) => {
            e.stopPropagation();
            this.setState({
              isShowOrderRemarkModal: true,
              selectedOrder: record,
            });
          },
        });

        //未审核
        if (+auditStatus === 0 && btnAuth.indexOf('MARKET_ACTIVITY_DISTRIBUTE_ORDER') > -1) {
          if (!followupUserId) {
            // 未分配
            return [
              {
                text: '分配',
                onClick: (e) => {
                  e.stopPropagation();
                  this.setState({
                    selectedOrder: record,
                    selectedOrders: [record],
                    distrbuteModalVisible: true,
                  });
                },
              },
              ...btn,
            ];
          }
          //已分配
          btn.push({
            text: '重新分配',
            onClick: (e) => {
              e.stopPropagation();
              this.setState({
                selectedOrder: record,
                selectedOrders: [record],
                distrbuteModalVisible: true,
              });
            },
          });
        }
        if (+auditStatus === 1 || +auditStatus === 3) {
          return btn;
        }
        if (cancelReason) {
          btn.push({
            text: '查看',
            onClick: (e) => {
              e.stopPropagation();
              this.setState({
                selectedOrder: record,
                failReasonModalVisible: true,
              });
            },
          });
        }
        if (followupUserId === userId) {
          // 审核未通过 并且不处于申请退款或者退款中状态
          if (+auditStatus === 2 && +surrenderStatus !== 2 && +surrenderStatus !== 1) {
            btn.unshift({
              text: '重审',
              onClick: (e) => {
                e.stopPropagation();
                this.onReopenClick(record);
              },
            });
          }
          // 只要处于未审核状态都可以进行修改或者审核 支付不支付 订单状态不进行限制
          if (!+auditStatus) {
            btn.unshift({
              text: '审核',
              onClick: (e) => {
                e.stopPropagation();
                this.editStdData = {
                  phone: record.phone,
                  id: record.studentNo,
                  orderNo: record.orderNo,
                  name: record.studentName,
                  gradeId: record.gradeId,
                  schoolId: record.schoolNo,
                  blCampusId: record.blCampusId,
                  studentId: record.bossStudentId,
                  blCampusName: record.blCampusName,
                  studentPhone: record.realStudentPhone,
                  campusId: record.intentionCampusId,
                  campusName: record.intentionCampusName,
                };
                this.setState({
                  isClickAuditButton: true,
                  selectedOrder: record,
                });
                this.switchEditStdInfo();
              },
            });
            if (btnAuth.indexOf('BTN_EDIT_ACTIVITY_STUD') > -1) {
              btn.unshift({
                text: '修改',
                onClick: (e) => {
                  e.stopPropagation();
                  this.setState({ selectedOrder: record });
                  this.editStdData = {
                    phone: record.phone,
                    id: record.studentNo,
                    gradeId: record.gradeId,
                    orderNo: record.orderNo,
                    name: record.studentName,
                    schoolId: record.schoolNo,
                    blCampusId: record.blCampusId,
                    studentId: record.bossStudentId,
                    blCampusName: record.blCampusName,
                    campusId: record.intentionCampusId,
                    studentPhone: record.realStudentPhone,
                    campusName: record.intentionCampusName,
                  };
                  this.switchEditStdInfo();
                },
              });
            }
          }
          if (+payStatus === 1 && +auditStatus === 2 && (+surrenderStatus === 0 || +surrenderStatus === 4)) {
            btn.unshift({
              text: '申请退款',
              onClick: (e) => {
                e.stopPropagation();
                this.handleRefundButton(record);
              },
            });
          }
        }
        return btn;
      },
    });
  }

  componentDidMount() {
    this.initContent();
  }

  parseTime = (time) => {
    let [beginTime, endTime] = time;
    return {
      beginTime: moment(beginTime).format('YYYY-MM-DD HH:mm:ss'),
      endTime: moment(endTime).format('YYYY-MM-DD HH:mm:ss'),
    };
  };

  loadListData = async (params) => {
    const {
      time,
      name,
      followupUserName,
      bossStudentNo,
      intentionCampusId,
      gradeId,
      inviteUserId,
      payStatus,
      auditStatus,
      classStatus,
      allocationFlag,
      blCampusId,
      studyManegerId,
      pageNo,
      pageSize,
      grouponStatus,
      branchId,
      activityTypeId,
      activityName,
      activityId,
    } = this.state;

    // 开始和结束时间
    let beginTime;
    let endTime;
    if (time === null) {
      beginTime = null;
      endTime = null;
    } else {
      let timer = this.parseTime(time);
      beginTime = timer.beginTime;
      endTime = timer.endTime;
    }

    // 如果大于10就是退款状态 不然就是支付状态
    const refundStatusOrPayStatus = surrenderStatusMap.decrease(payStatus);
    const currentAuditStatus = auditStatus.length ? auditStatus : null;

    // 意向校区
    const intentionCampusIdParam = intentionCampusId
      ? [intentionCampusId]
      : formatIntentionCampusId(this.state.campusDict);

    let apiParams = {
      name: name ? name.trim() : null,
      followupUserName: followupUserName || null,
      bossStudentNo: bossStudentNo || null,
      intentionCampusId: intentionCampusIdParam,
      gradeId: gradeId || null,
      inviteUserId: inviteUserId.split('//')[1] || '',
      auditStatus: currentAuditStatus,
      branchId: branchId ? [branchId] : null,
      classStatus,
      allocationFlag,
      blCampusId,
      studyManegerId,
      grouponStatus,
      pageNo,
      pageSize,
      beginTime,
      endTime,
      activityTypeId,
      activityId: activityId ? [activityId] : null,
      ...refundStatusOrPayStatus,
      ...params,
    };

    if (apiParams.name) {
      apiParams['orderNo'] = apiParams.name.length > 11 ? apiParams.name : null;
      apiParams['customerPhone'] = apiParams.name.length === 11 ? apiParams.name : null;
      apiParams['studentName'] = !+apiParams.name || apiParams.name.length < 11 ? apiParams.name : null;
    }
    apiParams.name = activityName ? activityName.trim() : null;

    apiParams = Object.keys(apiParams).reduce((obj, ele) => {
      const value = apiParams[ele];
      if (value !== null && value !== '' && value !== void 0) {
        obj[ele] = apiParams[ele];
      }
      return obj;
    }, {});

    const res = await loader.loadOrderList(apiParams);
    if (!res) {
      return {
        records: [],
      };
    }
    if (res && res.records) {
      // 原来的productId没有剔除已经退款的产品id 这里进行剔除
      res.records.map((order) => {
        const { refundProductId = [], productId, productName, refundProductName = [] } = order;
        if (Array.isArray(refundProductId) && refundProductId.length) {
          // 未退款产品的id和名字
          order.productId = productId.filter((id) => !refundProductId.includes(id));
          order.productName = productName.filter((name) => !refundProductName.includes(name));
          order.originProductId = productId;
          order.originProductName = productName;
        }
        return order;
      });
    }
    return res;
  };

  // 获取邀请人列表 根据值搜索
  getFollowerDict = (key) => {
    debounce(this.searchDict, 600)(key);
  };

  searchDict = async (value) => {
    // 空字符串不查询 否则查出几万数据 不允许
    if (!/[^\s]/.test(value)) {
      return false;
    }
    try {
      const asyncCalls = [loader.loadFollowerDict({ showBelong: 'market', userName: value })];
      let [inviteUserDict] = await Promise.all(asyncCalls);
      inviteUserDict = inviteUserDict.filter((ele) => !!ele);
      this.setState({
        inviteUserDict,
      });
    } catch (e) {}
  };

  initContent = async () => {
    this.setState({ loading: true });
    const { branchId } = this.state;
    try {
      const asyncCalls = [
        loadBranchCampusDict(),
        loadCommonDict('STUDENT_GRADE'),
        getManagerDict({ roleCode: 'STUDY_MANAGER,STUDY_MANAGER_HEAD' }),
        loadBranchDict(),
        loadCommonDict('MARKET_ACTIVITY_TYPE'),
      ];
      let [blCampusDict, gradeDict, managerObj, branchDict, activityTypeDict] = await Promise.all(asyncCalls);

      this.branchObj = this.dictToObj(branchDict);
      this.activityTypeObj = this.dictToObj(activityTypeDict);

      //默认第一个分公司，当前年份
      this.defaultBranch = branchId ? branchId : branchDict[0] ? branchDict[0].val : null;

      // 初始化字典
      const campusDict = blCampusDict.filter((ele) => ele.val === this.defaultBranch);
      const intentionCampusId = formatIntentionCampusId(campusDict);

      //学管师数据处理
      let _managerObj = managerObj.value;
      let managerArray = Object.keys(_managerObj);
      let managerDict = managerArray.map((v) => {
        return { text: _managerObj[v], val: v };
      });

      // 获取订单列表
      const res = await this.loadListData({
        branchId: [this.defaultBranch],
        intentionCampusId,
      });

      this.setState({
        branchDict,
        blCampusDict,
        campusDict,
        gradeDict,
        managerDict,
        activityTypeDict,
        branchId: this.defaultBranch,
        rows: res.records,
        total: res.total,
      });
    } catch (e) {
      showMessage(e.text, 'error');
    }
    this.setState({ loading: false });
  };

  async getActivityDicts(selectedOrder) {
    const id = selectedOrder.activityId;

    try {
      const activityDetail = await loader.loadDetail({ id });
      const productList = await loader.loadActivityProductList({
        productIds: activityDetail.data.productId,
      });

      //下拉框帅选的数据
      const productDict = productList.map((ele) => {
        return {
          text: ele.name,
          val: ele.id,
          product: ele,
          gradeId: ele.gradeId,
        };
      });
      let activityProductMap = {};
      // 转校的校区数组
      let transferSchoolCampusDict = [];
      if (activityDetail) {
        const { activityProductList, campus } = activityDetail.data;
        activityProductMap = activityProductList.reduce((obj, ele) => {
          obj[ele.product.id] = ele;
          return obj;
        }, {});
        transferSchoolCampusDict = campus.map(({ name, id }) => ({
          text: name,
          val: id,
        }));
      }
      this.setState({
        productDict,
        activityProductMap,
        transferSchoolCampusDict,
      });
    } catch (err) {}
  }

  /**
   * 切换分公司
   *
   * @param val
   */
  onBranchChange = (val) => {
    const campusDict = (this.state.blCampusDict || []).filter((ele) => ele.val === val);

    this.setState({
      branchId: val,
      campusDict,
      intentionCampusId: null,
    });
  };

  dictToObj = (dict) => {
    return (dict || []).reduce((obj, ele) => {
      obj[ele.val] = ele.text;
      return obj;
    }, {});
  };

  search = async () => {
    this.setState({ listLoading: true, activityId: null }, async () => {
      try {
        const res = await this.loadListData({ pageNo: 1 });
        this.setState({
          rows: res.records,
          total: res.total,
          pageNo: 1,
          selectedRowKeys: [],
        });
      } catch (e) {
        showMessage(e.text, 'error');
      }
      this.setState({
        listLoading: false,
      });
    });
  };

  reset = async () => {
    this.setState({
      name: null,
      followupUserName: null,
      bossStudentNo: null,
      intentionCampusId: null,
      gradeId: null,
      blCampusId: null,
      classStatus: null,
      allocationFlag: null,
      studyManegerId: null,
      inviteUserId: '',
      payStatus: [],
      surrenderStatus: [],
      auditStatus: [],
      grouponStatus: null,
      activityName: '',
      activityTypeId: null,
      branchId: this.defaultBranch,
      time: null,
    });
  };

  pageChange = async ({ current, pageSize }) => {
    this.setState({ listLoading: true });
    try {
      const res = await this.loadListData({ pageNo: current, pageSize });
      this.setState({
        listLoading: false,
        rows: res.records,
        total: res.total,
        pageNo: current,
        pageSize,
        selectedRowKeys: [],
      });
    } catch (e) {
      showMessage(e.text, 'error');
    }
    this.setState({
      listLoading: false,
    });
  };

  renderStatus = (detail, record) => {
    const { auditStatus } = record;
    const color = auditStatusColorObj['' + auditStatus];
    return (
      <p key="asd" style={{ display: 'flex' }}>
        {!color ? '' : <b style={{ fontSize: '25px', color: color, marginRight: '3px' }}>·</b>}
        {auditStatusObj[auditStatus] || ''}
      </p>
    );
  };

  onExportClick = async (hasPhoneNumber, showRemark) => {
    this.setState({ loading: true });
    const {
      campusDict,
      time,
      name,
      intentionCampusId,
      gradeId,
      inviteUserId,
      payStatus,
      auditStatus,
      classStatus,
      allocationFlag,
      blCampusId,
      studyManegerId,
      grouponStatus,
      followupUserName,
      bossStudentNo,
      activityName,
      activityTypeId,
      branchId,
      activityId,
    } = this.state;
    try {
      const refundStatusOrPayStatus = surrenderStatusMap.decrease(payStatus);
      const currentAuditStatus = auditStatus.length ? auditStatus : null;

      // 开始和结束时间
      let beginTime;
      let endTime;
      if (time === null) {
        beginTime = null;
        endTime = null;
      } else {
        let timer = this.parseTime(time);
        beginTime = timer.beginTime;
        endTime = timer.endTime;
      }

      let apiParams = {
        name,
        branchId: [branchId],
        endTime,
        beginTime,
        activityTypeId,
        followupUserName,
        bossStudentNo,
        intentionCampusId,
        gradeId,
        inviteUserId: inviteUserId.split('//')[1],
        auditStatus: currentAuditStatus,
        classStatus,
        allocationFlag,
        blCampusId,
        studyManegerId,
        grouponStatus,
        activityId: activityId ? [activityId] : null,
        showPhoneFlag: hasPhoneNumber ? 1 : 0,
        showRemark: typeof showRemark === 'boolean' ? showRemark : null,
        ...refundStatusOrPayStatus,
      };
      const totalCampusDict = campusDict.reduce((array, parentObj) => {
        const childArray = Array.isArray(parentObj.options) ? parentObj.options.map((item) => item.val) : [];
        return [...array, ...childArray];
      }, []);

      apiParams['intentionCampusId'] = apiParams['intentionCampusId']
        ? [apiParams['intentionCampusId']]
        : totalCampusDict;
      if (apiParams.name) {
        apiParams['orderNo'] = apiParams.name.length > 11 ? apiParams.name : null;
        apiParams['customerPhone'] = apiParams.name.length === 11 ? apiParams.name : null;
        apiParams['studentName'] = !+apiParams.name ? apiParams.name : null;
      }

      apiParams.name = activityName ? activityName.trim() : null;

      apiParams = Object.keys(apiParams).reduce((obj, ele) => {
        const value = apiParams[ele];
        if (value !== null && value !== '' && value !== void 0) {
          obj[ele] = apiParams[ele];
        }
        return obj;
      }, {});

      await loader.exportExcel(apiParams);
      showMessage('导出成功！', 'success');
    } catch (e) {
      showMessage('导出失败：' + e.resultMessage, 'error');
    }
    this.setState({ loading: false });
  };

  updateRow = (raw, changed) => {
    this.setState({ listLoading: true });
    const idx = this.state.rows.findIndex((ele) => ele.orderNo === raw.orderNo);
    const rows = this.state.rows.map((row, i) => {
      if (i === idx) {
        return { ...row, ...changed };
      }
      return row;
    });
    this.setState({
      listLoading: false,
      rows,
    });
    return { ...raw, ...changed };
  };

  onRowSelected = (keys) => {
    const { rows } = this.state;
    const { btnAuth } = this.props;

    const sortedKeys = keys.filter((key) => {
      const row = rows[key];

      return row.auditStatus === 0 && btnAuth.indexOf('MARKET_ACTIVITY_DISTRIBUTE_ORDER') > -1;
    });

    const batchDistrbuteDisabled = keys.length ? sortedKeys.length !== keys.length : true;
    const selectedOrders = !batchDistrbuteDisabled ? keys.map((ele) => rows[ele]) : [];
    this.setState({
      selectedRowKeys: keys,
      selectedOrders,
      batchDistrbuteDisabled,
    });
  };

  onDistrbuteConfirm = (change) => {
    this.state.selectedOrders.map((ele) => {
      return this.updateRow(ele, change);
    });

    this.setState({
      selectedOrders: [],
      selectedRowKeys: [],
    });
  };
  //退款modal点击确定
  onRefundConfirm = async ({ refundReason, refundXQYYZR, selectProductArray }) => {
    const { selectedOrder, pageNo, pageSize } = this.state;
    this.setState({ listLoading: true });
    try {
      await applyRefund({
        orderNo: selectedOrder.orderNo,
        refundReason,
        auditUserId: refundXQYYZR.val,
        refundProductId: selectProductArray,
      });
      await this.pageChange({
        current: pageNo,
        pageSize,
      });
      showMessage('操作成功', 'success');
    } catch (e) {
      showMessage('操作失败：' + e.text, 'error');
    }
    this.setState({
      listLoading: false,
      refundModalVisible: false,
      orderDetail: {},
    });
  };
  // 点击退款获取营运主任打开modal
  handleRefundButton = async (record) => {
    const { intentionCampusId } = record;
    const orgId = this.state.branchId;
    const canSelectOrgIdArray = [intentionCampusId, orgId];
    try {
      const orderDetail = await this.getOrderDetail(record);
      const getRefundPersonFn = loader[REFUND_PERSON_LOADER_NAME];
      const data = await getRefundPersonFn({ orgId });

      // 退款审核人的数据只筛选 分公司架构就是orgId ===orgId 或者orgId===intentionCampusId 意向校区的
      const handleRefundArray = data.filter((person) => canSelectOrgIdArray.indexOf(person.orgId) > -1);

      await this.getActivityDicts(record);

      this.setState({
        orderDetail,
        refundModalVisible: true,
        selectedOrder: record,
        handleRefundXQYYZRArray: handleRefundArray,
      });
    } catch (e) {
      showMessage('获取数据失败：' + e.text, 'error');
    }
  };

  // 转校确定逻辑
  onTransferConfirm = async ({ intentionCampusId, studentNo }) => {
    const { selectedOrder, pageNo, pageSize } = this.state;
    this.setState({ listLoading: true });
    try {
      await loader.transSchool({
        orderNo: selectedOrder.orderNo,
        intentionCampusId,
        activityId: selectedOrder.activityId,
        studentNo,
      });
      await this.pageChange({
        current: pageNo,
        pageSize,
      });
      showMessage('操作成功', 'success');
    } catch (e) {
      showMessage('操作失败：' + e.text, 'error');
    }
    this.setState({
      listLoading: false,
      transferSchoolModal: false,
    });
  };
  onAuditConfirm = async (data) => {
    this.setState({ listLoading: true });
    const { selectedOrder, pageSize, pageNo } = this.state;
    const { productTypeId } = selectedOrder;
    // 是否双师或者小班 这两个要报班

    const isSmallOTwoAudis = ['SMALL_CLASS', 'TWO_TEACHER'].indexOf(productTypeId) > -1;
    const isPaid = +selectedOrder.payStatus === 1 && +selectedOrder.surrenderStatus !== 3;
    try {
      //审核情况分两种 审核成功与审核失败
      if (!data.auditStatus) {
        // 审核失败
        await Promise.all(
          [selectedOrder].map(async (ele) => {
            await loader.rejectOrder({
              cancelReason: data.failReason,
              orderNo: ele.orderNo,
            });
          }),
        );
      } else {
        // 审核成功种 第一种 小班或者双师并且已支付 并且 不是退款成功状态的话 那么需要填写报班以及课时数据 其他的情况不用填写小班以及课时数据
        if (isSmallOTwoAudis && isPaid) {
          //小班或者双师审核成功 已支付状态
          await this.onCourseConfirm(data);
        } else {
          // 其余的不用填写状态
          const updateAuditParam = data.updateAuditArray.map((product) => ({
            productId: product.val,
            classId: '',
            firstSchoolTime: '',
            studentIds: [],
            contractIds: [],
            contractProductIds: [],
            courseIds: [],
            orderNo: selectedOrder.orderNo,
            isGroupPromotion: data.isGroupPromotion,
            contractCampusId: selectedOrder.contractCampusId,
            purchaseAgreement: selectedOrder.purchaseAgreement,
          }));
          await loader.auditOrder(updateAuditParam);
        }
      }
      await this.pageChange({
        current: pageNo,
        pageSize,
      });
      showMessage('操作成功', 'success');
      this.setState({
        auditModalVisible: false,
        selectedOrder: null,
      });
    } catch (e) {
      this.setState({
        auditModalVisible: false,
      });
      showMessage('提交审核失败：' + e.text.replace('999-系统错误，请联系管理员:', ''), 'error');
    }
    this.setState({
      listLoading: false,
    });
  };

  onReopenClick = async (data) => {
    const { pageNo, pageSize } = this.state;
    this.setState({ listLoading: true });
    try {
      await this.getActivityDicts(data);
      await loader.reopenOrder({
        orderNo: data.orderNo,
      });
      await this.pageChange({
        current: pageNo,
        pageSize,
      });
      this.setState({
        selectedOrder: data,
        auditModalVisible: true,
      });
    } catch (e) {
      showMessage('操作失败：' + e.text, 'error');
    }
    this.setState({
      listLoading: false,
    });
  };

  // 小班或者双师选中课时以及班级后的审核
  onCourseConfirm = async (auditData) => {
    //审核通过
    const { selectedOrder, pageNo, pageSize } = this.state;
    this.setState({
      listLoading: true,
    });
    const { updateAuditArray, isGroupPromotion } = auditData;
    // 分两种情况双师和小班
    const updateAuditParam = updateAuditArray.map((productItem) => {
      const { val, hasSelectedClass, hasSelectedCourse } = productItem;
      return {
        productId: val,
        // hasSelectedClass不存在就没有排课
        classId: hasSelectedClass ? hasSelectedClass.classId : null,
        firstSchoolTime: hasSelectedCourse[0] ? hasSelectedCourse[0].courseDate : '',
        studentIds: [],
        contractIds: [],
        contractProductIds: [],
        courseIds: hasSelectedCourse.map((ele) => ele.classCourseId),
        orderNo: selectedOrder.orderNo,
        isGroupPromotion,
        contractCampusId: selectedOrder.contractCampusId,
        purchaseAgreement: selectedOrder.purchaseAgreement,
      };
    });
    await loader.auditOrder(updateAuditParam);
    this.setState({
      selectedOrder: null,
      auditModalVisible: false,
    });
    await this.pageChange({
      current: pageNo,
      pageSize,
    });
    this.setState({
      listLoading: false,
      auditModalVisible: false,
    });
  };

  listRowClick = (record, index) => {
    const selectedRowKeys = this.state.selectedRowKeys.slice();
    const currentSelectedRowKeys = selectedRowKeys.includes(index)
      ? selectedRowKeys.filter((rowIndex) => rowIndex !== index)
      : selectedRowKeys.concat(index).sort((a, b) => a - b);
    this.onRowSelected(currentSelectedRowKeys);
  };
  // 切换显示修改学生信息弹窗
  switchEditStdInfo() {
    this.setState(
      (prevState) => ({
        editStdInfoVisible: !prevState.editStdInfoVisible,
      }),
      () => {
        if (!this.state.editStdInfoVisible) this.editStdData = null;
      },
    );
  }
  // 提交修改学生信息回调
  async afterSubmitEditStdInfo(studentInfo) {
    const { isClickAuditButton, selectedOrder } = this.state;
    this.switchEditStdInfo();
    if (isClickAuditButton) {
      await this.getActivityDicts(selectedOrder);
      this.setState({
        isClickAuditButton: false,
        auditModalVisible: true,
        selectedOrder: {
          ...selectedOrder,
          contractCampusName: studentInfo.contractCampusName,
          contractCampusId: studentInfo.contractCampusId,
        },
      });
      return;
    }
    this.setState({ listLoading: true });
    const res = await this.loadListData();
    this.setState({
      rows: res.records,
      total: res.total,
      selectedRowKeys: [],
      listLoading: false,
    });
  }
  // 切换更多信息弹窗显隐
  switchModalStdQuestionnaire(studentNo, studentName) {
    if (studentNo && studentName) {
      this.studentNo = studentNo;
      this.studentName = studentName;
    }
    this.setState(
      (prevState) => ({
        visibleStdQuestionnaire: !prevState.visibleStdQuestionnaire,
      }),
      () => {
        if (!this.state.visibleStdQuestionnaire) {
          this.studentNo = null;
          this.studentName = null;
        }
      },
    );
  }

  // 批量分配
  handleGroupDistribute = () => {
    const { selectedOrders } = this.state;
    // 批量分配只允许同一校区的批量分配
    const selectedCampus = Array.from(new Set(selectedOrders.map((order) => order.intentionCampusId)));
    if (selectedCampus.length > 1) {
      // 存在多个校区
      showMessage('操作失败：请选择同一校区的订单进行分配', 'error');
      return false;
    }
    this.setState({
      distrbuteModalVisible: true,
    });
  };

  // 关闭报告Modal
  closeReporterModal = () => {
    this.setState({ isReporter: false });
  };

  // 获取订单详情数据
  getOrderDetail = async (records) => {
    try {
      const res = await loader.getOrderDetail({
        orderNo: records.orderNo,
      });
      return res.data;
    } catch (e) {
      throw { text: e.text };
    }
  };

  render() {
    const {
      loading,
      searchExpand,
      listLoading,
      time,
      rows,
      pageNo,
      pageSize,
      total,
      selectedRowKeys,
      handleRefundXQYYZRArray,
      name,
      followupUserName,
      bossStudentNo,
      intentionCampusId,
      gradeId,
      inviteUserId,
      payStatus,
      auditStatus,
      campusDict,
      gradeDict,
      inviteUserDict,
      blCampusId,
      classStatus,
      auditModalVisible,
      studyManegerId,
      productDict,
      managerDict,
      selectedOrders,
      orderDetail,
      transferSchoolCampusDict,
      failReasonModalVisible,
      refundModalVisible,
      selectedOrder,
      distrbuteModalVisible,
      batchDistrbuteDisabled,
      editStdInfoVisible,
      transferSchoolModal,
      orderStatusModal,
      orderModalType,
      activityProductMap,
      fightGroupModal,
      visibleStdQuestionnaire,
      grouponStatus,
      groupStatusDict,
      allocationFlag,
      isReporter,
      isClickAuditButton,
      isShowOrderRemarkModal,
      activityName,
      branchId,
      activityTypeId,
      branchDict,
      activityTypeDict,
    } = this.state;
    const { route, btnAuth, user } = this.props;

    const isShowExportOrderBtn = btnAuth.indexOf('EXPORT_ACTIVITY_ORDER') > -1;
    const isShowBtnWithoutPhone = btnAuth.indexOf('EXPORT_ACTIVITY_NO_PHONE') > -1;
    const isShowBtnOrderRemark = btnAuth.indexOf('EXPORT_ACTIVITY_REMARK') > -1;
    const showButtonWrap = isShowExportOrderBtn || isShowBtnWithoutPhone || isShowBtnOrderRemark;
    const exportBtnArray = [];

    if (isShowExportOrderBtn) {
      exportBtnArray.push({
        text: '含手机号码导出',
        onClick: () => this.onExportClick(true),
      });
    }
    if (isShowBtnWithoutPhone) {
      exportBtnArray.push({
        text: '不含手机号码导出',
        onClick: () => this.onExportClick(false),
      });
    }

    if (isShowBtnOrderRemark) {
      // 是否展示跟进记录数据
      exportBtnArray.push({
        text: '跟进记录导出',
        onClick: () => this.onExportClick(false, true),
      });
    }

    return (
      <Page
        loading={loading}
        route={route}
        fitWindowHeight={true}
        subHeader={
          <div className={commonStyles.searchBar} style={{ paddingRight: '29px' }}>
            <div
              className={commonStyles.addShare}
              style={{
                display: 'flex',
                justifyContent: 'flex-start',
                width: 235,
              }}
            >
              <Button type="primary" onClick={this.handleGroupDistribute} disabled={batchDistrbuteDisabled}>
                批量分配
              </Button>
              {showButtonWrap && (
                <ButtonWrap
                  btnStyle={{ marginLeft: 15 }}
                  btnText="导出Excel"
                  popPlacement="bottom"
                  actions={exportBtnArray}
                />
              )}
            </div>
            <div>
              <SearchBar
                onSearch={this.search}
                onReset={this.reset}
                childWidth={{ index: [15], width: [2] }}
                expand={searchExpand}
                isShowMoreBtn
                onToggleExpand={(expand) => this.setState({ searchExpand: expand })}
              >
                <Input
                  placeholder="订单编号/学生名称/客户号码"
                  value={name}
                  onChange={(e) => this.setState({ name: e.target.value })}
                />
                <Input
                  placeholder="跟进人"
                  value={followupUserName}
                  onChange={(e) => this.setState({ followupUserName: e.target.value })}
                />
                <Input
                  placeholder="准考证号"
                  value={bossStudentNo}
                  onChange={(e) => this.setState({ bossStudentNo: e.target.value })}
                />
                <Select
                  placeholder="意向校区"
                  dict={campusDict}
                  value={intentionCampusId}
                  allowClear={true}
                  showSearch={true}
                  onChange={(intentionCampusId) => this.setState({ intentionCampusId })}
                />
                <Select
                  placeholder="学籍校区"
                  dict={studentStatusCampusDict}
                  value={blCampusId}
                  allowClear={true}
                  showSearch={true}
                  onChange={(blCampusId) => this.setState({ blCampusId })}
                />
                <Select
                  placeholder="是否在读"
                  dict={attendSchoolDict}
                  value={classStatus}
                  allowClear={true}
                  showSearch={true}
                  onChange={(classStatus) => this.setState({ classStatus })}
                />
                <Select
                  placeholder="学管师"
                  showSearch={true}
                  value={studyManegerId}
                  dict={managerDict}
                  allowClear={true}
                  onChange={(studyManegerId) => this.setState({ studyManegerId })}
                />
                <Select
                  placeholder="选择年级"
                  showSearch={true}
                  dict={gradeDict}
                  value={gradeId}
                  onChange={(gradeId) => this.setState({ gradeId })}
                />
                <Select
                  placeholder="邀请人"
                  searchNotFoundTips="该老师未分享引流活动"
                  showSearch={true}
                  value={inviteUserId}
                  dict={inviteUserDict}
                  allowClear={true}
                  onChange={(inviteUserId) =>
                    this.setState({
                      inviteUserId: inviteUserId ? inviteUserId : '',
                    })
                  }
                  onSearchKeyChange={this.getFollowerDict}
                />
                <MultiSelect
                  placeholder="订单状态"
                  dict={payStatusDict}
                  value={payStatus}
                  allowClear={true}
                  showSearch={true}
                  onChange={(payStatus) => this.setState({ payStatus: payStatus || [] })}
                />
                <MultiSelect
                  placeholder="审核状态"
                  dict={auditStatusDict}
                  value={auditStatus}
                  allowClear={true}
                  showSearch={true}
                  onChange={(auditStatus) => this.setState({ auditStatus: auditStatus || [] })}
                />
                <Select
                  placeholder="是否分配"
                  value={allocationFlag}
                  dict={distributeDict}
                  allowClear={true}
                  onChange={(allocationFlag) => this.setState({ allocationFlag })}
                />
                <Input
                  placeholder="活动名称"
                  value={activityName}
                  onChange={(e) => this.setState({ activityName: e.target.value })}
                />
                <Select
                  placeholder="分公司"
                  value={branchId}
                  dict={branchDict}
                  allowClear={false}
                  showSearch={true}
                  onChange={this.onBranchChange}
                />
                <Select
                  placeholder="活动类型"
                  value={activityTypeId}
                  dict={activityTypeDict}
                  allowClear={true}
                  onChange={(activityTypeId) => this.setState({ activityTypeId })}
                />
                <RangePicker
                  style={{ width: '100%' }}
                  size="large"
                  format={dateFormat}
                  showTime={{
                    hideDisabledOptions: true,
                    defaultValue: [moment('00:00:00', 'HH:mm:ss'), moment('23:59:59', 'HH:mm:ss')],
                  }}
                  placeholder={['开始时间', '结束时间']}
                  value={time}
                  onChange={(time) => {
                    this.setState({ time: time.length === 0 ? null : time });
                  }}
                />
                <Select
                  placeholder="拼团状态"
                  showSearch={true}
                  value={grouponStatus}
                  dict={groupStatusDict}
                  allowClear={true}
                  onChange={(grouponStatus) => this.setState({ grouponStatus })}
                />
              </SearchBar>
            </div>
          </div>
        }
      >
        <List
          columns={this.columns}
          loading={listLoading}
          dataSource={rows}
          pagination={{
            current: pageNo,
            pageSize,
            total: total,
          }}
          onChange={this.pageChange}
          rowSelection={{
            selectedRowKeys,
            onChange: this.onRowSelected,
          }}
          onRowClick={this.listRowClick}
          className={commonStyles.activityOrderList}
        />
        {auditModalVisible && (
          <AuditModal
            branchId={branchId}
            order={selectedOrder}
            productDict={productDict}
            productTypeId={selectedOrder.productTypeId}
            contractCampusId={selectedOrder.contractCampusId}
            contractCampusName={selectedOrder.contractCampusName}
            onClose={() => {
              this.pageChange({
                current: pageNo,
                pageSize,
              })
                .then(() => {
                  this.setState({
                    selectedOrder: null,
                    auditModalVisible: false,
                  });
                })
                .catch(() => {
                  this.setState({
                    selectedOrder: null,
                    auditModalVisible: false,
                  });
                });
            }}
            onConfirm={this.onAuditConfirm}
          />
        )}
        {failReasonModalVisible && (
          <FailReasonModal order={selectedOrder} onClose={() => this.setState({ failReasonModalVisible: false })} />
        )}
        {distrbuteModalVisible && (
          <DistrbuteModal
            managerDict={managerDict}
            orders={selectedOrders}
            onClose={() => this.setState({ distrbuteModalVisible: false })}
            onConfirm={this.onDistrbuteConfirm}
          />
        )}
        {refundModalVisible && (
          <RefundModal
            activityProductMap={activityProductMap}
            order={selectedOrder}
            orderDetail={orderDetail}
            refundXQYYZRArray={handleRefundXQYYZRArray}
            onClose={() => this.setState({ refundModalVisible: false })}
            onConfirm={this.onRefundConfirm}
          />
        )}
        {editStdInfoVisible && (
          <ModalEditStdInfo
            name={isClickAuditButton ? '请核对学生基础信息（务必认真核对）' : ''}
            okText={isClickAuditButton ? '我已完善并核对学生信息，保存并进入下一步' : ''}
            isAudit={isClickAuditButton}
            data={this.editStdData}
            onCancel={() => {
              this.setState({
                isClickAuditButton: false,
              });
              this.switchEditStdInfo();
            }}
            callback={this.afterSubmitEditStdInfo}
          />
        )}
        {transferSchoolModal && (
          <TransferSchoolModal
            order={selectedOrder}
            campusDict={transferSchoolCampusDict}
            onClose={() => this.setState({ transferSchoolModal: false })}
            onConfirm={this.onTransferConfirm}
          />
        )}
        {orderStatusModal && (
          <OrderStatusModal
            onClose={() => this.setState({ orderStatusModal: false })}
            orderModalType={orderModalType}
          />
        )}
        {fightGroupModal && (
          <FightGroupModal onClose={() => this.setState({ fightGroupModal: false })} order={selectedOrder} />
        )}
        {visibleStdQuestionnaire && (
          <ModalStdQuestionnaire
            activityId={selectedOrder.activityId}
            studentNo={this.studentNo}
            studentName={this.studentName}
            onCancel={this.switchModalStdQuestionnaire}
          />
        )}
        {isReporter && <TeachingReporter />}
        {isShowOrderRemarkModal && (
          <OrderRemarkModal
            externalOrderId={selectedOrder.orderNo}
            user={user}
            onCancel={() => {
              this.setState({ isShowOrderRemarkModal: false });
            }}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  let data = state.main.pageParams;
  let pathName = state.routing.locationBeforeTransitions.pathname;
  return {
    params: data[pathName] || { activity: {} },
    user: state.account.user,
    btnAuth: state.account.btnAuths || [],
    questionnaireBtnAuth: state.account.btnAuths.indexOf('ORDER_QUESTIONNAIRE_DETAIL') === -1,
  };
};
export default connect(mapStateToProps, null)(MarketActivityOrder);
