import { getReceptionistGroupByUserName, getDistinctUserGroupByCampus } from 'SERVICE/common';
import { loadConsultantDict } from 'PAGE/common/loaders';
import { findStudentById } from 'SERVICE/student';
import {
  findActivityDetail,
  getProductByIds,
  listPageOrder,
  dispatchOrder,
  cancelOrder,
  openOrder,
  exportActivityOrder,
  getRechargeMoney,
  checkAllowAddStudent4MiniClass,
  checkAllowAddStudent4TwoTeacherClass,
  findCourseByClassTwoId,
  transferSchool,
  getOrderDetailByOrderNo,
  getInviteData,
  listOrderRemark,
  addOrderRemark,
  listOrderRemarkDetail,
  listStudentByCustomer,
  createStudent,
} from 'SERVICE/marketActivityManage';
import { getMiniClassListForEnrollInClass, getClassTwoListForEnrollInClass } from 'SERVICE/course';
import fetch from 'dva/fetch';
import { urlBase } from 'CONST/config';

// 旧boss接口
const loadFollowerDict = async (params, orgParams) => {
  const res = await getReceptionistGroupByUserName({ ...params });
  const dictObj = res.reduce((obj, ele) => {
    if (!obj[ele.orgId]) {
      obj[ele.orgId] = {
        label: ele.orgName,
        options: [],
        val: ele.orgId,
      };
    }
    obj[ele.orgId].options.push({
      val: ele.orgId + '//' + ele.userId,
      text: ele.userName,
      group: ele.orgName,
    });

    return obj;
  }, {});
  if (orgParams) {
    // 数组
    if (Array.isArray(orgParams)) {
      return orgParams.map((orgId) => dictObj[orgId]);
    }
    // 字符串
    return [dictObj[orgParams]];
  }
  return Object.keys(dictObj).reduce((arr, ele) => {
    arr.push(dictObj[ele]);
    return arr;
  }, []);
};

// 双师课程顾问字典
const loadCourseTeacherDict = async (params) => {
  const api = await getDistinctUserGroupByCampus({
    ...params,
    showBelong: 1,
    job: ['ZXS'],
    // isFilterAcademic: true, // 是否过滤教务角色，true-是，false-否，默认不过滤
  });
  return api.map((d) => {
    return { val: d.userId, text: d.userName };
  });
};
// 双师营运组负责人字典
const load2TeacherYYZFZRDict = async (params) => {
  const api = await getDistinctUserGroupByCampus({
    ...params,
    showBelong: 2,
    job: ['YYZFZR'],
  });
  return api.map((d) => {
    return {
      val: d.userId,
      text: d.userRole,
      name: d.userName,
      orgId: d.orgId,
    };
  });
};

const loadXQYYZRDict = async (params) => {
  try {
    const XQYYFZRData = await getDistinctUserGroupByCampus({
      ...params,
      showBelong: 2,
      job: ['XQYYFZR'],
    });
    if (XQYYFZRData.length) {
      return XQYYFZRData.map((d) => {
        return {
          val: d.userId,
          text: d.userRole,
          name: d.userName,
          orgId: d.orgId,
        };
      });
    }
    const XQZRData = await getDistinctUserGroupByCampus({
      ...params,
      showBelong: 2,
      job: ['XQZR'],
    });
    if (XQZRData.length) {
      return XQZRData.map((d) => {
        return {
          val: d.userId,
          text: d.userRole,
          name: d.userName,
          orgId: d.orgId,
        };
      });
    }
    const XQJXZRData = await getDistinctUserGroupByCampus({
      ...params,
      showBelong: 2,
      job: ['XQJXZR'],
    });
    if (XQJXZRData.length) {
      return XQJXZRData.map((d) => {
        return {
          val: d.userId,
          text: d.userRole,
          name: d.userName,
          orgId: d.orgId,
        };
      });
    }
  } catch (e) {
    return Promise.reject(e);
  }
};

const loadMiniClassList = async (params) => {
  const res = await getMiniClassListForEnrollInClass(params);
  if (res.resultCode !== 0) {
    throw { text: res.resultMessage };
  }
  return res;
};
const loadTwoTeacherClassList = async (params) => {
  const res = await getClassTwoListForEnrollInClass(params);
  if (res.resultCode !== 0) {
    throw { text: res.resultMessage };
  }
  return res;
};
const getConsultantDict = async (params) => {
  return await loadConsultantDict(params);
};

//获取分公司培优引流项目退费分班字典（就是咨询师）
const loadPeiYouDistrbuteDict = async (params) => {
  const api = await getDistinctUserGroupByCampus({
    ...params,
    showBelong: 1,
    job: ['YLXMCY'],
  });
  return api.map((d) => {
    return {
      val: d.userId,
      text: d.userRole,
      name: d.userName,
      orgId: d.orgId,
    };
  });
};

//获取分公司培优引流项目负责人字典（退费时选择的人）
const loadPeiYouRefundPersonDict = async (params) => {
  const api = await getDistinctUserGroupByCampus({
    ...params,
    showBelong: 2,
    job: ['YLXMFZR'],
  });
  return api.map((d) => {
    return {
      val: d.userId,
      text: d.userRole,
      name: d.userName,
      orgId: d.orgId,
    };
  });
};

// 新报读接口---------------------
const loadOrderList = async (params = {}) => {
  const { pageNo, pageSize, ...extraParams } = params;
  const res = await listPageOrder({
    current: pageNo,
    size: pageSize,
    params: extraParams,
  });
  if (res.code !== 2000000) {
    throw { text: res.msg };
  }
  return res.data;
};

// 未迁移的新报读接口
const loadDetail = async (params) => {
  const res = await findActivityDetail(params);
  if (res.resultCode !== 0) {
    throw new Error(res.resultMessage);
  }
  return res;
};

const loadActivityProductList = (params) => {
  return getProductByIds(params);
};

const distrbute = async (params) => {
  const data = await dispatchOrder(params);
  if (data.resultCode !== 0) {
    throw { text: data.resultMessage };
  }
  return data;
};

const rejectOrder = async (params) => {
  const res = await cancelOrder(params);
  if (res.resultCode !== 0) {
    throw { text: res.resultMessage };
  }
  return res;
};

const auditOrder = async (params) => {
  const orderNo = params[0].orderNo;
  const useGrouponDiscount = params[0].isGroupPromotion;
  const purchaseAgreement = params[0].purchaseAgreement;
  const contractCampusId = params[0].contractCampusId;
  let url = `${urlBase}MarketActivityController/storageOrder.do?orderNo=${orderNo}&useGrouponDiscount=${useGrouponDiscount}&contractCampusId=${contractCampusId}`;
  if (purchaseAgreement !== undefined) {
    url = `${url}&signedAgreement=${purchaseAgreement}`;
  }
  const res = await fetch(url, {
    credentials: 'same-origin',
    method: 'POST',
    body: JSON.stringify(params),
    headers: {
      'Content-Type': 'application/json;charset=utf-8',
    },
  });
  const data = await res.json();
  if (data.resultCode !== 0) {
    throw { text: data.resultMessage };
  }
  return data;
};

const reopenOrder = async (params) => {
  const res = await openOrder(params);
  if (res.resultCode !== 0) {
    throw { text: res.resultMessage };
  }
  return res;
};

const exportExcel = async (params) => {
  const res = await exportActivityOrder(params);
  return res;
};

const loadRechargeMoney = async (params) => {
  const res = await getRechargeMoney(params);
  if (res.resultCode !== 0) {
    throw { text: res.resultMessage };
  }
  return res;
};

const transSchool = async (params) => {
  const res = await transferSchool(params);
  if (res.resultCode !== 0) {
    throw { text: res.resultMessage };
  }
  return res;
};

const getOrderDetail = async (params) => {
  const data = await getOrderDetailByOrderNo(params);
  if (data.resultCode !== 0) {
    throw { text: data.resultMessage };
  }
  return data;
};

const getInviteDetail = async (params) => {
  const data = await getInviteData(params);
  if (data.resultCode !== 0) {
    throw new Error(data.resultMessage);
  }
  return data;
};

//获取引流订单的跟进记录
const getOrderRemarkList = async (params) => {
  const res = await listOrderRemark(params);
  if (res.code !== 2000000) {
    throw { text: res.msg };
  }
  return res.data;
};

//获取引流订单的跟进记录详情
const getOrderRemarkDetail = async (remarkId) => {
  if (!remarkId) return;
  const res = await listOrderRemarkDetail({ remarkId });
  if (res.code !== 2000000) {
    throw { text: res.msg };
  }
  return res.data;
};

//添加引流订单的跟进记录
const createOrderRemark = async (params) => {
  const res = await addOrderRemark(params);
  if (res.code !== 2000000) {
    throw { text: res.msg };
  }
  return res.data;
};

// 判断新老生 根据已有的旧逻辑
const getStudentById = (stuId) => {
  return findStudentById({ id: stuId });
};

const getStudentList = (customerPhone) => {
  return listStudentByCustomer({ customerPhone });
};

const addStudent = (params) => {
  return createStudent(params);
};

export default {
  loadDetail,
  loadActivityProductList,
  loadFollowerDict,
  loadOrderList,
  distrbute,
  loadMiniClassList,
  loadTwoTeacherClassList,
  rejectOrder,
  reopenOrder,
  auditOrder,
  exportExcel,
  loadRechargeMoney,
  loadXQYYZRDict,
  loadCourseTeacherDict,
  load2TeacherYYZFZRDict,
  checkAllowAddStudent4MiniClass,
  checkAllowAddStudent4TwoTeacherClass,
  findCourseByClassTwoId,
  transSchool,
  getOrderDetail,
  getInviteDetail,
  loadPeiYouDistrbuteDict,
  loadPeiYouRefundPersonDict,
  getConsultantDict,
  getOrderRemarkList,
  getOrderRemarkDetail,
  createOrderRemark,
  getStudentById,
  getStudentList,
  addStudent,
};
