import React from 'react';
import Modal from 'COMPONENT/Modal';
import FormItem from 'COMPONENT/FormItem';
import TextArea from 'COMPONENT/TextArea';
import Select from 'COMPONENT/Select';
import Radio from 'COMPONENT/Radio';
import CheckBox from 'COMPONENT/Checkbox';
import { getMiniClassCourseDetail, findCourseByClassTwoId } from 'SERVICE/course';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import loader from '../loader';
import { classTypeMap } from '../../classTypeDict';
import ClassModal from './ClassModal';
import ChooseBeginDateModal from '../ChooseBeginDateModal';
import Styles from '../styles.less';
import classNames from 'classnames';

const groupStatusMap = {
  0: '未成团',
  1: '已成团',
  2: '单独购买',
};
const REASON_FOR_NO_PASS = [
  { val: '考试不通过', text: '考试不通过' },
  { val: '上课意愿低', text: '上课意愿低' },
  { val: '时间不合适', text: '时间不合适' },
  { val: '其他（另填）', text: '其他（另填）' },
];
class AuditModal extends React.Component {
  constructor(props) {
    super(props);
    const { productDict, order, productTypeId } = this.props;

    const { productId, originProductId, grouponStatus, insightMode } = order;
    const productMap = this.dictToObject(productDict);
    const selectedProduct = productDict.filter((ele) => ele.val === productId)[0];
    const isShowSelect = classTypeMap[productTypeId].isGradeNeeded;
    // 这里如果是全部退款的产品那么 productId九尾空数组 产品退款完了 所以退款完的产品应该拿原始的订单id
    const productIdArray = productId.length ? productId : originProductId;
    const selectedProductArray = productIdArray.map((itemId) => productMap[itemId]);
    // 下拉框是否锁死不让选择 组合模式锁死
    const isLock = insightMode === 1;
    // 拼团的默认勾选拼团优惠  单独购买的默认不拼团优惠
    const isGroupPromotion = grouponStatus === 1 || grouponStatus === 0;

    this.state = {
      loading: false,
      auditStatus: true,
      failReason: null,
      selectedProduct: null, // 当前小班报班的课程
      selectedClass: null, //当前订单小班报的课程
      courseTimeList: [], // 当前班课的选课时间
      selectedProductArray,
      // 兼容网校的产品 需要多传一个oldProductId  切换前的产品id
      //如果是普通产品组的产品切换成网校产品组的话，审核引流订单切换产品的接口 getRechargeMoney 新增一个入参 oldProductId  切换前的产品id
      defaultProductId: productIdArray,
      productDict,
      productMap,
      showNotice: !!selectedProduct,
      notice: '',
      isShowSelect,
      isLock,
      // 是否使用拼团优惠 默认都为true
      isGroupPromotion: isGroupPromotion,
      // 选班modal
      chooseBeginDateModalVisible: false,
      classModalVisible: false,
    };
  }

  beforeConfirmTip = (price) => {
    return new Promise((resolve, reject) => {
      Modal.confirm({
        width: '460px',
        content: (
          <div>
            <p className={Styles.tipModalTitle}>提示</p>
            <p className={Styles.tipModalContent}>该订单审核通过后，合同需要补收{Math.abs(price)}元！！！</p>
            <p className={Styles.tipModalContent}>
              请确保引流活动配置是否正常，如不清楚如何排查，请联系IT客服再审核报班~
            </p>
          </div>
        ),
        okText: '已沟通确认',
        onOk: resolve,
        onCancel: reject,
      });
    });
  };

  onSelectReasonForNoPass = (reasonForNoPass) => {
    const failReason = reasonForNoPass === '其他（另填）' ? '' : reasonForNoPass;
    this.setState({ failReason, reasonForNoPass });
  };

  onConfirm = async () => {
    this.setState({ loading: true });
    const { onConfirm, productDict, productTypeId, order } = this.props;
    const { notice, auditStatus, failReason, isShowSelect, selectedProductArray, isGroupPromotion } = this.state;
    let isComplete = true;
    let failMsg = '请补充完整审批数据';
    if (!auditStatus) {
      // 审核不通过 只需要原因即可
      if (!failReason) {
        isComplete = false;
        failMsg = '请补充退款理由';
      }
    } else {
      // 审核通过的话并且已支付 如果是小班就需要填写完整课时报班的等数据 已支付并且不能使退款成功的状态
      if (classTypeMap[productTypeId].isSignClassNeeded && +order.payStatus === 1 && +order.surrenderStatus !== 3) {
        isComplete = selectedProductArray.every((item) => item.hasSelectedCourse);
        failMsg = '请选择报班班级';
      }
    }

    if (!isComplete) {
      this.modal.showOkTips(failMsg);
      this.setState({ loading: false });
      return;
    }
    // 如果是小班 就需要报班课时的数据 其余的只要productId数据 但是一对一一对多是可选的 其他讲座拿字典的productVal
    let updateProductArray = isShowSelect ? selectedProductArray.slice() : productDict.slice();

    if (auditStatus && order.payStatus === 1 && !!+notice && +notice < 0) {
      try {
        await this.beforeConfirmTip(notice);
      } catch (e) {
        return;
      }
    }

    await onConfirm({
      auditStatus,
      failReason,
      updateAuditArray: updateProductArray,
      isGroupPromotion,
    });
    if (auditStatus) {
      this.setState({ loading: false });
    }
  };

  componentDidMount() {
    const { selectedProductArray } = this.state;
    this.calcRechargeMoney(selectedProductArray);
  }

  calcRechargeMoney = async (selectedProductArray) => {
    const { order } = this.props;
    const { isGroupPromotion, defaultProductId } = this.state;
    const productIds = selectedProductArray.map((product) => product.val).join(',');
    // 退款成功不用计算金额
    if (order.surrenderStatus === 3) {
      return false;
    }
    this.setState({
      loading: true,
    });
    try {
      const res = await loader.loadRechargeMoney({
        orderNo: order.orderNo,
        productId: productIds,
        useGrouponDiscount: isGroupPromotion,
        oldProductId: defaultProductId,
      });
      this.setState({
        notice: res.data,
        showNotice: true,
      });
    } catch (err) {
      console.log(err);
      this.setState({
        showNotice: false,
      });
    }
    this.setState({
      loading: false,
    });
  };

  /**
   * 加载课程列表
   * */
  loadCourseList(classId) {
    const { selectedProduct } = this.state;
    const params = {
      page: 1,
      rows: 100,
    };
    switch (selectedProduct.category) {
      case 'SMALL_CLASS':
        return getMiniClassCourseDetail({ ...params, miniClassId: classId });
      case 'TWO_TEACHER':
        return findCourseByClassTwoId({
          ...params,
          twoTeacherClassTwoId: classId,
        });
      default:
        throw new Error(`invalid product type`);
    }
  }

  // noClassSelectedClass没有排课课程的课程id 没有排课不需要选择上课时间
  onCourseConfirm = async ({ selectedCourse, noClassSelectedClass }) => {
    const { selectedProduct, selectedClass, selectedProductArray } = this.state;

    // 如果存在noClassSelectedClass 说明state的selectedClass为空 取传入的值
    const truelySelectedClass = noClassSelectedClass || selectedClass;

    // 兼容双师的courseId数据 这样子外层使用的都是classCourseId
    const hasSelectedCourse = selectedCourse.map((course) => {
      return {
        ...course,
        classCourseId: selectedProduct.category === 'TWO_TEACHER' ? course.courseId : course.miniClassCourseId,
      };
    });

    // 修改selectedProductArray数据
    const _selectedProductArray = selectedProductArray.map((product) => {
      // 这里不能直接修改product 因为修改的是引用会把productDict的引用也修改了
      let _product = { ...product };
      if (product.val === selectedProduct.id) {
        _product.hasSelectedClass = truelySelectedClass;
        _product.hasSelectedCourse = hasSelectedCourse;
      }
      return _product;
    });
    this.setState({
      selectedProductArray: _selectedProductArray,
      chooseBeginDateModalVisible: false,
      classModalVisible: false,
    });
  };
  onClassConfirm = async (data) => {
    try {
      const { selectedClass } = data;
      const res = await this.loadCourseList(selectedClass.classId);
      if (!res.rows.length) {
        // 没有排课 直接调用确认方法不过传入的是空数组 就是没有排课
        this.onCourseConfirm({
          selectedCourse: [],
          noClassSelectedClass: selectedClass,
        });
      } else {
        this.setState({
          courseTimeList: res.rows,
          selectedClass: data.selectedClass,
          chooseBeginDateModalVisible: true,
        });
      }
    } catch (error) {
      console.error(error);
    }
  };

  handleClassModal = (selectProduct) => {
    this.setState({
      classModalVisible: true,
      selectedProduct: selectProduct.product,
    });
  };

  // 一对一对多小班 不匹配年级进行下拉重新课程
  handleChangeMoneySelect = (val, index) => {
    const { selectedProductArray, productMap } = this.state;
    // 修改新的列表数据
    const newSelectedProductArray = selectedProductArray.map((product, arrayIndex) => {
      return index === arrayIndex ? productMap[val] : product;
    });
    this.setState({
      selectedProductArray: newSelectedProductArray,
    });
    // 重新获取金额数据
    this.calcRechargeMoney(newSelectedProductArray);
  };

  dictToObject = (dict) => {
    return dict.reduce((obj, cur) => {
      obj[cur.val] = cur;
      return obj;
    }, {});
  };

  clearProductDict = (currentProduct) => {
    const { selectedProductArray, productDict } = this.state;
    const newSelectArray = selectedProductArray.map((item) => item.val);
    // 下拉框需要排除掉已经选中的dict
    return productDict.filter((product) => {
      return product.val === currentProduct.val || !newSelectArray.includes(product.val);
    });
  };

  handleGroupPromotion = (e) => {
    const { selectedProductArray } = this.state;
    const checked = e.target.checked;
    this.setState(
      {
        isGroupPromotion: checked,
      },
      () => {
        // 重新计算优惠价格
        this.calcRechargeMoney(selectedProductArray);
      },
    );
  };
  render() {
    const { onClose, order, productTypeId, branchId } = this.props;
    const {
      loading,
      auditStatus,
      failReason,
      selectedProductArray,
      isLock,
      showNotice,
      notice,
      isShowSelect,
      classModalVisible,
      chooseBeginDateModalVisible,
      selectedProduct,
      selectedClass,
      courseTimeList,
      isGroupPromotion,
    } = this.state;

    const { grouponStatus } = order;

    // 显示是否选择拼团优惠
    const isGroupPromotionShow = grouponStatus === 1 || grouponStatus === 0;
    // 只有已成团的订单才会禁选
    const isGroupCheckboxDisabled = grouponStatus === 1;
    return (
      <div>
        <Modal
          title="审核"
          loading={loading}
          onCancel={onClose}
          okText="确定"
          onOk={this.onConfirm}
          ref={(c) => (this.modal = getWrappedInstance(c))}
          width={430}
        >
          <div className={Styles.auditModalContentWrapper}>
            <div style={{ color: '#333', fontSize: '14px', fontWeight: '500' }}>{`${order.studentName}(${
              order.phone || order.customerPhone
            })`}</div>
            {isGroupPromotionShow && auditStatus && (
              <div className={Styles.groupAuditWrapper}>
                <div>{`拼团状态：${groupStatusMap[grouponStatus]}`}</div>
                <CheckBox
                  checked={isGroupPromotion}
                  disabled={isGroupCheckboxDisabled}
                  onChange={this.handleGroupPromotion}
                >
                  使用拼团优惠
                </CheckBox>
              </div>
            )}
            <div className={classNames({ activityModalContainer: 1, [Styles.auditModalContainer]: 1 })}>
              <FormItem name="审核:" required="true">
                <Radio checked={auditStatus === true} onChange={() => this.setState({ auditStatus: true })}>
                  通过
                </Radio>
                <Radio
                  checked={auditStatus === false}
                  onChange={() => this.setState({ auditStatus: false })}
                  style={{ marginLeft: 20 }}
                >
                  不通过
                </Radio>
              </FormItem>
            </div>
            {auditStatus === false && (
              <div className={Styles.reasonForNoPassWrapper}>
                <FormItem name="不通过原因:" required="true">
                  <Select
                    placeholder="请选择原因"
                    dict={REASON_FOR_NO_PASS}
                    value={this.state.reasonForNoPass}
                    onChange={this.onSelectReasonForNoPass}
                  ></Select>
                </FormItem>
                {this.state.reasonForNoPass === '其他（另填）' && (
                  <div className="activityModalContainerSelect">
                    <FormItem required="true">
                      <TextArea
                        maxLength={70}
                        wordCount={true}
                        value={failReason}
                        onChange={(e) => this.setState({ failReason: e.target.value })}
                        placeholder="请输入原因"
                      />
                    </FormItem>
                  </div>
                )}
              </div>
            )}
            {isShowSelect &&
              auditStatus === true &&
              +order.payStatus === 1 &&
              (+order.surrenderStatus === 0 || +order.surrenderStatus === 4) && (
                <div className="activityModalContainerSelect">
                  <FormItem required="true">
                    {selectedProductArray.map((product, index) => (
                      <div className={Styles.auditSelectWrapper} key={index}>
                        <Select
                          placeholder="选择报读产品"
                          dict={this.clearProductDict(product)}
                          value={product.val}
                          showSearch
                          onChange={(val) => this.handleChangeMoneySelect(val, index)}
                          disabled={isLock}
                          allowClear={false}
                        />
                        {classTypeMap[productTypeId].isSignClassNeeded && (
                          <div
                            className={Styles.signClass + ' ' + (!!product.hasSelectedCourse && Styles.isSign)}
                            onClick={() => this.handleClassModal(product, index)}
                          >
                            {!!product.hasSelectedCourse ? '已报' : '报名'}
                          </div>
                        )}
                      </div>
                    ))}
                  </FormItem>
                </div>
              )}
            {/*锁死的话有匹配那么价格就付款正确*/}
            {auditStatus === true && +order.payStatus === 1 && showNotice && !!+notice && (
              <div className="activityModalContainerSelect">
                <FormItem>
                  {+notice >= 0 ? (
                    <div style={{ color: '#aaa' }}>提示：剩余资金{notice}元将充值到电子账户</div>
                  ) : (
                    <div style={{ color: '#aaa' }}>提示：审核通过后需要在合同列表补收{Math.abs(notice)}元</div>
                  )}
                </FormItem>
              </div>
            )}
          </div>
        </Modal>
        {classModalVisible && (
          <ClassModal
            order={order}
            branchId={branchId}
            product={selectedProduct}
            defaultBlCampus={this.props.contractCampusId}
            defaultBlCampusName={this.props.contractCampusName}
            onClose={() => this.setState({ classModalVisible: false })}
            onConfirm={this.onClassConfirm}
          />
        )}
        {chooseBeginDateModalVisible && (
          <ChooseBeginDateModal
            courseTimeList={courseTimeList}
            product={selectedProduct}
            selectedClass={selectedClass}
            onClose={() => this.setState({ chooseBeginDateModalVisible: false })}
            onConfirm={this.onCourseConfirm}
          />
        )}
      </div>
    );
  }
}

export default AuditModal;
