import React from 'react';
import { connect } from 'dva';
import Modal from 'COMPONENT/Modal';
import Input from 'COMPONENT/Input';
import Select from 'COMPONENT/Select';
import FormItem from 'COMPONENT/FormItem';
import loader from '../loader';

class CreateStudentModal extends React.Component {
  constructor(props) {
    super(props);
    const { dictSchool = [] } = props;
    this.state = {
      studentData: [],
      dictSchool: dictSchool.filter((item) => item.val !== 'other'),
      selectedRow: null,
      validator: {},
      validateResult: true,
      gradeId: null,
      schoolNo: null,
      loading: false,
    };
  }

  // 校验规则-不能超过n个字数
  static validateStrLen(len) {
    return function (val) {
      return val.length > len ? `不能超过${len}个字` : null;
    };
  }

  onConfirmModal = async () => {
    const { orderNo, phone } = this.props;
    const { studentName, gradeId, schoolNo } = this.state;

    const validateResult = Object.values(this.state.validator).every((item) => !Boolean(item));

    this.setState({
      validateResult,
    });
    if (validateResult) {
      this.setState({ loading: true });

      try {
        const bossStudentId = await loader.addStudent({
          customerPhone: phone,
          schoolNo,
          gradeId,
          studentName,
          orderNo,
        });
        this.setState({ loading: false });

        this.props.onOk({
          studentName,
          gradeId,
          schoolNo,
          bossStudentId,
        });
      } catch (err) {
        console.error(err);
        this.setState({ loading: false });
      }
    }
  };

  onSelectChange = (_selectedRowKeys, selectedRows) => {
    this.setState({ selectedRow: selectedRows[0] });
  };

  // 处理表单校验值更新
  handleValidator({ type, val }) {
    this.setState((prevState) => ({
      validator: {
        ...prevState.validator,
        [type]: val,
      },
    }));
  }

  render() {
    const { onCancel, dictGrade } = this.props;
    const { studentName, validator, validateResult, gradeId, schoolNo, dictSchool, loading } = this.state;

    return (
      <Modal
        title="创建新生"
        onCancel={onCancel}
        loading={loading}
        width={520}
        okText="保存"
        onOk={this.onConfirmModal}
      >
        <FormItem name="学生名称" labelWidth={100} required="true">
          <Input
            placeholder="请输入学生姓名(最多20字)"
            value={studentName}
            validator={{
              rules: ['required', CreateStudentModal.validateStrLen(20)],
              silent: !Boolean(validator.studentName) || validateResult,
              showStatus: true,
              showTips: true,
              onErrorChange: (err) => {
                this.handleValidator({ type: 'studentName', val: err });
              },
            }}
            onChange={(e) => this.setState({ studentName: e.target.value })}
          />
        </FormItem>
        <FormItem name="学生年级" labelWidth={100} required="true">
          <Select
            placeholder="请选择学生年级"
            dict={dictGrade}
            value={gradeId}
            showSearch
            allowClear={false}
            onChange={(val) => this.setState({ gradeId: val })}
            validator={{
              rules: ['required'],
              silent: !Boolean(validator.gradeId) || validateResult,
              onErrorChange: (err) => this.handleValidator({ type: 'gradeId', val: err }),
            }}
          />
        </FormItem>
        <FormItem name="在读学校" labelWidth={100} required="true">
          <Select
            placeholder="请选择在读学校"
            dict={dictSchool}
            value={schoolNo}
            showSearch
            allowClear={false}
            onChange={(val) => this.setState({ schoolNo: val })}
            validator={{
              rules: ['required'],
              silent: !Boolean(validator.schoolNo) || validateResult,
              onErrorChange: (err) => this.handleValidator({ type: 'schoolNo', val: err }),
            }}
          />
        </FormItem>
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    dictGrade: state.dict.grade,
  };
};

export default connect(mapStateToProps)(CreateStudentModal);
