import React from 'react';
import List from 'COMPONENT/List';
import Modal from 'COMPONENT/Modal';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import CreateStudentModal from './CreateStudentModal';
import styles from '../styles.less';
import loader from '../loader';

const { COL_WIDTH_S } = List;

class SelectStudentModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      studentData: [],
      selectedRow: null,
      selectedRowKeys: [props.bossStudentId],
      createStudentModalVis: false,
      loading: false,
    };
  }

  componentDidMount() {
    this.queryStudents();
  }

  async queryStudents() {
    this.setState({ loading: true });
    try {
      const { phone } = this.props;
      const resp = await loader.getStudentList(phone);
      this.setState({ studentData: resp });
    } catch (err) {
      console.error(err);
    }
    this.setState({ loading: false });
  }

  onConfirmModal = () => {
    const { selectedRow } = this.state;

    if (!selectedRow) {
      this.modal.showOkTips('请选择学生');
      return;
    }

    this.props.onOk(selectedRow);
  };

  onSelectChange = (_selectedRowKeys, selectedRows) => {
    this.setState({
      selectedRow: selectedRows[0],
      selectedRowKeys: [selectedRows[0].bossStudentId],
    });
  };

  onOk = (data) => {
    this.setState({ createStudentModalVis: false });
    this.props.onOk(data);
  };

  render() {
    const { onCancel, dictSchool, dictGrade, orderNo, phone } = this.props;
    const { createStudentModalVis, studentData, loading, selectedRowKeys } = this.state;

    const columns = [
      {
        title: '学生姓名',
        dataIndex: 'studentName',
        width: COL_WIDTH_S,
      },
      {
        title: '年级',
        dataIndex: 'gradeId',
        width: COL_WIDTH_S,
        render(_text, record) {
          const grade = dictGrade.find((item) => item.val === record.gradeId) || {};
          return grade.text || '-';
        },
      },
      {
        title: '在读学校',
        dataIndex: 'schoolNo',
        width: COL_WIDTH_S,
        render(_text, record) {
          const schoolItem = dictSchool.find((item) => item.val === record.schoolNo) || {};
          return schoolItem.text || '-';
        },
      },
    ];

    const rowSelection = {
      type: 'radio',
      hideDefaultSelections: true,
      onChange: this.onSelectChange,
      selectedRowKeys: selectedRowKeys,
    };

    return (
      <div>
        <Modal
          title="选择学生"
          onCancel={onCancel}
          loading={loading}
          width={800}
          okText="保存"
          onOk={this.onConfirmModal}
          ref={(c) => (this.modal = getWrappedInstance(c))}
        >
          <div>
            <List
              rowSelection={rowSelection}
              columns={columns}
              dataSource={studentData}
              pagination={true}
              rowKey="bossStudentId"
            />
            <div className={styles.createStudentContainer}>
              以上都不是，
              <span
                className={styles.createStudentText}
                onClick={() => {
                  this.setState({ createStudentModalVis: true });
                }}
              >
                创建新生
              </span>
            </div>
          </div>
        </Modal>
        {createStudentModalVis && (
          <CreateStudentModal
            onOk={this.onOk}
            onCancel={() => this.setState({ createStudentModalVis: false })}
            dictSchool={dictSchool}
            orderNo={orderNo}
            phone={phone}
          />
        )}
      </div>
    );
  }
}

export default SelectStudentModal;
