import React from 'react';
import { connect } from 'dva';
import style from './index.less';
import { storeName } from '../store/constants';
import Page from 'COMPONENT/Page';
import FormDetail from './FormDetail';
import { Row, Col } from 'antd';
import FormItem from 'COMPONENT/FormItem';

class Detail extends React.Component {
  constructor(props) {
    super(props);
    this.init = this.init.bind(this);
    this.beforeDestroy = this.beforeDestroy.bind(this);
    this.state = {
      loading: false,
    };
  }
  componentDidMount() {
    this.init();
  }
  componentWillUnmount() {
    this.beforeDestroy();
  }

  async init() {
    this.setState({ loading: true });
    const { formId } = this.props.pageParams[`/${this.props.route.path}`];
    await new Promise((resolve) => {
      this.props.dispatch({
        type: `${storeName}/getQuestionnaireForDetail`,
        payload: {
          id: formId,
          callback() {
            resolve();
          },
        },
      });
    });
    this.setState({ loading: false });
  }

  beforeDestroy() {
    this.props.dispatch({ type: `${storeName}/resetDetail` });
  }

  render() {
    return (
      <Page className={style.Detail} loading={this.state.loading} route={this.props.route}>
        <Row>
          <Col span={9}>
            <FormItem name="表单名称" labelWidth={100}>
              <span>{this.props.name}</span>
            </FormItem>
          </Col>
          <Col span={9}>
            <FormItem name="分公司" labelWidth={100}>
              <span>{this.props.branchName}</span>
            </FormItem>
          </Col>
        </Row>
        <FormDetail list={this.props.list} />
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    pageParams: state.main.pageParams,
    name: state[storeName].detailName,
    branchName: state[storeName].detailBranchName,
    list: state[storeName].detailList,
  };
};

export default connect(mapStateToProps)(Detail);
