/**
 * Created by Haolin on 2019/6/4.
 */
import React from 'react';
import PropTypes from 'prop-types';
import style from './index.less';
import Modal from 'COMPONENT/Modal';
import FormItem from 'COMPONENT/FormItem';
import FormDetail from '../Detail/FormDetail';
import { selectQuestionnaireById } from 'SERVICE/marketActivityManage';
import { errorsHandling } from '../store/utils';
import notDataImg from 'ASSET/imgs/notDataDef.png';

const classPrefix = 'ModalStdQuestionnaire';

const NoData = (props) => {
  return (
    <div className={style[`${classPrefix}-NoData`]}>
      <img className={style[`${classPrefix}-NoData-img`]} src={notDataImg} alt="暂无数据" />
      <p className={style[`${classPrefix}-NoData-text`]}>{props.text || '暂无数据'}</p>
    </div>
  );
};

class ModalStdQuestionnaire extends React.Component {
  constructor(props) {
    super(props);
    this.init = this.init.bind(this);
    this.state = {
      name: '更多信息',
      loading: false,
      topicList: [],
    };
  }
  static propTypes = {
    activityId: PropTypes.number.isRequired,
    studentNo: PropTypes.string.isRequired,
    studentName: PropTypes.string.isRequired,
    onCancel: PropTypes.func.isRequired,
  };
  componentDidMount() {
    this.init();
  }

  async init() {
    this.setState({ loading: true });
    let payload = { loading: false };
    const rsp = await selectQuestionnaireById(
      {
        activityId: this.props.activityId,
        studentNo: this.props.studentNo,
      },
      true,
    ).catch(errorsHandling());
    if (rsp && rsp.resultCode === 0 && rsp.data) {
      payload.topicList = rsp.data.topicList;
    }
    this.setState(payload);
  }

  render() {
    return (
      <Modal
        className={style[classPrefix]}
        title={this.state.name}
        widthSize="M"
        loading={this.state.loading}
        onCancel={this.props.onCancel}
        okBtnShow={false}
      >
        <FormItem name="姓名" labelWidth={60}>
          <span>{this.props.studentName}</span>
        </FormItem>
        <div style={{ minHeight: 300 }}>
          {this.state.topicList.length > 0 ? (
            <FormDetail bodyClassName={style[`${classPrefix}-list-wrap`]} list={this.state.topicList} />
          ) : (
            <NoData />
          )}
        </div>
      </Modal>
    );
  }
}

export default ModalStdQuestionnaire;
