/**
 * Created by Haolin on 2019/5/29.
 */
import React from 'react';
import { connect } from 'dva';
import style from './index.less';
import { storeName } from '../store/constants';
import Page from 'COMPONENT/Page';
import Wrapper from '../components/Wrapper';
import { Row, Col } from 'antd';
import FormItem from 'COMPONENT/FormItem';
import Input from 'COMPONENT/Input';
import Select from 'COMPONENT/Select';
import Button from 'COMPONENT/Button';
import Deck from 'COMPONENT/Deck';
import ContentBlock from '../components/ContentBlock';
import TopicCard from '../components/TopicCard';
import AddFormTopicBtn from '../components/AddFormTopicBtn';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { trim } from 'lodash';

const classPrefix = 'QuestionForm';
/**
 * 问题卡片
 *
 * 后期可在里面加问题类型的判断逻辑
 * */
const topicCardRender = ({ onUpdate, onDelete, onAddLine, onDelLine, onComplete }) => {
  return (props, index) => {
    return (
      <TopicCard
        {...props}
        onUpdate={onUpdate}
        onDelete={onDelete}
        onAddLine={onAddLine}
        onDelLine={onDelLine}
        onComplete={onComplete}
        index={index}
        key={index}
      />
    );
  };
};

class QuestionForm extends React.Component {
  constructor(props) {
    super(props);
    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.initDefaultData = this.initDefaultData.bind(this);
    this.initData = this.initData.bind(this);
    this.updateFormName = this.updateFormName.bind(this);
    this.setDefaultFormName = this.setDefaultFormName.bind(this);
    this.updateBranch = this.updateBranch.bind(this);
    this.onSelectRowKeyChange = this.onSelectRowKeyChange.bind(this);
    this.updateFormTopic = this.updateFormTopic.bind(this);
    this.onAddFormTopic = this.onAddFormTopic.bind(this);
    this.onDelFormTopic = this.onDelFormTopic.bind(this);
    this.onAddTopicOption = this.onAddTopicOption.bind(this);
    this.onDelTopicOption = this.onDelTopicOption.bind(this);
    this.onCompleteEdit = this.onCompleteEdit.bind(this);
    this.onSubmit = this.onSubmit.bind(this);
    this.submit = this.submit.bind(this);
    this.beforeDestroy = this.beforeDestroy.bind(this);
    this.errorChange = this.errorChange.bind(this);
    this.state = {
      loading: false,
      selectedRowKeys: [],
      error: null,
    };
  }
  componentDidMount() {
    this.init();
  }
  componentWillUnmount() {
    this.beforeDestroy();
  }

  static checkTextLength(val) {
    return val.length <= 30 ? null : '不能超过30个字';
  }

  async init() {
    this.setState({ loading: true });
    await new Promise(this.initDict);
    const { actionType, formId } = this.props.pageParams[`/${this.props.route.path}`];
    if (actionType === 'Edit') {
      await new Promise(this.initData(formId));
    } else {
      await new Promise(this.initDefaultData);
      this.setState({ selectedRowKeys: [0] });
      this.updateBranch(this.props.branchDict[0].val);
    }
    this.setState({ loading: false });
  }

  initDict(resolve) {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['branch'],
        callback() {
          resolve();
        },
      },
    });
  }

  initDefaultData(resolve) {
    this.props.dispatch({
      type: `${storeName}/setQuestionnaireDefault`,
      payload: {
        callback() {
          resolve();
        },
      },
    });
  }

  initData(id) {
    return (resolve) => {
      this.props.dispatch({
        type: `${storeName}/getQuestionnaireForEdit`,
        payload: {
          id,
          callback() {
            resolve();
          },
        },
      });
    };
  }

  updateFormName(ele) {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: { qFormName: ele.target.value },
    });
  }

  setDefaultFormName() {
    let qFormName = trim(this.props.qFormName);
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: { qFormName },
    });
  }

  updateBranch(val) {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: { qFormBranch: val },
    });
  }

  onSelectRowKeyChange(val) {
    this.setState({ selectedRowKeys: val });
  }

  updateFormTopic(data, indexArr) {
    this.props.dispatch({
      type: `${storeName}/updateFormTopic`,
      payload: {
        data,
        indexArr,
      },
    });
  }

  onAddFormTopic(type) {
    let len = this.props.qFormList.length;
    if (len === 50) {
      return showMessage('该表单最多只能添加50个，无法继续添加', 'error');
    }
    this.setState({ selectedRowKeys: [len] });
    this.props.dispatch({
      type: `${storeName}/addFormTopic`,
      payload: { type },
    });
  }

  onDelFormTopic(index) {
    if (this.state.selectedRowKeys[0] === index) {
      this.onCompleteEdit();
    }
    this.props.dispatch({
      type: `${storeName}/delFormTopic`,
      payload: { index },
    });
  }

  onAddTopicOption(indexArr) {
    this.props.dispatch({
      type: `${storeName}/addTopicOption`,
      payload: { indexArr },
    });
  }

  onDelTopicOption(indexArr) {
    this.props.dispatch({
      type: `${storeName}/delTopicOption`,
      payload: { indexArr },
    });
  }

  onCompleteEdit() {
    this.setState({ selectedRowKeys: [] });
  }

  async onSubmit() {
    this.setState({ loading: true });
    const isSuccess = await new Promise(this.submit);
    if (isSuccess) {
      this.props.dispatch({
        type: 'main/openPage',
        payload: {
          path: '/Questionnaire',
          params: {},
          refresh: true,
        },
      });
    }
    this.setState({ loading: false });
  }

  submit(resolve) {
    this.props.dispatch({
      type: `${storeName}/submitQuestionForm`,
      payload: {
        callback(bool) {
          resolve(bool);
        },
      },
    });
  }

  beforeDestroy() {
    this.props.dispatch({ type: `${storeName}/resetQuestionForm` });
  }

  errorChange(err) {
    this.setState({ error: err });
  }

  render() {
    return (
      <Page className={style[classPrefix]} loading={this.state.loading} route={this.props.route}>
        <Wrapper
          header={
            <Row className={style[`${classPrefix}-top`]}>
              <Col span={9}>
                <FormItem name="表单名称" labelWidth={100} required>
                  <Input
                    placeholder="请输入表单名称"
                    value={this.props.qFormName}
                    onChange={this.updateFormName}
                    onBlur={this.setDefaultFormName}
                    validator={{
                      rules: ['required', QuestionForm.checkTextLength],
                      silent: !Boolean(this.state.error),
                      showStatus: false,
                      showTips: true,
                      onErrorChange: this.errorChange,
                    }}
                  />
                </FormItem>
              </Col>
              <Col span={9}>
                <FormItem name="分公司" labelWidth={100} required>
                  <Select
                    placeholder="请选择分公司"
                    dict={this.props.branchDict}
                    value={this.props.qFormBranch}
                    onChange={this.updateBranch}
                    allowClear={false}
                    showSearch
                  />
                </FormItem>
              </Col>
            </Row>
          }
          footer={
            <div style={{ textAlign: 'center' }}>
              <Button
                type="primary"
                onClick={this.onSubmit}
                disabled={this.state.error || this.props.qFormBranch === null || this.props.qFormList.length === 0}
              >
                完成编辑
              </Button>
            </div>
          }
        >
          <ContentBlock title="信息收集">
            <Wrapper footer={<AddFormTopicBtn onClick={this.onAddFormTopic} />}>
              <Deck
                dataSource={this.props.qFormList}
                render={topicCardRender({
                  onUpdate: this.updateFormTopic,
                  onDelete: this.onDelFormTopic,
                  onAddLine: this.onAddTopicOption,
                  onDelLine: this.onDelTopicOption,
                  onComplete: this.onCompleteEdit,
                })}
                rowSelection={{
                  singleSelect: true,
                  selectedRowKeys: this.state.selectedRowKeys,
                  onChange: this.onSelectRowKeyChange,
                }}
              />
            </Wrapper>
          </ContentBlock>
        </Wrapper>
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    pageParams: state.main.pageParams,
    branchDict: state.dict.branch,
    qFormName: state[storeName].qFormName,
    qFormBranch: state[storeName].qFormBranch,
    qFormList: state[storeName].qFormList,
  };
};

export default connect(mapStateToProps)(QuestionForm);
