import React from 'react';
import ReactDOM from 'react-dom';
import style from './index.less';
import { questionTypeDict } from '../../store/constants';
import { checkTextLength } from '../../store/utils';
import Deck from 'COMPONENT/Deck';
import Button from 'COMPONENT/Button';
import FormItem from 'COMPONENT/FormItem';
import Input from 'COMPONENT/Input';
import Radio from 'COMPONENT/Radio';
import Checkbox from 'COMPONENT/Checkbox';
import IconButton from 'COMPONENT/IconButton';
import TopicDetail from '../TopicDetail';
import { trim } from 'lodash';

const classPrefix = 'TopicCard';

const StopPropagation = Deck.stopPropagation;

const FooterBtnGroup = (props) => {
  return (
    <div className={style[`${classPrefix}-FooterBtnGroup`]}>
      <Button type="default" onClick={props.onDelete}>
        删除
      </Button>
      <Button type="primary" onClick={props.onComplete}>
        完成
      </Button>
    </div>
  );
};

const TopicOption = (props) => {
  return (
    <div className={style[`${classPrefix}-TopicCard`]}>
      <div className={style[`${classPrefix}-TopicCard-Input-wrap`]}>
        <Input
          placeholder="请输入选项内容"
          value={props.name}
          onChange={props.onTextChange}
          onClick={props.onClick}
          onBlur={props.onTextBlur}
        />
      </div>
      <div className={style[`${classPrefix}-TopicCard-IconButton-wrap`]}>
        <IconButton type="plus_circle" onClick={props.onAdd} size={18} />
        <IconButton type="minus_circle" onClick={props.onDel} size={18} disabled={props.delDisabled} />
      </div>
      <div className={style[`${classPrefix}-TopicCard-Checkbox-wrap`]}>
        <Checkbox checked={props.allowRemark} onChange={props.onAllowChange}>
          允许填空
        </Checkbox>
      </div>
    </div>
  );
};

class TopicCard extends React.Component {
  constructor(props) {
    super(props);
    this.updateTitle = this.updateTitle.bind(this);
    this.setDefaultTitle = this.setDefaultTitle.bind(this);
    this.onClickCleanTitle = this.onClickCleanTitle.bind(this);
    this.updateType = this.updateType.bind(this);
    this.updateRequired = this.updateRequired.bind(this);
    this.updateOptionName = this.updateOptionName.bind(this);
    this.setDefaultOptionName = this.setDefaultOptionName.bind(this);
    this.onClickCleanOption = this.onClickCleanOption.bind(this);
    this.updateOptionAllowRemark = this.updateOptionAllowRemark.bind(this);
    this.addOption = this.addOption.bind(this);
    this.delOption = this.delOption.bind(this);
    this.deleteTopic = this.deleteTopic.bind(this);
    this.expandOnly = this.expandOnly.bind(this);
    this.state = {};
    this.$cardEdit = null;
    this.$cardEditContainer = null;
  }
  componentDidMount() {
    if (this.props.isSelected) {
      this.$cardEdit.style.height = `${this.$cardEditContainer.offsetHeight}px`;
    }
  }
  componentWillReceiveProps(nextProps, nextContext) {
    if (nextProps.isSelected !== this.props.isSelected) {
      this.$cardEdit.style.height = nextProps.isSelected ? `${this.$cardEditContainer.offsetHeight}px` : '0px';
    }
  }

  updateTitle(ele) {
    let val = ele.target.value;
    if (val.length > 200) {
      val = val.slice(0, 200);
    }
    this.props.onUpdate({ topic: val }, [this.props.index]);
  }

  setDefaultTitle() {
    let topic = trim(this.props.topic);
    if (topic === '') {
      this.props.onUpdate({ topic: '标题' }, [this.props.index]);
    } else {
      this.props.onUpdate({ topic }, [this.props.index]);
    }
  }

  onClickCleanTitle() {
    if (this.props.topic === '标题') {
      this.props.onUpdate({ topic: '' }, [this.props.index]);
    }
  }

  updateType(val) {
    this.props.onUpdate({ type: val }, [this.props.index]);
  }

  updateRequired(val) {
    this.props.onUpdate({ required: val }, [this.props.index]);
  }

  updateOptionName(val, index) {
    if (val.length > 20) {
      val = val.slice(0, 20);
    }
    this.props.onUpdate({ name: val }, [this.props.index, index]);
  }

  setDefaultOptionName(val, index) {
    let value = trim(val);
    if (value === '') {
      this.updateOptionName('选项', index);
    } else {
      this.updateOptionName(value, index);
    }
  }

  onClickCleanOption(val, index) {
    if (val === '选项') {
      this.updateOptionName('', index);
    }
  }

  updateOptionAllowRemark(val, index) {
    this.props.onUpdate({ allowRemark: val }, [this.props.index, index]);
  }

  addOption(index) {
    this.props.onAddLine([this.props.index, index]);
    this.$cardEdit.style.height = `${this.$cardEditContainer.offsetHeight + 65}px`;
  }

  delOption(index) {
    this.props.onDelLine([this.props.index, index]);
    this.$cardEdit.style.height = `${this.$cardEditContainer.offsetHeight - 65}px`;
  }

  deleteTopic() {
    this.props.onDelete(this.props.index);
  }

  expandOnly(e) {
    if (this.props.isSelected) {
      e.stopPropagation();
    }
  }

  render() {
    return (
      <div className={style[classPrefix]} onClick={this.expandOnly}>
        <div className={style[`${classPrefix}-preview`]}>
          <TopicDetail {...this.props} />
        </div>
        <StopPropagation>
          <div
            className={style[`${classPrefix}-edit`]}
            ref={(c) => (this.$cardEdit = this.$cardEdit || ReactDOM.findDOMNode(c))}
          >
            <div
              className={style[`${classPrefix}-edit-container`]}
              ref={(c) => (this.$cardEditContainer = this.$cardEditContainer || ReactDOM.findDOMNode(c))}
            >
              <div className={style[`${classPrefix}-edit-body`]}>
                <FormItem name="标题" labelWidth={125}>
                  <Input
                    placeholder="请输入标题内容"
                    value={this.props.topic}
                    onChange={this.updateTitle}
                    onClick={this.onClickCleanTitle}
                    onBlur={this.setDefaultTitle}
                  />
                </FormItem>
                <FormItem name="选项类型" labelWidth={125}>
                  {questionTypeDict.map((item) => (
                    <Radio
                      checked={this.props.type === Number(item.val)}
                      onChange={() => {
                        this.updateType(Number(item.val));
                      }}
                      key={item.val}
                    >
                      {item.text}
                    </Radio>
                  ))}
                </FormItem>
                <FormItem name="用户必答" labelWidth={125}>
                  <Radio
                    checked={this.props.required}
                    onClick={() => {
                      this.updateRequired(true);
                    }}
                  >
                    是
                  </Radio>
                  <Radio
                    checked={!this.props.required}
                    onClick={() => {
                      this.updateRequired(false);
                    }}
                  >
                    否
                  </Radio>
                </FormItem>
                {this.props.optionList.map((item, index, list) => (
                  <FormItem name={`选项${index + 1}`} labelWidth={125} key={index}>
                    <TopicOption
                      {...item}
                      key={index}
                      onClick={() => {
                        this.onClickCleanOption(item.name, index);
                      }}
                      onTextBlur={() => {
                        this.setDefaultOptionName(item.name, index);
                      }}
                      onTextChange={(ele) => {
                        this.updateOptionName(ele.target.value, index);
                      }}
                      onAllowChange={(ele) => {
                        this.updateOptionAllowRemark(ele.target.checked, index);
                      }}
                      onAdd={() => {
                        this.addOption(index);
                      }}
                      onDel={() => {
                        this.delOption(index);
                      }}
                      delDisabled={list.length === 1}
                    />
                  </FormItem>
                ))}
              </div>
              <div className={style[`${classPrefix}-edit-footer`]}>
                <FooterBtnGroup onDelete={this.deleteTopic} onComplete={this.props.onComplete} />
              </div>
            </div>
          </div>
        </StopPropagation>
      </div>
    );
  }
}

export default TopicCard;
