import React from 'react';
import style from './index.less';
import { Radio as RadioVal, Checkbox as CheckboxVal } from '../../store/constants';
import classNames from 'classnames';
import Label from '../Label';
import Input from 'COMPONENT/Input';
import Radio from 'COMPONENT/Radio';
import Checkbox from 'COMPONENT/Checkbox';

const classPrefix = 'TopicDetail';

const inputStyle = {
  width: 480,
  marginLeft: 16,
};
// 单选
const RadioOptions = (props) => {
  return (
    <Radio checked={props.selected} disabled={props.disabled}>
      <span className={style[`${classPrefix}-option-body`]}>
        <span>{props.name}</span>
        {props.allowRemark && (
          <Input placeholder="请输入" onChange={function () {}} value={props.remark} style={inputStyle} />
        )}
      </span>
    </Radio>
  );
};
// 多选
const CheckboxOptions = (props) => {
  return (
    <Checkbox checked={props.selected} disabled={props.disabled}>
      <span className={style[`${classPrefix}-option-body`]}>
        <span>{props.name}</span>
        {props.allowRemark && (
          <Input placeholder="请输入" onChange={function () {}} value={props.remark} style={inputStyle} />
        )}
      </span>
    </Checkbox>
  );
};
// 构建自定义选项
const buildOptions = (props) => {
  if (props.type === RadioVal) {
    return props.optionList.map((item, index) => (
      <RadioOptions {...item} disabled={props.disabled} key={`detail_${index}`} />
    ));
  } else if (props.type === CheckboxVal) {
    return props.optionList.map((item, index) => (
      <CheckboxOptions {...item} disabled={props.disabled} key={`detail_${index}`} />
    ));
  } else {
    return null;
  }
};

const TopicDetail = (props) => {
  return (
    <div className={classNames(style[classPrefix], props.className || '')}>
      <Label classname={style[`${classPrefix}-Label`]} name={props.topic} required={props.required} />
      <div className={style[`${classPrefix}-option-wrap`]}>{buildOptions(props)}</div>
    </div>
  );
};
TopicDetail.defaultProps = {
  disabled: false,
};

export default TopicDetail;
