/**
 * Created by Haolin on 2019/5/29.
 */
import React from 'react';
import { connect } from 'dva';
import style from './index.less';
import { storeName, validity } from './store/constants';
import Page from 'COMPONENT/Page';
import List from 'COMPONENT/List';
import IndexBar from './IndexBar';
import ListAction from 'COMPONENT/ListAction';

const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L } = List;

const classPrefix = 'Questionnaire';

class Questionnaire extends React.Component {
  constructor(props) {
    super(props);
    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.loadList = this.loadList.bind(this);
    this.updatePage = this.updatePage.bind(this);
    this.search = this.search.bind(this);
    this.goTo = this.goTo.bind(this);
    this.addNew = this.addNew.bind(this);
    this.changePage = this.changePage.bind(this);
    this.formEdit = this.formEdit.bind(this);
    this.formDetail = this.formDetail.bind(this);
    this.switchValidity = this.switchValidity.bind(this);
    this.beforeDestroy = this.beforeDestroy.bind(this);
    this.state = {
      name: '更多信息(用于收集学生更详细的个人信息)',
      loading: false,
    };
    this.columns = [
      { title: '序号', dataIndex: 'number', key: 'number', width: COL_WIDTH_S },
      { title: '表单名称', dataIndex: 'formName', key: 'formName', width: COL_WIDTH_L },
      { title: '分公司', dataIndex: 'branchName', key: 'branchName', width: COL_WIDTH_M },
      { title: '状态', dataIndex: 'formStatus', key: 'formStatus', width: COL_WIDTH_S },
      { title: '创建时间', dataIndex: 'createTime', key: 'createTime', width: COL_WIDTH_L },
      {
        title: '操作',
        dataIndex: 'formId',
        key: 'actionPlus',
        width: COL_WIDTH_M,
        render: (text, record, index) => (
          <ListAction
            config={[
              {
                text: '修改',
                onClick: () => {
                  this.formEdit(text, record, index);
                },
              },
              {
                text: '详情',
                onClick: () => {
                  this.formDetail(text, record, index);
                },
              },
              {
                text: validity[record.status === 0 ? 1 : 0],
                onClick: () => {
                  this.switchValidity(text, record, index);
                },
                needConfirm: true,
              },
            ]}
          />
        ),
      },
    ];
  }
  componentDidMount() {
    this.init();
  }
  componentWillUnmount() {
    this.beforeDestroy();
  }

  async init() {
    this.setState({ loading: true });
    await new Promise(this.initDict);
    await new Promise(this.loadList);
    this.setState({ loading: false });
  }

  initDict(resolve) {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['branch'],
        callback() {
          resolve();
        },
      },
    });
  }

  loadList(resolve) {
    this.props.dispatch({
      type: `${storeName}/loadFormList`,
      payload: {
        callback() {
          resolve();
        },
      },
    });
  }

  updatePage(pageCurrent, pageSize) {
    let payload = {};
    if (pageSize && pageSize !== this.props.pageSize) {
      payload.pageCurrent = 1;
      payload.pageSize = pageSize;
    } else {
      payload.pageCurrent = pageCurrent;
    }
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload,
    });
  }

  async changePage({ current, pageSize }) {
    this.updatePage(current, pageSize);
    this.setState({ loading: true });
    await new Promise(this.loadList);
    this.setState({ loading: false });
  }

  search() {
    this.changePage({
      current: 1,
      pageSize: this.props.pageSize,
    });
  }

  goTo(path, params = null, refresh = true) {
    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path,
        params,
        refresh,
      },
    });
  }

  addNew() {
    this.goTo('/Questionnaire/Add', { actionType: 'Add' });
  }

  formEdit(text) {
    this.goTo('/Questionnaire/Edit', { actionType: 'Edit', formId: text });
  }

  formDetail(text) {
    this.goTo('/Questionnaire/Detail', { formId: text });
  }

  async switchValidity(text, record, index) {
    this.setState({ loading: true });
    await new Promise((resolve) => {
      this.props.dispatch({
        type: `${storeName}/switchValidity`,
        payload: {
          formId: text,
          status: record.status === 0 ? 1 : 0,
          index,
          callback() {
            resolve();
          },
        },
      });
    });
    this.setState({ loading: false });
  }

  beforeDestroy() {
    this.props.dispatch({
      type: `${storeName}/resetQuestionnaire`,
    });
  }

  render() {
    return (
      <Page
        name={this.state.name}
        className={style[classPrefix]}
        loading={this.state.loading}
        route={this.props.route}
        fitWindowHeight={true}
        subHeader={<IndexBar onSearch={this.search} onAdd={this.addNew} />}
      >
        <List
          dataSource={this.props.list}
          columns={this.columns}
          pagination={{
            total: this.props.total,
            current: this.props.pageCurrent,
            pageSize: this.props.pageSize,
          }}
          onChange={this.changePage}
        />
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    list: state[storeName].formList,
    total: state[storeName].formTotal,
    pageCurrent: state[storeName].pageCurrent,
    pageSize: state[storeName].pageSize,
  };
};

export default connect(mapStateToProps)(Questionnaire);
