/**
 * Created by Haolin<haolinhom@gmail.com> on 2019/5/29.
 */
import { storeName, validity } from './constants';
import { errorsHandling, createTopic, createOption, buildTopicListForSave } from './utils';
import {
  listAllQuestionnaires,
  saveQuestionnaire,
  selectById,
  updateQuestionnaireStatus,
} from 'SERVICE/marketActivityManage';
import { clone } from 'lodash';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import moment from 'moment';

export default {
  namespace: storeName,
  state: {
    /**
     * 更多信息列表
     * */
    sKey_branch: [],
    sKey_validity: null,
    formList: [],
    formTotal: 0,
    pageCurrent: 1,
    pageSize: 20,
    /**
     * 新增/修改
     * */
    qFormId: null,
    qFormName: '',
    qFormBranch: null,
    qFormList: [],
    qFormDelTopic: [],
    qFormDelOption: {},
    /**
     * 详情
     * */
    detailId: null,
    detailName: null,
    detailBranchName: null,
    detailList: [],
  },
  reducers: {
    updateState(state, { payload }) {
      return {
        ...state,
        ...payload,
      };
    },
    // 更新表单列表
    updateFormList(state, { payload: { list, total } }) {
      let _payload = {
        formList: list.map((item, index) => ({
          number: index + 1 + state.pageSize * (state.pageCurrent - 1),
          formId: item.id,
          formName: item.name,
          branchName: item.orgName,
          status: item.status,
          formStatus: validity[item.status],
          createTime: moment(item.createTime).format('YYYY-MM-DD HH:mm:ss'),
          _origin: item,
        })),
      };
      if (total) {
        _payload.formTotal = total;
      }
      return {
        ...state,
        ..._payload,
      };
    },
    // 更新表单状态
    updateFormStatus(state, { payload: { status, index } }) {
      let formList = clone(state.formList);
      formList[index].status = status;
      formList[index].formStatus = validity[status];
      return {
        ...state,
        formList,
      };
    },
    // 重置表单列表页状态
    resetQuestionnaire(state) {
      return {
        ...state,
        sKey_branch: [],
        sKey_validity: null,
        formList: [],
        formTotal: 0,
        pageCurrent: 1,
        pageSize: 20,
      };
    },
    // 更新表单问题/选项内容
    updateFormTopic(state, { payload: { data, indexArr } }) {
      let qFormList = clone(state.qFormList);
      if (indexArr.length === 1) {
        qFormList[indexArr[0]] = {
          ...qFormList[indexArr[0]],
          ...data,
        };
      } else if (indexArr.length === 2) {
        qFormList[indexArr[0]].optionList[indexArr[1]] = {
          ...qFormList[indexArr[0]].optionList[indexArr[1]],
          ...data,
        };
      }
      return {
        ...state,
        qFormList,
      };
    },
    // 新加表单问题
    addFormTopic(state, { payload: { type } }) {
      let qFormList = clone(state.qFormList);
      qFormList.push(createTopic());
      return {
        ...state,
        qFormList,
      };
    },
    // 删除表单问题
    delFormTopic(state, { payload: { index } }) {
      let _payload = { ...state };
      let qFormList = clone(state.qFormList);
      if (qFormList[index].id) {
        qFormList[index].isDelete = 1;
        _payload.qFormList = qFormList;
        let qFormDelTopic = clone(state.qFormDelTopic);
        qFormDelTopic.push(qFormList.splice(index, 1)[0]);
        _payload.qFormDelTopic = qFormDelTopic;
      } else {
        qFormList.splice(index, 1);
        _payload.qFormList = qFormList;
      }
      return _payload;
    },
    // 新加选项
    addTopicOption(state, { payload: { indexArr } }) {
      let qFormList = clone(state.qFormList);
      qFormList[indexArr[0]].optionList.splice(indexArr[1] + 1, 0, createOption());
      return {
        ...state,
        qFormList,
      };
    },
    // 删除选项
    delTopicOption(state, { payload: { indexArr } }) {
      let _payload = { ...state };
      let qFormList = clone(state.qFormList);
      if (qFormList[indexArr[0]].optionList[indexArr[1]].id) {
        qFormList[indexArr[0]].optionList[indexArr[1]].isDelete = 1;
        _payload.qFormList = qFormList;
        let qFormDelOption = clone(state.qFormDelOption);
        if (!qFormDelOption[qFormList[indexArr[0]].id]) {
          qFormDelOption[qFormList[indexArr[0]].id] = [];
        }
        qFormDelOption[qFormList[indexArr[0]].id].push(qFormList[indexArr[0]].optionList.splice(indexArr[1], 1)[0]);
        _payload.qFormDelOption = qFormDelOption;
      } else {
        qFormList[indexArr[0]].optionList.splice(indexArr[1], 1);
        _payload.qFormList = qFormList;
      }
      return _payload;
    },
    // 重置表单新增/修改页状态
    resetQuestionForm(state) {
      return {
        ...state,
        qFormId: null,
        qFormName: '',
        qFormBranch: null,
        qFormList: [],
        qFormDelTopic: [],
        qFormDelOption: {},
      };
    },
    // 重置详情页状态
    resetDetail(state) {
      return {
        ...state,
        detailId: null,
        detailName: null,
        detailBranchName: null,
        detailList: [],
      };
    },
  },
  effects: {
    // 加载列表数据
    *loadFormList({ payload: { callback } }, { select, put }) {
      let apiParams = yield select((state) => ({
        orgIds: state[storeName].sKey_branch,
        status: state[storeName].sKey_validity ? Number(state[storeName].sKey_validity) : null,
        whetherPaging: true,
        page: state[storeName].pageCurrent,
        rows: state[storeName].pageSize,
        getTotal: state[storeName].pageCurrent === 1,
      }));
      const rsp = yield listAllQuestionnaires(apiParams, true).catch(errorsHandling());
      if (rsp && rsp.resultCode === 0) {
        let payload = { list: rsp.rows || [] };
        if (apiParams.getTotal) {
          payload.total = rsp.records;
        }
        yield put({
          type: 'updateFormList',
          payload,
        });
      }
      if (callback) callback();
    },
    // 切换表单状态
    *switchValidity({ payload: { formId, status, index, callback } }, { put }) {
      let apiParams = {
        id: formId,
        status,
      };
      const rsp = yield updateQuestionnaireStatus(apiParams, true).catch(errorsHandling());
      if (rsp && rsp.resultCode === 0) {
        showMessage('成功修改表单状态！', 'success');
        yield put({
          type: 'updateFormStatus',
          payload: { status, index },
        });
      }
      if (callback) callback();
    },
    // 提交表单新增/修改
    *submitQuestionForm({ payload: { callback } }, { select }) {
      let apiParams = yield select((state) => ({
        bossQuestionnaire: {
          id: state[storeName].qFormId,
          name: state[storeName].qFormName,
          orgId: state[storeName].qFormBranch,
          isDelete: 0,
        },
        topicList: buildTopicListForSave(
          state[storeName].qFormList,
          state[storeName].qFormDelTopic,
          state[storeName].qFormDelOption,
        ),
      }));
      const rsp = yield saveQuestionnaire(apiParams, true).catch(errorsHandling());
      if (rsp && rsp.resultCode === 0) {
        showMessage('更多信息保存成功！', 'success');
        if (callback) callback(true);
      } else {
        if (callback) callback(false);
      }
    },
    // 新增表单时设置问卷默认数据
    *setQuestionnaireDefault({ payload: { callback } }, { put }) {
      yield put({
        type: 'updateState',
        payload: { qFormList: [createTopic()] },
      });
      if (callback) callback();
    },
    // 获取修改时的表单详情
    *getQuestionnaireForEdit({ payload: { id, callback } }, { put }) {
      const rsp = yield selectById({ id }, true).catch(errorsHandling());
      if (rsp && rsp.resultCode === 0) {
        yield put({
          type: 'updateState',
          payload: {
            qFormId: rsp.data.bossQuestionnaire.id,
            qFormName: rsp.data.bossQuestionnaire.name,
            qFormBranch: rsp.data.bossQuestionnaire.orgId,
            qFormList: rsp.data.topicList,
          },
        });
      }
      if (callback) callback();
    },
    // 获取表单详情
    *getQuestionnaireForDetail({ payload: { id, callback } }, { put }) {
      const rsp = yield selectById({ id }, true).catch(errorsHandling());
      if (rsp && rsp.resultCode === 0) {
        yield put({
          type: 'updateState',
          payload: {
            detailId: rsp.data.bossQuestionnaire.id,
            detailName: rsp.data.bossQuestionnaire.name,
            detailBranchName: rsp.data.bossQuestionnaire.orgName,
            detailList: rsp.data.topicList,
          },
        });
      }
      if (callback) callback();
    },
  },
};
