/**
 * Created by Haolin<haolinhom@gmail.com> on 2019/5/29.
 */
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { Radio, Checkbox, validity } from './constants';
import { clone } from 'lodash';

function errorsHandling(handleErr, errKeyName) {
  return function (err) {
    showMessage(errKeyName ? err[errKeyName] : err.text, 'error');
    if (handleErr) handleErr();
  };
}
/**
 * 创建问题
 *
 * 单选(Radio)/多选(Checkbox)
 * */
function createTopic(type) {
  let result;
  switch (type) {
    case undefined:
    case Radio:
    case Checkbox:
      result = new Object({
        id: null,
        topic: '标题',
        type: type || Radio,
        required: false,
        isDelete: 0,
        optionList: [
          {
            id: null,
            name: '选项',
            allowRemark: false,
            remark: '',
            isDelete: 0,
            selected: false,
          },
          {
            id: null,
            name: '选项',
            allowRemark: false,
            remark: '',
            isDelete: 0,
            selected: false,
          },
        ],
      });
      break;
    default:
      throw new Error(`Error: createTopic's type is not allow! Type(${type})`);
  }
  return result;
}
// 创建选项
function createOption(type) {
  let result;
  switch (type) {
    case undefined:
    case Radio:
    case Checkbox:
      result = new Object({
        id: null,
        name: '选项',
        allowRemark: false,
        remark: '',
        isDelete: 0,
        selected: false,
      });
      break;
    default:
      throw new Error(`Error: createOption's type is not allow! Type(${type})`);
  }
  return result;
}
// 过滤掉已删除数据
function filterIsDelete(list) {
  let tempList = clone(list).filter((item) => !item.isDelete);
  return tempList.map((topic) => ({
    ...topic,
    optionList: topic.optionList.filter((item) => !item.isDelete),
  }));
}
// 构建保存/修改接口的optionList参数
function buildTopicListForSave(list, delTopic, delOption) {
  let result = list;
  if (delTopic.length > 0) {
    result = list.concat(delTopic);
  }
  let optionKeys = Object.keys(delOption);
  if (optionKeys.length > 0) {
    let topicKeyIndex = {};
    result.forEach((item, index) => {
      topicKeyIndex[item.id] = index;
    });
    optionKeys.forEach((item) => {
      delOption[item].forEach((option) => {
        result[topicKeyIndex[item]].optionList.push(option);
      });
    });
  }
  return result;
}

export { errorsHandling, createTopic, createOption, filterIsDelete, buildTopicListForSave };
