import React from 'react';
import SearchBar from 'COMPONENT/SearchBar';
import MultiSelect from 'COMPONENT/MultiSelect';
import Input from 'COMPONENT/Input';
import DatePicker from 'COMPONENT/DatePicker';

const Search = (props) => {
  const {
    valueMap: {
      beginTime,
      endTime,
      activityName,
      blCampusId,
      activityTypeId,
      auditStatus,
      tradeNo,
      studentName,
      customerPhone,
    },
    dict: { campusDict, activityTypeDict, auditStatusDict },
    handleKeyword,
    getDisabledStartDate,
    getDisabledEndDate,
    ...resetProps
  } = props;
  return (
    <SearchBar className="class-search" {...resetProps}>
      <DatePicker
        placeholder="开始时间"
        value={beginTime}
        disabledDate={getDisabledStartDate}
        onChange={(val) => handleKeyword({ type: 'beginTime', val })}
      />
      <DatePicker
        placeholder="结束时间"
        value={endTime}
        disabledDate={getDisabledEndDate}
        onChange={(val) => handleKeyword({ type: 'endTime', val })}
      />
      <Input
        placeholder="请输入引流活动名称"
        value={activityName}
        onChange={(e) => handleKeyword({ type: 'activityName', val: e.target.value })}
      />
      <MultiSelect
        placeholder="学习中心"
        dict={campusDict}
        value={blCampusId}
        allowClear={true}
        onChange={(val) => handleKeyword({ type: 'blCampusId', val })}
      />
      <MultiSelect
        placeholder="活动类型"
        dict={activityTypeDict}
        value={activityTypeId}
        allowClear={true}
        showSearch={false}
        onChange={(val) => handleKeyword({ type: 'activityTypeId', val })}
      />
      <MultiSelect
        placeholder="审核状态"
        dict={auditStatusDict}
        value={auditStatus}
        allowClear={true}
        showSearch={false}
        onChange={(val) => handleKeyword({ type: 'auditStatus', val })}
      />
      <Input
        placeholder="输入交易单号"
        value={tradeNo}
        onChange={(e) => handleKeyword({ type: 'tradeNo', val: e.target.value })}
      />
      <Input
        placeholder="输入学生名称"
        value={studentName}
        onChange={(e) => handleKeyword({ type: 'studentName', val: e.target.value })}
      />
      <Input
        placeholder="输入客户号码"
        value={customerPhone}
        onChange={(e) => handleKeyword({ type: 'customerPhone', val: e.target.value })}
      />
    </SearchBar>
  );
};
export default Search;
