import React from 'react';
import { connect } from 'dva';
import Button from 'COMPONENT/Button';
import Page from 'COMPONENT/Page';
import List from 'COMPONENT/List';
import SearchBar from './components/Search';
import { loadCommonDict, loadCampusDict, loadBranchCampusDict } from 'UTIL/dict';
import moment from 'moment';
import styles from '../MarketActivity/styles.less';
import commonStyles from '../common/commonStyle.less';
import { dateToMoment } from 'COMPONENT/utils/form';
import { loadRefundList, passRefund, disPassRefund, applyRefund } from './loader';
import showMessage from 'SRC/components/utils/globalTips/showMessage';
import showConfirm from 'SRC/components/utils/globalTips/showConfirm';
import { renderMultiLine } from '../common';

const buttonTypeMap = {
  PASS: 'PASS', // 通过退款
  FAIL: 'FAIL', // 不通过退款
  RESET: 'RESET', // 重新退款
};
// 退款状态map
const surrenderStatus = {
  0: {
    text: '',
  },
  1: {
    text: '未退款',
    handlePass: true, // 通过操作
    handleUnPass: true, // 不通过操作
  },
  2: {
    text: '退款中',
  },
  3: {
    text: '退款成功',
  },
  4: {
    text: '退款失败',
    handleResetFund: true, //重新退款
  },
};
const auditStatusMap = {
  0: '未审核',
  1: '已通过',
  2: '未通过',
};
const auditStatusDict = [
  { text: '未审核', val: '0' },
  { text: '已通过', val: '1' },
  { text: '未通过', val: '2' },
];
let branchObj = {};
let activityTypeObj = {};

const _beginTime = moment().subtract(1, 'months').format().slice(0, 10);
const _endTime = moment().format().slice(0, 10);

class RefundDetail extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // boolean 页面加载状态
      isRefundConfirmModal: false,
      searchExpand: true,
      //查询条件
      beginTime: _beginTime,
      endTime: _endTime,
      activityName: null,
      blCampusId: null,
      tradeNo: null, //交易单号
      auditStatus: ['0'],
      studentName: null,
      customerPhone: null,
      activityTypeId: null, //活动类型
      //列表数据'
      listLoading: false,
      rows: [],
      pageNo: 1,
      pageSize: 10,
      total: null,
      selectedRowKeys: [], //列表选中项
      filterSelectionArray: [], //可以执行通过不通过的列表数组
      //数据字典数据
      campusDict: [],
      activityTypeDict: [],
      auditStatusDict: auditStatusDict,
    };
  }

  componentDidMount() {
    this.initContent();
  }

  loadListData = async () => {
    const {
      activityTypeId,
      activityName,
      beginTime,
      endTime,
      customerPhone,
      studentName,
      blCampusId,
      tradeNo,
      auditStatus,
      pageNo,
      pageSize,
    } = this.state;
    let apiParams = {
      activityName,
      activityTypeId,
      auditStatus,
      beginTime,
      blCampusId,
      customerPhone,
      endTime,
      pageNo,
      pageSize,
      studentName,
      tradeNo,
    };

    try {
      // 清除勾选项
      this.setState({
        selectedRowKeys: [],
      });
      return await loadRefundList(apiParams);
    } catch (e) {
      console.log(e);
      showMessage(e.text);
      return [];
    }
  };

  initContent = async () => {
    this.setState({ loading: true });
    const asyncCalls = [loadBranchCampusDict(), loadCommonDict('MARKET_ACTIVITY_TYPE')];
    let [campusDict, activityTypeDict] = await Promise.all(asyncCalls);
    const campusDictToArray = campusDict.reduce((arr, ele) => {
      return [...arr, ...ele.options];
    }, []);
    branchObj = this.dictToObj(campusDictToArray);
    activityTypeObj = this.dictToObj(activityTypeDict);
    //获取列表数据
    const list = await this.loadListData();
    this.setState({
      loading: false,
      rows: list.datas,
      total: list.rowCount,
      pageNo: list.pageNo,
      campusDict,
      activityTypeDict,
    });
  };

  dictToObj = (dict) => {
    return (dict || []).reduce((obj, ele) => {
      obj[ele.val] = ele.text ? ele.text : ele.name;
      return obj;
    }, {});
  };
  listRowClick = (record, index, event) => {
    const { rows } = this.state;
    const oldSelectedRowKeys = this.state.selectedRowKeys;
    let newSelectedRowKeys = [];
    if (oldSelectedRowKeys.indexOf(index) > -1) {
      newSelectedRowKeys = oldSelectedRowKeys.filter((item) => item !== index);
    } else {
      newSelectedRowKeys = [...oldSelectedRowKeys, index].sort();
    }
    let filterSelectionArray = newSelectedRowKeys
      .map((item) => rows[item])
      .filter((item) => item.surrenderStatus == 1 && item.auditUserId === this.props.user.userId);
    // 如果勾选了其他的不同类型的列表 只允许是全部
    if (newSelectedRowKeys.length > filterSelectionArray.length) {
      filterSelectionArray = [];
    }
    this.setState({ selectedRowKeys: newSelectedRowKeys, filterSelectionArray });
  };

  search = async () => {
    await this.updateState({ listLoading: true, pageNo: 1 });
    const list = await this.loadListData();
    this.updateState({
      listLoading: false,
      rows: list.datas,
      total: list.rowCount,
      pageNo: list.pageNo,
    });
  };
  // 重新加载当前页面数据
  resetLoadData = async () => {
    await this.updateState({ listLoading: true });
    const list = await this.loadListData();
    this.updateState({
      listLoading: false,
      rows: list.datas,
      total: list.rowCount,
      pageNo: list.pageNo,
    });
  };

  reset = async () => {
    this.updateState({
      beginTime: _beginTime,
      endTime: _endTime,
      activityName: null,
      blCampusId: null,
      tradeNo: null,
      auditStatus: ['0'],
      studentName: null,
      customerPhone: null,
      activityTypeId: null,
    });
  };

  pageChange = async ({ current, pageSize }) => {
    await this.updateState({ listLoading: true, pageSize, pageNo: current });
    const list = await this.loadListData();
    this.updateState({
      listLoading: false,
      rows: list.datas,
      total: list.rowCount,
      pageNo: list.pageNo,
    });
  };

  updateRow = (raw, changed) => {
    this.setState({ listLoading: true });
    const idx = this.state.rows.findIndex((ele) => ele.id === raw.id);
    const rows = this.state.rows.map((row, i) => {
      if (i === idx) {
        return { ...row, ...changed };
      }
      return row;
    });
    this.setState({
      listLoading: false,
      rows,
    });
    return { ...raw, ...changed };
  };

  /**
   * 获取要禁用的开始日期值
   * @param  {string} beginTime 过滤前的日期值
   * @return {boolean} 是否要禁用
   */
  getDisabledStartDate = (beginTime) => {
    const { endTime } = this.state;
    if (!beginTime || !endTime) {
      return false;
    }
    return dateToMoment(beginTime).valueOf() > dateToMoment(endTime).valueOf();
  };

  /**
   * 获取要禁用的结束日期值
   * @param  {string} endTime   过滤前的日期值
   * @return {boolean}      是否要禁用
   */
  getDisabledEndDate = (endTime) => {
    const { beginTime } = this.state;
    if (!endTime || !beginTime) {
      return false;
    }
    return dateToMoment(endTime).valueOf() <= dateToMoment(beginTime).valueOf();
  };

  /**
   * 更新state
   * */
  updateState = (changedState) => {
    const newState = {
      ...this.state,
      ...changedState,
    };
    return new Promise((resolve) => {
      this.setState(newState, () => {
        resolve();
      });
    });
  };

  handleSetState = ({ type, val }) => {
    this.setState({
      [type]: val,
    });
  };

  //添加列表项时 帅选出能够编辑通过和不通过的列表项
  filterSelectionKey = (keys) => {
    const { rows } = this.state;
    let filterSelectionArray = keys
      .map((item) => rows[item])
      .filter((item) => item.surrenderStatus == 1 && item.auditUserId === this.props.user.userId);
    // 如果勾选了其他的不同类型的列表 只允许是全部
    if (keys.length > filterSelectionArray.length) {
      filterSelectionArray = [];
    }
    this.updateState({
      filterSelectionArray,
      selectedRowKeys: keys,
    });
  };

  //点击按钮 ‘pass’通过 'fail'不通过 ‘reset 重新退款’
  handleButtonRefund = (handleType, records) => {
    const { PASS, FAIL, RESET } = buttonTypeMap;
    // 兼容老数据 优先取merchRefundNo值 没有的话才拿订单id
    const recordArray = records.map((item) => item.merchRefundNo || +item.orderNo);
    switch (handleType) {
      // 退款
      case PASS:
        showConfirm('确定通过退款吗？', () => {
          this.updateState({
            loading: true,
          });
          passRefund(recordArray).then(
            () => {
              showMessage('操作成功！', 'success');
              this.resetLoadData();
              this.updateState({
                loading: false,
              });
            },
            (err) => {
              showMessage(err.text, 'error');
              this.updateState({
                loading: false,
              });
            },
          );
        });
        break;
      // 不退款
      case FAIL:
        this.updateState({
          loading: true,
        });
        disPassRefund(recordArray).then(
          () => {
            showMessage('操作成功！', 'success');
            this.resetLoadData();
            this.updateState({
              loading: false,
            });
          },
          (err) => {
            showMessage(err.text, 'error');
            this.updateState({
              loading: false,
            });
          },
        );
        break;
      // 重新退款
      case RESET:
        const { auditUserId, orderNo, refundReason } = records[0];
        this.updateState({
          loading: true,
        });
        applyRefund({
          orderNo,
          refundReason,
          auditUserId,
        }).then(
          () => {
            showMessage('操作成功！', 'success');
            this.resetLoadData();
            this.updateState({
              loading: false,
            });
          },
          (err) => {
            showMessage(err.text, 'error');
            this.updateState({
              loading: false,
            });
          },
        );
        break;
    }
  };

  /**
   * 表头
   */
  columns = [
    {
      title: '序号',
      dataIndex: 'index',
      key: 'index',
      width: 60,
      fixed: 'left',
      render: (text, record, index) => index + 1,
    },
    { title: '学生名称', dataIndex: 'studentName', key: 'studentName', width: 100 },
    {
      title: '审核状态',
      dataIndex: 'auditStatus',
      key: 'auditStatus',
      width: 80,
      render: (text) => {
        return auditStatusMap[+text];
      },
    },
    {
      title: '退款状态',
      dataIndex: 'surrenderStatus',
      key: 'surrenderStatus',
      width: 80,
      render: (text) => {
        return surrenderStatus[+text].text;
      },
    },
    { title: '年级', dataIndex: 'gradeName', key: 'gradeName', width: 80 },
    {
      title: '学习中心',
      dataIndex: 'blCampusName',
      key: 'blCampusName',
      width: 100,
      render: (text, record) => {
        const campusId = record.blCampusId;
        return text || branchObj[campusId];
      },
    },
    { title: '活动名称', dataIndex: 'activityName', key: 'activityName', width: 110 },
    { title: '活动类型', dataIndex: 'activityTypeName', key: 'activityTypeName', width: 110 },
    {
      title: '退款产品',
      dataIndex: 'refundProductName',
      key: 'refundProductName',
      width: 110,
      render: renderMultiLine,
    },
    {
      title: '退款金额',
      dataIndex: 'amount',
      key: 'amount',
      width: 80,
      render: (text) => {
        return text / 100;
      },
    },
    { title: '申请人', dataIndex: 'applyUserName', key: 'applyUserName', width: 150 },
    { title: '退款审批人', dataIndex: 'auditUserName', key: 'auditUserName', width: 150 },
    {
      title: '申请时间',
      dataIndex: 'applyTime',
      key: 'applyTime',
      width: 150,
      render: (text) => {
        return moment(text).format('YYYY-MM-DD HH:mm');
      },
    },
    { title: '退款原因', dataIndex: 'refundReason', key: 'refundReason', width: 250 },
    { title: '交易单号', dataIndex: 'tradeNo', key: 'tradeNo', width: 220 },
    { title: '客户号码', dataIndex: 'customerPhone', key: 'customerPhone', width: 130 },
    {
      title: '操作',
      key: 'action',
      width: 180,
      fixed: 'right',
      getActionConfig: (text, record) => {
        const { PASS, FAIL, RESET } = buttonTypeMap;
        if (record.auditUserId !== this.props.user.userId) return [];
        switch (record.surrenderStatus) {
          case 1:
            return [
              {
                text: '退款',
                onClick: () => {
                  this.handleButtonRefund(PASS, [record]);
                },
              },
              {
                text: '拒绝',
                onClick: () => {
                  this.handleButtonRefund(FAIL, [record]);
                },
              },
            ];
          // case 4 :
          //   return [{ text: '重新退款', onClick: () => {this.handleButtonRefund(RESET, [record])}}]
          default:
            return [];
        }
      },
    },
  ];

  render() {
    const {
      loading,
      searchExpand,
      listLoading,
      rows,
      pageNo,
      pageSize,
      total,
      beginTime,
      endTime,
      activityName,
      blCampusId,
      activityTypeId,
      auditStatus,
      tradeNo,
      studentName,
      customerPhone,
      campusDict,
      activityTypeDict,
      auditStatusDict,
      selectedRowKeys,
      filterSelectionArray,
    } = this.state;
    const valueMap = {
      beginTime,
      endTime,
      activityName,
      blCampusId,
      activityTypeId,
      auditStatus,
      tradeNo,
      studentName,
      customerPhone,
    };
    const dictMap = {
      campusDict,
      activityTypeDict,
      auditStatusDict,
    };

    const { PASS, FAIL } = buttonTypeMap;
    return (
      <Page
        loading={loading}
        route={this.props.route}
        fitWindowHeight
        subHeader={
          <div className={commonStyles.searchBar} style={{ paddingRight: '0px', minHeight: '102px' }}>
            <SearchBar
              onSearch={this.search}
              onReset={this.reset}
              expand={searchExpand}
              onToggleExpand={(expand) => this.setState({ searchExpand: expand })}
              handleKeyword={this.handleSetState}
              getDisabledStartDate={this.getDisabledStartDate}
              getDisabledEndDate={this.getDisabledEndDate}
              valueMap={valueMap}
              dict={dictMap}
            />
            <div className={styles.buttonWrapper}>
              <Button
                className={styles.button}
                onClick={() => {
                  this.handleButtonRefund(PASS, filterSelectionArray);
                }}
                disabled={!filterSelectionArray.length}
              >
                退款
              </Button>
              <Button
                className={styles.button}
                onClick={() => {
                  this.handleButtonRefund(FAIL, filterSelectionArray);
                }}
                disabled={!filterSelectionArray.length}
              >
                拒绝
              </Button>
            </div>
          </div>
        }
      >
        <List
          columns={this.columns}
          loading={listLoading}
          dataSource={rows}
          pagination={{
            current: pageNo,
            pageSize,
            total: total,
          }}
          rowSelection={{
            selectedRowKeys,
            onChange: (keys) => this.filterSelectionKey(keys),
          }}
          onChange={this.pageChange}
          onRowClick={this.listRowClick}
        />
        {/*
          {
          isRefundConfirmModal &&
          <RefundConfirmModal
            onConfirm={ () => this.handleButtonApi(PASS,filterSelectionArray)}
            onClose={ () => this.setState({isRefundConfirmModal: false})}
          />
          }
          * */}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  let data = state.main.pageParams;
  let pathName = state.routing.locationBeforeTransitions.pathname;
  return {
    params: data[pathName] || {},
    user: state.account.user,
  };
};

export default connect(mapStateToProps)(RefundDetail);
