import React from 'react';
import ReactDOM from 'react-dom';
import { Cascader as AntdCascader } from 'antd';
import PropTypes from 'prop-types';

export default class Cascader extends React.Component {
  static propTypes = {
    className: PropTypes.string, //附加在根元素上的类名
    dict: PropTypes.arrayOf(
      PropTypes.shape({
        val: PropTypes.string,
        text: PropTypes.string,
        disabled: PropTypes.bool,
        title: PropTypes.string,
        label: PropTypes.string,
        options: PropTypes.arrayOf(PropTypes.object),
      }),
    ), //选项字典，为选项或选项组集合，以是否含有label字段来区分选项和分组
    allowClear: PropTypes.bool, //是否允许清除
    style: PropTypes.object, //附加在根元素上的样式
    placeholder: PropTypes.string, //占位文字
    showSearch: PropTypes.bool, //是否可搜索
    size: PropTypes.oneOf(['large', 'small', 'default']), //大小
    onChange: PropTypes.func, //值变化时的回调
    expandTrigger: PropTypes.oneOf(['click', 'hover']), //次级菜单的展开方式，可选 'click' 和 'hover'
    defaultValue: [],
  };
  static defaultProps = {
    allowClear: true,
    showSearch: true,
    placeholder: '请输入内容搜索',
    dict: [],
    size: 'large',
    expandTrigger: 'click',
    onChange: () => {},
  };
  render() {
    const { dict, onChange, ...extraProps } = this.props;

    const currentDict = dict.map((item) => ({
      ...item,
      children: item.options.map((childrenItem) => ({ ...childrenItem })),
    }));
    return <AntdCascader options={currentDict} onChange={onChange} {...extraProps} />;
  }
}
