import React from 'react';
import moment from 'moment';
import SearchBar from 'COMPONENT/SearchBar';
import DatePicker from 'COMPONENT/DatePicker';
import Select from 'COMPONENT/Select';
import Input from 'COMPONENT/Input';
import { loadCommonDict, loadBranchDict, loadCampusDict } from 'UTIL/dict';
import { loadSourceDetailDict } from 'PAGE/common/loaders';
import SchoolModal from '../InputCustomerOnLine/SchoolModal';

const timeoutDict = [
  { val: '#5', text: '5分钟以内' },
  { val: '5#10', text: '5-10分钟' },
  { val: '10#20', text: '10-20分钟' },
  { val: '20#30', text: '20-30分钟' },
  { val: '30#120', text: '30-120分钟' },
  { val: '120#', text: '120分钟以上' },
];
const performanceTypeDict = [
  { val: 'true', text: '有效' },
  { val: 'false', text: '无效' },
];

class FollowupSearchBar extends React.Component {
  constructor(props) {
    super(props);

    this.defaultSearchConditions = {
      userName: '',
      startTime: '',
      endTime: '',
      customerName: '',
      customerContact: '',
      cusType: '',
      cusOrg: '',
      deliverTargetName: '',
      firstFollowStartTime: '',
      firstFollowEndTime: '',
      branchId: '',
      campusId: '',
      studentName: '',
      firstContractStartTime: '',
      firstContractEndTime: '',
      valid: '',
    };

    this.defaultDicts = {
      resTypeDict: [],
      cusOrgDict: [], //related
      brenchDict: [],
      blCampusDict: [], //related
      timeoutDict: timeoutDict, //static
      performanceTypeDict: performanceTypeDict, //static
    };

    this.state = Object.assign(
      {
        searchExpand: false, // {boolean} 搜索栏展开状态
        schoolModalVisible: false,
        timeout: '',
      },
      this.defaultSearchConditions,
      this.defaultDicts,
    );
  }

  onSearch = () => {
    const searchConditions = Object.keys(this.defaultSearchConditions).reduce((obj, ele) => {
      if (this.state[ele]) {
        obj[ele] = this.state[ele];
        if (ele.toLowerCase().indexOf('starttime') >= 0) {
          obj[ele] = moment(obj[ele]).startOf('day').format('YYYY-MM-DD HH:mm:ss');
        }
        if (ele.toLowerCase().indexOf('endtime') >= 0) {
          obj[ele] = moment(obj[ele]).endOf('day').format('YYYY-MM-DD HH:mm:ss');
        }
      }
      return obj;
    }, {});
    if (this.state.timeout) {
      const timeoutArr = this.state.timeout.split('#');
      searchConditions['minTimeout'] = timeoutArr[0];
      searchConditions['maxTimeout'] = timeoutArr[1];
    }
    console.log(searchConditions);
    this.props.onSearch(searchConditions);
  };

  onReset = () => {
    this.setState({
      ...this.defaultSearchConditions,
      timeout: '',
      cusOrgDict: [],
      blCampusDict: [],
    });
  };

  getDisabledStartDate = (endDate) => {
    return (beginDate) => {
      if (!beginDate || !endDate) {
        return false;
      }
      return moment(beginDate).valueOf() > moment(endDate).valueOf();
    };
  };

  getDisabledEndDate = (beginDate) => {
    return (endDate) => {
      if (!beginDate || !endDate) {
        return false;
      }
      return moment(endDate).valueOf() < moment(beginDate).valueOf();
    };
  };

  getCusOrgDict = (cusType) => {
    if (cusType) {
      loadSourceDetailDict(cusType).then((cusOrgDict) => this.setState({ cusOrgDict, cusOrg: '' }));
    } else {
      this.setState({ cusOrgDict: [], cusOrg: '' });
    }
  };

  getBlCampusDict = (branchId) => {
    if (branchId) {
      loadCampusDict(branchId).then((blCampusDict) => this.setState({ blCampusDict, campusId: '' }));
    } else {
      this.setState({ blCampusDict: [], campusId: '' });
    }
  };

  componentDidMount() {
    Promise.all([loadCommonDict('RES_TYPE'), loadBranchDict()]).then(([resTypeDict, brenchDict]) => {
      this.setState({
        resTypeDict,
        brenchDict,
      });
    });
  }

  render() {
    const {
      searchExpand,
      userName,
      startTime,
      endTime,
      customerName,
      customerContact,
      cusType,
      cusOrg,
      deliverTargetName,
      firstFollowStartTime,
      firstFollowEndTime,
      branchId,
      campusId,
      studentName,
      firstContractStartTime,
      firstContractEndTime,
      timeout,
      valid,
      resTypeDict,
      cusOrgDict,
      brenchDict,
      blCampusDict,
      timeoutDict,
      performanceTypeDict,
      schoolModalVisible,
    } = this.state;

    return (
      <div>
        <SearchBar
          expand={searchExpand}
          onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
          onSearch={this.onSearch}
          onReset={this.onReset}
        >
          <Input placeholder="录入人" value={userName} onChange={(e) => this.setState({ userName: e.target.value })} />
          <DatePicker
            placeholder="录入开始日期"
            value={startTime}
            onChange={(val) => this.setState({ startTime: val })}
            disabledDate={this.getDisabledStartDate(endTime)}
          />
          <DatePicker
            placeholder="录入结束日期"
            value={endTime}
            onChange={(val) => this.setState({ endTime: val })}
            disabledDate={this.getDisabledEndDate(startTime)}
          />
          <Input
            placeholder="客户姓名"
            value={customerName}
            onChange={(e) => this.setState({ customerName: e.target.value })}
          />
          <Input
            placeholder="联系方式"
            value={customerContact}
            onChange={(e) => this.setState({ customerContact: e.target.value })}
          />
          <Select
            placeholder="资源来源"
            dict={resTypeDict}
            value={cusType}
            onChange={(val) => {
              this.setState({ cusType: val });
              this.getCusOrgDict(val);
            }}
          />
          <Select
            placeholder="来源细分"
            dict={cusOrgDict}
            value={cusOrg}
            onChange={(val) => this.setState({ cusOrg: val })}
          />
          <Input
            placeholder="跟进人"
            value={deliverTargetName}
            onChange={(e) => this.setState({ deliverTargetName: e.target.value })}
          />
          <DatePicker
            placeholder="首次跟进记录开始日期"
            value={firstFollowStartTime}
            onChange={(val) => this.setState({ firstFollowStartTime: val })}
            disabledDate={this.getDisabledStartDate(firstFollowEndTime)}
          />
          <DatePicker
            placeholder="首次跟进记录结束日期"
            value={firstFollowEndTime}
            onChange={(val) => this.setState({ firstFollowEndTime: val })}
            disabledDate={this.getDisabledEndDate(firstFollowStartTime)}
          />
          <Select
            placeholder="分公司"
            showSearch
            dict={brenchDict}
            value={branchId}
            onChange={(val) => {
              this.setState({ branchId: val });
              this.getBlCampusDict(val);
            }}
          />
          <Select
            placeholder="学习中心"
            showSearch
            dict={blCampusDict}
            value={campusId}
            onChange={(val) => this.setState({ campusId: val })}
          />
          <Input
            placeholder="学生姓名"
            value={studentName}
            onChange={(e) => this.setState({ studentName: e.target.value })}
          />
          <DatePicker
            placeholder="首次签单开始日期"
            value={firstContractStartTime}
            onChange={(val) => this.setState({ firstContractStartTime: val })}
            disabledDate={this.getDisabledStartDate(firstContractEndTime)}
          />
          <DatePicker
            placeholder="首次签单结束日期"
            value={firstContractEndTime}
            onChange={(val) => this.setState({ firstContractEndTime: val })}
            disabledDate={this.getDisabledEndDate(firstContractStartTime)}
          />
          <Select
            placeholder="业绩类型"
            dict={performanceTypeDict}
            value={valid}
            onChange={(val) => this.setState({ valid: val })}
          />
          <Select
            placeholder="超时时间"
            dict={timeoutDict}
            value={timeout}
            onChange={(val) => this.setState({ timeout: val })}
          />
        </SearchBar>
        {schoolModalVisible && (
          <SchoolModal onCommit={this.schoolModalCommit} onClose={() => this.setState({ schoolModalVisible: false })} />
        )}
      </div>
    );
  }
}

export default FollowupSearchBar;
