import React from 'react';
import Modal from 'COMPONENT/Modal';
import Block from 'COMPONENT/Block';
import Icon from 'COMPONENT/Icon';
import Badage from 'COMPONENT/Badage';
import styleVars from 'COMPONENT/theme/variables';
import { Row, Col, Tree, Form } from 'antd';
import loaders from './loaders';

const TreeNode = Tree.TreeNode;
const FormItem = Form.Item;
const studentForm = {
  name: '学员姓名',
  contact: '联系方式',
  sex: '性别',
  bothday: '出生日期',
  schoolName: '就读学校',
  gradeName: '所在年级',
  fatherName: '父亲姓名',
  fatherPhone: '联系方式',
  motherName: '母亲姓名',
  notherPhone: '联系方式',
  address: '学员位置',
};
const formItemCol = {
  labelCol: { span: 9 },
  wrapperCol: { span: 15 },
};
const sexCnNameObj = {
  0: '女',
  1: '男',
};

class ResourceInfoModal extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: true,
      customerDynamicStatusList: [],
      customer: {},
      students: [],
    };
  }

  componentDidMount() {
    this.setState({ loading: true });
    Promise.all([
      loaders.customerDynamicStatus({
        customerId: this.props.customerId,
        studentId: this.props.studentId,
      }),
      loaders.findCustomerInfoForNetWork({ id: this.props.id }),
      loaders.checkPilotCampusCustomer({ contact: this.props.contact }),
    ]).then(([customerDynamicStatus, customer, isPilot]) => {
      console.log(`is pilot?: ${isPilot}`);
      this.setState({
        loading: false,
        customer: customer,
        customerDynamicStatusList:
          '' + isPilot !== 'true'
            ? customerDynamicStatus
            : customerDynamicStatus.filter((ele) => ele.studentId === this.props.studentId || ele.studentId === null),
        students:
          '' + isPilot !== 'true'
            ? customer.studentVos
            : customer.studentVos.filter((ele) => ele.id === this.props.studentId),
      });
    });
  }

  render() {
    const { onClose } = this.props;
    const { loading, customerDynamicStatusList, customer, students } = this.state;
    return (
      <Modal
        loading={loading}
        title="客户资源信息"
        onCancel={onClose}
        cancelText={'关闭'}
        okBtnShow={false}
        width={1068}
      >
        <Row gutter={10}>
          <Col span={15}>
            <div
              style={{
                maxHeight: '500px',
                overflowY: 'scroll',
              }}
            >
              <Block title="基本信息">
                <Row type="flex" justify="start">
                  <Col span={12}>
                    <FormItem label="客户姓名" required={true} {...formItemCol}>
                      <span>{customer.name}</span>
                    </FormItem>
                  </Col>
                  <Col span={12}>
                    <FormItem label="联系方式" required={true} {...formItemCol}>
                      <span>{customer.contact}</span>
                    </FormItem>
                  </Col>
                </Row>
                <Row type="flex" justify="start">
                  <Col span={12}>
                    <FormItem label="资源入口" {...formItemCol}>
                      <span>{customer.resEntranceName}</span>
                    </FormItem>
                  </Col>
                  <Col span={12}>
                    <FormItem label="资源来源" {...formItemCol}>
                      <span>{customer.cusTypeName}</span>
                    </FormItem>
                  </Col>
                </Row>
                <Row type="flex" justify="start">
                  <Col span={12}>
                    <FormItem label="来源细分" {...formItemCol}>
                      <span>{customer.cusOrgName}</span>
                    </FormItem>
                  </Col>
                  <Col span={12}>
                    <FormItem label="分配人" {...formItemCol}>
                      <span>{+customer.isImport ? '' : customer.lastDeliverName}</span>
                    </FormItem>
                  </Col>
                </Row>
                <Row type="flex" justify="start">
                  <Col span={12}>
                    <FormItem label="跟进人" {...formItemCol}>
                      <span>{customer.deliverTargetName}</span>
                    </FormItem>
                  </Col>
                  <Col span={12}>
                    <FormItem label="跟进状态" {...formItemCol}>
                      <span>{customer.dealStatusName}</span>
                    </FormItem>
                  </Col>
                </Row>
                <Row type="flex" justify="start">
                  <Col span={12}>
                    <FormItem label="所属学习中心" {...formItemCol}>
                      <span>{customer.blCampusName}</span>
                    </FormItem>
                  </Col>
                  <Col span={12}>
                    <FormItem label="所属分公司" {...formItemCol}>
                      <span>{customer.brenchName}</span>
                    </FormItem>
                  </Col>
                </Row>
              </Block>
              <Block title="学员档案" type="dark">
                {students.length <= 0 ? (
                  <div style={{ textAlign: 'center' }}>{'暂无学员记录'}</div>
                ) : (
                  students.map((student, i) => {
                    return (
                      <div key={'student-' + i}>
                        {i !== 0 ? <hr /> : ''}
                        <Row type="flex" justify="start">
                          {Object.keys(studentForm).map((ele) => {
                            return (
                              <Col key={ele} span={12}>
                                <FormItem label={studentForm[ele]} {...formItemCol}>
                                  <span>{ele === 'sex' ? sexCnNameObj['' + student[ele]] : student[ele]}</span>
                                </FormItem>
                              </Col>
                            );
                          })}
                        </Row>
                      </div>
                    );
                  })
                )}
              </Block>
            </div>
          </Col>
          <Col span={9}>
            <div
              style={{
                height: '500px',
                maxHeight: '500px',
                overflowY: 'scroll',
                overflowX: 'hidden',
              }}
            >
              <Tree showLine defaultExpandAll>
                <TreeNode title="客户动态" key="0-0">
                  {customerDynamicStatusList.map((ele, i) => {
                    return (
                      <TreeNode
                        title={
                          <span
                            style={{
                              maxWidth: '250px',
                              wordBreak: 'break-word',
                              whiteSpace: 'pre-wrap',
                              display: 'inline-block',
                            }}
                          >
                            <Badage
                              style={{
                                marginLeft: 20,
                                background: styleVars.successColor,
                              }}
                            >
                              <Icon type="time_mc" />
                              {ele.occourTime}
                            </Badage>
                            {' ' + ele.description}
                          </span>
                        }
                        key={'0-0-' + i}
                      />
                    );
                  })}
                </TreeNode>
              </Tree>
            </div>
          </Col>
        </Row>
      </Modal>
    );
  }
}
export default ResourceInfoModal;
