import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import Block from 'COMPONENT/Block';
import List from 'COMPONENT/List';
import CustomerSearchBar from './CustomerSearchBar';
import ResourceInfoModal from './ResourceInfoModal';
import loaders from './loaders';
import moment from 'moment';
import jslog from 'SRC/jslog';

const { COL_WIDTH_L } = List;

class CustomerListOnline extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: true,
      listLoading: true,
      dataSource: [],
      total: 0,
      current: 1,
      pageSize: 20,
      searchConditions: {},
      resourceInfoModalVisible: false,
      resourceInfoModalData: {},
    };
  }

  onSearch = async (val) => {
    const page = 1;
    this.setState({ listLoading: true });
    const res = await loaders.getNetWorkCustomerList({
      ...val,
      pageSize: this.state.pageSize,
      pageNo: page,
    });

    jslog.report('QUERY', {
      function_name: '网络工作台>网络客户列表>查询',
      page: page,
      total: res.rowCount,
      size: res.datas.length,
    });
    this.setState({
      searchConditions: val,
      current: page,
      total: res.rowCount,
      dataSource: res.datas,
      listLoading: false,
    });
    return;
  };

  onChange = async ({ current = this.state.current, pageSize = this.state.pageSize }) => {
    this.setState({ listLoading: true });
    const res = await loaders.getNetWorkCustomerList({
      ...this.state.searchConditions,
      pageSize: pageSize,
      pageNo: current,
    });
    this.setState({
      current: res.pageNo,
      pageSize: res.pageSize,
      total: res.rowCount,
      dataSource: res.datas,
      listLoading: false,
    });
    return;
  };

  onModalCommit = () => this.onChange({});

  dateRender = (val) => {
    if (!val) {
      return '';
    }
    const arr = val.split(' ');
    return (
      <div>
        <div>{arr[0]}</div>
        <div>{arr[1]}</div>
      </div>
    );
  };

  componentDidMount() {
    Promise.all([this.onSearch()]).then(() => {
      this.setState({ loading: false });
    });
  }

  render() {
    const columns = [
      { title: '客户姓名', dataIndex: 'customerName', fixed: 'left' },
      { title: '学生姓名', dataIndex: 'studentName', fixed: 'left' },
      { title: '联系方式', dataIndex: 'customerContact', fixed: 'left' },
      {
        title: '录入人',
        dataIndex: 'userName',
        render: (val, row) => {
          if (+row.isImport) {
            return '';
          }
          return val;
        },
      },
      { title: '录入部门', dataIndex: 'organizationName' },
      { title: '资源入口', dataIndex: 'resEntranceName' },
      { title: '资源来源', dataIndex: 'cusTypeName' },
      { title: '资源细分', dataIndex: 'cusOrgName' },
      { title: '录入时间', dataIndex: 'createTime', render: this.dateRender },
      {
        title: '分配时间',
        dataIndex: 'distributionTime',
        render: this.dateRender,
      },
      {
        title: '学习中心获取时间',
        dataIndex: 'obtainTime',
        render: this.dateRender,
      },
      {
        title: '超时时间',
        render: (val, row, i) => {
          const time = diffTime(row['obtainTime'], row['distributionTime']);
          return <span>{time}</span>;
        },
      },
      { title: '跟进人', dataIndex: 'obtainUserName' },
      {
        title: '业绩类型',
        dataIndex: 'valid',
        render: (val) => (val ? '有效' : '无效'),
      },
      { title: '客户状态', dataIndex: 'customerDealStatusName' },
      { title: '所属学习中心', dataIndex: 'campusName' },
      { title: '所属分公司', dataIndex: 'branchName' },
      {
        title: '首次签单时间',
        dataIndex: 'firstContractTime',
        render: this.dateRender,
      },
      { title: '首次签单金额', dataIndex: 'contractTotalAmount' },
      {
        title: '首次跟进时间',
        dataIndex: 'firstFollowTime',
        render: this.dateRender,
      },
      {
        title: '操作',
        key: 'action',
        fixed: 'right',
        getActionConfig: (val, row, i) => {
          return [
            {
              text: '详情',
              onClick: () => {
                jslog.report('click', {
                  function_name: '网络工作台>网络客户列表>详情',
                });

                this.setState({
                  resourceInfoModalVisible: true,
                  resourceInfoModalData: row,
                });
              },
            },
          ];
        },
      },
    ];

    const {
      loading,
      dataSource,
      total,
      current,
      pageSize,
      listLoading,
      resourceInfoModalVisible,
      resourceInfoModalData,
    } = this.state;

    return (
      <Page
        loading={loading}
        route={this.props.route}
        fitWindowHeight
        subHeader={<CustomerSearchBar onSearch={this.onSearch} />}
      >
        <List
          dataSource={dataSource}
          columns={columns}
          pagination={{
            current,
            total,
            pageSize,
          }}
          loading={listLoading}
          onChange={this.onChange}
        />
        {resourceInfoModalVisible && (
          <ResourceInfoModal
            id={resourceInfoModalData.id}
            customerId={resourceInfoModalData.customerId}
            studentId={resourceInfoModalData.studentId}
            contact={resourceInfoModalData.customerContact}
            onClose={() => this.setState({ resourceInfoModalVisible: false })}
          />
        )}
      </Page>
    );
  }
}

function diffTime(date1, date2) {
  if (!date1 || !date2) {
    return '';
  }

  let diff = Math.floor((new Date(date1) - new Date(date2)) / 1000);

  const schedule = [
    { unitName: '秒', unit: 60 },
    { unitName: '分钟', unit: 60 },
    { unitName: '小时', unit: 24 },
    { unitName: '天', unit: Infinity },
  ];
  return schedule.reduce((acc, ele) => {
    const calc = diff % ele.unit;
    diff = Math.floor(diff / ele.unit);
    return (acc = (calc ? `${calc}${ele.unitName}` : '') + acc);
  }, '');
}

export default connect()(CustomerListOnline);
