import React from 'react';
import Modal from 'COMPONENT/Modal';
import FormItem from 'COMPONENT/FormItem';
import Select from 'COMPONENT/Select';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import loaders from './loaders';
import { Row, Col } from 'antd';

class DistrbuteResourceModal extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: true,
      userId: '',
      userDict: [],
    };
  }

  onCommit = async () => {
    try {
      const {
        onClose,
        onCommit,
        data: { cusId, studentId, deliverTargetId },
      } = this.props;

      if (!this.state.userId) {
        this._modal.showOkTips(`请选择分配对象`);
        return;
      }

      this.setState({ loading: true });
      const params = {
        cusId,
        deliverTarget: this.state.userId.substring(0, this.state.userId.indexOf('-')),
        deliverTargetOld: deliverTargetId,
        studentId,
      };
      const res = await loaders.distributeCustomerForNetWork(params);
      if (res.resultCode == 0) {
        onCommit();
        onClose();
        showMessage('操作成功', 'success');
      } else {
        showMessage('操作失败：' + res.resultMessage, 'error');
      }
    } catch (e) {
      console.log(e);
    }
    this.setState({ loading: false });
  };

  componentDidMount() {
    this.setState({ loading: true });

    const {
      data: { cusId: customerId, deliverTargetId: deliverTarget, studentId },
    } = this.props;
    loaders
      .userForNetWorkDict({
        'job[]': ['XQYYFZR'],
        customerId,
        deliverTarget,
        studentId,
      })
      .then((res) => {
        const userDict = res.map((group) => {
          group.options = group.options.map((option) => {
            option.val = option.val + '-' + option.group + '-' + option.text;
            return option;
          });
          return group;
        });
        this.setState({ userDict, loading: false });
      });
  }

  render() {
    const { onClose } = this.props;
    const { userId, userDict, loading } = this.state;

    return (
      <Modal
        loading={loading}
        title="资源分配"
        onCancel={onClose}
        okText="保存"
        onOk={this.onCommit}
        ref={(c) => (this._modal = getWrappedInstance(c))}
        width={650}
      >
        <Row>
          <Col span={24}>
            <FormItem name="分配对象" required="true">
              <Select
                placeholder="请选择分配对象"
                dict={userDict}
                value={userId}
                showSearch
                onChange={(val) => this.setState({ userId: val })}
                filter={(searchKey, option) => {
                  return option.val.indexOf(searchKey) >= 0;
                }}
              />
            </FormItem>
          </Col>
        </Row>
      </Modal>
    );
  }
}

export default DistrbuteResourceModal;
