import React from 'react';
import moment from 'moment';
import SearchBar from 'COMPONENT/SearchBar';
import Select from 'COMPONENT/Select';
import Input from 'COMPONENT/Input';
import { loadCommonDict, loadAllBranchCampusDict } from 'UTIL/dict';
import { loadSourceDetailDict } from 'PAGE/common/loaders';

class FollowupSearchBar extends React.Component {
  constructor(props) {
    super(props);

    this.defaultSearchConditions = {
      cusName: '',
      contact: '',
      resEntranceId: '',
      cusType: '',
      cusOrg: '',
      intentionCampusId: '',
      studentName: '',
    };

    this.defaultDicts = {
      resEntranceDict: [],
      campusDict: [],
      resTypeDict: [],
      cusOrgDict: [],
    };

    this.state = Object.assign(
      {
        searchExpand: false, // {boolean} 搜索栏展开状态
      },
      this.defaultSearchConditions,
      this.defaultDicts,
    );
  }

  onSearch = () => {
    const searchConditions = Object.keys(this.defaultSearchConditions).reduce((obj, ele) => {
      if (this.state[ele]) {
        obj[ele] = this.state[ele];
      }
      return obj;
    }, {});
    this.props.onSearch(searchConditions);
  };

  onReset = () => {
    this.setState({ ...this.defaultSearchConditions, cusOrgDict: [] });
  };

  getCusOrgDict = (resEntranceId) => {
    if (resEntranceId) {
      loadSourceDetailDict(resEntranceId).then((cusOrgDict) => this.setState({ cusOrgDict, cusOrg: '' }));
    } else {
      this.setState({ cusOrgDict: [], cusOrg: '' });
    }
  };

  componentDidMount() {
    Promise.all([loadCommonDict('RES_ENTRANCE'), loadCommonDict('RES_TYPE'), loadAllBranchCampusDict()]).then(
      ([resEntranceDict, resTypeDict, campusDict]) => {
        this.setState({
          resEntranceDict,
          resTypeDict,
          campusDict,
        });
      },
    );
  }

  render() {
    const {
      searchExpand,
      cusName,
      contact,
      resEntranceId,
      cusType,
      cusOrg,
      intentionCampusId,
      studentName,
      resEntranceDict,
      campusDict,
      resTypeDict,
      cusOrgDict,
    } = this.state;

    return (
      <div>
        <SearchBar
          expand={searchExpand}
          onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
          onSearch={this.onSearch}
          onReset={this.onReset}
        >
          <Input placeholder="客户姓名" value={cusName} onChange={(e) => this.setState({ cusName: e.target.value })} />
          <Input placeholder="联系方式" value={contact} onChange={(e) => this.setState({ contact: e.target.value })} />
          <Select
            placeholder="资源入口"
            dict={resEntranceDict}
            value={resEntranceId}
            onChange={(val) => this.setState({ resEntranceId: val })}
          />
          <Select
            placeholder="资源来源"
            dict={resTypeDict}
            value={cusType}
            onChange={(val) => {
              this.setState({ cusType: val });
              this.getCusOrgDict(val);
            }}
          />
          <Select
            placeholder="来源细分"
            dict={cusOrgDict}
            value={cusOrg}
            onChange={(val) => this.setState({ cusOrg: val })}
          />
          <Select
            placeholder="意向学习中心"
            showSearch
            dict={campusDict}
            value={intentionCampusId}
            onChange={(val) => this.setState({ intentionCampusId: val })}
          />
          <Input
            placeholder="学生姓名"
            value={studentName}
            onChange={(e) => this.setState({ studentName: e.target.value })}
          />
        </SearchBar>
      </div>
    );
  }
}

export default FollowupSearchBar;
