import React from 'react';
import Block from 'COMPONENT/Block';
import Select from 'COMPONENT/Select';
import { Row, Col, DatePicker } from 'antd';
import moment from 'moment';

const { MonthPicker, RangePicker } = DatePicker;
const dateFormat = 'YYYY/MM/DD';
const monthFormat = 'YYYY/MM';
const typeDict = [
  { val: 'range', text: '天' },
  { val: 'month', text: '月' },
];
const overviewObj = {
  news: '新增',
  stayfollow: '待跟进',
  following: '跟进中',
  visitcome: '已上门',
  sign: '已签单',
  // release: "释放",
  // invalid: "无效",
};

class OverviewTile extends React.Component {
  constructor(props) {
    super(props);

    this.date = new Date();
    this.state = {
      type: 'range',
    };
  }

  onSelectChange = (val) => {
    this.setState({ type: val });
    const date = moment(this.date);
    let params = {
      startDate: date.format('YYYY-MM-DD'),
      endDate: date.format('YYYY-MM-DD'),
    };
    if (val === 'month') {
      params = {
        startDate: date.startOf('month').format('YYYY-MM-DD'),
        endDate: date.endOf('month').format('YYYY-MM-DD'),
      };
    }
    this.props.onchange(params);
  };

  onRangeChange = (date) => {
    if (date.length) {
      const params = {
        startDate: date[0].format('YYYY-MM-DD'),
        endDate: date[1].format('YYYY-MM-DD'),
      };
      this.props.onchange(params);
    }
  };

  onMonthChange = (date) => {
    if (date) {
      const params = {
        startDate: date.startOf('month').format('YYYY-MM-DD'),
        endDate: date.endOf('month').format('YYYY-MM-DD'),
      };
      this.props.onchange(params);
    }
  };

  render() {
    const { overview } = this.props;
    const { type } = this.state;
    return (
      <Block title="客户跟进概览">
        <Row gutter={16}>
          {Object.keys(overviewObj).map((ele) => {
            return (
              <Col key={ele} span={2} style={{ textAlign: 'center' }}>
                <div>{overviewObj[ele]}</div>
                <div>{!Number.isInteger(overview[ele]) ? '-' : overview[ele]}</div>
              </Col>
            );
          })}
          <Col offset={8} span={2}>
            <Select allowClear={false} dict={typeDict} value={type} onChange={this.onSelectChange} />
          </Col>
          <Col span={4}>
            {type !== 'range' ? (
              ''
            ) : (
              <RangePicker
                size="large"
                defaultValue={[moment(this.date, dateFormat), moment(this.date, dateFormat)]}
                format={dateFormat}
                onChange={this.onRangeChange}
              />
            )}
            {type !== 'month' ? (
              ''
            ) : (
              <MonthPicker
                size="large"
                defaultValue={moment(this.date, monthFormat)}
                format={monthFormat}
                onChange={this.onMonthChange}
              />
            )}
          </Col>
        </Row>
      </Block>
    );
  }
}
export default OverviewTile;
