import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import Block from 'COMPONENT/Block';
import List from 'COMPONENT/List';
import FollowupSearchBar from './FollowupSearchBar';
import AppointmentModal from './AppointmentModal';
import ResourceInfoModal from './ResourceInfoModal';
import DistrbuteResourceModal from './DistrbuteResourceModal';
import loaders from './loaders';
import OverviewTile from './OverviewTile';
import moment from 'moment';
import jslog from 'SRC/jslog';

const { COL_WIDTH_L } = List;

class FollowupCustomersOnLine extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: true,
      listLoading: true,
      appointmentModalVisible: false,
      appointmentModalData: {},
      resourceInfoModalVisible: false,
      resourceInfoModalData: {},
      distrbuteResourceModalVisible: false,
      distrbuteResourceModalData: {},
      dataSource: [],
      total: 0,
      current: 1,
      pageSize: 20,
      searchConditions: {},
      overview: {},
    };

    this.date = moment(new Date());
  }

  onOverviewChange = async (params) => {
    const defaultDate = {
      startDate: this.date.format('YYYY-MM-DD'),
      endDate: this.date.format('YYYY-MM-DD'),
    };
    const res = await loaders.customerOverviewForNetWork(Object.assign({}, defaultDate, params));
    this.setState({ overview: res });
  };

  onSearch = async (val) => {
    const page = 1;
    this.setState({ listLoading: true });
    const res = await loaders.getNetWorkFollowupCustomers({
      ...val,
      pageSize: this.state.pageSize,
      pageNo: page,
    });

    jslog.report('QUERY', {
      function_name: '网络工作台>跟进客户管理（新）>查询',
      page: page,
      total: res.rowCount,
      size: res.datas.length,
    });

    this.setState({
      searchConditions: val,
      current: page,
      total: res.rowCount,
      dataSource: res.datas,
      listLoading: false,
    });
    return;
  };

  onChange = async ({ current = this.state.current, pageSize = this.state.pageSize }) => {
    this.setState({ listLoading: true });
    const res = await loaders.getNetWorkFollowupCustomers({
      ...this.state.searchConditions,
      pageSize: pageSize,
      pageNo: current,
    });
    this.setState({
      current: res.pageNo,
      pageSize: res.pageSize,
      total: res.rowCount,
      dataSource: res.datas,
      listLoading: false,
    });
    return;
  };

  onModalCommit = () => this.onChange({});

  dateRender = (val) => {
    if (!val) {
      return '';
    }
    const arr = val.split(' ');
    return (
      <div>
        <div>{arr[0]}</div>
        <div>{arr[1]}</div>
      </div>
    );
  };

  componentDidMount() {
    Promise.all([this.onSearch()]).then(() => this.setState({ loading: false }));
  }

  render() {
    const curDate = new Date();
    const columns = [
      { title: '客户姓名', dataIndex: 'cusName', fixed: 'left' },
      { title: '学生姓名', dataIndex: 'studentName', fixed: 'left' },
      { title: '联系方式', dataIndex: 'contact', fixed: 'left' },
      { title: '资源入口', dataIndex: 'resEntranceName' },
      { title: '资源来源', dataIndex: 'cusType' },
      { title: '资源细分', dataIndex: 'cusOrg' },
      {
        title: '分配人',
        dataIndex: 'lastDeliverName',
        render: (val, row) => {
          if (+row.isImport) {
            return '';
          }
          return val;
        },
      },
      { title: '意向学习中心', dataIndex: 'intentionCampusName' },
      {
        title: '操作',
        key: 'action',
        width: COL_WIDTH_L,
        fixed: 'right',
        getActionConfig: (val, row, i) => {
          return [
            {
              text: '详情',
              onClick: () => {
                jslog.report('click', {
                  function_name: '网络工作台>跟进客户管理（新）>详情',
                });

                this.setState({
                  resourceInfoModalVisible: true,
                  resourceInfoModalData: row,
                });
              },
            },
            {
              text: '资源分配',
              onClick: () =>
                this.setState({
                  distrbuteResourceModalVisible: true,
                  distrbuteResourceModalData: row,
                }),
            },
          ];
        },
      },
    ];

    const {
      loading,
      dataSource,
      total,
      current,
      pageSize,
      listLoading,
      appointmentModalVisible,
      appointmentModalData,
      resourceInfoModalVisible,
      resourceInfoModalData,
      distrbuteResourceModalVisible,
      distrbuteResourceModalData,
      overview,
    } = this.state;

    return (
      <Page
        loading={loading}
        route={this.props.route}
        fitWindowHeight
        subHeader={<FollowupSearchBar onSearch={this.onSearch} />}
      >
        {/* <OverviewTile overview={overview} onchange={this.onOverviewChange} /> */}
        <List
          dataSource={dataSource}
          columns={columns}
          pagination={{
            current,
            total,
            pageSize,
          }}
          loading={listLoading}
          onChange={this.onChange}
        />
        {appointmentModalVisible && (
          <AppointmentModal
            data={appointmentModalData}
            onCommit={this.onModalCommit}
            onClose={() => this.setState({ appointmentModalVisible: false })}
          />
        )}
        {resourceInfoModalVisible && (
          <ResourceInfoModal
            customerId={resourceInfoModalData.cusId}
            studentId={resourceInfoModalData.studentId}
            target={resourceInfoModalData.deliverTargetId}
            contact={resourceInfoModalData.contact}
            onClose={() => this.setState({ resourceInfoModalVisible: false })}
          />
        )}
        {distrbuteResourceModalVisible && (
          <DistrbuteResourceModal
            data={distrbuteResourceModalData}
            onCommit={this.onModalCommit}
            onClose={() => this.setState({ distrbuteResourceModalVisible: false })}
          />
        )}
      </Page>
    );
  }
}

function diffDays(date1, date2) {
  const days = Math.round((date1 - date2) / (24 * 3600 * 1000));
  return `${days}`;
}

export default connect()(FollowupCustomersOnLine);
