import React from 'react';
import Modal from 'COMPONENT/Modal';
import FormItem from 'COMPONENT/FormItem';
import Select from 'COMPONENT/Select';
import TextArea from 'COMPONENT/TextArea';
import loaders from './loaders';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

class FollowTargetModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: true,
      students: [],
      followStuId: '',
      remark: '',
    };
  }

  onCommit = async () => {
    const { customerId, onClose } = this.props;
    const { remark, followStuId } = this.state;
    if (!remark || !followStuId) {
      return showMessage('保存失败：请补充填入项', 'error');
    }
    this.setState({ loading: true });
    const res = await loaders.saveFollowRecord({
      followStuId,
      remark,
      appointmentType: 'FOLLOW_UP',
      customerId,
    });
    if (res.resultCode != 0) {
      return showMessage('保存失败：' + res.resultMessage, 'error');
    }
    showMessage('保存成功', 'success');
    onClose();
  };

  componentDidMount() {
    this.setState({ loading: true });
    const { students, customerId } = this.props;
    if (students) {
      this.setState({
        loading: false,
        students: students,
      });
    } else {
      Promise.all([loaders.relatedStudents({ customerId })]).then(([relatedStudents]) => {
        this.setState({
          loading: false,
          students: relatedStudents.result.map((ele) => {
            return { val: ele.studentId, text: ele.studentName };
          }),
        });
      });
    }
  }

  render() {
    const { onClose, children } = this.props;
    const { loading, students, remark, followStuId } = this.state;
    return (
      <Modal loading={loading} title="添加跟进记录" onCancel={onClose} okText="保存" onOk={this.onCommit}>
        {children ? <div style={{ height: '60px', textAlign: 'center' }}>{children}</div> : ''}
        <FormItem name="咨询学生">
          <Select
            placeholder="请选择咨询学生"
            dict={students}
            value={followStuId}
            onChange={(val) => this.setState({ followStuId: val })}
          />
        </FormItem>
        <FormItem name="咨询内容">
          <TextArea maxLength={120} value={remark} onChange={(e) => this.setState({ remark: e.target.value })} />
        </FormItem>
      </Modal>
    );
  }
}

export default FollowTargetModal;
