import React from 'react';
import { connect } from 'dva';
import moment from 'moment';
import Page from 'COMPONENT/Page';
import Block from 'COMPONENT/Block';
import Button from 'COMPONENT/Button';
import TextArea from 'COMPONENT/TextArea';
import DatePicker from 'COMPONENT/DatePicker';
import Select from 'COMPONENT/Select';
import Section from 'COMPONENT/Section';
import TextLink from 'COMPONENT/TextLink';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { Form, Input, Checkbox, Row, Col, Tooltip } from 'antd';
import { loadCommonDict, loadCommonDictNew, loadAllBranchCampusDict } from 'UTIL/dict';
import { loadSourceDetailDict } from 'PAGE/common/loaders';
import loaders from './loaders';
import SchoolModal from './SchoolModal';
import FollowTargetModal from './FollowTargetModal';
import fetch from 'dva/fetch';
import { urlBase } from 'CONST/config';
import Icon from 'COMPONENT/Icon';
import styleVars from 'COMPONENT/theme/variables';
import styles from './styles.less';
import { loginUser, isIdCard } from 'UTIL/common';
import { addCusFormCfg } from '../../common/AddCustomerMain/utils';
const CERTIFICATE_TYPE_ID_CARD = 'CT_IDENTITY_CARD';

const FormItem = Form.Item;
const CheckboxGroup = Checkbox.Group;
const sexDict = [
  { val: '1', text: '男' },
  { val: '0', text: '女' },
];
const formItemCol = {
  labelCol: { span: 8 },
  wrapperCol: { span: 15 },
};
const linkFormCfg = {
  labelCol: { span: 5 },
  wrapperCol: { span: 12 },
  style: {
    padding: '0 10px',
    marginBottom: 10,
    lineHeight: '24px',
  },
};
const phoneRegExp = /^1\d{10}$/;

const resTypeDict = [{ text: '网络推广', val: 'INTERNET' }];

class InputCustomerOnLine extends React.Component {
  constructor(props) {
    super(props);

    this.dicts = {
      campusDict: [],
      studentGradeDict: [],
      cusOrgDict: [],
      certificateTypeDict: [],
    };

    this.state = Object.assign(this.dicts, {
      loading: true,
      disabled: false,
      organization: {},
      studentList: [],
      studentListSelectable: [],
      checkAll: false,
      newStudents: [0],
      oldStudents: [],
      schoolModalVisible: false,
      schoolModalData: {},
      customerId: '',
      oldStudentLoading: false,
      followTarget: '',
      followTargetVisible: false,
      followedStudents: '',
      idCardType: '',
    });

    this.schoolInput = [];
  }

  handleSubmit = (e) => {
    e.preventDefault();
    this.props.form.validateFieldsAndScroll((err, values) => {
      if (err) {
        return showMessage('保存失败：请确认必填输入项是否填写完毕', 'error');
      }
      let params = Object.assign({}, values, {
        workBenchType: 'NETWORK',
      });
      params = Object.keys(params).reduce((obj, ele) => {
        obj[ele] = typeof params[ele] === 'string' ? params[ele].trim() : params[ele];
        return obj;
      }, {});

      // if (params.idCardNo && !isIdCard(params.idCardNo)) {
      //   return showMessage('保存失败：客户身份证格式错误', 'error');
      // }
      const { idCardType, idCardNo } = values;
      const shouldReturn = (idCardType && !idCardNo) || (!idCardType && idCardNo);
      if (shouldReturn) {
        const msg = !idCardType ? '请选择证件类型！' : '请填写证件号码！';
        return showMessage(msg);
      }

      // 校验身份证号码
      if (idCardType) {
        if (idCardType === CERTIFICATE_TYPE_ID_CARD && !isIdCard(idCardNo)) {
          return showMessage('身份证格式错误');
        } else if (!idCardNo.trim()) {
          return showMessage('证件号码错误');
        }
      }

      let studentImportVos = params['studentImportVos'] ? params['studentImportVos'] : [];
      studentImportVos = studentImportVos.reduce((arr, ele, i) => {
        let tmp = Object.assign({}, ele);
        tmp = Object.keys(tmp).reduce((obj, ele) => {
          obj[ele] = typeof tmp[ele] === 'string' ? tmp[ele].trim() : tmp[ele];
          return obj;
        }, {});

        if (i === 0 && this.state.oldStudents.length === 0 && !tmp.name) {
          tmp.name = params.name;
          tmp.contact = params.contact;
        }
        if (tmp.name) {
          arr.push(tmp);
        }
        return arr;
      }, []);

      studentImportVos = [
        ...studentImportVos,
        ...this.state.oldStudents.map((ele) => {
          return {
            gradeId: ele.gradeId || '',
            name: ele.studentName.trim(),
            schoolId: ele.schoolId || '',
            id: ele.studentId,
          };
        }),
      ];
      if (studentImportVos.length <= 0) {
        return showMessage('保存失败：请确认学员档案是否填写完毕', 'error');
      }
      const repeatName = studentImportVos
        .map((ele) => ele.name)
        .filter((ele, index, arr) => arr.indexOf(ele) !== index);
      if (repeatName.length) {
        return showMessage(`保存失败：选中学员姓名重复（${repeatName.join('、')}）`, 'error');
      }

      if (studentImportVos.length) {
        let tempIndex = 0;
        for (let value of studentImportVos) {
          ++tempIndex;
          if (value.idCardNo && !isIdCard(value.idCardNo)) {
            break;
          }
        }
        let tempStatus = studentImportVos.every((value) => !value.idCardNo || isIdCard(value.idCardNo));
        if (!tempStatus) {
          return showMessage(`保存失败：学员${tempIndex}身份证格式错误`);
        }
      }

      this.saveCustomer({ ...params, studentImportVos: studentImportVos });
    });
  };

  saveCustomer = async (params) => {
    try {
      const res = await loaders.checkPilotCampus({
        id: params.intentionCampusId,
      });
      this.setState({ loading: true });
      if ('' + res !== 'true') {
        await this.nonPilot(params);
      } else {
        await this.isPilot(params);
      }
    } catch (e) {
      console.log(e);
    }
    this.setState({ loading: false });
  };

  nonPilot = async (params) => {
    if (params.studentImportVos.length <= 0) {
      return showMessage('保存失败：请确认是否已勾选学员或填写学员档案', 'error');
    }

    const response = await fetch(
      `${urlBase}CustomerAction/checkCustomerContact.do?contact=${params.contact}&workbrenchType=ON_LINE`,
      { method: 'get', credentials: 'same-origin' },
    );
    const data = await response.json();
    let disabled = false;

    if (data.resultCode == 0) {
      showMessage('校验通过', 'success');
    } else if (data.resultCode == 1 - 100) {
      showMessage('校验通过:该号码的客户存在资源池，本次提交将覆盖该客户的信息', 'success');
    } else if (data.resultCode == 2 - 100) {
      showMessage('校验通过:该号码的客户无效，本次提交将覆盖该无效客户的信息', 'success');
    } else if (data.resultCode == 10 - 100) {
      showMessage('校验通过:关联学生状态已经超过结课/停课天数限制,本次提交将覆盖该客户的信息', 'success');
    } else if (data.resultCode == 11 - 100) {
      showMessage('校验通过:关联学生结课已经超过15天，本次提交将覆盖该客户的信息', 'success');
    } else if (data.resultCode == 12 - 100) {
      showMessage('校验通过:关联学生停课已经超过90天，本次提交将覆盖该客户的信息', 'success');
    } else if (data.resultCode == 13 - 100) {
      showMessage(data.resultMessage, 'error');
      disabled = true;
    } else if (data.resultCode == -36) {
      showMessage(data.resultMessage, 'error');
    } else {
      if (data.resultCode == -3) {
        const msg = data.resultMessage.split('#');
        showMessage(msg[2], 'warning');
        this.setState({
          followTarget: msg[1] + ' 正在跟进此客户',
          followTargetVisible: true,
        });
      } else {
        showMessage(data.resultMessage, 'warning');
      }
      disabled = true;
    }

    if (disabled) {
      return;
    }
    const res = await loaders.saveOrUpdateCustomer(params);
    if (res.resultCode != 0) {
      return showMessage('保存失败：' + res.resultMessage, 'error');
    }
    showMessage('保存成功', 'success');
    this.reset();
    return;
  };

  isPilot = async (params) => {
    const res = await loaders.saveOrUpdateCustomer(params);
    if (res.resultCode != 0) {
      return showMessage('保存失败：' + res.resultMessage, 'error');
    }
    showMessage('保存成功', 'success');
    this.reset();
    return;
  };

  getDisabledDate = (date) => {
    if (!date) {
      return false;
    }
    return moment(date).valueOf() > moment(new Date()).valueOf();
  };

  getCusOrgDict = (resEntranceId) => {
    if (!resEntranceId) {
      this.setState({ cusOrgDict: [] });
      this.props.form.setFieldsValue({
        cusOrg: '',
      });
      return;
    }
    loadSourceDetailDict(resEntranceId).then((cusOrgDict) => {
      this.setState({ cusOrgDict });
      this.props.form.setFieldsValue({ cusOrg: '' });
    });
  };

  getStudentsOnChange = (e) => this.getStudents(e.target.value);

  getStudents = async (val) => {
    if (phoneRegExp.test(val)) {
      this.setState({ loading: true, oldStudentLoading: true });
      const res = await loaders.studentList({ contact: val });
      const studentList = res.data.studentList || [];
      this.setState({
        studentList,
        studentListSelectable: studentList.filter((ele) => ele.canFollow),
        customerId: res.data.customerId || '',
        oldStudentLoading: false,
        loading: false,
      });
    } else {
      this.setState({
        studentList: [],
        studentListSelectable: [],
        customerId: '',
      });
    }
    this.setState({
      oldStudents: [],
    });
  };

  onCheckBoxChange = (val) => {
    this.setState({
      oldStudents: val,
      checkAll: val.length === this.state.studentListSelectable.length,
    });
  };

  increaseStudend = () => {
    const students = [...this.state.newStudents];
    students.push(students.length ? students[students.length - 1] + 1 : 0);
    this.setState({ newStudents: students });
  };

  decreaseStudend = (i) => {
    const students = [...this.state.newStudents];
    this.setState({
      newStudents: [...students.slice(0, i), ...students.slice(i + 1)],
    });
  };

  schoolModalCommit = (params) => {
    const index = this.state.schoolModalData.index;
    this.props.form.setFieldsValue({
      ['studentImportVos[' + index + '].schoolName']: params.schoolName,
      ['studentImportVos[' + index + '].schoolId']: params.schoolId,
    });
  };

  reset = () => {
    this.props.form.resetFields();
    this.props.form.setFieldsValue({ remark: '' });
    this.setState({
      customerId: '',
      newStudents: [0],
      oldStudents: [],
      followTarget: '',
      studentList: [],
      idCardType: '',
    });
  };

  emitEmpty = (index) => {
    return () => {
      this.props.form.setFieldsValue({
        ['studentImportVos[' + index + '].schoolName']: '',
        ['studentImportVos[' + index + '].schoolId']: '',
      });
    };
  };

  componentDidMount() {
    Promise.all([
      loadCommonDictNew('STUDENT_GRADE'),
      loadAllBranchCampusDict(),
      loaders.organization({
        id: this.props.user.campusId || this.props.user.brenchId,
      }),
      loadCommonDict('CERTIFICATE_TYPE'),
    ]).then(([studentGradeDict, campusDict, organization, certificateTypeDict]) => {
      let tempCertificateTypeDict =
        this.state.cusIdCardType && this.state.cusIdCardType !== 'CT_IDENTITY_CARD'
          ? certificateTypeDict
          : [{ text: '身份证', val: 'CT_IDENTITY_CARD' }];
      this.setState({
        studentGradeDict,
        campusDict,
        organization,
        loading: false,
        certificateTypeDict: tempCertificateTypeDict,
      });
    });
    if (this.props.contactFromOtherTab) {
      this.props.form.setFieldsValue({ contact: this.props.contactFromOtherTab });
      this.getStudents(this.props.contactFromOtherTab);
    }
  }

  getToolTipText = ({ status, finishDay, studentName, blCampusName }) => {
    return {
      CLASSING: `【学生：${studentName}/学籍：${blCampusName}/状态：上课中】不满足激活条件，请到【学生列表】操作签单`,
      STOP_CLASS: `【学生：${studentName}/学籍：${blCampusName}/状态：停课${
        finishDay || 0
      }天】不满足激活条件，请到【学生列表】操作签单`,
      FINISH_CLASS: `【学生：${studentName}/学籍：${blCampusName}/状态：结课${
        finishDay || 0
      }天】不满足激活条件，请到【学生列表】操作签单`,
    }[status];
  };

  render() {
    const { getFieldDecorator, getFieldValue } = this.props.form;
    const {
      loading,
      studentGradeDict,
      campusDict,
      cusOrgDict,
      studentList,
      studentListSelectable,
      checkAll,
      newStudents,
      oldStudents,
      schoolModalVisible,
      oldStudentLoading,
      disabled,
      followTargetVisible,
      followTarget,
      customerId,
      followedStudents,
    } = this.state;
    return (
      <Page loading={loading} title="客户录入">
        <Form onSubmit={this.handleSubmit}>
          <Block title="客户信息">
            <Row type="flex" justify="start">
              <Col span={8}>
                <FormItem label="客户姓名" {...formItemCol}>
                  {getFieldDecorator('name', {
                    rules: [{ required: true, message: '请填写客户姓名' }],
                  })(<Input />)}
                </FormItem>
              </Col>
              <Col span={8}>
                <FormItem label="联系方式" {...formItemCol}>
                  {getFieldDecorator('contact', {
                    rules: [
                      {
                        required: true,
                        pattern: phoneRegExp,
                        message: '请填写正确的联系方式',
                      },
                    ],
                  })(<Input onChange={this.getStudentsOnChange} />)}
                  {/* {studentList.length > 0 &&
                    <div className={styles.tipsWrap}>
                      <Icon type="warning" color={styleVars.warningColor} className={styles.tipsIcon} />
                      <div className={styles.phoneFormItemTips}>此号码已经存在系统，并有学生信息在此号码名下，如果您要输入的是同一学生，请点击下方关联按钮，并保存。</div>
                    </div>
                  } */}
                </FormItem>
              </Col>
            </Row>
            <Row type="flex" justify="start">
              <Col span={8}>
                {' '}
                <FormItem label="资源来源" {...formItemCol}>
                  {getFieldDecorator(
                    'cusType',
                    {},
                  )(
                    <Select
                      placeholder="资源来源"
                      dict={resTypeDict}
                      onChange={(val) => {
                        this.getCusOrgDict(val);
                      }}
                    />,
                  )}
                </FormItem>
              </Col>
              <Col span={8}>
                <FormItem label="来源细分" {...formItemCol}>
                  {getFieldDecorator('cusOrg', {})(<Select placeholder="来源细分" dict={cusOrgDict} />)}
                </FormItem>
              </Col>
            </Row>
            <Row type="flex" justify="start">
              <Col span={8}>
                {' '}
                <FormItem label="意向学习中心" {...formItemCol}>
                  {getFieldDecorator('intentionCampusId', {
                    rules: [{ required: true, message: '请选择意向学习中心' }],
                  })(<Select placeholder="请选择学习中心" showSearch dict={campusDict} />)}
                </FormItem>
              </Col>
            </Row>
            {loginUser.isXingHuoSym() && (
              <Row>
                <Col span={8}>
                  <FormItem label="证件类型" {...formItemCol}>
                    {getFieldDecorator('idCardType', {
                      initialValue: this.state.idCardType,
                      rules: [{ required: false }],
                    })(
                      <Select
                        disabled={Boolean(this.state.idCardType && this.state.idCardType !== 'CT_IDENTITY_CARD')}
                        dict={this.state.certificateTypeDict}
                        onChange={(val) => {
                          this.setState({ idCardType: val });
                        }}
                      />,
                    )}
                  </FormItem>
                </Col>
                <Col span={8}>
                  <FormItem label="证件号码" {...formItemCol}>
                    {getFieldDecorator('idCardNo', {
                      rules: [{ required: false }],
                    })(
                      <Input
                        disabled={Boolean(this.state.idCardType && this.state.idCardType !== 'CT_IDENTITY_CARD')}
                      />,
                    )}
                  </FormItem>
                </Col>
              </Row>
            )}
            <Row type="flex" justify="start">
              <Col span={8}>
                <FormItem label="备注" {...formItemCol}>
                  {getFieldDecorator('remark', {})(<TextArea />)}
                </FormItem>
              </Col>
            </Row>
          </Block>
          {studentList.length <= 0 ? (
            ''
          ) : (
            <Section
              loading={oldStudentLoading}
              title="关联学生"
              style={{ paddingBottom: '30px', marginBottom: '30px' }}
              toolBar={
                <Checkbox
                  checked={checkAll}
                  onChange={(e) => {
                    this.setState({
                      checkAll: !checkAll,
                      oldStudents: e.target.checked ? [...studentListSelectable] : [],
                    });
                  }}
                >
                  全部跟进
                </Checkbox>
              }
            >
              <div className={styles.tipInfo}>
                <span>同一学生，请勾选关联并保存即可</span>
              </div>
              {studentList.length <= 0 ? (
                <div style={{ height: '60px', textAlign: 'center' }}>无关联学生</div>
              ) : (
                <CheckboxGroup onChange={this.onCheckBoxChange} value={oldStudents}>
                  {studentList.map((ele) => {
                    const tag = ele.studentType === 'ENROLLED' && !ele.canFollow && '在读学生';
                    const toolTipText = this.getToolTipText(ele);
                    return (
                      <Row key={ele.studentId} style={{ marginLeft: 20 }}>
                        <Col span={1} className={styles.xsCheckboxCol}>
                          {ele.canFollow ? (
                            <Checkbox value={ele} />
                          ) : toolTipText ? (
                            <Tooltip placement="right" title={toolTipText} overlayClassName={styles.toolTipWrapper}>
                              <Checkbox disabled />
                            </Tooltip>
                          ) : (
                            <Checkbox disabled />
                          )}
                        </Col>
                        <Col span={6}>
                          <FormItem label="学生" {...linkFormCfg}>
                            <div
                              style={{
                                wordBreak: 'break-word',
                                display: 'flex',
                                justifyContent: 'center',
                                alignItems: 'baseline',
                              }}
                            >
                              <span style={{ wordBreak: 'break-word', flex: '1' }}>
                                {ele.studentName}
                                {ele.haveLabel && <i style={{ color: 'red' }}>*</i>}
                              </span>
                              {tag && (
                                <span
                                  style={{
                                    marginLeft: '10px',
                                    border: '1px #ff8080 solid',
                                    padding: '0 4px',
                                    borderRadius: '4px',
                                    color: '#ff8080',
                                    display: 'inline-block',
                                    lineHeight: '20px',
                                    fontSize: '12px',
                                  }}
                                >
                                  {tag}
                                </span>
                              )}
                            </div>
                          </FormItem>
                        </Col>
                        <Col span={6}>
                          <FormItem label="年级" {...linkFormCfg}>
                            {ele.gradeName}
                          </FormItem>
                        </Col>
                        <Col span={6}>
                          <FormItem label="学校" {...linkFormCfg}>
                            {ele.schoolName}
                          </FormItem>
                        </Col>
                      </Row>
                    );
                  })}
                </CheckboxGroup>
              )}
            </Section>
          )}
          <Section
            title="学员档案"
            style={{ paddingBottom: '30px', marginBottom: '30px' }}
            // toolBar={
            //   <TextLink onClick={this.increaseStudend}>增加学员</TextLink>
            // }
          >
            {newStudents.length <= 0 ? (
              <div style={{ height: '60px', textAlign: 'center' }}>请新增学员</div>
            ) : (
              newStudents.map((ele, i) => {
                return (
                  <Section
                    title={'学员' + (i + 1)}
                    style={{
                      paddingBottom: '30px',
                      marginBottom: '30px',
                    }}
                    // toolBar={
                    //   <TextLink onClick={() => this.decreaseStudend(i)}>
                    //     删除学员
                    //   </TextLink>
                    // }
                    key={ele}
                  >
                    <Row type="flex" justify="start">
                      <Col span={8}>
                        <FormItem label="学员姓名" {...formItemCol}>
                          {getFieldDecorator('studentImportVos[' + ele + '].name', {})(<Input />)}
                        </FormItem>
                      </Col>
                      <Col span={8}>
                        <FormItem label="学员电话" {...formItemCol}>
                          {getFieldDecorator('studentImportVos[' + ele + '].contact', {
                            rules: [
                              {
                                pattern: phoneRegExp,
                                message: '请填写正确的联系方式',
                              },
                            ],
                          })(<Input />)}
                        </FormItem>
                      </Col>
                    </Row>
                    <Row type="flex" justify="start">
                      <Col span={8}>
                        <FormItem label="性别" {...formItemCol}>
                          {getFieldDecorator(
                            'studentImportVos[' + ele + '].sex',
                            {},
                          )(<Select placeholder="请选择" dict={sexDict} />)}
                        </FormItem>
                      </Col>
                      <Col span={8}>
                        <FormItem label="出生日期" {...formItemCol}>
                          {getFieldDecorator(
                            'studentImportVos[' + ele + '].bothday',
                            {},
                          )(<DatePicker placeholder="出生日期" disabledDate={this.getDisabledDate} />)}
                        </FormItem>
                      </Col>
                    </Row>
                    <Row type="flex" justify="start">
                      <Col span={8}>
                        <FormItem label="就读学校" {...formItemCol}>
                          {getFieldDecorator(
                            'studentImportVos[' + ele + '].schoolName',
                            {},
                          )(
                            <Input
                              autoComplete="off"
                              onClick={() => {
                                this.setState({
                                  schoolModalData: { index: ele },
                                  schoolModalVisible: true,
                                });
                              }}
                              suffix={
                                getFieldValue('studentImportVos[' + ele + '].schoolId') ? (
                                  <Icon
                                    type="cross_circle"
                                    style={{
                                      color: '#6d67ff',
                                    }}
                                    onClick={this.emitEmpty(ele)}
                                  />
                                ) : null
                              }
                              style={{ caretColor: 'transparent' }}
                            />,
                          )}
                        </FormItem>
                        <div style={{ display: 'none' }}>
                          <FormItem>
                            {getFieldDecorator('studentImportVos[' + ele + '].schoolId', {})(<Input />)}
                          </FormItem>
                        </div>
                      </Col>
                      <Col span={8}>
                        <FormItem label="就读年级" {...formItemCol}>
                          {getFieldDecorator(
                            'studentImportVos[' + ele + '].gradeId',
                            {},
                          )(<Select placeholder="请选择年级" showSearch dict={studentGradeDict} />)}
                        </FormItem>
                      </Col>
                    </Row>
                    <Row type="flex" justify="start">
                      <Col span={8}>
                        <FormItem label="父亲姓名" {...formItemCol}>
                          {getFieldDecorator('studentImportVos[' + ele + '].fatherName', {})(<Input />)}
                        </FormItem>
                      </Col>
                      <Col span={8}>
                        <FormItem label="父亲联系方式" {...formItemCol}>
                          {getFieldDecorator('studentImportVos[' + ele + '].fatherPhone', {
                            rules: [
                              {
                                pattern: phoneRegExp,
                                message: '请填写正确的联系方式',
                              },
                            ],
                          })(<Input />)}
                        </FormItem>
                      </Col>
                    </Row>
                    <Row type="flex" justify="start">
                      <Col span={8}>
                        <FormItem label="母亲姓名" {...formItemCol}>
                          {getFieldDecorator('studentImportVos[' + ele + '].motherName', {})(<Input />)}
                        </FormItem>
                      </Col>
                      <Col span={8}>
                        <FormItem label="母亲联系方式" {...formItemCol}>
                          {getFieldDecorator('studentImportVos[' + ele + '].notherPhone', {
                            rules: [
                              {
                                pattern: phoneRegExp,
                                message: '请填写正确的联系方式',
                              },
                            ],
                          })(<Input />)}
                        </FormItem>
                      </Col>
                    </Row>
                    <Row type="flex" justify="start">
                      <Col span={8}>
                        <FormItem label="学员位置" {...formItemCol}>
                          {getFieldDecorator(
                            'studentImportVos[' + ele + '].address',
                            {},
                          )(
                            <Input
                              placeholder="请按省、市、区/县（乡/镇 村）、街道、门牌号的格式填写"
                              title="请按省、市、区/县（乡/镇 村）、街道、门牌号的格式填写"
                            />,
                          )}
                        </FormItem>
                      </Col>
                      {loginUser.isXingHuoSym() && (
                        <Col span={8}>
                          {' '}
                          <FormItem label="学生身份证" {...formItemCol}>
                            {getFieldDecorator('studentImportVos[' + ele + '].idCardNo', {
                              rules: [{ required: false }],
                            })(<Input />)}
                          </FormItem>
                        </Col>
                      )}
                    </Row>
                  </Section>
                );
              })
            )}
          </Section>
          <Block>
            <Row type="flex" justify="start">
              <Col offset={10} span={2}>
                <FormItem wrapperCol={{ span: 12, offset: 5 }}>
                  <Button type="primary" htmlType="submit" disabled={disabled}>
                    保存
                  </Button>
                </FormItem>
              </Col>
              <Col span={2}>
                <FormItem wrapperCol={{ span: 12, offset: 5 }}>
                  <Button htmlType="button" onClick={this.reset}>
                    重置
                  </Button>
                </FormItem>
              </Col>
            </Row>
          </Block>
        </Form>
        {schoolModalVisible && (
          <SchoolModal
            orgId={getFieldValue('intentionCampusId')}
            onCommit={this.schoolModalCommit}
            onClose={() => this.setState({ schoolModalVisible: false })}
          />
        )}
        {followTargetVisible && (
          <FollowTargetModal
            customerId={customerId}
            students={followedStudents}
            onClose={() => this.setState({ followTargetVisible: false })}
          >
            {followTarget}
          </FollowTargetModal>
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const data = state.main.pageParams['/InputCustomerOnLine'];
  return {
    user: state.account.user,
    contactFromOtherTab: data ? data.contact : '',
    workbrenchType: data ? data.workbrenchType : '',
  };
};

export default connect(mapStateToProps)(Form.create()(InputCustomerOnLine));
