/**
 * 无效资源池页面
 */

import React, { Component } from 'react';
import { connect } from 'dva';
import { Form, message } from 'antd';
import { Page, List, Modal, Select, FormItem, Button } from 'COMPONENT';
import SearchPage from './search';
import { namespace, menuID, orgTypeConfig } from './store/constant';
import { outCallResourcePools } from 'SERVICE/outcallWorkbench';
import moment from 'moment';
import { TreeSelect } from 'antd';

const { COL_WIDTH_M, COL_WIDTH_S, COL_WIDTH_L } = List;
const Time_Col_Width = COL_WIDTH_M + 10;
const ResourceTypeOut = 1; // 资源类型，1-外呼；2-常规

function formatNodesToParentChildren(nodes = [], parentId = '') {
  if (nodes.length === 0) {
    return [];
  }

  const parents = [];
  const rest = [];
  for (let node of nodes) {
    node.parentId === parentId ? parents.push(node) : rest.push(node);
  }

  for (let parent of parents) {
    parent.children = formatNodesToParentChildren(rest, parent.nodeId);
  }

  return parents;
}

class InvalidResourcePool extends Component {
  state = {
    pagination: {
      current: 1,
      pageSize: 50,
    },
    loading: false,
    searchData: {},
    selectedRowKeys: [],
    delModalVisible: false,
    delModalLoading: false,
    recoveryModalVisible: false,
    recoveryModalLoading: false,
    resultModalVisible: false,
    allocateCampusId: '', // 分配学习中心
    treeDict: [],
    deliverTarget: '',
    treeDefaultExpandedKeys: [],
    nodeType: '',
  };

  columns = [
    { title: '学生姓名', dataIndex: 'studentName', key: 'studentName', width: COL_WIDTH_S },
    { title: '客户姓名', dataIndex: 'customerName', key: 'customerName', width: COL_WIDTH_S },
    { title: '客户手机号码', dataIndex: 'customerContact', key: 'customerContact', width: COL_WIDTH_M },
    { title: '资源入口', dataIndex: 'resEntranceName', key: 'resEntranceName', width: COL_WIDTH_S },
    { title: '所属组织', dataIndex: 'blOrganizationName', key: 'blOrganizationName', width: COL_WIDTH_M },
    { title: '跟进人', dataIndex: 'deliverTargetName', key: 'deliverTargetName', width: COL_WIDTH_S },
    {
      title: '创建时间',
      dataIndex: 'resourceCreateTime',
      key: 'resourceCreateTime',
      width: Time_Col_Width,
      render: (timestamp, record, index) => {
        return moment(new Date(timestamp)).format('YYYY-MM-DD hh:mm:ss');
      },
    },
    {
      title: '无效时间',
      dataIndex: 'createTime',
      key: 'createTime',
      width: Time_Col_Width,
      render: (timestamp, record, index) => {
        return moment(new Date(timestamp)).format('YYYY-MM-DD hh:mm:ss');
      },
    },
    { title: '操作删除人', dataIndex: 'updateUserName', key: 'updateUserName', width: COL_WIDTH_M },
    {
      title: '操作删除时间',
      dataIndex: 'updateTime',
      key: 'updateTime',
      width: Time_Col_Width,
      render: (timestamp, record, index) => {
        return moment(new Date(timestamp)).format('YYYY-MM-DD hh:mm:ss');
      },
    },
    {
      title: '是否删除',
      dataIndex: 'isDeleted',
      key: 'isDeleted',
      width: COL_WIDTH_S,
      render: (t) => (t ? '是' : '否'),
    },
  ];

  resultsColumns = [
    { title: '学生姓名', dataIndex: 'studentName', key: 'studentName', width: COL_WIDTH_M },
    { title: '客户姓名', dataIndex: 'customerName', key: 'customerName', width: COL_WIDTH_M },
    { title: '手机号码', dataIndex: 'contact', key: 'contact', width: COL_WIDTH_M },
    {
      title: '分配结果',
      dataIndex: 'businessErrorMessage',
      key: 'businessErrorMessage',
      width: COL_WIDTH_M,
      render: (t, r) => `分配${r.success ? '成功' : '失败'}(${t ? t : '无'})`,
    },
  ];

  componentDidMount() {
    this.initDict();
    this.getDistributePoolData();
  }

  getDistributePoolData = async () => {
    const { data } = await outCallResourcePools({
      /* 外呼资源池树 */ resourceType: 1,
      withUser: true,
      withCapacity: true,
    });
    // get nodeIds map
    const nodeIdsMap = {};
    for (let node of data) {
      const { nodeId } = node;
      nodeIdsMap[nodeId] = node;
    }

    // get distributableNodes' parents
    const parents = new Set();
    const distributableNodes = data.filter((item) => item.distributable);
    for (let { parentId } of distributableNodes) {
      if (nodeIdsMap[parentId]) {
        parents.add(nodeIdsMap[parentId]);
      }
    }

    // remove duplicate
    const nodesSet = new Set(distributableNodes.concat([...parents]));

    // map nodes to TreeSelect format
    const nodes = [...nodesSet].map((node) => ({
      ...node,
      label: `${node.nodeName}（${node.usedCapacity || 0}/${node.capacity || 0}）`,
      value: node.nodeId,
      key: node.nodeId,
    }));
    const parentNode = nodes.find((node) => node.orgType === 'GROUNP');
    parentNode.children = formatNodesToParentChildren([...nodes], parentNode.nodeId);
    parentNode.disabled = true;

    this.setState({
      treeDict: [parentNode],
      treeDefaultExpandedKeys: [parentNode.nodeId],
    });
  };

  onTreeSelectChange = (deliverTarget, { props: { nodeType } }) => {
    this.setState({
      nodeType,
      deliverTarget,
    });
  };

  userFilter = (searchKey, option) => {
    return option.text.indexOf(searchKey) >= 0;
  };

  initDict = () => {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['invalid_sign'],
      },
    });
  };

  // 查询 无效资源列表
  queryInvalidResource = (params) => {
    const {
      pagination: { current, pageSize },
      searchData,
    } = this.state;
    const data = {
      page: current,
      rows: pageSize,
      ...params,
      ...searchData,
      isRecovered: 0,
    };

    this.setState({ loading: true });
    this.props.dispatch({
      type: `${namespace}/queryInvalidResource`,
      payload: { params: data, callback: () => this.setState({ loading: false, selectedRowKeys: [] }) },
    });
  };

  // 查询事件回调
  handleSearch = (searchData) => {
    this.setState({ searchData }, () => {
      this.queryInvalidResource();
    });
  };

  // 分页切换
  changePage = ({ current, pageSize }) => {
    this.setState({ pagination: { current, pageSize } });
    this.queryInvalidResource({ page: current, rows: pageSize });
  };

  // 表格多选
  onSelectChange = (selectedRowKeys) => {
    this.setState({ selectedRowKeys });
  };

  // 彻底删除确认
  handleDelOk = () => {
    const { selectedRowKeys } = this.state;
    this.setState({ delModalLoading: true });
    const params = {
      ids: selectedRowKeys,
      resourceType: ResourceTypeOut,
    };
    this.props.dispatch({
      type: `${namespace}/deleteInvalidResource`,
      payload: {
        params,
        callback: () => {
          this.setState({ delModalVisible: false, delModalLoading: false, selectedRowKeys: [] });
          this.queryInvalidResource();
        },
      },
    });
  };

  // 搜索框彻底删除回调
  handleDel = () => {
    this.setState({ delModalVisible: true });
  };

  // 搜索框彻底恢复回调
  handleRecovery = async () => {
    await this.getDistributePoolData();
    this.setState({ recoveryModalVisible: true });
  };

  // 彻底恢复 - 确认
  handleRecoveryOk = async () => {
    const { selectedRowKeys: ids, nodeType, deliverTarget } = this.state;
    this.setState({ recoveryModalLoading: true });

    // 节点类型：1-组织；2-用户
    const nodeTypeMap = {
      1: 'CUSTOMER_RESOURCE_POOL',
      2: 'PERSONAL_POOL',
    };

    const params = {
      ids,
      deliverTarget,
      resourceType: ResourceTypeOut,
      deliverType: nodeTypeMap[nodeType],
    };
    this.props.dispatch({
      type: `${namespace}/recoverResource`,
      payload: {
        params,
        callback: (isSuccess) => {
          if (isSuccess) {
            this.setState(
              {
                recoveryModalVisible: false,
                selectedRowKeys: [],
              },
              () => {
                this.queryInvalidResource();
              },
            );
          }

          this.setState({
            recoveryModalLoading: false,
          });
        },
      },
    });
  };

  render() {
    const { route, total, list, distributionResults, btnAuths } = this.props;
    const {
      pagination: { current, pageSize },
      loading,
      delModalVisible,
      delModalLoading,
      recoveryModalLoading,
      resultModalVisible,
      selectedRowKeys,
      recoveryModalVisible,
      treeDict,
      deliverTarget,
      treeDefaultExpandedKeys,
    } = this.state;

    const rowSelection = {
      selectedRowKeys,
      onChange: this.onSelectChange,
      getCheckboxProps: (record) => ({
        disabled: record.isDeleted == '1' ? true : false,
      }),
    };

    return (
      <Page
        fitWindowHeight={true}
        route={route}
        headerToolBar={
          <div>
            {btnAuths.includes('INVALID_POOL_RESTORE') && (
              <Button type="primary" onClick={this.handleRecovery} disabled={selectedRowKeys.length === 0}>
                恢复资源
              </Button>
            )}
            {btnAuths.includes('INVALID_POOL_DELETE') && (
              <Button
                style={{ marginLeft: '15px' }}
                type="primary"
                onClick={this.handleDel}
                disabled={selectedRowKeys.length === 0}
              >
                彻底删除
              </Button>
            )}
          </div>
        }
        subHeader={<SearchPage onSearch={this.handleSearch} getOptionalOrganization={this.getOptionalOrganization} />}
      >
        <List
          dataSource={list}
          columns={this.columns}
          loading={loading}
          pagination={{
            current,
            pageSize,
            total,
            pageSizeOptions: ['50', '100'],
          }}
          rowKey={(record) => record.id}
          onChange={this.changePage}
          rowSelection={rowSelection}
        />
        {delModalVisible && (
          <Modal
            title="彻底删除"
            widthSize="s"
            onOk={this.handleDelOk}
            loading={delModalLoading}
            onCancel={() => {
              this.setState({ delModalVisible: false });
            }}
          >
            <span>本操作会将所选资源从无效资源中彻底删除，删除后不可恢复，请确认是否需要彻底删除</span>
          </Modal>
        )}
        {recoveryModalVisible && (
          <Modal
            title="请选择分配对象"
            widthSize="s"
            onOk={this.handleRecoveryOk}
            okText="确认恢复"
            loading={recoveryModalLoading}
            onCancel={() => {
              this.setState({ recoveryModalVisible: false });
            }}
          >
            <TreeSelect
              size="large"
              allowClear
              value={deliverTarget}
              dropdownStyle={{ maxHeight: 400, overflow: 'auto' }}
              treeData={treeDict}
              placeholder="请选择分配对象"
              treeDefaultExpandedKeys={treeDefaultExpandedKeys}
              onSelect={this.onTreeSelectChange}
              showSearch
              treeNodeFilterProp="label"
            />
          </Modal>
        )}
        {resultModalVisible && (
          <Modal
            title="分配结果"
            widthSize="m"
            okBtnShow={false}
            loading={recoveryModalLoading}
            cancelText="关闭"
            onCancel={() => {
              this.setState({ resultModalVisible: false });
            }}
          >
            <List dataSource={distributionResults} columns={this.resultsColumns} />
          </Modal>
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const { list, total, distributionObjects, distributionResults } = state[namespace];
  return {
    btnAuths: state.account.btnAuths || '',
    list,
    total,
    distributionResults,
    distributionObjects,
  };
};

export default connect(mapStateToProps)(InvalidResourcePool);
