import React from 'react';
import { DatePicker } from 'antd';
import { connect } from 'dva';
import moment from 'moment';
import { Select, Input, SearchFilter, SearchField } from 'COMPONENT';
import { is_delete } from '../store/constant';
import { loadSourceDetailDict } from 'PAGE/common/loaders';

const { RangePicker } = DatePicker;

const currentDay = moment().format('"YYYY-MM-DD');

const initialState = {
  searchParams: {
    createTime: [moment(currentDay), moment(currentDay)],
  },
  organizationId: '',
  studentName: '',
  customerContact: '',
  invalidSignId: '',
  isDeleted: '',
};

class SearchForm extends React.Component {
  state = {
    ...initialState,
  };

  componentDidMount() {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['branch', 'resEntrance', 'resType'],
      },
    });
    this.handleSearch();
  }

  handleSearch = () => {
    const { createTime, ...params } = this.state.searchParams;

    if (createTime.length > 0) {
      const [beginCreateTime, endCreateTime] = createTime;
      params.beginCreateTime = beginCreateTime.valueOf();
      const endCreateTimeValue = endCreateTime.valueOf();
      params.endCreateTime = moment(endCreateTimeValue).add(1, 'days').valueOf() - 1;
    }

    const filteredParams = {};
    for (const [key, val] of Object.entries(params)) {
      if (val === '') {
        continue;
      }
      filteredParams[key] = val;
    }

    this.props.onSearch(filteredParams);
  };

  /**
   * 查询重置
   * @memberof SearchPage
   */
  handleReset = () => {
    this.setState(
      {
        ...initialState,
        sourceDetailDict: [],
      },
      () => {
        // this.handleSearch();
      },
    );
  };

  handleSearchParamChange = (key) => {
    return (e) => {
      let { searchParams } = this.state;
      const update = {};
      const val = e ? (e.target ? e.target.value : e) : '';
      update[key] = typeof val === 'string' ? val.trim() : val;
      // 级联资源细分
      if (key === 'cusType') {
        update.cusOrg = '';
        if (update[key]) {
          loadSourceDetailDict(update[key]).then((dict) => {
            this.setState({
              sourceDetailDict: dict,
            });
          });
        } else {
          this.setState({
            sourceDetailDict: [],
          });
        }
      }
      this.setState({ searchParams: { ...searchParams, ...update } });
    };
  };

  render() {
    const { branchDict, invalid_sign, resEntrance, resTypeDict } = this.props;
    const { searchParams } = this.state;

    return (
      <SearchFilter truncationNum={5} onSearch={this.handleSearch} onReset={this.handleReset}>
        <SearchField>
          <RangePicker
            value={searchParams.createTime}
            // showTime
            format="YYYY-MM-DD"
            placeholder={['无效日期开始', '无效日期结束']}
            size="large"
            allowClear
            onChange={this.handleSearchParamChange('createTime')}
            style={{ width: '250px' }}
          />
        </SearchField>
        <SearchField>
          <Select
            value={searchParams.organizationId}
            placeholder="所属组织"
            dict={branchDict}
            showSearch={true}
            onChange={this.handleSearchParamChange('organizationId')}
            allowClear={true}
          />
        </SearchField>
        <SearchField>
          <Input
            value={searchParams.studentName}
            onChange={this.handleSearchParamChange('studentName')}
            placeholder="学生姓名"
            allowClear={true}
          />
        </SearchField>
        <SearchField>
          <Input
            value={searchParams.customerContact}
            onChange={this.handleSearchParamChange('customerContact')}
            placeholder="客户手机号码"
            allowClear={true}
          />
        </SearchField>
        <SearchField>
          <Select
            value={searchParams.invalidSignId}
            onChange={this.handleSearchParamChange('invalidSignId')}
            placeholder="无效标签"
            dict={invalid_sign}
            allowClear={true}
          />
        </SearchField>
        <SearchField>
          <Select
            value={searchParams.isDeleted}
            onChange={this.handleSearchParamChange('isDeleted')}
            placeholder="是否删除"
            dict={is_delete}
            allowClear={true}
          />
        </SearchField>
        <SearchField>
          <Select
            value={searchParams.resEntrance}
            onChange={this.handleSearchParamChange('resEntrance')}
            placeholder="资源入口"
            dict={resEntrance}
            allowClear={true}
          />
        </SearchField>
        <SearchField>
          <Select
            value={searchParams.cusType}
            onChange={this.handleSearchParamChange('cusType')}
            placeholder="资源来源"
            dict={resTypeDict}
            allowClear={true}
          />
        </SearchField>
        <SearchField>
          <Select
            value={searchParams.cusOrg}
            onChange={this.handleSearchParamChange('cusOrg')}
            placeholder="资源细分"
            dict={this.state.sourceDetailDict}
            allowClear={true}
          />
        </SearchField>
      </SearchFilter>
    );
  }
}

const mapStateToProps = (state) => {
  const {
    dict: { invalid_sign, branch: branchDict },
  } = state;

  return {
    branchDict,
    invalid_sign,
    resTypeDict: state.dict.resType,
    resEntrance: state.dict.resEntrance,
  };
};

export default connect(mapStateToProps)(SearchForm);
