import { message } from 'antd';
import { runFunc } from 'UTIL/common';
import { namespace, orgTypeConfig } from './constant';
import {
  queryInvalidResource,
  deleteInvalidResource,
  getOptionalOrganization,
  recoverResource,
  resourceRecordRecover,
  resourceRecordDelete,
  resourceRecordList,
} from 'SERVICE/customer';
import { transformArrayToSelect } from 'UTIL/dict';
import { errorsHandling } from './util';

export default {
  namespace,
  state: {
    list: [], // 无效资源池列表
    total: 0,
    branchDict: [], // 分公司枚举
    campusDict: [], // 学习中心枚举
    distributionObjects: [], // 分配对象
    distributionResults: [], // 分配结果
  },
  reducers: {
    updateState(state, { payload }) {
      return {
        ...state,
        ...payload,
      };
    },
  },
  effects: {
    // 请求无效资源列表
    *queryInvalidResource({ payload }, { put }) {
      const { params, callback } = payload;
      const res = yield resourceRecordList(params).catch(errorsHandling());
      if (res && res.resultCode === 0) {
        const { data } = res;
        yield put({
          type: 'updateState',
          payload: { list: data.list, total: data.total },
        });
      }
      runFunc(callback);
    },
    // 彻底删除
    *deleteInvalidResource({ payload }) {
      const { params, callback } = payload;
      const res = yield resourceRecordDelete(params).catch(errorsHandling());
      if (res && res.resultCode === 0) {
        message.success('彻底删除成功');
      }
      runFunc(callback);
    },
    // 恢复无效资源
    *recoverResource({ payload }) {
      const { params, callback } = payload;
      const res = yield resourceRecordRecover(params).catch(errorsHandling());
      let isSuccess = res && res.resultCode === 0;
      callback(isSuccess);
    },
  },
};
