import React from 'react';
import PropTypes from 'prop-types';
import InputButton from '../InputButton';
import ModalSelectSchool from 'BLOC/ModalSelectSchool';
import { runFunc } from 'UTIL/common';

class SchoolButton extends React.Component {
  static propTypes = {
    id: PropTypes.string.isRequired,
    text: PropTypes.string,
    value: PropTypes.string,
    onChange: PropTypes.func,
  };

  constructor(props) {
    super(props);

    this.switchVisible = this.switchVisible.bind(this);
    this.onSelect = this.onSelect.bind(this);

    this.state = {
      visible: false,
    };
  }

  switchVisible() {
    this.setState((prevState) => ({ visible: !prevState.visible }));
  }

  onSelect(data) {
    runFunc(this.props.onChange, data.id, data.name);
    this.switchVisible();
  }

  render() {
    const { props, state } = this;
    return (
      <InputButton
        placeholder="学校"
        text={props.text || ''}
        value={props.value || ''}
        onClick={this.switchVisible}
        slot={
          state.visible && (
            <ModalSelectSchool
              blCampusId={props.id}
              schoolId={props.value}
              onCancel={this.switchVisible}
              onOk={this.onSelect}
              useOtherRemark={false}
            />
          )
        }
      />
    );
  }
}

export default SchoolButton;
