import React from 'react';
import styles from './styles.less';
import { loadCusNews } from 'PAGE/common/CusDetailPage/loaders';
import Loading from 'COMPONENT/Loading';
import classnames from 'classnames';

class CusNewsBlock extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      list: [], // 列表数据，[{ time, content }]
      loading: false,
    };
  }

  initContent = async () => {
    this.setState({ loading: true });
    const { customerId, stuId, deliverTarget } = this.props;
    try {
      const list = await loadCusNews(customerId, stuId, deliverTarget);
      this.setState({ list });
    } finally {
      this.setState({ loading: false });
    }
  };

  componentDidMount() {
    this.initContent();
  }

  render() {
    const { list, loading } = this.state;
    return (
      <div className={styles.wrap}>
        <div className={styles.inner}>
          <Loading loading={loading} />
          {list.length ? (
            list.map((li, idx) => {
              return (
                <div key={idx} className={classnames(styles.item, idx === 0 ? styles.active : '')}>
                  <p className={styles.content}>{li.content}</p>
                  <p className={styles.time}>{li.time}</p>
                </div>
              );
            })
          ) : (
            <p style={{ textAlign: 'center', color: '#ccc', fontSize: '18px' }}>暂无数据</p>
          )}
        </div>
      </div>
    );
  }
}
export default CusNewsBlock;
