import React from 'react';
import { connect } from 'dva';
import { Tabs } from 'antd';
import { Button, Section, Loading } from 'COMPONENT';
import styles from '../styles.less';
import CallRecord from 'PAGE/consultant/CallRecord';
import CusNewsBlock from '../components/CusNewsBlock';
import Record from 'PAGE/common/CusDetailPage/Record';
import ModalCallDetail from 'PAGE/consultant/ModalCallDetail';
import ModalPhoneCallFollow from 'PAGE/consultant/ModalPhoneCallFollow';
import AddFollowRecord from '../../customerFollow/AddFollowRecord';
import { CALL_DIRECT_CALL } from 'CONST/callConfig';
import { getMyCustomerFollowUpWrappers } from 'SERVICE/consultant';
import { checkAgentBeforeCall } from 'SRC/utils/callCenter';
import { CALL_LOGIN_STATUS_ABNORMAL } from 'CONST/callConfig';

const { TabPane } = Tabs;

class CustomerTabPanel extends React.Component {
  constructor(props) {
    super(props);
    const { pageParams } = props;
    this.pageParams = pageParams[`/${props.route.path}`];
    const { studentId, deliverTarget: userId } = this.pageParams;
    this.audioAuth = props.btnAuth.indexOf('CALL_AUTH_PLAY_AUDIO') > -1;
    this.phoneCallFollowData = null;
    this.callRecordDefaultSKey = {
      userId,
      studentId,
      callDirection: CALL_DIRECT_CALL,
    };
    this.tabRefs = {};
    this.state = {
      loading: false,
      isChecking: false,
      activeKey: 'followRecrods',
      modalCallDetailVisible: false,
      modalPhoneCallFollowVisible: false,
      modalFlowRecordVisible: false,
      recordList: [],
    };
  }

  componentDidMount() {
    this.initDict();
  }

  initDict = () => {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['customerSignType'],
        callback: this.loadMyFollowList,
      },
    });
  };

  switchModalCallDetail = (detail) => {
    this.setState((prevState) => {
      this.callDetail = prevState.modalCallDetailVisible ? null : detail;
      return { modalCallDetailVisible: !prevState.modalCallDetailVisible };
    });
  };

  loadMyFollowList = async () => {
    this.setState({ loading: true });
    const { customerId, studentId } = this.pageParams;
    try {
      const res = await getMyCustomerFollowUpWrappers({ customerId, studentId });
      if (res.data) {
        this.setState({
          recordList: res.data.myFollowingList || [],
        });
      }
    } finally {
      this.setState({ loading: false });
    }
  };

  followByCall = async () => {
    this.setState({ isChecking: true });
    try {
      const agentLoginStatus = await new Promise(this.checkAgentLoginStatus);
      if (checkAgentBeforeCall(agentLoginStatus)) {
        this.switchModalPhoneCallFollow();
      }
    } finally {
      this.setState({ isChecking: false });
    }
  };

  checkAgentLoginStatus = (resolve) => {
    this.props.dispatch({
      type: 'callCenter/fetchToUpdAgentInfo',
      payload: {
        callback(agentObj) {
          resolve((agentObj || {}).agentLoginStatus || CALL_LOGIN_STATUS_ABNORMAL);
        },
      },
    });
  };

  switchModalPhoneCallFollow = () => {
    const { customerId, studentId } = this.pageParams;
    const { studentName, customerContact, gradeName, schoolName, resEntranceName, customerName } =
      this.props.customerInfo;
    this.setState(
      (prevState) => {
        this.phoneCallFollowData = prevState.modalPhoneCallFollowVisible
          ? null
          : {
              customerId,
              studentId,
              studentName,
              phoneNum: customerContact,
              gradeName,
              schoolName,
              resEntranceName,
              customerName,
            };
        return { modalPhoneCallFollowVisible: !prevState.modalPhoneCallFollowVisible };
      },
      () => {
        if (!this.state.modalPhoneCallFollowVisible) {
          this.onTabClick('followRecrods');
        }
        GlobalEvent.emit('CrmNotification:followCustomerFromDetail');
      },
    );
  };

  switchFlowRecordModal = () => {
    this.setState((prevState) => {
      return {
        modalFlowRecordVisible: !prevState.modalFlowRecordVisible,
      };
    });
  };

  getTabInstance = (comp, key) => {
    this.tabRefs[key] = comp;
  };

  // 点击tab刷新
  onTabClick = (key) => {
    const ref = this.tabRefs[key];
    if (ref) {
      switch (key) {
        case 'followRecrods':
          this.loadMyFollowList();
          break;
        case 'phoneRecords':
          ref.init();
          break;
        case 'customerState':
          ref.initContent();
          break;
      }
    }
    this.setState({ activeKey: key });
  };

  render() {
    const {
      recordList,
      activeKey,
      loading,
      isChecking,
      modalPhoneCallFollowVisible,
      modalCallDetailVisible,
      modalFlowRecordVisible,
    } = this.state;
    const { canEdit, customerId, studentId, deliverTarget } = this.pageParams;
    const { customerName } = this.props.customerInfo;
    const hasOutcallWorkbrench = this.props.callCenterConfig.hasOutcallWorkbrench;
    const alreadyBind = this.props.alreadyBind;

    return (
      <Section
        title="跟进信息"
        style={{ marginTop: '20px' }}
        toolBar={
          canEdit
            ? [
                alreadyBind && hasOutcallWorkbrench ? (
                  <Button
                    type="primary"
                    key="外呼跟进"
                    icon="phone"
                    style={{ marginRight: '10px' }}
                    loading={isChecking}
                    onClick={this.followByCall}
                  >
                    外呼跟进
                  </Button>
                ) : null,
                <Button type="primary" key="添加跟进记录" icon="plus" onClick={this.switchFlowRecordModal}>
                  添加跟进记录
                </Button>,
              ]
            : null
        }
      >
        <Loading loading={loading} />
        <Tabs className={styles.customer_tab} activeKey={activeKey} onTabClick={this.onTabClick}>
          <TabPane tab="跟进记录" key="followRecrods" style={{ minHeight: '200px' }}>
            <Record
              ref={(comp) => this.getTabInstance(comp, 'followRecrods')}
              list={recordList}
              isOutCallResource={true}
              intentionDict={this.props.intentionDict}
              handleCallDetail={this.switchModalCallDetail}
              callProps={this.props.callProps}
              page="outcall"
            />
          </TabPane>
          {hasOutcallWorkbrench && (
            <TabPane tab="通话记录" key="phoneRecords" style={{ minHeight: '200px' }}>
              <CallRecord
                ref={(comp) => this.getTabInstance(comp, 'phoneRecords')}
                searchKeys={this.callRecordDefaultSKey}
                audioAuth={this.audioAuth}
              />
            </TabPane>
          )}
          <TabPane tab="客户动态" key="customerState" style={{ minHeight: '200px' }}>
            <CusNewsBlock
              ref={(comp) => this.getTabInstance(comp, 'customerState')}
              customerId={customerId}
              stuId={studentId}
              deliverTarget={deliverTarget}
            />
          </TabPane>
        </Tabs>
        {modalCallDetailVisible && (
          <ModalCallDetail data={this.callDetail} onClose={this.switchModalCallDetail} audioAuth={this.audioAuth} />
        )}
        {modalPhoneCallFollowVisible && (
          <ModalPhoneCallFollow
            isOutCallResource={true}
            data={this.phoneCallFollowData}
            onClose={this.switchModalPhoneCallFollow}
          />
        )}
        <AddFollowRecord
          visible={modalFlowRecordVisible}
          customerName={customerName}
          customerId={customerId}
          studentId={studentId}
          onCancel={this.switchFlowRecordModal}
          onSuccess={() => {
            this.setState({ modalFlowRecordVisible: false });
            this.onTabClick('followRecrods');
          }}
        />
      </Section>
    );
  }
}

export default connect((state) => {
  return {
    pageParams: state.main.pageParams || {},
    intentionDict: state.dict.customerSignType || [],
    btnAuth: state.account.btnAuths,
    callCenterConfig: state.account.callCenterConfig,
    alreadyBind: state.callCenter.alreadyBind,
  };
})(CustomerTabPanel);
