import React from 'react';
import { Modal } from 'COMPONENT';
import AddCustomerMain from '../index';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import { WORKBENCH_CALL_OUT } from 'PAGE/common/utils';

/**
 * 录入客户弹窗
 *
 * @param props.className {string}
 * @param props.title {string?} 弹窗标题
 * @param props.okText {string?} 确定按钮文字
 * @param props.cancelBtnShow {boolean?} 是否显示关闭弹窗
 * @param props.onCancel {function} 关闭弹窗回调
 * @param props.workBench {string?} 工作台
 * @param props.callType {string?}
 * @param props.customerFormItemConf {object?} 客户信息表单项配置
 * @param props.studentFormItemConf {object?} 学生信息表单项配置
 * @param props.btnAddStudentVisible {boolean?} 是否显示添加学生按钮
 * @param props.afterConfirm {function?} 新增成功后的回调
 */
class ModalAddCustomer extends React.Component {
  static defaultProps = {
    className: null,
    title: '录入客户',
    okText: '保存客户',
    cancelBtnShow: false,
    onCancel: () => {},
    workBench: WORKBENCH_CALL_OUT,
    callType: 'CONSULTANT',
    customerFormItemConf: null,
    studentFormItemConf: null,
    btnAddStudentVisible: true,
    afterConfirm: () => {},
  };

  constructor(props) {
    super(props);

    this.state = {
      loading: false,
    };
    this.modalRef = null;
    this.addCustomerMainRef = null;
  }

  toggleLoading = () => {
    this.setState((prevState) => {
      return {
        loading: !prevState.loading,
      };
    });
  };

  render() {
    const { className, title, okText, cancelBtnShow, onCancel, afterConfirm, workBench, callType } = this.props;
    const { customerFormItemConf, studentFormItemConf, btnAddStudentVisible } = this.props;

    return (
      <Modal
        className={className}
        width={820}
        title={title}
        okText={okText}
        cancelBtnShow={cancelBtnShow}
        loading={this.state.loading}
        onCancel={onCancel}
        onOk={this.addCustomerMainRef ? this.addCustomerMainRef.onOk : null}
        ref={(c) => (this.modalRef = getWrappedInstance(c))}
      >
        <AddCustomerMain
          isConsultant={true}
          workBench={workBench}
          callType={callType}
          customerFormItemConf={customerFormItemConf}
          studentFormItemConf={studentFormItemConf}
          btnAddStudentVisible={btnAddStudentVisible}
          onLoadingStart={this.toggleLoading}
          onLoadingEnd={this.toggleLoading}
          onError={this.modalRef ? this.modalRef.showOkTips : null}
          onSaveSuccess={() => {
            afterConfirm();
            onCancel();
          }}
          ref={(c) => (this.addCustomerMainRef = getWrappedInstance(c))}
        />
      </Modal>
    );
  }
}

export default ModalAddCustomer;
