import React from 'react';
import { Row, Col } from 'antd';
import { Input, Select, DatePicker, FormItem } from 'COMPONENT';
import { addCusFormCfg, addCusModalRowStyle } from '../utils';
import * as dicts from 'PAGE/common/dicts';
import SelectSchoolModal from 'PAGE/common/SelectSchoolModal';
import { loginUser, isIdCard } from 'UTIL/common';

class NewStudent extends React.Component {
  state = {
    selectSchoolModalVis: false, // {boolean} 选择学校弹框显示状态
  };

  onChange = (filedName, val) => {
    const { data, onChange } = this.props;
    onChange({ ...data, [filedName]: val });
  };

  render() {
    const {
      gradeDict,
      data: { name, grade, cls, schoolName, sex, birthday, contract, address, idCardNo },
      userProvince,
      userCity,
      canShowIdNumber,
      idNumberErrorEle,
    } = this.props;
    const { selectSchoolModalVis } = this.state;

    return (
      <Row style={addCusModalRowStyle}>
        <Col span={12}>
          <FormItem {...addCusFormCfg} name="学生姓名">
            <Input value={name} onChange={(e) => this.onChange('name', e.target.value)} />
          </FormItem>
        </Col>
        <Col span={12}>
          <FormItem {...addCusFormCfg} name="学生年级">
            <Select dict={gradeDict} value={grade} onChange={(val) => this.onChange('grade', val)} />
          </FormItem>
        </Col>
        <Col span={12} style={{ clear: 'both' }}>
          <FormItem {...addCusFormCfg} name="学生班级">
            <Input value={cls} onChange={(e) => this.onChange('cls', e.target.value)} />
          </FormItem>
        </Col>
        <Col span={12}>
          <FormItem {...addCusFormCfg} name="学生学校">
            <Input
              readOnly
              value={schoolName}
              onClick={() => {
                this.setState({ selectSchoolModalVis: true });
              }}
            />
          </FormItem>
        </Col>
        <Col span={12}>
          <FormItem {...addCusFormCfg} name="性别">
            <Select dict={dicts.sexDict} value={sex} onChange={(val) => this.onChange('sex', val)} />
          </FormItem>
        </Col>
        <Col span={12}>
          <FormItem {...addCusFormCfg} name="出生日期">
            <DatePicker value={birthday} onChange={(val) => this.onChange('birthday', val)} />
          </FormItem>
        </Col>
        <Col span={12}>
          <FormItem {...addCusFormCfg} name="联系方式">
            <Input value={contract} onChange={(e) => this.onChange('contract', e.target.value)} />
          </FormItem>
        </Col>
        <Col span={12}>
          <FormItem {...addCusFormCfg} name="家庭住址">
            <Input value={address} onChange={(e) => this.onChange('address', e.target.value)} />
          </FormItem>
        </Col>
        {loginUser.isXingHuoSym() && canShowIdNumber && (
          <Col span={12}>
            <FormItem {...addCusFormCfg} name="学生身份证">
              <Input value={idCardNo} onChange={(e) => this.onChange('idCardNo', e.target.value)} />
            </FormItem>
          </Col>
        )}

        {selectSchoolModalVis && (
          <SelectSchoolModal
            defaultProvince={userProvince}
            defaultCity={userCity}
            onConfirm={(school, schoolName) => {
              const { data, onChange } = this.props;
              onChange({ ...data, school, schoolName });
              this.setState({ selectSchoolModalVis: false });
            }}
            onClose={() => this.setState({ selectSchoolModalVis: false })}
          />
        )}
      </Row>
    );
  }
}

export default NewStudent;
