import { addOutcallCustomer, getStudentListByCusContact } from 'SERVICE/outcallWorkbench';
import { getLoginUserProvinceCity } from 'SERVICE/system';

const addCustomer = async ({
  name,
  phone,
  source,
  sourceDetail,
  remark,
  newStudents,
  cusIdCardNo,
  cusIdCardType,
  checkedLinkStudents,
}) => {
  // 录入新学生
  const studentImportVos = newStudents
    ? newStudents.map((stu) => {
        const { name, grade, cls, school, sex, birthday, contract, address, idCardNo } = stu;
        return {
          studentName: name.trim(),
          gradeId: grade,
          classes: cls,
          schoolId: school,
          sex: sex,
          bothday: birthday,
          contact: contract,
          address: address,
          idCardNo: idCardNo,
        };
      })
    : [];

  // 关联旧学生
  const oldStudents = checkedLinkStudents
    ? checkedLinkStudents.map((stu) => {
        return { studentId: stu.id, schoolId: stu.schoolId, gradeId: stu.gradeId };
      })
    : [];

  const params = {
    resourceType: '1',
    customerName: name,
    contact: phone,
    cusType: source,
    cusOrg: sourceDetail,
    idCardNo: cusIdCardNo,
    idCardType: cusIdCardType,
    remark: remark,
    studentItemDtos: [...studentImportVos, ...oldStudents],
  };

  return addOutcallCustomer(params);
};

const loadLinkStudents = async ({ phone }) => {
  try {
    const {
      data: { studentList, customerId },
    } = await getStudentListByCusContact({ contact: phone }, true);
    if (!studentList) {
      return { customerId: null, list: null };
    }
    let list = studentList.map((stu) => {
      const {
        status,
        canFollow,
        studentId,
        gradeId,
        haveLabel,
        finishDay,
        schoolId,
        gradeName,
        studentType,
        studentName,
        schoolName,
        blCampusName,
      } = stu;
      return {
        status,
        haveLabel,
        finishDay,
        studentName,
        blCampusName,
        id: studentId,
        name: studentName,
        gradeName: gradeName,
        schoolName: schoolName,
        canFollow: canFollow,
        follower: null,
        gradeId: gradeId,
        schoolId: schoolId,
        studentType: studentType,
      };
    });
    /*    list = [
          ...list,
          {id: 'xx1', name: 'xxx1', gradeName: 'dd', schoolName: 'dddd', canFollow: true},
          {id: 'xx2', name: 'xxx2', gradeName: 'dd', schoolName: 'dddd', canFollow: true},
          {id: 'xx3', name: 'xxx3', gradeName: 'dd', schoolName: 'dddd', canFollow: false},
          ]*/
    /*  const list = [
        { id: '1', name: '王甜甜', grade: '402881e647ce90330147ce94ef900006', schoolName: '培正中学' },
        { id: '2', name: '李甜甜', grade: '402881e647ce90330147ce94ef900006', schoolName: '培正中学' },
      ]*/
    return { customerId, list };
  } catch (e) {
    throw e;
  }
};

const loadLoginUserProvinceCity = async () => {
  const apiData = await getLoginUserProvinceCity();
  const data = apiData ? apiData.data : {};
  return { province: data.provinceId || null, city: data.cityId || null };
};

export default {
  addCustomer,
  loadLinkStudents,
  loadLoginUserProvinceCity,
};
