import { WORKBENCH_RECEPTION } from 'PAGE/common/utils';

const addCusFormCfg = {
  labelWidth: 100,
  style: { padding: '0 25px' },
};

const addCusModalRowStyle = {
  marginRight: -22,
};

const noop = () => {};

/**
 * 判断工作台是否为 WORKBENCH_RECEPTION
 *
 * @param workbench {string}
 * @return {boolean}
 */
const calcWorkbenchReception = function (workbench) {
  return workbench === WORKBENCH_RECEPTION;
};

// 客户信息默认表单项配置，workBench === 'CALL_OUT'
const calloutCustomerConf = {
  // 客户姓名
  name: {
    initialValue: null,
    required: true,
    visible: true,
    disabled: false,
  },
  // 客户电话
  phone: {
    initialValue: null,
    required: true,
    visible: true,
    disabled: false,
  },
  // 资源入口
  entrance: {
    initialValue: null,
    required: false,
    visible: false,
    disabled: false,
  },
  // 资源来源
  source: {
    initialValue: null,
    required: true,
    visible: true,
    disabled: false,
  },
  // 来源细分
  sourceDetail: {
    initialValue: null,
    required: true,
    visible: true,
    disabled: false,
  },
  // 意向学习中心
  campus: {
    initialValue: null,
    required: false,
    visible: false,
    disabled: false,
  },
  // 分配跟进人
  follower: {
    initialValue: null,
    required: false,
    visible: false,
    disabled: false,
  },
  // 客户意向产品
  customerSign: {
    initialValue: null,
    required: false,
    visible: false,
    disabled: false,
  },
  // 备注
  remark: {
    initialValue: null,
    required: false,
    visible: true,
    disabled: false,
  },
};

// 客户信息默认表单项配置，workBench === 'RECEPTION'
const receptionCustomerConf = {
  // 客户姓名
  name: {
    initialValue: '',
    required: true,
    visible: true,
    disabled: false,
  },
  // 客户电话
  phone: {
    initialValue: '',
    required: true,
    visible: true,
    disabled: false,
  },
  // 资源入口
  entrance: {
    initialValue: '',
    required: true,
    visible: true,
    disabled: false,
  },
  // 资源来源
  source: {
    initialValue: '',
    required: false,
    visible: true,
    disabled: false,
  },
  // 来源细分
  sourceDetail: {
    initialValue: '',
    required: false,
    visible: true,
    disabled: false,
  },
  // 意向学习中心
  campus: {
    initialValue: '',
    required: true,
    visible: true,
    disabled: false,
  },
  // 分配跟进人
  follower: {
    initialValue: '',
    required: true,
    visible: true,
    disabled: false,
  },
  // 客户意向产品
  customerSign: {
    initialValue: '',
    required: true,
    visible: true,
    disabled: false,
  },
  // 备注
  remark: {
    initialValue: '',
    required: false,
    visible: true,
    disabled: false,
  },
};

/**
 * 深度复制表单配置项
 *
 * @param origin {object} 默认配置项
 * @param extend {object} 自定义配置项
 */
function extendFormItemCfg(origin = {}, extend = {}) {
  let result = {};

  for (const i in origin) {
    if (Object.prototype.toString.call(origin[i]) === '[object Object]') {
      let item = {};

      for (const j in origin[i]) {
        item[j] = origin[i][j];
      }

      result[i] = item;
    } else {
      result[i] = origin[i];
    }
  }

  for (const key in extend) {
    if (result.hasOwnProperty(key)) {
      for (const subKey in extend[key]) {
        if (result[key].hasOwnProperty(subKey)) {
          result[key][subKey] = extend[key][subKey];
        }
      }
    }
  }

  return result;
}

/**
 * 格式化客户信息表单项配置
 *
 * @param workBench 工作台
 * @param conf 配置
 * @return {*}
 */
const formatCustomerFormItemConf = function (workBench, conf) {
  const extend = conf || {};

  if (workBench === WORKBENCH_RECEPTION) {
    return extendFormItemCfg(receptionCustomerConf, extend);
  }

  return extendFormItemCfg(calloutCustomerConf, extend);
};

/**
 * 初始化客户信息初始状态
 *
 * @param conf {object} 客户信息表单配置项
 * @return {{}}
 */
const formatCustomerInitState = function (conf = {}) {
  if (!conf) {
    return {};
  }

  let state = {};
  Object.keys(conf).forEach((key) => {
    state[key] = conf[key].initialValue !== undefined ? conf[key].initialValue : null;
  });

  return state;
};

export default {
  addCusFormCfg,
  addCusModalRowStyle,
  noop,
  calcWorkbenchReception,
  formatCustomerFormItemConf,
  formatCustomerInitState,
};
