import React from 'react';
import PropTypes from 'prop-types';
import { connect } from 'dva';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { FormItem, Button, Input, Select } from 'COMPONENT';
import { Modal, DatePicker, TimePicker } from 'antd';
import { appointmentMetting } from 'SERVICE/outcallWorkbench';
import { getCommonErrorTips } from 'COMPONENT/utils/form';

const formCfg = {
  labelWidth: 110,
};

class Appointment extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      // 跟进记录表单数据
      appointmentDetail: {
        campus: '',
        date: null,
        time: null,
      },
      checking: false,
      loading: false,
    };
    this.errors = {};
    this.transformRef = null;
  }

  static propTypes = {
    id: PropTypes.number,
    customerName: PropTypes.string,
    onSuccess: PropTypes.func,
    onCancel: PropTypes.func,
  };

  static defaultProps = {
    onSuccess: function () {},
    onCancel: function () {},
  };

  componentWillReceiveProps(props) {
    if (props.visible === false) {
      this.resetForm();
      if (this.transformRef) {
        this.transformRef.state.inputValue = '';
      }
    }
  }

  // 设置单个表单值
  setFormValue = (key, value) => {
    this.setState({
      appointmentDetail: {
        ...this.state.appointmentDetail,
        [key]: value,
      },
    });
  };

  resetForm() {
    this.setState({
      appointmentDetail: {
        campus: '',
        date: null,
        time: null,
      },
      loading: false,
      checking: false,
    });
  }

  errorChange = (key, error) => {
    this.errors[key] = error;
  };

  onSubmit = () => {
    const { campus, date, time } = this.state.appointmentDetail;
    const { id, onSuccess } = this.props;
    const errors = { ...this.errors };
    const errorTips = getCommonErrorTips(errors);

    this.setState({ checking: true });

    if (errorTips) {
      return;
    }

    if (!date || !time) {
      return showMessage('请选择日期以及时间', 'error');
    }
    let d = date.format('YYYY-MM-DD');
    let t = time.format('YYYY-MM-DD HH:mm:ss').split(' ')[1];
    let appointmentDate = new Date(`${d} ${t}`).getTime();

    let params = {
      csdId: id,
      transformToOrganizationId: campus,
      appointmentDate: appointmentDate,
    };

    this.setState({ loading: true });
    appointmentMetting(params)
      .then(() => {
        this.setState({ loading: false });
        showMessage('保存成功', 'success');
        onSuccess();
      })
      .catch(() => {
        this.setState({ loading: false });
      });
  };

  onCancel = () => {
    this.props.onCancel();
  };

  getDisabledDate = (date) => {
    let Day = new Date();
    let DayTime = `${Day.getFullYear()}-${Day.getMonth() + 1}-${Day.getDate()}`;
    let currentDate = new Date(DayTime).getTime();

    return new Date(date).getTime() < currentDate;
  };

  render() {
    const { customerName, visible, onCancel, campusList } = this.props;
    const { checking, loading } = this.state;
    const { campus, date, time } = this.state.appointmentDetail;

    return (
      <Modal
        title="预约上门"
        width={800}
        onCancel={onCancel}
        maskClosable={false}
        footer={
          <div>
            <Button type="primary" loading={loading} onClick={this.onSubmit}>
              提交
            </Button>
            <Button onClick={this.onCancel}>取消</Button>
          </div>
        }
        visible={visible}
      >
        <FormItem {...formCfg} name="客户姓名">
          <Input value={customerName} disabled />
        </FormItem>

        <FormItem {...formCfg} name="预约上门学习中心" required>
          <Select
            ref={(e) => (this.transformRef = e)}
            showSearch
            dict={campusList}
            value={campus}
            mode="multiple"
            onChange={(value) => {
              this.setFormValue('campus', value);
            }}
            validator={{
              rules: ['required'],
              silent: !checking,
              onErrorChange: (e) => this.errorChange('campus', e),
            }}
          />
        </FormItem>

        <FormItem {...formCfg} name="预约上门时间" required>
          <DatePicker
            style={{
              width: '200px',
              dispaly: 'inline-block',
              marginRight: '6px',
            }}
            size="large"
            value={date}
            placeholder="请选择预约上门日期"
            disabledDate={(val) => this.getDisabledDate(val)}
            onChange={(value) => {
              this.setFormValue('date', value);
            }}
            validator={{
              rules: ['required'],
              silent: !checking,
              onErrorChange: (e) => {
                this.errorChange('date', e);
              },
            }}
          />
          <TimePicker
            style={{ width: '200px', dispaly: 'inline-block' }}
            size="large"
            format="HH:mm"
            value={time}
            onChange={(value) => {
              this.setFormValue('time', value);
            }}
          />
        </FormItem>
      </Modal>
    );
  }
}

export default connect((state) => {
  return {
    campusList: state.dict['branchAllCampus'],
  };
})(Appointment);
