import React from 'react';
import { connect } from 'dva';
import { Menu, Dropdown, Icon } from 'antd';
import { List, TextLink } from 'COMPONENT';
import { CALL_LOGIN_STATUS_ABNORMAL } from 'CONST/callConfig';

import { getFollowList } from 'SERVICE/outcallWorkbench';

import Search from './search';
import AddFollowRecord from '../AddFollowRecord';
import Appointment from './appointment';
import InviteToGroup from './inviteToGroup';
import PhoneCall from '../PhoneCall';
import ModalPhoneCallFollow from 'PAGE/consultant/ModalPhoneCallFollow';

import styles from './styles.less';
import Loading from 'COMPONENT/Loading';
import { checkAgentBeforeCall } from 'UTIL/callCenter';

import columns from './column';
const { COL_WIDTH_XL } = List;
class FollowList extends React.Component {
  constructor(props) {
    super(props);

    this.state = Object.assign({
      loading: false, // {boolean} 内容加载状态
      list: [], // {Array} 列表数据，
      totalRecords: 0, // {number} 总记录数
      currentPage: 1, // {number} 当前页码
      pageSize: 20, // {number} 每页记录数
      searchParams: {},
      modalPhoneCallFollowVisible: false,
      showAddFollowRecord: false,
      showAppointment: false,
      showInviteToGruop: false,
      currentSelectRow: {},
    });

    this.phoneCallFollowList = [];
  }

  fetchFollowList = () => {
    this.setRequestLoading(true);
    let searchParams = this.state.searchParams;
    let pagination = {
      pageSize: this.state.pageSize,
      pageNum: this.state.currentPage,
    };

    let params = {
      ...searchParams,
      ...pagination,
      onlyMine: !this.props.isManagePage,
    };

    getFollowList(params)
      .then((res) => {
        this.setState({
          list: res.data.list || [],
          totalRecords: res.data.total,
          loading: false,
        });
      })
      .catch(() => {
        this.setRequestLoading(false);
      });
  };

  onSearch = (searchParams) => {
    this.setState(
      {
        searchParams,
      },
      this.fetchFollowList,
    );
  };

  onPaginationChange = (pageData) => {
    this.setState(
      {
        currentPage: pageData.current,
        pageSize: pageData.pageSize,
      },
      this.fetchFollowList,
    );
  };

  setRequestLoading = (loading) => {
    this.setState({
      loading,
    });
  };

  setDialogVisible = (key, row) => {
    this.setState({
      [key]: true,
      currentSelectRow: { ...row },
    });
  };

  handleResourceTransfer = (menu, row) => {
    switch (menu.key) {
      case 'appointment':
        this.setDialogVisible('showAppointment', row);
        break;
      case 'invite':
        this.setDialogVisible('showInviteToGruop', row);
        break;
    }
  };

  phoneCall = async (record, index) => {
    this.setState({ loading: true });
    const agentLoginStatus = await new Promise(this.checkAgentLoginStatus);
    this.setState({ loading: false });
    if (checkAgentBeforeCall(agentLoginStatus)) {
      this.phoneCallFollowList = this.state.list.slice(index).map((item) => {
        item.phoneNum = item.customerContact;
        return item;
      });
      this.switchModalPhoneCallFollow();
    }
  };

  checkAgentLoginStatus = (resolve) => {
    this.props.dispatch({
      type: 'callCenter/fetchToUpdAgentInfo',
      payload: {
        callback(agentObj) {
          resolve((agentObj || {}).agentLoginStatus || CALL_LOGIN_STATUS_ABNORMAL);
        },
      },
    });
  };

  switchModalPhoneCallFollow = () => {
    if (this.state.modalPhoneCallFollowVisible) {
      this.fetchFollowList();
    }
    this.setState((prevState) => {
      return {
        modalPhoneCallFollowVisible: !prevState.modalPhoneCallFollowVisible,
      };
    });
  };

  toCustomerDetail(row) {
    const detailPath = this.props.isManagePage
      ? '/OutcallCustomerFollowManage/detail'
      : '/OutcallCustomerFollow/detail';

    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path: detailPath,
        refresh: true,
        params: {
          id: row.id,
          customerId: row.customerId,
          studentId: row.studentId,
          deliverTarget: row.deliverTarget,
          resourceType: 1,
          canEdit: !this.props.isManagePage,
        },
      },
    });
  }

  render() {
    const {
      loading,
      list,
      totalRecords,
      currentPage,
      pageSize,
      showAddFollowRecord,
      showAppointment,
      showInviteToGruop,
      currentSelectRow,
    } = this.state;

    const { isManagePage } = this.props;
    const hasOutcallWorkbrench = this.props.callCenterConfig.hasOutcallWorkbrench;
    const alreadyBind = this.props.alreadyBind;

    let operation = columns.concat({
      title: '操作',
      width: COL_WIDTH_XL,
      key: 'action',
      fixed: 'right',
      render: (text, row) => {
        const detailBtn = (
          <TextLink
            className={styles.textLink}
            onClick={() => {
              this.toCustomerDetail(row);
            }}
          >
            详情
          </TextLink>
        );

        if (isManagePage) {
          return detailBtn;
        }

        return (
          <span>
            {detailBtn}

            <TextLink
              className={styles.textLink}
              onClick={() => {
                this.setDialogVisible('showAddFollowRecord', row);
              }}
            >
              跟进
            </TextLink>

            <Dropdown
              overlay={
                <Menu onClick={(e) => this.handleResourceTransfer(e, row)}>
                  <Menu.Item key="appointment">预约上门</Menu.Item>
                  <Menu.Item key="invite">邀请进群</Menu.Item>
                </Menu>
              }
              trigger={['click']}
            >
              <TextLink className={styles.textLink}>
                资源转化 <Icon type="down" />
              </TextLink>
            </Dropdown>
          </span>
        );
      },
    });

    operation.splice(2, 0, {
      title: '客户电话',
      dataIndex: 'phoneCall',
      key: 'phoneCall',
      width: 160,
      render: (text, record, index) => {
        // 管理员或者没有开启外呼功能，不显示外呼按钮
        if (isManagePage || !hasOutcallWorkbrench || !alreadyBind) {
          return String(record.customerContact).replace(/(\d{4})(\d{4})(\d{3})/, '$1****$3');
        }

        return record.customerContact ? (
          <PhoneCall
            phoneNum={record.customerContact}
            onCall={() => {
              this.setState({
                currentSelectRow: record,
              });
              this.phoneCall(record, index);
            }}
            callVisible={this.props.enabled}
          />
        ) : null;
      },
    });

    return (
      <div className={styles.wrap}>
        <AddFollowRecord
          visible={showAddFollowRecord}
          customerName={currentSelectRow.customerName}
          customerId={currentSelectRow.customerId}
          studentId={currentSelectRow.studentId}
          onCancel={() => {
            this.setState({ showAddFollowRecord: false });
          }}
          onSuccess={() => {
            this.fetchFollowList();
            this.setState({ showAddFollowRecord: false });
          }}
        />
        <Appointment
          visible={showAppointment}
          id={currentSelectRow.id}
          customerName={currentSelectRow.customerName}
          onCancel={() => {
            this.setState({ showAppointment: false });
          }}
          onSuccess={() => {
            this.fetchFollowList();
            this.setState({ showAppointment: false });
          }}
        />

        <InviteToGroup
          visible={showInviteToGruop}
          customerName={currentSelectRow.customerName}
          id={currentSelectRow.id}
          onCancel={() => {
            this.setState({ showInviteToGruop: false });
          }}
          onSuccess={() => {
            this.fetchFollowList();
            this.setState({ showInviteToGruop: false });
          }}
        />

        {this.state.modalPhoneCallFollowVisible && (
          <ModalPhoneCallFollow
            isOutCallResource={true}
            list={this.phoneCallFollowList}
            onClose={this.switchModalPhoneCallFollow}
          />
        )}

        <Search isManagePage={isManagePage} onSearch={this.onSearch} gradeDict={this.props.gradeDict} />

        <List
          dataSource={list}
          columns={operation}
          pagination={{
            current: currentPage,
            total: totalRecords,
            pageSize,
          }}
          onChange={this.onPaginationChange}
          bordered={true}
          className={styles.customList}
        />
        <Loading loading={loading} />
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    callCenterConfig: state.account.callCenterConfig,
    alreadyBind: state.callCenter.alreadyBind,
  };
};

export default connect(mapStateToProps, null, null, { withRef: true })(FollowList);
