import React from 'react';
import PropTypes from 'prop-types';
import { connect } from 'dva';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { FormItem, Button, Input, Select } from 'COMPONENT';
import { Modal } from 'antd';
import { inviteToGroup } from 'SERVICE/outcallWorkbench';
import { getCommonErrorTips } from 'COMPONENT/utils/form';
import UploadImg from 'PAGE/course/modals/UploadImg';
import styles from './styles.less';

const formCfg = {
  labelWidth: 110,
};

class InviteToGroupDialog extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      // 跟进记录表单数据
      appointmentDetail: {
        campus: '',
        fileList: null,
      },
      checking: false,
      loading: false,
    };
    this.errors = {};
    this.transformRef = null;
  }

  static propTypes = {
    id: PropTypes.number,
    customerName: PropTypes.string,
    onSuccess: PropTypes.func,
    onCancel: PropTypes.func,
  };

  static defaultProps = {
    onSuccess: function () {},
    onCancel: function () {},
  };

  componentWillReceiveProps(props) {
    if (props.visible === false) {
      this.resetForm();
      if (this.transformRef) {
        this.transformRef.state.inputValue = '';
      }
    }
  }

  onFileUpload = (file) => {
    this.setState({
      appointmentDetail: {
        ...this.state.appointmentDetail,
        fileList: file,
      },
    });
  };

  deleteComplete = () => {
    this.setState({
      appointmentDetail: {
        ...this.state.appointmentDetail,
        fileList: null,
      },
    });
  };

  // 设置单个表单值
  setFormValue = (key, value) => {
    this.setState({
      appointmentDetail: {
        ...this.state.appointmentDetail,
        [key]: value,
      },
    });
  };

  resetForm() {
    this.setState({
      appointmentDetail: {
        campus: '',
        fileList: null,
      },
      loading: false,
      checking: false,
    });
  }

  errorChange = (key, error) => {
    this.errors[key] = error;
  };

  onSubmit = () => {
    const { campus, fileList } = this.state.appointmentDetail;
    const { id, onSuccess } = this.props;
    const errors = { ...this.errors };
    const errorTips = getCommonErrorTips(errors);

    this.setState({ checking: true });

    if (errorTips) {
      return;
    }

    let params = {
      csdId: id,
      transformToOrganizationId: campus,
      transformImgUri: fileList ? [fileList.fileKey] : [],
    };
    params.transformImgUri = JSON.stringify(params.transformImgUri);

    this.setState({ loading: true });
    inviteToGroup(params)
      .then(() => {
        this.setState({ loading: false });
        showMessage('保存成功', 'success');
        onSuccess();
      })
      .catch(() => {
        this.setState({ loading: false });
      });
  };

  onCancel = () => {
    this.props.onCancel();
  };

  render() {
    const { customerName, visible, onCancel, campusList } = this.props;
    const { checking, loading } = this.state;
    const { campus, fileList } = this.state.appointmentDetail;

    return (
      <Modal
        title="邀请进群"
        width={800}
        onCancel={onCancel}
        maskClosable={false}
        footer={
          <div>
            <Button type="primary" loading={loading} onClick={this.onSubmit}>
              提交
            </Button>
            <Button onClick={this.onCancel}>取消</Button>
          </div>
        }
        visible={visible}
      >
        <FormItem {...formCfg} name="客户姓名">
          <Input value={customerName} disabled />
        </FormItem>

        <FormItem {...formCfg} name="转化学习中心" required>
          <Select
            ref={(e) => (this.transformRef = e)}
            showSearch
            className={styles.Dropdown}
            dropdownClassName={styles['transfer-campus']}
            dict={campusList}
            value={campus}
            onChange={(value) => {
              this.setFormValue('campus', value);
            }}
            validator={{
              rules: ['required'],
              silent: !checking,
              onErrorChange: (e) => this.errorChange('campus', e),
            }}
          />
        </FormItem>

        <FormItem {...formCfg} name="社群截图" className={styles.groupFile}>
          <UploadImg
            className={styles.groupFile}
            value={fileList}
            text="点击上传"
            size={100}
            uploadComplete={(val) => this.onFileUpload(val)}
            deleteComplete={(val) => this.deleteComplete(val)}
          />
        </FormItem>
      </Modal>
    );
  }
}

export default connect((state) => {
  return {
    campusList: state.dict['branchAllCampus'],
  };
})(InviteToGroupDialog);
