import React from 'react';
import { connect } from 'dva';
import { List } from 'COMPONENT';
import Loading from 'COMPONENT/Loading';
import { getInvalidList } from 'SERVICE/outcallWorkbench';
import Search from './search';
import columns from './column';
class InvalidList extends React.Component {
  constructor(props) {
    super(props);

    this.state = Object.assign({
      loading: false, // {boolean} 内容加载状态
      list: [], // {Array} 列表数据，
      totalRecords: 0, // {number} 总记录数
      currentPage: 1, // {number} 当前页码
      pageSize: 20, // {number} 每页记录数
      searchParams: {},
    });
  }

  fetchFollowList() {
    this.setRequestLoading(true);

    let searchParams = this.state.searchParams;
    let pagination = {
      pageSize: this.state.pageSize,
      pageNum: this.state.currentPage,
    };

    let params = {
      ...searchParams,
      ...pagination,
      onlyMine: !this.props.isManagePage,
    };

    getInvalidList(params)
      .then((res) => {
        this.setState({
          list: res.data.list || [],
          totalRecords: res.data.total,
          loading: false,
        });
      })
      .catch(() => {
        this.setRequestLoading(false);
      });
  }

  onSearch = (searchParams) => {
    this.setState(
      {
        searchParams,
      },
      this.fetchFollowList,
    );
  };

  onPaginationChange = (pageData) => {
    this.setState(
      {
        currentPage: pageData.current,
        pageSize: pageData.pageSize,
      },
      this.fetchFollowList,
    );
  };

  setRequestLoading = (loading) => {
    this.setState({
      loading,
    });
  };

  setDialogVisible = (key, row) => {
    this.setState({
      [key]: true,
      currentSelectRow: { ...row },
    });
  };

  render() {
    const { loading, list, totalRecords, currentPage, pageSize } = this.state;

    return (
      <div>
        <Search onSearch={this.onSearch} isManagePage={this.props.isManagePage} gradeDict={this.props.gradeDict} />

        <List
          dataSource={list}
          columns={columns}
          pagination={{
            current: currentPage,
            total: totalRecords,
            pageSize,
          }}
          onChange={this.onPaginationChange}
          bordered={true}
        />
        <Loading loading={loading} />
      </div>
    );
  }
}

export default connect(null, null, null, { withRef: true })(InvalidList);
