import React from 'react';
import { connect } from 'dva';
import { SearchBar, Input, Select } from 'COMPONENT';
import moment from 'moment';
import { resizeTimerange } from '../utils';
import { ONE_DAY_MILLSECONDS } from '../dicts';
import { DatePicker } from 'antd';
import { loadSourceDetailDict } from 'PAGE/common/loaders';
const { RangePicker } = DatePicker;
const startTime = moment().format('YYYY-MM-DD');

const getSearchInitValue = (isInit) => {
  return {
    studentName: null, // {string} 查询条件 学生姓名
    customerName: null, // {string} 查询条件 客户姓名
    customerContact: null, // {string} 查询条件 电话
    gradeId: null, // {string} 查询条件 年级
    resEntrance: null, // {string} 查询条件 资源入口
    invalidTIme: isInit ? [moment(startTime), moment(startTime)] : null,
    cusOrg: null,
    cusType: null,
  };
};

class Search extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false, // {boolean} 内容加载状态
      searchExpand: false, // {boolean} 搜索栏展开状态
      searchParams: getSearchInitValue(true),
      sourceDetailDict: [],
    };
  }

  componentDidMount() {
    resizeTimerange();
    this.props.onSearch(this.formatParams(this.state.searchParams));
  }

  formatTime = (momentTime) => {
    return Number(momentTime.format('x'));
  };

  formatParams = (params) => {
    return Object.keys(params).reduce((obj, key) => {
      if (params[key]) {
        switch (key) {
          case 'invalidTIme':
            if (params[key].length > 0) {
              obj.beginCreateTime = this.formatTime(params[key][0].startOf('day'));
              obj.endCreateTime = this.formatTime(params[key][1].endOf('day'));
            }
            break;
          default:
            obj[key] = params[key];
        }
      }

      return obj;
    }, {});
  };

  setSearchParams = (key, value) => {
    let extraParam = {};
    if (key == 'cusType') {
      extraParam.cusOrg = '';
    }
    this.setState({
      searchParams: {
        ...this.state.searchParams,
        [key]: value,
        ...extraParam,
      },
    });
  };

  resetSearchParams = () => {
    this.setState({
      searchParams: getSearchInitValue(),
      sourceDetailDict: [],
    });
  };
  render() {
    const { searchExpand, searchParams } = this.state;

    const {
      customerName,
      studentName,
      customerContact,
      gradeId,
      resEntrance,
      invalidTIme,
      deliverTarget,
      organizationId,
      cusOrg,
      cusType,
    } = searchParams;

    const { /*STUDENT_GRADE,*/ RES_ENTRANCE, COMPANY, OUTCALL_USER, isManagePage } = this.props;

    return (
      <SearchBar
        expand={searchExpand}
        onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
        onSearch={() => this.props.onSearch(this.formatParams(searchParams))}
        onReset={this.resetSearchParams}
      >
        <RangePicker
          className="outcall-time-range-search"
          size="large"
          value={invalidTIme}
          placeholder={['无效时间(起)', '无效时间(止)']}
          onChange={(value) => {
            this.setSearchParams('invalidTIme', value);
          }}
        />

        {isManagePage && (
          <Select
            placeholder="所属分公司"
            showSearch
            dict={COMPANY}
            value={organizationId}
            onChange={(value) => {
              this.setSearchParams('organizationId', value);
            }}
          />
        )}

        {isManagePage && (
          <Select
            placeholder="外呼跟进人"
            showSearch
            dict={OUTCALL_USER}
            value={deliverTarget}
            onChange={(value) => {
              this.setSearchParams('deliverTarget', value);
            }}
          />
        )}

        <Input
          placeholder="客户姓名"
          value={customerName}
          onChange={(e) => {
            this.setSearchParams('customerName', e.target.value);
          }}
        />
        <Input
          placeholder="学生姓名"
          value={studentName}
          onChange={(e) => {
            this.setSearchParams('studentName', e.target.value);
          }}
        />

        <Input
          placeholder="客户电话"
          value={customerContact}
          onChange={(e) => {
            this.setSearchParams('customerContact', e.target.value);
          }}
        />
        <Select
          placeholder="年级"
          dict={this.props.gradeDict}
          value={gradeId}
          onChange={(value) => {
            this.setSearchParams('gradeId', value);
          }}
        />
        <Select
          placeholder="资源入口"
          dict={RES_ENTRANCE}
          value={resEntrance}
          onChange={(value) => {
            this.setSearchParams('resEntrance', value);
          }}
        />
        <Select
          value={cusType}
          placeholder="资源来源"
          noOptionTips="暂无数据"
          dict={this.props.resTypeDict}
          onChange={(val) => {
            this.setSearchParams('cusType', val);
            if (val) {
              loadSourceDetailDict(val).then((dict) => {
                this.setState({
                  sourceDetailDict: dict,
                });
              });
            } else {
              this.setState({
                sourceDetailDict: [],
              });
            }
          }}
        />
        <Select
          value={cusOrg}
          placeholder="资源细分"
          noOptionTips="暂无数据"
          dict={this.state.sourceDetailDict}
          onChange={(value) => {
            this.setSearchParams('cusOrg', value);
          }}
        />
      </SearchBar>
    );
  }
}

export default connect((state) => {
  return {
    // STUDENT_GRADE: state.dict["grade"],
    RES_ENTRANCE: state.dict['resEntrance'],
    OUTCALL_USER: state.dict['outcallUser'],
    COMPANY: state.dict['branch'],
    resTypeDict: state.dict.resType,
  };
})(Search);
