import React from 'react';
import { connect } from 'dva';
import { List, Modal, PreviewImgs } from 'COMPONENT';
import Loading from 'COMPONENT/Loading';
import { getTransferList } from 'SERVICE/outcallWorkbench';
import Search from './search';
import columns from './column';
class TransferList extends React.Component {
  constructor(props) {
    super(props);

    this.state = Object.assign({
      loading: false, // {boolean} 内容加载状态
      list: [], // {Array} 列表数据，
      totalRecords: 0, // {number} 总记录数
      currentPage: 1, // {number} 当前页码
      pageSize: 20, // {number} 每页记录数
      searchParams: {},
      currentImg: '',
      isPreviewImg: false,
    });
  }

  fetchFollowList() {
    this.setRequestLoading(true);

    let searchParams = this.state.searchParams;
    let pagination = {
      pageSize: this.state.pageSize,
      pageNum: this.state.currentPage,
    };

    let params = {
      ...searchParams,
      ...pagination,
      onlyMine: !this.props.isManagePage,
    };

    getTransferList(params)
      .then((res) => {
        this.setState({
          list: res.data.list || [],
          totalRecords: res.data.total,
          loading: false,
        });
      })
      .catch(() => {
        this.setRequestLoading(false);
      });
  }

  onSearch = (searchParams) => {
    this.setState(
      {
        searchParams,
      },
      this.fetchFollowList,
    );
  };

  onPaginationChange = (pageData) => {
    this.setState(
      {
        currentPage: pageData.current,
        pageSize: pageData.pageSize,
      },
      this.fetchFollowList,
    );
  };

  setRequestLoading = (loading) => {
    this.setState({
      loading,
    });
  };

  setDialogVisible = (key, row) => {
    this.setState({
      [key]: true,
      currentSelectRow: { ...row },
    });
  };

  render() {
    const { loading, list, totalRecords, currentPage, pageSize, isPreviewImg, currentImg } = this.state;
    const listColumn = columns.slice(0);

    const imgColumn = {
      title: '转化截图',
      dataIndex: 'transformImgUris',
      key: 'transformImgUris',
      width: 150,
      render: (text) => {
        return text ? (
          <div style={{ width: '100%' }}>
            <img
              alt="转化截图"
              style={{ maxWidth: '100%', height: '100px', objectFit: 'scale-down' }}
              onClick={() => {
                this.setState({ isPreviewImg: true, currentImg: text });
              }}
              src={text}
            />
          </div>
        ) : (
          ''
        );
      },
    };

    listColumn.splice(4, 0, imgColumn);

    return (
      <div>
        <Search onSearch={this.onSearch} isManagePage={this.props.isManagePage} gradeDict={this.props.gradeDict} />

        <List
          dataSource={list}
          columns={listColumn}
          pagination={{
            current: currentPage,
            total: totalRecords,
            pageSize,
          }}
          onChange={this.onPaginationChange}
          bordered={true}
        />
        <Loading loading={loading} />

        {isPreviewImg && (
          <Modal
            width={800}
            okBtnShow={false}
            cancelBtnShow={false}
            closable={true}
            onClose={() => {
              this.setState({ isPreviewImg: false });
            }}
          >
            <PreviewImgs list={[currentImg]} activeSrc={currentImg} showThumbnail={false} />
          </Modal>
        )}
      </div>
    );
  }
}

export default connect(null, null, null, { withRef: true })(TransferList);
