import React from 'react';
import { connect } from 'dva';

import FollowList from './FollowList';
import TransferList from './TransferList';
import LossList from './LossList';
import InvalidList from './InvalidList';

import { Page, Button, Loading } from 'COMPONENT';

import { cusListDict } from './dicts';
import AddCusModal from './AddCusModal';
import { Tabs } from 'antd';
import { loadCommonDictNew } from 'UTIL/dict';
import getWrappedInstance from 'UTIL/getWrappedInstance';

const FOLLOW = '1',
  TRANSFER = '2',
  LOSS = '3',
  INVALID = '4';

const TabPane = Tabs.TabPane;

class CustomerFollow extends React.Component {
  constructor(props) {
    super(props);

    this.currentTabRef = null;

    this.state = {
      loading: false,
      gradeDict: [],
      activeCusList: FOLLOW, // 当前激活状态的客户列表
      showAddCustomerDialog: false, // 录入客户弹框显示状态
      searchingState: {},
      currentTabRef: null,
    };
  }

  componentDidMount() {
    this.initDictionary();
    GlobalEvent.on('CrmNotification:followCustomerFromDetail', () => {
      this.currentTabRef.fetchFollowList();
    });
  }

  initDictionary() {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: [
          'branch',
          /*"grade",*/
          'campus',
          'resEntrance',
          'invalid_sign',
          'branchAllCampus',
          'customerIntention',
          'outcallUser',
          'resType',
        ],
      },
    });
    loadCommonDictNew('STUDENT_GRADE').then((gradeDict) => {
      this.setState({ gradeDict });
    });
  }

  changeCusList = (type) => {
    this.setState({ loading: false, activeCusList: type });
  };

  onAddCustomerSuccess = () => {
    if (this.currentTabRef && this.currentTabRef.fetchFollowList) {
      this.currentTabRef.fetchFollowList();
    }
  };

  getCusListToolBar = () => {
    return (
      <Button iconType="plus" onClick={() => this.setState({ showAddCustomerDialog: true })}>
        录入客户
      </Button>
    );
  };

  getTabContent = (type) => {
    const isManagePage = this.props.isManagePage;

    switch (type) {
      case FOLLOW:
        return (
          <FollowList
            gradeDict={this.state.gradeDict}
            ref={(r) => (this.currentTabRef = getWrappedInstance(r))}
            isManagePage={isManagePage}
          />
        );

      case TRANSFER:
        return (
          <TransferList
            gradeDict={this.state.gradeDict}
            ref={(r) => (this.currentTabRef = getWrappedInstance(r))}
            isManagePage={isManagePage}
          />
        );

      case LOSS:
        return (
          <LossList
            gradeDict={this.state.gradeDict}
            ref={(r) => (this.currentTabRef = getWrappedInstance(r))}
            isManagePage={isManagePage}
          />
        );

      case INVALID:
        return (
          <InvalidList
            gradeDict={this.state.gradeDict}
            ref={(r) => (this.currentTabRef = getWrappedInstance(r))}
            isManagePage={isManagePage}
          />
        );
    }
  };

  render() {
    const { activeCusList, showAddCustomerDialog, loading } = this.state;
    const { isManagePage, hasRecordPermisison } = this.props;

    return (
      <Page title={null} route={this.props.route}>
        <div>
          <div className="Section-body custom-tabs">
            <Loading loading={loading} />
            <Tabs
              tabBarExtraContent={isManagePage || !hasRecordPermisison ? null : this.getCusListToolBar()}
              onChange={this.changeCusList}
              activeKey={activeCusList}
              tabBarStyle={{ border: 'none', margin: '25px 0' }}
            >
              {cusListDict.map((c) => {
                return (
                  <TabPane key={c.val} tab={c.text} forceRender={true}>
                    {activeCusList === c.val ? this.getTabContent(c.val) : null}
                  </TabPane>
                );
              })}
            </Tabs>
          </div>
          {showAddCustomerDialog && (
            <AddCusModal
              onSaveSuccess={this.onAddCustomerSuccess}
              onClose={() => this.setState({ showAddCustomerDialog: false })}
            />
          )}
        </div>
      </Page>
    );
  }
}

function mapStateToProps(state) {
  const { btnAuths } = state.account;
  const hasRecordPermisison = btnAuths.indexOf('OUTCALL_FOLLOW_ADD_CUSTOMER') > -1;

  return {
    hasRecordPermisison,
  };
}

export default connect(mapStateToProps)(CustomerFollow);
