import React from 'react';
import Modal from 'COMPONENT/Modal';
import noop from 'lodash/noop';
import FormItem from 'COMPONENT/FormItem';
import { TreeSelect } from 'antd';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import { outCallResourcePools } from 'SERVICE/outcallWorkbench';

function resolveTree(list = [], parentId = '') {
  if (list.length === 0) return [];
  const parents = [];
  const remains = [];
  list.forEach((item) => {
    if (item.parentId === parentId) {
      parents.push(item);
    } else {
      remains.push(item);
    }
  });
  parents.forEach((parent) => {
    parent.children = resolveTree(remains, parent.nodeId);
  });
  return parents;
}
class DistrbuteResourceModal extends React.Component {
  static defaultProps = {
    tipText: '',
    onClose: noop,
    onSave: noop,
  };

  constructor(props) {
    super(props);
    this.modal = null;
    this.state = {
      userId: '',
      user: null,
      treeDict: [],
      showDistributeDetailModal: false,
    };
  }

  onCommit = () => {
    if (!this.state.userId) {
      return this.modal.showOkTips(`请选择分配对象`);
    }
    this.props.onSave(this.state.user);
  };

  onClose = (reload = false) => {
    this.setState({ showDistributeDetailModal: false });
    this.props.onClose(reload);
  };

  componentDidMount() {
    this.getDistributePoolData();
  }

  getDistributePoolData = async () => {
    try {
      const { data } = await outCallResourcePools({
        /* 外呼资源池树 */ resourceType: 1,
        withUser: true,
        withCapacity: true,
      });
      // get nodeIds map
      const nodeIdsMap = {};
      for (let node of data) {
        const { nodeId } = node;
        nodeIdsMap[nodeId] = node;
      }

      // get distributableNodes' parents
      const parents = new Set();
      const distributableNodes = data.filter((item) => item.distributable);
      for (let { parentId } of distributableNodes) {
        if (nodeIdsMap[parentId]) {
          parents.add(nodeIdsMap[parentId]);
        }
      }

      // remove duplicate
      const nodesSet = new Set(distributableNodes.concat([...parents]));
      const nodes = [...nodesSet].map((r) => ({
        ...r,
        value: r.nodeId,
        label: `${r.nodeName}（${r.usedCapacity || 0}/${r.capacity || 0}）`,
        key: r.nodeId,
        disabled: r.nodeType !== 2 /** 只能分配给个人 */,
      }));
      const rootNode = nodes.find((node) => node.orgType === 'GROUNP');
      if (rootNode) {
        rootNode.children = resolveTree(nodes, rootNode.nodeId);
        this.setState({ treeDict: [rootNode] });
      }
    } catch (e) {
      throw e;
    }
  };

  onSelectChange = (value, node) => {
    const { capacity = 0, usedCapacity = 0 } = node.props;
    this.setState({ userId: value, user: { capacity, usedCapacity, val: value } });
  };

  render() {
    const { userId, treeDict, loading } = this.state;

    return (
      <Modal
        loading={loading}
        title="资源分配"
        onCancel={this.onClose}
        okText="保存"
        onOk={this.onCommit}
        ref={(c) => (this.modal = getWrappedInstance(c))}
        width={650}
      >
        {this.props.tipText && <p style={{ padding: '0 50px 20px' }}>{this.props.tipText}</p>}
        <FormItem name="分配对象" required="true">
          <TreeSelect
            size="large"
            placeholder="请选择分配对象"
            treeData={treeDict}
            value={userId}
            allowClear
            showSearch
            treeNodeFilterProp="label"
            treeDefaultExpandAll
            onSelect={this.onSelectChange}
            dropdownStyle={{ maxHeight: 400, overflow: 'auto' }}
          />
        </FormItem>
      </Modal>
    );
  }
}

export default DistrbuteResourceModal;
