import React from 'react';
import Modal from 'COMPONENT/Modal';
import { Row, Col } from 'antd';
import FormItem from 'COMPONENT/FormItem';
import Select from 'COMPONENT/Select';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { updateHeaderMaster } from 'SERVICE/personnelManage';
import { loadBTeacherDict, formatDictValue, sexDict, statusDict, workTypeDict } from '../dicts';

const labelWidth = 80;

export default class TeacherEditModal extends React.Component {
  static defaultProps = {
    row: {},
    onClose: () => {},
    refreshList: () => {},
  };

  constructor(props) {
    super(props);

    this.state = {
      BTeacherDict: [], // 班主任下拉
      reUserId: '', // B角色班主任ID
    };

    this.initPage = this.initPage.bind(this);
    this.onCommit = this.onCommit.bind(this);
    this.formatParam = this.formatParam.bind(this);
  }

  componentDidMount() {
    this.initPage();
  }

  initPage() {
    const { row } = this.props;
    const userId = row.userId || '';

    this.setState({
      reUserId: row.reUserId || '',
    });

    Promise.all([loadBTeacherDict(userId)]).then(([BTeacherDict]) => {
      this.setState({
        BTeacherDict,
      });
    });
  }

  async onCommit() {
    let param = this.formatParam();

    /*if (!param.reUserId) {
      showMessage('请选择B角班主任！', 'warning');
      return ;
    }*/

    await updateHeaderMaster(param);

    showMessage('修改成功', 'success');
    this.props.onClose();
    this.props.refreshList();
  }

  formatParam() {
    const { row } = this.props;
    let { reUserId } = this.state;
    let stateRow = { reUserId };

    return { ...row, ...stateRow };
  }

  render() {
    const { row, onClose } = this.props;
    const { BTeacherDict, reUserId } = this.state;

    return (
      <Modal
        loading={false}
        onCancel={onClose}
        title="编辑"
        width={960}
        cancelText="关闭"
        okBtnShow={true}
        onOk={this.onCommit}
        okBtnDisabled={false}
      >
        <Row gutter={10}>
          <Col span={8}>
            <FormItem name="工号" labelWidth={labelWidth}>
              {row.employeeNo}
            </FormItem>
          </Col>
          <Col span={8}>
            <FormItem name="姓名" labelWidth={labelWidth}>
              {row.userName}
            </FormItem>
          </Col>
          <Col span={8}>
            <FormItem name="性别" labelWidth={labelWidth}>
              {formatDictValue(row.userSex, sexDict)}
            </FormItem>
          </Col>

          <Col span={8}>
            <FormItem name="联系方式" labelWidth={labelWidth}>
              {row.userContact}
            </FormItem>
          </Col>
          <Col span={8}>
            <FormItem name="状态" labelWidth={labelWidth}>
              {formatDictValue(row.userEnableFlag, statusDict)}
            </FormItem>
          </Col>
          <Col span={8}>
            <FormItem name="用户类型" labelWidth={labelWidth}>
              {formatDictValue(row.workType, workTypeDict)}
            </FormItem>
          </Col>

          <Col span={8}>
            <FormItem name="B角班主任" labelWidth={labelWidth}>
              <Select
                placeholder="B角班主任"
                showSearch={true}
                dict={BTeacherDict}
                value={reUserId}
                onChange={(val) => {
                  this.setState({ reUserId: val });
                }}
              />
            </FormItem>
          </Col>
        </Row>
      </Modal>
    );
  }
}
