import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import SearchBar from 'COMPONENT/SearchBar';
import List from 'COMPONENT/List';
import Select from 'COMPONENT/Select';
import Input from 'COMPONENT/Input';
import { getHeaderMasterList } from 'SERVICE/personnelManage';
import { statusDict, sexDict, workTypeDict, loadBTeacherDict, formatDictValue } from './dicts';
import TeacherEditModal from './TeacherEditModal';

const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;

/**
 * 学习中心班主任管理
 */
class HeadMasterManage extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false,

      BTeacherDict: [], // 字典 B角班主任列表

      employeeNo: '', // 员工号
      name: '', // 姓名
      contact: '', // 联系方式
      enableFlag: '', // 状态
      bTeacherId: '', // B角班主任ID
      relationUserId: '', // 使用其他角色权限时传入相应userId

      listLoading: false,
      list: [], // 列表数据
      totalRecords: 0, // 总记录数
      pageNo: 1, // 当前页码
      pageSize: 20, // 每页记录数

      showEditModal: false, // 显示编辑弹窗
      selectedRow: null, // 选中的行
    };

    this.initSearchParam = {
      employeeNo: '', // 员工号
      name: '', // 姓名
      contact: '', // 联系方式
      enableFlag: '', // 状态
      bTeacherId: '', // B角班主任ID
      relationUserId: '', // 使用其他角色权限时传入相应userId
    };

    this.initPage = this.initPage.bind(this);
    this.onSearch = this.onSearch.bind(this);
    this.onReset = this.onReset.bind(this);
    this.onEdit = this.onEdit.bind(this);
    this.onBTeacherChange = this.onBTeacherChange.bind(this);
  }

  componentDidMount() {
    this.initPage();
  }

  initPage() {
    const { userId } = this.props;

    this.setState({ loading: true });

    Promise.all([loadBTeacherDict(userId)]).then(([BTeacherDict]) => {
      this.setState({
        loading: false,
        BTeacherDict,
      });

      this.onSearch();
    });
  }

  onSearch() {
    this.loadList({ pageNo: 1 });
  }

  onReset() {
    this.setState(this.initSearchParam);
  }

  async loadList(changedParams = {}) {
    const userId = this.props.userId || '';
    const { employeeNo, name, contact, enableFlag, bTeacherId, relationUserId, pageNo, pageSize } = this.state;
    const tempParam = {
      userId,
      employeeNo,
      name,
      contact,
      enableFlag,
      bTeacherId,
      relationUserId,
      pageNo,
      pageSize,
    };
    const param = { ...tempParam, ...changedParams };

    // 修改param参数
    param.page = param.pageNo;
    param.rows = param.pageSize;

    this.setState({ listLoading: true });

    const apiData = await getHeaderMasterList(param);
    const { rowCount, datas } = apiData;
    this.setState({
      listLoading: false,
      list: datas || [],
      totalRecords: rowCount || 0,
      pageNo: param.pageNo,
      pageSize: param.pageSize,
    });
  }

  onEdit(row, index) {
    this.setState({ showEditModal: true, selectedRow: row });
  }

  onBTeacherChange(value) {
    this.setState({ bTeacherId: value });
  }

  render() {
    const { loading, employeeNo, name, contact, bTeacherId, enableFlag } = this.state;
    const { BTeacherDict } = this.state;
    const { listLoading, list, pageNo, totalRecords, pageSize } = this.state;
    const { showEditModal, selectedRow } = this.state;

    const columns = [
      {
        title: '序号',
        dataIndex: 'index',
        key: 'index',
        width: COL_WIDTH_S,
        render: (text, record, index) => {
          return index + 1;
        },
      },
      { title: '工号', dataIndex: 'employeeNo', key: 'employeeNo', width: COL_WIDTH_M },
      { title: '姓名', dataIndex: 'userName', key: 'userName', width: COL_WIDTH_M },
      {
        title: '性别',
        dataIndex: 'userSex',
        key: 'userSex',
        width: COL_WIDTH_M,
        render: (text, record) => {
          return formatDictValue(text, sexDict);
        },
      },
      { title: '联系方式', dataIndex: 'userContact', key: 'userContact', width: COL_WIDTH_M },
      {
        title: '状态',
        dataIndex: 'userEnableFlag',
        key: 'userEnableFlag',
        width: COL_WIDTH_XL,
        render: (text, record) => {
          return formatDictValue(text, statusDict);
        },
      },
      {
        title: '用户类型',
        dataIndex: 'workType',
        key: 'workType',
        width: COL_WIDTH_M,
        render: (text, record) => {
          return formatDictValue(text, workTypeDict);
        },
      },
      { title: 'B角班主任', dataIndex: 'reUserName', key: 'reUserName', width: COL_WIDTH_XL },
      {
        title: '操作',
        key: 'action',
        width: COL_WIDTH_XL,
        fixed: 'right',
        getActionConfig: (text, record, index) => {
          return [{ text: '修改', onClick: () => this.onEdit(record, index), disabled: this.props.btnAuth_edit }];
        },
      },
    ];

    return (
      <Page
        fitWindowHeight={true}
        loading={loading}
        route={this.props.route}
        subHeader={
          <SearchBar onSearch={this.onSearch} onReset={this.onReset}>
            <Input
              placeholder="工号"
              value={employeeNo}
              onChange={(e) => this.setState({ employeeNo: e.target.value })}
            />
            <Input placeholder="姓名" value={name} onChange={(e) => this.setState({ name: e.target.value })} />
            <Input
              placeholder="联系方式"
              value={contact}
              onChange={(e) => this.setState({ contact: e.target.value })}
            />
            {/*<Select
            placeholder="状态"
            showSearch={true}
            dict={statusDict}
            value={enableFlag}
            onChange={val => {this.setState({enableFlag: val})}} />*/}
            <Select
              placeholder="B角班主任"
              showSearch={true}
              dict={BTeacherDict}
              value={bTeacherId}
              onChange={(val) => this.onBTeacherChange(val)}
            />
          </SearchBar>
        }
      >
        <List
          dataSource={list}
          columns={columns}
          loading={!loading && listLoading}
          pagination={{
            current: pageNo,
            total: totalRecords,
            pageSize: pageSize,
          }}
          onChange={({ current, pageSize }) => this.loadList({ pageNo: current, pageSize: pageSize })}
        />
        {showEditModal && (
          <TeacherEditModal
            row={selectedRow}
            onClose={() => this.setState({ showEditModal: false })}
            refreshList={() => this.loadList()}
          />
        )}
      </Page>
    );
  }
}

function mapStateToProps(state) {
  const { userId } = state.account.user;
  const btnAuth = state.account.btnAuths;
  let btnAuth_edit = btnAuth.indexOf('BTN_HEAD_MASTER_MANAGE_EDIT') < 0;

  return {
    userId: userId,
    btnAuth_edit,
  };
}

export default connect(mapStateToProps)(HeadMasterManage);
