import React from 'react';
import { connect } from 'dva';
import { Row, Col, message, Form } from 'antd';
import { Page, Input, Button } from 'COMPONENT';
import { httpGet } from 'UTIL/http';
import * as Api from 'SERVICE/personnelManage';
import styles from './style.less';
import { QUARTER_SPRING, QUARTER_SUMMER, QUARTER_AUTUMN, QUARTER_WINTER } from 'PAGE/common/utils.js';
const FormItem = Form.Item;
const ItemCol = {
  labelCol: {
    span: 6,
  },
  wrapperCol: {
    span: 18,
  },
};
class LeadClassRuleAdd extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false,
      isDetail: false,
      editStatus: {
        // 详情每期带班数默认不可编辑
        spring: false,
        autumn: false,
        winter1: false,
        winter2: false,
        winter3: false,
        winter4: false,
        winter5: false,
        winter6: false,
        summer1: false,
        summer2: false,
        summer3: false,
        summer4: false,
        summer5: false,
        summer6: false,
      },
      initialData: {},
    };

    this.onSubmit = this.onSubmit.bind(this);
    this.getDetail = this.getDetail.bind(this);
  }
  onSubmit(e) {
    e.preventDefault();
    const mcvs = [];
    const { dispatch, params } = this.props;
    this.props.form.validateFields(async (err, form) => {
      if (!err) {
        let status = true;
        // 春秋季带班数
        mcvs.push({
          termNumber: 1,
          maxClassNumber: parseInt(form['spring']),
          quarterId: QUARTER_SPRING,
          salaryCoefficient: parseFloat(form.salaryCoefficient),
          yearId: 'DAT000000000623', // 暂时写死
        });
        mcvs.push({
          termNumber: 1,
          maxClassNumber: parseInt(form['autumn']),
          quarterId: QUARTER_AUTUMN,
          salaryCoefficient: parseFloat(form.salaryCoefficient),
          yearId: 'DAT000000000623', // 暂时写死
        });
        // 校验五期带班数是否间隔填入/非递增填入
        for (let prev = 1, next = 2; prev <= 6; prev++, next++) {
          if ((form['winter' + prev] == '' || form['winter' + prev] == undefined) && form['winter' + next]) {
            message.warning(`请输入寒假第 ${prev} 期最大带班数`);
            status = false;
            break;
          }
          if (parseInt(form['winter' + prev]) > parseInt(form['winter' + next])) {
            message.warning(`寒假第 ${prev} 期带班数不能多于第 ${next} 期带班数`);
            status = false;
            break;
          }
          // 寒假带班数规则校验通过，预先处理数据格式
          if (form['winter' + prev]) {
            mcvs.push({
              termNumber: prev,
              maxClassNumber: parseInt(form['winter' + prev]),
              quarterId: QUARTER_WINTER,
              salaryCoefficient: parseFloat(form.salaryCoefficient),
              yearId: 'DAT000000000623', // 暂时写死
            });
          }
          if ((form['summer' + prev] == '' || form['summer' + prev] == undefined) && form['summer' + next]) {
            message.warning(`请输入暑假第 ${prev} 期最大带班数`);
            status = false;
            break;
          }
          if (parseInt(form['summer' + prev]) > parseInt(form['summer' + next])) {
            message.warning(`暑假第 ${prev} 期带班数不能多于第 ${next} 期带班数`);
            status = false;
            break;
          }
          // 暑假带班数规则校验通过，预先处理数据格式
          if (form['summer' + prev]) {
            mcvs.push({
              termNumber: prev,
              maxClassNumber: parseInt(form['summer' + prev]),
              quarterId: QUARTER_SUMMER,
              salaryCoefficient: parseFloat(form.salaryCoefficient),
              yearId: 'DAT000000000623', // 暂时写死
            });
          }
        }
        // 根据id判断是新增还是修改
        if (status && params.id) {
          const { resultCode } = await Api.updateLimitRule({
            type: 1,
            ruleId: params.id,
            objectId: params.objectId,
            parentId: params.parentId,
            name: form.name,
            salaryCoefficient: parseFloat(form.salaryCoefficient),
            mcNumberLimitRuleDetailSaveDtos: mcvs,
          });
          if (resultCode === 0) {
            message.success('修改成功!');
            dispatch({
              type: 'main/removeTab',
              payload: {
                id: '/leadClassRules/add',
              },
            });
            dispatch({
              type: 'main/refreshTab',
              payload: {
                id: '/leadClassRules',
              },
            });
            this.props.history.goBack();
          }
        } else if (status && !params.id) {
          const { resultCode } = await Api.addLimitRule({
            type: 1,
            objectId: '000001', // 暂时写死
            name: form.name,
            salaryCoefficient: form.salaryCoefficient,
            mcNumberLimitRuleDetailSaveDtos: mcvs,
          });
          if (resultCode === 0) {
            message.success('添加成功!');
            dispatch({
              type: 'main/removeTab',
              payload: {
                id: '/leadClassRules/add',
              },
            });
            dispatch({
              type: 'main/refreshTab',
              payload: {
                id: '/leadClassRules',
              },
            });
            this.props.history.goBack();
          }
        }
      }
    });
  }
  componentDidMount() {
    const { params } = this.props;
    if (params && params.id) {
      // 编辑进来
      this.setState({
        isDetail: true,
      });
      this.getDetail(params);
    }
  }
  // 根据 id 拿详细信息
  async getDetail(params) {
    const { resultCode, data } = await httpGet({
      url: `boss-xinghuo-api/miniClass/numberLimitRule/${params.id}`,
    });
    if (resultCode == 0 && data) {
      // 季度数据，方便展示
      if (data.mcNumberLimitRuleDetailVos) {
        // 新增自定义各期字段，方便展示
        data.mcNumberLimitRuleDetailVos.forEach((mvo) => {
          if (mvo.quarterId === QUARTER_SPRING) {
            data['spring'] = mvo.maxClassNumber;
          }
          if (mvo.quarterId === QUARTER_AUTUMN) {
            data['autumn'] = mvo.maxClassNumber;
          }

          if (mvo.quarterId === QUARTER_WINTER) {
            data['winter' + mvo.termNumber] = mvo.maxClassNumber;
          }

          if (mvo.quarterId === QUARTER_SUMMER) {
            data['summer' + mvo.termNumber] = mvo.maxClassNumber;
          }
        });
      }
      this.setState({
        initialData: data,
      });
    }
  }

  goBack() {
    this.props.dispatch({
      type: 'main/removeTab',
      payload: {
        id: '/leadClassRules/add',
      },
    });
    this.props.history.goBack();
  }

  render() {
    const { loading, isDetail, editStatus, initialData } = this.state;
    const { getFieldDecorator } = this.props.form;
    return (
      <Page route={this.props.route} loading={loading} className="LeadClassRules">
        <Form onSubmit={this.onSubmit} layout="inline" className={styles.ruleForm}>
          <Row type="flex" justify="start">
            <Col span={8}>
              <FormItem label="规则名称" {...ItemCol} className="ant-col-24">
                {getFieldDecorator('name', {
                  rules: [{ required: true, message: '请输入规则名称' }],
                  initialValue: initialData && initialData.name,
                })(<Input placeholder="输入规则名称" />)}
              </FormItem>
            </Col>
            <Col span={8} offset={2}>
              <FormItem label="课酬补贴系数" {...ItemCol} className="ant-col-24">
                {getFieldDecorator('salaryCoefficient', {
                  rules: [
                    { required: true, message: '请输入课酬补贴系数' },
                    { pattern: /^\d+(\.{0,1}\d+){0,1}$/, message: '课酬补贴系数不能小于0' },
                  ],
                  initialValue: initialData && initialData.salaryCoefficient,
                })(<Input type="number" placeholder="输入课酬补贴系数" />)}
              </FormItem>
            </Col>
          </Row>
          <Row type="flex" justify="start">
            <Col span={8}>
              <FormItem label="春季最大带班数" {...ItemCol} className="ant-col-24">
                {getFieldDecorator('spring', {
                  rules: [
                    { required: true, message: '请输入春季最大带班数' },
                    { pattern: /^[+]{0,1}(\d+)$/, message: '请输入正整数' },
                  ],
                  initialValue: initialData && initialData.spring,
                })(<Input type="number" placeholder="输入最大带班数" disabled={isDetail && !editStatus.spring} />)}
              </FormItem>
            </Col>
            <Col span={1} offset={1}>
              {isDetail && (
                <a
                  href="javascript:void(0)"
                  onClick={() => this.setState({ editStatus: { ...editStatus, spring: true } })}
                >
                  修改
                </a>
              )}
            </Col>
            <Col span={8}>
              <FormItem label="秋季最大带班数" {...ItemCol} className="ant-col-24">
                {getFieldDecorator('autumn', {
                  rules: [
                    { required: true, message: '请输入秋季最大带班数' },
                    { pattern: /^[+]{0,1}(\d+)$/, message: '请输入正整数' },
                  ],
                  initialValue: initialData && initialData.autumn,
                })(<Input type="number" placeholder="输入最大带班数" disabled={isDetail && !editStatus.autumn} />)}
              </FormItem>
            </Col>
            <Col span={1} offset={1}>
              {isDetail && (
                <a
                  href="javascript:void(0)"
                  onClick={() => this.setState({ editStatus: { ...editStatus, autumn: true } })}
                >
                  修改
                </a>
              )}
            </Col>
          </Row>
          <Row type="flex" justify="start">
            <Col span={8}>寒假最大带班数：</Col>
          </Row>
          <Row type="flex" justify="start">
            <Col span={8}>
              <FormItem label="1期" {...ItemCol} className="ant-col-24">
                {getFieldDecorator('winter1', {
                  rules: [
                    { required: true, message: '请输入寒假一期最大带班数' },
                    { pattern: /^[+]{0,1}(\d+)$/, message: '请输入正整数' },
                  ],
                  initialValue: initialData && initialData.winter1,
                })(<Input type="number" placeholder="输入最大带班数" disabled={isDetail && !editStatus.winter1} />)}
              </FormItem>
            </Col>
            <Col span={1} offset={1}>
              {isDetail && initialData.winter1 >= 0 && (
                <a
                  href="javascript:void(0)"
                  onClick={() => this.setState({ editStatus: { ...editStatus, winter1: true } })}
                >
                  修改
                </a>
              )}
            </Col>
          </Row>
          <Row type="flex" justify="start">
            <Col span={8}>
              <FormItem label="2期" {...ItemCol} className="ant-col-24">
                {getFieldDecorator('winter2', {
                  rules: [{ pattern: /^[+]{0,1}(\d+)$/, message: '请输入正整数' }],
                  initialValue: initialData && initialData.winter2,
                })(
                  <Input
                    type="number"
                    placeholder="输入最大带班数"
                    disabled={isDetail && !editStatus.winter2 && !!initialData.winter2}
                  />,
                )}
              </FormItem>
            </Col>
            <Col span={1} offset={1}>
              {isDetail && initialData.winter2 >= 0 && (
                <a
                  href="javascript:void(0)"
                  onClick={() => this.setState({ editStatus: { ...editStatus, winter2: true } })}
                >
                  修改
                </a>
              )}
            </Col>
          </Row>
          <Row type="flex" justify="start">
            <Col span={8}>
              <FormItem label="3期" {...ItemCol} className="ant-col-24">
                {getFieldDecorator('winter3', {
                  rules: [{ pattern: /^[+]{0,1}(\d+)$/, message: '请输入正整数' }],
                  initialValue: initialData && initialData.winter3,
                })(
                  <Input
                    type="number"
                    placeholder="输入最大带班数"
                    disabled={isDetail && !editStatus.winter3 && !!initialData.winter3}
                  />,
                )}
              </FormItem>
            </Col>
            <Col span={1} offset={1}>
              {isDetail && initialData.winter3 >= 0 && (
                <a
                  href="javascript:void(0)"
                  onClick={() => this.setState({ editStatus: { ...editStatus, winter3: true } })}
                >
                  修改
                </a>
              )}
            </Col>
          </Row>
          <Row type="flex" justify="start">
            <Col span={8}>
              <FormItem label="4期" {...ItemCol} className="ant-col-24">
                {getFieldDecorator('winter4', {
                  rules: [{ pattern: /^[+]{0,1}(\d+)$/, message: '请输入正整数' }],
                  initialValue: initialData && initialData.winter4,
                })(
                  <Input
                    type="number"
                    placeholder="输入最大带班数"
                    disabled={isDetail && !editStatus.winter4 && !!initialData.winter4}
                  />,
                )}
              </FormItem>
            </Col>
            <Col span={1} offset={1}>
              {isDetail && initialData.winter4 >= 0 && (
                <a
                  href="javascript:void(0)"
                  onClick={() => this.setState({ editStatus: { ...editStatus, winter4: true } })}
                >
                  修改
                </a>
              )}
            </Col>
          </Row>
          <Row type="flex" justify="start">
            <Col span={8}>
              <FormItem label="5期" {...ItemCol} className="ant-col-24">
                {getFieldDecorator('winter5', {
                  rules: [{ pattern: /^[+]{0,1}(\d+)$/, message: '请输入正整数' }],
                  initialValue: initialData && initialData.winter5,
                })(
                  <Input
                    type="number"
                    placeholder="输入最大带班数"
                    disabled={isDetail && !editStatus.winter5 && !!initialData.winter5}
                  />,
                )}
              </FormItem>
            </Col>
            <Col span={1} offset={1}>
              {isDetail && initialData.winter5 >= 0 && (
                <a
                  href="javascript:void(0)"
                  onClick={() => this.setState({ editStatus: { ...editStatus, winter5: true } })}
                >
                  修改
                </a>
              )}
            </Col>
          </Row>
          <Row>
            <Col span={8}>
              <FormItem label="6期" {...ItemCol} className="ant-col-24">
                {getFieldDecorator('winter6', {
                  rules: [{ pattern: /^[+]{0,1}(\d+)$/, message: '请输入正整数' }],
                  initialValue: initialData && initialData.winter6,
                })(
                  <Input
                    type="number"
                    placeholder="输入最大带班数"
                    disabled={isDetail && !editStatus.winter6 && !!initialData.winter6}
                  />,
                )}
              </FormItem>
            </Col>
            <Col span={1} offset={1}>
              {isDetail && initialData.winter6 >= 0 && (
                <a
                  href="javascript:void(0)"
                  onClick={() => this.setState({ editStatus: { ...editStatus, winter6: true } })}
                >
                  修改
                </a>
              )}
            </Col>
          </Row>
          <Row type="flex" justify="start">
            <Col span={2}>暑假最大带班数：</Col>
          </Row>
          <Row type="flex" justify="start">
            <Col span={8}>
              <FormItem label="1期" {...ItemCol} className="ant-col-24">
                {getFieldDecorator('summer1', {
                  rules: [
                    { required: true, message: '请输入暑假一期最大带班数' },
                    { pattern: /^[+]{0,1}(\d+)$/, message: '请输入正整数' },
                  ],
                  initialValue: initialData && initialData.summer1,
                })(
                  <Input
                    type="number"
                    placeholder="输入最大带班数"
                    disabled={isDetail && !editStatus.summer1 && !!initialData.summer1}
                  />,
                )}
              </FormItem>
            </Col>
            <Col span={1} offset={1}>
              {isDetail && initialData.summer1 >= 0 && (
                <a
                  href="javascript:void(0)"
                  onClick={() => this.setState({ editStatus: { ...editStatus, summer1: true } })}
                >
                  修改
                </a>
              )}
            </Col>
          </Row>
          <Row type="flex" justify="start">
            <Col span={8}>
              <FormItem label="2期" {...ItemCol} className="ant-col-24">
                {getFieldDecorator('summer2', {
                  rules: [{ pattern: /^[+]{0,1}(\d+)$/, message: '请输入正整数' }],
                  initialValue: initialData && initialData.summer2,
                })(
                  <Input
                    type="number"
                    placeholder="输入最大带班数"
                    disabled={isDetail && !editStatus.summer2 && !!initialData.summer2}
                  />,
                )}
              </FormItem>
            </Col>
            <Col span={1} offset={1}>
              {isDetail && initialData.summer2 >= 0 && (
                <a
                  href="javascript:void(0)"
                  onClick={() => this.setState({ editStatus: { ...editStatus, summer2: true } })}
                >
                  修改
                </a>
              )}
            </Col>
          </Row>
          <Row type="flex" justify="start">
            <Col span={8}>
              <FormItem label="3期" {...ItemCol} className="ant-col-24">
                {getFieldDecorator('summer3', {
                  rules: [{ pattern: /^[+]{0,1}(\d+)$/, message: '请输入正整数' }],
                  initialValue: initialData && initialData.summer3,
                })(
                  <Input
                    type="number"
                    placeholder="输入最大带班数"
                    disabled={isDetail && !editStatus.summer3 && !!initialData.summer3}
                  />,
                )}
              </FormItem>
            </Col>
            <Col span={1} offset={1}>
              {isDetail && initialData.summer3 >= 0 && (
                <a
                  href="javascript:void(0)"
                  onClick={() => this.setState({ editStatus: { ...editStatus, summer3: true } })}
                >
                  修改
                </a>
              )}
            </Col>
          </Row>
          <Row type="flex" justify="start">
            <Col span={8}>
              <FormItem label="4期" {...ItemCol} className="ant-col-24">
                {getFieldDecorator('summer4', {
                  rules: [{ pattern: /^[+]{0,1}(\d+)$/, message: '请输入正整数' }],
                  initialValue: initialData && initialData.summer4,
                })(
                  <Input
                    type="number"
                    placeholder="输入最大带班数"
                    disabled={isDetail && !editStatus.summer4 && !!initialData.summer4}
                  />,
                )}
              </FormItem>
            </Col>
            <Col span={1} offset={1}>
              {isDetail && initialData.summer4 >= 0 && (
                <a
                  href="javascript:void(0)"
                  onClick={() => this.setState({ editStatus: { ...editStatus, summer4: true } })}
                >
                  修改
                </a>
              )}
            </Col>
          </Row>
          <Row type="flex" justify="start">
            <Col span={8}>
              <FormItem label="5期" {...ItemCol} className="ant-col-24">
                {getFieldDecorator('summer5', {
                  rules: [{ pattern: /^[+]{0,1}(\d+)$/, message: '请输入正整数' }],
                  initialValue: initialData && initialData.summer5,
                })(
                  <Input
                    type="number"
                    placeholder="输入最大带班数"
                    disabled={isDetail && !editStatus.summer5 && !!initialData.summer5}
                  />,
                )}
              </FormItem>
            </Col>
            <Col span={1} offset={1}>
              {isDetail && initialData.summer5 >= 0 && (
                <a
                  href="javascript:void(0)"
                  onClick={() => this.setState({ editStatus: { ...editStatus, summer5: true } })}
                >
                  修改
                </a>
              )}
            </Col>
          </Row>
          <Row type="flex" justify="start">
            <Col span={8}>
              <FormItem label="6期" {...ItemCol} className="ant-col-24">
                {getFieldDecorator('summer6', {
                  rules: [{ pattern: /^[+]{0,1}(\d+)$/, message: '请输入正整数' }],
                  initialValue: initialData && initialData.summer6,
                })(
                  <Input
                    type="number"
                    placeholder="输入最大带班数"
                    disabled={isDetail && !editStatus.summer6 && !!initialData.summer6}
                  />,
                )}
              </FormItem>
            </Col>
            <Col span={1} offset={1}>
              {isDetail && initialData.summer6 >= 0 && (
                <a
                  href="javascript:void(0)"
                  onClick={() => this.setState({ editStatus: { ...editStatus, summer6: true } })}
                >
                  修改
                </a>
              )}
            </Col>
          </Row>
          <Row type="flex" justify="start">
            <Col offset={10}>
              <Button onClick={() => this.goBack()}>取消</Button>
            </Col>
            <Col offset={1}>
              <Button type="primary" htmlType="submit">
                保存
              </Button>
            </Col>
          </Row>
        </Form>
      </Page>
    );
  }
}
function mapStateToProps(state) {
  const data = state.main.pageParams;
  const pathName = state.routing.locationBeforeTransitions.pathname;
  return {
    params: data[pathName],
  };
}
export default connect(mapStateToProps)(Form.create()(LeadClassRuleAdd));
