import React from 'react';
import { connect } from 'dva';
import { Page, SearchBar, Input, Button, List } from 'COMPONENT';
import * as Api from 'SERVICE/personnelManage';
import { message } from 'antd';
import { QUARTER_SPRING, QUARTER_SUMMER, QUARTER_AUTUMN, QUARTER_WINTER } from 'PAGE/common/utils.js';
const { COL_WIDTH_XS, COL_WIDTH_XL, COL_WIDTH_M, COL_WIDTH_L } = List;
class LeadClassRules extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      ruleName: '',
      loading: false,
      dataList: [],
      // 分页
      pageNum: 1,
      pageSize: 10,
      total: 0,
    };
    this.columns = [
      { title: '规则', dataIndex: 'name', width: COL_WIDTH_XL },
      { title: '课酬补贴系数', dataIndex: 'salaryCoefficient', width: COL_WIDTH_L },
      {
        title: '分公司寒假设班期数',
        width: 350,
        children: [
          { title: '1期', dataIndex: 'winter1', width: COL_WIDTH_XS },
          { title: '2期', dataIndex: 'winter2', width: COL_WIDTH_XS },
          { title: '3期', dataIndex: 'winter3', width: COL_WIDTH_XS },
          { title: '4期', dataIndex: 'winter4', width: COL_WIDTH_XS },
          { title: '5期', dataIndex: 'winter5', width: COL_WIDTH_XS },
          { title: '6期', dataIndex: 'winter6', width: COL_WIDTH_XS },
        ],
      },
      { title: '春季', dataIndex: 'spring', width: COL_WIDTH_L },
      {
        title: '分公司暑假设班期数',
        width: 350,
        children: [
          { title: '1期', dataIndex: 'summer1', width: COL_WIDTH_XS },
          { title: '2期', dataIndex: 'summer2', width: COL_WIDTH_XS },
          { title: '3期', dataIndex: 'summer3', width: COL_WIDTH_XS },
          { title: '4期', dataIndex: 'summer4', width: COL_WIDTH_XS },
          { title: '5期', dataIndex: 'summer5', width: COL_WIDTH_XS },
          { title: '6期', dataIndex: 'summer6', width: COL_WIDTH_XS },
        ],
      },
      { title: '秋季', dataIndex: 'autumn', width: COL_WIDTH_L },
      {
        title: '状态',
        dataIndex: 'status',
        width: COL_WIDTH_L,
        render: (text) => {
          return text == 1 ? '可用' : '不可用';
        },
      },
      {
        title: '操作',
        key: 'action',
        width: COL_WIDTH_M,
        getActionConfig: (text, record, index) => {
          return [
            {
              text: record.status == 1 ? '禁用' : '启用',
              onClick: () => this.onRowBtnClick('change', text, record, index),
            },
            { text: '编辑', onClick: () => this.onRowBtnClick('edit', text, record, index) },
          ];
        },
      },
    ];
    if (!this.props.canEdit) {
      this.columns.splice(-1, 1); // 需要配置编辑权限
    }

    this.addRule = this.addRule.bind(this);
    this.loadList = this.loadList.bind(this);
    this.resetList = this.resetList.bind(this);
  }

  componentDidMount() {
    this.loadList();
  }

  // 新增规则
  addRule() {
    const { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: {
        path: '/leadClassRules/add',
        refresh: true,
      },
    });
  }

  // 操作 action
  onRowBtnClick(type, text, record, index) {
    type === 'change' ? this.changeStatus(record) : this.edit(record);
  }

  // 修改状态
  async changeStatus(rowData) {
    const { resultCode } = await Api.switchStatus({
      ruleId: rowData.id,
    });
    if (resultCode === 0) {
      message.success('修改成功');
      this.loadList();
    }
  }

  // 编辑
  edit(rowData) {
    const { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: {
        path: '/leadClassRules/add',
        params: {
          ...rowData,
        },
        refresh: true,
      },
    });
  }

  async loadList({ pageNum, pageSize } = { pageNum: 1, pageSize: 10 }) {
    const { resultCode, data } = await Api.getNumberLimitRule({
      type: 1, // 1-集团；99-老师
      withDetails: true,
      name: this.state.ruleName,
      pageNum: pageNum,
      pageSize: pageSize,
    });
    if (resultCode == 0 && data) {
      // 季度数据，方便展示
      data.list.forEach((row) => {
        if (row.mcNumberLimitRuleDetailVos) {
          row.mcNumberLimitRuleDetailVos.forEach((mvo) => {
            if (mvo.quarterId === QUARTER_SPRING) {
              row['spring'] = mvo.maxClassNumber;
            }
            if (mvo.quarterId === QUARTER_AUTUMN) {
              row['autumn'] = mvo.maxClassNumber;
            }

            if (mvo.quarterId === QUARTER_WINTER) {
              row['winter' + mvo.termNumber] = mvo.maxClassNumber;
            }

            if (mvo.quarterId === QUARTER_SUMMER) {
              row['summer' + mvo.termNumber] = mvo.maxClassNumber;
            }
          });
        }
      });
      this.setState({
        dataList: data.list,
        total: data.total,
        pageNum,
        pageSize,
      });
    }
  }

  resetList() {
    this.setState(
      {
        ruleName: '',
      },
      () => this.loadList(),
    );
  }

  render() {
    const { ruleName, dataList, loading, pageNum, pageSize, total } = this.state;
    const { canEdit } = this.props;
    return (
      <Page
        loading={loading}
        route={this.props.route}
        headerToolBar={
          canEdit ? (
            <Button type="primary" onClick={this.addRule}>
              添加规则
            </Button>
          ) : (
            ''
          )
        }
        subHeader={
          <SearchBar onSearch={this.loadList} onReset={this.resetList}>
            <Input
              placeholder="规则名称"
              value={ruleName}
              onChange={(e) => this.setState({ ruleName: e.target.value })}
            />
          </SearchBar>
        }
      >
        <List
          dataSource={dataList}
          columns={this.columns}
          bordered
          pagination={{
            current: pageNum,
            total: total,
            pageSize: pageSize,
          }}
          onChange={({ current, pageSize }) => this.loadList({ pageNum: current, pageSize })}
        />
      </Page>
    );
  }
}
function mapStateToProps(state) {
  return {
    canEdit: state.account.btnAuths.indexOf('BTN_EDIT_RULES') > -1,
  };
}
export default connect(mapStateToProps)(LeadClassRules);
