import React from 'react';
import Button from 'COMPONENT/Button';
import styles from './index.less';

import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { FILE_TYPE_MAP_EXTENSION } from 'COMPONENT/FileIcon/config';

export default class ChooseFileButton extends React.Component {
  static defaultProps = {
    size: 'large', // 按钮大小
    btnText: '选择文件', // 按钮文字
    accept: 'image/jpeg,image/jpg,image/png,image/bmp',
    fileType: ['png', 'jpg', 'bmp'], // 文件类型
    maxSize: 1024 * 1024 * 10, // 文件大小
    maxNameLength: 50, // 文件名长度
  };

  constructor(props) {
    super(props);

    this.state = {
      file: null,
    };

    this.onInputChange = this.onInputChange.bind(this);
    this.canUploadFile = this.canUploadFile.bind(this);
  }

  /**
   * 选择文件
   * @param e
   */
  onInputChange(e) {
    e.preventDefault();

    let $target = e.dataTransfer || e.target || null;
    let files = $target && $target.files;

    if (files && files.length > 0) {
      let file = files[0];
      if (this.canUploadFile(file)) {
        this.props.onChange(file);
      } else {
        this.props.onChange(null);
      }
    } else {
      this.props.onChange(null);
    }

    // 清空输入框
    $target.value = '';
  }

  /**
   * 校验文件是否允许上传
   * @param file 选中的文件
   */
  canUploadFile(file) {
    const { fileType, maxSize, maxNameLength } = this.props;

    // ["png","jpg","jpeg","bmp"]
    let allowFileExtends = [];
    fileType.forEach((type) => {
      allowFileExtends = [...allowFileExtends, ...FILE_TYPE_MAP_EXTENSION[type]];
    });

    let fileNameResult = this.formatFileName(file);
    if (allowFileExtends.indexOf(fileNameResult.nameExtend.toLowerCase()) < 0) {
      showMessage('文件格式不正确', 'warning');
      return false;
    } else if (file.size > maxSize) {
      showMessage('文件大小超过了10MB', 'warning');
      return false;
    } else if (fileNameResult.nameLength > maxNameLength) {
      showMessage('文件名超过50字', 'warning');
      return false;
    } else {
      this.setState({ file: file });
      return true;
    }
  }

  /**
   * 格式化文件信息
   * @param file
   * @return {*}
   */
  formatFileName = (file) => {
    if (!file) {
      return {};
    }

    const fileName = file.name;
    const index = fileName.lastIndexOf('.');
    const nameExtend = fileName.slice(index + 1); // jpeg
    const nameLength = fileName.length - nameExtend.length - 1;

    return {
      fileName,
      nameExtend,
      nameLength,
    };
  };

  render() {
    const { size, btnText, accept } = this.props;

    return (
      <Button size={size}>
        {btnText}
        <input type="file" accept={accept} className={styles.inputFile} onChange={this.onInputChange} />
      </Button>
    );
  }
}
