import React from 'react';
import Modal from 'COMPONENT/Modal';
import Button from 'COMPONENT/Button';
import ChooseFileButton from '../ChooseFileButton';
import IconSlider from '../IconSlider';
import styles from './index.less';
import { getSseKey } from 'SERVICE/common';
import { uuid } from 'COMPONENT/utils/basic/str';

import Cropper from 'react-cropper'; // 引入Cropper
import 'cropperjs/dist/cropper.css'; // 引入Cropper对应的css

export default class CropperImgModal extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      file: null, // 本地图片文件
      src: '', // 本地图片路径
      upLoading: false, // 上传中

      initZoomValue: null, // 保存初始化的缩放等级

      showSlider: true, // 是否显示滑块
      sliderMin: 1, // 滑块最小值
      sliderMax: 10, // 滑块最大值
      sliderValDefault: 1, // 滑块默认值
    };

    this.initFile = this.initFile.bind(this);
    this.onCancel = this.onCancel.bind(this);
    this.onReChoose = this.onReChoose.bind(this);
    this.onUpLoad = this.onUpLoad.bind(this);
    this.doUpload = this.doUpload.bind(this);

    this.onCropperReady = this.onCropperReady.bind(this);
    this.onZoom = this.onZoom.bind(this);
    this.setZoom = this.setZoom.bind(this);
    this.onSlider = this.onSlider.bind(this);
  }

  componentDidMount() {
    this.initFile(this.props.file);
  }

  /**
   * 初始化选中的图片并且把文件保存起来
   * @param file
   */
  initFile(file) {
    const fileReader = new FileReader();
    fileReader.onload = (e) => {
      const dataURL = e.target.result;
      this.setState({ src: dataURL });
    };
    fileReader.readAsDataURL(file);

    this.setState({ file: file });
  }

  /**
   * 关闭编辑弹窗
   */
  onCancel() {
    this.props.onClose();
  }

  /**
   * 重新选择
   * @param file
   */
  onReChoose(file) {
    this.initFile(file);
  }

  /**
   * 上传
   */
  onUpLoad() {
    if (!this.state.upLoading) {
      // 这里可以修改上传图片的尺寸
      const croppedCanvas = this.cropper.getCroppedCanvas();
      croppedCanvas.toBlob(async (blob) => {
        await this.doUpload(blob);
      });
    }
  }
  async doUpload(blob) {
    this.setState({ upLoading: true });

    let configKey = await getSseKey();
    let fileKey = uuid();
    const { file } = this.state;

    let formData = new FormData();
    formData.append('x:realname', file.name);
    formData.append('x:uploadtype', 'PC');
    formData.append('x:uploadid', fileKey);
    formData.append('key', fileKey);
    Object.keys(configKey).forEach((key, index) => {
      if (key !== 'host') {
        formData.append(key, configKey[key]);
      }
    });
    // 阿里云oss，表单域指定key一定要放在表单域file前面！！！
    formData.append('file', blob, file.name);

    let xhr = new XMLHttpRequest();
    xhr.open('POST', configKey.host, true);
    xhr.upload.onloadstart = (e) => {
      // this.setState({upLoading: true});
    };
    xhr.upload.onprogress = (e) => {
      // console.log(e.loaded, e.total);
    };
    xhr.onload = (e) => {
      let imgSrc = configKey.host + fileKey;
      this.setState({ upLoading: false });
      // 把选中裁切好的的图片及上传完的图片URL传给父组件
      this.props.onOk(imgSrc, blob);
      // 关闭弹窗
      this.props.onClose();
    };
    xhr.send(formData);
  }

  /**
   * 初始化裁剪组件并获取初始的缩放值
   * @param e
   */
  onCropperReady(e) {
    // console.log(e.target.cropper);
  }

  /**
   * 缩放事件
   * @param e
   * @return {boolean}
   */
  onZoom(e) {
    // console.log('e.detail => ', e.detail);
  }

  /**
   * 设置缩放
   * zoom in 缩小，正数
   * zoom out 放大，负数
   * @param z 缩放值 0-1
   */
  setZoom(z) {
    const cropper = this.cropper;
    cropper.zoom(z);
  }

  /**
   * 滑动事件
   * @param value
   * @param oldValue
   */
  onSlider(value, oldValue) {
    let z = Number(value) - Number(oldValue);
    z = z / 10;
    this.setZoom(z);
  }

  render() {
    let { upLoading, showSlider, sliderMin, sliderMax, sliderValDefault } = this.state;

    return (
      <Modal
        title="上传"
        width={680}
        loading={false}
        onCancel={this.onCancel}
        onOk={() => {}}
        okBtnShow={true}
        cancelText="取消"
        okBtnText="确定"
        footer={[
          <ChooseFileButton key="back" btnText={'重新选择'} onChange={(file) => this.onReChoose(file)} />,
          <Button key="true" type="primary" loading={upLoading} onClick={this.onUpLoad}>
            {(upLoading && '上传中') || '上传'}
          </Button>,
        ]}
      >
        <section className={styles.cropperWrap}>
          <div className={styles.cropperWrap__left}>
            <Cropper
              modal={false}
              viewMode={1}
              zoomable={true}
              aspectRatio={1} // 固定为1:1  可以自己设置比例, 默认情况为自由比例
              guides={true}
              movable={true}
              src={this.state.src}
              ref={(cropper) => (this.cropper = cropper)}
              ready={(e) => {
                this.onCropperReady(e);
              }}
              zoom={(e) => {
                this.onZoom(e);
              }}
              className="cropper"
              preview=".cropper__preview"
              style={{ height: '100%', width: '100%' }}
            />
          </div>

          <div className={styles.cropperWrap__right}>
            <div className={styles.cropperWrap__right__cover} />
            <div className={styles.cropper__preview + ' cropper__preview'} />
            <div className={styles.t_c + ' ' + styles.mt_10}>预览</div>
          </div>
        </section>

        {showSlider && (
          <section className={styles.sliderContainer}>
            <IconSlider
              min={sliderMin}
              max={sliderMax}
              defaultValue={sliderValDefault}
              // tipFormatter={null}
              onSlider={(value, oldValue) => {
                this.onSlider(value, oldValue);
              }}
            />
          </section>
        )}
      </Modal>
    );
  }
}
