import React from 'react';
import { Slider } from 'antd';
import Icon from 'COMPONENT/Icon';
import request from 'UTIL/http/fetch';
import SvgLib from 'COMPONENT/assets/svgs/symbol-defs.svg';
import styles from './index.less';

export default class IconSlider extends React.Component {
  static defaultProps = {
    min: 0,
    max: 100,
    defaultValue: 0,
    onSlider: (value, oldValue) => {},
  };

  constructor(props) {
    super(props);

    this.state = {
      oldValue: 0, // 旧滑动值
      value: 0, // 新滑动值

      // icon 相关
      ids: [],
      mIds: [],
    };

    this.initValue = this.initValue.bind(this);
    this.initIcon = this.initIcon.bind(this);
    this.onSlider = this.onSlider.bind(this);
    this.onZoom = this.onZoom.bind(this);
  }

  componentDidMount() {
    this.initValue();
    this.initIcon();
  }

  initValue() {
    this.setState({
      oldValue: this.props.defaultValue || this.props.min || 0,
      value: this.props.defaultValue || this.props.min || 0,
    });
  }

  /**
   * icon相关
   */
  initIcon() {
    request(SvgLib, { method: 'GET' }).then((d) => {
      let strs = d.data.match(/<symbol id="Icon-((\w|-)+)"\s/gi);
      let ids = [];
      let mIds = [];
      strs.forEach((s) => {
        const id = s.replace('<symbol id="Icon-', '').replace('" ', '');
        if (this.checkMultiColor(id)) {
          mIds.push(id);
        } else {
          ids.push(id);
        }
      });
      this.setState({ ids, mIds });
    });
  }
  checkMultiColor = (type) => {
    const arry = type.split('_');
    return arry[arry.length - 1] === 'mc' || arry[arry.length - 2] === 'mc';
  };

  /**
   * 滑动
   * @param value
   */
  onSlider(value) {
    this.setState({ value: value });
    this.props.onSlider(value, this.state.oldValue);
    // 最后重新赋值旧滑动值
    this.setState({ oldValue: value });
  }

  /**
   * 点击图标放大/缩小
   * @param type 默认放大
   * @return {boolean}
   */
  onZoom(type = 'out') {
    const { min, max } = this.props;
    let nowValue = this.state.value;
    let newValue = 0;

    if (type === 'out') {
      if (nowValue >= max) {
        return false;
      }

      newValue = nowValue + 1;
    } else {
      if (nowValue <= min) {
        return false;
      }

      newValue = nowValue - 1;
    }

    this.onSlider(newValue);
  }

  render() {
    let { value } = this.state;

    return (
      <div className={styles.sliderContainer}>
        <div className={styles.iconWrap + ' ' + styles.icon__left}>
          <Icon
            type="zoom_small"
            size={16}
            onClick={() => {
              this.onZoom('in');
            }}
            style={{ display: 'inline-block', verticalAlign: 'middle' }}
          />
        </div>
        <div className={styles.sliderWrap}>
          <Slider {...this.props} onChange={this.onSlider} value={value} />
        </div>
        <div className={styles.iconWrap + ' ' + styles.icon__right}>
          <Icon
            type="zoom_big"
            size={16}
            onClick={() => {
              this.onZoom('out');
            }}
            style={{ display: 'inline-block', verticalAlign: 'middle' }}
          />
        </div>
      </div>
    );
  }
}
