import React from 'react';
import Modal from 'COMPONENT/Modal';
import { Row, Col } from 'antd';
import FormItem from 'COMPONENT/FormItem';
import Input from 'COMPONENT/Input';
import TextArea from 'COMPONENT/TextArea';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { objectDeepEquals } from 'COMPONENT/utils/basic/obj';
import { struct } from 'cubec';
import CropperImgModal from '../CropperImgModal';
import ChooseFileButton from '../ChooseFileButton';
import { updateTeacherAttribute } from 'SERVICE/personnelManage';
import styles from './index.less';
import defaultHead from '../../../../assets/imgs/default_head.png';

const _trim = struct.string('trim');
const labelWidth = 80;

export default class TeacherEditModal extends React.Component {
  static defaultProps = {
    row: {
      id: null,
      name: null, // 教师名称
      university: null, // 毕业院校
      pic_url: null, // 头像
      teachingStyle: null, // 教学风格
    },
    refreshList: () => {},
  };

  constructor(props) {
    super(props);

    this.state = {
      university: null, // 毕业院校
      picUrl: null, // 图片
      teachingStyle: null, // 教学特点

      showCropperModal: false, // 显示裁剪弹窗
      file: null, // 用户选择的图片, 裁剪图片时用到
    };

    this.initContent = this.initContent.bind(this);
    this.onSelectFile = this.onSelectFile.bind(this);
    this.onFileUploaded = this.onFileUploaded.bind(this);
    this.onCommit = this.onCommit.bind(this);
    this.formatParam = this.formatParam.bind(this);
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化弹窗内容
   */
  initContent() {
    const { row } = this.props;

    this.setState({
      university: row.university,
      picUrl: row.pic_url,
      teachingStyle: row.teachingStyle,
    });
  }

  /**
   * 选择头像
   * @param file
   */
  onSelectFile(file) {
    if (file) {
      this.setState(
        {
          file: file,
        },
        () => {
          this.setState({ showCropperModal: true });
        },
      );
    } else {
      this.setState({ file: null });
    }
  }

  /**
   * 裁剪上传完
   * @param imgSrc
   * @param blob
   */
  onFileUploaded(imgSrc, blob) {
    this.setState({
      picUrl: imgSrc,
    });
  }

  /**
   * 保存修改信息
   */
  async onCommit() {
    let param = this.formatParam();

    if (!param.picUrl) {
      showMessage('请选择老师头像！', 'warning');
      return;
    }

    if (!param.teachingStyle) {
      showMessage('请填写教学特点！', 'warning');
      return;
    }

    await updateTeacherAttribute(param);
    showMessage('修改成功', 'success');
    this.props.onClose();
    this.props.refreshList();
  }

  /**
   * 列表字段与修改字段不一致...
   */
  formatParam() {
    const { row } = this.props;
    let { university, picUrl, teachingStyle } = this.state;
    let stateRow = { university, picUrl, teachingStyle };
    let param = { ...row, ...stateRow };
    param.userId = param.user_id;
    delete param.pic_url;
    delete param.user_id;
    return param;
  }

  render() {
    const { university, picUrl, teachingStyle } = this.state;
    const { showCropperModal, file } = this.state;
    const { row } = this.props;

    return (
      <Modal
        loading={false}
        onCancel={this.props.onClose}
        title="修改"
        width={960}
        cancelText="关闭"
        okBtnShow={true}
        onOk={this.onCommit}
        okBtnDisabled={false}
      >
        <Row gutter={10}>
          <Col span={6}>
            <div className={styles.pictureWrap}>
              <div className={styles.headWrap}>
                <div className={styles.headWrap__cover} />
                <img src={picUrl || defaultHead} className={styles.pictureWrap__img} />
              </div>
              <div className={styles.pictureWrap__text}>
                照片格式建议为jpg、png <br /> 大小不超过10M
              </div>
              <div className={styles.mt__10}>
                <ChooseFileButton
                  key="upload"
                  size={'small'}
                  btnText={'上传本地图片'}
                  onChange={(file) => this.onSelectFile(file)}
                />
              </div>
            </div>
          </Col>

          <Col span={18}>
            <FormItem name="教师名称" labelWidth={labelWidth}>
              {row.name}
            </FormItem>
            <FormItem name="毕业院校" labelWidth={labelWidth}>
              <Input
                placeholder="请输入毕业院校"
                value={university}
                maxLength={30}
                onChange={(e) => this.setState({ university: _trim(e.target.value || '') })}
              />
            </FormItem>
            <FormItem name="教学特点" labelWidth={labelWidth} required>
              <TextArea
                maxLength={500}
                rows={5}
                wordCount={true}
                value={teachingStyle || ''}
                onChange={(e) => this.setState({ teachingStyle: _trim(e.target.value || '') })}
                className={styles.myTextarea}
              />
            </FormItem>
          </Col>

          {showCropperModal && (
            <CropperImgModal
              file={file}
              onClose={() => {
                this.setState({ showCropperModal: false });
              }}
              onOk={this.onFileUploaded}
            />
          )}
        </Row>
      </Modal>
    );
  }
}
