/**
 * 老师介绍
 */
import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import SearchBar from 'COMPONENT/SearchBar';
import List from 'COMPONENT/List';
import Select from 'COMPONENT/Select';
import Input from 'COMPONENT/Input';
import MultiSelect from 'COMPONENT/MultiSelect';
import {
  loadCommonDict,
  loadBranchDict,
  loadAllBranchCampusDict,
  loadBranchCampusDict,
  loadCampusDict,
  loadSmallClassTcDict,
} from 'UTIL/dict';
import { getTeacherList } from 'SERVICE/personnelManage';
import TeacherDetailModal from './TeacherDetailModal';
import TeacherEditModal from './TeacherEditModal';
import { statusDict } from './dicts';

const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;

class TeacherIntroduction extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      listLoading: false,

      name: null, // 查询条件 教师名称
      teaId: null, // 临时 教师ID
      branchId: null, // 查询条件 分公司 '123'
      blCampusId: null, // 查询条件 学习中心 []
      enableFlg: null, // 查询条件 是否在职
      gradeId: null, // 查询条件 年级 ['123', '234']
      subjectId: null, // 查询条件 科目 ['123', '234']

      smallClassTcDict: null, // 字典 小班授课老师
      branchDict: null, // 字典 所属分公司
      branchCampusDict: null, // 字典 学习中心
      gradeDict: null, // 字典 年级
      subjectDict: null, // 字典 科目

      list: [], // 列表数据
      totalRecords: 0, // 总记录数
      pageNo: 1, // 当前页码
      pageSize: 20, // 每页记录数

      showDetailModal: false, // 显示详情弹窗
      showEditModal: false, // 显示修改弹窗
      selectedRow: null, // 选中的行
    };

    this.initSearchParam = {
      name: null, // 教师名称
      teaId: null, // 查询条件 教师ID
      branchId: null, // 分公司
      blCampusId: null, // 学习中心 []
      enableFlg: null, // 是否在职
      gradeId: null, // 年级 []
      subjectId: null, // 科目 []
    };

    this.onSearch = this.onSearch.bind(this);
    this.onReset = this.onReset.bind(this);
    this.onTeacherSelectChange = this.onTeacherSelectChange.bind(this);
    this.onBranchChange = this.onBranchChange.bind(this);
    this.onRowBtnClick = this.onRowBtnClick.bind(this);
  }

  componentDidMount() {
    this.initPage();
  }

  /**
   * 初始化数据
   */
  initPage() {
    this.setState({ loading: true });

    Promise.all([
      loadBranchDict(),
      loadBranchCampusDict(),
      loadCommonDict('STUDENT_GRADE'),
      loadCommonDict('SUBJECT'),
    ]).then(([branchDict, branchCampusDict, gradeDict, subjectDict]) => {
      // 根据用户所属组织初始化分公司及学习中心，当是集团的人，默认取第一个值
      const { uBranchId, uCampusId } = this.props;
      let isGroup = uBranchId === uCampusId;
      let branchVal = isGroup ? branchDict[0].val : uBranchId;
      let campusVal = isGroup ? [branchCampusDict.find((item) => item.val === branchVal).options[0].val] : [uCampusId];

      this.setState({
        loading: false,
        branchDict,
        branchCampusDict,
        gradeDict,
        subjectDict,
        branchId: branchVal,
        blCampusId: campusVal,
      });

      this.onSearch();
    });
  }

  /**
   * 搜索
   */
  onSearch() {
    this.loadList({ pageNo: 1 });
  }
  onReset() {
    this.setState(this.initSearchParam);
  }

  /**
   * 老师名字模糊搜索
   * @param id
   */
  onTeacherSelectChange(id) {
    let arr = this.state.smallClassTcDict.filter((item) => {
      return item.val === id;
    });
    let name = '';
    if (arr && arr.length > 0) {
      name = arr[0].text;
    }
    this.setState({
      name: name,
      teaId: id,
    });
  }

  /**
   * 所属分公司切换
   * @param id
   */
  onBranchChange(id) {
    let { branchId } = this.state;

    if (branchId !== id) {
      // 先清空学习中心ID
      this.setState({
        blCampusId: null,
        branchId: id,
      });
    }
  }

  /**
   * 加载列表数据
   * @param changedParams 要更新的列表数据参数，包括查询条件、页码状态
   */
  loadList = async (changedParams = {}) => {
    let { name, teaId, branchId, blCampusId, enableFlg, gradeId, subjectId, pageNo, pageSize } = this.state;
    let userId = this.props.userId || '';
    let tempParam = {
      name,
      userId,
      branchId,
      blCampusId,
      enableFlg,
      gradeId,
      subjectId,
      pageNo,
      pageSize,
    };
    const param = { ...tempParam, ...changedParams };

    this.setState({ listLoading: true });

    const apiData = await getTeacherList(param);
    const { rowCount, datas } = apiData;
    this.setState({
      listLoading: false,
      list: datas || [],
      totalRecords: rowCount || 0,
      pageNo: param.pageNo,
      pageSize: param.pageSize,
    });
  };

  /**
   * 点击操作按钮
   * @param type
   * @param row
   */
  onRowBtnClick(type = 'view', row) {
    if (type === 'view') {
      this.setState({ showDetailModal: true, selectedRow: row });
    } else {
      this.setState({ showEditModal: true, selectedRow: row });
    }
  }

  render() {
    let { loading, name, teaId, branchId, blCampusId, enableFlg, gradeId, subjectId } = this.state;
    let { smallClassTcDict, branchDict, branchCampusDict, gradeDict, subjectDict } = this.state;
    let { list, listLoading, pageNo, totalRecords, pageSize } = this.state;
    let { showDetailModal, showEditModal, selectedRow } = this.state;

    const columns = [
      {
        title: '序号',
        dataIndex: 'number',
        key: 'number',
        width: COL_WIDTH_S,
        render: (text, record, index) => {
          return index + 1;
        },
      },
      { title: '老师名称', dataIndex: 'name', width: COL_WIDTH_M },
      { title: '分公司', dataIndex: 'branchName', width: COL_WIDTH_M, lines: 2 },
      { title: '学习中心', dataIndex: 'campusName', width: COL_WIDTH_M },
      {
        title: '可教科目',
        dataIndex: 'gradeAndSubject',
        width: COL_WIDTH_XL,
        lines: 3,
        render: (text, record) => {
          return text || '暂无';
        },
      },
      {
        title: '毕业院校',
        dataIndex: 'university',
        width: COL_WIDTH_M,
        render: (text, record) => {
          return text || '暂无';
        },
      },
      {
        title: '图片',
        dataIndex: 'pic_url',
        width: COL_WIDTH_M,
        render: (text, record) => {
          return (text && '已上传') || '暂无';
        },
      },
      { title: '教学特点', dataIndex: 'teachingStyle', width: COL_WIDTH_XL, lines: 3 },
      {
        title: '操作',
        key: 'action',
        width: COL_WIDTH_XL,
        fixed: 'right',
        getActionConfig: (text, record, index) => {
          return [
            {
              text: '修改',
              onClick: () => this.onRowBtnClick('edit', record, index, name),
              disabled: this.props.btnAuth_edit,
            },
            {
              text: '详情',
              onClick: () => this.onRowBtnClick('view', record, index, name),
              disabled: this.props.btnAuth_detail,
            },
          ];
        },
      },
    ];

    return (
      <Page
        fitWindowHeight={true}
        loading={loading}
        route={this.props.route}
        subHeader={
          <SearchBar onSearch={this.onSearch} onReset={this.onReset}>
            <Input placeholder="老师名称" value={name} onChange={(e) => this.setState({ name: e.target.value })} />
            {/*<Select
              placeholder="老师名称"
              showSearch={true}
              dict={smallClassTcDict}
              value={teaId}
              onChange={val => this.onTeacherSelectChange(val)} />*/}
            <Select
              placeholder="所属分公司"
              showSearch={true}
              dict={branchDict}
              value={branchId}
              onChange={(val) => this.onBranchChange(val)}
            />
            <MultiSelect
              placeholder="请选择学习中心"
              dict={
                branchId
                  ? branchCampusDict && branchCampusDict.filter((item) => item.val === branchId)
                  : branchCampusDict
              }
              value={blCampusId}
              showSearch={true}
              onChange={(val) => this.setState({ blCampusId: val })}
            />
            <Select
              placeholder="是否在职"
              dict={statusDict}
              value={enableFlg}
              onChange={(val) => this.setState({ enableFlg: val })}
            />
            <MultiSelect
              placeholder="年级"
              dict={gradeDict}
              value={gradeId}
              onChange={(val) => this.setState({ gradeId: val })}
            />
            <MultiSelect
              placeholder="科目"
              dict={subjectDict}
              value={subjectId}
              onChange={(val) => this.setState({ subjectId: val })}
            />
          </SearchBar>
        }
      >
        <List
          dataSource={list}
          columns={columns}
          loading={!loading && listLoading}
          pagination={{
            current: pageNo,
            total: totalRecords,
            pageSize,
          }}
          onChange={({ current, pageSize }) => this.loadList({ pageNo: current, pageSize })}
        />
        {showDetailModal && (
          <TeacherDetailModal row={selectedRow} onClose={() => this.setState({ showDetailModal: false })} />
        )}

        {showEditModal && (
          <TeacherEditModal
            row={selectedRow}
            onClose={() => this.setState({ showEditModal: false })}
            refreshList={() => this.loadList()}
          />
        )}
      </Page>
    );
  }
}

function mapStateToProps(state) {
  const { brenchId, campusId, userId } = state.account.user;
  const btnAuth = state.account.btnAuths;
  let btnAuth_edit = btnAuth.indexOf('PERSONNELMANAGE_EDIT') === -1;
  let btnAuth_detail = btnAuth.indexOf('PERSONNELMANAGE_DETAIL') === -1;

  return {
    uBranchId: brenchId || null,
    uCampusId: campusId || null,
    userId: userId,
    btnAuth_edit,
    btnAuth_detail,
  };
}

export default connect(mapStateToProps)(TeacherIntroduction);
