import React from 'react';
import { connect } from 'dva';
import {
  Page,
  Block,
  FormItem,
  Input,
  InputNumber,
  Select,
  Button,
  MultiSelect,
  Radio,
  TextArea,
  IconButton,
  Search,
  Checkbox,
  PopTip,
} from 'COMPONENT';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import CardList from 'COMPONENT/List/CardList';
import { Row, Col, DatePicker, Pagination } from 'antd';
import moment from 'moment';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import styles from './index.less';
import ChangeTabs from '../Components/Tabs';
import InfoBoard from '../Components/InfoBoard';
import Steps from '../Components/Steps/index';
import { getSelectOption, savePromotion, getProductList, getPromotionInfoById } from 'SERVICE/preferential';
import {
  mapTips,
  mapDataToDict,
  getTextByDictValue,
  getTextByDictMultipleValue,
  mapPromotionType,
  promotionAreaDict,
  ProductCharacterMap,
} from '../common';
import { loadCommonDictByGroup } from 'UTIL/dict';
import { clone, difference, differenceWith, intersectionWith, indexOf } from 'lodash';
import {
  continuityDict,
  NONE,
  isNeedContinue,
  FullReducePromotion,
  DiscountPromotion,
  MONEY,
  COURSE_HOURS,
  PRODUCT_NUMS,
  PROMOTION_FULL_SALE_NORMAL,
  PROMOTION_FULL_SALE_EVERY,
  PROMOTION_GIVE_HOURS,
  PROMOTION_DISCOUNT_CASH,
  PROMOTION_DISCOUNT_COURSE,
  PROMOTION_DISCOUNT_PRODUCT,
  promotionsPrefixCfg,
  conditionUnitCfg,
  promotionsTypeCfg,
  promotionsUnitCfg,
  continuityPromotionsTypeCfg,
  continuityQuarterCfg,
  dateFormatCfg,
} from '../../config';
import {
  checkContinuePromotion,
  getUseCondition,
  transformRuleVosToCourse,
  transformRuleVosToHours,
} from '../../utils';
import { submitDataAdapter, detailDataAdapter } from './utils';

const Group = Radio.Group;
const [COL_WIDTH_XL, COL_WIDTH_L, COL_WIDTH_M] = [150, 600, 150];
const mapType = {
  PROMOTION_FULL_SALE_NORMAL: '满减',
  PROMOTION_FULL_SALE_EVERY: '每满减',
  PROMOTION_DISCOUNT_CASH: '现金折扣',
  PROMOTION_DISCOUNT_COURSE: '小时折扣',
  PROMOTION_DISCOUNT_PRODUCT: '产品折扣',
};
// 不连报
const NoContinue = NONE;
let mapDefaultPromotionType = {
  // 满减促销
  FullReducePromotion: PROMOTION_FULL_SALE_NORMAL,
  // 折扣促销
  DiscountPromotion: PROMOTION_DISCOUNT_CASH,
  // 满赠促销
  FullGivePromotion: PROMOTION_GIVE_HOURS,
};

/**
 * Page组件底部toolbar
 * */
const renderFooter = (props) => {
  const { handlePrev, handleNext, currentIndex, tabKey, handleBtn, popTipVisible, onClose, content } = props;
  return (
    <div style={{ display: 'flex', justifyContent: 'flex-end' }}>
      {currentIndex === 1 && tabKey === 1 && (
        <Button
          onClick={() => {
            handleBtn({ tabKey });
          }}
          style={{ marginRight: 20 }}
        >
          确认移除
        </Button>
      )}
      <PopTip content={content} visible={popTipVisible} onClose={onClose}>
        <Button type="primary" onClick={handleNext}>
          {currentIndex < 2 ? '下一步' : '完成'}
        </Button>
      </PopTip>
      {currentIndex <= 2 && currentIndex > 0 && (
        <Button style={{ marginLeft: '20px' }} onClick={handlePrev}>
          上一步
        </Button>
      )}
    </div>
  );
};

class Add extends React.Component {
  tags = 0;
  searchKeys = {};
  hideTage = null;
  copyBranchVos = {};
  selectedStorage = [];

  constructor(props) {
    super(props);
    this.init = this.init.bind(this);
    this.updatePromotionType = this.updatePromotionType.bind(this);
    this.setContinueReduceCondition = this.setContinueReduceCondition.bind(this);
    this.renderDiscountCondition = this.renderDiscountCondition.bind(this);
    this.updateContinueType = this.updateContinueType.bind(this);
    this.updateSameSubjectCheck = this.updateSameSubjectCheck.bind(this);
    this.updateRuleUnit = this.updateRuleUnit.bind(this);
    this.renderTabTopPagination = this.renderTabTopPagination.bind(this);
    this.renderSelectedProductsNumber = this.renderSelectedProductsNumber.bind(this);
    this.handleResetSearch = this.handleResetSearch.bind(this);
    // 是否为 满减或折扣优惠
    this.isFullReduceOrDiscountPromotion = [DiscountPromotion, FullReducePromotion].includes(this.props.type);
    this.state = {
      currentIndex: 0,
      // 优惠场景
      promotionArea: null,
      ruleVos: [{}],
      perFullReduce: {},
      promotionType: mapDefaultPromotionType[this.props.type],
      ruleUnit: 'MONEY',
      contractCondition: 'ALL',
      customerCondition: 'ALL',
      // 连报相关
      continuityType: 'NONE',
      continuitySubjectType: 0,
      sameSubjectCheck: false,
      //
      branchSecond: [],

      currentPage: 1,
      pageSize: 100,
      totalRecords: 0,
      list: [],
      selectedListKeys: [],
      selectedList: [],
      selectPage: 1,
      selectPageSize: 100,
      selectedRowKeysTwo: [],

      expandedRowKeys: [],

      stepTwoTabKey: 0, //step 2 tab key
      averagePromotionAcount: 0, // 最大优惠分摊金额
      popTipVisible: false, // 第2步校验不通过提示框
      content: '', //提示框内容
    };
    // 满减优惠-满减条件 单选配置
    this.radioCfg = [
      { text: '金额', value: MONEY },
      // { text: '小时', value: COURSE_HOURS },
      { text: '产品数', value: PRODUCT_NUMS, disabled: [PROMOTION_FULL_SALE_EVERY] },
    ];
  }

  componentDidMount() {
    this.init();
  }

  async init() {
    let { id } = this.props;
    this.setState({ loading: true });
    let [{ data: cusData }, { data: conData }] = await Promise.all([
      getSelectOption({ category: 'PromotionCusCondition' }),
      getSelectOption({ category: 'PromotionConCondition' }),
      id ? this.loadDataById(id) : null,
    ]);
    this.setState({ loading: false });
    let cusDict = mapDataToDict(cusData);
    let conDict = mapDataToDict(conData);
    this.setState({
      cusDict,
      conDict,
    });
  }

  /**
   * 编辑时根据id获取信息
   * @param promotionId
   * @returns {Promise.<void>}
   */
  loadDataById = async (promotionId) => {
    let { data } = await getPromotionInfoById({ id: promotionId });
    let {
      promotionName,
      startDate,
      endDate,
      branchVos,
      promotionTypeId,
      ruleVos,
      usePlat,
      customerCondition,
      contractCondition,
      remark,
      promotionComment,
      productVos,
      id,
      status,
      hightAmount,
      continuityType,
      continuitySubjectType,
      useCase,
    } = data;
    this.selectedStorage = productVos.map((v, k) => {
      return { ...v.productVo, promotionId: v.promotionId, id: v.id, productId: v.productId };
    });

    let payload = {
      promotionName,
      promotionArea: useCase || 0,
      startDate: moment(startDate, dateFormatCfg),
      endDate: moment(endDate, dateFormatCfg),
      branchVos: branchVos.map((v, k) => v.branchId),
      promotionType: promotionTypeId,
      ruleVos,
      usePlat,
      customerCondition,
      contractCondition,
      remark,
      explain: promotionComment,
      selectedList: productVos.map((v, k) => {
        return { ...v.productVo, promotionId: v.promotionId, id: v.id, productId: v.productId };
      }),
      id,
      status,
      perFullReduce: { ...ruleVos[0], maxAmount: hightAmount },
      ruleUnit: ruleVos[0].ruleUnit,
      continuityType: continuityType || 'NONE',
      continuitySubjectType: continuitySubjectType || 0,
      sameSubjectCheck: continuitySubjectType && continuitySubjectType === 1,
    };

    payload.ruleVos = transformRuleVosToHours(ruleVos, this.props.type);

    payload = detailDataAdapter(payload);

    this.setState(payload);
  };
  next = () => {
    let { currentIndex, ruleVos, selectedList, continuityType, promotionType, ruleUnit } = this.state;
    let { branchDict, type } = this.props;
    let error = (text) => getWrappedInstance(this.pageRef).showOkTips(text);
    // 是否满减产品数/产品折扣的连报优惠
    let isContinuePromotion =
      continuityType !== NONE &&
      ((promotionType === PROMOTION_FULL_SALE_NORMAL && ruleUnit === PRODUCT_NUMS) ||
        promotionType === PROMOTION_DISCOUNT_PRODUCT);

    if (currentIndex === 0) {
      let {
        promotionName,
        startDate,
        endDate,
        branchVos = [],
        ruleVos,
        perFullReduce,
        contractCondition,
        customerCondition,
        explain,
        remark,
        cusDict,
        conDict,
        promotionArea,
      } = this.state;
      if (!promotionName) {
        this.setState({ popTipVisible: true, content: '请输入活动名称！' });
        return;
      }
      if (!startDate) {
        this.setState({ popTipVisible: true, content: '请输入起始时间！' });
        return;
      }
      if (!endDate) {
        this.setState({ popTipVisible: true, content: '请输入终止时间！' });
        return;
      }
      if (!branchVos.length) {
        this.setState({ popTipVisible: true, content: '请选择所属分公司！' });
        return;
      }
      if (!promotionType) {
        this.setState({ popTipVisible: true, content: '请选择活动类型！' });
        return;
      }
      if (typeof promotionArea !== 'number') {
        this.setState({ popTipVisible: true, content: '请选择优惠场景！' });
        return;
      }

      if (promotionType === PROMOTION_FULL_SALE_EVERY) {
        if (!perFullReduce['promotionRule'] || !perFullReduce['promotion'] || !perFullReduce['maxAmount']) {
          this.setState({ popTipVisible: true, content: '请完善满减条件！' });
          return;
        }
        if (ruleUnit === 'MONEY' && perFullReduce['promotionRule'] < perFullReduce['promotion']) {
          this.setState({ popTipVisible: true, content: '满减金额不能大于满减条件，请改正！' });
          return;
        }
      } else {
        let checkRule = ruleVos.filter((v, k) => {
          if (isContinuePromotion && !v.promotion) {
            let checkContinuePromotion =
              Array.isArray(v.detailList) &&
              v.detailList.filter((item) => typeof item.promotion !== 'number').length === 0;
            return v.promotionRule === undefined || !checkContinuePromotion;
          } else {
            return v.promotionRule === undefined || v.promotion === undefined;
          }
        });
        if (checkRule.length > 0) {
          this.setState({ popTipVisible: true, content: '请完善满减条件！' });
          return;
        }
        let len = ruleVos.length;
        let checkedOne, checkedTwo;

        if (promotionType === PROMOTION_FULL_SALE_NORMAL && ruleUnit === 'MONEY') {
          for (let i = 0; i < len; i++) {
            if (ruleVos[i].promotionRule < ruleVos[i].promotion) {
              checkedTwo = true;
              break;
            }
          }
        }
        if (checkedTwo) {
          this.setState({ popTipVisible: true, content: '满减金额不能大于满减条件，请改正！' });
          return;
        }
        let array = ruleVos.map((v, k) => v.promotionRule);
        if (array.length !== [...new Set(array)].length) {
          this.setState({ popTipVisible: true, content: '满减条件不能重复，请改正！' });
          return;
        }
      }

      if (!contractCondition) {
        this.setState({ popTipVisible: true, content: '请选择合同条件！' });
        return;
      }
      if (!customerCondition) {
        this.setState({ popTipVisible: true, content: '请选择客户条件！' });
        return;
      }

      let customerConditionName = getTextByDictValue(cusDict, customerCondition);
      let contractConditionName = getTextByDictValue(conDict, contractCondition);
      let branchName = getTextByDictMultipleValue(branchDict, branchVos);

      let unit = conditionUnitCfg[promotionType];
      if (this.props.type === FullReducePromotion) {
        unit = unit[ruleUnit];
      }
      let handle = promotionsTypeCfg[promotionType];
      let result = promotionsUnitCfg[promotionType];
      let condition = '';
      if (promotionType === PROMOTION_FULL_SALE_EVERY) {
        condition = `每满${perFullReduce['promotionRule']}${perFullReduce['ruleUnitName'] || unit}减${
          perFullReduce['promotion']
        }元，最高减${perFullReduce['maxAmount']}元`;
      } else {
        ruleVos.forEach((v, k) => {
          if (isNeedContinue && Array.isArray(v.detailList) && !v.promotion) {
            let str = `满${v.promotionRule}${unit}`;
            v.detailList.forEach((item, index) => {
              str += `${continuityPromotionsTypeCfg[this.props.type][continuityType][index]}${item.promotion}${result}`;
            });
            condition += `${str}；`;
          } else {
            condition += `满${v.promotionRule}${unit}${handle}${v.promotion}${result}；`;
          }
        });
      }
      let data = [
        { name: '优惠活动名称', info: promotionName },
        { name: '使用平台', info: null },
        { name: '活动时间', info: `${startDate.format(dateFormatCfg)}至${endDate.format(dateFormatCfg)}` },
        { name: '客户条件', info: customerConditionName },
        { name: '所属分公司', info: branchName },
        { name: '合同条件', info: contractConditionName },
        { name: '状态', info: status === '1' ? '无效' : '有效' },
        { name: '种类', info: '' },
        { name: '类型', info: mapType[promotionType] },
        { name: '使用条件', info: condition },
        { name: '活动说明', info: explain },
        { name: '活动备注', info: remark },
      ];

      this.setState({ data });
      if (this.tags === 0 || JSON.stringify(this.copyBranchVos) !== JSON.stringify(branchVos)) {
        this.getSecondOption();
        this.copyBranchVos = branchVos;
        this.tags = 1;
      } else {
        const { conditions, ruleUnit, promotionType } = this.state;
        let selectProduct;
        if (ruleUnit === 'PRODUCT_NUMS') {
          selectProduct = this.productDict.filter((v, k) => {
            return v.val === 'SMALL_CLASS' || v.val === 'TWO_TEACHER';
          });
        }
        if (
          ruleUnit === 'COURSE_HOURS' ||
          promotionType === PROMOTION_DISCOUNT_COURSE ||
          promotionType === PROMOTION_GIVE_HOURS
        ) {
          selectProduct = this.productDict.filter((v, k) => {
            return v.val === 'ONE_ON_ONE_COURSE' || v.val === 'ONE_ON_MANY';
          });
        }
        if (promotionType === PROMOTION_DISCOUNT_PRODUCT) {
          selectProduct = this.productDict.filter((v, k) => {
            return 'SMALL_CLASS,TWO_TEACHER,LIVE,LECTURE,OTHERS'.includes(v.val);
          });
        }
        let lastProductDict = selectProduct ? selectProduct : this.productDict;
        conditions[0]['dict'] = lastProductDict;
        this.setState({ conditions });
        this.getProductList();
      }
    } else if (currentIndex === 1) {
      // 除折扣外的优惠走这个判断条件
      if ([PROMOTION_FULL_SALE_NORMAL, PROMOTION_FULL_SALE_EVERY, PROMOTION_GIVE_HOURS].includes(promotionType)) {
        let totalAmountArray = selectedList
          .map((value, index, selectedList) => {
            return value.totalAmount;
          })
          .filter((value) => {
            return value !== null;
          });
        let min = Math.min(...totalAmountArray);
        // 满多少件减多少元之后，平均每件减的数额
        let promotionArray = ruleVos.map((value, index, rule) => {
          return (value.promotion / value.promotionRule).toFixed(2) - 0;
        });

        let averagePromotionAcount = Math.max(...promotionArray).toFixed(2) - 0;
        if (min < averagePromotionAcount) {
          this.setState({
            popTipVisible: true,
            content: `每件产品的总价必须大于最大优惠分摊金额：${averagePromotionAcount}元`,
          });
          return;
        }
      }
    } else if (currentIndex === 2) {
      this.saveData();
    }
    if (currentIndex < 2) {
      this.setState({ currentIndex: currentIndex + 1 }, () => {
        if (currentIndex === 0 && this.props.id) {
          this.$changeTabs.onChange(1);
        }
      });
    }
  };
  /**
   * 保存优惠信息
   * @returns {Promise.<void>}
   */
  saveData = async () => {
    let {
      branchVos,
      contractCondition,
      customerCondition,
      startDate,
      endDate,
      explain,
      remark,
      promotionName,
      promotionType,
      ruleVos,
      perFullReduce,
      ruleUnit,
      limitProductType,
      id,
      status,
      continuityType,
      continuitySubjectType,
      promotionArea,
    } = this.state;
    const { actionType } = this.props;
    // 是否满减产品数/产品折扣的连报优惠
    let isContinuePromotion =
      continuityType !== NONE &&
      ((promotionType === PROMOTION_FULL_SALE_NORMAL && ruleUnit === PRODUCT_NUMS) ||
        promotionType === PROMOTION_DISCOUNT_PRODUCT);
    let branchVosData = branchVos.map((v, k) => {
      return { branchId: v, id: 0, promotionId: 0 };
    });
    let unit = 'MONEY';
    switch (promotionType) {
      case PROMOTION_DISCOUNT_CASH:
        ruleUnit = 'MONEY';
        break;
      case PROMOTION_DISCOUNT_COURSE:
        ruleUnit = 'COURSE_HOURS';
        break;
      case PROMOTION_DISCOUNT_PRODUCT:
        ruleUnit = 'PRODUCT_NUMS';
        break;
      case PROMOTION_GIVE_HOURS:
        ruleUnit = 'COURSE_HOURS';
        unit = 'COURSE_HOURS';
        break;
    }
    let ruleVosData;
    if (promotionType === PROMOTION_FULL_SALE_EVERY) {
      ruleVosData = [
        {
          giftVos: [],
          id: actionType === 'Copy' ? null : perFullReduce.id,
          promotion: perFullReduce.promotion,
          promotionId: actionType === 'Copy' ? null : perFullReduce.promotionId,
          promotionIds: 0,
          promotionRule: perFullReduce.promotionRule,
          ruleName: actionType === 'Copy' ? '' : perFullReduce.useNum,
          ruleUnit,
          unit,
          useNum: actionType === 'Copy' ? 0 : perFullReduce.useNum,
        },
      ];
    } else {
      ruleVosData = ruleVos.map((v, k) => {
        let rule = {
          giftVos: [],
          id: actionType === 'Copy' ? null : v.id,
          promotionId: actionType === 'Copy' ? null : v.promotionId,
          promotionIds: 0,
          promotionRule: v.promotionRule,
          ruleName: actionType === 'Copy' ? '' : v.useNum,
          ruleUnit,
          unit,
          useNum: actionType === 'Copy' ? 0 : v.useNum,
        };
        if (isContinuePromotion && !v.promotion) {
          if (actionType === 'Copy') {
            rule.detailList = v.detailList.map((detail) => ({
              ...detail,
              id: null,
            }));
          } else {
            rule.detailList = v.detailList;
          }
          rule.promotion = null;
        } else {
          rule.detailList = [];
          rule.promotion = v.promotion;
        }
        return rule;
      });
    }
    let productVosData = this.buildProductVos();
    // 复制的时候去掉原id
    if (actionType === 'Copy') {
      productVosData = productVosData.map((prd) => ({
        ...prd,
        id: null,
      }));
    }

    let apiParams = {
      branchVos: branchVosData, //分公司信息
      // 优惠场景
      useCase: promotionArea,
      contractCondition, //合同条件
      customerCondition, //用户条件
      startDate: startDate.format(dateFormatCfg), //开始日期
      endDate: endDate.format(dateFormatCfg), //结束日期
      firstLevel: 0, //暂时为0
      hightAmount: promotionType === 'PROMOTION_FULL_SALE_EVERY' ? perFullReduce['maxAmount'] : null, //最高优惠金额
      id: actionType === 'Copy' ? 0 : id,
      limitProductType: limitProductType || 'ALL',
      productVos: productVosData, //对应优惠产品信息
      promotionCode: '', //优惠码 暂时传空
      promotionComment: explain, //活动说明，
      promotionKindId: 'PROMOTION', //优惠类型
      promotionLittleTypeId: mapPromotionType(this.props.type), //优惠小类
      promotionName: promotionName, //优惠名字
      promotionTypeId: promotionType, //小小类
      publishNum: 0, //发布数量
      receiveEndDate: null, //领取结束时间
      receiveStartDate: null, //领取开始时间
      receiveLimit: 'ALL', //领取限制
      remark, //备注
      ruleVos: ruleVosData, //规则信息
      status: Number(status) || 0,
      superPosition: 0,
      useNum: 0, //使用数量 新增为空
    };
    const pageType = this.props.type;
    // 连报参数
    if (pageType === FullReducePromotion || pageType === DiscountPromotion) {
      apiParams.continuityType = continuityType;
      apiParams.continuitySubjectType = continuitySubjectType;
    }

    apiParams = submitDataAdapter(apiParams);

    // 小时数转换课时数
    apiParams.ruleVos = transformRuleVosToCourse(apiParams.ruleVos, this.props.type);

    await savePromotion(apiParams);
    showMessage('保存成功', 'success');
    this.props.onSaveSuccess();
  };

  setPromotionType = (val) => {
    this.setState({
      promotionType: val,
      ruleUnit: 'MONEY',
    });
  };
  // 更新活动类型
  updatePromotionType(val) {
    let payload = {
      promotionType: val,
      ruleUnit: 'MONEY',
    };
    if (this.state.continuityType !== NONE) {
      let deleteKeyName = [PROMOTION_FULL_SALE_NORMAL, PROMOTION_DISCOUNT_PRODUCT].includes(val)
        ? 'promotion'
        : 'detailList';
      let ruleVos = clone(this.state.ruleVos);
      ruleVos.forEach((item) => {
        delete item[deleteKeyName];
      });
      payload.ruleVos = ruleVos;
    }
    this.setState(payload);
  }
  /**
   * 设置优惠条件
   * @param type
   * @param value
   * @param index
   * @param item
   */
  setReduceCondition = (type, value, index, item) => {
    let { ruleVos, perFullReduce } = this.state;
    if (type === PROMOTION_FULL_SALE_EVERY) {
      perFullReduce[item] = value;
      this.setState({ perFullReduce });
    } else {
      ruleVos[index] = ruleVos[index] || {};
      ruleVos[index][item] = value;
      this.setState({ ruleVos });
    }
  };
  // 设置连报优惠条件
  setContinueReduceCondition(promotion, quarter, detailListIndex, ruleVosIndex) {
    let ruleVos = clone(this.state.ruleVos);
    if (!ruleVos[ruleVosIndex].detailList) {
      ruleVos[ruleVosIndex].detailList = [];
    }
    if (!ruleVos[ruleVosIndex].detailList[detailListIndex]) {
      ruleVos[ruleVosIndex].detailList[detailListIndex] = {};
    }
    ruleVos[ruleVosIndex].detailList[detailListIndex].promotion = promotion;
    ruleVos[ruleVosIndex].detailList[detailListIndex].quarter = quarter;
    let otherDetailListIndex = detailListIndex === 0 ? 1 : 0;
    if (!ruleVos[ruleVosIndex].detailList[otherDetailListIndex]) {
      ruleVos[ruleVosIndex].detailList[otherDetailListIndex] = {
        promotion: null,
        quarter: null,
      };
    }
    this.setState({ ruleVos });
  }
  // 渲染满减条件
  renderDiscountCondition() {
    let { promotionType, ruleVos, perFullReduce = {}, ruleUnit = MONEY, continuityType } = this.state;
    const prefixText = promotionsPrefixCfg[promotionType];
    let prefixUnit = conditionUnitCfg[promotionType];
    if (this.props.type === FullReducePromotion) {
      prefixUnit = prefixUnit[ruleUnit];
    }
    if (typeof prefixUnit !== 'string') {
      prefixUnit = '';
    }
    let resultText = promotionsTypeCfg[promotionType];
    let resultUnit = promotionsUnitCfg[promotionType];
    // 这里不应该拆的，应该全部都放在ruleVos里(暂不修改，改造成本略高，整份文件涉及该逻辑的都必须改) FWord!
    if (promotionType === PROMOTION_FULL_SALE_EVERY) {
      return (
        <div className={styles.perFullReduce}>
          <span className={styles.reduceConditionItem}>
            <span>{prefixText}</span>
            <InputNumber
              value={perFullReduce.promotionRule}
              min={0}
              precision={ruleUnit === 'MONEY' ? 2 : 0}
              max={9999999}
              onChange={(val) => this.setReduceCondition(promotionType, val, 0, 'promotionRule')}
            />
            <span>{prefixUnit}</span>
          </span>
          <span className={styles.reduceConditionItem}>
            <span>{resultText}</span>
            <InputNumber
              value={perFullReduce.promotion}
              min={0}
              precision={2}
              max={9999999}
              onChange={(val) => this.setReduceCondition(promotionType, val, 0, 'promotion')}
            />
            <span>{resultUnit}</span>
          </span>
          <span className={styles.reduceConditionItem}>
            <span>最高减</span>
            <InputNumber
              value={perFullReduce.maxAmount}
              min={0}
              precision={2}
              max={9999999}
              onChange={(val) => this.setReduceCondition(promotionType, val, 0, 'maxAmount')}
            />
            <span>{resultUnit}</span>
          </span>
        </div>
      );
    } else {
      let len = ruleVos.length;
      let isNeedContinue = false;
      // 连报 且是 满减+产品数 或 产品折扣
      checkContinuePromotion(
        {
          continuityType,
          promotionType,
          ruleUnit,
        },
        () => {
          isNeedContinue = true;
        },
      );
      return ruleVos.map((v, k) => {
        return (
          <div key={k} className={styles.reduceCondition}>
            <span className={styles.reduceConditionItem}>
              <span>{prefixText}</span>
              <InputNumber
                value={v.promotionRule}
                min={0}
                precision={prefixUnit === '元' ? 2 : 0}
                max={9999999}
                onChange={(val) => this.setReduceCondition(promotionType, val, k, 'promotionRule')}
              />
              <span>{prefixUnit}</span>
            </span>
            {isNeedContinue && !v.promotion ? (
              <span>
                <span className={styles.reduceConditionItem}>
                  <span>{continuityPromotionsTypeCfg[this.props.type][continuityType][0]}</span>
                  <InputNumber
                    value={v.detailList && v.detailList[0] ? v.detailList[0].promotion : null}
                    min={0}
                    precision={/*promotionType === PROMOTION_FULL_SALE_NORMAL ? 2 : 0*/ 2}
                    max={9999999}
                    onChange={(val) =>
                      this.setContinueReduceCondition(val, continuityQuarterCfg[continuityType][0], 0, k)
                    }
                  />
                  <span>{resultUnit}</span>
                </span>
                <span className={styles.reduceConditionItem}>
                  <span>{continuityPromotionsTypeCfg[this.props.type][continuityType][1]}</span>
                  <InputNumber
                    value={v.detailList && v.detailList[1] ? v.detailList[1].promotion : null}
                    min={0}
                    precision={/*promotionType === PROMOTION_FULL_SALE_NORMAL ? 2 : 0*/ 2}
                    max={9999999}
                    onChange={(val) =>
                      this.setContinueReduceCondition(val, continuityQuarterCfg[continuityType][1], 1, k)
                    }
                  />
                  <span>{resultUnit}</span>
                </span>
              </span>
            ) : (
              <span className={styles.reduceConditionItem}>
                <span>{resultText}</span>
                <InputNumber
                  value={v.promotion}
                  min={0}
                  precision={/*promotionType === PROMOTION_FULL_SALE_NORMAL ? 2 : 0*/ 2}
                  max={
                    promotionType === PROMOTION_FULL_SALE_NORMAL || promotionType === PROMOTION_GIVE_HOURS
                      ? 9999999
                      : 100
                  }
                  onChange={(val) => this.setReduceCondition(promotionType, val, k, 'promotion')}
                />
                <span>{resultUnit}</span>
              </span>
            )}
            <span className={styles.reduceConditionItem}>
              {len === 1 && (
                <IconButton
                  onClick={(e) => this.changeReduceCondition(k, 'plus')}
                  type="plus_circle"
                  size={20}
                  style={{ position: 'relative', top: 5 }}
                />
              )}
              {len !== 1 && k + 1 < len && (
                <IconButton
                  onClick={(e) => this.changeReduceCondition(k, 'minus')}
                  type="minus_circle"
                  size={20}
                  style={{ position: 'relative', top: 5 }}
                />
              )}
              {len !== 1 && k + 1 === len && (
                <span>
                  <IconButton
                    onClick={(e) => this.changeReduceCondition(k, 'plus')}
                    type="plus_circle"
                    size={20}
                    style={{ position: 'relative', top: 5, marginRight: 20 }}
                  />
                  <IconButton
                    onClick={(e) => this.changeReduceCondition(k, 'minus')}
                    type="minus_circle"
                    size={20}
                    style={{ position: 'relative', top: 5 }}
                  />
                </span>
              )}
            </span>
          </div>
        );
      });
    }
  }

  changeReduceCondition = (k, type) => {
    let { ruleVos, promotionType } = this.state;
    if (type === 'plus') {
      if (ruleVos.length === 10) {
        showMessage('最多可设置10档！');
        return;
      }
      if (promotionType === PROMOTION_GIVE_HOURS && ruleVos.length === 10) {
        showMessage('最多可设置10档！');
        return;
      }
      ruleVos.push({});
    } else {
      ruleVos.splice(k, 1);
    }
    this.setState({ ruleVos });
  };
  getDisabledStartDate = (startDate) => {
    let { endDate } = this.state;
    if (!startDate || !endDate) {
      return false;
    }
    return moment(startDate).valueOf() > moment(endDate).valueOf();
  };
  getDisabledEndDate = (endDate) => {
    const { startDate } = this.state;
    if (!endDate || !startDate) {
      return false;
    }
    return moment(endDate).valueOf() < moment(startDate).valueOf();
  };

  /**
   * 获取第二步筛选数据
   * @returns {Promise.<void>}
   */
  getSecondOption = async () => {
    this.setState({ loading: true });
    const { branchDict } = this.props;
    const { ruleUnit, promotionType } = this.state;

    const categoryArr =
      'PRODUCT_TYPE, PRODUCT_CHARACTER, STUDENT_GRADE, SUBJECT, PRODUCT_QUARTER, SMALL_CLASS_PHASE, CLASS_TYPE, PRODUCT_GROUP, COURSE_SERIES, PRODUCT_VERSION';
    const [batchDict] = await Promise.all([loadCommonDictByGroup(categoryArr), this.getProductList()]);
    const {
      productDict,
      productCharacterDict,
      gradeDict,
      subjectDict,
      quarterDict,
      phaseDict,
      classDict,
      groupDict,
      seriesDict,
      yearDict,
    } = this.formatDictResponse(batchDict);

    this.productDict = productDict;
    let selectProduct;
    if (
      ruleUnit === 'PRODUCT_NUMS' ||
      promotionType === PROMOTION_DISCOUNT_PRODUCT ||
      promotionType === PROMOTION_DISCOUNT_CASH
    ) {
      selectProduct = productDict.filter((v, k) => {
        return v.val === 'SMALL_CLASS' || v.val === 'TWO_TEACHER';
      });
    }
    if (
      ruleUnit === 'COURSE_HOURS' ||
      promotionType === PROMOTION_DISCOUNT_COURSE ||
      promotionType === PROMOTION_GIVE_HOURS
    ) {
      selectProduct = productDict.filter((v, k) => {
        return v.val === 'ONE_ON_ONE_COURSE' || v.val === 'ONE_ON_MANY';
      });
    }
    /** @desc 20210406版 去除类型为其他的产品类型 */
    const lastProductDict = (selectProduct ? selectProduct : productDict).filter((r) => r.val !== 'OTHERS');
    const baseCondition = [
      { title: '产品类型', field: 'productType', dict: lastProductDict, isMultiSelect: true },
      { title: '分公司', field: 'branchSecond', dict: branchDict },
      { title: '年份', field: 'year', dict: yearDict },
      { title: '年级', field: 'grade', dict: gradeDict },
      { title: '科目', field: 'subject', dict: subjectDict },
    ];

    let moreConditions = [];
    if (promotionType === PROMOTION_GIVE_HOURS) {
      moreConditions = [
        { title: '产品组', field: 'group', dict: groupDict },
        { title: '产品系列', field: 'series', dict: seriesDict },
      ];
    } else {
      moreConditions = [
        { title: '季度', field: 'holiday', dict: quarterDict },
        { title: '期数', field: 'periods', dict: phaseDict },
        { title: '班级类型', field: 'classType', dict: classDict },
        { title: '产品组', field: 'group', dict: groupDict },
        { title: '产品系列', field: 'series', dict: seriesDict },
      ];
    }

    /** @desc 0520增加产品性质维度筛选 */
    if (this.isFullReduceOrDiscountPromotion) {
      let first = baseCondition.shift();
      baseCondition.unshift({
        title: '产品性质',
        field: 'productCharacter',
        dict: productCharacterDict,
        isMultiSelect: true,
      });
      baseCondition.unshift(first);
    }

    const conditions = [...baseCondition, ...moreConditions].map((item) => {
      if (item.isMultiSelect) {
        // 已经支持多选的
        return item;
      } else {
        return { ...item, isMultiSelect: this.isFullReduceOrDiscountPromotion };
      }
    });
    this.setState({ conditions });
  };
  /**
   * 格式化批量字典返回值
   *
   * @param dicts
   */
  formatDictResponse = (dicts = {}) => {
    return {
      productDict: dicts['PRODUCT_TYPE'],
      productCharacterDict: dicts['PRODUCT_CHARACTER'],
      gradeDict: dicts['STUDENT_GRADE'],
      subjectDict: dicts['SUBJECT'],
      quarterDict: dicts['PRODUCT_QUARTER'],
      phaseDict: dicts['SMALL_CLASS_PHASE'],
      classDict: dicts['CLASS_TYPE'],
      groupDict: dicts['PRODUCT_GROUP'],
      seriesDict: dicts['COURSE_SERIES'],
      yearDict: dicts['PRODUCT_VERSION'],
    };
  };

  getProductList = async (pageKeys = {}) => {
    await this.initSelectedKeysByLoadList();
    let { promotionType, ruleUnit, branchVos } = this.state;
    let loadCourse =
      promotionType === PROMOTION_DISCOUNT_COURSE ||
      promotionType === PROMOTION_GIVE_HOURS ||
      ruleUnit === 'COURSE_HOURS';
    let loadProduct = promotionType === PROMOTION_DISCOUNT_PRODUCT;
    let types = this.searchKeys.productType ? this.searchKeys.productType.join(',') : '';

    //当为选择产品数时，只加载小班、双师
    if (ruleUnit === 'PRODUCT_NUMS' && types === '') {
      types = 'SMALL_CLASS,TWO_TEACHER';
    }
    //当为选择课时的时候 只加载一对一 一对多数据
    if (loadCourse && types === '') {
      types = 'ONE_ON_MANY,ONE_ON_ONE_COURSE';
    }

    //当为选择产品的时候 只加载小班、双师、直播、讲座、其他数据
    if (loadProduct && types === '') {
      types = 'SMALL_CLASS,TWO_TEACHER,LIVE,LECTURE,OTHERS';
    }
    // 分公司
    let branchIds = [];
    if (this.searchKeys.branchSecond) {
      let tempVal = this.searchKeys.branchSecond;
      if (this.isFullReduceOrDiscountPromotion) {
        // 折扣，满减多选
        branchIds = tempVal.filter((r) => !!r);
      } else {
        branchIds = [tempVal];
      }
    }

    if (!branchIds || !branchIds.length) {
      branchIds = branchVos;
    }

    let apiParams = {
      branchIds: branchIds,
      name: this.searchKeys['inputValue'],
      productCharacters: this.searchKeys.productCharacter,
      yearIds: this.searchKeys.year,
      gradeIds: this.searchKeys.grade,
      subjects: this.searchKeys.subject,
      seasonIds: this.searchKeys.holiday,
      phaseIds: this.searchKeys.periods,
      classTypeIds: this.searchKeys.classType,
      productGroupIds: this.searchKeys.group,
      courseSeriesIds: this.searchKeys.series,
      promotionId: '',
      types,
      // 页数是从0开始
      pageNo: pageKeys.currentPage ? pageKeys.currentPage - 1 : 0,
      pageSize: pageKeys.pageSize || this.state.pageSize,
      showOtherPromotion: this.hideTage,
    };
    let { datas, rowCount } = await getProductList(apiParams);
    this.setState({ list: datas, totalRecords: rowCount, loading: false, selectedRowKeys: [] });
    this.initSelectedByLoadList({ list: datas });
  };

  /**
   * 点击筛选项加载产品数据
   * @param keys
   * @returns {Promise.<void>}
   */
  onSearch = async (keys) => {
    this.searchKeys = keys;
    await this.setState({ loading: true, currentPage: 1 });
    this.getProductList();
  };

  handleResetSearch() {
    this.searchKeys = {};
    this.setState({ loading: true, currentPage: 1 }, this.getProductList);
  }

  getListByChangePage = async (pageKeys) => {
    await this.setState({ loading: true, currentPage: pageKeys.currentPage, pageSize: pageKeys.pageSize });
    this.getProductList(pageKeys);
  };
  onInputChange = (value) => {
    this.searchKeys['inputValue'] = value;
  };
  // step2 表格操作_查看优惠
  expandedRowRender = (record) => {
    let { infoVos } = record;
    infoVos = infoVos.filter((v, k) => {
      return (
        v.promotionLittleTypeId === 'PROMOTION_FULL_SALE' ||
        v.promotionLittleTypeId === 'PROMOTION_DISCOUNT' ||
        v.promotionLittleTypeId === 'PROMOTION_GIVE'
      );
    });
    return (
      <div className={'CardList-expand-wrap'}>
        <div className={styles.expandContent}>
          {infoVos.map((v, k) => (
            <div key={k} className={styles.item}>
              <span>
                <span>{v.promotionTypeName}</span>
              </span>
              <span>
                <div>{v.promotionName}</div>
                <div>
                  <span>{getUseCondition(v)}</span>
                </div>
              </span>
            </div>
          ))}
        </div>
      </div>
    );
  };

  onExpand = (index, record) => {
    let { expandedRowKeys } = this.state;
    let idx = expandedRowKeys.findIndex((v, k) => {
      return v === index;
    });
    if (idx === -1) {
      expandedRowKeys.push(index);
    } else {
      expandedRowKeys.splice(idx, 1);
    }
    this.setState({ expandedRowKeys });
  };

  selectChangeTwo = (selectedRowKeysTwo, selectedRows) => {
    this.setState({ selectedRowKeysTwo });
  };

  DontShowList = (e) => {
    this.hideTage = e.target.checked ? 1 : null;
    this.setState({ loading: true });
    this.getProductList();
  };
  // 监听Tab key
  onSwitchTab = (params) => {
    this.setState((prevState) => ({ stepTwoTabKey: params }));
  };
  /**
   * 在线报读4期
   * */
  updateState = (newState) => {
    this.setState((prevState, props) => {
      return {
        ...newState,
      };
    });
  };
  /**
   * 选择产品
   * list:当前页面选中项[array]
   * selectedList:全部选中项[array]
   * this.selectStorage:修改/编辑时，从接口获取的已选中项
   * */
  chooseItem = (keys) => {
    if (this.checkSelectType(keys)) {
      // 增加选择
      const list = this.getSelectedItem(keys);
      let selectedList = this.buildSelectedListByAdd(this.selectedStorage, list);
      this.selectedStorage = clone(selectedList);
      this.setState({ selectedList });
    } else {
      // 删减选择
      let subItemArr = this.getSubItemArr(keys);
      let selectedList = this.buildSelectedListBySub(this.selectedStorage, subItemArr);
      this.selectedStorage = clone(selectedList);
      this.setState({ selectedList });
    }
    this.setSelectedKeys(keys);
  };
  // 判断chooseItem是增操作还是减操作
  checkSelectType = (params) => {
    const { selectedListKeys } = this.state;
    return params.length > selectedListKeys.length;
  };
  // 设置选中状态,更新selectedListKeys,返回已选中项的数据 - 用于chooseItem
  setSelectedKeys = (params) => {
    this.updateState({ selectedListKeys: params });
  };
  // 获取当前列表的选中项
  getSelectedItem = (params) => {
    const { list } = this.state;
    return params.map((item) => list[item]);
  };
  // 构建参数SelectedList增 - 用于chooseItem
  buildSelectedListByAdd = (origin, other) => {
    const otherDiff = differenceWith(other, origin, this.diffComparator);
    return origin.concat(otherDiff).map(this.auxBuildItem);
  };
  // 辅助构建item
  auxBuildItem = (item, index, arr) => {
    let id = item.productId ? item.id : 0;
    let productId = item.productId || item.id;
    return {
      ...item,
      productId,
      id,
    };
  };
  // 比较两数组各项是否为同一项 - 用于differenceWith, intersectionWith
  diffComparator = (origin, other) => {
    return origin.id === other.id || origin.id === other.productId;
  };
  diffComparatorBySub = (origin, other) => {
    return origin.productId === other.id || origin.productId === other.productId;
  };
  // 在切换页码时初始化已选择优惠相关数据 - 用于getProductList
  initSelectedKeysByLoadList = () => {
    this.updateState({ selectedListKeys: [] });
  };
  // (编辑/修改时)加载产品列表时,更新已选产品 - 用于getProductList
  initSelectedByLoadList = (params) => {
    const { list } = params;
    let same = intersectionWith(list, this.selectedStorage, this.diffComparator);
    let arr = [];
    same.forEach((item) => {
      let checkIndex = indexOf(list, item);
      if (checkIndex !== -1) {
        arr.push(checkIndex);
      }
    });
    this.setSelectedKeys(arr.length > 0 ? arr : []);
  };
  // 将已选数据处理处理成所需(productVos)格式 - 用于saveData
  buildProductVos = () => {
    const { selectedList } = this.state;
    let list = clone(selectedList);
    return list.map((item) => ({
      amount: item.amount || 0,
      id: item.id || 0,
      productId: item.productId,
      promotionId: item.promotionId || 0,
    }));
  };
  // 获取当前去除项
  getSubItemArr = (params) => {
    const { selectedListKeys, list } = this.state;
    let sub = difference(selectedListKeys, params);
    return sub.map((item) => this.auxBuildItem(list[item]));
  };
  // 构建参数SelectedList减 - 用于chooseItem
  buildSelectedListBySub = (origin, other) => {
    return differenceWith(origin, other, this.diffComparatorBySub);
  };
  // 移除选择按钮事件
  subChosenItem = () => {
    const { selectedList, selectedRowKeysTwo, list } = this.state;
    let subItemArr = clone(selectedRowKeysTwo.map((item) => selectedList[item]));
    let sList = this.buildSelectedListBySub(this.selectedStorage, subItemArr);
    this.selectedStorage = clone(sList);
    this.setState({ selectedList: sList });
    this.updateState({ selectedRowKeysTwo: [] });
    this.initSelectedByLoadList({ list });
  };
  // 更新连报条件
  updateContinueType(e) {
    let val = e.target.value;
    let payload = { continuityType: val };
    if (val === NONE) {
      payload.sameSubjectCheck = false;
      payload.continuitySubjectType = isNeedContinue.noNeed;
    }
    const { promotionType, ruleUnit } = this.state;
    let ruleVos = clone(this.state.ruleVos);
    ruleVos.forEach((item) => {
      if (val === NONE) {
        delete item.detailList;
      } else {
        if (Array.isArray(item.detailList)) {
          item.detailList.forEach((detail, index) => {
            detail.quarter = continuityQuarterCfg[val][index];
          });
        }
        checkContinuePromotion({ continuityType: val, promotionType, ruleUnit }, () => {
          delete item.promotion;
        });
      }
    });
    payload.ruleVos = ruleVos;
    this.setState(payload);
  }
  // 更新连报条件的是否相同科目
  updateSameSubjectCheck(e) {
    let val = e.target.checked;
    this.setState({
      sameSubjectCheck: val,
      continuitySubjectType: val ? isNeedContinue.need : isNeedContinue.noNeed,
    });
  }
  // 更新满减条件类型
  updateRuleUnit(e) {
    let val = e.target.value;
    let payload = { ruleUnit: val };
    if (this.state.continuityType !== NONE && this.state.promotionType === PROMOTION_FULL_SALE_NORMAL) {
      let ruleVos = clone(this.state.ruleVos);
      let deleteKeyName = val === PRODUCT_NUMS ? 'promotion' : 'detailList';
      ruleVos.forEach((item) => {
        delete item[deleteKeyName];
      });
      payload.ruleVos = ruleVos;
    }
    this.setState(payload);
  }

  /**
   * @desc 0406调整优化： 折扣促销及满减促销 DiscountPromotion|FullReducePromotion
   * @returns null|Array ReactElement[]
   */
  renderTabTopPagination() {
    const { totalRecords, currentPage, pageSize, selectedList, selectPage, selectPageSize } = this.state;
    return this.isFullReduceOrDiscountPromotion
      ? [
          <Pagination
            showSizeChanger
            showQuickJumper
            pageSizeOptions={['20', '50', '100']}
            total={totalRecords}
            current={currentPage}
            pageSize={pageSize}
            showTotal={(total) => `共${total}条记录`}
            onShowSizeChange={(current, pageSize) => this.getListByChangePage({ currentPage: current, pageSize })}
            onChange={(current, pageSize) => this.getListByChangePage({ currentPage: current, pageSize })}
          />,
          <Pagination
            showSizeChanger
            showQuickJumper
            pageSizeOptions={['20', '50', '100']}
            total={selectedList.length}
            current={selectPage}
            pageSize={selectPageSize}
            showTotal={(total) => `共${total}条记录`}
            onShowSizeChange={(current, pageSize) => this.setState({ selectPage: current, selectPageSize: pageSize })}
            onChange={(current, pageSize) => this.setState({ selectPage: current, selectPageSize: pageSize })}
          />,
        ]
      : [];
  }

  renderSelectedProductsNumber() {
    const { selectedList } = this.state;
    return this.isFullReduceOrDiscountPromotion ? (
      <p style={{ marginBottom: 10 }}>共{selectedList.length}条记录</p>
    ) : null;
  }

  /**
   * end
   */

  render() {
    let { branchDict, typeDict, id, type } = this.props;
    let {
      loading,
      promotionName,
      startDate,
      endDate,
      branchVos,
      promotionType,
      ruleUnit,
      contractCondition,
      customerCondition,
      cusDict = [],
      conDict = [],
      explain,
      remark,
      currentIndex,
      stepTwoTabKey,
      popTipVisible,
      content,
      continuityType,
      sameSubjectCheck,
      conditions = [],
      activeKey = 0,
      promotionArea,
      totalRecords,
      expandedRowKeys,
      currentPage,
      pageSize,
      list,
      selectedList,
      selectedListKeys,
      selectPage,
      selectPageSize,
      selectedRowKeys = [],
      selectedRowKeysTwo,
      viewPage = 1,
      viewPageSize = 10,
      data,
    } = this.state;
    let columns = [
      {
        title: '活动信息',
        dataIndex: 'info',
        width: COL_WIDTH_XL,
        render: (text, record, index) => {
          let { name, organizationName, courseSeriesName, classTypeName, categoryName, category, productCharacter } =
            record;
          return (
            <div className={styles.activityInfo} style={{ paddingLeft: 0 }}>
              <div>{name}</div>
              <div>
                <span>{organizationName}</span>
                <span>{courseSeriesName}</span>
                <span>{classTypeName}</span>
                <span>{categoryName}</span>
                <span>{ProductCharacterMap[productCharacter]}</span>
              </div>
              <span className={styles.tips}>
                <span></span>
                <span>{mapTips[category]}</span>
              </span>
            </div>
          );
        },
      },
      { title: '单价', dataIndex: 'price', width: COL_WIDTH_M },
      { title: '小时', dataIndex: 'miniClassTotalhours', width: COL_WIDTH_M },
      { title: '总价', dataIndex: 'totalAmount', width: COL_WIDTH_M },
      {
        title: '查看',
        dataIndex: 'view',
        width: COL_WIDTH_M,
        render: (text, record, index) => {
          let { infoVos } = record;
          infoVos = infoVos.filter((v, k) => {
            return (
              v.promotionLittleTypeId === 'PROMOTION_FULL_SALE' ||
              v.promotionLittleTypeId === 'PROMOTION_DISCOUNT' ||
              v.promotionLittleTypeId === 'PROMOTION_GIVE'
            );
          });
          return (
            <span onClick={(e) => this.onExpand(index, record)} className={styles.showPromote}>
              {infoVos.length > 0 ? '查看优惠' : ''}
            </span>
          );
        },
      },
    ];
    // 根据活动类型渲染满减条件单选按钮

    //第三步
    let viewColumns = [
      {
        title: '活动信息',
        dataIndex: 'info',
        render: (text, record, index) => {
          let { name, organizationName, courseSeriesName, classTypeName, categoryName, category } = record;
          return (
            <div className={styles.activityInfo}>
              <div>{name}</div>
              <div>
                <span>{organizationName}</span>
                <span>{courseSeriesName}</span>
                <span>{classTypeName}</span>
                <span>{categoryName}</span>
              </div>
              <span>
                <span></span>
                <span>{mapTips[category]}</span>
              </span>
            </div>
          );
        },
      },
      { title: '单价', dataIndex: 'price', width: COL_WIDTH_M },
      { title: '小时', dataIndex: 'miniClassTotalhours', width: COL_WIDTH_M },
      {
        title: '总价',
        dataIndex: 'totalAmount',
        width: COL_WIDTH_M,
        render: (text, record, index) => {
          return <span className={styles.totalPrice}>{record.totalAmount}</span>;
        },
      },
    ];
    return (
      <Page
        loading={loading}
        footer={renderFooter({
          currentIndex,
          handlePrev: (e) => {
            this.setState({ currentIndex: currentIndex - 1 });
          },
          handleNext: this.next,
          tabKey: stepTwoTabKey,
          handleBtn: this.subChosenItem,
          popTipVisible,
          onClose: () => {
            this.setState({ popTipVisible: false });
          },
          content,
        })}
        route={this.props.route}
        className={styles.activity}
        ref={(c) => (this.pageRef = c)}
      >
        <Steps currentIndex={currentIndex}>
          <div>活动信息</div>
          <div>选择产品</div>
          <div>核对完成</div>
        </Steps>
        {currentIndex === 0 && (
          <Block title={'活动信息'}>
            <Row>
              <Col span={12}>
                <FormItem name="活动名称" required={true}>
                  <Input
                    placeholder="请输入"
                    value={promotionName}
                    maxLength={50}
                    onChange={(e) => this.setState({ promotionName: e.target.value })}
                  />
                </FormItem>
              </Col>
              <Col span={12}>
                <FormItem name="优惠场景" required={true}>
                  <Group
                    style={{ display: 'flex' }}
                    onChange={(e) => this.setState({ promotionArea: e.target.value })}
                    value={promotionArea}
                  >
                    {promotionAreaDict.map(({ val, text, warnMsg }) => (
                      <div key={val}>
                        <Radio value={val}>{text}</Radio>
                        <div style={{ lineHeight: '20px', color: 'red' }}>{warnMsg}</div>
                      </div>
                    ))}
                  </Group>
                </FormItem>
              </Col>
              <Col span={12}>
                <FormItem name="起始时间" required={true}>
                  <DatePicker
                    style={{ width: '100%' }}
                    size="large"
                    showTime={{ defaultValue: moment('00:00:00', 'HH:mm:ss') }}
                    format="YYYY-MM-DD HH:mm:ss"
                    value={startDate}
                    onChange={(val) => this.setState({ startDate: val })}
                    disabledDate={(val) => this.getDisabledStartDate(val)}
                  />
                </FormItem>
              </Col>
              <Col span={12}>
                <FormItem name="终止时间" required={true}>
                  <DatePicker
                    style={{ width: '100%' }}
                    size="large"
                    showTime={{ defaultValue: moment('24:00:00', 'HH:mm:ss') }}
                    format="YYYY-MM-DD HH:mm:ss"
                    value={endDate}
                    onChange={(val) => this.setState({ endDate: val })}
                    disabledDate={(val) => this.getDisabledEndDate(val)}
                  />
                </FormItem>
              </Col>
              <Col span={12}>
                <FormItem name="所属分公司" required={true}>
                  <MultiSelect
                    dict={branchDict}
                    value={branchVos}
                    onChange={(val) => this.setState({ branchVos: val })}
                    disabled={!!id}
                  />
                </FormItem>
              </Col>
              <Col span={24} style={{ position: 'relative' }}>
                <FormItem name="活动类型" required={true} style={{ width: '50%' }}>
                  <Select dict={typeDict} value={promotionType} onChange={this.updatePromotionType} />
                </FormItem>
                <IconButton
                  type={'warning'}
                  size={20}
                  style={{ position: 'absolute', top: 8, right: '50%', zIndex: 1 }}
                  title={{
                    title:
                      promotionType === PROMOTION_FULL_SALE_EVERY
                        ? '只能设置一档，用户订单金额满足指定额度，便享受减价优惠，一个订单可重复多次享受优惠'
                        : '最少设置1档，最多可设置10档，满足条件递增设置。用户满足哪档额度，则享受哪一档的减价优惠，一个订单只能享受一次最高档优惠',
                    placement: 'right',
                  }}
                />
              </Col>
              {(type === FullReducePromotion || type === DiscountPromotion) && (
                <Col span={24}>
                  <FormItem name="连报条件" required={true}>
                    <Group onChange={this.updateContinueType} value={continuityType}>
                      {continuityDict.map((v, k) => {
                        return (
                          <Radio value={v.val} key={v.val}>
                            {v.text}
                          </Radio>
                        );
                      })}
                    </Group>
                    {continuityType !== NoContinue && (
                      <Checkbox checked={sameSubjectCheck} onChange={this.updateSameSubjectCheck}>
                        相同科目连报
                      </Checkbox>
                    )}
                  </FormItem>
                </Col>
              )}
              <Col span={24}>
                <FormItem name="满减条件" required={true} style={{ background: '#f7f7f7', paddingBottom: 10 }}>
                  {this.props.type === FullReducePromotion && (
                    <Group onChange={this.updateRuleUnit} value={ruleUnit}>
                      {this.radioCfg.map((cfg) =>
                        cfg.disabled && cfg.disabled.includes(promotionType) ? null : (
                          <Radio value={cfg.value} key={cfg.value}>
                            {cfg.text}
                          </Radio>
                        ),
                      )}
                    </Group>
                  )}
                  {this.renderDiscountCondition()}
                </FormItem>
              </Col>
              <Col span={24} style={{ display: 'none' }}>
                <FormItem name="客户条件" required={true}>
                  <Group
                    onChange={(e) => this.setState({ customerCondition: e.target.value })}
                    value={customerCondition}
                  >
                    {cusDict.map((v, k) => {
                      return (
                        <Radio value={v.val} key={v.val}>
                          {v.text}
                        </Radio>
                      );
                    })}
                  </Group>
                </FormItem>
              </Col>
              <Col span={24} style={{ display: 'none' }}>
                <FormItem name="合同条件" required={true}>
                  <Group
                    onChange={(e) => this.setState({ contractCondition: e.target.value })}
                    value={contractCondition}
                  >
                    {conDict.map((v, k) => {
                      return (
                        <Radio value={v.val} key={v.val}>
                          {v.text}
                        </Radio>
                      );
                    })}
                  </Group>
                </FormItem>
              </Col>
              <Col span={24}>
                <FormItem name="活动声明" style={{ width: '50%' }}>
                  <TextArea
                    rows={3}
                    value={explain}
                    maxLength={100}
                    onChange={(e) => this.setState({ explain: e.target.value })}
                  />
                </FormItem>
              </Col>
              <Col span={24}>
                <FormItem name="活动备注" style={{ width: '50%' }}>
                  <TextArea
                    rows={3}
                    value={remark}
                    maxLength={100}
                    onChange={(e) => this.setState({ remark: e.target.value })}
                  />
                </FormItem>
              </Col>
            </Row>
          </Block>
        )}
        {currentIndex === 1 && (
          <div style={{ position: 'relative', margin: '10px 50px' }} className={styles.second}>
            <Search
              conditions={conditions}
              selectedDatas={this.searchKeys}
              onSearch={this.onSearch}
              result={totalRecords}
              isShowResult={false}
              onReset={this.handleResetSearch}
              onInputChange={this.onInputChange}
            />
            <ChangeTabs
              ref={(e) => (this.$changeTabs = e)}
              count={totalRecords}
              selectCount={selectedList.length}
              DontShowList={this.DontShowList}
              handleKey={this.onSwitchTab}
              paginations={this.renderTabTopPagination()}
            >
              <div>
                <CardList
                  expandedRowRender={(record) => this.expandedRowRender(record)}
                  dataSource={list}
                  rowSelection={{
                    selectedRowKeys: selectedListKeys,
                    onChange: this.chooseItem,
                  }}
                  pagination={{
                    current: currentPage,
                    total: totalRecords,
                    pageSize,
                    pageSizeOptions: ['20', '50', '100'],
                  }}
                  className={styles.table}
                  expandedRowKeys={expandedRowKeys}
                  detailIndent={true}
                  columns={columns}
                  onChange={({ current, pageSize }) => this.getListByChangePage({ currentPage: current, pageSize })}
                />
              </div>
              <div>
                <CardList
                  expandedRowRender={(record) => this.expandedRowRender(record)}
                  dataSource={selectedList.slice((selectPage - 1) * selectPageSize, selectPage * selectPageSize)}
                  rowSelection={{
                    selectedRowKeys: selectedRowKeysTwo,
                    onChange: this.selectChangeTwo,
                  }}
                  pagination={{
                    current: selectPage,
                    total: selectedList.length,
                    pageSize: selectPageSize,
                    pageSizeOptions: ['20', '50', '100'],
                  }}
                  className={styles.table}
                  expandedRowKeys={expandedRowKeys}
                  detailIndent={true}
                  columns={columns}
                  onChange={({ current, pageSize }) => this.setState({ selectPage: current, selectPageSize: pageSize })}
                />
              </div>
            </ChangeTabs>
          </div>
        )}
        {currentIndex === 2 && (
          <div className={styles.third} style={{ margin: '10px 50px' }}>
            <Block title="活动信息">
              <InfoBoard data={data} />
            </Block>
            <Block title="参与产品">
              {this.renderSelectedProductsNumber()}
              <CardList
                expandedRowRender={(record) => this.expandedRowRender()}
                dataSource={selectedList.slice((viewPage - 1) * viewPageSize, viewPage * viewPageSize)}
                columns={viewColumns}
                loading={false}
                pagination={{
                  current: viewPage,
                  total: selectedList.length,
                  pageSize: viewPageSize,
                }}
                expandedRowKeys={[]}
                className={styles.viewTable}
                onChange={({ current, pageSize }) => this.setState({ viewPage: current, viewPageSize: pageSize })}
              />
            </Block>
          </div>
        )}
      </Page>
    );
  }
}

let mapStateToProps = (state) => {
  let data = state.main.pageParams;
  let pathName = state.routing.locationBeforeTransitions.pathname;
  if (data[pathName]) {
    return {
      type: data[pathName].type,
      handle: data[pathName].handle,
      id: data[pathName].id,
      branchDict: data[pathName].branchDict,
      typeDict: data[pathName].typeDict,
      onSaveSuccess: data[pathName].onSaveSuccess,
      actionType: data[pathName].actionType,
    };
  }
  return {};
};
export default connect(mapStateToProps)(Add);
