/**
 * Created by luoming on 2018/3/28
 */
import React from 'react';
import { connect } from 'dva';
import styles from './index.less';
import moment from 'moment';
import Page from 'COMPONENT/Page';
import List from 'COMPONENT/List';
import Input from 'COMPONENT/Input';
import DatePicker from 'COMPONENT/DatePicker';
import InputNumber from 'COMPONENT/InputNumber';
import IconButton from 'COMPONENT/IconButton';
import showMessage from '../../../../components/utils/globalTips/showMessage';

const [COL_WIDTH_L, COL_WIDTH_M, , COL_WIDTH_S] = [370, 140, 100];

class Copy extends React.Component {
  constructor(props) {
    super(props);
    let { list } = this.props;
    let copyList = JSON.parse(JSON.stringify(list));
    this.state = {
      list: copyList,
    };
  }

  componentDidMount() {}

  /**
   * 点击保存按钮
   */
  handleOk = () => {};
  updateConditions = (index, value, type, key) => {};
  updateRow = (index, changed) => {
    this.setState({
      list: this.state.list.map((row, i) => {
        if (i === index) {
          return { ...row, ...changed };
        }
        return row;
      }),
    });
  };
  listBtnClick = (type, record, index) => {
    let { list } = this.state;
    if (type === 'delete') {
      list.splice(index, 1);
    } else {
      list.splice(index + 1, 0, list[index]);
    }
    this.setState({ list });
  };
  handleCondition = (type, index) => {
    if (type === 'delete') {
    } else {
      showMessage('最多只能设置3档！');
      return;
    }
  };
  /**
   * 获取开始 禁用日期
   * @param startDate
   * @param index
   * @returns {boolean}
   */
  getDisabledStartDate = (startDate, index) => {
    let { list } = this.state;
    const endDate = list[index].endDate;
    if (!startDate || !endDate) {
      return false;
    }
    return moment(startDate).valueOf() > moment(endDate).valueOf();
  };
  /**
   * 获取禁用结束日期
   * @param endDate
   * @param index
   * @returns {boolean}
   */
  getDisabledEndDate = (endDate, index) => {
    let { list } = this.state;
    const startDate = list[index].startDate;
    if (!endDate || !startDate) {
      return false;
    }
    return moment(endDate).valueOf() <= moment(startDate).valueOf();
  };

  render() {
    let { loading, list } = this.state;
    let columns = [
      { title: '活动名称', dataIndex: 'name', width: COL_WIDTH_M },
      {
        title: '新增名称',
        dataIndex: 'newName',
        width: COL_WIDTH_M,
        render: (text, record, index) => {
          let { newName } = record;
          return <Input value={text} onChange={(e) => this.updateRow(index, { newName: e.target.value })} />;
        },
      },
      {
        title: '上架日期',
        dataIndex: 'startDate',
        width: COL_WIDTH_M,
        render: (text, record, index) => {
          let { startDate } = record;
          return (
            <DatePicker
              value={startDate}
              onChange={(val) => this.updateRow(index, { startDate: val })}
              disabledDate={(val) => this.getDisabledStartDate(val, index)}
            />
          );
        },
      },
      {
        title: '下架日期',
        dataIndex: 'endDate',
        width: COL_WIDTH_M,
        render: (text, record, index) => {
          let { endDate } = record;
          return (
            <DatePicker
              value={endDate}
              onChange={(val) => this.updateRow(index, { endDate: val })}
              disabledDate={(val) => this.getDisabledEndDate(val, index)}
            />
          );
        },
      },
      {
        title: '满足条件',
        dataIndex: 'conditions',
        width: COL_WIDTH_L,
        render: (text, record, index) => {
          let { conditions } = record;
          let array = [1, 1, 1];
          return (
            <div className={styles.conditions}>
              {array.map((v, k) => {
                return (
                  <div key={k}>
                    满&nbsp;&nbsp; <InputNumber value={100} />元 &nbsp;&nbsp;&nbsp;&nbsp; 减&nbsp;&nbsp;{' '}
                    <InputNumber value={100} onChange={(val) => this.updateConditions(index, val, 'conditions', k)} />
                    {k === 0 && (
                      <IconButton type={'plus_circle'} size={20} onClick={(e) => this.handleCondition('add', index)} />
                    )}
                    {k !== 0 && (
                      <IconButton
                        type={'minus_circle'}
                        size={20}
                        onClick={(e) => this.handleCondition('delete', index)}
                      />
                    )}
                  </div>
                );
              })}
            </div>
          );
        },
      },
      {
        title: '操作',
        key: 'action',
        width: COL_WIDTH_S,
        getActionConfig: (text, record, index) => {
          let { productId } = record;
          return [
            { text: '移除', onClick: () => this.listBtnClick('delete', record, index) },
            { text: '添加', onClick: () => this.listBtnClick('add', record, index) },
          ];
        },
      },
    ];
    const tips = <div className={styles.tips}>{`当前已选择${list.length}条数据`}</div>;
    return (
      <Page
        footer={true}
        fitWindowHeight={true}
        loading={loading}
        route={this.props.route}
        onOk={this.handleOk}
        headerToolBar={tips}
        className={styles.copy}
        ref={(c) => (this._page = c)}
      >
        <List dataSource={list} columns={columns} pagination={false} />
      </Page>
    );
  }
}

let mapStateToProps = (state) => {
  return {
    list: state.main.pageParams['/FullReducePromotion/Copy'].list,
  };
};
export default connect(mapStateToProps)(Copy);
