import React from 'react';
import { connect } from 'dva';

import {
  Page,
  Block,
  FormItem,
  Input,
  InputNumber,
  Select,
  Button,
  Radio,
  TextArea,
  IconButton,
  Search,
  PopTip,
  Modal,
} from 'COMPONENT';
import CardList from 'COMPONENT/List/CardList';
import { Row, Col, DatePicker, Pagination } from 'antd';

import moment from 'moment';
import NP from 'number-precision';

import styles from './index.less';
import ChangeTabs from '../../Components/Tabs';
import InfoBoard from '../../Components/InfoBoard';
import Steps from '../../Components/Steps/index';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import {
  getSelectOption,
  savePromotion,
  getProductList,
  getPromotionInfoById,
  previewSavePromotionInfo,
} from 'SERVICE/preferential';

import {
  mapTips,
  mapDataToDict,
  getTextByDictValue,
  getTextByDictMultipleValue,
  promotionAreaDict,
  ProductCharacterMap,
  promotionAreaDictForPersonalized,
} from '../../common';
import { loadCommonDictByGroup } from 'UTIL/dict';
import { clone, difference, differenceWith, intersectionWith, indexOf } from 'lodash';
import {
  FullReducePromotion,
  DiscountPromotion,
  MONEY,
  PRODUCT_NUMS,
  PROMOTION_FULL_SALE_NORMAL,
  PROMOTION_FULL_SALE_EVERY,
  PROMOTION_GIVE_HOURS,
  PROMOTION_DISCOUNT_CASH,
  PROMOTION_PERSONALIZED_HOURS,
  dateFormatCfg,
} from '../../../config';
import { getUseCondition, transformRuleVosToHours } from '../../../utils';

const Group = Radio.Group;
const [COL_WIDTH_XL, COL_WIDTH_L, COL_WIDTH_M] = [150, 600, 150];
const mapType = {
  PROMOTION_FULL_SALE_NORMAL: '减',
  PROMOTION_DISCOUNT_COURSE: '折后',
  PROMOTION_GIVE_HOURS: '赠',
};
const mapUnit = {
  PROMOTION_FULL_SALE_NORMAL: '元',
  PROMOTION_DISCOUNT_COURSE: '%',
  PROMOTION_GIVE_HOURS: '小时',
};

let mapDefaultPromotionType = {
  // 满减促销
  FullReducePromotion: PROMOTION_FULL_SALE_NORMAL,
  // 折扣促销
  DiscountPromotion: PROMOTION_DISCOUNT_CASH,
  // 满赠促销
  FullGivePromotion: PROMOTION_GIVE_HOURS,
  personalizedClassHourDiscount: PROMOTION_PERSONALIZED_HOURS,
};

const getSingleRuleName = ({ promotionRule, itemList }) => {
  const matchItemList = itemList?.filter((v) => !!v.promotion);
  return matchItemList?.length
    ? `满${promotionRule}小时${itemList
        ?.filter((v) => !!v.promotion)
        .map(({ promotion, promotionType }) => `${mapType[promotionType]}${promotion}${mapUnit[promotionType]}`)
        .join('+')}`
    : '';
};

const getRuleName = (rules) => {
  return rules.map((rule) => getSingleRuleName(rule)).join('，');
};

/**
 * Page组件底部toolbar
 * */
const renderFooter = (props) => {
  const { handlePrev, handleNext, currentIndex, tabKey, handleBtn, popTipVisible, onClose, content } = props;
  return (
    <div style={{ display: 'flex', justifyContent: 'flex-end' }}>
      {currentIndex === 1 && tabKey === 1 && (
        <Button
          onClick={() => {
            handleBtn({ tabKey });
          }}
          style={{ marginRight: 20 }}
        >
          确认移除
        </Button>
      )}
      <PopTip content={content} visible={popTipVisible} onClose={onClose}>
        <Button type="primary" onClick={handleNext}>
          {currentIndex < 2 ? '下一步' : '完成'}
        </Button>
      </PopTip>
      {currentIndex <= 2 && currentIndex > 0 && (
        <Button style={{ marginLeft: '20px' }} onClick={handlePrev}>
          上一步
        </Button>
      )}
    </div>
  );
};

class Add extends React.Component {
  tags = 0;
  searchKeys = {};
  hideTage = null;
  copyBranchVos = {};
  selectedStorage = [];

  constructor(props) {
    super(props);
    this.init = this.init.bind(this);
    this.renderDiscountCondition = this.renderDiscountCondition.bind(this);
    this.renderTabTopPagination = this.renderTabTopPagination.bind(this);
    this.renderSelectedProductsNumber = this.renderSelectedProductsNumber.bind(this);
    this.handleResetSearch = this.handleResetSearch.bind(this);
    // 是否为 满减或折扣优惠
    this.isFullReduceOrDiscountPromotion = [DiscountPromotion, FullReducePromotion].includes(this.props.type);
    this.state = {
      currentIndex: 0,
      // 优惠场景
      promotionArea: null,
      ruleVos: [{}],
      perFullReduce: {},
      isAuditModalVisible: false,
      isAuditModalLoading: false,
      promotionType: mapDefaultPromotionType[this.props.type],
      ruleUnit: 'MONEY',
      contractCondition: 'ALL',
      customerCondition: 'ALL',
      // 连报相关
      continuityType: 'NONE',
      continuitySubjectType: 0,
      sameSubjectCheck: false,
      //
      branchSecond: [],

      currentPage: 1,
      pageSize: 100,
      totalRecords: 0,
      list: [],
      selectedListKeys: [],
      selectedList: [],
      selectPage: 1,
      selectPageSize: 100,
      selectedRowKeysTwo: [],
      approvalUserId: null,

      expandedRowKeys: [],

      stepTwoTabKey: 0, //step 2 tab key
      averagePromotionAcount: 0, // 最大优惠分摊金额
      popTipVisible: false, // 第2步校验不通过提示框
      content: '', //提示框内容
    };
    // 满减优惠-满减条件 单选配置
    this.radioCfg = [
      { text: '金额', value: MONEY },
      // { text: '小时', value: COURSE_HOURS },
      { text: '产品数', value: PRODUCT_NUMS, disabled: [PROMOTION_FULL_SALE_EVERY] },
    ];
  }

  componentDidMount() {
    this.init();
  }

  async init() {
    let { id } = this.props;
    this.setState({ loading: true });
    let [{ data: cusData }, { data: conData }] = await Promise.all([
      getSelectOption({ category: 'PromotionCusCondition' }),
      getSelectOption({ category: 'PromotionConCondition' }),
      id ? this.loadDataById(id) : null,
    ]);
    this.setState({ loading: false });
    let cusDict = mapDataToDict(cusData);
    let conDict = mapDataToDict(conData);
    this.setState({
      cusDict,
      conDict,
    });
  }

  isPersonalizedClassHourDiscount = (props = this.props) => {
    return props.type === 'personalizedClassHourDiscount';
  };

  /**
   * 编辑时根据id获取信息
   * @param promotionId
   * @returns {Promise.<void>}
   */
  loadDataById = async (promotionId) => {
    let { data } = await getPromotionInfoById({ id: promotionId });
    let {
      promotionName,
      startDate,
      endDate,
      branchVos,
      promotionTypeId,
      ruleVos,
      usePlat,
      customerCondition,
      contractCondition,
      remark,
      promotionComment,
      productVos,
      id,
      status,
      auditStatus,
      hightAmount,
      continuityType,
      continuitySubjectType,
      useCase,
      auditUserId,
    } = data;
    this.selectedStorage = productVos.map((v, k) => {
      return { ...v.productVo, promotionId: v.promotionId, id: v.id, productId: v.productId };
    });

    let payload = {
      auditStatus,
      promotionName,
      approvalUserId: auditUserId,
      promotionArea: useCase || 0,
      startDate: moment(startDate, dateFormatCfg),
      endDate: moment(endDate, dateFormatCfg),
      branchVos: branchVos.map((v, k) => v.branchId),
      promotionType: promotionTypeId,
      ruleVos,
      usePlat,
      customerCondition,
      contractCondition,
      remark,
      explain: promotionComment,
      selectedList: productVos.map((v, k) => {
        return { ...v.productVo, promotionId: v.promotionId, id: v.id, productId: v.productId };
      }),
      id,
      status,
      perFullReduce: { ...ruleVos[0], maxAmount: hightAmount },
      ruleUnit: ruleVos[0].ruleUnit,
      continuityType: continuityType || 'NONE',
      continuitySubjectType: continuitySubjectType || 0,
      sameSubjectCheck: continuitySubjectType && continuitySubjectType === 1,
    };

    payload.ruleVos = transformRuleVosToHours(ruleVos, this.props.type);

    this.setState(payload);
  };

  getAuditStatusName = (auditStatus) => {
    return this.props.actionType === 'Copy'
      ? '审核中'
      : (
          [
            { val: '1', text: '审核通过' },
            { val: '2', text: '审核中' },
            { val: '3', text: '审核不通过' },
          ].find(({ val }) => val == auditStatus) || {}
        ).text || '审核中';
  };

  next = () => {
    let { currentIndex, ruleVos, selectedList, promotionType } = this.state;
    let { branchDict } = this.props;

    if (currentIndex === 0) {
      let {
        promotionName,
        startDate,
        endDate,
        branchVos = [],
        ruleVos,
        contractCondition,
        customerCondition,
        explain,
        remark,
        cusDict,
        conDict,
        promotionArea,
      } = this.state;

      if (!promotionName) {
        this.setState({ popTipVisible: true, content: '请输入活动名称！' });
        return;
      }
      if (!startDate) {
        this.setState({ popTipVisible: true, content: '请输入起始时间！' });
        return;
      }
      if (!endDate) {
        this.setState({ popTipVisible: true, content: '请输入终止时间！' });
        return;
      }
      if (!branchVos.length) {
        this.setState({ popTipVisible: true, content: '请选择所属分公司！' });
        return;
      }
      if (!promotionType) {
        this.setState({ popTipVisible: true, content: '请选择活动类型！' });
        return;
      }
      if (typeof promotionArea !== 'number') {
        this.setState({ popTipVisible: true, content: '请选择优惠场景！' });
        return;
      }

      let checkRule = ruleVos.filter((v) => {
        return (
          v.promotionRule == null ||
          !v?.itemList ||
          v?.itemList?.every((item) => item.promotion == null || item.promotion == 0)
        );
      });
      if (checkRule.length > 0) {
        this.setState({ popTipVisible: true, content: '请完善满减条件！' });
        return;
      }

      let array = ruleVos.map((v) => v.promotionRule);
      if (array.length !== [...new Set(array)].length) {
        this.setState({ popTipVisible: true, content: '满减条件不能重复，请改正！' });
        return;
      }

      if (!contractCondition) {
        this.setState({ popTipVisible: true, content: '请选择合同条件！' });
        return;
      }

      if (!customerCondition) {
        this.setState({ popTipVisible: true, content: '请选择客户条件！' });
        return;
      }

      let customerConditionName = getTextByDictValue(cusDict, customerCondition);
      let contractConditionName = getTextByDictValue(conDict, contractCondition);
      let branchName = getTextByDictMultipleValue(branchDict, branchVos);

      let condition = getRuleName(ruleVos);
      let data = [
        { name: '优惠活动名称', info: promotionName },
        { name: '使用平台', info: null },
        { name: '活动时间', info: `${startDate.format(dateFormatCfg)}至${endDate.format(dateFormatCfg)}` },
        { name: '客户条件', info: customerConditionName },
        { name: '所属分公司', info: branchName },
        { name: '合同条件', info: contractConditionName },
        { name: '状态', info: this.state.status === '1' ? '有效' : '无效' },
        { name: '种类', info: '个性化课时优惠' },
        { name: '审核状态', info: this.getAuditStatusName(this.state.auditStatus) },
        { name: '使用条件', info: condition },
        { name: '活动说明', info: explain },
        { name: '活动备注', info: remark },
      ];

      this.setState({ data });
      if (this.tags === 0 || JSON.stringify(this.copyBranchVos) !== JSON.stringify(branchVos)) {
        this.getSecondOption();
        this.copyBranchVos = branchVos;
        this.tags = 1;
      } else {
        const { conditions } = this.state;
        let selectProduct = this.productDict.filter((v) => {
          return v.val === 'ONE_ON_ONE_COURSE' || v.val === 'ONE_ON_MANY';
        });
        conditions[0]['dict'] = selectProduct;
        this.setState({ conditions });
        this.getProductList();
      }
    } else if (currentIndex === 1) {
      // 除折扣外的优惠走这个判断条件
      if ([PROMOTION_FULL_SALE_NORMAL, PROMOTION_FULL_SALE_EVERY, PROMOTION_GIVE_HOURS].includes(promotionType)) {
        let totalAmountArray = selectedList
          .map((value) => {
            return value.totalAmount;
          })
          .filter((value) => {
            return value !== null;
          });
        let min = Math.min(...totalAmountArray);
        // 满多少件减多少元之后，平均每件减的数额
        let promotionArray = ruleVos.map((value) => {
          return (value.promotion / value.promotionRule).toFixed(2) - 0;
        });

        let averagePromotionAcount = Math.max(...promotionArray).toFixed(2) - 0;
        if (min < averagePromotionAcount) {
          this.setState({
            popTipVisible: true,
            content: `每件产品的总价必须大于最大优惠分摊金额：${averagePromotionAcount}元`,
          });
          return;
        }
      }
    } else if (currentIndex === 2) {
      this.saveData();
    }
    if (currentIndex < 2) {
      this.setState({ currentIndex: currentIndex + 1 }, () => {
        if (currentIndex === 0 && this.props.id) {
          this.$changeTabs.onChange(1);
        }
      });
    }
  };

  /**
   * 保存优惠信息
   * @returns {Promise.<void>}
   */
  saveData = async () => {
    let {
      branchVos,
      contractCondition,
      customerCondition,
      startDate,
      endDate,
      explain,
      remark,
      promotionName,
      ruleVos,
      limitProductType,
      id,
      status,
      promotionArea,
    } = this.state;

    const { actionType } = this.props;
    let branchVosData = branchVos.map((v, k) => {
      return { branchId: v, id: 0, promotionId: 0 };
    });

    let ruleVosData = ruleVos.map((v) => {
      let rule = {
        unit: 'COURSE_HOURS',
        ruleUnit: 'COURSE_HOURS',
        promotionRule: v.promotionRule,
        id: actionType === 'Copy' ? null : v.id,
        useNum: actionType === 'Copy' ? 0 : v.useNum,
        promotionId: actionType === 'Copy' ? null : v.promotionId,
      };
      if (actionType === 'Copy') {
        rule.itemList = v.itemList.map((detail) => ({
          ...detail,
          id: null,
        }));
      } else {
        rule.itemList = v.itemList;
      }
      rule.ruleName = getSingleRuleName(rule);
      return rule;
    });

    let productVosData = this.buildProductVos();
    // 复制的时候去掉原id
    if (actionType === 'Copy') {
      productVosData = productVosData.map((prd) => ({
        ...prd,
        id: null,
      }));
    }

    let apiParams = {
      branchVos: branchVosData, //分公司信息
      // 优惠场景
      useCase: promotionArea,
      contractCondition, //合同条件
      customerCondition, //用户条件
      startDate: startDate.format(dateFormatCfg), //开始日期
      endDate: endDate.format(dateFormatCfg), //结束日期
      firstLevel: 0, //暂时为0
      hightAmount: null, //最高优惠金额
      id: actionType === 'Copy' ? 0 : id,
      limitProductType: limitProductType || 'ALL',
      productVos: productVosData, //对应优惠产品信息
      promotionCode: '', //优惠码 暂时传空
      promotionComment: explain, //活动说明，
      promotionKindId: 'PROMOTION', //优惠类型
      promotionLittleTypeId: 'PROMOTION_PERSONALIZED', //优惠小类
      promotionName: promotionName, //优惠名字
      promotionTypeId: 'PROMOTION_PERSONALIZED_HOURS', //小小类
      publishNum: 0, //发布数量
      receiveEndDate: null, //领取结束时间
      receiveStartDate: null, //领取开始时间
      receiveLimit: 'ALL', //领取限制
      remark, //备注
      ruleVos: ruleVosData, //规则信息
      status: Number(status) || 0,
      superPosition: 0,
      useNum: 0, //使用数量 新增为空
    };

    previewSavePromotionInfo(apiParams)
      .then(async (res) => {
        if (!res || res.resultCode !== 0 || !res.data) {
          return showMessage(res.resultMessage || '操作失败，请稍后重试', 'error');
        }

        apiParams.nodeId = res.data.nodeId;

        if (res.data?.approvalUsers?.length === 1) {
          apiParams.auditUserId = res.data?.approvalUsers[0].userId;
          await savePromotion(apiParams);
          showMessage('保存成功', 'success');
          this.props.onSaveSuccess();
        } else if (res.data?.approvalUsers?.length > 1) {
          let approvalUserId = null;
          this.apiParams = apiParams;
          const approvalUsers = (res.data.approvalUsers || []).map(({ userId, userName }) => {
            return {
              val: userId,
              text: userName,
            };
          });
          if (this.state.approvalUserId) {
            approvalUserId =
              approvalUsers?.find((item) => item.val == this.state.approvalUserId)?.val || approvalUsers[0].val;
          } else {
            approvalUserId = approvalUsers[0].val;
          }
          this.setState({
            approvalUsers,
            approvalUserId,
            isAuditModalVisible: true,
          });
        } else {
          showMessage('接收人列表为空，无法保存', 'error');
        }
      })
      .catch((e) => {
        showMessage(e.resultMessage || '操作失败，请稍后重试', 'error');
      });
  };

  /**
   * 设置优惠条件
   * @param type
   * @param value
   * @param index
   * @param item
   */
  setReduceCondition = (value, index, item) => {
    let { ruleVos } = this.state;
    ruleVos[index] = ruleVos[index] || {};

    if (item === 'promotionRule') {
      ruleVos[index][item] = value;
    } else {
      ruleVos[index]['itemList'] = ruleVos[index]['itemList'] || [
        { promotion: 0, promotionType: 'PROMOTION_FULL_SALE_NORMAL', sort_order: 0 },
        { promotion: 0, promotionType: 'PROMOTION_DISCOUNT_COURSE', sort_order: 1 },
        { promotion: 0, promotionType: 'PROMOTION_GIVE_HOURS', sort_order: 2 },
      ];
      const targetIndex = { PROMOTION_FULL_SALE_NORMAL: 0, PROMOTION_DISCOUNT_COURSE: 1, PROMOTION_GIVE_HOURS: 2 }[
        item
      ];
      ruleVos[index]['itemList'][targetIndex].promotion = value || 0;
    }

    this.setState({ ruleVos });
  };

  // 渲染满减条件
  renderDiscountCondition() {
    let { ruleVos } = this.state;
    const questionBtn = (
      <IconButton
        type="question_circle"
        style={{ marginLeft: 10, position: 'relative', top: '2px' }}
        title="最少设置1档，最多可设置10档，满足条件递增设置。用户满足哪档额度，则享受哪一档的减价优惠，一个订单只能享受一次最高档优惠。当设置组合优惠时，都按照原价计算优惠金额，进行叠加。"
      />
    );

    let len = ruleVos.length;

    return ruleVos.map((v, k) => {
      return (
        <div key={k} className={styles.reduceCondition}>
          <span className={styles.reduceConditionItem}>
            <span>满</span>
            <InputNumber
              value={v.promotionRule}
              min={0}
              precision={2}
              max={10000}
              onChange={(val) => this.setReduceCondition(val, k, 'promotionRule')}
            />
            <span>小时</span>
          </span>
          <span className={styles.reduceConditionItem}>
            <span>减</span>
            <InputNumber
              value={this.getPromotionValue(v?.itemList, 'PROMOTION_FULL_SALE_NORMAL')}
              min={0}
              precision={2}
              max={1000000}
              onChange={(val) => this.setReduceCondition(val, k, 'PROMOTION_FULL_SALE_NORMAL')}
            />
            <span>元</span>
          </span>
          <span>+</span>
          <span className={styles.reduceConditionItem}>
            <span>折后</span>
            <InputNumber
              value={this.getPromotionValue(v?.itemList, 'PROMOTION_DISCOUNT_COURSE')}
              min={0}
              precision={2}
              max={100}
              onChange={(val) => this.setReduceCondition(val, k, 'PROMOTION_DISCOUNT_COURSE')}
            />
            <span>%</span>
          </span>
          <span>+</span>
          <span className={styles.reduceConditionItem}>
            <span>赠</span>
            <InputNumber
              value={this.getPromotionValue(v?.itemList, 'PROMOTION_GIVE_HOURS')}
              min={0}
              precision={2}
              max={1000000}
              onChange={(val) => this.setReduceCondition(val, k, 'PROMOTION_GIVE_HOURS')}
            />
            <span>小时</span>
          </span>
          <span className={styles.reduceConditionItem}>
            {len === 1 && (
              <span>
                <IconButton
                  onClick={(e) => this.changeReduceCondition(k, 'plus')}
                  type="plus_circle"
                  size={20}
                  style={{ position: 'relative', top: 5 }}
                />
                {questionBtn}
              </span>
            )}
            {len !== 1 && k + 1 < len && (
              <span>
                <IconButton
                  onClick={(e) => this.changeReduceCondition(k, 'minus')}
                  type="minus_circle"
                  size={20}
                  style={{ position: 'relative', top: 5 }}
                />
                {k === 0 && questionBtn}
              </span>
            )}
            {len !== 1 && k + 1 === len && (
              <span>
                <IconButton
                  onClick={(e) => this.changeReduceCondition(k, 'plus')}
                  type="plus_circle"
                  size={20}
                  style={{ position: 'relative', top: 5, marginRight: 20 }}
                />
                <IconButton
                  onClick={(e) => this.changeReduceCondition(k, 'minus')}
                  type="minus_circle"
                  size={20}
                  style={{ position: 'relative', top: 5 }}
                />
                {k === 0 && questionBtn}
              </span>
            )}
          </span>
        </div>
      );
    });
  }

  changeReduceCondition = (k, type) => {
    let { ruleVos } = this.state;
    if (type === 'plus') {
      if (ruleVos.length === 10) {
        showMessage('最多可设置10档！');
        return;
      }
      ruleVos.push({});
    } else {
      ruleVos.splice(k, 1);
    }
    this.setState({ ruleVos });
  };

  getPromotionValue = (itemList, promotionType) => {
    const value = ((itemList || []).find((item) => item.promotionType === promotionType) || {}).promotion;
    return value == 0 ? null : value;
  };

  getDisabledStartDate = (startDate) => {
    let { endDate } = this.state;
    if (!startDate || !endDate) {
      return false;
    }
    return moment(startDate).valueOf() > moment(endDate).valueOf();
  };

  getDisabledEndDate = (endDate) => {
    const { startDate } = this.state;
    if (!endDate || !startDate) {
      return false;
    }
    return moment(endDate).valueOf() < moment(startDate).valueOf();
  };

  /**
   * 获取第二步筛选数据
   * @returns {Promise.<void>}
   */
  getSecondOption = async () => {
    this.setState({ loading: true });
    const { branchDict } = this.props;
    const categoryArr =
      'PRODUCT_TYPE, PRODUCT_CHARACTER, STUDENT_GRADE, SUBJECT, PRODUCT_GROUP, COURSE_SERIES, PRODUCT_VERSION';
    const [batchDict] = await Promise.all([loadCommonDictByGroup(categoryArr), this.getProductList()]);
    const { productDict, gradeDict, subjectDict, groupDict, seriesDict, yearDict } = this.formatDictResponse(batchDict);

    this.productDict = productDict;
    let selectProduct = productDict.filter((v, k) => {
      return v.val === 'ONE_ON_ONE_COURSE' || v.val === 'ONE_ON_MANY';
    });
    const lastProductDict = (selectProduct ? selectProduct : productDict).filter((r) => r.val !== 'OTHERS');
    const baseCondition = [
      { title: '产品类型', field: 'productType', dict: lastProductDict, isMultiSelect: true },
      { title: '分公司', field: 'branchSecond', dict: branchDict },
      { title: '年份', field: 'year', dict: yearDict },
      { title: '年级', field: 'grade', dict: gradeDict },
      { title: '科目', field: 'subject', dict: subjectDict },
    ];

    let moreConditions = [
      { title: '产品组', field: 'group', dict: groupDict },
      { title: '产品系列', field: 'series', dict: seriesDict },
    ];

    const conditions = [...baseCondition, ...moreConditions].map((item) => {
      if (item.isMultiSelect) {
        // 已经支持多选的
        return item;
      } else {
        return { ...item, isMultiSelect: this.isFullReduceOrDiscountPromotion };
      }
    });
    this.setState({ conditions });
  };

  /**
   * 格式化批量字典返回值
   *
   * @param dicts
   */
  formatDictResponse = (dicts = {}) => {
    return {
      productDict: dicts['PRODUCT_TYPE'],
      productCharacterDict: dicts['PRODUCT_CHARACTER'],
      gradeDict: dicts['STUDENT_GRADE'],
      subjectDict: dicts['SUBJECT'],
      quarterDict: dicts['PRODUCT_QUARTER'],
      phaseDict: dicts['SMALL_CLASS_PHASE'],
      classDict: dicts['CLASS_TYPE'],
      groupDict: dicts['PRODUCT_GROUP'],
      seriesDict: dicts['COURSE_SERIES'],
      yearDict: dicts['PRODUCT_VERSION'],
    };
  };

  getProductList = async (pageKeys = {}) => {
    await this.initSelectedKeysByLoadList();
    let { branchVos } = this.state;
    let types = this.searchKeys?.productType?.length
      ? this.searchKeys.productType.join(',')
      : 'ONE_ON_MANY,ONE_ON_ONE_COURSE';

    if (!types) {
      types = 'ONE_ON_MANY,ONE_ON_ONE_COURSE';
    }

    // 分公司
    let branchIds = [];
    if (this.searchKeys.branchSecond) {
      let tempVal = this.searchKeys.branchSecond;
      if (this.isFullReduceOrDiscountPromotion) {
        // 折扣，满减多选
        branchIds = tempVal.filter((r) => !!r);
      } else {
        branchIds = [tempVal];
      }
    }

    if (!branchIds || !branchIds.length) {
      branchIds = branchVos;
    }

    let apiParams = {
      branchIds: branchIds,
      name: this.searchKeys['inputValue'],
      productCharacters: this.searchKeys.productCharacter,
      yearIds: this.searchKeys.year,
      gradeIds: this.searchKeys.grade,
      subjects: this.searchKeys.subject,
      seasonIds: this.searchKeys.holiday,
      phaseIds: this.searchKeys.periods,
      classTypeIds: this.searchKeys.classType,
      productGroupIds: this.searchKeys.group,
      courseSeriesIds: this.searchKeys.series,
      promotionId: '',
      types,
      // 页数是从0开始
      pageNo: pageKeys.currentPage ? pageKeys.currentPage - 1 : 0,
      pageSize: pageKeys.pageSize || this.state.pageSize,
      showOtherPromotion: this.hideTage,
    };
    let { datas, rowCount } = await getProductList(apiParams);
    this.setState({ list: datas, totalRecords: rowCount, loading: false, selectedRowKeys: [] });
    this.initSelectedByLoadList({ list: datas });
  };

  /**
   * 点击筛选项加载产品数据
   * @param keys
   * @returns {Promise.<void>}
   */
  onSearch = async (keys) => {
    this.searchKeys = keys;
    await this.setState({ loading: true, currentPage: 1 });
    this.getProductList();
  };

  handleResetSearch() {
    this.searchKeys = {};
    this.setState({ loading: true, currentPage: 1 }, this.getProductList);
  }

  getListByChangePage = async (pageKeys) => {
    await this.setState({ loading: true, currentPage: pageKeys.currentPage, pageSize: pageKeys.pageSize });
    this.getProductList(pageKeys);
  };

  onInputChange = (value) => {
    this.searchKeys['inputValue'] = value;
  };

  // step2 表格操作_查看优惠
  expandedRowRender = (record) => {
    let { infoVos } = record;
    infoVos = infoVos.filter((v, k) => {
      return (
        v.promotionLittleTypeId === 'PROMOTION_FULL_SALE' ||
        v.promotionLittleTypeId === 'PROMOTION_DISCOUNT' ||
        v.promotionLittleTypeId === 'PROMOTION_GIVE'
      );
    });
    return (
      <div className={'CardList-expand-wrap'}>
        <div className={styles.expandContent}>
          {infoVos.map((v, k) => (
            <div key={k} className={styles.item}>
              <span>
                <span>{v.promotionTypeName}</span>
              </span>
              <span>
                <div>{v.promotionName}</div>
                <div>
                  <span>{getUseCondition(v)}</span>
                </div>
              </span>
            </div>
          ))}
        </div>
      </div>
    );
  };

  onExpand = (index, record) => {
    let { expandedRowKeys } = this.state;
    let idx = expandedRowKeys.findIndex((v, k) => {
      return v === index;
    });
    if (idx === -1) {
      expandedRowKeys.push(index);
    } else {
      expandedRowKeys.splice(idx, 1);
    }
    this.setState({ expandedRowKeys });
  };

  selectChangeTwo = (selectedRowKeysTwo) => {
    this.setState({ selectedRowKeysTwo });
  };

  DontShowList = (e) => {
    this.hideTage = e.target.checked ? 1 : null;
    this.setState({ loading: true });
    this.getProductList();
  };
  // 监听Tab key
  onSwitchTab = (params) => {
    this.setState((prevState) => ({ stepTwoTabKey: params }));
  };
  /**
   * 在线报读4期
   * */
  updateState = (newState) => {
    this.setState((prevState, props) => {
      return {
        ...newState,
      };
    });
  };
  /**
   * 选择产品
   * list:当前页面选中项[array]
   * selectedList:全部选中项[array]
   * this.selectStorage:修改/编辑时，从接口获取的已选中项
   * */
  chooseItem = (keys) => {
    if (this.checkSelectType(keys)) {
      // 增加选择
      const list = this.getSelectedItem(keys);
      let selectedList = this.buildSelectedListByAdd(this.selectedStorage, list);
      this.selectedStorage = clone(selectedList);
      this.setState({ selectedList });
    } else {
      // 删减选择
      let subItemArr = this.getSubItemArr(keys);
      let selectedList = this.buildSelectedListBySub(this.selectedStorage, subItemArr);
      this.selectedStorage = clone(selectedList);
      this.setState({ selectedList });
    }
    this.setSelectedKeys(keys);
  };
  // 判断chooseItem是增操作还是减操作
  checkSelectType = (params) => {
    const { selectedListKeys } = this.state;
    return params.length > selectedListKeys.length;
  };
  // 设置选中状态,更新selectedListKeys,返回已选中项的数据 - 用于chooseItem
  setSelectedKeys = (params) => {
    this.updateState({ selectedListKeys: params });
  };
  // 获取当前列表的选中项
  getSelectedItem = (params) => {
    const { list } = this.state;
    return params.map((item) => list[item]);
  };
  // 构建参数SelectedList增 - 用于chooseItem
  buildSelectedListByAdd = (origin, other) => {
    const otherDiff = differenceWith(other, origin, this.diffComparator);
    return origin.concat(otherDiff).map(this.auxBuildItem);
  };
  // 辅助构建item
  auxBuildItem = (item, index, arr) => {
    let id = item.productId ? item.id : 0;
    let productId = item.productId || item.id;
    return {
      ...item,
      productId,
      id,
    };
  };
  // 比较两数组各项是否为同一项 - 用于differenceWith, intersectionWith
  diffComparator = (origin, other) => {
    return origin.id === other.id || origin.id === other.productId;
  };
  diffComparatorBySub = (origin, other) => {
    return origin.productId === other.id || origin.productId === other.productId;
  };
  // 在切换页码时初始化已选择优惠相关数据 - 用于getProductList
  initSelectedKeysByLoadList = () => {
    this.updateState({ selectedListKeys: [] });
  };
  // (编辑/修改时)加载产品列表时,更新已选产品 - 用于getProductList
  initSelectedByLoadList = (params) => {
    const { list } = params;
    let same = intersectionWith(list, this.selectedStorage, this.diffComparator);
    let arr = [];
    same.forEach((item) => {
      let checkIndex = indexOf(list, item);
      if (checkIndex !== -1) {
        arr.push(checkIndex);
      }
    });
    this.setSelectedKeys(arr.length > 0 ? arr : []);
  };
  // 将已选数据处理处理成所需(productVos)格式 - 用于saveData
  buildProductVos = () => {
    const { selectedList } = this.state;
    let list = clone(selectedList);
    return list.map((item) => ({
      amount: item.amount || 0,
      id: item.id || 0,
      productId: item.productId,
      promotionId: item.promotionId || 0,
    }));
  };

  // 获取当前去除项
  getSubItemArr = (params) => {
    const { selectedListKeys, list } = this.state;
    let sub = difference(selectedListKeys, params);
    return sub.map((item) => this.auxBuildItem(list[item]));
  };

  // 构建参数SelectedList减 - 用于chooseItem
  buildSelectedListBySub = (origin, other) => {
    return differenceWith(origin, other, this.diffComparatorBySub);
  };

  // 移除选择按钮事件
  subChosenItem = () => {
    const { selectedList, selectedRowKeysTwo, list } = this.state;
    let subItemArr = clone(selectedRowKeysTwo.map((item) => selectedList[item]));
    let sList = this.buildSelectedListBySub(this.selectedStorage, subItemArr);
    this.selectedStorage = clone(sList);
    this.setState({ selectedList: sList });
    this.updateState({ selectedRowKeysTwo: [] });
    this.initSelectedByLoadList({ list });
  };

  /**
   * @desc 0406调整优化： 折扣促销及满减促销 DiscountPromotion|FullReducePromotion
   * @returns null|Array ReactElement[]
   */
  renderTabTopPagination() {
    const { totalRecords, currentPage, pageSize, selectedList, selectPage, selectPageSize } = this.state;
    return this.isFullReduceOrDiscountPromotion
      ? [
          <Pagination
            showSizeChanger
            showQuickJumper
            pageSizeOptions={['20', '50', '100']}
            total={totalRecords}
            current={currentPage}
            pageSize={pageSize}
            showTotal={(total) => `共${total}条记录`}
            onShowSizeChange={(current, pageSize) => this.getListByChangePage({ currentPage: current, pageSize })}
            onChange={(current, pageSize) => this.getListByChangePage({ currentPage: current, pageSize })}
          />,
          <Pagination
            showSizeChanger
            showQuickJumper
            pageSizeOptions={['20', '50', '100']}
            total={selectedList.length}
            current={selectPage}
            pageSize={selectPageSize}
            showTotal={(total) => `共${total}条记录`}
            onShowSizeChange={(current, pageSize) => this.setState({ selectPage: current, selectPageSize: pageSize })}
            onChange={(current, pageSize) => this.setState({ selectPage: current, selectPageSize: pageSize })}
          />,
        ]
      : [];
  }

  renderSelectedProductsNumber() {
    const { selectedList } = this.state;
    return this.isFullReduceOrDiscountPromotion ? (
      <p style={{ marginBottom: 10 }}>共{selectedList.length}条记录</p>
    ) : null;
  }

  handleSubmit = async () => {
    try {
      this.setState({ isAuditModalLoading: true });
      await savePromotion({
        ...this.apiParams,
        auditUserId: this.state.approvalUserId,
      });
      showMessage('保存成功', 'success');
      this.props.onSaveSuccess();
      this.setState({ isAuditModalVisible: false, isAuditModalLoading: false });
    } catch (e) {
      showMessage(e.resultMessage || '保存失败，请稍后重试');
      this.setState({ isAuditModalLoading: false });
    }
  };

  handleCloseModal = () => {
    this.setState({ isAuditModalVisible: false, isAuditModalLoading: false });
  };

  render() {
    let { branchDict, id } = this.props;
    let {
      loading,
      promotionName,
      startDate,
      endDate,
      branchVos,
      contractCondition,
      customerCondition,
      cusDict = [],
      conDict = [],
      explain,
      remark,
      currentIndex,
      stepTwoTabKey,
      popTipVisible,
      content,
      conditions = [],
      promotionArea,
      totalRecords,
      expandedRowKeys,
      currentPage,
      pageSize,
      list,
      selectedList,
      selectedListKeys,
      selectPage,
      selectPageSize,
      selectedRowKeysTwo,
      viewPage = 1,
      viewPageSize = 10,
      data,
    } = this.state;
    let columns = [
      {
        title: '活动信息',
        dataIndex: 'info',
        width: COL_WIDTH_XL,
        render: (text, record, index) => {
          let { name, organizationName, courseSeriesName, classTypeName, categoryName, category, productCharacter } =
            record;
          return (
            <div className={styles.activityInfo} style={{ paddingLeft: 0 }}>
              <div>{name}</div>
              <div>
                <span>{organizationName}</span>
                <span>{courseSeriesName}</span>
                <span>{classTypeName}</span>
                <span>{categoryName}</span>
                <span>{ProductCharacterMap[productCharacter]}</span>
              </div>
              <span className={styles.tips}>
                <span></span>
                <span>{mapTips[category]}</span>
              </span>
            </div>
          );
        },
      },
      { title: '单价', dataIndex: 'price', width: COL_WIDTH_M },
      { title: '小时', dataIndex: 'miniClassTotalhours', width: COL_WIDTH_M },
      { title: '总价', dataIndex: 'totalAmount', width: COL_WIDTH_M },
      {
        title: '查看',
        dataIndex: 'view',
        width: COL_WIDTH_M,
        render: (text, record, index) => {
          let { infoVos } = record;
          infoVos = infoVos.filter((v, k) => {
            return (
              v.promotionLittleTypeId === 'PROMOTION_FULL_SALE' ||
              v.promotionLittleTypeId === 'PROMOTION_DISCOUNT' ||
              v.promotionLittleTypeId === 'PROMOTION_GIVE'
            );
          });
          return (
            <span onClick={(e) => this.onExpand(index, record)} className={styles.showPromote}>
              {infoVos.length > 0 ? '查看优惠' : ''}
            </span>
          );
        },
      },
    ];
    // 根据活动类型渲染满减条件单选按钮

    //第三步
    let viewColumns = [
      {
        title: '活动信息',
        dataIndex: 'info',
        render: (text, record, index) => {
          let { name, organizationName, courseSeriesName, classTypeName, categoryName, category } = record;
          return (
            <div className={styles.activityInfo}>
              <div>{name}</div>
              <div>
                <span>{organizationName}</span>
                <span>{courseSeriesName}</span>
                <span>{classTypeName}</span>
                <span>{categoryName}</span>
              </div>
              <span>
                <span></span>
                <span>{mapTips[category]}</span>
              </span>
            </div>
          );
        },
      },
      { title: '单价', dataIndex: 'price', width: COL_WIDTH_M },
      { title: '小时', dataIndex: 'miniClassTotalhours', width: COL_WIDTH_M },
      {
        title: '总价',
        dataIndex: 'totalAmount',
        width: COL_WIDTH_M,
        render: (text, record, index) => {
          return <span className={styles.totalPrice}>{record.totalAmount}</span>;
        },
      },
    ];

    return (
      <Page
        loading={loading}
        footer={renderFooter({
          currentIndex,
          handlePrev: (e) => {
            this.setState({ currentIndex: currentIndex - 1 });
          },
          handleNext: this.next,
          tabKey: stepTwoTabKey,
          handleBtn: this.subChosenItem,
          popTipVisible,
          onClose: () => {
            this.setState({ popTipVisible: false });
          },
          content,
        })}
        route={this.props.route}
        className={styles.activity}
        ref={(c) => (this.pageRef = c)}
      >
        <Steps currentIndex={currentIndex}>
          <div>活动信息</div>
          <div>选择产品</div>
          <div>核对完成</div>
        </Steps>
        {currentIndex === 0 && (
          <Block title={'活动信息'}>
            <Row>
              <Col span={12}>
                <FormItem name="活动名称" required={true}>
                  <Input
                    placeholder="请输入"
                    value={promotionName}
                    maxLength={50}
                    onChange={(e) => this.setState({ promotionName: e.target.value })}
                  />
                </FormItem>
              </Col>
              <Col span={12}>
                <FormItem name="优惠场景" required={true}>
                  <Group
                    style={{ display: 'flex' }}
                    onChange={(e) => this.setState({ promotionArea: e.target.value })}
                    value={promotionArea}
                  >
                    {(this.isPersonalizedClassHourDiscount()
                      ? promotionAreaDictForPersonalized
                      : promotionAreaDict
                    ).map(({ val, text, warnMsg }) => (
                      <div key={val} style={{ marginRight: '8px' }}>
                        <Radio value={val}>{text}</Radio>
                        <div style={{ lineHeight: '20px', color: 'red' }}>{warnMsg}</div>
                      </div>
                    ))}
                  </Group>
                </FormItem>
              </Col>
              <Col span={12}>
                <FormItem name="起始时间" required={true}>
                  <DatePicker
                    style={{ width: '100%' }}
                    size="large"
                    showTime={{ defaultValue: moment('00:00:00', 'HH:mm:ss') }}
                    format="YYYY-MM-DD HH:mm:ss"
                    value={startDate}
                    onChange={(val) => this.setState({ startDate: val })}
                    disabledDate={(val) => this.getDisabledStartDate(val)}
                  />
                </FormItem>
              </Col>
              <Col span={12}>
                <FormItem name="终止时间" required={true}>
                  <DatePicker
                    style={{ width: '100%' }}
                    size="large"
                    showTime={{ defaultValue: moment('24:00:00', 'HH:mm:ss') }}
                    format="YYYY-MM-DD HH:mm:ss"
                    value={endDate}
                    onChange={(val) => this.setState({ endDate: val })}
                    disabledDate={(val) => this.getDisabledEndDate(val)}
                  />
                </FormItem>
              </Col>
              <Col span={12}>
                <FormItem name="所属分公司" required={true}>
                  <Select
                    dict={branchDict}
                    value={(branchVos || [])[0]}
                    onChange={(val) => this.setState({ branchVos: [val] })}
                    disabled={!!id}
                  />
                </FormItem>
              </Col>
              <Col span={24}>
                <FormItem name="满减条件" required={true} style={{ background: '#f7f7f7', paddingBottom: 10 }}>
                  {this.renderDiscountCondition()}
                </FormItem>
              </Col>
              <Col span={24} style={{ display: 'none' }}>
                <FormItem name="客户条件" required={true}>
                  <Group
                    onChange={(e) => this.setState({ customerCondition: e.target.value })}
                    value={customerCondition}
                  >
                    {cusDict.map((v, k) => {
                      return (
                        <Radio value={v.val} key={v.val}>
                          {v.text}
                        </Radio>
                      );
                    })}
                  </Group>
                </FormItem>
              </Col>
              <Col span={24} style={{ display: 'none' }}>
                <FormItem name="合同条件" required={true}>
                  <Group
                    onChange={(e) => this.setState({ contractCondition: e.target.value })}
                    value={contractCondition}
                  >
                    {conDict.map((v, k) => {
                      return (
                        <Radio value={v.val} key={v.val}>
                          {v.text}
                        </Radio>
                      );
                    })}
                  </Group>
                </FormItem>
              </Col>
              <Col span={24}>
                <FormItem name="活动声明" style={{ width: '50%' }}>
                  <TextArea
                    rows={3}
                    value={explain}
                    maxLength={100}
                    onChange={(e) => this.setState({ explain: e.target.value })}
                  />
                </FormItem>
              </Col>
              <Col span={24}>
                <FormItem name="活动备注" style={{ width: '50%' }}>
                  <TextArea
                    rows={3}
                    value={remark}
                    maxLength={100}
                    onChange={(e) => this.setState({ remark: e.target.value })}
                  />
                </FormItem>
              </Col>
            </Row>
          </Block>
        )}
        {currentIndex === 1 && (
          <div style={{ position: 'relative', margin: '10px 50px' }} className={styles.second}>
            <Search
              conditions={conditions}
              selectedDatas={this.searchKeys}
              onSearch={this.onSearch}
              result={totalRecords}
              isShowResult={false}
              onReset={this.handleResetSearch}
              onInputChange={this.onInputChange}
            />
            <ChangeTabs
              ref={(e) => (this.$changeTabs = e)}
              count={totalRecords}
              selectCount={selectedList.length}
              DontShowList={this.DontShowList}
              handleKey={this.onSwitchTab}
              paginations={this.renderTabTopPagination()}
            >
              <div>
                <CardList
                  expandedRowRender={(record) => this.expandedRowRender(record)}
                  dataSource={list}
                  rowSelection={{
                    selectedRowKeys: selectedListKeys,
                    onChange: this.chooseItem,
                  }}
                  pagination={{
                    current: currentPage,
                    total: totalRecords,
                    pageSize,
                    pageSizeOptions: ['20', '50', '100'],
                  }}
                  className={styles.table}
                  expandedRowKeys={expandedRowKeys}
                  detailIndent={true}
                  columns={columns}
                  onChange={({ current, pageSize }) => this.getListByChangePage({ currentPage: current, pageSize })}
                />
              </div>
              <div>
                <CardList
                  expandedRowRender={(record) => this.expandedRowRender(record)}
                  dataSource={selectedList.slice((selectPage - 1) * selectPageSize, selectPage * selectPageSize)}
                  rowSelection={{
                    selectedRowKeys: selectedRowKeysTwo,
                    onChange: this.selectChangeTwo,
                  }}
                  pagination={{
                    current: selectPage,
                    total: selectedList.length,
                    pageSize: selectPageSize,
                    pageSizeOptions: ['20', '50', '100'],
                  }}
                  className={styles.table}
                  expandedRowKeys={expandedRowKeys}
                  detailIndent={true}
                  columns={columns}
                  onChange={({ current, pageSize }) => this.setState({ selectPage: current, selectPageSize: pageSize })}
                />
              </div>
            </ChangeTabs>
          </div>
        )}
        {currentIndex === 2 && (
          <div className={styles.third} style={{ margin: '10px 50px' }}>
            <Block title="活动信息">
              <InfoBoard data={data} />
            </Block>
            <Block title="参与产品">
              {this.renderSelectedProductsNumber()}
              <CardList
                expandedRowRender={(record) => this.expandedRowRender()}
                dataSource={selectedList.slice((viewPage - 1) * viewPageSize, viewPage * viewPageSize)}
                columns={viewColumns}
                loading={false}
                pagination={{
                  current: viewPage,
                  total: selectedList.length,
                  pageSize: viewPageSize,
                }}
                expandedRowKeys={[]}
                className={styles.viewTable}
                onChange={({ current, pageSize }) => this.setState({ viewPage: current, viewPageSize: pageSize })}
              />
            </Block>
          </div>
        )}
        {this.state.isAuditModalVisible && (
          <Modal
            title="请选择下工序接收人"
            width={600}
            okBtnShow={true}
            cancelBtnShow={true}
            closable={true}
            onOk={this.handleSubmit}
            onClose={this.handleCloseModal}
            loading={this.state.isAuditModalLoading}
          >
            <div>
              <FormItem name="接收人">
                <Select
                  showSearch
                  dict={this.state.approvalUsers}
                  value={this.state.approvalUserId}
                  onChange={(val) => this.setState({ approvalUserId: val })}
                />
              </FormItem>
            </div>
          </Modal>
        )}
      </Page>
    );
  }
}

let mapStateToProps = (state) => {
  let data = state.main.pageParams;
  let pathName = state.routing.locationBeforeTransitions.pathname;
  if (data[pathName]) {
    return {
      type: data[pathName].type,
      handle: data[pathName].handle,
      id: data[pathName].id,
      branchDict: data[pathName].branchDict,
      typeDict: data[pathName].typeDict,
      onSaveSuccess: data[pathName].onSaveSuccess,
      actionType: data[pathName].actionType,
    };
  }
  return {};
};
export default connect(mapStateToProps)(Add);
