/**
 * Created by luoming on 2018/3/22
 */
import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import List from 'COMPONENT/List';
import FormItem from 'COMPONENT/FormItem';
import Button from 'COMPONENT/Button';
import CardList from 'COMPONENT/List/CardList';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import Block from 'COMPONENT/Block';
import { Row, Col, Input } from 'antd';
import moment from 'moment';
import styles from './index.less';
import { listWorkflowDynamic, getPromotionInfoById } from 'SERVICE/preferential';
import {
  getTextByDictMultipleValue,
  getTextByDictValue,
  mapTips,
  promotionAreaDictForPersonalized,
} from '../../common';
import { getDictTextByVal } from 'UTIL/dict';
import { continuityDict, isNeedContinuityDict } from '../../../config';
import { getUseCondition, transformRuleVosToHours } from '../../../utils';
import { clone } from 'lodash';

const [COL_WIDTH_L, COL_WIDTH_M] = [300, 100];

class View extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      list: [],
      currentPage: 1,
      pageSize: 10,
      totalRecords: 0,
      approvalList: [],
      // 连报相关
      continuityType: null,
      continuitySubjectType: null,
      continuityTypeText: '',
      continuitySubjectTypeText: '',
    };
    this.columns = [
      {
        title: '活动信息',
        dataIndex: 'info',
        key: 'info',
        width: COL_WIDTH_L,
        render: (text, record, index) => (
          <div className={styles.activityInfo}>
            <div>{record.promotionText}</div>
            <div>
              <span>{record.organizationName}</span>
              <span>{record.courseSeriesName}</span>
              <span>{record.classTypeName}</span>
              <span>{record.categoryName}</span>
            </div>
            <span>
              <span> </span>
              <span>{mapTips[record.category]}</span>
            </span>
          </div>
        ),
      },
      { title: '单价', dataIndex: 'price', key: 'price', width: COL_WIDTH_M },
      { title: '小时', dataIndex: 'miniClassTotalhours', key: 'miniClassTotalhours', width: COL_WIDTH_M },
      {
        title: '总价',
        dataIndex: 'totalAmount',
        key: 'totalAmount',
        width: COL_WIDTH_M,
        render: (text, record, index) => <span className={styles.totalPrice}>{record.totalAmount}</span>,
      },
    ];
    this.auditListColumns = [
      { title: '处理人', dataIndex: 'userName', key: 'userName', width: 150 },
      {
        title: 'OA流程编号',
        dataIndex: 'oaInstanceId',
        key: 'oaInstanceId',
        width: 150,
        render: (text) => text || '-',
      },
      { title: '审批节点', dataIndex: 'title', key: 'title', width: 130 },
      { title: '操作类型', dataIndex: 'operateStatusName', key: 'operateStatusName', width: 140 },
      {
        title: '操作时间',
        dataIndex: 'operateTime',
        key: 'operateTime',
        width: 140,
        render: (time) => (time ? moment(time).format('YYYY-MM-DD HH:mm:ss') : '-'),
      },
      { title: '批注', dataIndex: 'notes', key: 'notes', width: 250 },
    ];
  }

  componentDidMount() {
    this.loadData();
  }

  loadData = async () => {
    let apiParams = { id: this.props.id };
    let { branchDict } = this.props;
    this.setState({ loading: true });
    let { data } = await getPromotionInfoById(apiParams);
    let branchIds = data.branchVos.map((v) => v.branchId);

    data.ruleVos = transformRuleVosToHours(data.ruleVos, this.props.type);

    let conditions = getUseCondition(data);

    const continuityType = data.continuityType;
    const continuitySubjectType = data.continuitySubjectType;

    let list = data.productVos.map((item) => {
      const infoVos = item.productVo.infoVos.map((info) => info.promotionName);
      return {
        ...item.productVo,
        promotionText: infoVos.join('；'),
        promotionList: infoVos,
      };
    });

    this.setState({
      loading: false,
      ruleVos: data.ruleVos,
      auditStatus: data.auditStatus,
      promotionName: data.promotionName,
      usePlatName: data.usePlatName,
      startDate: moment(data.startDate).format('YYYY-MM-DD'),
      endDate: moment(data.endDate).format('YYYY-MM-DD'),
      customerConditionName: data.customerConditionName,
      branch: getTextByDictMultipleValue(branchDict, branchIds),
      contractConditionName: data.contractConditionName,
      status: data.status,
      promotionTypeName: data.promotionTypeName,
      conditions,
      promotionComment: data.promotionComment,
      remark: data.remark,
      list,
      totalRecords: list.length,
      copyList: clone(list),
      continuityType,
      continuitySubjectType,
      continuityTypeText: continuityType ? getDictTextByVal(continuityDict, continuityType) : '不连报',
      continuitySubjectTypeText:
        !continuityType || continuityType === 'NONE'
          ? ''
          : `${getDictTextByVal(isNeedContinuityDict, continuitySubjectType)}相同科目连报`,
      promotionArea: getTextByDictValue(promotionAreaDictForPersonalized, data.useCase),
    });

    // 获取审核记录
    listWorkflowDynamic({
      type: 11,
      flowId: this.props.id,
    })
      .then((res) => {
        if (!res || res.resultCode !== 0) {
          this.setState({ approvalList: [] });
          showMessage(res.resultMessage || '获取审核记录失败');
          return;
        }
        this.setState({ approvalList: res.data || [] });
      })
      .catch((e) => {
        this.setState({ approvalList: [] });
        showMessage(e?.resultMessage || '获取审核记录失败');
      });
  };

  search = () => {
    const { value, copyList } = this.state;
    let searchList;
    if (value) {
      searchList = copyList.filter((copyItem) => !!~copyItem?.categoryName?.indexOf(value));
    } else {
      searchList = clone(copyList);
    }
    this.setState({ list: searchList });
  };

  render() {
    let {
      value,
      loading,
      ruleVos,
      currentPage,
      totalRecords,
      pageSize,
      promotionName,
      startDate,
      endDate,
      branch,
      status,
      auditStatus,
      promotionTypeName,
      promotionComment,
      remark,
      list,
      promotionArea,
    } = this.state;
    return (
      <Page
        loading={loading}
        footer={false}
        route={this.props.route}
        ref={(c) => (this.pageRef = c)}
        className={styles.wrapContent}
      >
        <Block title="活动信息">
          <div className={styles.wrap}>
            <Row>
              <Col span={12}>
                <FormItem name="优惠活动名称" size="small">
                  {promotionName}
                </FormItem>
              </Col>
              <Col span={12}>
                <FormItem name="活动时间" size="small">
                  {startDate + '至' + endDate}
                </FormItem>
              </Col>
              <Col span={12}>
                <FormItem name="所属分公司" size="small">
                  {branch}
                </FormItem>
              </Col>
              <Col span={12}>
                <FormItem name="状态" size="small">
                  {status === '0' ? '有效' : '无效'}
                </FormItem>
              </Col>
              <Col span={12}>
                <FormItem name="种类" size="small">
                  {promotionTypeName}
                </FormItem>
              </Col>
              <Col span={12}>
                <FormItem name="审核状态" size="small">
                  {
                    (
                      [
                        { val: '1', text: '审核通过' },
                        { val: '2', text: '审核中' },
                        { val: '3', text: '审核不通过' },
                      ].find(({ val }) => val == auditStatus) || {}
                    ).text
                  }
                </FormItem>
              </Col>
              <Col span={12}>
                <FormItem name="使用条件" size="small">
                  {ruleVos
                    ?.filter((item) => item.ruleName)
                    ?.map((item) => item.ruleName)
                    ?.join('，')}
                </FormItem>
              </Col>
              <Col span={12}>
                <FormItem name="活动说明" size="small">
                  {promotionComment}
                </FormItem>
              </Col>
              <Col span={12}>
                <FormItem name="活动备注" size="small">
                  {remark}
                </FormItem>
              </Col>
              <Col span={12}>
                <FormItem name="优惠场景" size="small">
                  {promotionArea}
                </FormItem>
              </Col>
            </Row>
          </div>
        </Block>
        <Block title="参与产品">
          <div className={styles.wrap}>
            <div className={styles.searchBar}>
              <Input
                type="text"
                value={value}
                allowClear={true}
                placeholder="请输入产品名称"
                onChange={(e) => {
                  this.setState({ value: e.target.value });
                }}
              />
              <Button type="primary" className={styles.searchBtn} onClick={this.search}>
                搜索
              </Button>
            </div>
            <div>
              <CardList
                expandedRowRender={(record) => this.expandedRowRender()}
                dataSource={list.slice((currentPage - 1) * pageSize, currentPage * pageSize)}
                columns={this.columns}
                loading={false}
                pagination={{
                  current: currentPage,
                  total: totalRecords,
                  pageSize,
                }}
                expandedRowKeys={[]}
                className={styles.table}
                onChange={({ current, pageSize }) => this.setState({ currentPage: current, pageSize })}
              />
            </div>
          </div>
        </Block>
        <Block title="审核记录">
          <List dataSource={this.state.approvalList} columns={this.auditListColumns} border={false} />
        </Block>
      </Page>
    );
  }
}

let mapStateToProps = (state) => {
  let data = state.main.pageParams;
  let pathName = state.routing.locationBeforeTransitions.pathname;
  if (data[pathName]) {
    return {
      type: data[pathName].type,
      id: data[pathName].id,
      branchDict: data[pathName].branchDict,
    };
  } else {
    return {};
  }
};
export default connect(mapStateToProps)(View);
