import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import style from './index.less';
import { Row, Col, DatePicker } from 'antd';
import FormItem from 'COMPONENT/FormItem';
import Button from 'COMPONENT/Button';
import MultiSelect from 'COMPONENT/MultiSelect';
import { loadBranchDict } from 'UTIL/dict';
import PreferentialModal from './PreferentialModal';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import moment from 'moment';
import { formatIndexNumber, renderContinuityType, renderActivityDate } from './util';
import { getUseCondition } from '../../../utils';
import { MAX_PREFERENTIAL, dateFormatter, timeFormatter } from '../config';
import { savePromotionPackage, getPromotionPackageById } from 'SERVICE/preferential';

const { RangePicker } = DatePicker;
const labelWidth = 80;

class Add extends React.Component {
  static defaultProps = {
    pageType: 'add', // 'add'|'edit'|'detail'
  };

  constructor(props) {
    super(props);

    this.state = {
      pageKey: new Date().getTime(), // 用于刷新

      detail: null, // 叠加方案详情，修改和查看详情有效

      branchIdArr: null, // 选中分公司
      startTime: null, // 有效起始时间
      endTime: null, // 有效截止时间
      preferentialList: [], // 已选优惠方案
      tempTime: [], // 临时时间时间

      showAddModal: false, // 添加优惠方案弹窗
      branchDict: null, // 分公司字典 从父组件传入或从后端获取

      pageEditable: true, // 是否允许编辑 仅当 props.pageType === 'detail' 时不可编辑
    };

    this.initState = {
      detail: null,
      branchIdArr: null,
      startTime: null,
      endTime: null,
      preferentialList: [],
      tempTime: [],
    };

    this.initDict = this.initDict.bind(this);
    this.initSelectedBranch = this.initSelectedBranch.bind(this);
    this.initPage = this.initPage.bind(this);
    this.fetchPromotionDetail = this.fetchPromotionDetail.bind(this);
    this.formatPromotionDetail = this.formatPromotionDetail.bind(this);
    this.onTimePickerChange = this.onTimePickerChange.bind(this);
    this.renderPreferentialList = this.renderPreferentialList.bind(this);
    this.onShowPreferentialModal = this.onShowPreferentialModal.bind(this);
    this.onClosePreferentialModal = this.onClosePreferentialModal.bind(this);
    this.onSavePreferential = this.onSavePreferential.bind(this);
    this.doSavePromotionPackage = this.doSavePromotionPackage.bind(this);
    this.formatApiParam = this.formatApiParam.bind(this);
  }

  componentDidMount() {
    this.initDict();
    this.initPage();
  }

  componentWillUnmount() {
    this.setState(this.initState);
  }

  /**
   * 初始化数据字典
   */
  initDict() {
    const { branchDict } = this.props;

    if (!branchDict) {
      Promise.all([loadBranchDict()]).then(([result]) => {
        this.setState({ branchDict: result }, () => {
          this.initSelectedBranch();
        });
      });
    } else {
      this.setState({ branchDict: branchDict }, () => {
        this.initSelectedBranch();
      });
    }
  }

  /**
   * 新增优惠方案时初始化选中的分公司
   */
  initSelectedBranch() {
    if (this.props.pageType !== 'add') {
      return;
    }

    const { user } = this.props;
    const { isBlCampus, isBlBrench, isBlGrounp, brenchId } = user;
    const isCampus = isBlCampus === 'true' || isBlCampus === true;
    const isBranch = !isCampus && (isBlBrench === 'true' || isBlBrench === true);
    let branchIdArr = [];

    if ((isCampus || isBranch) && brenchId) {
      branchIdArr.push(brenchId);
    }

    this.setState({
      branchIdArr: branchIdArr,
    });
  }

  /**
   * 初始化页面，判断页面是新增还是修改，修改则需要初始化选中条件等
   */
  initPage() {
    const { pageType, record } = this.props;

    if (pageType === 'edit' || pageType === 'detail') {
      this.fetchPromotionDetail(record.id);
      this.setState({ pageEditable: pageType !== 'detail' });
    }
  }

  /**
   * 根据优惠档案ID查询方案详情
   */
  fetchPromotionDetail(id) {
    if (!id) {
      return false;
    }

    getPromotionPackageById({ id: id })
      .then((result) => {
        if (result && Number(result.resultCode) === 0) {
          this.formatPromotionDetail(result.data);
        }
      })
      .catch((e) => {});
  }

  /**
   * 格式化叠加优惠详情
   * @param data
   */
  formatPromotionDetail(data) {
    if (!data) {
      return;
    }

    const { branchId, branchList, beginTime, endTime, packageDetailList } = data;

    // 分公司
    let branchIdArr = [];
    if (!branchId && branchList) {
      branchList.forEach((branch) => {
        branchIdArr.push(branch.branchId);
      });
    }

    // 时间
    let startDate = beginTime ? moment(new Date(beginTime)) : null;
    let endDate = endTime ? moment(new Date(endTime)) : null;

    // 优惠方案需转换getPromotionList所返回数据格式
    let preferentialList = [];
    if (packageDetailList && packageDetailList.length > 0) {
      packageDetailList.forEach((item) => {
        preferentialList.push(item.promotionInfoVo);
      });
    }

    this.setState({
      pageKey: new Date().getTime(),
      detail: data,
      branchIdArr: branchId || branchIdArr || [],
      startTime: startDate,
      endTime: endDate,
      preferentialList: preferentialList,
    });
  }

  /**
   * 选择时间
   * @param val
   */
  onTimePickerChange(val) {
    this.setState({
      tempTime: val,
      startTime: val[0],
      endTime: val[1],
    });
  }

  /**
   * 渲染优惠方案
   * @param list
   * @param editable 是否允许修改
   */
  renderPreferentialList(list = [], editable = true) {
    if (!list || list.length < 1) {
      return '';
    }

    return (
      <section className={style.cardWrap}>
        <ul className={style.ul}>
          {list.map((item, index) => {
            return (
              <li key={index}>
                <div className={style.li__left}>
                  <span className={style.li__left__icon}>方案{formatIndexNumber(index)}</span>
                </div>

                <div className={style.li__right}>
                  <h5 className={style.li__right__title}>{item.promotionName || ''}</h5>
                  <p className={style.li__right__desc}>
                    <span>使用条件：</span>
                    <span>{getUseCondition(item)}</span>
                  </p>
                  <p className={style.li__right__desc}>
                    <span>优惠种类：</span>
                    <span>{item.promotionLittleTypeName}</span>
                  </p>
                  <p className={style.li__right__desc}>
                    <span>连报条件：</span>
                    <span>{renderContinuityType(item)}</span>
                  </p>
                  <p className={style.li__right__desc}>
                    <span>活动时间：</span>
                    <span>{renderActivityDate(item)}</span>
                  </p>
                </div>
              </li>
            );
          })}
        </ul>
      </section>
    );
  }

  /**
   * 打开添加弹窗
   */
  onShowPreferentialModal() {
    const { branchIdArr } = this.state;

    if (!branchIdArr || branchIdArr.length < 1) {
      showMessage('请先选择分公司', 'error');
      return false;
    }

    this.setState({ showAddModal: true });
  }

  /**
   * 关闭弹窗同时保存选中方案
   * @param selectedRows
   */
  onClosePreferentialModal(selectedRows) {
    if (selectedRows && selectedRows.length > 0) {
      this.setState({ preferentialList: selectedRows });
    }

    this.setState({ showAddModal: false });
  }

  /**
   * 最终保存提交并返回上一页面
   */
  onSavePreferential() {
    const { branchIdArr, tempTime, preferentialList } = this.state;

    if (!branchIdArr) {
      showMessage('请选择分公司！', 'error');
      return false;
    } else if (!tempTime) {
      showMessage('请选择时间！', 'error');
      return false;
    } else if (!preferentialList || preferentialList.length < 1) {
      showMessage('请添加优惠方案！', 'error');
      return false;
    }

    this.doSavePromotionPackage();
  }

  /**
   * 保存
   */
  doSavePromotionPackage() {
    const { dispatch } = this.props;
    const param = this.formatApiParam();

    savePromotionPackage(param)
      .then((result) => {
        if (result && Number(result.resultCode) === 0) {
          showMessage('保存成功！', 'success');
          dispatch({
            type: 'main/openPage',
            payload: {
              path: '/SuperposePreferential',
              refresh: true,
            },
          });
        } else {
          showMessage(result.resultMessage || '保存失败，请重试！');
        }
      })
      .catch((e) => {
        showMessage(e.resultMessage || e.text || '保存失败，请重试！');
      });
  }

  /**
   * 组装请求参数
   * 这个接口协议设计有点傻屌
   */
  formatApiParam() {
    const { detail, branchIdArr, startTime, endTime, preferentialList } = this.state;
    const packageDetailList = (detail && detail.packageDetailList) || [];

    // 分公司列表ID
    let branchList = [];
    branchIdArr.forEach((id) => {
      branchList.push({ branchId: id });
    });

    let param = {
      branchId: branchIdArr.join(','),
      beginTime: startTime.format(dateFormatter),
      endTime: endTime.format(dateFormatter),
      packageDetailList: [],
      branchList: branchList,
    };

    // packageDetailList
    let tempPackageList = [];
    if (detail && preferentialList.length > 0) {
      preferentialList.forEach((item, index) => {
        const isContainObj = packageDetailList.find((prevList) => Number(prevList.promotion) === Number(item.id));
        if (isContainObj) {
          const { id, promotion } = isContainObj;
          tempPackageList.push({ id, promotion });
        } else {
          tempPackageList.push({
            promotion: item.id,
          });
        }
      });
    } else {
      preferentialList.forEach((item) => {
        tempPackageList.push({ promotion: item.id });
      });
    }
    tempPackageList = tempPackageList.filter((list) => list.promotion);
    param.packageDetailList = tempPackageList;

    // 修改需要传ID
    if (detail) {
      param.id = detail.id;
    }

    return param;
  }

  render() {
    const { pageKey, branchIdArr, branchDict, preferentialList, showAddModal } = this.state;
    const { startTime, endTime, pageEditable } = this.state;

    return (
      <Page route={this.props.route}>
        <section key={pageKey}>
          <Row>
            <Col span={12}>
              <FormItem name="分公司" labelWidth={labelWidth} required={true}>
                <MultiSelect
                  dict={branchDict}
                  value={branchIdArr}
                  onChange={(val) => this.setState({ branchIdArr: val })}
                  disabled={!pageEditable}
                />
              </FormItem>
            </Col>
          </Row>

          <Row>
            <Col span={12}>
              <FormItem name="生效时间" labelWidth={labelWidth} required={true}>
                <RangePicker
                  size="large"
                  style={{ width: '100%' }}
                  format={dateFormatter}
                  showTime={{
                    format: timeFormatter,
                    defaultValue: [moment('00:00:00', 'HH:mm:ss'), moment('00:00:00', 'HH:mm:ss')],
                  }}
                  placeholder={['开始时间', '结束时间']}
                  defaultValue={[
                    startTime ? moment(startTime, dateFormatter) : null,
                    endTime ? moment(endTime, dateFormatter) : null,
                  ]}
                  onChange={this.onTimePickerChange}
                  disabled={!pageEditable}
                />
              </FormItem>
            </Col>
          </Row>

          <Row>
            <Col span={24}>
              <FormItem name="优惠方案" labelWidth={labelWidth} required={true}>
                {preferentialList && preferentialList.length > 0
                  ? this.renderPreferentialList(preferentialList, pageEditable)
                  : pageEditable && (
                      <div>
                        <Button iconType="plus" onClick={this.onShowPreferentialModal}>
                          添加优惠方案
                        </Button>
                      </div>
                    )}

                <div className={style.mark}>注：目前只支持{MAX_PREFERENTIAL}个优惠方案叠加，都按照原价基础计算优惠</div>
              </FormItem>
            </Col>
          </Row>

          <Row>
            <Col span={24}>
              <FormItem name="" labelWidth={labelWidth}>
                {Array.isArray(preferentialList) && preferentialList.length > 0 && pageEditable && (
                  <Button onClick={this.onShowPreferentialModal} style={{ marginRight: 10 }}>
                    重选
                  </Button>
                )}
                {pageEditable && (
                  <Button type="primary" onClick={this.onSavePreferential}>
                    保存
                  </Button>
                )}
              </FormItem>
            </Col>
          </Row>
        </section>

        {showAddModal && (
          <PreferentialModal
            branchIdArr={branchIdArr}
            defaultSelectedRows={preferentialList}
            onClose={(selectedRows) => {
              this.onClosePreferentialModal(selectedRows);
            }}
            refreshList={() => this.loadList()}
          />
        )}
      </Page>
    );
  }
}

function mapStateToProps(state) {
  const user = state.account.user;
  const data = state.main.pageParams;
  const pathName = state.routing.locationBeforeTransitions.pathname;

  let branchDict = null; // 分公司字典
  let pageType = 'add'; // 页面类型 'add'|'edit'
  let record = null; // 修改地行数据

  if (data[pathName]) {
    branchDict = data[pathName].branchDict;
    pageType = data[pathName].pageType;
    record = data[pathName].record;
  }

  return {
    user: user || {},
    branchDict,
    pageType,
    record,
  };
}

export default connect(mapStateToProps)(Add);
