import React from 'react';
import { connect } from 'dva';
import styles from './index.less';
import Page from 'COMPONENT/Page';
import SearchBar from 'COMPONENT/SearchBar';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import Input from 'COMPONENT/Input';
import Button from 'COMPONENT/Button';
import List from 'COMPONENT/List';
import ShowMessage from 'COMPONENT/utils/globalTips/showMessage';
import ShowConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import { loadBranchDict } from 'UTIL/dict';
import { promotionStatusDict, PROMOTION_STATUS_VALID } from './config';
import { listPromotionPackage, setValid, setUnValid } from 'SERVICE/preferential';
import moment from 'moment';
import { dateFormatter } from './config';
import { renderMultiLine } from '../common';
const { COL_WIDTH_S, COL_WIDTH_XL } = List;

// 格式化第n中优惠的内筒
function formatPromotionItem(promotionArray, index) {
  if (!promotionArray[index]) return '暂无';
  return promotionArray[index].promotionName || '';
}
class SuperposePreferential extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false,

      searchName: null,
      searchBranchId: null,
      searchStatus: null,

      listLoading: false,
      list: [], // 列表数据
      totalRecords: 0, // 总记录数
      pageNo: 1, // 当前页码
      pageSize: 20, // 每页记录数

      branchDict: null, // 分公司字典
    };

    this.initSearchParam = {
      searchName: null,
      searchBranchId: null,
      searchStatus: null,
    };

    this.initPage = this.initPage.bind(this);
    this.loadList = this.loadList.bind(this);
    this.onSearch = this.onSearch.bind(this);
    this.onReset = this.onReset.bind(this);
    this.onRowBtnClick = this.onRowBtnClick.bind(this);
    this.onPageToAddOrEdit = this.onPageToAddOrEdit.bind(this);
    this.formatColumn = this.formatColumn.bind(this);
  }

  componentDidMount() {
    this.initPage();
  }

  initPage() {
    this.setState({ loading: true });

    Promise.all([loadBranchDict()]).then(([branchDict]) => {
      const branchId = this.formatBranchId(branchDict);

      this.setState({
        loading: false,
        searchBranchId: branchId,
        branchDict: branchDict,
      });

      this.onSearch();
    });
  }

  /**
   * 格式化全选中分公司
   * @param branchDict
   * @return {Array}
   */
  formatBranchId(branchDict = []) {
    let branchIdArr = [];

    if (branchDict) {
      branchIdArr = branchDict.map((item) => {
        return item.val;
      });
    }

    return branchIdArr;
  }

  /**
   * 获取优惠列表
   */
  loadList(changedParams = {}) {
    const { searchName, searchBranchId, searchStatus, pageNo, pageSize } = this.state;
    const tempParam = {
      searchName,
      searchBranchId,
      searchStatus,
      pageNo,
      pageSize,
      ...changedParams,
    };
    const param = this.formatApiParam(tempParam);

    this.setState({ listLoading: true });

    listPromotionPackage(param)
      .then((response) => {
        const result = this.formatResponse(response);
        this.setState({
          listLoading: false,
          list: result.data,
          totalRecords: result.rowCount,
          pageNo: param.pageNo,
          pageSize: param.pageSize,
        });
      })
      .catch((e) => {
        const result = this.formatResponse({});

        this.setState({
          listLoading: false,
          list: result.data,
          totalRecords: result.rowCount,
          pageNo: param.pageNo,
          pageSize: param.pageSize,
        });
      });
  }

  formatApiParam(param) {
    return {
      promotionName: param.searchName,
      branchId: (param.searchBranchId && param.searchBranchId.join(',')) || '',
      status: param.searchStatus,
      pageNo: param.pageNo,
      pageSize: param.pageSize,
    };
  }
  /**
   * 格式化接口返回参数
   * 将两个优惠券列表添加到data[]里面，拼成一维数组
   * [{promotionOne: '第一种优惠', promotionTwo: '第二种优惠', branchName: '', ...}]
   */
  formatResponse(response = {}) {
    let data = response.datas || [];
    const rowCount = response.rowCount || 0;

    data.forEach((item) => {
      const promotion = item.packageDetailList || [];

      // 优惠方案
      item.promotionOne = formatPromotionItem(promotion, 0);
      item.promotionTwo = formatPromotionItem(promotion, 1);
      item.promotionThree = formatPromotionItem(promotion, 2);

      // 优惠方案是否齐全，不齐全的话禁止修改查看详情
      item.isPromotionFull = Boolean(
        promotion[0] && promotion[0].promotionName && promotion[1] && promotion[1].promotionName,
      );
      // 分公司名称拼接
      let branchNameArr = [];
      if (!item.branchName && item.branchList) {
        item.branchList.forEach((branch) => {
          branchNameArr.push(branch.branchName || '');
        });
      }
      item.branchName = item.branchName || branchNameArr.join(',');
    });

    return {
      data: data,
      rowCount: rowCount,
    };
  }

  onSearch() {
    this.loadList({ pageNo: 1 });
  }

  onReset() {
    const { branchDict } = this.state;
    const branchId = this.formatBranchId(branchDict);

    let initParam = { ...this.initSearchParam };
    initParam.searchBranchId = branchId;

    this.setState(initParam);
  }

  /**
   * 详情/修改/有效无效
   */
  onRowBtnClick(type, record) {
    switch (type) {
      case 'detail':
        this.onPageToAddOrEdit('detail', record);
        break;
      case 'edit':
        this.onPageToAddOrEdit('edit', record);
        break;
      case 'trigger':
        this.triggerStatus(record);
        break;
      default:
        break;
    }
  }

  /**
   * 设置有效/无效
   * @param record
   */
  triggerStatus(record) {
    const { status, id } = record;

    let title = '有效';
    let promise = setValid;
    if (status === PROMOTION_STATUS_VALID) {
      title = '无效';
      promise = setUnValid;
    }

    ShowConfirm(`是否将该优惠设为${title}？`, async () => {
      await promise({ id });
      ShowMessage('操作成功！', 'success');
      this.loadList();
    });
  }

  /**
   * 添加或修改方案
   * @param pageType 'add'|'edit'|'detail'
   * @param record 当前编辑行数据
   */
  onPageToAddOrEdit(pageType = 'add', record) {
    const { dispatch } = this.props;
    const { branchDict } = this.state;
    const route = (pageType === 'add' && 'Add') || (pageType === 'edit' && 'Edit') || 'Detail';

    dispatch({
      type: 'main/openPage',
      payload: {
        path: `/SuperposePreferential/${route}`,
        params: {
          branchDict,
          pageType,
          record: record,
        },
        refresh: true,
      },
    });
  }

  /**
   * 表格列
   */
  formatColumn() {
    return [
      {
        title: '序号',
        dataIndex: 'number',
        key: 'number',
        width: COL_WIDTH_S,
        render: (text, record, index) => {
          return index + 1;
        },
      },
      { title: '分公司', dataIndex: 'branchName', key: 'branchName', width: 120, lines: 2, render: renderMultiLine },
      {
        title: '第一种优惠',
        dataIndex: 'promotionOne',
        key: 'promotionOne',
        width: 160,
        lines: 2,
        render: (text) => {
          return text ? text : <span style={{ color: 'red' }}>该优惠方案已删除</span>;
        },
      },
      {
        title: '第二种优惠',
        dataIndex: 'promotionTwo',
        key: 'promotionTwo',
        width: 160,
        lines: 2,
        render: (text) => {
          return text ? text : <span style={{ color: 'red' }}>该优惠方案已删除</span>;
        },
      },
      {
        title: '第三种优惠',
        dataIndex: 'promotionThree',
        key: 'promotionThree',
        width: 160,
        lines: 2,
        render: (text) => {
          return text ? text : <span style={{ color: 'red' }}>该优惠方案已删除</span>;
        },
      },
      {
        title: '生效时间',
        dataIndex: 'validTime',
        key: 'validTime',
        width: 210,
        render: (text, record, index) => {
          let str = '';
          const { beginTime, endTime } = record;

          try {
            str = `${moment(new Date(beginTime)).format(dateFormatter)} - ${moment(new Date(endTime)).format(
              dateFormatter,
            )}`;
          } catch (e) {}

          return str;
        },
      },
      { title: '状态', dataIndex: 'status', key: 'status', width: COL_WIDTH_S, dict: promotionStatusDict },
      {
        title: '操作',
        key: 'action',
        width: 160,
        fixed: 'right',
        getActionConfig: (text, record, index) => {
          let statusText = (record.status === PROMOTION_STATUS_VALID && '无效') || '有效';
          let statusDisabled = new Date().getTime() > new Date(record.endTime).getTime();
          // 存在优惠方案被删除 禁止修改查看
          if (!record.isPromotionFull) {
            return [];
          }
          return [
            { text: '详情', onClick: () => this.onRowBtnClick('detail', record, index) },
            { text: '修改', onClick: () => this.onRowBtnClick('edit', record, index) },
            { text: statusText, onClick: () => this.onRowBtnClick('trigger', record, index), disabled: statusDisabled },
          ];
        },
      },
    ];
  }

  render() {
    const {
      loading,
      searchName,
      searchBranchId,
      searchStatus,
      list,
      listLoading,
      pageNo,
      totalRecords,
      pageSize,
      branchDict,
    } = this.state;
    const columns = this.formatColumn();

    return (
      <Page
        className={styles.activity}
        fitWindowHeight={true}
        loading={loading}
        route={this.props.route}
        subHeader={
          <div className={styles.searchBar}>
            <Button className={styles.add} onClick={(e) => this.onPageToAddOrEdit('add')}>
              新增
            </Button>

            <SearchBar onSearch={this.onSearch} onReset={this.onReset}>
              <Input
                placeholder="方案名称"
                value={searchName}
                onChange={(e) => this.setState({ searchName: e.target.value })}
              />
              <MultiSelect
                placeholder="所属分公司"
                showSearch={true}
                dict={branchDict}
                value={searchBranchId}
                onChange={(val) => this.setState({ searchBranchId: val })}
              />
              <Select
                placeholder="是否有效"
                dict={promotionStatusDict}
                value={searchStatus}
                onChange={(val) => this.setState({ searchStatus: val })}
              />
            </SearchBar>
          </div>
        }
      >
        <List
          dataSource={list}
          columns={columns}
          loading={!loading && listLoading}
          pagination={{
            current: pageNo,
            total: totalRecords,
            pageSize,
          }}
          onChange={({ current, pageSize }) => this.loadList({ pageNo: current, pageSize })}
        />
      </Page>
    );
  }
}

function mapStateToProps(state) {
  return {};
}

export default connect(mapStateToProps)(SuperposePreferential);
