/**
 * Created by luoming on 2018/3/22
 */
import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import FormItem from 'COMPONENT/FormItem';
import Button from 'COMPONENT/Button';
import Tabs from 'COMPONENT/Tabs';
import CardList from 'COMPONENT/List/CardList';
import Block from 'COMPONENT/Block';
import { Row, Col } from 'antd';
import moment from 'moment';
import styles from './index.less';
import { getPromotionInfoById } from 'SERVICE/preferential';
import { getTextByDictMultipleValue, getTextByDictValue, mapTips, promotionAreaDict } from '../common';
import { getDictTextByVal, getDictValByText } from 'UTIL/dict';
import { continuityDict, FullReducePromotion, isNeedContinuityDict } from '../../config';
import { getUseCondition, transformRuleVosToHours } from '../../utils';
import { clone } from 'lodash';

const Tab = Tabs.Tab;
const [COL_WIDTH_L, COL_WIDTH_M] = [300, 100];

class View extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      list: [],
      currentPage: 1,
      pageSize: 10,
      totalRecords: 0,
      // 连报相关
      continuityType: null,
      continuitySubjectType: null,
      continuityTypeText: '',
      continuitySubjectTypeText: '',
    };
    this.columns = [
      {
        title: '活动信息',
        dataIndex: 'info',
        key: 'info',
        width: COL_WIDTH_L,
        render: (text, record, index) => (
          <div className={styles.activityInfo}>
            <div>{record.promotionText}</div>
            <div>
              <span>{record.organizationName}</span>
              <span>{record.courseSeriesName}</span>
              <span>{record.classTypeName}</span>
              <span>{record.categoryName}</span>
            </div>
            <span>
              <span> </span>
              <span>{mapTips[record.category]}</span>
            </span>
          </div>
        ),
      },
      { title: '单价', dataIndex: 'price', key: 'price', width: COL_WIDTH_M },
      { title: '小时', dataIndex: 'miniClassTotalhours', key: 'miniClassTotalhours', width: COL_WIDTH_M },
      {
        title: '总价',
        dataIndex: 'totalAmount',
        key: 'totalAmount',
        width: COL_WIDTH_M,
        render: (text, record, index) => <span className={styles.totalPrice}>{record.totalAmount}</span>,
      },
    ];
  }

  componentDidMount() {
    this.loadData();
  }

  loadData = async () => {
    let apiParams = { id: this.props.id };
    let { branchDict } = this.props;
    this.setState({ loading: true });
    let { data } = await getPromotionInfoById(apiParams);
    let branchIds = data.branchVos.map((v) => v.branchId);

    data.ruleVos = transformRuleVosToHours(data.ruleVos, this.props.type);

    let conditions = getUseCondition(data);

    const continuityType = data.continuityType;
    const continuitySubjectType = data.continuitySubjectType;

    let list = data.productVos.map((item) => {
      const infoVos = item.productVo.infoVos.map((info) => info.promotionName);
      return {
        ...item.productVo,
        promotionText: infoVos.join('；'),
        promotionList: infoVos,
      };
    });

    this.setState({
      loading: false,
      promotionName: data.promotionName,
      usePlatName: data.usePlatName,
      startDate: moment(data.startDate).format('YYYY-MM-DD'),
      endDate: moment(data.endDate).format('YYYY-MM-DD'),
      customerConditionName: data.customerConditionName,
      branch: getTextByDictMultipleValue(branchDict, branchIds),
      contractConditionName: data.contractConditionName,
      status: data.status,
      promotionLittleTypeName: data.promotionLittleTypeName,
      promotionTypeName: data.promotionTypeName,
      conditions,
      promotionComment: data.promotionComment,
      remark: data.remark,
      list,
      totalRecords: list.length,
      copyList: clone(list),
      continuityType,
      continuitySubjectType,
      continuityTypeText: continuityType ? getDictTextByVal(continuityDict, continuityType) : '不连报',
      continuitySubjectTypeText:
        !continuityType || continuityType === 'NONE'
          ? ''
          : `${getDictTextByVal(isNeedContinuityDict, continuitySubjectType)}相同科目连报`,
      promotionArea: getTextByDictValue(promotionAreaDict, data.useCase),
    });
  };

  search = () => {
    const { value, copyList } = this.state;
    let searchList;
    if (value) {
      searchList = copyList.filter((copyItem) => {
        let filterList = copyItem.promotionList.filter((item) => item.includes(value));
        return filterList.length > 0;
      });
    } else {
      searchList = clone(copyList);
    }
    this.setState({ list: searchList });
  };

  render() {
    let {
      value,
      loading,
      currentPage,
      totalRecords,
      pageSize,
      promotionName,
      usePlatName,
      startDate,
      endDate,
      customerConditionName,
      branch,
      contractConditionName,
      status,
      promotionLittleTypeName,
      promotionTypeName,
      conditions,
      promotionComment,
      remark,
      list,
      continuityTypeText,
      continuitySubjectTypeText,
      promotionArea,
    } = this.state;
    let { propsType, type } = this.props;
    return (
      <Page
        loading={loading}
        footer={false}
        route={this.props.route}
        className={styles.activity}
        ref={(c) => (this.pageRef = c)}
      >
        <Block title="活动信息">
          <div className={styles.wrap}>
            <Row>
              <Col span={12}>
                <FormItem name="优惠活动名称" size="small">
                  {promotionName}
                </FormItem>
              </Col>
              <Col span={12} style={{ display: 'none' }}>
                <FormItem name="使用平台" size="small">
                  {usePlatName}
                </FormItem>
              </Col>
              <Col span={12}>
                <FormItem name="活动时间" size="small">
                  {startDate + '至' + endDate}
                </FormItem>
              </Col>
              <Col span={12} style={{ display: 'none' }}>
                <FormItem name="客户条件" size="small">
                  {customerConditionName}
                </FormItem>
              </Col>
              <Col span={12}>
                <FormItem name="所属分公司" size="small">
                  {branch}
                </FormItem>
              </Col>
              <Col span={12} style={{ display: 'none' }}>
                <FormItem name="合同条件" size="small">
                  {contractConditionName}
                </FormItem>
              </Col>
              <Col span={12}>
                <FormItem name="状态" size="small">
                  {status === '0' ? '有效' : '无效'}
                </FormItem>
              </Col>
              <Col span={12}>
                <FormItem name="种类" size="small">
                  {promotionLittleTypeName}
                </FormItem>
              </Col>
              <Col span={12}>
                <FormItem name="类型" size="small">
                  {promotionTypeName}
                </FormItem>
              </Col>
              <Col span={12}>
                <FormItem name="使用条件" size="small">
                  {conditions}
                </FormItem>
              </Col>
              <Col span={12}>
                <FormItem name="活动说明" size="small">
                  {promotionComment}
                </FormItem>
              </Col>
              {(type === 'FullReducePromotion' || type === 'DiscountPromotion') && (
                <Col span={12}>
                  <FormItem name="连报条件" size="small">
                    {continuityTypeText} {continuitySubjectTypeText}
                  </FormItem>
                </Col>
              )}
              <Col span={12}>
                <FormItem name="活动备注" size="small">
                  {remark}
                </FormItem>
              </Col>
              <Col span={12}>
                <FormItem name="优惠场景" size="small">
                  {promotionArea}
                </FormItem>
              </Col>
            </Row>
          </div>
        </Block>
        <Block title="参与产品">
          <div className={styles.wrap}>
            {propsType !== 'FullGive' && (
              <div className={styles.searchBar}>
                <input
                  type="text"
                  value={value}
                  placeholder="请输入活动名称"
                  onChange={(e) => {
                    this.setState({ value: e.target.value });
                  }}
                />
                <Button type="primary" className={styles.searchBtn} onClick={this.search}>
                  搜索
                </Button>
              </div>
            )}
            {propsType === 'FullGive' && (
              <div className={styles.tabs}>
                <Tabs>
                  <Tab title="产品">页签 1 的内容</Tab>
                  <Tab title="赠品">页签 2 的内容</Tab>
                </Tabs>
                <div className={styles.searchBar}>
                  <input
                    type="text"
                    value={value}
                    placeholder="请输入活动名称"
                    onChange={(e) => {
                      this.setState({ value: e.target.value });
                    }}
                  />
                  <Button type="primary" className={styles.searchBtn} onClick={this.search}>
                    搜索
                  </Button>
                </div>
              </div>
            )}
            <div>
              {propsType !== 'FullGive' && (
                <CardList
                  expandedRowRender={(record) => this.expandedRowRender()}
                  dataSource={list.slice((currentPage - 1) * pageSize, currentPage * pageSize)}
                  columns={this.columns}
                  loading={false}
                  pagination={{
                    current: currentPage,
                    total: totalRecords,
                    pageSize,
                  }}
                  expandedRowKeys={[]}
                  className={styles.table}
                  onChange={({ current, pageSize }) => this.setState({ currentPage: current, pageSize })}
                />
              )}
            </div>
          </div>
        </Block>
      </Page>
    );
  }
}

let mapStateToProps = (state) => {
  let data = state.main.pageParams;
  let pathName = state.routing.locationBeforeTransitions.pathname;
  if (data[pathName]) {
    return {
      type: data[pathName].type,
      id: data[pathName].id,
      branchDict: data[pathName].branchDict,
    };
  } else {
    return {};
  }
};
export default connect(mapStateToProps)(View);
