import { Tooltip } from 'antd';
import commonStyles from 'PAGE/marketActivityManage/common/commonStyle.less';

/**
 * Created by luoming on 2018/4/10
 */
const mapTips = {
  ONE_ON_ONE_COURSE: '一',
  SMALL_CLASS: '小',
  ECS_CLASS: '标',
  OTHERS: '他',
  ONE_ON_MANY: '多',
  TWO_TEACHER: '双',
  LECTURE: '讲',
  LIVE: '播',
};

/** desc 产品性质map bad practice */
const ProductCharacterMap = {
  // test
  DAT000000000645: '常规课程',
  DAT000000000657: '引流课程',
  // pre
  DAT000000000585: '常规课程',
  DAT000000000586: '引流课程',
  // prd
  DAT000000000588: '常规课程',
  DAT000000000589: '引流课程',
};

const mapDataToDict = (data) => {
  return data.map((v, k) => {
    return { val: v.value, text: v.name };
  });
};
const getTextByDictValue = (dict, value) => {
  return dict.filter((v, k) => {
    return v.val === value;
  })[0]['text'];
};
const getTextByDictMultipleValue = (dict, value) => {
  let array = [];
  value.forEach((v, k) => {
    dict.forEach((V, K) => {
      if (v === V.val) {
        array.push(V.text);
      }
    });
  });
  return array.join(',');
};
const mapPromotionType = (value) => {
  let str;
  switch (value) {
    // 满减促销
    case 'FullReducePromotion':
      str = 'PROMOTION_FULL_SALE';
      break;
    // 折扣促销
    case 'DiscountPromotion':
      str = 'PROMOTION_DISCOUNT';
      break;
    // 满赠促销
    case 'FullGivePromotion':
      str = 'PROMOTION_GIVE';
      break;
    // 满减券
    case 'Voucher':
      str = 'COUPON_FULL_SALE';
      break;
    // 折扣券
    case 'DiscountCoupons':
      str = 'COUPON_DISCOUNT';
      break;
    // 满减码
    case 'FullReduceCode':
      str = 'PROMOTION_CODE_SALE';
      break;
    // 折扣码
    case 'DiscountCode':
      str = 'PROMOTION_CODE_DISCOUNT';
    // 个性化课时优惠
    case 'personalizedClassHourDiscount':
      str = 'PROMOTION_PERSONALIZED';
  }
  return str;
};
const mapUnit = {
  MONEY: '元',
  COURSE_HOURS: '小时',
  PRODUCT_NUMS: '个产品',
};
const statusDict = [
  { val: '0', text: '有效' },
  { text: '无效', val: '1' },
];

const promotionAreaDict = [
  { val: 0, text: '不限定' },
  { val: 1, text: '续报场景', warnMsg: '只针对【在线报读续报】和boss【续费合同】使用' },
  { val: 2, text: '自由报读/新签场景', warnMsg: '只针对【在线报读的自由报读】和boss【新签合同】使用' },
];
const promotionAreaDictForPersonalized = [
  { val: 0, text: '不限定' },
  { val: 1, text: '续报场景', warnMsg: '仅续费合同可享受该优惠' },
  { val: 2, text: '新签场景', warnMsg: '仅新签合同可享受该优惠' },
];

const renderMultiLine = (detail, rowData, index) => {
  if (!detail) return null;
  const lines = typeof detail === 'string' ? detail.split(',') : detail;
  const displayLines = lines.slice(0, 2);
  const otherLines = lines.slice(2);
  //弹出框内容
  const content = (
    <div>
      {otherLines.map((branch, index) => {
        return (
          <p style={{ lineHeight: 2 }} key={index}>
            {branch}
          </p>
        );
      })}
    </div>
  );
  if (!lines.length) {
    return null;
  }
  //超出两个才省略
  if (lines.length <= 2) {
    return (
      <div>
        <p>{displayLines[0]} ;</p>
        <p>{displayLines[1]}</p>
      </div>
    );
  } else {
    return (
      <Tooltip placement="top" overlayClassName={commonStyles.maxHeight} title={content}>
        <p>{displayLines[0]} ;</p>
        <p>{displayLines[1]}</p>
        <p>...</p>
      </Tooltip>
    );
  }
};

export {
  mapTips,
  mapDataToDict,
  getTextByDictValue,
  getTextByDictMultipleValue,
  mapPromotionType,
  statusDict,
  mapUnit,
  promotionAreaDict,
  renderMultiLine,
  ProductCharacterMap,
  promotionAreaDictForPersonalized,
};
