/**
 * Created by luoming on 2018/3/21
 */
import React from 'react';
import { connect } from 'dva';
import moment from 'moment';
import styles from './index.less';
import List from 'COMPONENT/List';
import SearchBar from 'COMPONENT/SearchBar';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import Input from 'COMPONENT/Input';
import Button from 'COMPONENT/Button';
import DatePicker from 'COMPONENT/DatePicker';
import ShowMessage from 'COMPONENT/utils/globalTips/showMessage';
import ShowConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import { loadBranchDict } from 'UTIL/dict';
import { getSelectOption, getPromotionList, invalidPromotionById, deletePromotionById } from 'SERVICE/preferential';
import {
  mapPromotionType,
  statusDict,
  promotionAreaDict,
  getTextByDictValue,
  promotionAreaDictForPersonalized,
} from './common';
import Wrapper from 'COMPONENT/Wrapper';
import ShowMore from 'COMPONENT/ShowMore';
import { dateFormatCfg, PROMOTION_DISCOUNT_COURSE } from '../config';
import { getUseCondition, transformRuleVosToHours } from '../utils';

const { COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL, COL_WIDTH_S } = List;

class Activity extends React.Component {
  constructor(props) {
    super(props);
    const isPersonalizedClassHourDiscount = this.isPersonalizedClassHourDiscount(props);
    this.init = this.init.bind(this);
    this.listDataAdapter = this.listDataAdapter.bind(this);
    this.copyPromotions = this.copyPromotions.bind(this);
    this.state = {
      list: [],
      statusDict,
      branchDict: [],
      pageSize: 10,
      loading: false,
      branchId: null,
      currentPage: 0,
      totalRecords: 0,
      promotionArea: null,
      searchExpand: false,
      checkedProductKeys: [],
      promotionLittleTypeId: mapPromotionType(props.type),
      promotionAreaDict: isPersonalizedClassHourDiscount ? promotionAreaDictForPersonalized : promotionAreaDict,
    };
    this.defaultSKey = {};
    this.auditStatusDict = isPersonalizedClassHourDiscount
      ? [
          { val: '1', text: '审核通过' },
          { val: '2', text: '审核中' },
          { val: '3', text: '审核不通过' },
        ]
      : null;
    this.columns = [
      { title: '活动名称', dataIndex: 'promotionName', key: 'promotionName', width: COL_WIDTH_L, fixed: 'left' },
      isPersonalizedClassHourDiscount
        ? { title: '种类', dataIndex: 'promotionTypeName', key: 'promotionTypeName', width: COL_WIDTH_M }
        : { title: '种类', dataIndex: 'promotionLittleTypeName', key: 'promotionLittleTypeName', width: COL_WIDTH_M },
      isPersonalizedClassHourDiscount
        ? null
        : { title: '小类', dataIndex: 'promotionTypeName', key: 'promotionTypeName', width: COL_WIDTH_M },
      isPersonalizedClassHourDiscount
        ? {
            title: '使用条件',
            dataIndex: 'useCondition',
            key: 'useCondition',
            width: COL_WIDTH_XL,
            render: (_, record) =>
              record.ruleVos
                .filter((item) => item.ruleName)
                .map((item) => item.ruleName)
                .join('，'),
          }
        : { title: '使用条件', dataIndex: 'useCondition', key: 'useCondition', width: COL_WIDTH_XL },
      {
        title: '所属分公司',
        dataIndex: 'branchVos',
        key: 'branchVos',
        width: COL_WIDTH_L,
        render: (text) => <ShowMore data={text.map((item) => item.branchName)} splitStartIndex={3} theme="dark" />,
      },
      { title: '活动时间', dataIndex: 'dateRange', key: 'dateRange', width: 280 },
      {
        title: '优惠场景',
        dataIndex: 'useCase',
        key: 'useCase',
        width: 100,
        render: (text) =>
          getTextByDictValue(
            isPersonalizedClassHourDiscount ? promotionAreaDictForPersonalized : promotionAreaDict,
            text,
          ),
      },
      { title: '状态', dataIndex: 'statusText', key: 'statusText', width: COL_WIDTH_S },
      isPersonalizedClassHourDiscount
        ? {
            title: '审核状态',
            dataIndex: 'auditStatus',
            key: 'auditStatus',
            width: COL_WIDTH_M,
            render: (value) => (this.auditStatusDict.find(({ val }) => val == value) || {}).text || '-',
          }
        : null,
      {
        title: '操作',
        key: 'action',
        width: 180,
        fixed: 'right',
        getActionConfig: (text, record, index) => {
          let isBetween = moment().isBetween(
            moment(record.startDate).subtract(1, 'days'),
            moment(record.endDate).add(1, 'days'),
          );
          let statusIsZero = record.status === '0';
          return [
            { text: '查看', onClick: () => this.listBtnClick('view', record, index) },
            { text: '编辑', onClick: () => this.listBtnClick('edit', record, index) },
            {
              menus: [
                statusIsZero
                  ? {
                      text: statusIsZero ? '无效' : '有效',
                      onClick: () => this.deletePromotion(record.id, 'cancel', statusIsZero ? '1' : '0'),
                      disabled: !isBetween,
                    }
                  : null,
                {
                  text: '复制',
                  onClick: () => {
                    this.copyPromotions(record, index);
                  },
                },
                { text: '删除', onClick: () => this.deletePromotion(record.id, 'del'), disabled: false },
              ].filter((v) => v),
            },
          ];
        },
      },
    ].filter((v) => !!v);
  }

  componentDidMount() {
    this.init();
  }

  async init() {
    let [branchDict, { data }] = await Promise.all([
      loadBranchDict(),
      getSelectOption({ category: 'ACTIVITY_TYPE', parentId: this.state.promotionLittleTypeId }),
    ]);

    let typeDict = data.map((v) => ({ val: v.value, text: v.name }));
    if (this.state.promotionLittleTypeId === 'PROMOTION_GIVE') {
      typeDict = typeDict.filter((v) => v.val === 'PROMOTION_GIVE_HOURS');
    }
    if (this.state.promotionLittleTypeId === 'PROMOTION_DISCOUNT') {
      typeDict = typeDict.filter((v) => v.val !== PROMOTION_DISCOUNT_COURSE);
    }
    this.defaultSKey.branchId = branchDict.map((item) => item.val);

    await this.setState({
      branchDict,
      typeDict,
      branchId: this.defaultSKey.branchId,
    });

    this.loadData();
  }

  isPersonalizedClassHourDiscount = (props = this.props) => {
    return props.type === 'personalizedClassHourDiscount';
  };

  loadData = async (searchKey) => {
    this.setState({ loading: true });
    let {
      branchId,
      promotionLittleTypeId,
      promotionName,
      promotionTypeId,
      startDate,
      endDate,
      statusId,
      auditStatus,
      promotionArea,
    } = this.state;
    let apiParams = {
      branchId: this.auxBuildParams(branchId),
      promotionKindId: 'PROMOTION', //大类
      promotionLittleTypeId,
      promotionName,
      promotionTypeId,
      startDate,
      endDate,
      status: statusId,
      useCase: promotionArea,
      pageNo: searchKey ? searchKey.currentPage - 1 : 0,
      pageSize: searchKey ? searchKey.pageSize : 10,
      auditStatus: this.isPersonalizedClassHourDiscount() ? auditStatus : undefined,
    };
    let rsp = await getPromotionList(apiParams);
    this.setState({
      list: this.listDataAdapter(rsp.datas),
      totalRecords: rsp.rowCount,
      currentPage: searchKey ? searchKey.currentPage : 1,
      pageSize: searchKey ? searchKey.pageSize : 10,
      loading: false,
    });
  };

  // 构建list数据
  listDataAdapter(origin) {
    return origin.map((item) => {
      const ruleVos = transformRuleVosToHours(item.ruleVos, this.props.type);
      const newItem = {
        ...item,
        ruleVos,
      };

      return {
        ...newItem,
        dateRange: `${moment(item.startDate).format(dateFormatCfg)}至${moment(item.endDate).format(dateFormatCfg)}`,
        statusText: item.status === '0' ? '有效' : '无效',
        useCondition: getUseCondition(newItem),
      };
    });
  }

  auxBuildParams = (params) => {
    let result = null;
    if (params && params.length > 0) {
      result = params.length > 1 ? params.join(',') : params[0];
    }
    return result;
  };

  loadList = (searchKey) => {
    this.loadData(searchKey);
  };

  search = async () => {
    this.loadData({ currentPage: 1, pageSize: 10 });
  };

  reset = () => {
    this.setState({
      branchId: null,
      auditStatus: null,
      promotionName: null,
      promotionTypeId: null,
      startDate: null,
      endDate: null,
      statusId: null,
      promotionArea: null,
      ...this.defaultSKey,
    });
  };

  listBtnClick = (handle, record = {}, index) => {
    let { dispatch, type } = this.props;
    let { branchDict } = this.state;
    if (handle === 'view') {
      dispatch({
        type: 'main/openPage',
        payload: {
          path: `/${type}/View`,
          params: {
            id: record.id,
            branchDict,
            type,
          },
          refresh: true,
        },
      });
    } else {
      this.AddOrEdit('Edit', record.id);
    }
  };

  AddOrEdit = (handle, id, actionType) => {
    let { dispatch, type } = this.props;
    let { branchDict, typeDict } = this.state;
    dispatch({
      type: 'main/openPage',
      payload: {
        path: `/${type}/${handle}`,
        params: {
          id,
          type,
          handle,
          typeDict,
          branchDict,
          actionType: actionType || handle,
          onSaveSuccess: this.editSaveSuccess,
        },
        refresh: true,
      },
    });
  };

  editSaveSuccess = () => {
    const { dispatch, type } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: { path: `/${type}`, params: {} },
    });
    this.loadData({ currentPage: 1, pageSize: 10 });
  };

  cancelCheck = () => {
    this.setState({ checking: false, checkedProductKeys: null });
  };

  goCopyEditPage = () => {
    let { checkedProductKeys, list } = this.state;
    let { dispatch } = this.props;
    if (checkedProductKeys.length === 0) {
      ShowMessage('请至少选择一个！');
      return;
    }
    let copyList = list.filter((v, k) => {
      return checkedProductKeys.includes(k);
    });
    dispatch({
      type: 'main/openPage',
      payload: {
        path: this.isPersonalizedClassHourDiscount()
          ? `/personalizedClassHourDiscount/Copy`
          : `/FullReducePromotion/Copy`,
        params: { list: copyList },
        refresh: false,
      },
    });
  };

  deletePromotion = (id, handle, status) => {
    if (handle === 'cancel') {
      ShowConfirm(
        `是否将该优惠设为${status === '1' ? '无效' : '有效'}？`,
        async () => {
          await invalidPromotionById({ id, status });
          ShowMessage('操作成功！', 'success');
          this.loadData();
        },
        () => {},
      );
    } else {
      ShowConfirm('该操作将删除此优惠信息，是否继续？', async () => {
        await deletePromotionById({ id });
        ShowMessage('删除成功', 'success');
        this.loadData();
      });
    }
  };

  getDisabledStartDate = (startDate) => {
    let { endDate } = this.state;
    if (!startDate || !endDate) {
      return false;
    }
    return moment(startDate).valueOf() > moment(endDate).valueOf();
  };

  getDisabledEndDate = (endDate) => {
    const { startDate } = this.state;
    if (!endDate || !startDate) {
      return false;
    }
    return moment(endDate).valueOf() < moment(startDate).valueOf();
  };

  copyPromotions(record, index) {
    this.AddOrEdit('Edit', record.id, 'Copy');
  }

  render() {
    let {
      currentPage,
      totalRecords,
      pageSize,
      loading,
      searchExpand,
      promotionName,
      typeDict,
      promotionTypeId,
      branchDict,
      branchId,
      startDate,
      endDate,
      statusDict,
      statusId,
      auditStatus,
      promotionArea,
      checking,
      checkedProductKeys = [],
      list,
    } = this.state;
    return (
      <Wrapper
        className={styles.activity}
        bodyFitHeight
        header={
          <div className={styles.searchBar}>
            <Button className={styles.add} onClick={(e) => this.AddOrEdit('Add')}>
              新增
            </Button>
            <SearchBar
              expand={searchExpand}
              onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
              onSearch={this.search}
              onReset={this.reset}
            >
              <Input
                placeholder="名称"
                value={promotionName}
                onChange={(e) => this.setState({ promotionName: e.target.value })}
              />
              {this.isPersonalizedClassHourDiscount() ? null : (
                <Select
                  allowClear
                  placeholder="小类"
                  showSearch={true}
                  dict={typeDict}
                  value={promotionTypeId}
                  onChange={(val) => this.setState({ promotionTypeId: val })}
                />
              )}
              <MultiSelect
                placeholder="所属分公司"
                showSearch={true}
                dict={branchDict}
                value={branchId}
                onChange={(val) => this.setState({ branchId: val })}
              />
              <DatePicker
                placeholder="起始日期"
                value={startDate}
                onChange={(val) => this.setState({ startDate: val })}
                disabledDate={(val) => this.getDisabledStartDate(val)}
              />
              <DatePicker
                placeholder="截止日期"
                value={endDate}
                onChange={(val) => this.setState({ endDate: val })}
                disabledDate={(val) => this.getDisabledEndDate(val)}
              />
              <Select
                allowClear
                placeholder="状态"
                showSearch={true}
                dict={statusDict}
                value={statusId}
                onChange={(val) => this.setState({ statusId: val })}
              />
              {this.isPersonalizedClassHourDiscount() ? (
                <Select
                  placeholder="审核状态"
                  showSearch={true}
                  allowClear
                  dict={this.auditStatusDict}
                  value={auditStatus}
                  onChange={(val) => this.setState({ auditStatus: val })}
                />
              ) : null}
              <Select
                allowClear
                showSearch={true}
                placeholder="优惠场景"
                value={promotionArea}
                onChange={(val) => this.setState({ promotionArea: val })}
                dict={this.isPersonalizedClassHourDiscount() ? promotionAreaDictForPersonalized : promotionAreaDict}
              />
            </SearchBar>
          </div>
        }
      >
        <List
          dataSource={list}
          columns={this.columns}
          loading={loading}
          rowSelection={
            checking
              ? {
                  selectedRowKeys: checkedProductKeys,
                  onChange: (keys) => this.setState({ checkedProductKeys: keys }),
                }
              : null
          }
          pagination={{
            current: currentPage,
            total: totalRecords,
            pageSize,
          }}
          onChange={({ current, pageSize }) => this.loadList({ currentPage: current, pageSize })}
        />
        <div>
          {/*{checking ?*/}
          {/*<div>*/}
          {/*<Button type="primary" onClick={this.goCopyEditPage}>确认复制</Button>*/}
          {/*<Button type="primary" ghost onClick={this.cancelCheck}*/}
          {/*style={{marginLeft: 10}}>取消</Button>*/}
          {/*</div>*/}
          {/*:*/}
          {/*<Button type="primary" ghost onClick={() => this.setState({checking: true})}>批量复制</Button>*/}
          {/*}*/}
        </div>
      </Wrapper>
    );
  }
}

export default connect()(Activity);
