/**
 * Created by luoming on 2018/3/30
 */
import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import Block from 'COMPONENT/Block';
import FormItem from 'COMPONENT/FormItem';
import Input from 'COMPONENT/Input';
import InputNumber from 'COMPONENT/InputNumber';
import Select from 'COMPONENT/Select';
import Button from 'COMPONENT/Button';
// import DatePicker from 'COMPONENT/DatePicker';
import MultiSelect from 'COMPONENT/MultiSelect';
import Radio from 'COMPONENT/Radio';
import TextArea from 'COMPONENT/TextArea';
import IconButton from 'COMPONENT/IconButton';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import Search from 'COMPONENT/Search';
import CardList from 'COMPONENT/List/CardList';
import List from 'COMPONENT/List';
import { Row, Col, DatePicker } from 'antd';
import moment from 'moment';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import styles from '../../Activity/Add/index.less';
import InfoBoard from '../../Activity/Components/InfoBoard';
import PopTip from 'COMPONENT/PopTip';
import { getSelectOption, savePromotion, getProductList, getPromotionInfoById } from 'SERVICE/preferential';
import {
  mapTips,
  mapDataToDict,
  getTextByDictValue,
  getTextByDictMultipleValue,
  mapPromotionType,
  mapUnit,
} from '../../Activity/common';
import Steps from '../../Activity/Components/Steps/index';
import ChangeTabs from '../../Activity/Components/Tabs';
import { loadCommonDict, loadBranchDict } from 'UTIL/dict';
import { cloneDeep, difference, differenceWith, intersectionWith, indexOf } from 'lodash';

const [COL_WIDTH_XL, COL_WIDTH_L, COL_WIDTH_M] = [150, 600, 150];
const mapType = {
  COUPON_FULL_SALE_NORMAL: '满减',
  COUPON_FULL_SALE_EVERY: '每满减',
  COUPON_DISCOUNT_CASH: '现金折扣',
  COUPON_DISCOUNT_COURSE: '课时折扣',
};
const Group = Radio.Group;

/**
 * Page组件底部toolbar
 * */
const renderFooter = (props) => {
  const {
    handlePrev,
    handleNext,
    currentIndex,
    tabKey,
    handleBtn,
    popTipVisible,
    averagePromotionAcount,
    onClose,
    content,
  } = props;
  return (
    <div style={{ display: 'flex', justifyContent: 'flex-end' }}>
      {currentIndex === 1 && tabKey === 1 && (
        <Button
          onClick={() => {
            handleBtn({ tabKey });
          }}
          style={{ marginRight: 20 }}
        >
          确认移除
        </Button>
      )}
      <PopTip content={content} visible={popTipVisible} onClose={onClose}>
        <Button type="primary" onClick={handleNext}>
          {currentIndex < 2 ? '下一步' : '完成'}
        </Button>
      </PopTip>
      {currentIndex <= 2 && currentIndex > 0 && (
        <Button style={{ marginLeft: '20px' }} onClick={handlePrev}>
          上一步
        </Button>
      )}
    </div>
  );
};

class Add extends React.Component {
  tags = 0;
  searchKeys = {};
  hideTage = null;
  copyBranchVos = {};

  selectedStorage = [];

  constructor(props) {
    super(props);
    this.state = {
      currentIndex: 0,
      ruleVos: [{}],
      perFullReduce: {},
      promotionType: this.props.type === 'Voucher' ? 'COUPON_FULL_SALE_NORMAL' : 'COUPON_DISCOUNT_CASH',

      ruleUnit: 'MONEY',
      contractCondition: 'ALL',
      customerCondition: 'ALL',
      getCondition: 'ALL',
      //
      currentPage: 1,
      pageSize: 10,
      totalRecords: 0,
      list: [],
      selectedListKeys: [],
      selectedList: [],
      selectPage: 1,
      selectPageSize: 10,
      selectedRowKeysTwo: [],

      expandedRowKeys: [],

      stepTwoTabKey: 0, //step 2 tab key
    };
  }

  async componentDidMount() {
    let { id } = this.props;
    this.setState({ loading: true });
    let [{ data: getData }, { data: cusData }, { data: conData }] = await Promise.all([
      getSelectOption({ category: 'PromotionGetCondition' }),
      getSelectOption({ category: 'PromotionCusCondition' }),
      getSelectOption({ category: 'PromotionConCondition' }),
      id ? this.loadDataById(id) : null,
    ]);
    this.setState({ loading: false });
    let getDict = mapDataToDict(getData);
    let cusDict = mapDataToDict(cusData);
    let conDict = mapDataToDict(conData);
    this.setState({
      getDict,
      cusDict,
      conDict,
    });
  }

  loadDataById = async (promotionId) => {
    let { data } = await getPromotionInfoById({ id: promotionId });
    let {
      promotionName,
      startDate,
      endDate,
      receiveStartDate,
      receiveEndDate,
      publishNum,
      branchVos,
      promotionTypeId,
      ruleVos,
      usePlat,
      receiveLimit,
      customerCondition,
      contractCondition,
      remark,
      promotionComment,
      productVos,
      id,
      status,
      hightAmount,
    } = data;
    this.selectedStorage = productVos.map((v, k) => {
      return { ...v.productVo, promotionId: v.promotionId, id: v.id, productId: v.productId };
    });
    this.setState({
      promotionName,
      startDate: moment(startDate, 'YYYY-MM-DD HH:mm:ss'),
      endDate: moment(endDate, 'YYYY-MM-DD HH:mm:ss'),
      activityStartDate: moment(receiveStartDate, 'YYYY-MM-DD HH:mm:ss'),
      activityEndDate: moment(receiveEndDate, 'YYYY-MM-DD HH:mm:ss'),
      branchVos: branchVos.map((v, k) => v.branchId),
      promotionType: promotionTypeId,
      ruleVos,
      publishNum,
      // usePlat,
      getCondition: receiveLimit,
      customerCondition,
      contractCondition,
      remark,
      explain: promotionComment,
      selectedList: productVos.map((v, k) => {
        return { ...v.productVo, promotionId: v.promotionId, id: v.id, productId: v.productId };
      }),
      id,
      status,
      perFullReduce: { ...ruleVos[0], maxAmount: hightAmount },
      ruleUnit: ruleVos[0].ruleUnit,
    });
  };
  next = () => {
    let { currentIndex } = this.state;
    let { branchDict, type } = this.props;
    let error = (text) => getWrappedInstance(this.pageRef).showOkTips(text);
    if (currentIndex === 0) {
      let {
        promotionName,
        startDate,
        endDate,
        activityStartDate,
        activityEndDate,
        branchVos = [],
        publishNum,
        ruleVos,
        perFullReduce,
        promotionType,
        contractCondition,
        customerCondition,
        getCondition,
        explain,
        remark,
        cusDict,
        conDict,
        getDict,
        ruleUnit,
        status,
      } = this.state;
      if (!promotionName) {
        error('请输入活动名称！');
        return;
      }
      if (!startDate) {
        error('请输入起始时间！');
        return;
      }
      if (!endDate) {
        error('请输入终止时间！');
        return;
      }
      if (!activityStartDate) {
        error('请输入领取开始时间！');
        return;
      }
      if (!activityEndDate) {
        error('请输入领取结束时间！');
        return;
      }
      if (!publishNum) {
        error('请输入发券数量！');
        return;
      }
      if (!branchVos.length) {
        error('请选择所属分公司！');
        return;
      }
      if (!promotionType) {
        error('请选择活动类型！');
        return;
      }

      if (promotionType === 'COUPON_FULL_SALE_EVERY') {
        if (!perFullReduce['promotionRule'] || !perFullReduce['promotion'] || !perFullReduce['maxAmount']) {
          error('请完善满减条件！');
          return;
        }
        if (ruleUnit === 'MONEY' && perFullReduce['promotionRule'] < perFullReduce['promotion']) {
          error('满减金额不能大于满减条件，请改正！');
          return;
        }
      } else {
        let checkRule = ruleVos.filter((v, k) => {
          return v.promotionRule === undefined || v.promotion === undefined;
        });
        if (checkRule.length > 0) {
          error('请完善满减条件！');
          return;
        }

        let len = ruleVos.length;
        let checkedOne, checkedTwo;
        if (promotionType === 'COUPON_FULL_SALE_NORMAL' && ruleUnit === 'MONEY') {
          for (let i = 0; i < len; i++) {
            if (ruleVos[i].promotionRule < ruleVos[i].promotion) {
              checkedTwo = true;
              break;
            }
          }
        }
        if (checkedTwo) {
          error('满减金额不能大于满减条件，请改正！');
          return;
        }
        let array = ruleVos.map((v, k) => v.promotionRule);
        if (array.length !== [...new Set(array)].length) {
          error('满减条件不能重复，请改正！');
          return;
        }
      }

      if (!contractCondition) {
        error('请选择合同条件！');
        return;
      }
      if (!customerCondition) {
        error('请选择客户条件！');
        return;
      }
      if (!getCondition) {
        error('请选择领取条件！');
        return;
      }

      let customerConditionName = getTextByDictValue(cusDict, customerCondition);
      let getConditionName = getTextByDictValue(getDict, getCondition);
      let contractConditionName = getTextByDictValue(conDict, contractCondition);
      let branchName = getTextByDictMultipleValue(branchDict, branchVos);

      let unit =
        type === 'Voucher'
          ? ruleUnit === 'MONEY'
            ? '元'
            : '课时'
          : promotionType === 'COUPON_DISCOUNT_CASH'
          ? '元'
          : promotionType === 'COUPON_DISCOUNT_COURSE'
          ? '课时'
          : '个产品';
      let condition = '';
      if (promotionType === 'COUPON_FULL_SALE_EVERY') {
        condition = `每满${perFullReduce['promotionRule']}${perFullReduce['ruleUnitName'] || unit}减${
          perFullReduce['promotion']
        }元，最高减${perFullReduce['maxAmount']}元`;
      } else {
        ruleVos.forEach((v, k) => {
          condition += `满${v.promotionRule}${unit}${type === 'Voucher' ? '减' : '折'}${v.promotion}${
            type === 'Voucher' ? '元' : '%'
          };`;
        });
      }
      let data = [
        { name: '优惠活动名称', info: promotionName },
        { name: '领取限制', info: getConditionName },
        {
          name: '活动时间',
          info: `${startDate.format('YYYY-MM-DD HH:mm:ss')}至${endDate.format('YYYY-MM-DD HH:mm:ss')}`,
        },
        {
          name: '客户条件',
          info: customerConditionName,
        },
        { name: '所属分公司', info: branchName },
        { name: '合同条件', info: contractConditionName },
        {
          name: '状态',
          info: status === '1' ? '无效' : '有效',
        },
        { name: '种类', info: '' },
        {
          name: '类型',
          info: mapType[promotionType],
        },
        { name: '使用条件', info: condition },
        { name: '发放数量', info: publishNum },
        {
          name: '活动说明',
          info: explain,
        },
        { name: '活动备注', info: remark },
      ];

      this.setState({ data });
      if (this.tags === 0 || JSON.stringify(this.copyBranchVos) !== JSON.stringify(branchVos)) {
        this.getSecondOption();
        this.copyBranchVos = branchVos;
        this.tags = 1;
      } else {
        const { conditions, ruleUnit, promotionType } = this.state;
        let selectProduct;
        if (ruleUnit === 'COURSE_HOURS' || promotionType === 'COUPON_DISCOUNT_COURSE') {
          selectProduct = this.productDict.filter((v, k) => {
            return v.val === 'ONE_ON_ONE_COURSE' || v.val === 'ONE_ON_MANY';
          });
        }

        let lastProductDict = selectProduct ? selectProduct : this.productDict;
        conditions[0]['dict'] = lastProductDict;
        this.setState({ conditions });
        this.getProductList();
      }
    } else if (currentIndex === 1) {
      // if (this.state.selectedList.length === 0) {
      //   error('请选择产品！');
      //   return;
      // }
    } else if (currentIndex === 2) {
      this.saveData();
    }
    if (currentIndex < 2) {
      this.setState({ currentIndex: currentIndex + 1 }, () => {
        if (currentIndex === 0 && this.props.id) {
          this.$changeTabs.onChange(1);
        }
      });
    }
  };
  saveData = async () => {
    let {
      branchVos,
      contractCondition,
      customerCondition,
      getCondition,
      platform,
      startDate,
      endDate,
      publishNum,
      explain,
      remark,
      activityStartDate,
      activityEndDate,
      promotionName,
      promotionType,
      ruleVos,
      perFullReduce,
      ruleUnit,
      selectedList,
      id,
      status,
    } = this.state;
    let branchVosData = branchVos.map((v, k) => {
      return { branchId: v, id: 0, promotionId: 0 };
    });
    switch (promotionType) {
      case 'COUPON_DISCOUNT_CASH':
        ruleUnit = 'MONEY';
        break;
      case 'COUPON_DISCOUNT_COURSE':
        ruleUnit = 'COURSE_HOURS';
        break;
    }

    let ruleVosData;
    if (promotionType === 'COUPON_FULL_SALE_EVERY') {
      ruleVosData = [
        {
          giftVos: [],
          id: perFullReduce.id,
          promotion: perFullReduce.promotion,
          promotionId: perFullReduce.promotionId,
          promotionIds: 0,
          promotionRule: perFullReduce.promotionRule,
          ruleName: perFullReduce.useNum,
          ruleUnit,
          unit: 'MONEY',
          useNum: perFullReduce.useNum,
        },
      ];
    } else {
      ruleVosData = ruleVos.map((v, k) => {
        return {
          giftVos: [],
          id: v.id,
          promotion: v.promotion,
          promotionId: v.promotionId,
          promotionIds: 0,
          promotionRule: v.promotionRule,
          ruleName: v.useNum,
          ruleUnit,
          unit: 'MONEY',
          useNum: v.useNum,
        };
      });
    }
    let productVosData = this.buildProductVos();
    let apiParams = {
      branchVos: branchVosData, //分公司信息
      contractCondition, //合同条件
      customerCondition, //用户条件
      startDate: startDate.format('YYYY-MM-DD HH:mm:ss'), //开始日期
      endDate: endDate.format('YYYY-MM-DD HH:mm:ss'), //结束日期
      firstLevel: 0, //暂时为0
      hightAmount: perFullReduce['maxAmount'], //最高优惠金额
      id: id || 0,
      limitProductType: 'ALL',
      productVos: productVosData, //对应优惠产品信息
      promotionCode: '', //优惠码 暂时传空
      promotionComment: explain, //活动说明，
      promotionKindId: 'COUPON', //优惠类型
      promotionLittleTypeId: mapPromotionType(this.props.type), //优惠小类
      promotionName: promotionName, //优惠名字
      promotionTypeId: promotionType, //小小类
      publishNum, //发布数量
      receiveEndDate: activityEndDate.format('YYYY-MM-DD HH:mm:ss'), //领取结束时间
      receiveStartDate: activityStartDate.format('YYYY-MM-DD HH:mm:ss'), //领取开始时间
      receiveLimit: getCondition, //领取限制
      remark, //备注
      ruleVos: ruleVosData, //规则信息
      status: Number(status) || 0,
      superPosition: 0,
      useNum: 0, //使用数量 新增为空
      usePlat: platform, //使用平台
    };
    await savePromotion(apiParams);
    showMessage('保存成功', 'success');
    this.props.onSaveSuccess();
  };

  setReduceCondition = (type, value, index, item) => {
    let { ruleVos, perFullReduce } = this.state;
    if (type === 'COUPON_FULL_SALE_EVERY') {
      perFullReduce[item] = value;
      this.setState({ perFullReduce });
    } else {
      ruleVos[index] = ruleVos[index] || {};
      ruleVos[index][item] = value;
      this.setState({ ruleVos });
    }
  };
  renderReduceCondition = () => {
    let { type } = this.props;
    let { promotionType, ruleVos, perFullReduce = {}, ruleUnit = 'MONEY' } = this.state;
    let chineseUnit =
      type === 'Voucher'
        ? ruleUnit === 'MONEY'
          ? '元'
          : '课时'
        : promotionType === 'COUPON_DISCOUNT_CASH'
        ? '元'
        : '课时';
    let len = ruleVos.length;
    if (promotionType === 'COUPON_FULL_SALE_EVERY') {
      return (
        <div className={styles.perFullReduce}>
          <span>
            <span>每满</span>
            <InputNumber
              min={0}
              precision={ruleUnit === 'MONEY' ? 2 : 0}
              value={perFullReduce.promotionRule}
              max={9999999}
              onChange={(val) => this.setReduceCondition(promotionType, val, 0, 'promotionRule')}
            />{' '}
            <span>{ruleUnit === 'MONEY' ? '元' : '课时'}</span>{' '}
          </span>
          <span>
            <span>减</span>{' '}
            <InputNumber
              min={0}
              precision={2}
              value={perFullReduce.promotion}
              max={9999999}
              onChange={(val) => this.setReduceCondition(promotionType, val, 0, 'promotion')}
            />{' '}
            <span>元</span>
          </span>
          <br />
          <span>
            <span>最高减</span>
            <InputNumber
              min={0}
              precision={2}
              value={perFullReduce.maxAmount}
              max={9999999}
              onChange={(val) => this.setReduceCondition(promotionType, val, 0, 'maxAmount')}
            />
            <span>元</span>
          </span>
        </div>
      );
    } else {
      return ruleVos.map((v, k) => {
        return (
          <div key={k} className={styles.reduceCondition}>
            <span>
              <span>满</span>
              <InputNumber
                value={v.promotionRule}
                min={0}
                precision={chineseUnit === '元' ? 2 : 0}
                max={9999999}
                onChange={(val) => this.setReduceCondition(promotionType, val, k, 'promotionRule')}
              />{' '}
              <span>{chineseUnit}</span>
            </span>
            <span>
              <span>{promotionType === 'COUPON_FULL_SALE_NORMAL' ? '减' : '折'}</span>
              <InputNumber
                value={v.promotion}
                min={0}
                precision={promotionType === 'COUPON_FULL_SALE_NORMAL' ? 2 : 0}
                max={promotionType === 'COUPON_FULL_SALE_NORMAL' ? 9999999 : 100}
                onChange={(val) => this.setReduceCondition(promotionType, val, k, 'promotion')}
              />{' '}
              <span>{promotionType === 'COUPON_FULL_SALE_NORMAL' ? '元' : '%'}</span>
            </span>
            <span>
              {len === 1 && (
                <IconButton
                  onClick={(e) => this.changeReduceCondition(k, 'plus')}
                  type="plus_circle"
                  size={20}
                  style={{ position: 'relative', top: 5 }}
                />
              )}
              {len !== 1 && k + 1 < len && (
                <IconButton
                  onClick={(e) => this.changeReduceCondition(k, 'minus')}
                  type="minus_circle"
                  size={20}
                  style={{ position: 'relative', top: 5 }}
                />
              )}
              {len !== 1 && k + 1 === len && (
                <span>
                  <IconButton
                    onClick={(e) => this.changeReduceCondition(k, 'plus')}
                    type="plus_circle"
                    size={20}
                    style={{ position: 'relative', top: 5, marginRight: 20 }}
                  />
                  <IconButton
                    onClick={(e) => this.changeReduceCondition(k, 'minus')}
                    type="minus_circle"
                    size={20}
                    style={{ position: 'relative', top: 5 }}
                  />
                </span>
              )}
            </span>
          </div>
        );
      });
    }
  };
  changeReduceCondition = (k, type) => {
    let { ruleVos } = this.state;
    if (type === 'plus') {
      if (ruleVos.length === 3) {
        showMessage('最多可设置3档！');
        return;
      }
      ruleVos.push({});
    } else {
      ruleVos.pop();
    }
    this.setState({ ruleVos });
  };
  getDisabledStartDate = (startDate) => {
    let { endDate, activityStartDate, activityEndDate } = this.state;
    if (!startDate || (!endDate && !activityStartDate && !activityEndDate)) {
      return false;
    }
    let min = moment.min(
      endDate ? moment(endDate) : moment('2099-12-31'),
      activityStartDate ? moment(activityStartDate) : moment('2099-12-31'),
      activityEndDate ? moment(activityEndDate) : moment('2099-12-31'),
    );
    return moment(startDate).valueOf() > min.valueOf();
  };
  getDisabledEndDate = (endDate) => {
    const { startDate, activityStartDate, activityEndDate } = this.state;
    if (!endDate || (!startDate && !activityStartDate && !activityEndDate)) {
      return false;
    }
    let max = moment.max(
      startDate ? moment(startDate) : moment('2000-01-01'),
      activityStartDate ? moment(activityStartDate) : moment('2000-01-01'),
      activityEndDate ? moment(activityEndDate) : moment('2000-01-01'),
    );
    return moment(endDate).valueOf() < max.valueOf();
  };
  getActivityDisabledStartDate = (activityStartDate) => {
    let { activityEndDate, startDate, endDate } = this.state;
    if (!activityStartDate || (!activityEndDate && !startDate && !endDate)) {
      return false;
    }
    let min = moment
      .min(
        endDate ? moment(endDate) : moment('2099-12-31'),
        activityEndDate ? moment(activityEndDate) : moment('2099-12-31'),
      )
      .add(1, 'days');
    return !moment(activityStartDate).isBetween(startDate ? moment(startDate).subtract(1, 'days') : '2000-01-01', min);
  };
  getActivityDisabledEndDate = (activityEndDate) => {
    const { activityStartDate, startDate, endDate } = this.state;
    if (!activityEndDate || (!activityStartDate && !startDate && !endDate)) {
      return false;
    }
    let max = moment
      .max(
        startDate ? moment(startDate) : moment('2000-01-01'),
        activityStartDate ? moment(activityStartDate) : moment('2000-01-01'),
      )
      .subtract(1, 'days');
    return !moment(activityEndDate).isBetween(max, endDate ? moment(endDate).add(1, 'days') : '2099-12-31');
  };
  //second
  getSecondOption = async () => {
    this.setState({ loading: true });
    let { branchDict } = this.props;
    let { ruleUnit, promotionType } = this.state;
    let [productDict, gradeDict, subjectDict, quarterDict, phaseDict, classDict, groupDict, seriesDict, yearDict] =
      await Promise.all([
        loadCommonDict('PRODUCT_TYPE'), //产品类型
        loadCommonDict('STUDENT_GRADE'), //年级
        loadCommonDict('SUBJECT'), //科目
        loadCommonDict('PRODUCT_QUARTER'), //季度
        loadCommonDict('SMALL_CLASS_PHASE'), //期数
        loadCommonDict('CLASS_TYPE'), //班级类型
        loadCommonDict('PRODUCT_GROUP'), //产品组
        loadCommonDict('COURSE_SERIES'), //课程系列
        loadCommonDict('PRODUCT_VERSION'), //年份
        this.getProductList(),
      ]);
    this.productDict = productDict;
    let selectProduct;
    if (ruleUnit === 'COURSE_HOURS' || promotionType === 'COUPON_DISCOUNT_COURSE') {
      selectProduct = productDict.filter((v, k) => {
        return v.val === 'ONE_ON_ONE_COURSE' || v.val === 'ONE_ON_MANY';
      });
    }
    let lastProductDict = selectProduct ? selectProduct : productDict;
    let conditions = [
      { title: '产品类型', field: 'productType', dict: lastProductDict, isMultiSelect: true },
      { title: '分公司', field: 'branchSecond', dict: branchDict },
      { title: '年份', field: 'year', dict: yearDict },
      { title: '年级', field: 'grade', dict: gradeDict },
      { title: '科目', field: 'subject', dict: subjectDict },
      {
        title: [
          { text: '季度', val: 'holiday' },
          { text: '期数', val: 'periods' },
          { text: '班级类型', val: 'classType' },
          { text: '产品组', val: 'group' },
          { text: '产品系列', val: 'series' },
        ],
        type: 'moreSelect',
        dict: {
          holiday: quarterDict,
          periods: phaseDict,
          classType: classDict,
          group: groupDict,
          series: seriesDict,
        },
      },
    ];
    this.setState({ conditions });
  };
  getProductList = async (pageKeys = {}) => {
    await this.initSelectedKeysByLoadList();
    let { productName, promotionType, ruleUnit, branchVos } = this.state;
    let loadCourse = promotionType === 'COUPON_DISCOUNT_COURSE' || ruleUnit === 'COURSE_HOURS';
    let types = this.searchKeys.productType ? this.searchKeys.productType.join(',') : '';
    if (loadCourse && types === '') {
      types = 'ONE_ON_MANY,ONE_ON_ONE_COURSE';
    }
    let apiParams = {
      name: this.searchKeys['inputValue'],
      brenchId: this.searchKeys.branchSecond,
      brenchIds: branchVos.join(','),
      classTypeId: this.searchKeys.classType,
      courseSeriesId: this.searchKeys.series,
      gradeId: this.searchKeys.grade,
      phareId: this.searchKeys.periods,
      productGroupId: this.searchKeys.group,
      promotionId: '',
      seasonId: this.searchKeys.holiday,
      subjectIds: this.searchKeys.subject,
      types,
      // 页数是从0开始
      pageNo: pageKeys.currentPage ? pageKeys.currentPage - 1 : 0,
      pageSize: pageKeys.pageSize || this.state.pageSize,
      showOtherPromotion: this.hideTage,
      yearId: this.searchKeys.year,
    };
    let { datas, rowCount } = await getProductList(apiParams);
    this.setState({ list: datas, totalRecords: rowCount, loading: false, selectedRowKeys: [] });
    this.initSelectedByLoadList({ list: datas });
  };
  onSearch = async (keys) => {
    this.searchKeys = keys;
    this.setState({ loading: true });
    this.getProductList();
  };
  getListByChangePage = async (pageKeys) => {
    await this.setState({ loading: true, currentPage: pageKeys.currentPage, pageSize: pageKeys.pageSize });
    this.getProductList(pageKeys);
  };
  onInputChange = (value) => {
    this.searchKeys['inputValue'] = value;
    // this.setState({productName: value});
  };
  expandedRowRender = (record) => {
    let { infoVos } = record;
    infoVos = infoVos.filter((v, k) => {
      return (
        v.promotionLittleTypeId === 'PROMOTION_FULL_SALE' ||
        v.promotionLittleTypeId === 'PROMOTION_DISCOUNT' ||
        v.promotionLittleTypeId === 'PROMOTION_GIVE'
      );
    });
    return (
      <div className={'CardList-expand-wrap'}>
        <div className={styles.expandContent}>
          {infoVos.map((v, k) => {
            // let handle = (v.promotionTypeName === '满减' || v.promotionTypeName === '每满减') ? '减' : '折';
            // let unit = (v.promotionTypeName === '满减' || v.promotionTypeName === '每满减') ? '元' : '%';

            let handle =
              v.promotionTypeName === '满减' || v.promotionTypeName === '每满减'
                ? '减'
                : v.promotionTypeName === '赠课时'
                ? '赠'
                : '折';
            let unit =
              v.promotionTypeName === '满减' || v.promotionTypeName === '每满减'
                ? '元'
                : v.promotionTypeName === '赠课时'
                ? '课时'
                : '%';

            return (
              <div key={k} className={styles.item}>
                <span>
                  <span>{v.promotionTypeName}</span>
                </span>
                <span>
                  <div>{v.promotionName}</div>
                  <div>
                    {v.ruleVos.map((V, K) => {
                      let chineseUnit = mapUnit[V.ruleUnit];
                      if (v.promotionTypeName === '每满减') {
                        return (
                          <span key={K}>
                            {'每满' +
                              V.promotionRule +
                              chineseUnit +
                              handle +
                              V.promotion +
                              unit +
                              '，最高减' +
                              v.hightAmount +
                              unit}
                          </span>
                        );
                      } else {
                        return (
                          <span key={K}>
                            {'满' + V.promotionRule + chineseUnit + handle + V.promotion + unit + ';'}
                          </span>
                        );
                      }
                    })}
                  </div>
                </span>
              </div>
            );
          })}
        </div>
      </div>
    );
  };
  onExpand = (index, record) => {
    let { expandedRowKeys } = this.state;
    let idx = expandedRowKeys.findIndex((v, k) => {
      return v === index;
    });
    if (idx === -1) {
      expandedRowKeys.push(index);
    } else {
      expandedRowKeys.splice(idx, 1);
    }
    this.setState({ expandedRowKeys });
  };

  selectChange = (selectedRowKeys, selectedRows) => {
    this.setState({ selectedRowKeys });
  };

  selectChangeTwo = (selectedRowKeysTwo, selectedRows) => {
    this.setState({ selectedRowKeysTwo });
  };
  DontShowList = (e) => {
    this.hideTage = e.target.checked ? 1 : null;
    this.setState({ loading: true });
    this.getProductList();
  };

  /**
   * 监听Tab key
   * */
  onSwitchTab = (params) => {
    this.setState((prevState) => ({ stepTwoTabKey: params }));
  };

  // ** 在线报读4期 **
  /**
   * 更新state
   * */
  updateState = (newState) => {
    this.setState((prevState, props) => {
      return {
        ...newState,
      };
    });
  };
  /**
   * 选择产品
   * list:当前页面选中项[array]
   * selectedList:全部选中项[array]
   * this.selectStorage:修改/编辑时，从接口获取的已选中项
   * */
  chooseItem = (keys) => {
    if (this.checkSelectType(keys)) {
      // 增加选择
      const list = this.getSelectedItem(keys);
      let selectedList = this.buildSelectedListByAdd(this.selectedStorage, list);
      this.selectedStorage = cloneDeep(selectedList);
      this.setState({ selectedList });
    } else {
      // 删减选择
      let subItemArr = this.getSubItemArr(keys);
      let selectedList = this.buildSelectedListBySub(this.selectedStorage, subItemArr);
      this.selectedStorage = cloneDeep(selectedList);
      this.setState({ selectedList });
    }
    this.setSelectedKeys(keys);
  };
  /**
   * 判断chooseItem是增操作还是减操作
   * */
  checkSelectType = (params) => {
    const { selectedListKeys } = this.state;
    return params.length > selectedListKeys.length;
  };
  /**
   * 设置选中状态
   * 更新selectedListKeys,返回已选中项的数据
   * -用于chooseItem
   * */
  setSelectedKeys = (params) => {
    this.updateState({ selectedListKeys: params });
  };
  /**
   * 获取当前列表的选中项
   * */
  getSelectedItem = (params) => {
    const { list } = this.state;
    return params.map((item) => list[item]);
  };
  /**
   * 构建参数SelectedList增
   * -用于chooseItem
   * */
  buildSelectedListByAdd = (origin, other) => {
    const otherDiff = differenceWith(other, origin, this.diffComparator);
    return origin.concat(otherDiff).map(this.auxBuildItem);
  };
  /**
   * 辅助构建item
   * */
  auxBuildItem = (item, index, arr) => {
    let id = item.productId ? item.id : 0;
    let productId = item.productId || item.id;
    return {
      ...item,
      productId,
      id,
    };
  };
  /**
   * 比较两数组各项是否为同一项
   * -用于differenceWith, intersectionWith
   * */
  diffComparator = (origin, other) => {
    return origin.id === other.id || origin.id === other.productId;
  };
  diffComparatorBySub = (origin, other) => {
    return origin.productId === other.id || origin.productId === other.productId;
  };
  /**
   * 在切换页码时初始化已选择优惠相关数据
   * -用于getProductList
   * */
  initSelectedKeysByLoadList = () => {
    this.updateState({ selectedListKeys: [] });
  };
  /**
   * (编辑/修改时)加载产品列表时,更新已选产品
   * -用于getProductList
   * */
  initSelectedByLoadList = (params) => {
    const { list } = params;
    let same = intersectionWith(list, this.selectedStorage, this.diffComparator);
    let arr = [];
    same.forEach((item) => {
      let checkIndex = indexOf(list, item);
      if (checkIndex !== -1) {
        arr.push(checkIndex);
      }
    });
    this.setSelectedKeys(arr.length > 0 ? arr : []);
  };
  /**
   * 将已选数据处理处理成所需(productVos)格式
   * -用于saveData
   * */
  buildProductVos = () => {
    const { selectedList } = this.state;
    let list = cloneDeep(selectedList);
    return list.map((item) => ({
      amount: item.amount || 0,
      id: item.id || 0,
      productId: item.productId,
      promotionId: item.promotionId || 0,
    }));
  };
  /**
   * 获取当前去除项
   * */
  getSubItemArr = (params) => {
    const { selectedListKeys, list } = this.state;
    let sub = difference(selectedListKeys, params);
    return sub.map((item) => this.auxBuildItem(list[item]));
  };
  /**
   * 构建参数SelectedList减
   * -用于chooseItem
   * */
  buildSelectedListBySub = (origin, other) => {
    return differenceWith(origin, other, this.diffComparatorBySub);
  };
  /**
   * 移除选择按钮事件
   * */
  subChosenItem = () => {
    const { selectedList, selectedRowKeysTwo, list } = this.state;
    let subItemArr = cloneDeep(selectedRowKeysTwo.map((item) => selectedList[item]));
    let sList = this.buildSelectedListBySub(this.selectedStorage, subItemArr);
    this.selectedStorage = cloneDeep(sList);
    this.setState({ selectedList: sList });
    this.updateState({ selectedRowKeysTwo: [] });
    this.initSelectedByLoadList({ list });
  };

  render() {
    let { branchDict, typeDict, id } = this.props;
    let {
      loading,
      promotionName,
      startDate,
      endDate,
      activityStartDate,
      activityEndDate,
      publishNum,
      branchVos,
      promotionType,
      ruleUnit,
      contractCondition,
      customerCondition,
      getCondition,
      getDict = [],
      cusDict = [],
      conDict = [],
      explain,
      remark,
      currentIndex,
      stepTwoTabKey,
    } = this.state;

    let {
      conditions = [],
      totalRecords,
      expandedRowKeys,
      currentPage,
      pageSize,
      list,
      selectedList,
      selectedRowKeys = [],
      selectedRowKeysTwo = [],
      viewPage = 1,
      viewPageSize = 10,
      data,
      selectedListKeys,
      selectPage,
      selectPageSize,
    } = this.state;
    //表格
    let columns = [
      {
        title: '活动信息',
        dataIndex: 'info',
        width: COL_WIDTH_XL,
        render: (text, record, index) => {
          let { name, organizationName, courseSeriesName, classTypeName, categoryName, category } = record;
          return (
            <div className={styles.activityInfo} style={{ paddingLeft: 0 }}>
              <div>{name}</div>
              <div>
                <span>{organizationName}</span>
                <span>{courseSeriesName}</span>
                <span>{classTypeName}</span>
                <span>{categoryName}</span>
              </div>
              <span className={styles.tips}>
                <span></span>
                <span>{mapTips[category]}</span>
              </span>
            </div>
          );
        },
      },
      { title: '单价', dataIndex: 'price', width: COL_WIDTH_M },
      { title: '课时', dataIndex: 'miniClassTotalhours', width: COL_WIDTH_M },
      { title: '总价', dataIndex: 'totalAmount', width: COL_WIDTH_M },
      {
        title: '查看',
        dataIndex: 'view',
        width: COL_WIDTH_M,
        render: (text, record, index) => {
          let { infoVos } = record;
          infoVos = infoVos.filter((v, k) => {
            return (
              v.promotionLittleTypeId === 'PROMOTION_FULL_SALE' ||
              v.promotionLittleTypeId === 'PROMOTION_DISCOUNT' ||
              v.promotionLittleTypeId === 'PROMOTION_GIVE'
            );
          });
          return (
            <span onClick={(e) => this.onExpand(index, record)} className={styles.showPromote}>
              {infoVos.length > 0 ? '查看优惠' : ''}
            </span>
          );
        },
      },
    ];

    //第三步
    let viewColumns = [
      {
        title: '活动信息',
        dataIndex: 'info',
        render: (text, record, index) => {
          let { name, organizationName, courseSeriesName, classTypeName, categoryName, category } = record;
          return (
            <div className={styles.activityInfo}>
              <div>{name}</div>
              <div>
                <span>{organizationName}</span>
                <span>{courseSeriesName}</span>
                <span>{classTypeName}</span>
                <span>{categoryName}</span>
              </div>
              <span>
                <span></span>
                <span>{mapTips[category]}</span>
              </span>
            </div>
          );
        },
      },
      { title: '单价', dataIndex: 'price', width: COL_WIDTH_M },
      { title: '课时', dataIndex: 'miniClassTotalhours', width: COL_WIDTH_M },
      {
        title: '总价',
        dataIndex: 'totalAmount',
        width: COL_WIDTH_M,
        render: (text, record, index) => {
          return <span className={styles.totalPrice}>{record.totalAmount}</span>;
        },
      },
    ];
    return (
      <Page
        loading={loading}
        footer={renderFooter({
          currentIndex,
          handlePrev: (e) => {
            this.setState({ currentIndex: currentIndex - 1 });
          },
          handleNext: this.next,
          tabKey: stepTwoTabKey,
          handleBtn: this.subChosenItem,
        })}
        route={this.props.route}
        className={styles.activity}
        ref={(c) => (this.pageRef = c)}
      >
        <Steps currentIndex={currentIndex}>
          <div>活动信息</div>
          <div>选择产品</div>
          <div>核对完成</div>
        </Steps>
        {currentIndex === 0 && (
          <Block title={'优惠券信息'} className={styles.first}>
            <Row>
              <FormItem name="优惠券名称" required={true}>
                <Row>
                  <Col span={10}>
                    <Input
                      placeholder="请输入"
                      maxLength={50}
                      value={promotionName}
                      onChange={(e) => this.setState({ promotionName: e.target.value })}
                    />
                  </Col>
                </Row>
              </FormItem>
              <FormItem name="活动时间" required={true}>
                <Row>
                  <Col span={10}>
                    <DatePicker
                      style={{ width: '100%' }}
                      size="large"
                      showTime={{ defaultValue: moment('00:00:00', 'HH:mm:ss') }}
                      format="YYYY-MM-DD HH:mm:ss"
                      value={startDate}
                      onChange={(val) => this.setState({ startDate: val })}
                      disabledDate={(val) => this.getDisabledStartDate(val)}
                    />
                  </Col>
                  <Col span={4}>
                    <div style={{ textAlign: 'center' }}>至</div>
                  </Col>
                  <Col span={10}>
                    <DatePicker
                      style={{ width: '100%' }}
                      size="large"
                      showTime={{ defaultValue: moment('24:00:00', 'HH:mm:ss') }}
                      format="YYYY-MM-DD HH:mm:ss"
                      value={endDate}
                      onChange={(val) => this.setState({ endDate: val })}
                      disabledDate={(val) => this.getDisabledEndDate(val)}
                    />
                  </Col>
                </Row>
              </FormItem>
              <FormItem name="领取时间" required={true}>
                <Row>
                  <Col span={10}>
                    <DatePicker
                      style={{ width: '100%' }}
                      size="large"
                      showTime={{ defaultValue: moment('00:00:00', 'HH:mm:ss') }}
                      format="YYYY-MM-DD HH:mm:ss"
                      value={activityStartDate}
                      onChange={(val) => this.setState({ activityStartDate: val })}
                      disabledDate={(val) => this.getActivityDisabledStartDate(val)}
                    />
                  </Col>
                  <Col span={4}>
                    <div style={{ textAlign: 'center' }}>至</div>
                  </Col>
                  <Col span={10}>
                    <DatePicker
                      style={{ width: '100%' }}
                      size="large"
                      showTime={{ defaultValue: moment('24:00:00', 'HH:mm:ss') }}
                      format="YYYY-MM-DD HH:mm:ss"
                      value={activityEndDate}
                      onChange={(val) => this.setState({ activityEndDate: val })}
                      disabledDate={(val) => this.getActivityDisabledEndDate(val)}
                    />
                  </Col>
                </Row>
              </FormItem>
              <FormItem name="发券数量" required={true}>
                <Row>
                  <Col span={10}>
                    <InputNumber
                      value={publishNum}
                      min={1}
                      max={100000}
                      precision={0}
                      onChange={(val) => this.setState({ publishNum: val })}
                    />
                  </Col>
                </Row>
              </FormItem>
              <FormItem name="所属分公司" required={true}>
                <Row>
                  <Col span={10}>
                    <MultiSelect
                      dict={branchDict}
                      value={branchVos}
                      onChange={(val) => this.setState({ branchVos: val })}
                      disabled={id ? true : false}
                    />
                  </Col>
                </Row>
              </FormItem>
              <FormItem name="活动类型" required={true}>
                <Row>
                  <Col span={10} style={{ position: 'relative' }}>
                    <Select
                      dict={typeDict}
                      value={promotionType}
                      onChange={(val) => {
                        this.setState({ promotionType: val });
                      }}
                    />
                    <IconButton
                      type={'warning'}
                      size={20}
                      style={{ position: 'absolute', top: 8, right: '-10%', zIndex: 1 }}
                      title={{
                        title:
                          promotionType === 'COUPON_FULL_SALE_EVERY'
                            ? '只能设置一档，用户订单金额满足指定额度，便享受减价优惠，一个订单可重复多次享受优惠'
                            : '最少设置1档，最多可设置3档，满足条件递增设置。用户满足哪档额度，则享受哪一档的减价优惠，一个订单只能享受一次最高档优惠',
                        placement: 'right',
                      }}
                    />
                  </Col>
                </Row>
              </FormItem>
              <FormItem
                name="满减条件"
                required={true}
                style={{ width: '50%', background: '#f7f7f7', paddingBottom: 10 }}
              >
                <Row>
                  <Col span={24}>
                    {this.props.type === 'Voucher' && (
                      <Group
                        onChange={(e) => {
                          this.setState({ ruleUnit: e.target.value });
                        }}
                        value={ruleUnit}
                      >
                        <Radio value={'MONEY'}>金额</Radio>
                        <Radio value={'COURSE_HOURS'}>课时</Radio>
                      </Group>
                    )}
                    <div>{this.renderReduceCondition()}</div>
                  </Col>
                </Row>
              </FormItem>
              <FormItem name="领取限制" required={true}>
                <Row>
                  <Col span={10}>
                    <Group onChange={(e) => this.setState({ getCondition: e.target.value })} value={getCondition}>
                      {getDict.map((v, k) => {
                        return (
                          <Radio value={v.val} key={v.val}>
                            {v.text}
                          </Radio>
                        );
                      })}
                    </Group>
                  </Col>
                </Row>
              </FormItem>
              <FormItem name="客户条件" required={true}>
                <Row>
                  <Col span={10}>
                    <Group
                      onChange={(e) => this.setState({ customerCondition: e.target.value })}
                      value={customerCondition}
                    >
                      {cusDict.map((v, k) => {
                        return (
                          <Radio value={v.val} key={v.val}>
                            {v.text}
                          </Radio>
                        );
                      })}
                    </Group>
                  </Col>
                </Row>
              </FormItem>
              <FormItem name="合同条件" required={true}>
                <Row>
                  <Col span={10}>
                    <Group
                      onChange={(e) => this.setState({ contractCondition: e.target.value })}
                      value={contractCondition}
                    >
                      {conDict.map((v, k) => {
                        return (
                          <Radio value={v.val} key={v.val}>
                            {v.text}
                          </Radio>
                        );
                      })}
                    </Group>
                  </Col>
                </Row>
              </FormItem>
              <FormItem name="优惠券说明">
                <Row>
                  <Col span={10}>
                    <TextArea
                      rows={3}
                      value={explain}
                      maxLength={100}
                      onChange={(e) => this.setState({ explain: e.target.value })}
                    />
                  </Col>
                </Row>
              </FormItem>
              <FormItem name="备注">
                <Row>
                  <Col span={10}>
                    <TextArea
                      rows={3}
                      value={remark}
                      maxLength={100}
                      onChange={(e) => this.setState({ remark: e.target.value })}
                    />
                  </Col>
                </Row>
              </FormItem>
            </Row>
          </Block>
        )}
        {currentIndex === 1 && (
          <div style={{ position: 'relative', margin: '10px 50px' }} className={styles.second}>
            <Search
              conditions={conditions}
              selectedDatas={this.searchKeys}
              onInitFinish={() => {}}
              onSearch={this.onSearch}
              result={totalRecords}
              isShowResult={false}
              onInputChange={this.onInputChange}
            />

            <ChangeTabs
              ref={(e) => (this.$changeTabs = e)}
              count={totalRecords}
              selectCount={selectedList.length}
              DontShowList={this.DontShowList}
              handleKey={this.onSwitchTab}
            >
              <div>
                <CardList
                  expandedRowRender={(record) => this.expandedRowRender(record)}
                  dataSource={list}
                  rowSelection={{
                    selectedRowKeys: selectedListKeys,
                    onChange: this.chooseItem,
                  }}
                  pagination={{
                    current: currentPage,
                    total: totalRecords,
                    pageSize,
                  }}
                  className={styles.table}
                  expandedRowKeys={expandedRowKeys}
                  detailIndent={true}
                  columns={columns}
                  onChange={({ current, pageSize }) => this.getListByChangePage({ currentPage: current, pageSize })}
                />
              </div>
              <div>
                <CardList
                  expandedRowRender={(record) => this.expandedRowRender(record)}
                  dataSource={selectedList.slice((selectPage - 1) * selectPageSize, selectPage * selectPageSize)}
                  rowSelection={{
                    selectedRowKeys: selectedRowKeysTwo,
                    onChange: this.selectChangeTwo,
                  }}
                  pagination={{
                    current: selectPage,
                    total: selectedList.length,
                    pageSize: selectPageSize,
                  }}
                  className={styles.table}
                  expandedRowKeys={expandedRowKeys}
                  detailIndent={true}
                  columns={columns}
                  onChange={({ current, pageSize }) => this.setState({ selectPage: current, selectPageSize: pageSize })}
                />
              </div>
            </ChangeTabs>
          </div>
        )}
        {currentIndex === 2 && (
          <div style={{ margin: '10px 50px' }} className={styles.third}>
            <Block title="活动信息">
              <InfoBoard data={data} />
            </Block>
            <Block title="参与产品">
              <CardList
                expandedRowRender={(record) => this.expandedRowRender()}
                dataSource={selectedList.slice((viewPage - 1) * viewPageSize, viewPage * viewPageSize)}
                columns={viewColumns}
                loading={false}
                pagination={{
                  current: viewPage,
                  total: selectedList.length,
                  pageSize: viewPageSize,
                }}
                expandedRowKeys={[]}
                className={styles.viewTable}
                onChange={({ current, pageSize }) => this.setState({ viewPage: current, viewPageSize: pageSize })}
              />
            </Block>
          </div>
        )}
      </Page>
    );
  }
}

let mapStateToProps = (state) => {
  let data = state.main.pageParams;
  let pathName = state.routing.locationBeforeTransitions.pathname;
  if (data[pathName]) {
    return {
      type: data[pathName].type,
      handle: data[pathName].handle,
      id: data[pathName].id,
      branchDict: data[pathName].branchDict,
      typeDict: data[pathName].typeDict,
      onSaveSuccess: data[pathName].onSaveSuccess,
    };
  }
  return {};
};
export default connect(mapStateToProps)(Add);
