import React from 'react';
import PropTypes from 'prop-types';
import { connect } from 'dva';
import moment from 'moment';
import NP from 'number-precision';
import { Row, Col, DatePicker, Tooltip } from 'antd';
import {
  Page,
  Button,
  Form,
  Input,
  Select,
  Radio,
  TextArea,
  DropdownMenu,
  InputNumber,
  IconButton,
  TextLink,
  MultiSelect,
} from 'COMPONENT';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { getRemarkTemplates, savePromotion, getPromotionInfoById, getPromotionProductById } from 'SERVICE/preferential';
import { getBossOrganizationList } from 'SERVICE/system';
import { loginUser, runFunc, errorsHandle, isInputNumber } from 'UTIL/common';
import {
  timeFormat,
  rangeFormat,
  ssbossProductTypeDict,
  superpositionDict,
  periodValidDict,
  continuityDict,
  PERIOD_VALID_REGULAR,
  PERIOD_VALID_RELATIVE,
  LIMIT_SCOPE_FINITE,
  LIMIT_SCOPE_PRODUCT_TYPE,
  COUPON_FULL_SALE_NORMAL,
  COUPON,
  NONE,
  SUM_AUT,
  WIN_SPR,
  PRODUCT_NUMS,
  MONEY,
  BOSS,
  AUDIT_UNREVIEW,
  AUDIT_REVIEWED,
  AUDIT_REVIEWING,
  AUDIT_Fail,
  AUDIT_INVALID,
  quarterTextCfg,
  publishObjectDict,
  STUDENT,
  SHOW_SCOPE_OTHERS,
  showScopeDict,
} from '../config';
import ModalSelectPrd from './ModalSelectPrd';
import {
  createUid,
  submitDataAdapterExtension as submitDataAdapter,
  fieldDataAdapter,
  fieldRuleAdapter,
  isIncreaseArray,
} from './utils';
import Title from './components/Title';
import TipIcon from './components/TipIcon';

import style from './styles.module.less';

import {
  contractStateDict,
  ANOTHER_PRODUCT_NUMS,
  extendRuleTextCfg as ruleTextCfg,
  limitScopeDict,
  xhbossProductTypeDict,
  ProductTypeCate,
} from './const';

const FormItem = Form.Item;
const RadioGroup = Radio.Group;
const { RangePicker } = DatePicker;

const defaultBeginTime = '00:00:00';
const defaultEndTime = '23:59:59';

// 抵扣券产品类型
const DepositCouponType = ['班课', '一对一', '一对多'];

// 条件检查锁，保证检测一次
let checkLock = true;

/**
 * 单行优惠使用条件
 */
class RuleItem extends React.Component {
  constructor(props) {
    super(props);

    const [num = undefined, price = undefined, subPrice = undefined] = Array.isArray(props.value) ? props.value : [];

    this.state = {
      num,
      price,
      subPrice,
    };
  }

  static propTypes = {
    value: PropTypes.array,
    onChange: PropTypes.func,
    addDisabled: PropTypes.bool,
    delDisabled: PropTypes.bool,
    addVisible: PropTypes.bool,
    delVisible: PropTypes.bool,
    disabled: PropTypes.bool,
    rowId: PropTypes.string.isRequired,
    continuityType: PropTypes.oneOf([NONE, SUM_AUT, WIN_SPR]),
    ruleCfg: PropTypes.object,
    onAdd: PropTypes.func,
    onDel: PropTypes.func,
  };

  static defaultProps = {
    addDisabled: false,
    delDisabled: false,
    addVisible: true,
    delVisible: true,
    continuityType: NONE,
    disabled: false,
  };

  componentDidUpdate(prevProps, prevState) {
    this.shouldResetState(prevProps, prevState);
    this.shouldUpdateState(prevProps, prevState);
  }
  // 连报条件`continuityType`更新时需重置state
  shouldResetState = (prevProps) => {
    if (prevProps.continuityType !== this.props.continuityType) {
      this.setState({
        num: undefined,
        price: undefined,
        subPrice: undefined,
      });
    }
  };
  // props.value更新时需同步更新state
  shouldUpdateState = (prevProps) => {
    const [prevNum = undefined, prevPrice = undefined, preSubPrice = undefined] = prevProps.value || [];
    const [curNum = undefined, curPrice = undefined, curSubPrice = undefined] = this.props.value || [];

    if (prevNum !== curNum || prevPrice !== curPrice || preSubPrice !== curSubPrice) {
      this.setState({
        num: curNum,
        price: curPrice,
        subPrice: curSubPrice,
      });
    }
  };

  finishChange = () => {
    const { num, price, subPrice } = this.state;

    runFunc(this.props.onChange, [
      isInputNumber(num) ? num : undefined,
      isInputNumber(price) ? price : undefined,
      isInputNumber(subPrice) ? subPrice : undefined,
    ]);
  };

  handleNumChange = (val) => {
    this.setState({ num: val }, this.finishChange);
  };

  handlePriceChange = (val) => {
    this.setState({ price: val }, this.finishChange);
  };

  handleSubPriceChange = (val) => {
    this.setState({ subPrice: val }, this.finishChange);
  };

  handleAdd = () => {
    runFunc(this.props.onAdd);
  };

  handleDel = () => {
    runFunc(this.props.onDel, this.props.rowId);
  };

  getQuarterText = (continuityType, index) => {
    const quarterTextArr = continuityType.split('_');

    return quarterTextCfg[quarterTextArr[index]];
  };

  render() {
    const { disabled, ruleCfg, continuityType } = this.props;
    const isContinue = continuityType !== NONE;

    return (
      <div className={style.ruleItem}>
        <span>{ruleTextCfg[ruleCfg.type]}</span>
        <InputNumber
          value={this.state.num}
          onChange={this.handleNumChange}
          min={disabled ? 0 : 1} // 编辑回填会小于1
          precision={0}
          disabled={disabled}
        />
        <span>{ruleTextCfg[ruleCfg.ruleUnit]}</span>
        <span>，{isContinue && this.getQuarterText(continuityType, 0)}优惠</span>
        <InputNumber
          value={this.state.price}
          onChange={this.handlePriceChange}
          min={0}
          precision={2}
          disabled={disabled}
        />
        <span>{ruleTextCfg[ruleCfg.unit]}</span>
        {isContinue &&
          this.props.type === 0 && [
            <span key={0}>，{this.getQuarterText(continuityType, 1)}优惠</span>,
            <InputNumber
              key={1}
              value={this.state.subPrice}
              onChange={this.handleSubPriceChange}
              min={0}
              precision={2}
              disabled={disabled}
            />,
            <span key={2}> 元</span>,
          ]}
        <span>，合同使用时抵扣{this.props.discount}元</span>
        {this.props.addVisible && (
          <IconButton
            type="plus_circle"
            size={20}
            className={style.ruleAction}
            onClick={this.handleAdd}
            disabled={this.props.addDisabled}
          />
        )}
        {this.props.delVisible && (
          <IconButton
            type="minus_circle"
            size={20}
            className={style.ruleAction}
            onClick={this.handleDel}
            disabled={this.props.delDisabled}
          />
        )}
        <TipIcon left="100%" title="最高抵扣金额=购买金额+优惠金额" />
      </div>
    );
  }
}

/**
 * 优惠使用条件
 */
const CouponCondition = (props) => {
  const {
    disabled,
    continuityType,
    ruleList,
    rulesValidator,
    onAdd,
    onDel,
    type = 0,
    price = 0,
    discountAmount,
    ...restFormProps
  } = props;

  const addVisible = !disabled;
  const delVisible = !disabled;

  const displayRuleList = ruleList.length > 0 ? ruleList : [createUid()];

  const isMulti = displayRuleList.length > 1;
  const lastIndex = displayRuleList.length - 1;

  // 1对1 1对多 单位为课时
  const ruleCfg =
    type === 0
      ? {
          type: COUPON_FULL_SALE_NORMAL,
          ruleUnit: PRODUCT_NUMS,
          unit: MONEY,
        }
      : {
          type: COUPON_FULL_SALE_NORMAL,
          ruleUnit: ANOTHER_PRODUCT_NUMS,
          unit: MONEY,
        };

  return (
    <div>
      {displayRuleList.map((rowId, index) => {
        const isFirst = index === 0;

        return (
          <FormItem
            key={rowId}
            name={`rules_${rowId}`}
            fieldDecoratorOpt={{
              rules: [
                {
                  required: true,
                  validator: rulesValidator,
                },
              ],
            }}
            formItemProps={{
              className: isFirst ? '' : style.formItemNotRequired,
              label: isFirst ? '使用条件' : ' ',
              colon: false,
              labelCol: { span: 4 },
              wrapperCol: { span: 20 },
            }}
            {...restFormProps}
          >
            <RuleItem
              rowId={rowId}
              addDisabled={isMulti && index !== lastIndex}
              addVisible={addVisible}
              delVisible={delVisible && isMulti}
              disabled={disabled}
              continuityType={continuityType}
              ruleCfg={ruleCfg}
              onAdd={onAdd}
              onDel={onDel}
              type={type}
              discount={discountAmount[index]}
            />
          </FormItem>
        );
      })}
    </div>
  );
};

/**
 * 购买金额输入行
 */
class PriceInputField extends React.Component {
  constructor(props) {
    super(props);
  }

  static propTypes = {
    value: PropTypes.oneOfType([PropTypes.number, PropTypes.string]),
    onChange: PropTypes.func,
    disabled: PropTypes.bool,
  };

  render() {
    const { value, onChange, disabled = false } = this.props;

    return (
      <div className={style.priceRow}>
        <InputNumber
          className={style.priceInputNum}
          min={0.01}
          precision={2}
          disabled={disabled}
          value={value}
          onChange={onChange}
        />
      </div>
    );
  }
}

const ROLE_TYPE_GROUP = 'GROUP';
const ROLE_TYPE_BRANCH = 'BRANCH';
const ROLE_TYPE_CAMPUS = 'CAMPUS';

class AddEdit extends React.Component {
  constructor(props) {
    super(props);

    this.getRoleType = this.getRoleType.bind(this);
    this.isFieldDisabled = this.isFieldDisabled.bind(this);
    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.loadData = this.loadData.bind(this);
    this.loadTemplateMenus = this.loadTemplateMenus.bind(this);
    this.renderTemplateMenu = this.renderTemplateMenu.bind(this);
    this.createComment = this.createComment.bind(this);
    this.updatePromotionComment = this.updatePromotionComment.bind(this);

    this.renderLimitScopeContent = this.renderLimitScopeContent.bind(this);
    this.handleProdTypesChecked = this.handleProdTypesChecked.bind(this);

    this.switchModalSelectPrd = this.switchModalSelectPrd.bind(this);
    this.handleProductChange = this.handleProductChange.bind(this);

    this.addRule = this.addRule.bind(this);
    this.delRule = this.delRule.bind(this);
    this.getDiscountAmount = this.getDiscountAmount.bind(this);

    this.promotionNameValidator = this.promotionNameValidator.bind(this);
    this.limitScopeValidator = this.limitScopeValidator.bind(this);
    this.buyingDateRangeValidator = this.buyingDateRangeValidator.bind(this);
    this.validDateRangeValidator = this.validDateRangeValidator.bind(this);
    this.rulesValidator = this.rulesValidator.bind(this);

    this.isContinueRule = this.isContinueRule.bind(this);
    this.handleFieldsChange = this.handleFieldsChange.bind(this);
    this.submit = this.submit.bind(this);
    this.submitFail = this.submitFail.bind(this);
    this.goBackRefresh = this.goBackRefresh.bind(this);

    this.handledSave = this.handledSave.bind(this);

    // 表单引用
    this.formRef = null;

    const productTypeDict = loginUser.isXingHuoSym() ? xhbossProductTypeDict : ssbossProductTypeDict;
    const productTypeOptions = productTypeDict.map((el) => ({
      value: el.val,
      label: el.text,
    }));
    const productTypeList = productTypeDict.map((el) => el.val);

    const params = props.pageParams[`/${props.route.path}`];

    const roleType = this.getRoleType({
      isGroupRole: loginUser.isGroupRole(),
      isBranchRole: loginUser.isBranchRole(),
      isCampusRole: loginUser.isCampusRole(),
    });

    /**
     * @description type 抵扣券类型 小班 0 一对一 1 一对多 2
     */
    this.type = params.type || 0;

    this.state = {
      loading: false,
      mounted: false,
      platType: BOSS,
      isBossPlat: true,
      roleType,
      // 券状态
      auditStatus: params.auditStatus || AUDIT_UNREVIEW,
      // 抵扣券模板 id
      couponId: params.couponId || null,
      // 购买对象
      publishObject: STUDENT,
      // 合同状态
      useCase: contractStateDict[0].val,
      // 产品类型 checkbox 数据源
      productTypeOptions,
      // 产品类型列表 string[]
      productTypeList,
      // 从服务端同步的抵扣券详情
      detailStorage: {},
      // 分公司 ID
      branchId: null,
      // 产品范围
      limitScope: LIMIT_SCOPE_PRODUCT_TYPE,
      // 需要限制的产品类型
      limitProductTypes: productTypeList.slice(0, 1),
      // 需要限制的产品列表
      prdList: [],
      // 需要限制的产品列表数量
      total: 0,
      // 有效期类型，默认固定有效期
      periodType: periodValidDict[0].val,
      // 连报条件
      continuityType: NONE,
      // 使用条件
      ruleList: [createUid()],
      discountAmount: [0],
      // 使用说明历史
      templateMenus: [{ text: '暂无使用说明历史', disabled: true }],
      modalVisible: false,
      modalInitSKey: {
        category: ProductTypeCate[this.type],
      },
      // 售价
      price: void 0,
      // 所属学习中心列表
      campusList: [],
    };
  }

  componentDidMount() {
    this.init();
  }

  /**
   * 获取角色权限类型
   * @param {Object} payload
   */
  getRoleType(payload) {
    const { isGroupRole, isBranchRole, isCampusRole } = payload;

    if (isGroupRole) {
      return ROLE_TYPE_GROUP;
    } else if (isBranchRole) {
      return ROLE_TYPE_BRANCH;
    } else if (isCampusRole) {
      return ROLE_TYPE_CAMPUS;
    } else {
      return ROLE_TYPE_CAMPUS;
    }
  }

  /**
   * 判断表单域是否禁用
   * @param {string} fieldName
   */
  isFieldDisabled(fieldName) {
    if (fieldName === 'campusIds' || !this.isEditing()) {
      return false;
    }

    const { auditStatus, roleType } = this.state;

    const statusTable = [
      {
        auditStatusList: [AUDIT_UNREVIEW, AUDIT_Fail],
        roleTypes: [ROLE_TYPE_GROUP, ROLE_TYPE_BRANCH],
        enabledFields: [
          'campusIds',
          'promotionName',
          'branchId',
          'limitScope',
          'superPosition',
          'receiveNumLimit',
          'buyingDateRange',
          'periodType',
          'validDateRange',
          'relativePeriod',
          'continuityType',
          'useRules',
          'useCase',
          'price',
          'promotionComment',
          'publishObject',
          'showScope',
        ],
      },
      {
        auditStatusList: [AUDIT_REVIEWING, AUDIT_INVALID],
        roleTypes: [ROLE_TYPE_GROUP, ROLE_TYPE_BRANCH],
        enabledFields: [],
      },
      {
        auditStatusList: [AUDIT_REVIEWED],
        roleTypes: [ROLE_TYPE_GROUP, ROLE_TYPE_BRANCH],
        enabledFields: ['buyingDateRange', 'promotionComment'],
      },
      {
        auditStatusList: [AUDIT_UNREVIEW, AUDIT_REVIEWING, AUDIT_REVIEWED, AUDIT_Fail, AUDIT_INVALID],
        roleTypes: [ROLE_TYPE_CAMPUS],
        enabledFields: [],
      },
    ];

    const [{ enabledFields = [] } = {}] = statusTable.filter((config) => {
      const { auditStatusList, roleTypes } = config;

      return auditStatusList.indexOf(auditStatus) > -1 && roleTypes.indexOf(roleType) > -1;
    });

    return enabledFields.indexOf(fieldName) === -1;
  }

  /**
   * 初始化数据
   * 包括字典、使用说明历史记录、抵扣券数据
   */
  async init() {
    this.setState({
      loading: true,
      mounted: false,
    });

    await new Promise(this.initDict);
    await new Promise(this.loadTemplateMenus);

    if (this.isEditing()) {
      await new Promise(this.loadData);
    } else {
      this.initBranchId();
    }

    this.setState({
      loading: false,
      mounted: true,
    });
  }

  isEditing() {
    return !!this.state.couponId;
  }

  /**
   * 初始化字典
   * @param {Function} resolve promise resolve
   */
  initDict(resolve) {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['branch'],
        callback() {
          resolve();
        },
      },
    });
  }

  /**
   * 初始化分公司表单项
   */
  initBranchId() {
    if (this.isEditing()) {
      return;
    }
    const { branchDict } = this.props;
    if (branchDict.length === 1) {
      const branchId = branchDict[0].val;
      this.setState({ branchId }, () => {
        this.loadBossOrganizationList(branchId);
        this.formRef.setFieldsValue({ branchId });
      });
    }
  }

  /**
   * 加载使用说明历史记录
   * @param {Function} resolve
   */
  async loadTemplateMenus(resolve) {
    const rsp = await getRemarkTemplates(
      {
        promotionKindId: COUPON,
        promotionLittleTypeId: 'COUPON_FULL_SALE',
        rows: 3,
      },
      true,
    ).catch(errorsHandle());

    if (rsp && rsp.resultCode === 0 && Array.isArray(rsp.data) && rsp.data.length > 0) {
      this.setState({
        templateMenus: rsp.data.map((item) => ({ text: item })),
      });
    }
    resolve();
  }

  /**
   * 编辑场景下加载优惠券详情数据
   * 给表单赋值
   * @param {Function} resolve promise resolve
   */
  async loadData(resolve) {
    const { couponId } = this.state;

    const fetchList = [
      getPromotionInfoById({ id: couponId }, true),
      getPromotionProductById({ promotionInfoId: couponId }, true),
    ];
    const [rspDetail, rspProduct] = await Promise.all(fetchList, true).catch(errorsHandle());
    const payload = {};

    if (rspProduct && rspProduct.data) {
      payload.prdList = rspProduct.data || [];
      payload.total = payload.prdList.length;
    }

    let formFieldModel = null;

    if (rspDetail && rspDetail.data) {
      const { ruleVos = [], ...restDetailData } = rspDetail.data;
      const proRuleVos = ruleVos.map((rule) => {
        const ruleRowId = createUid();

        return {
          ...rule,
          ruleRowId,
        };
      });

      const rulesModel = fieldRuleAdapter(proRuleVos, this.type);
      const otherModel = fieldDataAdapter(restDetailData);
      const combinedModel = { ...rulesModel, ...otherModel };

      formFieldModel = Object.keys(combinedModel).reduce((prev, key) => {
        const value = combinedModel[key];

        if (value !== null && value !== undefined) {
          prev[key] = value;
        }

        return prev;
      }, {});

      // 优惠券详情冗余
      payload.detailStorage = {
        ...restDetailData,
        ruleVos: proRuleVos,
      };

      // 分公司 id
      payload.branchId = formFieldModel.branchId;

      // 购买渠道：默认星火教育, 兼容旧数据
      payload.showScope = formFieldModel.showScope || SHOW_SCOPE_OTHERS;

      // 产品范围
      payload.limitScope = formFieldModel.limitScope;

      // 限制产品类型
      payload.limitProductTypes = Array.isArray(restDetailData.limitProductTypes)
        ? restDetailData.limitProductTypes.filter((el) => this.state.productTypeList.indexOf(el) > -1)
        : this.state.productTypeList.slice(0, 1);

      // 有效期方式
      payload.periodType = formFieldModel.periodType;

      // 连报条件
      payload.continuityType = formFieldModel.continuityType;

      // 使用条件规则
      payload.ruleList = proRuleVos.map((rule) => rule.ruleRowId);

      // 抵扣金额
      const goodsVo = restDetailData.goodsVo || {};

      // 售价
      payload.price = formFieldModel.price;

      // 合同
      payload.useCase = formFieldModel.useCase;

      // 更新discountAmount
      const discountList = payload.ruleList
        .map((ruleName) => rulesModel[`rules_${ruleName}`])
        .map((rule) => rule.slice(1));
      payload.discountAmount = discountList.map((rule) =>
        rule.reduce((pre, curr) => NP.plus(pre, curr), goodsVo.price || 0),
      );

      // 所属学习中心
      payload.campusIds = formFieldModel.campusIds;
    }

    this.setState(payload, () => {
      if (formFieldModel) {
        this.loadBossOrganizationList(formFieldModel.branchId, false);
        const values = {
          ...formFieldModel,
        };
        this.formRef.setFieldsValue(values);
      }
      resolve();
    });
  }

  createComment() {
    const ruleNames = this.state.ruleList.map((ruleRowId) => `rules_${ruleRowId}`);

    const ruleCfg =
      this.type === 0
        ? {
            type: COUPON_FULL_SALE_NORMAL,
            ruleUnit: PRODUCT_NUMS,
            unit: MONEY,
          }
        : {
            type: COUPON_FULL_SALE_NORMAL,
            ruleUnit: ANOTHER_PRODUCT_NUMS,
            unit: MONEY,
          };

    if (ruleNames.length) {
      const rulesModel = this.formRef.getFieldsValue(ruleNames);
      const ruleModelList = ruleNames.map((name) => (rulesModel[name] ? rulesModel[name] : []));
      const texts = ruleModelList
        .map((rule, index) => {
          const [num] = rule;
          const amount = this.state.discountAmount[index];
          if (num > 0 && amount > 0) {
            return `${ruleTextCfg[ruleCfg.type]}${num}${ruleTextCfg[ruleCfg.ruleUnit]}，合同使用时抵扣${amount}元`;
          }
          return '';
        })
        .filter((r) => !!r)
        .join('；');
      if (texts) {
        this.formRef.setFieldsValue({ promotionComment: texts });
      }
    }
  }

  /**
   * 渲染说明模板菜单
   */
  renderTemplateMenu() {
    const { templateMenus } = this.state;

    return templateMenus.map((item) => {
      return {
        ...item,
        onClick: this.updatePromotionComment,
      };
    });
  }

  /**
   * 更新使用说明
   * 组件库 menu 组件用 key 做索引，很不严谨
   * @param {Object} param0
   * @param {number} key 说明模板索引
   */
  updatePromotionComment({ key }) {
    if (key) {
      const promotionComment = this.state.templateMenus[key].text || null;
      this.formRef.setFieldsValue({ promotionComment });
    }
  }

  /**
   * 渲染产品范围 Radio 内容
   * @param {string} limitScope 产品范围类型
   * @param {boolean} disabled 是否禁用
   */
  renderLimitScopeContent(limitScope, disabled) {
    const limitScopeContentMap = {
      [LIMIT_SCOPE_PRODUCT_TYPE]: {
        val: LIMIT_SCOPE_PRODUCT_TYPE,
        render: () => null,
      },
      [LIMIT_SCOPE_FINITE]: {
        val: LIMIT_SCOPE_FINITE,
        render: (state, disabled) => {
          return (
            <div className={style.checkboxContent}>
              <span style={{ marginRight: 16 }}>
                已选
                <span className={style.prdNum}>{state.total}</span>
                个产品
              </span>
              {state.branchId ? (
                <TextLink onClick={this.switchModalSelectPrd} disabled={disabled}>
                  选择产品
                </TextLink>
              ) : (
                <Tooltip title="请先选择分公司" placement="top">
                  <TextLink disabled>选择产品</TextLink>
                </Tooltip>
              )}
            </div>
          );
        },
      },
    };

    const { render } = limitScopeContentMap[limitScope];

    return render(this.state, disabled);
  }

  /**
   * 处理限制产品类型选中变化
   * @param {string[]} checkedValue 产品类型
   */
  handleProdTypesChecked(checkedValue) {
    this.setState(
      {
        limitProductTypes: [...checkedValue],
      },
      () => {
        // 限制产品类型更新时，触发`limitScope`校验
        this.formRef.validateFields(['limitScope'], { force: true });
      },
    );
  }

  /**
   * 添加一行使用条件
   */
  addRule() {
    this.setState((prevState) => {
      const { ruleList, discountAmount } = prevState;

      return {
        ruleList: [...ruleList, createUid()],
        discountAmount: [...discountAmount, 0],
      };
    });
  }

  /**
   * 删除某行使用条件
   * @param {string} rowId 要删除的 rule 行 id
   */
  delRule(rowId) {
    this.setState(
      (prevState) => {
        const { ruleList, discountAmount } = prevState;

        return {
          ruleList: ruleList.filter((ruleRowId) => ruleRowId !== rowId),
          discountAmount: discountAmount.slice(0, -1),
        };
      },
      () => {
        const ruleNames = this.state.ruleList.map((ruleRowId) => `rules_${ruleRowId}`);
        this.formRef.setFieldsValue({ promotionComment: '' });
        this.formRef.validateFields(ruleNames, { force: true });
      },
    );
  }

  // 抵扣券名称校验
  promotionNameValidator(rule, value, callback) {
    if (!value || value.length === 0) {
      return callback('抵扣券名称不能为空');
    }
    if (value.length > 100) {
      return callback('抵扣券名称字数不能多于100个字');
    }
    callback();
  }

  // 校验产品范围
  limitScopeValidator(rule, value, callback) {
    if (value === LIMIT_SCOPE_FINITE && this.state.total === 0) {
      return callback('限制商品数量不能为0');
    }
    callback();
  }

  /**
   * 购买日期校验
   * @param {Object} rule
   * @param {Array<moment>} value
   * @param {Function} callback
   */
  buyingDateRangeValidator(rule, value, callback) {
    if (!Array.isArray(value) || value.length === 0) {
      return callback('购买时间不能为空');
    }

    const validDateRange = this.formRef.getFieldValue('validDateRange');

    if (!Array.isArray(validDateRange) || validDateRange.length === 0) {
      return callback();
    }

    const [, validEndDate] = validDateRange;
    const [, buyingEndDate] = value;

    if (buyingEndDate.valueOf() > validEndDate.valueOf()) {
      return callback('买券结束时间不能晚于有效期结束时间');
    }

    callback();
  }

  /**
   * 有效期校验
   * @param {Object} rule
   * @param {Array} value
   * @param {Function} callback
   */
  validDateRangeValidator(rule, value, callback) {
    if (this.state.periodType !== PERIOD_VALID_REGULAR) {
      return callback();
    }

    if (!Array.isArray(value) || value.length === 0) {
      return callback('有效期不能为空');
    }

    const buyingDateRange = this.formRef.getFieldValue('buyingDateRange');

    if (!Array.isArray(buyingDateRange) || buyingDateRange.length === 0) {
      return callback();
    }

    const [, validEndDate] = value;
    const [, buyingEndDate] = buyingDateRange;

    if (validEndDate.valueOf() < buyingEndDate.valueOf()) {
      return callback('有效期结束时间须晚于买券结束时间');
    }

    callback();
  }

  /**
   * 券使用条件校验
   * @param {Object} rule
   * @param {Array} value
   * @param {Function} callback
   */
  rulesValidator(rule, value, callback) {
    if (!Array.isArray(value)) {
      return callback('使用条件不能为空');
    }
    if (this.isContinueRule()) {
      if (value.some((val) => isNaN(val))) {
        return callback('使用条件不能为空');
      } else {
        let [num] = value;
        if (num < 2) {
          return callback('连报时报读的班数不可少于2个');
        }
      }
    }
    if (value.slice(0, 2).some((val) => isNaN(val))) {
      return callback('使用条件不能为空');
    }

    // 阶梯验证
    const ruleNames = this.state.ruleList.map((ruleRowId) => `rules_${ruleRowId}`);

    if (ruleNames.length > 1) {
      const rulesModel = this.formRef.getFieldsValue(ruleNames);
      const ruleModelList = ruleNames.map((name) => (rulesModel[name] ? rulesModel[name] : []));
      const Nums = ruleModelList.map(([num = 0]) => num);
      if (Nums.every((r) => !!r)) {
        if (!isIncreaseArray(Nums)) {
          checkLock = false;
          return callback('如有阶梯，数量需要阶梯递增');
        } else if (!checkLock) {
          // 继续检查其它的且检查一次
          const { feild } = rule;
          const others = ruleNames.filter((rule) => rule !== feild);
          checkLock = true;
          this.formRef.validateFields(others, { force: true });
        }
      }
    }

    callback();
  }

  isContinueRule() {
    return this.state.continuityType !== NONE;
  }

  handleFieldsChange(fields) {
    if (!this.state.mounted) {
      return;
    }

    // 所属分公司✔
    if (fields.branchId && fields.branchId.value !== this.state.branchId) {
      const payload = {
        branchId: fields.branchId.value,
      };

      if (this.state.branchId !== null) {
        payload.prdList = [];
        payload.total = 0;
      }

      this.setState(payload);
    }

    // 产品范围✔
    if (fields.limitScope && fields.limitScope.value !== this.state.limitScope) {
      const payload = {
        limitScope: fields.limitScope.value,
      };

      if (fields.limitScope.value === LIMIT_SCOPE_PRODUCT_TYPE) {
        payload.prdList = [];
        payload.total = 0;
      }

      this.setState(payload);
    }

    // 有效期✔
    if (fields.periodType && fields.periodType.value !== this.state.periodType) {
      this.setState(
        {
          periodType: fields.periodType.value,
        },
        () => {
          const payload =
            fields.periodType.value === PERIOD_VALID_REGULAR ? { relativePeriod: null } : { validDateRange: null };
          this.formRef.setFieldsValue(payload);
        },
      );
    }

    // 连报条件✔
    if (fields.continuityType && fields.continuityType.value !== this.state.continuityType) {
      this.setState(
        {
          continuityType: fields.continuityType.value,
          ruleList: [createUid()],
        },
        () => {
          const discounts = this.getDiscountAmount();

          if (discounts.length !== this.state.discountAmount.length) {
            this.setState({
              discountAmount: discounts,
            });
          } else {
            const isChange = this.state.discountAmount.some((item, index) => discounts[index] !== item);
            if (isChange) {
              this.setState({
                discountAmount: discounts,
              });
            }
          }
        },
      );
    }

    // 使用条件✔
    const isRuleModelChange = this.state.ruleList.some((ruleRowId) => {
      const ruleFormItemName = `rules_${ruleRowId}`;
      const ruleModel = fields[ruleFormItemName];

      return ruleModel;
    });

    if (isRuleModelChange) {
      const discountRs = this.getDiscountAmount();

      if (discountRs.length !== this.state.discountAmount.length) {
        this.setState({
          discountAmount: discountRs,
        });
      } else {
        const isChange = this.state.discountAmount.some((item, index) => discountRs[index] !== item);
        if (isChange) {
          this.setState({
            discountAmount: discountRs,
          });
        }
      }
    }

    // 购买金额✔
    if (fields.price) {
      const discountPs = this.getDiscountAmount();

      if (discountPs.length !== this.state.discountAmount.length) {
        this.setState({
          discountAmount: discountPs,
        });
      } else {
        const isChange = this.state.discountAmount.some((item, index) => discountPs[index] !== item);
        if (isChange) {
          this.setState({
            discountAmount: discountPs,
          });
        }
      }
    }
  }

  /**
   * 渲染抵扣金额
   */
  getDiscountAmount() {
    const ruleNames = this.state.ruleList.map((ruleRowId) => `rules_${ruleRowId}`);
    const rulesModel = this.formRef.getFieldsValue(ruleNames);

    const ruleModelList = ruleNames.map((name) => (rulesModel[name] ? rulesModel[name] : []));

    const price = this.formRef.getFieldValue('price') || 0;

    const ruleDiscountList = ruleModelList.map((model) => {
      const [, discountA = 0, discountB = 0] = model;

      return NP.plus(discountA || 0, discountB || 0, price);
    });

    return ruleDiscountList;
  }

  /**
   * 切换产品选择弹层
   */
  switchModalSelectPrd() {
    this.setState((prevState) => {
      const { modalInitSKey } = this.state;

      if (!prevState.modalVisible) {
        modalInitSKey.organizationId = prevState.branchId;
      }

      return {
        modalVisible: !prevState.modalVisible,
        modalInitSKey,
      };
    });
  }

  /**
   * 处理已选产品变化
   * @param {Array} list
   */
  handleProductChange(list) {
    this.setState(
      {
        prdList: list,
        total: list.length,
      },
      () => {
        // 已选产品更新时，触发`limitScope`校验
        this.formRef.validateFields(['limitScope'], { force: true });
      },
    );
    this.switchModalSelectPrd();
  }

  handleSelectBranchId = (branchId) => {
    // 先清空已选值
    this.formRef.setFieldsValue({
      campusIds: undefined,
    });
    if (branchId) {
      this.loadBossOrganizationList(branchId);
    } else {
      this.setState({ campusList: [] });
    }
  };

  async loadBossOrganizationList(parentId, isSelectedAll = true) {
    const result = await getBossOrganizationList({ parentId, type: 'BOSS' }).catch(errorsHandle());
    if (!result || result.resultCode !== 0 || !result.data.length) {
      this.setState({ campusList: [] });
    } else {
      const campusList = result.data
        .filter(({ status }) => status !== 1)
        .map(({ id, name }) => ({
          val: id,
          text: name,
        }));
      this.setState({ campusList }, () => {
        isSelectedAll &&
          campusList.length &&
          this.formRef.setFieldsValue({
            campusIds: campusList.map(({ val }) => val),
          });
      });
    }
  }

  /**
   * 表单提交 handler
   * @param {Object} model
   */
  async submit(model) {
    this.setState({ loading: true });

    const { couponId, prdList, detailStorage = {} } = this.state;

    // 限制产品类型时传 limitProductTypes
    const limitProdTypesObj =
      model.limitScope === LIMIT_SCOPE_PRODUCT_TYPE
        ? {
            limitProductTypes: [xhbossProductTypeDict[this.type].val],
          }
        : {};

    const apiParams = submitDataAdapter(
      {
        id: couponId,
        productVos: prdList,
        ...limitProdTypesObj,
        ...model,
        type: this.type,
      },
      detailStorage,
    );

    const rsp = await savePromotion(apiParams, true).catch(errorsHandle());

    this.setState({ loading: false });

    if (rsp && rsp.resultCode === 0) {
      showMessage(`${apiParams.id ? '修改' : '新增'}抵扣券成功！`, 'success');

      return this.goBackRefresh();
    }
  }

  /**
   * 响应表单提交失败
   * @param {Error} err
   */
  submitFail() {
    showMessage('请完善好所有信息后再保存！', 'error');
  }

  /**
   * 返回抵扣券列表页并刷新
   */
  goBackRefresh() {
    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path: '/deductionCoupon',
        refresh: true,
      },
    });
  }

  /**
   * 保存抵扣券修改结果
   */
  handledSave() {
    runFunc(this.formRef.submitFields);
  }

  render() {
    // 是否固定有效期
    const isFixedPeriod = this.state.periodType === PERIOD_VALID_REGULAR;
    // 是否相对有效期
    const isRelativePeriod = this.state.periodType === PERIOD_VALID_RELATIVE;

    const { branchDict, route } = this.props;

    const extendRoute = { ...route, name: `${route.name}-${DepositCouponType[this.type]}` };

    return (
      <Page
        loading={this.state.loading}
        route={extendRoute}
        footer={
          <div className={style.pageFooter}>
            <Button onClick={this.handledSave} type="primary">
              保存
            </Button>
          </div>
        }
        fitWindowHeight={false}
      >
        <div className={style.formWrap}>
          <Form
            ref={(c) => (this.formRef = c)}
            onSubmit={this.submit}
            onSubmitFail={this.submitFail}
            onFieldsChange={this.handleFieldsChange}
          >
            <Title>基础属性</Title>

            <FormItem
              name="promotionName"
              rules={[{ required: true, validator: this.promotionNameValidator }]}
              formItemProps={{
                label: '抵扣券名称',
                colon: false,
                labelCol: { span: 4 },
                wrapperCol: { span: 12 },
              }}
            >
              <Input placeholder="抵扣券名称" disabled={this.isFieldDisabled('promotionName')} />
            </FormItem>
            <FormItem
              name="branchId"
              rules={[{ required: true, message: '所属分公司不能为空' }]}
              formItemProps={{
                label: '所属分公司',
                colon: false,
                labelCol: { span: 4 },
                wrapperCol: { span: 12 },
              }}
            >
              <Select
                placeholder="所属分公司"
                dict={branchDict}
                showSearch
                allowClear={false}
                onChange={this.handleSelectBranchId}
                disabled={this.isFieldDisabled('branchId')}
              />
            </FormItem>
            <FormItem
              name="campusIds"
              rules={[{ required: true, message: '所属学习中心不能为空' }]}
              formItemProps={{
                label: '所属学习中心',
                colon: false,
                labelCol: { span: 4 },
                wrapperCol: { span: 12 },
              }}
            >
              <MultiSelect
                showSearch
                allowClear={true}
                placeholder="请选择学习中心(多选)"
                dict={this.state.campusList}
                disabled={this.isFieldDisabled('campusIds')}
              />
            </FormItem>

            <Title>购券规则</Title>

            <FormItem
              name="buyingDateRange"
              rules={[{ required: true, validator: this.buyingDateRangeValidator }]}
              formItemProps={{
                label: '购买时间',
                colon: false,
                labelCol: { span: 4 },
                wrapperCol: { span: 12 },
              }}
              slot={<TipIcon left={618} title="开放给用户购买的时间窗口" />}
            >
              <RangePicker
                placeholder={['开始时间', '结束时间']}
                showTime={{
                  hideDisabledOptions: true,
                  defaultValue: [moment(defaultBeginTime, timeFormat), moment(defaultEndTime, timeFormat)],
                }}
                format={rangeFormat}
                size="large"
                disabled={this.isFieldDisabled('buyingDateRange')}
              />
            </FormItem>

            <FormItem
              name="publishObject"
              rules={[{ required: true, message: '购买身份不能为空' }]}
              initialValue={STUDENT}
              formItemProps={{
                label: '购买身份',
                colon: false,
                labelCol: { span: 4 },
                wrapperCol: { span: 20 },
              }}
            >
              <RadioGroup disabled={this.isFieldDisabled('publishObject')}>
                {/** 目前版本仅对学生购买开放 */}
                {publishObjectDict
                  .filter((item) => item.val === STUDENT)
                  .map((item) => (
                    <Radio value={item.val} key={item.val}>
                      {item.text}
                    </Radio>
                  ))}
              </RadioGroup>
            </FormItem>

            <FormItem
              name="receiveNumLimit"
              wrapClassName={style.receiveNumLimitWrapper}
              rules={[{ required: true, message: '可购买张数不能为空' }]}
              initialValue={1}
              formItemProps={{
                label: '可购买张数',
                colon: false,
                labelCol: { span: 4 },
                wrapperCol: { span: 20 },
              }}
              slot={
                <TipIcon
                  left={240}
                  title={`每名${this.state.publishObject === STUDENT ? '学生' : '家长'}可购买的张数`}
                />
              }
            >
              <InputNumber
                className={style.receiveNumLimitInput}
                min={1}
                max={100}
                precision={0}
                disabled={this.isFieldDisabled('receiveNumLimit')}
              />
            </FormItem>

            <FormItem
              name="price"
              rules={[{ required: true, message: '购买金额不能为空' }]}
              formItemProps={{
                label: '购买金额',
                colon: false,
                labelCol: { span: 4 },
                wrapperCol: { span: 20 },
              }}
              slot={
                <TipIcon left={240} title="购买金额：用户购买抵扣券实际支付金额，抵扣券最低可抵扣的金额=购买金额" />
              }
            >
              <PriceInputField
                onChange={(price) => this.setState({ price })}
                disabled={this.isFieldDisabled('price')}
              />
            </FormItem>

            {this.type === 0 && (
              <FormItem
                name="showScope"
                rules={[{ required: true, message: '购买渠道不能为空' }]}
                initialValue={showScopeDict[0].val}
                formItemProps={{
                  label: '购买渠道',
                  colon: false,
                  labelCol: { span: 4 },
                  wrapperCol: { span: 20 },
                }}
              >
                <RadioGroup disabled={this.isFieldDisabled('showScope')}>
                  {showScopeDict.map((item) => (
                    <Radio value={item.val} key={item.val}>
                      {item.text}
                    </Radio>
                  ))}
                </RadioGroup>
              </FormItem>
            )}

            <Title>用券规则</Title>

            <FormItem
              name="limitScope"
              rules={[{ required: true, validator: this.limitScopeValidator }]}
              initialValue={limitScopeDict[0].val}
              formItemProps={{
                label: '产品范围',
                colon: false,
                labelCol: { span: 4 },
                wrapperCol: { span: 20 },
              }}
            >
              <RadioGroup disabled={this.isFieldDisabled('limitScope')}>
                {limitScopeDict.map((item) => (
                  <Radio className={style.radioBlock} value={item.val} key={item.val}>
                    <div className={style.checkboxContentWrapper}>
                      <span>{item.text}</span>
                      {this.state.limitScope === item.val &&
                        this.renderLimitScopeContent(item.val, this.isFieldDisabled('limitScope'))}
                    </div>
                  </Radio>
                ))}
              </RadioGroup>
            </FormItem>

            <FormItem
              name="superPosition"
              rules={[{ required: true, message: '叠加方式不能为空' }]}
              initialValue={superpositionDict[1].val}
              formItemProps={{
                label: '叠加方式',
                colon: false,
                labelCol: { span: 4 },
                wrapperCol: { span: 20 },
              }}
              slot={
                <TipIcon
                  title={
                    this.type === 0
                      ? '是否可与其他类型优惠券叠加使用'
                      : '个性化的抵扣券每个产品仅可使用一张，但一个合同里若有多个产品，可通过这个设置来控制是否可使用多张'
                  }
                />
              }
            >
              <RadioGroup disabled={this.isFieldDisabled('superPosition')}>
                {superpositionDict.map((item) => (
                  <Radio value={item.val} key={item.val}>
                    {item.text}
                  </Radio>
                ))}
              </RadioGroup>
            </FormItem>

            <FormItem
              name="periodType"
              rules={[{ required: true, message: '有效期方式不能为空' }]}
              initialValue={periodValidDict[0].val}
              formItemProps={{
                label: '有效期方式',
                colon: false,
                labelCol: { span: 4 },
                wrapperCol: { span: 12 },
              }}
            >
              <RadioGroup disabled={this.isFieldDisabled('periodType')}>
                {periodValidDict.map((item) => (
                  <Radio value={item.val} key={item.val}>
                    {item.text}
                  </Radio>
                ))}
              </RadioGroup>
            </FormItem>

            <FormItem
              name="validDateRange"
              rules={[
                {
                  required: isFixedPeriod,
                  validator: this.validDateRangeValidator,
                },
              ]}
              wrapClassName={isRelativePeriod ? style.displayNone : ''}
              formItemProps={{
                label: '有效期',
                colon: false,
                labelCol: { span: 4 },
                wrapperCol: { span: 20 },
              }}
            >
              <RangePicker
                placeholder={['开始时间', '结束时间']}
                showTime={{
                  hideDisabledOptions: true,
                  defaultValue: [moment(defaultBeginTime, timeFormat), moment(defaultEndTime, timeFormat)],
                }}
                format={rangeFormat}
                size="large"
                disabled={this.isFieldDisabled('validDateRange')}
              />
            </FormItem>

            <FormItem
              name="relativePeriod"
              rules={[{ required: isRelativePeriod, message: '有效期不能为空' }]}
              wrapClassName={isFixedPeriod ? style.displayNone : ''}
              formItemProps={{
                label: '有效期',
                colon: false,
                labelCol: { span: 4 },
                wrapperCol: { span: 20 },
              }}
              slot={
                <Row className={style.rowPeriodNum}>
                  <Col span={4} />
                  <Col span={4}>
                    <span className={style.prevPeriodText}>购买后</span>
                  </Col>
                  <Col span={1} />
                  <Col span={4}>
                    <span>天有效</span>
                  </Col>
                </Row>
              }
            >
              <InputNumber
                className={style.inputPeriodNum}
                min={1}
                precision={0}
                disabled={this.isFieldDisabled('relativePeriod')}
              />
            </FormItem>

            <FormItem
              name="useCase"
              rules={[{ required: true, message: '合同状态不能为空' }]}
              initialValue={contractStateDict[0].val}
              formItemProps={{
                label: '合同状态',
                colon: false,
                labelCol: { span: 4 },
                wrapperCol: { span: 20 },
              }}
            >
              <RadioGroup disabled={this.isFieldDisabled('useCase')}>
                {contractStateDict.map((item) => (
                  <Radio value={item.val} key={item.val}>
                    {item.text}
                  </Radio>
                ))}
              </RadioGroup>
            </FormItem>

            {/** 一对一、一对多类型没有连报条件选项 */}
            {this.type === 0 && (
              <FormItem
                name="continuityType"
                rules={[{ required: true, message: '连报条件不能为空' }]}
                initialValue={continuityDict[0].val}
                formItemProps={{
                  label: '连报条件',
                  colon: false,
                  labelCol: { span: 4 },
                  wrapperCol: { span: 20 },
                }}
              >
                <RadioGroup disabled={this.isFieldDisabled('continuityType')}>
                  {continuityDict.map((item) => (
                    <Radio value={item.val} key={item.val}>
                      {item.text}
                    </Radio>
                  ))}
                </RadioGroup>
              </FormItem>
            )}

            <CouponCondition
              disabled={this.isFieldDisabled('useRules')}
              continuityType={this.state.continuityType}
              ruleList={this.state.ruleList}
              rulesValidator={this.rulesValidator}
              discountAmount={this.state.discountAmount}
              onAdd={this.addRule}
              onDel={this.delRule}
              type={this.type}
              price={this.state.price}
            />

            <FormItem
              name="promotionComment"
              formItemProps={{
                label: '使用说明',
                colon: false,
                labelCol: { span: 4 },
                wrapperCol: { span: 20 },
              }}
              wrapClassName={style.FortItemComment}
              slot={
                <div className={style.DropdownMenuWrap}>
                  <Row>
                    <Col span={4} />
                    <Col span={8}>
                      <DropdownMenu
                        dropdownClassName={style.dropDownMenu}
                        text="使用说明历史"
                        menus={this.renderTemplateMenu()}
                        hasArrow={true}
                        placement="topLeft"
                      />
                    </Col>
                  </Row>
                </div>
              }
            >
              <TextArea
                placeholder="请输入抵扣券使用说明(最多200字)"
                wordCount={true}
                maxLength={200}
                rows={3}
                onFocus={this.createComment}
                disabled={this.isFieldDisabled('promotionComment')}
              />
            </FormItem>
          </Form>
        </div>

        {this.state.modalVisible && (
          <ModalSelectPrd
            initList={this.state.prdList}
            initSKey={this.state.modalInitSKey}
            platType={this.state.platType}
            onClose={this.switchModalSelectPrd}
            onSubmit={this.handleProductChange}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const { activeTabId, pageParams } = state.main;
  const { branch: branchDict } = state.dict;

  return {
    activeTabId,
    pageParams,
    branchDict,
  };
};

export default connect(mapStateToProps)(AddEdit);
