import React from 'react';
import { Modal, FormItem, Button } from 'COMPONENT';
import moment from 'moment';
import { batchUpdateCouponValidPeriod } from 'SERVICE/preferential';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { DatePicker } from 'antd';
import PropTypes from 'prop-types';

const initState = {
  startTime: null,
  endTime: null,
  loading: false,
};

const ModalFooterBtn = (props) => {
  const { onOk, onCancel, confirmBtnDisabled } = props;
  return (
    <div style={{ display: 'flex', justifyContent: 'flex-end' }}>
      <Button type="primary" onClick={onOk} disabled={confirmBtnDisabled}>
        提交修改
      </Button>
      <Button onClick={onCancel}>取消</Button>
    </div>
  );
};

export default class EditExpiryDateModal extends React.Component {
  static propTypes = {
    onConfirm: PropTypes.func.isRequired,
  };

  state = {
    ...initState,
  };

  constructor(props) {
    super(props);
  }

  getDisabledStartDate = (startDate) => {
    const { endTime } = this.state;
    const { oldStartTime, relativePeriod } = this.props;

    if (relativePeriod) {
      if (!startDate || !endTime) {
        return false;
      }

      return moment(startDate).valueOf() > moment(endTime).valueOf();
    }

    return moment(startDate).valueOf() > moment(oldStartTime).valueOf();
  };

  getDisabledEndDate = (endDate) => {
    const { startTime } = this.state;
    const { oldStartTime, relativePeriod } = this.props;
    if (relativePeriod) {
      if (!endDate || !startTime) {
        return false;
      }
      return moment(endDate).valueOf() < moment(startTime).valueOf();
    }

    return moment(endDate).valueOf() < moment(oldStartTime).valueOf();
  };

  onOk = () => {
    this.setState({
      loading: true,
    });

    const params = {
      couponId: this.props.couponIds,
      promotionInfoId: this.props.promotionInfoId,
      startDate: moment(this.state.startTime).format('YYYY-MM-DD HH:mm:ss'),
      endDate: moment(this.state.endTime).format('YYYY-MM-DD HH:mm:ss'),
    };

    batchUpdateCouponValidPeriod(params)
      .then((res) => {
        showMessage('操作成功', 'success');
        setTimeout(() => {
          this.props.onClose();
        }, 10);
      })
      .catch((error) => {})
      .finally(() => {
        this.props.onConfirm();
        this.setState({
          loading: false,
        });
      });
  };

  onClose = () => {
    this.setState({
      ...initState,
    });
    this.props.onClose();
  };

  render() {
    return (
      <Modal
        loading={this.state.loading}
        onClose={this.onClose}
        title="修改有效期"
        width={500}
        footer={
          <ModalFooterBtn
            onOk={this.onOk}
            onCancel={this.onClose}
            confirmBtnDisabled={!this.state.startTime || !this.state.endTime}
          />
        }
      >
        <div style={{ paddingTop: '30px' }}>
          <FormItem name="有效期开始时间" required={true}>
            <DatePicker
              placeholder="有效期开始时间"
              format="YYYY-MM-DD HH:mm:ss"
              size="large"
              value={this.state.startTime}
              onChange={(startTime) => this.setState({ startTime })}
              disabledDate={this.getDisabledStartDate}
              allowClear={true}
              showTime
              style={{ width: '100%' }}
            />
          </FormItem>
          <FormItem name="有效期截止时间" required={true}>
            <DatePicker
              format="YYYY-MM-DD HH:mm:ss"
              size="large"
              placeholder="有效期截止时间"
              value={this.state.endTime}
              onChange={(endTime) => this.setState({ endTime })}
              disabledDate={this.getDisabledEndDate}
              allowClear={true}
              showTime
              style={{ width: '100%' }}
            />
          </FormItem>
        </div>
      </Modal>
    );
  }
}
