import React from 'react';
import { connect } from 'dva';
import { Row, Col } from 'antd';
import { Page, List, Button } from 'COMPONENT';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import apis from 'SERVICE/preferential';
import Search from './Search';
import EditExpiryDateModal from './EditExpiryDateModal';
import { statusMap } from './config';

import styles from './styles.module.less';

const { COL_WIDTH_M, COL_WIDTH_L } = List;
const COL_WIDTH_XL = COL_WIDTH_L + 20;

const dataColumns = [
  {
    title: '券编号',
    dataIndex: 'id',
    key: 'id',
    width: COL_WIDTH_M,
  },
  {
    title: '学生姓名',
    dataIndex: 'studentName',
    key: 'studentName',
    width: COL_WIDTH_M,
  },
  {
    title: '学生编号',
    dataIndex: 'studentId',
    key: 'studentId',
    width: COL_WIDTH_XL,
  },
  {
    title: '客户号码',
    dataIndex: 'customerContact',
    key: 'customerContact',
    width: COL_WIDTH_L,
  },
  {
    title: '分公司',
    dataIndex: 'branchName',
    key: 'branchName',
    width: COL_WIDTH_M,
  },
  {
    title: '学籍学习中心',
    dataIndex: 'campusName',
    key: 'campusName',
    width: COL_WIDTH_XL,
  },
  {
    title: '意向学习中心',
    dataIndex: 'intentionCampusName',
    key: 'intentionCampusName',
    width: COL_WIDTH_XL,
  },
  {
    title: '年级',
    dataIndex: 'gradeName',
    key: 'gradeName',
    width: COL_WIDTH_M,
  },
  {
    title: '买券时间',
    dataIndex: 'createTime',
    key: 'createTime',
    width: COL_WIDTH_XL,
  },
  {
    title: '有效期开始日',
    dataIndex: 'startTime',
    key: 'startTime',
    width: COL_WIDTH_XL,
  },
  {
    title: '有效期截止日',
    dataIndex: 'endTime',
    key: 'endTime',
    width: COL_WIDTH_XL,
  },
  {
    title: '状态',
    dataIndex: 'status',
    key: 'status',
    width: COL_WIDTH_M,
    render(status) {
      return statusMap[status];
    },
  },
  {
    title: '买券订单编号',
    dataIndex: 'buyOrderNo',
    key: 'buyOrderNo',
    width: COL_WIDTH_XL,
  },
  {
    title: '合同编号',
    dataIndex: 'contractId',
    key: 'contractId',
    width: COL_WIDTH_M,
  },
  {
    title: '已抵扣金额',
    dataIndex: 'discountDecimal',
    key: 'discountDecimal',
    width: COL_WIDTH_M,
  },
];

class BuyingRecord extends React.Component {
  constructor(props) {
    super(props);

    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.loadList = this.loadList.bind(this);
    this.changePage = this.changePage.bind(this);
    this.search = this.search.bind(this);
    this.handleExport = this.handleExport.bind(this);

    this.state = {
      name: '买券历史',
      loading: false,
      listLoading: false,
      list: [],
      total: 0,
      pageCurrent: 1,
      pageSize: 20,
      searchModel: {},
      selectedRowKeys: [],
      selectedRows: [],
      editExpiryDateModalVisible: false,
      startDate: '',
      relativePeriod: '',
    };
  }

  componentDidMount() {
    this.init();
  }

  async init() {
    this.setState({ loading: true });

    await new Promise(this.initDict);
    await this.getPromotionInfoById();
    await this.loadList(this.state.pageCurrent, this.state.pageSize);

    this.setState({ loading: false });
  }

  /**
   * 初始化字典
   * @param {function} resolve promise resolve
   */
  initDict(resolve) {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['branch', 'branchCampus'],
        callback: resolve,
      },
    });
  }

  /**
   * 加载列表
   * @param {number} pageNo 页码
   * @param {number} pageSize 单页容量
   */
  async loadList(pageNo, pageSize) {
    this.setState({
      listLoading: true,
      pageCurrent: pageNo,
      pageSize,
    });

    try {
      const { searchModel } = this.state;
      const response = await apis.getBuyingUsingCouponRecord({
        // 券模板 id
        promotionInfoId: this.props.couponId,
        pageNo,
        pageSize,
        ...searchModel,
      });

      if (response) {
        this.setState({
          list: response.datas,
          total: response.rowCount,
        });
      }
    } catch (e) {
      showMessage(e.message, 'error');
    } finally {
      this.setState({ listLoading: false });
    }
  }

  getPromotionInfoById = async () => {
    return apis
      .getPromotionInfoById({ id: this.props.couponId })
      .then(({ data: { startDate, relativePeriod } }) => {
        this.setState({
          startDate,
          relativePeriod,
        });
      })
      .catch((e) => {});
  };

  /**
   * 调整表格页码设置
   * @param {Object} param0 负载
   * @param {number} current 当前页
   * @param {number} pageSize 每页容量
   */
  changePage({ current, pageSize }) {
    this.setState({
      pageCurrent: current,
      pageSize,
    });
    this.loadList(current, pageSize);
  }

  /**
   * 查询列表
   * @param {Object} model 查询键值对
   */
  search(model) {
    this.setState(
      (prevState) => {
        const { searchModel } = prevState;

        return {
          searchModel: {
            ...searchModel,
            ...model,
          },
        };
      },
      () => {
        this.loadList(1, this.state.pageSize);
      },
    );
  }

  /**
   * 导出
   */
  async handleExport() {
    this.setState({ loading: true });

    try {
      const { searchModel } = this.state;
      const response = await apis.exportBuyingUsingCouponRecord({
        // 券模板 id
        promotionInfoId: this.props.couponId,
        // 手机号码脱敏
        isEncryption: true,
        ...searchModel,
      });

      if (response && response.resultCode === 0) {
        this.props.dispatch({
          type: 'main/exportNotice',
        });
      } else {
        showMessage('导出失败', 'error');
      }
    } catch (e) {
      showMessage(e.message, 'error');
    } finally {
      this.setState({ loading: false });
    }
  }

  onSelectChange = (selectedRowKeys, selectedRows) => {
    this.setState({ selectedRowKeys, selectedRows });
  };

  openEditExpiryDateModal = () => {
    this.setState({
      editExpiryDateModalVisible: true,
    });
  };

  closeEditExpiryDateModal = () => {
    this.setState({
      editExpiryDateModalVisible: false,
    });
  };

  render() {
    const { route, btnAuths } = this.props;
    const { name, loading, listLoading, list, total, pageCurrent, pageSize } = this.state;

    return (
      <Page
        name={name}
        fitWindowHeight
        loading={loading}
        route={route}
        subHeader={
          <div className={styles.search}>
            <div className={styles.searchForm}>
              <Search handleSearch={this.search} />
            </div>
            <div className={styles.searchButtons}>
              <Button size="large" onClick={this.handleExport}>
                导出
              </Button>
              {this.props.btnAuth_COUPON_MODIFY_EXPIRY_TIME && (
                <Button
                  style={{ marginLeft: '20px' }}
                  type="primary"
                  onClick={this.openEditExpiryDateModal}
                  disabled={this.state.selectedRowKeys.length < 1}
                >
                  修改有效期
                </Button>
              )}
            </div>
          </div>
        }
      >
        <List
          key="id"
          dataSource={list}
          columns={dataColumns}
          loading={!loading && listLoading}
          pagination={{
            current: pageCurrent,
            pageSize,
            total,
          }}
          onChange={this.changePage}
          rowSelection={{
            selectedRowKeys: this.state.selectedRowKeys,
            onChange: this.onSelectChange,
            getCheckboxProps: (record) => ({
              disabled: record.status != 0,
            }),
          }}
        />
        {this.state.editExpiryDateModalVisible && (
          <EditExpiryDateModal
            onClose={this.closeEditExpiryDateModal}
            oldStartTime={this.state.startDate}
            relativePeriod={this.state.relativePeriod}
            couponIds={this.state.selectedRows.map((row) => row.id)}
            promotionInfoId={this.props.couponId}
            onConfirm={this.search}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const { couponId } = state.main.pageParams['/deductionCoupon/buyingRecord'];
  const { btnAuths } = state.account;

  return {
    couponId,
    btnAuths,
    btnAuth_COUPON_MODIFY_EXPIRY_TIME: btnAuths.indexOf('COUPON_MODIFY_EXPIRY_TIME') !== -1,
  };
};

export default connect(mapStateToProps)(BuyingRecord);
