import React from 'react';
import PropTypes from 'prop-types';
import { Modal, Button, Radio, Deck, Select } from 'COMPONENT';
import SelectedCard from '../AddEdit/ModalSelectPrd/SelectedCard';
import Regular from './regular';
import { Form, Input, DatePicker } from 'antd';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';

import styles from './styles.module.less';
import { loadBranchDict } from 'UTIL/dict';
import { errorsHandle } from 'UTIL/common';
import { getPromotionInfoById, getPromotionProductById, auditPass, auditNotPass } from 'SERVICE/preferential';
import { getBossOrganizationList } from 'SERVICE/system';
import { limitScopeDict, contractStateDict } from '../AddEdit/const';

import {
  periodValidDict,
  superpositionDict,
  continuityDict,
  PERIOD_VALID_REGULAR,
  PERIOD_VALID_RELATIVE,
  PRODUCT_TYPE_SMALL_CLASS,
  SHOW_SCOPE_OTHERS,
  showScopeDict,
} from '../config';

import { fieldDataAdapter } from '../AddEdit/utils';

const RadioGroup = Radio.Group;
const FormItem = Form.Item;
const { RangePicker } = DatePicker;
const { TextArea } = Input;
const SMALL_CLASS = PRODUCT_TYPE_SMALL_CLASS;

const formItemLayout = {
  labelCol: { span: 4 },
  wrapperCol: { span: 20 },
};

function DisabledFormItem(props) {
  return (
    <FormItem {...props} {...formItemLayout} required disabled>
      {props.children}
    </FormItem>
  );
}

class CouponAudit extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      couponData: { ruleVos: [], prdList: [] },
      branchList: [],
      campusList: [],
      reason: '',
    };
    this.currentId = null;
  }

  static propTypes = {
    visible: PropTypes.bool,
    id: PropTypes.oneOfType([PropTypes.number, PropTypes.string]),
    type: PropTypes.string,
    onSuccess: PropTypes.func,
    onCancel: PropTypes.func,
    // detai: '详情'
    // check: '审核'
    mode: PropTypes.oneOf(['detail', 'check']),
  };

  static defaultProps = {
    mode: 'detail',
    onSuccess: function () {},
    onCancel: function () {},
  };

  componentDidMount() {
    this.reset();
    this.fetchBranchList();

    if (this.props.visible && this.props.id) {
      this.currentId = this.props.id;
      this.fetchDetail(this.props.id);
    }
  }

  componentWillReceiveProps(nextProps) {
    if (nextProps.visible && this.props.id !== nextProps.id) {
      this.currentId = nextProps.id;
      this.reset();
      this.fetchDetail(nextProps.id);
    }
  }

  reset() {
    this.setState({
      reason: '',
      couponData: { ruleVos: [], prdList: [] },
    });
  }

  async loadBossOrganizationList(parentId) {
    const result = await getBossOrganizationList({ parentId, type: 'BOSS' }).catch(errorsHandle());
    if (!result || result.resultCode !== 0 || !result.data.length) {
      this.setState({ campusList: [] });
    } else {
      const campusList = result.data
        .filter(({ status }) => status !== 1)
        .map(({ id, name }) => ({
          val: id,
          text: name,
        }));
      this.setState({ campusList });
    }
  }

  // 获取公司列表
  fetchBranchList() {
    loadBranchDict().then((res) => {
      this.setState({
        branchList: res,
      });
    });
  }
  // 获取详情
  async fetchDetail(couponId) {
    if (!couponId) return;

    const fetchList = [
      getPromotionInfoById({ id: couponId }, true),
      getPromotionProductById({ promotionInfoId: couponId }, true),
    ];
    const [rspDetail, rspProduct] = await Promise.all(fetchList, true).catch(errorsHandle());
    const payload = {};

    if (rspProduct && rspProduct.data) {
      payload.prdList = rspProduct.data || [];
    }

    let formFieldModel = null;

    if (rspDetail && rspDetail.data) {
      const { ruleVos = [], ...restDetailData } = rspDetail.data;

      const otherModel = fieldDataAdapter(restDetailData);
      const combinedModel = { ...otherModel };

      formFieldModel = Object.keys(combinedModel).reduce((prev, key) => {
        const value = combinedModel[key];

        if (value !== null && value !== undefined) {
          prev[key] = value;
        }

        return prev;
      }, {});

      // 优惠券详情冗余
      payload.detailStorage = {
        ...restDetailData,
        ruleVos,
      };
    }

    const couponData = {
      ...payload.detailStorage,
      ...payload,
      ...formFieldModel,
    };

    if (formFieldModel && formFieldModel.branchId) {
      this.loadBossOrganizationList(formFieldModel.branchId);
    }

    this.setState({
      couponData,
      reason: couponData.auditFailReason,
    });
  }

  // 审核通过
  couponCheckPass = () => {
    const { id, onSuccess } = this.props;

    showConfirm('提示：确定审核通过吗？', async () => {
      try {
        await auditPass({ id });

        showMessage('已审核', 'success');
        onSuccess();
      } catch (e) {
        showMessage(e.text, 'error');
      }
    });
  };

  // 审核不通过
  couponCheckReject = () => {
    const { id, onSuccess } = this.props;
    const reason = this.state.reason;

    if (!reason) {
      return showMessage('请输入审核不通过原因', 'error');
    }

    if (reason.length > 250) {
      return showMessage('不超过250个字符', 'error');
    }

    showConfirm('提示：确定审核不通过吗？', async () => {
      try {
        await auditNotPass({
          id,
          auditFailReason: reason,
        });

        showMessage('已审核', 'success');
        onSuccess();
      } catch (e) {
        showMessage(e.text, 'error');
      }
    });
  };

  render() {
    const { visible, mode, onCancel, type } = this.props;
    const { couponData, reason } = this.state;
    const isReadonly = mode === 'detail';

    return visible ? (
      <Modal
        className={styles.modal}
        title={isReadonly ? '查看抵扣券' : '抵扣券审核'}
        width={800}
        onCancel={onCancel}
        footer={
          <div className={styles.operation}>
            <p className={styles.title} />
            <TextArea
              rows={4}
              value={reason}
              className={styles.reason}
              placeholder="请输入审核意见"
              disabled={isReadonly}
              onChange={(e) => this.setState({ reason: e.target.value })}
            />
            {isReadonly ? (
              <Button type="primary" onClick={onCancel}>
                确定
              </Button>
            ) : (
              <div>
                <Button type="primary" onClick={this.couponCheckPass}>
                  审核通过
                </Button>
                <Button type="danger" onClick={this.couponCheckReject}>
                  审核不通过
                </Button>
              </div>
            )}
          </div>
        }
      >
        <Form className={styles.couponForm}>
          <block>
            <p className={styles.title}>基础属性</p>
            <DisabledFormItem label="抵扣券名称">
              <Input value={couponData.promotionName} disabled />
            </DisabledFormItem>

            <DisabledFormItem label="所属分公司">
              <Select value={couponData.branchId} placeholder="所属分公司" disabled dict={this.state.branchList} />
            </DisabledFormItem>

            <DisabledFormItem label="所属学习中心">
              {(couponData.promotionCampusVos || []).map(({ name }) => name).join('；')}
            </DisabledFormItem>
          </block>

          <block>
            <p className={styles.title}>购买规则</p>
            <DisabledFormItem label="购买时间">
              <RangePicker value={couponData.buyingDateRange} format="YYYY-MM-DD HH:mm:ss" disabled />
            </DisabledFormItem>

            <DisabledFormItem label="购买身份">
              <RadioGroup value={couponData.publishObject} disabled>
                <Radio value="STUDENT">学生</Radio>
              </RadioGroup>
            </DisabledFormItem>

            <DisabledFormItem label="可购买张数">
              <Input value={couponData.receiveNumLimit} disabled />
            </DisabledFormItem>

            <DisabledFormItem label="购买金额">
              <Input value={couponData.price} suffix={<span>元</span>} disabled />
            </DisabledFormItem>

            {type === SMALL_CLASS && (
              <DisabledFormItem label="购买渠道">
                <RadioGroup value={couponData.showScope || SHOW_SCOPE_OTHERS} disabled>
                  {showScopeDict.map((item) => (
                    <Radio value={item.val} key={item.val}>
                      {item.text}
                    </Radio>
                  ))}
                </RadioGroup>
              </DisabledFormItem>
            )}
          </block>

          <block>
            <p className={styles.title}>用券规则</p>
            <DisabledFormItem label="产品范围">
              <RadioGroup value={couponData.limitScope} disabled>
                {limitScopeDict.map((item) => (
                  <Radio value={item.val} key={item.val}>
                    {item.text}
                  </Radio>
                ))}
              </RadioGroup>
              <Deck
                dataSource={couponData.prdList}
                className={styles.selectedCard}
                bodyClassName={styles.selectedCardContent}
                render={(props, index) => <SelectedCard {...props} index={index} key={index} />}
              />
            </DisabledFormItem>
            <DisabledFormItem label="叠加方式">
              <RadioGroup value={couponData.superPosition} disabled>
                {superpositionDict.map((item) => (
                  <Radio value={item.val} key={item.val}>
                    {item.text}
                  </Radio>
                ))}
              </RadioGroup>
            </DisabledFormItem>
            <DisabledFormItem label="有效期方式">
              <RadioGroup value={couponData.periodType} disabled>
                {periodValidDict.map((item) => (
                  <Radio value={item.val} key={item.val}>
                    {item.text}
                  </Radio>
                ))}
              </RadioGroup>
            </DisabledFormItem>

            <DisabledFormItem label="有效期">
              {couponData.periodType === PERIOD_VALID_REGULAR && (
                <RangePicker value={couponData.validDateRange} size="large" format="YYYY-MM-DD HH:mm:ss" disabled />
              )}
              {couponData.periodType === PERIOD_VALID_RELATIVE && (
                <p>
                  购买后
                  {<Input className={styles.price} value={couponData.relativePeriod} disabled />}
                  天有效
                </p>
              )}
            </DisabledFormItem>

            <DisabledFormItem label="合同状态">
              <RadioGroup value={couponData.useCase} disabled>
                {contractStateDict.map((item) => (
                  <Radio value={item.val} key={item.val}>
                    {item.text}
                  </Radio>
                ))}
              </RadioGroup>
            </DisabledFormItem>

            {/* 小班有连报条件 */}
            {type === SMALL_CLASS && (
              <DisabledFormItem label="连报条件">
                <RadioGroup value={couponData.continuityType} disabled>
                  {continuityDict.map((item) => (
                    <Radio value={item.val} key={item.val}>
                      {item.text}
                    </Radio>
                  ))}
                </RadioGroup>
              </DisabledFormItem>
            )}

            <DisabledFormItem label="抵扣规则">
              <Regular type={type} regularList={couponData.ruleVos} couponData={couponData} />
            </DisabledFormItem>
            <FormItem label="使用说明" {...formItemLayout}>
              <TextArea rows={4} disabled value={couponData.promotionComment} />
            </FormItem>
          </block>
        </Form>
      </Modal>
    ) : null;
  }
}

export default CouponAudit;
