import styles from './styles.module.less';
import NP from 'number-precision';
import { extendRuleTextCfg } from '../AddEdit/const';
import { quarterTextCfg, PRODUCT_TYPE_SMALL_CLASS, COURSE_HOURS } from '../config';
import { COURSE_HOURS_TRANSFORM_MULTIPLE } from '../store/constant';

const SMALL_CLASS = PRODUCT_TYPE_SMALL_CLASS;

export default function (props) {
  const { type, regularList, couponData = {} } = props;

  // 普通显示
  const NormalRegular = ({ rule }) => {
    const per = extendRuleTextCfg[couponData.promotionTypeId];
    const unit = Object.assign({ [COURSE_HOURS]: '个小时' }, extendRuleTextCfg)[rule.ruleUnit];
    const compute =
      rule.promotionRule && type !== SMALL_CLASS
        ? NP.divide(rule.promotionRule, COURSE_HOURS_TRANSFORM_MULTIPLE).toFixed(2)
        : rule.promotionRule;

    return (
      <div>
        {per}
        <span className={styles.limit}>{compute}</span>
        {unit}，优惠
        <span className={styles.limit}>{rule.promotion}</span>
        元，合同使用时抵扣
        <span className={styles.limit}>{NP.plus(rule.promotion, couponData.price)}</span>
        元；
      </div>
    );
  };

  // 小班选了续报类型的显示
  const ContinueRegular = ({ rule }) => {
    const per = extendRuleTextCfg[couponData.promotionTypeId];
    const unit = extendRuleTextCfg[rule.ruleUnit];
    const classType = couponData.continuityType.split('_');

    return (
      <div>
        {per}
        <span className={styles.limit}>{rule.promotionRule}</span>
        {unit}，{quarterTextCfg[classType[0]]}优惠
        <span className={styles.limit}>{rule.quarterPromotionA}</span>元，
        {quarterTextCfg[classType[1]]}优惠
        <span className={styles.limit}>{rule.quarterPromotionB}</span>
        元，合同使用时抵扣
        <span className={styles.limit}>{NP.plus(rule.promotion, couponData.price)}</span>
        元；
      </div>
    );
  };

  if (!regularList || regularList.length == 0) return null;

  if (type === SMALL_CLASS && couponData.continuityType !== 'NONE') {
    return (
      <div>
        {regularList.map((rule, i) => {
          return <ContinueRegular key={i} rule={rule} />;
        })}
      </div>
    );
  } else {
    return (
      <div>
        {regularList.map((rule, i) => {
          return <NormalRegular key={i} rule={rule} />;
        })}
      </div>
    );
  }
}
