import React from 'react';
import PropTypes from 'prop-types';
import style from './index.style.less';
import { PopBtnGroups } from 'COMPONENT';

const ButtonGroup = (props) => {
  return (
    <div className={style.buttonGroup}>
      {props.columns.map((item, index) => {
        if (Array.isArray(item.groups) && item.groups.length > 0) {
          return (
            <PopBtnGroups groups={item.groups} data={item} key={index}>
              <div className={style.buttonGroupItem} key={index}>
                {item.text}
              </div>
            </PopBtnGroups>
          );
        }

        return item.disabled ? (
          <div className={style.buttonGroupDisabled} key={index}>
            {item.text}
          </div>
        ) : (
          <div className={style.buttonGroupItem} onClick={item.onClick} key={index}>
            {item.text}
          </div>
        );
      })}
    </div>
  );
};

ButtonGroup.propTypes = {
  columns: PropTypes.arrayOf(
    PropTypes.shape({
      text: PropTypes.string.isRequired,
      onClick: PropTypes.func,
      disabled: PropTypes.bool,
      groups: PropTypes.arrayOf(
        PropTypes.shape({
          name: PropTypes.string.isRequired,
          onClick: PropTypes.func.isRequired,
          disabled: PropTypes.bool,
        }),
      ),
    }),
  ).isRequired,
};

ButtonGroup.defaultProps = {
  disabled: false,
};

export default ButtonGroup;
