import React from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';
import { Tooltip } from 'antd';
import { Card, TextLink } from 'COMPONENT';
import { runFunc, loginUser } from 'UTIL/common';
import CouponIcon from './CouponIcon';
import ButtonGroup from './ButtonGroup';
import Info from './Info';
import DetailWithPop from './DetailWithPop';
import MoreInfoBtn from './MoreInfoBtn';
import ModalViewCouponData from './ModalViewCouponData';
import {
  LIMIT_SCOPE_INFINITE,
  LIMIT_SCOPE_FINITE,
  LIMIT_SCOPE_PRODUCT_TYPE,
  xhbossProductTypeDict,
  ssbossProductTypeDict,
  superpositionCfg,
  continuityTextCfg,
  auditStatusTextCfg,
  AUDIT_UNREVIEW,
  AUDIT_REVIEWED,
  AUDIT_REVIEWING,
  AUDIT_Fail,
  SHOW_SCOPE_SHAONIANGONG_MINI_PROGRAM,
} from '../../config';

import style from './index.style.less';

class CouponCard extends React.Component {
  constructor(props) {
    super(props);

    this.switchModalProduct = this.switchModalProduct.bind(this);
    this.switchMoreInfo = this.switchMoreInfo.bind(this);

    this.state = {
      modalProductVisible: false,
      moreInfoVisible: false,
    };
  }

  static propTypes = {
    actions: PropTypes.func.isRequired,
    index: PropTypes.number,
  };

  buildBtnColumns(props) {
    let columns = [
      {
        text: '查看',
        disabled: false,
        onClick() {
          runFunc(props.actions, { actionType: 'VIEW_DETAIL', data: props });
        },
      },
    ];

    const isValid = props.status === '0';
    const isPublish = props.publishNum > 0;
    const isReviewed = props.auditStatus === AUDIT_REVIEWED;
    const isReviewing = isReviewed ? false : props.auditStatus === AUDIT_REVIEWING;
    const isUnReview = isReviewed ? false : props.auditStatus === AUDIT_UNREVIEW || props.auditStatus === AUDIT_Fail;

    if (isValid && isUnReview && props.btnAuth.indexOf('DEDUCTION_COUPON_APPLY') > -1) {
      columns.push({
        text: '提交审核',
        disabled: false,
        onClick() {
          runFunc(props.actions, { actionType: 'APPLY', data: props });
        },
      });
    }

    if (isValid && isReviewing && props.btnAuth.indexOf('DEDUCTION_COUPON_REVOKE') > -1) {
      columns.push({
        text: '撤回',
        disabled: false,
        onClick() {
          runFunc(props.actions, { actionType: 'REVOKE', data: props });
        },
      });
    }

    if (isValid && isReviewing && props.btnAuth.indexOf('DEDUCTION_COUPON_REVIEW') > -1) {
      columns.push({
        text: '审核',
        disabled: false,
        onClick() {
          runFunc(props.actions, { actionType: 'AUDIT', data: props });
        },
      });
    }

    if (isValid && isReviewed && props.btnAuth.indexOf('DEDUCTION_COUPON_SHARE') > -1) {
      let shareGroups = [
        {
          name: '分享该券',
          disabled: false,
          onClick() {
            runFunc(props.actions, { actionType: 'SHARE', data: props });
          },
        },
      ];

      if (props.showScope !== SHOW_SCOPE_SHAONIANGONG_MINI_PROGRAM) {
        shareGroups.push({
          name: '分享券列表',
          disabled: false,
          onClick() {
            runFunc(props.actions, { actionType: 'SHARE_LIST', data: props });
          },
        });
      }

      columns.push({
        text: '分享',
        disabled: false,
        groups: shareGroups,
      });
    }

    if (isPublish && props.btnAuth.indexOf('DEDUCTION_COUPON_HISTORY') > -1) {
      columns.push({
        text: '买券历史',
        disabled: false,
        onClick() {
          runFunc(props.actions, { actionType: 'HISTORY', data: props });
        },
      });
    }

    if (isValid && !isReviewing && props.btnAuth.indexOf('DEDUCTION_COUPON_EDIT') > -1) {
      columns.push({
        text: '编辑',
        disabled: false,
        onClick() {
          runFunc(props.actions, { actionType: 'EDIT', data: props });
        },
      });
    }

    if (isValid && isReviewed && props.btnAuth.indexOf('DEDUCTION_COUPON_INVALID') > -1) {
      columns.push({
        text: '下架',
        disabled: false,
        onClick() {
          runFunc(props.actions, { actionType: 'INVALID', data: props });
        },
      });
    }

    return columns;
  }

  switchModalProduct() {
    this.setState((prevState) => ({ modalProductVisible: !prevState.modalProductVisible }));
  }

  switchMoreInfo() {
    this.setState((prevState) => ({ moreInfoVisible: !prevState.moreInfoVisible }));
  }

  /**
   * 渲染抵扣券产品类型
   * @param {Object} itemProps 抵扣券列表中单条信息
   */
  renderLimitProductTypes(itemProps) {
    const productTypeDict = loginUser.isTwoTeacherSym() ? ssbossProductTypeDict : xhbossProductTypeDict;
    const productTypeMapping = productTypeDict.reduce((prev, el) => {
      prev[el.val] = el.text;

      return prev;
    }, {});
    const { limitProductTypes } = itemProps;
    const productTypeLabels = limitProductTypes.map((name) => productTypeMapping[name]).filter((el) => el);
    const isValid = Array.isArray(productTypeLabels) && productTypeLabels.length > 0;

    return isValid ? productTypeLabels.join(',') : '全部';
  }

  renderLimitScope(itemProps) {
    const { limitScope, productVos } = itemProps;
    const productList = productVos || [];
    let retVal = '';

    switch (String(limitScope)) {
      case LIMIT_SCOPE_INFINITE:
        retVal = '不限制产品';
        break;
      case LIMIT_SCOPE_FINITE:
        retVal = (
          <span>
            <span>限制{productList.length}种产品 </span>
            {productList.length > 0 && <TextLink onClick={this.switchModalProduct}>查看</TextLink>}
          </span>
        );
        break;
      case LIMIT_SCOPE_PRODUCT_TYPE:
        retVal = '不限制产品';
        break;
    }

    return retVal;
  }

  render() {
    const btnColumns = this.buildBtnColumns(this.props);
    const goodsVo = this.props.goodsVo || {};

    return (
      <Card className={style.card}>
        <div className={style.wrapper}>
          <div className={style.header}>
            <div className={style.headerTitle}>
              <CouponIcon status={this.props.status} />
              <Tooltip title={this.props.promotionName}>
                <div className={style.headerTitleText}>{this.props.promotionName}</div>
              </Tooltip>
            </div>

            <div className={style.headerTool}>
              <span>已用 / 已买</span>
              <span className={style.headerToolText}>{`${this.props.useNum} / ${this.props.publishNum}`}</span>
            </div>
          </div>

          <div className={style.subHeader}>
            <div className={style.subHeaderTitle}>
              <span className={style.subHeaderTitleText}>
                购买金额{goodsVo.price || '-'}元，最高抵{goodsVo.discount || '-'}元
              </span>
            </div>

            <div className={style.subHeaderTool}>{btnColumns.length > 0 && <ButtonGroup columns={btnColumns} />}</div>
          </div>

          <div className={style.dashedLine} />

          <div className={style.body}>
            <div className={style.bodyMain}>
              <Info label="所属分公司">
                <DetailWithPop data={(this.props.branchVos || []).map((item) => item.branchName)} />
              </Info>
              <Info label="产品类型">{this.renderLimitProductTypes(this.props)}</Info>
              <Info label="产品范围">{this.renderLimitScope(this.props)}</Info>
              <Info label="叠加方式">{superpositionCfg[`${this.props.superPosition}`]}</Info>
              <Info label="可购买张数">{this.props.receiveNumLimit}张</Info>
              <Info label="连报条件">{continuityTextCfg[this.props.continuityType]}</Info>
              <Info label="买券时间">
                {goodsVo.beginTime || ''} - {goodsVo.endTime || ''}
              </Info>
              <Info label="有效期方式">{this.props.relativePeriod ? '相对有效期' : '固定有效期'}</Info>
              {this.props.relativePeriod ? (
                <Info label="有效期">领取或发券后{this.props.relativePeriod}天</Info>
              ) : (
                <Info label="有效期">
                  {this.props.startDate} - {this.props.endDate}
                </Info>
              )}
              <Info label="状态">{auditStatusTextCfg[this.props.auditStatus]}</Info>
              <Info label="创建者">{this.props.createUserName}</Info>
            </div>

            <div className={style.bodyTool}>
              <MoreInfoBtn
                trigger={this.state.moreInfoVisible}
                onClick={this.switchMoreInfo}
                text="使用说明"
                activeText="使用说明"
              />
            </div>
          </div>

          <div className={style.footer}>
            <div
              className={classNames(style.footerMoreInfo, { [style.footerMoreInfoActive]: this.state.moreInfoVisible })}
            >
              <span>使用说明：</span>
              <span className={style.wordWrap}>{this.props.promotionComment || '暂无'}</span>
            </div>
          </div>
        </div>

        {this.state.modalProductVisible && (
          <ModalViewCouponData initData={this.props} onCancel={this.switchModalProduct} />
        )}
      </Card>
    );
  }
}

export default CouponCard;
