import React from 'react';
import { connect } from 'dva';
import classNames from 'classnames';
import moment from 'moment';
import { stringify } from 'qs';
import { DatePicker } from 'antd';
import {
  Page,
  Button,
  SearchFilterForm,
  SearchFieldForm,
  Input,
  Select,
  MultiSelect,
  Deck,
  NoDataPlus,
  ShareQRCode,
  PopBtnGroups,
} from 'COMPONENT';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import { errorsHandle } from 'UTIL/common';
import {
  COUPON,
  COUPON_FULL_SALE,
  COUPON_FULL_SALE_NORMAL,
  COUPON_DEPOSIT,
  xhbossProductTypeDict,
  ssbossProductTypeDict,
  superpositionDict,
  auditStatusDict,
  rangeFormat,
  timeFormat,
  defaultBeginTime,
  defaultEndTime,
  continuityDict,
  AUDIT_REVIEWING,
  PRODUCT_TYPE_SMALL_CLASS,
  PRODUCT_TYPE_ONE_TO_ONE,
  PRODUCT_TYPE_ONE_TO_MANY,
  SHOW_SCOPE_SHAONIANGONG_MINI_PROGRAM,
} from '../config';
import CouponDetailModal from '../Detail';
import CouponCard from './CouponCard';
import { storeName } from '../store/constant.js';
import styles from './index.style.less';
import { prd, pre, twoTeacher, twoTeacherPre } from 'CONST/buildEnv';
import { SYSTEM_MARK } from 'CONST/config';
import { getPromotionList, invalidPromotionById, submitAudit, recallAudit } from 'SERVICE/preferential';
import { loginUser } from 'UTIL/common';

const preArr = [pre, twoTeacherPre];
const prdArr = [prd, twoTeacher];

const couponCardRender = function ({ actions, btnAuth }) {
  return (props, index) => {
    return <CouponCard {...props} btnAuth={btnAuth} actions={actions} index={index} key={props.id} />;
  };
};

const PAGE_ID = '/deductionCoupon';

class DeductionCouponMgmtEntrance extends React.Component {
  constructor(props) {
    super(props);

    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.initSKey = this.initSKey.bind(this);
    this.getListParams = this.getListParams.bind(this);
    this.loadList = this.loadList.bind(this);
    this.updatePage = this.updatePage.bind(this);
    this.changePage = this.changePage.bind(this);
    this.search = this.search.bind(this);
    this.updatePromotionInfoStatus = this.updatePromotionInfoStatus.bind(this);
    this.onResolveTodo = this.onResolveTodo.bind(this);
    this.newCoupon = this.newCoupon.bind(this);
    this.cardAction = this.cardAction.bind(this);
    this.applyCoupon = this.applyCoupon.bind(this);
    this.revokeApply = this.revokeApply.bind(this);
    this.auditCoupon = this.auditCoupon.bind(this);
    this.confirmInvalid = this.confirmInvalid.bind(this);
    this.invalidCoupon = this.invalidCoupon.bind(this);
    this.goToHistory = this.goToHistory.bind(this);
    this.goToEdit = this.goToEdit.bind(this);
    this.openPage = this.openPage.bind(this);
    this.switchShareQrCode = this.switchShareQrCode.bind(this);
    this.shareCouponList = this.shareCouponList.bind(this);
    this.destroy = this.destroy.bind(this);

    this.state = {
      name: '抵扣券',
      loading: false,
      searchKey: {},
      defaultSKey: {},
      list: [],
      total: 0,
      pageCurrent: 1,
      pageSize: 10,
      platType: null,
      shareQRCodeVisible: false,
      detailConfig: {
        modalVisible: false,
        mode: 'check',
        couponId: '',
        productType: '',
      },
    };

    this.searchRef = null;
    this.shareQRCodeData = {};
  }

  static classPrefix = 'DeductionCouponMgmtEntrance';

  componentDidMount() {
    this.init();
    GlobalEvent.on('CrmNotification:deductionCoupon', this.onResolveTodo);
  }

  componentWillUnmount() {
    GlobalEvent.off('CrmNotification:deductionCoupon', this.onResolveTodo);
    this.props.dispatch({
      type: 'main/deletePageParams',
      payload: { id: PAGE_ID },
    });
    this.destroy();
  }

  async init() {
    this.setState({ loading: true });

    await new Promise(this.initDict);
    await new Promise(this.initSKey);
    await new Promise(this.loadList);

    this.setState({ loading: false });
  }

  initDict(resolve) {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['branch'],
        callback() {
          resolve();
        },
      },
    });
  }

  initSKey(resolve) {
    if (this.props.pageParams.isTodo) {
      this.updatePromotionInfoStatus(resolve);
    } else {
      resolve();
    }
  }

  getListParams() {
    let searchKey = {
      // 接口的第一页是 0 页
      pageNo: this.state.pageCurrent - 1,
      pageSize: this.state.pageSize,
      // 优惠券
      promotionKindId: COUPON,
      // 抵扣券
      promotionLittleTypeId: COUPON_FULL_SALE,
      promotionTypeId: COUPON_FULL_SALE_NORMAL,
      promotionNature: COUPON_DEPOSIT,
    };

    for (let key in this.state.searchKey) {
      searchKey[key] = this.state.searchKey[key];
    }

    return searchKey;
  }

  async loadList(resolve) {
    const apiParams = this.getListParams();

    const rsp = await getPromotionList(apiParams, true).catch(errorsHandle());
    if (rsp && rsp.resultCode === 0) {
      this.setState({
        list: rsp.datas || [],
        total: rsp.rowCount,
      });
    }

    resolve();
  }

  updatePage(pageCurrent, pageSize) {
    return new Promise((resolve) => {
      this.setState(
        (prevState) => {
          let payload = {};
          if (pageSize && pageSize !== prevState.pageSize) {
            payload.pageCurrent = 1;
            payload.pageSize = pageSize;
          } else {
            payload.pageCurrent = pageCurrent;
          }
          return payload;
        },
        () => {
          resolve();
        },
      );
    });
  }

  async changePage(pageCurrent, pageSize) {
    await this.updatePage(pageCurrent, pageSize);
    this.setState({ loading: true });
    await new Promise(this.loadList);
    this.setState({ loading: false });
  }

  search(params) {
    this.setState(
      {
        searchKey: { ...this.props.initSKey, ...this.searchKeyAdapter(params) },
      },
      () => {
        this.changePage(1, this.state.pageSize);
      },
    );
  }

  searchKeyAdapter = (params) => {
    const formattedParams = {};

    if (params.buyStartTime) {
      formattedParams.buyStartTime = params.buyStartTime.format(rangeFormat);
    }
    if (params.buyEndTime) {
      formattedParams.buyEndTime = params.buyEndTime.format(rangeFormat);
    }
    if (params.startDate) {
      formattedParams.startDate = params.startDate.format(rangeFormat);
    }
    if (params.endDate) {
      formattedParams.endDate = params.endDate.format(rangeFormat);
    }

    // 接口入参为 array 形式
    if (params.limitProductTypes) {
      formattedParams.limitProductTypes = [params.limitProductTypes];
    }

    return {
      ...params,
      ...formattedParams,
    };
  };

  updatePromotionInfoStatus(resolve) {
    this.searchRef.resetFields();
    this.searchRef.setFieldsValue({ promotionInfoStatus: AUDIT_REVIEWING });
    this.setState({ searchKey: { promotionInfoStatus: AUDIT_REVIEWING } }, () => {
      resolve();
    });
  }

  async onResolveTodo() {
    await new Promise(this.updatePromotionInfoStatus);
    this.changePage(1, this.state.pageSize);
  }

  newCoupon(type = 1) {
    /**
     * @description type 抵扣券类型 小班 0 一对一 1 一对多 2
     */
    this.openPage('add', { couponId: null, isSend: false, type });
  }

  cardAction(params) {
    switch (params.actionType) {
      case 'VIEW_DETAIL':
        this.viewDetail(params.data);
        break;
      case 'APPLY':
        this.applyCoupon(params.data);
        break;
      case 'REVOKE':
        this.revokeApply(params.data);
        break;
      case 'AUDIT':
        this.auditCoupon(params.data);
        break;
      case 'SHARE':
        this.switchShareQrCode(params.data);
        break;
      case 'SHARE_LIST':
        this.shareCouponList(params.data);
        break;
      case 'HISTORY':
        this.goToHistory(params.data);
        break;
      case 'EDIT':
        this.goToEdit(params.data);
        break;
      case 'INVALID':
        this.confirmInvalid(params.data);
        break;
      default:
        console.error(`Error: invalid card actionType(${params.type})!`);
    }
  }

  async applyCoupon(data) {
    this.setState({ loading: true });
    const res = await submitAudit({ id: data.id }).catch(errorsHandle());
    if (res && res.resultCode === 0) {
      showMessage('提交审核成功！', 'success');
      await new Promise(this.loadList);
    }
    this.setState({ loading: false });
  }

  async revokeApply(data) {
    this.setState({ loading: true });
    const res = await recallAudit({ id: data.id }).catch(errorsHandle());
    if (res && res.resultCode === 0) {
      showMessage('撤回成功！', 'success');
      await new Promise(this.loadList);
    }
    this.setState({ loading: false });
  }

  viewDetail(data) {
    const limitProductTypes = Array.isArray(data.limitProductTypes) ? data.limitProductTypes : [];
    const [productType = ''] = limitProductTypes;

    this.setState({
      detailConfig: {
        modalVisible: true,
        mode: 'detail',
        couponId: data.id,
        productType,
      },
    });
  }

  auditCoupon(data) {
    const limitProductTypes = Array.isArray(data.limitProductTypes) ? data.limitProductTypes : [];
    const [productType = ''] = limitProductTypes;

    this.setState({
      detailConfig: {
        modalVisible: true,
        mode: 'check',
        couponId: data.id,
        productType,
      },
    });
  }

  confirmInvalid(data) {
    showConfirm(
      <span>
        是否下架该抵扣券(<span style={{ color: '#000' }}>{data.promotionName}</span>)？
      </span>,
      () => {
        this.invalidCoupon(data);
      },
    );
  }

  async invalidCoupon(data) {
    this.setState({ loading: true });
    const res = await invalidPromotionById({ id: data.id, status: '1' }).catch(errorsHandle());
    if (res && res.resultCode === 0) {
      showMessage('下架成功！', 'success');
      await new Promise(this.loadList);
    }
    this.setState({ loading: false });
  }

  /**
   * 买券历史
   * @param {Object} data 券列表单行记录
   */
  goToHistory(data) {
    this.openPage('buyingRecord', {
      couponId: data.id,
    });
  }

  goToEdit(data) {
    let type;
    if (data.limitProductTypes.includes(PRODUCT_TYPE_SMALL_CLASS)) {
      type = 0;
    } else if (data.limitProductTypes.includes(PRODUCT_TYPE_ONE_TO_ONE)) {
      type = 1;
    } else if (data.limitProductTypes.includes(PRODUCT_TYPE_ONE_TO_MANY)) {
      type = 2;
    }
    this.openPage('edit', {
      couponId: data.id,
      auditStatus: data.auditStatus,
      isSend: data.publishNum > 0 || !!data.outId,
      type,
    });
  }

  openPage(path, params) {
    let payload = {
      path: `/deductionCoupon/${path}`,
      refresh: true,
    };
    if (params) {
      payload.params = params;
    }
    this.props.dispatch({
      type: 'main/openPage',
      payload,
    });
  }

  getShareLink = (data) => {
    const branchId = Array.isArray(data.branchVos) && data.branchVos.length ? data.branchVos[0].branchId : '';
    let prefix = '';
    let appId = '';

    if (prdArr.includes(process.env.BUILD_ENV)) {
      prefix = 'xhmall';
      appId = 'wx2b1fe6dca1db781e';
    } else if (preArr.includes(process.env.BUILD_ENV)) {
      prefix = 'xhmall-pre';
      appId = 'wxb6f98188c2b39d30';
    } else {
      prefix = 'xhmall-test';
      appId = 'wx3ecb878239508b99';
    }

    const querystr = stringify({
      promotionId: data.id,
      sys: SYSTEM_MARK,
      branchId,
      appid: appId,
    });

    const route = data.showScope === SHOW_SCOPE_SHAONIANGONG_MINI_PROGRAM ? 'DepositPalaceBuyPage' : 'DepositBuyPage';
    const url = `https://${prefix}.xiao100.com/mapp/course-registration/h5/views/common/${route}?${querystr}`;

    return url;
  };

  switchShareQrCode(data) {
    this.setState((prevState) => {
      this.shareQRCodeData = prevState.shareQRCodeVisible
        ? {}
        : {
            ...data,
            shareTitle: data.promotionName,
            shareText: this.getShareLink(data),
          };
      return { shareQRCodeVisible: !prevState.shareQRCodeVisible };
    });
  }

  shareCouponList(data) {
    const mapping = {
      [prd]: {
        prefix: 'xhmall',
        appId: 'wx2b1fe6dca1db781e',
      },
      [pre]: {
        prefix: 'xhmall-pre',
        appId: 'wxb6f98188c2b39d30',
      },
      test: {
        prefix: 'xhmall-test',
        appId: 'wxb6f98188c2b39d30',
      },
    };

    const env = [prd, pre].indexOf(process.env.BUILD_ENV) > -1 ? process.env.BUILD_ENV : 'test';

    const { prefix, appId } = mapping[env] || {};
    const branchId = Array.isArray(data.branchVos) && data.branchVos.length ? data.branchVos[0].branchId : '';
    const querystr = stringify({
      sys: SYSTEM_MARK,
      branchId,
      appid: appId,
    });

    const link = `https://${prefix}.xiao100.com/mapp/course-registration/h5/views/common/DiscountCenter?${querystr}`;

    this.setState((prevState) => {
      this.shareQRCodeData = prevState.shareQRCodeVisible
        ? {}
        : {
            shareTitle: '优惠券列表',
            shareText: link,
          };

      return {
        shareQRCodeVisible: !prevState.shareQRCodeVisible,
      };
    });
  }

  destroy() {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        list_coupon: [],
        total_coupon: 0,
      },
    });
  }

  render() {
    return (
      <Page
        name={this.state.name}
        className={DeductionCouponMgmtEntrance.classPrefix}
        fitWindowHeight={true}
        loading={this.state.loading}
        route={this.props.route}
        headerToolBar={
          <div className={styles.btnWrapper}>
            {this.props.isAddingEnabled && (
              <PopBtnGroups
                popoverProps={{ placement: 'bottomRight', trigger: 'hover' }}
                groups={[
                  {
                    name: (
                      <span className={classNames(!this.props.isClassAddingEnabled ? styles.btnDisabled : '')}>
                        班课 抵扣券
                      </span>
                    ),
                    onClick: () => this.newCoupon(0),
                    disabled: !this.props.isClassAddingEnabled,
                  },
                  {
                    name: (
                      <span className={classNames(!this.props.isO2OAddingEnabled ? styles.btnDisabled : '')}>
                        1 对 1 抵扣券
                      </span>
                    ),
                    onClick: () => this.newCoupon(1),
                    disabled: !this.props.isO2OAddingEnabled,
                  },
                  {
                    name: (
                      <span className={classNames(!this.props.isO2MAddingEnabled ? styles.btnDisabled : '')}>
                        1 对多 抵扣券
                      </span>
                    ),
                    onClick: () => this.newCoupon(2),
                    disabled: !this.props.isO2MAddingEnabled,
                  },
                ]}
              >
                <Button type="primary">新增</Button>
              </PopBtnGroups>
            )}
          </div>
        }
        subHeader={
          <SearchFilterForm onSearch={this.search} truncationNum={4} ref={(c) => (this.searchRef = c)}>
            <SearchFieldForm name="promotionName">
              <Input placeholder="抵扣券名称" />
            </SearchFieldForm>
            <SearchFieldForm name="productName">
              <Input placeholder="产品名称" />
            </SearchFieldForm>
            <SearchFieldForm name="limitProductTypes">
              <Select
                placeholder="产品类型"
                dict={loginUser.isTwoTeacherSym() ? ssbossProductTypeDict : xhbossProductTypeDict}
              />
            </SearchFieldForm>
            <SearchFieldForm name="branchId">
              <MultiSelect placeholder="分公司" dict={this.props.branchDict} showSearch />
            </SearchFieldForm>
            <SearchFieldForm name="promotionInfoStatus">
              <Select placeholder="券状态" dict={auditStatusDict} />
            </SearchFieldForm>
            <SearchFieldForm name="continuityType">
              <Select placeholder="连报条件" dict={continuityDict} />
            </SearchFieldForm>
            <SearchFieldForm name="superPosition">
              <Select placeholder="是否叠加" dict={superpositionDict} />
            </SearchFieldForm>
            <SearchFieldForm name="buyStartTime" className={styles.datePickerWrap}>
              <DatePicker
                placeholder="买券开始时间"
                format={rangeFormat}
                showTime={{
                  defaultValue: moment(defaultBeginTime, timeFormat),
                }}
                showToday={false}
              />
            </SearchFieldForm>
            <SearchFieldForm name="buyEndTime" className={styles.datePickerWrap}>
              <DatePicker
                placeholder="买券结束时间"
                format={rangeFormat}
                showTime={{ defaultValue: moment(defaultEndTime, timeFormat) }}
                showToday={false}
              />
            </SearchFieldForm>
            <SearchFieldForm name="startDate" className={styles.datePickerWrap}>
              <DatePicker
                placeholder="有效开始时间"
                format={rangeFormat}
                showTime={{
                  defaultValue: moment(defaultBeginTime, timeFormat),
                }}
                showToday={false}
              />
            </SearchFieldForm>
            <SearchFieldForm name="endDate" className={styles.datePickerWrap}>
              <DatePicker
                placeholder="有效结束时间"
                format={rangeFormat}
                showTime={{ defaultValue: moment(defaultEndTime, timeFormat) }}
                showToday={false}
              />
            </SearchFieldForm>
          </SearchFilterForm>
        }
      >
        {this.state.list.length > 0 ? (
          <Deck
            render={couponCardRender({
              actions: this.cardAction,
              btnAuth: this.props.btnAuth,
            })}
            dataSource={this.state.list}
            pagination={{
              total: this.state.total,
              current: this.state.pageCurrent,
              size: this.state.pageSize,
              onChange: this.changePage,
            }}
            fixedBodyHeight={true}
          />
        ) : (
          <NoDataPlus />
        )}
        <CouponDetailModal
          visible={this.state.detailConfig.modalVisible}
          mode={this.state.detailConfig.mode}
          type={this.state.detailConfig.productType}
          id={this.state.detailConfig.couponId}
          onSuccess={async () => {
            this.setState({
              loading: true,
              detailConfig: {
                modalVisible: false,
                mode: 'detail',
                couponId: '',
                productType: '',
              },
            });

            await new Promise(this.loadList);

            this.setState({
              loading: false,
            });
          }}
          onCancel={() => {
            this.setState({
              detailConfig: {
                modalVisible: false,
                mode: 'detail',
                couponId: '',
                productType: '',
              },
            });
          }}
        />
        <ShareQRCode
          title={this.shareQRCodeData.showScope === SHOW_SCOPE_SHAONIANGONG_MINI_PROGRAM ? '' : '分享券'}
          shareTitle={this.shareQRCodeData.shareTitle}
          shareText={this.shareQRCodeData.shareText || 'no data'}
          fileName="分享券二维码"
          visible={this.state.shareQRCodeVisible}
          onCancel={this.switchShareQrCode}
          className={this.shareQRCodeData.showScope === SHOW_SCOPE_SHAONIANGONG_MINI_PROGRAM ? styles.qrcode : ''}
        />
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const btnAuth = state.account.btnAuths;
  const isO2OAddingEnabled = btnAuth.indexOf('DEDUCTION_COUPON_O2O_ADD') > -1;
  const isO2MAddingEnabled = btnAuth.indexOf('DEDUCTION_COUPON_O2M_ADD') > -1;
  const isClassAddingEnabled = btnAuth.indexOf('DEDUCTION_COUPON_CLASS_ADD') > -1;

  return {
    branchDict: state.dict.branch,
    btnAuth,
    isO2OAddingEnabled,
    isO2MAddingEnabled,
    isClassAddingEnabled,
    isAddingEnabled: isO2OAddingEnabled || isO2MAddingEnabled || isClassAddingEnabled,
    pageParams: state.main.pageParams[PAGE_ID] || {},
  };
};

export default connect(mapStateToProps)(DeductionCouponMgmtEntrance);
