import React from 'react';
import { connect } from 'dva';
import moment from 'moment';
import * as Api from 'SERVICE/preferential';
import { Page, SearchFilterForm, SearchFieldForm, Input, Select, DateRangePicker, Button, List } from 'COMPONENT';
import { payStatus, useStatus } from './constant';
import { downloadDirect } from 'UTIL/file';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

const { COL_WIDTH_XS, COL_WIDTH_S } = List;
class DepositCoupon extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      name: '定金券使用',
      loading: false,
      branchList: [], // 分公司
      dataList: [],
      campus: [],
      /** 搜索条件 */
      searchParams: {},
      /** 分页 */
      pageNum: 1,
      pageSize: 10,
      total: 0,
    };

    this.export = this.export.bind(this);
    this.search = this.search.bind(this);
    this.initDict = this.initDict.bind(this);
    this.pickBranchDict = this.pickBranchDict.bind(this);

    this.columns = [
      { title: '分公司', dataIndex: 'branchName', width: COL_WIDTH_S },
      { title: '学习中心', dataIndex: 'campusName', width: COL_WIDTH_S },
      { title: '建券员工', dataIndex: 'createUserName', width: COL_WIDTH_XS },
      { title: '金额', dataIndex: 'purchaseOrderAmount', width: COL_WIDTH_XS },
      { title: '客户手机号', dataIndex: 'customerContact', width: COL_WIDTH_XS },
      {
        title: '建券时间',
        dataIndex: 'purchaseOrderCreateTime',
        width: 75,
        render: (time) => moment(time).format('YYYY-MM-DD HH:mm:ss'),
      },
      {
        title: '使用状态',
        dataIndex: 'couponStatus',
        width: COL_WIDTH_XS,
        render: (txt) => {
          const { text } = useStatus.find((status) => status.val == txt) || { text: '' };
          return text;
        },
      },
      {
        title: '使用/退款时间',
        dataIndex: 'finishedTime',
        key: 'finishedTime',
        width: 80,
        render: (time) => {
          return !time ? '-' : moment(time).format('YYYY-MM-DD HH:mm:ss');
        },
      },
      { title: '关联合同编号', dataIndex: 'contractId', key: 'contractId', width: COL_WIDTH_S },
      {
        title: '支付状态',
        dataIndex: 'payStatus',
        width: COL_WIDTH_XS,
        render: (txt) => {
          const { text } = payStatus.find((status) => status.val == txt) || { text: '' };
          return text;
        },
      },
      {
        title: '关联学生',
        dataIndex: 'assocStudentNames',
        width: COL_WIDTH_S,
        render: (students) => {
          return students && students instanceof Array ? students.join('，') : '';
        },
      },
      {
        title: '关联学生ID',
        dataIndex: 'assocStudentIds',
        width: COL_WIDTH_S,
        render: (studentIds) => {
          return studentIds && studentIds instanceof Array ? studentIds.join('，') : '';
        },
      },
      {
        title: '学生年级',
        dataIndex: 'assocStudentGradeNames',
        width: COL_WIDTH_S,
        render: (assocStudentGradeNames) => {
          return assocStudentGradeNames && assocStudentGradeNames instanceof Array
            ? assocStudentGradeNames.join('，')
            : '';
        },
      },
      {
        title: '意向使用年级',
        dataIndex: 'gradeName',
        width: COL_WIDTH_S,
      },
      { title: '备注', dataIndex: 'purchaseOrderRemark' },
    ];

    this.formRef = null;
  }

  async init() {
    await new Promise(this.initDict);
    await this.loadList({ pageSize: 10, pageNum: 1 });
  }

  componentDidMount() {
    this.init();
  }
  // 导出
  async export() {
    const { resultCode, data, resultMessage } = await Api.pageDepositOnlyOrderExport({
      ...this.state.searchParams,
      pageNum: 1,
      pageSize: 10000,
      resourceId: 'expositCoupon',
      beginTime: new Date(this.state.searchParams.beginTime + ' 00:00:00').getTime() || '',
      endTime: new Date(this.state.searchParams.endTime + ' 23:59:59').getTime() || '',
    });
    if (resultCode === 0) {
      if (data) {
        downloadDirect(data, data.split('/').reverse()[0]);
      } else {
        showMessage('导出列表无数据,请重新选择筛选项!', 'error');
      }
    } else {
      showMessage(resultMessage || '导出失败', 'error');
    }
  }
  search(params) {
    if (params.dateRange) {
      const [start, end] = Array.from(params.dateRange || ['', '']);
      delete params.dateRange;
      params.beginTime = start;
      params.endTime = end;
    }
    this.setState(
      {
        searchParams: params,
      },
      () => {
        this.loadList({ pageNum: 1, pageSize: 10 });
      },
    );
  }

  /**
   * 选择分公司，过滤学习中心
   * @param {string[]} branchIds
   */
  async pickBranchDict(branchId) {
    this.formRef.setFieldsValue({
      campusId: '',
    });
    const response = await Api.listUserAllCampusByResourceIdAndBranchId({ resourceId: 'expositCoupon', branchId });
    if (response && response.resultCode === 0 && response.data && response.data.length) {
      this.setState(() => {
        return {
          campus: response.data.map((ele) => {
            return {
              text: ele.name,
              val: ele.id,
            };
          }),
          searchParams: {
            branchId,
          },
        };
      });
    } else {
      this.setState(() => {
        return {
          campus: [],
          searchParams: {
            branchId,
          },
        };
      });
    }
  }
  // 获取分公司字典
  async initDict(resolve) {
    const response = await Api.listUserAllBranchByResourceId({ resourceId: 'expositCoupon' });
    if (response && response.resultCode === 0 && response.data && response.data.length) {
      this.setState(
        {
          branchList: response.data.map((ele) => {
            return {
              text: ele.name,
              val: ele.id,
            };
          }),
        },
        () => {
          if (response.data.length === 1) {
            this.setState({
              searchParams: {
                branchId: response.data[0]['id'],
              },
            });
          }
        },
      );
    }
    resolve();
  }

  // 获取列表数据
  async loadList({ pageNum, pageSize }) {
    this.setState({ loading: true });
    const response = await Api.pageDepositOnlyOrder({
      ...this.state.searchParams,
      pageNum,
      pageSize,
      resourceId: 'expositCoupon',
      beginTime: new Date(this.state.searchParams.beginTime + ' 00:00:00').getTime() || '',
      endTime: new Date(this.state.searchParams.endTime + ' 23:59:59').getTime() || '',
    });
    if (response && response.resultCode === 0) {
      this.setState({
        dataList: (response.data && response.data.list) || [],
        pageNum: pageNum || 1,
        pageSize: pageSize || 10,
        total: (response.data && response.data.total) || 0,
        loading: false,
      });
    }
  }

  render() {
    const { name, loading, branchList, campus, dataList, pageNum, total, pageSize } = this.state;
    return (
      <Page
        name={name}
        fitWindowHeight={true}
        loading={loading}
        route={this.props.route}
        headerToolBar={this.props.showExportButton && <Button onClick={this.export}>导出</Button>}
        subHeader={
          <SearchFilterForm onSearch={this.search} ref={(e) => (this.formRef = e)} truncationNum={6}>
            <SearchFieldForm name="branchId">
              <Select showSearch placeholder="所属分公司" dict={branchList} onChange={this.pickBranchDict} />
            </SearchFieldForm>
            <SearchFieldForm name="campusId">
              <Select showSearch placeholder="所属学习中心" dict={campus} />
            </SearchFieldForm>
            <SearchFieldForm name="customerContact">
              <Input placeholder="客户手机号" />
            </SearchFieldForm>
            <SearchFieldForm name="dateRange">
              <DateRangePicker style={{ width: '250px' }} />
            </SearchFieldForm>

            <SearchFieldForm name="payStatus">
              <Select placeholder="支付状态" dict={payStatus} />
            </SearchFieldForm>

            <SearchFieldForm name="couponStatus">
              <Select placeholder="使用状态" dict={useStatus} />
            </SearchFieldForm>
          </SearchFilterForm>
        }
      >
        <List
          dataSource={dataList}
          columns={this.columns}
          pagination={{
            current: pageNum,
            total: total,
            pageSize,
          }}
          onChange={({ current, pageSize }) => this.loadList({ pageNum: current, pageSize })}
        />
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const btnAuths = state.account.btnAuths;
  const showExportButton = btnAuths.includes('BTN_EXPOSIT_COUPON');

  return {
    branchCampusTree: state.dict.branchCampusTree, // 分公司-学习中心字典
    showExportButton, // 导出按钮权限
  };
};

export default connect(mapStateToProps)(DepositCoupon);
