import React from 'react';
import PropType from 'prop-types';
import { connect } from 'dva';
import style from './styles.module.less';
import {
  Page,
  Button,
  Form,
  Input,
  Select,
  Radio,
  TextArea,
  DropdownMenu,
  InputNumber,
  IconButton,
  TextLink,
  Checkbox,
  Icon,
} from 'COMPONENT';
import { Row, Col, DatePicker, Tooltip } from 'antd';
import { loginUser, runFunc, getKeyValArrByFilter, errorsHandle, isInputNumber, deleteKeyByFilter } from 'UTIL/common';
import {
  limitScopeDict,
  userRangeDict,
  needToReceiveDict,
  superpositionDict,
  periodValidDict,
  PERIOD_VALID_REGULAR,
  PERIOD_VALID_RELATIVE,
  continuityDict,
  BOSS,
  INSIGHT_MARKET,
  promotionVarietyDict,
  useScopeDict,
  showScopeDict,
  LIMIT_SCOPE_INFINITE,
  LIMIT_SCOPE_FINITE,
  promotionVarietyMapProductType,
  NONE,
  SUM_AUT,
  WIN_SPR,
  quarterTextCfg,
  ruleTextCfg,
  COUPON,
  COUPON_FULL_SALE,
  timeFormat,
  rangeFormat,
  NEED_RECEIVE,
  NO_NEED_RECEIVE,
  COUPON_FULL_SALE_NORMAL,
  MONEY,
  PRODUCT_NUMS,
  NOT_ASSIGN_SOMEONE,
  PROMOTION_VARIETY_EXP_CLASS,
} from '../config';
import { rulesFilter, submitDataAdapter, fieldDataAdapter, fieldRuleAdapter } from './utils';
import moment from 'moment';
import ModalSelectPrd from '../ModalSelectPrd';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { getRemarkTemplates, savePromotion, getPromotionInfoById, getPromotionProductById } from 'SERVICE/preferential';
import Title from './components/Title';
import SlotWrap from './components/SlotWrap';
import TipIcon from './components/TipIcon';
import TipWrap from './components/TipWrap';
import NP from 'number-precision';

const FormItem = Form.Item;
const { Group } = Radio;
const { RangePicker } = DatePicker;

const defaultBeginTime = '00:00:00';
const defaultEndTime = '23:59:59';

class RuleItem extends React.Component {
  constructor(props) {
    super(props);
    let initState = {
      num: null,
      price: null,
      subPrice: null,
    };
    if (props.hasOwnProperty('value') && Array.isArray(props.value)) {
      initState.num = props.value[0] || null;
      initState.price = props.value[1] || null;
      initState.subPrice = props.value[2] || null;
    }
    this.state = initState;
  }

  static propTypes = {
    addDisabled: PropType.bool,
    delDisabled: PropType.bool,
    addVisible: PropType.bool,
    delVisible: PropType.bool,
    onAdd: PropType.func,
    onDel: PropType.func,
    index: PropType.number.isRequired,
    continuityType: PropType.oneOf([NONE, SUM_AUT, WIN_SPR]),
  };

  static defaultProps = {
    addDisabled: false,
    delDisabled: false,
    addVisible: true,
    delVisible: true,
    continuityType: NONE,
  };

  static getPrecision(unit) {
    if (unit === MONEY) {
      return 2;
    }
    return 0;
  }

  componentDidUpdate(prevProps, prevState) {
    this.shouldResetState(prevProps, prevState);
    this.shouldUpdateState(prevProps, prevState);
  }

  // 连报条件`continuityType`更新时需重置state
  shouldResetState = (prevProps, prevState) => {
    if (prevProps.continuityType !== this.props.continuityType) {
      this.setState({
        num: null,
        price: null,
        subPrice: null,
      });
    }
  };
  // props.value更新时需同步更新state
  shouldUpdateState = (prevProps, prevState) => {
    const prevValue = prevProps.value || [null, null, null];
    const value = this.props.value || [null, null, null];
    if (!(prevValue[0] === value[0] && prevValue[1] === value[1] && prevValue[2] === value[2])) {
      this.setState({
        num: value[0],
        price: value[1],
        subPrice: value[2],
      });
    }
  };

  finishChange = () => {
    const { num, price, subPrice } = this.state;
    runFunc(this.props.onChange, [
      isInputNumber(num) ? num : null,
      isInputNumber(price) ? price : null,
      isInputNumber(subPrice) ? subPrice : null,
    ]);
  };

  handleNumChange = (val) => {
    this.setState({ num: val }, this.finishChange);
  };

  handlePriceChange = (val) => {
    this.setState({ price: val }, this.finishChange);
  };

  handleSubPriceChange = (val) => {
    this.setState({ subPrice: val }, this.finishChange);
  };

  handleAdd = () => {
    runFunc(this.props.onAdd);
  };

  handleDel = () => {
    runFunc(this.props.onDel, this.props.index);
  };

  getQuarterText = (continuityType, index) => {
    const quarterTextArr = continuityType.split('_');
    return quarterTextCfg[quarterTextArr[index]];
  };

  render() {
    const ruleCfg = this.props.ruleCfg;
    const isContinue = this.props.continuityType !== NONE;

    return (
      <div className={style.ruleItem}>
        <span>{ruleTextCfg[ruleCfg.type]} </span>
        <InputNumber
          value={this.state.num}
          onChange={this.handleNumChange}
          min={1}
          precision={0}
          disabled={this.props.disabled}
        />
        <span>
          {' '}
          {isContinue
            ? `${ruleTextCfg[ruleCfg.ruleUnit]}，${this.getQuarterText(this.props.continuityType, 0)}减`
            : `${ruleTextCfg[ruleCfg.ruleUnit]}，减`}{' '}
        </span>
        <InputNumber
          value={this.state.price}
          onChange={this.handlePriceChange}
          min={0}
          precision={2}
          disabled={this.props.disabled}
        />
        <span> {ruleTextCfg[ruleCfg.unit]}</span>
        {isContinue && [
          <span key={0}>，{this.getQuarterText(this.props.continuityType, 1)}减</span>,
          <InputNumber
            value={this.state.subPrice}
            onChange={this.handleSubPriceChange}
            min={0}
            precision={2}
            key={1}
            disabled={this.props.disabled}
          />,
          <span key={2}> 元</span>,
        ]}
        {this.props.addVisible && (
          <IconButton
            type="plus_circle"
            size={20}
            className={style.ruleAction}
            onClick={this.handleAdd}
            disabled={this.props.addDisabled}
          />
        )}
        {this.props.delVisible && (
          <IconButton
            type="minus_circle"
            size={20}
            className={style.ruleAction}
            onClick={this.handleDel}
            disabled={this.props.delDisabled}
          />
        )}
      </div>
    );
  }
}

const ruleItemRender = (props = {}) => <RuleItem {...props} />;

const CouponCondition = (props) => {
  const { list, continuityType, ruleCfg, addVisible, delVisible, onAdd, onDel, rules, disabled, ...restProps } = props;
  const ruleLen = list.length;
  const isMoreThenOne = ruleLen > 1;
  const lastIndex = ruleLen - 1;
  let fieldDecoratorOpt = {};
  if (Array.isArray(rules)) {
    fieldDecoratorOpt = { rules };
  }

  return (
    <div>
      {list.map((item, index) => {
        const isFirst = index === 0;
        return (
          <FormItem
            name={`rules_${index}`}
            fieldDecoratorOpt={fieldDecoratorOpt}
            required
            formItemProps={{
              className: isFirst ? '' : style.formItemNotRequired,
              label: isFirst ? '买满金额' : ' ',
              colon: false,
              labelCol: { span: 4 },
              wrapperCol: { span: 20 },
            }}
            {...restProps}
            key={index}
          >
            {item.render({
              addDisabled: isMoreThenOne && index !== lastIndex,
              addVisible,
              delVisible: delVisible && isMoreThenOne,
              onAdd,
              onDel,
              index,
              continuityType,
              ruleCfg,
              disabled,
            })}
          </FormItem>
        );
      })}
    </div>
  );
};

class AddEdit extends React.Component {
  constructor(props) {
    super(props);

    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.loadTemplateMenus = this.loadTemplateMenus.bind(this);
    this.loadData = this.loadData.bind(this);
    this.handledSave = this.handledSave.bind(this);
    this.submit = this.submit.bind(this);
    this.promotionNameValidator = this.promotionNameValidator.bind(this);
    this.limitScopeValidator = this.limitScopeValidator.bind(this);
    this.rulesValidator = this.rulesValidator.bind(this);
    this.needToReceiveValidator = this.needToReceiveValidator.bind(this);
    this.handleFieldsChange = this.handleFieldsChange.bind(this);
    this.updatePromotionComment = this.updatePromotionComment.bind(this);
    this.addRule = this.addRule.bind(this);
    this.delRule = this.delRule.bind(this);
    this.switchModalSelectPrd = this.switchModalSelectPrd.bind(this);
    this.handleSelect = this.handleSelect.bind(this);
    this.goBackRefresh = this.goBackRefresh.bind(this);

    const params = props.pageParams[`/${props.route.path}`];

    this.couponId = params.couponId || null;
    this.platType = params.platType;
    this.isBossPlat = this.platType === BOSS;
    this.isInsightPlat = this.platType === INSIGHT_MARKET;
    this.isSend = params.isSend;
    this.isNotPeiYouSym = !loginUser.isPeiYouSym();

    this.state = {
      loading: false,
      prdList: [],
      total: 0,
      isCanSelectPrd: false,
      branchId: null,
      productCharacter: null,
      promotionVariety: PROMOTION_VARIETY_EXP_CLASS,
      limitScope: LIMIT_SCOPE_INFINITE,
      userRange: NOT_ASSIGN_SOMEONE,
      needToRecive: NEED_RECEIVE,
      dateType: periodValidDict[0].val,
      continuityType: NONE,
      continuitySubjectVisible: false,
      rulesList: [
        {
          render: ruleItemRender,
          _origin: null,
        },
      ],
      templateMenus: [{ text: '暂无使用说明历史', disabled: true }],
      modalVisible: false,
      modalInitSKey: {},
      modalVariety: null,
    };
    this.formRef = null;
    this.detailStorage = null;

    this.bossRuleCfg = {
      type: COUPON_FULL_SALE_NORMAL,
      ruleUnit: PRODUCT_NUMS,
      unit: MONEY,
    };
    this.insightRuleCfg = {
      type: COUPON_FULL_SALE_NORMAL,
      ruleUnit: MONEY,
      unit: MONEY,
    };
  }

  get drainageDictValue() {
    return ((this.props.productCharacterDict || []).find(({ text }) => text.startsWith('引流')) || {}).val || '';
  }

  componentDidMount() {
    this.init();
  }

  async init() {
    this.setState({ loading: true });
    await new Promise(this.initDict);
    await new Promise(this.loadTemplateMenus);
    if (this.couponId) {
      await new Promise(this.loadData);
    }
    this.setState({ loading: false });
  }

  initDict(resolve) {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['branch', 'productCharacter'],
        callback() {
          resolve();
        },
      },
    });
  }

  async loadTemplateMenus(resolve) {
    const rsp = await getRemarkTemplates(
      {
        promotionKindId: COUPON,
        promotionLittleTypeId: COUPON_FULL_SALE,
        rows: 3,
      },
      true,
    ).catch(errorsHandle());
    if (rsp && rsp.resultCode === 0 && Array.isArray(rsp.data) && rsp.data.length > 0) {
      this.setState({
        templateMenus: rsp.data.map((item) => ({
          text: item,
          onClick: this.updatePromotionComment,
        })),
      });
    }
    resolve();
  }

  async loadData(resolve) {
    let fetchList = [
      getPromotionInfoById({ id: this.couponId }, true),
      getPromotionProductById({ promotionInfoId: this.couponId }, true),
    ];
    const [rspDetail, rspProduct] = await Promise.all(fetchList, true).catch(errorsHandle());
    let payload = {};
    let fieldData = null;

    if (rspDetail && rspDetail.data) {
      this.detailStorage = rspDetail.data;

      fieldData = fieldDataAdapter(rspDetail.data);
      fieldData = deleteKeyByFilter(fieldData, (key) => fieldData[key] === null || fieldData[key] === undefined);

      payload.branchId = fieldData.branchId;
      payload.promotionVariety = fieldData.promotionVariety;
      payload.limitScope = fieldData.limitScope ? fieldData.limitScope.toString() : fieldData.limitScope;
      payload.userRange = fieldData.userRange ? fieldData.userRange.toString() : fieldData.userRange;
      payload.needToRecive = fieldData.needToRecive ? fieldData.needToRecive.toString() : fieldData.needToRecive;
      payload.dateType = fieldData.dateType;
      payload.continuityType = fieldData.continuityType;
      payload.continuitySubjectVisible = fieldData.continuityType !== NONE;
      payload.rulesList = rspDetail.data.ruleVos.map((rule) => ({
        render: ruleItemRender,
        _origin: rule,
      }));
      payload.modalVariety = promotionVarietyMapProductType[fieldData.promotionVariety] || null;

      this.setState(payload, () => {
        // 设置了除rules外的fieldsValue
        this.formRef.setFieldsValue(fieldData);
      });
    }

    if (rspProduct && rspProduct.data) {
      payload.prdList = rspProduct.data || [];
      payload.total = payload.prdList.length;
    }

    this.setState(payload, () => {
      this.formRef.setFieldsValue(fieldRuleAdapter((this.detailStorage || {}).ruleVos || []));
      resolve();
    });
  }

  handledSave() {
    runFunc(this.formRef.submitFields);
  }

  async submit(values) {
    this.setState({ loading: true });
    let apiParams = submitDataAdapter(
      {
        id: this.couponId,
        promotionPlatType: this.platType,
        productVos: this.state.prdList,

        ...values,
      },
      {
        rulesList: this.state.rulesList.map((item) => item._origin),
        ...(this.detailStorage || {}),
      },
    );
    const rsp = await savePromotion(apiParams, true).catch(errorsHandle());
    if (rsp && rsp.resultCode === 0) {
      showMessage(`${apiParams.id ? '修改' : '新增'}满减券成功！`, 'success');
      return this.goBackRefresh();
    }
    this.setState({ loading: false });
  }

  submitFail = (err) => {
    showMessage('请完善好所有信息后再保存！', 'error');
  };

  // 优惠券名称
  promotionNameValidator(rule, value, callback) {
    if (!value || value.length === 0) {
      return callback('满减券名称不能为空');
    }
    if (value.length > 100) {
      return callback('满减券名称字数不能多于100个字');
    }
    callback();
  }
  // 校验产品范围
  limitScopeValidator(rule, value, callback) {
    if (value === LIMIT_SCOPE_FINITE && this.state.total === 0) {
      return callback('限制商品数量不能为0');
    }
    callback();
  }
  // 校验获取方式
  needToReceiveValidator(rule, value, callback) {
    if (this.state.userRange === NOT_ASSIGN_SOMEONE && value === NO_NEED_RECEIVE) {
      return callback('用户范围选择“不指定对象”时只能设为“需领取”');
    }
    callback();
  }
  // 校验连报条件
  rulesValidator(rule, value, callback) {
    if (!Array.isArray(value)) {
      return callback('买满金额不能为空');
    }
    if (this.state.continuitySubjectVisible && value.some((val) => typeof val !== 'number')) {
      return callback('买满金额不能为空');
    }
    if (value.slice(0, 2).some((val) => typeof val !== 'number')) {
      return callback('买满金额不能为空');
    }
    if (rule.ruleUnit === MONEY && value[0] < NP.plus(value[1], value[2] || 0)) {
      return callback('买满金额不能小于优惠金额');
    }
    callback();
  }

  handleFieldsChange(fields) {
    // 所属分公司✔
    if (fields.branchId) {
      let payload = { branchId: fields.branchId.value };
      if (this.state.branchId !== null && this.state.branchId !== fields.branchId.value) {
        payload.prdList = [];
        payload.total = 0;
      }
      this.setState(payload);
    }
    // 券类型✔
    if (fields.promotionVariety) {
      let payload = { promotionVariety: fields.promotionVariety.value };
      if (this.state.promotionVariety !== fields.promotionVariety.value) {
        payload.prdList = [];
        payload.total = 0;
      }
      this.setState(payload);
    }
    // 产品范围✔
    if (fields.limitScope) {
      let payload = { limitScope: fields.limitScope.value };
      if (fields.limitScope.value === LIMIT_SCOPE_INFINITE) {
        payload.prdList = [];
        payload.total = 0;
      }
      this.setState(payload);
    }
    // 用户范围✔
    if (fields.userRange) {
      this.setState({ userRange: fields.userRange.value }, () => {
        this.formRef.validateFields(['needToRecive'], { force: true });
      });
    }
    // 获取方式✔
    if (fields.needToRecive && fields.needToRecive.value !== this.state.needToRecive) {
      this.setState({ needToRecive: fields.needToRecive.value }, () => {
        if (this.state.needToRecive === NO_NEED_RECEIVE) {
          this.formRef.setFieldsValue({ receiveDate: null });
        }
      });
    }
    // 有效期✔
    if (fields.dateType && fields.dateType.value !== this.state.dateType) {
      this.setState({ dateType: fields.dateType.value }, () => {
        let payload = fields.dateType.value === PERIOD_VALID_REGULAR ? { relativePeriod: null } : { date: null };
        this.formRef.setFieldsValue(payload);
      });
    }
    // 连报条件✔
    if (fields.continuityType && fields.continuityType.value !== this.state.continuityType) {
      this.setState(
        {
          continuityType: fields.continuityType.value,
          continuitySubjectVisible: fields.continuityType.value !== NONE,
          rulesList: [
            {
              render: ruleItemRender,
              _origin: null,
            },
          ],
        },
        () => {
          this.formRef.setFieldsValue({ rules_0: null });
        },
      );
    }
  }

  updatePromotionComment({ key }) {
    if (key) {
      const promotionComment = this.state.templateMenus[key].text || null;
      this.formRef.setFieldsValue({ promotionComment });
    }
  }

  addRule() {
    this.setState((prevState) => {
      prevState.rulesList.push({
        render: ruleItemRender,
        _origin: null,
      });
      return { rulesList: prevState.rulesList };
    });
  }

  delRule(index) {
    let newFields = {};

    let rulesFieldArr = getKeyValArrByFilter(this.formRef.getFieldsValue(), rulesFilter);
    rulesFieldArr = rulesFieldArr.filter((item) => item.key !== `rules_${index}`);
    rulesFieldArr.forEach((item, index) => {
      newFields[`rules_${index}`] = item.value;
    });

    this.setState(
      (prevState) => {
        prevState.rulesList.splice(index, 1);
        return { rulesList: prevState.rulesList };
      },
      () => {
        this.formRef.setFieldsValue(newFields);
      },
    );
  }

  switchModalSelectPrd() {
    this.setState((prevState) => {
      if (prevState.modalVisible) {
        this.modalInitSKey = {};
      } else {
        this.modalInitSKey = { organizationId: this.state.branchId };
        if (this.state.total > 0) {
          if (this.isBossPlat) {
            this.modalInitSKey.category = this.state.prdList[0].category;
          } else {
            this.modalInitSKey.type = this.state.prdList[0].type;
          }
        } else if (this.isInsightPlat) {
          this.modalInitSKey.type =
            promotionVarietyMapProductType[this.formRef.getFieldValue('promotionVariety')] || null;
        }
      }
      return { modalVisible: !prevState.modalVisible };
    });
  }

  handleSelect(list) {
    this.setState(
      {
        prdList: list,
        total: list.length,
      },
      () => {
        // 已选产品更新时，触发`limitScope`校验
        this.formRef.validateFields(['limitScope'], { force: true });
      },
    );
    this.switchModalSelectPrd();
  }

  handleSelectProductCharacter = ({ target }) => {
    this.setState({ productCharacter: target.value });
    this.formRef && this.formRef.setFieldsValue({ continuityType: continuityDict[0].val });
  };

  goBackRefresh() {
    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path: '/Voucher',
        refresh: true,
      },
    });
  }

  render() {
    const isFixedPeriod = this.state.dateType === PERIOD_VALID_REGULAR;
    const isRelativePeriod = this.state.dateType === PERIOD_VALID_RELATIVE;
    const isNeedReceive = this.state.needToRecive === NEED_RECEIVE;
    const isCanSelectPrd = this.state.limitScope === LIMIT_SCOPE_FINITE;
    const ruleCfg = this.isBossPlat ? this.bossRuleCfg : this.insightRuleCfg;
    const { productCharacterDict } = this.props;

    return (
      <Page
        loading={this.state.loading}
        route={this.props.route}
        footer={
          <div className={style.pageFooter}>
            <Button onClick={this.handledSave} type="primary">
              保存
            </Button>
          </div>
        }
        fitWindowHeight={false}
      >
        <div className={style.formWrap}>
          <Form
            ref={(c) => (this.formRef = c)}
            onSubmit={this.submit}
            onSubmitFail={this.submitFail}
            onFieldsChange={this.handleFieldsChange}
          >
            <Title>基础属性</Title>
            <FormItem
              name="promotionName"
              rules={[{ required: true, validator: this.promotionNameValidator }]}
              formItemProps={{
                label: '满减券名称',
                colon: false,
                labelCol: { span: 4 },
                wrapperCol: { span: 20 },
              }}
            >
              <Input placeholder="满减券名称" disabled={this.isSend} />
            </FormItem>
            <FormItem
              name="branchId"
              rules={[{ required: true, message: '所属分公司不能为空' }]}
              formItemProps={{
                label: '所属分公司',
                colon: false,
                labelCol: { span: 4 },
                wrapperCol: { span: 20 },
              }}
            >
              <Select
                placeholder="所属分公司"
                dict={this.props.branchDict}
                showSearch
                allowClear={false}
                disabled={this.isSend}
              />
            </FormItem>

            <Title>使用条件</Title>
            {this.isInsightPlat && (
              <FormItem
                name="promotionVariety"
                rules={[{ required: true, message: '券类型不能为空' }]}
                initialValue={promotionVarietyDict[0].val}
                formItemProps={{
                  label: '券类型',
                  colon: false,
                  labelCol: { span: 4 },
                  wrapperCol: { span: 20 },
                }}
              >
                <Group>
                  {promotionVarietyDict.map((item) => (
                    <TipWrap title={item.tip} className={style.radioWrap}>
                      <Radio value={item.val} key={item.val} disabled={this.isSend}>
                        {item.text}
                      </Radio>
                    </TipWrap>
                  ))}
                </Group>
              </FormItem>
            )}
            {this.isBossPlat && (
              <FormItem
                name="limitProductCharacter"
                initialValue={
                  productCharacterDict && productCharacterDict.length ? productCharacterDict[0].val : undefined
                }
                rules={[{ required: true, message: '产品性质不能为空' }]}
                formItemProps={{
                  colon: false,
                  label: '产品性质',
                  labelCol: { span: 4 },
                  wrapperCol: { span: 20 },
                }}
              >
                <Group onChange={this.handleSelectProductCharacter}>
                  {(productCharacterDict || []).map((item) => (
                    <Radio value={item.val} key={item.val} disabled={this.isSend}>
                      {item.text}
                    </Radio>
                  ))}
                </Group>
              </FormItem>
            )}
            {this.isNotPeiYouSym && (
              <FormItem
                name="limitScope"
                rules={[{ required: true, validator: this.limitScopeValidator }]}
                initialValue={limitScopeDict[0].val}
                formItemProps={{
                  label: '产品范围',
                  colon: false,
                  labelCol: { span: 4 },
                  wrapperCol: { span: 20 },
                }}
                slot={
                  isCanSelectPrd ? (
                    <SlotWrap left={320}>
                      <span style={{ marginRight: 16 }}>
                        已选<span className={style.prdNum}>{this.state.total}</span>个产品
                      </span>
                      {this.state.branchId ? (
                        <TextLink onClick={this.switchModalSelectPrd} disabled={this.isSend}>
                          选择产品
                        </TextLink>
                      ) : (
                        <Tooltip title="请先选择分公司" placement="top">
                          <TextLink disabled>选择产品</TextLink>
                        </Tooltip>
                      )}
                    </SlotWrap>
                  ) : null
                }
              >
                <Group>
                  {limitScopeDict.map((item) => (
                    <Radio value={item.val} key={item.val} disabled={this.isSend}>
                      {item.text}
                    </Radio>
                  ))}
                </Group>
              </FormItem>
            )}
            {this.isBossPlat && (
              <FormItem
                name="userRange"
                rules={[{ required: true, message: '用户范围不能为空' }]}
                initialValue={userRangeDict[0].val}
                formItemProps={{
                  label: '用户范围',
                  colon: false,
                  labelCol: { span: 4 },
                  wrapperCol: { span: 20 },
                }}
                slot={<TipIcon title="指定对象时需要发券" />}
              >
                <Group>
                  {userRangeDict.map((item) => (
                    <Radio value={item.val} key={item.val} disabled={this.isSend}>
                      {item.text}
                    </Radio>
                  ))}
                </Group>
              </FormItem>
            )}
            {this.isBossPlat && (
              <FormItem
                name="needToRecive"
                rules={[{ required: true, validator: this.needToReceiveValidator }]}
                initialValue={needToReceiveDict[this.isBossPlat ? 1 : 0].val}
                formItemProps={{
                  label: '获取方式',
                  colon: false,
                  labelCol: { span: 4 },
                  wrapperCol: { span: 20 },
                }}
                slot={<TipIcon title="若选择需要领取，则发券成功后，用户需在报读小程序中领取才可使用" />}
              >
                <Group>
                  {needToReceiveDict.map((item) => (
                    <Radio value={item.val} key={item.val} disabled={this.isSend}>
                      {item.text}
                    </Radio>
                  ))}
                </Group>
              </FormItem>
            )}
            {this.isBossPlat && this.isNotPeiYouSym && (
              <FormItem
                name="superPosition"
                rules={[{ required: true, message: '叠加方式不能为空' }]}
                initialValue={superpositionDict[0].val}
                formItemProps={{
                  label: '叠加方式',
                  colon: false,
                  labelCol: { span: 4 },
                  wrapperCol: { span: 20 },
                }}
                slot={<TipIcon title="是否可与其他满减券叠加使用" />}
              >
                <Group>
                  {superpositionDict.map((item) => (
                    <Radio value={item.val} key={item.val} disabled={this.isSend}>
                      {item.text}
                    </Radio>
                  ))}
                </Group>
              </FormItem>
            )}
            {this.isInsightPlat && (
              <FormItem
                name="useScope"
                rules={[{ required: true, message: '使用平台不能为空' }]}
                initialValue={useScopeDict[0].val}
                formItemProps={{
                  label: '使用平台',
                  colon: false,
                  labelCol: { span: 4 },
                  wrapperCol: { span: 20 },
                }}
              >
                <Group>
                  {useScopeDict.map((item) => (
                    <Radio value={item.val} key={item.val} disabled={this.isSend}>
                      {item.text}
                    </Radio>
                  ))}
                </Group>
              </FormItem>
            )}
            {/*this.isInsightPlat &&
              <FormItem
                name="showScope"
                rules={[{ required: true, message: '展示平台不能为空' }]}
                initialValue={showScopeDict[0].val}
                formItemProps={{
                  label: '展示平台',
                  colon: false,
                  labelCol: { span: 4 },
                  wrapperCol: { span: 20 },
                }}>
                <Group>
                  {
                    showScopeDict.map((item) => (
                      <Radio value={item.val} key={item.val} disabled={this.isSend}>{item.text}</Radio>
                    ))
                  }
                </Group>
              </FormItem>*/}
            <Title>活动规则</Title>
            {this.isBossPlat && (
              <FormItem
                name="receiveDate"
                rules={[{ required: isNeedReceive, message: '领券时间不能为空' }]}
                wrapClassName={isNeedReceive ? '' : style.displayNone}
                formItemProps={{
                  label: '领券时间',
                  colon: false,
                  labelCol: { span: 4 },
                  wrapperCol: { span: 20 },
                }}
              >
                <RangePicker
                  placeholder={['开始时间', '结束时间']}
                  showTime={{
                    hideDisabledOptions: true,
                    defaultValue: [moment(defaultBeginTime, timeFormat), moment(defaultEndTime, timeFormat)],
                  }}
                  format={rangeFormat}
                  size="large"
                />
              </FormItem>
            )}
            <FormItem
              name="dateType"
              rules={[{ required: true, message: '有效期方式不能为空' }]}
              initialValue={periodValidDict[0].val}
              formItemProps={{
                label: '有效期方式',
                colon: false,
                labelCol: { span: 4 },
                wrapperCol: { span: 20 },
              }}
            >
              <Group>
                {periodValidDict.map((item) => (
                  <Radio value={item.val} key={item.val} disabled={this.isSend}>
                    {item.text}
                  </Radio>
                ))}
              </Group>
            </FormItem>
            <FormItem
              name="date"
              rules={[{ required: isFixedPeriod, message: '有效期不能为空' }]}
              wrapClassName={isRelativePeriod ? style.displayNone : ''}
              formItemProps={{
                label: '有效期',
                colon: false,
                labelCol: { span: 4 },
                wrapperCol: { span: 20 },
              }}
            >
              <RangePicker
                placeholder={['开始时间', '结束时间']}
                showTime={{
                  hideDisabledOptions: true,
                  defaultValue: [moment(defaultBeginTime, timeFormat), moment(defaultEndTime, timeFormat)],
                }}
                format={rangeFormat}
                size="large"
                disabled={this.isSend}
              />
            </FormItem>
            <FormItem
              name="relativePeriod"
              rules={[{ required: isRelativePeriod, message: '有效期不能为空' }]}
              wrapClassName={isFixedPeriod ? style.displayNone : ''}
              formItemProps={{
                label: '有效期',
                colon: false,
                labelCol: { span: 4 },
                wrapperCol: { span: 20 },
              }}
              slot={
                <Row className={style.rowPeriodNum}>
                  <Col span={4} />
                  <Col span={4}>
                    <span className={style.prevPeriodText}>领取或发券后</span>
                  </Col>
                  <Col span={4} />
                  <Col span={4}>
                    <span>天有效</span>
                  </Col>
                </Row>
              }
            >
              <InputNumber className={style.inputPeriodNum} min={1} disabled={this.isSend} />
            </FormItem>
            <FormItem
              name="continuityType"
              rules={[{ required: true, message: '连报条件不能为空' }]}
              initialValue={continuityDict[0].val}
              formItemProps={{
                label: '连报条件',
                colon: false,
                labelCol: { span: 4 },
                wrapperCol: { span: 20 },
              }}
              slot={
                this.state.continuitySubjectVisible ? (
                  <SlotWrap left={380}>
                    <Checkbox checked={true} disabled>
                      相同科目连报
                    </Checkbox>
                  </SlotWrap>
                ) : null
              }
            >
              <Group>
                {continuityDict.map((item) => (
                  <Radio
                    value={item.val}
                    key={item.val}
                    disabled={
                      this.isSend ||
                      !(
                        this.isBossPlat &&
                        this.formRef &&
                        this.formRef.getFieldValue('limitProductCharacter') !== this.drainageDictValue
                      )
                    }
                  >
                    {item.text}
                  </Radio>
                ))}
              </Group>
            </FormItem>
            <CouponCondition
              list={this.state.rulesList}
              ruleCfg={ruleCfg}
              continuityType={this.state.continuityType}
              rules={[
                {
                  required: true,
                  validator: this.rulesValidator,
                  ruleUnit: ruleCfg.ruleUnit,
                },
              ]}
              addVisible={this.isBossPlat && !this.isSend}
              delVisible={this.isBossPlat && !this.isSend}
              disabled={this.isSend}
              onAdd={this.addRule}
              onDel={this.delRule}
            />
            <FormItem
              name="promotionComment"
              formItemProps={{
                label: '使用说明',
                colon: false,
                labelCol: { span: 4 },
                wrapperCol: { span: 20 },
              }}
              wrapClassName={style.FortItemComment}
              slot={
                <div className={style.DropdownMenuWrap}>
                  <Row>
                    <Col span={4} />
                    <Col span={8}>
                      <DropdownMenu
                        dropdownClassName={style.dropDownMenu}
                        text="使用说明模板"
                        menus={this.state.templateMenus}
                        hasArrow={true}
                        placement="topLeft"
                      />
                    </Col>
                  </Row>
                </div>
              }
            >
              <TextArea placeholder="请输入满减券使用说明(最多200字)" wordCount={true} maxLength={200} rows={3} />
            </FormItem>
          </Form>
        </div>
        {this.state.modalVisible && (
          <ModalSelectPrd
            platType={this.platType}
            onSubmit={this.handleSelect}
            initList={this.state.prdList}
            initSKey={this.modalInitSKey}
            onClose={this.switchModalSelectPrd}
            productCharacter={this.formRef && this.formRef.getFieldValue('limitProductCharacter')}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    branchDict: state.dict.branch,
    pageParams: state.main.pageParams,
    activeTabId: state.main.activeTabId,
    productCharacterDict: state.dict.productCharacter,
  };
};

export default connect(mapStateToProps)(AddEdit);
