import { getKeyValArrByFilter, isNumber } from 'UTIL/common';
import {
  NONE,
  ruleUintCfgByPlatType,
  MONEY,
  promotionVarietyMapProductType,
  rangeFormat,
  PERIOD_VALID_REGULAR,
  PERIOD_VALID_RELATIVE,
  NEED_RECEIVE,
  BOSS,
} from '../config';
import NP from 'number-precision';
import moment from 'moment';

export function rulesFilter(item) {
  return item ? /rules_/.test(item) : false;
}

export function submitDataAdapter(params, originData) {
  originData = originData || {};
  let result = {
    id: null,
    // 优惠类型
    promotionKindId: 'COUPON',
    // 优惠小类
    promotionLittleTypeId: 'COUPON_FULL_SALE',
    // 优惠小小类
    promotionTypeId: 'COUPON_FULL_SALE_NORMAL',
    // 合同条件
    contractCondition: 'ALL',
    // (暂0)
    firstLevel: 0,
    // 最高金额
    hightAmount: 0,
    // 产品类型限制
    limitProductType: 'ALL',
    // 优惠码(暂空)
    promotionCode: '',
    // 领取限制
    receiveLimit: 'ALL',
    // 使用平台
    usePlat: 'ALL',
    // 备注
    remark: '',
    // 发布数量
    publishNum: originData.publishNum || 0,
    // 使用数量
    useNum: originData.useNum || 0,
    status: originData.status || 0,
    customerCondition: originData.customerCondition || null,
    // 优惠产品
    productVos: [],
    // 优惠规则
    ruleVos: [],
    ...params,
  };

  const isContinue = result.continuityType !== NONE;
  const isBossPlat = result.promotionPlatType === BOSS;
  // 分公司
  if (result.hasOwnProperty('branchId')) {
    result.branchVos = result.branchId
      ? [
          {
            branchId: result.branchId,
            id: 0,
            promotionId: 0,
          },
        ]
      : [];
    // delete result.branchId;
  }
  // 领券时间
  if (result.needToRecive === NEED_RECEIVE && Array.isArray(result.receiveDate)) {
    result.receiveStartDate = result.receiveDate[0].format(rangeFormat);
    result.receiveEndDate = result.receiveDate[1].format(rangeFormat);
  }
  delete result.receiveDate;
  // 有效时间
  if (result.dateType === PERIOD_VALID_REGULAR && result.hasOwnProperty('date')) {
    result.startDate = result.date[0].format(rangeFormat);
    result.endDate = result.date[1].format(rangeFormat);
  }
  delete result.date;
  delete result.dateType;
  // 连报条件
  if (Boolean(result.continuityType)) {
    result.continuitySubjectType = result.continuityType === NONE ? 0 : 1;
  }
  // 产品
  if (result.hasOwnProperty('productVos')) {
    result.productVos = (result.productVos || []).map((item) => {
      let keyName = isBossPlat ? 'productId' : 'activityId';
      let tag = (originData.productVos || []).find((oriPrd) => oriPrd[keyName] === item[keyName]) || item;
      return {
        id: tag.id || null,
        productId: tag.productId || null,
        activityId: tag.activityId || null,
        promotionProductType: promotionVarietyMapProductType[result.promotionVariety] || null,
      };
    });
  }
  // 规则
  let rulesFieldArr = getKeyValArrByFilter(params, rulesFilter);
  if (rulesFieldArr.length > 0) {
    result.ruleVos = rulesFieldArr.map((rule, index) => {
      let tag = originData.rulesList[index] || {};
      return {
        promotionRule: rule.value[0],
        promotion: NP.plus(rule.value[1], rule.value[2] || 0),
        quarterPromotionA: isBossPlat && isContinue ? rule.value[1] : 0,
        quarterPromotionB: isBossPlat && isContinue ? rule.value[2] : 0,
        ruleUnit: ruleUintCfgByPlatType[result.promotionPlatType],
        unit: MONEY,
        id: tag.id || null,
        promotionId: tag.promotionId || null,
        useNum: tag.useNum || 0,
        promotionIds: tag.promotionIds || 0,
        giftVos: tag.giftVos || [],
        ruleName: tag.ruleName || '',
        needToRecive: isBossPlat ? result.needToRecive : null,
      };
    });
    rulesFieldArr.forEach((ruleField) => {
      delete result[ruleField.key];
    });
  }

  return result;
}

export function fieldDataAdapter(data) {
  return {
    promotionName: data.promotionName,
    branchId: data.branchVos[0].branchId,
    promotionVariety: data.promotionVariety,
    limitProductCharacter: data.limitProductCharacter,
    limitScope: isNumber(data.limitScope) ? data.limitScope.toString() : data.limitScope || null,
    userRange: isNumber(data.userRange) ? data.userRange.toString() : data.userRange || null,
    needToRecive: isNumber(data.needToRecive) ? data.needToRecive.toString() : data.needToRecive || null,
    superPosition: isNumber(data.superPosition) ? data.superPosition.toString() : data.superPosition || null,
    receiveDate:
      data.receiveStartDate && data.receiveEndDate
        ? [moment(data.receiveStartDate), moment(data.receiveEndDate)]
        : null,
    dateType: isNumber(data.relativePeriod) && data.relativePeriod > 0 ? PERIOD_VALID_RELATIVE : PERIOD_VALID_REGULAR,
    date: data.startDate && data.endDate ? [moment(data.startDate), moment(data.endDate)] : null,
    relativePeriod: data.relativePeriod,
    continuityType: data.continuityType,
    useScope: isNumber(data.useScope) ? data.useScope.toString() : data.useScope || null,
    showScope: isNumber(data.showScope) ? data.showScope.toString() : data.showScope || null,
    promotionComment: data.promotionComment,
    // rules单独处理
  };
}

export function fieldRuleAdapter(ruleVos) {
  let result = {};
  ruleVos.forEach((rule, index) => {
    result[`rules_${index}`] = [
      rule.promotionRule,
      rule.quarterPromotionB ? rule.quarterPromotionA : rule.promotion,
      rule.quarterPromotionB,
    ];
  });
  return result;
}
