import React from 'react';
import { connect } from 'dva';
import { Popover, ShareQRCode, PopBtnGroups, Modal, TextLink } from 'COMPONENT';
import { Tooltip } from 'antd';
import compStyle from './compStyle.less';
import classNames from 'classnames';
import 'COMPONENT/DropdownMenu/styles.less';
import ModalImportVoucher from '../../ModalImportVoucher';
import CouponIcon from './CouponIcon';
import { peiYouTest, peiYouPre, peiYou, twoTeacherTest, twoTeacherPre, twoTeacher, pre, prd } from 'CONST/buildEnv';
import { superpositionMap } from '../../store/constant';
import { calcBossPlatType, calcIMPlatType, calcLimitScopeFinite } from '../../util';
import ModalViewCouponData from '../../ModalViewCouponData';
import { NEED_RECEIVE, ASSIGN_SOMEONE } from '../../config';

const DashedLine = () => {
  return <div className={compStyle['DashedLine']}> </div>;
};

const Info = (props) => {
  const { label, children } = props;
  const classPrefix = 'Info';
  return (
    <span className={compStyle[classPrefix]}>
      <span className={compStyle[`${classPrefix}-label`]}>{label}：</span>
      <span className={compStyle[`${classPrefix}-text`]}>{children}</span>
    </span>
  );
};

const ButtonGroup = (props) => {
  const { config, data } = props;
  const classPrefix = 'ButtonGroup';
  return (
    <div className={compStyle[classPrefix]}>
      {config.map((item, index) => {
        return item.disabled ? (
          <div className={compStyle[`${classPrefix}-disabled`]} key={index}>
            {item.name}
          </div>
        ) : item.type === 'popover' ? (
          <PopBtnGroups groups={item.groups} data={data} key={index}>
            <div className={compStyle[`${classPrefix}-btn`]} key={index}>
              {item.name}
            </div>
          </PopBtnGroups>
        ) : (
          <div
            className={compStyle[`${classPrefix}-btn`]}
            onClick={() => {
              item.onClick(data);
            }}
            key={index}
          >
            {item.name}
          </div>
        );
      })}
    </div>
  );
};

const BranchListWithPop = (props) => {
  const data = props.data || [];
  const classPrefix = 'BranchListWithPop';
  let checkLen = data.length > 1;
  const popList = data.slice(1);
  return (
    <span>
      {checkLen ? (
        <Popover
          overlayClassName={compStyle[`${classPrefix}-Popover`]}
          maxWidth={200}
          content={popList.map((item, index) => (
            <p className={compStyle[`${classPrefix}-item`]} key={index}>
              {item}
            </p>
          ))}
          getPopupContainer={(ele) => ele.parentNode}
        >
          <span className={compStyle[classPrefix]}>{`${data[0]}`}</span>
        </Popover>
      ) : (
        <span>{data[0]}</span>
      )}
    </span>
  );
};

class MoreInfo extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      arrowUp: false,
    };
    this.switchArrowUp = this.switchArrowUp.bind(this);
    this.handleClick = this.handleClick.bind(this);
  }
  static classPrefix = 'MoreInfo';
  switchArrowUp() {
    this.setState((prevState, prevProps) => ({
      arrowUp: !prevState.arrowUp,
    }));
  }
  handleClick() {
    this.switchArrowUp();
    this.props.onClick();
  }
  render() {
    const { text } = this.props;
    const { arrowUp } = this.state;
    const classPrefix = MoreInfo.classPrefix;
    return (
      <div
        className={classNames(compStyle[classPrefix], arrowUp ? compStyle[`${classPrefix}-open`] : '')}
        onClick={this.handleClick}
      >
        <span className={compStyle[`${classPrefix}-text`]}>{text || '更多'}</span>
        <span className={classNames('DropdownMenu-arrow', arrowUp ? 'DropdownMenu-arrow-up' : '')}> </span>
      </div>
    );
  }
}

class CouponCard extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      moreInfo: false,
      shareQRCodeVisible: false,
      visibleImportVoucher: false, // 导入发券弹窗
      reminder: false, // 导入后提醒框的显示与隐藏
      toggleViewCouponModal: false,
    };
    this.switchMoreInfo = this.switchMoreInfo.bind(this);
    this.btnExcel = this.btnExcel.bind(this);
    this.btnIssue = this.btnIssue.bind(this);
    this.btnEdit = this.btnEdit.bind(this);
    this.btnHistory = this.btnHistory.bind(this);
    this.btnUsedInfo = this.btnUsedInfo.bind(this);
    this.buildBtnConfig = this.buildBtnConfig.bind(this);
  }
  static classPrefix = 'CouponCard';

  switchMoreInfo() {
    this.setState((prevState, prevProps) => ({
      moreInfo: !prevState.moreInfo,
    }));
  }

  btnExcel(coupon) {
    this.props.action({ type: 'Excel', val: coupon });
  }

  btnIssue(coupon) {
    this.props.action({ type: 'Issue', val: coupon });
  }

  btnEdit(coupon) {
    this.props.action({
      type: 'Edit',
      val: coupon.id,
      promotionPlatType: coupon.promotionPlatType,
      isSend: coupon.couponTotal > 0,
    });
  }

  btnHistory(coupon) {
    this.props.action({ type: 'History', val: coupon });
  }

  btnUsedInfo(coupon) {
    this.props.action({ type: 'UsedInfo', val: coupon });
  }

  btnImportHistory = (coupon) => {
    this.props.action({ type: 'ImportingHistory', val: coupon });
  };

  /**
   * 操作按钮
   *
   * @param status
   * @param total
   * @param usedTotal
   * @param needToRecive
   * @param promotionPlatType 平台类型
   * @param userRange
   * @return {Array}
   */
  buildBtnConfig({ status, total, usedTotal, needToRecive, promotionPlatType, userRange }) {
    const { btnAuth = [] } = this.props;

    const isInsightMarket = calcIMPlatType(promotionPlatType);
    // 非已过期都是有效
    const checkStatus = status !== '1';
    let config = [];
    // 常规券-是否指定对象(不指定对象的，不展示`导入发券`、`导入历史`、`发券`)
    const hadIssueAction = isInsightMarket || userRange === Number(ASSIGN_SOMEONE);

    let issueBtn = {
      name: '发券',
      disabled: false,
      onClick: this.btnIssue,
      /* TODO: 暂时屏蔽`按Excel发券`入口(未测试完 20191224) */
      // type: 'popover',
      groups: [
        {
          name: '按Excel发券',
          onClick: this.btnExcel,
          disabled: false,
        },
        {
          name: '按学生名单发券',
          onClick: this.btnIssue,
          disabled: false,
        },
      ],
    };
    let shareBtn = {
      name: '分享',
      onClick: () => this.setState({ shareQRCodeVisible: true }),
      disabled: !btnAuth.includes('VoucherShareButton'),
    };
    let editBtn = {
      name: '编辑',
      onClick: this.btnEdit,
      disabled: false,
    };
    let historyBtn = {
      name: '发券历史',
      onClick: this.btnHistory,
      disabled: false,
    };
    let usedInfoBtn = {
      name: '用券情况',
      onClick: this.btnUsedInfo,
      disabled: false,
    };

    if (btnAuth.indexOf('ISSUE_FULL_SALE_COUPON') === -1) {
      issueBtn.disabled = true;
    }
    if (btnAuth.indexOf('EDIT_FULL_SALE_COUPON') === -1) {
      editBtn.disabled = true;
    }
    if (btnAuth.indexOf('HISTORY_FULL_SALE_COUPON') === -1) {
      historyBtn.disabled = true;
    }
    if (btnAuth.indexOf('USED_INFO_FULL_SALE_COUPON') === -1) {
      usedInfoBtn.disabled = true;
    }

    if (checkStatus) {
      if (needToRecive !== 0 && !isInsightMarket) {
        config.push(shareBtn);
      }

      if (hadIssueAction) {
        config.push({
          name: '导入发券',
          onClick: () => this.setState({ visibleImportVoucher: true }),
          disabled: !btnAuth.includes('VoucherImportVoucherButton'),
        });
      }
    }

    if (hadIssueAction) {
      config.push({
        name: '导入历史',
        onClick: this.btnImportHistory,
        disabled: !btnAuth.includes('VoucherImportHistoryButton'),
      });
    }

    const checkTotal = total > 0;
    const checkUsed = usedTotal > 0;

    if (checkStatus && checkTotal) {
      // 有效/已发券
      config = hadIssueAction ? [...config, issueBtn, historyBtn, editBtn] : [...config, historyBtn, editBtn];
      if (checkUsed) config.push(usedInfoBtn);
    } else if (checkStatus && !checkTotal) {
      // 有效/未发券
      config = hadIssueAction ? [...config, issueBtn, editBtn] : [...config, editBtn];
    } else if (!checkStatus && checkTotal) {
      // 无效/已发券
      config = [...config, historyBtn, editBtn];
      if (checkUsed) config.push(usedInfoBtn);
    } else if (!checkStatus && !checkTotal) {
      // 无效/未发券
      config = [...config, editBtn];
    }

    return config;
  }

  // 分享发券 - 取消按钮
  handleshareQRCodeVisibleCancel = () => {
    this.setState({ shareQRCodeVisible: false });
  };

  // 导入分券 - 导入回调
  handleImportVoucherOk = () => {
    this.setState({ visibleImportVoucher: false, reminder: true });
  };

  // 跳转到导入历史
  handelToImportHistory = () => {
    const { data } = this.props;
    this.setState({ reminder: false });
    this.btnImportHistory(data);
  };

  // 获取分享二维码链接
  getShareLink = () => {
    const {
      data: { id, _origin },
    } = this.props;
    const { branchVos = [] } = _origin;
    let prefix = 'xhmall-test';
    let sys = 'XINGHUO';
    let appId = 'wx3ecb878239508b99';
    let branchId;
    const env = process.env.BUILD_ENV;
    if (env === peiYouTest || env === peiYouPre || env === peiYou) sys = 'ADVANCE';
    if (env === twoTeacherTest || env === twoTeacherPre || env === twoTeacher) sys = 'TWO_TEACHER';
    if (env === pre || env === peiYouPre || env === twoTeacherPre) {
      prefix = 'xhmall-pre';
      appId = 'wxb6f98188c2b39d30';
    }
    if (env === prd || env === peiYou || env === twoTeacher) {
      prefix = 'xhmall';
      appId = 'wx2b1fe6dca1db781e';
    }

    if (Array.isArray(branchVos) && branchVos.length) {
      branchId = branchVos[0].branchId;
    }
    return `https://${prefix}.xiao100.com/mapp/course-registration/h5/views/common/DiscountCenter?couponId=${id}&sys=${sys}&appid=${appId}&branchId=${branchId}`;
  };

  // 产品性质
  getProductCharacterName = (productCharacter) => {
    console.log(this.props.productCharacterDict);
    return ((this.props.productCharacterDict || []).find(({ val }) => val === productCharacter) || {}).text || '-';
  };

  /**
   * 查看产品
   */
  handleViewPromotionProduct = () => {
    this.setState({ toggleViewCouponModal: true });
  };

  /**
   * 关闭产品详情的弹窗
   * */
  closeViewProductModal = () => {
    this.setState({ toggleViewCouponModal: false });
  };

  /**
   * 修改编辑产品弹框 直接调用编辑就可以了
   * */
  editCouponProduct = () => {
    const { data } = this.props;
    this.setState({ toggleViewCouponModal: false });
    this.btnEdit(data);
  };

  render() {
    const { data = {} } = this.props;
    const { moreInfo, shareQRCodeVisible, visibleImportVoucher, reminder, toggleViewCouponModal } = this.state;
    const classPrefix = CouponCard.classPrefix;

    const {
      status,
      name,
      couponUsedNum,
      couponTotal,
      needToRecive,
      continuityName,
      branchName,
      beginDate,
      endDate,
      description,
      couponsInfo,
      superPosition,
      promotionPlatType,
      promotionVarietyName,
      relativePeriod,
      limitProductCharacter,
      receiveStartDate,
      receiveEndDate,
      limitScope,
      productVos,
      showScopeName,
      useScopeName,
      userRange,
    } = data;
    console.log(data);

    const needToReciveConfig = ['不需领取', '需领取'];
    const isBossPlat = calcBossPlatType(promotionPlatType);
    const isInsightPlat = calcIMPlatType(promotionPlatType);
    const isFinite = calcLimitScopeFinite(limitScope);

    return (
      <div className={compStyle[classPrefix]}>
        <div className={compStyle[`${classPrefix}-header`]}>
          <div className={compStyle[`${classPrefix}-header-wrap`]}>
            <div className={compStyle[`${classPrefix}-header-title`]}>
              <div className={compStyle[`${classPrefix}-header-title-main`]}>
                <CouponIcon status={status} promotionPlatType={promotionPlatType} />
                <Tooltip title={name}>
                  <div className={compStyle.couponTitle}>{name}</div>
                </Tooltip>
              </div>
              <div className={compStyle[`${classPrefix}-header-title-vice`]}>
                <span>已用 / 已发</span>
                <span
                  className={compStyle[`${classPrefix}-header-title-vice-num`]}
                >{`${couponUsedNum} / ${couponTotal}`}</span>
              </div>
            </div>
            <div className={compStyle[`${classPrefix}-header-content`]}>
              <div className={compStyle[`${classPrefix}-header-content-text`]}>{couponsInfo}</div>
              <div className={compStyle[`${classPrefix}-header-content-btn`]}>
                <ButtonGroup
                  config={this.buildBtnConfig({
                    status,
                    total: couponTotal,
                    usedTotal: couponUsedNum,
                    needToRecive,
                    promotionPlatType,
                    userRange,
                  })}
                  data={data}
                />
              </div>
            </div>
          </div>
        </div>
        <DashedLine />
        <div className={compStyle[`${classPrefix}-body`]}>
          <div className={compStyle[`${classPrefix}-body-content`]}>
            <Info label="所属分公司">
              <BranchListWithPop data={branchName} />
            </Info>
            {isBossPlat && <Info label="产品性质">{this.getProductCharacterName(limitProductCharacter)}</Info>}
            {isInsightPlat && <Info label="券分类">{promotionVarietyName || '-'}</Info>}
            {isFinite ? (
              <Info label="产品范围">
                <span>限制产品 已选{(productVos && productVos.length) || 0}个产品 </span>
                {productVos && productVos.length > 0 && (
                  <TextLink onClick={this.handleViewPromotionProduct}>查看</TextLink>
                )}
              </Info>
            ) : (
              <Info label="产品范围">不限制产品</Info>
            )}
            {isInsightPlat && <Info label="展示平台">{showScopeName || '-'}</Info>}
            {isInsightPlat && <Info label="使用平台">{useScopeName || '-'}</Info>}
            {isBossPlat && <Info label="用户范围">{['-', '不指定对象', '指定对象'][userRange] || '-'}</Info>}
            {isBossPlat && <Info label="领取方式">{needToReciveConfig[needToRecive] || '-'}</Info>}
            {isBossPlat && <Info label="是否叠加">{superpositionMap[superPosition]}</Info>}
            {isBossPlat && <Info label="连报条件">{continuityName}</Info>}
            {isBossPlat && needToRecive.toString() === NEED_RECEIVE && (
              <Info label="领券时间">
                {receiveStartDate && receiveEndDate ? `${receiveStartDate} - ${receiveEndDate}` : '-'}
              </Info>
            )}
            <Info label="有效期方式">{relativePeriod ? '相对有效期' : '固定有效期'}</Info>
            {relativePeriod ? (
              <Info label="有效期">领取或发券后{relativePeriod}天</Info>
            ) : (
              <Info label="有效期">{`${beginDate} - ${endDate}`}</Info>
            )}
          </div>
          <div className={compStyle[`${classPrefix}-body-btn`]}>
            <MoreInfo text="使用说明" onClick={this.switchMoreInfo} />
          </div>
        </div>
        <div className={classNames(compStyle[`${classPrefix}-footer`], moreInfo ? compStyle['show-more'] : '')}>
          <span>使用说明：</span>
          <span className={compStyle.wordWrap}>{description || '暂无'}</span>
        </div>
        <ShareQRCode
          title="分享券"
          shareText={this.getShareLink()}
          fileName="分享券二维码"
          shareTitle={name}
          visible={shareQRCodeVisible}
          onCancel={this.handleshareQRCodeVisibleCancel}
        />
        {visibleImportVoucher && (
          <ModalImportVoucher
            data={data}
            onCancel={() => this.setState({ visibleImportVoucher: false })}
            onOK={this.handleImportVoucherOk}
          />
        )}
        {reminder && (
          <Modal
            title="导入提醒"
            visible={true}
            widthSize="s"
            onOk={() => {
              this.setState({ reminder: false });
            }}
            onCancel={() => {
              this.setState({ reminder: false });
            }}
            cancelBtnShow={false}
          >
            <div className={compStyle.reminder}>
              文件导入中！可在
              <span onClick={this.handelToImportHistory}>历史导入</span>
              中查看
            </div>
          </Modal>
        )}
        {toggleViewCouponModal && (
          <ModalViewCouponData initData={data} onCancel={this.closeViewProductModal} onEdit={this.editCouponProduct} />
        )}
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  const { btnAuths } = state.account;
  const { productCharacter } = state.dict;
  return {
    btnAuth: btnAuths,
    productCharacterDict: productCharacter,
  };
};

export default connect(mapStateToProps)(CouponCard);
