/**
 * Created by Haolin on 2018/12/11.
 */
import React from 'react';
import { connect } from 'dva';
import { Pagination } from 'antd';
import CouponCard from './CouponCard';
import compStyle from './compStyle.less';
import notDataImg from 'ASSET/imgs/notDataDef.png';

const CouponDeskWrap = (props) => {
  const { children, footer } = props;
  const classPrefix = 'CouponDeskWrap';
  return (
    <div className={compStyle[classPrefix]}>
      <div className={compStyle[`${classPrefix}-body`]} style={{ height: 'calc(100% - 60px)' }}>
        {children}
      </div>
      <div className={compStyle[`${classPrefix}-footer`]}>{footer}</div>
    </div>
  );
};
const NoData = (props) => {
  const classPrefix = 'NoData';
  return (
    <div className={compStyle[classPrefix]}>
      <img className={compStyle[`${classPrefix}-img`]} src={notDataImg} alt="暂无数据" />
      <p className={compStyle[`${classPrefix}-text`]}>{props.text || '暂无数据'}</p>
    </div>
  );
};

class CouponDesk extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      pageCurrent: 1,
      pageSize: 10,
    };
    this.pageSizeOptions = ['10', '20', '50'];
    this.init = this.init.bind(this);
    this.onPageChange = this.onPageChange.bind(this);
    this.onShowSizeChange = this.onShowSizeChange.bind(this);
  }

  static showTotal(total, range) {
    return <span>共 {total} 条记录</span>;
  }
  componentDidMount() {
    this.init();
  }

  componentDidUpdate(prevProps, prevState) {
    this.updatePage();
  }

  updatePage = () => {
    let payload = {};
    if (this.props.pagination.current !== this.state.pageCurrent) {
      payload.pageCurrent = this.props.pagination.current;
    }
    if (this.props.pagination.size !== this.state.pageSize) {
      payload.pageSize = this.props.pagination.size;
    }
    if (Object.keys(payload).length > 0) {
      this.setState(payload);
    }
  };

  init() {
    this.setState({
      pageCurrent: this.props.current || 1,
      pageSize: this.props.size || 10,
    });
  }

  onPageChange(current, size) {
    this.setState({ pageCurrent: current });
    this.props.onChange(current, size);
  }
  onShowSizeChange(current, size) {
    this.setState({
      pageCurrent: 1,
      pageSize: size,
    });
    this.props.onChange(1, size);
  }

  render() {
    const {
      dataSource,
      pagination: { total },
      cardAction,
    } = this.props;
    const { pageCurrent, pageSize } = this.state;

    return (
      <CouponDeskWrap
        footer={
          dataSource.length > 0 ? (
            <Pagination
              defaultCurrent={1}
              current={pageCurrent}
              pageSize={pageSize}
              total={total}
              showQuickJumper={true}
              showTotal={CouponDesk.showTotal}
              onChange={this.onPageChange}
              pageSizeOptions={this.pageSizeOptions}
              showSizeChanger
              onShowSizeChange={this.onShowSizeChange}
            />
          ) : (
            ''
          )
        }
      >
        {dataSource.length > 0 ? (
          dataSource.map((item, index) => {
            return <CouponCard data={item} key={index} action={cardAction} />;
          })
        ) : (
          <NoData />
        )}
      </CouponDeskWrap>
    );
  }
}

const mapStateToProps = (state) => {
  return {};
};

export default connect(mapStateToProps)(CouponDesk);
