import React from 'react';
import { connect } from 'dva';
import style from './index.less';
import { storeName, importTemplateUrl } from '../store/constant';
import Page from 'COMPONENT/Page';
import List from 'COMPONENT/List';
import Wrapper from 'COMPONENT/Wrapper';
import Button from 'COMPONENT/Button';
import TextLink from 'COMPONENT/TextLink';
import ChooseFileButton from 'PAGE/personnelManage/TeacherIntroduction/ChooseFileButton';
import file from 'UTIL/file';

const { COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;

const classPrefix = 'Excel';

class Excel extends React.Component {
  constructor(props) {
    super(props);
    this.init = this.init.bind(this);
    this.loadData = this.loadData.bind(this);
    this.updatePage = this.updatePage.bind(this);
    this.changePage = this.changePage.bind(this);
    this.importCoupon = this.importCoupon.bind(this);
    this.downloadFail = this.downloadFail.bind(this);
    this.exportFailExcel = this.exportFailExcel.bind(this);
    this.state = {
      name: '按Excel发券',
      loading: false,
    };
    this.columns = [
      { title: '文件名称', dataIndex: 'uploadFileName', key: 'uploadFileName', width: COL_WIDTH_XL },
      { title: '导入状态', dataIndex: 'status', key: 'status', width: COL_WIDTH_M },
      { title: '成功数量', dataIndex: 'successNum', key: 'successNum', width: COL_WIDTH_M },
      { title: '未匹配数量', dataIndex: 'failNum', key: 'failNum', width: COL_WIDTH_M, render: this.downloadFail },
      { title: '上传者', dataIndex: 'createUserName', key: 'createUserName', width: COL_WIDTH_M },
      { title: '上传时间', dataIndex: 'createTime', key: 'createTime', width: COL_WIDTH_L },
    ];
  }

  componentDidMount() {
    this.init();
  }

  async init() {
    this.setState({ loading: true });
    await new Promise(this.loadData);
    this.setState({ loading: false });
  }

  loadData(resolve) {
    this.props.dispatch({
      type: `${storeName}/loadImportHistoryData`,
      payload: {
        callback() {
          resolve();
        },
      },
    });
  }

  updatePage(pageCurrent, pageSize) {
    let payload = {};
    if (pageSize && pageSize !== this.props.pageSize) {
      payload.importHistoryPageCurrent = 1;
      payload.importHistoryPageSize = pageSize;
    } else {
      payload.importHistoryPageCurrent = pageCurrent;
    }
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload,
    });
  }

  async changePage({ current, pageSize }) {
    this.updatePage(current, pageSize);
    this.setState({ loading: true });
    await new Promise(this.loadData);
    this.setState({ loading: false });
  }

  async importCoupon(file) {
    this.setState({ loading: true });
    await new Promise((resolve) => {
      this.props.dispatch({
        type: `${storeName}/importCoupon`,
        payload: {
          file,
          callback() {
            resolve();
          },
        },
      });
    });
    this.setState({ loading: false });
  }

  static downloadTemplate() {
    file.downloadDirect(importTemplateUrl);
  }

  downloadFail(data, record, index) {
    return (
      <div>
        <span>{data}</span>
        {data > 0 && (
          <TextLink
            onClick={() => {
              this.exportFailExcel(record);
            }}
            style={{ marginLeft: 8 }}
          >
            导出
          </TextLink>
        )}
      </div>
    );
  }

  async exportFailExcel(record) {
    this.setState({ loading: true });
    await new Promise((resolve) => {
      this.props.dispatch({
        type: `${storeName}/exportFailExcel`,
        payload: {
          id: record.id,
          fileName: record.uploadFileName,
          callback() {
            resolve();
          },
        },
      });
    });
    this.setState({ loading: false });
  }

  render() {
    return (
      <Page
        name={this.state.name}
        fitWindowHeight={true}
        loading={this.state.loading}
        route={this.props.route}
        subHeader={
          <Wrapper
            header={
              <div className={style[`${classPrefix}-btnGroups`]}>
                <ChooseFileButton
                  btnText="导入Excel"
                  accept="application/vnd.openxmlformats-officedocument.spreadsheetml.sheet,application/vnd.ms-excel"
                  fileType={['excel']}
                  onChange={this.importCoupon}
                />
                <Button onClick={Excel.downloadTemplate}>下载模板</Button>
              </div>
            }
          >
            <div className={style[`${classPrefix}-title`]}>导出历史</div>
          </Wrapper>
        }
      >
        <List
          columns={this.columns}
          dataSource={this.props.list}
          pagination={{
            total: this.props.total,
            pageSize: this.props.pageSize,
            current: this.props.pageCurrent,
          }}
          onChange={this.changePage}
        />
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    list: state[storeName].importHistoryList,
    total: state[storeName].importHistoryTotal,
    pageSize: state[storeName].importHistoryPageSize,
    pageCurrent: state[storeName].importHistoryPageCurrent,
  };
};

export default connect(mapStateToProps)(Excel);
