/**
 * 导入历史页面
 *  */
import React, { Component } from 'react';
import { Page, List } from 'COMPONENT';
import { connect } from 'dva';
import { Row, Tooltip } from 'antd';
import style from './index.less';
import { download } from 'UTIL/file';
import { storeName } from '../store/constant.js';

const reload = require('ASSET/imgs/reload.svg');
const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L } = List;

class ImportingHistoryPage extends Component {
  state = {
    listLoading: false,
    pagination: {
      current: 1,
      pageSize: 10,
    },
  };

  componentDidMount() {
    this.initList();
  }

  // 初始化列表
  initList = (params = {}) => {
    const {
      coupon: { id },
    } = this.props;
    const {
      pagination: { current, pageSize },
    } = this.state;
    const _params = {
      promotionInfoId: id,
      page: params.current || current,
      rows: params.pageSize || pageSize,
      callback: () => {
        this.setState({ listLoading: false });
      },
    };
    this.setState({ listLoading: true });
    this.props.dispatch({ type: `${storeName}/findPageByImportCoupon`, payload: _params });
  };

  // 翻页
  changePage = ({ current, pageSize }) => {
    this.setState({
      pagination: {
        current,
        pageSize,
      },
    });
    this.initList({ current, pageSize });
  };

  // 时间搓转化函数
  timestampToTime(timestamp) {
    if (!timestamp) return '';
    const date = new Date(timestamp); //时间戳为10位需*1000，时间戳为13位的话不需乘1000
    const Y = date.getFullYear() + '-';
    const M = (date.getMonth() + 1 < 10 ? '0' + (date.getMonth() + 1) : date.getMonth() + 1) + '-';
    const D = (date.getDate() < 10 ? '0' + date.getDate() : date.getDate()) + ' ';
    const h = (date.getHours() < 10 ? '0' + date.getHours() : date.getHours()) + ':';
    const m = (date.getMinutes() < 10 ? '0' + date.getMinutes() : date.getMinutes()) + ':';
    const s = date.getSeconds() < 10 ? '0' + date.getSeconds() : date.getSeconds();
    return Y + M + D + h + m + s;
  }

  columns = [
    {
      title: '序号',
      dataIndex: 'number',
      key: 'number',
      width: COL_WIDTH_S,
      render: (t, r, i) => {
        const { current, pageSize } = this.state.pagination;
        return (current - 1) * pageSize + i + 1;
      },
    },
    { title: '导入文件名', dataIndex: 'fileName', key: 'fileName', width: COL_WIDTH_L },
    {
      title: '导入时间',
      dataIndex: 'createTime',
      key: 'createTime',
      width: COL_WIDTH_M,
      render: (t) => this.timestampToTime(t),
    },
    { title: '操作人', dataIndex: 'createUserName', key: 'createUserName', width: COL_WIDTH_S },
    {
      title: '导入状态',
      dataIndex: 'importStatus',
      key: 'importStatus',
      width: COL_WIDTH_S,
      render: (t) => {
        return ['待导入', '导入中', '导入完成'][t];
      },
    },
    { title: '总条数', dataIndex: 'total', key: 'total', width: COL_WIDTH_S },
    {
      title: '成功条数',
      dataIndex: 'success',
      key: 'success',
      width: COL_WIDTH_S,
      render: (t, r) => r.total - r.fail,
    },
    { title: '失败条数', dataIndex: 'fail', key: 'fail', width: COL_WIDTH_S },
    {
      title: '完成时间',
      dataIndex: 'endTime',
      key: 'endTimeendTime',
      width: COL_WIDTH_M,
      render: (t) => this.timestampToTime(t),
    },
    {
      title: '操作',
      key: 'homAction',
      width: COL_WIDTH_M,
      fixed: 'right',
      render: (t, r) => {
        const params = {
          url: r.resultUrl,
          fileName: '错误报告',
          format: 'xlsx',
        };
        return r.fail > 0 ? (
          <span className={style.list_error_button} onClick={() => download(params)}>
            下载错误报告
          </span>
        ) : null;
      },
    },
  ];

  render() {
    const { route, list, total } = this.props;
    const { loading, listLoading, pagination } = this.state;
    const { pageSize, current } = pagination;

    return (
      <Page name="导入历史" className={style.importingHistory} route={route} fitWindowHeight={true}>
        <Row className={style.title_row}>
          <span>导入结果</span>
          <Tooltip title="刷新">
            <img className={style.reload} src={reload} onClick={this.initList}></img>
          </Tooltip>
        </Row>
        <List
          dataSource={list}
          columns={this.columns}
          loading={!loading && listLoading}
          pagination={{
            current,
            pageSize,
            total,
          }}
          onChange={this.changePage}
        />
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const { coupon } = state.main.pageParams['/Voucher/ImportingHistory'];
  const { importingHistoryList, importingHistoryListTotal } = state[storeName];
  return {
    coupon,
    list: importingHistoryList || [],
    total: importingHistoryListTotal,
  };
};

export default connect(mapStateToProps)(ImportingHistoryPage);
