/**
 * Created by Haolin on 2018/12/12.
 */
import React from 'react';
import { connect } from 'dva';
import { Popconfirm, Row, Col, Icon, Menu, Dropdown } from 'antd';
import { Page, List, TextLink, Button } from 'COMPONENT';
import Search from './Search';
import { storeName } from '../store/constant.js';

const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;
// dict_couponUsedStatus 中的正常状态值为 '0'
const NOT_USE_COUPON_STATUS_VAL = [0, 5];
const LinkBtnWithPop = (props) => {
  const { text, title, okText, cancelText, confirm, cancel, disabled } = props;
  const classPrefix = 'LinkBtnWithPop';
  return (
    <div className={classPrefix}>
      <Popconfirm title={title} onConfirm={confirm} onCancel={cancel} okText={okText} cancelText={cancelText}>
        <TextLink fontSize={12} disabled={disabled}>
          {text}
        </TextLink>
      </Popconfirm>
    </div>
  );
};
LinkBtnWithPop.defaultProps = {
  text: '操作',
  title: '确定执行该操作吗？',
  okText: '确定',
  cancelText: '取消',
  confirm: function () {},
  cancel: function () {},
  disabled: false,
};

class History extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      name: '发券历史',
      loading: false,
      listLoading: false,
      pageCurrent: 1,
      pageSize: 20,
    };
    this.columns = [
      { title: '序号', dataIndex: 'number', key: 'number', width: COL_WIDTH_S },
      { title: '学生姓名', dataIndex: 'studentName', key: 'studentName', width: COL_WIDTH_M },
      { title: '客户号码', dataIndex: 'customerContact', key: 'customerContact', width: COL_WIDTH_L },
      { title: '学生编码', dataIndex: 'studentId', key: 'studentId', width: COL_WIDTH_L },
      { title: '分公司', dataIndex: 'branchName', key: 'branchName', width: COL_WIDTH_M },
      { title: '学习中心', dataIndex: 'campusName', key: 'campusName', width: COL_WIDTH_M },
      { title: '年级', dataIndex: 'gradeName', key: 'gradeName', width: COL_WIDTH_M },
      { title: '发券人', dataIndex: 'createUserName', key: 'createUserName', width: COL_WIDTH_M },
      { title: '状态', dataIndex: 'statusStr', key: 'statusStr', width: COL_WIDTH_M },
      { title: '发券时间', dataIndex: 'createTime', key: 'createTime', width: COL_WIDTH_XL },
      {
        title: '操作',
        key: 'homAction',
        width: COL_WIDTH_S,
        fixed: 'right',
        render: (text, record, index) => {
          return NOT_USE_COUPON_STATUS_VAL.includes(record.status) ? (
            <LinkBtnWithPop
              text="撤回"
              title="确定要撤回该满减券吗？"
              confirm={() => {
                this.moreAction('undo', record, index);
              }}
              disabled={this.props.uoDoCouponAuth}
            />
          ) : (
            ''
          );
        },
      },
    ];
    this.init = this.init.bind(this);
    this.loadList = this.loadList.bind(this);
    this.changePage = this.changePage.bind(this);
    this.search = this.search.bind(this);
    this.moreAction = this.moreAction.bind(this);
    this.modalLoading = this.modalLoading.bind(this);
    this.destroy = this.destroy.bind(this);
  }
  static classPrefix = 'History';
  componentDidMount() {
    this.init();
  }
  componentWillUnmount() {
    this.destroy();
  }
  /**
   * 初始化
   * */
  init() {
    this.setState({ loading: true });
    const _self = this;
    this.props.dispatch({
      type: `${storeName}/initHistory`,
      payload: {
        couponId: this.props.coupon.id,
        page: this.state.pageCurrent,
        rows: this.state.pageSize,
        callback() {
          _self.setState({ loading: false });
        },
      },
    });
  }
  /**
   * 加载列表
   * */
  loadList(page, rows) {
    this.setState({ listLoading: true });
    const _self = this;
    this.props.dispatch({
      type: `${storeName}/loadHistoryList`,
      payload: {
        couponId: this.props.coupon.id,
        page,
        rows,
        callback() {
          _self.setState({ listLoading: false });
        },
      },
    });
  }
  /**
   * 翻页
   * */
  changePage({ current, pageSize }) {
    this.setState({
      pageCurrent: current,
      pageSize,
    });
    this.loadList(current, pageSize);
  }
  /**
   * 搜索
   * */
  search() {
    this.loadList(1, this.state.pageSize);
  }

  moreAction(type, record, index) {
    this.setState({ loading: true });
    const _self = this;
    this.props.dispatch({
      type: `${storeName}/undoCoupon`,
      payload: {
        id: record.id,
        index,
        callback() {
          _self.loadList(_self.state.pageCurrent, _self.state.pageSize);
          _self.setState({ loading: false });
        },
      },
    });
  }

  modalLoading(params) {
    this.setState({ loading: params });
  }

  destroy() {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        historyList: [],
        historyTotal: 0,
      },
    });
  }

  // 按钮 - 导出
  handleMenuClick = async ({ key }) => {
    const params = {
      promotionInfoId: this.props.coupon.id,
      isEncryption: key === '1',
      ...this.props.search,
    };
    this.props.dispatch({ type: `${storeName}/exportPromotionSendCoupon`, payload: params });
  };

  render() {
    const { route, list, total, btnAuths } = this.props;
    const { name, loading, listLoading, pageCurrent, pageSize } = this.state;
    const classPrefix = History.classPrefix;

    const menu = (
      <Menu onClick={this.handleMenuClick}>
        <Menu.Item key="1" disabled={!btnAuths.includes('PhoneNumberEncryptionExport')}>
          手机加密导出
        </Menu.Item>
        <Menu.Item key="2" disabled={!btnAuths.includes('PhoneNumberNOEncryptionExport')}>
          手机号不加密导出
        </Menu.Item>
      </Menu>
    );

    return (
      <Page
        name={name}
        className={classPrefix}
        fitWindowHeight={true}
        loading={loading}
        route={route}
        subHeader={
          <Row>
            <Col span={22}>
              <Search classPrefix={classPrefix} handleSearch={this.search} handleLoading={this.modalLoading} />
            </Col>
            <Col span={2}>
              <Dropdown overlay={menu}>
                <Button size="large">
                  导出
                  <Icon type="down" />
                </Button>
              </Dropdown>
            </Col>
          </Row>
        }
      >
        <List
          dataSource={list}
          columns={this.columns}
          loading={!loading && listLoading}
          pagination={{
            current: pageCurrent,
            pageSize,
            total,
          }}
          onChange={this.changePage}
        />
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const { coupon } = state.main.pageParams['/Voucher/History'];
  const { btnAuths } = state.account;
  const {
    historyList,
    historyTotal,
    sKey_history_stdName,
    sKey_history_beginDate,
    sKey_history_endDate,
    sKey_history_branch,
    sKey_history_campus,
    sKey_history_status,
  } = state[storeName];
  const uoDoCouponAuth = state.account.btnAuths.indexOf('UN_DO_FULL_SALE_COUPON') === -1;
  return {
    coupon,
    btnAuths,
    list: historyList,
    total: historyTotal,
    uoDoCouponAuth,
    search: {
      name: sKey_history_stdName,
      startTime: sKey_history_beginDate,
      endTime: sKey_history_endDate,
      branchId: sKey_history_branch,
      campusId: sKey_history_campus,
      status: sKey_history_status,
    },
  };
};

export default connect(mapStateToProps)(History);
