/**
 * Created by Haolin on 2018/12/12.
 */
import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import List from 'COMPONENT/List';
import Button from 'COMPONENT/Button';
import { storeName } from '../store/constant.js';
import ChooseStdSvg from 'ASSET/imgs/ChooseStd.svg';
import ModalChooseClass from '../ModalChooseClass';
import ModalChooseStd from '../ModalChooseStd';
import ModalPublish from '../ModalPublish';
import compStyle from './compStyle.less';
import classNames from 'classnames';
import Modal from 'COMPONENT/Modal';
import jslog from 'SRC/jslog';

const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_XL } = List;
const NoStdList = (props) => {
  const { onClassBtn, onStdBtn } = props;
  const classPrefix = 'NoStdList';
  return (
    <div className={compStyle[classPrefix]}>
      <img className={compStyle[`${classPrefix}-img`]} src={ChooseStdSvg} alt="请选择学生" />
      <p className={compStyle[`${classPrefix}-text`]}>请从以下两种途径添加学生</p>
      <Button className={compStyle[`${classPrefix}-btn`]} onClick={onClassBtn} type="primary">
        选择班级
      </Button>
      <Button className={compStyle[`${classPrefix}-btn`]} onClick={onStdBtn} type="default">
        选择学生
      </Button>
    </div>
  );
};
const HeaderWrap = (props) => {
  const { front, end } = props;
  const classPrefix = 'HeaderWrap';
  return (
    <div className={compStyle[classPrefix]}>
      <div className={compStyle[`${classPrefix}-front`]}>{front}</div>
      <div className={compStyle[`${classPrefix}-end`]}>{end}</div>
    </div>
  );
};
const ButtonGroup = (props) => {
  const { group } = props;
  const classPrefix = 'ButtonGroup';
  return (
    <div className={compStyle[classPrefix]}>
      {group.map((item, index) => {
        return item.disabled ? (
          <button className={classNames(compStyle[`${classPrefix}-btn`], compStyle['disabled'])} key={index}>
            {item.text}
          </button>
        ) : (
          <button className={compStyle[`${classPrefix}-btn`]} onClick={item.handleClick} key={index}>
            {item.text}
          </button>
        );
      })}
    </div>
  );
};
ButtonGroup.defaultProps = {
  disabled: false,
};
const ColText = (props) => {
  const { disabled, children } = props;
  const classPrefix = 'ColText';
  return <div className={disabled ? compStyle[`${classPrefix}-disabled`] : ''}>{children || ''}</div>;
};
const PhoneNum = (props) => {
  const { num } = props;
  return <span>{num ? num.toString().replace(/(\d{3})\d{4}(\d{4})/, '$1****$2') : '无'}</span>;
};

class Issue extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      name: '发券',
      loading: false,
      listLoading: false,
      toggle_chooseClassModal: false,
      toggle_chooseStdModal: false,
      toggle_publishModal: false,
      pageCurrent: 1,
      pageSize: 20,
      actionStorage: {
        // key: studentId, val: Boolean
      },
    };
    this.buildColumns = this.buildColumns.bind(this);
    this.init = this.init.bind(this);
    this.switchChooseClassModal = this.switchChooseClassModal.bind(this);
    this.switchChooseStdModal = this.switchChooseStdModal.bind(this);
    this.chooseClass = this.chooseClass.bind(this);
    this.chooseStd = this.chooseStd.bind(this);
    this.loadList = this.loadList.bind(this);
    this.changePage = this.changePage.bind(this);
    this.moreAction = this.moreAction.bind(this);
    this.deleteStd = this.deleteStd.bind(this);
    this.realDeleteStd = this.realDeleteStd.bind(this);
    this.undoDeleteStd = this.undoDeleteStd.bind(this);
    this.switchPublishModal = this.switchPublishModal.bind(this);
    this.submit = this.submit.bind(this);
    this.goBack = this.goBack.bind(this);
    this.destroy = this.destroy.bind(this);
  }
  static classPrefix = 'Issue';
  componentDidMount() {
    this.init();
  }
  componentWillUnmount() {
    this.destroy();
  }
  buildColumns(storage) {
    return [
      { title: '序号', dataIndex: 'number', key: 'number', width: COL_WIDTH_S },
      {
        title: '学生姓名',
        dataIndex: 'studentName',
        key: 'studentName',
        width: COL_WIDTH_M,
        render: (text, record, index) => <ColText disabled={storage[record.studentId]}>{text}</ColText>,
      },
      {
        title: '学生编号',
        dataIndex: 'studentId',
        key: 'studentId',
        width: COL_WIDTH_M,
        render: (text, record, index) => <ColText disabled={storage[record.studentId]}>{text}</ColText>,
      },
      {
        title: '客户号码',
        dataIndex: 'contact',
        key: 'contact',
        width: COL_WIDTH_M,
        render: (text, record, index) => (
          <ColText disabled={storage[record.studentId]}>
            <PhoneNum num={text} />
          </ColText>
        ),
      },
      {
        title: '班级名称',
        dataIndex: 'className',
        key: 'className',
        width: COL_WIDTH_XL,
        render: (text, record, index) => <ColText disabled={storage[record.studentId]}>{text}</ColText>,
      },
      {
        title: '操作',
        key: 'action',
        width: COL_WIDTH_S,
        fixed: 'right',
        getActionConfig: (text, record, index) => {
          const check = storage[record.studentId];
          let actionText = check ? '撤回' : '删除';
          let type = check ? 'undoDeleteStd' : 'deleteStd';
          return [
            {
              text: actionText,
              onClick: () => {
                this.moreAction(type, record, index);
              },
            },
          ];
        },
      },
    ];
  }
  /**
   * 初始化
   * */
  init() {
    this.setState({ actionStorage: {} });
  }
  /**
   * 切换选择班级弹窗显示
   * */
  switchChooseClassModal() {
    this.setState((prevState, prevProps) => ({
      toggle_chooseClassModal: !prevState.toggle_chooseClassModal,
    }));
  }
  /**
   * 切换选择学生弹窗显示
   * */
  switchChooseStdModal() {
    this.setState((prevState, prevProps) => ({
      toggle_chooseStdModal: !prevState.toggle_chooseStdModal,
    }));
  }
  /**
   * 完成选择班级
   * */
  chooseClass() {
    this.switchChooseClassModal();
    this.loadList('loadStdListByClass', this.state.pageCurrent, this.state.pageSize);
  }
  /**
   * 完成选择学生
   * */
  chooseStd() {
    this.switchChooseStdModal();
    this.loadList('loadStdListByStd', this.state.pageCurrent, this.state.pageSize);
  }
  /**
   * 加载学生列表
   * */
  loadList(effectsName, page, rows) {
    this.setState({ loading: true });
    const _self = this;
    this.props.dispatch({
      type: `${storeName}/${effectsName}`,
      payload: {
        page,
        rows,
        callback() {
          _self.setState({ loading: false });
        },
      },
    });
  }
  /**
   * 翻页
   * */
  changePage({ current, pageSize }) {
    this.setState({
      pageCurrent: current,
      pageSize,
    });
    let effectsName = '';
    const { classChosenList, stdChosenList } = this.props;
    if (classChosenList.length > 0) {
      effectsName = 'loadStdListByClass';
    } else if (stdChosenList.length > 0) {
      effectsName = 'loadStdListByStd';
    } else {
      return;
    }
    this.loadList(effectsName, current, pageSize);
  }
  /**
   * 列表更多操作
   * */
  moreAction(type, record, index) {
    this[type](record);
  }
  /**
   * 删除学生
   * */
  deleteStd(record) {
    const { total, stdDeleteList } = this.props;
    if (total - 1 === stdDeleteList.length) {
      // 删除到最后一条数据时
      Modal.confirm({
        content: '删除所有之后将清空本次选中的学生！',
        onOk: () => {
          this.destroy();
        },
      });
    } else {
      this.realDeleteStd(record);
    }
  }
  realDeleteStd(record) {
    this.setState((prevState, prevProps) => {
      prevState.actionStorage[record.studentId] = true;
      return {
        actionStorage: prevState.actionStorage,
      };
    });
    this.props.dispatch({
      type: `${storeName}/updateDeleteStd`,
      payload: { stdId: record.studentId },
    });
  }
  /**
   * 撤回删除
   * */
  undoDeleteStd(record) {
    this.setState((prevState, prevProps) => {
      delete prevState.actionStorage[record.studentId];
      return {
        actionStorage: prevState.actionStorage,
      };
    });
    this.props.dispatch({
      type: `${storeName}/updateUndoDeleteStd`,
      payload: { stdId: record.studentId },
    });
  }
  /**
   * 切换发券确认弹窗显示
   * */
  switchPublishModal() {
    this.setState((prevState, prevProps) => ({
      toggle_publishModal: !prevState.toggle_publishModal,
    }));
  }
  /**
   * 发券
   * */
  submit(code) {
    const _self = this;
    jslog.report('click', {
      function_name: '优惠方案>优惠券>满减券>发券>发布',
    });
    this.props.dispatch({
      type: `${storeName}/publishCoupon`,
      payload: {
        couponId: this.props.coupon.id,
        msgCode: code,
        callback() {
          _self.switchPublishModal();
          _self.goBack();
        },
      },
    });
  }
  goBack() {
    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path: '/Voucher',
        refresh: true,
      },
    });
  }
  /**
   * 组件注销前销毁状态存储
   * */
  destroy() {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        classChosenList: [],
        stdChosenList: [],
        stdDeleteList: [],
        stdList: [],
        total_stdList: 0,
      },
    });
  }

  render() {
    const { route, list, total, classChosenList, stdChosenList, stdDeleteList, coupon } = this.props;
    const {
      name,
      loading,
      listLoading,
      toggle_chooseClassModal,
      toggle_chooseStdModal,
      toggle_publishModal,
      pageCurrent,
      pageSize,
      actionStorage,
    } = this.state;
    const classPrefix = Issue.classPrefix;

    return (
      <Page
        name={name}
        className={classPrefix}
        fitWindowHeight={true}
        loading={loading}
        route={route}
        subHeader={
          list.length > 0 && (
            <HeaderWrap
              front={
                <Button type="primary" onClick={this.switchPublishModal} disabled={total === stdDeleteList.length}>
                  立即发券
                </Button>
              }
              end={
                <ButtonGroup
                  group={[
                    {
                      text: '选择班级',
                      handleClick: this.switchChooseClassModal,
                      disabled: stdChosenList.length > 0,
                    },
                    {
                      text: '选择学生',
                      handleClick: this.switchChooseStdModal,
                      disabled: classChosenList.length > 0,
                    },
                  ]}
                />
              }
            />
          )
        }
      >
        {list.length > 0 ? (
          <List
            dataSource={list}
            columns={this.buildColumns(actionStorage)}
            loading={!loading && listLoading}
            pagination={{
              current: pageCurrent,
              pageSize,
              total,
            }}
            onChange={this.changePage}
          />
        ) : (
          <NoStdList onClassBtn={this.switchChooseClassModal} onStdBtn={this.switchChooseStdModal} />
        )}
        {toggle_chooseClassModal && (
          <ModalChooseClass onConfirm={this.chooseClass} onCancel={this.switchChooseClassModal} />
        )}
        {toggle_chooseStdModal && <ModalChooseStd onConfirm={this.chooseStd} onCancel={this.switchChooseStdModal} />}
        {toggle_publishModal && (
          <ModalPublish
            data={{
              name: coupon.name,
              count: total - stdDeleteList.length,
            }}
            onConfirm={this.submit}
            onCancel={this.switchPublishModal}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const { coupon } = state.main.pageParams['/Voucher/Issue'];
  const { classChosenList, stdChosenList, stdDeleteList, stdList, total_stdList } = state[storeName];
  return {
    coupon,
    classChosenList,
    stdChosenList,
    stdDeleteList,
    list: stdList,
    total: total_stdList,
  };
};

export default connect(mapStateToProps)(Issue);
