/**
 * Created by Haolin on 2018/12/17.
 */
import React from 'react';
import { connect } from 'dva';
import Modal from 'COMPONENT/Modal';
import List from 'COMPONENT/List';
import { storeName, TWO_TEACHER_VAL } from '../store/constant.js';
import Search from './Search';

const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_XL } = List;

class ModalChooseClass extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      name: '选择班级',
      loading: false,
      error: null,
      listLoading: false,
      selectedRowKeys: [],
    };
    this.init = this.init.bind(this);
    this.loadList = this.loadList.bind(this);
    this.search = this.search.bind(this);
    this.confirm = this.confirm.bind(this);
    this.destroy = this.destroy.bind(this);
    this.modalLoading = this.modalLoading.bind(this);
  }
  static classPrefix = 'ModalChooseClass';
  static buildColumns(prdType) {
    let isTwoTc = prdType === TWO_TEACHER_VAL;
    let mainTcTitle = isTwoTc ? '主讲老师' : '辅导老师';
    let auxTcTitle = isTwoTc ? '班主任' : '任课老师';
    return [
      { title: '序号', dataIndex: 'number', key: 'number', width: COL_WIDTH_S },
      { title: '班级名称', dataIndex: 'className', key: 'className', width: COL_WIDTH_XL },
      { title: '期/星期', dataIndex: 'phaseName', key: 'phaseName', width: COL_WIDTH_M },
      { title: mainTcTitle, dataIndex: 'studentManageName', key: 'studentManageName', width: COL_WIDTH_M },
      { title: auxTcTitle, dataIndex: 'teacherName', key: 'teacherName', width: COL_WIDTH_M },
      { title: '上课学生', dataIndex: 'studentNum', key: 'studentNum', width: COL_WIDTH_M },
    ];
  }
  componentDidMount() {
    this.init();
  }
  componentWillUnmount() {
    this.destroy();
  }
  /**
   * 初始化
   * */
  init() {
    this.setState({ loading: true });
    const _self = this;
    this.props.dispatch({
      type: `${storeName}/initModalChooseClass`,
      payload: {
        callback(selectedRowKeys) {
          _self.setState({
            loading: false,
            selectedRowKeys,
          });
        },
        handleErr() {
          _self.setState({
            loading: false,
          });
        },
      },
    });
  }
  /**
   * 加载列表
   * */
  loadList() {
    this.setState({ listLoading: true });
    const _self = this;
    this.props.dispatch({
      type: `${storeName}/loadClassList`,
      payload: {
        callback(selectedRowKeys) {
          _self.setState({
            listLoading: false,
            selectedRowKeys,
          });
        },
        handleErr() {
          _self.setState({
            listLoading: false,
          });
        },
      },
    });
  }
  /**
   * 搜索
   * */
  search() {
    this.loadList();
  }
  /**
   * 确认选择
   * */
  confirm() {
    let classChosenList = this.state.selectedRowKeys.map((item) => ({
      classId: this.props.list[item].classId,
      productType: this.props.productType,
    }));
    this.props.dispatch({
      type: `${storeName}/selectClass`,
      payload: { classChosenList },
    });
    this.props.onConfirm();
  }
  /**
   * 注销前销毁
   * */
  destroy() {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: { class_list: [] },
    });
  }
  //
  modalLoading(params) {
    this.setState({ loading: params });
  }

  render() {
    const { onCancel, productType, list } = this.props;
    const { name, loading, error, listLoading, selectedRowKeys } = this.state;

    return (
      <Modal
        title={name}
        className={ModalChooseClass.classPrefix}
        loading={loading}
        error={error}
        width={1100}
        onCancel={onCancel}
        onOk={this.confirm}
        okBtnDisabled={selectedRowKeys.length === 0}
      >
        <Search
          classPrefix={ModalChooseClass.classPrefix}
          handleSearch={this.search}
          handleLoading={this.modalLoading}
        />
        <List
          dataSource={list}
          columns={ModalChooseClass.buildColumns(productType)}
          loading={!loading && listLoading}
          rowSelection={{
            selectedRowKeys,
            onChange: (keys) => {
              this.setState({ selectedRowKeys: keys });
            },
          }}
        />
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  const { class_productType, class_list } = state[storeName];
  return {
    productType: class_productType,
    list: class_list,
  };
};

export default connect(mapStateToProps)(ModalChooseClass);
