/**
 * Created by Haolin on 2018/12/17.
 */
import React from 'react';
import { connect } from 'dva';
import { Row, Col } from 'antd';
import Modal from 'COMPONENT/Modal';
import List from 'COMPONENT/List';
import Input from 'COMPONENT/Input';
import Button from 'COMPONENT/Button';
import { storeName } from '../store/constant.js';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L } = List;
const Search = (props) => {
  const { stdName, onChange, onSearch } = props;
  const classPrefix = 'Search';
  return (
    <Row className={classPrefix} style={{ marginBottom: 20 }}>
      <Col span={8}>
        <Input placeholder="输入学生名称/学生编号/客户号码" value={stdName} onChange={onChange} />
      </Col>
      <Col span={4}>
        <Button type="primary" onClick={onSearch} style={{ marginLeft: 20 }}>
          查询
        </Button>
      </Col>
    </Row>
  );
};

class ModalChooseStd extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      name: '选择学生',
      loading: false,
      error: null,
      stdName: '',
      listLoading: false,
      selectedRowKeys: [],
    };
    this.columns = [
      { title: '序号', dataIndex: 'number', key: 'number', width: COL_WIDTH_S },
      { title: '学生姓名', dataIndex: 'studentName', key: 'studentName', width: COL_WIDTH_M },
      { title: '客户号码', dataIndex: 'contact', key: 'contact', width: COL_WIDTH_L },
      { title: '分公司', dataIndex: 'branchName', key: 'branchName', width: COL_WIDTH_L },
      { title: '学籍学习中心', dataIndex: 'blCampusName', key: 'blCampusName', width: COL_WIDTH_L },
      { title: '年级', dataIndex: 'gradeName', key: 'gradeName', width: COL_WIDTH_M },
    ];
    this.loadList = this.loadList.bind(this);
    this.search = this.search.bind(this);
    this.updateStdName = this.updateStdName.bind(this);
    this.confirm = this.confirm.bind(this);
    this.destroy = this.destroy.bind(this);
  }
  static classPrefix = 'ModalChooseClass';
  componentWillUnmount() {
    this.destroy();
  }

  loadList() {
    this.setState({ listLoading: true });
    const _self = this;
    this.props.dispatch({
      type: `${storeName}/loadStdList`,
      payload: {
        stdName: this.state.stdName,
        callback(selectedRowKeys) {
          _self.setState({
            listLoading: false,
            stdName: '',
            selectedRowKeys,
          });
        },
      },
    });
  }
  search() {
    const { stdName } = this.state;
    if (stdName && stdName !== '') {
      this.loadList();
    } else {
      showMessage('请输入学生名称/学生编号/客户号码后再查询！', 'error');
    }
  }
  updateStdName(ele) {
    let val = ele.target.value;
    this.setState({ stdName: val });
  }
  confirm() {
    let stdList = this.state.selectedRowKeys.map((item) => this.props.list[item]);
    this.props.dispatch({
      type: `${storeName}/selectStd`,
      payload: { stdList },
    });
    this.props.onConfirm();
  }
  destroy() {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: { std_list: [] },
    });
  }

  render() {
    const { onCancel, list } = this.props;
    const { name, loading, error, stdName, listLoading, selectedRowKeys } = this.state;

    return (
      <Modal
        title={name}
        className={ModalChooseStd.classPrefix}
        loading={loading}
        error={error}
        width={960}
        onCancel={onCancel}
        onOk={this.confirm}
        okBtnDisabled={selectedRowKeys.length === 0}
      >
        <Search stdName={stdName} onChange={this.updateStdName} onSearch={this.search} />
        <List
          dataSource={list}
          columns={this.columns}
          loading={!loading && listLoading}
          rowSelection={{
            selectedRowKeys,
            onChange: (keys) => {
              this.setState({ selectedRowKeys: keys });
            },
          }}
        />
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  const { std_list } = state[storeName];
  return {
    list: std_list,
  };
};

export default connect(mapStateToProps)(ModalChooseStd);
