import React, { Component } from 'react';
import { Modal, TextLink, DropdownMenu, TextArea } from 'COMPONENT';
import { Row, Icon, Col, message } from 'antd';
import style from './index.less';
import { OSS_BASE_PREFIX } from 'CONST/ossConfig';
import { onUploadFile, download } from 'UTIL/file';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { importCouponByPromotionInfo } from 'SERVICE/preferential';
import ChooseFileButton from 'PAGE/personnelManage/TeacherIntroduction/ChooseFileButton';
import compStyle from '../ModalPublish/compStyle.less';
import { storeName } from '../store/constant';
import { connect } from 'dva';

const xls = require('ASSET/imgs/xls.png');

const couponExcelMap = {
  INSIGHT_MARKET: {
    excel: `${OSS_BASE_PREFIX}/客户导入模版.xlsx`,
    fileName: '客户导入模版',
    apiKey: 'PHONE',
  },
  BOSS: {
    excel: `${OSS_BASE_PREFIX}/满减券-学生导入模板.xlsx`,
    fileName: '学生导入模板',
    apiKey: 'STUDENT',
  },
};
const ContentItem = (props) => {
  const { label, children } = props;
  const classPrefix = 'ContentItem';
  return (
    <div className={compStyle[classPrefix]}>
      <span className={compStyle[`${classPrefix}-label`]}>{label}</span>
      <span className={compStyle[`${classPrefix}-content`]}>{children}</span>
    </div>
  );
};

class ModalImportVoucher extends Component {
  constructor(props) {
    super(props);
    const { data = {} } = props;
    const { promotionPlatType } = data;
    this.state = {
      // 短信模板以及code
      promotionOrderSmsTemp: null,
      promotionOrderSmsCode: null,
      file: {}, // 导入的文件
      loading: false, // modal loading控制
      // 是否普通的券
      isNormalCoupon: promotionPlatType === 'BOSS',
    };
  }

  componentDidMount() {
    this.init();
  }

  init = async () => {
    this.setState({ loading: true });
    await new Promise(this.initMsgTempMenus);
    this.setState({ loading: false });
  };

  initMsgTempMenus = (resolve) => {
    this.props.dispatch({
      type: `${storeName}/initMsgTempMenus`,
      payload: {
        callback() {
          resolve();
        },
      },
    });
  };
  // 按钮 - 导入
  handleOk = () => {
    const { file } = this.state;
    this.setState({ loading: true }, async () => {
      await onUploadFile([file], this.onloadCallback);
    });
  };

  /**
   * 下载模板
   */
  downloadTemplate = () => {
    const { data = {} } = this.props;
    const { promotionPlatType } = data;
    const { excel, fileName } = couponExcelMap[promotionPlatType] || couponExcelMap['BOSS'];
    const format = 'xlsx';
    download({ url: excel, fileName, format });
  };

  /**
   * 选择文件
   */
  handleSelectFile = (file) => {
    this.setState({ file: file || {} });
  };

  /**
   * 上传文件回调
   */
  onloadCallback = async (xhr, key) => {
    const { onOK, data = {} } = this.props;
    const { promotionOrderSmsCode } = this.state;
    const { promotionPlatType } = data;
    const { apiKey } = couponExcelMap[promotionPlatType] || couponExcelMap['BOSS'];
    if (xhr.status === 200 && xhr.readyState === 4) {
      try {
        const { resultCode } =
          (await importCouponByPromotionInfo({
            fileKey: key,
            promotionInfoId: data.id,
            couponImportType: apiKey,
            templateCode: promotionOrderSmsCode,
          })) || {};
        if (resultCode === 0) {
          onOK();
          this.setState({ loading: false });
        } else {
          showMessage('导入失败,请重新导入!');
          this.setState({ loading: false });
        }
      } catch (e) {
        this.setState({ loading: false });
      }
    } else {
      showMessage('系统错误');
      this.setState({ loading: false });
    }
  };

  // 短信模板
  buildMsgTempMenus = (menus) => {
    if (menus && menus.length) {
      return menus.map((item) => ({
        ...item,
        onClick: () =>
          this.setState({
            promotionOrderSmsTemp: item.text,
            promotionOrderSmsCode: item.code,
          }),
      }));
    }
    return [];
  };

  render() {
    const { onCancel, menus } = this.props;
    const { file = {}, loading, promotionOrderSmsTemp, isNormalCoupon } = this.state;

    return (
      <Modal
        title={isNormalCoupon ? '导入学生' : '导入客户'}
        visible={true}
        widthSize="s"
        okText="导入"
        wrapClassName={style.import_voucher}
        onOk={this.handleOk}
        onCancel={onCancel}
        okBtnDisabled={!file.name}
        loading={loading}
      >
        <Row>
          <img className={style.img_excel} src={xls} alt="" />
          <TextLink style={{ marginLeft: 30 }} onClick={() => this.downloadTemplate()}>
            导入模版下载
          </TextLink>
        </Row>
        <Row className={style.choose_title}>上传文件</Row>
        {file.name ? (
          <Row className={style.select_file} gutter={8}>
            <Col span={2}>
              <Icon type="file-excel" />
            </Col>
            <Col span={20}>
              <span style={{ marginLeft: '15px' }}>{file.name || ''}</span>
            </Col>
            <Col span={2}>
              <Icon type="close-circle-o" onClick={() => this.setState({ file: {} })} />
            </Col>
          </Row>
        ) : (
          <Row className={style.file_row}>
            <ChooseFileButton
              key="upload"
              size="small"
              btnText="选择文件"
              accept="application/vnd.openxmlformats-officedocument.spreadsheetml.sheet,application/vnd.ms-excel"
              fileType={['excel']}
              onChange={(file) => this.handleSelectFile(file)}
            />
          </Row>
        )}
        <Row style={{ marginBottom: 5 }}>
          <Col span={24}>
            <ContentItem label="短信通知">
              <span style={{ color: '#999' }}>不选择模板即视为不发短信通知</span>
            </ContentItem>
          </Col>
        </Row>
        <Row>
          <TextArea placeholder="请点击下方选择框选择短信模板" value={promotionOrderSmsTemp} rows={3} disabled />
          <DropdownMenu
            style={{ margin: '10px 0 5px', display: 'block' }}
            text="短信模板"
            menus={this.buildMsgTempMenus(menus)}
            hasArrow={true}
          />
        </Row>
      </Modal>
    );
  }
}
const mapStateToProps = (state) => {
  const { msgTempMenus } = state[storeName];
  return {
    menus: msgTempMenus,
  };
};

export default connect(mapStateToProps)(ModalImportVoucher);
