import React from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';
import style from './styles.module.less';
import Card from 'COMPONENT/Card';
import { runFunc } from 'UTIL/common';

const MoreInfo = (props) => {
  return (
    <span className={style.MoreInfo} style={{ display: props.text ? 'inline-block' : 'none' }}>
      <span>{props.label}：</span>
      <span>{props.text}</span>
    </span>
  );
};
MoreInfo.propTypes = {
  label: PropTypes.string.isRequired,
  text: PropTypes.string,
};

const Content = (props) => {
  const productTypeName = props.activityId ? props.typeName : props.categoryName;

  return (
    <div className={style.Content}>
      <div className={style.mainContent}>
        <div className={style.title}>
          <span className={style.titleType}>{productTypeName}</span>
          <span>{props.name}</span>
        </div>
        <div className={style.body}>
          <MoreInfo label="年级" text={props.gradeName || '无'} />
          <MoreInfo label="年份" text={props.productVersionName} />
          <MoreInfo label="季度" text={props.productQuarterName || '无'} />
          <MoreInfo label="产品类型" text={productTypeName} />
        </div>
      </div>
    </div>
  );
};

const ToBoSelectedCard = (props) => {
  return (
    <Card className={style.ToBoSelectedCard}>
      <div className={style.cardBody}>
        <Content {...props} />
      </div>
      <div className={style.btnWrap}>
        <button
          className={classNames(style.addBtn, props.isSelected ? style.disabled : '')}
          onClick={() => {
            !props.isSelected && runFunc(props.onSelect, props.index);
          }}
        >
          {props.isSelected ? '已选' : '选择'}
        </button>
      </div>
    </Card>
  );
};

export default ToBoSelectedCard;
