import React from 'react';
import PropTypes from 'prop-types';
import { connect } from 'dva';
import style from './styles.module.less';
import { Modal } from 'COMPONENT/ShoppingCart';
import Wrapper from 'COMPONENT/Wrapper';
import Deck from 'COMPONENT/Deck';
import NoDataPlus from 'COMPONENT/NoDataPlus';
import ToBeSelectedCard from './ToBeSelectedCard';
import SelectedCard from './SelectedCard';
import { runFunc, errorsHandle, loginUser } from 'UTIL/common';
import { querySelectProduct } from 'SERVICE/preferential';
import { SearchFilterForm, SearchFieldForm, Input, Select } from 'COMPONENT';
import { BOSS, INSIGHT_MARKET, insightProductTypeDict, bossProductTypeDict } from '../config';
import { SMALL_CLASS, TWO_TEACHER } from './config';

// 待选班render
function classCardRender({ onClick }) {
  return (props, index) => {
    return <ToBeSelectedCard {...props} onSelect={onClick} index={index} key={index} />;
  };
}
// 已选班render
function selectedCardRender({ onDelete }) {
  return (props, index) => {
    return <SelectedCard {...props} onDelete={onDelete} index={index} key={index} />;
  };
}

class ModalSelectClass extends React.Component {
  constructor(props) {
    super(props);
    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.initSKey = this.initSKey.bind(this);
    this.loadList = this.loadList.bind(this);
    this.updatePage = this.updatePage.bind(this);
    this.changePage = this.changePage.bind(this);
    this.search = this.search.bind(this);
    this.handleSelect = this.handleSelect.bind(this);
    this.handleDelete = this.handleDelete.bind(this);
    this.updateSelectedRowKeys = this.updateSelectedRowKeys.bind(this);
    this.onOk = this.onOk.bind(this);
    this.state = {
      loading: false,
      // 筛选项
      searchKey: null,
      defaultSKey: {},
      // 待选列表内已选项
      selectedRowKeys: [],
      // 待选列表
      list: [],
      total: 0,
      pageCurrent: 1,
      pageSize: 10,
      // 已选列表
      selectedList: props.initList || [],
    };
    this.isBossPrd = props.platType === BOSS;
  }

  static propTypes = {
    // 满减券类型
    platType: PropTypes.oneOf([BOSS, INSIGHT_MARKET]),
    // 弹窗标题
    title: PropTypes.string,
    // 需初始化的筛选项
    initSKey: PropTypes.object,
    initList: PropTypes.array,
    // 关闭
    onClose: PropTypes.func.isRequired,
    // 提交
    onSubmit: PropTypes.func.isRequired,
  };

  static defaultProps = {
    platType: BOSS,
    title: '选择产品',
    initSKey: null,
    initList: null,
    visible: false,
  };

  componentDidMount() {
    this.init();
  }

  async init() {
    this.setState({ loading: true });
    await new Promise(this.initDict);
    await new Promise(this.initSKey);
    await new Promise(this.loadList);
    this.setState({ loading: false });
  }

  initDict(resolve) {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['year', 'quarter', 'productCharacter'],
        callback() {
          resolve();
        },
      },
    });
  }

  initSKey(resolve) {
    let newSKey = {};
    if (this.isBossPrd) {
      newSKey = {
        productCharacter: this.props.productCharacter,
        category: loginUser.isTwoTeacherSym() ? TWO_TEACHER : SMALL_CLASS,
        productQuarterId: (this.props.quarterDict || [{ val: null }])[0].val,
        productVersionId: (
          (this.props.yearDict || []).find((y) => +y.text === new Date().getFullYear()) || { val: null }
        ).val,
      };
    }
    newSKey = {
      ...newSKey,
      ...this.props.initSKey,
    };
    this.setState(
      {
        searchKey: newSKey,
        defaultSKey: { ...newSKey },
      },
      () => {
        resolve();
      },
    );
  }

  getListParams() {
    let searchKey = {
      page: this.state.pageCurrent,
      rows: this.state.pageSize,
    };

    for (let key in this.state.searchKey) {
      searchKey[key] = this.state.searchKey[key];
    }

    return searchKey;
  }

  async loadList(resolve) {
    const apiParams = this.getListParams();

    const rsp = await querySelectProduct(apiParams, true).catch(errorsHandle());

    if (rsp && rsp.resultCode === 0) {
      this.setState(
        {
          list: rsp.datas || [],
          total: rsp.rowCount,
          selectedRowKeys: [],
        },
        () => {
          this.updateSelectedRowKeys();
        },
      );
    }

    resolve();
  }

  updatePage(pageCurrent, pageSize) {
    return new Promise((resolve) => {
      this.setState(
        (prevState) => {
          let payload = {};
          if (pageSize && pageSize !== prevState.pageSize) {
            payload.pageCurrent = 1;
            payload.pageSize = pageSize;
          } else {
            payload.pageCurrent = pageCurrent;
          }
          return payload;
        },
        () => {
          resolve();
        },
      );
    });
  }

  async changePage(pageCurrent, pageSize) {
    await this.updatePage(pageCurrent, pageSize);
    this.setState({ loading: true });
    await new Promise(this.loadList);
    this.setState({ loading: false });
  }

  search(params) {
    this.setState(
      {
        searchKey: { ...this.props.initSKey, ...params },
      },
      () => {
        this.changePage(1, this.state.pageSize);
      },
    );
  }

  handleSelect(index) {
    let tagItem = this.state.list[index];
    this.setState(
      (prevState) => {
        prevState.selectedList.push(tagItem);
        return { selectedList: prevState.selectedList };
      },
      () => {
        this.updateSelectedRowKeys();
      },
    );
  }

  handleDelete(index) {
    this.setState(
      (prevState) => {
        prevState.selectedList.splice(index, 1);
        return { selectedList: prevState.selectedList };
      },
      () => {
        this.updateSelectedRowKeys();
      },
    );
  }

  updateSelectedRowKeys() {
    let result = [];
    let keyName = this.isBossPrd ? 'productId' : 'activityId';
    this.state.list.forEach((item, index) => {
      if (this.state.selectedList.findIndex((selected) => selected[keyName] === item[keyName]) > -1) {
        result.push(index);
      }
    });

    this.setState({ selectedRowKeys: result });
  }

  onOk() {
    runFunc(this.props.onSubmit, this.state.selectedList);
  }

  render() {
    const hadSelected = this.state.selectedList.length > 0;
    return (
      <Modal
        loading={this.state.loading}
        onOk={this.onOk}
        okText="保存"
        onClose={this.props.onClose}
        contentOptions={{
          title: <span style={{ fontSize: 18 }}>{this.props.title}</span>,
          children: (
            <Wrapper
              header={
                <div className={style.SearchWrapper}>
                  <SearchFilterForm onSearch={this.search} resetDisabled={hadSelected} truncationNum={5}>
                    <SearchFieldForm name="name">
                      <Input placeholder="产品名称" />
                    </SearchFieldForm>
                    {this.isBossPrd && (
                      <SearchFieldForm
                        name="productVersionId"
                        initialValue={this.state.defaultSKey.productVersionId || null}
                      >
                        <Select placeholder="年份" dict={this.props.yearDict} allowClear={false} />
                      </SearchFieldForm>
                    )}
                    {this.isBossPrd && (
                      <SearchFieldForm
                        name="productQuarterId"
                        initialValue={this.state.defaultSKey.productQuarterId || null}
                      >
                        <Select placeholder="季度" dict={this.props.quarterDict} />
                      </SearchFieldForm>
                    )}
                    {this.isBossPrd && (
                      <SearchFieldForm
                        name="productCharacter"
                        initialValue={this.state.defaultSKey.productCharacter || null}
                      >
                        <Select disabled={true} placeholder="产品性质" dict={this.props.productCharacterDict} />
                      </SearchFieldForm>
                    )}
                    {this.isBossPrd && (
                      <SearchFieldForm name="category" initialValue={this.state.defaultSKey.category || null}>
                        <Select
                          placeholder="产品类型"
                          dict={bossProductTypeDict}
                          allowClear={false}
                          disabled={hadSelected}
                        />
                      </SearchFieldForm>
                    )}
                    {!this.isBossPrd && (
                      <SearchFieldForm name="type" initialValue={this.state.defaultSKey.type || null}>
                        <Select
                          placeholder="产品类型"
                          dict={insightProductTypeDict}
                          allowClear={false}
                          disabled={true}
                        />
                      </SearchFieldForm>
                    )}
                  </SearchFilterForm>
                </div>
              }
            >
              {this.state.list.length > 0 ? (
                <Deck
                  dataSource={this.state.list}
                  render={classCardRender({
                    onClick: this.handleSelect,
                  })}
                  pagination={{
                    total: this.state.total,
                    current: this.state.pageCurrent,
                    size: this.state.pageSize,
                    onChange: this.changePage,
                  }}
                  rowSelection={{
                    selectedRowKeys: this.state.selectedRowKeys,
                    options: { noFragment: true },
                  }}
                  className={style.ToBeSelectCard}
                />
              ) : (
                <NoDataPlus className={style.NoData} />
              )}
            </Wrapper>
          ),
        }}
        sideOptions={{
          title: '已选班级',
          children: (
            <Deck
              dataSource={this.state.selectedList}
              render={selectedCardRender({
                onDelete: this.handleDelete,
              })}
              className={style.SelectedCard}
            />
          ),
        }}
      />
    );
  }
}

const mapStateToProps = (state) => {
  return {
    yearDict: state.dict.year,
    quarterDict: state.dict.quarter,
    productCharacterDict: state.dict.productCharacter,
  };
};

export default connect(mapStateToProps)(ModalSelectClass);
