import React, { Component } from 'react';
import { Modal, DropdownMenu, TextArea } from 'COMPONENT';
import { Row, Col, message } from 'antd';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import compStyle from '../ModalPublish/compStyle.less';
import { storeName } from '../store/constant';
import { connect } from 'dva';

const ContentItem = (props) => {
  const { label, children } = props;
  const classPrefix = 'ContentItem';
  return (
    <div className={compStyle[classPrefix]}>
      <span className={compStyle[`${classPrefix}-label`]}>{label}</span>
      <span className={compStyle[`${classPrefix}-content`]}>{children}</span>
    </div>
  );
};

class ModalVoucherByPhone extends Component {
  state = {
    // 短信模板以及code
    promotionOrderSmsTemp: null,
    promotionOrderSmsCode: null,
    phoneText: '',
    loading: false, // modal loading控制
  };

  componentDidMount() {
    this.init();
  }

  init = async () => {
    this.setState({ loading: true });
    await new Promise(this.initMsgTempMenus);
    this.setState({ loading: false });
  };

  initMsgTempMenus = (resolve) => {
    this.props.dispatch({
      type: `${storeName}/initMsgTempMenus`,
      payload: {
        callback() {
          resolve();
        },
      },
    });
  };
  // 按钮确定
  handleOk = () => {
    const { phoneText, promotionOrderSmsCode } = this.state;
    const { onFinish, initData } = this.props;
    const inputPhoneArray = Array.from(new Set(phoneText.match(/\d+/g) || []));
    if (!inputPhoneArray.length) {
      showMessage('请输入客户号码', 'error');
      return false;
    }
    const errorPhoneArray = inputPhoneArray.filter((phone) => !/^1\d{10}/.test(phone));
    if (errorPhoneArray.length) {
      showMessage(`输入的手机号码：${errorPhoneArray.join(',')}格式不对`, 'error');
      return false;
    }
    this.setState({
      loading: true,
    });
    this.props.dispatch({
      type: `${storeName}/publishCoupon`,
      payload: {
        couponId: initData.id,
        msgCode: promotionOrderSmsCode,
        phone: inputPhoneArray.join(','),
        type: 'saleCoupon',
        callback: (rsp) => {
          this.setState({
            loading: false,
          });
          if (rsp) {
            // 成功
            onFinish && onFinish();
          }
        },
      },
    });
  };

  // 短信模板
  buildMsgTempMenus = (menus) => {
    if (menus && menus.length) {
      return menus.map((item) => ({
        ...item,
        onClick: () =>
          this.setState({
            promotionOrderSmsTemp: item.text,
            promotionOrderSmsCode: item.code,
          }),
      }));
    }
    return [];
  };

  render() {
    const { onCancel, menus } = this.props;
    const { loading, promotionOrderSmsTemp, phoneText } = this.state;

    return (
      <Modal width={600} title="发券" onOk={this.handleOk} onCancel={onCancel} loading={loading}>
        <Row style={{ marginBottom: 5 }}>
          <Col span={24}>
            <ContentItem label="客户号码">
              <span style={{ color: '#999' }}>请输入客户号码（每行一个）</span>
            </ContentItem>
          </Col>
        </Row>
        <Row>
          <TextArea
            placeholder="请输入客户号码（每行一个）"
            value={phoneText}
            onChange={(e) => this.setState({ phoneText: e.target.value })}
            rows={10}
          />
        </Row>
        <Row style={{ marginBottom: 5 }}>
          <Col span={24}>
            <ContentItem label="短信通知">
              <span style={{ color: '#999' }}>不选择模板即视为不发短信通知</span>
            </ContentItem>
          </Col>
        </Row>
        <Row>
          <TextArea placeholder="请点击下方选择框选择短信模板" value={promotionOrderSmsTemp} rows={3} disabled />
          <DropdownMenu
            style={{ margin: '10px 0 5px', display: 'block' }}
            text="短信模板"
            menus={this.buildMsgTempMenus(menus)}
            hasArrow={true}
          />
        </Row>
      </Modal>
    );
  }
}
const mapStateToProps = (state) => {
  const { msgTempMenus } = state[storeName];
  return {
    menus: msgTempMenus,
  };
};

export default connect(mapStateToProps)(ModalVoucherByPhone);
