/**
 * Created by Haolin on 2018/12/17.
 */
import React, { PropTypes } from 'react';
import { connect } from 'dva';
import DatePicker from 'COMPONENT/DatePicker';
import SearchBar from 'COMPONENT/SearchBar';
import Select from 'COMPONENT/Select';
import Input from 'COMPONENT/Input';
import { storeName } from '../../store/constant.js';
import moment from 'moment';

class Search extends React.Component {
  constructor(props) {
    super(props);
    this.search = this.search.bind(this);
    this.reset = this.reset.bind(this);
    this.updateProps = this.updateProps.bind(this);
    this.updateKeyword = this.updateKeyword.bind(this);
    this.getDisabledStartDate = this.getDisabledStartDate.bind(this);
    this.getDisabledEndDate = this.getDisabledEndDate.bind(this);
    this.destroy = this.destroy.bind(this);
  }
  static propTypes = {
    classPrefix: PropTypes.string,
    handleSearch: PropTypes.func,
  };

  componentWillUnmount() {
    this.destroy();
  }
  /**
   * 检索
   * */
  search() {
    this.props.handleSearch();
  }
  /**
   * 重置检索条件
   * */
  reset() {
    this.updateProps({
      sKey_used_name: '',
      sKey_used_beginDate: null,
      sKey_used_endDate: null,
      sKey_used_branch: null,
      sKey_used_campus: null,
    });
  }
  /**
   * 更新数据
   * */
  updateProps(params) {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: params,
    });
  }
  updateKeyword(params) {
    const { type, val } = params;
    let payload = { [type]: val };
    if (type === 'sKey_used_branch') {
      payload.sKey_used_campus = null;
    }
    this.updateProps(payload);
  }
  // 起始日期限制
  getDisabledStartDate(startDate) {
    let { sKey_used_endDate: endData } = this.props;
    if (!startDate || !endData) {
      return false;
    }
    return moment(startDate).valueOf() > moment(endData).valueOf();
  }
  getDisabledEndDate(endDate) {
    const { sKey_used_beginDate: beginDate } = this.props;
    if (!endDate || !beginDate) {
      return false;
    }
    return moment(endDate).valueOf() < moment(beginDate).valueOf();
  }
  /**
   * 注销前销毁
   * */
  destroy() {
    this.reset();
  }

  render() {
    const {
      classPrefix,
      dict_branch,
      dict_branchCampus,
      dict_campus,
      sKey_used_name,
      sKey_used_beginDate,
      sKey_used_endDate,
      sKey_used_branch,
      sKey_used_campus,
    } = this.props;
    return (
      <SearchBar className={`${classPrefix}-search`} onSearch={this.search} onReset={this.reset}>
        <Input
          placeholder="学生姓名/编号/号码"
          value={sKey_used_name}
          onChange={(ele) => {
            this.updateKeyword({ type: 'sKey_used_name', val: ele.target.value });
          }}
        />
        <DatePicker
          placeholder="起始日期"
          value={sKey_used_beginDate}
          onChange={(val) => {
            this.updateKeyword({ type: 'sKey_used_beginDate', val });
          }}
          disabledDate={this.getDisabledStartDate}
        />
        <DatePicker
          placeholder="截止日期"
          value={sKey_used_endDate}
          onChange={(val) => {
            this.updateKeyword({ type: 'sKey_used_endDate', val });
          }}
          disabledDate={this.getDisabledEndDate}
        />
        <Select
          placeholder="分公司"
          dict={dict_branch}
          value={sKey_used_branch}
          showSearch={true}
          onChange={(val) => {
            this.updateKeyword({ type: 'sKey_used_branch', val });
          }}
        />
        <Select
          placeholder="学习中心"
          dict={
            sKey_used_branch
              ? dict_branchCampus && dict_branchCampus.filter((item) => item.val === sKey_used_branch)
              : dict_branchCampus
          }
          value={sKey_used_campus}
          showSearch={true}
          onChange={(val) => {
            this.updateKeyword({ type: 'sKey_used_campus', val });
          }}
        />
      </SearchBar>
    );
  }
}

const mapStateToProps = (state) => {
  const {
    dict_branch,
    dict_branchCampus,
    dict_campus,
    sKey_used_name,
    sKey_used_beginDate,
    sKey_used_endDate,
    sKey_used_branch,
    sKey_used_campus,
  } = state[storeName];
  return {
    dict_branch,
    dict_branchCampus,
    dict_campus,
    sKey_used_name,
    sKey_used_beginDate,
    sKey_used_endDate,
    sKey_used_branch,
    sKey_used_campus,
  };
};

export default connect(mapStateToProps)(Search);
