/**
 * Created by Haolin on 2018/12/12.
 */
import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import List from 'COMPONENT/List';
import Search from './Search';
import ButtonWrap from '../../../../signUpOnline/StdntApptMgmt/ButtonWrap';
import { storeName } from '../store/constant.js';
import compStyle from './compStyle.less';
import jslog from 'SRC/jslog';

const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;

const FooterItem = (props) => {
  const { label, children } = props;
  const classPrefix = 'footerItem';
  return (
    <div className={compStyle[classPrefix]}>
      <div className={compStyle[`${classPrefix}-label`]}>{label}</div>
      <div className={compStyle[`${classPrefix}-content`]}>{children}</div>
    </div>
  );
};

class UsedInfo extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      name: '用券情况',
      loading: false,
      listLoading: false,
      pageCurrent: 1,
      pageSize: 20,
    };
    this.columns = [
      { title: '序号', dataIndex: 'number', key: 'number', width: COL_WIDTH_S },
      { title: '学生姓名', dataIndex: 'studentName', key: 'studentName', width: COL_WIDTH_M },
      { title: '客户号码', dataIndex: 'customerContact', key: 'customerContact', width: COL_WIDTH_L },
      { title: '满减券', dataIndex: 'name', key: 'name', width: COL_WIDTH_M },
      { title: '分公司', dataIndex: 'branchName', key: 'branchName', width: COL_WIDTH_M },
      { title: '学习中心', dataIndex: 'campusName', key: 'campusName', width: COL_WIDTH_M },
      { title: '合同编号', dataIndex: 'contractId', key: 'contractId', width: COL_WIDTH_L },
      { title: '抵扣金额', dataIndex: 'discountDecimal', key: 'discountDecimal', width: COL_WIDTH_M },
      { title: '用券时间', dataIndex: 'useTime', key: 'useTime', width: COL_WIDTH_XL },
    ];
    this.init = this.init.bind(this);
    this.loadList = this.loadList.bind(this);
    this.changePage = this.changePage.bind(this);
    this.search = this.search.bind(this);
    this.modalLoading = this.modalLoading.bind(this);
    this.dispatchExportExcel = this.dispatchExportExcel.bind(this);
    this.exportWithContact = this.exportWithContact.bind(this);
    this.exportWithoutContact = this.exportWithoutContact.bind(this);
    this.destroy = this.destroy.bind(this);
  }
  static classPrefix = 'UsedInfo';
  componentDidMount() {
    this.init();
  }
  componentWillUnmount() {
    this.destroy();
  }
  /**
   * 初始化
   * */
  init() {
    this.setState({ loading: true });
    const _self = this;
    this.props.dispatch({
      type: `${storeName}/initUsed`,
      payload: {
        couponId: this.props.coupon.id,
        page: this.state.pageCurrent,
        rows: this.state.pageSize,
        callback() {
          _self.setState({ loading: false });
        },
      },
    });
  }
  /**
   * 加载列表
   * */
  loadList(page, rows) {
    this.setState({ listLoading: true });
    const _self = this;
    this.props.dispatch({
      type: `${storeName}/loadUsedList`,
      payload: {
        couponId: this.props.coupon.id,
        page,
        rows,
        callback() {
          _self.setState({ listLoading: false });
        },
      },
    });
  }
  /**
   * 翻页
   * */
  changePage({ current, pageSize }) {
    this.setState({
      pageCurrent: current,
      pageSize,
    });
    this.loadList(current, pageSize);
  }
  /**
   * 搜索
   * */
  search() {
    this.loadList(1, this.state.pageSize);
  }

  modalLoading(params) {
    this.setState({ loading: params });
  }

  /**
   * 导出excel
   * */
  dispatchExportExcel(params) {
    this.setState({ loading: true });
    const _self = this;
    this.props.dispatch({
      type: `${storeName}/exportExcel`,
      payload: {
        couponId: this.props.coupon.id,
        withContact: params,
        callback() {
          _self.setState({ loading: false });
        },
      },
    });
  }
  exportWithContact() {
    jslog.report('click', {
      function_name: '优惠方案>优惠券>满减券>用券情况>导出Excel(含客户号码)',
    });
    this.dispatchExportExcel(true);
  }
  exportWithoutContact() {
    jslog.report('click', {
      function_name: '优惠方案>优惠券>满减券>用券情况>导出Excel(不含客户号码)',
    });
    this.dispatchExportExcel(false);
  }

  destroy() {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        usedList: [],
        usedTotal: 0,
      },
    });
  }

  render() {
    const { route, list, total, amount, exportWithContactAuth, exportWithoutContactAuth } = this.props;
    const { name, loading, listLoading, pageCurrent, pageSize } = this.state;
    const classPrefix = UsedInfo.classPrefix;
    const checkList = list.length === 0;

    return (
      <Page
        name={name}
        className={classPrefix}
        fitWindowHeight={true}
        loading={loading}
        route={route}
        subHeader={
          <div className={compStyle[`${classPrefix}-bar-wrap`]}>
            <Search classPrefix={classPrefix} handleSearch={this.search} handleLoading={this.modalLoading} />
            <div className={compStyle[`${classPrefix}-bar-btn`]}>
              <ButtonWrap
                btnClass={compStyle[`${classPrefix}-button`]}
                btnText="导出Excel"
                btnType="primary"
                popPlacement="bottom"
                actions={[
                  { text: '含客户号码', onClick: this.exportWithContact, disabled: exportWithContactAuth },
                  { text: '不含客户号码', onClick: this.exportWithoutContact, disabled: exportWithoutContactAuth },
                ]}
                btnDisabled={(exportWithContactAuth && exportWithoutContactAuth) || checkList}
              />
            </div>
          </div>
        }
        headerToolBar={!checkList && <FooterItem label="合计抵扣金额">{amount}</FooterItem>}
      >
        <List
          dataSource={list}
          columns={this.columns}
          loading={!loading && listLoading}
          pagination={{
            current: pageCurrent,
            pageSize,
            total,
          }}
          onChange={this.changePage}
        />
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const { coupon } = state.main.pageParams['/Voucher/UsedInfo'];
  const { usedList, usedTotal, couponAmountTotal } = state[storeName];
  const exportWithContactAuth = state.account.btnAuths.indexOf('EXPORT_WITH_CONTACT_COUPON') === -1;
  const exportWithoutContactAuth = state.account.btnAuths.indexOf('EXPORT_WITHOUT_CONTACT_COUPON') === -1;
  return {
    coupon,
    list: usedList,
    total: usedTotal,
    amount: couponAmountTotal,
    exportWithContactAuth,
    exportWithoutContactAuth,
  };
};

export default connect(mapStateToProps)(UsedInfo);
