import React from 'react';
import { connect } from 'dva';
import { storeName } from './store/constant.js';
import compStyle from './compStyle.less';
import CouponDesk from './CouponDesk';
import { Page, Button, SearchFilterForm, SearchFieldForm, Input, Select, MultiSelect, PopBtnGroups } from 'COMPONENT';
import { DatePicker } from 'antd';
import ModalVoucherByPhone from './ModalVoucherByPhone';
import {
  continuityDict,
  superpositionDict,
  couponStatusDict,
  COUPON_NORMAL,
  COUPON_MARKET,
  promotionNatureDict,
  ruleTextCfg,
  userRangeDict,
  needToReceiveDict,
  rangeFormat,
  timeFormat,
  INSIGHT_MARKET,
  NONE,
  quarterTextCfg,
} from './config';
import { loginUser, errorsHandle } from 'UTIL/common';
import { getDictTextByVal } from 'UTIL/dict';
import { getPromotionList } from 'SERVICE/preferential';
import moment from 'moment';

class OverMinusCoupon extends React.Component {
  constructor(props) {
    super(props);
    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.initSKey = this.initSKey.bind(this);
    this.getListParams = this.getListParams.bind(this);
    this.loadList = this.loadList.bind(this);
    this.listAdapter = this.listAdapter.bind(this);
    this.updatePage = this.updatePage.bind(this);
    this.changePage = this.changePage.bind(this);
    this.search = this.search.bind(this);
    this.newCoupon = this.newCoupon.bind(this);
    this.cardAction = this.cardAction.bind(this);
    this.openPage = this.openPage.bind(this);
    this.destroy = this.destroy.bind(this);
    this.state = {
      name: '满减券',
      loading: false,
      searchKey: null,
      defaultSKey: {},
      list: [],
      total: 0,
      pageCurrent: 1,
      pageSize: 10,
      platType: null,
      toggleVoucherPhoneModal: false,
    };
    this.modalInit = null;
  }

  static classPrefix = 'OverMinusCoupon';

  componentDidMount() {
    this.init();
  }

  componentWillUnmount() {
    this.destroy();
  }

  async init() {
    this.setState({ loading: true });
    await new Promise(this.initDict);
    await new Promise(this.initSKey);
    await new Promise(this.loadList);
    this.setState({ loading: false });
  }

  initDict(resolve) {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['branch', 'productCharacter'],
        callback() {
          resolve();
        },
      },
    });
  }

  initSKey(resolve) {
    const isGroup = loginUser.isGroupRole();
    let newSKey = {
      branchId: [isGroup ? (this.props.branchDict || [{ val: null }])[0].val : loginUser.userData.brenchId],
      status: '0',
      // 满减券券类型不选时默认传2种类型
      promotionNature: [COUPON_NORMAL, COUPON_MARKET],
    };
    this.setState(
      {
        searchKey: newSKey,
        defaultSKey: { ...newSKey },
      },
      () => {
        resolve();
      },
    );
  }

  getListParams() {
    let searchKey = {
      // 接口的第一页是 0 页
      pageNo: this.state.pageCurrent - 1,
      pageSize: this.state.pageSize,
      // 优惠券
      promotionKindId: 'COUPON',
      // 满减券
      promotionLittleTypeId: 'COUPON_FULL_SALE',
      // 满减规则(还有每满减规则，这里没有)
      promotionTypeId: 'COUPON_FULL_SALE_NORMAL',
    };

    for (let key in this.state.searchKey) {
      searchKey[key] = this.state.searchKey[key];
    }

    return searchKey;
  }

  async loadList(resolve) {
    const apiParams = this.getListParams();

    const rsp = await getPromotionList(apiParams, true).catch(errorsHandle());
    if (rsp && rsp.resultCode === 0) {
      this.setState({
        list: this.listAdapter(rsp.datas) || [],
        total: rsp.rowCount,
      });
    }

    resolve();
  }

  listAdapter(list) {
    return list.map((item) => ({
      id: item.id,
      name: item.promotionName,
      limitProductCharacter: item.limitProductCharacter,
      continuityName: getDictTextByVal(continuityDict, item.continuityType),
      branchName: item.branchVos.map((branch) => branch.branchName),
      coupons: item.ruleVos.map((coupon) => ({
        productNum: coupon.promotionRule,
        amount: coupon.promotion,
        _origin: coupon,
      })),
      beginDate: OverMinusCoupon.fixDateDisplay(item.startDate),
      endDate: OverMinusCoupon.fixDateDisplay(item.endDate),
      description: item.promotionComment,
      statusName: getDictTextByVal(couponStatusDict, item.status),
      superPosition: item.superPosition,
      status: item.status,
      couponTotal: item.publishNum,
      couponsInfo: OverMinusCoupon.buildCouponsInfo({
        ruleVos: item.ruleVos,
        promotionTypeId: item.promotionTypeId,
        continuityType: item.continuityType,
        promotionPlatType: item.promotionPlatType,
      }),
      couponUsedNum: item.useNum,
      needToRecive: item.needToRecive,
      promotionPlatType: item.promotionPlatType,
      promotionVarietyName: item.promotionVarietyName,
      relativePeriod: item.relativePeriod,
      limitScope: item.limitScope,
      productVos: item.productVos,
      useScopeName: item.useScopeName,
      showScopeName: item.showScopeName,
      receiveStartDate: item.receiveStartDate,
      receiveEndDate: item.receiveEndDate,
      userRange: item.userRange,
      _origin: item,
    }));
  }

  static fixDateDisplay(dateStr) {
    if (!dateStr) {
      return '';
    }

    let index = dateStr.lastIndexOf('.');
    return index === -1 ? dateStr : dateStr.slice(0, index);
  }

  static buildCouponsInfo({ ruleVos, promotionTypeId, continuityType, promotionPlatType }) {
    if (promotionPlatType === INSIGHT_MARKET || continuityType === NONE) {
      return ruleVos
        .map((coupon) => {
          const { promotionRule, ruleUnit, promotion, unit } = coupon;
          return `${ruleTextCfg[promotionTypeId]}${promotionRule}${ruleTextCfg[ruleUnit]}减${promotion}${ruleTextCfg[unit]}`;
        })
        .join('，');
    }
    const [quarter1, quarter2] = continuityType.split('_');
    return ruleVos
      .map((coupon) => {
        const { promotionRule, ruleUnit, promotion, quarterPromotionA, quarterPromotionB, unit } = coupon;
        // 兼容旧数据
        if (quarterPromotionA === 0 && quarterPromotionB === 0 && promotion > 0) {
          return `${ruleTextCfg[promotionTypeId]}${promotionRule}${ruleTextCfg[ruleUnit]}减${promotion}${ruleTextCfg[unit]}`;
        }
        return `${ruleTextCfg[promotionTypeId]}${promotionRule}${ruleTextCfg[ruleUnit]}${quarterTextCfg[quarter1]}减${quarterPromotionA}${ruleTextCfg[unit]}${quarterTextCfg[quarter2]}减${quarterPromotionB}${ruleTextCfg[unit]}`;
      })
      .join('，');
  }

  updatePage(pageCurrent, pageSize) {
    return new Promise((resolve) => {
      this.setState(
        (prevState) => {
          let payload = {};
          if (pageSize && pageSize !== prevState.pageSize) {
            payload.pageCurrent = 1;
            payload.pageSize = pageSize;
          } else {
            payload.pageCurrent = pageCurrent;
          }
          return payload;
        },
        () => {
          resolve();
        },
      );
    });
  }

  async changePage(pageCurrent, pageSize) {
    await this.updatePage(pageCurrent, pageSize);
    this.setState({ loading: true });
    await new Promise(this.loadList);
    this.setState({ loading: false });
  }

  search(params) {
    this.setState(
      {
        searchKey: { ...this.props.initSKey, ...this.searchKeyAdapter(params) },
      },
      () => {
        this.changePage(1, this.state.pageSize);
      },
    );
  }

  searchKeyAdapter = (params) => {
    if (!params.promotionNature) {
      params.promotionNature = [COUPON_NORMAL, COUPON_MARKET];
    }
    if (params.receiveStartDate) {
      params.receiveStartDate = params.receiveStartDate.format(rangeFormat);
    }
    if (params.receiveEndDate) {
      params.receiveEndDate = params.receiveEndDate.format(rangeFormat);
    }
    if (params.startDate) {
      params.startDate = params.startDate.format(rangeFormat);
    }
    if (params.endDate) {
      params.endDate = params.endDate.format(rangeFormat);
    }
    return params;
  };

  /**
   * 新增
   *
   * @param promotionPlatType 优惠券平台类型 {BOSS: BOSS平台, INSIGHT_MARKET: 流量营销平台}
   * */
  newCoupon(promotionPlatType) {
    this.openPage('add', {
      platType: promotionPlatType,
      couponId: null,
      isSend: false,
    });
  }

  cardAction(params) {
    if (params.type === 'Edit') {
      // 编辑
      this.openPage('edit', {
        platType: params.promotionPlatType,
        couponId: params.val,
        isSend: params.isSend,
      });
    } else if (params.type === 'Issue' && params.val.promotionPlatType === 'INSIGHT_MARKET') {
      // 如果操作为发券并且当前券是营销的券 那么打开新的弹框进行手机号发券
      this.setState({
        toggleVoucherPhoneModal: true,
      });
      this.modalInit = params.val;
    } else {
      // 发券/发券历史/用券情况
      this.openPage(params.type, { coupon: params.val });
    }
  }

  openPage(path, params) {
    let payload = {
      path: `/Voucher/${path}`,
      refresh: true,
    };
    if (params) {
      payload.params = params;
    }
    this.props.dispatch({
      type: 'main/openPage',
      payload,
    });
  }

  destroy() {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        list_coupon: [],
        total_coupon: 0,
      },
    });
  }

  /**
   * 关闭手机号发券的弹窗
   * */
  closePhoneModal = () => {
    this.modalInit = null;
    this.setState({ toggleVoucherPhoneModal: false });
  };

  /**
   * 弹框手机号发券
   * */
  salePhoneSubmit = () => {
    this.setState({
      toggleVoucherPhoneModal: false,
    });
    this.modalInit = null;
    // 重新刷新
    this.loadList({
      current: this.state.pageCurrent,
      size: this.state.pageSize,
    });
  };

  render() {
    const classPrefix = OverMinusCoupon.classPrefix;

    return (
      <Page
        name={this.state.name}
        className={classPrefix}
        fitWindowHeight={true}
        loading={this.state.loading}
        route={this.props.route}
        headerToolBar={
          <div className={compStyle.btnWrapper}>
            {this.props.addCouponVisible && (
              <PopBtnGroups
                popoverProps={{ placement: 'bottomLeft', trigger: 'hover' }}
                groups={[
                  {
                    name: (
                      <div>
                        <p className={compStyle.btnTitle}>常规-满减券</p>
                        <p className={compStyle.btnTips}>报读及boss中使用</p>
                      </div>
                    ),
                    onClick: () => {
                      this.newCoupon('BOSS');
                    },
                    disabled: this.props.addBossCouponDisabled,
                  },
                  {
                    name: (
                      <div>
                        <p className={compStyle.btnTitle}>营销-满减券</p>
                        <p className={compStyle.btnTips}>市场活动、讲座、系列课使用</p>
                      </div>
                    ),
                    onClick: () => {
                      this.newCoupon('INSIGHT_MARKET');
                    },
                    disabled: this.props.addInsightCouponDisabled,
                  },
                ]}
              >
                <Button type="primary">新增</Button>
              </PopBtnGroups>
            )}
          </div>
        }
        subHeader={
          <SearchFilterForm onSearch={this.search} truncationNum={4}>
            <SearchFieldForm name="promotionName" span={1}>
              <Input placeholder="优惠券名称" />
            </SearchFieldForm>
            <SearchFieldForm name="productName" span={1}>
              <Input placeholder="产品名称" />
            </SearchFieldForm>
            <SearchFieldForm name="branchId" span={1} initialValue={this.state.defaultSKey.branchId || []}>
              <MultiSelect placeholder="分公司" dict={this.props.branchDict} allowClear={false} showSearch />
            </SearchFieldForm>
            <SearchFieldForm name="status" span={1} initialValue={this.state.defaultSKey.status || null}>
              <Select placeholder="状态" dict={couponStatusDict} allowClear={false} />
            </SearchFieldForm>
            <SearchFieldForm
              name="promotionNature"
              span={1}
              initialValue={this.state.defaultSKey.promotionNature || null}
            >
              <Select placeholder="券类型" dict={promotionNatureDict} />
            </SearchFieldForm>
            <SearchFieldForm name="continuityType" span={1}>
              <Select placeholder="连报条件" dict={continuityDict} />
            </SearchFieldForm>
            <SearchFieldForm name="userRange" span={1}>
              <Select placeholder="用户范围" dict={userRangeDict} />
            </SearchFieldForm>
            <SearchFieldForm name="needToRecive" span={1}>
              <Select placeholder="获取方式" dict={needToReceiveDict} />
            </SearchFieldForm>
            <SearchFieldForm name="superPosition" span={1}>
              <Select placeholder="是否叠加" dict={superpositionDict} />
            </SearchFieldForm>
            <SearchFieldForm name="receiveStartDate" span={1} className={compStyle.datePickerWrap}>
              <DatePicker
                placeholder="领券开始时间"
                format={rangeFormat}
                showTime={{ defaultValue: moment('00:00:00', timeFormat) }}
                showToday={false}
              />
            </SearchFieldForm>
            <SearchFieldForm name="receiveEndDate" span={1} className={compStyle.datePickerWrap}>
              <DatePicker
                placeholder="领券结束时间"
                format={rangeFormat}
                showTime={{ defaultValue: moment('00:00:00', timeFormat) }}
                showToday={false}
              />
            </SearchFieldForm>
            <SearchFieldForm name="startDate" span={1} className={compStyle.datePickerWrap}>
              <DatePicker
                placeholder="有效开始时间"
                format={rangeFormat}
                showTime={{ defaultValue: moment('00:00:00', timeFormat) }}
                showToday={false}
              />
            </SearchFieldForm>
            <SearchFieldForm name="endDate" span={1} className={compStyle.datePickerWrap}>
              <DatePicker
                placeholder="有效结束时间"
                format={rangeFormat}
                showTime={{ defaultValue: moment('00:00:00', timeFormat) }}
                showToday={false}
              />
            </SearchFieldForm>
          </SearchFilterForm>
        }
      >
        <CouponDesk
          dataSource={this.state.list}
          pagination={{
            current: this.state.pageCurrent,
            size: this.state.pageSize,
            total: this.state.total,
          }}
          onChange={this.changePage}
          cardAction={this.cardAction}
        />
        {this.state.toggleVoucherPhoneModal && (
          <ModalVoucherByPhone
            initData={this.modalInit}
            onCancel={this.closePhoneModal}
            onFinish={this.salePhoneSubmit}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const btnAuth = state.account.btnAuths;
  const addBossCouponAuth = btnAuth.indexOf('ADD_FULL_SALE_COUPON');
  const addInsightCouponAuth = btnAuth.indexOf('ADD_FULL_SALE_INSIGHT_COUPON');

  return {
    branchDict: state.dict.branch,
    addBossCouponDisabled: addBossCouponAuth === -1,
    addInsightCouponDisabled: addInsightCouponAuth === -1,
    addCouponVisible: addBossCouponAuth > -1 || addInsightCouponAuth > -1,
  };
};

export default connect(mapStateToProps)(OverMinusCoupon);
