import {
  storeName,
  TWO_TEACHER_VAL,
  SMALL_CLASS_VAL,
  COUPON,
  COUPON_FULL_SALE,
  COUPON_FULL_SALE_NORMAL,
  COUPON_FULL_SALE_EVERY,
  PRODUCT_NUMS,
  MONEY,
} from './constant.js';
import {
  loadCommonDict,
  loadBranchDict,
  loadCampusDict,
  loadBranchCampusDict,
  loadClientTypeDict,
  loadClassLevelDict,
  loadMainClassTcDict,
  loadAuxClassTcDict,
  loadSmallMainTc,
  loadSmallClassTcDict,
  getDictValByText,
  getDictTextByVal,
} from 'UTIL/dict';
import {
  getClassStudentQualification,
  queryStudentsForSendCoupons,
  getClassStudentQualificationInfo,
  savePromotion,
  getPromotionList,
  getPromotionInfoById,
  getRemarkTemplates,
  sendPromotionCoupons,
  listSendPromotionCouponsHistory,
  undoPromotionCoupon,
  getSmsTemplate,
  listUsedCouponsHistory,
  getTotalUsedCouponsDiscount,
  exportUsedCouponsHistory,
  listImportRecord,
  exportImportRecord,
  findPageByImportCoupon,
  exportPromotionSendCoupon,
  getPromotionProductById,
} from 'SERVICE/preferential';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { cloneDeep } from 'lodash';
import moment from 'moment';
import { PROJECT_NAME } from 'CONST/config';
import { runFunc, errorsHandle } from 'UTIL/common';
import { urlBase } from 'CONST/config';
import file from 'UTIL/file';
import {
  PERIOD_VALID_REGULAR,
  PERIOD_VALID_RELATIVE,
  LIMIT_SCOPE_INFINITE,
  SUPERPOSITION_DISABLE,
  USE_SCOPE_H5,
  SHOW_SCOPE_H5,
} from './constant';
import {
  PROMOTION_VARIETY_EXP_CLASS,
  promotionVarietyMapProductType,
  ruleUintCfgByPlatType,
  superpositionDict,
} from '../config';

const fileUpload = file.upload;
const downloadFile = file.downloadDirect;

const _fn = {
  // 用于辅助构建满减券页面获取主列表时的参数
  buildClassSKey(valArr) {
    let len = valArr ? valArr.length : 0;
    let ret;
    switch (len) {
      case 0:
        ret = null;
        break;
      case 1:
        ret = valArr[0];
        break;
      default:
        ret = valArr.join(',');
    }
    return ret;
  },
  // 初始化选择班级弹窗所需字典
  initChooseClassDict() {
    return new Promise(async (resolve, reject) => {
      const [
        dict_year,
        dict_quarter,
        dict_phase,
        dict_weekDay,
        dict_grade,
        dict_subject,
        dict_classType_twoTc,
        dict_classType_smallClass,
        dict_campus,
        dict_branchCampus,
        dict_mainTc_twoTc,
        dict_auxTc_twoTc,
        dict_mainTc_smallClass,
        dict_auxTc_smallClass,
      ] = await Promise.all([
        loadCommonDict('PRODUCT_VERSION'),
        loadCommonDict('PRODUCT_QUARTER'),
        loadCommonDict('SMALL_CLASS_PHASE'),
        loadCommonDict('WeekDay'),
        loadCommonDict('STUDENT_GRADE'),
        loadCommonDict('SUBJECT'),
        loadCommonDict('CLASS_TYPE', TWO_TEACHER_VAL),
        loadCommonDict('CLASS_TYPE', SMALL_CLASS_VAL),
        loadCampusDict(),
        loadBranchCampusDict(),
        loadMainClassTcDict(),
        loadAuxClassTcDict(),
        loadSmallMainTc(),
        loadSmallClassTcDict(),
      ]);
      resolve({
        initChooseClassDict: true,
        dict_year,
        dict_quarter,
        dict_phase,
        dict_weekDay,
        dict_grade,
        dict_subject,
        dict_classType_twoTc,
        dict_classType_smallClass,
        dict_campus,
        dict_branchCampus,
        dict_mainTc_twoTc,
        dict_auxTc_twoTc,
        dict_mainTc_smallClass,
        dict_auxTc_smallClass,
      });
    });
  },
  // 初始化选择班级弹窗检索栏默认值
  initChooseClassSKey(state, productType) {
    const { brenchId, campusId } = state.account.user;
    const dict_branch = state.dict.branch;
    const {
      dict_branchCampus,
      dict_year,
      dict_quarter,
      dict_grade,
      dict_subject,
      dict_classType_twoTc,
      dict_classType_smallClass,
      dict_classLevel,
    } = state[storeName];
    const isGroup = brenchId === campusId;
    let class_branch = isGroup ? dict_branch[0].val : brenchId;
    let class_campus = isGroup ? dict_branchCampus.find((item) => item.val === class_branch).options[0].val : campusId;
    const nowYear = new Date().getFullYear().toString();
    let class_year = getDictValByText(dict_year, nowYear) || null;
    let class_quarter = getDictValByText(dict_quarter, '暑假') || null;
    let dictClassType = productType === TWO_TEACHER_VAL ? dict_classType_twoTc : dict_classType_smallClass;
    return {
      class_productType: productType,
      class_branch,
      class_campus,
      class_year,
      class_quarter,
      class_grade: dict_grade && dict_grade[0] ? [dict_grade[0].val] : [],
      class_subject: dict_subject && dict_subject[0] ? [dict_subject[0].val] : [],
      class_classType: dictClassType && dictClassType[0] ? [dictClassType[0].val] : [],
      class_classLevel: dict_classLevel && dict_classLevel[0] ? [dict_classLevel[0].val] : [],
    };
  },
  // 构建通过选择学生弹窗的已选学生列表
  buildIssueStdList(newList, oldList) {
    let clone = cloneDeep(oldList);
    let cloneStdIds = clone.map((item) => item.studentId);
    newList.forEach((item) => {
      if (cloneStdIds.indexOf(item.studentId) === -1) {
        clone.push(item);
      }
    });
    return clone.map((item, index) => ({
      ...item,
      number: index + 1,
    }));
  },
  // 构建通过选择班级弹窗的已选班级列表
  buildChosenClassList(newList, oldList) {
    let clone = cloneDeep(oldList);
    let cloneClassIds = clone.map((item) => item.classId);
    newList.forEach((item) => {
      if (cloneClassIds.indexOf(item.classId) === -1) {
        clone.push(item);
      }
    });
    return clone;
  },
  // 将已选班级列表分为小班与双师两部分
  buildClassIds(state) {
    const { classChosenList } = state[storeName];
    let smallClass = [];
    let twoTc = [];
    classChosenList.forEach((item) => {
      if (item.productType === TWO_TEACHER_VAL) {
        twoTc.push(item.classId);
      } else {
        smallClass.push(item.classId);
      }
    });
    let twoTeacherId = twoTc.length > 0 ? twoTc.join(',') : '';
    let smallClassId = smallClass.length > 0 ? smallClass.join(',') : '';
    return {
      twoTeacherId,
      smallClassId,
    };
  },
  // 构建提交发券所需班级id及学生id
  buildPublishParams(state) {
    const { classChosenList, stdChosenList, stdDeleteList } = state[storeName];
    let classIdArr = [];
    let studentIdArr = [];
    if (classChosenList.length > 0) {
      classIdArr = classChosenList.map((item) => `${item.productType}_${item.classId}`);
      studentIdArr = stdDeleteList;
    } else if (stdChosenList.length > 0) {
      studentIdArr = stdChosenList.map((item) => item.studentId);
      if (stdDeleteList.length > 0) {
        let originSet = new Set(studentIdArr);
        let delSet = new Set(stdDeleteList);
        studentIdArr = Array.from(new Set([...originSet].filter((x) => !delSet.has(x))));
      }
    }
    return {
      classIds: classIdArr.length > 0 ? classIdArr.join(',') : '',
      studentIds: studentIdArr.length > 0 ? studentIdArr.join(',') : '',
    };
  },
  // 返回promise resolve(something)
  resolveData(params) {
    return new Promise((resolve, reject) => {
      resolve(params || null);
    });
  },
  // 初始化发券历史所需字典
  initHistoryAndUsedDict({ dict_branch, dict_branchCampus, dict_campus }) {
    return new Promise(async (resolve, reject) => {
      let fetchList = [];
      fetchList.push(!dict_branch ? loadBranchDict() : _fn.resolveData(dict_branch));
      fetchList.push(!dict_branchCampus ? loadBranchCampusDict() : _fn.resolveData(dict_branchCampus));
      fetchList.push(!dict_campus ? loadCampusDict() : _fn.resolveData(dict_campus));
      const [_dict_branch, _dict_branchCampus, _dict_campus] = await Promise.all(fetchList);
      resolve({
        dict_branch: _dict_branch,
        dict_branchCampus: _dict_branchCampus,
        dict_campus: _dict_campus,
      });
    });
  },
  // // 初始化发券历史
  // initHistorySKey (state) {
  //   const { dict_couponUsedStatus } = state[storeName];
  //   return {
  //     sKey_history_status: dict_couponUsedStatus[0].val,
  //   }
  // },
  // 构建发券历史请求参数
  buildHistoryParams(state) {
    const {
      sKey_history_stdName,
      sKey_history_beginDate,
      sKey_history_endDate,
      sKey_history_branch,
      sKey_history_campus,
      sKey_history_status,
    } = state[storeName];
    return {
      name: sKey_history_stdName,
      startTime: sKey_history_beginDate ? `${sKey_history_beginDate} 00:00:00` : null,
      endTime: sKey_history_endDate ? `${sKey_history_endDate} 23:59:59` : null,
      branchId: sKey_history_branch || null,
      campusId: sKey_history_campus || null,
      status: sKey_history_status || null,
      gradeId: null,
    };
  },
  // 构建用券情况请求参数
  buildUsedParams(state) {
    const { sKey_used_name, sKey_used_beginDate, sKey_used_endDate, sKey_used_branch, sKey_used_campus } =
      state[storeName];
    return {
      name: sKey_used_name,
      startTime: sKey_used_beginDate ? `${sKey_used_beginDate} 00:00:00` : null,
      endTime: sKey_used_endDate ? `${sKey_used_endDate} 23:59:59` : null,
      branchId: sKey_used_branch || null,
      campusId: sKey_used_campus || null,
      gradeId: null,
    };
  },
  // 用券情况excel文件名
  buildExcelName(state) {
    const dict_branch = state.dict.branch;
    const { dict_campus, sKey_used_branch, sKey_used_campus } = state[storeName];
    let branchName = getDictTextByVal(dict_branch, sKey_used_branch) || '';
    let campusName = getDictTextByVal(dict_campus, sKey_used_campus) || '';
    return `${branchName}${campusName}用券情况`;
  },
  // 下载文件
  downloadFile(url, fileName) {
    return new Promise((resolve, reject) => {
      const xhr = new XMLHttpRequest();
      xhr.open('GET', url);
      xhr.responseType = 'blob';
      xhr.onload = () => {
        const link = document.createElement('a');
        const body = document.body;
        link.href = window.URL.createObjectURL(xhr.response);
        link.download = decodeURIComponent(fileName + '.xls');
        link.style.display = 'none';
        body.appendChild(link);
        link.click();
        window.URL.revokeObjectURL(link.href);
        body.removeChild(link);
        showMessage('导出成功', 'success');
        resolve();
      };
      xhr.onerror = () => {
        showMessage('导出失败', 'error');
        resolve();
      };
      xhr.send();
    });
  },
};

// 新增/修改
const defaultCouponData = {
  promotionPlatType: null, // 优惠券平台类型
  couponDetail: null,
  couponProductList: null, // 优惠券的关联产品
  templateMenus: [
    {
      text: '暂无使用说明历史',
      disabled: true,
    },
  ],
  couponName: '',
  branch: null, // 改为单选
  continuityType: null,
  continuitySubjectType: 0,
  sameSubject: false,
  clientType: null,
  momentRange: [null, null],
  dateRange: [null, null],
  needToRecive: 0, // 获取方式
  superPosition: null, // 叠加方式
  periodValidType: PERIOD_VALID_REGULAR, // 有效期方式(仅前端使用，保存时有效期字段需根据此值判断)
  relativePeriod: null, // int 相对有效期
  limitScope: LIMIT_SCOPE_INFINITE, // 优惠券限制范围(提交时需转为number)
  useScope: USE_SCOPE_H5, // 优惠券使用平台
  showScope: SHOW_SCOPE_H5, // 优惠券展示平台
  promotionVariety: PROMOTION_VARIETY_EXP_CLASS, // 优惠券类型
  couponCondition: [
    {
      productNum: null,
      offerPrice: null,
      _origin: null,
    },
  ],
  description: '',
};

export default {
  namespace: storeName,
  state: {
    // 字典数据
    dict_branch: null,
    dict_productType: [
      { text: '双师', val: TWO_TEACHER_VAL },
      { text: '小班', val: SMALL_CLASS_VAL },
    ],
    dict_year: null,
    dict_quarter: null,
    dict_phase: null,
    dict_weekDay: null,
    dict_grade: null,
    dict_subject: null,
    dict_classType_twoTc: null,
    dict_classType_smallClass: null,
    dict_classLevel: null,
    dict_campus: null,
    dict_branchCampus: null,
    dict_mainTc_twoTc: null,
    dict_auxTc_twoTc: null,
    dict_mainTc_smallClass: null,
    dict_auxTc_smallClass: null,
    dict_couponUsedStatus: [
      {
        text: '未领取',
        val: '5',
      },
      {
        text: '未使用',
        val: '0',
      },
      {
        text: '已占用',
        val: '4',
      },
      {
        text: '已使用',
        val: '1',
      },
      {
        text: '已过期',
        val: '3',
      },
      {
        text: '撤回',
        val: '2',
      },
    ],
    dict_superimposedStatus: superpositionDict,
    //
    initChooseClassDict: false,
    // 满减券页面
    sKey_couponName: '',
    sKey_clientType: null,
    sKey_continuity: null,
    sKey_branch: [],
    sKey_couponStatus: null,
    sKey_superimposedStatus: null,
    sKey_promotionPlatType: null,
    list_coupon: [],
    total_coupon: 0,
    // 新增/修改
    ...defaultCouponData,
    // 发券
    classChosenList: [],
    stdChosenList: [],
    stdDeleteList: [],
    stdList: [],
    total_stdList: 0,
    // 导入Excel历史记录
    importHistoryList: [],
    importHistoryTotal: 0,
    importHistoryPageSize: 20,
    importHistoryPageCurrent: 1,
    // 按班级选择
    class_className: '',
    class_productType: null,
    class_year: null,
    class_quarter: null,
    class_phase: null,
    class_grade: [],
    class_subject: [],
    class_classType: [],
    class_classLevel: [],
    class_branch: null,
    class_campus: null,
    class_mainTc: null,
    class_auxTc: null,
    class_list: [],
    // 按学生选择
    std_list: [],
    // 发券短信通知模板
    msgTempMenus: [],
    // 发券历史
    sKey_history_stdName: '',
    sKey_history_beginDate: null,
    sKey_history_endDate: null,
    sKey_history_branch: null,
    sKey_history_campus: null,
    sKey_history_status: null,
    historyList: [],
    historyTotal: 0,
    // 用券情况
    sKey_used_name: '',
    sKey_used_beginDate: null,
    sKey_used_endDate: null,
    sKey_used_branch: null,
    sKey_used_campus: null,
    usedList: [],
    usedTotal: 0,
    couponAmountTotal: 0,
    // 导入历史
    importingHistoryList: [],
    importingHistoryListTotal: 0,
  },
  reducers: {
    // 更新state
    updateState(state, action) {
      const { payload } = action;
      return {
        ...state,
        ...payload,
      };
    },
    // 更新选择班级列表数据
    updateClassList(state, action) {
      const {
        payload: { list },
      } = action;
      let class_list = list.map((item, index) => ({
        ...item,
        number: index + 1,
      }));
      return {
        ...state,
        class_list,
      };
    },
    // 存储已选中班级数据
    selectClass(state, action) {
      const {
        payload: { classChosenList },
      } = action;
      let list = _fn.buildChosenClassList(classChosenList, state.classChosenList);
      return {
        ...state,
        classChosenList: list,
      };
    },
    // 更新选择学生列表数据
    updateStdList(state, action) {
      const {
        payload: { list },
      } = action;
      let std_list = list.map((item, index) => ({
        ...item,
        number: index + 1,
      }));
      return {
        ...state,
        std_list,
      };
    },
    // 存储已选中学生数据
    selectStd(state, action) {
      const {
        payload: { stdList },
      } = action;
      let list = _fn.buildIssueStdList(stdList, state.stdChosenList);
      let total_stdList = list.length;
      return {
        ...state,
        stdChosenList: list,
        total_stdList,
      };
    },
    // 更新发券页面主列表数据
    updateIssueStdList(state, action) {
      const { stdList: oldList } = state;
      const {
        payload: { list, total },
      } = action;
      let stdList = _fn.buildIssueStdList(list, oldList);
      return {
        ...state,
        stdList,
        total_stdList: total,
      };
    },
    // 存储发券页面删除的学生数据
    updateDeleteStd(state, action) {
      let { stdDeleteList } = state;
      const {
        payload: { stdId },
      } = action;
      stdDeleteList.push(stdId);
      stdDeleteList = Array.from(new Set(stdDeleteList));
      return {
        ...state,
        stdDeleteList,
      };
    },
    // 去除对应的发券页面删除的学生数据
    updateUndoDeleteStd(state, action) {
      let { stdDeleteList } = state;
      const {
        payload: { stdId },
      } = action;
      let index = stdDeleteList.indexOf(stdId);
      if (index !== -1) {
        stdDeleteList.splice(index, 1);
      }
      return {
        ...state,
        stdDeleteList,
      };
    },
    //
    updateHistoryList(state, action) {
      const {
        payload: { list, total, page, rows },
      } = action;
      let historyList = list.map((item, index) => ({
        ...item,
        number: index + 1 + (page - 1) * rows,
      }));
      return {
        ...state,
        historyList,
        historyTotal: total,
      };
    },
    //
    updateAfterUndoCoupon(state, action) {
      const { index } = action.payload;
      let { historyList } = state;
      historyList[index].status = 2;
      historyList[index].statusStr = '撤回';
      return {
        ...state,
        historyList: historyList,
      };
    },
    //
    updateUsedList(state, action) {
      const {
        payload: { list, total, page, rows },
      } = action;
      let usedList = list.map((item, index) => ({
        ...item,
        number: index + 1 + (page - 1) * rows,
      }));
      return {
        ...state,
        usedList,
        usedTotal: total,
      };
    },

    updateMsgTempMenus(state, { payload: { data } }) {
      const keyList = Object.keys(data);
      return {
        ...state,
        msgTempMenus: keyList.map((key) => ({
          code: key,
          text: data[key],
        })),
      };
    },
  },
  effects: {
    /**
     * 初始化选择班级弹窗
     * */
    *initModalChooseClass(action, effects) {
      const {
        payload: { callback, handleErr },
      } = action;
      const { put, select } = effects;
      const alreadyInitDict = yield select((state) => state[storeName].initChooseClassDict);
      if (!alreadyInitDict) {
        const dict = yield _fn.initChooseClassDict();
        yield put({
          type: 'updateState',
          payload: dict,
        });
      }
      const productType = PROJECT_NAME === 'twoteacher' ? TWO_TEACHER_VAL : SMALL_CLASS_VAL;
      // 初始化班级层次字典
      const classType = yield select((state) => {
        const { dict_classType_twoTc, dict_classType_smallClass } = state[storeName];
        return productType === TWO_TEACHER_VAL ? dict_classType_twoTc[0].val : dict_classType_smallClass[0].val;
      });
      const dict_classLevel = yield loadClassLevelDict(classType);
      yield put({
        type: 'updateState',
        payload: { dict_classLevel },
      });
      let initData = yield select((state) => _fn.initChooseClassSKey(state, productType));
      yield put({
        type: 'updateState',
        payload: initData,
      });
      yield put({
        type: 'loadClassList',
        payload: { callback, handleErr },
      });
    },
    /**
     * 加载选择班级列表
     * */
    *loadClassList(action, effects) {
      const {
        payload: { callback, handleErr },
      } = action;
      const { put, call, select } = effects;
      let fetchParams = yield select((state) => {
        const {
          class_className,
          class_productType,
          class_year,
          class_quarter,
          class_phase,
          class_grade,
          class_subject,
          class_classType,
          class_classLevel,
          class_branch,
          class_campus,
          class_mainTc,
          class_auxTc,
        } = state[storeName];
        return {
          branchId: class_branch,
          campusId: class_campus,
          className: class_className || null,
          phaseId: class_phase || null,
          productTypeId: class_productType,
          quarter: class_quarter,
          studentManageId: class_mainTc || null,
          teacherId: class_auxTc || null,
          years: class_year,
          subjectId: _fn.buildClassSKey(class_subject),
          gradeId: _fn.buildClassSKey(class_grade),
          classTypeId: _fn.buildClassSKey(class_classType),
          classLevel: _fn.buildClassSKey(class_classLevel),
        };
      });
      const rsp = yield getClassStudentQualification(fetchParams, true).catch((err) => {
        showMessage(err.text, 'error');
        if (handleErr) handleErr();
      });

      if (rsp && rsp.resultCode === 0) {
        yield put({
          type: 'updateClassList',
          payload: { list: rsp.data },
        });
      }
      yield call(function () {
        let selectedRowKeys = rsp && rsp.data && rsp.data.length > 0 ? rsp.data.map((item, index) => index) : [];
        if (callback) callback(selectedRowKeys);
      });
    },
    /**
     * 重置选择班级检索栏条件
     * */
    *resetClassKeyword(action, effects) {
      const { put, select } = effects;
      const initData = yield select(_fn.initChooseClassSKey);
      let defaultSKey = {
        class_className: '',
        class_productType: null,
        class_year: null,
        class_quarter: null,
        class_phase: null,
        class_grade: [],
        class_subject: [],
        class_classType: [],
        class_classLevel: [],
        class_branch: null,
        class_campus: null,
        class_mainTc: null,
        class_auxTc: null,
        ...initData,
      };
      yield put({
        type: 'updateState',
        payload: defaultSKey,
      });
    },
    /**
     * 更新选择班级检索栏特定条件(产品类型/班级类型)
     * */
    *updateClassKeyword(action, effects) {
      const {
        payload: { type, val, callback },
      } = action;
      const { put, call, select } = effects;
      if (type === 'class_classType') {
        const dict_classLevel = yield loadClassLevelDict(val);
        yield put({
          type: 'updateState',
          payload: {
            dict_classLevel,
          },
        });
      }
      // if (type === 'class_productType' || type === 'class_classType') {
      //   let payload = {};
      //   let classType;
      //   if (type === 'class_productType') {
      //     const strLast = val === TWO_TEACHER_VAL ? 'twoTc' : 'smallClass';
      //     const dict_classType = yield select(state => state[storeName][`dict_classType_${strLast}`]);
      //     classType = dict_classType && dict_classType[0].val ? [ dict_classType[0].val ] : [];
      //     payload.class_classType = classType;
      //   } else {
      //     classType = val;
      //   }
      //   const dict_classLevel = yield loadClassLevelDict(classType);
      //   payload.dict_classLevel = dict_classLevel;
      //   payload.class_classLevel = dict_classLevel && dict_classLevel[0] ? [ dict_classLevel[0].val ] : [];
      //   yield put({
      //     type: 'updateState',
      //     payload,
      //   });
      // }
      yield call(function () {
        if (callback) callback();
      });
    },

    /**
     * 加载选择学生列表
     * */
    *loadStdList(action, effects) {
      const {
        payload: { stdName, callback, handleErr },
      } = action;
      const { put, call, select } = effects;
      let fetchParams = yield select((state) => {
        return { name: stdName };
      });
      const rsp = yield queryStudentsForSendCoupons(fetchParams, true).catch((err) => {
        showMessage(err.text, 'error');
        if (handleErr) handleErr();
        if (callback) callback([]);
      });
      if (rsp) {
        yield put({
          type: 'updateStdList',
          payload: { list: rsp.data },
        });
      }
      yield call(function () {
        let selectedRowKeys = rsp.data.length > 0 ? rsp.data.map((item, index) => index) : [];
        if (callback) callback(selectedRowKeys);
      });
    },

    /**
     * 按班级加载学生列表
     * */
    *loadStdListByClass(action, effects) {
      const {
        payload: { page, rows, callback, handleErr },
      } = action;
      const { put, call, select } = effects;
      const classIdObj = yield select(_fn.buildClassIds);
      let fetchParams = {
        twoteacherClasssId: classIdObj.twoTeacherId,
        smallClassId: classIdObj.smallClassId,
        pageNo: page,
        pageSize: rows,
      };
      const rsp = yield getClassStudentQualificationInfo(fetchParams, true).catch((err) => {
        showMessage(err.text, 'error');
        if (handleErr) handleErr();
      });
      if (rsp) {
        yield put({
          type: 'updateIssueStdList',
          payload: {
            list: rsp.datas,
            total: rsp.rowCount,
            page,
            rows,
          },
        });
      }
      yield call(function () {
        if (callback) callback();
      });
    },
    /**
     * 按页码展示选择的学生列表
     * */
    *loadStdListByStd(action, effects) {
      const {
        payload: { page, rows, callback, handleErr },
      } = action;
      const { put, call, select } = effects;
      const stdChosenList = yield select((state) => {
        const { stdChosenList } = state[storeName];
        return stdChosenList;
      });
      let begin = (page - 1) * rows;
      let end = page * rows;
      yield put({
        type: 'updateState',
        payload: { stdList: stdChosenList.slice(begin, end) },
      });
      yield call(function () {
        if (callback) callback();
      });
    },

    *initMsgTempMenus({ payload: { callback } }, { select, put }) {
      const isNeedInit = yield select((state) => state[storeName].msgTempMenus.length === 0);
      if (isNeedInit) {
        const rsp = yield getSmsTemplate().catch(errorsHandle());
        if (rsp && rsp.resultCode === 0) {
          yield put({
            type: 'updateMsgTempMenus',
            payload: {
              data: rsp.data || [],
            },
          });
        }
      }
      runFunc(callback);
    },
    /**
     * 发券
     * */
    *publishCoupon(action, effects) {
      const {
        payload: { couponId, msgCode, phone, type, callback, handleErr },
      } = action;
      const { put, call, select } = effects;
      let fetchParams = {};
      if (type === 'saleCoupon') {
        // 营销券，通过手机号发送
        fetchParams.phones = phone;
      } else {
        // 普通券
        fetchParams = yield select(_fn.buildPublishParams);
      }
      fetchParams.promotionInfoId = couponId;
      fetchParams.templateCode = msgCode;
      // console.log('发券: ', fetchParams);
      const rsp = yield sendPromotionCoupons(fetchParams, true).catch((err) => {
        showMessage(err.text, 'error');
        if (handleErr) handleErr();
      });
      if (rsp) {
        showMessage(`发券成功！`, 'success');
      }
      yield call(function () {
        if (callback) callback(rsp);
      });
    },

    /**
     * 初始化发券历史
     */
    *initHistory(action, effects) {
      const {
        payload: { couponId, page, rows, callback, handleErr },
      } = action;
      const { put, select } = effects;
      const _initDict = yield select((state) => ({
        dict_branch: state.dict.branch,
        dict_branchCampus: state[storeName].dict_branchCampus,
        dict_campus: state[storeName].dict_campus,
      }));
      const dict = yield _fn.initHistoryAndUsedDict(_initDict);
      yield put({
        type: 'updateState',
        payload: dict,
      });
      // const initData = yield select(_fn.initHistorySKey);
      // yield put({
      //   type: 'updateState',
      //   payload: initData,
      // });
      yield put({
        type: 'loadHistoryList',
        payload: { couponId, page, rows, callback, handleErr },
      });
    },
    /**
     * 加载发券历史列表
     * */
    *loadHistoryList(action, effects) {
      const {
        payload: { couponId, page, rows, callback, handleErr },
      } = action;
      const { call, put, select } = effects;
      let fetchParams = yield select(_fn.buildHistoryParams);
      fetchParams.promotionInfoId = couponId;
      fetchParams.pageNo = page;
      fetchParams.pageSize = rows;
      const rsp = yield listSendPromotionCouponsHistory(fetchParams, true).catch((err) => {
        showMessage(err.text, 'error');
        if (handleErr) handleErr();
      });
      if (rsp) {
        yield put({
          type: 'updateHistoryList',
          payload: { list: rsp.datas, total: rsp.rowCount, page, rows },
        });
      }
      yield call(function () {
        if (callback) callback();
      });
    },
    /**
     * 撤回优惠券
     * */
    *undoCoupon(action, effects) {
      const {
        payload: { id, index, callback, handleErr },
      } = action;
      const { call, put, select } = effects;
      const rsp = yield undoPromotionCoupon({ id }, true).catch((err) => {
        showMessage(err.text, 'error');
        if (handleErr) handleErr();
      });
      if (rsp) {
        showMessage(`撤回成功！`, 'success');
        // yield put({
        //   type: 'updateAfterUndoCoupon',
        //   payload: { index },
        // });
      }
      yield call(function () {
        if (callback) callback();
      });
    },

    /**
     * 初始化用券情况
     * */
    *initUsed(action, effects) {
      const {
        payload: { couponId, page, rows, callback, handleErr },
      } = action;
      const { put, select } = effects;
      const _initDict = yield select((state) => ({
        dict_branch: state.dict.branch,
        dict_branchCampus: state[storeName].dict_branchCampus,
        dict_campus: state[storeName].dict_campus,
      }));
      const dict = yield _fn.initHistoryAndUsedDict(_initDict);
      yield put({
        type: 'updateState',
        payload: dict,
      });
      const rsp = yield getTotalUsedCouponsDiscount({ promotionInfoId: couponId }, true).catch((err) => {
        showMessage(err.text, 'error');
        if (handleErr) handleErr();
      });
      if (rsp) {
        yield put({
          type: 'updateState',
          payload: { couponAmountTotal: rsp.data },
        });
      }
      yield put({
        type: 'loadUsedList',
        payload: { couponId, page, rows, callback, handleErr },
      });
    },
    /**
     * 加载用券情况列表
     * */
    *loadUsedList(action, effects) {
      const {
        payload: { couponId, page, rows, callback, handleErr },
      } = action;
      const { call, put, select } = effects;
      let fetchParams = yield select(_fn.buildUsedParams);
      fetchParams.promotionInfoId = couponId;
      fetchParams.pageNo = page;
      fetchParams.pageSize = rows;
      const rsp = yield listUsedCouponsHistory(fetchParams, true).catch((err) => {
        showMessage(err.text, 'error');
        if (handleErr) handleErr();
      });
      if (rsp) {
        yield put({
          type: 'updateUsedList',
          payload: { list: rsp.datas, total: rsp.rowCount, page, rows },
        });
      }
      yield call(function () {
        if (callback) callback();
      });
    },
    /**
     * 导出excel
     * */
    *exportExcel(action, effects) {
      const {
        payload: { couponId, withContact, callback, handleErr },
      } = action;
      const { call, put, select } = effects;
      let fetchParams = yield select(_fn.buildUsedParams);
      fetchParams.promotionInfoId = couponId;
      fetchParams.exportCustomerContact = withContact;
      const rsp = yield exportUsedCouponsHistory(fetchParams, true).catch((err) => {
        showMessage(err.text, 'error');
        if (handleErr) handleErr();
      });
      if (rsp && rsp.resultCode === 0) {
        let fileName = yield select(_fn.buildExcelName);
        fileName = `${fileName}${withContact ? '(含客户号码)' : '(不含客户号码)'}`;
        yield _fn.downloadFile(rsp.data, fileName);
      }
      yield call(function () {
        if (callback) callback();
      });
    },

    // 满减券导入记录
    *loadImportHistoryData({ payload: { callback } }, { select, put }) {
      const apiParams = yield select((state) => ({
        page: state[storeName].importHistoryPageCurrent,
        rows: state[storeName].importHistoryPageSize,
      }));

      const rsp = yield listImportRecord(apiParams, true).catch(errorsHandle());

      if (rsp && rsp.resultCode === 0 && rsp.data) {
        yield put({
          type: 'updateState',
          payload: {
            importHistoryList: rsp.data.datas,
            importHistoryTotal: rsp.data.rowCount,
          },
        });
      }
      runFunc(callback);
    },
    // 导入满减券
    *importCoupon({ payload: { file, callback } }, { put }) {
      const rsp = yield fileUpload({ file }, `${urlBase}/PromotionController/importStudentSendCoupon.do`).catch(
        errorsHandle(),
      );

      if (rsp && rsp.resultCode === 0) {
        showMessage('导入满减券成功！', 'success');
        yield put({
          type: 'loadImportHistoryData',
          payload: {},
        });
      }

      runFunc(callback);
    },
    // 导出失败数据
    *exportFailExcel({ payload: { id, fileName, callback } }, { put }) {
      const rsp = yield exportImportRecord({ importId: id }, true).catch(errorsHandle());

      if (rsp && rsp.resultCode === 0) {
        yield downloadFile(rsp.data);
      }

      runFunc(callback);
    },
    // 导入历史查询
    *findPageByImportCoupon({ payload }, { put }) {
      const { promotionInfoId, page, rows, callback } = payload;
      const rsp = yield findPageByImportCoupon({ promotionInfoId, page, rows }, true).catch(errorsHandle());
      const { rowCount, datas, resultCode } = rsp || {};
      if (resultCode === 0) {
        yield put({
          type: 'updateState',
          payload: {
            importingHistoryList: datas,
            importingHistoryListTotal: rowCount,
          },
        });
      }
      runFunc(callback);
    },
    // 发券历史 - 导出
    *exportPromotionSendCoupon({ payload }, { put, call }) {
      const result = yield call(exportPromotionSendCoupon, payload);
      if (result && result.resultCode === 0) {
        yield put({ type: 'main/exportNotice' });
      }
    },
  },
};
