/**
 * Created by luoming on 2018/3/29
 */

import React from 'react';
import { connect } from 'dva';
import moment from 'moment';
import Page from 'COMPONENT/Page';
import FormItem from 'COMPONENT/FormItem';
import Button from 'COMPONENT/Button';
import List from 'COMPONENT/List';
import CardList from 'COMPONENT/List/CardList';
import Block from 'COMPONENT/Block';
import { Row, Col } from 'antd';
import styles from './index.less';
import { getPromotionInfoById } from 'SERVICE/preferential';
import { getTextByDictMultipleValue, mapTips } from '../../Activity/common';

const [COL_WIDTH_L, COL_WIDTH_M] = [300, 100];
class View extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      list: [],
      currentPage: 1,
      pageSize: 10,
      totalRecords: 0,
    };
  }
  componentDidMount() {
    this.loadData();
  }
  loadData = async () => {
    let apiParams = { id: this.props.id };
    let { branchDict } = this.props;
    this.setState({ loading: true });
    let { data } = await getPromotionInfoById(apiParams);
    let branchIds = data.branchVos.map((v, k) => {
      return v.branchId;
    });
    let conditions = '';
    if (data.promotionTypeName === '满减') {
      data.ruleVos.forEach((v, k) => {
        conditions += `满${v.promotionRule}${v.ruleUnit === 'MONEY' ? '元' : '课时'}减${v.promotion}元；`;
      });
    } else if (data.promotionTypeName === '每满减') {
      conditions += `每满${data.ruleVos[0].promotionRule}${data.ruleVos[0].ruleUnit === 'MONEY' ? '元' : '课时'}减${
        data.ruleVos[0].promotion
      }元，最高减${data.hightAmount}元；`;
    } else {
      data.ruleVos.forEach((v, k) => {
        conditions += `满${v.promotionRule}${data.promotionTypeName === '现金折扣' ? '元' : '课时'}折${v.promotion}%；`;
      });
    }
    let list = data.productVos.map((item) => item.productVo);
    this.setState({
      loading: false,
      promotionName: data.promotionName,
      receiveLimitName: data.receiveLimitName,
      startDate: moment(data.startDate).format('YYYY-MM-DD'),
      endDate: moment(data.endDate).format('YYYY-MM-DD'),
      receiveEndDate: moment(data.receiveEndDate).format('YYYY-MM-DD'),
      receiveStartDate: moment(data.receiveStartDate).format('YYYY-MM-DD'),
      customerConditionName: data.customerConditionName,
      branch: getTextByDictMultipleValue(branchDict, branchIds),
      contractConditionName: data.contractConditionName,
      status: data.status,
      promotionLittleTypeName: data.promotionLittleTypeName,
      promotionTypeName: data.promotionTypeName,
      conditions,
      promotionComment: data.promotionComment,
      remark: data.remark,
      publishNum: data.publishNum,
      list,
      totalRecords: list.length,
      copyList: JSON.parse(JSON.stringify(list)),
    });
  };

  search = () => {
    let { value, copyList } = this.state;
    let searchList = copyList.filter((v, k) => {
      return v.name.includes(value);
    });
    this.setState({
      list: searchList,
    });
  };
  render() {
    let { loading, value, currentPage, totalRecords, pageSize } = this.state;
    let {
      promotionName,
      receiveLimitName,
      startDate,
      endDate,
      customerConditionName,
      branch,
      contractConditionName,
      status,
      promotionLittleTypeName,
      promotionTypeName,
      conditions,
      getUrl,
      publishNum,
      promotionComment,
      remark,
      list,
    } = this.state;
    let columns = [
      {
        title: '活动信息',
        dataIndex: 'info',
        width: COL_WIDTH_L,
        render: (text, record, index) => {
          let { name, organizationName, courseSeriesName, classTypeName, categoryName, category } = record;
          return (
            <div className={styles.activityInfo}>
              <div>{name}</div>
              <div>
                <span>{organizationName}</span>
                <span>{courseSeriesName}</span>
                <span>{classTypeName}</span>
                <span>{categoryName}</span>
              </div>
              <span>
                <span></span>
                <span>{mapTips[category]}</span>
              </span>
            </div>
          );
        },
      },
      { title: '单价', dataIndex: 'price', width: COL_WIDTH_M },
      { title: '课时', dataIndex: 'miniClassTotalhours', width: COL_WIDTH_M },
      {
        title: '总价',
        dataIndex: 'totalAmount',
        width: COL_WIDTH_M,
        render: (text, record, index) => {
          return <span className={styles.totalPrice}>{record.totalAmount}</span>;
        },
      },
    ];
    return (
      <Page
        loading={loading}
        footer={false}
        route={this.props.route}
        className={styles.coupons}
        ref={(c) => (this.pageRef = c)}
      >
        <div className="flexWrap">
          <Block title="活动信息">
            <div className={styles.wrap}>
              <Row>
                <Col span={12}>
                  <FormItem name="优惠活动名称" size="small">
                    {promotionName}
                  </FormItem>
                </Col>
                <Col span={12}>
                  <FormItem name="客户条件" size="small">
                    {customerConditionName}
                  </FormItem>
                </Col>
                <Col span={12}>
                  <FormItem name="活动时间" size="small">
                    {startDate + '至' + endDate}
                  </FormItem>
                </Col>
                <Col span={12}>
                  <FormItem name="合同条件" size="small">
                    {contractConditionName}
                  </FormItem>
                </Col>
                <Col span={12}>
                  <FormItem name="所属分公司" size="small">
                    {branch}
                  </FormItem>
                </Col>
                <Col span={12}>
                  <FormItem name="状态" size="small">
                    {' '}
                    {status === '0' ? '有效' : '无效'}
                  </FormItem>
                </Col>
                <Col span={12}>
                  <FormItem name="优惠券类型" size="small">
                    {promotionLittleTypeName}
                  </FormItem>
                </Col>
                <Col span={12}>
                  <FormItem name="使用条件" size="small">
                    {conditions}
                  </FormItem>
                </Col>
                <Col span={12}>
                  <FormItem name="领取时间" size="small">
                    {startDate + '至' + endDate}
                  </FormItem>
                </Col>
                <Col span={12}>
                  <FormItem name="领取链接" size="small">
                    {getUrl}
                  </FormItem>
                </Col>
                <Col span={12}>
                  <FormItem name="发放数量" size="small">
                    {publishNum}
                  </FormItem>
                </Col>
                <Col span={12}>
                  <FormItem name="领券方式" size="small">
                    {receiveLimitName}
                  </FormItem>
                </Col>
                <Col span={12}>
                  <FormItem name="活动说明" size="small">
                    {promotionComment}
                  </FormItem>
                </Col>
                <Col span={12}>
                  <FormItem name="活动备注" size="small">
                    {remark}
                  </FormItem>
                </Col>
              </Row>
            </div>
          </Block>
          <Block title="参与产品">
            <div className={styles.searchBar}>
              <input
                type="text"
                value={value}
                placeholder="请输入活动名称"
                onChange={(e) => {
                  this.setState({ value: e.target.value });
                }}
              />
              <Button type="primary" className={styles.searchBtn} onClick={this.search}>
                搜索
              </Button>
            </div>
            <CardList
              expandedRowRender={(record) => this.expandedRowRender()}
              dataSource={list.slice((currentPage - 1) * pageSize, currentPage * pageSize)}
              columns={columns}
              loading={false}
              pagination={{
                current: currentPage,
                total: totalRecords,
                pageSize,
              }}
              expandedRowKeys={[]}
              className={styles.table}
              onChange={({ current, pageSize }) => this.setState({ currentPage: current, pageSize })}
            />
          </Block>
        </div>
      </Page>
    );
  }
}
let mapStateToProps = (state) => {
  let data = state.main.pageParams;
  let pathName = state.routing.locationBeforeTransitions.pathname;
  if (data[pathName]) {
    return {
      type: data[pathName].type,
      id: data[pathName].id,
      branchDict: data[pathName].branchDict,
    };
  } else {
    return {};
  }
};
export default connect(mapStateToProps)(View);
