/**
 * Created by luoming on 2018/3/30
 */
import React from 'react';
import Page from 'COMPONENT/Page';
import { connect } from 'dva';
import moment from 'moment';
import styles from './index.less';
import SearchBar from 'COMPONENT/SearchBar';
import Select from 'COMPONENT/Select';
import Input from 'COMPONENT/Input';
import DatePicker from 'COMPONENT/DatePicker';
import NoData from 'COMPONENT/NoData';
import { Pagination } from 'antd';
import Card from './Card/index';
import { loadCommonDict, loadBranchDict, loadCampusDict } from 'UTIL/dict';
import Modal from './Modal/index';
import { statusDict } from '../../Activity/common';

import {
  getSelectOption,
  getPromotionList,
  savePromotion,
  getPromotionInfoById,
  invalidPromotionById,
  deletePromotionById,
} from 'SERVICE/preferential';

class Manage extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      expandedRowKeys: [],
      currentPage: 1,
      pageSize: 10,
      totalRecords: 100,
    };
  }

  async componentDidMount() {
    this.loadData();
    let [branchDict, { data }] = await Promise.all([
      loadBranchDict(),
      getSelectOption({ category: 'PROMOTION_TYPE', parentId: 'PROMOTION_CODE ' }),
    ]);

    let promotionLittleTypeDict = data.map((v, k) => {
      return { val: v.value, text: v.name };
    });
    this.setState({ branchDict, promotionLittleTypeDict });
  }

  loadData = async (searchKey) => {
    this.setState({ loading: true });
    let {
      branchId,
      promotionKindId,
      promotionLittleTypeId,
      promotionName,
      promotionTypeId,
      startDate,
      endDate,
      statusId,
    } = this.state;
    let apiParams = {
      branchId,
      promotionKindId: 'PROMOTION_CODE', //大类
      promotionLittleTypeId: promotionLittleTypeId,
      promotionName,
      promotionTypeId: '',
      startDate,
      endDate,
      status: statusId,
      pageNo: searchKey ? searchKey.currentPage - 1 : 0,
      pageSize: searchKey ? searchKey.pageSize : 10,
    };
    let { datas, rowCount } = await getPromotionList(apiParams);
    this.setState({
      list: datas,
      totalRecords: rowCount,
      currentPage: searchKey ? searchKey.currentPage : 1,
      pageSize: searchKey ? searchKey.pageSize : 10,
      loading: false,
    });
  };

  loadList = (page, pageSize) => {
    this.loadData({ currentPage: page, pageSize });
  };
  copyCode = () => {
    this.$card.copyCode();
  };
  onExpand = (index, record) => {
    let { expandedRowKeys } = this.state;
    let idx = expandedRowKeys.findIndex((v, k) => {
      return v === index;
    });
    if (idx === -1) {
      expandedRowKeys.push(index);
    } else {
      expandedRowKeys.splice(idx, 1);
    }
    this.setState({ expandedRowKeys });
  };
  getDisabledStartDate = (startDate) => {
    let { endDate } = this.state;
    if (!startDate || !endDate) {
      return false;
    }
    return moment(startDate).valueOf() > moment(endDate).valueOf();
  };
  getDisabledEndDate = (endDate) => {
    const { startDate } = this.state;
    if (!endDate || !startDate) {
      return false;
    }
    return moment(endDate).valueOf() < moment(startDate).valueOf();
  };
  reset = () => {
    this.setState({
      promotionName: null,
      promotionLittleTypeId: null,
      branchId: null,
      startDate: null,
      endDate: null,
      statusId: null,
    });
  };

  showModal = (id) => {
    this.setState({ id, showModal: true });
  };
  render() {
    let {
      loading = false,
      searchExpand = false,
      promotionName,
      promotionLittleTypeDict,
      promotionLittleTypeId,
      branchDict = [],
      branchId,
      startDate,
      endDate,
      statusId,
      list = [],
      currentPage,
      totalRecords,
      pageSize,
      showModal,
      id,
    } = this.state;
    let pagination = {
      showQuickJumper: true,
      showSizeChanger: true,
      pageSizeOptions: ['10', '20', '50'],
      showTotal: (total) => `共 ${total} 条记录`,
      current: currentPage,
      total: totalRecords,
      pageSize,
      onChange: this.loadList,
      onShowSizeChange: this.loadList,
    };
    return (
      <Page loading={loading} footer={false} route={this.props.route} ref={(c) => (this.pageRef = c)}>
        <div className="flexWrap">
          <div className={styles.searchBar}>
            <SearchBar
              expand={searchExpand}
              onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
              onSearch={this.loadData}
              onReset={this.reset}
            >
              <Input
                placeholder="请输入名称"
                value={promotionName}
                onChange={(e) => this.setState({ promotionName: e.target.value })}
              />
              <Select
                placeholder="种类"
                showSearch={true}
                dict={promotionLittleTypeDict}
                value={promotionLittleTypeId}
                onChange={(val) => this.setState({ promotionLittleTypeId: val })}
              />
              <Select
                placeholder="所属分公司"
                showSearch={true}
                dict={branchDict}
                value={branchId}
                onChange={(val) => this.setState({ branchId: val })}
              />
              <DatePicker
                placeholder="生效日期"
                value={startDate}
                onChange={(val) => this.setState({ startDate: val })}
                disabledDate={(val) => this.getDisabledStartDate(val)}
              />
              <DatePicker
                placeholder="截止日期"
                value={endDate}
                onChange={(val) => this.setState({ endDate: val })}
                disabledDate={(val) => this.getDisabledEndDate(val)}
              />
              <Select
                placeholder="状态"
                showSearch={true}
                dict={statusDict}
                value={statusId}
                onChange={(val) => this.setState({ statusId: val })}
              />
            </SearchBar>
          </div>
          <div className="flexInnerFit">
            {list.map((v, k) => {
              return (
                <Card
                  key={k}
                  data={v}
                  ref={(e) => (this.$card = e)}
                  branchDict={branchDict}
                  showModal={this.showModal}
                />
              );
            })}
            {list.length === 0 && <NoData />}
          </div>
          {list.length > 0 && (
            <div style={{ display: 'block' }}>
              <Pagination {...pagination} />
            </div>
          )}
        </div>
        {showModal && <Modal id={id} onClose={(e) => this.setState({ showModal: false })} />}
      </Page>
    );
  }
}

export default Manage;
