/**
 * Created by luoming on 2018/3/29
 */
import React from 'react';
import { connect } from 'dva';
import moment from 'moment';
import styles from './index.less';
import List from 'COMPONENT/List';
import SearchBar from 'COMPONENT/SearchBar';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import Input from 'COMPONENT/Input';
import Button from 'COMPONENT/Button';
import DatePicker from 'COMPONENT/DatePicker';
import ShowMessage from 'COMPONENT/utils/globalTips/showMessage';
import ShowConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import { loadCommonDict, loadBranchDict, loadCampusDict } from 'UTIL/dict';

import {
  getSelectOption,
  getPromotionList,
  savePromotion,
  getPromotionInfoById,
  invalidPromotionById,
  deletePromotionById,
} from 'SERVICE/preferential';

import { mapPromotionType, getTextByDictMultipleValue, statusDict } from '../Activity/common';

let { COL_WIDTH_L, COL_WIDTH_M, COL_WIDTH_S, COL_WIDTH_XL } = List;

class Coupons extends React.Component {
  constructor(props) {
    super(props);
    let promotionLittleTypeId = mapPromotionType(this.props.type);
    this.state = {
      branchDict: [],
      statusDict,
      currentPage: 0,
      totalRecords: 0,
      pageSize: 10,
      loading: false,
      searchExpand: false,
      checkedProductKeys: [],
      list: [],
      promotionLittleTypeId,
      branchId: null,
    };
  }

  async componentDidMount() {
    let [branchDict, { data }] = await Promise.all([
      loadBranchDict(),
      getSelectOption({ category: 'ACTIVITY_TYPE', parentId: this.state.promotionLittleTypeId }),
    ]);
    let typeDict = data.map((v, k) => {
      return { val: v.value, text: v.name };
    });
    await this.setState({
      // branchId: [ branchDict[0].val ],
      branchDict,
      typeDict,
    });
    this.loadData();
  }

  loadData = async (searchKey) => {
    this.setState({ loading: true });
    let {
      branchId,
      promotionKindId,
      promotionLittleTypeId,
      promotionName,
      promotionTypeId,
      startDate,
      endDate,
      statusId,
    } = this.state;
    let apiParams = {
      branchId: this.auxBuildParams(branchId),
      promotionKindId: 'PROMOTION_CODE', //大类
      promotionLittleTypeId: promotionLittleTypeId,
      promotionName,
      promotionTypeId,
      startDate,
      endDate,
      status: statusId,
      pageNo: searchKey ? searchKey.currentPage - 1 : 0,
      pageSize: searchKey ? searchKey.pageSize : 10,
    };
    let { datas, rowCount } = await getPromotionList(apiParams);
    this.setState({
      list: datas,
      totalRecords: rowCount,
      currentPage: searchKey ? searchKey.currentPage : 1,
      pageSize: searchKey ? searchKey.pageSize : 10,
      loading: false,
    });
  };
  auxBuildParams = (params) => {
    let result = null;
    if (params && params.length > 0) {
      result = params.length > 1 ? params.join(',') : params[0];
    }
    return result;
  };
  search = async () => {
    this.loadData({ currentPage: 1, pageSize: 10 });
  };
  reset = () => {
    this.setState({
      branchId: null,
      promotionName: null,
      promotionTypeId: null,
      startDate: null,
      endDate: null,
      statusId: null,
    });
  };
  loadList = async (searchKey) => {
    this.loadData(searchKey);
  };

  openNewPage = (handle, record, index) => {
    let { dispatch, type } = this.props;
    let { branchDict } = this.state;
    if (handle === 'view') {
      dispatch({
        type: 'main/openPage',
        payload: { path: `/${type}/View`, params: { id: record.id, branchDict, type }, refresh: true },
      });
    } else {
      this.AddOrEdit('Edit', record.id);
    }
  };
  onSaveSuccess = async () => {
    const { dispatch, type } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: { path: `/${type}`, params: {} },
    });
    this.loadData({ currentPage: 1, pageSize: 10 });
  };
  AddOrEdit = (handle, id) => {
    let { dispatch, type } = this.props;
    let { branchDict, typeDict } = this.state;
    dispatch({
      type: 'main/openPage',
      payload: {
        path: `/${type}/${handle}`,
        params: { type, id, handle, branchDict, typeDict, onSaveSuccess: this.onSaveSuccess },
        refresh: true,
      },
    });
  };

  getUrl = (record, index) => {};
  deletePromotion = (id, handle, status) => {
    if (handle === 'cancel') {
      ShowConfirm(
        `是否将该优惠设为${status === '1' ? '无效' : '有效'}？`,
        async () => {
          await invalidPromotionById({ id, status });
          ShowMessage('操作成功！', 'success');
          this.loadData();
        },
        () => {},
      );
    } else {
      ShowConfirm('该操作将删除此优惠信息，是否继续？', async () => {
        await deletePromotionById({ id });
        ShowMessage('删除成功', 'success');
        this.loadData();
      });
    }
  };
  getDisabledStartDate = (startDate) => {
    let { endDate } = this.state;
    if (!startDate || !endDate) {
      return false;
    }
    return moment(startDate).valueOf() > moment(endDate).valueOf();
  };
  getDisabledEndDate = (endDate) => {
    const { startDate } = this.state;
    if (!endDate || !startDate) {
      return false;
    }
    return moment(endDate).valueOf() < moment(startDate).valueOf();
  };

  render() {
    let {
      currentPage,
      totalRecords,
      pageSize,
      loading,
      searchExpand,
      promotionName,
      typeDict,
      promotionTypeId,
      branchDict,
      branchId,
      startDate,
      endDate,
      statusDict,
      statusId,
      checking,
      checkedProductKeys = [],
      list,
    } = this.state;

    let columns = [
      { title: '促销码名称', dataIndex: 'promotionName', width: COL_WIDTH_L, fixed: 'left' },
      // {title: '编码', dataIndex: 'encrypt', width: COL_WIDTH_L,},
      { title: '种类', dataIndex: 'promotionLittleTypeName', width: COL_WIDTH_M },
      { title: '小类', dataIndex: 'promotionTypeName', width: COL_WIDTH_M },
      {
        title: '使用条件',
        dataIndex: 'conditions',
        width: COL_WIDTH_XL,
        render: (text, record, index) => {
          let { ruleVos, promotionTypeName, hightAmount } = record;
          let condition = '';
          if (promotionTypeName === '满减') {
            ruleVos.forEach((v, k) => {
              condition += `满${v.promotionRule}${v.ruleUnit === 'MONEY' ? '元' : '课时'}减${v.promotion}元；`;
            });
          } else if (promotionTypeName === '每满减') {
            condition += `每满${ruleVos[0].promotionRule}${ruleVos[0].ruleUnit === 'MONEY' ? '元' : '课时'}减${
              ruleVos[0].promotion
            }元，最高减${hightAmount}元；`;
          } else {
            ruleVos.forEach((v, k) => {
              condition += `满${v.promotionRule}${promotionTypeName === '现金折扣' ? '元' : '课时'}折${v.promotion}%；`;
            });
          }
          return <span>{condition}</span>;
        },
      },
      { title: '客户条件', dataIndex: 'customerConditionName', width: COL_WIDTH_M },
      { title: '合同条件', dataIndex: 'contractConditionName', width: COL_WIDTH_M },
      { title: '发码量', dataIndex: 'publishNum', width: COL_WIDTH_S },

      { title: '用码量', dataIndex: 'useNum', width: COL_WIDTH_S },

      // {title: '使用平台', dataIndex: 'platform', width: COL_WIDTH_M},
      {
        title: '所属分公司',
        dataIndex: 'branch',
        width: COL_WIDTH_L,
        render: (text, record, index) => {
          let { branchVos } = record;
          let branchIds = branchVos.map((v, k) => {
            return v.branchId;
          });
          return <span>{getTextByDictMultipleValue(branchDict, branchIds)}</span>;
        },
      },
      {
        title: '起止时间',
        dataIndex: 'useTime',
        width: 280,
        render: (text, record, index) => {
          const { startDate, endDate } = record;
          return (
            <span>
              {moment(startDate).format('YYYY-MM-DD HH:mm:ss')}至{moment(endDate).format('YYYY-MM-DD HH:mm:ss')}
            </span>
          );
        },
      },
      {
        title: '状态',
        dataIndex: 'status',
        width: COL_WIDTH_S,
        render: (text, record, index) => {
          let { status } = record;
          return <span>{status === '0' ? '有效' : '无效'}</span>;
        },
      },
      {
        title: '操作',
        key: 'action',
        width: COL_WIDTH_XL,
        getActionConfig: (text, record, index) => {
          let { id, status, startDate, endDate } = record;
          let isBetween = moment().isBetween(moment(startDate).subtract(1, 'days'), moment(endDate).add(1, 'days'));
          return [
            { text: '查看', onClick: () => this.openNewPage('view', record, index) },
            { text: '编辑', onClick: () => this.openNewPage('edit', record, index) },
            {
              menus: [
                {
                  text: status === '0' ? '无效' : '有效',
                  onClick: () => this.deletePromotion(id, 'cancel', status === '0' ? '1' : '0'),
                  disabled: !isBetween,
                },
                { text: '删除', onClick: () => this.deletePromotion(id, 'del'), disabled: false },
              ],
            },
          ];
        },
      },
    ];
    return (
      <div className={styles.coupons}>
        <div className="flexWrap">
          <div className={styles.searchBar}>
            <Button className={styles.add} onClick={(e) => this.AddOrEdit('Add')}>
              新增
            </Button>
            <SearchBar
              expand={searchExpand}
              onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
              onSearch={this.search}
              onReset={this.reset}
            >
              <Input
                placeholder="请输入名称"
                value={promotionName}
                onChange={(e) => this.setState({ promotionName: e.target.value })}
              />
              <Select
                placeholder="种类"
                showSearch={true}
                dict={typeDict}
                value={promotionTypeId}
                onChange={(val) => this.setState({ promotionTypeId: val })}
              />
              <MultiSelect
                placeholder="所属分公司"
                showSearch={true}
                dict={branchDict}
                value={branchId}
                onChange={(val) => this.setState({ branchId: val })}
              />
              <DatePicker
                placeholder="起始日期"
                value={startDate}
                onChange={(val) => this.setState({ startDate: val })}
                disabledDate={(val) => this.getDisabledStartDate(val)}
              />
              <DatePicker
                placeholder="截止日期"
                value={endDate}
                onChange={(val) => this.setState({ endDate: val })}
                disabledDate={(val) => this.getDisabledEndDate(val)}
              />
              <Select
                placeholder="状态"
                showSearch={true}
                dict={statusDict}
                value={statusId}
                onChange={(val) => this.setState({ statusId: val })}
              />
            </SearchBar>
          </div>
          <div className="flexInnerFit">
            <List
              dataSource={list}
              columns={columns}
              loading={loading}
              // rowSelection={checking ? {
              //   selectedRowKeys: checkedProductKeys,
              //   onChange: (keys) => this.setState({checkedProductKeys: keys}),
              // } : null}
              pagination={{
                current: currentPage,
                total: totalRecords,
                pageSize,
              }}
              onChange={({ current, pageSize }) => this.loadList({ currentPage: current, pageSize })}
            />
          </div>
        </div>
      </div>
    );
  }
}
let mapStateToProps = (state) => {
  return {};
};
export default connect(mapStateToProps)(Coupons);
