import React from 'react';
import PropType from 'prop-types';
import classNames from 'classnames';
import { connect } from 'dva';
import moment from 'moment';
import { Row, Col, DatePicker, Tooltip } from 'antd';
import {
  Page,
  Button,
  Form,
  Input,
  Select,
  Radio,
  TextArea,
  DropdownMenu,
  InputNumber,
  TextLink,
  Checkbox,
} from 'COMPONENT';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { getRemarkTemplates, savePromotion, getPromotionInfoById, getPromotionProductById } from 'SERVICE/preferential';
import { loginUser, runFunc, errorsHandle, deleteKeyByFilter } from 'UTIL/common';
import {
  PRODUCT_TYPE_SHUANG_SHI,
  xhbossProductTypeDict,
  ssbossProductTypeDict,
  limitScopeDict,
  publishObjectDict,
  receiveLimitDict,
  superpositionDict,
  periodValidDict,
  PERIOD_VALID_REGULAR,
  PERIOD_VALID_RELATIVE,
  BOSS,
  LIMIT_SCOPE_INFINITE,
  LIMIT_SCOPE_FINITE,
  LIMIT_SCOPE_PRODUCT_TYPE,
  RECEIVE_LIMIT_INFINITE,
  RECEIVE_LIMIT_FOR_PHONE,
  RECEIVE_LIMIT_FOR_STUD,
  CUSTOMER,
  STUDENT,
  COUPON,
  COUPON_CASH,
  timeFormat,
  rangeFormat,
} from '../config';
import ModalSelectPrd from './ModalSelectPrd';
import { submitDataAdapter, fieldDataAdapter } from './utils';
import Title from './components/Title';
import TipIcon from './components/TipIcon';

import style from './styles.module.less';

const FormItem = Form.Item;
const RadioGroup = Radio.Group;
const { RangePicker } = DatePicker;

const defaultBeginTime = '00:00:00';
const defaultEndTime = '23:59:59';

const receiveLimitRenderDict = [
  {
    val: RECEIVE_LIMIT_INFINITE,
    relevance: '',
    render: () => '不限制',
  },
  {
    val: RECEIVE_LIMIT_FOR_PHONE,
    relevance: CUSTOMER,
    render: (numVal, onChange, disalbed = false) => {
      return (
        <span className={style.receiveNumWrapper}>
          每个手机号限发送
          {disalbed ? (
            numVal
          ) : (
            <InputNumber
              className={style.receiveNumLimitInput}
              min={1}
              precision={0}
              value={numVal}
              onChange={onChange}
            />
          )}
          次
        </span>
      );
    },
  },
  {
    val: RECEIVE_LIMIT_FOR_STUD,
    relevance: STUDENT,
    render: (numVal, onChange, disalbed = false) => {
      return (
        <span className={style.receiveNumWrapper}>
          每个学生限发送
          {disalbed ? (
            numVal
          ) : (
            <InputNumber
              className={style.receiveNumLimitInput}
              min={1}
              precision={0}
              value={numVal}
              onChange={onChange}
            />
          )}
          次
        </span>
      );
    },
  },
];

class AddEdit extends React.Component {
  constructor(props) {
    super(props);

    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.loadTemplateMenus = this.loadTemplateMenus.bind(this);
    this.loadData = this.loadData.bind(this);
    this.handledSave = this.handledSave.bind(this);
    this.submit = this.submit.bind(this);
    this.promotionNameValidator = this.promotionNameValidator.bind(this);
    this.limitScopeValidator = this.limitScopeValidator.bind(this);

    this.handleFieldsChange = this.handleFieldsChange.bind(this);
    this.updatePromotionComment = this.updatePromotionComment.bind(this);

    this.switchModalSelectPrd = this.switchModalSelectPrd.bind(this);
    this.handleSelect = this.handleSelect.bind(this);
    this.goBackRefresh = this.goBackRefresh.bind(this);

    this.handleReceiveNumLimitChange = this.handleReceiveNumLimitChange.bind(this);
    this.renderLimitScopeContent = this.renderLimitScopeContent.bind(this);

    // 表单引用
    this.formRef = null;

    const params = props.pageParams[`/${props.route.path}`];

    this.couponId = params.couponId || null;
    this.platType = BOSS;
    this.isBossPlat = this.platType === BOSS;
    this.isSend = params.isSend;
    this.isNotPeiYouSym = !loginUser.isPeiYouSym();

    const productTypeDict = loginUser.isXingHuoSym() ? xhbossProductTypeDict : ssbossProductTypeDict;
    const productTypeOptions = productTypeDict.map((el) => ({
      value: el.val,
      label: el.text,
    }));
    const productTypeList = productTypeDict.map((el) => el.val);

    this.state = {
      loading: false,
      mounted: false,
      // 产品类型 checkbox 数据源
      productTypeOptions,
      // 产品类型列表 string[]
      productTypeList,
      // 分公司 ID
      branchId: null,
      // 产品范围
      limitScope: LIMIT_SCOPE_INFINITE,
      // 需要限制的产品类型
      limitProductTypes: productTypeList[0],
      // 需要限制的产品列表
      prdList: [],
      // 需要限制的产品列表数量
      total: 0,
      // 发放对象
      publishObject: CUSTOMER,
      // 发送限制
      receiveLimit: RECEIVE_LIMIT_INFINITE,
      // 领取次数限制
      receiveNumLimit: 1,
      // 有效期类型，默认固定有效期
      dateType: periodValidDict[0].val,
      // 使用说明历史
      templateMenus: [{ text: '暂无使用说明历史', disabled: true }],
      modalVisible: false,
      modalInitSKey: {},
      modalVariety: null,
      detailStorage: {},
    };
  }

  componentDidMount() {
    this.init();
  }

  /**
   * 初始化数据
   * 包括字典、使用说明历史记录、代金券数据
   */
  async init() {
    this.setState({
      loading: true,
      mounted: false,
    });

    await new Promise(this.initDict);
    await new Promise(this.loadTemplateMenus);

    if (this.couponId) {
      await new Promise(this.loadData);
    } else {
      this.initBranchId();
    }

    this.setState({
      loading: false,
      mounted: true,
    });
  }

  /**
   * 初始化字典
   * @param {Function} resolve promise resolve
   */
  initDict(resolve) {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['branch'],
        callback() {
          resolve();
        },
      },
    });
  }

  initBranchId() {
    if (this.couponId) {
      return;
    }

    const { branchDict } = this.props;

    if (branchDict.length === 1) {
      this.setState(
        {
          branchId: branchDict[0].val,
        },
        () => {
          this.formRef.setFieldsValue({
            branchId: branchDict[0].val,
          });
        },
      );
    }
  }

  /**
   * 加载使用说明历史记录
   * @param {Function} resolve
   */
  async loadTemplateMenus(resolve) {
    const rsp = await getRemarkTemplates(
      {
        promotionKindId: COUPON,
        promotionLittleTypeId: COUPON_CASH,
        rows: 3,
      },
      true,
    ).catch(errorsHandle());
    if (rsp && rsp.resultCode === 0 && Array.isArray(rsp.data) && rsp.data.length > 0) {
      this.setState({
        templateMenus: rsp.data.map((item) => ({
          text: item,
          onClick: this.updatePromotionComment,
        })),
      });
    }
    resolve();
  }

  updatePromotionComment({ key }) {
    if (key) {
      const promotionComment = this.state.templateMenus[key].text || null;
      this.formRef.setFieldsValue({ promotionComment });
    }
  }

  async loadData(resolve) {
    let fetchList = [
      getPromotionInfoById({ id: this.couponId }, true),
      getPromotionProductById({ promotionInfoId: this.couponId }, true),
    ];
    const [rspDetail, rspProduct] = await Promise.all(fetchList, true).catch(errorsHandle());
    let payload = {};
    let fieldData = null;

    if (rspDetail && rspDetail.data) {
      const { receiveNumLimit, limitProductTypes } = rspDetail.data;
      payload.detailStorage = rspDetail.data;

      fieldData = fieldDataAdapter(rspDetail.data);

      payload.branchId = fieldData.branchId;
      payload.limitScope = fieldData.limitScope ? fieldData.limitScope.toString() : fieldData.limitScope;

      // 限制产品类型 状态
      payload.limitProductTypes = Array.isArray(limitProductTypes)
        ? limitProductTypes.filter(
            (el) => !!~this.state.productTypeList.indexOf(el) && el !== PRODUCT_TYPE_SHUANG_SHI,
          )[0]
        : this.state.productTypeList.slice(0, 1)[0];
      payload.publishObject = fieldData.publishObject;
      payload.receiveLimit = fieldData.receiveLimit;
      payload.receiveNumLimit = Number(receiveNumLimit) > -1 ? receiveNumLimit : 1;
      payload.dateType = fieldData.dateType;

      payload.detailStorage.limitProductTypes = payload.limitProductTypes;

      fieldData.limitProductTypes = payload.limitProductTypes;
      fieldData = deleteKeyByFilter(fieldData, (key) => fieldData[key] == null);
    }

    if (rspProduct && rspProduct.data) {
      payload.prdList = rspProduct.data || [];
      payload.total = payload.prdList.length;
    }

    this.setState(payload, () => {
      if (fieldData) {
        this.formRef.setFieldsValue(fieldData);
      }
      resolve();
    });
  }

  /**
   * 保存代金券修改结果
   */
  handledSave() {
    runFunc(this.formRef.submitFields);
  }

  /**
   * 表单提交 handler
   * @param {Object} values
   */
  async submit(values) {
    this.setState({ loading: true });

    const { receiveNumLimit, prdList, detailStorage } = this.state;
    const receiveNumLimitObj = values.receiveLimit === RECEIVE_LIMIT_INFINITE ? {} : { receiveNumLimit }; // 不限制发送次数时，不传 receiveNumLimit
    const apiParams = submitDataAdapter(
      { id: this.couponId, productVos: prdList, ...receiveNumLimitObj, ...values },
      { ...(detailStorage || {}) },
    );
    const rsp = await savePromotion(apiParams, true).catch(errorsHandle());

    if (rsp && rsp.resultCode === 0) {
      showMessage(`${apiParams.id ? '修改' : '新增'}代金券成功！`, 'success');
      return this.goBackRefresh();
    }

    this.setState({ loading: false });
  }

  submitFail = (err) => {
    showMessage('请完善好所有信息后再保存！', 'error');
  };

  // 代金券名称校验
  promotionNameValidator(_, value, callback) {
    if (!value || value.length === 0) {
      return callback('代金券名称不能为空');
    }
    if (value.length > 100) {
      return callback('代金券名称字数不能多于100个字');
    }
    callback();
  }

  // 校验产品范围
  limitScopeValidator(_, value, callback) {
    if (value === LIMIT_SCOPE_FINITE && this.state.total === 0) {
      return callback('限制商品数量不能为0');
    }
    callback();
  }

  handleFieldsChange(fields) {
    if (!this.state.mounted) {
      return;
    }

    // 所属分公司✔
    if (fields.branchId && fields.branchId.value !== this.state.branchId) {
      const payload = {
        branchId: fields.branchId.value,
      };

      if (this.state.branchId !== null) {
        payload.prdList = [];
        payload.total = 0;
      }

      this.setState(payload);
    }

    // 产品范围✔
    if (fields.limitScope && fields.limitScope.value !== this.state.limitScope) {
      const payload = {
        limitScope: fields.limitScope.value,
      };

      if (fields.limitScope.value === LIMIT_SCOPE_INFINITE) {
        payload.prdList = [];
        payload.total = 0;
      }

      this.setState(payload);
    }

    // 发放对象✔
    if (fields.publishObject && fields.publishObject.value !== this.state.publishObject) {
      this.setState(
        {
          publishObject: fields.publishObject.value,
        },
        () => {
          if (this.state.receiveLimit !== RECEIVE_LIMIT_INFINITE) {
            this.formRef.setFieldsValue({
              receiveLimit: RECEIVE_LIMIT_INFINITE,
            });
          }
        },
      );
    }

    // 发送限制✔
    if (fields.receiveLimit && fields.receiveLimit.value !== this.state.receiveLimit) {
      this.setState({ receiveLimit: fields.receiveLimit.value });
    }

    // 有效期✔
    if (fields.dateType && fields.dateType.value !== this.state.dateType) {
      this.setState(
        {
          dateType: fields.dateType.value,
        },
        () => {
          const payload = fields.dateType.value === PERIOD_VALID_REGULAR ? { relativePeriod: null } : { date: null };
          this.formRef.setFieldsValue(payload);
        },
      );
    }
  }

  switchModalSelectPrd() {
    this.setState((prevState) => {
      if (prevState.modalVisible) {
        this.modalInitSKey = {};
      } else {
        this.modalInitSKey = { organizationId: this.state.branchId };
        if (this.state.total > 0) {
          if (this.isBossPlat) {
            this.modalInitSKey.category = this.state.prdList[0].category;
          } else {
            this.modalInitSKey.type = this.state.prdList[0].type;
          }
        }
      }
      return { modalVisible: !prevState.modalVisible };
    });
  }

  handleSelect(list) {
    this.setState(
      {
        prdList: list,
        total: list.length,
      },
      () => {
        // 已选产品更新时，触发`limitScope`校验
        this.formRef.validateFields(['limitScope'], { force: true });
      },
    );
    this.switchModalSelectPrd();
  }

  handleReceiveNumLimitChange(value) {
    this.setState({
      receiveNumLimit: value,
    });
  }

  goBackRefresh() {
    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path: '/cashVoucher',
        refresh: true,
      },
    });
  }

  renderLimitScopeContent(limitScope, disabled) {
    const limitScopeContentMap = {
      [LIMIT_SCOPE_INFINITE]: {
        val: LIMIT_SCOPE_INFINITE,
        render: () => null,
      },
      [LIMIT_SCOPE_FINITE]: {
        val: LIMIT_SCOPE_FINITE,
        render: (state, disabled) => {
          return (
            <div className={style.checkboxContent}>
              <span style={{ marginRight: 16 }}>
                已选
                <span className={style.prdNum}>{state.total}</span>
                个产品
              </span>
              {state.branchId ? (
                <TextLink onClick={this.switchModalSelectPrd} disabled={disabled}>
                  选择产品
                </TextLink>
              ) : (
                <Tooltip title="请先选择分公司" placement="top">
                  <TextLink disabled>选择产品</TextLink>
                </Tooltip>
              )}
            </div>
          );
        },
      },
    };

    const { render } = limitScopeContentMap[limitScope];

    return render(this.state, disabled);
  }

  render() {
    // 是否固定有效期
    const isFixedPeriod = this.state.dateType === PERIOD_VALID_REGULAR;
    // 是否相对有效期
    const isRelativePeriod = this.state.dateType === PERIOD_VALID_RELATIVE;

    const { branchDict } = this.props;

    return (
      <Page
        loading={this.state.loading}
        route={this.props.route}
        footer={
          <div className={style.pageFooter}>
            <Button onClick={this.handledSave} type="primary">
              保存
            </Button>
          </div>
        }
        fitWindowHeight={false}
      >
        <div className={style.formWrap}>
          <Form
            ref={(c) => (this.formRef = c)}
            onSubmit={this.submit}
            onSubmitFail={this.submitFail}
            onFieldsChange={this.handleFieldsChange}
          >
            <Title>基础属性</Title>

            <FormItem
              name="promotionName"
              rules={[{ required: true, validator: this.promotionNameValidator }]}
              formItemProps={{
                label: '代金券名称',
                colon: false,
                labelCol: { span: 4 },
                wrapperCol: { span: 20 },
              }}
            >
              <Input placeholder="代金券名称" disabled={this.isSend} />
            </FormItem>
            <FormItem
              name="branchId"
              rules={[{ required: true, message: '所属分公司不能为空' }]}
              formItemProps={{
                label: '所属分公司',
                colon: false,
                labelCol: { span: 4 },
                wrapperCol: { span: 20 },
              }}
            >
              <Select placeholder="所属分公司" dict={branchDict} showSearch allowClear={false} disabled={this.isSend} />
            </FormItem>

            <Title>使用条件</Title>
            <FormItem
              name="limitProductTypes"
              initialValue={this.state.productTypeOptions[0].value}
              rules={[{ required: true, message: '产品类型不能为空' }]}
              formItemProps={{ colon: false, label: '产品类型', labelCol: { span: 4 }, wrapperCol: { span: 20 } }}
            >
              <RadioGroup>
                {this.state.productTypeOptions.map((item) => (
                  <Radio value={item.value} key={item.value} disabled={this.isSend}>
                    {item.label}
                  </Radio>
                ))}
              </RadioGroup>
            </FormItem>
            {this.isNotPeiYouSym && (
              <FormItem
                name="limitScope"
                rules={[{ required: true, validator: this.limitScopeValidator }]}
                initialValue={limitScopeDict[0].val}
                formItemProps={{
                  label: '产品范围',
                  colon: false,
                  labelCol: { span: 4 },
                  wrapperCol: { span: 20 },
                }}
              >
                <RadioGroup>
                  {limitScopeDict.map((item) => (
                    <Radio key={item.val} value={item.val} disabled={this.isSend}>
                      <div className={style.checkboxContentWrapper}>
                        <span>{item.text}</span>
                        {this.state.limitScope === item.val && this.renderLimitScopeContent(item.val, this.isSend)}
                      </div>
                    </Radio>
                  ))}
                </RadioGroup>
              </FormItem>
            )}
            {this.isBossPlat && (
              <FormItem
                name="publishObject"
                rules={[{ required: true, message: '发放对象不能为空' }]}
                initialValue={publishObjectDict[0].val}
                formItemProps={{
                  label: '发放对象',
                  colon: false,
                  labelCol: { span: 4 },
                  wrapperCol: { span: 20 },
                }}
                slot={
                  <TipIcon
                    title="家长：代金券创建后可直接发放到家长手机号码下，只需要导入家长号码进行发放;&nbsp;&nbsp;&nbsp;&#10;学生：代金券创建后需要发放到学生名下，需要导入学生ID进行发放"
                  />
                }
              >
                <RadioGroup>
                  {publishObjectDict.map((item) => (
                    <Radio value={item.val} key={item.val} disabled={this.isSend}>
                      {item.text}
                    </Radio>
                  ))}
                </RadioGroup>
              </FormItem>
            )}
            {this.isBossPlat && (
              <FormItem
                name="receiveLimit"
                rules={[{ required: true, message: '发送限制不能为空' }]}
                initialValue={receiveLimitDict[0].val}
                formItemProps={{
                  label: '发送限制',
                  colon: false,
                  labelCol: { span: 4 },
                  wrapperCol: { span: 20 },
                }}
              >
                <RadioGroup>
                  {receiveLimitRenderDict.map((item) => (
                    <Radio
                      className={classNames({
                        [style.displayNone]:
                          item.val !== RECEIVE_LIMIT_INFINITE && this.state.publishObject !== item.relevance,
                      })}
                      value={item.val}
                      key={item.val}
                      disabled={this.isSend}
                    >
                      {item.render(this.state.receiveNumLimit, this.handleReceiveNumLimitChange, this.isSend)}
                    </Radio>
                  ))}
                </RadioGroup>
              </FormItem>
            )}
            {this.isBossPlat && this.isNotPeiYouSym && (
              <FormItem
                name="superPosition"
                rules={[{ required: true, message: '叠加方式不能为空' }]}
                initialValue={superpositionDict[1].val}
                formItemProps={{
                  label: '叠加方式',
                  colon: false,
                  labelCol: { span: 4 },
                  wrapperCol: { span: 20 },
                }}
                slot={<TipIcon title="是否可以与其他无门槛代金券叠加使用" />}
              >
                <RadioGroup>
                  {superpositionDict.map((item) => (
                    <Radio value={item.val} key={item.val} disabled={this.isSend}>
                      {item.text}
                    </Radio>
                  ))}
                </RadioGroup>
              </FormItem>
            )}

            <Title>活动规则</Title>

            <FormItem
              name="dateType"
              rules={[{ required: true, message: '有效期方式不能为空' }]}
              initialValue={periodValidDict[0].val}
              formItemProps={{
                label: '有效期方式',
                colon: false,
                labelCol: { span: 4 },
                wrapperCol: { span: 20 },
              }}
            >
              <RadioGroup>
                {periodValidDict.map((item) => (
                  <Radio value={item.val} key={item.val} disabled={this.isSend}>
                    {item.text}
                  </Radio>
                ))}
              </RadioGroup>
            </FormItem>
            <FormItem
              name="date"
              rules={[{ required: isFixedPeriod, message: '有效期不能为空' }]}
              wrapClassName={isRelativePeriod ? style.displayNone : ''}
              formItemProps={{
                label: '有效期',
                colon: false,
                labelCol: { span: 4 },
                wrapperCol: { span: 20 },
              }}
            >
              <RangePicker
                placeholder={['开始时间', '结束时间']}
                showTime={{
                  hideDisabledOptions: true,
                  defaultValue: [moment(defaultBeginTime, timeFormat), moment(defaultEndTime, timeFormat)],
                }}
                format={rangeFormat}
                size="large"
                disabled={this.isSend}
              />
            </FormItem>
            <FormItem
              name="relativePeriod"
              rules={[{ required: isRelativePeriod, message: '有效期不能为空' }]}
              wrapClassName={isFixedPeriod ? style.displayNone : ''}
              formItemProps={{
                label: '有效期',
                colon: false,
                labelCol: { span: 4 },
                wrapperCol: { span: 20 },
              }}
              slot={
                <Row className={style.rowPeriodNum}>
                  <Col span={4} />
                  <Col span={4}>
                    <span className={style.prevPeriodText}>领取或发券后</span>
                  </Col>
                  <Col span={4} />
                  <Col span={4}>
                    <span>天有效</span>
                  </Col>
                </Row>
              }
            >
              <InputNumber className={style.inputPeriodNum} min={1} precision={0} disabled={this.isSend} />
            </FormItem>

            <FormItem
              name="hightAmount"
              rules={[{ required: true, message: '代金券面额不能为空' }]}
              formItemProps={{
                label: '代金券面额',
                colon: false,
                labelCol: { span: 4 },
                wrapperCol: { span: 20 },
              }}
              slot={
                <Row className={style.cashAmountRow}>
                  <Col span={8} />
                  <Col span={4}>
                    <span>元</span>
                  </Col>
                </Row>
              }
            >
              <InputNumber className={style.cashAmountInputNum} min={0.01} precision={2} disabled={this.isSend} />
            </FormItem>

            <FormItem
              name="promotionComment"
              formItemProps={{
                label: '使用说明',
                colon: false,
                labelCol: { span: 4 },
                wrapperCol: { span: 20 },
              }}
              wrapClassName={style.FortItemComment}
              slot={
                <div className={style.DropdownMenuWrap}>
                  <Row>
                    <Col span={4} />
                    <Col span={8}>
                      <DropdownMenu
                        dropdownClassName={style.dropDownMenu}
                        text="使用说明模板"
                        menus={this.state.templateMenus}
                        hasArrow={true}
                        placement="topLeft"
                      />
                    </Col>
                  </Row>
                </div>
              }
            >
              <TextArea placeholder="请输入代金券使用说明(最多200字)" wordCount={true} maxLength={200} rows={3} />
            </FormItem>
          </Form>
        </div>

        {this.state.modalVisible && (
          <ModalSelectPrd
            initList={this.state.prdList}
            initSKey={this.modalInitSKey}
            platType={this.platType}
            onClose={this.switchModalSelectPrd}
            onSubmit={this.handleSelect}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    pageParams: state.main.pageParams,
    branchDict: state.dict.branch,
    activeTabId: state.main.activeTabId,
  };
};

export default connect(mapStateToProps)(AddEdit);
