import moment from 'moment';
import { isNumber } from 'UTIL/common';
import { rangeFormat, PERIOD_VALID_REGULAR, PERIOD_VALID_RELATIVE, BOSS } from '../config';

/**
 * 处理代金券提交数据
 * @param {Object} params 待提交数据
 * @param {Object} originData 上次保存的详情数据
 * @return {Object}
 */
export function submitDataAdapter(params, originData) {
  originData = originData || {};
  let result = {
    id: null,
    // 优惠类型
    promotionKindId: 'COUPON',
    // 优惠小类
    promotionLittleTypeId: 'COUPON_CASH',
    // 优惠小小类
    promotionTypeId: 'COUPON_CASH_REDEEM',
    // 券类型 - 无门槛优惠券
    promotionNature: 'COUPON_REDEEM',
    // 使用渠道 - 转介绍
    useChannel: 'TRANSFER',
    // 合同条件
    contractCondition: 'ALL',
    // 用户条件
    customerCondition: originData.customerCondition || null,
    // (暂0)
    firstLevel: 0,
    // 优惠码(暂空)
    promotionCode: '',
    // 备注
    remark: '',
    // 发布数量
    publishNum: originData.publishNum || 0,
    // 使用数量
    useNum: originData.useNum || 0,
    // 状态
    status: originData.status || 0,
    // 优惠产品
    productVos: [],
    // 是否需要领取 - 不需领取
    needToRecive: 0,
    // 领取限制
    receiveLimit: 'ALL',
    // userRange 为 2，指定对象
    userRange: 2,
    // 最大优惠金额
    hightAmount: 1,
    // 优惠规则
    ruleVos: [],
    ...params,
    limitProductTypes: [params.limitProductTypes], // 历史原因，产品类型为多选类型
  };

  // 分公司
  if (result.hasOwnProperty('branchId')) {
    result.branchVos = result.branchId
      ? [
          {
            branchId: result.branchId,
            id: 0,
            promotionId: 0,
          },
        ]
      : [];
    // delete result.branchId;
  }

  // 有效时间
  if (result.dateType === PERIOD_VALID_REGULAR && result.hasOwnProperty('date')) {
    result.startDate = result.date[0].format(rangeFormat);
    result.endDate = result.date[1].format(rangeFormat);
  }
  delete result.date;
  delete result.dateType;

  // 产品
  if (result.hasOwnProperty('productVos')) {
    result.productVos = (result.productVos || []).map((item) => {
      let keyName = 'productId';
      let tag = (originData.productVos || []).find((oriPrd) => oriPrd[keyName] === item[keyName]) || item;
      return {
        id: tag.id || null,
        productId: tag.productId || null,
        activityId: tag.activityId || null,
      };
    });
  }

  return result;
}

/**
 * 处理从远端加载的代金券详情数据
 * @param {Object} data 代金券详情数据
 * @return {Object}
 */
export function fieldDataAdapter(data) {
  return {
    promotionName: data.promotionName,
    branchId: data.branchVos[0].branchId,
    limitScope: isNumber(data.limitScope) ? data.limitScope.toString() : data.limitScope || null,
    publishObject: data.publishObject,
    receiveLimit: data.receiveLimit,
    superPosition: isNumber(data.superPosition) ? data.superPosition.toString() : data.superPosition || null,
    dateType: isNumber(data.relativePeriod) && data.relativePeriod > 0 ? PERIOD_VALID_RELATIVE : PERIOD_VALID_REGULAR,
    date: data.startDate && data.endDate ? [moment(data.startDate), moment(data.endDate)] : null,
    relativePeriod: data.relativePeriod,
    hightAmount: data.hightAmount,
    promotionComment: data.promotionComment,
  };
}
