/**
 * Created by Haolin on 2018/12/17.
 */
import React, { PropTypes } from 'react';
import { connect } from 'dva';
import { SearchBar, Select, MultiSelect, Input } from 'COMPONENT';
import { getDictTextByVal } from 'UTIL/dict';
import { storeName, TWO_TEACHER_VAL, SMALL_CLASS_VAL } from '../../../store/constant.js';

import styles from './styles.module.less';

class Search extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      checkValidator: true,
      errors: {},
    };
    this.search = this.search.bind(this);
    this.reset = this.reset.bind(this);
    this.updateKeyword = this.updateKeyword.bind(this);
    this.phaseOrWeekDict = this.phaseOrWeekDict.bind(this);
    this.destroy = this.destroy.bind(this);
    this.errorChange = this.errorChange.bind(this);
    this.checkRequired = this.checkRequired.bind(this);
  }
  static propTypes = {
    classPrefix: PropTypes.string,
    handleSearch: PropTypes.func,
  };

  componentWillUnmount() {
    this.destroy();
  }
  /**
   * 检索
   * */
  search() {
    let checkValidator = this.checkRequired(this.state.errors);
    this.setState({ checkValidator });
    if (checkValidator) {
      this.props.handleSearch();
    }
  }
  /**
   * 重置检索条件
   * */
  reset() {
    this.props.dispatch({
      type: `${storeName}/resetClassKeyword`,
    });
  }
  /**
   * 更新检索条件
   * */
  updateKeyword(params) {
    const { type, val } = params;
    let payload = {};
    let needRefresh = false;
    let notValNull = true;
    switch (type) {
      case 'class_productType':
        payload = {
          class_classType: [],
          class_classLevel: [],
          class_mainTc: null,
          class_auxTc: null,
        };
        break;
      case 'class_quarter':
        payload = {
          class_phase: null,
        };
        break;
      case 'class_classType':
        payload = {
          class_classLevel: [],
        };
        needRefresh = true;
        break;
      case 'class_branch':
        payload = {
          class_campus: null,
        };
        break;
    }
    // let notAllowNull = [
    //   'class_grade',
    //   'class_subject',
    //   'class_classType',
    //   'class_classLevel',
    // ];
    // if (notAllowNull.indexOf(type) !== -1 && (!val || val.length === 0)) {
    //   notValNull = false;
    // }
    if (notValNull) {
      this.props.dispatch({
        type: `${storeName}/updateState`,
        payload: {
          ...payload,
          [type]: val,
        },
      });
      if (needRefresh) {
        this.props.handleLoading(true);
        const _self = this;
        this.props.dispatch({
          type: `${storeName}/updateClassKeyword`,
          payload: {
            type,
            val,
            callback() {
              _self.props.handleLoading(false);
            },
          },
        });
      }
    }
  }
  /**
   * 期数/星期数 字典
   * */
  phaseOrWeekDict(quarterId) {
    const { dict_quarter, dict_phase, dict_weekDay } = this.props;
    const quarterText = getDictTextByVal(dict_quarter, quarterId);
    let result = null;
    switch (quarterText) {
      case '寒假':
      case '暑假':
        result = dict_phase;
        break;
      case '春季':
      case '秋季':
        result = dict_weekDay;
        break;
      // default:
      //   console.warn(`phaseOrWeekDict-quarterText: ${quarterText}`)
    }
    return result;
  }
  /**
   * 注销前销毁
   * */
  destroy() {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        class_className: '',
        class_productType: null,
        class_year: null,
        class_quarter: null,
        class_phase: null,
        class_grade: [],
        class_subject: [],
        class_classType: [],
        class_classLevel: [],
        class_branch: null,
        class_campus: null,
        class_mainTc: null,
        class_auxTc: null,
      },
    });
  }
  /**
   * 用于校验表单是否存在错误
   * */
  errorChange(params) {
    const { type, val } = params;
    this.setState((prevState) => {
      let err = prevState.errors;
      err[type] = val;
      return { errors: err };
    });
  }
  /**
   * 校验数据
   * */
  checkRequired(params) {
    return Object.values(params).every((item) => !Boolean(item));
  }

  render() {
    const {
      classPrefix,
      dict_year,
      dict_quarter,
      dict_grade,
      dict_subject,
      dict_productType,
      dict_classType_twoTc,
      dict_classType_smallClass,
      dict_classLevel,
      dict_branch,
      dict_branchCampus,
      dict_mainTc_twoTc,
      dict_auxTc_twoTc,
      dict_mainTc_smallClass,
      dict_auxTc_smallClass,
      class_className,
      class_productType,
      class_year,
      class_quarter,
      class_phase,
      class_grade,
      class_subject,
      class_classType,
      class_classLevel,
      class_branch,
      class_campus,
      class_mainTc,
      class_auxTc,
    } = this.props;
    const { checkValidator, errors } = this.state;

    return (
      <div className={styles[`${classPrefix}-search`]}>
        <SearchBar onSearch={this.search} onReset={this.reset} expand>
          <Input
            placeholder="班级名称"
            value={class_className}
            onChange={(e) => {
              this.updateKeyword({ type: 'class_className', val: e.target.value });
            }}
          />
          <Select
            placeholder="产品类型"
            dict={dict_productType}
            value={class_productType}
            allowClear={false}
            onChange={(val) => {
              this.updateKeyword({ type: 'class_productType', val });
            }}
            validator={{
              rules: ['required'],
              silent: !Boolean(errors['class_productType']) || checkValidator,
              showStatus: true,
              showTips: false,
              onErrorChange: (val) => {
                this.errorChange({ type: 'class_productType', val });
              },
            }}
          />
          <Select
            placeholder="产品年份"
            dict={dict_year}
            value={class_year}
            allowClear={false}
            onChange={(val) => {
              this.updateKeyword({ type: 'class_year', val });
            }}
            validator={{
              rules: ['required'],
              silent: !Boolean(errors['class_year']) || checkValidator,
              showStatus: true,
              showTips: false,
              onErrorChange: (val) => {
                this.errorChange({ type: 'class_year', val });
              },
            }}
          />
          <Select
            placeholder="产品季度"
            dict={dict_quarter}
            value={class_quarter}
            allowClear={false}
            onChange={(val) => {
              this.updateKeyword({ type: 'class_quarter', val });
            }}
            validator={{
              rules: ['required'],
              silent: !Boolean(errors['class_quarter']) || checkValidator,
              showStatus: true,
              showTips: false,
              onErrorChange: (val) => {
                this.errorChange({ type: 'class_quarter', val });
              },
            }}
          />
          <Select
            placeholder="期/星期"
            dict={this.phaseOrWeekDict(class_quarter)}
            value={class_phase}
            onChange={(val) => {
              this.updateKeyword({ type: 'class_phase', val });
            }}
            disabled={!class_quarter}
          />
          <MultiSelect
            placeholder="年级"
            dict={dict_grade}
            value={class_grade}
            // selectAll={false}
            allowClear={false}
            showSearch={true}
            onChange={(val) => {
              this.updateKeyword({ type: 'class_grade', val });
            }}
            validator={{
              rules: ['required'],
              silent: !Boolean(errors['class_grade']) || checkValidator,
              showStatus: true,
              showTips: false,
              onErrorChange: (val) => {
                this.errorChange({ type: 'class_grade', val });
              },
            }}
          />
          <Select
            placeholder="分公司"
            dict={dict_branch}
            value={class_branch}
            showSearch={true}
            allowClear={false}
            onChange={(val) => {
              this.updateKeyword({ type: 'class_branch', val });
            }}
            validator={{
              rules: ['required'],
              silent: !Boolean(errors['class_branch']) || checkValidator,
              showStatus: true,
              showTips: false,
              onErrorChange: (val) => {
                this.errorChange({ type: 'class_branch', val });
              },
            }}
          />
          <Select
            placeholder="学习中心"
            dict={
              class_branch
                ? dict_branchCampus && dict_branchCampus.filter((item) => item.val === class_branch)
                : dict_branchCampus
            }
            value={class_campus}
            showSearch={true}
            allowClear={false}
            onChange={(val) => {
              this.updateKeyword({ type: 'class_campus', val });
            }}
            validator={{
              rules: ['required'],
              silent: !Boolean(errors['class_campus']) || checkValidator,
              showStatus: true,
              showTips: false,
              onErrorChange: (val) => {
                this.errorChange({ type: 'class_campus', val });
              },
            }}
          />
          <MultiSelect
            placeholder="班级类型"
            dict={class_productType === TWO_TEACHER_VAL ? dict_classType_twoTc : dict_classType_smallClass}
            value={class_classType}
            // selectAll={false}
            allowClear={false}
            onChange={(val) => {
              this.updateKeyword({ type: 'class_classType', val });
            }}
            validator={{
              rules: ['required'],
              silent: !Boolean(errors['class_classType']) || checkValidator,
              showStatus: true,
              showTips: false,
              onErrorChange: (val) => {
                this.errorChange({ type: 'class_classType', val });
              },
            }}
            disabled={!class_productType}
          />
          <MultiSelect
            placeholder="班级层次"
            dict={dict_classLevel}
            value={class_classLevel}
            // selectAll={false}
            allowClear={false}
            onChange={(val) => {
              this.updateKeyword({ type: 'class_classLevel', val });
            }}
            validator={{
              rules: ['required'],
              silent: !Boolean(errors['class_classLevel']) || checkValidator,
              showStatus: true,
              showTips: false,
              onErrorChange: (val) => {
                this.errorChange({ type: 'class_classLevel', val });
              },
            }}
            disabled={!class_classType || class_classType.length === 0}
          />
          <Select
            placeholder={class_productType === TWO_TEACHER_VAL ? '主讲老师' : '班主任'}
            dict={class_productType === TWO_TEACHER_VAL ? dict_mainTc_twoTc : dict_mainTc_smallClass}
            value={class_mainTc}
            showSearch={true}
            onChange={(val) => {
              this.updateKeyword({ type: 'class_mainTc', val });
            }}
          />
          <Select
            placeholder={class_productType === TWO_TEACHER_VAL ? '辅导老师' : '任课老师'}
            dict={class_productType === TWO_TEACHER_VAL ? dict_auxTc_twoTc : dict_auxTc_smallClass}
            value={class_auxTc}
            showSearch={true}
            onChange={(val) => {
              this.updateKeyword({ type: 'class_auxTc', val });
            }}
          />
          <MultiSelect
            placeholder="科目"
            dict={dict_subject}
            value={class_subject}
            showSearch={true}
            // selectAll={false}
            allowClear={false}
            onChange={(val) => {
              this.updateKeyword({ type: 'class_subject', val });
            }}
            validator={{
              rules: ['required'],
              silent: !Boolean(errors['class_subject']) || checkValidator,
              showStatus: true,
              showTips: false,
              onErrorChange: (val) => {
                this.errorChange({ type: 'class_subject', val });
              },
            }}
          />
        </SearchBar>
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  const {
    dict_year,
    dict_quarter,
    dict_phase,
    dict_weekDay,
    dict_grade,
    dict_subject,
    dict_productType,
    dict_classType_twoTc,
    dict_classType_smallClass,
    dict_classLevel,
    dict_campus,
    dict_branchCampus,
    dict_mainTc_twoTc,
    dict_auxTc_twoTc,
    dict_mainTc_smallClass,
    dict_auxTc_smallClass,
    class_className,
    class_productType,
    class_year,
    class_quarter,
    class_phase,
    class_grade,
    class_subject,
    class_classType,
    class_classLevel,
    class_branch,
    class_campus,
    class_mainTc,
    class_auxTc,
  } = state[storeName];
  return {
    dict_year,
    dict_quarter,
    dict_phase,
    dict_weekDay,
    dict_grade,
    dict_subject,
    dict_productType,
    dict_classType_twoTc,
    dict_classType_smallClass,
    dict_classLevel,
    dict_branch: state.dict.branch,
    dict_campus,
    dict_branchCampus,
    dict_mainTc_twoTc,
    dict_auxTc_twoTc,
    dict_mainTc_smallClass,
    dict_auxTc_smallClass,
    class_className,
    class_productType,
    class_year,
    class_quarter,
    class_phase,
    class_grade,
    class_subject,
    class_classType,
    class_classLevel,
    class_branch,
    class_campus,
    class_mainTc,
    class_auxTc,
  };
};

export default connect(mapStateToProps)(Search);
