/**
 * Created by Haolin on 2018/12/20.
 */
import React from 'react';
import { connect } from 'dva';
import { Row, Col } from 'antd';
import { Modal, TextArea, DropdownMenu } from 'COMPONENT';
import { storeName } from '../../store/constant.js';
import styles from './styles.module.less';

const ContentItem = (props) => {
  const { label, children } = props;
  const classPrefix = 'ContentItem';
  return (
    <div className={styles[classPrefix]}>
      <span className={styles[`${classPrefix}-label`]}>{label}</span>
      <span className={styles[`${classPrefix}-content`]}>{children}</span>
    </div>
  );
};

class ModalPublish extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      name: '发券确认',
      loading: false,
      loadingTip: null,
      error: null,
      msg: '',
    };
    this.code = '';
    this.init = this.init.bind(this);
    this.initMsgTempMenus = this.initMsgTempMenus.bind(this);
    this.selectMsgTemp = this.selectMsgTemp.bind(this);
    this.buildMsgTempMenus = this.buildMsgTempMenus.bind(this);
    this.confirm = this.confirm.bind(this);
  }
  static classPrefix = 'ModalPublish';

  componentDidMount() {
    this.init();
  }

  async init() {
    this.setState({ loading: true });
    await new Promise(this.initMsgTempMenus);
    this.setState({ loading: false });
  }

  initMsgTempMenus(resolve) {
    this.props.dispatch({
      type: `${storeName}/initMsgTempMenus`,
      payload: {
        callback() {
          resolve();
        },
      },
    });
  }

  selectMsgTemp(text, code) {
    this.setState({ msg: text });
    this.code = code;
  }

  buildMsgTempMenus(menus) {
    return menus.map((item) => ({
      ...item,
      onClick: () => {
        this.selectMsgTemp(item.text, item.code);
      },
    }));
  }

  confirm() {
    this.setState({
      loading: true,
      loadingTip: '正在发券...',
    });
    this.props.onConfirm(this.code);
  }

  render() {
    const {
      onCancel,
      data: { name: couponName, count },
      menus,
    } = this.props;
    const { name, loading, loadingTip, error, msg } = this.state;

    return (
      <Modal
        title={name}
        className={styles[ModalPublish.classPrefix]}
        loading={loading}
        loadingTip={loadingTip}
        error={error}
        onCancel={onCancel}
        onOk={this.confirm}
        widthSize="M"
      >
        <Row>
          <Col span={24}>
            <ContentItem label="代金券">{couponName}</ContentItem>
          </Col>
        </Row>
        <Row>
          <Col span={24}>
            <ContentItem label="发券量">{count}</ContentItem>
          </Col>
        </Row>
        <Row>
          <Col span={24}>
            <ContentItem label="短信通知">
              <span style={{ color: '#999' }}>不选择模板即视为不发短信通知</span>
            </ContentItem>
          </Col>
        </Row>
        <Row style={{ marginTop: 5 }}>
          <Col span={24}>
            <TextArea placeholder="请点击下方选择框选择短信模板" value={msg} rows={3} disabled />
            <DropdownMenu
              text="短信模板"
              menus={this.buildMsgTempMenus(menus)}
              dropdownClassName={styles['DropDownMenu']}
              hasArrow={true}
            />
          </Col>
        </Row>
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  const { msgTempMenus } = state[storeName];
  return {
    menus: msgTempMenus,
  };
};

export default connect(mapStateToProps)(ModalPublish);
