import React from 'react';
import PropTypes from 'prop-types';
import { Modal, List } from 'COMPONENT';
import { querySelectedProductByCoupon } from 'SERVICE/preferential';
import styles from './index.style.less';

const { COL_WIDTH_M, COL_WIDTH_XL } = List;

export default class ModalViewCouponData extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      data: [],
    };

    this.columns = [
      {
        title: '产品类型',
        key: 'productTypeName',
        width: COL_WIDTH_M,
        render: (record) => {
          return record.productId ? record.categoryName : record.typeName;
        },
      },
      { title: '产品名称', key: 'name', dataIndex: 'name', width: COL_WIDTH_XL },
    ];
  }

  static propTypes = {
    initData: PropTypes.object,
    onCancel: PropTypes.func.isRequired,
  };

  componentDidMount() {
    this.initContent();
  }

  initContent = async () => {
    const { initData = {} } = this.props;
    const { id } = initData;
    this.setState({ loading: true });
    const detail = await querySelectedProductByCoupon({ promotionInfoId: id });
    this.setState({
      loading: false,
      data: detail.data,
    });
  };

  render() {
    return (
      <Modal
        title="详情"
        width={600}
        cancelText="关闭"
        onCancel={this.props.onCancel}
        okBtnShow={false}
        loading={this.state.loading}
      >
        <div className={styles.modalViewWrapper}>
          <List columns={this.columns} dataSource={this.state.data} />
        </div>
      </Modal>
    );
  }
}
