import React from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';
import { Tooltip } from 'antd';
import { Card, TextLink } from 'COMPONENT';
import { runFunc } from 'UTIL/common';
import CouponIcon from './CouponIcon';
import ButtonGroup from './ButtonGroup';
import Info from './Info';
import DetailWithPop from './DetailWithPop';
import MoreInfoBtn from './MoreInfoBtn';
import ModalViewCouponData from './ModalViewCouponData';

import {
  LIMIT_SCOPE_INFINITE,
  LIMIT_SCOPE_FINITE,
  LIMIT_SCOPE_PRODUCT_TYPE,
  xhbossProductTypeDict,
  PRODUCT_TYPE_SHUANG_SHI,
  useChannelLabelMap,
  publishObjectCfg,
  receiveLimitCfg,
  superpositionCfg,
} from '../../config';

import style from './index.style.less';

class CouponCard extends React.Component {
  constructor(props) {
    super(props);

    this.switchModalProduct = this.switchModalProduct.bind(this);
    this.switchMoreInfo = this.switchMoreInfo.bind(this);
    this.renderLimitScope = this.renderLimitScope.bind(this);

    this.state = {
      modalProductVisible: false,
      moreInfoVisible: false,
    };
  }

  static propTypes = {
    actions: PropTypes.func.isRequired,
    index: PropTypes.number,
  };

  buildBtnColumns(props) {
    let columns = [];

    const { status, publishNum, useNum, actions, btnAuth } = props;

    const isValid = status === '0';
    const isPublish = publishNum > 0;
    const isUsed = useNum > 0;

    //导入发券权限
    const isImportIssueEnabled = btnAuth.indexOf('importIssueCaVch') > -1;

    //导入历史权限
    const isImportHistoryEnabled = btnAuth.indexOf('caVchImportIssueHist') > -1;

    //发券权限
    const isIssueEnabled = btnAuth.indexOf('issueCaVch') > -1;

    //发券历史权限
    const isIssueHistoryEnabled = btnAuth.indexOf('caVchIssueHist') > -1;

    //用券情况权限
    const isUsedInfoEnabled = btnAuth.indexOf('caVchUsedInfo') > -1;

    //编辑权限
    const isEditingEnabled = btnAuth.indexOf('editCaVch') > -1;

    if (isValid && isImportIssueEnabled) {
      columns.push({
        text: '导入发券',
        disabled: false,
        onClick() {
          runFunc(actions, { actionType: 'IMPORT_ISSUE', data: props });
        },
      });
    }

    if (isPublish && isImportHistoryEnabled) {
      columns.push({
        text: '导入历史',
        disabled: false,
        onClick() {
          runFunc(actions, { actionType: 'IMPORT_HISTORY', data: props });
        },
      });
    }

    if (isValid && isIssueEnabled) {
      columns.push({
        text: '发券',
        disabled: false,
        onClick() {
          runFunc(actions, { actionType: 'ISSUE', data: props });
        },
      });
    }

    if (isPublish && isIssueHistoryEnabled) {
      columns.push({
        text: '发券历史',
        disabled: false,
        onClick() {
          runFunc(actions, { actionType: 'HISTORY', data: props });
        },
      });
    }

    if (isValid && isEditingEnabled) {
      columns.push({
        text: '编辑',
        disabled: false,
        onClick() {
          runFunc(actions, { actionType: 'EDIT', data: props });
        },
      });
    }

    if (isUsed && isUsedInfoEnabled) {
      columns.push({
        text: '用券情况',
        disabled: false,
        onClick() {
          runFunc(actions, { actionType: 'USED_INFO', data: props });
        },
      });
    }

    return columns;
  }

  switchModalProduct() {
    this.setState((prevState) => ({ modalProductVisible: !prevState.modalProductVisible }));
  }

  switchMoreInfo() {
    this.setState((prevState) => ({ moreInfoVisible: !prevState.moreInfoVisible }));
  }

  renderLimitScope(itemProps) {
    const { limitScope, productVos, limitProductTypes } = itemProps;
    const productList = productVos || [];
    let retVal = '';

    switch (String(limitScope)) {
      case LIMIT_SCOPE_INFINITE:
        retVal = '不限制产品';
        break;
      case LIMIT_SCOPE_FINITE:
        retVal = (
          <span>
            <span>限制产品 已选{productList.length}个产品 </span>
            {productList.length > 0 && <TextLink onClick={this.switchModalProduct}>查看</TextLink>}
          </span>
        );
        break;
      case LIMIT_SCOPE_PRODUCT_TYPE:
        retVal = <span>限制产品类型 已选{limitProductTypes.length}种 </span>;
        break;
    }

    return retVal;
  }

  renderLimitProductTypes(itemProps) {
    return (itemProps.limitProductTypes || [])
      .filter((value) => {
        return value !== PRODUCT_TYPE_SHUANG_SHI;
      })
      .map((value) => {
        return (xhbossProductTypeDict.find(({ val }) => val === value) || {}).text;
      })
      .filter((value) => value)
      .join(',');
  }

  renderReceiveLimit(limitType, limitNum) {
    const limitStr = receiveLimitCfg[limitType];

    if (limitStr.indexOf('1') > -1) {
      return limitStr.replace('1', limitNum);
    } else {
      return limitStr;
    }
  }

  render() {
    const btnColumns = this.buildBtnColumns(this.props);

    return (
      <Card className={style.card}>
        <div className={style.wrapper}>
          <div className={style.header}>
            <div className={style.headerTitle}>
              <CouponIcon status={this.props.status} />
              <Tooltip title={this.props.promotionName}>
                <div className={style.headerTitleText}>{this.props.promotionName}</div>
              </Tooltip>
            </div>

            <div className={style.headerTool}>
              <span>已用 / 已发</span>
              <span className={style.headerToolText}>{`${this.props.useNum} / ${this.props.publishNum}`}</span>
            </div>
          </div>

          <div className={style.subHeader}>
            <div className={style.subHeaderTitle}>
              <span className={style.subHeaderTitleText}>面额￥{this.props.hightAmount}元</span>
            </div>

            <div className={style.subHeaderTool}>
              <ButtonGroup columns={btnColumns} />
            </div>
          </div>

          <div className={style.dashedLine} />

          <div className={style.body}>
            <div className={style.bodyMain}>
              <Info label="所属分公司">
                <DetailWithPop data={(this.props.branchVos || []).map((item) => item.branchName)} />
              </Info>
              <Info label="产品类型">{this.renderLimitProductTypes(this.props)}</Info>
              <Info label="产品范围">{this.renderLimitScope(this.props)}</Info>
              <Info label="发放对象">{publishObjectCfg[this.props.publishObject]}</Info>
              <Info label="发送限制">
                {this.renderReceiveLimit(this.props.receiveLimit, this.props.receiveNumLimit)}
              </Info>
              <Info label="叠加方式">{superpositionCfg[`${this.props.superPosition}`]}</Info>
              <Info label="使用渠道">{useChannelLabelMap[this.props.useChannel]}</Info>
              <Info label="有效期方式">{this.props.relativePeriod ? '相对有效期' : '固定有效期'}</Info>
              {this.props.relativePeriod ? (
                <Info label="有效期">领取或发券后{this.props.relativePeriod}天</Info>
              ) : (
                <Info label="有效期">
                  {this.props.startDate} - {this.props.endDate}
                </Info>
              )}
            </div>

            <div className={style.bodyTool}>
              <MoreInfoBtn
                trigger={this.state.moreInfoVisible}
                onClick={this.switchMoreInfo}
                text="使用说明"
                activeText="使用说明"
              />
            </div>
          </div>

          <div className={style.footer}>
            <div
              className={classNames(style.footerMoreInfo, { [style.footerMoreInfoActive]: this.state.moreInfoVisible })}
            >
              <span>使用说明：</span>
              <span className={style.wordWrap}>{this.props.promotionComment || '暂无'}</span>
            </div>
          </div>
        </div>

        {this.state.modalProductVisible && (
          <ModalViewCouponData initData={this.props} onCancel={this.switchModalProduct} />
        )}
      </Card>
    );
  }
}

export default CouponCard;
