import React, { Component } from 'react';
import { connect } from 'dva';
import { Row, Icon, Col, message } from 'antd';
import { Modal, TextLink, DropdownMenu, TextArea } from 'COMPONENT';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { OSS_BASE_PREFIX } from 'CONST/ossConfig';
import { onUploadFile, download } from 'UTIL/file';
import { importCouponByPromotionInfo } from 'SERVICE/preferential';
import ChooseFileButton from 'PAGE/personnelManage/TeacherIntroduction/ChooseFileButton';
import { storeName } from '../../store/constant';
import { CUSTOMER, STUDENT } from '../../config';

import style from './styles.module.less';

const xls = require('ASSET/imgs/xls.png');

// 导入模板 map
const couponExcelMap = {
  [CUSTOMER]: {
    title: '导入客户',
    excel: `${OSS_BASE_PREFIX}/客户导入模版.xlsx`,
    fileName: '客户导入模版',
    apiKey: 'PHONE',
  },
  [STUDENT]: {
    title: '导入学生',
    excel: `${OSS_BASE_PREFIX}/满减券-学生导入模板.xlsx`,
    fileName: '学生导入模板',
    apiKey: 'STUDENT',
  },
};

const ContentItem = (props) => {
  const { label, children } = props;

  return (
    <div className={style.contentItem}>
      <span className={style.contentItemLabel}>{label}</span>
      <span>{children}</span>
    </div>
  );
};

class ModalImportVoucher extends Component {
  constructor(props) {
    super(props);

    this.state = {
      // modal loading控制
      loading: false,
      // 短信模板以及code
      promotionOrderSmsTemp: null,
      // 短信code
      promotionOrderSmsCode: null,
      // 导入的文件
      file: {},
    };
  }

  componentDidMount() {
    this.init();
  }

  init = async () => {
    this.setState({ loading: true });

    await new Promise(this.initMsgTempMenus);

    this.setState({ loading: false });
  };

  initMsgTempMenus = (resolve) => {
    this.props.dispatch({
      type: `${storeName}/initMsgTempMenus`,
      payload: {
        callback() {
          resolve();
        },
      },
    });
  };
  // 按钮 - 导入
  handleOk = () => {
    const { file } = this.state;
    this.setState({ loading: true }, async () => {
      await onUploadFile([file], this.onloadCallback);
    });
  };

  /**
   * 下载模板
   */
  downloadTemplate = () => {
    const { data = {} } = this.props;
    const { publishObject } = data;
    const { excel, fileName } = couponExcelMap[publishObject];
    const format = 'xlsx';
    download({ url: excel, fileName, format });
  };

  /**
   * 选择文件
   */
  handleSelectFile = (file) => {
    this.setState({ file: file || {} });
  };

  /**
   * 上传文件回调
   */
  onloadCallback = async (xhr, key) => {
    const { onOK, data = {} } = this.props;
    const { promotionOrderSmsCode } = this.state;
    const { publishObject } = data;
    const { apiKey } = couponExcelMap[publishObject];

    if (xhr.status === 200 && xhr.readyState === 4) {
      try {
        const { resultCode } =
          (await importCouponByPromotionInfo({
            fileKey: key,
            promotionInfoId: data.id,
            couponImportType: apiKey,
            templateCode: promotionOrderSmsCode,
          })) || {};
        if (resultCode === 0) {
          this.setState({ loading: false });

          onOK(data);
        } else {
          this.setState({ loading: false });

          showMessage('导入失败,请重新导入!');
        }
      } catch (e) {
        this.setState({ loading: false });
      }
    } else {
      showMessage('系统错误');
      this.setState({ loading: false });
    }
  };

  // 短信模板
  buildMsgTempMenus = (menus) => {
    if (menus && menus.length) {
      return menus.map((item) => ({
        ...item,
        onClick: () =>
          this.setState({
            promotionOrderSmsTemp: item.text,
            promotionOrderSmsCode: item.code,
          }),
      }));
    }
    return [];
  };

  render() {
    const { loading } = this.state;
    const { data = {}, onCancel } = this.props;
    const { title } = couponExcelMap[data.publishObject] || {};

    if (!title) {
      return (
        <Modal
          wrapClassName={style.importVoucher}
          title={title}
          visible={true}
          loading={loading}
          widthSize="s"
          okBtnShow={false}
          onCancel={onCancel}
        >
          数据错误
        </Modal>
      );
    }

    const { menus } = this.props;
    const { file = {}, promotionOrderSmsTemp } = this.state;

    // 临时：发券给家长时屏蔽短信
    const isSMSEnabled = data.publishObject === STUDENT;

    return (
      <Modal
        wrapClassName={style.importVoucher}
        title={title}
        widthSize="s"
        visible={true}
        loading={loading}
        okText="导入"
        onOk={this.handleOk}
        onCancel={onCancel}
        okBtnDisabled={!file.name}
      >
        <Row>
          <img className={style.imgExcel} src={xls} alt="" />
          <TextLink style={{ marginLeft: 30 }} onClick={() => this.downloadTemplate()}>
            导入模版下载
          </TextLink>
        </Row>
        <Row className={style.chooseTitle}>上传文件</Row>
        {file.name ? (
          <Row className={style.selectFile} gutter={8}>
            <Col span={2}>
              <Icon type="file-excel" />
            </Col>
            <Col span={20}>
              <span style={{ marginLeft: '15px' }}>{file.name || ''}</span>
            </Col>
            <Col span={2}>
              <Icon type="close-circle-o" onClick={() => this.setState({ file: {} })} />
            </Col>
          </Row>
        ) : (
          <Row className={style.fileRow}>
            <ChooseFileButton
              key="upload"
              size="small"
              btnText="选择文件"
              accept="application/vnd.openxmlformats-officedocument.spreadsheetml.sheet,application/vnd.ms-excel"
              fileType={['excel']}
              onChange={(file) => this.handleSelectFile(file)}
            />
          </Row>
        )}
        {isSMSEnabled && (
          <div style={{ marginBottom: 5 }}>
            <Row>
              <Col span={24}>
                <ContentItem label="短信通知">
                  <span style={{ color: '#999' }}>不选择模板即视为不发短信通知</span>
                </ContentItem>
              </Col>
            </Row>
            <Row>
              <TextArea placeholder="请点击下方选择框选择短信模板" value={promotionOrderSmsTemp} rows={3} disabled />
              <DropdownMenu
                style={{ margin: '10px 0 5px', display: 'block' }}
                text="短信模板"
                menus={this.buildMsgTempMenus(menus)}
                hasArrow={true}
              />
            </Row>
          </div>
        )}
      </Modal>
    );
  }
}
const mapStateToProps = (state) => {
  const { msgTempMenus } = state[storeName];
  return {
    menus: msgTempMenus,
  };
};

export default connect(mapStateToProps)(ModalImportVoucher);
