import React from 'react';
import { connect } from 'dva';
import moment from 'moment';
import { DatePicker } from 'antd';
import {
  Page,
  Button,
  SearchFilterForm,
  SearchFieldForm,
  Input,
  Select,
  MultiSelect,
  Deck,
  NoDataPlus,
} from 'COMPONENT';
import { loginUser, errorsHandle } from 'UTIL/common';
import { getPromotionList } from 'SERVICE/preferential';

import {
  CUSTOMER,
  STUDENT,
  COUPON,
  COUPON_REDEEM,
  COUPON_CASH,
  COUPON_CASH_REDEEM,
  superpositionDict,
  couponStatusDict,
  publishObjectDict,
  rangeFormat,
  timeFormat,
} from '../config';

import CouponCard from './CouponCard';
import ModalImportVoucher from './ModalImportVoucher';
import ModalImportRemind from './ModalImportRemind';
import SendingToPhoneModal from './SendingToPhoneModal';

import { storeName } from '../store/constant.js';
import styles from './index.style.less';

const couponCardRender = function ({ actions, btnAuth }) {
  return (props, index) => {
    return <CouponCard {...props} btnAuth={btnAuth} actions={actions} index={index} key={props.id} />;
  };
};

class CashVoucherMgmtEntrance extends React.Component {
  constructor(props) {
    super(props);

    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.initSKey = this.initSKey.bind(this);
    this.getListParams = this.getListParams.bind(this);
    this.loadList = this.loadList.bind(this);
    this.updatePage = this.updatePage.bind(this);
    this.changePage = this.changePage.bind(this);
    this.search = this.search.bind(this);
    this.newCoupon = this.newCoupon.bind(this);
    this.cardAction = this.cardAction.bind(this);
    this.openPage = this.openPage.bind(this);
    this.destroy = this.destroy.bind(this);

    this.state = {
      name: '代金券',
      loading: false,
      searchKey: null,
      defaultSKey: {},
      list: [],
      total: 0,
      pageCurrent: 1,
      pageSize: 10,
      platType: null,
      modalImportData: null,
      modalRemindData: null,
      modalImportVisible: false,
      modalRemindVisible: false,
      // 发送给家长弹层可见性
      sendToPhoneModalVisible: false,
      // 发送给家长的代金券信息
      sendtoPhoneModalData: null,
    };
  }

  static classPrefix = 'CashVoucherMgmtEntrance';

  componentDidMount() {
    this.init();
  }

  componentWillUnmount() {
    this.destroy();
  }

  async init() {
    this.setState({ loading: true });

    await new Promise(this.initDict);
    await new Promise(this.initSKey);
    await new Promise(this.loadList);

    this.setState({ loading: false });
  }

  initDict(resolve) {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['branch'],
        callback() {
          resolve();
        },
      },
    });
  }

  initSKey(resolve) {
    const isGroup = loginUser.isGroupRole();
    let newSKey = {
      branchId: [isGroup ? (this.props.branchDict || [{ val: null }])[0].val : loginUser.userData.brenchId],
      status: '0',
    };
    this.setState(
      {
        searchKey: newSKey,
        defaultSKey: { ...newSKey },
      },
      () => {
        resolve();
      },
    );
  }

  getListParams() {
    let searchKey = {
      // 接口的第一页是 0 页
      pageNo: this.state.pageCurrent - 1,
      pageSize: this.state.pageSize,
      // 优惠券
      promotionKindId: COUPON,
      // 代金券
      promotionLittleTypeId: COUPON_CASH,
      promotionTypeId: COUPON_CASH_REDEEM,
      promotionNature: COUPON_REDEEM,
    };

    for (let key in this.state.searchKey) {
      searchKey[key] = this.state.searchKey[key];
    }

    return searchKey;
  }

  async loadList(resolve) {
    const apiParams = this.getListParams();

    const rsp = await getPromotionList(apiParams, true).catch(errorsHandle());
    if (rsp && rsp.resultCode === 0) {
      this.setState({
        list: rsp.datas || [],
        total: rsp.rowCount,
      });
    }

    resolve();
  }

  updatePage(pageCurrent, pageSize) {
    return new Promise((resolve) => {
      this.setState(
        (prevState) => {
          let payload = {};
          if (pageSize && pageSize !== prevState.pageSize) {
            payload.pageCurrent = 1;
            payload.pageSize = pageSize;
          } else {
            payload.pageCurrent = pageCurrent;
          }
          return payload;
        },
        () => {
          resolve();
        },
      );
    });
  }

  async changePage(pageCurrent, pageSize) {
    await this.updatePage(pageCurrent, pageSize);
    this.setState({ loading: true });
    await new Promise(this.loadList);
    this.setState({ loading: false });
  }

  search(params) {
    this.setState(
      {
        searchKey: { ...this.props.initSKey, ...this.searchKeyAdapter(params) },
      },
      () => {
        this.changePage(1, this.state.pageSize);
      },
    );
  }

  searchKeyAdapter = (params) => {
    if (params.startDate) {
      params.startDate = params.startDate.format(rangeFormat);
    }
    if (params.endDate) {
      params.endDate = params.endDate.format(rangeFormat);
    }
    return params;
  };

  newCoupon() {
    this.openPage('add', { couponId: null, isSend: false });
  }

  cardAction(params) {
    switch (params.actionType) {
      case 'IMPORT_ISSUE':
        this.switchImportIssue(params.data);
        break;
      case 'IMPORT_HISTORY':
        this.openPage('ImportedRecord', { coupon: params.data });
        break;
      case 'ISSUE':
        this.sendVoucher(params.data);
        break;
      case 'HISTORY':
        this.openPage('IssuingRecord', { coupon: params.data });
        break;
      case 'EDIT':
        this.editVoucher(params.data);
        break;
      case 'USED_INFO':
        this.openPage('UsedInfo', { coupon: params.data });
        break;
      default:
        console.error(`Error: invalid card actionType(${params.type})!`);
    }
  }

  /**
   * 发券
   * @param {Object} data 某张代金券详情数据
   */
  sendVoucher = (data) => {
    if (data.publishObject === STUDENT) {
      this.openPage('Issue', {
        coupon: data,
      });
    } else if (data.publishObject === CUSTOMER) {
      this.toggleSendingToPhone(data);
    }
  };

  /**
   * 切换导入发券弹层
   * @param {Object} data 代金券详情
   */
  switchImportIssue = (data) => {
    this.setState((prevState) => {
      return {
        modalImportData: prevState.modalImportVisible ? null : data,
        modalImportVisible: !prevState.modalImportVisible,
      };
    });
  };

  /**
   * 切换导入发券记录弹层
   * @param {Object} data 代金券详情
   */
  switchImportRemind = (data) => {
    this.setState((prevState) => {
      return {
        modalRemindData: prevState.modalRemindVisible ? null : data,
        modalRemindVisible: !prevState.modalRemindVisible,
      };
    });
  };

  /**
   * 切换发送给家长弹层
   * @param {Object} data 代金券详情
   */
  toggleSendingToPhone = (data) => {
    this.setState((prevState) => {
      return {
        sendToPhoneModalVisible: !prevState.sendToPhoneModalVisible,
        sendtoPhoneModalData: prevState.sendToPhoneModalVisible ? null : data,
      };
    });
  };

  /**
   * 家长发券成功后触发
   */
  handleSendingToPhoneFinish = async () => {
    // 切换家长发券弹层状态
    this.toggleSendingToPhone();

    // 重新刷新
    this.setState({ loading: true });

    await new Promise(this.loadList);

    this.setState({ loading: false });
  };

  openImportHistory = () => {
    this.openPage('ImportedRecord', { coupon: this.state.modalRemindData });
  };

  editVoucher = (data) => {
    // publishNum 发送次数
    // outId 有值标识代金券被其他业务线关联
    this.openPage('edit', {
      couponId: data.id,
      isSend: data.publishNum > 0 || !!data.outId,
    });
  };

  openPage(path, params) {
    let payload = {
      path: `/cashVoucher/${path}`,
      refresh: true,
    };
    if (params) {
      payload.params = params;
    }
    this.props.dispatch({
      type: 'main/openPage',
      payload,
    });
  }

  destroy() {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: {
        list_coupon: [],
        total_coupon: 0,
      },
    });
  }

  render() {
    const classPrefix = CashVoucherMgmtEntrance.classPrefix;

    const { btnAuth } = this.props;

    // 新增权限
    const isAddingEnabled = btnAuth.indexOf('addCaVch') > -1;

    return (
      <Page
        name={this.state.name}
        className={classPrefix}
        fitWindowHeight={true}
        loading={this.state.loading}
        route={this.props.route}
        headerToolBar={
          isAddingEnabled && (
            <div className={styles.btnWrapper}>
              <Button type="primary" onClick={this.newCoupon} disabled={false}>
                新增
              </Button>
            </div>
          )
        }
        subHeader={
          <SearchFilterForm onSearch={this.search} truncationNum={3}>
            <SearchFieldForm name="promotionName" span={1}>
              <Input placeholder="代金券名称" />
            </SearchFieldForm>
            <SearchFieldForm name="branchId" span={1} initialValue={this.state.defaultSKey.branchId || []}>
              <MultiSelect placeholder="分公司" dict={this.props.branchDict} allowClear={false} showSearch />
            </SearchFieldForm>
            <SearchFieldForm name="status" span={1} initialValue={this.state.defaultSKey.status || null}>
              <Select placeholder="状态" dict={couponStatusDict} allowClear={false} />
            </SearchFieldForm>
            <SearchFieldForm name="publishObject" span={1} initialValue={this.state.defaultSKey.publishObject || null}>
              <Select placeholder="发放对象" dict={publishObjectDict} />
            </SearchFieldForm>
            <SearchFieldForm name="superPosition" span={1}>
              <Select placeholder="是否叠加" dict={superpositionDict} />
            </SearchFieldForm>
            <SearchFieldForm name="startDate" span={1} className={styles.datePickerWrap}>
              <DatePicker
                placeholder="有效开始时间"
                format={rangeFormat}
                showTime={{ defaultValue: moment('00:00:00', timeFormat) }}
                showToday={false}
              />
            </SearchFieldForm>
            <SearchFieldForm name="endDate" span={1} className={styles.datePickerWrap}>
              <DatePicker
                placeholder="有效结束时间"
                format={rangeFormat}
                showTime={{ defaultValue: moment('00:00:00', timeFormat) }}
                showToday={false}
              />
            </SearchFieldForm>
          </SearchFilterForm>
        }
      >
        {this.state.list.length > 0 ? (
          <Deck
            render={couponCardRender({
              actions: this.cardAction,
              btnAuth,
            })}
            dataSource={this.state.list}
            pagination={{
              total: this.state.total,
              current: this.state.pageCurrent,
              size: this.state.pageSize,
              onChange: this.changePage,
            }}
            fixedBodyHeight={true}
          />
        ) : (
          <NoDataPlus />
        )}
        {this.state.modalImportVisible && (
          <ModalImportVoucher
            data={this.state.modalImportData}
            onCancel={this.switchImportIssue}
            onOK={(data) => {
              // 关闭导入发券弹层
              this.switchImportIssue();

              // 打开导入提醒弹层
              this.switchImportRemind(data);
            }}
          />
        )}
        {this.state.modalRemindVisible && (
          <ModalImportRemind
            onCancel={this.switchImportRemind}
            onConfirm={this.switchImportRemind}
            onBtnClick={this.openImportHistory}
          />
        )}
        {this.state.sendToPhoneModalVisible && (
          <SendingToPhoneModal
            data={this.state.sendtoPhoneModalData}
            onCancel={this.toggleSendingToPhone}
            onFinish={this.handleSendingToPhoneFinish}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const btnAuth = state.account.btnAuths;

  return {
    branchDict: state.dict.branch,
    btnAuth,
  };
};

export default connect(mapStateToProps)(CashVoucherMgmtEntrance);
