import React from 'react';
import { Tooltip, Popover } from 'antd';
import { Deck, Modal, List, Card, Loading, TextLink, NoDataPlus } from 'COMPONENT';
import Regular from '../../Coupons/DeductionCoupon/Detail/regular';
import Info from '../../Coupons/DeductionCoupon/MgmtEntrance/CouponCard/Info';
import CouponIcon from '../../Coupons/DeductionCoupon/MgmtEntrance/CouponCard/CouponIcon';

import { errorsHandle } from 'SRC/utils/common';
import { querySelectedProductByCoupon, listUnUsedPromotionInfoByStudentId } from 'SERVICE/preferential';
import {
  superpositionCfg,
  continuityTextCfg,
  LIMIT_SCOPE_FINITE,
  LIMIT_SCOPE_INFINITE,
  xhbossProductTypeDict,
  LIMIT_SCOPE_PRODUCT_TYPE,
} from '../../Coupons/DeductionCoupon/config';

import style from './styles.module.less';

const { COL_WIDTH_M, COL_WIDTH_XL } = List;
const limitScopeDetailList = function ({ list }) {
  const columns = [
    {
      title: '产品类型',
      width: COL_WIDTH_M,
      key: 'productTypeName',
      render: (record) => (record.productId ? record.categoryName : record.typeName),
    },
    {
      key: 'name',
      title: '产品名称',
      dataIndex: 'name',
      width: COL_WIDTH_XL,
    },
  ];
  return <List columns={columns} dataSource={list} />;
};
const ruleDetailList = function ({ couponData }) {
  const regularList = couponData.ruleVos;
  const [type = ''] = Array.isArray(couponData.limitProductTypes) ? couponData.limitProductTypes : [];
  const props = { type, regularList, couponData: { ...couponData, price: (couponData.goodsVo || {}).price || 0 } };
  return <Regular {...props} />;
};

export default class DeductionCouponInfoModal extends React.Component {
  state = {
    list: [],
    loading: false,
  };

  constructor(props) {
    super(props);
  }

  componentDidMount() {
    this.init(this.props.studentId);
  }

  componentWillReceiveProps(props, nextProps) {
    if (nextProps.visible && nextProps.studentId && nextProps.studentId !== props.studentId) {
      this.loadData(nextProps.studentId);
    }
  }

  init(studentId) {
    this.loadData(studentId);
  }

  async loadData(studentId) {
    this.setState({ list: [] });

    if (!studentId) {
      return;
    }
    this.setState({ loading: true });
    const result = await listUnUsedPromotionInfoByStudentId({
      studentId,
      pageNo: 1,
      pageSize: 1000,
    }).catch(errorsHandle());

    if (!result || !result.datas || !result.datas.length) {
      this.setState({ loading: false });
      return;
    }
    this.setState({ list: result.datas, loading: false });
  }

  handleCancelModal = () => {
    this.props.onClose();
  };

  handleViewLimitScopeDetail = async (visible, id, parentId) => {
    if (!visible) {
      return;
    }
    const { list } = this.state;
    const match = list.find((item) => item.id === parentId)._limitScope;
    if (match != null) {
      return;
    }
    const detail = await querySelectedProductByCoupon({ promotionInfoId: id }).catch(errorsHandle());
    this.setState({
      list: list.map((item) => {
        return item.id === parentId ? { ...item, _limitScope: detail.data } : { ...item };
      }),
    });
  };

  generateCard = (props, index) => {
    const item = props.promotionInfoVo;
    return (
      <Card className={style.card} key={item.id} index={index}>
        <div className={style.wrapper}>
          <div className={style.header}>
            <div className={style.headerTitle}>
              <CouponIcon status={item.status} />
              <Tooltip title={item.promotionName}>
                <div className={style.headerTitleText}>{item.promotionName}</div>
              </Tooltip>
            </div>
          </div>
          <div className={style.body}>
            <div className={style.bodyMain}>
              {this.generateLimitProductTypes(item)}
              {this.generateLimitScope(item, props)}
              {this.generateSuperposition(item)}
              {this.generateContinuityType(item)}
              {this.generateBuyTime(item)}
              {this.generateValidateDate(item)}
              {this.generateRuleInfo(item)}
            </div>
          </div>
        </div>
      </Card>
    );
  };

  generateCardRender() {
    return this.generateCard;
  }

  generateModal() {
    const props = {
      footer: null,
      widthSize: 'xl',
      title: '抵扣券明细',
      visible: this.props.visible,
      onCancel: this.handleCancelModal,
      className: 'deduction-coupon-info-modal',
    };

    return <Modal {...props}>{this.state.loading ? this.generateLoading() : this.generateList()}</Modal>;
  }

  // 产品类型
  generateLimitProductTypes(item) {
    const productTypeMapping = xhbossProductTypeDict.reduce((prev, el) => {
      prev[el.val] = el.text;
      return prev;
    }, {});
    const { limitProductTypes } = item;
    const productTypeLabels = limitProductTypes.map((name) => productTypeMapping[name]).filter((el) => el);
    const isValid = Array.isArray(productTypeLabels) && productTypeLabels.length > 0;

    return <Info label="产品类型">{isValid ? productTypeLabels.join(',') : '全部'}</Info>;
  }

  // 产品范围
  generateLimitScope(item, props) {
    let retVal = '';
    const limitScope = `${item.limitScope}`;
    const productList = item.productVos || [];

    if (limitScope === LIMIT_SCOPE_INFINITE || limitScope === LIMIT_SCOPE_PRODUCT_TYPE) {
      retVal = '不限制产品';
    } else if (limitScope === LIMIT_SCOPE_FINITE) {
      retVal = (
        <span>
          <span>限制{productList.length}种产品</span>
          {productList.length && (
            <Popover
              overlayClassName="limitScopePopover"
              content={limitScopeDetailList({ list: props._limitScope || [] })}
              title="限制产品"
              onVisibleChange={(visible) => this.handleViewLimitScopeDetail(visible, item.id, props.id)}
            >
              <TextLink>查看</TextLink>
            </Popover>
          )}
        </span>
      );
    }

    return <Info label="产品范围">{retVal}</Info>;
  }

  // 叠加方式
  generateSuperposition(item) {
    return <Info label="叠加方式">{superpositionCfg[item.superPosition]}</Info>;
  }

  // 连报条件
  generateContinuityType(item) {
    return <Info label="连报条件">{continuityTextCfg[item.continuityType]}</Info>;
  }

  // 买券时间
  generateBuyTime(item) {
    const goodsVo = item.goodsVo || {};
    return (
      <Info label="买券时间">
        {goodsVo.beginTime || ''} - {goodsVo.endTime || ''}
      </Info>
    );
  }

  // 有效期
  generateValidateDate(item) {
    return item.relativePeriod ? (
      <Info label="有效期">领取或发券后{item.relativePeriod}天</Info>
    ) : (
      <Info label="有效期">
        {item.startDate} - {item.endDate}
      </Info>
    );
  }

  generateRuleInfo(item) {
    return (
      !!item.ruleVos &&
      !!item.ruleVos.length && (
        <Info label="抵扣规则">
          <Popover
            overlayClassName="ruleDetailPopover"
            content={ruleDetailList({ couponData: { ...item } })}
            title="抵扣规则"
          >
            <TextLink>查看</TextLink>
          </Popover>
        </Info>
      )
    );
  }

  // 生成列表
  generateList() {
    return this.state.list.length ? (
      <Deck pagination={null} fixedBodyHeight={true} dataSource={this.state.list} render={this.generateCardRender()} />
    ) : (
      <NoDataPlus />
    );
  }

  generateLoading() {
    return <Loading loading={this.state.loading} />;
  }

  render() {
    return this.generateModal();
  }
}
