import React from 'react';
import Tag from 'COMPONENT/Tag';
import Select from 'COMPONENT/Select';
import Button from 'COMPONENT/Button';
import styles from './styles.less';
import { getDictTextByVal } from '../../../../../utils/dict/index';

class TagGroup extends React.Component {
  constructor(props) {
    super(props);
    this.tagIdx = 1;
    this.state = {
      addEditing: false, // {boolean} 添加按钮是否处于编辑状态
      tags: this.getTags(props.value), // {object[]} 标签列表，{ id, val }
    };
  }

  componentWillReceiveProps(nextProps) {
    this.setState({ tags: this.getTags(nextProps.value) });
  }

  componentDidUpdate(prevProps, prevState) {
    if (!prevState.addEditing && this.state.addEditing) {
      this.selectRef.setOpenState(true);
    }
  }

  getTags = (value) => {
    return value
      ? value.map((v) => {
          return { id: '' + this.tagIdx++, val: v };
        })
      : [];
  };

  confirmAdd = (addValue) => {
    const { tags } = this.state;
    const newTags = tags.push({ id: '' + this.tagIdx++, val: addValue });
    this.setState({ addEditing: false, tags: newTags });
    this.onChange(tags);
  };

  deleteTag = (id) => {
    const { tags } = this.state;
    this.onChange(tags.filter((t) => t.id !== id));
  };

  onChange = (tags) => {
    const value = tags.length === 0 ? null : tags.map((t) => t.val);
    this.props.onChange(value);
  };

  getAddBtn = () => {
    const { dict } = this.props;
    const { addValue, addEditing } = this.state;
    if (addEditing) {
      return (
        <div style={{ width: 200, display: 'inline-block', margin: '0 20px 20px 0' }}>
          <Select
            dict={dict}
            value={addValue}
            onChange={this.confirmAdd}
            ref={(c) => (this.selectRef = c)}
            showSearch
          />
        </div>
      );
    } else {
      return (
        <Button
          iconType="plus"
          onClick={() => this.setState({ addEditing: true })}
          style={{ position: 'relative', top: -1, margin: '0 20px 20px 0' }}
        >
          添加
        </Button>
      );
    }
  };

  render() {
    const { dict } = this.props;
    const { tags } = this.state;
    console.log(tags);
    return (
      <div className={styles.wrap}>
        {tags.map((t) => {
          return (
            <Tag key={t.id} closable onClose={() => this.deleteTag(t.id)}>
              {getDictTextByVal(dict, t.val)}
            </Tag>
          );
        })}
        {this.getAddBtn()}
      </div>
    );
  }
}

export default TagGroup;
