/**
 * 编辑产品页面，新增或修改
 * @param {string?} productId 产品ID，新增时为空
 * @param {number?} sellCount 销售数量
 * @param {function} onSaveSucess 保存成功时的回调
 */
import React from 'react';
import { connect } from 'dva';
import { Row, Col } from 'antd';
import Input from 'COMPONENT/Input';
import InputNumber from 'COMPONENT/InputNumber';
import Checkbox from 'COMPONENT/Checkbox';
import Page from 'COMPONENT/Page';
import Select from 'COMPONENT/Select';
import TagSelect from 'COMPONENT/TagSelect';
import DatePicker from 'COMPONENT/DatePicker';
import TextArea from 'COMPONENT/TextArea';
import Block from 'COMPONENT/Block';
import FormItem from 'COMPONENT/FormItem';
import { cutText } from 'COMPONENT/utils/basic/str';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import showAlert from 'COMPONENT/utils/globalTips/showAlert';
import { dateToMoment } from 'COMPONENT/utils/form';
import { getDictTextByVal, loadCommonDict, loadAllBranchDict } from 'UTIL/dict';
import { getProductById, saveOrUpdateProduct } from 'SERVICE/product';
import { getDataDictBySonId } from 'SERVICE/system';
import { loadLinkProductsDict, gradeLevelDict, getGradeLevel, currentYearText } from '../../common';
import { productTypeDict } from 'CONST/dicts/product';
import styles from 'PAGE/product/editPage.less';
import sty from './styles.less';
import IconButton from 'COMPONENT/IconButton';
import Popover from 'COMPONENT/Popover';
import Button from 'COMPONENT/Button';
import HoursPlanModal from '../HoursPlanModal';
import { dictAddOptionTitle, formatGroupDict } from 'UTIL/dict';
import { getSummaryBySubjectGroupList } from '../utils';
import { QUARTER_SUMMER, QUARTER_AUTUMN, QUARTER_WINTER, QUARTER_SPRING, QUARTER_All_YEAR } from 'PAGE/common/utils';
import { formatCurrency } from 'COMPONENT/utils/basic/num';
import TagGroup from './TagGroup';
import TotalHoursPlanPop from '../TotalHoursPlanPop';

const productTypeVal = 'ECS_CLASS';
const productType = productTypeDict.find((p) => p.val === productTypeVal);

class EditPage extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 内容加载状态
      additionName: null, // {string} 附加名称
      year: null, // {string} 产品年份
      grade: null, // {string} 年级
      serise: null, // {string} 产品系列
      group: null, // {string} 产品组
      classTimeLen: 40, // {number} 课时时长
      price: null, // {number} 单价
      everyCourseHours: null, //{}课时数/次
      startDate: null, // {string} 上架日期
      endDate: null, // {string} 下架日期
      feeRule: 'chargeCosumeNow', // {string} 扣费规则
      feelClass: 'noneLessons', // {string} 体验课设置
      branchs: null, // {[string]} 所属分公司
      linkSubjects: null, // {[string]} 关联科目
      linkProducts: null, // {[string]} 其他关联产品
      saleNums: 0, // {number} 销售数量
      editable: true, // {boolean} 关联产品允许编辑
      remark: '', // {string} 备注
      yearDict: null, // {[object]} 字典 产品年份
      gradeDict: null, // {[object]} 字典 年级
      seriseDict: null, // {[object]} 字典 产品系列
      groupDict: null, // {[object]} 字典 产品组
      feeRuleDict: null, // {[object]} 字典 扣费规则
      branchsDict: null, // {[object]} 字典 分公司
      linkSubjectsDict: null, // {[object]} 字典 关联科目
      linkProductsDict: null, // {[object]} 字典 其他关联产品
      quarterDict: null, //{[object]} 字典 季度
      quarter: null, //{string} 季度
      classType: null, //{string} 班级类型
      classTypeDict: null, //{[object]}  字典 班级类型
      name: null, //{string}  产品名字
      gradeLevel: null, //{string}年级段
      gradeLevelName: null, // {string} 年级段名称
      isCouseHoursPlaned: false, // {boolean} 是否已规划课时
      subjectGroupList: [], // {object[]} 课时规划科目列表，格式如下：
      /*[
              {
                quarterId
                subjects: [
                  {
                    id string
                    courseHours	课时	string
                    subjectNums	科目数	string
                    teachNums 讲数 string
                  }
                ]
              }
            ]
          */
      isAfter2019: false, // {boolean} 是否2019年及以后的产品
      miniClassTotalhours: null, // {number} 总课时数
      hoursPlanModalVis: false, // {boolean} 课时规划弹框显示状态
    };
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化内容
   */
  initContent = async () => {
    const {
      params: { productId },
    } = this.props;
    this.setState({ loading: true });
    let calls = [
      loadCommonDict('PRODUCT_VERSION'),
      loadCommonDict('PRODUCT_QUARTER'),
      loadCommonDict('STUDENT_GRADE'),
      loadCommonDict('COURSE_SERIES'),
      loadCommonDict('CLASS_TYPE', 'ECS_CLASS'),
      loadCommonDict('PRODUCT_GROUP'),
      loadCommonDict('PRODUCT_CHARGE_RULE'),
      loadCommonDict('PRODUCT_CLASS_MODAL'),
      loadAllBranchDict(),
      loadCommonDict('SUBJECT'),
    ];
    if (productId) {
      calls.push(this.loadProductDetail(productId));
    }
    const [
      yearDict,
      quarterDict,
      gradeDict,
      seriseDict,
      classTypeDict,
      groupDict,
      feeRuleDict,
      classLenTypeDict,
      branchsDict,
      linkSubjectsDict,
      productDetail,
    ] = await Promise.all(calls);
    let changed = {
      yearDict,
      quarterDict,
      gradeDict,
      seriseDict,
      groupDict,
      feeRuleDict,
      branchsDict,
      linkSubjectsDict,
      classTypeDict,
      classLenTypeDict,
    };
    if (productDetail) {
      const linkProductsDict = await loadLinkProductsDict(productDetail.branchs, productTypeVal);
      const year = productDetail.year;
      changed = {
        ...changed,
        ...productDetail,
        linkProductsDict,
        isAfter2019: year && parseInt(getDictTextByVal(yearDict, year)) >= 2019,
      };
    } else {
      const yearOpt = yearDict.find((y) => y.text === currentYearText);
      changed = { ...changed, year: yearOpt ? yearOpt.val : null };
    }
    this.setState({ loading: false, ...changed });
  };

  /**
   * 加载产品详情，并转换格式
   * @param  {string} productId 产品ID
   */
  loadProductDetail = async (productId) => {
    const apiData = await getProductById({ productId });
    const subjectList = apiData.subjectList;
    let subjectGroupList = [];
    if (subjectList) {
      subjectList.forEach((s) => {
        let group = subjectGroupList.find((g) => g.quarterId === s.quarterId);
        if (group) {
          group.subjects = [...group.subjects, s];
        } else {
          subjectGroupList.push({ quarterId: s.quarterId, subjects: [s] });
        }
      });
    }
    /*
        const subjectGroupList = [
          { quarterId: QUARTER_SUMMER,
            subjects: [
              { id: '1', subjectNums: 2, courseHours: 3, teachNums: 10 },
              { id: '2', subjectNums: 1, courseHours: 3, teachNums: 15 },
            ]
          },
          { quarterId: QUARTER_AUTUMN,
            subjects: [
              { id: '1', subjectNums: 2, courseHours: 3, teachNums: 10 },
            ]
          },
          { quarterId: QUARTER_WINTER,
            subjects: [
              { id: '1', subjectNums: 2, courseHours: 3, teachNums: 10 },
            ]
          },
          { quarterId: QUARTER_SPRING,
            subjects: [
              { id: '1', subjectNums: 2, courseHours: 3, teachNums: 10 },
            ]
          },
        ]*/
    const data = {
      additionName: apiData.shortname,
      year: apiData.productVersionId,
      grade: apiData.gradeId,
      gradeLevel: apiData.gradeSegmentId,
      gradeLevelName: apiData.gradeSegmentName,
      serise: apiData.courseSeriesId,
      group: apiData.productGroupId,
      classTimeLen: apiData.classTimeLength,
      price: apiData.price,
      startDate: apiData.validStartDate,
      endDate: apiData.validEndDate,
      feeRule: apiData.chargeRuleId,
      feelClass: apiData.experienceLessonId,
      branchs: apiData.branchId,
      linkSubjects: apiData.subjectIdSet,
      linkProducts: apiData.sonProduct.map((p) => p.id),
      saleNums: apiData.saleNums,
      editable: apiData.canEdit === 1 ? false : true,
      remark: apiData.remark || '',
      quarter: apiData.productQuarterId,
      classType: apiData.classTypeId,
      classLenType: apiData.classModalId,
      name: apiData.name,
      everyCourseHours: apiData.everyCourseHours,
      isCouseHoursPlaned: !!apiData.subjectList && apiData.subjectList.length > 0,
      subjectGroupList,
      miniClassTotalhours: apiData.miniClassTotalhours,
    };
    return data;
  };

  /**
   * 分公司改变时加载其他关联产品选项
   * @param  {string} branchs 分公司
   */
  branchChange = (branchs) => {
    this.setState({ branchs, linkProducts: null, linkProductsDict: null });
    loadLinkProductsDict(branchs, productTypeVal).then((linkProductsDict) => {
      this.setState({ linkProductsDict });
    });
  };
  /**
   * 年级值变化时,更新年级段
   * @param {string} grade 年级ID
   */
  gradeChange = (grade) => {
    this.setState({ gradeLevelName: '' });
    if (grade) {
      getDataDictBySonId({ sonId: grade }, true).then((item) => {
        this.setState({ gradeLevel: item.id, gradeLevelName: item.name, grade });
      });
    }
  };

  /**
   * 获取产品名称自动生成部分
   * @param {string} year 年份ID
   * @param {string} grade 年级ID
   */
  getAutoName = (year, grade, quarter, classType) => {
    const { yearDict, gradeDict, quarterDict, classTypeDict } = this.state;
    return (
      (getDictTextByVal(yearDict, year) || '') +
      (getDictTextByVal(gradeDict, grade) || '') +
      (getDictTextByVal(quarterDict, quarter) || '') +
      (getDictTextByVal(classTypeDict, classType) || '')
    );
  };

  /**
   * 保存按钮点击处理
   */
  handleOk = () => {
    const {
      params: { productId, onSaveSuccess },
    } = this.props;
    const {
      additionName,
      year,
      grade,
      quarter,
      classLenType,
      classType,
      serise,
      group,
      classTimeLen,
      price,
      startDate,
      endDate,
      feeRule,
      feelClass,
      branchs,
      linkSubjects,
      linkProducts,
      editable,
      remark,
      everyCourseHours,
      gradeLevel,
      subjectGroupList,
      miniClassTotalhours,
      yearDict,
    } = this.state;
    const useHoursPlan = year && parseInt(getDictTextByVal(yearDict, year)) >= 2019;
    const error = (text) => this._page.showOkTips(text);
    if (!year) {
      error('请填写产品年份');
      return;
    }
    if (!grade) {
      error('请填写年级');
      return;
    }
    if (!quarter) {
      error('请填写季度');
      return;
    }
    if (!classType) {
      error('请填写班级类型');
      return;
    }
    if (useHoursPlan) {
      if (!subjectGroupList || subjectGroupList.length === 0) {
        error('请规划课时');
        return;
      }
    } else {
      if (!miniClassTotalhours) {
        error('填写总课时数');
        return;
      }
    }
    /*        if (!everyCourseHours) {
            error('请填写课时数/(次)')
            return
        }*/
    if (!classTimeLen) {
      error('请填写课时时长');
      return;
    }
    if (!price) {
      error('请填写单价');
      return;
    }
    if (!startDate || !endDate) {
      error('请填写上下架日期');
      return;
    }
    if (!branchs || branchs.length === 0) {
      error('请填写分公司');
      return;
    }
    if (!feeRule) {
      error('请填写扣费规则');
      return;
    }
    /*        if (parseInt(endDate.slice(0, 4)) < parseInt(yearDict.find(d => d.val === year).text)) {
          error('下架日期的年份不能小于产品年份')
          return
        }*/
    // if (!linkSubjects || linkSubjects.length === 0) {
    //     error('请填写关联科目')
    //     return
    // }
    let totalHours;
    if (useHoursPlan) {
      const subjectSummary = getSummaryBySubjectGroupList(subjectGroupList);
      totalHours = subjectSummary ? subjectSummary.totalHours : null;
    } else {
      totalHours = miniClassTotalhours;
    }
    /*      if(totalHours % everyCourseHours !== 0){
          error('课时数/次，必须能被总课时数整除')
          return
      }*/
    let subjectList = [];
    subjectGroupList.forEach((g) => {
      g.subjects.forEach((s) => {
        subjectList.push({ quarterId: g.quarterId, ...s });
      });
    });
    let saveData = {
      name: this.getAutoName(year, grade, quarter, classType) + (additionName || ''),
      shortname: additionName,
      productVersionId: year,
      gradeId: grade,
      courseSeriesId: serise,
      productGroupId: group,
      classTimeLength: classTimeLen,
      gradeSegmentId: gradeLevel,
      price,
      productQuarterId: quarter,
      validStartDate: startDate,
      validEndDate: endDate,
      chargeRuleId: feeRule,
      experienceLessonId: feelClass,
      branchId: branchs,
      subjectId: linkSubjects,
      sonProductId: linkProducts,
      canEdit: editable ? 0 : 1,
      remark,
      classModalId: classLenType,
      classTypeId: classType,
      category: productTypeVal,
      productCategoryId: productType.id,
      miniClassTotalhours: totalHours,
      everyCourseHours,
      subjectList,
    };
    if (productId) {
      saveData.id = productId;
    }
    //console.log(saveData, JSON.stringify(saveData))
    this.setState({ loading: true });
    saveOrUpdateProduct(saveData, true).then(
      () => {
        showMessage('保存成功', 'success');
        this.setState({ loading: false });
        onSaveSuccess();
      },
      (e) => {
        if (e.text) {
          error(e.text);
        }
        showMessage('保存失败');
        this.setState({ loading: false });
      },
    );
  };

  /**
   * 获取要禁用的开始日期值
   * @param  {string} startDate 过滤前的日期值
   * @return {boolean} 是否要禁用
   */
  getDisabledStartDate = (startDate) => {
    const { endDate } = this.state;
    if (!startDate || !endDate) {
      return false;
    }
    return dateToMoment(startDate).valueOf() > dateToMoment(endDate).valueOf();
  };

  /**
   * 获取要禁用的结束日期值
   * @param  {string} endDate   过滤前的日期值
   * @return {boolean}      是否要禁用
   */
  getDisabledEndDate = (endDate) => {
    const { startDate } = this.state;
    if (!endDate || !startDate) {
      return false;
    }
    return dateToMoment(endDate).valueOf() <= dateToMoment(startDate).valueOf();
  };

  hoursPlanBtnClick = () => {
    const { quarter } = this.state;
    if (!quarter) {
      showAlert('请先选择产品季度');
      return;
    }
    this.setState({ hoursPlanModalVis: true });
  };

  quarterChange = (val) => {
    const { quarter } = this.state;
    const isFullState = quarter === QUARTER_SUMMER || quarter === QUARTER_All_YEAR;
    const isFull = val === QUARTER_SUMMER || val === QUARTER_All_YEAR;
    if ((isFullState && !isFull) || (!isFullState && quarter !== val)) {
      this.setState({ subjectGroupList: [] });
    }
    this.setState({ quarter: val });
  };

  render() {
    const {
      params: { sellCount, productId },
    } = this.props;
    const {
      loading,
      additionName,
      year,
      quarter,
      classType,
      classLenType,
      classLenTypeDict,
      grade,
      serise,
      group,
      classTimeLen,
      price,
      startDate,
      endDate,
      feeRule,
      feelClass,
      branchs,
      linkSubjects,
      linkProducts,
      saleNums,
      yearDict,
      quarterDict,
      classTypeDict,
      gradeDict,
      feeRuleDict,
      branchsDict,
      linkSubjectsDict,
      linkProductsDict,
      editable,
      remark,
      everyCourseHours,
      gradeLevelName,
      isCouseHoursPlaned,
      subjectGroupList,
      hoursPlanModalVis,
      isAfter2019,
      miniClassTotalhours,
    } = this.state;
    const partEdit = !sellCount && sellCount > 0;
    const useHoursPlan = year && parseInt(getDictTextByVal(yearDict, year)) >= 2019;
    let totalHours;
    if (useHoursPlan) {
      const subjectSummary = getSummaryBySubjectGroupList(subjectGroupList);
      totalHours = subjectSummary ? subjectSummary.totalHours : null;
    } else {
      totalHours = miniClassTotalhours;
    }
    return (
      <Page
        name={productId ? '编辑产品' : '新增产品'}
        footer={true}
        loading={loading}
        route={this.props.route}
        onOk={this.handleOk}
        ref={(c) => (this._page = c)}
      >
        <Block title="产品信息" style={{ marginTop: -5, marginLeft: 45 }} className={styles.wrap}>
          <Row style={{ marginLeft: -26, marginTop: 5 }}>
            <Col span={12}>
              <FormItem name="产品名称">
                <Row gutter={10}>
                  <Col span={14} className="text-ellipsis">
                    {this.getAutoName(year, grade, quarter, classType)}
                  </Col>
                  <Col span={10}>
                    <Input
                      placeholder="附加名称"
                      value={additionName}
                      onChange={(e) => this.setState({ additionName: cutText(e.target.value, 25) })}
                      disabled={partEdit}
                    />
                  </Col>
                </Row>
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="年份" required={true}>
                <Select
                  dict={yearDict}
                  value={year}
                  onChange={(val) => this.setState({ year: val })}
                  disabled={partEdit || isAfter2019}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="年级" required={true}>
                <Select dict={gradeDict} value={grade} onChange={this.gradeChange} disabled={partEdit} />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="年级段">
                {/*{getDictTextByVal(gradeLevelDict, getGradeLevel(grade))}*/}
                {gradeLevelName}
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="季度" required={true}>
                <Select
                  dict={quarterDict}
                  value={quarter}
                  onChange={this.quarterChange}
                  disabled={partEdit || isAfter2019}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="班级类型" required={true}>
                <Select
                  dict={classTypeDict}
                  value={classType}
                  onChange={(val) => this.setState({ classType: val })}
                  disabled={partEdit}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="总课时数" required={true}>
                <div className={sty.totalHoursWrap}>
                  {useHoursPlan && !!totalHours && (
                    <div className={sty.totalHoursInfo}>
                      {totalHours}
                      <TotalHoursPlanPop subjectGroupList={subjectGroupList} quarterDict={quarterDict}>
                        <IconButton type="message_mc" style={{ marginLeft: 15, position: 'relative', top: 1 }} />
                      </TotalHoursPlanPop>
                    </div>
                  )}
                  {!partEdit && useHoursPlan && (
                    <div className={sty.totalHoursBtn}>
                      <Button onClick={this.hoursPlanBtnClick}>规划课时</Button>
                    </div>
                  )}
                  {!useHoursPlan && (
                    <InputNumber
                      min={0.1}
                      value={miniClassTotalhours}
                      precision={1}
                      onChange={(val) => this.setState({ miniClassTotalhours: val })}
                      disabled={partEdit}
                    />
                  )}
                </div>
              </FormItem>
            </Col>
            {/*                        <Col span={12}>
                            <FormItem name="课时数/（次）" required={true}>
                                <InputNumber min={0.1} value={everyCourseHours} precision={1}
                                             onChange={val => this.setState({everyCourseHours: val})} disabled={partEdit}/>
                            </FormItem>
                        </Col>*/}

            <Col span={12}>
              <FormItem name="单价（元）" required={true}>
                <InputNumber
                  min={0.01}
                  value={price}
                  precision={2}
                  onChange={(val) => this.setState({ price: val })}
                  disabled={partEdit}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="总价（元）">
                {!isNaN(parseFloat(totalHours)) && !isNaN(parseFloat(price))
                  ? formatCurrency(totalHours * price)
                  : null}
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="课时时长（分）" required={true}>
                <InputNumber
                  min={1}
                  value={classTimeLen}
                  precision={0}
                  onChange={(val) => this.setState({ classTimeLen: val })}
                  disabled={partEdit}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="扣费规则" required={true}>
                <Select
                  dict={feeRuleDict}
                  value={feeRule}
                  onChange={(val) => this.setState({ feeRule: val })}
                  disabled={partEdit}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="上架日期" required={true}>
                <DatePicker
                  value={startDate}
                  onChange={(val) => this.setState({ startDate: val })}
                  disabled={partEdit}
                  disabledDate={(val) => this.getDisabledStartDate(val)}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="下架日期" required={true}>
                <DatePicker
                  value={endDate}
                  onChange={(val) => this.setState({ endDate: val })}
                  disabledDate={(val) => this.getDisabledEndDate(val)}
                />
              </FormItem>
            </Col>
            <Col span={20}>
              <FormItem name="所属分公司" required={true}>
                <TagSelect dict={branchsDict} value={branchs} onChange={this.branchChange} disabled={partEdit} />
              </FormItem>
            </Col>
            {/*<Col span={20}>*/}
            {/*<FormItem name="关联科目" required={true}>*/}
            {/*<TagSelect dict={linkSubjectsDict} value={linkSubjects}*/}
            {/*onChange={val => {*/}
            {/*this.setState({linkSubjects: val})*/}
            {/*}}*/}
            {/*disabled={partEdit}*/}
            {/*/>*/}
            {/*</FormItem>*/}
            {/*</Col>*/}
            <Col span={20}>
              <FormItem name="其他关联产品" style={{ marginBottom: 0 }}>
                <TagGroup
                  dict={dictAddOptionTitle(formatGroupDict(linkProductsDict, 'branchName'))}
                  value={linkProducts}
                  onChange={(val) => {
                    this.setState({ linkProducts: val });
                  }}
                  disabled={partEdit}
                />
              </FormItem>
            </Col>
            <Col span={4}>
              <FormItem labelWidth={0} style={{ paddingLeft: 0, paddingRight: 5, marginLeft: -30 }}>
                <Checkbox
                  checked={editable}
                  onChange={(e) => this.setState({ editable: e.target.checked })}
                  disabled={partEdit}
                >
                  关联产品允许编辑
                </Checkbox>
              </FormItem>
            </Col>
            <Col span={24}>
              <FormItem name="备注" style={{ marginTop: 20, lineHeight: '36px' }}>
                <TextArea
                  rows={3}
                  value={remark}
                  onChange={(e) => this.setState({ remark: cutText(e.target.value, 50) })}
                  disabled={partEdit}
                />
              </FormItem>
            </Col>
          </Row>
        </Block>
        {hoursPlanModalVis && (
          <HoursPlanModal
            productId={productId}
            quarter={quarter}
            onClose={() => this.setState({ hoursPlanModalVis: false })}
            subjectGroupList={subjectGroupList}
            onConfirm={(subjectGroupList) => this.setState({ hoursPlanModalVis: false, subjectGroupList })}
            quarterDict={quarterDict}
            isSaled={saleNums > 0}
            isCouseHoursPlaned={isCouseHoursPlaned}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    params: state.main.pageParams['/ProductEcsClass/Edit'],
  };
};

export default connect(mapStateToProps)(EditPage);
