import React from 'react';
import Modal from 'COMPONENT/Modal';
import styles from './styles.less';
import { getDictTextByVal } from 'UTIL/dict';
import { getCommonErrorTips } from 'COMPONENT/utils/form';
import QuaterSubjectList from './QuaterSubjectList';
import classnames from 'classnames';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import FormItem from 'COMPONENT/FormItem';
import { getSummaryBySubjectGroupList } from '../utils';
import {
  QUARTER_SUMMER,
  QUARTER_AUTUMN,
  QUARTER_WINTER,
  QUARTER_SPRING,
  QUARTER_All_YEAR,
  includeQuarters,
} from 'PAGE/common/utils';
import { Row, Col } from 'antd';

const formCfg = {
  labelWidth: 70,
};

class HoursPlanModal extends React.Component {
  constructor(props) {
    super(props);
    const { subjectGroupList: subjectGroupListProp, quarter } = props;
    const showQuarters = includeQuarters[quarter];
    let subjectGroupList = [];
    showQuarters.forEach((qId) => {
      const group = subjectGroupListProp.find((g) => g.quarterId === qId);
      if (group) {
        subjectGroupList.push(group);
      } else {
        subjectGroupList.push({ quarterId: qId, subjects: [{ courseHours: 3, subjectNums: null, teachNums: null }] });
      }
    });
    const summary = getSummaryBySubjectGroupList(subjectGroupListProp);
    this.state = {
      loading: false, // {boolean} 等待状态
      subjectGroupList, // {object[]} 按季度分组的科目列表
      checking: false, // {boolean} 校验开启状态
      initTotalHours: summary ? summary.totalHours : null, // {number} 初始的总课时数
    };
    this.errors = {};
  }

  onOk = () => {
    const { isSaled, onConfirm, isCouseHoursPlaned } = this.props;
    const { checking, subjectGroupList, initTotalHours } = this.state;
    if (!checking) {
      this.setState({ checking: true });
    }
    const errorTips = getCommonErrorTips(this.errors);
    if (errorTips) {
      this.modal.showOkTips(errorTips);
      return;
    }
    if (isCouseHoursPlaned && isSaled && getSummaryBySubjectGroupList(subjectGroupList).totalHours !== initTotalHours) {
      this.modal.showOkTips('该产品已被销售，不能改变总课时数');
      return;
    }
    onConfirm(subjectGroupList);
    /*
        saveData = { achievementProStandards: saveData }
        //console.log('saveData:', saveData, JSON.stringify(saveData))
        updateAchievementProStandard(saveData, true).then(() => {
          showMessage('保存成功', 'success')
          this.setState({ loading: false })
          onClose()
        }, (e) => {
          if(e.text){
            this.modal.showOkTips(e.text)
          } else {
            showMessage('保存失败')
          }
          this.setState({ loading: false })
        })
        */
  };

  onChange = (quarterId, data) => {
    this.setState({
      subjectGroupList: this.state.subjectGroupList.map((g) => {
        if (g.quarterId === quarterId) {
          return data;
        }
        return g;
      }),
    });
  };

  onErrorChange = (quarterId, errors) => {
    let newErrors = {};
    Object.keys(this.errors).forEach((key) => {
      const a = key.split('_');
      if (a[0] !== quarterId) {
        newErrors[key] = this.errors[key];
      }
    });
    this.errors = newErrors;
    if (errors) {
      let errorsHasFullKey = {};
      Object.keys(errors).forEach((k) => {
        errorsHasFullKey[quarterId + '_' + k] = errors[k];
      });
      this.errors = { ...this.errors, ...errorsHasFullKey };
    }
  };

  render() {
    const { onClose, quarterDict } = this.props;
    const { loading, subjectGroupList, checking } = this.state;
    const modalCls = {
      [styles.wrap]: 1,
      [styles.modal]: 1,
    };
    const summary = getSummaryBySubjectGroupList(subjectGroupList);
    return (
      <Modal
        className={classnames(modalCls)}
        loading={loading}
        title="科次课时规划"
        onOk={this.onOk}
        width={640}
        onClose={onClose}
        closable
        noLine
        ref={(c) => (this.modal = getWrappedInstance(c))}
      >
        {subjectGroupList.map((g) => {
          return (
            <QuaterSubjectList
              key={g.quarterId}
              data={g}
              onChange={(data) => this.onChange(g.quarterId, data)}
              onErrorChange={(errors) => this.onErrorChange(g.quarterId, errors)}
              checking={checking}
              quarterDict={quarterDict}
            />
          );
        })}
        <Row className={styles.footerInfoWrap}>
          <Col span={8}>
            <FormItem {...formCfg} name="总科次数">
              {summary.totalSubjectNums}
            </FormItem>
          </Col>
          <Col span={8}>
            <FormItem {...formCfg} name="总讲数">
              {summary.totalTeachNums}
            </FormItem>
          </Col>
          <Col span={8}>
            <FormItem {...formCfg} name="总课时数">
              {summary.totalHours}
            </FormItem>
          </Col>
        </Row>
      </Modal>
    );
  }
}

export default HoursPlanModal;
