import React from 'react';
import { connect } from 'dva';
import InputNumber from 'COMPONENT/InputNumber';
import Input from 'COMPONENT/Input';
import Button from 'COMPONENT/Button';
import Page from 'COMPONENT/Page';
import SearchBar from 'COMPONENT/SearchBar';
import List from 'COMPONENT/List';
import Select from 'COMPONENT/Select';
import DatePicker from 'COMPONENT/DatePicker';
import { getDictTextByVal, loadCommonDict, loadBranchDict, loadAllBranchDict, formateDict } from 'UTIL/dict';
import { getUserByRoldCodesSelection } from 'SERVICE/common';
import { getProductPageList, cancelProduct } from 'SERVICE/product';
import { statusDict } from 'CONST/dicts/product';
import ProductDetailViewModal from './ModalDetailView';
import styles from 'PAGE/product/listPage.less';

const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;
const productTypeVal = 'LIVE';

class Live extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 内容加载状态
      searchExpand: false, // {boolean} 搜索栏展开状态

      productName: null, // {string} 查询条件 产品名称
      branch: null, // {string} 查询条件 所属分公司
      teacher: null, // {string} 查询条件 主讲老师
      grade: null, // {string} 查询条件 年级
      subject: null, // {string} 查询条件 科目
      classType: null, // {string} 查询条件 类型
      quarter: null, // {string} 查询条件 季度
      price: null, // {string} 查询条件 产品单价
      totalHours: null, // {string} 查询条件 总课时数
      classTimeLength: null, // {string} 查询条件 课时时长
      startDate: null, // {string} 查询条件 上架日期
      endDate: null, // {string} 查询条件 下架日期
      feeRule: null, // {string} 查询条件 扣费规则
      status: null, // {string} 查询条件 状态

      branchDict: null, // {[object]} 字典 所属分公司
      teacherDict: null, // {[object]} 字典 主讲老师
      gradeDict: null, // {[object]} 字典 年级
      subjectDict: null, // {[object]} 字典 科目
      classTypeDict: null, // {[object]} 字典 类型
      quarterDict: null, // {[object]} 字典 季度
      feeRuleDict: null, // {[object]} 字典 扣费规则

      list: [], // {Array} 列表数据
      listLoading: false, // {false} 列表数据加载状态
      listSearchKeys: null, // {object} 列表数据的查询关键字
      totalRecords: 0, // {number} 总记录数
      currentPage: 1, // {number} 当前页码
      pageSize: 20, // {number} 每页记录数

      viewModalVisible: false, // {boolean} 查看弹框显示状态
      currentRecord: null, // {object} 当前激活状态的列表行数据
      allBranchDict: null, // {[object]} 字典 所有分公司
    };
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化内容
   */
  initContent = () => {
    this.setState({ loading: true });
    Promise.all([
      loadBranchDict(),
      this.loadTeacherDict(),
      loadCommonDict('STUDENT_GRADE'),
      loadCommonDict('SUBJECT'),
      loadCommonDict('CLASS_TYPE'),
      loadCommonDict('PRODUCT_QUARTER'),
      loadCommonDict('PRODUCT_CHARGE_RULE'),
      loadAllBranchDict(),
    ]).then(
      ([branchDict, teacherDict, gradeDict, subjectDict, classTypeDict, quarterDict, feeRuleDict, allBranchDict]) => {
        this.setState({
          loading: false,
          branchDict,
          teacherDict,
          gradeDict,
          subjectDict,
          classTypeDict,
          quarterDict,
          feeRuleDict,
          allBranchDict,
        });
      },
    );
    this.search();
  };

  /**
   * 加载老师字典
   */
  loadTeacherDict = async () => {
    const apiData = await getUserByRoldCodesSelection({ roleCode: 'TEATCHER' });
    return formateDict(apiData);
  };

  /**
   * 加载列表数据，并转换格式
   * @param  {object} changedParams 要更新的列表数据参数，包括查询条件、页码状态
   */
  loadList = async (changedParams = {}) => {
    const {
      productName,
      branch,
      teacher,
      grade,
      subject,
      classType,
      quarter,
      price,
      totalHours,
      classTimeLength,
      startDate,
      endDate,
      feeRule,
      status,
      currentPage,
      pageSize,
      listSearchKeys,
    } = this.state;
    const searchKeys = listSearchKeys || {
      productName,
      branch,
      teacher,
      grade,
      subject,
      classType,
      quarter,
      price,
      totalHours,
      classTimeLength,
      startDate,
      endDate,
      feeRule,
      status,
    };
    const params = { currentPage, pageSize, ...searchKeys, ...changedParams };
    const apiParams = {
      type: productTypeVal,
      page: params.currentPage,
      rows: params.pageSize,
      name: params.productName,
      brenchId: params.branch,
      teacherId: params.teacher,
      gradeId: params.grade,
      subjectIds: params.subject ? [params.subject].join(',') : '',
      classTypeId: params.classType,
      seasonId: params.quarter,
      price: params.price,
      totalHours: params.totalHours,
      classTimeLength: params.classTimeLength,
      startDate: params.startDate,
      endDate: params.endDate,
      chargeRuleId: params.feeRule,
      proStatus: params.status,
    };
    console.log(apiParams, JSON.stringify(apiParams));
    this.setState({ listLoading: true });
    const apiData = await getProductPageList(apiParams);
    const { records, rows } = apiData;
    const formatedRows = rows.map((row) => {
      return {
        productId: row.id,
        productName: row.name,
        branchNames: row.organizationName,
        teacherName: row.teacherName,
        gradeName: row.gradeName,
        subjectName: row.subjectName,
        classTypeName: row.classTypeName,
        quarterName: row.productQuarterName,
        price: row.price,
        totalHours: row.miniClassTotalhours,
        totalMoney: row.totalAmount,
        classTimeLength: row.classTimeLength,
        startDate: row.validStartDate,
        endDate: row.validEndDate,
        sellCount: 0,
        feeRuleName: row.chargeRuleName,
        statusVal: row.proStatus,
      };
    });
    const data = {
      records,
      rows: formatedRows,
    };
    console.log(data);
    const { currentPage: newCurrentPage, pageSize: newPageSize, ...newListSearchKeys } = params;
    this.setState({
      listLoading: false,
      list: data.rows,
      totalRecords: data.records,
      currentPage: newCurrentPage,
      pageSize: newPageSize,
      listSearchKeys: newListSearchKeys,
    });
    return true;
  };

  /**
   * 查询
   */
  search = () => {
    const {
      productName,
      branch,
      teacher,
      grade,
      subject,
      classType,
      quarter,
      price,
      totalHours,
      classTimeLength,
      startDate,
      endDate,
      feeRule,
      status,
    } = this.state;
    this.loadList({
      productName,
      branch,
      teacher,
      grade,
      subject,
      classType,
      quarter,
      price,
      totalHours,
      classTimeLength,
      startDate,
      endDate,
      feeRule,
      status,
      currentPage: 1,
    });
  };

  /**
   * 重置查询条件
   */
  reset = () => {
    this.setState({
      productName: null,
      branch: null,
      teacher: null,
      grade: null,
      subject: null,
      classType: null,
      quarter: null,
      price: null,
      totalHours: null,
      classTimeLength: null,
      startDate: null,
      endDate: null,
      feeRule: null,
      status: null,
    });
  };

  /**
   * 列表操作按钮点击处理
   * @param  {string} type 操作类型
   * @param  {object} record 列表行数据
   * @param  {number} index 列表行序号
   */
  listBtnClick = (type, record, index) => {
    const { productId } = record;
    if (type === 'view') {
      this.setState({ viewModalVisible: true, currentRecord: record });
    } else if (type === 'edit') {
      this.edit(productId);
    }
  };

  /**
   * 编辑产品保存成功后的处理
   */
  editSaveSuccess = () => {
    const { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: { path: '/ProductLive' },
    });
    this.loadList({ currentPage: 1 });
  };

  /**
   * 进入编辑产品页面
   * @param  {string} productId 产品ID
   */
  edit = (productId) => {
    const { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: {
        path: '/ProductLive/Edit',
        params: {
          productId: productId || null,
          onSaveSuccess: this.editSaveSuccess,
        },
        refresh: true,
      },
    });
  };

  render() {
    const {
      loading,
      searchExpand,
      productName,
      branch,
      teacher,
      grade,
      subject,
      classType,
      quarter,
      price,
      totalHours,
      classTimeLength,
      startDate,
      endDate,
      feeRule,
      status,
      branchDict,
      teacherDict,
      gradeDict,
      subjectDict,
      classTypeDict,
      quarterDict,
      feeRuleDict,
      list,
      totalRecords,
      currentPage,
      pageSize,
      listLoading,
      viewModalVisible,
      currentRecord,
      allBranchDict,
    } = this.state;
    const columns = [
      { title: '产品名称', dataIndex: 'productName', width: COL_WIDTH_L },
      { title: '所属分公司', dataIndex: 'branchNames', width: COL_WIDTH_XL, lines: 2 },
      { title: '主讲老师', dataIndex: 'teacherName', width: COL_WIDTH_M },
      { title: '年级', dataIndex: 'gradeName', width: COL_WIDTH_S },
      { title: '科目', dataIndex: 'subjectName', width: COL_WIDTH_M, lines: 2 },
      { title: '类型', dataIndex: 'classTypeName', width: COL_WIDTH_M },
      { title: '季度', dataIndex: 'quarterName', width: COL_WIDTH_S },
      { title: '产品单价', dataIndex: 'price', width: COL_WIDTH_M, format: 'money' },
      { title: '总课时数', dataIndex: 'totalHours', width: COL_WIDTH_M },
      { title: '总价', dataIndex: 'totalMoney', width: COL_WIDTH_M, format: 'money' },
      { title: '课时时长', dataIndex: 'classTimeLength', width: COL_WIDTH_M },
      {
        title: '上 / 下架日期',
        key: 'dateRange',
        width: COL_WIDTH_L,
        render: (text, record) => {
          return (
            <div>
              <div>{`起：${record.startDate}`}</div>
              <div>{`止：${record.endDate}`}</div>
            </div>
          );
        },
      },
      { title: '销售数量', dataIndex: 'sellCount', width: COL_WIDTH_M },
      { title: '扣费规则', dataIndex: 'feeRuleName', width: COL_WIDTH_M },
      {
        title: '状态',
        key: 'statusName',
        width: COL_WIDTH_S,
        render: (text, record) => getDictTextByVal(statusDict, record.statusVal),
      },
      {
        title: '操作',
        fixed: 'right',
        key: 'action',
        width: COL_WIDTH_M,
        getActionConfig: (text, record, index) => {
          const { sellCount, productId } = record;
          const isInvalid = record.statusVal === '1';
          return [
            { text: '查看', onClick: () => this.listBtnClick('view', record, index) },
            {
              text: '修改',
              onClick: () => this.listBtnClick('edit', record, index),
              disabled: sellCount > 0,
              hide: isInvalid,
            },
          ];
        },
      },
    ];

    return (
      <Page
        fitWindowHeight={true}
        loading={loading}
        route={this.props.route}
        subHeader={
          <SearchBar
            expand={searchExpand}
            onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
            onSearch={this.search}
            onReset={this.reset}
          >
            <Input
              placeholder="产品名称"
              value={productName}
              onChange={(e) => this.setState({ productName: e.target.value })}
            />
            <Select
              placeholder="所属分公司"
              dict={branchDict}
              value={branch}
              onChange={(val) => this.setState({ branch: val })}
            />
            <Select
              placeholder="主讲老师"
              dict={teacherDict}
              value={teacher}
              onChange={(val) => this.setState({ teacher: val })}
            />
            <Select
              placeholder="年级"
              dict={gradeDict}
              value={grade}
              onChange={(val) => this.setState({ grade: val })}
            />
            <Select
              placeholder="科目"
              dict={subjectDict}
              value={subject}
              onChange={(val) => this.setState({ subject: val })}
            />
            <Select
              placeholder="类型"
              dict={classTypeDict}
              value={classType}
              onChange={(val) => this.setState({ classType: val })}
            />
            <Select
              placeholder="季度"
              dict={quarterDict}
              value={quarter}
              onChange={(val) => this.setState({ quarter: val })}
            />
            <InputNumber placeholder="产品单价" value={price} onChange={(val) => this.setState({ price: val })} />
            <InputNumber
              placeholder="总课时数"
              value={totalHours}
              onChange={(val) => this.setState({ totalHours: val })}
            />
            <InputNumber
              placeholder="课时时长"
              value={classTimeLength}
              onChange={(val) => this.setState({ classTimeLength: val })}
            />
            <DatePicker
              placeholder="上架日期"
              value={startDate}
              onChange={(val) => this.setState({ startDate: val })}
            />
            <DatePicker placeholder="下架日期" value={endDate} onChange={(val) => this.setState({ endDate: val })} />
            <Select
              placeholder="扣费规则"
              dict={feeRuleDict}
              value={feeRule}
              onChange={(val) => this.setState({ feeRule: val })}
            />
            <Select
              placeholder="状态"
              dict={statusDict}
              value={status}
              onChange={(val) => this.setState({ status: val })}
            />
          </SearchBar>
        }
      >
        <List
          dataSource={list}
          columns={columns}
          loading={!loading && listLoading}
          pagination={{
            current: currentPage,
            total: totalRecords,
            pageSize,
          }}
          onChange={({ current, pageSize }) => this.loadList({ currentPage: current, pageSize })}
        />
        {viewModalVisible && (
          <ProductDetailViewModal
            productId={currentRecord.productId}
            allBranchDict={allBranchDict}
            subjectDict={subjectDict}
            onClose={() => this.setState({ viewModalVisible: false })}
          />
        )}
      </Page>
    );
  }
}

export default connect()(Live);
