/**
 * 编辑产品页面，新增或修改
 * @param {string?} productId 产品ID，新增时为空
 * @param {function} onSaveSucess 保存成功时的回调
 */
import React from 'react';
import { connect } from 'dva';
import { Row, Col } from 'antd';
import Input from 'COMPONENT/Input';
import InputNumber from 'COMPONENT/InputNumber';
import Checkbox from 'COMPONENT/Checkbox';
import Button from 'COMPONENT/Button';
import Page from 'COMPONENT/Page';
import Select from 'COMPONENT/Select';
import TagSelect from 'COMPONENT/TagSelect';
import DatePicker from 'COMPONENT/DatePicker';
import TextArea from 'COMPONENT/TextArea';
import Block from 'COMPONENT/Block';
import FormItem from 'COMPONENT/FormItem';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { cutText } from 'COMPONENT/utils/basic/str';
import { dateToMoment } from 'COMPONENT/utils/form';
import { getDictTextByVal, loadCommonDict, loadBranchDict, loadAllBranchDict, formatGroupDict } from 'UTIL/dict';
import { getDataDictBySonId } from 'SERVICE/system';
import { getProductById, saveOrUpdateProduct } from 'SERVICE/product';
import SelectBookModal from '../../modals/SelectBook';
import { loadLinkProductsDict, currentYearText } from '../../common';
import { productTypeDict } from 'CONST/dicts/product';
import styles from 'PAGE/product/editPage.less';
import vars from 'COMPONENT/theme/variables';
import { formatCurrency } from 'COMPONENT/utils/basic/num';

const productTypeVal = 'SMALL_CLASS';
const productType = productTypeDict.find((p) => p.val === productTypeVal);

class EditPage extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 内容加载状态
      selectBookModalVisible: false, // {boolean} 选择教材弹框显示状态

      year: null, // {string} 产品年份
      quarter: null, // {string} 季度
      grade: null, // {string} 年级
      gradeLevel: null, // {string} 年级段
      gradeLevelName: null, // {string} 年级段名称
      subject: null, // {[string]} 科目
      serise: null, // {string} 产品系列
      system: null, // {string} 产品体系
      systemName: null, // {string} 产品体系名称
      classType: null, // {string} 层次
      additionName: null, // {string} 附加名称
      feeType: null, // {string} 类型
      classLenType: null, // {string} 班型
      group: null, // {string} 产品组
      classTimeLen: null, // {string} 课时时长
      price: null, // {number} 单价（元）
      totalHours: null, // {number} 总课时数
      totalHoursCount: null, // {number} 课时数/次
      startDate: null, // {string} 上架日期
      endDate: null, // {string} 下架日期
      branchs: null, // {[string]} 所属分公司
      linkProducts: null, // {[string]} 其他关联产品
      editable: true, // {boolean} 关联产品允许编辑
      classBook: [], // {[object]} 教材，ID和名称，数据格式同字典[{ val, text }]
      feeRule: 'chargeCosumeNow', // {string} 扣费规则
      remark: '', // {string} 备注
      autoName: null, // {string} 产品名称自动生成部分
      sellCount: null, // {number} 销售数量

      yearDict: null, // {[object]} 字典 产品年份
      quarterDict: null, // {[object]} 字典 季度
      gradeDict: null, // {[object]} 字典 年级
      subjectDict: null, // {[object]} 字典 科目
      seriseDict: null, // {[object]} 字典 产品系列
      classTypeDict: null, // {[object]} 字典 层次
      feeTypeDict: null, // {[object]} 字典 类型
      classLenTypeDict: null, // {[object]} 字典 班型
      systemDict: null, // {[object]} 字典 产品体系
      groupDict: null, // {[object]} 字典 产品组
      feeRuleDict: null, // {[object]} 字典 扣费规则
      branchsDict: null, // {[object]} 字典 分公司
      linkProductsDict: null, // {[object]} 字典 其他关联产品
    };
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化内容
   */
  initContent = async () => {
    const {
      params: { productId },
    } = this.props;
    this.setState({ loading: true });
    let calls = [
      loadCommonDict('PRODUCT_VERSION'),
      loadCommonDict('PRODUCT_QUARTER'),
      loadCommonDict('STUDENT_GRADE'),
      loadCommonDict('SUBJECT'),
      loadCommonDict('COURSE_SERIES'),
      loadCommonDict('CLASS_TYPE'),
      loadCommonDict('PRODUCT_FEE_TYPE'),
      loadCommonDict('PRODUCT_CLASS_MODAL'),
      loadCommonDict('PRODUCT_SYSTEM'),
      loadCommonDict('PRODUCT_GROUP'),
      loadCommonDict('PRODUCT_CHARGE_RULE'),
      loadAllBranchDict(),
    ];
    if (productId) {
      calls.push(this.loadProductDetail(productId));
    }
    const [
      yearDict,
      quarterDict,
      gradeDict,
      subjectDict,
      seriseDict,
      classTypeDict,
      feeTypeDict,
      classLenTypeDict,
      systemDict,
      groupDict,
      feeRuleDict,
      branchsDict,
      productDetail,
    ] = await Promise.all(calls);
    let changed = {
      yearDict,
      quarterDict,
      gradeDict,
      subjectDict,
      seriseDict,
      classTypeDict,
      feeTypeDict,
      classLenTypeDict,
      systemDict,
      groupDict,
      feeRuleDict,
      branchsDict,
    };
    if (productDetail) {
      const linkProductsDict = await loadLinkProductsDict(productDetail.branchs, productTypeVal);
      changed = { ...changed, ...productDetail, linkProductsDict };
    } else {
      const yearOpt = yearDict.find((y) => y.text === currentYearText);
      changed = { ...changed, year: yearOpt ? yearOpt.val : null };
    }
    this.setState({ loading: false, ...changed });
  };

  /**
   * 加载产品详情，并转换格式
   * @param  {string} productId 产品ID
   */
  loadProductDetail = async (productId) => {
    const apiData = await getProductById({ productId });
    const { name, shortname } = apiData;
    let autoName;
    if (shortname) {
      const idx = name.lastIndexOf(shortname);
      autoName = name.substring(0, idx);
    } else {
      autoName = name;
    }
    /*

    const textBookId = apiData.textBookId + ''
    const textBookName = apiData.textBookName
    let classBook = null
    if (textBookId) {
      classBook = [{ val: textBookId, text: textBookName }]
    }
*/

    const data = {
      year: apiData.productVersionId,
      quarter: apiData.productQuarterId,
      grade: apiData.gradeId,
      gradeLevel: apiData.gradeSegmentId,
      gradeLevelName: apiData.gradeSegmentName,
      subject: apiData.subjectIdSet,
      serise: apiData.courseSeriesId,
      system: apiData.productSystemId,
      systemName: apiData.productSystemName,
      classType: apiData.classTypeId,
      autoName,
      additionName: shortname,
      feeType: apiData.feeTypeId,
      classLenType: apiData.classModalId,
      group: apiData.productGroupId,
      classTimeLen: apiData.classTimeLength,
      price: apiData.price,
      totalHours: apiData.miniClassTotalhours,
      totalHoursCount: apiData.everyCourseHours,
      startDate: apiData.validStartDate,
      endDate: apiData.validEndDate,
      branchs: apiData.branchId,
      linkProducts: apiData.sonProduct.map((p) => p.id),
      editable: apiData.canEdit === 1 ? false : true,
      classBook: apiData.textbookList.map((book) => {
        return { val: book.textBookId, text: book.textBookName };
      }),
      feeRule: apiData.chargeRuleId,
      remark: apiData.remark || '',
      sellCount: apiData.sellCount,
    };
    return data;
  };

  /**
   * 分公司改变时加载其他关联产品选项
   * @param  {string} branchs 分公司
   */
  branchChange = (branchs) => {
    this.setState({ branchs, linkProducts: null, linkProductsDict: null });
    loadLinkProductsDict(branchs, productTypeVal).then((linkProductsDict) => {
      this.setState({ linkProductsDict });
    });
  };

  /**
   * 获取产品名称自动生成部分
   * @param {string} year 年份ID
   * @param {string} quarter 季度ID
   * @param {string} grade 年级ID
   * @param {string} subject 科目ID
   * @param {string} serise 产品系列ID
   * @param {string} classType 班次ID
   */
  getAutoName = ({ year, quarter, grade, subject, serise, classType }) => {
    const { yearDict, quarterDict, gradeDict, subjectDict, seriseDict, classTypeDict } = this.state;
    const subjectNames = subject ? subject.map((s) => getDictTextByVal(subjectDict, s)).join('') : '';
    return (
      (getDictTextByVal(yearDict, year) || '') +
      (getDictTextByVal(quarterDict, quarter) || '') +
      (getDictTextByVal(gradeDict, grade) || '') +
      (subjectNames || '') +
      (getDictTextByVal(seriseDict, serise) || '') +
      (getDictTextByVal(classTypeDict, classType) || '') +
      productType.text
    );
  };

  /**
   * 影响产品名称自动生成部分的字段（年份、季度、年级、科目、产品系列、班次）改变时的处理
   * @param {object?} changed 改变的字段键值对
   */
  autoNameFieldsChange = (changed = {}) => {
    const { year, quarter, grade, subject, serise, classType } = this.state;
    this.setState({
      ...changed,
      autoName: this.getAutoName({ year, quarter, grade, subject, serise, classType, ...changed }),
    });
  };

  /**
   * 年级值变化时，1、更新年级值  2、更新产品名称自动拼凑部分  3、更新年级段
   * @param {string} grade 年级ID
   */
  gradeChange = (grade) => {
    this.autoNameFieldsChange({ grade });
    this.setState({ gradeLevelName: '' });
    if (grade) {
      getDataDictBySonId({ sonId: grade }, true).then((item) => {
        this.setState({ gradeLevel: item.id, gradeLevelName: item.name });
      });
    }
  };

  /**
   * 产品系列值变化时，1、更新产品系列值  2、更新产品名称自动拼凑部分  3、更新产品体系
   * @param {string} grade 年级ID
   */
  seriseChange = (serise) => {
    this.autoNameFieldsChange({ serise });
    this.setState({ systemName: '' });
    if (serise) {
      getDataDictBySonId({ sonId: serise }, true).then((item) => {
        this.setState({ system: item.id, systemName: item.name });
      });
    }
  };

  /**
   * 保存按钮点击处理
   */
  handleOk = () => {
    const {
      params: { productId, onSaveSuccess },
    } = this.props;
    const {
      year,
      quarter,
      grade,
      gradeLevel,
      subject,
      serise,
      system,
      classType,
      additionName,
      feeType,
      classLenType,
      group,
      classTimeLen,
      price,
      totalHours,
      totalHoursCount,
      startDate,
      endDate,
      branchs,
      linkProducts,
      editable,
      classBook,
      feeRule,
      remark,
      autoName,
      yearDict,
    } = this.state;
    const error = (text) => this._page.showOkTips(text);
    if (!autoName) {
      error('请填写产品名称');
      return;
    }
    if (!year) {
      error('请填写产品年份');
      return;
    }
    if (!quarter) {
      error('请填写季度');
      return;
    }
    if (!grade) {
      error('请填写年级');
      return;
    }
    /*    if(!subject || subject.length === 0){
      error('请填写科目')
      return
    }*/
    if (!serise) {
      error('请填写产品系列');
      return;
    }
    if (!classType) {
      error('请填写层次');
      return;
    }
    if (!feeType) {
      error('请填写类型');
      return;
    }
    if (!classLenType) {
      error('请填写班型');
      return;
    }
    if (!classTimeLen) {
      error('请填写课时时长');
      return;
    }
    if (!price) {
      error('请填写单价');
      return;
    }
    if (!totalHours) {
      error('请填写总课时数');
      return;
    }
    if (!totalHoursCount) {
      error('请填写课时数/次');
      return;
    }
    if (!startDate) {
      error('请填写上架日期');
      return;
    }
    if (!endDate) {
      error('请填写下架日期');
      return;
    }
    if (!branchs || branchs.length === 0) {
      error('请填写分公司');
      return;
    }
    if (!feeRule) {
      error('请填写扣费规则');
      return;
    }
    if (totalHours % totalHoursCount !== 0) {
      error('课时数/次，必须能被总课时数整除');
      return;
    }
    if (parseInt(endDate.slice(0, 4)) < parseInt(yearDict.find((d) => d.val === year).text)) {
      error('下架日期的年份不能小于产品年份');
      return;
    }
    let saveData = {
      productVersionId: year,
      productQuarterId: quarter,
      gradeId: grade,
      gradeSegmentId: gradeLevel,
      subjectId: subject,
      courseSeriesId: serise,
      productSystemId: system,
      classTypeId: classType,
      shortname: additionName || '',
      feeTypeId: feeType,
      classModalId: classLenType,
      productGroupId: group,
      classTimeLength: classTimeLen,
      price,
      miniClassTotalhours: totalHours,
      everyCourseHours: totalHoursCount,
      validStartDate: startDate,
      validEndDate: endDate,
      branchId: branchs,
      sonProductId: linkProducts,
      canEdit: editable ? 0 : 1,
      textBooks: classBook.map((book) => {
        return { textBookId: book.val, textBookName: book.text };
      }),
      chargeRuleId: feeRule,
      remark,
      category: productTypeVal,
      productCategoryId: productType.id,
      name: (autoName || '') + (additionName || ''),
    };
    if (productId) {
      saveData.id = productId;
    }
    console.log(JSON.stringify(saveData));
    this.setState({ loading: true });
    saveOrUpdateProduct(saveData, true).then(
      () => {
        showMessage('保存成功', 'success');
        this.setState({ loading: false });
        onSaveSuccess();
      },
      (e) => {
        if (e.text) {
          error(e.text);
        }
        showMessage('保存失败');
        this.setState({ loading: false });
      },
    );
  };

  /**
   * 获取要禁用的开始日期值
   * @param  {string} startDate 过滤前的日期值
   * @return {boolean} 是否要禁用
   */
  getDisabledStartDate = (startDate) => {
    const { endDate } = this.state;
    if (!startDate || !endDate) {
      return false;
    }
    return dateToMoment(startDate).valueOf() > dateToMoment(endDate).valueOf();
  };

  /**
   * 获取要禁用的结束日期值
   * @param  {string} endDate   过滤前的日期值
   * @return {boolean}      是否要禁用
   */
  getDisabledEndDate = (endDate) => {
    const { startDate } = this.state;
    if (!endDate || !startDate) {
      return false;
    }
    return dateToMoment(endDate).valueOf() <= dateToMoment(startDate).valueOf();
  };

  /**
   * 教材值变化时的处理
   * @param  {[string]} val 值
   */
  classBookChange = (val) => {
    const { classBook } = this.state;
    const newClassBook = classBook.filter((c) => val && val.find((v) => v === c.val));
    this.setState({ classBook: newClassBook });
  };

  /**
   * 选中教材时的处理
   * @param  {[object]} books 选中的教材，格式如[{ val, text }]
   */
  classBookSelect = (books) => {
    const { classBook } = this.state;
    let newClassBook = [...classBook];
    if (books) {
      books.forEach((b) => {
        if (!classBook || !classBook.find((c) => c.val === b.val)) {
          newClassBook.push(b);
        }
      });
    }
    this.setState({ selectBookModalVisible: false, classBook: newClassBook });
  };

  render() {
    const {
      params: { sellCount, productId },
    } = this.props;
    const {
      loading,
      selectBookModalVisible,
      year,
      quarter,
      grade,
      gradeLevelName,
      subject,
      serise,
      systemName,
      classType,
      additionName,
      feeType,
      classLenType,
      group,
      classTimeLen,
      price,
      totalHours,
      totalHoursCount,
      startDate,
      endDate,
      branchs,
      linkProducts,
      editable,
      classBook,
      feeRule,
      remark,
      autoName,
      yearDict,
      quarterDict,
      gradeDict,
      subjectDict,
      seriseDict,
      classTypeDict,
      feeTypeDict,
      classLenTypeDict,
      systemDict,
      groupDict,
      feeRuleDict,
      branchsDict,
      linkProductsDict,
    } = this.state;
    const partEdit = sellCount && sellCount > 0 ? true : false;
    return (
      <Page
        name={productId ? '编辑产品' : '新增产品'}
        footer={true}
        loading={loading}
        route={this.props.route}
        onOk={this.handleOk}
        ref={(c) => (this._page = c)}
      >
        <Block title="产品信息" style={{ marginTop: -5, marginLeft: 45 }} className={styles.wrap}>
          <Row style={{ marginLeft: -26, marginTop: 5 }}>
            <Col span={12}>
              <FormItem name="产品名称">
                <Row gutter={10}>
                  <Col span={14} className="text-ellipsis">
                    <Input
                      value={autoName}
                      title={autoName}
                      onChange={(e) => this.setState({ autoName: cutText(e.target.value, 60) })}
                      disabled={partEdit}
                    />
                  </Col>
                  <Col span={10}>
                    <Input
                      placeholder="附加名称"
                      value={additionName}
                      onChange={(e) => this.setState({ additionName: cutText(e.target.value, 25) })}
                      disabled={partEdit}
                    />
                  </Col>
                </Row>
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="产品年份" required={true}>
                <Select
                  dict={yearDict}
                  value={year}
                  onChange={(val) => this.autoNameFieldsChange({ year: val })}
                  disabled={partEdit}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="年级" required={true}>
                <Select dict={gradeDict} value={grade} onChange={this.gradeChange} disabled={partEdit} />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="年级段">{gradeLevelName}</FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="产品系列" required={true}>
                <Select dict={seriseDict} value={serise} onChange={this.seriseChange} disabled={partEdit} />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="产品体系">{systemName}</FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="产品组">
                <Select
                  dict={groupDict}
                  value={group}
                  onChange={(val) => this.setState({ group: val })}
                  disabled={partEdit}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="课时时长（分）" required={true}>
                <InputNumber
                  min={1}
                  precision={0}
                  value={classTimeLen}
                  onChange={(val) => this.setState({ classTimeLen: val })}
                  disabled={partEdit}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="单价（元）" required={true}>
                <InputNumber
                  min={0.01}
                  precision={2}
                  value={price}
                  onChange={(val) => this.setState({ price: val })}
                  disabled={partEdit}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="总课时数" required={true}>
                <InputNumber
                  min={1}
                  precision={0}
                  value={totalHours}
                  onChange={(val) => this.setState({ totalHours: val })}
                  disabled={partEdit}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="课时数/次" required={true}>
                <InputNumber
                  min={0.1}
                  precision={1}
                  value={totalHoursCount}
                  onChange={(val) => this.setState({ totalHoursCount: val })}
                  disabled={partEdit}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="总价（元）">
                {!isNaN(parseFloat(price)) && !isNaN(parseFloat(totalHours))
                  ? formatCurrency(price * totalHours)
                  : null}
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="季度" required={true}>
                <Select
                  dict={quarterDict}
                  value={quarter}
                  onChange={(val) => this.autoNameFieldsChange({ quarter: val })}
                  disabled={partEdit}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="层次" required={true}>
                <Select
                  dict={classTypeDict}
                  value={classType}
                  onChange={(val) => this.autoNameFieldsChange({ classType: val })}
                  disabled={partEdit}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="类型" required={true}>
                <Select
                  dict={feeTypeDict}
                  value={feeType}
                  onChange={(val) => this.setState({ feeType: val })}
                  disabled={partEdit}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="班型" required={true}>
                <Select
                  dict={classLenTypeDict}
                  value={classLenType}
                  onChange={(val) => this.setState({ classLenType: val })}
                  disabled={partEdit}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="上架日期" required={true}>
                <DatePicker
                  value={startDate}
                  onChange={(val) => this.setState({ startDate: val })}
                  disabled={partEdit}
                  disabledDate={(val) => this.getDisabledStartDate(val)}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="下架日期" required={true}>
                <DatePicker
                  value={endDate}
                  onChange={(val) => this.setState({ endDate: val })}
                  disabledDate={(val) => this.getDisabledEndDate(val)}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="扣费规则" required={true}>
                <Select
                  dict={feeRuleDict}
                  value={feeRule}
                  onChange={(val) => this.setState({ feeRule: val })}
                  disabled={partEdit}
                />
              </FormItem>
            </Col>
            <Col span={20}>
              <FormItem name="教材">
                <TagSelect
                  selectable={false}
                  dict={classBook}
                  value={classBook ? classBook.map((c) => c.val) : null}
                  onChange={this.classBookChange}
                  disabled={partEdit}
                />
              </FormItem>
            </Col>
            <Col span={4}>
              <FormItem
                labelWidth={0}
                style={{
                  paddingLeft: 0,
                  paddingRight: 5,
                  marginLeft: -30,
                  marginTop: -1,
                  lineHeight: vars.inputHeight,
                }}
              >
                <Button
                  type="primary"
                  onClick={() => this.setState({ selectBookModalVisible: true })}
                  disabled={partEdit}
                >
                  选择
                </Button>
              </FormItem>
            </Col>
            <Col span={20}>
              <FormItem name="所属分公司" required={true}>
                <TagSelect dict={branchsDict} value={branchs} onChange={this.branchChange} disabled={partEdit} />
              </FormItem>
            </Col>
            <Col span={20}>
              <FormItem name="关联科目">
                <TagSelect
                  dict={subjectDict}
                  value={subject}
                  onChange={(val) => this.autoNameFieldsChange({ subject: val })}
                  disabled={partEdit}
                />
              </FormItem>
            </Col>
            <Col span={20}>
              <FormItem name="关联其他产品">
                <TagSelect
                  dict={formatGroupDict(linkProductsDict, 'branchName')}
                  value={linkProducts}
                  onChange={(val) => this.setState({ linkProducts: val })}
                  disabled={partEdit}
                />
              </FormItem>
            </Col>
            <Col span={4}>
              <FormItem labelWidth={0} style={{ paddingLeft: 0, paddingRight: 5, marginLeft: -30 }}>
                <Checkbox
                  checked={editable}
                  onChange={(e) => this.setState({ editable: e.target.checked })}
                  disabled={partEdit}
                >
                  关联产品允许编辑
                </Checkbox>
              </FormItem>
            </Col>
            <Col span={24}>
              <FormItem name="备注" style={{ marginTop: 20, lineHeight: '36px' }}>
                <TextArea
                  rows={3}
                  value={remark}
                  onChange={(e) => this.setState({ remark: cutText(e.target.value, 50) })}
                  disabled={partEdit}
                />
              </FormItem>
            </Col>
          </Row>
        </Block>
        {selectBookModalVisible && (
          <SelectBookModal
            onSelect={this.classBookSelect}
            onClose={() => this.setState({ selectBookModalVisible: false })}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    params: state.main.pageParams['/ProductMiniClass/Edit'],
  };
};

export default connect(mapStateToProps)(EditPage);
