import React from 'react';
import { browserHistory } from 'dva/router';
import { connect } from 'dva';
import Icon from 'COMPONENT/Icon';
import Input from 'COMPONENT/Input';
import Button from 'COMPONENT/Button';
import Page from 'COMPONENT/Page';
import Modal from 'COMPONENT/Modal';
import SearchBar from 'COMPONENT/SearchBar';
import List from 'COMPONENT/List';
import Select from 'COMPONENT/Select';
import DatePicker from 'COMPONENT/DatePicker';
import Popover from 'COMPONENT/Popover';
import { getDictTextByVal, loadCommonDict, loadBranchDict, loadAllBranchDict } from 'UTIL/dict';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import { getProductPageList, cancelProduct } from 'SERVICE/product';
import { statusDict } from 'CONST/dicts/product';
import ProductDetailViewModal from './ModalDetailView';
import styles from 'PAGE/product/listPage.less';

const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;
const productTypeVal = 'SMALL_CLASS';

class MiniClass extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 内容加载状态
      searchExpand: false, // {boolean} 搜索栏展开状态

      productName: null, // {string} 查询条件 产品名称
      branch: null, // {string} 查询条件 所属分公司
      quarter: null, // {string} 查询条件 季度
      grade: null, // {string} 查询条件 年级
      subject: null, // {string} 查询条件 科目
      serise: null, // {string} 查询条件 产品系列
      classType: null, // {string} 查询条件 班次（分层）
      feeType: null, // {string} 查询条件 类型
      classLenType: null, // {string} 查询条件 班型
      system: null, // {string} 查询条件 体系
      startDate: null, // {string} 查询条件 上架日期
      endDate: null, // {string} 查询条件 下架日期
      feeRule: null, // {string} 查询条件 扣费规则
      status: null, // {string} 查询条件 状态

      branchDict: null, // {[object]} 字典 所属分公司
      quarterDict: null, // {[object]} 字典 季度
      gradeDict: null, // {[object]} 字典 年级
      subjectDict: null, // {[object]} 字典 科目
      seriseDict: null, // {[object]} 字典 产品系列
      classTypeDict: null, // {[object]} 字典 班次（分层）
      feeTypeDict: null, // {[object]} 字典 类型
      classLenTypeDict: null, // {[object]} 字典 班型
      systemDict: null, // {[object]} 字典 产品体系
      feeRuleDict: null, // {[object]} 字典 扣费规则

      list: [], // {Array} 列表数据
      listLoading: false, // {false} 列表数据加载状态
      listSearchKeys: null, // {object} 列表数据的查询关键字
      totalRecords: 0, // {number} 总记录数
      currentPage: 1, // {number} 当前页码
      pageSize: 20, // {number} 每页记录数
      viewModalVisible: false, // {boolean} 查看弹框显示状态
      currentRecord: null, // {object} 当前激活状态的列表行数据
      allBranchDict: null, // {[object]} 字典 所有分公司
      checking: false, // {boolean} 是否处于选择状态
      checkedProductKeys: null, // {[string]} 选中的产品Key
    };
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化内容
   */
  initContent = () => {
    this.setState({ loading: true });
    Promise.all([
      loadBranchDict(),
      loadCommonDict('PRODUCT_QUARTER'),
      loadCommonDict('STUDENT_GRADE'),
      loadCommonDict('SUBJECT'),
      loadCommonDict('COURSE_SERIES'),
      loadCommonDict('CLASS_TYPE'),
      loadCommonDict('PRODUCT_FEE_TYPE'),
      loadCommonDict('PRODUCT_CLASS_MODAL'),
      loadCommonDict('PRODUCT_SYSTEM'),
      loadCommonDict('PRODUCT_CHARGE_RULE'),
      loadAllBranchDict(),
    ]).then(
      ([
        branchDict,
        quarterDict,
        gradeDict,
        subjectDict,
        seriseDict,
        classTypeDict,
        feeTypeDict,
        classLenTypeDict,
        systemDict,
        feeRuleDict,
        allBranchDict,
      ]) => {
        this.setState({
          loading: false,
          branchDict,
          quarterDict,
          gradeDict,
          subjectDict,
          seriseDict,
          classTypeDict,
          feeTypeDict,
          classLenTypeDict,
          systemDict,
          feeRuleDict,
          allBranchDict,
        });
      },
    );
    this.search();
  };

  /**
   * 加载列表数据，并转换格式
   * @param  {object} changedParams 要更新的列表数据参数，包括查询条件、页码状态
   */
  loadList = async (changedParams = {}) => {
    const {
      productName,
      branch,
      quarter,
      grade,
      subject,
      serise,
      classType,
      feeType,
      classLenType,
      system,
      startDate,
      endDate,
      feeRule,
      status,
      currentPage,
      pageSize,
      listSearchKeys,
    } = this.state;
    const searchKeys = listSearchKeys || {
      productName,
      branch,
      quarter,
      grade,
      subject,
      serise,
      classType,
      feeType,
      classLenType,
      system,
      endDate,
      feeRule,
      status,
    };
    const params = { currentPage, pageSize, ...searchKeys, ...changedParams };
    const apiParams = {
      type: productTypeVal,
      page: params.currentPage,
      rows: params.pageSize,
      name: params.productName,
      brenchId: params.branch,
      seasonId: params.quarter,
      gradeId: params.grade,
      subjectIds: params.subject ? [params.subject].join(',') : '',
      courseSeriesId: params.serise,
      classTypeId: params.classType,
      feeType: params.feeType,
      classModal: params.classLenType,
      productSystem: params.system,
      startDate: params.startDate,
      endDate: params.endDate,
      chargeRuleId: params.feeRule,
      proStatus: params.status,
    };
    console.log(apiParams, JSON.stringify(apiParams));
    this.setState({ listLoading: true });
    const apiData = await getProductPageList(apiParams);
    const { records, rows } = apiData;
    const formatedRows = rows.map((row) => {
      return {
        productId: row.id,
        productName: row.name,
        branchNames: row.organizationName,
        quarterName: row.productQuarterName,
        gradeName: row.gradeName,
        subjectName: row.subjectName,
        seriesName: row.courseSeriesName,
        classTypeName: row.classTypeName,
        feeTypeName: row.feeTypeName,
        classLenTypeName: row.classModalName,
        systemName: row.productSystemName,
        price: row.price,
        totalHours: row.miniClassTotalhours,
        totalMoney:
          !isNaN(parseFloat(row.price)) && !isNaN(parseFloat(row.miniClassTotalhours))
            ? row.price * row.miniClassTotalhours
            : null,
        startDate: row.validStartDate,
        endDate: row.validEndDate,
        sellCount: 0,
        feeRuleName: row.chargeRuleName,
        statusVal: row.proStatus,
      };
    });
    const data = {
      records,
      rows: formatedRows,
    };
    console.log(data);
    const { currentPage: newCurrentPage, pageSize: newPageSize, ...newListSearchKeys } = params;
    this.setState({
      listLoading: false,
      list: data.rows,
      totalRecords: data.records,
      currentPage: newCurrentPage,
      pageSize: newPageSize,
      listSearchKeys: newListSearchKeys,
    });
    return true;
  };

  /**
   * 查询
   */
  search = () => {
    const {
      productName,
      branch,
      quarter,
      grade,
      subject,
      serise,
      classType,
      feeType,
      classLenType,
      system,
      startDate,
      endDate,
      feeRule,
      status,
    } = this.state;
    this.loadList({
      productName,
      branch,
      quarter,
      grade,
      subject,
      serise,
      classType,
      feeType,
      classLenType,
      system,
      startDate,
      endDate,
      feeRule,
      status,
      currentPage: 1,
    });
  };

  /**
   * 重置查询条件
   */
  reset = () => {
    this.setState({
      productName: null,
      branch: null,
      quarter: null,
      grade: null,
      subject: null,
      serise: null,
      classType: null,
      feeType: null,
      classLenType: null,
      system: null,
      startDate: null,
      endDate: null,
      feeRule: null,
      status: null,
    });
  };

  /**
   * 删除或无效产品
   * @param  {string} productId 产品ID
   * @param  {string} type      操作类型 del 删除 cancel 无效 active 有效
   */
  delete = (productId, type) => {
    if (type === 'del') {
      showConfirm(`确定删除该产品吗？`, () => {
        cancelProduct({ productId, oper: type }).then(() => {
          showMessage('删除成功', 'success');
          this.loadList({ currentPage: 1 });
        });
      });
    } else {
      cancelProduct({ productId, oper: type }).then(() => {
        showMessage('操作成功', 'success');
        this.loadList({ currentPage: 1 });
      });
    }
  };

  /**
   * 列表操作按钮点击处理
   * @param  {string} type 操作类型
   * @param  {object} record 列表行数据
   * @param  {number} index 列表行序号
   */
  listBtnClick = (type, record, index) => {
    const { productId } = record;
    if (type === 'view') {
      this.setState({ viewModalVisible: true, currentRecord: record });
    } else if (type === 'copy') {
      this.setState({ checkedProductKeys: [index] }, () => {
        this.goCopyEditPage();
      });
    }
  };

  /**
   * 编辑产品保存成功后的处理
   */
  editSaveSuccess = () => {
    const { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: { path: '/ProductMiniClass' },
    });
    this.loadList({ currentPage: 1 });
  };

  /**
   * 进入编辑产品页面
   * @param  {string?} productId 产品ID，新增时不传
   */
  edit = (productId) => {
    const { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: {
        path: '/ProductMiniClass/Edit',
        params: {
          productId: productId || null,
          onSaveSuccess: this.editSaveSuccess,
        },
        refresh: true,
      },
    });
  };

  /**
   * 取消选择
   */
  cancelCheck = () => {
    this.setState({ checking: false, checkedProductKeys: null });
  };

  /**
   * 进入复制编辑页面
   */
  goCopyEditPage = () => {
    const { dispatch } = this.props;
    const { list, checkedProductKeys } = this.state;
    this.cancelCheck();
    dispatch({
      type: 'main/openPage',
      payload: {
        path: '/ProductMiniClass/CopyEdit',
        params: {
          productIds: checkedProductKeys.map((key) => list[key].productId),
          onSaveSuccess: this.editSaveSuccess,
        },
        refresh: true,
      },
    });
  };

  render() {
    const { btnAuths } = this.props;
    const {
      loading,
      searchExpand,
      productName,
      branch,
      quarter,
      grade,
      subject,
      serise,
      classType,
      feeType,
      classLenType,
      system,
      startDate,
      endDate,
      feeRule,
      status,
      branchDict,
      quarterDict,
      gradeDict,
      subjectDict,
      seriseDict,
      classTypeDict,
      feeTypeDict,
      classLenTypeDict,
      systemDict,
      feeRuleDict,
      list,
      totalRecords,
      currentPage,
      pageSize,
      listLoading,
      viewModalVisible,
      currentRecord,
      allBranchDict,
      checking,
      checkedProductKeys,
    } = this.state;
    const columns = [
      { title: '产品名称', dataIndex: 'productName', width: COL_WIDTH_XL },
      { title: '所属分公司', dataIndex: 'branchNames', width: COL_WIDTH_M, lines: 2 },
      {
        title: '年级 / 产品系列',
        key: 'gradeSeries',
        width: COL_WIDTH_L,
        render: (text, record) => {
          const { gradeName, seriesName } = record;
          let strs = [];
          if (gradeName) {
            strs.push(gradeName);
          }
          if (seriesName) {
            strs.push(seriesName);
          }
          return strs.join(' · ');
        },
      },
      { title: '总课时数', dataIndex: 'totalHours', width: COL_WIDTH_M },
      { title: '产品单价', dataIndex: 'price', width: COL_WIDTH_M, format: 'money' },
      { title: '总价', dataIndex: 'totalMoney', width: COL_WIDTH_M, format: 'money' },
      { title: '季度', dataIndex: 'quarterName', width: COL_WIDTH_S },
      { title: '科目', dataIndex: 'subjectName', width: COL_WIDTH_M, lines: 2 },
      { title: '班次（分层）', dataIndex: 'classTypeName', width: COL_WIDTH_M },
      { title: '类型', dataIndex: 'feeTypeName', width: COL_WIDTH_M },
      { title: '班型', dataIndex: 'classLenTypeName', width: COL_WIDTH_M },
      { title: '体系', dataIndex: 'systemName', width: COL_WIDTH_M },
      {
        title: '状态',
        key: 'statusName',
        width: COL_WIDTH_S,
        render: (text, record) => getDictTextByVal(statusDict, record.statusVal),
      },
      { title: '销售数量', dataIndex: 'sellCount', width: COL_WIDTH_M },
      { title: '扣费规则', dataIndex: 'feeRuleName', width: COL_WIDTH_M },
      {
        title: '上 / 下架日期',
        key: 'dateRange',
        width: COL_WIDTH_L,
        render: (text, record) => {
          return (
            <div>
              <div>{`起：${record.startDate}`}</div>
              <div>{`止：${record.endDate}`}</div>
            </div>
          );
        },
      },
      {
        title: '操作',
        key: 'action',
        width: COL_WIDTH_XL,
        fixed: 'right',
        getActionConfig: (text, record, index) => {
          const { sellCount, productId } = record;
          const isInvalid = record.statusVal === '1';
          return [
            { text: '查看', onClick: () => this.listBtnClick('view', record, index) },
            { text: '复制', onClick: () => this.listBtnClick('copy', record, index, name), hide: isInvalid },
            {
              menus: [
                { text: '修改', onClick: () => this.edit(productId, sellCount), hide: isInvalid },
                { text: '无效', onClick: () => this.delete(productId, 'cancel'), hide: isInvalid },
                { text: '有效', onClick: () => this.delete(productId, 'active'), hide: !isInvalid },
                { text: '删除', onClick: () => this.delete(productId, 'del'), disabled: sellCount > 0 },
              ],
            },
          ];
        },
      },
    ];

    return (
      <Page
        fitWindowHeight={true}
        loading={loading}
        route={this.props.route}
        subHeader={
          <div>
            <a className={styles.addBtn} onClick={() => this.edit()}>
              <Icon type="plus" size={24} />
            </a>
            <SearchBar
              expand={searchExpand}
              onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
              onSearch={this.search}
              onReset={this.reset}
            >
              <Input
                placeholder="产品名称"
                value={productName}
                onChange={(e) => this.setState({ productName: e.target.value })}
              />
              <Select
                placeholder="所属分公司"
                dict={branchDict}
                value={branch}
                onChange={(val) => this.setState({ branch: val })}
              />
              <Select
                placeholder="季度"
                dict={quarterDict}
                value={quarter}
                onChange={(val) => this.setState({ quarter: val })}
              />
              <Select
                placeholder="年级"
                dict={gradeDict}
                value={grade}
                onChange={(val) => this.setState({ grade: val })}
              />
              <Select
                placeholder="科目"
                dict={subjectDict}
                value={subject}
                onChange={(val) => this.setState({ subject: val })}
              />
              <Select
                placeholder="产品系列"
                dict={seriseDict}
                value={serise}
                onChange={(val) => this.setState({ serise: val })}
              />
              <Select
                placeholder="班次（分层）"
                dict={classTypeDict}
                value={classType}
                onChange={(val) => this.setState({ classType: val })}
              />
              <Select
                placeholder="类型"
                dict={feeTypeDict}
                value={feeType}
                onChange={(val) => this.setState({ feeType: val })}
              />
              <Select
                placeholder="班型"
                dict={classLenTypeDict}
                value={classLenType}
                onChange={(val) => this.setState({ classLenType: val })}
              />
              <Select
                placeholder="体系"
                dict={systemDict}
                value={system}
                onChange={(val) => this.setState({ system: val })}
              />
              <DatePicker
                placeholder="上架日期"
                value={startDate}
                onChange={(val) => this.setState({ startDate: val })}
              />
              <DatePicker placeholder="下架日期" value={endDate} onChange={(val) => this.setState({ endDate: val })} />
              <Select
                placeholder="扣费规则"
                dict={feeRuleDict}
                value={feeRule}
                onChange={(val) => this.setState({ feeRule: val })}
              />
              <Select
                placeholder="状态"
                dict={statusDict}
                value={status}
                onChange={(val) => this.setState({ status: val })}
              />
            </SearchBar>
          </div>
        }
      >
        <List
          dataSource={list}
          columns={columns}
          loading={!loading && listLoading}
          pagination={{
            current: currentPage,
            total: totalRecords,
            pageSize,
          }}
          onChange={({ current, pageSize }) => this.loadList({ currentPage: current, pageSize })}
          rowSelection={
            checking
              ? {
                  selectedRowKeys: checkedProductKeys,
                  onChange: (keys) => this.setState({ checkedProductKeys: keys }),
                }
              : null
          }
        />
        <div className={styles.bottomBtns}>
          {checking ? (
            <div>
              <Button type="primary" onClick={this.goCopyEditPage}>
                确认复制
              </Button>
              <Button type="primary" ghost onClick={this.cancelCheck} style={{ marginLeft: 10 }}>
                取消
              </Button>
            </div>
          ) : (
            <Button type="primary" ghost onClick={() => this.setState({ checking: true })}>
              批量复制
            </Button>
          )}
        </div>
        {viewModalVisible && (
          <ProductDetailViewModal
            productId={currentRecord.productId}
            allBranchDict={allBranchDict}
            subjectDict={subjectDict}
            onClose={() => this.setState({ viewModalVisible: false })}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    btnAuths: state.account.btnAuths,
  };
};

export default connect(mapStateToProps)(MiniClass);
