import React from 'react';
import { connect } from 'dva';
import Icon from 'COMPONENT/Icon';
import Input from 'COMPONENT/Input';
import Button from 'COMPONENT/Button';
import Page from 'COMPONENT/Page';
import Modal from 'COMPONENT/Modal';
import SearchBar from 'COMPONENT/SearchBar';
import List from 'COMPONENT/List';
import Select from 'COMPONENT/Select';
import DatePicker from 'COMPONENT/DatePicker';
import { getDictTextByVal, loadCommonDict, loadBranchDict, loadAllBranchDict } from 'UTIL/dict';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { cancelProduct, getProductPageList } from 'SERVICE/product';
import { o2mTypeDict, statusDict } from 'CONST/dicts/product';
import ProductDetailViewModal from './ProductDetailView';
import styles from 'PAGE/product/listPage.less';
import LongCell from 'COMPONENT/List/LongCell';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';

const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;

const productType = 'ONE_ON_MANY';

let FEEL_CLASS_DICT = [
  { val: 'DAT000000000465', text: '体验课9课时' },
  {
    val: 'DAT000000000466',
    text: '体验课6课时',
  },
  { val: 'DAT000000000467', text: '不允许体验课' },
];

class OneToMany extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 内容加载状态
      searchExpand: false, // {boolean} 搜索栏展开状态

      productName: null, // {string} 查询条件 产品名称
      branch: null, // {string} 查询条件 所属分公司
      grade: null, // {string} 查询条件 年级
      productGroup: null, // {string} 查询条件 产品组
      startDate: null, // {string} 查询条件 上架日期
      endDate: null, // {string} 查询条件 下架日期
      status: null, // {string} 查询条件 状态
      feeRule: null, // {string} 查询条件 扣费规则
      price: null, // {string} 查询条件 单价
      classTimeLength: null, // {string} 查询条件 课时时长
      branchDict: null, // {[object]} 字典 所属分公司
      gradeDict: null, // {[object]} 字典 年级
      productGroupDict: null, // {[object]} 字典 产品组
      statusDict: null, // {[object]} 字典 状态
      list: [], // {Array} 列表数据
      listLoading: false, // {false} 列表数据加载状态
      listSearchKeys: null, // {object} 列表数据的查询关键字
      totalRecords: 0, // {number} 总记录数
      currentPage: 1, // {number} 当前页码
      pageSize: 20, // {number} 每页记录数
      viewModalVisible: false, // {boolean} 查看弹框显示状态
      currentRecord: null, // {object} 当前激活状态的列表行数据
      allBranchDict: null, // {[object]} 字典 所有分公司
      subjectDict: null, // {[object]} 字典 科目
      checking: false, // {boolean} 是否处于选择状态
      checkedProductKeys: null, // {[string]} 选中的产品Key
      productType: productType, //产品类型 'ONE_ON_MANY'
      oneOnManyType: null,
      experienceLessonName: null,
      experienceLessonId: null,
      name: null,
    };
  }

  componentDidMount() {
    this.initData();
  }

  componentDidUpdate(prevProps, prevState) {
    if (prevState.searchExpand !== this.state.searchExpand) {
      this._list.refreshScrollV();
    }
  }
  /**
   * 初始化内容
   */
  initData() {
    //搜索动画
    this.setState({ loading: true });
    Promise.all([
      //所属分公司
      loadBranchDict(),
      //年级
      loadCommonDict('STUDENT_GRADE'),
      //产品组
      loadCommonDict('PRODUCT_GROUP'),

      loadAllBranchDict(),
      loadCommonDict('SUBJECT'),
      loadCommonDict('PRODUCT_CHARGE_RULE'),
    ]).then(([branchDict, gradeDict, productGroupDict, allBranchDict, subjectDict, feeRuleDict]) => {
      this.setState({
        loading: false,
        branchDict,
        gradeDict,
        productGroupDict,
        allBranchDict,
        subjectDict,
        feeRuleDict,
      });
    });
    //请求列表数据
    this.search();
  }

  /**
   * 查询
   */
  search = () => {
    //初始加载的条件
    const {
      productName,
      branch,
      grade,
      price,
      classTimeLength,
      feeRule,
      oneOnManyType,
      productGroup,
      startDate,
      endDate,
      experienceLessonId,
      status,
      productType,
    } = this.state;
    this.loadList({
      productName,
      branch,
      grade,
      price,
      classTimeLength,
      feeRule,
      oneOnManyType,
      productGroup,
      startDate,
      endDate,
      experienceLessonId,
      status,
      currentPage: 1,
      productType,
    });
  };
  /**
   * 重置查询条件
   */
  reset = () => {
    this.setState({
      productName: null,
      branch: null,
      grade: null,
      productGroup: null,
      startDate: null,
      endDate: null,
      status: null,
      price: null,
      classTimeLength: null,
      feeRule: null,
      oneOnManyType: null,
      experienceLessonId: null,
    });
  };
  //删除或无效确认弹出框
  showConfirm = (content, onOk) => {
    Modal.confirm({
      content,
      onOk,
      iconType: 'exclamation-circle',
    });
  };

  /**
   * 列表操作按钮点击处理
   * @param  {string} type 操作类型
   * @param  {object} record 列表行数据
   * @param  {number} index 列表行序号
   */
  listBtnClick = (type, record, index, name) => {
    const { productId } = record;
    if (type === 'view') {
      this.setState({ viewModalVisible: true, currentRecord: record, name });
    } else if (type === 'copy') {
      this.setState({ checkedProductKeys: [index] }, () => {
        this.goCopyEditPage();
      });
    }
  };
  /**
   * 取消选择
   */
  cancelCheck = () => {
    this.setState({ checking: false, checkedProductKeys: null });
  };

  /**
   * 进入复制编辑页面
   */
  goCopyEditPage = () => {
    const { dispatch } = this.props;
    const { list, checkedProductKeys } = this.state;
    this.cancelCheck();
    dispatch({
      type: 'main/openPage',
      payload: {
        path: '/ProductOneToMany/CopyEdit',
        params: {
          //获取所选择需要复制的产品id
          productIds: checkedProductKeys.map((key) => list[key].productId),
          onSaveSuccess: this.editSaveSuccess,
        },
        refresh: true,
      },
    });
  };

  /**
   * 进入编辑产品页面
   * @param  {string?} productId 产品ID，新增时不传
   */
  edit = (productId, sellCount) => {
    let { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: {
        path: '/ProductOneToMany/Edit',
        params: {
          productId: productId || null,
          sellCount: sellCount || 0,
          onSaveSuccess: this.editSaveSuccess,
        },
        refresh: true,
      },
    });
  };

  /**
   * 删除或无效产品
   * @param  {string} productId 产品ID
   * @param  {string} type      操作类型 del 删除 cancel 无效 active 有效
   */
  delete = (productId, type) => {
    if (type === 'del') {
      showConfirm(`确定删除该产品吗？`, () => {
        cancelProduct({ productId, oper: type }).then(() => {
          showMessage('删除成功', 'success');
          this.loadList({ currentPage: 1 });
        });
      });
    } else {
      cancelProduct({ productId, oper: type }).then(() => {
        showMessage('操作成功', 'success');
        this.loadList({ currentPage: 1 });
      });
    }
  };

  /**
   * 编辑产品保存成功后的处理
   */
  editSaveSuccess = () => {
    const { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: { path: '/ProductOneToMany' },
    });
    this.loadList({ currentPage: 1 });
  };

  /**
   * 加载列表数据
   * @param {object} changedParams  筛选条件
   * @returns {Promise.<boolean>}
   */
  async loadList(changedParams = {}) {
    const {
      productName,
      branch,
      grade,
      price,
      classTimeLength,
      feeRule,
      oneOnManyType,
      productGroup,
      startDate,
      endDate,
      experienceLessonId,
      status,
      currentPage,
      pageSize,
      listSearchKeys,
      productType,
    } = this.state;
    const searchKeys = listSearchKeys || {
      productName,
      branch,
      grade,
      price,
      classTimeLength,
      feeRule,
      oneOnManyType,
      productGroup,
      startDate,
      endDate,
      experienceLessonId,
      status,
    };
    const params = { productType, currentPage, pageSize, ...searchKeys, ...changedParams };
    const apiParams = {
      page: params.currentPage,
      rows: params.pageSize,
      name: params.productName,
      brenchId: params.branch,
      gradeId: params.grade,
      productGroupId: params.productGroup,
      startDate: params.startDate,
      endDate: params.endDate,
      proStatus: params.status,
      type: params.productType,
      price: params.price,
      classTimeLength: params.classTimeLength,
      chargeRuleId: params.feeRule,
      experienceLessonId: params.experienceLessonId,
    };
    this.setState({ listLoading: true });
    const apiData = await getProductPageList(apiParams);
    const { records, rows } = apiData;
    const formatedRows = rows.map((row) => {
      return {
        productId: row.id,
        productName: row.name,
        branchIds: row.branchId,
        gradeName: row.gradeName,
        seriesName: row.courseSeriesName,
        productPrice: row.price,
        productGroupName: row.productGroupName,
        courseTimeLength: row.classTimeLength,
        startDate: row.validStartDate,
        endDate: row.validEndDate,
        sellCount: null,
        statusVal: row.proStatus,
        branchNames: row.organizationName,
        categoryName: row.categoryName,
        chargeRuleName: row.chargeRuleName,
        oneOnManyType: row.oneOnManyType,
        experienceLessonName: row.experienceLessonName,
        experienceLessonId: row.experienceLessonId,
        productVersionName: row.productVersionName,
      };
    });
    const data = {
      records,
      rows: formatedRows,
    };
    const { currentPage: newCurrentPage, pageSize: newPageSize, ...newListSearchKeys } = params;
    this.setState({
      listLoading: false,
      list: data.rows,
      totalRecords: data.records,
      currentPage: newCurrentPage,
      pageSize: newPageSize,
      listSearchKeys: newListSearchKeys,
    });
    return true;
  }

  /**
   *
   * @returns {XML}
   */
  render() {
    const { btnAuths } = this.props;
    const {
      searchExpand,
      loading,
      productName,
      branch,
      grade,
      productGroup,
      startDate,
      endDate,
      status,
      feeRule,
      price,
      classTimeLength,
      experienceLessonName,
      experienceLessonId,
      oneOnManyType,
      branchDict,
      gradeDict,
      feeRuleDict,
      productGroupDict,
      list,
      totalRecords,
      currentPage,
      pageSize,
      listLoading,
      viewModalVisible,
      currentRecord,
      allBranchDict,
      subjectDict,
      checking,
      checkedProductKeys,
      categoryName,
      name,
    } = this.state;
    const columns = [
      { title: '产品名称', dataIndex: 'productName', width: COL_WIDTH_L },
      {
        title: '所属分公司',
        dataIndex: 'branchNames',
        width: COL_WIDTH_M,
        render: (text) => <LongCell lines={2}>{text}</LongCell>,
      },
      {
        title: '年级 / 类型',
        key: 'gradeSeries',
        width: COL_WIDTH_L,
        render: (text, record) => {
          const { gradeName, oneOnManyType, categoryName } = record;
          let strs = [];
          if (gradeName) {
            strs.push(gradeName);
          }
          if (oneOnManyType) {
            strs.push(getDictTextByVal(o2mTypeDict, '' + oneOnManyType));
          }
          return strs.join(' · ');
        },
      },
      { title: '产品单价', dataIndex: 'productPrice', width: COL_WIDTH_M, format: 'money' },
      { title: '课时时长', dataIndex: 'courseTimeLength', width: COL_WIDTH_M },
      {
        title: '上 / 下架日期',
        key: 'dateRange',
        width: COL_WIDTH_L,
        render: (text, record) => {
          return (
            <div>
              <div>{`起：${record.startDate}`}</div>
              <div>{`止：${record.endDate}`}</div>
            </div>
          );
        },
      },
      { title: '销售数量', dataIndex: 'sellCount', width: COL_WIDTH_M },
      { title: '扣费规则', dataIndex: 'chargeRuleName', width: COL_WIDTH_M },
      { title: '体验课', dataIndex: 'experienceLessonName', width: COL_WIDTH_M },
      {
        title: '状态',
        key: 'statusName',
        width: COL_WIDTH_S,
        render: (text, record) => getDictTextByVal(statusDict, record.statusVal),
      },
      {
        title: '操作',
        key: 'action',
        width: COL_WIDTH_XL,
        fixed: 'right',
        getActionConfig: (text, record, index) => {
          let { productVersionName, gradeName, shortname, oneOnManyType, productId, sellCount, categoryName } = record;
          shortname = shortname ? shortname : '';
          let name = productVersionName + gradeName + getDictTextByVal(o2mTypeDict, '' + oneOnManyType) + shortname;
          const isInvalid = record.statusVal === '1';
          return [
            { text: '查看', onClick: () => this.listBtnClick('view', record, index, name) },
            { text: '复制', onClick: () => this.listBtnClick('copy', record, index, name), hide: isInvalid },
            {
              menus: [
                { text: '修改', onClick: () => this.edit(productId, sellCount), hide: isInvalid },
                { text: '无效', onClick: () => this.delete(productId, 'cancel'), hide: isInvalid },
                { text: '有效', onClick: () => this.delete(productId, 'active'), hide: !isInvalid },
                { text: '删除', onClick: () => this.delete(productId, 'del'), disabled: sellCount > 0 },
              ],
            },
          ];
        },
      },
    ];

    return (
      <Page
        fitWindowHeight={true}
        footer={false}
        loading={loading}
        route={this.props.route}
        subHeader={
          <div>
            <a className={styles.addBtn} onClick={() => this.edit()}>
              <Icon type="plus" size={24} />
            </a>
            <SearchBar
              onSearch={this.search}
              onReset={this.reset}
              onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
              expand={searchExpand}
            >
              <Input
                placeholder="产品名称"
                value={productName}
                onChange={(e) => this.setState({ productName: e.target.value })}
              />
              <Select
                placeholder="所属分公司"
                dict={branchDict}
                value={branch}
                onChange={(val) => this.setState({ branch: val })}
              />
              <Select
                placeholder="一对多类型"
                dict={o2mTypeDict}
                value={oneOnManyType}
                onChange={(val) => this.setState({ oneOnManyType: val })}
              />
              <Select
                placeholder="年级"
                dict={gradeDict}
                value={grade}
                onChange={(val) => this.setState({ grade: val })}
              />
              <Input placeholder="单价" value={price} onChange={(e) => this.setState({ price: e.target.value })} />
              <Input
                placeholder="课时时长"
                value={classTimeLength}
                onChange={(e) => this.setState({ classTimeLength: e.target.value })}
              />
              <DatePicker
                placeholder="上架日期"
                value={startDate}
                onChange={(val) => this.setState({ startDate: val })}
              />
              <DatePicker placeholder="下架日期" value={endDate} onChange={(val) => this.setState({ endDate: val })} />
              <Select
                placeholder="体验课"
                dict={FEEL_CLASS_DICT}
                value={experienceLessonId}
                onChange={(val) => this.setState({ experienceLessonId: val })}
              />
              <Select
                placeholder="状态"
                dict={statusDict}
                value={status}
                onChange={(val) => this.setState({ status: val })}
              />
              <Select
                placeholder="扣费规则"
                dict={feeRuleDict}
                value={feeRule}
                onChange={(val) => this.setState({ feeRule: val })}
              />
            </SearchBar>
          </div>
        }
      >
        <List
          dataSource={list}
          columns={columns}
          loading={!loading && listLoading}
          pagination={{
            current: currentPage,
            total: totalRecords,
            pageSize,
          }}
          onChange={({ current, pageSize }) => this.loadList({ currentPage: current, pageSize })}
          rowSelection={
            checking
              ? {
                  selectedRowKeys: checkedProductKeys,
                  onChange: (keys) => this.setState({ checkedProductKeys: keys }),
                }
              : null
          }
          ref={(c) => (this._list = c)}
        />
        <div className={styles.bottomBtns}>
          {checking ? (
            <div>
              <Button type="primary" onClick={this.goCopyEditPage}>
                确认复制
              </Button>
              <Button type="primary" ghost onClick={this.cancelCheck} style={{ marginLeft: 10 }}>
                取消
              </Button>
            </div>
          ) : (
            <Button type="primary" ghost onClick={() => this.setState({ checking: true })}>
              批量复制
            </Button>
          )}
        </div>
        {viewModalVisible && (
          <ProductDetailViewModal
            name={name}
            productId={currentRecord.productId}
            allBranchDict={allBranchDict}
            subjectDict={subjectDict}
            onClose={() => this.setState({ viewModalVisible: false })}
          />
        )}
      </Page>
    );
  }
}

export default connect()(OneToMany);
