import React from 'react';
import { browserHistory } from 'dva/router';
import { connect } from 'dva';
import Icon from 'COMPONENT/Icon';
import Input from 'COMPONENT/Input';
import Button from 'COMPONENT/Button';
import InputNumber from 'COMPONENT/InputNumber';
import Page from 'COMPONENT/Page';
import Modal from 'COMPONENT/Modal';
import SearchBar from 'COMPONENT/SearchBar';
import List from 'COMPONENT/List';
import Select from 'COMPONENT/Select';
import DatePicker from 'COMPONENT/DatePicker';
import Popover from 'COMPONENT/Popover';
import { getDictTextByVal, loadCommonDict, loadBranchDict, loadAllBranchDict } from 'UTIL/dict';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import { getProductPageList, cancelProduct } from 'SERVICE/product';
import { statusDict } from 'CONST/dicts/product';
import ProductDetailViewModal from './ModalDetailView';
import styles from 'PAGE/product/listPage.less';
const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;

const productTypeVal = 'ONE_ON_ONE_COURSE';

class OneToOne extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 内容加载状态
      searchExpand: false, // {boolean} 搜索栏展开状态

      productName: null, // {string} 查询条件 产品名称
      branch: null, // {string} 查询条件 所属分公司
      grade: null, // {string} 查询条件 年级
      classTimeLength: null, // {number} 查询条件 课时时长
      price: null, // {number} 查询条件 单价
      serise: null, // {string} 查询条件 产品系列
      productGroup: null, // {string} 查询条件 产品组
      startDate: null, // {string} 查询条件 上架日期
      endDate: null, // {string} 查询条件 下架日期
      feeRule: null, // {string} 查询条件 扣费规则
      status: null, // {string} 查询条件 状态

      branchDict: null, // {[object]} 字典 所属分公司
      gradeDict: null, // {[object]} 字典 年级
      seriseDict: null, // {[object]} 字典 产品系列
      productGroupDict: null, // {[object]} 字典 产品组
      feeRuleDict: null, // {[object]} 字典 扣费规则
      statusDict: null, // {[object]} 字典 状态

      list: [], // {Array} 列表数据
      listLoading: false, // {false} 列表数据加载状态
      listSearchKeys: null, // {object} 列表数据的查询关键字
      totalRecords: 0, // {number} 总记录数
      currentPage: 1, // {number} 当前页码
      pageSize: 20, // {number} 每页记录数
      viewModalVisible: false, // {boolean} 查看弹框显示状态
      currentRecord: null, // {object} 当前激活状态的列表行数据
      allBranchDict: null, // {[object]} 字典 所有分公司
      subjectDict: null, // {[object]} 字典 科目
      checking: false, // {boolean} 是否处于选择状态
      checkedProductKeys: null, // {[string]} 选中的产品Key
    };
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化内容
   */
  initContent = () => {
    this.setState({ loading: true });
    Promise.all([
      loadBranchDict(),
      loadCommonDict('STUDENT_GRADE'),
      loadCommonDict('COURSE_SERIES'),
      loadCommonDict('PRODUCT_GROUP'),
      loadCommonDict('PRODUCT_CHARGE_RULE'),
      loadAllBranchDict(),
      loadCommonDict('SUBJECT'),
    ]).then(([branchDict, gradeDict, seriseDict, productGroupDict, feeRuleDict, allBranchDict, subjectDict]) => {
      this.setState({
        loading: false,
        branchDict,
        gradeDict,
        seriseDict,
        productGroupDict,
        feeRuleDict,
        allBranchDict,
        subjectDict,
      });
    });
    this.search();
  };
  /**
   * 加载列表数据，并转换格式
   * @param  {object} changedParams 要更新的列表数据参数，包括查询条件、页码状态
   */
  loadList = async (changedParams = {}) => {
    const {
      productName,
      branch,
      grade,
      classTimeLength,
      price,
      serise,
      productGroup,
      startDate,
      endDate,
      feeRule,
      status,
      currentPage,
      pageSize,
      listSearchKeys,
    } = this.state;
    const searchKeys = listSearchKeys || {
      productName,
      branch,
      grade,
      classTimeLength,
      price,
      serise,
      productGroup,
      startDate,
      endDate,
      feeRule,
      status,
    };
    const params = { currentPage, pageSize, ...searchKeys, ...changedParams };
    const apiParams = {
      type: productTypeVal,
      page: params.currentPage,
      rows: params.pageSize,
      name: params.productName,
      brenchId: params.branch,
      gradeId: params.grade,
      classTimeLength: params.classTimeLength,
      price: params.price,
      courseSeriesId: params.serise,
      productGroupId: params.productGroup,
      startDate: params.startDate,
      endDate: params.endDate,
      chargeRuleId: params.feeRule,
      proStatus: params.status,
    };
    this.setState({ listLoading: true });
    const apiData = await getProductPageList(apiParams);
    const { records, rows } = apiData;
    const formatedRows = rows.map((row) => {
      return {
        productId: row.id,
        productName: row.name,
        branchIds: row.branchId,
        gradeName: row.gradeName,
        seriesName: row.courseSeriesName,
        productPrice: row.price,
        productGroupName: row.productGroupName,
        courseTimeLength: row.classTimeLength,
        startDate: row.validStartDate,
        endDate: row.validEndDate,
        sellCount: 0,
        statusVal: row.proStatus,
        branchNames: row.organizationName,
        feeRuleName: row.chargeRuleName,
      };
    });
    const data = {
      records,
      rows: formatedRows,
    };
    console.log(data);
    const { currentPage: newCurrentPage, pageSize: newPageSize, ...newListSearchKeys } = params;
    this.setState({
      listLoading: false,
      list: data.rows,
      totalRecords: data.records,
      currentPage: newCurrentPage,
      pageSize: newPageSize,
      listSearchKeys: newListSearchKeys,
    });
    return true;
  };
  /**
   * 查询
   */
  search = () => {
    const {
      productName,
      branch,
      grade,
      classTimeLength,
      price,
      serise,
      productGroup,
      startDate,
      endDate,
      feeRule,
      status,
    } = this.state;
    this.loadList({
      productName,
      branch,
      grade,
      classTimeLength,
      price,
      serise,
      productGroup,
      startDate,
      endDate,
      feeRule,
      status,
      currentPage: 1,
    });
  };
  /**
   * 重置查询条件
   */
  reset = () => {
    this.setState({
      productName: null,
      branch: null,
      grade: null,
      classTimeLength: null,
      price: null,
      serise: null,
      productGroup: null,
      startDate: null,
      endDate: null,
      feeRule: null,
      status: null,
    });
  };

  /**
   * 删除或无效产品
   * @param  {string} productId 产品ID
   * @param  {string} type      操作类型 del 删除 cancel 无效 active 有效
   */
  delete = (productId, type) => {
    if (type === 'del') {
      showConfirm(`确定删除该产品吗？`, () => {
        cancelProduct({ productId, oper: type }).then(() => {
          showMessage('删除成功', 'success');
          this.loadList({ currentPage: 1 });
        });
      });
    } else {
      cancelProduct({ productId, oper: type }).then(() => {
        showMessage('操作成功', 'success');
        this.loadList({ currentPage: 1 });
      });
    }
  };

  /**
   * 列表操作按钮点击处理
   * @param  {string} type 操作类型
   * @param  {object} record 列表行数据
   * @param  {number} index 列表行序号
   */
  listBtnClick = (type, record, index) => {
    const { productId } = record;
    if (type === 'view') {
      this.setState({ viewModalVisible: true, currentRecord: record });
    } else if (type === 'copy') {
      this.setState({ checkedProductKeys: [index] }, () => {
        this.goCopyEditPage();
      });
    }
  };
  /**
   * 编辑产品保存成功后的处理
   */
  editSaveSuccess = () => {
    const { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: { path: '/ProductOneToOne' },
    });
    this.loadList({ currentPage: 1 });
  };
  /**
   * 进入编辑产品页面
   * @param  {string?} productId 产品ID，新增时不传
   */
  edit = (productId) => {
    const { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: {
        path: '/ProductOneToOne/Edit',
        params: {
          productId: productId || null,
          onSaveSuccess: this.editSaveSuccess,
        },
        refresh: true,
      },
    });
  };
  /**
   * 取消选择
   */
  cancelCheck = () => {
    this.setState({ checking: false, checkedProductKeys: null });
  };
  /**
   * 进入复制编辑页面
   */
  goCopyEditPage = () => {
    const { dispatch } = this.props;
    const { list, checkedProductKeys } = this.state;
    this.cancelCheck();
    dispatch({
      type: 'main/openPage',
      payload: {
        path: '/ProductOneToOne/CopyEdit',
        params: {
          productIds: checkedProductKeys.map((key) => list[key].productId),
          onSaveSuccess: this.editSaveSuccess,
        },
        refresh: true,
      },
    });
  };
  render() {
    const { btnAuths } = this.props;
    const {
      loading,
      searchExpand,
      productName,
      branch,
      grade,
      classTimeLength,
      price,
      serise,
      productGroup,
      startDate,
      endDate,
      feeRule,
      status,
      branchDict,
      gradeDict,
      seriseDict,
      productGroupDict,
      feeRuleDict,
      list,
      totalRecords,
      currentPage,
      pageSize,
      listLoading,
      viewModalVisible,
      currentRecord,
      allBranchDict,
      subjectDict,
      checking,
      checkedProductKeys,
    } = this.state;
    const columns = [
      { title: '产品名称', dataIndex: 'productName', width: COL_WIDTH_L },
      { title: '所属分公司', dataIndex: 'branchNames', width: COL_WIDTH_M, lines: 2 },
      {
        title: '年级 / 产品系列',
        key: 'gradeSeries',
        width: COL_WIDTH_L,
        render: (text, record) => {
          const { gradeName, seriesName } = record;
          let strs = [];
          if (gradeName) {
            strs.push(gradeName);
          }
          if (seriesName) {
            strs.push(seriesName);
          }
          return strs.join(' · ');
        },
      },
      { title: '产品单价', dataIndex: 'productPrice', width: COL_WIDTH_M, format: 'money' },
      { title: '产品组', dataIndex: 'productGroupName', width: COL_WIDTH_M },
      { title: '课时时长', dataIndex: 'courseTimeLength', width: COL_WIDTH_M },
      {
        title: '上 / 下架日期',
        key: 'dateRange',
        width: COL_WIDTH_L,
        render: (text, record) => {
          return (
            <div>
              <div>{`起：${record.startDate}`}</div>
              <div>{`止：${record.endDate}`}</div>
            </div>
          );
        },
      },
      { title: '销售数量', dataIndex: 'sellCount', width: COL_WIDTH_M },
      { title: '扣费规则', dataIndex: 'feeRuleName', width: COL_WIDTH_M },
      {
        title: '状态',
        key: 'statusName',
        width: COL_WIDTH_S,
        render: (text, record) => getDictTextByVal(statusDict, record.statusVal),
      },
      {
        title: '操作',
        key: 'action',
        width: COL_WIDTH_XL,
        fixed: 'right',
        getActionConfig: (text, record, index) => {
          const { sellCount, productId } = record;
          const isInvalid = record.statusVal === '1';
          return [
            { text: '查看', onClick: () => this.listBtnClick('view', record, index) },
            { text: '复制', onClick: () => this.listBtnClick('copy', record, index, name), hide: isInvalid },
            {
              menus: [
                { text: '修改', onClick: () => this.edit(productId, sellCount), hide: isInvalid },
                { text: '无效', onClick: () => this.delete(productId, 'cancel'), hide: isInvalid },
                { text: '有效', onClick: () => this.delete(productId, 'active'), hide: !isInvalid },
                { text: '删除', onClick: () => this.delete(productId, 'del'), disabled: sellCount > 0 },
              ],
            },
          ];
        },
      },
    ];
    return (
      <Page
        fitWindowHeight={true}
        loading={loading}
        route={this.props.route}
        subHeader={
          <div>
            <a className={styles.addBtn} onClick={() => this.edit()}>
              <Icon type="plus" size={24} />
            </a>
            <SearchBar
              expand={searchExpand}
              onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
              onSearch={this.search}
              onReset={this.reset}
            >
              <Input
                placeholder="产品名称"
                value={productName}
                onChange={(e) => this.setState({ productName: e.target.value })}
              />
              <Select
                placeholder="所属分公司"
                dict={branchDict}
                value={branch}
                onChange={(val) => this.setState({ branch: val })}
              />
              <Select
                placeholder="年级"
                dict={gradeDict}
                value={grade}
                onChange={(val) => this.setState({ grade: val })}
              />
              <InputNumber
                placeholder="课时时长"
                value={classTimeLength}
                onChange={(val) => this.setState({ classTimeLength: val })}
              />
              <InputNumber placeholder="单价" value={price} onChange={(val) => this.setState({ price: val })} />
              <Select
                placeholder="产品系列"
                dict={seriseDict}
                value={serise}
                onChange={(val) => this.setState({ serise: val })}
              />
              <Select
                placeholder="产品组"
                dict={productGroupDict}
                value={productGroup}
                onChange={(val) => this.setState({ productGroup: val })}
              />
              <DatePicker
                placeholder="上架日期"
                value={startDate}
                onChange={(val) => this.setState({ startDate: val })}
              />
              <DatePicker placeholder="下架日期" value={endDate} onChange={(val) => this.setState({ endDate: val })} />
              <Select
                placeholder="扣费规则"
                dict={feeRuleDict}
                value={feeRule}
                onChange={(val) => this.setState({ feeRule: val })}
              />
              <Select
                placeholder="状态"
                dict={statusDict}
                value={status}
                onChange={(val) => this.setState({ status: val })}
              />
            </SearchBar>
          </div>
        }
      >
        <List
          dataSource={list}
          columns={columns}
          loading={!loading && listLoading}
          pagination={{
            current: currentPage,
            total: totalRecords,
            pageSize,
          }}
          onChange={({ current, pageSize }) => this.loadList({ currentPage: current, pageSize })}
          rowSelection={
            checking
              ? {
                  selectedRowKeys: checkedProductKeys,
                  onChange: (keys) => this.setState({ checkedProductKeys: keys }),
                }
              : null
          }
        />

        <div className={styles.bottomBtns}>
          {checking ? (
            <div>
              <Button type="primary" onClick={this.goCopyEditPage}>
                确认复制
              </Button>
              <Button type="primary" ghost onClick={this.cancelCheck} style={{ marginLeft: 10 }}>
                取消
              </Button>
            </div>
          ) : (
            <Button type="primary" ghost onClick={() => this.setState({ checking: true })}>
              批量复制
            </Button>
          )}
        </div>
        {viewModalVisible && (
          <ProductDetailViewModal
            productId={currentRecord.productId}
            allBranchDict={allBranchDict}
            subjectDict={subjectDict}
            onClose={() => this.setState({ viewModalVisible: false })}
          />
        )}
      </Page>
    );
  }
}
const mapStateToProps = (state) => {
  return {
    btnAuths: state.account.btnAuths,
  };
};
export default connect(mapStateToProps)(OneToOne);
