import React from 'react';
import moment from 'moment';
import { connect } from 'dva';
import { isNumber } from 'lodash';
import { Page, Button } from 'COMPONENT';
import { AddPeriodCharge, EditPeriodCharge, getPeriodChargeById } from 'SERVICE/product';
import AddForm from '../components/AddForm/AddForm';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import LogList from '../components/LogList';
import SelectedProductList from '../components/SelectedList';
import SelectProductModal from '../components/SelectProductModal';
import ProductListDelModal from '../components/DelProductModal';
import { SUMMER_AND_AUTUMN, SINGLE_AUTUMN, SINGLE_SPRING, WINTER_AND_SPRING } from '../const';

import style from './style.module.less';

const OpenUp = 'OpenUp'; // 打开上段
const OpenDown = 'OpenDown'; // 打开下段

class PeriodChargePage extends React.Component {
  constructor(props) {
    super(props);

    this.isEdit = false;
    this.formRef = null;
    const { id } = props.pageParams[`/${props.route.path}`];
    if (id) {
      this.isEdit = true;
      this.id = id;
    }

    this.onSubmit = this.onSubmit.bind(this);
    this.onSave = this.onSave.bind(this);
    this.onUpClick = this.onUpClick.bind(this);
    this.onDownClick = this.onDownClick.bind(this);
    this.onSelectProductChange = this.onSelectProductChange.bind(this);
    this.onSelectProductDel = this.onSelectProductDel.bind(this);
    this.openDelModal = this.openDelModal.bind(this);
    this.handleFieldsChange = this.handleFieldsChange.bind(this);

    this.state = {
      loading: false,
      showSearchModal: false,
      showDelModal: false,
      // 组织id
      organizationId: '',
      // 连报类型
      type: SUMMER_AND_AUTUMN,
      // 产品选择类型
      openType: OpenDown,
      // 上段新增产品
      upAddProducts: [],
      // 下段新增产品
      downAddProducts: [],
      // 删除产品
      delProducts: [],
      // 编辑回填上段产品
      editUpProductIds: [],
      // 编辑回填下段产品
      editDownProductIds: [],
    };
  }

  async componentDidMount() {
    if (this.isEdit) {
      await this.loadDetail();
    }
  }

  async loadDetail() {
    try {
      this.setState({ loading: true });
      const { resultCode, data } = await getPeriodChargeById({ segId: this.id });
      if (resultCode === 0) {
        const {
          organizationId,
          name,
          type,
          upNatureMinutesLimit,
          upStartTime,
          downStartTime,
          upSegProductIds,
          downSegProductIds,
          validStartTime,
          validEndTime,
          upFirstQuarterPromotionScaleLimit,
          upSecondQuarterPromotionScaleLimit,
        } = data;
        const formValues = {
          name,
          type: String(type),
          organizationId,
          upStartTime: moment(upStartTime),
          downStartTime: moment(downStartTime),
        };
        //  单季度，表单没这个字段
        if (
          [SUMMER_AND_AUTUMN, WINTER_AND_SPRING].includes(String(type)) &&
          isNumber(upFirstQuarterPromotionScaleLimit)
        ) {
          formValues.upFirstQuarterPromotionScaleLimit = upFirstQuarterPromotionScaleLimit * 100;
        }
        if (isNumber(upSecondQuarterPromotionScaleLimit)) {
          formValues.upSecondQuarterPromotionScaleLimit = upSecondQuarterPromotionScaleLimit * 100;
        }
        if (isNumber(upNatureMinutesLimit)) {
          formValues.hours = Math.floor(upNatureMinutesLimit / 60);
          formValues.minutes = Math.floor(upNatureMinutesLimit % 60);
        }
        if (validStartTime && validEndTime) {
          formValues.validateTime = [moment(validStartTime), moment(validEndTime)];
        }

        const payload = {
          type: String(type),
          organizationId,
          // api maybe return null
          editUpProductIds: upSegProductIds || [],
          editDownProductIds: downSegProductIds || [],
        };
        this.setState(payload, () => {
          this.formRef.setFieldsValue(formValues);
        });
      }
    } finally {
      this.setState({ loading: false });
    }
  }

  onSave() {
    this.formRef.validateFields((err, values) => {
      if (!err) {
        this.onSubmit(values);
      } else {
        showMessage('请完善好所有信息后再保存！', 'error');
      }
    });
  }

  hasSelectProducts() {
    const isSingleQuarter = this.isSingleQuarter();
    const { upAddProducts, downAddProducts } = this.state;
    return isSingleQuarter ? downAddProducts.length !== 0 : downAddProducts.length !== 0 && upAddProducts.length !== 0;
  }

  getCaseMessage() {
    const isSingleQuarter = this.isSingleQuarter();
    const { upAddProducts, downAddProducts } = this.state;
    return isSingleQuarter
      ? '请选择产品'
      : upAddProducts.length === 0 && downAddProducts.length === 0
      ? '请选择产品'
      : upAddProducts.length === 0
      ? '请选择阶段一产品'
      : '请选择阶段二产品';
  }

  onSubmit(values) {
    const { upStartTime, downStartTime, validateTime, hours, minutes } = values;
    const { upAddProducts, downAddProducts, delProducts, editUpProductIds, editDownProductIds } = this.state;

    const upSegStartTime = upStartTime.startOf('day').valueOf();
    const downSegStartTime = downStartTime.startOf('day').valueOf();
    // 有效期
    const [validStartTime, validEndTime] = validateTime;
    // 小时跟分钟 以分钟单位传给后端
    const upNatureMinutesLimit = Number(hours) * 60 + Number(minutes);
    // 将百分比替换
    if (!values.upFirstQuarterPromotionScaleLimit) {
      // 如果为单季度，则阶段一优惠比例为0
      values.upFirstQuarterPromotionScaleLimit = 0;
    } else {
      values.upFirstQuarterPromotionScaleLimit = Number(values.upFirstQuarterPromotionScaleLimit / 100);
    }
    values.upSecondQuarterPromotionScaleLimit = Number(values.upSecondQuarterPromotionScaleLimit / 100);
    // 去除原有自定义参数
    delete values.hours;
    delete values.minutes;
    delete values.validateTime;

    if (downSegStartTime <= upSegStartTime) {
      return showMessage('阶段二开始收费时间不能早于阶段一开始收费时间', 'error');
    }

    // 产品必选
    if (this.isEdit) {
      const all = [...editUpProductIds, ...editDownProductIds];
      if (/** 已删除完 */ delProducts.length === all.length) {
        if (!this.hasSelectProducts()) {
          return showMessage(this.getCaseMessage(), 'error');
        }
      }
    } else {
      if (!this.hasSelectProducts()) {
        return showMessage(this.getCaseMessage(), 'error');
      }
    }

    const addUpIds = upAddProducts.map((r) => r.id);
    const addDownIds = downAddProducts.map((r) => r.id);

    if (this.isEdit) {
      const deleteProductIds = delProducts.map((r) => r.productId);
      this.handleEdit({
        ...values,
        id: this.id,
        deleteProductIds,
        upNatureMinutesLimit,
        addUpSegProductIds: addUpIds,
        addDownSegProductIds: addDownIds,
        upStartTime: upSegStartTime,
        downStartTime: downSegStartTime,
        validStartTime: validStartTime.valueOf(),
        validEndTime: validEndTime.valueOf(),
      });
    } else {
      this.handleAdd({
        ...values,
        upNatureMinutesLimit,
        upSegProductIds: addUpIds,
        downSegProductIds: addDownIds,
        upStartTime: upSegStartTime,
        downStartTime: downSegStartTime,
        validStartTime: validStartTime.valueOf(),
        validEndTime: validEndTime.valueOf(),
      });
    }
  }

  async handleEdit(params) {
    try {
      this.setState({ loading: true });
      const { resultCode } = await EditPeriodCharge(params);
      if (resultCode === 0) {
        showMessage('修改成功', 'success');
        this.props.dispatch({
          type: 'main/openPage',
          payload: {
            path: '/periodCharge',
            refresh: true,
          },
        });
      }
    } finally {
      this.setState({ loading: false });
    }
  }

  async handleAdd(params) {
    try {
      this.setState({ loading: true });
      const { resultCode } = await AddPeriodCharge(params);
      if (resultCode === 0) {
        showMessage('新增成功', 'success');
        this.props.dispatch({
          type: 'main/openPage',
          payload: {
            path: '/periodCharge',
            refresh: true,
          },
        });
      }
    } finally {
      this.setState({ loading: false });
    }
  }

  /** 上段按钮点击 */
  onUpClick() {
    this.setState({
      showSearchModal: true,
      openType: OpenUp,
    });
  }

  /** 下段按钮点击 */
  onDownClick() {
    this.setState({
      showSearchModal: true,
      openType: OpenDown,
    });
  }

  onSelectProductChange(list = []) {
    const payload = { showSearchModal: false };
    if (list.length) {
      const { openType } = this.state;
      if (openType === OpenDown) {
        payload.downAddProducts = list;
      } else {
        payload.upAddProducts = list;
      }
    }
    this.setState(payload);
  }

  onSelectProductDel(list = []) {
    const payload = { showDelModal: false };
    if (list.length) {
      payload.delProducts = list;
    }
    this.setState(payload);
  }

  getDisableProductIds(isOpenDown) {
    const { upAddProducts, downAddProducts, delProducts, editUpProductIds, editDownProductIds } = this.state;
    const upAddIds = upAddProducts.map((r) => r.id);
    const downAddIds = downAddProducts.map((r) => r.id);
    if (this.isEdit) {
      const delIds = delProducts.map((r) => r.id);
      const upIds = upAddIds.concat(editUpProductIds);
      const downIds = downAddIds.concat(editDownProductIds);
      return (isOpenDown ? upIds : downIds).filter((id) => delIds.indexOf(id) === -1);
    } else {
      return [...upAddIds, ...downAddIds];
    }
  }

  // 获取所选产品总数量
  getProductsCountInfo() {
    const { upAddProducts, downAddProducts } = this.state;
    return {
      upNumber: upAddProducts.length,
      downNumber: downAddProducts.length,
    };
  }

  getSelectedProducts(isOpenDown) {
    const { upAddProducts, downAddProducts } = this.state;
    return isOpenDown ? downAddProducts : upAddProducts;
  }

  openDelModal() {
    this.setState({ showDelModal: true });
  }

  renderDelTipText() {
    const { delProducts } = this.state;
    return delProducts.length > 0 ? (
      <span className={style.tipText}>
        将删除<span className={style.num}>{delProducts.length}</span>个产品
      </span>
    ) : null;
  }

  clearWhenOrgChange(organizationId) {
    this.setState({
      organizationId,
      upAddProducts: [],
      downAddProducts: [],
    });
  }

  clearWhenTypeChange(type) {
    this.setState({
      type,
      upAddProducts: [],
      downAddProducts: [],
    });
  }

  handleFieldsChange(fields) {
    const { organizationId, type } = this.state;
    // 组织改变
    if (fields.organizationId && fields.organizationId.value !== organizationId) {
      this.clearWhenOrgChange(fields.organizationId.value);
    }
    // 类型改变
    if (fields.type && fields.type.value !== type) {
      this.clearWhenTypeChange(fields.type.value);
    }
  }

  // 是否单报
  isSingleQuarter() {
    return [SINGLE_SPRING, SINGLE_AUTUMN].includes(this.state.type);
  }

  // 获取查询产品列表季度关键字
  getQuarterKey(isOpenDown) {
    const { type } = this.state;
    if (this.isSingleQuarter()) {
      return type === SINGLE_SPRING ? '春' : '秋';
    } else {
      return type === SUMMER_AND_AUTUMN ? (isOpenDown ? '秋' : '暑') : isOpenDown ? '春' : '寒';
    }
  }

  render() {
    const { openType, type, organizationId } = this.state;
    const isOpenDown = openType === OpenDown;
    const selectedList = this.getSelectedProducts(isOpenDown);
    const disabledIds = this.getDisableProductIds(isOpenDown);
    const { upNumber, downNumber } = this.getProductsCountInfo();
    const quarterKey = this.getQuarterKey(isOpenDown);

    return (
      <Page
        loading={this.state.loading}
        route={this.props.route}
        footer={
          <div className={style.pageFooter}>
            <Button onClick={this.onSave} type="primary">
              保存
            </Button>
          </div>
        }
        fitWindowHeight={false}
      >
        <AddForm
          isEdit={this.isEdit}
          upClick={this.onUpClick}
          downClick={this.onDownClick}
          upNumber={upNumber}
          downNumber={downNumber}
          organizationId={organizationId}
          type={type}
          onFieldsChange={this.handleFieldsChange}
          ref={(c) => (this.formRef = c)}
        />
        {this.isEdit && (
          <div className={style.contentWrap}>
            <p className={style.title}>已选产品</p>
            <p className={style.operate}>
              <Button size="small" onClick={this.openDelModal}>
                删除产品
              </Button>
              {this.renderDelTipText()}
            </p>
            <SelectedProductList id={this.id} />
            <p className={style.title}>操作历史</p>
            <LogList id={this.id} />
          </div>
        )}
        {this.state.showSearchModal && (
          <SelectProductModal
            quarterKey={quarterKey}
            disabledIds={disabledIds}
            selectedList={selectedList}
            onChange={this.onSelectProductChange}
            organizationId={this.state.organizationId}
          />
        )}
        {this.state.showDelModal && (
          <ProductListDelModal id={this.id} selectedList={this.state.delProducts} onChange={this.onSelectProductDel} />
        )}
      </Page>
    );
  }
}

function mapStateToProps(state) {
  return {
    pageParams: state.main.pageParams,
  };
}

export default connect(mapStateToProps)(PeriodChargePage);
