import { noop } from 'lodash';
import { TextLink } from 'COMPONENT';
import { Col } from 'antd';
import { SUMMER_AND_AUTUMN, WINTER_AND_SPRING, SINGLE_SPRING, SINGLE_AUTUMN } from '../../const';
import style from './style.module.less';

/**
 * @desc 季度内容渲染
 * @param {*} type
 * @param {*} selections
 */

export const QuarterRenderMap = {
  [SUMMER_AND_AUTUMN](actionText, selections = {}, disabled = true, { onClickUp = noop, onClickDown = noop }) {
    const { upNumber = 0, downNumber = 0 } = selections;
    return (
      <Col span={4} className={style.rowContent}>
        <p className={style.row}>
          <span>暑假</span>
          <TextLink onClick={onClickUp} disabled={disabled} className={style.btn} type="primary">
            {actionText}产品
          </TextLink>
          {upNumber > 0 && (
            <span className={style.btn} type="primary">
              已选{upNumber}个
            </span>
          )}
        </p>
        <p className={style.row}>
          <span>秋季</span>
          <TextLink onClick={onClickDown} disabled={disabled} className={style.btn} type="primary">
            {actionText}产品
          </TextLink>
          {downNumber > 0 && (
            <span className={style.btn} type="primary">
              已选{downNumber}个
            </span>
          )}
        </p>
      </Col>
    );
  },
  [WINTER_AND_SPRING](actionText, selections = {}, disabled = true, { onClickUp = noop, onClickDown = noop }) {
    const { upNumber = 0, downNumber = 0 } = selections;
    return (
      <Col span={4} className={style.rowContent}>
        <p className={style.row}>
          <span>寒假</span>
          <TextLink onClick={onClickUp} disabled={disabled} className={style.btn} type="primary">
            {actionText}产品
          </TextLink>
          {upNumber > 0 && (
            <span className={style.btn} type="primary">
              已选{upNumber}个
            </span>
          )}
        </p>
        <p className={style.row}>
          <span>春季</span>
          <TextLink onClick={onClickDown} disabled={disabled} className={style.btn} type="primary">
            {actionText}产品
          </TextLink>
          {downNumber > 0 && (
            <span className={style.btn} type="primary">
              已选{downNumber}个
            </span>
          )}
        </p>
      </Col>
    );
  },
  [SINGLE_SPRING](actionText, selections = {}, disabled = true, { onClickDown = noop }) {
    const { downNumber = 0 } = selections;
    return (
      <Col span={4} className={style.rowContent}>
        <p className={style.row}>
          <span>春季</span>
          <TextLink onClick={onClickDown} disabled={disabled} className={style.btn} type="primary">
            {actionText}产品
          </TextLink>
          {downNumber > 0 && (
            <span className={style.btn} type="primary">
              已选{downNumber}个
            </span>
          )}
        </p>
      </Col>
    );
  },
  [SINGLE_AUTUMN](actionText, selections = {}, disabled = true, { onClickDown = noop }) {
    const { downNumber = 0 } = selections;
    return (
      <Col span={4} className={style.rowContent}>
        <p className={style.row}>
          <span>秋季</span>
          <TextLink onClick={onClickDown} disabled={disabled} className={style.btn} type="primary">
            {actionText}产品
          </TextLink>
          {downNumber > 0 && (
            <span className={style.btn} type="primary">
              已选{downNumber}个
            </span>
          )}
        </p>
      </Col>
    );
  },
};

/**
 * @desc 季度相应文字
 */
export const QuarterTextMap = {
  [SUMMER_AND_AUTUMN]: '秋季',
  [WINTER_AND_SPRING]: '春季',
  [SINGLE_SPRING]: '春季',
  [SINGLE_AUTUMN]: '秋季',
};
