import React from 'react';
import moment from 'moment';
import { List } from 'COMPONENT';
import { getPeriodChargeLog } from 'SERVICE/product';

const { COL_WIDTH_M, COL_WIDTH_XL } = List;

export default class LogList extends React.Component {
  columns = [
    {
      title: '操作人',
      dataIndex: 'createUserName',
      key: 'createUserName',
      width: COL_WIDTH_M,
    },
    {
      title: '操作时间',
      dataIndex: 'createTime',
      key: 'createTime',
      width: COL_WIDTH_XL,
      render: function (val) {
        return val ? moment(val).format('YYYY-MM-DD HH:mm:ss') : null;
      },
    },
    {
      title: '修改内容',
      dataIndex: 'content',
      key: 'content',
    },
  ];

  constructor(props) {
    super(props);
    this.state = {
      dataList: [],
      pagination: {
        total: 0,
        current: 1,
        pageSize: 20,
      },
    };

    this.onPageChange = this.onPageChange.bind(this);
  }

  componentDidMount() {
    this.loadData();
  }

  async loadData() {
    try {
      this.setState({ loading: true });
      const { pagination } = this.state;
      const { resultCode, data } = await getPeriodChargeLog({
        segId: this.props.id,
        pageSize: pagination.pageSize,
        pageNum: pagination.current,
      });
      if (resultCode === 0) {
        const { total, list: dataList } = data;
        this.setState({
          dataList,
          pagination: {
            ...pagination,
            total,
          },
        });
      }
    } finally {
      this.setState({ loading: false });
    }
  }

  onPageChange({ pageSize, current }) {
    const { pagination } = this.state;
    this.setState(
      {
        pagination: {
          ...pagination,
          pageSize,
          current,
        },
      },
      this.loadData,
    );
  }

  render() {
    const { pagination, dataList } = this.state;

    return <List dataSource={dataList} columns={this.columns} pagination={pagination} onChange={this.onPageChange} />;
  }
}
