import React from 'react';
import { List } from 'COMPONENT';
import { getPeriodChargeProductList } from 'SERVICE/product';
import { ProductSegType } from '../../const';

const { COL_WIDTH_M } = List;

export default class SelectedProductList extends React.Component {
  columns = [
    {
      title: '产品名称',
      dataIndex: 'productName',
      key: 'productName',
      width: COL_WIDTH_M,
    },
    {
      title: '分段类型',
      dataIndex: 'segType',
      key: 'segType',
      width: COL_WIDTH_M,
      render: function (type) {
        return type ? ProductSegType[type] : '';
      },
    },
    {
      title: '产品系列',
      dataIndex: 'courseSeriesName',
      key: 'courseSeriesName',
      width: COL_WIDTH_M,
    },
    {
      title: '班级类型',
      dataIndex: 'classTypeName',
      key: 'classTypeName',
      width: COL_WIDTH_M,
    },
    {
      title: '产品季度',
      dataIndex: 'quarterName',
      key: 'quarterName',
      width: COL_WIDTH_M,
    },
    {
      title: '年级',
      dataIndex: 'gradeName',
      key: 'gradeName',
      width: COL_WIDTH_M,
    },
  ];

  constructor(props) {
    super(props);
    this.state = {
      dataList: [],
      pagination: {
        total: 0,
        current: 1,
        pageSize: 20,
      },
    };

    this.onPageChange = this.onPageChange.bind(this);
  }

  componentDidMount() {
    this.loadData();
  }

  async loadData() {
    try {
      this.setState({ loading: true });
      const { pagination } = this.state;
      const { resultCode, data } = await getPeriodChargeProductList({
        segId: this.props.id,
        pageSize: pagination.pageSize,
        pageNum: pagination.current,
      });
      if (resultCode === 0) {
        const { total, list: dataList } = data;
        this.setState({
          dataList,
          pagination: {
            ...pagination,
            total,
          },
        });
      }
    } finally {
      this.setState({ loading: false });
    }
  }

  onPageChange({ pageSize, current }) {
    const { pagination } = this.state;
    this.setState(
      {
        pagination: {
          ...pagination,
          pageSize,
          current,
        },
      },
      this.loadData,
    );
  }

  render() {
    const { pagination, dataList } = this.state;

    return <List dataSource={dataList} columns={this.columns} pagination={pagination} onChange={this.onPageChange} />;
  }
}
