import React from 'react';
import { connect } from 'dva';
import { isNumber } from 'lodash';
import moment from 'moment';
import { Page, List, Button, Input, TextLink, MultiSelect, Select, SearchFilterForm, SearchFieldForm } from 'COMPONENT';
import { Popconfirm } from 'antd';
import { StatusDict, PeriodTypeDict, PeriodTextMap, StatusTextMap } from './const';
import { getPeriodChargeList, inValidPeriodCharge, getUserBranchList } from 'SERVICE/product';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import NP from 'number-precision';

const title = '分段收费';

const { COL_WIDTH_M, COL_WIDTH_XL } = List;

const searchParams = {
  name: '',
  type: '',
  status: '1',
  organizationIds: [],
};

class ProductPeriodCharge extends React.Component {
  columns = [
    {
      title: '分公司',
      dataIndex: 'organizationName',
      key: 'organizationName',
      width: COL_WIDTH_M,
    },
    {
      title: '分段收费名称',
      dataIndex: 'name',
      key: 'name',
      width: COL_WIDTH_M,
      lines: 2,
    },
    {
      title: '连报条件',
      dataIndex: 'type',
      key: 'type',
      width: COL_WIDTH_M,
      render: function (val) {
        return val ? PeriodTextMap.get(String(val)) : '';
      },
    },
    {
      title: '阶段一收费小时',
      dataIndex: 'upNatureMinutesLimit',
      key: 'upNatureMinutesLimit',
      width: COL_WIDTH_XL,
      render: function (val) {
        return val !== null && val !== undefined ? Number(val / 60).toFixed(2) : '';
      },
    },
    {
      title: '阶段一优惠比例',
      width: COL_WIDTH_XL,
      render: function (record) {
        let quarterFirst = '';
        let quarterSecond = '';
        const { type, upFirstQuarterPromotionScaleLimit, upSecondQuarterPromotionScaleLimit } = record;
        switch (String(type)) {
          case '1':
            quarterFirst = '暑假';
            quarterSecond = '秋季';
            break;
          case '2':
            quarterFirst = '寒假';
            quarterSecond = '春季';
            break;
          case '3':
            quarterFirst = '春季';
            break;
          case '4':
            quarterFirst = '秋季';
            break;
        }
        if (isNumber(upFirstQuarterPromotionScaleLimit) && isNumber(upSecondQuarterPromotionScaleLimit)) {
          if (['1', '2'].includes(String(type))) {
            return (
              <span>
                {quarterFirst}：{NP.times(upFirstQuarterPromotionScaleLimit, 100)}%
                <br />
                {quarterSecond}：{NP.times(upSecondQuarterPromotionScaleLimit, 100)}%
              </span>
            );
          } else {
            return `${quarterFirst}：${NP.times(upSecondQuarterPromotionScaleLimit, 100)}%`;
          }
        }
        return null;
      },
    },
    {
      title: '阶段一开始收费时间',
      dataIndex: 'upStartTime',
      key: 'upStartTime',
      width: COL_WIDTH_XL,
      render: function (val) {
        return val ? moment(val).format('YYYY-MM-DD HH:mm:ss') : '';
      },
    },
    {
      title: '阶段二优惠比例',
      width: COL_WIDTH_XL,
      render: function (record) {
        let quarterFirst = '';
        let quarterSecond = '';
        const { type, upFirstQuarterPromotionScaleLimit, upSecondQuarterPromotionScaleLimit } = record;
        switch (String(type)) {
          case '1':
            quarterFirst = '暑假';
            quarterSecond = '秋季';
            break;
          case '2':
            quarterFirst = '寒假';
            quarterSecond = '春季';
            break;
          case '3':
            quarterFirst = '春季';
            break;
          case '4':
            quarterFirst = '秋季';
            break;
        }
        if (isNumber(upFirstQuarterPromotionScaleLimit) && isNumber(upSecondQuarterPromotionScaleLimit)) {
          let upFirstQuarterPromotionScaleLimitPercent = NP.times(upFirstQuarterPromotionScaleLimit, 100);
          let upSecondQuarterPromotionScaleLimitPercent = NP.times(upSecondQuarterPromotionScaleLimit, 100);
          if (['1', '2'].includes(String(type))) {
            return (
              <span>
                {quarterFirst}：{NP.minus(100, upFirstQuarterPromotionScaleLimitPercent)}%
                <br />
                {quarterSecond}：{NP.minus(100, upSecondQuarterPromotionScaleLimitPercent)}%
              </span>
            );
          } else {
            return `${quarterFirst}：${NP.minus(100, upSecondQuarterPromotionScaleLimitPercent)}%`;
          }
        }
        return null;
      },
    },
    {
      title: '阶段二开始收费时间',
      dataIndex: 'downStartTime',
      key: 'downStartTime',
      width: COL_WIDTH_XL,
      render: function (val) {
        return val ? moment(val).format('YYYY-MM-DD HH:mm:ss') : '';
      },
    },
    {
      title: '状态',
      dataIndex: 'status',
      key: 'status',
      width: COL_WIDTH_M,
      render: function (val) {
        return String(val) ? StatusTextMap.get(val) : '';
      },
    },
    {
      title: '有效期',
      width: 300,
      render: function (text, record) {
        const { validStartTime, validEndTime } = record;
        if (validStartTime && validEndTime) {
          return `${moment(validStartTime).format('YYYY-MM-DD HH:mm:ss')} 至 ${moment(validEndTime).format(
            'YYYY-MM-DD HH:mm:ss',
          )}`;
        }
        return null;
      },
    },
    {
      title: '操作',
      width: COL_WIDTH_M,
      fixed: 'right',
      render: (text, record) => {
        const { status } = record;
        const buttons = [];
        if (status === 1) {
          // 有效状态
          if (this.props.btnAuth_edit) {
            buttons.push(
              <TextLink key="edit" fontSize={12} style={{ marginRight: 10 }} onClick={() => this.editItem(record)}>
                编辑
              </TextLink>,
            );
          }
          if (this.props.btnAuth_invalid) {
            buttons.push(
              <Popconfirm
                key="invalid"
                title="确定执行该操作吗？"
                onConfirm={() => this.inValidItem(record)}
                okText="确定"
                cancelText="取消"
              >
                <TextLink fontSize={12}>无效</TextLink>
              </Popconfirm>,
            );
          }
        }
        return buttons;
      },
    },
  ];

  constructor(props) {
    super(props);
    this.state = {
      dataList: [],
      searchParams,
      loading: false,
      branchDict: [],
      pagination: {
        total: 0,
        current: 1,
        pageSize: 20,
      },
    };

    this.onSearch = this.onSearch.bind(this);
    this.onReset = this.onReset.bind(this);
    this.addItem = this.addItem.bind(this);
    this.editItem = this.editItem.bind(this);
    this.inValidItem = this.inValidItem.bind(this);
    this.onPageChange = this.onPageChange.bind(this);
  }

  componentDidMount() {
    this.initDict();
    this.loadData();
  }

  async initDict() {
    const { resultCode, data } = await getUserBranchList({ resourceId: 'PERIOD_CHARGE' });
    if (resultCode === 0) {
      this.setState({ branchDict: data.map((r) => ({ val: r.id, text: r.name })) });
    }
  }

  transformParams() {
    const {
      pagination: { current, pageSize },
      searchParams,
    } = this.state;
    if (!searchParams.organizationIds) {
      searchParams.organizationIds = [];
    }
    return {
      pageSize,
      pageNum: current,
      ...searchParams,
    };
  }

  async loadData() {
    try {
      this.setState({ loading: true });
      const { resultCode, data } = await getPeriodChargeList(this.transformParams());
      if (resultCode === 0) {
        const { total, list: dataList } = data;
        const { pagination } = this.state;
        this.setState({
          dataList,
          pagination: {
            ...pagination,
            total,
          },
        });
      }
    } finally {
      this.setState({ loading: false });
    }
  }

  onSearch(params) {
    const { pagination, searchParams } = this.state;
    this.setState(
      {
        searchParams: {
          ...searchParams,
          ...params,
        },
        pagination: {
          ...pagination,
          current: 1,
        },
      },
      this.loadData,
    );
  }

  onReset() {
    const { pagination } = this.state;
    this.setState({
      searchParams: {
        ...this.state.searchParams,
        ...searchParams,
      },
      pagination: {
        ...pagination,
        current: 1,
      },
    });
  }

  onPageChange({ pageSize, current }) {
    const { pagination } = this.state;
    this.setState(
      {
        pagination: {
          ...pagination,
          pageSize,
          current,
        },
      },
      this.loadData,
    );
  }

  addItem() {
    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path: '/periodCharge/Add',
        refresh: true,
      },
    });
  }

  editItem(record) {
    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path: '/periodCharge/Edit',
        params: {
          id: record.id,
        },
        refresh: true,
      },
    });
  }

  async inValidItem(record) {
    const { id: segId } = record;
    try {
      this.setState({ loading: true });
      const { resultCode } = await inValidPeriodCharge({ segId });
      if (resultCode === 0) {
        showMessage('操作成功', 'success');
        await this.loadData();
      }
    } finally {
      this.setState({ loading: false });
    }
  }

  render() {
    const { route } = this.props;
    const { loading, pagination, dataList, branchDict } = this.state;

    return (
      <Page
        name={title}
        route={route}
        fitWindowHeight
        loadingTip="正在加载数据"
        loading={loading}
        headerToolBar={
          this.props.btnAuth_add && (
            <Button type="primary" onClick={this.addItem}>
              新增
            </Button>
          )
        }
        subHeader={
          <SearchFilterForm onSearch={this.onSearch} onReset={this.onReset} truncationNum={4}>
            <SearchFieldForm name="organizationIds" span={1}>
              <MultiSelect showSearch placeholder="请选择分公司" dict={branchDict} />
            </SearchFieldForm>
            <SearchFieldForm name="name" span={1}>
              <Input placeholder="请输入分段收费名称" />
            </SearchFieldForm>
            <SearchFieldForm name="type" span={1}>
              <Select allowClear dict={PeriodTypeDict} placeholder="请选择连报条件" />
            </SearchFieldForm>
            <SearchFieldForm initialValue="1" name="status" span={1}>
              <Select allowClear dict={StatusDict} placeholder="请选择状态" />
            </SearchFieldForm>
          </SearchFilterForm>
        }
      >
        <List dataSource={dataList} columns={this.columns} pagination={pagination} onChange={this.onPageChange} />
      </Page>
    );
  }
}

function mapStateToProps(state) {
  const { btnAuths } = state.account;
  const btnAuth_add = btnAuths.indexOf('ADD_PERIOD_CHARGE') > -1;
  const btnAuth_edit = btnAuths.indexOf('EDIT_PERIOD_CHARGE') > -1;
  const btnAuth_invalid = btnAuths.indexOf('INVALID_PERIOD_CHARGE') > -1;
  return {
    btnAuth_add,
    btnAuth_edit,
    btnAuth_invalid,
  };
}

export default connect(mapStateToProps)(ProductPeriodCharge);
