import React from 'react';
import Page from 'COMPONENT/Page';
import List from 'COMPONENT/List';
import { TWO_TEACHER, SMALL_CLASS, ONE_ON_ONE_COURSE, ONE_ON_MANY, ECS_CLASS } from 'CONST/dicts/product';
import Loader from './loader';
import ImportDetail from './modals/ImportDetail';
import styles from './style.less';

const { COL_WIDTH_S, COL_WIDTH_M } = List;

export default class ImportHistory extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      list: [], // {Array} 列表数据
      listLoading: false, // {false} 列表数据加载状态
      totalRecords: 0, // {number} 总记录数
      page: 1, // {number} 当前页码
      rows: 20, // {number} 每页记录数
      showModal: false, //是否显示导入历史详情模态框
      editHistory: null, //正编辑的某个导入历史
    };
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化内容
   */
  initContent = () => {
    this.search();
  };

  /**
   * 加载列表数据，并转换格式
   * @param  {object} changedParams 要更新的列表数据参数，包括查询条件、页码状态
   */
  loadList = async (changedParams = {}) => {
    const { page, rows } = this.state;

    const apiParams = {
      page,
      rows,
      ...changedParams,
    };
    this.setState({ listLoading: true });
    const { data: apiData } = await Loader.loadImportHistory(apiParams);
    if (apiData) {
      const { records, rows } = apiData;
      const { page: newCurrentPage, rows: newPageSize } = apiParams;
      this.setState({
        listLoading: false,
        list: rows,
        totalRecords: records,
        page: newCurrentPage,
        rows: newPageSize,
      });
      return true;
    } else {
      this.setState({ listLoading: false });
    }
  };

  /**
   * 查询
   */
  search = () => {
    this.loadList({ page: 1 });
  };

  showHistoryModal = (record) => {
    this.setState({
      showModal: true,
      editHistory: {
        recordId: record.recordId,
        productType: record.category,
      },
    });
  };

  hideHistoryModal = () => {
    this.setState({
      showModal: false,
    });
  };

  renderResult = (text, record) => {
    if (text === 1) {
      return <span className={`${styles.importResult} ${styles.successImport}`}>导入成功</span>;
    } else if (record.importStatus !== 1) {
      return <span className={`${styles.importResult} ${styles.failedImport}`}>导入失败</span>;
    } else {
      return <span>--</span>;
    }
  };

  renderProductType = (text) => {
    switch (text) {
      case TWO_TEACHER:
        return '双师';
      case SMALL_CLASS:
        return '小班';
      case ONE_ON_ONE_COURSE:
        return '一对一';
      case ONE_ON_MANY:
        return '一对多';
      case ECS_CLASS:
        return '精英班';
    }
  };

  columns = [
    { title: '序号', dataIndex: 'whatever', width: COL_WIDTH_S, render: (text, record, index) => index + 1 },
    { title: '类型', dataIndex: 'category', width: COL_WIDTH_M, render: this.renderProductType },
    { title: '导入时间', dataIndex: 'createTime', width: COL_WIDTH_M },
    { title: '操作人', dataIndex: 'createUserName', width: COL_WIDTH_M },
    {
      title: '导入状态',
      dataIndex: 'importStatus',
      width: COL_WIDTH_M,
      render: (text) => (text === 1 ? '导入中' : '导入完成'),
    },
    { title: '导入结果', dataIndex: 'importResult', width: COL_WIDTH_M, render: this.renderResult },
    { title: '完成时间', dataIndex: 'endTime', width: COL_WIDTH_M },
    {
      title: '操作',
      key: 'action',
      width: COL_WIDTH_S,
      fixed: 'right',
      getActionConfig: (text, record) => {
        return [{ text: '详情', onClick: () => this.showHistoryModal(record) }];
      },
    },
  ];

  render() {
    const { list, totalRecords, page, rows, listLoading, showModal, editHistory } = this.state;

    return (
      <Page
        fitWindowHeight={true}
        route={this.props.route}
        title={<span className={styles.historyTitle}>产品导入结果</span>}
      >
        <List
          dataSource={list}
          columns={this.columns}
          loading={listLoading}
          pagination={{
            current: page,
            total: totalRecords,
            pageSize: rows,
          }}
          onChange={({ current, pageSize }) => this.loadList({ page: current, rows: pageSize })}
        />
        {showModal && <ImportDetail editHistory={editHistory} onClose={this.hideHistoryModal} />}
      </Page>
    );
  }
}
