import React from 'react';
import { connect } from 'dva';
import Block from 'COMPONENT/Block';
import { Row, Col } from 'antd';
import Select from 'COMPONENT/Select';
import { STORE_NAME } from '../../store/config';
import { GROUP_ID } from '../../config';
import style from '../common.less';
import { loginUser } from 'UTIL/common';

class RelevancePreferentialBlock extends React.Component {
  constructor(props) {
    super(props);

    this.onUpdateStore = this.onUpdateStore.bind(this);
  }

  /**
   * 更新store分公司ID
   * @Attention 更新完会触发清除选中的优惠/产品列表，并重新加载列表
   * @param key 键
   * @param value 值
   */
  onUpdateStore(key, value) {
    const { dispatch } = this.props;

    dispatch({
      type: `${STORE_NAME.branch}/onBranchChange`,
      payload: { [key]: value },
    });
  }

  render() {
    const { branchDict, val_branchId } = this.props;

    return (
      <Block type="dark" className={style.container}>
        <Row gutter={0}>
          <Col span={2} className={style.block_title}>
            <span>优惠关联</span>
          </Col>

          <Col span={4}>
            <Select
              placeholder="分公司"
              showSearch={true}
              allowClear={false}
              dict={branchDict}
              value={val_branchId}
              onChange={(val) => {
                this.onUpdateStore('val_branchId', val);
              }}
            />
          </Col>
        </Row>
      </Block>
    );
  }
}

const mapStateToProps = (state) => {
  const { branch } = state.dict;
  const { val_branchId } = state[STORE_NAME.branch];

  // Attention: 前端写死一个星火集团
  let branchDict = [];
  if (branch) {
    branchDict = [...branch];
    branchDict.unshift({
      val: GROUP_ID,
      text: loginUser.isXingHuoSym() ? '星火集团' : '双师事业部',
    });
  }

  return {
    branchDict,
    val_branchId,
  };
};

export default connect(mapStateToProps)(RelevancePreferentialBlock);
