import React from 'react';
import { connect } from 'dva';
import List from 'COMPONENT/List';
import style from './index.less';
import SelectedNumber from '../SelectedNumber';
import { STORE_NAME } from '../../../../store/config';

const { COL_WIDTH_S, COL_WIDTH_L, COL_WIDTH_XL } = List;

class ListBlock extends React.Component {
  constructor(props) {
    super(props);

    this.pageSizeOptions = ['10', '30', '50', '100'];

    this.loadList = this.loadList.bind(this);
    this.onPageChange = this.onPageChange.bind(this);
    this.formatColumn = this.formatColumn.bind(this);
    this.onRowSelect = this.onRowSelect.bind(this);
    this.onSelectAll = this.onSelectAll.bind(this);
  }

  /**
   * 请求列表
   * @param changeParam
   * @return {boolean}
   */
  loadList(changeParam = {}) {
    const { paneKey } = this.props;

    this.props.dispatch({
      type: `${STORE_NAME.tab}/loadList`,
      payload: {
        paneKey,
        ...changeParam,
      },
    });
  }

  onPageChange({ current, pageSize }) {
    const changeParam = { pageNo: current, pageSize };
    this.loadList(changeParam);
  }

  /**
   * 表格列
   * @param checkedPromotion 选中的优惠列表
   */
  formatColumn(checkedPromotion) {
    return [
      { title: '产品名称', dataIndex: 'productName', key: 'productName', width: COL_WIDTH_L, lines: 2 },
      { title: '所属分公司', dataIndex: 'organizationName', key: 'organizationName', width: COL_WIDTH_L },
      {
        title: '优惠',
        dataIndex: 'promotionName',
        key: 'promotionName',
        width: COL_WIDTH_XL,
        lines: 2,
        render: (text, record, index) => {
          return this.formatPromotionName(text, checkedPromotion || []);
        },
      },
    ];
  }

  /**
   * 格式化优惠名称，将选中的优惠列表里面存在的名称标红并用`,`分割
   * @param text 产品的优惠名称
   * @param promotionList 选中的优惠列表
   * @param splitStr 优惠名称分割符号
   * @param joinStr 优惠名称拼接符号
   */
  formatPromotionName(text, promotionList = [], splitStr = ',', joinStr = '、') {
    if (!text) {
      return '';
    }

    if (!promotionList || promotionList.length <= 0) {
      return text.split(splitStr).join(joinStr);
    }

    const nameArr = text.split(splitStr);
    const node = (
      <div>
        {nameArr.map((name, index) => {
          const joinString = index === nameArr.length - 1 ? '' : joinStr;

          if (name) {
            const hasPromotion = promotionList.some((item) => {
              return item.name === name || item.promotionName === name;
            });

            if (hasPromotion) {
              return (
                <span key={index}>
                  <span style={{ color: '#6D67FF' }}>{name + joinString}</span>
                </span>
              );
            } else {
              return name + joinString;
            }
          } else {
            return '';
          }
        })}
      </div>
    );

    return node;
  }

  /**
   * 选择列表,列表选中用key记录
   * @Attention 由于分页，所有分页选中的列与当前页的选中key并无绝对的联系
   * @param row 当前列
   * @param isSelected 是否选中
   * @param checkedRows 当前页选中的所有列
   */
  onRowSelect(row, isSelected, checkedRows) {
    const { paneKey, tab_one, tab_two, dispatch } = this.props;

    // 找到对应tab的store选中产品
    let storeKey = 'tab_one';
    let storeCheckedList = tab_one.allPageChecked;

    if (paneKey === '2') {
      storeKey = 'tab_two';
      storeCheckedList = tab_two.allPageChecked;
    }

    const selectedRows = storeCheckedList || [];
    // 新的store选中产品
    let tempSelectedRows = [...selectedRows];

    if (isSelected) {
      // 先判断是否在所有选中列表中存在
      let hasSelected = false;
      selectedRows.forEach((item) => {
        if (item.id === row.id) {
          hasSelected = true;
          return false;
        }
      });

      if (!hasSelected) {
        tempSelectedRows.push(row);
      }
    } else {
      tempSelectedRows.forEach((tempRow, index) => {
        if (tempRow.id === row.id) {
          tempSelectedRows.splice(index, 1);
          return false;
        }
      });
    }

    // 更新store
    let payload = {};
    payload[storeKey] = {
      allPageChecked: tempSelectedRows,
    };

    dispatch({
      type: `${STORE_NAME.tab}/updateState`,
      payload: {
        ...payload,
      },
    });
  }

  /**
   * 选中/取消选中全部
   * @param selected 全选/取消全选
   * @param selectedRows 当前list选中的所有列表
   * @param changeRows 全选/取消全选改变的列表
   */
  onSelectAll(selected, selectedRows, changeRows) {
    const { paneKey, tab_one, tab_two, dispatch } = this.props;

    // 找到对应tab的store选中产品
    let storeKey = 'tab_one';
    let storeCheckedList = tab_one.allPageChecked;

    if (paneKey === '2') {
      storeKey = 'tab_two';
      storeCheckedList = tab_two.allPageChecked;
    }

    const allSelectedRows = storeCheckedList || [];
    // 新的store选中产品
    let tempSelectedRows = [...allSelectedRows];
    let resultSelectedRows = [];

    // 判断是选中还是全不选中
    if (selected) {
      // 选中并去重
      tempSelectedRows = tempSelectedRows.concat(changeRows);
      tempSelectedRows.forEach((row) => {
        const isExist = resultSelectedRows.some((item) => {
          return item.id === row.id;
        });

        if (!isExist) {
          resultSelectedRows.push(row);
        }
      });
    } else {
      // 取消勾选，去掉取消的列表
      tempSelectedRows.forEach((row, index) => {
        const isDelete = changeRows.some((item) => {
          return item.id === row.id;
        });

        if (!isDelete) {
          resultSelectedRows.push(row);
        }
      });
    }

    // 更新store
    let payload = {};
    payload[storeKey] = {
      allPageChecked: resultSelectedRows,
    };

    dispatch({
      type: `${STORE_NAME.tab}/updateState`,
      payload: {
        ...payload,
      },
    });
  }

  /**
   * 计算当前分页列表选中的列表
   * @param storeCheckedList 当前tab所有选中的列表
   * @param list 当前list数据
   */
  formatPageSelectedKeys(storeCheckedList = [], list = []) {
    const selectedRows = storeCheckedList || [];
    let tempPageSelectedKeys = [];

    selectedRows.forEach((row) => {
      let listIndex = 0;
      let hasThisRow = false;

      list.forEach((item, index) => {
        if (item.id === row.id) {
          listIndex = index;
          hasThisRow = true;
          return false;
        }
      });

      if (hasThisRow) {
        tempPageSelectedKeys.push(listIndex);
      }
    });

    return tempPageSelectedKeys;
  }

  render() {
    const { checkedPromotion, paneKey, tab_one, tab_two } = this.props;
    const tabStore = paneKey === '1' ? tab_one : tab_two;
    const { listLoading, list, total, pageNo, pageSize, allPageChecked } = tabStore || {};
    const storeCheckedList = allPageChecked || [];
    const columns = this.formatColumn(checkedPromotion);
    const pageSelectKey = this.formatPageSelectedKeys(storeCheckedList, list);

    return (
      <section className={style.table_container}>
        <SelectedNumber paneKey={paneKey} list={list} checkedList={storeCheckedList} />

        <List
          dataSource={list || []}
          columns={columns}
          loading={listLoading}
          pagination={{
            current: pageNo,
            total: total || 0,
            pageSize,
            pageSizeOptions: this.pageSizeOptions,
          }}
          onChange={this.onPageChange}
          rowSelection={{
            selectedRowKeys: pageSelectKey,
            onSelect: (record, selected, selectedRows) => {
              this.onRowSelect(record, selected, selectedRows);
            },
            onChange: (selectedRowKeys, selectedRows) => {},
            onSelectAll: (selected, selectedRows, changeRows) => {
              this.onSelectAll(selected, selectedRows, changeRows);
            },
          }}
        />
      </section>
    );
  }
}

const mapStateToProps = (state) => {
  const { allPageChecked } = state[STORE_NAME.promotion];
  const tabStore = state[STORE_NAME.tab];
  const { tab_one, tab_two } = tabStore;

  return {
    checkedPromotion: allPageChecked, // 渲染列表需要
    tab_one,
    tab_two,
  };
};

export default connect(mapStateToProps)(ListBlock);
