import React from 'react';
import { connect } from 'dva';
import Modal from 'COMPONENT/Modal';
import IconButton from 'COMPONENT/IconButton';
import style from './index.less';
import { STORE_NAME } from '../../../../store/config';

class SelectedListModal extends React.Component {
  constructor(props) {
    super(props);

    this.onDeleteItem = this.onDeleteItem.bind(this);
    this.onDeleteAll = this.onDeleteAll.bind(this);
  }

  /**
   * 删除其中某个选中
   * @param checkedList
   * @param index
   */
  onDeleteItem(checkedList, index) {
    const { dispatch, paneKey } = this.props;

    // 找到对应tab的store选中产品
    let tabKey = 'tab_one';
    if (paneKey === '2') {
      tabKey = 'tab_two';
    }

    let tempChecked = [...checkedList];
    tempChecked.splice(index, 1);

    // 更新store
    let payload = {};
    payload[tabKey] = {
      allPageChecked: tempChecked,
    };

    dispatch({
      type: `${STORE_NAME.tab}/updateState`,
      payload: {
        ...payload,
      },
    });
  }

  /**
   * 删除当前tab所有选中
   */
  onDeleteAll() {
    const { dispatch, paneKey } = this.props;

    dispatch({
      type: `${STORE_NAME.tab}/resetTabChecked`,
      payload: {
        paneKey,
      },
    });
  }

  render() {
    const { checkedList } = this.props;

    return (
      <Modal
        loading={false}
        onClose={this.props.onClose}
        onOk={this.props.onClose}
        onCancel={this.onDeleteAll}
        cancelBtnShow={true}
        title="已选产品"
        okText="确认"
        cancelText="清空所有"
        width={400}
        className={style.modal_wrap}
      >
        <div className={style.wrap}>
          <ul>
            {checkedList &&
              checkedList.map((item, index) => {
                return (
                  <li key={index}>
                    <div className={style.li__left}>{item.name || item.productName}</div>
                    <div className={style.li__right}>
                      <IconButton
                        type="delete_mc"
                        onClick={(e) => {
                          this.onDeleteItem(checkedList, index);
                        }}
                      />
                    </div>
                  </li>
                );
              })}
          </ul>
        </div>
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  return {};
};

export default connect(mapStateToProps)(SelectedListModal);
