import React from 'react';
import { connect } from 'dva';
import Block from 'COMPONENT/Block';
import { Row, Col, Tabs } from 'antd';
import TabPaneBlock from './TabPaneBlock';
import style from '../common.less';
import { STORE_NAME } from '../../store/config';

const { TabPane } = Tabs;

class RelevanceProductBlock extends React.Component {
  constructor(props) {
    super(props);

    this.tabs = [
      {
        name: '未关联产品',
        key: '1',
      },
      {
        name: '已关联产品',
        key: '2',
        showSearchStatus: true,
      },
    ];

    this.onTabChange = this.onTabChange.bind(this);
  }

  /**
   * 更新store
   * @param key 键
   * @param value 值
   */
  onTabChange(key, value) {
    this.props.dispatch({
      type: `${STORE_NAME.tab}/updateState`,
      payload: { [key]: value },
    });
  }

  render() {
    const { tab_key } = this.props;

    return (
      <Block type="dark" className={style.container}>
        <Row gutter={0}>
          <Col span={2} className={style.block_title}>
            <span>关联产品</span>
          </Col>
        </Row>

        <section className={style.product_wrap}>
          <Tabs
            activeKey={tab_key}
            onChange={(key) => {
              this.onTabChange('tab_key', key);
            }}
          >
            {this.tabs.map((item) => {
              const { name, key, showSearchStatus } = item;

              return (
                <TabPane tab={name} key={key}>
                  <TabPaneBlock paneKey={key} showSearchStatus={!!showSearchStatus} />
                </TabPane>
              );
            })}
          </Tabs>
        </section>
      </Block>
    );
  }
}

const mapStateToProps = (state) => {
  const { tab_key } = state[STORE_NAME.tab];

  return { tab_key };
};

export default connect(mapStateToProps)(RelevanceProductBlock);
