import React from 'react';
import { connect } from 'dva';
import SearchBar from 'COMPONENT/SearchBar';
import Select from 'COMPONENT/Select';
import Input from 'COMPONENT/Input';
import { STORE_NAME } from '../../../../store/config';

class SearchBlock extends React.Component {
  constructor(props) {
    super(props);

    this.onSearch = this.onSearch.bind(this);
    this.onReset = this.onReset.bind(this);
    this.onUpdateStore = this.onUpdateStore.bind(this);
  }

  onSearch() {
    this.props.dispatch({ type: `${STORE_NAME.promotion}/onSearch` });
  }

  onReset() {
    this.props.dispatch({ type: `${STORE_NAME.promotion}/resetSearch` });
  }

  /**
   * 更新store
   * @param key 键
   * @param value 值
   */
  onUpdateStore(key, value) {
    const { dispatch } = this.props;
    dispatch({
      type: `${STORE_NAME.promotion}/updateState`,
      payload: { [key]: value },
    });
  }

  render() {
    const { promotionTypeDict, val_promotionType, val_promotionName } = this.props;

    return (
      <SearchBar onSearch={this.onSearch} onReset={this.onReset} showExpandButton={false}>
        <Select
          placeholder="优惠类型"
          dict={promotionTypeDict}
          value={val_promotionType}
          onChange={(val) => {
            this.onUpdateStore('val_promotionType', val);
          }}
        />

        <Input
          placeholder="请输入优惠名称名称"
          value={val_promotionName}
          onChange={(e) => {
            this.onUpdateStore('val_promotionName', e.target.value);
          }}
        />
      </SearchBar>
    );
  }
}

const mapStateToProps = (state) => {
  const { promotionType } = state.dict;
  const { val_promotionType, val_promotionName } = state[STORE_NAME.promotion];

  return {
    promotionTypeDict: promotionType,
    val_promotionType,
    val_promotionName,
  };
};

export default connect(mapStateToProps)(SearchBlock);
