import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import style from './index.less';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import RelevancePreferentialBlock from './component/RelevancePreferentialBlock';
import SelectPreferentialBlock from './component/SelectPreferentialBlock';
import RelevanceProductBlock from './component/RelevanceProductBlock';
import ConfirmModal from './component/ConfirmModal';
import { getRelevanceType } from './config';
import { saveProductPromotion } from 'SERVICE/product';
import { STORE_NAME } from './store/config';

class RelevancePreferential extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      showConfirmModal: false,
    };

    this.initPage = this.initPage.bind(this);
    this.initDict = this.initDict.bind(this);
    this.destroy = this.destroy.bind(this);
    this.onCommit = this.onCommit.bind(this);
    this.doSave = this.doSave.bind(this);
    this.formatApiParam = this.formatApiParam.bind(this);
  }

  componentDidMount() {
    this.initPage();
  }

  componentWillUnmount() {
    this.destroy();
  }

  async initPage() {
    this.setState({ loading: true });
    await this.initDict();
    this.setState({ loading: false });
  }

  initDict() {
    const { dispatch } = this.props;

    return new Promise((resolve, reject) => {
      dispatch({
        type: 'dict/initDict',
        payload: {
          list: [
            'branch',
            'year',
            'quarter',
            'grade',
            'subject',
            'promotionType',
            'productCharacter',
            'productTypeTree',
            'courseSeries',
          ],
          callback() {
            resolve();
          },
        },
      });
    });
  }

  /**
   * 页面销毁需要重置所有状态
   */
  destroy() {
    this.props.dispatch({ type: `${STORE_NAME.branch}/resetPage` });
    this.props.dispatch({ type: `${STORE_NAME.promotion}/resetPage` });
    this.props.dispatch({ type: `${STORE_NAME.tab}/resetPage` });
  }

  /**
   * 提交保存
   * 先判断有无选中产品
   * 再判断取消关联还是确认关联，取消关联需要弹窗确认
   */
  onCommit() {
    const { tab_key } = this.props;
    const param = this.formatApiParam();

    if (param.products.length <= 0) {
      showMessage('请先选择产品！');
      return false;
    }

    if (tab_key === '1') {
      this.doSave();
    } else {
      this.setState({ showConfirmModal: true });
    }
  }

  doSave() {
    const param = this.formatApiParam();
    this.setState({ loading: true });

    saveProductPromotion(param)
      .then((result) => {
        showMessage('保存成功！', 'success');
        this.refreshProductList();
        this.setState({
          loading: false,
          showConfirmModal: false,
        });
      })
      .catch((e) => {
        showMessage(e.text || '保存失败，请重试！');
        this.setState({ loading: false });
      });
  }

  formatApiParam() {
    const { promotionList, productList, relevanceType } = this.props;

    let promotionIds = [];
    if (promotionList && promotionList.length > 0) {
      promotionList.forEach((item) => {
        promotionIds.push(item.promotionId);
      });
    }

    let productIds = [];
    if (productList && productList.length > 0) {
      productList.forEach((item) => {
        productIds.push(item.productId);
      });
    }

    return {
      promotions: promotionIds,
      products: productIds,
      relevanceType,
    };
  }

  /**
   * 刷新重新列表
   */
  refreshProductList() {
    const { dispatch, tab_key } = this.props;

    dispatch({
      type: `${STORE_NAME.tab}/onTabSearch`,
      payload: { paneKey: tab_key },
    });

    dispatch({
      type: `${STORE_NAME.tab}/resetTabChecked`,
      payload: { paneKey: tab_key },
    });
  }

  render() {
    const { loading, showConfirmModal } = this.state;
    const { tab_key } = this.props;
    const btnText = (tab_key === '1' && '确定关联') || '确定解绑';

    return (
      <Page
        title={false}
        footer={true}
        okBtnText={btnText}
        loading={loading}
        route={this.props.route}
        onOk={this.onCommit}
        ref={(ref) => {}}
        hasBackground={false}
        className={style.article}
      >
        <RelevancePreferentialBlock />

        <SelectPreferentialBlock />

        <RelevanceProductBlock />

        {showConfirmModal && (
          <ConfirmModal
            onClose={() => {
              this.setState({ showConfirmModal: false });
            }}
            onOk={this.doSave}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const tabStore = state[STORE_NAME.tab];
  const { tab_key, tab_one, tab_two } = tabStore;
  const productList = tab_key === '1' ? tab_one.allPageChecked : tab_two.allPageChecked;
  const { allPageChecked } = state[STORE_NAME.promotion];

  return {
    tab_key,
    promotionList: allPageChecked,
    productList,
    relevanceType: getRelevanceType(tab_key),
  };
};

export default connect(mapStateToProps)(RelevancePreferential);
