import { clone } from 'lodash';
import { defaultState, defaultSearch, defaultChecked } from './state';
import { STORE_NAME } from '../config';
import { loadPromotionList } from './loader';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';

export default {
  namespace: STORE_NAME.promotion,
  state: {
    ...clone(defaultState),
  },
  reducers: {
    updateState(state, { payload }) {
      return {
        ...state,
        ...payload,
      };
    },
  },
  effects: {
    /**
     * 重置优惠state
     * @param action
     * @param put
     * @return {IterableIterator<*>}
     */
    *resetPage(action, { put }) {
      yield put({
        type: 'updateState',
        payload: {
          ...clone(defaultState),
        },
      });
    },

    /**
     * 清除搜索条件
     * @param action
     * @param put
     * @return {IterableIterator<*>}
     */
    *resetSearch(action, { put }) {
      yield put({
        type: 'updateState',
        payload: {
          ...defaultSearch,
        },
      });
    },

    /**
     * 清除选中
     * @param action
     * @param put
     * @return {IterableIterator<*>}
     */
    *resetChecked(action, { put }) {
      yield put({
        type: 'updateState',
        payload: {
          ...defaultChecked,
        },
      });
    },

    /**
     * 刷新优惠模块
     * @param action
     * @param put
     * @return {IterableIterator<*>}
     */
    *onRefresh(action, { put }) {
      yield put({ type: 'resetChecked' });
      yield put({ type: 'onSearch' });
    },

    /**
     * 搜索第一页（这里主要为了给store直接调用）
     * @return {IterableIterator<*>}
     */
    *onSearch(action, { put }) {
      const changeParam = { pageNo: 1 };

      yield put({
        type: 'loadList',
        payload: { ...changeParam },
      });
    },

    /**
     * 选择优惠时判断是否存在产品列表或选中产品，有则需要清空
     * @param action
     * @param put
     * @param select
     * @return {IterableIterator<*>}
     */
    *onCheckedChange({ payload, doUpdateState, doResetTabData }, { put, select }) {
      const tabStore = yield select((state) => {
        return {
          tab_one: state[STORE_NAME.tab]['tab_one'],
          tab_two: state[STORE_NAME.tab]['tab_two'],
        };
      });
      const { tab_one, tab_two } = tabStore;
      const shouldRefreshProduct =
        (tab_one.allPageChecked && tab_one.allPageChecked.length > 0) ||
        (tab_one.list && tab_one.list.length > 0) ||
        (tab_two.allPageChecked && tab_two.allPageChecked.length > 0) ||
        (tab_two.list && tab_two.list.length > 0);

      if (shouldRefreshProduct) {
        showConfirm(
          `您的优惠已更新，系统将清空已勾选的产品，请确认是否继续？`,
          () => {
            doUpdateState();
            doResetTabData();
          },
          () => {},
        );
      } else {
        doUpdateState();
      }
    },

    /**
     * 获取优惠列表
     * @param payload (changeParam)外部传入的请求参数
     * @param put
     * @param select
     * @return {IterableIterator<*>}
     */
    *loadList({ payload }, { put, select }) {
      const listLoading = yield select((state) => {
        return state[STORE_NAME.promotion].listLoading;
      });

      if (listLoading) {
        return false;
      }

      // 组装搜索条件
      const searchParam = yield select((state) => {
        const { val_branchId } = state[STORE_NAME.branch];
        const { val_promotionType, val_promotionName, pageNo, pageSize } = state[STORE_NAME.promotion];

        return {
          branchId: val_branchId,
          promotionType: val_promotionType,
          promotionName: val_promotionName,
          pageNo,
          pageSize,
        };
      });
      const param = { ...searchParam, ...payload };

      if (!param.branchId) {
        showMessage('请选择分公司!');
        return false;
      }

      // 转换为api参数，把pageNo改为page，pageSize改为rows
      let apiParam = { ...param };
      apiParam.page = param.pageNo;
      apiParam.rows = param.pageSize;
      delete apiParam.pageNo;
      delete apiParam.pageSize;

      yield put({
        type: 'updateState',
        payload: { listLoading: true },
      });

      const result = yield loadPromotionList(apiParam, true);
      const newStore = {
        ...result,
        pageNo: param.pageNo,
        pageSize: param.pageSize,
        listLoading: false,
      };

      yield put({
        type: 'updateState',
        payload: { ...newStore },
      });
    },
  },
};
