import React from 'react';
import List from 'COMPONENT/List/CardList';
import Select from 'COMPONENT/Select';
import Button from 'COMPONENT/Button';
import Modal from 'COMPONENT/Modal';
import Loader from '../loader';
import OriginDetail from './OriginDetail';
import { TWO_TEACHER, SMALL_CLASS, ONE_ON_ONE_COURSE, ONE_ON_MANY, ECS_CLASS } from 'CONST/dicts/product';
import {
  renderClassType,
  renderCourseSeries,
  renderGroup,
  renderQuarter,
  renderSingleSubject,
  renderSubject,
  renderTotalAmount,
  renderTotalHour,
  renderCourseHour,
  renderContractPayCategory,
} from './commonRender';
import styles from '../style.less';
import { urlBase } from 'CONST/config';
import { makeQueryString } from 'UTIL/http/index';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

const { COL_WIDTH_S, COL_WIDTH_M } = List;

const resultDict = [
  { val: '', text: '全部' },
  { val: '1', text: '导入成功' },
  { val: '0', text: '导入失败' },
];

export default class ImportDetail extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      list: [], // {Array} 列表数据
      listLoading: false, // {false} 列表数据加载状态
      totalRecords: 0, // {number} 总记录数
      page: 1, // {number} 当前页码
      rows: 10, // {number} 每页记录数
      importResult: '', //导出结果下拉框
      expandedRowKey: [], //列表展开行的key
      showOriginDetail: false, //是否显示历史详情
      editOriginDetail: null, //当前编辑的历史详情
    };
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化内容
   */
  initContent = () => {
    this.search();
  };

  /**
   * 加载列表数据，并转换格式
   * @param  {object} changedParams 要更新的列表数据参数，包括查询条件、页码状态
   */
  loadList = async (changedParams = {}) => {
    const { page, rows, importResult } = this.state;
    const { recordId } = this.props.editHistory;

    const apiParams = {
      page: page,
      rows: rows,
      importResult,
      recordId,
      ...changedParams,
    };

    this.setState({ listLoading: true });
    const { data: apiData } = await Loader.loadImportDetail(apiParams);
    if (apiData) {
      const { records, rows } = apiData;
      const { page: newCurrentPage, rows: newPageSize } = apiParams;
      this.setState({
        listLoading: false,
        list: rows,
        totalRecords: records,
        page: newCurrentPage,
        rows: newPageSize,
      });
      return true;
    }
  };

  /**
   * 查询
   */
  search = () => {
    this.loadList({ page: 1 });
  };

  /**
   * 渲染详情
   * @param record
   */
  // (双师)   年份  季度 	产品名称	产品性质	产品系列	班级类型	  科目	          所属分公司	年级	 生效日期	失效日期	课时单价	总课时	课时时长	关联其他产品	总价	优惠方案
  // (小班)   年份	 季度	产品名称	产品性质	产品系列	班级类型	  科目	    产品组	所属分公司	年级	 生效日期	失效日期	课时单价	总课时	课时时长	关联其他产品	总价	优惠方案
  // (一对一) 年份	      产品名称	产品性质	产品系列	          关联科目	      所属分公司	年级	 生效日期	失效日期	课时单价	      课时时长	关联其他产品	    优惠方案
  // (一对多) 年份	      产品名称	产品性质	        一对多类型	              所属分公司	年级	 生效日期	失效日期	课时单价	总课时	课时时长	关联其他产品	    优惠方案
  // (精英班) 年份	 季度	产品名称	产品性质	        班级类型	                所属分公司	年级	 生效日期	失效日期	课时单价	总课时	课时时长	关联其他产品	总价	优惠方案	单科/全科
  renderDetail = (record) => {
    const { productType } = this.props.editHistory;
    const isSmallClass = productType === 'SMALL_CLASS';
    return (
      <div className={styles.cardDetail}>
        <ul className={styles.colFirst}>
          <li>
            <label>年份:&nbsp;&nbsp;</label>
            <span>{record.productVersion}</span>
          </li>
          {renderQuarter(productType, record)}
          {renderSubject(productType, record)}
          <li>
            <label>年级:&nbsp;&nbsp;</label>
            <span>{record.grade}</span>
          </li>
          <li>
            <label>产品性质:&nbsp;&nbsp;</label>
            <span>{record.productCharacter}</span>
          </li>
          {renderCourseSeries(productType, record)}
        </ul>

        <ul className={styles.colSecond}>
          {renderGroup(productType, record)}
          {renderClassType(productType, record)}
          <li>
            <label>生效日期:&nbsp;&nbsp;</label>
            <span>{record.validStartDate}</span>
          </li>
          <li>
            <label>失效日期:&nbsp;&nbsp;</label>
            <span>{record.validEndDate}</span>
          </li>
          {renderTotalHour(productType, record)}
          {renderCourseHour(record)}
          {renderContractPayCategory(productType, record)}
        </ul>

        <ul className={styles.colThird}>
          {renderTotalAmount(productType, record)}
          <li>
            <label>优惠方案:&nbsp;&nbsp;</label>
            <span>{record.promotion}</span>
          </li>
          <li>
            <label>所属分公司:&nbsp;&nbsp;</label>
            <span>{record.organizationName}</span>
          </li>
          <li>
            <label>关联其它产品:&nbsp;&nbsp;</label>
            <span>{record.otherProduct}</span>
          </li>
          <li>
            <label>课时时长:&nbsp;&nbsp;</label>
            <span>{record.classTimeLength}</span>
          </li>
          {renderSingleSubject(productType, record)}
          {isSmallClass && (
            <li>
              <label>小时数:&nbsp;&nbsp;</label>
              <span>{record.classTotalNatureHours}</span>
            </li>
          )}
        </ul>
      </div>
    );
  };

  renderResult = (text) => {
    if (text === 1) {
      return <span className={`${styles.importResult} ${styles.successImport}`}>导入成功</span>;
    } else {
      return <span className={`${styles.importResult} ${styles.failedImport}`}>导入失败</span>;
    }
  };

  showOriginDetail = (record) => {
    this.setState({
      showOriginDetail: true,
      editOriginDetail: {
        id: record.productId,
        productType: this.props.editHistory.productType,
        versionType: record.versionType,
      },
    });
  };

  hideOriginDetail = (record) => {
    this.setState({
      showOriginDetail: false,
    });
  };

  /**
   * 点击行展开
   */
  onRowClick = (record, index) => {
    const { expandedRowKey } = this.state;
    if (expandedRowKey === index) {
      this.setState({
        expandedRowKey: null,
      });
    } else {
      this.setState({
        expandedRowKey: index,
      });
    }
  };

  /**
   * 下载excel文件
   * @returns {Promise<void>}
   */
  exportExcel = async () => {
    const { importResult } = this.state;
    const { recordId } = this.props.editHistory;
    this.setState({ exporting: true });
    const xhr = new XMLHttpRequest();
    xhr.open(
      'GET',
      urlBase +
        'ProductImportController/exportRecordDetailByRecordId.do?' +
        makeQueryString({ importResult, recordId }),
    );
    xhr.responseType = 'blob';
    xhr.onload = () => {
      const link = document.createElement('a');
      const body = document.body;
      const disposition = xhr.getResponseHeader('content-disposition');
      const fileName = disposition.match(/filename=(.+)/)[1];
      link.href = window.URL.createObjectURL(xhr.response);
      link.download = decodeURIComponent(fileName);
      link.style.display = 'none';
      body.appendChild(link);
      link.click();
      window.URL.revokeObjectURL(link.href);
      body.removeChild(link);
      this.setState({ exporting: false });
    };
    xhr.onerror = () => {
      showMessage('导出失败');
      this.setState({ exporting: false });
    };
    xhr.send();
  };

  /**
   * 表头
   * @type {*[]}
   */
  columns = [
    { title: '序号', dataIndex: 'whatever', width: COL_WIDTH_S, render: (text, record, index) => index + 1 },
    { title: '产品名称', dataIndex: 'productName', width: COL_WIDTH_M },
    { title: '导入结果', dataIndex: 'importResult', width: COL_WIDTH_M, render: this.renderResult },
    { title: '失败原因', dataIndex: 'failureReason', width: COL_WIDTH_M },
    {
      title: '覆盖详情',
      key: 'action',
      width: COL_WIDTH_S,
      getActionConfig: (text, record, index) => {
        //导入成功，并且有覆盖数据才显示详情
        if (record.isOverwrite && !!record.productId && record.importResult === 1) {
          return [{ text: '详情', onClick: () => this.showOriginDetail(record, index) }];
        } else {
          return [];
        }
      },
    },
  ];

  render() {
    const {
      list,
      totalRecords,
      page,
      rows,
      listLoading,
      importResult,
      expandedRowKey,
      showOriginDetail,
      editOriginDetail,
      exporting,
    } = this.state;

    return (
      <Modal
        onClose={this.props.onClose}
        footer={null}
        closable={true}
        title={<span style={{ color: '#363636', fontSize: 18 }}>结果详情</span>}
        width={1000}
      >
        <div style={{ display: 'flex', marginBottom: '1em' }}>
          <div style={{ width: 200 }}>
            <Select
              value={importResult}
              dict={resultDict}
              onChange={(importResult) => this.setState({ importResult })}
            />
          </div>
          <Button style={{ margin: '0 2em' }} onClick={this.search} type="primary">
            查询
          </Button>
          <Button onClick={this.exportExcel} disabled={exporting} type="primary">
            {exporting ? '导出中...' : '导出'}
          </Button>
        </div>
        <List
          dataSource={list}
          columns={this.columns}
          loading={listLoading}
          pagination={{
            current: page,
            total: totalRecords,
            pageSize: rows,
          }}
          style={{ height: 600, paddingBottom: 20 }}
          onRowClick={this.onRowClick}
          className={styles.customList}
          onChange={({ current, pageSize }) => this.loadList({ page: current, rows: pageSize })}
          expandedRowRender={this.renderDetail}
          expandedRowKey={expandedRowKey}
          onExpandedRowChange={(expandedRowKey) => this.setState({ expandedRowKey })}
        />
        {showOriginDetail && <OriginDetail editOriginDetail={editOriginDetail} onClose={this.hideOriginDetail} />}
      </Modal>
    );
  }
}
