import React from 'react';
import Modal from 'COMPONENT/Modal';
import NoData from 'COMPONENT/NoData';
import {
  renderClassType,
  renderContractPayCategory,
  renderCourseSeries,
  renderGroup,
  renderQuarter,
  renderSingleSubject,
  renderSubject,
  renderTotalAmount,
  renderTotalHour,
} from './commonRender';
import Loader from '../loader';
import styles from '../style.less';

export default class OriginDetail extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false,
      record: null,
    };
  }
  componentDidMount() {
    this.initContent();
  }

  initContent = async () => {
    const { id, versionType } = this.props.editOriginDetail;
    this.setState({ loading: true });
    const { data } = await Loader.loadOriginDetail({ productId: id });
    if (data) {
      if (data.data) {
        this.setState({
          record: { versionType, ...data.data[0] },
          loading: false,
        });
      } else {
        this.setState({
          loading: false,
        });
      }
    }
  };

  render() {
    const { loading, record } = this.state;
    const { productType } = this.props.editOriginDetail;
    return (
      <Modal
        loading={loading}
        className={styles.originDetailModal}
        title="原始数据"
        width={680}
        closable={true}
        footer={null}
        onClose={this.props.onClose}
      >
        {record ? (
          <div className={styles.cardDetail}>
            <ul>
              <li>
                <label>年份:&nbsp;&nbsp;</label>
                <span>{record.productVersion}</span>
              </li>
              {renderQuarter(productType, record)}
              {renderSubject(productType, record)}
              <li>
                <label>年级:&nbsp;&nbsp;</label>
                <span>{record.grade}</span>
              </li>
              <li>
                <label>产品性质:&nbsp;&nbsp;</label>
                <span>{record.productCharacter}</span>
              </li>
              {renderCourseSeries(productType, record)}
              {renderGroup(productType, record)}
              {renderClassType(productType, record)}
              <li>
                <label>课时时长:&nbsp;&nbsp;</label>
                <span>{record.classTimeLength}</span>
              </li>
              <li>
                <label>小时单价:&nbsp;&nbsp;</label>
                <span>{record.naturePrice}</span>
              </li>
            </ul>

            <ul>
              <li>
                <label>生效日期:&nbsp;&nbsp;</label>
                <span>{record.validStartDate}</span>
              </li>
              <li>
                <label>失效日期:&nbsp;&nbsp;</label>
                <span>{record.validEndDate}</span>
              </li>
              {renderTotalHour(productType, record)}
              {renderTotalAmount(productType, record)}
              <li>
                <label>优惠方案:&nbsp;&nbsp;</label>
                <span>{record.promotion}</span>
              </li>
              <li>
                <label>所属分公司:&nbsp;&nbsp;</label>
                <span>{record.organizationName}</span>
              </li>
              <li>
                <label>关联其它产品:&nbsp;&nbsp;</label>
                <span>{record.otherProduct}</span>
              </li>
              {renderSingleSubject(productType, record)}
              {renderContractPayCategory(productType, record)}
            </ul>
          </div>
        ) : (
          <NoData style={{ margin: '50px 0' }} />
        )}
      </Modal>
    );
  }
}
