import { TWO_TEACHER, SMALL_CLASS, ONE_ON_ONE_COURSE, ONE_ON_MANY, ECS_CLASS } from 'CONST/dicts/product';

/**
 * 渲染季度
 * @param productType
 * @param record
 * @returns {*}
 */
export const renderQuarter = (productType, record) => {
  if (productType === ONE_ON_MANY || productType === ONE_ON_ONE_COURSE) {
    return null;
  } else {
    return (
      <li>
        <label>季度:&nbsp;&nbsp;</label>
        <span>{record.productQuarter}</span>
      </li>
    );
  }
};

/**
 * 渲染产品系列
 * @param productType
 * @param record
 * @returns {*}
 */
export const renderCourseSeries = (productType, record) => {
  if (productType === ONE_ON_MANY || productType === ECS_CLASS) {
    return null;
  } else {
    return (
      <li>
        <label>产品系列:&nbsp;&nbsp;</label>
        <span>{record.courseSeries}</span>
      </li>
    );
  }
};

/**
 * 渲染班级类型
 * @param productType
 * @param record
 * @returns {*}
 */
export const renderClassType = (productType, record) => {
  if (productType === ONE_ON_ONE_COURSE) {
    return null;
  } else if (productType === ONE_ON_MANY) {
    return (
      <li>
        <label>一对多类型:&nbsp;&nbsp;</label>
        <span>{record.oneOnManyType}</span>
      </li>
    );
  } else {
    return (
      <li>
        <label>班级类型:&nbsp;&nbsp;</label>
        <span>{record.classType}</span>
      </li>
    );
  }
};

/**
 * 渲染科目
 * @param productType
 * @param record
 * @returns {*}
 */
export const renderSubject = (productType, record) => {
  if (productType === TWO_TEACHER || productType === SMALL_CLASS) {
    return (
      <li>
        <label>科目:&nbsp;&nbsp;</label>
        <span>{record.subjectName}</span>
      </li>
    );
  } else if (productType === ONE_ON_ONE_COURSE) {
    return (
      <li>
        <label>关联科目:&nbsp;&nbsp;</label>
        <span>{record.relateSubjectNames}</span>
      </li>
    );
  } else {
    return null;
  }
};

/**
 * 渲染产品组
 * @param productType
 * @param record
 * @returns {*}
 */
export const renderGroup = (productType, record) => {
  if (productType === SMALL_CLASS) {
    return (
      <li>
        <label>产品组:&nbsp;&nbsp;</label>
        <span>{record.productGroup}</span>
      </li>
    );
  } else {
    return null;
  }
};

/**
 * 渲染总课时
 * @param productType
 * @param record
 * @returns {*}
 */
export const renderTotalHour = (productType, record) => {
  if (productType === ONE_ON_ONE_COURSE) {
    return null;
  } else if (record.versionType === 'ORIGINAL') {
    return (
      <li>
        <label>总课时:&nbsp;&nbsp;</label>
        <span>{record.classTotalHour}</span>
      </li>
    );
  } else if (record.versionType === 'CONVERT_TO_HOUR') {
    if (productType === 'SMALL_CLASS') {
      return (
        <li>
          <label>分钟数:&nbsp;&nbsp;</label>
          <span>{record.classTotalNatureMinutes}</span>
        </li>
      );
    } else {
      return (
        <li>
          <label>总时长（分钟）:&nbsp;&nbsp;</label>
          <span>{record.classTotalNatureMinutes}</span>
        </li>
      );
    }
  }
};

/**
 * 根据类型渲染课时
 * @param versionType
 * @param record
 * @returns {*}
 */
export const renderCourseHour = (record) => {
  if (record.versionType === 'ORIGINAL') {
    return (
      <li>
        <label>课时单价:&nbsp;&nbsp;</label>
        <span>{record.price}</span>
      </li>
    );
  } else if (record.versionType === 'CONVERT_TO_HOUR') {
    return (
      <li>
        <label>小时单价:&nbsp;&nbsp;</label>
        <span>{record.naturePrice}</span>
      </li>
    );
  }
};

/**
 * 渲染总价
 * @param productType
 * @param record
 * @returns {*}
 */
export const renderTotalAmount = (productType, record) => {
  if (productType === ONE_ON_MANY || productType === ONE_ON_ONE_COURSE) {
    return null;
  } else {
    return (
      <li>
        <label>课程总价:&nbsp;&nbsp;</label>
        <span>{record.totalAmount}</span>
      </li>
    );
  }
};

/**
 * 渲染单全科
 * @param productType
 * @param record
 * @returns {*}
 */
export const renderSingleSubject = (productType, record) => {
  if (productType === ECS_CLASS) {
    return (
      <li>
        <label>单科/全科:&nbsp;&nbsp;</label>
        <span>{record.singleSubject}</span>
      </li>
    );
  }
};

/* 产品分账 */
export const renderContractPayCategory = (productType, record) => {
  if ([SMALL_CLASS, ONE_ON_ONE_COURSE, ONE_ON_MANY].includes(productType)) {
    return (
      <li>
        <label>产品分账:&nbsp;&nbsp;</label>
        <span>{record.contractPayCategory}</span>
      </li>
    );
  }
  return null;
};
