import React from 'react';
import { connect } from 'react-redux';
import Input from 'COMPONENT/Input';
import Modal from 'COMPONENT/Modal';
import Button from 'COMPONENT/Button';
import Select from 'COMPONENT/Select';
import Radio from 'COMPONENT/Radio';
import Checkbox from 'COMPONENT/Checkbox';
import TextLink from 'COMPONENT/TextLink';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { TWO_TEACHER, SMALL_CLASS, ONE_ON_ONE_COURSE, ONE_ON_MANY, ECS_CLASS } from 'CONST/dicts/product';
import { urlBase, PROJECT_NAME, xinghuoInstitution } from 'CONST/config';
import { OSS_BASE_PREFIX, OSS_BASE_PREFIX_SS } from 'CONST/ossConfig';
import { loginUser } from 'UTIL/common';
import excelPng from './assets/excel.png';
import filePng from './assets/file.png';
import delPng from './assets/del.png';
import styles from './style.less';

const TWO_TEACHER_URL = '/%E5%8F%8C%E5%B8%88%E4%BA%A7%E5%93%81%E5%AF%BC%E5%85%A5.xlsx';
const SMALL_CLASS_URL = '/%E5%B0%8F%E7%8F%AD%E4%BA%A7%E5%93%81%E5%AF%BC%E5%85%A5.xlsx';
const ONE_ON_ONE_COURSE_URL =
  Index.getLoginUser().institution === xinghuoInstitution
    ? '/%E6%98%9F%E7%81%AB%E4%B8%80%E5%AF%B9%E4%B8%80%E4%BA%A7%E5%93%81%E5%AF%BC%E5%85%A5.xlsx'
    : '/%E4%B8%80%E5%AF%B9%E4%B8%80%E4%BA%A7%E5%93%81%E5%AF%BC%E5%85%A5.xlsx';
const ONE_ON_MANY_URL =
  Index.getLoginUser().institution === xinghuoInstitution
    ? '/%E6%98%9F%E7%81%AB%E4%B8%80%E5%AF%B9%E5%A4%9A%E4%BA%A7%E5%93%81%E5%AF%BC%E5%85%A5.xlsx'
    : '/%E4%B8%80%E5%AF%B9%E5%A4%9A%E4%BA%A7%E5%93%81%E5%AF%BC%E5%85%A5.xlsx';
const ECS_CLASS_URL = '/%E7%9B%AE%E6%A0%87%E7%8F%AD%E5%AF%BC%E5%85%A5.xlsx';

let productTypeDict;

if (PROJECT_NAME === 'spark') {
  productTypeDict = [
    { val: TWO_TEACHER, text: '双师', url: TWO_TEACHER_URL },
    { val: SMALL_CLASS, text: '小班', url: SMALL_CLASS_URL },
    { val: ONE_ON_ONE_COURSE, text: '一对一', url: ONE_ON_ONE_COURSE_URL },
    { val: ONE_ON_MANY, text: '一对多', url: ONE_ON_MANY_URL },
    { val: ECS_CLASS, text: '精英班', url: ECS_CLASS_URL, disabled: true },
  ];
} else if (PROJECT_NAME === 'twoteacher') {
  productTypeDict = [
    { val: TWO_TEACHER, text: '双师', url: TWO_TEACHER_URL },
    { val: SMALL_CLASS, text: '小班', url: SMALL_CLASS_URL },
  ];
} else {
  productTypeDict = [{ val: SMALL_CLASS, text: '小班', url: SMALL_CLASS_URL }];
}

class ProductImportModal extends React.Component {
  constructor(props) {
    super(props);
    this.updateOverwriteOffer = this.updateOverwriteOffer.bind(this);
    this.enAbleOverWrite = this.changeOverWrite.bind(this, true);
    this.disAbleOverWrite = this.changeOverWrite.bind(this, false);
    this.onOAWorkNumChange = this.onOAWorkNumChange.bind(this);
    this.state = {
      loading: false,
      //1：待上传，2：上传完成
      status: 1,
      productTypeId: null,
      files: [],
      isOverwrite: false,
      onlyOverwriteOffer: false,
      oaInstanceId: '',
    };
  }

  getTitle = () => {
    const { status } = this.state;
    if (status === 1) {
      return '产品导入';
    } else {
      return '导入提醒';
    }
  };

  /**
   * 点击选择文件事件
   * @param event
   */
  pickFile = (event) => {
    this.setState({
      files: Array.prototype.map.call(event.target.files, (file) => ({
        file,
        name: file.name,
        size: file.size,
        type: file.type,
      })),
    });
  };

  /**
   * 删除已选文件
   */
  onDelete = () => {
    this.setState({
      files: [],
    });
  };

  changeOverWrite = (value) => {
    this.setState({
      isOverwrite: value,
    });
  };

  onOAWorkNumChange(e) {
    this.setState({
      oaInstanceId: e.target.value,
    });
  }

  downloadTemplate = (productType) => {
    const url = this.formatDownloadUrl(productType.url);
    const link = document.createElement('a');
    const body = document.body;

    link.href = url;
    link.style.display = 'none';
    link.download = productType.text + '模板.xls';
    body.appendChild(link);
    link.click();
    body.removeChild(link);
  };

  /**
   * 格式化模版下载地址
   *
   * @param fileName {string}
   * @return {string}
   */
  formatDownloadUrl = (fileName) => {
    if (loginUser.isTwoTeacherSym()) {
      return OSS_BASE_PREFIX_SS + fileName;
    }

    return OSS_BASE_PREFIX + fileName;
  };

  // 更新是否覆盖优惠
  updateOverwriteOffer(ele) {
    this.setState({ onlyOverwriteOffer: ele.target.checked });
  }

  /**
   * 渲染选择的文件或按钮
   */
  renderFiles = () => {
    const { files } = this.state;
    if (files.length > 0) {
      return (
        <div className={styles.fileArea}>
          {files.map((file) => {
            return (
              <div key={file.name} className={styles.file}>
                <span className={styles.fileWrap}>
                  <img src={filePng} alt="文件" width="16" height="16" />
                  <span className={styles.fileName}>{file.name}</span>
                </span>
                <img
                  src={delPng}
                  width="16"
                  height="16"
                  onClick={this.onDelete}
                  className={styles.fileDel}
                  alt="删除"
                />
              </div>
            );
          })}
        </div>
      );
    } else {
      return (
        <div className={styles.pickFile}>
          选择文件
          <input
            type="file"
            accept="application/vnd.openxmlformats-officedocument.spreadsheetml.sheet, application/vnd.ms-excel"
            className={styles.uploadBtn}
            onClick={(e) => (e.target.value = null)}
            onChange={this.pickFile}
          />
        </div>
      );
    }
  };

  /**
   * 渲染模态框待上传状态
   */
  renderUpload = () => {
    const { productTypeId, isOverwrite } = this.state;
    const productType = productTypeDict.find((type) => type.val === productTypeId);
    return (
      <div>
        <p className={`${styles.required} ${styles.label}`}>产品类型</p>
        <Select
          dict={productTypeDict}
          value={productTypeId}
          style={{ margin: '12px 0' }}
          onChange={(productTypeId) => this.setState({ productTypeId })}
        />

        {productTypeId && (
          <div>
            <p style={{ display: 'flex', margin: '12px 0 24px 0' }}>
              <img src={excelPng} width="22" height="22" alt="excel" />
              <TextLink style={{ marginLeft: 10 }} onClick={() => this.downloadTemplate(productType)}>
                {productType.text + '模板下载'}
              </TextLink>
            </p>

            <p className={styles.label}>上传文件</p>

            {this.renderFiles()}

            <div className={styles.label}>
              <span>覆盖数据</span>
              <label style={{ marginLeft: 30 }}>
                <Radio
                  checked={isOverwrite}
                  style={{ verticalAlign: '10%' }}
                  name="isOverwrite"
                  onChange={this.enAbleOverWrite}
                />
                是
              </label>
              <label style={{ marginLeft: 24 }}>
                <Radio
                  checked={!isOverwrite}
                  style={{ verticalAlign: '10%' }}
                  name="isOverwrite"
                  onChange={this.disAbleOverWrite}
                />
                否
              </label>
            </div>
            {isOverwrite && (
              <div className={styles.label}>
                <span>仅覆盖优惠</span>
                <label style={{ marginLeft: 30 }}>
                  <Checkbox checked={this.state.onlyOverwriteOffer} onChange={this.updateOverwriteOffer}>
                    {' '}
                  </Checkbox>
                </label>
              </div>
            )}
            <div className={styles.mark}>
              注：覆盖指批量修改历史产品，未被销售使用的产品均可被批量覆盖，已被销售使用的产品，仅支持覆盖优惠方案。
            </div>
          </div>
        )}
        <p className={`${styles.required} ${styles.label}`}>OA工单号</p>
        <Input value={this.state.oaInstanceId} onChange={this.onOAWorkNumChange} />
      </div>
    );
  };

  /**
   * 渲染模态框内容
   * @returns {*}
   */
  renderBody = () => {
    const { status } = this.state;
    if (status === 1) {
      return this.renderUpload();
    } else {
      return <p className={styles.result}>数据正在导入，您可在“导入历史”中查看导入结果！</p>;
    }
  };

  /**
   * 渲染底部按钮
   * @returns {*}
   */
  renderFooter = () => {
    const { status, files, oaInstanceId } = this.state;
    if (status === 1) {
      return [
        <Button key="222" type="primary" disabled={files.length === 0 || !oaInstanceId} onClick={this.onImport}>
          导入
        </Button>,
        <Button key="111" onClick={this.onCancel}>
          取消
        </Button>,
      ];
    } else {
      return (
        <Button type="primary" onClick={this.onCancel}>
          确定
        </Button>
      );
    }
  };

  /**
   * 点击上传事件
   */
  onImport = () => {
    const { files, productTypeId, isOverwrite, onlyOverwriteOffer, oaInstanceId } = this.state;
    const formData = new FormData();
    files.forEach((file) => {
      formData.append('file', file.file);
    });
    formData.append('oaInstanceId', oaInstanceId);
    formData.append('category', productTypeId);
    formData.append('isOverwrite', isOverwrite);
    formData.append('isOverWritePromotion', onlyOverwriteOffer);
    const xhr = new XMLHttpRequest();
    xhr.open('POST', urlBase + '/ProductImportController/importProduct.do');
    xhr.onload = () => {
      try {
        const result = JSON.parse(xhr.responseText);
        if (xhr.status === 200 && result.resultCode === 0) {
          this.setState({ status: 2 });
          //自动刷新产品导入历史tab
          this.props.dispatch({
            type: 'main/refreshTab',
            payload: {
              id: '/ProductImportHistory',
            },
          });
        } else {
          showMessage(result.resultMessage);
        }
        this.setState({ loading: false });
      } catch (err) {
        showMessage('系统错误');
        this.setState({ loading: false });
      }
    };
    xhr.onerror = () => {
      showMessage('网络错误!');
      this.setState({ loading: false });
    };
    xhr.send(formData);
    this.setState({ loading: true });
  };

  /**
   * 取消
   */
  onCancel = () => {
    this.setState({ status: 1 });
    this.props.onClose();
  };

  render() {
    const { loading } = this.state;
    return (
      <Modal
        style={{ marginTop: -150 }}
        loading={loading}
        width={400}
        className={styles.customModal}
        title={this.getTitle()}
        onClose={this.props.onClose}
        footer={this.renderFooter()}
        closable
      >
        {this.renderBody()}
      </Modal>
    );
  }
}

export default connect(null, null)(ProductImportModal);
