/**
 * 可折叠搜索框
 * @property {string} title 标题
 * @property {[object]} dict 提供搜索条件选项的字典
 * @property {string} value 选中值
 * @property {function} onChange 选中一个搜搜条件时的回调
 */
import React from 'react';
import ReactDOM from 'react-dom';
import { Icon } from 'antd';
import { objectDeepEquals } from 'COMPONENT/utils/basic/obj';
import styles from './CollapsableSearchBox.less';

class CollapsableSearchBox extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      moreBtnShow: false, //更多按钮显示状态
      expand: false, //展开状态
    };
  }

  componentDidUpdate(prevProps) {
    if (!objectDeepEquals(this.props.dict, prevProps.dict)) {
      this.refreshMoreBtnStatus();
    }
  }

  /**
   * 刷新更多按钮的显示状态
   */
  refreshMoreBtnStatus = () => {
    if (!this.state.moreBtnShow) {
      setTimeout(() => {
        const $content = this.$content;
        const $inner = this.$inner;
        console.log($inner, $inner.offsetHeight);
        if ($inner.offsetHeight > 32) {
          this.setState({ moreBtnShow: true });
        }
      }, 500);
    }
  };

  render() {
    const { title, dict, value, onChange } = this.props;
    const { moreBtnShow, expand } = this.state;

    return (
      <div className={styles.wrap}>
        <div className={styles.title}>{title}</div>
        <div
          className={styles.content + ' ' + (moreBtnShow && expand ? styles.expand : '')}
          ref={(c) => (this.$content = ReactDOM.findDOMNode(c))}
        >
          <div className={styles.inner} ref={(c) => (this.$inner = ReactDOM.findDOMNode(c))}>
            {dict &&
              dict.map((opt) => {
                const { val, text } = opt;
                return (
                  <div
                    key={val}
                    className={styles.item + ' ' + (value === val ? styles.active : '')}
                    onClick={() => onChange(value === val ? null : val)}
                  >
                    {text}
                  </div>
                );
              })}
          </div>
        </div>
        <div
          className={styles.moreBtn}
          style={{ display: moreBtnShow ? 'block' : 'none' }}
          onClick={() => this.setState({ expand: !expand })}
        >
          <span>更多</span>
          <Icon type={expand ? 'up' : 'down'} />
        </div>
      </div>
    );
  }
}

export default CollapsableSearchBox;
