import React from 'react';
import { connect } from 'dva';
import { STORE_NAME, FRONT_PAGER, formatTabAttribute } from '../../store/config';
import List from 'COMPONENT/List';
import { formatIndex, formatRanking, formatAmount, formatUserName } from './util';

const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;

class ListBlock extends React.Component {
  constructor(props) {
    super(props);

    this.loadList = this.loadList.bind(this);
    this.onPageChange = this.onPageChange.bind(this);
    this.onPageChangeOnly = this.onPageChangeOnly.bind(this);
    this.formatColumn = this.formatColumn.bind(this);
  }

  /**
   * 请求列表
   * @param changeParam
   * @return {boolean}
   */
  loadList(changeParam = {}) {
    const { paneKey, dispatch } = this.props;

    dispatch({
      type: `${STORE_NAME}/loadList`,
      payload: {
        paneKey,
        ...changeParam,
      },
    });
  }

  /**
   * 后端分页，加载数据
   * @param current
   * @param pageSize
   */
  onPageChange({ current, pageSize }) {
    const changeParam = { pageNo: current, pageSize };
    this.loadList(changeParam);
  }

  /**
   * ATTENTION 前端分页
   * @param current
   * @param pageSize
   */
  onPageChangeOnly({ current, pageSize }) {
    const { paneKey, dispatch } = this.props;
    const { storeKey } = formatTabAttribute(paneKey);
    const changeParam = { pageNo: current, pageSize };

    let payload = {};
    payload[storeKey] = {
      ...changeParam,
    };

    dispatch({
      type: `${STORE_NAME}/updateState`,
      payload: { ...payload },
    });
  }

  /**
   * 格式化表格
   * @param paneKey
   */
  formatColumn(paneKey = '1') {
    const indexTitle =
      (paneKey === '1' && '咨询业绩排名') ||
      (paneKey === '2' && '学管业绩排名') ||
      (paneKey === '3' && '学习中心现金流排名');
    const teachTitle = (paneKey === '1' && '咨询师') || (paneKey === '2' && '学管师') || (paneKey === '3' && '');
    const amountTitle =
      (paneKey === '1' && '业绩金额') || (paneKey === '2' && '业绩金额') || (paneKey === '3' && '现金流金额');
    const rankTitle = (paneKey === '3' && '现金流环比') || '业绩环比';

    let leftColumn = [
      {
        title: indexTitle,
        dataIndex: 'number',
        key: 'number',
        width: COL_WIDTH_M,
        render: (text, record, index) => {
          return formatIndex(record, this.props.user);
        },
      },
    ];

    let centerColumn = [];
    if (paneKey === '1' || paneKey === '2') {
      centerColumn = [
        {
          title: teachTitle,
          dataIndex: 'userName',
          key: 'userName',
          width: COL_WIDTH_L,
          lines: 2,
          render: (text, record, index) => {
            return formatUserName(record, this.props.user);
          },
        },
      ];
    }

    let rightColumn = [
      { title: '所属分公司', dataIndex: 'branchName', key: 'branchName', width: COL_WIDTH_L, lines: 2 },
      { title: '所属学习中心', dataIndex: 'campusName', key: 'campusName', width: COL_WIDTH_L, lines: 2 },
      { title: amountTitle, dataIndex: 'amount', key: 'amount', width: COL_WIDTH_L, lines: 2 },
      {
        title: '排名环比',
        dataIndex: 'rankingUpdown',
        key: 'rankingUpdown',
        width: COL_WIDTH_L,
        lines: 2,
        render: (text, record, index) => {
          const { rankingUpdown, rankingDelta } = record;
          return formatRanking(rankingUpdown, rankingDelta);
        },
      },
      {
        title: rankTitle,
        dataIndex: 'amountUpdown',
        key: 'amountUpdown',
        width: COL_WIDTH_L,
        lines: 2,
        render: (text, record, index) => {
          const { amountUpdown, amountDelta } = record;
          return formatAmount(amountUpdown, amountDelta);
        },
      },
    ];

    return leftColumn.concat(centerColumn, rightColumn);
  }

  render() {
    const { paneKey } = this.props;
    const { storeKey } = formatTabAttribute(paneKey);
    const tabStore = this.props[storeKey];
    const { listLoading, list, total, pageNo, pageSize, pageSizeOptions } = tabStore || {};
    const columns = this.formatColumn(paneKey);

    return (
      <List
        dataSource={list || []}
        columns={columns}
        loading={listLoading}
        pagination={{
          current: pageNo,
          total: total || 0,
          pageSize,
          pageSizeOptions,
        }}
        onChange={FRONT_PAGER ? this.onPageChangeOnly : this.onPageChange}
      />
    );
  }
}

const mapStateToProps = (state) => {
  const { user } = state.account;
  const { tab_one, tab_two, tab_three } = state[STORE_NAME];

  return {
    user,
    tab_one,
    tab_two,
    tab_three,
  };
};

export default connect(mapStateToProps)(ListBlock);
