import style from './index.less';

/**
 * 格式化排名
 * @param record
 * @param user
 * @return {string|*}
 */
export function formatIndex(record, user) {
  const { userName: uName, userId: uId, rank } = record;
  const { userId, name } = user;

  if (!user) {
    return rank;
  }

  if (uId === userId || uName === name) {
    return (
      <div className={style.index_wrap}>
        <span className={style.index_wrap_text}>{rank}</span>
      </div>
    );
  }

  return rank;
}

/**
 * 格式化咨询师名称
 * @param record
 * @param user
 * @return {*}
 */
export function formatUserName(record, user) {
  const { userName: uName, userId: uId } = record;
  const { userId, name } = user;

  if (!user) {
    return uName;
  }

  if (uId === userId || uName === name) {
    return (
      <div>
        {uName}
        <span className={style.color_red}> (本人)</span>
      </div>
    );
  }

  return uName;
}

/**
 * 格式化颜色字体
 * @param text
 * @return {string|*}
 */
export function formatColorText(text = '') {
  if (text === '上升') {
    return <span className={style.color_green}>{text}</span>;
  } else if (text === '下降') {
    return <span className={style.color_red}>{text}</span>;
  }

  return text;
}

/**
 * 排名环比
 * @param text
 * @param number
 * @return {string}
 */
export function formatRanking(text = '', number = 0) {
  if (!text) {
    return '-';
  }

  let num = Number(number || 0);
  let str = '';

  if (num > 0) {
    str = num + '名';
  }

  return (
    <div>
      <span>{formatColorText(text)}</span>
      <span> {str}</span>
    </div>
  );
}

/**
 * 业绩环比
 * @param text
 * @param number
 * @return {string}
 */
export function formatAmount(text = '', number = 0) {
  if (!text) {
    return '-';
  }

  const num = Number(number || 0) * 100;
  const str = parseInt(num) + '%';

  return (
    <div>
      <span>{formatColorText(text)}</span>
      <span> {str}</span>
    </div>
  );
}
