import React from 'react';
import { connect } from 'dva';
import { showExportButton, STORE_NAME } from './store/config';
import { Tabs } from 'antd';
import Page from 'COMPONENT/Page';
import Button from 'COMPONENT/Button';
import TabPaneBlock from './component/TabPaneBlock';
import style from './index.less';

const { TabPane } = Tabs;

class BranchTrackReport extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false,
    };

    this.initPage = this.initPage.bind(this);
    this.initDict = this.initDict.bind(this);
    this.destroy = this.destroy.bind(this);
    this.onTabChange = this.onTabChange.bind(this);
    this.onExportExcel = this.onExportExcel.bind(this);
  }

  componentDidMount() {
    this.initPage();
  }

  componentWillUnmount() {
    this.destroy();
  }

  async initPage() {
    this.setState({ loading: true });
    await this.initDict();
    this.setState({ loading: false });
  }

  initDict() {
    const { dispatch } = this.props;

    return new Promise((resolve, reject) => {
      dispatch({
        type: 'dict/initDict',
        payload: {
          list: ['branchCampusTree'],
          callback() {
            resolve();
            dispatch({ type: `${STORE_NAME}/initTabs` });
            dispatch({ type: `${STORE_NAME}/initPageSearch` });
          },
        },
      });
    });
  }

  /**
   * 页面销毁需要重置所有状态
   */
  destroy() {
    this.props.dispatch({ type: `${STORE_NAME}/resetPage` });
  }

  /**
   * 更新store
   * @param key 键
   * @param value 值
   */
  onTabChange(key, value) {
    this.props.dispatch({
      type: `${STORE_NAME}/updateState`,
      payload: { [key]: value },
    });
  }

  /**
   * 导出报表
   */
  onExportExcel() {
    this.props.dispatch({ type: `${STORE_NAME}/exportExcel` });
  }

  render() {
    const { loading } = this.state;
    const { btnAuth, tabs, tab_key } = this.props;
    const showExport = showExportButton(tab_key, btnAuth);

    return (
      <Page
        fitWindowHeight={true}
        loading={loading}
        route={this.props.route}
        className={style.article}
        headerToolBar={
          <div>
            {showExport && (
              <Button
                type="primary"
                onClick={this.onExportExcel}
                style={{ position: 'absolute', top: '50px', right: '30px', zIndex: 999 }}
              >
                导出
              </Button>
            )}
          </div>
        }
      >
        <Tabs
          activeKey={tab_key}
          onChange={(key) => {
            this.onTabChange('tab_key', key);
          }}
        >
          {tabs.map((item) => {
            const { name, key } = item;

            return (
              <TabPane tab={name} key={key}>
                <TabPaneBlock paneKey={key} />
              </TabPane>
            );
          })}
        </Tabs>
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const { btnAuths } = state.account;
  const { tabs, tab_key } = state[STORE_NAME];

  return {
    btnAuth: btnAuths,
    tabs,
    tab_key,
  };
};

export default connect(mapStateToProps)(BranchTrackReport);
