// 全局状态命名空间
export const STORE_NAME = 'BranchTrackReportTab';
// 前端分页开关
export const FRONT_PAGER = true;
// 所有tabs
export const TABS = [
  {
    name: '咨询师业绩排名',
    key: '1',
    auth: 'BRANCH_TRACK_REPORT_COUNSELOR',
  },
  {
    name: '学管师业绩排名',
    key: '2',
    auth: 'BRANCH_TRACK_REPORT_MANAGER',
  },
  {
    name: '学习中心现金流排名',
    key: '3',
    auth: 'BRANCH_TRACK_REPORT_CAMPUS',
  },
];

/**
 * 根据paneKey格式化tab的各项属性
 * @param paneKey
 * @return {*}
 */
export function formatTabAttribute(paneKey = '1') {
  let storeKey = 'tab_one'; // 状态key
  let showCounselor = true; // 是否显示咨询师
  let counselorName = ''; // 咨询师输入框描述
  let reportType = ''; // 报表类型 PerformanceRankingReportType
  let roleCode = []; // 角色

  switch (paneKey) {
    case '1':
      storeKey = 'tab_one';
      counselorName = '咨询师';
      reportType = 'CONSULTANT';
      roleCode = ['CONSULTOR', 'CONSULTOR_DIRECTOR'];
      break;
    case '2':
      storeKey = 'tab_two';
      counselorName = '学管师';
      reportType = 'STUMANAGER';
      roleCode = ['STUDY_MANAGER', 'STUDY_MANAGER_HEAD'];
      break;
    case '3':
      storeKey = 'tab_three';
      showCounselor = false;
      reportType = 'CONSULTANT';
      roleCode = ['CONSULTOR', 'CONSULTOR_DIRECTOR'];
      break;
    default:
      break;
  }

  return {
    storeKey,
    showCounselor,
    counselorName,
    reportType,
    roleCode,
  };
}

/**
 * 判断导出按钮权限
 * @param paneKey
 * @param btnAuth
 * @return {boolean}
 */
export function showExportButton(paneKey = '1', btnAuth = []) {
  if (!paneKey || !btnAuth || btnAuth.length <= 0) {
    return false;
  }

  return (
    (paneKey === '1' && btnAuth.indexOf('TRACK_REPORT_COUNSELOR_EXPORT') > -1) ||
    (paneKey === '2' && btnAuth.indexOf('TRACK_REPORT_MANAGER_EXPORT') > -1) ||
    (paneKey === '3' && btnAuth.indexOf('TRACK_REPORT_CAMPUS_EXPORT') > -1)
  );
}

/**
 * 月份value
 * @type {string[]}
 */
export const monthValueMap = ['JAN', 'FEB', 'MAR', 'APR', 'MAY', 'JUN', 'JUL', 'AUG', 'SEPT', 'OCT', 'NOV', 'DEC'];

/**
 * 获取当前月第一天
 * @param date
 * @returns {Date}
 */
export function getFirstDayOfMonth(date) {
  let newDate = new Date(date);

  newDate.setDate(1);

  return newDate;
}
