import { getBonusReport, getCashFlowReport, exportBonusReport, exportCashFlowReport } from 'SERVICE/report';
import { getDistinctUserGroupByCampus } from 'SERVICE/common';
import { formatTabAttribute } from './config';

/**
 * 根据分公司/学习中心获取咨询师、学管师
 * @param param JSON {orgId: '', job: ['ZXS', 'ZXZG']}
 */
export async function loadCounselorDict(param) {
  const apiParam = {
    showBelong: 1,
    ...param,
  };

  try {
    const response = await getDistinctUserGroupByCampus(apiParam);

    return response.map((item) => {
      return { val: item.userId, text: item.userName };
    });
  } catch (e) {
    return [];
  }
}

/**
 * 获取咨询师/学管师报表
 * @param param
 */
export async function loadCounselorList(param) {
  try {
    const response = await getBonusReport(param);
    return formatResponse(response);
  } catch (e) {
    return formatResponse({});
  }
}

/**
 * 获取现金流报表
 * @param param
 */
export async function loadCashList(param) {
  try {
    const response = await getCashFlowReport(param);
    return formatResponse(response);
  } catch (e) {
    return formatResponse({});
  }
}

/**
 * 组装咨询师/学管师API参数
 * @param tabStore
 * @param payload
 */
export function formatCounselorParam(tabStore = {}, payload = {}) {
  const { paneKey } = payload;
  const { reportType, roleCode } = formatTabAttribute(paneKey);

  const { val_branchId, val_campusId, val_counselor, val_startTime, val_endTime, pageNo, pageSize } = tabStore || {};
  const searchParam = {
    type: reportType,
    roleCodes: roleCode,
    branchId: val_branchId,
    campusId: val_campusId,
    startDate: val_startTime || '',
    endDate: val_endTime || '',
    userId: val_counselor, // 咨询师ID
    pageNo,
    pageSize,
  };

  return { ...searchParam, ...payload };
}

/**
 * 组装现金流API参数
 * @param tabStore
 * @param payload
 */
export function formatCashParam(tabStore = {}, payload = {}) {
  const { paneKey } = payload;
  const { reportType, roleCode } = formatTabAttribute(paneKey);

  const { val_branchId, val_campusId, val_startTime, val_endTime, pageNo, pageSize } = tabStore || {};

  const param = {
    type: reportType,
    roleCodes: roleCode,
    branchId: val_branchId,
    campusId: val_campusId,
    startDate: val_startTime || '',
    endDate: val_endTime || '',
    pageNo,
    pageSize,
  };

  return { ...param, ...payload };
}

/**
 * 格式化请求结果, 给列表添加key字段
 * @param response
 * @return {{total: (*|number), list: (*|Array)}}
 */
export function formatResponse(response = {}) {
  let data = response.data || [];
  let total = response.records || 0;

  data.forEach((item, index) => {
    item.key = index;
  });

  return {
    list: data,
    total: total,
  };
}

/**
 * 导出咨询师/学管师报表
 * @return {*}
 */
export function exportCounselorReport(param) {
  return exportBonusReport(param);
}

/**
 * 导出现金流报表
 * @param param
 */
export function exportCashReport(param) {
  return exportCashFlowReport(param);
}
