import { loginUser } from 'UTIL/common';
import { getFirstDayOfMonth } from './config';
import { momentToDate } from 'COMPONENT/utils/form';
import moment from 'moment';

const defaultTabs = [];
const defaultTabKey = '';

export const defaultTabSearch = {
  val_branchId: null, // 分公司
  val_campusId: null, // 学习中心
  val_counselor: null, // 咨询师
  val_startTime: null, // 起始时间
  val_endTime: null, // 结束时间
  disable_branch: false, // 分公司禁用
  counselorDict: null, // 咨询师字典
};
export const defaultTabList = {
  listLoading: false,
  list: [], // 列表数据
  total: 0, // 总记录数
  pageNo: 1, // 当前页码
  pageSize: 50, // 每页记录数
  pageSizeOptions: ['10', '30', '50', '100'],
};

export const defaultState = {
  tabs: defaultTabs,
  tab_key: defaultTabKey,
  tab_one: {
    ...defaultTabSearch,
    ...defaultTabList,
  },
  tab_two: {
    ...defaultTabSearch,
    ...defaultTabList,
  },
  tab_three: {
    ...defaultTabSearch,
    ...defaultTabList,
  },
};

/**
 * 计算初始化的搜索条件
 * @param state
 * @return {{val_branchId: null, disable_branch: boolean}}
 */
export function formatInitValue(state) {
  const user = state.account.user;
  const { branchCampusTree } = state.dict;

  const { brenchId } = user;
  const isCampus = loginUser.isCampusRole();
  const isBranch = loginUser.isBranchRole();
  const isGroup = loginUser.isGroupRole();

  let val_branchId = null;
  let disable_branch = false;

  if ((isCampus || isBranch) && brenchId) {
    val_branchId = brenchId;
    disable_branch = true;
  } else if (isGroup && branchCampusTree && branchCampusTree.length > 0) {
    val_branchId = branchCampusTree[0].val;
  }

  const date = new Date();
  const firstDate = getFirstDayOfMonth(date);
  const val_startTime = momentToDate(moment(firstDate));
  const val_endTime = momentToDate(moment(date));

  return {
    val_branchId,
    disable_branch,
    val_startTime,
    val_endTime,
  };
}
