/**
 * Created by Haolin on 2019/2/11.
 */
import React from 'react';
import { connect } from 'dva';
import style from './index.less';
import { Menu, Icon } from 'antd';
import noDataImg from 'ASSET/imgs/notDataDef.png';
import menuConfig from '../menuConfig.js';

const SubMenu = Menu.SubMenu;
const NoAction = () => {
  const prefix = 'NoAction';
  return (
    <div className={style[prefix]}>
      <img className={style[`${prefix}-img`]} src={noDataImg} alt="请从左侧菜单选择要导出的报表" />
      <p className={style[`${prefix}-text`]}>请从左侧菜单选择要导出的报表</p>
    </div>
  );
};

class Main extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      // 激活的根菜单(用于Menu)
      openKeys: [],
      // 激活的选项(用于Menu)
      selectedKeys: [],
    };
    // 根菜单项(用于subMenuChange)
    this.rootSubmenuKeys = [];
    // 激活选项对应的页面组件
    this.activeKeyComponent = null;
    // 菜单配置(支持3级菜单)
    this.menuConfig = menuConfig;
    this.buildMenu = this.buildMenu.bind(this);
    this.buildSubMenu = this.buildSubMenu.bind(this);
    // this.buildMenuItem = this.buildMenuItem.bind(this);
    this.subMenuChange = this.subMenuChange.bind(this);
    this.optionClick = this.optionClick.bind(this);
    this.getOptionComponent = this.getOptionComponent.bind(this);
    this.getObjVal = this.getObjVal.bind(this);
  }
  static classPrefix = 'Main';
  // 生成菜单栏
  buildMenu(item, authStr) {
    if (item.items) {
      return this.buildSubMenu(item, authStr);
    } else {
      return Main.buildMenuItem(item, authStr);
    }
  }
  buildSubMenu({ key, optionName, icon, items, auth }, authStr) {
    if (!auth || authStr.indexOf(auth) !== -1) {
      let title = (
        <span>
          {icon && <Icon type={icon} />}
          <span>{optionName}</span>
        </span>
      );
      return (
        <SubMenu key={key} title={title}>
          {items.map((item) => this.buildMenu(item, authStr))}
        </SubMenu>
      );
    } else {
      return '';
    }
  }
  static buildMenuItem({ key, optionName, auth }, authStr) {
    if (!auth || authStr.indexOf(auth) !== -1) {
      return <Menu.Item key={key}>{optionName}</Menu.Item>;
    } else {
      return '';
    }
  }
  // 子菜单项点击
  subMenuChange(openKeys) {
    const latestOpenKey = openKeys.find((key) => this.state.openKeys.indexOf(key) === -1);
    if (this.rootSubmenuKeys.indexOf(latestOpenKey) === -1) {
      this.setState({ openKeys });
    } else {
      this.setState({
        openKeys: latestOpenKey ? [latestOpenKey] : [],
      });
    }
  }
  // 菜单项点击
  optionClick({ item, key, keyPath }) {
    this.activeKeyComponent = this.getOptionComponent(keyPath.reverse());
    this.setState({ selectedKeys: [key] });
  }
  getOptionComponent(keyPath) {
    let option = this.getObjVal(this.menuConfig, keyPath);
    return option ? option.component : null;
  }
  getObjVal(arr, keyArr) {
    let key = keyArr.shift();
    let obj = arr.find((item) => item.key === key);
    if (keyArr.length > 0 && obj.items) {
      return this.getObjVal(obj.items, keyArr);
    } else {
      return obj;
    }
  }

  render() {
    const { btnAuth } = this.props;
    const { openKeys, selectedKeys } = this.state;
    const prefix = Main.classPrefix;

    return (
      <div className={style[prefix]}>
        <div className={style[`${prefix}-wrap`]}>
          <div className={style[`${prefix}-menu`]}>
            <Menu
              mode="inline"
              openKeys={openKeys}
              selectedKeys={selectedKeys}
              onOpenChange={this.subMenuChange}
              onClick={this.optionClick}
            >
              {this.menuConfig.map((item) => this.buildMenu(item, btnAuth))}
            </Menu>
          </div>
          <div className={style[`${prefix}-body`]}>{this.activeKeyComponent || <NoAction />}</div>
        </div>
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    btnAuth: state.account.btnAuths,
  };
};

export default connect(mapStateToProps)(Main);
