/**
 * Created by Haolin on 2019/2/12.
 */
import React from 'react';
import { connect } from 'dva';
// import style from './index.less';
import Form from '../../components/Form';
import FormItem from 'COMPONENT/FormItem';
import { DatePicker } from 'antd';
import Validator from 'COMPONENT/Validator';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import { exportStudentReturnFeeExcel } from 'SERVICE/exportFile';
import { cloneDeep } from 'lodash';
import TipsOfLimit from '../../components/TipsOfLimit';

const RangePicker = DatePicker.RangePicker;

class Refund extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false,
      momentRange: [null, null],
      dateRange: [null, null],
      campus: null,
      receiptType: null,
      // 表单校验相关
      errorValidator: {},
      checkValidator: true,
      formTip: false,
      formTipContent: '有必填项未填！',
    };
    this.init = this.init.bind(this);
    this.updateDateRange = this.updateDateRange.bind(this);
    this.updateCampus = this.updateCampus.bind(this);
    this.updateReceipt = this.updateReceipt.bind(this);
    this.reset = this.reset.bind(this);
    this.checkBeforeSubmit = this.checkBeforeSubmit.bind(this);
    this.submit = this.submit.bind(this);
    this.submitBtnTips = this.submitBtnTips.bind(this);
    this.exportNotice = this.exportNotice.bind(this);
    this.errorChange = this.errorChange.bind(this);
  }
  static classPrefix = 'Refund';
  componentDidMount() {
    this.init();
  }
  // 初始化
  async init() {
    this.setState({ loading: true });
    await this.initDict();
    this.setState({ loading: false });
  }
  initDict() {
    return new Promise((resolve, reject) => {
      this.props.dispatch({
        type: 'dict/initDict',
        payload: {
          list: ['branchCampus'],
          callback() {
            resolve();
          },
        },
      });
    });
  }
  // 更新日期范围
  updateDateRange(moments, dates) {
    this.setState({
      momentRange: moments,
      dateRange: dates,
    });
  }
  // 更新学习中心
  updateCampus(val) {
    this.setState({ campus: val });
  }
  // 更新收款类型
  updateReceipt(val) {
    this.setState({ receiptType: val });
  }
  // 重置
  reset() {
    this.setState({
      momentRange: [null, null],
      dateRange: [null, null],
      campus: null,
      receiptType: null,
    });
  }
  // 提交前校验
  checkBeforeSubmit() {
    let checkRequired = Object.values(this.state.errorValidator).every((item) => !Boolean(item));
    this.setState({ checkValidator: checkRequired });
    return checkRequired;
  }
  // 提交
  async submit() {
    this.setState({ loading: true });
    if (this.checkBeforeSubmit()) {
      // 校验通过
      let apiParams = {
        startDate: this.state.dateRange[0],
        endDate: this.state.dateRange[1],
        campusIdArr: this.state.campus,
      };
      const rsp = await exportStudentReturnFeeExcel(apiParams, true);
      if (rsp && rsp.resultCode === 0) {
        this.exportNotice();
        this.reset();
      }
    } else {
      // 校验不通过
      this.submitBtnTips();
    }
    this.setState({ loading: false });
  }
  // 提交错误提示
  submitBtnTips() {
    this.setState((prevState) => ({ formTip: !prevState.formTip }));
  }
  // 提示导出
  exportNotice() {
    this.props.dispatch({
      type: 'main/exportNotice',
    });
  }
  // 校验带Validator的表单项
  errorChange({ type, val }) {
    this.setState((prevState) => {
      let err = cloneDeep(prevState.errorValidator);
      err[type] = val;
      return { errorValidator: err };
    });
  }

  render() {
    const { branchCampusDict } = this.props;
    const { loading, momentRange, campus, receiptType, errorValidator, checkValidator, formTip, formTipContent } =
      this.state;
    const prefix = Refund.classPrefix;

    return (
      <TipsOfLimit>
        <Form
          loading={loading}
          onSubmit={this.submit}
          onReset={this.reset}
          showTip={formTip}
          tipContent={formTipContent}
          handleTip={this.submitBtnTips}
        >
          <FormItem name="选择日期" labelWidth={80} required>
            <Validator
              value={momentRange[0] && momentRange[1]}
              rules={['required']}
              silent={!Boolean(errorValidator.momentRange) || checkValidator}
              showStatus={true}
              showTips={false}
              onErrorChange={(val) => {
                this.errorChange({ type: 'momentRange', val });
              }}
            >
              <RangePicker
                placeholder={['开始时间', '结束时间']}
                value={momentRange}
                onChange={this.updateDateRange}
                format="YYYY-MM-DD"
                size="large"
                style={{ width: '100%' }}
              />
            </Validator>
          </FormItem>
          <FormItem name="选择学习中心" labelWidth={80}>
            <MultiSelect
              placeholder="不选时默认导出所有学习中心"
              showSearch
              dict={branchCampusDict}
              value={campus}
              onChange={this.updateCampus}
            />
          </FormItem>
        </Form>
      </TipsOfLimit>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    branchCampusDict: state.dict.branchCampus,
  };
};

export default connect(mapStateToProps)(Refund);
