/**
 * Created by Haolin on 2019/2/15.
 */
import React, { PropTypes } from 'react';
import { connect } from 'dva';
import style from './index.less';
import classNames from 'classnames';
import { Icon } from 'antd';

const OpenDrawerBtn = (props) => {
  return <div className={style['OpenDrawerBtn']}>{props.text}</div>;
};
OpenDrawerBtn.propTypes = {
  text: PropTypes.string,
};
const CloseDrawerBtn = (props) => {
  return (
    <div className={style['CloseDrawerBtn']}>
      <Icon type="right" />
    </div>
  );
};
const PopCover = (props) => {
  const prefix = 'PopCover';
  return (
    <div className={classNames(style[prefix], props.show ? style['active'] : '')}>
      {props.onClick && (
        <div className={style[`${prefix}-body`]} onClick={props.onClick}>
          {' '}
        </div>
      )}
    </div>
  );
};
PopCover.propTypes = {
  show: PropTypes.bool,
  onClick: PropTypes.func,
};

class Drawer extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      active: false,
    };
    this.toggleDrawer = this.toggleDrawer.bind(this);
  }
  static classPrefix = 'Drawer';
  static propTypes = {
    children: PropTypes.node,
    btnNode: PropTypes.oneOfType([PropTypes.string, PropTypes.node]),
    onOpen: PropTypes.func,
    onClose: PropTypes.func,
  };
  static defaultProps = {
    btnNode: '打开抽屉',
  };
  // 切换侧边抽屉开关
  toggleDrawer() {
    this.setState(
      (prevState) => ({
        active: !prevState.active,
      }),
      () => {
        if (this.state.active) {
          if (this.props.onOpen) this.props.onOpen();
        } else {
          if (this.props.onClose) this.props.onClose();
        }
      },
    );
  }

  render() {
    const { children, btnNode } = this.props;
    const { active } = this.state;
    const prefix = Drawer.classPrefix;

    return (
      <div className={style[prefix]}>
        <div className={classNames(style[`${prefix}-wrap`], active ? style['active'] : '')}>
          <div className={style[`${prefix}-body`]}>{children}</div>
          <div className={style[`${prefix}-btn`]} onClick={this.toggleDrawer}>
            <div className={style[`${prefix}-btn-wrap`]}>
              {active ? <CloseDrawerBtn /> : typeof btnNode === 'string' ? <OpenDrawerBtn text={btnNode} /> : btnNode}
            </div>
            <div className={style[`${prefix}-btn-blank`]}> </div>
          </div>
        </div>
        <PopCover show={active} onClick={this.toggleDrawer} />
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  return {};
};

export default connect(mapStateToProps)(Drawer);
