/**
 * Created by Haolin<haolinhom@gmail.com> on 2019/2/12.
 */
import { PropTypes } from 'react';
import style from './index.less';
import classNames from 'classnames';
import Button from 'COMPONENT/Button';
import Loading from 'COMPONENT/Loading';
import PopTip from 'COMPONENT/PopTip';

const Form = (props) => {
  const prefix = 'Form';
  return (
    <div
      className={classNames(style[prefix], props.className)}
      style={{ marginTop: props.top, width: props.width, ...props.style }}
    >
      <div className={style[`${prefix}-body`]}>{props.children}</div>
      <div className={style[`${prefix}-btn-wrap`]}>
        <div className={style[`${prefix}-btn`]}>
          <PopTip visible={props.showTip} content={props.tipContent} onClose={props.handleTip} icon="error">
            <Button type="primary" onClick={props.onSubmit}>
              提交
            </Button>
          </PopTip>
          {props.resetBtn && <Button onClick={props.onReset}>重置</Button>}
        </div>
      </div>
      <Loading loading={props.loading} />
    </div>
  );
};
Form.propTypes = {
  className: PropTypes.string,
  top: PropTypes.number,
  width: PropTypes.number,
  resetBtn: PropTypes.bool,
  onSubmit: PropTypes.func,
  onReset: PropTypes.func,
  style: PropTypes.object,
  loading: PropTypes.bool,
};
Form.defaultProps = {
  className: '',
  top: 80,
  width: 368,
  resetBtn: true,
  style: {},
  loading: false,
  showTip: false,
  tipContent: '提示！',
  handleTip: () => {},
};

export default Form;
