/**
 * Created by luoming on 2018/5/9
 */

import React from 'react';
import { connect } from 'dva';
import moment from 'moment';
import Page from 'COMPONENT/Page';
import List from 'COMPONENT/List';
import Tabs from 'COMPONENT/Tabs';
import SearchBar from 'COMPONENT/SearchBar';
import DatePicker from 'COMPONENT/DatePicker';
import Button from 'COMPONENT/Button';
import Input from 'COMPONENT/Input';
import Select from 'COMPONENT/Select';
import Popover from 'COMPONENT/Popover';
import Icon from 'COMPONENT/Icon';
import styles from './index.less';

import { loadCommonDict, loadBranchDict, loadCampusDict } from 'UTIL/dict';
import { getDetailsClass, getOverViewClass, getManagerDict, getTeacherDict } from 'SERVICE/followReport';

const Tab = Tabs.Tab;
const [COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL] = [93, 108, 183];
const roleDict = [
  { text: '班主任', val: 'STUDY_MANAGER' },
  { text: '老师', val: 'TEATCHER' },
];

const startDate = moment().subtract(30, 'days').format('YYYY-MM-DD');
const endDate = moment().format('YYYY-MM-DD');

class FollowReport extends React.Component {
  searchKey = { currentPage: 1, pageSize: 20 };
  dsearchKey = { currentPage: 1, pageSize: 20 };
  tags = 0;

  constructor(props) {
    super(props);
    let { brenchId } = this.props;
    if (brenchId === '000001') brenchId = '000002';
    this.state = {
      startDate,
      endDate,
      dstartDate: startDate,
      dendDate: endDate,
      branchId: brenchId,
      dbranchId: brenchId,
      list: [],
      detailsList: [],
    };
  }

  async componentDidMount() {
    let { branchId } = this.state;
    this.setState({ loading: true });
    const [gradeDict, branchDict, campusDict] = await Promise.all([
      loadCommonDict('STUDENT_GRADE'),
      loadBranchDict(),
      loadCampusDict(branchId),
      this.loadData(),
    ]);
    this.setState({ loading: false, gradeDict, branchDict, campusDict, dcampusDict: campusDict });
  }

  loadData = async (searchKey) => {
    let { branchId, campusId, startDate, endDate, gradeId, studentName, currentPage, pageSize } = this.state;
    if (searchKey) {
      currentPage = searchKey.currentPage;
      pageSize = searchKey.pageSize;
      if (JSON.stringify(this.searchKey) === JSON.stringify(searchKey)) {
        return;
      }
      this.searchKey = searchKey;
    } else {
      currentPage = 1;
      pageSize = 20;
    }
    const apiParams = {
      blCampusId: campusId,
      branchId,
      startTime: startDate,
      endTime: endDate,
      gradeId,
      studentName,
      page: currentPage,
      rows: pageSize,
    };
    this.setState({ loading: true });
    let { records, rows: data } = await getOverViewClass(apiParams);

    this.setState({ totalRecords: records, list: data, loading: false, currentPage, pageSize });
  };
  reset = () => {
    let { brenchId } = this.props;
    if (brenchId === '000001') brenchId = '000002';
    this.setState({ startDate, endDate, studentName: null, gradeId: null, branchId: brenchId, campusId: null });
  };
  goToFollowPage = (ids, record, file) => {
    const { dispatch } = this.props;
    const { student_id, student_name } = record;
    const params = {
      studentId: student_id,
      studentName: student_name,
      from: 'report',
      fileIds: ids,
    };
    if (ids) {
      dispatch({
        type: 'main/openPage',
        payload: { path: file ? '/StudentFile' : '/StudentFollowVisit', params, refresh: true },
      });
    }
  };
  changeBranch = async (val) => {
    this.setState({ branchId: val, campusDict: [], campusId: null });
    if (val) {
      let campusDict = await loadCampusDict(val);
      this.setState({ campusDict });
    }
  };
  getDisabledStartDate = (startDate) => {
    let { endDate } = this.state;
    if (!startDate || !endDate) {
      return false;
    }
    return moment(startDate).valueOf() > moment(endDate).valueOf();
  };
  getDisabledEndDate = (endDate) => {
    const { startDate } = this.state;
    if (!endDate || !startDate) {
      return false;
    }
    return moment(endDate).valueOf() < moment(startDate).valueOf();
  };

  getDisabledStartDated = (startDate) => {
    let { dendDate } = this.state;
    if (!startDate || !dendDate) {
      return false;
    }
    return moment(startDate).valueOf() > moment(dendDate).valueOf();
  };
  getDisabledEndDated = (endDate) => {
    const { dstartDate } = this.state;
    if (!endDate || !dstartDate) {
      return false;
    }
    return moment(endDate).valueOf() < moment(dstartDate).valueOf();
  };

  tabsChange = async (activeKey) => {
    const { detailsList } = this.state;
    if (activeKey === 1 && this.tags === 0) {
      this.setState({ loading: true });
      let [{ value: managerObj }, teacherDict, subjectDict] = await Promise.all([
        getManagerDict({ roleCode: 'STUDY_MANAGER,STUDY_MANAGER_HEAD' }),
        getTeacherDict({ roleCode: 'TEATCHER', term: '' }),
        loadCommonDict('SUBJECT'),
        this.loadDetailData(),
      ]);
      this.tags = 1;
      let managerArray = Object.keys(managerObj);
      let managerDict = managerArray.map((v, k) => {
        return { text: managerObj[v], val: v };
      });
      teacherDict = teacherDict.map((v, k) => {
        return { val: v.value, text: v.label };
      });
      this.setState({ loading: false, managerDict, teacherDict, subjectDict });
    }
  };

  loadDetailData = async (searchKey) => {
    let {
      dbranchId,
      dcampusId,
      dstartDate,
      dendDate,
      dgradeId,
      dstudentName,
      dcurrentPage,
      dpageSize,
      roleId,
      studyManagerId,
      subjectId,
      teacherId,
      miniClassName,
    } = this.state;
    if (searchKey) {
      dcurrentPage = searchKey.currentPage;
      dpageSize = searchKey.pageSize;
      if (JSON.stringify(this.searchKey) === JSON.stringify(searchKey)) {
        return;
      }
      this.dsearchKey = searchKey;
    } else {
      dcurrentPage = 1;
      dpageSize = 20;
    }
    this.setState({ loading: true });
    const apiParams = {
      blCampusId: dcampusId,
      branchId: dbranchId,
      startTime: dstartDate,
      endTime: dendDate,
      gradeId: dgradeId,
      page: dcurrentPage,
      rows: dpageSize,
      role: roleId,
      studentName: dstudentName,
      studyManagerId,
      subjectId,
      teacherId,
      miniClassName,
    };
    let { records, rows: data } = await getDetailsClass(apiParams);
    this.setState({ dtotalRecords: records, detailsList: data, loading: false, dcurrentPage, dpageSize });
  };
  changeDBranch = async (val) => {
    this.setState({ dbranchId: val, dcampusDict: [], dcampusId: null });
    if (val) {
      let dcampusDict = await loadCampusDict(val);
      this.setState({ dcampusDict });
    }
  };
  resetDetails = () => {
    let { brenchId } = this.props;
    if (brenchId === '000001') brenchId = '000002';
    this.setState({
      dstartDate: startDate,
      dendDate: endDate,
      dstudentName: null,
      dgradeId: null,
      dbranchId: brenchId,
      dcampusId: null,
      subjectId: null,
      roleId: null,
      studyManagerId: null,
      teacherId: null,
      miniClassName: null,
    });
  };

  render() {
    let {
      loading,
      searchExpand,
      startDate,
      endDate,
      studentName,
      gradeDict,
      gradeId,
      branchDict,
      branchId,
      campusDict,
      campusId,
      currentPage = 1,
      totalRecords,
      pageSize = 20,
      list,

      detailsList,
      dbranchId,
      dcampusId,
      dcampusDict,
      dstartDate,
      dendDate,
      dgradeId,
      dstudentName,
      dcurrentPage = 1,
      dtotalRecords,
      dpageSize = 20,
      studyManagerId,
      subjectId,
      teacherId,
      roleId,
      teacherDict,
      managerDict,
      subjectDict,
      miniClassName,
    } = this.state;
    console.log(detailsList);

    const baseColumns = [
      {
        title: '跟进次数',
        dataIndex: 'followAcount',
        width: COL_WIDTH_M * 4,
        children: [
          {
            title: '电话',
            dataIndex: 'phone_follow_count',
            width: COL_WIDTH_M,
            sorter: (a, b) => a.phone_follow_count - b.phone_follow_count,
            render: (text, record, idx) => {
              const { phone_follow_ids } = record;
              return (
                <span onClick={(e) => this.goToFollowPage(phone_follow_ids, record)} className={styles.countNum}>
                  {text}
                </span>
              );
            },
          },
          {
            title: '微信',
            dataIndex: 'wechat_follow_count',
            width: COL_WIDTH_M,
            sorter: (a, b) => a.wechat_follow_count - b.wechat_follow_count,
            render: (text, record, idx) => {
              const { wechat_follow_ids } = record;
              return (
                <span onClick={(e) => this.goToFollowPage(wechat_follow_ids, record)} className={styles.countNum}>
                  {text}
                </span>
              );
            },
          },
          {
            title: '面谈',
            dataIndex: 'facetalk_follow_count',
            width: COL_WIDTH_M,
            sorter: (a, b) => a.facetalk_follow_count - b.facetalk_follow_count,
            render: (text, record, idx) => {
              const { facetalk_follow_ids } = record;
              return (
                <span onClick={(e) => this.goToFollowPage(facetalk_follow_ids, record)} className={styles.countNum}>
                  {text}
                </span>
              );
            },
          },
          {
            title: '总次数',
            dataIndex: 'total_follow_count',
            width: COL_WIDTH_M,
            sorter: (a, b) => a.total_follow_count - b.total_follow_count,
            render: (text, record, idx) => {
              const { total_follow_ids } = record;
              return (
                <span onClick={(e) => this.goToFollowPage(total_follow_ids, record)} className={styles.countNum}>
                  {text}
                </span>
              );
            },
          },
        ],
      },
      {
        title: '档案数量',
        dataIndex: 'filesAcount',
        width: COL_WIDTH_M * 7,
        children: [
          {
            title: '五方会谈',
            dataIndex: 'five_meeting_file_count',
            width: COL_WIDTH_M,
            sorter: (a, b) => a.five_meeting_file_count - b.five_meeting_file_count,
            render: (text, record, idx) => {
              const { five_meeting_file_ids } = record;
              return (
                <span
                  onClick={(e) => this.goToFollowPage(five_meeting_file_ids, record, 'file')}
                  className={styles.countNum}
                >
                  {text}
                </span>
              );
            },
          },
          {
            title: '学习计划',
            dataIndex: 'study_plan_file_count',
            width: COL_WIDTH_M,
            sorter: (a, b) => a.study_plan_file_count - b.study_plan_file_count,
            render: (text, record, idx) => {
              const { study_plan_file_ids } = record;
              return (
                <span
                  onClick={(e) => this.goToFollowPage(study_plan_file_ids, record, 'file')}
                  className={styles.countNum}
                >
                  {text}
                </span>
              );
            },
          },
          {
            title: '家长会',
            dataIndex: 'parent_meeting_file_count',
            width: COL_WIDTH_M,
            sorter: (a, b) => a.parent_meeting_file_count - b.parent_meeting_file_count,
            render: (text, record, idx) => {
              const { parent_meeting_file_ids } = record;
              return (
                <span
                  onClick={(e) => this.goToFollowPage(parent_meeting_file_ids, record, 'file')}
                  className={styles.countNum}
                >
                  {text}
                </span>
              );
            },
          },
          {
            title: '试卷',
            dataIndex: 'exam_paper_file_count',
            width: COL_WIDTH_M,
            sorter: (a, b) => a.exam_paper_file_count - b.exam_paper_file_count,
            render: (text, record, idx) => {
              const { exam_paper_file_ids } = record;
              return (
                <span
                  onClick={(e) => this.goToFollowPage(exam_paper_file_ids, record, 'file')}
                  className={styles.countNum}
                >
                  {text}
                </span>
              );
            },
          },
          {
            title: '教案',
            dataIndex: 'teaching_plan_file_count',
            width: COL_WIDTH_M,
            sorter: (a, b) => a.teaching_plan_file_count - b.teaching_plan_file_count,
            render: (text, record, idx) => {
              const { teaching_plan_file_ids } = record;
              return (
                <span
                  onClick={(e) => this.goToFollowPage(teaching_plan_file_ids, record, 'file')}
                  className={styles.countNum}
                >
                  {text}
                </span>
              );
            },
          },
          {
            title: '其他',
            dataIndex: 'other_file_count',
            width: COL_WIDTH_M,
            sorter: (a, b) => a.other_file_count - b.other_file_count,
            render: (text, record, idx) => {
              const { other_file_ids } = record;
              return (
                <span onClick={(e) => this.goToFollowPage(other_file_ids, record, 'file')} className={styles.countNum}>
                  {text}
                </span>
              );
            },
          },
          {
            title: '总数量',
            dataIndex: 'total_file_count',
            width: COL_WIDTH_M,
            sorter: (a, b) => a.total_file_count - b.total_file_count,
            render: (text, record, idx) => {
              const { total_file_ids } = record;
              return (
                <span onClick={(e) => this.goToFollowPage(total_file_ids, record, 'file')} className={styles.countNum}>
                  {text}
                </span>
              );
            },
          },
        ],
      },
    ];
    const columns = [
      { title: '学生姓名', dataIndex: 'student_name', width: COL_WIDTH_M, fixed: 'left' },
      { title: '年级', dataIndex: 'grade_name', width: COL_WIDTH_M, fixed: 'left' },
      { title: '分公司', dataIndex: 'branch_name', width: COL_WIDTH_L, fixed: 'left' },
      { title: '学籍学习中心', dataIndex: 'campus_name', width: COL_WIDTH_L, fixed: 'left' },
      { title: '产品类型', dataIndex: 'product_type', width: COL_WIDTH_L },
      ...baseColumns,
    ];
    const columnsDetail = [
      { title: '学生姓名', dataIndex: 'student_name', width: COL_WIDTH_M, fixed: 'left' },
      { title: '年级', dataIndex: 'grade_name', width: COL_WIDTH_M, fixed: 'left' },
      { title: '分公司', dataIndex: 'branch_name', width: COL_WIDTH_L, fixed: 'left' },
      { title: '学籍学习中心', dataIndex: 'campus_name', width: COL_WIDTH_L, fixed: 'left' },
      { title: '产品类型', dataIndex: 'product_type', width: COL_WIDTH_L },
      { title: '班主任', dataIndex: 'study_manager_name', width: COL_WIDTH_M },
      { title: '老师', dataIndex: 'teacher_name', width: COL_WIDTH_M },
      { title: '科目', dataIndex: 'subject_name', width: COL_WIDTH_M },
      { title: '班课名称', dataIndex: 'miniClass_name', width: COL_WIDTH_XL },
      ...baseColumns,
    ];
    const title = ['跟进监控列表', '跟进回访明细', '档案明细'].map((v, k) => {
      return <span key={k}>{v}</span>;
    });
    return (
      <Page loading={loading} footer={false} route={this.props.route} title={false} className={styles.followReport}>
        <Tabs navAlign={'center'} className={styles.tabs} onChange={this.tabsChange}>
          <Tab title="总览">
            <SearchBar
              expand={searchExpand}
              onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
              onSearch={this.loadData}
              onReset={this.reset}
            >
              <DatePicker
                placeholder="开始日期"
                value={startDate}
                onChange={(val) => this.setState({ startDate: val })}
                disabledDate={(val) => this.getDisabledStartDate(val)}
              />
              <DatePicker
                placeholder="结束日期"
                value={endDate}
                onChange={(val) => this.setState({ endDate: val })}
                disabledDate={(val) => this.getDisabledEndDate(val)}
              />
              <Input
                placeholder="学生姓名"
                value={studentName}
                onChange={(e) => this.setState({ studentName: e.target.value })}
              />
              <Select
                placeholder="年级"
                dict={gradeDict}
                value={gradeId}
                onChange={(val) => this.setState({ gradeId: val })}
              />
              <Select placeholder="分公司" dict={branchDict} value={branchId} onChange={this.changeBranch} />
              <Select
                placeholder="学籍学习中心"
                dict={campusDict}
                value={campusId}
                onChange={(val) => this.setState({ campusId: val })}
              />
            </SearchBar>
            <List
              className={styles.pandect}
              dataSource={list}
              columns={columns}
              loading={false}
              pagination={{
                current: currentPage,
                total: totalRecords,
                pageSize,
              }}
              onChange={({ current, pageSize }) => this.loadData({ currentPage: current, pageSize })}
            />
          </Tab>
          <Tab title="明细">
            <SearchBar
              expand={searchExpand}
              onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
              onSearch={this.loadDetailData}
              onReset={this.resetDetails}
            >
              <DatePicker
                placeholder="开始日期"
                value={dstartDate}
                onChange={(val) => this.setState({ dstartDate: val })}
                disabledDate={(val) => this.getDisabledStartDated(val)}
              />
              <DatePicker
                placeholder="结束日期"
                value={dendDate}
                onChange={(val) => this.setState({ dendDate: val })}
                disabledDate={(val) => this.getDisabledEndDated(val)}
              />
              <Input
                placeholder="学生姓名"
                value={dstudentName}
                onChange={(e) => this.setState({ dstudentName: e.target.value })}
              />
              <Select
                placeholder="年级"
                dict={gradeDict}
                value={dgradeId}
                onChange={(val) => this.setState({ dgradeId: val })}
              />
              <Select placeholder="分公司" dict={branchDict} value={dbranchId} onChange={this.changeDBranch} />
              <Select
                placeholder="学籍学习中心"
                dict={dcampusDict}
                value={dcampusId}
                onChange={(val) => this.setState({ dcampusId: val })}
              />

              <Select
                placeholder="班主任"
                dict={managerDict}
                value={studyManagerId}
                showSearch
                onChange={(val) => this.setState({ studyManagerId: val })}
              />

              <Select
                placeholder="老师"
                dict={teacherDict}
                value={teacherId}
                showSearch
                onChange={(val) => this.setState({ teacherId: val })}
              />
              <Select
                placeholder="科目"
                dict={subjectDict}
                value={subjectId}
                onChange={(val) => this.setState({ subjectId: val })}
              />
              <Input
                placeholder="班课名称"
                value={miniClassName}
                onChange={(e) => this.setState({ miniClassName: e.target.value })}
              />
              <Select
                placeholder="角色数据筛选"
                dict={roleDict}
                value={roleId}
                onChange={(val) => this.setState({ roleId: val })}
              />
            </SearchBar>
            <List
              dataSource={detailsList}
              columns={columnsDetail}
              loading={false}
              className={styles.detail}
              pagination={{
                current: dcurrentPage,
                total: dtotalRecords,
                pageSize: dpageSize,
              }}
              onChange={({ current, pageSize }) => this.loadDetailData({ currentPage: current, pageSize })}
            />
          </Tab>
        </Tabs>
        {/*<div>*/}
        {/*<div className={'exportExcel'}>*/}
        {/*<Popover*/}
        {/*title={title}*/}
        {/*trigger={'click'}*/}
        {/*placement={'bottomLeft'}*/}
        {/*getPopupContainer={() => document.querySelector('.exportExcel')}*/}
        {/*>*/}
        {/*<Button>导出Excel</Button>*/}
        {/*</Popover>*/}
        {/*</div>*/}
        {/*</div>*/}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const { brenchId } = state.account.user;
  return { brenchId: brenchId || '000001' };
};
export default connect(mapStateToProps)(FollowReport);
