import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import List from 'COMPONENT/List';
import loader from './loader';
import TextLink from 'COMPONENT/TextLink';
import Button from 'COMPONENT/Button';
import loaders from 'PAGE/consultant/FollowCustomer/loaders';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

class FollowupRateDetail extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      listLoading: false,
      dataSource: [],
      total: 0,
      current: 1,
      pageSize: 20,
    };

    const { startDate, endDate, branchId, campusId, deliverTarget, resEntranceId } = this.props.params;

    this.params = {
      createDateStart: startDate,
      createDateEnd: endDate,
      branchId,
      campusId,
      deliverTarget,
      resEntranceId,
    };
  }

  /**
   * 初始化数据
   */
  init = async () => {
    const loadFunction = this.formatLoadFunction();
    const current = 1;
    const res = await loadFunction({
      ...this.params,
      rows: this.state.pageSize,
      page: current,
    });

    if (res && res.resultCode === 0) {
      this.setState({
        dataSource: res.rows,
        total: res.records,
        current: current,
      });
    }
  };

  /**
   * 根据页面参数判断使用哪一个接口，0跟进学生数详情接口跟其他所有接口不同
   * 1、dataType === 'ZERO' 0跟进学生数
   */
  formatLoadFunction = () => {
    const { dataType } = this.props.params;
    const isZero = this.isZeroType({ dataType });
    let loadFunction = loader.followRateReportNewDetails;

    if (isZero) {
      loadFunction = loader.followRateReportUnFollowDetails;
    }

    return loadFunction;
  };

  onChange = async ({ current = this.state.current, pageSize = this.state.pageSize }) => {
    const loadFunction = this.formatLoadFunction();

    this.setState({ listLoading: true });

    const res = await loadFunction({
      ...this.params,
      rows: pageSize,
      page: current,
    });

    this.setState({ listLoading: false });

    if (res && res.resultCode === 0) {
      this.setState({
        dataSource: res.rows,
        total: res.records,
        current: current,
        pageSize: pageSize,
      });
    }
  };

  componentDidMount() {
    this.setState({ loading: true });
    Promise.all([this.init()])
      .then(() => {
        this.setState({ loading: false });
      })
      .catch((e) => {
        this.setState({ loading: false });
      });
  }

  /**
   * 跳转到客户详情页
   * @param param JSON {id: '客户ID'，stuId：'学生ID', deliverTarget: 'deliverTargetId', type: '???'}
   */
  onPageToCustomerDetail = (param = {}) => {
    const { dispatch } = this.props;
    const params = { ...param };

    dispatch({
      type: 'main/openPage',
      payload: {
        path: '/CommonCusDetailPage',
        params,
        refresh: true,
        mirrorId: `ConsultantFollowCustomer/CusListSec/${param.type}/${param.id}/${param.stuId}/${
          param.detailEditable ? 'detail' : 'follow'
        }`,
      },
    });
  };

  /**
   * 获取
   * @param id 客户ID
   * @param stuId 学生ID
   */
  obtain = ({ id, stuId }) => {
    loaders.obtainCustomer({ id, stuId }).then(
      () => {
        showMessage('获取成功', 'success');
        this.onChange({ current: this.state.current, pageSize: this.state.pageSize });
      },
      (e) => {
        showMessage(e.text || '获取失败');
      },
    );
  };

  /**
   * 格式化列表column
   * 1、学生不在该咨询师名下则不能跟进
   */
  formatColumn = () => {
    const { userId } = this.props;
    const { dataType, deliverTarget, campusId } = this.props.params;
    const isZero = this.isZeroType({ dataType });

    let column = [
      { title: '学生姓名', dataIndex: 'studentName' },
      { title: '年级', dataIndex: 'gradeName' },
      { title: '资源入口', dataIndex: 'resEntranceName' },
      { title: '创建日期', dataIndex: 'createDate' },
    ];

    if (isZero) {
      column = column.concat([
        {
          title: '操作',
          dataIndex: 'id',
          render: (val, row) => {
            const { studentId, customerId, deliverTargetId, dealStatus } = row;
            const param = {
              id: customerId,
              stuId: studentId,
              campusId: campusId,
              deliverTarget: deliverTargetId,
              type: 'FollowupRate', // 不知有无使用
              detailEditable: false, // true 能编辑详情
              followEditable: true, // true 能编辑跟进记录
              isManager: false, // false 能跟进
            };
            const disabled = deliverTarget !== userId;
            const isFollow = dealStatus === 'STAY_FOLLOW';

            if (isFollow) {
              return (
                <Button
                  type="primary"
                  size="small"
                  disabled={disabled}
                  onClick={(e) => {
                    this.obtain(param);
                  }}
                >
                  获取
                </Button>
              );
            }

            return (
              <TextLink
                disabled={disabled}
                onClick={() => {
                  this.onPageToCustomerDetail(param);
                }}
              >
                跟进
              </TextLink>
            );
          },
        },
      ]);
    } else {
      column = column.concat([{ title: '学生状态', dataIndex: 'dealStatus' }]);
    }

    return column;
  };

  /**
   * 判断是否为总体的0跟进学生
   * @param dataType 'ZERO'
   * @return {boolean}
   */
  isZeroType = ({ dataType }) => {
    return dataType === 'ZERO';
  };

  render() {
    const { loading, listLoading, dataSource, total, current, pageSize } = this.state;
    const column = this.formatColumn();

    return (
      <Page loading={loading} route={this.props.route} fitWindowHeight>
        <List
          dataSource={dataSource}
          columns={column}
          pagination={{
            current,
            total,
            pageSize,
          }}
          loading={listLoading}
          onChange={this.onChange}
        />
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const user = state.account.user;
  const { userId } = user || {};
  const data = state.main.pageParams;
  const pathName = state.routing.locationBeforeTransitions.pathname || '';

  return {
    userId,
    params: data[pathName] || {},
  };
};

export default connect(mapStateToProps, null)(FollowupRateDetail);
