import React from 'react';
import Modal from 'COMPONENT/Modal';
import { loadBranchDict } from 'UTIL/dict';
import { Radio } from 'antd';
import styles from './styles.less';

const RadioGroup = Radio.Group;

class ExportModal extends React.Component {
  constructor(props) {
    super(props);

    const { organizationType, branchId } = this.props;
    this.state = {
      loading: true,
      defaultValue: '',
      options: [],
      value: organizationType !== 'GROUNP' ? branchId : null,
    };
  }

  componentDidMount() {
    this.setState({ loading: true });
    Promise.all([loadBranchDict()]).then(([branchDict]) => {
      const options = (branchDict || []).map((ele) => {
        return {
          value: ele.val,
          label: ele.text,
        };
      });
      this.setState({
        options,
        loading: false,
      });
    });
  }

  onChange = (e) => {
    this.setState({
      value: e.target.value,
    });
  };

  onConfirm = async () => {
    const { onConfirm } = this.props;
    this.setState({ loading: true });
    await onConfirm(this.state.value);
    this.setState({ loading: false });
  };

  render() {
    const { onClose } = this.props;
    const { loading, options, value } = this.state;
    return (
      <Modal
        loading={loading}
        title="请选择需要导出数据的分公司"
        onCancel={onClose}
        cancelText={'关闭'}
        onText={'确认导出'}
        onOk={this.onConfirm}
        okBtnShow={!!value}
        closable={true}
        widthSize="l"
      >
        <div>
          <RadioGroup options={options} onChange={this.onChange} value={value} className="custom-radio-group" />
        </div>
      </Modal>
    );
  }
}
export default ExportModal;
