import React from 'react';
import ReactDOM from 'react-dom';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import SearchBar from 'COMPONENT/SearchBar';
import List from 'COMPONENT/List';
import DatePicker from 'COMPONENT/DatePicker';
import Button from 'COMPONENT/Button';
import Select from 'COMPONENT/Select';
import Radio from 'COMPONENT/Radio';
import { Cascader } from 'antd';
import { loadBranchDict, loadCampusDict, dictAddOptionTitle } from 'UTIL/dict';
import { loadConsultantDict } from 'PAGE/common/loaders';
import loader from './loader';
import moment from 'moment';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import ExportModal from './exportModal';
import TextLink from 'COMPONENT/TextLink';

const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;
const COL_WIDTH_N = COL_WIDTH_M;
const COL_WIDTH_N_HEADER = 4 * COL_WIDTH_N;

const followRateDict = [
  { text: '25%以下', val: '0.25' },
  { text: '25%-49%', val: '25.50' },
  { text: '50%-74%', val: '50.75' },
  { text: '75%-99%', val: '75.100' },
  { text: '100%', val: '100.' },
];
const resEntranceObj = {
  DIRECT_VISIT: '直访',
  CALL_IN: '呼入',
  ON_LINE: '网络',
  TRANSFER: '转介绍',
  OUTSTANDING_VISIT: '拉访',
};
const resEntrance = Object.keys(resEntranceObj).map((ele) => {
  return {
    val: ele,
    text: resEntranceObj[ele],
  };
});

const validRoleIdList = ['4028818d46e1abf10146e1abf3fd0001'];
const invalidRoleIdList = ['ROL0000000008', 'ROL0000000077', 'ROL0000000015'];
const isValidRole = (validRoleIdList, roleIdList) => {
  return !!(roleIdList || []).filter((ele) => validRoleIdList.indexOf(ele) >= 0).length;
};
const isInvalidRole = (invalidRoleIdList, roleIdList) => {
  return !!(roleIdList || []).filter((ele) => invalidRoleIdList.indexOf(ele) >= 0).length;
};

class FollowupRate extends React.Component {
  constructor(props) {
    super(props);

    const { grounpId, grounpName, brenchId, brenchName, campusId, campusName, userId, name, organizationType, roleId } =
      this.props.user;

    this.branchObj = {};
    this.campusObj = {};
    this.deliverTargetObj = {};

    this.table = null;
    this.currentDate = moment();
    this.lastMonthFirst = moment().date(0).startOf('month');
    this.listData = {};

    this.cascaderSettings = {
      GROUNP: {
        defaultSearchVal: [grounpId],
        configs: [
          this.getGroupDictFactory(grounpId, grounpName),
          this.getBranchDictFactory(),
          this.getCampusDictFactory(),
          this.getDeliverTargetDictFactory(),
        ],
      },
      BRENCH: {
        defaultSearchVal: [grounpId, brenchId],
        configs: [
          this.getBranchDictFactory(brenchId, brenchName),
          this.getCampusDictFactory(),
          this.getDeliverTargetDictFactory(),
        ],
      },
      CAMPUS: {
        defaultSearchVal: [grounpId, brenchId, campusId],
        configs: [this.getCampusDictFactory(campusId, campusName), this.getDeliverTargetDictFactory()],
      },
      CAMPUS_CONSULTOR: {
        defaultSearchVal: [grounpId, brenchId, campusId, userId],
        configs: [this.getCampusDictFactory(campusId, campusName), this.getDeliverTargetDictFactory(userId, name)],
      },
    };

    let settingType = organizationType;
    if (
      organizationType === 'CAMPUS' &&
      isValidRole(validRoleIdList, roleId.split(',')) &&
      !isInvalidRole(invalidRoleIdList, roleId.split(','))
    ) {
      settingType = 'CAMPUS_CONSULTOR';
    }
    this.cascaderSetting = this.cascaderSettings[settingType];

    this.state = {
      loading: false, // boolean 页面加载状态
      searchExpand: false,
      position: null,

      startDate: this.currentDate.format('YYYY-MM-DD'),
      endDate: this.currentDate.format('YYYY-MM-DD'),
      cascaderOptions: [],
      cascaderVal: [],
      followRate: null,
      searchCascaderCondition: [],

      listLoading: false,
      rows: [],
      pageNo: 1,
      pageSize: 20,
      total: 0,
      exportModalVisible: false,
    };
  }

  componentDidMount() {
    this.init();
  }

  loadListData = async (params = {}, forceRequest) => {
    try {
      const { cascaderVal, startDate, endDate, followRate } = this.state;
      const defaultSearchVal = this.cascaderSetting.defaultSearchVal;
      const pageNo = params['pageNo'] || this.state.pageNo;
      const pageSize = params['pageSize'] || this.state.pageSize;
      const [grounpId, branchId, campusId, deliverTarget] = !cascaderVal.length
        ? defaultSearchVal
        : [...defaultSearchVal.slice(0, 4 - this.cascaderSetting.configs.length), ...cascaderVal];

      delete params['pageSize'];
      delete params['pageNo'];

      const apiParams = {
        startDate,
        endDate,
        ...params,
      };
      const [minFollowRate, maxFollowRate] = (followRate || '').split('.');
      if (minFollowRate) {
        apiParams['minFollowRate'] = minFollowRate;
      }
      if (maxFollowRate) {
        apiParams['maxFollowRate'] = maxFollowRate;
      }
      if (branchId !== undefined) {
        apiParams['branchId'] = branchId;
      }
      if (campusId !== undefined) {
        apiParams['campusId'] = campusId;
      }
      if (deliverTarget !== undefined) {
        apiParams['deliverTarget'] = deliverTarget;
      }

      let listData = this.listData[`${branchId}.${campusId}.${deliverTarget}`];
      if (forceRequest || !listData) {
        listData = await loader.getFollowRateReport(apiParams);
        this.listData = {};
        this.listData[`${branchId}.${campusId}.${deliverTarget}`] = listData;
      }

      this.setState({
        searchCascaderCondition: [grounpId, branchId, campusId, deliverTarget].filter((ele) => ele !== undefined),
      });

      return {
        rows: listData.slice((pageNo - 1) * pageSize, pageNo * pageSize),
        total: listData.length,
        pageNo: pageNo,
        pageSize: pageSize,
      };
    } catch (e) {
      return {
        rows: [],
        total: 0,
        pageNo: pageNo,
        pageSize: pageSize,
      };
    }
  };

  initCascaderOptions = (cascaderOptions, cascaderVal, i = 1) => {
    return Promise.all(
      cascaderOptions.map(async (ele) => {
        if (cascaderVal[i - 1] === ele.value) {
          if (i < cascaderVal.length) {
            let dict = await this.cascaderSetting.configs[i](ele.value);
            dict = await this.initCascaderOptions(dict, cascaderVal, ++i);
            ele.children = dict;
          }
        }
        return ele;
      }),
    );
  };

  init = async () => {
    this.setState({ loading: true });
    if (!this.cascaderSetting) {
      throw new Error(`invalid organization type`);
    }

    let cascaderOptions = await this.cascaderSetting.configs[0]();
    const defaultSearchVal = this.cascaderSetting.defaultSearchVal;
    let dafaultCascaderVal = defaultSearchVal.slice(4 - this.cascaderSetting.configs.length);
    cascaderOptions = await this.initCascaderOptions(cascaderOptions, dafaultCascaderVal);

    const list = await this.loadListData();
    this.setState({
      cascaderOptions,
      cascaderVal: dafaultCascaderVal,
      rows: list.rows,
      total: list.total,
      pageNo: list.pageNo,
      pageSize: list.pageSize,
    });
    this.setState({ loading: false });
  };

  getDisabledStartDate = (startDate) => {
    const { endDate } = this.state;
    if (!startDate) {
      return false;
    }

    const isInRange = moment(startDate).valueOf() <= this.currentDate;
    if (!endDate) {
      return !isInRange;
    }
    return !(
      moment(startDate).valueOf() <= moment(endDate).valueOf() &&
      isInRange &&
      moment(startDate).add(31, 'days').valueOf() > moment(endDate).valueOf()
    );
  };

  getDisabledEndDate = (endDate) => {
    const { startDate } = this.state;
    if (!endDate) {
      return false;
    }

    const isInRange = moment(endDate).valueOf() <= this.currentDate;
    if (!startDate) {
      return !isInRange;
    }
    return !(
      moment(startDate).valueOf() <= moment(endDate).valueOf() &&
      isInRange &&
      moment(startDate).add(31, 'days').valueOf() > moment(endDate).valueOf()
    );
  };

  getGroupDictFactory = (grounpId, grounpName) => {
    if (grounpId) {
      this.branchObj[grounpId] = grounpName;
    }
    return async () => {
      return [
        {
          value: grounpId,
          label: grounpName,
          isLeaf: false,
        },
      ];
    };
  };

  getBranchDictFactory = (branchId, branchName) => {
    if (branchId) {
      this.branchObj[branchId] = branchName;
    }
    return async (groupId) => {
      if (branchId) {
        return [
          {
            value: branchId,
            label: branchName,
            isLeaf: false,
          },
        ];
      }
      const dict = [{ value: '', label: '全部分公司' }].concat(
        ((await loadBranchDict()) || []).map((ele) => {
          this.branchObj[ele.val] = ele.text;
          return {
            value: ele.val,
            label: ele.text,
            isLeaf: false,
          };
        }),
      );
      return dict;
    };
  };

  getCampusDictFactory = (campusId, campusName) => {
    if (campusId) {
      this.campusObj[campusId] = campusName;
    }
    return async (branchId) => {
      if (campusId) {
        return [
          {
            value: campusId,
            label: campusName,
            isLeaf: false,
          },
        ];
      }

      const dict = [{ value: '', label: '全部学习中心' }].concat(
        ((await loadCampusDict(branchId)) || []).map((ele) => {
          this.campusObj[ele.val] = ele.text;
          return {
            value: ele.val,
            label: ele.text,
            isLeaf: false,
          };
        }),
      );
      return dict;
    };
  };

  getDeliverTargetDictFactory = (deliverTargetId, deliverTargetName) => {
    if (deliverTargetId) {
      this.deliverTargetObj[deliverTargetId] = deliverTargetName;
    }
    return async (campusId) => {
      if (deliverTargetId) {
        return [
          {
            value: deliverTargetId,
            label: deliverTargetName,
            isLeaf: true,
          },
        ];
      }

      const dict = [{ value: '', label: '全部咨询师' }].concat(
        dictAddOptionTitle((await loadConsultantDict({ orgId: campusId, includeLeft: true })) || []).map((ele) => {
          this.deliverTargetObj[ele.val] = ele.text;
          return {
            value: ele.val,
            label: ele.text,
          };
        }),
      );
      return dict;
    };
  };

  onSearch = async () => {
    this.setState({ listLoading: true });
    const { cascaderVal, startDate, endDate } = this.state;
    const params = { pageNo: 1 };
    if (!startDate || !endDate) {
      showMessage('查询失败：请补充查询时间范围！', 'error');
      this.setState({
        listLoading: false,
      });
      return;
    }
    const list = await this.loadListData(params, true);
    if (!cascaderVal.length) {
      this.reset();
    }

    this.setState({
      listLoading: false,
      rows: list.rows,
      total: list.total,
      pageNo: list.pageNo,
      pageSize: list.pageSize,
    });
  };

  onReset = () => {
    const defaultSearchVal = this.cascaderSetting.defaultSearchVal;
    this.setState({
      cascaderVal: defaultSearchVal.slice(4 - this.cascaderSetting.configs.length),
      startDate: this.currentDate.format('YYYY-MM-DD'),
      endDate: this.currentDate.format('YYYY-MM-DD'),
      followRate: null,
    });
  };

  onPageChange = async ({ current, pageSize }) => {
    this.setState({ listLoading: true });
    const list = await this.loadListData({ pageSize, pageNo: current });
    this.setState({
      listLoading: false,
      rows: list.rows,
      total: list.total,
      pageNo: list.pageNo,
      pageSize: list.pageSize,
    });
  };

  onExportClick = async (branchId) => {
    try {
      const { startDate, endDate } = this.state;

      if (!branchId) {
        throw new Error('请选择分公司');
      }
      const apiParams = { startDate, endDate, branchId };

      await loader.exportFile(apiParams);
      this.props.dispatch({
        type: 'main/exportNotice',
      });
      this.setState({ exportModalVisible: false });
    } catch (e) {
      showMessage('导出失败：' + e.message || e.text, 'error');
    }
  };

  loadCascaderData = async (selectedOptions) => {
    const targetOption = selectedOptions[selectedOptions.length - 1];
    targetOption.loading = true;
    if (!targetOption.children) {
      targetOption.children = await this.cascaderSetting.configs[selectedOptions.length](targetOption.value);
    }
    targetOption.loading = false;
    const cascaderOptions = [...this.state.cascaderOptions];
    this.setState({
      cascaderOptions,
    });
    return cascaderOptions;
  };

  jumpToHeader = (i) => {
    const tableDOM = ReactDOM.findDOMNode(this.table).querySelector('.ant-table-body');
    const offset = (i || 0) * COL_WIDTH_N_HEADER;
    tableDOM.scrollTo(offset, 0);
  };

  /**
   * 跳转详情
   * @param params 查询条件
   */
  goDetail = (params) => {
    const { dispatch } = this.props;
    params = { ...params };
    const { dataType } = params;
    const path = dataType && dataType === 'ZERO' ? '/FollowupRate/ZeroDetail' : '/FollowupRate/Detail';

    dispatch({
      type: 'main/openPage',
      payload: {
        path,
        params,
        refresh: true,
      },
    });
  };

  /**
   * 表头分组
   * @param dataIndexPrefix dataIndexPrefix === 'TOTAL' 时，跳转详情不能带入该参数
   */
  childrenColumns = (dataIndexPrefix) => {
    let column = [
      {
        title: '新增资源',
        dataIndex: `cntNew_${dataIndexPrefix}`,
        width: COL_WIDTH_N,
        render: (val, row) => {
          const { searchCascaderCondition, startDate, endDate } = this.state;
          const cascaderValLength = searchCascaderCondition.length;
          const { deliverTargetId, branchId, campusId } = row;

          if (+val && cascaderValLength >= 3) {
            return (
              <TextLink
                onClick={() => {
                  this.goDetail({
                    startDate,
                    endDate,
                    branchId,
                    campusId,
                    deliverTarget: deliverTargetId,
                    resEntranceId: this.formatEntranceId(dataIndexPrefix),
                  });
                }}
              >
                {val}
              </TextLink>
            );
          }

          return val || 0;
        },
      },
      {
        title: '及时跟进学生',
        dataIndex: `cntTimelyFollow_${dataIndexPrefix}`,
        width: COL_WIDTH_N,
        render: (val) => val || 0,
      },
      {
        title: '0跟进学生数',
        dataIndex: `cntUnFollow_${dataIndexPrefix}`,
        width: COL_WIDTH_N,
        render: (val, row) => {
          const { searchCascaderCondition, startDate, endDate } = this.state;
          const cascaderValLength = searchCascaderCondition.length;
          const { branchId, campusId, deliverTargetId } = row;

          if (+val && cascaderValLength >= 3) {
            return (
              <TextLink
                onClick={() => {
                  this.goDetail({
                    startDate,
                    endDate,
                    branchId,
                    campusId,
                    deliverTarget: deliverTargetId,
                    resEntranceId: this.formatEntranceId(dataIndexPrefix),
                    dataType: 'ZERO',
                  });
                }}
              >
                {val}
              </TextLink>
            );
          }

          return val || 0;
        },
      },
      {
        title: '及时跟进率',
        dataIndex: `followRate_${dataIndexPrefix}`,
        width: COL_WIDTH_N,
        render: (val) => (val ? `${val}%` : 0),
      },
    ];

    return column;
  };

  /**
   * 格式化 resEntranceId
   * @param dataIndexPrefix
   * @return {string}
   */
  formatEntranceId = (dataIndexPrefix) => {
    return dataIndexPrefix === 'TOTAL' ? '' : dataIndexPrefix;
  };

  columns = [
    {
      title: '所属分公司',
      dataIndex: 'branchId',
      fixed: 'left',
      width: COL_WIDTH_M,
      render: (val) => this.branchObj[val] || '',
    },
    {
      title: '所属学习中心',
      dataIndex: 'campusId',
      fixed: 'left',
      width: COL_WIDTH_M,
      render: (val) => this.campusObj[val] || '',
    },
    {
      title: '咨询师',
      dataIndex: 'deliverTargetId',
      fixed: 'left',
      width: COL_WIDTH_N,
      render: (val) => this.deliverTargetObj[val] || '',
    },
    {
      title: '总体',
      width: COL_WIDTH_N_HEADER,
      children: this.childrenColumns('TOTAL'),
    },
    ...resEntrance.map((ele) => {
      return {
        title: ele.text,
        width: COL_WIDTH_N_HEADER,
        children: this.childrenColumns(ele.val),
      };
    }),
  ];

  render() {
    const { btnAuth } = this.props;
    const { organizationType, brenchId: branchId } = this.props.user;
    const {
      loading,
      searchExpand,
      position,
      listLoading,
      startDate,
      endDate,
      cascaderOptions,
      cascaderVal,
      followRate,
      rows,
      pageNo,
      pageSize,
      total,
      exportModalVisible,
    } = this.state;
    return (
      <Page
        loading={loading}
        route={this.props.route}
        subHeader={
          <div>
            <div className="custom-serchbar">
              <SearchBar
                onSearch={this.onSearch}
                onReset={this.onReset}
                expand={searchExpand}
                onToggleExpand={(expand) => this.setState({ searchExpand: expand })}
              >
                <DatePicker
                  placeholder="开始时间"
                  disabledDate={this.getDisabledStartDate}
                  value={startDate}
                  onChange={(val) => this.setState({ startDate: val })}
                />
                <DatePicker
                  placeholder="结束时间"
                  disabledDate={this.getDisabledEndDate}
                  value={endDate}
                  onChange={(val) => this.setState({ endDate: val })}
                />
                <Cascader
                  allowClear={false}
                  options={cascaderOptions}
                  value={cascaderVal}
                  onChange={(val) => this.setState({ cascaderVal: val })}
                  changeOnSelect
                  size="large"
                  loadData={this.loadCascaderData}
                  style={{ width: '400px' }}
                  popupClassName="custom-cascader"
                />
                <Select
                  placeholder="请选择跟进率范围"
                  allowClear
                  dict={followRateDict}
                  value={followRate}
                  onChange={(val) => {
                    this.setState({ followRate: val });
                  }}
                />
              </SearchBar>
              {btnAuth.indexOf('EXPORT_FOLLOW_UP_RATE') > -1 && (
                <Button onClick={() => this.setState({ exportModalVisible: true })}>导出咨询师明细</Button>
              )}
            </div>
            <label>
              报表快速定位
              <span>
                <Radio
                  key={''}
                  checked={position === ''}
                  onChange={() => {
                    this.setState({ position: '' });
                    this.jumpToHeader(0);
                  }}
                  style={{ marginLeft: 20 }}
                >
                  总体
                </Radio>
                {resEntrance.map((ele, i) => {
                  return (
                    <Radio
                      key={ele.val}
                      checked={position === ele.val}
                      onChange={() => {
                        this.setState({ position: ele.val });
                        this.jumpToHeader(i + 1);
                      }}
                      style={{ marginLeft: 20 }}
                    >
                      {ele.text}
                    </Radio>
                  );
                })}
              </span>
            </label>
          </div>
        }
      >
        <div className={'custom-list ' + (rows.length === 0 && 'custom-list-empty')}>
          <List
            dataSource={rows}
            columns={this.columns}
            pagination={{
              current: pageNo,
              total,
              pageSize,
            }}
            loading={listLoading}
            bordered={true}
            onChange={this.onPageChange}
            ref={(c) => (this.table = c)}
          />
        </div>
        {exportModalVisible && (
          <ExportModal
            organizationType={organizationType}
            branchId={branchId}
            onClose={() => this.setState({ exportModalVisible: false })}
            onConfirm={this.onExportClick}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    user: state.account.user,
    btnAuth: state.account.btnAuths || [],
  };
};
export default connect(mapStateToProps)(FollowupRate);
