import React from 'react';

import { Form, Row, Col } from 'antd';
import { Select, Button, DateRangePicker } from 'COMPONENT';

import moment from 'moment';
import { connect } from 'dva';

import showMessage from 'SRC/components/utils/globalTips/showMessage';

import style from './index.less';

const { Item } = Form;
const initDateRange = [
  moment().startOf('month').startOf('day').format('YYYY-MM-DD'),
  moment().endOf('day').format('YYYY-MM-DD'),
];

class SearchPage extends React.Component {
  init = false;

  /**
   * 查询事件
   * @memberof SearchPage
   */
  handleSearch = () => {
    const {
      form: { getFieldsValue },
      onSearch,
    } = this.props;
    const values = getFieldsValue();
    const params = {
      ...values,
      startDate: values.dateRange[0],
      endDate: values.dateRange[1],
      basicOperationQueryLevelType: 'GROUP',
    };
    delete params.dateRange;
    if (
      moment(params.endDate).startOf('day').subtract(3, 'month').valueOf() >
      moment(params.startDate).startOf('day').valueOf()
    ) {
      return showMessage('起止时间跨度不能超过3个月', 'warning');
    }
    onSearch(params);
  };

  /**
   * 查询重置
   * @memberof SearchPage
   */
  handleReset = () => {
    const {
      form: { resetFields },
    } = this.props;
    resetFields();
  };

  render() {
    const {
      form: { getFieldDecorator },
    } = this.props;

    return (
      <Form layout="inline">
        <Row className={style.voucher_search} gutter={10}>
          <Col span={6}>
            <Item>
              {getFieldDecorator('dateRange', { initialValue: initDateRange })(
                <DateRangePicker placeholder={['开始时间', '结束时间']} allowClear={false} />,
              )}
            </Item>
          </Col>
          <Col span={3}>
            <Item>
              {getFieldDecorator('productType', { initialValue: '' })(
                <Select placeholder="产品类型" dict={this.props?.productTypeDict} />,
              )}
            </Item>
          </Col>
          <Col className={style.search_button} span={14}>
            <Button type="primary" onClick={this.handleSearch}>
              查询
            </Button>
            <Button onClick={this.handleReset}>重置</Button>
          </Col>
        </Row>
      </Form>
    );
  }
}

const mapStateToProps = (state) => {
  const {
    dict: { productType },
  } = state;
  return {
    productTypeDict: productType,
  };
};

export default connect(mapStateToProps)(Form.create()(SearchPage));
