import React from 'react';
import { connect } from 'dva';
import { Row, Col } from 'antd';
import { namespace } from '../../store/constant';
import { formatCurrency } from 'COMPONENT/utils/basic/num';
import { Modal, TextLink, List, Input, Select, TextArea } from 'COMPONENT';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
class AdjustmentModal extends React.Component {
  state = {
    cacheData: [],
    loading: false,
    adjustData: {},
    remarkEdit: false,
    _remark: '',
  };

  cacheEditId = [];

  columns = [
    {
      title: '项目调整',
      dataIndex: 'adjustType',
      key: 'adjustType',
      render: (t, r, i) => {
        if (!r.isEdit) return r.adjustTypeName;
        const { adjustDict } = this.props;
        return (
          <Select
            placeholder="请选择调整项目"
            dict={adjustDict}
            value={t}
            onChange={(val) => this.handleChange(val, 'adjustType', i)}
          />
        );
      },
    },
    {
      title: '调整数目',
      dataIndex: 'amount',
      key: 'amount',
      render: (t, r, i) => {
        if (!r.isEdit) return t;
        return <Input value={t} type="number" onChange={(e) => this.handleChange(e.target.value, 'amount', i)} />;
      },
    },
    {
      title: '详细说明',
      dataIndex: 'remark',
      key: 'remark  ',
      render: (t, r, i) => {
        if (!r.isEdit) return t;
        return <Input value={t} onChange={(e) => this.handleChange(e.target.value, 'remark', i)} />;
      },
    },
    {
      title: '操作',
      dataIndex: 'opertion',
      key: 'opertion',
      render: (_, r, index) => {
        const { type } = this.props;
        if (type === 0) return null;
        if (r.isEdit) return <TextLink onClick={() => this.handleSaveItem(r)}>保存</TextLink>;
        return (
          <div>
            <TextLink
              onClick={() => {
                this.cacheEditId.push(r.id);
                this.handleChange(true, 'isEdit', index);
              }}
            >
              修改
            </TextLink>
            &nbsp;&nbsp;
            <TextLink onClick={() => this.handlDelItem(r.id)}>删除</TextLink>
          </div>
        );
      },
    },
  ];

  componentDidMount() {
    this.findPaymentReceipt();
  }

  /**
   * 查询调整列表
   * @memberof AdjustmentModal
   */
  findPaymentReceipt = () => {
    const { data = {}, dispatch, onOk } = this.props;
    const { campusId, countMonth } = data;
    this.setState({ loading: true });
    dispatch({
      type: `${namespace}/findPaymentReceipt`,
      payload: {
        data: { campusId, countMonth },
        callback: (data = {}) => {
          onOk();
          this.setState({
            loading: false,
            cacheData: this.dataFiltering(data.adjustItemList || []),
            adjustData: data,
            _remark: data.remark,
          });
        },
      },
    });
  };

  /**
   * 对数据进行是否编辑状态处理，主要对单个列表保存删除后重新查询的状态处理
   * @memberof AdjustmentModal
   */
  dataFiltering = (list) => {
    const { type } = this.props;
    list.map((item) => (item.isEdit = this.cacheEditId.includes(item.id)));
    if (type === 1)
      list.push({
        id: '-1',
        adjustType: '',
        amount: '',
        remark: '',
        isEdit: true,
      });
    return list;
  };

  /**
   * 更新cacheData
   * @memberof AdjustmentModal
   */
  handleChange = (val, key, index) => {
    const { cacheData } = this.state;
    const item = cacheData[index];
    item[key] = val;
    cacheData[index] = item;
    this.setState({ cacheData });
  };

  /**
   * 删除行项
   * @memberof AdjustmentModal
   */
  handlDelItem = (itemId) => {
    this.props.dispatch({
      type: `${namespace}/deleteAdjustItem`,
      payload: {
        data: { itemId },
        callback: this.findPaymentReceipt,
      },
    });
  };

  /**
   * 编辑保存行
   * @memberof AdjustmentModal
   */
  handleSaveItem = (r) => {
    const { data = {}, dispatch } = this.props;
    const { campusId, countMonth } = data;
    const { adjustType, amount, remark, summaryId, id } = r;
    const { adjustData } = this.state;
    if (!adjustType) {
      showMessage('请选择调整项目');
      return;
    }
    if (!amount) {
      showMessage('请填写数目');
      return;
    }
    if (!remark) {
      showMessage('请填写备注');
      return;
    }
    const params = {
      id: id === '-1' ? undefined : id,
      campusId,
      countMonth,
      summaryId: summaryId || adjustData.id,
      adjustType,
      amount,
      remark,
    };
    dispatch({
      type: `${namespace}/editAdjustItem`,
      payload: {
        data: params,
        callback: () => {
          this.cacheEditId = this.cacheEditId.filter((id) => r.id !== id);
          this.findPaymentReceipt();
        },
      },
    });
  };

  /**
   * 保存备注信息
   * @memberof AdjustmentModal
   */
  handleSaveRemark = () => {
    const { data = {}, dispatch } = this.props;
    const { adjustData, _remark } = this.state;
    const { campusId, countMonth } = data;
    const params = {
      id: adjustData.id,
      campusId,
      countMonth,
      remark: _remark,
    };
    dispatch({
      type: `${namespace}/editAdjustSummary`,
      payload: {
        data: params,
        callback: () => {
          this.setState({ remarkEdit: false });
          this.findPaymentReceipt();
        },
      },
    });
  };

  render() {
    const { onCancel, data, type } = this.props;
    const {
      cacheData,
      loading,
      adjustData: { amount = 0 },
      remarkEdit,
      _remark,
    } = this.state;
    const { campusName, branchName, receiptStatusName, updateDate, countMonthStr, totalFinace } = data;

    return (
      <Modal
        title="收款凭证调整"
        widthSize="l"
        onOk={this.handleOk}
        onCancel={onCancel}
        okBtnShow={false}
        loading={loading}
        cancelText="关闭"
      >
        <Row>
          <Col span={8}>
            <span>{`分公司：${branchName}`}</span>
          </Col>
          <Col span={8}>
            <span>{`学习中心：${campusName}`}</span>
          </Col>
          <Col span={8}>
            <span>{`凭证月份：${countMonthStr}`}</span>
          </Col>
        </Row>
        <br />
        <Row>
          <Col span={8}>
            <span>{`凭证日期: ${updateDate}`}</span>
          </Col>
          <Col span={8}>
            <span>{`状态：${receiptStatusName}`}</span>
          </Col>
        </Row>
        <br />
        <List dataSource={cacheData} columns={this.columns} rowKey={(record) => record.id} />
        <br />
        <Row>
          <Col span={8}>
            <span>{`汇总金额：${formatCurrency(totalFinace)}`}</span>
          </Col>
          <Col span={8}>
            <span>{`调整总金额：${formatCurrency(amount)}`}</span>
          </Col>
          <Col span={8}>
            <span style={{ color: 'red' }}>{`调整后总金额：${formatCurrency(totalFinace + amount)}`}</span>
          </Col>
        </Row>
        <br />
        <Row gutter={10}>
          <Col span={2}>
            <span>凭证备注</span>
          </Col>
          <Col span={18}>
            <TextArea
              value={_remark}
              onChange={(e) => this.setState({ _remark: e.target.value })}
              rows={3}
              maxLength={50}
              disabled={!(type !== 0 && remarkEdit)}
            />
          </Col>
          <Col span={2}>
            {type === 1 &&
              (!remarkEdit ? (
                <TextLink onClick={() => this.setState({ remarkEdit: true })}>修改</TextLink>
              ) : (
                <TextLink onClick={() => this.handleSaveRemark()}>保存</TextLink>
              ))}
          </Col>
        </Row>
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  const {
    dict: { adjustProject },
  } = state;
  return {
    adjustDict: adjustProject,
  };
};

export default connect(mapStateToProps)(AdjustmentModal);
