import React from 'react';
import { Modal } from 'COMPONENT';
import { connect } from 'dva';
import style from './index.less';
import PrintDom from 'VIEW/PrintDom';
import { namespace } from '../../store/constant';
import { formatCurrency } from 'COMPONENT/utils/basic/num';
class FinancialVoucher extends React.Component {
  componentDidMount() {
    const { data = {}, dispatch } = this.props;
    const { campusId, countMonth } = data;
    const params = { campusId, countMonth };
    dispatch({
      type: `${namespace}/printReceiptMonthlyStatisticsStatistics`,
      payload: { data: params },
    });
  }

  handleOk = () => {
    const { type, dispatch, data, onOk, onCancel } = this.props;

    if (type === 0) {
      Modal.confirm({
        content: '是否审批通过',
        okText: '确认',
        cancelText: '取消',
        onOk: () => {
          const { campusId, countMonth } = data;
          const params = { campusId, monthDate: countMonth };
          dispatch({
            type: `${namespace}/auditPaymentReceiptCampusMonthlyStatistics`,
            payload: {
              data: params,
              callback: () => {
                onOk();
                onCancel();
              },
            },
          });
        },
      });
    } else {
      this.printDom && this.printDom.handlePrint();
    }
  };

  render() {
    const { onCancel, printFormData = {}, data, type } = this.props;
    const okText = ['审批通过', '打印'];
    return (
      <Modal
        title="收款凭证"
        widthSize="l"
        onOk={this.handleOk}
        onCancel={onCancel}
        okText={okText[type]}
        className={style.financial_voucher}
      >
        {type === 1 && (
          <PrintDom
            ref={(ref) => {
              this.printDom = ref;
            }}
            content={() => this.formDom}
          />
        )}
        <div
          ref={(ref) => {
            this.formDom = ref;
          }}
          className={style.form_body}
        >
          <p className={style.title}>收款凭证</p>
          <table cellSpacing="1" width="100%">
            <tr>
              <th>凭证所属</th>
              <td>{data.campusName}</td>
              <th>凭证类型</th>
              <td>收款凭证</td>
            </tr>
            <tr>
              <th>凭证月份</th>
              <td>{data.countMonthStr}</td>
              <th>凭证日期</th>
              <td>{data.updateDate}</td>
            </tr>
            <tr>
              <th>审批状态</th>
              <td>{data.receiptStatusName}</td>
              <th>最新审批人 </th>
              <td>{printFormData.currentAuditUserName}</td>
            </tr>
          </table>
          <br />
          <table cellSpacing="1" width="100%">
            <tr>
              <th className={style.table_title} colSpan={6}>
                收款
              </th>
            </tr>
            <tr>
              <th>本月新签收款</th>
              <td>{formatCurrency(data.newMoney)}</td>
              <th>本月续费收款</th>
              <td>{formatCurrency(data.reMoney)}</td>
              <th>本月新签网校产品收款</th>
              <td>{formatCurrency(data.liveNewMoney)}</td>
            </tr>
            <tr>
              <th>本月实际收款</th>
              <td>{formatCurrency(data.allMoney)}</td>
              <th>本月冲销金额</th>
              <td>{formatCurrency(data.washMoney)}</td>
              <th>本月收款小计</th>
              <td>{formatCurrency(data.totalMoney)}</td>
            </tr>
            <tr>
              <th className={style.table_title} colSpan={6}>
                退费
              </th>
            </tr>
            <tr>
              <th>正常退费金额</th>
              <td>{formatCurrency(data.refundMoney)}</td>
              <th>特殊退费金额</th>
              <td>{formatCurrency(data.specialRefundMoney)}</td>
              <th>线下特殊退费金额</th>
              <td>{formatCurrency(data.offlineSpecialRefundMoney)}</td>
            </tr>
            <tr>
              <th>实际退费金额</th>
              <td>{formatCurrency(data.totalRefundMoney)}</td>
              <th></th>
              <td></td>
              <th></th>
              <td></td>
            </tr>
            <tr>
              <th className={style.table_title} colSpan={6}>
                汇总
              </th>
            </tr>
            <tr>
              <th>现金流合计</th>
              <td>{formatCurrency(data.totalFinace)}</td>
              <th></th>
              <td></td>
              <th></th>
              <td></td>
            </tr>
          </table>
          <br />
          <p style={{ fontSize: '20px' }}>调整</p>
          <table cellSpacing="1" width="100%">
            <tr>
              <th>调整金额</th>
              <td>{formatCurrency(data.modifyMoney)}</td>
              <th>调整后总计 </th>
              <td>{formatCurrency(data.afterModifyMoney)}</td>
            </tr>
            <tr>
              <th className={style.table_title} colSpan={4}>
                备注信息
              </th>
            </tr>
            <tr>
              <td colSpan={4} className={style.remark}>
                {data.remark}
              </td>
            </tr>
          </table>
          <br />
          <table cellSpacing="1" width="100%">
            <tr>
              <th>学习中心确认</th>
              <th>财务初审</th>
              <th>分公司确认</th>
              <th>财务终审</th>
            </tr>
            <tr>
              <td>{printFormData.campusConfirmUserName}</td>
              <td>{printFormData.financeFirstAuditUserName}</td>
              <td>{printFormData.branchConfirmUserName}</td>
              <td>{printFormData.financeEndAuditUserName}</td>
            </tr>
            <tr>
              <td>{printFormData.campusConfirmTime}</td>
              <td>{printFormData.financeFirstAuditTime}</td>
              <td>{printFormData.branchConfirmTime}</td>
              <td>{printFormData.financeEndAuditTime}</td>
            </tr>
            <tr>
              <td>{printFormData.campusConfirmTypeName}</td>
              <td>{printFormData.financeFirstAuditTypeName}</td>
              <td>{printFormData.branchConfirmTypeName}</td>
              <td>{printFormData.financeEndAuditTypeName}</td>
            </tr>
            <tr>
              <td className={style.yz}>印章</td>
              <td className={style.yz}>印章</td>
              <td className={style.yz}>印章</td>
              <td className={style.yz}>印章</td>
            </tr>
          </table>
        </div>
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  const { printFormData } = state[namespace];
  return {
    printFormData,
  };
};

export default connect(mapStateToProps)(FinancialVoucher);
