import React from 'react';
import { Modal, TextLink, List } from 'COMPONENT';
import { connect } from 'dva';
import { Row, Col } from 'antd';
import { OLDPAGEBASE } from 'CONST/config';
import { browserHistory } from 'dva/router';
import { namespace } from '../../store/constant';

class VoucherAuditModel extends React.Component {
  state = {
    loading: false,
  };
  columns = [
    { title: '收款类型', dataIndex: 'channelName', key: 'channelName' },
    {
      title: '审核比例',
      dataIndex: 'validate',
      key: 'validate',
      render: (_, t) => {
        let percent = 0;
        if (t.unValidate + t.unAudit + t.validate === 0) percent = 1;
        else percent = t.validate / (t.unValidate + t.unAudit + t.validate);
        return Math.round(percent * 10000) / 100 + '%';
      },
    },
    {
      title: '操作',
      dataIndex: 'opertion',
      key: 'opertion',
      render: (_, r) => <TextLink onClick={() => this.handleOpertion(r)}>收款记录</TextLink>,
    },
  ];

  componentDidMount() {
    const { data = {}, dispatch } = this.props;
    const { campusId, countMonth } = data;
    const params = {
      campusId,
      receiptMonth: (countMonth + '').substr(0, 4) + '-' + (countMonth + '').substr(4),
    };

    dispatch({
      type: `${namespace}/getCampusFundsAuditRate`,
      payload: { data: params, callback: () => this.setState({ loading: false }) },
    });
  }

  /**
   * 审批
   * @memberof VoucherAuditModel
   */
  handleOk = () => {
    const { onOk } = this.props;
    onOk();
  };

  /**
   * 收款记录点击事件
   * @param 行数据
   * @memberof VoucherAuditModel
   */
  handleOpertion = (r) => {
    const { campusId, countMonth } = this.props.data;
    const { CHANNEL } = r;
    const receiptMonth = (countMonth + '').substr(0, 4) + '-' + (countMonth + '').substr(4);
    browserHistory.push({
      pathname: OLDPAGEBASE,
      query: {
        src: `function/contract/listFundHIstory.html?campusId=${campusId}&channel=${CHANNEL}&receiptMonth=${receiptMonth}`,
        title: '收款记录',
      },
    });
  };

  render() {
    const { onCancel, auditRateList, data = {}, auditOkCheck } = this.props;
    const { campusName, branchName, receiptStatusName, updateTime, countMonthStr } = data;
    const { loading } = this.state;

    return (
      <Modal
        title="收款凭证审核"
        widthSize="l"
        loading={loading}
        onOk={this.handleOk}
        onCancel={onCancel}
        okText="审核"
        okBtnDisabled={auditOkCheck}
      >
        <Row>
          <Col span={8}>
            <span>{`分公司：${branchName}`}</span>
          </Col>
          <Col span={8}>
            <span>{`学习中心：${campusName}`}</span>
          </Col>
          <Col span={8}>
            <span>{`凭证月份：${countMonthStr}`}</span>
          </Col>
        </Row>
        <br />
        <Row>
          <Col span={8}>
            <span>{`凭证日期: ${updateTime}`}</span>
          </Col>
          <Col span={8}>
            <span>{`状态：${receiptStatusName}`}</span>
          </Col>
        </Row>
        <br />
        <Row>
          <List dataSource={auditRateList} columns={this.columns} rowKey={(record) => record.id} />
        </Row>
        <span>备注：凭证月份收款审核比例需为100%方可点击审核，如不达标，请跳转至收款记录处理差异</span>
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  const { auditRateList, auditOkCheck } = state[namespace];
  return {
    auditRateList,
    auditOkCheck,
  };
};

export default connect(mapStateToProps)(VoucherAuditModel);
