/**
 * 现金流凭证
 * @author zero
 *  */
import React from 'react';
import { connect } from 'dva';
import SearchPage from './search';
import { namespace, exportUrl } from './store/constant';
import { formatCurrency } from 'COMPONENT/utils/basic/num';
import { Page, List, Button, TextLink } from 'COMPONENT';
import { exportExcel } from './utils';
import styles from './index.less';
import AdjustmentModal from './component/AdjustmentModal/index.js';
import VoucherAuditModel from './component/VoucherAuditModel/index.js';
import FinancialVoucher from './component/FinancialVoucher/index.js';
const { COL_WIDTH_XL } = List;

class cashFlowVoucher extends React.Component {
  state = {
    loading: false,
    pagination: {
      current: 1,
      pageSize: 999,
    },
    searchData: {},
    queryLevelType: 'GROUP',
    branchId: '',
    campusId: '',
    selectRow: {},
    voucherAuditModel: false,
    financialVoucherVisible: false,
    adjustModal: false,
    financialVoucherType: 0, // 收款凭证打开模式： 0 - 审批， 1 - 打印
    adjustType: 0, // 现金流凭证调整打开模式： 0 - 查看， 1 - 调整（即可编辑）
  };

  columns = [
    {
      title: '集团',
      dataIndex: 'groupName',
      key: 'groupName',
      render: (t) => (
        <TextLink onClick={() => this.getReceiptMonthlyStatisticsList({ queryLevelType: 'GROUP' })}>{t}</TextLink>
      ),
    },
    {
      title: '分公司',
      dataIndex: 'branchName',
      key: 'branchName',
      render: (t, r) => (
        <TextLink
          onClick={() => this.getReceiptMonthlyStatisticsList({ queryLevelType: 'BRANCH', branchId: r.branchId })}
        >
          {t}
        </TextLink>
      ),
    },
    {
      title: '学习中心',
      dataIndex: 'campusName',
      key: 'campusName',
      render: (t, r) => (
        <TextLink
          onClick={() => {
            r.id !== -1 && this.getReceiptMonthlyStatisticsList({ queryLevelType: 'CAMPUS', campusId: r.campusId });
          }}
        >
          {t}
        </TextLink>
      ),
    },
    { title: '学生', dataIndex: 'studentName', key: 'studentName' },
    { title: '本月新签收款', dataIndex: 'newMoney', key: 'newMoney', render: (t) => formatCurrency(t) },
    { title: '本月续费收款', dataIndex: 'reMoney', key: 'reMoney', render: (t) => formatCurrency(t) },
    {
      title: '本月新签网校产品收款',
      dataIndex: 'liveNewMoney',
      key: 'liveNewMoney',
      width: COL_WIDTH_XL,
      render: (t) => formatCurrency(t),
    },
    { title: '本月实际收款', dataIndex: 'allMoney', key: 'allMoney', render: (t) => formatCurrency(t) },
    { title: '本月冲销金额', dataIndex: 'washMoney', key: 'washMoney', render: (t) => formatCurrency(t) },
    { title: '本月收款小计', dataIndex: 'totalMoney', key: 'totalMoney', render: (t) => formatCurrency(t) },
    { title: '正常退费金额', dataIndex: 'refundMoney', key: 'refundMoney', render: (t) => formatCurrency(t) },
    {
      title: '超额退费金额',
      dataIndex: 'specialRefundMoney',
      key: 'specialRefundMoney',
      render: (t) => formatCurrency(t),
    },
    {
      title: '线下特殊退费金额',
      dataIndex: 'offlineSpecialRefundMoney',
      key: 'offlineSpecialRefundMoney',
      width: COL_WIDTH_XL,
      render: (t) => formatCurrency(t),
    },
    { title: '实际退费金额', dataIndex: 'totalRefundMoney', key: 'totalRefundMoney', render: (t) => formatCurrency(t) },
    { title: '现金流合计', dataIndex: 'totalFinace', key: 'totalFinace', render: (t) => formatCurrency(t) },
    { title: '调整金额', dataIndex: 'modifyMoney', key: 'modifyMoney', render: (t) => formatCurrency(t) },
    { title: '调整后合计', dataIndex: 'afterModifyMoney', key: 'afterModifyMoney', render: (t) => formatCurrency(t) },
    { title: '备注信息', dataIndex: 'remark', key: 'remark', width: COL_WIDTH_XL },
    { title: '凭证月份', dataIndex: 'countMonthStr', key: 'countMonthStr' },
    { title: '凭证日期', dataIndex: 'updateDate', key: 'updateDate', width: COL_WIDTH_XL },
    { title: '状态', dataIndex: 'receiptStatusName', key: 'receiptStatusName' },
    {
      title: '操作',
      dataIndex: 'opertion',
      key: 'opertion',
      width: COL_WIDTH_XL,
      render: (_, r) => this.getOpertion(r),
    },
  ];

  componentDidMount() {
    const { dispatch } = this.props;
    // 请求产品类型枚举
    dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['year', 'month', 'branch', 'branchCampus', 'evidenceAuditStatus', 'adjustProject'],
      },
    });
  }

  componentWillUnmount() {
    this.props.dispatch({ type: `${namespace}/initState` });
  }

  /**
   * 查询现金流列表
   * @param params: 参数列表
   * @memberof cashFlowVoucher
   */
  getReceiptMonthlyStatisticsList = (params = {}) => {
    const {
      pagination: { current, pageSize },
      searchData,
      queryLevelType,
    } = this.state;
    const _queryLevelType = params.queryLevelType || queryLevelType;
    const _branchId = params.branchId;
    const _campusId = params.campusId;
    const data = {
      ...searchData,
      page: current,
      rows: pageSize,
      ...params,
      branchId: _branchId,
      campusId: _campusId,
      queryLevelType: _queryLevelType,
    };
    this.setState({ loading: true });
    this.props.dispatch({
      type: `${namespace}/getReceiptMonthlyStatisticsList`,
      payload: {
        data,
        callback: () => {
          this.setState({ loading: false, queryLevelType: _queryLevelType, branchId: _branchId, campusId: _campusId });
        },
      },
    });
  };

  /**
   * 打印表单显示
   * @memberof cashFlowVoucher
   */
  handlePrint = (row) => {
    this.setState({
      financialVoucherVisible: true,
      financialVoucherType: 1,
      selectRow: row,
    });
  };

  /**
   * 打开调整弹窗
   * @memberof cashFlowVoucher
   */
  handleAdjustment = (row, type) => {
    this.setState({
      adjustModal: true,
      adjustType: type,
      selectRow: row,
    });
  };

  /**
   * 获取操作列 dom
   * @param row 行数据
   * @memberof cashFlowVoucher
   */
  getOpertion = (row) => {
    let jobId = '';
    let nextStatus = '';
    const { receiptStatus, isClosed, id } = row;
    const { queryLevelType } = this.state;
    if (id === -1 || queryLevelType === 'CAMPUS') return;
    if (id) {
      if (receiptStatus === 'NOT_AUDIT') {
        jobId = 'xqpzshz';
        nextStatus = 'CAMPUS_AUDITED';
      } else if (receiptStatus === 'CAMPUS_AUDITED') {
        jobId = 'pzcwcsz';
        nextStatus = 'FINANCE_FIRST_AUDITED';
      } else if (receiptStatus === 'FINANCE_FIRST_AUDITED') {
        jobId = 'fgspzshz';
        nextStatus = 'BRENCH_AUDITED';
      } else if (receiptStatus === 'BRENCH_AUDITED') {
        jobId = 'pzcwzsz';
        nextStatus = 'FINANCE_END_AUDITED';
      }
      const brench_job = row.branchId + '_' + jobId;
      const campus_job = row.campusId + '_' + jobId;
      const deptRoles = this.props.deptRoles || {};
      const flag =
        row.receiptStatus !== 'FINANCE_END_AUDITED' && (deptRoles[brench_job] === 1 || deptRoles[campus_job] === 1);
      const dom = [];
      if (flag) {
        const canDoCheck = isClosed == 1;
        if (receiptStatus === 'CAMPUS_AUDITED' || receiptStatus === 'BRENCH_AUDITED')
          dom.push(
            <TextLink key={`${id}adjustment`} onClick={() => this.handleAdjustment(row, 1)}>
              调整
            </TextLink>,
          );
        if (canDoCheck)
          dom.push(
            <TextLink
              key={`${id}audit`}
              onClick={() => {
                this.setState({ selectRow: row, voucherAuditModel: true });
              }}
            >
              审核
            </TextLink>,
          );
      } else {
        if (receiptStatus === 'FINANCE_END_AUDITED')
          dom.push(
            <TextLink key={`${id}print`} onClick={() => this.handlePrint(row)}>
              {' '}
              打印
            </TextLink>,
          );
        dom.push(
          <TextLink key={`${id}look`} onClick={() => this.handleAdjustment(row, 0)}>
            {' '}
            查看调整
          </TextLink>,
        );
      }
      return dom;
    }
    return <TextLink> 打印</TextLink>;
  };

  /**
   * 查询事件回调
   * @param searchData 查询数据
   * @memberof cashFlowVoucher
   */
  handleSearch = (searchData) => {
    this.setState({ searchData });
    this.getReceiptMonthlyStatisticsList(searchData);
  };

  /**
   * 导出回调
   * @memberof cashFlowVoucher
   */
  onExport = () => {
    const {
      searchData,
      queryLevelType,
      pagination: { current, pageSize },
      branchId,
      campusId,
    } = this.state;
    var data = {
      ...searchData,
      branchId,
      campusId,
      queryLevelType,
      numOfRecordsLimitation: 2000,
      page: current,
      rows: pageSize,
    };
    exportExcel({ apiUrl: exportUrl, data, fileName: '现金流凭证' });
  };

  /**
   * 分页 回调
   * @param current 页数
   * @param pageSize 页码
   * @memberof cashFlowVoucher
   */
  changePage = ({ current, pageSize }) => {
    const { campusId, branchId } = this.state;
    this.setState({ pagination: { current, pageSize } });
    this.getReceiptMonthlyStatisticsList({ page: current, rows: pageSize, campusId, branchId });
  };

  /**
   * 收款凭证审核 - 审核
   * @memberof cashFlowVoucher
   */
  handelVoucherAuditOK = () => {
    this.setState({
      voucherAuditModel: false,
      financialVoucherVisible: true,
      financialVoucherType: 0,
    });
  };

  render() {
    const { route, list, total, count_date } = this.props;
    const {
      loading,
      pagination: { current, pageSize },
      voucherAuditModel,
      selectRow,
      financialVoucherVisible,
      adjustModal,
      adjustType,
      financialVoucherType,
    } = this.state;
    return (
      <Page
        fitWindowHeight={true}
        route={route}
        className={styles.cash_flow_voucher}
        subHeader={<SearchPage onSearch={this.handleSearch} />}
        headerToolBar={<Button onClick={this.onExport}> 导出excel</Button>}
      >
        <div style={{ marginTop: '10px' }}>
          现金流凭证（含收费表明细）<span style={{ color: 'red' }}>{`最近更新时间：${count_date || ''}`}</span>
        </div>
        <List
          dataSource={list}
          columns={this.columns}
          loading={loading}
          pagination={{
            total,
            current,
            pageSize: list.length > 999 ? pageSize + 1 : pageSize,
            pageSizeOptions: ['999'],
          }}
          rowKey={(record) => record.id}
          onChange={this.changePage}
        />
        {voucherAuditModel && (
          <VoucherAuditModel
            data={selectRow}
            onOk={this.handelVoucherAuditOK}
            onCancel={() => {
              this.setState({ voucherAuditModel: false });
            }}
          />
        )}
        {financialVoucherVisible && (
          <FinancialVoucher
            data={selectRow}
            type={financialVoucherType}
            onOk={this.getReceiptMonthlyStatisticsList}
            onCancel={() => {
              this.setState({ financialVoucherVisible: false });
            }}
          />
        )}
        {adjustModal && (
          <AdjustmentModal
            data={selectRow}
            type={adjustType}
            onOk={this.getReceiptMonthlyStatisticsList}
            onCancel={() => {
              this.setState({ adjustModal: false });
            }}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const {
    account: {
      user: { deptRoles },
    },
  } = state;
  const { list, total, count_date } = state[namespace];
  return {
    list,
    total,
    count_date,
    deptRoles,
  };
};

export default connect(mapStateToProps)(cashFlowVoucher);
