import { runFunc } from 'UTIL/common';
import { namespace } from './constant';
import {
  deleteAdjustItem,
  editAdjustItem,
  editAdjustSummary,
  getReceiptMonthlyStatisticsList,
  getCampusFundsAuditRate,
  printReceiptMonthlyStatisticsStatistics,
  auditPaymentReceiptCampusMonthlyStatistics,
  findPaymentReceiptCampusMonthlyStatisticsAdjustSummary,
} from 'SERVICE/report';
const _state = {
  list: [], // 现金流凭证列表
  total: 0,
  count_date: '', // 最近更新时间
  auditRateList: [], // 凭证审核列表
  auditOkCheck: false, // 审核按钮是否disabled
  printFormData: {},
};

export default {
  namespace,
  state: _state,
  reducers: {
    updateState(state, { payload }) {
      return {
        ...state,
        ...payload,
      };
    },
    initState(state) {
      return {
        ...state,
        ..._state,
      };
    },
  },
  effects: {
    /**
     * 现金流凭证列表
     * @param {*} { payload }
     * @param {*} { call, put }
     */
    *getReceiptMonthlyStatisticsList({ payload }, { put }) {
      const { data, callback } = payload;
      const result = yield getReceiptMonthlyStatisticsList(data).catch(() => runFunc(callback));
      const { resultCode, records, rows = [] } = result;
      if (resultCode === 0 && rows) {
        if (rows.length > 0) {
          // 造一条合计数据
          const totalRow = {
            id: -1,
            campusName: '合计: ',
            studentId: null,
            newMoney: 0,
            allMoney: 0,
            washMoney: 0,
            totalMoney: 0,
            bonusMoney: 0,
            refundMoney: 0,
            specialRefundMoney: 0,
            offlineSpecialRefundMoney: 0,
            totalRefundMoney: 0,
            liveNewMoney: 0,
            totalFinace: 0,
            modifyMoney: 0,
            reMoney: 0,
            afterModifyMoney: 0,
          };
          rows.map((item) => {
            totalRow.newMoney += item.newMoney;
            totalRow.reMoney += item.reMoney;
            totalRow.liveNewMoney += item.liveNewMoney;
            totalRow.allMoney += item.allMoney;
            totalRow.washMoney += item.washMoney;
            totalRow.totalMoney += item.totalMoney;
            totalRow.refundMoney += item.refundMoney;
            totalRow.specialRefundMoney += item.specialRefundMoney;
            totalRow.offlineSpecialRefundMoney += item.offlineSpecialRefundMoney;
            totalRow.totalRefundMoney += item.totalRefundMoney;
            totalRow.totalFinace += item.totalFinace;
            totalRow.modifyMoney += item.modifyMoney;
            totalRow.afterModifyMoney += item.afterModifyMoney;
          });
          rows.push(totalRow);
        }

        yield put({
          type: 'updateState',
          payload: { total: records, list: rows, count_date: rows[0] ? rows[0].updateTime : '' },
        });
      }
      runFunc(callback);
    },
    /**
     * 审核列表
     * @param {*} { payload }
     * @param {*} { call, put }
     */
    *getCampusFundsAuditRate({ payload }, { call, put }) {
      const { data, callback } = payload;
      const list = yield call(getCampusFundsAuditRate, data);
      let auditOkCheck = false;
      if (list) {
        for (let i = 0, len = list.length; i < len; i += 1) {
          const { unAudit, unValidate, validate } = list[i];
          if ((unAudit && unAudit > 0) || validate !== unValidate + unAudit + validate) {
            auditOkCheck = true;
            break;
          }
          // if ()
        }
        yield put({
          type: 'updateState',
          payload: { auditRateList: list, auditOkCheck },
        });
      }
      runFunc(callback);
    },
    /**
     * 打印相关数据
     * @param {*} { payload }
     * @param {*} { call, put }
     */
    *printReceiptMonthlyStatisticsStatistics({ payload }, { call, put }) {
      const { data, callback } = payload;
      const result = yield call(printReceiptMonthlyStatisticsStatistics, data);
      yield put({
        type: 'updateState',
        payload: { printFormData: result ? result : {} },
      });
      runFunc(callback);
    },
    /**
     * 审核请求
     * @param {*} { payload }
     * @param {*} { call }
     */
    *auditPaymentReceiptCampusMonthlyStatistics({ payload }, { call }) {
      const { data, callback } = payload;
      try {
        const result = yield call(auditPaymentReceiptCampusMonthlyStatistics, data);
        if (result && result.resultCode === 0) runFunc(callback);
      } catch (e) {
        // Todo
      }
    },
    /**
     * 获取调整列表数据
     * @param {*} { payload }
     * @param {*} { call }
     */
    *findPaymentReceipt({ payload }) {
      const { data, callback } = payload;
      const result = yield findPaymentReceiptCampusMonthlyStatisticsAdjustSummary(data).catch(() =>
        runFunc(callback, {}),
      );
      const list = result || {};
      runFunc(callback, list);
    },
    /**
     * 删除调整项
     * @param {*} { payload }
     * @param {*} { call, put }
     */
    *deleteAdjustItem({ payload }, { call }) {
      const { data, callback } = payload;
      try {
        const result = yield call(deleteAdjustItem, data);
        if (result && result.resultCode === 0) runFunc(callback);
      } catch (e) {
        // Todo
      }
    },
    /**
     * 编辑保存行
     *
     * @param {*} { payload }
     * @param {*} { call }
     */
    *editAdjustItem({ payload }, { call }) {
      const { data, callback } = payload;
      try {
        const result = yield call(editAdjustItem, data);
        if (result) runFunc(callback);
      } catch (e) {
        // Todo
      }
    },
    /**
     * 保存凭证备注
     * @param {*} { payload }
     * @param {*} { call }
     */
    *editAdjustSummary({ payload }, { call }) {
      const { data, callback } = payload;
      try {
        const result = yield call(editAdjustSummary, data);
        if (result) runFunc(callback);
      } catch (e) {
        // Todo
      }
    },
  },
};
