import React from 'react';
import { Modal } from 'COMPONENT';
import { connect } from 'dva';
import style from './index.less';
import PrintDom from 'VIEW/PrintDom';
import { namespace } from '../../store/constant';
import { formatCurrency } from 'COMPONENT/utils/basic/num';
import { loginUser } from 'UTIL/common';

/**
 * 营收凭证报表
 * tips: 产品组'HL'对应为'AI'，名称不同ID相同
 */
const FinanceDirectorAuditType = {
  HUMAN: '人工审核',
  SYSTEM: '系统审核',
};

const tipZero = (m) => (m < 10 ? `0${m}` : m);

const formateDate = (timeStamp) => {
  if (!timeStamp) return '';
  let date = new Date(timeStamp);
  let y = date.getFullYear();
  let m = date.getMonth() + 1;
  let d = date.getDate();
  let h = date.getHours();
  let min = date.getMinutes();
  let s = date.getSeconds();
  return `${y}-${tipZero(m)}-${tipZero(d)} ${tipZero(h)}:${tipZero(min)}:${tipZero(s)}`;
};

class FinancialVoucher extends React.Component {
  constructor(props) {
    super(props);
    this.isTwoTeacher = loginUser.isTwoTeacherSym();
  }

  componentDidMount() {
    const { data = {}, dispatch } = this.props;
    const { campusId, countMonth } = data;
    const params = { campusId, countMonth };

    dispatch({
      type: `${namespace}/printReceiptMonthlyStatisticsStatistics`,
      payload: { data: params },
    });
  }

  handleOk = () => {
    const { type, dispatch, data, onOk, onCancel } = this.props;

    if (type === 0) {
      Modal.confirm({
        content: '是否审批通过',
        okText: '确认',
        cancelText: '取消',
        onOk: () => {
          const { campusId, countMonth } = data;
          const params = { campusId, monthDate: countMonth };

          dispatch({
            type: `${namespace}/auditIncomeCampusMonthlyStatistics`,
            payload: {
              data: params,
              callback: () => {
                onOk();
                onCancel();
              },
            },
          });
        },
      });
    } else {
      this.printDom && this.printDom.handlePrint();
    }
  };

  render() {
    const { onCancel, printFormData = {}, data, type } = this.props;
    const okText = ['审批通过', '打印'];
    return (
      <Modal
        title="收款凭证"
        widthSize="l"
        onOk={this.handleOk}
        onCancel={onCancel}
        okText={okText[type]}
        className={style.financial_voucher}
      >
        {type === 1 && (
          <PrintDom
            ref={(ref) => {
              this.printDom = ref;
            }}
            content={() => this.formDom}
          />
        )}
        <div
          ref={(ref) => {
            this.formDom = ref;
          }}
          className={style.form_body}
        >
          <p className={style.title}>营收凭证</p>
          <table cellSpacing="1" width="100%">
            <tbody>
              <tr>
                <th>凭证所属</th>
                <td>{data.campusName}</td>
                <th>凭证类型</th>
                <td>营收凭证</td>
              </tr>
              <tr>
                <th>凭证月份</th>
                <td>{data.countMonthStr}</td>
                <th>凭证日期</th>
                <td>{data.updateDate}</td>
              </tr>
              <tr>
                <th>审批状态</th>
                <td>{data.evidenceAuditStatusName}</td>
                <th>最新审批人</th>
                <td>{printFormData.currentAuditUserName}</td>
              </tr>
            </tbody>
          </table>
          <br />
          <table cellSpacing="1" width="100%">
            <tbody>
              <tr>
                <th className={style.table_title} colSpan={6}>
                  一对一营收
                </th>
              </tr>
              <tr>
                <th>1对1实收资金</th>
                <td>{formatCurrency(data.oneOnOneRealAmount)}</td>
                <th>1对1优惠资金</th>
                <td>{formatCurrency(data.oneOnOnePromotionAmount)}</td>
                <th>1对1实收历史冲销</th>
                <td>{formatCurrency(data.oneOnOneRealWashAmount)}</td>
              </tr>
              <tr>
                <th>1对1优惠历史冲销</th>
                <td>{formatCurrency(data.oneOnOnePromotionWashAmount)}</td>
                <th>1对1小计</th>
                <td>{formatCurrency(data.oneOnOneTotalAmount)}</td>
                <th></th>
                <td></td>
              </tr>
              <tr>
                <th className={style.table_title} colSpan={6}>
                  其他（1对1）营收
                </th>
              </tr>
              <tr>
                <th>其他（1对1）实收资金</th>
                <td>{formatCurrency(data.oneOnOneOtherRealAmount)}</td>
                <th>其他（1对1）优惠资金</th>
                <td>{formatCurrency(data.oneOnOneOtherPromotionAmount)}</td>
                <th>其他（1对1）实收历史冲销</th>
                <td>{formatCurrency(data.oneOnOneOtherRealWashAmount)}</td>
              </tr>
              <tr>
                <th>其他（1对1）优惠历史冲销</th>
                <td>{formatCurrency(data.oneOnOneOtherPromotionWashAmount)}</td>
                <th>其他（1对1）小计</th>
                <td>{formatCurrency(data.oneOnOneOtherTotalAmount)}</td>
                <th></th>
                <td></td>
              </tr>
              <tr>
                <th className={style.table_title} colSpan={6}>
                  1对多（非素质）营收
                </th>
              </tr>
              <tr>
                <th>1对多（非素质）实收资金</th>
                <td>{formatCurrency(data.otmClassExamRealAmount)}</td>
                <th>1对多（非素质）优惠资金</th>
                <td>{formatCurrency(data.otmClassExamPromotionAmount)}</td>
                <th>1对多（非素质）实收历史冲销</th>
                <td>{formatCurrency(data.otmClassExamRealWashAmount)}</td>
              </tr>
              <tr>
                <th>1对多（非素质）优惠历史冲销</th>
                <td>{formatCurrency(data.otmClassExamPromotionWashAmount)}</td>
                <th>1对多（非素质）小计</th>
                <td>{formatCurrency(data.otmClassExamTotalAmount)}</td>
                <th></th>
                <td></td>
              </tr>
              <tr>
                <th className={style.table_title} colSpan={6}>
                  其他（1对多非素质）营收
                </th>
              </tr>
              <tr>
                <th>其他（1对多非素质）实收资金</th>
                <td>{formatCurrency(data.otmClassOtherExamRealAmount)}</td>
                <th>其他（1对多非素质）优惠资金</th>
                <td>{formatCurrency(data.otmClassOtherExamPromotionAmount)}</td>
                <th>其他（1对多非素质）实收历史冲销</th>
                <td>{formatCurrency(data.otmClassOtherExamRealWashAmount)}</td>
              </tr>
              <tr>
                <th>其他（1对多非素质）优惠历史冲销</th>
                <td>{formatCurrency(data.otmClassOtherExamPromotionWashAmount)}</td>
                <th>其他（1对多非素质）小计</th>
                <td>{formatCurrency(data.otmClassOtherExamTotalAmount)}</td>
                <th></th>
                <td></td>
              </tr>
              <tr>
                <th className={style.table_title} colSpan={6}>
                  1对多（素质）营收
                </th>
              </tr>
              <tr>
                <th>1对多（素质）实收资金</th>
                <td>{formatCurrency(data.otmClassQualityRealAmount)}</td>
                <th>1对多（素质）优惠资金</th>
                <td>{formatCurrency(data.otmClassQualityPromotionAmount)}</td>
                <th>1对多（素质）实收历史冲销</th>
                <td>{formatCurrency(data.otmClassQualityRealWashAmount)}</td>
              </tr>
              <tr>
                <th>1对多（素质）优惠历史冲销</th>
                <td>{formatCurrency(data.otmClassQualityPromotionWashAmount)}</td>
                <th>1对多（素质）小计</th>
                <td>{formatCurrency(data.otmClassQualityTotalAmount)}</td>
                <th></th>
                <td></td>
              </tr>
              <tr>
                <th className={style.table_title} colSpan={6}>
                  其他（1对多素质）营收
                </th>
              </tr>
              <tr>
                <th>其他（1对多素质）实收资金</th>
                <td>{formatCurrency(data.otmClassOtherQualityRealAmount)}</td>
                <th>其他（1对多素质）优惠资金</th>
                <td>{formatCurrency(data.otmClassOtherQualityPromotionAmount)}</td>
                <th>其他（1对多素质）实收历史冲销</th>
                <td>{formatCurrency(data.otmClassOtherQualityRealWashAmount)}</td>
              </tr>
              <tr>
                <th>其他（1对多素质）优惠历史冲销</th>
                <td>{formatCurrency(data.otmClassOtherQualityPromotionWashAmount)}</td>
                <th>其他（1对多素质）小计</th>
                <td>{formatCurrency(data.otmClassOtherQualityTotalAmount)}</td>
                <th></th>
                <td></td>
              </tr>
              <tr>
                <th className={style.table_title} colSpan={6}>
                  小班课（非素质）营收
                </th>
              </tr>
              <tr>
                <th>小班课（非素质）实收资金</th>
                <td>{formatCurrency(data.smallClassExamRealAmount)}</td>
                <th>小班课（非素质）优惠资金</th>
                <td>{formatCurrency(data.smallClassExamPromotionAmount)}</td>
                <th>小班课（非素质）实收历史冲销</th>
                <td>{formatCurrency(data.smallClassExamRealWashAmount)}</td>
              </tr>
              <tr>
                <th>小班课（非素质）优惠历史冲销</th>
                <td>{formatCurrency(data.smallClassExamPromotionWashAmount)}</td>
                <th>小班课（非素质）小计</th>
                <td>{formatCurrency(data.smallClassExamTotalAmount)}</td>
                <th></th>
                <td></td>
              </tr>
              <tr>
                <th className={style.table_title} colSpan={6}>
                  其他（小班非素质）营收
                </th>
              </tr>
              <tr>
                <th>其他（小班非素质）实收资金</th>
                <td>{formatCurrency(data.smallClassOtherExamRealAmount)}</td>
                <th>其他（小班非素质）优惠资金</th>
                <td>{formatCurrency(data.smallClassOtherExamPromotionAmount)}</td>
                <th>其他（小班非素质）实收历史冲销</th>
                <td>{formatCurrency(data.smallClassOtherExamRealWashAmount)}</td>
              </tr>
              <tr>
                <th>其他（小班非素质）优惠历史冲销</th>
                <td>{formatCurrency(data.smallClassOtherExamPromotionWashAmount)}</td>
                <th>其他（小班非素质）小计</th>
                <td>{formatCurrency(data.smallClassOtherExamTotalAmount)}</td>
                <th></th>
                <td></td>
              </tr>
              <tr>
                <th className={style.table_title} colSpan={6}>
                  小班课（素质）营收
                </th>
              </tr>
              <tr>
                <th>小班课（素质）实收资金</th>
                <td>{formatCurrency(data.smallClassQualityRealAmount)}</td>
                <th>小班课（素质）优惠资金</th>
                <td>{formatCurrency(data.smallClassQualityPromotionAmount)}</td>
                <th>小班课（素质）实收历史冲销</th>
                <td>{formatCurrency(data.smallClassQualityRealWashAmount)}</td>
              </tr>
              <tr>
                <th>小班课（素质）优惠历史冲销</th>
                <td>{formatCurrency(data.smallClassQualityPromotionWashAmount)}</td>
                <th>小班课（素质）小计</th>
                <td>{formatCurrency(data.smallClassQualityTotalAmount)}</td>
                <th></th>
                <td></td>
              </tr>
              <tr>
                <th className={style.table_title} colSpan={6}>
                  其他（小班课素质）营收
                </th>
              </tr>
              <tr>
                <th>其他（小班课素质）实收资金</th>
                <td>{formatCurrency(data.smallClassQualityRealAmount)}</td>
                <th>其他（小班课素质）优惠资金</th>
                <td>{formatCurrency(data.smallClassQualityPromotionAmount)}</td>
                <th>其他（小班课素质）实收历史冲销</th>
                <td>{formatCurrency(data.smallClassQualityRealWashAmount)}</td>
              </tr>
              <tr>
                <th>其他（小班课素质）优惠历史冲销</th>
                <td>{formatCurrency(data.smallClassQualityPromotionWashAmount)}</td>
                <th>其他（小班课素质）小计</th>
                <td>{formatCurrency(data.smallClassQualityTotalAmount)}</td>
                <th></th>
                <td></td>
              </tr>
              <tr>
                <th className={style.table_title} colSpan={6}>
                  精英班营收
                </th>
              </tr>
              <tr>
                <th>精英班实收资金</th>
                <td>{formatCurrency(data.ecsClassRealAmount)}</td>
                <th>精英班优惠资金</th>
                <td>{formatCurrency(data.ecsClassPromotionAmount)}</td>
                <th>精英班实收历史冲销</th>
                <td>{formatCurrency(data.ecsClassRealWashAmount)}</td>
              </tr>
              <tr>
                <th>精英班优惠历史冲销</th>
                <td>{formatCurrency(data.ecsClassPromotionWashAmount)}</td>
                <th>精英班小计</th>
                <td>{formatCurrency(data.ecsClassTotalAmount)}</td>
                <th></th>
                <td></td>
              </tr>
              <tr>
                <th className={style.table_title} colSpan={6}>
                  其他（精英班）营收
                </th>
              </tr>
              <tr>
                <th> 其他（精英班）实收资金</th>
                <td>{formatCurrency(data.ecsClassOtherRealAmount)}</td>
                <th> 其他（精英班）优惠资金</th>
                <td>{formatCurrency(data.ecsClassOtherPromotionAmount)}</td>
                <th> 其他（精英班）实收历史冲销</th>
                <td>{formatCurrency(data.ecsClassOtherRealWashAmount)}</td>
              </tr>
              <tr>
                <th> 其他（精英班）优惠历史冲销</th>
                <td>{formatCurrency(data.ecsClassOtherPromotionWashAmount)}</td>
                <th> 其他（精英班）小计</th>
                <td>{formatCurrency(data.ecsClassOtherTotalAmount)}</td>
                <th></th>
                <td></td>
              </tr>
              <tr>
                <th className={style.table_title} colSpan={6}>
                  讲座营收
                </th>
              </tr>
              <tr>
                <th>讲座实收资金</th>
                <td>{formatCurrency(data.lectureRealAmount)}</td>
                <th>讲座优惠资金</th>
                <td>{formatCurrency(data.lecturePromotionAmount)}</td>
                <th>讲座实收历史冲销</th>
                <td>{formatCurrency(data.lectureRealWashAmount)}</td>
              </tr>
              <tr>
                <th>讲座优惠历史冲销</th>
                <td>{formatCurrency(data.lecturePromotionWashAmount)}</td>
                <th>讲座小计</th>
                <td>{formatCurrency(data.lectureTotalAmount)}</td>
                <th></th>
                <td></td>
              </tr>
              <tr>
                <th className={style.table_title} colSpan={6}>
                  其他（其他）营收
                </th>
              </tr>
              <tr>
                <th>其他（其他）实收资金</th>
                <td>{formatCurrency(data.otherOtherRealAmount)}</td>
                <th>其他（其他）优惠资金</th>
                <td>{formatCurrency(data.otherOtherPromotionAmount)}</td>
                <th>其他（其他）实收历史冲销</th>
                <td>{formatCurrency(data.otherOtherRealWashAmount)}</td>
              </tr>
              <tr>
                <th>其他（其他）优惠历史冲销</th>
                <td>{formatCurrency(data.otherOtherPromotionWashAmount)}</td>
                <th>其他（其他）小计</th>
                <td>{formatCurrency(data.otherOtherTotalAmount)}</td>
                <th></th>
                <td></td>
              </tr>
              <tr>
                <th className={style.table_title} colSpan={6}>
                  其他（汇总）营收
                </th>
              </tr>
              <tr>
                <th>其他（汇总）实收资金</th>
                <td>{formatCurrency(data.allOthersRealAmount)}</td>
                <th>其他（汇总）优惠资金</th>
                <td>{formatCurrency(data.allOthersPromotionAmount)}</td>
                <th>其他（汇总）实收历史冲销</th>
                <td>{formatCurrency(data.allOthersRealWashAmount)}</td>
              </tr>
              <tr>
                <th>其他（汇总）优惠历史冲销</th>
                <td>{formatCurrency(data.allOthersPromotionWashAmount)}</td>
                <th>其他（汇总）小计</th>
                <td>{formatCurrency(data.allOthersTotalAmount)}</td>
                <th></th>
                <td></td>
              </tr>
              <tr>
                <th className={style.table_title} colSpan={6}>
                  非经营性营收
                </th>
              </tr>
              <tr>
                <th>非经营划归收入</th>
                <td>{formatCurrency(data.isNormalRealAmount)}</td>
                <th>非经营优惠对销</th>
                <td>{formatCurrency(data.isNormalPromotionAmount)}</td>
                <th>非经营历史冲销</th>
                <td>{formatCurrency(data.isNormalHistoryWashAmount)}</td>
              </tr>
              <tr>
                <th>非经营划归小计</th>
                <td>{formatCurrency(data.isNormalTotalAmount)}</td>
                <th></th>
                <td></td>
                <th></th>
                <td></td>
              </tr>
              <tr>
                <th className={style.table_title} colSpan={6}>
                  汇总
                </th>
              </tr>
              <tr>
                <th>实收资金汇总</th>
                <td>{formatCurrency(data.realTotalAmount)}</td>
                <th>优惠资金汇总</th>
                <td>{formatCurrency(data.promotionTotalAmount)}</td>
                <th>实收历史冲销汇总</th>
                <td>{formatCurrency(data.realWashTotalAmount)}</td>
              </tr>
              <tr>
                <th>优惠历史冲销汇总</th>
                <td>{formatCurrency(data.promotionWashTotalAmount)}</td>
                <th>历史冲销汇总</th>
                <td>{formatCurrency(data.historyWashTotalAmount)}</td>
                <th>小计汇总</th>
                <td>{formatCurrency(data.totalAmount)}</td>
              </tr>
            </tbody>
          </table>
          <br />
          <p style={{ fontSize: '20px' }}>调整</p>
          <table cellSpacing="1" width="100%">
            <tbody>
              <tr>
                <th>调整金额</th>
                <td>{formatCurrency(data.adjustAmount)}</td>
                <th>调整后总计</th>
                <td>{formatCurrency(data.totalAmount + data.adjustAmount)}</td>
              </tr>
              <tr>
                <th className={style.table_title} colSpan={4}>
                  备注信息
                </th>
              </tr>
              <tr>
                <td colSpan={4} className={style.remark}>
                  {data.remark}
                </td>
              </tr>
            </tbody>
          </table>
          <br />
          <table cellSpacing="1" width="100%">
            <tbody>
              <tr>
                <th>学习中心确认</th>
                <th>财务初审</th>
                <th>分公司确认</th>
                <th>财务终审</th>
                <th>财务负责人确认</th>
              </tr>
              <tr>
                <td>{printFormData.campusConfirmUserName}</td>
                <td>{printFormData.financeFirstAuditUserName}</td>
                <td>{printFormData.branchConfirmUserName}</td>
                <td>{printFormData.financeEndAuditUserName}</td>
                <td>{printFormData.financeDirectorAuditUserName}</td>
              </tr>
              <tr>
                <td>{printFormData.campusConfirmTime}</td>
                <td>{printFormData.financeFirstAuditTime}</td>
                <td>{printFormData.branchConfirmTime}</td>
                <td>{printFormData.financeEndAuditTime}</td>
                <td>{formateDate(printFormData.financeDirectorAuditTime)}</td>
              </tr>
              <tr>
                <td>{printFormData.campusConfirmTypeName}</td>
                <td>{printFormData.financeFirstAuditTypeName}</td>
                <td>{printFormData.branchConfirmTypeName}</td>
                <td>{printFormData.financeEndAuditTypeName}</td>
                <td>{FinanceDirectorAuditType[printFormData.financeDirectorAuditType]}</td>
              </tr>
              <tr>
                <td className={style.yz}>印章</td>
                <td className={style.yz}>印章</td>
                <td className={style.yz}>印章</td>
                <td className={style.yz}>印章</td>
                <td className={style.yz}>印章</td>
              </tr>
            </tbody>
          </table>
        </div>
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  const { printFormData } = state[namespace];

  return {
    printFormData,
  };
};

export default connect(mapStateToProps)(FinancialVoucher);
