import React from 'react';
import { Modal, TextLink, List } from 'COMPONENT';
import { connect } from 'dva';
import { Row, Col } from 'antd';
import { OLDPAGEBASE } from 'CONST/config';
import { browserHistory } from 'dva/router';
import { namespace } from '../../store/constant';
import { calcPercentAndRound } from '../../utils';

class VoucherAuditModel extends React.Component {
  state = {
    loading: false,
    list: [],
  };
  list = [];
  columns = [
    { title: '收款类型', dataIndex: 'channelName', key: 'channelName' },
    {
      title: '审核比例',
      dataIndex: 'validate',
      key: 'validate',
      render: (_, t) => {
        return this.formatAuditRate(t);
      },
    },
    {
      title: '操作',
      dataIndex: 'opertion',
      key: 'opertion',
      render: (_, r) => <TextLink onClick={() => this.handleOpertion(r)}>{`${r.channelName}课程汇总`}</TextLink>,
    },
  ];

  componentDidMount() {
    const { data = {}, dispatch } = this.props;
    const { campusId, countMonth } = data;
    const params = {
      campusId,
      receiptDate: (countMonth + '').substr(0, 4) + '-' + (countMonth + '').substr(4),
    };
    this.setState({ loading: true });
    dispatch({
      type: `${namespace}/getCampusFundsAuditRate`,
      payload: {
        data: params,
        callback: (item) => {
          this.setState({ loading: false });
        },
      },
    });
  }

  /**
   * 审批
   * @memberof VoucherAuditModel
   */
  handleOk = () => {
    this.props.onOk();
  };

  /**
   * 收款记录点击事件
   * @param 行数据
   * @memberof VoucherAuditModel
   */
  handleOpertion = (r) => {
    const { campusId, countMonth, campusName } = this.props.data;
    const { type, channelName } = r;
    const receiptMonth = (countMonth + '').substr(0, 4) + '-' + (countMonth + '').substr(4);
    let url = '';
    switch (type) {
      case 'ONE_ON_MANY':
        url = 'function/realtimereport/otmClassCourseCollect.html';
        break;
      case 'ONE_ON_ONE_COURSE':
        url = 'function/realtimereport/totalOneOnOneCourse.html';
        break;
      case 'SMALL_CLASS':
        url = 'function/realtimereport/miniClassCourseCollect.html';
        break;
      case 'TWO_TEACHER':
        url = 'function/realtimereport/twoTeacherCourseCollect.html';
        break;
      default:
        break;
    }
    browserHistory.push({
      pathname: OLDPAGEBASE,
      query: {
        src: `${url}?campusId=${campusId}&campusName=${campusName}&receiptMonth=${receiptMonth}`,
        title: `${channelName}课程汇总`,
      },
    });
  };

  /**
   * 格式化审核列表，隐藏total为0的数据
   *
   * @param list
   * @return {Array}
   */
  formatAuditRateList = (list) => {
    if (!list || list.length <= 0) {
      return [];
    }

    let result = [];

    // 接口返回total字段不统一，需前端计算总数=审核+未审核
    list.forEach((item) => {
      const { audit, unAudit, total } = item;
      const realTotal = total ? total : Number(audit || 0, 10) + Number(unAudit || 0, 10);

      if (realTotal > 0) {
        result.push(item);
      }
    });

    return result;
  };

  /**
   * 计算营收凭证的审核比例
   *
   * @param record {object} 行数据
   * @return {string}
   */
  formatAuditRate = (record = {}) => {
    const type = record.type;
    const audit = Number(record.audit || 0, 10);
    const unAudit = Number(record.unAudit || 0, 10);
    const total = Number(record.total || 0, 10);
    let rate = '100%';

    if (type === 'ONE_ON_MANY' || type === 'ONE_ON_ONE_COURSE') {
      const noAudit = audit === 0 && unAudit === 0;
      if (!noAudit) {
        const percent = audit / (audit + unAudit);
        rate = calcPercentAndRound(percent);
      }
    } else if (type === 'SMALL_CLASS') {
      const noAudit = audit === 0 && unAudit === 0;
      if (!noAudit) {
        const percent = audit / unAudit;
        rate = calcPercentAndRound(percent);
      }
    } else if (type === 'TWO_TEACHER') {
      const noAudit = audit === 0 && total === 0;
      if (!noAudit) {
        const percent = audit / total;
        rate = calcPercentAndRound(percent);
      }
    }

    return rate;
  };

  render() {
    const { onCancel, data = {}, auditRateList, auditOkCheck } = this.props;
    const { campusName, branchName, evidenceAuditStatusName, updateDate, countMonthStr } = data;
    const { loading } = this.state;
    const list = this.formatAuditRateList(auditRateList);

    return (
      <Modal
        title="收款凭证审核"
        widthSize="l"
        loading={loading}
        onOk={this.handleOk}
        onCancel={onCancel}
        okText="审核"
        okBtnDisabled={auditOkCheck}
      >
        <Row>
          <Col span={8}>
            <span>{`分公司：${branchName}`}</span>
          </Col>
          <Col span={8}>
            <span>{`学习中心：${campusName}`}</span>
          </Col>
          <Col span={8}>
            <span>{`凭证月份：${countMonthStr}`}</span>
          </Col>
        </Row>
        <br />
        <Row>
          <Col span={8}>
            <span>{`凭证日期: ${updateDate}`}</span>
          </Col>
          <Col span={8}>
            <span>{`状态：${evidenceAuditStatusName}`}</span>
          </Col>
        </Row>
        <br />
        <Row>
          <List dataSource={list} columns={this.columns} rowKey={(record) => record.id} />
        </Row>
        <span style={{ color: 'red' }}>
          备注：凭证月份课程审核比例需为100%方可点击营收凭证审核，如不达标，请跳转至各课程汇总处
          理差异，处理差异后，需重新取数更新凭证审核比例（审核比例计算涉及数据与当前凭证数据保持一
          致，而非实时数据，请注意）。
        </span>
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  const { auditRateList, auditOkCheck } = state[namespace];
  return {
    auditRateList,
    auditOkCheck,
  };
};

export default connect(mapStateToProps)(VoucherAuditModel);
