/**
 * 营收凭证
 * @author zero
 *  */
import React from 'react';
import { connect } from 'dva';
import SearchPage from './search';
import { Table } from 'antd';
import { namespace, exportUrl } from './store/constant';
import { formatCurrency } from 'COMPONENT/utils/basic/num';
import { Page, List, Button, TextLink } from 'COMPONENT';
import { exportExcel } from './utils';
import styles from './index.less';
import AdjustmentModal from './component/AdjustmentModal/index.js';
import VoucherAuditModel from './component/VoucherAuditModel/index.js';
import FinancialVoucher from './component/FinancialVoucher/index.js';
import { loginUser } from 'UTIL/common';
import { getSelectOption } from 'SERVICE/common';
const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;

class cashFlowVoucher extends React.Component {
  state = {
    loading: false,
    pagination: {
      current: 1,
      pageSize: 999,
    },
    searchData: {},
    queryLevelType: 'GROUP',
    branchId: '',
    campusId: '',
    selectRow: {},
    voucherAuditModel: false,
    financialVoucherVisible: false,
    adjustModal: false,
    financialVoucherType: 0, // 收款凭证打开模式： 0 - 审批， 1 - 打印
    adjustType: 0, // 营收凭证调整打开模式： 0 - 查看， 1 - 调整（即可编辑）
    evidenceAuditStatusDict: [], // 审核数据字典
  };

  componentDidMount() {
    const { dispatch } = this.props;
    // 请求产品类型枚举
    dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['year', 'month', 'branch', 'branchCampus', 'evidenceAuditStatus', 'adjustProject'],
      },
    });
    this.getAuditDict();
  }

  // 请求审核状态数据字典
  getAuditDict = async () => {
    try {
      const { value: obj } = await getSelectOption({ selectOptionCategory: 'IncomeEvidenceAuditStatus' }, true);
      const evidenceAuditStatusDict = [];
      Object.keys(obj).forEach((key) => {
        key && evidenceAuditStatusDict.push({ text: obj[key], val: key });
      });
      this.setState({ evidenceAuditStatusDict });
    } catch (e) {
      throw new Error('获取审核数据字典失败');
    }
  };

  componentWillUnmount() {
    this.props.dispatch({ type: `${namespace}/initState` });
  }

  /**
   * 查询现金流列表
   * @param params: 参数列表
   * @memberof cashFlowVoucher
   */
  getReceiptMonthlyStatisticsList = (params = {}) => {
    const {
      pagination: { current, pageSize },
      searchData,
      queryLevelType,
    } = this.state;
    const _queryLevelType = params.queryLevelType || queryLevelType;
    const _branchId = params.branchId;
    const _campusId = params.campusId;
    const data = {
      ...searchData,
      page: current,
      rows: pageSize,
      ...params,
      branchId: _branchId,
      campusId: _campusId,
      queryLevelType: _queryLevelType,
    };
    this.setState({ loading: true });
    console.log(data, 'data');
    this.props.dispatch({
      type: `${namespace}/getReceiptMonthlyStatisticsList`,
      payload: {
        data,
        callback: () => {
          this.setState({ loading: false, queryLevelType: _queryLevelType, branchId: _branchId, campusId: _campusId });
        },
      },
    });
  };

  /**
   * 打印表单显示
   * @memberof cashFlowVoucher
   */
  handlePrint = (row) => {
    this.setState({
      financialVoucherVisible: true,
      financialVoucherType: 1,
      selectRow: row,
    });
  };

  /**
   * 打开调整弹窗
   * @memberof cashFlowVoucher
   */
  handleAdjustment = (row, type) => {
    this.setState({
      adjustModal: true,
      adjustType: type,
      selectRow: row,
    });
  };

  /**
   * 获取操作列 dom
   * @param row 行数据
   * @memberof cashFlowVoucher
   */
  getOpertion = (row) => {
    const { evidenceAuditStatus, id, isClosed, finished } = row;
    const { queryLevelType } = this.state;
    if (id === -1 || queryLevelType === 'CAMPUS') return;
    if (id) {
      const isFinished = String(finished) === '1'; //是否完成
      const dom = [];
      if (!isFinished) {
        if (isClosed == 1) {
          dom.push(
            <TextLink
              key={`${id}audit`}
              onClick={() => {
                this.setState({ selectRow: row, voucherAuditModel: true });
              }}
            >
              审核
            </TextLink>,
          );
        }
        if (evidenceAuditStatus === 'CAMPUS_AUDITED' || evidenceAuditStatus === 'BRENCH_AUDITED') {
          dom.push(
            <TextLink key={`${id}adjustment`} onClick={() => this.handleAdjustment(row, 1)}>
              调整
            </TextLink>,
          );
        }
      } else {
        dom.push(
          <TextLink key={`${id}print`} onClick={() => this.handlePrint(row)}>
            打印
          </TextLink>,
          <TextLink key={`${id}look`} onClick={() => this.handleAdjustment(row, 0)}>
            {' '}
            查看调整
          </TextLink>,
        );
      }
      return dom;
    }
    return null;
  };

  /**
   * 查询事件回调
   * @param searchData 查询数据
   * @memberof cashFlowVoucher
   */
  handleSearch = (searchData) => {
    this.setState({ searchData });
    this.getReceiptMonthlyStatisticsList(searchData);
  };

  /**
   * 导出回调
   * @memberof cashFlowVoucher
   */
  onExport = () => {
    const {
      searchData,
      queryLevelType,
      pagination: { current, pageSize },
      branchId,
      campusId,
    } = this.state;
    var data = {
      ...searchData,
      branchId,
      campusId,
      queryLevelType,
      numOfRecordsLimitation: 2000,
      page: current,
      rows: pageSize,
    };
    exportExcel({ apiUrl: exportUrl, data, fileName: '营收凭证' });
  };

  /**
   * 分页 回调
   * @param current 页数
   * @param pageSize 页码
   * @memberof cashFlowVoucher
   */
  changePage = ({ current, pageSize }) => {
    const { campusId, branchId } = this.state;
    this.setState({ pagination: { current, pageSize } });
    this.getReceiptMonthlyStatisticsList({ page: current, rows: pageSize, campusId, branchId });
  };

  /**
   * 收款凭证审核 - 审核
   * @memberof cashFlowVoucher
   */
  handelVoucherAuditOK = () => {
    this.setState({
      voucherAuditModel: false,
      financialVoucherVisible: true,
      financialVoucherType: 0,
    });
  };

  /**
   * 格式化表格列
   *
   * @return {*[]}
   */
  formatColumn = () => {
    const { queryLevelType } = this.state;
    const _column =
      queryLevelType === 'CAMPUS'
        ? []
        : [
            {
              title: '调整金额',
              width: COL_WIDTH_M,
              dataIndex: 'adjustAmount',
              key: 'adjustAmount',
              render: (t) => formatCurrency(t),
            },
            {
              title: '调整后合计',
              width: COL_WIDTH_M,
              dataIndex: 'afterModifyMoney',
              key: 'afterModifyMoney',
              render: (t, r) => formatCurrency(r.totalAmount + r.adjustAmount),
            },
            { title: '备注信息', dataIndex: 'edivenceRemark', key: 'edivenceRemark', width: COL_WIDTH_XL },
          ];
    const _noTwoTeacherColum = [
      {
        title: '1对1',
        children: [
          {
            title: '实收资金',
            dataIndex: 'oneOnOneRealAmount',
            key: 'oneOnOneRealAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠资金',
            dataIndex: 'oneOnOnePromotionAmount',
            key: 'oneOnOnePromotionAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '实收历史冲销',
            dataIndex: 'oneOnOneRealWashAmount',
            key: 'oneOnOneRealWashAmount',
            width: COL_WIDTH_L,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠历史冲销',
            dataIndex: 'oneOnOnePromotionWashAmount',
            key: 'oneOnOnePromotionWashAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '小计',
            dataIndex: 'oneOnOneTotalAmount',
            key: 'oneOnOneTotalAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
        ],
      },
      {
        title: '其他（1对1）',
        children: [
          {
            title: '实收资金',
            dataIndex: 'oneOnOneOtherRealAmount',
            key: 'oneOnOneOtherRealAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠资金',
            dataIndex: 'oneOnOneOtherPromotionAmount',
            key: 'oneOnOneOtherPromotionAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '实收历史冲销',
            dataIndex: 'oneOnOneOtherRealWashAmount',
            key: 'oneOnOneOtherRealWashAmount',
            width: COL_WIDTH_L,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠历史冲销',
            dataIndex: 'oneOnOneOtherPromotionWashAmount',
            key: 'oneOnOneOtherPromotionWashAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '小计',
            dataIndex: 'oneOnOneOtherTotalAmount',
            key: 'oneOnOneOtherTotalAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
        ],
      },
      {
        title: '1对多（非素质）',
        children: [
          {
            title: '实收资金',
            dataIndex: 'otmClassExamRealAmount',
            key: 'otmClassExamRealAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠资金',
            dataIndex: 'otmClassExamPromotionAmount',
            key: 'otmClassExamPromotionAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '实收历史冲销',
            dataIndex: 'otmClassExamRealWashAmount',
            key: 'otmClassExamRealWashAmount',
            width: COL_WIDTH_L,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠历史冲销',
            dataIndex: 'otmClassExamPromotionWashAmount',
            key: 'otmClassExamPromotionWashAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '小计',
            dataIndex: 'otmClassExamTotalAmount',
            key: 'otmClassExamTotalAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
        ],
      },
      {
        title: '其他（1对多非素质）',
        children: [
          {
            title: '实收资金',
            dataIndex: 'otmClassOtherExamRealAmount',
            key: 'otmClassOtherExamRealAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠资金',
            dataIndex: 'otmClassOtherExamPromotionAmount',
            key: 'otmClassOtherExamPromotionAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '实收历史冲销',
            dataIndex: 'otmClassOtherExamRealWashAmount',
            key: 'otmClassOtherExamRealWashAmount',
            width: COL_WIDTH_L,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠历史冲销',
            dataIndex: 'otmClassOtherExamPromotionWashAmount',
            key: 'otmClassOtherExamPromotionWashAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '小计',
            dataIndex: 'otmClassOtherExamTotalAmount',
            key: 'otmClassOtherExamTotalAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
        ],
      },
      {
        title: '1对多（素质）',
        children: [
          {
            title: '实收资金',
            dataIndex: 'otmClassQualityRealAmount',
            key: 'otmClassQualityRealAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠资金',
            dataIndex: 'otmClassQualityPromotionAmount',
            key: 'otmClassQualityPromotionAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '实收历史冲销',
            dataIndex: 'otmClassQualityRealWashAmount',
            key: 'otmClassQualityRealWashAmount',
            width: COL_WIDTH_L,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠历史冲销',
            dataIndex: 'otmClassQualityPromotionWashAmount',
            key: 'otmClassQualityPromotionWashAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '小计',
            dataIndex: 'otmClassQualityTotalAmount',
            key: 'otmClassQualityTotalAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
        ],
      },
      {
        title: '其他（1对多素质）',
        children: [
          {
            title: '实收资金',
            dataIndex: 'otmClassOtherQualityRealAmount',
            key: 'otmClassOtherQualityRealAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠资金',
            dataIndex: 'otmClassOtherQualityPromotionAmount',
            key: 'otmClassOtherQualityPromotionAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '实收历史冲销',
            dataIndex: 'otmClassOtherQualityRealWashAmount',
            key: 'otmClassOtherQualityRealWashAmount',
            width: COL_WIDTH_L,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠历史冲销',
            dataIndex: 'otmClassOtherQualityPromotionWashAmount',
            key: 'otmClassOtherQualityPromotionWashAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '小计',
            dataIndex: 'otmClassOtherQualityTotalAmount',
            key: 'otmClassOtherQualityTotalAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
        ],
      },
      {
        title: '小班（非素质）',
        children: [
          {
            title: '实收资金',
            dataIndex: 'smallClassExamRealAmount',
            key: 'smallClassExamRealAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠资金',
            dataIndex: 'smallClassExamPromotionAmount',
            key: 'smallClassExamPromotionAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '实收历史冲销',
            dataIndex: 'smallClassExamRealWashAmount',
            key: 'smallClassExamRealWashAmount',
            width: COL_WIDTH_L,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠历史冲销',
            dataIndex: 'smallClassExamPromotionWashAmount',
            key: 'smallClassExamPromotionWashAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '小计',
            dataIndex: 'smallClassExamTotalAmount',
            key: 'smallClassExamTotalAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
        ],
      },
      {
        title: '其他（小班非素质）',
        children: [
          {
            title: '实收资金',
            dataIndex: 'smallClassOtherExamRealAmount',
            key: 'smallClassOtherExamRealAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠资金',
            dataIndex: 'smallClassOtherExamPromotionAmount',
            key: 'smallClassOtherExamPromotionAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '实收历史冲销',
            dataIndex: 'smallClassOtherExamRealWashAmount',
            key: 'smallClassOtherExamRealWashAmount',
            width: COL_WIDTH_L,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠历史冲销',
            dataIndex: 'smallClassOtherExamPromotionWashAmount',
            key: 'smallClassOtherExamPromotionWashAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '小计',
            dataIndex: 'smallClassOtherExamTotalAmount',
            key: 'smallClassOtherExamTotalAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
        ],
      },
      {
        title: '小班（素质）',
        children: [
          {
            title: '实收资金',
            dataIndex: 'smallClassQualityRealAmount',
            key: 'smallClassQualityRealAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠资金',
            dataIndex: 'smallClassQualityPromotionAmount',
            key: 'smallClassQualityPromotionAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '实收历史冲销',
            dataIndex: 'smallClassQualityRealWashAmount',
            key: 'smallClassQualityRealWashAmount',
            width: COL_WIDTH_L,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠历史冲销',
            dataIndex: 'smallClassQualityPromotionWashAmount',
            key: 'smallClassQualityPromotionWashAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '小计',
            dataIndex: 'smallClassQualityTotalAmount',
            key: 'smallClassQualityTotalAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
        ],
      },
      {
        title: '其他（小班素质）',
        children: [
          {
            title: '实收资金',
            dataIndex: 'smallClassOtherQualityRealAmount',
            key: 'smallClassOtherQualityRealAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠资金',
            dataIndex: 'smallClassOtherQualityPromotionAmount',
            key: 'smallClassOtherQualityPromotionAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '实收历史冲销',
            dataIndex: 'smallClassOtherQualityRealWashAmount',
            key: 'smallClassOtherQualityRealWashAmount',
            width: COL_WIDTH_L,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠历史冲销',
            dataIndex: 'smallClassOtherQualityPromotionWashAmount',
            key: 'smallClassOtherQualityPromotionWashAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '小计',
            dataIndex: 'smallClassOtherQualityTotalAmount',
            key: 'smallClassOtherQualityTotalAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
        ],
      },
      {
        title: '精英班',
        children: [
          {
            title: '实收资金',
            dataIndex: 'ecsClassRealAmount',
            key: 'ecsClassRealAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠资金',
            dataIndex: 'ecsClassPromotionAmount',
            key: 'ecsClassPromotionAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '实收历史冲销',
            dataIndex: 'ecsClassRealWashAmount',
            key: 'ecsClassRealWashAmount',
            width: COL_WIDTH_L,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠历史冲销',
            dataIndex: 'ecsClassPromotionWashAmount',
            key: 'ecsClassPromotionWashAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '小计',
            dataIndex: 'ecsClassTotalAmount',
            key: 'ecsClassTotalAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
        ],
      },
      {
        title: '其他（精英班）',
        children: [
          {
            title: '实收资金',
            dataIndex: 'ecsClassOtherRealAmount',
            key: 'ecsClassOtherRealAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠资金',
            dataIndex: 'ecsClassOtherPromotionAmount',
            key: 'ecsClassOtherPromotionAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '实收历史冲销',
            dataIndex: 'ecsClassOtherRealWashAmount',
            key: 'ecsClassOtherRealWashAmount',
            width: COL_WIDTH_L,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠历史冲销',
            dataIndex: 'ecsClassOtherPromotionWashAmount',
            key: 'ecsClassOtherPromotionWashAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '小计',
            dataIndex: 'ecsClassOtherTotalAmount',
            key: 'ecsClassOtherTotalAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
        ],
      },
    ];

    const column = [
      {
        title: '集团',
        dataIndex: 'groupName',
        key: 'groupName',
        width: COL_WIDTH_M,
        render: (t) => (
          <TextLink onClick={() => this.getReceiptMonthlyStatisticsList({ queryLevelType: 'GROUP' })}>{t}</TextLink>
        ),
      },
      {
        title: '分公司',
        dataIndex: 'branchName',
        key: 'branchName',
        width: COL_WIDTH_M,
        render: (t, r) => (
          <TextLink
            onClick={() => this.getReceiptMonthlyStatisticsList({ queryLevelType: 'BRANCH', branchId: r.branchId })}
          >
            {t}
          </TextLink>
        ),
      },
      {
        title: '学习中心',
        dataIndex: 'campusName',
        key: 'campusName',
        width: COL_WIDTH_M,
        render: (t, r) => (
          <TextLink
            onClick={() => {
              r.id !== -1 && this.getReceiptMonthlyStatisticsList({ queryLevelType: 'CAMPUS', campusId: r.campusId });
            }}
          >
            {t}
          </TextLink>
        ),
      },
      { title: '学生', dataIndex: 'studentName', width: COL_WIDTH_M, key: 'studentName' },
      ..._noTwoTeacherColum,
      {
        title: '讲座',
        children: [
          {
            title: '实收资金',
            dataIndex: 'lectureRealAmount',
            key: 'lectureRealAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠资金',
            dataIndex: 'lecturePromotionAmount',
            key: 'lecturePromotionAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '实收历史冲销',
            dataIndex: 'lectureRealWashAmount',
            key: 'lectureRealWashAmount',
            width: COL_WIDTH_L,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠历史冲销',
            dataIndex: 'lecturePromotionWashAmount',
            key: 'lecturePromotionWashAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '小计',
            dataIndex: 'lectureTotalAmount',
            key: 'lectureTotalAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
        ],
      },
      {
        title: '其他（其他）',
        children: [
          {
            title: '实收资金',
            dataIndex: 'otherOtherRealAmount',
            key: 'otherOtherRealAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠资金',
            dataIndex: 'otherOtherPromotionAmount',
            key: 'otherOtherPromotionAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '实收历史冲销',
            dataIndex: 'otherOtherRealWashAmount',
            key: 'otherOtherRealWashAmount',
            width: COL_WIDTH_L,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠历史冲销',
            dataIndex: 'otherOtherPromotionWashAmount',
            key: 'otherOtherPromotionWashAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '小计',
            dataIndex: 'otherOtherTotalAmount',
            key: 'otherOtherTotalAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
        ],
      },
      {
        title: '其他（汇总）',
        children: [
          {
            title: '实收资金',
            dataIndex: 'allOthersRealAmount',
            key: 'allOthersRealAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠资金',
            dataIndex: 'allOthersPromotionAmount',
            key: 'allOthersPromotionAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '实收历史冲销',
            dataIndex: 'allOthersRealWashAmount',
            key: 'allOthersRealWashAmount',
            width: COL_WIDTH_L,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠历史冲销',
            dataIndex: 'allOthersPromotionWashAmount',
            key: 'allOthersPromotionWashAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '小计',
            dataIndex: 'allOthersTotalAmount',
            key: 'allOthersTotalAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
        ],
      },
      {
        title: '非经营',
        children: [
          {
            title: '划归收入',
            dataIndex: 'isNormalRealAmount',
            key: 'isNormalRealAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠对销',
            dataIndex: 'isNormalPromotionAmount',
            key: 'isNormalPromotionAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '历史冲销',
            dataIndex: 'isNormalHistoryWashAmount',
            key: 'isNormalHistoryWashAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '划归小计',
            dataIndex: 'isNormalTotalAmount',
            key: 'isNormalTotalAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
        ],
      },
      {
        title: '汇总',
        children: [
          {
            title: '实收资金',
            dataIndex: 'realTotalAmount',
            key: 'realTotalAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠资金',
            dataIndex: 'promotionTotalAmount',
            key: 'promotionTotalAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '实收历史冲销',
            dataIndex: 'realWashTotalAmount',
            key: 'realWashTotalAmount',
            width: COL_WIDTH_L,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠历史冲销',
            dataIndex: 'promotionWashTotalAmount',
            key: 'promotionWashTotalAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          /*{
            title: '历史冲销',
            dataIndex: 'historyWashTotalAmount',
            key: 'historyWashTotalAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },*/
          {
            title: '小计',
            dataIndex: 'totalAmount',
            key: 'totalAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
        ],
      },
      ..._column,
      { title: '凭证月份', width: COL_WIDTH_M, dataIndex: 'countMonthStr', key: 'countMonthStr' },
      { title: '凭证日期', dataIndex: 'updateDate', key: 'updateDate', width: COL_WIDTH_M },
      { title: '状态', width: COL_WIDTH_S, dataIndex: 'evidenceAuditStatusName', key: 'evidenceAuditStatusName' },
      {
        title: '操作',
        dataIndex: 'operation',
        key: 'operation',
        width: COL_WIDTH_L,
        render: (_, r) => this.getOpertion(r),
      },
    ];
    return column;
  };

  render() {
    const { route, list, total, count_date } = this.props;
    const {
      loading,
      pagination: { current, pageSize },
      voucherAuditModel,
      selectRow,
      financialVoucherVisible,
      adjustModal,
      adjustType,
      financialVoucherType,
      evidenceAuditStatusDict,
    } = this.state;
    return (
      <Page
        fitWindowHeight={true}
        route={route}
        className={styles.finan_flow_voucher}
        subHeader={
          <SearchPage
            onSearch={this.handleSearch}
            count_date={count_date}
            evidenceAuditStatusDict={evidenceAuditStatusDict}
          />
        }
        headerToolBar={<Button onClick={this.onExport}> 导出excel</Button>}
      >
        <div style={{ marginTop: '10px' }}>
          营收凭证（含收费表明细）<span style={{ color: 'red' }}>{`最近更新时间：${count_date || ''}`}</span>
        </div>
        <Table
          dataSource={list}
          columns={this.formatColumn()}
          loading={loading}
          bordered
          pagination={{
            total,
            current,
            showSizeChanger: true,
            showQuickJumper: true,
            showTotal: (total) => `共 ${total} 条记录`,
            pageSize: list.length > 999 ? pageSize + 1 : pageSize,
            pageSizeOptions: ['999'],
          }}
          rowKey={(record) => record.id}
          onChange={this.changePage}
          scroll={{ x: 16000, y: document.body.clientHeight - 440 }}
        />
        {voucherAuditModel && (
          <VoucherAuditModel
            data={selectRow}
            onOk={this.handelVoucherAuditOK}
            onCancel={() => {
              this.setState({ voucherAuditModel: false });
            }}
          />
        )}
        {financialVoucherVisible && (
          <FinancialVoucher
            data={selectRow}
            type={financialVoucherType}
            onOk={this.getReceiptMonthlyStatisticsList}
            onCancel={() => {
              this.setState({ financialVoucherVisible: false });
            }}
          />
        )}
        {adjustModal && (
          <AdjustmentModal
            data={selectRow}
            type={adjustType}
            onOk={this.getReceiptMonthlyStatisticsList}
            onCancel={() => {
              this.setState({ adjustModal: false });
            }}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const {
    account: {
      user: { deptRoles },
    },
  } = state;
  const { list, total, count_date } = state[namespace];
  return {
    list,
    total,
    count_date,
    deptRoles,
  };
};

export default connect(mapStateToProps)(cashFlowVoucher);
