import React from 'react';
import { Form, Row, Col } from 'antd';
import { connect } from 'dva';
import moment from 'moment';
import { Select, Button, Input } from 'COMPONENT';
import { monthEnum } from '../store/constant';
import style from './index.less';
import { getDictValByText, getDictTextByVal } from 'UTIL/dict';

const { Item } = Form;
class SearchPage extends React.Component {
  init = false;
  componentDidMount() {
    this.querySearch();
  }
  componentDidUpdate() {
    this.querySearch();
  }

  querySearch = () => {
    const { monthDict, yearDict } = this.props;
    if (this.init) return;
    if (monthDict.length > 0 || yearDict.length > 0) {
      this.init = true;
      this.handleSearch();
    }
  };

  /**
   * 查询事件
   * @memberof SearchPage
   */
  handleSearch = () => {
    const {
      form: { getFieldsValue },
      onSearch,
      yearDict,
      monthDict,
      count_date,
    } = this.props;
    const values = getFieldsValue();
    const { countDate, countMonth } = values;
    const year = getDictTextByVal(yearDict, countDate);
    const monthText = getDictTextByVal(monthDict, countMonth);
    let monthKey = '01';
    for (let i = 0, len = monthEnum.length; i < len; i += 1) {
      const { text, key } = monthEnum[i];
      if (text === monthText) {
        monthKey = key;
        break;
      }
    }
    const params = {
      ...values,
      queryLevelType: 'BRANCH',
      countDate: `${year}-${monthKey}-01`,
      countMonth: parseInt(`${year}${monthKey}`, 10),
    };
    onSearch(params);
  };

  /**
   * 查询重置
   * @memberof SearchPage
   */
  handleReset = () => {
    const {
      form: { resetFields },
    } = this.props;
    resetFields();
  };

  /**
   * 根据公司 获取 对应学习中心,分公司为空时返回所有学习中心
   * @memberof SearchPage
   */
  getCampusByBranch = () => {
    const {
      form: { getFieldValue },
      campusDict,
    } = this.props;
    const branchId = getFieldValue('branchId');
    if (!branchId) return campusDict;
    const array = campusDict.filter((item) => item.val === branchId);
    return array[0] ? array[0].options : [];
  };

  render() {
    const {
      branchDict,
      yearDict,
      form: { getFieldDecorator },
      monthDict,
      evidenceAuditStatusDict,
      count_date,
    } = this.props;
    const defaultYear = getDictValByText(yearDict, new Date().getFullYear() + '');
    const defaultMonth = getDictValByText(monthDict, monthEnum[new Date().getMonth()].text);
    return (
      <Form layout="inline">
        <Row className={style.voucher_search} gutter={16}>
          <Col span={2}>
            <Item>
              {getFieldDecorator('countDate', { initialValue: defaultYear })(
                <Select placeholder="请选择年份" dict={yearDict} allowClear={false} />,
              )}
            </Item>
          </Col>
          <Col span={2}>
            <Item>
              {getFieldDecorator('countMonth', { initialValue: defaultMonth })(
                <Select placeholder="请选择月份" dict={monthDict} allowClear={false} />,
              )}
            </Item>
          </Col>
          <Col span={3}>
            <Item>
              {getFieldDecorator('auditStatus', { initialValue: '' })(
                <Select placeholder="审核状态" dict={evidenceAuditStatusDict} />,
              )}
            </Item>
          </Col>
          <Col span={3}>
            <Item>
              {getFieldDecorator('branchId')(<Select placeholder="请选择分公司" dict={branchDict} showSearch={true} />)}
            </Item>
          </Col>
          <Col span={4}>
            <Item>
              {getFieldDecorator('campusId')(
                <Select placeholder="请选择学习中心" dict={this.getCampusByBranch()} showSearch={true} />,
              )}
            </Item>
          </Col>
          <Col span={4}>
            <span className={style.text}>{`数据版本：${count_date ? count_date.split(' ')[0] : ''}`}</span>
          </Col>
          <Col className={style.search_button} span={6}>
            <Button type="primary" onClick={this.handleSearch}>
              查询
            </Button>
            <Button type="primary" onClick={this.handleReset}>
              重置
            </Button>
          </Col>
        </Row>
      </Form>
    );
  }
}

const mapStateToProps = (state) => {
  const {
    account: { btnAuths = [] },
    dict: { year, month, branch = [], branchCampus = [] },
  } = state;
  return {
    btnAuths,
    yearDict: year || [],
    monthDict: month || [],
    branchDict: branch,
    campusDict: branchCampus,
  };
};

export default connect(mapStateToProps)(Form.create()(SearchPage));
