/**
 * 营收凭证
 * @author zero
 *  */
import React from 'react';
import { connect } from 'dva';
import SearchPage from './search';
import { Table } from 'antd';
import { getIncomingAnalyze } from 'SERVICE/report';
import { formatCurrency } from 'COMPONENT/utils/basic/num';
import { Page, List, TextLink } from 'COMPONENT';
import styles from './index.less';
import { getSelectOption } from 'SERVICE/common';
const { COL_WIDTH_M, COL_WIDTH_L } = List;

const realAmountKeys = [
  'oneOnoneRealAmount',
  'othersOtoRealAmount',
  'otmClassQualityRealAmount',
  'othersOtmQualityRealAmount',
  'otmClassNotQualityRealAmount',
  'othersOtmNotQualityRealAmount',
  'smallClassQualityRealAmount',
  'othersSmallQualityRealAmount',
  'smallClassNotQualityRealAmount',
  'othersSmallNotQualityRealAmount',
  'escClassRealAmount',
  'othersEcsRealAmount',
  'lectureRealAmount',
  'othersOthersRealAmount',
  'isNormalRealAmount',
];
const promotionAmountKeys = [
  'oneOnonePromotionAmount',
  'othersOtoPromotionAmount',
  'otmClassQualityPromotionAmount',
  'othersOtmQualityPromotionAmount',
  'otmClassNotQualityPromotionAmount',
  'othersOtmNotQualityPromotionAmount',
  'smallClassQualityPromotionAmount',
  'othersSmallQualityPromotionAmount',
  'smallClassNotQualityPromotionAmount',
  'othersSmallNotQualityPromotionAmount',
  'escClassPromotionAmount',
  'othersEcsPromotionAmount',
  'lecturePromotionAmount',
  'othersOthersPromotionAmount',
  'isNormalPromotionAmount',
];
const totalKeys = [
  'oneOnoneTotalAmount',
  'othersOtoTotalAmount',
  'otmClassQualityTotalAmount',
  'othersOtmQualityTotalAmount',
  'otmClassNotQualityTotalAmount',
  'othersOtmNotQualityTotalAmount',
  'smallClassQualityTotalAmount',
  'othersSmallQualityTotalAmount',
  'smallClassNotQualityTotalAmount',
  'othersSmallNotQualityTotalAmount',
  'escClassTotalAmount',
  'othersEcsTotalAmount',
  'lectureTotalAmount',
  'othersOthersTotalAmount',
  'isNormalTotalAmount',
];

const transform = (keys, list) => {
  return _.transform(
    keys,
    (result, value) => {
      result[value] = _.reduce(list, (sum, item) => (sum += item[value]), 0);
    },
    {},
  );
};
const getValues = (value) => {
  return value?.split(/_/).filter((v) => v);
};
const transToPercentAndRound = (data) => {
  if (typeof data == 'number') {
    return Math.round(data * 10000) / 100 + '%';
  } else return '';
};
const calcData = (list) => {
  const rows = list.concat({
    studentId: '0_合计：',
    ...transform(realAmountKeys, list),
    ...transform(promotionAmountKeys, list),
    ...transform(totalKeys, list),
  });
  return rows?.map((row) => {
    const totalRealAmount = _.reduce(realAmountKeys, (sum, key) => (sum += row[key]), 0);
    const totalPromotionAmount = _.reduce(promotionAmountKeys, (sum, key) => (sum += row[key]), 0);
    const totalAmount = totalRealAmount + totalPromotionAmount;
    return {
      ...row,
      totalAmount,
      totalRealAmount,
      totalPromotionAmount,
    };
  });
};

class IncomingAnalyze extends React.Component {
  state = {
    total: 0,
    list: [],
    loading: false,
    pagination: {
      current: 1,
      pageSize: 999,
    },
    branchId: '',
    campusId: '',
    searchData: {},
    queryLevelType: 'GROUP',
    dimensionalityDict: [],
  };

  componentDidMount() {
    this.getDimensionalityDict();
  }

  // 请求审核状态数据字典
  getDimensionalityDict = async () => {
    try {
      const { value: obj } = await getSelectOption({ selectOptionCategory: 'Dimensionality' }, true);
      const dimensionalityDict = [
        { val: '', text: '请选择' },
        ...Object.keys(obj)
          .map((key) => ({ val: key, text: obj[key] }))
          .filter((item) => item.val),
      ];
      this.setState({ dimensionalityDict });
    } catch (e) {
      throw new Error('获取维度字典失败');
    }
  };

  /**
   * 查询现金流列表
   * @param params: 参数列表
   * @memberof cashFlowVoucher
   */
  getReceiptMonthlyStatisticsList = (params = {}) => {
    const {
      pagination: { current, pageSize },
      searchData,
      queryLevelType,
    } = this.state;
    const _queryLevelType = params.queryLevelType || queryLevelType;
    const _branchId = params.branchId;
    const _campusId = params.campusId;
    const data = {
      ...searchData,
      ...params,
      page: current,
      rows: pageSize,
      role: 'TEATCHER',
      branchId: _branchId,
      campusId: _campusId,
      basicOperationQueryLevelType: _queryLevelType,
    };
    this.setState({ loading: true });
    Promise.all([
      getIncomingAnalyze({
        ...data,
        getTotal: false,
      }),
      getIncomingAnalyze({
        ...data,
        getTotal: true,
      }),
    ])
      .then(([listResult, totalResult]) => {
        this.setState({ list: calcData(listResult?.rows), total: totalResult?.total });
      })
      .finally(() => {
        this.setState({ loading: false });
      })
      .catch((e) => {
        console.log(e);
      });
  };

  /**
   * 查询事件回调
   * @param searchData 查询数据
   * @memberof cashFlowVoucher
   */
  handleSearch = (searchData) => {
    this.setState({ searchData });
    this.getReceiptMonthlyStatisticsList(searchData);
  };

  /**
   * 分页 回调
   * @param current 页数
   * @param pageSize 页码
   * @memberof cashFlowVoucher
   */
  changePage = ({ current, pageSize }) => {
    const { campusId, branchId } = this.state;
    this.setState({ pagination: { current, pageSize } });
    this.getReceiptMonthlyStatisticsList({ page: current, rows: pageSize, campusId, branchId });
  };

  /**
   * 格式化表格列
   *
   * @return {*[]}
   */
  formatColumn = () => {
    const column = [
      {
        title: '集团',
        dataIndex: 'groupId',
        key: 'groupId',
        width: 250,
        render: (t) => {
          const values = getValues(t);
          return values?.length ? (
            <TextLink onClick={() => this.getReceiptMonthlyStatisticsList({ queryLevelType: 'GROUNP' })}>
              {values[1]}
            </TextLink>
          ) : (
            '-'
          );
        },
      },
      {
        title: '分公司',
        dataIndex: 'brenchId',
        key: 'brenchId',
        width: 250,
        render: (t, r) => {
          const values = getValues(t);
          return values?.length ? (
            <TextLink
              onClick={() => this.getReceiptMonthlyStatisticsList({ queryLevelType: 'BRENCH', blCampusId: values[0] })}
            >
              {values[1]}
            </TextLink>
          ) : (
            '-'
          );
        },
      },
      {
        title: '学习中心',
        dataIndex: 'campusId',
        key: 'campusId',
        width: 250,
        render: (t, r) => {
          const values = getValues(t);
          return values?.length ? (
            <TextLink
              onClick={() => {
                this.getReceiptMonthlyStatisticsList({ queryLevelType: 'CAMPUS', blCampusId: values[0] });
              }}
            >
              {values[1]}
            </TextLink>
          ) : (
            '-'
          );
        },
      },
      {
        title: '学生',
        dataIndex: 'studentId',
        width: 150,
        key: 'studentId',
        render: (value) => {
          const values = getValues(value);
          return values?.length ? values[1] : '-';
        },
      },
      {
        title: '1对1',
        children: [
          {
            title: '实际收入',
            dataIndex: 'oneOnoneRealAmount',
            key: 'oneOnoneRealAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠收入',
            dataIndex: 'oneOnonePromotionAmount',
            key: 'oneOnonePromotionAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '总营收',
            dataIndex: 'oneOnoneTotalAmount',
            key: 'oneOnoneTotalAmount',
            width: COL_WIDTH_L,
            render: (t, row) => {
              return formatCurrency(row.oneOnoneRealAmount + row.oneOnonePromotionAmount);
            },
          },
        ],
      },
      {
        title: '其他（1对1）',
        children: [
          {
            title: '实际收入',
            dataIndex: 'othersOtoRealAmount',
            key: 'othersOtoRealAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠收入',
            dataIndex: 'othersOtoPromotionAmount',
            key: 'othersOtoPromotionAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '总营收',
            dataIndex: 'othersOtoTotalAmount',
            key: 'othersOtoTotalAmount',
            width: COL_WIDTH_L,
            render: (t, row) => {
              return formatCurrency(row.othersOtoRealAmount + row.othersOtoPromotionAmount);
            },
          },
        ],
      },
      {
        title: '1对多（素质）',
        children: [
          {
            title: '实际收入',
            dataIndex: 'otmClassQualityRealAmount',
            key: 'otmClassQualityRealAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠收入',
            dataIndex: 'otmClassQualityPromotionAmount',
            key: 'otmClassQualityPromotionAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '总营收',
            dataIndex: 'otmClassQualityTotalAmount',
            key: 'otmClassQualityTotalAmount',
            width: COL_WIDTH_L,
            render: (t, row) => {
              return formatCurrency(row.otmClassQualityRealAmount + row.otmClassQualityPromotionAmount);
            },
          },
        ],
      },
      {
        title: '其他（1对多素质）',
        children: [
          {
            title: '实际收入',
            dataIndex: 'othersOtmQualityRealAmount',
            key: 'othersOtmQualityRealAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠收入',
            dataIndex: 'othersOtmQualityPromotionAmount',
            key: 'othersOtmQualityPromotionAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '总营收',
            dataIndex: 'othersOtmQualityTotalAmount',
            key: 'othersOtmQualityTotalAmount',
            width: COL_WIDTH_L,
            render: (t, row) => {
              return formatCurrency(row.othersOtmQualityRealAmount + row.othersOtmQualityPromotionAmount);
            },
          },
        ],
      },
      {
        title: '1对多（非素质）',
        children: [
          {
            title: '实际收入',
            dataIndex: 'otmClassNotQualityRealAmount',
            key: 'otmClassNotQualityRealAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠收入',
            dataIndex: 'otmClassNotQualityPromotionAmount',
            key: 'otmClassNotQualityPromotionAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '总营收',
            dataIndex: 'otmClassNotQualityTotalAmount',
            key: 'otmClassNotQualityTotalAmount',
            width: COL_WIDTH_L,
            render: (t, row) => {
              return formatCurrency(row.otmClassNotQualityRealAmount + row.otmClassNotQualityPromotionAmount);
            },
          },
        ],
      },
      {
        title: '其他（1对多非素质）',
        children: [
          {
            title: '实际收入',
            dataIndex: 'othersOtmNotQualityRealAmount',
            key: 'othersOtmNotQualityRealAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠收入',
            dataIndex: 'othersOtmNotQualityPromotionAmount',
            key: 'othersOtmNotQualityPromotionAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '总营收',
            dataIndex: 'othersOtmNotQualityTotalAmount',
            key: 'othersOtmNotQualityTotalAmount',
            width: COL_WIDTH_L,
            render: (t, row) => {
              return formatCurrency(row.othersOtmNotQualityRealAmount + row.othersOtmNotQualityPromotionAmount);
            },
          },
        ],
      },
      {
        title: '小班（素质）',
        children: [
          {
            title: '实际收入',
            dataIndex: 'smallClassQualityRealAmount',
            key: 'smallClassQualityRealAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠收入',
            dataIndex: 'smallClassQualityPromotionAmount',
            key: 'smallClassQualityPromotionAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '总营收',
            dataIndex: 'smallClassQualityTotalAmount',
            key: 'smallClassQualityTotalAmount',
            width: COL_WIDTH_L,
            render: (t, row) => {
              return formatCurrency(row.smallClassQualityRealAmount + row.smallClassQualityPromotionAmount);
            },
          },
        ],
      },
      {
        title: '其他（小班素质）',
        children: [
          {
            title: '实际收入',
            dataIndex: 'othersSmallQualityRealAmount',
            key: 'othersSmallQualityRealAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠收入',
            dataIndex: 'othersSmallQualityPromotionAmount',
            key: 'othersSmallQualityPromotionAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '总营收',
            dataIndex: 'othersSmallQualityTotalAmount',
            key: 'othersSmallQualityTotalAmount',
            width: COL_WIDTH_L,
            render: (t, row) => {
              return formatCurrency(row.othersSmallQualityRealAmount + row.othersSmallQualityPromotionAmount);
            },
          },
        ],
      },
      {
        title: '小班（非素质）',
        children: [
          {
            title: '实际收入',
            dataIndex: 'smallClassNotQualityRealAmount',
            key: 'smallClassNotQualityRealAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠收入',
            dataIndex: 'smallClassNotQualityPromotionAmount',
            key: 'smallClassNotQualityPromotionAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '总营收',
            dataIndex: 'smallClassNotQualityTotalAmount',
            key: 'smallClassNotQualityTotalAmount',
            width: COL_WIDTH_L,
            render: (t, row) => {
              return formatCurrency(row.smallClassNotQualityRealAmount + row.smallClassNotQualityPromotionAmount);
            },
          },
        ],
      },
      {
        title: '其他（小班非素质）',
        children: [
          {
            title: '实际收入',
            dataIndex: 'othersSmallNotQualityRealAmount',
            key: 'othersSmallNotQualityRealAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠收入',
            dataIndex: 'othersSmallNotQualityPromotionAmount',
            key: 'othersSmallNotQualityPromotionAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '总营收',
            dataIndex: 'othersSmallNotQualityTotalAmount',
            key: 'othersSmallNotQualityTotalAmount',
            width: COL_WIDTH_L,
            render: (t, row) => {
              return formatCurrency(row.othersSmallNotQualityRealAmount + row.othersSmallNotQualityPromotionAmount);
            },
          },
        ],
      },
      {
        title: '精英班',
        children: [
          {
            title: '实际收入',
            dataIndex: 'escClassRealAmount',
            key: 'escClassRealAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠收入',
            dataIndex: 'escClassPromotionAmount',
            key: 'escClassPromotionAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '总营收',
            dataIndex: 'escClassTotalAmount',
            key: 'escClassTotalAmount',
            width: COL_WIDTH_L,
            render: (t, row) => {
              return formatCurrency(row.escClassRealAmount + row.escClassPromotionAmount);
            },
          },
        ],
      },
      {
        title: '其他（精英班）',
        children: [
          {
            title: '实际收入',
            dataIndex: 'othersEcsRealAmount',
            key: 'othersEcsRealAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠收入',
            dataIndex: 'othersEcsPromotionAmount',
            key: 'othersEcsPromotionAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '总营收',
            dataIndex: 'othersEcsTotalAmount',
            key: 'othersEcsTotalAmount',
            width: COL_WIDTH_L,
            render: (t, row) => {
              return formatCurrency(row.othersEcsRealAmount + row.othersEcsPromotionAmount);
            },
          },
        ],
      },
      {
        title: '讲座',
        children: [
          {
            title: '实际收入',
            dataIndex: 'lectureRealAmount',
            key: 'lectureRealAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠收入',
            dataIndex: 'lecturePromotionAmount',
            key: 'lecturePromotionAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '总营收',
            dataIndex: 'lectureTotalAmount',
            key: 'lectureTotalAmount',
            width: COL_WIDTH_L,
            render: (t, row) => {
              return formatCurrency(row.lectureRealAmount + row.lecturePromotionAmount);
            },
          },
        ],
      },
      {
        title: '其他（其他）',
        children: [
          {
            title: '实际收入',
            dataIndex: 'othersOthersRealAmount',
            key: 'othersOthersRealAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠收入',
            dataIndex: 'othersOthersPromotionAmount',
            key: 'othersOthersPromotionAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '总营收',
            dataIndex: 'othersOthersTotalAmount',
            key: 'othersOthersTotalAmount',
            width: COL_WIDTH_L,
            render: (t, row) => {
              return formatCurrency(row.othersOthersRealAmount + row.othersOthersPromotionAmount);
            },
          },
        ],
      },
      {
        title: '划归',
        children: [
          {
            title: '实际收入',
            dataIndex: 'isNormalRealAmount',
            key: 'isNormalRealAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '优惠收入',
            dataIndex: 'isNormalPromotionAmount',
            key: 'isNormalPromotionAmount',
            width: COL_WIDTH_M,
            render: (t) => formatCurrency(t),
          },
          {
            title: '总营收',
            dataIndex: 'isNormalTotalAmount',
            key: 'isNormalTotalAmount',
            width: COL_WIDTH_L,
            render: (t, row) => {
              return formatCurrency(row.isNormalRealAmount + row.isNormalPromotionAmount);
            },
          },
        ],
      },
      {
        title: '汇总',
        children: [
          {
            title: '实际收入汇总',
            dataIndex: 'totalRealAmount',
            key: 'totalRealAmount',
            width: COL_WIDTH_M,
            render: (t) => {
              return formatCurrency(t);
            },
          },
          {
            title: '优惠收入汇总',
            dataIndex: 'totalPromotionAmount',
            key: 'totalPromotionAmount',
            width: COL_WIDTH_M,
            render: (t) => {
              return formatCurrency(t);
            },
          },
          {
            title: '总营收汇总',
            dataIndex: 'totalAmount',
            key: 'totalAmount',
            width: COL_WIDTH_L,
            render: (t) => {
              return formatCurrency(t);
            },
          },
          {
            title: '实际营收比',
            dataIndex: 'realRate',
            key: 'realRate',
            width: COL_WIDTH_L,
            render: (_, row) => {
              return row.totalAmount > 0 ? transToPercentAndRound(row.totalRealAmount / row.totalAmount) : '-';
            },
          },
        ],
      },
    ];
    return column;
  };

  render() {
    const { list, total } = this.state;
    const { route } = this.props;
    const {
      loading,
      pagination: { current, pageSize },
      dimensionalityDict,
    } = this.state;
    const columns = this.formatColumn();
    const width = _.reduce(columns, (sum, data) => (sum += data.width || 100), 0) + 300;
    return (
      <Page
        fitWindowHeight={true}
        route={route}
        className={styles.finan_flow_voucher}
        subHeader={<SearchPage onSearch={this.handleSearch} dimensionalityDict={dimensionalityDict} />}
      >
        <Table
          dataSource={list}
          columns={columns}
          loading={loading}
          bordered
          pagination={{
            total,
            current,
            showSizeChanger: true,
            showQuickJumper: true,
            showTotal: (total) => `共 ${total} 条记录`,
            pageSize: list.length > 999 ? pageSize + 1 : pageSize,
            pageSizeOptions: ['999'],
          }}
          rowKey={(record) => record.id}
          onChange={this.changePage}
          scroll={{ x: width, y: document.body.clientHeight - 440 }}
        />
      </Page>
    );
  }
}

const mapStateToProps = () => {
  return {};
};

export default connect(mapStateToProps)(IncomingAnalyze);
