import React from 'react';
import { Form, Row, Col } from 'antd';
import { connect } from 'dva';
import moment from 'moment';
import DateRangePicker from 'COMPONENT/DateRangePicker';
import { Select, Button } from 'COMPONENT';
import style from './index.less';

const { Item } = Form;
class SearchPage extends React.Component {
  init = false;
  componentDidMount() {
    this.querySearch();
  }
  componentDidUpdate() {
    this.querySearch();
  }

  querySearch = () => {
    const { monthDict, yearDict } = this.props;
    if (this.init) return;
    if (monthDict.length > 0 || yearDict.length > 0) {
      this.init = true;
      this.handleSearch();
    }
  };

  /**
   * 查询事件
   * @memberof SearchPage
   */
  handleSearch = () => {
    const {
      form: { getFieldsValue },
      onSearch,
    } = this.props;
    const values = getFieldsValue();
    const [startDate, endDate] = values.dateRange;
    onSearch({
      queryLevelType: 'GROUNP',
      dimensionality: values.dimensionality,
      endDate,
      startDate,
    });
  };

  /**
   * 查询重置
   * @memberof SearchPage
   */
  handleReset = () => {
    const {
      form: { resetFields },
    } = this.props;
    resetFields();
  };

  /**
   * 根据公司 获取 对应学习中心,分公司为空时返回所有学习中心
   * @memberof SearchPage
   */
  getCampusByBranch = () => {
    const {
      form: { getFieldValue },
      campusDict,
    } = this.props;
    const branchId = getFieldValue('branchId');
    if (!branchId) return campusDict;
    const array = campusDict.filter((item) => item.val === branchId);
    return array[0] ? array[0].options : [];
  };

  render() {
    const {
      dimensionalityDict,
      form: { getFieldDecorator },
    } = this.props;
    return (
      <Form layout="inline">
        <Row className={style.voucher_search} gutter={16}>
          <Col span={6}>
            <Item>
              {getFieldDecorator('dateRange', {
                initialValue: [
                  moment().startOf('month').startOf('day').format('YYYY-MM-DD'),
                  moment().endOf('day').format('YYYY-MM-DD'),
                ],
              })(<DateRangePicker allowClear={false} format="YYYY-MM-DD" />)}
            </Item>
          </Col>
          <Col span={2}>
            <Item>
              {getFieldDecorator('dimensionality', { initialValue: null })(
                <Select placeholder="请选择维度" dict={dimensionalityDict} allowClear={false} />,
              )}
            </Item>
          </Col>
          <Col className={style.search_button} span={6}>
            <Button type="primary" onClick={this.handleSearch}>
              查询
            </Button>
            <Button type="primary" onClick={this.handleReset}>
              重置
            </Button>
          </Col>
        </Row>
      </Form>
    );
  }
}

const mapStateToProps = (state) => {
  const {
    account: { btnAuths = [] },
    dict: { year, month, branch = [], branchCampus = [] },
  } = state;
  return {
    btnAuths,
    yearDict: year || [],
    monthDict: month || [],
    branchDict: branch,
    campusDict: branchCampus,
  };
};

export default connect(mapStateToProps)(Form.create()(SearchPage));
