import { runFunc } from 'UTIL/common';
import { namespace } from './constant';
import {
  deleteIncomeAdjustItem,
  editIncomeAdjustItem,
  editIncomeAdjustSummary,
  getIncomeMonthlyStatisticsList,
  findIncomeAuditRate,
  printIncomeMonthlyStatistics,
  auditIncomeCampusMonthlyStatistics,
  findAdjustSummary,
} from 'SERVICE/report';
import { computedTotal } from '../utils';

const _state = {
  list: [], // 现金流凭证列表
  total: 0,
  count_date: '', // 最近更新时间
  auditRateList: [], // 凭证审核列表
  auditOkCheck: false, // 审核按钮是否disabled
  printFormData: {},
};

export default {
  namespace,
  state: _state,
  reducers: {
    updateState(state, { payload }) {
      return {
        ...state,
        ...payload,
      };
    },
    initState(state) {
      return {
        ...state,
        ..._state,
      };
    },
  },
  effects: {
    /**
     * 营收凭证列表
     * @param {*} { payload }
     * @param {*} { call, put }
     */
    *getReceiptMonthlyStatisticsList({ payload }, { put }) {
      const { data, callback } = payload;
      const result = yield getIncomeMonthlyStatisticsList(data).catch(() => runFunc(callback));
      const { resultCode, records, rows = [] } = result;
      if (resultCode === 0 && rows) {
        yield put({
          type: 'updateState',
          payload: { total: records, list: computedTotal(rows), count_date: rows[0] ? rows[0].updateTime : '' },
        });
      }
      runFunc(callback);
    },
    /**
     * 审核列表
     * @param {*} { payload }
     * @param {*} { call, put }
     */
    *getCampusFundsAuditRate({ payload }, { call, put }) {
      const { data, callback } = payload;
      try {
        let result1 = yield call(findIncomeAuditRate, { ...data, type: 'ONE_ON_MANY' });
        result1 = result1.map((item) => ({ ...item, type: 'ONE_ON_MANY', channelName: '一对多' }));
        let result2 = yield call(findIncomeAuditRate, { ...data, type: 'ONE_ON_ONE_COURSE' });
        result2 = result2.map((item) => ({ ...item, type: 'ONE_ON_ONE_COURSE', channelName: '一对一' }));
        let result3 = yield call(findIncomeAuditRate, { ...data, type: 'SMALL_CLASS' });
        result3 = result3.map((item) => ({ ...item, type: 'SMALL_CLASS', channelName: '小班' }));
        let result4 = yield call(findIncomeAuditRate, { ...data, type: 'TWO_TEACHER' });
        result4 = result4.map((item) => ({ ...item, type: 'TWO_TEACHER', channelName: '双师' }));
        let list = [...result1, ...result2, ...result3, ...result4].filter(
          (item) => !(item.audit === 0 && item.unAudit === 0),
        );
        let auditOkCheck = false;

        if (list) {
          for (let i = 0, len = list.length; i < len; i += 1) {
            const { unAudit, audit, total, type } = list[i];
            if (['ONE_ON_MANY', 'ONE_ON_ONE_COURSE'].includes(type) && unAudit > 0) {
              auditOkCheck = true;
              break;
            }
            if (['SMALL_CLASS'].includes(type) && audit !== unAudit) {
              auditOkCheck = true;
              break;
            }
            if (['TWO_TEACHER'].includes(type) && total - audit > 0) {
              auditOkCheck = true;
              break;
            }
          }
          yield put({
            type: 'updateState',
            payload: { auditRateList: list, auditOkCheck },
          });
        }
        runFunc(callback);
        runFunc(callback);
      } catch (e) {
        runFunc(callback);
      }
    },
    /**
     * 打印相关数据
     * @param {*} { payload }
     * @param {*} { call, put }
     */
    *printReceiptMonthlyStatisticsStatistics({ payload }, { call, put }) {
      const { data, callback } = payload;
      const result = yield call(printIncomeMonthlyStatistics, data);
      if (result) {
        yield put({
          type: 'updateState',
          payload: { printFormData: result },
        });
      }
      runFunc(callback);
    },
    /**
     * 审核请求
     * @param {*} { payload }
     * @param {*} { call }
     */
    *auditIncomeCampusMonthlyStatistics({ payload }, { call }) {
      const { data, callback } = payload;
      try {
        const result = yield call(auditIncomeCampusMonthlyStatistics, data);
        if (result && result.resultCode === 0) runFunc(callback);
      } catch (e) {
        // Todo
      }
    },
    /**
     * 获取调整列表数据
     * @param {*} { payload }
     * @param {*} { call }
     */
    *findPaymentReceipt({ payload }) {
      const { data, callback } = payload;
      const result = yield findAdjustSummary(data).catch(() => runFunc(callback, {}));
      const list = result || {};
      runFunc(callback, list);
    },
    /**
     * 删除调整项
     * @param {*} { payload }
     * @param {*} { call, put }
     */
    *deleteAdjustItem({ payload }, { call }) {
      const { data, callback } = payload;
      try {
        const result = yield call(deleteIncomeAdjustItem, data);
        if (result && result.resultCode === 0) runFunc(callback);
      } catch (e) {
        // Todo
      }
    },
    /**
     * 编辑保存行
     *
     * @param {*} { payload }
     * @param {*} { call }
     */
    *editAdjustItem({ payload }, { call }) {
      const { data, callback } = payload;
      try {
        const result = yield call(editIncomeAdjustItem, data);
        if (result) runFunc(callback);
      } catch (e) {
        // Todo
      }
    },
    /**
     * 保存凭证备注
     * @param {*} { payload }
     * @param {*} { call }
     */
    *editAdjustSummary({ payload }, { call }) {
      const { data, callback } = payload;
      try {
        const result = yield call(editIncomeAdjustSummary, data);
        if (result) runFunc(callback);
      } catch (e) {
        // Todo
      }
    },
  },
};
