/**
 * 导出Excel
 */
import { urlBase } from 'CONST/config';
import { loginUser } from 'UTIL/common';
import { makeQueryString } from 'UTIL/http/index';

export function exportExcel(params) {
  const { apiUrl, data, fileName, success, fail } = params;
  const paramsString = makeQueryString(data);
  return new Promise((resolve, reject) => {
    let request = new XMLHttpRequest();
    request.open('get', urlBase + apiUrl + (paramsString ? '?' + paramsString : ''), true);
    request.responseType = 'blob';
    request.setRequestHeader('Content-type', 'application/json');
    request.onload = function () {
      if (this.status === 200) {
        let blob = this.response;
        let reader = new FileReader();
        reader.readAsDataURL(blob);
        reader.onload = function (event) {
          resolve();
          let a = document.createElement('a');
          a.href = window.URL.createObjectURL(request.response);
          a.download = fileName + '.xls';
          document.body.appendChild(a);
          a.click();
          a.remove();
          if (success) {
            success();
          }
        };
      } else {
        let reader = new FileReader();
        reader.addEventListener('loadend', (event) => {
          let text = event.srcElement.result;
          reject(JSON.parse(text));
        });
        reader.readAsText(this.response);
      }
    };
    request.onerror = (error) => {
      reject(error);
      if (fail) {
        fail(error);
      }
    };
    request.send(JSON.stringify(data));
  });
}

// 处理营收凭证各小计数据
export function computedTotal(data = []) {
  if (data.length === 0) {
    return [];
  }

  const numFields = [
    /**
     * 小班（面授）
     */
    //实收资金（含税）
    'smallClassFfRealAmount',
    //实收资金（不含税）
    'smallClassFfRealAmountNoTax',
    //实收资金（税金）
    'smallClassFfRealAmountTax',
    //优惠资金
    'smallClassFfPromotionAmount',
    //实收历史冲销（含税）
    'smallClassFfRealWashAmount',
    //实收历史冲销（不含税）
    'smallClassFfRealWashAmountNoTax',
    //实收历史冲销（税金）
    'smallClassFfRealWashAmountTax',
    //优惠历史冲销
    'smallClassFfPromotionWashAmount',
    //小计（含税）
    'smallClassFfTotalAmount',
    /**
     * 小班（AI）
     */
    //实收资金（含税）
    'smallClassHlRealAmount',
    //实收资金（不含税）
    'smallClassHlRealAmountNoTax',
    //实收资金（税金）
    'smallClassHlRealAmountTax',
    //优惠资金
    'smallClassHlPromotionAmount',
    //实收历史冲销（含税）
    'smallClassHlRealWashAmount',
    //实收历史冲销（不含税）
    'smallClassHlRealWashAmountNoTax',
    //实收历史冲销（税金）
    'smallClassHlRealWashAmountTax',
    //优惠历史冲销
    'smallClassHlPromotionWashAmount',
    //小计（含税）
    'smallClassHlTotalAmount',
    /**
     * 1对1
     */
    //实收资金（含税）
    'oneOnOneRealAmount',
    //实收资金（不含税）
    'oneOnOneRealAmountNoTax',
    //实收资金（税金）
    'oneOnOneRealAmountTax',
    //优惠资金
    'oneOnOnePromotionAmount',
    //实收历史冲销（含税）
    'oneOnOneRealWashAmount',
    //实收历史冲销（不含税）
    'oneOnOneRealWashAmountNoTax',
    //实收历史冲销（税金）
    'oneOnOneRealWashAmountTax',
    //优惠历史冲销
    'oneOnOnePromotionWashAmount',
    //小计（含税）
    'oneOnOneTotalAmount',
    /**
     * 其他（1对1）
     */
    // 实收资金
    'oneOnOneOtherRealAmount',
    // 优惠资金
    'oneOnOneOtherPromotionAmount',
    // 实收历史冲销
    'oneOnOneOtherRealWashAmount',
    // 优惠历史冲销
    'oneOnOneOtherPromotionWashAmount',
    // 小计
    'oneOnOneOtherTotalAmount',
    /**
     * 1对多
     */
    //实收资金（含税）
    'otmClassRealAmount',
    //实收资金（不含税）
    'otmClassRealAmountNoTax',
    //实收资金（税金）
    'otmClassRealAmountTax',
    //优惠资金
    'otmClassPromotionAmount',
    //实收历史冲销（含税）
    'otmClassRealWashAmount',
    //实收历史冲销（不含税）
    'otmClassRealWashAmountNoTax',
    //实收历史冲销（税金）
    'otmClassRealWashAmountTax',
    //优惠历史冲销
    'otmClassPromotionWashAmount',
    //小计（含税）
    'otmClassTotalAmount',
    'otmClassExamRealAmount',
    'otmClassExamPromotionAmount',
    'otmClassExamRealWashAmount',
    'otmClassExamPromotionWashAmount',
    'otmClassExamTotalAmount',
    'otmClassOtherExamRealAmount',
    'otmClassOtherExamPromotionAmount',
    'otmClassOtherExamRealWashAmount',
    'otmClassOtherExamPromotionWashAmount',
    'otmClassOtherExamTotalAmount',
    'otmClassQualityRealAmount',
    'otmClassQualityPromotionAmount',
    'otmClassQualityRealWashAmount',
    'otmClassQualityPromotionWashAmount',
    'otmClassQualityTotalAmount',
    'otmClassOtherQualityRealAmount',
    'otmClassOtherQualityPromotionAmount',
    'otmClassOtherQualityRealWashAmount',
    'otmClassOtherQualityPromotionWashAmount',
    'otmClassOtherQualityTotalAmount',
    'smallClassExamRealAmount',
    'smallClassExamPromotionAmount',
    'smallClassExamRealWashAmount',
    'smallClassExamPromotionWashAmount',
    'smallClassExamTotalAmount',
    'smallClassOtherExamRealAmount',
    'smallClassOtherExamPromotionAmount',
    'smallClassOtherExamRealWashAmount',
    'smallClassOtherExamPromotionWashAmount',
    'smallClassOtherExamTotalAmount',
    'smallClassQualityRealAmount',
    'smallClassQualityPromotionAmount',
    'smallClassQualityRealWashAmount',
    'smallClassQualityPromotionWashAmount',
    'smallClassQualityTotalAmount',
    'smallClassOtherQualityRealAmount',
    'smallClassOtherQualityPromotionAmount',
    'smallClassOtherQualityRealWashAmount',
    'smallClassOtherQualityPromotionWashAmount',
    'smallClassOtherQualityTotalAmount',
    /**
     * 小班
     */
    //实收资金（含税）
    'smallClassRealAmount',
    //实收资金（不含税）
    'smallClassRealAmountNoTax',
    //实收资金（税金）
    'smallClassRealAmountTax',
    //优惠资金
    'smallClassPromotionAmount',
    //实收历史冲销（含税）
    'smallClassRealWashAmount',
    //实收历史冲销（不含税）
    'smallClassRealWashAmountNoTax',
    //实收历史冲销（税金）
    'smallClassRealWashAmountTax',
    //优惠历史冲销
    'smallClassPromotionWashAmount',
    //小计（含税）
    'smallClassTotalAmount',
    /**
     * 网校
     */
    //新签实际收入
    'liveNewRealAmount',
    //新签实际收入分成
    'liveNewRealDivide',
    //续费实际收入
    'liveRenewRealAmount',
    //续费实际收入分成
    'liveRenewRealDivide',
    //营收
    'liveTotalAmount',
    //营收分成（含税）
    'liveTotalDivide',
    //营收分成（不含税）
    'liveRealAmountNoTax',
    //营收分成（税金）
    'liveRealAmountTax',
    /**
     * 精英班
     */
    //实收资金（含税）
    'ecsClassRealAmount',
    //实收资金（不含税）
    'ecsClassRealAmountNoTax',
    //实收资金（税金）
    'ecsClassRealAmountTax',
    //优惠资金
    'ecsClassPromotionAmount',
    //实收历史冲销（含税）
    'ecsClassRealWashAmount',
    //实收历史冲销（不含税）
    'ecsClassRealWashAmountNoTax',
    //实收历史冲销（税金）
    'ecsClassRealWashAmountTax',
    //优惠历史冲销
    'ecsClassPromotionWashAmount',
    //小计（含税）
    'ecsClassTotalAmount',
    'ecsClassOtherRealAmount',
    'ecsClassOtherPromotionAmount',
    'ecsClassOtherRealWashAmount',
    'ecsClassOtherPromotionWashAmount',
    'ecsClassOtherTotalAmount',
    /**
     * 双师
     */
    //实收资金（含税）
    'twoTeacherRealAmount',
    //实收资金（不含税）
    'twoTeacherRealAmountNoTax',
    //实收资金（税金）
    'twoTeacherRealAmountTax',
    //优惠资金
    'twoTeacherPromotionAmount',
    //实收历史冲销（含税）
    'twoTeacherRealWashAmount',
    //实收历史冲销（不含税）
    'twoTeacherRealWashAmountNoTax',
    //实收历史冲销（税金）
    'twoTeacherRealWashAmountTax',
    //优惠历史冲销
    'twoTeacherPromotionWashAmount',
    //小计（含税）
    'twoTeacherTotalAmount',
    /**
     * 讲座
     */
    //实收资金（含税）
    'lectureRealAmount',
    //实收资金（不含税）
    'lectureRealAmountNoTax',
    //实收资金（税金）
    'lectureRealAmountTax',
    //优惠资金
    'lecturePromotionAmount',
    //实收历史冲销（含税）
    'lectureRealWashAmount',
    //实收历史冲销（不含税）
    'lectureRealWashAmountNoTax',
    //实收历史冲销（税金）
    'lectureRealWashAmountTax',
    //优惠历史冲销
    'lecturePromotionWashAmount',
    //小计（含税）
    'lectureTotalAmount',
    'otherOtherRealAmount',
    'otherOtherPromotionAmount',
    'otherOtherRealWashAmount',
    'otherOtherPromotionWashAmount',
    'otherOtherTotalAmount',
    /**
     * 其他（服务费）
     * */
    //实收资金（含税）
    'othersServiceRealAmount',
    //实收资金（不含税）
    'othersServiceRealAmountNoTax',
    //实收资金（税金）
    'othersServiceRealAmountTax',
    //优惠资金
    'othersServicePromotionAmount',
    //实收历史冲销（含税）
    'othersServiceRealWashAmount',
    //实收历史冲销（不含税）
    'othersServiceRealWashAmountNoTax',
    //实收历史冲销（税金）
    'othersServiceRealWashAmountTax',
    //优惠历史冲销
    'othersServicePromotionWashAmount',
    //小计（含税）
    'othersServiceTotalAmount',
    /**
     * 其他（非服务费）
     */
    //实收资金（含税）
    'othersRealAmount',
    //实收资金（不含税）
    'othersRealAmountNoTax',
    //实收资金（税金）
    'othersRealAmountTax',
    //优惠资金
    'othersPromotionAmount',
    //实收历史冲销（含税）
    'othersRealWashAmount',
    //实收历史冲销（不含税）
    'othersRealWashAmountNoTax',
    //实收历史冲销（税金）
    'othersRealWashAmountTax',
    //优惠历史冲销
    'othersPromotionWashAmount',
    //小计（含税）
    'othersTotalAmount',
    /**
     * 其他（汇总）
     */
    //实收资金（含税）
    'allOthersRealAmount',
    //实收资金（不含税）
    'allOthersRealAmountNoTax',
    //实收资金（税金）
    'allOthersRealAmountTax',
    //优惠资金
    'allOthersPromotionAmount',
    //实收历史冲销（含税）
    'allOthersRealWashAmount',
    //实收历史冲销（不含税）
    'allOthersRealWashAmountNoTax',
    //实收历史冲销（税金）
    'allOthersRealWashAmountTax',
    //优惠历史冲销
    'allOthersPromotionWashAmount',
    //小计（含税）
    'allOthersTotalAmount',
    /**
     * 非经营
     */
    //划归收入（含税）
    'isNormalRealAmount',
    //划归收入（不含税）
    'isNormalRealAmountNoTax',
    //划归收入（税金）
    'isNormalRealAmountTax',
    //优惠对销
    'isNormalPromotionAmount',
    //历史冲销（含税）
    'isNormalHistoryWashAmount',
    //历史冲销（不含税）
    'isNormalHistoryWashAmountNoTax',
    //历史冲销（税金）
    'isNormalHistoryWashAmountTax',
    //划归小计
    'isNormalTotalAmount',
    /**
     * 汇总
     */
    //实收资金（含税）
    'realTotalAmount',
    //实收资金（不含税）
    'realTotalAmountNoTax',
    //实收资金（税金）
    'realTotalAmountTax',
    //优惠资金
    'promotionTotalAmount',
    //实收历史冲销（含税）
    'realWashTotalAmount',
    //实收历史冲销（不含税）
    'realWashTotalAmountNoTax',
    //实收历史冲销（税金）
    'realWashTotalAmountTax',
    //优惠历史冲销
    'promotionWashTotalAmount',
    //历史冲销
    'historyWashTotalAmount',
    //小计
    'totalAmount',
    /**
     * 调整
     */
    //调整金额
    'adjustAmount',
    //调整后合计
    'afterModifyMoney',
  ];

  const totalRow = data.reduce(
    (prev, row) => {
      return numFields.reduce((totalRow, field) => {
        if (totalRow[field] === undefined) {
          totalRow[field] = 0;
        }

        totalRow[field] += row[field];

        return totalRow;
      }, prev);
    },
    {
      id: -1,
      campusName: '合计: ',
    },
  );

  const newData = data.concat(totalRow);

  return newData;
}

/**
 * 计算百分比
 *
 * @param data {number}
 * @return {string}
 */
export function calcPercentAndRound(data) {
  if (Object.prototype.toString.call(data) === '[object Number]') {
    return Math.round(data * 10000) / 100 + '%';
  }

  return '';
}
