import React from 'react';
import { connect } from 'dva';
import SearchPage from './search/index.js';
import { Table } from 'antd';
import { errorsHandle } from 'SRC/utils/common';
import showMessage from 'SRC/components/utils/globalTips/showMessage';
import { formatCurrency } from 'COMPONENT/utils/basic/num';
import { Page, TextLink } from 'COMPONENT';
import { getListStaffBonus, exportListStaffBonus } from 'SERVICE/report';
import { computedTotal } from './utils.js';

import styles from './index.less';

const Utils = {
  downloadFile(downFileUrl, fileName) {
    const a = document.createElement('a');
    console.log(fileName);
    a.href = downFileUrl;
    a.setAttribute('download', fileName);
    a.style.display = 'none';
    document.body.appendChild(a);
    a.addEventListener('click', () => {
      a.parentNode.removeChild(a);
    });
    a.click();
  },
};

class PersonalPerformanceStatisticsForBanke extends React.Component {
  state = {
    list: [],
    total: 0,
    loading: false,
    isExporting: false,
    pagination: {
      current: 1,
      pageSize: 999,
    },
    blCampusId: '',
    searchData: {},
    basicOperationQueryLevelType: 'GROUP',
  };

  componentDidMount() {
    this.exportParams = {};
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['productType'],
      },
    });
  }

  componentWillUnmount() {}

  /**
   * 查询现金流列表
   * @param params: 参数列表
   * @memberof financialVoucherForGrade
   */
  getListData = (params = {}) => {
    const {
      pagination: { current, pageSize },
      searchData,
      basicOperationQueryLevelType,
    } = this.state;
    const _basicOperationQueryLevelType = params.basicOperationQueryLevelType || basicOperationQueryLevelType;
    const data = {
      ...searchData,
      pageNum: current,
      pageSize: pageSize,
      bonusType: 'CLASS',
      ...params,
      basicOperationQueryLevelType: _basicOperationQueryLevelType,
    };
    this.exportParams = { ...data };
    this.setState({ loading: true, basicOperationQueryLevelType: _basicOperationQueryLevelType });
    getListStaffBonus(data)
      .then(({ data }) => {
        const { total, list } = data;
        this.setState({
          list,
          total,
        });
      })
      .catch(() => {
        this.setState({
          list: [],
          total: 0,
        });
      })
      .finally(() => {
        this.setState({ loading: false });
      });
  };

  /**
   * 查询事件回调
   * @param searchData 查询数据
   * @memberof financialVoucherForGrade
   */
  handleSearch = (searchData) => {
    this.setState({ searchData });
    this.getListData(searchData);
  };

  /**
   * 导出回调
   * @memberof financialVoucherForGrade
   */
  onExport = async () => {
    if (!this.state.total) {
      return showMessage('当前查询数据为空');
    }
    this.setState({ isExporting: true });
    const { data, resultCode } = await exportListStaffBonus({ ...this.exportParams }).catch(errorsHandle());
    this.setState({ isExporting: false });
    if (resultCode !== 0 || !data) {
      return showMessage('导出数据失败，请稍后重试');
    }
    Utils.downloadFile(data, data.split(/\//).pop());
  };

  /**
   * 分页 回调
   * @param current 页数
   * @param pageSize 页码
   * @memberof financialVoucherForGrade
   */
  changePage = ({ current, pageSize }) => {
    const { campusId, branchId } = this.state;
    this.setState({ pagination: { current, pageSize } });
    this.getListData({ page: current, rows: pageSize, campusId, branchId });
  };

  /**
   * 格式化表格列
   *
   * @return {*[]}
   */
  formatColumn = () => {
    return [
      {
        title: '集团',
        dataIndex: 'groupName',
        key: 'groupName',
        width: 160,
        render: (t, r) =>
          t ? (
            <TextLink
              onClick={() =>
                this.getListData({
                  basicOperationQueryLevelType: 'GROUP',
                  blCampusId: r.groupId,
                  campusId: undefined,
                })
              }
            >
              {t}
            </TextLink>
          ) : r.id == -1 ? null : (
            '-'
          ),
      },
      {
        title: '分公司',
        dataIndex: 'branchName',
        key: 'branchName',
        width: 160,
        render: (t, r) =>
          t ? (
            <TextLink
              onClick={() =>
                this.getListData({
                  basicOperationQueryLevelType: 'BRANCH',
                  blCampusId: r.branchId,
                  campusId: undefined,
                })
              }
            >
              {t}
            </TextLink>
          ) : r.id == -1 ? null : (
            '-'
          ),
      },
      {
        title: '学习中心',
        dataIndex: 'campusName',
        key: 'campusName',
        width: 160,
        render: (t, r) =>
          t ? (
            <TextLink
              onClick={() => {
                r.id !== -1 &&
                  this.getListData({
                    basicOperationQueryLevelType: 'CAMPUS',
                    blCampusId: r.campusId,
                  });
              }}
            >
              {t}
            </TextLink>
          ) : r.id == -1 ? null : (
            '-'
          ),
      },
      {
        title: '业绩人员',
        dataIndex: 'staffName',
        key: 'staffName',
        width: 160,
        render: (t, r) =>
          t ? (
            <TextLink
              onClick={() => {
                r.id !== -1 &&
                  this.getListData({
                    basicOperationQueryLevelType: 'USER',
                    blCampusId: r.campusId,
                  });
              }}
            >
              {t}
            </TextLink>
          ) : r.id == -1 ? null : (
            '-'
          ),
      },
      //{ title: '归属部门', dataIndex: 'signStaffOrganization', width: 210, key: 'signStaffOrganization' },
      {
        title: '业绩来源',
        dataIndex: 'fundCampusName',
        width: 160,
        key: 'fundCampusName',
        render: (t, r) => (r.id == -1 ? null : t || '-'),
      },
      {
        title: '签单人/操作人',
        dataIndex: 'signStaffName',
        width: 120,
        key: 'signStaffName',
        render: (t, r) => (r.id == -1 ? null : t || '-'),
      },
      {
        title: '合同年级',
        dataIndex: 'contractGradeName',
        width: 110,
        key: 'contractGradeName',
        render: (t, r) => (r.id == -1 ? null : t || '-'),
      },
      {
        title: '学生',
        dataIndex: 'studentName',
        width: 110,
        key: 'studentName',
        render: (t, r) => (r.id == -1 ? null : t || '-'),
      },
      /*{
        title: '合同类型',
        dataIndex: 'contractType',
        width: 110,
        key: 'contractType',
        render(data) {
          if (data) {
            if (data == 'NEW_CONTRACT') {
              return '新增合同';
            } else if (data == 'RE_CONTRACT') {
              return '续费合同';
            } else if (data == 'LIVE_CONTRACT') {
              return '直播同步合同';
            } else if (data == 'REFUND') {
              return '退款';
            } else {
              return '其他';
            }
          } else {
            return '';
          }
        },
      },*/
      {
        title: '产品类型',
        dataIndex: 'productType',
        width: 110,
        key: 'productType',
        render(data, r) {
          if (data) {
            if (data == 'ONE_ON_ONE_COURSE') {
              return '一对一';
            } else if (data == 'SMALL_CLASS') {
              return '小班';
            } else if (data == 'ECS_CLASS') {
              return '精英班';
            } else if (data == 'ONE_ON_MANY') {
              return '一对多';
            } else if (data == 'OTHERS') {
              return '其他';
            } else if (data == 'LECTURE') {
              return '讲座';
            } else if (data == 'TWO_TEACHER') {
              return '双师';
            } else if (data == 'LIVE') {
              return '直播';
            } else {
              return '小计（元）：';
            }
          } else {
            return r.id == -1 ? null : data || '-';
          }
        },
      },
      {
        title: '新签合同业绩',
        dataIndex: 'newBonusAmount',
        width: 110,
        key: 'newBonusAmount',
        render(data) {
          return formatCurrency(data);
        },
      },
      {
        title: '续费合同业绩',
        dataIndex: 'reBonusAmount',
        width: 110,
        key: 'reBonusAmount',
        render(data) {
          return formatCurrency(data);
        },
      },
      {
        title: '收款业绩',
        dataIndex: 'receiptBonusAmount',
        width: 110,
        key: 'receiptBonusAmount',
        render(data) {
          return formatCurrency(data);
        },
      },
      {
        title: '退费责任业绩',
        dataIndex: 'refundBonusAmount',
        width: 110,
        key: 'refundBonusAmount',
        render(data) {
          return formatCurrency(data);
        },
      },
      {
        title: '总业绩',
        dataIndex: 'totalBonusAmount',
        width: 110,
        key: 'totalBonusAmount',
        render(data) {
          return formatCurrency(data);
        },
      },
    ];
  };

  render() {
    const { route } = this.props;
    const { list, total } = this.state;
    const {
      loading,
      pagination: { current, pageSize },
    } = this.state;
    const columns = this.formatColumn();
    return (
      <Page
        fitWindowHeight={true}
        route={route}
        className={styles.finan_flow_voucher}
        subHeader={
          <div>
            <SearchPage onSearch={this.handleSearch} onExport={this.onExport} loading={this.state.isExporting} />
            <div className={styles.tips}>
              <p>
                <i>*</i>
                本业绩报表统计数据来源于收款记录的业绩分配，如果发现业绩存在差异，请先移步收款记录页面查实业绩分配是否存在与正确
              </p>
              <p>
                <i>*</i>本业绩报表只统计班课团队人员的业绩
              </p>
            </div>
          </div>
        }
      >
        <Table
          dataSource={computedTotal(list, columns.slice(8))}
          columns={columns}
          loading={loading}
          bordered
          pagination={{
            total,
            current,
            showSizeChanger: true,
            showQuickJumper: true,
            showTotal: (total) => `共 ${total} 条记录`,
            pageSize: list.length > 999 ? pageSize + 1 : pageSize,
            pageSizeOptions: ['999'],
          }}
          rowKey={(record) => record.id}
          onChange={this.changePage}
          scroll={{ x: columns.reduce((item, sum) => (sum += item.width), 0), y: document.body.clientHeight - 420 }}
        />
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {};
};

export default connect(mapStateToProps)(PersonalPerformanceStatisticsForBanke);
