/**
 * Created by luoming on 2018/3/6
 */
import React from 'react';
import moment from 'moment';
import styles from './index.less';
import { Page, SearchBar, Select, DatePicker, Tabs } from 'COMPONENT';
import { Row, Col } from 'antd';
import echarts from 'ASSET/js/echarts.min.js';
import { typeOption, moneyOption } from './options';
import { loadBranchDict, loadCampusDict } from 'UTIL/dict';
import { getPosDataByCount, getPosDataByMoney } from 'SERVICE/posReport';
import ShowMessage from 'COMPONENT/utils/globalTips/showMessage';
import noDataSvg from 'ASSET/imgs/no_data.svg';
import NP from 'number-precision';
import { loginUser } from 'UTIL/common';

const Tab = Tabs.Tab;

export default class Report extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false,
      activeKey: 1,
      num: 0,
      showTotal: true,
      showTotalTwo: true,
      startDate: moment().startOf('week').format('YYYY-MM-DD'),
      endDate: moment().endOf('week').format('YYYY-MM-DD'),
      branchId: 'all',
      campusId: 'all',
      campusDict: [{ val: 'all', text: '全部学习中心' }],
    };
  }

  componentDidMount() {
    let typeEcharts = echarts.init(this.typeRate);
    let moneyEcharts = echarts.init(this.moneyRate);

    this.setState({ typeEcharts, moneyEcharts }, () => {
      this.initBranchDict();
    });

    this.bindMouseEvent(typeEcharts, moneyEcharts);
    window.addEventListener('resize', this.resize, false);
  }

  initBranchDict = () => {
    loadBranchDict().then(async (branchDict) => {
      const lastBranchDict = branchDict ? [...branchDict] : [];
      lastBranchDict.unshift({ val: 'all', text: '全部分公司' });

      const state = { branchDict: lastBranchDict };
      if (loginUser.isGroupRole() && branchDict.length > 0) {
        const branchId = branchDict[0].val;
        await this.changeBranchData(branchId);
        const campusDict = this.state.campusDict || [];

        state.branchId = branchId;
        state.campusId = campusDict.length > 1 ? campusDict[1].val : 'all';
      }

      this.setState(state);
      this.loadData();
    });
  };

  resize = () => {
    let { typeEcharts, moneyEcharts } = this.state;
    typeEcharts.resize();
    moneyEcharts.resize();
  };

  /**
   * echarts实例对象绑定鼠标事件
   * @param typeEcharts
   * @param moneyEcharts
   */
  bindMouseEvent = (typeEcharts, moneyEcharts) => {
    typeEcharts.on('mouseover', () => {
      this.setState({ showTotal: false });
    });
    typeEcharts.on('mouseout', () => {
      this.setState({ showTotal: true });
    });
    moneyEcharts.on('mouseover', () => {
      this.setState({ showTotalTwo: false });
    });
    moneyEcharts.on('mouseout', () => {
      this.setState({ showTotalTwo: true });
    });
  };

  componentWillUnmount() {
    let { typeEcharts, moneyEcharts } = this.state;
    //解绑echarts对象鼠标事件
    typeEcharts.off('mouseover');
    typeEcharts.off('mouseout');
    moneyEcharts.off('mouseover');
    moneyEcharts.off('mouseout');
    typeEcharts.dispose();
    moneyEcharts.dispose();
    window.removeEventListener('resize', this.resize, false);
  }

  /**
   * 加载数据
   */
  loadData = async () => {
    let { branchId, campusId, startDate, endDate, typeEcharts, moneyEcharts } = this.state;
    if (!branchId) {
      ShowMessage('请选择分公司！');
      return;
    }
    if (!campusId) {
      ShowMessage('请选择学习中心！');
      return;
    }
    if (!startDate) {
      ShowMessage('请选择开始时间！');
      return;
    }
    if (!endDate) {
      ShowMessage('请选择结束时间！');
      return;
    }
    this.setState({ loading: true });
    branchId = branchId === 'all' ? null : branchId;
    campusId = campusId === 'all' ? null : campusId;
    let apiParams = {
      branchId,
      campusId,
      endDate,
      startDate,
    };
    Promise.all([getPosDataByCount(apiParams), getPosDataByMoney(apiParams)])
      .then(([_countData = {}, _moneyData = {}]) => {
        const countData = [
          {
            typeName: 'A型',
            num: _countData.typeANum,
            children: [
              {
                name: '分享支付',
                num: _countData.sharePayANum,
              },
              {
                name: '微信支付',
                num: _countData.webChartPayANum,
              },
              {
                name: 'POS机',
                num: _countData.posANum,
              },
              {
                name: '支付宝支付',
                num: _countData.aliPayANum,
              },
              {
                name: '电子账户',
                num: _countData.electronicAccountANum,
              },
              {
                name: '现金笔数',
                num: _countData.cashANum,
              },
              {
                name: '第三方支付笔数',
                num: _countData.thirdPartyPaymentANum,
              },
              {
                name: '银行汇款',
                num: _countData.bankTransferANum,
              },
            ],
          },
          {
            typeName: 'B型',
            num: _countData.typeBNum,
            children: [
              {
                name: '分享支付',
                num: _countData.sharePayBNum,
              },
              {
                name: '微信支付',
                num: _countData.webChartPayBNum,
              },
              {
                name: 'POS机',
                num: _countData.posBNum,
              },
              {
                name: '支付宝支付',
                num: _countData.aliPayBNum,
              },
              {
                name: '电子账户',
                num: _countData.electronicAccountBNum,
              },
              {
                name: '现金笔数',
                num: _countData.cashBNum,
              },
              {
                name: '第三方支付笔数',
                num: _countData.thirdPartyPaymentBNum,
              },
              {
                name: '银行汇款',
                num: _countData.bankTransferBNum,
              },
            ],
          },
        ];
        const moneyData = [
          {
            typeName: 'A型',
            amount: _moneyData.typeAMoney,
            children: [
              {
                name: '分享支付',
                amount: _moneyData.sharePayAMoney,
              },
              {
                name: '微信支付',
                amount: _moneyData.webChartPayAMoney,
              },
              {
                name: 'POS机',
                amount: _moneyData.posAMoney,
              },
              {
                name: '支付宝支付',
                amount: _moneyData.aliPayAMoney,
              },
              {
                name: '电子账户',
                amount: _moneyData.electronicAccountAMoney,
              },
              {
                name: '现金金额',
                amount: _moneyData.cashAMoney,
              },
              {
                name: '第三方支付金额',
                amount: _moneyData.thirdPartyPaymentAMoney,
              },
              {
                name: '银行汇款',
                amount: _moneyData.bankTransferAMoney,
              },
            ],
          },
          {
            typeName: 'B型',
            amount: _moneyData.typeBMoney,
            children: [
              {
                name: '分享支付',
                amount: _moneyData.sharePayBMoney,
              },
              {
                name: '微信支付',
                amount: _moneyData.webChartPayBMoney,
              },
              {
                name: 'POS机',
                amount: _moneyData.posBMoney,
              },
              {
                name: '支付宝支付',
                amount: _moneyData.aliPayBMoney,
              },
              {
                name: '电子账户',
                amount: _moneyData.electronicAccountBMoney,
              },
              {
                name: '现金金额',
                amount: _moneyData.cashBMoney,
              },
              {
                name: '第三方支付金额',
                amount: _moneyData.thirdPartyPaymentBMoney,
              },
              {
                name: '银行汇款',
                amount: _moneyData.bankTransferBMoney,
              },
            ],
          },
        ];

        let totalnum = countData.reduce((total, current) => NP.plus(total, current['num']), 0);
        let totalAmount = moneyData.reduce((total, current) => NP.plus(total, current['amount']), 0);
        this.setState({ loading: false, num: totalnum + '笔', amount: totalAmount + '元', countData, moneyData });
        typeEcharts.resize();
        moneyEcharts.resize();
        typeEcharts.setOption(this.mapDataToTypeOption(countData, typeOption, 'num'));
        moneyEcharts.setOption(this.mapDataToTypeOption(moneyData, moneyOption, 'amount'));
      })
      .catch((e) => {
        this.setState({ loading: false });
      });
  };

  mapDataToTypeOption = (data, option, type) => {
    option.legend.data = [];
    option.series[0]['data'] = [];
    let totalNum = data.reduce((total, current) => total + current[type], 0);
    data.forEach((v, k) => {
      option.legend.data.push(v.typeName);
      option.series[0]['data'].push({ value: v[type], name: v.typeName });
    });
    option.legend.formatter = (name) => {
      let item = data.find((v, k) => v.typeName === name);
      let { children = [] } = item;
      let text = `{a|${name} ${this.calculatePercentage(item[type], totalNum)} ${
        type === 'num' ? '' : `${item[type]}元`
      }}`;
      children.map(
        (_item) =>
          (text += `\n {c||} {b|${_item.name} ${this.calculatePercentage(_item[type], totalNum)} ${
            type === 'num' ? '' : `${_item[type]}元`
          }}`),
      );
      return text;
    };

    return option;
  };

  /**
   *计算百分比
   * @param {num} a 除数
   * @param {num} b 被除数
   * @returns 百分比
   * @memberof Report
   */
  calculatePercentage(a, b) {
    if (a === 0 || b === 0) return '0%';
    return `${((a / b) * 100).toFixed(2)}%`;
  }

  /**
   * 重置查询条件
   */
  reset = () => {
    this.setState({ branchId: null, campusId: null, startDate: null, endDate: null });
  };
  /**
   * 分公司改变时 实时请求学习中心数据字典，清空学习中心id
   * @param branchId
   * @returns {Promise.<void>}
   */
  changeBranchData = async (branchId) => {
    this.setState({ branchId, campusId: null });
    if (branchId) {
      let dict = await loadCampusDict(branchId);
      this.setState({ campusDict: [...[{ val: 'all', text: '全部学习中心' }], ...dict] });
    } else {
      this.setState({ campusDict: [] });
    }
  };
  /**
   * 设置当天、本周、本月的起始终止时间
   * @param activeKey
   */
  onChange = (activeKey) => {
    //取本周，本月时间段
    let startDate, endDate;
    if (activeKey === 0) {
      startDate = endDate = moment().format('YYYY-MM-DD');
    } else if (activeKey === 1) {
      startDate = moment().startOf('week').format('YYYY-MM-DD');
      endDate = moment().endOf('week').format('YYYY-MM-DD');
    } else {
      startDate = moment().startOf('month').format('YYYY-MM-DD');
      endDate = moment().endOf('month').format('YYYY-MM-DD');
    }
    this.setState({ activeKey, startDate, endDate }, () => {
      this.loadData();
    });
  };
  getDisabledStartDate = (startDate) => {
    let { endDate } = this.state;
    if (!startDate || !endDate) {
      return false;
    }
    return moment(startDate).valueOf() > moment(endDate).valueOf();
  };
  getDisabledEndDate = (endDate) => {
    const { startDate } = this.state;
    if (!endDate || !startDate) {
      return false;
    }
    return moment(endDate).valueOf() < moment(startDate).valueOf();
  };

  render() {
    let {
      loading,
      activeKey,
      startDate,
      endDate,
      campusDict,
      campusId,
      branchDict,
      branchId,
      showTotal,
      showTotalTwo,
      num,
      amount,
      moneyData = [],
      countData = [],
    } = this.state;
    return (
      <Page
        loading={loading}
        footer={false}
        route={this.props.route}
        className={styles.OTOCourseCheck}
        ref={(c) => (this.pageRef = c)}
        subHeader={
          <div className={styles.searchBar}>
            <Tabs
              type="button"
              className={styles.tabs}
              onChange={(activeKey) => this.onChange(activeKey)}
              activeKey={activeKey}
            >
              <Tab title="今日"> </Tab>
              <Tab title="本周"> </Tab>
              <Tab title="本月"> </Tab>
            </Tabs>
            <SearchBar expand={false} onSearch={this.loadData} onReset={this.reset}>
              <Select
                placeholder="所属分公司"
                showSearch={true}
                dict={branchDict}
                value={branchId}
                onChange={(val) => this.changeBranchData(val)}
              />
              <Select
                placeholder="所属学习中心"
                showSearch={true}
                dict={campusDict}
                value={campusId}
                onChange={(val) => this.setState({ campusId: val })}
              />
              <DatePicker
                placeholder="开始日期"
                value={startDate}
                onChange={(val) => this.setState({ startDate: val })}
                disabledDate={(val) => this.getDisabledStartDate(val)}
              />
              <DatePicker
                placeholder="结束日期"
                value={endDate}
                onChange={(val) => this.setState({ endDate: val })}
                disabledDate={(val) => this.getDisabledEndDate(val)}
              />
            </SearchBar>
          </div>
        }
      >
        <Row style={{ display: countData.length > 0 ? 'block' : 'none', marginTop: 120 }}>
          <Col span={12}>
            {showTotal && (
              <div className={styles.total}>
                <span>共计</span>
                <span>{num}</span>
              </div>
            )}
            <div ref={(e) => (this.typeRate = e)} style={{ height: 450 }}>
              {' '}
            </div>
          </Col>
          <Col span={12}>
            {showTotalTwo && (
              <div className={styles.total}>
                <span>共计</span>
                <span>{amount}</span>
              </div>
            )}
            <div ref={(e) => (this.moneyRate = e)} style={{ height: 450 }}>
              {' '}
            </div>
          </Col>
        </Row>
        <div className={styles.noData} style={{ display: countData.length === 0 ? 'block' : 'none' }}>
          <span>
            <img src={noDataSvg} alt="" />
            <br />
            <span>暂无数据</span>
          </span>
        </div>
      </Page>
    );
  }
}
