import React from 'react';
import { connect } from 'dva';
import styles from './index.less';
import {
  loadCommonDict,
  loadBranchDict,
  loadCampusDict,
  getDictTextByVal,
  getDictValByText,
  loadBranchCampusDict,
} from 'UTIL/dict';
import {
  loadMiniClassScoreReport,
  loadMiniClassScoreReportDetail,
  exportMiniClassScoreReport,
  exportMiniClassScoreReportByClass,
} from './loaders';
import Page from 'COMPONENT/Page';
import SearchBar from 'COMPONENT/SearchBar';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import List from 'COMPONENT/List';
import Button from 'COMPONENT/Button';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import Input from 'COMPONENT/Input';
import DropdownMenu from 'COMPONENT/DropdownMenu';

const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;
const campusTempDict = [{ text: '请先选择分公司', val: '', disabled: true }];
const yesOrNoDict = [
  { text: '大于0', val: 1 },
  { text: '不大于0', val: 0 },
];

class MiniClassScore extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      brenchId: null, // 分公司Id,默认为 props 传进来的分公司 Id
      campusId: null, // 学习中心Id
      productVersion: null, // 年份Id
      productQuarter: null, // 季度Id
      grade: null, // 年级Id
      subject: null, // 科目Id
      pageSize: 10, // 分页大小

      miniClassName: '', // 搜索小班名称
      gradeExpandSubNum: null, // 总成绩人数是否大于0
      newsignNum: null, // 新签人数是否大于0
      classTypeId: null, // 班级类型 id
      classLevelId: null, // 班级层次 id
      classType: null, // 班级类型 文本值
      classLevel: null, // 班级层次 文本值
      courseSeries: null, // 产品系列
    };
  }

  componentWillReceiveProps(nextProps) {
    const { defaultBranch: brenchId } = nextProps;
    const { defaultBranch } = this.props;
    if (defaultBranch === null) {
      this.setState({ brenchId: [brenchId] });
    }
  }

  /**
   * 重置
   */
  onReset = () => {
    const { defaultBranch, onSearch, resetDict } = this.props;
    resetDict();
    this.setState({
      brenchId: defaultBranch,
      campusId: null, // 学习中心Id
      productVersion: null, // 年份Id
      productQuarter: null, // 季度Id
      grade: null, // 年级Id
      subject: null, // 科目Id
      miniClassName: null, //搜索小班名称
      gradeExpandSubNum: null, // 总成绩人数是否大于0
      newsignNum: null, // 新签人数是否大于0
      classTypeId: null, // 班级类型
      classLevelId: null, // 班级层次

      classType: null, // 班级类型 文本值
      classLevel: null, // 班级层次 文本值
      courseSeries: null, // 产品系列
    });
    onSearch({
      brenchId: defaultBranch,
      campusId: null, // 学习中心Id
      productVersion: null, // 年份Id
      productQuarter: null, // 季度Id
      grade: null, // 年级Id
      subject: null, // 科目Id
      miniClassName: null, //搜索小班名称
      gradeExpandSubNum: null, // 总成绩人数是否大于0
      newsignNum: null, // 新签人数是否大于0
      classTypeId: null, // 班级类型
      classLevelId: null, // 班级层次

      classType: null, // 班级类型 文本值
      classLevel: null, // 班级层次 文本值
      courseSeries: null, // 产品系列
    });
  };

  /**
   * 分页大小改变
   */
  onChange = (pagination, searchParams) => {
    const { onSearch } = this.props;
    onSearch(Object.assign(searchParams, { page: pagination.current, rows: pagination.pageSize }));
    this.setState({ pageSize: pagination.pageSize });
  };

  render() {
    const {
      branchDict,
      campusDict,
      yearDict,
      quarterDict,
      gradeDict,
      subjectDict,
      miniClassScoresReport,
      onSearch,
      openMiniClassScoreDetail,
      loading,
      hasExcelAuth,
      classTypeDict,
      classLevelDict,
      handleClassTypeChange,
      exportExcel,
      exportExcelByClass,
      courseSeriesDict,
    } = this.props;
    const {
      brenchId,
      campusId,
      productVersion,
      productQuarter,
      grade,
      subject,
      pageSize,
      miniClassName,
      gradeExpandSubNum,
      newsignNum,
      classTypeId,
      classLevelId,
      classType,
      classLevel,
      courseSeries,
    } = this.state;
    if (miniClassScoresReport === null) {
      return <span></span>;
    } else {
      // 学习中心字典根据选中分公司变化
      const campusDictPlus = brenchId
        ? campusDict &&
          campusDict
            .filter((branch) => brenchId.includes(branch.val))
            .reduce((pre, value) => {
              return pre.concat(value);
            }, [])
        : campusTempDict;
      // 列表数据拆分
      const { rows: dataSource, page: current, records, miniclassId } = miniClassScoresReport;
      const pagination = { current, total: records };
      const columns = [
        {
          title: '序号',
          dataIndex: 'key',
          render(currentValue, currentRowData, currentIndex) {
            return currentIndex + 1;
          },
          width: COL_WIDTH_XS,
        },
        { title: '小班名称', dataIndex: 'miniclassName' },
        { title: '分公司', dataIndex: 'brenchName' },
        { title: '学习中心', dataIndex: 'campusName' },
        { title: '年份', dataIndex: 'productVersion' },
        { title: '季度', dataIndex: 'productQuarter' },
        { title: '年级', dataIndex: 'grade' },
        { title: '科目', dataIndex: 'subject' },
        { title: '产品系列', dataIndex: 'courseSeries' },
        { title: '班级层次', dataIndex: 'classLevel' },
        { title: '总人数/成绩人数', dataIndex: 'totalGradeNum' },
        { title: '新签人数/成绩人数', dataIndex: 'newsignGradeNum' },
        { title: '扩科人数/成绩人数', dataIndex: 'expandGradeNum' },
        { title: '续报人数/成绩人数', dataIndex: 'resignGradeNum' },
        {
          title: '操作',
          dataIndex: 'miniclassId',
          render(currentValue, currentRowData) {
            return (
              <span
                className={styles.detailButton}
                onClick={() => {
                  openMiniClassScoreDetail(currentValue, currentRowData.miniclassName, currentRowData.classLevel);
                }}
              >
                详情
              </span>
            );
          },
        },
      ];

      const searchParams = {
        brenchId,
        campusId,
        productVersion,
        productQuarter,
        grade,
        subject,
        miniClassName,
        gradeExpandSubNum,
        newsignNum,
        classType,
        classLevel,
        courseSeries:
          courseSeries !== null
            ? courseSeriesDict
                .filter((courseSerie) => {
                  return courseSeries.includes(courseSerie.val) === true;
                })
                .map((courseSerie) => {
                  return courseSerie.text;
                })
                .join(',')
            : null,
      };

      // 导出菜单配置
      let menus = [
        {
          text: '以学生为单位',
          disabled: !hasExcelAuth,
          onClick: () => {
            exportExcel(searchParams);
          },
        },
        {
          text: '以班级为单位',
          disabled: !hasExcelAuth,
          onClick: () => {
            exportExcelByClass(searchParams);
          },
        },
      ];
      return (
        <Page name="小班成绩报表" route={this.props.route} loading={loading}>
          <div className={styles.header + ' ' + 'clearfix'}>
            <div className={styles.left}>
              <SearchBar
                className={styles.left}
                onSearch={() => {
                  onSearch(Object.assign(searchParams, { rows: pageSize }));
                }}
                onReset={() => {
                  this.onReset();
                }}
              >
                <Input
                  placeholder="搜索小班名称"
                  maxLength={20}
                  value={miniClassName}
                  onChange={(event) => {
                    let value = event.currentTarget.value;
                    this.setState({ miniClassName: value });
                  }}
                />
                <MultiSelect
                  placeholder="分公司"
                  dict={branchDict}
                  value={brenchId}
                  allowClear={false}
                  showSearch={true}
                  onChange={(brenchId) => this.setState({ brenchId, campusId: null })}
                  expand={true}
                />
                <MultiSelect
                  placeholder="学习中心"
                  dict={campusDictPlus}
                  value={campusId}
                  allowClear={false}
                  showSearch={true}
                  onChange={(campusId) => {
                    this.setState({ campusId });
                  }}
                />
                <MultiSelect
                  placeholder="年份"
                  dict={yearDict}
                  value={productVersion}
                  allowClear={false}
                  showSearch={true}
                  onChange={(productVersion) => {
                    this.setState({ productVersion });
                  }}
                />
                <MultiSelect
                  placeholder="产品季度"
                  dict={quarterDict}
                  value={productQuarter}
                  allowClear={false}
                  showSearch={true}
                  onChange={(productQuarter) => {
                    this.setState({ productQuarter });
                  }}
                />
                <MultiSelect
                  placeholder="年级"
                  dict={gradeDict}
                  value={grade}
                  allowClear={false}
                  showSearch={true}
                  onChange={(grade) => {
                    this.setState({ grade });
                  }}
                />
                <MultiSelect
                  placeholder="科目"
                  dict={subjectDict}
                  value={subject}
                  allowClear={false}
                  showSearch={true}
                  onChange={(subject) => {
                    this.setState({ subject });
                  }}
                />
                <MultiSelect
                  placeholder="产品系列"
                  dict={courseSeriesDict}
                  allowClear={true}
                  value={courseSeries}
                  onChange={(value) => {
                    this.setState({ courseSeries: value });
                  }}
                />
                <Select
                  placeholder="总成绩人数"
                  dict={yesOrNoDict}
                  value={gradeExpandSubNum}
                  allowClear={true}
                  onChange={(value) => {
                    this.setState({ gradeExpandSubNum: value });
                  }}
                />
                <Select
                  placeholder="新签/扩科人数"
                  dict={yesOrNoDict}
                  value={newsignNum}
                  onChange={(value) => {
                    this.setState({ newsignNum: value });
                  }}
                />
                <Select
                  placeholder="班级类型"
                  dict={classTypeDict}
                  allowClear={true}
                  value={classTypeId}
                  onChange={(value) => {
                    this.setState({ classLevel: null });
                    if (value !== null) {
                      let classType = classTypeDict.filter((classType) => {
                        return classType.val === value;
                      })[0].text;
                      this.setState({ classType });
                    } else {
                      this.setState({ classType: null });
                    }
                    this.setState({ classTypeId: value, classLevelId: null });
                    handleClassTypeChange(value);
                  }}
                />
                <MultiSelect
                  placeholder="班级层次"
                  dict={classLevelDict}
                  value={classLevelId}
                  onChange={(classLevelId) => {
                    if (classLevelId !== null) {
                      let classLevel = classLevelDict
                        .filter((classLevel) => {
                          return classLevelId.includes(classLevel.val);
                        })
                        .map((classLevel) => {
                          return classLevel.text;
                        })
                        .join(',');
                      this.setState({ classLevel });
                    } else {
                      this.setState({ classLevel: null });
                    }
                    this.setState({ classLevelId });
                  }}
                />
              </SearchBar>
            </div>
            <div className={styles.right}>
              {/* <Button className={styles.right} onClick={this.exportExcel}>导出excel</Button> */}
              <DropdownMenu className={styles.exportExcel} text="导出excel" hasArrow={false} menus={menus} />
            </div>
          </div>

          <List
            dataSource={dataSource}
            columns={columns}
            pagination={pagination}
            onChange={(pagination) => {
              this.onChange(pagination, searchParams);
            }}
          />
        </Page>
      );
    }
  }
}

class MiniClassScoreContainer extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: true,
      branchDict: null, // 分公司字典
      campusDict: null, // 学习中心字典
      yearDict: null, // 年份字典
      quarterDict: null, // 季度字典
      gradeDict: null, // 年级字典
      subjectDict: null, // 科目字典
      courseSeriesDict: null, // 产品系列 字典
      defaultBranch: null, // 默认分公司
      miniClassScoresReport: null, // 小班成绩报表

      classTypeDict: null, // 班级类型字典
      classLevelDict: null, // 班级层次字典
    };
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化
   */
  initContent = async () => {
    const [branchDict, campusDict, yearDict, quarterDict, gradeDict, subjectDict, classTypeDict, courseSeriesDict] =
      await Promise.all([
        loadBranchDict(),
        loadBranchCampusDict(),
        loadCommonDict('PRODUCT_VERSION'),
        loadCommonDict('PRODUCT_QUARTER'),
        loadCommonDict('STUDENT_GRADE'),
        loadCommonDict('SUBJECT'),
        loadCommonDict('CLASS_TYPE', 'SMALL_CLASS'),
        loadCommonDict('COURSE_SERIES', 'SMALL_CLASS'),
      ]);
    // 默认第一个分公司
    const defaultBranch = branchDict[0] ? branchDict[0].val : null;
    // 默认获取第一个分公司全部筛选项数据
    const miniClassScoresReport = await loadMiniClassScoreReport({ brenchId: defaultBranch, rows: 10 });
    this.setState({
      branchDict,
      campusDict,
      yearDict,
      quarterDict,
      gradeDict,
      subjectDict,
      classTypeDict,
      courseSeriesDict,
      defaultBranch,
      miniClassScoresReport,
    });
    this.setState({ loading: false });
  };

  /**
   * 根据班级类型获取班级层次字典
   */
  handleClassTypeChange = async (classTypeId) => {
    this.setState({ loading: true, classLevelDict: null });
    if (classTypeId) {
      let classLevelDict = loadCommonDict('CLASS_LEVEL', classTypeId);
      this.setState({ classLevelDict });
    }
    this.setState({ loading: false });
  };
  /**
   * 搜索小班成绩报表
   */
  onSearch = async (params) => {
    this.setState({ loading: true });
    const miniClassScoresReport = await loadMiniClassScoreReport(params);
    this.setState({ miniClassScoresReport });
    this.setState({ loading: false });
  };

  /**
   * 打开小班成绩详情
   */
  openMiniClassScoreDetail = (miniClassId, miniClassName, classLevelName) => {
    const { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: {
        path: '/MiniClassScoreReport/MiniClassScoreDetail',
        params: { miniClassId, miniClassName, classLevelName },
        refresh: true,
      },
    });
  };

  /**
   * 班级类型改变获取相应的班级层次
   */
  handleClassTypeChange = async (classTypeId) => {
    this.setState({ loading: true, classLevelDict: null, classLevel: null });
    if (classTypeId) {
      let classLevelDict = await loadCommonDict('CLASS_LEVEL', classTypeId);
      this.setState({ classLevelDict });
    }
    this.setState({ loading: false });
  };

  /**
   * 判断是否有导出excel权限
   */
  hasExcelAuth = () => {
    const { btnAuths } = this.props;
    return btnAuths.includes('SCORE_REPORT_EXPORT_EXCEL');
  };

  /**
   * 重置字典
   */
  resetDict = () => {
    this.setState({ classLevelDict: null });
  };

  /**
   * 导出 Excel
   */
  exportExcel = async (params) => {
    this.setState({ loading: true });
    // const { brenchId, campusId, grade, productQuarter, productVersion, subject, miniClassName, , gradeExpandSubNum, newsignNum, classType, classLevel  } = this.state
    // const params = { brenchId, campusId, grade, productQuarter, productVersion, subject, miniClassName, expandSubNum, gradeExpandSubNum, newsignNum, classType, classLevel  }
    const responseData = await exportMiniClassScoreReport(params).catch(() => {
      this.setState({ loading: false });
    });
    if (responseData) {
      const a = document.createElement('a');
      a.setAttribute('download', true);
      a.setAttribute('href', responseData.url);
      a.click();
      a.remove();
      this.setState({ loading: false });
    }
  };

  /**
   * 导出 Excel, 通过班级
   */
  exportExcelByClass = async (params) => {
    this.setState({ loading: true });
    // const { brenchId, campusId, productVersion, productQuarter, grade, subject, miniClassName, expandSubNum, gradeExpandSubNum, newsignNum, classType, classLevel } = this.state
    // let params = { brenchId, campusId, productVersion, productQuarter, grade, subject, miniClassName, expandSubNum, gradeExpandSubNum, newsignNum, classType, classLevel }
    const responseData = await exportMiniClassScoreReportByClass(params).catch(() => {
      this.setState({ loading: false });
    });
    if (responseData) {
      const a = document.createElement('a');
      a.setAttribute('download', true);
      a.setAttribute('href', responseData.url);
      a.click();
      a.remove();
      this.setState({ loading: false });
    }
  };
  render() {
    const { route } = this.props;
    const {
      branchDict,
      campusDict,
      yearDict,
      quarterDict,
      gradeDict,
      subjectDict,
      defaultBranch,
      miniClassScoresReport,
      loading,
      classTypeDict,
      classLevelDict,
      courseSeriesDict,
    } = this.state;
    return (
      <MiniClassScore
        loading={loading}
        branchDict={branchDict}
        campusDict={campusDict}
        yearDict={yearDict}
        quarterDict={quarterDict}
        gradeDict={gradeDict}
        subjectDict={subjectDict}
        classTypeDict={classTypeDict}
        classLevelDict={classLevelDict}
        defaultBranch={defaultBranch}
        miniClassScoresReport={miniClassScoresReport}
        onSearch={this.onSearch}
        openMiniClassScoreDetail={this.openMiniClassScoreDetail}
        route={route}
        hasExcelAuth={this.hasExcelAuth()}
        classTypeDict={classTypeDict}
        classLevelDict={classLevelDict}
        handleClassTypeChange={this.handleClassTypeChange}
        resetDict={this.resetDict}
        exportExcelByClass={this.exportExcelByClass}
        exportExcel={this.exportExcel}
        courseSeriesDict={courseSeriesDict}
      />
    );
  }
}

let mapStateToProps = (state) => {
  let account = state.account;
  return {
    account: account,
    btnAuths: state.account.btnAuths,
  };
};

export default connect(mapStateToProps, null)(MiniClassScoreContainer);
