import React from 'react';
import { connect } from 'dva';
import styles from './index.less';
import Page from 'COMPONENT/Page';
import Select from 'COMPONENT/Select';
import List from 'COMPONENT/List';
import Modal from 'COMPONENT/Modal';
import PreviewImgs from 'COMPONENT/PreviewImgs';
import { loadMiniClassScoreReport, loadMiniClassScoreReportDetail } from '../loaders';
import {
  loadCommonDict,
  loadBranchDict,
  loadCampusDict,
  getDictTextByVal,
  getDictValByText,
  loadBranchCampusDict,
} from 'UTIL/dict';
import Popover from 'COMPONENT/Popover';
import Icon from 'COMPONENT/Icon';
const matchDict = [
  { text: '是', val: 1 },
  { text: '否', val: 0 },
];

class MiniClassScoreDetail extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false,

      examTypeDictId: '', // 考试类型Id
      showImg: false, // 是否打开图片
      activeSrc: '', // 当前图片src
      isMatch: '', // 是否匹配

      pagination: { current: 1, total: 0, pageSize: 20 },
    };
  }

  /**
   * 查看答卷图片
   */
  checkImg = (index, imgUrlList) => {
    this.setState({ showImg: true, activeSrc: imgUrlList[index] });
  };

  componentWillReceiveProps(nextProps) {
    if (nextProps.miniClassScoreReportDetail !== null) {
      this.setState({ pagination: { current: 1, total: nextProps.miniClassScoreReportDetail.records, pageSize: 20 } });
    }
  }

  render() {
    const { miniClassScoreReportDetail, examTypeDict, onSearch, miniClassId, miniClassName, classLevelName } =
      this.props;
    const { loading, examTypeDictId, activeSrc, showImg, isMatch, pagination } = this.state;
    const checkImg = this.checkImg;
    if (miniClassScoreReportDetail === null) {
      return <span></span>;
    } else {
      const { rows: dataSource, page, records } = miniClassScoreReportDetail;
      const imgUrlList = miniClassScoreReportDetail.rows.map((reportDeatil) => {
        return reportDeatil.paperPic;
      });
      const columns = [
        {
          title: '序号',
          dataIndex: 'key',
          render(currentValue, currentRowData, currentIndex) {
            return currentIndex + 1;
          },
        },
        { title: '学生名称', dataIndex: 'stuName' },
        { title: '学生产品类型', dataIndex: 'productType' },
        { title: '科次来源', dataIndex: 'sourceFlag' },
        { title: '考试学习中心', dataIndex: 'campusName' },
        { title: '考试类型', dataIndex: 'examType' },
        { title: '试卷名称', dataIndex: 'paperName' },
        { title: '试卷总分', dataIndex: 'paperTotalScore' },
        { title: '试卷得分', dataIndex: 'paperScore' },
        { title: '得分等级', dataIndex: 'scoreLevel' },
        {
          title: '是否匹配',
          dataIndex: 'isMatch',
          render(currentValue, currentData, currentIndex) {
            if (currentValue === '1') {
              return '匹配';
            } else if (currentValue === '0') {
              return '不匹配';
            }
          },
          filterDropdownVisible: false,
          filterIcon: (
            <Popover overlayClassName="list-title-popover" content="得分等级与班级层次相同为匹配，否则为不匹配">
              <Icon className="list-title-icon" type="question_circle" size={12} />
            </Popover>
          ),
          filterDropdown: <span> </span>,
        },
        {
          title: '答卷图片',
          dataIndex: 'paperPic',
          render(currentValue, currentData, currentIndex) {
            if (currentValue !== '无') {
              return (
                <span
                  onClick={() => {
                    checkImg(currentIndex, imgUrlList);
                  }}
                >
                  查看
                </span>
              );
            } else {
              return <span>{currentValue}</span>;
            }
          },
        },
        {
          title: '报班时间',
          dataIndex: 'signupDate',
          render(currentValue) {
            const date = currentValue.slice(0, 10);
            return <span>{date}</span>;
          },
        },
      ];

      return (
        <Page loading={loading} name="小班成绩详情" route={this.props.route}>
          <div className={styles.miniClassName}>小班名称: {miniClassName}</div>
          <div className={styles.filter}>
            <Select
              className={styles.examType}
              placeholder="考试类型"
              dict={examTypeDict}
              value={examTypeDictId}
              allowClear={true}
              showSearch={true}
              onChange={(examTypeDictId) => {
                this.setState({ examTypeDictId });
                onSearch({ miniClassId, examTypeId: examTypeDictId, isMatch, classLevelName });
              }}
            />
            <Select
              className={styles.examType}
              placeholder="是否匹配"
              dict={matchDict}
              value={isMatch}
              allowClear={true}
              onChange={(isMatch) => {
                this.setState({ isMatch });
                onSearch({ miniClassId, examTypeId: examTypeDictId, isMatch, classLevelName });
              }}
            />
          </div>
          <List
            dataSource={dataSource}
            columns={columns}
            pagination={pagination}
            onChange={async (pagination) => {
              this.setState({ loading: true });
              await onSearch({
                miniClassId,
                examTypeId: examTypeDictId,
                isMatch,
                classLevelName,
                page: pagination.current,
                rows: pagination.pageSize,
              });
              this.setState({
                pagination: { current: pagination.current, total: records, pageSize: pagination.pageSize },
                loading: false,
              });
            }}
            locale={{ filterTitle: '' }}
          />
          {showImg && (
            <Modal
              width={800}
              okBtnShow={false}
              cancelBtnShow={false}
              closable={true}
              onClose={() => this.setState({ showImg: false })}
            >
              <PreviewImgs list={imgUrlList} activeSrc={activeSrc} showThumbnail={false} showButton={false} />
            </Modal>
          )}
        </Page>
      );
    }
  }
}

class MiniClassScoreDetailContainer extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      miniClassScoreReportDetail: null, // 小班成绩报表详情
      examTypeDict: null, // 考试类型
    };
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化内容
   */
  initContent = async () => {
    const { miniClassId, classLevelName } = this.props.params;
    const [miniClassScoreReportDetail, examTypeDict] = await Promise.all([
      loadMiniClassScoreReportDetail({ miniClassId, classLevelName }),
      loadCommonDict('BRANCH_EXAM_TYPE'),
    ]);
    this.setState({ miniClassScoreReportDetail, examTypeDict });
  };

  /**
   * 搜索小班成绩报表详情
   */
  onSearch = async (params) => {
    const miniClassScoreReportDetail = await loadMiniClassScoreReportDetail(params);
    this.setState({ miniClassScoreReportDetail });
  };

  render() {
    const { miniClassId, miniClassName, classLevelName } = this.props.params;
    const { route } = this.props;
    const { miniClassScoreReportDetail, examTypeDict } = this.state;
    return (
      <MiniClassScoreDetail
        route={route}
        miniClassScoreReportDetail={miniClassScoreReportDetail}
        examTypeDict={examTypeDict}
        onSearch={this.onSearch}
        miniClassId={miniClassId}
        miniClassName={miniClassName}
        classLevelName={classLevelName}
        paginationChange={this.paginationChange}
      ></MiniClassScoreDetail>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    params: state.main.pageParams['/MiniClassScoreReport/MiniClassScoreDetail'],
  };
};

export default connect(mapStateToProps, null)(MiniClassScoreDetailContainer);
