import React from 'react';
import { connect } from 'dva';
import {
  storeName,
  contractTypeDict,
  contractPrdStatusDict,
  dateFormat,
  productTypeAllow,
  SMALL_CLASS_VAL,
} from './store/config';
import { SearchButton, MultiSelect, Select, Input, Validator } from 'COMPONENT';
import { DatePicker } from 'antd';
import {
  buildBranchCampusDict,
  buildProductSeriesDict,
  buildClassTypeDict,
  buildClassLevelDict,
} from 'MODEL/dict/utils';
import { clone } from 'lodash';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { amountRateDict } from 'CONST/dicts';
import { loginUser } from 'UTIL/common';
import { getDictTextByVal } from 'UTIL/dict';

const RangePicker = DatePicker.RangePicker;

class Search extends React.Component {
  constructor(props) {
    super(props);

    this.updateProps = this.updateProps.bind(this);
    this.updatePropsWithEffects = this.updatePropsWithEffects.bind(this);
    this.updateValue = this.updateValue.bind(this);
    this.updateDateRange = this.updateDateRange.bind(this);
    this.updateBranch = this.updateBranch.bind(this);
    this.updateProductType = this.updateProductType.bind(this);
    this.updateClassType = this.updateClassType.bind(this);
    this.updateProductName = this.updateProductName.bind(this);
    this.updateClassName = this.updateClassName.bind(this);
    this.updateNeedValidator = this.updateNeedValidator.bind(this);
    this.errorChange = this.errorChange.bind(this);
    this.onSearch = this.onSearch.bind(this);
    this.onReset = this.onReset.bind(this);

    this.isTwoTeacher = loginUser.isTwoTeacherSym();
    this.state = {
      isNeedValidator: false,
      error: {},
    };
  }

  /**
   * 更新数据
   * */
  updateProps(params) {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: params,
    });
  }

  updatePropsWithEffects(type, val) {
    return new Promise((resolve) => {
      this.props.dispatch({
        type: `${storeName}/updateStateWithEffects`,
        payload: {
          data: { [type]: val },
          callback() {
            resolve();
          },
        },
      });
    });
  }

  updateValue(type, val) {
    this.updateProps({ [type]: val });
  }

  updateDateRange(moments, dates) {
    this.updateProps({
      sKey_momentRange: moments,
      sKey_dateRange: dates,
    });
  }

  updateBranch(val) {
    let payload = { sKey_branch: val };
    let sKey_campus = clone(this.props.sKey_campus || []);
    if (val && val.length > 0) {
      let filterBranchCampusDict = buildBranchCampusDict(val, this.props.branchCampusDictOrigin);
      let filterCampusDict = filterBranchCampusDict.map((item) => (item ? item.options : []));
      let filterValArr = filterCampusDict.reduce((arr, current) => arr.concat(current), []).map((item) => item.val);
      sKey_campus = sKey_campus.filter((val) => filterValArr.includes(val));
      payload.sKey_campus = sKey_campus;
    }
    this.updateProps(payload);
  }

  /**
   * 选择产品类型
   *
   * @param val
   */
  updateProductType(val) {
    let payload = { sKey_productType: val };

    if (val) {
      const { sKey_productSeries, sKey_classType, sKey_classLevel, productTypeTree } = this.props;
      if (sKey_productSeries) {
        const _productSeriesDict = buildProductSeriesDict(productTypeTree, val) || [];
        // 是否需要清空产品系列
        if (_productSeriesDict.findIndex((item) => item.val === sKey_productSeries) === -1) {
          payload.sKey_productSeries = null;
        }
      }
      if (sKey_classType) {
        const _classTypeDict = buildClassTypeDict(productTypeTree, val) || [];
        // 是否需要清空班级类型
        if (_classTypeDict.findIndex((item) => item.val === sKey_classType) === -1) {
          payload.sKey_classType = null;
        }
      }
      if (sKey_classLevel) {
        const _classLevelDict = buildClassLevelDict(productTypeTree, val, sKey_classType) || [];
        // 是否需要清空班级层次
        if (_classLevelDict.findIndex((item) => item.val === sKey_classLevel) === -1) {
          payload.sKey_classLevel = null;
        }
      }
    }

    if (this.isTwoTeacher && val !== SMALL_CLASS_VAL) {
      payload.sKey_productGroup = null;
    }

    this.updateProps(payload);
  }

  updateClassType(val) {
    let payload = { sKey_classType: val };
    if (val) {
      const { sKey_productType, sKey_classLevel, productTypeTree } = this.props;
      if (sKey_classLevel) {
        const _classLevelDict = buildClassLevelDict(productTypeTree, sKey_productType, val) || [];
        // 是否需要清空班级层次
        if (_classLevelDict.findIndex((item) => item.val === sKey_classLevel) === -1) {
          payload.sKey_classLevel = null;
        }
      }
    }
    this.updateProps(payload);
  }

  /**
   * 根据季度值切换期数/星期数字典
   *
   * @param quarterVal
   * @return {Array}
   */
  buildPhaseDict = (quarterVal) => {
    const { quarterDict, phaseDict, weekDayDict } = this.props;
    const quarterText = getDictTextByVal(quarterDict, quarterVal);

    if (!quarterVal) {
      return phaseDict;
    }

    if (quarterText === '暑假' || quarterText === '寒假') {
      return phaseDict;
    } else if (quarterText === '春季' || quarterText === '秋季' || quarterText === '全年') {
      return weekDayDict;
    } else {
      return [];
    }
  };

  /**
   * 季度改变
   *
   * @param val {string} 季度
   * @param resultPhaseDict {Array} 切换后的期数字典
   */
  updateQuarterId = (val, resultPhaseDict) => {
    const { sKey_phaseId } = this.props;
    // 判断是否需要清空期数
    const shouldClearPhase = sKey_phaseId && !resultPhaseDict.some((item) => item.val === sKey_phaseId);
    const phaseId = shouldClearPhase ? null : sKey_phaseId;

    this.updatePropsWithEffects('sKey_quarter', val);
    this.updatePropsWithEffects('sKey_phaseId', phaseId);
  };

  /**
   * 更新产品名称
   *
   * @param ele
   * @return {Promise<void>}
   */
  async updateProductName(ele) {
    let val = ele.target.value;
    await this.updatePropsWithEffects('sKey_productName', val);
    this.updateNeedValidator({ sKey_productName: val });
  }

  /**
   * 更新班级名称
   *
   * @param ele
   * @return {Promise<void>}
   */
  async updateClassName(ele) {
    let val = ele.target.value;
    await this.updatePropsWithEffects('sKey_className', val);
    this.updateNeedValidator({ sKey_className: val });
  }

  /**
   * 更新期数
   *
   * @param val {string} 期数
   * @return {Promise<void>}
   */
  updatePhaseId = async (val) => {
    await this.updatePropsWithEffects('sKey_phaseId', val);
    this.updateNeedValidator({ sKey_phaseId: val });
  };

  /**
   * 更新表单校验规则
   *
   * @param params
   */
  updateNeedValidator(params) {
    const productName = params.sKey_productName || this.props.sKey_productName;
    const className = params.sKey_className || this.props.sKey_className;
    const phaseId = params.sKey_phaseId || this.props.sKey_phaseId;

    this.setState({ isNeedValidator: Boolean(productName || className || phaseId) });
  }

  errorChange(err, keyName) {
    this.setState((prevState) => ({
      error: {
        ...prevState.error,
        [keyName]: err,
      },
    }));
  }

  /**
   * 搜索
   */
  onSearch() {
    const { sKey_productName, sKey_className, sKey_phaseId } = this.props;
    const isSearchByPrdNameOrClassName = sKey_productName || sKey_className || sKey_phaseId;
    const isPrdTypeOrQuarterEmpty = !(this.props.sKey_productType && this.props.sKey_quarter);

    // 通过产品名称、班级名称、期数进行筛选时产品类型和季度必填
    if (isSearchByPrdNameOrClassName && isPrdTypeOrQuarterEmpty) {
      let errorMsgStart = '';
      if (sKey_productName) {
        errorMsgStart = '产品名称';
      }

      if (sKey_className) {
        errorMsgStart = errorMsgStart === '' ? '班级名称' : `${errorMsgStart}、班级名称`;
      }

      if (sKey_phaseId) {
        errorMsgStart = errorMsgStart === '' ? '期数' : `${errorMsgStart}、期数`;
      }

      let errorMsgKeyword = '';
      if (!this.props.sKey_productType) {
        errorMsgKeyword = '产品类型';
      }

      if (!this.props.sKey_quarter) {
        errorMsgKeyword = errorMsgKeyword === '' ? '季度' : `${errorMsgKeyword}、季度`;
      }

      return showMessage(`通过${errorMsgStart}搜索时，${errorMsgKeyword}为必选项`, 'error');
    }

    // 起始日期未选时
    if (!(this.props.sKey_momentRange[0] && this.props.sKey_momentRange[1])) {
      return showMessage('起始日期为必填项！', 'error');
    }

    this.props.onSearch();
  }

  onReset() {
    this.setState(
      {
        isNeedValidator: false,
        error: {},
      },
      () => {
        this.props.onReset();
      },
    );
  }

  render() {
    const resultPhaseDict = this.buildPhaseDict(this.props.sKey_quarter);

    return (
      <SearchButton
        girdConfig={[2]}
        searchBtnLoading={this.props.searchBtnLoading}
        onSearch={this.onSearch}
        onReset={this.onReset}
      >
        <Validator
          value={this.props.sKey_momentRange[0] && this.props.sKey_momentRange[1] ? 'true' : null}
          rules={['required']}
          silent={!Boolean(this.state.error.sKey_momentRange)}
          showStatus={true}
          showTips={false}
          onErrorChange={(err) => this.errorChange(err, 'sKey_momentRange')}
        >
          <RangePicker
            value={this.props.sKey_momentRange}
            onChange={this.updateDateRange}
            placeholder={['开始时间', '结束时间']}
            format={dateFormat}
            showTime={false}
            size="large"
          />
        </Validator>
        <MultiSelect
          placeholder="合同分校"
          dict={this.props.branchDict}
          value={this.props.sKey_branch}
          onChange={this.updateBranch}
          showSearch={true}
        />
        <MultiSelect
          placeholder="合同学习中心"
          dict={this.props.branchCampusDict}
          value={this.props.sKey_campus}
          onChange={(val) => this.updateValue('sKey_campus', val)}
          showSearch={true}
        />
        <Select
          placeholder="产品类型"
          dict={this.props.productTypeTree}
          value={this.props.sKey_productType}
          onChange={this.updateProductType}
          validator={
            this.state.isNeedValidator
              ? {
                  rules: ['required'],
                  silent: !Boolean(this.state.error.sKey_productType),
                  showStatus: true,
                  showTips: false,
                  onErrorChange: (err) => this.errorChange(err, 'sKey_productType'),
                }
              : null
          }
        />
        {this.isTwoTeacher && (
          <Select
            placeholder="产品组"
            dict={this.props.productGroupDict}
            value={this.props.sKey_productGroup}
            onChange={(val) => this.updateValue('sKey_productGroup', val)}
            disabled={this.props.sKey_productType !== SMALL_CLASS_VAL}
            allowClear={true}
          />
        )}
        <Select
          placeholder="产品系列"
          dict={this.props.productSeriesDict}
          value={this.props.sKey_productSeries}
          onChange={(val) => this.updateValue('sKey_productSeries', val)}
        />
        <Select
          placeholder="产品性质"
          dict={this.props.productCharacterDict}
          value={this.props.sKey_productCharacter}
          onChange={(val) => this.updateValue('sKey_productCharacter', val)}
        />
        <Select
          placeholder="班级类型"
          dict={this.props.classTypeDict}
          value={this.props.sKey_classType}
          onChange={this.updateClassType}
          disabled={!this.props.sKey_productType}
        />
        <Select
          placeholder="班级层次"
          dict={this.props.classLevelDict}
          value={this.props.sKey_classLevel}
          onChange={(val) => this.updateValue('sKey_classLevel', val)}
          disabled={!this.props.sKey_productType}
        />
        <Select
          placeholder="签单类型"
          dict={contractTypeDict}
          value={this.props.sKey_contractType}
          onChange={(val) => this.updateValue('sKey_contractType', val)}
        />
        <Select
          placeholder="季度"
          dict={this.props.quarterDict}
          value={this.props.sKey_quarter}
          onChange={(val) => this.updateQuarterId(val, resultPhaseDict)}
          validator={
            this.state.isNeedValidator
              ? {
                  rules: ['required'],
                  silent: !Boolean(this.state.error.sKey_quarter),
                  showStatus: true,
                  showTips: false,
                  onErrorChange: (err) => this.errorChange(err, 'sKey_quarter'),
                }
              : null
          }
        />
        <Select
          placeholder="期数/星期数"
          dict={resultPhaseDict}
          value={this.props.sKey_phaseId}
          onChange={this.updatePhaseId}
        />
        <MultiSelect
          placeholder="报读状态"
          dict={contractPrdStatusDict}
          value={this.props.sKey_contractPrdStatus}
          onChange={(val) => this.updateValue('sKey_contractPrdStatus', val)}
        />
        <Select
          placeholder="科目"
          dict={this.props.subjectDict}
          value={this.props.sKey_subject}
          onChange={(val) => this.updateValue('sKey_subject', val)}
          showSearch={true}
        />
        <Input
          placeholder="合同编号"
          value={this.props.sKey_contractId}
          onChange={(ele) => this.updateValue('sKey_contractId', ele.target.value)}
        />
        <Input placeholder="产品名称" value={this.props.sKey_productName} onChange={this.updateProductName} />
        <Input placeholder="班级名称" value={this.props.sKey_className} onChange={this.updateClassName} />
        <Input
          placeholder="学生姓名"
          value={this.props.sKey_studentName}
          onChange={(ele) => this.updateValue('sKey_studentName', ele.target.value)}
        />
        <Input
          placeholder="业绩人"
          value={this.props.sKey_performancePersonName}
          onChange={(ele) => this.updateValue('sKey_performancePersonName', ele.target.value)}
        />
        <Select
          placeholder="招生科次"
          dict={amountRateDict}
          value={this.props.sKey_amountRate}
          onChange={(val) => this.updateValue('sKey_amountRate', val)}
        />
      </SearchButton>
    );
  }
}

let productTypeTreeStorage = null;

const mapStateToProps = (state) => {
  const { productGroup } = state.dict;
  const isTwoTeacher = loginUser.isTwoTeacherSym();

  // 过滤学习中心字典
  let branchCampusDict = buildBranchCampusDict(state[storeName].sKey_branch, state.dict.branchCampus);
  // 过滤与产品类型相关字典
  if (!productTypeTreeStorage && state.dict.productTypeTree) {
    productTypeTreeStorage = state.dict.productTypeTree.filter((item) => productTypeAllow.includes(item.val));
    productTypeTreeStorage.forEach((item) => {
      if (item.val === SMALL_CLASS_VAL) {
        item.text = isTwoTeacher ? '小班' : '面授';
      }
    });
  }
  let sKey_productType = state[storeName].sKey_productType;

  return {
    // 字典
    branchDict: state.dict.branch,
    branchCampusDict,
    branchCampusDictOrigin: state.dict.branchCampus,
    productCharacterDict: state.dict.productCharacter,
    quarterDict: state.dict.quarter,
    phaseDict: state.dict.phase,
    weekDayDict: state.dict.weekDay,
    subjectDict: state.dict.subject,
    productTypeTree: productTypeTreeStorage,
    productSeriesDict: buildProductSeriesDict(productTypeTreeStorage, sKey_productType),
    productGroupDict: productGroup ? productGroup['SMALL_CLASS'] : null,
    classTypeDict: buildClassTypeDict(productTypeTreeStorage, sKey_productType),
    classLevelDict: buildClassLevelDict(productTypeTreeStorage, sKey_productType, state[storeName].sKey_classType),
    // 筛选项值
    sKey_momentRange: state[storeName].sKey_momentRange,
    sKey_branch: state[storeName].sKey_branch,
    sKey_campus: state[storeName].sKey_campus,
    sKey_productType,
    sKey_productGroup: state[storeName].sKey_productGroup,
    sKey_productSeries: state[storeName].sKey_productSeries,
    sKey_productCharacter: state[storeName].sKey_productCharacter,
    sKey_classType: state[storeName].sKey_classType,
    sKey_classLevel: state[storeName].sKey_classLevel,
    sKey_contractType: state[storeName].sKey_contractType,
    sKey_quarter: state[storeName].sKey_quarter,
    sKey_phaseId: state[storeName].sKey_phaseId,
    sKey_contractPrdStatus: state[storeName].sKey_contractPrdStatus,
    sKey_subject: state[storeName].sKey_subject,
    sKey_contractId: state[storeName].sKey_contractId,
    sKey_productName: state[storeName].sKey_productName,
    sKey_className: state[storeName].sKey_className,
    sKey_studentName: state[storeName].sKey_studentName,
    sKey_performancePersonName: state[storeName].sKey_performancePersonName,
    sKey_amountRate: state[storeName].sKey_amountRate,
  };
};

export default connect(mapStateToProps)(Search);
