import React from 'react';
import { connect } from 'dva';
import { Page, Button, List } from 'COMPONENT';
import Info from './components/Info';
import { storeName } from './store/config';
import Search from './Search';

const { COL_WIDTH_M, COL_WIDTH_L, createColTips } = List;
const InfoWrap = Info.InfoWrap;

/**
 * 创建按字数所占表格单元格宽度
 *
 * return { function }
 * */
function createSetWidth() {
  const storage = {};
  return function (fontNum, fontSize = 12) {
    let storageKey = fontNum.toString() + fontSize.toString();
    if (!storage[storageKey]) {
      const padding = 15;
      storage[storageKey] = fontNum * fontSize + padding * 2;
    }
    return storage[storageKey];
  };
}
const setWidth = createSetWidth();

class PerformanceStatistics extends React.Component {
  constructor(props) {
    super(props);

    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.initDefault = this.initDefault.bind(this);
    this.loadData = this.loadData.bind(this);
    this.updatePage = this.updatePage.bind(this);
    this.changePage = this.changePage.bind(this);
    this.search = this.search.bind(this);
    this.reset = this.reset.bind(this);
    this.exportExcel = this.exportExcel.bind(this);
    this.beforeDestroy = this.beforeDestroy.bind(this);

    this.state = {
      name: '业绩科次统计',
      loading: false,
    };
    this.columns = [
      { title: '合同学习中心', dataIndex: 'contractCampusName', key: 'contractCampusName', width: setWidth(5) },
      { title: '业绩人', dataIndex: 'performancePersonName', key: 'performancePersonName', width: setWidth(4) },
      { title: '学生姓名', dataIndex: 'studentName', key: 'studentName', width: setWidth(4) },
      { title: '合同编号', dataIndex: 'contractId', key: 'contractId', width: COL_WIDTH_L },
      { title: '合同日期', dataIndex: 'contractDate', key: 'contractDate', width: COL_WIDTH_M },
      { title: '产品名称', dataIndex: 'productName', key: 'productName', width: setWidth(8) },
      { title: '班级名称', dataIndex: 'className', key: 'className', width: setWidth(8) },
      { title: '优惠', dataIndex: 'promotionNames', key: 'promotionNames', width: setWidth(8) },
      {
        title: '报读状态',
        dataIndex: 'statusName',
        key: 'statusName',
        width: setWidth(5.5),
        ...createColTips({
          content: (
            <div>
              <p>结课：指退班并且退费或退回电子账户的课程；</p>
              <p>结束：指所在班级所有课程已全部结束；</p>
              <p>无效：指产品退款并取消合同；</p>
              <p>冻结：申请退费未完成审批的状态；</p>
            </div>
          ),
        }),
      },
      { title: '招生科次', dataIndex: 'amountRate', key: 'amountRate', width: setWidth(4) },
      { title: '类型', dataIndex: 'productTypeName', key: 'productTypeName', width: setWidth(5) },
      { title: '产品系列', dataIndex: 'productSeriesName', key: 'productSeriesName', width: setWidth(4) },
      { title: '产品性质', dataIndex: 'productCharacterName', key: 'productCharacterName', width: setWidth(4) },
      { title: '产品季度', dataIndex: 'quarterName', key: 'quarterName', width: setWidth(4) },
      { title: '科目', dataIndex: 'subjectName', key: 'subjectName', width: setWidth(3) },
      { title: '期数/期数', dataIndex: 'phaseName', key: 'phaseName', width: setWidth(5) },
      { title: '应收金额', dataIndex: 'totalAmount', key: 'totalAmount', width: setWidth(4) },
      { title: '待收金额', dataIndex: 'pendingAmount', key: 'pendingAmount', width: setWidth(4) },
    ];
    this.toolBarStyle = { marginRight: 20 };
  }

  componentDidMount() {
    this.init();
  }

  componentWillUnmount() {
    this.beforeDestroy();
  }

  async init() {
    this.setState({ loading: true });
    await new Promise(this.initDict);
    await new Promise(this.initDefault);
    await new Promise(this.loadData);
    this.setState({ loading: false });
  }

  initDict(resolve) {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: [
          'branch',
          'branchCampus',
          'productTypeTree',
          'productCharacter',
          { name: 'productGroup', params: 'SMALL_CLASS' },
          'quarter',
          'subject',
          'phase',
          'weekDay',
        ],
        callback() {
          resolve();
        },
      },
    });
  }

  initDefault(resolve) {
    this.props.dispatch({
      type: `${storeName}/initSKey`,
      payload: {
        callback() {
          resolve();
        },
      },
    });
  }

  loadData(resolve) {
    this.props.dispatch({
      type: `${storeName}/loadData`,
      payload: {
        callback() {
          resolve();
        },
      },
    });
  }

  updatePage(pageCurrent, pageSize) {
    let payload = {};

    if (pageSize && pageSize !== this.props.pageSize) {
      payload.pageCurrent = 1;
      payload.pageSize = pageSize;
    } else {
      payload.pageCurrent = pageCurrent;
    }

    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload,
    });
  }

  async changePage({ current, pageSize }) {
    this.updatePage(current, pageSize);
    this.setState({ loading: true });
    await new Promise(this.loadData);
    this.setState({ loading: false });
  }

  search() {
    this.changePage({
      current: 1,
      pageSize: this.props.pageSize,
    });
  }

  reset() {
    this.props.dispatch({
      type: `${storeName}/resetSKey`,
    });
  }

  exportExcel() {
    this.setState({ loading: true });
    const _self = this;
    this.props.dispatch({
      type: `${storeName}/exportExcel`,
      payload: {
        callback() {
          _self.setState({ loading: false });
        },
      },
    });
  }

  beforeDestroy() {
    this.props.dispatch({
      type: `${storeName}/resetPage`,
    });
  }

  render() {
    return (
      <Page
        loading={this.state.loading}
        route={this.props.route}
        headerToolBar={
          <div>
            {this.props.exportBtnAuth && (
              <Button onClick={this.exportExcel} type="default" style={this.toolBarStyle}>
                导出excel
              </Button>
            )}
            <Search searchBtnLoading={this.state.loading} onSearch={this.search} onReset={this.reset} />
          </div>
        }
        subHeader={
          <InfoWrap>
            <Info label="招生科次" text={this.props.studentAmountRate} />
            <Info label="常规科次" text={this.props.normalAmountRate} />
            <Info label="引流科次" text={this.props.yinLiuAmountRate} />
            <Info label="有效科次" text={this.props.validAmountRate} />
            <Info label="退费科次" text={this.props.refundAmountRate} />
          </InfoWrap>
        }
        fitWindowHeight
      >
        <List
          columns={this.columns}
          dataSource={this.props.list}
          pagination={{
            total: this.props.total,
            current: this.props.pageCurrent,
            pageSize: this.props.pageSize,
          }}
          onChange={this.changePage}
        />
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    exportBtnAuth: state.account.btnAuths.includes('PERFORMANCE_STATISTICS_EXCEL'),
    studentAmountRate: state[storeName].studentAmountRate.toString(),
    normalAmountRate: state[storeName].normalAmountRate.toString(),
    yinLiuAmountRate: state[storeName].yinLiuAmountRate.toString(),
    validAmountRate: state[storeName].validAmountRate.toString(),
    refundAmountRate: state[storeName].refundAmountRate.toString(),
    list: state[storeName].list,
    total: state[storeName].total,
    pageSize: state[storeName].pageSize,
    pageCurrent: state[storeName].pageCurrent,
  };
};

export default connect(mapStateToProps)(PerformanceStatistics);
