import { storeName, dateFormat, contractTypeConf } from './config';
import { errorsHandling } from './utils';
import { clone } from 'lodash';
import moment from 'moment';
import { getStaffDistributionStatisticsList, getStaffDistributionStatisticsInfo } from 'SERVICE/contract';
import { exportStaffDistributionStatisticsListExcel } from 'SERVICE/exportFile';

// 业绩科次统计默认state
const defaultState = {
  sKey: {
    sKey_momentRange: [null, null],
    sKey_dateRange: [null, null],
    // 可多选
    sKey_branch: null,
    // 可多选
    sKey_campus: null,
    sKey_productType: null,
    sKey_productGroup: null,
    sKey_productSeries: null,
    sKey_productCharacter: null,
    sKey_classType: null,
    sKey_classLevel: null,
    sKey_contractType: null,
    sKey_quarter: null,
    sKey_phaseId: null,
    // 报读状态(合同产品状态), 可多选
    sKey_contractPrdStatus: null,
    sKey_subject: null,
    sKey_contractId: null,
    sKey_productName: null,
    sKey_className: null,
    sKey_studentName: null,
    sKey_performancePersonName: null,
    sKey_amountRate: null,
  },
  statisticalNum: {
    // 统计数据
    studentAmountRate: 0,
    normalAmountRate: 0,
    yinLiuAmountRate: 0,
    validAmountRate: 0,
    refundAmountRate: 0,
  },
  list: {
    // 列表数据
    list: [],
    total: 0,
    pageSize: 10,
    pageCurrent: 1,
  },
};

function setProductName(productType) {
  return productType === 'SMALL_CLASS' ? '面授' : '双师';
}
// 构建列表/合计/导出的参数
function _buildMainParams(state) {
  const _state = state[storeName];
  return {
    startDate: _state.sKey_dateRange[0],
    endDate: _state.sKey_dateRange[1],
    // 改为合同分公司/学习中心
    contractBranchIdArr: _state.sKey_branch,
    contractCampusIdArr: _state.sKey_campus,

    productType: _state.sKey_productType,
    productGroupId: _state.sKey_productGroup,
    courseSeriesId: _state.sKey_productSeries,
    productCharacterId: _state.sKey_productCharacter,
    classTypeId: _state.sKey_classType,
    classLevelId: _state.sKey_classLevel,
    contractType: _state.sKey_contractType,
    productQuarterId: _state.sKey_quarter,
    phaseId: _state.sKey_phaseId,
    contractProductStatus: _state.sKey_contractPrdStatus,
    subjectId: _state.sKey_subject,
    contractId: _state.sKey_contractId,
    productName: _state.sKey_productName,
    className: _state.sKey_className,
    studentName: _state.sKey_studentName,
    distributionStaffName: _state.sKey_performancePersonName,
    page: _state.pageCurrent,
    rows: _state.pageSize,
    getTotal: false,

    amountRateTag: _state.sKey_amountRate,
  };
}

export default {
  namespace: storeName,
  state: {
    ...clone(defaultState.sKey),
    ...clone(defaultState.statisticalNum),
    ...clone(defaultState.list),
    // 筛选条件初始化缓存
    sKeyStorage: null,
  },
  reducers: {
    updateState(state, { payload }) {
      return {
        ...state,
        ...payload,
      };
    },
    // 更新业绩科次统计页面 - 列表与统计数据
    updatePageData(state, { payload: { list, total, amountRate } }) {
      let payload = {};
      if (list) {
        payload.list = list.map((item) => ({
          ...item,
          performancePersonName: item.distributeStaffName,
          contractDate: item.signDate,
          contractAndPrdType: `${contractTypeConf[item.contractType]}/${setProductName(item.productType)}`,
          productSeriesName: item.courseSeriesName,
          quarterName: item.productQuarterName,
          totalAmount: item.realAmount,
          statusName: item.contractProductStatusName,
        }));
      }
      if (typeof total === 'number') {
        payload.total = total;
      }
      if (amountRate) {
        payload.studentAmountRate = amountRate.totalAmountRate || 0;
        payload.normalAmountRate = amountRate.commonAmountRate || 0;
        payload.yinLiuAmountRate = amountRate.drainageAmountRate || 0;
        payload.validAmountRate = amountRate.validAmountRate || 0;
        payload.refundAmountRate = amountRate.refundAmountRate || 0;
      }
      return {
        ...state,
        ...payload,
      };
    },
  },
  effects: {
    *initSKey({ payload: { callback } }, { select, put }) {
      let initData = yield select((state) => {
        if (state[storeName].sKeyStorage) {
          return state[storeName].sKeyStorage;
        } else {
          const { brenchId, campusId } = state.account.user;
          const isGroup = brenchId === campusId;
          let sKey_branch = isGroup ? [state.dict.branch[0].val] : [brenchId];
          let sKey_campus = isGroup ? null : [campusId];
          let today = new Date();
          let startMoment = moment(today).startOf('month');
          let endMoment = moment(today);
          return {
            sKey_branch,
            sKey_campus,
            sKey_momentRange: [startMoment, endMoment],
            sKey_dateRange: [startMoment.format(dateFormat), endMoment.format(dateFormat)],
            sKey_amountRate: '1',
          };
        }
      });
      yield put({
        type: 'updateState',
        payload: {
          ...initData,
          sKeyStorage: initData,
        },
      });
      if (callback) callback();
    },
    *loadData({ payload: { callback } }, { select, put }) {
      let apiParams = yield select(_buildMainParams);
      const isFirstPage = apiParams.page === 1;
      let fetchList = [];
      if (isFirstPage) {
        const _apiParams = clone(apiParams);
        _apiParams.getTotal = true;
        // 统计数据
        fetchList.unshift(getStaffDistributionStatisticsInfo(apiParams, true));
        fetchList.unshift(getStaffDistributionStatisticsList(_apiParams, true));
      }
      // 列表数据
      fetchList.unshift(getStaffDistributionStatisticsList(apiParams, true));
      const rspAll = yield Promise.all(fetchList).catch(errorsHandling());
      if (rspAll) {
        const [rspList, rspTotal, rspTotalInfo] = rspAll;
        let payload = { isFirstPage };
        if (rspList) {
          payload.list = rspList.rows || [];
        }
        if (rspTotal) {
          payload.total = rspTotal.records;
        }
        if (rspTotalInfo) {
          payload.amountRate = rspTotalInfo.data;
        }
        yield put({
          type: 'updatePageData',
          payload,
        });
      }
      if (callback) callback();
    },
    *resetSKey(action, { select, put }) {
      const sKeyStorage = yield select((state) => state[storeName].sKeyStorage);
      yield put({
        type: 'updateState',
        payload: {
          ...clone(defaultState.sKey),
          ...sKeyStorage,
        },
      });
    },
    *resetPage(action, { put }) {
      yield put({
        type: 'updateState',
        payload: {
          ...clone(defaultState.sKey),
          ...clone(defaultState.statisticalNum),
          ...clone(defaultState.list),
        },
      });
    },
    *updateStateWithEffects({ payload: { data, callback } }, { put }) {
      yield put({
        type: 'updateState',
        payload: data,
      });
      if (callback) callback();
    },
    *exportExcel({ payload: { callback } }, { select, put }) {
      let apiParams = yield select(_buildMainParams);
      const rsp = yield exportStaffDistributionStatisticsListExcel(apiParams, true).catch(errorsHandling());
      if (rsp && rsp.resultCode === 0) {
        yield put({ type: 'main/exportNotice' });
      }
      if (callback) callback();
    },
  },
};
