import React from 'react';
import { connect } from 'dva';
import { storeName, contractTypeDict, dateFormat, productTypeAllow } from './store/config';
import { SearchButton, MultiSelect, Select, Input, Validator } from 'COMPONENT';
import { DatePicker } from 'antd';
import { getRelationsDictPlus, getRelationPayloadPlus } from 'MODEL/dict/utils/index';
import { productType, productSeries, classType, classLevel, branch, campus } from 'MODEL/dict/utils/config';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { loginUser } from 'UTIL/common';
import { SMALL_CLASS_VAL } from './store/config';

const RangePicker = DatePicker.RangePicker;

class Search extends React.Component {
  constructor(props) {
    super(props);

    this.updateProps = this.updateProps.bind(this);
    this.updatePropsWithEffects = this.updatePropsWithEffects.bind(this);
    this.updateValue = this.updateValue.bind(this);
    this.updateDateRange = this.updateDateRange.bind(this);
    this.updateBranch = this.updateBranch.bind(this);
    this.updateProductType = this.updateProductType.bind(this);
    this.updateClassType = this.updateClassType.bind(this);
    this.updateProductName = this.updateProductName.bind(this);
    this.updateClassName = this.updateClassName.bind(this);
    this.updateNeedValidator = this.updateNeedValidator.bind(this);
    this.errorChange = this.errorChange.bind(this);
    this.onSearch = this.onSearch.bind(this);
    this.onReset = this.onReset.bind(this);

    this.isTwoTeacher = loginUser.isTwoTeacherSym();
    this.state = {
      isNeedValidator: false,
      error: {},
    };
    // 产品类型关联关系type、key对应关系配置
    this.productTypeTreeMap = {
      [productType]: 'sKey_productType',
      [productSeries]: 'sKey_productSeries',
      [classType]: 'sKey_classType',
      [classLevel]: 'sKey_classLevel',
    };
    // 分公司关联关系type、key对应关系配置
    this.branchTreeMap = {
      [branch]: 'sKey_branch',
      [campus]: 'sKey_campus',
    };
  }

  updateProps(params) {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: params,
    });
  }

  updatePropsWithEffects(type, val) {
    return new Promise((resolve) => {
      this.props.dispatch({
        type: `${storeName}/updateStateWithEffects`,
        payload: {
          data: { [type]: val },
          callback() {
            resolve();
          },
        },
      });
    });
  }

  updateValue(type, val) {
    this.updateProps({ [type]: val });
  }

  updateDateRange(moments, dates) {
    this.updateProps({
      sKey_momentRange: moments,
      sKey_dateRange: dates,
    });
  }

  updateBranch(val) {
    let payload = getRelationPayloadPlus('sKey_branch', val, {
      type: branch,
      dictTree: this.props.branchCampusTree,
      state: this.props,
      map: this.branchTreeMap,
    });
    this.updateProps(payload);
  }

  /**
   * 选择产品类型
   *
   * @param val
   */
  updateProductType(val) {
    let payload = getRelationPayloadPlus('sKey_productType', val, {
      type: productType,
      dictTree: this.props.productTypeTree,
      state: this.props,
      map: this.productTypeTreeMap,
    });

    if (this.isTwoTeacher && val !== SMALL_CLASS_VAL) {
      payload.sKey_productGroup = null;
    }

    this.updateProps(payload);
  }

  updateClassType(val) {
    let payload = getRelationPayloadPlus('sKey_classType', val, {
      type: classType,
      dictTree: this.props.productTypeTree,
      state: this.props,
      map: this.productTypeTreeMap,
    });
    this.updateProps(payload);
  }

  async updateProductName(ele) {
    let val = ele.target.value;
    await this.updatePropsWithEffects('sKey_productName', val);
    this.updateNeedValidator({ sKey_productName: val });
  }

  async updateClassName(ele) {
    let val = ele.target.value;
    await this.updatePropsWithEffects('sKey_className', val);
    this.updateNeedValidator({ sKey_className: val });
  }

  updateNeedValidator(params) {
    const productName = params.sKey_productName || this.props.sKey_productName;
    const className = params.sKey_className || this.props.sKey_className;
    this.setState({ isNeedValidator: Boolean(productName || className) });
  }

  errorChange(err, keyName) {
    this.setState((prevState) => ({
      error: {
        ...prevState.error,
        [keyName]: err,
      },
    }));
  }

  onSearch() {
    const isSearchByPrdNameOrClassName = this.props.sKey_productName || this.props.sKey_className;
    const isPrdTypeOrQuarterEmpty = !(this.props.sKey_productType && this.props.sKey_quarter);
    // 通过产品名称、班级名称进行筛选
    if (isSearchByPrdNameOrClassName && isPrdTypeOrQuarterEmpty) {
      let errorMsgStart = '';
      if (this.props.sKey_productName) {
        errorMsgStart = '产品名称';
      }
      if (this.props.sKey_className) {
        errorMsgStart = errorMsgStart === '' ? '班级名称' : `${errorMsgStart}、班级名称`;
      }
      let errorMsgKeyword = '';
      if (!this.props.sKey_productType) {
        errorMsgKeyword = '产品类型';
      }
      if (!this.props.sKey_quarter) {
        errorMsgKeyword = errorMsgKeyword === '' ? '季度' : `${errorMsgKeyword}、季度`;
      }
      return showMessage(`通过${errorMsgStart}搜索时，${errorMsgKeyword}为必选项`, 'error');
    }
    // 起始日期未选时
    if (!(this.props.sKey_momentRange[0] && this.props.sKey_momentRange[1])) {
      return showMessage('起始日期为必填项！', 'error');
    }
    this.props.onSearch();
  }

  onReset() {
    this.setState(
      {
        isNeedValidator: false,
        error: {},
      },
      () => {
        this.props.onReset();
      },
    );
  }

  render() {
    return (
      <SearchButton
        girdConfig={[2]}
        searchBtnLoading={this.props.searchBtnLoading}
        onSearch={this.onSearch}
        onReset={this.onReset}
      >
        <Validator
          value={this.props.sKey_momentRange[0] && this.props.sKey_momentRange[1] ? 'true' : null}
          rules={['required']}
          silent={!Boolean(this.state.error.sKey_momentRange)}
          showStatus={true}
          showTips={false}
          onErrorChange={(err) => this.errorChange(err, 'sKey_momentRange')}
        >
          <RangePicker
            value={this.props.sKey_momentRange}
            onChange={this.updateDateRange}
            placeholder={['开始时间', '结束时间']}
            format={dateFormat}
            showTime={false}
            size="large"
          />
        </Validator>
        <MultiSelect
          placeholder="分校"
          dict={this.props.branchDict}
          value={this.props.sKey_branch}
          onChange={this.updateBranch}
          showSearch={true}
        />
        <MultiSelect
          placeholder="学习中心"
          dict={this.props.branchCampusDict}
          value={this.props.sKey_campus}
          onChange={(val) => this.updateValue('sKey_campus', val)}
          showSearch={true}
        />
        <Select
          placeholder="产品类型"
          dict={this.props.productTypeTree}
          value={this.props.sKey_productType}
          onChange={this.updateProductType}
          validator={
            this.state.isNeedValidator
              ? {
                  rules: ['required'],
                  silent: !Boolean(this.state.error.sKey_productType),
                  showStatus: true,
                  showTips: false,
                  onErrorChange: (err) => this.errorChange(err, 'sKey_productType'),
                }
              : null
          }
        />
        {this.isTwoTeacher && (
          <Select
            placeholder="产品组"
            dict={this.props.productGroupDict}
            value={this.props.sKey_productGroup}
            onChange={(val) => this.updateValue('sKey_productGroup', val)}
            disabled={this.props.sKey_productType !== SMALL_CLASS_VAL}
            allowClear={true}
          />
        )}
        <Select
          placeholder="产品系列"
          dict={this.props.productSeriesDict}
          value={this.props.sKey_productSeries}
          onChange={(val) => this.updateValue('sKey_productSeries', val)}
        />
        <Select
          placeholder="产品性质"
          dict={this.props.productCharacterDict}
          value={this.props.sKey_productCharacter}
          onChange={(val) => this.updateValue('sKey_productCharacter', val)}
        />
        <Select
          placeholder="班级类型"
          dict={this.props.classTypeDict}
          value={this.props.sKey_classType}
          onChange={this.updateClassType}
          disabled={!this.props.sKey_productType}
        />
        <Select
          placeholder="班级层次"
          dict={this.props.classLevelDict}
          value={this.props.sKey_classLevel}
          onChange={(val) => this.updateValue('sKey_classLevel', val)}
          disabled={!this.props.sKey_productType}
        />
        <Select
          placeholder="签单类型"
          dict={contractTypeDict}
          value={this.props.sKey_contractType}
          onChange={(val) => this.updateValue('sKey_contractType', val)}
        />
        <Select
          placeholder="季度"
          dict={this.props.quarterDict}
          value={this.props.sKey_quarter}
          onChange={(val) => this.updateValue('sKey_quarter', val)}
          validator={
            this.state.isNeedValidator
              ? {
                  rules: ['required'],
                  silent: !Boolean(this.state.error.sKey_quarter),
                  showStatus: true,
                  showTips: false,
                  onErrorChange: (err) => this.errorChange(err, 'sKey_quarter'),
                }
              : null
          }
        />
        <Select
          placeholder="科目"
          dict={this.props.subjectDict}
          value={this.props.sKey_subject}
          onChange={(val) => this.updateValue('sKey_subject', val)}
          showSearch={true}
        />
        <Input
          placeholder="责任人"
          value={this.props.sKey_blamePersonName}
          onChange={(ele) => this.updateValue('sKey_blamePersonName', ele.target.value)}
        />
        <Input
          placeholder="合同编号"
          value={this.props.sKey_contractId}
          onChange={(ele) => this.updateValue('sKey_contractId', ele.target.value)}
        />
        <Input placeholder="产品名称" value={this.props.sKey_productName} onChange={this.updateProductName} />
        <Input placeholder="班级名称" value={this.props.sKey_className} onChange={this.updateClassName} />
      </SearchButton>
    );
  }
}

let productTypeTreeStorage = null;

const mapStateToProps = (state) => {
  const { productGroup } = state.dict;
  const isTwoTeacher = loginUser.isTwoTeacherSym();

  // 过滤与产品类型相关字典
  if (!productTypeTreeStorage && state.dict.productTypeTree) {
    productTypeTreeStorage = state.dict.productTypeTree.filter((item) => productTypeAllow.includes(item.val));
    productTypeTreeStorage.forEach((item) => {
      if (item.val === SMALL_CLASS_VAL) {
        item.text = isTwoTeacher ? '小班' : '面授';
      }
    });
  }
  let sKey_productType = state[storeName].sKey_productType;
  let sKey_classType = state[storeName].sKey_classType;

  return {
    // 字典
    branchDict: state.dict.branchCampusTree,
    branchCampusDict: getRelationsDictPlus(campus)(state.dict.branchCampusTree, state[storeName].sKey_branch),
    branchCampusTree: state.dict.branchCampusTree,
    productCharacterDict: state.dict.productCharacter,
    quarterDict: state.dict.quarter,
    subjectDict: state.dict.subject,
    productTypeTree: productTypeTreeStorage,
    productGroupDict: productGroup ? productGroup['SMALL_CLASS'] : null,
    productSeriesDict: getRelationsDictPlus(productSeries)(productTypeTreeStorage, sKey_productType),
    classTypeDict: getRelationsDictPlus(classType)(productTypeTreeStorage, sKey_productType),
    classLevelDict: getRelationsDictPlus(classLevel)(productTypeTreeStorage, sKey_productType, sKey_classType),
    // 筛选项
    sKey_momentRange: state[storeName].sKey_momentRange,
    sKey_dateRange: state[storeName].sKey_dateRange,
    sKey_branch: state[storeName].sKey_branch,
    sKey_campus: state[storeName].sKey_campus,
    sKey_productType,
    sKey_productGroup: state[storeName].sKey_productGroup,
    sKey_productSeries: state[storeName].sKey_productSeries,
    sKey_productCharacter: state[storeName].sKey_productCharacter,
    sKey_classType,
    sKey_classLevel: state[storeName].sKey_classLevel,
    sKey_contractType: state[storeName].sKey_contractType,
    sKey_quarter: state[storeName].sKey_quarter,
    sKey_subject: state[storeName].sKey_subject,
    sKey_blamePersonName: state[storeName].sKey_blamePersonName,
    sKey_contractId: state[storeName].sKey_contractId,
    sKey_productName: state[storeName].sKey_productName,
    sKey_className: state[storeName].sKey_className,
  };
};

export default connect(mapStateToProps)(Search);
