import React from 'react';
import { connect } from 'dva';
import { storeName } from './store/config';
import { Page, Button, List } from 'COMPONENT';
import Search from './Search';

const { COL_WIDTH_L } = List;
const setWidth = List.createSetWidth();

class RefundStatistics extends React.Component {
  constructor(props) {
    super(props);

    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.initDefault = this.initDefault.bind(this);
    this.loadData = this.loadData.bind(this);
    this.updatePage = this.updatePage.bind(this);
    this.changePage = this.changePage.bind(this);
    this.search = this.search.bind(this);
    this.reset = this.reset.bind(this);
    this.exportExcel = this.exportExcel.bind(this);
    this.beforeDestroy = this.beforeDestroy.bind(this);

    this.state = {
      name: '退费科次统计',
      loading: false,
    };
    this.columns = [
      { title: '责任学习中心', dataIndex: 'campusName', key: 'campusName', width: setWidth(5) },
      { title: '责任人', dataIndex: 'blamePersonName', key: 'blamePersonName', width: setWidth(4) },
      { title: '岗位', dataIndex: 'jobName', key: 'jobName', width: setWidth(4) },
      { title: '学生姓名', dataIndex: 'studentName', key: 'studentName', width: setWidth(4) },
      { title: '合同编号', dataIndex: 'contractId', key: 'contractId', width: COL_WIDTH_L },
      { title: '产品名称', dataIndex: 'productName', key: 'productName', width: setWidth(8) },
      { title: '班级名称', dataIndex: 'className', key: 'className', width: setWidth(8) },
      { title: '类型', dataIndex: 'productTypeName', key: 'productTypeName', width: setWidth(5) },
      { title: '产品系列', dataIndex: 'productSeriesName', key: 'productSeriesName', width: setWidth(4) },
      { title: '产品性质', dataIndex: 'productCharacterName', key: 'productCharacterName', width: setWidth(4) },
      { title: '产品季度', dataIndex: 'quarterName', key: 'quarterName', width: setWidth(4) },
      { title: '科目', dataIndex: 'subjectName', key: 'subjectName', width: setWidth(3) },
      { title: '退费金额', dataIndex: 'refundAmount', key: 'refundAmount', width: setWidth(4) },
      { title: '退费类型', dataIndex: 'refundTypeName', key: 'refundTypeName', width: setWidth(4) },
      { title: '退费科次', dataIndex: 'refundRate', key: 'refundRate', width: setWidth(4) },
    ];
    this.toolBarStyle = { marginRight: 20 };
  }
  componentDidMount() {
    this.init();
  }
  componentWillUnmount() {
    this.beforeDestroy();
  }

  async init() {
    this.setState({ loading: true });

    await new Promise(this.initDict);
    await new Promise(this.initDefault);
    await new Promise(this.loadData);

    this.setState({ loading: false });
  }

  initDict(resolve) {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: [
          'branchCampusTree',
          'productTypeTree',
          'productCharacter',
          { name: 'productGroup', params: 'SMALL_CLASS' },
          'quarter',
          'subject',
        ],
        callback() {
          resolve();
        },
      },
    });
  }

  initDefault(resolve) {
    this.props.dispatch({
      type: `${storeName}/initSKey`,
      payload: {
        callback() {
          resolve();
        },
      },
    });
  }

  loadData(resolve) {
    this.props.dispatch({
      type: `${storeName}/loadData`,
      payload: {
        callback() {
          resolve();
        },
      },
    });
  }

  updatePage(pageCurrent, pageSize) {
    let payload = {};

    if (pageSize && pageSize !== this.props.pageSize) {
      payload.pageCurrent = 1;
      payload.pageSize = pageSize;
    } else {
      payload.pageCurrent = pageCurrent;
    }

    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload,
    });
  }

  async changePage({ current, pageSize }) {
    this.updatePage(current, pageSize);
    this.setState({ loading: true });
    await new Promise(this.loadData);
    this.setState({ loading: false });
  }

  search() {
    this.changePage({
      current: 1,
      pageSize: this.props.pageSize,
    });
  }

  reset() {
    this.props.dispatch({
      type: `${storeName}/resetSKey`,
    });
  }

  exportExcel() {
    this.setState({ loading: true });
    const _self = this;
    this.props.dispatch({
      type: `${storeName}/exportExcel`,
      payload: {
        callback() {
          _self.setState({ loading: false });
        },
      },
    });
  }

  beforeDestroy() {
    this.props.dispatch({
      type: `${storeName}/resetPage`,
    });
  }

  render() {
    return (
      <Page
        loading={this.state.loading}
        route={this.props.route}
        fitWindowHeight
        headerToolBar={
          <div>
            {this.props.exportBtnAuth && (
              <Button onClick={this.exportExcel} type="default" style={this.toolBarStyle}>
                导出excel
              </Button>
            )}
            <Search searchBtnLoading={this.state.loading} onSearch={this.search} onReset={this.reset} />
          </div>
        }
      >
        <List
          columns={this.columns}
          dataSource={this.props.list}
          pagination={{
            total: this.props.total,
            current: this.props.pageCurrent,
            pageSize: this.props.pageSize,
          }}
          onChange={this.changePage}
        />
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    exportBtnAuth: state.account.btnAuths.includes('REFUND_STATISTICS_EXCEL'),
    list: state[storeName].list,
    total: state[storeName].total,
    pageSize: state[storeName].pageSize,
    pageCurrent: state[storeName].pageCurrent,
  };
};

export default connect(mapStateToProps)(RefundStatistics);
