import { storeName } from './config';
import { clone } from 'lodash';
import moment from 'moment';
import { contractTypeConf, dateFormat } from './config';
import { errorsHandling, setProductName } from './utils';
import { getRefundResponsibleContractProductList } from 'SERVICE/contract';
import { exportRefundResponsibleContractProductListExcel } from 'SERVICE/exportFile';

// 构建列表/合计/导出的参数
function _buildMainParams(state) {
  const _state = state[storeName];
  return {
    startDate: _state.sKey_dateRange[0],
    endDate: _state.sKey_dateRange[1],
    branchIdArr: _state.sKey_branch,
    campusIdArr: _state.sKey_campus,
    productType: _state.sKey_productType,
    productGroupId: _state.sKey_productGroup,
    courseSeriesId: _state.sKey_productSeries,
    productCharacterId: _state.sKey_productCharacter,
    classTypeId: _state.sKey_classType,
    classLevelId: _state.sKey_classLevel,
    contractType: _state.sKey_contractType,
    productQuarterId: _state.sKey_quarter,
    subjectId: _state.sKey_subject,
    responsibleStaffName: _state.sKey_blamePersonName,
    contractId: _state.sKey_contractId,
    productName: _state.sKey_productName,
    className: _state.sKey_className,
    page: _state.pageCurrent,
    rows: _state.pageSize,
    getTotal: false,
  };
}

const defaultState = {
  // 筛选项
  sKey: {
    sKey_momentRange: [null, null],
    sKey_dateRange: [null, null],
    // 可多选
    sKey_branch: null,
    // 可多选
    sKey_campus: null,
    sKey_productType: null,
    sKey_productGroup: null,
    sKey_productSeries: null,
    sKey_productCharacter: null,
    sKey_classType: null,
    sKey_classLevel: null,
    sKey_contractType: null,
    sKey_quarter: null,
    sKey_subject: null,
    sKey_blamePersonName: null,
    sKey_contractId: null,
    sKey_productName: null,
    sKey_className: null,
  },
  // 列表数据
  list: {
    list: [],
    total: 0,
    pageSize: 10,
    pageCurrent: 1,
  },
};

export default {
  namespace: storeName,
  state: {
    ...clone(defaultState.sKey),
    ...clone(defaultState.list),
    // 筛选条件初始化缓存
    sKeyStorage: null,
  },
  reducers: {
    updateState(state, { payload }) {
      return {
        ...state,
        ...payload,
      };
    },
    // 更新业绩科次统计列表数据
    updatePageData(state, { payload: { list, total } }) {
      let payload = {};
      if (list) {
        payload.list = list.map((item) => ({
          ...item,
          className: item.className || '无',
          blamePersonName: item.responsibleStaffName,
          jobName: item.responsibleJobCodeName,
          contractDate: item.signDate,
          contractAndPrdType: `${contractTypeConf[item.contractType]}/${setProductName(item.productType)}`,
          productSeriesName: item.courseSeriesName,
          quarterName: item.productQuarterName,
          refundAmount: item.refundAmount,
          refundTypeName: item.refundFormTypeName,
          refundRate: item.responsibleRate,
        }));
      }
      if (typeof total === 'number') {
        payload.total = total;
      }
      return {
        ...state,
        ...payload,
      };
    },
  },
  effects: {
    *initSKey({ payload: { callback } }, { select, put }) {
      let initData = yield select((state) => {
        if (state[storeName].sKeyStorage) {
          return state[storeName].sKeyStorage;
        } else {
          const { brenchId, campusId } = state.account.user;
          const isGroup = brenchId === campusId;
          let sKey_branch = isGroup ? [state.dict.branchCampusTree[0].val] : [brenchId];
          let sKey_campus = isGroup ? null : [campusId];
          let today = new Date();
          let startMoment = moment(today).startOf('month');
          let endMoment = moment(today);
          return {
            sKey_branch,
            sKey_campus,
            sKey_momentRange: [startMoment, endMoment],
            sKey_dateRange: [startMoment.format(dateFormat), endMoment.format(dateFormat)],
          };
        }
      });
      yield put({
        type: 'updateState',
        payload: {
          ...initData,
          sKeyStorage: initData,
        },
      });
      if (callback) callback();
    },
    *loadData({ payload: { callback } }, { select, put }) {
      let apiParams = yield select(_buildMainParams);
      const isFirstPage = apiParams.page === 1;
      let fetchList = [];
      if (isFirstPage) {
        const _apiParams = clone(apiParams);
        _apiParams.getTotal = true;
        fetchList.unshift(getRefundResponsibleContractProductList(_apiParams, true));
      }
      // 列表数据
      fetchList.unshift(getRefundResponsibleContractProductList(apiParams, true));
      const rspAll = yield Promise.all(fetchList).catch(errorsHandling());
      if (rspAll) {
        const [rspList, rspTotal] = rspAll;
        let payload = { list: rspList.rows || [] };
        if (rspTotal) {
          payload.total = rspTotal.records;
        }
        yield put({
          type: 'updatePageData',
          payload,
        });
      }
      if (callback) callback();
    },
    *resetSKey(action, { select, put }) {
      const sKeyStorage = yield select((state) => state[storeName].sKeyStorage);
      yield put({
        type: 'updateState',
        payload: {
          ...clone(defaultState.sKey),
          ...sKeyStorage,
        },
      });
    },
    *resetPage(action, { put }) {
      yield put({
        type: 'updateState',
        payload: {
          ...clone(defaultState.sKey),
          ...clone(defaultState.statisticalNum),
          ...clone(defaultState.list),
        },
      });
    },
    *updateStateWithEffects({ payload: { data, callback } }, { put }) {
      yield put({
        type: 'updateState',
        payload: data,
      });
      if (callback) callback();
    },
    *exportExcel({ payload: { callback } }, { select, put }) {
      let apiParams = yield select(_buildMainParams);
      const rsp = yield exportRefundResponsibleContractProductListExcel(apiParams, true).catch(errorsHandling());
      if (rsp && rsp.resultCode === 0) {
        yield put({ type: 'main/exportNotice' });
      }
      if (callback) callback();
    },
  },
};
