import React from 'react';
import { connect } from 'dva';
import { Page, Button, List } from 'COMPONENT';
import { SMALL_CLASS_VAL, storeName } from './store/config';
import Info from '../PerformanceStatistics/components/Info';
import Search from './Search';
import moment from 'moment';
const InfoWrap = Info.InfoWrap;

class TransferStatistics extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      isError: false,
      searchOptions: {
        startDate: moment().startOf('month').format('YYYY-MM-DD'),
        endDate: moment().format('YYYY-MM-DD'),
        productType: SMALL_CLASS_VAL,
      },
      current: 1,
      pageSize: 10,
    };

    this.columns = [
      { title: '类型', dataIndex: 'operationTypeName', key: 'operationTypeName', width: 60 },
      { title: '分公司', dataIndex: 'branchName', key: 'branchName' },
      { title: '课程学习中心', dataIndex: 'campusName', key: 'campusName' },
      { title: '学生编号', dataIndex: 'studentId', key: 'studentId', width: 150 },
      { title: '学生姓名', dataIndex: 'studentName', key: 'studentName' },
      {
        title: '调/转/退前',
        children: [
          { title: '产品名称', dataIndex: 'beforeProductName', key: 'beforeProductName', width: 147 },
          { title: '年级', dataIndex: 'beforeGradeName', key: 'beforeGradeName', width: 73 },
          { title: '班级类型', dataIndex: 'beforeClassTypeName', key: 'beforeClassTypeName', width: 100 },
          { title: '班级层次', dataIndex: 'beforeClassLevelName', key: 'beforeClassLevelName', width: 100 },
          { title: '班级名称', dataIndex: 'beforeClassName', key: 'beforeClassName', width: 166 },
        ],
        key: 'before',
      },
      {
        title: '调/转/退后',
        children: [
          { title: '产品名称', dataIndex: 'afterProductName', key: 'afterProductName', width: 147 },
          { title: '年级', dataIndex: 'afterGradeName', key: 'afterGradeName', width: 73 },
          { title: '班级类型', dataIndex: 'afterClassTypeName', key: 'afterClassTypeName', width: 100 },
          { title: '班级层次', dataIndex: 'afterClassLevelName', key: 'afterClassLevelName', width: 100 },
          { title: '班级名称', dataIndex: 'afterClassName', key: 'afterClassName', width: 166 },
        ],
        key: 'after',
      },
      { title: '操作者', dataIndex: 'operator', key: 'operator' },
      { title: '操作时间', dataIndex: 'operateTime', key: 'operateTime' },
    ];

    this.init = this.init.bind(this);
    this.loadDict = this.loadDict.bind(this);
    this.onSearchOptionsChange = this.onSearchOptionsChange.bind(this);
    this.onSearch = this.onSearch.bind(this);
    this.exportExcel = this.exportExcel.bind(this);
  }

  componentDidMount() {
    this.init();
  }

  async init() {
    this.setState({ loading: true });

    await new Promise(this.loadDict);
    await this.onSearch();

    this.setState({ loading: false });
  }

  loadDict(resolve, reject) {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['branch', 'branchCampus', { name: 'productGroup', params: SMALL_CLASS_VAL }],
        callback() {
          resolve();
        },
      },
    });
  }

  onSearchOptionsChange(state) {
    if (state === undefined) {
      this.setState({
        searchOptions: {
          startDate: moment().startOf('month').format('YYYY-MM-DD'),
          endDate: moment().format('YYYY-MM-DD'),
          current: 1,
          pageSize: 10,
          productType: SMALL_CLASS_VAL,
        },
      });
    } else {
      this.setState({ searchOptions: Object.assign({}, this.state.searchOptions, state), current: 1 });
    }
  }

  async onSearch() {
    this.setState({ loading: true });

    const { searchOptions, current: page, pageSize: rows } = this.state;
    if (searchOptions.startDate === null || searchOptions.endDate === null) {
      this.setState({ isError: true, loading: false });
      return;
    }

    await new Promise((resolve) => {
      this.props.dispatch({
        type: `${storeName}/loadListData`,
        payload: {
          params: {
            ...searchOptions,
            page,
            rows,
          },
          callback() {
            resolve();
          },
        },
      });
    });

    this.setState({ loading: false });
  }

  exportExcel() {
    const { searchOptions } = this.state;

    this.props.dispatch({
      type: `${storeName}/exportExcel`,
      payload: {
        params: {
          ...searchOptions,
        },
      },
    });
  }

  render() {
    const { loading, isError, current, pageSize } = this.state;
    const { dataSource, total, changeClassNum, dropClassNum, changeCourseNum } = this.props;
    const canExport = this.props.btnAuths.includes('TRANSFERSTATISTICS_EXPORT_EXCEL');

    return (
      <Page
        loading={loading}
        route={this.props.route}
        headerToolBar={
          <div>
            {canExport && (
              <Button onClick={this.exportExcel} type="default" style={{ marginRight: '20px' }}>
                导出
              </Button>
            )}
            <Search onSearchOptionsChang={this.onSearchOptionsChange} onSearch={this.onSearch} isError={isError} />
          </div>
        }
        subHeader={
          <InfoWrap>
            <Info label="转班数量" text={changeClassNum} />
            <Info label="退班数量" text={dropClassNum} />
            <Info label="调课数量" text={changeCourseNum} />
          </InfoWrap>
        }
      >
        <List
          columns={this.columns}
          dataSource={dataSource}
          onChange={({ current, pageSize }) => {
            this.setState({ current, pageSize }, () => {
              this.onSearch();
            });
          }}
          pagination={{
            pageSize,
            current,
            total,
          }}
          scroll={{ x: true, y: true }}
        />
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const {
    account: { btnAuths },
  } = state;

  return {
    ...state[storeName],
    btnAuths,
  };
};

export default connect(mapStateToProps)(TransferStatistics);
