import { storeName } from './config';
import { clone } from 'lodash';
import {
  findClassCourseChangeRecords,
  findClassCourseChangeRecordCountInfo,
  exportClassCourseChangeRecords,
} from 'SERVICE/twoTeacher';
import { errorsHandling } from './utils';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

const defaultState = {
  statisticalNum: {
    changeClassNum: '',
    dropClassNum: '',
    changeCourseNum: '',
  },
  list: {
    // 列表数据
    dataSource: [],
    total: 0,
  },
};

export default {
  namespace: storeName,
  state: {
    ...clone(defaultState.statisticalNum),
    ...clone(defaultState.list),
  },
  reducers: {
    updateData(state, { payload }) {
      return {
        ...state,
        ...payload,
      };
    },
  },
  effects: {
    *loadListData({ payload: { params, callback } }, { select, put, call }) {
      try {
        const [{ data: statisticalNum }, { data: dataSource, rowCount: total }] = yield Promise.all([
          findClassCourseChangeRecordCountInfo(params),
          findClassCourseChangeRecords(params),
        ]);

        yield put({
          type: 'updateData',
          payload: {
            ...statisticalNum,
            dataSource,
            total,
          },
        });
      } catch (err) {
        showMessage(err.text, 'error');
      }

      if (callback) callback();
    },
    *exportExcel({ payload: { params }, callback }, { select, put, call }) {
      const rsp = yield exportClassCourseChangeRecords({ ...params }).catch(errorsHandling());
      if (rsp && rsp.resultCode === 0) {
        yield put({ type: 'main/exportNotice' });
      }
      if (callback) callback();
    },
  },
};
