import React from 'react';
import { connect } from 'dva';
import Button from 'COMPONENT/Button';
import Page from 'COMPONENT/Page';
import List from 'COMPONENT/List';
import SearchBar from 'COMPONENT/SearchBar';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import Input from 'COMPONENT/Input';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import AddNewModal from './modal/AddNewModal';
import PreviewModal from './modal/PreviewModal';
import moment from 'moment';
import { buildBaseUrl } from './utils';
import loader from './loader';
import { renderMultiLine } from '../common';
import ShareQRCode from 'COMPONENT/ShareQRCode';
import styles from './styles.less';
import showConfirm from 'SRC/components/utils/globalTips/showConfirm';
const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;

let defaultBranchArray = null;
let defaultBranch = null;
const statusDict = [
  { text: '上架', val: 1 },
  { text: '下架', val: 0 },
];
let branchObj = {};

class AdManagement extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: true, // boolean 页面加载状态
      searchExpand: false,
      shareAdObj: {},
      shareModalVisible: false,
      addNewAdModal: false,
      previewModal: false,
      //查询条件
      branchId: null,
      name: null,
      status: null,
      //列表数据
      listLoading: false,
      rows: [],
      pageNo: 1,
      pageSize: 10,
      total: null,
      //数据字典数据
      branchDict: [],
    };
  }

  componentDidMount() {
    this.initContent();
  }

  updateState = (changedState) => {
    const newState = {
      ...this.state,
      ...changedState,
    };
    return new Promise((resolve) => {
      this.setState(newState, () => {
        resolve();
      });
    });
  };

  loadListData = async (params) => {
    const { branchId, name, pageNo, pageSize, status } = this.state;
    const apiParams = {
      name,
      pageNum: pageNo,
      pageSize,
      status,
      ...params,
    };
    if (Array.isArray(branchId) && branchId.length > 0) {
      apiParams.orgIds = branchId.join(',');
    }
    try {
      return await loader.loadList(apiParams);
    } catch (e) {
      showMessage(e.text);
      return [];
    }
  };

  initContent = async () => {
    this.setState({ loading: true });
    const asyncCalls = [loader.getBranchDict()];
    Promise.all(asyncCalls).then(([branchDict]) => {
      branchObj = this.dictToObj(branchDict);
      //默认第一个分公司，当前年份
      defaultBranchArray = branchDict[0] ? [branchDict[0].val] : null;
      defaultBranch = branchDict[0] ? branchDict[0].val : null;
      //获取列表数据
      this.loadListData({ orgIds: defaultBranch }).then((list) => {
        console.log(list);
        this.setState({
          branchId: defaultBranchArray,
          loading: false,
          rows: list.rows,
          total: list.records,
          pageNo: list.page,
          branchDict,
        });
      });
    });
  };

  dictToObj = (dict) => {
    return (dict || []).reduce((obj, ele) => {
      obj[ele.val] = ele.text;
      return obj;
    }, {});
  };

  search = async () => {
    await this.updateState({ listLoading: true, pageNo: 1 });
    const list = await this.loadListData();
    this.updateState({
      listLoading: false,
      rows: list.rows,
      total: list.records,
      pageNo: list.page,
    });
  };

  reset = async () => {
    this.updateState({
      branchId: defaultBranchArray,
      name: null,
      status: null,
    });
  };

  pageChange = async ({ current, pageSize }) => {
    await this.updateState({ listLoading: true, pageSize, pageNo: current });
    const list = await this.loadListData();
    this.updateState({
      listLoading: false,
      rows: list.rows,
      total: list.records,
      pageNo: list.page,
    });
  };

  share = (record) => {
    this.setState({
      shareAdObj: record,
      shareModalVisible: true,
    });
  };

  addNewAdConfirm = () => {
    this.setState({ addNewAdModal: false });
    this.search();
  };

  renderStatus = (detail) => {
    detail = String(detail);
    return (
      <p key="asd" style={{ display: 'flex' }}>
        <b style={{ fontSize: '25px', color: detail === '1' ? '#12ce66' : '#AAA', marginRight: '3px' }}>·</b>
        {detail === '1' ? '上架中' : '已下架'}
      </p>
    );
  };

  renderTime = (detail) => {
    return (
      <div className={styles.timeWrapper}>
        <p>{moment(detail).format('YYYY-MM-DD HH:mm:ss')}</p>
      </div>
    );
  };

  editAdStatus = async (record) => {
    const { id, status } = record;

    const statusMap = {
      0: {
        editStatus: 1,
        statusText: '上架',
      },
      1: {
        editStatus: 0,
        statusText: '下架',
      },
    };

    const { editStatus, statusText } = statusMap[status];

    showConfirm(
      `确定${statusText}${record.name}？`,
      async () => {
        this.setState({ listLoading: true });
        try {
          const list = await loader.editAdStatus({ id, status: editStatus });
          const idx = this.state.rows.findIndex((ele) => ele.id === id);
          const rows = this.state.rows.map((row, i) => {
            if (i === idx) {
              return { ...row, status: editStatus };
            }
            return row;
          });
          this.setState({
            listLoading: false,
            rows,
          });
        } catch (e) {
          showMessage(e.message);
        }
      },
      (err) => {},
    );
  };
  deleteAd = async (record) => {
    const { id, name } = record;
    const total = this.state.total;
    showConfirm(
      `确定删除${name}？`,
      async () => {
        this.setState({ listLoading: true });
        try {
          // 状态为2就是删除
          const list = await loader.editAdStatus({ id, status: -1 });
          const idx = this.state.rows.findIndex((ele) => ele.id === id);
          const rows = this.state.rows.filter((row, i) => {
            return i !== idx;
          });
          this.setState({
            listLoading: false,
            rows,
            total: total - 1,
          });
        } catch (e) {
          showMessage(e.message);
        }
      },
      (err) => {},
    );
  };
  /**
   * 表头
   */
  columns = [
    { title: '序号', dataIndex: 'index', width: 70, fixed: 'left', render: (text, record, index) => index + 1 },
    { title: '宣传页名称', dataIndex: 'name', width: COL_WIDTH_L },
    { title: '分公司', dataIndex: 'orgName', width: COL_WIDTH_M, render: renderMultiLine },
    { title: '访问数量', dataIndex: 'visitTimes', width: 90 },
    { title: '创建者', dataIndex: 'createUserName', width: COL_WIDTH_M },
    { title: '上传时间', dataIndex: 'createTime', width: 150, render: this.renderTime },
    { title: '状态', dataIndex: 'status', width: 90, render: this.renderStatus },
    {
      title: '操作',
      key: 'action',
      width: COL_WIDTH_XL,
      fixed: 'right',
      getActionConfig: (text, record) => {
        const { btnAuth } = this.props;
        const btns = [];
        btns.push({
          text: '查看图片',
          onClick: () => {
            this.setState({ previewModal: true, selectedRecord: record });
          },
        });
        if (record.status === 0) {
          if (btnAuth.indexOf('AD_MANAGE_UP') > -1) {
            btns.push({
              text: '上架',
              onClick: () => {
                this.editAdStatus(record);
              },
            });
          }
          return btns;
        }
        if (btnAuth.indexOf('AD_MANAGE_SHARE') > -1) {
          btns.push({
            text: '分享',
            onClick: () => {
              this.share(record);
            },
          });
        }
        if (btnAuth.indexOf('AD_MANAGE_DOWN') > -1) {
          btns.push({
            text: '下架',
            onClick: () => {
              this.editAdStatus(record);
            },
          });
        }

        // if(btnAuth.indexOf('AD_MANAGE_DELETE') > -1){
        //   btns.push(
        //     { text: '删除', onClick: () => {this.deleteAd(record)} },
        //   );
        // }
        return btns;
      },
    },
  ];

  render() {
    const {
      loading,
      searchExpand,
      listLoading,
      rows,
      pageNo,
      pageSize,
      total,
      branchId,
      name,
      status,
      branchDict,
      shareModalVisible,
      shareAdObj,
      addNewAdModal,
      previewModal,
      selectedRecord,
      defaultBranchArray,
    } = this.state;
    const { route, btnAuth } = this.props;
    const shareUrl = buildBaseUrl({ orgId: shareAdObj.orgId });
    return (
      <Page
        className={styles.adManagement}
        loading={loading}
        route={route}
        fitWindowHeight={true}
        subHeader={
          <div className={styles.searchBar} style={{ paddingRight: '29px' }}>
            {btnAuth.indexOf('AD_MANAGE_ADD') > -1 && (
              <Button className={styles.addQuli} onClick={() => this.setState({ addNewAdModal: true })}>
                新增
              </Button>
            )}
            <SearchBar
              onSearch={this.search}
              onReset={this.reset}
              expand={searchExpand}
              onToggleExpand={(expand) => this.setState({ searchExpand: expand })}
            >
              <Input placeholder="宣传页名称" value={name} onChange={(e) => this.setState({ name: e.target.value })} />
              <MultiSelect
                placeholder="分公司"
                dict={branchDict}
                value={branchId}
                allowClear={false}
                showSearch={true}
                onChange={(branchId) => this.setState({ branchId })}
              />
              <Select
                placeholder="状态"
                dict={statusDict}
                value={status}
                allowClear={true}
                onChange={(status) => this.setState({ status })}
              />
            </SearchBar>
          </div>
        }
      >
        <List
          columns={this.columns}
          loading={listLoading}
          dataSource={rows || []}
          pagination={{
            current: pageNo,
            pageSize,
            total: total,
          }}
          onChange={this.pageChange}
        />
        {addNewAdModal && (
          <AddNewModal
            branchDict={branchDict}
            defaultBranchId={defaultBranch}
            onConfirm={this.addNewAdConfirm}
            onClose={() => this.setState({ addNewAdModal: false })}
          />
        )}
        {previewModal && (
          <PreviewModal
            onClose={() => this.setState({ previewModal: false, selectedRecord: {} })}
            src={selectedRecord.attachmentUrl}
          />
        )}
        <ShareQRCode
          title="分享"
          shareTitle={shareAdObj.name || '测试'}
          shareText={shareUrl}
          fileName="宣传页管理_二维码"
          visible={shareModalVisible}
          onCancel={() => {
            this.setState({ shareModalVisible: false, shareAdObj: {} });
          }}
        />
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    user: state.account.user,
    btnAuth: state.account.btnAuths || [],
  };
};
export default connect(mapStateToProps)(AdManagement);
