import React, { Component } from 'react';
import Select from 'COMPONENT/Select';
import Input from 'COMPONENT/Input';
import Modal from 'COMPONENT/Modal';
import Upload from 'COMPONENT/Upload';
import FormItem from 'COMPONENT/FormItem';
import Button from 'COMPONENT/Button';
import { Icon } from 'antd';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import styles from '../styles.less';
import PreviewModal from './PreviewModal';
import loader from '../loader';
import adExample from 'ASSET/imgs/adExample.png';

function ModalTitle() {
  return (
    <div>
      新增宣传页
      <span style={{ fontSize: '14px', color: '#999', marginLeft: '10px' }}>（进入在线报读的第一个页面）</span>
    </div>
  );
}

export default class AddNewModal extends Component {
  constructor(props) {
    super(props);

    const { defaultBranchId } = this.props;
    this.state = {
      branchId: defaultBranchId,
      name: '',
      loading: false,
      previewModal: false,
      file: {},
      isExample: false,
    };
  }

  confirm = async () => {
    const { branchId, name, file } = this.state;
    if (!branchId) {
      showMessage('请选择分公司');
      return false;
    }
    if (!name) {
      showMessage('请填写宣传页名称');
      return false;
    }
    if (!(file && file.src)) {
      showMessage('请选择宣传图片');
      return false;
    }
    const { onConfirm } = this.props;
    this.setState({ loading: true });
    try {
      const result = await loader.addAd({
        name,
        orgId: branchId,
        attachmentId: file['x:fileid'],
      });
      this.setState({ loading: true });
      onConfirm();
    } catch (e) {
      console.warn(e);
      showMessage(e.message);
      this.setState({ loading: true });
    }
  };

  onUploadChange = (file) => {
    if (file && file.src) {
      this.setState({ file: file });
    }
  };
  beforeUpload = (fileArray, fileList) => {
    const file = fileArray[0];
    const fileType = ['image/jpeg', 'image/png', 'image/jpg'];
    let isJPGOrPNG = fileType.indexOf(file.type) > -1;

    let isLt1m = file.size / 1024 / 1024 < 1;

    if (!isJPGOrPNG) {
      showMessage('只能上传jpg,png,jpeg文件');
      return false;
    }
    if (!isLt1m) {
      showMessage('图片大小超过1M');
      return false;
    }
    return fileArray;
  };

  render() {
    const { onClose, branchDict } = this.props;
    const { name, branchId, loading, file, previewModal, isExample } = this.state;

    return (
      <Modal
        title={<ModalTitle />}
        onCancel={onClose}
        okText="确定"
        cancelText="取消"
        onOk={this.confirm}
        width={650}
        loading={loading}
      >
        <div className={styles.addModal}>
          <FormItem labelWidth={100} name="宣传页名称" required>
            <Input
              isTrim
              placeholder="宣传页名称"
              value={name}
              onChange={(e) => this.setState({ name: e.target.value })}
            />
          </FormItem>
          <FormItem labelWidth={100} name="分公司" required>
            <Select
              placeholder="请选择分公司"
              dict={branchDict}
              value={branchId}
              onChange={(val) => this.setState({ branchId: val })}
            />
          </FormItem>
          <FormItem labelWidth={100} name="上传图片">
            {!file.src ? (
              <Upload onChange={this.onUploadChange} beforeUpload={this.beforeUpload}>
                <div className={styles.uploadWrapper}>
                  <span className={styles.dot}>+</span>
                  <span>支持格式：jpg.png.jpeg</span>
                  <span>分辨率建议为750*1334，大小不超过1M</span>
                </div>
              </Upload>
            ) : (
              <div
                className="hom-upload-img-view"
                style={{ width: 250, height: 133 }}
                // onMouseEnter={this.btnCoverToggle}
                // onMouseLeave={this.btnCoverToggle}
              >
                <div className="hom-upload-img-view-wrapper">
                  <img className="hom-upload-img-view-img" src={file.src} />
                </div>

                <div className="hom-upload-img-cover">
                  <div
                    className="hom-upload-img-detail"
                    onClick={(e) => this.setState({ previewModal: true })}
                    title="预览图片"
                  >
                    <Icon type="eye" style={{ color: '#fff', fontSize: 28 }} color="#fff" />
                  </div>
                  {
                    <div
                      className="hom-upload-img-delete"
                      onClick={(e) => this.setState({ file: {} })}
                      title="删除图片"
                    >
                      <Icon type="delete" style={{ color: '#fff', fontSize: 28 }} color="#fff" />
                    </div>
                  }
                </div>
              </div>
            )}
          </FormItem>
          <FormItem labelWidth={100} name="">
            <span
              style={{ color: '#6d67ff', cursor: 'pointer' }}
              onClick={() => this.setState({ isExample: true, previewModal: true })}
            >
              查看示例图
            </span>
          </FormItem>
          {previewModal && (
            <PreviewModal
              onClose={() => this.setState({ previewModal: false, isExample: false })}
              src={isExample ? adExample : file.src}
            />
          )}
        </div>
      </Modal>
    );
  }
}
