import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import List from 'COMPONENT/List';
import SearchBar from './NewSearchBar';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import Input from 'COMPONENT/Input';
import Popover from 'COMPONENT/Popover';
import Icon from 'COMPONENT/Icon';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { loadBranchCampusDict, loadBranchDict } from 'UTIL/dict';
import { DatePicker } from 'antd';
import * as loader from './loaders';
import moment from 'moment';
import SightDetailModal from './modal/SightDetailModal';
import styles from './styles.less';
import ButtonWrap from '../StdntApptMgmt/ButtonWrap';
import { renderMultiLine } from 'PAGE/signUpOnline/common';

const { RangePicker } = DatePicker;
const dateFormat = 'YYYY-MM-DD HH:mm';

const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;

function dictToObj(dict) {
  return (dict || []).reduce((obj, ele) => {
    obj[ele.val] = ele.text;
    return obj;
  }, {});
}

let defaultBranch = null; //默认第一个学习中心
let defaultCampus = null;
// 数据来源字典
const dataSourceDict = [
  { text: '购课单', val: 'cart' },
  { text: '订单管理', val: 'order' },
];
// 订单状态字典
const orderStatusDict = [
  { text: '未支付', val: 0 },
  { text: '已支付', val: 1 },
  { text: '支付中', val: 2 },
];

const typeMap = dictToObj(dataSourceDict);
class AdvanceSightUp extends React.Component {
  constructor(props) {
    super(props);
    const isCampusOrg = 'CAMPUS' === this.props.user.orgType;
    this.state = {
      loading: true, // boolean 页面加载状态
      searchExpand: true,
      shareActivity: null,
      selectedRecord: {},
      detailModalVisible: false,
      //查询条件
      branchId: [],
      intentionCampusId: null,
      blCampusId: null,
      name: null,
      studentName: null,
      payStatus: null,
      type: null,
      time: [moment('00:00:00', 'HH:mm:ss').subtract(1, 'week').add(1, 'day'), moment('23:59:59', 'HH:mm:ss')],
      //列表数据
      listLoading: false,
      rows: [],
      pageNo: 1,
      pageSize: 10,
      total: null,
      isCampusOrg,
      //数据字典数据
      branchDict: [],
      campusDict: [],
    };
  }

  componentDidMount() {
    this.initContent();
  }

  updateState = (changedState) => {
    const newState = {
      ...this.state,
      ...changedState,
    };
    return new Promise((resolve) => {
      this.setState(newState, () => {
        resolve();
      });
    });
  };

  parseTime = (time) => {
    let [beginTime, endTime] = time;
    return {
      beginTime: moment(beginTime).format('YYYY-MM-DD HH:mm:ss'),
      endTime: moment(endTime).format('YYYY-MM-DD HH:mm:ss'),
    };
  };

  getParams = (params = {}) => {
    const {
      intentionCampusId,
      blCampusId,
      branchId,
      payStatus,
      type,
      name,
      studentName,
      pageNo,
      pageSize,
      status,
      campusDict,
      time,
    } = this.state;
    // 拿的是支付中状态 那么就是未支付 并且orderStatus = [1]
    let orderStatus = null;
    let truePayStatus = typeof payStatus === 'number' ? [payStatus] : null;
    if (payStatus === 2) {
      orderStatus = [1];
      truePayStatus = [0, 2];
    }
    // 开始和结束时间
    let beginTime;
    let endTime;
    if (time === null) {
      beginTime = null;
      endTime = null;
    } else {
      let timer = this.parseTime(time);
      beginTime = timer.beginTime;
      endTime = timer.endTime;
    }

    return {
      intentionCampusId,
      branchId,
      blCampusId: blCampusId,
      customerPhone: name,
      studentName,
      payStatus: truePayStatus,
      type,
      pageNo,
      pageSize,
      status,
      orderStatus,
      beginTime,
      endTime,
      ...params,
    };
  };

  onExportClick = async (isExportWithPhone) => {
    this.setState({ loading: true });
    const apiParams = this.getParams();
    if (isExportWithPhone) {
      // 手机号码也导出
      apiParams.exportContact = true;
    }
    try {
      const href = await loader.getOrderDownloadUrl(apiParams);
      await loader.exportDemo({ href, download: '预报名管理' });
    } catch (e) {
      showMessage('导出失败：' + e.resultMessage, 'error');
    }
    this.setState({ loading: false });
  };

  loadListData = async (params) => {
    const apiParams = this.getParams(params);
    const list = await loader.getList(apiParams);
    const studentIdArray = Array.from(new Set(list.records.map((record) => record.bossStudentId)));
    const studentList = await loader.getStudentDataById({
      studentNo: studentIdArray,
    });
    const studentMap = studentList.reduce((obj, cur) => {
      obj[cur.id] = cur;
      return obj;
    }, {});
    list.records = list.records.map((record) => {
      const { blCampusName, gradeName = '无', name = '无' } = studentMap[record.bossStudentId] || {};
      const newRecords = {
        ...record,
        bossGrade: gradeName,
        bossName: name,
      };
      if (blCampusName) {
        newRecords.blCampusName = blCampusName;
      }
      return newRecords;
    });
    return list;
  };

  initContent = async () => {
    this.setState({ loading: true });
    const asyncCalls = [loadBranchCampusDict()];
    try {
      const [branchCampusDict] = await Promise.all(asyncCalls);

      const branchDict = branchCampusDict.map((branchData) => ({
        text: branchData.label,
        val: branchData.val,
      }));
      //默认第一个分公司，当前年份
      defaultBranch = branchCampusDict[0] ? [branchCampusDict[0].val] : [];
      defaultCampus = branchCampusDict[0].options ? [branchCampusDict[0].options[0].val] : [];
      const campusParams = this.state.isCampusOrg
        ? { intentionCampusId: defaultCampus, blCampusId: defaultCampus }
        : {};
      //获取列表数据
      const params = {
        ...campusParams,
        branchId: defaultBranch,
      };
      await this.updateState({
        branchId: defaultBranch,
        branchDict,
        campusDict: branchCampusDict,
        ...params,
      });
      const list = await this.loadListData(params);
      this.setState({
        loading: false,
        rows: list.records,
        total: list.total,
      });
    } catch (e) {
      this.setState({
        loading: false,
      });
      showMessage(e.message);
    }
  };

  search = async () => {
    await this.updateState({ listLoading: true, pageNo: 1 });
    try {
      const list = await this.loadListData();
      this.updateState({
        listLoading: false,
        rows: list.records,
        total: list.total,
      });
    } catch (error) {
      this.setState({
        listLoading: false,
      });
      showMessage(error.message);
    }
  };

  reset = async () => {
    const params = this.state.isCampusOrg
      ? { intentionCampusId: defaultCampus, blCampusId: defaultCampus }
      : { intentionCampusId: null, blCampusId: null };
    this.updateState({
      time: [moment('00:00:00', 'HH:mm:ss').subtract(1, 'week').add(1, 'day'), moment('23:59:59', 'HH:mm:ss')],
      payStatus: null,
      type: null,
      branchId: defaultBranch,
      name: null,
      studentName: null,
      ...params,
    });
  };

  pageChange = async ({ current, pageSize }) => {
    await this.updateState({ listLoading: true, pageSize, pageNo: current });
    try {
      const list = await this.loadListData();
      this.updateState({
        listLoading: false,
        rows: list.records,
        total: list.total,
        pageNo: current,
      });
    } catch (error) {
      this.setState({
        listLoading: false,
      });
      showMessage(error.message);
    }
  };

  updateRow = (raw, changed) => {
    this.setState({ listLoading: true });
    const idx = this.state.rows.findIndex((ele) => ele.id === raw.id);
    const rows = this.state.rows.map((row, i) => {
      if (i === idx) {
        return { ...row, ...changed };
      }
      return row;
    });
    this.setState({
      listLoading: false,
      rows,
    });
    return { ...raw, ...changed };
  };

  onShareConfirm = () => {
    const shareActivity = this.updateRow(this.state.shareActivity, {
      shareNum: (this.state.shareActivity.shareNum || 0) + 1,
    });
    this.setState({
      shareActivity,
    });
  };

  getSightUpDetail = (record) => {
    this.setState({
      detailModalVisible: true,
      selectedRecord: record,
    });
  };
  renderTime = (createTime) => {
    return (
      <div className={styles.timeWrapper}>
        <p>{moment(createTime).format('YYYY-MM-DD HH:mm:ss')}</p>
      </div>
    );
  };

  /**
   * 表头
   */
  columns = [
    {
      title: '序号',
      dataIndex: 'index',
      width: 70,
      fixed: 'left',
      render: (text, record, index) => index + 1,
    },
    {
      title: '移动端名称',
      dataIndex: 'studentName',
      width: COL_WIDTH_M,
      filterDropdownVisible: false,
      filterIcon: (
        <Popover overlayClassName="sectionPayment-list-popover" content="在app、报读H5中家长填写的名字，可由家长编辑">
          <Icon type="question_circle" size={12} style={{ marginLeft: 4 }} />
        </Popover>
      ),
      filterDropdown: <span className="empty-content"> </span>,
    },
    {
      title: '移动端年级',
      dataIndex: 'gradeName',
      width: COL_WIDTH_M,
      filterDropdownVisible: false,
      filterIcon: (
        <Popover overlayClassName="sectionPayment-list-popover" content="在app、报读H5中家长填写的年级，可由家长编辑">
          <Icon type="question_circle" size={12} style={{ marginLeft: 4 }} />
        </Popover>
      ),
      filterDropdown: <span className="empty-content"> </span>,
    },
    {
      title: 'boss名称',
      dataIndex: 'bossName',
      width: COL_WIDTH_M,
      filterDropdownVisible: false,
      filterIcon: (
        <Popover overlayClassName="sectionPayment-list-popover" content="在boss中学生的名字">
          <Icon type="question_circle" size={12} style={{ marginLeft: 4 }} />
        </Popover>
      ),
      filterDropdown: <span className="empty-content"> </span>,
    },
    {
      title: 'boss年级',
      dataIndex: 'bossGrade',
      width: COL_WIDTH_M,
      filterDropdownVisible: false,
      filterIcon: (
        <Popover overlayClassName="sectionPayment-list-popover" content="在boss中学生的学籍年级">
          <Icon type="question_circle" size={12} style={{ marginLeft: 4 }} />
        </Popover>
      ),
      filterDropdown: <span className="empty-content"> </span>,
    },
    { title: '客户手机号码', dataIndex: 'customerPhone', width: COL_WIDTH_M },
    {
      title: '合同编号',
      dataIndex: 'contractId',
      width: COL_WIDTH_M,
      render: (text) => text || '-',
    },

    {
      title: '准考证号',
      dataIndex: 'bossStudentId',
      width: COL_WIDTH_L,
      render: (text) => (text ? text.replace('STU', '') : '暂无'),
      filterDropdownVisible: false,
      filterIcon: (
        <Popover content="用于教学平台的阅卷系统使用">
          <Icon type="question_circle" size={12} style={{ marginLeft: 2 }} />
        </Popover>
      ),
      filterDropdown: <span className="empty-content"> </span>,
    },
    { title: '分公司', dataIndex: 'branchName', width: COL_WIDTH_M },
    {
      title: '课程所属学习中心',
      dataIndex: 'intentionCampusName',
      width: COL_WIDTH_L,
      render: renderMultiLine,
      filterDropdownVisible: false,
      filterIcon: (
        <Popover content="订单内课程的所属学习中心">
          <Icon type="question_circle" size={12} style={{ marginLeft: 2 }} />
        </Popover>
      ),
      filterDropdown: <span className="empty-content"> </span>,
    },
    { title: '学籍学习中心', dataIndex: 'blCampusName', width: COL_WIDTH_M },
    {
      title: '数据来源',
      dataIndex: 'type',
      width: COL_WIDTH_M,
      render: (text) => typeMap[text],
    },
    {
      title: '订单状态',
      dataIndex: 'payStatus',
      width: 200,
      render: (_, record) => {
        const { payStatus, totalAmount, type } = record;
        if (type === 'cart') {
          return '-';
        }
        if (+payStatus === 0) {
          return '未支付';
        } else if (+payStatus === 1) {
          return `已支付（${totalAmount / 100} 元）`;
        } else if (+payStatus === 2) {
          return '支付中';
        }
      },
    },
    { title: '在读学校', dataIndex: 'schoolName', width: 150 },
    {
      title: '创建时间',
      dataIndex: 'createTime',
      width: 150,
      render: this.renderTime,
    },
    {
      title: '操作',
      key: 'action',
      width: COL_WIDTH_XL,
      fixed: 'right',
      getActionConfig: (text, record) => {
        const { btnAuth } = this.props;
        const btns = [];
        btns.push({
          text: '详情',
          onClick: () => {
            this.getSightUpDetail(record);
          },
        });
        return btns;
      },
    },
  ];

  render() {
    const {
      loading,
      searchExpand,
      listLoading,
      rows,
      pageNo,
      pageSize,
      total,
      branchId,
      intentionCampusId,
      type,
      name,
      studentName,
      payStatus,
      branchDict,
      campusDict,
      shareActivity,
      blCampusId,
      detailModalVisible,
      selectedRecord,
      isCampusOrg,
      time,
    } = this.state;
    const { route, btnAuth } = this.props;

    const campusDictMap = campusDict.reduce((obj, cur) => {
      obj[cur.val] = cur;
      return obj;
    }, {});
    let selectCampusDict = Array.isArray(branchId) ? branchId.map((id) => campusDictMap[id]) : campusDictMap;
    const exportBtnArray = [
      {
        text: '含手机号码导出',
        onClick: () => this.onExportClick(true),
        key: 'BTN_ADVANCE_SIGHT',
      },
      {
        text: '不含手机号码导出',
        onClick: () => this.onExportClick(false),
        key: 'BTN_ADVANCE_NO_PHONE',
      },
    ];
    const exportBtnActionArray = exportBtnArray.filter((btnAuthItem) => btnAuth.indexOf(btnAuthItem.key) > -1);
    const isShowExportBtn = !!exportBtnActionArray.length;
    return (
      <Page
        loading={loading}
        route={route}
        // fitWindowHeight={true}
        subHeader={
          <div className={styles.searchBar} style={{ paddingRight: '66px' }}>
            {isShowExportBtn && (
              <ButtonWrap
                btnStyle={{ position: 'absolute', right: 0, top: 0 }}
                btnText="导出Excel"
                popPlacement="bottom"
                actions={exportBtnActionArray}
              />
            )}
            <SearchBar
              childWidth={[1, 1, 2, 1, 1, 1, 1, 1]}
              onSearch={this.search}
              onReset={this.reset}
              expand={searchExpand}
              onToggleExpand={(expand) => this.setState({ searchExpand: expand })}
            >
              <Input
                placeholder="客户手机号码/客户名称"
                value={name}
                onChange={(e) =>
                  this.setState({
                    name: e.target.value.replace(/(^\s*)|(\s*$)/g, ''),
                  })
                }
              />
              <Input
                placeholder="学生boss名称/学生移动端名称"
                value={studentName}
                onChange={(e) =>
                  this.setState({
                    studentName: e.target.value.replace(/(^\s*)|(\s*$)/g, ''),
                  })
                }
              />
              <RangePicker
                style={{ width: '100%' }}
                size="large"
                format={dateFormat}
                showTime={{
                  hideDisabledOptions: true,
                  defaultValue: [moment('00:00:00', 'HH:mm:ss'), moment('23:59:59', 'HH:mm:ss')],
                }}
                placeholder={['开始时间', '结束时间']}
                value={time}
                onChange={(time) => {
                  this.setState({ time: time.length === 0 ? null : time });
                }}
              />
              <MultiSelect
                placeholder="分公司"
                dict={branchDict}
                value={branchId}
                allowClear={false}
                showSearch={true}
                onChange={(branchId) =>
                  this.setState({
                    branchId: branchId || [],
                    intentionCampusId: null,
                    blCampusId: null,
                  })
                }
              />
              <MultiSelect
                placeholder="课程所属学习中心"
                dict={selectCampusDict}
                value={intentionCampusId}
                allowClear={!isCampusOrg}
                showSearch={true}
                onChange={(intentionCampusId) => this.setState({ intentionCampusId: intentionCampusId })}
              />
              <MultiSelect
                placeholder="学籍学习中心"
                dict={selectCampusDict}
                value={blCampusId}
                allowClear={!isCampusOrg}
                showSearch={true}
                onChange={(blCampusId) => this.setState({ blCampusId: blCampusId })}
              />
              <Select
                placeholder="数据来源"
                dict={dataSourceDict}
                value={type}
                allowClear={true}
                onChange={(type) => this.setState({ type })}
              />
              <Select
                placeholder="订单状态"
                dict={orderStatusDict}
                value={payStatus}
                allowClear={true}
                onChange={(payStatus) => this.setState({ payStatus })}
              />
            </SearchBar>
          </div>
        }
      >
        <List
          columns={this.columns}
          loading={listLoading}
          dataSource={rows || []}
          pagination={{
            current: pageNo,
            pageSize,
            total: total,
          }}
          onChange={this.pageChange}
        />
        {detailModalVisible && (
          <SightDetailModal
            selectRecord={selectedRecord}
            onClose={() => {
              this.setState({ detailModalVisible: false, selectedRecord: {} });
            }}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    user: state.account.user,
    btnAuth: state.account.btnAuths || [],
  };
};
export default connect(mapStateToProps)(AdvanceSightUp);
