import React from 'react';
import Modal from 'COMPONENT/Modal';
import Block from 'COMPONENT/Block';
import List from 'COMPONENT/List';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { SYSTEM_MARK } from 'CONST/config';

import styles from '../styles.less';

import * as loader from '../loaders';

const { COL_WIDTH_S, COL_WIDTH_L } = List;
const columns = [
  { title: '课程名称', dataIndex: 'productName', width: 160 },
  { title: '所属学习中心', dataIndex: 'organizationName', width: COL_WIDTH_S },
];

export default class SightDetailModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false,
      cartData: [],
      orderData: {},
      showOrderData: false,
    };
  }
  componentDidMount() {
    this.initContent();
  }

  initContent = async () => {
    const {
      selectRecord: { type, studentNo, orderNo },
    } = this.props;
    this.setState({
      loading: true,
    });
    const typeFetch = {
      cart: {
        fetchFn: 'getCartDetail',
        params: {
          studentNo,
          orgSysType: SYSTEM_MARK,
        },
      },
      order: {
        fetchFn: 'getOrderDetail',
        params: {
          orderNo,
          orgSysType: SYSTEM_MARK,
        },
      },
    };
    const { fetchFn, params } = typeFetch[type];
    try {
      const detail = await loader[fetchFn](params);
      let setStateParams = {};
      if (type === 'cart') {
        setStateParams = {
          cartData: detail,
        };
      } else {
        const { orderItems = [], orderTotalAmount, discount = 0, contractId, payAmount = 0 } = detail;
        const orderData = {
          orderId: contractId,
          discount,
          orderTotalAmount,
          payAmount,
        };
        // 支付成功了才显示订单详情
        setStateParams = {
          cartData: orderItems,
          orderData,
          showOrderData: true,
        };
      }
      this.setState(setStateParams);
    } catch (e) {
      showMessage('数据错误：' + e.resultMessage, 'error');
    }
    this.setState({ loading: false });
  };
  render() {
    const {
      loading,
      orderData: { discount, orderTotalAmount, payAmount, orderId },
      showOrderData,
      cartData = [],
    } = this.state;
    const {
      onClose,
      selectRecord: { studentName, customerPhone, gradeName },
    } = this.props;
    return (
      <Modal title="报读详情" width={600} cancelText="关闭" onCancel={onClose} okBtnShow={false} loading={loading}>
        <div>{`${studentName}(${customerPhone})   ${gradeName}`}</div>
        <Block title="添加的课程">
          <List columns={columns} dataSource={cartData} />
        </Block>
        {showOrderData && (
          <Block title="订单信息">
            <div className={styles.sightModalContent}>
              <span className={styles.sightModalText}>订单原价： </span>
              <span>{`${(discount + orderTotalAmount) / 100}元`}</span>
            </div>
            <div className={styles.sightModalContent}>
              <span className={styles.sightModalText}>优惠金额： </span>
              <span>{`${discount / 100}元`}</span>
            </div>
            <div className={styles.sightModalContent}>
              <span className={styles.sightModalText}>待支付金额： </span>
              <span>{`${(orderTotalAmount - payAmount) / 100}元`}</span>
            </div>
            <div className={styles.sightModalContent}>
              <span className={styles.sightModalText}>已支付： </span>
              <span>{`${payAmount / 100}元`}</span>
            </div>
            {orderId && (
              <div className={styles.sightModalContent}>
                <span className={styles.sightModalText}>合同编号： </span>
                <span>{orderId}</span>
              </div>
            )}
          </Block>
        )}
      </Modal>
    );
  }
}
