import React from 'react';
import { connect } from 'dva';
import Button from 'COMPONENT/Button';
import Page from 'COMPONENT/Page';
import List from 'COMPONENT/List';
import SearchBar from 'COMPONENT/SearchBar';
import Input from 'COMPONENT/Input';
import Select from 'COMPONENT/Select';
import EditModal from './editModal/index';
import ShowConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import ShowMessage from 'COMPONENT/utils/globalTips/showMessage';
import { loadCommonDict, loadBranchDict } from 'UTIL/dict';
import SelectPreferential from '../common/SelectPreferential/index';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import Loader from './loader';
import moment from 'moment';
import { PROJECT_NAME } from '../../../constants/config';

import { renderMultiLine } from '../common';

import commonStyles from '../common/commonStyle.less';
const spingId = 'DAT0000000231'; //春季id
const autumnId = 'DAT0000000233'; //秋季id

const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL, createSetWidth } = List;
const setWidth = createSetWidth();
const validDict = [
  { text: '有效', val: '1' },
  { text: '无效', val: '0' },
];
let defaultBranch = null; //默认第一个学习中心
const productTypeMap = {
  TWO_TEACHER: 'TWO_TEACHER',
  SMALL_CLASS: 'SMALL_CLASS',
};
const productTypeDict = [
  { text: '双师', val: productTypeMap.TWO_TEACHER },
  { text: '小班', val: productTypeMap.SMALL_CLASS },
];
const getValByText = (dict, text) => {
  const item = dict.find((item) => {
    return item.text === text;
  });
  if (item) {
    return item.val;
  } else {
    return null;
  }
};

class EnrollPeriod extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: true, // boolean 页面加载状态
      //查询条件
      name: null,
      studentId: null,
      branchId: null,
      phaseId: null,
      examTypeId: null,
      productTypeId: PROJECT_NAME === 'twoteacher' ? productTypeMap.TWO_TEACHER : productTypeMap.SMALL_CLASS,
      yearId: null,
      quarterId: null,
      gradeId: null,
      subjectId: null,
      tagType: null,
      valid: null,
      //列表数据
      listLoading: false,
      rows: [],
      page: 1,
      pageSize: 20,
      total: null,
      //数据字典数据
      weekDict: [],
      branchDict: null,
      yearDict: null,
      quarterDict: null,
      gradeDict: null,
      subjetDict: null,
      classTypeDict: null,
      tagTypeDict: null,
      entryTypeDict: null,
      //其他
      editQuliShow: false,
      editModalShow: false,
      searchExpand: false,
      editId: null,
      editMode: null,
      paperScoreImgs: [],
      selectModalShow: false,
      discountIds: [],
      characterMap: {},
      editCharacterId: null,
      discountMode: null,
      startTime: null,
      endTime: null,
      discountBranch: null,
    };
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 更新状态
   * @param  {object} changedState 改变的状态
   * @return {Promise}
   */
  updateState = (changedState) => {
    const newState = {
      ...this.state,
      ...changedState,
    };
    return new Promise((resolve) => {
      this.setState(newState, () => {
        resolve();
      });
    });
  };

  /**
   * 获取列表数据
   */
  loadListData = async (params) => {
    const {
      name,
      studentId,
      branchId,
      productTypeId,
      yearId,
      quarterId,
      subjectId,
      gradeId,
      pageSize,
      page,
      tagType,
      valid,
      phaseId,
    } = this.state;
    const apiParams = {
      name,
      studentId,
      branchId,
      productTypeId,
      yearId,
      quarterId,
      subjectId,
      gradeId,
      pageSize,
      page,
      tagType,
      valid,
      pageNo: page,
      phaseId,
      ...params,
    };
    try {
      return await Loader.loadList(apiParams);
    } catch (e) {
      ShowMessage(e.text);
    }
  };

  /**
   * 初始化内容
   */
  initContent = async () => {
    this.setState({ loading: true });
    const asyncCalls = [
      loadBranchDict(),
      loadCommonDict('PRODUCT_VERSION'),
      loadCommonDict('PRODUCT_QUARTER'),
      loadCommonDict('STUDENT_GRADE'),
      loadCommonDict('SUBJECT'),
      loadCommonDict('QUALIFICATION_TAG_TYPE'),
      loadCommonDict('ENTRY_TYPE'),
      loadCommonDict('SMALL_CLASS_PHASE'),
      loadCommonDict('PRODUCT_CHARACTER'),
      loadCommonDict('WeekDay'),
    ];
    Promise.all(asyncCalls).then(
      ([
        branchDict,
        yearDict,
        quarterDict,
        gradeDict,
        subjetDict,
        tagTypeDict,
        entryTypeDict,
        phaseDict,
        characterDict,
        weekDict,
      ]) => {
        //默认第一个分公司，当前年份
        defaultBranch = branchDict[0] ? branchDict[0].val : null;
        const defaultYear = getValByText(yearDict, new Date().getFullYear().toString()) || null;
        const characterMap = characterDict.reduce((prev, obj) => {
          prev[obj.val] = obj.text;
          return prev;
        }, {});

        //获取列表数据
        this.loadListData({ yearId: defaultYear, branchId: defaultBranch }).then((list) => {
          this.setState({
            yearId: defaultYear,
            branchId: defaultBranch,
            loading: false,
            rows: list.datas,
            total: list.rowCount,
            page: list.pageNo,
            branchDict,
            yearDict,
            quarterDict,
            gradeDict,
            subjetDict,
            tagTypeDict,
            entryTypeDict,
            phaseDict,
            weekDict,
            characterMap,
          });
        });
      },
    );
  };

  /**
   * 查询列表
   */
  search = async () => {
    await this.updateState({ listLoading: true, page: 1 });
    const list = await this.loadListData();
    this.updateState({
      listLoading: false,
      rows: list.datas,
      total: list.rowCount,
      page: list.pageNo,
    });
  };

  /**
   * 重置搜索表单
   */
  reset = async () => {
    this.updateState({
      name: null,
      studentId: null,
      branchId: defaultBranch,
      examTypeId: null,
      productTypeId: null,
      yearId: null,
      quarterId: null,
      subjectId: null,
      gradeId: null,
      tagType: null,
      valid: null,
      phaseId: null,
    });
  };

  /**
   * 翻页
   */

  pageChange = async ({ current, pageSize }) => {
    await this.updateState({ listLoading: true, pageSize, page: current });
    const list = await this.loadListData();
    this.updateState({
      listLoading: false,
      rows: list.datas,
      total: list.rowCount,
      page: list.pageNo,
    });
  };

  /**
   * 显示编辑模态框
   */
  showEditModal = (record, mode) => {
    this.setState({
      editMode: mode,
      editId: record.id,
      editModalShow: true,
      editCharacterId: record.productCharacterId,
    });
  };

  /**
   * 关闭模态框的回调
   */
  handleModalOk = () => {
    this.setState({
      editId: null,
      editMode: null,
      editModalShow: false,
      editCharacterId: null,
    });
    this.search();
  };

  /**
   * 改变无效和有效状态
   */
  changeStatus = (id) => {
    ShowConfirm('确定改变当前周期状态？', async () => {
      await this.updateState({ loading: true });
      try {
        const data = await Loader.changeStatus({ id });
        ShowMessage(data.resultMessage, 'success');
        await this.updateState({ loading: false });
        this.search();
      } catch (e) {
        ShowMessage(e.text);
        this.setState({
          loading: false,
        });
      }
    });
  };

  /**
   * 显示隐藏选择优惠
   */
  toggleSelectPreferen = (discountIds, mode, startTime, endTime, discountBranch) => {
    this.setState({
      selectModalShow: !this.state.selectModalShow,
      discountIds,
      discountMode: mode,
      startTime,
      endTime,
      discountBranch,
    });
  };

  /**
   * 保存优惠
   */
  onSavePreferen = (ids) => {
    this.editRef.updateDiscount(ids, this.state.discountMode);
    this.toggleSelectPreferen();
  };

  disableBtn = (record) => {
    const originalEnrollStartTime = moment(record.originalEnrollStartTime || Date.now()).valueOf(),
      now = moment().valueOf(),
      acrossEnrollEndTime = moment(record.acrossEnrollEndTime || Date.now()).valueOf();

    return now < originalEnrollStartTime || now > acrossEnrollEndTime;
  };

  renderStatus = (detail, rowData, index) => {
    return (
      <p key="asd" style={{ display: 'flex' }}>
        <b style={{ fontSize: '25px', color: detail === '有效' ? '#FFCC49' : '#AAA', marginRight: '3px' }}>·</b>
        {detail}
      </p>
    );
  };

  /**
   * 渲染续报活动明细
   */
  renderDetail = (detail, rowData, index) => {
    const { originalEnrollStartTime, originalEnrollEndTime, acrossEnrollStartTime, acrossEnrollEndTime } = rowData;
    return (
      <div>
        <p key="asd">
          {originalEnrollStartTime
            ? '原班续报：' + originalEnrollStartTime + ' ~ ' + originalEnrollEndTime
            : '原班续报：暂无'}
        </p>
        <p key="zxc">
          {acrossEnrollStartTime
            ? '跨班续报：' + acrossEnrollStartTime + ' ~ ' + acrossEnrollEndTime
            : '跨班续报：暂无'}
        </p>
      </div>
    );
  };

  renderCharacter = (detail, rowData, index) => {
    if (!detail) {
      return '';
    }
    const characterString = detail
      .split(',')
      .map((id) => this.state.characterMap[id])
      .join(',');
    return renderMultiLine(characterString);
  };

  handleQuarter = (quarterId) => {
    // 春秋的话需要把字典切成星期数
    this.setState({ quarterId, phaseId: null });
  };

  /**
   * 表头
   */
  columns = [
    {
      title: '序号',
      dataIndex: 'index',
      width: setWidth(2),
      fixed: 'left',
      render: (text, record, index) => index + 1,
    },
    { title: '续报活动名称', dataIndex: 'name', width: COL_WIDTH_XL, fixed: 'left' },
    { title: '产品类型', dataIndex: 'productTypeName', width: COL_WIDTH_M },
    { title: '年份', dataIndex: 'yearName', width: COL_WIDTH_S },
    { title: '季度', dataIndex: 'quarterName', width: COL_WIDTH_S },
    { title: '期数/星期数', dataIndex: 'phaseName', width: COL_WIDTH_M },
    { title: '分公司', dataIndex: 'branchName', width: COL_WIDTH_M, render: renderMultiLine },
    { title: '年级', dataIndex: 'gradeName', width: COL_WIDTH_S, render: renderMultiLine },
    { title: '产品性质', dataIndex: 'productCharacterId', width: COL_WIDTH_M, render: this.renderCharacter },
    { title: '科目', dataIndex: 'subjectName', width: COL_WIDTH_S, render: renderMultiLine },
    { title: '续报活动明细', dataIndex: 'originalEnrollStartTime', width: 300, render: this.renderDetail },
    { title: '状态', dataIndex: 'status', width: COL_WIDTH_S, render: this.renderStatus },
    {
      title: '操作',
      key: 'action',
      width: COL_WIDTH_L,
      fixed: 'right',
      getActionConfig: (text, record, index) => {
        const { status } = record;
        const btns = [{ text: '详情', onClick: () => this.showEditModal(record, 'detail') }];
        if (this.props.btnAuths.indexOf('BTN_PERIOD_EDIT') >= 0) {
          btns.unshift({ text: '修改', onClick: () => this.showEditModal(record) });
        }
        if (this.props.btnAuths.indexOf('BTN_PERIOD_TOGGLE') >= 0) {
          btns.push({
            text: status === '无效' ? '有效' : '无效',
            onClick: () => this.changeStatus(record.id),
            disabled: this.disableBtn(record),
          });
        }
        return btns;
      },
    },
  ];

  render() {
    const {
      loading,
      searchExpand,
      listLoading,
      rows,
      page,
      pageSize,
      total,
      name,
      branchId,
      productTypeId,
      yearId,
      quarterId,
      gradeId,
      subjectId,
      valid,
      branchDict,
      yearDict,
      quarterDict,
      gradeDict,
      subjetDict,
      editModalShow,
      editId,
      editMode,
      selectModalShow,
      phaseDict,
      weekDict,
      phaseId,
      discountIds,
      startTime,
      endTime,
      discountBranch,
      editCharacterId,
    } = this.state;
    let phaseOrWeekDict = [];
    if (quarterId) {
      if (quarterId === autumnId || quarterId === spingId) {
        phaseOrWeekDict = weekDict;
      } else {
        phaseOrWeekDict = phaseDict;
      }
    }
    return (
      <Page
        loading={loading}
        route={this.props.route}
        subHeader={
          <div className={commonStyles.searchBar}>
            {this.props.btnAuths.indexOf('BTN_PERIOD_ADD') >= 0 && (
              <Button
                className={commonStyles.addQuli}
                onClick={() => this.setState({ editModalShow: true, editId: null, editCharacterId: null })}
              >
                新增
              </Button>
            )}
            <SearchBar
              onSearch={this.search}
              onReset={this.reset}
              expand={searchExpand}
              onToggleExpand={(expand) => this.setState({ searchExpand: expand })}
            >
              <Input
                value={name}
                placeholder="续报活动名称"
                onChange={(e) => this.setState({ name: e.target.value })}
              />
              <Select
                placeholder="分公司"
                dict={branchDict}
                value={branchId}
                allowClear={false}
                showSearch={true}
                onChange={(branchId) => this.setState({ branchId })}
              />
              <Select
                placeholder="产品类型"
                dict={productTypeDict}
                value={productTypeId}
                allowClear={true}
                onChange={(productTypeId) => this.setState({ productTypeId: productTypeId })}
              />
              <Select
                placeholder="年份"
                dict={yearDict}
                value={yearId}
                allowClear={true}
                onChange={(yearId) => this.setState({ yearId })}
              />
              <Select
                placeholder="季度"
                dict={quarterDict}
                value={quarterId}
                allowClear={true}
                onChange={this.handleQuarter}
              />
              <Select
                placeholder="年级"
                dict={gradeDict}
                value={gradeId}
                allowClear={true}
                onChange={(gradeId) => this.setState({ gradeId })}
              />
              <Select
                placeholder="科目"
                dict={subjetDict}
                value={subjectId}
                allowClear={true}
                onChange={(subjectId) => this.setState({ subjectId })}
              />
              <Select
                placeholder="状态"
                dict={validDict}
                value={valid}
                allowClear={true}
                onChange={(valid) => this.setState({ valid })}
              />
              <Select
                placeholder={quarterId ? '请选择期数/星期数' : '请先选择季度'}
                dict={phaseOrWeekDict}
                value={phaseId}
                disabled={!quarterId}
                allowClear={true}
                onChange={(phaseId) => this.setState({ phaseId })}
              />
            </SearchBar>
          </div>
        }
        fitWindowHeight
      >
        <List
          columns={this.columns}
          loading={listLoading}
          dataSource={rows}
          pagination={{
            current: page,
            pageSize,
            total: total,
          }}
          onChange={this.pageChange}
        />
        {editModalShow && (
          <EditModal
            editId={editId}
            editMode={editMode}
            editCharacterId={editCharacterId}
            ref={(e) => {
              this.editRef = getWrappedInstance(e);
            }}
            onOk={this.handleModalOk}
            toggleSelectPreferen={this.toggleSelectPreferen}
            onCancel={() => {
              this.setState({ editModalShow: false, editId: null, editMode: null, editCharacterId: null });
            }}
          />
        )}
        {selectModalShow && (
          <SelectPreferential
            editMode={editMode}
            startTime={startTime}
            endTime={endTime}
            discountBranch={discountBranch}
            discountIds={discountIds}
            onCancel={this.toggleSelectPreferen}
            onOk={this.onSavePreferen}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProp = (state) => {
  return {
    btnAuths: state.account.btnAuths,
  };
};

export default connect(mapStateToProp, null)(EnrollPeriod);
