import React from 'react';
import Modal from 'COMPONENT/Modal';
import { Row, Col } from 'antd';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import FormItem from 'COMPONENT/FormItem';
import { loadCommonDict } from 'UTIL/dict';
import { getCommonErrorTips } from 'COMPONENT/utils/form';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import ShowMessage from 'COMPONENT/utils/globalTips/showMessage';
import Loader from './loader';

export default class EditModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false,
      productTypeDict: [
        { text: '双师', val: 'TWO_TEACHER' },
        { text: '小班', val: 'SMALL_CLASS' },
      ],
      yearId: null,
      productTypeId: null,
      quarterId: null,
      checking: false,
    };
    this.errors = {};
  }
  componentDidMount() {
    this.initContent();
  }

  initContent = () => {
    this.setState({ loading: true });
    const asyncCalls = [loadCommonDict('PRODUCT_VERSION'), loadCommonDict('PRODUCT_QUARTER')];
    Promise.all(asyncCalls).then(
      ([yearDict, quarterDict]) => {
        this.setState({
          yearDict,
          quarterDict,
        });
        this.setState({ loading: false });
      },
      () => this.setState({ loading: false }),
    );
  };

  onOk = async () => {
    const { productTypeId, yearId, quarterId, checking } = this.state;
    if (!checking) {
      this.setState({ checking: true });
    }
    let errorTips = getCommonErrorTips(this.errors);
    if (errorTips) {
      this.modal.showOkTips(errorTips);
      return;
    }

    this.setState({ loading: true });
    try {
      await Loader.addQuarter({ productTypeId, yearId, quarterId });
      ShowMessage('新增成功', 'success');
      this.props.onOk();
    } catch (err) {
      ShowMessage(err.message);
      this.setState({ loading: false });
    }
  };

  errorChange = (key, err) => {
    this.errors[key] = err;
  };

  render() {
    const { loading, productTypeDict, productTypeId, yearDict, yearId, quarterDict, quarterId, checking } = this.state;
    const { editId } = this.props;
    return (
      <Modal
        title={editId ? '编辑报读季度' : '新增报读季度'}
        showOkBtn
        width={600}
        closable
        onOk={this.onOk}
        onCancel={this.props.onCancel}
        loading={loading}
        ref={(c) => (this.modal = getWrappedInstance(c))}
      >
        <div style={{ padding: '50px 0' }}>
          <Row>
            <Col span={7}>
              <Select
                required
                placeholder="产品类型"
                dict={productTypeDict}
                value={productTypeId}
                onChange={(productTypeId) => this.setState({ productTypeId })}
                validator={{
                  silent: !checking,
                  onErrorChange: (e) => this.errorChange('productTypeId', e),
                }}
              />
            </Col>
            <Col span={7} offset={1}>
              <Select
                required
                placeholder="产品年份"
                dict={yearDict}
                value={yearId}
                onChange={(yearId) => this.setState({ yearId })}
                validator={{
                  silent: !checking,
                  onErrorChange: (e) => this.errorChange('yearId', e),
                }}
              />
            </Col>
            <Col span={7} offset={1}>
              <MultiSelect
                required
                placeholder="产品季度"
                dict={quarterDict}
                value={quarterId}
                onChange={(quarterId) => this.setState({ quarterId })}
                validator={{
                  silent: !checking,
                  onErrorChange: (e) => this.errorChange('quarterId', e),
                }}
              />
            </Col>
          </Row>
        </div>
      </Modal>
    );
  }
}
