import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import IconButton from 'COMPONENT/IconButton';
import EditModal from './EditModal';
import Loader from './loader';
import ShowMessage from 'COMPONENT/utils/globalTips/showMessage';
import ShowConfim from 'COMPONENT/utils/globalTips/showConfirm';
import ShowAlert from 'COMPONENT/utils/globalTips/showAlert';
import { loadCommonDict, loadBranchDict, loadBranchCampusDict } from 'UTIL/dict';
import styles from './styles.less';

/**
 * UI给的svg
 * @returns {*}
 * @constructor
 */
function AddIcon(props) {
  return (
    <svg {...props} xmlns="http://www.w3.org/2000/svg" width="24" height="25" viewBox="0 0 22 23">
      <g fill="#666">
        <path d="M11.6875,0 C12.0671958,0 12.375,0.302866199 12.375,0.676470588 L12.375,22.3235294 C12.375,22.6971338 12.0671958,23 11.6875,23 C11.3078042,23 11,22.6971338 11,22.3235294 L11,0.676470588 C11,0.302866199 11.3078042,0 11.6875,0 Z" />
        <path d="M11.6875 10.8235294L21.3125 10.8235294C21.6921958 10.8235294 22 11.1263956 22 11.5 22 11.8736044 21.6921958 12.1764706 21.3125 12.1764706L11.6875 12.1764706C11.3078042 12.1764706 11 11.8736044 11 11.5 11 11.1263956 11.3078042 10.8235294 11.6875 10.8235294zM.6875 10.8235294L8.9375 10.8235294C9.31719577 10.8235294 9.625 11.1263956 9.625 11.5 9.625 11.8736044 9.31719577 12.1764706 8.9375 12.1764706L.6875 12.1764706C.307804234 12.1764706 0 11.8736044 0 11.5 0 11.1263956.307804234 10.8235294.6875 10.8235294z" />
      </g>
    </svg>
  );
}

class EnrollQuarter extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // boolean 页面加载状态
      list: [], //季度列表
      showModal: false, //模态框
      editId: null, //当前编辑的卡片Id
      quarterId: null,
      productTypeId: null,
      yearId: null,
    };
  }

  componentDidMount() {
    ShowAlert('只有设置报读季度的课程才会在H5显示');
    this.initContent();
  }

  /**
   * 更新状态
   * @param  {object} changedState 改变的状态
   * @return {Promise}
   */
  updateState = (changedState) => {
    const newState = {
      ...this.state,
      ...changedState,
    };
    return new Promise((resolve) => {
      this.setState(newState, () => {
        resolve();
      });
    });
  };

  /**
   * 获取列表数据
   */
  loadListData = async () => {
    try {
      return await Loader.loadQuarters();
    } catch (e) {
      ShowMessage(e.message);
      return [];
    }
  };

  /**
   * 初始化内容
   */
  initContent = async () => {
    this.setState({ loading: true });

    //获取列表数据
    const list = await this.loadListData();
    this.setState({
      list,
      loading: false,
    });
  };

  onAddCard = () => {
    this.setState({
      showModal: true,
      editId: null,
      quarterId: null,
      productTypeId: null,
      yearId: null,
    });
  };

  onDelCard = (id) => {
    ShowConfim('确认删除该报读季度？', async () => {
      this.setState({ loading: true });

      try {
        await Loader.delQuarter({ id });
        ShowMessage('删除成功', 'success');
        this.initContent();
      } catch (err) {
        ShowMessage(err.message);
        this.setState({ loading: false });
      }
    });
  };

  onOk = () => {
    this.setState({ showModal: false });
    this.initContent();
  };

  onCancel = () => {
    this.setState({ showModal: false });
  };

  render() {
    const { loading, list, showModal, editId } = this.state;
    return (
      <Page
        loading={loading}
        className={styles.customPage}
        name="报读季度设置 (只有设置报读季度的课程才会在H5显示)"
        route={this.props.route}
      >
        {list.map((card) => {
          return (
            <div className={styles.card} key={card.id}>
              <div className={styles.cardHeader}>
                <p className={styles.cardTitle}>{card.productTypeName}</p>
                <IconButton type="delete_mc" size={18} onClick={() => this.onDelCard(card.id)} />
              </div>
              <div className={styles.cardBody}>
                <p>产品年份:&nbsp;{card.yearName}</p>
                <p>产品季度:&nbsp;{card.quarterName}</p>
              </div>
            </div>
          );
        })}
        <div
          className={styles.card}
          style={{
            justifyContent: 'center',
            alignItems: 'center',
            cursor: 'pointer',
          }}
          onClick={this.onAddCard}
        >
          <AddIcon />
          {/*<IconButton type="plus" size={25} onClick={this.onAddCard} />*/}
        </div>

        {showModal && <EditModal onOk={this.onOk} onCancel={this.onCancel} editId={editId} />}
      </Page>
    );
  }
}

const mapStateToProp = (state) => {
  return {
    btnAuths: state.account.btnAuths,
  };
};

export default connect(mapStateToProp, null)(EnrollQuarter);
