/**
 * Created by Haolin on 2018/8/14.
 */
import React from 'react';
import { Row, Col } from 'antd';
import Block from 'COMPONENT/Block';
import FormItem from 'COMPONENT/FormItem';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import InputNumber from 'COMPONENT/InputNumber';
import Input from 'COMPONENT/Input';

class BaseInfo extends React.Component {
  constructor(props) {
    super(props);
  }

  render() {
    const {
      handleUpdate,
      error,
      checkValidator,
      handleValidator,
      data: { name, branch, campus, version, quarter, grade, subject, limitNum, deadline },
      dict: { branchDict, branchCampusDict, versionDict, quarterDict, gradeDict, subjectDict, deadlineDict },
      notEdit,
    } = this.props;

    return (
      <Block title="基础信息" className="exam-site-base-info">
        <Row>
          <Col span={10}>
            <FormItem name="考场名称" labelWidth={100} required>
              <Input
                placeholder="考场名称"
                value={name}
                maxLength={30}
                onChange={(e) => {
                  handleUpdate({ type: 'name', val: e.target.value });
                }}
                validator={{
                  rules: ['required'],
                  silent: !Boolean(error['name']) || checkValidator,
                  showStatus: true,
                  showTips: false,
                  onErrorChange: (e) => {
                    handleValidator({ type: 'name', val: e });
                  },
                }}
                disabled={notEdit}
              />
            </FormItem>
          </Col>
          <Col span={14}>
            <span style={{ lineHeight: '40px', marginLeft: 16 }}>提示：将用于预约分享页的标题</span>
          </Col>
        </Row>
        <Row>
          <Col span={10}>
            <FormItem name="分公司" labelWidth={100} required>
              <Select
                placeholder="请选择分公司"
                dict={branchDict}
                value={branch}
                showSearch={true}
                allowClear={false}
                onChange={(val) => handleUpdate({ type: 'branch', val: val })}
                validator={{
                  rules: ['required'],
                  silent: !Boolean(error['branch']) || checkValidator,
                  showStatus: true,
                  showTips: false,
                  onErrorChange: (e) => handleValidator({ type: 'branch', val: e }),
                }}
                disabled={notEdit}
              />
            </FormItem>
          </Col>
          <Col span={4}> </Col>
          <Col span={10}>
            <FormItem name="学习中心" labelWidth={100} required>
              <MultiSelect
                placeholder="请选择学习中心"
                dict={
                  branch ? branchCampusDict && branchCampusDict.filter((item) => item.val === branch) : branchCampusDict
                }
                value={campus}
                showSearch={true}
                allowClear={false}
                onChange={(val) => handleUpdate({ type: 'campus', val: val })}
                validator={{
                  rules: ['required'],
                  silent: !Boolean(error['campus']) || checkValidator,
                  showStatus: true,
                  showTips: false,
                  onErrorChange: (e) => handleValidator({ type: 'campus', val: e }),
                }}
                disabled={notEdit}
              />
            </FormItem>
          </Col>
        </Row>
        <Row>
          <Col span={10}>
            <FormItem name="年份" labelWidth={100} required>
              <Select
                placeholder="请选择年份"
                dict={versionDict}
                value={version}
                onChange={(val) => handleUpdate({ type: 'version', val: val })}
                validator={{
                  rules: ['required'],
                  silent: !Boolean(error['version']) || checkValidator,
                  showStatus: true,
                  showTips: false,
                  onErrorChange: (e) => handleValidator({ type: 'version', val: e }),
                }}
                disabled={notEdit}
              />
            </FormItem>
          </Col>
          <Col span={4}> </Col>
          <Col span={10}>
            <FormItem name="季度" labelWidth={100} required>
              <Select
                placeholder="请选择季度"
                dict={quarterDict}
                value={quarter}
                onChange={(val) => handleUpdate({ type: 'quarter', val: val })}
                validator={{
                  rules: ['required'],
                  silent: !Boolean(error['quarter']) || checkValidator,
                  showStatus: true,
                  showTips: false,
                  onErrorChange: (e) => handleValidator({ type: 'quarter', val: e }),
                }}
                disabled={notEdit}
              />
            </FormItem>
          </Col>
        </Row>
        <Row>
          <Col span={10}>
            <FormItem name="年级" labelWidth={100} required>
              <MultiSelect
                placeholder="请选择年级"
                dict={gradeDict}
                value={grade}
                showSearch={true}
                onChange={(val) => handleUpdate({ type: 'grade', val: val })}
                validator={{
                  rules: ['required'],
                  silent: !Boolean(error['grade']) || checkValidator,
                  showStatus: true,
                  showTips: false,
                  onErrorChange: (e) => handleValidator({ type: 'grade', val: e }),
                }}
                disabled={notEdit}
              />
            </FormItem>
          </Col>
          <Col span={4}> </Col>
          <Col span={10}>
            <FormItem name="科目" labelWidth={100}>
              <MultiSelect
                placeholder="请选择科目"
                dict={subjectDict}
                value={subject}
                showSearch={true}
                onChange={(val) => handleUpdate({ type: 'subject', val: val })}
                // validator={{
                //   rules: ['required'],
                //   silent: !Boolean(error["subject"])||checkValidator,
                //   showStatus:true,
                //   showTips:false,
                //   onErrorChange: e => handleValidator({type:"subject",val:e}),
                // }}
                disabled={notEdit}
              />
            </FormItem>
          </Col>
        </Row>
        <Row>
          <Col span={10}>
            <FormItem name="人数限制" labelWidth={100} required>
              <InputNumber
                placeholder="请输入人数限制"
                value={limitNum}
                max={100000000}
                min={0}
                onChange={(val) => handleUpdate({ type: 'limitNum', val })}
                validator={{
                  rules: ['required'],
                  silent: !Boolean(error['limitNum']) || checkValidator,
                  showStatus: true,
                  showTips: false,
                  onErrorChange: (e) => handleValidator({ type: 'limitNum', val: e }),
                }}
                disabled={notEdit}
              />
            </FormItem>
          </Col>
          <Col span={4}> </Col>
          <Col span={10}>
            <FormItem name="截止日期" labelWidth={100} required>
              <Select
                placeholder="请选择截止日期"
                dict={deadlineDict}
                value={deadline}
                onChange={(val) => handleUpdate({ type: 'deadline', val })}
                validator={{
                  rules: ['required'],
                  silent: !Boolean(error['deadline']) || checkValidator,
                  showStatus: true,
                  showTips: false,
                  onErrorChange: (e) => handleValidator({ type: 'deadline', val: e }),
                }}
                disabled={notEdit}
              />
            </FormItem>
          </Col>
        </Row>
      </Block>
    );
  }
}

export default BaseInfo;
