/**
 * Created by Haolin on 2018/8/14.
 */
import React from 'react';
import { Row, Col } from 'antd';
import Block from 'COMPONENT/Block';
import TimePicker from 'COMPONENT/TimePicker';
import MultiSelect from 'COMPONENT/MultiSelect';
import Calendar from '../Cal';
import style from './style.less';
import { sortedUniq } from 'lodash';

const buildTimeToShow = (params) => {
  let paramsStr = params.toString();
  let y = paramsStr.slice(0, 4);
  let m = paramsStr.slice(4, 6);
  let d = paramsStr.slice(6, 8);
  return `${y}-${m}-${d}`;
};
const SetTimeRow = (props) => {
  const {
    data: { beginTime, endTime, activeDay, activeDayStr, index, notChange, subjectId },
    handleUpdate,
    onChange,
    disabledHours,
    disabledMinutes,
    showDate,
    dict: { subjectDict },
  } = props;
  return (
    <Row className={style['set-time-row']}>
      <Col span={3}>{showDate && <div className={style['set-time-row-label']}>{activeDayStr}</div>}</Col>
      <Col span={6}>
        <MultiSelect
          className={style['set-time-row-picker']}
          placeholder="请选择科目"
          dict={subjectDict}
          value={subjectId}
          onChange={(val) => {
            handleUpdate({ actType: 'set', type: 'subjectId', now: activeDay, val, index });
          }}
          showSearch={false}
          disabled={notChange}
        />
      </Col>
      <Col span={6}>
        <TimePicker
          className={style['set-time-row-picker']}
          placeholder="请选择开始时间"
          value={beginTime}
          onChange={(val) =>
            onChange(
              { actType: 'set', type: 'beginTime', now: activeDay, val, index, restrictData: endTime },
              handleUpdate,
            )
          }
          disabledHours={() => disabledHours({ type: 'beginTime', val: beginTime, otherVal: endTime })}
          disabledMinutes={(params) =>
            disabledMinutes({ type: 'beginTime', val: beginTime, otherVal: endTime, thisHour: params })
          }
          disabled={notChange}
        />
      </Col>
      <Col span={6}>
        <TimePicker
          className={style['set-time-row-picker']}
          placeholder="请选择结束时间"
          value={endTime}
          onChange={(val) =>
            onChange(
              { actType: 'set', type: 'endTime', now: activeDay, val, index, restrictData: beginTime },
              handleUpdate,
            )
          }
          disabledHours={() => disabledHours({ type: 'endTime', val: endTime, otherVal: beginTime })}
          disabledMinutes={(params) =>
            disabledMinutes({ type: 'endTime', val: endTime, otherVal: beginTime, thisHour: params })
          }
          disabled={notChange}
        />
      </Col>
      <Col span={3}>
        <div className={style['set-time-row-btns']}>
          <span
            className={style['set-time-row-add']}
            onClick={() => {
              handleUpdate({ actType: 'add', now: activeDay, index });
            }}
          >
            新增
          </span>
          {!notChange && (
            <span
              className={style['set-time-row-delete']}
              onClick={() => {
                handleUpdate({ actType: 'delete', now: activeDay, index });
              }}
            >
              删除
            </span>
          )}
        </div>
      </Col>
    </Row>
  );
};
const SetTimeRowTree = (props) => {
  const { source, handleUpdate, dict, onChange, disabledHours, disabledMinutes } = props;
  const tree = sortedUniq(Object.keys(source).map((item) => Number(item)));
  return (
    <div>
      {tree.map((originItem) => {
        return source[originItem].map((item, index) => {
          const { beginTime, endTime, notChange, subjectId } = item;
          let params = {
            activeDay: originItem.toString(),
            activeDayStr: buildTimeToShow(originItem),
            beginTime,
            endTime,
            subjectId,
            notChange,
            index,
          };
          return (
            <SetTimeRow
              dict={dict}
              data={params}
              handleUpdate={handleUpdate}
              onChange={onChange}
              disabledHours={disabledHours}
              disabledMinutes={disabledMinutes}
              showDate={index === 0}
              key={index}
            />
          );
        });
      })}
    </div>
  );
};

class SetTime extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      activeDay: null, //日历当前已选日期 type:"save"
      activeDayStr: null,
    };
  }

  /**
   * 更新state
   * */
  updateState = (newState) => {
    this.setState((prevState, props) => {
      return {
        ...newState,
      };
    });
  };
  /**
   * 更新日历当前已选日期
   * */
  updateActive = (params) => {
    const { type, activeDay, activeDayStr } = params;
    const { handleTime } = this.props;
    this.updateState({ activeDay, activeDayStr });
    if (type === 'new') {
      //添加默认时间
      handleTime({ actType: 'new', now: activeDay });
    }
  };
  /**
   * 更新时间并辅助限制时间输入
   * 与disabledHours、disabledMinutes逻辑上有耦合，暂时先这样
   * */
  updateTime = (params, callback) => {
    const { type, val, restrictData } = params;
    let result = val;
    let other = restrictData;
    if (other && val) {
      let tagTime = val.split(':');
      let otherTagTime = other.split(':');
      if (tagTime[0] === otherTagTime[0]) {
        let tagMinNum = Number(tagTime[1]);
        let otherTagMinNum = Number(otherTagTime[1]);
        if (type === 'beginTime') {
          if (tagMinNum >= otherTagMinNum) {
            result = this.buildTime(Number(tagTime[0]), otherTagMinNum - 1);
          }
        } else {
          if (tagMinNum <= otherTagMinNum) {
            result = this.buildTime(Number(tagTime[0]), otherTagMinNum + 1);
          }
        }
      }
    }
    let newParams = { ...params, val: result };
    callback(newParams);
  };
  /**
   * 构造时间字符串
   * */
  buildTime = (h, m) => {
    h = h < 10 ? '0' + h.toString() : h.toString();
    m = m < 10 ? '0' + m.toString() : m.toString();
    return h + ':' + m;
  };
  /**
   * 限制小时输入
   * */
  restrictHours = (params) => {
    const { type, otherVal } = params;
    let result = [];
    if (otherVal) {
      let otherArr = otherVal.split(':');
      let otherHour = Number(otherArr[0]);
      if (type === 'beginTime') {
        if (otherArr[1] === '00') {
          --otherHour;
        }
        for (let i = 23; i > otherHour; i--) {
          result.push(i);
        }
      } else {
        if (otherArr[1] === '59') {
          ++otherHour;
        }
        for (let i = 0; i < otherHour; i++) {
          result.push(i);
        }
      }
    }
    return result;
  };
  /**
   * 限制分钟输入
   * */
  restrictMinutes = (params) => {
    const { type, otherVal, thisHour } = params;
    let result = [];
    if (otherVal) {
      let otherArr = otherVal.split(':');
      let otherHour = Number(otherArr[0]);
      if (otherHour === thisHour) {
        let otherMinute = Number(otherArr[1]);
        if (type === 'beginTime') {
          for (let i = 59; i >= otherMinute; i--) {
            result.push(i);
          }
        } else {
          for (let i = 0; i <= otherMinute; i++) {
            result.push(i);
          }
        }
      }
    }
    return result;
  };
  /**
   * 统计已选预约时间段
   * */
  countChosen = (params) => {
    let keys = Object.keys(params);
    let result = 0;
    keys.forEach((item) => {
      result += params[item].length;
    });
    return result;
  };

  render() {
    const { date, time, defaultTime, handleDate, handleTime, handleDefault, handleTimeTemp, dict } = this.props;
    const tempLen = defaultTime.length;
    const count = this.countChosen(time);
    const checkSelectTime = Object.keys(time).length > 0;

    return (
      <Block title="设置考试时间" className="exam-site-set-time">
        <div className={style['exam-site-set-block']}>
          <div className={style['set-time-temp']}>
            <div className={style['set-time-temp-title']}>请设置默认的考试时间段</div>
            {defaultTime.map((item, index) => {
              const { beginTime, endTime } = item;
              return (
                <Row key={index} className={style['set-time-temp-row']}>
                  <Col span={6}>
                    <TimePicker
                      placeholder="请选择开始时间"
                      value={beginTime}
                      onChange={(val) =>
                        this.updateTime(
                          { actType: 'update', type: 'beginTime', val, index, restrictData: endTime },
                          handleDefault,
                        )
                      }
                      disabledHours={() => this.restrictHours({ type: 'beginTime', val: beginTime, otherVal: endTime })}
                      disabledMinutes={(params) =>
                        this.restrictMinutes({ type: 'beginTime', val: beginTime, otherVal: endTime, thisHour: params })
                      }
                    />
                  </Col>
                  <Col span={1}>
                    <div className={style['set-time-temp-text']}>至</div>
                  </Col>
                  <Col span={6}>
                    <TimePicker
                      placeholder="请选择结束时间"
                      value={endTime}
                      onChange={(val) =>
                        this.updateTime(
                          { actType: 'update', type: 'endTime', val, index, restrictData: beginTime },
                          handleDefault,
                        )
                      }
                      disabledHours={() => this.restrictHours({ type: 'endTime', val: endTime, otherVal: beginTime })}
                      disabledMinutes={(params) =>
                        this.restrictMinutes({ type: 'endTime', val: endTime, otherVal: beginTime, thisHour: params })
                      }
                    />
                  </Col>
                  <Col span={3}>
                    <div className={style['set-time-row-btns']}>
                      <span
                        className={style['set-time-row-add']}
                        onClick={() => {
                          handleTimeTemp({ actType: 'add', index });
                        }}
                      >
                        新增
                      </span>
                      {tempLen > 1 && (
                        <span
                          className={style['set-time-row-delete']}
                          onClick={() => {
                            handleTimeTemp({ actType: 'delete', index });
                          }}
                        >
                          删除
                        </span>
                      )}
                    </div>
                  </Col>
                </Row>
              );
            })}
          </div>
          <Row>
            <Col span={24} style={{ position: 'relative' }}>
              <Calendar date={date} handleActive={this.updateActive} handleDate={handleDate} />
            </Col>
          </Row>
        </div>
        <Row>
          <Col span={24}>
            <div className={style['set-time-count']}>
              <span className={style['set-time-count-label']}>已选时间段： </span>
              <span className={style['set-time-count-num']}>{count}</span>
            </div>
          </Col>
        </Row>
        {checkSelectTime && (
          <Row className={style['set-time-row-header']}>
            <Col span={3} className={style['set-time-row-header-text']}>
              <div>日期</div>
            </Col>
            <Col span={6} className={style['set-time-row-header-text']}>
              <div>科目</div>
            </Col>
            <Col span={6} className={style['set-time-row-header-text']}>
              <div>开始时间</div>
            </Col>
            <Col span={6} className={style['set-time-row-header-text']}>
              <div>结束时间</div>
            </Col>
            <Col span={3} className={style['set-time-row-header-text']}>
              <div>操作</div>
            </Col>
          </Row>
        )}
        {checkSelectTime && (
          <SetTimeRowTree
            source={time}
            dict={dict}
            handleUpdate={handleTime}
            onChange={this.updateTime}
            disabledHours={this.restrictHours}
            disabledMinutes={this.restrictMinutes}
          />
        )}
      </Block>
    );
  }
}

export default SetTime;
