/**
 * Created by Haolin on 2018/8/14.
 */
import React from 'react';
import Modal from 'COMPONENT/Modal';
import Button from 'COMPONENT/Button';
import PopTip from 'COMPONENT/PopTip';
import BaseInfo from './BaseInfo';
import SetTime from './SetTime';
import './style.less';
import { cloneDeep } from 'lodash';
import { addExamSite, deleteExamRoomTime } from 'SERVICE/signUpOnline';
import { getDictTextByVal } from 'UTIL/dict';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

const ModalFooterBtn = (props) => {
  const { content, popTipVisible, onClose, onOk, onCancel, okText } = props;
  return (
    <div style={{ display: 'flex', justifyContent: 'flex-end' }}>
      <Button onClick={onCancel}>取消</Button>
      <PopTip icon="error" content={content} visible={popTipVisible} onClose={onClose} maxWidth={200}>
        <Button type="primary" onClick={onOk}>
          {okText ? okText : '确定'}
        </Button>
      </PopTip>
    </div>
  );
};

class ExamSiteModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false,
      error: null,
      errors: {},
      checkValidator: true,
      toggle_btnTip: false,
      btnTipContent: '',
      //基础信息
      nameVal: null,
      branchVal: null,
      campusVal: null,
      versionVal: null,
      quarterVal: null,
      gradeVal: null,
      subjectVal: null,
      examSiteId: null,
      limitNumVal: null,
      deadlineVal: null,
      //设置时间
      resultDate: [], //保存用于提交的日期格式
      chosenDate: [], //保存用于查询chosenTime的日期格式
      chosenTime: {}, //保存已选的时间
      examTime: [
        {
          beginTime: null,
          endTime: null,
          subjectId: null,
        },
      ], //设置时间，已为后面“批量设置开始结束时间”做好准备
      alreadySignUp: false, //是否有学生报名
      deleteExamSiteTimeArr: [], //保存编辑模式时，删除的预约时间id，待提交时再提交删除请求
    };
  }
  componentWillMount() {
    // 临时限制-截至日期(BaseInfo中需更改相应的disabled)
    this.setState({ deadlineVal: '1' });
    this.init();
  }

  /**
   * 初始化
   * */
  init = () => {
    const {
      type,
      initData,
      dict: { subjectDict },
    } = this.props;
    if (type === 'edit') {
      // console.log("初始化*修改考场*",initData);
      const {
        id,
        name,
        branchId,
        campusId,
        versionId,
        quarterId,
        gradeId,
        subjectId,
        limitNum,
        deadlineVal,
        beginTime,
        endTime,
        peopleCount,
        examRoomTime,
      } = initData;
      let idArr = examRoomTime.map((item) => item.id);
      let apptData = this.buildOriginData({ examRoomTime, idArr });
      let examTime = this.setDefaultSubject({
        origin: this.buildData({ type: 'examTime', val: { beginArr: beginTime, endArr: endTime } }),
        val: subjectId,
      });
      this.updateState({
        nameVal: name,
        branchVal: branchId,
        campusVal: campusId,
        versionVal: versionId,
        quarterVal: quarterId,
        gradeVal: gradeId,
        subjectVal: subjectId,
        limitNumVal: limitNum,
        deadlineVal,
        examSiteId: id,
        resultDate: this.buildData({ type: 'resultDate', val: apptData }),
        chosenDate: this.buildData({ type: 'chosenDate', val: apptData }),
        chosenTime: this.buildData({ type: 'chosenTime', val: apptData }),
        examTime,
        alreadySignUp: !peopleCount.every((item) => item === 0),
      });
    }
  };
  /**
   * 用于辅助初始化预约时间数据
   * */
  buildData = (params) => {
    const { type, val } = params;
    let result;
    switch (type) {
      case 'resultDate':
      case 'chosenDate':
        let dateArr = val.map((item) => item[type]);
        result = Array.from(new Set(dateArr));
        break;
      case 'chosenTime':
        result = {};
        val.forEach((item) => {
          const { chosenDate, chosenTime } = item;
          if (!result[chosenDate]) {
            result[chosenDate] = [];
          }
          result[chosenDate].push(chosenTime);
        });
        break;
      case 'examTime':
        const { beginArr, endArr } = val;
        let beginLen = beginArr.length;
        let endLen = endArr.length;
        if (beginLen > 0 || endLen > 0) {
          let len = beginLen === endLen ? (beginLen > endLen ? beginLen : endLen) : beginLen;
          result = [];
          for (let i = 0; i < len; i++) {
            result.push({
              beginTime: beginArr[i] ? beginArr[i] : null,
              endTime: endArr[i] ? endArr[i] : null,
            });
          }
        } else {
          result = [
            {
              beginTime: null,
              endTime: null,
            },
          ];
        }
        break;
      default:
        result = null;
        console.warn('# error from buildData,undefined type! #');
    }
    return result;
  };
  buildOriginData = (params) => {
    const { examRoomTime, idArr } = params;
    return examRoomTime.map((item, index) => {
      const { appointmentTime, number, subjectId } = item;
      let itemArr = appointmentTime.split(' ');
      let timeArr = itemArr[1].split('-');
      return {
        resultDate: itemArr[0],
        chosenDate: this.buildSaveDate(new Date(itemArr[0])),
        chosenTime: {
          beginTime: timeArr[0],
          endTime: timeArr[1],
          notChange: number > 0,
          id: idArr[index],
          subjectId: subjectId ? subjectId.split(',') : null,
        },
      };
    });
  };
  buildSaveDate = (date) => {
    let y = date.getFullYear();
    let m = date.getMonth() + 1;
    let d = date.getDate();
    m = m < 10 ? '0' + m : m.toString();
    d = d < 10 ? '0' + d : d.toString();
    return y.toString() + m + d;
  };

  /**
   * 更新state
   * */
  updateState = (newState) => {
    this.setState((prevState, props) => {
      return {
        ...newState,
      };
    });
  };
  /**
   * 更新基础信息数据
   * */
  updateBaseInfo = (params) => {
    const { type, val } = params;
    // console.log(type, val);
    if (type === 'subject') {
      const { examTime } = this.state;
      this.setState({ examTime: this.setDefaultSubject({ origin: examTime, val }) });
    }
    this.updateState({ [`${type}Val`]: val });
  };
  setDefaultSubject = (params) => {
    const { origin, val } = params;
    return cloneDeep(origin).map((item) => {
      item.subjectId = val;
      return item;
    });
  };
  /**
   * 设置时间-更新默认开始/结束时间
   * */
  updateDefaultTime = (params) => {
    const { actType, type, val, index } = params;
    const { examTime } = this.state;
    let timeList = examTime.slice(0);
    switch (actType) {
      case 'update':
        timeList[index][type] = val;
        break;
      default:
        console.log('** 缺少updateDefault类型 **');
    }
    this.updateState({ examTime: timeList });
  };
  /**
   * 设置时间-更新已选日期
   * */
  updateDate = (params) => {
    const { type, chosen, result } = params;
    const { resultDate, chosenDate } = this.state;
    if (type === 'new') {
      let resultList = resultDate.slice();
      let chosenList = chosenDate.slice();
      resultList.push(result);
      chosenList.push(chosen);
      this.updateState({
        resultDate: resultList,
        chosenDate: chosenList,
      });
    }
  };
  /**
   * 设置时间-更新已选时间
   * */
  updateTime = (params) => {
    const { actType, type, val, index, now } = params;
    const { chosenTime, resultDate, chosenDate, examTime, deleteExamSiteTimeArr } = this.state;
    let cloneObj = cloneDeep(chosenTime);
    switch (actType) {
      case 'new':
        cloneObj[now] = cloneDeep(examTime);
        break;
      case 'add':
        cloneObj[now].splice(index + 1, 0, {
          beginTime: null,
          endTime: null,
          subjectId: cloneDeep(this.state.subjectVal),
        });
        break;
      case 'delete':
        let delObj = cloneObj[now].splice(index, 1);
        let nowId = delObj[0].id;
        if (this.props.type === 'edit' && nowId) {
          let deleteObj = deleteExamSiteTimeArr.slice(0);
          deleteObj.push({
            timeId: nowId,
            appointmentTime: `${this.buildTimeToShow(now)} ${delObj[0].beginTime}-${delObj[0].endTime}`,
            subjectId: delObj[0].subjectId.join(','),
          });
          this.setState({ deleteExamSiteTimeArr: deleteObj });
          // this.deleteExamSiteTime({timeId:nowId});
        }
        //校验长度，当为0时，取消该日期的选中状态
        if (cloneObj[now].length === 0) {
          let result = resultDate.slice(0);
          let chosen = chosenDate.slice(0);
          let position = chosen.indexOf(now);
          if (position !== -1) {
            chosen.splice(position, 1);
            result.splice(position, 1);
          }
          this.setState({
            resultDate: result,
            chosenDate: chosen,
          });
        }
        break;
      case 'set':
        cloneObj[now][index][type] = val;
        break;
    }
    this.updateState({
      chosenTime: cloneObj,
    });
  };
  /**
   * 设置时间模板
   * */
  updateTimeTemp = (params) => {
    const { actType, index } = params;
    const { examTime } = this.state;
    let cloneObj = cloneDeep(examTime);
    switch (actType) {
      case 'add':
        cloneObj.splice(index + 1, 0, {
          beginTime: null,
          endTime: null,
          subjectId: cloneDeep(this.state.subjectVal),
        });
        break;
      case 'delete':
        cloneObj.splice(index, 1);
        break;
    }
    this.updateState({ examTime: cloneObj });
  };
  /**
   * 删除考场时间
   * */
  deleteExamSiteTime = async (params, option) => {
    await this.updateState({ loading: true });
    const { submit } = option;
    deleteExamRoomTime(params, true).then(
      (data) => {
        if (!submit) {
          showMessage('删除成功！', 'success');
          this.updateState({ loading: false });
        }
      },
      (err) => {
        if (!submit) {
          this.updateState({ loading: false });
        }
        this.updateState({ loading: false });
        showMessage(err.text, 'error');
      },
    );
  };
  deleteWhenSubmit = async (params, origin) => {
    // 先校验是否有删除后又新增一样的数据并将其过滤后再请求删除接口
    let delList = [];
    params.forEach((item) => {
      let noNeedDel = origin.some(
        (originItem) => originItem.appointmentTime === item.appointmentTime && originItem.subjectId === item.subjectId,
      );
      if (!noNeedDel) {
        delList.push(item);
      }
    });
    // console.log('examRoomTime: ', delList);
    delList = delList.map((item) => ({ timeId: item.timeId }));
    await delList.forEach((item) => {
      this.deleteExamSiteTime(item, { submit: true });
    });
  };
  /**
   * 用于校验表单是否存在错误
   * */
  errorChange = (params) => {
    const { type, val } = params;
    // console.log(type,val,index);
    this.setState((prevState) => {
      let err = cloneDeep(prevState.errors);
      err[type] = val;
      return { errors: err };
    });
  };
  /**
   * 校验数据
   * */
  checkBeforeSubmit = () => {
    const { errors, resultDate, chosenDate, chosenTime } = this.state;
    let checkApptObj = {
      checkEmpty: true,
      errEmpty: [],
      checkRepeat: true,
      errRepeat: [],
      checkSubject: true,
      errSubject: [],
    };
    let tipMsg = [];
    let checkRequired = this.checkRequired(errors);
    if (!checkRequired) {
      tipMsg.push('您有信息未填写 ');
    }
    let hadChosen = resultDate.length === 0;
    if (hadChosen) {
      tipMsg.push('请选择考场预约日期 ');
    } else {
      checkApptObj = this.checkApptTime(chosenTime);
      if (!checkApptObj.checkSubject) {
        let errStr = checkApptObj.errSubject.reduce((addUp, current) => addUp + '，' + current);
        tipMsg.push(errStr + ' 请选择预约考试科目 ');
      }
      if (!checkApptObj.checkEmpty) {
        let errStr = checkApptObj.errEmpty.reduce((addUp, current) => addUp + '，' + current);
        tipMsg.push(errStr + ' 请选择考场预约时间 ');
      }
      if (!checkApptObj.checkRepeat) {
        let errStr = checkApptObj.errRepeat.reduce((addUp, current) => addUp + '，' + current);
        tipMsg.push(errStr + ' 预约时间有冲突，请重选 ');
      }
    }
    this.updateState({ checkValidator: checkRequired });
    let check =
      checkRequired && !hadChosen && checkApptObj.checkSubject && checkApptObj.checkEmpty && checkApptObj.checkRepeat;
    if (!check) {
      this.updateState({
        btnTipContent: (
          <ul style={{ listStyle: 'inside' }}>
            {tipMsg.map((item, index) => (
              <li key={index}>{item}</li>
            ))}
          </ul>
        ),
        toggle_btnTip: true,
      });
      console.log('校验不通过');
    }
    return check;
  };
  checkRequired = (params) => {
    return Object.values(params).every((item) => {
      return !Boolean(item);
    });
  };
  checkApptTime = (params) => {
    let result = {
      checkEmpty: true,
      errEmpty: [],
      checkRepeat: true,
      errRepeat: [],
      checkSubject: true,
      errSubject: [],
    };
    let arr = Object.values(params);
    let keys = Object.keys(params);
    arr.forEach((item, index) => {
      //校验是否有空值
      item.forEach((_item) => {
        // 已有学生报名的不可修改，故不进行校验
        if (!_item.notChange) {
          const { beginTime, endTime, subjectId } = _item;
          if (!subjectId || subjectId.length === 0) {
            result.checkSubject = false;
            result.errSubject.push(keys[index]);
          }
          if (!beginTime || !endTime || beginTime === '' || endTime === '') {
            result.checkEmpty = false;
            result.errEmpty.push(keys[index]);
          }
        }
      });
      //校验是否有重复值(科目/开始时间/结束时间)
      let list = item.map((_item) => _item.subjectId + _item.beginTime + _item.endTime);
      // console.log(list);
      let arrLen = list.length;
      let setSize = new Set(list).size;
      if (arrLen !== setSize) {
        result.checkRepeat = false;
        result.errRepeat.push(keys[index]);
      }
    });
    if (!result.checkSubject) {
      result.errSubject = Array.from(new Set(result.errSubject)).map((item) => this.buildTimeToShow(item));
    }
    if (!result.checkEmpty) {
      result.errEmpty = Array.from(new Set(result.errEmpty)).map((item) => this.buildTimeToShow(item));
    }
    if (!result.checkRepeat) {
      result.errRepeat = Array.from(new Set(result.errRepeat)).map((item) => this.buildTimeToShow(item));
    }
    return result;
  };
  buildTimeToShow = (params) => {
    let paramsStr = params.toString();
    let y = paramsStr.slice(0, 4);
    let m = paramsStr.slice(4, 6);
    let d = paramsStr.slice(6, 8);
    return `${y}-${m}-${d}`;
  };
  /**
   * 保存
   * */
  submit = async () => {
    //校验
    let checkVal = this.checkBeforeSubmit();
    if (checkVal) {
      await this.updateState({ loading: true });
      const {
        type,
        initData,
        dict: { branchCampusDict, gradeDict },
        handleSubmit,
      } = this.props;
      const {
        nameVal,
        branchVal,
        campusVal,
        versionVal,
        quarterVal,
        gradeVal,
        limitNumVal,
        deadlineVal,
        resultDate,
        chosenDate,
        chosenTime,
        examTime,
        examSiteId,
        deleteExamSiteTimeArr,
      } = this.state;
      const subject = this.buildParams({ type: 'subject', val: chosenTime });
      let apiParams = {
        name: nameVal,
        branchId: branchVal,
        blCampusId: this.buildParams({ type: 'arrStr', val: campusVal }),
        blCampusName: this.buildParams({
          type: 'arrStr',
          val: campusVal.map((item) =>
            getDictTextByVal(branchCampusDict.filter((item) => item.val === branchVal)[0].options, item),
          ),
        }),
        yearId: versionVal,
        quarterId: quarterVal,
        gradeId: this.buildParams({ type: 'arrStr', val: gradeVal }),
        gradeName: this.buildParams({ type: 'arrStr', val: gradeVal.map((item) => getDictTextByVal(gradeDict, item)) }),
        subjectId: subject.map((item) => item.subjectId).join(','),
        subjectName: subject.map((item) => item.subjectName).join(','),
        examRoomTime: this.buildParams({ type: 'examRoomTime', val: { resultDate, chosenDate, chosenTime } }),
        startTime: this.buildParams({ type: 'arrStr', val: examTime.map((item) => item.beginTime) }),
        endTime: this.buildParams({ type: 'arrStr', val: examTime.map((item) => item.endTime) }),
        limitNumber: limitNumVal,
        deadline: deadlineVal === 'null' ? null : Number(deadlineVal),
      };
      // 过滤examRoomTime数据，只提交有变动的部分
      if (type === 'edit') {
        if (deleteExamSiteTimeArr.length > 0) {
          // TODO: 考场修改时删除考试时间处理方式待优化
          await this.deleteWhenSubmit(deleteExamSiteTimeArr, apiParams.examRoomTime);
        }
        apiParams = {
          ...apiParams,
          id: examSiteId,
          examRoomTime: this.filterEditData(apiParams.examRoomTime, initData.examRoomTime),
        };
      }
      // console.log('apiParams: ', apiParams);
      // return;
      addExamSite(apiParams, true).then(
        (data) => {
          this.updateState({ loading: false });
          showMessage(type === 'add' ? '新增考场成功！' : '修改考场成功！', 'success');
          handleSubmit();
        },
        (err) => {
          this.updateState({ loading: false });
          showMessage(err.text, 'error');
        },
      );
    }
  };
  /**
   * 构造参数-用于submit
   * */
  buildParams = (params) => {
    const { type, val } = params;
    const subjectDict = this.props.dict.subjectDict;
    let result;
    switch (type) {
      case 'examRoomTime':
        let { resultDate, chosenDate, chosenTime } = val;
        let arr = chosenDate.map((item, index) => {
          return chosenTime[item].map((item) => {
            let { id, beginTime, endTime, subjectId } = item;
            return {
              id: id ? id : null,
              appointmentTime: `${resultDate[index]} ${beginTime}-${endTime}`,
              subjectId: subjectId && subjectId.length > 0 ? subjectId.join(',') : null,
              subjectName:
                subjectId && subjectId.length > 0
                  ? subjectId.map((item) => getDictTextByVal(subjectDict, item)).join(',')
                  : null,
            };
          });
        });
        result = arr.reduce((addUp, current) => addUp.concat(current));
        if (this.props.type === 'edit') {
          let examObj = this.props.initData.examRoomTime;
          result.forEach((item) => {
            let examVal = examObj.find((examItem) => examItem.id === item.id);
            if (examVal) {
              item.examId = examVal.examId;
              item.id = examVal.id;
            } else {
              delete item.id;
            }
          });
        }
        break;
      case 'subject':
        const subjectTree = Object.keys(val).map((item) => val[item]);
        const subjectArr = subjectTree.reduce((addUp, current) => addUp.concat(current));
        const subjectIdArr = subjectArr.map((item) => item.subjectId || []);
        const subjectSetArr = Array.from(new Set(subjectIdArr.reduce((addUp, current) => addUp.concat(current))));
        result = subjectSetArr.map((item) => ({
          subjectId: item,
          subjectName: getDictTextByVal(subjectDict, item),
        }));
        break;
      case 'arrStr':
        result = val.length > 1 ? val.join(',') : val[0];
        break;
      default:
        result = null;
    }
    return result;
  };
  /**
   * 当是edit模式时过滤数据，只返回新增与修改的数据
   * */
  filterEditData = (params, origin) => {
    const originApptTime = origin.map((item) => item.appointmentTime);
    let result = [];
    let paramsObj = {};
    Object.keys(params).forEach((item) => {
      paramsObj[params[item].appointmentTime] = params[item];
    });
    originApptTime.forEach((item, index) => {
      if (paramsObj[item]) {
        if (origin[index].subjectId === paramsObj[item].subjectId) {
          delete paramsObj[item];
        }
      }
    });
    const paramsArr = Object.values(paramsObj);
    if (paramsArr.length > 0) {
      result = paramsArr;
    }
    // console.log('filterEditData: ', paramsArr);
    return result;
  };

  render() {
    const { type, handleCancel, dict } = this.props;
    const {
      loading,
      error,
      errors,
      checkValidator,
      branchVal,
      campusVal,
      versionVal,
      quarterVal,
      gradeVal,
      subjectVal,
      limitNumVal,
      deadlineVal,
      chosenDate,
      chosenTime,
      examTime,
      nameVal,
      toggle_btnTip,
      btnTipContent,
      alreadySignUp,
    } = this.state;

    const baseData = {
      name: nameVal,
      branch: branchVal,
      campus: campusVal,
      version: versionVal,
      quarter: quarterVal,
      grade: gradeVal,
      subject: subjectVal,
      limitNum: limitNumVal,
      deadline: deadlineVal,
    };

    return (
      <Modal
        title={type === 'add' ? '新增考场' : '修改考场'}
        className="exam-site-modal"
        loading={loading}
        error={error}
        width={1100}
        onCancel={() => handleCancel()}
        footer={
          <ModalFooterBtn
            content={btnTipContent}
            popTipVisible={toggle_btnTip}
            onClose={() => this.updateState({ toggle_btnTip: false })}
            okText="保存"
            onOk={() => this.submit()}
            onCancel={() => handleCancel()}
          />
        }
      >
        <BaseInfo
          handleUpdate={this.updateBaseInfo}
          notEdit={alreadySignUp}
          dict={dict}
          data={baseData}
          error={errors}
          checkValidator={checkValidator}
          handleValidator={this.errorChange}
        />
        <SetTime
          defaultTime={examTime}
          dict={dict}
          date={chosenDate}
          time={chosenTime}
          handleDefault={this.updateDefaultTime}
          handleDate={this.updateDate}
          handleTime={this.updateTime}
          handleTimeTemp={this.updateTimeTemp}
        />
      </Modal>
    );
  }
}

export default ExamSiteModal;
