/**
 * Created by Haolin on 2018/8/16.
 */
import React from 'react';
import { connect } from 'dva';
import { Row, Col } from 'antd';
import Modal from 'COMPONENT/Modal';
import Button from 'COMPONENT/Button';
import Input from 'COMPONENT/Input';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import './style.less';
import style from './compStyle.less';
import qrCodeBg from 'ASSET/imgs/exam_site_share_bg.png';
import QrCode from './QrCode';
import {
  uat,
  pre,
  prd,
  peiYouTest,
  peiYouPre,
  peiYou,
  twoTeacherTest,
  twoTeacherPre,
  twoTeacher,
} from 'CONST/buildEnv';
import html2canvas from 'html2canvas';

const SystemMark = {
  xinghuo: 'XINGHUO',
  advance: 'ADVANCE',
  two_teacher: 'TWO_TEACHER',
};
const ModalFooterBtn = (props) => {
  const { text, handleClick, disabled } = props;
  return (
    <div className={style['modal-footer-btn-wrap']}>
      <Button type="primary" className={style['modal-footer-btn']} onClick={handleClick} disabled={disabled}>
        {text}
      </Button>
    </div>
  );
};
const LinkContent = (props) => {
  const { url } = props;
  return (
    <div>
      <Row>
        <Col span={24}>
          <Input id="exam-share-link-input" defaultValue={url} />
        </Col>
      </Row>
      <Row>
        <Col span={24}>
          <div>注：家长可以通过访问链接进行考场预约。</div>
        </Col>
      </Row>
    </div>
  );
};
const QrCodeContent = (props) => {
  const { id, url } = props;
  return (
    <div id={id} className={style['exam-share-modal-snapshot']}>
      <img src={qrCodeBg} className={style['exam-share-modal-qr-bg']} />
      <div className={style['exam-share-modal-qr']}>
        <QrCode text={url} width={160} height={160} />
      </div>
    </div>
  );
};

class ShareModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false,
      error: null,
      shareLink: null,
      modalTitle: '分享-考场预约',
      btnText: '分享',
      snapshotWrapId: 'exam-share-modal-snapshot',
    };
    this.btnFunc = null;
  }
  componentWillMount() {
    this.init();
  }

  init = () => {
    const { type, system, data } = this.props;
    this.updateState({
      modalTitle: `分享-考场预约${this.setModalTitle(type)}`,
      btnText: this.setBtnText(type),
      shareLink: this.buildBaseUrl() + 'sys=' + SystemMark[system] + '&id=' + data,
    });
    this.btnFunc = this.setBtnFunc(type);
  };
  buildBaseUrl = () => {
    // 链接已切换-------------
    let result = 'https://cmall-test.xiao100.com/bd/exam_rsv?';
    if (process.env) {
      switch (process.env.BUILD_ENV) {
        case pre:
        case peiYouPre:
        case twoTeacherPre:
          // 链接已切换-------------

          result = 'https://cmall-pre.xiao100.com/bd/exam_rsv?';
          break;
        case prd:
        case peiYou:
        case twoTeacher:
          // 链接已切换-------------

          result = 'https://cmall.xiao100.com/bd/exam_rsv?';
          break;
        default:
          // 链接已切换-------------

          result = 'https://cmall-test.xiao100.com/bd/exam_rsv?';
      }
    }
    return result;
  };

  /**
   * 复制链接
   * */
  copyLink = () => {
    const input = document.querySelector('#exam-share-link-input');
    input.select();
    if (document.execCommand('copy')) {
      document.execCommand('copy');
      showMessage('复制成功', 'success');
    }
  };
  /**
   * 下载二维码分享图
   * */
  downloadShareImage = () => {
    this.updateState({ loading: true });
    this.getImage(() => {
      this.updateState({ loading: false });
    });
  };
  /**
   * 合成图片
   * */
  getImage = (callback) => {
    const { snapshotWrapId } = this.state;
    const config = {
      logging: false,
      backgroundColor: null,
    };
    html2canvas(document.querySelector(`#${snapshotWrapId}`), config).then((canvas) => {
      this.downloadImage(canvas, '二维码');
      if (callback) {
        callback();
      }
    });
  };
  /**
   * 下载图片
   * */
  downloadImage = (canvas, fileName) => {
    const img = this.canvasToImage(canvas);
    this.download(img.src, fileName);
  };
  /**
   * canvas转img
   * */
  canvasToImage = function (canvas) {
    let image = new Image();
    image.crossOrigin = 'Anonymous';
    image.src = canvas.toDataURL('image/png');
    return image;
  };
  /**
   * 下载文件
   * */
  download = function (href, title) {
    const a = document.createElement('a');
    a.setAttribute('href', href);
    a.setAttribute('download', title);
    a.click();
  };
  /**
   * 更新state
   * */
  updateState = (newState) => {
    this.setState((prevState, props) => {
      return {
        ...newState,
      };
    });
  };
  /**
   * 根据type设置title btnText btnFunc
   * */
  setModalTitle = (type) => {
    return type === 'link' ? '链接' : '二维码';
  };
  setBtnText = (type) => {
    return type === 'link' ? '复制链接' : '下载二维码';
  };
  setBtnFunc = (type) => {
    if (type === 'link') {
      return this.copyLink;
    } else {
      return this.downloadShareImage;
    }
  };

  render() {
    const { type, handleClose } = this.props;
    const { loading, error, shareLink, modalTitle, btnText, snapshotWrapId } = this.state;

    return (
      <Modal
        title={modalTitle}
        className="exam-share-modal"
        loading={loading}
        error={error}
        width={480}
        closable={true}
        onCancel={handleClose}
        footer={<ModalFooterBtn text={btnText} handleClick={this.btnFunc} disabled={loading} />}
      >
        {type === 'link' ? <LinkContent url={shareLink} /> : <QrCodeContent id={snapshotWrapId} url={shareLink} />}
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  const { institution } = state.account.user;
  const { sharedStorage } = state.main;
  return {
    system: institution || null,
    sharedStorage,
  };
};

export default connect(mapStateToProps)(ShareModal);
