/**
 * Created by Haolin on 2018/8/13.
 */
import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import Button from 'COMPONENT/Button';
import SearchBar from 'COMPONENT/SearchBar';
import List from 'COMPONENT/List';
import Icon from 'COMPONENT/Icon';
import Popover from 'COMPONENT/Popover';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import {
  loadCommonDict,
  loadBranchDict,
  loadCampusDict,
  loadBranchCampusDict,
  getDictTextByVal,
  getDictValByText,
} from 'UTIL/dict';
import ExamSiteModal from './ExamSiteModal';
import ModalShareLink from './ModalShareLink';
import { getExamSiteList, deleteExamSite } from 'SERVICE/signUpOnline';
import './style.less';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import dataBuryingPoint from 'UTIL/dataBuryingPoint';
import moment from 'moment';
import jslog from 'SRC/jslog';
import ShareQRCode from 'COMPONENT/ShareQRCode';
import { pre, prd, peiYouPre, peiYou, twoTeacherPre, twoTeacher } from 'CONST/buildEnv';
import { PROJECT_NAME } from 'CONST/config';

const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_XL } = List;

const RowWrapper = (props) => {
  const { rightSide, children, rightSize } = props;
  return (
    <div className="search-row-wrapper" style={{ paddingRight: rightSize - 50 }}>
      {children}
      <div className="search-wrapper-right" style={{ width: rightSize }}>
        {rightSide}
      </div>
    </div>
  );
};
const Search = (props) => {
  const {
    handleSearch,
    handleReset,
    handleKeyword,
    handleAdd,
    dict: { branchDict, branchCampusDict, versionDict, quarterDict, gradeDict, subjectDict },
    data: { branch, campus, version, quarter, grade, subject },
    btnAuths,
  } = props;
  return (
    <RowWrapper
      rightSize={100}
      rightSide={
        <Button onClick={handleAdd} disabled={btnAuths.indexOf('EXAM_ROOM_ADD') === -1}>
          新增考场
        </Button>
      }
    >
      <SearchBar onSearch={handleSearch} onReset={handleReset}>
        <Select
          placeholder="分公司"
          dict={branchDict}
          value={branch}
          showSearch={true}
          allowClear={false}
          onChange={(val) => handleKeyword({ type: 'branch', val: val })}
        />
        <MultiSelect
          placeholder="学习中心"
          dict={branch ? branchCampusDict && branchCampusDict.filter((item) => item.val === branch) : branchCampusDict}
          value={campus}
          showSearch={true}
          allowClear={false}
          onChange={(val) => handleKeyword({ type: 'campus', val: val })}
        />
        <Select
          placeholder="产品年份"
          dict={versionDict}
          value={version}
          onChange={(val) => handleKeyword({ type: 'version', val: val })}
        />
        <MultiSelect
          placeholder="产品季度"
          dict={quarterDict}
          value={quarter}
          onChange={(val) => handleKeyword({ type: 'quarter', val: val })}
        />
        <MultiSelect
          placeholder="年级"
          dict={gradeDict}
          value={grade}
          onChange={(val) => handleKeyword({ type: 'grade', val: val })}
        />
        <MultiSelect
          placeholder="科目"
          dict={subjectDict}
          value={subject}
          onChange={(val) => handleKeyword({ type: 'subject', val: val })}
        />
      </SearchBar>
    </RowWrapper>
  );
};
/**
 * 自定义列表单元格render
 * */
const TableSell = (props) => {
  const { type, record } = props;
  let list = record[type];
  let nodeList = list.map((item, i) => <p key={i}>{item}</p>);

  return (
    <div>
      {nodeList.slice(0, 2)}
      {list.length > 2 && (
        <p>
          <Popover overlayClassName="exam-site-sell-popover" content={nodeList}>
            <span className="sell-show-all">......</span>
          </Popover>
        </p>
      )}
    </div>
  );
};
// 系统标识
const SystemMark = {
  spark: 'XINGHUO',
  peiyou: 'ADVANCE',
  twoteacher: 'TWO_TEACHER',
};

class ExamSiteMgmt extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      name: '考场管理',
      loading: false,
      errorMsg: false,
      listLoading: false,
      pageCurrent: 1,
      pageTotal: 0,
      pageSize: 20,
      //搜索栏-字典
      branchDict: null,
      branchCampusDict: null,
      versionDict: null,
      quarterDict: null,
      gradeDict: null,
      subjectDict: null,
      deadlineDict: [
        {
          val: 'null',
          text: '不限制',
        },
        {
          val: '0',
          text: '当天0点截止',
        },
        {
          val: '1',
          text: '提前1天0点截止',
        },
        {
          val: '2',
          text: '提前2天0点截止',
        },
      ],
      //搜索栏-搜索条件
      branchVal: null,
      campusVal: null,
      versionVal: null,
      quarterVal: null,
      gradeVal: null,
      subjectVal: null,
      defaultVal: {
        branchVal: null,
        campusVal: null,
        versionVal: null,
        quarterVal: null,
        gradeVal: null,
        subjectVal: null,
        pageCurrent: 1,
      },
      //列表数据
      examSiteList: null,
      //考场弹窗
      toggle_examSite: false,
      modalType: null,
      editData: null,
      // 分享二维码
      toggle_shareQRCode: false,
      //
      toggle_shareLink: false,
    };
    // 考场链接
    this.shareUrl = '';
    this.shareExamSite = this.shareExamSite.bind(this);
    this.buildShareUrl = this.buildShareUrl.bind(this);
    this.switchShareQRCode = this.switchShareQRCode.bind(this);
    this.switchShareLink = this.switchShareLink.bind(this);
  }
  componentWillMount() {
    this.init();
  }
  /**
   * 初始化
   * */
  init = () => {
    this.updateState({ loading: true });
    this.initCheck();
  };
  initCheck = () => {
    const { uBranchId, uCampusId } = this.props;
    if (uBranchId && uCampusId) {
      this.initPage();
    } else {
      const that = this;
      setTimeout(() => {
        that.initCheck();
      }, 200);
    }
  };
  initPage = async () => {
    const { uBranchId, uCampusId } = this.props;
    const { pageSize, defaultVal } = this.state;
    const [branchDict, branchCampusDict, versionDict, quarterDict, gradeDict, subjectDict] = await Promise.all([
      loadBranchDict(),
      loadBranchCampusDict(),
      loadCommonDict('PRODUCT_VERSION'),
      loadCommonDict('PRODUCT_QUARTER'),
      loadCommonDict('STUDENT_GRADE'),
      loadCommonDict('SUBJECT'),
    ]);
    //根据用户所属组织初始化分公司及学习中心，当是集团的人，默认取第一个值
    let isGroup = uBranchId === uCampusId;
    let branchVal = isGroup ? branchDict[0].val : uBranchId;
    let campusVal = isGroup ? [branchCampusDict.find((item) => item.val === branchVal).options[0].val] : [uCampusId];
    let todayObj = new Date();
    let versionVal = getDictValByText(versionDict, todayObj.getFullYear().toString());
    await this.updateState({
      branchVal,
      campusVal,
      versionVal,
      // defaultVal:{...defaultVal,branchVal,campusVal}
    });
    // console.log("* ",isGroup?"集团员工":"分公司员工",":",branchVal,campusVal);
    await Promise.all([this.loadData({ pageCurrent: 1, pageSize })]);
    this.updateState({
      loading: false,
      branchDict,
      branchCampusDict,
      versionDict,
      quarterDict,
      gradeDict,
      subjectDict,
    });
  };
  /**
   * 更新state
   * */
  updateState = (newState) => {
    this.setState((prevState, props) => {
      return {
        ...newState,
      };
    });
  };
  /**
   * 搜索栏-查询
   * */
  updateSearch = () => {
    const { pageSize } = this.state;
    this.loadData({ pageCurrent: 1, pageSize });
  };
  /**
   * 搜索栏-重置
   * */
  updateReset = () => {
    const { defaultVal } = this.state;
    this.updateState({ ...defaultVal });
  };
  /**
   * 搜索栏-更新搜索条件
   * */
  updateKeyword = (params) => {
    const { type, val } = params;
    switch (type) {
      case 'branch':
        this.setState({ campusVal: null });
        break;
    }
    this.updateState({ [type + 'Val']: val });
  };
  /**
   * 新增/修改弹窗开关
   * */
  toggleExamSite = () => {
    this.setState((prevState) => ({
      toggle_examSite: !prevState.toggle_examSite,
    }));
  };
  /**
   * 关闭考场弹窗
   * */
  modalClose = () => {
    this.setState({
      modalType: null,
      editData: null,
    });
    this.updateState({ toggle_examSite: false });
  };
  /**
   * 考场弹窗成功回调
   * */
  modalSubmit = () => {
    const examSiteModalType = this.state.modalType;
    this.modalClose();
    if (examSiteModalType === 'edit') {
      this.loadData();
    } else {
      this.updateReset();
      this.updateSearch();
    }
  };
  /**
   * 新增考场
   * */
  addExamSite = () => {
    this.setState({
      modalType: 'add',
    });
    this.toggleExamSite();
  };
  /**
   * 加载考场列表
   * */
  loadData = (params) => {
    this.updateState({ listLoading: true });
    const { branchVal, campusVal, versionVal, quarterVal, gradeVal, subjectVal, pageCurrent, pageSize, deadlineDict } =
      this.state;
    let page, rows;
    if (params) {
      page = params.pageCurrent;
      rows = params.pageSize;
      this.updateState({
        pageCurrent: page,
        pageSize: rows,
      });
    } else {
      page = pageCurrent;
      rows = pageSize;
    }
    const apiParams = {
      branchId: branchVal,
      blCampusId: this.buildParams(campusVal),
      yearId: versionVal,
      quarterId: this.buildParams(quarterVal),
      gradeId: this.buildParams(gradeVal),
      subjectId: this.buildParams(subjectVal),
      page,
      rows,
    };
    getExamSiteList(apiParams, true).then(
      (data) => {
        let apiData = data.rows;
        let list = apiData.map((item, index) => {
          let {
            id,
            name,
            branchId,
            blCampusId,
            yearId,
            quarterId,
            gradeId,
            subjectId,
            startTime,
            endTime,
            branchName,
            blCampusName,
            yearName,
            quarterName,
            gradeName,
            subjectName,
            createTime,
            examRoomTime,
            createUserId,
            limitNumber,
            deadline,
          } = item;
          let time = examRoomTime.map((item) => item.appointmentTime);
          let number = examRoomTime.map((item) => item.number);
          let deadlineVal = typeof deadline === 'number' ? deadline.toString() : 'null';
          return {
            id,
            name: name || null,
            index: index + 1 + Number(((page - 1) * rows).toFixed(0)),
            branch: branchName,
            campus: this.analyzeParams(blCampusName),
            version: yearName,
            quarter: quarterName,
            grade: this.analyzeParams(gradeName),
            subject: this.analyzeParams(subjectName),
            appointment: time,
            peopleCount: number,
            subAndApptAndCount: this.analyzeSAC(examRoomTime),
            createTime: moment(createTime).format('YYYY-MM-DD hh:mm:ss'),
            branchId,
            campusId: this.analyzeParams(blCampusId),
            versionId: yearId,
            quarterId,
            gradeId: this.analyzeParams(gradeId),
            subjectId: this.analyzeParams(subjectId),
            beginTime: startTime ? this.analyzeParams(startTime) : [],
            endTime: endTime ? this.analyzeParams(endTime) : [],
            examRoomTime,
            createUserId,
            limitNum: limitNumber,
            deadline: getDictTextByVal(deadlineDict, deadlineVal),
            deadlineVal: deadlineVal,
          };
        });
        this.updateState({
          listLoading: false,
          examSiteList: list,
          pageTotal: data.records,
        });
        return true;
      },
      (err) => {
        this.updateState({
          listLoading: false,
        });
        console.log('错误：', err);
        return false;
      },
    );
  };
  /**
   * 辅助构建参数(用于loadData)
   * */
  buildParams = (params) => {
    return params ? (params.length > 1 ? params.join(',') : params[0]) : null;
  };
  /**
   * 辅助解析数据(用于loadData)
   * */
  analyzeParams = (params) => {
    return params.indexOf(',') !== -1 ? params.split(',') : [params];
  };
  analyzeSAC = (params) => {
    return params && params.length > 0
      ? params.map((item) => {
          const { subjectName, appointmentTime, number } = item;
          return <span>{`${subjectName || '无'} / ${appointmentTime} / ${number}`}</span>;
        })
      : [];
  };
  /**
   * 列表更多操作
   * */
  moreAction = (params) => {
    const { type, record, index } = params;
    switch (type) {
      case 'share':
        this.shareExamSite(params.shareType, record.id);
        let shareTypeText = params.shareType === 'qrCode' ? '二维码' : '链接';
        jslog.report('click', {
          function_name: `课程管理>在线报读管理>考场管理>分享考场预约${shareTypeText}`,
        });
        dataBuryingPoint.send({ operation: '考场管理的分享操作' });
        break;
      case 'edit':
        this.setState({
          modalType: 'edit',
          editData: record,
        });
        this.toggleExamSite();
        break;
      case 'delete':
        showConfirm('确定删除该考场吗？', () => {
          this.deleteAction(record.id);
        });
        break;
    }
  };
  // 分享考场
  shareExamSite(type, id) {
    this.shareUrl = this.buildShareUrl(id);
    if (type === 'qrCode') {
      // 分享二维码
      this.switchShareQRCode();
    } else {
      // 分享链接
      this.switchShareLink();
    }
  }
  // 构建分享链接
  buildShareUrl(id) {
    let baseUrl = 'https://cmall-test.xiao100.com/bd/exam_rsv';
    if (process.env) {
      if ([pre, peiYouPre, twoTeacherPre].includes(process.env.BUILD_ENV)) {
        baseUrl = 'https://cmall-pre.xiao100.com/bd/exam_rsv';
      } else if ([prd, peiYou, twoTeacher].includes(process.env.BUILD_ENV)) {
        baseUrl = 'https://cmall.xiao100.com/bd/exam_rsv';
      }
    }
    return `${baseUrl}?sys=${SystemMark[PROJECT_NAME]}&id=${id}`;
  }
  // 切换分享二维码弹窗显示
  switchShareQRCode() {
    this.setState(
      (prevState) => ({ toggle_shareQRCode: !prevState.toggle_shareQRCode }),
      () => {
        if (!this.state.toggle_shareQRCode) {
          this.shareUrl = '';
        }
      },
    );
  }
  // 切换分享链接弹窗显示
  switchShareLink() {
    this.setState(
      (prevState) => ({ toggle_shareLink: !prevState.toggle_shareLink }),
      () => {
        if (!this.state.toggle_shareLink) {
          this.shareUrl = '';
        }
      },
    );
  }
  /**
   * 删除考场
   * */
  deleteAction = (params) => {
    deleteExamSite({ id: params }, true).then(
      (data) => {
        showMessage('删除成功！', 'success');
        this.loadData();
      },
      (err) => {
        showMessage(err.text, 'error');
      },
    );
  };

  render() {
    const { route, uUserId, btnAuths } = this.props;
    const {
      name,
      loading,
      errorMsg,
      listLoading,
      pageCurrent,
      pageTotal,
      pageSize,
      branchDict,
      branchCampusDict,
      versionDict,
      quarterDict,
      gradeDict,
      subjectDict,
      deadlineDict,
      branchVal,
      campusVal,
      versionVal,
      quarterVal,
      gradeVal,
      subjectVal,
      toggle_shareQRCode,
      examSiteList,
      toggle_examSite,
      modalType,
      editData,
      toggle_shareLink,
      shareExamSiteId,
      shareType,
    } = this.state;
    const searchData = {
      branch: branchVal,
      campus: campusVal,
      version: versionVal,
      quarter: quarterVal,
      grade: gradeVal,
      subject: subjectVal,
    };
    const dict = {
      branchDict,
      branchCampusDict,
      versionDict,
      quarterDict,
      gradeDict,
      subjectDict,
      deadlineDict,
    };
    const btnAuthShare = btnAuths.indexOf('EXAM_ROOM_SHARE') === -1;
    const btnAuthEdit = btnAuths.indexOf('EXAM_ROOM_EDIT') === -1;
    const btnAuthDelete = btnAuths.indexOf('EXAM_ROOM_DELETE') === -1;
    const columns = [
      { title: '序号', dataIndex: 'index', key: 'index', width: COL_WIDTH_S },
      { title: '考场名称', dataIndex: 'name', key: 'name', width: COL_WIDTH_XL },
      { title: '分公司', dataIndex: 'branch', key: 'branch', width: COL_WIDTH_M },
      {
        title: '学习中心',
        dataIndex: 'campus',
        key: 'campus',
        width: COL_WIDTH_M,
        render: (text, record, index) => TableSell({ type: 'campus', text, record, index }),
      },
      { title: '年份', dataIndex: 'version', key: 'version', width: COL_WIDTH_M },
      { title: '季度', dataIndex: 'quarter', key: 'quarter', width: COL_WIDTH_M },
      {
        title: '年级',
        dataIndex: 'grade',
        key: 'grade',
        width: COL_WIDTH_M,
        render: (text, record, index) => TableSell({ type: 'grade', text, record, index }),
      },
      // { title: '科目', dataIndex: 'subject', key: 'subject',width:COL_WIDTH_M,
      //   render:(text, record, index)=>TableSell({type:"subject",text, record, index})},
      {
        title: '科目 / 考试时间 / 预约人数',
        dataIndex: 'subAndApptAndCount',
        key: 'apptAndCount',
        width: 300,
        render: (text, record, index) => TableSell({ type: 'subAndApptAndCount', text, record, index }),
      },
      {
        title: '人数限制',
        dataIndex: 'limitNum',
        key: 'limitNum',
        width: COL_WIDTH_M,
        filterDropdownVisible: false,
        filterIcon: (
          <Popover overlayClassName="exam-list-popover" content="人数限制：同一时间段预约人数不超过的数量">
            <Icon className="exam-list-icon" type="question_circle" size={12} />
          </Popover>
        ),
        filterDropdown: <span className="empty-content"> </span>,
      },
      { title: '截止时间', dataIndex: 'deadline', key: 'deadline', width: 130 },
      { title: '创建时间', dataIndex: 'createTime', key: 'createTime', width: 160 },
      {
        title: '操作',
        key: 'action',
        width: 165,
        fixed: 'right',
        getActionConfig: (text, record, index) => {
          return [
            {
              text: '修改',
              onClick: () => this.moreAction({ type: 'edit', record, index }),
              disabled: btnAuthEdit || uUserId !== record.createUserId,
            },
            {
              text: '删除',
              onClick: () => this.moreAction({ type: 'delete', record, index }),
              disabled:
                btnAuthDelete || !record.peopleCount.every((item) => item === 0) || uUserId !== record.createUserId,
            },
          ];
        },
      },
    ];

    return (
      <Page
        name={name}
        loading={loading}
        route={route}
        subHeader={
          <Search
            handleSearch={this.updateSearch}
            handleReset={this.updateReset}
            handleKeyword={this.updateKeyword}
            handleAdd={this.addExamSite}
            dict={dict}
            data={searchData}
            btnAuths={btnAuths}
          />
        }
        fitWindowHeight
      >
        <List
          dataSource={examSiteList ? examSiteList : []}
          columns={columns}
          loading={!loading && listLoading}
          pagination={{
            current: pageCurrent,
            total: pageTotal,
            pageSize,
          }}
          onChange={({ current, pageSize }) => this.loadData({ pageCurrent: current, pageSize: pageSize })}
          locale={{ filterTitle: '' }}
        />
        {toggle_examSite && (
          <ExamSiteModal
            type={modalType}
            initData={editData}
            dict={dict}
            handleSubmit={this.modalSubmit}
            handleCancel={this.modalClose}
          />
        )}
        <ShareQRCode
          title="分享 - 考场预约二维码"
          shareTitle="抢先预约分班测"
          shareText={this.shareUrl}
          fileName="抢先预约分班测_二维码"
          visible={toggle_shareQRCode}
          onCancel={this.switchShareQRCode}
        />
        {toggle_shareLink && <ModalShareLink url={this.shareUrl} onCancel={this.switchShareLink} />}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const { brenchId, campusId, userId } = state.account.user;
  return {
    uBranchId: brenchId || null,
    uCampusId: campusId || null,
    uUserId: userId || null,
    btnAuths: state.account.btnAuths || '',
  };
};

export default connect(mapStateToProps)(ExamSiteMgmt);
