import React from 'react';
import { connect } from 'dva';
import Button from 'COMPONENT/Button';
import Page from 'COMPONENT/Page';
import List from 'COMPONENT/List';
import SearchBar from 'COMPONENT/SearchBar';
import Input from 'COMPONENT/Input';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import { Tooltip } from 'antd';
import Icon from 'COMPONENT/Icon';
import DropdownMenu from 'COMPONENT/DropdownMenu';
import ShowConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import ShowMessage from 'COMPONENT/utils/globalTips/showMessage';
import { loadCommonDict } from 'UTIL/dict';
import Loader from './loader';
import moment from 'moment';
//TODO:暂时耦合了续报关联的代码，因为续报的代码同时也在改，下一次迭代再提取到公共代码使用
import { loadMainTc, loadAssistTc, loadSmallTc } from '../../course/ReEnroll/loader';
import commonStyles from '../common/commonStyle.less';
import showMessage from '../../../components/utils/globalTips/showMessage';
import { weekDayDict, SUMMER, WINTER } from 'CONST/dicts/common';

const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL, createSetWidth } = List;
const setWidth = createSetWidth();

const fullDict = [
  { text: '满班', val: '2' },
  { text: '未满班', val: '1' },
];
const limitDict = [
  { text: '已限制', val: '1' },
  { text: '未限制', val: '2' },
];
const uploadDict = [
  { text: '已上架', val: '1' },
  { text: '未上架', val: '2' },
];
const TWO_TEACHER_VAL = 'TWO_TEACHER';
/**
 * 数组转换为字符串，如果入参为空则返回undefinded，否则拼接为字符串
 */
function arrToStr(arr) {
  if (arr && arr.length > 0) {
    return arr.join(',');
  } else {
    return undefined;
  }
}

class EnrollDetail extends React.Component {
  constructor(props) {
    super(props);
    const { blCampusId, blCampusName, quarterId, quarterName } = this.props.params || {};
    const blCampusNameArray = blCampusName.split(',');
    const campusDict = blCampusId
      .split(',')
      .map((campusId, index) => ({ text: blCampusNameArray[index], val: campusId }));
    const canSelectQuarterId = quarterId.split(',');
    this.state = {
      loading: true, // boolean 页面加载状态
      //查询条件
      name: null,
      isOnline: null,
      isLimit: null,
      phaseId: null,
      examTypeId: null,
      classTypeId: null,
      mainTeacherId: null,
      assistTeacherId: null,
      gradeId: null,
      quarterId: [],
      campusId: [],
      subjectId: null,
      isFull: null,
      classLevel: null,
      //列表数据
      listLoading: false,
      rows: [],
      page: 1,
      pageSize: 50,
      total: null,
      //数据字典数据
      quarterDict: [],
      gradeDict: null,
      subjetDict: null,
      classTypeDict: null,
      entryTypeDict: null,
      mainTcDict: [],
      assistTcDict: [],
      classLevelDict: null,
      campusDict,
      //其他
      searchExpand: false,
      selectedRowKeys: [], //选中行的key的集合
      canSelectQuarterId,
    };
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 更新状态
   * @param  {object} changedState 改变的状态
   * @return {Promise}
   */
  updateState = (changedState) => {
    const newState = {
      ...this.state,
      ...changedState,
    };
    return new Promise((resolve) => {
      this.setState(newState, () => {
        resolve();
      });
    });
  };

  /**
   * 获取列表数据
   */
  loadListData = async (params) => {
    const {
      name,
      isOnline,
      isLimit,
      classTypeId,
      mainTeacherId,
      assistTeacherId,
      subjectId,
      gradeId,
      pageSize,
      page,
      isFull,
      phaseId,
      classLevel,
      quarterId,
      campusId,
      campusDict,
      canSelectQuarterId,
    } = this.state;
    //上一个列表传过来的参数
    const { productTypeId, yearId, gradeId: originalgradeId } = this.props.params;
    const formatBlCampusId = arrToStr(campusId && campusId.length ? campusId : campusDict.map((item) => item.val));
    const formatQuarterId = arrToStr(quarterId && quarterId.length ? quarterId : canSelectQuarterId);
    const apiParams = {
      className: name,
      classTypeId: classTypeId,
      mainTeacherId,
      assistTeacherId,
      subjectId: arrToStr(subjectId),
      gradeId: arrToStr(gradeId),
      pageSize,
      pageNo: page,
      phaseId: arrToStr(phaseId),
      blCampusId: formatBlCampusId,
      quarterId: formatQuarterId,
      productTypeId,
      yearId,
      originalgradeId,
      classLevel: arrToStr(classLevel),
      ...params,
    };
    isOnline && (apiParams.isOnline = isOnline);
    isLimit && (apiParams.isLimit = isLimit);
    isFull && (apiParams.isFull = isFull);
    try {
      return await Loader.loadDetailListApi(apiParams);
    } catch (e) {
      ShowMessage(e.text);
    }
  };

  /**
   * 初始化内容
   */
  initContent = async () => {
    this.setState({ loading: true });
    const { canSelectQuarterId } = this.state;
    const classQuarter = this.props.params.quarterId;
    const productTypeId = this.props.params.productTypeId;
    const asyncCalls = [
      loadCommonDict('CLASS_TYPE', productTypeId),
      loadCommonDict('PRODUCT_QUARTER'),
      loadCommonDict('STUDENT_GRADE'),
      loadCommonDict('SUBJECT'),
      loadCommonDict('ENTRY_TYPE'),

      productTypeId === TWO_TEACHER_VAL ? loadMainTc() : loadSmallTc(),
      productTypeId === TWO_TEACHER_VAL ? loadAssistTc() : Promise.resolve(), //双师才需要加载辅导老师
      this.loadListData(),
    ];
    //寒暑要获取期数
    if (classQuarter === WINTER || classQuarter === SUMMER) {
      asyncCalls.push(loadCommonDict('SMALL_CLASS_PHASE'));
    }
    Promise.all(asyncCalls).then(
      ([
        classTypeDict,
        quarterDict,
        gradeDict,
        subjetDict,
        entryTypeDict,
        mainTcDict,
        assistTcDict,
        list,
        phaseDict,
      ]) => {
        const canSelectQuarterDict = quarterDict.filter((quarter) => canSelectQuarterId.indexOf(quarter.val) > -1);
        this.setState({
          loading: false,
          rows: list.datas,
          total: list.rowCount,
          page: list.pageNo,
          quarterDict: canSelectQuarterDict,
          gradeDict,
          subjetDict,
          entryTypeDict,
          mainTcDict,
          assistTcDict,
          classTypeDict,
          phaseDict: phaseDict || weekDayDict,
        });
      },
    );
  };

  /**
   * 查询列表
   */
  search = async () => {
    await this.updateState({ listLoading: true, page: 1 });
    const list = await this.loadListData();
    this.updateState({
      listLoading: false,
      rows: list.datas,
      total: list.rowCount,
      page: list.pageNo,
    });
  };

  /**
   * 重置搜索表单
   */
  reset = async () => {
    this.updateState({
      name: null,
      isOnline: null,
      isLimit: null,
      examTypeId: null,
      classTypeId: null,
      mainTeacherId: null,
      assistTeacherId: null,
      subjectId: null,
      gradeId: null,
      isFull: null,
      phaseId: null,
    });
  };

  /**
   * 翻页
   */

  pageChange = async ({ current, pageSize }) => {
    await this.updateState({ listLoading: true, pageSize, page: current, selectedRowKeys: [] });
    const list = await this.loadListData();
    this.updateState({
      listLoading: false,
      rows: list.datas,
      total: list.rowCount,
      page: list.pageNo,
    });
  };

  /**
   * 批量操作前校验
   */
  batchValid = () => {
    const { selectedRowKeys } = this.state;
    if (selectedRowKeys && selectedRowKeys.length > 0) {
      return selectedRowKeys;
    } else {
      showMessage('请先选择班课!');
      return false;
    }
  };

  /**
   * 根据参数生成过滤id串，例如，批量上架要过滤掉已经上架的id
   * @param rows
   * @param selected
   * @param name
   * @param value
   * @returns {*}
   */
  filterRowKeyByValue = (rows, selected, name, value) => {
    return selected.reduce((acc, key) => {
      if (rows[key][name] !== value) {
        //第一个时，acc为空字符串，不需要加 ,
        if (acc) {
          return acc + ',' + rows[key].classId;
        } else {
          return '' + rows[key].classId;
        }
      } else {
        return acc;
      }
    }, '');
  };

  /**
   * 批量上架
   */
  batchUpload = async () => {
    const selectedRowKeys = this.batchValid();
    if (!selectedRowKeys) {
      return;
    }

    const { rows } = this.state;
    const { id: enrollFreeId } = this.props.params;
    //找出已下架(isOnline!==1)的classId串
    const afterFilter = this.filterRowKeyByValue(rows, selectedRowKeys, 'isOnline', 1);
    console.log(afterFilter);
    if (afterFilter) {
      this.setState({ listLoading: true });
      try {
        await Loader.toggleOnline({
          classId: afterFilter,
          isOnline: 1,
          productTypeId: this.props.params.productTypeId,
          enrollFreeId,
        });
        this.search();
      } catch (e) {
        showMessage(e.text);
      }
    }
  };

  /**
   * 批量下架
   */
  batchDownload = async () => {
    const selectedRowKeys = this.batchValid();
    if (!selectedRowKeys) {
      return;
    }

    const { rows } = this.state;
    const { id: enrollFreeId } = this.props.params;
    //找出已上架(isOnline!==2)的classId串
    const afterFilter = this.filterRowKeyByValue(rows, selectedRowKeys, 'isOnline', 2);
    console.log(afterFilter);
    if (afterFilter) {
      this.setState({ listLoading: true });
      try {
        await Loader.toggleOnline({
          classId: afterFilter,
          isOnline: 2,
          productTypeId: this.props.params.productTypeId,
          enrollFreeId,
        });
        this.search();
      } catch (e) {
        showMessage(e.text);
      }
    }
  };

  /**
   * 批量限制
   */
  batchLimit = async () => {
    const selectedRowKeys = this.batchValid();
    if (!selectedRowKeys) {
      return;
    }

    const { rows } = this.state;
    const { id: enrollFreeId } = this.props.params;
    //找出未限制(isLimit!==1)的classId串
    const afterFilter = this.filterRowKeyByValue(rows, selectedRowKeys, 'isLimit', 1);
    console.log(afterFilter);
    if (afterFilter) {
      this.setState({ listLoading: true });
      try {
        await Loader.toggleLimit({
          classId: afterFilter,
          isLimit: 1,
          productTypeId: this.props.params.productTypeId,
          enrollFreeId,
        });
        this.search();
      } catch (e) {
        showMessage(e.text);
      }
    }
  };

  /**
   * 批量取消限制
   */
  batchUnlimit = async () => {
    const selectedRowKeys = this.batchValid();
    if (!selectedRowKeys) {
      return;
    }

    const { rows } = this.state;
    const { id: enrollFreeId } = this.props.params;
    //找出已限制(isLimit!==2)的classId串
    const afterFilter = this.filterRowKeyByValue(rows, selectedRowKeys, 'isLimit', 2);
    console.log(afterFilter);
    if (afterFilter) {
      this.setState({ listLoading: true });
      try {
        await Loader.toggleLimit({
          classId: afterFilter,
          isLimit: 2,
          productTypeId: this.props.params.productTypeId,
          enrollFreeId,
        });
        this.search();
      } catch (e) {
        showMessage(e.text);
      }
    }
  };

  /**
   * 切换上下架
   */
  toggleUpload = async (record) => {
    const { isOnline, classId } = record;
    const { id: enrollFreeId } = this.props.params;
    await this.updateState({ listLoading: true });
    try {
      const data = await Loader.toggleOnline({
        classId,
        isOnline: isOnline === 2 ? 1 : 2,
        productTypeId: this.props.params.productTypeId,
        enrollFreeId,
      });
      ShowMessage(data.resultMessage, 'success');
      this.search();
    } catch (e) {
      ShowMessage(e.text);
      this.setState({
        listLoading: false,
      });
    }
  };

  /**
   * 切换限制
   */
  toggleLimit = (record) => {
    const { isLimit, classId } = record;
    const { id: enrollFreeId } = this.props.params;
    const text =
      isLimit === 2 ? '是否对所选班课进行报读限制处理？限制后学生需要有报读资格才允许报名' : '是否对所选班课取消限制？';
    ShowConfirm(text, async () => {
      await this.updateState({ listLoading: true });
      try {
        const data = await Loader.toggleLimit({
          classId,
          isLimit: isLimit === 2 ? 1 : 2,
          productTypeId: this.props.params.productTypeId,
          enrollFreeId,
        });
        ShowMessage(data.resultMessage, 'success');
        this.search();
      } catch (e) {
        ShowMessage(e.text);
        this.setState({
          listLoading: false,
        });
      }
    });
  };

  disableBtn = (record) => {
    const originalEnrollStartTime = moment(record.originalEnrollStartTime || Date.now()).valueOf(),
      now = moment().valueOf(),
      acrossEnrollEndTime = moment(record.acrossEnrollEndTime || Date.now()).valueOf();

    return now < originalEnrollStartTime || now > acrossEnrollEndTime;
  };

  renderOnline = (detail) => {
    return (
      <p key="asd" style={{ display: 'flex' }}>
        <b style={{ fontSize: '25px', color: detail === 1 ? '#FFCC49' : '#AAA', marginRight: '3px' }}>·</b>
        {detail === 1 ? '已上架' : '未上架'}
      </p>
    );
  };

  renderLimit = (detail) => {
    return (
      <p key="asd" style={{ display: 'flex' }}>
        <b style={{ fontSize: '25px', color: detail === 1 ? '#FFCC49' : '#AAA', marginRight: '3px' }}>·</b>
        {detail === 1 ? '已限制' : '未限制'}
      </p>
    );
  };

  /**
   * 班级类型改变，获取班级层次字典（班级类型与班级层次联动）
   */
  handleClassTypeChange = async (classTypeId) => {
    this.setState({ classTypeId, classLevelDict: null, classLevel: null, loading: true });
    let classLevelDict = await loadCommonDict('CLASS_LEVEL', classTypeId);
    this.setState({ classLevelDict, loading: false });
  };

  /**
   * 表头
   */
  columns = [
    { title: '班课名称', dataIndex: 'className', width: COL_WIDTH_XL, fixed: 'left' },
    { title: '期数/星期数', dataIndex: 'phaseName', width: COL_WIDTH_M },
    { title: '学习中心', dataIndex: 'campusName', width: COL_WIDTH_M },
    {
      title: '季度',
      dataIndex: 'productQuarterId',
      width: COL_WIDTH_M,
      render: (text) => {
        const quarterDict = Array.isArray(this.state.quarterDict) ? this.state.quarterDict : [];
        const quarterMap = quarterDict.reduce((obj, cur) => {
          obj[cur.val] = cur;
          return obj;
        }, {});
        const quarterArray = text ? text.split(',').map((quarterId) => quarterMap[quarterId].text) : [];
        return quarterArray.join(',');
      },
    },
    { title: '年级', dataIndex: 'gradeName', width: COL_WIDTH_S },
    { title: '科目', dataIndex: 'subjectName', width: COL_WIDTH_S },
    { title: '班级类型', dataIndex: 'classTypeName', width: setWidth(5) },
    { title: '班级层次', dataIndex: 'classLevelName', width: setWidth(5) },
    { title: '主讲老师', dataIndex: 'mainTeacherName', width: setWidth(5) },
    { title: '辅导老师', dataIndex: 'assistTeacherName', width: setWidth(5) },
    {
      title: '上课时间',
      dataIndex: 'classStartTime',
      width: COL_WIDTH_M,
      render: (detail, row) => detail + ' - ' + moment(detail, 'HH:mm').add(row.classLength, 'm').format('HH:mm'),
    },
    { title: '是否上架', dataIndex: 'isOnline', width: setWidth(5), render: this.renderOnline },
    {
      title: (
        <p style={{ display: 'flex', alignItems: 'center' }}>
          是否报读限制
          <Tooltip title="已限制：需要报读资格才可以报名；未限制：不需要报读资格就可以报名">
            <Icon type="question_mc_h" style={{ marginLeft: '5px' }} />
          </Tooltip>
        </p>
      ),
      dataIndex: 'isLimit',
      width: setWidth(8),
      render: this.renderLimit,
    },
    {
      title: '报读情况',
      dataIndex: 'fullNum',
      width: setWidth(5),
      render: (detail, row) => detail + ' / ' + row.allNum,
    },
    {
      title: '操作',
      key: 'action',
      width: COL_WIDTH_L,
      fixed: 'right',
      getActionConfig: (text, record, index) => {
        const { isLimit, isOnline } = record;
        return [
          { text: isOnline === 2 ? '上架' : '下架', onClick: () => this.toggleUpload(record) },
          { text: isLimit === 2 ? '限制报读' : '取消限制', onClick: () => this.toggleLimit(record) },
        ];
      },
    },
  ];

  render() {
    const {
      loading,
      searchExpand,
      listLoading,
      rows,
      page,
      pageSize,
      total,
      name,
      isOnline,
      isLimit,
      classTypeId,
      mainTeacherId,
      assistTeacherId,
      gradeId,
      campusId,
      quarterId,
      subjectId,
      isFull,
      mainTcDict,
      assistTcDict,
      classTypeDict,
      campusDict,
      quarterDict,
      gradeDict,
      subjetDict,
      phaseDict,
      phaseId,
      selectedRowKeys,
      classLevel,
      classLevelDict,
    } = this.state;
    const { productTypeId } = this.props.params;

    const menus = [
      { text: '批量上架', onClick: () => this.batchUpload() },
      { text: '批量下架', onClick: () => this.batchDownload() },
      { text: '批量限制报读', onClick: () => this.batchLimit() },
      { text: '批量取消限制', onClick: () => this.batchUnlimit() },
    ];

    return (
      <Page
        loading={loading}
        route={this.props.route}
        subHeader={
          <div className={commonStyles.searchBarWrapper}>
            <div className={commonStyles.searchBar} style={{ paddingRight: '81px' }}>
              <DropdownMenu
                className={commonStyles.addQuli}
                menus={menus}
                isCustom
                content={<Button>批量操作</Button>}
                activeContent={<Button>批量操作</Button>}
                disabled={!!!selectedRowKeys.length}
              />
              <SearchBar
                onSearch={this.search}
                onReset={this.reset}
                expand={searchExpand}
                onToggleExpand={(expand) => this.setState({ searchExpand: expand })}
              >
                <Input value={name} placeholder="班课名称" onChange={(e) => this.setState({ name: e.target.value })} />
                <MultiSelect
                  placeholder="期数/星期数"
                  dict={phaseDict}
                  value={phaseId}
                  allowClear={true}
                  onChange={(phaseId) => this.setState({ phaseId })}
                />
                <MultiSelect
                  placeholder="年级"
                  dict={gradeDict}
                  value={gradeId}
                  allowClear={true}
                  onChange={(gradeId) => this.setState({ gradeId })}
                />
                <MultiSelect
                  placeholder="科目"
                  dict={subjetDict}
                  value={subjectId}
                  allowClear={true}
                  onChange={(subjectId) => this.setState({ subjectId })}
                />
                <Select
                  placeholder="班级类型"
                  dict={classTypeDict}
                  value={classTypeId}
                  allowClear={true}
                  onChange={this.handleClassTypeChange}
                />
                <Select
                  placeholder="主讲老师"
                  dict={mainTcDict}
                  value={mainTeacherId}
                  showSearch
                  allowClear={true}
                  onChange={(mainTeacherId) => this.setState({ mainTeacherId })}
                />
                <MultiSelect
                  placeholder="学习中心"
                  dict={campusDict}
                  value={campusId}
                  showSearch
                  allowClear={true}
                  onChange={(campusId) => this.setState({ campusId: campusId || [] })}
                />
                <MultiSelect
                  placeholder="季度"
                  dict={quarterDict}
                  value={quarterId}
                  allowClear={true}
                  onChange={(quarterId) => this.setState({ quarterId: quarterId || [] })}
                />
                {productTypeId === TWO_TEACHER_VAL && (
                  <Select
                    placeholder="辅导老师"
                    dict={assistTcDict}
                    value={assistTeacherId}
                    showSearch
                    allowClear={true}
                    onChange={(assistTeacherId) => this.setState({ assistTeacherId })}
                  />
                )}
                <Select
                  placeholder="是否上架"
                  dict={uploadDict}
                  value={isOnline}
                  allowClear={true}
                  onChange={(isOnline) => this.setState({ isOnline })}
                />
                <Select
                  placeholder="是否限制报读"
                  dict={limitDict}
                  value={isLimit}
                  allowClear={true}
                  onChange={(isLimit) => this.setState({ isLimit })}
                />
                <Select
                  placeholder="是否满班"
                  dict={fullDict}
                  value={isFull}
                  allowClear={true}
                  onChange={(isFull) => this.setState({ isFull })}
                />
                <MultiSelect
                  placeholder="班级层次"
                  dict={classLevelDict}
                  value={classLevel}
                  allowClear={true}
                  noOptionTips="请先选择班级类型"
                  onChange={(classLevel) => this.setState({ classLevel })}
                />
              </SearchBar>
            </div>
            <div className={commonStyles.searchBarBtnWrapper}>
              {menus.map((menu) => (
                <Button type="primary" disabled={!!!selectedRowKeys.length} onClick={menu.onClick}>
                  {menu.text}
                </Button>
              ))}
            </div>
          </div>
        }
        fitWindowHeight
      >
        <List
          columns={this.columns}
          loading={listLoading}
          dataSource={rows}
          pagination={{
            current: page,
            pageSize,
            total: total,
          }}
          rowSelection={{
            selectedRowKeys,
            onChange: (keys) => this.setState({ selectedRowKeys: keys }),
          }}
          onChange={this.pageChange}
        />
      </Page>
    );
  }
}

const mapStateToProp = (state) => {
  let data = state.main.pageParams;
  let pathName = state.routing.locationBeforeTransitions.pathname;
  if (data[pathName]) {
    return {
      params: data[pathName],
    };
  } else {
    return {
      params: {},
    };
  }
};

export default connect(mapStateToProp, null)(EnrollDetail);
