import {
  loadFreeListApi,
  loadFreeDetailApi,
  addFreeApi,
  editFreeApi,
  changeFreeStatusApi,
  loadDetailListApi,
  toggleOnlineApi,
  getDataDict,
  toggleLimitApi,
  saveShareData,
  getUserInfoByUserId,
} from 'SERVICE/signUpOnline';

/**
 * 获取列表数据
 */
const loadList = async (params) => {
  // const data = await request('/EnrollManagerStudentQualificationController/listEnrollManagerStudentQualificationInfos.do?'+makeQueryString(params))
  // return data.data
  const data = await loadFreeListApi(params);
  if (data.resultCode !== 0) {
    throw { text: data.resultMessage };
  }
  return data;
};

/**
 * 获取详情
 */
const loadDetail = async (params) => {
  const data = await loadFreeDetailApi(params);
  if (data.resultCode !== 0) {
    throw new Error(data.resultMessage);
  }
  return data;
};

/**
 * 新增
 */
const add = async (params) => {
  const data = await addFreeApi(params);
  if (data.resultCode !== 0) {
    throw new Error(data.resultMessage);
  }
  return data;
};

/**
 * 编辑
 */
const edit = async (params) => {
  const data = await editFreeApi(params);
  if (data.resultCode !== 0) {
    throw new Error(data.resultMessage);
  }
  return data;
};

/**
 * 改变状态
 */
const changeStatus = async (params) => {
  const data = await changeFreeStatusApi(params);
  if (data.resultCode !== 0) {
    throw new Error(data.resultMessage);
  }
  return data;
};

const loadDetailList = async (params) => {
  const data = await loadDetailListApi(params);
  if (data.resultCode !== 0) {
    throw { text: data.resultMessage };
  }
  return data;
};

/**
 * 改变上下架
 */
const toggleOnline = async (params) => {
  const data = await toggleOnlineApi(params);
  if (data.resultCode !== 0) {
    throw { text: data.resultMessage };
  }
  return data;
};

/**
 * 改变限制
 */
const toggleLimit = async (params) => {
  const data = await toggleLimitApi(params);
  if (data.resultCode !== 0) {
    throw { text: data.resultMessage };
  }
  return data;
};

// 当前用户分享数据字典
const getRoleDict = async (params) => {
  const data = await getDataDict(params);
  if (data.resultCode !== 0) {
    throw new Error(data.resultMessage);
  }
  return data;
};

// 保存分享数据字典获取二维码
const getUserInfo = async (params) => {
  const data = await getUserInfoByUserId(params);
  if (data.resultCode !== 0) {
    throw new Error(data.resultMessage);
  }
  return data;
};

// 获取用户组织架构详情
const getShareId = async (params) => {
  const data = await saveShareData(params);
  if (data.resultCode !== 0) {
    throw new Error(data.resultMessage);
  }
  return data;
};

export default {
  loadList,
  loadDetail,
  add,
  edit,
  changeStatus,
  loadDetailListApi,
  toggleOnline,
  toggleLimit,
  getRoleDict,
  getShareId,
  getUserInfo,
};
