import React from 'react';
import { connect } from 'dva/index';
import Modal from 'COMPONENT/Modal';
import Select from 'COMPONENT/Select';
import FormItem from 'COMPONENT/FormItem';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import loader from '../loader';
import { getManagerDict } from 'SERVICE/followReport';
import { loadConsultantDict } from 'PAGE/common/loaders';
import {
  peiYouTest,
  peiYouPre,
  peiYou,
  twoTeacherTest,
  twoTeacherPre,
  twoTeacher,
  uat,
  pre,
  prd,
} from 'CONST/buildEnv';
import ShareQRCode from 'COMPONENT/ShareQRCode';
//form样式
const formCfg = {
  labelWidth: 100,
  style: {
    padding: '0',
    width: '520px',
  },
};
//学管师
const STUDY_MANAGER = 'ROL0000000001';
//小班班主任
const SMALL_CLASS_MANAGER = 'ROL0000000037';
//学管主管
const STUDY_MANAGER_EDITOR = 'ROL0000000007';

//咨询师
const CONSULTANT = '4028818d46e1abf10146e1abf3fd0001';
//咨询主管
const CONSULTANT_MANAGER = 'ROL0000000015';
//学习中心营运主任
const CAMPUS_OPERATION_DIRECTOR = 'ROL0000000077';
//学习中心主任
const CAMPUS_DIRECTOR = 'ROL0000000008';

const objToDict = (obj) => {
  return Object.keys(obj)
    .filter((ele) => !!ele)
    .map((ele) => {
      return {
        text: obj[ele],
        val: ele,
      };
    });
};

const cusOrgObjToDictObj = (cusOrgObj) => {
  return Object.keys(cusOrgObj).reduce((obj, ele) => {
    obj[ele] = cusOrgObj[ele].map((ele) => {
      return {
        text: ele.name,
        val: ele.id,
      };
    });
    return obj;
  }, {});
};

const buildBaseUrl = () => {
  let result = '';
  if (process.env) {
    switch (process.env.BUILD_ENV) {
      case pre:
      case peiYouPre:
      case twoTeacherPre:
        result = 'https://cmall-pre.xiao100.com/bd/advertisement?';
        break;
      case prd:
      case peiYou:
      case twoTeacher:
        result = 'https://cmall.xiao100.com/bd/advertisement?';
        break;
      default:
        result = 'https://cmall-test.xiao100.com/bd/advertisement?';
        break;
    }
    switch (process.env.BUILD_ENV) {
      case uat:
      case pre:
      case prd:
        result += 'sys=XINGHUO&';
        break;
      case peiYouTest:
      case peiYouPre:
      case peiYou:
        result += 'sys=ADVANCE&';
        break;
      case twoTeacherTest:
      case twoTeacherPre:
      case twoTeacher:
        result += 'sys=TWO_TEACHER&';
        break;
      default:
        result += 'sys=XINGHUO&';
        break;
    }
  }
  return result;
};

class ShareModal extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: true,
      resEntranceId: null,
      cusOrgId: null,
      resTypeId: null,
      userId: null,
      advId: null,
      advDict: [],
      userManagerDict: [],
      resTypeDict: [],
      cusOrgDict: [],
      resEntranceDict: [],
      resEntranceSelectorDisabled: false,
      cusOrgSelectorDisabled: false,
      resTypeSelectorDisabled: false,
      inviteUserSelectorDisabled: true,
      isUseSelectDisabled: true,
      qrcodeVisiable: false,
      qrcodeVisible: false,
      qrcodeUrl: '',

      checking: false,
    };
    this.cusOrgDictObj = {};
    this.errors = {};
  }

  confirm = async () => {
    const { resEntranceId, cusOrgId, resTypeId, checking, userId, advId, isUseSelectDisabled } = this.state;
    const { shareActivity, onConfirm } = this.props;
    this.setState({ loading: true });
    try {
      if (!checking) {
        this.setState({ checking: true });
      }
      const errors = Object.values(this.errors).filter((ele) => ele);
      if (errors.length) {
        this.modal.showOkTips('保存失败：请确认必填输入项是否填写完毕');
        this.setState({
          loading: false,
        });
        return;
      }
      // 能够选择下拉但是userId为空提示要选前担任
      if (!isUseSelectDisabled && !userId) {
        this.modal.showOkTips('请选择签单人');
        this.setState({
          loading: false,
        });
        return;
      }
      // 真正选择的用户id currentUserId
      // userId可以为空 说明是默认营运学习中心主任直接传空即可
      let currentUserId;
      let follweType;
      if (userId) {
        [follweType, currentUserId] = userId.split('//');
      }

      const params = {
        enrollId: shareActivity.id,
        cusOrg: cusOrgId,
        resEntranceId,
        resType: resTypeId,
        signUserId: currentUserId,
        advId,
      };
      const res = await loader.getShareId(params);
      let qrcodeUrl = buildBaseUrl();
      qrcodeUrl += `shareCodeId=${res.data}`;
      // 如果选中了宣传页 那么拼接上advId
      if (advId) {
        qrcodeUrl += `&advId=${advId}`;
      }
      this.setState({
        qrcodeVisiable: true,
        qrcodeUrl,
      });
      onConfirm && onConfirm();
    } catch (e) {
      console.log(e);
      showMessage(e.text || e.message, 'error');
    }
    this.setState({ loading: false });
  };

  componentDidMount() {
    this.initContent();
  }

  initContent = async () => {
    this.setState({ loading: true });
    const { shareActivity, user } = this.props;
    const { branchId } = shareActivity;
    const asyncCalls = [
      getManagerDict({ roleCode: 'STUDY_MANAGER,STUDY_MANAGER_HEAD' }),
      loadConsultantDict({ orgId: branchId }),
      loader.getRoleDict({ orgId: branchId }),
      loader.getUserInfo({ userId: user.userId }),
    ];
    const [managerObj, consultantDict, res, userInfo] = await Promise.all(asyncCalls);

    //学管师数据处理
    let _managerObj = managerObj.value;
    let managerArray = Object.keys(_managerObj);
    let managerDict = managerArray.map((v) => {
      return { text: _managerObj[v], val: v };
    });
    //合并咨询师学管师字典
    let userManagerDict = [
      {
        label: '学管师',
        options: managerDict.map((manager) => ({ ...manager, group: '学管师', val: `manager//${manager.val}` })),
        val: 'manager',
      },
      {
        label: '咨询师',
        options: consultantDict.map((user) => ({ ...user, group: '咨询师', val: `user//${user.val}` })),
        val: 'user',
      },
    ];

    // 处理主组织架构的可选可不选问题
    const { userOrgRoles } = userInfo.data;
    let defaultUesId = null;
    let isUseSelectDisabled;
    // 如果userOrgRoles存在用这个 这个是主组织架构的角色
    const { roleId, userId, userName } = userOrgRoles.find((role) => role.isMain === 0);

    // 主组织架构用user拿
    const { organizationType } = this.props.user;
    const isCampus = organizationType === 'CAMPUS';
    const isBranch = organizationType === 'BRENCH';
    // 如果是学习中心或者分公司的学管师（ROL0000000001） 小班班主任(SMALL_CLASS_MANAGER) 学管主任(STUDY_MANAGER_EDITOR) 则为自己
    const userArray = [STUDY_MANAGER, STUDY_MANAGER_EDITOR, SMALL_CLASS_MANAGER];
    if ((isCampus || isBranch) && userArray.indexOf(roleId) > -1) {
      defaultUesId = `user//${userId}`;
      isUseSelectDisabled = true;

      // 如果默认自己那么字典只有一个并且值也是自己
      userManagerDict = [{ text: userName, val: defaultUesId }];
    } else if (isCampus && (roleId === CONSULTANT || roleId === CONSULTANT_MANAGER)) {
      //如果角色为学习中心咨询师(4028818d46e1abf10146e1abf3fd0001)咨询主管("ROL0000000015") 则为自己
      //如果角色为学习中心营运主任("ROL0000000077")和学习中心主任(ROL0000000008)则选择本学习中心的咨询师或者学管师
      // 其他的不允许选择 添加默认学习中心营运主任的提示语

      defaultUesId = `user//${userId}`;
      isUseSelectDisabled = true;
      // 如果默认自己那么字典只有一个并且值也是自己
      userManagerDict = [{ text: userName, val: defaultUesId }];
    } else if (isCampus && (roleId === CAMPUS_OPERATION_DIRECTOR || roleId === CAMPUS_DIRECTOR)) {
      // 正常加载数据 不做操作
    } else {
      isUseSelectDisabled = true;
    }

    //资料来源数据管理
    const { resType, resEntrance, cusOrg, adv } = res.data;
    const resTypeDict = (resType || []).map(({ key, value }) => ({ text: value, val: key })).filter((res) => res.val);
    const resEntranceDict = objToDict(resEntrance || {});
    this.cusOrgDictObj = cusOrgObjToDictObj(cusOrg || {});
    const resEntranceId = resEntranceDict.length === 1 ? resEntranceDict[0].val : '';
    const advDict = (adv || []).map(({ name, attachmentId }) => ({ text: name, val: attachmentId }));
    this.setState({
      resTypeDict,
      resEntranceDict,
      loading: false,
      userManagerDict,
      resEntranceId,
      advDict,
      resEntranceSelectorDisabled: resEntranceDict.length === 1 || resTypeDict.length === 2,
      userId: defaultUesId,
      isUseSelectDisabled,
    });
  };

  onResTypeChange = (resTypeId) => {
    const { resTypeDict, resEntranceId } = this.state;
    let newResEntranceId = resEntranceId;
    if (resTypeDict.length === 2) {
      switch (resTypeId) {
        case 'b07e325eadf52ffc67d615ddc08e344d':
          newResEntranceId = 'NEW_MEDIA';
          break;
        case 'TEL_CONSULTATION':
          newResEntranceId = 'GROUNDPROMOTION';
          break;
      }
    }
    const cusOrgDict = !resTypeId ? [] : this.cusOrgDictObj[resTypeId];
    this.setState({
      cusOrgDict,
      cusOrgId: '',
      resEntranceId: newResEntranceId,
    });
  };

  errorChange = (key, error) => {
    this.errors[key] = error;
  };

  render() {
    const { onClose, enrollName } = this.props;
    const {
      loading,
      resEntranceId,
      cusOrgId,
      resTypeId,
      resTypeDict,
      cusOrgDict,
      resEntranceDict,
      isUseSelectDisabled,
      qrcodeVisiable,
      qrcodeUrl,
      resEntranceSelectorDisabled,
      cusOrgSelectorDisabled,
      resTypeSelectorDisabled,
      checking,
      userManagerDict,
      userId,
      advDict,
      advId,
    } = this.state;

    return (
      <Modal
        loading={loading}
        title="分享"
        onCancel={onClose}
        okText="生成二维码"
        cancelText="关闭"
        onOk={this.confirm}
        ref={(c) => (this.modal = getWrappedInstance(c))}
        width={650}
      >
        <div>
          <FormItem {...formCfg} name="资源入口" required>
            <Select
              placeholder="请选择"
              dict={resEntranceDict}
              value={resEntranceId}
              onChange={(val) => this.setState({ resEntranceId: val })}
              disabled={resEntranceSelectorDisabled}
              validator={{
                rules: [{ required: '请选择资源入口' }],
                silent: !checking,
                onErrorChange: (e) => this.errorChange('resEntranceId', e),
              }}
            />
          </FormItem>
          <FormItem {...formCfg} name="资源来源" required>
            <Select
              placeholder="请选择"
              dict={resTypeDict}
              value={resTypeId}
              onChange={(val) => {
                this.setState({ resTypeId: val });
                this.onResTypeChange(val);
              }}
              disabled={resTypeSelectorDisabled}
              validator={{
                rules: [{ required: '请选择资源来源' }],
                silent: !checking,
                onErrorChange: (e) => this.errorChange('resTypeId', e),
              }}
            />
          </FormItem>
          <FormItem {...formCfg} name="来源细分" required>
            <Select
              placeholder="请选择"
              dict={cusOrgDict}
              value={cusOrgId}
              onChange={(val) => this.setState({ cusOrgId: val })}
              disabled={cusOrgSelectorDisabled}
              validator={{
                rules: [{ required: '请选择来源细分' }],
                silent: !checking,
                onErrorChange: (e) => this.errorChange('cusOrgId', e),
              }}
            />
          </FormItem>
          <FormItem {...formCfg} name="签单人" required={!isUseSelectDisabled}>
            <Select
              placeholder={isUseSelectDisabled ? '签单人默认学习中心营运主任' : '请选择'}
              dict={userManagerDict}
              value={userId}
              showSearch
              disabled={isUseSelectDisabled}
              validator={
                isUseSelectDisabled
                  ? null
                  : {
                      rules: [{ required: '请选择签单人' }],
                      silent: !checking,
                      onErrorChange: (e) => this.errorChange('userId', e),
                    }
              }
              onChange={(val) => this.setState({ userId: val })}
            />
          </FormItem>
          <FormItem {...formCfg} name="宣传页">
            <Select
              placeholder="请选择宣传页"
              dict={advDict}
              value={advId}
              showSearch
              onChange={(val) => this.setState({ advId: val })}
            />
          </FormItem>
        </div>
        <ShareQRCode
          title="分享"
          shareTitle={enrollName}
          shareText={qrcodeUrl}
          fileName="在线报读_二维码"
          visible={qrcodeVisiable}
          onCancel={() => {
            this.setState({ qrcodeVisiable: false });
          }}
        />
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    user: state.account.user,
  };
};
export default connect(mapStateToProps)(ShareModal);
