import React from 'react';
import { connect } from 'dva';
import { MultiSelect, ShareQRCode, Select, Input, SearchBar, List, Page, Button } from 'COMPONENT';
import ShowMessage from 'COMPONENT/utils/globalTips/showMessage';
import moment from 'moment';
import commonStyles from '../common/commonStyle.less';
import { renderMultiLine } from '../common';
import ProductModal from './modal/ProductModal';
import ShareProductModal from './modal/ShareProductModal';
import ShareLinkModal from './modal/ShareLinkModal';
import { isTwoTeacher, newCmallLoader, defaultProductTypeId, productTypeMap } from './utils';
import getXhmallShareUrl from './utils/getXhmallShareUrl';

const spingId = 'DAT0000000231'; //春季id
const autumnId = 'DAT0000000233'; //秋季id

let defaultBranch = null; //默认第一个学习中心
let defaultCampus = null; // 默认第一个学习中心

const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL, createSetWidth } = List;
const setWidth = createSetWidth();

const fullDict = [
  { text: '满班', val: '0' },
  { text: '未满班', val: '1' },
];
const POST_MESSAGE_TYPE = ['SMALL_CLASS', 'TWO_TEACHER'];

/**
 * 数组转换为字符串，如果入参为空则返回undefinded，否则拼接为字符串
 */
function arrToStr(arr) {
  if (Array.isArray(arr)) {
    return arr.join(',');
  } else {
    return arr;
  }
}

function arrayToMap(arr, obj = {}) {
  if (Array.isArray(arr)) {
    return arr.reduce((obj, cur) => {
      return {
        [cur.val]: cur,
        ...obj,
      };
    }, obj);
  }
  return obj;
}

class NewSightUpOnline extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: true, // boolean 页面加载状态
      showProductModal: false,
      showShareProductModal: false,
      shareQRCodeVisible: false,
      shareLinkVisible: false,
      // 分享的二维码链接
      shareLink: '',
      //查询条件
      name: null,
      isOnline: null,
      isLimit: null,
      phaseId: null,
      examTypeId: null,
      classTypeId: null,
      mainTeacherId: null,
      assistTeacherId: null,
      gradeId: null,
      quarterId: null,
      campusId: [],
      subjectId: null,
      isFull: null,
      classLevel: null,
      // 产品类型id
      productTypeId: defaultProductTypeId,
      //列表数据
      listLoading: false,
      rows: [],
      page: 1,
      pageSize: 50,
      total: null,
      //数据字典数据
      quarterDict: [],
      gradeDict: [],
      subjetDict: [],
      classTypeDict: [],
      mainTcDict: [],
      assistTcDict: [],
      classLevelDict: [],
      yearDict: [],
      branchCampusDict: [],
      branchDict: [],
      phaseDict: [],
      weekDict: [],
      // 期数星期数的map数据
      phaseOrWeekMap: {},
      //其他
      shareTitle: '',
      searchExpand: true,
      selectedRowKeys: [], //选中行的key的集合
      selectedRows: [], // 选中的数据
      // 要编辑课程详情的列表数据
      editProductList: [],
    };
  }

  componentDidMount() {
    // 监听iframe编辑页面传来的刷新页面事件
    window.addEventListener(
      'message',
      async (e) => {
        if (
          e.data &&
          e.data.mark === 'NEXT_COMPACT' &&
          e.data.TYPE === 'SUCCESS' &&
          POST_MESSAGE_TYPE.indexOf(e.data.productType) > -1
        ) {
          this.props.dispatch({
            type: 'main/removeTab',
            payload: { id: 'NewSignUpCourseManage/EditCourseCompact' },
          });
          this.props.dispatch({
            type: 'main/selectTab',
            payload: {
              id: '/NewSignUpCourseManage',
            },
          });
          //重新加载列表 关闭产品选择模态框
          await this.updateState({ showProductModal: false });
          // 0521版本之后列表页不需要重新刷新
          // await this.updateState({ listLoading: true, page: 1, showProductModal: false })
          // const list = await this.loadListData()
          // this.updateState({
          //   listLoading: false,
          //   rows: list.records,
          //   total: list.total,
          //   page: list.current
          // })
        }
      },
      false,
    );
    this.initContent();
  }

  /**
   * 更新状态
   * @param  {object} changedState 改变的状态
   * @return {Promise}
   */
  updateState = (changedState) => {
    const newState = {
      ...this.state,
      ...changedState,
    };
    return new Promise((resolve) => {
      this.setState(newState, () => {
        resolve();
      });
    });
  };

  /**
   * 获取列表数据
   */
  loadListData = async (params) => {
    const {
      name,
      branchId,
      campusId,
      quarterId,
      subjectId,
      gradeId,
      pageSize,
      page,
      phaseId,
      classTypeId,
      classLevel,
      isFull,
      mainTeacherId,
    } = this.state;
    const apiParams = {
      name,
      branchId: arrToStr(branchId),
      blCampusId: arrToStr(campusId),
      productQuarterId: quarterId,
      subjectId: arrToStr(subjectId),
      gradeId: arrToStr(gradeId),
      pageSize,
      pageNo: page,
      phaseId: arrToStr(phaseId),
      miniClassTypeId: arrToStr(classTypeId),
      teacherId: mainTeacherId,
      classLevelId: arrToStr(classLevel),
      ...params,
    };
    isFull && (apiParams.isClassFull = isFull);
    try {
      return await newCmallLoader.loadList(apiParams);
    } catch (e) {
      ShowMessage(e.text);
    }
  };

  /**
   * 初始化内容
   */
  initContent = async () => {
    const { getSelectOption, loadTeacher, getBranchCampusDict, getBranchDict } = newCmallLoader;
    const { uBranchId, uCampusId } = this.props;
    this.setState({ loading: true });
    const asyncCalls = [
      getSelectOption('CLASS_TYPE', 'SMALL_CLASS'),
      getBranchDict(),
      getBranchCampusDict(),
      getSelectOption('PRODUCT_VERSION'),
      getSelectOption('PRODUCT_QUARTER'),
      getSelectOption('STUDENT_GRADE'),
      getSelectOption('SUBJECT'),
      getSelectOption('SMALL_CLASS_PHASE'),
      getSelectOption('PRODUCT_CHARACTER'),
      getSelectOption('WEEK_DAY'),
    ];
    loadTeacher({ branchIds: [uBranchId] }).then((mainTcDict) => {
      this.setState({
        mainTcDict,
      });
    });
    Promise.all(asyncCalls)
      .then(
        ([
          classTypeDict,
          branchDict,
          branchCampusDict,
          yearDict,
          quarterDict,
          gradeDict,
          subjetDict,
          phaseDict,
          characterDict,
          weekDict,
        ]) => {
          //根据用户所属组织初始化分公司及学习中心，当是集团的人，默认取第一个值
          let isGroup = uBranchId === uCampusId;
          //默认第一个分公司，当前年份
          defaultBranch = isGroup ? branchDict[0].val : uBranchId;
          const characterMap = characterDict.reduce((prev, obj) => {
            prev[obj.val] = obj.text;
            return prev;
          }, {});

          // 不要短期班
          classTypeDict = classTypeDict.filter((item) => {
            return item.val.indexOf('DAT0000000239') === -1;
          });
          console.log(classTypeDict);
          const phaseOrWeekMap = arrayToMap(weekDict, arrayToMap(phaseDict));
          //获取列表数据
          this.loadListData({ branchId: arrToStr(defaultBranch) }).then((list) => {
            this.setState({
              branchId: defaultBranch,
              loading: false,
              rows: list.records,
              total: list.total,
              page: list.current,
              branchDict,
              yearDict,
              quarterDict,
              gradeDict,
              subjetDict,
              phaseDict,
              weekDict,
              characterMap,
              classTypeDict,
              branchCampusDict,
              phaseOrWeekMap,
            });
          });
        },
      )
      .catch((error) => {
        ShowMessage(error.text || '字典获取失败！请联系管理员');
      });
  };

  /**
   * 查询列表
   */
  search = async () => {
    await this.updateState({ listLoading: true, page: 1 });
    const list = await this.loadListData();
    this.updateState({
      listLoading: false,
      rows: list.records,
      total: list.total,
      page: list.current,
    });
  };

  /**
   * 重置搜索表单
   */
  reset = async () => {
    this.updateState({
      name: null,
      isOnline: null,
      isLimit: null,
      examTypeId: null,
      classTypeId: null,
      mainTeacherId: null,
      assistTeacherId: null,
      subjectId: null,
      gradeId: null,
      isFull: null,
      phaseId: null,
      quarterId: null,
      campusId: [],
      classLevel: [],
    });
  };

  /**
   * 翻页
   */

  pageChange = async ({ current, pageSize }) => {
    await this.updateState({
      listLoading: true,
      pageSize,
      page: current,
      selectedRowKeys: [],
      selectedRows: [],
    });
    const list = await this.loadListData();
    this.updateState({
      listLoading: false,
      rows: list.records,
      total: list.total,
      page: list.current,
    });
  };

  /**
   * 班级类型改变，获取班级层次字典（班级类型与班级层次联动）
   */
  handleClassTypeChange = async (classTypeId) => {
    if (!classTypeId) {
      this.setState({ classTypeId, classLevelDict: [] });
      return false;
    }
    this.setState({
      classTypeId,
      classLevelDict: null,
      classLevel: null,
      loading: true,
    });
    let classLevelDict = await newCmallLoader.getSelectOption('CLASS_LEVEL', classTypeId);
    this.setState({ classLevelDict, loading: false });
  };

  /**
   * 批量增加或者编辑课程列表
   * @param productList 课程列表
   */
  groupNewOrEditCourse = (productList) => {
    const { branchId } = this.state;
    const miniClassId = productList.map((product) => product.miniClassId).join(',');
    // 跳转修改页面
    let url = '/xhmall/admin/cmall/course/edit';
    if (isTwoTeacher) {
      url += '/ss';
    }
    const path = encodeURIComponent(url);
    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path: 'NewSignUpCourseManage/EditCourseCompact',
        params: {
          path,
          miniClassId,
          branchId,
        },
        refresh: true,
      },
    });
  };
  handleEditCourse = (records) => {
    const { branchId } = this.state;
    const courseIds = records.map((record) => record.id).join(',');
    const miniClassIds = records.map((record) => record.miniClassId).join(',');
    // 跳转修改页面
    let url = '/xhmall/admin/cmall/course/edit';
    if (isTwoTeacher) {
      url += '/ss';
    }
    const path = encodeURIComponent(url);
    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path: 'NewSignUpCourseManage/EditCourseCompact',
        params: {
          path,
          miniClassId: miniClassIds,
          branchId,
          courseId: courseIds,
        },
        refresh: true,
      },
    });
  };

  /**
   * 分享课程列表确认分享
   * @param shareLink 分享的链接
   * @param type 类型 qr 二维码 link 链接
   * @param title 标题
   */
  shareUrl = ({ shareLink, type, title } = {}) => {
    if (type === 'qr') {
      this.setState({
        shareQRCodeVisible: true,
      });
    } else {
      // 分享链接
      this.setState({
        shareLinkVisible: true,
      });
    }
    this.setState({
      shareLink,
      showShareProductModal: false,
      shareTitle: title,
    });
  };

  /**
   * 单课程分享
   * @param shareType 分享类型
   * @param record 分享的数据
   */
  shareAction = ({ shareType, record }) => {
    const { name, miniClassId } = record;
    const courseId = (isTwoTeacher ? 'SS-' : 'XB-') + miniClassId;
    const shareUrl = getXhmallShareUrl({
      url: `mapp/normal-sales/course/${courseId}/detail`,
      extraParams: {
        enrollCourseType: 'FREE',
      },
    });
    this.shareUrl({
      shareLink: shareUrl,
      type: shareType,
      title: name,
    });
  };
  /**
   * 小班双师表头
   */
  columns = [
    {
      title: isTwoTeacher ? '班课名称' : '课程名称',
      dataIndex: 'name',
      width: COL_WIDTH_XL,
      fixed: 'left',
    },
    {
      title: '期数/星期数',
      dataIndex: 'phaseId',
      width: COL_WIDTH_M,
      render: (text, record) => {
        const { phaseOrWeekMap } = this.state;
        if (isTwoTeacher) {
          return record.phaseName || '暂无';
        }
        return (phaseOrWeekMap[text] || {}).text || '暂无';
      },
    },
    { title: '分公司', dataIndex: 'branchName', width: COL_WIDTH_M },
    {
      title: '学习中心',
      dataIndex: 'blCampusName',
      width: COL_WIDTH_M,
      render: (text) => text || '暂无',
    },
    { title: '季度', dataIndex: 'productQuarterName', width: COL_WIDTH_S },
    {
      title: '年级',
      dataIndex: 'gradeName',
      width: COL_WIDTH_S,
      render: renderMultiLine,
    },
    {
      title: '科目',
      dataIndex: 'subjectName',
      width: COL_WIDTH_S,
      render: renderMultiLine,
    },
    { title: '班级类型', dataIndex: 'classTypeName', width: setWidth(5) },
    {
      title: '班级层次',
      dataIndex: 'classLevelName',
      width: setWidth(5),
      render: (text) => text || '暂无',
    },
    { title: '主讲老师', dataIndex: 'teacherName', width: setWidth(5) },
    {
      title: '辅导老师',
      dataIndex: 'assistTeacherName',
      width: setWidth(5),
      render: (text) => text || '暂无',
    },
    {
      title: '上课时间',
      dataIndex: 'classTime',
      width: COL_WIDTH_M,
      render: (detail, row) =>
        detail +
        ' - ' +
        moment(detail, 'HH:mm')
          .add(row.classTimeLength * row.everyCourseClassNum, 'm')
          .format('HH:mm'),
    },
    {
      title: '报读情况',
      dataIndex: 'miniClassPeopleNum',
      width: COL_WIDTH_M,
      render: (detail, row) => `${detail || 0} / ${row.peopleQuantity}`,
    },
    {
      title: '操作',
      key: 'action',
      width: 170,
      fixed: 'right',
      getActionConfig: (text, record) => {
        const btns = [
          {
            text: '完善课程信息',
            onClick: () => this.handleEditCourse([record]),
          },
        ];

        if (!isTwoTeacher) {
          btns.push({
            menus: [
              {
                text: '二维码',
                onClick: () => this.shareAction({ shareType: 'qr', record }),
              },
              {
                text: '链接',
                onClick: () => this.shareAction({ shareType: 'link', record }),
              },
            ],
            menuText: '分享',
          });
        }
        return btns;
      },
    },
  ];
  render() {
    const {
      loading,
      searchExpand,
      listLoading,
      rows,
      page,
      pageSize,
      total,
      selectedRows,
      name,
      classTypeId,
      mainTeacherId,
      gradeId,
      campusId,
      quarterId,
      showProductModal,
      showShareProductModal,
      shareQRCodeVisible,
      shareLink,
      subjectId,
      isFull,
      mainTcDict,
      classTypeDict,
      quarterDict,
      branchId,
      yearDict,
      branchCampusDict,
      weekDict,
      shareLinkVisible,
      shareTitle,
      gradeDict,
      subjetDict,
      phaseDict,
      phaseId,
      selectedRowKeys,
      classLevel,
      classLevelDict,
      branchDict,
    } = this.state;
    let phaseOrWeekDict = [];
    if (quarterId) {
      if (quarterId === autumnId || quarterId === spingId) {
        phaseOrWeekDict = weekDict;
      } else {
        phaseOrWeekDict = phaseDict;
      }
    }
    const campusDict = branchId
      ? branchCampusDict && branchCampusDict.filter((item) => branchId === item.val)
      : branchCampusDict;

    const addProductDict = {
      branchCampusDict,
      yearDict,
      gradeDict,
      subjetDict,
      quarterDict,
      branchDict,
      classTypeDict,
    };

    const shareProductDict = {
      branchCampusDict,
      yearDict,
      gradeDict,
      subjetDict,
      quarterDict,
      branchDict,
      weekDict,
      phaseDict,
      mainTcDict,
      fullDict,
    };
    return (
      <Page
        loading={loading}
        route={this.props.route}
        subHeader={
          <div className={commonStyles.searchBar} style={{ paddingRight: '81px' }}>
            <Button className={commonStyles.addQuli} onClick={() => this.setState({ showProductModal: true })}>
              新增课程
            </Button>
            {/* 暂时屏蔽此功能 */}
            {/* {!isTwoTeacher && (
              <Button
                className={commonStyles.shareBtn}
                onClick={() => this.setState({ showShareProductModal: true })}
              >
                分享课程列表
              </Button>
            )} */}
            <Button
              disabled={selectedRows.length === 0}
              className={commonStyles.groupEditBtn}
              onClick={() => this.handleEditCourse(selectedRows)}
            >
              批量完善课程信息
            </Button>
            <SearchBar onSearch={this.search} onReset={this.reset} expand={searchExpand} showExpandButton={false}>
              <Input value={name} placeholder="班课名称" onChange={(e) => this.setState({ name: e.target.value })} />
              <Select
                placeholder="分公司"
                dict={branchDict}
                value={branchId}
                allowClear={false}
                showSearch={true}
                onChange={(branchId) => this.setState({ branchId: branchId, campusId: [] })}
              />
              <MultiSelect
                placeholder="学习中心"
                dict={campusDict}
                value={campusId}
                showSearch={true}
                allowClear={false}
                onChange={(campusId) => this.setState({ campusId: campusId || [] })}
              />
              <Select
                placeholder="季度"
                dict={quarterDict}
                value={quarterId}
                allowClear={true}
                showSearch
                onChange={(quarterId) => this.setState({ quarterId, phaseId: null })}
              />
              <Select
                placeholder={quarterId ? '请选择期数/星期数' : '请先选择季度'}
                dict={phaseOrWeekDict}
                value={phaseId}
                disabled={!quarterId}
                allowClear={true}
                onChange={(phaseId) => this.setState({ phaseId })}
              />
              <MultiSelect
                placeholder="年级"
                dict={gradeDict}
                value={gradeId}
                allowClear={true}
                onChange={(gradeId) => this.setState({ gradeId })}
              />
              <MultiSelect
                placeholder="科目"
                dict={subjetDict}
                value={subjectId}
                allowClear={true}
                onChange={(subjectId) => this.setState({ subjectId })}
              />
              <Select
                placeholder="班级类型"
                dict={classTypeDict}
                value={classTypeId}
                allowClear={true}
                onChange={this.handleClassTypeChange}
              />
              <MultiSelect
                placeholder="班级层次"
                dict={classLevelDict}
                value={classLevel}
                allowClear={true}
                noOptionTips="请先选择班级类型"
                onChange={(classLevel) => this.setState({ classLevel: classLevel || [] })}
              />
              <Select
                placeholder="主讲老师"
                dict={mainTcDict}
                value={mainTeacherId}
                showSearch
                allowClear={true}
                onChange={(mainTeacherId) => this.setState({ mainTeacherId })}
              />
              <Select
                placeholder="是否满班"
                dict={fullDict}
                value={isFull}
                allowClear={true}
                onChange={(isFull) => this.setState({ isFull })}
              />
            </SearchBar>
          </div>
        }
        fitWindowHeight
      >
        <List
          columns={this.columns}
          loading={listLoading}
          dataSource={rows}
          pagination={{
            current: page,
            pageSize,
            total: total,
          }}
          rowSelection={{
            selectedRowKeys,
            onChange: (keys, selectedRows) => this.setState({ selectedRowKeys: keys, selectedRows }),
          }}
          onChange={this.pageChange}
        />
        {showProductModal && (
          <ProductModal
            addProductDict={addProductDict}
            title={`选择${isTwoTeacher ? '双师' : '小班'}课程`}
            onConfirm={this.groupNewOrEditCourse}
            onClose={() => this.setState({ showProductModal: false })}
          />
        )}
        {showShareProductModal && (
          <ShareProductModal
            shareProductDict={shareProductDict}
            title="分享课程列表"
            onConfirm={this.shareUrl}
            onClose={() => this.setState({ showShareProductModal: false })}
          />
        )}
        {shareLinkVisible && (
          <ShareLinkModal
            title={shareTitle}
            shareLink={shareLink}
            onConfirm={() =>
              this.setState({
                shareLinkVisible: false,
                shareLink: null,
                shareTitle: '',
              })
            }
            onClose={() =>
              this.setState({
                shareLinkVisible: false,
                shareLink: null,
                shareTitle: '',
              })
            }
          />
        )}
        <ShareQRCode
          shareTitle={shareTitle}
          shareText={shareLink}
          fileName={shareTitle}
          title="抢先报读课程"
          visible={shareQRCodeVisible}
          onCancel={() =>
            this.setState({
              shareQRCodeVisible: false,
              shareLink: null,
              shareTitle: '',
            })
          }
        />
      </Page>
    );
  }
}

const mapStateToProp = (state) => {
  const { brenchId, campusId, userId } = state.account.user;
  return {
    uBranchId: brenchId || null,
    uCampusId: campusId || null,
    uUserId: userId || null,
    btnAuths: state.account.btnAuths || '',
    activeTabId: state.main.activeTabId,
  };
};

export default connect(mapStateToProp, null)(NewSightUpOnline);
