import {
  //星火1对1小班的
  listMiniClass,
  getClassList,
  getSelectOption,
  loadSmallTc,
  // 双师的
  listTwoTeacherMiniClass,
  getTwoTeacherClassList,
  loadTwoTeacherTc,
  getTwoTeacherSelectOption,
  // 1对1
  listOneToOneMiniClass,
  getOneToOneClassList,
  updateOneToOneCourse,
  // 通用
  cmallGetBranchDict,
  cmallGetCampusByLoginUser,
} from 'SERVICE/signUpOnline';
import { SYSTEM_MARK } from 'CONST/config';

const sys = (function (syetem) {
  const sysMap = {
    XINGHUO: 'XINGHUO',
    TWO_TEACHER: 'TWO_TEACHER',
    ADVANCE: 'ADVANCE',
  };
  return sysMap[syetem] || 'XINGHUO';
})(SYSTEM_MARK);

/*星火小班1对1的*/
const loadMiniList = async (params = {}) => {
  const { pageNo, pageSize, ...extraParams } = params;
  const res = await listMiniClass({
    current: pageNo,
    size: pageSize,
    params: extraParams,
  });
  if (res.code !== 200) {
    throw { text: res.msg };
  }
  return res.data;
};

const loadSmallClassList = async (params = {}) => {
  let { pageNo, pageSize, ...extraParams } = params;
  const classLevelId = extraParams.classLevelId;
  delete extraParams.classLevelId;
  extraParams.classLevel = classLevelId;
  const res = await getClassList({
    current: pageNo,
    size: pageSize,
    params: extraParams,
  });
  if (res.code !== 200) {
    throw { text: res.msg };
  }
  return res.data;
};

const getNewSelectOption = async (selectOptionCategory, parentId) => {
  const params = {
    selectOptionCategory,
    parentId,
  };
  const res = await getSelectOption(params);
  if (res.code !== 200) {
    throw { text: res.msg };
  }
  if (Array.isArray(res.data)) {
    return res.data.map((dict) => {
      const { id, name } = dict;
      return {
        val: id,
        text: name,
      };
    });
  }
  return res.data;
};

const loadSmallClassTeacher = async (params = {}) => {
  const res = await loadSmallTc(params);
  if (res.code !== 200) {
    throw { text: res.msg };
  }
  if (Array.isArray(res.data)) {
    return res.data.map((dict) => {
      const { userId, name } = dict;
      return {
        val: userId,
        text: name,
      };
    });
  }
  return res.data;
};

/*双师的接口*/
const loadTwoTeacherMiniList = async (params = {}) => {
  const { pageNo, pageSize, ...extraParams } = params;
  const res = await listTwoTeacherMiniClass({
    current: pageNo,
    size: pageSize,
    params: extraParams,
  });
  if (res.code !== 200) {
    throw { text: res.msg };
  }
  const { records = [] } = res.data;
  return {
    ...res.data,
    records: records.map((classData) => ({
      ...classData,
      miniClassId: classData.classId,
      assistTeacherName: classData.subTeacherName,
      classTime: classData.classStartTime,
      miniClassPeopleNum: classData.classPeopleNum,
    })),
  };
};

const loadTwoTeacherList = async (params = {}) => {
  let { pageNo, pageSize, ...extraParams } = params;
  const classLevelId = extraParams.classLevelId;
  delete extraParams.classLevelId;
  extraParams.classLevel = classLevelId;
  const res = await getTwoTeacherClassList({
    current: pageNo,
    size: pageSize,
    params: extraParams,
  });
  if (res.code !== 200) {
    throw { text: res.msg };
  }
  const { records = [] } = res.data;
  return {
    ...res.data,
    records: records.map((classData) => {
      const { classId, name, blCampusName, classStartTime, teacherName } = classData;
      const startTime = classStartTime.indexOf(' - ') ? classStartTime.split(' - ')[0] : classStartTime;
      return {
        ...classData,
        miniClassId: classId,
        className: name,
        campusName: blCampusName,
        classStartTime: startTime,
        mainTeacherName: teacherName,
      };
    }),
  };
};

const getTwoTeacherNewSelectOption = async (selectOptionCategory, parentId) => {
  const params = {
    selectOptionCategory,
    parentId,
  };
  const res = await getTwoTeacherSelectOption(params);
  if (res.code !== 200) {
    throw { text: res.msg };
  }
  if (Array.isArray(res.data)) {
    return res.data.map((dict) => {
      const { id, name } = dict;
      return {
        val: id,
        text: name,
      };
    });
  }
  return res.data;
};

const loadTwoTeacherTeacher = async (params = {}) => {
  const res = await loadTwoTeacherTc(params);
  if (res.code !== 200) {
    throw { text: res.msg };
  }
  if (Array.isArray(res.data)) {
    return res.data.map((dict) => {
      const { userId, name } = dict;
      return {
        val: userId,
        text: name,
      };
    });
  }
  return res.data;
};

/*1对1*/
const loadOTOMiniList = async (params = {}) => {
  const { pageNo, pageSize, ...extraParams } = params;
  const res = await listOneToOneMiniClass({
    current: pageNo,
    size: pageSize,
    params: extraParams,
  });
  if (res.code !== 200) {
    throw { text: res.msg };
  }
  const { records = [] } = res.data;
  return {
    ...res.data,
    records: records.map((classData) => ({ ...classData, miniClassId: classData.productId })),
  };
};

const loadOTOList = async (params = {}) => {
  let { pageNo, pageSize, ...extraParams } = params;
  const classLevelId = extraParams.classLevelId;
  delete extraParams.classLevelId;
  extraParams.classLevel = classLevelId;
  const res = await getOneToOneClassList({
    current: pageNo,
    size: pageSize,
    params: extraParams,
  });
  if (res.code !== 200) {
    throw { text: res.msg };
  }
  const { records = [] } = res.data;
  return {
    ...res.data,
    records: records.map((classData) => ({ ...classData, miniClassId: classData.productId })),
  };
};

const updateOtOStatus = async (params = {}) => {
  const res = await updateOneToOneCourse(params);
  if (res.code !== 200) {
    throw { text: res.msg };
  }
};

/**
 * 加载当前用户所属分公司字典
 */
const getBranchDict = async () => {
  const res = await cmallGetBranchDict({ orgType: 'BRENCH', orgId: '', sys }, true);
  if (res.code !== 200) {
    throw { text: res.msg };
  }
  if (Array.isArray(res.data)) {
    return res.data
      .filter((dict) => dict.value)
      .map((dict) => {
        const { value, name } = dict;
        return {
          val: value,
          text: name,
        };
      });
  }
  return res.data;
};

/**
 * 加载当前用户所属分公司字典
 */
const getBranchCampusDict = async () => {
  const res = await cmallGetCampusByLoginUser({ sys });
  if (res.code !== 200) {
    throw { text: res.msg };
  }
  if (Array.isArray(res.data)) {
    return res.data.map((group) => {
      const branch = group[0] || {};
      const campuses = group && group.length > 1 ? group.slice(1) : [];
      return {
        val: branch.id,
        label: branch.name,
        options: campuses.map((d) => {
          return { val: d.id, text: d.name };
        }),
      };
    });
  }
  return res.data;
};

export {
  loadMiniList,
  loadSmallClassList,
  getNewSelectOption,
  loadSmallClassTeacher,
  /*双师*/
  loadTwoTeacherMiniList,
  loadTwoTeacherList,
  getTwoTeacherNewSelectOption,
  loadTwoTeacherTeacher,
  // 1对1
  loadOTOMiniList,
  loadOTOList,
  updateOtOStatus,
  /*通用*/
  getBranchDict,
  getBranchCampusDict,
};
