import React from 'react';
import Modal from 'COMPONENT/Modal';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import SearchBar from 'COMPONENT/SearchBar';
import Input from 'COMPONENT/Input';
import Loading from 'COMPONENT/Loading';
import ShowMessage from 'COMPONENT/utils/globalTips/showMessage';
import { Card, Col, Pagination, Row } from 'antd';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import { transformArrayToString } from 'UTIL/common';
import commonStyles from '../../common/commonStyle.less';
import 'COMPONENT/Page/styles.less';
import styles from '../styles.less';
import moment from 'moment';
import { oneToOneLoader } from '../utils';

const pageSizeOptions = ['10', '20', '50'];

const CATEGORY_NAME = '1对1';
class OneToOneProductModal extends React.Component {
  constructor(props) {
    super(props);
    const { addProductDict, productTypeId } = this.props;
    const { gradeDict, branchDict } = addProductDict;
    const branchId = branchDict[0].val;
    this.state = {
      loading: true,
      searchExpand: true,
      listLoading: false,
      selectedList: [],
      selectedListObj: {},
      name: null,
      list: [],
      productGrade: null,
      pageNo: 1,
      pageSize: 10,
      total: null,
      gradeDict,
      branchDict,
      branchId,
    };
  }

  componentDidMount() {
    this.initContent();
  }

  initContent = async () => {
    this.setState({ loading: true });

    const list = await this.loadListData();
    this.setState({
      list: list.records,
      total: list.total,
      pageNo: list.current,
      loading: false,
    });
  };

  loadListData = async (params) => {
    const { name, branchId, productGrade, pageSize, pageNo } = this.state;

    const apiParams = {
      branchId,
      name,
      gradeId: transformArrayToString(productGrade),
      pageSize,
      pageNo,
      ...params,
    };
    try {
      return await oneToOneLoader.loadClassList(apiParams);
    } catch (e) {
      ShowMessage(e.text);
      return [];
    }
  };

  search = async () => {
    this.setState({ listLoading: true });
    const list = await this.loadListData({
      pageNo: 1,
    });
    this.setState({
      listLoading: false,
      list: list.records,
      total: list.total,
      pageNo: list.current,
    });
  };

  pageChange = async (current, pageSize) => {
    this.setState({ listLoading: true });
    const list = await this.loadListData({
      pageSize,
      pageNo: current,
    });
    this.setState({
      listLoading: false,
      list: list.records,
      total: list.total,
      pageNo: list.current,
      pageSize,
    });
  };

  reset = () => {
    this.setState({
      name: '',
      productGrade: [],
    });
  };

  addProduct = (product) => {
    const { selectedList, selectedListObj } = this.state;
    this.setState({
      selectedList: [...selectedList, product],
      selectedListObj: {
        ...selectedListObj,
        [product.miniClassId]: product,
      },
    });
  };

  removeProduct = (product) => {
    const { selectedList, selectedListObj } = this.state;
    //todo
    const newSelectedList = selectedList.filter((ele) => ele.miniClassId !== product.miniClassId);
    delete selectedListObj[product.miniClassId];
    this.setState({
      selectedList: newSelectedList,
    });
  };

  onCommit = async () => {
    this.setState({ loading: true });
    try {
      const { onConfirm } = this.props;
      const { selectedList } = this.state;
      if (!selectedList.length) {
        ShowMessage('请选择1对1产品', 'error');
        this.setState({ loading: false });
        return false;
      }
      onConfirm(selectedList);
    } catch (e) {
      this.setState({ loading: false });
    }
  };

  selectCardRender = (selectProduct) => {
    const { gradeName } = selectProduct;
    return gradeName;
  };

  render() {
    const { onClose, title } = this.props;
    const {
      loading,
      searchExpand,
      name,
      gradeDict,
      branchDict,
      productGrade,
      branchId,
      list,
      pageNo,
      pageSize,
      total,
      listLoading,
      selectedList,
      selectedListObj,
    } = this.state;

    return (
      <Modal
        loading={loading}
        onCancel={onClose}
        okText="确认选择"
        cancelText="关闭"
        onOk={this.onCommit}
        themeConfig={{
          bgc: 'gray',
          footerTopShadow: true,
        }}
        widthSize="XL"
        ref={(c) => (this.modal = getWrappedInstance(c))}
        className={styles.newCmallProduct}
      >
        <div className={commonStyles.wrap + ' ' + styles.container}>
          <div className={'flexWrap' + ' ' + styles.content}>
            <h2 style={{ padding: '15px 0px 15px', fontSize: '18px' }}>{title}</h2>
            <div className="Page-scroller" style={{ margin: '0', width: 'calc(100% + 28px)', paddingRight: '18px' }}>
              <div className={'custom-SearchBar ' + styles.searchBar} style={{ marginBottom: '5px' }}>
                <SearchBar
                  onSearch={this.search}
                  onReset={this.reset}
                  expand={searchExpand}
                  onToggleExpand={(expand) => this.setState({ searchExpand: expand })}
                >
                  <Input
                    placeholder="产品名称"
                    value={name}
                    onChange={(e) => this.setState({ name: e.target.value })}
                  />
                  <Select
                    placeholder="分公司"
                    dict={branchDict}
                    value={branchId}
                    showSearch={false}
                    allowClear={false}
                    onChange={(branchId) => this.setState({ branchId, productCampus: [] })}
                  />
                  <MultiSelect
                    placeholder="年级"
                    dict={gradeDict}
                    value={productGrade}
                    allowClear={true}
                    onChange={(gradeId) => this.setState({ productGrade: gradeId || [] })}
                  />
                </SearchBar>
              </div>
              {listLoading ? (
                <div className="custom-loading">
                  <Loading loading={true} />
                </div>
              ) : (
                <div className={'flexInnerFit'}>
                  {list.map((ele) => {
                    const { name, productGroupName, validStartTime, gradeName, validEndTime } = ele;
                    return (
                      <div key={ele.miniClassId} className={styles.card + ' custom-card'}>
                        <Card style={{ width: '100%' }} bodyStyle={{ padding: '22px 20px', display: 'flex' }}>
                          <div className={styles.cardContent}>
                            <div className={styles.cardHeader}>
                              <span className={styles.cardMarkup}>{CATEGORY_NAME}</span>
                              <span className={styles.cardTitle}>{name}</span>
                            </div>
                            <div className="custom-row">
                              <Row>
                                <Col span={6}>
                                  <span className={styles.cardContext}>年级：{gradeName || '无'}</span>
                                </Col>
                                <Col span={6}>
                                  <span className={styles.cardContext}>产品组：{productGroupName || '无'}</span>
                                </Col>
                                <Col span={6}>
                                  <span className={styles.cardContext}>生效日期：{validStartTime || '无'}</span>
                                </Col>
                                <Col span={6}>
                                  <span className={styles.cardContext}>截止日期：{validEndTime || '无'}</span>
                                </Col>
                              </Row>
                            </div>
                          </div>
                          <div className={styles.cardAction}>
                            {selectedListObj[ele.miniClassId] ? (
                              <div className={styles.cardButton + ' ' + styles.disabled}>已选</div>
                            ) : (
                              <div
                                className={styles.cardButton}
                                onClick={() => {
                                  this.addProduct(ele);
                                }}
                              >
                                选择
                              </div>
                            )}
                          </div>
                        </Card>
                      </div>
                    );
                  })}
                  {!list.length ? (
                    ''
                  ) : (
                    <div style={{ marginRight: '30px' }}>
                      <Pagination
                        current={pageNo}
                        total={total}
                        onChange={this.pageChange}
                        showSizeChanger
                        onShowSizeChange={this.pageChange}
                        pageSizeOptions={pageSizeOptions}
                        pageSize={pageSize}
                        showTotal={(total) => `共 ${total} 条`}
                      />
                    </div>
                  )}
                </div>
              )}
            </div>
          </div>
          <div className={styles.vrContainer}>
            <div className={styles.vr}></div>
          </div>
          <div className={'flexWrap' + ' ' + styles.content}>
            <h2 style={{ padding: '15px 0px', fontSize: '18px' }}>已选产品（{selectedList.length}）</h2>
            <div className={'flexInnerFit'}>
              <div className="Page-scroller" style={{ margin: '0', width: 'calc(100% + 20px)', paddingRight: '10px' }}>
                <div className={'flexInnerFit'}>
                  {selectedList.map((ele) => {
                    const { name } = ele;
                    return (
                      <div key={ele.miniClassId} className={styles.card + ' custom-card'}>
                        <Card style={{ width: '100%' }} bodyStyle={{ padding: '22px 20px', display: 'flex' }}>
                          <div className={styles.cardContent}>
                            <div className={styles.cardHeader}>
                              <span className={styles.cardMarkup}>{CATEGORY_NAME}</span>
                              <span className={styles.cardTitle}>{name}</span>
                            </div>
                            <div className={'custom-row ' + styles.cardContext}>{this.selectCardRender(ele)}</div>
                          </div>
                          <div
                            className={styles.deleteBtn}
                            onClick={() => {
                              this.removeProduct(ele);
                            }}
                          ></div>
                        </Card>
                      </div>
                    );
                  })}
                </div>
              </div>
            </div>
          </div>
        </div>
      </Modal>
    );
  }
}

export default OneToOneProductModal;
