import React from 'react';
import Modal from 'COMPONENT/Modal';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import SearchBar from 'COMPONENT/SearchBar';
import Input from 'COMPONENT/Input';
import Loading from 'COMPONENT/Loading';
import ShowMessage from 'COMPONENT/utils/globalTips/showMessage';
import { Card, Col, Pagination, Row } from 'antd';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import { transformArrayToString } from 'UTIL/common';
import commonStyles from '../../common/commonStyle.less';
import 'COMPONENT/Page/styles.less';
import styles from '../styles.less';
import { SYSTEM_MARK } from '../../../../constants/config';
import moment from 'moment';
import { newCmallLoader } from '../utils';
// 是否双师
const isTwoTeacher = SYSTEM_MARK === 'TWO_TEACHER';

const pageSizeOptions = ['10', '20', '50'];

const CATEGORY_NAME = isTwoTeacher ? '双师' : '小班';
const defaultCategory = isTwoTeacher ? 'TWO_TEACHER' : 'SMALL_CLASS';
class ProductModal extends React.Component {
  constructor(props) {
    super(props);
    const { addProductDict, productTypeId } = this.props;
    const { branchCampusDict, yearDict, gradeDict, subjetDict, quarterDict, branchDict, classTypeDict } =
      addProductDict;
    const branchId = branchDict[0].val;
    const currentProductVersion = yearDict.find((y) => +y.text === new Date().getFullYear()).val;
    this.state = {
      loading: true,
      searchExpand: true,
      listLoading: false,
      selectedList: [],
      selectedListObj: {},
      name: null,
      productQuarter: null,
      productVersion: [currentProductVersion],
      classTypeId: null,
      classLevel: null,
      productTypeId,
      list: [],
      pageNo: 1,
      pageSize: 10,
      total: null,
      gradeDict,
      subjetDict,
      branchCampusDict,
      branchDict,
      branchId,
      yearDict,
      quarterDict,
      classTypeDict,
    };
  }

  componentDidMount() {
    this.initContent();
  }

  initContent = async () => {
    this.setState({ loading: true });

    const list = await this.loadListData();
    this.setState({
      list: list.records,
      total: list.total,
      pageNo: list.current,
      loading: false,
    });
  };

  loadListData = async (params) => {
    const {
      name,
      branchId,
      productCampus,
      productVersion,
      productQuarter,
      productSubject,
      classTypeId,
      productGrade,
      classLevel,
      branchCampusDict,
      pageSize,
      pageNo,
    } = this.state;
    const trueCampusDict = branchId ? branchCampusDict && branchCampusDict.filter((item) => branchId === item.val) : [];
    const trueCampusId = productCampus || trueCampusDict[0].options.map((campus) => campus.val);

    const apiParams = {
      branchId,
      className: name,
      campusId: transformArrayToString(trueCampusId),
      yearId: transformArrayToString(productVersion),
      quarterId: transformArrayToString(productQuarter),
      subjectId: transformArrayToString(productSubject),
      gradeId: transformArrayToString(productGrade),
      classTypeId: classTypeId,
      classLevelId: transformArrayToString(classLevel),
      pageSize,
      pageNo,
      ...params,
    };
    try {
      return await newCmallLoader.loadClassList(apiParams);
    } catch (e) {
      ShowMessage(e.text);
      return [];
    }
  };

  search = async () => {
    this.setState({ listLoading: true });
    const list = await this.loadListData({
      pageNo: 1,
    });
    this.setState({
      listLoading: false,
      list: list.records,
      total: list.total,
      pageNo: list.current,
    });
  };

  pageChange = async (current, pageSize) => {
    this.setState({ listLoading: true });
    const list = await this.loadListData({
      pageSize,
      pageNo: current,
    });
    this.setState({
      listLoading: false,
      list: list.records,
      total: list.total,
      pageNo: list.current,
      pageSize,
    });
  };

  reset = () => {
    this.setState({
      name: '',
      productCampus: null,
      productYear: null,
      productQuarter: null,
      productSubject: null,
      classTypeId: null,
      productGrade: [],
      classLevel: null,
      productVersion: [],
    });
  };

  addProduct = (product) => {
    const { selectedList, selectedListObj } = this.state;
    this.setState({
      selectedList: [...selectedList, product],
      selectedListObj: {
        ...selectedListObj,
        [product.miniClassId]: product,
      },
    });
  };

  removeProduct = (product) => {
    const { selectedList, selectedListObj } = this.state;
    const newSelectedList = selectedList.filter((ele) => ele.miniClassId !== product.miniClassId);
    delete selectedListObj[product.miniClassId];
    this.setState({
      selectedList: newSelectedList,
    });
  };

  onCommit = async () => {
    this.setState({ loading: true });
    try {
      const { onConfirm } = this.props;
      const { selectedList } = this.state;
      if (!selectedList.length) {
        ShowMessage('请选择课程', 'error');
        this.setState({ loading: false });
        return false;
      }
      onConfirm(selectedList);
    } catch (e) {
      this.setState({ loading: false });
    }
  };

  selectCardRender = (selectProduct) => {
    const { gradeName, subjectName, quarterName, yearName } = selectProduct;
    return (
      `${gradeName ? gradeName + '·' : ''}` +
      `${yearName}年·` +
      `${quarterName ? quarterName + '·' : ''}` +
      `${subjectName}`
    );
  };
  /**
   * 班级类型改变，获取班级层次字典（班级类型与班级层次联动）
   */
  handleClassTypeChange = async (classTypeId) => {
    this.setState({ classTypeId, classLevelDict: null, classLevel: null, loading: true });
    let classLevelDict = await newCmallLoader.getSelectOption('CLASS_LEVEL', classTypeId);
    this.setState({ classLevelDict, loading: false });
  };

  render() {
    const { onClose, title } = this.props;
    const {
      loading,
      searchExpand,
      name,
      productQuarter,
      productVersion,
      yearDict,
      quarterDict,
      gradeDict,
      subjetDict,
      branchCampusDict,
      branchDict,
      classTypeDict,
      classLevelDict,
      classLevel,
      productGrade,
      productSubject,
      classTypeId,
      productCampus,
      branchId,
      list,
      pageNo,
      pageSize,
      total,
      listLoading,
      selectedList,
      selectedListObj,
    } = this.state;

    const campusDict = branchId ? branchCampusDict && branchCampusDict.filter((item) => branchId === item.val) : [];
    return (
      <Modal
        loading={loading}
        onCancel={onClose}
        okText="确认选择"
        cancelText="关闭"
        onOk={this.onCommit}
        themeConfig={{
          bgc: 'gray',
          footerTopShadow: true,
        }}
        widthSize="XL"
        ref={(c) => (this.modal = getWrappedInstance(c))}
        className={styles.newCmallProduct}
      >
        <div className={commonStyles.wrap + ' ' + styles.container}>
          <div className={'flexWrap' + ' ' + styles.content}>
            <h2 style={{ padding: '15px 0px 15px', fontSize: '18px' }}>{title}</h2>
            <div className="Page-scroller" style={{ margin: '0', width: 'calc(100% + 28px)', paddingRight: '18px' }}>
              <div className={'custom-SearchBar ' + styles.searchBar} style={{ marginBottom: '5px' }}>
                <SearchBar
                  onSearch={this.search}
                  onReset={this.reset}
                  expand={searchExpand}
                  onToggleExpand={(expand) => this.setState({ searchExpand: expand })}
                >
                  <Input placeholder="名称" value={name} onChange={(e) => this.setState({ name: e.target.value })} />
                  <Select
                    placeholder="分公司"
                    dict={branchDict}
                    value={branchId}
                    allowClear={false}
                    showSearch={true}
                    onChange={(branchId) => this.setState({ branchId, productCampus: [] })}
                  />
                  <MultiSelect
                    placeholder="学习中心"
                    dict={campusDict}
                    value={productCampus}
                    allowClear={true}
                    onChange={(campusId) => this.setState({ productCampus: campusId || [] })}
                  />
                  <MultiSelect
                    placeholder="年份"
                    dict={yearDict}
                    value={productVersion}
                    onChange={(productVersion) => this.setState({ productVersion: productVersion || [] })}
                    allowClear={false}
                  />
                  <MultiSelect
                    placeholder="季度"
                    dict={quarterDict}
                    value={productQuarter}
                    allowClear={true}
                    onChange={(productQuarter) => this.setState({ productQuarter: productQuarter || [] })}
                  />
                  <MultiSelect
                    placeholder="年级"
                    dict={gradeDict}
                    value={productGrade}
                    allowClear={true}
                    onChange={(gradeId) => this.setState({ productGrade: gradeId || [] })}
                  />
                  <MultiSelect
                    placeholder="科目"
                    dict={subjetDict}
                    value={productSubject}
                    allowClear={true}
                    onChange={(subjectId) => this.setState({ productSubject: subjectId || [] })}
                  />
                  <Select
                    placeholder="班级类型"
                    dict={classTypeDict}
                    value={classTypeId}
                    onChange={this.handleClassTypeChange}
                  />
                  <MultiSelect
                    placeholder="班级层次"
                    dict={classLevelDict}
                    value={classLevel}
                    allowClear={true}
                    noOptionTips="请先选择班级类型"
                    onChange={(classLevel) => this.setState({ classLevel: classLevel || [] })}
                  />
                </SearchBar>
              </div>
              {listLoading ? (
                <div className="custom-loading">
                  <Loading loading={true} />
                </div>
              ) : (
                <div className={'flexInnerFit'}>
                  {list.map((ele) => {
                    const {
                      className,
                      phaseName,
                      campusName,
                      gradeName,
                      subjectName,
                      classLevelName,
                      classStartDate,
                      classEndDate,
                      classStartTime,
                      classTimeLength,
                      everyCourseClassNum = 1,
                      mainTeacherName,
                    } = ele;
                    return (
                      <div key={ele.miniClassId} className={styles.card + ' custom-card'}>
                        <Card style={{ width: '100%' }} bodyStyle={{ padding: '22px 20px', display: 'flex' }}>
                          <div className={styles.cardContent}>
                            <div className={styles.cardHeader}>
                              <span className={styles.cardMarkup}>{CATEGORY_NAME}</span>
                              <span className={styles.cardTitle}>{className}</span>
                            </div>
                            <div className="custom-row">
                              <Row>
                                <Col span={4}>
                                  <span className={styles.cardContext}>期数：{phaseName || '无'}</span>
                                </Col>
                                <Col span={4}>
                                  <span className={styles.cardContext}>学习中心：{campusName}</span>
                                </Col>
                                <Col span={4}>
                                  <span className={styles.cardContext}>年级：{gradeName || '无'}</span>
                                </Col>
                                <Col span={4}>
                                  <span className={styles.cardContext}>科目：{subjectName}</span>
                                </Col>
                                <Col span={4}>
                                  <span className={styles.cardContext}>班级层次：{classLevelName}</span>
                                </Col>
                              </Row>
                              <Row>
                                <Col span={8}>
                                  <span className={styles.cardContext}>
                                    上课日期：{classStartDate + (classEndDate ? ' ~ ' + classEndDate : '')}
                                  </span>
                                </Col>
                                <Col span={8}>
                                  <span className={styles.cardContext}>
                                    上课时间：
                                    {classStartTime +
                                      ' ~ ' +
                                      moment(classStartTime, 'HH:mm')
                                        .add(classTimeLength * everyCourseClassNum, 'm')
                                        .format('HH:mm')}
                                  </span>
                                </Col>
                                <Col span={8}>
                                  <span className={styles.cardContext}>授课老师：{mainTeacherName}</span>
                                </Col>
                              </Row>
                            </div>
                          </div>
                          <div className={styles.cardAction}>
                            {selectedListObj[ele.miniClassId] ? (
                              <div className={styles.cardButton + ' ' + styles.disabled}>已选</div>
                            ) : (
                              <div
                                className={styles.cardButton}
                                onClick={() => {
                                  this.addProduct(ele);
                                }}
                              >
                                选择
                              </div>
                            )}
                          </div>
                        </Card>
                      </div>
                    );
                  })}
                  {!list.length ? (
                    ''
                  ) : (
                    <div style={{ marginRight: '30px' }}>
                      <Pagination
                        current={pageNo}
                        total={total}
                        onChange={this.pageChange}
                        showSizeChanger
                        onShowSizeChange={this.pageChange}
                        pageSizeOptions={pageSizeOptions}
                        pageSize={pageSize}
                        showTotal={(total) => `共 ${total} 条`}
                      />
                    </div>
                  )}
                </div>
              )}
            </div>
          </div>
          <div className={styles.vrContainer}>
            <div className={styles.vr}></div>
          </div>
          <div className={'flexWrap' + ' ' + styles.content}>
            <h2 style={{ padding: '15px 0px', fontSize: '18px' }}>已选课程（{selectedList.length}）</h2>
            <div className={'flexInnerFit'}>
              <div className="Page-scroller" style={{ margin: '0', width: 'calc(100% + 20px)', paddingRight: '10px' }}>
                <div className={'flexInnerFit'}>
                  {selectedList.map((ele) => {
                    const { className } = ele;
                    return (
                      <div key={ele.miniClassId} className={styles.card + ' custom-card'}>
                        <Card style={{ width: '100%' }} bodyStyle={{ padding: '22px 20px', display: 'flex' }}>
                          <div className={styles.cardContent}>
                            <div className={styles.cardHeader}>
                              <span className={styles.cardMarkup}>{CATEGORY_NAME}</span>
                              <span className={styles.cardTitle}>{className}</span>
                            </div>
                            <div className={'custom-row ' + styles.cardContext}>{this.selectCardRender(ele)}</div>
                          </div>
                          <div
                            className={styles.deleteBtn}
                            onClick={() => {
                              this.removeProduct(ele);
                            }}
                          ></div>
                        </Card>
                      </div>
                    );
                  })}
                </div>
              </div>
            </div>
          </div>
        </div>
      </Modal>
    );
  }
}

export default ProductModal;
