import React from 'react';
import Modal from 'COMPONENT/Modal';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import SearchBar from 'COMPONENT/SearchBar';
import Loading from 'COMPONENT/Loading';
import Button from 'COMPONENT/Button';
import ShowMessage from 'COMPONENT/utils/globalTips/showMessage';
import { Card, Col, Pagination, Row, message } from 'antd';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import { transformArrayToString } from 'UTIL/common';
import courseTimeDict from '../utils/getTimeDict';
import commonStyles from '../../common/commonStyle.less';
import 'COMPONENT/Page/styles.less';
import styles from './styles.less';
import { SYSTEM_MARK } from 'CONST/config';
import moment from 'moment';
import { newCmallLoader } from '../utils';
import { getDictTextByVal } from 'UTIL/dict';
import getXhmallShareUrl from '../utils/getXhmallShareUrl';
// 是否双师
const isTwoTeacher = SYSTEM_MARK === 'TWO_TEACHER';
const spingId = 'DAT0000000231'; //春季id
const autumnId = 'DAT0000000233'; //秋季id

const pageSizeOptions = ['10', '20', '50'];
function arrayToMap(arr, obj = {}) {
  if (Array.isArray(arr)) {
    return arr.reduce((obj, cur) => {
      return {
        [cur.val]: cur,
        ...obj,
      };
    }, obj);
  }
  return obj;
}

const CATEGORY_NAME = isTwoTeacher ? '双师' : '小班';
class ShareProductModal extends React.Component {
  constructor(props) {
    super(props);
    const { shareProductDict, productTypeId } = this.props;
    const {
      branchCampusDict,
      yearDict,
      gradeDict,
      subjetDict,
      quarterDict,
      branchDict,
      weekDict,
      phaseDict,
      mainTcDict,
      fullDict,
    } = shareProductDict;
    const branchId = branchDict[0].val;
    const phaseOrWeekMap = arrayToMap(weekDict, arrayToMap(phaseDict));

    this.state = {
      loading: true,
      searchExpand: true,
      listLoading: false,
      name: null,
      productQuarter: null,
      mainTeacherId: null,
      courseTime: null,
      isFull: null,
      phaseId: null,
      phaseOrWeekMap,

      list: [],
      pageNo: 1,
      pageSize: 10,
      total: null,
      gradeDict,
      subjetDict,
      branchCampusDict,
      branchDict,
      branchId,
      yearDict,
      quarterDict,
      weekDict,
      phaseDict,
      mainTcDict,
      fullDict,
    };
  }

  componentDidMount() {
    this.initContent();
  }

  initContent = async () => {
    this.setState({ loading: true });

    const list = await this.loadListData();
    this.setState({
      list: list.records,
      total: list.total,
      pageNo: list.current,
      loading: false,
    });
  };

  // 获取api参数
  getApiParams = (params = {}) => {
    const {
      branchId,
      productCampus,
      productQuarter,
      productSubject,
      courseTime,
      productGrade,
      mainTeacherId,
      isFull,
      phaseId,
      pageSize,
      pageNo,
    } = this.state;
    const trueCampusId = productCampus ? [productCampus] : [];

    const apiParams = {
      branchId,
      blCampusId: transformArrayToString(trueCampusId),
      productQuarterId: productQuarter,
      subjectId: productSubject,
      courseTime: courseTime,
      gradeId: productGrade,
      teacherId: mainTeacherId,
      isFull,
      phaseId,
      pageSize,
      pageNo,
      ...params,
    };
    isFull && (apiParams.isClassFull = isFull);
    return apiParams;
  };

  // 获取分享参数
  getShareApiParams = () => {
    const {
      branchId,
      productCampus,
      productQuarter,
      productSubject,
      courseTime,
      productGrade,
      mainTeacherId,
      isFull,
      phaseId,
      gradeDict,
    } = this.state;
    const trueCampusId = productCampus ? [productCampus] : [];

    const gradeName = productGrade ? getDictTextByVal(gradeDict, productGrade) : null;
    return {
      branchId,
      blCampusId: transformArrayToString(trueCampusId),
      quarterId: productQuarter,
      subjectId: productSubject,
      courseTime: courseTime,
      gradeId: productGrade,
      teacherId: mainTeacherId,
      fullFlag: isFull === '0' ? 'true' : 'false', // 外面给的 val 就是这样的，虽然这么写不规范
      phaseId,
      gradeName: encodeURIComponent(gradeName),
    };
  };

  loadListData = async (params) => {
    const apiParams = this.getApiParams(params);
    try {
      return await newCmallLoader.loadList(apiParams);
    } catch (e) {
      ShowMessage(e.text);
      return [];
    }
  };

  search = async () => {
    this.setState({ listLoading: true });
    const list = await this.loadListData({
      pageNo: 1,
    });
    this.setState({
      listLoading: false,
      list: list.records,
      total: list.total,
      pageNo: list.current,
    });
  };

  pageChange = async (current, pageSize) => {
    this.setState({ listLoading: true });
    const list = await this.loadListData({
      pageSize,
      pageNo: current,
    });
    this.setState({
      listLoading: false,
      list: list.records,
      total: list.total,
      pageNo: list.current,
      pageSize,
    });
  };

  reset = () => {
    this.setState({
      productCampus: null,
      productYear: null,
      productQuarter: null,
      productSubject: null,
      courseTime: null,
      productGrade: null,
      mainTeacherId: null,
      isFull: null,
      phaseId: null,
    });
  };

  onCommit = async () => {
    this.setState({ loading: true });
    try {
      const { onConfirm } = this.props;
      const { selectedList } = this.state;
      if (!selectedList.length) {
        ShowMessage('请选择课程', 'error');
        this.setState({ loading: false });
        return false;
      }
      onConfirm(selectedList);
    } catch (e) {
      this.setState({ loading: false });
    }
  };

  // 获取分享二维码链接
  getShareLink = () => {
    const extraParamsMap = this.getShareApiParams();
    return getXhmallShareUrl({
      url: 'mapp/normal-sales/course/selection',
      extraParams: extraParamsMap,
    });
  };

  checkAndConfirm = (options) => {
    const { onConfirm } = this.props;
    const { productGrade } = this.state;
    if (!productGrade) {
      return message.warn('请选择年级');
    }
    onConfirm(options);
  };

  render() {
    const { onClose, title } = this.props;
    const {
      loading,
      searchExpand,
      productQuarter,
      quarterDict,
      gradeDict,
      subjetDict,
      branchCampusDict,
      branchDict,
      weekDict,
      phaseDict,
      mainTcDict,
      fullDict,
      productGrade,
      productSubject,
      courseTime,
      productCampus,
      branchId,
      mainTeacherId,
      isFull,
      phaseId,
      list,
      pageNo,
      pageSize,
      total,
      listLoading,
      phaseOrWeekMap,
    } = this.state;
    let phaseOrWeekDict = [];
    if (productQuarter) {
      if (productQuarter === autumnId || productQuarter === spingId) {
        phaseOrWeekDict = weekDict;
      } else {
        phaseOrWeekDict = phaseDict;
      }
    }

    const campusDict = branchId ? branchCampusDict && branchCampusDict.filter((item) => branchId === item.val) : [];

    const shareUrl = this.getShareLink();
    return (
      <Modal
        loading={loading}
        onCancel={onClose}
        onOk={this.onCommit}
        themeConfig={{
          bgc: 'gray',
          footerTopShadow: true,
        }}
        width={1000}
        ref={(c) => (this.modal = getWrappedInstance(c))}
        className={styles.newShareProduct}
        footer={
          <div>
            <Button
              type="primary"
              onClick={() =>
                this.checkAndConfirm({
                  shareLink: shareUrl,
                  type: 'qr',
                  title: '分享-课程二维码',
                })
              }
            >
              分享二维码
            </Button>
            <Button
              type="primary"
              onClick={() =>
                this.checkAndConfirm({
                  shareLink: shareUrl,
                  type: 'link',
                  title: '分享-课程链接',
                })
              }
            >
              分享链接
            </Button>
            <Button onClick={onClose}>取消</Button>
          </div>
        }
      >
        <div className={commonStyles.wrap + ' ' + styles.container}>
          <div className={'flexWrap' + ' ' + styles.content}>
            <h2>{title}</h2>
            <div className="shareProductWrapper" style={{ margin: '0', paddingRight: '18px' }}>
              <div className={'custom-SearchBar ' + styles.searchBar} style={{ marginBottom: '5px' }}>
                <SearchBar
                  onSearch={this.search}
                  onReset={this.reset}
                  expand={searchExpand}
                  onToggleExpand={(expand) => this.setState({ searchExpand: expand })}
                >
                  <Select
                    placeholder="分公司"
                    dict={branchDict}
                    value={branchId}
                    allowClear={false}
                    showSearch={true}
                    onChange={(branchId) => this.setState({ branchId, productCampus: null })}
                  />
                  <Select
                    placeholder="学习中心"
                    dict={campusDict}
                    value={productCampus}
                    allowClear={true}
                    onChange={(campusId) => this.setState({ productCampus: campusId })}
                  />
                  <Select
                    placeholder="季度"
                    dict={quarterDict}
                    value={productQuarter}
                    allowClear={true}
                    onChange={(productQuarter) => this.setState({ productQuarter })}
                  />
                  <Select
                    placeholder={productQuarter ? '请选择期数/星期数' : '请先选择季度'}
                    dict={phaseOrWeekDict}
                    value={phaseId}
                    disabled={!productQuarter}
                    allowClear={true}
                    onChange={(phaseId) => this.setState({ phaseId: phaseId })}
                  />
                  <Select
                    placeholder="年级"
                    dict={gradeDict}
                    value={productGrade}
                    allowClear={true}
                    onChange={(gradeId) => this.setState({ productGrade: gradeId })}
                  />
                  <Select
                    placeholder="科目"
                    dict={subjetDict}
                    value={productSubject}
                    allowClear={true}
                    onChange={(subjectId) => this.setState({ productSubject: subjectId })}
                  />
                  <Select
                    placeholder="上课时间"
                    dict={courseTimeDict}
                    value={courseTime}
                    allowClear={true}
                    onChange={(courseTime) => this.setState({ courseTime })}
                  />
                  <Select
                    placeholder="主讲老师"
                    dict={mainTcDict}
                    value={mainTeacherId}
                    showSearch
                    allowClear={true}
                    onChange={(mainTeacherId) => this.setState({ mainTeacherId })}
                  />
                  <Select
                    placeholder="是否满班"
                    dict={fullDict}
                    value={isFull}
                    allowClear={true}
                    onChange={(isFull) => this.setState({ isFull })}
                  />
                </SearchBar>
              </div>
              {listLoading ? (
                <div className="custom-loading">
                  <Loading loading={true} />
                </div>
              ) : (
                <div className="Page-scroller">
                  {list.map((ele) => {
                    const {
                      name,
                      phaseId,
                      blCampusName,
                      gradeName,
                      subjectName,
                      classLevelName,
                      startDate,
                      endDate,
                      classTime,
                      classTimeLength,
                      everyCourseClassNum = 1,
                      teacherName,
                    } = ele;

                    let phaseName;
                    if (isTwoTeacher) {
                      phaseName = ele.phaseName || '暂无';
                    } else {
                      phaseName = (phaseOrWeekMap[phaseId] || {}).text || '暂无';
                    }
                    return (
                      <div key={ele.miniClassId} className={styles.card + ' custom-card'}>
                        <Card style={{ width: '100%' }} bodyStyle={{ padding: '22px 20px', display: 'flex' }}>
                          <div className={styles.cardContent}>
                            <div className={styles.cardHeader}>
                              <span className={styles.cardMarkup}>{CATEGORY_NAME}</span>
                              <span className={styles.cardTitle}>{name}</span>
                            </div>
                            <div className="custom-row">
                              <Row>
                                <Col span={4}>
                                  <span className={styles.cardContext}>期数：{phaseName || '无'}</span>
                                </Col>
                                <Col span={4}>
                                  <span className={styles.cardContext}>学习中心：{blCampusName}</span>
                                </Col>
                                <Col span={4}>
                                  <span className={styles.cardContext}>年级：{gradeName || '无'}</span>
                                </Col>
                                <Col span={4}>
                                  <span className={styles.cardContext}>科目：{subjectName}</span>
                                </Col>
                                <Col span={4}>
                                  <span className={styles.cardContext}>班级层次：{classLevelName}</span>
                                </Col>
                              </Row>
                              <Row>
                                <Col span={8}>
                                  <span className={styles.cardContext}>
                                    上课日期：
                                    {startDate + (endDate ? ' ~ ' + endDate : '')}
                                  </span>
                                </Col>
                                <Col span={8}>
                                  <span className={styles.cardContext}>
                                    上课时间：
                                    {classTime +
                                      ' ~ ' +
                                      moment(classTime, 'HH:mm')
                                        .add(classTimeLength * everyCourseClassNum, 'm')
                                        .format('HH:mm')}
                                  </span>
                                </Col>
                                <Col span={8}>
                                  <span className={styles.cardContext}>授课老师：{teacherName}</span>
                                </Col>
                              </Row>
                            </div>
                          </div>
                        </Card>
                      </div>
                    );
                  })}
                </div>
              )}
              {!list.length ? (
                ''
              ) : (
                <div style={{ marginRight: '30px' }}>
                  <Pagination
                    current={pageNo}
                    total={total}
                    onChange={this.pageChange}
                    showSizeChanger
                    onShowSizeChange={this.pageChange}
                    pageSizeOptions={pageSizeOptions}
                    pageSize={pageSize}
                    showTotal={(total) => `共 ${total} 条`}
                  />
                </div>
              )}
            </div>
          </div>
        </div>
      </Modal>
    );
  }
}

export default ShareProductModal;
