import React from 'react';
import { connect } from 'dva';
import Button from 'COMPONENT/Button';
import Page from 'COMPONENT/Page';
import List from 'COMPONENT/List';
import SearchBar from 'COMPONENT/SearchBar';
import Input from 'COMPONENT/Input';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import ShowMessage from 'COMPONENT/utils/globalTips/showMessage';
import commonStyles from '../common/commonStyle.less';
import { renderMultiLine } from '../common';
import OneToOneProductModal from './modal/OneToOneProductModal';
import { oneToOneLoader } from './utils';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
let defaultBranch = null; //默认第一个学习中心

const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL, createSetWidth } = List;

/**
 * 数组转换为字符串，如果入参为空则返回undefinded，否则拼接为字符串
 */
function arrToStr(arr) {
  if (Array.isArray(arr)) {
    return arr.join(',');
  } else {
    return arr;
  }
}

function arrayToMap(arr, obj = {}) {
  if (Array.isArray(arr)) {
    return arr.reduce((obj, cur) => {
      return {
        [cur.val]: cur,
        ...obj,
      };
    }, obj);
  }
  return obj;
}

const onlineStatusDict = [
  {
    text: '已上架',
    val: 1,
  },
  {
    text: '已下架',
    val: 0,
  },
];

class NewSightUpOnline extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: true, // boolean 页面加载状态
      showProductModal: false,
      //查询条件
      name: null,
      gradeId: null,
      onlineStatus: null,
      campusId: [],
      //列表数据
      listLoading: false,
      rows: [],
      page: 1,
      pageSize: 50,
      total: null,
      //数据字典数据
      gradeDict: [],
      branchCampusDict: [],
      branchDict: [],
      //其他
      searchExpand: false,
      selectedRowKeys: [], //选中行的key的集合
      selectedRows: [], // 选中的数据
      // 要编辑产品详情的列表数据
      editProductList: [],
    };
  }

  componentDidMount() {
    // 监听iframe编辑页面传来的刷新页面事件
    window.addEventListener(
      'message',
      async (e) => {
        if (
          e.data &&
          e.data.mark === 'NEXT_COMPACT' &&
          e.data.TYPE === 'SUCCESS' &&
          e.data.productType === 'ONE_TO_ONE'
        ) {
          this.props.dispatch({
            type: 'main/removeTab',
            payload: { id: 'NewSignUpCourseManage/EditCourseCompact' },
          });
          this.props.dispatch({
            type: 'main/selectTab',
            payload: {
              id: '/OTONewSignUpCourseManage',
            },
          });
          //重新加载列表 关闭产品选择模态框
          await this.updateState({ showProductModal: false });
          // 0521版本之后列表页不需要重新刷新
          // await this.updateState({ listLoading: true, page: 1, showProductModal: false })
          // const list = await this.loadListData()
          // this.updateState({
          //   listLoading: false,
          //   rows: list.records,
          //   total: list.total,
          //   page: list.current
          // })
        }
      },
      false,
    );
    this.initContent();
  }

  /**
   * 更新状态
   * @param  {object} changedState 改变的状态
   * @return {Promise}
   */
  updateState = (changedState) => {
    const newState = {
      ...this.state,
      ...changedState,
    };
    return new Promise((resolve) => {
      this.setState(newState, () => {
        resolve();
      });
    });
  };

  /**
   * 获取列表数据
   */
  loadListData = async (params) => {
    const { name, branchId, gradeId, onlineStatus, pageSize, page } = this.state;
    const apiParams = {
      name,
      branchId: arrToStr(branchId),
      gradeId: arrToStr(gradeId),
      onlineStatus,
      pageSize,
      pageNo: page,
      ...params,
    };
    try {
      return await oneToOneLoader.loadList(apiParams);
    } catch (e) {
      ShowMessage(e.text);
    }
  };

  /**
   * 初始化内容
   */
  initContent = async () => {
    const { getSelectOption, getBranchDict } = oneToOneLoader;
    const { uBranchId, uCampusId } = this.props;
    this.setState({ loading: true });
    const asyncCalls = [getBranchDict(), getSelectOption('STUDENT_GRADE')];
    Promise.all(asyncCalls).then(([branchDict, gradeDict]) => {
      //根据用户所属组织初始化分公司及学习中心，当是集团的人，默认取第一个值
      let isGroup = uBranchId === uCampusId;
      //默认第一个分公司，当前年份
      defaultBranch = isGroup ? branchDict[0].val : uBranchId;
      //获取列表数据
      this.loadListData({ branchId: arrToStr(defaultBranch) }).then((list) => {
        this.setState({
          branchId: defaultBranch,
          loading: false,
          rows: list.records,
          total: list.total,
          page: list.current,
          branchDict,
          gradeDict,
        });
      });
    });
  };

  /**
   * 查询列表
   */
  search = async () => {
    await this.updateState({ listLoading: true, page: 1 });
    const list = await this.loadListData();
    this.updateState({
      listLoading: false,
      rows: list.records,
      total: list.total,
      page: list.current,
    });
  };

  /**
   * 重置搜索表单
   */
  reset = async () => {
    this.updateState({
      name: null,
      gradeId: null,
      onlineStatus: null,
    });
  };

  /**
   * 翻页
   */

  pageChange = async ({ current, pageSize }) => {
    await this.updateState({ listLoading: true, pageSize, page: current, selectedRowKeys: [], selectedRows: [] });
    const list = await this.loadListData();
    this.updateState({
      listLoading: false,
      rows: list.records,
      total: list.total,
      page: list.current,
    });
  };

  ProductModalConfirm = (productList) => {
    const { branchId } = this.state;
    const miniClassId = productList.map((product) => product.miniClassId).join(',');
    // 跳转1对1修改页面
    const path = encodeURIComponent('/xhmall/admin/cmall/course/edit/ydy');
    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path: 'NewSignUpCourseManage/EditCourseCompact',
        params: {
          path,
          miniClassId,
          branchId,
        },
        refresh: true,
      },
    });
  };
  handleEditCourse = (record) => {
    const { branchId } = this.state;
    const { id, miniClassId } = record;
    // 跳转修改页面
    const path = encodeURIComponent('/xhmall/admin/cmall/course/edit/ydy');
    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path: 'NewSignUpCourseManage/EditCourseCompact',
        params: {
          path,
          miniClassId,
          branchId,
          courseId: id,
        },
        refresh: true,
      },
    });
  };

  handleCourseStatus = (record) => {
    const { onlineStatus, id } = record;
    showConfirm(`确认${onlineStatus ? '下架' : '上架'}该课程？`, async () => {
      try {
        this.setState({ loading: true });
        await oneToOneLoader.updateOtOStatus({ id, onlineStatus: onlineStatus ? 0 : 1 });
        this.setState({ loading: false });
        await this.search();
      } catch (e) {
        showMessage(e.text, 'error');
        this.setState({ loading: false });
      }
    });
  };
  /**
   *  1对1表头
   */
  columns = [
    { title: '产品名称', dataIndex: 'name', width: COL_WIDTH_XL },
    { title: '分公司', dataIndex: 'branchName', width: COL_WIDTH_M },
    { title: '年级', dataIndex: 'gradeName', width: COL_WIDTH_S, render: renderMultiLine },
    { title: '产品组', dataIndex: 'productGroupName', width: COL_WIDTH_M },
    { title: '生效日期', dataIndex: 'validStartTime', width: COL_WIDTH_M },
    { title: '截止日期', dataIndex: 'validEndTime', width: COL_WIDTH_M },
    {
      title: '是否上架',
      dataIndex: 'onlineStatus',
      width: COL_WIDTH_M,
      render: (text, record) => (record.onlineStatus ? '已上架' : '已下架'),
    },
    {
      title: '操作',
      key: 'action',
      width: COL_WIDTH_L,
      getActionConfig: (text, record) => {
        return [
          { text: '完善产品信息', onClick: () => this.handleEditCourse(record) },
          { text: record.onlineStatus === 1 ? '下架' : '上架', onClick: () => this.handleCourseStatus(record) },
        ];
      },
    },
  ];

  render() {
    const {
      loading,
      searchExpand,
      listLoading,
      rows,
      page,
      pageSize,
      total,
      onlineStatus,
      name,
      gradeId,
      showProductModal,
      branchId,
      branchCampusDict,
      gradeDict,
      selectedRowKeys,
      branchDict,
    } = this.state;

    const addProductDict = {
      branchDict,
      gradeDict,
    };
    return (
      <Page
        loading={loading}
        route={this.props.route}
        subHeader={
          <div className={commonStyles.searchBar} style={{ paddingRight: '81px' }}>
            {
              <Button className={commonStyles.addQuli} onClick={() => this.setState({ showProductModal: true })}>
                新增产品
              </Button>
            }
            <SearchBar
              onSearch={this.search}
              onReset={this.reset}
              expand={searchExpand}
              onToggleExpand={(expand) => this.setState({ searchExpand: expand })}
            >
              <Input value={name} placeholder="产品名称" onChange={(e) => this.setState({ name: e.target.value })} />
              <Select
                placeholder="分公司"
                dict={branchDict}
                value={branchId}
                allowClear={false}
                showSearch={true}
                onChange={(branchId) => this.setState({ branchId: branchId, campusId: [] })}
              />
              <MultiSelect
                placeholder="年级"
                dict={gradeDict}
                value={gradeId}
                allowClear={true}
                onChange={(gradeId) => this.setState({ gradeId })}
              />
              <Select
                placeholder="是否上架"
                dict={onlineStatusDict}
                value={onlineStatus}
                allowClear={true}
                onChange={(onlineStatus) => this.setState({ onlineStatus })}
              />
            </SearchBar>
          </div>
        }
        fitWindowHeight
      >
        <List
          columns={this.columns}
          loading={listLoading}
          dataSource={rows}
          pagination={{
            current: page,
            pageSize,
            total: total,
          }}
          rowSelection={{
            selectedRowKeys,
            onChange: (keys, selectedRows) => this.setState({ selectedRowKeys: keys, selectedRows }),
          }}
          onChange={this.pageChange}
        />
        {showProductModal && (
          <OneToOneProductModal
            addProductDict={addProductDict}
            title="选择1对1产品"
            onConfirm={this.ProductModalConfirm}
            onClose={() => this.setState({ showProductModal: false })}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProp = (state) => {
  const { brenchId, campusId, userId } = state.account.user;
  return {
    uBranchId: brenchId || null,
    uCampusId: campusId || null,
    uUserId: userId || null,
    btnAuths: state.account.btnAuths || '',
    activeTabId: state.main.activeTabId,
  };
};

export default connect(mapStateToProp, null)(NewSightUpOnline);
