import React from 'react';
import { connect } from 'dva';
import Button from 'COMPONENT/Button';
import Page from 'COMPONENT/Page';
import List from 'COMPONENT/List';
import SearchBar from 'COMPONENT/SearchBar';
import Input from 'COMPONENT/Input';
import Select from 'COMPONENT/Select';
import Popover from 'COMPONENT/Popover';
import Icon from 'COMPONENT/Icon';
import EditModal from './editModal/index';
import ShowConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import ShowMessage from 'COMPONENT/utils/globalTips/showMessage';
import { loadCommonDict, loadBranchDict } from 'UTIL/dict';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import Loader from './loader';
import moment from 'moment';
import { PROJECT_NAME } from '../../../constants/config';

import { renderMultiLine } from '../common';

import commonStyles from '../common/commonStyle.less';
import styles from './style.less';

const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;

const validDict = [
  { text: '有效', val: '1' },
  { text: '无效', val: '0' },
];
let defaultBranch = null; //默认第一个学习中心
const productTypeMap = {
  TWO_TEACHER: 'TWO_TEACHER',
  SMALL_CLASS: 'SMALL_CLASS',
};
const produtctTypeString = {
  TWO_TEACHER: '双师',
  SMALL_CLASS: '小班',
};
const productTypeDict = [
  { text: '双师', val: productTypeMap.TWO_TEACHER },
  { text: '小班', val: productTypeMap.SMALL_CLASS },
];
const getValByText = (dict, text) => {
  const item = dict.find((item) => {
    return item.text === text;
  });
  if (item) {
    return item.val;
  } else {
    return null;
  }
};

class SectionPayment extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: true, // boolean 页面加载状态
      //查询条件
      name: null,
      branchId: null,
      productTypeId: PROJECT_NAME === 'twoteacher' ? productTypeMap.TWO_TEACHER : productTypeMap.SMALL_CLASS,
      yearId: null,
      quarterId: null,
      isValid: null,
      //列表数据
      listLoading: false,
      rows: [],
      page: 1,
      pageSize: 20,
      total: null,
      //数据字典数据
      branchDict: null,
      yearDict: null,
      quarterDict: [
        { text: '春季', val: 'DAT0000000231' },
        { text: '秋季', val: 'DAT0000000233' },
      ],
      //其他
      editModalShow: false,
      searchExpand: false,
      editRecord: {},
      editMode: null,
    };
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 更新状态
   * @param  {object} changedState 改变的状态
   * @return {Promise}
   */
  updateState = (changedState) => {
    const newState = {
      ...this.state,
      ...changedState,
    };
    return new Promise((resolve) => {
      this.setState(newState, () => {
        resolve();
      });
    });
  };

  /**
   * 获取列表数据
   */
  loadListData = async (params) => {
    const { name, branchId, productTypeId, yearId, quarterId, pageSize, page, isValid } = this.state;
    const apiParams = {
      name,
      branchId,
      productTypeId,
      yearId,
      quarterId,
      pageSize,
      page,
      isValid,
      pageNo: page,
      ...params,
    };
    try {
      return await Loader.loadList(apiParams);
    } catch (e) {
      ShowMessage(e.text);
    }
  };

  /**
   * 初始化内容
   */
  initContent = async () => {
    this.setState({ loading: true });
    const asyncCalls = [loadBranchDict(), loadCommonDict('PRODUCT_VERSION'), loadCommonDict('PRODUCT_QUARTER')];
    Promise.all(asyncCalls).then(([branchDict, yearDict]) => {
      //默认第一个分公司，当前年份
      defaultBranch = branchDict[0] ? branchDict[0].val : null;
      const defaultYear = getValByText(yearDict, new Date().getFullYear().toString()) || null;
      //获取列表数据
      this.loadListData({ yearId: defaultYear, branchId: defaultBranch }).then((list) => {
        this.setState({
          yearId: defaultYear,
          branchId: defaultBranch,
          loading: false,
          rows: list.datas,
          total: list.rowCount,
          page: list.pageNo,
          branchDict,
          yearDict,
        });
      });
    });
  };

  /**
   * 查询列表
   */
  search = async () => {
    await this.updateState({ listLoading: true, page: 1 });
    const list = await this.loadListData();
    this.updateState({
      listLoading: false,
      rows: list.datas,
      total: list.rowCount,
      page: list.pageNo,
    });
  };

  /**
   * 重置搜索表单
   */
  reset = async () => {
    this.updateState({
      name: null,
      branchId: defaultBranch,
      productTypeId: null,
      yearId: null,
      quarterId: null,
      isValid: null,
    });
  };

  /**
   * 翻页
   */

  pageChange = async ({ current, pageSize }) => {
    await this.updateState({ listLoading: true, pageSize, page: current });
    const list = await this.loadListData();
    this.updateState({
      listLoading: false,
      rows: list.datas,
      total: list.rowCount,
      page: list.pageNo,
    });
  };

  /**
   * 显示编辑模态框
   */
  showEditModal = (record, mode) => {
    this.setState({
      editMode: mode,
      editRecord: record,
      editModalShow: true,
    });
  };

  /**
   * 关闭模态框的回调
   */
  handleModalOk = () => {
    this.setState({
      editRecord: null,
      editMode: null,
      editModalShow: false,
    });
    this.search();
  };

  /**
   * 改变无效和有效状态
   */
  changeStatus = (id) => {
    ShowConfirm('确定改变当前分段收费状态？', async () => {
      this.setState({ loading: true });
      try {
        const data = await Loader.changeStatus({ id });
        ShowMessage(data.resultMessage || '操作成功', 'success');
        this.setState({ loading: false });
        this.search();
      } catch (e) {
        console.log(e);
        ShowMessage(e.text);
        this.setState({
          loading: false,
        });
      }
    });
  };

  renderStatus = (detail, rowData, index) => {
    return (
      <p key="asd" style={{ display: 'flex' }}>
        <b style={{ fontSize: '20px', color: detail === 1 ? '#FFCC49' : '#AAA', marginRight: '3px' }}>·</b>
        {detail === 1 ? '有效' : '无效'}
      </p>
    );
  };

  renderSectionTime = (record) => {
    const rowArray = record.periods.map((rowData, index) => {
      const { chargeBeginTime, chargeEndTime, hours } = rowData;
      const startDate = moment(chargeBeginTime).format('YYYY-MM-DD HH:mm');
      const endDate = moment(chargeEndTime).format('YYYY-MM-DD HH:mm');
      return `${startDate} - ${endDate}之间收取${index ? '剩余' : hours}课时`;
    });
    return renderMultiLine(rowArray.join(','));
  };

  /**
   * 表头
   */
  columns = [
    { title: '序号', dataIndex: 'index', width: 50, fixed: 'left', render: (text, record, index) => index + 1 },
    { title: '分段收费名称', dataIndex: 'name', width: 142 },
    { title: '产品类型', dataIndex: 'productTypeId', width: 50, render: (text) => produtctTypeString[text] },
    { title: '年份', dataIndex: 'year', width: 50 },
    { title: '季度', dataIndex: 'quarter', width: 50 },
    { title: '分公司', dataIndex: 'branchName', width: 70, render: renderMultiLine },
    {
      title: '是否连报',
      dataIndex: 'isContinue',
      key: 'dataIndex',
      width: 70,
      render: (text) => {
        return text ? '是' : '否';
      },
      filterDropdownVisible: false,
      filterIcon: (
        <Popover overlayClassName="sectionPayment-list-popover" content="连报：连续两个季度的都有报科次。">
          <Icon className="sectionPayment-list-icon" type="question_circle" size={12} />
        </Popover>
      ),
      filterDropdown: <span className="empty-content"> </span>,
    },
    { title: '状态', dataIndex: 'isValid', width: 70, render: this.renderStatus },
    {
      title: '分段收费',
      dataIndex: 'acrossEnrollEndTime',
      width: 250,
      render: (text, record) => this.renderSectionTime(record),
    },
    {
      title: '操作',
      key: 'action',
      width: 180,
      fixed: 'right',
      getActionConfig: (text, record, index) => {
        const { isValid } = record;
        const btns = [{ text: '详情', onClick: () => this.showEditModal(record, 'detail') }];
        if (this.props.btnAuths.indexOf('BTN_SECTION_EDIT') >= 0) {
          btns.unshift({ text: '修改', onClick: () => this.showEditModal(record) });
        }
        btns.push({ text: '无效', onClick: () => this.changeStatus(record.id), disabled: isValid === 0 });
        return btns;
      },
    },
  ];

  render() {
    const {
      loading,
      searchExpand,
      listLoading,
      rows,
      page,
      pageSize,
      total,
      name,
      branchId,
      yearId,
      quarterId,
      isValid,
      branchDict,
      yearDict,
      quarterDict,
      productTypeId,
      editModalShow,
      editRecord,
      editMode,
    } = this.state;
    return (
      <Page
        loading={loading}
        route={this.props.route}
        subHeader={
          <div className={commonStyles.searchBar}>
            {this.props.btnAuths.indexOf('BTN_SECTION_ADD') >= 0 && (
              <Button
                className={commonStyles.addQuli}
                onClick={() => this.setState({ editModalShow: true, editRecord: null })}
              >
                新增
              </Button>
            )}
            <SearchBar
              onSearch={this.search}
              onReset={this.reset}
              expand={searchExpand}
              onToggleExpand={(expand) => this.setState({ searchExpand: expand })}
            >
              <Input
                value={name}
                placeholder="分段收费名称"
                onChange={(e) => this.setState({ name: e.target.value })}
              />
              <Select
                placeholder="请选择分公司"
                dict={branchDict}
                value={branchId}
                allowClear={false}
                showSearch={true}
                onChange={(branchId) => this.setState({ branchId })}
              />
              <Select
                placeholder="请选择产品类型"
                dict={productTypeDict}
                value={productTypeId}
                allowClear={true}
                onChange={(productTypeId) => this.setState({ productTypeId: productTypeId })}
              />
              <Select
                placeholder="请选择年份"
                dict={yearDict}
                value={yearId}
                allowClear={true}
                onChange={(yearId) => this.setState({ yearId })}
              />
              <Select
                placeholder="请选择季度"
                dict={quarterDict}
                value={quarterId}
                allowClear={true}
                onChange={(quarterId) => this.setState({ quarterId })}
              />
              <Select
                placeholder="是否有效"
                dict={validDict}
                value={isValid}
                allowClear={true}
                onChange={(isValid) => this.setState({ isValid })}
              />
            </SearchBar>
          </div>
        }
        fitWindowHeight
      >
        <List
          columns={this.columns}
          loading={listLoading}
          dataSource={rows}
          pagination={{
            current: page,
            pageSize,
            total: total,
          }}
          onChange={this.pageChange}
          locale={{ filterTitle: '' }}
        />
        {editModalShow && (
          <EditModal
            editRecord={editRecord}
            editMode={editMode}
            ref={(e) => {
              this.editRef = getWrappedInstance(e);
            }}
            onOk={this.handleModalOk}
            onCancel={() => {
              this.setState({ editModalShow: false, editRecord: null, editMode: null });
            }}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProp = (state) => {
  return {
    btnAuths: state.account.btnAuths,
  };
};

export default connect(mapStateToProp, null)(SectionPayment);
