import React from 'react';
import { Modal, FormItem, Select } from 'COMPONENT';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import loader from '../loader';

class EditCityModal extends React.Component {
  constructor(props) {
    super(props);

    const { city } = props.studentData;
    this.state = {
      loading: false,
      cityId: city.id,
      cityDict: [],
    };
  }

  componentDidMount() {
    this.init();
  }

  async init() {
    this.setState({ loading: true });
    const cityDict = await loader.getAllCityData();
    const formatCityDict = cityDict.map(({ id, name, branchId }) => ({
      branchId,
      text: name,
      val: id,
    }));
    const cityMap = formatCityDict.reduce((obj, cur) => {
      obj[cur.val] = cur;
      return obj;
    }, {});
    this.setState({ loading: false, cityDict: formatCityDict, cityMap });
  }

  onOk = () => {
    const { onConfirm, studentData } = this.props;
    const { cityId, cityMap } = this.state;
    const { loginMarkId } = studentData;
    if (!cityId) {
      this.modalRef.showOkTips('请选择报读城市');
      return false;
    }
    onConfirm && onConfirm({ ...cityMap[cityId], loginMarkId });
  };

  render() {
    const { loading, cityId, cityDict } = this.state;
    let { onClose } = this.props;
    return (
      <Modal
        loading={loading}
        onCancel={onClose}
        onOk={this.onOk}
        title="修改报读城市"
        width={480}
        okBtnShow={true}
        cancelText="取消"
        okText="确定"
        ref={(c) => (this.modalRef = getWrappedInstance(c))}
      >
        <FormItem labelWidth={80} name="报读城市" required>
          <Select
            placeholder="请选择报读城市"
            dict={cityDict}
            value={cityId}
            showSearch
            onChange={(cityId) => this.setState({ cityId })}
          />
        </FormItem>
      </Modal>
    );
  }
}

export default EditCityModal;
