/**
 * Created by Haolin on 2018/8/20.
 */
import React from 'react';
import { Row, Col } from 'antd';
import Block from 'COMPONENT/Block';
import FormItem from 'COMPONENT/FormItem';
import Select from 'COMPONENT/Select';
import style from './style.less';

class ChooseAppt extends React.Component {
  constructor(props) {
    super(props);
    this.state = {};
  }

  render() {
    const {
      dict: { subjectDict, appointmentDict },
      data,
      handleUpdate,
      canSelect,
      error,
      checkValidator,
      handleValidator,
      canAdd,
    } = this.props;
    let showDel = data.length > 1;

    return (
      <Block title="选择时间" type="dark" className="exam-site-choose-appt">
        <div className={style['exam-site-choose-appt-wrapper']}>
          {data.map((item, index) => {
            const { subjectId, appointmentId } = item;
            return (
              <Row key={index}>
                <Col span={11}>
                  <FormItem name="科目" labelWidth={80} required>
                    <Select
                      placeholder="请选择科目"
                      dict={subjectDict}
                      value={subjectId}
                      onChange={(val) => handleUpdate({ actType: 'update', type: 'subjectId', val: val, index: index })}
                      validator={{
                        rules: ['required'],
                        silent: !Boolean(error['appointment_' + index + '_subject']) || checkValidator,
                        showStatus: true,
                        showTips: false,
                        onErrorChange: (e) => handleValidator({ type: 'appointment_' + index + '_subject', val: e }),
                      }}
                    />
                  </FormItem>
                </Col>
                <Col span={11}>
                  <FormItem name="预约时间" labelWidth={80} required>
                    <Select
                      placeholder="请选择预约时间"
                      dict={
                        subjectId && subjectId !== '' ? (appointmentDict ? appointmentDict[subjectId] : null) : null
                      }
                      value={appointmentId}
                      onChange={(val) =>
                        handleUpdate({ actType: 'update', type: 'appointmentId', val: val, index: index })
                      }
                      disabled={!(subjectId && subjectId !== '')}
                      validator={{
                        rules: ['required'],
                        silent: !Boolean(error['appointment_' + index + '_time']) || checkValidator,
                        showStatus: true,
                        showTips: false,
                        onErrorChange: (e) => handleValidator({ type: 'appointment_' + index + '_time', val: e }),
                      }}
                    />
                  </FormItem>
                </Col>
                <Col span={2}>
                  {canAdd && (
                    <div className={style['choose-time-row-btns']}>
                      <span
                        className={style['choose-time-row-add']}
                        onClick={() => {
                          handleUpdate({ actType: 'add', index: index });
                        }}
                      >
                        新增
                      </span>
                      {showDel && (
                        <span
                          className={style['choose-time-row-delete']}
                          onClick={() => {
                            handleUpdate({ actType: 'delete', index: index });
                          }}
                        >
                          删除
                        </span>
                      )}
                    </div>
                  )}
                </Col>
              </Row>
            );
          })}
          {!canSelect && <div className={style['exam-site-choose-appt-cover']}> </div>}
        </div>
      </Block>
    );
  }
}

export default ChooseAppt;
