/**
 * Created by Haolin on 2018/8/14.
 */
import React from 'react';
import { Row, Col } from 'antd';
import Block from 'COMPONENT/Block';
import FormItem from 'COMPONENT/FormItem';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import Input from 'COMPONENT/Input';

class UserInfo extends React.Component {
  constructor(props) {
    super(props);
  }

  render() {
    const {
      handleUpdate,
      error,
      checkValidator,
      handleValidator,
      data: { studentName, phoneNum, campus, grade, version, quarter },
      dict: { branchCampusDict, gradeDict, versionDict, quarterDict },
    } = this.props;

    return (
      <Block title="基础信息" type="dark" className="exam-site-base-info">
        <Row>
          <Col span={12}>
            <FormItem name="学生姓名" labelWidth={80} required>
              <Input
                placeholder="请输入姓名"
                value={studentName}
                onChange={(e) => handleUpdate({ type: 'studentName', val: e.target.value })}
                validator={{
                  rules: ['required'],
                  silent: !Boolean(error['studentName']) || checkValidator,
                  showStatus: true,
                  showTips: false,
                  onErrorChange: (e) => handleValidator({ type: 'studentName', val: e }),
                }}
              />
            </FormItem>
          </Col>
          <Col span={12}>
            <FormItem name="客户手机" labelWidth={80} required>
              <Input
                placeholder="请输入客户手机号码"
                value={phoneNum}
                onChange={(e) => handleUpdate({ type: 'phoneNum', val: e.target.value })}
                validator={{
                  rules: ['phone'],
                  silent: !Boolean(error['phoneNum']) || checkValidator,
                  showStatus: true,
                  showTips: false,
                  onErrorChange: (e) => handleValidator({ type: 'phoneNum', val: e }),
                }}
              />
            </FormItem>
          </Col>
        </Row>
        <Row>
          <Col span={12}>
            <FormItem name="学习中心" labelWidth={80} required>
              <Select
                placeholder="请选择学习中心"
                dict={branchCampusDict}
                value={campus}
                showSearch={true}
                onChange={(val) => handleUpdate({ type: 'campus', val: val })}
                validator={{
                  rules: ['required'],
                  silent: !Boolean(error['campus']) || checkValidator,
                  showStatus: true,
                  showTips: false,
                  onErrorChange: (e) => handleValidator({ type: 'campus', val: e }),
                }}
              />
            </FormItem>
          </Col>
          <Col span={12}>
            <FormItem name="年级" labelWidth={80} required>
              <Select
                placeholder="请选择年级"
                dict={gradeDict}
                value={grade}
                showSearch={true}
                onChange={(val) => handleUpdate({ type: 'grade', val: val })}
                validator={{
                  rules: ['required'],
                  silent: !Boolean(error['grade']) || checkValidator,
                  showStatus: true,
                  showTips: false,
                  onErrorChange: (e) => handleValidator({ type: 'grade', val: e }),
                }}
              />
            </FormItem>
          </Col>
        </Row>
        <Row>
          <Col span={12}>
            <FormItem name="年份" labelWidth={80} required>
              <Select
                placeholder="请选择年份"
                dict={versionDict}
                value={version}
                onChange={(val) => handleUpdate({ type: 'version', val: val })}
                validator={{
                  rules: ['required'],
                  silent: !Boolean(error['version']) || checkValidator,
                  showStatus: true,
                  showTips: false,
                  onErrorChange: (e) => handleValidator({ type: 'version', val: e }),
                }}
              />
            </FormItem>
          </Col>
          <Col span={12}>
            <FormItem name="季度" labelWidth={80} required>
              <Select
                placeholder="请选择季度"
                dict={quarterDict}
                value={quarter}
                onChange={(val) => handleUpdate({ type: 'quarter', val: val })}
                validator={{
                  rules: ['required'],
                  silent: !Boolean(error['quarter']) || checkValidator,
                  showStatus: true,
                  showTips: false,
                  onErrorChange: (e) => handleValidator({ type: 'quarter', val: e }),
                }}
              />
            </FormItem>
          </Col>
        </Row>
      </Block>
    );
  }
}

export default UserInfo;
