/**
 * Created by Haolin on 2018/8/17.
 */
import React from 'react';
import { connect } from 'dva';
import Modal from 'COMPONENT/Modal';
import Button from 'COMPONENT/Button';
import PopTip from 'COMPONENT/PopTip';
import { cloneDeep } from 'lodash';
import { loadCommonDict, getDictTextByVal } from 'UTIL/dict';
import UserInfo from './UserInfo';
import ChooseAppt from './ChooseAppt';
import './style.less';
import { addStdnAppt } from 'SERVICE/signUpOnline';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { getExamAppointmentInfo } from 'SERVICE/signUpOnline';

const ModalFooterBtn = (props) => {
  const { content, popTipVisible, onClose, onOk, onCancel, okText } = props;
  return (
    <div style={{ display: 'flex', justifyContent: 'flex-end' }}>
      <Button onClick={onCancel}>取消</Button>
      <PopTip icon="error" content={content} visible={popTipVisible} onClose={onClose} maxWidth={200}>
        <Button type="primary" onClick={onOk}>
          {okText ? okText : '确定'}
        </Button>
      </PopTip>
    </div>
  );
};

class ApptModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      name: '新增预约',
      loading: false,
      error: null,
      errors: {},
      checkValidator: true,
      toggle_btnTip: false,
      btnTipContent: '',

      quarterDict: null,
      subjectDict: null,
      appointmentDict: null,

      studentName: null,
      phoneNum: null,
      grade: null,
      campus: null,
      version: null,
      quarter: null,

      appointment: [
        {
          subjectId: null,
          appointmentId: null,
          appointmentDate: null,
          appointmentTime: null,
          timeId: null,
          examId: null,
        },
      ],
    };
  }
  componentWillMount() {
    this.init();
  }

  /**
   * 初始化
   * */
  init = async () => {
    const { type, initData } = this.props;
    await this.updateState({
      loading: true,
      name: type === 'edit' ? '修改预约' : '新增预约',
    });
    const [quarterDict] = await Promise.all([loadCommonDict('PRODUCT_QUARTER')]);
    let result = {
      loading: false,
      quarterDict,
    };
    if (type === 'edit') {
      const {
        studentName,
        phoneNum,
        blCampusId,
        gradeId,
        yearId,
        quarterId,
        subjectId,
        appointmentDate,
        appointmentTime,
        timeId,
        examId,
      } = initData;
      getExamAppointmentInfo(
        {
          gradeId,
          blCampusId,
          yearId,
          quarterId,
        },
        true,
      ).then(
        (data) => {
          const {
            data: { times, subjects },
          } = data;
          let ApptDict = this.buildApptDict(times);
          let subjectDict = this.buildSubjectDict(subjects);
          const state = {
            ...result,
            subjectDict: subjectDict,
            appointmentDict: ApptDict,
            studentName: studentName,
            phoneNum: phoneNum,
            appointment: this.buildApptInitData({ subjectId, appointmentDate, appointmentTime, timeId, examId }),
            grade: gradeId,
            campus: blCampusId,
            version: yearId,
            quarter: quarterId,
          };
          this.updateState(state);
        },
        (err) => {
          showMessage(err.text, 'error');
          this.updateState({ loading: false });
        },
      );
    } else {
      this.updateState(result);
    }
  };
  buildApptInitData = (params) => {
    const { subjectId, appointmentDate, appointmentTime, timeId, examId } = params;
    return [
      {
        subjectId: subjectId,
        appointmentId: appointmentDate + ' ' + appointmentTime + ' ' + timeId + ' ' + examId,
        appointmentDate: appointmentDate,
        appointmentTime: appointmentTime,
        timeId,
        examId,
      },
    ];
  };
  /**
   * 加载预约时间字典
   * */
  loadApptDict = (params) => {
    this.updateState({ loading: true });
    const { grade, campus, version, quarter } = params;
    let apiParams = {
      blCampusId: campus,
      gradeId: grade,
      quarterId: quarter,
      yearId: version,
    };
    getExamAppointmentInfo(apiParams, true).then(
      (data) => {
        const {
          data: { times, subjects },
        } = data;
        let ApptDict = this.buildApptDict(times);
        let subjectDict = this.buildSubjectDict(subjects);
        this.updateState({
          loading: false,
          subjectDict: subjectDict,
          appointmentDict: ApptDict,
        });
      },
      (err) => {
        showMessage(err.text, 'error');
        this.updateState({ loading: false });
      },
    );
  };
  /**
   * 构建预约时间段字典
   * */
  buildApptDict = (params) => {
    let keys = Object.keys(params);
    let values = Object.values(params);
    let result = {};
    keys.forEach((keyItem, keyIndex) => {
      values[keyIndex].forEach((item) => {
        const { appointmentTime, timeId, examId } = item;
        if (!Array.isArray(result[keyItem])) {
          result[keyItem] = [];
        }
        result[keyItem].push({
          val: appointmentTime + ' ' + timeId + ' ' + examId,
          text: appointmentTime,
        });
      });
    });
    return result;
  };
  /**
   * 构建考试科目字典
   * */
  buildSubjectDict = (params) => {
    let keyArr = [];
    let ValArr = [];
    params.forEach((item) => {
      keyArr.push(Object.keys(item)[0]);
      ValArr.push(Object.values(item)[0]);
    });
    return keyArr.map((item, index) => ({ val: item, text: ValArr[index] }));
  };
  /**
   * 更新state
   * */
  updateState = (newState) => {
    this.setState((prevState, props) => {
      return {
        ...newState,
      };
    });
  };
  /**
   * 更新用户信息
   * */
  updateUserInfo = (params) => {
    const { type, val } = params;
    const { studentName, phoneNum, grade, campus, version, quarter } = this.state;
    this.updateState({ [type]: val });
    let data = {
      studentName,
      phoneNum,
      grade,
      campus,
      version,
      quarter,
      [type]: val,
    };
    if (Object.values(data).every((item) => item && item !== '') && type !== 'phoneNum' && type !== 'studentName') {
      //更新预约时间字典
      this.loadApptDict(data);
    }
  };
  /**
   * 更新预约
   * */
  updateAppointment = (params) => {
    const { actType, type, val, index } = params;
    const { appointment, errors } = this.state;
    let list = cloneDeep(appointment);
    switch (actType) {
      case 'update':
        if (type === 'appointmentId') {
          let date, time, tId, eId;
          if (val && val !== '') {
            [date, time, tId, eId] = val.split(' ');
          } else {
            date = null;
            time = null;
            tId = null;
            eId = null;
          }
          list[index].appointmentDate = date;
          list[index].appointmentTime = time;
          list[index].timeId = tId;
          list[index].examId = eId;
        }
        list[index][type] = val;
        break;
      case 'add':
        list.splice(index + 1, 0, {
          subjectId: null,
          appointmentId: null,
        });
        break;
      case 'delete':
        list.splice(index, 1);
        let err = cloneDeep(errors);
        delete err['appointment_' + index + '_subject'];
        delete err['appointment_' + index + '_time'];
        this.setState({ errors: err });
        break;
    }
    this.updateState({ appointment: list });
  };

  /**
   * 用于校验表单是否存在错误
   * */
  errorChange = (params) => {
    const { type, val } = params;
    this.setState((prevState) => {
      let err = cloneDeep(prevState.errors);
      err[type] = val;
      return { errors: err };
    });
  };
  /**
   * 校验数据
   * */
  checkBeforeSubmit = () => {
    const { errors } = this.state;
    let tipMsg = [];
    let checkRequired = this.checkRequired(errors);
    if (!checkRequired) {
      tipMsg.push('您有信息未正确填写 ');
    }
    this.updateState({ checkValidator: checkRequired });

    let check = checkRequired;
    if (!check) {
      this.updateState({
        btnTipContent: (
          <ul style={{ listStyle: 'inside' }}>
            {tipMsg.map((item, index) => (
              <li key={index}>{item}</li>
            ))}
          </ul>
        ),
        toggle_btnTip: true,
      });
    }
    return check;
  };
  checkRequired = (params) => {
    return Object.values(params).every((item) => {
      return !Boolean(item);
    });
  };
  /**
   * 构建供提交接口使用的参数
   * */
  buildApiParams = (params) => {
    const { studentName, phoneNum, campus, grade, version, quarter, appointment } = params;
    const { type, initData } = this.props;
    let result = appointment.map((item) => {
      const { subjectId, appointmentId, appointmentDate, appointmentTime, timeId, examId } = item;
      return {
        studentName,
        studentPhone: phoneNum,
        blCampusId: campus,
        gradeId: grade,
        yearId: version,
        quarterId: quarter,
        subjectId: subjectId,
        appointmentDate: appointmentDate,
        appointmentTime: appointmentTime,
        appointmentWay: 1,
        timeId,
        examId,
      };
    });
    if (type === 'edit') {
      result.forEach((item) => {
        item.id = initData.id;
        item.status = 2;
      });
    }
    return result;
  };
  /**
   * 提交
   * */
  submit = async () => {
    //校验
    let checkVal = this.checkBeforeSubmit();
    if (checkVal) {
      await this.updateState({ loading: true });
      const { handleSubmit, type } = this.props;
      const { studentName, phoneNum, campus, grade, version, quarter, appointment } = this.state;
      let apiParams = this.buildApiParams({ studentName, phoneNum, campus, grade, version, quarter, appointment });
      addStdnAppt(apiParams, true).then(
        (data) => {
          const tipMsg = type === 'edit' ? '修改预约成功！' : '新增预约成功！';
          showMessage(tipMsg, 'success');
          this.updateState({ loading: false });
          handleSubmit();
        },
        (err) => {
          showMessage(err.text, 'error');
          this.updateState({ loading: false });
        },
      );
    }
  };

  render() {
    const {
      type,
      handleCancel,
      dict: { branchCampusDict, versionDict, gradeDict },
      uBranchId,
    } = this.props;
    const {
      name,
      loading,
      error,
      errors,
      checkValidator,
      quarterDict,
      subjectDict,
      appointmentDict,
      studentName,
      phoneNum,
      campus,
      grade,
      version,
      quarter,
      appointment,
      toggle_btnTip,
      btnTipContent,
    } = this.state;

    let userInfoDict = {
      branchCampusDict:
        uBranchId && uBranchId !== '000001'
          ? branchCampusDict.filter((item) => item.val === uBranchId)
          : branchCampusDict,
      versionDict,
      gradeDict,
      quarterDict,
    };
    let userInfoData = {
      studentName,
      phoneNum,
      campus,
      grade,
      version,
      quarter,
    };
    let chooseApptDict = {
      subjectDict,
      appointmentDict,
    };
    let canSelectAppt = studentName && phoneNum && campus && grade && version && quarter;
    let canAdd = type !== 'edit';

    return (
      <Modal
        title={name}
        className="student-appointment-modal"
        loading={loading}
        error={error}
        width={1100}
        onCancel={handleCancel}
        footer={
          <ModalFooterBtn
            content={btnTipContent}
            popTipVisible={toggle_btnTip}
            onClose={() => this.updateState({ toggle_btnTip: false })}
            okText="保存"
            onOk={this.submit}
            onCancel={handleCancel}
          />
        }
      >
        <UserInfo
          dict={userInfoDict}
          data={userInfoData}
          handleUpdate={this.updateUserInfo}
          error={errors}
          checkValidator={checkValidator}
          handleValidator={this.errorChange}
        />
        <ChooseAppt
          canAdd={canAdd}
          dict={chooseApptDict}
          data={appointment}
          handleUpdate={this.updateAppointment}
          canSelect={canSelectAppt}
          error={errors}
          checkValidator={checkValidator}
          handleValidator={this.errorChange}
        />
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  const { brenchId } = state.account.user;
  return {
    uBranchId: brenchId ? brenchId : null,
  };
};

export default connect(mapStateToProps)(ApptModal);
