/**
 * Created by Haolin on 2018/9/12.
 */
import React from 'react';
import { connect } from 'dva';
import { Row, Col } from 'antd';
import Modal from 'COMPONENT/Modal';
import Button from 'COMPONENT/Button';
import PopTip from 'COMPONENT/PopTip';
import Select from 'COMPONENT/Select';
import { loadCommonDict, getDictTextByVal } from 'UTIL/dict';
import style from './style.less';
import './style.less';
import { cloneDeep, debounce } from 'lodash';
import { getReceptionistGroupByCampus } from 'SERVICE/signUpOnline';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

const ModalFooterBtn = (props) => {
  const { content, popTipVisible, onClose, onOk, okText } = props;
  return (
    <div style={{ display: 'flex', justifyContent: 'flex-end', position: 'relative' }}>
      <div style={{ position: 'absolute', top: 0, left: 0, lineHeight: '36px' }}>提示：老生仅签到，新生签到并录入</div>
      <PopTip icon="error" content={content} visible={popTipVisible} onClose={onClose} maxWidth={200}>
        <Button type="primary" onClick={onOk}>
          {okText ? okText : '确定'}
        </Button>
      </PopTip>
    </div>
  );
};

class ArrivedModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      name: '确认到场',
      loading: false,
      error: null,
      toggle_btnTip: false,
      btnTipContent: '',
      // 资源入口字典
      resourceEntranceDict: [
        {
          val: 'CALL_IN',
          text: '呼入',
        },
        {
          val: 'DIRECT_VISIT',
          text: '直访',
        },
        {
          val: 'OUTSTANDING_VISIT',
          text: '拉访',
        },
      ],
      // 分配跟进入字典
      consultantDict: null,
      isGroup: false,
      branch: null,
      campus: null,
      resourceEntrance: null,
      consultant: null,
      // 是否显示咨询师搜索结果
      searchKey: null,
      dictOptions: null,
    };
    /**
     * 节流-获取咨询师字典数据
     * */
    this.getConsultantDict = debounce(this.searchDict, 330);
  }
  componentWillMount() {
    this.init();
  }
  componentDidMount() {}
  componentWillUnmount() {}

  /**
   * 初始化
   * */
  init = async () => {
    await this.updateState({ loading: true });
    await this.loadConsultantDict();
    const {
      uBranchId,
      uCampusId,
      dict: { branchDict, branchCampusDict },
    } = this.props;
    let isGroup = uBranchId === uCampusId;
    let branchVal = isGroup ? branchDict[0].val : uBranchId;
    let campusVal = isGroup ? branchCampusDict.find((item) => item.val === branchVal).options[0].val : uCampusId;
    this.updateState({
      loading: false,
      isGroup,
      branch: branchVal,
      campus: campusVal,
      resourceEntrance: 'DIRECT_VISIT',
    });
  };
  /**
   * 加载咨询师
   * */
  loadConsultantDict = () => {
    getReceptionistGroupByCampus(null, true).then(
      (data) => {
        this.updateState({ consultantDict: data });
      },
      (err) => {
        showMessage(err.text, 'error');
      },
    );
  };
  /**
   * 构建 分配跟进入字典
   * */
  buildConsultantDict = (params) => {
    function Label(params) {
      return new Object({
        label: params.orgName,
        val: params.orgId,
        options: [
          {
            val: params.userId,
            text: params.userName,
          },
        ],
      });
    }
    let result = [];
    let labelArr = [];
    let count = 0;
    params.forEach((item) => {
      if (labelArr.indexOf(item.orgName) === -1) {
        labelArr.push(item.orgName);
        result.push(new Label(item));
        ++count;
      } else {
        result[count - 1].options.push({
          val: item.userId,
          text: item.userName,
        });
      }
    });
    return result;
  };
  /**
   * 更新input/select数据
   * */
  updateData = (params) => {
    const { type, val } = params;
    this.updateState({ [type]: val });
  };
  /**
   * 检索跟进人
   * */
  searchDict = (params) => {
    let result = null;
    let keyWord = null;
    if (params && params !== '') {
      const { consultantDict } = this.state;
      result = consultantDict.filter((item) => item.userName.indexOf(params) !== -1);
      if (result.length > 0) {
        result = this.buildConsultantDict(result);
      } else {
        result = null;
      }
      keyWord = params;
    }
    this.updateState({ searchKey: keyWord, dictOptions: result });
  };
  /**
   * 更新state
   * */
  updateState = (newState) => {
    this.setState((prevState, props) => {
      return {
        ...newState,
      };
    });
  };
  /**
   * 提交前校验
   * */
  checkBeforeSubmit = () => {
    const { consultant } = this.state;
    let result = true;
    if (!consultant) {
      this.updateState({
        btnTipContent: '请选择分配跟进人',
        toggle_btnTip: true,
      });
      result = false;
    }
    return result;
  };
  /**
   * 提交
   * */
  submit = () => {
    const check = this.checkBeforeSubmit();
    if (check) {
      const { handleSubmit } = this.props;
      const { campus, resourceEntrance, consultant } = this.state;
      handleSubmit({ campus, resourceEntrance, consultant });
    }
  };

  render() {
    const {
      loading: propLoading,
      dict: { branchCampusDict },
      handleCancel,
    } = this.props;
    const {
      name,
      loading,
      error,
      toggle_btnTip,
      btnTipContent,
      resourceEntranceDict,
      searchKey,
      dictOptions,
      isGroup,
      branch,
      campus,
      resourceEntrance,
      consultant,
    } = this.state;

    return (
      <Modal
        title={name}
        className="arrived-modal"
        loading={propLoading || loading}
        error={error}
        width={640}
        closable={true}
        onCancel={handleCancel}
        footer={
          <ModalFooterBtn
            content={btnTipContent}
            popTipVisible={toggle_btnTip}
            onClose={() => this.updateState({ toggle_btnTip: false })}
            okText="签到并导入"
            onOk={this.submit}
          />
        }
      >
        <Row className={style['arrived-modal-row']}>
          <Col span={24}>请选择客户信息导入到客户列表，只有录入到客户列表才能导入成绩</Col>
        </Row>
        <Row>
          <Col span={8}>
            <Select
              className={style['arrived-modal-select']}
              placeholder="资源入口"
              dict={resourceEntranceDict}
              value={resourceEntrance}
              allowClear={false}
              onChange={(val) => {
                this.updateData({ type: 'resourceEntrance', val });
              }}
            />
          </Col>
          <Col span={8}>
            <Select
              className={style['arrived-modal-select']}
              placeholder="意向学习中心"
              dict={
                branch && !isGroup
                  ? branchCampusDict && branchCampusDict.filter((item) => item.val === branch)
                  : branchCampusDict
              }
              value={campus}
              showSearch={true}
              allowClear={false}
              onChange={(val) => {
                this.updateData({ type: 'campus', val });
              }}
            />
          </Col>
          <Col span={8}>
            <Select
              placeholder="分配跟进人"
              dict={dictOptions}
              value={consultant}
              showSearch={true}
              onSearchKeyChange={(keyword) => {
                this.searchDict(keyword);
              }}
              noOptionTips={searchKey && !dictOptions ? '找不到匹配的数据' : '请输入姓名进行查询'}
              onChange={(val) => {
                this.updateData({ type: 'consultant', val });
              }}
            />
          </Col>
        </Row>
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  const { brenchId, campusId } = state.account.user;
  return {
    uBranchId: brenchId || null,
    uCampusId: campusId || null,
  };
};

export default connect(mapStateToProps)(ArrivedModal);
