import React from 'react';
import { Row, Col, Pagination } from 'antd';
import Modal from 'COMPONENT/Modal';
import Button from 'COMPONENT/Button';
import Input from 'COMPONENT/Input';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { getExamAppointmentInfo } from 'SERVICE/signUpOnline';
import './style.less';
import Paper from './Paper';
import PrintTable from '../PrintTable';

/**
 * important: 打印样式文件统一放在 public 文件夹内，供iframe调用
 * */

class PrintModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false,
      errorMsg: null,
    };
    this.printing = false;
  }

  componentWillMount() {
    // this.init();
  }
  componentDidMount() {}
  componentWillUnmount() {}

  init = () => {
    const { init } = this.props;
    if (init) {
      //使用选中项数据渲染
    } else {
      //加载列表...
    }
  };

  /**
   * 更新state
   * */
  updateState = (newState) => {
    this.setState((prevState, props) => {
      return {
        ...newState,
      };
    });
  };
  /**
   * 加载学生预约列表
   * */
  loadData = (params) => {
    this.updateState({ listLoading: true });
    const {
      studentNameVal,
      branchVal,
      campusVal,
      versionVal,
      gradeVal,
      dateVal,
      takeStateVal,
      pointLvlVal,
      appointmentTypeVal,
      pageCurrent,
      pageSize,
      takeStateDict,
      appointmentDict,
    } = this.state;
    let page, rows;
    if (params) {
      page = params.pageCurrent;
      rows = params.pageSize;
      this.updateState({
        pageCurrent: page,
        pageSize: rows,
      });
    } else {
      page = pageCurrent;
      rows = pageSize;
    }
    const apiParams = {
      studentName: studentNameVal,
      branchId: branchVal,
      blCampusId: this.buildParams(campusVal),
      yearId: versionVal,
      gradeId: this.buildParams(gradeVal),
      appointmentDate: dateVal,
      isCome: takeStateVal,
      classTypeId: pointLvlVal,
      appointmentWay: appointmentTypeVal,
      page,
      rows,
    };
    getStdnApptList(apiParams, true).then(
      (data) => {
        // console.log("success:",JSON.stringify(data.rows));
        let apiData = data.rows;
        let list = apiData.map((item, index) => {
          let {
            id,
            studentName,
            studentPhone,
            branchName,
            blCampusName,
            gradeName,
            appointmentDate,
            appointmentTime,
            subjectName,
            score,
            totalPoint,
            classTypeName,
            isCome,
            appointmentWay,
          } = item;
          return {
            id,
            index: index + 1 + Number(((page - 1) * rows).toFixed(0)),
            studentName,
            phoneNum: studentPhone,
            branch: branchName,
            campus: blCampusName,
            grade: gradeName,
            appointmentDate: appointmentDate,
            appointmentTime: appointmentTime,
            examPoint: subjectName + '： ' + (score ? score + '/' + totalPoint : '无'),
            pointLvl: classTypeName ? classTypeName : '无效',
            takeStateVal: isCome,
            takeState: getDictTextByVal(takeStateDict, isCome.toString()),
            appointmentType: getDictTextByVal(appointmentDict, appointmentWay.toString()),
            subject: subjectName,
          };
        });
        this.updateState({
          listLoading: false,
          appointmentList: list,
          pageTotal: data.records,
        });
        return true;
      },
      (err) => {
        this.updateState({ listLoading: false });
        console.error('# Load student appointment list error! #', err.text);
        return false;
      },
    );
  };

  /**
   * 打印
   * */
  print = () => {
    if (!this.printing) {
      this.printing = true;
      this.updateState({ loading: true });
      let paperNode = document.querySelector('#print-paper');
      let iFrameNode = document.querySelector('#print-iframe');
      let htmlStr =
        '<html><head>' +
        '<link rel="stylesheet" href="' +
        location.origin +
        '/print/Paper.css">' +
        '<link rel="stylesheet" href="' +
        location.origin +
        '/print/StdntApptMgmt.css">' +
        '</head><body>' +
        paperNode.innerHTML +
        '</body></html>';
      iFrameNode.contentDocument.write(htmlStr);
      iFrameNode.contentDocument.close();
      const _self = this;
      setTimeout(() => {
        _self.updateState({ loading: false });
        iFrameNode.contentWindow.print();
        this.printing = false;
      }, 1000);
    }
  };

  render() {
    const {
      loading: dataLoading,
      header,
      body,
      footer,
      pagination: { current, total, pageSize },
      changePage,
      handleCancel,
    } = this.props;
    const { loading, errorMsg } = this.state;

    return (
      <Modal
        title={null}
        className="print-modal"
        loading={loading || dataLoading}
        error={errorMsg}
        width={595}
        footer={null}
        closable={true}
        onCancel={handleCancel}
      >
        <Paper header={header} footer={footer}>
          {body}
        </Paper>
        <Row>
          <Col span={20}>
            <Pagination
              className="print-pagination"
              defaultCurrent={1}
              current={current}
              pageSize={pageSize}
              total={total}
              showQuickJumper={true}
              onChange={(page, pageSize) => changePage({ pageCurrent: page, pageSize: pageSize }, 'print')}
            />
          </Col>
          <Col span={4}>
            <Button
              size="small"
              onClick={() => {
                this.print();
              }}
            >
              打印
            </Button>
          </Col>
        </Row>
        <iframe id="print-iframe" src="" width="0" height="0" frameBorder="0">
          {' '}
        </iframe>
      </Modal>
    );
  }
}

export default PrintModal;
