/**
 * Created by Haolin on 2018/8/13.
 */
import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import Input from 'COMPONENT/Input';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import DatePicker from 'COMPONENT/DatePicker';
import Button from 'COMPONENT/Button';
import SearchBar from 'COMPONENT/SearchBar';
import List from 'COMPONENT/List';
import Icon from 'COMPONENT/Icon';
import Popover from 'COMPONENT/Popover';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import {
  loadCommonDict,
  loadBranchDict,
  loadCampusDict,
  loadBranchCampusDict,
  getDictTextByVal,
  getDictValByText,
} from 'UTIL/dict';
import './style.less';
import ApptModal from './ApptModal';
import PrintModal from './PrintModal';
import PrintTable from './PrintTable';
import ButtonWrap from './ButtonWrap';
import { getStdnApptList, confirmStdnArrival } from 'SERVICE/signUpOnline';
import utils from './utils.js';
import { cloneDeep } from 'lodash';
import dataBuryingPoint from 'UTIL/dataBuryingPoint';
import { OLDPAGEBASE } from 'CONST/config';
import { browserHistory } from 'react-router';
import ArrivedModal from './ArrivedModal';
import jslog from 'SRC/jslog';

const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L } = List;

const RowWrapper = (props) => {
  const { rightSide, children, rightSize } = props;
  return (
    <div className="search-row-wrapper-two-row" style={{ paddingRight: rightSize }}>
      {children}
      <div className="search-wrapper-right" style={{ width: rightSize }}>
        {rightSide}
      </div>
    </div>
  );
};
const Search = (props) => {
  const {
    handleSearch,
    handleReset,
    handleKeyword,
    handleBtn,
    btnType,
    dict: {
      branchDict,
      branchCampusDict,
      versionDict,
      gradeDict,
      appointmentDict,
      pointLvlDict,
      takeStateDict,
      subjectDict,
    },
    data: { studentName, branch, campus, version, grade, date, takeState, pointLvl, appointmentType, subject },
    btnAuths,
  } = props;
  let type = btnType ? btnType : '';

  let baseSize = 120;

  return (
    <RowWrapper
      rightSize={baseSize}
      rightSide={
        <div>
          <div className="search-wrapper-right-first-row">
            <Button
              onClick={() => handleBtn({ type: 'new' })}
              disabled={btnAuths.indexOf('EXAM_APPOINTMENT_ADD') === -1}
            >
              新增预约
            </Button>
          </div>
          <div className="search-wrapper-right-second-row" style={{ width: baseSize + 260 }}>
            <Button
              type={type}
              onClick={() => handleBtn({ type: 'print' })}
              style={{ width: 'auto' }}
              disabled={btnAuths.indexOf('EXAM_APPOINTMENT_EXPORT_PRINT') === -1}
            >
              打 印
            </Button>
            <ButtonWrap
              btnText="导出Excel"
              popPlacement="bottom"
              actions={[
                { text: '预约名单', onClick: () => handleBtn({ type: 'excel_stdntAppt' }) },
                { text: '成绩录入模板', onClick: () => handleBtn({ type: 'excel_scoreTemp' }) },
              ]}
              btnDisabled={btnAuths.indexOf('EXAM_APPOINTMENT_EXPORT_EXCEL') === -1}
            />
            <Button
              type={type}
              onClick={() => handleBtn({ type: 'confirm' })}
              disabled={btnAuths.indexOf('EXAM_APPOINTMENT_CONFIRM_ARRIVED') === -1 || !btnType}
            >
              确认到场
            </Button>
          </div>
        </div>
      }
    >
      <SearchBar onSearch={handleSearch} onReset={handleReset}>
        <Input
          placeholder="姓名/手机"
          value={studentName}
          onChange={(e) => handleKeyword({ type: 'studentName', val: e.target.value })}
        />
        <Select
          placeholder="分公司"
          dict={branchDict}
          value={branch}
          showSearch={true}
          allowClear={false}
          onChange={(val) => handleKeyword({ type: 'branch', val: val })}
        />
        <MultiSelect
          placeholder="学习中心"
          dict={branch ? branchCampusDict && branchCampusDict.filter((item) => item.val === branch) : branchCampusDict}
          value={campus}
          showSearch={true}
          allowClear={false}
          onChange={(val) => handleKeyword({ type: 'campus', val: val })}
        />
        <Select
          placeholder="产品年份"
          dict={versionDict}
          value={version}
          onChange={(val) => handleKeyword({ type: 'version', val: val })}
        />
        <DatePicker placeholder="预约日期" value={date} onChange={(val) => handleKeyword({ type: 'date', val: val })} />
        <Select
          placeholder="到场状态"
          dict={takeStateDict}
          value={takeState}
          onChange={(val) => handleKeyword({ type: 'takeState', val: val })}
        />
        <MultiSelect
          placeholder="年级"
          dict={gradeDict}
          value={grade}
          onChange={(val) => handleKeyword({ type: 'grade', val: val })}
        />
        <MultiSelect
          placeholder="科目"
          dict={subjectDict}
          value={subject}
          onChange={(val) => handleKeyword({ type: 'subject', val: val })}
        />
        <Select
          placeholder="得分评级"
          dict={pointLvlDict}
          value={pointLvl}
          onChange={(val) => handleKeyword({ type: 'pointLvl', val: val })}
        />
        <Select
          placeholder="预约方式"
          dict={appointmentDict}
          value={appointmentType}
          onChange={(val) => handleKeyword({ type: 'appointmentType', val: val })}
        />
      </SearchBar>
    </RowWrapper>
  );
};

class StdntApptMgmt extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      name: '学生预约管理',
      loading: false,
      errorMsg: false,
      listLoading: false,
      pageCurrent: 1,
      pageTotal: 0,
      pageSize: 20,
      //搜索栏-字典
      branchDict: null,
      branchCampusDict: null,
      versionDict: null,
      gradeDict: null,
      campusDict: null,
      subjectDict: null,
      takeStateDict: [
        {
          val: '1',
          text: '已到考场',
        },
        {
          val: '2',
          text: '未到考场',
        },
      ],
      pointLvlDict: [
        {
          val: '1',
          text: '成绩有效',
        },
        {
          val: '2',
          text: '成绩无效',
        },
      ],
      appointmentDict: [
        {
          val: '1',
          text: '现场预约',
        },
        {
          val: '2',
          text: '在线预约',
        },
      ],
      //搜索栏-搜索条件
      studentNameVal: null,
      branchVal: null,
      campusVal: null, //{Array}
      versionVal: null,
      gradeVal: null, //{Array}
      dateVal: null,
      takeStateVal: null,
      pointLvlVal: null,
      appointmentTypeVal: null,
      subjectVal: null,
      //列表数据
      appointmentList: [],
      //列表选中项
      selectedRowKeys: [],
      //新增预约弹窗
      toggle_newAppt: false,
      //打印弹窗
      toggle_print: false,
      printLoading: false,
      printTitle: ' ',
      printList: [],
      printCurrent: 1,
      printTotal: 0,
      printPageSize: 30,
      modalType: null,
      editData: null,
      // 确认到场弹窗
      arrivedLoading: false,
      toggle_arrived: false,
      arrivedIdArr: null,
    };
  }
  componentWillMount() {
    this.init();
  }

  static buildPhoneNumForShow(num) {
    if (num.length === 11) {
      return num.slice(0, 3) + '****' + num.slice(-4);
    } else {
      return num;
    }
  }

  /**
   * 初始化
   * */
  init = () => {
    this.updateState({ loading: true });
    this.initCheck();
  };
  initCheck = () => {
    const { uBranchId, uCampusId } = this.props;
    if (uBranchId && uCampusId) {
      this.initPage();
    } else {
      const that = this;
      setTimeout(() => {
        that.initCheck();
      }, 200);
    }
  };
  initPage = async () => {
    const { uBranchId, uCampusId } = this.props;
    const { pageSize } = this.state;
    const [branchDict, branchCampusDict, versionDict, gradeDict, campusDict, subjectDict] = await Promise.all([
      loadBranchDict(),
      loadBranchCampusDict(),
      loadCommonDict('PRODUCT_VERSION'),
      loadCommonDict('STUDENT_GRADE'),
      loadCampusDict(),
      loadCommonDict('SUBJECT'),
    ]);
    //根据用户所属组织初始化分公司及学习中心，当是集团的人，默认取第一个值
    let isGroup = uBranchId === uCampusId;
    let branchVal = isGroup ? branchDict[0].val : uBranchId;
    let campusVal = isGroup ? [branchCampusDict.find((item) => item.val === branchVal).options[0].val] : [uCampusId];
    let todayObj = new Date();
    let versionVal = getDictValByText(versionDict, todayObj.getFullYear().toString());
    let dateVal = this.buildDateVal(todayObj);
    await this.updateState({
      branchVal,
      campusVal,
      versionVal,
      dateVal,
    });
    // console.log("* ",isGroup?"集团员工":"分公司员工",":",branchVal,campusVal);
    await Promise.all([this.loadData({ pageCurrent: 1, pageSize })]);
    this.updateState({
      loading: false,
      branchDict,
      branchCampusDict,
      versionDict,
      gradeDict,
      campusDict,
      subjectDict,
    });
  };
  /**
   * 构建日期val string
   * */
  /**
   * 格式化
   * */
  buildDateVal = (date) => {
    let y = date.getFullYear();
    let m = date.getMonth() + 1;
    let d = date.getDate();
    m = m < 10 ? '0' + m : m.toString();
    d = d < 10 ? '0' + d : d.toString();
    return y.toString() + '-' + m + '-' + d;
  };
  /**
   * 更新state
   * */
  updateState = (newState) => {
    this.setState((prevState, props) => {
      return {
        ...newState,
      };
    });
  };

  /**
   * 搜索栏-查询
   * */
  updateSearch = () => {
    const { pageSize } = this.state;
    this.loadData({ pageCurrent: 1, pageSize });
  };
  /**
   * 搜索栏-重置
   * */
  updateReset = () => {
    this.updateState({
      studentNameVal: null,
      branchVal: null,
      campusVal: null,
      versionVal: null,
      gradeVal: null,
      dateVal: null,
      takeStateVal: null,
      pointLvlVal: null,
      appointmentTypeVal: null,
      subjectVal: null,
    });
  };
  /**
   * 搜索栏-更新搜索条件
   * */
  updateKeyword = (params) => {
    const { type, val } = params;
    switch (type) {
      case 'branch':
        this.setState({ campusVal: null });
        break;
    }
    this.updateState({ [type + 'Val']: val });
  };
  /**
   * 搜索栏按钮操作
   * */
  btnAction = (params) => {
    const { type } = params;
    switch (type) {
      case 'new':
        this.updateState({
          toggle_newAppt: true,
          modalType: 'new',
        });
        break;
      case 'print':
        this.printAppointment();
        break;
      case 'excel_stdntAppt':
        jslog.report('click', { function_name: '课程管理>在线报读管理>学生预约管理>导出Excel' });
        this.exportStdntAppt();
        break;
      case 'excel_scoreTemp':
        jslog.report('click', { function_name: '课程管理>在线报读管理>学生预约管理>导出Excel' });
        this.exportScoreTemp();
        break;
      case 'confirm':
        const { appointmentList, selectedRowKeys } = this.state;
        let idList = selectedRowKeys.map((item) => {
          const { id, studentName, phoneNum, gradeId } = appointmentList[item];
          return {
            contact: phoneNum,
            cusType: null,
            deliverTarget: null,
            id,
            intentionCampusId: null,
            name: `${studentName}家长`,
            studentImportVos: [
              {
                contact: phoneNum,
                fatherName: `${studentName}家长`,
                fatherPhone: phoneNum,
                gradeId,
                name: studentName,
              },
            ],
          };
          // return appointmentList[item].id
        });
        this.updateState({
          toggle_arrived: true,
          arrivedIdArr: idList,
        });
        break;
    }
  };
  /**
   * 打印
   * */
  printAppointment = () => {
    this.updateState({ toggle_print: true, printLoading: true });
    jslog.report('click', { function_name: '课程管理>在线报读管理>学生预约管理>打印' });
    const { appointmentList, selectedRowKeys, campusVal, dateVal, campusDict } = this.state;
    let campusStr =
      campusVal && campusVal.length > 0 ? campusVal.map((item) => getDictTextByVal(campusDict, item)).join(',') : '';
    let dateStr = dateVal ? dateVal : '';
    let title = campusStr + dateStr + '入学测试报名表';
    this.setState({ printTitle: title });
    //判断是否有选中项
    if (selectedRowKeys.length > 0) {
      //有选中项:将数据过滤
      let origin = cloneDeep(appointmentList);
      let list = selectedRowKeys.map((item) => origin[item]);
      list.forEach((item, index) => {
        item.index = index + 1;
      });
      this.updateState({
        printList: list,
        printTotal: list.length,
        printLoading: false,
      });
    } else {
      //无选中项:请求打印数据
      const { printCurrent, printPageSize } = this.state;
      this.loadData({ pageCurrent: printCurrent, pageSize: printPageSize }, 'print');
    }
    dataBuryingPoint.send({ operation: '学生预约管理的打印操作' });
  };
  /**
   * 导出excel
   * */
  exportExcel = async (params) => {
    await this.updateState({ loading: true });
    const { api, lastFileName } = params;
    const {
      studentNameVal,
      branchVal,
      campusVal,
      versionVal,
      gradeVal,
      dateVal,
      takeStateVal,
      pointLvlVal,
      appointmentTypeVal,
      subjectVal,
      appointmentList,
      selectedRowKeys,
      branchDict,
      campusDict,
      versionDict,
      gradeDict,
      takeStateDict,
    } = this.state;
    let apiParams = {
      studentName: studentNameVal,
      blCampusId: this.buildParams(campusVal),
      branchId: branchVal,
      yearId: versionVal,
      gradeId: this.buildParams(gradeVal),
      appointmentDate: dateVal,
      isCome: takeStateVal,
      classTypeId: pointLvlVal,
      appointmentWay: appointmentTypeVal,
      subjectId: this.buildParams(subjectVal),
    };
    if (selectedRowKeys && selectedRowKeys.length > 0) {
      apiParams.ids = selectedRowKeys
        .map((item) => {
          return appointmentList[item].id;
        })
        .join(',');
    }
    const _self = this;
    // 分公司+学习中心+年份+年级+到场状态+文件名
    let branchStr = branchVal ? getDictTextByVal(branchDict, branchVal) : '';
    let campusStr =
      campusVal && campusVal.length > 0 ? campusVal.map((item) => getDictTextByVal(campusDict, item)).join(',') : '';
    let versionStr = versionVal ? getDictTextByVal(versionDict, versionVal) : '';
    let gradeStr =
      gradeVal && gradeVal.length > 0 ? gradeVal.map((item) => getDictTextByVal(gradeDict, item)).join(',') : '';
    // let takeStateStr = takeStateVal?getDictTextByVal(takeStateDict,takeStateVal):"";
    let fileName = branchStr + campusStr + versionStr + gradeStr + lastFileName;
    // console.log("fileName:",fileName);
    utils.exportExcel({
      apiUrl: api,
      data: apiParams,
      fileName: fileName,
      success: () => {
        _self.updateState({ loading: false });
        showMessage('导出成功！', 'success');
      },
      fail: (err) => {
        _self.updateState({ loading: false });
        showMessage(err.text, 'error');
      },
    });
  };
  /**
   * 导出预约名单excel
   * */
  exportStdntAppt = () => {
    const api = 'ExamRoomController/exportExamAppointmentReport.do';
    this.exportExcel({ api, lastFileName: '入学考试名单' });
  };
  /**
   * 导出成绩录入模板
   * */
  exportScoreTemp = () => {
    const api = 'ExamRoomController/exportExamAppointmentReportCard.do';
    this.exportExcel({ api, lastFileName: '成绩录入模板' });
  };
  /**
   * 列表更多操作
   * */
  moreAction = (params) => {
    const { actType, record, index } = params;
    switch (actType) {
      case 'confirm':
        this.updateState({
          toggle_arrived: true,
          arrivedIdArr: [
            {
              contact: record.phoneNum,
              cusType: null,
              deliverTarget: null,
              id: record.id,
              intentionCampusId: null,
              name: `${record.studentName}家长`,
              studentImportVos: [
                {
                  contact: record.phoneNum,
                  fatherName: `${record.studentName}家长`,
                  fatherPhone: record.phoneNum,
                  gradeId: record.gradeId,
                  name: record.studentName,
                },
              ],
            },
          ],
        });
        break;
      case 'contractNew':
        this.goToContractNew(record);
        break;
      case 'edit':
        this.updateState({
          toggle_newAppt: true,
          modalType: 'edit',
          editData: record,
        });
        break;
    }
  };
  /**
   * 新签合同(新版页面)
   * */
  goToContractNew = (record) => {
    sessionStorage.setItem('StdnApptMgmt', JSON.stringify(record));
    let title = '添加合同';
    browserHistory.push({
      pathname: OLDPAGEBASE,
      query: { src: `function/contract/editContractReactFeat.html`, title },
    });
  };
  /**
   * 加载学生预约列表
   * */
  loadData = (params, option) => {
    if (option && option === 'print') {
      this.updateState({ printLoading: true });
    } else {
      this.updateState({ listLoading: true });
    }
    const {
      studentNameVal,
      branchVal,
      campusVal,
      versionVal,
      gradeVal,
      dateVal,
      takeStateVal,
      pointLvlVal,
      appointmentTypeVal,
      subjectVal,
      pageCurrent,
      pageSize,
      takeStateDict,
      appointmentDict,
    } = this.state;
    let page, rows;
    if (params) {
      page = params.pageCurrent;
      rows = params.pageSize;
      if (option && option === 'print') {
        this.updateState({
          printCurrent: page,
        });
      } else {
        this.updateState({
          pageCurrent: page,
          pageSize: rows,
        });
      }
    } else {
      page = pageCurrent;
      rows = pageSize;
    }

    const apiParams = {
      studentName: studentNameVal,
      branchId: branchVal,
      blCampusId: this.buildParams(campusVal),
      yearId: versionVal,
      gradeId: this.buildParams(gradeVal),
      appointmentDate: dateVal,
      isCome: takeStateVal,
      classTypeId: pointLvlVal,
      appointmentWay: appointmentTypeVal,
      subjectId: this.buildParams(subjectVal),
      page,
      rows,
    };
    getStdnApptList(apiParams, true).then(
      (data) => {
        // console.log("success:",JSON.stringify(data.rows));
        jslog.report('query', {
          function_name: '课程管理>在线报读管理>学生预约管理>查询',
          page: data.page,
          total: data.records,
          size: data.rows.length,
        });

        let apiData = data.rows;
        let list = apiData.map((item, index) => {
          let {
            id,
            studentName,
            studentPhone,
            branchName,
            blCampusName,
            gradeName,
            appointmentDate,
            appointmentTime,
            subjectName,
            score,
            totalPoint,
            classTypeName,
            isCome,
            appointmentWay,
            classTypeId,
            branchId,
            blCampusId,
            gradeId,
            yearId,
            quarterId,
            subjectId,
            timeId,
            examId,
          } = item;
          return {
            id,
            index: index + 1 + Number(((page - 1) * rows).toFixed(0)),
            studentName,
            phoneNum: studentPhone,
            phoneNumForShow: StdntApptMgmt.buildPhoneNumForShow(studentPhone),
            branch: branchName,
            campus: blCampusName,
            grade: gradeName,
            appointmentDate,
            appointmentTime,
            examPoint: subjectName + '： ' + (score ? score + '/' + totalPoint : '无'),
            pointLvl: classTypeName ? classTypeName : '无效',
            takeStateVal: isCome,
            takeState: getDictTextByVal(takeStateDict, isCome.toString()),
            appointmentType: getDictTextByVal(appointmentDict, appointmentWay.toString()),
            subject: subjectName,
            classTypeName,
            score,
            branchId,
            blCampusId,
            gradeId,
            yearId,
            quarterId,
            subjectId,
            timeId,
            examId,
            classTypeId,
          };
        });

        if (option && option === 'print') {
          this.updateState({
            printLoading: false,
            printList: list,
            printTotal: data.records,
          });
        } else {
          this.updateState({
            listLoading: false,
            appointmentList: list,
            pageTotal: data.records,
          });
        }
        return true;
      },
      (err) => {
        this.updateState({ listLoading: false });
        console.error('# Load student appointment list error! #', err.text);
        return false;
      },
    );
  };
  /**
   * 辅助构建参数(用于loadData)
   * */
  buildParams = (params) => {
    return params ? (params.length > 1 ? params.join(',') : params[0]) : null;
  };
  /**
   * 确认考生到场
   * */
  submitStdnArrival = async (params) => {
    await this.updateState({ arrivedLoading: true });
    const { campus, resourceEntrance, consultant } = params;
    const { arrivedIdArr } = this.state;

    let apiParams = cloneDeep(arrivedIdArr);
    apiParams.forEach((item) => {
      item.cusType = resourceEntrance;
      item.intentionCampusId = campus;
      item.deliverTarget = consultant;
    });
    // console.log('* submitStdnArrival: ', apiParams);
    confirmStdnArrival(apiParams, true).then(
      (data) => {
        showMessage('确认成功！', 'success');
        this.setState({
          arrivedLoading: false,
          selectedRowKeys: [],
          arrivedIdArr: null,
          toggle_arrived: false,
        });
        this.updateSearch();
      },
      (err) => {
        showMessage(err.text, 'error');
        this.updateState({
          arrivedLoading: false,
        });
      },
    );
  };
  /**
   * 新增预约弹窗成功回调
   * */
  modalSubmit = () => {
    this.modalClose();
    this.updateReset();
    this.updateSearch();
  };
  /**
   * 关闭弹窗
   * */
  modalClose = () => {
    this.updateState({ toggle_newAppt: false });
  };

  render() {
    const { route, btnAuths } = this.props;
    const {
      name,
      loading,
      errorMsg,
      listLoading,
      pageCurrent,
      pageTotal,
      pageSize,
      branchDict,
      branchCampusDict,
      versionDict,
      gradeDict,
      takeStateDict,
      pointLvlDict,
      appointmentDict,
      subjectDict,
      studentNameVal,
      branchVal,
      campusVal,
      versionVal,
      gradeVal,
      dateVal,
      takeStateVal,
      pointLvlVal,
      appointmentTypeVal,
      appointmentList,
      selectedRowKeys,
      toggle_newAppt,
      subjectVal,
      modalType,
      editData,
      toggle_print,
      printLoading,
      printList,
      printCurrent,
      printTotal,
      printPageSize,
      printTitle,
      toggle_arrived,
      arrivedLoading,
    } = this.state;

    const searchData = {
      studentName: studentNameVal,
      branch: branchVal,
      campus: campusVal,
      version: versionVal,
      grade: gradeVal,
      date: dateVal,
      takeState: takeStateVal,
      pointLvl: pointLvlVal,
      appointmentType: appointmentTypeVal,
      subject: subjectVal,
    };
    const dict = {
      branchDict,
      branchCampusDict,
      versionDict,
      gradeDict,
      takeStateDict,
      pointLvlDict,
      appointmentDict,
      subjectDict,
    };
    const btnAuthConfirm = btnAuths.indexOf('EXAM_APPOINTMENT_CONFIRM_ARRIVED') === -1;
    // 临时屏蔽 签合同 按钮
    const btnAuthContract = true;
    // const btnAuthContract = btnAuths.indexOf('EXAM_APPOINTMENT_CONTRACT')===-1;
    const btnAuthEdit = btnAuths.indexOf('EXAM_APPOINTMENT_EDIT') === -1;
    const btnAuthContractNew = btnAuths.indexOf('EXAM_APPOINTMENT_CONTRACT_NEW') > -1;
    const columns = [
      { title: '序号', dataIndex: 'index', key: 'index', width: COL_WIDTH_S },
      { title: '学生姓名', dataIndex: 'studentName', key: 'studentName', width: 88 },
      { title: '客户手机', dataIndex: 'phoneNumForShow', key: 'phoneNumForShow', width: 120 },
      { title: '考试分公司', dataIndex: 'branch', key: 'branch', width: COL_WIDTH_M },
      { title: '考试学习中心', dataIndex: 'campus', key: 'campus', width: COL_WIDTH_M },
      { title: '年级', dataIndex: 'grade', key: 'grade', width: 80 },
      { title: '预约日期', dataIndex: 'appointmentDate', key: 'appointmentDate', width: COL_WIDTH_M },
      { title: '预约时间', dataIndex: 'appointmentTime', key: 'appointmentTime', width: COL_WIDTH_M },
      {
        title: '考试成绩',
        dataIndex: 'examPoint',
        key: 'examPoint',
        width: COL_WIDTH_L,
        filterDropdownVisible: false,
        filterIcon: (
          <Popover overlayClassName="exam-list-popover" content="科目：考试分数/试卷总分">
            <Icon className="exam-list-icon" type="question_circle" size={12} />
          </Popover>
        ),
        filterDropdown: <span className="empty-content"> </span>,
      },
      { title: '得分评级', dataIndex: 'pointLvl', key: 'pointLvl', width: COL_WIDTH_M },
      { title: '是否到场', dataIndex: 'takeState', key: 'takeState', width: COL_WIDTH_M },
      { title: '预约方式', dataIndex: 'appointmentType', key: 'appointmentType', width: COL_WIDTH_M },
      {
        title: '操作',
        key: 'action',
        width: 140,
        fixed: 'right',
        getActionConfig: (text, record, index) => {
          const { takeStateVal, score, classTypeId } = record;
          let check = takeStateVal.toString() === '1';
          let str = check ? '已确认' : '确认到场';
          let actionButtons = [
            {
              text: str,
              disabled: btnAuthConfirm || check,
              onClick: () => this.moreAction({ actType: 'confirm', record, index }),
            },
          ];
          if (classTypeId && classTypeId !== 'invalid') {
            actionButtons.push({
              text: '签合同(新)',
              onClick: () => this.moreAction({ actType: 'contractNew', record, index }),
              disabled: !btnAuthContractNew,
            });
          }
          if (!score || !classTypeId || classTypeId === 'invalid') {
            actionButtons.push({
              text: '修改',
              onClick: () => this.moreAction({ actType: 'edit', record, index }),
              disabled: btnAuthEdit || score,
            });
          }
          return actionButtons;
        },
      },
    ];

    return (
      <Page
        name={name}
        loading={loading}
        route={route}
        subHeader={
          <Search
            handleSearch={this.updateSearch}
            handleReset={this.updateReset}
            handleKeyword={this.updateKeyword}
            handleBtn={this.btnAction}
            btnType={selectedRowKeys.length > 0}
            dict={dict}
            data={searchData}
            btnAuths={btnAuths}
          />
        }
        fitWindowHeight
      >
        <List
          dataSource={appointmentList ? appointmentList : []}
          columns={columns}
          loading={!loading && listLoading}
          pagination={{
            current: pageCurrent,
            total: pageTotal,
            pageSize,
          }}
          locale={{ filterTitle: '' }}
          rowSelection={{
            selectedRowKeys,
            onChange: (keys) => this.updateState({ selectedRowKeys: keys }),
          }}
          onChange={({ current, pageSize }) => this.loadData({ pageCurrent: current, pageSize })}
        />
        {toggle_newAppt && (
          <ApptModal
            type={modalType}
            initData={editData}
            dict={dict}
            handleSubmit={this.modalSubmit}
            handleCancel={() => this.updateState({ toggle_newAppt: false, modalType: null, editData: null })}
          />
        )}
        {toggle_print && (
          <PrintModal
            loading={printLoading}
            header={printTitle}
            body={<PrintTable data={printList} />}
            footer={'第 ' + printCurrent + ' 页'}
            pagination={{
              current: printCurrent,
              total: printTotal,
              pageSize: printPageSize,
            }}
            changePage={this.loadData}
            handleCancel={() =>
              this.updateState({
                toggle_print: false,
                printLoading: false,
                printList: [],
                printCurrent: 1,
                printTotal: 0,
                printTitle: ' ',
              })
            }
          />
        )}
        {toggle_arrived && (
          <ArrivedModal
            loading={arrivedLoading}
            dict={dict}
            handleSubmit={this.submitStdnArrival}
            handleCancel={() => this.updateState({ toggle_arrived: false })}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  const { brenchId, campusId } = state.account.user;
  return {
    uBranchId: brenchId ? brenchId : null,
    uCampusId: campusId ? campusId : null,
    btnAuths: state.account.btnAuths || '',
    menusData: state.main.menus,
  };
};

export default connect(mapStateToProps)(StdntApptMgmt);
