import React from 'react';
import ReactDOM from 'react-dom';
import Checkbox from 'COMPONENT/Checkbox';
import Icon from 'COMPONENT/Icon';
import styles from './styles.less';
import colors from 'COMPONENT/theme/variables';
import Loading from 'COMPONENT/Loading';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import Loader from './loader';

export default class PreferenCard extends React.PureComponent {
  constructor(props) {
    super(props);
    this.state = {
      expand: false,
      loading: false,
      products: null,
    };
  }

  toggle = (e) => {
    e.stopPropagation();
    this.setState({
      expand: !this.state.expand,
    });
    this.loadProducts();
  };

  loadProducts = async () => {
    if (this.state.products) return;
    await new Promise((resolve) => {
      this.setState(
        {
          loading: true,
        },
        () => resolve(),
      );
    });
    const { id } = this.props;
    let products;
    try {
      products = await Loader.loadProducts(id);
    } catch (e) {
      showMessage(e.text);
      return this.setState({ loading: false });
    }

    this.setState({
      products,
      loading: false,
    });
  };

  onChange = (e) => {
    const { id, updateChecked } = this.props;
    e.stopPropagation();
    updateChecked(id, e.target.checked);
  };

  toggleChecked = () => {
    ReactDOM.findDOMNode(this.checkbox).querySelector('input').click();
  };

  render() {
    const { expand, loading, products } = this.state;
    const { editMode, title, body, checked } = this.props;
    return (
      <div className={styles.preferenCard} onClick={this.toggleChecked}>
        <Loading loading={loading} />
        <div className={styles.leftTop} />
        <div className={styles.rightTop} />
        <div className={styles.leftBottom} />
        <div className={styles.rightBottom} />

        {/*优惠名称*/}
        <div className={styles.firstLayer}>
          <div className={styles.cardHeader}>
            <Checkbox
              defaultChecked={checked}
              disabled={editMode === 'detail'}
              onChange={this.onChange}
              onClick={(e) => e.stopPropagation()}
              ref={(checkbox) => {
                this.checkbox = checkbox;
              }}
            />

            <div className={styles.cardTitle} style={expand ? { color: colors.primaryColor } : {}}>
              {title}
            </div>

            <Icon
              type={expand ? 'up' : 'down'}
              className={styles.cardIcon}
              color={colors.primaryColor}
              onClick={this.toggle}
            />
          </div>

          <div className={styles.cardBody}>{body}</div>
        </div>

        {/*优惠课程*/}
        {expand && (
          <div className={styles.secondLayer}>
            <div className={styles.leftMiddle} />
            <div className={styles.rightMiddle} />
            {products &&
              products.map((product, index) => {
                return (
                  <div key={index} style={{ marginBottom: '10px' }}>
                    {`产品${index + 1} ${product.productName}`}
                  </div>
                );
              })}
          </div>
        )}
      </div>
    );
  }
}
