import React from 'react';
import Modal from 'COMPONENT/Modal';
import PreferenCard from './PreferenCard';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import styles from './styles.less';
import commonStyle from '../commonStyle.less';
import Loader from './loader';
import NoData from 'COMPONENT/NoData';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import Select from 'COMPONENT/Select';
import { getUseCondition } from '../../../preferential/utils';

const customerConditionDict = [
  { text: '全部', val: 'ALL' },
  { text: '新客户', val: 'NEW' },
  { text: '老客户', val: 'OLD' },
];
export default class Index extends React.Component {
  constructor(props) {
    super();
    this.state = {
      cards: [],
      checkedCards: props.discountIds,
      startTime: props.startTime,
      endTime: props.endTime,
      discountBranch: props.discountBranch,
      loading: true,
      customerCondition: 'ALL',
    };
  }

  componentDidMount() {
    const { customerCondition } = this.state;
    this.loadData(customerCondition);
  }

  loadData = async (customerCondition) => {
    let data;
    const { startTime, endTime, discountBranch: branchId } = this.state;
    try {
      data = await Loader.loadPromotionByTime({ startTime, endTime, branchId, customerCondition });
    } catch (e) {
      showMessage(e.text);
      return this.setState({ loading: false });
    }

    const cards = data.map((item) => {
      return {
        id: String(item.promotionId),
        title: item.promotionName,
        body: getUseCondition(item),
      };
    });
    this.setState({
      cards,
      loading: false,
    });
  };

  updateChecked = (id, flag) => {
    //如果是勾选，则添加
    if (flag) {
      this.setState({
        checkedCards: this.state.checkedCards.concat([id]),
      });
      //如果是取消勾选，则剔除
    } else {
      this.setState({
        checkedCards: this.state.checkedCards.filter((ck) => ck !== id),
      });
    }
  };

  onOk = async () => {
    this.setState({ loading: true });
    try {
      if (this.state.checkedCards.length) {
        await Loader.getIsmorePromotionId({ promotionId: this.state.checkedCards });
      }
      const { onOk } = this.props;
      onOk(this.state.checkedCards);
    } catch (error) {
      this.modal.showOkTips(error.text);
    }
    this.setState({ loading: false });
  };
  render() {
    const { cards, checkedCards, loading, customerCondition } = this.state;
    // const { editMode } = this.props
    // 0429版本开始不允许编辑只允许查看 所以这里直接置为detail就可以了
    const editMode = 'detail';
    return (
      <Modal
        title="设置优惠"
        okBtnShow={editMode !== 'detail'}
        closable
        cancelText={editMode === 'detail' ? '关闭' : '取消'}
        onOk={this.onOk}
        onCancel={this.props.onCancel}
        ref={(c) => (this.modal = getWrappedInstance(c))}
        width={532}
        loading={loading}
      >
        <Select
          placeholder="客户"
          dict={customerConditionDict}
          value={customerCondition}
          onChange={(customerCondition) => {
            this.setState({ customerCondition, checkedCards: [] });
            this.loadData(customerCondition);
          }}
        />
        <div className={`${commonStyle.customScrollbar} ${styles.area}`}>
          {cards.map((card) => {
            return (
              <PreferenCard
                updateChecked={this.updateChecked}
                checked={checkedCards.indexOf(String(card.id)) > -1}
                key={card.id}
                editMode={editMode}
                id={card.id}
                title={card.title}
                body={card.body}
              />
            );
          })}
          {cards.length === 0 && <NoData />}
        </div>
      </Modal>
    );
  }
}
