import React from 'react';
import style from './index.less';
import List from 'COMPONENT/List';
import TextLink from 'COMPONENT/TextLink';

const { COL_WIDTH_L, createSetWidth } = List;
const setWidth = new createSetWidth();

const classPrefix = 'ActivatingFail';

const columns = [
  { title: '序号', dataIndex: 'number', key: 'number', width: setWidth(2) },
  { title: '学生姓名', dataIndex: 'studentName', key: 'studentName', width: setWidth(4) },
  { title: '结果', dataIndex: 'resultText', key: 'resultText', width: setWidth(4) },
  { title: '原因', dataIndex: 'reason', key: 'reason', width: COL_WIDTH_L },
];

const Bar = (props) => {
  return (
    <div className={style[`${classPrefix}-Bar`]}>
      <div className={style[`${classPrefix}-Bar-title`]}>{props.title}</div>
      <div className={style[`${classPrefix}-Bar-tools`]}>{props.tools}</div>
    </div>
  );
};

const Info = (props) => {
  return (
    <div className={style[`${classPrefix}-Info`]}>
      <div className={style[`${classPrefix}-Info-label`]}>{props.label}</div>
      <div className={style[`${classPrefix}-Info-content`]}>{props.children}</div>
    </div>
  );
};

const TipText = (props) => {
  const type = {
    info: '#108ee9',
    success: '#00a854',
    warning: '#ffbf00',
    fail: '#f04134',
  };
  return <div style={{ color: type[props.type] || type.info }}>{props.children}</div>;
};
TipText.defaultProps = {
  type: 'info',
};

const ActivatingFail = (props) => {
  return (
    <div className={style[classPrefix]}>
      <Bar
        title={[
          <Info label="激活成功" key="successNum">
            <TipText type="success">{props.successNum}</TipText>
          </Info>,
          <Info label="激活失败" key="failNum">
            <TipText type="fail">{props.failNum}</TipText>
          </Info>,
        ]}
        tools={<TextLink onClick={props.handleExport}>结果导出</TextLink>}
      />
      <List columns={columns} dataSource={props.list} />
    </div>
  );
};

export default ActivatingFail;
