import React from 'react';
import { connect } from 'dva';
import style from './index.less';
import { storeName } from '../store/config';
import Modal from 'COMPONENT/Modal';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import FollowUpPerson from './FollowUpPerson';
import Loading from 'COMPONENT/Loading';
import ActivatingFail from './ActivatingFail';
import ActivatingSuccess from './ActivatingSuccess';

const modalSizeS = 'S';
const modalSizeM = 'M';

const classPrefix = 'ModalBatchActivating';

const Block = (props) => {
  return <div className={style[`${classPrefix}-block`]}>{props.children}</div>;
};

class ModalBatchActivating extends React.Component {
  constructor(props) {
    super(props);
    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.updateFollowUpPerson = this.updateFollowUpPerson.bind(this);
    this.submit = this.submit.bind(this);
    this.batchFinish = this.batchFinish.bind(this);
    this.exportExcel = this.exportExcel.bind(this);
    this.getExcelFileUrl = this.getExcelFileUrl.bind(this);
    this.beforeDestroy = this.beforeDestroy.bind(this);
    this.state = {
      name: '批量激活',
      loading: false,
      confirmBtnVisible: true,
      modalSize: modalSizeS,
      compFollowUpPerson: true,
      compBatchProcess: false,
      compActivatingSuccess: false,
      compActivatingFail: false,
      cancelBtnText: '取消',
    };
    this.modalRef = null;
  }
  componentDidMount() {
    this.init();
  }
  componentWillUnmount() {
    this.beforeDestroy();
  }

  async init() {
    this.setState({ loading: true });
    await new Promise(this.initDict);
    this.setState({ loading: false });
  }

  initDict(resolve, reject) {
    this.props.dispatch({
      type: `${storeName}/initDict`,
      payload: {
        callback() {
          resolve();
        },
      },
    });
  }

  updateFollowUpPerson(followUpPersonId) {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload: { followUpPersonId },
    });
  }

  submit() {
    if (this.props.followUpPersonId) {
      const _self = this;
      this.props.dispatch({
        type: `${storeName}/submit`,
        payload: {
          customerStudents: _self.props.customerStudents,
          callback: _self.batchFinish,
        },
      });
      this.setState({
        confirmBtnVisible: false,
        compFollowUpPerson: false,
        compBatchProcess: true,
      });
    } else {
      this.modalRef.showOkTips('请先选择跟进人！');
    }
  }

  batchFinish(isFail) {
    let payload = {
      name: '本次激活结果',
      cancelBtnText: '关闭',
      modalSize: modalSizeM,
      compBatchProcess: false,
    };
    if (isFail) {
      payload.compActivatingFail = true;
    } else {
      payload.compActivatingSuccess = true;
    }
    this.setState(payload);
  }

  async exportExcel() {
    this.setState({ loading: true });
    await new Promise(this.getExcelFileUrl);
    this.setState({ loading: false });
  }

  getExcelFileUrl(resolve) {
    this.props.dispatch({
      type: `${storeName}/exportExcel`,
      payload: {
        callback() {
          resolve();
        },
      },
    });
  }

  beforeDestroy() {
    this.props.dispatch({ type: `${storeName}/resetModal` });
  }

  render() {
    return (
      <Modal
        className={style[classPrefix]}
        title={this.state.name}
        loading={this.state.loading}
        cancelText={this.state.cancelBtnText}
        onCancel={this.props.onCancel}
        onOk={this.submit}
        okBtnShow={this.state.confirmBtnVisible}
        ref={(c) => (this.modalRef = this.modalRef || getWrappedInstance(c))}
        widthSize={this.state.modalSize}
      >
        {this.state.compFollowUpPerson && (
          <Block>
            <FollowUpPerson
              dict={this.props.followUpPersonDict}
              value={this.props.followUpPersonId}
              onChange={this.updateFollowUpPerson}
            />
          </Block>
        )}
        {this.state.compBatchProcess && (
          <Block>
            <Loading loading={true} tip="批量激活中，请耐心等待" size="large" />
          </Block>
        )}
        {this.state.compActivatingSuccess && (
          <Block>
            <ActivatingSuccess successNum={this.props.successNum} />
          </Block>
        )}
        {this.state.compActivatingFail && (
          <ActivatingFail
            list={this.props.failList}
            successNum={this.props.successNum}
            failNum={this.props.failNum}
            handleExport={this.exportExcel}
          />
        )}
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    followUpPersonDict: state[storeName].followUpPersonDict,
    followUpPersonId: state[storeName].followUpPersonId,
    processPercentage: state[storeName].processPercentage,
    failList: state[storeName].failList,
    successNum: state[storeName].successNum,
    failNum: state[storeName].failNum,
  };
};

export default connect(mapStateToProps)(ModalBatchActivating);
