import { storeName } from './config';
import { clone } from 'lodash';
import { getFollowUpPersonDict } from 'UTIL/dict';
import { loginUser, runFunc, errorsHandle } from 'UTIL/common';
import { batchActivation, batchActivationResult, batchActivationResultExport } from 'SERVICE/student';
import file from 'UTIL/file';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

const errorsHandleFunc = errorsHandle();

const defaultState = {
  followUpPersonId: null,
  processPercentage: 0,
  failList: [],
  batchNo: null,
  failId: null,
  successNum: 0,
  failNum: 0,
};
Object.freeze(defaultState);

export default {
  namespace: storeName,
  state: {
    ...clone(defaultState),
    followUpPersonDict: null,
  },
  reducers: {
    updateState(state, { payload }) {
      return {
        ...state,
        ...payload,
      };
    },
    updateFailList(state, { payload: { list } }) {
      return {
        ...state,
        failList: list.map((item, index) => ({
          number: index + 1,
          studentName: item.studentName,
          resultText: '失败',
          reason: item.message,
        })),
      };
    },
  },
  effects: {
    *initDict({ payload: { callback } }, { select, put }) {
      let dict = yield select((state) => state[storeName].followUpPersonDict);
      if (!dict) {
        let apiParams = {
          showBelong: null,
          jobs: ['ZXS', 'ZXZG'],
        };
        if (loginUser.isCampusRole()) {
          apiParams.showBelong = '1';
        } else if (loginUser.isBranchRole()) {
          apiParams.showBelong = '2';
        }
        dict = yield getFollowUpPersonDict(apiParams).catch(errorsHandleFunc);
        yield put({
          type: 'updateState',
          payload: { followUpPersonDict: dict },
        });
      }
      runFunc(callback);
    },
    *submit({ payload: { customerStudents, callback } }, { select, put }) {
      let apiParams = yield select((state) => ({
        customerStudents,
        deliverTargetId: state[storeName].followUpPersonId,
      }));
      const rsp = yield batchActivation(apiParams).catch(errorsHandleFunc);
      if (rsp && rsp.resultCode === 0 && rsp.data) {
        yield put({
          type: 'updateState',
          payload: { batchNo: rsp.data },
        });
        yield put({
          type: 'checkProcess',
          payload: { callback },
        });
      } else {
        runFunc(callback, false);
      }
    },
    *checkProcess({ payload: { callback } }, { select, put }) {
      const apiParams = yield select((state) => ({ batchNo: state[storeName].batchNo }));
      const rsp = yield batchActivationResult(apiParams).catch(errorsHandleFunc);
      if (rsp && rsp.resultCode === 0) {
        const { finished, success, failed, details } = rsp.data;
        if (finished) {
          yield put({
            type: 'updateFailList',
            payload: { list: details },
          });
          yield put({
            type: 'updateState',
            payload: {
              successNum: success,
              failNum: failed,
            },
          });
          runFunc(callback, failed > 0);
        } else {
          yield put({
            type: 'checkProcess',
            payload: { apiParams, callback },
          });
        }
      } else {
        runFunc(callback, false);
      }
    },
    *exportExcel({ payload: { callback } }, { select, call }) {
      const apiParams = yield select((state) => ({ batchNo: state[storeName].batchNo }));
      const rsp = yield batchActivationResultExport(apiParams).catch(errorsHandleFunc);
      if (rsp && rsp.data) {
        file.downloadDirect(rsp.data);
        showMessage('导出成功！', 'success');
      }
      runFunc(callback);
    },
    *resetModal(action, { put }) {
      yield put({
        type: 'updateState',
        payload: {
          ...clone(defaultState),
        },
      });
    },
  },
};
