import React from 'react';
import { connect } from 'dva';
import { Page, List, SearchFilter, Input, Select, SearchField } from 'COMPONENT';
import { DatePicker } from 'antd';
import { getUserByRoleCodes } from 'SERVICE/common';
import { consultantStudentList } from 'SERVICE/student';
import { loadCommonDictNew } from 'UTIL/dict';

const title = '学生列表（咨询）';
const { RangePicker } = DatePicker;
const { COL_WIDTH_L, COL_WIDTH_M } = List;
const searchParams = {
  studyManagerId: '',
  homeroomTeacherId: '',
  studentName: '',
  schoolName: '',
  gradeId: '',
  time: [],
};

class ConsultantStudentList extends React.Component {
  columns = [
    {
      title: '学生编号',
      dataIndex: 'studentId',
      key: 'studentId',
      width: COL_WIDTH_L,
    },
    {
      title: '学生姓名',
      dataIndex: 'studentName',
      key: 'studentName',
      width: COL_WIDTH_M,
    },
    {
      title: '在读学校',
      dataIndex: 'schoolName',
      key: 'schoolName',
      width: COL_WIDTH_M,
    },
    {
      title: '年级',
      dataIndex: 'gradeName',
      key: 'gradeName',
      width: COL_WIDTH_M,
    },
    {
      title: '学习中心',
      dataIndex: 'blCampusName',
      key: 'blCampusName',
      width: COL_WIDTH_M,
    },
    {
      title: '学管师',
      dataIndex: 'studyManageName',
      key: 'studyManageName',
      width: COL_WIDTH_M,
    },
    {
      title: '学生班主任',
      dataIndex: 'homeroomTeacherName',
      key: 'homeroomTeacherName',
      width: COL_WIDTH_M,
    },
    {
      title: '学生状态',
      dataIndex: 'statusName',
      key: 'statusName',
      width: COL_WIDTH_M,
    },
    {
      title: '1对1学生状态',
      dataIndex: 'oneOnOneStatusName',
      key: 'oneOnOneStatusName',
      width: COL_WIDTH_M,
    },
  ];

  constructor(props) {
    super(props);
    this.mounted = false;
    this.state = {
      dataList: [],
      searchParams,
      loading: false,
      studyManagerDict: [],
      homeTeacherDict: [],
      pagination: {
        total: 0,
        current: 1,
        pageSize: 20,
      },
    };
    this.initDict = this.initDict.bind(this);
    this.transformParams = this.transformParams.bind(this);
    this.loadData = this.loadData.bind(this);
    this.handleSearch = this.handleSearch.bind(this);
    this.handleReset = this.handleReset.bind(this);
    this.onTimeChange = this.onTimeChange.bind(this);
    this.onSearchParamsChange = this.onSearchParamsChange.bind(this);
    this.onPageChange = this.onPageChange.bind(this);
    this.transferDict = this.transferDict.bind(this);
  }

  componentDidMount() {
    this.mounted = true;
    this.initDict();
    this.loadDict();
  }

  componentWillUnmount() {
    this.mounted = false;
  }

  async loadDict() {
    const [{ data: tearchData }, { data: studyTeacherData }] = await Promise.all([
      getUserByRoleCodes({ haveRoleCodes: ['CONSULTOR', 'CONSULTOR_DIRECTOR'], queryRoleCodes: ['HOMEROOM_TEACHER'] }),
      getUserByRoleCodes({
        haveRoleCodes: ['CONSULTOR', 'CONSULTOR_DIRECTOR'],
        queryRoleCodes: ['STUDY_MANAGER', 'STUDY_MANAGER_HEAD'],
      }),
    ]);
    const homeTeacherDict = this.transferDict(tearchData);
    const studyManagerDict = this.transferDict(studyTeacherData);
    if (this.mounted) {
      this.setState({ homeTeacherDict, studyManagerDict });
    }
  }

  initDict() {
    /*this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['grade'],
        callback: this.loadData
      }
    })*/
    loadCommonDictNew('STUDENT_GRADE').then((gradeDict) => {
      this.setState({ gradeDict });
      this.loadData();
    });
  }

  transferDict(data) {
    if (Array.isArray(data)) {
      return data.map((item) => ({ ...item, val: item.userId, text: item.name }));
    }
    return [];
  }

  transformParams() {
    const { searchParams, pagination } = this.state;
    const { current: pageNum, pageSize } = pagination;
    const { studyManagerId, homeroomTeacherId, studentName, schoolName, gradeId, time } = searchParams;
    let [beginCreateTime = '', endCreateTime = ''] = time;
    if (beginCreateTime) {
      /** case startDate is Moment object */
      beginCreateTime = beginCreateTime.startOf('day').valueOf();
    }
    if (endCreateTime) {
      /** case startDate is Moment object */
      endCreateTime = endCreateTime.endOf('day').valueOf();
    }
    return {
      studyManagerId,
      homeroomTeacherId,
      studentName,
      schoolName,
      gradeId,
      beginCreateTime,
      endCreateTime,
      pageNum,
      pageSize,
    };
  }

  async loadData() {
    try {
      this.setState({ loading: true });
      const params = this.transformParams();
      const { data, resultCode } = await consultantStudentList(params);
      /** use setState ensure component is not destoryed */
      if (resultCode === 0 && this.mounted) {
        const { list, total } = data;
        this.setState({
          dataList: list,
          pagination: { ...this.state.pagination, total },
        });
      }
    } finally {
      this.setState({ loading: false });
    }
  }

  handleSearch() {
    const { pagination } = this.state;
    this.setState({ pagination: { ...pagination, current: 1 } }, this.loadData);
  }

  handleReset() {
    const { pagination } = this.state;
    this.setState({ searchParams, pagination: { ...pagination, current: 1, pageSize: 20 } });
  }

  onTimeChange(value) {
    const params = this.state.searchParams;
    this.setState({ searchParams: { ...params, time: value } });
  }

  onPageChange({ current, pageSize }) {
    const { pagination } = this.state;
    this.setState({ pagination: { ...pagination, pageSize, current } }, this.loadData);
  }

  onSearchParamsChange(key) {
    return function updater(e) {
      const { searchParams } = this.state;
      const update = {};
      update[key] = e ? (e.target ? e.target.value : e) : '';
      this.setState({ searchParams: { ...searchParams, ...update } });
    }.bind(this);
  }

  render() {
    const { studyManagerDict, homeTeacherDict, loading, dataList, pagination, searchParams } = this.state;

    return (
      <Page
        name={title}
        route={this.props.route}
        fitWindowHeight
        loadingTip="正在加载数据"
        loading={loading}
        subHeader={
          <SearchFilter truncationNum={4} onSearch={this.handleSearch} onReset={this.handleReset}>
            <SearchField>
              <Select
                value={searchParams.studyManagerId}
                placeholder="学管师"
                allowClear
                showSearch
                dict={studyManagerDict}
                onChange={this.onSearchParamsChange('studyManagerId')}
              />
            </SearchField>
            <SearchField>
              <Select
                value={searchParams.homeroomTeacherId}
                placeholder="学生班主任"
                allowClear
                showSearch
                dict={homeTeacherDict}
                onChange={this.onSearchParamsChange('homeroomTeacherId')}
              />
            </SearchField>
            <SearchField>
              <Input
                value={searchParams.studentName}
                placeholder="学生姓名或编号"
                allowClear
                maxLength={50}
                onChange={this.onSearchParamsChange('studentName')}
              />
            </SearchField>
            <SearchField>
              <Input
                value={searchParams.schoolName}
                placeholder="在读学校"
                allowClear
                maxLength={50}
                onChange={this.onSearchParamsChange('schoolName')}
              />
            </SearchField>
            <SearchField>
              <Select
                value={searchParams.gradeId}
                placeholder="年级"
                allowClear
                showSearch
                dict={this.state.gradeDict}
                onChange={this.onSearchParamsChange('gradeId')}
              />
            </SearchField>
            <SearchField style={{ flex: '0 0 20%' }}>
              <RangePicker
                value={searchParams.time}
                format="YYYY-MM-DD"
                placeholder={['开始时间', '结束时间']}
                size="large"
                onChange={this.onTimeChange}
              />
            </SearchField>
          </SearchFilter>
        }
      >
        <List dataSource={dataList} columns={this.columns} pagination={pagination} onChange={this.onPageChange} />
      </Page>
    );
  }
}

export default connect(function mapStateToProps(state) {
  return {
    gradeDict: state.dict.grade,
  };
})(ConsultantStudentList);
