import React from 'react';
import { Row, Col } from 'antd';
import Modal from 'COMPONENT/Modal';
import Button from 'COMPONENT/Button';
import Block from 'COMPONENT/Block';
import FormItem from 'COMPONENT/FormItem';
import { loadDetail } from '../loaders';
import { getDictTextByVal } from 'UTIL/dict';
import DatePicker from 'COMPONENT/DatePicker';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import TextArea from 'COMPONENT/TextArea';
import Popover from 'COMPONENT/Popover';
import QrcodeFileUpload from 'COMPONENT/QrcodeFileUpload';
import Upload from 'COMPONENT/Upload';
import IconButton from 'COMPONENT/IconButton';
import FileList from 'COMPONENT/FileList';
import loaders from '../loaders';
import styles from './styles.less';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import showAlert from 'COMPONENT/utils/globalTips/showAlert';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import { FILE_TYPE_MAP_EXTENSION } from 'COMPONENT/FileIcon/config';
import { saveDetail } from '../loaders';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import { getCommonErrorTips } from 'COMPONENT/utils/form';
import { getIconSrc } from '../utils';
import moment from 'moment';
import { getCurrentDateTime } from 'SERVICE/common';

const maxUploadFileCount = 9;
const validateFileTypes = ['word', 'excel', 'ppt', 'pdf', 'txt', 'zip', 'rar', 'png', 'jpg', 'bmp'];
const clearSty = { style: { clear: 'left' } };

class DetailModal extends React.Component {
  constructor(props) {
    super(props);
    const { type, data, productDict } = props;
    this.fileIdx = 1;
    let initData;
    if (type === 'new') {
      initData = {
        followMode: null, // {string} 跟进方式
        follower: props.type === 'new' ? props.userName : null, // {string} 跟进人
        followerId: props.type === 'new' ? props.userId : null, // {string} 跟进人ID
        date: null, // {string} 跟进时间
        content: `一、学习情况


二、亮点及问题


三、家长反馈


四、其他

`, // {string} 跟进内容
        nextDate: null, // {string} 下次跟进时间
        fileList: [], // {object[]} 附件列表
        productTypes: [], //产品类型
        miniClassArray: [],
        subjectArray: [],
      };
    } else {
      let {
        mode,
        follower,
        followerId,
        date,
        content,
        nextDate,
        fileList,
        productTypes,
        miniClassArray,
        subjectArray,
      } = data;
      if (!productTypes) {
        productTypes = [];
      }
      let productArray = productDict.map((v, k) => v.val);
      for (let i = 0; i < productTypes.length; i++) {
        if (!productArray.includes(productTypes[i])) {
          productTypes.splice(i, 1);
          i--;
        }
      }
      const newFileList = fileList.map((file) => {
        return { ...file, listId: this.genFileId() };
      });

      initData = {
        followMode: mode,
        follower,
        followerId,
        date,
        content,
        nextDate,
        fileList: newFileList,
        productTypes,
        miniClassArray,
        subjectArray,
      };
    }
    this.state = {
      loading: false, // {boolean} 内容加载状态
      checking: false, // {boolean} 校验开启状态
      ...initData,
    };
    this.errors = {};
    this.onlineDate = null;
  }

  componentDidMount() {
    this.init();
  }

  init = () => {
    this.setState({ loading: true });
    getCurrentDateTime().then((data) => {
      this.onlineDate = data.split(' ')[0];
      this.setState({ loading: false });
    });
  };

  genFileId = () => {
    return '' + this.fileIdx++;
  };

  save = () => {
    const {
      checking,
      followMode,
      follower,
      followerId,
      date,
      content,
      nextDate,
      fileList,
      miniClassArray,
      subjectArray,
      productTypes,
    } = this.state;
    if (!checking) {
      this.setState({ checking: true });
    }
    let errorTips = getCommonErrorTips(this.errors);
    if (!errorTips) {
      if (content.replace(/\n+/g, '').length < 20) {
        errorTips = '跟进内容不得少于20字';
      }
    }
    if (errorTips) {
      this.modal.showOkTips(errorTips);
      return;
    }
    if (fileList.some((file) => file.error)) {
      this.modal.showOkTips('请先处理上传失败的文件');
      return;
    }
    if (
      fileList.some((file) => {
        const name = file.name;
        if (name.substring(0, name.lastIndexOf('.')) === '') {
          return true;
        }
        return false;
      })
    ) {
      this.modal.showOkTips('附件名不能为空');
      return;
    }
    const { onSaveSuccess, data, type, studentId } = this.props;
    this.setState({ loading: true });
    saveDetail(type, studentId, {
      id: type === 'new' ? null : data.id,
      followMode,
      follower,
      followerId,
      date,
      content: this.formatContent(content),
      nextDate,
      fileList,
      miniClassArray: productTypes.includes('SMALL_CLASS') ? miniClassArray : [],
      subjectArray: productTypes.includes('ONE_ON_ONE_COURSE') ? subjectArray : [],
    }).then(
      () => {
        this.setState({ loading: false });
        onSaveSuccess(type === 'new' ? null : data.id);
        showMessage('保存成功', 'success');
      },
      (e) => {
        this.setState({ loading: false });
        if (e.text) {
          this.modal.showOkTips(e.text);
        } else {
          showMessage('保存失败');
        }
      },
    );
  };

  formatContent = (content) => {
    if (!content) {
      return '';
    }
    return content.replace(/\n+/g, '\n');
  };

  get2dUploadImgInfo = (info) => {
    console.log(info);
    const { fileList } = this.state;
    if (fileList.length >= maxUploadFileCount) {
      return;
    }
    const newFileList = [...fileList];
    info.forEach((file) => {
      const name = file.realFileName;
      const src = file.filePath;
      if (!newFileList.some((nf) => nf.src === src) && newFileList.length < maxUploadFileCount) {
        newFileList.push({ id: file.fileid, name, src, iconSrc: getIconSrc(name, src), listId: this.genFileId() });
      }
    });
    this.setState({
      fileList: newFileList,
    });
  };

  onUploadChange = (info) => {
    this.setState({
      fileList: this.state.fileList.map((file) => {
        const newFile = info.find((inf) => inf.fileKey === file.fileKey);
        if (newFile) {
          const name = file.name;
          const src = newFile.src;
          return {
            ...newFile,
            id: newFile['x:fileid'],
            src,
            name,
            listId: file.listId,
            error: newFile.errorCode ? '上传失败' : null,
            iconSrc: newFile.errorCode ? null : getIconSrc(name, src),
          };
        }
        return file;
      }),
    });
  };

  checkLength = (files) => {
    console.log(files);
    let filesMayUpload = [];
    let filesWillUpload = [];
    const restCount = maxUploadFileCount - this.state.fileList.length;
    files.slice(0, restCount).map((file) => {
      const listId = this.genFileId();
      let newFile = { fileKey: file.fileKey, name: file.name, listId };
      let fileExtends = [];
      validateFileTypes.forEach((type) => {
        fileExtends = [...fileExtends, ...FILE_TYPE_MAP_EXTENSION[type]];
      });
      const name = file.name;
      const idx = name.lastIndexOf('.');
      const nameExtend = name.slice(idx + 1);
      if (file.size > 1024 * 1024 * 10) {
        newFile = { ...newFile, error: '文件大小超过了10MB' };
      } else if (fileExtends.indexOf(nameExtend.toLowerCase()) < 0) {
        newFile = { ...newFile, error: '文件格式不正确' };
      } else if (name.length - nameExtend.length - 1 > 50) {
        newFile = { ...newFile, error: '文件名超过50字' };
      } else {
        filesWillUpload.push(file);
      }
      filesMayUpload.push(newFile);
    });
    if (restCount < files.length) {
      return new Promise((res, rej) => {
        showConfirm(
          `最多上传${maxUploadFileCount}个文件，超出部分将不上传`,
          () => {
            this.setState({ fileList: [...this.state.fileList, ...filesMayUpload] });
            res(filesWillUpload);
          },
          () => {
            rej();
          },
        );
      });
    } else {
      this.setState({ fileList: [...this.state.fileList, ...filesMayUpload] });
      return filesWillUpload;
    }
    /*     if (this.state.fileList.length + files.length > 9) {
          //1
          // showMessage('最多只能上传2张图片');
          // return false;

          //2
          return new Promise((res, rej) => {
            showConfirm('最多上传9个文件，超出部分将不上传', () => {
              res(true)
            }, () => {
              rej(false)
            })
          })
        }   */
  };

  onFileNameChange = (listId, name) => {
    this.setState({
      fileList: this.state.fileList.map((file) => {
        if (file.listId === listId) {
          return { ...file, name };
        }
        return file;
      }),
    });
  };

  errorChange = (key, error) => {
    this.errors[key] = error;
  };

  getDisabledDate = (date) => {
    // if (!date) {
    //   return false
    // }
    const { nextDate } = this.state;
    if (nextDate) {
      return (
        moment(date).valueOf() > moment(nextDate).valueOf() ||
        moment(date).valueOf() > moment(this.onlineDate).valueOf()
      );
    } else {
      return moment(date).valueOf() > moment(this.onlineDate).valueOf();
    }
  };

  getDisabledNextDate = (nextDate) => {
    const { date } = this.state;
    if (!date || !nextDate) {
      return false;
    }
    return moment(date).valueOf() > moment(nextDate).valueOf();
  };
  changeProductTypes = (val) => {
    this.setState({ productTypes: val });
    if (!val.includes('ONE_ON_ONE_COURSE')) {
      delete this.errors['subjectArray'];
    }
    if (!val.includes('SMALL_CLASS')) {
      delete this.errors['miniClassArray'];
    }
  };
  render() {
    const { type, followModeDict, onClose, subjectDict, miniClassDict, productDict } = this.props;
    let {
      loading,
      followMode,
      follower,
      date,
      content,
      nextDate,
      productTypes,
      miniClassArray,
      subjectArray,
      fileList,
      checking,
    } = this.state;
    const FormItemConfig = {
      labelWidth: 100,
      style: { marginBottom: 19 },
    };
    if (!productTypes) productTypes = [];
    return (
      <Modal
        loading={loading}
        okBtnDisabled={fileList.some((file) => file.showProgress)}
        onCancel={onClose}
        title={type === 'new' ? '新增跟进回访' : type === 'view' ? '跟进回访详情' : '编辑跟进回访'}
        width={818}
        okBtnShow={type === 'view' ? false : true}
        onOk={this.save}
        noLine
        className={styles.modal + ' ' + styles.xModal}
        ref={(c) => (this.modal = getWrappedInstance(c))}
      >
        <Block title={name} style={{ margin: '20px -45px 0' }}>
          <Row gutter={10}>
            <Col span={12}>
              <FormItem {...FormItemConfig} name="跟进方式" required>
                <Select
                  dict={followModeDict}
                  value={followMode}
                  onChange={(val) => this.setState({ followMode: val })}
                  validator={{
                    silent: !checking,
                    onErrorChange: (e) => this.errorChange('followMode', e),
                  }}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem {...FormItemConfig} name="跟进人">
                {follower}
              </FormItem>
            </Col>
            <Col span={12} {...clearSty}>
              <FormItem {...FormItemConfig} name="跟进时间" required>
                <DatePicker
                  value={date}
                  onChange={(val) => this.setState({ date: val })}
                  validator={{
                    silent: !checking,
                    onErrorChange: (e) => this.errorChange('date', e),
                  }}
                  disabledDate={this.getDisabledDate}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem {...FormItemConfig} name="下次跟进时间" required>
                <DatePicker
                  value={nextDate}
                  onChange={(val) => this.setState({ nextDate: val })}
                  validator={{
                    silent: !checking,
                    onErrorChange: (e) => this.errorChange('nextDate', e),
                  }}
                  disabledDate={this.getDisabledNextDate}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem {...FormItemConfig} name="产品类型" required>
                <MultiSelect
                  dict={productDict}
                  value={productTypes}
                  onChange={this.changeProductTypes}
                  dropdownClassName={styles.multiSelect}
                  validator={{
                    silent: !checking,
                    onErrorChange: (e) => this.errorChange('productType', e),
                  }}
                />
              </FormItem>
            </Col>
            {productTypes.includes('ONE_ON_ONE_COURSE') && (
              <Col span={12}>
                <FormItem {...FormItemConfig} name="一对一科目" required>
                  <MultiSelect
                    dict={subjectDict}
                    value={subjectArray}
                    onChange={(val) => this.setState({ subjectArray: val })}
                    dropdownClassName={styles.multiSelect}
                    noOptionTips={'请先到课时管理分配课时'}
                    validator={{
                      silent: !checking,
                      onErrorChange: (e) => this.errorChange('subjectArray', e),
                    }}
                  />
                </FormItem>
              </Col>
            )}
            {productTypes.includes('SMALL_CLASS') && (
              <Col span={12}>
                <FormItem {...FormItemConfig} name="班课名称" required>
                  <MultiSelect
                    dict={miniClassDict}
                    value={miniClassArray}
                    onChange={(val) => this.setState({ miniClassArray: val })}
                    dropdownClassName={styles.multiSelect}
                    noOptionTips={'请先报入小班'}
                    validator={{
                      silent: !checking,
                      onErrorChange: (e) => this.errorChange('miniClassArray', e),
                    }}
                  />
                </FormItem>
              </Col>
            )}
            <Col span={24}>
              <FormItem {...FormItemConfig} name="跟进内容" required>
                <TextArea
                  placeholder="建议从学习情况、亮点及问题、家长反馈三方面进行记录，至少输入20字，最多输入500字"
                  rows={8}
                  value={content}
                  onChange={(e) => this.setState({ content: e.target.value })}
                  wordCount={true}
                  maxLength={500}
                  // style={{marginBottom: -10}}
                  validator={{
                    silent: !checking,
                    onErrorChange: (e) => this.errorChange('content', e),
                  }}
                />
              </FormItem>
            </Col>
            <Col span={24}>
              <FormItem {...FormItemConfig} name="相关附件">
                <div>
                  <div style={{ display: 'inline-block', marginRight: 20 }}>
                    {fileList.length >= maxUploadFileCount ? (
                      <Button disabled>点击上传</Button>
                    ) : (
                      <Upload
                        onChange={this.onUploadChange}
                        multiple={true}
                        beforeUpload={(files) => this.checkLength(files)}
                      >
                        <Button>点击上传</Button>
                      </Upload>
                    )}
                  </div>
                  <div style={{ display: 'inline-block' }}>
                    {fileList.length >= maxUploadFileCount ? (
                      <Button disabled>二维码传图</Button>
                    ) : (
                      <Popover
                        content={
                          <QrcodeFileUpload height={160} width={160} callback={this.get2dUploadImgInfo} multiple />
                        }
                        overlayClassName={styles.codePopWrap}
                        placement="right"
                      >
                        <Button>扫码传图</Button>
                      </Popover>
                    )}
                  </div>
                  <div className={styles.tips}>
                    支持格式有{validateFileTypes.join('、')}，最多上传9个附件，可多选上传
                  </div>
                </div>
                <div style={{ marginTop: 5 }}>
                  <FileList
                    data={fileList.map((file) => {
                      return { ...file, fileId: file.id, id: file.listId };
                    })}
                    onChange={(list) =>
                      this.setState({
                        fileList: list.map((file) => {
                          return { ...file, id: file.fileId, listId: file.id };
                        }),
                      })
                    }
                  />
                </div>
              </FormItem>
            </Col>
          </Row>
        </Block>
      </Modal>
    );
  }
}

export default DetailModal;
