import React from 'react';
import styles from './styles.less';
import TextLink from 'COMPONENT/TextLink';
import Input from 'COMPONENT/Input';
import Icon from 'COMPONENT/Icon';
import IconButton from 'COMPONENT/IconButton';
import FileIcon from 'COMPONENT/FileIcon';
import styleVars from 'COMPONENT/theme/variables';
import classnames from 'classnames';
import { Progress } from 'antd';

class FileItem extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      editing: false,
      ...this.getName(props),
    };
  }

  componentWillReceiveProps = (nextProps) => {
    if (nextProps.data && 'name' in nextProps.data) {
      this.setState(this.getName(nextProps));
    }
  };

  getName = (props) => {
    if (props.data && 'name' in props.data) {
      const fullName = props.data.name;
      const idx = fullName.lastIndexOf('.');
      return { name: fullName.slice(0, idx), nameExtend: fullName.slice(idx) };
    }
    return { name: null, nameExtend: null };
  };

  onOk = () => {
    const { name, nameExtend } = this.state;
    this.props.onNameChange(name + nameExtend);
    this.setState({ editing: false });
  };

  getNameEle = () => {
    const { type } = this.props;
    const { editing, name, nameExtend } = this.state;
    let content;
    if (editing) {
      content = (
        <div>
          <Input
            className={styles.nameLeft}
            value={name}
            style={{ width: 230 }}
            onChange={(e) => this.setState({ name: e.target.value })}
          />
          <div className={styles.nameRight}>{nameExtend}</div>
        </div>
      );
    } else {
      if (type === 'view') {
        content = (
          <TextLink type="secondary" onClick={null}>
            {name + nameExtend}
          </TextLink>
        );
      } else {
        content = name + nameExtend;
      }
    }
    return <div className={styles.nameWrap}>{content}</div>;
  };

  getBtnsEle = () => {
    const {
      type,
      data: { error, src, showProgress },
      onDelete,
    } = this.props;
    const { editing } = this.state;
    let content;
    if (type === 'view') {
      content = (
        <a target="_blank" download="xx" href={src} className={styles.btnItem}>
          <IconButton type="move_down_mc" />
        </a>
      );
    } else if (showProgress) {
    } else if (error) {
      content = (
        <span>
          <IconButton type="delete_mc" className={styles.btnItem} onClick={onDelete} />
        </span>
      );
    } else {
      if (editing) {
        content = (
          <span>
            <TextLink onClick={this.onOk} className={styles.btnItem}>
              保存
            </TextLink>
            <TextLink type="secondary" className={styles.btnItem} onClick={() => this.setState({ editing: false })}>
              取消
            </TextLink>
          </span>
        );
      } else {
        content = (
          <span>
            <IconButton type="edit_mc" className={styles.btnItem} onClick={() => this.setState({ editing: true })} />
            <IconButton type="delete_mc" className={styles.btnItem} onClick={onDelete} />
          </span>
        );
      }
    }
    return <div className={styles.btnsWrap}>{content}</div>;
  };

  render() {
    const {
      type,
      data: { showProgress, percent, error, src },
      onNameChange,
      onDelete,
    } = this.props;
    const { editing, name, nameExtend } = this.state;

    const cls = {
      [styles.wrap]: 1,
      [styles.wrapEditing]: editing,
    };

    return (
      <div className={classnames(cls)}>
        <div className={styles.main}>
          <FileIcon extensionName={nameExtend.slice(1)} className={styles.icon} />
          {this.getNameEle()}
          {this.getBtnsEle()}
        </div>
        {showProgress && !error && percent < 100 && (
          <div className={styles.progressWrap}>
            <Progress percent={percent} size="small" showInfo={false} strokeWidth={4} />
          </div>
        )}
        {error && (
          <div className={styles.errorWrap}>
            <Icon type="warning" color={styleVars.highlightColor} className={styles.btnItem} size={12} />
            <span className={styles.error}>{error}</span>
          </div>
        )}
      </div>
    );
  }
}

export default FileItem;
