import React from 'react';
import moment from 'moment';
import styles from './styles.less';
import TextLink from 'COMPONENT/TextLink';
import classnames from 'classnames';
import FileList from 'COMPONENT/FileList';
import { getDictTextByVal } from 'UTIL/dict';
import PopConfirm from 'COMPONENT/PopConfirm';
import previewFile from 'UTIL/previewFile';

const wordPartCount = 125;

class ListCard extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      contentExpand: false,
    };
  }

  componentDidMount() {}

  getContentComp = (lines) => {
    if (!lines) {
      return null;
    }
    return (
      <div>
        {lines.map((line, idx) => {
          if (line === true) {
            return (
              <div key={idx} style={{ visibility: 'hidden' }}>
                ...
              </div>
            );
          }
          return <div key={idx}>{line}</div>;
        })}
      </div>
    );
  };

  isFullCorner = (str) => {
    if (['~', '@', '#', '%', '^', '&', '<', '>'].indexOf(str) >= 0) {
      return true;
    }
    return /[^\x00-\xff]/.test(str);
  };

  getFullLength = (str) => {
    let count = 0;
    for (let i = 0; i < str.length; i++) {
      const ci = str[i];
      if (this.isFullCorner(ci)) {
        count += 1;
      } else {
        count += 0.5;
      }
    }
    return count;
  };

  sliceFullCount = (str, cnt) => {
    let count = 0;
    let s = '';
    for (let i = 0; i < str.length; i++) {
      const ci = str[i];
      s += ci;
      if (this.isFullCorner(ci)) {
        count += 1;
      } else {
        count += 0.5;
      }
      if (Math.ceil(count) >= cnt) {
        break;
      }
    }
    return s;
  };

  getContentDetail = (content) => {
    const lineCount = 66;
    let lines = [];
    const sections = content.split('\n');
    sections.forEach((sec) => {
      let count = 0;
      let line = '';
      for (let i = 0; i < sec.length; i++) {
        const ci = sec[i];
        line += ci;
        if (this.isFullCorner(ci)) {
          count++;
        } else {
          count += 0.5;
        }
        if (Math.ceil(count) >= lineCount) {
          lines.push(line);
          count = 0;
          line = '';
        }
      }
      if (count > 0) {
        lines.push(line);
        count = 0;
        line = '';
      }
    });
    let partLines = null;
    if (lines.length > 3) {
      partLines = lines.slice(0, 3).map((line, idx) => {
        if (idx === 2) {
          return this.sliceFullCount(line, lineCount - 4) + '...';
        }
        return line;
      });
      if (this.getFullLength(lines[lines.length - 1]) > lineCount - 3) {
        lines.push(true);
      }
    }
    return { content: this.getContentComp(lines), contentPart: this.getContentComp(partLines) };
  };

  render() {
    let {
      data: {
        id,
        role,
        date,
        mode,
        follower,
        followerId,
        creatorId,
        productTypes,
        content: contentProp,
        nextDate,
        fileList,
        createTime,
        miniClassArray,
        subjectArray,
      },
      followModeDict,
      onEdit,
      onDelete,
      hasLine,
      userId,
      auth,
    } = this.props;
    const { contentExpand } = this.state;
    const cls = {
      [styles.wrap]: 1,
      [styles.wrapHasLine]: hasLine,
    };
    const contentCls = {
      [styles.content]: 1,
      [styles.contentExpand]: contentExpand,
    };
    const { content, contentPart } = this.getContentDetail(contentProp);
    let showEdit;
    if (!productTypes) {
      productTypes = ['ONE_ON_ONE_COURSE', 'SMALL_CLASS'];
    }
    productTypes.forEach((v, k) => {
      if (auth.includes(v)) {
        showEdit = true;
      }
    });
    return (
      <div className={classnames(cls)} id={`studentFollowVisitListItem_${id}`}>
        <div className={styles.inner}>
          <div className={styles.header}>
            <div className={styles.headerLeft}>{date}</div>
            {userId === creatorId && (
              <div className={styles.headerRight}>
                {showEdit && <TextLink onClick={onEdit}>编辑</TextLink>}
                <PopConfirm content={`你确定要删除本次跟进回访吗？`} onOk={onDelete}>
                  <TextLink style={{ marginLeft: 20 }}>删除</TextLink>
                </PopConfirm>
              </div>
            )}
          </div>
          <div className={styles.body}>
            <div className={styles.bodyLeft}>
              <div className={styles.head + ' ' + (role === 'TEATCHER' ? styles.mark : '')}>{follower.slice(0, 1)}</div>
            </div>
            <div className={styles.bodyRight}>
              <div className={styles.titleBar}>
                <span>
                  <div className={styles.follower}>{follower}</div>
                  {mode && <div className={styles.mode}>{getDictTextByVal(followModeDict, mode)}</div>}
                  {miniClassArray.length > 0 && <div className={styles.productType}>十人课堂</div>}
                  {subjectArray.length > 0 && <div className={styles.productType}>1对1</div>}
                  <div className={styles.createTime}>创建时间：{moment(createTime).format('YYYY-MM-DD')}</div>
                </span>
                <span>
                  <div className={styles.nextDate}>
                    <span style={{ marginRight: 20 }}>下次跟进时间</span>
                    {nextDate || '-'}
                  </div>
                </span>
              </div>
              <div className={classnames(contentCls)}>
                {contentPart && !contentExpand ? contentPart : content}
                {contentPart && (
                  <TextLink className={styles.moreBtn} onClick={() => this.setState({ contentExpand: !contentExpand })}>
                    {contentExpand ? '收起' : '更多'}
                  </TextLink>
                )}
              </div>
              <FileList data={fileList} type="view" onPreview={previewFile} />
            </div>
          </div>
        </div>
      </div>
    );
  }
}

export default ListCard;
