import React from 'react';
import { connect } from 'dva';
import moment from 'moment';
import Page from 'COMPONENT/Page';
import SearchBar from './SearchBar';
import Select from 'COMPONENT/Select';
import Checkbox from 'COMPONENT/Checkbox';
import Input from 'COMPONENT/Input';
import DatePicker from 'COMPONENT/DatePicker';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import {
  loadList,
  deleteRecord,
  loadYearDict,
  getMiniClass,
  getSubject,
  loadStudyManagerDict,
  loadTeacherDict,
  loadHomeroomTeacherDict,
} from './loaders';

import Button from 'COMPONENT/Button';
import DetailModal from './DetailModal';
import styles from './styles.less';
import listCardStyles from './ListCard/styles.less';
import ListSec from './ListSec';
import dicts from './dicts';
import { addClass, removeClass } from 'COMPONENT/utils/dom';
import FormItem from 'COMPONENT/FormItem';
import NoData from 'COMPONENT/NoData';

const initSearchKeys = {
  startTime: moment().format('YYYY') - 2 + '-01-01', // {string} 查询条件 开始年份
  endTime: moment().format('YYYY-MM-DD'), // {string} 查询条件 结束年份
  followMode: null, // {string} 查询条件 跟进方式
};
const productDict = [
  { val: 'ONE_ON_ONE_COURSE', text: '1对1' },
  { val: 'SMALL_CLASS', text: '十人课堂' },
];

class FollowVisit extends React.Component {
  auth = 'all';
  modalProductDict = [
    { val: 'ONE_ON_ONE_COURSE', text: '1对1' },
    { val: 'SMALL_CLASS', text: '十人课堂' },
  ];

  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 内容加载状态
      detailModalVis: false, // {boolean} 详情弹框显示状态
      detailModalType: null, // {string} 详情弹框类型，新增为'new'
      detailModalDataId: null, // {string} 详情弹框的数据ID

      startYearDict: [], // {object[]} 字典 开始年份
      endYearDict: [], //  {object[]} 字典 结束年份
      followModeDict: dicts.followDict, //  {object[]} 字典 跟进方式
      list: [], // {object[]} 列表数据，格式如下：
      /*
        [{
          year {string} - 年份
          data {object[]} - 一年的跟进信息列表
          [{
            date {string} - 跟进日期
            mode {string} - 跟进方式
            follower {string} - 跟进人
            content {string} - 跟进内容
            nextDate {string} - 下次跟进日期
            fileList {object[]} - 附件列表，结构为：{ id, name, src }
          }]
        }]
      */
      searchKeys: initSearchKeys, //当前列表的查询条件
      ...initSearchKeys,
    };
    this.studentId = this.props.params.studentId;
    this.studentName = this.props.params.studentName;
    this.studyManagerName = this.props.params.studyManagerName;
  }

  componentDidMount() {
    this.initContent();
    this.pageScroller = this.pageRef.getScrollerElement();
  }

  initContent = () => {
    const { searchKeys } = this.state;
    this.setState({ loading: true });
    let {
      params: { from, fileIds },
    } = this.props;
    Promise.all([
      from ? loadList(this.studentId, fileIds, true) : loadList(this.studentId, searchKeys),
      loadYearDict(),
      getSubject(this.studentId),
      getMiniClass(this.studentId),
      loadStudyManagerDict(this.studentId),
      loadTeacherDict(this.studentId),
      loadHomeroomTeacherDict(this.studentId),
    ]).then(([list, yearDict, subjectDict, miniClassDict, managerDict, teacherDict, homeroomTeacherDict]) => {
      let {
        user: { userId },
      } = this.props;
      this.judgeAuth(userId, managerDict, teacherDict, homeroomTeacherDict);
      this.setState({
        loading: false,
        list,
        startYearDict: yearDict,
        endYearDict: yearDict,
        subjectDict,
        miniClassDict,
        managerDict,
        teacherDict,
        homeroomTeacherDict,
      });
    });
  };

  onSaveSuccess = (id) => {
    this.setState({ detailModalVis: false });
    this.reloadAndLocate(id);
  };

  reloadAndLocate = (id) => {
    const { list, searchKeys } = this.state;
    this.setState({ loading: true });
    loadList(this.studentId, searchKeys).then(
      (newList) => {
        let newId;
        if (id) {
          newId = id;
        } else {
          newList.forEach((sec) => {
            const { data } = sec;
            if (data) {
              data.forEach((item) => {
                if (!this.isItemInList(item.id, list)) {
                  newId = item.id;
                }
              });
            }
          });
        }
        this.setState({ list: newList }, () => {
          setTimeout(() => {
            this.setState({ loading: false });
            this.locate(newId);
          }, 500);
        });
      },
      () => {
        this.setState({ loading: false });
      },
    );
  };

  isItemInList = (id, list) => {
    return list.some((sec) => {
      const { data } = sec;
      if (data) {
        return data.some((item) => item.id === id);
      }
      return false;
    });
  };

  locate = (id) => {
    if (id) {
      const $pageBook = document.querySelector('.PageBook');
      const $item = document.getElementById(`studentFollowVisitListItem_${id}`);
      const pageHeaderH = 60;
      const pageBookH = $pageBook.offsetHeight;
      const scrollH = this.pageScroller.scrollHeight;
      const itemT = $item.offsetTop + pageHeaderH;
      if (scrollH - itemT > pageBookH) {
        this.pageScroller.scrollTop = itemT;
      } else {
        this.pageScroller.scrollTop = scrollH - pageBookH;
      }
      this.itemBlink($item);
    } else {
      this.pageScroller.scrollTop = 0;
    }
  };

  itemBlink = ($item) => {
    addClass($item, listCardStyles.listCardActive);
    setTimeout(() => {
      removeClass($item, listCardStyles.listCardActive);
    }, 1500);
  };

  search = (showSelf) => {
    const { startTime, endTime, followMode, miniClassName, productType, studyManager, subjectId, teacher } = this.state;
    const { userId } = this.props.user;
    this.setState({ loading: true });
    loadList(this.studentId, {
      startTime,
      endTime,
      followMode,
      miniClassName,
      productType,
      studyManager,
      subjectId,
      teacher,
      followUpUserId: showSelf ? userId : '',
    }).then(
      (list) => {
        this.setState({
          loading: false,
          list,
          searchKeys: { startTime, endTime, followMode, miniClassName, productType, studyManager, subjectId, teacher },
        });
      },
      () => {
        this.setState({ loading: false });
      },
    );
  };

  reset = () => {
    this.setState({
      startTime: null,
      endTime: null,
      followMode: null,
      searchKeys: initSearchKeys,
      productType: null,
      studyManager: null,
      teacher: null,
      subjectId: null,
      miniClassName: null,
    });
  };
  showSelf = (e) => {
    this.search(e.target.checked);
  };
  getRecord = (id) => {
    const { list } = this.state;
    let rtn;
    list.forEach((sec) => {
      const record = sec.data.find((record) => record.id === id);
      if (record) {
        rtn = record;
        return false;
      }
    });
    return rtn;
  };

  delete = (id) => {
    this.setState({ loading: true });
    deleteRecord(id).then(
      () => {
        this.setState({ loading: false });
        showMessage('删除成功', 'success');
        let newList = [];
        this.state.list.forEach((sec) => {
          const data = sec.data.filter((da) => da.id !== id);
          if (data.length > 0) {
            newList.push({ ...sec, data });
          }
        });
        this.setState({ list: newList });
      },
      (e) => {
        this.setState({ loading: false });
        showMessage(e.text || '删除失败');
      },
    );
  };

  newAdd = () => {
    let {
      user: { userId },
    } = this.props;
    let { managerDict, teacherDict, homeroomTeacherDict } = this.state;
    let productDict = this.judgeAuth(userId, managerDict, teacherDict, homeroomTeacherDict);
    if (!productDict) {
      showMessage('只允许学生的学管师、班主任或授课老师/辅导老师新增跟进回访！');
      return;
    }
    this.setState({ detailModalVis: true, detailModalType: 'new' });
  };

  judgeAuth = (userId, managerDict, teacherDict, homeroomTeacherDict) => {
    let isOTOTeacher, isSmallTeacher, isManager, isMaster, isOTOHomeroom, isSmallHomeroom;
    let teaItem = teacherDict.find((v, k) => v.val === userId);
    if (teaItem) {
      if (teaItem.type.includes('ONE_ON_ONE_COURSE')) {
        isOTOTeacher = true;
      }
      if (teaItem.type.includes('SMALL_CLASS')) {
        isSmallTeacher = true;
      }
    }
    let managerItem = managerDict.find((v, k) => v.val === userId);
    if (managerItem) {
      if (managerItem.type.includes('ONE_ON_ONE_COURSE')) {
        isManager = true;
      }
      if (managerItem.type.includes('SMALL_CLASS')) {
        isMaster = true;
      }
    }
    let homeroomItem = homeroomTeacherDict.find((v, k) => v.val === userId);
    if (homeroomItem) {
      if (homeroomItem.type.includes('ONE_ON_ONE_COURSE')) {
        isOTOHomeroom = true;
      }
      if (homeroomItem.type.includes('SMALL_CLASS')) {
        isSmallHomeroom = true;
      }
    }
    console.log(isOTOTeacher, isSmallTeacher, isManager, isMaster, isOTOHomeroom, isSmallHomeroom);
    if (!isOTOTeacher && !isSmallTeacher && !isManager && !isMaster && !isOTOHomeroom && !isSmallHomeroom) {
      this.auth = [];
      return;
    }
    // if ((isSmallTeacher && !isOTOTeacher) || (isMaster && !isManager)) {
    //   this.auth = ['SMALL_CLASS'];
    //   this.modalProductDict = [{val: 'SMALL_CLASS', text: '十人课堂'}];
    // }
    if ((isSmallTeacher || isMaster || isSmallHomeroom) && !isOTOTeacher && !isManager && !isOTOHomeroom) {
      this.auth = ['SMALL_CLASS'];
      this.modalProductDict = [{ val: 'SMALL_CLASS', text: '十人课堂' }];
    }
    // if ((!isSmallTeacher && isOTOTeacher) || (!isMaster && isManager)) {
    //   this.auth = ['ONE_ON_ONE_COURSE'];
    //   this.modalProductDict = [{val: 'ONE_ON_ONE_COURSE', text: '1对1'}];
    // }
    if ((isOTOTeacher || isManager || isOTOHomeroom) && !isSmallTeacher && !isMaster && !isSmallHomeroom) {
      this.auth = ['ONE_ON_ONE_COURSE'];
      this.modalProductDict = [{ val: 'ONE_ON_ONE_COURSE', text: '1对1' }];
    }
    // if ((isSmallTeacher && isOTOTeacher) || (isMaster && isManager)) {
    //   this.auth = ['ONE_ON_ONE_COURSE', 'SMALL_CLASS'];
    //   this.modalProductDict = [{val: 'ONE_ON_ONE_COURSE', text: '1对1'}, {val: 'SMALL_CLASS', text: '十人课堂'}];
    // }
    if ((isOTOTeacher || isManager || isOTOHomeroom) && (isMaster || isSmallTeacher || isSmallHomeroom)) {
      this.auth = ['ONE_ON_ONE_COURSE', 'SMALL_CLASS'];
      this.modalProductDict = [
        { val: 'ONE_ON_ONE_COURSE', text: '1对1' },
        { val: 'SMALL_CLASS', text: '十人课堂' },
      ];
    }
    return true;
  };

  edit = (id) => {
    let {
      user: { userId },
    } = this.props;
    let { managerDict, teacherDict, homeroomTeacherDict } = this.state;
    this.judgeAuth(userId, managerDict, teacherDict, homeroomTeacherDict);
    this.setState({ detailModalVis: true, detailModalType: null, detailModalDataId: id });
  };
  startYearChange = (val) => {
    this.setState({
      startYear: val,
      endYearDict: this.state.endYearDict.map((opt) => {
        const newOpt = { val: opt.val, text: opt.text };
        if (parseInt(opt.val) >= parseInt(val)) {
          return newOpt;
        } else {
          return { ...newOpt, disabled: true };
        }
      }),
    });
  };

  endYearChange = (val) => {
    this.setState({
      endYear: val,
      startYearDict: this.state.startYearDict.map((opt) => {
        const newOpt = { val: opt.val, text: opt.text };
        if (parseInt(opt.val) <= parseInt(val)) {
          return newOpt;
        } else {
          return { ...newOpt, disabled: true };
        }
      }),
    });
  };

  getDisabledStartDate = (startTime) => {
    let { endTime } = this.state;
    if (!startTime || !endTime) {
      return false;
    }
    return moment(startTime).valueOf() > moment(endTime).valueOf();
  };
  getDisabledEndDate = (endTime) => {
    const { startTime } = this.state;
    if (!endTime || !startTime) {
      return false;
    }
    return moment(endTime).valueOf() < moment(startTime).valueOf();
  };

  render() {
    const { user } = this.props;
    const {
      loading,
      list,
      startYear,
      endYear,
      startTime,
      endTime,
      followMode,
      startYearDict,
      endYearDict,
      followModeDict,
      detailModalVis,
      detailModalType,
      detailModalDataId,
      managerDict = [],
      studyManager,
      teacherDict = [],
      teacher,
      subjectDict,
      subjectId,
      miniClassName,
      productType,
      miniClassDict,
    } = this.state;
    return (
      <Page
        loading={loading}
        route={this.props.route}
        ref={(c) => (this.pageRef = c)}
        className={styles.page}
        headerToolBar={
          <div className={styles.headerToolBar}>
            <FormItem name="学管师/班主任" labelWidth={100}>
              {managerDict[0] ? managerDict[0].text : null}
            </FormItem>
            <FormItem name="学生" labelWidth={43}>
              {this.studentName}
            </FormItem>
          </div>
        }
      >
        <div className={styles.wrap}>
          <div className={styles.rightBtnWrap}>
            <Button onClick={this.newAdd}>新增</Button>
          </div>
          <SearchBar onSearch={this.search} onReset={this.reset}>
            <DatePicker
              placeholder="开始日期"
              value={startTime}
              onChange={(val) => this.setState({ startTime: val })}
              disabledDate={(val) => this.getDisabledStartDate(val)}
            />
            <DatePicker
              placeholder="结束日期"
              value={endTime}
              onChange={(val) => this.setState({ endTime: val })}
              disabledDate={(val) => this.getDisabledEndDate(val)}
            />

            <Select
              placeholder="产品类型"
              dict={productDict}
              value={productType}
              onChange={(val) => this.setState({ productType: val })}
            />
            <Select
              placeholder="跟进方式"
              dict={followModeDict}
              value={followMode}
              onChange={(val) => this.setState({ followMode: val })}
            />
            <Select
              placeholder="学管师/班主任"
              className={styles.selectWrap}
              dict={managerDict}
              value={studyManager}
              onChange={(val) => this.setState({ studyManager: val })}
            />
            <Select
              placeholder="老师"
              dict={teacherDict}
              value={teacher}
              onChange={(val) => this.setState({ teacher: val })}
            />
            <Select
              placeholder="科目"
              dict={subjectDict}
              value={subjectId}
              onChange={(val) => this.setState({ subjectId: val })}
            />
            <Input
              placeholder="班课名称"
              value={miniClassName}
              onChange={(e) => this.setState({ miniClassName: e.target.value })}
            />
          </SearchBar>
          <div style={{ paddingLeft: 20 }}>
            <Checkbox onChange={this.showSelf}>仅看自己</Checkbox>
          </div>
          <div style={{ padding: '10px 35px 0' }}>
            {list.length > 0 &&
              list.map((li, idx) => {
                const { year, data } = li;
                return (
                  <ListSec
                    key={year}
                    year={year}
                    data={data}
                    followModeDict={followModeDict}
                    onEdit={this.edit}
                    onDelete={this.delete}
                    userId={user.userId}
                    auth={this.auth}
                  />
                );
              })}
            {list.length === 0 && <NoData style={{ marginTop: 120 }} />}
          </div>
        </div>
        {detailModalVis && (
          <DetailModal
            type={detailModalType}
            data={this.getRecord(detailModalDataId)}
            followModeDict={followModeDict}
            miniClassDict={miniClassDict}
            productDict={this.modalProductDict}
            subjectDict={subjectDict}
            onClose={() => this.setState({ detailModalVis: false })}
            onSaveSuccess={this.onSaveSuccess}
            userName={user.name}
            userId={user.userId}
            studentId={this.studentId}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    user: state.account.user,
    params: state.main.pageParams['/StudentFollowVisit'],
  };
};

export default connect(mapStateToProps)(FollowVisit);
