import {
  getDataFromReport,
  getStudentFollowUpsByStudentId,
  saveFollowUp,
  deleteFollowUpById,
  getSubjectById,
  getMiniClassById,
  getStudyManager,
  getTeacher,
  getHomeroomTeacherByStudentId,
} from 'SERVICE/student';
import dicts from './dicts';
import moment from 'moment';
import { DATE_FORMAT } from 'COMPONENT/utils/form';
import { getSelectOption } from 'SERVICE/common';
import { getIconSrc } from './utils';

const loadList = async (studentId, pa = {}, isFromReport) => {
  let params, apiData;
  if (isFromReport) {
    params = { ids: pa, studentId };
    apiData = await getDataFromReport(params);
  } else {
    params = {
      studentId,
      startTime: pa.startTime,
      endTime: pa.endTime,
      followUpType: pa.followMode,
      miniClassName: pa.miniClassName,
      followUpUserId: pa.followUpUserId,
      productType: pa.productType,
      studyManager: pa.studyManager,
      subjectId: pa.subjectId,
      teacher: pa.teacher,
      // followUpUserId:pa.followUpUserId
    };
    apiData = await getStudentFollowUpsByStudentId(params, true);
  }
  let list = [];
  apiData.forEach((da) => {
    const {
      followUpTime,
      followUpType,
      followUpUserId,
      followUpUserName,
      id,
      nextTime,
      remark,
      studentFollowFileVos,
      createUserId,
      createTime,
      miniClassList,
      subjectList,
      productTypes,
      role,
    } = da;
    const year = followUpTime.slice(0, 4);
    const fileList = studentFollowFileVos.map((file) => {
      const name = file.fileName;
      const src = file.filePath;
      let info = {
        id: file.fileid,
        name,
        src,
        previewPath: file.previewFilePath,
        iconSrc: getIconSrc(name, src),
      };
      return info;
    });
    const miniClassArray = miniClassList.map((v, k) => v.key);
    const subjectArray = subjectList.map((v, k) => v.key);
    const record = {
      id,
      date: followUpTime ? followUpTime.slice(0, 10) : null,
      mode: followUpType,
      follower: followUpUserName,
      followerId: followUpUserId,
      content: remark,
      nextDate: nextTime ? nextTime.replace(/(^\s*)|(\s*$)/g, '').slice(0, 10) : null,
      fileList,
      creatorId: createUserId,
      createTime,
      miniClassArray,
      subjectArray,
      productTypes,
      role,
    };
    const group = list.find((g) => g.year === year);
    if (group) {
      group.data.push(record);
      group.data = group.data.sort((a, b) => {
        return moment(a.date, DATE_FORMAT).isBefore(moment(b.date, DATE_FORMAT)) ? 1 : -1;
      });
    } else {
      list.push({
        year,
        data: [record],
      });
      list = list.sort((a, b) => {
        return parseInt(b.year) - parseInt(a.year);
      });
    }
  });

  return list;
};

const saveDetail = async (type, studentId, data) => {
  const studentFollowFileVos = data.fileList.map((file) => {
    return {
      fileid: file.id,
      previewFilePath: file.previewPath,
      fileName: file.name,
      filePath: file.src,
    };
  });
  let params = {
    studentFollowFileVos,
    followUpTime: data.date,
    followUpType: data.followMode,
    nextTime: data.nextDate,
    remark: data.content,
    studentId,
    miniClassIds: data.miniClassArray,
    subjectIds: data.subjectArray,
  };
  if (type !== 'new') {
    params.id = data.id;
  }
  try {
    await saveFollowUp(params, true);
    return true;
  } catch (e) {
    throw e;
  }
};

const deleteRecord = async (id) => {
  try {
    await deleteFollowUpById({ id }, true);
    return true;
  } catch (e) {
    throw e;
  }
};

const loadYearDict = async () => {
  try {
    const data = await getSelectOption({ selectOptionCategory: 'YEAR' }, true);
    let dict = [];
    Object.keys(data.value).forEach((key) => {
      const val = data.value[key];
      if (key && val) {
        dict.push({ val, text: val });
      }
    });
    return dict;
  } catch (e) {
    return [];
  }
};

const getSubject = async (studentId) => {
  try {
    const data = await getSubjectById({ studentId }, true);
    let dict = [];
    data.forEach((v, k) => {
      dict.push({ val: v.id, text: v.name });
    });
    return dict;
  } catch (e) {
    return [];
  }
};
const getMiniClass = async (studentId) => {
  try {
    const data = await getMiniClassById({ studentId }, true);
    let dict = [];
    data.forEach((v, k) => {
      dict.push({ val: v.id, text: v.name });
    });
    return dict;
  } catch (e) {
    return [];
  }
};

const loadStudyManagerDict = async (studentId) => {
  try {
    const data = await getStudyManager({ studentId }, true);
    let dict = [];
    data.forEach((v, k) => {
      let idx = dict.findIndex((V, K) => V.val === v.userId);
      if (idx === -1) {
        dict.push({ val: v.userId, text: v.userName, type: v.productType });
      } else {
        dict[idx].type += ',' + v.productType;
      }
    });
    return dict;
  } catch (e) {
    return [];
  }
};
const loadTeacherDict = async (studentId) => {
  try {
    const data = await getTeacher({ studentId }, true);
    let dict = [];
    data.forEach((v, k) => {
      let idx = dict.findIndex((V, K) => V.val === v.userId);
      if (idx === -1) {
        dict.push({ val: v.userId, text: v.userName, type: v.productType });
      } else {
        dict[idx].type += ',' + v.productType;
      }
    });
    return dict;
  } catch (e) {
    return [];
  }
};
const loadHomeroomTeacherDict = async (studentId) => {
  try {
    const data = await getHomeroomTeacherByStudentId({ studentId }, true);
    let dict = [];
    data.forEach((v, k) => {
      let idx = dict.findIndex((V, K) => V.val === v.userId);
      if (idx === -1) {
        dict.push({ val: v.userId, text: v.userName, type: v.productType });
      } else {
        dict[idx].type += ',' + v.productType;
      }
    });
    return dict;
  } catch (e) {
    return [];
  }
};

export default {
  loadList,
  saveDetail,
  deleteRecord,
  loadYearDict,
  getSubject,
  getMiniClass,
  loadStudyManagerDict,
  loadTeacherDict,
  loadHomeroomTeacherDict,
};
