import React from 'react';
import PropTypes from 'prop-types';
import { List, ListAction } from 'COMPONENT';
import { errorsHandle, runFunc } from 'UTIL/common';
import { getRelationOperationList, restoreCustomerStudent } from 'SERVICE/student';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

const setWidth = List.createSetWidth();
const errorsHandler = errorsHandle();

class ActionRecord extends React.Component {
  static propTypes = {
    studentId: PropTypes.string.isRequired,
    onRestore: PropTypes.func,
  };

  constructor(props) {
    super(props);

    this.init = this.init.bind(this);
    this.loadData = this.loadData.bind(this);
    this.changePage = this.changePage.bind(this);
    this.refresh = this.refresh.bind(this);
    this.restore = this.restore.bind(this);

    this.state = {
      loading: false,
      list: [],
      total: 0,
      pageCurrent: 1,
      pageSize: 10,
    };

    this.columns = [
      { title: '操作时间', dataIndex: 'createTime', key: 'createTime', width: setWidth(5) },
      { title: '操作人', dataIndex: 'operateUser', key: 'operateUser', width: setWidth(4) },
      { title: '操作类型', dataIndex: 'operateTypeName', key: 'operateTypeName', width: setWidth(6) },
      { title: '客户姓名', dataIndex: 'customerName', key: 'customerName', width: setWidth(4) },
      { title: '客户电话', dataIndex: 'customerContact', key: 'customerContact', width: setWidth(5) },
      { title: '关系', dataIndex: 'relation', key: 'relation', width: setWidth(6) },
      {
        title: '操作',
        dataIndex: 'viewButton',
        key: 'actionPlus',
        width: setWidth(2),
        render: (text, record) =>
          text && (
            <ListAction
              config={[
                {
                  text: '还原',
                  onClick: () => {
                    this.restore(record);
                  },
                  needConfirm: true,
                  confirmTips: '是否确认还原？',
                },
              ]}
            />
          ),
      },
    ];
  }

  componentDidMount() {
    this.init();
  }

  init() {
    if (this.props.studentId) {
      this.loadData(this.state.pageCurrent, this.state.pageSize);
    }
  }

  async loadData(pageNum, pageSize) {
    this.setState({ loading: true });
    let apiParams = {
      studentId: this.props.studentId,
      page: pageNum,
      rows: pageSize,
    };
    const res = await getRelationOperationList(apiParams).catch(errorsHandler);
    if (res && res.resultCode === 0) {
      this.setState({
        list: res.rows || [],
        total: res.records,
      });
    }
    this.setState({ loading: false });
  }

  changePage({ current, pageSize }) {
    this.setState({
      pageCurrent: current,
      pageSize: pageSize,
    });
    this.loadData(current, pageSize);
  }

  refresh() {
    this.changePage({
      current: 1,
      pageSize: this.state.pageSize,
    });
  }

  async restore(record) {
    this.setState({ loading: true });
    const apiParams = {
      operationRecordId: record.id,
    };
    const res = await restoreCustomerStudent(apiParams).catch(errorsHandler);
    if (res && res.resultCode === 0) {
      showMessage('还原成功！', 'success');
      this.loadData(this.state.pageCurrent, this.state.pageSize);
      runFunc(this.props.onRestore);
    } else {
      this.setState({ loading: false });
    }
  }

  render() {
    return (
      <List
        loading={this.state.loading}
        columns={this.columns}
        dataSource={this.state.list}
        pagination={{
          total: this.state.total,
          current: this.state.pageCurrent,
          pageSize: this.state.pageSize,
        }}
        onChange={this.changePage}
      />
    );
  }
}

export default ActionRecord;
