import React from 'react';
import PropTypes from 'prop-types';
import { List, ListAction } from 'COMPONENT';
import { errorsHandle, runFunc } from 'UTIL/common';
import { getRelationConfirmList } from 'SERVICE/student';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm.js';
import { cancelCustomerStudentRelation } from 'SERVICE/student';

const setWidth = List.createSetWidth();
const errorsHandler = errorsHandle();

class ConfirmRecord extends React.Component {
  static propTypes = {
    studentId: PropTypes.string.isRequired,
    onSend: PropTypes.func,
  };

  constructor(props) {
    super(props);

    this.init = this.init.bind(this);
    this.loadData = this.loadData.bind(this);
    this.changePage = this.changePage.bind(this);
    this.refresh = this.refresh.bind(this);
    this.sendMsg = this.sendMsg.bind(this);
    this.cancel = this.cancel.bind(this);

    this.state = {
      loading: false,
      list: [],
      total: 0,
      pageCurrent: 1,
      pageSize: 10,
    };

    this.columns = [
      { title: '操作时间', dataIndex: 'createTime', key: 'createTime', width: setWidth(5) },
      { title: '操作人', dataIndex: 'operateUser', key: 'operateUser', width: setWidth(4) },
      { title: '客户姓名', dataIndex: 'customerName', key: 'customerName', width: setWidth(4) },
      { title: '客户电话', dataIndex: 'customerContact', key: 'customerContact', width: setWidth(5) },
      {
        title: '家长是否确认',
        dataIndex: 'overType',
        overType: 'over',
        width: setWidth(6),
        render: (text) => {
          const typeMap = ['未确认', '已确认', '已取消'];
          return typeMap[text];
        },
      },
      {
        title: '操作',
        dataIndex: 'over',
        key: 'actionPlus',
        width: setWidth(4),
        render: (text, record) =>
          !record.overType && (
            <div style={{ display: 'flex' }}>
              <ListAction
                config={[
                  {
                    text: record.update ? '重新发送确认' : '发送确认信息',
                    onClick: () => {
                      this.sendMsg(record);
                    },
                  },
                ]}
              />
              {!record.over && (
                <div style={{ marginLeft: '15px' }}>
                  <ListAction
                    config={[
                      {
                        text: '取消',
                        needConfirm: true,
                        confirmTips: '确定取消吗？',
                        onClick: () => {
                          this.cancel(record);
                        },
                      },
                    ]}
                  />
                </div>
              )}
            </div>
          ),
      },
    ];
  }

  componentDidMount() {
    this.init();
  }

  init() {
    if (this.props.studentId) {
      this.loadData(this.state.pageCurrent, this.state.pageSize);
    }
  }

  async loadData(pageNum, pageSize) {
    this.setState({ loading: true });
    let apiParams = {
      studentId: this.props.studentId,
      page: pageNum,
      rows: pageSize,
    };
    const res = await getRelationConfirmList(apiParams).catch(errorsHandler);
    if (res && res.resultCode === 0) {
      this.setState({
        list: res.rows || [],
        total: res.records,
      });
    }
    this.setState({ loading: false });
  }

  changePage({ current, pageSize }) {
    this.setState({
      pageCurrent: current,
      pageSize: pageSize,
    });
    this.loadData(current, pageSize);
  }

  refresh() {
    this.changePage({
      current: 1,
      pageSize: this.state.pageSize,
    });
  }

  async sendMsg(record) {
    runFunc(this.props.onSend, {
      id: record.id,
      phoneNumber: record.customerContact,
      customerName: record.customerName,
    });
  }
  async cancel(record) {
    cancelCustomerStudentRelation({ confirmRecordId: record.id }).then(async () => {
      await this.init();
      runFunc(this.props.onCancel);
    });
  }

  render() {
    return (
      <List
        loading={this.state.loading}
        columns={this.columns}
        dataSource={this.state.list}
        pagination={{
          total: this.state.total,
          current: this.state.pageCurrent,
          pageSize: this.state.pageSize,
        }}
        onChange={this.changePage}
      />
    );
  }
}

export default ConfirmRecord;
