import React from 'react';
import PropTypes from 'prop-types';
import ConfirmByShareWay from 'BLOC/ConfirmByShareWay';
import { getSmsTemplate, sendContractConfirmSms } from 'SERVICE/contract';
import { getFrontEndConfig } from 'SERVICE/common';
import { sendRelationConfirmSms } from 'SERVICE/student';
import { errorsHandle } from 'UTIL/common';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

const { radioMessage, radioPoster } = ConfirmByShareWay;
const errorsHandler = errorsHandle();

class ModalConfirmWay extends React.Component {
  static propTypes = {
    id: PropTypes.oneOfType([PropTypes.number, PropTypes.string]).isRequired,
    phoneNumber: PropTypes.string.isRequired,
    customerName: PropTypes.string.isRequired,
    studentName: PropTypes.string.isRequired,
  };

  constructor(props) {
    super(props);

    this.init = this.init.bind(this);
    this.initData = this.initData.bind(this);
    this.sendMessage = this.sendMessage.bind(this);

    this.state = {
      loading: false,
      url: undefined,
      template: undefined,
      messageInfo: [{ text: `发送号码：${props.phoneNumber}` }, { text: `家长姓名：${props.customerName}` }],
      posterInfo: [
        { text: `关联家长：${props.customerName}` },
        { text: `关联电话：${props.phoneNumber}` },
        { text: `关联学生：${props.studentName}` },
      ],
      radios: [radioMessage, radioPoster],
    };
  }

  componentDidMount() {
    this.init();
  }

  init() {
    this.initData();
  }

  async initData() {
    this.setState({ loading: true });
    try {
      let fetchList = [
        getFrontEndConfig({
          key: 'customer_student_relation_confirm_h5_url',
          module: 'crm',
        }),
        getFrontEndConfig({
          key: 'customer_student_relation_confirm_sms',
          module: 'crm',
        }),
      ];

      const [resUrl, resTemplate] = await Promise.all(fetchList);

      this.setState({
        url: resUrl.data.replace('#{confirmRecordId}', this.props.id),
        template: resTemplate.data,
      });
    } catch (e) {
      errorsHandler(e);
    }
    this.setState({ loading: false });
  }

  async sendMessage() {
    const apiParams = {
      confirmRecordId: this.props.id,
    };
    const res = await sendRelationConfirmSms(apiParams).catch(errorsHandler);
    if (res && res.resultCode === 0) {
      showMessage('发送成功', 'success');
    }
  }

  render() {
    return (
      <ConfirmByShareWay
        title="关联家长确认"
        loading={this.state.loading}
        onCancel={this.props.onCancel}
        radios={this.state.radios}
        message={{
          infos: this.state.messageInfo,
          template: this.state.template,
          onSend: this.sendMessage,
        }}
        poster={{
          infos: this.state.posterInfo,
          url: this.state.url,
          tips: '请扫码确认学生关联信息',
        }}
      />
    );
  }
}

export default ModalConfirmWay;
