import React from 'react';
import ReactDOM from 'react-dom';
import styles from './styles.module.less';
import { Select } from 'antd';

const SelectOption = Select.Option;

class _Select extends React.Component {
  constructor(props) {
    super(props);

    this.switchPlaceholder = this.switchPlaceholder.bind(this);
    this.focusSelect = this.focusSelect.bind(this);
    this.refresh = this.refresh.bind(this);

    this.state = {
      imitationVisible: true,
    };

    this.selectRef = null;
    this.selectNode = null;
  }

  switchPlaceholder() {
    this.setState((prevState, prevProps) => ({
      imitationVisible: prevProps.value ? false : !prevState.imitationVisible,
    }));
  }

  focusSelect() {
    if (!this.selectNode) {
      this.selectNode = ReactDOM.findDOMNode(this.selectRef);
    }
    this.selectNode.click();
  }

  refresh() {
    this.setState({ imitationVisible: true });
  }

  render() {
    const { onSearch, dict, placeholder, ...restProps } = this.props;

    return (
      <div className={styles.selectWrap}>
        <Select
          {...restProps}
          ref={(c) => (this.selectRef = c)}
          showSearch={true}
          notFoundContent=""
          showArrow={false}
          defaultActiveFirstOption={false}
          filterOption={false}
          onSearch={onSearch}
          onFocus={this.switchPlaceholder}
          onBlur={this.switchPlaceholder}
        >
          {dict.map((item) => (
            <SelectOption value={item.value} key={item.value}>
              {item.label}
            </SelectOption>
          ))}
        </Select>
        {placeholder && this.state.imitationVisible && (
          <div className={styles.imitation} onClick={this.focusSelect}>
            {placeholder}
          </div>
        )}
      </div>
    );
  }
}

export default _Select;
