import React from 'react';
import styles from './styles.module.less';
import { Page } from 'COMPONENT';
import { Tabs } from 'antd';
import Block from './components/Block';
import AddRelationForm from './AddRelationForm';
import ParentList from './ParentList';
import ActionRecord from './ActionRecord';
import ConfirmRecord from './ConfirmRecord';
import ModalConfirmWay from './ModalConfirmWay';
import { runFunc } from 'UTIL/common';
import { sendRelationConfirmSms } from 'SERVICE/student';

const TabPane = Tabs.TabPane;

class Index extends React.Component {
  constructor(props) {
    super(props);

    this.setPageLoading = this.setPageLoading.bind(this);
    this.handleSend = this.handleSend.bind(this);
    this.handleCancel = this.handleCancel.bind(this);
    this.handleAdd = this.handleAdd.bind(this);
    this.handleDelete = this.handleDelete.bind(this);
    this.handleModify = this.handleModify.bind(this);
    this.handleRestore = this.handleRestore.bind(this);
    this.closeModal = this.closeModal.bind(this);

    const routeParams = JSON.parse(props.routeParams.params || {});

    this.state = {
      studentId: routeParams.studentId,
      studentName: routeParams.studentName,
      isAllowRender: routeParams.studentId && routeParams.studentName,
      loading: false,
      msmTemplate: null,
      modalVisible: false,
    };

    this.confirmData = {};

    this.parentListRef = null;
    this.confirmRecordRef = null;
    this.actionRecordRef = null;
  }

  setPageLoading(val) {
    this.setState({ loading: val });
  }

  handleSend(data) {
    this.confirmData = data;
    this.setState({ modalVisible: true });
  }

  handleAdd(data) {
    if (this.confirmRecordRef) {
      runFunc(this.confirmRecordRef.refresh);
    }
    this.handleSend(data);
  }

  handleDelete() {
    if (this.actionRecordRef) {
      runFunc(this.actionRecordRef.refresh);
    }
  }

  handleModify() {
    if (this.actionRecordRef) {
      runFunc(this.actionRecordRef.refresh);
    }
  }

  handleRestore() {
    if (this.parentListRef) {
      runFunc(this.parentListRef.refresh);
    }
  }
  handleCancel() {
    runFunc(this.actionRecordRef.refresh);
  }
  closeModal() {
    this.setState({ modalVisible: false });
    this.confirmData = {};
  }

  render() {
    return (
      <Page
        className={styles.page}
        loading={this.state.loading}
        title={
          <div className={styles.header}>
            <div className={styles.headerItem}>
              <span className={styles.headerLabel}>学生姓名</span>
              <span className={styles.headerContent}>{this.state.studentName}</span>
            </div>
            <div className={styles.headerItem}>
              <span className={styles.headerLabel}>学生ID</span>
              <span className={styles.headerContent}>{this.state.studentId}</span>
            </div>
          </div>
        }
      >
        {this.state.isAllowRender && (
          <div>
            <Block title="添加关联家长">
              <AddRelationForm
                studentId={this.state.studentId}
                setLoading={this.setPageLoading}
                onAdd={this.handleAdd}
              />
            </Block>

            <Block title="关联家长">
              <ParentList
                ref={(c) => (this.parentListRef = c)}
                studentId={this.state.studentId}
                onDelete={this.handleDelete}
                onModify={this.handleModify}
              />
            </Block>

            <Block>
              <Tabs defaultActiveKey="1">
                <TabPane tab="家长确认记录" key="1">
                  <ConfirmRecord
                    ref={(c) => (this.confirmRecordRef = c)}
                    studentId={this.state.studentId}
                    onSend={this.handleSend}
                    onCancel={this.handleCancel}
                  />
                </TabPane>

                <TabPane tab="操作历史" key="2">
                  <ActionRecord
                    ref={(c) => (this.actionRecordRef = c)}
                    studentId={this.state.studentId}
                    onRestore={this.handleRestore}
                  />
                </TabPane>
              </Tabs>
            </Block>
          </div>
        )}

        {this.state.modalVisible && (
          <ModalConfirmWay
            id={this.confirmData.id}
            phoneNumber={this.confirmData.phoneNumber}
            customerName={this.confirmData.customerName}
            studentName={this.state.studentName}
            onCancel={this.closeModal}
          />
        )}
      </Page>
    );
  }
}

export default Index;
