import React from 'react';
import { connect } from 'dva';
import { message } from 'antd';
import PropTypes from 'prop-types';
import { Modal, Form, Input, Select } from 'COMPONENT';
import { Row, Col } from 'antd';
import { getSchoolByStudentId, updateSchoolList } from 'SERVICE/student';
import { schoolType, schoolStatus } from '../dict';
import ModalSelectSchool from 'BLOC/ModalSelectSchool';
let uuid = 0;

const FormRow = (props) => {
  const { form, uuid, onDelete, openSchoolModal, changeSchoolType } = props;

  return (
    <Row type="flex" justify="start">
      <Col span={2}>学校</Col>
      <Col span={5} offset={1}>
        <Form.Item name={`schoolType-${uuid}`} form={form} rules={[{ required: true, message: '请选择学校类型' }]}>
          <Select
            dict={schoolType}
            placeholder="请选择学校类型"
            onChange={(v) => changeSchoolType(v, uuid)}
            optionFilterProp="children"
          />
        </Form.Item>
      </Col>
      <Col span={5} offset={1}>
        <Form.Item name={`attendState-${uuid}`} form={form} rules={[{ required: true, message: '请选择就读状态' }]}>
          <Select dict={schoolStatus} placeholder="请选择就读状态" optionFilterProp="children" />
        </Form.Item>
      </Col>
      <Col span={5} offset={1}>
        <Form.Item name={`schoolName-${uuid}`} form={form} rules={[{ required: true, message: '请选择学校' }]}>
          <Input readOnly onClick={() => openSchoolModal(uuid)} />
        </Form.Item>
      </Col>
      <Col span={2} offset={1}>
        <a href="javascript:void(0)" onClick={() => onDelete(uuid)}>
          删除学校
        </a>
      </Col>
      <Col style={{ display: 'none' }} span={5} offset={1}>
        <Form.Item name={`id-${uuid}`} form={form} initialValue="">
          <Input />
        </Form.Item>
        <Form.Item name={`schoolId-${uuid}`} form={form} initialValue="">
          <Input />
        </Form.Item>
      </Col>
    </Row>
  );
};

class ModalModify extends React.Component {
  static PropTypes = {
    id: PropTypes.string.isRequired,
    onCancel: PropTypes.func.isRequired,
    onSubmit: PropTypes.func,
    grade: PropTypes.object,
    editRow: PropTypes.object,
  };

  constructor(props) {
    super(props);
    this.state = {
      keys: [],
      chooseSchoolVisible: false,
      openIndex: '',
      citys: [],
      campus: [],
    };
    uuid = 0;

    this.formRef = null;
    this.schoolRef = null;

    this.saveSchool = this.saveSchool.bind(this);
    this.closeSchoolModal = this.closeSchoolModal.bind(this);
    this.openSchoolModal = this.openSchoolModal.bind(this);
    this.delete = this.delete.bind(this);
    this.addNew = this.addNew.bind(this);
    this.submit = this.submit.bind(this);
    this.changeSchoolType = this.changeSchoolType.bind(this);
  }

  async componentDidMount() {
    const { dispatch, provinceCityDict } = this.props;
    const { resultCode, data } = await getSchoolByStudentId({ studentId: this.props.id });
    if (resultCode === 0) {
      this.setState(
        (state) => {
          data.forEach(() => {
            state.keys.push(uuid++);
          });
          return state;
        },
        () => {
          data.forEach((item, index) => {
            this.formRef.setFieldsValue({
              [`schoolType-${index}`]: item['schoolType'],
              [`schoolName-${index}`]: item['schoolName'],
              [`attendState-${index}`]: item['attendState'] + '',
              [`id-${index}`]: item['id'],
              [`schoolId-${index}`]: item['schoolId'],
            });
          });
        },
      );
    }
    // 初始化所有省市数据，已有不再请求
    if (!provinceCityDict) {
      dispatch({
        type: 'dict/initDict',
        payload: {
          list: [{ name: 'provinceCityDict', params: 'dict' }],
        },
      });
    }
  }

  submit() {
    const { keys } = this.state;
    this.formRef.validateFields({ force: true }, async (err, form) => {
      if (!err) {
        let matchGrade = false;
        const schoolList = [];
        const {
          id,
          grade, // 需要改成的学校信息
        } = this.props;
        keys.forEach((key) => {
          if (form[`schoolType-${key}`] === grade.parentId && form[`attendState-${key}`] == '0') matchGrade = true; // 存在在读且类型匹配
          schoolList.push({
            id: form[`id-${key}`] || '',
            studentId: id,
            schoolId: form[`schoolId-${key}`],
            attendState: +form[`attendState-${key}`],
          });
        });
        if (!matchGrade) {
          message.warning(`请录入一条就读状态为【在读】的${grade.parentName}学校信息。`);
          return;
        }
        const { resultCode } = await updateSchoolList({
          studentId: id,
          schoolList,
        });
        if (resultCode == 0) {
          message.success('修改成功');
          this.props.onCancel();
        }
      }
    });
  }

  addNew() {
    this.setState((state) => {
      state.keys.push(++uuid);
      return state;
    });
  }

  delete(key) {
    this.setState((state) => {
      const index = state.keys.findIndex((item) => item === key);
      if (index > -1) {
        this.state.keys.splice(index, 1);
      }
      return state;
    });
  }

  openSchoolModal(uuid) {
    this.setState({
      chooseSchoolVisible: true,
      openIndex: uuid,
      schoolType: this.formRef.getFieldValue(`schoolType-${uuid}`),
    });
  }

  // 二级选择学校窗口保存
  saveSchool(...schools) {
    const { openIndex } = this.state;
    this.formRef.setFieldsValue({
      [`schoolName-${openIndex}`]: schools[0].name,
      [`schoolId-${openIndex}`]: schools[0].id,
    });
    this.setState({
      chooseSchoolVisible: false,
      openIndex: '',
    });
  }

  // 二级选择学校窗口关闭
  closeSchoolModal() {
    this.setState({
      chooseSchoolVisible: false,
    });
  }

  onFieldsChange = (fields) => {
    const key = Object.keys(fields)[0];
    if (key && key.indexOf('attendState') > -1 && fields[key]['value'] === '0') {
      let payload = {};
      this.state.keys.forEach((e) => {
        payload[`attendState-${e}`] = '1';
      });
      delete payload[key];
      this.formRef.setFieldsValue(payload);
    }
  };
  changeSchoolType(v, i) {
    this.formRef.setFieldsValue({
      [`schoolName-${i}`]: '',
    });
    this.formRef.validateFields([`schoolName-${i}`]);
  }
  render() {
    const { props, state } = this;
    const dict = (props.provinceCityDict && props.provinceCityDict.dict) || [];
    const provinceData = [];
    if (dict.length) {
      dict.forEach((d) => {
        provinceData.push({
          text: d.label,
          val: d.val,
        });
      });
    }
    return (
      <Modal title="就读学校" onCancel={props.onCancel} onOk={this.submit}>
        <Form ref={(c) => (this.formRef = c)} onSubmit={this.submit} onFieldsChange={this.onFieldsChange}>
          {state.keys.map((key) => (
            <FormRow
              onDelete={this.delete}
              uuid={key}
              key={key}
              openSchoolModal={this.openSchoolModal}
              changeSchoolType={this.changeSchoolType}
            />
          ))}
        </Form>
        <Row type="flex" justify="start">
          <Col span={10} style={{ color: 'red' }}>
            在读学校信息已失效，请及时更新。
          </Col>
          <Col span={14}>
            <a href="javascript:void(0)" onClick={this.addNew}>
              添加学校
            </a>
          </Col>
        </Row>
        {state.chooseSchoolVisible && (
          <ModalSelectSchool
            schoolId={this.formRef.getFieldValue(`schoolId-${state.openIndex}`)}
            blCampusId={props.editRow.campusId}
            schoolType={state.schoolType}
            useOtherRemark={false}
            onCancel={this.closeSchoolModal}
            onOk={this.saveSchool}
          />
        )}
      </Modal>
    );
  }
}

export default connect((state) => ({ provinceCityDict: state.dict.provinceCityDict }))(ModalModify);
