import React from 'react';
import { Page, List } from 'COMPONENT';
import { Form } from 'antd';
import ModalModify from './ModalModify';
import { connect } from 'dva';
import { getSchoolByStudentId, getGradeMatchingList } from 'SERVICE/student';
import { schoolType, schoolTypeTree } from './dict';
class ModifyStudentSchool extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      pageNum: 1,
      pageSize: 10,
      total: 0,
      visible: false,
      dataList: [],
      studentId: '', // 编辑的学生id
      grade: '', // 学校类型
      editStudentArea: {},
      editRow: {}, // 修改某条数据
    };
    this.columns = [
      { title: '学生姓名', dataIndex: 'studentName' },
      { title: '学习中心', dataIndex: 'campusName' },
      { title: '年级', dataIndex: 'gradeName' },
      { title: '就读学校', dataIndex: 'schoolName' },
      {
        title: '学校级别',
        dataIndex: 'schoolType',
        render: (type) => {
          return schoolType.find((sc) => sc.val == type) ? schoolType.find((sc) => sc.val == type)['text'] : '';
        },
      },
      {
        title: '操作',
        dataIndex: 'studentId',
        render: (studentId, row) => {
          return (
            <a href="javascript:void(0)" onClick={() => this.edit(studentId, row)}>
              修改就读学校
            </a>
          );
        },
      },
    ];

    this._getGradeMatchingList = this._getGradeMatchingList.bind(this);
    this.handleCancel = this.handleCancel.bind(this);
  }

  componentDidMount() {
    const { dispatch, provinceCityDict } = this.props;
    // 初始化所有省市数据，已有不再请求
    if (!provinceCityDict) {
      dispatch({
        type: 'dict/initDict',
        payload: {
          list: [{ name: 'provinceCityDict', params: 'dict' }],
        },
      });
    }
    this._getGradeMatchingList();
  }

  // 关闭编辑窗口
  handleCancel() {
    this.setState({
      visible: false,
      studentId: '',
    });
    this._getGradeMatchingList();
  }

  // 获取不匹配列表
  async _getGradeMatchingList({ pageNum, pageSize } = { pageNum: 1, pageSize: 10 }) {
    const { resultCode, rows, records } = await getGradeMatchingList({ page: pageNum, rows: pageSize });
    if (resultCode === 0) {
      this.setState({
        dataList: rows,
        total: records,
      });
    }
  }

  /**
   * 修改就读学校
   * @param {string} studentId
   * @param {object} row
   */
  async edit(studentId, row) {
    // 获取学生学校信息
    let grade = '';
    if (row.grade) {
      grade = schoolTypeTree.find((ele) => ele.value === row.grade);
    }
    this.setState({
      visible: true,
      studentId,
      grade,
      editRow: row,
      editStudentArea: {
        // 用于初始化地区选择器
        province: row.province,
        city: row.city,
      },
    });
  }
  /**
   * 切换列表分页
   */
  loadList({ pageNum, pageSize } = { pageNum: 1, pageSize: 10 }) {
    this.setState({
      pageNum,
      pageSize,
    });
    this._getGradeMatchingList({ pageNum, pageSize });
  }

  render() {
    const { pageNum, pageSize, total, dataList, visible, studentId, editStudentArea, grade, editRow } = this.state;
    const { provinceCityDict } = this.props;
    const dict = (provinceCityDict && provinceCityDict.dict) || [];
    const provinceData = [];
    if (dict.length) {
      dict.forEach((d) => {
        provinceData.push({
          text: d.label,
          val: d.val,
        });
      });
    }
    return (
      <Page name="学校与年级不匹配名单" route={this.props.route}>
        <List
          dataSource={dataList}
          columns={this.columns}
          pagination={{
            current: pageNum,
            pageSize,
            total,
          }}
          onChange={({ current, pageSize }) => this.loadList({ pageNum: current, pageSize })}
        ></List>
        {visible && (
          <ModalModify
            id={studentId}
            onCancel={this.handleCancel}
            {...editStudentArea}
            grade={grade}
            editRow={editRow}
          />
        )}
      </Page>
    );
  }
}
export default connect((state) => ({ provinceCityDict: state.dict.provinceCityDict }))(
  Form.create()(ModifyStudentSchool),
);
