/**
 * Created by luoming on 2018/4/19
 */
import React from 'react';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import Modal from 'COMPONENT/Modal';
import InputNumber from 'COMPONENT/InputNumber';
import ShowMessage from 'COMPONENT/utils/globalTips/showMessage';
import styles from './rule.less';

import { saveRules, getRules } from 'SERVICE/scoreManage';

export default class RuleSetting extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      ruleArray: [],
    };
  }

  componentDidMount() {
    this.loadData();
  }

  loadData = async () => {
    this.setState({ loading: true });
    let {
      data: {
        branchGraduatingRate,
        branchNonGraduatingRate,
        branchValidSubjectCount,
        campusGraduatingRate,
        campusNonGraduatingRate,
        campusValidSubjectCount,
        studyManagerGraduatingRate,
        studyManagerNonGraduatingRate,
        studyManagerValidSubjectCount,
        subjectGroupGraduatingRate,
        subjectGroupNonGraduatingRate,
        subjectGroupValidSubjectCount,
        teacherGraduatingRate,
        teacherNonGraduatingRate,
        teacherValidSubjectCount,
      },
    } = await getRules();
    let ruleArray = [
      {
        ruleName: '分公司统计率规则',
        notFinishClass: branchNonGraduatingRate,
        finishClass: branchGraduatingRate,
        valid: branchValidSubjectCount,
      },
      {
        ruleName: '学习中心统计率规则',
        notFinishClass: campusNonGraduatingRate,
        finishClass: campusGraduatingRate,
        valid: campusValidSubjectCount,
      },
      {
        ruleName: '科组统计率规则',
        notFinishClass: subjectGroupNonGraduatingRate,
        finishClass: subjectGroupGraduatingRate,
        valid: subjectGroupValidSubjectCount,
      },
      {
        ruleName: '老师统计率规则',
        notFinishClass: teacherNonGraduatingRate,
        finishClass: teacherGraduatingRate,
        valid: teacherValidSubjectCount,
      },
      {
        ruleName: '学管师统计率规则',
        notFinishClass: studyManagerNonGraduatingRate,
        finishClass: studyManagerGraduatingRate,
        valid: studyManagerValidSubjectCount,
      },
    ];
    this.setState({ ruleArray, loading: false });
  };
  changeData = (value, key, type) => {
    let { ruleArray } = this.state;
    ruleArray[key][type] = value;
    this.setState({ ruleArray });
  };
  onOk = async () => {
    let { ruleArray } = this.state;
    let { onOk, onClose } = this.props;
    let error = (text) => getWrappedInstance(this.$modal).showOkTips(text);
    let checkRule = ruleArray.filter((v, k) => {
      return v.notFinishClass === undefined || v.finishClass === undefined || v.valid === undefined;
    });
    if (checkRule.length > 0) {
      error('请完善统计规则！');
      return;
    }
    this.setState({ loading: true });
    let apiParams = {
      branchGraduatingRate: ruleArray[0].finishClass,
      branchNonGraduatingRate: ruleArray[0].notFinishClass,
      branchValidSubjectCount: ruleArray[0].valid,
      campusGraduatingRate: ruleArray[1].finishClass,
      campusNonGraduatingRate: ruleArray[1].notFinishClass,
      campusValidSubjectCount: ruleArray[1].valid,
      subjectGroupGraduatingRate: ruleArray[2].finishClass,
      subjectGroupNonGraduatingRate: ruleArray[2].notFinishClass,
      subjectGroupValidSubjectCount: ruleArray[2].valid,
      teacherGraduatingRate: ruleArray[3].finishClass,
      teacherNonGraduatingRate: ruleArray[3].notFinishClass,
      teacherValidSubjectCount: ruleArray[3].valid,
      studyManagerGraduatingRate: ruleArray[4].finishClass,
      studyManagerNonGraduatingRate: ruleArray[4].notFinishClass,
      studyManagerValidSubjectCount: ruleArray[4].valid,
    };
    await saveRules(apiParams);
    ShowMessage('保存成功', 'success');
    onClose();
    onOk();
  };

  render() {
    let { loading, ruleArray } = this.state;
    let { onClose } = this.props;

    return (
      <Modal
        loading={loading}
        onCancel={onClose}
        onOk={this.onOk}
        title="统计率规则设置"
        width={960}
        noLine={true}
        closable={false}
        className={styles.modal}
        ref={(e) => (this.$modal = e)}
      >
        <div className={styles.ruleSetting}>
          <div className={styles.header}>
            <div>统计率规则</div>
            <div>非毕业班科次统计率</div>
            <div>毕业班科次统计率</div>
            <div>有效统计科次数量</div>
          </div>
          <div className={styles.content}>
            {ruleArray.map((v, k) => {
              return (
                <div className={styles.item} key={k}>
                  <div>{v.ruleName}</div>
                  <div>
                    ≥
                    <InputNumber
                      value={v.notFinishClass}
                      min={0}
                      max={100}
                      onChange={(val) => this.changeData(val, k, 'notFinishClass')}
                    />
                  </div>
                  <div>
                    ≥
                    <InputNumber
                      value={v.finishClass}
                      min={0}
                      max={100}
                      onChange={(val) => this.changeData(val, k, 'finishClass')}
                    />
                  </div>
                  <div>
                    ≥
                    <InputNumber
                      value={v.valid}
                      min={0}
                      max={100}
                      onChange={(val) => this.changeData(val, k, 'valid')}
                    />
                  </div>
                </div>
              );
            })}
          </div>
        </div>
      </Modal>
    );
  }
}
