/**
 * Created by luoming on 2018/4/19
 */
import React from 'react';
import Modal from 'COMPONENT/Modal';
import List from 'COMPONENT/List';
import { getDictTextByVal } from 'UTIL/dict';
import styles from './index.less';
import { getStudentInfos } from 'SERVICE/scoreManage';

const [COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL] = [110, 150, 180];

export default class StudentList extends React.Component {
  constructor(props) {
    super(props);
    let {
      prefixTitle,
      dict: { schoolYearDict, semesterDict, examinationTypeDict },
      rules,
    } = this.props;
    const title =
      prefixTitle +
      getDictTextByVal(schoolYearDict, rules.schoolYear) +
      '' +
      getDictTextByVal(semesterDict, rules.semester) +
      getDictTextByVal(examinationTypeDict, rules.examinationType) +
      '统计情况';
    this.state = {
      title,
    };
  }

  async componentDidMount() {
    this.loadList();
  }

  loadList = async (searchKey = {}) => {
    let { currentPage = 1, pageSize = 10 } = searchKey;
    this.setState({ loading: true });
    let { studentInfoKey, rules } = this.props;
    let apiParams = {
      ...studentInfoKey,
      ...rules,
      pageSize,
      pageNo: currentPage,
    };
    let { data, rowCount } = await getStudentInfos(apiParams);
    this.setState({ loading: false, currentPage, pageSize, totalRecords: rowCount, list: data });
  };
  onOk = () => {};

  render() {
    let { loading, currentPage, totalRecords, pageSize, list = [], title } = this.state;
    let { onClose } = this.props;
    let columns = [
      { title: '学生', dataIndex: 'studentName', width: COL_WIDTH_M },
      { title: '年级', dataIndex: 'gradeName', width: COL_WIDTH_M },
      { title: '科目', dataIndex: 'subjectName', width: COL_WIDTH_M },

      {
        title: '对比成绩统计状态',
        dataIndex: 'countFlag',
        width: COL_WIDTH_XL,
        render: (text, record, idx) => {
          return <span>{text === 'N' ? '未统计' : '已统计'}</span>;
        },
      },
      { title: '进步判定备注', dataIndex: 'standardCategory', width: COL_WIDTH_L },

      { title: '学管师', dataIndex: 'studyManagerName', width: COL_WIDTH_M },
      { title: '老师', dataIndex: 'teacherName', width: COL_WIDTH_M },
    ];
    return (
      <Modal
        loading={loading}
        onCancel={onClose}
        title={title}
        width={960}
        noLine={true}
        okBtnShow={false}
        cancelBtnShow={false}
        closable={true}
        className={styles.modal}
        ref={(e) => (this.$modal = e)}
      >
        <div className={styles.studentList}>
          <List
            dataSource={list}
            columns={columns}
            loading={false}
            pagination={{
              current: currentPage,
              total: totalRecords,
              pageSize,
            }}
            onChange={({ current, pageSize }) => this.loadList({ currentPage: current, pageSize })}
          />
        </div>
      </Modal>
    );
  }
}
