/**
 * Created by luoming on 2018/4/18
 */
import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import SearchBar from 'COMPONENT/SearchBar';
import List from 'COMPONENT/List';
import Select from 'COMPONENT/Select';
import Button from 'COMPONENT/Button';
import IconButton from 'COMPONENT/IconButton';
import Loading from 'COMPONENT/Loading';
import NoData from 'COMPONENT/NoData';
import download from 'COMPONENT/utils/file/downloadFile';
import ShowMessage from 'COMPONENT/utils/globalTips/showMessage';
import HandleBar from './HandleBar';
import ChartBlock from './ChartBlock';
import ScoreSetting from './Modal/scoreSetting';
import RuleSetting from './Modal/ruleSetting';
import StudentList from './Modal/studentList';
import { option } from './option';
import echarts from 'ASSET/js/echarts.min.js';
import { loadCommonDict, loadBranchDict, loadCampusDict, getDictTextByVal } from 'UTIL/dict';
import {
  getSubjectStatisticsInfo,
  getOrgRankList,
  getOrgRankInfoForLevel,
  getStudentInfos,
  getTop50RankInfos,
  getRules,
  getYearAndSemester,
  getGroupList,
  getInfoByManager, //schoolYear,semester
  getInfosByTeacher,
  exportTop50,
  exportManager,
  exportTeacher,
  exportOrg,
} from 'SERVICE/scoreManage';

// const examinationTypeDict=[{val:'DAT000000000461',text:'期中考'},{val:'DAT000000000469',text:'期末考'}]
const semesterDict = [
  { val: 'LAST_SEMESTER', text: '上学期' },
  { val: 'NEXT_SEMESTER', text: '下学期' },
];
const { COL_WIDTH_S, COL_WIDTH_M } = List;
import styles from './index.less';

const dict1 = [
  { val: 'BRANCH', text: '分公司进步率' },
  { val: 'CAMPUS', text: '学习中心进步率' },
  { val: 'SUBJECT_GROUP', text: '科组进步率' },
  {
    val: 'TEACHER',
    text: '老师进步率',
  },
  { val: 'STUDY_MANAGER', text: '学管师进步率' },
];

const dict2 = [
  { val: 'branch', text: '分公司统计率' },
  { val: 'campus', text: '学习中心统计率' },
  { val: 'subject_group', text: '科组统计率' },
  {
    val: 'teacher',
    text: '老师统计率',
  },
  { val: 'study_manager', text: '学管师统计率' },
];
//东莞分公司
const initBranchId = '000002';

class ProgressRate extends React.Component {
  state = {
    list: [],
    active: 'BRANCH',
    route: this.props.route.path,
    expandedRowKeys: [],
  };
  rules = {};
  status = null;
  echartsBigs = null;
  exportTop50Params = {};
  limitClickTag = 1;
  targetType = 'BRANCH';
  async componentDidMount() {
    this.setState({ pageLoading: true });
    let { route } = this.state;
    let { branchId, campusId, grounpId, subjectGroupId } = this.props;
    branchId = branchId === '000001' ? initBranchId : branchId;
    campusId = campusId === '000001' ? '' : campusId;
    // grounpId=grounpId==='000001'?'':grounpId;
    subjectGroupId = '';
    if (route === 'StudentProgressRate') {
      let echartsBigs = echarts.init(this.echartsBig);
      this.echartsBigs = echartsBigs;
      window.addEventListener('resize', this.resize, false);
    }
    let [branchDict, campusDict, schoolYearDict, examTypeDict, { data }] = await Promise.all([
      loadBranchDict(),
      loadCampusDict(branchId),
      loadCommonDict('SCHOOL_YEAR'),
      loadCommonDict('EXAMINATION_TYPE'),
      getYearAndSemester(),
    ]);
    examTypeDict = examTypeDict.filter((v, k) => v.text === '期中考' || v.text === '期末考');
    //默认期末考
    let examinationType = examTypeDict.filter((v, k) => v.text === '期末考')[0].val;
    this.rules = { ...data, examinationType };
    let apiParams = { branchId, campusId, ...this.rules };

    // let {data:groupList}= (campusId !=='000001' && campusId && grounpId)? await getGroupList(apiParams):{data:[]};
    let { data: groupList } = campusId && campusId !== '000001' ? await getGroupList(apiParams) : { data: [] };

    let groupDict = groupList.map((v, k) => {
      return { val: v.ID, text: v.NAME };
    });
    // this.loadData(branchId,campusId,grounpId);
    this.loadData(branchId, campusId, subjectGroupId);
    this.setState({
      pageLoading: false,
      branchDict,
      campusDict,
      campusDictF: campusDict,
      schoolYearDict,
      data: this.rules,
      branchId,
      branchIdF: branchId,
      campusId,
      campusIdF: campusId,
      sectionId: subjectGroupId,
      sectionIdF: subjectGroupId,
      sectionDictF: groupDict,
      sectionDict: groupDict,
      examinationTypeDict: examTypeDict,
    });
  }

  resize = () => {
    this.echartsBigs.resize();
  };
  loadData = async (branchId, campusId, subjectGroupId) => {
    let { route } = this.state;
    this.getSubjectInfo(branchId, campusId, subjectGroupId);
    route === 'StudentProgressRate' && this.getTop50RankInfos();
    this.getOrgRankList({}, branchId, campusId, subjectGroupId);
  };
  saveSettingSuccess = (obj) => {
    this.setState({ data: obj, showScore: false });
    this.rules = { ...obj };
    this.loadData();
  };
  //加载统计率 进步退步持平率
  getSubjectInfo = async (branchId, campusId, subjectGroupId) => {
    this.setState({ loadingTop: true });
    let { branchIdF, campusIdF, sectionIdF, route } = this.state;
    if (branchId) branchIdF = branchId;
    if (campusId) campusIdF = campusId;
    if (subjectGroupId) sectionIdF = subjectGroupId;
    let searchType = route === 'StudentProgressRate' ? 1 : route === 'ManagerProgressRate' ? 3 : 2;
    branchIdF && (this.status = 'branch');
    branchIdF && campusIdF && (this.status = 'campus');
    branchIdF && campusIdF && sectionIdF && (this.status = 'section');
    let apiParams = {
      examinationType: this.rules.examinationType,
      schoolYear: this.rules.schoolYear,
      searchType, //1：集团，2：老师，3学管师
      semester: this.rules.semester,
      branchId: branchIdF,
      campusId: campusIdF,
      subjectGroupId: sectionIdF,
    };
    let [{ data }, { data: rules }] = await Promise.all([getSubjectStatisticsInfo(apiParams), getRules()]);
    this.$chartBlock.initCharts(data, rules);
    this.setState({ loadingTop: false, rules });
  };
  //加载集团top50数据
  getTop50RankInfos = async ({ rankType, targetType } = {}) => {
    this.setState({ loadingMid: true });
    this.echartsBigs.clear();
    let apiParams = {
      examinationType: this.rules.examinationType,
      rankType: rankType || 2, //1：统计率，2：进步率，3：持平率，4：退步率
      schoolYear: this.rules.schoolYear,
      semester: this.rules.semester,
      targetType: targetType || 'BRANCH', //分公司：BRANCH  学习中心：CAMPUS  科组：SUBJECT_GROUP  老师：TEACHER  学管师：STUDY_MANAGER
    };
    this.exportTop50Params = apiParams;
    let { data } = await getTop50RankInfos(apiParams);

    option.xAxis.data = [];
    option.series[0].data = [];
    let noData;
    if (!data) {
      data = [];
      noData = true;
    }
    data.forEach((v, k) => {
      option.xAxis.data.push(v.name);
      let value = v.rate * 100;
      value = value.toFixed(2);
      option.series[0].data.push({ value, name: v.selfFlag });
    });
    this.echartsBigs.setOption(option);

    this.setState({ loadingMid: false, noData });
  };
  //一对一进步率列表
  getOrgRankList = async (searchKey = {}, branchIdKey, campusIdKey, subjectGroupIdKey) => {
    this.setState({ loading: true, expandedRowKeys: [] });
    let { branchId, campusId, sectionId, route } = this.state;
    if (branchIdKey) branchId = branchIdKey;
    if (campusIdKey) campusId = campusIdKey;
    if (subjectGroupIdKey) sectionId = subjectGroupIdKey;
    let { schoolYear, semester, examinationType } = this.rules;
    let apiParams;
    let pageNo = searchKey.pageNo || 1;
    let pageSize = searchKey.pageSize || 10;
    if (route !== 'StudentProgressRate') {
      apiParams = {
        schoolYear,
        semester,
        pageNo,
        pageSize,
      };
      let getListData = route === 'ManagerProgressRate' ? getInfoByManager : getInfosByTeacher;
      let { data, rowCount } = await getListData(apiParams);
      if (!data) data = [];
      this.setState({ list: data, loading: false, pageSize, currentPage: pageNo, totalRecords: rowCount });
    } else {
      apiParams = {
        branchId,
        campusId,
        examinationType,
        pageNo,
        pageSize,
        schoolYear,
        semester,
        subjectGroupId: sectionId,
      };
      let { datas, rowCount } = await getOrgRankList(apiParams);
      !datas && (datas = []);
      let list = datas.map((v, k) => {
        return {
          branchName: v.branchName,
          branchId: v.branchId,
          campusId: v.campusId,
          campusName: v.campusName,
          subjectGroupName: v.subjectGroupName,
          subjectGroupId: v.subjectGroupId,
          graduatingReadingSubjectCount: v.graduatingReadingSubjectCount, //毕业班在读科次
          nonGraduatingReadingSubjectCount: v.nonGraduatingReadingSubjectCount, //非毕业班在读科次
          readingSubjectCount: v.readingSubjectCount, //在读科次
          statisticalSubjectCount: v.statisticalSubjectCount, //已统计科次
          graduatingStatisticalSubjectCount: v.graduatingStatisticalSubjectCount, //毕业班统计科次
          nonGraduatingStatisticalSubjectCount: v.nonGraduatingStatisticalSubjectCount, //非毕业班统计科次
          validStatisticalSubjectCount: v.validStatisticalSubjectCount,
          progressSubjectCount: v.progressSubjectCount,
          statisticalRate: v.statisticalRate,
          progressRate: v.progressRate,
          progressRateRank: v.progressRateRank,
          statisticalRateStatus: v.statisticalRateStatus,
          children: v.subjectGroupId ? null : [],
          key: k + 1,
        };
      });
      //设置searchKey
      this.setState({ loading: false, list, pageSize, currentPage: pageNo, totalRecords: rowCount });
    }
  };

  changeBranchDataF = async (value) => {
    this.setState({ branchIdF: value, campusIdF: null });
    if (value) {
      let campusDictF = await loadCampusDict(value);
      this.setState({ campusDictF });
    }
  };
  changeCampusDataF = async (value) => {
    let { branchIdF } = this.state;
    this.setState({ campusIdF: value, sectionIdF: null });
    if (value) {
      let apiParams = { branchId: branchIdF, campusId: value, ...this.rules };
      let { data } = await getGroupList(apiParams);
      let dict = data.map((v, k) => {
        return { val: v.ID, text: v.NAME };
      });
      this.setState({ sectionDictF: dict });
    }
  };

  resetF = () => {
    this.setState({ campusIdF: null, sectionIdF: null });
  };
  reset = () => {
    this.setState({ campusId: null, sectionId: null });
  };
  changeBranchData = async (val) => {
    this.setState({ branchId: val, campusId: null });
    if (val) {
      let campusDict = await loadCampusDict(val);
      this.setState({ campusDict });
    }
  };
  changeCampusData = async (val) => {
    let { branchId } = this.state;
    this.setState({ campusId: val, sectionId: null });
    if (val) {
      let apiParams = { branchId: branchId, campusId: val, ...this.rules };
      let { data } = await getGroupList(apiParams);
      let dict = data.map((v, k) => {
        return { val: v.ID, text: v.NAME };
      });
      this.setState({ sectionDict: dict });
    }
  };
  //列表数据
  onExpand = async (expanded, record) => {
    let { list, expandedRowKeys } = this.state;
    if (expanded) {
      expandedRowKeys.push(record.key);
      let { branchId, campusId } = record;
      let { sectionId } = this.state;
      this.setState({ loading: true });
      let { data } = await getOrgRankInfoForLevel({ branchId, campusId, ...this.rules });
      if (!data) data = [];
      if (record.parentId) {
        let sub = list.find((v, k) => {
          console.log(v);
          return v.key === record.parentId;
        });
        let obj = sub.children.find((v, k) => {
          return v.key === record.key;
        });
        let children = data.map((v, k) => {
          return {
            branchName: v.branchName,
            branchId: v.branchId,
            campusId: v.campusId,
            campusName: v.campusName,
            subjectGroupName: v.subjectGroupName,
            subjectGroupId: v.subjectGroupId,
            graduatingReadingSubjectCount: v.graduatingReadingSubjectCount, //毕业班在读科次
            nonGraduatingReadingSubjectCount: v.nonGraduatingReadingSubjectCount, //非毕业班在读科次
            readingSubjectCount: v.readingSubjectCount, //在读科次
            statisticalSubjectCount: v.statisticalSubjectCount, //已统计科次
            graduatingStatisticalSubjectCount: v.graduatingStatisticalSubjectCount, //毕业班统计科次
            nonGraduatingStatisticalSubjectCount: v.nonGraduatingStatisticalSubjectCount, //非毕业班统计科次
            validStatisticalSubjectCount: v.validStatisticalSubjectCount,
            progressSubjectCount: v.progressSubjectCount,
            statisticalRate: v.statisticalRate,
            progressRate: v.progressRate,
            progressRateRank: v.progressRateRank,
            statisticalRateStatus: v.statisticalRateStatus,
            parentId: obj.key,
            key: Math.random(),
          };
        });
        obj.children = children;
      } else {
        let obj = list.find((v, k) => {
          return v.key === record.key;
        });
        let children = data.map((v, k) => {
          return {
            branchName: v.branchName,
            branchId: v.branchId,
            campusId: v.campusId,
            campusName: v.campusName,
            subjectGroupName: v.subjectGroupName,
            subjectGroupId: v.subjectGroupId,
            graduatingReadingSubjectCount: v.graduatingReadingSubjectCount, //毕业班在读科次
            nonGraduatingReadingSubjectCount: v.nonGraduatingReadingSubjectCount, //非毕业班在读科次
            readingSubjectCount: v.readingSubjectCount, //在读科次
            statisticalSubjectCount: v.statisticalSubjectCount, //已统计科次
            graduatingStatisticalSubjectCount: v.graduatingStatisticalSubjectCount, //毕业班统计科次
            nonGraduatingStatisticalSubjectCount: v.nonGraduatingStatisticalSubjectCount, //非毕业班统计科次
            validStatisticalSubjectCount: v.validStatisticalSubjectCount,
            progressSubjectCount: v.progressSubjectCount,
            statisticalRate: v.statisticalRate,
            progressRate: v.progressRate,
            progressRateRank: v.progressRateRank,
            statisticalRateStatus: v.statisticalRateStatus,
            children: v.subjectGroupId ? null : [],
            parentId: obj.key,
            key: Math.random(),
          };
        });
        obj.children = children;
      }
      this.setState({ list, loading: false, expandedRowKeys });
    } else {
      let idx = expandedRowKeys.findIndex((v, k) => record.key === v);
      expandedRowKeys.splice(idx, 1);
      this.setState({ expandedRowKeys });
    }
  };

  onExpandedRowsChange = (expandedRows) => {};

  //显示学生弹窗
  showStudentModal = (record) => {
    console.log(record);
    let { branchId, branchName, campusId, campusName, subjectGroupId, subjectGroupName } = record;
    let prefixTitle = branchName + campusName + subjectGroupName;
    this.setState({ showStudentList: true, studentInfoKey: { branchId, campusId, subjectGroupId }, prefixTitle });
  };
  //翻页加载数据
  loadListByPage = ({ current, pageSize }) => {
    this.getOrgRankList({ pageNo: current, pageSize: pageSize });
  };

  changeAdvanceValue = (val, type) => {
    this.setState({ active: val, [type]: val });
    this.targetType = val;
    this.getTop50RankInfos({ rankType: type === 'advanceValue' ? 2 : 1, targetType: val.toUpperCase() });
  };
  //点击跳转成绩管理页面
  goToNewPage = (record) => {
    let { dispatch, name } = this.props;
    let { studentName, studentId } = record;
    let type = this.state.route === 'ManagerProgressRate' ? 'normal' : 'other';
    dispatch({
      type: 'main/openPage',
      payload: {
        path: '/scoreManage',
        params: { studentId, studentName, studyManagerName: name, headTeacherName: name, type },
      },
    });
  };

  //导出报表
  exportTop50 = async () => {
    if (this.limitClickTag === 1) {
      let { schoolYearDict, examinationTypeDict } = this.state;
      this.setState({ loadingMid: true });
      this.limitClickTag = 2;
      let res = await exportTop50(this.exportTop50Params).catch((res) => this.setState({ loadingMid: false }));
      let dict = this.targetType === this.targetType.toUpperCase() ? dict1 : dict2;
      this.limitClickTag = 1;
      this.setState({ loadingMid: false });
      let { data, resultCode, resultMessage } = res;
      if (resultCode === 0) {
        let fileName =
          getDictTextByVal(schoolYearDict, this.exportTop50Params.schoolYear) +
          getDictTextByVal(semesterDict, this.exportTop50Params.semester) +
          getDictTextByVal(examinationTypeDict, this.exportTop50Params.examinationType) +
          getDictTextByVal(dict, this.targetType) +
          '.xls';
        download(data, fileName);
      } else {
        ShowMessage(resultMessage);
      }
    }
  };
  downloadClick = () => {
    let { route } = this.state;
    if (route === 'StudentProgressRate') {
      this.exportOTOList();
    } else {
      this.exportList();
    }
  };

  exportOTOList = async () => {
    if (this.limitClickTag === 1) {
      let { branchId, campusId, sectionId, branchDict, campusDict, sectionDict } = this.state;
      let apiParams = {
        branchId,
        campusId,
        examinationType: this.rules.examinationType,
        schoolYear: this.rules.schoolYear,
        semester: this.rules.semester,
        subjectGroupId: sectionId,
      };
      let { schoolYearDict, examinationTypeDict } = this.state;
      this.setState({ loading: true });
      this.limitClickTag = 2;
      let res = await exportOrg(apiParams).catch((res) => this.setState({ loading: false }));
      this.limitClickTag = 1;
      this.setState({ loading: false });
      let { data, resultCode, resultMessage } = res;
      if (resultCode === 0) {
        let fileName =
          getDictTextByVal(schoolYearDict, this.rules.schoolYear) +
          getDictTextByVal(semesterDict, this.rules.semester) +
          getDictTextByVal(examinationTypeDict, this.rules.examinationType) +
          getDictTextByVal(branchDict, branchId) +
          (campusId ? getDictTextByVal(campusDict, campusId) : '') +
          (sectionId ? getDictTextByVal(sectionDict, sectionId) : '') +
          '指标统计.xls';
        download(data, fileName);
      } else {
        ShowMessage(resultMessage);
      }
    }
  };

  exportList = async () => {
    let { route } = this.state;
    let fn = route === 'ManagerProgressRate' ? exportManager : exportTeacher;
    if (this.limitClickTag === 1) {
      let { schoolYear, semester } = this.rules;
      let apiParams = { schoolYear, semester };
      this.setState({ loading: true });
      this.limitClickTag = 2;
      let res = await fn(apiParams).catch((res) => this.setState({ loading: false }));
      this.limitClickTag = 1;
      this.setState({ loading: false });
      let { data, resultCode, resultMessage } = res;
      if (resultCode === 0) {
        download(data, '我的学生列表.xls');
      } else {
        ShowMessage(resultMessage);
      }
    }
  };

  componentWillUnmount() {
    this.echartsBigs && this.echartsBigs.dispose();
    window.removeEventListener('resize', this.resize, false);
  }

  render() {
    let {
      examinationTypeDict,
      active,
      pageLoading,
      loading,
      loadingTop = false,
      loadingMid = false,
      currentPage,
      pageSize,
      totalRecords,
      list,
      showScore,
      showRule,
      branchDict,
      showStudentList,
      campusDict,
      sectionDict,
      branchId,
      branchIdF,
      campusId,
      campusIdF,
      sectionId,
      sectionIdF,
      advanceValue = 'BRANCH',
      countValue = 'branch',
      schoolYearDict,
      campusDictF,
      sectionDictF,
      studentInfoKey,
      route,
      noData = false,
      data = {},
      prefixTitle,
      rules,
      expandedRowKeys,
    } = this.state;

    let columns;
    if (route === 'StudentProgressRate') {
      columns = [
        { title: '', dataIndex: 'space', key: 'space' },
        { title: '分公司', dataIndex: 'branchName', width: COL_WIDTH_M, key: 'branchName' },
        { title: '学习中心', dataIndex: 'campusName', width: COL_WIDTH_M, key: 'campusName' },
        { title: '科组', dataIndex: 'subjectGroupName', width: COL_WIDTH_M, key: 'subjectGroupName' },
        { title: '在读科次', dataIndex: 'readingSubjectCount', width: COL_WIDTH_M, key: 'readingSubjectCount' },
        {
          title: '已统计科次',
          dataIndex: 'statisticalSubjectCount',
          width: COL_WIDTH_M,
          key: 'statisticalSubjectCount',
        },
        {
          title: '有效科次',
          dataIndex: 'validStatisticalSubjectCount',
          width: COL_WIDTH_M,
          key: 'validStatisticalSubjectCount',
        },
        { title: '进步科次', dataIndex: 'progressSubjectCount', width: COL_WIDTH_M, key: 'progressSubjectCount' },
        {
          title: '统计率',
          dataIndex: 'statisticalRate',
          width: COL_WIDTH_M,
          key: 'statisticalRate',
          render: (text, record, idx) => {
            let {
              nonGraduatingReadingSubjectCount,
              graduatingReadingSubjectCount,
              graduatingStatisticalSubjectCount,
              nonGraduatingStatisticalSubjectCount,
              validStatisticalSubjectCount,
              branchId,
              campusId,
              subjectGroupId,
              statisticalRateStatus,
            } = record;
            let valid = statisticalRateStatus === 'Y';
            let tags;
            if (branchId) {
              tags = 'branch';
              if (campusId) {
                tags = 'campus';
                if (subjectGroupId) {
                  tags = 'subjectGroup';
                }
              }
            }
            let validCountStandrad, graduateStandrad, nonGraduateStandrad;
            if (tags === 'branch') {
              validCountStandrad = rules.branchValidSubjectCount;
              graduateStandrad = (rules.branchGraduatingRate * graduatingReadingSubjectCount) / 100;
              nonGraduateStandrad = (rules.branchNonGraduatingRate * nonGraduatingReadingSubjectCount) / 100;
            } else if (tags === 'campus') {
              validCountStandrad = rules.campusValidSubjectCount;
              graduateStandrad = (rules.campusGraduatingRate * graduatingReadingSubjectCount) / 100;
              nonGraduateStandrad = (rules.campusNonGraduatingRate * nonGraduatingReadingSubjectCount) / 100;
            } else {
              validCountStandrad = rules.subjectGroupValidSubjectCount;
              graduateStandrad = (rules.subjectGroupGraduatingRate * graduatingReadingSubjectCount) / 100;
              nonGraduateStandrad = (rules.subjectGroupNonGraduatingRate * nonGraduatingReadingSubjectCount) / 100;
            }
            let graduateNum = Math.round(graduateStandrad) - graduatingStatisticalSubjectCount;
            let nonGraduateNum = Math.round(nonGraduateStandrad) - nonGraduatingStatisticalSubjectCount;
            return (
              <span style={{ paddingLeft: 15, position: 'relative', color: valid ? '' : 'red' }}>
                {validCountStandrad > validStatisticalSubjectCount ? (
                  <IconButton
                    style={{ position: 'absolute', top: 2, left: -5 }}
                    title={{
                      placement: 'bottom',
                      overlayClassName: 'compareTips',
                      title: `有效统计科次需达到${validCountStandrad}个`,
                    }}
                    type="warning"
                  />
                ) : graduateNum > 0 || nonGraduateNum > 0 ? (
                  <IconButton
                    style={{ position: 'absolute', top: 2, left: -5 }}
                    title={{
                      placement: 'bottom',
                      overlayClassName: 'compareTips',
                      title:
                        (graduateNum > 0 ? `毕业班科次还需统计${graduateNum}个；` : '') +
                        (nonGraduateNum > 0 ? `非毕业班科次还需统计${nonGraduateNum}个；` : ''),
                    }}
                    type="warning"
                  />
                ) : (
                  ''
                )}
                {text}
              </span>
            );
          },
        },
        {
          title: '进步率',
          dataIndex: 'progressRate',
          width: COL_WIDTH_M,
          key: 'progressRate',
          render: (text, record, idx) => {
            let { statisticalRateStatus } = record;
            let valid = statisticalRateStatus === 'Y';
            return (
              <span style={{ paddingLeft: 15, position: 'relative', color: valid ? '' : 'red' }}>
                {valid ? (
                  ''
                ) : (
                  <IconButton
                    style={{ position: 'absolute', top: 2, left: -5 }}
                    title={{
                      placement: 'bottom',
                      overlayClassName: 'compareTips',
                      title: `统计率未达标`,
                    }}
                    type="warning"
                  />
                )}
                {text}
              </span>
            );
          },
        },
        {
          title: '集团排行',
          dataIndex: 'progressRateRank',
          width: COL_WIDTH_M,
          key: 'progressRateRank',
          render: (text, record, idx) => {
            let { statisticalRateStatus } = record;
            let valid = statisticalRateStatus === 'Y';
            return (
              <span style={{ paddingLeft: 15, position: 'relative', color: valid ? '' : 'red' }}>
                {valid ? (
                  ''
                ) : (
                  <IconButton
                    style={{ position: 'absolute', top: 2, left: -5 }}
                    title={{
                      placement: 'bottom',
                      overlayClassName: 'compareTips',
                      title: `统计率未达标`,
                    }}
                    type="warning"
                  />
                )}
                {text}
              </span>
            );
          },
        },
        {
          title: '操作',
          width: COL_WIDTH_M,
          key: 'achievementTypeList',
          render: (text, record, index) => {
            let { subjectGroupName } = record;
            return subjectGroupName ? (
              <span
                className={styles.handle}
                onClick={(e) => {
                  this.showStudentModal(record);
                }}
              >
                查看学生
              </span>
            ) : null;
          },
        },
      ];
    } else {
      columns = [
        { title: '学生', dataIndex: 'studentName', width: COL_WIDTH_M, key: 'studentName' },
        { title: '年级', dataIndex: 'gradeName', width: COL_WIDTH_M, key: 'gradeName' },
        { title: '1对1学生状态', dataIndex: 'studentStatus', width: COL_WIDTH_M, key: 'studentStatus' },
        { title: '进步判定', dataIndex: 'standardCategory', width: COL_WIDTH_M, key: 'standardCategory' },
        { title: '进步判定备注', dataIndex: 'benchmarkType', width: COL_WIDTH_M, key: 'benchmarkType' },
        { title: '进步判定原因', dataIndex: 'reason', width: COL_WIDTH_M, key: 'reason' },
        { title: '基准成绩名称', dataIndex: 'benchmarkName', width: COL_WIDTH_M, key: 'benchmarkName' },
        { title: '基准成绩', dataIndex: 'benchmarkScore', width: COL_WIDTH_M, key: 'benchmarkScore' },
        { title: '对比成绩名称', dataIndex: 'compareName', width: COL_WIDTH_M, key: 'compareName' },
        { title: '对比成绩', dataIndex: 'compareScore', width: COL_WIDTH_M, key: 'compareScore' },
        {
          title: '操作',
          dataIndex: 'action',
          width: COL_WIDTH_M,
          key: 'action',
          render: (text, record, index) => {
            return (
              <span className={styles.handle} onClick={(e) => this.goToNewPage(record)}>
                成绩管理
              </span>
            );
          },
        },
      ];
    }
    return (
      <Page loading={pageLoading} footer={false} title={false} route={this.props.route} className={styles.progressRate}>
        <HandleBar
          data={data}
          dict={{ schoolYearDict, semesterDict, examinationTypeDict }}
          showScoreSetting={(e) => this.setState({ showScore: true })}
          showRuleSetting={(e) => this.setState({ showRule: true })}
          showRuleBtn={route === 'StudentProgressRate'}
        />
        <ChartBlock ref={(e) => (this.$chartBlock = e)} status={this.status} route={route}>
          <Loading loading={loadingTop} />
          {route === 'StudentProgressRate' && (
            <SearchBar expand={false} onSearch={this.getSubjectInfo} onReset={this.resetF}>
              <Select
                placeholder="分公司"
                showSearch={true}
                dict={branchDict}
                value={branchIdF}
                allowClear={false}
                onChange={this.changeBranchDataF}
              />
              <Select
                placeholder="学习中心"
                showSearch={true}
                dict={campusDictF}
                value={campusIdF}
                onChange={this.changeCampusDataF}
              />
              <Select
                placeholder="科组"
                showSearch={true}
                dict={sectionDictF}
                value={sectionIdF}
                onChange={(val) => this.setState({ sectionIdF: val })}
              />
            </SearchBar>
          )}
        </ChartBlock>
        {route === 'StudentProgressRate' && (
          <div className={styles.section + ' ' + styles.download}>
            <div>
              <div>集团榜单</div>
              <div>
                <IconButton
                  type={'question_mc'}
                  style={{ marginRight: 15 }}
                  title={{
                    placement: 'bottom',
                    overlayClassName: 'compareTips',
                    title: `可导出当前榜单对应的数据为Excel格式`,
                  }}
                />
                <a>
                  <Button onClick={this.exportTop50}>导出EXCEL</Button>
                </a>
              </div>
            </div>
            <div>
              <div className={styles.select}>
                <Select
                  allowClear={false}
                  onSelect={(val) => this.changeAdvanceValue(val, 'advanceValue')}
                  dropdownClassName={styles.dropDown}
                  dict={dict1}
                  value={advanceValue}
                  className={dict1.filter((v, k) => v.val === active).length === 1 ? 'active' : ''}
                />
                <Select
                  allowClear={false}
                  onSelect={(val) => this.changeAdvanceValue(val, 'countValue')}
                  dropdownClassName={styles.dropDown}
                  dict={dict2}
                  value={countValue}
                  className={dict2.filter((v, k) => v.val === active).length === 1 ? 'active' : ''}
                />
              </div>
              {noData && <NoData />}
              <div ref={(e) => (this.echartsBig = e)} style={{ height: 420, display: noData ? 'none' : 'block' }}></div>
            </div>
            <Loading loading={loadingMid} />
          </div>
        )}
        <div className={styles.section}>
          <div>{route === 'StudentProgressRate' ? '指标统计' : '我的学生列表'}</div>
          {route === 'StudentProgressRate' ? (
            <SearchBar expand={false} onSearch={this.getOrgRankList} onReset={this.reset}>
              <Select
                placeholder="分公司"
                showSearch={true}
                dict={branchDict}
                value={branchId}
                allowClear={false}
                onChange={this.changeBranchData}
              />
              <Select
                placeholder="学习中心"
                showSearch={true}
                dict={campusDict}
                value={campusId}
                onChange={this.changeCampusData}
              />
              <Select
                placeholder="科组"
                showSearch={true}
                dict={sectionDict}
                value={sectionId}
                onChange={(val) => this.setState({ sectionId: val })}
              />
            </SearchBar>
          ) : (
            ''
          )}
          <div style={{ position: 'absolute', top: 15, right: 15 }}>
            <IconButton
              type={'question_mc'}
              style={{ marginRight: 15 }}
              title={{
                placement: 'bottom',
                overlayClassName: 'compareTips',
                title: `可导出当前列表数据为Excel格式`,
              }}
            />
            <a>
              <Button onClick={this.downloadClick}>导出EXCEL</Button>
            </a>
          </div>
          <List
            dataSource={list}
            columns={columns}
            loading={loading}
            pagination={{
              current: currentPage,
              total: totalRecords,
              pageSize,
            }}
            expandedRowKeys={expandedRowKeys}
            onExpandedRowsChange={this.onExpandedRowsChange}
            onExpand={this.onExpand}
            indentSize={0}
            onChange={this.loadListByPage}
          />
        </div>
        {showScore && (
          <ScoreSetting
            dict={{ schoolYearDict, semesterDict, examinationTypeDict }}
            data={data}
            onOk={this.saveSettingSuccess}
            onClose={(e) => this.setState({ showScore: false })}
          />
        )}
        {showRule && <RuleSetting onOk={this.loadData} onClose={(e) => this.setState({ showRule: false })} />}
        {showStudentList && (
          <StudentList
            rules={data}
            prefixTitle={prefixTitle}
            dict={{ schoolYearDict, semesterDict, examinationTypeDict }}
            studentInfoKey={studentInfoKey}
            onClose={(e) => this.setState({ showStudentList: false })}
          />
        )}
      </Page>
    );
  }
}
const mapStateToProps = (state) => {
  const user = state.account.user;
  return {
    branchId: user.brenchId,
    campusId: user.campusId,
    grounpId: user.grounpId,
    name: user.name,
  };
};
export default connect(mapStateToProps)(ProgressRate);
