import React from 'react';
import Page from 'COMPONENT/Page';
import { connect } from 'dva';
import { loadCommonDict, loadBranchDict, loadCampusDict } from 'UTIL/dict';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import styles from './styles.less';
import { Row, Col } from 'antd';
import List from 'COMPONENT/List';
import Button from 'COMPONENT/Button';
import loaders from '../loader';
import SelectClassModalContainer from '../selectClassModal';
import SelectStudentModal from '../selectStudentModal';
import showAlert from 'COMPONENT/utils/globalTips/showAlert';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import showConfirmClose from 'COMPONENT/utils/globalTips/showConfirmClose';
import CheckStudentsList from '../checkStudentsList';
import { loadMainTcApi, loadAssitTcApi } from 'SERVICE/course';
import Steps from './components/Steps';
import Footer from './components/Footer';
import { SYSTEM_MARK } from 'CONST/config';
const isXINGHUO = SYSTEM_MARK === 'XINGHUO';

class SelectQualificationType extends React.Component {
  constructor(props) {
    super(props);

    this.state = {};
  }

  componentDidMount() {}

  componentWillReceiveProps(nextProps) {
    this.setState({
      branchId: nextProps.branchId,
      campusId: nextProps.campusId,
      productTypeId: nextProps.productTypeId,
      yearId: nextProps.yearId,
      quarterId: nextProps.quarterId,
      gradeId: nextProps.gradeId,
      subjectId: nextProps.subjectId,
      classTypeId: nextProps.classTypeId,
      classLevelId: nextProps.classLevelId,
    });
  }

  render() {
    const {} = this.state;
    const {
      loading,
      branchDict,
      campusDict,
      productTypeDict,
      yearDict,
      quarterDict,
      gradeDict,
      subjetDict,
      classTypeDict,
      classLevelDict,
      branchId,
      campusId,
      productTypeId,
      yearId,
      quarterId,
      gradeId,
      subjectId,
      classTypeId,
      classLevelId,
      handleClassTypeChange,
      handleBranchChange,
      handleSelected,
      handleProductTypeChange,
      classIds,
      students,
    } = this.props;
    return (
      <div className={styles.paddingLeft}>
        <h1>选择报读资格类型</h1>
        <Row gutter={56}>
          <Col span={8} className={styles.colFlex}>
            <div className={styles.tipFont}>分公司</div>
            <Select
              value={branchId}
              dict={branchDict}
              className={styles.select}
              allowClear={false}
              onChange={(branchId) => {
                handleBranchChange(branchId);
              }}
            />
          </Col>
          <Col span={8} className={styles.colFlex}>
            <div className={styles.tipFont}>产品类型</div>
            <Select
              value={productTypeId}
              dict={productTypeDict}
              className={styles.select}
              allowClear={false}
              disabled={students.length !== 0}
              onChange={(productTypeId) => {
                handleProductTypeChange(productTypeId);
              }}
            />
          </Col>
          <Col span={8} className={styles.colFlex}>
            <div className={styles.tipFont}>产品年份</div>
            <Select
              value={yearId}
              dict={yearDict}
              className={styles.select}
              allowClear={false}
              onChange={(yearId) => {
                handleSelected(yearId, 'yearId');
              }}
            />
          </Col>
          <Col span={8} className={styles.colFlex}>
            <div className={styles.tipFont}>产品季度</div>
            <MultiSelect
              value={quarterId}
              dict={quarterDict}
              className={styles.select}
              allowClear={false}
              onChange={(quarterId) => {
                handleSelected(quarterId, 'quarterId');
              }}
            />
          </Col>
          <Col span={8} className={styles.colFlex}>
            <div className={styles.tipFont}>年级</div>
            <Select
              value={gradeId}
              dict={gradeDict}
              className={styles.select}
              allowClear={false}
              onChange={(gradeId) => {
                handleSelected(gradeId, 'gradeId');
              }}
            />
          </Col>
          <Col span={8} className={styles.colFlex}>
            <div className={styles.tipFont}>科目</div>
            <MultiSelect
              value={subjectId}
              dict={subjetDict}
              className={styles.select}
              allowClear={false}
              onChange={(subjectId) => {
                handleSelected(subjectId, 'subjectId');
              }}
            />
          </Col>
          <Col span={8} className={styles.colFlex}>
            <div className={styles.tipFont}>班级类型</div>
            <Select
              value={classTypeId}
              dict={classTypeDict}
              className={styles.select}
              allowClear={false}
              onChange={(value) => {
                handleClassTypeChange(value);
              }}
            />
          </Col>
          <Col span={8} className={styles.colFlex}>
            <div className={styles.tipFont}>班级分层</div>
            <Select
              value={classLevelId}
              dict={classLevelDict}
              className={styles.select}
              allowClear={false}
              onChange={(classLevelId) => {
                handleSelected(classLevelId, 'classLevelId');
              }}
            />
          </Col>
          <Col span={8} className={styles.colFlex}>
            <div className={styles.tipFont}>资格类型</div>
            <span>免考</span>
          </Col>
        </Row>
      </div>
    );
  }
}

class SelectQualificationTypeContainer extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false,
      branchDict: null,
      campusDict: null,
      productTypeDict: [
        { val: 'SMALL_CLASS', text: '小班' },
        { val: 'TWO_TEACHER', text: '双师' },
      ],
      yearDict: null,
      quarterDict: null,
      gradeDict: null,
      subjetDict: null,
      classTypeDict: null,
      classLevelDict: null,
      branchId: null,
      campusId: null,
      productTypeId: isXINGHUO ? 'SMALL_CLASS' : 'TWO_TEACHER',
      yearId: null,
      quarterId: null,
      gradeId: null,
      subjectId: null,
      classTypeId: null,
      classLevelId: null,
      isSelectClass: false,
      isSelectStudent: false,
      students: [], // 选中
      studentIds: null, // 选择学生中新增的学生id
      classIds: null, // 选择班级中的选中的班级Id
      current: null, // 当前页码
      total: null, // 总数据量
      deleteStudents: [], // 列表中被删除学生
      mainTeacher: null, //主讲老师
      assitTeacher: null, //辅讲老师
      // steps 进度数
      currentIndex: 0,
      content: '',
      // 按钮弹框
      popTipVisible: false,
    };
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化
   */
  initContent = async () => {
    this.setState({ loading: true });
    let [branchDict, yearDict, quarterDict, gradeDict, subjetDict, classTypeDict, mainTeacher, assitTeacher] =
      await Promise.all([
        loadBranchDict(),
        loadCommonDict('PRODUCT_VERSION'),
        loadCommonDict('PRODUCT_QUARTER'),
        loadCommonDict('STUDENT_GRADE'),
        loadCommonDict('SUBJECT'),
        loadCommonDict('CLASS_TYPE', 'SMALL_CLASS'),
        loadMainTcApi(),
        loadAssitTcApi(),
      ]);
    mainTeacher = mainTeacher.map((teacher) => {
      return {
        val: teacher.userId,
        text: teacher.name,
      };
    });
    assitTeacher = assitTeacher.map((teacher) => {
      return {
        val: teacher.userId,
        text: teacher.name,
      };
    });
    this.setState({
      branchDict,
      yearDict,
      quarterDict,
      gradeDict,
      subjetDict,
      classTypeDict,
      mainTeacher,
      assitTeacher,
    });
    let currentYear = new Date().toString().slice(11, 15);
    let yearId = yearDict.filter((year) => {
      return year.text === currentYear;
    })[0].val;
    this.handleBranchChange(branchDict[0].val);
    this.handleClassTypeChange(classTypeDict[0].val);
    this.setState({
      branchId: branchDict[0].val,
      yearId: yearId,
      quarterId: [quarterDict[0].val],
      gradeId: gradeDict[0].val,
      subjectId: [subjetDict[0].val],
      classTypeId: classTypeDict[0].val,
    });
    const classLevelDict = await loadCommonDict('CLASS_LEVEL', classTypeDict[0].val);
    this.setState({ classLevelDict, classLevelId: classLevelDict[0].val });
    this.setState({ loading: false });
  };

  /**
   * 分公司改变获取对应的学习中心字典
   */
  handleBranchChange = async (branchId) => {
    await this.setState({ branchId, campusId: null });
    if (branchId) {
      const campusDict = await loadCampusDict(branchId);
      this.setState({
        campusDict,
        campusId: campusDict[0].val,
      });
    }
  };

  /**
   * 班级类型改变获取对应的班级层次
   */
  handleClassTypeChange = async (classTypeId) => {
    await this.setState({ classTypeId, classLevelId: null });
    if (classTypeId) {
      const classLevelDict = await loadCommonDict('CLASS_LEVEL', classTypeId);
      this.setState({
        classLevelDict,
      });
    }
  };

  /**
   *  产品类型改变获取对应的班级类型
   */
  handleProductTypeChange = async (productTypeId) => {
    this.setState({ classTypeId: null, classLevelId: null, productTypeId });
    if (productTypeId) {
      const classTypeDict = await loadCommonDict('CLASS_TYPE', productTypeId);
      this.setState({ classTypeDict });
    }
  };

  /**
   * 处理下拉框选中改变
   */
  handleSelected = (value, type) => {
    this.setState({ [type]: value });
  };

  /**
   * 打开选择班级Modal
   */
  openSelectClass = () => {
    this.setState({ isSelectClass: true });
  };

  /**
   * 关闭选择班级Modal
   */
  closeSelectClass = () => {
    this.setState({ isSelectClass: false });
  };

  /**
   * 打开选择班级Modal
   */
  openSelectStudent = () => {
    this.setState({ isSelectStudent: true });
  };

  /**
   * 关闭选择学生Modal
   */
  closeSelectStudent = () => {
    this.setState({ isSelectStudent: false });
  };

  /**
   * 保存学生列表及分页信息
   */
  saveStudents = (data) => {
    this.setState({ students: data.studentDatas, current: data.current, total: data.total });
  };

  /**
   * 保存班级Id
   */
  saveClassIds = (classId) => {
    let { classIds } = this.state;

    let nextClassIds = classIds;
    if (classIds) {
      nextClassIds += `,${classId}`;
    } else {
      nextClassIds = classId;
    }
    this.setState({ classIds: nextClassIds });
  };

  /**
   * 重复选择学生，覆盖学生被删除状态
   */
  coverStudentStatus = (newStudentIds) => {
    let { deleteStudents } = this.state;
    newStudentIds = newStudentIds.split(',');
    deleteStudents = deleteStudents.filter((student) => {
      return newStudentIds.includes(student) !== true;
    });
    this.setState({ deleteStudents });
  };

  /**
   * 保存学生Id并保存学生数据
   */
  saveNewStudentsId = (newStudentIds, newStudents) => {
    let { students, studentIds } = this.state;
    if (studentIds !== null) {
      studentIds = studentIds + ',' + newStudentIds;
    } else {
      studentIds = newStudentIds;
    }
    studentIds = Array.from(new Set(studentIds.split(','))).join(',');
    students = students.concat(newStudents);
    const studentsObj = students.reduce((obj, ele) => {
      obj[ele.studentId] = ele;
      return obj;
    }, {});
    students = Object.keys(studentsObj).map((ele) => studentsObj[ele]);
    // let result = {};
    // let finalResult=[];
    // for(let i=0;i<students.length;i++){
    //   result[students[i].gradeName]=students[i];
    // }
    // for(var item in result){
    //   finalResult.push(result[item]);
    // }
    this.coverStudentStatus(newStudentIds);
    this.setState({ studentIds, students });
  };

  /**
   * 删除选择班级的学生
   */
  deleteStudent = (studentId) => {
    let { students, deleteStudents, total } = this.state;
    // students = students.filter((student)=>{
    //   return student.studentId !== studentId
    // })

    deleteStudents.push(studentId);
    this.setState({ deleteStudents });
    if (deleteStudents.length === total) {
      showConfirmClose(
        '是否清空学生列表?',
        () => {
          this.setState({ studentIds: null, classIds: null, students: [] });
          deleteStudents = [];
          this.setState({ deleteStudents });
        },
        () => {
          deleteStudents.pop();
          this.setState({ deleteStudents });
        },
      );
    }
  };

  /**
   * 删除选择学生的学生
   */
  deleteStudentOfSelectStudent = (studentId) => {
    let { students, deleteStudents, studentIds } = this.state;
    // students = students.filter((student)=>{
    //   return student.studentId !== studentId
    // })
    deleteStudents.push(studentId);
    this.setState({ deleteStudents });
    if (deleteStudents.length === students.length) {
      showConfirmClose(
        '是否清空学生列表?',
        () => {
          this.setState({ studentIds: null, classIds: null, students: [] });
          deleteStudents = [];
          this.setState({ deleteStudents });
        },
        () => {
          deleteStudents.pop();
          this.setState({ deleteStudents });
        },
      );
    } else {
      studentIds = studentIds
        .split(',')
        .filter((id) => {
          return id !== studentId;
        })
        .join(',');
      this.setState({ studentIds });
    }
  };

  /**
   * 恢复被删除的学生
   */
  restoreDeleteStudent = (studentId) => {
    let { deleteStudents, studentIds } = this.state;
    deleteStudents = deleteStudents.filter((student) => {
      return student !== studentId;
    });
    if (studentIds) {
      studentIds = studentIds + ',' + studentId;
    }
    this.setState({ deleteStudents, studentIds });
  };
  /**
   * 通过班级Id获取学生
   */
  searchStudents = async (pageNo = 1, pageSize) => {
    const { classIds, productTypeId } = this.state;
    let params =
      productTypeId === 'SMALL_CLASS'
        ? {
            smallClassId: classIds,
            twoteacherClasssId: '',
            productTypeId,
            pageNo,
            pageSize,
          }
        : { twoteacherClasssId: classIds, smallClassId: '', productTypeId, pageNo, pageSize };
    let data = await loaders.getClassStudentQualificationInfoApi(params);
    this.setState({ students: data.studentDatas, current: data.current, total: data.total });
  };

  /**
   *  生成报读资质
   */
  generateQualification = async () => {
    let {
      studentIds,
      branchId,
      campusId,
      classIds,
      classLevelId,
      classTypeId,
      gradeId,
      productTypeId,
      quarterId,
      deleteStudents,
      subjectId,
      yearId,
    } = this.state;
    if (campusId === null || classLevelId === null) {
      showAlert('学习中心或班级层次不能为空。');
      return;
    }
    if (!studentIds && !classIds) {
      showAlert('请选择学生。');
      return;
    }
    this.setState({ loading: true });
    let params;
    let quarter = quarterId
      .reduce((pre, currentValue) => {
        return pre + ',' + currentValue;
      }, '')
      .slice(1);

    let removeStudentId = deleteStudents
      .reduce((pre, currentValue) => {
        return pre + ',' + currentValue;
      }, '')
      .slice(1);

    subjectId = subjectId
      .reduce((pre, currentValue) => {
        return pre + ',' + currentValue;
      }, '')
      .slice(1);
    // 20200518学习中心不用传
    if (!!studentIds) {
      params = {
        addStudentId: studentIds,
        branchId,
        classLevel: classLevelId,
        classTypeId,
        gradeId,
        productTypeId,
        quarter,
        subjectId,
        years: yearId,
      };
    } else if (!!classIds) {
      params = {
        branchId,
        classLevel: classLevelId,
        classTypeId,
        gradeId,
        productTypeId,
        quarter,
        subjectId,
        years: yearId,
        removeStudentId,
        classId: classIds,
      };
    }

    let response = await loaders.addBatchStudentQualificationApi(params);
    if (response.resultCode === 0) {
      if (response.resultMessage !== '') {
        showMessage(response.resultMessage, 'success');
        this.setState({ loading: false, students: [], studentIds: null });
      } else {
        showMessage('成功生成报读资格。', 'success');
        this.setState({ loading: false, students: [], classIds: null, studentIds: null });
      }
    }
  };

  // 分步骤条的下一步按钮
  next = () => {
    const { currentIndex, studentIds, classIds } = this.state;

    // 生成资格
    if (currentIndex === 1) {
      return this.generateQualification();
    } else {
      // 显示筛选 设置currentIndex ===1 前面要做学生检验
      if (!studentIds && !classIds) {
        showAlert('请选择学生。');
        return false;
      }
      this.setState({
        currentIndex: currentIndex + 1,
      });
    }
  };
  render() {
    const {
      loading,
      branchDict,
      campusDict,
      productTypeDict,
      yearDict,
      quarterDict,
      gradeDict,
      subjetDict,
      classTypeDict,
      classLevelDict,
      branchId,
      campusId,
      productTypeId,
      yearId,
      quarterId,
      gradeId,
      subjectId,
      classTypeId,
      classLevelId,
      isSelectClass,
      isSelectStudent,
      students,
      studentIds,
      classIds,
      current,
      total,
      deleteStudents,
      mainTeacher,
      assitTeacher,
      currentIndex,
      content,
      popTipVisible,
    } = this.state;
    return (
      <Page
        loading={loading}
        name="新增报读资格"
        route={this.props.route}
        footer={
          <div className={styles.footer}>
            <Footer
              currentIndex={currentIndex}
              handlePrev={(e) => {
                this.setState({ currentIndex: currentIndex - 1 });
              }}
              handleNext={this.next}
              popTipVisible={popTipVisible}
              onClose={() => {
                this.setState({ popTipVisible: false });
              }}
              content={content}
            />
          </div>
        }
      >
        <Steps currentIndex={currentIndex}>
          <div>选择学生</div>
          <div>选择报读资格类型</div>
        </Steps>
        {currentIndex === 0 && (
          <CheckStudentsList
            students={students}
            openSelectClass={this.openSelectClass}
            openSelectStudent={this.openSelectStudent}
            studentIds={studentIds}
            classIds={classIds}
            deleteStudent={this.deleteStudent}
            restoreDeleteStudent={this.restoreDeleteStudent}
            deleteStudentOfSelectStudent={this.deleteStudentOfSelectStudent}
            current={current}
            total={total}
            searchStudents={this.searchStudents}
            deleteStudents={deleteStudents}
          />
        )}
        {currentIndex === 1 && (
          <SelectQualificationType
            loading={loading}
            route={this.props.route}
            branchDict={branchDict}
            campusDict={campusDict}
            productTypeDict={productTypeDict}
            yearDict={yearDict}
            quarterDict={quarterDict}
            gradeDict={gradeDict}
            subjetDict={subjetDict}
            classTypeDict={classTypeDict}
            classLevelDict={classLevelDict}
            branchId={branchId}
            campusId={campusId}
            productTypeId={productTypeId}
            yearId={yearId}
            quarterId={quarterId}
            gradeId={gradeId}
            subjectId={subjectId}
            classTypeId={classTypeId}
            classLevelId={classLevelId}
            handleClassTypeChange={this.handleClassTypeChange}
            handleBranchChange={this.handleBranchChange}
            handleSelected={this.handleSelected}
            handleProductTypeChange={this.handleProductTypeChange}
            classIds={classIds}
            students={students}
          />
        )}
        {isSelectClass && (
          <SelectClassModalContainer
            branchDict={branchDict}
            campusDict={campusDict}
            productTypeDict={productTypeDict}
            yearDict={yearDict}
            quarterDict={quarterDict}
            gradeDict={gradeDict}
            subjetDict={subjetDict}
            classTypeDict={classTypeDict}
            classLevelDict={classLevelDict}
            branchId={branchId}
            campusId={campusId}
            productTypeId={productTypeId}
            yearId={yearId}
            quarterId={quarterId}
            gradeId={gradeId}
            subjectId={subjectId}
            classTypeId={classTypeId}
            classLevelId={classLevelId}
            handleClassTypeChange={this.handleClassTypeChange}
            closeSelectClass={this.closeSelectClass}
            saveStudents={this.saveStudents}
            saveClassIds={this.saveClassIds}
            mainTeacher={mainTeacher}
            assitTeacher={assitTeacher}
            classIds={classIds}
          />
        )}
        {isSelectStudent && (
          <SelectStudentModal
            campusId={campusId}
            closeSelectStudent={this.closeSelectStudent}
            saveNewStudentsId={this.saveNewStudentsId}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    params: state.main.pageParams['/QualificationManager/addNewqualification'],
  };
};

export default connect(mapStateToProps)(SelectQualificationTypeContainer);
