import React, { Component } from 'react';
import Modal from 'COMPONENT/Modal';
import { connect } from 'dva';
import { Row, Col } from 'antd';
import FormItem from 'COMPONENT/FormItem';
import Select from 'COMPONENT/Select';
import Popover from 'COMPONENT/Popover';
import { loadCommonDict, loadBranchDict, loadCampusDict } from 'UTIL/dict';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import showAlert from 'COMPONENT/utils/globalTips/showAlert';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import { getCommonErrorTips } from 'COMPONENT/utils/form';
import { debounce } from 'lodash';
import Loaders from '../loader';
import styles from '../styles.less';
const classTypeTempDict = [{ text: '请先选择产品类型', val: '', disabled: true }];
const studentTempDict = [{ text: '请先选择客户号码', val: '', disabled: true }];
const campusTempDict = [{ text: '请先选择分公司', val: '', disabled: true }];
class EditQuliModal extends Component {
  constructor(props) {
    super(props);
    console.log('this.props.editRowData');
    console.log(this.props.editRowData);
    let { branchId, campusId, productTypeId, years, quarter, gradeId, subjectId, classTypeId, classLevel } =
      this.props.editRowData;
    this.state = {
      loading: true,
      //表单数据
      checking: false,
      customerPhoneId: null,
      studentId: null,
      branchId,
      campusId,
      productTypeId,
      years,
      quarter,
      gradeId,
      subjectId,
      classTypeId,
      entryTypeName: '免考',
      //数据字典
      customerPhoneDict: null,
      studentDict: null,
      branchDict: null,
      campusDict: null,
      productTypeDict: [
        { val: 'TWO_TEACHER', text: '双师' },
        { val: 'SMALL_CLASS', text: '小班' },
      ],
      yearDict: null,
      quarterDict: null,
      gradeDict: null,
      subjectDict: null,
      classTypeDict: null,
      //文件
      fileList: [],
      classLevel,
      classLevelDict: null,
    };
    this.checking = false; //是否显示校验提示
    this.errors = {}; //校验错误信息
    this.dictVersion = 1; //客户号码Dict版本
  }

  componentDidMount() {
    this.initContent();
  }
  /**
   * 初始化数据
   */
  initContent = async () => {
    const { editQuliId, editMode, editRowData } = this.props;
    let { classTypeId, classLevel, productTypeId, tagTypeName } = editRowData;
    const asyncCalls = [
      loadBranchDict(),
      loadCommonDict('PRODUCT_VERSION'),
      loadCommonDict('PRODUCT_QUARTER'),
      loadCommonDict('STUDENT_GRADE'),
      loadCommonDict('SUBJECT'),
    ];
    if (editQuliId) {
      asyncCalls.push(Loaders.loadDetail({ id: editQuliId }));
    }
    Promise.all(asyncCalls).then(async ([branchDict, yearDict, quarterDict, gradeDict, subjectDict, detailData]) => {
      if (!detailData) {
        this.updateState({
          loading: false,
          branchDict,
          yearDict,
          quarterDict,
          gradeDict,
          subjectDict,
        });
      } else {
        if (editMode === 'add') {
          const [campusDict, classTypeDict, classLevelDict] = await Promise.all([
            loadCampusDict(this.props.editRowData.branchId),
            loadCommonDict('CLASS_TYPE', productTypeId),
            loadCommonDict('CLASS_LEVEL', classTypeId),
          ]);
          this.updateState({
            loading: false,
            studentId: detailData.data.studentId,
            campusDict,
            classTypeDict,
            classLevelDict,
            entryTypeName: tagTypeName,
            branchDict,
            yearDict,
            quarterDict,
            gradeDict,
            subjectDict,
          });
        } else {
          loadCommonDict('CLASS_TYPE', detailData.data.productTypeId).then((classTypeDict) => {
            this.setState({
              classTypeDict,
            });
          });
          loadCampusDict(detailData.branchId).then((campusDict) => {
            this.updateState({
              loading: false,
              branchDict,
              yearDict,
              quarterDict,
              gradeDict,
              subjectDict,
              campusDict,
              ...detailData.data,
              entryTypeName: tagTypeName,
            });
          });
        }
      }
    });
    let classLevelDict = await loadCommonDict('CLASS_LEVEL', classTypeId);
    this.setState({ classLevel, classLevelDict });
  };
  /**
   * 输入用户号码搜索关键词
   */
  onSearchKeyChange = (key) => {
    debounce(this.search, 300)(key);
  };

  /**
   * 搜索
   */
  search = async (key) => {
    if (key) {
      const dictVersion = ++this.dictVersion;
      await this.setState({ loading: true });
      const result = await Loaders.loadContact({ term: key, monitoringUserId: this.props.monitoringUserId });
      if (this.dictVersion === dictVersion) {
        this.setState({ customerPhoneDict: result, loading: false });
      }
    } else {
      this.dictVersion++;
      this.setState({ customerPhoneDict: [], loading: false });
    }
  };
  /**
   * 保存
   */
  saveDetail = async (params) => {
    const { editQuliId, editMode } = this.props;
    if (editQuliId) {
      if (editMode === 'add') {
        return await Loaders.add(params);
      }
      return await Loaders.edit(params);
    } else {
      return await Loaders.add(params);
    }
  };
  /**
   * 更新状态
   * @param  {object} changedState 改变的状态
   * @return {Promise}
   */
  updateState = (changedState) => {
    const newState = {
      ...this.state,
      ...changedState,
    };
    return new Promise((resolve) => {
      this.setState(newState, () => {
        resolve();
      });
    });
  };

  /**
   * 班级类型改变，下载班级层次字典
   */
  loadClassLevelDict = async (classTypeId) => {
    if (classTypeId === '') {
      this.setState({ classLevelDict: null, classLevel: null });
    } else {
      let classLevelDict = await loadCommonDict('CLASS_LEVEL', classTypeId);
      this.setState({ classLevelDict });
    }
  };

  /**
   * 改变客户号码时获取对应的学生字典
   */
  handlePhoneChange = async (customerPhoneId) => {
    await this.updateState({ customerPhoneId, studentId: null });
    if (customerPhoneId) {
      await this.setState({ loading: true });
      const studentDict = await Loaders.loadStudentDict({ id: customerPhoneId });
      this.updateState({
        studentDict,
        loading: false,
      });
    }
  };
  /**
   * 分公司改变获取对应的学习中心字典
   */
  handleBranchChange = async (branchId) => {
    await this.updateState({ branchId, campusId: null });
    // if (branchId) {
    //   await this.setState({loading: true})
    //   loadCampusDict(branchId)
    //     .then((campusDict) => {
    //       this.updateState({
    //         campusDict,
    //         loading: false
    //       })
    //     })
    // }
  };
  /**
   * 确认按钮
   */
  handleOk = async () => {
    const {
      checking,
      studentId,
      branchId,
      productTypeId,
      years,
      quarter,
      gradeId,
      subjectId,
      classTypeId,
      classLevel,
    } = this.state;
    if (!checking) {
      this.setState({ checking: true });
    }
    let errorTips = getCommonErrorTips(this.errors);
    if (errorTips) {
      this.modal.showOkTips(errorTips);
      return;
    }
    const { editQuliId } = this.props;
    this.setState({ loading: true });
    this.saveDetail({
      id: editQuliId ? editQuliId : null,
      studentId,
      branchId,
      productTypeId,
      years,
      quarter,
      gradeId,
      subjectId,
      classTypeId,
      classLevel: classLevel,
    }).then(
      () => {
        this.setState({ loading: false });
        //保存后的回掉，关闭模态框,更新列表等
        this.props.onOk();
        showMessage('保存成功', 'success');
      },
      (e) => {
        this.setState({ loading: false });
        if (e.text) {
          this.modal.showOkTips(e.text);
        } else {
          showMessage('保存失败');
        }
      },
    );
  };
  /**
   * 保存校验信息
   */
  errorChange = (key, error) => {
    this.errors[key] = error;
  };
  /**
   * 取消按钮
   */
  handleCancel = () => {
    this.props.onCancel();
  };
  /**
   * 获取模态框标题
   */
  getTitle = () => {
    const { editQuliId, editMode, editStudentName } = this.props;
    if (!editQuliId) {
      return '新增报读资格';
    } else {
      if (editMode === 'add') {
        return '新增报读资格 (' + (editStudentName || '') + ' ' + editQuliId + ')';
      } else {
        return '编辑报读资格 (' + (editStudentName || '') + ' ' + editQuliId + ')';
      }
    }
  };

  render() {
    const {
      checking,
      loading,
      customerPhoneId,
      studentId,
      branchId,
      campusId,
      productTypeId,
      years,
      quarter,
      gradeId,
      subjectId,
      classTypeId,
      customerPhoneDict,
      studentDict,
      branchDict,
      campusDict,
      productTypeDict,
      yearDict,
      quarterDict,
      gradeDict,
      subjectDict,
      classTypeDict,
      entryTypeName,
      classLevel,
      classLevelDict,
    } = this.state;
    const { editQuliId, editStudentName } = this.props;
    return (
      <Modal
        title={this.getTitle()}
        loading={loading}
        onOk={this.handleOk}
        onCancel={this.handleCancel}
        ref={(c) => (this.modal = getWrappedInstance(c))}
        width="900"
      >
        <div>
          {!editQuliId && (
            <Row>
              <Col span={12}>
                <FormItem name="客户号码" labelWidth={90} required>
                  <Select
                    value={customerPhoneId}
                    dict={customerPhoneDict}
                    onChange={this.handlePhoneChange}
                    onSearchKeyChange={this.onSearchKeyChange}
                    showSearch={true}
                    validator={{
                      silent: !checking,
                      onErrorChange: (e) => this.errorChange('customerPhoneId', e),
                    }}
                  />
                </FormItem>
              </Col>
              <Col span={12}>
                <FormItem name="学生姓名" labelWidth={90} required>
                  <Select
                    value={studentId}
                    dict={customerPhoneId ? studentDict : studentTempDict}
                    onChange={(studentId) => {
                      this.updateState({ studentId });
                    }}
                    showSearch={true}
                    validator={{
                      silent: !checking,
                      onErrorChange: (e) => this.errorChange('studentId', e),
                    }}
                  />
                </FormItem>
              </Col>
            </Row>
          )}
          <Row>
            <Col span={12}>
              <FormItem name="分公司" labelWidth={90} required>
                <Select
                  value={branchId}
                  dict={branchDict}
                  onChange={this.handleBranchChange}
                  showSearch={true}
                  validator={{
                    silent: !checking,
                    onErrorChange: (e) => this.errorChange('branchId', e),
                  }}
                />
              </FormItem>
            </Col>
            {/*
            <Col span={12}>
              <FormItem name="学习中心" labelWidth={90}>
                <Select
                  value={campusId}
                  dict={branchId ? campusDict : campusTempDict}
                  showSearch={true}
                  onChange={(campusId)=>{this.updateState({campusId})}}
                />
              </FormItem>
            </Col>
*/}
          </Row>
          <Row>
            <Col span={12}>
              <FormItem name="产品类型" labelWidth={90} required>
                <Select
                  value={productTypeId}
                  dict={productTypeDict}
                  showSearch={true}
                  onChange={async (productTypeId) => {
                    this.setState({ productTypeId, classTypeId: null, classLevel: null });
                    if (productTypeId) {
                      const classTypeDict = await loadCommonDict('CLASS_TYPE', productTypeId);
                      this.setState({
                        classTypeDict,
                      });
                    }
                  }}
                  validator={{
                    silent: !checking,
                    onErrorChange: (e) => this.errorChange('productTypeId', e),
                  }}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="年份" labelWidth={90} required>
                <Select
                  value={years}
                  dict={yearDict}
                  showSearch={true}
                  onChange={(years) => {
                    this.updateState({ years });
                  }}
                  validator={{
                    silent: !checking,
                    onErrorChange: (e) => this.errorChange('years', e),
                  }}
                />
              </FormItem>
            </Col>
          </Row>
          <Row>
            <Col span={12}>
              <FormItem name="季度" labelWidth={90} required>
                <Select
                  value={quarter}
                  dict={quarterDict}
                  showSearch={true}
                  onChange={(quarter) => {
                    this.updateState({ quarter });
                  }}
                  validator={{
                    silent: !checking,
                    onErrorChange: (e) => this.errorChange('quarter', e),
                  }}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="年级" labelWidth={90} required>
                <Select
                  value={gradeId}
                  dict={gradeDict}
                  showSearch={true}
                  onChange={(gradeId) => {
                    this.updateState({ gradeId });
                  }}
                  validator={{
                    silent: !checking,
                    onErrorChange: (e) => this.errorChange('gradeId', e),
                  }}
                />
              </FormItem>
            </Col>
          </Row>
          <Row>
            <Col span={12}>
              <FormItem name="科目" labelWidth={90} required>
                <Select
                  value={subjectId}
                  dict={subjectDict}
                  showSearch={true}
                  onChange={(subjectId) => {
                    this.updateState({ subjectId });
                  }}
                  validator={{
                    silent: !checking,
                    onErrorChange: (e) => this.errorChange('subjectId', e),
                  }}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="班级类型" labelWidth={90} required>
                <Select
                  value={classTypeId}
                  dict={productTypeId ? classTypeDict : classTypeTempDict}
                  showSearch={true}
                  onChange={(classTypeId) => {
                    this.updateState({ classTypeId });
                    this.loadClassLevelDict(classTypeId);
                  }}
                  validator={{
                    silent: !checking,
                    onErrorChange: (e) => this.errorChange('classTypeId', e),
                  }}
                />
              </FormItem>
            </Col>
          </Row>
          <Row>
            <Col span={12}>
              <FormItem name="班级层次" labelWidth={90} required>
                <Select
                  value={classLevel}
                  dict={classLevelDict}
                  showSearch={true}
                  onChange={(classLevel) => {
                    this.updateState({ classLevel });
                  }}
                  validator={{
                    silent: !checking,
                    onErrorChange: (e) => this.errorChange('classLevel', e),
                  }}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="资格类型" labelWidth={90}>
                <span>{entryTypeName}</span>
              </FormItem>
            </Col>
          </Row>
        </div>
      </Modal>
    );
  }
}
const mapStateToProps = (state) => {
  return {
    monitoringUserId: state.account.user.userId,
  };
};
export default connect(mapStateToProps)(EditQuliModal);
