import React from 'react';
import { connect } from 'dva';
import Button from 'COMPONENT/Button';
import { Icon as AntdIcon } from 'antd';
import Page from 'COMPONENT/Page';
import List from 'COMPONENT/List';
import { Tooltip } from 'antd';
import SearchBar from 'COMPONENT/SearchBar';
import Input from 'COMPONENT/Input';
import Select from 'COMPONENT/Select';
import EditQuliModal from './editQuli';
import EditSightUpLimitModal from './editSightUpLimit';
import ShowMessage from 'COMPONENT/utils/globalTips/showMessage';
import { loadCommonDict, loadBranchDict, loadCampusDict } from 'UTIL/dict';
import { loginUser } from 'UTIL/common';
import Loaders from './loader';
import styles from './styles.less';
import showAlert from 'COMPONENT/utils/globalTips/showAlert';
import { SYSTEM_MARK } from 'CONST/config';

const isXINGHUO = SYSTEM_MARK === 'XINGHUO';
const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;
class QualificationManager extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: true, // boolean 页面加载状态
      //查询条件
      studentNameOrContract: null,
      studentId: null,
      branchId: null,
      campusId: null,
      examTypeId: null,
      productTypeId: isXINGHUO ? 'SMALL_CLASS' : 'TWO_TEACHER',
      years: null,
      quarter: null,
      subjectId: null,
      gradeId: null,
      classTypeId: null,
      tagType: null,
      entryType: null,
      classLevelId: null,
      //列表数据
      listLoading: false,
      rows: [],
      currentPage: 1,
      pageSize: 20,
      totalRecords: null,
      //数据字典数据
      branchDict: null,
      productTypeDict: [
        { val: 'TWO_TEACHER', text: '双师' },
        { val: 'SMALL_CLASS', text: '小班' },
      ],
      yearDict: null,
      quarterDict: null,
      gradeDict: null,
      subjetDict: null,
      classTypeDict: null,
      tagTypeDict: null,
      entryTypeDict: null,
      classLevelDict: null,
      //其他
      editQuliShow: false,
      addQuliShow: false,
      editSightUpLimitShow: false,
      editQuliId: null,
      editStudentName: null,
      paperScoreImgs: [],
      selectedRowKeys: [],
      selectedRows: null,
      editRowData: null,
    };
  }
  componentDidMount() {
    this.initContent();
  }
  /**
   * 更新状态
   * @param  {object} changedState 改变的状态
   * @return {Promise}
   */
  updateState = (changedState) => {
    const newState = {
      ...this.state,
      ...changedState,
    };
    return new Promise((resolve) => {
      this.setState(newState, () => {
        resolve();
      });
    });
  };
  /**
   * 获取列表数据
   */
  loadListData = async (params) => {
    const {
      studentNameOrContract,
      clientContact,
      studentId,
      branchId,
      campusId,
      productTypeId,
      years,
      quarter,
      subjectId,
      gradeId,
      pageSize,
      currentPage,
      classTypeId,
      tagType,
      entryType,
      classLevelId,
    } = this.state;
    const apiParams = {
      studentNameOrContract,
      clientContact,
      studentId,
      branchId,
      campusId,
      productTypeId,
      years,
      quarter,
      subjectId,
      gradeId,
      currentPage,
      classTypeId,
      tagType,
      entryType,
      pageSize,
      pageNo: currentPage,
      classLevel: classLevelId,
      ...params,
    };
    try {
      return await Loaders.loadList(apiParams);
    } catch (e) {
      ShowMessage(e.text);
      return;
    }
  };
  /**
   * @description: 获取系统字符串类型
   * @param {type}
   * @return:
   */
  getSystemTypeString() {
    let type = 'TWO_TEACHER';
    if (loginUser.isXingHuoSym()) {
      type = 'SMALL_CLASS';
    } else if (loginUser.isTwoTeacherSym()) {
      type = 'TWO_TEACHER';
    }
    return type;
  }
  /**
   * 初始化内容
   */
  initContent = async () => {
    this.setState({ loading: true });
    // 获取系统类型
    let type = this.getSystemTypeString();
    const [branchDict, yearDict, quarterDict, gradeDict, subjetDict, tagTypeDict, entryTypeDict, classTypeDict] =
      await Promise.all([
        loadBranchDict(),
        loadCommonDict('PRODUCT_VERSION'),
        loadCommonDict('PRODUCT_QUARTER'),
        loadCommonDict('STUDENT_GRADE'),
        loadCommonDict('SUBJECT'),
        loadCommonDict('QUALIFICATION_TAG_TYPE'),
        loadCommonDict('ENTRY_TYPE'),
        loadCommonDict('CLASS_TYPE', type),
      ]);
    let currentYear = new Date().toString().slice(11, 15);
    let years = yearDict.filter((year) => {
      return year.text === currentYear;
    })[0].val;
    this.updateState({
      loading: false,
      branchDict,
      yearDict,
      quarterDict,
      gradeDict,
      subjetDict,
      tagTypeDict,
      entryTypeDict,
      branchId: branchDict[0].val,
      classTypeDict,
      years,
    });
    const [campusDict, list] = await Promise.all([loadCampusDict(branchDict[0].val), this.loadListData()]);
    this.setState({ campusDict, rows: list.datas, totalRecords: list.rowCount, currentPage: list.pageNo });
  };

  /**
   * 打开新增报读资格
   */
  openAddNewQualification = () => {
    const { dispatch } = this.props;
    const {} = this.state;
    dispatch({
      type: 'main/openPage',
      payload: {
        path: '/QualificationManager/addNewqualification',
        params: {},
        refresh: true,
      },
    });
  };

  // 设置报读资格限制
  editQualification = () => {
    this.setState({
      editSightUpLimitShow: true,
    });
  };
  /**
   * 班级类型改变，下载班级层次字典
   */
  loadClassLevelDict = async (classTypeId) => {
    if (!classTypeId) {
      this.setState({ classLevelDict: null, classLevelId: null });
    } else {
      let classLevelDict = await loadCommonDict('CLASS_LEVEL', classTypeId);
      this.setState({ classLevelDict });
    }
  };

  /**
   * 查询列表
   */
  search = async () => {
    await this.updateState({ listLoading: true, currentPage: 1 });
    const list = await this.loadListData();
    this.updateState({
      listLoading: false,
      rows: list.datas,
      totalRecords: list.rowCount,
      currentPage: list.pageNo,
    });
  };
  /**
   * 删除资质
   */
  deleteQualification = async (id) => {
    const { selectedRows } = this.state;
    if (!id && !selectedRows) {
      showAlert('请选择学生。');
      return;
    }
    this.setState({ loading: true });
    if (id) {
      (await Loaders.deleteQualification(id)) === 0 ? ShowMessage('删除成功', 'success') : null;
    } else {
      let ids = selectedRows
        .map((row) => {
          return row.id;
        })
        .reduce((preValue, curValue, index) => {
          return preValue + ',' + curValue;
        }, '')
        .slice(1);
      (await Loaders.deleteQualification(ids)) === 0 ? ShowMessage('删除成功', 'success') : null;
    }
    this.search();
    this.setState({ loading: false, selectedRowKeys: [], selectedRows: [] });
  };
  /**
   * 重置搜索表单
   */
  reset = async () => {
    const { branchDict, productTypeDict, yearDict } = this.state;
    let currentYear = new Date().toString().slice(11, 15);
    let years = yearDict.filter((year) => {
      return year.text === currentYear;
    })[0].val;
    this.updateState({
      studentNameOrContract: null,
      clientContact: null,
      studentId: null,
      branchId: branchDict[0].val,
      campusId: null,
      examTypeId: null,
      productTypeId: isXINGHUO ? 'SMALL_CLASS' : 'TWO_TEACHER',
      years,
      quarter: null,
      subjectId: null,
      gradeId: null,
      classLevelId: null,
      classTypeId: null,
      tagType: null,
      entryType: null,
      selectedRowKeys: [],
    });
  };
  /**
   * 翻页
   */
  pageChange = async ({ current, pageSize }) => {
    await this.updateState({ listLoading: true, pageSize, currentPage: current });
    const list = await this.loadListData();
    this.updateState({
      listLoading: false,
      rows: list.datas,
      totalRecords: list.rowCount,
      currentPage: list.pageNo,
    });
  };
  /**
   * 分公司改变获取对应的学习中心字典
   */
  handleBranchChange = async (branchId) => {
    await this.updateState({ branchId, campusId: null });
  };
  /**
   * 关闭模态框的回调
   */
  handleModalOk = (type) => {
    if (type === 'edit') {
      this.updateState({
        editQuliShow: false,
        editQuliId: null,
        editStudentName: null,
      });
    } else {
      this.updateState({
        editQuliId: null,
        editStudentName: null,
        addQuliShow: false,
      });
    }
    this.search();
  };
  /**
   * 判断是否有新增资格权限
   */
  hasAddAuth = () => {
    const { btnAuths } = this.props;
    return btnAuths.includes('BTN_ADDQULI_EXPORT');
  };
  /**
   * 判断是否有修改资格权限
   */
  hasEditAuth = () => {
    const { btnAuths } = this.props;
    return btnAuths.includes('BTN_EDITQULI_EXPORT');
  };
  /**
   * 判断是否有删除资格权限
   */
  hasDeleteAuth = () => {
    const { btnAuths } = this.props;
    return btnAuths.includes('BTN_DELETEQULI_EXPORT');
  };
  /**
   * 判断是否有设置报读资格限制权限
   */
  hasLimitAuth = () => {
    const { btnAuths } = this.props;
    return btnAuths.includes('BTN_SIGN_UP_LIMIT');
  };
  /**
   * 渲染录入方式项
   */
  renderEntryType = (data, rowData, index) => {
    const tips = rowData.tips;
    let HoverIcon, tipsContent;
    //如果有匹配标签
    if (tips) {
      tipsContent = tips.map((tip, index) => {
        return (
          <div key={index}>
            <p>{tip.time}</p>
            <p>{tip.message}</p>
          </div>
        );
      });
      HoverIcon = (
        <Tooltip placement="top" title={<div>{tipsContent}</div>}>
          <AntdIcon type="info-circle" className={styles.infoIcon} />
        </Tooltip>
      );
    }
    return (
      <div>
        {tips && HoverIcon}
        {data}
      </div>
    );
  };
  /**
   * 表头
   */
  columns = [
    { title: '学生姓名', dataIndex: 'studentName', width: COL_WIDTH_M },
    { title: '学生编号', dataIndex: 'studentId', width: COL_WIDTH_L },
    { title: '分公司', dataIndex: 'branchName', width: COL_WIDTH_M },
    { title: '产品类型', dataIndex: 'productTypeName', width: COL_WIDTH_M },
    { title: '年份', dataIndex: 'yearsName', width: COL_WIDTH_M },
    { title: '季度', dataIndex: 'quarterName', width: COL_WIDTH_M },
    { title: '年级', dataIndex: 'gradeName', width: COL_WIDTH_M },
    { title: '科目', dataIndex: 'subjectName', width: COL_WIDTH_M },
    { title: '班级类型', dataIndex: 'classTypeName', width: COL_WIDTH_M },
    { title: '班级层次', dataIndex: 'classLevelName', width: COL_WIDTH_M },
    { title: '资格类型', dataIndex: 'tagTypeName', width: COL_WIDTH_M },
    { title: '录入方式', dataIndex: 'entryTypeName', width: COL_WIDTH_M, render: this.renderEntryType },
    {
      title: '操作',
      key: 'action',
      width: COL_WIDTH_L,
      fixed: 'right',
      getActionConfig: (text, record, index) => {
        const { id, studentNameOrContract } = record;
        const ret = [];
        const ret1 = [
          {
            menus: ret,
            menuText: '更多',
          },
        ];
        if (this.hasAddAuth()) {
          ret1.push({
            text: '新增',
            onClick: () =>
              this.updateState({
                addQuliShow: true,
                editQuliId: id,
                editStudentName: studentNameOrContract,
                editRowData: record,
              }),
          });
        }
        if (this.hasEditAuth()) {
          ret.push({
            text: '编辑',
            onClick: () =>
              this.updateState({
                editQuliShow: true,
                editQuliId: id,
                editStudentName: studentNameOrContract,
                editRowData: record,
              }),
          });
        }
        if (this.hasDeleteAuth()) {
          ret.push({ text: '删除', onClick: () => this.deleteQualification(id) });
        }
        return ret1;
      },
    },
  ];
  render() {
    const {
      loading,
      searchExpand,
      listLoading,
      rows,
      currentPage,
      pageSize,
      totalRecords,
      studentNameOrContract,
      clientContact,
      studentId,
      branchId,
      campusId,
      productTypeId,
      years,
      quarter,
      gradeId,
      subjectId,
      classTypeId,
      tagType,
      entryType,
      entryTypeDict,
      classLevelId,
      branchDict,
      productTypeDict,
      yearDict,
      quarterDict,
      gradeDict,
      subjetDict,
      campusDict,
      tagTypeDict,
      classTypeDict,
      quliTypeDict,
      classLevelDict,
      addQuliShow,
      editQuliShow,
      editQuliId,
      editStudentName,
      selectedRowKeys,
      editRowData,
      editSightUpLimitShow,
    } = this.state;

    const editSightUpLimitModalDictMap = {
      branchDict,
      tagTypeDict,
    };
    const isLimitAuth = this.hasLimitAuth();
    return (
      <Page
        name="报读资格管理"
        fitWindowHeight={true}
        loading={loading}
        route={this.props.route}
        subHeader={
          <div className={styles.searchBar} style={{ height: searchExpand ? 'auto' : '100px' }}>
            <SearchBar
              onSearch={this.search}
              onReset={this.reset}
              expand={searchExpand}
              onToggleExpand={(expand) => this.setState({ searchExpand: expand })}
            >
              <Input
                value={studentNameOrContract}
                placeholder="学生姓名/客户号码"
                onChange={(e) => this.setState({ studentNameOrContract: e.target.value })}
              />
              {/* <Input value={clientContact} placeholder="客户号码" onChange={e => this.setState({clientContact: e.target.value})} />
                <Input value={studentId} placeholder="学生ID" onChange={e => this.setState({studentId: e.target.value})} /> */}
              <Select
                placeholder="分公司"
                dict={branchDict}
                value={branchId}
                allowClear={true}
                showSearch={true}
                onChange={this.handleBranchChange}
              />
              <Select
                placeholder="产品类型"
                dict={productTypeDict}
                value={productTypeId}
                allowClear={true}
                onChange={async (productTypeId) => {
                  this.setState({
                    productTypeId,
                    classTypeId: null,
                    classLevelId: null,
                    classTypeDict: null,
                    classLevelDict: null,
                  });
                  if (productTypeId) {
                    const classTypeDict = await loadCommonDict('CLASS_TYPE', productTypeId);
                    //得分等级还有一个无效项
                    classTypeDict.push({ text: '无效', val: 'invalid' });
                    //还有一个未激活
                    classTypeDict.push({ text: '未激活', val: 'inactive' });
                    this.setState({
                      classTypeDict,
                    });
                  }
                }}
              />
              <Select
                placeholder="年份"
                dict={yearDict}
                value={years}
                allowClear={true}
                onChange={(years) => this.updateState({ years })}
              />
              <Select
                placeholder="季度"
                dict={quarterDict}
                value={quarter}
                allowClear={true}
                onChange={(quarter) => this.updateState({ quarter })}
              />
              <Select
                placeholder="年级"
                dict={gradeDict}
                value={gradeId}
                allowClear={true}
                onChange={(gradeId) => this.updateState({ gradeId })}
              />
              <Select
                placeholder="科目"
                dict={subjetDict}
                value={subjectId}
                allowClear={true}
                onChange={(subjectId) => this.updateState({ subjectId })}
              />
              <Select
                placeholder="班级类型"
                dict={classTypeDict}
                value={classTypeId}
                allowClear={true}
                onChange={(classTypeId) => {
                  this.updateState({ classTypeId, classLevelId: null, classLevelDict: null });
                  this.loadClassLevelDict(classTypeId);
                }}
              />
              <Select
                placeholder="班级分层"
                dict={classLevelDict}
                value={classLevelId}
                allowClear={true}
                onChange={(classLevelId) => {
                  this.setState({ classLevelId });
                }}
              />
              <Select
                placeholder="资格类型"
                dict={tagTypeDict}
                value={tagType}
                allowClear={true}
                onChange={(tagType) => this.updateState({ tagType })}
              />
              <Select
                placeholder="录入方式"
                dict={entryTypeDict}
                value={entryType}
                allowClear={true}
                onChange={(entryType) => this.updateState({ entryType })}
              />
            </SearchBar>
            <div className={styles.btnWrapper}>
              <Button onClick={() => this.openAddNewQualification()}>新增报读资格</Button>
              {isXINGHUO && isLimitAuth && <Button onClick={() => this.editQualification()}>设置报读资格限制</Button>}
              <Button onClick={() => this.deleteQualification()} disabled={selectedRowKeys.length === 0}>
                删除
              </Button>
            </div>
          </div>
        }
      >
        <List
          rowSelection={{
            selectedRowKeys,
            onChange: (selectedRowKeys, selectedRows) => {
              this.setState({ selectedRowKeys, selectedRows });
            },
          }}
          columns={this.columns}
          loading={listLoading}
          dataSource={rows}
          pagination={{
            current: currentPage,
            pageSize,
            total: totalRecords,
          }}
          onChange={this.pageChange}
        />
        {addQuliShow && (
          <EditQuliModal
            editRowData={editRowData}
            editQuliId={editQuliId}
            editStudentName={editStudentName}
            editMode="add"
            onOk={() => this.handleModalOk('add')}
            onCancel={() => {
              this.updateState({ addQuliShow: false, editQuliId: null, editStudentName: null });
            }}
          />
        )}
        {editQuliShow && (
          <EditQuliModal
            editRowData={editRowData}
            editQuliId={editQuliId}
            editStudentName={editStudentName}
            onOk={() => this.handleModalOk('edit')}
            onCancel={() => {
              this.updateState({ editQuliShow: false, editQuliId: null, editStudentName: null });
            }}
          />
        )}
        {editSightUpLimitShow && (
          <EditSightUpLimitModal
            onOk={() => this.updateState({ editSightUpLimitShow: false })}
            dict={editSightUpLimitModalDictMap}
            onCancel={() => this.updateState({ editSightUpLimitShow: false })}
          />
        )}
      </Page>
    );
  }
}
const mapStateToProp = (state) => {
  return {
    btnAuths: state.account.btnAuths,
  };
};
export default connect(mapStateToProp)(QualificationManager);
