import request from 'UTIL/http/fetch';
import {
  loadTagListApi,
  loadTagDetailApi,
  addTagApi,
  editTagApi,
  loadContactApi,
  loadStudentApi,
  delEnrollManagerStudentQualification,
  getClassStudentQualificationInfo,
  getClassStudentQualification,
  getSingleStudentQualificationInfo,
  addBatchStudentQualification,
  getQualificationLimitDetail,
  updateQualificationLimit,
} from 'SERVICE/enrollManager';
/**
 * 将json形式的参数拼接成以&连接的url参数字符串
 * @param  {object} params
 * @return {string}
 */
const makeQueryString = (params) => {
  let queryString = '';
  if (params) {
    Object.keys(params).forEach((key, i) => {
      const value = params[key];
      if (i > 0) {
        queryString += '&';
      }
      queryString += `${key}=${value === null || value == undefined ? '' : value}`;
    });
  }
  return queryString;
};
/**
 * 获取列表数据
 */
const loadList = async (params) => {
  // const data = await request('/EnrollManagerStudentQualificationController/listEnrollManagerStudentQualificationInfos.do?'+makeQueryString(params))
  // return data.data
  return loadTagListApi(params);
};
/**
 * 获取标签详情
 */
const loadDetail = async (params) => {
  return loadTagDetailApi(params);
};
/**
 * 新增标签
 */
const add = async (params) => {
  return addTagApi(params);
};
/**
 * 编辑标签
 */
const edit = async (params) => {
  return editTagApi(params);
};
/**
 * 获取客户号码
 */
const loadContact = async (params) => {
  const response = await loadContactApi(params);
  if (response) {
    if (response.data) {
      return response.data;
    } else {
      return [];
    }
  } else {
    return [];
  }
};
/**
 * 获取学生数据字典
 */
const loadStudentDict = async (params) => {
  const response = await loadStudentApi(params);
  if (response) {
    return response.studentVos.map((dict) => {
      return {
        text: dict.name,
        val: dict.id,
      };
    });
  } else {
    return [];
  }
};
/**
 * 删除报读资质
 */
const deleteQualification = async (id) => {
  let response = await delEnrollManagerStudentQualification({ id });
  return response.resultCode;
};

/**
 *  获取班级中的学生信息
 */
const getClassStudentQualificationInfoApi = async (params) => {
  let response = await getClassStudentQualificationInfo(params);
  return {
    studentDatas: response.datas,
    current: response.pageNo,
    total: response.rowCount,
  };
};

/**
 * 获取班级列表
 */

const getClassStudentQualificationApi = async (params) => {
  let response = await getClassStudentQualification(params);
  return response.data;
};
/**
 * 获取单个学生信息
 */
const getSingleStudentQualificationInfoApi = async (params) => {
  let response = await getSingleStudentQualificationInfo(params);
  return response.data;
};
/**
 * 批量生成资质
 */
const addBatchStudentQualificationApi = async (params) => {
  let response = await addBatchStudentQualification(params);
  return response;
};

/**
 * 获取资质限制详情
 */
const getQualificationLimitData = async (params) => {
  let response = await getQualificationLimitDetail(params);
  return response.data;
};
/**
 * 更新资质限制详情
 */
const updateQualificationLimitData = async (params) => {
  let response = await updateQualificationLimit(params);
  return response.data;
};

export default {
  loadList,
  loadDetail,
  add,
  edit,
  loadContact,
  loadStudentDict,
  deleteQualification,
  getClassStudentQualificationApi,
  getClassStudentQualificationInfoApi,
  getSingleStudentQualificationInfoApi,
  addBatchStudentQualificationApi,
  getQualificationLimitData,
  updateQualificationLimitData,
};
