import React from 'react';
import Modal from 'COMPONENT/Modal';
import SearchBar from 'COMPONENT/SearchBar';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import Input from 'COMPONENT/Input';
import List from 'COMPONENT/List';
import { loadCommonDict, loadCampusDict } from 'UTIL/dict';
import loaders from '../loader';
import compStyle from './compStyle.less';
import { Row, Col } from 'antd';
import { SYSTEM_MARK } from 'CONST/config';

class SelectClassModal extends React.Component {
  render() {
    const {
      dataSource,
      closeSelectClass,
      errorChange,
      onSearch,
      handleBranchChange,
      phaseDict,
      reset,
      selectedRowKeys,
      checkedClass,
      searchStudents,
      mainTeacher,
      assitTeacher,
      currentClassId,
    } = this.props;
    const {
      branchId,
      campusId,
      productTypeId,
      yearId,
      quarterId,
      phaseId,
      gradeId,
      subjectId,
      classTypeId,
      classLevelId,
      checking,
      errors,
    } = this.props;
    const {
      loading,
      branchDict,
      campusDict,
      productTypeDict,
      yearDict,
      quarterDict,
      gradeDict,
      subjetDict,
      classTypeDict,
      classLevelDict,
      handleSelected,
      handleInputChange,
      handleQuarterChange,
      handleClassTypeChange,
      handleProductTypeChange,
      className,
      studentManageId,
      teacherId,
    } = this.props;
    let columns = [
      {
        title: '序号',
        width: 68,
        render(currentValue, rowData, index) {
          return index + 1;
        },
      },
      { title: '班级名称', width: 304, dataIndex: 'className', width: 260 },
      { title: '期数/星期数', width: 112, dataIndex: 'phaseName' },
      { title: '主讲老师/班主任', width: 134, dataIndex: 'studentManageName' },
      { title: '辅导老师', width: 134, dataIndex: 'teacherName' },
      { title: '上课学生', width: 102, dataIndex: 'studentNum' },
    ];
    return (
      <Modal title="选择班级" width={1100} onCancel={closeSelectClass} onOk={searchStudents} loading={loading}>
        <div className={compStyle.ModalChooseClass}>
          <SearchBar onSearch={onSearch} onReset={reset} expand>
            <Input
              placeholder="班级名称"
              value={className}
              onChange={(event) => {
                handleInputChange(event, 'className');
              }}
            />
            <Select
              placeholder="产品年份"
              dict={yearDict}
              value={yearId}
              allowClear={false}
              onChange={(yearId) => {
                handleSelected(yearId, 'yearId');
              }}
              validator={{
                rules: ['required'],
                silent: !errors['year'] || !checking,
                showTips: false,
                showStatus: true,
                onErrorChange: (e) => errorChange('year', e),
              }}
            />
            <MultiSelect
              placeholder="产品季度"
              dict={quarterDict}
              value={quarterId}
              allowClear={false}
              onChange={(quarterId) => {
                handleQuarterChange(quarterId);
              }}
              validator={{
                rules: ['required'],
                silent: !errors['quarter'] || !checking,
                showTips: false,
                showStatus: true,
                onErrorChange: (e) => errorChange('quarter', e),
              }}
            />
            <Select
              placeholder="产品类型"
              dict={productTypeDict}
              value={productTypeId}
              allowClear={false}
              disabled={true}
              onChange={(productTypeId) => {
                handleProductTypeChange(productTypeId);
              }}
              validator={{
                rules: ['required'],
                silent: !errors['productType'] || !checking,
                showTips: false,
                showStatus: true,
                onErrorChange: (e) => errorChange('productType', e),
              }}
            />
            <MultiSelect
              placeholder="年级"
              dict={gradeDict}
              value={gradeId}
              allowClear={false}
              onChange={(gradeId) => {
                handleSelected(gradeId, 'gradeId');
              }}
              validator={{
                rules: ['required'],
                silent: !errors['grade'] || !checking,
                showTips: false,
                showStatus: true,
                onErrorChange: (e) => errorChange('grade', e),
              }}
            />
            <MultiSelect
              placeholder="科目"
              dict={subjetDict}
              value={subjectId}
              allowClear={false}
              onChange={(subjectId) => {
                handleSelected(subjectId, 'subjectId');
              }}
              validator={{
                rules: ['required'],
                silent: !errors['subject'] || !checking,
                showTips: false,
                showStatus: true,
                onErrorChange: (e) => errorChange('subject', e),
              }}
            />
            <Select
              placeholder="班级类型"
              dict={classTypeDict}
              value={classTypeId}
              allowClear={false}
              onChange={(classTypeId) => {
                handleClassTypeChange(classTypeId);
              }}
              validator={{
                rules: ['required'],
                silent: !errors['tagType'] || !checking,
                showTips: false,
                showStatus: true,
                onErrorChange: (e) => errorChange('tagType', e),
              }}
            />
            <Select
              placeholder="班级层次"
              dict={classLevelDict}
              value={classLevelId}
              allowClear={false}
              onChange={(classLevelId) => {
                handleSelected(classLevelId, 'classLevelId');
              }}
              validator={{
                rules: ['required'],
                silent: !errors['classLevel'] || !checking,
                showTips: false,
                showStatus: true,
                onErrorChange: (e) => errorChange('classLevel', e),
              }}
            />
            <Select
              placeholder="分公司"
              dict={branchDict}
              value={branchId}
              allowClear={false}
              onChange={(branchId) => {
                handleBranchChange(branchId);
              }}
              validator={{
                rules: ['required'],
                silent: !errors['branch'] || !checking,
                showTips: false,
                showStatus: true,
                onErrorChange: (e) => errorChange('branch', e),
              }}
            />
            <Select
              placeholder="学习中心"
              dict={campusDict}
              value={campusId}
              allowClear={false}
              onChange={(campusId) => {
                handleSelected(campusId, 'campusId');
              }}
              validator={{
                rules: ['required'],
                silent: !errors['campus'] || !checking,
                showTips: false,
                showStatus: true,
                onErrorChange: (e) => errorChange('campus', e),
              }}
            />
            <Select
              placeholder="主讲老师"
              dict={mainTeacher}
              value={studentManageId}
              showSearch={true}
              onChange={(studentManageId) => {
                handleSelected(studentManageId, 'studentManageId');
              }}
            />
            <Select
              placeholder="辅讲老师"
              dict={assitTeacher}
              value={teacherId}
              showSearch={true}
              onChange={(teacherId) => {
                handleSelected(teacherId, 'teacherId');
              }}
            />
          </SearchBar>
        </div>

        <List
          dataSource={dataSource}
          columns={columns}
          rowSelection={{
            selectedRowKeys,
            onChange: (keyArray, rowDataArray) => {
              let currentClassId = rowDataArray
                .map((rowData) => {
                  return rowData.classId;
                })
                .reduce((pre, currentValue) => {
                  return pre + ',' + currentValue;
                }, '')
                .slice(1);
              checkedClass(keyArray, currentClassId);
            },
          }}
        ></List>
      </Modal>
    );
  }
}

class SelectClassModalContainer extends React.Component {
  constructor(props) {
    super(props);

    const defaultProductTypeId = SYSTEM_MARK === 'TWO_TEACHER' ? 'TWO_TEACHER' : 'SMALL_CLASS';
    this.state = {
      campusDict: this.props.campusDict || null,
      classLevelDict: this.props.classLevelDict || null,
      classTypeDict: this.props.classTypeDict || null,
      phaseDict: null,
      loading: false,
      dataSource: [],
      className: null,
      studentManageId: null,
      teacherId: null,
      branchId: null,
      campusId: null,
      phaseId: null,
      productTypeId: defaultProductTypeId,
      yearId: null,
      quarterId: null,
      gradeId: null,
      subjectId: null,
      classTypeId: null,
      classLevelId: null,
      checking: false,
      errors: {},
      selectedRowKeys: [], // 选中行集合
      classId: this.props.classIds,
      defaultValue: {}, // 默认值对象
      currentClassId: '', // 当前选中的班级id
    };
  }

  componentWillMount() {
    const {
      branchId,
      campusId,
      classLevelId,
      gradeId,
      productTypeId,
      quarterId,
      subjectId,
      classTypeId,
      yearId,
      campusDict,
      classLevelDict,
      phaseDict,
    } = this.props;
    const defaultValue = {
      branchId,
      campusId,
      classLevelId,
      gradeId,
      productTypeId,
      quarterId,
      subjectId,
      classTypeId,
      yearId,
      campusDict,
      classLevelDict,
      phaseDict,
    };
    this.setState({ defaultValue });
  }

  componentDidMount() {
    const { branchId, campusId, productTypeId, yearId, quarterId, subjectId, classTypeId, classLevelId } = this.props;
    this.setState({ branchId, campusId, productTypeId, yearId, quarterId, subjectId, classTypeId, classLevelId });
    this.handleQuarterChange(quarterId);
  }

  /**
   * 分公司改变获取对应的学习中心字典
   */
  handleBranchChange = async (branchId) => {
    await this.setState({ branchId, campusId: null });
    if (branchId) {
      const campusDict = await loadCampusDict(branchId);
      this.setState({
        campusDict,
      });
    }
  };

  /**
   * 班级类型改变获取对应的班级层次
   */
  handleClassTypeChange = async (classTypeId) => {
    await this.setState({ classTypeId, classLevelId: null });
    if (classTypeId) {
      const classLevelDict = await loadCommonDict('CLASS_LEVEL', classTypeId);
      this.setState({
        classLevelDict,
      });
    }
  };

  /**
   *  产品类型改变获取对应的班级类型
   */
  handleProductTypeChange = async (productTypeId) => {
    this.setState({ classTypeId: null, classLevelId: null, productTypeId });
    if (productTypeId) {
      const classTypeDict = await loadCommonDict('CLASS_TYPE', productTypeId);
      this.setState({ classTypeDict });
    }
  };

  /**
   * 根据产品季度获取期/星期数
   */
  handleQuarterChange = async (quarterId) => {
    await this.setState({ quarterId, phaseId: null });
    const phaseDict = await loadCommonDict(
      quarterId === 'DAT0000000231' || quarterId === 'DAT0000000233' ? 'WeekDay' : 'SMALL_CLASS_PHASE',
    );
    this.setState({ phaseDict });
  };

  /**
   * 处理文本框改变
   */
  handleInputChange = (e, type) => {
    this.setState({ [type]: e.target.value });
  };

  /**
   * 处理下拉框选中改变
   */
  handleSelected = (value, type) => {
    this.setState({ [type]: value });
  };

  /**
   * 搜索班级列表
   */
  onSearch = async () => {
    let { checking, errors, classId } = this.state;
    if (!checking) {
      this.setState({ checking: true });
    }
    if (Object.values(errors).find((v) => v)) {
      return;
    }
    this.setState({ loading: true });
    let {
      className,
      teacherId,
      branchId,
      campusId,
      productTypeId,
      yearId,
      quarterId,
      gradeId,
      subjectId,
      classTypeId,
      classLevelId,
      studentManageId,
    } = this.state;
    let quarter = quarterId
      .reduce((pre, currentValue, index) => {
        return pre + ',' + currentValue;
      }, '')
      .slice(1);
    let newSubjectId = subjectId
      .reduce((pre, currentValue, index) => {
        return pre + ',' + currentValue;
      }, '')
      .slice(1);
    gradeId = gradeId.join(',');
    const params = {
      className,
      studentManageId,
      teacherId,
      branchId,
      campusId,
      productTypeId,
      years: yearId,
      quarter,
      gradeId,
      subjectId: newSubjectId,
      classTypeId,
      classLevel: classLevelId,
    };
    let data = await loaders.getClassStudentQualificationApi(params).catch(() => {
      this.setState({ loading: false });
    });
    let selectedRowKeys = data.map((value, index) => {
      return index;
    });
    let currentClassId = data
      .map((rowData) => {
        return rowData.classId;
      })
      .reduce((pre, currentValue) => {
        return pre + ',' + currentValue;
      }, '')
      .slice(1);
    this.setState({ dataSource: data, selectedRowKeys, currentClassId, loading: false });
  };

  /**
   * 班级列表选中
   */
  checkedClass = (keyArray, currentClassId) => {
    this.setState({ selectedRowKeys: keyArray, currentClassId });
  };

  /**
   * 重置
   */
  reset = () => {
    const { defaultValue } = this.state;
    this.setState({ ...defaultValue });
  };

  // 采集校验信息进 errors 对象
  errorChange = (key, error) => {
    this.setState((preState) => {
      const { errors } = preState;
      return {
        errors: {
          ...errors,
          [key]: error,
        },
      };
    });
  };

  /**
   *  查找学生并保存选中班级Id
   */
  searchStudents = async () => {
    this.setState({ loading: true });
    const { saveStudents, saveClassIds, closeSelectClass } = this.props;
    let { classId, productTypeId, currentClassId } = this.state;
    classId = classId === null ? currentClassId : classId + ',' + currentClassId;
    let params =
      productTypeId === 'TWO_TEACHER'
        ? { twoteacherClasssId: classId, smallClassId: '', productTypeId, pageSize: 10 }
        : { twoteacherClasssId: '', smallClassId: classId, productTypeId, pageSize: 10 };
    let data = await loaders.getClassStudentQualificationInfoApi(params);
    this.setState({ loading: false });
    saveStudents(data);
    saveClassIds(classId);
    closeSelectClass();
  };

  render() {
    const {
      branchDict,
      productTypeDict,
      yearDict,
      quarterDict,
      gradeDict,
      subjetDict,
      handleSelected,
      mainTeacher,
      assitTeacher,
    } = this.props;
    const { closeSelectClass } = this.props;
    const { dataSource } = this.state;
    const {
      loading,
      branchId,
      campusId,
      productTypeId,
      yearId,
      quarterId,
      gradeId,
      subjectId,
      classTypeId,
      classLevelId,
      checking,
      errors,
      campusDict,
      classTypeDict,
      classLevelDict,
      phaseDict,
      phaseId,
      className,
      studentManageId,
      teacherId,
      selectedRowKeys,
      currentClassId,
    } = this.state;
    return (
      <SelectClassModal
        loading={loading}
        branchDict={branchDict}
        campusDict={campusDict}
        productTypeDict={productTypeDict}
        yearDict={yearDict}
        quarterDict={quarterDict}
        gradeDict={gradeDict}
        subjetDict={subjetDict}
        phaseDict={phaseDict}
        classTypeDict={classTypeDict}
        classLevelDict={classLevelDict}
        branchId={branchId}
        campusId={campusId}
        productTypeId={productTypeId}
        yearId={yearId}
        quarterId={quarterId}
        gradeId={gradeId}
        subjectId={subjectId}
        classTypeId={classTypeId}
        phaseId={phaseId}
        classLevelId={classLevelId}
        handleClassTypeChange={this.handleClassTypeChange}
        handleBranchChange={this.handleBranchChange}
        closeSelectClass={closeSelectClass}
        className={className}
        studentManageId={studentManageId}
        teacherId={teacherId}
        checking={checking}
        errors={errors}
        dataSource={dataSource}
        errorChange={this.errorChange}
        onSearch={this.onSearch}
        handleSelected={this.handleSelected}
        handleInputChange={this.handleInputChange}
        handleQuarterChange={this.handleQuarterChange}
        reset={this.reset}
        selectedRowKeys={selectedRowKeys}
        checkedClass={this.checkedClass}
        searchStudents={this.searchStudents}
        mainTeacher={mainTeacher}
        assitTeacher={assitTeacher}
        handleProductTypeChange={this.handleProductTypeChange}
        currentClassId={currentClassId}
      ></SelectClassModal>
    );
  }
}

export default SelectClassModalContainer;
