import React from 'react';
import Modal from 'COMPONENT/Modal';
import Input from 'COMPONENT/Input';
import Button from 'COMPONENT/Button';
import List from 'COMPONENT/List';
import loaders from '../loader';
import styles from './styles.less';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

class SelectStudentModal extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      queryId: null,
      studentList: [],
      selectedRowKeys: [],
      studentIds: null,
      students: [],
    };
  }

  /**
   * 搜索学生列表
   */
  searchStudentList = async () => {
    this.setState({ loading: true });
    const { queryId } = this.state;
    const { campusId } = this.props;
    if (!queryId) {
      showMessage('查询参数不能为空。');
      this.setState({ loading: false });
      return;
    }
    let studentList = await loaders.getSingleStudentQualificationInfoApi({ queryId, campusId });
    let selectedRowKeys = studentList.map((student, index) => {
      return index;
    });
    let studentIds = studentList
      .map((student) => {
        return student.studentId;
      })
      .reduce((pre, currentValue) => {
        return pre + ',' + currentValue;
      }, '')
      .slice(1);
    this.setState({ studentList, students: studentList, studentIds, loading: false, selectedRowKeys });
  };

  render() {
    const { loading, studentList, selectedRowKeys } = this.state;
    const { closeSelectStudent, saveNewStudentsId } = this.props;
    let columns = [
      {
        title: '序号',
        width: 68,
        render(currentValue, rowData, index) {
          return index + 1;
        },
      },
      { title: '学生名称', width: 156, dataIndex: 'studentName' },
      {
        title: '客户号码',
        width: 172,
        dataIndex: 'contact',
        render(currentValue, rowData, index) {
          return currentValue.slice(0, 3) + '****' + currentValue.slice(7);
        },
      },
      { title: '分公司', width: 160, dataIndex: 'branchName' },
      { title: '学籍学习中心', width: 168, dataIndex: 'blCampusName' },
      { title: '年级', width: 132, dataIndex: 'gradeName' },
    ];
    return (
      <Modal
        loading={loading}
        title="选择学生"
        width={960}
        onCancel={closeSelectStudent}
        onOk={() => {
          const { studentIds, students } = this.state;
          saveNewStudentsId(studentIds, students);
          closeSelectStudent();
        }}
      >
        <div className={styles.search}>
          <Input
            className={styles.input}
            placeholder="查询学生名称/学生编号/客户号码"
            onChange={(event) => {
              this.setState({ queryId: event.target.value });
            }}
          />
          <Button onClick={this.searchStudentList} className={styles.button}>
            查询
          </Button>
        </div>
        <List
          dataSource={studentList}
          columns={columns}
          rowSelection={{
            selectedRowKeys,
            onChange: (selectedRowKeys, rowDataArray) => {
              let studentIds = rowDataArray
                .map((rowData) => {
                  return rowData.studentId;
                })
                .reduce((pre, currentValue) => {
                  return pre + ',' + currentValue;
                }, '')
                .slice(1);
              this.setState({ studentIds, selectedRowKeys, students: rowDataArray });
            },
          }}
        ></List>
      </Modal>
    );
  }
}

export default SelectStudentModal;
