import React from 'react';
import { connect } from 'dva';
import { Row, Col, Tooltip } from 'antd';
import IconButton from 'COMPONENT/IconButton';
import Loading from 'COMPONENT/Loading';
import styles from './styles.less';
import echarts from 'ASSET/js/echarts.min.js';
import riseSvg from 'ASSET/imgs/rise.svg';
import fallSvg from 'ASSET/imgs/fall.svg';
import sameSvg from 'ASSET/imgs/same.svg';
import { setJudgeResult, getComparedDetails, getComparedDetailsAuto } from 'SERVICE/scoreManage';
import { option, option2, option3 } from './option';
import getCurrentYearAndTerm from '../common';
let mapObject = {
  SCORE: 'score',
  GRADE_RANKING: 'gradeRanking',
  GRADE: 'evaluationType',
  CLASS_RANKING: 'classRanking',
};

let mapObjectSvg = [
  { value: 'PROGRESS', src: riseSvg },
  { value: 'FLAT', src: sameSvg },
  {
    value: 'BACKWARD',
    src: fallSvg,
  },
];
let mapObjectSvgTwo = { PROGRESS: riseSvg, FLAT: sameSvg, BACKWARD: fallSvg };

class CompareCard extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      showTipIndex: null, //{number} 修改判定结果的 索引
      dataList: [], //{array<object>} 列表数据项
      comparedId: null, //对比编号
      loading: false, //loading状态
      compareYearTerm: null, //{string} 对比成绩学年学期
      currentYearTerm: null, //{string} 当前学年学期
    };
  }

  componentDidMount() {
    this.setState({ loading: true });
  }

  componentWillReceiveProps(nextProps) {
    let { nearlyFiveData, nearlyFiveTimeList, count } = nextProps;
    if (count !== this.props.count) {
      let { cardListData, nearlyFiveData, nearlyFiveTimeList } = nextProps;
      let copyNearlyFiveData = JSON.parse(JSON.stringify(nearlyFiveTimeList));
      this.getComparedData(nearlyFiveData, copyNearlyFiveData, cardListData);
    } else {
    }
  }

  componentWillUnmount() {
    window.onresize = null;
  }

  /**
   * 加载对比数据
   * @returns {Promise.<void>}
   */
  getComparedData = async (nearlyFiveData, nearlyFiveTimeList, cardListData) => {
    let { studentId } = this.props;
    this.setState({ loading: true });
    let apiParams = { studentId };
    if (!cardListData.id) {
      let apiData = await getComparedDetailsAuto(apiParams);
      if (apiData) {
        this.handleDataToOption(apiData, nearlyFiveData, nearlyFiveTimeList);
      } else {
        let { checkBaseSubjectId } = this.props;
        checkBaseSubjectId({ baseSubjectId: [], baseScoreType: [] });
        this.setState({ loading: false, dataList: [] });
      }
    } else {
      this.handleDataToOption(cardListData, nearlyFiveData, nearlyFiveTimeList);
    }
  };

  /**
   * 将数据拼凑成echarts options
   * @param apiData
   * @param nearlyFiveData
   * @param nearlyFiveTimeList {} 最近五次时间 x轴
   */
  handleDataToOption = (apiData, nearlyFiveData, nearlyFiveTimeList) => {
    this.setState({ loading: false });
    let { checkBaseSubjectId } = this.props;
    let Date = getCurrentYearAndTerm();
    //得到对比科目和当前时间的学年学期
    this.setState({
      compareYearTerm: apiData.comparativeName,
      currentYearTerm: Date.schoolYear + (Date.term === 'NEXT_SEMESTER' ? '下学期' : '上学期'),
    });

    //对比记录详情数据
    let data = apiData.achievementBenchmarks || [];

    //得到基准科目id
    let baseSubjectId = data.map((v, k) => {
      return v.benchmarkSubjectId;
    });
    let baseScoreType = data.map((v, k) => {
      return mapObject[v.benchmarkAchievementTypeValue];
    });
    checkBaseSubjectId({ baseSubjectId, baseScoreType });

    let array = [];
    data.forEach((v, k) => {
      array[k] = [];
      //遍历最近五次数据，拿到五次成绩下面对应科目的数据
      nearlyFiveData.forEach((V, K) => {
        let time = Object.keys(V)[0];
        //得到每次成绩记录下面所有科目成绩对象
        let data = Object.values(V)[0];
        if (data) {
          let filterarray = data.filter((value, key) => {
            return value.subjectId === v.subjectId;
          });
          array[k].push(filterarray[0]);
        } else {
          array[k].push(undefined);
        }
      });
    });
    let yaxisList = [];
    //科目名称数组
    array.forEach((v, k) => {
      yaxisList[k] = [];
      v.forEach((value, key) => {
        if (!value) {
          //不存在此科目
          // yaxisList[k].push(undefined);
          yaxisList[k].push(null);
        } else {
          let type = data[k].benchmarkAchievementTypeValue;
          if (type === 'GRADE') {
            this.mapGradeToNumber(yaxisList[k], value[mapObject[type]]);
          } else if (type === 'GRADE_RANKING' || type === 'CLASS_RANKING') {
            this.turnRanking(yaxisList[k], value[mapObject[type]] || undefined);
            // yaxisList[k].push(value[mapObject[type]] || undefined);
          } else {
            yaxisList[k].push(value[mapObject[type]] || undefined);
          }
        }
      });
    });
    let echartsInstanceArray = [];
    nearlyFiveTimeList.reverse();
    this.setState({ dataList: data, comparedId: apiData.id, loading: false }, () => {
      data.forEach((v, k) => {
        yaxisList[k].reverse();
        if (v.benchmarkTypeName !== '无效') {
          let baseIndex = nearlyFiveTimeList.findIndex((V, K) => {
            return v.benckmarkExaminationDate === V;
          });

          let comparedIndex = nearlyFiveTimeList.findIndex((V, K) => {
            return v.compareExaminationDate === V;
          });

          //重合情形
          if (baseIndex === comparedIndex) {
            option2.series.forEach((v, k) => {
              v.data = [];
            });
            option2.series[0].data = yaxisList[k].map((v, k) => {
              if (k === baseIndex) {
                return {
                  value: v,
                  symbolSize: 12,
                  symbol: 'circle',
                  itemStyle: { normal: { borderWidth: 2, borderColor: '#FFB744' } },
                };
              } else {
                return {
                  value: v,
                  symbolSize: 10,
                  itemStyle: { normal: { color: '#666', borderColor: '#fff', borderWidth: 4 } },
                };
              }
            });

            let len = yaxisList[k].length;
            //尾部虚线
            if (yaxisList[k][len - 1] === null || yaxisList[k][len - 1] === undefined) {
              let lastDashData = this.getLastDashData(yaxisList[k], len);
              option2.series[2].data = lastDashData.map((v, k) => {
                if (k === comparedIndex) {
                  return {
                    value: v,
                    symbolSize: 12,
                    // symbol: 'emptyCircle',
                    // itemStyle: {normal: {color: '#FFB744', borderWidth: 4}}
                    itemStyle: { normal: { borderWidth: 2, color: '#8E82FF', borderColor: '#FFB744' } },
                  };
                } else if (k === baseIndex) {
                  return {
                    value: v,
                    symbolSize: 10,
                    symbol: 'emptyCircle',
                    itemStyle: { normal: { color: '#8E82FF', borderWidth: 4 } },
                  };
                } else {
                  return {
                    value: v,
                    symbolSize: 10,
                    itemStyle: { normal: { color: '#666', borderColor: '#fff', borderWidth: 4 } },
                  };
                }
              });
            }
            //头部虚线
            if (yaxisList[k][0] === null || yaxisList[k][0] === undefined) {
              let firstDashData = this.getFirstDashData(yaxisList[k], len);
              let allUndefined;
              firstDashData.forEach((v) => {
                if (v) {
                  allUndefined = v;
                }
              });
              option2.series[3].data = firstDashData.map((v, k) => {
                if (k === baseIndex) {
                  return {
                    value: allUndefined ? v : 15,
                    symbolSize: 12,
                    // symbol: 'emptyCircle',
                    itemStyle: { normal: { borderWidth: 2, color: '#8E82FF', borderColor: '#FFB744' } },
                  };
                }
                // else if (k === comparedIndex) {
                //   return {
                //     value: allUndefined ? v : 15,
                //     symbolSize: 10,
                //     symbol: 'emptyCircle',
                //     itemStyle: {normal: {color: 'blue', borderWidth: 4}}
                //   }
                // }
                else {
                  return {
                    value: allUndefined ? v : 15,
                    symbolSize: 10,
                    itemStyle: { normal: { color: '#666', borderColor: '#fff', borderWidth: 4 } },
                  };
                }
              });
            }
            option2.series[0].markPoint.data[0].coord = [baseIndex, yaxisList[k][baseIndex]];
            if (v.benchmarkAchievementTypeValue === 'GRADE') {
              option2.series[0].markPoint.label.normal.formatter = (params) => {
                let markData = params.data.coord[1];
                return this.mapNumberToGrade(markData);
              };
            } else if (
              v.benchmarkAchievementTypeValue === 'GRADE_RANKING' ||
              v.benchmarkAchievementTypeValue === 'CLASS_RANKING'
            ) {
              option2.series[0].markPoint.label.normal.formatter = (params) => {
                return 50 - params.data.coord[1];
              };
            } else {
              option2.series[0].markPoint.label.normal.formatter = (params) => {
                return params.data.coord[1];
              };
            }
            option2.xAxis.data = nearlyFiveTimeList;
            option2.tooltip.formatter = (params) => {
              if (v.benchmarkAchievementTypeValue === 'GRADE') {
                this.mapParamsValueToGrade(params);
              }
              if (
                v.benchmarkAchievementTypeValue === 'GRADE_RANKING' ||
                v.benchmarkAchievementTypeValue === 'CLASS_RANKING'
              ) {
                this.reverseTurnRanking(params);
              }
              if (params[0].value === null) {
                return params[0].axisValue + '&nbsp;&nbsp;&nbsp;' + `无${v.subjectName}科目`;
              }
              return (
                params[0].axisValue +
                '&nbsp;&nbsp;&nbsp;' +
                v.subjectName +
                '<br/>' +
                v.benchmarkAchievementTypeName +
                '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;' +
                (params[0].value ? params[0].value : '无')
              );
            };
            let echartsInstance = echarts.init(this.refs['$echarts' + k]);
            echartsInstanceArray.push(echartsInstance);
            echartsInstance.clear();
            echartsInstance.setOption(option2);
          } else {
            option.series.forEach((v, k) => {
              v.data = [];
            });
            let len = yaxisList[k].length;
            //尾部虚线
            if (yaxisList[k][len - 1] === null || yaxisList[k][len - 1] === undefined) {
              let lastDashData = this.getLastDashData(yaxisList[k], len);
              option.series[2].data = lastDashData.map((v, k) => {
                if (k < comparedIndex) {
                  return {
                    value: undefined,
                  };
                } else {
                  if (k === comparedIndex) {
                    return {
                      value: v,
                      symbolSize: 10,
                      symbol: 'emptyCircle',
                      itemStyle: { normal: { color: '#FFB744', borderWidth: 4 } },
                    };
                  } else if (k === baseIndex) {
                    return {
                      value: v,
                      symbolSize: 10,
                      symbol: 'emptyCircle',
                      itemStyle: { normal: { color: '#8E82FF', borderWidth: 4 } },
                    };
                  } else {
                    return {
                      value: v,
                      symbolSize: 10,
                      itemStyle: { normal: { color: '#666', borderColor: '#fff', borderWidth: 4 } },
                    };
                  }
                }
              });
            }
            //头部虚线
            if (yaxisList[k][0] === null || yaxisList[k][0] === undefined) {
              // if (yaxisList[k][0] === null) {
              let firstDashData = this.getFirstDashData(yaxisList[k], len);
              let allUndefined;
              firstDashData.forEach((v) => {
                if (v) {
                  allUndefined = v;
                }
              });
              option.series[3].data = firstDashData.map((v, k) => {
                if (k === baseIndex) {
                  return {
                    value: allUndefined ? v : 15,
                    symbolSize: 10,
                    symbol: 'emptyCircle',
                    itemStyle: { normal: { color: '#8E82FF', borderWidth: 4 } },
                  };
                } else if (k === comparedIndex) {
                  return {
                    value: allUndefined ? v : 15,
                    symbolSize: 10,
                    symbol: 'emptyCircle',
                    itemStyle: { normal: { color: '#FFB744', borderWidth: 4 } },
                  };
                } else {
                  return {
                    value: allUndefined ? v : 15,
                    symbolSize: 10,
                    itemStyle: { normal: { color: '#666', borderWidth: 4, borderColor: '#fff' } },
                  };
                }
              });
            }

            let markBaseData = yaxisList[k][baseIndex];
            let markCompareData = yaxisList[k][comparedIndex];

            option.series[0].markPoint.data[0].coord = [baseIndex, yaxisList[k][baseIndex]];
            let tructMarkCompareData = markCompareData;

            //取最相邻的有效值
            if (!tructMarkCompareData) {
              for (let i = comparedIndex; i < len; i++) {
                if (yaxisList[k][i]) {
                  tructMarkCompareData = yaxisList[k][i];
                  break;
                }
              }
            }
            if (!tructMarkCompareData) {
              for (let i = comparedIndex; i > -1; i--) {
                if (yaxisList[k][i]) {
                  tructMarkCompareData = yaxisList[k][i];
                  break;
                }
              }
            }
            option.series[0].markPoint.data[1].coord = [comparedIndex, tructMarkCompareData];
            if (v.benchmarkAchievementTypeValue === 'GRADE') {
              option.series[0].markPoint.label.normal.formatter = (params) => {
                let markData = params.data.coord[1];
                if (params.data.coord[0] === comparedIndex) {
                  return this.mapNumberToGrade(markCompareData);
                } else {
                  return this.mapNumberToGrade(markData);
                }
              };
            } else if (
              v.benchmarkAchievementTypeValue === 'GRADE_RANKING' ||
              v.benchmarkAchievementTypeValue === 'CLASS_RANKING'
            ) {
              option.series[0].markPoint.label.normal.formatter = (params) => {
                return 50 - params.data.coord[1];
              };
            } else {
              option.series[0].markPoint.label.normal.formatter = (params) => {
                if (params.data.coord[0] === comparedIndex) {
                  return parseFloat(v.compareValue) ? parseFloat(v.compareValue) : v.compareValue;
                } else {
                  return parseFloat(v.benchmarkValue) ? parseFloat(v.benchmarkValue) : v.benchmarkValue;
                }
              };
            }
            option.series[0].data = yaxisList[k].map((v, k) => {
              if (k === baseIndex) {
                return { value: v, symbolSize: 10, symbol: 'emptyCircle', itemStyle: { normal: { borderWidth: 4 } } };
              } else if (k === comparedIndex) {
                return {
                  value: tructMarkCompareData,
                  symbolSize: 10,
                  symbol: 'emptyCircle',
                  itemStyle: { normal: { borderWidth: 4, borderColor: '#FFB744' } },
                };
              } else {
                return { value: v, symbolSize: 10, itemStyle: { normal: { borderColor: '#fff', borderWidth: 4 } } };
              }
            });

            option.series[1].data = yaxisList[k].map((v, k) => {
              if (k === comparedIndex) {
                return {
                  value: v,
                  symbolSize: 10,
                  symbol: 'emptyCircle',
                  itemStyle: { normal: { color: '#FFB744', borderWidth: 4 } },
                };
              } else if ((k >= baseIndex && k < comparedIndex) || (k < baseIndex && k >= comparedIndex)) {
                return { value: v, itemStyle: { normal: { color: 'transparent' } } };
              } else {
                return {
                  value: v,
                  symbolSize: 10,
                  itemStyle: { normal: { color: '#666', borderColor: '#fff', borderWidth: 4 } },
                };
              }
            });
            option.visualMap.pieces = [{ gte: baseIndex, lte: comparedIndex, color: '#8E82FF' }];
            option.xAxis.data = nearlyFiveTimeList;
            option.tooltip.formatter = (params) => {
              if (v.benchmarkAchievementTypeValue === 'GRADE') {
                this.mapParamsValueToGrade(params);
              }
              if (
                v.benchmarkAchievementTypeValue === 'GRADE_RANKING' ||
                v.benchmarkAchievementTypeValue === 'CLASS_RANKING'
              ) {
                this.reverseTurnRanking(params);
              }
              if (params[0].value === null) {
                return params[0].axisValue + '&nbsp;&nbsp;&nbsp;' + `无${v.subjectName}科目`;
              }
              if (params[0].dataIndex === comparedIndex) {
                if (!markCompareData) {
                  return (
                    params[0].axisValue +
                    '&nbsp;&nbsp;&nbsp;' +
                    v.subjectName +
                    '<br/>' +
                    v.benchmarkAchievementTypeName +
                    '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;' +
                    '无'
                  );
                }
              }
              return (
                params[0].axisValue +
                '&nbsp;&nbsp;&nbsp;' +
                v.subjectName +
                '<br/>' +
                v.benchmarkAchievementTypeName +
                '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;' +
                (params[0].value ? params[0].value : '无')
              );
            };

            let echartsInstance = echarts.init(this.refs['$echarts' + k]);
            echartsInstanceArray.push(echartsInstance);
            echartsInstance.clear();
            echartsInstance.setOption(option);
          }
        } else {
          option3.series.forEach((v, k) => {
            v.data = [];
          });
          option3.xAxis.data = [v.benckmarkExaminationDate, v.compareExaminationDate, '', '', ''];
          let value = Number(v.benchmarkValue) ? Number(v.benchmarkValue) : this.checkGradeToNumber(v.benchmarkValue);
          option3.series[0].data = [
            {
              value: value,
              symbolSize: 10,
              symbol: 'emptyCircle',
              itemStyle: {
                normal: {
                  borderWidth: 4,
                },
              },
            },
            {
              value: value,
              symbolSize: 10,
              symbol: 'emptyCircle',
              itemStyle: {
                normal: {
                  color: '#FFB744',
                  borderWidth: 4,
                },
              },
            },
          ];
          option3.tooltip.formatter = (params) => {
            if (params[0].axisValue === v.compareExaminationDate) {
              if (v.compareSubjectName) {
                return '无对比成绩';
              } else {
                return '无对比科目';
              }
            }

            let value = v.benchmarkValue;
            value = parseFloat(value) ? parseFloat(value) : value;
            return (
              params[0].axisValue +
              '&nbsp;&nbsp;&nbsp;' +
              v.subjectName +
              '<br/>' +
              v.benchmarkAchievementTypeName +
              '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;' +
              (value ? value : '无')
            );
          };
          option3.series[0].markPoint.data[0].coord = [0, value];
          option3.series[0].markPoint.data[1].coord = [1, value];
          option3.series[0].markPoint.label.normal.formatter = (params) => {
            if (params.dataIndex === 1) {
              return 'N/A';
            } else {
              let value = v.benchmarkValue;
              value = parseFloat(value) ? parseFloat(value) : value;
              return value;
            }
          };

          let echartsInstance = echarts.init(this.refs['$echarts' + k]);
          echartsInstanceArray.push(echartsInstance);
          //清空画布
          echartsInstance.clear();
          echartsInstance.setOption(option3);
        }
      });

      window.onresize = () => {
        echartsInstanceArray.forEach((v, k) => {
          v.resize();
        });
      };
    });
  };

  turnRanking = (array, value) => {
    if (value) {
      array.push(50 - value);
    } else {
      array.push(value);
    }
  };
  reverseTurnRanking = (params) => {
    if (params[0].value) {
      params[0].value = 50 - params[0].value;
    }
  };
  /**
   * 得到markPoint到点的差值
   * 绘图区域grid高度为85px，上移10px，y坐标最大值为 yaxisList[k]中最大值*2
   */
  getDistance = (array) => {
    let copyArray = JSON.parse(JSON.stringify(array));
    //去除 undefined,存在undefined的话求最大值会得到NaN
    for (let i = 0; i < copyArray.length; i++) {
      if (copyArray[i] === undefined) {
        copyArray.splice(i, 1);
        i--;
      }
    }
    let maxData = Math.max.apply(null, copyArray);
    return (maxData * 10 * 2) / 85;
  };

  /**
   * 获取头部虚线部分数据
   * @param array
   * @param len
   * @returns {Array|*}
   */
  getFirstDashData = (array, len) => {
    let firstIndex;
    for (let i = 0; i < len; i++) {
      if (array[i]) {
        firstIndex = i;
        break;
      }
    }
    let firstDashData = array.map((V, K) => {
      if (K <= firstIndex) {
        return array[firstIndex];
      }
      return undefined;
    });
    return firstDashData;
  };

  /**
   * 获取尾部虚线数据
   * @param array
   * @param len
   * @returns {Array|*}
   */
  getLastDashData = (array, len) => {
    let lastIndex;
    for (let i = len - 1; i > -1; i--) {
      if (array[i]) {
        lastIndex = i;
        break;
      }
    }
    let lastDashData = array.map((V, K) => {
      if (K >= lastIndex) {
        return array[lastIndex];
      }
      return undefined;
    });
    return lastDashData;
  };

  /**
   * 将评级转换为数值
   * @param array
   * @param grade
   */
  mapGradeToNumber = (array, grade) => {
    if (grade === 'A') {
      array.push(100);
    } else if (grade === 'B') {
      array.push(80);
    } else if (grade === 'C') {
      array.push(60);
    } else if (grade === 'D') {
      array.push(40);
    } else if (grade === 'E') {
      array.push(20);
    } else {
      array.push(undefined);
    }
  };

  /**
   * 将数值转换为ABCDE
   * @param markData
   * @returns {*}
   */
  mapNumberToGrade = (markData) => {
    let grade;
    if (markData === 100) {
      grade = 'A';
    } else if (markData === 80) {
      grade = 'B';
    } else if (markData === 60) {
      grade = 'C';
    } else if (markData === 40) {
      grade = 'D';
    } else if (markData === 20) {
      grade = 'E';
    } else {
      grade = 'N/A';
    }
    return grade;
  };

  checkGradeToNumber = (grade) => {
    if (grade === 'A') {
      return 100;
    } else if (grade === 'B') {
      return 80;
    } else if (grade === 'C') {
      return 60;
    } else if (grade === 'D') {
      return 40;
    } else if (grade === 'E') {
      return 20;
    } else {
      return 0;
    }
  };

  mapParamsValueToGrade = (params) => {
    if (params[0].value === 100) {
      params[0].value = 'A';
    } else if (params[0].value === 80) {
      params[0].value = 'B';
    } else if (params[0].value === 60) {
      params[0].value = 'C';
    } else if (params[0].value === 40) {
      params[0].value = 'D';
    } else if (params[0].value === 20) {
      params[0].value = 'E';
    }
  };
  /**
   * 鼠标悬浮在判定图标上显示
   * @param e
   * @param k
   */
  showTips = (e, k) => {
    let { btnAuths } = this.props;
    if (btnAuths.includes('MODIFY_COMPARISON_BTN')) {
      this.setState({ showTipIndex: k });
    }
  };
  /**
   * 鼠标移出隐藏判定图标
   * @param e
   * @param k
   */
  hideTips = (e, k) => {
    this.setState({ showTipIndex: null });
  };

  /**
   * 点击更改判定结果
   * @param item {} 对比项
   * @param value {value:'PROGRESS',src: riseSvg} 判定的值对象
   */
  changeTips = (item, value) => {
    let { studentId } = this.props;
    let { comparedId } = this.state;
    let apiParams = {
      achievementBenchmarks: [{ id: item.id, standardCategory: value.value, version: item.version }],
      id: comparedId,
    };
    this.setState({ loading: true });
    setJudgeResult(apiParams)
      .then((res) => {
        let apiParams = { comparisonId: comparedId };
        getComparedDetails(apiParams).then(({ achievementBenchmarks }) => {
          this.setState({
            dataList: achievementBenchmarks,
            showTipIndex: null,
            loading: false,
          });
        });
      })
      .catch((res) => {
        this.setState({ loading: false });
      });
  };

  render() {
    let { onSettingClick, cardListData, btnAuths } = this.props;
    let { showTipIndex, dataList, loading, compareYearTerm, currentYearTerm } = this.state;
    return (
      <div className={styles.wrap}>
        <div className={styles.title}>
          <div style={{ float: 'left' }}>
            <i>成绩数据分析</i>
            <IconButton
              type="question_mc"
              style={{ position: 'absolute', top: 24 }}
              title={{
                overlayClassName: 'compareTips',
                title: '每学期都需要设置一套基准成绩和对比成绩的组合',
                placement: 'right',
              }}
            />
          </div>
          <div style={{ textAlign: 'end' }}>
            <IconButton
              onClick={onSettingClick}
              type="setting_mc"
              size={24}
              title={{ title: '点击按钮设置基准成绩和对比成绩', placement: 'left' }}
            />
          </div>
        </div>

        {dataList.map((v, k) => {
          return (
            <div className={styles.content} key={k}>
              <Row gutter={0}>
                <Col span={15} className={styles.contentLeft}>
                  <div style={{ marginTop: 55 }}>
                    <span>
                      <div>
                        <span style={{ marginRight: 5 }}>{v.subjectName}</span>

                        <span
                          className={'wrapTips'}
                          onMouseEnter={(e) => {
                            this.showTips(e, k);
                          }}
                          onMouseLeave={(e) => {
                            this.hideTips(e, k);
                          }}
                        >
                          {showTipIndex === k && (
                            <div
                              className={'hideTips'}
                              style={{
                                display: showTipIndex === k ? 'block' : 'none',
                                top:
                                  v.standardCategoryValue === 'FLAT'
                                    ? -88
                                    : v.standardCategoryValue === 'PROGRESS'
                                    ? -36
                                    : -140,
                              }}
                            >
                              {mapObjectSvg.map((item, index) => {
                                return (
                                  <div key={index} className={v.standardCategoryValue === item.value ? 'current' : ''}>
                                    <img
                                      src={item.src}
                                      alt=""
                                      onClick={(e) => {
                                        this.changeTips(v, item);
                                      }}
                                    />
                                  </div>
                                );
                              })}
                            </div>
                          )}

                          {v.standardCategoryValue && showTipIndex !== k && (
                            <img className={styles.imgTips} src={mapObjectSvgTwo[v.standardCategoryValue]} alt="" />
                          )}
                        </span>
                      </div>
                      <div>
                        {v.benchmarkTypeName}
                        <span className={styles.question}>
                          <IconButton
                            title={{
                              placement: 'bottom',
                              overlayClassName: 'compareTips',
                              title:
                                v.benchmarkTypeName === '仅作记录'
                                  ? v.readOnlyDesc
                                  : v.benchmarkTypeName === '无效'
                                  ? v.compareSubjectName
                                    ? '无对比成绩，请补充后查看'
                                    : '无对比科目，请补充后查看'
                                  : v.modifyUserName
                                  ? `判定结果由${v.modifyUserName}修改`
                                  : '',
                            }}
                            type="question_mc"
                          />
                        </span>
                      </div>
                    </span>
                    <span>
                      <div className={styles.compared}>
                        <span>对比</span>
                        <span>
                          <i>{parseFloat(v.compareValue) ? parseFloat(v.compareValue) : v.compareValue || 'N / A'}</i>
                          {v.compareAchievementTypeName === '分数' && <i>{'/' + v.compareTotalScore}</i>}
                          <i>
                            {v.compareValue
                              ? v.compareAchievementTypeName === '分数'
                                ? '分'
                                : v.compareAchievementTypeName === '评级'
                                ? ''
                                : '名'
                              : ''}
                          </i>
                        </span>
                        {v.benchmarkTypeName !== '无效' && compareYearTerm != currentYearTerm && (
                          <Tooltip
                            overlayClassName={'compareTips'}
                            title={`现在是${currentYearTerm}，对比成绩不是最新`}
                          >
                            <span>{v.compareSubjectName}</span>
                          </Tooltip>
                        )}
                        {v.benchmarkTypeName !== '无效' && compareYearTerm == currentYearTerm && (
                          <span>{v.compareSubjectName}</span>
                        )}
                        {!v.compareSubjectName && <span>无对比科目，请补充后查看</span>}
                        {v.benchmarkTypeName === '无效' && v.compareSubjectName && (
                          <span>无对比成绩，请补充后查看</span>
                        )}
                      </div>
                      <div className={styles.base}>
                        <span>基准</span>
                        <span>
                          <i>{parseFloat(v.benchmarkValue) ? parseFloat(v.benchmarkValue) : v.benchmarkValue}</i>
                          {v.benchmarkAchievementTypeName === '分数' && <i>{'/' + v.benchmarkTotalScore}</i>}
                          <i>
                            {v.benchmarkAchievementTypeName === '分数'
                              ? '分'
                              : v.benchmarkAchievementTypeName === '评级'
                              ? ''
                              : '名'}
                          </i>
                        </span>
                        <span>{v.benchmarkSubjectName}</span>
                      </div>
                    </span>
                  </div>
                </Col>
                <Col span={9}>
                  <div className={'lineEcharts'} ref={'$echarts' + k}></div>
                </Col>
              </Row>
            </div>
          );
        })}
        {dataList.length === 0 && (
          <div className={`${styles.content} ${'no_data'}`}>
            <h3>未设置基准成绩和对比成绩</h3>
            <div>录入成绩后，可点击右上角按钮进行设置</div>
          </div>
        )}
        <Loading loading={loading} />
      </div>
    );
  }
}

export default connect()(CompareCard);
