import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import SearchBar from 'COMPONENT/SearchBar';
import CardList from 'COMPONENT/List/CardList';
import Select from 'COMPONENT/Select';
import DatePicker from 'COMPONENT/DatePicker';
import Button from 'COMPONENT/Button';
import { getDictTextByVal, loadCommonDict, loadBranchDict, loadAllBranchDict, mapDataToDict } from 'UTIL/dict';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import PopConfirm from 'COMPONENT/PopConfirm';
import { getProductPageList, cancelProduct } from 'SERVICE/product';
import SettingViewModal from './viewModel/settingViewModel';
import InputViewModal from './viewModel/resultInputViewModel';
import BigPhotoViewModel from './viewModel/bigPhoto';
import CompareCard from './compareCard';
import styles from './listPage.less';
import localStyles from './styles.less';
import TextLink from 'COMPONENT/TextLink';
import TopInfo from './studentInfo/index';
import { scoreType, premissionBtnKeyMap } from './premissionBtnMap';
import moment from 'moment';

import {
  getScoreList,
  getSelectOption,
  getSubjectByScoreID,
  getAllScoreById,
  deleteScore,
  getComparedDetails,
  getComparedDetailsAuto,
  getCityAndGradeById,
} from 'SERVICE/scoreManage';

import { peiYou, twoTeacher, prd } from 'CONST/buildEnv';

import qrcodeImage from './images/boss_achievement_qrcode.png';
import testQrcodeImage from './images/test_boss_achievement_qrcode.png';

const { COL_WIDTH_M, COL_WIDTH_L } = CardList;
const isProductionNodeEnv = !!process.env && [prd, twoTeacher, peiYou].includes(process.env.BUILD_ENV);

class GradeManage extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false,
      listLoading: false,
      totalRecords: 0, // {number} 总记录数
      currentPage: 1, // {number} 当前页码
      pageSize: 20, // {number} 每页记录数
      searchkeyList: {},
      searchExpand: false,

      list: [], //{array} 列表数据项
      yearDict: null, //{array<object>} 学年字典
      termDict: null, //{array<object>} 学期字典
      examTypeDict: null, //{array<object>} 考试类型字典

      year: null, //{string} 查询条件 年份
      term: null, //{string} 查询条件 学期
      examType: null, //{string} 查询条件 考试类型
      examDate: null, //{string} 查询条件 考试时间
      inputModalVisible: false, //{boolean} 成绩录入弹框
      settingModalVisible: false, //{boolean} 设置弹框

      searchKey: null,
      rows: [],
      cardListData: {}, //{object} 对比数据
      nearlyFiveData: [], //{array} 最近五次成绩
      count: 1,
      baseSubjectId: [], //{array} 基准成绩科目id
      baseScoreType: [], //{array} 基准成绩类型
      expandedRowKey: '',
      expandedRowKeys: [0],
    };
  }

  componentDidMount() {
    this.loadData();
    let { studentId } = this.props;
    Promise.all([
      getSelectOption({ selectOptionCategory: 'SCHOOL_YEAR' }),
      getSelectOption({ selectOptionCategory: 'Semester' }),
      getSelectOption({ selectOptionCategory: 'EXAMINATION_TYPE' }),
      getSelectOption({ selectOptionCategory: 'AchievementAbstractType' }),
      getCityAndGradeById({ studentId }),
    ]).then(([{ value: yearDict }, { value: termDict }, { value: examTypeDict }, res, studentInfo]) => {
      let yearDictarr = mapDataToDict(yearDict);
      let termDictarr = mapDataToDict(termDict);
      let examTypeDictarr = mapDataToDict(examTypeDict);
      let { gradeId, cityId } = studentInfo;
      this.setState({
        yearDict: yearDictarr,
        termDict: termDictarr,
        examTypeDict: examTypeDictarr,
        gradeId,
        cityId,
      });
    });
  }

  /**
   * 加载列表数据
   * @param searchKey
   * @returns {Promise.<void>}
   */
  loadData = async (searchKey) => {
    let { currentPage, pageSize, searchkeyList, list } = this.state;
    let { studentId } = this.props;
    let apiParams = { studentId, page: currentPage, rows: pageSize };
    searchkeyList = { page: currentPage, rows: pageSize, ...searchkeyList, ...searchKey };
    apiParams = { ...apiParams, ...searchkeyList };

    let { page: newCurrentPage, rows: newPageSize } = searchkeyList;
    this.setState({ listLoading: true });
    let apiData = await getScoreList(apiParams);
    let { rows, records } = apiData;
    rows = rows ? rows : [];
    let scoreRows = rows.map((v, k) => {
      return {
        year: v.schoolYearName,
        term: v.semesterName,
        examType: v.examinationTypeName,
        city: v.cityName,
        grade: v.gradeName,
        date: v.examinationDate,
        achievementTypeList: v.achievementTypeName,
        countRate: v.statisticalRate,
        evidence: v.achievementEvidence,
        id: v.id,
        achievementTemplateId: v.achievementTemplateId,
        achievementEvidence: v.achievementEvidence,
        version: v.version,
        totalClassRanking: v.totalClassRanking,
        totalEvaluationTypeValue: v.totalEvaluationTypeValue,
        totalGradeRanking: v.totalGradeRanking,
        totalScore: v.totalScore,
        achievementTypeValue: v.achievementTypeValue,
        expandData: list[k] ? list[k]['expandData'] : null,
        template: list[k] ? list[k]['template'] : false,
        createTime: v.createTime,
      };
    });
    let nearlyFiveIdList = rows.map((v, k) => {
      return v.id;
    });
    //加载最近一次数据
    if (scoreRows[0]) {
      await this.getSubjectAndSetTemplate(scoreRows[0]['id'], scoreRows, 0);
    }
    //最近五次数据时间
    let nearlyFiveTimeList = rows.map((v, k) => {
      return v.examinationDate;
    });
    if (nearlyFiveTimeList.length > 5 || nearlyFiveIdList.length > 5) {
      nearlyFiveTimeList = nearlyFiveTimeList.slice(0, 5);
      nearlyFiveIdList = nearlyFiveIdList.slice(0, 5);
    }
    if (!searchKey) {
      this.getNearlyFiveData(nearlyFiveIdList, nearlyFiveTimeList);
    }
    this.setState({
      list: scoreRows,
      rows,
      listLoading: false,
      currentPage: newCurrentPage,
      pageSize: newPageSize,
      searchkeyList,
      totalRecords: records,
    });
    return scoreRows;
  };
  /**
   * 获取最近五次数据
   * @returns {Promise.<void>}
   */
  getNearlyFiveData = async (nearlyFiveIdList, nearlyFiveTimeList) => {
    let { count } = this.state;
    let { studentId } = this.props;
    let fiveData = [],
      array = [];
    let data = await Promise.all(
      nearlyFiveIdList.map((v, k) => {
        return getSubjectByScoreID({ achievementId: v });
      }),
    );
    data.forEach((v, k) => {
      array.push({ [nearlyFiveTimeList[k]]: v });
      fiveData.push(v);
    });
    let comparedData = await getComparedDetailsAuto({ studentId });
    let cardListData = comparedData ? comparedData : {};
    let { id } = cardListData;

    this.setState({
      nearlyFiveData: array,
      nearlyFiveTimeList,
      cardListData,
      comparedId: id,
      count: count + 10,
      fiveData,
      nearlyFiveIdList,
    });
  };
  /**
   * 编辑成绩
   * @param record
   * @param index
   */
  editData = (record, index) => {
    let { id } = record;
    let { rows } = this.state;
    let records = rows[index];
    this.setState({ inputModalVisible: true, achievementId: id, records });
  };

  /**
   * 删除成绩
   * @param record
   */
  deleteData = (record) => {
    let { id } = record;
    deleteScore({ achievementId: id }).then((res) => {
      showMessage('删除成功', 'success');
      this.loadData();
    });
  };
  /**
   * 查询
   */
  search = () => {
    let { searchKey, year, term, examType, examDate } = this.state;
    this.loadData({ examinationDate: examDate, examinationTypeId: examType, schoolYearId: year, semester: term });
  };
  /**
   * 重置查询条件
   */
  reset = () => {
    this.setState({ year: null, term: null, examType: null, examDate: null });
  };
  /**
   * 新增录入成绩
   */
  add = () => {
    this.setState({ inputModalVisible: true, records: null, achievementId: null });
  };

  /**
   * 设置基准和对比成绩
   */
  setCompared = () => {
    this.setState({ settingModalVisible: true });
  };

  /**
   * 渲染展开函数
   * @param record {object} 对应行数据对象
   * @returns {XML}
   */
  expandedRowRender = (record) => {
    let { id, template, expandData } = record;
    let { baseSubjectId, baseScoreType } = this.state;
    return (
      <div>
        {template && (
          <div className={'CardList-expand-wrap'}>
            {((expandData.length > 0 && !expandData[0].evaluationType) || expandData.length === 0) && (
              <div style={{ overflow: 'hidden', paddingBottom: 15 }} className={localStyles.scoreContent}>
                <span className={localStyles.columns + ' ' + 'type'}>
                  <div>分数</div>
                  <div>班名</div>
                  <div>年名</div>
                </span>
                {expandData.map((v, k) => {
                  return (
                    <span
                      key={k}
                      className={localStyles.columns + ' ' + (baseSubjectId.includes(v.id) ? 'checked' : 'unChecked')}
                    >
                      <div>{v.subjectName}</div>
                      <div>
                        <span
                          className={
                            baseScoreType[baseSubjectId.indexOf(v.id)] === 'score' ? 'subChecked' : 'subUnChecked'
                          }
                        >
                          {v.score}
                        </span>
                        <span>{'/' + v.totalScore}</span>
                      </div>
                      <div>
                        <span
                          className={
                            baseScoreType[baseSubjectId.indexOf(v.id)] === 'classRanking'
                              ? 'subChecked'
                              : 'subUnChecked'
                          }
                        >
                          {v.classRanking}
                        </span>
                      </div>
                      <div>
                        <span
                          className={
                            baseScoreType[baseSubjectId.indexOf(v.id)] === 'gradeRanking'
                              ? 'subChecked'
                              : 'subUnChecked'
                          }
                        >
                          {v.gradeRanking}
                        </span>
                      </div>
                    </span>
                  );
                })}
                <span
                  className={localStyles.totalColumns + ' ' + (expandData.length === 0 ? 'noLeftLine' : 'leftLine')}
                >
                  <div>总分</div>
                  <div>
                    <span>{record.totalScore}</span>
                  </div>
                  <div>
                    <span>{record.totalClassRanking}</span>
                  </div>
                  <div>
                    <span>{record.totalGradeRanking}</span>
                  </div>
                </span>
              </div>
            )}
            {expandData.length > 0 && expandData[0].evaluationType && (
              <div style={{ overflow: 'hidden', paddingBottom: 15 }} className={localStyles.scoreContent}>
                <span className={localStyles.columns + ' ' + 'type'}>
                  <div style={{ marginTop: 10 }}>评级</div>
                </span>

                {expandData.map((v, k) => {
                  return (
                    <span
                      key={k}
                      className={localStyles.columns + ' ' + (baseSubjectId.includes(v.id) ? 'checked' : 'unChecked')}
                    >
                      <div>
                        <span>{v.subjectName}</span>
                      </div>
                      <div>
                        <span
                          className={
                            baseScoreType[baseSubjectId.indexOf(v.id)] === 'evaluationType'
                              ? 'subChecked'
                              : 'subUnChecked'
                          }
                        >
                          {v.evaluationType}
                        </span>
                      </div>
                    </span>
                  );
                })}
              </div>
            )}
          </div>
        )}
      </div>
    );
  };

  /**
   * 查看成绩凭证
   * @param src {string} 图片地址
   */
  showScorePic = (src) => {
    this.setState({ bigPhotoModalVisible: true, src });
  };

  /**
   * 展开的行变化时触发
   * @param expandedRow {array} 展开行的索引
   */
  onExpandedRowsChange = (expandedRow) => {
    this.setState({ expandedRowKeys: expandedRow });
  };
  /**
   * 点击展开图标时触发
   * @param expanded {boolean} 是否展开
   * @param record {object} 当前条目
   * @returns {Promise.<void>}
   */
  onExpand = async (expanded, record) => {
    if (expanded) {
      let { id, key, template } = record;
      let { list } = this.state;
      if (!template) {
        await this.getSubjectAndSetTemplate(id, list, key);
        this.setState({ list });
      }
    }
  };

  getSubjectAndSetTemplate = async (id, list, key) => {
    let data = await getSubjectByScoreID({ achievementId: id });
    list[key].expandData = data || [];
    list[key].template = true;
  };

  /**
   * 设置前端展示 请求数据
   * @param id 对比成绩编号
   * @returns {Promise.<void>}
   */
  showComparedDetails = async (id) => {
    let data = await getComparedDetails({ comparisonId: id });
    let { count } = this.state;
    let { id: comparedId } = data;
    showMessage('设置成功', 'success');
    this.setState({ cardListData: data, comparedId, settingModalVisible: false, count: count + 10 });
  };

  /**
   * 成绩修改录入保存后的回调 不管当前行是展开还是收起 修改后都展开
   * @param reload {string}
   */
  onClose = async (reload) => {
    let { records, expandedRowKeys } = this.state;
    if (reload === 'reload') {
      let list = await this.loadData();
      if (records) {
        list.forEach(async (v, k) => {
          if (v.id === records.id) {
            await this.getSubjectAndSetTemplate(records.id, list, k);
            if (!expandedRowKeys.includes(k)) {
              expandedRowKeys.push(k);
            }
          }
          this.setState({ list, expandedRowKeys });
        });
      }
    }
    this.setState({ inputModalVisible: false });
  };

  render() {
    let {
      searchExpand,
      loading,
      listLoading,
      totalRecords,
      currentPage,
      pageSize,
      yearDict,
      termDict,
      examTypeDict,
      year,
      term,
      examType,
      examDate,
      inputModalVisible,
      settingModalVisible,
      bigPhotoModalVisible,
      src,
      list,
      rows,
      achievementId,
      records,
      cardListData,
      nearlyFiveData,
      nearlyFiveTimeList,
      count,
      gradeId,
      cityId,
      comparedId,
      expandedRowKey,
      expandedRowKeys,
    } = this.state;
    let { studentId, btnAuths, type, headTeacherName, studyManageName, studentName } = this.props;
    //新增 编辑 删除按钮是否显示
    const scoreBtnPremission = premissionBtnKeyMap[scoreType[type]];
    let columns = [
      { title: '学年', dataIndex: 'year', width: COL_WIDTH_M, key: 'year' },
      { title: '学期', dataIndex: 'term', width: COL_WIDTH_M, key: 'term' },
      { title: '考试类型', dataIndex: 'examType', width: COL_WIDTH_L, key: 'examType' },
      { title: '城市', dataIndex: 'city', width: COL_WIDTH_L, key: 'city' },
      { title: '年级', dataIndex: 'grade', width: COL_WIDTH_L, key: 'grade' },
      { title: '考试时间', dataIndex: 'date', width: COL_WIDTH_L, key: 'date' },
      {
        title: '创建时间',
        dataIndex: 'createTime',
        width: COL_WIDTH_L,
        key: 'createTime',
        render(currentValue) {
          return moment(currentValue).format('YYYY-MM-DD');
        },
      },
      { title: '成绩类型', dataIndex: 'achievementTypeList', width: COL_WIDTH_L, key: 'achievementTypeList' },
      {
        title: '科目统计率',
        dataIndex: 'countRate',
        key: 'countRate',
        width: COL_WIDTH_L,
        render: (text, record, idx) => {
          let { countRate } = record;
          return <span>{countRate + '%'}</span>;
        },
      },
      {
        title: '成绩凭证',
        dataIndex: 'achievementEvidence',
        key: 'achievementEvidence',
        width: COL_WIDTH_L,
        render: (text, record, index) => {
          let { achievementEvidence } = record;
          if (achievementEvidence) {
            return (
              <div className={'fixImgParent'} style={{ position: 'relative' }}>
                <img
                  src={achievementEvidence}
                  alt=""
                  style={{ maxWidth: 28, maxHeight: 20, verticalAlign: 'middle', cursor: 'pointer' }}
                />
                <TextLink
                  href="#"
                  onClick={(e) => this.showScorePic(achievementEvidence)}
                  style={{ marginLeft: 10, position: 'absolute', left: 30, top: 3 }}
                >
                  查看
                </TextLink>
              </div>
            );
          } else {
            return <span>-</span>;
          }
        },
      },
      {
        title: '操作',
        key: 'action',
        render: (text, record, index) => {
          let { id } = record;
          return (
            <span>
              {btnAuths.includes(scoreBtnPremission.edit) && (
                <TextLink className="list-btn" onClick={() => this.editData(record, index)}>
                  编辑
                </TextLink>
              )}
              <PopConfirm
                content="该操作可能删除对比记录，确定删除吗？"
                okBtnText="是"
                cancelBtnText="否"
                onOk={(e) => this.deleteData(record)}
              >
                {btnAuths.includes(scoreBtnPremission.delete) && <TextLink className="list-btn">删除</TextLink>}
              </PopConfirm>
            </span>
          );
        },
      },
    ];
    return (
      <Page loading={loading} title={false} route={this.props.route} className={localStyles.scoreManage}>
        <TopInfo
          type={type}
          headTeacherName={headTeacherName}
          studyManageName={studyManageName}
          studentName={studentName}
        />
        <CompareCard
          onSettingClick={this.setCompared}
          nearlyFiveTimeList={nearlyFiveTimeList}
          cardListData={cardListData}
          count={count}
          studentId={studentId}
          btnAuths={btnAuths}
          checkBaseSubjectId={(val) => {
            this.setState(val);
          }}
          nearlyFiveData={nearlyFiveData}
        />
        <div className={styles.wrap} style={{ marginBottom: 10 }}>
          <div className={localStyles.title}>成绩单查询</div>
          <div className={localStyles.scoreQueryContent}>
            <div className={localStyles.scoreQueryContentDetail}>
              <div className={localStyles.scoreQueryQRCodeWrapper}>
                <div className={localStyles.scoreQueryQRCode}>
                  <img src={!isProductionNodeEnv ? testQrcodeImage : qrcodeImage} width="100%" height="100%" />
                </div>
              </div>
              <div className={localStyles.scoreQueryDesc}>
                <p className={localStyles.scoreQueryDescTitle}>查询方式：</p>
                <p className={localStyles.scoreQueryDescContent}>
                  将二维码发送给家长，让家长填写【学生编号】及【学生姓名】后查询
                </p>
                <p className={localStyles.scoreQueryDescWarn}>
                  注意：请给对应的家长提供正确的学生编号及姓名，学生编号与姓名不匹配则无法查询
                </p>
              </div>
            </div>
          </div>
        </div>
        <div className={styles.wrap}>
          <div className={localStyles.title}>成绩列表</div>
          {btnAuths.includes(scoreBtnPremission.add) && (
            <Button onClick={() => this.add()} className={localStyles.newAdd}>
              新 增
            </Button>
          )}
          {/* 此页面的.flexWrap无需修改 */}
          <div className="flexWrap">
            <div className={styles.searchBar}>
              <SearchBar
                onSearch={this.search}
                onReset={this.reset}
                onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
                expand={searchExpand}
              >
                <Select
                  placeholder="学年"
                  dict={yearDict}
                  value={year}
                  onChange={(val) => this.setState({ year: val })}
                />
                <Select
                  placeholder="学期"
                  dict={termDict}
                  value={term}
                  onChange={(val) => this.setState({ term: val })}
                />
                <Select
                  placeholder="考试类型"
                  dict={examTypeDict}
                  value={examType}
                  onChange={(val) => this.setState({ examType: val })}
                />

                <DatePicker
                  placeholder="考试时间"
                  value={examDate}
                  onChange={(val) => this.setState({ examDate: val })}
                />
              </SearchBar>
            </div>

            <div className="flexInnerFit scoreList">
              <CardList
                dataSource={list}
                columns={columns}
                loading={!loading && listLoading}
                expandedRowRender={(record) => this.expandedRowRender(record)}
                pagination={{
                  current: currentPage,
                  total: totalRecords,
                  pageSize,
                }}
                expandedRowKeys={expandedRowKeys}
                onExpandedRowsChange={(expandedRows) => {
                  this.onExpandedRowsChange(expandedRows);
                }}
                onChange={({ current, pageSize }) => this.loadData({ page: current, rows: pageSize })}
                onExpand={(expanded, record) => {
                  this.onExpand(expanded, record);
                }}
              />
            </div>
          </div>
          {inputModalVisible && (
            <InputViewModal
              name={name}
              yearDict={yearDict}
              termDict={termDict}
              examTypeDict={examTypeDict}
              achievementId={achievementId}
              record={records}
              cityId={cityId}
              gradeId={gradeId}
              studentId={studentId}
              onClose={this.onClose}
            />
          )}

          {settingModalVisible && (
            <SettingViewModal
              name={name}
              yearDict={yearDict}
              termDict={termDict}
              list={rows}
              studentId={studentId}
              comparedId={comparedId}
              showComparedDetails={this.showComparedDetails}
              onClose={() => this.setState({ settingModalVisible: false })}
            />
          )}
          {bigPhotoModalVisible && (
            <BigPhotoViewModel src={src} onClose={(e) => this.setState({ bigPhotoModalVisible: false })} />
          )}
        </div>
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  // console.log(state.main.pageParams['/scoreManage']);
  return {
    studentId: state.main.pageParams['/scoreManage'].studentId,
    btnAuths: state.account.btnAuths,
    studentName: state.main.pageParams['/scoreManage'].studentName,
    studyManageName: state.main.pageParams['/scoreManage'].studyManagerName,
    headTeacherName: state.main.pageParams['/scoreManage'].headTeacherName,
    type: state.main.pageParams['/scoreManage'].type,
  };
};
export default connect(mapStateToProps)(GradeManage);
