import React from 'react';
import Modal from 'COMPONENT/Modal';
import Block from 'COMPONENT/Block';
import Select from 'COMPONENT/Select';
// import { DatePicker } from 'antd'
import DatePicker from 'COMPONENT/DatePicker';
import FormItem from 'COMPONENT/FormItem';
import IconButton from 'COMPONENT/IconButton';
import Popover from 'COMPONENT/Popover';
import PopConfirm from 'COMPONENT/PopConfirm';
import Loading from 'COMPONENT/Loading';
import InputNumber from 'COMPONENT/InputNumber';
import QrcodeFileUpLoad from 'COMPONENT/QrcodeFileUpload';
import TextLink from 'COMPONENT/TextLink';
import styles from './styles.less';
import { Row, Col, Input, Icon, Radio } from 'antd';
import totalSvg from 'COMPONENT/assets/svgs/symbol-defs.svg';
import noDataSvg from 'ASSET/imgs/no_data.svg';
import moment from 'moment';

import {
  getTemplate,
  saveNewScore,
  getSelectOption,
  getSubjectByScoreID,
  getCityAndGradeById,
  checkScoreHasChange,
  deleteEvidence,
  getTemplateById,
  getSchoolYearRange,
} from 'SERVICE/scoreManage';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { loadProvinceCityDict, mapDataToDict } from 'UTIL/dict/index';

import { getDictTextByVal, loadCommonDict } from 'UTIL/dict';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';

const RadioButton = Radio.Button;
const RadioGroup = Radio.Group;
// 不需要的年级数据(幼儿园、毕业)
const notAllowGrade = ['DAT0000000393', 'DAT0000000409'];

const semesterDict = {};

let getCurrentYearAndTerm = () => {
  let schoolYear;
  let term;
  let date = new Date();
  let year = date.getFullYear();
  let month = date.getMonth() + 1;
  let day = date.getDate();

  if (month < 9) {
    schoolYear = year - 1 + '-' + year;
    if ((month > 3 || (month === 3 && day >= 2)) && (month < 9 || (month === 9 && day <= 1))) {
      term = 'NEXT_SEMESTER';
    } else {
      term = 'LAST_SEMESTER';
    }
  } else {
    schoolYear = year + '-' + (year + 1);
    if ((month > 3 || (month === 3 && day >= 2)) && (month < 9 || (month === 9 && day <= 1))) {
      term = 'NEXT_SEMESTER';
    } else {
      term = 'LAST_SEMESTER';
    }
  }
  return { schoolYear, term };
};

class ModelView extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      totalClassRanking: null, //{string} 总分班级排名
      totalEvaluationTypeValue: null, //{string} 总分评级
      totalGradeRanking: null, //{string} 总分年级排名
      totalScore: null, //{string} 总分
      loading: true, //{boolean} 加载动画
      cityDict: null, //{array} 城市字典
      gradeDict: null, //{array} 年级字典
      scoreTypeDict: null,

      score: null,
      subjects: [],
      EvaluationVal: [],
      readyCountSubject: [], //{array} 已统计科目
      unCountSubject: [], //{array} 可统计科目
      overTipArray: [],

      isInitTemplate: false, //
      showTipForDatePicker: false, // 是否显示DatePicker disabled时的气泡弹窗提示
      beginDate: null, // 限制可选日期的开始日期
      endDate: null, // 限制可选日期的结束日期
      defaultDateVal: null, // 默认日期
    };
  }

  componentDidMount() {
    let { achievementId, record, yearDict, termDict } = this.props;
    let { readyCountSubject } = this.state;
    if (achievementId) {
      let {
        schoolYearId,
        semesterValue,
        examinationTypeId,
        cityId,
        gradeId,
        examinationDate,
        achievementTypeValue,
        achievementTemplateId,
        achievementTypeList,
        totalClassRanking,
        totalEvaluationTypeValue: totalEvaluationType,
        totalGradeRanking,
        totalScore,
        achievementEvidence,
      } = record;
      //编辑时加载数据
      Promise.all([
        getSubjectByScoreID({ achievementId: achievementId }),
        getSelectOption({ selectOptionCategory: 'AchievementAbstractType' }),
        loadCommonDict('STUDENT_GRADE'), //加载年级字典
        loadProvinceCityDict(), //加载城市列表
        getTemplateById({ templateId: achievementTemplateId }), //根据模板编号查询模板数据
        this.updateYearRange({ schoolYearId, semesterValue, examinationTypeId }),
      ]).then(([res, { value: scoreDict }, gradeDict, cityDict, templateData]) => {
        let scoreDictarr = mapDataToDict(scoreDict);
        let typeListArray = achievementTypeValue.split(',');
        let scoreTypeDict = [];
        scoreDictarr.forEach((v, k) => {
          if (typeListArray.includes(v.val)) {
            scoreTypeDict.push(v);
          }
        });
        let clearGradeDict = gradeDict.filter((item) => notAllowGrade.indexOf(item.val) === -1);

        let totalSubject = [];
        templateData.forEach((v, k) => {
          if (v.gradeId === gradeId) {
            totalSubject = v.subjects;
          }
        });
        for (let i = 0; i < res.length; i++) {
          for (let j = 0; j < totalSubject.length; j++) {
            if (totalSubject[j].subjectId == res[i].subjectId) {
              totalSubject.splice(j, 1);
              break;
            }
          }
        }
        this.setState({
          readyCountSubject: res || [],
          schoolYearId,
          semesterValue,
          examinationTypeId,
          cityId,
          gradeId,
          examinationDate,
          achievementTypeValue,
          achievementTemplateId,
          achievementTypeList,
          scoreDict: scoreTypeDict,
          achievementEvidence,
          gradeDict: clearGradeDict,
          cityDict,
          allScoreDict: scoreDictarr,
          totalClassRanking,
          totalEvaluationType,
          totalGradeRanking,
          totalScore,
          loading: false,
          filePath: achievementEvidence,
          subjects: totalSubject,
          defaultDateVal: examinationDate,
          isInitTemplate: true,
        });
      });
    } else {
      this.loadTemplateData();
    }
  }

  /**
   * 新建时加载模板数据
   * @returns {Promise.<void>}
   */
  loadTemplateData = async () => {
    let { cityId, gradeId, yearDict, termDict } = this.props;
    let currentTime = getCurrentYearAndTerm();
    let semesterValue = currentTime.term;
    let schoolYearId;
    let schoolYearObj = yearDict.filter((v, k) => {
      return v.text === currentTime.schoolYear;
    })[0];
    if (schoolYearObj) {
      schoolYearId = schoolYearObj.val;
    }
    let copyCityId = JSON.parse(JSON.stringify(cityId));
    let copyGradeId = JSON.parse(JSON.stringify(gradeId));

    Promise.all([
      getSelectOption({ selectOptionCategory: 'AchievementAbstractType' }),
      loadCommonDict('STUDENT_GRADE'), //加载年级字典
      loadProvinceCityDict(), //加载城市列表
    ]).then(([{ value: scoreDict }, gradeDict, cityDict]) => {
      let clearGradeDict = gradeDict.filter((item) => notAllowGrade.indexOf(item.val) === -1);
      let scoreDictarr = mapDataToDict(scoreDict);
      // let examinationDate = moment().format('YYYY-MM-DD');
      this.setState({
        scoreDict: scoreDictarr,
        gradeDict: clearGradeDict,
        cityDict,
        allScoreDict: scoreDictarr,
        loading: false,
        semesterValue,
        schoolYearId,
        gradeId: copyGradeId,
        cityId: copyCityId,
        // examinationDate,
      });
      // if (cityId && examinationDate && gradeId) {
      //   let apiParams = {cityId, examinationDate, gradeId};
      //   getTemplate(apiParams).then((apiData) => {
      //     if (apiData) {
      //       let {achievementTemplateId, achievementTypeList, gradeId, gradeName, id, subjects} = apiData;
      //       let typeListArray = achievementTypeList.split(',');
      //       let scoreTypeDict = [];
      //       scoreDictarr.forEach((v, k) => {
      //         if (typeListArray.includes(v.val)) {
      //           scoreTypeDict.push(v);
      //         }
      //       });
      //       //自动选中模板中所设置的成绩类型
      //       let achievementTypeValue;
      //       if (scoreTypeDict.length > 0) {
      //         achievementTypeValue = scoreTypeDict[0].val;
      //       }
      //       this.setState({
      //         achievementTemplateId,
      //         achievementTypeList,
      //         scoreDict: scoreTypeDict,
      //         achievementTypeValue,
      //         gradeName,
      //         id,
      //         subjects,
      //       })
      //     }
      //   })
      // }
    });
  };

  /**
   * 城市，年级，考试时间变化时加载模板
   * @param value
   */
  loadTemplateBySelect = (changeKey, type) => {
    let { allScoreDict, readyCountSubject } = this.state;
    changeKey = { ...changeKey };
    this.setState(changeKey, () => {
      let { cityId, gradeId, examinationDate } = this.state;
      if (cityId && gradeId && examinationDate) {
        this.setState({ loading: true });
        let apiParams = { cityId, examinationDate, gradeId };
        getTemplate(apiParams).then(
          (res) => {
            let { achievementTemplateId, achievementTypeList, gradeId, gradeName, id, subjects } = res;
            let scoreTypeDict = [];
            if (res) {
              let typeListArray = achievementTypeList.split(',');
              allScoreDict.forEach((v, k) => {
                if (typeListArray.includes(v.val)) {
                  scoreTypeDict.push(v);
                }
              });
            }
            let achievementTypeValue;
            if (scoreTypeDict.length > 0) {
              achievementTypeValue = scoreTypeDict[0].val;
            }
            if (readyCountSubject.length !== 0) {
              for (let i = 0; i < readyCountSubject.length; i++) {
                subjects = subjects.filter((subject) => {
                  return subject.subjectId !== readyCountSubject[i].subjectId;
                });
              }
            }
            if (type === 'examinationDate') {
              this.setState({
                achievementTemplateId,
                achievementTypeList,
                gradeName,
                id,
                subjects,
                // readyCountSubject: [],
                scoreDict: scoreTypeDict,
                achievementTypeValue,
                loading: false,
                isInitTemplate: true,
              });
            } else {
              this.setState({
                achievementTemplateId,
                achievementTypeList,
                gradeName,
                id,
                subjects,
                readyCountSubject: [],
                scoreDict: scoreTypeDict,
                achievementTypeValue,
                loading: false,
                isInitTemplate: true,
              });
            }
          },
          (err) => {
            this.setState({
              isInitTemplate: true,
            });
          },
        );
      }
    });
  };

  changeAchievementType = (val) => {
    let { readyCountSubject } = this.state;
    if (val === 'SCORE_RANKING') {
      readyCountSubject.forEach((v, k) => {
        v.evaluationType = null;
      });
    } else {
      readyCountSubject.forEach((v, k) => {
        v.classRanking = null;
        v.gradeRanking = null;
        v.score = null;
      });
    }
    this.setState({ readyCountSubject, achievementTypeValue: val });
  };
  /**
   * 点击保存按钮
   */
  onOk = () => {
    let { record, onClose, achievementId, studentId } = this.props;
    let version;
    if (record) {
      version = record.version;
    }
    let {
      readyCountSubject,
      achievementTemplateId,
      achievementTypeList,
      examinationTypeId,
      schoolYearId,
      semesterValue,
      cityId,
      gradeId,
      examinationDate,
      achievementTypeValue,
      achievementEvidence,
      filePath,
      totalClassRanking,
      totalEvaluationType,
      totalGradeRanking,
      totalScore,
    } = this.state;
    let len = readyCountSubject.length;
    for (let i = 0; i < len; i++) {
      if (readyCountSubject[i]['score'] == undefined && readyCountSubject[i]['evaluationType'] == undefined) {
        showMessage(`${readyCountSubject[i]['subjectName']}成绩未填写，请填写完整再提交`);
        return;
      }
    }
    if (readyCountSubject.length === 0 && !totalScore) {
      showMessage('您还没有录入任何一门成绩！');
      return;
    }
    let overTipArray = [];
    let showWarning;
    readyCountSubject.forEach((v, k) => {
      if (v.score > v.totalScore) {
        showWarning = true;
        overTipArray.push(true);
      } else {
        overTipArray.push(false);
      }
    });
    if (showWarning) {
      this.setState({ overTipArray });
      showMessage('有科目分数超出了总分限制，请修改');
      return;
    }
    // totalScore
    let subjectSet = readyCountSubject.map((v, k) => {
      return {
        classRanking: v.classRanking,
        score: achievementTypeValue === 'SCORE_RANKING' ? v.score : '',
        subjectId: v.subjectId,
        gradeRanking: v.gradeRanking,
        evaluationType: achievementTypeValue === 'GRADE' ? v.evaluationType : null,
      };
    });
    if (!cityId) {
      showMessage('请选择城市');
      return;
    }

    if (!schoolYearId) {
      showMessage('请选择学年');
      return;
    }
    if (!semesterValue) {
      showMessage('请选择学期');
      return;
    }
    if (!gradeId) {
      showMessage('请选择年级');
      return;
    }
    if (!examinationDate) {
      showMessage('请选择考试时间');
      return;
    }
    if (!examinationTypeId) {
      showMessage('请选择考试类型');
      return;
    }
    let apiParams = {
      id: achievementId ? achievementId : null,
      achievementTemplateId: achievementTemplateId, //成绩模板编号
      achievementType: achievementTypeValue, //成绩类型
      cityId: cityId, //城市编号
      examinationDate: examinationDate, //考试时间
      examinationTypeId: examinationTypeId, //考试类型
      gradeId: gradeId, //年级编号
      schoolYearId: schoolYearId, //学年编号
      semester: semesterValue, //学期
      // studentId: 'STU171027000001',//学生id
      studentId: studentId, //学生id
      subjectSet: subjectSet,
      version: version ? version : 0, //版本
      achievementEvidence: filePath ? achievementEvidence : null, //成绩凭证url
      totalClassRanking, //总班级排名
      totalEvaluationType, //总评级
      totalGradeRanking, //总年级排名
      totalScore, //总分
    };
    if (achievementId) {
      checkScoreHasChange(apiParams).then((res) => {
        if (res) {
          showConfirm(`该操作可能影响成绩数据分析结果，确定保存吗？`, () => {
            this.saveData(apiParams);
          });
        } else {
          this.saveData(apiParams);
        }
      });
    } else {
      this.saveData(apiParams);
    }
  };

  /**
   * 保存录入成绩
   * @param apiParams
   */
  saveData = (apiParams) => {
    let { onClose } = this.props;
    this.setState({ loading: true });
    saveNewScore(apiParams)
      .then((res) => {
        this.setState({ loading: false });
        showMessage('保存成功', 'success');
        onClose('reload');
      })
      .catch((res) => {
        this.setState({ loading: false });
      });
  };

  /**
   * 点击可统计科目的‘+’按钮
   * @param v
   * @param k
   */
  onClick = (v, k) => {
    let { subjects, readyCountSubject, achievementTypeValue } = this.state;
    if (!achievementTypeValue) {
      showMessage('请选择成绩类型!');
      return;
    }
    subjects.splice(k, 1);
    readyCountSubject.push(v);
    this.setState({ subjects, readyCountSubject });
  };
  /**
   * 点击以统计科目的‘-’按钮
   * @param value {object} 科目对象
   * @param key {number} 索引
   */
  cancelSubject = (value, key) => {
    let { subjects, readyCountSubject, achievementTypeValue } = this.state;
    if (achievementTypeValue === 'SCORE_RANKING') {
      value.classRanking = null;
      value.gradeRanking = null;
      value.score = null;
    } else {
      value.evaluationType = null;
    }
    readyCountSubject.splice(key, 1);
    if (readyCountSubject.length === 0) {
      this.setState({ totalScore: null, totalClassRanking: null, totalGradeRanking: null });
    }
    subjects.push(value);
    this.setState({ subjects, readyCountSubject });
  };
  /**
   * 填写成绩
   * @param k {number} 成绩列表索引
   * @param key {string} 成绩项 <score,classRanking....>
   * @param value {string} 成绩值
   */
  changeData = (k, key, value, total) => {
    let { readyCountSubject, overTipArray, totalScore } = this.state;
    readyCountSubject[k][key] = value;

    if (total) {
      totalScore = 0;
      readyCountSubject.forEach((v, k) => {
        totalScore += v.score || 0;
      });
    }
    if (key === 'score' && value <= total) {
      overTipArray[k] = false;
      this.setState({ overTipArray });
    }
    this.setState({ readyCountSubject, totalScore });
  };
  /**
   * 扫码上传图片回调
   * @param fileInfo
   */
  getFileInfo = (fileInfo) => {
    let filePath = fileInfo.filePath;
    this.setState({ filePath, achievementEvidence: filePath });
  };

  deletePhoto = () => {
    this.setState({ loading: true });
    this.setState({ filePath: null, loading: false });
  };

  /**
   * 20180906修复bug
   * */
  updateContents = async (params) => {
    const { type, val } = params;
    console.log(type, val);
    await this.setState((prevState) => ({ [type]: val }));
    switch (type) {
      case 'schoolYearId':
      case 'semesterValue':
        await this.setState((prevState) => ({ examinationDate: null, defaultDateVal: null }));
        this.checkNeedUpdate();
        break;
      case 'examinationTypeId':
        await this.setState((prevState) => ({ examinationDate: null, defaultDateVal: null }));
        this.checkNeedUpdate();
        break;
    }
  };
  checkNeedUpdate = () => {
    const { schoolYearId, semesterValue, examinationTypeId } = this.state;
    if (schoolYearId && semesterValue && examinationTypeId) {
      this.updateYearRange({
        schoolYearId,
        semesterValue,
        examinationTypeId,
      });
    }
  };
  updateYearRange = async (params) => {
    await this.setState((prevState) => ({ loading: true }));
    const { schoolYearId, semesterValue, examinationTypeId } = params;
    const { yearDict } = this.props;
    let yearRangeArr = getDictTextByVal(yearDict, schoolYearId).split('-');
    let beginY = yearRangeArr[0];
    let endY = yearRangeArr[1];
    let apiParams = {
      startYear: beginY,
      finishYear: endY,
      semester: semesterValue,
      examinationType: examinationTypeId,
    };
    getSchoolYearRange(apiParams, true).then(
      (apiDate) => {
        const { data } = apiDate;
        this.setState((prevState) => ({
          loading: false,
          beginDate: data ? data.resultStartDate : null,
          endDate: data ? data.resultEndDate : null,
        }));
      },
      (err) => {
        this.setState((prevState) => ({ loading: false }));
        showMessage(err.text, 'error');
      },
    );
  };
  /**
   * 获取要禁用的结束日期值
   * @param  {string} currentDate   过滤前的日期值
   * @return {boolean}      是否要禁用
   */
  getDisabledDate = (currentDate) => {
    const { beginDate, endDate } = this.state;
    const nowMoment = moment(new Date()).valueOf();
    const currentMoment = moment(currentDate).valueOf();
    if (!beginDate || !endDate) {
      return currentMoment > nowMoment;
    }
    const beginMoment = moment(beginDate).valueOf();
    const endMoment = moment(endDate).valueOf();
    if (beginMoment < nowMoment) {
      if (endMoment < nowMoment) {
        return currentMoment < beginMoment || currentMoment > endMoment;
      } else {
        return currentMoment < beginMoment || currentMoment > nowMoment;
      }
    } else {
      return true;
    }
  };

  render() {
    let { yearDict, termDict, examTypeDict, onClose, achievementId } = this.props;
    let {
      loading,
      schoolYearId,
      semesterValue,
      examinationTypeId,
      cityId,
      gradeId,
      examinationDate,
      achievementTypeValue,
      scoreDict,
      cityDict,
      gradeDict,
      scoreTypeDict,
      score,
      subjects,
      achievementTypeList,
      achievementTemplateId,
      EvaluationVal,
      readyCountSubject,
      unCountSubject,
      filePath,
      totalClassRanking,
      totalEvaluationType,
      totalGradeRanking,
      totalScore,
      overTipArray,
      showTipForDatePicker,
      defaultDateVal,
      isInitTemplate,
    } = this.state;

    let canChooseDate = schoolYearId && semesterValue && examinationTypeId;
    // let canChooseDate = true;

    return (
      <Modal
        loading={loading}
        onCancel={onClose}
        onOk={this.onOk}
        title="成绩录入"
        width={1218}
        okBtnShow={true}
        cancelText="取消"
        okText="保存"
        className={styles.modal}
      >
        <Block title={name} style={{ marginTop: -10 }}>
          <div className={styles.header}>
            <Row gutter={10}>
              <Col span={3}>学年</Col>
              <Col span={3}>学期</Col>
              <Col span={3}>考试类型</Col>
              <Col span={3}>城市</Col>
              <Col span={3}>年级</Col>
              <Col span={3}>考试时间</Col>
              <Col span={3}>成绩类型</Col>
              <Col span={3}>成绩凭证</Col>
            </Row>
          </div>
          <div className={styles.content}>
            <Row gutter={10}>
              <Col span={3}>
                <Select
                  size={'small'}
                  placeholder="学年"
                  dict={yearDict}
                  value={schoolYearId}
                  onChange={(val) => this.updateContents({ type: 'schoolYearId', val: val })}
                />
              </Col>
              <Col span={3}>
                <Select
                  size={'small'}
                  placeholder="学期"
                  dict={termDict}
                  value={semesterValue}
                  onChange={(val) => this.updateContents({ type: 'semesterValue', val: val })}
                />
              </Col>
              <Col span={3}>
                <Select
                  size={'small'}
                  placeholder="考试类型"
                  dict={examTypeDict}
                  value={examinationTypeId}
                  onChange={(val) => this.updateContents({ type: 'examinationTypeId', val: val })}
                />
              </Col>
              <Col span={3}>
                <Select
                  showSearch={true}
                  size={'small'}
                  placeholder="城市"
                  dict={cityDict}
                  value={cityId}
                  onChange={(val) => this.loadTemplateBySelect({ cityId: val })}
                />
              </Col>
              <Col span={3}>
                <Select
                  size={'small'}
                  placeholder="年级"
                  dict={gradeDict}
                  value={gradeId}
                  onChange={(val) => this.loadTemplateBySelect({ gradeId: val })}
                />
              </Col>
              <Col span={3}>
                <Popover
                  visible={!canChooseDate && showTipForDatePicker}
                  content="请先选择“学年”、“学期”、“考试类型”后再进行操作"
                >
                  <div
                    onMouseEnter={() => {
                      this.setState((prevState) => ({ showTipForDatePicker: true }));
                    }}
                    onMouseLeave={() => {
                      this.setState((prevState) => ({ showTipForDatePicker: false }));
                    }}
                  >
                    <DatePicker
                      size={'small'}
                      placeholder="考试时间"
                      defaultValue={defaultDateVal}
                      value={examinationDate}
                      onChange={(val) => this.loadTemplateBySelect({ examinationDate: val }, 'examinationDate')}
                      disabledDate={this.getDisabledDate}
                      disabled={!canChooseDate}
                    />
                  </div>
                </Popover>
              </Col>
              <Col span={3}>
                <Select
                  size={'small'}
                  placeholder="成绩类型"
                  dict={scoreDict}
                  value={achievementTypeValue}
                  onChange={(val) => {
                    this.changeAchievementType(val);
                  }}
                />
              </Col>
              <Col span={3} className={'fixParent'} style={{ borderRight: 'none', paddingTop: 0 }}>
                {filePath && (
                  <Popover
                    content={<img src={filePath} style={{ maxHeight: 200, maxWidth: 200 }} />}
                    getPopupContainer={() => {
                      return document.querySelector('.fixParent');
                    }}
                  >
                    <img src={filePath} alt="" style={{ maxWidth: 28, maxHeight: 20, verticalAlign: 'middle' }} />
                  </Popover>
                )}

                <Popover
                  content={<QrcodeFileUpLoad height="160" width="160" callback={this.getFileInfo} />}
                  getPopupContainer={() => {
                    return document.querySelector('.fixParent');
                  }}
                >
                  <a className={'qrcode'}>{filePath ? '重新上传' : '上传图片'}</a>
                </Popover>
                {filePath && (
                  <PopConfirm
                    content="该操作将删除成绩凭证，是否继续？"
                    okBtnText="是"
                    cancelBtnText="否"
                    onOk={this.deletePhoto}
                  >
                    <a href="#">删除</a>
                  </PopConfirm>
                )}
              </Col>
            </Row>
          </div>

          <div className={styles.score}>
            {!achievementTemplateId && !achievementId && (
              <div style={{ minHeight: 260 }}>
                <div className={styles.nodata}></div>
                <p>{isInitTemplate ? '未找到成绩模板,请联系分公司教学人员' : '请先完善考试类型与考试时间'}</p>
              </div>
            )}
            {(achievementTemplateId || achievementId) && (
              <div>
                <Row gutter={20}>
                  <Col span={24} className={styles.unCount}>
                    可统计科目
                  </Col>
                  <Col span={24} className={styles.unCountItem}>
                    {subjects.map((v, k) => {
                      return (
                        <div key={k}>
                          <div>{v.subjectName}</div>
                          <IconButton
                            data-id={v.subjectId}
                            type="plus_circle"
                            color={'#666'}
                            style={{ top: 2, position: 'relative' }}
                            size={14}
                            onClick={(e) => {
                              this.onClick(v, k);
                            }}
                          />
                        </div>
                      );
                    })}
                    {subjects.length === 0 && (
                      <span style={{ display: 'inline-block', marginTop: 15, color: '#ccc' }}>无</span>
                    )}
                  </Col>
                </Row>
                <Row gutter={20}>
                  <Col span={24} className={styles.readyCount} style={{ marginBottom: 15 }}>
                    已统计科目
                  </Col>
                  <Col span={24} className={styles.readyCountItem} style={{ whiteSpace: 'normal' }}>
                    {readyCountSubject.map((v, k) => {
                      if (achievementTypeValue === 'SCORE_RANKING') {
                        return (
                          <span key={k}>
                            <div className={styles.subject}>
                              {v.subjectName}
                              <IconButton
                                type="minus_circle"
                                style={{ marginTop: 4, float: 'right' }}
                                color={'#666'}
                                size={14}
                                onClick={(e) => {
                                  this.cancelSubject(v, k);
                                }}
                              />
                            </div>
                            <div className={styles.scoreItem}>
                              <FormItem name="分数" required={true}>
                                <InputNumber
                                  size={'small'}
                                  placeholder={v.totalScore}
                                  value={v.score}
                                  min={0}
                                  // validator={}
                                  style={{ borderColor: overTipArray[k] ? 'red' : '' }}
                                  onChange={(e) => this.changeData(k, 'score', e, v.totalScore)}
                                />
                              </FormItem>
                              <FormItem name="班名">
                                <InputNumber
                                  size={'small'}
                                  placeholder=""
                                  value={v.classRanking}
                                  min={0}
                                  onChange={(e) => this.changeData(k, 'classRanking', e)}
                                />
                              </FormItem>
                              <FormItem name="年名">
                                <InputNumber
                                  size={'small'}
                                  placeholder=""
                                  value={v.gradeRanking}
                                  min={0}
                                  onChange={(e) => this.changeData(k, 'gradeRanking', e)}
                                />
                              </FormItem>
                            </div>
                          </span>
                        );
                      } else {
                        return (
                          <span key={k}>
                            <div className={styles.subject}>
                              {v.subjectName}
                              <IconButton
                                type="minus_circle"
                                style={{ marginTop: 4, float: 'right' }}
                                size={14}
                                color={'#666'}
                                onClick={(e) => {
                                  this.cancelSubject(v, k);
                                }}
                              />
                            </div>
                            <div className={styles.scoreItem}>
                              <FormItem name="评级" required={true}>
                                <RadioGroup
                                  defaultValue={v.evaluationType}
                                  onChange={(e) => this.changeData(k, 'evaluationType', e.target.value)}
                                >
                                  <RadioButton value="A">A</RadioButton>
                                  <RadioButton value="B">B</RadioButton>
                                  <RadioButton value="C">C</RadioButton>
                                  <RadioButton value="D">D</RadioButton>
                                  <RadioButton value="E">E</RadioButton>
                                </RadioGroup>
                              </FormItem>
                            </div>
                          </span>
                        );
                      }
                    })}
                    {achievementTypeValue === 'SCORE_RANKING' && (
                      <span
                        className={styles.totalScore}
                        style={{ marginLeft: readyCountSubject.length === 0 ? 0 : 21 }}
                      >
                        <div className={styles.subject}>{'总分'}</div>
                        <div className={styles.scoreItem}>
                          <FormItem name="分数" required={false}>
                            <InputNumber
                              size={'small'}
                              placeholder=""
                              value={totalScore}
                              min={0}
                              onChange={(e) => this.setState({ totalScore: e })}
                            />
                          </FormItem>
                          <FormItem name="班名">
                            <InputNumber
                              size={'small'}
                              placeholder=""
                              value={totalClassRanking}
                              min={0}
                              onChange={(e) => this.setState({ totalClassRanking: e })}
                            />
                          </FormItem>
                          <FormItem name="年名">
                            <InputNumber
                              size={'small'}
                              placeholder=""
                              value={totalGradeRanking}
                              min={0}
                              onChange={(e) => this.setState({ totalGradeRanking: e })}
                            />
                          </FormItem>
                        </div>
                      </span>
                    )}
                  </Col>
                </Row>
              </div>
            )}
          </div>
        </Block>
      </Modal>
    );
  }
}

export default ModelView;
