import React from 'react';
import { Radio } from 'antd';
import Modal from 'COMPONENT/Modal';
import Block from 'COMPONENT/Block';
import moment from 'moment';
import CardList from 'COMPONENT/List/CardList';
import IconButton from 'COMPONENT/IconButton';
import Select from 'COMPONENT/Select';
import PopConfirm from 'COMPONENT/PopConfirm';
import Button from 'COMPONENT/Button';
import styles from '../listPage.less';
import localStyles from './setting.less';
const { COL_WIDTH_L, COL_WIDTH_XL } = CardList;
const RadioGroup = Radio.Group;

import { getDictTextByVal, mapDataToDict } from 'UTIL/dict';
import getCurrentYearAndTerm from '../common';

import {
  getOTOSubiect,
  getSubjectSelected,
  getSubjectByScoreID,
  getSelectOption,
  getComparedListById,
  saveBaseAndCompared,
  getBaseScoreById,
  getComparedScoreById,
} from 'SERVICE/scoreManage';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

let mapObject = {
  CLASS_RANKING: 'classRanking',
  SCORE: 'score',
  GRADE_RANKING: 'gradeRanking',
  GRADE: 'evaluationType',
};

class ModelView extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      totalRecords: 0, // {number} 总记录数
      currentPage: 1, // {number} 当前页码
      pageSize: 20, // {number} 每页记录数
      searchExpand: false,
      listLoading: false, //{boolean} 加载动画
      year: null, //{string} 搜索项学年
      term: null, //{string} 搜索项学期
      yearDict: null, //{array<object>} 学年字典
      termDict: null, //{array<object>} 学期字典
      editIndex: null, //{number} 当前编辑的索引
      defaultExpandedRowKeys: [0], //列表默认展开项
      arr: [{ subjectDict: null, allSubject: null, subject: null, type: null, score: null, typeDict: null, id: null }], //{array<object>}  编辑状态下基准成绩列表
      tipIndex: null, //{number} 前端展示弹出层索引
      list: [], //{array} 列表数据项
      listArray: [],
      comparedList: [], //{array} 对比记录列表
      comparedId: null, //{string} 对比成绩id
      baseList: [], //{array} 查看情况下 基准成绩列表
      subjectSelected: null,
      readyShowId: null,
      baseTimeArray: [],
    };
  }

  componentDidMount() {
    let { studentId } = this.props;
    this.loadData();

    Promise.all([
      //获取成绩类型
      getSelectOption({ selectOptionCategory: 'AchievementType' }),

      //获取基准成绩学年下拉框数据
      getSubjectSelected({ studentId }),
    ]).then(([typeRes, selectedRes]) => {
      let typeDict = mapDataToDict(typeRes.value);
      //修改
      this.setState({ typeDict, subjectSelected: selectedRes });
    });
  }

  /**
   * 加载列表数据
   * @param searchkey  查找项
   * @returns {Promise.<void>}
   */
  loadData = async (searchkey) => {
    let { editIndex, defaultExpandedRowKeys } = this.state;
    this.setState({ listLoading: true });
    let { baseList, comparedList } = this.state;
    let { studentId } = this.props;
    let apiParams = { studentId };
    apiParams = { ...apiParams, ...searchkey };
    let apiData = await getComparedListById(apiParams);

    if (apiData) {
      Promise.all([
        getBaseScoreById({ comparisonId: apiData[editIndex || 0]['id'] }),
        getComparedScoreById({ studentAchievementId: apiData[editIndex || 0]['comparativeAchievementId'] }),
      ]).then(([baseRes, comparedRes]) => {
        baseList[editIndex || 0] = baseRes;
        comparedList[editIndex || 0] = comparedRes;
        //获取对比成绩考试时间
        let { examinationDate } = comparedRes;
        //获取基准成绩中  最晚的考试时间
        let baseTimeArray = baseRes.map((v, k) => {
          return v.examinationDate;
        });

        let baseLatestTime = moment
          .max(
            ...baseTimeArray.map((v, k) => {
              return moment(v);
            }),
          )
          .format('YYYY-MM-DD');
        //点击保存后继续展开所编辑的行
        defaultExpandedRowKeys.forEach((v, k) => {
          apiData[v].isExpand = true;
        });
        this.setState({
          baseList,
          comparedList,
          list: apiData,
          listLoading: false,
          editIndex: null,
          newAdd: false,
          comparedDate: examinationDate,
          baseTimeArray,
          baseLatestTime,
        });
      });
    } else {
      this.setState({ baseList: [], comparedList: [], list: [], listLoading: false, editIndex: null, newAdd: false });
    }
  };
  onOk = () => {};

  /**
   * 点击'-' 减少基准成绩
   * @param k {number} 当前基准索引
   */
  reduceBase = (k) => {
    let { arr, baseTimeArray } = this.state;
    if (arr.length >= 2) {
      arr.splice(k, 1);
      baseTimeArray.splice(k, 1);
      let baseLatestTime = moment
        .max(
          ...baseTimeArray.map((v, k) => {
            return moment(v);
          }),
        )
        .format('YYYY-MM-DD');
      this.setState({ arr, baseLatestTime });
    }
  };
  /**
   * 点击'+' 增加基准成绩
   * @param k {number} 当前基准成绩索引
   */
  increaseBase = (k) => {
    let { arr } = this.state;
    arr.push({ subjectDict: null, allSubject: null, subject: null, type: null, score: null, typeDict: null, id: null });
    this.setState({ arr });
  };

  /**
   * 选择基准成绩学年根据成绩编号获取所有科目
   * @param val{}成绩编号
   */
  getAllSubject = async (val, k) => {
    let { subjectSelected, baseLatestTime, baseTimeArray } = this.state;
    let { arr } = this.state;
    arr[k].subject = null;
    arr[k].type = null;
    arr[k].typeDict = null;
    arr[k].score = null;
    arr[k].allSubject = val;

    if (val) {
      let text = subjectSelected.filter((v, k) => {
        return v.val === val;
      })[0].text;
      let time;
      text.replace(/\d{4}\-\d{2}\-\d{2}/, ($1) => {
        time = $1;
      });

      baseTimeArray[k] = time;
      baseLatestTime = moment
        .max(
          ...baseTimeArray.map((v, k) => {
            return moment(v);
          }),
        )
        .format('YYYY-MM-DD');

      let res = await getSubjectByScoreID({ achievementId: val });
      let subjectDict = res.map((v, k) => {
        return {
          text: v.subjectName,
          val: v.subjectId,
          classRanking: v.classRanking,
          gradeRanking: v.gradeRanking,
          score: v.score,
          subjectId: v.subjectId,
          id: v.id,
          evaluationType: v.evaluationType,
        };
      });
      arr.forEach((v, k) => {
        subjectDict.forEach((V, K) => {
          if (v.subject === V.val) {
            V.disabled = true;
          }
        });
      });
      arr[k]['allSubject'] = val;
      arr[k]['subjectDict'] = subjectDict;
    }
    this.setState({ arr, baseLatestTime });
  };
  /**
   * 选择科目 处理成绩类型数据
   * @param val
   * @param k
   */
  selectSubject = (val, k) => {
    let { arr, typeDict } = this.state;
    let cloneTypeDict, selectSubjects;
    if (val) {
      let { subjectDict } = arr[k];
      //被选中的科目
      let selectType = subjectDict.find((v, k) => {
        return v.val === val;
      });
      let typeArray = Object.keys(selectType);
      //清空null项
      let cleanTypeArray = typeArray.filter((v, k) => {
        return selectType[v] !== null;
      });
      //获得每一个科目的对应评级下拉框
      cloneTypeDict = typeDict.filter((v, k) => {
        return cleanTypeArray.includes(mapObject[v.val]);
      });
      //查找成绩科目id
      selectSubjects = arr[k]['subjectDict'].find((v, k) => {
        return val === v.subjectId;
      });
    }

    arr[k].type = null;
    arr[k].score = null;
    arr[k]['subject'] = val;
    arr[k]['typeDict'] = cloneTypeDict || [];
    arr[k]['id'] = selectSubjects ? selectSubjects.id : null;
    this.setState({ arr });
  };
  /**
   * 选择成绩类型
   * @param val
   * @param type
   * @param k
   */
  selectType = (val, k) => {
    let { arr } = this.state;
    let { subject, subjectDict } = arr[k];

    if (subjectDict && subject) {
      let idx = subjectDict.findIndex((v, k) => {
        return v.val === subject;
      });
      let score = subjectDict[idx][mapObject[val]];
      arr[k]['type'] = val;
      arr[k]['score'] = score;
      this.setState({ arr });
    }
  };

  /**
   * 选择对比成绩
   * @param e
   */
  comparedChange = (e) => {
    let { list: allComparedList } = this.props;
    let id = e.target.value;
    let comparedDate = allComparedList.filter((v, k) => {
      return v.id === id;
    })[0].examinationDate;
    this.setState({ comparedId: id, comparedDate });
  };
  /**
   * 表格展开内容
   * @param record
   * @returns {XML}
   */
  expandContent = (record, idx) => {
    let {
      editIndex,
      allSubject,
      subject,
      type,
      score,
      arr,
      subjectSelected,
      subjectDict,
      typeDict,
      comparedId,
      comparedList,
      baseList,
      newAdd,
      comparedDate,
      baseLatestTime,
    } = this.state;
    let { list: allComparedList } = this.props;

    subjectSelected.forEach((v, k) => {
      v.text.replace(/\d{4}\-\d{2}\-\d{2}/, ($1) => {
        if ($1 > comparedDate) {
          v.disabled = true;
        } else {
          v.disabled = false;
        }
      });
    });
    let arrLen = arr.length;
    let { isExpand } = record;
    return (
      <div>
        {isExpand && (
          <div className={'CardList-expand-wrap'}>
            <div className={localStyles.baseScore}>
              <div className={localStyles.title}>基准成绩</div>
              <div className={localStyles.baseContent}>
                {!(idx == editIndex) &&
                  baseList[idx].map((v, k) => {
                    return (
                      <div key={k}>
                        <span className={localStyles.left}>
                          <span>{v.studentAchievementName.split(v.subjectName)[0]}</span>
                          <span>{' . ' + v.subjectName}</span>
                        </span>

                        <span className={localStyles.right}>
                          <span style={{ minWidth: 50, marginRight: 15, color: '#aaa', display: 'inline-block' }}>
                            {v.achievementTypeName}
                          </span>
                          <span>{parseFloat(v.value) ? parseFloat(v.value) : v.value}</span>
                        </span>
                      </div>
                    );
                  })}
                {idx == editIndex &&
                  arr.map((v, k) => {
                    return (
                      <div key={k} className={localStyles.baseContentItem}>
                        <span>
                          <Select
                            dropdownStyle={{ fontSize: 12 }}
                            placeholder="考试名称"
                            dict={subjectSelected || null}
                            value={v.allSubject || null}
                            onChange={(val) => {
                              this.getAllSubject(val, k);
                            }}
                          />
                        </span>
                        <span>
                          <Select
                            dropdownStyle={{ fontSize: 12 }}
                            placeholder="科目"
                            dict={v.subjectDict || null}
                            value={v.subject || null}
                            onChange={(val) => {
                              this.selectSubject(val, k);
                            }}
                          />
                        </span>
                        <span>
                          <Select
                            dropdownStyle={{ fontSize: 12 }}
                            placeholder="成绩类型"
                            dict={v.typeDict || null}
                            value={v.type || null}
                            onChange={(val) => {
                              this.selectType(val, k);
                            }}
                          />
                        </span>
                        <span>
                          <span>{parseFloat(v.score) || v.score}</span>
                        </span>
                        <span>
                          {k == arrLen - 1 && (
                            <div>
                              {k > 0 && (
                                <span className={localStyles.iconWrap}>
                                  <IconButton
                                    color={'#666'}
                                    type="minus_circle"
                                    onClick={(e) => {
                                      this.reduceBase(k);
                                    }}
                                  />
                                </span>
                              )}
                              <span className={localStyles.iconWrap}>
                                <IconButton
                                  color={'#666'}
                                  type="plus_circle"
                                  onClick={(e) => {
                                    this.increaseBase(k);
                                  }}
                                />
                              </span>
                            </div>
                          )}
                          {k != arrLen - 1 && (
                            <span className={localStyles.iconWrap}>
                              <IconButton
                                color={'#666'}
                                type="minus_circle"
                                onClick={(e) => {
                                  this.reduceBase(k);
                                }}
                              />
                            </span>
                          )}
                        </span>
                      </div>
                    );
                  })}
              </div>
            </div>
            <div className={localStyles.comparedScore}>
              <div className={localStyles.title}>对比成绩</div>
              <div className={localStyles.comparedContent}>
                {idx == editIndex && (
                  <RadioGroup onChange={this.comparedChange} value={comparedId} style={{ paddingLeft: 0 }}>
                    {allComparedList.map((v, k) => {
                      return (
                        <Radio
                          className={localStyles.radio}
                          value={v.id}
                          disabled={v.examinationDate < baseLatestTime}
                          key={k}
                        >
                          {' '}
                          {v.name + '（' + v.examinationDate + '）'}
                        </Radio>
                      );
                    })}
                  </RadioGroup>
                )}
                {!(idx == editIndex) && (
                  <span className={localStyles.radio}>
                    {' '}
                    {comparedList[idx].schoolYearName +
                      comparedList[idx].semesterName +
                      comparedList[idx].examinationTypeName}
                  </span>
                )}
              </div>
            </div>
          </div>
        )}
      </div>
    );
  };

  /**
   * 编辑或保存按钮
   * @param index
   */
  edit = (index, record) => {
    let {
      editIndex,
      defaultExpandedRowKeys,
      baseList,
      comparedList,
      list: listArray,
      comparedId,
      newAdd,
      typeDict,
    } = this.state;
    let { list } = this.props;
    //拷贝一份 防止关联改动
    let cloneList = JSON.parse(JSON.stringify(list));

    let key = record.key;
    if (!defaultExpandedRowKeys.includes(index)) {
      defaultExpandedRowKeys.push(index);
    }
    let semesterValue = listArray[index].semesterValue;
    let schoolYearId = listArray[index].schoolYearId;

    if (editIndex == index) {
      this.saveData(record);
    } else {
      if (newAdd) {
        listArray.shift();
        key -= 1;
        this.setState({ newAdd: false });
      }
      this.setState({ listLoading: true });
      Promise.all([
        getBaseScoreById({ comparisonId: listArray[key]['id'] }),
        getComparedScoreById({ studentAchievementId: listArray[key]['comparativeAchievementId'] }),
      ]).then(([baseRes, comparedRes]) => {
        baseList[key] = baseRes;
        comparedList[key] = comparedRes;
        //设置是否渲染
        listArray[key].isExpand = true;

        //取基准成绩编号获取科目列表
        let idList = baseRes.map((v, k) => {
          return v.studentAchievementId;
        });
        let subjectList = baseRes.map((v, k) => {
          return v.subjectId;
        });
        let valueList = baseRes.map((v, k) => {
          return v.value;
        });
        let typeList = baseRes.map((v, k) => {
          return v.achievementTypeValue;
        });

        this.getSubjectList(idList).then((res) => {
          let arr;
          if (res.length > 0) {
            arr = res.map((v, k) => {
              let subjectDict = v.map((value, key) => {
                return {
                  text: value.subjectName,
                  val: value.subjectId,
                  classRanking: value.classRanking,
                  gradeRanking: value.gradeRanking,
                  score: value.score,
                  subjectId: value.subjectId,
                  id: value.id,
                  evaluationType: value.evaluationType,
                };
              });
              let selectSubjects = subjectDict.find((V, K) => {
                // subjectList ['DAT0000000014',...]
                return V.subjectId === subjectList[k];
              });

              let typeArray = ['classRanking', 'evaluationType', 'gradeRanking', 'score'];
              let hasType = typeArray.filter((v, k) => {
                return selectSubjects[v] !== null;
              });
              let truthTypeDict = typeDict.filter((v, k) => {
                return hasType.includes(mapObject[v.val]);
              });
              return {
                subjectDict: subjectDict,
                allSubject: String(idList[k]),
                subject: subjectList[k],
                type: typeList[k],
                score: valueList[k],
                typeDict: truthTypeDict,
                id: selectSubjects.id,
              };
            });
          } else {
            arr = [{}];
          }
          this.setState({
            baseList,
            comparedList,
            list: listArray,
            listLoading: false,
            comparedId: comparedRes.id,
            arr,
            editIndex: index,
          });
        });
      });
    }
    this.setState({
      defaultExpandedRowKeys,
      comparedList: cloneList,
      selectschoolYearId: schoolYearId,
      selectsemesterValue: semesterValue,
    });
  };

  /**
   * 获取多条基准成绩的科目列表
   * @param idList {array}
   * @returns {Promise.<*[]>}
   */
  getSubjectList = async (idList) => {
    let promises = idList.map((v) => getSubjectByScoreID({ achievementId: v }));
    return await Promise.all(promises);
  };

  /**
   * 保存数据
   * @param record{object}  当前行数据对象
   */
  saveData = (record) => {
    let { arr, comparedId, selectschoolYearId, selectsemesterValue, newAdd } = this.state;
    let { studentId } = this.props;
    let { version } = record;
    for (let i = 0; i < arr.length; i++) {
      if (!arr[i]['score']) {
        showMessage('请完善基准成绩');
        return;
      }
    }
    if (!comparedId) {
      showMessage('请选择对比成绩');
      return;
    }
    if (!selectschoolYearId) {
      showMessage('请选择学年');
      return;
    }
    if (!selectsemesterValue) {
      showMessage('请选择学期');
      return;
    }
    let subjectIds = [];
    let benchmarks = arr.map((v, k) => {
      subjectIds.push(v.id);
      return {
        achievementType: v.type,
        studentAchievementId: parseInt(v.allSubject),
        studentAchievementSubjectId: v.id,
        subjectId: v.subject,
      };
    });
    if ([...new Set(subjectIds)].length !== subjectIds.length) {
      showMessage('存在相同的基准科目，请重新选择');
      return;
    }
    if (benchmarks.length === 0) {
      showMessage('请至少选择一门基准成绩!');
      return;
    }
    let apiParams = {
      benchmarks: benchmarks,
      comparativeAchievementId: comparedId,
      schoolYearId: selectschoolYearId,
      semester: selectsemesterValue,
      id: record.id,
      studentId,
      subjectIds: record.subjectIds,
      subjectNames: record.subjectNames,
      version: version ? version : 0,
    };

    saveBaseAndCompared(apiParams).then((res) => {
      let { comparedId } = this.props;
      showMessage('保存成功', 'success');
      let { baseList } = this.state;
      this.setState({ readyShowId: record.id });
      this.loadData().then((apiData) => {
        let { data } = res;
        //当新增对比记录 或 编辑 已前端展示 的记录时  保存成功后直接关闭弹窗更新图表
        //编辑其他则不做后续操作，需要点击展示才继续
        if (newAdd || comparedId === data) {
          this.showFronend({}, data);
        }
      });
    });
  };

  /**
   * 点击取消按钮
   * @param idx{number} 当前行索引
   */
  cancelEdit = (idx) => {
    let { newAdd, list } = this.state;
    if (idx === 0 && newAdd) {
      list.shift();
    }
    this.setState({ editIndex: null, newAdd: false });
  };
  /**
   * 前端展示弹出层 显示隐藏的回调
   * @param index {number} 当前行索引
   */
  handleVisibleChange = (index) => {
    this.setState({ tipIndex: index });
  };
  /**
   * 点击'+'展开触发
   * @param expanded {boolean} 是否展开
   * @param record {object} 当前行数据对象
   */
  onExpanded = (expanded, record) => {
    let { list, baseList, comparedList } = this.state;
    let key = record.key;
    if (expanded && list[key]['id']) {
      Promise.all([
        getBaseScoreById({ comparisonId: list[key]['id'] }),
        getComparedScoreById({ studentAchievementId: list[key]['comparativeAchievementId'] }),
      ]).then(([baseRes, comparedRes]) => {
        baseList[key] = baseRes;
        list[key].isExpand = true;
        comparedList[key] = comparedRes;
        this.setState({ baseList, comparedList, list });
      });
    }

    //展开索引，可多个
    let { defaultExpandedRowKeys } = this.state;
    let index = defaultExpandedRowKeys.findIndex((v, k) => {
      return v == key;
    });
    if (index < 0) {
      defaultExpandedRowKeys.push(key);
    } else {
      defaultExpandedRowKeys.splice(index, 1);
    }
    this.setState({ defaultExpandedRowKeys });
  };

  /**
   * 根据学年和学期获取在读一对一科目
   * @param item
   * @param val
   * @param idx
   */
  selectChage = (item, val, idx) => {
    let { list } = this.state;
    let { studentId } = this.props;
    let key = 'select' + item;
    list[idx][item] = val;
    this.setState({ list, [key]: val }, () => {
      let { selectschoolYearId, selectsemesterValue } = this.state;
      if (selectschoolYearId && selectsemesterValue) {
        getOTOSubiect({
          schoolYearId: selectschoolYearId,
          semester: selectsemesterValue,
          studentId,
        }).then((res) => {
          list[idx].subjectIds = res.value;
          list[idx].subjectNames = res.name;
          this.setState({ list });
        });
      }
    });
  };
  /**
   * 新增对比记录
   */
  add = async () => {
    let { list: listArray, studentId, yearDict } = this.props;
    let subjectIds;
    let subjectNames;
    let { newAdd, list } = this.state;
    //当处于新增时,点击新增按钮不响应
    if (newAdd) {
      return;
    }
    this.listRef.setExpandRow();
    let copyListArray = JSON.parse(JSON.stringify(listArray));

    let currentTime = getCurrentYearAndTerm();
    let semesterValue = currentTime.term;

    let schoolYearId;
    let schoolYearObj = yearDict.filter((v, k) => {
      return v.text === currentTime.schoolYear;
    })[0];
    if (schoolYearObj) {
      schoolYearId = schoolYearObj.val;
    }
    if (schoolYearId && semesterValue) {
      getOTOSubiect({
        schoolYearId: schoolYearId,
        semester: semesterValue,
        studentId,
      }).then((res) => {
        list[0].subjectIds = res.value;
        list[0].subjectNames = res.name;
        this.setState({ list });
      });
    }
    list.unshift({
      comparativeAchievementId: '',
      schoolYearId: '',
      schoolYearName: '',
      semesterName: '',
      semesterValue: '',
      studentId,
      subjectIds,
      subjectNames,
      isExpand: true,
      semesterValue,
      schoolYearId,
    });

    this.setState({
      selectschoolYearId: schoolYearId,
      selectsemesterValue: semesterValue,
      list,
      editIndex: 0,
      comparedId: null,
      comparedList: copyListArray,
      newAdd: true,
      defaultExpandedRowKeys: [0],
      arr: [{ subjectDict: null, allSubject: null, subject: null, type: null, score: null, typeDict: null, id: null }],
    });
  };
  /**
   * 查询
   */
  search = () => {
    let { term, year } = this.state;
    let searchkey = { schoolYearId: year, semester: term };
    this.loadData(searchkey);
  };
  /**
   * 重置查询条件
   */
  reset = () => {
    this.setState({ term: null, year: null });
  };

  /**
   * 点击前端展示 确认 按钮
   * @param index
   */
  showFronend = (record, comparedId) => {
    let { showComparedDetails } = this.props;
    let id = comparedId ? comparedId : record.id;
    showComparedDetails(id);
  };

  render() {
    let {
      totalRecords,
      currentPage,
      pageSize,
      searchExpand,
      loading,
      listLoading,
      judgementDict,
      judge,
      year,
      term,
      editIndex,
      defaultExpandedRowKeys,
      tipIndex,
      list,
      readyShowId,
    } = this.state;
    let { onClose, yearDict, termDict, comparedId } = this.props;
    let columns = [
      {
        title: '学年',
        dataIndex: 'schoolYearId',
        width: COL_WIDTH_L,
        key: 'schoolYearId',
        render: (text, record, idx) => {
          if (editIndex === idx) {
            return (
              <Select
                placeholder="学年"
                dict={yearDict}
                value={text}
                style={{ width: 100 }}
                onChange={(val) => this.selectChage('schoolYearId', val, idx)}
              />
            );
          } else {
            return <span>{getDictTextByVal(yearDict, text)}</span>;
          }
        },
      },
      {
        title: '学期',
        width: COL_WIDTH_L,
        dataIndex: 'semesterValue',
        key: 'semesterValue',
        render: (text, record, idx) => {
          if (editIndex === idx) {
            return (
              <Select
                placeholder="学期"
                dict={termDict}
                value={text}
                onChange={(val) => this.selectChage('semesterValue', val, idx)}
                style={{ width: 90 }}
              />
            );
          }
          return <span>{getDictTextByVal(termDict, text)}</span>;
        },
      },
      { title: '在读一对一科目', dataIndex: 'subjectNames', width: COL_WIDTH_XL, key: 'OTOSubject' },
      {
        title: '操作',
        width: COL_WIDTH_L,
        render: (text, record, idx) => {
          let { showFrontend } = record;
          return (
            <div className={(showFrontend ? 'showFrontend' : '') + ' ' + 'popover'}>
              {/* <span className={'edit'} onClick={e => this.edit(idx, record)}>{editIndex == idx ? '保存' : '编辑'}</span> */}
              {/* &nbsp;&nbsp;&nbsp;
            {(editIndex == idx) &&
            <span className={'frontShow'} onClick={e => {
              this.cancelEdit(idx)
            }}>取消</span>
            } */}
              {editIndex != idx && (
                <span>
                  {comparedId === record.id && readyShowId !== comparedId && <span>已展示</span>}
                  {(comparedId !== record.id || readyShowId === comparedId) && (
                    <PopConfirm
                      content="这将替换当前展示的基准成绩和对比成绩，是否继续？"
                      okBtnText="是"
                      cancelBtnText="否"
                      onOk={(e) => this.showFronend(record)}
                    >
                      <span className={'frontShow'}>前端展示</span>
                    </PopConfirm>
                  )}
                </span>
              )}
            </div>
          );
        },
      },
    ];
    return (
      <Modal
        loading={loading}
        onCancel={onClose}
        onOk={this.onOk}
        title="基准成绩和对比成绩设置"
        width={800}
        okBtnShow={false}
        cancelText="关闭"
        okText="保存"
        closable={true}
        footer={null}
        className={localStyles.setting}
      >
        <Block title={name} style={{ marginTop: -10 }}>
          <div className={styles.wrap} style={{ minWidth: 600, padding: 0, border: 'none' }}>
            <CardList
              dataSource={list}
              columns={columns}
              loading={listLoading}
              expandedRowKeys={defaultExpandedRowKeys}
              onExpand={this.onExpanded}
              expandedRowRender={(record, idx) => this.expandContent(record, idx)}
              noDataTips={'暂无数据，请点击新增按钮'}
              ref={(e) => (this.listRef = e)}
            />
          </div>
        </Block>
      </Modal>
    );
  }
}

export default ModelView;
