import React from 'react';
import styles from './GradeTemplate.less';
import { getDictTextByVal } from 'UTIL/dict';
import { Icon } from 'antd';
import Button from 'COMPONENT/Button';
import Checkbox from 'COMPONENT/Checkbox';
import Tag from 'COMPONENT/Tag';
import InputTag from 'COMPONENT/InputTag';
import SelectSubjectPopover from './SelectSubjectPopover';
import Validator from 'COMPONENT/Validator';
import { checkNumber } from 'COMPONENT/Validator/checkers';
import { SCORE_TYPE_1, SCORE_TYPE_2 } from '../common';

class GradeTemplate extends React.Component {
  remove = (subjectId) => {
    const { data, onChange } = this.props;
    const newData = {
      ...data,
      subjects: data.subjects.filter((s) => s.id !== subjectId),
    };
    onChange(newData);
  };

  add = (subjectIds) => {
    const { data, onChange } = this.props;
    const subjects = subjectIds.map((s) => {
      return { id: s, totalScore: 100 };
    });
    const newData = {
      ...data,
      subjects: [...data.subjects, ...subjects],
    };
    onChange(newData);
  };

  totalScoreChange = (subjectId, totalScore) => {
    const { data, onChange } = this.props;
    const newData = {
      ...data,
      subjects: data.subjects.map((s) => {
        if (s.id === subjectId) {
          return { id: s.id, totalScore };
        } else {
          return s;
        }
      }),
    };
    onChange(newData);
  };

  scoreTypeChange = (type, checked) => {
    const { data, onChange } = this.props;
    let newTypes;
    if (checked) {
      newTypes = [...data.types, type];
    } else {
      newTypes = data.types.filter((t) => t !== type);
    }
    const newData = {
      ...data,
      types: newTypes,
    };
    onChange(newData);
  };

  render() {
    const {
      gradeDict,
      subjectDict,
      scoreTypeDict,
      data,
      onScoreTypeErrorChange,
      onSubjectsErrorChange,
      onTotalScoreErrorChange,
      checking,
    } = this.props;

    const subjects = data.subjects || [];
    const hasTotalScore = data.types.indexOf(SCORE_TYPE_1) >= 0;

    return (
      <div className={styles.wrap}>
        <div className={styles.left}>{getDictTextByVal(gradeDict, data.gradeId)}</div>
        <div className={styles.right}>
          <div className={styles.label}>成绩类型</div>
          <div className={styles.content} style={{ paddingBottom: 20 }}>
            <Validator
              rules={[{ required: '请选择成绩类型' }]}
              onErrorChange={onScoreTypeErrorChange}
              silent={!checking}
              value={data.types}
            >
              <div className={styles.formFieldInner}>
                <div className={styles.checkbox}>
                  <Checkbox
                    size="small"
                    checked={data.types.indexOf(SCORE_TYPE_1) >= 0}
                    onChange={(e) => this.scoreTypeChange(SCORE_TYPE_1, e.target.checked)}
                  >
                    {getDictTextByVal(scoreTypeDict, SCORE_TYPE_1)}
                  </Checkbox>
                </div>
                <div className={styles.checkbox}>
                  <Checkbox
                    size="small"
                    checked={data.types.indexOf(SCORE_TYPE_2) >= 0}
                    onChange={(e) => this.scoreTypeChange(SCORE_TYPE_2, e.target.checked)}
                  >
                    {getDictTextByVal(scoreTypeDict, SCORE_TYPE_2)}
                  </Checkbox>
                </div>
              </div>
            </Validator>
          </div>
          <div>
            <div className={styles.label} style={{ marginBottom: 15, marginTop: -5 }}>{`科目${
              hasTotalScore ? '总分' : ''
            }`}</div>
            <div className={styles.content} style={{ paddingBottom: 15 }}>
              <Validator
                rules={[{ required: '请添加科目' }]}
                onErrorChange={onSubjectsErrorChange}
                silent={!checking}
                value={data.subjects}
              >
                <div className={styles.formFieldInner} style={{ marginBottom: -15 }}>
                  {subjects.map((s, i) => {
                    return (
                      <div key={s.id} className={styles.subject}>
                        {hasTotalScore ? (
                          <InputTag
                            inputWidth={60}
                            closable
                            onClose={() => this.remove(s.id)}
                            type="number"
                            size="small"
                            min={0}
                            value={s.totalScore}
                            onChange={(val) => this.totalScoreChange(s.id, val)}
                            validator={{
                              onErrorChange: (e) => onTotalScoreErrorChange(s.id, e),
                              silent: !checking,
                              keepHeight: true,
                              rules: ['required', (v) => checkNumber(v, { min: 0, max: 999 })],
                            }}
                          >
                            {getDictTextByVal(subjectDict, s.id)}
                          </InputTag>
                        ) : (
                          <Tag closable onClose={() => this.remove(s.id)} size="small" color="#fff">
                            {getDictTextByVal(subjectDict, s.id)}
                          </Tag>
                        )}
                      </div>
                    );
                  })}
                  {subjectDict.filter((s) => !subjects.find((s2) => s2.id === s.val)).length > 0 && (
                    <div key="add" className={styles.subject}>
                      <SelectSubjectPopover
                        subjectDict={subjectDict.filter((s) => !subjects.find((s2) => s2.id === s.val))}
                        onOk={(subjects) => this.add(subjects)}
                        ref={(c) => (this.pop = c)}
                      >
                        <Button size="small" onClick={() => this.pop.show()}>
                          <Icon type="plus" />
                          添加
                        </Button>
                      </SelectSubjectPopover>
                    </div>
                  )}
                </div>
              </Validator>
            </div>
          </div>
        </div>
      </div>
    );
  }
}

export default GradeTemplate;
