/**
 * 选择科目气泡弹框
 * @property {ReactNode} children - 触发弹出气泡的元素
 * @property {object[]} subjectDict - 可选的科目字典
 * @property {Function} onOk - 确定的回调
 */
import React from 'react';
import { Popover } from 'antd';
import Button from 'COMPONENT/Button';
import Checkbox from 'COMPONENT/Checkbox';
import { getDictTextByVal } from 'UTIL/dict';
import styles from './SelectSubjectPopover.less';

class SelectSubjectPopover extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      visible: false,
      selected: [],
    };
  }

  ok = () => {
    const { onOk } = this.props;
    const { selected } = this.state;
    this.setState({ visible: false, selected: [] });
    if (onOk && selected.length > 0) {
      onOk(selected);
    }
  };

  cancel = () => {
    this.setState({ visible: false, selected: [] });
  };

  show = () => {
    this.setState({ visible: true });
  };

  select = (id, checked) => {
    const { selected } = this.state;
    let newSelected;
    if (checked) {
      newSelected = [...selected, id];
    } else {
      newSelected = selected.filter((s) => s !== id);
    }
    this.setState({ selected: newSelected });
  };

  render() {
    const { children, subjectDict, onOk } = this.props;
    const { visible, selected } = this.state;
    const content = (
      <div className={styles.content}>
        <div className={styles.body}>
          {subjectDict.map((s) => {
            return (
              <div key={s.val} className={styles.item}>
                <Checkbox checked={selected.indexOf(s.val) >= 0} onChange={(e) => this.select(s.val, e.target.checked)}>
                  {s.text}
                </Checkbox>
              </div>
            );
          })}
        </div>
        <div className={styles.footer}>
          <Button type="primary" onClick={this.ok}>
            确定
          </Button>
          <Button onClick={this.cancel} style={{ marginLeft: 20 }}>
            取消
          </Button>
        </div>
      </div>
    );

    return (
      <Popover content={content} trigger="click" visible={visible}>
        {children}
      </Popover>
    );
  }
}

export default SelectSubjectPopover;
