import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import List from 'COMPONENT/List/CardList';
import { loadCommonDict } from 'UTIL/dict';
import { getTemplateDetail, loadTemplateDetail } from '../common';
import { findPageAchievementTemplateByCityId, deleteAchievementTemplateById } from 'SERVICE/student';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import PopConfirm from 'COMPONENT/PopConfirm';
import Badage from 'COMPONENT/Badage';
import TextLink from 'COMPONENT/TextLink';

const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;

class History extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 内容加载状态

      list: [], // {Array} 列表数据
      listLoading: false, // {false} 列表数据加载状态
      totalRecords: 0, // {number} 总记录数
      currentPage: 1, // {number} 当前页码
      pageSize: 20, // {number} 每页记录数
      expandedRowKeys: [], // Number[] 列表展开行的key
    };
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化内容
   */
  initContent = () => {
    this.loadList();
  };

  /**
   * 加载列表数据，并转换格式
   */
  loadList = async (changedParams = {}) => {
    const {
      params: { cityId },
    } = this.props;
    const { currentPage, pageSize } = this.state;
    const searchKeys = { cityId };
    const params = { currentPage, pageSize, ...searchKeys, ...changedParams };
    const apiParams = {
      page: params.currentPage,
      rows: params.pageSize,
      cityId: params.cityId,
    };
    this.setState({ listLoading: true });
    const apiData = await findPageAchievementTemplateByCityId(apiParams);
    /*    const apiData = await new Promise(resolve => {
      setTimeout(() => {
        resolve({
          records: 11,
          rows: [
            { cityId: '11', cityName: '东莞市', startDate: '2017-03-02', createTime: '2017-02-28 09:30', version: 'V1', creator: '李老师', template: [
                { gradeId: '402881e647ce90330147ce94ef900002', types: ['1'], subjects: [
                    { id: 'DAT0000000015', totalScore: 100 },
                    { id: 'DAT0000000016', totalScore: 100 },
                  ]
                },
                { gradeId: '402881e647ce90330147ce94ef900003', types: ['1', '2'], subjects: [
                    { id: 'DAT0000000015', totalScore: 100 },
                    { id: 'DAT0000000016', totalScore: 100 },
                  ]
                },
              ]
            },
            { cityId: '11', cityName: '东莞市', startDate: '2017-03-02', createTime: '2017-02-28 09:30', version: 'V1', creator: '李老师', template: [
                { gradeId: '402881e647ce90330147ce94ef900002', types: ['1'], subjects: [
                    { id: 'DAT0000000015', totalScore: 100 },
                    { id: 'DAT0000000016', totalScore: 100 },
                  ]
                },
                { gradeId: '402881e647ce90330147ce94ef900003', types: ['1', '2'], subjects: [
                    { id: 'DAT0000000015', totalScore: 100 },
                    { id: 'DAT0000000016', totalScore: 100 },
                  ]
                },
              ]
            },
          ],
        })
      }, 500)
    })*/
    const { records, rows, page } = apiData;
    const formatedRows = rows.map((row, i) => {
      return {
        isNew: page === 1 && i === 0,
        cityId: row.cityId,
        cityName: row.cityName,
        startDate: row.effectiveDate,
        createTime: row.createTime.slice(0, 16),
        templateVersionName: 'V' + row.achievementVersion,
        creator: row.createUserName,
        templateId: row.id,
        version: row.version,
      };
    });
    const data = {
      records,
      rows: formatedRows,
    };
    const { currentPage: newCurrentPage, pageSize: newPageSize } = params;
    this.setState({
      expandedRowKeys: [],
      listLoading: false,
      list: data.rows,
      totalRecords: data.records,
      currentPage: newCurrentPage,
      pageSize: newPageSize,
    });
    return true;
  };

  edit = (record) => {
    const {
      dispatch,
      params: { gradeDict, subjectDict, scoreTypeDict },
    } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: {
        path: '/StudentScoreTemplate/History/Edit',
        params: {
          record,
          gradeDict,
          subjectDict,
          scoreTypeDict,
          onSaveSuccess: () => this.editSaveSuccess(record.cityId),
        },
        refresh: true,
      },
    });
  };

  /**
   * 编辑保存成功后的处理
   */
  editSaveSuccess = (cityId) => {
    const {
      dispatch,
      params: { gradeDict, subjectDict, scoreTypeDict },
    } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: { path: '/StudentScoreTemplate/History', params: { cityId, gradeDict, subjectDict, scoreTypeDict } },
    });
    this.refresh();
  };

  delete = (record) => {
    this.setState({ loading: true });
    deleteAchievementTemplateById({ templateId: record.templateId }, true).then(
      () => {
        showMessage('删除成功', 'success');
        this.setState({ loading: false });
        this.refresh();
      },
      (e) => {
        showMessage(e.text || '删除失败');
        this.setState({ loading: false });
      },
    );
  };

  refresh = () => {
    const { dispatch } = this.props;
    this.loadList();
    dispatch({
      type: 'main/refreshPage',
      payload: { pathname: '/StudentScoreTemplate' },
    });
  };

  onExpand = (expanded, record) => {
    const { list } = this.state;
    const { template, templateId } = record;
    if (!template) {
      loadTemplateDetail(templateId).then((t) => {
        const newList = list.map((row) => {
          if (row.templateId === templateId) {
            return { ...row, template: t };
          }
          return row;
        });
        this.setState({ list: newList });
      });
    }
  };

  render() {
    const {
      params: { gradeDict, subjectDict, scoreTypeDict },
    } = this.props;
    const { loading, list, totalRecords, currentPage, pageSize, listLoading, expandedRowKeys } = this.state;
    const columns = [
      {
        title: '城市',
        dataIndex: 'cityName',
        render: (text, record, index) => {
          if (record.isNew) {
            return (
              <span>
                <span>{text}</span>
                <Badage style={{ marginLeft: 10 }}>最新</Badage>
              </span>
            );
          }
          return text;
        },
      },
      { title: '起始生效日期', dataIndex: 'startDate' },
      { title: '创建时间', dataIndex: 'createTime' },
      { title: '版本', dataIndex: 'templateVersionName', width: COL_WIDTH_S },
      { title: '创建人', dataIndex: 'creator' },
      {
        title: '操作',
        key: 'action',
        render: (text, record, index) => {
          return (
            <span>
              <PopConfirm
                key={1}
                content="该操作可能影响成绩统计和进步率分析结果，是否继续？"
                okBtnText="是"
                cancelBtnText="否"
                onOk={() => this.edit(record)}
              >
                <TextLink className="list-btn">编辑</TextLink>
              </PopConfirm>
              <PopConfirm
                key={2}
                content={`确定要删除${record.templateVersionName}版本的模板吗？`}
                okBtnText="是"
                cancelBtnText="否"
                onOk={() => this.delete(record)}
              >
                <TextLink className="list-btn">删除</TextLink>
              </PopConfirm>
            </span>
          );
        },
      },
    ];

    return (
      <Page loading={loading} route={this.props.route}>
        <List
          dataSource={list}
          columns={columns}
          loading={!loading && listLoading}
          pagination={{
            current: currentPage,
            total: totalRecords,
            pageSize,
          }}
          onChange={({ current, pageSize }) => this.loadList({ currentPage: current, pageSize })}
          expandedRowRender={(record) => getTemplateDetail(record, { gradeDict, subjectDict, scoreTypeDict })}
          noDataTips="暂无历史版本"
          onExpand={this.onExpand}
          expandedRowKeys={expandedRowKeys}
          onExpandedRowsChange={(expandedRowKeys) => this.setState({ expandedRowKeys })}
        />
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    params: state.main.pageParams['/StudentScoreTemplate/History'],
  };
};

export default connect(mapStateToProps)(History);
