/**
 * 学生进步标准设置模态框
 * @property {function} onClose 关闭弹框的回调
 */
import React from 'react';
import classnames from 'classnames';
import Modal from 'COMPONENT/Modal';
import InputNumber from 'COMPONENT/InputNumber';
import IconButton from 'COMPONENT/IconButton';
import styles from './styles.less';
import { getDictTextByVal } from 'UTIL/dict';
import { listAllAchievementProStandard, updateAchievementProStandard } from 'SERVICE/student';
import { studentTypes } from '../common';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import DotButton from 'COMPONENT/DotButton';
import Validator from 'COMPONENT/Validator';
import { objectDeepEquals } from 'COMPONENT/utils/basic/obj';
import { checkNumber } from 'COMPONENT/Validator/checkers';
import getWrappedInstance from 'UTIL/getWrappedInstance';

const STANDARD_CATE_1 = 'PROGRESS';
const STANDARD_CATE_2 = 'FLAT';
const STANDARD_CATE_3 = 'BACKWARD';
const numberConfig = { min: -999, max: 999 };
const numberConfig2 = { ...numberConfig, int: true };
class ProgressStandardModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 等待状态
      data: null, // {object[]} 进步标准设置数据，格式如下：
      /**
                  [
                    {
                      name {string} - 类别名称
                      settings: [ - 设置项数组，长度固定为，分别为进步、持平、退步的设置项
                        {
                          score {number|[number, number]}, - 分数相差，持平设置项为两个值组成的范围
                          class: {number|[number, number]}, - 班级排名相差，持平设置项为两个值组成的范围
                          grade: {number|[number, number]}, - 年级排名相差，持平设置项为两个值组成的范围
                          levels: {string[]}, - 评级，A类学生进步设置没有此项
                        },
                      ]
                    }
                  ]
                  */
      checking: false, //校验开启状态
      initData: null, // {Array} 备份初始数据，用于判断是否修改
    };
    this.errors = {}; //校验错误
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化内容
   */
  initContent = () => {
    this.setState({ loading: true });
    Promise.all([this.loadData()]).then(([data]) => {
      this.setState({
        loading: false,
        data,
        initData: data,
      });
    });
  };

  /**
   * 加载设置数据
   */
  loadData = async () => {
    const { standardCategoryDict } = this.props;
    const apiData = await listAllAchievementProStandard();
    let data = studentTypes.map((s) => {
      return { name: s, settings: [{}, {}, {}] };
    });
    /*
        "classRankingBtwEnd": -1,
        "classRankingBtwStart": 1,
        "evaluationList": 1,
        "gradeRankingBtwEnd": -1,
        "gradeRankingBtwStart": 1,
        "greaterTypeName": "大于",
        "greaterTypeValue": "GREATER_THAN",
        "id": 1,
        "lessTypeName": 1,
        "lessTypeValue": 1,
        "name": "A类",
        "scoreBetweenEnd": 1,
        "scoreBetweenStart": 0,
        "standardCategoryName": "进步",
        "standardCategoryValue": "PROGRESS",
        "version": 3
     */
    console.log(apiData);
    apiData.forEach((d) => {
      let setting = {
        greaterTypeValue: d.greaterTypeValue,
        id: d.id,
        lessTypeValue: d.lessTypeValue,
        standardCategoryValue: d.standardCategoryValue,
        version: d.version,
      };
      let idx;
      if (d.standardCategoryValue === STANDARD_CATE_1) {
        idx = 0;
        setting = {
          ...setting,
          score: d.scoreBetweenStart,
          class: d.classRankingBtwEnd,
          grade: d.gradeRankingBtwEnd,
        };
      } else if (d.standardCategoryValue === STANDARD_CATE_2) {
        idx = 1;
        setting = {
          ...setting,
          score: [d.scoreBetweenStart, d.scoreBetweenEnd],
          class: [d.classRankingBtwStart, d.classRankingBtwEnd],
          grade: [d.gradeRankingBtwStart, d.gradeRankingBtwEnd],
        };
      } else if (d.standardCategoryValue === STANDARD_CATE_3) {
        idx = 2;
        setting = {
          ...setting,
          score: d.scoreBetweenEnd,
          class: d.classRankingBtwStart,
          grade: d.gradeRankingBtwStart,
        };
      }
      if (d.evaluationList) {
        setting['levels'] = d.evaluationList.split(',');
      }
      data.find((da) => da.name === d.name).settings[idx] = setting;
    });
    return data;
  };

  onOk = () => {
    const { checking } = this.state;
    if (!checking) {
      this.setState({ checking: true });
    }
    if (Object.values(this.errors).find((v) => v)) {
      this.modal.showOkTips('请正确填写信息');
      return;
    }
    const { onClose } = this.props;
    const { data } = this.state;
    /*
        "classRankingBtwEnd": -1,
        "classRankingBtwStart": 1,
        "evaluationList": 1,
        "gradeRankingBtwEnd": -1,
        "gradeRankingBtwStart": 1,
        "greaterTypeName": "大于",
        "greaterTypeValue": "GREATER_THAN",
        "id": 1,
        "lessTypeName": 1,
        "lessTypeValue": 1,
        "name": "A类",
        "scoreBetweenEnd": 1,
        "scoreBetweenStart": 0,
        "standardCategoryName": "进步",
        "standardCategoryValue": "PROGRESS",
        "version": 3
     */
    let saveData = [];
    data.forEach((d) => {
      d.settings.forEach((s) => {
        let scoreBetweenEnd,
          scoreBetweenStart,
          classRankingBtwEnd,
          classRankingBtwStart,
          gradeRankingBtwEnd,
          gradeRankingBtwStart;
        if (s.standardCategoryValue === STANDARD_CATE_1) {
          scoreBetweenEnd = null;
          scoreBetweenStart = s.score;
          classRankingBtwEnd = s.class;
          classRankingBtwStart = null;
          gradeRankingBtwEnd = s.grade;
          gradeRankingBtwStart = null;
        } else if (s.standardCategoryValue === STANDARD_CATE_2) {
          scoreBetweenEnd = s.score[1];
          scoreBetweenStart = s.score[0];
          classRankingBtwEnd = s.class[1];
          classRankingBtwStart = s.class[0];
          gradeRankingBtwEnd = s.grade[1];
          gradeRankingBtwStart = s.grade[0];
        } else if (s.standardCategoryValue === STANDARD_CATE_3) {
          scoreBetweenEnd = s.score;
          scoreBetweenStart = null;
          classRankingBtwEnd = null;
          classRankingBtwStart = s.class;
          gradeRankingBtwEnd = null;
          gradeRankingBtwStart = s.grade;
        }
        saveData.push({
          greaterType: s.greaterTypeValue,
          id: s.id,
          lessType: s.lessTypeValue,
          name: d.name,
          standardCategory: s.standardCategoryValue,
          version: s.version,
          evaluationList: s.levels ? s.levels.join(',') : null,
          scoreBetweenEnd,
          scoreBetweenStart,
          classRankingBtwEnd,
          classRankingBtwStart,
          gradeRankingBtwEnd,
          gradeRankingBtwStart,
        });
      });
    });
    saveData = { achievementProStandards: saveData };
    //console.log('saveData:', saveData, JSON.stringify(saveData))
    updateAchievementProStandard(saveData, true).then(
      () => {
        showMessage('保存成功', 'success');
        this.setState({ loading: false });
        onClose();
      },
      (e) => {
        if (e.text) {
          this.modal.showOkTips(e.text);
        } else {
          showMessage('保存失败');
        }
        this.setState({ loading: false });
      },
    );
  };

  valueChange = (typeIdx, rowIdx, colType, value, rangeIdx) => {
    const { data } = this.state;
    const newData = data.map((d, i) => {
      if (i === typeIdx) {
        const newSettings = d.settings.map((s, j) => {
          if (j === rowIdx) {
            let changed;
            if (rangeIdx === 0) {
              changed = { [colType]: [value, s[colType][1]] };
            } else if (rangeIdx === 1) {
              changed = { [colType]: [s[colType][0], value] };
            } else {
              changed = { [colType]: value };
            }
            return { ...s, ...changed };
          }
          return s;
        });
        return { ...d, settings: newSettings };
      }
      return d;
    });
    this.setState({ data: newData });
  };

  levelChange = (typeIdx, rowIdx, value) => {
    const { data } = this.state;
    const levels = data[typeIdx].settings[rowIdx].levels;
    let newLevels;
    if (levels.indexOf(value) < 0) {
      newLevels = [...levels, value];
    } else {
      newLevels = levels.filter((level) => level !== value);
    }
    this.valueChange(typeIdx, rowIdx, 'levels', newLevels);
  };

  errorChange = (key, error) => {
    this.errors[key] = error;
  };

  render() {
    const { onClose, studentLevelDict } = this.props;
    const { loading, data, initData, checking } = this.state;

    return (
      <Modal
        className={styles.modalWrap}
        loading={loading}
        title={
          <span>
            <span>学生进步标准设置</span>
            <IconButton
              type="question_mc"
              title={{ title: '差值=对比成绩-基准成绩', placement: 'right' }}
              style={{ marginLeft: 10 }}
            />
          </span>
        }
        width={1202}
        onOk={this.onOk}
        onCancel={onClose}
        cancelConfirm={
          objectDeepEquals(data, initData)
            ? null
            : { content: '取消保存本次修改？', okBtnText: '是', cancelBtnText: '否' }
        }
        noLine
        ref={(c) => (this.modal = getWrappedInstance(c))}
      >
        <div className={styles.wrap}>
          <div className={styles.header}>
            <div className={styles.labelCol}></div>
            <div className={styles.scoreCol + ' ' + styles.th}>分数相差</div>
            <div className={styles.classCol + ' ' + styles.th}>班级排名相差</div>
            <div className={styles.gradeCol + ' ' + styles.th}>年级排名相差</div>
            <div className={styles.levelCol + ' ' + styles.th}>评级</div>
          </div>
          {data &&
            data.map((d, i) => {
              return (
                <div key={i} className={styles.type}>
                  <div className={styles.typeTitle}>{d.name}</div>
                  <div className={styles.setting}>
                    {d.settings.map((row, j) => {
                      const label = j === 0 ? '进步' : j === 1 ? '持平' : '退步';
                      return (
                        <div key={j} className={styles.row}>
                          <div className={styles.labelCol + ' ' + styles.td}>{label}</div>
                          <div className={styles.scoreCol + ' ' + styles.td}>
                            {j === 1 ? (
                              <span>
                                <div className={styles.input}>
                                  <InputNumber
                                    value={row.score[0]}
                                    onChange={(val) => this.valueChange(i, j, 'score', val, 0)}
                                    validator={{
                                      onErrorChange: (e) => this.errorChange(i + '' + j + 'score0', e),
                                      silent: !checking,
                                      rules: ['required', (v) => checkNumber(v, numberConfig)],
                                    }}
                                  />
                                </div>
                                <span className={styles.sign2}>{`≤ 差值 ${d.name === 'A类' ? '≤' : '<'}`}</span>
                                <div className={styles.input}>
                                  <InputNumber
                                    value={row.score[1]}
                                    onChange={(val) => this.valueChange(i, j, 'score', val, 1)}
                                    validator={{
                                      onErrorChange: (e) => this.errorChange(i + '' + j + 'score1', e),
                                      silent: !checking,
                                      rules: ['required', (v) => checkNumber(v, numberConfig)],
                                    }}
                                  />
                                </div>
                              </span>
                            ) : (
                              <span>
                                <span className={styles.sign}>{j === 0 ? (d.name === 'A类' ? `>` : `≥`) : `<`}</span>
                                <div className={styles.input}>
                                  <InputNumber
                                    value={row.score}
                                    onChange={(val) => this.valueChange(i, j, 'score', val)}
                                    validator={{
                                      onErrorChange: (e) => this.errorChange(i + '' + j + 'score', e),
                                      silent: !checking,
                                      rules: ['required', (v) => checkNumber(v, numberConfig)],
                                    }}
                                  />
                                </div>
                              </span>
                            )}
                          </div>
                          <div className={styles.classCol + ' ' + styles.td}>
                            {j === 1 ? (
                              <span>
                                <div className={styles.input}>
                                  <InputNumber
                                    value={row.class[0]}
                                    onChange={(val) => this.valueChange(i, j, 'class', val, 0)}
                                    validator={{
                                      onErrorChange: (e) => this.errorChange(i + '' + j + 'class0', e),
                                      silent: !checking,
                                      rules: ['required', (v) => checkNumber(v, numberConfig2)],
                                    }}
                                  />
                                </div>
                                <span className={styles.sign2}>{`≤ 差值 ≤`}</span>
                                <div className={styles.input}>
                                  <InputNumber
                                    value={row.class[1]}
                                    onChange={(val) => this.valueChange(i, j, 'class', val, 1)}
                                    validator={{
                                      onErrorChange: (e) => this.errorChange(i + '' + j + 'class1', e),
                                      silent: !checking,
                                      rules: ['required', (v) => checkNumber(v, numberConfig2)],
                                    }}
                                  />
                                </div>
                              </span>
                            ) : (
                              <span>
                                <span className={styles.sign}>{j === 0 ? `≤` : `≥`}</span>
                                <div className={styles.input}>
                                  <InputNumber
                                    value={row.class}
                                    onChange={(val) => this.valueChange(i, j, 'class', val)}
                                    validator={{
                                      onErrorChange: (e) => this.errorChange(i + '' + j + 'class', e),
                                      silent: !checking,
                                      rules: ['required', (v) => checkNumber(v, numberConfig2)],
                                    }}
                                  />
                                </div>
                              </span>
                            )}
                          </div>
                          <div className={styles.gradeCol + ' ' + styles.td}>
                            {j === 1 ? (
                              <span>
                                <div className={styles.input}>
                                  <InputNumber
                                    value={row.grade[0]}
                                    onChange={(val) => this.valueChange(i, j, 'grade', val, 0)}
                                    validator={{
                                      onErrorChange: (e) => this.errorChange(i + '' + j + 'grade0', e),
                                      silent: !checking,
                                      rules: ['required', (v) => checkNumber(v, numberConfig2)],
                                    }}
                                  />
                                </div>
                                <span className={styles.sign2}>{`≤ 差值 ≤`}</span>
                                <div className={styles.input}>
                                  <InputNumber
                                    value={row.grade[1]}
                                    onChange={(val) => this.valueChange(i, j, 'grade', val, 1)}
                                    validator={{
                                      onErrorChange: (e) => this.errorChange(i + '' + j + 'grade1', e),
                                      silent: !checking,
                                      rules: ['required', (v) => checkNumber(v, numberConfig2)],
                                    }}
                                  />
                                </div>
                              </span>
                            ) : (
                              <span>
                                <span className={styles.sign}>{j === 0 ? `≤` : `≥`}</span>
                                <div className={styles.input}>
                                  <InputNumber
                                    value={row.grade}
                                    onChange={(val) => this.valueChange(i, j, 'grade', val)}
                                    validator={{
                                      onErrorChange: (e) => this.errorChange(i + '' + j + 'grade', e),
                                      silent: !checking,
                                      rules: ['required', (v) => checkNumber(v, numberConfig2)],
                                    }}
                                  />
                                </div>
                              </span>
                            )}
                          </div>
                          <div className={styles.levelCol + ' ' + styles.td}>
                            {(i === 0 && j === 0) || (i === data.length - 1 && j === 2) ? (
                              '-'
                            ) : (
                              <div className={styles.levels}>
                                <Validator
                                  rules={[{ required: '请选择评级' }]}
                                  onErrorChange={(e) => this.errorChange(i + '' + j + 'levels', e)}
                                  silent={!checking}
                                  value={row.levels}
                                >
                                  {studentLevelDict.map((s, n) => {
                                    const isActive = row.levels.indexOf(s.val) >= 0;
                                    return (
                                      <DotButton
                                        key={n}
                                        size={36}
                                        shape="square"
                                        className={styles.levelBtn}
                                        type={isActive ? 'primary' : null}
                                        onClick={() => this.levelChange(i, j, s.val)}
                                      >
                                        {s.text}
                                      </DotButton>
                                    );
                                  })}
                                </Validator>
                              </div>
                            )}
                          </div>
                        </div>
                      );
                    })}
                  </div>
                </div>
              );
            })}
        </div>
      </Modal>
    );
  }
}

export default ProgressStandardModal;
