/**
 * 选择城市模态框
 * @property {string} provinceCityDict 按省份分组的城市字典
 * @property {function} onSaveSuccess 保存成功的回调
 * @property {function} onClose 关闭弹框的回调
 */
import React from 'react';
import classnames from 'classnames';
import Checkbox from 'COMPONENT/Checkbox';
import Modal from 'COMPONENT/Modal';
import styles from './styles.less';
import { listTemplateCityIdsByProvinceId, initAchievementTemplateByCitys } from 'SERVICE/student';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import getWrappedInstance from 'UTIL/getWrappedInstance';

class SelectCityModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 等待状态
      selectedProvince: null, // {string} 当前选中的省份ID
      selectedCitys: [], // {[string]} 当前选中的城市ID
      fixSelectedCitys: [], // {[string]} 固定选中的城市ID
    };
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化内容
   */
  initContent = () => {
    /*    this.setState({ loading: true })
    Promise.all([
      this.loadFixSelectedCitys(),
    ]).then(([fixSelectedCitys]) => {
      this.setState({
        loading: false, fixSelectedCitys,
      })
    })*/
  };

  /**
   * 加载固定选中的城市
   */
  loadFixSelectedCitys = async (provinceId) => {
    const apiData = await listTemplateCityIdsByProvinceId({ provinceId });
    return apiData || [];
  };

  cityChange = (cityId, checked) => {
    console.log(cityId, checked);
    const { selectedCitys } = this.state;
    let newSelectedCity;
    if (checked) {
      newSelectedCity = [...selectedCitys, cityId];
    } else {
      newSelectedCity = selectedCitys.filter((c) => c !== cityId);
    }
    this.setState({ selectedCitys: newSelectedCity });
  };

  provinceChange = (provinceId) => {
    this.setState({ loading: true });
    this.loadFixSelectedCitys(provinceId).then((citys) => {
      this.setState({
        loading: false,
        selectedProvince: provinceId,
        fixSelectedCitys: [...this.state.fixSelectedCitys, ...citys],
      });
    });
  };

  onOk = () => {
    const { onSaveSuccess, onClose } = this.props;
    const { selectedCitys } = this.state;
    console.log(selectedCitys);
    if (!selectedCitys || selectedCitys.length === 0) {
      onClose();
      return;
    }
    console.log('selectedCitys:', selectedCitys);
    initAchievementTemplateByCitys({ cityIds: selectedCitys.join(',') }, true).then(
      () => {
        showMessage('保存成功', 'success');
        this.setState({ loading: false });
        onSaveSuccess();
      },
      (e) => {
        if (e.text) {
          this.modal.showOkTips(e.text);
        } else {
          showMessage('保存失败');
        }
        this.setState({ loading: false });
      },
    );
  };

  render() {
    const { provinceCityDict, onClose } = this.props;
    const { loading, selectedProvince, selectedCitys, fixSelectedCitys } = this.state;
    let subOptions = [];
    if (selectedProvince) {
      const selectedProvinceOption = provinceCityDict.find((p) => p.val === selectedProvince);
      subOptions = selectedProvinceOption.options || [selectedProvinceOption];
    }
    return (
      <Modal
        className={styles.modalWrap}
        loading={loading}
        title="选择城市"
        width={660}
        onOk={this.onOk}
        onCancel={onClose}
        noLine
        ref={(c) => (this.modal = getWrappedInstance(c))}
      >
        <div className={styles.wrap}>
          <div className={styles.left}>
            {provinceCityDict.map((p, i) => {
              const cls = {
                [styles.pItem]: 1,
                [styles.pItemSelected]: p.val === selectedProvince,
              };
              return (
                <div key={p.val} className={classnames(cls)} onClick={() => this.provinceChange(p.val)}>
                  {p.label || p.text}
                </div>
              );
            })}
          </div>
          <div className={styles.right}>
            {subOptions.map((c, i) => {
              return (
                <div key={c.val} className={styles.cItem}>
                  <Checkbox
                    checked={[...selectedCitys, ...fixSelectedCitys].indexOf(c.val) >= 0}
                    disabled={fixSelectedCitys.indexOf(c.val) >= 0}
                    onChange={(e) => this.cityChange(c.val, e.target.checked)}
                  >
                    {c.text}
                  </Checkbox>
                </div>
              );
            })}
          </div>
        </div>
      </Modal>
    );
  }
}

export default SelectCityModal;
