import React from 'react';
import { getDictTextByVal } from 'UTIL/dict';
import FormItem from 'COMPONENT/FormItem';
import Loading from 'COMPONENT/Loading';
import styles from './styles.less';
import { listAllTemplateGradeByTemplateId } from 'SERVICE/student';

const SCORE_TYPE_1 = 'SCORE_RANKING';
const SCORE_TYPE_2 = 'GRADE';

const formItemCfg = {
  size: 'small',
  labelWidth: 75,
  style: { lineHeight: '36px' },
};

/**
 * 加载模板详情
 * @param {number} id - 模板ID
 */
const loadTemplateDetail = async (id) => {
  const apiData = await listAllTemplateGradeByTemplateId({ templateId: id });
  const grades = (apiData || []).map((d) => {
    const subjects = d.subjects.map((s) => {
      return { id: s.subjectId, totalScore: s.totalScore };
    });
    return { gradeId: d.gradeId, types: d.achievementTypeList.split(','), subjects };
  });
  return grades;
};

const getTemplateDetail = (record, dicts) => {
  const { gradeDict, subjectDict, scoreTypeDict } = dicts;
  const { template } = record;
  return (
    <div className="CardList-expand-wrap">
      {template && template.length > 0 ? (
        template.map((t, i) => {
          const hasTotalScore = t.types.indexOf(SCORE_TYPE_1) >= 0;
          return (
            <div key={t.gradeId} className={styles.gItem}>
              <div className={styles.gItemLeft}>{getDictTextByVal(gradeDict, t.gradeId)}</div>
              <div className={styles.gItemRight}>
                <FormItem name="成绩类型" {...formItemCfg}>
                  {t.types.map((type) => {
                    return (
                      <div key={type} className={styles.textItem}>
                        {getDictTextByVal(scoreTypeDict, type)}
                      </div>
                    );
                  })}
                </FormItem>
                <FormItem name={hasTotalScore ? '科目总分' : '科目'} {...formItemCfg}>
                  {t.subjects.map((subject) => {
                    const name = getDictTextByVal(subjectDict, subject.id);
                    return (
                      <div key={subject.id} className={styles.textItem}>
                        {name}
                        {hasTotalScore && subject.totalScore && ` ${subject.totalScore}`}
                      </div>
                    );
                  })}
                </FormItem>
              </div>
            </div>
          );
        })
      ) : (
        <div className="empty-tips" style={{ position: 'relative' }}>
          暂无数据
          <Loading loading={!template} />
        </div>
      )}
    </div>
  );
};

const studentTypes = ['A类', 'B类', 'C类', 'D类', 'E类'];

export default {
  loadTemplateDetail,
  getTemplateDetail,
  studentTypes,
  SCORE_TYPE_1,
  SCORE_TYPE_2,
};
