import React from 'react';
import { connect } from 'dva';
/*import { Dropdown, Menu } from 'antd'*/
import Page from 'COMPONENT/Page';
import Modal from 'COMPONENT/Modal';
import SearchBar from 'COMPONENT/SearchBar';
import List from 'COMPONENT/List/CardList';
import { checkEmptyCell } from 'COMPONENT/List/utils';
import Select from 'COMPONENT/Select';
import Button from 'COMPONENT/Button';
import { loadProvinceCityDict, loadCommonDict } from 'UTIL/dict';
import SelectCityModal from './SelectCityModal';
import StudentTypeModal from './StudentTypeModal';
import ProgressStandardModal from './ProgressStandardModal';
import { getTemplateDetail, loadTemplateDetail } from './common';
import styles from './styles.less';
import Icon from 'COMPONENT/Icon';
import { findPageAchievementTemplate } from 'SERVICE/student';
import PopConfirm from 'COMPONENT/PopConfirm';
import TextLink from 'COMPONENT/TextLink';
import DropdownMenu from 'COMPONENT/DropdownMenu';

const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;

class StudentScoreTemplate extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 内容加载状态

      cityId: null, // {string} 查询条件 城市ID
      provinceCityDict: null, // {[object]} 字典 按省份分组的城市
      gradeDict: null, // {[object]} 字典 年级
      subjectDict: null, // {[object]} 字典 科目
      scoreTypeDict: null, // {[object]} 字典 成绩类型
      studentLevelDict: null, //字典，学生评级
      standardCategoryDict: null, //字典，学生进步标准类型

      list: [], // {Array} 列表数据
      listLoading: false, // {false} 列表数据加载状态
      listSearchKeys: null, // {object} 列表数据的查询关键字
      totalRecords: 0, // {number} 总记录数
      currentPage: 1, // {number} 当前页码
      pageSize: 20, // {number} 每页记录数
      expandedRowKey: null, // Number[] 列表展开行的key

      selectCityModalVisible: false, // {boolean} 选择城市弹框显示状态
      studentTypeModalVisible: false, // {boolean} 学生类别设置弹框显示状态
      progressStandardModalVisible: false, // {boolean} 进步标准设置弹框显示状态
    };
  }

  componentDidMount() {
    this.initContent();
  }

  /**
   * 初始化内容
   */
  initContent = () => {
    this.setState({ loading: true });
    Promise.all([
      loadProvinceCityDict(),
      loadCommonDict('STUDENT_GRADE'),
      loadCommonDict('SUBJECT'),
      loadCommonDict('AchievementAbstractType'),
      loadCommonDict('EvaluationType'),
      loadCommonDict('StandardCategory'),
    ]).then(([provinceCityDict, gradeDict, subjectDict, scoreTypeDict, studentLevelDict, standardCategoryDict]) => {
      //console.log(provinceCityDict, gradeDict, subjectDict, scoreTypeDict, standardCategoryDict)
      this.setState({
        loading: false,
        provinceCityDict,
        gradeDict: gradeDict.filter((g) => g.val !== 'DAT0000000393' && g.val !== 'DAT0000000409'),
        subjectDict,
        scoreTypeDict,
        studentLevelDict,
        standardCategoryDict,
      });
    });
    this.search();
  };

  /**
   * 加载列表数据，并转换格式
   * @param  {object} changedParams 要更新的列表数据参数，包括查询条件、页码状态
   */
  loadList = async (changedParams = {}) => {
    const { cityId, currentPage, pageSize, listSearchKeys } = this.state;
    const searchKeys = listSearchKeys || { cityId };
    const params = { currentPage, pageSize, ...searchKeys, ...changedParams };
    const apiParams = {
      page: params.currentPage,
      rows: params.pageSize,
      cityId: params.cityId,
    };
    this.setState({ listLoading: true });
    const apiData = await findPageAchievementTemplate(apiParams);

    /*    const apiData = await new Promise(resolve => {
      setTimeout(() => {
        resolve({
          records: 11,
          rows: [
            { cityId: '11', cityName: '东莞市', startDate: '2017-03-02', createTime: '2017-02-28 09:30', version: 'V1', creator: '李老师', template: [
                { gradeId: '402881e647ce90330147ce94ef900002', types: ['1'], subjects: [
                    { id: 'DAT0000000015', totalScore: 100 },
                    { id: 'DAT0000000016', totalScore: 100 },
                  ]
                },
                { gradeId: '402881e647ce90330147ce94ef900003', types: ['1', '2'], subjects: [
                    { id: 'DAT0000000015', totalScore: 100 },
                    { id: 'DAT0000000016', totalScore: 100 },
                  ]
                },
              ]
            },
            { cityId: '12', cityName: '惠州市', startDate: '2017-03-02', createTime: '2017-02-28 09:30', version: 'V1', creator: '李老师', template: [
                { gradeId: '402881e647ce90330147ce94ef900002', types: ['1'], subjects: [
                    { id: 'DAT0000000015', totalScore: 100 },
                    { id: 'DAT0000000016', totalScore: 100 },
                  ]
                },
                { gradeId: '402881e647ce90330147ce94ef900003', types: ['1', '2'], subjects: [
                    { id: 'DAT0000000015', totalScore: 100 },
                    { id: 'DAT0000000016', totalScore: 100 },
                  ]
                },
              ]
            },
          ],
        })
      }, 500)
    })*/
    const { records, rows } = apiData;
    const formatedRows = rows.map((row) => {
      return {
        cityId: row.cityId,
        cityName: row.cityName,
        startDate: row.effectiveDate,
        createTime: row.createTime.slice(0, 16),
        templateVersionName: 'V' + row.achievementVersion,
        creator: row.createUserName,
        templateId: row.id,
        version: row.version,
        isNew: row.effectiveDate ? false : true,
      };
    });
    const data = {
      records,
      rows: formatedRows,
    };
    console.log(data);
    const { currentPage: newCurrentPage, pageSize: newPageSize, ...newListSearchKeys } = params;
    this.setState({
      expandedRowKey: null,
      listLoading: false,
      list: data.rows,
      totalRecords: data.records,
      currentPage: newCurrentPage,
      pageSize: newPageSize,
      listSearchKeys: newListSearchKeys,
    });
    return true;
  };

  /**
   * 查询
   */
  search = () => {
    const { cityId } = this.state;
    this.loadList({ cityId, currentPage: 1 });
  };

  /**
   * 重置查询条件
   */
  reset = () => {
    this.setState({ cityId: null });
  };

  settingMenuClick = (key) => {
    if (key === 'studentType') {
      this.setState({ studentTypeModalVisible: true });
    } else if (key === 'progressStandard') {
      this.setState({ progressStandardModalVisible: true });
    }
  };

  showHistory = (cityId) => {
    const { dispatch } = this.props;
    const { gradeDict, subjectDict, scoreTypeDict } = this.state;
    dispatch({
      type: 'main/openPage',
      payload: {
        path: '/StudentScoreTemplate/History',
        params: { cityId, gradeDict, subjectDict, scoreTypeDict },
        refresh: true,
      },
    });
  };

  create = (record) => {
    const { cityId, cityName } = record;
    const { dispatch } = this.props;
    const { gradeDict, subjectDict, scoreTypeDict } = this.state;
    dispatch({
      type: 'main/openPage',
      payload: {
        path: '/StudentScoreTemplate/Edit',
        params: {
          record: { cityId, cityName },
          gradeDict,
          subjectDict,
          scoreTypeDict,
          onSaveSuccess: this.editSaveSuccess,
        },
        refresh: true,
      },
    });
  };

  edit = (record) => {
    const { dispatch } = this.props;
    const { gradeDict, subjectDict, scoreTypeDict } = this.state;
    dispatch({
      type: 'main/openPage',
      payload: {
        path: '/StudentScoreTemplate/Edit',
        params: { record, gradeDict, subjectDict, scoreTypeDict, onSaveSuccess: this.editSaveSuccess },
        refresh: true,
      },
    });
  };

  /**
   * 编辑保存成功后的处理
   */
  editSaveSuccess = () => {
    const { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: { path: '/StudentScoreTemplate' },
    });
    this.search();
  };

  onExpand = (expanded, record) => {
    const { list } = this.state;
    const { template, templateId } = record;
    if (!template) {
      loadTemplateDetail(templateId).then((t) => {
        const newList = list.map((row) => {
          if (row.templateId === templateId) {
            return { ...row, template: t };
          }
          return row;
        });
        this.setState({ list: newList });
      });
    }
  };

  getConfirmListBtn = (key, btn, onOk) => {
    return (
      <PopConfirm
        key={key}
        content="该操作可能影响成绩统计和进步率分析结果，是否继续？"
        okBtnText="是"
        cancelBtnText="否"
        onOk={onOk}
        getPopupContainer={() => this.pageRef.getScrollerElement()}
      >
        {btn}
      </PopConfirm>
    );
  };

  render() {
    const { btnAuths } = this.props;
    const {
      loading,
      cityId,
      provinceCityDict,
      gradeDict,
      subjectDict,
      scoreTypeDict,
      studentLevelDict,
      standardCategoryDict,
      list,
      totalRecords,
      currentPage,
      pageSize,
      listLoading,
      selectCityModalVisible,
      studentTypeModalVisible,
      progressStandardModalVisible,
      expandedRowKey,
    } = this.state;

    const selectCityShow = btnAuths.indexOf('ACHIEVEMENT_CITY_BTN') >= 0;
    const typeSettingShow = btnAuths.indexOf('ACHIEVEMENT_CATEGORY_BTN') >= 0;
    const standardSettingShow = btnAuths.indexOf('ACHIEVEMENT_PRO_STANDARD_BTN') >= 0;
    /*    const selectCityShow = true
    const typeSettingShow = true
    const standardSettingShow = true

    console.log(btnAuths, selectCityShow, typeSettingShow, standardSettingShow)*/

    const columns = [
      { title: '城市', dataIndex: 'cityName' },
      { title: '起始生效日期', dataIndex: 'startDate', render: checkEmptyCell },
      { title: '创建时间', dataIndex: 'createTime' },
      { title: '版本', dataIndex: 'templateVersionName', width: COL_WIDTH_S },
      { title: '创建人', dataIndex: 'creator' },
      {
        title: '操作',
        key: 'action',
        width: COL_WIDTH_L,
        render: (text, record, index) => {
          let btns = [
            this.getConfirmListBtn(1, <TextLink className="list-btn">新建</TextLink>, () => this.create(record)),
          ];
          if (!record.isNew) {
            btns = [
              ...btns,
              this.getConfirmListBtn(2, <TextLink className="list-btn">编辑</TextLink>, () => this.edit(record)),
              <TextLink key={3} className="list-btn" onClick={() => this.showHistory(record.cityId)}>
                历史版本
              </TextLink>,
            ];
          }
          return <span>{btns}</span>;
        },
      },
    ];

    let settingMenus = [];
    if (typeSettingShow) {
      settingMenus.push({ text: '学生类别设置', onClick: () => this.settingMenuClick('studentType') });
    }
    if (standardSettingShow) {
      settingMenus.push({ text: '学生进步标准设置', onClick: () => this.settingMenuClick('progressStandard') });
    }

    return (
      <Page
        fitWindowHeight={true}
        loading={loading}
        route={this.props.route}
        ref={(c) => (this.pageRef = c)}
        subHeader={
          <div className={styles.wrap}>
            <SearchBar onSearch={this.search} onReset={this.reset}>
              <Select
                placeholder="请选择城市"
                showSearch
                dict={provinceCityDict}
                value={cityId}
                onChange={(val) => this.setState({ cityId: val })}
              />
            </SearchBar>
            <div className={styles.rightBtnWrap}>
              {selectCityShow && (
                <Button type="primary" onClick={() => this.setState({ selectCityModalVisible: true })}>
                  选择城市
                </Button>
              )}
              {(typeSettingShow || standardSettingShow) && (
                <DropdownMenu
                  menus={settingMenus}
                  isCustom
                  content={<Icon type="setting_mc" size={24} />}
                  activeContent={<Icon type="setting_mc_a" size={24} />}
                  className={styles.settingBtn}
                />
              )}
            </div>
          </div>
        }
      >
        <div className={styles['list-wrapper']}>
          <List
            dataSource={list}
            columns={columns}
            loading={!loading && listLoading}
            pagination={{
              current: currentPage,
              total: totalRecords,
              pageSize,
            }}
            onChange={({ current, pageSize }) => this.loadList({ currentPage: current, pageSize })}
            expandedRowRender={(record) => getTemplateDetail(record, { gradeDict, subjectDict, scoreTypeDict })}
            noDataTips="暂无数据，请点击选择城市按钮"
            onExpand={this.onExpand}
            expandedRowKey={expandedRowKey}
            onExpandedRowChange={(expandedRowKey) => this.setState({ expandedRowKey })}
          />
        </div>
        {selectCityModalVisible && (
          <SelectCityModal
            provinceCityDict={provinceCityDict}
            onSaveSuccess={() => {
              this.setState({ selectCityModalVisible: false });
              this.search();
            }}
            onClose={() => {
              this.setState({ selectCityModalVisible: false });
            }}
          />
        )}
        {studentTypeModalVisible && (
          <StudentTypeModal
            studentLevelDict={studentLevelDict}
            onClose={() => {
              this.setState({ studentTypeModalVisible: false });
            }}
          />
        )}
        {progressStandardModalVisible && (
          <ProgressStandardModal
            studentLevelDict={studentLevelDict}
            standardCategoryDict={standardCategoryDict}
            onClose={() => {
              this.setState({ progressStandardModalVisible: false });
            }}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    btnAuths: state.account.btnAuths,
  };
};

export default connect(mapStateToProps)(StudentScoreTemplate);
