import React from 'react';
import { Row, Col } from 'antd';
import moment from 'moment';
import styles from './index.less';
import preview from 'UTIL/previewFile';
import { getIconSrc } from '../../FollowVisit/utils';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import { FILE_TYPE_MAP_EXTENSION } from 'COMPONENT/FileIcon/config';
import { saveStudentFile } from 'SERVICE/student';
import {
  Button,
  Block,
  FormItem,
  Modal,
  DatePicker,
  Select,
  MultiSelect,
  Input,
  TagsInput,
  Popover,
  TextLink,
  QrcodeFileUpload,
  Upload,
  FileList,
  PreviewImgs,
} from 'COMPONENT';
import file from 'UTIL/file';

const maxUploadFileCount = 9;
const validateFileTypes = ['word', 'excel', 'ppt', 'pdf', 'txt', 'zip', 'rar', 'png', 'jpg', 'bmp'];
const FormItemConfig = {
  labelWidth: 120,
};
let mapFileType = {
  五方会谈: 'FIVE_METTING',
  学习计划: 'STUDYPLAN',
  教案: 'TEACHINGPLAN',
  家长会: 'PARENT_METTING',
  试卷: 'EXAMPAPER',
  其他: 'OTHER',
};
export default class ViewModal extends React.Component {
  constructor(props) {
    super(props);
    this.fileIdx = 1;

    let {
      record: { studentFileRecordVos, hobby, subjectList = [], miniClassList = [], productTypes },
      title,
      productDict,
    } = this.props;
    if (!productTypes) productTypes = [];
    // this.placeholder = (title === '五方会谈' || title === '家长会') ? '可输入500个字，可从会前纪要、会中纪要、家长建议三方面进行记录。' : '可输入500个字';
    let hobbyList = hobby ? hobby.split(',') : [];
    let fileList = studentFileRecordVos
      ? studentFileRecordVos.map((file) => {
          const name = file.fileName;
          const src = file.filePath;
          return {
            id: file.fileid,
            name,
            src,
            previewPath: file.previewFilePath,
            iconSrc: getIconSrc(name, src),
            listId: this.genFileId(),
          };
        })
      : [];
    let subjectArray = subjectList.map((v, k) => v.key);
    let miniClassArray = miniClassList.map((v, k) => v.key);

    this.state = {
      edit: false,
      loading: false,
      loadingTip: null,
      previewSrc: '',
      showpreviewImgs: false,
      fileList,
      hobbyList,
      subjectArray,
      miniClassArray,

      subjectList,
      miniClassList,
      ...this.props.record,
      productTypes: JSON.parse(JSON.stringify(productTypes)),
    };
  }

  getValByText = (text, dict, type) => {
    let item = dict.find((v, k) => {
      return text === v[type];
    });
    return item ? item[type === 'text' ? 'val' : 'text'] : '';
  };

  save = () => {
    let {
      hobbyList,
      id,
      keFuYiYuan,
      xiaoGuoPingJia,
      fileList: studentFileRecordVos,
      subjectIds,
      subject,
      title: head,
      docDescription,
      productTypes = [],
      subjectArray,
      miniClassArray,
      meetingTime,
    } = this.state;
    let { studyManager, studentId, title } = this.props;
    if (!head) {
      this.modal.showOkTips('请填档案标题！');
      return;
    }
    if (title === '五方会谈' && !keFuYiYuan) {
      this.modal.showOkTips('请填写课辅意愿！');
      return;
    }

    if (studentFileRecordVos && !studentFileRecordVos.length) {
      this.modal.showOkTips('请上传档案文件!');
      return;
    }

    if ((title === '家长会' || title === '其他') && productTypes.length === 0) {
      this.modal.showOkTips('请选择产品类型！');
      return;
    }

    if ((title === '家长会' || title === '五方会谈') && !meetingTime) {
      this.modal.showOkTips('请填写会议召开时间！');
      return;
    }

    if ((title === '家长会' || title === '其他') && productTypes.length !== 0) {
      if (productTypes.includes('ONE_ON_ONE_COURSE') && !subjectArray.length) {
        this.modal.showOkTips('请选择一对一科目！');
        return;
      }
      if (productTypes.includes('SMALL_CLASS') && !miniClassArray.length) {
        this.modal.showOkTips('请选择班课名称！');
        return;
      }
    }
    if (title !== '家长会' && title !== '其他' && subjectArray.length === 0) {
      this.modal.showOkTips('请选择一对一科目！');
      return;
    }

    if (title === '家长会' && !xiaoGuoPingJia) {
      this.modal.showOkTips('请选择效果评价！');
      return;
    }

    if (studentFileRecordVos.some((file) => file.error)) {
      this.modal.showOkTips('请先处理上传失败的文件！');
      return;
    }
    let fileList = [];
    for (let file of studentFileRecordVos) {
      if (Number(file.name.split('.')[0]) === 0) {
        this.modal.showOkTips('附件名不能为空！');
        fileList = [];
        return;
      } else {
        fileList.push({
          fileid: file.id,
          previewFilePath: file.previewPath,
          fileName: file.name,
        });
      }
    }
    docDescription = (docDescription || '').replace(/\n/g, '<br/>');
    let apiParams = {
      hobby: hobbyList.join(','),
      id,
      studentId,
      keFuYiYuan,
      studentFileRecordVos: fileList,
      studentFileType: mapFileType[title],
      subjectIds:
        title === '家长会' || title === '其他'
          ? productTypes.includes('ONE_ON_ONE_COURSE')
            ? subjectArray
            : []
          : subjectArray,
      miniClassIds:
        title === '家长会' || title === '其他'
          ? productTypes.includes('SMALL_CLASS')
            ? miniClassArray
            : []
          : miniClassArray,
      subject,
      meetingTime,
      title: head,
      xiaoGuoPingJia,
      docDescription,
      studyManager,
    };
    this.setState({ loading: true });
    saveStudentFile(apiParams).then((res) => {
      showMessage('保存成功！', 'success');
      this.setState({ loading: false });
      this.props.onSuccess();
    });
  };
  getTextByMultiVal = (dict) => {
    return dict.map((v, k) => v.value).join(' ');
  };
  genFileId = () => {
    return '' + this.fileIdx++;
  };
  get2dUploadImgInfo = (info) => {
    for (let item of info) {
      if (this.state.fileList.length < 9) {
        if (JSON.stringify(this.state.fileList).indexOf(JSON.stringify(item.filePath)) === -1) {
          this.state.fileList.push({
            id: item.fileid,
            name: item.realFileName,
            src: item.filePath,
            iconSrc: getIconSrc(item.realFileName, item.filePath),
            listId: this.genFileId(),
          });
        }
      }
    }
    this.setState({
      fileList: [...this.state.fileList],
    });
  };
  onUploadChange = (info) => {
    this.setState({
      fileList: this.state.fileList.map((file) => {
        const newFile = info.find((inf) => inf.fileKey === file.fileKey);
        if (newFile) {
          const name = file.name;
          const src = newFile.src;
          return {
            ...newFile,
            id: newFile['x:fileid'],
            src,
            name,
            listId: file.listId,
            error: newFile.errorCode ? '上传失败' : null,
            iconSrc: newFile.errorCode ? null : getIconSrc(name, src),
          };
        }
        return file;
      }),
    });
  };
  checkLength = (files) => {
    let filesMayUpload = [];
    let filesWillUpload = [];
    const restCount = maxUploadFileCount - this.state.fileList.length;
    files.slice(0, restCount).map((file) => {
      const listId = this.genFileId();
      let newFile = { fileKey: file.fileKey, name: file.name, listId };
      let fileExtends = [];
      validateFileTypes.forEach((type) => {
        fileExtends = [...fileExtends, ...FILE_TYPE_MAP_EXTENSION[type]];
      });
      const name = file.name;
      const idx = name.lastIndexOf('.');
      const nameExtend = name.slice(idx + 1);
      if (file.size > 1024 * 1024 * 10) {
        newFile = { ...newFile, error: '文件大小超过了10MB' };
      } else if (fileExtends.indexOf(nameExtend.toLowerCase()) < 0) {
        newFile = { ...newFile, error: '文件格式不正确' };
      } else if (name.length - nameExtend.length - 1 > 50) {
        newFile = { ...newFile, error: '文件名超过50字' };
      } else {
        filesWillUpload.push(file);
      }
      filesMayUpload.push(newFile);
    });
    if (restCount < files.length) {
      return new Promise((res, rej) => {
        showConfirm(
          `最多上传${maxUploadFileCount}个文件，超出部分将不上传`,
          () => {
            this.setState({ fileList: [...this.state.fileList, ...filesMayUpload] });
            res(filesWillUpload);
          },
          () => {
            rej();
          },
        );
      });
    } else {
      this.setState({ fileList: [...this.state.fileList, ...filesMayUpload] });
      return filesWillUpload;
    }
  };

  setTimeLimit = (val) => {
    return moment().valueOf() < moment(val).valueOf();
  };

  static splitByFormat(fullFileName) {
    const formatPointIndex = fullFileName.lastIndexOf('.');
    if (formatPointIndex > -1) {
      return {
        name: fullFileName.slice(0, formatPointIndex),
        format: fullFileName.slice(formatPointIndex + 1),
      };
    }
    return null;
  }

  onPreviewPath = ({ previewPath, src, type, fileName }) => {
    const _path = src || previewPath;
    // 符合图片格式可点击预览
    const imageFormat = ['PNG', 'png', 'JPEG', 'jpeg', 'JPG', 'jpg', 'BMP', 'bmp', 'GIF', 'gif', 'SVG', 'svg'];
    if (imageFormat.includes(type)) {
      this.setState({ previewSrc: _path, showpreviewImgs: true });
      return;
    } else if (type !== 'pdf') {
      const result = ViewModal.splitByFormat(fileName);
      if (result) {
        const { name, format } = result;
        /**
         * 由于更换新的oss，而没有在OSS管理控制台设置对应文件类型的HTTP头，所以文件的Mime-Type都默认变为application/octet-stream
         * 导致在下载文件的时候文件没有后缀
         * */
        this.setState({
          loading: true,
          loadingTip: '文件加载中...',
        });
        return file
          .download({
            url: previewPath || src,
            fileName: name,
            format,
          })
          .then(() => {
            this.setState({
              loading: false,
              loadingTip: null,
            });
          });
      }
    }

    preview({ previewPath, src, type });
  };

  render() {
    let {
      title,
      auth,
      type,
      onClose,
      keFuYiYuanDict,
      xiaoGuoPingJiaDict,
      record,
      studentName,
      studentId,
      studyManager,
      account,
      productDict,
      subjectDict,
      miniClassDict,
    } = this.props;

    let {
      edit,
      loading,
      loadingTip,
      hobbyList,
      keFuYiYuanId,
      subjectArray,
      miniClassArray,
      subjectList,
      miniClassList,
      xiaoGuoPingJiaId,
      docDescription,
      createUserName,
      createTime,
      keFuYiYuan,
      xiaoGuoPingJia,
      fileList,
      title: head,
      meetingTime,
      productTypes = [],
      previewSrc,
      showpreviewImgs,
    } = this.state;

    let showEdit = type !== 'view' || edit === true;
    if (showEdit) {
      let productArray = productDict.map((v, k) => v.val);
      for (let i = 0; i < productTypes.length; i++) {
        if (!productArray.includes(productTypes[i])) {
          productTypes.splice(i, 1);
          i--;
        }
      }
    }

    keFuYiYuanId = keFuYiYuanId ? keFuYiYuanId : this.getValByText(keFuYiYuan, keFuYiYuanDict, 'val');
    xiaoGuoPingJiaId = xiaoGuoPingJiaId
      ? xiaoGuoPingJiaId
      : this.getValByText(xiaoGuoPingJia, xiaoGuoPingJiaDict, 'val');
    return (
      <Modal
        loading={loading}
        loadingTip={loadingTip}
        onCancel={onClose}
        title={type === 'new' ? '新增' + title : title + '详情'}
        width={890}
        okBtnShow={showEdit ? true : false}
        cancelBtnShow={showEdit ? true : false}
        closable={showEdit ? false : true}
        onOk={this.save}
        okBtnDisabled={fileList.some((v, k) => {
          return v.showProgress === true;
        })}
        className={styles.studentFile}
        ref={(c) => (this.modal = getWrappedInstance(c))}
      >
        <Block title={name} style={{ margin: '10px -45px 0' }}>
          {!showEdit &&
            createUserName === account &&
            (auth === 'all' || (auth === 'jiazhanghui' && (title === '家长会' || title === '其他'))) && (
              <div className={styles.top}>
                <span>{head}</span>
                <TextLink onClick={(e) => this.setState({ edit: true })}>编辑</TextLink>
              </div>
            )}
          <Row gutter={50}>
            {showEdit && (
              <Col span={12}>
                <FormItem {...FormItemConfig} name="档案标题" required size={!showEdit ? 'small' : 'default'}>
                  <Input
                    placeholder={'可输入20个字'}
                    value={head}
                    onChange={(e) => this.setState({ title: e.target.value })}
                    maxLength={'20'}
                  />
                </FormItem>
              </Col>
            )}

            <Col span={12}>
              <FormItem {...FormItemConfig} name="学生姓名" size={!showEdit ? 'small' : 'default'}>
                {studentName}
              </FormItem>
            </Col>
            {title === '五方会谈' && (
              <Col span={12}>
                <FormItem
                  {...FormItemConfig}
                  name="课辅意愿"
                  required={showEdit ? true : false}
                  size={!showEdit ? 'small' : 'default'}
                >
                  {showEdit ? (
                    <Select
                      placeholder={'请选择'}
                      dict={keFuYiYuanDict}
                      value={keFuYiYuan}
                      onChange={(val) => this.setState({ keFuYiYuan: val })}
                    />
                  ) : (
                    <span>{keFuYiYuanId}</span>
                  )}
                </FormItem>
              </Col>
            )}
            {title === '五方会谈' && (
              <Col span={12}>
                <FormItem
                  name="兴趣爱好"
                  {...FormItemConfig}
                  classNmae={styles.hobby}
                  size={!showEdit ? 'small' : 'default'}
                >
                  {showEdit ? (
                    <TagsInput
                      size={'default'}
                      maxNum={3}
                      maxLength={5}
                      value={hobbyList}
                      onChange={(e) => this.setState({ hobbyList: e })}
                    />
                  ) : (
                    <div className={styles.hobby}>
                      {hobbyList.map((v, k) => {
                        return <span key={k}>{v}</span>;
                      })}
                    </div>
                  )}
                </FormItem>
              </Col>
            )}

            <Col span={12}>
              <FormItem
                {...FormItemConfig}
                name="产品类型"
                classNmae={styles.productType}
                required={showEdit ? true : false}
                size={!showEdit ? 'small' : 'default'}
              >
                {title === '家长会' || title === '其他' ? (
                  <div>
                    {showEdit ? (
                      <MultiSelect
                        placeholder={'请选择'}
                        dropdownClassName={styles.multiSelect}
                        dict={productDict}
                        value={productTypes}
                        onChange={(val) => this.setState({ productTypes: val })}
                      />
                    ) : (
                      <span>
                        {(productTypes.includes('ONE_ON_ONE_COURSE') ? '1对1' : '') +
                          ' ' +
                          (productTypes.includes('SMALL_CLASS') ? '十人课堂' : '')}
                      </span>
                    )}
                  </div>
                ) : (
                  <span>1对1</span>
                )}
              </FormItem>
            </Col>
            {((title !== '家长会' && title !== '其他') ||
              ((title === '家长会' || title === '其他') && productTypes.includes('ONE_ON_ONE_COURSE'))) && (
              <Col span={12}>
                <FormItem
                  {...FormItemConfig}
                  name="1对1科目"
                  required={showEdit ? true : false}
                  size={!showEdit ? 'small' : 'default'}
                >
                  {showEdit ? (
                    <MultiSelect
                      placeholder={'请选择'}
                      dropdownClassName={styles.multiSelect}
                      dict={subjectDict}
                      value={subjectArray}
                      noOptionTips={'请先到课时管理分配课时'}
                      onChange={(val) => this.setState({ subjectArray: val })}
                    />
                  ) : (
                    <span>{this.getTextByMultiVal(subjectList)}</span>
                  )}
                </FormItem>
              </Col>
            )}
            {(title === '家长会' || title === '其他') && productTypes.includes('SMALL_CLASS') && (
              <Col span={12}>
                <FormItem
                  {...FormItemConfig}
                  name="班课名称"
                  required={showEdit ? true : false}
                  size={!showEdit ? 'small' : 'default'}
                >
                  {showEdit ? (
                    <MultiSelect
                      placeholder={'请选择'}
                      dict={miniClassDict}
                      value={miniClassArray}
                      noOptionTips={'请先报入小班'}
                      onChange={(val) => this.setState({ miniClassArray: val })}
                    />
                  ) : (
                    <span>{this.getTextByMultiVal(miniClassList)}</span>
                  )}
                </FormItem>
              </Col>
            )}

            {title === '家长会' && (
              <Col span={12}>
                <FormItem
                  {...FormItemConfig}
                  name="效果评价"
                  required={showEdit ? true : false}
                  size={!showEdit ? 'small' : 'default'}
                >
                  {showEdit ? (
                    <Select
                      placeholder={'请选择'}
                      dict={xiaoGuoPingJiaDict}
                      value={xiaoGuoPingJia}
                      onChange={(val) => this.setState({ xiaoGuoPingJia: val })}
                    />
                  ) : (
                    <span>{xiaoGuoPingJiaId}</span>
                  )}
                </FormItem>
              </Col>
            )}
            {title === '家长会' && (
              <Col span={12}>
                <FormItem {...FormItemConfig} name="学管师" size={!showEdit ? 'small' : 'default'}>
                  {studyManager}
                </FormItem>
              </Col>
            )}
            {!showEdit && (
              <Col span={12}>
                <FormItem {...FormItemConfig} name="创建人" size={!showEdit ? 'small' : 'default'}>
                  {createUserName}
                </FormItem>
              </Col>
            )}
            {!showEdit && (
              <Col span={12}>
                <FormItem {...FormItemConfig} name="创建时间" size={!showEdit ? 'small' : 'default'}>
                  {createTime}
                </FormItem>
              </Col>
            )}
            {(title === '家长会' || title === '五方会谈') && (
              <Col span={12}>
                <FormItem {...FormItemConfig} name="会议召开时间" classNmae={styles.productType} required={true}>
                  {showEdit && !record.meetingTime ? (
                    <DatePicker
                      value={meetingTime}
                      placeholder="请选择日期"
                      onChange={(val) => this.setState({ meetingTime: val })}
                      disabledDate={(val) => this.setTimeLimit(val)}
                    />
                  ) : (
                    <span>{record.meetingTime}</span>
                  )}
                </FormItem>
              </Col>
            )}
            <Col span={24}>
              <FormItem {...FormItemConfig} name="档案文件" size={!showEdit ? 'small' : 'default'} required={true}>
                {showEdit ? (
                  <div>
                    <div>
                      <div style={{ display: 'inline-block', marginRight: 20 }}>
                        {fileList.length >= maxUploadFileCount ? (
                          <Button disabled>点击上传</Button>
                        ) : (
                          <Upload
                            onChange={this.onUploadChange}
                            multiple={true}
                            beforeUpload={(files) => this.checkLength(files)}
                          >
                            <Button>点击上传</Button>
                          </Upload>
                        )}
                      </div>
                      <div style={{ display: 'inline-block' }}>
                        {fileList.length >= maxUploadFileCount ? (
                          <Button disabled>二维码传图</Button>
                        ) : (
                          <Popover
                            content={
                              <QrcodeFileUpload
                                height={160}
                                width={160}
                                multiple={true}
                                callback={this.get2dUploadImgInfo}
                              />
                            }
                            overlayClassName={styles.codePopWrap}
                            placement="right"
                          >
                            <Button>扫码传图</Button>
                          </Popover>
                        )}
                      </div>
                      <div className={styles.tips}>
                        支持格式有{validateFileTypes.join('、')}，最多上传9个附件，可多选上传
                      </div>
                    </div>
                    <div style={{ marginTop: 5 }}>
                      <FileList
                        data={fileList.map((file) => {
                          return { ...file, fileId: file.id, id: file.listId };
                        })}
                        onChange={(list) =>
                          this.setState({
                            fileList: list.map((file) => {
                              return { ...file, id: file.fileId, listId: file.id };
                            }),
                          })
                        }
                      />
                    </div>
                  </div>
                ) : (
                  <FileList
                    data={fileList.map((file) => {
                      return { ...file, fileId: file.id, id: file.listId };
                    })}
                    type="view"
                    onPreview={this.onPreviewPath}
                  />
                )}
              </FormItem>
            </Col>

            <Col span={24}>
              <FormItem {...FormItemConfig} name="备注" required={false} size={!showEdit ? 'small' : 'default'}>
                {showEdit ? (
                  <Input
                    placeholder={this.placeholder}
                    value={
                      docDescription
                        ? docDescription.includes('<br/>')
                          ? docDescription.replace(/\<br\/\>/g, '\n').replace(/(\n){2,}/g, '\n')
                          : docDescription
                        : ''
                    }
                    onChange={(e) => this.setState({ docDescription: e.target.value })}
                  />
                ) : (
                  <span
                    dangerouslySetInnerHTML={{ __html: docDescription.replace(/(\<br\/\>){2,}/g, '<br/>') }}
                    className={styles.details}
                  ></span>
                )}
              </FormItem>
            </Col>
          </Row>
        </Block>
        {showpreviewImgs && (
          <Modal
            title="图片预览"
            onCancel={() => {
              this.setState({ showpreviewImgs: false });
            }}
            cancelBtnShow={false}
            okBtnShow={false}
          >
            <PreviewImgs showButton={false} activeSrc={previewSrc} showThumbnail={false} />
          </Modal>
        )}
      </Modal>
    );
  }
}
