import React, { PropTypes } from 'react';
import style from './index.less';

class Iframe extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      height: 'auto',
    };
    this.resizing = false;
    this.init = this.init.bind(this);
    this.setWrapHeight = this.setWrapHeight.bind(this);
    this.receiveMessage = this.receiveMessage.bind(this);
    this.beforeDestroy = this.beforeDestroy.bind(this);
  }
  static classPrefix = 'Iframe';
  static propTypes = {
    src: PropTypes.string.isRequired,
    receiveMsgConfig: PropTypes.array,
  };
  componentDidMount() {
    this.init();
    window.addEventListener('resize', this.setWrapHeight, false);
  }
  componentWillUnmount() {
    this.beforeDestroy();
    window.removeEventListener('resize', this.setWrapHeight, false);
  }
  // 初始化
  init() {
    this.setWrapHeight();
    const { receiveMsgConfig } = this.props;
    if (receiveMsgConfig && receiveMsgConfig.length > 0) {
      window.addEventListener('message', this.receiveMessage, false);
    }
  }
  // 设置iframe高度
  setWrapHeight() {
    if (!this.resizing) {
      this.resizing = true;
      this.setState({ height: document.body.clientHeight - 125 }, () => {
        this.resizing = false;
      });
    }
  }
  // 接收iframe回传的消息
  receiveMessage({ origin, data }) {
    const { receiveMsgConfig } = this.props;
    let index = receiveMsgConfig.findIndex((item) => item.event === data.event);
    if (index !== -1 && receiveMsgConfig[index].callback) {
      receiveMsgConfig[index].callback();
    }
  }
  // 组件注销前执行
  beforeDestroy() {
    const { receiveMsgConfig } = this.props;
    if (receiveMsgConfig && receiveMsgConfig.length > 0) {
      window.removeEventListener('message', this.receiveMessage, false);
    }
  }

  render() {
    const { src } = this.props;
    const { height } = this.state;
    const prefix = Iframe.classPrefix;

    return (
      <div className={style[prefix]}>
        <div className={style[`${prefix}-wrap`]}>
          <iframe src={src} style={{ height }}>
            {' '}
          </iframe>
        </div>
      </div>
    );
  }
}

export default Iframe;
