/**
 * Created by Haolin on 2019/1/21.
 */
import React, { PropTypes } from 'react';
import classNames from 'classnames';
import compStyle from './index.less';

/**
 * 切换显示组件子容器
 * @property {jsx} children - 内容
 * @property {string} type children - 显示类型 'show' | 'if'
 * @property {string, number} key
 * @property {string, number} activeKey
 * */
const SwitchItemWrap = (props) => {
  const { children, type, comKey, activeKey } = props;
  const prefix = 'SwitchItem';
  let typeShow = type === 'show';
  return (
    <div
      className={compStyle[`${prefix}-type-${type}`]}
      style={typeShow ? { display: activeKey === comKey ? 'block' : 'none' } : {}}
    >
      {typeShow ? children : activeKey === comKey && children}
    </div>
  );
};

class Switch extends React.Component {
  constructor(props) {
    super(props);
  }
  static classPrefix = 'Switch';
  static propTypes = {
    activeKey: PropTypes.oneOfType([PropTypes.string, PropTypes.number]).isRequired,
  };

  render() {
    const { children, activeKey, className, style } = this.props;
    const prefix = Switch.classPrefix;
    const checkNodeLen = children.length > 0;

    return (
      <div className={classNames(compStyle[prefix], className || '')} style={style || {}}>
        {checkNodeLen &&
          children.map((item, index) => {
            const { children, type, comKey } = item.props;
            return (
              <SwitchItemWrap type={type} comKey={comKey} activeKey={activeKey} key={comKey} {...item.props}>
                {children}
              </SwitchItemWrap>
            );
          })}
      </div>
    );
  }
}

class SwitchItem extends React.Component {
  static propTypes = {
    children: PropTypes.node,
    type: PropTypes.string,
    comKey: PropTypes.oneOfType([PropTypes.string, PropTypes.number]).isRequired,
  };
  static defaultProps = {
    type: 'show',
  };

  render() {
    return <div {...this.props}> </div>;
  }
}

Switch.SwitchItem = SwitchItem;

export default Switch;
