/**
 * Created by luoming on 2018/3/13
 */
import React, { PropTypes } from 'react';
import { connect } from 'dva';
import moment from 'moment';
import styles from './index.less';
import Page from 'COMPONENT/Page';
import SearchInput from 'COMPONENT/Input/SearchInput';
import SearchBar from 'COMPONENT/SearchBar';
import DatePicker from 'COMPONENT/DatePicker';
import Select from 'COMPONENT/Select';
import Input from 'COMPONENT/Input';
import Button from 'COMPONENT/Button';
import Popover from 'COMPONENT/Popover';
import FormItem from 'COMPONENT/FormItem';
import NoData from 'COMPONENT/NoData';
import Block from './Block/index';
import getWindowSize from 'UTIL/getWindowSize';
import ViewModal from './ViewModal';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { deleteStudentFile, getFileList, showPreview, getFileDataFromReport } from 'SERVICE/student';
import { getTeaToken, queryBossTutorPlanList, createBossTutorPlan, delBossTutorPlan } from 'SERVICE/jiaoxueApi';
import {
  getSubject,
  getMiniClass,
  loadStudyManagerDict,
  loadTeacherDict,
  loadHomeroomTeacherDict,
} from '../FollowVisit/loaders';
import { loadCommonDict } from 'UTIL/dict';
import Switch from './components/Switch';
import Iframe from './components/Iframe';
import { jiaoXuePageUrl } from 'CONST/otherSystemConfig';

const SwitchItem = Switch.SwitchItem;
let mapType = {
  // FIVE_METTING: '五方会谈',
  STUDYPLAN: '学习计划',
  TEACHINGPLAN: '教案',
  // PARENT_METTING: '家长会',
  EXAMPAPER: '试卷',
  OTHER: '其他',
};
const filesDict = [
  { val: 'TUTOR_PLAN', text: '续费个辅方案' },
  // { val: 'FIVE_METTING', text: '五方会谈' },
  { val: 'STUDYPLAN', text: '学习计划' },
  { val: 'TEACHINGPLAN', text: '教案' },
  // { val: 'PARENT_METTING', text: '家长会' },
  { val: 'EXAMPAPER', text: '试卷' },
  { val: 'OTHER', text: '其他' },
];
const productDict = [
  { val: 'ONE_ON_ONE_COURSE', text: '1对1' },
  { val: 'SMALL_CLASS', text: '十人课堂' },
];
const ONE_ON_ONE_COURSE_VAL = 'ONE_ON_ONE_COURSE';
const SMALL_CLASS_VAL = 'SMALL_CLASS';
const jiaoXueEditPageRouter = 'one-to-one-teaching-service/boss-individual-tutor-plan-edit';
const jiaoXueViewPageRouter = 'one-to-one-teaching-service/individual-tutor-plan-detail';

const AddNewBtnPop = (props) => {
  const { config, activeKey } = props;
  return (
    <div className={styles.btnWrap}>
      {config.map((item, index) => {
        return (
          <Button
            // type={activeKey === index ? 'primary' : 'default'}
            onClick={() => {
              item.onClick(item);
            }}
            key={index}
          >
            {item.text}
          </Button>
        );
      })}
    </div>
  );
};
AddNewBtnPop.propTypes = {
  config: PropTypes.array.isRequired,
};

class StudentFile extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      startDate: moment().year() - 2 + '-01-01',
      endDate: moment().format('YYYY-MM-DD'),
      id: null,
      title: null,
      viewModal: false,
      winW: 0,
      markArray: [],
      loading: false,
      list: [],
      record: {},
      productType: null,
      studentFileType: null,
      // 续费-课程方案相关状态
      tutorPlanList: [],
      // 参照原作者 markArray 写的，配合 showTutorPlanAll 使用
      tutorPlanMarkArray: [],
      switchActiveKey: 'page',
    };
    this.hadToken = false;
    this.tutorPlanUrl = '';
    this.auth = 'all';
    this.modalProductDict = [];
    // 续费课程方案-权限
    this.userAuthStorage = null;
    this.init = this.init.bind(this);
    this.getTeaAccessToken = this.getTeaAccessToken.bind(this);
    this.loadTutorPlanList = this.loadTutorPlanList.bind(this);
    this.getTutorPlanList = this.getTutorPlanList.bind(this);
    this.buildTutorPlanList = this.buildTutorPlanList.bind(this);
    this.tutorPlanClick = this.tutorPlanClick.bind(this);
    this.tutorPlanMoreAction = this.tutorPlanMoreAction.bind(this);
    this.buildAddNewConfig = this.buildAddNewConfig.bind(this);
    this.getUserAuth = this.getUserAuth.bind(this);
    this.checkAuth = this.checkAuth.bind(this);
    this.addNewClick = this.addNewClick.bind(this);
    this.addTutorPlan = this.addTutorPlan.bind(this);
    this.getTutorPlanId = this.getTutorPlanId.bind(this);
    this.openTutorPlanEdit = this.openTutorPlanEdit.bind(this);
    this.openTutorPlanView = this.openTutorPlanView.bind(this);
    this.switchPage = this.switchPage.bind(this);
    this.deleteTutorPlan = this.deleteTutorPlan.bind(this);
    this.delBossTutorPlan = this.delBossTutorPlan.bind(this);
  }
  // 无权限时提示消息
  static notPassMsg(auth) {
    let msg = '';
    switch (auth) {
      case null:
        msg = '只允许学生的学管师、班主任或授课老师/辅导老师新增档案！';
        break;
      default:
        msg = null;
    }
    if (msg) showMessage(msg, 'error');
  }

  componentDidMount() {
    this.updateWinW();
    window.addEventListener('resize', this.updateWinW, false);
    this.init();
  }
  componentWillUnmount() {
    window.removeEventListener('resize', this.updateWinW, false);
  }

  async init() {
    this.setState({ loading: true });
    let { studentId, from, ids } = this.props;
    const [
      { list, data },
      keFuYiYuanDict,
      xiaoGuoPingJiaDict,
      subjectDict,
      miniClassDict,
      managerDict,
      teacherDict,
      homeroomTeacherDict,
    ] = await Promise.all([
      from ? this.loadData(ids) : this.loadData(),
      loadCommonDict('KeFuYiYuan'),
      loadCommonDict('XiaoGuoPingJia'),
      getSubject(studentId),
      getMiniClass(studentId),
      loadStudyManagerDict(studentId),
      loadTeacherDict(studentId),
      loadHomeroomTeacherDict(studentId),
    ]);
    let { userId } = this.props;
    this.judgeAuth(undefined, userId, managerDict, teacherDict, homeroomTeacherDict);
    const _self = this;
    this.setState(
      {
        keFuYiYuanDict,
        xiaoGuoPingJiaDict,
        subjectDict,
        list,
        initData: data,
        miniClassDict,
        managerDict,
        teacherDict,
        homeroomTeacherDict,
      },
      () => {
        _self.getUserAuth();
      },
    );
    const getToken = await this.getTeaAccessToken().catch(() => {});
    let tutorPlanList = [];
    if (getToken) {
      tutorPlanList = await this.getTutorPlanList().catch((err) => {
        this.setState({ loading: false });
      });
    }
    this.setState({
      tutorPlanList,
      loading: false,
    });
  }
  search = async () => {
    this.setState({ loading: true });
    let { list, data } = await this.loadData();
    let tutorPlanList = [];
    let rspList = await this.getTutorPlanList().catch(() => {});
    if (rspList) tutorPlanList = rspList;
    this.setState({
      loading: false,
      list,
      initData: data,
      tutorPlanList,
    });
  };
  showAll = (e) => {
    let { list, markArray } = this.state;
    let index = list.findIndex((v, k) => {
      return v === e;
    });
    markArray[index] = !markArray[index];
    this.setState({ markArray });
  };
  showTutorPlanAll = (e) => {
    let { tutorPlanList, tutorPlanMarkArray } = this.state;
    let index = tutorPlanList.findIndex((v, k) => {
      return v === e;
    });
    tutorPlanMarkArray[index] = !tutorPlanMarkArray[index];
    this.setState({ tutorPlanMarkArray });
  };
  loadData = async (ids) => {
    let { studentId } = this.props;
    let apiParams, data;
    if (ids) {
      apiParams = { ids, studentId };
      data = await getFileDataFromReport(apiParams);
    } else {
      let { fileName, startDate, endDate, miniClassName, productType, studentFileType, managerId, subjectId, teacher } =
        this.state;
      apiParams = {
        studentId,
        title: fileName,
        startTime: startDate,
        endTime: endDate,
        miniClassName,
        subjectId,
        productType,
        studentFileType,
        studyManager: managerId,
        teacher,
      };
      data = await getFileList(apiParams);
    }
    let typeArray = [];
    let list = [];
    data.forEach((v, k) => {
      if (!typeArray.includes(v.studentFileType)) {
        typeArray.push(v.studentFileType);
      }
    });
    typeArray.forEach((v, k) => {
      let item = { title: mapType[v], list: [] };
      data.forEach((V, K) => {
        if (v === V.studentFileType) {
          item.list.push(V);
        }
      });
      list.push(item);
    });
    return { list, data };
  };
  reset = () => {
    this.setState({
      fileName: null,
      startDate: null,
      endDate: null,
      miniClassName: null,
      productType: null,
      studentFileType: null,
      studyManager: null,
      subjectId: null,
      teacher: null,
    });
  };
  updateWinW = () => {
    this.setState({ winW: getWindowSize().width });
  };
  dropDownClick = (type, id, title) => {
    if (type === 'edit') {
      this.showModalClick(id, title, type);
    } else {
      showConfirm(
        '确定删除这条档案么？',
        () => {
          this.setState({ loading: true });
          deleteStudentFile({ id })
            .then(async (res) => {
              showMessage('删除成功！', 'success');
              let { list, data } = await this.loadData();
              this.setState({ list, initData: data, loading: false });
            })
            .catch((res) => {
              this.setState({ loading: false });
            });
        },
        () => {},
      );
    }
  };
  showModalClick = (id, title, type) => {
    let { initData, managerDict, teacherDict, homeroomTeacherDict } = this.state;
    let { userId } = this.props;
    let record = initData.find((v, k) => {
      return v.id === id;
    });
    this.judgeAuth(title, userId, managerDict, teacherDict, homeroomTeacherDict);

    this.setState({ id, viewModal: true, title, type, record });
  };
  newAddClick = (title, index) => {
    let { managerDict, teacherDict, homeroomTeacherDict } = this.state;
    let { userId } = this.props;
    let data = this.judgeAuth(title, userId, managerDict, teacherDict, homeroomTeacherDict);
    if (typeof data === 'string') {
      showMessage(data);
      return;
    }
    this.setState({ visible: false, title, viewModal: true, type: 'new', record: {} });
  };
  judgeAuth = (title, userId, managerDict, teacherDict, homeroomTeacherDict) => {
    let isOTOTeacher, isSmallTeacher, isManager, isMaster, isOTOHomeroom, isSmallHomeroom;
    let teaItem = teacherDict.find((v, k) => v.val === userId);
    if (teaItem) {
      if (teaItem.type.includes('ONE_ON_ONE_COURSE')) {
        isOTOTeacher = true;
      }
      if (teaItem.type.includes('SMALL_CLASS')) {
        isSmallTeacher = true;
      }
    }
    let managerItem = managerDict.find((v, k) => v.val === userId);
    if (managerItem) {
      if (managerItem.type.includes('ONE_ON_ONE_COURSE')) {
        isManager = true;
      }
      if (managerItem.type.includes('SMALL_CLASS')) {
        isMaster = true;
      }
    }
    let homeroomItem = homeroomTeacherDict.find((v, k) => v.val === userId);
    if (homeroomItem) {
      if (homeroomItem.type.includes('ONE_ON_ONE_COURSE')) {
        isOTOHomeroom = true;
      }
      if (homeroomItem.type.includes('SMALL_CLASS')) {
        isSmallHomeroom = true;
      }
    }

    if (!isOTOTeacher && !isSmallTeacher && !isManager && !isMaster && !isOTOHomeroom && !isSmallHomeroom) {
      this.auth = null;
      return '只允许学生的学管师、班主任或授课老师/辅导老师新增档案！';
    }
    if ((isSmallTeacher || isMaster || isSmallHomeroom) && !isOTOTeacher && !isManager && !isOTOHomeroom) {
      this.auth = 'jiazhanghui';
      if (title !== '家长会' && title !== '其他') {
        return '班课老师和班主任只能增加档案类型为家长会和其他的档案！';
      }
      this.modalProductDict = [{ val: 'SMALL_CLASS', text: '十人课堂' }];
    }
    if ((isOTOTeacher || isManager || isOTOHomeroom) && !isSmallTeacher && !isMaster && !isSmallHomeroom) {
      this.auth = 'all';
      this.modalProductDict = [{ val: 'ONE_ON_ONE_COURSE', text: '1对1' }];
    }
    if ((isOTOTeacher || isManager || isOTOHomeroom) && (isMaster || isSmallTeacher || isSmallHomeroom)) {
      this.auth = 'all';
      this.modalProductDict = [
        { val: 'ONE_ON_ONE_COURSE', text: '1对1' },
        { val: 'SMALL_CLASS', text: '十人课堂' },
      ];
    }
  };
  onSuccess = async () => {
    let { list, data } = await this.loadData();
    this.setState({ list, initData: data, viewModal: false });
  };
  getDisabledStartDate = (startDate) => {
    let { endDate } = this.state;
    if (!startDate || !endDate) {
      return false;
    }
    return moment(startDate).valueOf() > moment(endDate).valueOf();
  };
  getDisabledEndDate = (endDate) => {
    const { startDate } = this.state;
    if (!endDate || !startDate) {
      return false;
    }
    return moment(endDate).valueOf() < moment(startDate).valueOf();
  };

  // 用于教学的页面后退一页
  static goBackForIframe() {
    history.back();
  }
  // 用于getTeaAccessToken，创建一个日期数字(例如: 20190123)
  static nowDateNum() {
    let date = new Date();
    let y = date.getFullYear().toString();
    let m = date.getMonth() + 1;
    let d = date.getDate();
    if (m < 10) m = `0${m}`;
    if (d < 10) d = `0${d}`;
    return Number(`${y}${m}${d}`);
  }
  // 获取token
  getTeaAccessToken() {
    const { userId } = this.props;
    return new Promise((resolve, reject) => {
      let tokenStorage = localStorage.getItem('teaToken');
      let tokenDate = StudentFile.nowDateNum();
      if (tokenStorage) {
        tokenStorage = JSON.parse(tokenStorage);
        if (tokenStorage.date === tokenDate && tokenStorage.userId === userId) {
          this.hadToken = true;
          resolve(true);
          return;
        }
      }
      let fetchParams = { code: Index.getLoginUser().token };
      getTeaToken({ params: fetchParams })
        .then((rsp) => {
          if (rsp && rsp.success) {
            localStorage.setItem(
              'teaToken',
              JSON.stringify({
                date: tokenDate,
                token: rsp.data['access-token'],
                userId,
              }),
            );
            this.hadToken = true;
            resolve(true);
          } else {
            showMessage('获取教学平台token失败，请联系管理员！', 'error');
            reject(false);
          }
        })
        .catch((err) => {
          showMessage('获取教学平台token失败，请联系管理员！', 'error');
          console.error(err);
          reject(false);
        });
    });
  }
  // 加载续费-课程方案列表
  async loadTutorPlanList() {
    this.setState({ loading: true });
    let tutorPlanList = [];
    let rspList = await this.getTutorPlanList().catch(() => {});
    if (rspList) tutorPlanList = rspList;
    this.setState({
      tutorPlanList,
      loading: false,
    });
  }
  // 获取续费-课程方案列表数据
  getTutorPlanList() {
    const _self = this;
    const { startDate, endDate, managerId, teacher } = this.state;
    return new Promise((resolve, reject) => {
      if (!_self.hadToken) {
        resolve(null);
      }
      let bossTeaIds = [];
      if (managerId) {
        bossTeaIds.push(managerId);
      }
      if (teacher) {
        bossTeaIds.push(teacher);
      }
      bossTeaIds = Array.from(new Set(bossTeaIds));
      if (bossTeaIds.length === 0) {
        bossTeaIds = null;
      }
      let fetchParams = {
        startDate: startDate ? StudentFile.buildDateParams(startDate) : null,
        endDate: endDate ? StudentFile.buildDateParams(endDate) : null,
        bossTeaIds,
        bossStuId: _self.props.studentId,
        pageNum: 1,
        pageSize: 100,
      };
      let headers = { 'access-token': JSON.parse(localStorage.getItem('teaToken')).token };
      queryBossTutorPlanList({ params: fetchParams, headers })
        .then((rsp) => {
          let list = [];
          if (rsp && rsp.success && rsp.data) {
            let originList = rsp.data.result;
            if (originList && originList.length > 0) {
              list = _self.buildTutorPlanList(originList);
              resolve([{ title: '续费课程方案', list }]);
            } else {
              resolve([]);
            }
          }
        })
        .catch((err) => {
          showMessage(`获取学管续费个辅列表数据错误，请联系管理员！(${err.text})`, 'error');
          reject();
        });
    });
  }
  static buildDateParams(dateStr) {
    return dateStr.replace(/-/g, '/');
  }
  // 将教学返回的数据转成Block组件所需数据格式
  buildTutorPlanList(list) {
    return list.map((item) => ({
      title: `${item.bossStuName}/${item.bossGradeName}/续费课程方案`,
      id: item.bossTutorPlanId,
      createTime: moment(item.createDate).format('YYYY-MM-DD HH:mm:ss'),
      createUserName: item.bossTeaName,
      miniClassList: [],
      // subjectList: [{}],
      subjectList: [],
      studentFileRecordVos: [],
    }));
  }
  // 点击续费-课程方案卡片
  tutorPlanClick(id, title, data) {
    if (data.createUserName === this.props.account) {
      this.openTutorPlanEdit(id);
    } else {
      this.openTutorPlanView(id);
    }
  }
  // 续费-课程方案卡片更多操作
  tutorPlanMoreAction(type, id, title) {
    if (type === 'edit') {
      this.openTutorPlanEdit(id, type);
    } else if (type === 'delete') {
      this.deleteTutorPlan(id);
    }
  }
  // 构建新增按钮弹窗的弹窗content配置
  buildAddNewConfig() {
    return [
      {
        text: '续费-课程方案',
        onClick: this.addTutorPlan,
        auth: ['all'],
      },
      // {
      //   text: '五方会谈',
      //   onClick: this.addNewClick,
      //   auth: ['all', 'oneOnOne'],
      // },
      {
        text: '学习计划',
        onClick: this.addNewClick,
        auth: ['all', 'oneOnOne'],
      },
      // {
      //   text: '家长会',
      //   onClick: this.addNewClick,
      //   auth: ['all', 'oneOnOne', 'smallClass'],
      // },
      {
        text: '试卷',
        onClick: this.addNewClick,
        auth: ['all', 'oneOnOne'],
      },
      {
        text: '教案',
        onClick: this.addNewClick,
        auth: ['all', 'oneOnOne'],
      },
      {
        text: '其他',
        onClick: this.addNewClick,
        auth: ['all', 'oneOnOne', 'smallClass'],
      },
    ];
  }
  // 获取当前登录用户权限
  getUserAuth() {
    let result = null;
    if (this.userAuthStorage) {
      // 使用缓存权限
      result = this.userAuthStorage;
    } else {
      // 创建权限
      const { userId } = this.props;
      const { teacherDict, managerDict } = this.state;
      let teacher = teacherDict.find((item) => item.val === userId);
      let manager = managerDict.find((item) => item.val === userId);
      if (manager || teacher) {
        result = 'all';
      }
      // let oneOnOne = {
      //   teacher: teacher ? teacher.type.includes(ONE_ON_ONE_COURSE_VAL) : false,
      //   manager: manager ? manager.type.includes(ONE_ON_ONE_COURSE_VAL) : false,
      // };
      // let smallClass = {
      //   teacher: teacher ? teacher.type.includes(SMALL_CLASS_VAL) : false,
      //   manager: manager ? manager.type.includes(SMALL_CLASS_VAL) : false,
      // };
      // let isOneOnOne = oneOnOne.teacher || oneOnOne.manager;
      // let isSmallClass = smallClass.teacher || smallClass.manager;
      // let notOneOnOne = !oneOnOne.teacher && !oneOnOne.manager;
      // let notSmallClass = !smallClass.teacher && !smallClass.manager;
      //
      // if (notOneOnOne && notSmallClass) {
      //   result = null;
      // } else if (isSmallClass && notOneOnOne) {
      //   result = 'smallClass';
      //   return this.auth;
      // } else if (isOneOnOne && notSmallClass) {
      //   result = 'oneOnOne'
      // } else {
      //   result = 'all';
      // }
      this.userAuthStorage = result;
    }
    return result;
  }
  // 校验权限
  checkAuth(authList) {
    let auth = this.getUserAuth();
    if (authList.indexOf(auth) === -1) {
      StudentFile.notPassMsg(auth);
      return false;
    }
    return true;
  }
  // 新增档案(除续费-课程方案)
  addNewClick(params) {
    this.newAddClick(params.text);
  }
  // 新增续费-课程方案
  async addTutorPlan(params) {
    if (!this.hadToken) {
      showMessage('缺少教学平台token，无法新增续费-课程方案，请联系管理员！');
      return;
    }
    if (!this.checkAuth(params.auth)) {
      return;
    }
    /**
     * 去掉了权限校验
     *
     * this.props.isOneOnOne
     * */
    this.setState({ loading: true, visible: false });
    let newPage = window.open('about:blank');
    const rsp = await this.getTutorPlanId();
    if (rsp && rsp.data) {
      // 打开教学页面
      let newDoc = newPage.document;
      newDoc.location.replace(`${jiaoXuePageUrl}/#/${jiaoXueEditPageRouter}?tutorPlanId=${rsp.data}`);
      // 刷新续费课程方案列表
      let tutorPlanList = await this.getTutorPlanList();
      this.setState({
        tutorPlanList,
        loading: false,
      });
    } else {
      this.setState({ loading: false });
    }
  }
  // 获取续费个辅id
  getTutorPlanId() {
    let fetchParams = {
      bossGradeId: this.props.gradeId,
      bossGradeName: this.props.gradeName,
      bossStuId: this.props.studentId,
      bossStuName: this.props.studentName,
      // bossTeaId: this.props.userId,
      // bossTeaName: this.props.account,
    };
    let headers = { 'access-token': JSON.parse(localStorage.getItem('teaToken')).token };
    return createBossTutorPlan({ params: fetchParams, headers }).catch((err) => {
      showMessage('获取学管续费个辅ID错误，请联系管理员！', 'error');
      console.error('Error - createBossTutorPlan :', err);
    });
  }
  // 打开续费个辅页面
  openTutorPlanEdit(id) {
    // 浏览器新标签页打开
    let newPage = window.open('about:blank');
    setTimeout(() => {
      let newDoc = newPage.document;
      newDoc.location.replace(`${jiaoXuePageUrl}/#/${jiaoXueEditPageRouter}?tutorPlanId=${id}`);
    }, 1);
    // iframe打开
    // this.setState({ switchActiveKey: 'iframe' });
  }
  openTutorPlanView(id) {
    // 浏览器新标签页打开
    let newPage = window.open('about:blank');
    setTimeout(() => {
      let newDoc = newPage.document;
      newDoc.location.replace(`${jiaoXuePageUrl}/#/${jiaoXueViewPageRouter}?tutorPlanId=${id}&type=renewxg`);
    }, 1);
    // iframe打开
    // this.setState({ switchActiveKey: 'iframe' });
  }
  // 从iframe切回原页面
  switchPage() {
    this.tutorPlanUrl = '';
    this.setState({ switchActiveKey: 'page' }, () => {
      this.loadTutorPlanList();
    });
  }
  // 点击删除续费课程方案
  deleteTutorPlan(id) {
    showConfirm(
      '确定删除这条档案么？',
      async () => {
        this.setState({ loading: true });
        const rsp = await this.delBossTutorPlan(id).catch((err) => {
          this.setState({ loading: false });
        });
        if (rsp && rsp.success) {
          showMessage('删除成功！', 'success');
          this.loadTutorPlanList();
        } else {
          showMessage(rsp.desc, 'error');
          this.setState({ loading: false });
        }
      },
      () => {},
    );
  }
  // 请求删除续费课程方案
  delBossTutorPlan(id) {
    let params = { bossTutorPlanId: id };
    let headers = { 'access-token': JSON.parse(localStorage.getItem('teaToken')).token };
    return new Promise((resolve, reject) => {
      delBossTutorPlan({ params, headers })
        .then((rsp) => {
          resolve(rsp);
        })
        .catch((err) => {
          showMessage('删除续费课程方案错误，请联系管理员！', 'error');
          console.error('Error - queryBossTutorPlanList :', err);
          reject(err);
        });
    });
  }
  static checkNotNull(val) {
    return val && val !== '';
  }

  render() {
    let {
      loading,
      visible,
      activeKey,
      list,
      markArray,
      winW,
      viewModal,
      type,
      id,
      keFuYiYuanDict,
      xiaoGuoPingJiaDict,
      subjectDict,
      record,
      searchExpand,
      startDate,
      endDate,
      productType,
      studentFileType,
      managerId,
      teacher,
      subjectId,
      miniClassName,
      title,
      miniClassDict,
      managerDict = [],
      teacherDict = [],
      fileName,
      tutorPlanList,
      tutorPlanMarkArray,
      switchActiveKey,
    } = this.state;
    let { studentId, studentName, studyManager, account, userId } = this.props;
    let colCount = winW < 1600 ? 2 : winW < 2200 ? 3 : 4;
    let checkPage = switchActiveKey === 'page';
    let showTutorPlanList =
      productType !== 'SMALL_CLASS' &&
      (!studentFileType || studentFileType === 'TUTOR_PLAN') &&
      !StudentFile.checkNotNull(subjectId) &&
      !StudentFile.checkNotNull(miniClassName) &&
      !StudentFile.checkNotNull(fileName);

    return (
      <Switch activeKey={switchActiveKey}>
        <SwitchItem comKey="page">
          <Page
            title={checkPage ? undefined : false}
            loading={loading}
            footer={false}
            route={this.props.route}
            headerToolBar={
              checkPage ? (
                <div className={styles.headerToolBar}>
                  <FormItem name="学生" labelWidth={43}>
                    {this.props.studentName}
                  </FormItem>
                  <FormItem name="学管师/班主任" labelWidth={100}>
                    {managerDict[0] ? managerDict[0].text : null}
                  </FormItem>
                </div>
              ) : (
                false
              )
            }
            subHeader={
              <div className={styles.top}>
                <SearchBar
                  expand={searchExpand}
                  onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
                  onSearch={this.search}
                  onReset={this.reset}
                >
                  <DatePicker
                    placeholder="开始日期"
                    value={startDate}
                    onChange={(val) => this.setState({ startDate: val })}
                    disabledDate={(val) => this.getDisabledStartDate(val)}
                  />
                  <DatePicker
                    placeholder="结束日期"
                    value={endDate}
                    onChange={(val) => this.setState({ endDate: val })}
                    disabledDate={(val) => this.getDisabledEndDate(val)}
                  />
                  <Select
                    placeholder="产品类型"
                    dict={productDict}
                    value={productType}
                    onChange={(val) => this.setState({ productType: val })}
                  />

                  <Select
                    placeholder="档案类型"
                    dict={filesDict}
                    value={studentFileType}
                    onChange={(val) => this.setState({ studentFileType: val })}
                  />
                  <Select
                    placeholder="学管师/班主任"
                    dict={managerDict}
                    value={managerId}
                    className={styles.selectWrap}
                    onChange={(val) => this.setState({ managerId: val })}
                  />
                  <Select
                    placeholder="老师"
                    dict={teacherDict}
                    value={teacher}
                    onChange={(val) => this.setState({ teacher: val })}
                  />
                  <Select
                    placeholder="科目"
                    showSearch={true}
                    dict={subjectDict}
                    value={subjectId}
                    onChange={(val) => this.setState({ subjectId: val })}
                  />
                  <Input
                    placeholder="班课名称"
                    value={miniClassName}
                    onChange={(e) => this.setState({ miniClassName: e.target.value })}
                  />
                  <Input
                    placeholder="档案名称"
                    value={fileName}
                    onChange={(e) => this.setState({ fileName: e.target.value })}
                  />
                </SearchBar>
                <Popover
                  trigger="click"
                  visible={visible}
                  content={<AddNewBtnPop config={this.buildAddNewConfig()} />}
                  placement="bottomRight"
                  overlayClassName="studentFileOverlay"
                  onVisibleChange={(visible) => this.setState({ visible })}
                >
                  <Button className={styles.newAdd}>新增档案</Button>
                </Popover>
              </div>
            }
          >
            {showTutorPlanList &&
              tutorPlanList.map((v, k) => {
                return (
                  <Block
                    className="studentFileBlock"
                    showAll={tutorPlanMarkArray[k]}
                    data={v}
                    key={k}
                    auth={this.userAuthStorage}
                    colCount={colCount}
                    showModalClick={this.tutorPlanClick}
                    dropDownClick={this.tutorPlanMoreAction}
                    showAllClick={(e) => this.showTutorPlanAll(e)}
                    account={account}
                  />
                );
              })}
            {list.map((v, k) => {
              return (
                <Block
                  className="studentFileBlock"
                  showAll={markArray[k]}
                  data={v}
                  key={k}
                  auth={this.auth}
                  colCount={colCount}
                  showModalClick={(id, title) => this.showModalClick(id, title, 'view')}
                  dropDownClick={(type, id, title) => this.dropDownClick(type, id, title)}
                  showAllClick={(e) => this.showAll(e)}
                  account={account}
                />
              );
            })}
            {list.length === 0 && tutorPlanList.length === 0 && <NoData />}
            {viewModal && (
              <ViewModal
                id={id}
                studentId={studentId}
                studentName={studentName}
                studyManager={studyManager}
                onClose={(e) => this.setState({ viewModal: false })}
                onSuccess={this.onSuccess}
                title={title}
                auth={this.auth}
                type={type}
                record={record}
                keFuYiYuanDict={keFuYiYuanDict}
                subjectDict={subjectDict}
                productDict={this.modalProductDict}
                miniClassDict={miniClassDict}
                xiaoGuoPingJiaDict={xiaoGuoPingJiaDict}
                account={account}
              />
            )}
          </Page>
        </SwitchItem>
        <SwitchItem comKey="iframe" type="if">
          <Iframe
            src={this.tutorPlanUrl}
            receiveMsgConfig={[
              { event: 'backToList', callback: this.switchPage },
              { event: 'historyBack', callback: StudentFile.goBackForIframe },
            ]}
          />
        </SwitchItem>
      </Switch>
    );
  }
}

let mapStateToProps = (state) => {
  const data = state.main.pageParams['/StudentFile'];
  return {
    account: state.account.user.name,
    userId: state.account.user.userId,
    studentId: data.studentId,
    studentName: data.studentName,
    studyManager: data.studyManager,
    from: data.from,
    ids: data.fileIds,
    oneOnOneStatus: data.oneOnOneStatus,
    isOneOnOne: data.oneOnOneStatus && data.oneOnOneStatus !== '',
    studyManagerId: data.studyManagerId,
    gradeId: data.gradeId,
    gradeName: data.gradeName,
  };
};

export default connect(mapStateToProps)(StudentFile);
