import React from 'react';
import { connect } from 'dva';
import Loaders from './loader';
import style from './index.less';
import Modal from 'COMPONENT/Modal';
import Select from 'COMPONENT/Select';
import { message } from 'antd';
import { uuid } from 'COMPONENT/utils/basic/str';
import { clone } from 'lodash';

class StudentIntentionManage extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false,
      saveData: [],
      subjectDict: [],
      courseTimeDict: [],
    };
  }

  async componentDidMount() {
    await this.getInitData();
    await this.getInitDict();
  }

  // 提交
  submit() {
    let { saveData } = this.state;
    let tempArr = saveData.map((value) => {
      return {
        intentionItemId: value.intentionItemId,
        intentionItemType: value.intentionItemType,
        intentionProductType: value.intentionProductType,
        studentId: value.studentId,
      };
    });
    Loaders.saveOrUpdateStudentIntentionInfoApi(tempArr)
      .then((res) => {
        if (res.resultCode === 0) {
          message.success('保存成功');
          this.props.onClose();
          GlobalEvent.emit('REFRESH_OTM_STUDENTLIST');
        } else {
          message.error(res.resultMessage);
        }
      })
      .catch((err) => {
        console.log(err);
      });
  }

  // 初始化弹框的数据
  getInitData() {
    let { studentId } = this.props;
    Loaders.listStudentIntentionInfoApi({ studentId: studentId })
      .then((res) => {
        if (res.resultCode === 0) {
          if (res.data.length) {
            let tempSubject = 0,
              tempCourseTime = 0;
            this.setState({
              saveData: res.data.map((value) => {
                if (value.intentionItemType === 'SUBJECT') {
                  tempSubject++;
                } else if (value.intentionItemType === 'CLASS_PERIOD') {
                  tempCourseTime++;
                }
                return {
                  ...value,
                  key: uuid(),
                };
              }),
            });
            if (!tempSubject) {
              this.addData('SUBJECT');
            } else if (!tempCourseTime) {
              this.addData('CLASS_PERIOD');
            }
          } else {
            this.addData('SUBJECT');
            this.addData('CLASS_PERIOD');
          }
        } else {
          message.error(res.resultMessage);
        }
      })
      .catch((err) => {
        console.log(err);
      });
  }

  getArrayList(obj) {
    if (obj && obj.constructor === Object) {
      let tempArr = [];
      Object.entries(obj).forEach((value) => {
        if (value[0]) {
          tempArr.push({
            val: value[0],
            text: value[1],
          });
        }
      });
      return tempArr;
    } else {
      return [];
    }
  }

  // 初始化下拉框的字典数据
  getInitDict() {
    Loaders.getSelectOptionApi({ selectOptionCategory: 'SUBJECT' })
      .then((res) => {
        if (res.resultCode === 0) {
          this.setState({
            subjectDict: this.getArrayList(res.value),
          });
        } else {
          message.error(res.resultMessage);
        }
      })
      .catch((err) => {
        console.log(err);
      });
    Loaders.getSelectOptionApi({ selectOptionCategory: 'CLASS_PERIOD' })
      .then((res) => {
        if (res.resultCode === 0) {
          this.setState({
            courseTimeDict: this.getArrayList(res.value),
          });
        } else {
          message.error(res.resultMessage);
        }
      })
      .catch((err) => {
        console.log(err);
      });
  }

  // 保存的数据修改
  dataChange(val, dataKey) {
    let tempArr = clone(this.state.saveData);
    tempArr.forEach((value, key, array) => {
      if (value.key === dataKey) {
        array[key].intentionItemId = val;
      }
    });
    this.setState({
      saveData: tempArr,
    });
  }

  // 意向科目数据
  intentionSubject(value, key) {
    let tempKey = key;
    let tempValue = value || '';
    const { subjectDict, saveData } = this.state;
    let tempSum = 0;
    saveData.forEach((value) => {
      if (value.intentionItemType === 'SUBJECT') {
        tempSum++;
      }
    });
    return (
      <div key={key}>
        <Select
          dict={subjectDict}
          value={tempValue}
          defaultValue={tempValue}
          placeholder="意向科目"
          className={style.selectItem}
          onChange={(val) => {
            this.dataChange(val, key);
          }}
        />
        {saveData.length && (
          <span className={style.addData} onClick={() => this.addData('SUBJECT')}>
            继续新增
          </span>
        )}
        {tempSum > 1 && (
          <span className={style.deleteData} onClick={() => this.deleteData(tempKey)}>
            删除
          </span>
        )}
      </div>
    );
  }

  // 意向上课时段数据
  intentionCourseTime(value, key) {
    let tempKey = key;
    let tempValue = value || '';
    const { courseTimeDict, saveData } = this.state;
    let tempSum = 0;
    saveData.forEach((value) => {
      if (value.intentionItemType === 'CLASS_PERIOD') {
        tempSum++;
      }
    });
    return (
      <div key={key}>
        <Select
          dict={courseTimeDict}
          value={tempValue}
          defaultValue={tempValue}
          placeholder="意向上课时段"
          className={style.selectItem}
          onChange={(val) => {
            this.dataChange(val, key);
          }}
        />
        {saveData.length && (
          <span className={style.addData} onClick={() => this.addData('CLASS_PERIOD')}>
            继续新增
          </span>
        )}
        {tempSum > 1 && (
          <span className={style.deleteData} onClick={() => this.deleteData(tempKey)}>
            删除
          </span>
        )}
      </div>
    );
  }

  // 删除数据
  deleteData(key) {
    let tempList = [];
    this.state.saveData.forEach((value) => {
      if (value.key !== key) {
        tempList.push(value);
      }
    });
    this.setState({
      saveData: tempList,
    });
  }

  // 新增数据
  addData(type) {
    let key = uuid();
    let { studentId } = this.props;
    let tempList = clone(this.state.saveData);
    tempList.push({
      studentId: studentId,
      intentionProductType: 'ONE_ON_MANY',
      intentionItemType: type,
      intentionItemId: '',
      key: key,
    });
    this.setState({
      saveData: tempList,
    });
  }

  render() {
    let { loading, saveData } = this.state;
    let { onClose } = this.props;
    return (
      <Modal
        loading={loading}
        onCancel={onClose}
        title="学生意向管理"
        width={400}
        okText="保存"
        onOk={() => this.submit()}
      >
        <div className={style.studentIntentionManage}>
          <div>
            <div className={style.intentionSubject}>意向科目</div>
            {saveData.map((value) => {
              if (value.intentionItemType === 'SUBJECT') {
                return this.intentionSubject(value.intentionItemId, value.key);
              }
            })}
          </div>
          <div>
            <div className={style.intentionCourseTime}>意向上课时段</div>
            {saveData.map((value) => {
              if (value.intentionItemType === 'CLASS_PERIOD') {
                return this.intentionCourseTime(value.intentionItemId, value.key);
              }
            })}
          </div>
        </div>
      </Modal>
    );
  }
}

const mapStateToProps = (state) => {
  return {};
};

export default connect(mapStateToProps)(StudentIntentionManage);
