import React from 'react';
import { connect } from 'dva';
import { Row, Col, Switch } from 'antd';
import Page from 'COMPONENT/Page';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import Input from 'COMPONENT/Input';
import InputNumber from 'COMPONENT/InputNumber';
import Block from 'COMPONENT/Block';
import FormItem from 'COMPONENT/FormItem';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { getDictTextByVal, loadCommonDict, loadBranchDict, loadAllBranchDict, formatGroupDict } from 'UTIL/dict';
import styles from './styles.less';
import Button from 'COMPONENT/Button';
import { FIELD_FILLER, checkFieldEmpty, getFieldWithFiller, getCommonErrorTips } from 'COMPONENT/utils/form';
import loaders from '../loaders';
import ScoreLevels from '../ScoreLevels';
import Files from '../Files';
import dicts from '../dicts';
import { checkNumber } from 'COMPONENT/Validator/checkers';
import TextLink from 'COMPONENT/TextLink';
import PopConfirm from 'COMPONENT/PopConfirm';
import showAlert from '../../../../components/utils/globalTips/showAlert';

const defaultScoreLevels = [
  { id: '1', type: null, start: null, end: null },
  { id: '2', type: dicts.SCORE_LEVEL_INVALID, start: 0, end: null },
];

const formCfg = {
  style: { padding: '0 25px', marginBottom: 20 },
  labelWidth: 80,
};

class Detail extends React.Component {
  constructor(props) {
    super(props);
    const {
      params: { type },
    } = props;
    this.state = {
      loading: false, // {boolean} 内容加载状态
      editing: type === 'add' || type === 'copy', // {boolean} 页面编辑状态
      checking: false, // {boolean} 校验开启状态

      id: null, // {string} 试卷ID
      status: dicts.STATUS_VALID, // {string} 状态
      name: null, // {string} 试卷名称
      testType: null, // {string} 考试类型

      branch: null, // {string} 分公司
      productType: null, // {string} 产品类型
      year: null, // {string} 年份
      quarter: [], // {string} 季度
      quarterName: null, // string 季度名称
      grade: null, // {string} 年级
      subject: null, // {string} 科目
      classTypeId: null, // {string} 班级类型
      totalScore: null, // {string} 总分

      scoreLevelActive: false, // {boolean} 得分等级激活状态
      scoreLevelActiveInit: false, // {boolean} 得分等级激活状态备份，用于关闭得分等级时判断是否需要提示
      scoreLevels: null, // {object[]} 得分等级列表，[{ id, type, start, end }]
      scoreLevelsInit: null, // {object[]} 得分等级列表备份，用于得分等级激活状态设为未激活时恢复数据

      testPaperFiles: null, // {object[]} 试卷文件[{ listId, name, src }]
      answerFiles: null, // {object[]} 答案文件[{ listId, name, src }]

      dataInit: null, // {boolean} 初始数据，用于详情模式取消编辑时恢复初始状态

      newClassTypeDict: this.props.params.classTypeDict, // 班级类型字典，用于班级类型下拉框
    };
    this.errors = {};
  }

  componentDidMount() {
    const {
      params: { type, data, gradeDict, editing },
    } = this.props;
    if (!type || type === 'copy') {
      const detailData = { ...data, scoreLevelActiveInit: data.scoreLevelActive, scoreLevelsInit: data.scoreLevels };
      const newState =
        type === 'copy'
          ? { id: null, status: dicts.STATUS_VALID, testPaperFiles: null, answerFiles: null }
          : { dataInit: detailData };
      this.setState({ loading: false, ...detailData, ...newState });
    }
    if (editing === true) {
      this.setState({ editing });
    }
  }

  onOk = () => {
    const {
      params: { onSaveSuccess, scoreLevelDict },
    } = this.props;
    const {
      checking,
      id,
      status,
      name,
      testType,
      branch,
      productType,
      classTypeId,
      year,
      quarter,
      grade,
      subject,
      totalScore,
      scoreLevelActive,
      scoreLevels,
      testPaperFiles,
      answerFiles,
    } = this.state;
    if (!checking) {
      this.setState({ checking: true });
    }
    const errorTips = getCommonErrorTips(this.errors);
    if (errorTips) {
      this.pageRef.showOkTips(errorTips);
      return;
    }
    if (testPaperFiles.some((f) => f.error)) {
      this.pageRef.showOkTips('试卷附件中包含不合法的文件，请删除后再保存');
      return;
    }
    if (answerFiles.some((f) => f.error)) {
      this.pageRef.showOkTips('答案附件中包含不合法的文件，请删除后再保存');
      return;
    }
    const scoreLevelsHasTypeName = scoreLevels
      ? scoreLevels.map((s) => {
          return { ...s, typeName: getDictTextByVal(scoreLevelDict, s.type) };
        })
      : null;
    this.setState({ loading: true });
    loaders
      .saveDetail({
        id,
        status,
        name,
        testType,
        branch,
        productType,
        classTypeId,
        year,
        quarter,
        grade,
        subject,
        totalScore,
        scoreLevelActive,
        scoreLevels: scoreLevelsHasTypeName,
        testPaperFiles,
        answerFiles,
      })
      .then(
        () => {
          showMessage('保存成功', 'success');
          onSaveSuccess();
        },
        (e) => {
          this.setState({ loading: false });
          showMessage(e.text || '保存失败');
        },
      );
  };

  errorChange = (key, error) => {
    this.errors[key] = error;
  };

  onScoreLevelActiveChange = (check) => {
    /*    const { params: { type, id } } = this.props
    if (type === 'add' || type === 'copy') {*/
    this.setScoreLevelActive(check);
    /*    } else {
      this.setState({ loading: true })
      loaders.setScoreActive({ id, active: check }).then(() => {
        showMessage('设置成功', 'success')
        this.setScoreLevelActive(check)
        this.setState({ loading: false })
      }, e => {
        this.setState({ loading: false })
        showMessage(e.text || '设置失败')
      })
    }*/
  };

  setScoreLevelActive = (check) => {
    const { scoreLevelsInit, totalScore } = this.state;
    this.setState({ scoreLevelActive: check });
    if (check) {
      const scoreLevels = (scoreLevelsInit && scoreLevelsInit.length > 0 ? scoreLevelsInit : defaultScoreLevels).map(
        (s, i) => {
          if (i === 0) {
            return { ...s, end: totalScore };
          }
          return s;
        },
      );
      this.setState({ scoreLevels });
      if (scoreLevelsInit && scoreLevelsInit.length > 0 && scoreLevelsInit[0].end !== totalScore) {
        showAlert('得分等级的总分已被修改，已更新到最新总分');
      }
    } else {
      this.setState({ scoreLevels: scoreLevelsInit });
      this.clearScoreLevelError();
    }
  };

  getTotalScoreToolBar = () => {
    const {
      params: { type },
    } = this.props;
    const { scoreLevelActive, editing, scoreLevelActiveInit } = this.state;
    const cfg = {
      checked: scoreLevelActive,
      disabled: !editing,
    };
    if (!type && editing && scoreLevelActive && scoreLevelActiveInit) {
      return (
        <PopConfirm
          content="关闭后，后续使用该试卷的学生成绩无法自动匹配班级类型"
          onOk={() => this.onScoreLevelActiveChange(false)}
          getPopupContainer={() => this.scoreLevelBlockRef}
        >
          <span>
            <Switch {...cfg} />
          </span>
        </PopConfirm>
      );
    } else {
      return (
        <span>
          <Switch {...cfg} onChange={(check) => this.onScoreLevelActiveChange(check)} />
        </span>
      );
    }
  };

  clearScoreLevelError = (id) => {
    let newErrors = {};
    Object.keys(this.errors).forEach((key) => {
      const a = key.split('_');
      if (!(a[0] === 'scoreLevels' && (id ? a[1] === id : true))) {
        newErrors[key] = this.errors[key];
      }
    });
    this.errors = newErrors;
  };

  totalScoreChange = (val) => {
    this.setState({ totalScore: val });
    const { scoreLevelActive, scoreLevels } = this.state;
    if (scoreLevelActive) {
      this.setState({
        scoreLevels: scoreLevels.map((s, idx) => {
          if (idx === 0) {
            return { ...s, end: val };
          }
          return s;
        }),
      });
    }
  };

  /**
   *  产品类型改变获取对应的班级类型
   */
  handleProductTypeChange = async (productType) => {
    this.setState({ classTypeId: null, productType, name: this.getName({ productType }) });
    if (productType) {
      const newClassTypeDict = await loadCommonDict('CLASS_TYPE', productType);
      this.setState({ newClassTypeDict });
    }
  };

  getName = (changed) => {
    const {
      params: {
        branchDict,
        testTypeDict,
        productTypeDict,
        yearDict,
        quarterDict,
        gradeDict,
        subjectDict,
        classTypeDict,
        scoreLevelDict,
        statusDict,
      },
    } = this.props;
    const { year, quarter, branch, productType, grade, subject, testType } = { ...this.state, ...changed };
    const getFieldText = (dict, val) => {
      return getDictTextByVal(dict, val) || '';
    };
    return (
      getFieldText(yearDict, year) +
      getFieldText(quarterDict, quarter) +
      getFieldText(branchDict, branch) +
      getFieldText(productTypeDict, productType) +
      getFieldText(gradeDict, grade) +
      getFieldText(subjectDict, subject) +
      getFieldText(testTypeDict, testType)
    );
  };

  // getEditBtn = () => {
  //   const { btnAuths, params: { type } } = this.props
  //   const { editing, dataInit } = this.state
  //   if (!type && btnAuths.indexOf('STUDENT_TEST_PAPER_EDIT_BTN') >= 0) {
  //     if (editing) {
  //       return <TextLink onClick={() => this.setState({ ...dataInit, editing: false })}>取消</TextLink>
  //     }
  //     return <TextLink onClick={() => this.setState({ editing: true })}>编辑</TextLink>
  //   }
  //   return null
  // }

  render() {
    const {
      params: {
        type,
        branchDict,
        testTypeDict,
        productTypeDict,
        yearDict,
        quarterDict,
        gradeDict,
        subjectDict,
        classTypeDict,
        scoreLevelDict,
        statusDict,
      },
    } = this.props;
    const {
      loading,
      editing,
      checking,
      id,
      status,
      name,
      testType,
      branch,
      productType,
      year,
      quarter,
      quarterName,
      grade,
      subject,
      totalScore,
      scoreLevelActive,
      scoreLevels,
      testPaperFiles,
      answerFiles,
      dataInit,
      classTypeId,
      newClassTypeDict,
    } = this.state;
    let pageName;
    if (type === 'add') {
      pageName = '新增试卷';
    } else if (type === 'copy') {
      pageName = '复制试卷';
    } else {
      pageName = '试卷详情';
    }
    return (
      <Page
        className={styles.page}
        name={pageName}
        footer={editing}
        loading={loading}
        route={this.props.route}
        onOk={this.onOk}
        ref={(c) => (this.pageRef = c)}
      >
        <Block title="基础信息" /*headerToolBar={this.getEditBtn()}*/>
          <Row>
            <Col span={12}>
              <FormItem {...formCfg} name={'试卷ID'}>
                {getFieldWithFiller(id)}
              </FormItem>
            </Col>
          </Row>
          <Row>
            <Col span={12}>
              <FormItem {...formCfg} name={'状态'}>
                {getDictTextByVal(statusDict, status, true)}
              </FormItem>
            </Col>
          </Row>
          <Row>
            <Col span={12}>
              <FormItem {...formCfg} name={'试卷名称'} required={editing}>
                {editing ? (
                  <div>
                    <div>
                      <Input
                        placeholder={'自动生成，可编辑'}
                        value={name}
                        onChange={(e) => this.setState({ name: e.target.value })}
                        validator={{
                          onErrorChange: (e) => this.errorChange('name', e),
                          silent: !checking,
                        }}
                      />
                    </div>
                    <div className={styles.tips}>年份+分公司+产品类型+年级+科目+考试类型</div>
                  </div>
                ) : (
                  <div>{getFieldWithFiller(name)}</div>
                )}
              </FormItem>
            </Col>
          </Row>
          <Row>
            <Col span={12}>
              <FormItem {...formCfg} name={'考试类型'} required={editing}>
                {editing ? (
                  <Select
                    dict={testTypeDict}
                    value={testType}
                    onChange={(val) => this.setState({ testType: val, name: this.getName({ testType: val }) })}
                    validator={{
                      onErrorChange: (e) => this.errorChange('testType', e),
                      silent: !checking,
                    }}
                  />
                ) : (
                  <div>{getDictTextByVal(testTypeDict, testType, true)}</div>
                )}
              </FormItem>
            </Col>
          </Row>
        </Block>
        <Block title="适用范围">
          <Row>
            <Col span={8}>
              <FormItem {...formCfg} name={'分公司'} required={editing}>
                {editing ? (
                  <Select
                    dict={branchDict}
                    value={branch}
                    onChange={(val) => this.setState({ branch: val, name: this.getName({ branch: val }) })}
                    validator={{
                      onErrorChange: (e) => this.errorChange('branch', e),
                      silent: !checking,
                    }}
                  />
                ) : (
                  <div>{getDictTextByVal(branchDict, branch, true)}</div>
                )}
              </FormItem>
            </Col>
            <Col span={8}>
              <FormItem {...formCfg} name={'产品类型'} required={editing}>
                {editing ? (
                  <Select
                    dict={productTypeDict}
                    value={productType}
                    onChange={(val) => this.handleProductTypeChange(val)}
                    validator={{
                      onErrorChange: (e) => this.errorChange('productType', e),
                      silent: !checking,
                    }}
                  />
                ) : (
                  <div>{getDictTextByVal(productTypeDict, productType, true)}</div>
                )}
              </FormItem>
            </Col>
            <Col span={8}>
              <FormItem {...formCfg} name={'年份'} required={editing}>
                {editing ? (
                  <Select
                    dict={yearDict}
                    value={year}
                    onChange={(val) => this.setState({ year: val, name: this.getName({ year: val }) })}
                    validator={{
                      onErrorChange: (e) => this.errorChange('year', e),
                      silent: !checking,
                    }}
                  />
                ) : (
                  <div>{getDictTextByVal(yearDict, year, true)}</div>
                )}
              </FormItem>
            </Col>
            <Col span={8} style={{ clear: 'both' }}>
              <FormItem {...formCfg} name={'季度'} required={editing}>
                {editing ? (
                  <MultiSelect
                    dict={quarterDict}
                    value={quarter}
                    onChange={(val) => this.setState({ quarter: val })}
                    validator={{
                      onErrorChange: (e) => this.errorChange('quarter', e),
                      silent: !checking,
                    }}
                  />
                ) : (
                  <div>{quarterName}</div>
                )}
              </FormItem>
            </Col>
            <Col span={8}>
              <FormItem {...formCfg} name={'年级'} required={editing}>
                {editing ? (
                  <Select
                    dict={gradeDict}
                    value={grade}
                    onChange={(val) => this.setState({ grade: val, name: this.getName({ grade: val }) })}
                    validator={{
                      onErrorChange: (e) => this.errorChange('grade', e),
                      silent: !checking,
                    }}
                  />
                ) : (
                  <div>{getDictTextByVal(gradeDict, grade, true)}</div>
                )}
              </FormItem>
            </Col>
            <Col span={8}>
              <FormItem {...formCfg} name={'科目'} required={editing}>
                {editing ? (
                  <Select
                    dict={subjectDict}
                    value={subject}
                    onChange={(val) => this.setState({ subject: val, name: this.getName({ subject: val }) })}
                    validator={{
                      onErrorChange: (e) => this.errorChange('subject', e),
                      silent: !checking,
                    }}
                  />
                ) : (
                  <div>{getDictTextByVal(subjectDict, subject, true)}</div>
                )}
              </FormItem>
            </Col>
            <Col span={8} style={{ clear: 'both' }}>
              <FormItem {...formCfg} name={'班级类型'}>
                {editing ? (
                  <Select
                    dict={newClassTypeDict}
                    value={classTypeId}
                    onChange={(val) => this.setState({ classTypeId: val, name: this.getName({ classTypeId: val }) })}
                  />
                ) : (
                  <div>{getDictTextByVal(classTypeDict, classTypeId, true)}</div>
                )}
              </FormItem>
            </Col>
            <Col span={8}>
              <FormItem {...formCfg} name={'总分'} required={editing}>
                {editing ? (
                  <InputNumber
                    min={0.1}
                    max={999.9}
                    precision={1}
                    value={totalScore}
                    onChange={this.totalScoreChange}
                    validator={{
                      onErrorChange: (e) => this.errorChange('totalScore', e),
                      silent: !checking,
                    }}
                  />
                ) : (
                  <div>{getFieldWithFiller(totalScore)}</div>
                )}
              </FormItem>
            </Col>
          </Row>
        </Block>
        <Block
          title={'得分等级'}
          headerToolBar={this.getTotalScoreToolBar()}
          ref={(c) => (this.scoreLevelBlockRef = c)}
          className={styles.scoreLevelBlock}
        >
          {scoreLevelActive ? (
            <ScoreLevels
              totalScore={totalScore}
              classTypeDict={classTypeDict}
              productType={productType}
              scoreLevelDict={scoreLevelDict}
              editing={editing}
              value={scoreLevels}
              onChange={(val) => this.setState({ scoreLevels: val })}
              onErrorChange={(key, e) => this.errorChange(`scoreLevels_${key}`, e)}
              onClearError={this.clearScoreLevelError}
              checking={checking}
              classTypeId={classTypeId}
            />
          ) : (
            <div className={styles.tips}>
              得分等级如未激活并填写，则使用该试卷的学生成绩无法自动匹配试卷得分对应的班级类型
            </div>
          )}
        </Block>
        <Block title={'附加信息'}>
          <FormItem {...formCfg} name={'试卷'} required={editing}>
            {!editing && (!testPaperFiles || testPaperFiles.length === 0) ? (
              FIELD_FILLER
            ) : (
              <Files
                editing={editing}
                value={testPaperFiles}
                onChange={(val) => this.setState({ testPaperFiles: val })}
                validator={{
                  rules: [{ required: '请上传附件' }],
                  onErrorChange: (e) => this.errorChange('testPaperFiles', e),
                  silent: !checking,
                }}
                defName={name}
              />
            )}
          </FormItem>
          <FormItem {...formCfg} name={'答案'} required={editing}>
            {!editing && (!answerFiles || answerFiles.length === 0) ? (
              FIELD_FILLER
            ) : (
              <Files
                editing={editing}
                value={answerFiles}
                onChange={(val) => this.setState({ answerFiles: val })}
                validator={{
                  rules: [{ required: '请上传附件' }],
                  onErrorChange: (e) => this.errorChange('answerFiles', e),
                  silent: !checking,
                }}
                defName={name ? `${name}答案` : null}
              />
            )}
          </FormItem>
          {editing && (
            <div className={styles.tips} style={{ marginLeft: 105 }}>
              支持格式：word、excel、ppt、pdf、txt、zip、rar、png、jpg、bmp；且单个文件不超过10MB
            </div>
          )}
        </Block>
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    params: state.main.pageParams['/StudentTestPaperManage/Detail'],
    btnAuths: state.account.btnAuths,
  };
};

export default connect(mapStateToProps)(Detail);
