import React from 'react';
import styles from './styles.less';
import { getIconSrc } from '../utils';
import { FILE_TYPE_MAP_EXTENSION } from 'COMPONENT/FileIcon/config';
import QrcodeFileUpload from 'COMPONENT/QrcodeFileUpload';
import Upload from 'COMPONENT/Upload';
import FileList from 'COMPONENT/FileList';
import Button from 'COMPONENT/Button';
import Popover from 'COMPONENT/Popover';
import previewFile from 'UTIL/previewFile';
import Validator from 'COMPONENT/Validator';

const maxUploadFileCount = 9999;
const validateFileTypes = ['word', 'excel', 'ppt', 'pdf', 'txt', 'zip', 'rar', 'png', 'jpg', 'bmp'];

class Files extends React.Component {
  constructor(props) {
    super(props);
    this.fileIdx = 1;
    this.state = {
      fileList: props.value || [], // {object[]} 附件列表
    };
  }

  componentWillReceiveProps = (nextProps) => {
    this.setState({ fileList: nextProps.value || [] });
  };

  get2dUploadImgInfo = (info) => {
    console.log(info);
    const { fileList } = this.state;
    if (fileList.length >= maxUploadFileCount) {
      return;
    }
    const newFileList = [...fileList];
    let exsistNames = [];
    info.forEach((file) => {
      const name = this.genFileName(file.realFileName, exsistNames);
      exsistNames.push(name);
      const src = file.filePath;
      if (!newFileList.some((nf) => nf.src === src) && newFileList.length < maxUploadFileCount) {
        newFileList.push({ id: file.fileid, name, src, iconSrc: getIconSrc(name, src), listId: this.genFileId() });
      }
    });
    this.props.onChange(newFileList);
  };

  onUploadChange = (info) => {
    console.log(info);
    const newFileList = this.state.fileList.map((file) => {
      const newFile = info.find((inf) => inf.fileKey === file.fileKey);
      console.log(newFile);
      if (newFile) {
        const name = file.name;
        const src = newFile.src;
        return {
          ...newFile,
          id: newFile['x:fileid'],
          src,
          name,
          listId: file.listId,
          error: newFile.errorCode ? '上传失败' : null,
          iconSrc: newFile.errorCode ? null : getIconSrc(name, src),
        };
      }
      return file;
    });
    this.props.onChange(newFileList);
  };

  checkLength = (files) => {
    console.log(files);
    let filesMayUpload = [];
    let filesWillUpload = [];
    const restCount = maxUploadFileCount - this.state.fileList.length;
    let exsistNames = [];
    files.slice(0, restCount).map((file) => {
      const listId = this.genFileId();
      const name = this.genFileName(file.name, exsistNames);
      exsistNames.push(name);
      let newFile = { fileKey: file.fileKey, name, listId };
      let fileExtends = [];
      validateFileTypes.forEach((type) => {
        fileExtends = [...fileExtends, ...FILE_TYPE_MAP_EXTENSION[type]];
      });
      const idx = name.lastIndexOf('.');
      const nameExtend = name.slice(idx + 1);
      if (file.size > 1024 * 1024 * 10) {
        newFile = { ...newFile, error: '文件大小超过了10MB' };
      } else if (fileExtends.indexOf(nameExtend.toLowerCase()) < 0) {
        newFile = { ...newFile, error: '文件格式不正确' };
      } else if (name.length - nameExtend.length - 1 > 50) {
        newFile = { ...newFile, error: '文件名超过50字' };
      } else {
        filesWillUpload.push(file);
      }
      filesMayUpload.push(newFile);
    });
    if (restCount < files.length) {
      return new Promise((res, rej) => {
        showConfirm(
          `最多上传${maxUploadFileCount}个文件，超出部分将不上传`,
          () => {
            this.props.onChange([...this.state.fileList, ...filesMayUpload]);
            res(filesWillUpload);
          },
          () => {
            rej();
          },
        );
      });
    } else {
      this.props.onChange([...this.state.fileList, ...filesMayUpload]);
      return filesWillUpload;
    }
  };

  genFileId = () => {
    return 'f_' + this.fileIdx++;
  };

  genFileName = (name, exsistNames) => {
    const { defName } = this.props;
    const { fileList } = this.state;
    const exsistNamesAll = [...fileList.map((f) => f.name), ...exsistNames];
    if (!defName) {
      return name;
    }
    const extension = name.slice(name.lastIndexOf('.') + 1);
    let newName = defName + '.' + extension;
    if (!exsistNamesAll.some((f) => f === newName)) {
      return newName;
    }
    for (let i = 2; i < 9999; i++) {
      newName = `${defName}（${i}）.${extension}`;
      if (!exsistNamesAll.some((f) => f === newName)) {
        return newName;
      }
    }
    return name;
  };

  render() {
    const { editing, validator, onChange } = this.props;
    const { fileList } = this.state;
    let comp = (
      <div className={styles.wrap}>
        {editing && (
          <div style={{ marginBottom: 5 }}>
            <div style={{ display: 'inline-block', marginRight: 20 }}>
              {fileList.length >= maxUploadFileCount ? (
                <Button disabled>点击上传</Button>
              ) : (
                <Upload
                  onChange={this.onUploadChange}
                  multiple={true}
                  beforeUpload={(files) => this.checkLength(files)}
                >
                  <Button>点击上传</Button>
                </Upload>
              )}
            </div>
            <div style={{ display: 'inline-block' }}>
              {fileList.length >= maxUploadFileCount ? (
                <Button disabled>二维码传图</Button>
              ) : (
                <Popover
                  content={<QrcodeFileUpload height={160} width={160} callback={this.get2dUploadImgInfo} multiple />}
                  overlayClassName={styles.codePopWrap}
                  placement="right"
                >
                  <Button>扫码传图</Button>
                </Popover>
              )}
            </div>
          </div>
        )}
        <div>
          {editing ? (
            <FileList
              data={fileList.map((file) => {
                return { ...file, fileId: file.id, id: file.listId };
              })}
              onChange={(list) => {
                const newList = list.map((file) => {
                  return { ...file, id: file.fileId, listId: file.id };
                });
                onChange(newList);
              }}
            />
          ) : (
            <FileList
              data={fileList.map((file) => {
                return { ...file, fileId: file.id, id: file.listId };
              })}
              type="view"
              onPreview={previewFile}
            />
          )}
        </div>
      </div>
    );
    if (validator) {
      comp = (
        <Validator value={this.props.value} {...validator}>
          {comp}
        </Validator>
      );
    }
    return comp;
  }
}

export default Files;
