import React from 'react';
import styles from './styles.less';
import { objectDeepEquals } from '../../../../components/utils/basic/obj';
import { FIELD_FILLER, checkFieldEmpty, getFieldWithFiller, getCommonErrorTips } from 'COMPONENT/utils/form';
import { loadCommonDict } from 'UTIL/dict';
import { getDictTextByVal } from '../../../../utils/dict/index';
import dicts from '../dicts';
import InputNumber from 'COMPONENT/InputNumber';
import Select from 'COMPONENT/Select';
import TextLink from 'COMPONENT/TextLink';
import Validator from 'COMPONENT/Validator';
import tips from 'COMPONENT/Validator/tips';

class ScoreLevels extends React.Component {
  constructor(props) {
    super(props);
    this.idx = 1;
    this.state = {
      value: props.value,
      // classTypeDict: this.getClassTypeDict(props),
      classLevelDict: [],
    };
  }

  componentWillReceiveProps = (nextProps) => {
    this.getClassLevelDict(nextProps);
    this.setState({ value: nextProps.value });
    if (this.props.classTypeId !== nextProps.classTypeId) {
      const newValue = this.state.value.map((v) => {
        if (v.type === dicts.SCORE_LEVEL_INVALID) {
          return v;
        }
        return { ...v, type: null };
      });
      this.props.onChange(newValue);
    }
  };

  componentDidMount = () => {
    this.getClassLevelDict(this.props);
  };

  // getClassTypeDict = props => {
  //   return props.classTypeDict.filter(d => d.parentVal === props.productType)
  // }

  getClassLevelDict = async (props) => {
    let { classTypeId } = props;
    if (!classTypeId) {
      this.setState({ classLevelDict: [] });
      return [];
    } else {
      const classLevelDict = await loadCommonDict('CLASS_LEVEL', classTypeId);
      this.setState({ classLevelDict });
      return classLevelDict;
    }
  };

  onChange = (id, fieldName, val) => {
    const { onChange } = this.props;
    const { value } = this.state;
    let newValue = value.map((v, i) => {
      if (v.id === id) {
        return { ...v, [fieldName]: val };
      }
      return v;
    });
    if (fieldName === 'start') {
      newValue = this.formatValue(newValue);
    }
    onChange(newValue);
  };

  formatValue = (value) => {
    const newValue = value.map((v, i) => {
      if (i === 0) {
        return v;
      }
      return { ...v, end: value[i - 1].start };
    });
    return newValue;
  };

  addRow = (id) => {
    const { onChange } = this.props;
    const { value } = this.state;
    const score = value.find((v) => v.id === id).start;
    const newValue = [
      ...value.slice(0, value.length - 1),
      { id: 'sl' + this.idx++, type: null, start: null, end: score },
      ...value.slice(value.length - 1),
    ];
    onChange(newValue);
  };

  deleteRow = (id) => {
    const { onChange, onClearError } = this.props;
    const { value } = this.state;
    let newValue = value.filter((v) => v.id !== id);
    newValue = this.formatValue(newValue);
    onChange(newValue);
    onClearError(id);
  };

  checkStartScore = (val, isFirstRow) => {
    if (!val.start && val.start !== 0) {
      return tips.required;
    }
    if (isFirstRow && val.end && val.start > val.end) {
      return '需小于或等于总分';
    }
    if (!isFirstRow && val.end && val.start >= val.end) {
      return '需小于左侧分数';
    }
    return null;
  };

  render() {
    const { classTypeId, scoreLevelDict, editing, onErrorChange, totalScore, checking } = this.props;
    const { value, classLevelDict } = this.state;
    let wrapW = '80%';
    let colW = '20%';
    let colWScores = '60%';
    if (!editing) {
      wrapW = '64%';
      colW = '25%';
      colWScores = '75%';
    }
    // 已经被选中的得分等级
    let selectedClassLevels = value.map((v) => {
      return v.type;
    });
    return (
      <div className={styles.wrap} style={{ width: wrapW }}>
        <div className={styles.header}>
          <div className={styles.col} style={{ width: colW }}>
            班级层次
          </div>
          <div className={styles.col} style={{ width: colWScores }}>
            得分要求
          </div>
          {editing && (
            <div className={styles.col} style={{ width: colW }}>
              操作
            </div>
          )}
        </div>
        <div className={styles.body}>
          {value.map((v, idx) => {
            const isLastRow = idx === value.length - 1;
            return (
              <div key={v.id} className={styles.row}>
                <div className={styles.col} style={{ width: colW }}>
                  {isLastRow ? (
                    <div>{dicts.scoreLevelDictItemInvalid.text}</div>
                  ) : (
                    <div>
                      {editing ? (
                        <Select
                          size={'small'}
                          dict={classLevelDict.filter((classLevel) => {
                            if (classLevel.val === v.type) {
                              return classLevel;
                            } else {
                              return selectedClassLevels.includes(classLevel.val) === false;
                            }
                          })}
                          noOptionTips={!classTypeId ? '请先选择班级类型' : undefined}
                          value={v.type}
                          onChange={(val) => this.onChange(v.id, 'type', val)}
                          validator={{
                            onErrorChange: (e) => onErrorChange(`${v.id}_type`, e),
                            silent: !checking,
                          }}
                        />
                      ) : (
                        <div>{getDictTextByVal(classLevelDict, v.type, true)}</div>
                      )}
                    </div>
                  )}
                </div>
                <div className={styles.col} style={{ width: colW }}>
                  {getFieldWithFiller(v.end)}
                </div>
                <div className={styles.col} style={{ width: colW }}>
                  {`${idx === 0 ? '≥' : '>'} 学生成绩 ≥`}
                </div>
                <div className={styles.col} style={{ width: colW }}>
                  {isLastRow ? (
                    <div>0</div>
                  ) : (
                    <div>
                      {editing ? (
                        <Validator
                          rules={[(val) => this.checkStartScore(val, idx === 0)]}
                          onErrorChange={(e) => onErrorChange(`${v.id}_start`, e)}
                          silent={!checking}
                          value={v}
                        >
                          <InputNumber
                            size={'small'}
                            value={v.start}
                            onChange={(val) => this.onChange(v.id, 'start', val)}
                            min={0.1}
                            precision={1}
                          />
                        </Validator>
                      ) : (
                        <div>{getFieldWithFiller(v.start)}</div>
                      )}
                    </div>
                  )}
                </div>
                {editing && (
                  <div className={styles.col} style={{ width: '20%' }}>
                    {isLastRow ? (
                      <div>-</div>
                    ) : (
                      <div>
                        <TextLink onClick={() => this.addRow(v.id)}>新增</TextLink>
                        {idx > 0 && (
                          <TextLink style={{ marginLeft: 20 }} onClick={() => this.deleteRow(v.id)}>
                            删除
                          </TextLink>
                        )}
                      </div>
                    )}
                  </div>
                )}
              </div>
            );
          })}
        </div>
      </div>
    );
  }
}

export default ScoreLevels;
