import React from 'react';
import ReactDOM from 'react-dom';
import { connect } from 'dva';
import Input from 'COMPONENT/Input';
import Page from 'COMPONENT/Page';
import SearchBar from 'COMPONENT/SearchBar';
import List from 'COMPONENT/List';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import styles from './styles.less';
import loaders from './loaders';
import dicts from './dicts';
import Button from 'COMPONENT/Button';
import { getDictTextByVal, loadBranchDict, loadCommonDict } from '../../../utils/dict/index';
import showConfirm from '../../../components/utils/globalTips/showConfirm';
import ScoreLevelCell from './ScoreLevelCell';
import moment from 'moment';
import { RenderMultiLine } from '../TutorialScoreManager/renderMultiLine';

const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;
let defaultSearchKeys = {
  id: null,
  branch: null,
  testType: null,
  name: null,
  productType: 'SMALL_CLASS',
  year: null,
  quarter: null,
  grade: null,
  subject: null,
  status: '0',
};

class TestPaperNamage extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 内容加载状态
      searchExpand: false, // {boolean} 搜索栏展开状态

      id: null, // {string} 查询条件 试卷ID
      branch: null, // {string} 查询条件 分公司
      testType: null, // {string} 查询条件 考试类型
      name: null, // {string} 查询条件 试卷名称
      productType: 'SMALL_CLASS', // {string} 查询条件 产品类型
      year: null, // {string} 查询条件 年份
      quarter: null, // {string} 查询条件 季度
      grade: null, // {string} 查询条件 年级
      subject: null, // {string} 查询条件 科目
      status: '0', // {string} 查询条件 状态
      classTypeId: null, // {string} 查询条件 班级类型

      branchDict: [], // {[object]} 字典 分公司
      testTypeDict: [], // {[object]} 字典 考试类型
      productTypeDict: [
        { val: 'SMALL_CLASS', text: '小班' },
        { val: 'TWO_TEACHER', text: '双师' },
      ], // {[object]} 字典 产品类型
      yearDict: [], // {[object]} 字典 年份
      quarterDict: [], // {[object]} 字典 季度
      gradeDict: [], // {[object]} 字典 年级
      subjectDict: [], // {[object]} 字典 科目
      classTypeDict: [], // {[object]} 字典 班级类型
      scoreLevelDict: [], // {[object]} 字典 得分等级
      statusDict: dicts.statusDict, // {[object]} 字典 状态

      list: [], // {Array} 列表数据
      listSearchKeys: defaultSearchKeys, // {object} 列表数据的查询关键字
      totalRecords: 0, // {number} 总记录数
      currentPage: 1, // {number} 当前页码
      pageSize: 20, // {number} 每页记录数
    };
  }

  componentDidMount() {
    this.initContent();
  }

  initContent = async () => {
    this.setState({ loading: true });
    const { productType } = this.state;

    const [
      branchDict,
      yearDict,
      quarterDict,
      gradeDict,
      subjectDict,
      classTypeDict,
      testTypeDict,
      newClassTypeDict,
      classLevelDict,
    ] = await Promise.all([
      loadBranchDict(),
      loadCommonDict('PRODUCT_VERSION'),
      loadCommonDict('PRODUCT_QUARTER'),
      loadCommonDict('STUDENT_GRADE'),
      loadCommonDict('SUBJECT'),
      loaders.loadClassTypeDict(),
      loadCommonDict('BRANCH_EXAM_TYPE'),
      loadCommonDict('CLASS_TYPE', 'TWO_TEACHER'),
      loadCommonDict('CLASS_LEVEL'),
    ]);
    await this.setState(
      {
        branchDict,
        yearDict,
        quarterDict,
        gradeDict,
        subjectDict,
        classTypeDict,
        testTypeDict,
        newClassTypeDict,
        branch: branchDict[0].val,
      },
      async () => {
        const listData = await loaders.loadList(Object.assign(this.getListParams(), { branch: branchDict[0].val }));
        const { list, totalRecords } = listData;
        const scoreLevelDict = [...classLevelDict, dicts.scoreLevelDictItemInvalid];
        this.setState({ loading: false, list, totalRecords, scoreLevelDict });
      },
    );
  };

  getListParams = () => {
    const {
      listSearchKeys: { id, branch, testType, name, productType, year, quarter, grade, subject, status, classTypeId },
      currentPage,
      pageSize,
    } = this.state;
    return {
      id,
      branch,
      testType,
      name,
      productType,
      year,
      quarter,
      grade,
      subject,
      status,
      currentPage,
      pageSize,
      classTypeId,
    };
  };

  search = () => {
    let { id, branch, testType, name, productType, year, quarter, grade, subject, status, classTypeId } = this.state;
    classTypeId = classTypeId && classTypeId.length > 0 ? classTypeId.join(',') : classTypeId;
    const listSearchKeys = {
      id,
      branch,
      testType,
      name,
      productType,
      year,
      quarter,
      grade,
      subject,
      status,
      classTypeId,
    };
    this.setState({ loading: true });
    loaders.loadList({ ...this.getListParams(), ...listSearchKeys, currentPage: 1 }).then(({ list, totalRecords }) => {
      this.setState({ loading: false, listSearchKeys, currentPage: 1, list, totalRecords });
    });
  };

  reset = () => {
    const { branchDict } = this.state;
    let branch = branchDict[0].val;
    defaultSearchKeys = Object.assign(defaultSearchKeys, { branch });
    this.setState(defaultSearchKeys);
  };

  goDetail = (type, record, editing) => {
    const { dispatch } = this.props;
    const {
      branchDict,
      testTypeDict,
      productTypeDict,
      yearDict,
      quarterDict,
      gradeDict,
      subjectDict,
      classTypeDict,
      scoreLevelDict,
      statusDict,
      newClassTypeDict,
    } = this.state;
    let data;
    if (record) {
      const {
        id,
        status,
        name,
        testType,
        branch,
        productType,
        year,
        quarter,
        subject,
        classTypeId,
        classLevel,
        totalScore,
        scoreLevelActive,
        scoreLevels,
        testPaperFiles,
        answerFiles,
        grade,
        quarterName,
      } = record;
      data = {
        id,
        status,
        name,
        testType,
        branch,
        productType,
        year,
        quarter,
        subject,
        classTypeId,
        classLevel,
        totalScore,
        scoreLevelActive,
        scoreLevels,
        testPaperFiles,
        answerFiles,
        grade,
        quarterName,
      };
    }
    dispatch({
      type: 'main/openPage',
      payload: {
        path: `/StudentTestPaperManage/Detail`,
        params: {
          type,
          data,
          onSaveSuccess: this.onSaveDetailSuccess,
          branchDict,
          testTypeDict,
          productTypeDict,
          yearDict,
          quarterDict,
          gradeDict,
          subjectDict,
          newClassTypeDict,
          scoreLevelDict,
          statusDict,
          classTypeDict,
          editing,
        },
        refresh: true,
      },
    });
  };

  onSaveDetailSuccess = () => {
    const { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: {
        path: `/StudentTestPaperManage`,
      },
    });
    this.search();
  };

  /**
   *  产品类型改变获取对应的班级类型
   */
  handleProductTypeChange = async (productType) => {
    this.setState({ classTypeId: null, productType });
    if (productType) {
      const newClassTypeDict = await loadCommonDict('CLASS_TYPE', productType);
      this.setState({ newClassTypeDict });
    }
  };

  onPaginationChange = ({ current, pageSize }) => {
    this.setState({ loading: true });
    const changed = {
      currentPage: current,
      pageSize,
    };
    this.setState({ loading: true });
    loaders.loadList({ ...this.getListParams(), ...changed }).then(
      ({ list, totalRecords }) => {
        this.setState({ loading: false, ...changed, list, totalRecords });
      },
      () => {
        this.setState({ loading: false });
      },
    );
  };

  setStatus = (id, status) => {
    const { statusDict } = this.state;
    const _setStatus = () => {
      this.setState({ loading: true });
      loaders.setStatus({ id, status }).then(
        () => {
          showMessage(`试卷状态已设置为${getDictTextByVal(statusDict, status)}！`, 'success');
          this.search();
        },
        (e) => {
          this.setState({ loading: false });
          showMessage(e.text || '设置失败');
        },
      );
    };
    if (status === dicts.STATUS_INVALID) {
      showConfirm(
        <div>
          <div className={styles.confirmTitle}>是否确定无效当前试卷？</div>
          <div className={styles.confirmContent}>已使用该试卷的学生不受影响</div>
        </div>,
        _setStatus,
      );
    } else {
      _setStatus();
    }
  };

  refreshListFixedCol = (idx, px) => {
    /*    const evObj = document.createEvent('HTMLEvents')
    evObj.initEvent('resize',true,true)
    window.dispatchEvent(evObj)*/
    const trs = ReactDOM.findDOMNode(this.listRef).querySelectorAll('.ant-table-fixed-right .ant-table-fixed tbody tr');
    trs[idx].style.height = px + 'px';
  };

  render() {
    const { btnAuths } = this.props;
    const {
      loading,
      searchExpand,
      id,
      branch,
      testType,
      name,
      productType,
      year,
      quarter,
      grade,
      subject,
      status,
      branchDict,
      testTypeDict,
      productTypeDict,
      yearDict,
      quarterDict,
      gradeDict,
      subjectDict,
      classTypeDict,
      scoreLevelDict,
      statusDict,
      list,
      totalRecords,
      currentPage,
      pageSize,
      classTypeId,
      newClassTypeDict,
    } = this.state;
    const columns = [
      { title: '考试分公司', dataIndex: 'branch', dict: branchDict },
      { title: '试卷名称', dataIndex: 'name', width: COL_WIDTH_XL },
      // { title: '产品类型', dataIndex: 'productType', dict: productTypeDict },
      { title: '年份', dataIndex: 'year', dict: yearDict, width: COL_WIDTH_XS },
      {
        title: '季度',
        dataIndex: 'quarterName',
        width: COL_WIDTH_XS,
        render: (val) => {
          return <RenderMultiLine detail={val} maxLine={3} />;
        },
      },
      { title: '年级', dataIndex: 'grade', dict: gradeDict, width: COL_WIDTH_XS },
      { title: '科目', dataIndex: 'subject', dict: subjectDict, width: COL_WIDTH_XS },
      { title: '考试类型', dataIndex: 'testType', dict: testTypeDict },
      { title: '状态', dataIndex: 'status', dict: statusDict, width: COL_WIDTH_XS },
      {
        title: '更新时间',
        dataIndex: 'updateTime',
        render: (val) => {
          return moment(val).format('YYYY-MM-DD HH:mm:ss');
        },
      },
      {
        title: '创建者',
        dataIndex: 'createUser',
        render: (val) => {
          return val;
        },
      },
      // { title: '总分', dataIndex: 'totalScore' },
      // { title: '班级类型', dataIndex: 'classTypeId',dict: classTypeDict },
      // { title: '得分等级', dataIndex: 'scoreLevels', width: COL_WIDTH_XL,
      //   render: (text, record, index) => {
      //     if (!record.scoreLevelActive) {
      //       return '未激活'
      //     }
      //     return <ScoreLevelCell data={record.scoreLevels} scoreLevelDict={scoreLevelDict} onRefreshList={px => this.refreshListFixedCol(index, px)} />
      //   }
      // },
      {
        title: '操作',
        fixed: 'right',
        key: 'action',
        width: COL_WIDTH_L,
        getActionConfig: (text, record, index) => {
          const { id, status } = record;
          let actions = [{ text: '详情', onClick: () => this.goDetail(null, record) }];
          if (btnAuths.indexOf('STUDENT_TEST_PAPER_COPY_BTN') >= 0) {
            actions.push({ text: '复制', onClick: () => this.goDetail('copy', record) });
          }
          if (status === dicts.STATUS_VALID) {
            actions.push({ text: '无效', onClick: () => this.setStatus(id, dicts.STATUS_INVALID) });
          } else {
            actions.push({ text: '有效', onClick: () => this.setStatus(id, dicts.STATUS_VALID) });
          }
          if (btnAuths.indexOf('STUDENT_TEST_PAPER_EDIT_BTN') >= 0) {
            actions.push({
              text: '编辑',
              onClick: () => {
                this.goDetail(null, record, true);
              },
            });
          }
          return actions;
        },
      },
    ];

    return (
      <Page
        fitWindowHeight={true}
        loading={loading}
        route={this.props.route}
        subHeader={
          <div className={styles.wrap}>
            <SearchBar
              expand={searchExpand}
              onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
              onSearch={this.search}
              onReset={this.reset}
            >
              <Input placeholder="试卷ID" value={id} onChange={(e) => this.setState({ id: e.target.value })} />
              <Select
                placeholder="分公司"
                allowClear={false}
                dict={branchDict}
                value={branch}
                onChange={(val) => this.setState({ branch: val })}
              />
              <Select
                placeholder="考试类型"
                dict={testTypeDict}
                value={testType}
                onChange={(val) => this.setState({ testType: val })}
              />
              <Input placeholder="试卷名称" value={name} onChange={(e) => this.setState({ name: e.target.value })} />
              <Select
                placeholder="产品类型"
                dict={productTypeDict}
                value={productType}
                onChange={(val) => this.handleProductTypeChange(val)}
              />
              <Select
                placeholder="年份"
                dict={yearDict}
                value={year}
                onChange={(val) => this.setState({ year: val })}
              />
              <Select
                placeholder="季度"
                dict={quarterDict}
                value={quarter}
                onChange={(val) => this.setState({ quarter: val })}
              />
              <Select
                placeholder="年级"
                dict={gradeDict}
                value={grade}
                onChange={(val) => this.setState({ grade: val })}
              />
              <Select
                placeholder="科目"
                dict={subjectDict}
                value={subject}
                onChange={(val) => this.setState({ subject: val })}
              />
              <Select
                allowClear={false}
                placeholder="状态"
                dict={
                  !!~this.props.btnAuths.indexOf('STUDENT_TEST_PAPER_NEED_INVALID')
                    ? statusDict
                    : statusDict.filter((item) => item.val !== '1')
                }
                value={status}
                onChange={(val) => this.setState({ status: val })}
              />
              <MultiSelect
                placeholder="班级类型"
                dict={newClassTypeDict}
                value={classTypeId}
                onChange={(val) => this.setState({ classTypeId: val })}
              />
            </SearchBar>
            {btnAuths.indexOf('STUDENT_TEST_PAPER_ADD_BTN') >= 0 && (
              <div className={styles.rightBtnWrap}>
                <Button onClick={() => this.goDetail('add')}>新增</Button>
              </div>
            )}
          </div>
        }
      >
        <List
          dataSource={list}
          columns={columns}
          pagination={{
            current: currentPage,
            total: totalRecords,
            pageSize,
          }}
          onChange={this.onPaginationChange}
          ref={(c) => (this.listRef = c)}
        />
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    btnAuths: state.account.btnAuths,
  };
};

export default connect(mapStateToProps)(TestPaperNamage);
