import dicts from './dicts';
import { getIconSrc } from './utils';
import { getPaperList, upatePaperStatus, savePaper } from 'SERVICE/student';
import { getDataDictList } from 'SERVICE/system';

const loadList = async ({
  id,
  branch,
  testType,
  name,
  productType,
  year,
  quarter,
  grade,
  subject,
  status,
  currentPage,
  pageSize,
  classTypeId,
}) => {
  try {
    let params = {
      id,
      branchId: branch,
      typeId: testType,
      paperName: name,
      productId: productType,
      yearId: year,
      quarterId: quarter,
      gradeId: grade,
      subjectId: subject,
      page: currentPage,
      rows: pageSize,
      classTypeId,
    };
    if (status) {
      params.status = status === dicts.STATUS_VALID ? 2 : 1;
    }
    const data = await getPaperList(params, true);
    const rows = data.rows || [];
    const list = rows.map((d) => {
      const paperScoreRank = d.paperScoreRank || [];
      const scoreLevels = paperScoreRank.map((p, i) => {
        return {
          id: '' + i,
          type: p.classLevelId,
          start: p.minScore,
          end: p.maxScore,
        };
      });
      const paperFileVo = d.paperFileVo || [];
      const getFile = (p, i) => {
        const name = p.fileName;
        const src = p.filePath;
        return { listId: '' + i, id: p.fileId, name, src, previewPath: src, iconSrc: getIconSrc(name, src) };
      };
      const testPaperFiles = paperFileVo.filter((pa) => pa.fileType === 0).map(getFile);
      const answerFiles = paperFileVo.filter((pa) => pa.fileType === 1).map(getFile);
      return {
        id: d.id,
        branch: d.branchId,
        testType: d.typeId,
        name: d.paperName,
        productType: d.productId,
        year: d.yearId,
        quarter: d.quarterId.split(','),
        quarterName: d.quarterName,
        grade: d.gradeId,
        subject: d.subjectId,
        totalScore: d.totalPoint,
        scoreLevels,
        status: d.status ? (d.status === 2 ? dicts.STATUS_VALID : dicts.STATUS_INVALID) : null,
        scoreLevelActive: d.isScoreRank === 0 ? true : false,
        testPaperFiles,
        answerFiles,
        classTypeName: d.classTypeName,
        classTypeId: d.classTypeId,
        createUser: d.createUser,
        updateTime: d.updateTime,
      };
    });
    return { list, totalRecords: data.records };
  } catch (e) {
    return { list: [], totalRecords: 0 };
  }
  /*await new Promise(res => {
    setTimeout(() => {res()}, 1000)
  })
  const scoreLevels = [
    { type: 'bestClass', start: 90, end: 100 },
    { type: 'increaseClass', start: 80, end: 90 },
    { type: 'freeClass', start: 90, end: 100 },
    { type: 'longClass', start: 90, end: 100 },
    { type: 'shortClass', start: 90, end: 100 },
    { type: 'DAT0000000240', start: 90, end: 100 },
    { type: dicts.SCORE_LEVEL_INVALID, start: 0, end: 79 },
  ]
  const list = [
    { id: 'test1', branch: 'x', testType: 'x', name: '2018高考模拟试卷', productType: 'x', year: 'x',
      quarter: 'x', grade: 'x', subject: 'x', totalScore: '120', scoreLevels, status: dicts.STATUS_INVALID },
    { id: 'test2', branch: 'x', testType: 'x', name: '2018高考模拟试卷2', productType: 'x', year: 'x',
      quarter: 'x', grade: 'x', subject: 'x', totalScore: '120', scoreLevels, status: dicts.STATUS_VALID },
  ]
  return { list, totalRecords: 2 }*/
};

const setStatus = async ({ id, status }) => {
  await upatePaperStatus({ id, status: status ? (status === dicts.STATUS_VALID ? 2 : 1) : null }, true);
  return true;
};

const saveDetail = async ({
  id,
  status,
  name,
  testType,
  branch,
  productType,
  classTypeId,
  year,
  quarter,
  grade,
  subject,
  totalScore,
  scoreLevelActive,
  scoreLevels,
  testPaperFiles,
  answerFiles,
}) => {
  try {
    console.log({
      id,
      status,
      name,
      testType,
      branch,
      productType,
      year,
      quarter,
      grade,
      subject,
      totalScore,
      scoreLevelActive,
      scoreLevels,
      testPaperFiles,
      answerFiles,
    });
    const getPaperFile = (f, type) => {
      return { fileId: f.id, fileName: f.name, fileType: type };
    };
    const testPaperFilesVo = testPaperFiles ? testPaperFiles.map((f) => getPaperFile(f, 0)) : [];
    const answerFilesVo = answerFiles ? answerFiles.map((f) => getPaperFile(f, 1)) : [];
    const paperFileVo = [...testPaperFilesVo, ...answerFilesVo];
    const paperScoreRank = scoreLevels
      ? scoreLevels.map((s) => {
          return { classLevelId: s.type, classLevelName: s.typeName, maxScore: s.end, minScore: s.start };
        })
      : null;
    let params = {
      branchId: branch,
      gradeId: grade,
      paperFileVo,
      paperName: name,
      paperScoreRank,
      productId: productType,
      classTypeId,
      quarterId: quarter.join(','),
      status: status ? (status === dicts.STATUS_VALID ? 2 : 1) : null,
      subjectId: subject,
      totalPoint: totalScore,
      typeId: testType,
      yearId: year,
      isScoreRank: scoreLevelActive ? 0 : 1,
    };
    if (id) {
      params.id = id;
    }
    await savePaper(params, true);
    return true;
  } catch (e) {
    console.log(e);
    throw e;
  }
};

const loadClassTypeDict = async () => {
  const { rows } = await getDataDictList({ category: 'CLASS_TYPE', rows: 999 });
  return rows.map((d) => {
    return { val: d.id, text: d.name, parentVal: d.parentDataDictId };
  });
};

export default {
  loadList,
  setStatus,
  saveDetail,
  loadClassTypeDict,
};
