import React, { Component } from 'react';
import Modal from 'COMPONENT/Modal';
import { connect } from 'dva';
import { Row, Col } from 'antd';
import FormItem from 'COMPONENT/FormItem';
import Select from 'COMPONENT/Select';
import Input from 'COMPONENT/Input';
import Button from 'COMPONENT/Button';
import InputNumber from 'COMPONENT/InputNumber';
import Popover from 'COMPONENT/Popover';
import Upload from 'COMPONENT/Upload';
import QrcodeFileUpload from 'COMPONENT/QrcodeFileUpload';
import FileList from 'COMPONENT/FileList';
import { isImg } from 'COMPONENT/FileIcon/utils';
import { loadCommonDict, loadBranchDict, loadCampusDict } from 'UTIL/dict';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import PreviewImgs from 'COMPONENT/PreviewImgs';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import getWrappedInstance from 'UTIL/getWrappedInstance';
import { getCommonErrorTips } from 'COMPONENT/utils/form';
import { FILE_TYPE_MAP_EXTENSION } from 'COMPONENT/FileIcon/config';
import { debounce } from 'lodash';
import Loaders from '../loader';
import styles from '../styles.less';
const getIconSrc = (fileName, src) => {
  const extension = fileName.slice(fileName.lastIndexOf('.') + 1);
  if (src && isImg({ extension })) {
    return src + '?x-oss-process=image/resize,m_fixed,h_38,w_38';
  }
};
const validateFileTypes = ['png', 'jpg', 'bmp'];
const maxUploadFileCount = 5;
const studentTempDict = [{ text: '请先选择客户号码', val: '', disabled: true }];
const campusTempDict = [{ text: '请先选择分公司', val: '', disabled: true }];
const paperTempDict = [{ text: '请先选择分公司', val: '', disabled: true }];
class EditScoreModal extends Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: true,
      imgShow: false,
      attachmentUrl: null,
      //表单数据
      detailData: null,
      customerId: null,
      studentId: null,
      branchId: null,
      campusId: null,
      paperId: null,
      score: null,
      typeName: '自动匹配',
      productName: '自动匹配',
      yearName: '自动匹配',
      quarterName: '自动匹配',
      gradeName: '自动匹配',
      subjectName: '自动匹配',
      totalPoint: '自动匹配',
      gradeRanking: null,
      gradePeopleAmount: null,
      classRanking: null,
      classPeopleAmount: null,
      //数据字典
      branchDict: null,
      campusDict: null,
      studentDict: null,
      customerPhoneDict: null,
      //文件
      fileList: [],

      isScoreRank: null, // 是否激活,0 激活,1 未激活
      paperScoreRank: null, // 试卷得分等级

      classTypeId: null, // 试卷绑定的班级 ID
      classTypeName: null, // 试卷绑定的班级 名称
    };
    this.dictVersion = 1; //客户号码数据字典版本号
    this.paperDictVersion = 1; //试卷数据字典版本号
    this.fileIdx = 1; //文件index
    this.checking = false; //是否显示校验提示
    this.errors = {}; //校验错误信息
    this.classTypeResult = {
      classlevelName: undefined,
      classlevelId: undefined,
      firstIn: true,
    };
  }

  componentDidMount() {
    this.initContent();
  }
  /**
   * 初始化数据
   */
  initContent = async () => {
    const { editScoreId } = this.props;
    const asyncCalls = [loadBranchDict(), loadCommonDict('BRANCH_EXAM_TYPE'), loadCommonDict('PRODUCT_TYPE')];
    if (editScoreId) {
      asyncCalls.push(Loaders.loadDetail({ id: editScoreId }));
    }
    Promise.all(asyncCalls).then(async ([branchDict, examTypeDict, productTypeDict, detailData]) => {
      if (!detailData) {
        this.updateState({
          loading: false,
          branchDict,
          examTypeDict,
          productTypeDict,
        });
      } else {
        Promise.all([
          loadCampusDict(detailData.branchId),
          Loaders.loadPaperDetail({ branchId: detailData.branchId, id: detailData.paperId }),
        ]).then(([campusDict, paperDetail]) => {
          this.classTypeResult.classlevelId = detailData.classlevelId;
          this.classTypeResult.classlevelName = detailData.classlevelName;
          this.updateState({
            loading: false,
            detailData: detailData,
            branchDict,
            examTypeDict,
            productTypeDict,
            campusDict,
            ...detailData,
            ...paperDetail,
            classTypeId: detailData.classTypeId,
            classTypeName: detailData.classTypeName,
          });
        });
      }
    });
  };
  /**
   * 输入用户号码搜索关键词
   */
  onSearchKeyChange = (key) => {
    debounce(this.search, 300)(key);
  };

  /**
   * 搜索
   */
  search = async (key) => {
    if (key) {
      const dictVersion = ++this.dictVersion;
      await this.setState({ loading: true });
      const result = await Loaders.loadContact({
        term: key,
        monitoringUserId: this.props.monitoringUserId,
        type: 'achievement',
      });
      if (this.dictVersion === dictVersion) {
        this.setState({ customerPhoneDict: result, loading: false });
      }
    } else {
      this.dictVersion++;
      this.setState({ customerPhoneDict: [], loading: false });
    }
  };
  /**
   * 输入试卷名称搜索
   */
  onPaperKeyChange = (key) => {
    debounce(this.searchPaper, 300)(key);
  };

  /**
   * 搜索
   */
  searchPaper = async (key) => {
    if (key) {
      const dictVersion = ++this.paperDictVersion;
      const { branchId } = this.state;
      console.log(this.state);
      await this.setState({ loading: true });
      const result = await Loaders.loadPaperDict({ paperName: key, branchId });
      if (this.paperDictVersion === dictVersion) {
        this.setState({ paperDict: result, loading: false });
      }
    } else {
      this.paperDictVersion++;
      this.setState({ paperDict: [], loading: false });
    }
  };
  /**
   * 保存
   */
  saveDetail = async (params) => {
    console.log(params);
    const { editScoreId } = this.props;
    if (editScoreId) {
      return Loaders.edit(params);
    } else {
      return Loaders.add(params);
    }
  };
  /**
   * 更新状态
   * @param  {object} changedState 改变的状态
   * @return {Promise}
   */
  updateState = (changedState) => {
    const newState = {
      ...this.state,
      ...changedState,
    };
    return new Promise((resolve) => {
      this.setState(newState, () => {
        resolve();
      });
    });
  };
  /**
   * 改变客户号码时获取对应的学生字典
   */
  handlePhoneChange = async (customerId) => {
    await this.updateState({ customerId, studentId: null });
    if (customerId) {
      await this.updateState({ loading: true });
      const studentDict = await Loaders.loadStudentDict({ id: customerId });
      this.updateState({
        studentDict,
        loading: false,
      });
    }
  };
  /**
   * 改变试卷时匹配试卷的其它信息
   */
  handlePaperChange = async (paperId) => {
    await this.updateState({ paperId, loading: true });
    if (paperId) {
      const paperDetail = await Loaders.loadPaperDetail({ id: paperId, branchId: this.state.branchId });
      console.log('paperDetail');
      console.log(paperDetail);
      this.updateState({
        loading: false,
        ...paperDetail,
      });
    } else {
      this.updateState({
        yearName: '自动匹配',
        quarterName: '自动匹配',
        gradeName: '自动匹配',
        subjectName: '自动匹配',
        totalPoint: '自动匹配',
        loading: false,
      });
    }
  };
  /**
   * 分公司改变获取对应的学习中心字典
   */
  handleBranchChange = async (branchId) => {
    await this.updateState({ branchId, campusId: null, paperId: null, paperDict: [] });
    if (branchId) {
      await this.setState({ loading: true });
      loadCampusDict(branchId).then((campusDict) => {
        this.updateState({
          campusDict,
          loading: false,
        });
      });
    }
  };
  /**
   * 文件改变时
   */
  onUploadChange = (info) => {
    console.log(info);
    this.setState({
      fileList: this.state.fileList.map((file) => {
        const newFile = info.find((inf) => inf.fileKey === file.fileKey);
        if (newFile) {
          const name = file.name;
          const src = newFile.src;
          return {
            ...newFile,
            id: newFile['x:fileid'],
            src,
            name,
            listId: file.listId,
            error: newFile.errorCode ? '上传失败' : null,
            iconSrc: newFile.errorCode ? null : getIconSrc(name, src),
          };
        }
        return file;
      }),
    });
  };
  genFileId = () => {
    return '' + this.fileIdx++;
  };

  /**
   * 二维码传图
   */
  get2dUploadImgInfo = (info) => {
    const { fileList } = this.state;
    if (fileList.length >= maxUploadFileCount) {
      return;
    }
    const newFileList = [...fileList];
    info.forEach((file) => {
      const name = file.realFileName;
      const src = file.filePath;
      if (!newFileList.some((nf) => nf.src === src) && newFileList.length < maxUploadFileCount) {
        newFileList.push({ id: file.fileid, name, src, iconSrc: getIconSrc(name, src), listId: this.genFileId() });
      }
    });
    this.setState({
      fileList: newFileList,
    });
  };

  /**
   * 上传文件前检查文件
   */
  checkLength = (files) => {
    let filesMayUpload = [];
    let filesWillUpload = [];
    const restCount = maxUploadFileCount - this.state.fileList.length;
    files.slice(0, restCount).map((file) => {
      const listId = this.genFileId();
      let newFile = { fileKey: file.fileKey, name: file.name, listId };
      let fileExtends = [];
      validateFileTypes.forEach((type) => {
        fileExtends = [...fileExtends, ...FILE_TYPE_MAP_EXTENSION[type]];
      });
      const name = file.name;
      const idx = name.lastIndexOf('.');
      const nameExtend = name.slice(idx + 1);
      if (file.size > 1024 * 1024 * 10) {
        newFile = { ...newFile, error: '文件大小超过了10MB' };
      } else if (fileExtends.indexOf(nameExtend.toLowerCase()) < 0) {
        newFile = { ...newFile, error: '文件格式不正确' };
      } else if (name.length - nameExtend.length - 1 > 50) {
        newFile = { ...newFile, error: '文件名超过50字' };
      } else {
        filesWillUpload.push(file);
      }
      filesMayUpload.push(newFile);
    });
    if (restCount < files.length) {
      return new Promise((res, rej) => {
        showConfirm(
          `最多上传${maxUploadFileCount}个文件，超出部分将不上传`,
          () => {
            this.setState({ fileList: [...this.state.fileList, ...filesMayUpload] });
            res(filesWillUpload);
          },
          () => {
            rej();
          },
        );
      });
    } else {
      this.setState({ fileList: [...this.state.fileList, ...filesMayUpload] });
      return filesWillUpload;
    }
  };
  /**
   * 确认按钮
   */
  handleOk = async () => {
    const {
      checking,
      branchId,
      campusId,
      paperId,
      studentId,
      customerId,
      score,
      fileList,
      totalPoint,
      detailData,
      gradeRanking,
      gradePeopleAmount,
      classRanking,
      classPeopleAmount,
      classTypeId,
      classTypeName,
    } = this.state;
    const { classlevelId, classlevelName } = this.classTypeResult;
    if (!checking) {
      this.setState({ checking: true });
    }
    let errorTips = getCommonErrorTips(this.errors);
    if (errorTips) {
      this.modal.showOkTips(errorTips);
      return;
    }
    if (fileList.some((file) => file.error)) {
      this.modal.showOkTips('请先处理上传失败的文件');
      return;
    }
    if (Number(score) > Number(totalPoint)) {
      this.modal.showOkTips('试卷得分不能大于试卷总分');
      return;
    }
    if (
      fileList.some((file) => {
        const name = file.name;
        if (name.substring(0, name.lastIndexOf('.')) === '') {
          return true;
        }
        return false;
      })
    ) {
      this.modal.showOkTips('附件名不能为空');
      return;
    }
    const { editScoreId } = this.props;
    this.setState({ loading: true });
    this.saveDetail({
      ...detailData,
      classTypeId,
      classTypeName,
      studentId,
      branchId,
      campusId,
      paperId,
      score,
      classlevelId,
      classlevelName,
      customerId: customerId ? customerId : undefined,
      id: editScoreId ? editScoreId : undefined,
      gradeRanking,
      gradePeopleAmount,
      classRanking,
      classPeopleAmount,
      attachmentId: fileList.length
        ? fileList.map((file) => file.id).join(',')
        : this.props.editScoreId
        ? detailData.attachmentId
        : undefined,
    }).then(
      () => {
        this.setState({ loading: false });
        //保存后的回掉，关闭模态框,更新列表等
        this.props.onOk();
        showMessage('保存成功', 'success');
      },
      (e) => {
        this.setState({ loading: false });
        if (e.text) {
          this.modal.showOkTips(e.text);
        } else {
          showMessage('保存失败');
        }
      },
    );
  };
  /**
   * 保存校验信息
   */
  errorChange = (key, error) => {
    this.errors[key] = error;
  };
  /**
   * 分数改变
   */
  onScoreChange = (score) => {
    this.matchGrade(score);
    this.updateState({ score });
  };
  /**
   * 年级排名改变
   */
  onGradeRanking = (gradeRanking) => {
    this.updateState({ gradeRanking });
  };
  /**
   * 年级人数改变
   */
  onGradePeopleAmount = (gradePeopleAmount) => {
    this.updateState({ gradePeopleAmount });
  };
  /**
   * 班级排名改变
   */
  onClassRanking = (classRanking) => {
    this.updateState({ classRanking });
  };
  /**
   * 班级人数改变
   */
  onClassPeopleAmount = (classPeopleAmount) => {
    this.updateState({ classPeopleAmount });
  };
  /**
   * 匹配对应的等级
   */
  matchGrade = (scoreStr) => {
    const { paperScoreRank, isScoreRank } = this.state;
    const score = Number(scoreStr);
    let result = { classlevelName: '自动匹配' };
    if (score) {
      if (score) {
        result =
          paperScoreRank &&
          paperScoreRank.find((item) => {
            return score <= item.maxScore && score >= item.minScore;
          });
      }

      if (!result) {
        result = { classlevelName: '无匹配' };
      }

      if (isScoreRank !== 0 && paperScoreRank) {
        result = { classlevelName: '未激活' };
      }

      this.classTypeResult.classlevelName = result.classLevelName;
      this.classTypeResult.classlevelId = result.classLevelId;
      // return result.classlevelName
    }
  };
  /**
   * 展示图片
   */
  showImg = () => {
    this.setState({
      imgShow: true,
    });
  };
  /**
   * 取消按钮
   */
  handleCancel = () => {
    this.props.onCancel();
  };

  render() {
    const {
      checking,
      loading,
      customerId,
      studentId,
      branchId,
      campusId,
      paperId,
      score,
      typeName,
      productName,
      yearName,
      quarterName,
      gradeName,
      subjectName,
      totalPoint,
      customerPhoneDict,
      attachmentUrl,
      branchDict,
      campusDict,
      examTypeDict,
      productTypeDict,
      studentDict,
      imgShow,
      paperDict,
      fileList,
      gradeRanking,
      gradePeopleAmount,
      classRanking,
      classPeopleAmount,
    } = this.state;
    const urlArray = attachmentUrl ? attachmentUrl.split(',') || [] : [];
    const { editScoreId, editStudentName } = this.props;
    return (
      <Modal
        title={editScoreId ? '编辑成绩 (' + editStudentName + ' ' + editScoreId + ')' : '新增成绩'}
        loading={loading}
        onOk={this.handleOk}
        onCancel={this.handleCancel}
        ref={(c) => (this.modal = getWrappedInstance(c))}
        width="1000"
      >
        <div>
          {!editScoreId && (
            <Row>
              <Col span={12}>
                <FormItem name="客户号码" labelWidth={90} required>
                  <Select
                    value={customerId}
                    dict={customerPhoneDict}
                    onChange={this.handlePhoneChange}
                    onSearchKeyChange={this.onSearchKeyChange}
                    showSearch={true}
                    noOptionTips="请输入客户号码进行搜索"
                    validator={{
                      silent: !checking,
                      onErrorChange: (e) => this.errorChange('customerId', e),
                    }}
                  />
                </FormItem>
              </Col>
              <Col span={12}>
                <FormItem name="学生姓名" labelWidth={90} required>
                  <Select
                    value={studentId}
                    dict={customerId ? studentDict : studentTempDict}
                    onChange={(studentId) => {
                      this.updateState({ studentId });
                    }}
                    showSearch={true}
                    validator={{
                      silent: !checking,
                      onErrorChange: (e) => this.errorChange('studentId', e),
                    }}
                  />
                </FormItem>
              </Col>
            </Row>
          )}
          <Row>
            <Col span={12}>
              <FormItem name="考试分公司" labelWidth={90} required>
                <Select
                  value={branchId}
                  dict={branchDict}
                  onChange={this.handleBranchChange}
                  showSearch={true}
                  validator={{
                    silent: !checking,
                    onErrorChange: (e) => this.errorChange('branchId', e),
                  }}
                />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="考试学习中心" labelWidth={90} required>
                <Select
                  value={campusId}
                  dict={branchId ? campusDict : campusTempDict}
                  showSearch={true}
                  onChange={(campusId) => {
                    this.updateState({ campusId });
                  }}
                  validator={{
                    silent: !checking,
                    onErrorChange: (e) => this.errorChange('campusId', e),
                  }}
                />
              </FormItem>
            </Col>
          </Row>
          <Row>
            <Col span={24}>
              <FormItem name="考试试卷" labelWidth={90} required>
                <Select
                  value={paperId}
                  dict={branchId ? paperDict : paperTempDict}
                  showSearch={true}
                  onChange={this.handlePaperChange}
                  noOptionTips="请输入试卷名称进行搜索"
                  onSearchKeyChange={this.onPaperKeyChange}
                  validator={{
                    silent: !checking,
                    onErrorChange: (e) => this.errorChange('paperId', e),
                  }}
                />
              </FormItem>
            </Col>
          </Row>
          <Row>
            <Col span={12}>
              <FormItem name="考试类型" labelWidth={90}>
                <span>{typeName}</span>
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="产品类型" labelWidth={90}>
                <span>{productName}</span>
              </FormItem>
            </Col>
          </Row>
          <Row>
            <Col span={12}>
              <FormItem name="试卷年份" labelWidth={90}>
                <span>{yearName}</span>
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="试卷季度" labelWidth={90}>
                <span>{quarterName}</span>
              </FormItem>
            </Col>
          </Row>
          <Row>
            <Col span={12}>
              <FormItem name="试卷年级" labelWidth={90}>
                <span>{gradeName}</span>
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="试卷科目" labelWidth={90}>
                <span>{subjectName}</span>
              </FormItem>
            </Col>
          </Row>
          <Row>
            <Col span={12}>
              <FormItem name="试卷总分" labelWidth={90}>
                <span>{totalPoint}</span>
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="试卷得分" labelWidth={90} required>
                <InputNumber
                  value={score}
                  min={0}
                  precision={2}
                  // max={totalPoint !== '自动匹配' ? totalPoint : 999}
                  onChange={this.onScoreChange}
                  validator={{
                    silent: !checking,
                    rules: [
                      (value) => {
                        if (value > totalPoint) {
                          return '试卷得分不能大于试卷总分';
                        }
                      },
                    ],
                    onErrorChange: (e) => this.errorChange('score', e),
                  }}
                />
              </FormItem>
            </Col>
          </Row>
          <Row>
            <Col span={12}>
              <FormItem name="班级层次" labelWidth={90}>
                <span>{this.classTypeResult.classlevelName}</span>
              </FormItem>
            </Col>

            <Col span={12}>
              <FormItem name="年级排名" labelWidth={90}>
                <InputNumber value={gradeRanking} min={0} maxLength={6} onChange={this.onGradeRanking} />
              </FormItem>
            </Col>
          </Row>
          <Row>
            <Col span={12}>
              <FormItem name="年级人数" labelWidth={90}>
                <InputNumber value={gradePeopleAmount} min={0} maxLength={6} onChange={this.onGradePeopleAmount} />
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem name="班级排名" labelWidth={90}>
                <InputNumber value={classRanking} min={0} maxLength={6} onChange={this.onClassRanking} />
              </FormItem>
            </Col>
          </Row>
          <Row>
            <Col span={12}>
              <FormItem name="班级人数" labelWidth={90}>
                <InputNumber value={classPeopleAmount} min={0} maxLength={6} onChange={this.onClassPeopleAmount} />
              </FormItem>
            </Col>
          </Row>
          <Row>
            <Col span={24}>
              <FormItem labelWidth="90" name="答卷图片" required>
                {!editScoreId && (
                  <div>
                    <div>
                      <div style={{ display: 'inline-block', marginRight: 20 }}>
                        {fileList.length >= maxUploadFileCount ? (
                          <Button disabled>点击上传</Button>
                        ) : (
                          <Upload
                            onChange={this.onUploadChange}
                            multiple={true}
                            beforeUpload={(files) => this.checkLength(files)}
                          >
                            <Button>点击上传</Button>
                          </Upload>
                        )}
                      </div>
                      <div style={{ display: 'inline-block' }}>
                        {fileList.length >= maxUploadFileCount ? (
                          <Button disabled>二维码传图</Button>
                        ) : (
                          <Popover
                            content={
                              <QrcodeFileUpload height={160} width={160} callback={this.get2dUploadImgInfo} multiple />
                            }
                            placement="right"
                          >
                            <Button>扫码传图</Button>
                          </Popover>
                        )}
                      </div>
                      <div className={styles.tips}>
                        {` 支持格式有${validateFileTypes.join('、')}，只能上传${maxUploadFileCount}张图片`}
                      </div>
                    </div>
                    <div style={{ marginTop: 5 }}>
                      <FileList
                        data={fileList.map((file) => {
                          return { ...file, fileId: file.id, id: file.listId };
                        })}
                        onChange={(list) =>
                          this.setState({
                            fileList: list.map((file) => {
                              return { ...file, id: file.fileId, listId: file.id };
                            }),
                          })
                        }
                      />
                    </div>
                  </div>
                )}
                {editScoreId && !attachmentUrl && (
                  <div>
                    <div>
                      <div style={{ display: 'inline-block', marginRight: 20 }}>
                        {fileList.length >= maxUploadFileCount ? (
                          <Button disabled>点击上传</Button>
                        ) : (
                          <Upload
                            onChange={this.onUploadChange}
                            multiple={true}
                            beforeUpload={(files) => this.checkLength(files)}
                          >
                            <Button>点击上传</Button>
                          </Upload>
                        )}
                      </div>
                      <div style={{ display: 'inline-block' }}>
                        {fileList.length >= maxUploadFileCount ? (
                          <Button disabled>二维码传图</Button>
                        ) : (
                          <Popover
                            content={
                              <QrcodeFileUpload height={160} width={160} callback={this.get2dUploadImgInfo} multiple />
                            }
                            placement="right"
                          >
                            <Button>扫码传图</Button>
                          </Popover>
                        )}
                      </div>
                      <div className={styles.tips}>
                        {`支持格式有${validateFileTypes.join('、')}，只能上传${maxUploadFileCount}张图片`}
                      </div>
                    </div>
                    <div style={{ marginTop: 5 }}>
                      <FileList
                        data={fileList.map((file) => {
                          return { ...file, fileId: file.id, id: file.listId };
                        })}
                        onChange={(list) =>
                          this.setState({
                            fileList: list.map((file) => {
                              return { ...file, id: file.fileId, listId: file.id };
                            }),
                          })
                        }
                      />
                    </div>
                  </div>
                )}
                {editScoreId && attachmentUrl && fileList.length === 0 && (
                  <div>
                    {urlArray.map((url) => (
                      <img
                        style={{ cursor: 'pointer', marginRight: 20 }}
                        width="50px"
                        height="50px"
                        onClick={this.showImg}
                        src={url}
                        alt="试卷图片"
                      />
                    ))}
                    <Upload
                      onChange={this.onUploadChange}
                      multiple={true}
                      beforeUpload={(files) => this.checkLength(files)}
                    >
                      <Button>更换答卷</Button>
                    </Upload>
                  </div>
                )}
                {editScoreId && attachmentUrl && fileList.length !== 0 && (
                  <div>
                    <div style={{ display: 'inline-block', marginRight: 20 }}>
                      {fileList.length >= maxUploadFileCount ? (
                        <Button disabled>继续上传</Button>
                      ) : (
                        <Upload
                          onChange={this.onUploadChange}
                          multiple={true}
                          beforeUpload={(files) => this.checkLength(files)}
                        >
                          <Button>继续上传</Button>
                        </Upload>
                      )}
                    </div>
                    <div style={{ marginTop: 5 }}>
                      <FileList
                        data={fileList.map((file) => {
                          return { ...file, fileId: file.id, id: file.listId };
                        })}
                        onChange={(list) =>
                          this.setState({
                            fileList: list.map((file) => {
                              return { ...file, id: file.fileId, listId: file.id };
                            }),
                          })
                        }
                      />
                    </div>
                  </div>
                )}
              </FormItem>
            </Col>
          </Row>
        </div>
        {imgShow && (
          <Modal
            width={800}
            okBtnShow={false}
            cancelBtnShow={false}
            closable={true}
            onClose={() => this.updateState({ imgShow: false })}
          >
            <PreviewImgs list={urlArray} activeSrc={urlArray[0]} showThumbnail={false} />
          </Modal>
        )}
      </Modal>
    );
  }
}
const mapStateToProps = (state) => {
  return {
    monitoringUserId: state.account.user.userId,
  };
};
export default connect(mapStateToProps)(EditScoreModal);
