import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import Block from 'COMPONENT/Block';
import Button from 'COMPONENT/Button';
import Validator from 'COMPONENT/Validator';
import { SearchFilter, SearchField } from 'COMPONENT/SearchFilter';
import TextLink from 'COMPONENT/TextLink';
import Input from 'COMPONENT/Input';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import List from 'COMPONENT/List';
import { DatePicker } from 'antd';
import styles from '../styles.less';
import { urlBase } from 'CONST/config';
import { OSS_BASE_PREFIX } from 'CONST/ossConfig';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { getRelationsDictPlus, getRelationPayloadPlus } from 'MODEL/dict/utils/index';
import { storeName } from './store/config';
import { branch, campus } from 'MODEL/dict/utils/config';
import { exportFailingData } from 'SERVICE/student';
import Modal from 'COMPONENT/Modal';
import { RenderMultiLine } from '../renderMultiLine';
import { getAliyunOSSKeyWithoutCallBack } from 'SERVICE/course';
import { achievementBatchImport } from 'SERVICE/enrollManager';
import moment from 'moment';

const successAlert = Modal.success;

const RangePicker = DatePicker.RangePicker;
const templateUrl = '/%E6%88%90%E7%BB%A9%E5%AF%BC%E5%85%A5%E6%A8%A1%E6%9D%BF.xls';
const status = {
  IMPORTING: '导入中',
  FAILURE: '导入失败',
  SUCCESS: '导入成功',
};
const statusDict = [
  { val: 0, text: '导入失败' },
  { val: 1, text: '导入中' },
  { val: 2, text: '导入成功' },
];

class ImportScore extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false,

      error: {},
    };

    // 分公司关联关系type、key对应关系配置
    this.branchTreeMap = {
      [branch]: 'sKey_branch',
      [campus]: 'sKey_campus',
    };
    // List 列表列
    this.columns = [
      { title: '文件名称', dataIndex: 'fileName', key: 'fileName' },
      { title: '分公司', dataIndex: 'branch', key: 'branch' },
      {
        title: '学习中心',
        dataIndex: 'blCampusIds',
        key: 'blCampusIds',
        render: (val) => {
          const campusNames = val.join(',');
          return <RenderMultiLine detail={campusNames} maxLine={3} />;
        },
      },
      { title: '导入总数', dataIndex: 'importNum', key: 'importNum' },
      { title: '导入状态', dataIndex: 'status', key: 'status' },
      {
        title: '成功数量',
        dataIndex: 'successNum',
        key: 'successNum',
        render: (val) => {
          return <span>{val === null ? '-' : val}</span>;
        },
      },
      {
        title: '重复数量',
        dataIndex: 'duplicateNum',
        key: 'duplicateNum',
        render: (val) => {
          return <span>{val === null ? '-' : val}</span>;
        },
      },
      {
        title: '未导入数据',
        dataIndex: 'failureNum',
        key: 'failureNum',
        render: (val, row) => {
          const { importId } = row;
          if (val === null) {
            return '-';
          } else if (val === 0) {
            return 0;
          } else {
            return (
              <div>
                {val}
                <TextLink
                  onClick={() => {
                    this.exportExcel(importId);
                  }}
                  style={{ marginLeft: '10px' }}
                >
                  导出
                </TextLink>
              </div>
            );
          }
        },
      },
      { title: '上传者', dataIndex: 'createUser', key: 'createUser' },
      { title: '上传时间', dataIndex: 'createTimeText', key: 'createTimeText' },
    ];
  }

  componentDidMount() {
    this.init();
  }

  updateProps = (payload) => {
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload,
    });
  };

  init = async () => {
    this.setState({ loading: true });
    await new Promise(this.initDict);
    await new Promise(this.initDefault);
    await new Promise(this.loadData);
    this.setState({ loading: false });
  };

  initDict = (resolve) => {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['branchCampusTree'],
        callback() {
          resolve();
        },
      },
    });
  };

  initDefault = (resolve) => {
    const { branchDict } = this.props;
    this.props.dispatch({
      type: `${storeName}/initSKey`,
      payload: {
        params: {
          sKey_branch: branchDict[0].val,
        },
        callback() {
          resolve();
        },
      },
    });
  };

  updateValue = (key, val) => {
    this.updateProps({ [key]: val });
  };

  loadData = (resolve) => {
    this.props.dispatch({
      type: `${storeName}/loadData`,
      payload: {
        callback() {
          resolve();
        },
      },
    });
  };

  /**
   * 导入成绩
   */
  importScore = async (event) => {
    let file = event.target.files[0];
    const name = file.name;
    const idx = name.lastIndexOf('.');
    const nameExtend = name.slice(idx + 1);
    if (nameExtend.toLowerCase() !== 'xlsx' && nameExtend.toLowerCase() !== 'xls') {
      return showMessage('文件格式不正确');
    }
    const { OSSAccessKeyId, policy, signature, host, Expires, callback } = await getAliyunOSSKeyWithoutCallBack();
    let formData = new FormData();
    const time = moment().format('YYYYMMDDHHmmss');

    formData.append('OSSAccessKeyId', OSSAccessKeyId);
    formData.append('policy', policy);
    formData.append('Signature', signature);
    formData.append('success_action_status', '200');
    formData.append('key', time + name);
    formData.append('file', file);

    const xhr = new XMLHttpRequest();
    xhr.open('POST', host);
    xhr.upload.onloadstart = (file) => {
      this.setState({ loading: true });
    };

    xhr.onload = (res) => {
      if (res.target.status === 200) {
        const response = JSON.parse(res.target.response);
        if (response.resultCode === 0) {
          this.search();
        } else {
          showMessage(response.resultMessage);
        }
      } else {
        showMessage(JSON.parse(res.target.response).resultMessage);
      }
      this.setState({ loading: false });
    };
    xhr.onerror = (e) => {
      showMessage();
      this.setState({ loading: false });
    };

    xhr.onload = async () => {
      if (xhr.status === 200 && xhr.readyState === 4) {
        const params = { fileId: time + name };
        const { resultCode } = await achievementBatchImport(params).catch(() => {
          showMessage('网络错误!');
          this.setState({ loading: false });
        });
        this.search();
        this.setState({ loading: false });
      } else {
        showMessage('系统错误');
        this.setState({ loading: false });
      }
    };
    xhr.onerror = () => {
      showMessage('上传失败');
      this.setState({ loading: false });
    };
    xhr.send(formData);
  };

  /**
   * 下载模板
   */
  downloadTemplate = () => {
    const link = document.createElement('a');
    const body = document.body;
    link.href = `${OSS_BASE_PREFIX}/%E6%88%90%E7%BB%A9%E5%AF%BC%E5%85%A5%E6%A8%A1%E6%9D%BF.xls`;
    link.style.display = 'none';
    link.download = '批量建班模板.xls';
    body.appendChild(link);
    link.click();
    body.removeChild(link);
  };

  updateDateRange = (moments, dates) => {
    this.updateProps({
      sKey_momentRange: moments,
      sKey_dateRange: dates,
    });
  };

  updateBranch = (val) => {
    let payload = getRelationPayloadPlus('sKey_branch', val, {
      type: branch,
      dictTree: this.props.branchCampusTree,
      state: this.props,
      map: this.branchTreeMap,
    });
    this.updateProps(payload);
  };

  errorChange = (err, keyName) => {
    this.setState((prevState) => ({
      error: {
        ...prevState.error,
        [keyName]: err,
      },
    }));
  };

  updatePage = (pageCurrent, pageSize) => {
    let payload = {};
    if (pageSize && pageSize !== this.props.pageSize) {
      payload.pageCurrent = 1;
      payload.pageSize = pageSize;
    } else {
      payload.pageCurrent = pageCurrent;
    }
    this.props.dispatch({
      type: `${storeName}/updateState`,
      payload,
    });
  };

  changePage = async ({ current, pageSize }) => {
    this.updatePage(current, pageSize);
    this.setState({ loading: true });
    await new Promise(this.loadData);
    this.setState({ loading: false });
  };

  search = async () => {
    const {
      error: { sKey_momentRange },
    } = this.state;
    if (sKey_momentRange) {
      return;
    }
    this.setState({ loading: true });
    await new Promise(this.loadData);
    this.setState({ loading: false });
  };

  reset = () => {
    this.props.dispatch({
      type: `${storeName}/resetSKey`,
    });
  };

  exportExcel = (importId) => {
    this.props.dispatch({
      type: `${storeName}/exportExcel`,
      payload: {
        params: {
          importId,
        },
      },
    });
  };

  render() {
    const { loading } = this.state;
    return (
      <Page route={this.props.route} loading={loading}>
        <Block title="导入关联关系">
          <Button style={{ marginRight: '10px' }}>
            导入Excel
            <input
              type="file"
              className={styles.importBtn}
              onClick={(event) => {
                event.target.value = null;
              }}
              onChange={this.importScore}
            />
          </Button>
          <Button onClick={this.downloadTemplate}>模板下载</Button>
        </Block>
        <Block title="导入记录">
          <SearchFilter truncationNum={5} onSearch={this.search} onReset={this.reset}>
            <SearchField span={2}>
              <Validator
                value={this.props.sKey_momentRange[0] && this.props.sKey_momentRange[1] ? 'true' : null}
                rules={['required']}
                silent={!Boolean(this.state.error.sKey_momentRange)}
                showStatus={true}
                showTips={false}
                onErrorChange={(err) => this.errorChange(err, 'sKey_momentRange')}
              >
                <RangePicker
                  placeholder={['开始时间', '结束时间']}
                  value={this.props.sKey_momentRange}
                  onChange={this.updateDateRange}
                  format="YYYY-MM-DD"
                  showTime={false}
                  allowClear={false}
                  size="large"
                />
              </Validator>
            </SearchField>
            <SearchField>
              <Select
                placeholder="分公司"
                value={this.props.sKey_branch}
                dict={this.props.branchDict || []}
                onChange={this.updateBranch}
                allowClear={false}
                showSearch
              />
            </SearchField>
            <SearchField>
              <MultiSelect
                placeholder="学习中心"
                value={this.props.sKey_campus}
                dict={this.props.branchCampusDict || []}
                onChange={(val) => this.updateValue('sKey_campus', val)}
                showSearch
              />
            </SearchField>
            <SearchField>
              <Input
                placeholder="请输入上传者"
                value={this.props.sKey_createUser}
                onChange={(e) => this.updateValue('sKey_createUser', e.target.value)}
              />
            </SearchField>
            <SearchField>
              <Select
                placeholder="导入状态"
                value={this.props.sKey_status}
                dict={statusDict}
                onChange={(val) => this.updateValue('sKey_status', val)}
              />
            </SearchField>
          </SearchFilter>
          <List
            columns={this.columns}
            dataSource={this.props.dataSource}
            pagination={{
              total: this.props.total,
              current: this.props.current,
              pageSize: this.props.pageSize,
            }}
            onChange={this.changePage}
          />
        </Block>
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    sKey_momentRange: state[storeName].sKey_momentRange,
    sKey_dateRange: state[storeName].sKey_dateRange,
    sKey_branch: state[storeName].sKey_branch,
    sKey_campus: state[storeName].sKey_campus,
    sKey_createUser: state[storeName].sKey_createUser,
    sKey_status: state[storeName].sKey_status,

    branchDict: state.dict.branchCampusTree,
    branchCampusDict: getRelationsDictPlus(campus)(state.dict.branchCampusTree, state[storeName].sKey_branch),
    branchCampusTree: state.dict.branchCampusTree,

    dataSource: state[storeName].list,
    total: state[storeName].total,
    pageSize: state[storeName].pageSize,
    current: state[storeName].pageCurrent,
  };
};

export default connect(mapStateToProps)(ImportScore);
