import { clone } from 'lodash';
import { runFunc, errorsHandle, getFetchList } from 'UTIL/common';
import { storeName, dateFormat } from './config';
import moment from 'moment';
import { listImportRecord, exportFailingData } from 'SERVICE/student';

const defaultState = {
  sKey: {
    sKey_momentRange: [null, null],
    sKey_dateRange: [null, null],
    sKey_branch: null,
    sKey_campus: null,
    sKey_createUser: null,
    sKey_status: null,
  },
  list: {
    list: [],
    total: 0,
    pageSize: 10,
    pageCurrent: 1,
  },
};
Object.freeze(defaultState);

export default {
  namespace: storeName,
  state: {
    ...clone(defaultState.sKey),
    ...clone(defaultState.list),
    // 默认筛选条件初始化缓存
    sKeyStorage: null,
  },
  reducers: {
    updateState(state, { payload }) {
      return {
        ...state,
        ...payload,
      };
    },

    updateList(state, { payload: { list, total } }) {
      return {
        ...state,
        list: list.map((item) => ({
          ...item,
          createTimeText: moment(item.createTime).format(dateFormat),
        })),
        total,
      };
    },
  },
  effects: {
    *initSKey({ payload: { callback, params } }, { select, put }) {
      const { sKey_branch } = params;
      let defaultState = yield select((state) => {
        if (state[storeName].sKeyStorage) {
          return state[storeName].sKeyStorage;
        } else {
          const { serverDate } = state.account.user;
          let today = new Date(serverDate);
          let startMoment = moment(today).subtract(30, 'days');
          let endMoment = moment(today);
          return {
            sKey_momentRange: [startMoment, endMoment],
            sKey_dateRange: [startMoment.format('YYYY-MM-DD'), endMoment.format('YYYY-MM-DD')],
          };
        }
      });
      yield put({
        type: 'updateState',
        payload: {
          sKey_branch,
          ...defaultState,
          sKeyStorage: { ...defaultState, sKey_branch },
        },
      });
      runFunc(callback);
    },
    *loadData({ payload: { callback } }, { put, select }) {
      const params = yield select((state) => {
        const {
          sKey_dateRange: [startDate, endDate],
          sKey_branch,
          sKey_campus,
          sKey_createUser,
          pageCurrent,
          pageSize,
          sKey_status: status,
        } = state[storeName];
        return {
          startDate,
          endDate,
          branch: sKey_branch,
          blCampusId: sKey_campus,
          createUser: sKey_createUser,
          rows: pageSize,
          page: pageCurrent,
          status,
        };
      });

      const rsp = yield listImportRecord(params, true).catch(errorsHandle());

      if (rsp && rsp.resultCode === 0) {
        yield put({
          type: 'updateList',
          payload: {
            list: rsp.datas,
            total: rsp.rowCount,
          },
        });
      }

      runFunc(callback);
    },
    *resetSKey(action, { select, put }) {
      const sKeyStorage = yield select((state) => state[storeName].sKeyStorage);
      yield put({
        type: 'updateState',
        payload: {
          ...clone(defaultState.sKey),
          ...sKeyStorage,
        },
      });
    },
    *exportExcel({ payload: { params }, callback }, { select, put, call }) {
      const rsp = yield exportFailingData({ ...params }).catch(errorsHandle());
      if (rsp && rsp.resultCode === 0) {
        yield put({ type: 'main/exportNotice' });
      }
      if (callback) callback();
    },
  },
};
