import React from 'react';
import { connect } from 'dva';
import Icon from 'COMPONENT/Icon';
import Button from 'COMPONENT/Button';
import { Icon as AntdIcon } from 'antd';
import Page from 'COMPONENT/Page';
import List from 'COMPONENT/List';
import { Tooltip } from 'antd';
import SearchBar from 'COMPONENT/SearchBar';
import Input from 'COMPONENT/Input';
import Select from 'COMPONENT/Select';
import MultiSelect from 'COMPONENT/MultiSelect';
import Modal from 'COMPONENT/Modal';
import PreviewImgs from 'COMPONENT/PreviewImgs';
import download from 'COMPONENT/utils/file/downloadFile';
import ShowMessage from 'COMPONENT/utils/globalTips/showMessage';
import EditScoreModal from './editScore';
import { loadCommonDict, loadBranchDict, loadCampusDict, getDictTextByVal, loadBranchCampusDict } from 'UTIL/dict';
import Loaders from './loader';
import styles from './styles.less';
import moment from 'moment';
import { RenderMultiLine } from './renderMultiLine';

const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;

const campusTempDict = [{ text: '请先选择分公司', val: '', disabled: true }];

const classTypeTempDict = [{ text: '请先选择产品类型', val: '', disabled: true }];

const originalPlatformDict = [
  { text: '教学平台', val: 'EDU_PLATFORM' },
  { text: 'Boss平台', val: 'BOSS_PLATFORM' },
];

class ScoreManager extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: true, // boolean 页面加载状态
      //查询条件
      studentName: null,
      contact: null,
      studentId: null,
      branchId: null,
      campusId: [],
      typeId: null,
      paperName: null,
      productId: null,
      yearId: null,
      quarterId: null,
      gradeId: null,
      subjectId: null,
      classTypeId: null,
      classLevelId: null,
      originalPlatform: null,
      //列表数据
      listLoading: false,
      rows: [],
      currentPage: 1,
      pageSize: 20,
      totalRecords: null,
      //数据字典数据
      branchDict: null,
      branchCampusDict: null,
      campusDict: null,
      examTypeDict: null,
      productTypeDict: [
        { val: 'SMALL_CLASS', text: '小班' },
        { val: 'TWO_TEACHER', text: '双师' },
      ],
      paperYearDict: null,
      paperQuarterDict: null,
      paperGradeDict: null,
      paperSubjectDict: null,
      classTypeDict: null,
      classLevelDict: null,
      //其他
      addScoreShow: false,
      editScoreShow: false,
      editScoreId: null,
      editStudentName: null,
      imgShow: false,
      attachmentUrl: [],
      exportStatusName: '导出Excel',
    };
    this.limitClickTag = 1;
    this.exportExcelParams = null;
    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.initSKey = this.initSKey.bind(this);
  }

  componentDidMount() {
    this.init();
  }
  /**
   * 更新状态
   * @param  {object} changedState 改变的状态
   * @return {Promise}
   */
  updateState = (changedState) => {
    const newState = {
      ...this.state,
      ...changedState,
    };
    return new Promise((resolve) => {
      this.setState(newState, () => {
        resolve();
      });
    });
  };
  /**
   * 获取列表数据
   */
  loadListData = async (params) => {
    const {
      studentName,
      contact,
      studentId,
      branchId,
      campusId,
      typeId,
      paperName,
      productId,
      yearId,
      quarterId,
      gradeId,
      subjectId,
      classTypeId,
      pageSize,
      currentPage,
      classLevelId,
      originalPlatform,
    } = this.state;
    let campusLen = campusId ? campusId.length : 0;
    let campusVal = campusLen > 0 ? (campusLen === 1 ? campusId[0] : campusId.join(',')) : null;
    const apiParams = {
      campusId: campusVal,
      studentName,
      contact,
      studentId,
      branchId,
      paperName,
      productId,
      classTypeId,
      rows: pageSize,
      page: currentPage,
      isMatch: 3,
      yearId,
      quarterId,
      gradeId,
      subjectId,
      typeId,
      classlevelId: classLevelId,
      originalPlatform,
      ...params,
    };
    this.exportExcelParams = apiParams;
    return Loaders.loadList(apiParams);
  };
  /**
   * 获取字典
   */
  getSelectDict = (dictType) => {
    switch (dictType) {
      case 'branches':
        return this.state.branchDict;
      case 'branchCampusDict':
        return this.state.branchCampusDict;
      case 'campuses':
        return this.state.campusDict;
      case 'examTypes':
        return this.state.examTypeDict;
      case 'productTypes':
        return this.state.productTypeDict;
      case 'paperYears':
        return this.state.paperYearDict;
      case 'paperQuarters':
        return this.state.paperQuarterDict;
      case 'paperGrades':
        return this.state.paperGradeDict;
      case 'paperSubjects':
        return this.state.paperSubjectDict;
      case 'scoreGrades':
        return this.state.classTypeDict;
      case 'classType':
        return this.state.classTypeDict;
      default:
        return [];
    }
  };
  /**
   * 初始化内容
   */
  async init() {
    this.setState({ loading: true });
    // 初始化数据字典
    await this.initDict();
    // 初始化筛选栏赋值
    await this.initSKey();
    // 初始化列表
    const rsp = await this.loadListData().catch((err) => {});
    this.setState({
      loading: false,
      rows: rsp.rows,
      totalRecords: rsp.records,
      currentPage: rsp.page,
    });
  }
  initDict() {
    return new Promise((resolve, reject) => {
      Promise.all([
        loadBranchDict(),
        loadCampusDict(),
        loadBranchCampusDict(),
        loadCommonDict('BRANCH_EXAM_TYPE'),
        loadCommonDict('PRODUCT_VERSION'),
        loadCommonDict('PRODUCT_QUARTER'),
        loadCommonDict('STUDENT_GRADE'),
        loadCommonDict('SUBJECT'),
      ])
        .then(
          ([
            branchDict,
            campusDict,
            branchCampusDict,
            examTypeDict,
            paperYearDict,
            paperQuarterDict,
            paperGradeDict,
            paperSubjectDict,
          ]) => {
            this.setState(
              {
                branchDict,
                campusDict,
                branchCampusDict,
                examTypeDict,
                paperYearDict,
                paperQuarterDict,
                paperGradeDict,
                paperSubjectDict,
              },
              () => {
                resolve();
              },
            );
          },
        )
        .catch((err) => {
          ShowMessage(err.text, 'error');
          reject();
        });
    });
  }
  initSKey() {
    const { uBranchId, uCampusId } = this.props;
    const { branchDict, branchCampusDict } = this.state;
    let isGroup = uBranchId === uCampusId;
    let branchVal = isGroup ? branchDict[0].val : uBranchId;
    let campusVal = isGroup ? [branchCampusDict.find((item) => item.val === branchVal).options[0].val] : [uCampusId];
    return new Promise((resolve, reject) => {
      this.setState(
        {
          branchId: branchVal,
          campusId: campusVal,
        },
        () => {
          resolve();
        },
      );
    });
  }

  /**
   * 查询列表
   */
  search = async () => {
    await this.updateState({ listLoading: true, currentPage: 1 });
    const list = await this.loadListData();
    this.updateState({
      listLoading: false,
      rows: list.rows,
      totalRecords: list.records,
      currentPage: list.page,
    });
  };
  /**
   * 重置搜索表单
   */
  reset = async () => {
    const { uBranchId, uCampusId } = this.props;
    const { branchDict, branchCampusDict } = this.state;
    let isGroup = uBranchId === uCampusId;
    let branchVal = isGroup ? branchDict[0].val : uBranchId;
    let campusVal = isGroup ? [branchCampusDict.find((item) => item.val === branchVal).options[0].val] : [uCampusId];
    this.updateState({
      studentName: null,
      contact: null,
      studentId: null,
      branchId: branchVal,
      campusId: campusVal,
      typeId: null,
      paperName: null,
      productId: null,
      yearId: null,
      quarterId: null,
      gradeId: null,
      subjectId: null,
      classLevelId: null,
      classTypeId: null,
      classLevelDict: null,
      classTypeDict: null,
      originalPlatform: null,
    });
  };
  /**
   * 翻页
   */
  pageChange = async ({ current, pageSize }) => {
    await this.updateState({ listLoading: true, pageSize, currentPage: current });
    const list = await this.loadListData();
    this.updateState({
      listLoading: false,
      rows: list.rows,
      totalRecords: list.records,
      currentPage: list.page,
    });
  };
  /**
   * 导入成绩
   */
  importScore = async () => {
    const { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: {
        path: '/TutorialScoreManager/Import',
        params: {},
        refresh: true,
      },
    });
  };
  /**
   * 导出Excel
   */
  exportExcel = async () => {
    if (this.limitClickTag === 1) {
      //导出文件名字
      const { branchId, campusId, yearId, quarterId, typeId } = this.exportExcelParams;
      const { branchDict, campusDict, paperYearDict, paperQuarterDict, examTypeDict } = this.state;
      const exportDate = new Date();
      const conditionStr =
        (getDictTextByVal(branchDict, branchId) || '') + //分公司名字
        (getDictTextByVal(campusDict, campusId) || '') + //学习中心名字
        (getDictTextByVal(paperYearDict, yearId) || '') + //年份
        (getDictTextByVal(paperQuarterDict, quarterId) || '') + //季度
        (getDictTextByVal(examTypeDict, typeId) || ''); //考试类型
      const dateStr = exportDate.getFullYear() + '-' + (exportDate.getMonth() + 1) + '-' + exportDate.getDate(); //日期
      const exportName = (conditionStr ? conditionStr + '成绩' : '课外辅导成绩') + dateStr; //最终文件名
      //调用导出接口
      this.limitClickTag = 2;
      this.setState({ exportStatusName: '导出中...' });
      Loaders.exportExcel(this.exportExcelParams, exportName).then(
        () => {
          this.limitClickTag = 1;
          this.setState({ exportStatusName: '导出excel' });
        },
        (err) => {
          if (err.resultMessage) {
            ShowMessage(err.resultMessage);
          } else {
            ShowMessage('服务器错误');
          }
          this.limitClickTag = 1;
          this.setState({ exportStatusName: '导出excel' });
        },
      );
    }
  };
  /**
   * 查看试卷得分图片
   */
  handleShowImg = (attachmentUrl) => {
    this.updateState({
      imgShow: true,
      attachmentUrl: attachmentUrl.split(','),
    });
  };
  /**
   * 分公司改变获取对应的学习中心字典
   */
  handleBranchChange = (branchId) => {
    this.setState({
      branchId,
      campusId: [],
    });
  };
  /**
   * 关闭模态框的回调
   */
  handleModalOk = (type) => {
    if (type === 'edit') {
      this.updateState({
        editScoreShow: false,
        editScoreId: null,
        editStudentName: null,
      });
    } else {
      this.updateState({
        addScoreShow: false,
        editScoreId: null,
        editStudentName: null,
      });
    }
    this.search();
  };
  /**
   * 渲染试卷得分项
   */
  renderPaperScore = (data, rowData, index) => {
    const { attachmentUrl } = rowData;
    if (attachmentUrl) {
      return (
        <span style={{ display: 'flex', alignItems: 'center' }}>
          {data}
          <span style={{ display: 'flex' }} onClick={() => this.handleShowImg(attachmentUrl)}>
            <Icon type="image_mc" size={18} className={styles.imgIcon} />
          </span>
        </span>
      );
    } else {
      return data;
    }
  };
  /**
   * 判断是否有导出excel权限
   */
  hasExportAuth = () => {
    const { btnAuths } = this.props;
    return btnAuths.includes('BTN_TUSCORE_EXPORT');
  };
  /**
   * 判断是否有新增权限
   */
  hasAddAuth = () => {
    const { btnAuths } = this.props;
    return btnAuths.includes('BTN_TUSCORE_ADD');
  };
  /**
   * 判断是否有编辑权限
   */
  hasEditAuth = () => {
    const { btnAuths } = this.props;
    return btnAuths.includes('BTN_TUSCORE_EDIT');
  };
  /**
   * 渲染得分等级项
   */
  renderScoreGrade = (data, rowData, index) => {
    const courseInfoVos = rowData.courseInfoVos;
    let HoverIcon;
    if ((courseInfoVos || []).length > 0) {
      const content = courseInfoVos.map((info, index) => {
        return (
          <div key={index}>
            <p>{info.courseDate}</p>
            <p>{'报读' + info.courseName}</p>
            <p>{'(' + info.organizationName + '-' + info.teacherName + ')'}</p>
          </div>
        );
      });
      HoverIcon = (
        <Tooltip placement="top" title={content}>
          <AntdIcon type="info-circle" className={styles.infoIcon} />
        </Tooltip>
      );
    }
    return (
      <div>
        {courseInfoVos && HoverIcon}
        {rowData.classlevelName}
      </div>
    );
  };
  /**
   * 渲染得分率
   */
  renderScoreRate = (data, record, index) => {
    const score = parseFloat(record.score);
    const totalPoint = parseFloat(record.paperVo.totalPoint);
    if (score) {
      return ((score / totalPoint) * 100).toFixed(1) + '%';
    } else {
      return '自动匹配';
    }
  };
  /**
   * 渲染人数或排名
   */
  renderAmountOrRanking = (data) => {
    return data === null ? '无' : data;
  };
  /**
   * 表头
   */
  columns = [
    { title: '学生姓名', dataIndex: 'studentName', width: COL_WIDTH_M },
    { title: '学生编号', dataIndex: 'studentId', width: COL_WIDTH_M },
    { title: '考试分公司', dataIndex: 'branchName', width: COL_WIDTH_M },
    { title: '考试学习中心', dataIndex: 'campusName', width: COL_WIDTH_M },
    { title: '考试类型', dataIndex: 'paperVo.typeName', width: COL_WIDTH_M },
    { title: '产品类型', dataIndex: 'paperVo.productName', width: COL_WIDTH_M },
    { title: '试卷年份', dataIndex: 'paperVo.yearName', width: COL_WIDTH_M },
    {
      title: '试卷季度',
      dataIndex: 'paperVo.quarterName',
      width: COL_WIDTH_M,
      render: (val, record) => {
        return <RenderMultiLine detail={val} maxLine={3} />;
      },
    },
    { title: '试卷年级', dataIndex: 'paperVo.gradeName', width: COL_WIDTH_M },
    { title: '试卷科目', dataIndex: 'paperVo.subjectName', width: COL_WIDTH_M },
    { title: '试卷得分', dataIndex: 'score', width: COL_WIDTH_M, render: this.renderPaperScore },
    { title: '试卷总分', dataIndex: 'paperVo.totalPoint', width: COL_WIDTH_M },
    { title: '试卷得分率', dataIndex: 'scoreRate', width: COL_WIDTH_M, render: this.renderScoreRate },
    { title: '得分等级', dataIndex: 'classlevelName', width: COL_WIDTH_M, render: this.renderScoreGrade },
    // { title: '年级排名', dataIndex: 'gradeRanking', width: COL_WIDTH_M, render: this.renderAmountOrRanking },
    // { title: '年级人数', dataIndex: 'gradePeopleAmount', width: COL_WIDTH_M, render: this.renderAmountOrRanking },
    // { title: '班级排名', dataIndex: 'classRanking', width: COL_WIDTH_M, render: this.renderAmountOrRanking },
    // { title: '班级人数', dataIndex: 'classPeopleAmount', width: COL_WIDTH_M, render: this.renderAmountOrRanking },
    {
      title: '创建时间',
      dataIndex: 'createTime',
      width: COL_WIDTH_M,
      render: (value, row, index) => {
        return moment(value).format('YYYY-MM-DD HH:mm:ss');
      },
    },
    { title: '试卷名称', dataIndex: 'paperVo.paperName', width: COL_WIDTH_XL },
    { title: '考试名称', dataIndex: 'eduExamName', width: COL_WIDTH_L },
    { title: '成绩来源', dataIndex: 'originalPlatform', width: COL_WIDTH_M, dict: originalPlatformDict },
    {
      title: '操作',
      dataIndex: 'originalPlatform',
      key: 'action',
      width: COL_WIDTH_M,
      fixed: 'right',
      getActionConfig: (text, record, index) => {
        const { id, studentName } = record;
        const ret = [];
        if (this.hasEditAuth()) {
          ret.push({
            text: '编辑',
            onClick: () => {
              if (text === 'EDU_PLATFORM') {
                ShowMessage('该成绩由教学平台创建，请移步教学平台修改');
                return;
              }
              this.updateState({ editScoreShow: true, editScoreId: id, editStudentName: studentName });
            },
          });
        }
        return ret;
      },
    },
  ];

  render() {
    const {
      loading,
      searchExpand,
      listLoading,
      rows,
      currentPage,
      pageSize,
      totalRecords,
      studentName,
      contact,
      studentId,
      branchId,
      campusId,
      typeId,
      paperName,
      productId,
      yearId,
      quarterId,
      gradeId,
      subjectId,
      classTypeId,
      exportStatusName,
      branchCampusDict,
      addScoreShow,
      editScoreShow,
      editScoreId,
      imgShow,
      attachmentUrl,
      editStudentName,
      classTypeDict,
      classLevelDict,
      classLevelId,
      originalPlatform,
    } = this.state;
    const hasAuth = this.hasExportAuth();
    return (
      <Page
        fitWindowHeight={true}
        loading={loading}
        route={this.props.route}
        subHeader={
          <div className={styles.searchBar}>
            {this.hasAddAuth() && (
              <Button className={styles.addScore} onClick={() => this.updateState({ addScoreShow: true })}>
                新增
              </Button>
            )}
            {this.props.importAuth && (
              <Button className={hasAuth ? styles.importScores : styles.exportScores} onClick={this.importScore}>
                导入成绩
              </Button>
            )}
            {hasAuth && (
              <Button className={styles.exportScores} disabled={this.limitClickTag !== 1} onClick={this.exportExcel}>
                {exportStatusName}
              </Button>
            )}
            <SearchBar
              onSearch={this.search}
              onReset={this.reset}
              expand={true}
              onToggleExpand={(expand) => this.setState({ searchExpand: expand })}
            >
              <Input
                value={studentName}
                placeholder="学生姓名"
                onChange={(e) => this.setState({ studentName: e.target.value })}
              />
              <Input
                value={contact}
                placeholder="客户号码"
                onChange={(e) => this.setState({ contact: e.target.value })}
              />
              <Input
                value={studentId}
                placeholder="学生ID"
                onChange={(e) => this.setState({ studentId: e.target.value })}
              />
              <Select
                placeholder="考试分公司"
                dict={this.getSelectDict('branches')}
                value={branchId}
                allowClear={false}
                showSearch={true}
                onChange={this.handleBranchChange}
              />
              <MultiSelect
                placeholder="考试学习中心"
                dict={
                  branchId
                    ? branchCampusDict && branchCampusDict.filter((item) => item.val === branchId)
                    : branchCampusDict
                }
                value={campusId}
                showSearch={true}
                onChange={(campusId) => this.updateState({ campusId: campusId })}
              />
              <Select
                placeholder="考试类型"
                dict={this.getSelectDict('examTypes')}
                value={typeId}
                allowClear={true}
                onChange={(typeId) => this.updateState({ typeId: typeId })}
              />
              <Input
                value={paperName}
                placeholder="试卷名称"
                onChange={(e) => this.setState({ paperName: e.target.value })}
              />
              <Select
                placeholder="产品类型"
                dict={this.getSelectDict('productTypes')}
                value={productId}
                allowClear={true}
                onChange={async (productId) => {
                  this.setState({ productId, classTypeId: null, classLevelDict: null, classTypeDict: null });
                  if (productId) {
                    const classTypeDict = await loadCommonDict('CLASS_TYPE', productId);
                    //得分等级还有一个无效项
                    // classTypeDict.push({text: '无效', val: 'invalid'})
                    // //还有一个未激活
                    // classTypeDict.push({text: '未激活', val: 'inactive'})
                    this.setState({
                      classTypeDict,
                    });
                  }
                }}
              />
              <Select
                placeholder="试卷年份"
                dict={this.getSelectDict('paperYears')}
                value={yearId}
                allowClear={true}
                onChange={(yearId) => this.updateState({ yearId: yearId })}
              />
              <Select
                placeholder="试卷季度"
                dict={this.getSelectDict('paperQuarters')}
                value={quarterId}
                allowClear={true}
                onChange={(quarterId) => this.updateState({ quarterId: quarterId })}
              />
              <Select
                placeholder="试卷年级"
                dict={this.getSelectDict('paperGrades')}
                value={gradeId}
                allowClear={true}
                onChange={(gradeId) => this.updateState({ gradeId: gradeId })}
              />
              <Select
                placeholder="试卷科目"
                dict={this.getSelectDict('paperSubjects')}
                value={subjectId}
                allowClear={true}
                onChange={(subjectId) => this.updateState({ subjectId: subjectId })}
              />
              <Select
                placeholder="班级类型"
                dict={this.getSelectDict('classType')}
                noOptionTips={!productId ? '请先选择产品类型' : undefined}
                value={classTypeId}
                allowClear={true}
                onChange={async (classTypeId) => {
                  this.setState({ classTypeId, classLevelId: null, classLevelDict: null });
                  if (classTypeId) {
                    const classLevelDict = await loadCommonDict('CLASS_LEVEL', classTypeId);
                    this.setState({
                      classLevelDict,
                    });
                  }
                }}
              />
              <Select
                placeholder="得分等级"
                dict={classLevelDict}
                value={classLevelId}
                noOptionTips={!classTypeId ? '请先选择班级类型' : undefined}
                allowClear={true}
                onChange={(classLevelId) => this.updateState({ classLevelId: classLevelId })}
              />
              <Select
                placeholder="来源"
                dict={originalPlatformDict}
                value={originalPlatform}
                onChange={(originalPlatform) => this.updateState({ originalPlatform: originalPlatform })}
              />
            </SearchBar>
          </div>
        }
      >
        <List
          columns={this.columns}
          loading={listLoading}
          dataSource={rows}
          pagination={{
            current: currentPage,
            pageSize,
            total: totalRecords || 0,
          }}
          onChange={this.pageChange}
        />
        {addScoreShow && (
          <EditScoreModal
            onOk={() => this.handleModalOk('add')}
            onCancel={() => {
              this.updateState({ addScoreShow: false, editScoreId: null, editStudentName: null });
            }}
          />
        )}
        {editScoreShow && (
          <EditScoreModal
            editScoreId={editScoreId}
            editStudentName={editStudentName}
            onOk={() => this.handleModalOk('edit')}
            onCancel={() => {
              this.updateState({ editScoreShow: false, editScoreId: null, editStudentName: null });
            }}
          />
        )}
        {imgShow && (
          <Modal
            width={800}
            okBtnShow={false}
            cancelBtnShow={false}
            closable={true}
            onClose={() => this.updateState({ imgShow: false })}
          >
            <PreviewImgs list={attachmentUrl} activeSrc={attachmentUrl[0]} showThumbnail={false} />
          </Modal>
        )}
      </Page>
    );
  }
}

const mapStateToProp = (state) => {
  const { brenchId, campusId } = state.account.user;
  const btnAuths = state.account.btnAuths;

  return {
    btnAuths,
    uBranchId: brenchId,
    uCampusId: campusId,
    importAuth: btnAuths.includes('BTN_TUSCORE_IMPORT'),
  };
};

export default connect(mapStateToProp)(ScoreManager);
