import request from 'UTIL/http/fetch';
import {
  loadScoreListApi,
  importScoreExcelApi,
  exportDemoApi,
  loadScoreDetailApi,
  loadPaperDictApi,
  loadPaperDetailApi,
  addScoreBatchApi,
  addScoreApi,
  editScoreApi,
  delScoreApi,
  loadContactApi,
  loadStudentApi,
  delScoreApiGroup,
} from 'SERVICE/enrollManager';
import { urlBase } from 'CONST/config';
import { OSS_BASE_PREFIX } from 'CONST/ossConfig';

const makeQueryString = (params) => {
  let queryString = '';
  if (params) {
    Object.keys(params).forEach((key, i) => {
      const value = params[key];
      if (i > 0) {
        queryString += '&';
      }
      queryString += `${key}=${value === null || value == undefined ? '' : value}`;
    });
  }
  return queryString;
};
/**
 * 获取列表数据
 */
const loadList = async (params) => {
  return loadScoreListApi(params);
};
/**
 * 获取详情
 */
const loadDetail = async (params) => {
  const response = await loadScoreDetailApi(params);
  // const { examType: typeName, productType: productName, paperYear: yearName, paperName,
  //   paperQuarter: quarterName, paperGrade: gradeName,paperSubject: subjectName,
  //   paperTotalScore: totalPoint} = response.data.paperVo
  const { typeName, productName, yearName, paperName, quarterName, gradeName, subjectName, totalPoint } =
    response.data.paperVo;
  return {
    typeName,
    productName,
    yearName,
    paperName,
    quarterName,
    gradeName,
    subjectName,
    totalPoint,
    paperDict: [{ text: paperName, val: response.data.paperId }],
    ...response.data,
  };
  // return new Promise((resolve, reject)=>{
  //   setTimeout(()=>{
  //     resolve({
  //       customerPhoneId: '324313', studentNameId: '324313',
  //       branchId: '000002', campusId: 'ff808081482bb63001482c1a207a0003', paperId: '324313',
  //       paperScore: 110, examType: '期末考', 	productType: '十人课堂',
  //       paperYear: '2018', paperQuarter: '春季',paperGrade: '五年级',
  //       paperSubject: '数学', paperTotalScore: 150, classType: '尖子班',
  //     })
  //   }, 1000)
  // })
};
/**
 * 新增
 */
const add = async (params) => {
  // const data = await request('/EnrollManagerStudentQualificationController/addEnrollManagerStudentQualification.do', {
  //   body: params,
  //   body: makeQueryString(params),
  //   headers: {
  //     "Content-Type": "application/x-www-form-urlencoded"
  //   }
  // })
  // return data.data
  return addScoreApi(params);
  // return new Promise((resolve, reject) => {
  //   setTimeout(()=>{
  //     resolve()
  //   }, 1000)
  // })
};
/**
 * 编辑
 */
const edit = async (params) => {
  return editScoreApi(params);
  // return new Promise((resolve, reject) => {
  //   setTimeout(()=>{
  //     resolve()
  //   }, 1000)
  // })
};
/**
 * 删除
 */
const del = async (params) => {
  return delScoreApi(params);
  // return new Promise((resolve, reject) => {
  //   setTimeout(()=>{
  //     resolve()
  //   }, 1000)
  // })
};
// 批量删除
const delGroup = async (params) => {
  return delScoreApiGroup(params);
  // return new Promise((resolve, reject) => {
  //   setTimeout(()=>{
  //     resolve()
  //   }, 1000)
  // })
};
/**
 * 导出
 */
const exportExcel = async (params, exportName) => {
  const query = makeQueryString(params);
  return new Promise((resolve, reject) => {
    var httpRequest;
    if (window.XMLHttpRequest) {
      httpRequest = new XMLHttpRequest();
    } else if (window.ActiveXObject) {
      // 兼容ie6及之前的版本
      httpRequest = new ActiveXObject('Microsoft.XMLHTTP');
    }
    httpRequest.open('GET', urlBase + 'AchievementMgrController/batchExport.do?' + query, true);
    httpRequest.responseType = 'blob';
    httpRequest.onload = function () {
      if (this.status === 200) {
        var blob = this.response;
        var reader = new FileReader();
        reader.readAsDataURL(blob);
        reader.onload = function (ev) {
          //读入完成事件
          resolve();
          var a = document.createElement('a');
          a.href = window.URL.createObjectURL(httpRequest.response); //读取结果
          a.download = exportName + '.xls';
          document.body.appendChild(a);
          a.click();
          a.remove();
        };
      } else {
        const reader = new FileReader();
        reader.addEventListener('loadend', (e) => {
          const text = e.srcElement.result;
          reject(JSON.parse(text));
        });
        reader.readAsText(this.response);
      }
    };
    httpRequest.onerror = function (err) {
      reject(err);
    };
    httpRequest.send();
  });
};
/**
 * 导入
 */
const importExcel = async (params) => {
  return importScoreExcelApi(params);
  // return new Promise((resolve, reject)=>{
  //   setTimeout(()=>{
  //     resolve()
  //   }, 1000)
  // })
};
/**
 * 导出模板
 */
const exportDemo = async (params) => {
  const link = document.createElement('a');
  const body = document.querySelector('body');
  link.href = OSS_BASE_PREFIX + '/%E6%88%90%E7%BB%A9%E7%AE%A1%E7%90%86%E5%AF%BC%E5%85%A5%E6%A8%A1%E6%9D%BF.xls';
  link.download = '成绩管理导出模板.xls';
  // fix Firefox
  link.style.display = 'none';
  body.appendChild(link);
  link.click();
  body.removeChild(link);
};
/**
 * 获取客户号码
 */
const loadContact = async (params) => {
  const response = await loadContactApi(params);
  if (response) {
    if (response.data) {
      return response.data;
    } else {
      return [];
    }
  } else {
    return [];
  }
  // return new Promise(function(resolve){
  // 	setTimeout(function(){
  // 		resolve([
  // 			{ val: '12313', text: '14141234134'},
  // 			{ val: '324313', text: '254523451435'},
  // 		])
  // 	}, 1000)
  // })
};
/**
 * 获取学生数据字典
 */
const loadStudentDict = async (params) => {
  const response = await loadStudentApi(params);
  if (response) {
    return response.studentVos.map((dict) => {
      return {
        text: dict.name,
        val: dict.id,
      };
    });
  } else {
    return [];
  }
  // return new Promise(function(resolve){
  // 	setTimeout(function(){
  // 		resolve([
  // 			{ val: '12313', text: '14141234134'},
  // 			{ val: '324313', text: '254523451435'},
  // 		])
  // 	}, 1000)
  // })
};
const batch = async (params) => {
  return addScoreBatchApi(params);
  // return new Promise(function(resolve){
  // 	setTimeout(function(){
  // 		resolve([
  // 			{ val: '12313', text: '14141234134'},
  // 			{ val: '324313', text: '254523451435'},
  // 		])
  // 	}, 1000)
  // })
};
const loadPaperDict = async (params) => {
  const response = await loadPaperDictApi(params);
  if (response) {
    return response.map((paper) => {
      return {
        text: paper.paperName,
        val: paper.id,
        title: paper.paperName,
      };
    });
  } else {
    return [];
  }
  // return new Promise(function(resolve){
  // 	setTimeout(function(){
  // 		resolve([
  // 			{ val: '12313', text: '14141234134'},
  // 			{ val: '324313', text: '254523451435'},
  // 		])
  // 	}, 1000)
  // })
};
const loadPaperDetail = async (params) => {
  const response = await loadPaperDetailApi(params);
  if (response) {
    return response[0];
  } else {
    return {};
  }
  // return new Promise(function(resolve){
  // 	setTimeout(function(){
  // 		resolve({paperYear: '2018', paperQuarter: '春季',paperGrade: '六年级',
  // 		paperSubject: '语文', paperTotalScore: '150', classType: '尖子班'})
  // 	}, 1000)
  // })
};
export default {
  loadList,
  loadDetail,
  add,
  edit,
  del,
  delGroup,
  exportExcel,
  exportDemo,
  importExcel,
  batch,
  loadPaperDict,
  loadPaperDetail,
  loadContact,
  loadStudentDict,
};
