import React from 'react';
import { connect } from 'dva';

import { Rate, DatePicker, InputNumber } from 'antd';
import { Page, Select, TextArea } from 'COMPONENT';

import moment from 'moment';

import Required from '../components/Required';
import InfoForm from '../components/InfoForm';
import InfoBlock from '../components/InfoBlock';
import InfoItem from '../components/InfoForm/InfoItem';

import {
  getSubjectAlias,
  getSelectOption,
  addStudentAchievementTemplate,
  listUserAllBranchByResourceId,
  getStudentAchievementTemplateDetail,
} from 'SERVICE/scoreOrderTemplateManagement';

import styles from './index.less';

import showMessage from 'COMPONENT/utils/globalTips/showMessage';

import _ from 'lodash';

const { RangePicker } = DatePicker;
const dateFormat = 'YYYY-MM-DD';
const KEYS = [
  'level',
  'branchId',
  'knowledge',
  'studentNum',
  'nextPredict',
  'schoolYear',
  'semester',
  'examinationType',
  'gradeId',
  'subjectId',
  'subjectAnalysis',
  'examinationDateStart',
  'examinationDateEnd',
];

const formatDict = (dict) => {
  const result = Object.keys(dict).map((val) => ({ text: dict[val], val }));
  return [...result.filter(({ val }) => val)];
};

class AddPage extends React.Component {
  static defaultProps = {};

  state = {
    loading: false,

    yearDict: [],
    gradeDict: [],
    branchDict: [],
    subjectDict: [],
    semesterDict: [],
    examinationTypeDict: [],
  };

  constructor(props) {
    super(props);
  }

  componentWillMount() {
    this.initContent();
  }

  /**
   * 录入分数默认满分
   */
  initContent = () => {
    this.initDict();

    // 若有传入id，则是编辑
    if (this.props.currentId != null) {
      this.setState({ loading: true });
      getStudentAchievementTemplateDetail({ id: this.props.currentId })
        .then((res) => {
          const initValues = _.pick(res.data, KEYS);

          if (initValues.level != null) {
            initValues.level = initValues.level / 2;
          }

          this.loadSubjectAlias(
            {
              orgId: initValues.branchId,
              gradeId: initValues.gradeId,
            },
            () => {
              this.setState({ ...initValues });
            },
          );
        })
        .catch((e) => {
          showMessage(e?.text, 'error');
        })
        .finally(() => {
          this.setState({ loading: false });
        });
    }
  };

  initDict = () => {
    Promise.all([
      listUserAllBranchByResourceId({ resourceId: 'scoreOrderTemplateManagement' }),
      getSelectOption({ selectOptionCategory: 'SCHOOL_YEAR' }),
      getSelectOption({ selectOptionCategory: 'Semester' }),
      getSelectOption({ selectOptionCategory: 'EXAMINATION_TYPE' }),
      getSelectOption({ selectOptionCategory: 'STUDENT_GRADE' }),
    ]).then((result) => {
      let [
        { data: branchDict },
        { value: yearDict },
        { value: semesterDict },
        { value: examinationTypeDict },
        { value: gradeDict },
      ] = result;
      const examinationTypeDictList = formatDict(examinationTypeDict);
      this.setState({
        yearDict: formatDict(yearDict),
        gradeDict: formatDict(gradeDict),
        semesterDict: formatDict(semesterDict),
        examinationTypeDict: examinationTypeDictList,
        branchDict: branchDict.map(({ id, name }) => ({ text: name, val: id })),
        examinationType: examinationTypeDictList.find((item) => item.text === '月考')?.val,
      });
    });
  };

  /**
   * 选中学生
   *
   * @param student 学生信息
   */
  onStudentSelect = (student) => {
    this.setState({ studentInfo: student });
  };

  /**
   * 修改分数
   *
   * @param questionList 题目列表
   */
  onScoreChange = (questionList) => {
    this.setState({ questionList });
  };

  /**
   * 选择文件
   *
   * @param fileList 文件列表
   */
  onPaperChange = (fileList) => {
    this.setState({ fileList });
  };

  loadSubjectAlias = (params, callback) => {
    if (params.orgId && params.gradeId) {
      getSubjectAlias(params)
        .then((res) => {
          this.setState({
            subjectDict: res?.data.map((item) => ({ val: item.value, text: item.aliasName || item.name })),
          });
        })
        .catch(() => {
          this.setState({ subjectDict: [] });
        })
        .finally(() => {
          this.setState({ subjectId: null }, () => {
            callback && callback();
          });
        });
    } else {
      this.setState({ subjectDict: [], subjectId: null });
    }
  };

  handleLoadSubjectByBranchId = (orgId) => {
    const params = {
      orgId,
      gradeId: this.state.gradeId,
    };
    this.loadSubjectAlias(params);
  };

  handleLoadSubjectByGradeId = (gradeId) => {
    const params = {
      gradeId,
      orgId: this.state.branchId,
    };
    this.loadSubjectAlias(params);
  };

  /**
   * 新增
   */
  handleOk = () => {
    const params = _.pick(this.state, KEYS);

    this.setState({ loading: true });
    if (params.examinationDateEnd && moment.isMoment(params.examinationDateEnd)) {
      params.examinationDateEnd = params.examinationDateEnd.format('YYYY-MM-DD');
    }
    if (params.examinationDateStart && moment.isMoment(params.examinationDateStart)) {
      params.examinationDateStart = params.examinationDateStart.format('YYYY-MM-DD');
    }

    // 半星
    if (params.level) {
      params.level = params.level * 2;
    }

    if (this.props.currentId != null) {
      params.id = this.props.currentId;
    }

    params.studentNum = params.studentNum || 0;

    addStudentAchievementTemplate(params)
      .then(() => {
        showMessage('提交成功', 'success');
        setTimeout(() => this.props.onSaveSuccess(), 1500);
      })
      .catch((e) => {
        showMessage(e?.text || '提交失败，请稍后重试', 'error');
      })
      .finally(() => {
        this.setState({ loading: false });
      });
  };

  handleChangeDate = (date) => {
    const [examinationDateStart, examinationDateEnd] = date;
    this.setState({ examinationDateStart, examinationDateEnd });
  };

  render() {
    const {
      level,
      branchId,
      knowledge,
      studentNum,
      nextPredict,
      schoolYear,
      semester,
      examinationType,
      gradeId,
      subjectId,
      subjectAnalysis,
      examinationDateStart,
      examinationDateEnd,
    } = this.state;

    return (
      <div className={styles.wrap}>
        <Page
          name={this.props.pageTitle}
          footer={true}
          loading={this.state.loading}
          route={this.props.route}
          onOk={this.handleOk}
        >
          <InfoForm>
            <InfoBlock title="模板属性">
              <div className={styles.contentWrapper}>
                <InfoItem label={<Required>所属分公司</Required>} span={8}>
                  <Select
                    placeholder="请选择所属分公司"
                    dict={this.state.branchDict}
                    showSearch={true}
                    onSelect={(branchId) => {
                      this.setState({ branchId });
                      this.handleLoadSubjectByBranchId(branchId);
                    }}
                    value={branchId}
                  />
                </InfoItem>
                <InfoItem label={<Required>学年</Required>} span={8}>
                  <Select
                    placeholder="请选择学年"
                    dict={this.state.yearDict}
                    showSearch={true}
                    onSelect={(schoolYear) => this.setState({ schoolYear })}
                    value={schoolYear}
                  />
                </InfoItem>
                <InfoItem label={<Required>学期</Required>} span={8}>
                  <Select
                    placeholder="请选择学期"
                    dict={this.state.semesterDict}
                    showSearch={true}
                    onSelect={(semester) => this.setState({ semester })}
                    value={semester}
                  />
                </InfoItem>
                <InfoItem label={<Required>考试类型</Required>} span={8}>
                  <Select
                    placeholder="请选择考试类型"
                    disabled={true}
                    dict={this.state.examinationTypeDict}
                    showSearch={true}
                    onSelect={(examinationType) => this.setState({ examinationType })}
                    value={examinationType}
                  />
                </InfoItem>
                <InfoItem label={<Required>年级</Required>} span={8}>
                  <Select
                    placeholder="请选择年级"
                    dict={this.state.gradeDict}
                    showSearch={true}
                    onSelect={(gradeId) => {
                      this.setState({ gradeId });
                      this.handleLoadSubjectByGradeId(gradeId);
                    }}
                    value={gradeId}
                  />
                </InfoItem>
                <InfoItem label={<Required>科目</Required>} span={8}>
                  <Select
                    placeholder="请选择科目"
                    dict={this.state.subjectDict}
                    showSearch={true}
                    disabled={!branchId || !gradeId}
                    onSelect={(subjectId) => this.setState({ subjectId })}
                    value={subjectId}
                  />
                </InfoItem>
                <InfoItem label={<Required>考试时间区间</Required>} span={8}>
                  <RangePicker
                    size="large"
                    value={[
                      examinationDateStart ? moment(examinationDateStart, dateFormat) : null,
                      examinationDateEnd ? moment(examinationDateEnd, dateFormat) : null,
                    ]}
                    onChange={this.handleChangeDate}
                  />
                </InfoItem>
              </div>
            </InfoBlock>
            <InfoBlock title="模板内容">
              <div className={styles.contentWrapper}>
                <InfoItem label={<div>考试人数</div>} span={24}>
                  <InputNumber
                    max={1000000}
                    min={0}
                    size="large"
                    precision={0}
                    placeholder="请输入考试人数（仅支持阿拉伯数字）"
                    onChange={(studentNum) => this.setState({ studentNum })}
                    value={studentNum}
                  />
                </InfoItem>
                <InfoItem label={<div>考试难度</div>} span={24}>
                  <Rate allowHalf onChange={(level) => this.setState({ level })} value={level} />
                </InfoItem>
                <InfoItem label={<div>考试知识点</div>} span={24} style={{ marginBottom: 15 }}>
                  <TextArea
                    rows={8}
                    tipElementId="knowledge"
                    maxLength={1000}
                    wordCount={true}
                    value={knowledge}
                    onChange={({ target }) => this.setState({ knowledge: target?.value?.trim() })}
                    placeholder="请填写本场考试中关于当前所选科目的相关知识点"
                  />
                </InfoItem>
                <InfoItem label={<div>科目分析</div>} span={24} style={{ marginBottom: 15 }}>
                  <TextArea
                    rows={8}
                    tipElementId="subjectAnalysis"
                    maxLength={1000}
                    wordCount={true}
                    value={subjectAnalysis}
                    onChange={({ target }) => this.setState({ subjectAnalysis: target?.value?.trim() })}
                    placeholder="请填写本场考试中关于当前所选科目的考试分析"
                  />
                </InfoItem>
                <InfoItem label={<div>下次考试预告</div>} span={24} style={{ marginBottom: 15 }}>
                  <TextArea
                    rows={8}
                    tipElementId="nextPredict"
                    maxLength={1000}
                    wordCount={true}
                    value={nextPredict}
                    onChange={({ target }) => this.setState({ nextPredict: target?.value?.trim() })}
                    placeholder="请填写本场考试中关于当前所选科目的下次考试预告"
                  />
                </InfoItem>
              </div>
            </InfoBlock>
          </InfoForm>
        </Page>
      </div>
    );
  }
}

const mapStateTopProps = (state) => {
  const pageParams = state.main.pageParams || {};
  const data = pageParams['/scoreOrderTemplateManagement/Add'] || {};
  const { title, id, branchId, onSaveSuccess } = data;
  const params = {
    branchId,
    onSaveSuccess,
    currentId: id,
    pageTitle: title || '新增模板',
  };

  return {
    ...params,
  };
};

export default connect(mapStateTopProps)(AddPage);
