/**
 * Created by Haolin<haolinhom@gmail.com> on 2018/6/27.
 */
import React from 'react';
import { connect } from 'dva';
import { Row, Col, Modal } from 'antd';
import Page from 'COMPONENT/Page';
import Block from 'COMPONENT/Block';
import SearchBar from 'COMPONENT/SearchBar';
import List from 'COMPONENT/List';
import Input from 'COMPONENT/Input';
import Select from 'COMPONENT/Select';
import { getExpiredStudentList } from 'SERVICE/system';
import './styles.less';
import { format as formatDate } from '../utils/date';

const CLEAN_STATE_DICT = [
  { val: 1, text: '已清理' },
  { val: 0, text: '未清理' },
];

class CleanExpiredMiniClassDetail extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {Boolean} 内容加载状态
      error: null, // {string} 数据加载失败的错误信息

      studentName: null, // {String} 查询条件_学生名称
      courseName: null, // {String} 查询条件_小班/讲座名称
      productName: null, // {String} 查询条件_产品名称
      cleanState: null, // {String} 查询条件_清理状态
      list: [], // Array} 已清理产品的学生列表
      listLoading: false, // {false} 列表数据加载状态
      totalRecords: 0, // {number} 总记录数
      currentPage: 1, // {number} 当前页码
      pageSize: 20, // {number} 每页记录数
    };
    this.isUnmounted = false;
  }

  componentWillMount() {
    this.initPage();
  }

  componentWillUnmount() {
    this.isUnmounted = true;
  }

  /**
   * 初始化页面
   * */
  initPage = async () => {
    this.loadData();
  };

  /**
   * 更新状态
   * */
  updateState = (newState) => {
    this.setState((prevState, props) => {
      return {
        ...newState,
      };
    });
  };

  /**
   * 加载数据
   * */
  loadData = async (params = {}) => {
    await this.updateState({ listLoading: true, ...params });
    const {
      params: { cleanId },
    } = this.props;
    const { studentName, courseName, productName, cleanState, currentPage, pageSize } = this.state;
    const apiParams = {
      dataId: cleanId,
      stuName: studentName,
      detailName: courseName,
      isClean: cleanState,
      productName,
      page: currentPage,
      rows: pageSize,
    };
    const apiData = await getExpiredStudentList(apiParams);
    const { records, rows } = apiData;
    //拼装数据
    let data = rows.map((item, index, arr) => {
      const { stu_name, detail_name, product_name, clean_money, clean_time, sign_up_date } = item;
      return {
        num: index + 1,
        studentName: stu_name,
        courseName: detail_name,
        productName: product_name,
        contractAmount: clean_money,
        cleanTime: clean_time ? formatDate(clean_time, '年月日时间') : '未清理',
        contractTime: formatDate(sign_up_date, '年月日时间'),
      };
    });
    await this.updateState({
      listLoading: false,
      list: data,
      totalRecords: records,
    });
    return true;
  };

  /**
   * 查询
   * */
  search = () => {
    this.loadData({ currentPage: 1 });
  };

  /**
   * 重置搜索条件
   * */
  resetSearchKeys = () => {
    this.updateState({
      studentName: null,
      courseName: null,
      productName: null,
      cleanState: null,
    });
  };

  render() {
    //解构赋值
    // const {  } = this.props;
    const {
      loading,
      studentName,
      courseName,
      productName,
      cleanState,
      list,
      listLoading,
      totalRecords,
      currentPage,
      pageSize,
    } = this.state;
    const columns = [
      { title: '序号', dataIndex: 'num', key: 'num' },
      { title: '学生名称', dataIndex: 'studentName', key: 'studentName' },
      { title: '小班/讲座名称', dataIndex: 'courseName', key: 'courseName' },
      { title: '产品名称', dataIndex: 'productName', key: 'productName' },
      { title: '金额', dataIndex: 'contractAmount', key: 'contractAmount' },
      { title: '清理时间', dataIndex: 'cleanTime', key: 'cleanTime' },
      { title: '签约时间', dataIndex: 'contractTime', key: 'contractTime' },
    ];

    return (
      <Page className="clean-expired-product-detail" name="查看详情" loading={loading} route={this.props.route}>
        <Block className="c-e-p-d-block" title="查看详情">
          <div>
            <SearchBar onSearch={this.search} onReset={this.resetSearchKeys}>
              <Input
                placeholder="学生名称"
                value={studentName}
                onChange={(e) => this.updateState({ studentName: e.target.value })}
              />
              <Input
                placeholder="小班/讲座名称"
                value={courseName}
                onChange={(e) => this.updateState({ courseName: e.target.value })}
              />
              <Input
                placeholder="产品名称"
                value={productName}
                onChange={(e) => this.updateState({ productName: e.target.value })}
              />
              <Select
                placeholder="清理状态"
                dict={CLEAN_STATE_DICT}
                value={cleanState}
                onChange={(val) => this.updateState({ cleanState: val })}
              />
            </SearchBar>
          </div>
          <div>
            <List
              dataSource={list}
              columns={columns}
              loading={!loading && listLoading}
              pagination={{
                current: currentPage,
                total: totalRecords,
                pageSize,
              }}
              onChange={({ current, pageSize }) => this.loadData({ currentPage: current, pageSize })}
            />
          </div>
        </Block>
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    params: state.main.pageParams['/cleanExpiredProduct/detail'],
  };
};

export default connect(mapStateToProps)(CleanExpiredMiniClassDetail);
