/**
 * Created by Haolin on 2018/6/27.
 */
import React from 'react';
import { connect } from 'dva';
import { Row, Col, Modal } from 'antd';
import Page from 'COMPONENT/Page';
import Section from 'COMPONENT/Section';
import FormItem from 'COMPONENT/FormItem';
import Select from 'COMPONENT/Select';
import Radio from 'COMPONENT/Radio';
import Button from 'COMPONENT/Button';
import SearchBar from 'COMPONENT/SearchBar';
import List from 'COMPONENT/List';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { getDictTextByVal, getDictValByText, loadCommonDict } from 'UTIL/dict';
import { submitExpiredProduct, getExpiredProductList } from 'SERVICE/system';
import './styles.less';
import { format as formatDate } from './utils/date';

const RadioGroup = Radio.Group;

/**
 * 可清理的产品类型_字典
 * */
const cleanTypeDict = [
  {
    val: 'SMALL_CLASS',
    text: '小班',
  },
  {
    val: 'LECTURE',
    text: '讲座',
  },
];
/**
 * 退款方式_字典
 * */
const refundWayDict = [
  {
    val: '0',
    text: '电子账户',
  },
  {
    val: '1',
    text: '退回电子账户并划归收入',
  },
];

class CleanExpiredMiniClass extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {Boolean} 内容加载状态
      error: null, // {string} 数据加载失败的错误信息

      versionDict: [], // {Array} 年份字典
      quarterDict: [], // {Array} 季度字典

      cleanType: '', // {String} 查询条件_产品类型 ["SMALL_CLASS":"小班","LECTURE":"讲座"]
      cleanVersion: '', // {String} 查询条件_产品年份
      cleanQuarter: '', // {String} 查询条件_产品季度
      cleanRefundWay: '0', // {String} 查询条件_退款方式 ["0":"电子账户,","1":"退回电子账户并划归收入"]

      productType: '', // {String} 查询条件_产品类型 ["SMALL_CLASS":"小班","LECTURE":"讲座"]
      productVersion: '', // {String} 查询条件_产品年份
      productQuarter: '', // {String} 查询条件_产品季度
      productRefundWay: '', // {String} 查询条件_退款方式 ["0":"电子账户,","1":"退回电子账户并划归收入"]
      productList: [], // Array} 已清理产品列表
      listLoading: false, // {false} 列表数据加载状态
      totalRecords: 0, // {number} 总记录数
      currentPage: 1, // {number} 当前页码
      pageSize: 10, // {number} 每页记录数
    };
    this.isUnmounted = false;
  }

  componentWillMount() {
    this.initPage();
  }

  componentWillUnmount() {
    this.isUnmounted = true;
  }

  /**
   * 初始化页面
   * */
  initPage = async () => {
    await this.updateState({ loading: true });
    const [yearDict, quarterDict] = await Promise.all([
      loadCommonDict('PRODUCT_VERSION'),
      loadCommonDict('PRODUCT_QUARTER'),
    ]);
    await this.updateState({
      loading: false,
      versionDict: yearDict,
      quarterDict: quarterDict,
    });
    this.loadData();
  };

  /**
   * 更新状态
   * */
  updateState = (newState) => {
    this.setState((prevState, props) => {
      return {
        ...newState,
      };
    });
  };

  /**
   * 加载数据
   * */
  loadData = async (params = {}) => {
    await this.updateState({ listLoading: true, ...params });
    const { productType, productVersion, productQuarter, productRefundWay, currentPage, pageSize } = this.state;
    const apiParams = {
      productType: productType,
      productVersion: productVersion,
      productQuarter: productQuarter,
      refundWay: productRefundWay,
      page: currentPage,
      rows: pageSize,
    };
    const apiData = await getExpiredProductList(apiParams);
    const { records, rows } = apiData;
    //重新拼装数据
    let data = rows.map((item, index, arr) => {
      const {
        id,
        product_type,
        product_version,
        product_quarter,
        total_amount,
        total_money,
        refund_way,
        operate_time,
        operate_user_id,
      } = item;
      return {
        id: id,
        productType: getDictTextByVal(cleanTypeDict, product_type),
        productVersion: product_version,
        productQuarter: product_quarter ? product_quarter : '全年',
        productTotal: total_amount,
        productAmount: total_money ? total_money : 0,
        productRefundWay: getDictTextByVal(refundWayDict, refund_way.toString()),
        actionTime: formatDate(operate_time, '年月日时间'),
        actionPeople: operate_user_id,
      };
    });
    // console.log("列表数据:",data);
    await this.updateState({
      listLoading: false,
      productList: data,
      totalRecords: records,
    });
    return true;
  };

  /**
   * 查询
   * */
  search = () => {
    this.loadData({ currentPage: 1 });
  };

  /**
   * 重置搜索条件
   * */
  resetSearchKeys = () => {
    this.updateState({
      productType: '',
      productVersion: '',
      productQuarter: '',
      productRefundWay: '',
    });
  };

  /**
   * 选择产品类型
   * */
  chooseCleanType = (val) => {
    const { quarterDict } = this.state;
    let quarterObj = {};
    if (val === 'LECTURE') {
      quarterObj = {
        cleanQuarter: getDictValByText(quarterDict, '全年'),
      };
    }
    this.setState(() => {
      return { cleanType: val, ...quarterObj };
    });
  };

  /**
   * 提交清理
   * */
  submitClean = (type = 'search') => {
    const { cleanType, cleanVersion, cleanQuarter, cleanRefundWay } = this.state;
    let errorMsg = '';
    if (type === 'search') {
      if (!cleanType || cleanType === '') {
        errorMsg = '请选择要清理的产品类型';
      } else if (!cleanVersion || cleanVersion === '') {
        errorMsg = '请选择要清理的产品年份';
      } else if (!cleanQuarter || cleanQuarter === '') {
        errorMsg = '请选择要清理的产品季度';
      }
      if (errorMsg !== '') {
        Modal.info({
          title: errorMsg,
          content: '',
          className: 'homAntdModal',
        });
        return false;
      }
    }
    this.updateState({ loading: true });
    let params = {
      productType: cleanType,
      productVersion: cleanVersion,
      productQuarter: cleanQuarter,
      refundWay: cleanRefundWay,
      searchClean: type,
    };
    submitExpiredProduct(params, false).then(
      (json) => {
        this.setState({ loading: false });
        const { data, resultCode, resultMessage } = json;
        if (type === 'search') {
          const submitClean = this.submitClean;
          Modal.confirm({
            title: '提示',
            content: resultMessage,
            className: Number(data) > 0 ? 'homAntdModal' : 'homAntdModal_',
            iconType: '',
            onOk() {
              submitClean('clean');
            },
          });
        } else {
          showMessage('清理成功,明日可看到清理后的结果！', 'success');
          this.updateState({
            cleanType: '',
            cleanVersion: '',
            cleanQuarter: '',
            cleanRefundWay: '0',
          });
          this.loadData();
        }
      },
      (e) => {
        if (type === 'search') {
          showMessage('操作失败', 'error');
        } else {
          showMessage('清理失败', 'error');
        }
        if (e && e.text) {
          this.setState({ error: e.text, loading: false });
        }
      },
    );
  };

  /**
   * 列表的更多操作
   * */
  moreAction = (text, record, index) => {
    switch (text) {
      case 'detail':
        this.goToDetail(record.id);
        break;
      default:
        console.warn('Type Error in moreAction');
    }
  };

  /**
   * 查看详情
   * */
  goToDetail = (id) => {
    let { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: {
        path: '/cleanExpiredProduct/detail',
        params: {
          cleanId: id,
        },
        refresh: true,
      },
    });
  };

  render() {
    //解构赋值
    // const {  } = this.props;
    const {
      loading,
      error,
      versionDict,
      quarterDict,
      cleanType,
      cleanVersion,
      cleanQuarter,
      cleanRefundWay,
      productType,
      productVersion,
      productQuarter,
      productRefundWay,
      productList,
      listLoading,
      totalRecords,
      currentPage,
      pageSize,
    } = this.state;
    const columns = [
      { title: '产品类型', dataIndex: 'productType', key: 'productType' },
      { title: '产品年份', dataIndex: 'productVersion', key: 'productVersion' },
      { title: '产品季度', dataIndex: 'productQuarter', key: 'productQuarter' },
      { title: '总数量', dataIndex: 'productTotal', key: 'productTotal' },
      { title: '总金额', dataIndex: 'productAmount', key: 'productAmount', format: 'money' },
      { title: '退费方式', dataIndex: 'productRefundWay', key: 'productRefundWay' },
      { title: '操作时间', dataIndex: 'actionTime', key: 'actionTime' },
      { title: '操作人', dataIndex: 'actionPeople', key: 'actionPeople' },
      {
        title: '操作',
        key: 'action',
        getActionConfig: (text, record, index) => {
          return [{ text: '查看', onClick: () => this.moreAction('detail', record, index) }];
        },
      },
    ];

    return (
      <Page
        className="clean-expired-product"
        loading={loading}
        route={this.props.route}
        hasBackground={false}
        title={false}
      >
        <Section title="过季产品清理">
          <Row>
            <Col span={6}>
              <FormItem name="产品类型" required>
                <Select
                  placeholder="选择类型"
                  dict={cleanTypeDict}
                  value={cleanType}
                  onChange={(val) => this.chooseCleanType(val)}
                />
              </FormItem>
            </Col>
            <Col span={6}>
              <FormItem name="产品年份" required>
                <Select
                  placeholder="选择年份"
                  dict={versionDict}
                  value={cleanVersion}
                  onChange={(val) =>
                    this.setState(() => {
                      return { cleanVersion: val };
                    })
                  }
                />
              </FormItem>
            </Col>
            <Col span={6}>
              <FormItem name="产品季度" required>
                <Select
                  placeholder="选择季度"
                  dict={quarterDict}
                  value={cleanQuarter}
                  disabled={cleanType === 'LECTURE'}
                  onChange={(val) =>
                    this.setState(() => {
                      return { cleanQuarter: val };
                    })
                  }
                />
              </FormItem>
            </Col>
            <Col span={6}>
              <Button
                type="primary"
                onClick={() => {
                  this.submitClean();
                }}
              >
                清理
              </Button>
            </Col>
          </Row>
          <Row>
            <Col span={12}>
              <FormItem name="退费方式">
                <RadioGroup onChange={(e) => this.setState({ cleanRefundWay: e.target.value })} value={cleanRefundWay}>
                  <Radio value="0">退回到电子账户</Radio>
                  <Radio value="1">退回到电子账户并划归收入</Radio>
                </RadioGroup>
              </FormItem>
            </Col>
          </Row>
        </Section>
        <Section title="已清理产品">
          <div>
            <SearchBar onSearch={this.search} onReset={this.resetSearchKeys}>
              <Select
                placeholder="产品类型"
                dict={cleanTypeDict}
                value={productType}
                onChange={(val) =>
                  this.setState(() => {
                    return { productType: val };
                  })
                }
              />
              <Select
                placeholder="产品年份"
                dict={versionDict}
                value={productVersion}
                onChange={(val) =>
                  this.setState(() => {
                    return { productVersion: val };
                  })
                }
              />
              <Select
                placeholder="产品季度"
                dict={quarterDict}
                value={productQuarter}
                onChange={(val) =>
                  this.setState(() => {
                    return { productQuarter: val };
                  })
                }
              />
              <Select
                placeholder="退费方式"
                dict={refundWayDict}
                value={productRefundWay}
                onChange={(val) =>
                  this.setState(() => {
                    return { productRefundWay: val };
                  })
                }
              />
            </SearchBar>
          </div>
          <div>
            <List
              dataSource={productList}
              columns={columns}
              loading={!loading && listLoading}
              pagination={{
                current: currentPage,
                total: totalRecords,
                pageSize,
              }}
              onChange={({ current, pageSize }) => this.loadData({ currentPage: current, pageSize })}
            />
          </div>
        </Section>
      </Page>
    );
  }
}

export default connect()(CleanExpiredMiniClass);
