import React from 'react';
import ReactDOM from 'react-dom';
import { Page, NoData } from 'COMPONENT';
import {
  loadInitOrgList,
  loadOrgListByParentId,
  loadOrgListByName,
  loadDetail,
  loadCollectionAllSwitch,
} from './loaders';
import styles from './styles.less';
import Left from './Left';
import SwitchItems from './SwitchItems';
import { CIRC_TYPE_1, BOSS_ORG_TYPE_GROUP } from './common';

class CollectionTypeSwicther extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // boolean 页面加载状态
      treeData: [], // object[] 树数据
      treeDataPlat: [], // object[] 扁平的树数据
      onlyBossChecked: true, // boolean “只看BOSS组织架构”选中状态
      expandedKeys: [], // string[] 展开的结点key
      checkedKeys: [], // string[] 选中的结点key
      isSearchMode: false, // boolean 树是否处于显示搜索结果状态
      selectedNode: {}, // string 选中的树结点

      basicData: {}, // object 基本信息
      contractData: {}, // object 地址电话信息
      poolData: {}, // object 资源池信息

      basicEditingData: null, // object 正在编辑的基本信息
      contractEditingData: null, // object 正在编辑的地址电话信息
      poolEditingData: null, // object 正在编辑的资源池信息

      basicEditing: false, // boolean 基本信息编辑状态
      contractEditing: false, // boolean 地址电话信息编辑状态
      poolEditing: false, // boolean 资源池信息编辑状态

      basicChecking: false, // boolean 基本信息校验开启状态
      contractChecking: false, // boolean 地址电话信息校验开启状态
      poolChecking: false, // boolean 资源池信息校验开启状态

      provinceCityDict: [], //省份城市分组字典
      provinceDict: [], //省份字典
      orgTypeDict: [], //组织机构类型字典
      hrmsOrgTypeDict: [], //人事组织机构类型字典
      backflowNodeDict: [], //回流节点字典
      roleDict: [], //角色字典

      switchList: [], // 可切换收款类型状态的字典
    };
    this.basicErrors = {};
    this.contractErrors = {};
    this.poolErrors = {};
  }

  componentWillMount() {
    this.initContent();
  }

  initContent = () => {
    this.setState({ loading: true });
    loadInitOrgList(this.getType()).then(({ data, checkedKeys, platData }) => {
      const root = data[0];
      const rootId = root.id;
      Promise.all([loadDetail({ id: rootId, bossId: root.bossId })]).then(([{ basicData }]) => {
        const selectedNode = {
          ...root,
          bossId: basicData.bossId,
          name: basicData.name,
          valid: basicData.validBoolean,
          orgType: basicData.type,
          isChecked: basicData.isChecked,
          bossOrgType: basicData.bossOrgType,
        };

        loadCollectionAllSwitch({ campusId: selectedNode.bossId }).then((switchList) => {
          this.setState({
            loading: false,
            treeData: data,
            treeDataPlat: platData,
            expandedKeys: [rootId],
            checkedKeys,
            selectedNode,
            switchList: switchList || [],
          });
        });
      });
    });
  };

  getType = () => {
    return this.state.onlyBossChecked ? 'BOSS' : null;
  };

  onOnlyBossChange = (checked) => {
    this.onCancelBasic();
    this.onCancelContract();
    this.onCancelPool();
    this.setState({ loading: true, onlyBossChecked: checked }, () => {
      loadInitOrgList(this.getType()).then(({ data, checkedKeys }) => {
        const root = data[0];
        const rootId = root.id;
        loadDetail({ id: rootId, bossId: root.bossId }).then(({ basicData, contractData, poolData }) => {
          const selectedNode = {
            ...root,
            bossId: basicData.bossId,
            name: basicData.name,
            valid: basicData.validBoolean,
            orgType: basicData.type,
            isChecked: basicData.isChecked,
            bossOrgType: basicData.bossOrgType,
          };
          this.setState({
            loading: false,
            treeData: data,
            expandedKeys: [rootId],
            checkedKeys,
            selectedNode,
            basicData,
            contractData,
            poolData,
          });
        });
      });
    });
  };

  findTreeNodeDataById = (data, id) => {
    let node;
    data.forEach((d) => {
      if (d.id === id) {
        node = d;
        return false;
      } else if (d.children) {
        const dd = this.findTreeNodeDataById(d.children, id);
        if (dd) {
          node = dd;
          return false;
        }
      }
    });
    return node;
  };

  deleteTreeNode = (data, id) => {
    let newData = [];
    data.forEach((d) => {
      if (d.id !== id) {
        let newD;
        if (d.children) {
          newD = { ...d, children: this.deleteTreeNode(d.children, id) };
        } else {
          newD = { ...d };
        }
        newData.push(newD);
      }
    });
    return newData;
  };

  onExpandChange = (expandedKeys, { node, expanded }) => {
    const key = node.props.eventKey;
    if (expanded) {
      const { treeData } = this.state;
      let nextData = [...treeData];
      const nodeData = this.findTreeNodeDataById(nextData, key);
      if (nodeData.hasChildren && !nodeData.children) {
        this.setState({ loading: true });
        loadOrgListByParentId({
          parentId: this.state.onlyBossChecked && !this.state.isSearchMode ? nodeData.bossId : nodeData.id,
          type: this.getType(),
        }).then(({ data, checkedKeys }) => {
          nodeData.children = data;
          this.setState({
            loading: false,
            treeData: nextData,
            treeDataPlat: [...this.state.treeDataPlat, ...data],
            checkedKeys: [...this.state.checkedKeys, ...checkedKeys],
            expandedKeys,
          });
        });
      } else {
        this.setState({ expandedKeys });
      }
    } else {
      this.setState({ expandedKeys });
    }
  };

  onSearch = (key) => {
    this.setState({ loading: true });
    if (key) {
      loadOrgListByName({ name: key }).then(({ data, checkedKeys }) => {
        this.setState({ loading: false, treeData: data, checkedKeys, isSearchMode: true });
      });
    } else {
      loadInitOrgList(this.getType()).then(({ data, checkedKeys }) => {
        this.setState({ loading: false, treeData: data, expandedKeys: [data[0].id], checkedKeys, isSearchMode: false });
      });
    }
  };

  onSelectChange = (selectedKey) => {
    this.selectNode(selectedKey);
  };

  selectNode = (selectedKey) => {
    this.onCancelBasic();
    this.onCancelContract();
    this.onCancelPool();
    const selectedNode = this.findTreeNodeDataById(this.state.treeData, selectedKey);

    if (selectedNode.bossId) {
      this.setState({ loading: true, selectedNode }, () => {
        Promise.all([
          loadDetail({ id: selectedNode.id, bossId: selectedNode.bossId }),
          loadCollectionAllSwitch({ campusId: selectedNode.bossId }),
        ]).then(([{ basicData }, switchList]) => {
          const newSelectedData = {
            ...selectedNode,
            bossId: basicData.bossId,
            name: basicData.name,
            valid: basicData.validBoolean,
            orgType: basicData.type,
            isChecked: basicData.isChecked,
            bossOrgType: basicData.bossOrgType,
          };

          // 如果是分公司，显示类型为SCT_BRANCH_TYPE的开关
          if (newSelectedData.orgType === 'BRANCH') {
            switchList = (switchList || []).filter(function (switchItem) {
              return switchItem.type === 'SCT_BRANCH_TYPE';
            });
          } else {
            // 非分公司，去掉类型为SCT_BRANCH_TYPE的开关
            switchList = (switchList || []).filter(function (switchItem) {
              return switchItem.type !== 'SCT_BRANCH_TYPE';
            });
          }

          this.setState({
            loading: false,
            basicData,
            switchList,
            selectedNode: newSelectedData,
          });
        });
      });
    } else {
      this.setState({
        loading: false,
        basicData: { ...selectedNode },
        switchList: [],
        selectedNode: { ...selectedNode },
      });
    }
  };

  onCancelBasic = () => {
    this.setState({ basicEditing: false, basicEditingData: null, basicChecking: false });
    this.basicErrors = {};
  };

  onCancelContract = () => {
    this.setState({ contractEditing: false, contractEditingData: null, contractChecking: false });
    this.contractErrors = {};
  };

  onCancelPool = () => {
    this.setState({ poolEditing: false, poolEditingData: null, poolChecking: false });
    this.poolErrors = {};
  };

  render() {
    const { loading, treeData, onlyBossChecked, expandedKeys, isSearchMode, selectedNode, switchList } = this.state;

    return (
      <Page loading={loading} route={this.props.route} hasBackground={false} title={false} fitWindowHeight={true}>
        <div className={styles.wrap} ref={(c) => (this.$wrap = ReactDOM.findDOMNode(c))}>
          <div className={styles.left}>
            <Left
              data={treeData}
              onSearch={this.onSearch}
              isSearchMode={isSearchMode}
              onlyBossChecked={onlyBossChecked}
              onOnlyBossChange={this.onOnlyBossChange}
              expandedKeys={expandedKeys}
              onExpandChange={this.onExpandChange}
              selectedKey={selectedNode.id}
              onSelectChange={this.onSelectChange}
              selectedNodeName={selectedNode ? selectedNode.name : null}
            />
          </div>
          <div className={styles.right}>
            {switchList.length ? (
              <SwitchItems
                switchId={selectedNode.bossId}
                switchTitle={selectedNode.name}
                switchList={switchList}
                handleSwitchItem={(index, checked) => {
                  const newSwitchList = [...switchList];
                  newSwitchList[index].flag = checked;
                  this.setState({ switchList: newSwitchList });
                }}
              />
            ) : (
              <NoData />
            )}
          </div>
        </div>
      </Page>
    );
  }
}

export default CollectionTypeSwicther;
