import React from 'react';
import { Row, Col } from 'antd';
import FormItem from 'COMPONENT/FormItem';
import Input from 'COMPONENT/Input';
import InputNumber from 'COMPONENT/InputNumber';
import TextArea from 'COMPONENT/TextArea';
import dicts from '../common';
import { getDictTextByVal } from 'UTIL/dict';
import { FIELD_FILLER, checkFieldEmpty, getFieldWithFiller } from 'COMPONENT/utils/form';

class BasicContent extends React.Component {
  constructor(props) {
    super(props);
    this.state = {};
  }

  onChange = (key, val) => {
    const { data, onChange } = this.props;
    onChange({ ...data, [key]: val });
  };

  render() {
    const { editing, data, checking, onErrorChange, orgTypeDict } = this.props;
    const { oldName, name, belong, type, subType, valid, isBoss, remark, sourceProtectiveDay } = data;
    const formCfg = {
      size: 'small',
      labelWidth: 100,
      style: { padding: '0 5px' },
    };

    return (
      <div>
        <Row style={{ marginLeft: -5 }} gutter={50}>
          <Col span={12}>
            <FormItem {...formCfg} {...formCfg} name="原部门名称">
              {getFieldWithFiller(oldName)}
            </FormItem>
          </Col>
          <Col span={12}>
            <FormItem {...formCfg} name="部门名称" required={editing ? true : null}>
              {editing ? (
                <Input
                  value={name}
                  onChange={(e) => this.onChange('name', e.target.value)}
                  validator={{
                    onErrorChange: (e) => onErrorChange('name', e),
                    silent: !checking,
                  }}
                />
              ) : (
                <div>{getFieldWithFiller(name)}</div>
              )}
            </FormItem>
          </Col>
          <Col span={12}>
            <FormItem {...formCfg} name="归属上级">
              {getFieldWithFiller(belong)}
            </FormItem>
          </Col>
          <Col span={12}>
            <FormItem {...formCfg} name="类型">
              {getDictTextByVal(orgTypeDict, type, true)}
            </FormItem>
          </Col>
          <Col span={12}>
            <FormItem {...formCfg} name="子部门属性">
              {getDictTextByVal(dicts.subOrgTypeDict, subType, true)}
            </FormItem>
          </Col>
          <Col span={12}>
            <FormItem {...formCfg} name="有效性">
              {getDictTextByVal(dicts.statusDict, valid, true)}
            </FormItem>
          </Col>
          <Col span={12}>
            <FormItem {...formCfg} name="是否为Boss架构节点" labelWidth={160}>
              {getDictTextByVal(dicts.yesNoDict, isBoss, true)}
            </FormItem>
          </Col>
          <Col span={10}>
            <FormItem {...formCfg} name="资源保护期(天数)" required={editing ? true : null} labelWidth={160}>
              {editing ? (
                <InputNumber
                  value={sourceProtectiveDay}
                  onChange={(number) => this.onChange('sourceProtectiveDay', number)}
                  min={0}
                  validator={{
                    onErrorChange: (e) => onErrorChange('sourceProtectiveDay', e),
                    silent: !checking,
                  }}
                />
              ) : (
                <div>{sourceProtectiveDay}</div>
              )}
            </FormItem>
          </Col>
          <Col span={24}>
            <FormItem {...formCfg} name="备注">
              {editing ? (
                <TextArea
                  maxLength={200}
                  rows={3}
                  value={remark || ''}
                  onChange={(e) => this.onChange('remark', e.target.value)}
                />
              ) : (
                <div>{getFieldWithFiller(remark)}</div>
              )}
            </FormItem>
          </Col>
        </Row>
      </div>
    );
  }
}

export default BasicContent;
