import React from 'react';
import styles from './styles.less';
import IconButton from 'COMPONENT/IconButton';
import PropTypes from 'prop-types';
import styleVars from 'COMPONENT/theme/variables';
import classnames from 'classnames';
import getWindowSize from 'UTIL/getWindowSize';

const iconCfg = {
  color: styleVars.fontColorTips,
  hoverColor: '#fff',
};

class Gallery extends React.Component {
  static defaultProps = {
    activeIdx: 0,
    list: [],
  };
  constructor(props) {
    super(props);
    this.state = {
      zoomType: 'big',
      winSize: getWindowSize(),
      img: null,
    };
  }

  componentDidMount() {
    this.updateWinW();
    window.addEventListener('resize', this.updateWinSize, false);
    const { list } = this.props;
    const img = new Image();
    img.onload = (e) => {
      console.log('img is loaded');
      console.log(img.width, img.height, e);
      this.setState({ img });
    };
    img.onerror = () => {
      console.log('error!');
    };
    img.src = list[0].src;
  }

  componentWillUnmount() {
    window.removeEventListener('resize', this.updateWinSize, false);
  }

  updateWinW = () => {
    this.setState({ winSize: getWindowSize() });
  };

  prev = () => {};

  next = () => {};

  zoom = () => {};

  rotate = () => {};

  render() {
    const { list, activeIdx, onClose } = this.props;
    const {
      zoomType,
      winSize: { width: winW, height: winH },
      img,
    } = this.state;
    const arrowBtnTop = (winH - 200) / 2 - 21;

    return (
      <div className={styles.wrap}>
        <IconButton {...iconCfg} type="cross" onClick={onClose} size={18} className={styles.closeBtn} />
        <div className={styles.body}>
          <img src={list[activeIdx].src} />
        </div>
        <IconButton
          {...iconCfg}
          type="left"
          size={42}
          onClick={this.prev}
          className={styles.prevBtn}
          style={{ top: arrowBtnTop }}
        />
        <IconButton
          {...iconCfg}
          type="right"
          size={42}
          onClick={this.next}
          className={styles.nextBtn}
          style={{ top: arrowBtnTop }}
        />
        <div className={styles.toolbar}>
          <IconButton
            className={styles.toolBtn}
            {...iconCfg}
            size={24}
            type={zoomType === 'big' ? 'zoom_big' : 'zoom_small'}
            onClick={this.zoom}
          />
          <IconButton className={styles.toolBtn} {...iconCfg} size={24} type="rotate_right" onClick={this.rotate} />
        </div>
        <div className={styles.navbar}>
          {list.map((img, i) => {
            const cls = {
              [styles.navBtn]: 1,
              [styles.navBtnActive]: activeIdx === i,
            };
            return (
              <div key={i} className={classnames(cls)}>
                <img src={img.src} />
              </div>
            );
          })}
        </div>
      </div>
    );
  }
}

export default Gallery;
