import React from 'react';
import styles from './styles.less';
import Icon from 'COMPONENT/Icon';
import IconButton from 'COMPONENT/IconButton';
import styleVars from 'COMPONENT/theme/variables';
import PropTypes from 'prop-types';
/*import Gallery from '../Gallery'*/
import ImageView from '../ImageView';
import Upload from 'COMPONENT/Upload';
import { Progress } from 'antd';
import { objectDeepEquals } from 'COMPONENT/utils/basic/obj';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import CodeUploadPop from '../CodeUploadPop';

const iconBtnCfg = {
  color: '#fff',
  hoverColor: '#fff',
};
class ImageList extends React.Component {
  static defaultProps = {
    data: [],
  };
  constructor(props) {
    super(props);
    this.state = {
      addType: 'pc',
      galleryVisible: false,
      galleryActiveIdx: 0,
      viewVisible: false,
      viewImgSrc: null,
      list: props.data,
      codeUploadPopVisible: false,
    };
  }

  componentWillReceiveProps(nextProps) {
    if (!objectDeepEquals(this.props.data, nextProps.data)) {
      this.setState({ list: nextProps.data });
    }
  }

  onUploadChange = (info, index) => {
    const { list } = this.state;
    let newList;
    if (index === undefined) {
      const fixList = list.filter((li) => !info.some((inf) => inf.fileKey === li.fileKey));
      newList = [...fixList, ...info.slice(0, 9 - fixList.length)];
    } else {
      newList = list.map((li, idx) => {
        if (idx === index) {
          return { ...li, showProgress: info.showProgress, percent: info.percent, src: info.src ? info.src : null };
        }
        return li;
      });
    }
    this.props.onChange(newList);
  };

  checkLength = (files) => {
    if (this.state.list.length + files.length > 9) {
      //1
      // showMessage('最多只能上传2张图片');
      // return false;

      //2
      return new Promise((res, rej) => {
        showConfirm(
          '实景图上传数量不能超过9张，超出部分将不上传',
          () => {
            res(true);
          },
          () => {
            rej(false);
          },
        );
      });
    }
  };

  getAddImg = () => {
    const { addType } = this.state;
    const isCode = addType === 'code';
    return (
      <div key="add" className={styles.card + ' ' + styles.add}>
        {/*        <div className={styles.addIcon}>
          <IconButton type={isCode ? 'pc_half_mc' : '2d_code_half_mc' } size={22} onClick={() => this.setState({ addType: isCode ? 'pc' : 'code' })} />
        </div>
        <div className={styles.addCode} style={{ display: isCode ? 'block' : 'none' }}>
          <QrcodeFileUpLoad height={120} width={120} callback={this.get2dUploadImgInfo}/>
        </div>
        <div className={styles.addPc} style={{ display: !isCode ? 'block' : 'none' }}>*/}
        <div className={styles.addIcon}>
          <IconButton type="2d_code_half_mc" size={22} onClick={() => this.setState({ codeUploadPopVisible: true })} />
        </div>
        <div className={styles.addPc}>
          <Upload
            limit={{ size: 5120, type: 'png/jpg/jpeg' }}
            onChange={this.onUploadChange}
            multiple={true}
            beforeUpload={(files) => this.checkLength(files)}
          >
            <div className={styles.addPcBtn}>
              <Icon type="plus" size={18} />
              <div>单张限5M</div>
            </div>
          </Upload>
        </div>
      </div>
    );
  };

  getImg = (data, idx) => {
    const { editing } = this.props;
    const { src, showProgress, percent } = data;
    return (
      <div key={idx} className={styles.card + ' ' + styles.img}>
        <img src={src} />
        {showProgress ? (
          <div className={styles.progressWrap}>
            <Progress percent={percent} size="small" showInfo={false} strokeWidth={4} />
          </div>
        ) : (
          <div className={styles.btnWrap}>
            {editing && (
              <div className={styles.btn + ' ' + styles.btnUpload}>
                <Upload
                  limit={{ size: 5120, type: 'png/jpg/jpeg' }}
                  onChange={(info) => this.onUploadChange(info, idx)}
                  multiple={false}
                >
                  <div>
                    <IconButton type="upload" {...iconBtnCfg} />
                    <div className={styles.btnText}>上传</div>
                  </div>
                </Upload>
              </div>
            )}
            <div
              className={styles.btn}
              onClick={() =>
                this.setState({ galleryVisible: true, galleryActiveIdx: idx, viewVisible: true, viewImgSrc: src })
              }
            >
              <IconButton type="visible" {...iconBtnCfg} />
              <div className={styles.btnText}>查看</div>
            </div>
            {editing && (
              <div className={styles.btn} onClick={() => this.delete(idx)}>
                <IconButton type="delete" {...iconBtnCfg} />
                <div className={styles.btnText}>删除</div>
              </div>
            )}
          </div>
        )}
      </div>
    );
  };

  delete = (idx) => {
    this.props.onChange(this.state.list.filter((li, i) => i !== idx));
  };

  get2dUploadImgInfo = (info) => {
    console.log(info);
    const filePath = info.filePath;
    const { onChange } = this.props;
    const { list } = this.state;
    onChange([...list, { src: filePath }]);
  };

  render() {
    const { editing } = this.props;
    const { galleryVisible, galleryActiveIdx, viewVisible, viewImgSrc, list, codeUploadPopVisible } = this.state;

    return (
      <div className={styles.wrap}>
        {list.map(this.getImg)}
        {editing && list.length < 9 && this.getAddImg()}
        {/*galleryVisible && <Gallery list={list} activeIdx={galleryActiveIdx} onClose={() => this.setState({ galleryVisible: false })} />*/}
        {viewVisible && <ImageView src={viewImgSrc} onClose={() => this.setState({ viewVisible: false })} />}
        <CodeUploadPop
          src={viewImgSrc}
          onClose={() => this.setState({ codeUploadPopVisible: false })}
          onReceiveData={this.get2dUploadImgInfo}
          visible={codeUploadPopVisible}
        />
      </div>
    );
  }
}

export default ImageList;
