import React from 'react';
import ReactDOM from 'react-dom';
import styles from './styles.less';
import PropTypes from 'prop-types';
import styleVars from 'COMPONENT/theme/variables';
import classnames from 'classnames';
import IconButton from 'COMPONENT/IconButton';

const iconCfg = {
  color: styleVars.fontColorTips,
  hoverColor: '#fff',
};

class ImageView extends React.Component {
  static defaultProps = {};
  constructor(props) {
    super(props);
    this.state = {
      boxW: 0,
      boxH: 0,
      imgW: 0,
      imgH: 0,
    };
  }

  componentDidMount() {
    this.updateBoxSize();
    window.addEventListener('resize', this.updateBoxSize, false);
    const { src } = this.props;
    const img = new Image();
    img.onload = () => {
      const imgW = img.width;
      const imgH = img.height;
      this.setState({ imgW, imgH });
      ReactDOM.findDOMNode(this.imgWrapRef).appendChild(img);
    };
    img.onerror = () => {
      console.log('error!');
    };
    img.src = src;
  }

  componentWillUnmount() {
    window.removeEventListener('resize', this.updateBoxSize, false);
  }

  updateBoxSize = () => {
    const $box = ReactDOM.findDOMNode(this.boxRef);
    this.setState({ boxW: $box.offsetWidth, boxH: $box.offsetHeight });
  };

  getImageStyle = ({ boxW, boxH, imgW, imgH, imgR, boxR }, type) => {
    if (type === 'fitBoxWidth') {
      const h = boxW / imgR;
      return { width: boxW, height: h, left: 0, top: (boxH - h) / 2 };
    } else if (type === 'fitBoxHeight') {
      const w = boxH * imgR;
      return { width: w, height: boxH, left: (boxW - w) / 2, top: 0 };
    } else {
      return { width: imgW, height: imgH, left: (boxW - imgW) / 2, top: (boxH - imgH) / 2 };
    }
  };

  render() {
    const { onClose } = this.props;
    const { boxW, boxH, imgW, imgH } = this.state;
    let sty;
    if (boxW && boxH && imgW && imgH) {
      const imgR = imgW / imgH;
      const boxR = boxW / boxH;
      const cfg = { boxW, boxH, imgW, imgH, imgR, boxR };
      if (imgW > boxW && imgH > boxH) {
        if (imgR > boxR) {
          sty = this.getImageStyle(cfg, 'fitBoxWidth');
        } else {
          sty = this.getImageStyle(cfg, 'fitBoxHeight');
        }
      } else if (imgW > boxW) {
        sty = this.getImageStyle(cfg, 'fitBoxWidth');
      } else if (imgH > boxH) {
        sty = this.getImageStyle(cfg, 'fitBoxHeight');
      } else {
        sty = this.getImageStyle(cfg);
      }
    } else {
      sty = { visibility: 'hidden' };
    }
    return (
      <div className={styles.wrap}>
        <IconButton {...iconCfg} type="cross" onClick={onClose} size={18} className={styles.closeBtn} />
        <div className={styles.box} ref={(c) => (this.boxRef = c)}>
          <div className={styles.imgWrap} style={sty} ref={(c) => (this.imgWrapRef = c)}></div>
        </div>
      </div>
    );
  }
}

export default ImageView;
