import React from 'react';
import { Tree } from 'antd';
import Checkbox from 'COMPONENT/Checkbox';
import styles from './styles.less';
import Input from 'COMPONENT/Input';
import Icon from 'COMPONENT/Icon';
import { objectDeepEquals } from 'COMPONENT/utils/basic/obj';
import { BOSS_ORG_TYPE_GROUP } from '../common';

const TreeNode = Tree.TreeNode;

class Left extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      searchKey: null,
      searchIconType: 'search_mc',
    };
  }

  shouldComponentUpdate(nextProps, nextState) {
    if (objectDeepEquals(this.props, nextProps) && objectDeepEquals(this.state, nextState)) {
      return false;
    }
    return true;
  }

  getTreeNode = (data) => {
    const { id, name, children, valid, hasChildren, orgType } = data;
    const disableCheckbox =
      !valid || orgType === 'REGION' || orgType === 'HIERARCHY' || orgType === BOSS_ORG_TYPE_GROUP;
    if (hasChildren) {
      return (
        <TreeNode key={id} title={name} disableCheckbox={disableCheckbox}>
          {children && children.length > 0 ? children.map((d) => this.getTreeNode(d)) : []}
        </TreeNode>
      );
    } else {
      return <TreeNode key={id} title={name} disableCheckbox={disableCheckbox} />;
    }
  };

  search = () => {
    this.props.onSearch(this.state.searchKey);
  };

  searchInputChange = (e) => {
    this.setState({ searchKey: e.target.value });
  };

  searchInputKeyDown = (e) => {
    if (e.keyCode === 13) {
      this.search();
    }
  };

  onSelect = (keys) => {
    if (keys && keys.length > 0) {
      this.props.onSelectChange(keys[0]);
    }
  };

  render() {
    const {
      data,
      onlyBossChecked,
      onOnlyBossChange,
      expandedKeys,
      onExpandChange,
      checkedKeys,
      onCheckChange,
      isSearchMode,
      selectedKey,
      onSelectChange,
    } = this.props;
    const { searchIconType } = this.state;
    return (
      <div className={styles.wrap}>
        <div className={styles.search}>
          <div className={styles.searchIcon} onClick={this.search}>
            <Icon type={searchIconType} />
          </div>
          <Input
            placeholder="你可以输入名称搜索"
            style={{ width: 244 }}
            onChange={this.searchInputChange}
            onKeyDown={this.searchInputKeyDown}
            onFocus={() => this.setState({ searchIconType: 'search_mc_h' })}
            onBlur={() => this.setState({ searchIconType: 'search_mc' })}
          />
        </div>
        {isSearchMode ? (
          <div className={styles.resultWrap}>
            共搜索到
            <span className={styles.resultCount}>{data.length}</span>
            条结果
          </div>
        ) : (
          <div className={styles.checkWrap}>
            <Checkbox checked={onlyBossChecked} onChange={(e) => onOnlyBossChange(e.target.checked)}>
              只看Boss组织架构
            </Checkbox>
          </div>
        )}
        <div className={styles.treeWrap}>
          <Tree
            checkStrictly={true}
            autoExpandParent={false}
            checkable
            checkedKeys={checkedKeys}
            onCheck={onCheckChange}
            expandedKeys={expandedKeys}
            onExpand={onExpandChange}
            selectedKeys={selectedKey ? [selectedKey] : null}
            onSelect={this.onSelect}
          >
            {data && data.map((d) => this.getTreeNode(d))}
          </Tree>
        </div>
      </div>
    );
  }
}

export default Left;
