import React from 'react';
import ReactDOM from 'react-dom';
import { Row, Col } from 'antd';
import FormItem from 'COMPONENT/FormItem';
import Input from 'COMPONENT/Input';
import InputNumber from 'COMPONENT/InputNumber';
import Select from 'COMPONENT/Select';
import styles from './styles.less';
import dicts from '../common';
import { getDictTextByVal } from 'UTIL/dict';
import styleVars from 'COMPONENT/theme/variables';
import { objectDeepEquals } from 'COMPONENT/utils/basic/obj';
import { FIELD_FILLER, checkFieldEmpty, getFieldWithFiller } from 'COMPONENT/utils/form';
import { checkNumber } from 'COMPONENT/Validator/checkers';
import RoleList from '../RoleList';
import { TreeSelect } from 'antd';
const SHOW_PARENT = TreeSelect.SHOW_PARENT;

class ResourcePoolContent extends React.Component {
  onChange = (changed) => {
    const { data, onChange } = this.props;
    const newData = { ...data, ...changed, cycleType: dicts.CIRC_TYPE_2 };
    if (!objectDeepEquals(data, newData)) {
      onChange(newData);
    }
  };

  circTypeChange = (val) => {
    this.onChange({ cycleType: val });
    if (val !== dicts.CIRC_TYPE_1) {
      this.props.onErrorChange('returnNote', null);
      this.props.onErrorChange('recoveryPeriod', null);
    }
  };

  onTreeChange = (keys, labels, selectedNode) => {
    this.onChange({ returnNote: [selectedNode.triggerValue] });
  };

  render() {
    const { editing, data, checking, onErrorChange, backflowNodeDict, roleDict, bossOrgType } = this.props;
    const { active, name, capacity, cycleType, returnNote, recoveryPeriod, resourcePoolRoleSaveDtos } = data;
    const formCfg = {
      labelWidth: 130,
      size: editing ? null : 'small',
      style: { padding: '0 5px' },
    };

    const treeProps = {
      treeData: this.props.treeData,
      value: returnNote,
      onChange: this.onTreeChange,
      treeCheckable: true,
      showCheckedStrategy: SHOW_PARENT,
      searchPlaceholder: '请选择',
      // style: {
      //   width: 300,
      // },
      size: 'large',
    };

    return (
      <div className={styles.wrap + (editing ? ' ' + styles.wrapEditing : '')}>
        <div className={styles.title}>基本信息</div>
        <Row style={{ marginLeft: -5 }} gutter={50}>
          <Col span={12}>
            <FormItem {...formCfg} name="资源池名称" required={editing ? true : null}>
              {editing ? (
                <Input
                  value={name}
                  placeholder="请输入名称"
                  onChange={(e) => this.onChange({ name: e.target.value })}
                  validator={{
                    onErrorChange: (e) => onErrorChange('name', e),
                    silent: !checking,
                  }}
                  maxLength={15}
                  wordCount={true}
                />
              ) : (
                <div>{getFieldWithFiller(name)}</div>
              )}
            </FormItem>
          </Col>
          <Col span={12}>
            <FormItem {...formCfg} name="资源池容量（个）" required={editing ? true : null}>
              {editing ? (
                <InputNumber
                  value={capacity}
                  placeholder="请输入数量"
                  onChange={(val) => this.onChange({ capacity: val })}
                  validator={{
                    onErrorChange: (e) => onErrorChange('capacity', e),
                    silent: !checking,
                    rules: ['required', (v) => checkNumber(v, { excludeMin: 0, int: true })],
                  }}
                  maxLength={10}
                />
              ) : (
                <div>{getFieldWithFiller(capacity)}</div>
              )}
            </FormItem>
          </Col>
          <Col span={12} style={{ clear: 'left' }}>
            <FormItem {...formCfg} name="流转方式" required={editing ? true : null}>
              {editing ? (
                <Select
                  dict={dicts.recourcePoolCycleTypeDict}
                  value={dicts.CIRC_TYPE_2}
                  onChange={this.circTypeChange}
                  validator={{
                    onErrorChange: (e) => onErrorChange('cycleType', e),
                    silent: !checking,
                  }}
                  allowClear={false}
                  disabled={true}
                />
              ) : (
                <div>{cycleType ? dicts.circTypeMap[cycleType] : '-'}</div>
              )}
            </FormItem>
          </Col>
        </Row>
        <RoleList
          data={resourcePoolRoleSaveDtos}
          onChange={(val) => this.onChange({ resourcePoolRoleSaveDtos: val })}
          roleDict={roleDict}
          editing={editing}
        />
      </div>
    );
  }
}

export default ResourcePoolContent;
