import React from 'react';
import ReactDOM from 'react-dom';
import { Switch } from 'antd';
import Page from 'COMPONENT/Page';
import Modal from 'COMPONENT/Modal';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import showAlert from 'COMPONENT/utils/globalTips/showAlert';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import {
  saveHrmsOrgToBoss,
  cacelHrmsOrgInBoss,
  getWithRolesByOrgIdAndType,
  resourcePoolSave,
  resourcePoolSwitchStatus,
} from 'SERVICE/system';
import {
  loadInitOrgList,
  loadOrgListByParentId,
  loadOrgListByName,
  loadDetail,
  saveDetail,
  loadBackflowNodeDict,
  loadRoleDict,
  listResourcePoolsDict,
} from './loaders';
import { loadProvinceCityDict, loadCommonDict } from 'UTIL/dict';
import styles from './styles.less';
import Left from './Left';
import EditBlock from './EditBlock';
import BasicContent from './BasicContent';
import ContractContent from './ContractContent';
import PoolContent from './PoolContent';
import ResourcePool from './ResourcePool';
import { objectDeepEquals } from 'COMPONENT/utils/basic/obj';
import { getCommonErrorTips } from 'COMPONENT/utils/form';
import PopConfirm from 'COMPONENT/PopConfirm';
import {
  CIRC_TYPE_1,
  BOSS_ORG_TYPE_GROUP,
  BOSS_ORG_TYPE_BRANCH,
  ResourceTypeOut,
  ResourceStatusValid,
  BRANCH,
  isBranch,
} from './common';

class OrgTree extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // boolean 页面加载状态
      treeData: [], // object[] 树数据
      treeDataPlat: [], // object[] 扁平的树数据
      onlyBossChecked: true, // boolean “只看BOSS组织架构”选中状态
      expandedKeys: [], // string[] 展开的结点key
      checkedKeys: [], // string[] 选中的结点key
      isSearchMode: false, // boolean 树是否处于显示搜索结果状态
      selectedNode: {}, // string 选中的树结点

      basicData: {}, // object 基本信息
      contractData: {}, // object 地址电话信息
      poolData: {}, // object 资源池信息
      resourcePoolData: null, // object 外呼资源池信息

      basicEditingData: null, // object 正在编辑的基本信息
      contractEditingData: null, // object 正在编辑的地址电话信息
      poolEditingData: null, // object 正在编辑的资源池信息
      resourcePoolEditingData: null, // object 正在编辑的资源池信息

      basicEditing: false, // boolean 基本信息编辑状态
      contractEditing: false, // boolean 地址电话信息编辑状态
      poolEditing: false, // boolean 资源池信息编辑状态
      resourcePoolEditing: false, // boolean 资源池信息编辑状态

      basicChecking: false, // boolean 基本信息校验开启状态
      contractChecking: false, // boolean 地址电话信息校验开启状态
      poolChecking: false, // boolean 资源池信息校验开启状态
      resourcePoolChecking: false, // boolean 资源池信息校验开启状态

      provinceCityDict: [], //省份城市分组字典
      provinceDict: [], //省份字典
      orgTypeDict: [], //组织机构类型字典
      hrmsOrgTypeDict: [], //人事组织机构类型字典
      backflowNodeDict: [], //回流节点字典
      resourceBackFlowNodeDict: [],
      roleDict: [], //角色字典
      organizationId: '',
    };
    this.basicErrors = {};
    this.contractErrors = {};
    this.poolErrors = {};
    this.resourcePoolErrors = {};
  }

  componentWillMount() {
    this.initContent();
  }

  initContent = () => {
    this.setState({ loading: true });
    loadInitOrgList(this.getType()).then(({ data, checkedKeys, platData }) => {
      const root = data[0];
      const rootId = root.id;
      Promise.all([
        loadDetail({ id: rootId, bossId: root.bossId }),
        loadProvinceCityDict(),
        loadCommonDict('OrganizationTypeHrms'),
        loadCommonDict('OrganizationType'),
        loadBackflowNodeDict(root.bossId),
        loadRoleDict(),
      ]).then(
        ([
          { basicData, contractData, poolData },
          provinceCityDict,
          hrmsOrgTypeDict,
          orgTypeDict,
          backflowNodeDict,
          roleDict,
        ]) => {
          const provinceDict = provinceCityDict.map((d) => {
            return { val: d.val, text: d.label ? d.label : d.text };
          });
          const selectedNode = {
            ...root,
            bossId: basicData.bossId,
            name: basicData.name,
            valid: basicData.validBoolean,
            orgType: basicData.type,
            isChecked: basicData.isChecked,
            bossOrgType: basicData.bossOrgType,
          };
          this.setState({
            loading: false,
            treeData: data,
            treeDataPlat: platData,
            expandedKeys: [rootId],
            checkedKeys,
            selectedNode,
            basicData,
            contractData,
            poolData,
            provinceCityDict,
            provinceDict,
            orgTypeDict,
            hrmsOrgTypeDict,
            backflowNodeDict,
            roleDict,
          });
        },
      );
    });
  };

  getType = () => {
    return this.state.onlyBossChecked ? 'BOSS' : null;
  };

  onOnlyBossChange = (checked) => {
    this.onCancelBasic();
    this.onCancelContract();
    this.onCancelPool();
    this.setState({ loading: true, onlyBossChecked: checked }, () => {
      loadInitOrgList(this.getType()).then(({ data, checkedKeys }) => {
        const root = data[0];
        const rootId = root.id;
        loadDetail({ id: rootId, bossId: root.bossId }).then(({ basicData, contractData, poolData }) => {
          const selectedNode = {
            ...root,
            bossId: basicData.bossId,
            name: basicData.name,
            valid: basicData.validBoolean,
            orgType: basicData.type,
            isChecked: basicData.isChecked,
            bossOrgType: basicData.bossOrgType,
          };
          this.setState({
            loading: false,
            treeData: data,
            expandedKeys: [rootId],
            checkedKeys,
            selectedNode,
            basicData,
            contractData,
            poolData,
          });
        });
      });
    });
  };

  findTreeNodeDataById = (data, id) => {
    let node;
    data.forEach((d) => {
      if (d.id === id) {
        node = d;
        return false;
      } else if (d.children) {
        const dd = this.findTreeNodeDataById(d.children, id);
        if (dd) {
          node = dd;
          return false;
        }
      }
    });
    return node;
  };

  deleteTreeNode = (data, id) => {
    let newData = [];
    data.forEach((d) => {
      if (d.id !== id) {
        let newD;
        if (d.children) {
          newD = { ...d, children: this.deleteTreeNode(d.children, id) };
        } else {
          newD = { ...d };
        }
        newData.push(newD);
      }
    });
    return newData;
  };

  /*
  getParents = (data, id) => {
    let parents = []
    const node = data.find(d => d.id === id)
    if (node) {
      if (node.parentId) {
        const parentNode = data.find(d => d.id === node.parentId)
        parents.push(parentNode)
        parents = [...parents, ...this.getParents(data, parentNode.id)]
      }
    }
    return parents
  }*/

  onSaveError = (e, text) => {
    showMessage(e.text || text || '操作失败');
    this.setState({ loading: false });
  };

  onExpandChange = (expandedKeys, { node, expanded }) => {
    const key = node.props.eventKey;
    if (expanded) {
      const { treeData } = this.state;
      let nextData = [...treeData];
      const nodeData = this.findTreeNodeDataById(nextData, key);
      if (nodeData.hasChildren && !nodeData.children) {
        this.setState({ loading: true });
        loadOrgListByParentId({
          parentId: this.state.onlyBossChecked && !this.state.isSearchMode ? nodeData.bossId : nodeData.id,
          type: this.getType(),
        }).then(({ data, checkedKeys }) => {
          nodeData.children = data;
          this.setState({
            loading: false,
            treeData: nextData,
            treeDataPlat: [...this.state.treeDataPlat, ...data],
            checkedKeys: [...this.state.checkedKeys, ...checkedKeys],
            expandedKeys,
          });
        });
      } else {
        this.setState({ expandedKeys });
      }
    } else {
      this.setState({ expandedKeys });
    }
  };

  isSub = (node, k) => {
    if (node.children) {
      const subNode = this.findTreeNodeDataById(node.children, k);
      return subNode ? true : false;
    } else {
      return false;
    }
  };

  onCheckChange = (checkedKeys, { node, checked }) => {
    const key = node.props.eventKey;
    const keyNode = this.findTreeNodeDataById(this.state.treeData, key);
    const { id } = keyNode;
    if (checked) {
      /*      const parents = this.getParents(this.state.treeDataPlat, key)
      const has = parents.some(p => this.state.checkedKeys.indexOf(p.id) < 0)
      if (has) {
        showAlert('请先勾选父节点')
        return
      }*/
      this.setState({ loading: true });
      saveHrmsOrgToBoss({ id }, true).then(
        () => {
          Modal.success({ content: '新勾选的组织架构，如需配置该组织架构下的用户权限，请前往用户管理进行配置' });
          this.setState({ loading: false, checkedKeys: [...this.state.checkedKeys, key] });
          this.checkReloadDetail(key);
        },
        (e) => this.onSaveError(e, '勾选失败'),
      );
    } else {
      showConfirm(
        '取消勾选，当前组织架构和下级节点及节点下用户权限都将在Boss系统失效，确定取消勾选组织架构节点并保存？',
        () => {
          this.setState({ loading: true });
          cacelHrmsOrgInBoss({ id }, true).then(
            () => {
              showMessage('取消勾选成功', 'success');
              let changed = { loading: false };
              if (this.state.onlyBossChecked && !this.state.isSearchMode) {
                let newTreeData = [...this.state.treeData];
                newTreeData = this.deleteTreeNode(newTreeData, key);
                changed = { ...changed, treeData: newTreeData };
                if (key === this.state.selectedNode.id && this.state.treeData.length > 0) {
                  this.selectNode(this.state.treeData[0].id);
                }
              } else {
                changed = {
                  ...changed,
                  checkedKeys: this.state.checkedKeys.filter((k) => k !== key && !this.isSub(keyNode, k)),
                };
                this.checkReloadDetail(key);
              }
              this.setState(changed);
            },
            (e) => this.onSaveError(e, '取消勾选失败'),
          );
        },
      );
    }
  };

  checkReloadDetail = (key) => {
    if (key === this.state.selectedNode.id) {
      this.selectNode(key);
    }
  };

  onSearch = (key) => {
    const { onlyBossChecked } = this.state;
    this.setState({ loading: true });
    if (key) {
      loadOrgListByName({ name: key }).then(({ data, checkedKeys }) => {
        this.setState({ loading: false, treeData: data, checkedKeys, isSearchMode: true });
      });
    } else {
      loadInitOrgList(this.getType()).then(({ data, checkedKeys }) => {
        this.setState({ loading: false, treeData: data, expandedKeys: [data[0].id], checkedKeys, isSearchMode: false });
      });
    }
  };

  onSelectChange = (selectedKey) => {
    const { basicEditingData, contractEditingData, poolEditingData } = this.state;
    if (basicEditingData || contractEditingData || poolEditingData) {
      Modal.confirm({
        content: '当前节点有内容更改，是否保存并切换节点？',
        onOk: () => {
          this.saveAll().then(() => {
            this.selectNode(selectedKey);
          });
        },
        onCancel: () => {
          this.selectNode(selectedKey);
        },
        iconType: 'exclamation-circle',
        okText: '保存',
        cancelText: '不保存',
      });
    } else {
      this.selectNode(selectedKey);
    }
  };

  getResourcePoolData = async () => {
    return getWithRolesByOrgIdAndType({ orgId: this.state.organizationId, resourceType: ResourceTypeOut })
      .then(({ data: resourcePoolData }) => {
        if (resourcePoolData) {
          const { resourcePoolRoleVos, status, cycleType, ...rest } = resourcePoolData;
          this.setState({
            resourcePoolData: {
              ...rest,
              status: !!status,
              cycleType: cycleType + '',
              resourcePoolRoleSaveDtos: resourcePoolRoleVos.map((item) => ({
                ...item,
                name: item.roleId,
                max: item.oneTimeResource,
              })),
            },
          });
          return;
        }

        this.setState({
          resourcePoolData: null,
        });
      })
      .catch((err) => {
        this.setState({
          loading: false,
        });
      });
  };

  selectNode = (selectedKey) => {
    this.onCancelBasic();
    this.onCancelContract();
    this.onCancelPool();
    const selectedNode = this.findTreeNodeDataById(this.state.treeData, selectedKey);

    this.setState(
      {
        loading: true,
        selectedNode,
        organizationId: selectedNode.bossId,
        resourcePoolData: null,
        resourcePoolEditing: false,
        resourcePoolEditingData: null,
        resourcePoolChecking: false,
      },
      () => {
        if (isBranch(selectedNode.orgType) && selectedNode.isChecked) {
          this.getResourcePoolData();
        }

        Promise.all([
          loadDetail({ id: selectedNode.id, bossId: selectedNode.bossId }),
          loadBackflowNodeDict(selectedNode.bossId),
          listResourcePoolsDict(selectedNode.bossId, ResourceTypeOut),
        ])
          .then(([{ basicData, contractData, poolData }, backflowNodeDict, resourceBackFlowNodeDict]) => {
            const newSelectedData = {
              ...selectedNode,
              bossId: basicData.bossId,
              name: basicData.name,
              valid: basicData.validBoolean,
              orgType: basicData.type,
              isChecked: basicData.isChecked,
              bossOrgType: basicData.bossOrgType,
            };
            this.setState({
              loading: false,
              basicData,
              contractData,
              poolData,
              selectedNode: newSelectedData,
              backflowNodeDict,
              resourceBackFlowNodeDict,
            });
          })
          .catch((err) => {
            this.setState({
              loading: false,
            });
          });
      },
    );
  };

  saveAll = () => {
    const {
      basicChecking,
      basicEditingData,
      basicData,
      contractChecking,
      contractEditingData,
      contractData,
      poolChecking,
      poolEditingData,
      poolData,
      treeData,
      selectedNode,
    } = this.state;
    if (!basicChecking) {
      this.setState({ basicChecking: true });
    }
    if (!contractChecking) {
      this.setState({ contractChecking: true });
    }
    if (!poolChecking) {
      this.setState({ poolChecking: true });
    }
    if (Object.values(this.basicErrors).some((v) => v)) {
      this.basicBlockRef.showSaveTips();
      return;
    }
    if (Object.values(this.contractErrors).some((v) => v)) {
      this.contractBlockRef.showSaveTips();
      return;
    }
    if (Object.values(this.poolErrors).some((v) => v)) {
      this.poolBlockRef.showSaveTips();
      return;
    }

    if (isBranch(selectedNode.bossOrgType) && selectedNode.isChecked) {
      this.onSaveResourcePool();
    }

    return new Promise((resolve, reject) => {
      saveDetail({
        ...basicData,
        ...contractData,
        ...poolData,
        ...(basicEditingData || {}),
        ...(contractEditingData || {}),
        ...(poolEditingData || {}),
        modifyType: 'ALL',
      }).then(
        () => {
          showMessage('保存成功', 'success');
          let changed = {
            basicEditing: false,
            basicEditingData: null,
            basicData: basicEditingData || basicData,
            contractEditing: false,
            contractEditingData: null,
            contractData: contractEditingData || contractData,
            poolEditing: false,
            poolEditingData: null,
            poolData: poolEditingData || poolData,
            basicChecking: false,
            contractChecking: false,
            poolChecking: false,
          };
          if (basicEditingData) {
            let newTreeData = [...treeData];
            let node = this.findTreeNodeDataById(newTreeData, selectedNode.id);
            node.name = basicEditingData.name;
            changed = { ...changed, treeData: newTreeData };
          }
          this.setState(changed);
          resolve();
        },
        (e) => {
          this.onSaveError(e, '保存失败');
          reject();
        },
      );
    });
  };

  onSaveBasic = () => {
    const { basicChecking, basicEditingData, treeData, selectedNode } = this.state;
    if (!basicChecking) {
      this.setState({ basicChecking: true });
    }
    const errorTips = getCommonErrorTips(this.basicErrors);
    if (errorTips) {
      this.basicBlockRef.showSaveTips(errorTips);
      return;
    }
    this.setState({ loading: true });
    saveDetail({
      ...this.state.contractData,
      ...this.state.poolData,
      ...basicEditingData,
      modifyType: 'INFO',
    }).then(
      () => {
        showMessage('保存成功', 'success');
        let newTreeData = JSON.parse(JSON.stringify(treeData));
        let node = this.findTreeNodeDataById(newTreeData, selectedNode.id);
        node.name = basicEditingData.name;
        this.setState({
          loading: false,
          basicEditing: false,
          basicEditingData: null,
          basicData: basicEditingData,
          basicChecking: false,
          treeData: newTreeData,
        });
      },
      (e) => this.onSaveError(e, '保存失败'),
    );
  };

  onSaveContract = () => {
    const { contractChecking, contractEditingData } = this.state;
    if (!contractChecking) {
      this.setState({ contractChecking: true });
    }
    const errorTips = getCommonErrorTips(this.contractErrors);
    if (errorTips) {
      this.contractBlockRef.showSaveTips(errorTips);
      return;
    }
    this.setState({ loading: true });
    saveDetail({
      ...this.state.basicData,
      ...this.state.poolData,
      ...contractEditingData,
      modifyType: 'APPEND',
    }).then(
      () => {
        showMessage('保存成功', 'success');
        this.setState({
          loading: false,
          contractEditing: false,
          contractEditingData: null,
          contractData: contractEditingData,
          contractChecking: false,
        });
      },
      (e) => this.onSaveError(e, '保存失败'),
    );
  };

  onSavePool = () => {
    const { poolChecking, poolEditingData } = this.state;
    if (!poolChecking) {
      this.setState({ poolChecking: true });
    }
    const errorTips = getCommonErrorTips(this.poolErrors);
    if (errorTips) {
      this.poolBlockRef.showSaveTips(errorTips);
      return;
    }
    this.setState({ loading: true });
    saveDetail({
      ...this.state.basicData,
      ...this.state.contractData,
      ...poolEditingData,
      modifyType: 'RESOURCE_POOL',
    }).then(
      () => {
        showMessage('保存成功', 'success');
        this.setState({
          loading: false,
          poolEditing: false,
          poolEditingData: null,
          poolData: poolEditingData,
          poolChecking: false,
        });
      },
      (e) => this.onSaveError(e, '保存失败'),
    );
  };

  onSaveResourcePool = async () => {
    const { resourcePoolChecking, resourcePoolEditingData } = this.state;
    if (!resourcePoolChecking) {
      this.setState({ resourcePoolChecking: true });
    }
    const errorTips = getCommonErrorTips(this.resourcePoolErrors);
    if (errorTips) {
      this.resourcePoolBlockRef.showSaveTips(errorTips);
      return;
    }

    if (resourcePoolEditingData.resourcePoolRoleSaveDtos.length === 0) {
      this.resourcePoolBlockRef.showSaveTips('请设置资源池可见角色！');
      return;
    }

    this.setState({ loading: true });
    try {
      const { resourcePoolRoleSaveDtos, cycleType, ...restEditData } = resourcePoolEditingData;
      const commonParams = {
        organizationId: this.state.organizationId,
        resourceType: ResourceTypeOut,
      };
      await resourcePoolSave({
        ...commonParams,
        ...restEditData,
        resourcePoolRoleSaveDtos: resourcePoolRoleSaveDtos.map((item) => ({
          ...commonParams,
          ...item,
          roleId: item.name,
          oneTimeResource: item.max,
        })),
        status: resourcePoolEditingData.status ? 1 : 0,
        returnNote: this.state.organizationId,
        cycleType: cycleType === undefined ? 1 : +cycleType,
      });
      await this.getResourcePoolData();
    } finally {
      this.setState({
        loading: false,
        resourcePoolEditing: false,
        resourcePoolEditingData: null,
        resourcePoolChecking: false,
      });
    }
  };

  onCancelBasic = () => {
    this.setState({ basicEditing: false, basicEditingData: null, basicChecking: false });
    this.basicErrors = {};
  };

  onCancelContract = () => {
    this.setState({ contractEditing: false, contractEditingData: null, contractChecking: false });
    this.contractErrors = {};
  };

  onCancelPool = () => {
    this.setState({ poolEditing: false, poolEditingData: null, poolChecking: false });
    this.poolErrors = {};
  };

  // todo
  onCancelResourcePool = () => {
    this.setState({ resourcePoolEditing: false, resourcePoolEditingData: null, resourcePoolChecking: false });
    this.resourcePoolErrors = {};
  };

  onContractActiveChange = (active) => {
    const newContractData = { ...this.state.contractData, active };
    this.setState({ loading: true });
    saveDetail({
      ...this.state.basicData,
      ...this.state.poolData,
      ...newContractData,
      modifyType: 'APPEND',
    }).then(
      () => {
        showMessage(active ? '已激活成功' : '已关闭激活', 'success');
        this.setState({
          loading: false,
          contractEditing: false,
          contractEditingData: null,
          contractData: newContractData,
        });
      },
      (e) => this.onSaveError(e, '保存失败'),
    );
  };

  onPoolActiveChange = (active) => {
    let newPoolData;
    /*     if (!this.state.poolData.poolName) {
      console.log(this.state.backflowNodeDict)
      newPoolData = {
        active,
        poolName: this.state.basicData.name + '资源池',
        volume: 0,
        circType: CIRC_TYPE_1,
        backflowNode: this.state.backflowNodeDict[0] ? this.state.backflowNodeDict[0].val : null,
        cycle: 0,
        roleList: [],
      }
    } else { */
    newPoolData = { ...this.state.poolData, active };
    /*     } */
    this.setState({ loading: true });
    saveDetail({
      ...this.state.basicData,
      ...this.state.contractData,
      ...newPoolData,
      modifyType: 'RESOURCE_POOL',
    }).then(
      () => {
        showMessage(active ? '已激活成功' : '已关闭激活', 'success');
        this.setState({ loading: false, poolEditing: false, poolEditingData: null, poolData: newPoolData });
      },
      (e) => this.onSaveError(e, '保存失败'),
    );
  };

  // todo
  onResourcePoolActiveChange = async (status) => {
    this.setState({ loading: true });
    try {
      if (!this.state.resourcePoolData) {
        this.setState({
          resourcePoolData: {
            status,
            resourcePoolRoleSaveDtos: [],
          },
        });
        return;
      }

      await resourcePoolSwitchStatus({
        organizationId: this.state.organizationId,
        resourceType: ResourceTypeOut,
        status: status ? 1 : 0,
      });
      await this.getResourcePoolData();
      // showMessage(status ? '已激活成功' : '已关闭激活', 'success');
    } finally {
      this.setState({ loading: false, resourcePoolEditing: false, poolEditingData: null });
    }
  };

  render() {
    const {
      loading,
      treeData,
      onlyBossChecked,
      expandedKeys,
      checkedKeys,
      isSearchMode,
      selectedNode,
      basicData,
      contractData,
      poolData,
      basicEditing,
      contractEditing,
      poolEditing,
      basicChecking,
      contractChecking,
      poolChecking,
      basicEditingData,
      contractEditingData,
      poolEditingData,
      provinceCityDict,
      provinceDict,
      orgTypeDict,
      hrmsOrgTypeDict,
      backflowNodeDict,
      roleDict,
      resourcePoolEditing,
      resourcePoolEditingData,
      resourcePoolData,
      resourceBackFlowNodeDict,
      resourcePoolChecking,
    } = this.state;

    const contractActive = contractData ? contractData.active : false;
    const poolActive = poolData ? poolData.active : false;
    const editable = selectedNode.isChecked && selectedNode.valid && selectedNode.orgType !== 'REGION';
    let contractSwitchBtn = (
      <span>
        <Switch
          size="small"
          checked={contractActive}
          disabled={!editable}
          onChange={contractActive ? () => {} : this.onContractActiveChange}
        />
      </span>
    );
    if (contractActive && editable) {
      contractSwitchBtn = (
        <PopConfirm
          content="关闭地址电话信息，当前信息的数据将不删除但不可使用，是否确定关闭地址电话信息？"
          onOk={() => this.onContractActiveChange(false)}
          getPopupContainer={() => this.$wrap}
        >
          {contractSwitchBtn}
        </PopConfirm>
      );
    }

    let poolSwitchBtn = (
      <span>
        <Switch
          size="small"
          checked={poolActive}
          disabled={!editable}
          onChange={poolActive ? () => {} : this.onPoolActiveChange}
        />
      </span>
    );
    if (poolActive && editable) {
      poolSwitchBtn = (
        <PopConfirm
          content="关闭后，该资源池数据将不删除但不可使用，是否确定关闭？"
          onOk={() => this.onPoolActiveChange(false)}
          getPopupContainer={() => this.$wrap}
        >
          {poolSwitchBtn}
        </PopConfirm>
      );
    }

    const resourcePoolActive = resourcePoolData ? resourcePoolData.status : false;

    let resourcePoolSwitchBtn = (
      <span>
        <Switch
          size="small"
          checked={resourcePoolActive}
          disabled={!editable}
          onChange={resourcePoolActive ? () => {} : this.onResourcePoolActiveChange}
        />
      </span>
    );
    if (resourcePoolActive && editable) {
      resourcePoolSwitchBtn = (
        <PopConfirm
          content="关闭后，该资源池数据将不删除但不可使用，是否确定关闭？"
          onOk={() => this.onResourcePoolActiveChange(false)}
          getPopupContainer={() => this.$wrap}
        >
          {resourcePoolSwitchBtn}
        </PopConfirm>
      );
    }

    return (
      <Page loading={loading} route={this.props.route} hasBackground={false} title={false} fitWindowHeight={true}>
        <div className={styles.wrap} ref={(c) => (this.$wrap = ReactDOM.findDOMNode(c))}>
          <div className={styles.left}>
            <Left
              data={treeData}
              onSearch={this.onSearch}
              isSearchMode={isSearchMode}
              onlyBossChecked={onlyBossChecked}
              onOnlyBossChange={this.onOnlyBossChange}
              expandedKeys={expandedKeys}
              onExpandChange={this.onExpandChange}
              checkedKeys={checkedKeys}
              onCheckChange={this.onCheckChange}
              selectedKey={selectedNode.id}
              onSelectChange={this.onSelectChange}
              selectedNodeName={selectedNode ? selectedNode.name : null}
            />
          </div>
          <div className={styles.right}>
            <EditBlock
              title={'基本信息'}
              editable={editable}
              editing={basicEditing}
              onStartEdit={() => this.setState({ basicEditing: true, basicEditingData: basicData })}
              onSave={this.onSaveBasic}
              onCancel={this.onCancelBasic}
              content={
                <BasicContent
                  editing={basicEditing}
                  data={basicEditing ? basicEditingData : basicData}
                  checking={basicChecking}
                  onChange={(data) => this.setState({ basicEditingData: data })}
                  onErrorChange={(key, e) => (this.basicErrors[key] = e)}
                  orgTypeDict={hrmsOrgTypeDict}
                />
              }
              ref={(c) => (this.basicBlockRef = c)}
            />
            <EditBlock
              title={
                <span>
                  <span>地址电话信息</span>
                  <span style={{ marginLeft: 15, position: 'relative', top: -2 }}>{contractSwitchBtn}</span>
                </span>
              }
              editable={contractActive && editable}
              editing={contractEditing}
              onStartEdit={() => this.setState(() => ({ contractEditingData: contractData, contractEditing: true }))}
              onSave={this.onSaveContract}
              onCancel={this.onCancelContract}
              content={
                contractData.active ? (
                  <ContractContent
                    editing={contractEditing}
                    data={contractEditing ? contractEditingData : contractData}
                    checking={contractChecking}
                    onChange={(data) => this.setState(() => ({ contractEditingData: data }))}
                    onErrorChange={(key, e) => (this.contractErrors[key] = e)}
                    provinceCityDict={provinceCityDict}
                    provinceDict={provinceDict}
                  />
                ) : null
              }
              ref={(c) => (this.contractBlockRef = c)}
            />
            {selectedNode.bossOrgType !== BOSS_ORG_TYPE_GROUP && selectedNode.isChecked && (
              <EditBlock
                title={
                  <span>
                    <span>资源池设置</span>
                    <span style={{ marginLeft: 15, position: 'relative', top: -2 }}>{poolSwitchBtn}</span>
                  </span>
                }
                editable={poolActive && editable}
                editing={poolEditing}
                onStartEdit={() => this.setState({ poolEditing: true, poolEditingData: poolData })}
                onSave={this.onSavePool}
                onCancel={this.onCancelPool}
                content={
                  poolData.active ? (
                    <PoolContent
                      editing={poolEditing}
                      data={poolEditing ? poolEditingData : poolData}
                      checking={poolChecking}
                      onChange={(data) => this.setState({ poolEditingData: data })}
                      onErrorChange={(key, e) => (this.poolErrors[key] = e)}
                      backflowNodeDict={backflowNodeDict}
                      roleDict={roleDict}
                      bossOrgType={selectedNode.bossOrgType}
                    />
                  ) : null
                }
                ref={(c) => (this.poolBlockRef = c)}
              />
            )}
            {isBranch(selectedNode.bossOrgType) && selectedNode.isChecked && (
              <EditBlock
                title={
                  <span>
                    <span>外呼资源池设置</span>
                    <span style={{ marginLeft: 15, position: 'relative', top: -2 }}>{resourcePoolSwitchBtn}</span>
                  </span>
                }
                editable={resourcePoolActive && editable}
                editing={resourcePoolEditing}
                onStartEdit={() =>
                  this.setState({ resourcePoolEditing: true, resourcePoolEditingData: resourcePoolData })
                }
                onSave={this.onSaveResourcePool}
                onCancel={this.onCancelResourcePool}
                content={
                  resourcePoolData && resourcePoolData.status ? (
                    <ResourcePool
                      editing={resourcePoolEditing}
                      data={resourcePoolEditing ? resourcePoolEditingData : resourcePoolData}
                      checking={resourcePoolChecking}
                      onChange={(data) => this.setState({ resourcePoolEditingData: data })}
                      onErrorChange={(key, e) => (this.resourcePoolErrors[key] = e)}
                      treeData={resourceBackFlowNodeDict}
                      roleDict={roleDict}
                      bossOrgType={selectedNode.bossOrgType}
                      backflowNodeName={selectedNode.name}
                    />
                  ) : null
                }
                ref={(c) => (this.resourcePoolBlockRef = c)}
              />
            )}
          </div>
        </div>
      </Page>
    );
  }
}

export default OrgTree;
