import {
  getBossOrganizationList,
  getOrganizationById,
  getOrganizationAppendInfoById,
  saveBossOrganizationInfo,
  findResourcePoolByIdNew,
  getOrganizationListForResPool,
  getResourcePoolRoleList,
  listResourcePools,
} from 'SERVICE/system';
import { getRoleForSelection } from 'SERVICE/common';
import { formateDict } from 'UTIL/dict';
import dicts from './common';
// import defImg from './def.png'
// import img1 from './1.png'
// import img2 from './2.png'
// import img3 from './3.png'
// import img4 from './4.png'
import { PROJECT_NAME } from 'CONST/config';

let SPARK_ID = '';
switch (PROJECT_NAME) {
  case 'peiyou':
    SPARK_ID = '15460101177476096';
    break;
  case 'twoteacher':
    SPARK_ID = '15460184888443904';
    break;
  default:
    SPARK_ID = '15460069240996864';
}

const isBoss = (type) => {
  return type === 'BOSS';
};
const isValid = (status) => {
  return status === 0 ? true : false;
};
const loadInitOrgList = async (type) => {
  let params;
  if (isBoss(type)) {
    params = { type, level: 2 };
  } else {
    params = { level: 3, id: SPARK_ID, parentId: SPARK_ID };
  }
  const apiData = await getBossOrganizationList(params, true);
  let checkedKeys = [];
  const platData = apiData.data.map((d) => {
    const isChecked = d.bossUse === 0;
    const id = d.hrms_id;
    if (isChecked) {
      checkedKeys.push(id);
    }
    return {
      id,
      bossId: d.bossId,
      name: d.name,
      valid: isValid(d.status),
      hasChildren: d.hasChildren > 0 ? true : false,
      isRoot: !isBoss(type) && d.id === SPARK_ID ? true : d.parentId ? false : true,
      orgType: d.orgType,
      isChecked,
    };
  });
  const root = platData.find((d) => d.isRoot);
  const children = platData.filter((d) => !d.isRoot);
  return {
    data: [{ ...root, children, hasChildren: root.hasChildren > 0 ? true : false }],
    checkedKeys,
    platData,
  };
};

const loadOrgListByParentId = async ({ parentId, type }) => {
  const params = { parentId, type };
  const apiData = await getBossOrganizationList(params, true);
  let checkedKeys = [];
  const data = apiData.data.map((d) => {
    const isChecked = d.bossUse === 0;
    const id = d.hrms_id;
    if (isChecked) {
      checkedKeys.push(id);
    }
    return {
      id,
      bossId: d.bossId,
      name: d.name,
      valid: isValid(d.status),
      hasChildren: d.hasChildren > 0 ? true : false,
      orgType: d.orgType,
      isChecked,
    };
  });
  return { data, checkedKeys };
};

const loadOrgListByName = async ({ name }) => {
  const params = { name };
  const apiData = await getBossOrganizationList(params, true);
  let checkedKeys = [];
  const data = apiData.data.map((d) => {
    const isChecked = d.bossUse === 0;
    const id = d.hrms_id;
    const bossId = d.bossId;
    if (isChecked) {
      checkedKeys.push(id);
    }
    return { id, bossId, name: d.name, valid: isValid(d.status), hasChildren: false, orgType: d.orgType, isChecked };
  });
  return { data, checkedKeys };
};

const loadBasicInfo = async ({ id }) => {
  try {
    const params = { id };
    const { data } = await getOrganizationById(params, true);
    return {
      id: data.hrmsId,
      bossId: data.bossId,
      oldName: data.hrmsName,
      name: data.bossName,
      belong: data.parentName,
      sourceProtectiveDay: data.sourceProtectiveDay,
      type: data.orgType,
      subType: data.subOrgType + '',
      valid: isValid(data.status) ? dicts.STATUS_VALID : dicts.STATUS_INVALID,
      isBoss: data.bossUse === 0 ? dicts.YES_NO_TRUE : dicts.YES_NO_FALSE,
      remark: data.remark,
      validBoolean: isValid(data.status),
      isChecked: data.bossUse === 0,
      bossOrgType: data.bossOrgType,
    };
  } catch (e) {
    return {};
  }
};

const loadContractInfo = async ({ id }) => {
  try {
    const params = { id };
    const { data } = await getOrganizationAppendInfoById(params, true);
    const imgList = data.fileList.map((d) => {
      return {
        id: d.id,
        src: d.filePath,
      };
    });
    return {
      active: data.hasAppend === 1 ? false : true,
      province: data.province,
      city: data.cityId,
      phone: data.contact,
      headPerson: data.contactUser,
      address: data.address,
      lon: data.lon,
      lat: data.lat,
      traffic: data.trafficInfo,
      imgList,
    };
  } catch (e) {
    return {};
  }
};

const loadPoolInfo = async ({ bossId }) => {
  if (!bossId) {
    return {};
  }

  try {
    const [poolInfoApi, list] = await Promise.all([
      findResourcePoolByIdNew({ organizationId: bossId }, true),
      getResourcePoolRoleList({ organizationId: bossId }, true),
    ]);
    const roleList = list.data.map((li) => {
      return { name: li.roleId, max: li.oneTimeResource };
    });
    const poolInfo = poolInfoApi.data;
    return {
      active: poolInfo.status === 'VALID' ? true : false,
      poolName: poolInfo.name,
      volume: poolInfo.capacity,
      circType: poolInfo.cycleType,
      backflowNode: poolInfo.returnNote,
      cycle: poolInfo.recoveyPeriod,
      roleList,
    };
  } catch (e) {
    return {};
  }
  /*
  return {
    active: true,
    name: '嘻嘻嘻',
    volume: 10,
    circType: dicts.CIRC_TYPE_1,
    backflowNode: 'xx',
    cycle: 7,
    roleList: [
      { name: 'xx', max: 100 },
      { name: 'yy', max: 200 },
    ],
  } */
};

const loadDetail = async ({ id, bossId }) => {
  const [basicData, contractData, poolData] = await Promise.all([
    loadBasicInfo({ id }),
    loadContractInfo({ id }),
    loadPoolInfo({ bossId }),
  ]);
  return { basicData, contractData, poolData };
};

const saveDetail = async (data) => {
  try {
    const fileList = data.imgList.map((f) => {
      let info = {
        fileName: '',
        filePath: f.src,
        organizationId: data.bossId,
      };
      if (f.id) {
        info.id = f.id;
      }
      return info;
    });
    const params = {
      address: data.address,
      cityId: data.city,
      contact: data.phone,
      contactUser: data.headPerson,
      fileList,
      hasAppend: data.active ? 0 : 1,
      id: data.bossId,
      lat: data.lat,
      lon: data.lon,
      name: data.name,
      sourceProtectiveDay: data.sourceProtectiveDay,
      provinceId: data.province,
      remark: data.remark,
      trafficInfo: data.traffic,
      modifyType: data.modifyType,
      resourcePoolVo: {
        status: data.active ? 'VALID' : 'INVALID',
        capacity: data.volume,
        cycleType: data.circType,
        name: data.poolName,
        organizationId: data.bossId,
        recoveyPeriod: data.cycle,
        returnNote: data.backflowNode,
        roleVos: data.roleList
          ? data.roleList.map((li) => {
              return { oneTimeResource: li.max, roleId: li.name, organizationId: data.bossId };
            })
          : undefined,
      },
    };

    await saveBossOrganizationInfo(params, true);
    return true;
  } catch (e) {
    throw e;
  }
};

const loadBackflowNodeDict = async (bossId) => {
  if (!bossId) {
    return [];
  }
  const data = await getOrganizationListForResPool({ organizationId: bossId }, true);
  return formateDict(data);
};

const loadRoleDict = async () => {
  //return []
  const data = await getRoleForSelection({}, true);
  return formateDict(data);
};

function formatResourcePoolsDict(arr = [], pid = null) {
  if (arr.length === 0) {
    return [];
  }

  const currentLevelList = [];
  const arrCopy = [...arr];
  for (let i = 0; i < arrCopy.length; i++) {
    const item = arrCopy[i];
    item.label = item.nodeName;
    item.value = item.nodeId;
    item.key = item.nodeId;
    if (item.parentId == pid) {
      currentLevelList.push(item);
      arr.splice(i, 1);
    }
  }

  for (let pItem of currentLevelList) {
    pItem.children = formatResourcePoolsDict(arr, pItem.nodeId);
  }

  return currentLevelList;
}

const listResourcePoolsDict = async (organizationId, resourceType) => {
  const { data = [] } = await listResourcePools({ organizationId, resourceType, withUser: true });
  return formatResourcePoolsDict(data);
};

export default {
  listResourcePoolsDict,
  loadInitOrgList,
  loadOrgListByParentId,
  loadOrgListByName,
  loadBasicInfo,
  loadContractInfo,
  loadPoolInfo,
  loadDetail,
  saveDetail,
  loadBackflowNodeDict,
  loadRoleDict,
};
