import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import Block from 'COMPONENT/Block';
import List from 'COMPONENT/List';
import FormItem from 'COMPONENT/FormItem';
import Input from 'COMPONENT/Input';
import Button from 'COMPONENT/Button';
import { Row, Col } from 'antd';
import loaders from './loaders';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

class ParamsManageDetail extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: true,
      listLoading: true,
      dataSource: [],
      currentSystemParams: {},
      value: '',
    };
  }

  search = async () => {
    this.setState({
      loading: true,
      listLoading: true,
    });
    const {
      params: { type },
    } = this.props;
    const res = await loaders.getSystemParameterInfoByType({ type });
    this.setState({
      loading: false,
      listLoading: false,
      currentSystemParams: {
        id: res.data.id,
        type: res.data.type,
        name: res.data.name,
        value: res.data.value,
      },
      dataSource: res.data.logs || [],
      value: res.data.value,
    });
    return;
  };

  save = async () => {
    try {
      const {
        params: { type, onSaveSuccess },
      } = this.props;
      const { value } = this.state;
      if (!value) {
        return showMessage('请设置参数', 'error');
      }
      this.setState({ loading: true });
      const res = await loaders.updateSystemParameter({ type, value });
      if (res.resultCode != 0) {
        showMessage(res.resultMessage, 'error');
      } else {
        showMessage('设置成功', 'success');
        onSaveSuccess();
      }
    } catch (err) {
      console.log(err);
    }
    this.setState({ loading: false });
  };

  componentDidMount() {
    this.search();
  }

  render() {
    const columns = [
      { title: '时间', dataIndex: 'createTime' },
      { title: '操作详情', dataIndex: 'message' },
      { title: '操作人', dataIndex: 'userName' },
    ];

    const {
      loading,
      dataSource,
      listLoading,
      value,
      currentSystemParams: { type, name, value: curValue },
    } = this.state;

    return (
      <Page loading={loading} route={this.props.route}>
        <Block title="参数详情">
          <Row>
            <Col span={6}>
              <FormItem name="参数名称">{name}</FormItem>
            </Col>
            <Col span={6}>
              <FormItem name="参数类型">{type}</FormItem>
            </Col>
          </Row>
          <Row>
            <Col span={6}>
              <FormItem name="参数值">
                <Input
                  placeholder={curValue}
                  value={value}
                  onChange={(e) => this.setState({ value: e.target.value })}
                />
              </FormItem>
            </Col>
            <Col offset={4} span={6}>
              <Button type="primary" onClick={this.save} disabled={value === curValue || !value}>
                {'保存'}
              </Button>
            </Col>
          </Row>
        </Block>
        <Block title="操作流水">
          <List dataSource={dataSource} columns={columns} loading={listLoading} />
        </Block>
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    params: state.main.pageParams['/ParamsManage/Detail'],
  };
};
export default connect(mapStateToProps)(ParamsManageDetail);
