import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import Block from 'COMPONENT/Block';
import List from 'COMPONENT/List';
import loaders from './loaders';

class ParamsManage extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: true,
      listLoading: true,
      dataSource: [],
    };
  }

  onSaveSuccess = () => {
    const { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: { path: '/ParamsManage' },
    });
    this.search();
  };

  goDetail = (id, type) => {
    const { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: {
        path: '/ParamsManage/Detail',
        params: {
          type,
          id,
          onSaveSuccess: this.onSaveSuccess,
        },
        refresh: true,
      },
    });
  };

  search = async () => {
    this.setState({
      loading: true,
      listLoading: true,
    });
    const res = await loaders.listAllSystemParameter();
    this.setState({
      loading: false,
      listLoading: false,
      dataSource: res,
    });
  };

  componentDidMount() {
    this.search();
  }

  render() {
    const columns = [
      {
        title: '系统参数',
        dataIndex: 'name',
      },
      { title: '参数类型', dataIndex: 'type' },
      { title: '参数值', dataIndex: 'value' },
      {
        title: '操作',
        key: 'action',
        getActionConfig: (val, row, i) => {
          return [{ text: '详情', onClick: () => this.goDetail(row.id, row.type) }];
        },
      },
    ];

    const { loading, dataSource, listLoading } = this.state;

    return (
      <Page loading={loading} route={this.props.route}>
        <Block title="系统参数列表">
          <List dataSource={dataSource} columns={columns} loading={listLoading} />
        </Block>
      </Page>
    );
  }
}

export default connect()(ParamsManage);
