import React from 'react';
import { Tree } from 'antd';
import SearchInput from '../SearchInput';
import styles from './styles.less';

const TreeNode = Tree.TreeNode;

class AuthTree extends React.Component {
  state = {
    filteredTreeData: this.props.treeData,
    checked: [],
    halfChecked: [],
    expandedKeys: [],
  };

  componentWillReceiveProps(nextProps) {
    const { checked, treeData } = nextProps;
    const { filteredTreeData } = this.state;
    this.setState(this.getDetailCheckedFiltered(checked, treeData, filteredTreeData));
  }

  onSearchChange = (searchKey) => {
    const { checked, treeData } = this.props;
    const filteredTreeData = this.getFilteredData(treeData, searchKey);
    let expandedKeys = [];
    if (searchKey) {
      expandedKeys = filteredTreeData
        .filter((d) => filteredTreeData.some((da) => da.parentId === d.id))
        .map((dat) => dat.id);
    }
    this.setState({
      filteredTreeData,
      ...this.getDetailCheckedFiltered(checked, treeData, filteredTreeData),
      expandedKeys,
    });
  };

  getDetailCheckedFiltered = (checkedProp, treeData, filteredTreeData) => {
    const allDetailChecked = this.getDetailChecked(treeData, checkedProp);
    const checked = allDetailChecked.checked.filter((ck) => filteredTreeData.some((d) => d.id === ck));
    const halfChecked = allDetailChecked.halfChecked.filter((ck) => filteredTreeData.some((d) => d.id === ck));
    return { checked, halfChecked };
  };

  getDetailChecked = (treeData, checked) => {
    const halfChecked = checked.filter((ck) => {
      const allChildren = this.getAllChildren(
        treeData,
        treeData.find((td) => td.id === ck),
      );
      if (allChildren.length === 0) {
        return false;
      }
      return allChildren.some((ac) => checked.indexOf(ac.id) < 0);
    });
    return { checked: checked.filter((ck) => halfChecked.indexOf(ck) < 0), halfChecked };
  };

  getFilteredData = (treeData, searchKey) => {
    const filteredTreeData = treeData.filter((da) => {
      if (this.isMatchSearch(da, searchKey)) {
        return true;
      }
      const allChildren = this.getAllChildren(treeData, da);
      return allChildren.some((ac) => this.isMatchSearch(ac, searchKey));
    });
    return filteredTreeData;
  };

  getAllChildren = (data, item) => {
    if (!item) return [];

    let allChildren = [];
    const children = data.filter((da) => da.parentId === item.id);
    if (children && children.length > 0) {
      allChildren = [...allChildren, ...children];
      children.forEach((ch) => {
        allChildren = [...allChildren, ...this.getAllChildren(data, ch)];
      });
    }
    return allChildren;
  };

  isMatchSearch = (item, key) => {
    return !key || (item && item.name && item.name.indexOf(key) >= 0);
  };

  getStructruedTreeData = (allData, data, parentId) => {
    if (!data || data.length === 0) {
      return [];
    }
    let treeData = [];
    data.forEach((da) => {
      if (da.parentId === parentId) {
        const childrenData = allData.filter((d) => {
          return d.parentId === da.id;
        });
        treeData.push({
          id: da.id,
          name: da.name,
          isOff: da.isOff,
          parentId,
          children: this.getStructruedTreeData(allData, childrenData, da.id),
        });
      }
    });
    return treeData;
  };

  getTreeNode = (data) => {
    const { editable, isXiaoApp } = this.props;
    const { id, name, children, isOff, parentId } = data;
    const isRootNode = parentId === -1 || !parentId;
    const disableRootNodeCheckable = (isRootNode && isXiaoApp) || !editable; // 晓APP 一级菜单权限由OA维护
    const title = (
      <span>
        {name}
        {isOff && <span className={styles.isOff}>（已下架）</span>}
      </span>
    );
    if (children && children.length > 0) {
      return (
        <TreeNode key={id} title={title} disableCheckbox={disableRootNodeCheckable}>
          {children.map((d) => this.getTreeNode(d))}
        </TreeNode>
      );
    } else {
      return <TreeNode key={id} title={title} disableCheckbox={disableRootNodeCheckable} />;
    }
  };

  onCheck = (detailChecked) => {
    const { onCheckChange, treeData, checked: checkedProp } = this.props;
    const { checked: checkedState } = this.state;
    const { checked } = detailChecked;
    let newChecked = [...checkedProp];
    let checkId;
    let checkType;
    if (checked.length > checkedState.length) {
      checkId = checked.find((ck) => checkedState.indexOf(ck) < 0);
      checkType = 'check';
      if (newChecked.indexOf(checkId) < 0) {
        newChecked.push(checkId);
      }
      const match = checkId.match(/^([A-Z\_]*STU\_MODAL\_[A-Z]+)\_EDIT/);
      if (match && match[1]) {
        if (newChecked.indexOf(match[1]) < 0) {
          newChecked.push(match[1]);
        }
        if (newChecked.indexOf(`${match[1]}_VISIT`) < 0) {
          newChecked.push(`${match[1]}_VISIT`);
        }
      }
    } else {
      checkId = checkedState.find((ck) => checked.indexOf(ck) < 0);
      checkType = 'unCheck';
      newChecked = newChecked.filter((ck) => ck !== checkId);
    }
    const allChildren = this.getAllChildren(
      treeData,
      treeData.find((d) => d.id === checkId),
    );
    if (checkType == 'check') {
      allChildren.forEach((c) => {
        if (newChecked.indexOf(c.id) < 0) {
          newChecked.push(c.id);
        }
      });
    } else {
      newChecked = newChecked.filter((ck) => !allChildren.some((c) => c.id === ck));
    }
    const allParentIds = this.getAllParentIds(treeData, checkId);
    allParentIds.forEach((pId) => {
      if (!newChecked.some((ck) => ck === pId)) {
        newChecked = newChecked.filter((ck) => ck !== pId);
        const checkStatus = this.getCheckStatus(treeData, newChecked, pId);
        if (checkStatus === 'check' || checkStatus === 'halfCheck') {
          newChecked.push(pId);
        }
      }
    });
    onCheckChange(newChecked);
  };

  getCheckStatus = (treeData, checked, id) => {
    const allChildren = this.getAllChildren(
      treeData,
      treeData.find((d) => d.id === id),
    );
    if (allChildren.length === 0) {
      //目前不会进入这种情况，暂时不处理
      return null;
    }
    const childrenChecked = allChildren.filter((c) => checked.some((ck) => ck === c.id));
    if (childrenChecked.length === 0) {
      return 'unCheck';
    } else if (childrenChecked.length === allChildren.length) {
      return 'check';
    } else {
      return 'halfCheck';
    }
  };

  getAllParentIds = (treeData, id) => {
    let allParIds = [];
    const item = treeData.find((da) => da.id === id);
    let parentId = item.parentId;
    while (parentId) {
      allParIds.push(parentId);
      const par = treeData.find((da) => da.id === parentId);
      parentId = par.parentId;
    }
    return allParIds;
  };

  render() {
    const { checked, halfChecked, filteredTreeData, expandedKeys } = this.state;
    return (
      <div className={styles.wrap}>
        <SearchInput onChange={this.onSearchChange} placeholder="请输入名称查询" />
        {this.props.isXiaoApp && (
          <p className={styles.tip}>晓助手BOSSⅠ级应用菜单权限在OA系统维护，Ⅱ级应用菜单权限在BOSS系统维护</p>
        )}
        <Tree
          checkable
          checkStrictly={true}
          checkedKeys={{ checked, halfChecked }}
          onCheck={this.onCheck}
          expandedKeys={expandedKeys}
          onExpand={(expandedKeys) => this.setState({ expandedKeys })}
        >
          {this.getStructruedTreeData(filteredTreeData, filteredTreeData, null).map((d) => this.getTreeNode(d))}
        </Tree>
      </div>
    );
  }
}

export default AuthTree;
