import React from 'react';
import { connect } from 'dva';
import { Row, Col } from 'antd';
import loaders from '../loaders';
import Block from 'COMPONENT/Block';
import Page from 'COMPONENT/Page';
import TextArea from 'COMPONENT/TextArea';
import FormItem from 'COMPONENT/FormItem';
import styleVars from 'COMPONENT/theme/variables';
import styles from './styles.less';
import Select from 'COMPONENT/Select';
import Input from 'COMPONENT/Input';
import Button from 'COMPONENT/Button';
import { getDictTextByVal } from 'UTIL/dict';
import Validator from 'COMPONENT/Validator';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import { FIELD_FILLER, checkFieldEmpty, getFieldWithFiller, getCommonErrorTips } from 'COMPONENT/utils/form';
import Tabs from 'COMPONENT/Tabs';
import { objectDeepEquals } from 'COMPONENT/utils/basic/obj';
import AuthTree from '../AuthTree';
import CustomerInfo from '../CustomerInfo';
import dicts from '../dicts';
import OperationRecord from '../OperationRecord';
const Tab = Tabs.Tab;

const blockStyle = {
  margin: ' -5px 20px 10px 60px',
};

const rowCfg = {
  style: { marginLeft: -5 },
  gutter: 50,
};

class RoleDetail extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 页面加载状态

      id: props.id, // {string} 角色ID
      mode: props.mode, // {string} 页面模式 'Add'|'Modify'|'View'|'Copy'

      name: null, // {string} 角色名称
      type: null, // {string} 角色类型
      level: null, // {string} 角色等级
      status: null, // {string} 状态
      remark: '', // {string} 备注

      activeTabKey: 'auth', // {string} 权限与附加信息的当前标签页，'auth'|'customer'

      treeCheckedInit: [], //  {object[]} 初始的BOSS权限树选中项，用于检测是否有修改
      handTreeCheckedInit: [], //  {object[]} 初始的掌上BOSS权限树选中项，用于检测是否有修改
      treeChecked: [], //  {object[]} BOSS权限树选中项
      handTreeChecked: [], //  {object[]} 掌上BOSS权限树选中项

      customerInfoInit: {}, //  {Object} 初始的客户资源相关设置信息，用于检测是否有修改
      customerInfo: {}, //  {Object} 客户资源相关设置信息
      logs: [], // 日志数据
      /**
        active {boolean} - 激活状态
        volume {number} - 资源容量
        circType {string} - 流转方式
        cycle {number} - 回收周期
        auths {string[]} - 资源分配权限
      */

      checking: false, // {boolean}  校验开启状态，
      adjustReason: '', // 调整原因
    };
    this.errors = {}; //{boolean|string} 校验错误信息
  }

  componentDidMount() {
    const { mode } = this.state;
    if (mode === 'Add') {
      this.setState({ status: dicts.STATUS_VAILD });
    } else {
      this.initContent();
    }
  }

  initContent = () => {
    const { id, mode } = this.state;
    this.setState({ loading: true });
    Promise.all([loaders.loadDetail(id)]).then(([{ basicInfo, treeChecked, handTreeChecked, customerInfo, logs }]) => {
      let basic = { ...basicInfo };
      if (mode === 'Copy') {
        basic['name'] = null;
        basic['status'] = dicts.STATUS_VAILD;
      }
      this.setState({
        loading: false,
        ...basic,
        treeChecked,
        handTreeChecked,
        customerInfo,
        logs,
        treeCheckedInit: treeChecked,
        handTreeCheckedInit: handTreeChecked,
        customerInfoInit: customerInfo,
      });
    });
  };

  errorChange = (key, error) => {
    this.errors[key] = error;
  };

  save = () => {
    const { id, mode } = this.state;
    const { checking, treeChecked, handTreeChecked, name, type, level, status, remark, customerInfo, adjustReason } =
      this.state;
    const opReason = adjustReason.trim();
    if (!checking) {
      this.setState({ checking: true });
    }
    const errorTips = getCommonErrorTips(this.errors);

    if (errorTips) {
      return showMessage(errorTips);
    }
    if (treeChecked.length === 0 && handTreeChecked.length === 0) {
      return showMessage('请勾选页面权限');
    }

    if (!opReason || opReason.length > 200) {
      return showMessage('请输入调整原因，不超过200个字符');
    }

    this.setState({ loading: true });
    loaders
      .saveDetail({
        id: mode === 'Copy' ? null : id,
        mode,
        treeChecked,
        handTreeChecked,
        name,
        type,
        level,
        status,
        remark,
        customerInfo,
        opReason,
      })
      .then(
        (data) => {
          showMessage('保存成功', 'success');
          this.setState({
            loading: false,
            treeCheckedInit: treeChecked,
            handTreeCheckedInit: handTreeChecked,
            customerInfoInit: customerInfo,
            adjustReason: '',
          });
          if (mode === 'Modify') {
            this.initContent();
          }
          if (mode === 'Add') {
            const roleId = data ? data.roleId : null;
            if (roleId) {
              this.setState({ id: roleId, mode: 'Modify' });
              this.initContent();
            }
          }
          // 当前非复制操作的角色为当前用户角色(之一)
          if (mode !== 'Copy' && Index.getLoginUser().roleId.indexOf(id) !== -1) {
            // 更新权限配置(不用整个页面重新刷新)
            this.props.dispatch({
              type: 'account/updateBtnAuth',
              payload: {},
            });
            this.props.dispatch({
              type: 'main/loadMenus',
              payload: {},
            });
          }
        },
        (e) => {
          this.setState({ loading: false });
          if (e.text) {
            showMessage(e.text);
          } else {
            showMessage('保存失败');
          }
        },
      );
  };

  onCustomerActiveChange = (active) => {
    const { id, mode } = this.state;
    this.setState({ loading: true });
    loaders.setCustomerStatus(id, active).then(
      () => {
        showMessage(active ? '已激活成功' : '已关闭激活', 'success');
        this.setState({ loading: false, customerInfo: { ...this.state.customerInfo, active } });
        if (!active) {
          this.errors['volume'] = null;
          this.errors['circType'] = null;
          this.errors['cycle'] = null;
          this.errors['auths'] = null;
        }
      },
      (e) => {
        this.setState({ loading: false });
        showMessage(e.text || '设置失败');
      },
    );
  };

  tabChange = (key) => {
    const { treeCheckedInit, treeChecked, handTreeCheckedInit, handTreeChecked, customerInfoInit, customerInfo } =
      this.state;
    if (
      (key === 'customer' &&
        (!objectDeepEquals(treeCheckedInit, treeChecked) || !objectDeepEquals(handTreeCheckedInit, handTreeChecked))) ||
      (key === 'auth' && !objectDeepEquals(customerInfoInit, customerInfo))
    ) {
      showConfirm('当前页面有更新，是否保存？', () => {
        this.save();
      });
      this.setState({ activeTabKey: key === 'auth' ? 'customer' : 'auth' });
    } else {
      this.setState({ activeTabKey: key });
    }
  };

  render() {
    const { treeData, handTreeData, typeDict, levelDict, statusDict, roleDict } = this.props;
    const {
      loading,
      id,
      mode,
      checking,
      treeChecked,
      handTreeChecked,
      name,
      type,
      level,
      status,
      remark,
      customerInfo,
      activeTabKey,
      logs,
      adjustReason,
    } = this.state;

    const isView = mode === 'View';
    const formCfg = {
      size: isView ? 'small' : undefined,
      style: { padding: '0 5px' },
    };
    let title;
    if (isView) {
      title = '角色详情';
    } else if (mode === 'Modify') {
      title = '修改角色';
    } else if (mode === 'Copy') {
      title = '复制角色';
    } else if (mode === 'Add') {
      title = '新增角色';
    }

    const renderFooter = (
      <div className={styles.footer}>
        <FormItem name="调整原因" required className={styles.reason}>
          <TextArea
            placeholder="请输入调整原因"
            value={adjustReason}
            wordCount
            maxLength={200}
            rows={2}
            onChange={(e) => {
              this.setState({ adjustReason: e.target.value });
            }}
          />
        </FormItem>
        <div className={styles.operation}>
          <Button type="primary" onClick={this.save}>
            确定
          </Button>
          <Button onClick={() => history.back()}>取消</Button>
        </div>
      </div>
    );

    return (
      <Page
        className={styles.pageContainer}
        loading={loading}
        route={{ ...this.props.route, name: title }}
        footer={isView ? null : renderFooter}
        ref={(c) => (this.pageRef = c)}
      >
        <Block title="基本信息" style={blockStyle}>
          <Row {...rowCfg}>
            <Col span={12}>
              <FormItem {...formCfg} name="角色名称" required={isView ? null : true}>
                {isView ? (
                  <div>{getFieldWithFiller(name)}</div>
                ) : (
                  <Input
                    value={name}
                    onChange={(e) => this.setState({ name: e.target.value })}
                    validator={{
                      onErrorChange: (e) => this.errorChange('name', e),
                      silent: !checking,
                    }}
                  />
                )}
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem {...formCfg} name="角色类型" required={isView ? null : true}>
                {isView ? (
                  <div>{getDictTextByVal(typeDict, type, true)}</div>
                ) : (
                  <Select
                    dict={typeDict}
                    value={type}
                    onChange={(val) => this.setState({ type: val })}
                    validator={{
                      onErrorChange: (e) => this.errorChange('type', e),
                      silent: !checking,
                    }}
                  />
                )}
              </FormItem>
            </Col>
          </Row>
          <Row {...rowCfg}>
            <Col span={12}>
              <FormItem {...formCfg} name="角色等级" required={isView ? null : true}>
                {isView ? (
                  <div>{getDictTextByVal(levelDict, level, true)}</div>
                ) : (
                  <Select
                    dict={levelDict}
                    value={level}
                    onChange={(val) => this.setState({ level: val })}
                    validator={{
                      onErrorChange: (e) => this.errorChange('level', e),
                      silent: !checking,
                    }}
                  />
                )}
              </FormItem>
            </Col>
            <Col span={12}>
              <FormItem {...formCfg} name="状态">
                {getDictTextByVal(statusDict, status, true)}
              </FormItem>
            </Col>
            <Col span={24}>
              <FormItem {...formCfg} name="备注">
                {isView ? (
                  <div>{getFieldWithFiller(remark)}</div>
                ) : (
                  <TextArea value={remark} onChange={(e) => this.setState({ remark: e.target.value })} />
                )}
              </FormItem>
            </Col>
          </Row>
        </Block>
        <Block title="权限与附加信息" style={blockStyle}>
          <Tabs navAlign="center" className={styles.tabs} activeKey={activeTabKey} onChange={this.tabChange}>
            <Tab title="页面权限设置" key="auth" value="auth">
              <Row>
                <Col span={12}>
                  <div className={styles.treeTitle}>BOSS</div>
                  <AuthTree
                    treeData={treeData}
                    editable={isView ? false : true}
                    checked={treeChecked}
                    onCheckChange={(val) => this.setState({ treeChecked: val })}
                  />
                </Col>
                <Col span={12}>
                  <div className={styles.treeTitle}>晓助手（BOSS部分）</div>
                  <AuthTree
                    isXiaoApp
                    treeData={handTreeData}
                    editable={isView ? false : true}
                    checked={handTreeChecked}
                    onCheckChange={(val) => this.setState({ handTreeChecked: val })}
                  />
                </Col>
              </Row>
            </Tab>
            <Tab title="客户资源相关设置" className={styles.tab} key="customer" value="customer">
              <CustomerInfo
                data={customerInfo}
                editable={isView ? false : true}
                onChange={(val) => this.setState({ customerInfo: val })}
                checking={checking}
                onErrorChange={this.errorChange}
                roleDict={roleDict}
                onActiveChange={this.onCustomerActiveChange}
              />
            </Tab>
          </Tabs>
        </Block>
        {(mode === 'Modify' || mode === 'View') && (
          <Block title="操作流水" style={blockStyle}>
            <OperationRecord data={logs} />
          </Block>
        )}
      </Page>
    );
  }
}

export default connect()(RoleDetail);
