import React from 'react';
import { Tree } from 'antd';
import SearchInput from 'COMPONENT/Input/SearchInput';
import styles from './styles.less';
/*
data
editable
checkedKeys
onChange
 */
const TreeNode = Tree.TreeNode;

class AuthTree extends React.Component {
  state = {
    searchKey: null,
    hideChecked: [],
    showChecked: [],
  };

  componentWillReceiveProps(nextProps) {
    const { data, checkedKeys } = nextProps;
    this.setState({ ...this.getHideAndShowChecked(data, checkedKeys, this.state.searchKey) });
  }

  onCheck = (val) => {
    const { data, onChange } = this.props;
    const { hideChecked, showChecked } = this.state;
    const moreShowChecked = this.getMoreChecked(val);
    console.log(showChecked, moreShowChecked);
    this.setState({ showChecked: moreShowChecked });
    const allChecked = [...hideChecked, ...moreShowChecked];

    onChange(allChecked);
  };

  getMoreChecked = (checked) => {
    const { data } = this.props;
    let rtnChecked = [];
    checked.forEach((ch) => {
      rtnChecked.push(ch);
      const allParents = this.getAllParents(data, ch);
      allParents.forEach((par) => {
        if (rtnChecked.indexOf(par) < 0) {
          rtnChecked.push(par);
        }
      });
    });
    return rtnChecked;
  };

  getAllParents = (data, id) => {
    let allPars = [];
    const item = data.find((da) => da.id === id);
    let parentId = item.parentId;
    while (parentId) {
      allPars.push(parentId);
      const par = data.find((da) => da.id === parentId);
      parentId = par.parentId;
    }
    return allPars;
  };

  getHideAndShowChecked = (data, allChecked, searchKey) => {
    let hideChecked = [];
    let showChecked = [];
    data.forEach((da) => {
      if (!searchKey || da.name.indexOf(searchKey) >= 0) {
        if (allChecked.indexOf(da.id) >= 0) {
          showChecked.push(da.id);
        }
      } else {
        if (allChecked.indexOf(da.id) >= 0) {
          hideChecked.push(da.id);
        }
      }
    });
    return { hideChecked, showChecked };
  };

  onSearchChange = (e) => {
    const { data } = this.props;
    const { hideChecked, showChecked } = this.state;
    const val = e.target.value;
    const allChecked = [...hideChecked, ...showChecked];

    this.setState({ searchKey: val, ...this.getHideAndShowChecked(data, [...hideChecked, ...showChecked], val) });
  };

  getFullChecked = (data, checked) => {
    return checked.filter((ch) => {
      const allChildren = this.getAllChildren(
        data,
        data.find((da) => da.id === ch),
      );
      if (allChildren.some((c) => checked.indexOf(c.id) < 0)) {
        return false;
      }
      return true;
    });
  };

  getAllChildren = (data, item) => {
    let allChildren = [];
    const children = item.children;
    if (children && children.length > 0) {
      children.forEach((ch) => {
        allChildren = [...allChildren, ...allChildren.push(this.getAllChildren(data, ch))];
      });
    }
    return allChildren;
  };

  getTreeNode = (data) => {
    const { id, name, children } = data;
    if (children && children.length > 0) {
      return (
        <TreeNode key={id} title={name}>
          {children.map((d) => this.getTreeNode(d))}
        </TreeNode>
      );
    } else {
      return <TreeNode key={id} title={name} />;
    }
  };

  getTreeData = (allData, data, parentId) => {
    if (!data || data.length === 0) {
      return [];
    }
    let treeData = [];
    data.forEach((da) => {
      if (da.parentId === parentId) {
        const childrenData = allData.filter((d) => {
          return d.parentId === da.id;
        });
        treeData.push({ id: da.id, name: da.name, children: this.getTreeData(allData, childrenData, da.id) });
      }
    });
    return treeData;
  };

  render() {
    const { data, editable, checkedKeys, onChange } = this.props;
    const { searchKey, showChecked } = this.state;
    console.log(checkedKeys, showChecked);
    const filteredData = searchKey ? data.filter((da) => da.name.indexOf(searchKey) >= 0) : data;
    const treeData = this.getTreeData(filteredData, filteredData, null);
    const fullChecked = this.getFullChecked(filteredData, showChecked);
    console.log(filteredData, showChecked, treeData, fullChecked);
    return (
      <div className={styles.wrap}>
        <SearchInput value={searchKey} onChange={this.onSearchChange} />
        <Tree checkable checkedKeys={fullChecked} onCheck={this.onCheck}>
          {treeData.map((d) => this.getTreeNode(d))}
        </Tree>
      </div>
    );
  }
}

export default AuthTree;
