import React from 'react';
import TextLink from 'COMPONENT/TextLink';
import Select from 'COMPONENT/Select';
import SubBlock from 'COMPONENT/Block/SubBlock';
import { Row, Col } from 'antd';
import FormItem from 'COMPONENT/FormItem';
import styles from './styles.less';

const formItemCfg = {
  size: 'small',
  style: { padding: '0 50px 0 0' },
  required: true,
  labelWidth: 85,
};

class AuthBlock extends React.Component {
  constructor(props) {
    super(props);
  }

  onChange = (field, val) => {
    val = val || {};
    let changed;
    let isOrgChange = false;
    if (field === 'org') {
      isOrgChange = true;
      changed = { org: val.val, orgName: val.text };
    } else {
      changed = { role: val.val, roleName: val.text };
    }
    this.props.onChange(changed, isOrgChange);
  };

  render() {
    const { title, value, checking, onErrorChange, onDelete, sysRoleDict, initOrgDict } = this.props;

    return (
      <SubBlock
        title={title}
        headerToolBar={onDelete ? <TextLink onClick={onDelete}>删除</TextLink> : null}
        className={styles.wrap}
      >
        <Row style={{ marginRight: -70 }}>
          <Col span={12}>
            <FormItem {...formItemCfg} name="组织架构">
              <Select
                dict={initOrgDict}
                value={value.org ? { val: value.org, text: value.orgName } : null}
                onChange={(val) => this.onChange('org', val)}
                showSearch
                validator={{
                  onErrorChange: (e) => onErrorChange('org', e),
                  silent: !checking,
                }}
                textInValue
              />
            </FormItem>
          </Col>
          <Col span={12}>
            <FormItem {...formItemCfg} name="角色">
              <Select
                dict={sysRoleDict}
                value={value.role ? { val: value.role, text: value.roleName } : null}
                onChange={(val) => this.onChange('role', val)}
                showSearch
                validator={{
                  onErrorChange: (e) => onErrorChange('role', e),
                  silent: !checking,
                }}
                textInValue
              />
            </FormItem>
          </Col>
        </Row>
      </SubBlock>
    );
  }
}

export default AuthBlock;
