import React from 'react';
import { connect } from 'dva';
import { Row, Col } from 'antd';
import loaders from '../loaders';
import DropdownMenu from 'COMPONENT/DropdownMenu';
import Block from 'COMPONENT/Block';
import SubBlock from 'COMPONENT/Block/SubBlock';
import Page from 'COMPONENT/Page';
import FormItem from 'COMPONENT/FormItem';
import styleVars from 'COMPONENT/theme/variables';
import List from 'COMPONENT/List';
import styles from './styles.less';
import Select from 'COMPONENT/Select';
import Button from 'COMPONENT/Button';
import { getDictTextByVal } from 'UTIL/dict';
import AuthSettingModal from '../AuthSettingModal';
import Validator from 'COMPONENT/Validator';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;
const formItemCfg = {
  size: 'small',
  style: { padding: '0 50px 0 0' },
};
const blockStyle = {
  margin: ' -5px 20px 10px 60px',
};
const configuredDict = [
  { val: true, text: '需要配置角色' },
  { val: false, text: '不配置角色' },
];

class Detail extends React.Component {
  constructor(props) {
    super(props);

    const {
      params: { initOrgDict },
    } = this.props;
    this.initOrgDictObj = initOrgDict.reduce((obj, ele) => {
      obj[ele.val] = ele.text;
      return obj;
    }, {});
    this.state = {
      loading: false, // {boolean} 内容加载状态

      stationNames: [],
      enableRoleConfig: false,
      stationRoles: [],

      list: [], // {object[]} 操作流水列表数据
      totalRecords: 0, // {number} 总记录数
      currentPage: 1, // {number} 当前页码
      pageSize: 5, // {number} 每页记录数

      authSettingModalVis: false, // {boolean} 权限设置弹框显示状态

      checking: false, // {boolean}  校验开启状态
    };
    this.errors = {}; //{boolean|string} 校验错误信息
  }

  componentDidMount() {
    this.initContent();
  }

  initContent = () => {
    const {
      params: { id },
    } = this.props;
    this.setState({ loading: true });
    Promise.all([loaders.loadUserDetail({ id }), loaders.loadUserDetailLog({ id })]).then(([data, list]) => {
      const stationNames = data.stationName.split(',');
      const enableRoleConfig = data.enableRoleConfig;
      const stationRoles = data.stationRoles;
      this.setState({
        loading: false,
        stationNames,
        enableRoleConfig,
        stationRoles,
        totalRecords: list.length,
        list,
      });
    });
  };

  getDropdown = (title, data) => {
    return (
      <DropdownMenu
        text={`${title}（${data.length}个）`}
        menus={data}
        hasArrow={true}
        color={styleVars.fontColor}
        activeColor={styleVars.primaryColor}
      />
    );
  };

  getPostDropdown = (data) => {
    return this.getDropdown(
      '更多职能',
      data.map((d) => {
        return { text: `${d}` };
      }),
    );
  };

  getAuthDropdown = (data) => {
    return this.getDropdown(
      '更多权限',
      data.map((d) => {
        return { text: `${(this.initOrgDictObj[d.orgType] || d.orgType) + (d.primary ? '(主)' : '')}-${d.roleName}` };
      }),
    );
  };

  errorChange = (key, error) => {
    this.errors[key] = error;
  };

  onOk = () => {
    const {
      params: { id, onSaveSuccess },
    } = this.props;
    const { checking, stationNames, enableRoleConfig, stationRoles } = this.state;
    if (!checking) {
      this.setState({ checking: true });
    }
    if (enableRoleConfig && Object.values(this.errors).find((v) => v)) {
      this.pageRef.showOkTips('请配置角色后再提交');
      this.errors = {};
      return;
    }
    this.setState({ loading: true });
    loaders
      .saveDetail({
        id,
        enableRoleConfig,
        stationName: stationNames.join(','),
        stationRoles: enableRoleConfig ? stationRoles : [],
      })
      .then(
        () => {
          showMessage('保存成功', 'success');
          onSaveSuccess();
        },
        (e) => {
          console.log(e);
          showMessage(e.text || '保存失败');
          this.setState({ loading: false });
        },
      );
  };

  render() {
    const {
      params: { type, initOrgDict, sysRoleDict },
    } = this.props;
    const {
      loading,

      stationNames,
      enableRoleConfig,
      stationRoles,

      list,
      totalRecords,
      currentPage,
      pageSize,
      authSettingModalVis,
      checking,
    } = this.state;

    const columns = [
      { title: '时间', key: 'createTime', dataIndex: 'createTime' },
      { title: '操作详情', key: 'message', dataIndex: 'message', width: COL_WIDTH_XL },
      { title: '操作人', key: 'username', dataIndex: 'username' },
    ];

    const isView = type === 'view';

    return (
      <Page
        loading={loading}
        route={{ ...this.props.route, name: isView ? '查看详情' : '修改角色配置' }}
        footer={isView ? false : true}
        onOk={this.onOk}
        ref={(c) => (this.pageRef = c)}
      >
        <Block title="人事岗位" style={blockStyle}>
          <SubBlock title="岗位信息">
            <Row>
              <Col span={6}>
                <FormItem {...formItemCfg} name="岗位">
                  {stationNames.slice(0, 3).map((stationName) => {
                    return <div key={stationName}>{`${stationName}`}</div>;
                  })}
                  {stationNames.length > 3 && this.getPostDropdown(stationNames.slice(3))}
                </FormItem>
              </Col>
              <Col span={12}>
                <FormItem {...formItemCfg} name="">
                  <Select
                    dict={configuredDict}
                    value={enableRoleConfig}
                    onChange={(val) => this.setState({ enableRoleConfig: val })}
                    disabled={isView}
                    allowClear={false}
                  />
                </FormItem>
              </Col>
            </Row>
          </SubBlock>
        </Block>
        <Block title="权限信息" style={blockStyle}>
          {enableRoleConfig && (
            <SubBlock title="权限设置">
              <Row>
                <Col span={24}>
                  <FormItem {...formItemCfg} name="权限-角色" required={isView ? null : true}>
                    <Validator
                      rules={[{ required: '请配置角色' }]}
                      onErrorChange={(e) => this.errorChange('stationRoles', e)}
                      silent={!checking}
                      value={stationRoles}
                    >
                      <div className={styles.authWrap}>
                        <div className={styles.authWrapLeft}>
                          {stationRoles.slice(0, 3).map((stationRole) => {
                            return (
                              <div key={`${stationRole.roleId}_${stationRole.orgType}`}>
                                {`${
                                  (this.initOrgDictObj[stationRole.orgType] || stationRole.orgType) +
                                  (stationRole.primary ? '(主)' : '')
                                }-${stationRole.roleName}`}
                              </div>
                            );
                          })}
                          {stationRoles.length > 3 && this.getAuthDropdown(stationRoles.slice(3))}
                        </div>
                        {!isView && (
                          <div className={styles.authWrapRight}>
                            <Button onClick={() => this.setState({ authSettingModalVis: true })}>选择</Button>
                          </div>
                        )}
                      </div>
                    </Validator>
                  </FormItem>
                </Col>
              </Row>
            </SubBlock>
          )}
          <SubBlock title="操作流水">
            <div style={{ paddingTop: 10 }}>
              {list.length > 0 ? (
                <List
                  dataSource={list.slice((currentPage - 1) * pageSize, currentPage * pageSize)}
                  columns={columns}
                  pagination={{
                    current: currentPage,
                    total: totalRecords,
                    pageSize,
                    pageSizeOptions: ['5', '10', '20'],
                  }}
                  onChange={({ current, pageSize }) => this.setState({ currentPage: current, pageSize })}
                />
              ) : (
                <div className={styles.tips}>暂无操作流水信息</div>
              )}
            </div>
          </SubBlock>
        </Block>
        {authSettingModalVis && (
          <AuthSettingModal
            data={stationRoles}
            onOk={(data) => this.setState({ authSettingModalVis: false, stationRoles: data })}
            onCancel={() => this.setState({ authSettingModalVis: false })}
            initOrgDict={initOrgDict}
            sysRoleDict={sysRoleDict}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    params: state.main.pageParams['/RoleTemplateManage/Detail'],
  };
};

export default connect(mapStateToProps)(Detail);
