import { getStationRoleList } from 'SERVICE/system';
import { getRoleForSelection } from 'SERVICE/common';
import fetch from 'dva/fetch';
import { urlBase } from 'CONST/config';

const loadList = async (params) => {
  const apiParams = {
    page: params.currentPage,
    rows: params.pageSize,
  };
  if (params.stationName) {
    apiParams['stationName'] = params.stationName;
  }
  if (params.includedRole) {
    apiParams['includedRole'] = params.includedRole;
  }
  if (params.configured === true || params.configured === false) {
    apiParams['configured'] = params.configured ? 'true' : 'false';
  }
  const { rows, records } = await getStationRoleList(apiParams, true);
  return {
    list: rows,
    totalRecords: records,
  };
};

const loadSysRoleDict = async (name) => {
  const params = {};
  const apiData = await getRoleForSelection(params, true);
  const value = apiData ? apiData.value : {};
  let dict = [];
  Object.keys(value).forEach((key) => {
    dict.push({ val: key, text: value[key], title: value[key] });
  });
  return dict;
};

const loadUserDetail = async (params) => {
  const id = params.id;
  const raw = await fetch(`${urlBase}stationRole/info/${id}.do?`, {
    credentials: 'same-origin',
    method: 'GET',
    headers: {
      'Content-Type': 'application/json;charset=utf-8',
    },
  });
  const res = await raw.json();
  if (res.resultCode !== 0) {
    throw { text: res.resultMessage };
  }

  return res.data;
};

const loadUserDetailLog = async (params) => {
  const id = params.id;
  const raw = await fetch(`${urlBase}stationRole/info/${id}/logs.do?`, {
    credentials: 'same-origin',
    method: 'GET',
    headers: {
      'Content-Type': 'application/json;charset=utf-8',
    },
  });
  const res = await raw.json();
  if (res.resultCode !== 0) {
    throw { text: res.resultMessage };
  }

  return res.data;
};

const saveDetail = async (params) => {
  console.log(params);
  const id = params.id;
  const raw = await fetch(`${urlBase}stationRole/info/${id}.do?`, {
    credentials: 'same-origin',
    method: 'POST',
    headers: {
      'Content-Type': 'application/json;charset=utf-8',
    },
    body: JSON.stringify(params),
  });
  const res = await raw.json();
  if (res.resultCode !== 0) {
    throw { text: res.resultMessage };
  }

  return res;
};

export default {
  loadList,
  loadSysRoleDict,
  loadUserDetail,
  loadUserDetailLog,
  saveDetail,
};
