import React from 'react';
import Modal from 'COMPONENT/Modal';
import Input from 'COMPONENT/Input';
import loaders from './loaders';
import List from 'COMPONENT/List';
import SearchBar from 'COMPONENT/SearchBar';

const columns = [
  { title: '工号', dataIndex: 'employeeNo' },
  { title: '姓名', dataIndex: 'userName' },
  { title: '登陆账号', dataIndex: 'account' },
  { title: '联系方式', dataIndex: 'contactNo' },
];

class AddUserModal extends React.Component {
  constructor(props) {
    super(props);

    this.defaultSearchConditions = {
      userName: '',
      employeeId: '',
      account: '',
      contactNo: '',
    };

    this.state = {
      loading: false, // boolean 页面加载状态
      selectedRowKeys: [],
      list: [],
      listLoading: false,
      total: 0,
      current: 1,
      pageSize: 10,
      searchExpand: false,
      ...this.defaultSearchConditions,
    };
  }

  onSearch = async () => {
    const page = 1;
    const { userName, employeeId, account, contactNo } = this.state;
    const { campusId } = this.props;
    this.setState({ listLoading: true });
    const res = await loaders.getUsers({
      userName,
      employeeId,
      account,
      contactNo,
      campusId,
      rows: this.state.pageSize,
      page: page,
    });
    this.setState({
      current: page,
      total: res.records,
      list: res.rows,
      listLoading: false,
      selectedRowKeys: [],
    });
    return;
  };

  onChange = async ({ current = this.state.current, pageSize = this.state.pageSize }) => {
    const { userName, employeeId, account, contactNo } = this.state;
    const { campusId } = this.props;
    console.log(current, pageSize);
    this.setState({ listLoading: true });
    const res = await loaders.getUsers({
      userName,
      employeeId,
      account,
      contactNo,
      campusId,
      rows: pageSize,
      page: current,
    });
    this.setState({
      current,
      pageSize,
      total: res.records,
      list: res.rows,
      listLoading: false,
      selectedRowKeys: [],
    });
    return;
  };

  onReset = () => {
    this.setState({
      ...this.defaultSearchConditions,
    });
  };

  onOk = async () => {
    const { list, selectedRowKeys } = this.state;
    this.setState({ loading: true });
    const addedUsers = [];
    if (!selectedRowKeys.length) {
      this.props.onCancel();
      return;
    }
    selectedRowKeys.map((i) => {
      addedUsers.push(list[i]);
    });
    await this.props.onOk(addedUsers);
    this.setState({ loading: false });
    this.props.onCancel();
  };

  componentDidMount() {
    this.setState({ loading: true });
    Promise.all([this.onSearch()]).then(() => {
      this.setState({ loading: false });
    });
  }

  render() {
    const { onCancel } = this.props;
    const {
      listLoading,
      total,
      current,
      pageSize,
      userName,
      employeeId,
      account,
      contactNo,
      loading,
      list,
      selectedRowKeys,
      searchExpand,
    } = this.state;
    return (
      <Modal loading={loading} onOk={this.onOk} cancelText="关闭" onCancel={onCancel} width={1366} title="新增账户">
        <SearchBar
          expand={searchExpand}
          onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
          onSearch={this.onSearch}
          onReset={this.onReset}
        >
          <Input
            placeholder="工号"
            value={employeeId}
            onChange={(e) => this.setState({ employeeId: e.target.value })}
          />
          <Input placeholder="姓名" value={userName} onChange={(e) => this.setState({ userName: e.target.value })} />
          <Input placeholder="登陆账户" value={account} onChange={(e) => this.setState({ account: e.target.value })} />
          <Input
            placeholder="联系方式"
            value={contactNo}
            onChange={(e) => this.setState({ contactNo: e.target.value })}
          />
        </SearchBar>
        <List
          dataSource={list}
          columns={columns}
          rowSelection={{
            selectedRowKeys,
            onChange: (keys) => this.setState({ selectedRowKeys: keys }),
          }}
          loading={listLoading}
          onChange={this.onChange}
          pagination={{
            current,
            total,
            pageSize,
          }}
        />
      </Modal>
    );
  }
}

export default AddUserModal;
