import {
  getAccessUserListByBrenchId,
  findOrganization,
  setEmergentAuthority,
  getUserListByCampusId,
} from 'SERVICE/system';

const loadBranchUsers = async ({ brenchId }) => {
  const res = await getAccessUserListByBrenchId({ brenchId });
  return res.data.reduce((obj, ele) => {
    obj[ele.orgId] = obj[ele.orgId] || [];
    obj[ele.orgId].push(ele);
    return obj;
  }, {});
};

/**
 * @param  {string} branchId 分公司ID
 * @param  {string} campusId 学习中心ID
 * @param  {string} status 状态
 */
const loadList = async (branchId, campusId, status) => {
  const data = await findOrganization({ branchId, campusId, stateOfEmergency: status });
  const list = data.map((branch) => {
    const { branchId, branchName, organizations } = branch;
    return {
      id: branchId,
      name: branchName,
      children: organizations.map((campus) => {
        const { organizationId, organizationName, stateOfEmergency } = campus;
        return { id: organizationId, name: organizationName, status: stateOfEmergency };
      }),
    };
  });
  return list;
};

const updateUsers = (params) => {
  return setEmergentAuthority(params);
};

const getUsers = (params) => {
  return getUserListByCampusId(params);
};

export default {
  loadBranchUsers,
  loadList,
  updateUsers,
  getUsers,
};
