import React from 'react';
import { connect } from 'dva';
import { Switch } from 'antd';
import styles from './index.less';
import { NoData } from 'COMPONENT';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
import { statusDict, STATUS_NORMAL, STATUS_EMERGENCY } from '../dict';
import * as loaders from '../loader';
import * as utils from '../util';

/**
 * 资源列表
 *
 * @param props.list {array} 分公司资源列表 [{id, name, loading}]
 * @param props.beforeSwitch {function(<string>id)} 执行切换之前回调
 * @param props.afterSwitch {function(<string>id, <string>type)} 执行切换之后回调，出参为组织ID，是否切换成功'success'|'error'
 */
class ResourceList extends React.Component {
  static defaultProps = {
    list: null,
    beforeSwitch: (id) => {},
    afterSwitch: (id, type) => {},
  };

  constructor(props) {
    super(props);
  }

  /**
   * 切换分公司
   */
  onSwitchBranch = (id) => {
    const branch = this.props.list.find((item) => item.id === id);
    const status = branch.status || STATUS_NORMAL;
    const nextStatus = statusDict.find((s) => s.val !== status);
    const tips = `${branch.name}将切换为${nextStatus.text}状态，请确认是否切换？`;

    showConfirm(tips, () => {
      const emergencyStatus = utils.transformStatusToBoolean(nextStatus.val);
      const param = { organizationId: id, emergencyStatus };

      this.props.beforeSwitch(id);
      loaders
        .switchBranchStatus(param)
        .then(() => {
          this.props.afterSwitch(id, 'success');
        })
        .catch(() => {
          this.props.afterSwitch(id, 'error');
        });
    });
  };

  render() {
    const list = this.props.list || [];

    return (
      <div>
        {list.length > 0 ? (
          list.map((item, i) => {
            const { id, name } = item;
            const status = item.status || STATUS_NORMAL;
            const loading = !!item.loading;

            return (
              <div key={i} className={styles.branchItem}>
                <div className={styles.branchTitle}>
                  <span className={styles.branchName}>{name}</span>
                  <div className={styles.statusWrap}>
                    <Switch
                      disabled={!this.props.btnAuth_switch || loading}
                      checkedChildren="紧急"
                      unCheckedChildren="正常"
                      checked={status === STATUS_EMERGENCY}
                      onChange={() => this.onSwitchBranch(id)}
                    />
                  </div>
                </div>
              </div>
            );
          })
        ) : (
          <NoData />
        )}
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  const btnAuth = state.account.btnAuths;
  const btnAuth_switch = btnAuth.indexOf('SSResourceOfEmergencySwitchBtn') > -1;

  return {
    btnAuth_switch,
  };
};

export default connect(mapStateToProps)(ResourceList);
