import { listOrgEmergencyResource, updateOrgEmergencyResource } from 'SERVICE/system';
import { getAllBranch } from 'SERVICE/common';
import { transformStatusToString } from './util';

/**
 * 加载分公司字典
 *
 * @return {Promise<Array|*>}
 */
export async function loadBranchDict() {
  try {
    const data = await getAllBranch();

    return data.map((item) => {
      const { organizationId, organizationName } = item;

      return { val: organizationId, text: organizationName };
    });
  } catch (e) {
    return [];
  }
}

/**
 * 获取分公司资源列表
 * tips: emergencyStatus	{Boolean} 紧急资源状态 {true: 紧急, false: 正常}
 *
 * @param params.branchId {string} 分公司ID
 * @param params.stateOfEmergency {string} 紧急状态
 * @return {Promise<Array|*>}
 */
export async function loadResourceList(params) {
  try {
    const response = await listOrgEmergencyResource(params);
    const data = response.data || [];
    const list = data.map((branch) => {
      const { orgId, orgName, emergencyStatus } = branch;

      return {
        id: orgId,
        name: orgName,
        status: transformStatusToString(emergencyStatus),
        loading: false,
      };
    });

    return list;
  } catch (e) {
    return [];
  }
}

/**
 * 切换分公司紧急状态
 *
 * @param params.branchId {string} 分公司ID
 * @param params.emergencyStatus {string} 紧急资源状态 {true: 紧急, false: 正常}
 */
export function switchBranchStatus(params) {
  return updateOrgEmergencyResource(params);
}
