import { STATUS_NORMAL, STATUS_EMERGENCY } from './dict';

/**
 * 将后台字段的紧急状态 转换为 前端的字符串紧急状态
 *
 * @param status {boolean|*} 后台字段的紧急状态 {true: 紧急, false: 正常}
 * @return {string} 前端紧急状态字典值
 */
export function transformStatusToString(status) {
  if (status === null || status === undefined) {
    return STATUS_NORMAL;
  }

  return !!status ? STATUS_EMERGENCY : STATUS_NORMAL;
}

/**
 * 将前端的字符串紧急状态 转换为 后台字段的紧急状态
 *
 * @param status {string} 前端紧急状态字典值
 * @return {boolean} 后台字段的紧急状态 {true: 紧急, false: 正常}
 */
export function transformStatusToBoolean(status) {
  return status !== STATUS_NORMAL;
}

/**
 * 格式化搜索表单的紧急状态，可为空
 *
 * @param status {string} 前端紧急状态字典值
 * @return {null|boolean}
 */
export function formatApiStatus(status) {
  if (!status) {
    return null;
  }

  return status !== STATUS_NORMAL;
}

/**
 * 格式化资源列表的分公司字典
 *
 * @param list {Array} 分公司字典
 * @return {{val: *, text: *}[]|Array}
 */
export function formatResourceBranchDict(list = []) {
  if (!list || list.length <= 0) {
    return [];
  }

  return list.map((item) => {
    return { text: item.name, val: item.id };
  });
}
