import React from 'react';
import { connect } from 'dva';
import { Modal, Checkbox } from 'antd';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

import SelectTeacher from './selectTeacher';
import IntellignetSchedule from './intelligentSchedule';
import { Page, Button, List, TextLink, Select, Loading } from 'COMPONENT';

import { api } from 'SERVICE/teacherPlan';

const DISABLE_STATUS = 0;
const seasonMap = {
  spring: 'DAT0000000231',
  summer: 'DAT0000000232',
  autumn: 'DAT0000000233',
  winter: 'DAT0000000230',
};

const { getSeasonSubjectList, enableTeacherSubject, disableTeacherSubject, updateClassLevel, getLevelCategory } = api;

import style from './style.less';

const CheckboxGroup = Checkbox.Group;

const { COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_XL } = List;

let tableColumns = [
  {
    title: '科目',
    dataIndex: 'subjectName',
    key: 'subjectName',
  },
  {
    title: '可教年级',
    dataIndex: 'gradeName',
    key: 'gradeName',
    width: COL_WIDTH_S,
  },
  {
    title: '可带层级',
    dataIndex: 'level',
    key: 'level',
    width: COL_WIDTH_XL,
    render(text, row) {
      return (row.mcTeacherSubjectLimitRuleDetailVos || []).map((l) => l.classCategoryName).join();
    },
  },
  {
    title: '状态',
    dataIndex: 'status',
    key: 'status',
    width: COL_WIDTH_M,
    render(text) {
      return text === DISABLE_STATUS ? '禁用' : '可用';
    },
  },
];

class TwoTeacherLecture extends React.Component {
  constructor(props) {
    super(props);

    const defaultYear = props.YEAR.find((y) => y.text == new Date().getFullYear());

    const defaultQuarter = props.QUARTER && props.QUARTER[0] ? props.QUARTER[0] : null;

    this.params = props.pageParams[`/${props.route.path}`];
    this.state = {
      showEditLevel: false,
      showRegularModal: false,
      currentSelectedTeacher: this.params.teacher,
      levelList: [],

      loading: false,
      listData: {
        list: [],
        total: 0,
        pageNum: 1,
        pageSize: 10,
      },
      searchParams: {
        yearId: defaultYear ? defaultYear.val : '',
        quarterId: defaultQuarter ? defaultQuarter.val : '',
      },
      defaultYear: defaultYear ? defaultYear.val : '',
      defaultQuarter: defaultQuarter ? defaultQuarter.val : '',
      // 选中的需要修改层级的科目
      currentSelectedSubject: [],
      defaultSelectedLevel: [],
    };
  }

  componentDidMount() {
    // 获取字典
    this.fetchCategoryLevel();
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['branchAllCampus'],
      },
    });
    this.fetchSeasonSubjectList();
  }

  // 获取所有科目层级
  fetchCategoryLevel = () => {
    getLevelCategory({ category: 'CLASS_CATEGORY' }).then((res) => {
      const levelList = (res.data || []).map((l) => ({
        value: l.id,
        label: l.name,
      }));
      this.setState({
        levelList,
      });
    });
  };

  // 获取老师季度科目列表
  fetchSeasonSubjectList = () => {
    const {
      currentSelectedTeacher,
      listData,
      searchParams: { quarterId, yearId },
    } = this.state;
    const { pageSize, pageNum } = listData;

    this.setState({
      loading: true,
    });
    getSeasonSubjectList({
      teacherId: currentSelectedTeacher.value,
      pageNum,
      pageSize,
      yearId,
      quarterId,
    })
      .then((res) => {
        const detail = res.data || {};

        this.setState({
          listData: {
            list: detail.list,
            total: detail.total,
            pageSize: listData.pageSize,
            pageNum: listData.pageNum,
          },
        });
      })
      .finally(() => {
        this.setState({
          loading: false,
        });
      });
  };

  onPaginationChange = ({ current, pageSize }) => {
    this.setState(
      {
        listData: {
          ...this.state.listData,
          pageNum: current,
          pageSize: pageSize,
        },
      },
      this.fetchSeasonSubjectList,
    );
  };

  batchEditLevel = () => {
    this.setState({
      defaultSelectedLevel: [],
      showEditLevel: true,
      currentSelectedSubject: this.state.currentSelectedSubject,
    });
  };

  singleEditLevel = (row) => {
    const levelList = row.mcTeacherSubjectLimitRuleDetailVos || [];

    this.setState({
      defaultSelectedLevel: levelList.map((l) => l.classCategoryId),
      showEditLevel: true,
      currentSelectedSubject: [row.id],
    });
  };

  // 禁用课程
  enableCourse = (row) => {
    const teacherId = this.state.currentSelectedTeacher.value;

    enableTeacherSubject({
      id: row.id,
      teacherId,
    }).then(() => {
      this.fetchSeasonSubjectList();
      showMessage('保存成功', 'success');
    });
  };

  // 启用课程
  disableCourse = (row) => {
    const teacherId = this.state.currentSelectedTeacher.value;

    disableTeacherSubject({
      id: row.id,
      teacherId,
    }).then(() => {
      this.fetchSeasonSubjectList();
      showMessage('保存成功', 'success');
    });
  };

  onUpdateSubjectLevel = () => {
    const { currentSelectedTeacher, currentSelectedSubject, defaultSelectedLevel } = this.state;

    const params = {
      teacherId: currentSelectedTeacher.value,
      ids: currentSelectedSubject,
      classCategoryIds: defaultSelectedLevel,
    };

    if (!defaultSelectedLevel || !defaultSelectedLevel.length) {
      return showMessage('请选择层级');
    }

    updateClassLevel(params).then(() => {
      showMessage('保存成功', 'success');
      this.fetchSeasonSubjectList();
      this.setState({
        showEditLevel: false,
      });
    });
  };

  onTeacherChange = (teacher) => {
    this.setState(
      {
        currentSelectedTeacher: teacher,
        listData: {
          list: [],
          total: 0,
          pageNum: 1,
          pageSize: 10,
        },
        searchParams: {
          quarterId: this.state.defaultQuarter,
          yearId: this.state.defaultYear,
        },
      },
      this.fetchSeasonSubjectList,
    );
  };

  render() {
    const { route, YEAR, QUARTER, hasEditSubjectPermission } = this.props;
    const avalidQuarter = QUARTER.filter((q) => {
      return Object.keys(seasonMap).some((s) => seasonMap[s] === q.val);
    });

    const {
      name,
      loading,
      showEditLevel,
      currentSelectedTeacher,
      levelList,
      listData,
      defaultSelectedLevel,
      searchParams: { quarterId, yearId },
    } = this.state;

    const columns = tableColumns.concat({
      title: '操作',
      dataIndex: 'operation',
      key: 'operation',
      width: COL_WIDTH_M,
      render: (text, row) => {
        return (
          <div>
            {hasEditSubjectPermission && (
              <TextLink
                style={{ marginRight: '15px' }}
                onClick={() => {
                  this.singleEditLevel(row);
                }}
              >
                修改层级
              </TextLink>
            )}
            {row.status === DISABLE_STATUS ? (
              <TextLink
                onClick={() => {
                  this.enableCourse(row);
                }}
              >
                启用
              </TextLink>
            ) : (
              <TextLink
                style={{ marginRight: '15px' }}
                onClick={() => {
                  this.disableCourse(row);
                }}
              >
                禁用
              </TextLink>
            )}
          </div>
        );
      },
    });

    const rowSelection = {
      onChange: (selectedRowKeys, selectedRows) => {
        this.setState({
          currentSelectedSubject: selectedRows.map((row) => row.id),
        });
      },
    };

    return (
      <Page name={name} route={route} className={style['page-content']}>
        <Loading loading={loading} />
        <div className={style['container']}>
          <SelectTeacher teacher={this.params.teacher} onTeacherChange={this.onTeacherChange} />
          <div className={style['table']}>
            <div className={style['chose-teacher']}>
              <span>老师{currentSelectedTeacher.label}的可教课程</span>
            </div>
            <IntellignetSchedule teacherId={currentSelectedTeacher.value} />

            <div className={style['table-search']}>
              <div
                style={{
                  display: 'flex',
                  alignItems: 'center',
                  width: '500px',
                }}
              >
                <span>季度可教科目</span>
                <Select
                  value={yearId}
                  className={style['table-search-select']}
                  placeholder="选择年度"
                  dict={YEAR}
                  onChange={(value) => {
                    this.setState(
                      {
                        searchParams: {
                          quarterId,
                          yearId: value,
                        },
                        listData: {
                          ...listData,
                          pageSize: listData.pageSize,
                        },
                      },
                      this.fetchSeasonSubjectList,
                    );
                  }}
                />
                <Select
                  value={quarterId}
                  className={style['table-search-select']}
                  placeholder="选择季度"
                  dict={avalidQuarter}
                  onChange={(value) => {
                    this.setState(
                      {
                        searchParams: {
                          quarterId: value,
                          yearId,
                        },
                        listData: {
                          ...listData,
                          pageSize: listData.pageSize,
                        },
                      },
                      this.fetchSeasonSubjectList,
                    );
                  }}
                />
              </div>

              {hasEditSubjectPermission && <Button onClick={this.batchEditLevel}>批量修改可带层级</Button>}
            </div>

            <List
              bordered
              dataSource={listData.list}
              total={listData.total}
              columns={columns}
              rowSelection={rowSelection}
              pagination={{
                current: listData.pageNum,
                total: listData.total,
                pageSize: listData.pageSize,
                pageSizeOptions: ['10', '20', '30'],
              }}
              onChange={this.onPaginationChange}
            />

            <Modal
              title="修改可带层级"
              visible={showEditLevel}
              onCancel={() => {
                this.setState({
                  showEditLevel: false,
                });
              }}
              onOk={this.onUpdateSubjectLevel}
            >
              <div className={style.editLevel}>
                <p className={style.useAbleLevel}>可带层级:</p>
                <CheckboxGroup
                  options={levelList}
                  value={defaultSelectedLevel}
                  onChange={(value) => {
                    this.setState({
                      defaultSelectedLevel: value,
                    });
                  }}
                />
              </div>
            </Modal>
          </div>
        </div>
      </Page>
    );
  }
}

export default connect((state) => {
  const { btnAuths } = state.account;
  const hasEditSubjectPermission = btnAuths.indexOf('TEACHER_TEACHABLE_SUBJECT_EDIT') > -1;
  const hasSaveIntelligentRegularPermission = btnAuths.indexOf('SAVE_INTELLIGENT_SCHEDULE_REGULAR') > -1;

  return {
    CAMPUS: state.dict['branchAllCampus'],
    YEAR: state.dict['year'],
    QUARTER: state.dict['quarter'],
    pageParams: state.main.pageParams,
    hasEditSubjectPermission,
    hasSaveIntelligentRegularPermission,
  };
})(TwoTeacherLecture);
