import React from 'react';
import classnames from 'classnames';
import { getTeacherList } from 'SERVICE/teacherSchedule';
import { Button, Input, Loading } from 'COMPONENT';
import style from './style.less';

class SelectTeacher extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      searchKeyword: '',
      teacherList: [],
      teacherMap: {},
      loading: false,
      currentSelected: {},
    };
  }

  componentDidMount() {
    this.fetchTeach().then(() => {
      if (this.props.teacher) {
        this.toggleTeacher(this.props.teacher);

        const element = this.state.teacherMap[this.props.teacher.value];
        element && element.scrollIntoView();
      }
    });
  }

  fetchTeach = () => {
    let { searchKeyword } = this.state;
    this.setState({
      loading: true,
    });
    return getTeacherList({
      teacherName: searchKeyword,
    })
      .then((res) => {
        let data = res.value || {};
        let teacher = Object.keys(data).reduce((pre, value) => {
          pre.push({
            label: data[value],
            value,
          });
          return pre;
        }, []);
        this.setState({
          teacherList: teacher,
        });
      })
      .finally(() => {
        this.setState({
          loading: false,
        });
      });
  };

  toggleTeacher = (teacher) => {
    if (teacher.value !== this.state.currentSelected.value) {
      this.setState({
        currentSelected: teacher,
      });
      this.props.onTeacherChange(teacher);
    }
  };

  render() {
    const { teacherList, loading, currentSelected, teacherMap } = this.state;

    return (
      <div className={style['search']}>
        <Loading loading={loading} />
        <div className={style['search-bar']}>
          <Input
            className={style['search-bar-input']}
            placeholder="输入老师姓名进行搜索"
            onChange={(e) => {
              this.setState({
                searchKeyword: e.target.value,
              });
            }}
            onPressEnter={this.fetchTeach}
          />
          <Button onClick={this.fetchTeach}>搜索</Button>
        </div>

        <ul className={style['search-content']}>
          {teacherList.map((user) => {
            return (
              <li
                key={user.value}
                ref={(r) => {
                  teacherMap[user.value] = r;
                }}
                className={classnames({
                  [style['search-content-item']]: true,
                  [style.active]: user.value === currentSelected.value,
                })}
                onClick={() => {
                  this.toggleTeacher(user);
                }}
              >
                {user.label}
              </li>
            );
          })}
          {teacherList.length === 0 && !loading ? <li className={style['search-content-empty']}>暂无数据</li> : null}
        </ul>
      </div>
    );
  }
}

export default SelectTeacher;
