import React from 'react';
import { connect } from 'dva';
import { List, Loading, Page, TextLink } from 'COMPONENT';
import { getSubjectModifyHistory } from 'SERVICE/teacherSchedule';

import Search from './search';
import tableColumns from './column';
import Detail from './reularDetail';

class InvalidList extends React.Component {
  constructor(props) {
    super(props);

    this.params = props.pageParams[`/${props.route.path}`];

    this.state = Object.assign({
      loading: false, // {boolean} 内容加载状态
      list: [], // {Array} 列表数据，
      totalRecords: 0, // {number} 总记录数
      currentPage: 1, // {number} 当前页码
      pageSize: 10, // {number} 每页记录数
      searchParams: {},
      showDetailDialog: false,
      afterId: '',
      beforeId: '',
    });
  }

  fetchModifyHistory() {
    this.setRequestLoading(true);

    let searchParams = this.state.searchParams;
    let teacher = this.params.teacher;
    let pagination = {
      pageSize: this.state.pageSize,
      pageNum: this.state.currentPage,
    };

    let params = {
      ...searchParams,
      ...pagination,
      type: 99,
    };

    params.objectId = params.objectId || (teacher && teacher.value);

    getSubjectModifyHistory(params)
      .then((res) => {
        this.setState({
          list: res.data.list || [],
          totalRecords: res.data.total,
          loading: false,
        });
      })
      .catch(() => {
        this.setRequestLoading(false);
      });
  }

  onSearch = (searchParams) => {
    this.setState(
      {
        searchParams,
      },
      this.fetchModifyHistory,
    );
  };

  onPaginationChange = (pageData) => {
    this.setState(
      {
        currentPage: pageData.current,
        pageSize: pageData.pageSize,
      },
      this.fetchModifyHistory,
    );
  };

  showRegularDetail = (beforeId, afterId) => {
    this.setState({
      showDetailDialog: true,
      beforeId,
      afterId,
    });
  };

  setRequestLoading = (loading) => {
    this.setState({
      loading,
    });
  };

  render() {
    const { route } = this.props;
    const { loading, list, totalRecords, currentPage, pageSize, showDetailDialog, beforeId, afterId } = this.state;

    const columns = tableColumns.concat([
      {
        title: '修改前-是否限课',
        dataIndex: 'beforeUpdateIsLimit',
        key: 'beforeUpdateIsLimit',
        width: 125,
        render: (text, row) => {
          return row.beforeRuleId ? '是' : '否';
        },
      },
      {
        title: '修改前-限课规则',
        dataIndex: 'beforeRuleId',
        key: 'beforeRuleId',
        width: 140,
        render: (text, row) => {
          return row.beforeRuleId ? (
            <TextLink onClick={() => this.showRegularDetail(row.beforeRuleId, row.afterRuleId)}>限课规则</TextLink>
          ) : (
            ''
          );
        },
      },
      {
        title: '修改后-是否限课',
        dataIndex: 'afterUpdateIsLimit',
        key: 'afterUpdateIsLimit',
        width: 125,
        render: (text, row) => {
          return row.afterRuleId ? '是' : '否';
        },
      },
      {
        title: '修改后-限课规则',
        dataIndex: 'afterRuleId',
        key: 'afterRuleId',
        width: 140,
        render: (text, row) => {
          return row.afterRuleId ? (
            <TextLink onClick={() => this.showRegularDetail(row.beforeRuleId, row.afterRuleId)}>限课规则</TextLink>
          ) : (
            ''
          );
        },
      },
    ]);

    return (
      <Page route={route}>
        <Search onSearch={this.onSearch} defaultSelectTeacher={this.params.teacher} />

        <List
          dataSource={list}
          columns={columns}
          pagination={{
            current: currentPage,
            total: totalRecords,
            pageSize,
          }}
          onChange={this.onPaginationChange}
          bordered={true}
        />

        {showDetailDialog && (
          <Detail
            beforeId={beforeId}
            afterId={afterId}
            onClose={() => {
              this.setState({
                showDetailDialog: false,
              });
            }}
          />
        )}

        <Loading loading={loading} />
      </Page>
    );
  }
}

export default connect((state) => {
  return {
    pageParams: state.main.pageParams,
  };
})(InvalidList);
