import React from 'react';
import { Modal, Row, Col } from 'antd';

import { getRegularDetailById } from 'SERVICE/teacherPlan';
import { Loading } from 'COMPONENT';

import style from './style.less';

const seasonMap = {
  spring: 'DAT0000000231',
  summer: 'DAT0000000232',
  autumn: 'DAT0000000233',
  winter: 'DAT0000000230',
};

// 按季度格式化规则用于渲染使用
function formatRegular(regular) {
  const data = (regular || []).reduce((pre, cur) => {
    if (!pre[cur.quarterId]) {
      pre[cur.quarterId] = {
        quarterName: cur.quarterName,
        quarterId: cur.quarterId,
        children: [cur],
      };
    } else {
      pre[cur.quarterId].children.push(cur);
    }

    return pre;
  }, {});

  return Object.keys(data).map((k) => data[k]);
}

function isSummerOrWinter(quarterId) {
  return quarterId === seasonMap.summer || quarterId === seasonMap.winter;
}

class LimitRegularModal extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: true,
      // 用于填充展示的详细规则
      beforeRegularList: [],
      afterRegularList: [],
      beforeRegularListDetail: null,
      afterRegularListDetail: null,
      detail: null,
    };
  }

  async componentDidMount() {
    Promise.all([
      this.fetchRegularDetail(this.props.beforeId, 'beforeRegularList'),
      this.fetchRegularDetail(this.props.afterId, 'afterRegularList'),
    ]).finally(() => {
      this.setState({
        loading: false,
      });
    });
  }

  // 获取已设置的规则详情
  fetchRegularDetail = (id, type) => {
    if (!id) return Promise.resolve();

    return getRegularDetailById(id)().then((res) => {
      const detail = res.data;
      if (detail) {
        this.setState({
          [type + 'Detail']: detail,
          [type]: formatRegular(detail.mcNumberLimitRuleDetailVos),
        });
      }
    });
  };

  render() {
    const { beforeRegularList, afterRegularList, loading, beforeRegularListDetail, afterRegularListDetail } =
      this.state;

    const renderRegular = (regular, detail) => {
      if (loading) {
        return null;
      }
      if (!regular || !regular.length) {
        return <div className={style.unLimit}>不限课</div>;
      }
      return (
        <div>
          <p className={style.regularTitle}>规则名称：{detail ? detail.name : ''}</p>
          {regular.map((season) => {
            return (
              <div className={style.regularContainer} key={season.quarterId}>
                <span className={style.regularType}>{season.quarterName}最大带班数</span>
                <ul className={style.regularContent}>
                  {season.children.map((phase) => (
                    <li className={style.regularItem} key={phase.id}>
                      <span className={style.regularPhase}>
                        {isSummerOrWinter(season.quarterId) ? `${phase.termNumber}期` : null}
                      </span>
                      <span>{phase.maxClassNumber}</span>
                      <span className={style.regularLabel}>课酬补贴系数</span>
                      <span>{phase.salaryCoefficient}</span>
                    </li>
                  ))}
                </ul>
              </div>
            );
          })}
        </div>
      );
    };

    return (
      <Modal title="限课规则" width={820} visible={true} onCancel={this.props.onClose}>
        <div>
          <Loading loading={loading} />

          <div>
            <Row gutter={20} justify="space-between">
              <Col span={12}>
                <p className={style.regularTime}>修改前</p>
                <div>{renderRegular(beforeRegularList, beforeRegularListDetail)}</div>
              </Col>
              <Col span={12}>
                <p className={style.regularTime}>修改后</p>
                <div>
                  <div>{renderRegular(afterRegularList, afterRegularListDetail)}</div>
                </div>
              </Col>
            </Row>
          </div>
        </div>
      </Modal>
    );
  }
}

export default LimitRegularModal;
