import React from 'react';
import { connect } from 'dva';
import { SearchBar, Select } from 'COMPONENT';
import { getTeacherList } from 'SERVICE/teacherSchedule';

const getSearchInitValue = () => {
  return {
    objectId: '',
    blOrganizationId: '',
  };
};

class Search extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false, // {boolean} 内容加载状态
      searchParams: getSearchInitValue(true),
    };
  }

  componentDidMount() {
    const { defaultSelectTeacher } = this.props;
    this.fetchTeach();
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['branch'],
      },
    });

    if (defaultSelectTeacher) {
      this.setState({
        searchParams: {
          objectId: defaultSelectTeacher.value,
          blOrganizationId: '',
        },
      });
    }

    this.props.onSearch(this.state.searchParams);
  }

  fetchTeach = () => {
    getTeacherList().then((res) => {
      let data = res.value || {};
      let teacher = Object.keys(data).reduce((pre, value) => {
        pre.push({
          text: data[value],
          val: value,
        });
        return pre;
      }, []);
      this.setState({
        teacherList: teacher,
      });
    });
  };

  setSearchParams = (key, value) => {
    this.setState({
      searchParams: {
        ...this.state.searchParams,
        [key]: value,
      },
    });
  };

  resetSearchParams = () => {
    this.setState({
      searchParams: getSearchInitValue(),
    });
  };
  render() {
    const { searchParams, teacherList } = this.state;
    const { blOrganizationId, objectId } = searchParams;
    const { COMPANY } = this.props;

    return (
      <SearchBar expand={false} onSearch={() => this.props.onSearch(searchParams)} showResetButton={false}>
        <Select
          placeholder="所属分公司"
          showSearch
          dict={COMPANY}
          value={blOrganizationId}
          onChange={(value) => {
            this.setSearchParams('blOrganizationId', value);
          }}
        />
        <Select
          placeholder="老师"
          showSearch
          dict={teacherList}
          value={objectId}
          onChange={(value) => {
            this.setSearchParams('objectId', value);
          }}
        />
      </SearchBar>
    );
  }
}

export default connect((state) => {
  return {
    COMPANY: state.dict['branch'],
  };
})(Search);
