import React from 'react';
import Modal from 'COMPONENT/Modal';
import Button from 'COMPONENT/Button';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import TextLink from 'COMPONENT/TextLink';
import { OSS_BASE_PREFIX } from 'CONST/ossConfig';
import excelPng from './assets/excel.png';
import filePng from './assets/file.png';
import delPng from './assets/del.png';
import styles from './styles.less';
import { importRegulatFile } from 'SERVICE/teacherSchedule';
import { getAliyunOSSKeyWithoutCallBack } from 'SERVICE/course';
import moment from 'moment';

const templateUrl =
  '/miniclass/%E8%80%81%E5%B8%88%E5%B8%A6%E7%8F%AD%E6%95%B0%E9%99%90%E8%AF%BE%E8%A7%84%E5%88%99%E5%AF%BC%E5%85%A5%E6%A8%A1%E6%9D%BF.xlsx';

class ImportFile extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      //1：待上传，2：上传完成
      status: 1,
      files: [], // 上传文件
    };
  }

  /**
   * 下载模板
   */
  downloadTemplate = () => {
    const link = document.createElement('a');
    const body = document.body;
    link.href = OSS_BASE_PREFIX + templateUrl;
    link.style.display = 'none';
    link.download = '导入模板.xls';
    body.appendChild(link);
    link.click();
    body.removeChild(link);
  };

  /**
   * 点击选择文件事件
   * @param event
   */
  pickFile = (event) => {
    const files = Array.prototype.map.call(event.target.files, (file) => ({
      file,
      name: file.name,
      size: file.size,
      type: file.type,
    }));
    const file = files[0] || {};
    if (file.size && file.size / 1024 / 1024 > 1) {
      showMessage('文件大小超过1M,请重新选择');
      return;
    }
    this.setState({ files });
  };

  /**
   * 删除已选文件
   */
  onDelete = () => {
    this.setState({
      files: [],
    });
  };

  /**
   * 渲染底部按钮
   * @returns {*}
   */
  renderFooter = () => {
    const { status, files } = this.state;
    if (status === 1) {
      return [
        <Button key="222" type="primary" disabled={files.length === 0} onClick={this.onUploadFile}>
          导入
        </Button>,
        <Button key="111" onClick={this.onCancel}>
          取消
        </Button>,
      ];
    } else {
      return <Button onClick={this.onCancel}>取消</Button>;
    }
  };

  /**
   * 渲染模板下载
   */
  renderDownloadTemplate = () => {
    return (
      <div>
        <p style={{ display: 'flex', margin: '12px 0 24px 0' }}>
          <img src={excelPng} width="22" height="22" alt="excel" />
          <TextLink style={{ marginLeft: 10 }} onClick={() => this.downloadTemplate()}>
            模板下载
          </TextLink>
        </p>
        <p className={styles.label}>上传文件</p>
      </div>
    );
  };

  /**
   * 渲染选择的文件或按钮
   */
  renderUpload = () => {
    const { files } = this.state;
    if (files.length > 0) {
      return (
        <div>
          {this.renderDownloadTemplate()}
          <div className={styles.fileArea}>
            {files.map((file) => {
              return (
                <div key={file.name} className={styles.file}>
                  <span className={styles.fileWrap}>
                    <img src={filePng} alt="文件" width="16" height="16" />
                    <span className={styles.fileName}>{file.name}</span>
                  </span>
                  <img
                    src={delPng}
                    width="16"
                    height="16"
                    onClick={this.onDelete}
                    className={styles.fileDel}
                    alt="删除"
                  />
                </div>
              );
            })}
          </div>
        </div>
      );
    } else {
      return (
        <div>
          {this.renderDownloadTemplate()}
          <div className={styles.pickFile}>
            选择文件
            <input
              type="file"
              className={styles.uploadBtn}
              onClick={(e) => (e.target.value = null)}
              onChange={this.pickFile}
            />
          </div>
        </div>
      );
    }
  };

  onUploadFile = async () => {
    const { files } = this.state;
    const { OSSAccessKeyId, policy, signature, host } = await getAliyunOSSKeyWithoutCallBack();
    let formData = new FormData();
    const time = moment().format('YYYYMMDDHHmmss');
    const key = time + files[0].file.name;

    formData.append('OSSAccessKeyId', OSSAccessKeyId);
    formData.append('policy', policy);
    formData.append('Signature', signature);
    formData.append('success_action_status', '200');
    formData.append('key', key);
    files.forEach((file) => {
      formData.append('file', file.file);
    });
    const xhr = new XMLHttpRequest();
    xhr.open('POST', host);
    xhr.onload = async () => {
      if (xhr.status === 200 && xhr.readyState === 4) {
        this.importTemplate(key);
      } else {
        showMessage('系统错误');
        this.setState({ loading: false });
      }
    };
    xhr.onerror = () => {
      showMessage('网络错误!');
      this.setState({ loading: false });
    };
    xhr.send(formData);
    this.setState({ loading: true });
  };

  importTemplate = (uploadData) => {
    return importRegulatFile({
      type: 99,
      fileKey: uploadData,
    })
      .then((res) => {
        showMessage('导入成功', 'success');
        this.props.onSuccess(res.data);
      })
      .finally(() => {
        this.setState({ loading: false });
      });
  };

  /**
   * 取消
   */
  onCancel = () => {
    this.setState({ status: 1 });
    this.props.onClose();
  };

  render() {
    return (
      <Modal
        loading={this.state.loading}
        title="导入限课规则"
        width="446"
        onCancel={this.onCancel}
        confirmLoading={this.state.loading}
        footer={this.renderFooter()}
      >
        {this.renderUpload()}
      </Modal>
    );
  }
}

export default ImportFile;
