import React from 'react';
import { connect } from 'dva';
import { Modal, Row, Checkbox } from 'antd';
import { FormItem, Select } from 'COMPONENT';

import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { addTeacherSubject } from 'SERVICE/teacherSchedule';

const CheckboxGroup = Checkbox.Group;
const statusOptions = [
  {
    text: '正常',
    val: '0',
  },
  {
    text: '禁用',
    val: '1',
  },
];

class AddCourseDialog extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      showAddSubjectDialog: false,
      addModalData: {
        status: '0',
        subject: '',
        grade: [],
      },
      indeterminate: false,
      checkAll: false,
    };
  }

  onGradeChange = (checkedList) => {
    const gradeList = this.props.GRADE;
    const addModalData = this.state.addModalData;

    this.setState({
      addModalData: {
        ...addModalData,
        grade: checkedList,
      },
      indeterminate: !!checkedList.length && checkedList.length < gradeList.length,
      checkAll: checkedList.length === gradeList.length,
    });
  };

  onCheckAllChange = (e) => {
    const checked = e.target.checked;
    const gradeList = this.props.GRADE;
    const addModalData = this.state.addModalData;

    this.setState({
      addModalData: {
        ...addModalData,
        grade: checked ? gradeList.map((g) => g.val) : [],
      },
      indeterminate: false,
      checkAll: e.target.checked,
    });
  };

  addCourse = () => {
    const formValue = this.state.addModalData;
    const teacherId = this.props.teacherId;
    const allHaveValue = Object.keys(formValue).every((key) => {
      if (Array.isArray(formValue[key])) {
        return formValue[key].length > 0;
      } else {
        return !!formValue[key];
      }
    });

    if (!allHaveValue) {
      return showMessage('请填写完整课程信息');
    }

    const params = {
      'teacher.userId': teacherId,
      'subject.id': formValue.subject,
      subjectStatus: formValue.status,
      'grade.id': formValue.grade.join(),
    };

    addTeacherSubject(params).then(() => {
      showMessage('新增成功', 'success');
      this.props.onSuccess({
        subjectId: formValue.subject,
        gradeId: formValue.grade.join(),
        subjectStatus: formValue.status,
      });
    });
  };

  render() {
    const { SUBJECT, GRADE } = this.props;
    const { addModalData } = this.state;

    return (
      <Modal title="新增可教课程" width={700} visible={true} onCancel={this.props.onCancel} onOk={this.addCourse}>
        <div>
          <Row>
            <FormItem labelWidth={100} name="科目" required>
              <Select
                placeholder="请选择科目"
                showSearch
                dict={SUBJECT}
                value={addModalData.subject}
                onChange={(value) => {
                  this.setState({
                    addModalData: {
                      ...addModalData,
                      subject: value,
                    },
                  });
                }}
              />
            </FormItem>

            <FormItem labelWidth={100} name="年级" required>
              <Checkbox
                indeterminate={this.state.indeterminate}
                onChange={this.onCheckAllChange}
                checked={this.state.checkAll}
              >
                全选/反选
              </Checkbox>

              <CheckboxGroup value={addModalData.grade} onChange={(e) => this.onGradeChange(e)}>
                {GRADE.map((g) => (
                  <Checkbox key={g.val} value={g.val}>
                    {g.text}
                  </Checkbox>
                ))}
              </CheckboxGroup>
            </FormItem>

            <FormItem labelWidth={100} name="状态" required>
              <Select
                placeholder="请选择状态"
                dict={statusOptions}
                value={addModalData.status}
                onChange={(value) => {
                  this.setState({
                    addModalData: {
                      ...addModalData,
                      status: value,
                    },
                  });
                }}
              />
            </FormItem>
          </Row>
        </div>
      </Modal>
    );
  }
}

export default connect((state) => {
  return {
    SUBJECT: state.dict['subject'] || [],
    GRADE: state.dict['grade'] || [],
    YEAR: state.dict['year'] || [],
    QUARTER: state.dict['quarter'] || [],
  };
})(AddCourseDialog);
