import React from 'react';
import { connect } from 'dva';
import { Page, TextLink } from 'COMPONENT';

import SelectTeacher from './selectTeacher';
import LimitRegular from './limitRegular';
import TeachableList from './teachableScheduleList';
import OperateHistory from './operateHistory';
import AddCourseDialog from './addCourseModal';
import SyncSubject from './syncSubject';
import ImportFile from './ImportFile';
import ImportResult from './importResult';

import style from './style.less';
const commonMarginRight = {
  marginRight: '25px',
};

const enableStatus = '0';

class TeacherTeachableSubject extends React.Component {
  constructor(props) {
    super(props);
    this.scheduleTableRef = null;
    this.operateHistoryRef = null;

    this.state = {
      // 当前选择的教师
      currentSelectedTeacher: {},
      selectableTeacherList: [],
      showAddSubjectDialog: false,
      showSyncSubjectDialog: false,
      showImportDialog: false,
      showResultDialog: false,
      isHaveUsableSubject: false,
      // 传递给同步科目弹框的数据，无数据代表全部同步
      // 有数据则为只同步指定的科目
      extraSyncData: null,
      importResult: {
        failCount: 0,
        fileKey: '',
      },
    };
  }

  componentDidMount() {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['grade', 'subject', 'year', 'quarter'],
      },
    });
  }

  onTeacherChange = (teacher) => {
    this.setState({
      currentSelectedTeacher: teacher,
    });
  };

  onTeacherNumberChange = (selectableTeacherList) => {
    this.setState({
      selectableTeacherList: selectableTeacherList,
    });
  };

  toAddSchedule = () => {
    this.setState({
      showAddSubjectDialog: true,
    });
  };

  toSchedulePlanPage = () => {
    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path: '/teacherCoursePlan',
        params: {
          teacher: this.state.currentSelectedTeacher,
        },
        refresh: true,
      },
    });
  };

  toModifyHistory = () => {
    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path: '/teacherRegularModifyHistory',
        params: {
          teacher: this.state.currentSelectedTeacher,
        },
        refresh: true,
      },
    });
  };

  toSyncSubject = () => {
    this.setState({
      showSyncSubjectDialog: true,
    });
  };

  toImportRegular = () => {
    this.setState({
      showImportDialog: true,
    });
  };

  refresh = () => {
    this.operateHistoryRef.fetchHistory();
    this.scheduleTableRef.fetchCourse();
  };

  render() {
    const { route, hasImportPermisison, hasSyncPermisison, hasRegularRecordPermisison, hasSeasonPlanPermisison } =
      this.props;

    const {
      name,
      loading,
      currentSelectedTeacher,
      selectableTeacherList,
      showAddSubjectDialog,
      showSyncSubjectDialog,
      showImportDialog,
      showResultDialog,
      isHaveUsableSubject,
      extraSyncData,
      importResult,
    } = this.state;

    return (
      <Page name={name} loading={loading} route={route} className={style['page-content']}>
        <div className={style['container']}>
          <SelectTeacher onTeacherChange={this.onTeacherChange} onTeacherNumberChange={this.onTeacherNumberChange} />
          {(selectableTeacherList.length || currentSelectedTeacher.value) && (
            <div className={style['table']}>
              <div className={style['chose-teacher']}>
                <span>老师{currentSelectedTeacher.label}的可教课程</span>
                <span>
                  {/*220510老师可教科目:屏蔽同步到季度方案*/}
                  {/*hasSyncPermisison && (
                    <TextLink
                      style={commonMarginRight}
                      onClick={this.toSyncSubject}
                    >
                      同步到季度方案
                    </TextLink>
                  )*/}

                  {hasImportPermisison && (
                    <TextLink style={commonMarginRight} onClick={this.toImportRegular}>
                      导入限课规则
                    </TextLink>
                  )}

                  {hasRegularRecordPermisison && (
                    <TextLink style={commonMarginRight} onClick={this.toModifyHistory}>
                      限课规则修改记录
                    </TextLink>
                  )}

                  {/*220510老师可教科目:班课季度排课计划*/}
                  {/*hasSeasonPlanPermisison && (
                    <TextLink
                      style={commonMarginRight}
                      onClick={this.toSchedulePlanPage}
                      disabled={!isHaveUsableSubject}
                    >
                      班课季度排课计划
                    </TextLink>
                  )*/}

                  <TextLink style={commonMarginRight} onClick={this.toAddSchedule}>
                    新增可教课程
                  </TextLink>
                </span>
              </div>

              {/* 限课规则 */}
              <LimitRegular teacher={currentSelectedTeacher} teacherId={currentSelectedTeacher.value} />

              {/* 老师可教课程 */}
              <TeachableList
                teacher={currentSelectedTeacher}
                refresh={this.refresh}
                ref={(e) => {
                  this.scheduleTableRef = e;
                }}
                onTeachableNumberChange={(number) => {
                  this.setState({
                    isHaveUsableSubject: number > 0,
                  });
                }}
                onEnableSubject={(params) => {
                  this.setState({
                    extraSyncData: params,
                    showSyncSubjectDialog: true,
                  });
                }}
              />
              <OperateHistory teacher={currentSelectedTeacher} ref={(e) => (this.operateHistoryRef = e)} />

              {/* 添加课程 */}
              {showAddSubjectDialog && (
                <AddCourseDialog
                  teacherId={currentSelectedTeacher.value}
                  onCancel={() => {
                    this.setState({
                      showAddSubjectDialog: false,
                    });
                  }}
                  onSuccess={(params) => {
                    this.refresh();
                    this.setState({
                      extraSyncData: params,
                      showAddSubjectDialog: false,
                      showSyncSubjectDialog: /*params.subjectStatus == enableStatus*/ false,
                    });
                  }}
                />
              )}

              {/* 同步课程 */}
              {showSyncSubjectDialog && (
                <SyncSubject
                  teacherId={currentSelectedTeacher.value}
                  teacherName={currentSelectedTeacher.label}
                  extraSyncData={extraSyncData}
                  onCancel={() => {
                    this.setState({
                      showSyncSubjectDialog: false,
                      extraSyncData: null,
                    });
                  }}
                  onSuccess={() => {
                    this.refresh();
                    this.setState({
                      showSyncSubjectDialog: false,
                      extraSyncData: null,
                    });
                  }}
                />
              )}

              {/* 导入课程 */}
              {showImportDialog && (
                <ImportFile
                  onClose={() => {
                    this.setState({
                      showImportDialog: false,
                    });
                  }}
                  onSuccess={(result) => {
                    this.refresh();
                    this.setState({
                      importResult: result,
                      showImportDialog: false,
                      showResultDialog: true,
                    });
                  }}
                />
              )}

              {showResultDialog && (
                <ImportResult
                  failCount={importResult.failCount}
                  fileKey={importResult.fileKey}
                  onCancel={() => {
                    this.setState({
                      showResultDialog: false,
                    });
                  }}
                />
              )}
            </div>
          )}
        </div>
      </Page>
    );
  }
}

export default connect((state) => {
  const { btnAuths } = state.account;

  const hasImportPermisison = btnAuths.indexOf('TEACHER_TEACHABLE_IMPORT') > -1;
  const hasSyncPermisison = btnAuths.indexOf('TEACHER_TEACHABLE_SYNC') > -1;
  const hasRegularRecordPermisison = btnAuths.indexOf('TEACHER_TEACHABLE_RECORD') > -1;
  const hasSeasonPlanPermisison = btnAuths.indexOf('TEACHER_TEACHABLE_SEASON_PLAN') > -1;

  return {
    SUBJECT: state.dict['subject'] || [],
    GRADE: state.dict['grade'] || [],
    YEAR: state.dict['year'],
    QUARTER: state.dict['quarter'],
    hasImportPermisison,
    hasSyncPermisison,
    hasRegularRecordPermisison,
    hasSeasonPlanPermisison,
  };
})(TeacherTeachableSubject);
