import React from 'react';
import { connect } from 'dva';
import { TextLink } from 'COMPONENT';
import LimitRegularModal from './limitRegularModal';
import style from './style.less';

import { getTeacherScheduleDetail } from 'SERVICE/teacherSchedule';

class LimitRegular extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      showRegularModal: false,
      detail: null,
    };
  }

  componentWillReceiveProps(props, preProps) {
    if (props.teacherId !== preProps.teacherId) {
      const YEAR = props.YEAR || preProps.YEAR;
      if (YEAR) {
        this.fetchSchedule(props.teacherId, YEAR);
      }
    }
  }

  fetchSchedule = (teacherId, passYear) => {
    const objectId = teacherId || this.props.teacherId;
    const YEAR = passYear || this.props.YEAR;
    const defaultYear = YEAR.find((y) => y.text == new Date().getFullYear());

    getTeacherScheduleDetail({
      type: 99,
      objectId,
      yearId: defaultYear.val,
    }).then((res) => {
      this.setState({
        detail: res.data,
      });
    });
  };

  render() {
    const { showRegularModal, detail } = this.state;
    const { teacherId, hasLimitRegularPermisison } = this.props;

    return (
      <div>
        <div className={style['table-regular-save']}>
          <div>
            <span>
              老师是否限课：
              {detail ? '限课' : '不限'}
            </span>

            {hasLimitRegularPermisison && (
              <TextLink
                style={{ marginLeft: '50px' }}
                onClick={() => {
                  this.setState({
                    showRegularModal: true,
                  });
                }}
              >
                选择/查看限课规则
              </TextLink>
            )}
          </div>
        </div>

        {showRegularModal && (
          <LimitRegularModal
            visible={showRegularModal}
            teacherId={teacherId}
            onClose={() => {
              this.setState({
                showRegularModal: false,
              });
            }}
            onSuccess={() => {
              this.fetchSchedule();
              this.setState({
                showRegularModal: false,
              });
            }}
          />
        )}
      </div>
    );
  }
}

export default connect((state) => {
  const { btnAuths } = state.account;

  const hasLimitRegularPermisison = btnAuths.indexOf('TEACHER_TEACHABLE_LIMIT_REGULAR') > -1;

  return {
    YEAR: state.dict['year'],
    hasLimitRegularPermisison,
  };
})(LimitRegular);
