import { Component } from 'react';
import moment from 'moment';
import { List, Loading } from 'COMPONENT';
import style from './style.less';
import { getTeacherOperateHistory } from 'SERVICE/teacherSchedule';

const tableColumns = [
  {
    title: '时间',
    dataIndex: 'createTime',
    key: 'createTime',
    render(text) {
      return moment(text).format('YYYY-MM-DD HH:ss:mm');
    },
  },
  {
    title: '操作详情',
    dataIndex: 'content',
    key: 'content',
  },
  {
    title: '操作人',
    dataIndex: 'createUserName',
    key: 'createUserName',
  },
];

class TeachableScheduleList extends Component {
  constructor(props) {
    super(props);

    this.state = {
      data: [],
      loading: false,
      total: 0,
      currentPage: 1,
      pageSize: 5,
      teacher: {},
    };
  }

  componentWillReceiveProps(props) {
    if (props.teacher.value !== this.state.teacher.value) {
      this.setState(
        {
          currentPage: 1,
          teacher: props.teacher,
        },
        this.fetchHistory,
      );
    }
  }

  setLoading = (loading) => {
    this.setState({
      loading,
    });
  };

  onPaginationChange = ({ current, pageSize }) => {
    this.setState(
      {
        currentPage: current,
        pageSize: pageSize,
      },
      this.fetchHistory,
    );
  };

  fetchHistory() {
    const { currentPage, pageSize, teacher } = this.state;

    this.setLoading(true);
    getTeacherOperateHistory({
      pageNum: currentPage,
      pageSize: pageSize,
      teacherId: teacher.value,
    })
      .then((res) => {
        this.setState({
          data: res.data.list,
          total: res.data.total,
        });
      })
      .finally(() => {
        this.setLoading(false);
      });
  }

  render() {
    const { data, loading, currentPage, total, pageSize } = this.state;
    const { teacher } = this.props;

    return (
      <div style={{ marginTop: '50px' }}>
        <Loading loading={loading} />
        <div className={style['chose-teacher']}>
          <span>老师{teacher.label}的操作流水</span>
        </div>
        <List
          bordered
          dataSource={data}
          total={total}
          columns={tableColumns}
          pagination={{
            current: currentPage,
            total: total,
            pageSize: pageSize,
            pageSizeOptions: ['5', '10', '15'],
          }}
          onChange={this.onPaginationChange}
        />
      </div>
    );
  }
}

export default TeachableScheduleList;
