import React from 'react';
import classnames from 'classnames';
import { getTeacherListSelectionV2 } from 'SERVICE/teacherSchedule';
import { Button, Input, Loading } from 'COMPONENT';
import style from './style.less';

class SelectTeacher extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      searchKeyword: '',
      teacherList: [],
      loading: false,
      currentSelected: {},
    };
  }

  componentDidMount() {
    this.fetchTeach();
  }

  fetchTeach = () => {
    this.setState({
      loading: true,
    });
    getTeacherListSelectionV2({
      teacherName: this.state.searchKeyword,
    })
      .then((res) => {
        const teacher = (res.data || []).map(({ name, userId, employeeNo }) => ({
          value: userId,
          label: `${name}${employeeNo ? `（${employeeNo}）` : ''}`,
        }));
        this.setState({ teacherList: teacher });
        this.props.onTeacherNumberChange(teacher);

        //默认选中第一个老师
        let defaultTeacher = teacher[0];
        if (defaultTeacher) {
          this.setState({ currentSelected: defaultTeacher });
          this.props.onTeacherChange(defaultTeacher);
        }
      })
      .finally(() => {
        this.setState({ loading: false });
      });
  };

  toggleTeacher = (teacher) => {
    if (teacher.value !== this.state.currentSelected.value) {
      this.setState({ currentSelected: teacher });
      this.props.onTeacherChange(teacher);
    }
  };

  render() {
    const { teacherList, loading, currentSelected } = this.state;

    return (
      <div className={style['search']}>
        <Loading loading={loading} />
        <div className={style['search-bar']}>
          <Input
            className={style['search-bar-input']}
            placeholder="输入老师姓名进行搜索"
            onChange={(e) => {
              this.setState({
                searchKeyword: e.target.value,
              });
            }}
            onPressEnter={this.fetchTeach}
          />
          <Button onClick={this.fetchTeach}>搜索</Button>
        </div>

        <ul className={style['search-content']}>
          {teacherList.map((user) => {
            return (
              <li
                key={user.value}
                className={classnames({
                  [style['search-content-item']]: true,
                  [style.active]: user.value === currentSelected.value,
                })}
                onClick={() => {
                  this.toggleTeacher(user);
                }}
              >
                {user.label}
              </li>
            );
          })}
          {teacherList.length === 0 && !loading ? <li className={style['search-content-empty']}>暂无数据</li> : null}
        </ul>
      </div>
    );
  }
}

export default SelectTeacher;
