import React from 'react';
import { connect } from 'dva';
import { Modal, Tree } from 'antd';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { syncTeacherSubject } from 'SERVICE/teacherSchedule';

const TreeNode = Tree.TreeNode;
const seasonMap = {
  spring: 'DAT0000000231',
  summer: 'DAT0000000232',
  autumn: 'DAT0000000233',
  winter: 'DAT0000000230',
};

class AddCourseDialog extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      treeData: [],
      checkedKeys: [],
      confirmLoading: false,
    };
  }

  onConfirm = () => {
    const { checkedKeys } = this.state;
    const { extraSyncData } = this.props;
    const yearMap = checkedKeys.reduce((pre, cur) => {
      const [year, quarter] = cur.split('-');

      if (quarter) {
        pre[year] = pre[year] || [];
        pre[year].push(quarter);
      }

      return pre;
    }, {});

    const syncDetailDtos = Object.keys(yearMap).map((y) => {
      return {
        yearId: y,
        quarterIds: yearMap[y],
      };
    });

    const params = {
      teacherId: this.props.teacherId,
      syncDetailDtos,
    };

    if (extraSyncData) {
      params.subjectIds = extraSyncData.subjectId.split(',');
      params.gradeIds = extraSyncData.gradeId.split(',');
    }

    this.setState({
      confirmLoading: true,
    });

    syncTeacherSubject(params)
      .then(() => {
        showMessage('同步成功', 'success');
        this.props.onSuccess();
      })
      .finally(() => {
        this.setState({
          confirmLoading: false,
        });
      });
  };

  renderTreeNodes = (data) => {
    return data.map((item) => {
      if (item.children) {
        return (
          <TreeNode title={item.text} key={item.key} dataRef={item}>
            {this.renderTreeNodes(item.children)}
          </TreeNode>
        );
      }
      return <TreeNode {...item} />;
    });
  };

  onCheck = (checkedKeys) => {
    this.setState({
      checkedKeys,
    });
  };

  render() {
    const { confirmLoading } = this.state;
    const { YEAR, QUARTER, teacherName, extraSyncData } = this.props;
    const avalidYear = YEAR.filter((y) => {
      return y.text >= new Date().getFullYear();
    });
    const avalidQuarter = QUARTER.filter((q) => {
      return Object.keys(seasonMap).some((s) => seasonMap[s] === q.val);
    });

    const treeData = avalidYear.map((y) => ({
      ...y,
      key: y.val,
      title: y.text,
      children: avalidQuarter.map((c) => ({
        ...c,
        title: c.text,
        key: `${y.val}-${c.val}`,
      })),
    }));

    const title = extraSyncData ? '是否把该变更同步到季度可教科目列表？' : `${teacherName}老师可教科目同步到以下季度`;

    return (
      <Modal
        title={title}
        width={700}
        visible={true}
        onCancel={this.props.onCancel}
        onOk={this.onConfirm}
        confirmLoading={confirmLoading}
        okText="确认同步"
      >
        <Tree defaultExpandAll checkable onCheck={this.onCheck}>
          {this.renderTreeNodes(treeData)}
        </Tree>
      </Modal>
    );
  }
}

export default connect((state) => {
  return {
    YEAR: state.dict['year'] || [],
    QUARTER: state.dict['quarter'] || [],
  };
})(AddCourseDialog);
