import { Component } from 'react';

import { List, Loading, TextLink } from 'COMPONENT';
import { Modal } from 'antd';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { getTeacherTeachableSubject, toggleTeacherSubjectStatus, deleteTeacherSubject } from 'SERVICE/teacherSchedule';

const DISABLE_STATUS = '1';
const ENABLE_STATUS = '0';
const tableColumns = [
  {
    title: '老师',
    dataIndex: 'teacherName',
    key: 'teacherName',
  },
  {
    title: '可教年级',
    dataIndex: 'gradeName',
    key: 'gradeName',
  },
  {
    title: '可教课程',
    dataIndex: 'subjectName',
    key: 'subjectName',
  },
  {
    title: '状态',
    dataIndex: 'subjectStatus',
    key: 'subjectStatus',
    render(text) {
      return text === DISABLE_STATUS ? '禁用' : text === ENABLE_STATUS ? '正常' : '';
    },
  },
];

class TeachableScheduleList extends Component {
  constructor(props) {
    super(props);

    this.state = {
      data: [],
      loading: false,
      total: 0,
      currentPage: 1,
      pageSize: 20,
      teacher: {},
    };
  }

  componentWillReceiveProps(props) {
    if (props.teacher.value !== this.state.teacher.value) {
      this.setState(
        {
          currentPage: 1,
          teacher: props.teacher,
        },
        this.fetchCourse,
      );
    }
  }

  setLoading = (loading) => {
    this.setState({
      loading,
    });
  };

  onPaginationChange = ({ current, pageSize }) => {
    this.setState(
      {
        currentPage: current,
        pageSize: pageSize,
      },
      this.fetchCourse,
    );
  };

  fetchCourse = () => {
    const { currentPage, pageSize, teacher } = this.state;

    this.setLoading(true);
    getTeacherTeachableSubject({
      _search: false,
      sidx: 'id',
      sord: 'desc',
      'teacher.userId': teacher.value,
      rows: pageSize,
      page: currentPage,
    })
      .then((res) => {
        this.setState({
          data: res.rows,
          total: res.records,
        });
        this.props.onTeachableNumberChange(res.records);
      })
      .finally(() => {
        this.setLoading(false);
      });
  };

  toggleCourseStatus = (row) => {
    toggleTeacherSubjectStatus({
      teacherSubjectId: row.id,
    }).then(() => {
      showMessage('操作成功', 'success');
      this.props.refresh();
      if (row.subjectStatus === DISABLE_STATUS) {
        this.props.onEnableSubject(row);
      }
    });
  };

  deleteCourse = (row) => {
    Modal.confirm({
      content: '提示：确定要删除吗？',
      okText: '确认',
      cancelText: '取消',
      onOk: () => {
        return new Promise((reslove) => {
          return deleteTeacherSubject({
            id: row.id,
            oper: 'del',
          })
            .then(() => {
              showMessage('操作成功', 'success');
              this.props.refresh();
            })
            .finally(() => {
              reslove();
            });
        });
      },
    });
  };

  render() {
    const { data, loading, currentPage, total, pageSize } = this.state;

    const columns = tableColumns.concat({
      title: '操作',
      dataIndex: 'operation',
      key: 'operation',
      render: (text, row) => {
        const enableBtn = (
          <TextLink
            style={{ marginRight: '15px' }}
            onClick={() => {
              this.toggleCourseStatus(row);
            }}
          >
            启用
          </TextLink>
        );

        const disableBtn = (
          <TextLink
            style={{ marginRight: '15px' }}
            onClick={() => {
              this.toggleCourseStatus(row);
            }}
          >
            禁用
          </TextLink>
        );

        return (
          <div>
            {row.subjectStatus === DISABLE_STATUS ? enableBtn : disableBtn}
            <TextLink
              onClick={() => {
                this.deleteCourse(row);
              }}
            >
              删除
            </TextLink>
          </div>
        );
      },
    });

    return (
      <div>
        <Loading loading={loading} />
        <List
          bordered
          dataSource={data}
          total={total}
          columns={columns}
          pagination={{
            current: currentPage,
            total: total,
            pageSize: pageSize,
            pageSizeOptions: ['20', '50', '100'],
          }}
          onChange={this.onPaginationChange}
        />
      </div>
    );
  }
}

export default TeachableScheduleList;
