import React from 'react';
import ReactDOM from 'react-dom';
import TreeSelectComp from 'COMPONENT/TreeSelect';
import styles from './styles.less';
import { findFirstParent } from 'COMPONENT/utils/dom';

export default class TreeMultiSelect extends React.Component {
  state = {
    selectionText: '',
  };

  searchKeyChangable = true;

  componentDidMount() {
    this.$root = ReactDOM.findDOMNode(this.rootRef);
    this.$root.addEventListener('click', this.rootClick, false);
    const $textWrapHide = this.$root.querySelector('.ant-select-selection__rendered');
    const $textWrapParent = $textWrapHide.parentNode;
    this.$textWrap = document.createElement('div');
    this.$textWrap.setAttribute('class', styles.textWrap);
    $textWrapParent.insertBefore(this.$textWrap, $textWrapHide);
  }

  componentWillReceiveProps(nextProps) {
    const { value } = nextProps;
    this.setState({ selectionText: value ? value.map((v) => v.label).join('，') : '' });
  }

  componentDidUpdate() {
    this.$textWrap.innerHTML = this.state.selectionText;
  }

  rootClick = (e) => {
    if (!this.$searchIpt) {
      setTimeout(() => {
        const $tree = this.$root.querySelector('.ant-select-tree');
        this.$searchIpt = document.createElement('input');
        this.$searchIpt.setAttribute('class', styles.searchInput);
        this.$searchIpt.placeholder = '请输入内容搜索';
        //this.$searchIpt.addEventListener('keydown', this.searchIptChange, false)
        this.$searchIpt.addEventListener('input', this.searchIptChange, false);
        this.$searchIpt.addEventListener('compositionstart', this.searchIptCompositionstart, false);
        this.$searchIpt.addEventListener('compositionend', this.searchIptCompositionend, false);
        let $searchIptWrap = document.createElement('div');
        $searchIptWrap.setAttribute('class', styles.searchInputWrap);
        $searchIptWrap.appendChild(this.$searchIpt);
        $tree.parentNode.insertBefore($searchIptWrap, $tree);
        this.$searchIpt.focus();
      }, 100);
    } else {
      const $dropdown = findFirstParent(e.target, '.ant-select-dropdown');
      if (!$dropdown) {
        setTimeout(() => {
          this.$searchIpt.focus();
        }, 100);
        if (this.$searchIpt.value) {
          this.$searchIpt.value = '';
          this.searchIptChange();
        }
      }
    }
  };

  componentWillUnmount() {
    this.$root.removeEventListener('click', this.rootClick, false);
    if (this.$searchIpt) {
      //this.$searchIpt.removeEventListener('keydown', this.searchIptChange, false)
      this.$searchIpt.removeEventListener('input', this.searchIptChange, false);
      this.$searchIpt.removeEventListener('compositionstart', this.searchIptCompositionstart, false);
      this.$searchIpt.removeEventListener('compositionend', this.searchIptCompositionend, false);
    }
  }

  searchIptCompositionstart = () => {
    this.searchKeyChangable = false;
  };

  searchIptCompositionend = () => {
    this.searchKeyChangable = true;
    this.searchIptChange();
  };

  searchIptChange = () => {
    if (this.searchKeyChangable) {
      this.props.onSearchKeyChange(this.$searchIpt.value);
    }
  };

  searchIptKeydown = (e) => {
    if (e.keyCode === 13) {
      e.stopPropagation();
      e.preventDefault();
    }
  };

  onSelect = (selected) => {
    let value = this.props.value || [];
    if (value.some((v) => v.value === selected.value)) {
      value = value.filter((v) => v.value !== selected.value);
    } else {
      value = [...value, selected];
    }
    value = value.length > 0 ? value : null;
    this.props.onChange(value);
  };

  onChange = (val) => {
    if (val.length === 0) {
      this.props.onChange(null);
    }
  };

  render() {
    const { onChange, ...props } = this.props;
    const { selectionText } = this.state;

    return (
      <div className={styles.wrap} ref={(c) => (this.rootRef = c)} title={selectionText}>
        <TreeSelectComp
          {...props}
          treeCheckStrictly={true}
          multiple={true}
          onSelect={this.onSelect}
          getPopupContainer={() => ReactDOM.findDOMNode(this.rootRef)}
          onChange={this.onChange}
        />
      </div>
    );
  }
}
