import React from 'react';
import { connect } from 'dva';
import IconButton from 'COMPONENT/IconButton';
import Input from 'COMPONENT/Input';
import Page from 'COMPONENT/Page';
import Modal from 'COMPONENT/Modal';
import SearchBar from 'COMPONENT/SearchBar';
import List from 'COMPONENT/List';
import Select from 'COMPONENT/Select';
import Popover from 'COMPONENT/Popover';
import TreeMultiSelect from 'COMPONENT/TreeMultiSelect';
import { getDictTextByVal, loadCommonDict, loadBranchDict, loadAllBranchDict } from 'UTIL/dict';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';
// import styles from './styles.less';
import loaders from './loaders';
import { statusDict, sexDict, STATUS_INVAILD } from './dicts';
import { getTreeNode } from './common';
import { debounce } from 'lodash';
import jslog from 'SRC/jslog';

const { COL_WIDTH_XS, COL_WIDTH_S, COL_WIDTH_M, COL_WIDTH_L, COL_WIDTH_XL } = List;

class UserManage extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false, // {boolean} 内容加载状态
      searchExpand: false, // {boolean} 搜索栏展开状态

      name: null, // {string} 查询条件 姓名
      phone: null, // {string} 查询条件 电话
      school: null, // {string} 查询条件 学校
      grade: null, // {string} 查询条件 年级
      entrance: null, // {string} 查询条件 资源入口
      isVisit: null, // {boolean} 查询条件 是否上门
      intention: null, // {string} 查询条件 意向度
      cusTag: null, // {string} 查询条件 客户标签
      newFollowTime: null, // {string} 查询条件 最新跟进时间
      nextFollowTime: null, // {string} 查询条件 下次跟进时间
      appointmentTime: null, // {string} 查询条件 预约上门时间

      workTypeDict: null, // {[object]} 字典 用工类型
      userPropDict: null, // {[object]} 字典 用户属性
      initOrgDict: null, // {[object]} 字典 初始显示的组织架构
      orgDict: null, // {[object]} 字典 组织架构
      sysRoleDict: null, // {[object]} 字典 系统角色

      orgTreeExpandedKeys: [], // {[string]} 组织架构树选框，展开的结点

      list: [], // {Array} 列表数据
      listSearchKeys: {
        workNum: null,
        name: null,
        account: null,
        workType: null,
        userProp: null,
        contract: null,
        org: null,
        sysRole: null,
        status: null,
      }, // {object} 列表数据的查询关键字
      totalRecords: 0, // {number} 总记录数
      currentPage: 1, // {number} 当前页码
      pageSize: 20, // {number} 每页记录数
    };
    this.orgDictVersion = 1;
  }

  componentDidMount() {
    this.initContent();
  }

  initContent = () => {
    this.setState({ loading: true });
    Promise.all([
      loaders.loadInitOrgDict(),
      loadCommonDict('UserWorkType'),
      loadCommonDict('UserType'),
      loaders.loadSysRoleDict(),
      loaders.loadList(this.getListParams()),
    ]).then(([orgDict, workTypeDict, userPropDict, sysRoleDict, listData]) => {
      const { list, totalRecords } = listData;
      this.setState({
        loading: false,
        initOrgDict: orgDict,
        orgDict,
        orgTreeExpandedKeys: [orgDict[0].val],
        workTypeDict,
        userPropDict,
        sysRoleDict,
        list,
        totalRecords,
      });
    });
  };

  getOrgParam = (org) => {
    return org ? org.map((or) => or.val).join(',') : null;
  };

  getListParams = () => {
    const {
      listSearchKeys: { workNum, name, account, workType, userProp, contract, org, sysRole, status },
      currentPage,
      pageSize,
    } = this.state;
    return { workNum, name, account, workType, userProp, contract, org, sysRole, status, currentPage, pageSize };
  };

  search = () => {
    const { workNum, name, account, workType, userProp, contract, org, sysRole, status } = this.state;
    const listSearchKeys = {
      workNum,
      name,
      account,
      workType,
      userProp,
      contract,
      org: this.getOrgParam(org),
      sysRole,
      status,
    };
    this.setState({ loading: true });
    loaders.loadList({ ...this.getListParams(), ...listSearchKeys, currentPage: 1 }).then(({ list, totalRecords }) => {
      jslog.report('query', {
        function_name: '系统管理>用户管理>查询',
        page: 1,
        total: totalRecords,
        size: list ? list.length : 0,
      });

      this.setState({ loading: false, listSearchKeys, currentPage: 1, list, totalRecords });
    });
  };

  reset = () => {
    this.setState({
      workNum: null,
      name: null,
      account: null,
      workType: null,
      userProp: null,
      contract: null,
      org: null,
      sysRole: null,
      status: null,
    });
  };

  editSaveSuccess = () => {
    const { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: { path: '/SystemUserManage' },
    });
    this.search();
  };

  goDetail = (id, type) => {
    const { dispatch } = this.props;
    const { userPropDict, initOrgDict, sysRoleDict } = this.state;
    dispatch({
      type: 'main/openPage',
      payload: {
        path: '/SystemUserManage/Detail',
        params: {
          type,
          id,
          onSaveSuccess: this.editSaveSuccess,
          userPropDict,
          initOrgDict,
          sysRoleDict,
        },
        refresh: true,
      },
    });
  };

  onPaginationChange = ({ current, pageSize }) => {
    this.setState({ loading: true });
    const changed = {
      currentPage: current,
      pageSize,
    };
    this.setState({ loading: true });
    loaders.loadList({ ...this.getListParams(), ...changed }).then(({ list, totalRecords }) => {
      this.setState({ loading: false, ...changed, list, totalRecords });
    });
  };

  onLoadOrgTreeData = (treeNode) => {
    return new Promise((resolve) => {
      if (treeNode.props.children) {
        resolve();
        return;
      }
      const key = treeNode.props.eventKey;
      loaders.loadOrgDictByParentId(key).then((data) => {
        this.setState({
          orgDict: [...this.state.orgDict, ...data],
          orgTreeExpandedKeys: [...this.state.orgTreeExpandedKeys, key],
        });
        resolve();
      });
    });
  };

  onOrgTreeSearch = (key) => {
    debounce(this._onOrgTreeSearch, 300)(key);
  };

  _onOrgTreeSearch = (key) => {
    if (key) {
      this.orgDictVersion++;
      loaders.loadOrgDictByName(key, this.orgDictVersion).then(({ dict: orgDict, version }) => {
        if (this.orgDictVersion === version) {
          this.setState({ orgDict, orgTreeExpandedKeys: [] });
        }
      });
    } else {
      this.orgDictVersion++;
      this.setState({ orgDict: this.state.initOrgDict, orgTreeExpandedKeys: [this.state.initOrgDict[0].val] });
    }
  };

  render() {
    const {
      loading,
      searchExpand,
      workNum,
      name,
      account,
      workType,
      userProp,
      contract,
      org,
      sysRole,
      status,
      workTypeDict,
      userPropDict,
      orgDict,
      sysRoleDict,
      list,
      totalRecords,
      currentPage,
      pageSize,
      orgTreeExpandedKeys,
    } = this.state;
    const statusTitle = (
      <Popover content="有效无效状态切换需要在OA系统上进行用户状态操作">
        <IconButton type="question_mc" />
      </Popover>
    );
    const columns = [
      { title: '工号', dataIndex: 'workNum', fixed: 'left' },
      { title: '真实姓名', dataIndex: 'name', fixed: 'left' },
      { title: '登录账号', dataIndex: 'account' },
      { title: '性别', dataIndex: 'sex', width: COL_WIDTH_S, dict: sexDict },
      /*      { title: '用工类型', dataIndex: 'workType', dict: workTypeDict },*/
      { title: '联系方式', dataIndex: 'contract' },
      { title: '组织架构', dataIndex: 'org', width: COL_WIDTH_XL, lines: 1 },
      { title: '角色', dataIndex: 'sysRole', width: COL_WIDTH_XL, lines: 1 },
      { title: '状态', dataIndex: 'status', width: COL_WIDTH_S, dict: statusDict },
      {
        title: '操作',
        key: 'action',
        getActionConfig: (text, record) => {
          let tempStatus = false; // 操作者和被操作者是否都是超级管理员或者系统管理员
          let tempRoleCode = this.props.roleCode.indexOf('SUPER_ADMIN') > -1 ? 0 : -1;
          // 如果当前登录角色tempRoleCode为0，代表超级管理员

          const { id, status, sysRoleCodeType } = record;
          // 如果当前行的sysRoleCodeType为0，代表超级管理员

          // 当前行为超级，但是登录角色不为超级，不可编辑
          if (sysRoleCodeType == 0 && sysRoleCodeType != tempRoleCode) {
            tempStatus = true;
          }
          return [
            { text: '修改', onClick: () => this.goDetail(id), disabled: status === STATUS_INVAILD || tempStatus },
            { text: '详情', onClick: () => this.goDetail(id, 'view') },
          ];
        },
      },
    ];

    return (
      <Page
        fitWindowHeight={true}
        loading={loading}
        route={this.props.route}
        subHeader={
          <SearchBar
            expand={searchExpand}
            onToggleExpand={() => this.setState({ searchExpand: !searchExpand })}
            onSearch={this.search}
            onReset={this.reset}
          >
            <Input placeholder="工号" value={workNum} onChange={(e) => this.setState({ workNum: e.target.value })} />
            <Input placeholder="姓名" value={name} onChange={(e) => this.setState({ name: e.target.value })} />
            <Input
              placeholder="登录账号"
              value={account}
              onChange={(e) => this.setState({ account: e.target.value })}
            />
            {/*              <Select placeholder="用工类型" dict={workTypeDict} value={workType} onChange={val => this.setState({ workType: val })} />*/}
            <Select
              placeholder="用户属性"
              dict={userPropDict}
              value={userProp}
              onChange={(val) => this.setState({ userProp: val })}
            />
            <Input
              placeholder="联系方式"
              value={contract}
              onChange={(e) => this.setState({ contract: e.target.value })}
            />

            <TreeMultiSelect
              value={org}
              onChange={(val) => this.setState({ org: val })}
              dict={orgDict}
              placeholder="组织架构"
              onLoadChildren={this.onLoadOrgTreeData}
              onSearchKeyChange={this.onOrgTreeSearch}
              textInValue
              expandedKeys={orgTreeExpandedKeys}
              onExpandChange={(orgTreeExpandedKeys) => this.setState({ orgTreeExpandedKeys })}
            />
            <Select
              placeholder="系统角色"
              dict={sysRoleDict}
              showSearch
              value={sysRole}
              onChange={(val) => this.setState({ sysRole: val })}
            />
            <Select
              placeholder="状态"
              dict={statusDict}
              value={status}
              onChange={(val) => this.setState({ status: val })}
            />
          </SearchBar>
        }
      >
        <List
          dataSource={list}
          columns={columns}
          pagination={{
            current: currentPage,
            total: totalRecords,
            pageSize,
          }}
          onChange={this.onPaginationChange}
        />
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    roleCode: state.account.user.roleCode,
  };
};

export default connect(mapStateToProps, null)(UserManage);
