import { getBossOrganizationList, getUserInfoList, getUserInfoByUserId, modifyUserInfo } from 'SERVICE/system';
import { getRoleForSelection } from 'SERVICE/common';
import { SPARK_ID } from './common';
import { formateDict } from 'UTIL/dict';
import { indexOf } from 'lodash';
import moment from 'moment';

/*const loadList = async (params) => {
  console.log('loadList params:', params)
  let fakeData = []
  for (let i = 0; i < 33; i++) {
    fakeData.push({
      id: i + '',
      workNum: 'X10' + i,
      name: '用户' + i,
      account: 'account' + i,
      sex: null,
      workType: null,
      contract: '13377788987',
      org: '集团市场部（主）、莞城学习中心、城市广场学习中心、雍华庭学习中心、xx学习中心、yy学习中心',
      sysRole: '市场经理（主）、学习中心主任、运营副主任、老师、学管主管',
      status: null,
    })
  }
  const rtn = {
    list: fakeData,
    totalRecords: fakeData.length,
  }
  console.log(rtn)
  return rtn
}*/

const loadList = async (params) => {
  /*
  { workNum, name, account, workType, userProp, contract, org, sysRole, status, currentPage, pageSize }
   */
  const apiParams = {
    pageNo: params.currentPage - 1,
    pageSize: params.pageSize,
    contact: params.contract,
    name: params.name,
    account: params.account,
    enableFlag: params.status ? parseInt(params.status) : null,
    employeeNo: params.workNum,
    workType: params.workType,
    userType: params.userProp,
    orgId: params.org,
    roleId: params.sysRole,
  };
  const { datas, rowCount } = await getUserInfoList(apiParams, true);
  const list = datas.map((d) => {
    // 获取用户的最高角色（超级管理员和集团管理员，没有则为空）
    let tempSysRoleCodeArr = d.orgRoles.map((or) => or.roleCode);
    let tempNotMainOrgRoles = d.notMainOrgRoles.map((or) => or.roleCode);
    let sumRoleCode = tempSysRoleCodeArr.concat(tempNotMainOrgRoles);
    let tempSysRoleCode = -1;
    if (sumRoleCode.length) {
      if (indexOf(sumRoleCode, 'SUPER_ADMIN') > -1) {
        tempSysRoleCode = 0;
      }
    }
    return {
      id: d.userId,
      workNum: d.employeeNo,
      name: d.name,
      account: d.account,
      sex: d.sex + '',
      workType: d.workType,
      contract: d.contact,
      org: d.orgRoles.map((or) => or.organizationName).join('、'),
      sysRole: d.orgRoles.map((or) => or.roleName).join('、'),
      sysRoleCodeType: tempSysRoleCode,
      sysRoleCode: sumRoleCode.join(','),
      status: d.enableFlag + '',
    };
  });
  const rtn = {
    list,
    totalRecords: rowCount,
  };
  return rtn;
};

const loadInitOrgDict = async () => {
  let params = { type: 'BOSS', level: 2 };
  const apiData = await getBossOrganizationList(params, true);
  return apiData.data.map((d) => {
    return {
      val: d.id,
      text: d.name,
      isLeaf: d.hasChildren > 0 ? false : true,
      parentVal: d.id === SPARK_ID ? null : d.parentId,
    };
  });
};

const loadOrgDictByParentId = async (parentId) => {
  const params = { type: 'BOSS', parentId };
  const apiData = await getBossOrganizationList(params, true);
  const data = apiData.data.map((d) => {
    return {
      val: d.id,
      text: d.name,
      isLeaf: d.hasChildren > 0 ? false : true,
      parentVal: d.parentId,
    };
  });
  return data;
};

const loadOrgDictByName = async (name, version) => {
  const params = { type: 'BOSS', name };
  const apiData = await getBossOrganizationList(params, true);
  const data = apiData.data.map((d) => {
    return { val: d.id, text: d.name, isLeaf: true, parentVal: null };
  });
  return { dict: data, version };
};

const filterList = ['紧急状态管理员'];

const loadSysRoleDict = async (name) => {
  const params = {};
  const apiData = await getRoleForSelection(params, true);
  const value = apiData ? apiData.value : {};
  let dict = [];
  Object.keys(value).forEach((key) => {
    if (key && filterList.indexOf(value[key]) < 0) {
      dict.push({ val: key, text: value[key], title: value[key] });
    }
  });
  return dict;
};

const loadUserDetail = async (id) => {
  const params = { userId: id };
  const { data } = await getUserInfoByUserId(params, true);
  const userOrgRoles = data.userOrgRoles || [];
  const userInfo = data.userInfo || {};
  const stationInfo = userInfo.stationInfo || [];
  let postInfo = [];
  stationInfo.forEach((info) => {
    const isMain = info.isMajor === 0 ? true : false;
    const d = {
      dep: info.orgName,
      post: info.stationName,
      isMain,
    };
    if (isMain) {
      postInfo = [d, ...postInfo];
    } else {
      postInfo.push(d);
    }
  });
  let authInfo = [];
  let exAuthInfo = [];
  userOrgRoles.forEach((info) => {
    const isMain = info.isMain === 0 ? true : false;
    const d = {
      org: info.organizationId,
      role: info.roleId,
      orgName: info.organizationName,
      roleName: info.roleName,
      isMain,
      info,
      validityPeriod: info.validityPeriod ? moment(info.validityPeriod).format('YYYY-MM-DD') : '',
    };
    const isFiltered = filterList.indexOf(info.roleName) > -1;
    if (!isFiltered) {
      if (isMain) {
        authInfo = [d, ...authInfo];
      } else {
        authInfo.push(d);
      }
    } else {
      exAuthInfo.push(d);
    }
  });
  const list = (data.logs || []).map((d) => {
    return {
      datetime: d.createTime,
      detail: d.logInfo,
      user: d.createUserName,
      opReason: d.opReason,
    };
  });
  /*  let fakeList = []
  for (let i = 0; i < 33; i++) {
    fakeList.push({
      datetime: '2011-11-11 00:09:12',
      detail: '新增东莞分公司-市场经理；新增莞城学习中心学管组-学管师' + i,
      user: 'xhadmin',
    })
  }*/
  return {
    realName: userInfo.realName,
    workNum: userInfo.employeeNo,
    name: data.userName,
    account: userInfo.account,
    contract: userInfo.contact,
    sex: userInfo.sex,
    status: userInfo.enableFlag,
    workType: userInfo.workType,
    postInfo,
    inDate: userInfo.entryDate,
    outDate: userInfo.dimissionDate,
    userProp: data.type,
    authInfo,
    exAuthInfo,
    list,
  };
};

const saveDetail = async (data) => {
  const userOrgRoles = data.authInfo.map((a) => {
    return {
      isMain: a.isMain ? 0 : 1,
      organization: { id: a.org },
      role: { roleId: a.role },
      user: { userId: data.id },
      validityPeriod: a.validityPeriod,
    };
  });
  const params = {
    userId: data.id,
    type: data.userProp,
    userOrgRoles,
    opReason: data.opReason,
  };
  try {
    await modifyUserInfo(params, true);
    return true;
  } catch (e) {
    throw e;
  }
};

export default {
  loadList,
  loadInitOrgDict,
  loadOrgDictByParentId,
  loadOrgDictByName,
  loadSysRoleDict,
  loadUserDetail,
  saveDetail,
};
