import React from 'react';
import { connect } from 'dva';
import Page from 'COMPONENT/Page';
import Block from 'COMPONENT/Block';
import Button from 'COMPONENT/Button';
import TextArea from 'COMPONENT/TextArea';
import { Form, Input, Row, Col, Switch } from 'antd';
import loaders from './loaders';
import moment from 'moment';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import styles from './styles.less';
import MultiSwitchModal from './MultiSwitchModal';
import FormItem from 'COMPONENT/FormItem';
import showConfirm from 'COMPONENT/utils/globalTips/showConfirm';

const FormItemNative = Form.Item;
const workTypeObj = {
  FULL_TIME: '全职',
  PART_TIME: '兼职',
  DUMMY: '虚拟',
};
const sexObj = {
  1: '男',
  0: '女',
};

const formItemCol = {
  labelWidth: 80,
  style: {
    padding: '0 10px',
  },
};

const formItemInfoCol = {
  labelWidth: 80,
  style: {
    padding: '0 10px',
    marginBottom: 0,
    minHeight: '26px',
    lineHeight: '26px',
  },
};

const formItemInfoSecCol = {
  labelWidth: 80,
  style: {
    padding: '0 10px',
    minHeight: '26px',
    lineHeight: '26px',
  },
};

const formItemColExtra = {
  labelWidth: 50,
  style: { padding: '0px 10px 0 50px' },
};

class UserPersonalManage extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      loading: false,
      disabled: false,
      user: {},
      switchable: this.props.btnAuths.indexOf('BTN_CHANGE_EMERGENCY_STATU') >= 0,
      status: '',
      list: [],
      multiSwitchVisiable: false,
      isAdmin: false,
    };

    const { deptRoles } = this.props.user;
    this.multiCampusList = Object.keys(deptRoles)
      .map((ele) => ele && ele.split('_')[0])
      .filter((ele, i, arr) => arr.lastIndexOf(ele) === i);
  }

  handleSubmit = (e) => {
    e.preventDefault();
    const { userId } = this.props.user;
    this.props.form.validateFieldsAndScroll(async (err, values) => {
      this.setState({ loading: true, disabled: true });
      try {
        const { workType, account, sex, contact, password } = this.state.user;
        if (err) {
          return showMessage('保存失败：请确认必填输入项是否填写完毕', 'error');
        }
        await loaders.editUserPersonal({
          ...values,
          userId,
          workType,
          account,
          sex,
          contact,
          password,
        });
        showMessage(`保存成功`, 'success');
      } catch (err) {
        showMessage(`保存失败：${err}`, 'error');
      }
      this.setState({ loading: false, disabled: false });
    });
  };

  onSwitchChange = (e) => {
    const list = this.state.list;
    const campusStatus = list[0]['children'][0];
    showConfirm(`${campusStatus.name}将切换为${e ? '紧急' : '普通'}状态，请确认是否切换？`, async () => {
      this.setState({ loading: true, disabled: true });
      try {
        await loaders.updateOrgStatusByIdNew({ organizationId: campusStatus.id, statu: +!!e });
        showMessage(`切换成功`, 'success');
      } catch (err) {
        showMessage(`切换失败：${err}`, 'error');
      }
      campusStatus.status = e ? 'EMERGENCY' : 'NORMAL';
      this.setState({ list, disabled: false, loading: false });
    });
  };

  componentDidMount() {
    this.init();
  }

  async init() {
    this.setState({ loading: true });

    const { name, userId, brenchId: branchId, roleCode, organizationType } = this.props.user;
    let params = { userId };
    if (roleCode && roleCode.indexOf('SUPER_ADMIN') >= 0) {
      params = {};
      this.setState({ isAdmin: true });
    }
    const list = await loaders.loadOrganization(params);
    this.setState({ list });

    const user = await loaders.findUserById({
      monitoringUserId: userId,
      monitoringUserName: name,
      monitoringHtml: 'userpersonalmanage',
      monitoringTime: moment().format('YYYY-MM-DD HH-mm-ss'),
    });

    this.props.form.setFieldsValue({
      emengentPerson: user.emengentPerson,
      emengentPersonContact: user.emengentPersonContact,
      address: user.address,
      graduateSchool: user.graduateSchool,
      education: user.education,
      certification: user.certification,
    });
    this.setState({ user, loading: false });
  }

  toolBar = () => {
    const { switchable, list } = this.state;
    const { brenchId: branchId, campusId, organizationType } = this.props.user;
    let containEmergencyTag = 0;
    const filterList = list.filter((ele) => ele.emergency);
    containEmergencyTag = filterList.length;
    if (!switchable || !list.length || containEmergencyTag > 0) {
      return '';
    }
    const action =
      list.length === 1 && list[0]['children'].length === 1 ? (
        <div className={styles.campusStatus} style={{ float: 'left' }}>
          <Switch
            checkedChildren="紧急"
            unCheckedChildren="正常"
            checked={list[0]['children'][0]['status'] === 'EMERGENCY'}
            onChange={this.onSwitchChange}
          />
        </div>
      ) : (
        <Button type="primary" onClick={() => this.setState({ multiSwitchVisiable: true })}>
          紧急切换
        </Button>
      );
    return (
      <Row type="flex" justify="start">
        <Col span={5}>
          <FormItem name="切换状态" {...formItemCol}>
            <div>{action}</div>
          </FormItem>
        </Col>
      </Row>
    );
  };

  render() {
    const { getFieldDecorator } = this.props.form;
    const { brenchId: branchId, userId } = this.props.user;
    const { organization, role, name, workType, account, sex, contact, password, employeeNo } = this.state.user;
    const { isAdmin, loading, disabled, multiSwitchVisiable } = this.state;
    return (
      <Page route={this.props.route} loading={loading}>
        <Form onSubmit={this.handleSubmit}>
          <Block title={false}>
            <Row type="flex" justify="start">
              <Col span={24}>
                <h2 className={styles.header}>{name || '基础信息'}</h2>
              </Col>
            </Row>
            <Row type="flex" justify="start">
              <Col span={5}>
                <FormItem name="工号：" {...formItemInfoCol}>
                  {employeeNo}
                </FormItem>
              </Col>
              <Col span={6}>
                <FormItem name="学习中心：" {...formItemInfoCol}>
                  {organization && organization.length ? organization[0].name : ''}
                </FormItem>
              </Col>
              <Col span={6}>
                <FormItem name="用户类型：" {...formItemInfoCol}>
                  {workTypeObj[workType] || ''}
                </FormItem>
              </Col>
              <Col span={6}>
                <FormItem name="登陆账号：" {...formItemInfoCol}>
                  {account}
                </FormItem>
              </Col>
            </Row>
            <Row type="flex" justify="start">
              <Col span={5}>
                <FormItem name="性别：" {...formItemInfoSecCol}>
                  {sexObj[sex]}
                </FormItem>
              </Col>
              <Col span={6}>
                <FormItem name="职位：" {...formItemInfoSecCol}>
                  {role && role.length ? role[0].name : ''}
                </FormItem>
              </Col>
              <Col span={6}>
                <FormItem name="联系方式：" {...formItemInfoSecCol}>
                  {contact}
                </FormItem>
              </Col>
              <Col span={6}>
                <FormItem name="登陆密码：" {...formItemInfoSecCol}>
                  {password && new Array(password.length).fill('*').join('')}
                </FormItem>
              </Col>
            </Row>
            {this.toolBar()}
          </Block>
          <div className={styles.hrContainer}>
            <div className={styles.hr}></div>
          </div>
          <Block title={false}>
            <Row type="flex" justify="start">
              <Col span={24}>
                <h2 className={styles.header}>个人信息</h2>
              </Col>
            </Row>
            <Row type="flex" justify="start">
              <Col span={8}>
                <FormItem name="紧急联系人" {...formItemCol}>
                  <FormItemNative>
                    {getFieldDecorator('emengentPerson', {})(<Input placeholder={'紧急联系人'} />)}
                  </FormItemNative>
                </FormItem>
              </Col>
              <Col span={9}>
                <FormItem name="电话" {...formItemColExtra}>
                  <FormItemNative>
                    {getFieldDecorator('emengentPersonContact', {})(<Input placeholder={'电话'} />)}
                  </FormItemNative>
                </FormItem>
              </Col>
            </Row>
            <Row type="flex" justify="start">
              <Col span={17}>
                <FormItem name="现在住址" {...formItemCol}>
                  <FormItemNative>
                    {getFieldDecorator('address', {})(<Input placeholder={'现在住址'} />)}
                  </FormItemNative>
                </FormItem>
              </Col>
            </Row>
            <Row type="flex" justify="start">
              <Col span={17}>
                <FormItem name="毕业院校" {...formItemCol}>
                  <FormItemNative>
                    {getFieldDecorator('graduateSchool', {})(<Input placeholder={'毕业院校'} />)}
                  </FormItemNative>
                </FormItem>
              </Col>
            </Row>
            <Row type="flex" justify="start">
              <Col span={17}>
                <FormItem name="学历" {...formItemCol}>
                  <FormItemNative>{getFieldDecorator('education', {})(<Input placeholder={'学历'} />)}</FormItemNative>
                </FormItem>
              </Col>
            </Row>
            <Row type="flex" justify="start">
              <Col span={17}>
                <FormItem name="证书" {...formItemCol}>
                  <FormItemNative>
                    {getFieldDecorator('certification', {})(<TextArea placeholder={'证书'} />)}
                  </FormItemNative>
                </FormItem>
              </Col>
            </Row>
            <Row type="flex" justify="start">
              <Col span={8}>
                <FormItem name="" {...formItemCol}>
                  <Button type="primary" htmlType="submit" disabled={disabled}>
                    保存
                  </Button>
                </FormItem>
              </Col>
            </Row>
          </Block>
        </Form>
        {multiSwitchVisiable && (
          <MultiSwitchModal
            branchId={branchId}
            userId={userId}
            isAdmin={isAdmin}
            onClose={() => this.setState({ multiSwitchVisiable: false })}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    user: state.account.user,
    btnAuths: state.account.btnAuths,
  };
};

export default connect(mapStateToProps)(Form.create()(UserPersonalManage));
