import React from 'react';
import { connect } from 'dva';
import { Modal, DatePicker, Select, List, ListAction } from 'COMPONENT';
import { Form, Row, Col, Modal as antModal, message } from 'antd';
import { expireAgents } from 'SERVICE/callCenter';
import PropTypes from 'prop-types';
import { renewTypeDict } from '../config';
import moment from 'moment';
import np from 'number-precision';
const FormItem = Form.Item;
const momentFormat = 'YYYY-MM-DD HH:mm:ss';
class ModalBatchExpire extends React.Component {
  static protpTypes = {
    data: PropTypes.arrayOf.required,
    onCancel: PropTypes.func,
    onSuccess: PropTypes.func,
    clickType: PropTypes.oneOf(['batch', 'column']),
  };
  static defaultTypes = {
    data: [],
  };
  constructor(props) {
    super(props);
    this.state = {
      seatList: this.props.data,
      newExpireDate: '',
      expireMount: 0,
    };
    this.seatColumns = [
      { title: '坐席ID', dataIndex: 'agentId' },
      { title: '坐席名称', dataIndex: 'agentName' },
      { title: '所属组织', dataIndex: 'organizationName' },
      { title: '创建时间', dataIndex: 'createTime', render: (text) => moment(text).format(momentFormat) },
      {
        title: '到期日期',
        dataIndex: 'expireTime',
        dataIndex: 'expireTime',
        render: (text) => (text ? moment(text).format(momentFormat) : ''),
      },
      {
        title: '到期状态',
        dataIndex: 'expireStatus',
        key: 'expireStatus',
        render: (text) => (!text ? '' : text === 1 ? '未到期' : '已到期'),
      },
      {
        title: '操作',
        dataIndex: '',
        width: 70,
        fixed: 'right',
        render: (val, row, index) => (
          <ListAction
            config={[
              {
                text: '移除',
                onClick: () => {
                  this.deleteColumn(row);
                },
              },
            ]}
          />
        ),
      },
    ];
    if (this.props.clickType === 'column') {
      this.seatColumns.pop();
    }
    this.onSubmit = this.onSubmit.bind(this);
    this.handleChange = this.handleChange.bind(this);
    this.disabledDate = this.disabledDate.bind(this);
    this.calcExpire = this.calcExpire.bind(this);
  }

  componentDidMount() {
    this.calcExpire();
  }

  calcExpire() {
    if (this.state.seatList.length) {
      const arr = this.state.seatList.filter((item) => item.expireStatus == 2); // 到期数量
      this.setState({
        expireMount: arr.length,
      });
    }
  }

  onSubmit() {
    // 提交
    this.props.form.validateFields(async (error, form) => {
      if (!error) {
        const params = {
          startTime: form.date,
          expireTime: this.state.newExpireDate,
          cycle: form.type,
          agentIds: this.props.data.map((item) => item.agentId),
        };
        const { resultCode } = await expireAgents(params);
        if (resultCode === 0) {
          message.success('续费成功');
          this.props.onSuccess && this.props.onSuccess();
          this.props.dispatch({
            type: 'callCenter/fetchToUpdAgentInfo',
          });
        }
      }
    });
  }

  // 移除
  deleteColumn(deleteRow) {
    this.setState(
      {
        seatList: this.state.seatList.filter((item) => item.id !== deleteRow.id),
      },
      () => {
        if (this.state.seatList.length === 0) {
          antModal.warning({
            content: '已无选中坐席，请返回重新选择！',
            okText: '返回',
            onOk: this.props.onCancel,
          });
        } else {
          this.calcExpire();
        }
      },
    );
  }
  handleChange() {
    let date = this.props.form.getFieldValue('date');
    const circle = this.props.form.getFieldValue('type');
    let newDate = 0;
    if (!date || !circle) {
      this.setState({
        newExpireDate: '',
      });
      return;
    }
    if (circle == 1) {
      // 半年 -> 180天
      const newTime = np.plus(new Date(date).getTime(), 15552000000);
      newDate = moment(newTime).format('YYYY-MM-DD');
    } else if (circle == 2) {
      // 一年 -> 365天
      const newTime = np.plus(new Date(date).getTime(), 31536000000);
      newDate = moment(newTime).format('YYYY-MM-DD');
    } else {
      newDate = '';
    }
    this.setState({
      newExpireDate: newDate,
    });
  }
  disabledDate(currentDate) {
    return new Date(currentDate).getTime() < new Date(this.state.seatList[0]['createTime']).getTime();
  }
  render() {
    const { onCancel, data } = this.props;
    const { getFieldDecorator } = this.props.form;
    const formItemLayout = {
      labelCol: { span: 5 },
      wrapperCol: { span: 15 },
    };
    return (
      <Modal title="续费操作" onCancel={onCancel} onOk={this.onSubmit} widthSize="M">
        <Form onSubmit={this.onSubmit}>
          <FormItem label="设置开始日期" {...formItemLayout}>
            {getFieldDecorator('date', {
              initialValue: moment(Date.now()).format('YYYY-MM-DD'),
              rules: [{ required: true, message: '设置开始日期' }],
            })(<DatePicker size="small" onBlur={this.handleChange} disabledDate={this.disabledDate} />)}
          </FormItem>
          <FormItem label="选择续费周期" {...formItemLayout}>
            {getFieldDecorator('type', {
              rules: [{ required: true, message: '选择续费周期' }],
            })(<Select dict={renewTypeDict} onBlur={this.handleChange} />)}
          </FormItem>
          <Row>
            <Col span={24}>到期日期：{this.state.newExpireDate}</Col>
            <Col span={24}>
              已选{this.state.seatList.length}条坐席数据，其中{this.state.expireMount}条已到期
            </Col>
            <Col span={24}>
              <List columns={this.seatColumns} dataSource={this.state.seatList} />
            </Col>
          </Row>
        </Form>
      </Modal>
    );
  }
}

export default connect()(Form.create()(ModalBatchExpire));
