import React from 'react';
import { connect } from 'dva';
import styles from './styles.module.less';
import { Page, SearchFilterForm, SearchFieldForm, Input, Select, List, ListAction, Button } from 'COMPONENT';
import { Table } from 'antd';
import { errorsHandle } from 'UTIL/common';
import { clone, trim } from 'lodash';
import { assignTypeDict, agentStatusDict, expireTypeDict } from './config';
import { callAgentStatusCfg } from 'CONST/callConfig';
import ModalAgentDetail from '../ModalAgentDetail';
import ModalAgentAssign from './ModalAgentAssign';
import moment from 'moment';
import { pageAgents, exportList } from 'SERVICE/callCenter';
import { getRelationsDictPlus } from 'MODEL/dict/utils/index';
import { campus } from 'MODEL/dict/utils/config';
import { message } from 'antd';
import ModalBatchExpire from './ModalBatchExpire';
import { download } from 'UTIL/file';

const setWidth = List.createSetWidth();
const errorsHandler = errorsHandle();
const momentFormat = 'YYYY-MM-DD HH:mm:ss';

class BindCallAgent extends React.Component {
  constructor(props) {
    super(props);

    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.getListParams = this.getListParams.bind(this);
    this.loadList = this.loadList.bind(this);
    this.search = this.search.bind(this);
    this.updatePage = this.updatePage.bind(this);
    this.changePage = this.changePage.bind(this);
    this.refreshList = this.refreshList.bind(this);
    this.switchModalAgentDetail = this.switchModalAgentDetail.bind(this);
    this.onFieldsChange = this.onFieldsChange.bind(this);
    this.selectCallAgent = this.selectCallAgent.bind(this);
    this.tableRowClick = this.tableRowClick.bind(this);
    this.batchAssign = this.batchAssign.bind(this);
    this.closeModalAgentAssign = this.closeModalAgentAssign.bind(this);
    this.batchRenew = this.batchRenew.bind(this);
    this.batchExport = this.batchExport.bind(this);
    this.closeModalBatchExpire = this.closeModalBatchExpire.bind(this);
    this.expireSuccess = this.expireSuccess.bind(this);

    this.state = {
      loading: false,
      searchKey: null,
      list: [],
      total: 0,
      pageCurrent: 1,
      pageSize: 10,
      modalAgentDetailVisible: false,
      branchId: null,
      campusDict: null,
      selectedRowKeys: [],
      modalAgentAssignVisible: false,
      modalExpireVisible: false, // 批量续费Modal
      scrollY: 0,
    };

    this.columns = [
      { title: '坐席ID', dataIndex: 'agentId', key: 'agentId', width: setWidth(6) },
      { title: '坐席名称', dataIndex: 'agentName', key: 'agentName', width: setWidth(5) },
      { title: '坐席昵称', dataIndex: 'agentNick', key: 'agentNick', width: setWidth(5) },
      { title: '邮箱账号', dataIndex: 'agentEmail', key: 'agentEmail', width: setWidth(6) },
      { title: '所属组织', dataIndex: 'organizationName', key: 'organizationName', width: setWidth(6) },
      { title: '用户绑定数', dataIndex: 'bindNumber', key: 'bindNumber', width: setWidth(6) },
      { title: '坐席角色', dataIndex: 'agentRoleName', key: 'agentRoleName', width: setWidth(5) },
      { title: '技能组', dataIndex: 'groupName', key: 'groupName', width: setWidth(4) },
      {
        title: '坐席状态',
        dataIndex: 'agentStatus',
        key: 'agentStatus',
        width: setWidth(5),
        render: (text) => callAgentStatusCfg[text],
      },
      { title: 'SIP话机账号', dataIndex: 'voipAccount', key: 'voipAccount', width: setWidth(6) },
      { title: 'SIP话机密码', dataIndex: 'voipPwd', key: 'voipPwd', width: setWidth(6) },
      {
        title: '创建日期',
        dataIndex: 'createTime',
        key: 'createTime',
        width: setWidth(6),
        render: (text) => moment(text).format(momentFormat),
      },
      {
        title: '到期日期',
        dataIndex: 'expireTime',
        dataIndex: 'expireTime',
        width: setWidth(6),
        render: (text) => (text ? moment(text).format('YYYY-MM-DD') : ''),
      },
      {
        title: '到期状态',
        dataIndex: 'expireStatus',
        key: 'expireStatus',
        width: setWidth(5),
        render: (text) => (text == 1 ? '未到期' : '已到期'),
      },
      {
        title: '操作',
        dataIndex: 'agentId',
        key: 'actionPlus',
        width: setWidth(4),
        render: (text, record) => (
          <ListAction
            config={[
              {
                text: '详情',
                onClick: () => {
                  this.switchModalAgentDetail(text);
                },
              },
              {
                text: '续费',
                onClick: () => {
                  this.batchRenew(record);
                },
              },
            ]}
          />
        ),
      },
    ];

    this.searchRef = null;

    this.agentDetailData = {};

    this.agentData = [];
  }

  componentDidMount() {
    this.init();
    this.calculateScrollY();
  }

  async init() {
    this.setState({ loading: true });
    await new Promise(this.initDict);
    this.setState({
      campusDict: getRelationsDictPlus(campus)(this.props.branchCampusTree, null),
    });
    await new Promise(this.loadList);
    this.setState({ loading: false });
  }

  initDict(resolve) {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['branchCampusTree'],
        callback() {
          resolve();
        },
      },
    });
  }

  getListParams() {
    let searchKey = {
      pageNum: this.state.pageCurrent,
      pageSize: this.state.pageSize,
    };

    for (const key in this.state.searchKey) {
      searchKey[key] = this.state.searchKey[key];
    }

    return searchKey;
  }

  async loadList(resolve) {
    const apiParams = this.getListParams();
    const rsp = await pageAgents(apiParams).catch(errorsHandler);
    if (rsp && rsp.resultCode === 0 && rsp.data) {
      this.setState({
        list: rsp.data.list || [],
        total: rsp.data.total,
        selectedRowKeys: [],
      });
    }
    resolve();
  }

  search(params) {
    this.setState(
      {
        searchKey: { ...this.searchKeyAdapter(params) },
      },
      () => {
        this.changePage({
          current: 1,
          pageSize: this.state.pageSize,
        });
      },
    );
  }

  searchKeyAdapter = (params) => {
    let sKey = clone(params) || {};
    for (const key in sKey) {
      sKey[key] = sKey[key] ? trim(sKey[key]) : sKey[key];
    }
    if (sKey.hasOwnProperty('branchId')) {
      sKey.organizationId = sKey.branchId;
      delete sKey.branchId;
    }
    if (sKey.hasOwnProperty('campusId')) {
      sKey.organizationId = sKey.campusId || sKey.organizationId;
      delete sKey.campusId;
    }
    return sKey;
  };

  updatePage(pageCurrent, pageSize) {
    return new Promise((resolve) => {
      this.setState(
        (prevState) => {
          let payload = {};
          if (pageSize && pageSize !== prevState.pageSize) {
            payload.pageCurrent = 1;
            payload.pageSize = pageSize;
          } else {
            payload.pageCurrent = pageCurrent;
          }
          return payload;
        },
        () => {
          resolve();
        },
      );
    });
  }

  async changePage({ current, pageSize }) {
    await this.updatePage(current, pageSize);
    this.setState({ loading: true });
    await new Promise(this.loadList);
    this.setState({ loading: false });
  }

  async refreshList() {
    this.setState({ loading: true });
    await new Promise(this.loadList);
    this.setState({ loading: false });
  }

  switchModalAgentDetail(agentId) {
    this.setState((prevState) => {
      this.agentDetailData = prevState.modalAgentDetailVisible ? null : { agentId };
      return { modalAgentDetailVisible: !prevState.modalAgentDetailVisible };
    });
  }

  onFieldsChange(fields) {
    if (fields.branchId && fields.branchId.value !== this.state.branchId) {
      const statePayload = {
        branchId: fields.branchId.value,
        campusDict: getRelationsDictPlus(campus)(this.props.branchCampusTree, fields.branchId.value),
      };
      this.setState(statePayload);
      this.searchRef.setFieldsValue({ campusId: null });
    }
  }

  selectCallAgent(selectedRowKeys) {
    this.setState((prevState) => {
      prevState.selectedRowKeys = selectedRowKeys;
      return { selectedRowKeys: prevState.selectedRowKeys.sort() };
    });
  }

  tableRowClick(record, index, event) {
    const { selectedRowKeys } = this.state;
    if (event.target.tagName.toLowerCase() !== 'a') {
      const i = selectedRowKeys.length > 0 ? selectedRowKeys.findIndex((item) => item == index) : -1;
      if (i > -1) {
        selectedRowKeys.splice(i);
      } else {
        selectedRowKeys.push(index);
      }
      this.setState({ selectedRowKeys });
    }
  }

  batchAssign() {
    this.agentData = this.state.selectedRowKeys.map((key) => this.state.list[key]);
    this.setState({ modalAgentAssignVisible: true });
  }

  closeModalAgentAssign() {
    this.setState({ modalAgentAssignVisible: false });
    this.agentData = [];
  }
  // 批量续费
  batchRenew(selected) {
    this.agentData = this.state.selectedRowKeys.map((key) => this.state.list[key]);
    if (this.agentData.length > 1 && !selected.id) {
      const sameTime = this.agentData.every(
        (item) =>
          moment(item.createTime).format('YYYY-MM-DD') === moment(this.agentData[0].createTime).format('YYYY-MM-DD'),
      );
      if (!sameTime) {
        message.warn('请批量选择创建时间一致的坐席数据');
        return;
      }
    }
    let clickType = 'batch';
    if (selected.id) {
      this.agentData = [selected];
      clickType = 'column';
    }
    this.setState({ modalExpireVisible: true, clickType });
  }
  // 批量续费取消
  closeModalBatchExpire() {
    this.agentData = [];
    this.setState({ modalExpireVisible: false });
  }

  // 确认续费
  expireSuccess() {
    this.agentData = [];
    this.setState({ modalExpireVisible: false });
    this.search();
  }
  // 导出
  async batchExport() {
    const apiParams = this.getListParams();
    const { resultCode, data } = await exportList(apiParams);
    if (resultCode === 0 && data) {
      download({ url: data, fileName: moment(Date.now()).format('YYYYMMDD') + '坐席管理数据' });
    }
  }

  calculateScrollY = () => {
    const searchDomRect = document.querySelector('.Page-header').getClientRects()[0];
    // 搜索+分页高度
    let y = searchDomRect.top + searchDomRect.height + 150;
    this.setState({
      scrollY: `calc(100vh - ${y}px)`,
    });
  };

  render() {
    return (
      <Page
        name="坐席管理"
        className={styles.assignCallAgent}
        loading={this.state.loading}
        route={this.props.route}
        fitWindowHeight
        headerToolBar={
          <div>
            <Button
              onClick={this.batchAssign}
              disabled={this.state.selectedRowKeys.length === 0}
              type="primary"
              style={{ marginLeft: '10px' }}
            >
              坐席分配
            </Button>
            <Button
              onClick={this.batchRenew}
              type="primary"
              style={{ marginLeft: '10px' }}
              disabled={this.state.selectedRowKeys.length === 0}
            >
              批量续费
            </Button>
            <Button onClick={this.batchExport} type="primary" style={{ marginLeft: '10px' }}>
              导出
            </Button>
          </div>
        }
        subHeader={
          <SearchFilterForm
            ref={(c) => (this.searchRef = c)}
            onSearch={this.search}
            onFieldsChange={this.onFieldsChange}
            truncationNum={4}
          >
            <SearchFieldForm name="agentName">
              <Input placeholder="坐席名称" />
            </SearchFieldForm>
            <SearchFieldForm name="agentNick">
              <Input placeholder="坐席昵称" />
            </SearchFieldForm>
            <SearchFieldForm name="agentEmail">
              <Input placeholder="邮箱账号" />
            </SearchFieldForm>
            <SearchFieldForm name="voipAccount">
              <Input placeholder="SIP话机账号" />
            </SearchFieldForm>
            <SearchFieldForm name="branchId">
              <Select placeholder="分公司" dict={this.props.branchCampusTree} showSearch={true} />
            </SearchFieldForm>
            <SearchFieldForm name="campusId">
              <Select placeholder="学习中心" dict={this.state.campusDict} showSearch={true} />
            </SearchFieldForm>
            <SearchFieldForm name="agentStatus">
              <Select placeholder="坐席状态" dict={agentStatusDict} />
            </SearchFieldForm>
            <SearchFieldForm name="assignType">
              <Select placeholder="分配状态" dict={assignTypeDict} />
            </SearchFieldForm>
            <SearchFieldForm name="expireStatus">
              <Select placeholder="到期状态" dict={expireTypeDict} />
            </SearchFieldForm>
          </SearchFilterForm>
        }
        fitWindowHeight={true}
      >
        <Table
          rowKey="aaa"
          dataSource={this.state.list}
          columns={this.columns}
          loading={this.state.loading}
          rowSelection={{
            selectedRowKeys: this.state.selectedRowKeys,
            onChange: this.selectCallAgent,
          }}
          onRowClick={this.tableRowClick}
          pagination={{
            howQuickJumper: true,
            pageSizeOptions: ['10', '20', '50', '100'],
            showSizeChanger: true,
            showTotal: (total) => `共 ${total} 条记录`,
            current: this.state.pageCurrent,
            total: this.state.total,
            pageSize: this.state.pageSize,
          }}
          scroll={{
            y: this.state.scrollY,
          }}
          onChange={this.changePage}
        />

        {this.state.modalAgentDetailVisible && (
          <ModalAgentDetail data={this.agentDetailData} onClose={this.switchModalAgentDetail} />
        )}

        {this.state.modalAgentAssignVisible && (
          <ModalAgentAssign
            data={this.agentData}
            onCancel={this.closeModalAgentAssign}
            onSuccess={this.search}
            dispatch={this.props.dispatch}
            storeDict={this.props.storeDict}
          />
        )}

        {this.state.modalExpireVisible && (
          <ModalBatchExpire
            data={this.agentData}
            onCancel={this.closeModalBatchExpire}
            onSuccess={this.expireSuccess}
            clickType={this.state.clickType}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    branchCampusTree: state.dict.branchCampusTree,
    storeDict: state.dict,
  };
};

export default connect(mapStateToProps)(BindCallAgent);
