import React from 'react';
import PropTypes from 'prop-types';
import style from './styles.module.less';
import { Popover } from 'COMPONENT';

const TextContent = (props) => {
  const data = Array.isArray(props.data) ? props.data : [];
  const truncationNum = props.truncationNum > 0 ? Math.floor(props.truncationNum) : 1;

  const prev = data.slice(0, truncationNum);
  const more = data.slice(truncationNum);

  return (
    <div className={style.textContent}>
      {prev.join(props.joinSymbol)}
      {more.length > 0 && (
        <Popover
          overlayClassName={style.textContentPopover}
          content={more.map((item, index) => (
            <p key={index}>{item}</p>
          ))}
        >
          <span className={style.textContentMore}>{props.joinSymbol}更多</span>
        </Popover>
      )}
    </div>
  );
};

TextContent.propTypes = {
  data: PropTypes.arrayOf(PropTypes.string).isRequired,
  truncationNum: PropTypes.number,
  joinSymbol: PropTypes.string,
};

TextContent.defaultProps = {
  truncationNum: 1,
  joinSymbol: ',',
};

export default TextContent;
