import React from 'react';
import { connect } from 'dva';
import { Page, SearchFilterForm, SearchFieldForm, Input, Select, List, ListAction, Button } from 'COMPONENT';
import { errorsHandle } from 'UTIL/common';
import { clone, trim } from 'lodash';
import { bindStatusDict, agentStatusDict } from './config';
import { expireTypeDict } from '../AssignCallAgent/config';
import { callAgentStatusCfg, CALL_AGENT_STATUS_INACTIVE } from 'CONST/callConfig';
import ModalAgentDetail from '../ModalAgentDetail';
import ModalAgentBind from './ModalAgentBind';
import moment from 'moment';
import { pageCampusAgents, exportCampusAgents } from 'SERVICE/callCenter';
import { getRelationsDictPlus } from 'MODEL/dict/utils/index';
import { campus } from 'MODEL/dict/utils/config';
import { download } from 'UTIL/file';
const setWidth = List.createSetWidth();

const momentFormat = 'YYYY-MM-DD HH:mm:ss';

class BindCallAgent extends React.Component {
  constructor(props) {
    super(props);

    this.init = this.init.bind(this);
    this.initDict = this.initDict.bind(this);
    this.getListParams = this.getListParams.bind(this);
    this.loadList = this.loadList.bind(this);
    this.search = this.search.bind(this);
    this.updatePage = this.updatePage.bind(this);
    this.changePage = this.changePage.bind(this);
    this.refreshList = this.refreshList.bind(this);
    this.switchModalAgentDetail = this.switchModalAgentDetail.bind(this);
    this.switchModalAgentBind = this.switchModalAgentBind.bind(this);
    this.onFieldsChange = this.onFieldsChange.bind(this);
    this.batchExport = this.batchExport.bind(this);

    this.state = {
      loading: false,
      searchKey: null,
      list: [],
      total: 0,
      pageCurrent: 1,
      pageSize: 10,
      modalAgentDetailVisible: false,
      modalAgentBindVisible: false,
      branchId: null,
      campusDict: null,
    };

    this.columns = [
      { title: '坐席ID', dataIndex: 'agentId', key: 'agentId', width: setWidth(6) },
      { title: '坐席名称', dataIndex: 'agentName', key: 'agentName', width: setWidth(4) },
      { title: '坐席昵称', dataIndex: 'agentNick', key: 'agentNick', width: setWidth(4) },
      { title: '邮箱账号', dataIndex: 'agentEmail', key: 'agentEmail', width: setWidth(6) },
      { title: '所属组织', dataIndex: 'organizationName', key: 'organizationName', width: setWidth(6) },
      { title: '用户绑定数', dataIndex: 'bindNumber', key: 'bindNumber', width: setWidth(5) },
      { title: '坐席角色', dataIndex: 'agentRoleName', key: 'agentRoleName', width: setWidth(4) },
      { title: '技能组', dataIndex: 'groupName', key: 'groupName', width: setWidth(4) },
      {
        title: '坐席状态',
        dataIndex: 'agentStatus',
        key: 'agentStatus',
        width: setWidth(4),
        render: (text) => callAgentStatusCfg[text],
      },
      { title: 'SIP话机账号', dataIndex: 'voipAccount', key: 'voipAccount', width: setWidth(6) },
      { title: 'SIP话机密码', dataIndex: 'voipPwd', key: 'voipPwd', width: setWidth(6) },
      {
        title: '创建日期',
        dataIndex: 'createTime',
        key: 'createTime',
        width: setWidth(6),
        render: (text) => moment(text).format(momentFormat),
      },
      {
        title: '到期日期',
        dataIndex: 'expireTime',
        dataIndex: 'expireTime',
        width: setWidth(6),
        render: (text) => (text ? moment(text).format('YYYY-MM-DD') : ''),
      },
      {
        title: '到期状态',
        dataIndex: 'expireStatus',
        key: 'expireStatus',
        width: setWidth(5),
        render: (text) => (!text ? '' : text === 1 ? '未到期' : '已到期'),
      },
      {
        title: '操作',
        dataIndex: 'agentId',
        key: 'actionPlus',
        width: setWidth(4),
        fixed: 'right',
        render: (text, record) => (
          <ListAction
            config={[
              {
                text: '详情',
                onClick: () => {
                  this.switchModalAgentDetail(text);
                },
              },
              {
                text: '设置',
                onClick: () => {
                  this.switchModalAgentBind(record);
                },
                disabled: record.agentStatus === `${CALL_AGENT_STATUS_INACTIVE}`,
              },
            ]}
          />
        ),
      },
    ];

    this.searchRef = null;

    this.agentDetailData = {};

    this.agentBindData = {};
  }

  componentDidMount() {
    this.init();
  }

  async init() {
    this.setState({ loading: true });
    await new Promise(this.initDict);
    this.setState({
      campusDict: getRelationsDictPlus(campus)(this.props.branchCampusTree, null),
    });
    await new Promise(this.loadList);
    this.setState({ loading: false });
  }

  initDict(resolve) {
    this.props.dispatch({
      type: 'dict/initDict',
      payload: {
        list: ['branchCampusTree'],
        callback() {
          resolve();
        },
      },
    });
  }

  getListParams() {
    let searchKey = {
      pageNum: this.state.pageCurrent,
      pageSize: this.state.pageSize,
    };

    for (const key in this.state.searchKey) {
      searchKey[key] = this.state.searchKey[key];
    }

    return searchKey;
  }

  async loadList(resolve) {
    const apiParams = this.getListParams();
    const rsp = await pageCampusAgents(apiParams).catch(errorsHandle());
    if (rsp && rsp.resultCode === 0 && rsp.data) {
      this.setState({
        list: rsp.data.list || [],
        total: rsp.data.total,
      });
    }
    resolve();
  }

  search(params) {
    this.setState(
      {
        searchKey: { ...this.searchKeyAdapter(params) },
      },
      () => {
        this.changePage({
          current: 1,
          pageSize: this.state.pageSize,
        });
      },
    );
  }

  searchKeyAdapter = (params) => {
    let sKey = clone(params) || {};
    for (const key in sKey) {
      sKey[key] = sKey[key] ? trim(sKey[key]) : sKey[key];
    }
    if (sKey.hasOwnProperty('branchId')) {
      sKey.organizationId = sKey.branchId;
      delete sKey.branchId;
    }
    if (sKey.hasOwnProperty('campusId')) {
      sKey.organizationId = sKey.campusId || sKey.organizationId;
      delete sKey.campusId;
    }
    return sKey;
  };

  updatePage(pageCurrent, pageSize) {
    return new Promise((resolve) => {
      this.setState(
        (prevState) => {
          let payload = {};
          if (pageSize && pageSize !== prevState.pageSize) {
            payload.pageCurrent = 1;
            payload.pageSize = pageSize;
          } else {
            payload.pageCurrent = pageCurrent;
          }
          return payload;
        },
        () => {
          resolve();
        },
      );
    });
  }

  async changePage({ current, pageSize }) {
    await this.updatePage(current, pageSize);
    this.setState({ loading: true });
    await new Promise(this.loadList);
    this.setState({ loading: false });
  }

  async refreshList() {
    this.setState({ loading: true });
    await new Promise(this.loadList);
    this.setState({ loading: false });
  }

  switchModalAgentDetail(agentId) {
    this.setState((prevState) => {
      this.agentDetailData = prevState.modalAgentDetailVisible ? null : { agentId };
      return { modalAgentDetailVisible: !prevState.modalAgentDetailVisible };
    });
  }

  switchModalAgentBind(record) {
    this.setState((prevState) => {
      this.agentBindData = prevState.modalAgentBindVisible ? null : record;
      return { modalAgentBindVisible: !prevState.modalAgentBindVisible };
    });
  }

  onFieldsChange(fields) {
    if (fields.branchId && fields.branchId.value !== this.state.branchId) {
      const statePayload = {
        branchId: fields.branchId.value,
        campusDict: getRelationsDictPlus(campus)(this.props.branchCampusTree, fields.branchId.value),
      };
      this.setState(statePayload);
      this.searchRef.setFieldsValue({ campusId: null });
    }
  }

  async batchExport() {
    const apiParams = this.getListParams();
    const { resultCode, data } = await exportCampusAgents(apiParams).catch(errorsHandle());
    if (resultCode === 0) {
      download({ url: data, fileName: moment(Date.now()).format('YYYYMMDD') + '坐席管理数据' });
    }
  }

  render() {
    return (
      <Page
        name="坐席绑定"
        loading={this.state.loading}
        route={this.props.route}
        headerToolBar={
          <Button onClick={this.batchExport} type="primary">
            导出
          </Button>
        }
        subHeader={
          <SearchFilterForm
            ref={(c) => (this.searchRef = c)}
            onSearch={this.search}
            onFieldsChange={this.onFieldsChange}
            truncationNum={4}
          >
            <SearchFieldForm name="agentName">
              <Input placeholder="坐席名称" />
            </SearchFieldForm>
            <SearchFieldForm name="agentNick">
              <Input placeholder="坐席昵称" />
            </SearchFieldForm>
            <SearchFieldForm name="agentEmail">
              <Input placeholder="邮箱账号" />
            </SearchFieldForm>
            <SearchFieldForm name="voipAccount">
              <Input placeholder="SIP话机账号" />
            </SearchFieldForm>
            <SearchFieldForm name="branchId">
              <Select placeholder="分公司" dict={this.props.branchCampusTree} showSearch={true} />
            </SearchFieldForm>
            <SearchFieldForm name="campusId">
              <Select placeholder="学习中心" dict={this.state.campusDict} showSearch={true} />
            </SearchFieldForm>
            <SearchFieldForm name="bindStatus">
              <Select placeholder="绑定状态" dict={bindStatusDict} />
            </SearchFieldForm>
            <SearchFieldForm name="agentStatus">
              <Select placeholder="坐席状态" dict={agentStatusDict} />
            </SearchFieldForm>
            <SearchFieldForm name="expireStatus">
              <Select placeholder="到期状态" dict={expireTypeDict} />
            </SearchFieldForm>
          </SearchFilterForm>
        }
      >
        <List
          columns={this.columns}
          dataSource={this.state.list}
          pagination={{
            total: this.state.total,
            current: this.state.pageCurrent,
            pageSize: this.state.pageSize,
          }}
          onChange={this.changePage}
        />
        {this.state.modalAgentDetailVisible && (
          <ModalAgentDetail data={this.agentDetailData} onClose={this.switchModalAgentDetail} hideRenewRecord={true} />
        )}
        {this.state.modalAgentBindVisible && (
          <ModalAgentBind
            data={this.agentBindData}
            onClose={this.switchModalAgentBind}
            afterSubmit={this.refreshList}
          />
        )}
      </Page>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    branchCampusTree: state.dict.branchCampusTree,
  };
};

export default connect(mapStateToProps)(BindCallAgent);
