import React from 'react';
import Modal from 'COMPONENT/Modal';
import List from 'COMPONENT/List';
import { jiaoXuePageUrl } from 'CONST/otherSystemConfig';
import TextLink from 'COMPONENT/TextLink';
import moment from 'moment';

const jiaoXueViewPageRouter = '/teaching-banke#/assessment-center/entrance-test-report';

/**
 * 打印预览
 *
 * @param url
 */
const viewPrint = (url) => {
  if (!url) {
    return;
  }

  let newPage = window.open('about:blank');
  setTimeout(() => {
    newPage.document.location.replace(url);
  }, 1);
};

/**
 * 报告列表弹窗
 *
 * @param props.stuTicket {string} 学生ID，去掉`STU`字符
 * @param props.stuName {string} 学生名称
 * @return {*}
 */
const reporterListModal = (props) => {
  const { stuTicket, stuName } = props;

  const columns = [
    {
      title: '序号',
      dataIndex: 'index',
      key: 'index',
      width: 70,
      fixed: 'left',
      render: (text, record, index) => index + 1,
    },
    { title: '考试名', dataIndex: 'examName', key: 'examName' },
    { title: '试卷名', dataIndex: 'paperName', key: 'paperName' },
    {
      title: '创建时间',
      dataIndex: 'examTime',
      key: 'examTime',
      render: (val, record, index) => {
        const time = record.examTime || record.createTime;

        return time ? moment(new Date(time)).format('YYYY-MM-DD HH:mm:ss') : '';
      },
    },
    {
      title: '操作',
      key: 'action',
      render: (val, record, index) => {
        const { examId, examName } = record;
        let previewUrl = '';

        if (record.previewUrl) {
          previewUrl = record.previewUrl;
        } else {
          previewUrl = `${jiaoXuePageUrl}${jiaoXueViewPageRouter}?stuTicket=${stuTicket}&examId=${examId}&stuName=${stuName}&examName=${examName}&origin=boss`;
        }

        return (
          <TextLink
            onClick={() => {
              viewPrint(previewUrl);
            }}
          >
            预览
          </TextLink>
        );
      },
    },
  ];

  return (
    <Modal title="入学测报告" onOk={props.closeModal} onCancel={props.closeModal}>
      <List columns={columns} dataSource={props.dataSource} />
    </Modal>
  );
};

export default reporterListModal;
