import React from 'react';
import Loading from 'COMPONENT/Loading';
import ReporterListModal from './reporterListModal';
import { getStudentExamReport } from 'SERVICE/consultant';
import { getPaperReportByStudentId } from 'SERVICE/entranceExam';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';

const withTeachingReporterHOC = (studentId, closeReporterModal, stuName) => {
  return class extends React.Component {
    constructor(props) {
      super(props);

      this.state = {
        loading: false,
        list: [],
      };
    }

    componentDidMount() {
      this.loadList();
    }

    /**
     * 获取入学测及散测报告
     * tips: 入学测接口数据包含散测数据，目前只使用入学测接口数据，但保留两个接口
     * tips: 散测报告列表含有预览url字段`previewUrl`
     *
     * @return {Promise<void>}
     */
    loadList = async () => {
      this.setState({ loading: true });

      const [examReport, paperReport] = await Promise.all([
        getStudentExamReport({ studentId }),
        getPaperReportByStudentId({ studentId }),
      ]).catch((e) => {
        this.setState({ loading: false });
      });

      const examReportList = (examReport && examReport.data) || [];
      const paperReportList = (paperReport && paperReport.data) || [];
      const list = examReportList;

      this.setState({ loading: false, list });

      if (list.length === 0) {
        showMessage('该学生没有入学测报告!');
      }
    };

    closeModal = () => {
      closeReporterModal();
      this.setState({ list: [] });
    };

    render() {
      const { list, loading } = this.state;

      return (
        <div>
          <Loading loading={loading} />
          {Array.isArray(list) && list.length !== 0 && (
            <ReporterListModal dataSource={list} closeModal={this.closeModal} stuTicket={studentId} stuName={stuName} />
          )}
        </div>
      );
    }
  };
};

export default withTeachingReporterHOC;
