import React from 'react';
import { Router } from 'dva/router';
import registerModel from 'UTIL/registerModel';
// 固定引入的模块
import print from './print';
import generatePicture from './generatePicture';
// 下列路由在`xcli dev`时，会根据选择动态引入
import common from './commonPages';
import customer from './customer';
import course from './course';
import system from './system';
import product from './product';
import student from './student';
import finance from './finance';
import doc from './doc';
import dashboard from './dashboard';
import test from './test';
import ipad from './ipad';
import contract from './contract';
import report from './report';
import preferential from './preferential';
import consultant from './consultant';
import reception from './reception';
import signUpOnline from './signUpOnline';
import netWorkbench from './netWorkbench';
import marketActivityManage from './marketActivityManage';
import personnelManage from './personnelManage';
import targetManage from './targetManage';
import goalManage from './goalManage';
import crmBench from './crmBench';
import ssCRM from './ssCRM';
import approveManage from './approveManage';
import basicInformationManage from './basicInformationManage';
import outcallWorkbench from './outcallWorkbench';
import emergencyState from './emergencyState';

function RouterConfig({ history, app }) {
  const routes = [
    {
      path: '/oauth2Callback',
      getComponent(nextState, cb) {
        require.ensure([], (require) => {
          cb(null, require('VIEW/Oauth2Callback'));
        });
      },
    },
    {
      path: '/login',
      getComponent(nextState, cb) {
        require.ensure([], (require) => {
          cb(null, require('VIEW/Login'));
        });
      },
    },
    {
      path: '/mobileFileUpload',
      getComponent(nextState, cb) {
        require.ensure([], (require) => {
          cb(null, require('VIEW/MobileFileUpload'));
        });
      },
    },
    // 打印管理
    ...print,
    //
    ...generatePicture,
    // 文件打印预览
    {
      path: 'printFile',
      getComponent(nextState, cb) {
        require.ensure([], (require) => {
          cb(null, require('VIEW/PrintFile'));
        });
      },
    },
    {
      path: '/ipad/:id',
      getComponent(nextState, cb) {
        require.ensure([], (require) => {
          cb(null, require('VIEW/PadImgText'));
        });
      },
    },
    {
      path: '/',
      getComponent(nextState, cb) {
        require.ensure([], (require) => {
          registerModel(require('MODEL/main'));
          registerModel(require('MODEL/callCenter'));
          cb(null, require('VIEW/Main'));
        });
      },
      childRoutes: [
        {
          path: 'oldPage',
          name: 'oldPage',
          getComponent(nextState, cb) {
            require.ensure([], (require) => {
              cb(null, require('PAGE/OldPage'));
            });
          },
        },
        {
          path: 'next-compact',
          name: 'next-compact',
          getComponent(nextState, cb) {
            require.ensure([], (require) => {
              cb(null, require('PAGE/NextCompact'));
            });
          },
        },
        ...common,
        ...customer,
        ...course,
        ...system,
        ...product,
        ...student,
        ...finance,
        ...doc,
        ...dashboard,
        ...ipad,
        ...contract,
        ...report,
        ...preferential,
        ...test,
        ...consultant,
        ...reception,
        ...netWorkbench,
        ...signUpOnline,
        ...marketActivityManage,
        ...personnelManage,
        ...targetManage,
        ...goalManage,
        ...crmBench,
        ...ssCRM,
        ...approveManage,
        ...basicInformationManage,
        ...outcallWorkbench,
        ...emergencyState,
      ],
    },
    {
      path: '*',
      getComponent(nextState, cb) {
        require.ensure([], (require) => {
          cb(null, require('VIEW/404'));
        });
      },
    },
  ];

  return <Router history={history} routes={routes} />;
}

export default RouterConfig;
