import buildApi, { buildStrictFetchApi } from './buildApi';
import fetch from 'dva/fetch';

/**
 * Api配置信息，key为Api唯一标识，value为object-->{ url, method }，或者string-->url（method默认为GET）
 */
const config = {
  // 教室管理列表
  getClassroomManageList: { url: 'ClassroomNewController/getClassroomManageList.do', method: 'POST', dataType: 'json' },
  // 新增教室-星火专用
  ClassroomNewControllerAdd: { url: 'ClassroomNewController/add.do', method: 'POST', dataType: 'json' },
  // 修改教室-星火专用
  ClassroomNewControllerModify: { url: 'ClassroomNewController/modify.do', method: 'POST', dataType: 'json' },
  // 批量修改状态
  ClassroomNewBatchModifyStatus: {
    url: 'ClassroomNewController/batchModifyStatus.do',
    method: 'POST',
    dataType: 'json',
  },
  // 根据id查询教室
  ClassroomNewFindById: { url: 'ClassroomNewController/findById.do', method: 'GET' },
  // 申请批量变更教室状态
  applyBatchChangeClassroomStatus: {
    url: 'ClassroomNewController/applyBatchChangeClassroomStatus.do',
    method: 'POST',
    dataType: 'json',
  },
  // 根据组织ID查找修改教室详情审批人信息
  getAuditMsgForDetailChange: {
    url: 'ClassroomNewController/getAuditMsgForDetailChange.do',
    method: 'GET',
    dataType: 'json',
  },
  // 根据审批记录id获取审批记录
  listClassroomChangeDetailById: {
    url: 'ClassroomNewController/listClassroomChangeStatusFlowRecordItemsByFlowRecordId.do',
    method: 'GET',
  },
  // 查询教室审批列表
  pageClassroomChangeStatusFlowRecords: {
    url: 'ClassroomNewController/pageClassroomChangeStatusFlowRecords.do',
    method: 'GET',
  },
  // 场地管理->查询组织树
  getBelongOrg: { url: 'SystemAction/getBelongOrg.do', method: 'GET' },
  // 场地管理->列表 20210805废弃
  getOfficeSpaceManageList: { url: 'OfficeSpaceManageController/getOfficeSpaceManageList.do', method: 'GET' },

  // new场地管理->列表 http://47.106.118.192:13000/project/27/interface/api/56822
  getOfficeSpaceManageListNew: { url: 'boss-xinghuo-api/officeSpaceManage/page', method: 'POST', dataType: 'json' },
  // 场地管理->列表excel导出 https://xiaojiaoyukeji.yuque.com/newop/hk7g9o/ug97rx
  getOfficeSpaceManageListNewExport: {
    url: 'boss-xinghuo-api/officeSpaceManage/page?isExport=pull',
    method: 'POST',
    dataType: 'json',
  },
  // 场地管理->根据ID查询场地信息详情
  findOfficeSpaceById: { url: 'OfficeSpaceManageController/findOfficeSpaceById.do', method: 'GET' },
  // 场地管理->修改场地信息
  modifyOfficeSpaceManage: {
    url: 'OfficeSpaceManageController/modifyOfficeSpaceManage.do',
    method: 'POST',
    dataType: 'json',
  },
  // 抓拍记录表->抓拍记录表
  pageRecords: { url: 'boss-xinghuo-api/classroom/deviceCapture/pageRecords', method: 'POST', dataType: 'json' },
  // 抓拍记录表->导出记录表
  exportRecords: {
    url: 'boss-xinghuo-api/classroom/deviceCapture/pageRecords?isExport=pull',
    method: 'POST',
    dataType: 'json',
  },
  // 抓拍记录表->获取当前学习中心的所有教室
  getClassroomResponseByCampus: { url: 'ClassroomManageController/getClassroomResponseByCampus.do', method: 'GET' },
};

//
const configForEduBossApi = {
  // 抓拍记录表->获取当前学习中心/分公司的所有教室
  getClassroomResponseByOrg: {
    url: '/eduboss/boss-xinghuo-api/classroom/listByOrganizationId',
    method: 'POST',
    dataType: 'json',
  },
};

async function request(url, method, data, options = {}) {
  options.method = method;

  options.body = data;

  const response = await fetch(url, options);
  return response.json();
}

export default {
  ...buildApi(config),
  ...buildStrictFetchApi(configForEduBossApi),
  // 上传教室excel
  ClassroomNewImportClassroom: (data) => request('/eduboss/ClassroomNewController/importClassroom.do', 'POST', data),
};
