import { httpGet, httpPost, fetchGet, fetchPost } from 'UTIL/http';
import { isObject } from 'UTIL/lang/data-type';
import { cmallUrlBase, insightUrlBase } from '../constants/config';
/**
 * 根据配置信息构建可供调用的Api方法
 * @param  {object}  //配置信息集合
 * @return {object}  //Api方法集合
 */
const buildApi = (config) => {
  let output = {};
  Object.keys(config).forEach((key) => {
    const detail = config[key];
    const isObject = typeof detail === 'object';
    const func = isObject && detail.method === 'POST' ? httpPost : httpGet;
    const url = isObject ? detail.url : detail;
    output[key] = (params, noTips) => func({ url, params, dataType: detail.dataType, method: detail.method }, noTips);
  });
  return output;
};

/**
 * 生成请求配置
 * @param {Object} configs
 */
const buildStrictFetchApi = (configs) => {
  if (!isObject(configs)) {
    return {};
  }

  const nameList = Object.keys(configs);

  const ret = nameList.reduce((prev, name) => {
    const itemCfg = configs[name];

    if (!isObject(itemCfg)) {
      return prev;
    }

    const requestFn = itemCfg.method === 'POST' ? fetchPost : fetchGet;

    prev[name] = (params, errHandler) => {
      return requestFn(
        {
          url: itemCfg.url,
          headers: itemCfg.headers,
          dataType: itemCfg.dataType,
          params,
        },
        errHandler,
      );
    };

    return prev;
  }, {});

  return ret;
};

// 报读和流量的通用方法 不同的是接口路径
function commonCmallBuildApi(urlBase, config) {
  return function (config) {
    let output = {};
    // 处理错误信息
    const handleError = (error) => {
      if (error._data_) {
        throw {
          ...error,
          text: error._data_.msg,
        };
      }
      throw error;
    };
    Object.keys(config).forEach((key) => {
      const detail = config[key];
      const isObject = typeof detail === 'object';
      const func = isObject && detail.method === 'POST' ? fetchPost : fetchGet;
      const url = urlBase + (isObject ? detail.url : detail);
      output[key] = (params) => {
        const cmallToken = JSON.parse(sessionStorage.getItem('cmallToken'));
        const containTokenHeaders = {
          Authorization: cmallToken,
        };
        return func({ url, params, dataType: detail.dataType, headers: containTokenHeaders }, handleError);
      };
    });
    return output;
  };
}
// 新报读的
export const cmallBuildApi = commonCmallBuildApi(cmallUrlBase);
//流量
export const insightBuildApi = commonCmallBuildApi(insightUrlBase);

export { buildStrictFetchApi };

export default buildApi;
