import { buildStrictFetchApi } from './buildApi';

/**
 * 座席功能请求配置表
 */
const agentFeatReqMap = {
  // 获取智齿 token
  getSocketAuthToken: {
    url: '/crm/agent/getSobotToken',
    method: 'GET',
    dataType: 'json',
  },
  // 查询座席终端信息
  getAgentEndpointInfo: {
    url: '/crm/agent/getSoftPhoneExtInfoByAgentId',
    method: 'GET',
    dataType: 'json',
  },
  // 获取用户座席信息
  getAgentInfoByUser: {
    url: '/crm/agent/getUserAgentInfo',
    method: 'GET',
    dataType: 'json',
  },
  // 获取获取坐席状态信息
  getAgentInfo: {
    url: '/crm/agent/getAgentStatusInfoByAgentId',
    method: 'GET',
    dataType: 'json',
  },
  // 修改坐席手机号
  updateAgentPhone: {
    url: '/crm/agent/updateUserPhone',
    method: 'POST',
    dataType: 'json',
  },
  // 修改坐席状态（上班/下班）
  updateAgentStatus: {
    url: '/crm/agent/switchLoginStatus',
    method: 'POST',
    dataType: 'json',
  },
  // 外呼
  makeCall: {
    url: '/crm/call/makeCall',
    method: 'POST',
    dataType: 'json',
  },
  // 挂断外呼
  hangUp: {
    url: '/crm/call/stopAgentCall',
    method: 'POST',
    dataType: 'json',
  },
};

/**
 * 通话管理请求配置表
 */
const callMgmtReqMap = {
  // 获取通话记录
  getCallRecords: {
    url: '/crm/call/pageRecords',
    method: 'POST',
    dataType: 'json',
  },
};

/**
 * 座席管理请求配置表
 */
const agentMgmtReqMap = {
  // 获取座席列表
  getAgents: {
    url: '/crm/agent/pageAgents',
    method: 'POST',
    dataType: 'json',
  },
  // 获取座席操作日志列表
  getAgentOprLogs: {
    url: '/crm/agent/listUserAgentRelationLogsByAgentId',
    method: 'POST',
    dataType: 'json',
  },
  // 获取坐席绑定列表
  pageAgents: {
    url: '/crm/agent/pageAgents',
    method: 'POST',
    dataType: 'json',
  },
  // 获取(学习中心)坐席绑定列表
  pageCampusAgents: {
    url: '/crm/agent/pageCampusAgents',
    method: 'POST',
    dataType: 'json',
  },
  // 获取坐席详情
  getAgentByAgentId: {
    url: '/crm/agent/getAgentByAgentId',
    method: 'GET',
    dataType: 'json',
  },
  // 获取坐席绑定记录
  pageUserAgentRelationLogsByAgentId: {
    url: '/crm/agent/pageUserAgentRelationLogsByAgentId',
    method: 'POST',
    dataType: 'json',
  },
  // 获取坐席已绑定用户
  listUserAgentsByAgentId: {
    url: '/crm/agent/listUserAgentsByAgentId',
    method: 'GET',
    dataType: 'json',
  },
  // 获取坐席可绑定用户
  listCampusUsersByAgentId: {
    url: '/crm/agent/listCampusUsersByAgentId',
    method: 'POST',
    dataType: 'json',
  },
  // 用户绑定坐席
  bindUsers: {
    url: '/crm/agent/bindUsers',
    method: 'POST',
    dataType: 'json',
  },
  // 分配坐席
  assignAgents: {
    url: '/crm/agent/assignAgents',
    method: 'POST',
    dataType: 'json',
  },
  // 批量续费坐席
  expireAgents: {
    url: '/crm/agent/renewalAgents',
    method: 'POST',
    dataType: 'json',
  },
  // 续费记录
  getRenewRecord: {
    url: '/crm/agent/pageRenewalRecord',
    method: 'POST',
    dataType: 'json',
  },
  // 坐席管理导出
  exportList: {
    url: '/crm/agent/pageAgents?isExport=pull',
    method: 'POST',
    dataType: 'json',
  },
  // 坐席绑定导出
  exportCampusAgents: {
    url: '/crm/agent/pageCampusAgents?isExport=pull',
    method: 'POST',
    dataType: 'json',
  },
};

// 所有未经加工的请求配置
const basicReqMap = {
  ...agentFeatReqMap,
  ...agentMgmtReqMap,
  ...callMgmtReqMap,
};

// 接口 uri 前缀
const uriPrefix = '/eduboss/boss-xinghuo-api';

// 加工过的请求配置表
const exactReqMap = Object.keys(basicReqMap).reduce((prev, name) => {
  const reqConfig = basicReqMap[name];

  prev[name] = {
    ...reqConfig,
    url: uriPrefix + reqConfig.url,
  };

  return prev;
}, {});

export default buildStrictFetchApi(exactReqMap);
