import buildApi, { buildStrictFetchApi } from './buildApi';
import { bossApiUrlBase } from 'CONST/config';

/**
 * Api配置信息，key为Api唯一标识，value为object-->{ url, method }，或者string-->url（method默认为GET）
 */
const config = buildApi({
  login: 'UserController/loginCallBackNew.do', //登录
  getLoginUserInfo: 'SystemAction/getLoginUserInfo.do', //获取当前登录用户信息
  getButtonAuthTags: 'SystemAction/getButtonAuthTags.do', //获取当前登录用户操作权限（控制某些功能按钮显示与否）
  getLoginUserMainRole: 'SystemAction/getLoginUserMainRole.do', // 获取当前登录用户的主组织角色
  getUserToken: 'UserController/getUserToken.do', //获取用户令牌
  getCookieCallsPhoneCheckFail: 'MobileInterface/getCookieCallsPhoneCheckFail.do', //获取来电token验证失败的数据
  selectOrgByIdAndTypeOption: 'CommonAction/selectOrgByIdAndTypeOption.do', //根据ID或类型获取组织机构
  aliyuncs: 'https://xhboss-test.oss-cn-shenzhen.aliyuncs.com', // oss 地址
  /* 获取报读后台token */
  getCmallToken: {
    url: 'CommonAction/getMobileToken.do',
    method: 'GET',
    dataType: 'json',
  },
  /* 获取报读后台token */
  getRegionList: 'SystemAction/getRegionList.do', //获取行政区划（省、市）
  getAreaList: 'SystemAction/getCounties.do', //获取区
  findOrganizationBySchoolId: 'SystemAction/findOrganizationBySchoolId.do', //根据学校获取行政区划
  getSchoolByBlCampus: 'CommonAction/getSchoolByBlCampus.do', //根据学习中心(城市)获取学校(不是用学习中心id，而是城市id)
  getCampusByLoginUserNew: 'CommonAction/getCampusByLoginUserNew.do', //获取当前登录用户可见的学习中心（按分公司分组）
  // 获取 oss 直传后端签名
  getAliyunOSSKeyWithoutCallBack: 'CommonAction/getAliyunOSSKeyWithoutCallBack.do',
  // 初始化导入数据记录
  initRecord: {
    url: 'ExcelImport/initRecord.do',
    method: 'POST',
    dataType: 'json',
  },
  // 获取所有可见学习中心(按分公司分组)
  getCampusAll: 'CommonAction/getCampusAll.do',
  getUserByRoldCodesSelection: 'CommonAction/getUserByRoldCodesSelection.do', //根据角色获取用户  参数如：roleCode=TEATCHER
  // 获取指定角色对应组织的具有特定角色的用户列表
  getUserByRoleCodes: { url: '/boss-xinghuo-api/user/listByRoleCodes', method: 'POST', dataType: 'json' },
  // 根据查询字段和角色码获取用户列表 queryName:查询字符串，roleCode:角色码，number:数量限制，默认100，最大值100
  listByQueryNameAndRoleCode: { url: 'boss-xinghuo-api/user/listByQueryNameAndRoleCode', method: 'GET' },
  getRoleLevelForSelection: 'CommonAction/getRoleLevelForSelection.do', //获取角色等级
  // 获取当前登录角色下的分公司及学习中心
  getCampusByUserBelongBranch: { url: 'CommonAction/getCampusByUserBelongBranch.do', method: 'POST', dataType: 'json' },
  // 根据用户id获取用户下的分公司及学习中心
  getCampusByUserId: { url: 'CommonAction/listBelongBranchCampusByUserId.do', method: 'GET' },

  getSelectOption: 'CommonAction/getSelectOption.do', //获取字典
  // 通过 类别、父id、状态获取符合条件的字典记录
  getListDataDict: 'CommonAction/listDataDict.do',
  // 批量获取字典
  getSelectOptionGroup: { url: 'CommonAction/getSelectOptionGroup.do', method: 'POST', dataType: 'json' },
  // 获取字典
  getSelectOptionNew: 'CommonAction/getSelectOptionNew.do',
  // 获取其他系统菜单
  getOtherSystemMenuList: 'SystemAction/getOtherSystemMenuList.do',
  getMenuList: 'SystemAction/getMenuList.do', //获取菜单
  listMyCollection: 'SystemAction/listMyCollection.do', //获取收藏列表
  addMyCollection: 'SystemAction/addMyCollection.do', //添加收藏
  delMyCollection: 'SystemAction/delMyCollection.do', //根据ID删除收藏
  delMyCollectionByResId: 'SystemAction/delMyCollectionByResId.do', //根据菜单ID删除收藏
  getSystemConfigList: 'SystemAction/getSystemConfigList.do', //获取系统配置
  findPageFAQ: 'FAQController/findPageFAQ.do', //获取常见问题列表
  findOrganizationById: 'SystemAction/findOrganizationById.do', //根据分公司或学习中心获取行政区划
  getCampusByRegionId: 'CommonAction/getCampusByRegionId.do', //根据省市获取学习中心
  getStudentSchoolByRegionId: 'CommonAction/getStudentSchoolByRegionId.do', //根据省市获取公校
  getAllBranch: 'SystemAction/getAllBranch.do', //获取所有分公司
  getDeptAndCampusByBranchId: 'SystemAction/getDeptAndCampusByBranchId.do', //根据分公司id获取学习中心和分公司部门
  getBrenchForSelection: 'CommonAction/getBrenchForSelection.do', //获取所有分公司
  getRenkeHekuaxiaoquTeacher: 'TeacherSubjectController/getRenkeHekuaxiaoquTeacher.do', //获取任课老师和跨学习中心老师
  getSseKey: 'MobileInterface/getSseKey.do', //获取oss上传配置
  previewFile: 'StudentController/previewFile.do',
  getRoleForSelection: 'CommonAction/getRoleForSelection.do', //获取系统角色
  getCampusByLoginUser: 'CommonAction/getCampusByLoginUser.do', //所属学习中心
  getDistinctUserGroupByCampus: 'CommonAction/getDistinctUserGroupByCampus.do', //获取登录用户组织架构用户
  getAllCampusForSelection: 'CommonAction/getAllCampusForSelection.do', //获取所有学习中心
  findStudentsByCampusId: 'StudentController/findStudentsByCampusId.do', // 根据学习中心ID获取学生
  getTeachersByCampusId: 'CourseController/getTeacherBycampusId.do', //根据学习中心ID获取老师
  getClassroomByCampu: 'ClassroomManageController/getClassroomByCampus.do', //根据学习中心ID获取教室
  newGetClassroomByCampu: 'ClassroomManageController/getClassroomByCampusWithDevice.do', //根据学习中心ID获取教室(新)
  getConflictName: 'CourseController/getConflictName.do', //获取冲突名单
  getMiniClassInfoBySelect: 'CourseController/getMiniClassInfoBySelect.do', //获取排课列表
  getConflictMiniClassInfo: 'CourseController/getConflictMiniClassInfo.do', //获取冲突班课的信息
  getConflictMiniCourseInfo: 'CourseController/getConflictMiniCourseInfo.do', //获取冲突班课课程的信息
  changetoGetConflictInfo: {
    url: 'CourseController/changetoGetConflictInfo.do',
    method: 'POST',
    dataType: 'json',
  }, //换课时获取冲突的信息
  sendDataBuryingPoint: { url: 'OperateRecordCountController/saveCount.do', method: 'POST', dataType: 'JSON' }, //数据埋点(伟洪需求，熊魁数据库)
  changeClassRoomOrTeacher: {
    url: 'CourseController/changeClassRoomOrTeacher.do',
    method: 'POST',
    dataType: 'json',
  }, // 更换教室或老师或时间
  getStudentConflictCourse: 'CourseController/getStudentConflictCourse.do', // 调课界面显示原班课程是否冲突信息
  getStudentConflictCourseNew: 'CourseController/getStudentConflictCourseNew.do', // 调课界面显示原班课程是否冲突信息(批量)
  getChangeMiniClassList: 'CourseController/getChangeMiniClassList.do', //获取可调班的列表（换课
  changeCourse: {
    url: 'CourseController/changeCourse.do',
    method: 'POST',
    dataType: 'json',
  }, //学生换课
  getSubSelectOptionBySuper: 'DataDictController/getSubSelectOptionBySuper.do', //获取小班班级类型
  // 检测调课后是否依然有冲突（换课）
  checkConflictAfterChangeCourse: {
    url: 'CourseController/checkConflictAfterChangeCourse.do',
    method: 'POST',
    dataType: 'json',
  },
  // 根据学习中心id和日期获取学生列表(新)
  getStudentIdByBlCampusId: 'CourseController/getStudentIdByBlCampusId.do',
  // 根据多个学习中心ID和日期获取学生列表
  getStudentIdByCampusIdsParam: {
    url: 'CourseController/getStudentIdByCampusIdsParam.do',
    method: 'POST',
    dataType: 'json',
  },
  getCurrentDateTime: '/CommonAction/getCurrentDateTime.do', // 获取当前日期时间
  // 获取班级层次(通过班级类型)
  findClassLevelByClassType: 'ProductController/findClassLevelByClassType.do',
  getReceptionistGroupByCampus: 'CommonAction/getReceptionistGroupByCampus.do', // 获取前台分配咨询师
  getReceptionistGroupByUserName: 'CommonAction/getReceptionistGroupByUserName.do', // 获取邀请人（新增接口）

  /**
   * 获取双师主讲老师字典(当前登录用户所属学习中心)
   * */
  getMainClassTcDict: 'TwoTeacherClassController/getLoginUserMainTeacherList.do',
  /**
   * 获取双师辅导老师字典(当前登录用户所属学习中心)
   * */
  getAuxClassTcDict: 'TwoTeacherClassController/getLoginUserTwoTeacherList.do',
  /**
   * 获取小班班主任字典
   * */
  getSmallClassTcDict: 'PromiseClassController/findMiniClassTeacherForSelect.do',
  /**
   * 获取 产品类型->班级类型->班级层次 字典树
   * */
  findClassLevelTree: 'ProductController/findClassLevelTree.do',
  // 获取所有双师主讲老师字典
  getAllMainClassTcDict: 'LectureClassController/getAllMainTeacher.do',
  /**
   * 根据组织类型获取当前登录用户可见的所有组织
   */
  getAllOrganizationForSelection: 'CommonAction/selectOrgOptionOfBranch.do',
  /**
   * 获取产品类型，产品系列，班级类型树结构
   */
  getProductTree: 'ProductController/findClassLevelTree.do',
  /**
   * 获取产品类型，星火双师培优各不一样
   */
  getProductTypes: 'ProductController/getProductCategoryFirstFloor.do',
  // 获取`双师辅导老师`(所属分公司下所有学习中心，按学习中心分组)
  getLoginUserTwoTeacherListGroupByArea: 'TwoTeacherClassController/getLoginUserTwoTeacherListGroupByArea.do',
  // 获取`小班任课老师`(所属分公司下所有学习中心，按学习中心分组)
  findMiniClassTeacherForSelectGroupByArea: 'PromiseClassController/findMiniClassTeacherForSelectGroupByArea.do',
  getMiniClassTeacherListForClassChange: 'ClassChangeController/getMiniClassTeacherListForClassChange.do',
  /**
   * 跟进节点提醒(crm)
   * */
  getRemindCount: 'CustomerAction/v2/getRemindCount.do',
  // 根据开关配置CODE和学习中心获取对应的配置信息
  getSwitchConfigsByCodesAndCampusIds: 'SwitchConfigController/getSwitchConfigsByCodesAndCampusIds.do',
  // 根据类型获取开关配置信息
  getSwitchConfigsByItemCodes: 'CommonAction/getSwitchConfigsByItemCodes.do',
  // 获取资源入口
  getTaggedSelectOption: 'CommonAction/getTaggedSelectOption.do',
  // 通过城市获取区域
  getAreaByCity: 'SystemAction/getCounties.do',
  getSmallClassTeacher: {
    url: '/TeacherSubjectController/getTeacherListForSelect.do',
    method: 'POST',
    dataType: 'JSON',
  },
  // 根据模板code标签获取短信模板
  getSmsTemplate: 'SmsController/getSmsTemplate.do',
  // 获取当前月各个签单季度是否可修改
  getQuarterIfCanEditList: 'ContractNewController/getQuarterIfCanEditList.do',
  // 根据 orgId 及 gradeId 获取科目(别名)
  getSubjectAlias: 'DataDictController/listWithSubjectAlias.do',
  // 银行卡号查询支行
  getBankCardInfo: 'boss-xinghuo-api/hrms/bank/getBankCardInfo',
  // 模糊查询支行名称
  searchBranchBank: {
    url: 'boss-xinghuo-api/hrms/bank/getBankBranchPage',
    method: 'POST',
    dataType: 'json',
  },
  // 模糊查询学生
  getStudentAutoComplete: 'CourseController/getStudentAutoComplate.do',
});

const bossApiConfig = buildStrictFetchApi({
  // 获取acm配置
  getFrontEndConfig: {
    url: `${bossApiUrlBase}common/getFrontEndConfig`,
    method: 'GET',
  },
});

export default {
  ...config,
  ...bossApiConfig,
};
