import buildApi, { buildStrictFetchApi } from './buildApi';
import { bossApiUrlBase } from 'CONST/config';

/**
 * Api配置信息，key为Api唯一标识，value为object-->{ url, method }，或者string-->url（method默认为GET）
 */
const edubossContrConfig = {
  findProductById: 'ProductController/findProductById.do', //根据ID获取产品信息
  editProduct: 'ProductController/editProduct.do', //保存产品信息
  getContractById: 'ContractAction/getContractById.do', //根据ID获取产品信息
  getProductList: 'ProductController/getProductList.do', //获取产品列表
  findContractByStu: 'ContractAction/findContractByStu.do', //根据学生获取合同

  deleteContractSegmentByContractId: 'ContractController/deleteContractSegmentByContractId.do', // 删除分段收费
  batchFinishContract: {
    method: 'POST',
    dataType: 'json',
    url: 'ContractController/batchFinishContract.do',
  }, // 批量结束合同

  getStudentInfo: 'ContractNewController/getStudentInfo.do', //获取本分公司学生列表
  getCrossBranchStudentInfo: 'ContractNewController/getCrossBranchStudentInfo.do', //获取非本分公司学生列表
  getContractPromotions: { url: '/PromotionController/getContractPromotions.do', method: 'POST', dataType: 'json' }, // 获取签合同产品的优惠信息
  saveContractInfo: { url: '/ContractNewController/saveContractInfo.do', method: 'POST', dataType: 'json' }, // 保存合同
  saveCusAndStuInfoInContract: {
    url: '/ContractNewController/saveCusAndStuInfoInContract.do',
    method: 'POST',
    dataType: 'json',
  }, // 保存客户和学生
  checkStudentByNameAndPhone: 'ContractNewController/checkStudentByNameAndPhone.do', //校验是否同名学生

  getProductForContractSelect: 'ContractNewController/getProductForContractSelect.do', //获取精英班，一对一，一对多，其他产品列表接口
  getClassProduct: 'ContractNewController/getClassProduct.do', //获取小班产品列表接口
  getTwoTeacherClassProduct: 'ContractNewController/getTwoTeacherClassProduct.do', //获取双师产品列表接口
  getMiniClassCoursesById: 'ContractNewController/getMiniClassCoursesById.do', //（小班）上课时间
  getTwoTeacherClassTwoInfoById: 'ContractNewController/getTwoTeacherClassTwoInfoById.do', //(双师辅班)上课时间
  getStudentMiniClassInfo: 'ContractNewController/getStudentMiniClassInfo.do', //学生在读信息

  //合同详情编辑
  initContractPrintDetail: 'ContractPrintController/initPrintContractDetail.do',
  loadContractPrintDetail: 'ContractPrintController/findPrintContractDetail.do',
  checkContractPrintDetail: 'ContractPrintController/checkContractPrintArrangeInited.do',
  saveContractPrintDetail: {
    url: 'ContractPrintController/saveContractPrintArrange.do',
    method: 'POST',
    dataType: 'json',
  },

  //科次分配
  getUserForPerformance: 'UserController/getUserForPerformance.do', //用户下拉框搜索
  getDistributionList: {
    url: 'ContractDistributionController/getDistributionListByContractId.do',
    method: 'POST',
    dataType: 'json',
  }, //绩效分配列表
  saveDistributionList: {
    url: 'ContractDistributionController/saveIncomeDistribution.do',
    method: 'POST',
    dataType: 'json',
  }, //保存绩效分配
  getDistributionHistoryList: {
    url: 'ContractDistributionController/getDistributionHistoryListByContractId.do',
    method: 'POST',
    dataType: 'json',
  }, //绩效分配历史
  canUpdateIncomeDistribution: {
    url: 'ContractDistributionController/canUpdateIncomeDistribution.do',
    method: 'POST',
    dataType: 'json',
  }, //是否可以科次分配

  // 科次分配查询
  getStudentContractProductDistributionList: {
    url: 'ContractDistributionController/getStudentContractProductDistributionList.do',
    method: 'POST',
    dataType: 'json',
  }, //科次业绩分配查询
  getContractProductDistributionInfo: {
    url: 'ContractProductController/getContractProductDistributionInfo.do',
    method: 'POST',
    dataType: 'json',
  }, //产品合同的科次分配合计数据do
  getContractProductListByStudentId: {
    url: 'ContractProductController/getContractProductListByStudentId.do',
    method: 'POST',
    dataType: 'json',
  }, //获取学生的合同产品列表
  getContractProductDistributionInfoListByStudentId: {
    url: 'ContractProductController/getContractProductDistributionInfoListByStudentId.do',
    method: 'POST',
    dataType: 'json',
  }, //获取学生的合同产品列表（包括科次业绩分配统计）

  //添加合同产品
  getCurrentBranchTeacher: 'TeacherSubjectController/getCurrentBranchTeacher.do',
  findProduct: 'ProductController/findProductForChoose.do', //获取产品列表
  // 合同列表（管理）
  getContractInfoList: { url: 'ContractManagementController/getContractInfoList.do', method: 'POST', dataType: 'json' },
  // 合同列表（管理）统计
  getContractStaticInfo: {
    url: 'ContractManagementController/getContractStaticInfo.do',
    method: 'POST',
    dataType: 'json',
  },
  // 获取某合同所有产品
  getContractProductList: { url: 'ContractManagementController/getProductList.do', method: 'POST', dataType: 'json' },
  // 根据合同id校验当前合同的分享支付是否在支付中
  checkSharePayment: { url: 'ContractAction/checkSharePayment.do', method: 'POST', dataType: 'json' },
  // 生成分享支付二维码
  createSharePayment: { url: 'ContractAction/createSharePayment.do', method: 'POST', dataType: 'json' },
  // 分享支付订单列表
  findInternetPayByContractId: 'IntenetPayController/findInternetPayByContractId.do',
  // 合同id获取合同详情(分享支付用)
  getContractForPaymentById: 'ContractAction/getContractForPaymentById.do',
  // 业绩科次统计(列表)
  getStaffDistributionStatisticsList: {
    url: 'ContractDistributionController/getStaffDistributionStatisticsList.do',
    method: 'POST',
    dataType: 'json',
  },
  // 业绩科次统计(合计)
  getStaffDistributionStatisticsInfo: {
    url: 'ContractDistributionController/getStaffDistributionStatisticsInfo.do',
    method: 'POST',
    dataType: 'json',
  },
  // 业绩科次统计(列表)
  getRefundResponsibleContractProductList: {
    url: 'RefundController/getRefundResponsibleContractProductList.do',
    method: 'POST',
    dataType: 'json',
  },

  // 根据合同id获取合同的支付类型(A, B, AB)
  getFundsPayType: 'ContractAction/getFundsPayType.do',
  // 收款记录与银行流水一致性检查匹配失败列表(错误收款记录)
  getPosPayDataMatchErrorList: {
    url: 'PosPayDataController/getPosPayDataMatchErrorList.do',
    method: 'POST',
    dataType: 'json',
  },
  // 收款记录不匹配银行流水列表
  getNotMatchFundList: {
    url: 'FundsHistoryController/getNotMatchFundList.do',
    method: 'POST',
    dataType: 'json',
  },
  // 修改收款记录
  updateFundHistoryPos: 'ContractAction/updateFundHistoryPos.do',
  // 产品年级错误列表（分页）
  remindByProductGradeErrList: 'ContractAction/remindByProductGradeErrList.do',

  // 流量营销流水(列表)
  getPaymentRecordList: {
    url: 'InsightPaymentRecordController/getPaymentRecordList.do',
    method: 'POST',
    dataType: 'json',
  },
  // 流量营销流水-导出
  exportInsightPaymentRecordListExcel: {
    url: 'ExportCenterController/exportInsightPaymentRecordListExcel.do',
    method: 'POST',
    dataType: 'json',
  },

  // 查询合同最后一次分享支付新记录(返回数据空就是没有支付记录) @param.contractId
  selectLastContractSharePayment: 'ContractSharePaymentController/selectLastContractSharePayment.do',
  // 根据合同分享支付ID取消分享支付 @param.contractSharePaymentId
  cancelContractSharePayment: 'ContractSharePaymentController/cancleContractSharePayment.do',
  // 结束合同
  finishContract: { url: 'ContractController/finishContract.do', method: 'POST', dataType: 'json' },

  /**
   * 合同签订接口 singContract
   * */
  // 加载学生可选优惠券
  getPromotionCouponsByStudent: 'promotion/api/getPromotionCouponsByStudent.do',
  // 添加合同购物车-选择产品-查询
  selectProductForContract: 'ProductController/selectProductForContract.do',
  // 根据学生id判断合同类型
  getNewSignContractType: 'ContractAction/getNewSignContractType.do',
  // 获取客户学生信息
  getContractStudentInfo: 'contract/student/info.do',
  // 获取合同事务唯一ID
  getTransactionUUID: { url: 'CommonAction/getTransactionUUID.do', method: 'GET', dataType: 'text' },
  // 查询关联产品信息(同产品的会合并为一个数据)
  queryLinkProductInfo: 'ClassRenewStudentController/queryProductInfo.do',
  // 查询关联产品信息(平铺)
  queryProductInfoNew: 'ClassRenewStudentController/queryProductInfoNew.do',
  // 自动计算优惠(小班、双师的优惠活动、优惠券)
  calculatePromotion: { url: 'promotion/api/calculateActivityPromotion.do', method: 'POST', dataType: 'json' },
  // 根据organizationId获取产品优惠
  getProductActivePromotionList: 'PromotionController/getProductActivePromotionList.do',
  //  计算产品优惠(产品本身关联的优惠)
  calculatePrdPromotion: 'PromotionController/calculatePromotion.do',
  calculatePromotionNew: 'PromotionController/calculatePromotionNew.do',
  // 获取小班实际购买课时数
  findMiniClassWillBuyHour: 'CourseController/findMiniClassWillBuyHour.do',
  // 获取双师实际购买课时数
  findClassWillBuyHour: 'TwoTeacherClassController/findClassWillBuyHour.do',
  // 保存合同
  saveContractInfoNew: { url: 'ContractController/saveContractInfo.do', method: 'POST', dataType: 'json' },
  // 批量保存合同
  batchSaveContractInfo: { url: 'ContractController/batchSaveContractInfo.do', method: 'POST', dataType: 'json' },
  // 批量修改合同
  batchEditContractInfo: { url: 'ContractController/batchEditContractInfo.do', method: 'POST', dataType: 'json' },
  // 获取学生年级
  getStudentGradeInContinuedCycle: 'StudentController/getStudentGradeInContinuedCycle.do',
  // 订单结算时获取可选优惠券和不可选优惠券
  getSuitablePromotion: { url: 'promotion/api/getSuitablePromotion.do', method: 'POST', dataType: 'json' },
  // 订单结算时获取最优的优惠券
  getMaxCouponPromotion: { url: 'promotion/api/getMaxCouponPromotion.do', method: 'POST', dataType: 'json' },
  // 统计传入的班级中连报科目数和单报科目数,用于判断预存抵扣券叠加券使用数量上限(连报一科能用一张连报券/两张单报券，单报一科能用一张单报券)
  getLimitForUseCoupon: { url: 'promotion/api/getLimitForUseCoupon.do', method: 'POST', dataType: 'json' },
  // 客户手机号查询客户
  contractCustomerSearch: 'contract/student/search.do',
  // 客户id查询详情(包括客户名下的学生)
  findCustomerById: 'CustomerAction/findCustomerById.do',
  // 获取合同详情(新)
  getContractInfoById: 'ContractController/getContractById.do',
  // 保存修改合同
  editContractInfo: { url: 'ContractController/editContractInfo.do', method: 'POST', dataType: 'json' },
  // 校验是否存在同名学生
  checkCustomerStudentName: 'CustomerAction/checkCustomerStudengName.do',
  // 获取同名学生列表
  getStudentListByName: 'StudentController/getStudentListByName.do',
  // 计算优惠券(支持券叠加)
  calculateCoupon: { url: 'promotion/api/calculateCoupon.do', method: 'POST', dataType: 'json' },
  // 合同收款-获取配置化支付方式
  getContractFundsPayTypes: 'ContractNewController/getContractFundsPayTypes.do',
  // 默认的现金、第三方平台等支付方式之外额外配置的值方式(电子账户A/银行汇款A/微信A/支付宝A等)
  getCampusFundsPayTypes: 'ContractNewController/getCampusFundsPayTypes.do',
  // 网络支付接口
  // 学习中心所属公司
  getBelongBranchByCampusId: 'CommonAction/getBelongBranchByCampusId.do',
  // 合同学习中心支付方式
  getCampusPayway: 'ContractAction/getCampusPayway.do',
  // 通过学习中心获取终端号选项
  getPosNumSelectOptionByCampusId: 'PosMachineController/getPosNumSelectOptionByCampusId.do',
  // 请选择POS机登录帐号
  getPosNumSelectOptionByCampusIdAndTypes: 'PosMachineController/getPosNumSelectOptionByCampusIdAndTypes.do',
  // 选择微信/支付宝支付/微信A/支付宝A 生成收款码
  sendBatchPayDetailInfo: { url: 'IntenetPayController/sendBatchPayDetailInfo.do', method: 'POST', dataType: 'json' },
  // 合并支付-检查和更新支付结果
  updateAndCheckBossBatchPayInfo: 'IntenetPayController/updateAndCheckBossBatchPayInfo.do',
  // 检查是否有相同支付号与终端号的合同
  getFundsInfoByPosId: 'ContractNewController/getFundsInfoByPosId.do',
  // 根据模板code标签获取短信模板
  getSmsTemplate: 'SmsController/getSmsTemplate.do',
  // 发送短信提醒确认合同短信
  sendContractConfirmSmsMessage: { url: 'ContractNewController/sendContractConfirmSmsMessage.do', method: 'POST' },
  // 发送短信(确认历史合同)
  sendContractConfirmSms: { url: 'ContractConfirmController/history/sendContractConfirmSms.do', method: 'POST' },
  // 合同付款-保存合同
  saveFundOfContract: 'ContractAction/saveFundOfContract.do',
  // 获取多合同详情
  getContractByIds: 'ContractController/getContractByIds.do',
  // 一键分配金额(用于多合同收款)
  calculateBatchPayArrangeMoney: 'ContractNewController/calculateBatchPayArrangeMoney.do',

  // 获取合同修改记录
  getContractModifyList: 'ContractNewController/getContractModifyList.do',
  // 获取电子合同pdf文件路径
  getElectronicContractPath: 'ContractAction/getElectronicContractPath.do',
  // 确认并支付合同(返回确认合同的批次id)
  batchSaveFundOfContractAndConfirm: {
    url: 'ContractNewController/batchSaveFundOfContractAndConfirm.do',
    method: 'POST',
    dataType: 'json',
  },
  // (批量)确认合同
  batchConfirmingContract: {
    url: 'ContractNewController/batchConfirmingContract.do',
    method: 'POST',
    dataType: 'json',
  },
  // (批量)(分享)支付
  createBatchSharePayment: {
    url: 'ContractSharePaymentController/createBatchSharePayment.do',
    method: 'POST',
    dataType: 'json',
  },
  // (批量)确认、支付
  batchConfirmingContractAndSharePay: {
    url: 'ContractNewController/batchConfirmingContractAndSharePay.do',
    method: 'POST',
    dataType: 'json',
  },
  // 分享支付 支付完成订单
  getBatchSharePaymentRecordByContractId: '/ContractSharePaymentController/getBatchSharePaymentRecordByContractId.do',
  // 合同列表确认付款人 - 短信
  sendSmsCode: {
    url: 'SmsController/sendSmsCode.do',
    method: 'POST',
    dataType: 'json',
  },
  // 合同列表确认付款人
  confirmSavePayer: {
    url: 'ContractNewController/confirmSavePayer.do',
    method: 'POST',
    dataType: 'json',
  },
  // 获取线下流程审核人
  getContractConfirmAuditUserMsg: {
    url: 'ContractNewController/getContractConfirmAuditUserMsg.do',
    method: 'POST',
    dataType: 'json',
  },
  // 发起线下确认流程
  submitContractConfirmAudit: {
    url: 'ContractNewController/submitContractConfirmAudit.do',
    method: 'POST',
    dataType: 'json',
  },
  // 合并支付-线下录入付款信息
  batchSaveFundOfContract: {
    url: 'ContractNewController/batchSaveFundOfContract.do',
    method: 'POST',
    dataType: 'json',
  },
  // 获取客户名下的定金券
  getDepositCouponsByStudent: 'ContractNewController/getDepositCouponsByStudent.do',
  // 本次收款全额使用定金券抵扣
  batchDepositUsed: {
    url: 'ContractNewController/batchDepositUsed.do',
    method: 'POST',
    dataType: 'json',
  },
  // 根据合同ID获取合同确认记录
  getLastOfflineContractConfirmRecord: 'ContractNewController/getLastOfflineContractConfirmRecord.do',
  // 获取确认协议界面数据
  batchConfirmActivityContract: {
    url: 'ContractAction/batchConfirmActivityContract.do',
    method: 'POST',
    dataType: 'json',
  },
  // 发送确认协议短信
  sendSmsForActivityContract: {
    url: 'ContractAction/sendSmsForActivityContract.do',
    method: 'POST',
    dataType: 'json',
  },
  // 合同是否可以进行支付(合同使用的抵扣券是否已经生成了收款记录)
  getContractCouponDepositSaveFundTag: 'ContractController/getContractCouponDepositSaveFundTag.do',
  // 历史合同列表（双师）
  getHistoryContractListSs: 'twoTeacher/student/contract/history/list.do',
  // 查询用户（历史合同跟进人）
  getUserSimple: 'UserController/findUserSimpleVos.do',
  // 批量分配学生历史合同跟进人
  batchDistributeDeliverSs: {
    url: 'ContractHistoryStudentDeliverController/batchInsert.do',
    method: 'POST',
    dataType: 'json',
  },
  // 确认历史合同列表
  confirmHistoryContractList: {
    url: 'ContractConfirmController/history/pageUnConfirmAndConfirming.do',
    method: 'POST',
    dataType: 'json',
  },
  // 确认历史合同
  confirmHistoryContract: {
    url: 'ContractConfirmController/history/batchConfirm.do',
    method: 'POST',
  },
  // 确认历史合同批量发送短信
  confirmHistoryContractBatchSendSms: {
    url: 'ContractConfirmController/history/batchConfirmAndSendConfirmSms.do',
    method: 'POST',
    dataType: 'json',
  },
  saveFundWashOfContract: {
    url: 'ContractAction/saveFundWashOfContract.do',
    method: 'GET',
  },
  // 学生是否已签订常规合同
  getStudentIfHasContract: 'ContractController/getStudentIfHasContract.do',
  pageTransferRecords: {
    url: 'boss-xinghuo-api/income/electronicAccountTransfer/pageTransferRecords',
    method: 'POST',
    dataType: 'json',
  },
  getIncomeDistributionByFundsChangeId: {
    url: 'boss-xinghuo-api/income/electronicAccountTransfer/getIncomeDistributionByFundsChangeId',
    method: 'GET',
  },
  saveIncomeDistributionRecord: {
    url: 'boss-xinghuo-api/income/electronicAccountTransfer/manualUpdateIncomeDistribution',
    method: 'POST',
    dataType: 'json',
  },
  pageIncomeDistributionRecords: {
    url: 'boss-xinghuo-api/income/electronicAccountTransfer/pageIncomeDistributionRecords',
    method: 'POST',
    dataType: 'json',
  },
  /** 根据合同id修改合同的签单季度类型 */
  updateQuarterType: {
    url: 'ContractNewController/updateQuarterType.do',
    method: 'POST',
    dataType: 'json',
  },
  renameStudentName: {
    url: 'StudentController/onlyRenameStudent.do',
    method: 'POST',
    dataType: 'json',
  },

  /** 分段收费-合同列表 */
  getContractSegmentChargeInfo: 'boss-xinghuo-api/contract/getContractSegmentChargeInfo',
  updateContractSegmentChargeConfigSnapshot: {
    url: 'boss-xinghuo-api/contract/updateContractSegmentChargeConfigSnapshot',
    method: 'POST',
    dataType: 'json',
  },
  getSegmentChargeItemByContractIds: 'ContractController/getSegmentChargeItemByContractIds.do',
};

const xhbossApiContrConfig = {
  /**
   * 流量平台收款统计列表查询
   */
  getInsightChargeStatistics: {
    url: '/eduboss/boss-xinghuo-api/insightChargeStatistics/page',
    method: 'POST',
    dataType: 'json',
    withCredentials: true,
  },

  /**
   * 流量平台收款统计列表查询导出
   */
  exportInsightChargeStatistics: {
    url: '/eduboss/boss-xinghuo-api/insightChargeStatistics/page?isExport=pull',
    method: 'POST',
    dataType: 'json',
    withCredentials: true,
  },

  //查询科次业绩分配列表
  getXhSbjPerfAllocationList: {
    url: '/eduboss/boss-xinghuo-api/income/contractProduct/list',
    method: 'POST',
    dataType: 'json',
    withCredentials: true,
  },
  //修改科次业绩分配
  updXhSbjPerfAllocationList: {
    url: '/eduboss/boss-xinghuo-api/income/contractProduct/manualUpdate',
    method: 'POST',
    dataType: 'json',
    withCredentials: true,
  },
  //分页查询科次业绩分配记录
  getXhSbjPerfAllocationHistList: {
    url: '/eduboss/boss-xinghuo-api/income/contractProduct/pageStatements',
    method: 'POST',
    dataType: 'json',
    withCredentials: true,
  },
  // 获取小班签单季度
  getCurrentQuarterByOrganizationId: {
    url: bossApiUrlBase + 'contract/quarterOrganizationConfig/getCurrentQuarterByOrganizationId',
    method: 'GET',
    dataType: 'json',
  },
  // 业绩分配退费记录
  pageRefundIncomeRecords: {
    url: `${bossApiUrlBase}income/pageRefundIncomeRecords`,
    method: 'POST',
    dataType: 'json',
  },
  // 导出业绩分配退费记录
  exportPageRefundIncomeRecords: {
    url: `${bossApiUrlBase}income/pageRefundIncomeRecords?isExport=pull`,
    method: 'POST',
    dataType: 'json',
  },
  /** 查询学生有效合同产品信息，附带合同、产品、在班等信息
   * https://xiaojiaoyukeji.yuque.com/xh-zh/iy6qk9/vuvm7g
   */
  listValidSmallClassWithProductByStudentId: {
    url: '/eduboss/boss-xinghuo-api/contract/contractProduct/listValidSmallClassWithProductByStudentId',
    method: 'POST',
    dataType: 'json',
  },
  // 查询分公司全额收费开关接口
  getBranchSwitchConfigsByCodesAndCampusId: {
    url: '/eduboss/SwitchConfigController/getBranchSwitchConfigsByCodesAndCampusId.do',
    method: 'GET',
  },
};

// 财务合规化相关
const finacialConfig = {
  // 根据合同快照记录ID获取快照数据
  getContractSnapshotById: {
    url: bossApiUrlBase + 'contract/contractSnapshotRecord/getContractSnapshotById',
    method: 'GET',
    dataType: 'json',
    withCredentials: true,
  },
};

const edubossContrReqMap = buildApi(edubossContrConfig);

const xhbossApiReqMap = buildStrictFetchApi(xhbossApiContrConfig);

const finacialConfigMap = buildStrictFetchApi(finacialConfig);

const totalReqMap = {
  ...edubossContrReqMap,
  ...xhbossApiReqMap,
  ...finacialConfigMap,
};

export default totalReqMap;
