import buildApi, { buildStrictFetchApi } from './buildApi';

/**
 * Api配置信息，key为Api唯一标识，value为object-->{ url, method }，或者string-->url（method默认为GET）
 */
const eduBossConfig = {
  // 课程获取考勤照片或者凭证信息
  getAttendancePhotoOrCertInfo: {
    url: 'ClassCourseInfoController/course/getAttendancePhotoOrCertInfo.do',
    method: 'GET',
  },
  // 批量更新小班课程
  batchUpdateMiniClassCourse: {
    url: 'CourseController/batchUpdateMiniClassCourse.do',
    method: 'POST',
    dataType: 'json',
  },
  // 获取全部小班课程列表
  getAllMiniClassCourse: {
    url: 'boss-xinghuo-api/course/miniClassCourse/listMiniClassCourse',
    method: 'POST',
    dataType: 'json',
  },
  // 查询教室视图的小班课程列表
  listRoomViewOfMiniClassCourse: {
    url: 'boss-xinghuo-api/course/miniClassCourse/listRoomViewOfMiniClassCourse',
    method: 'POST',
    dataType: 'json',
  },
  // 获取课程状态
  getLevelCategory: {
    url: 'boss-xinghuo-api/dataDict/listByCategory',
    method: 'GET',
  },
  // 批量删除课程
  batchDeleteCourse: {
    url: 'CourseController/batchDelMiniClassCourses.do',
    method: 'POST',
    dataType: 'json',
  },
  // 批量删除课程
  batchModifyCourse: {
    url: 'CourseController/updateMiniClassCourseByCourseIds.do',
    method: 'POST',
    dataType: 'json',
  },
  // 根据课程ID批量修改上课方式
  updateTeachingTypeByIds: {
    url: 'CourseController/updateTeachingTypeByIds.do',
    method: 'POST',
    dataType: 'json',
  },
  // 根据条件批量修改小班课程上课方式
  updateTeachingTypeByParams: {
    url: 'boss-xinghuo-api/course/miniClassCourse/updateTeachingTypeByParams',
    method: 'POST',
    dataType: 'json',
  },
  getTeacherByCompany: {
    url: 'TeacherSubjectController/getTeacherListForSelect.do',
    method: 'POST',
  },
  // 获取 oss 直传后端签名
  getAliyunOSSKeyWithoutCallBack: 'CommonAction/getAliyunOSSKeyWithoutCallBack.do',
  // 批量建班根据导入类型获取导入记录
  listRecords: 'ExcelImport/listRecords.do',
  // 初始化导入数据记录
  initRecord: {
    url: 'ExcelImport/initRecord.do',
    method: 'POST',
    dataType: 'json',
  },
  editCourseRequirement: 'CourseController/editCourseRequirement.do', //保存排课需求信息
  getPromiseSubjectList: 'PromiseClassController/getPromiseSubjectList.do', //查询精英班科目分配列表
  findStudentInfoByPromiseStudentId: 'PromiseClassController/findStudentInfoByPromiseStudentId.do', //根据精英班学生ID查询详细信息
  savePromiseSubject: { url: 'PromiseClassController/savePromiseSubject.do', method: 'POST', dataType: 'json' }, //保存精英班科目分配
  getPromiseClassList: 'PromiseClassController/getPromiseClassList.do', //精英班列表
  findMiniClassTeacherForSelect: 'PromiseClassController/findMiniClassTeacherForSelect.do', //查询小班正常老师列表
  findMiniClassByPromiseSubjectId: 'PromiseClassController/findMiniClassByPromiseSubjectId.do', //查询可报读小班列表
  getPromiseStudentList: 'PromiseClassController/getPromiseStudentList.do', //精英班学生管理（班主任）列表
  getEcsContractInfo: 'PromiseClassController/getEcsContractInfo.do', //获取精英班合同信息
  getEcsContractChargeInfo: 'PromiseClassController/getEcsContractChargeInfo.do', //获取精英班合同扣费详情
  getDynamicVoByPromiseStudentId: { url: 'PromiseClassController/getDynamicVoByPromiseStudentId.do', method: 'POST' }, //查询审核动态
  getOaPromiseReturnAuditUserList: {
    url: 'PromiseClassController/getOaPromiseReturnAuditUserList.do',
    method: 'POST',
    dataType: 'json',
  }, //中途退费查询审核人接口
  savaPromiseReturnAuditInfo: {
    url: 'PromiseClassController/savaPromiseReturnAuditInfo.do',
    method: 'POST',
    dataType: 'json',
  }, //精英班退费申请或审批

  // 检查合同产品对应的合同是否存在未审核的收款记录
  isContrFundRecordUnaudited: {
    url: 'PromiseClassController/checkContractHasNotAuditFunds.do',
    method: 'GET',
  },

  getEcsContractChargeList: 'PromiseClassController/getEcsContractChargeList.do', // 精英班合同扣费详情列表
  getPromiseSubjectDetailList: 'PromiseClassController/getPromiseSubjectDetailList.do', // 精英班课程分配详情
  brenchIncludeOtherCampus: 'CommonAction/brenchIncludeOtherCampus.do', //校历与课程模板管理，获取分公司
  getCourseModalList: 'CourseModalController/getCourseModalList.do', //获取校历模板列表
  deleteCourseModal: { url: 'CourseModalController/deleteCourseModal.do', method: 'POST' }, //校历模板删除
  saveCourseModal: { url: 'CourseModalController/saveCourseModal.do', method: 'POST', dataType: 'json' }, //校历模板保存
  getAllCurrentClassroom: { url: 'ClassroomManageController/getAllCurrentClassroom.do', method: 'POST' }, //获取当前用户的所有教室列表
  getTeacherListForSelect: { url: 'TeacherSubjectController/getTeacherListForSelect.do', method: 'POST' }, //老师列表
  clearAllClassCourse: { url: 'SmallClassController/clearAllClassCourse.do', method: 'POST' }, //删除排课
  saveEditClassModal: { url: 'SmallClassController/saveEditClassModal.do', method: 'POST' }, //保存排课
  getSmallClassListOnModal: {
    url: 'SmallClassController/getSmallClassListOnModal.do',
    method: 'POST',
    dataType: 'json',
  }, //获取已排小班列表 （班课列表公用此接口）
  getSmallClassListByTeacher: {
    url: 'SmallClassController/getSmallClassListByTeacher.do',
    method: 'POST',
    dataType: 'json',
  }, //根据老师获取该老师所有的模版排课班级列表
  getStudentMiniClassAttendent: 'SmallClassController/getStudentMiniClassAttendent.do', //查询学生的小班课程列表miniClassId, studentId
  getMiniClassCourseDateInfo: 'SmallClassController/getMiniClassCourseDateInfo.do', // 获取课程日期列表miniClassId
  initPromiseClassSubject: 'PromiseClassController/initPromiseClassSubject.do', // 科目规划自动适配产品中的规划方案

  getStudentClassCourseList: { url: 'CourseController/getStudentClassCourseList.do', method: 'POST', dataType: 'json' }, //学生课程表
  getTeacherClassCourseList: { url: 'CourseController/getTeacherClassCourseList.do', method: 'POST', dataType: 'json' }, //老师课程表
  getClassCourseDetail: 'CourseController/getClassCourseDetail.do', // 班课详情接口courseId=COU0006205123&productClassId=COU000000815737
  findStudentByCampus: 'StudentController/findStudentByCampus.do', // 学生课程表的学生
  getUserAutoComplateEnhanced: 'UserController/getUserAutoComplateEnhanced.do', // 老师课程表的老师roleCode=TEATCHER

  getCampusByBranchIdApi: 'OriginalClassController/getCampusByBranchAndRole.do',
  getNextClassInfoApi: 'OriginalClassController/getNextYearsAndQuarterAndGrade.do',
  loadClassesApi: 'OriginalClassController/getClassList.do',
  loadStudentsApi: 'OriginalClassController/getStudentSeriesClassInfo.do',
  loadTcApi: 'OriginalClassController/getTeacherByName.do',
  loadMainTcApi: 'TwoTeacherClassController/getLoginUserMainTeacherList.do',
  loadAssitTcApi: 'TwoTeacherClassController/getLoginUserTwoTeacherList.do',
  loadSmallTcApi: 'TeacherSubjectController/getTeachersForSelect.do',
  loadSmallClassTcDict: 'PromiseClassController/findMiniClassTeacherForSelect.do', // 获取小班老师字典
  // 原班续报关联列表主查询(不含`关联状态`字段)
  loadListApi: 'OriginalClassController/getOriginalClassList.do',
  // 原班续报关联班级关联状态查询
  getClassAssociatedStatus: 'OriginalClassController/getClassAssociatedStatus.do',
  loadConflictApi: { url: 'OriginalClassController/getStudyConflictVo.do', method: 'POST', dataType: 'json' },
  saveApi: { url: 'OriginalClassController/addStudentSeriesClassInfo.do', method: 'POST', dataType: 'json' },
  delApi: 'OriginalClassController/deleteStudentSeriesClass.do',
  delAllApi: 'OriginalClassController/deleteSeriesClass.do',
  getOaPromiseClassTargetEndAuditUserList: {
    url: 'PromiseClassController/getOaPromiseClassTargetEndAuditUserList.do',
    method: 'POST',
    dataType: 'json',
  }, //完结退费查询审核人
  submitPromiseResult: { url: 'PromiseClassController/savaPromiseClassTargetEnd.do', method: 'POST', dataType: 'json' }, //提交精英班完结申请/审核
  getPromiseResultDetail: 'PromiseClassController/findTargetEndInfoByPromiseStudentId.do', //获取精英班完结申请/审核详情
  getEcsTargetEndContractChargeInfo: 'PromiseClassController/getEcsTargetEndContractChargeInfo.do', //获取课程状态为已完结（审批状态为审批通过）的精英班扣费详情
  /**
   * 小班
   * */
  // 获取小班班级列表
  getMiniClassList: 'CourseController/getMiniClassList.do',
  // 根据小班id获取课程
  getMiniClassCourseDetail: 'CourseController/getMiniClassCourseDetail.do',
  // 获取电子签名列表
  getEsignatureList: 'CourseController/listStudentAttendanceSign.do',

  listWithSubjectAlias: { url: 'DataDictController/listWithSubjectAlias.do', method: 'GET' },

  /**
   * 双师管理(集团)
   * */
  // 获取双师主班列表
  getMainClassList: 'TwoTeacherClassController/findTwoTeacherClassList.do',
  // 编辑或删除主班
  editAndDeleteMainClass: { url: 'TwoTeacherClassController/saveTwoTeacherClass.do', method: 'POST', dataType: 'json' },
  // 获取双师主班信息
  getMainClassInfo: 'TwoTeacherClassController/findTwoTeacheClassrById.do',
  // 获取产品选择列表(新)
  findTwoTeacherProductForChoose: 'ProductController/findTwoTeacherProductForChoosePagination.do',
  // 获取产品选择列表(旧版，弃用)
  getProductSelectList: 'ProductController/findTwoTeacherProductForChoose.do',
  // 获取双师课程模板
  getCourseTemplate: 'CourseModalController/getCourseModalList.do',
  // 获取老师字典数据
  getTeacherDict: 'TeacherSubjectController/getTeacherForSelectByName.do',
  // 批量新增主班
  submitAddMainClass: {
    url: 'TwoTeacherClassController/saveMultiTwoTeacherClass.do',
    method: 'POST',
    dataType: 'json',
  },
  // 获取主班-课程列表
  getCourseList: 'TwoTeacherClassController/findTwoTeacherClassCourseList.do',
  // 获取主班-学生列表
  getStudentList: 'TwoTeacherClassController/findTwoTeacherClassStudentList.do',
  // 获取总课时数和课时时长(用于新增主班)
  findTotalHoursAndClassTimeLength: 'ProductController/TwoTeacherProd/findTotalHoursAndClassTimeLength.do',
  // 获取双师辅班列表
  findTwoTeacherClassTwoList: 'TwoTeacherClassController/findTwoTeacherClassTwoList.do',
  // 根据双师辅班id获取课程
  findCourseByClassTwoId: 'TwoTeacherClassController/findCourseByClassTwoId.do',
  // 批量修改双师主讲老师
  batchUpdateTwoTeacherClassTeacher: {
    url: 'TwoTeacherClassController/batchUpdateTwoTeacherClassTeacher.do',
    method: 'POST',
    dataType: 'json',
  },
  // 批量修改双师直播间
  batchUpdateTwoTeacherClassLiveRoom: {
    url: 'TwoTeacherClassController/batchUpdateTwoTeacherClassLiveRoom.do',
    method: 'POST',
    dataType: 'json',
  },
  /**
   * 在线报读4期
   * */
  // 获取续报双师辅班列表
  getAuxClassList: { url: 'OriginalClassController/getAssistClassList.do', method: 'POST', dataType: 'json' },
  // 批量关联
  addStudentSeriesClassInfo: {
    url: 'OriginalClassController/addStudentSeriesClassInfos.do',
    method: 'POST',
    dataType: 'json',
  },
  // 获取关联进度
  getHandleProgress: { url: 'OriginalClassController/getHandleProgress.do', method: 'POST' },
  // 获取取消关联进度
  getCancelProgress: { url: 'OriginalClassController/getCancelProgress.do', method: 'POST' },
  // 获取关联记录
  getRenewClassRecordList: { url: 'OriginalClassController/getRenewClassRecordList.do', method: 'GET' },
  // 获取关联失败明细
  getLinkFailClassList: { url: 'OriginalClassController/getSeriesClassList.do', method: 'POST', dataType: 'json' },
  // 批量取消关联
  plentyDeleteSeriesClass: {
    url: 'OriginalClassController/plentyDeleteSeriesClass.do',
    method: 'POST',
    dataType: 'json',
  },
  // 调整冲突关联列表
  getConflictStudentSeriesList: {
    url: 'OriginalClassController/getConflictStudentSeriesList.do',
    method: 'POST',
    dataType: 'json',
  },
  // 调整冲突关联
  resolveConflictStudentSeries: {
    url: 'OriginalClassController/resolveConflictStudentSeries.do',
    method: 'POST',
    dataType: 'json',
  },
  /**
   * 双师/小班转班
   * */
  // 双师/小班学生列表(用于转班)
  getClassChangeStudentList: {
    url: 'ClassChangeController/getClassChangeStudentList.do',
    method: 'POST',
    dataType: 'json',
  },
  // 双师/小班学生列表(用于转班)
  findTwoTeacherClassTwoListSsNew: {
    url: 'ClassChangeNewSsController/findTwoTeacherClassTwoListSsNew.do',
    method: 'POST',
    dataType: 'json',
  },
  // 双师 小班-班级列表(用于转班)
  getClassChangeMiniClassListSs: {
    url: 'ClassChangeNewSsController/getClassChangeMiniClassListSsNew.do',
    method: 'POST',
    dataType: 'json',
  },
  // 双师/小班 转班
  studentChangeClassSs: {
    url: 'ClassChangeNewSsController/studentChangeClassSsNew.do',
    method: 'POST',
    dataType: 'json',
  },
  // 小班-班级列表(用于转班)
  getClassChangeMiniClassList: {
    url: 'ClassChangeController/getClassChangeMiniClassList.do',
    method: 'POST',
    dataType: 'json',
  },
  // 双师/小班 转班
  studentChangeClass: { url: 'ClassChangeController/studentChangeClass.do', method: 'POST', dataType: 'json' },

  // 星火 小班-班级列表(用于转班)
  getClassChangeMiniClassListXH: {
    url: 'ClassChangeController/getClassChangeMiniClassList.do',
    method: 'POST',
    dataType: 'json',
  },
  // 星火/小班 转班
  studentChangeClassXH: { url: 'ClassChangeController/studentChangeClass.do', method: 'POST', dataType: 'json' },

  // 双师/小班 转班 判断是否需要转班申请
  getFlagsOaByClassIds: {
    url: 'ClassChangeNewSsController/getChangeTypeByClassIdsStudentIdsSsNew.do',
    method: 'POST',
    dataType: 'json',
  },
  // 双师/小班 转班提交OA申请
  submitChangeClassOa: {
    url: 'ClassChangeNewSsController/submitChangeClassOaSsNew.do',
    method: 'POST',
    dataType: 'json',
  },
  // 双师/小班 重新申请转班OA
  reSubmitChangeClassOa: {
    url: 'ClassChangeNewSsController/reSubmitChangeClassOaSsNew.do',
    method: 'POST',
    dataType: 'json',
  },
  // 双师/小班 撤回审批
  abortChangeClassOa: {
    url: 'ClassChangeNewSsController/abortChangeClassOaByOaInstanceIdSsNew.do',
    method: 'GET',
    dataType: 'json',
  },
  // 转班 获取双师课程列表（新）
  getSSChangeClsCourse: {
    url: 'ClassChangeNewSsController/findCourseByOldClassIdNewClassIdSsNew.do',
    method: 'POST',
    dataType: 'json',
  },
  // 转班 获取小班课程列表（新）
  getSSChangeClsMiniClsCourse: {
    url: 'ClassChangeNewSsController/getMiniClassCourseByOldClassIdNewClassIdSsNew.do',
    method: 'POST',
    dataType: 'json',
  },
  // 转班 根据转班记录ID 获取转班OA记录详情
  getChangeClassOaRecordById: {
    url: 'ClassChangeNewSsController/getChangeClassOaRecordByIdSsNew.do',
    method: 'GET',
    dataType: 'json',
  },
  // 转班 根据学生id和转出班id 获取转班OA记录详情
  getClassChangeOaRecordByParams: {
    url: 'ClassChangeNewSsController/getClassChangeOaRecordByParamsSsNew.do',
    method: 'POST',
    dataType: 'json',
  },
  // 转班 获取转班申请记录列表
  getChangeClassOaRecordList: {
    url: 'ClassChangeNewSsController/getChangeClassOaRecordListSsNew.do',
    method: 'POST',
    dataType: 'json',
  },
  /**
   * 双师/小班报班
   * */
  // 双师/小班学生列表(用于报班)
  // getDropClassStudentList: 'TwoTeacherClassController/getDropClassStudentList.do',
  getDropClassStudentList: {
    url: 'boss-xinghuo-api/course/miniClassStudent/pageDropMiniClassStudent',
    method: 'POST',
    dataType: 'json',
  },
  // 双师-班级列表(用于报班)
  getClassTwoListForEnrollInClass: {
    url: 'TwoTeacherClassController/getClassTwoListForEnrollInClass.do',
    method: 'POST',
    dataType: 'json',
  },
  // 小班-班级列表(用于报班)
  getMiniClassListForEnrollInClass: {
    url: 'SmallClassController/getMiniClassListForEnrollInClass.do',
    method: 'POST',
    dataType: 'json',
  },
  // 双师报班
  signUpNewClassForTwoTeacher: {
    url: 'TwoTeacherClassController/signUpNewClassForTwoTeacher.do',
    method: 'POST',
    dataType: 'json',
  },
  // 小班报班
  signUpNewClassForMiniClass: {
    url: 'SmallClassController/signUpNewClassForMiniClass.do',
    method: 'POST',
    dataType: 'json',
  },
  // 导出Excel
  exportDropClassStudentList: 'TwoTeacherClassController/exportDropClassStudentList.do',
  /**
   * 双师(线上)讲座
   * */
  // 新增讲座(批量，多分公司，多讲座)
  saveLectureClassTw: { url: 'LectureClassController/saveLectureClassTw.do', method: 'POST', dataType: 'json' },
  // 获取产品信息
  lectureClassTwProductSelect: 'LectureClassController/lectureClassTwProductSelect.do',
  // 讲座主查询
  queryLectureClassTw: { url: 'LectureClassController/queryLectureClassTw.do', method: 'POST', dataType: 'json' },
  // 修改讲座
  updateLectureClassTw: { url: 'LectureClassController/updateLectureClassTw.do', method: 'POST', dataType: 'json' },
  // 删除讲座
  delLectureClassTw: 'LectureClassController/delLectureClassTw.do',
  // 讲座详情接口
  queryLectureDetailTw: 'LectureClassController/queryLectureDetailTw.do',
  // 获取可报名讲座的学生列表
  queryLectureWantStudent: 'LectureClassController/queryLectureWantStudent.do',
  // 报名讲座
  addStudentToLectureClass: 'LectureClassController/addStudentToLectureClass.do',
  // 讲座已报名学生列表
  queryLectureStudentTw: 'LectureClassController/queryLectureStudentTw.do',
  // 退讲座(退班)
  removeStudentFromLecture: 'LectureClassController/removeStudentFromLecture.do',
  // 讲座学生批量考勤
  auditLectureClassStudent: 'LectureClassController/auditLectureClassStudent.do',
  // 讲座学生批量扣费
  chargeLectureClassStudent: 'LectureClassController/chargeLectureClassStudent.do',
  // 获取线上考勤图片
  getLectureClassAttendanceFile: 'LectureClassController/getAttendanceFileListByLectureClassId.do',
  // 保存线上考勤图片
  saveLectureClassAttendanceFile: {
    url: 'LectureClassController/saveLectureClassAttendanceFile.do',
    method: 'POST',
    dataType: 'json',
  },
  // 获取更新模板对应的课程数量
  getClassDataByModal: 'CourseModalController/getClassDataByModal.do',
  /**
   * 双师主辅班课表
   * */
  // 获取双师主班课表数据
  getTwoTeacherClassCourseLiveRoom: 'TwoTeacherClassController/getTwoTeacherClassCourseLiveRoom.do',
  // 获取双师辅班课表数据
  getTwoTeacherClassController: 'TwoTeacherClassController/getTwoTeacherClassTwoCourseLiveRoom.do',

  // 小班续报列表
  findMiniClassRenewPage: 'MiniClassRenewController/findMiniClassRenewPage.do',
  // 学科负责人查询
  findSubjectLeader: 'MiniClassRenewController/findSubjectLeader.do',
  // 删除学科负责人
  deleteSubjectLeaderApi: 'MiniClassRenewController/deleteSubjectLeader.do',
  // 修改学科负责人
  modifySubjectLeaderApi: {
    url: 'MiniClassRenewController/modifySubjectLeader.do',
    method: 'POST',
    dataType: 'json',
  },
  // 导出 excel
  exportMiniClassRenewExcel: 'MiniClassRenewController/exportMiniClassRenewExcel.do',
  // 小班续报整体情况
  findMiniClassRenewRate: 'MiniClassRenewController/findMiniClassRenewRate.do',

  // 小班预退班
  prepareDropMiniClass: 'SmallClassController/prepareDropMiniClass.do',
  // 取消预退班查询课程信息
  getMiniClassCourseForPrepareDrop: 'SmallClassController/getMiniClassCourseForPrepareDrop.do',
  // 小班取消预退班
  cancelPrepareDropMiniClass: {
    url: 'SmallClassController/cancelPrepareDropMiniClass.do',
    method: 'POST',
    dataType: 'json',
  },
  // 小班取消退班审批人列表
  cancelMiniClasssApprovalUsers: {
    url: 'SmallClassController/listNodeAuditUsersForCancelPrepareDrop.do',
    method: 'POST',
    dataType: 'json',
  },
  // 小班预退获取预退理由
  getMiniClassPrepareDropReason: 'SmallClassController/getMiniClassPrepareDropReason.do',
  // 小班预退回班审批列表
  miniClassBackApprovalList: {
    url: 'boss-xinghuo-api/workflow/pagePrepareDropFlowRecord',
    method: 'POST',
    dataType: 'json',
  },
  // 小班预退班回班终止审批
  minClassStopAudit: 'SmallClassController/abortPreDropFlow.do',
  // 小班预退班回班申请明细
  minClassBackApprovalDetail: 'boss-xinghuo-api/workflow/getPrepareDropFlowRecord',
  // 小班预退班回班申请流程列表
  miniClassAprrovalList: 'workflow/listWorkflowDynamic.do',
  // 获取小班信息
  findMiniClassById: 'CourseController/findMiniClassById.do',
  // 批量新增小班课程
  batchAddMiniClassCourse: {
    url: 'CourseController/batchAddMiniClassCourse.do',
    method: 'POST',
    dataType: 'json',
  },
  // 获取课程课时信息
  getMiniClassCourseTimeAnalyze: 'CourseController/getMiniClassCourseTimeAnalyze.do',
  // 获取小班失败记录
  getSmallClassChargeRecord: {
    url: 'boss-xinghuo-api/course/miniClassCourse/pageClassCourseChargeFailRecord',
    method: 'POST',
    dataType: 'json',
  },
  // 重置考勤
  confirmResetAttendance: 'boss-xinghuo-api/course/miniClassCourse/replaceAttendantStatus',

  // 导入关联记录列表
  getImportLinkList: 'OriginalClassController/listImportRecord.do',

  // 导出未匹配关联列表
  exportUnMatchList: 'OriginalClassController/exportImportRecord.do',

  // 获取小班详情学生
  getMiniClassDetailStudentList: 'CourseController/getMiniClassDetailStudentList.do',
  // 根据上课学习中心id列表获取对应学生id列表
  getStudentIdByCampusIdsParam: {
    url: 'CourseConflictController/getStudentIdByCampusIdsParam.do',
    method: 'POST',
    dataType: 'json',
  },
  // 根据上课学习中心id列表获取对应老师id列表
  getTeacherIdByCampusIdsParam: {
    url: 'CourseConflictController/getTeacherIdByCampusIdsParam.do',
    method: 'POST',
    dataType: 'json',
  },
  // 根据上课学习中心id列表获取对应教室id列表
  getClassroomIdByCampusIdsParam: {
    url: 'CourseConflictController/getClassroomIdByCampusIdsParam.do',
    method: 'POST',
    dataType: 'json',
  },
  // 获取冲突名单
  getMiniClassTimeTableConflictName: {
    url: 'CourseConflictController/getMiniClassTimeTableConflictName.do',
    method: 'POST',
    dataType: 'json',
  },
  // 获取课程信息
  getMiniClassTimeTableInfo: {
    url: 'CourseConflictController/getMiniClassTimeTableInfo.do',
    method: 'POST',
    dataType: 'json',
  },

  // 小班排课考勤提醒列表【课次】（分页）
  miniClassRemindByAttendanceList: 'MiniClassController/remindByAttendanceList.do',
  // 小班排课考勤提醒列表【科次】导出
  miniClassRemindByAttendanceListExport: 'ExportCenterController/exportRemindByAttendanceListToExcel.do',
  // 小班排课考勤提醒列表【课程】（分页)
  miniClassRemindByCourseList: 'MiniClassController/remindByCourseList.do',
  // 小班排课考勤提醒列表【课程】导出
  miniClassRemindByCourseListExport: 'ExportCenterController/exportRemindByCourseListToExcel.do',
  // 【1对1】排课考勤提醒
  remindByCourseList: 'CourseController/remindByCourseList.do',
  // 【1对多】排课考勤提醒【课程】
  otmRemindByCourseList: 'OtmCourseController/remindByCourseList.do',
  // 【1对多】排课考勤提醒【课次】
  otmRemindByAttendanceList: 'OtmCourseController/remindByAttendanceList.do',
  // 双师批量转班多个分公司权限获取学习中心
  getBranchAndChildCampusForClassChange: {
    url: 'ClassChangeController/getBranchAndChildCampusForClassChange.do',
    method: 'POST',
    dataType: 'json',
  },

  /**
   * 续报关联管理(班续、人续)
   * */
  // 平移班续记录-列表
  queryTranslateRecordList: 'ClassRenewController/queryTranslateRecordList.do',
  // 平移班续记录-刷新
  getTranslateRecord: 'ClassRenewController/getTranslateRecord.do',
  // 平移班续记录-撤回
  submitUndoTranslateRecord: {
    url: 'ClassRenewController/sumitUndoTranslateRecord.do',
    method: 'POST',
    dataType: 'json',
  },
  // 平移班续记录-详情
  queryTranslateRecordDetailList: 'ClassRenewController/queryTranslateRecordDetailList.do',
  // 平移班续记录-详情-撤回
  undoTranslateRecordDetail: {
    url: 'ClassRenewController/undoTranslateRecordDetail.do',
    method: 'POST',
    dataType: 'json',
  },
  // 平移班续记录-详情-导出
  exportTranslateRecordDetail: 'ClassRenewController/exportTranslateRecordDetail.do',
  // classin同步失败记录下载
  exportSyncFailedRecordExcel: {
    url: 'EeoSyncRecordController/exportSyncFailedRecordExcel.do',
    method: 'POST',
    dataType: 'json',
  },
  // classin 最近同步时间获取
  getLastUpdateTime: 'EeoSyncRecordController/getLastUpdateTime.do',
  // 查询转班审批人列表
  listNodeAuditUsersForChangeClass: {
    url: 'ClassChangeController/listNodeAuditUsersForChangeClass.do',
    method: 'POST',
    dataType: 'json',
  },
  // 检查是否可以正常转班，以及是否需要走OA流程
  checkBeforeStudentChangeClass: {
    url: 'ClassChangeController/checkBeforeStudentChangeClass.do',
    method: 'POST',
    dataType: 'json',
  },
  // 提交转班审批
  changeClassWithFlow: { url: 'ClassChangeController/changeClassWithFlow.do', method: 'POST', dataType: 'json' },
  // 检查报班是否需要走OA流程
  checkBeforeStudentSignClass: {
    url: 'ClassChangeController/checkBeforeStudentSignClass.do',
    method: 'POST',
    dataType: 'json',
  },
  // 查询报班审批人列表
  listNodeAuditUsersForSignClass: {
    url: 'ClassChangeController/listNodeAuditUsersForSignClass.do',
    method: 'POST',
    dataType: 'json',
  },
  // 提交报班审批
  signClassWithFlow: { url: 'ClassChangeController/signClassWithFlow.do', method: 'POST', dataType: 'json' },
  // 查询审批记录列表
  listChangerClassFlowRecords: { url: 'ClassChangeController/listChangerClassFlowRecords.do', method: 'POST' },
  // 查询审批记录详情
  findChangerClassFlowRecordById: { url: 'ClassChangeController/findChangerClassFlowRecordById.do', method: 'GET' },
  // 中止转班流程
  stopChangerClassByFlowId: { url: 'ClassChangeController/stopChangerClassByFlowId.do', method: 'GET' },
  // 重新发送短信提醒家长转班
  sendMessageForChangerClass: { url: 'ClassChangeController/sendMessageForChangerClass.do', method: 'GET' },
  // 重新转入 审批通过后自动转班失败，手动重新转入
  doChangerClassByFlowId: { url: 'ClassChangeController/doChangerClassByFlowId.do', method: 'GET' },
  // 流程动态查询
  listWorkflowDynamic: { url: 'workflow/listWorkflowDynamic.do', method: 'GET' },
  /**
   * @desc 小班上下架
   */
  batchUpOnShelves: {
    url: 'boss-xinghuo-api/miniclass/batchUpOnShelves',
    method: 'POST',
    dataType: 'json',
  },
  batchDownOnShelves: {
    url: 'boss-xinghuo-api/miniclass/batchDownOnShelves',
    method: 'POST',
    dataType: 'json',
  },
  validateWhenDownOnShelves: {
    url: 'boss-xinghuo-api/miniclass/restrictMessageOnShelves',
    method: 'POST',
    dataType: 'json',
  },
  // 获取小班季度可教老师列表
  smallClassQuarterAavaliableTeacherList: {
    url: 'boss-xinghuo-api/miniClass/teacherSubjectLimitRule/pageTeacher',
    method: 'POST',
    dataType: 'json',
  },
  // 批量更新小班课程
  updateMiniClassCourseByCourseIds: {
    url: 'CourseController/updateMiniClassCourseByCourseIds.do',
    method: 'POST',
    dataType: 'json',
  },
  // 批量修改小班课节（用户通过手工来决定是否要同步小班修改信息）
  updateMiniClassCourseByClassIds: {
    url: 'MiniClassController/updateMiniClassCourseByClassIds.do',
    method: 'POST',
    dataType: 'json',
  },
  /**
   * http://47.106.118.192:13000/project/27/interface/api/56834
   * 检测时间范围内，指定组织范围内，老师、教室、学生 课节冲突名单
   */
  checkMiniClassCourseConflict: {
    url: 'boss-xinghuo-api/course/courseConflict/checkMiniClassCourseConflict',
    method: 'POST',
    dataType: 'json',
  },
  /**
   * http://47.106.118.192:13000/project/27/interface/api/56858
   * 检测时间范围内，学生课节情况
   */
  checkStudentCourseConflict: {
    url: 'boss-xinghuo-api/course/courseConflict/checkStudentCourseConflict',
    method: 'POST',
    dataType: 'json',
  },

  /**
   * https://xiaojiaoyukeji.yuque.com/newop/hk7g9o/gb5dcc
   * 导出教室冲突小班课节名单
   */
  fetchToexportClassroom: {
    url: 'boss-xinghuo-api/course/courseConflict/listClassroomConflictMiniClassCourse?isExport=pull',
    method: 'POST',
    dataType: 'json',
  },

  /**
   * https://xiaojiaoyukeji.yuque.com/newop/hk7g9o/gb5dcc
   * 导出老师冲突小班课节名单
   */
  fetchToexportTeacher: {
    url: 'boss-xinghuo-api/course/courseConflict/listTeacherConflictMiniClassCourse?isExport=pull',
    method: 'POST',
    dataType: 'json',
  },

  /**
   * https://xiaojiaoyukeji.yuque.com/newop/hk7g9o/gb5dcc
   * 导出学生冲突小班课节名单
   */
  fetchToexportStudent: {
    url: 'boss-xinghuo-api/course/courseConflict/listStudentConflictMiniClassCourse?isExport=pull',
    method: 'POST',
    dataType: 'json',
  },

  /** 根据合同产品ID查询课节信息
   * https://xiaojiaoyukeji.yuque.com/xh-zh/iy6qk9/vuvm7g
   */
  listMiniClassStudentCourseByCpId: {
    url: 'boss-xinghuo-api/course/miniClassCourse/listMiniClassStudentCourseByCpId',
    method: 'GET',
  },

  // 根据合同产品ID查询课节信息
  listMCStudentCourseByCpId: {
    url: 'boss-xinghuo-api/course/miniClassCourse/listMCStudentCourseByCpId',
    method: 'GET',
  },

  // 小班退课
  dropMiniClassStudentAttendent: {
    url: 'boss-xinghuo-api/course/miniClassCourse/dropMiniClassStudentAttendent',
    method: 'POST',
    dataType: 'json',
  },

  /* 获取小班学生剩余金额和课时 */
  getMiniClassStuRemainFinAndHour: {
    url: 'CourseController/getMiniClassStuRemainFinAndHour.do',
    method: 'GET',
  },

  /* 获取小班学生考勤 */
  getMiniClassAttendedStudent: {
    url: 'CourseController/getMiniClassAttendedStudent.do',
    method: 'GET',
  },

  /* 小班退班 */
  deleteStudentInMiniClasss: {
    url: 'CourseController/deleteStudentInMiniClasss.do',
    method: 'GET',
  },

  /* 新增查找学生接口 */
  searchStudent: {
    url: 'CourseController/v1/searchStudent.do',
    method: 'GET',
  },
};

const xhBossApiConfig = {
  /**
   * BI 排班
   */
  getBICourseConflictData: {
    url: '/eduboss/boss-xinghuo-api/course/courseConflict/getCourseConflict',
    method: 'GET',
  },
  /**
   * http://47.106.118.192:13000/mock/27/student/batchUpdateStudent
   * 批量修在读学校
   */
  batchUpdateStudent: {
    url: '/eduboss/boss-xinghuo-api/student/batchUpdateStudent',
    method: 'POST',
    dataType: 'json',
  },
};

const config = {
  ...buildApi(eduBossConfig),
  ...buildStrictFetchApi(xhBossApiConfig),
};

export default config;
