import buildApi, { buildStrictFetchApi } from './buildApi';

const eduBossConfig = {
  // 获取客户(学生)详情
  findCustomerById: 'CustomerAction/findCustomerById.do',
  getCitys: 'SystemAction/getCitys.do',
  getStudentListByCusContact: 'CustomerAction/getStudentListByCusContact.do',
  saveOrUpdateCustomer: {
    url: 'CustomerAction/v2/saveOrUpdateCustomer.do',
    method: 'POST',
    dataType: 'json',
  },
  getRelatedStudentsByCusId: 'CustomerAction/getRelatedStudentsByCusId.do',
  saveFollowRecord: {
    url: 'CustomerAction/saveFollowRecord.do',
    method: 'POST',
  },
  followupCustomerStatistics: 'CustomerAction/v2/followupCustomerStatistics.do',
  saveTransferCustomer: {
    url: 'CustomerAction/v2/saveTransferCustomer.do',
    method: 'POST',
    dataType: 'json',
  },
  followRateReport: 'CustomerAction/v2/followRateReport.do',
  followRateReportNewDetails: 'CustomerAction/v2/followRateReportNewDetails.do',
  // 总体0跟进学生数详情
  followRateReportUnFollowDetails: 'CustomerAction/v2/followRateReportUnFollowDetails.do',
  /* -------- 无效资源池 ---------- */
  // 查询无效资源列表
  queryInvalidResource: { url: 'CustomerAction/queryInvalidResource.do', dataType: 'json' },
  // 彻底删除
  deleteInvalidResource: { url: 'CustomerAction/deleteInvalidResource.do', method: 'POST', dataType: 'json' },
  // 恢复无效资源
  recoverResource: { url: 'CustomerAction/recoverResource.do', method: 'POST', dataType: 'json' },
  // 查询可选组织
  getOptionalOrganization: { url: 'OrganizationController/getOptionalOrganization.do', dataType: 'json' },
  // 客户资源调配
  deliverCustomer: 'CustomerAction/deliverCustomer.do',
  // 查找用户(用于转介绍奖金归属人，带组织架构)
  searchUser: 'UserController/searchUser.do',
  // 获取指定学习中心下的资源标签列表
  getResourcePoolTags: 'CustomerAction/getResourcePoolTags.do',
  // 获取客户手机号码修改记录
  getContactChangeLogByCustomerId: 'CustomerAction/findContactChangeLogByCustomerId.do',
  // 发送修改客户手机号确认短信
  sendConfirmMessage: { url: 'CustomerAction/sendConfirmMessage.do', method: 'POST', dataType: 'json' },
  // 重新发送修改客户手机号确认短信
  resendConfirmMessage: { url: 'CustomerAction/resendConfirmMessage.do', method: 'POST', dataType: 'json' },
  // 模糊查询客户
  getAutoCompleteByCustomer: 'CustomerAction/getAutoCompleteByCustomer.do',
  // 保存修改客户手机号记录
  saveCustomerContactChangeLog: {
    url: 'CustomerAction/saveCustomerContactChangeLog.do',
    method: 'POST',
    dataType: 'json',
  },
  // 发送客户手机号修改H5确认短信
  sendCustomerContactChangeH5ConfirmSms: {
    url: 'CustomerAction/sendCustomerContactChangeH5ConfirmSms.do',
    method: 'POST',
    dataType: 'json',
  },
};

const xhApiUriPrefix = '/eduboss/boss-xinghuo-api';

const xhApiConfig = {
  searchUserByRoleCode: {
    url: `${xhApiUriPrefix}/user/listByQueryNameAndRoleCode`,
    method: 'GET',
  },
  getCustInfoByPhone: {
    url: `${xhApiUriPrefix}/crm/customer/getByPhone`,
    method: 'GET',
  },
  // 检验新手机号是否在系统在已经存在(接口返回形式不适用buildApi)
  checkContactExist: {
    url: '/eduboss/CustomerAction/checkContactExist.do',
    method: 'GET',
  },
  // 无效资源恢复
  resourceRecordRecover: {
    url: `${xhApiUriPrefix}/crm/resourceRecord/invalid/recover`,
    method: 'POST',
    dataType: 'json',
  },
  // 批量彻底删除无效记录
  resourceRecordDelete: {
    url: `${xhApiUriPrefix}/crm/resourceRecord/invalid/delete`,
    method: 'POST',
    dataType: 'json',
  },

  // 获取资源无效记录分页列表 /crm/resourceRecord/invalid/page
  resourceRecordList: { url: `${xhApiUriPrefix}/crm/resourceRecord/invalid/page`, method: 'POST', dataType: 'json' },

  // 老生激活登记-根据资源和开关判断是否打开
  isOpenOnSwitchAndResource: {
    url: `${xhApiUriPrefix}/organization/isOpenOnSwitchAndResource`,
    method: 'GET',
  },
  //老生转介绍我的提交记录
  pageMyTransfer: {
    url: `${xhApiUriPrefix}/crm/transfer/pageMyTransfer`,
    method: 'POST',
    dataType: 'json',
  },
  //老生转介绍审核记录
  pageTransfer: {
    url: `${xhApiUriPrefix}/crm/transfer/pageTransfer`,
    method: 'POST',
    dataType: 'json',
  },
  // 提交审核
  saveSignStudentTransferCustomer: {
    url: `/eduboss/CustomerAction/v2/saveSignStudentTransferCustomer.do`,
    method: 'POST',
    dataType: 'json',
  },
  // 登记详情
  loadTransferCustomerResult: {
    url: `/eduboss/CustomerAction/loadTransferCustomerResult.do`,
    method: 'GET',
    dataType: 'json',
  },
  auditTransferCustomer: {
    url: '/eduboss/CustomerAction/v2/auditTransferCustomer.do',
    method: 'POST',
  },
  // 审核记录
  getAuditRecordById: {
    url: `/eduboss/CustomerAction/getAuditRecordNewById.do`,
    method: 'GET',
  },
  // 获取审核咨询师选项
  getTransferTargetByCampus: {
    url: `/eduboss/CustomerAction/getTransferTargetByCampus.do`,
    method: 'GET',
  },
  //---小学资源登记 start ---
  registerPrimarySchoolResource: {
    url: `${xhApiUriPrefix}/crm/primary-school-resource/register`,
    method: 'POST',
    dataType: 'json',
  },
  pageAuditPrimarySchoolResources: {
    url: `${xhApiUriPrefix}/crm/primary-school-resource/page/audit`,
    method: 'POST',
    dataType: 'json',
  },
  exportAuditPrimarySchoolResources: {
    url: `${xhApiUriPrefix}/crm/primary-school-resource/page/audit?isExport=pull`,
    method: 'POST',
    dataType: 'json',
  },
  auditPrimarySchoolResource: {
    url: `${xhApiUriPrefix}/crm/primary-school-resource/audit`,
    method: 'POST',
    dataType: 'json',
  },
  assignPrimarySchoolResource: {
    url: `${xhApiUriPrefix}/crm/primary-school-resource/assign`,
    method: 'POST',
    dataType: 'json',
  },
  pageMinePrimarySchoolResources: {
    url: `${xhApiUriPrefix}/crm/primary-school-resource/page/mine`,
    method: 'POST',
    dataType: 'json',
  },
  //---小学资源登记 end ---
};

const config = {
  ...buildApi(eduBossConfig),
  ...buildStrictFetchApi(xhApiConfig),
};

export default config;
