import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import ITserviceSvg from 'ASSET/imgs/ITservice.svg';

/**
 * 加载七鱼客服SDK
 *
 * @param config {Object}
 * @return {Promise<any>}
 */
function loadQiYuScript(config) {
  return new Promise((resolve, reject) => {
    try {
      let script = document.getElementById('qiyukf');

      if (!script) {
        downloadQiYuScript(window, document, 'ysf');

        ysf('onready', function () {
          ysf('logoff');

          resetQiYuSetting();

          return configQiYu(config);
        });
      } else {
        return resolve();
      }
    } catch (e) {
      reject('加载客服插件失败');
    }
  });
}

/**
 * 加载qiyukefu.js
 *
 * @param w
 * @param d
 * @param n
 * @param a
 * @param j
 */
function downloadQiYuScript(w, d, n, a, j) {
  w[n] =
    w[n] ||
    function () {
      return (w[n].a = w[n].a || []).push(arguments);
    };

  j = d.createElement('script');
  j.setAttribute('id', 'qiyukf');
  j.async = true;

  // 测试用，2021/3/25前有效
  j.src = 'https://qiyukf.com/script/6359160d4d1bb72b6fb233a3df062920.js?hidden=1';

  if (process.env.BUILD_ENV === 'PRD') {
    j.src = 'https://qiyukf.com/script/8ce367faf9cc78d97fac7a47cb4015eb.js?hidden=1';
  }

  d.body.appendChild(j);
}

/**
 * 修改IT客服样式
 */
function resetQiYuSetting() {
  const styleElm = document.createElement('style');
  styleElm.innerText = `#YSF-BTN-HOLDER:hover {
    filter: alpha(opacity=95);
    opacity: 1;
}
#YSF-BTN-HOLDER {
    opacity: .8;
    z-index: 1000;
}
`;
  document.head.appendChild(styleElm);

  const qiyuElm = document.getElementById('YSF-BTN-HOLDER');
  const entryElm = qiyuElm.querySelector('div:first-child');
  const circleBtnElm = qiyuElm.querySelector('#YSF-BTN-CIRCLE');
  const bubbleBtnElm = qiyuElm.querySelector('#YSF-BTN-BUBBLE');
  qiyuElm.setAttribute('style', 'right: 0;bottom: 89px;max-width: 34px;');
  entryElm.setAttribute(
    'style',
    `
background: #457FE2;
box-shadow: -1px 0px 8px 0px rgba(40,72,128,0.36);
border-radius: 5px 0 0 5px;
`,
  );
  entryElm.innerHTML = `<div style="display: flex;
flex-direction: column;
justify-content: center;
align-items: center;
height: 106px;
width: 34px;
font-size: 13px;
font-weight: bold;
color: #fff;
">
    <img src="${ITserviceSvg}" style="width: 18px; height: 18px; margin-bottom: 6px;" />
    <span style="display: block;width: 14px;text-align: center;line-height: 18px;">IT客服</span>
</div>`;
  circleBtnElm.setAttribute('style', 'right: 0px;top: -4px;');
  bubbleBtnElm.setAttribute('style', 'left: -280px;');
}

/**
 * 配置七鱼客服信息
 *
 * @param config
 * @return {Promise<any>}
 */
function configQiYu(config) {
  return new Promise((resolve, reject) => {
    let stringData = '';

    try {
      if (!config.data) {
        throw new Error();
      }

      if (!Array.isArray(config.data) || config.data.length === 0) {
        throw new Error();
      }

      stringData = JSON.stringify(config.data);

      if (!stringData.trim()) {
        throw new Error();
      }
    } catch (e) {
      return reject('客服插件配置参数异常,请刷新网页');
    }

    ysf('config', {
      uid: config.uid,
      data: stringData,
      success: function () {
        resolve(true);
      },
      error: function () {
        reject('客服插件配置参数异常,请刷新网页');
      },
    });
  });
}

function setQiYuVisibility(visible) {
  const qiyuElm = document.getElementById('YSF-BTN-HOLDER');

  if (qiyuElm && visible) {
    qiyuElm.style.display = 'unset';
  }
}

/**
 * 初始化七鱼客服
 *
 * @param config
 * @return {Promise<any|boolean>}
 */
export async function initQiYu(config) {
  let isQiyuReady = false;

  try {
    await loadQiYuScript(config);
    isQiyuReady = true;
  } catch (e) {
    showMessage(e.message, 'error');
    isQiyuReady = false;
  }

  setQiYuVisibility(isQiyuReady);

  return isQiyuReady;
}

/**
 * 退出七鱼客服
 */
export function exitQiYu() {
  try {
    if (ysf) {
      ysf('logoff');
    }
  } catch (e) {
    //
  }
}
