import buildApi from './buildApi';

/**
 * Api配置信息，key为Api唯一标识，value为object-->{ url, method }，或者string-->url（method默认为GET）
 */
const config = {
  findStudentById: 'StudentController/findStudentById.do', //根据ID获取学生信息
  editStudent: 'StudentController/editStudent.do', //保存学生信息
  getStudentList: 'StudentController/getStudentList.do', //获取学生列表
  checkUpdateStudentManage: 'StudentController/checkUpdateStudentManage.do', //检查学管师状态
  updateStudentStudyManager: 'StudentController/updateStudentStudyManager.do', //保存学管师信息
  getStudentAddressAndLatLog: 'StudentController/getStudentAddressAndLatLog.do', //获取学生地域数据

  findPageAchievementTemplate: 'AchievementController/findPageAchievementTemplate.do', //分页查询成绩模板
  listAllTemplateGradeByTemplateId: 'AchievementController/listAllTemplateGradeByTemplateId.do', //根据模板编号查询所有成绩模板年级
  findPageAchievementTemplateByCityId: 'AchievementController/findPageAchievementTemplateByCityId.do', //根据城市编号查询成绩模板Vo列表
  saveFullAchievementTemplate: {
    url: 'AchievementController/saveFullAchievementTemplate.do',
    method: 'POST',
    dataType: 'json',
  }, // 新增或修改成绩模板

  listAllAchievementCategory: 'AchievementController/listAllAchievementCategory.do', //获取成绩类别列表
  updateAchievementCategory: {
    url: 'AchievementController/updateAchievementCategory.do',
    method: 'POST',
    dataType: 'json',
  }, // 修改成绩类别

  listAllAchievementProStandard: 'AchievementController/listAllAchievementProStandard.do', //获取成绩进步标准列表
  updateAchievementProStandard: {
    url: 'AchievementController/updateAchievementProStandard.do',
    method: 'POST',
    dataType: 'json',
  }, // 修改成绩类别

  listTemplateCityIdsByProvinceId: 'AchievementController/listTemplateCityIdsByProvinceId.do', //根据省份编号获取已排成绩模板的城市编号列表
  initAchievementTemplateByCitys: 'AchievementController/initAchievementTemplateByCitys.do', //根据城市编号初始化成绩模板

  deleteAchievementTemplateById: 'AchievementController/deleteAchievementTemplateById.do', //根据模板编号删除模板
  listTemplateEffectiveDateByCityId: 'AchievementController/listTemplateEffectiveDateByCityId.do', //根据城市获取已有的模板日期

  getStudentFollowUpsByStudentId: 'StudentController/getStudentFollowUpsByStudentId.do', //学生的跟进回访列表
  saveFollowUp: { url: 'StudentController/saveFollowUp.do', method: 'POST', dataType: 'json' }, // 保存学生跟进记录
  deleteFollowUpById: { url: 'StudentController/deleteFollowUpById.do', method: 'POST' }, // 删除跟进记录
  getDataFromReport: 'StudentController/getStudentFollowUpsByIds.do', //从报表 跳转加载数据

  getSubjectById: 'StudentController/getStudentSubjectByStudentId.do', //获取一对一科目
  getMiniClassById: 'StudentController/getStudentMiniClassByStudentId.do', //获取班课名称
  getStudyManager: 'StudentController/getStudyManagerByStudentId.do', //获取学管师班主任
  getTeacher: 'StudentController/getTeacherByStudentId.do', //获取老师
  getHomeroomTeacherByStudentId: 'StudentController/getHomeroomTeacherByStudentId.do', //获取老师

  //学生档案接口
  saveStudentFile: { url: 'StudentDocumentController/saveStudentArchives.do', method: 'POST', dataType: 'json' }, //保存学生档案
  deleteStudentFile: 'StudentDocumentController/deleteStudentFileById.do', //删除学生档案
  getFileList: 'StudentDocumentController/getStudentFile.do', //获取学生档案列表
  showPreview: 'StudentController/previewFile.do', //预览文件
  getFileDataFromReport: 'StudentDocumentController/getStudentFileByIdsStuId.do', //从报表 跳转加载数据
  //试卷管理
  getPaperList: 'PaperController/getPaperList.do', //查询试卷列表
  upatePaperStatus: 'PaperController/upatePaperStatus.do', //更新试卷状态
  savePaper: { url: 'PaperController/savePaper.do', method: 'POST', dataType: 'json' }, // 保存试卷

  // 批量激活客户
  batchActivation: { url: 'student/batchActivation.do', method: 'POST', dataType: 'json' },
  // 查询批量激活结果
  batchActivationResult: 'student/batchActivation/result.do',
  // 导出批量激活失败数据
  batchActivationResultExport: 'student/batchActivation/result/export.do',

  // 课外辅导成绩导入记录查询
  listImportRecord: 'AchievementMgrController/listImportRecord.do',
  // 课外辅导成绩导出错误数据
  exportFailingData: 'ExportCenterController/exportFailingData.do',
  // 新增学生关联家长
  addCustomerStudent: { url: 'StudentController/addCustomerStudent.do', method: 'POST', dataType: 'json' },
  // 获取学生已关联家长
  getCustomerStudentList: 'StudentController/getCustomerStudentList.do',
  // 删除学生已关联家长
  deleteCustomerStudent: 'StudentController/deleteCustomerStudent.do',
  // 保存学生关联家长关系内容
  setRelation: {
    url: 'StudentController/setRelation.do',
    method: 'POST',
    dataType: 'json',
  },
  // 保存学生关联家长为默认家长
  setConfirmCustomer: {
    url: 'StudentController/setConfirmCustomer.do',
    method: 'POST',
    dataType: 'json',
  },
  // 获取学生关联家长确认记录
  getRelationConfirmList: 'StudentController/getRelationConfirmList.do',
  // 获取学生关联家长操作记录
  getRelationOperationList: 'StudentController/getRelationOperationList.do',
  // 还原已删除得学生关联家长
  restoreCustomerStudent: 'StudentController/restoreCustomerStudent.do',
  // 发送关联家长确认短信
  sendRelationConfirmSms: 'StudentController/sendRelationConfirmSms.do',
  // 修改学生姓名
  onlyRenameStudent: {
    url: 'StudentController/onlyRenameStudent.do',
    method: 'POST',
    dataType: 'json',
  },
  // 咨询师学生列表
  consultantStudentList: {
    url: 'boss-xinghuo-api/student/pageForSale',
    method: 'POST',
    dataType: 'json',
  },
  //----------------学生学校信息不匹配相关接口 start---------------------
  //查询学校年级不匹配列表接口【年级匹配】
  getGradeMatchingList: 'StudentController/gradeMatchingList.do',
  // 根据学生ID查找在读学校集合
  getSchoolByStudentId: 'StudentController/getSchoolByStudentId.do',
  // 更新在读学校
  updateSchoolList: {
    url: 'StudentController/updateSchoolList.do',
    method: 'POST',
    dataType: 'json',
  },
  // 取消家长确认
  cancelCustomerStudentRelation: 'StudentController/cancelCustomerStudentRelationOp.do',
  // 根据学生ID获取学生意向信息
  listStudentIntentionInfo: 'StudentIntentionController/listStudentIntentionInfo.do',
  // 保存学生意向信息
  saveOrUpdateStudentIntentionInfo: {
    url: 'StudentIntentionController/saveOrUpdateStudentIntentionInfo.do',
    method: 'POST',
    dataType: 'json',
  },

  //----------------学生学校信息不匹配相关接口 end-----------------------
  // 查询学生标签接口
  getStudentLabel: 'boss-xinghuo-api/studentLabel/getStudentLabel',
  // 查询所有标签接口
  getAllLabel: 'boss-xinghuo-api/studentLabel/getAllLabel',
  // 保存学生标签接口
  saveStudentLabel: {
    url: 'boss-xinghuo-api/studentLabel/saveStudentLabel',
    method: 'POST',
    dataType: 'json',
  },
  //  分页查询学生标签操作日志接口
  pageStudentLabelOperationLogList: {
    url: 'boss-xinghuo-api/studentLabel/pageStudentLabelOperationLogList',
    method: 'POST',
    dataType: 'json',
  },
};

export default buildApi(config);
