import { request } from './fetch';

const charset = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
const randomString = (length, charset) => {
  let result = '';

  for (let i = length; i > 0; --i) {
    result += charset[Math.floor(Math.random() * charset.length)];
  }

  return result;
};

export class AliLogTracker {
  constructor(option) {
    const { projectName, endpoint, logstoreName, ...restOpts } = option;

    this.trackingUrl = `//${projectName}.${endpoint}/logstores/${logstoreName}/track`;
    this.restLog = restOpts;
  }

  /**
   * 创建 trace id
   * @param {string} prefix
   */
  static createTraceId(prefix) {
    const randomId = randomString(16, charset);

    return prefix ? prefix + randomId : randomId;
  }

  /**
   * 上报日志
   * @param {Object} record
   */
  async log(record) {
    const { topic, ...validRecord } = Object.keys(record).reduce((prev, key) => {
      if (record[key]) {
        prev[key] = String(record[key]);
      }

      return prev;
    }, {});

    const log = {
      ...validRecord,
      ...this.restLog,
      location_path: window.location.pathname,
      location_search: window.location.search,
      location_hash: window.location.hash,
      user_agent: window.navigator.userAgent,
    };

    const reqBody = {
      __topic__: topic || void 0,
      __logs__: [log],
    };

    return await request(this.trackingUrl, {
      method: 'POST',
      headers: {
        'Content-Type': 'application/json;charset=utf-8',
        'x-log-apiversion': '0.6.0',
        'x-log-bodyrawsize': '1234',
      },
      body: JSON.stringify(reqBody),
    });
  }
}
