import {
  peiyouInstitution,
  twoTeacherInstitution,
  xinghuoInstitution,
  IS_SHOW_START_DATE_SWITCH,
} from '../constants/config';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { clone } from 'lodash';
import NP from 'number-precision';
import { isIdCard as idCardValid } from 'UTIL/idCardValidation';

// 插件管理
export const pluginManage = {
  hadLoadStorage: {},
  load(url) {
    return new Promise((resolve) => {
      if (!this.hadLoadStorage[url]) {
        let script = document.createElement('script');
        script.type = 'text/javascript';
        script.onload = (data) => {
          this.hadLoadStorage[url] = true;
          resolve();
        };
        script.src = url;
        document.body.appendChild(script);
      } else {
        resolve();
      }
    });
  },
};

export function runFunc(fn, ...arg) {
  if (fn && typeof fn === 'function') {
    return fn(...arg);
  }
  return null;
}

/**
 * 将字符串数组拼接成字符串
 * @param arr 字符串数组 ['www', 'eee']
 * @param splitStr 分隔符
 */
export function transformArrayToString(arr = [], splitStr = ',') {
  if (!arr || !Array.isArray(arr) || arr.length <= 0) {
    return null;
  }

  const trueArr = arr.filter((value) => {
    return !!value;
  });

  return trueArr.join(splitStr) || null;
}

export function errorsHandle(handleErr, errKeyName) {
  return function (err) {
    showMessage(errKeyName ? err[errKeyName] : err.text, 'error');
    runFunc(handleErr, err._data_ || err);
  };
}

/**
 * 获取列表请求(page === 1时获取总条数及数据，其他时只获取数据；通过`getTotal`参数区分)
 *
 * @param api {string} 接口
 * @param params {object} 参数
 * @param pageCurKey {string} 页数的key值
 * */
export function getFetchList(api, params, pageCurKey) {
  let list = [
    api(
      {
        ...params,
        getTotal: false,
      },
      true,
    ),
  ];
  if (params[pageCurKey] === 1) {
    list.push(
      api(
        {
          ...clone(params),
          getTotal: true,
        },
        true,
      ),
    );
  }
  return list;
}

// 登录用户相关判断方法
export const loginUser = {
  userData: null,
  isXinghuo: null,
  isTwoTeacher: null,
  isPeiYou: null,
  isGroup: null,
  isBranch: null,
  isCampus: null,
  initLoginUser() {
    if (!this.userData && global.Index) {
      this.userData = Index.getLoginUser();
    }
  },
  isXingHuoSym() {
    if (this.isXinghuo === null) {
      this.initLoginUser();
      if (this.userData) {
        this.isXinghuo = this.userData.institution === xinghuoInstitution;
      }
    }
    return this.isXinghuo;
  },
  isTwoTeacherSym() {
    if (this.isTwoTeacher === null) {
      this.initLoginUser();
      if (this.userData) {
        this.isTwoTeacher = this.userData.institution === twoTeacherInstitution;
      }
    }
    return this.isTwoTeacher;
  },
  isPeiYouSym() {
    if (this.isPeiYou === null) {
      this.initLoginUser();
      if (this.userData) {
        this.isPeiYou = this.userData.institution === peiyouInstitution;
      }
    }
    return this.isPeiYou;
  },
  isGroupRole() {
    if (this.isGroup === null) {
      this.initLoginUser();
      if (this.userData) {
        this.isGroup = this.userData.isBlGrounp && !this.userData.isBlBrench && !this.userData.isBlCampus;
      }
    }
    return this.isGroup;
  },
  isBranchRole() {
    if (this.isBranch === null) {
      this.initLoginUser();
      if (this.userData) {
        this.isBranch = this.userData.isBlBrench && !this.userData.isBlCampus;
      }
    }
    return this.isBranch;
  },
  isCampusRole() {
    if (this.isCampus === null) {
      this.initLoginUser();
      if (this.userData) {
        this.isCampus = this.userData.isBlCampus;
      }
    }
    return this.isCampus;
  },
  isExBoss() {
    if (!this.userData) {
      this.initLoginUser();
    }
    return this.userData ? this.userData.isExBoss : false;
  },
};

export function jsonParse(val, defaultVal) {
  let ret = defaultVal || null;
  if (val && typeof val === 'string') {
    try {
      ret = JSON.parse(val);
    } catch (e) {}
  }
  return ret;
}

export function getObjByFilter(originObj, filterFn) {
  let result = {};
  Object.keys(originObj)
    .filter(filterFn)
    .forEach((key) => {
      result[key] = originObj[key];
    });
  return result;
}

export function getKeyValArrByFilter(originObj, filterFn) {
  let result = [];
  Object.keys(originObj)
    .filter(filterFn)
    .forEach((key) => {
      result.push({
        key,
        value: originObj[key],
      });
    });
  return result;
}

export function isNumber(arg) {
  return typeof arg === 'number';
}

export function isInputNumber(arg) {
  const type = typeof arg;
  if (type === 'string') {
    return arg.lastIndexOf('.') > -1;
  }
  return type === 'number';
}

export function isNumberPlus(num) {
  var type = typeof num;
  if (type === 'number') {
    return true;
  } else if (type === 'string' && num !== '') {
    var cfg = {
      0: true,
      1: true,
      2: true,
      3: true,
      4: true,
      5: true,
      6: true,
      7: true,
      8: true,
      9: true,
      '.': true,
    };
    for (var i = 0; i < num.length; i++) {
      if (cfg[num[i]]) {
        if (num[i] === '.') {
          cfg['.'] = false;
        }
      } else {
        return false;
      }
    }
    return true;
  }
  return false;
}

export function deleteKeyByFilter(originObj, filterFn) {
  Object.keys(originObj)
    .filter(filterFn)
    .forEach((key) => {
      delete originObj[key];
    });
  return originObj;
}

// 十八位身份证验证
export function isIdCard(id) {
  return idCardValid(id);
}

/**
 *
 * @param {string} type  证件类型
 * @param {string} number 证件号码
 * @param {boolean} isRequire 是否必填
 */
export function validIdentityCardByType(type, number, isRequire = false) {
  const ID_CARD = 'CT_IDENTITY_CARD';

  // 如果都没值，返回是否必填返回true/false
  if (!type && !number) {
    return {
      type: 'noValue',
      valid: !isRequire,
    };
  }

  // 都有值，按类型进行校验
  if (type && number) {
    if (type === ID_CARD) {
      return {
        type: 'allValue',
        valid: isIdCard(number),
      };
    } else {
      return {
        type: 'allValue',
        valid: /^\S+$/.test(number),
      };
    }
  } else {
    // 只有某一个有值
    return {
      type: 'halfValue',
      valid: false,
    };
  }
}

/**
 * @param {*} arg
 * @return {string}
 * */
export function typeOf(arg) {
  return Object.prototype.toString.call(arg).slice(8, -1).toLowerCase();
}

/**
 * 隐藏部分手机号码(脱敏处理)
 *
 * @param number {string} 手机号码
 * @return {string|*}
 */
export function desensitizationPhoneNumber(number) {
  if (!number) {
    return '';
  }

  return number.replace(/(\d{3})(\d{4})(\d{4})/, '$1****$3');
}

// 强制resize
export function focusResize(delay = 0) {
  setTimeout(function () {
    window.dispatchEvent(new Event('resize'));
  }, delay);
}

export function numFormatToMoney(num) {
  if (!num) {
    return '0.00';
  } else if (typeof num !== 'number') {
    return num;
  }
  let ret = num.toFixed(2);
  ret = parseFloat(ret);
  ret = ret.toLocaleString();
  const lastIndex = ret.lastIndexOf('.');
  if (lastIndex === -1) {
    ret += '.00';
  } else if (ret.length - lastIndex === 2) {
    ret += '0';
  }
  return ret;
}

/**
 * 课时数转化为小时数
 * @param {number} courseNumber 课时数
 * @param {number} courseLength 课时时长
 * @returns number
 */
export function formatCourseToHour(courseNumber, courseLength) {
  if (!courseLength) return courseNumber;
  return NP.times(courseNumber, NP.divide(courseLength, 60)).toFixed(2);
}

/**
 * 课时数转化为小时数
 * @param {number} courseNumber 课时数
 * @param {number} courseLength 课时时长
 * @returns number
 */
export function formatCourseToHourNew(courseNumber, courseLength) {
  if (!courseLength) return courseNumber;
  const numbers = `${NP.times(courseNumber, NP.divide(courseLength, 60))}`.split(/\./);
  if (numbers.length === 0) {
    return NP.times(courseNumber, NP.divide(courseLength, 60));
  }
  if (numbers.length === 1) {
    return parseFloat(numbers[0]);
  }
  return parseFloat(`${numbers[0]}.${numbers[1].substring(0, 2)}`);
  //return NP.times(courseNumber, NP.divide(courseLength, 60)).toFixed(2);
}

/**
 * 小时转课时数
 * @param {number} hour
 * @param {number} courseLength
 * @returns number
 */
export function formatHourToCourse(hour, courseLength) {
  if (!courseLength) return hour;
  return NP.divide(NP.times(hour, 60), courseLength).toFixed(2);
}

/**
 * 通过课时单价计算小时单价
 * @param {number} price
 * @param {number} courseLength
 * @returns number
 */
export function computedHourPriceByCourse(price, courseLength) {
  if (!courseLength) return price;
  return NP.times(price, NP.divide(60, courseLength)).toFixed(2);
}

/**
 * 通过小时单价计算课时单价
 * @param {number} price
 * @param {number} courseLength
 * @returns number
 */
export function computedCoursePriceByHour(price, courseLength) {
  if (!courseLength) return price;
  return NP.times(NP.divide(price, 60), courseLength).toFixed(2);
}

/**
 * @description 通过小时+分钟以及课时时长换算出课时数
 * @param {number} hour
 * @param {number} minutes
 * @param {number} courseLength
 * @returns number
 */
export function transferHourAndMinutesToCourse(hour, minutes, courseLength) {
  const totalMinutes = NP.plus(NP.times(hour, 60), minutes);
  return NP.divide(totalMinutes, courseLength);
}

window.formatCourseToHour = formatCourseToHour;
window.formatCourseToHourNew = formatCourseToHourNew;
window.formatHourToCourse = formatHourToCourse;
window.computedHourPriceByCourse = computedHourPriceByCourse;
window.computedCoursePriceByHour = computedCoursePriceByHour;
window.transferHourAndMinutesToCourse = transferHourAndMinutesToCourse;
// 传递是否显示日期开关给旧boss
window.IS_SHOW_START_DATE_SWITCH = IS_SHOW_START_DATE_SWITCH;

export function getHeaders() {
  const xhr = new XMLHttpRequest();
  xhr.open('GET', `${location.origin}/eduboss/CommonAction/getCurrentDateTime.do`, false);
  xhr.send(null);
  const headers = xhr.getAllResponseHeaders().split('\r\n');
  return headers.reduce((accumulate, current) => {
    if (current !== '') {
      const index = current.indexOf(':');
      const key = current.slice(0, index);
      accumulate[key] = current.slice(index + 1).trim();
    }
    return accumulate;
  }, {});
}

// 格式化金额为大写汉字
export function moneyToUpper(num) {
  let strOutput = '';
  let strUnit = '仟佰拾亿仟佰拾万仟佰拾元角分';
  num += '00';
  const intPos = num.indexOf('.');
  if (intPos >= 0) num = num.substring(0, intPos) + num.substr(intPos + 1, 2);
  strUnit = strUnit.substr(strUnit.length - num.length);
  for (let i = 0; i < num.length; i++)
    strOutput += '零壹贰叁肆伍陆柒捌玖'.substr(num.substr(i, 1), 1) + strUnit.substr(i, 1);
  return strOutput
    .replace(/零角零分$/, '')
    .replace(/零[仟佰拾]/g, '零')
    .replace(/零{2,}/g, '零')
    .replace(/零([亿|万])/g, '$1')
    .replace(/零+元/, '元')
    .replace(/亿零{0,3}万/, '亿')
    .replace(/^元/, '零元');
}

export function download(url, filename) {
  console.log(filename);

  const a = document.createElement('a');
  a.href = url;
  a.setAttribute('download', filename);
  a.style.display = 'none';

  document.body.appendChild(a);
  a.addEventListener('click', () => {
    a.parentNode.removeChild(a);
  });
  a.click();
}
