import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import request from '../http/fetch';
import { parseRespone } from '../http';
import moment from 'moment';
import { random } from 'lodash';
import { getAliyunOSSKeyWithoutCallBack } from 'SERVICE/common';

export default {
  name: 'file',
  /**
   * 下载文件
   * @param { string, required } url 下载的文件连接
   * @param { string } fileName 文件名
   * @param { string } format 文件格式(文件后缀)
   * @return Promise
   * */
  download({ url, fileName, format }) {
    return new Promise((resolve, reject) => {
      const xhr = new XMLHttpRequest();
      xhr.open('GET', url);
      xhr.responseType = 'blob';
      xhr.onload = () => {
        const link = document.createElement('a');
        const body = document.body;
        link.href = window.URL.createObjectURL(xhr.response);
        let file = fileName || 'download';
        if (format) file = `${file}.${format}`;
        link.download = decodeURIComponent(file);
        link.style.display = 'none';
        body.appendChild(link);
        link.click();
        window.URL.revokeObjectURL(link.href);
        body.removeChild(link);
        showMessage('下载成功', 'success');
        resolve(true);
      };
      xhr.onerror = () => {
        showMessage('下载失败', 'error');
        resolve(false);
      };
      xhr.send();
    });
  },
  downloadDirect(url, fileName, format) {
    const link = document.createElement('a');
    link.href = url;
    let file = fileName || 'download';
    if (format) file = `${file}.${format}`;
    link.download = decodeURIComponent(file);
    link.style.display = 'none';
    const body = document.body;
    body.appendChild(link);
    link.click();
    body.removeChild(link);
  },
  /**
   * 上传文件
   * */
  async upload(params, url, method = 'POST') {
    const formData = new FormData();

    Object.keys(params).forEach((key) => {
      formData.append(key, params[key]);
    });

    const res = await request(url, {
      method,
      body: formData,
    });

    return parseRespone(res);
  },
  /**
   * 上传文件获取OSSkey
   * onloadCallback返回两个参数: xhr对象以及OSS都文件key
   */
  async onUploadFile(files, onloadCallback, onerror) {
    const { OSSAccessKeyId, policy, signature, host, Expires, callback } = await getAliyunOSSKeyWithoutCallBack();
    let formData = new FormData();
    const time = moment().format('YYYYMMDDHHmmss');
    const key = `${time}${random(0, 1000)}${files[0].name}`;
    formData.append('OSSAccessKeyId', OSSAccessKeyId);
    formData.append('policy', policy);
    formData.append('Signature', signature);
    formData.append('success_action_status', '200');
    formData.append('key', key);
    files.forEach((file) => {
      formData.append('file', file);
    });
    const xhr = new XMLHttpRequest();
    xhr.open('POST', host);
    xhr.onload = () => {
      onloadCallback && onloadCallback(xhr, key);
    };

    xhr.onerror = () => {
      onerror && onerror();
    };
    xhr.send(formData);
  },
  downloadFile(downFileUrl, fileName) {
    const a = document.createElement('a');
    a.href = downFileUrl;
    a.setAttribute('download', fileName);
    a.style.display = 'none';
    document.body.appendChild(a);
    a.addEventListener('click', () => {
      a.parentNode.removeChild(a);
    });
    a.click();
  },
};
