export const isIdCard = (id) => {
  const tempPattern = /^\d{17}(\d|X)$/;
  if (typeof id !== 'string') {
    return false;
  }

  if (!tempPattern.test(id) || id.length !== 18) {
    return false;
  }

  const areaCode = id.slice(0, 6);
  const birthCode = id.slice(6, 14);

  const areaStatus = areaValid(areaCode);
  const birthStatus = birthValid(birthCode);
  const endStatus = endValid(id);

  return areaStatus && birthStatus && endStatus;
};

// 地区校验，只校验省级是否合规
const areaValid = (str) => {
  const format = /^(([1][1-5])|([2][1-3])|([3][1-7])|([4][1-6])|([5][0-4])|([6][1-5])|([7][1])|([8][1-3]))\d{4}$/;
  return format.test(str);
};

// 生日校验，校验出生年月日是否合法，还有是否时间小于今天
const birthValid = (str) => {
  try {
    const beforeCalculateYear = Number(str.slice(0, 4)),
      beforeCalculateMonth = Number(str.slice(4, 6)),
      beforeCalculateDay = Number(str.slice(6, 8));
    const beforeCalculateBirthTime = new Date(beforeCalculateYear, beforeCalculateMonth - 1, beforeCalculateDay);
    const afterCalculateYear = beforeCalculateBirthTime.getFullYear();
    const afterCalculateMonty = beforeCalculateBirthTime.getMonth() + 1;
    const afterCalculateDay = beforeCalculateBirthTime.getDate();
    const nowTime = new Date().getTime();
    return (
      beforeCalculateYear === afterCalculateYear &&
      beforeCalculateMonth === afterCalculateMonty &&
      beforeCalculateDay === afterCalculateDay &&
      nowTime > beforeCalculateBirthTime.getTime()
    );
  } catch (e) {
    return false;
  }
};

// 尾号校验
const endValid = (str) => {
  const c = [7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2]; // 系数
  const b = ['1', '0', 'X', '9', '8', '7', '6', '5', '4', '3', '2']; // 校验码对照表
  const idArray = str.split('');
  const tempSum = c.reduce((prev, curr, index) => {
    return parseInt(idArray[index]) * curr + prev;
  }, 0);
  return idArray[17].toUpperCase() === b[tempSum % 11].toUpperCase();
};
