const primitiveCheckers = ['Undefined', 'Boolean', 'Number', 'String', 'Bigint', 'Symbol'].reduce((prev, el) => {
  const name = `is${el}`;
  const type = el.toLowerCase();

  prev[name] = (val) => typeof val === type;

  return prev;
}, {});

const toString = Object.prototype.toString.call.bind(Object.prototype.toString);

const typeChecker = (type) => {
  const toStringRet = `[object ${type}]`;

  return (val) => toString(val) === toStringRet;
};

export const isUndefined = primitiveCheckers.isUndefined;
export const isBoolean = primitiveCheckers.isBoolean;
export const isNumber = primitiveCheckers.isNumber;
export const isString = primitiveCheckers.isString;
export const isBigint = primitiveCheckers.isBigint;
export const isSymbol = primitiveCheckers.isSymbol;
export const isNull = typeChecker('Null');

export const isObject = typeChecker('Object');
export const isFunction = (val) => typeof val === 'function';
export const isArray = Array.isArray;
