import React from 'react';
import PropTypes from 'prop-types';
import styles from './styles.module.less';
import Main from '../Main';
import { QrCode } from 'COMPONENT';
import { tagId } from '../Main/config';
import xhPosterBg from 'ASSET/base64/confirmInfo/xh';

const defaultAlign = 'left';
const defaultWeight = 'normal';

class ConfirmInfo extends React.Component {
  constructor(props) {
    super(props);
  }

  static propTypes = {
    id: PropTypes.string,
    imageSource: PropTypes.string,
    isXH: PropTypes.bool,
    infos: PropTypes.arrayOf(
      PropTypes.shape({
        text: PropTypes.string,
        textAlign: PropTypes.oneOf(['left', 'center', 'right']),
        fontWeight: PropTypes.oneOf(['normal', 'bold']),
      }),
    ),
    url: PropTypes.string,
    tips: PropTypes.string,
  };

  static defaultProps = {
    id: tagId,
    isXH: true,
  };

  render() {
    const hasInfos = !!(this.props.infos || []).length;
    return (
      <div>
        <div
          id={this.props.id}
          className={styles.ConfirmInfo}
          style={{
            backgroundImage: `url("${xhPosterBg}")`,
          }}
        >
          <div
            className={styles.infoWrap}
            style={{ visibility: hasInfos ? 'visible' : 'hidden', height: hasInfos ? '373px' : '273px' }}
          >
            {(this.props.infos || []).map((item, index) => (
              <div
                className={styles.infoRow}
                key={index}
                style={{
                  textAlign: item.textAlign || defaultAlign,
                  fontWeight: item.fontWeight || defaultWeight,
                }}
              >
                <span>{item.text}</span>
              </div>
            ))}
          </div>
          <div className={styles.qrCode}>
            {this.props.url && <QrCode id="ConfirmInfoQrCode" text={this.props.url} width={150} height={150} />}
          </div>

          <div className={styles.qrCodeTips}>长按识别二维码</div>

          <div className={styles.footerTips}>{this.props.tips}</div>
        </div>

        {this.props.imageSource && <img className={styles.image} src={this.props.imageSource} alt="确认信息" />}
      </div>
    );
  }
}

export default (props) => {
  return (
    <Main {...props}>
      <ConfirmInfo />
    </Main>
  );
};
