import React from 'react';
import styles from './styles.module.less';
import { defaultConfig, tagId } from './config';
import { jsonParse } from 'UTIL/common';
import html2canvas from 'html2canvas';
import qs from 'qs';

class Main extends React.Component {
  constructor(props) {
    super(props);

    this.generatePic = this.generatePic.bind(this);

    this.state = {
      imageSource: null,
    };

    const qsObj = qs.parse(location.search.replace(/^\?/, ''));
    if (typeof qsObj !== 'object' || !qsObj.key) {
      throw new Error('ERROR: Missing parameters(key)!');
    }

    let storage = sessionStorage.getItem(qsObj.key);
    sessionStorage.removeItem(qsObj.key);
    storage = jsonParse(storage) || {};

    this.data = storage.data || {};
    this.config = Object.assign({}, defaultConfig, storage.config || {});

    this.id = 'generateTarget';
  }

  componentDidMount() {
    this.generatePic();
  }

  generatePic() {
    html2canvas(document.querySelector(`#${tagId}`), this.config).then((canvas) => {
      const imageSource = canvas.toDataURL('image/png');
      this.setState({ imageSource });
    });
  }

  static download(href, fileName, callback) {
    let a = document.createElement('a');
    a.setAttribute('href', href);
    a.setAttribute('download', fileName);
    document.body.appendChild(a);
    a.click();
    a.remove();
    if (callback) callback();
  }

  render() {
    let children = this.props.children || undefined;
    if (React.isValidElement(children)) {
      children = React.cloneElement(children, {
        ...this.data,
        id: this.id,
        imageSource: this.state.imageSource,
      });
    }

    return <div className={styles.wrapper}>{children}</div>;
  }
}

export default Main;
