/**
 * 收藏夹按钮
 */
import React from 'react';
import { connect } from 'dva';
import { Menu, Tooltip } from 'antd';
import Popover from 'COMPONENT/Popover';
import styles from './styles.less';
import IconButton from 'COMPONENT/IconButton';
import Icon from 'COMPONENT/Icon';
import vars from 'COMPONENT/theme/variables';
import TextLink from 'COMPONENT/TextLink';

import { OLDPAGEBASE } from 'CONST/config';

class BtnCollection extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      collectionShow: false, //收藏面板是否显示
      starIcontype: 'star_mc', // {string} 收藏图标按钮的状态
    };
  }

  componentWillMount() {
    const { dispatch } = this.props;
    dispatch({
      type: 'main/loadCollections',
    });
  }

  /**
   * 收藏项目点击处理
   * @param  {string} collection 菜单
   */
  collectionClick = (collection) => {
    const {
      dispatch,
      main: { menus },
    } = this.props;
    const menu = menus.find((m) => m.id === collection.resId);
    dispatch({
      type: 'main/openPage',
      payload: {
        path: menu.newUrl || `${OLDPAGEBASE}?src=${encodeURIComponent(collection.resUrl)}`,
        refresh: true,
      },
    });

    this.setState({ collectionShow: false, starIcontype: 'star_mc' });
  };

  /**
   * 删除收藏项目
   * @param  {string} resId 收藏菜单ID
   */
  deleteCollectionClick = (resId) => {
    const {
      dispatch,
      main: { collectionCommiting },
    } = this.props;
    if (collectionCommiting) {
      return;
    }
    dispatch({
      type: 'main/deleteCollection',
      payload: {
        resId,
      },
    });
  };

  render() {
    const {
      main: { collections },
    } = this.props;
    const { collectionShow, starIcontype } = this.state;
    const collectionColSize = collections.some((group) => {
      return group.items && group.items.length >= 2;
    })
      ? 2
      : 1;

    return (
      <Popover
        placement="bottomRight"
        content={
          <div className={styles.popoverContent}>
            {(!collections || collections.length === 0) && <span>当前无收藏记录</span>}
            <table className={styles.collectionsTable}>
              {collections &&
                collections.length > 0 &&
                collections.map((group, i) => {
                  let rows = [];
                  let row = [];
                  group.items.forEach((item, n) => {
                    if (n > 0 && n % collectionColSize === 0) {
                      rows.push(row);
                      row = [];
                    }
                    row.push(item);
                  });
                  if (row.length) {
                    rows.push(row);
                  }
                  return (
                    <tbody key={i}>
                      <tr>
                        <th className={styles.collectionsTitle} colSpan={collectionColSize}>
                          {group.title}
                        </th>
                      </tr>
                      {rows.map((row, k) => {
                        return (
                          <tr key={k}>
                            {row.map((item, j) => {
                              return (
                                <td key={j} className={styles.collectionCol}>
                                  <TextLink type="secondary" onClick={() => this.collectionClick(item)}>
                                    {item.resName}
                                  </TextLink>
                                  <span
                                    className={styles.popoverActIcon}
                                    onClick={() => this.deleteCollectionClick(item.resId)}
                                  >
                                    <IconButton
                                      type="cross"
                                      color={vars.primaryColor}
                                      hoverType="cross_circle"
                                      className={styles.deleteBtn}
                                    />
                                  </span>
                                </td>
                              );
                            })}
                          </tr>
                        );
                      })}
                    </tbody>
                  );
                })}
            </table>
          </div>
        }
        visible={collectionShow}
        onVisibleChange={(visible) => {
          this.setState({ collectionShow: visible }, () => {
            if (visible) {
              setTimeout(() => {
                this.setState({ starIcontype: 'star_mc_a' });
              }, 100);
            } else {
              this.setState({ starIcontype: 'star_mc' });
            }
          });
        }}
      >
        <Icon size={24} type={starIcontype} style={{ cursor: 'pointer', marginRight: 38 }} className="vMiddleItem" />
      </Popover>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    main: state.main,
  };
};

export default connect(mapStateToProps)(BtnCollection);
