import React from 'react';
import { connect } from 'dva';
import { Icon as AntdIcon } from 'antd';
import { Icon, PopoverPlus } from 'COMPONENT';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { isFunction, isNumber } from 'UTIL/lang/data-type';
import { runFunc } from 'UTIL/common';
import {
  callLoginStatusCfg,
  CALL_LOGIN_STATUS_OFFLINE,
  CALL_LOGIN_STATUS_LEISURE,
  CALL_LOGIN_STATUS_RINGING,
  CALL_LOGIN_STATUS_CALLING,
  CALL_WAY_WEB,
  CALL_WAY_SIP,
  CALL_WAY_PHONE,
  CALL_DIRECT_ANSWER,
} from 'CONST/callConfig';
import ccApi from 'SERVICE/callCenter';
import custApi from 'SERVICE/customer';
import { isUserMediaEnabled } from './utils';
import Panel from './Panel';

import styles from './styles.module.less';

class CallCenterWidget extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      isInit: false,
      popDefaultVisible: false,
      popLoading: false,
      callMsgData: {
        direction: null,
        calleePhone: '',
        calleeName: '',
        callerPhone: '',
        callerName: '',
      },
    };

    this.updateCallClient = this.updateCallClient.bind(this);
    this.getCallDirection = this.getCallDirection.bind(this);
    this.switchToOffline = this.switchToOffline.bind(this);
    this.switchToOnline = this.switchToOnline.bind(this);
    this.handleOnline = this.handleOnline.bind(this);
    this.handleMsgArrival = this.handleMsgArrival.bind(this);
    this.updateAgentWorkStatus = this.updateAgentWorkStatus.bind(this);
    this.fetchPhoneOwner = this.fetchPhoneOwner.bind(this);
    this.handlePhoneUpdate = this.handlePhoneUpdate.bind(this);
    this.renderIcon = this.renderIcon.bind(this);

    this.closePopover = this.closePopover.bind(this);
    this.$popover = null;
  }

  componentDidMount() {
    if (!this.state.isInit && this.props.isLogin) {
      this.initAgent();
      this.setState({
        isInit: true,
      });
    }
  }

  componentDidUpdate(prevProps, prevState, snapshot) {
    if (!this.state.isInit && this.props.isLogin && this.props.isLogin !== prevProps.isLogin) {
      this.initAgent();
    }
  }

  async initAgent() {
    this.setState({ popLoading: true });

    try {
      const result = await new Promise((resolve, reject) => {
        this.props.dispatch({
          type: 'callCenter/initAgent',
          payload: {
            resolve,
            reject,
          },
        });
      });

      const { available, agentLoginStatus, callWay } = result;
      const isOnline = agentLoginStatus !== CALL_LOGIN_STATUS_OFFLINE;

      if (available && isOnline) {
        await new Promise((resolve, reject) => {
          this.props.dispatch({
            type: 'callCenter/putCallCenterOnline',
            payload: {
              onlineCallback: () => {},
              messageCallback: this.handleMsgArrival,
              resolve,
              reject,
            },
          });
        });

        if (callWay === CALL_WAY_WEB) {
          if (isUserMediaEnabled()) {
            await new Promise((resolve, reject) => {
              this.props.dispatch({
                type: 'callCenter/bootstrapVerto',
                payload: {
                  getDirection: this.getCallDirection,
                  onCallChange: this.updateCallClient,
                  resolve,
                  reject,
                },
              });
            });
          } else {
            showMessage('当前环境不支持网络电话，请前往 https 站点使用', 'error');
          }
        }
      }

      this.setState({ popLoading: false });
    } catch (e) {
      this.setState({ popLoading: false });
    }
  }

  /**
   * 更新呼叫信息
   * @param {Object} payload
   * @param {number} direction 呼叫方向
   * @param {string} callId 通话id
   * @param {string} vertoCallId 软电话通话id
   */
  updateCallClient(payload = {}) {
    this.props.dispatch({
      type: 'callCenter/updateCallClient',
      payload,
    });
  }

  /**
   * 获取当前呼叫方向
   */
  getCallDirection() {
    return this.props.currentDirection;
  }

  /**
   * 切换到下线状态
   */
  async switchToOffline() {
    if (!this.props.isOnline) {
      return;
    }

    this.setState({ popLoading: true });

    try {
      await new Promise((resolve, reject) => {
        this.props.dispatch({
          type: 'callCenter/changeAgentOnlineStatus',
          payload: {
            agentLoginStatus: CALL_LOGIN_STATUS_OFFLINE,
            resolve,
            reject,
          },
        });
      });

      this.props.dispatch({
        type: 'callCenter/takeCallCenterOffline',
      });

      showMessage('已成功离线', 'success');

      this.setState({ popLoading: false });
    } catch (e) {
      showMessage('离线失败', 'error');

      this.setState({ popLoading: false });
    }
  }

  /**
   * 切换到上线状态
   */
  async switchToOnline({ callWay } = {}) {
    if (this.props.isOnline) {
      return;
    }

    if (callWay === CALL_WAY_WEB && !isUserMediaEnabled()) {
      showMessage('当前环境不支持网络电话，请前往 https 站点使用', 'error');

      return;
    }

    this.setState({ popLoading: true });

    try {
      const result = await new Promise((resolve, reject) => {
        this.props.dispatch({
          type: 'callCenter/fetchToUpdAgentInfo',
          payload: {
            resolve,
            reject,
          },
        });
      });

      const { available, agentLoginStatus } = result;
      const isOffline = agentLoginStatus === CALL_LOGIN_STATUS_OFFLINE;

      if (available && isOffline) {
        await new Promise((resolve, reject) => {
          this.props.dispatch({
            type: 'callCenter/putCallCenterOnline',
            payload: {
              onlineCallback: () => this.handleOnline({ callWay }),
              messageCallback: this.handleMsgArrival,
              resolve,
              reject,
            },
          });
        });
      }

      this.setState({ popLoading: false });
    } catch (e) {
      this.setState({ popLoading: false });
    }
  }

  /**
   * 上线响应回调
   * @param {Object} payload
   * @param {Object} payload.callWay 呼叫方式
   */
  async handleOnline(payload) {
    this.setState({ popLoading: true });

    const { callWay } = payload;

    try {
      await new Promise((resolve, reject) => {
        this.props.dispatch({
          type: 'callCenter/changeAgentOnlineStatus',
          payload: {
            agentLoginStatus: CALL_LOGIN_STATUS_LEISURE,
            callWay,
            resolve,
            reject,
          },
        });
      });

      if (callWay === CALL_WAY_WEB) {
        await new Promise((resolve, reject) => {
          this.props.dispatch({
            type: 'callCenter/bootstrapVerto',
            payload: {
              getDirection: this.getCallDirection,
              onCallChange: this.updateCallClient,
              resolve,
              reject,
            },
          });
        });
      }

      showMessage('已成功上线', 'success');

      this.setState({ popLoading: false });
    } catch (e) {
      // showMessage('上线失败', 'error');

      this.setState({ popLoading: false });
    }
  }

  /**
   * 接收消息后的处理回调
   * @param {Object} data 消息体
   * @param {Object} data.msgType 消息类型
   */
  handleMsgArrival(data) {
    switch (data.msgType) {
      case 'PhoneStateAgentRing':
        this.updateAgentWorkStatus(CALL_LOGIN_STATUS_RINGING);

        const direction = Number(data.direction);
        const callMsgData = {
          direction,
          calleePhone: data.callee,
          calleeName: '',
          callerPhone: data.caller,
          callerName: '',
        };

        this.setState({
          popDefaultVisible: direction === CALL_DIRECT_ANSWER,
          callMsgData,
        });

        this.updateCallClient({
          direction,
          callId: data.callId,
        });
        this.fetchPhoneOwner(callMsgData);

        break;
      case 'PhoneStateAgentConnected':
      case 'PhoneStateUserConnected':
        this.updateAgentWorkStatus(CALL_LOGIN_STATUS_CALLING);

        break;
      case 'StopService':
      case 'UserHangUp':
      case 'AgentHangUp':
      case 'AgentStateIdle':
        this.updateAgentWorkStatus(CALL_LOGIN_STATUS_LEISURE);
        this.updateCallClient({
          direction: null,
          callId: null,
        });

        this.setState({
          popDefaultVisible: false,
          callMsgData: {
            direction: null,
            calleePhone: '',
            calleeName: '',
            callerPhone: '',
            callerName: '',
          },
        });

        break;
    }
  }

  /**
   * 更新座席工作状态
   * @param {number} status
   */
  updateAgentWorkStatus(status) {
    this.props.dispatch({
      type: 'callCenter/updateState',
      payload: {
        agentLoginStatus: status,
        uniqueAgentLoginStatus: status,
      },
    });
  }

  async fetchPhoneOwner(callMsgData) {
    const { direction, callerPhone, calleePhone } = callMsgData;

    if (!isNumber(direction)) {
      return;
    }

    const isCallingIn = direction === CALL_DIRECT_ANSWER;
    const phone = isCallingIn ? callerPhone : calleePhone;
    const ownerFieldName = isCallingIn ? 'callerName' : 'calleeName';

    try {
      const response = await custApi.getCustInfoByPhone({ phone });
      const data = response && response.data ? response.data : {};
      const { name = '' } = data;

      this.setState((prevState) => {
        const { callMsgData } = prevState;

        return {
          ...prevState,
          callMsgData: {
            ...callMsgData,
            [ownerFieldName]: name,
          },
        };
      });
    } catch (e) {
      this.setState((prevState) => {
        const { callMsgData } = prevState;

        return {
          ...prevState,
          callMsgData: {
            ...callMsgData,
            [ownerFieldName]: '',
          },
        };
      });
    }
  }

  /**
   * 响应手机号码修改
   * @param {Object} payload
   * @param {string} payload.phoneNumber 手机号码
   * @param {string} payload.verificationCode 验证码
   */
  async handlePhoneUpdate(payload) {
    this.setState({ popLoading: true });

    const { agentId } = this.props;
    const { phoneNumber, verificationCode } = payload;

    let successful = false;

    try {
      const response = await ccApi.updateAgentPhone({
        agentId,
        phoneNumber,
        verifyCode: verificationCode,
      });

      successful = (response && response.data) || false;

      if (successful) {
        showMessage('已更新手机号码', 'success');

        this.props.dispatch({
          type: 'callCenter/updateState',
          payload: {
            phoneNumber,
          },
        });
      } else {
        showMessage('设置手机号码失败', 'error');
      }

      this.setState({ popLoading: false });
    } catch (e) {
      showMessage('设置手机号码失败', 'error');

      this.setState({ popLoading: false });
    }

    return successful;
  }

  closePopover() {
    runFunc(this.$popover && this.$popover.close);
  }

  renderIcon() {
    const { callWay } = this.props;

    const iconRenderTable = [
      {
        callWay: CALL_WAY_WEB,
        render: () => <AntdIcon className={styles.callCenterIcon} type="phone" title="网络电话" />,
      },
      {
        callWay: CALL_WAY_SIP,
        render: () => <Icon className={styles.callCenterIcon} type="sip-phone" title="SIP话机" />,
      },
      {
        callWay: CALL_WAY_PHONE,
        render: () => <AntdIcon className={styles.callCenterIcon} type="mobile" title="手机" />,
      },
    ];

    const [{ render } = {}] = iconRenderTable.filter((item) => item.callWay === callWay);

    return isFunction(render) ? render() : null;
  }

  render() {
    if (!this.props.enabled) {
      return null;
    }

    return (
      <div className={styles.callCenterWrapper}>
        <PopoverPlus
          ref={(c) => (this.$popover = c)}
          overlayClassName={styles.callCenterPop}
          initialVisible={this.state.popDefaultVisible}
          content={
            <Panel
              loading={this.state.popLoading}
              callMsgData={this.state.callMsgData}
              onPhoneUpdate={this.handlePhoneUpdate}
              onOnlineClick={this.switchToOnline}
              onOfflineClick={this.switchToOffline}
            />
          }
        >
          <div className={styles.callCenterStatusWrapper}>
            <span className={styles.callCenterIconWrapper}>{this.renderIcon()}</span>
            <span className={styles.callCenterStatusLabel}>{callLoginStatusCfg[this.props.agentLoginStatus]}</span>
          </div>
        </PopoverPlus>
        <audio
          id="cc-widget-call-ringer"
          className={styles.ccAudio}
          src="https://img.sobot.com/console/common/sound/bell_ring2.wav"
        ></audio>
        <audio id="cc-widget-call-audio" className={styles.ccAudio} autoPlay="autoplay"></audio>
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  const { isLogin } = state.account;
  const { enabled, available, agentLoginStatus, agentId, callWay, currentDirection } = state.callCenter;

  return {
    isLogin,
    enabled,
    available,
    agentLoginStatus,
    isOnline: agentLoginStatus !== CALL_LOGIN_STATUS_OFFLINE,
    agentId,
    callWay,
    currentDirection,
  };
};

export default connect(mapStateToProps)(CallCenterWidget);
