import React from 'react';
import { connect } from 'dva';
import { browserHistory } from 'react-router';
import { OLDPAGEBASE } from 'CONST/config';
import style from './styles.module.less';
import { Icon as AntdIcon } from 'antd';
import Deck from 'COMPONENT/Deck';
import Loading from 'COMPONENT/Loading';
import NoDataPlus from 'COMPONENT/NoDataPlus';
import RemindCard from './RemindCard';
import {
  FOLLOW_UP,
  VISIT,
  INITIATIVE_NEW_RESOURCE_PERSONAL,
  GENERAL_NEW_RESOURCE_PERSONAL,
  NEW_RESOURCE_POOL,
  NEW_NETWORK_RESOURCE,
  ECS_CLASS_RETURN_UNAUDIT,
  ECS_CLASS_ACCOUNTANT_RAUDIT,
  PROMOTION_INFO_RETURN_UNAUDIT,
  SCHOOL_STUDENT_NOTICE,
} from 'MODEL/CrmNotification/config';
import { handleCrmTodoCfg } from './utils';
import { runFunc } from 'UTIL/common';

const classPrefix = 'NotificationList';

class NotificationList extends React.Component {
  constructor(props) {
    super(props);

    this.setRefreshMission = this.setRefreshMission.bind(this);
    this.clearRefreshMission = this.clearRefreshMission.bind(this);
    this.loadList = this.loadList.bind(this);
    this.refresh = this.refresh.bind(this);
    this.resolveTodo = this.resolveTodo.bind(this);
    this.resolveFollowCustomer = this.resolveFollowCustomer.bind(this);
    this.resolveResourcePool = this.resolveResourcePool.bind(this);
    this.resolveNetworkResource = this.resolveNetworkResource.bind(this);
    this.resolveDeductionReview = this.resolveDeductionReview.bind(this);
    this.resolveStdNotice = this.resolveStdNotice.bind(this);

    this.state = {
      loading: false,
    };

    this.refreshMission = null;
  }

  componentDidMount() {
    this.setRefreshMission();
  }

  componentWillUnmount() {
    this.clearRefreshMission();
  }

  setRefreshMission() {
    this.refreshMission = window.setInterval(this.refresh, 300000);
  }

  clearRefreshMission() {
    window.clearInterval(this.refreshMission);
    this.refreshMission = null;
  }

  loadList(resolve) {
    this.props.dispatch({
      type: 'account/loadCrmNotificationList',
      payload: {
        callback() {
          resolve();
        },
      },
    });
  }

  async refresh() {
    this.setState({ loading: true });
    await new Promise(this.loadList);
    this.setState({ loading: false });
  }

  resolveTodo(params) {
    switch (params.remindType) {
      case FOLLOW_UP:
      case VISIT:
      case INITIATIVE_NEW_RESOURCE_PERSONAL:
      case GENERAL_NEW_RESOURCE_PERSONAL:
        this.resolveFollowCustomer(params);
        break;
      case NEW_RESOURCE_POOL:
        this.resolveResourcePool(params);
        break;
      case NEW_NETWORK_RESOURCE:
        this.resolveNetworkResource(params);
        break;
      case ECS_CLASS_RETURN_UNAUDIT:
      case ECS_CLASS_ACCOUNTANT_RAUDIT:
        this.resolveEcsClass(params);
        break;
      case PROMOTION_INFO_RETURN_UNAUDIT:
        this.resolveDeductionReview(params);
        break;
      case SCHOOL_STUDENT_NOTICE:
        this.resolveStdNotice(params);
        break;
      default:
        console.error(`ERROR: remindType(${params.remindType}) is invalid!`);
    }
    runFunc(this.props.afterClick);
  }

  // 预约跟进学生提醒、预约上门学生提醒、新增主动资源的提醒、新增普通资源的提醒
  resolveFollowCustomer(params) {
    let todoParams = handleCrmTodoCfg[params.remindType](params);
    const tagPageId = '/ConsultantFollowCustomer';

    if (this.props.activeTabId === tagPageId) {
      // 当前页面是`跟进客户`
      GlobalEvent.emit('CrmNotification:resolveFollowCustomer', todoParams);
    } else if (this.props.tabPanes.some((tab) => tab.id === tagPageId)) {
      // 已打开页面中存在`跟进客户`
      this.props.dispatch({
        type: 'main/changePage',
        payload: {
          id: tagPageId,
          callback() {
            GlobalEvent.emit('CrmNotification:resolveFollowCustomer', todoParams);
          },
        },
      });
    } else {
      // 打开`跟进客户`页面
      this.props.dispatch({
        type: 'main/openPage',
        payload: {
          path: tagPageId,
          params: {
            isTodo: true,
            todoParams,
          },
          refresh: true,
        },
      });
    }
  }

  // 资源池提醒
  resolveResourcePool(params) {
    browserHistory.push({
      pathname: OLDPAGEBASE,
      query: {
        src: `function/customer/customerResourceDeployment.html?${handleCrmTodoCfg[params.remindType](params)}`,
        title: '跟进客户资源调配',
      },
    });
  }

  // 网络资源池提醒
  resolveNetworkResource(params) {
    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path: `/NetworkResourcePool`,
        params: {
          isTodo: true,
          todoParams: handleCrmTodoCfg[params.remindType](params),
        },
        refresh: true,
      },
    });
  }

  // 精英班退费提醒
  resolveEcsClass(params) {
    const tagPageId = '/CoursePromiseClassManage';

    if (this.props.activeTabId === tagPageId) {
      // 当前页面是`精英班学生管理（班主任）`
      GlobalEvent.emit('CrmNotification:resolveEcsClass', params);
    } else if (this.props.tabPanes.some((tab) => tab.id === tagPageId)) {
      // 已打开页面中存在`精英班学生管理（班主任）`
      this.props.dispatch({
        type: 'main/changePage',
        payload: {
          id: tagPageId,
          callback() {
            GlobalEvent.emit('CrmNotification:resolveEcsClass', params);
          },
        },
      });
    } else {
      // 打开`精英班学生管理（班主任）`页面
      this.props.dispatch({
        type: 'main/openPage',
        payload: {
          path: tagPageId,
          params: {
            isTodo: true,
            todoParams: params,
          },
          refresh: true,
        },
      });
    }
  }

  // 抵扣券审批提醒
  resolveDeductionReview(params) {
    const tagPageId = '/deductionCoupon';

    if (this.props.activeTabId === tagPageId) {
      // 当前页面是`精英班学生管理（班主任）`
      GlobalEvent.emit('CrmNotification:deductionCoupon', params);
    } else if (this.props.tabPanes.some((tab) => tab.id === tagPageId)) {
      // 已打开页面中存在`精英班学生管理（班主任）`
      this.props.dispatch({
        type: 'main/changePage',
        payload: {
          id: tagPageId,
          callback() {
            GlobalEvent.emit('CrmNotification:deductionCoupon', params);
          },
        },
      });
    } else {
      // 打开`精英班学生管理（班主任）`页面
      this.props.dispatch({
        type: 'main/openPage',
        payload: {
          path: tagPageId,
          params: {
            isTodo: true,
            todoParams: params,
          },
          refresh: true,
        },
      });
    }
  }

  // 学生学校信息修改提醒
  resolveStdNotice(params) {
    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path: '/ModifyStudentSchool',
        params: params,
        refresh: true,
      },
    });
  }

  render() {
    return (
      <div className={style[classPrefix]} onClick={(e) => e.stopPropagation()}>
        <div className={style[`${classPrefix}-header`]}>
          <div className={style[`${classPrefix}-header-title`]}>系统提示</div>
          <div className={style[`${classPrefix}-header-tools`]}>
            <div className={style[`${classPrefix}-link-btn`]} onClick={this.refresh}>
              <AntdIcon type="reload" />
              <span style={{ marginLeft: 5 }}>点击刷新</span>
            </div>
          </div>
        </div>
        <div className={style[`${classPrefix}-body`]}>
          {this.props.list.length > 0 ? (
            <Deck
              dataSource={this.props.list}
              render={(props, index) => (
                <RemindCard {...props} key={index} onClick={() => this.resolveTodo(props.params)} />
              )}
            />
          ) : (
            <NoDataPlus showText={false} style={{ height: 200 }} />
          )}
        </div>
        <Loading loading={this.state.loading} />
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    list: state.account.crmNotificationList,
    activeTabId: state.main.activeTabId,
    tabPanes: state.main.tabpanes,
  };
};

export default connect(mapStateToProps)(NotificationList);
