import React from 'react';
import { connect } from 'dva';
import style from './styles.module.less';
import { Icon } from 'antd';
import PopoverPlus from 'COMPONENT/PopoverPlus';
import NotificationList from './NotificationList';
import { Badge } from 'antd';
import { runFunc } from 'UTIL/common';
import {
  CRM_REMIND_FOLLOW_STD,
  CRM_REMIND_VISIT_STD,
  CRM_REMIND_ACTIVE_RESOURCES,
  CRM_REMIND_GENERAL_RESOURCES,
  CRM_REMIND_RESOURCE_POOL,
  CRM_REMIND_NETWORK_POOL,
  ECS_REFUND_ACCOUNTING_APPROVE,
  ECS_REFUND_CASHIER_APPROVE,
  DEDUCTION_COUPON_REVIEW,
} from 'MODEL/CrmNotification/config';

const classPrefix = 'CrmNotification';

class CrmNotification extends React.Component {
  constructor(props) {
    super(props);
    this.removeOpenWhenSysInit = this.removeOpenWhenSysInit.bind(this);
    this.closePopover = this.closePopover.bind(this);
    this.state = {
      visible: true,
    };
    this.$popover = null;
  }

  removeOpenWhenSysInit() {
    if (this.props.openWhenSysInit) {
      this.setState({ visible: false }, () => {
        this.props.dispatch({
          type: 'account/updateState',
          payload: {
            openWhenSysInit: false,
          },
        });
        window.removeEventListener('click', this.removeOpenWhenSysInit, false);
      });
    }
  }

  closePopover() {
    runFunc(this.$popover && this.$popover.close);
  }

  render() {
    return (
      <div id={style[classPrefix]}>
        {this.props.hadAuth && (
          <PopoverPlus
            ref={(c) => (this.$popover = c)}
            overlayClassName={style[`${classPrefix}-pop`]}
            initialVisible={this.props.openWhenSysInit}
            content={<NotificationList afterClick={this.closePopover} />}
          >
            <Icon className={style[classPrefix]} type="exclamation-circle-o" title="系统提示" />
            <Badge className={style[`${classPrefix}-count`]} count={this.props.list.length} />
          </PopoverPlus>
        )}
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  const remindList = [
    // CRM跟进节点提醒
    CRM_REMIND_FOLLOW_STD,
    CRM_REMIND_VISIT_STD,
    CRM_REMIND_ACTIVE_RESOURCES,
    CRM_REMIND_GENERAL_RESOURCES,
    CRM_REMIND_RESOURCE_POOL,
    CRM_REMIND_NETWORK_POOL,
    // 精英班退费提醒
    ECS_REFUND_ACCOUNTING_APPROVE,
    ECS_REFUND_CASHIER_APPROVE,
    // 抵扣券审核提醒
    DEDUCTION_COUPON_REVIEW,
  ];

  return {
    hadAuth: remindList.some((authVal) => state.account.btnAuths.includes(authVal)),
    list: state.account.crmNotificationList,
    openWhenSysInit: state.account.openWhenSysInit,
  };
};

export default connect(mapStateToProps)(CrmNotification);
