/**
 * Created by Haolin on 2019/2/12.
 */
import React from 'react';
import { connect } from 'dva';
import style from './index.less';
import classNames from 'classnames';
import Loading from 'COMPONENT/Loading';
import FileIcon from 'COMPONENT/FileIcon';
import { FILE_TYPE_EXTENSIONS } from 'COMPONENT/FileIcon/config';
import { Icon as AntdIcon } from 'antd';
import Icon from 'COMPONENT/Icon';
import NoDataPlus from 'COMPONENT/NoDataPlus';
import file from 'UTIL/file';
import { exportStatus } from 'CONST/exportCenter';
import { Popconfirm } from 'antd';

const FileCard = (props) => {
  const prefix = 'FileCard';
  return (
    <div className={style[prefix]}>
      <div className={style[`${prefix}-icon`]}>
        <FileIcon type={FILE_TYPE_EXTENSIONS[props.data.format]} />
      </div>
      <div className={style[`${prefix}-info`]}>
        <p className={style[`${prefix}-name`]}>{`${props.data.name}.${props.data.format}`}</p>
        <p className={style[`${prefix}-date`]}>{props.data.date}</p>
      </div>
      <div className={style[`${prefix}-status`]}>
        <StatusIcon status={props.data.status} />
      </div>
      <div className={style[`${prefix}-action`]}>
        <StatusAction status={props.data.status} onAction={props.onAction} />
      </div>
    </div>
  );
};
const StatusIcon = (props) => {
  const prefix = 'StatusIcon';
  let another = '';
  let icon = '';
  let isAntdIcon = true;
  switch (props.status) {
    case exportStatus.success.val:
      icon = 'check-circle';
      another = `${prefix}-success`;
      break;
    case exportStatus.fail.val:
      icon = 'close-circle';
      another = `${prefix}-fail`;
      break;
    default:
      another = `${prefix}-default`;
      icon = 'loading-circle';
      isAntdIcon = false;
  }
  return (
    <div className={classNames(style[prefix], style[another])}>
      {isAntdIcon ? <AntdIcon type={icon} /> : <Icon type={icon} size={18} />}
    </div>
  );
};
const StatusAction = (props) => {
  const prefix = 'StatusAction';
  let text = '';
  let isBtn = true;
  switch (props.status) {
    case exportStatus.success.val:
      text = '下载';
      break;
    case exportStatus.fail.val:
      text = '重试';
      break;
    default:
      text = '导出中';
      isBtn = false;
  }
  return (
    <div className={style[prefix]}>
      {isBtn ? (
        <span className={style[`${prefix}-link-btn`]} onClick={props.onAction}>
          {text}
        </span>
      ) : (
        <span>{text}</span>
      )}
    </div>
  );
};
const MoreData = (props) => {
  const prefix = 'MoreData';
  return (
    <div className={style[prefix]}>
      {props.hadMore ? (
        <span className={style[`${prefix}-btn`]} onClick={props.onClick}>
          查看更多
        </span>
      ) : (
        <span className={style[`${prefix}-text`]}>没有更多数据</span>
      )}
    </div>
  );
};

class FileList extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false,
    };
    this.refreshMission = null;
    this.init = this.init.bind(this);
    this.setRefreshMission = this.setRefreshMission.bind(this);
    this.clearRefreshMission = this.clearRefreshMission.bind(this);
    this.loadList = this.loadList.bind(this);
    this.loadMore = this.loadMore.bind(this);
    this.checkNew = this.checkNew.bind(this);
    this.cleanAllData = this.cleanAllData.bind(this);
  }
  static classPrefix = 'FileList';
  componentDidMount() {
    this.init();
    // 监听父组件 ExportNotification 弹窗是否开启
    GlobalEvent.on('ExportNotification:refresh_list', this.checkNew);
    this.setRefreshMission();
  }
  componentWillUnmount() {
    GlobalEvent.off('ExportNotification:refresh_list', this.checkNew);
    this.clearRefreshMission();
  }
  // 初始化
  init() {
    this.loadList('init');
  }
  // 设置定时刷新任务
  setRefreshMission() {
    this.refreshMission = window.setInterval(this.checkNew, 60000);
  }
  // 清除定时刷新任务
  clearRefreshMission() {
    window.clearInterval(this.refreshMission);
    this.refreshMission = null;
  }
  // 加载数据
  loadList(type) {
    this.setState({ loading: true });
    const _self = this;
    this.props.dispatch({
      type: 'main/loadExportFileList',
      payload: {
        type,
        callback() {
          _self.setState({ loading: false });
        },
      },
    });
  }
  // 加载更多
  loadMore() {
    this.loadList('moreData');
  }
  // 检查文件列表是否有更新
  checkNew() {
    this.loadList('check');
  }
  // 清空数据
  cleanAllData() {
    this.setState({ loading: true });
    const _self = this;
    this.props.dispatch({
      type: 'main/cleanExportFileList',
      payload: {
        callback() {
          _self.setState({ loading: false });
        },
      },
    });
  }
  // 文件操作 下载/重试导出
  moreAction(params) {
    if (params.status === exportStatus.success.val) {
      // 下载文件
      file.download({
        url: params.filePath,
        fileName: params.name,
        format: params.format,
      });
    } else if (params.status === exportStatus.fail.val) {
      // 重试导出
      this.clearRefreshMission();
      this.setState({ loading: true });
      const _self = this;
      this.props.dispatch({
        type: 'main/reExportFile',
        payload: {
          id: params.id,
          callback() {
            _self.checkNew();
            _self.setRefreshMission();
          },
        },
      });
    }
  }

  render() {
    const { list, total, pageSize, hadMoreData } = this.props;
    const { loading } = this.state;
    const prefix = FileList.classPrefix;

    return (
      <div className={style[prefix]}>
        <div className={style[`${prefix}-header`]}>
          <div className={style[`${prefix}-header-title`]}>下载内容</div>
          <div className={style[`${prefix}-header-tool`]}>
            <Popconfirm placement="bottomRight" title="确定清空所有导出记录吗？" onConfirm={this.cleanAllData}>
              <div className={style[`${prefix}-link-btn`]}>
                <AntdIcon type="delete" />
                <span style={{ marginLeft: 5 }}>清空</span>
              </div>
            </Popconfirm>
          </div>
        </div>
        <div className={style[`${prefix}-body`]}>
          {total > 0 ? (
            list.map((item, index) => {
              return (
                <FileCard
                  data={item}
                  onAction={() => {
                    this.moreAction(item);
                  }}
                  key={index}
                />
              );
            })
          ) : (
            <NoDataPlus />
          )}
          {total > pageSize && <MoreData onClick={this.loadMore} hadMore={hadMoreData} />}
        </div>
        <Loading loading={loading} />
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  const { exportFileList, exportFileTotal, exportFile_pageSize } = state.main;
  let hadMoreData = exportFileList.length < exportFileTotal;
  return {
    list: exportFileList,
    total: exportFileTotal,
    pageSize: exportFile_pageSize,
    hadMoreData,
  };
};

export default connect(mapStateToProps)(FileList);
