import React from 'react';
import { Pagination } from 'antd';
import Input from 'COMPONENT/Input';
import Button from 'COMPONENT/Button';
import Modal from 'COMPONENT/Modal';
import { findPageFAQ } from 'SERVICE/common';
import styles from './styles.less';

class ModalFAQ extends React.Component {
  constructor(props) {
    super(props);

    const { searchKey } = this.props;

    this.state = {
      searchKey: searchKey || '',
      listData: [],
      total: 0,
      faqItem: null,
      loading: false,
    };
  }

  componentWillMount() {
    this.search();
  }

  search = (page) => {
    const { searchKey } = this.state;

    this.setState({ loading: true });

    findPageFAQ({
      title: searchKey,
      page: page || 1,
      rows: 10,
    }).then((data) => {
      this.setState({ listData: data.rows, total: data.records, faqItem: null, loading: false });
    });
  };

  handleInputKeyDown = (e) => {
    if (e.keyCode === 13) {
      this.search();
    }
  };

  render() {
    const { onClose } = this.props;
    const { searchKey, listData, total, faqItem, loading } = this.state;

    return (
      <Modal title="常见问题" width={1000} onCancel={onClose} okBtnShow={false} loading={loading}>
        <div className={styles.faqModalContent}>
          <div className={styles.section} style={{ width: 300 }}>
            <div className={styles.sectionTitle + ' text-ellipsis'}>问题列表</div>
            <div className={styles.sectionContent}>
              <div>
                <Input
                  placeholder="请输入问题"
                  style={{ width: 190 }}
                  value={searchKey}
                  onChange={(e) => this.setState({ searchKey: e.target.value })}
                  onKeyDown={this.handleInputKeyDown}
                />
                <Button type="primary" style={{ marginLeft: 10 }} onClick={() => this.search()}>
                  查询
                </Button>
              </div>
              <div className={styles.list} style={{ overflow: 'auto', height: 407, marginTop: 10 }}>
                {listData.map((item, i) => (
                  <div key={i} className={styles.listItem} onClick={() => this.setState({ faqItem: item })}>
                    {item.title}
                  </div>
                ))}
              </div>
              <div style={{ marginTop: 10, textAlign: 'center' }}>
                <Pagination size="small" onChange={(page) => this.search(page)} defaultPageSize={10} total={total} />
              </div>
            </div>
          </div>
          <div className={styles.section} style={{ width: 616, marginLeft: 20 }}>
            <div className={styles.sectionTitle + ' text-ellipsis'}>{faqItem && faqItem.title}</div>
            <div
              className={styles.sectionContent}
              style={{ height: 508, overflow: 'auto' }}
              dangerouslySetInnerHTML={{ __html: faqItem && faqItem.reply }}
            ></div>
          </div>
        </div>
      </Modal>
    );
  }
}

export default ModalFAQ;
