/**
 * 顶部栏
 * @property {function} openTab 打开标签页方法
 */
import React from 'react';
import { connect } from 'dva';
import BtnCollection from './BtnCollection';
import BtnAccountInfo from './BtnAccountInfo';
import ModalFaq from './ModalFaq';
import styles from './styles.less';
import IconButton from 'COMPONENT/IconButton';
import Input from 'COMPONENT/Input';
import Icon from 'COMPONENT/Icon';
import ExportNotification from './ExportNotification';
import CrmNotification from './CrmNotification';
import CallWidget, { isEnabled as isCCEnabled } from './CallWidget';
import DownloadWidget from './DownloadWidget';

class Header extends React.Component {
  state = {
    modalFaqVisible: false,
    searchKey: null,
    searchIconType: 'search_mc',
  };

  search = () => {
    this.setState({ modalFaqVisible: true });
  };

  searchInputChange = (e) => {
    this.setState({ searchKey: e.target.value });
  };

  searchInputKeyDown = (e) => {
    if (e.keyCode === 13) {
      this.search();
    }
  };

  render() {
    const { openTab, collections, collectionCommiting, deleteCollection, expand, onExpandBtnClick, callCenterConfig } =
      this.props;
    const { modalFaqVisible, searchKey, searchIconType } = this.state;

    return (
      <div className={styles.wrap}>
        <div className={styles.left + ' vMiddleContainer'}>
          <IconButton
            type="expand_mc"
            size={21}
            onClick={onExpandBtnClick}
            className={'vMiddleItem ' + (expand ? styles.expandBtnRevert : '')}
          />
        </div>
        <div className={styles.right}>
          <div className={styles.search + ' vMiddleContainer'}>
            <div className={styles.searchIcon + ' vMiddleItem'} onClick={this.search}>
              <Icon type={searchIconType} size={24} />
            </div>
            <Input
              placeholder="请输入常见问题"
              style={{ width: 240 }}
              onChange={this.searchInputChange}
              onKeyDown={this.searchInputKeyDown}
              onFocus={() => this.setState({ searchIconType: 'search_mc_h' })}
              onBlur={() => this.setState({ searchIconType: 'search_mc' })}
              className="vMiddleItem"
            />
          </div>
          <div className={styles.btns + ' vMiddleContainer'}>
            {/* 呼叫中心启用限制 */}
            {/* 需要开启咨询师外呼或者外呼工作台 */}
            {isCCEnabled() && callCenterConfig.hasConsultWorkbrench && callCenterConfig.hasOutcallWorkbrench && (
              <CallWidget />
            )}
            <CrmNotification />
            {/* TODO: 由权限控制是否加载 文件导出通知 */}
            <ExportNotification />
            {/* 下载中心 */}
            <DownloadWidget />
            <BtnCollection />
            <BtnAccountInfo />
          </div>
        </div>
        {modalFaqVisible && (
          <ModalFaq searchKey={searchKey} onClose={() => this.setState({ modalFaqVisible: false })} />
        )}
      </div>
    );
  }
}

export default connect((state) => {
  return {
    callCenterConfig: state.account.callCenterConfig,
  };
})(Header);
