/**
 * 图标
 * @property {string} type - 图标类型
 * @property {string} [size] - 大小（px），默认为普通字体大小
 * @property {string} [color] - 颜色，仅对单色图标有效，默认为普通字体色
 * @property {string} [className] - 附加在根元素的类名
 * @property {object} [style] - 附加在根元素的样式
 */
import React from 'react';
import SvgLib from 'ASSET/svgs/menu-icons.svg';
import styles from 'ASSET/svgs/menu-icons.less';

class Icon extends React.Component {
  render() {
    const { type, size, color, className, style: styleProp } = this.props;

    let style = {};

    if (color) {
      style = { ...style, color };
    }

    if (size) {
      style = { ...style, width: size, height: size, lineHeight: size + 'px' };
    }

    if (styleProp) {
      style = { ...style, ...styleProp };
    }

    return (
      <svg className={styles.icon + (className ? ' ' + className : '')} style={style}>
        <use xlinkHref={`${SvgLib}#icon-${type}`}></use>
      </svg>
    );
  }
}

export default Icon;
