/**
 * 侧边菜单
 * @property {function} openTab 打开标签页方法
 */
import React from 'react';
import { connect } from 'dva';
import { Link } from 'dva/router';
import { Menu } from 'antd';
import styles from './styles.less';
import { OLDPAGEBASE } from 'CONST/config';
import linkUrl from 'UTIL/linkUrl';
import Icon from './Icon';
import PopMenu from './PopMenu';
const { SubMenu, Item } = Menu;
import gioReport from 'UTIL/gioReport';
import { getMenuLevel } from './utils';

const nameMap = {
  RES0000000065: 'report', //报表中心
  RES0000000084: 'finance', //财务管理
  RES0000000086: 'products', //产品管理
  personWorkScedule: 'work-scedule', //工作日程
  RES0000000009: 'contract', //合同管理
  RES0000000082: 'information', //基础信息管理
  feedbackList: 'feedback', //个人信息反馈
  3: 'client', //客户管理
  RES0000000010: 'courses', //课程管理
  RES0000000087: 'stock', //库存管理
  RES0000000098: 'target', //目标管理
  1: 'front-desk', //前台工作台
  RES0000000085: 'personnel', //人事管理
  auditManageMenu: 'examine', //审批管理
  TMKWorkbench: 'call-out', //外呼工作台
  InternetWorkbench: 'networks', //网络工作台
  7: 'systems', //系统管理
  sysMonitor: 'monitoring', //系统使用监测
  RES0000000088: 'news', //消息公告
  RES0000000011: 'student', //学生管理
  2: 'consulting', //咨询师
  WORK_BRENCH_CONSULTOR_NEW: 'consulting', //咨询师（新）
  ipadBackEnd: 'ipad', //ipad后台管理
  Preferential: 'report',
  enrollManager: 'baoduManage', // 报读管理
  CrmBench: 'front-desk', // 报读管理,
  SSResourceManage: 'client', // 双师-资源管理
  // 监控管理
  MONITOR_MANAGEMENT: 'monitor',
  '2e0666e3e1cd11ee83721c34da502f3c': 'contract', //家长合同列表
};

/**
 * check path if it match https://...
 * and return address
 * @param {*} path
 */
const checkExternalPath = (path) => {
  if (!path) {
    return false;
  }
  const reg = path.match(/^http[s]?\:\/\/([^/]+)/i);
  return !reg || !reg[1] ? false : reg[1];
};

/**
 * 单个菜单项目组件（递归）
 * @param  {object}    options.menu     当前菜单项目的描述对象
 * @param  {Array}    options.allMenus 所有菜单项目的描述对象
 * @param  {...any} options.props    父组件的属性，包含菜单的事件响应处理方法等
 */
const MenuRecur = ({
  activeRoot,
  menu,
  allMenus,
  expand,
  popMenuVisibleChange,
  popMenuVisible,
  popMenuOpenChange,
  onLinkClick,
  ...props
}) => {
  const subMenus = allMenus.filter((m) => m.parentId === (menu ? menu.id : '-1'));
  let item = allMenus.find((v, k) => {
    return v.id === activeRoot;
  });
  if (item && item.parentId !== '-1') {
    activeRoot = item.parentId;
  }
  const subMenuItems = subMenus.map((m) => {
    if (allMenus.filter((m2) => m2.parentId === m.id).length > 0) {
      return (
        <MenuRecur
          activeRoot={activeRoot}
          key={m.id}
          menu={m}
          allMenus={allMenus}
          expand={expand}
          popMenuVisible={popMenuVisible}
          popMenuVisibleChange={popMenuVisibleChange}
          popMenuOpenChange={popMenuOpenChange}
          onLinkClick={onLinkClick}
        />
      );
    } else {
      const iconId = nameMap[m.id];
      const isExternalPath = checkExternalPath(m.newUrl);
      let linkToPath = '';
      if (!isExternalPath) {
        linkToPath = linkUrl.get(m.newUrl, m.rurl);
      }
      return (
        <Item key={m.id}>
          {m.parentId === '-1' && (
            <span>
              <Icon type={iconId} className="menu-svg" />
            </span>
          )}
          {isExternalPath ? (
            <a href={m.newUrl} target={m.id}>
              <span className="MainContainer-menu-text">{m.rname}</span>
            </a>
          ) : (
            <Link
              to={linkToPath}
              onClick={(e) => {
                onLinkClick(e, { name: m.rname, path: linkToPath });
              }}
            >
              <span className="MainContainer-menu-text">{m.rname}</span>
            </Link>
          )}
        </Item>
      );
    }
  });
  if (menu) {
    let titleComp;
    const iconId = nameMap[menu.id];
    const menuVisible = popMenuVisible[menu.id];
    if (!expand && menu.parentId === '-1') {
      titleComp = (
        <PopMenu
          visible={menuVisible}
          onVisibleChange={(visible, openKeys) => popMenuVisibleChange(menu.id, visible, openKeys)}
          subMenus={subMenuItems}
          iconId={iconId}
          menuId={menu.id}
          initOpenKeys={props.openKeys.filter((k) => k !== menu.id)}
          initSelectedKeys={props.selectedKeys}
        />
      );
    } else {
      titleComp = (
        <span className="MainContainer-menu-folderTitle">
          {menu.parentId === '-1' && (
            <span>
              <Icon type={iconId} className="menu-svg" />
            </span>
          )}
          <span className="MainContainer-menu-text">{menu.rname}</span>
        </span>
      );
    }
    const { onOpenChange, ...collapseProps } = props;
    return (
      <SubMenu
        key={menu.id}
        title={titleComp}
        onOpenChange={!expand && menu.parentId === '-1' ? () => {} : onOpenChange}
        {...collapseProps}
      >
        {!expand && menu.parentId === '-1' ? null : subMenuItems}
      </SubMenu>
    );
  } else {
    return (
      <Menu theme="dark" mode="inline" style={{ height: '100%' }} {...props}>
        {subMenuItems}
      </Menu>
    );
  }
};

class SideMenu extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      activeRoot: '', //激活状态的一级菜单（如果当前选中的菜单有父级，则activeRoot为最上层父级菜单，否则为当前菜单）
      expand: this.props.expand,
      popMenuVisible: {}, //收起状态时，弹出菜单显示状态集合
    };
  }

  componentWillReceiveProps(nextProps) {
    const {
      main: { menus, activeTabId, openedMenuFolderKeys: openedKeys, tabpanes },
      expand,
    } = nextProps;
    const activeTab = tabpanes.find((t) => t.id === activeTabId);
    const selectedMenu = activeTab ? activeTab.menuId : null;
    let activeRoot;
    if (selectedMenu) {
      activeRoot = selectedMenu;
    }
    if (activeRoot !== this.state.activeRoot) {
      this.setState({ activeRoot });
    }
    if (expand !== this.state.expand) {
      if (!expand) {
        setTimeout(() => {
          this.setState({ expand });
        }, 250);
      } else {
        this.setState({ expand });
      }
    }
  }

  popMenuVisibleChange = (menuKey, visible, subMenuOpenKeys) => {
    const {
      dispatch,
      main: { openedMenuFolderKeys },
    } = this.props;
    const { popMenuVisible } = this.state;
    let newOpenKeys;
    if (visible) {
      if (openedMenuFolderKeys.indexOf(menuKey) < 0) {
        newOpenKeys = [menuKey];
      }
    } else {
      newOpenKeys = subMenuOpenKeys ? [...subMenuOpenKeys, menuKey] : [];
    }
    if (newOpenKeys) {
      dispatch({
        type: 'main/updateState',
        payload: { openedMenuFolderKeys: newOpenKeys },
      });
    }
    this.setState({ popMenuVisible: { ...popMenuVisible, [menuKey]: visible } });
  };

  popMenuOpenChange = (openKeys) => {
    const { dispatch } = this.props;
    dispatch({
      type: 'main/updateState',
      payload: { openedMenuFolderKeys: openKeys },
    });
  };

  /**
   * 菜单开闭状态变化处理
   * @param  {Array} openKeys 当前为打开状态的菜单文件夹
   */
  handleOpenChange = (openKeys) => {
    const {
      dispatch,
      main: { openedMenuFolderKeys },
    } = this.props;
    const latestOpenKey = openKeys.find((key) => !(openedMenuFolderKeys.indexOf(key) > -1));
    const latestCloseKey = openedMenuFolderKeys.find((key) => !(openKeys.indexOf(key) > -1));
    let nextOpenKeys = [];
    if (latestOpenKey) {
      nextOpenKeys = this.getAncestorKeys(latestOpenKey).concat(latestOpenKey);
    }
    if (latestCloseKey) {
      nextOpenKeys = this.getAncestorKeys(latestCloseKey);
    }
    dispatch({
      type: 'main/updateState',
      payload: { openedMenuFolderKeys: nextOpenKeys },
    });
  };

  /**
   * 获取所有父级菜单（包含自身）
   * @param  {string} key 菜单唯一标识
   */
  getAncestorKeys = (key) => {
    const {
      main: { menus },
    } = this.props;
    const ancestorKeys = [];
    let currentMenu = menus.filter((menu) => menu.id === key)[0];
    let parentId = currentMenu.parentId;
    while (parentId && parentId !== '-1') {
      ancestorKeys.push(parentId);
      currentMenu = menus.filter((menu) => menu.id === parentId)[0];
      parentId = currentMenu.parentId;
    }
    return ancestorKeys;
  };

  // 上报通过菜单打开的页面
  reportOpen = (e, params) => {
    const menuLevelArr = getMenuLevel(e.target, [params.name]);
    gioReport._openPage({
      pageName: params.name,
      pagePath: decodeURIComponent(params.path),
      menuLevel: menuLevelArr.join(' -> '),
    });
  };

  render() {
    const { activeRoot, expand, popMenuVisible } = this.state;
    const {
      main: { menus, activeTabId, openedMenuFolderKeys, tabpanes },
    } = this.props;
    const activeTab = tabpanes.find((t) => t.id === activeTabId);
    return (
      <div className="Main-menu">
        <MenuRecur
          activeRoot={activeRoot}
          allMenus={menus}
          selectedKeys={[activeTab ? activeTab.menuId : null]}
          openKeys={openedMenuFolderKeys}
          onOpenChange={this.handleOpenChange}
          expand={expand}
          popMenuVisible={popMenuVisible}
          popMenuVisibleChange={this.popMenuVisibleChange}
          popMenuOpenChange={this.popMenuOpenChange}
          onLinkClick={this.reportOpen}
        />
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    main: state.main,
  };
};

export default connect(mapStateToProps)(SideMenu);
