/**
 * 比较节点是否属于
 * @param tag {Element}
 * @param selector {string} 选择器(暂时只支持 节点id('#id'), 节点类名('.class') 节点名('div'))
 *
 * @return {boolean}
 * */
function compareBySelector(tag, selector) {
  if (/^#/.test(selector)) {
    return tag.id === selector;
  } else if (/^\./.test(selector)) {
    return Array.prototype.includes.call(tag.classList, selector);
  } else {
    return tag.tagName === selector.toUpperCase();
  }
}

/**
 * 返回与选择器匹配的父级节点
 * @param dom {Element}
 * @param selector {string} 选择器(暂时只支持 节点id('#id'), 节点类名('.class') 节点名('div'))
 * @param endSelector {string?}
 *
 * @return {Element}
 * */
function findParentBySelector(dom, selector, endSelector) {
  const tag = dom.parentNode;
  if (compareBySelector(tag, selector)) {
    return tag;
  } else if (endSelector && compareBySelector(tag, endSelector)) {
    return null;
  } else {
    return findParentBySelector(tag, selector);
  }
}

/**
 * 获取antdMenu内subMenu的title
 * @param dom {Element}
 *
 * @return {string|null}
 * */
function getPrevLevelText(dom) {
  if (dom) {
    const prevNode = dom.previousElementSibling;
    if (prevNode) {
      const levelTextNode = prevNode.querySelector('.MainContainer-menu-text');
      if (levelTextNode) {
        return levelTextNode.textContent;
      }
    }
  }
  return null;
}

/**
 * 获取点击菜单完整的层级关系
 * @param dom {Element}
 * @param levelArr {Array}
 *
 * @return {Array}
 * */
function getMenuLevel(dom, levelArr = []) {
  const subMenu = findParentBySelector(dom, 'ul', '.Main-menu');
  const levelText = getPrevLevelText(subMenu);
  if (levelText) {
    levelArr.unshift(levelText);
    return getMenuLevel(subMenu, levelArr);
  }
  return levelArr;
}

export { getMenuLevel };
