/**
 * 主标签页右侧的工具栏
 */
import React from 'react';
import { connect } from 'dva';
import { Row, Col, Tabs, Tooltip } from 'antd';
import styles from './styles.less';
import IconButton from 'COMPONENT/IconButton';
import vars from 'COMPONENT/theme/variables';

const { TabPane } = Tabs;

class ToolBar extends React.Component {
  /**
   * 收藏操作，添加或删除
   * @param  {string} type 操作类型'add'|'delete'
   */
  handleCollection = (type) => {
    const {
      dispatch,
      main: { activeTabId, collectionCommiting, tabpanes },
    } = this.props;
    if (collectionCommiting) {
      return;
    }
    const activeTab = tabpanes.find((t) => t.id === activeTabId);
    const activeMenuId = activeTab ? activeTab.menuId : null;
    if (type === 'add') {
      dispatch({
        type: 'main/addCollection',
        payload: {
          resId: activeMenuId,
        },
      });
    } else if (type === 'delete') {
      dispatch({
        type: 'main/deleteCollection',
        payload: {
          resId: activeMenuId,
        },
      });
    }
  };

  render() {
    const {
      main: { activeTabId, collections, tabpanes },
    } = this.props;
    const activeTab = tabpanes.find((t) => t.id === activeTabId);
    const activeMenuId = activeTab ? activeTab.menuId : null;
    const collected = collections.find((group) => {
      return group.items.find((item) => activeMenuId === item.resId);
    });
    const { refreshCurrentTab } = this.props;
    return (
      <div className={styles.wrap}>
        <div className={styles.item}>
          <IconButton type="refresh_circle" onClick={refreshCurrentTab} title="刷新当前页" />
        </div>
        {activeMenuId && (
          <div className={styles.item}>
            {collected ? (
              <IconButton
                key="starLight"
                type="starLight"
                color={vars.warningColor}
                hoverColor={vars.warningColorActive}
                onClick={() => this.handleCollection('delete')}
                title="取消收藏当前页"
              />
            ) : (
              <IconButton key="star" type="star" onClick={() => this.handleCollection('add')} title="收藏当前页" />
            )}
          </div>
        )}
        <div className={styles.item}>
          <IconButton type="question_circle" title="暂无说明" />
        </div>
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    main: state.main,
  };
};

export default connect(mapStateToProps)(ToolBar);
