/**
 * 主页
 */
import React from 'react';
import { connect } from 'dva';
import { browserHistory } from 'dva/router';
import { Tabs, Modal } from 'antd';
import PageBook from './PageBook';
import { OLDPAGEBASE, WELCOMEHTML, NEXT_COMPACT } from 'CONST/config';
import { getPathLevel2, urlWithoutParam, decodeText } from 'UTIL/url';
import SideMenu from './SideMenu';
import Header from './Header';
import ToolBar from './ToolBar';
import styles from './styles.less';
import Loading from 'COMPONENT/Loading';
import { notification } from 'antd';
import TextLink from 'COMPONENT/TextLink';
import loaders from './loaders';
import Icon from 'COMPONENT/Icon';
import styleVars from 'COMPONENT/theme/variables';
import { jsonParse, focusResize } from 'UTIL/common';
import { logoutSystem } from 'SERVICE/system';
import { logout } from 'SERVICE/account';
import gioReport from 'UTIL/gioReport';

const TabPane = Tabs.TabPane;

class Main extends React.Component {
  constructor(props) {
    super(props);
    this.openPage = this.openPage.bind(this);
    this.handleRejectionErrors = this.handleRejectionErrors.bind(this);
    this.scheduledLogoutTasks = this.scheduledLogoutTasks.bind(this);
    this.nextCompactReg = new RegExp(/^\/next-compact/);
  }

  componentWillMount() {
    const { dispatch, history, location, account } = this.props;

    dispatch({
      type: 'main/init',
    });

    // const pathname = (location.pathname === '/' || location.pathname === '/oldPage') ? '/Dashboard' : location.pathname;
    // const url = (location.search && location.pathname !== '/oldPage') ? `${pathname}${location.search}` : `${pathname}`;

    history.push('/Dashboard');

    if (!global.loadURL) {
      global.loadURL = this.openTabByUrl;
    }
    if (!global.closeActivedTab) {
      global.closeActivedTab = this.closeActivedTab;
    }
    if (!global.Index) {
      global.Index = {};
    }
    global.Index = {
      ...global.Index,
      closeCurrentTab: this.closeActivedTab,
    };
  }

  componentDidMount() {
    window.addEventListener('BOSS_OPEN_PAGE', this.openPage, false);
    window.addEventListener('unhandledrejection', this.handleRejectionErrors, false);
    this.scheduledLogoutTasks();
  }

  /**
   * 路由变化时：
   * 1、标签页以及标签页下子页面的状态控制
   *   a、标签页不存在则新增，存在则激活。判断标签页是否存在的规则为：
   *     如果是新页面，则判断路由与newUrl的二级路由部分相等
   *     如果是旧页面，则判断路由中src参数与rurl的URL部分相等
   *   b、如果标签页存在，更新标签页下的页面信息，页面不存在则新增，存在则显示
   * 2、关闭所有打开的模态框
   */
  componentWillReceiveProps(nextProps) {
    if (nextProps.location.key !== this.props.location.key) {
      const {
        dispatch,
        main: { menus, tabpanes, activeTabId },
      } = this.props;
      const {
        location: {
          pathname,
          query: { src, title, mirrorId, path: nextPath },
          search,
          state,
        },
        children,
      } = nextProps;
      const stateTitle = (state && state.title) || null;

      const pathLevel2 = getPathLevel2(pathname);

      const isOld = pathLevel2 === OLDPAGEBASE;
      const isNextCompact = pathLevel2 === NEXT_COMPACT;

      const tabpane = tabpanes.find((pane) => {
        if (isNextCompact) {
          return pane.id === nextPath;
        } else if (isOld) {
          return urlWithoutParam(pane.rurl) === urlWithoutParam(src);
        } else {
          return mirrorId
            ? pathname === pane.newUrl && mirrorId === pane.mirrorId
            : pathLevel2 === getPathLevel2(pane.newUrl);
        }
      });

      if (tabpane) {
        // 再次打开页面(tab已存在)
        let newMenuId = tabpane.menuId;
        if (isNextCompact) {
          // 同页面除path外的还有其他query，且不一样时，需更新原有tab
          const fullPath = pathname + search;
          if (tabpane.rurl !== fullPath) {
            const menu = menus.find((m) => m.rurl === fullPath);

            dispatch({
              type: 'main/updateTab',
              payload: {
                id: tabpane.id,
                changed: {
                  name: title || (menu ? menu.title || menu.name : children.props.route.name),
                  rurl: fullPath,
                  menuId: newMenuId,
                },
              },
            });
          }
        } else if (isOld) {
          if (tabpane.rurl !== src) {
            const menu = menus.find((m) => m.rurl === src);
            const menuHalf = menus.find((m) => urlWithoutParam(m.rurl) === urlWithoutParam(src));
            newMenuId = menu ? menu.id : null;
            dispatch({
              type: 'main/updateTab',
              payload: {
                id: tabpane.id,
                changed: {
                  name: title ? title : menu ? menu.name : menuHalf ? menuHalf.name : null,
                  rurl: src,
                  menuId: newMenuId,
                },
              },
            });
            dispatch({
              type: 'main/refreshTab',
              payload: { id: tabpane.id },
            });
          }
        } else {
          const { id, pages, name } = tabpane;
          let newPages = [...pages];
          const page = pages.find((p) => p.id === pathname);
          if (!page) {
            newPages.push({ id: pathname, content: React.cloneElement(children) });
          }
          dispatch({
            type: 'main/updateTab',
            payload: {
              id,
              changed: {
                // tab name优先从location.state.title内获取
                name: stateTitle || name || children.props.route.name,
                newUrl: pathname,
                pages: newPages,
                activePageId: pathname,
              },
            },
          });
        }

        if (tabpane.id !== activeTabId) {
          // 切换tab
          dispatch({
            type: 'main/selectTab',
            payload: {
              id: tabpane.id,
            },
          });
          dispatch({
            type: 'main/expandMenuTreeById',
            payload: {
              id: newMenuId,
            },
          });
        }
      } else {
        // 首次打开页面(tab上不存在)

        let tabInfo = {
          pages: [{ id: pathname, content: React.cloneElement(children) }],
          activePageId: pathname,
          // tab name优先从location.state.title内获取
          name: stateTitle,
        };

        if (isNextCompact) {
          const fullPath = pathname + search;
          const menu = menus.find((m) => m.rurl === fullPath);
          tabInfo.menuId = menu ? menu.id : null;
          tabInfo.id = nextPath;
          tabInfo.name = title || (menu ? menu.title || menu.name : children.props.route.name);
          tabInfo.rurl = fullPath;
          tabInfo.mirrorId = mirrorId;
        } else if (isOld) {
          if (src === WELCOMEHTML) {
            tabInfo = {
              ...tabInfo,
              id: src,
              name: '欢迎使用',
              rurl: src,
            };
          } else {
            const menu = menus.find((m) => m.rurl === src);
            const menuHalf = menus.find((m) => urlWithoutParam(m.rurl) === urlWithoutParam(src));
            tabInfo = {
              ...tabInfo,
              menuId: menu ? menu.id : null,
              id: urlWithoutParam(src),
              name: title ? title : menu ? menu.name : menuHalf ? menuHalf.name : null,
              rurl: src,
            };
          }
        } else {
          const menu = menus.find((m) => pathLevel2 === m.newUrl);
          tabInfo = {
            ...tabInfo,
            menuId: menu ? menu.id : null,
            id: pathname + (mirrorId ? '?' + mirrorId : ''),
            newUrl: pathname,
            mirrorId,
          };
          if (!tabInfo.name) {
            tabInfo.name = title || (menu ? menu.title || menu.name : children.props.route.name);
          }
        }

        dispatch({
          type: 'main/addTab',
          payload: tabInfo,
        });

        if (!isOld || (isOld && tabInfo.rurl === src)) {
          dispatch({
            type: 'main/expandMenuTreeById',
            payload: {
              id: tabInfo.menuId,
            },
          });
        }
      }

      this.closeModals();
    }

    if (!this.props.account.isLogin && nextProps.account.isLogin) {
      setTimeout(() => {
        this.showSystemMessage();
      }, 1000);
    }
  }

  componentWillUnmount() {
    window.removeEventListener('BOSS_OPEN_PAGE', this.openPage, false);
    window.removeEventListener('unhandledrejection', this.handleRejectionErrors, false);
  }

  /**
   * 提供给旧页面调用的全局事件监听
   * */
  openPage(event) {
    const tag = event.target;
    let path = tag.dataset.path;

    if (!path) {
      return console.error('Error: Event BOSS_OPEN_PAGE is need path!');
    }

    let pathParams = jsonParse(tag.dataset.pathParams);
    if (pathParams) {
      if (Array.isArray(pathParams)) {
        pathParams = pathParams.join('/');
      } else if (typeof pathParams === 'object') {
        pathParams = JSON.stringify(pathParams);
      }
      path = `${path}/${pathParams}`;
    }
    let params = jsonParse(tag.dataset.params, {});
    let searchParams = jsonParse(tag.dataset.searchParams, {});
    let title = jsonParse(tag.dataset.title, '');

    this.props.dispatch({
      type: 'main/openPage',
      payload: {
        path,
        params,
        query: searchParams,
        title,
        refresh: true,
      },
    });
  }

  handleRejectionErrors(err) {
    const loadChunkFailedReg = new RegExp(/Loading chunk (\d)+ failed/g);
    if (loadChunkFailedReg.test(err.reason && err.reason.message)) {
      this.handleLoadChunkFailed();
    }
  }

  handleLoadChunkFailed = () => {
    Modal.info({
      iconType: null,
      content: <span className={styles.reloadPageTips}>boss运营支撑系统已更新，请刷新页面后继续使用</span>,
      okText: '刷新',
      onOk: function () {
        window.location.href = window.location.origin;
      },
    });
  };

  showSystemMessage = () => {
    loaders.loadSystemMessage().then((messages) => {
      messages.forEach((mes, idx) => {
        setTimeout(() => {
          const { title, content, path, params } = mes;
          const key = `open${Date.now()}`;
          const btn = (
            <span>
              {path && (
                <TextLink
                  onClick={() => {
                    this.openTab(path, params);
                    notification.close(key);
                  }}
                >
                  马上处理
                </TextLink>
              )}
              <TextLink type="secondary" onClick={() => notification.close(key)} style={{ marginLeft: 20 }}>
                我知道了
              </TextLink>
            </span>
          );
          notification.open({
            message: title,
            description: content,
            btn,
            key,
            onClose: () => notification.close(key),
            duration: 0,
            icon: <Icon type="warning" color={styleVars.warningColor} size={24} />,
          });
        }, idx * 200);
      });
    });
  };

  openTab = (path, params) => {
    const { dispatch } = this.props;
    dispatch({
      type: 'main/openPage',
      payload: {
        path,
        params,
        refresh: true,
      },
    });
  };

  /**
   * 关闭所有打开的模态框
   */
  closeModals = () => {
    const modalWraps = document.querySelectorAll('.ant-confirm');
    if (modalWraps.length) {
      modalWraps.forEach((m) => {
        const root = m.parentNode.parentNode;
        root.parentNode.removeChild(root);
      });
    }
    const {
      main: { modals },
    } = this.props;
    for (let i = modals.length - 1; i >= 0; i--) {
      if (modals[i].props.onClose) {
        modals[i].props.onClose();
      } else if (modals[i].props.onCancel) {
        modals[i].props.onCancel();
      }
    }
  };

  /**
   * 根据URL打开标签页（旧系统调用）
   * @param  {string} url   页面URL
   * @param  {string?} title 标题
   */
  openTabByUrl = (url, title) => {
    // eduboss旧前端页面有些 title 内如果包含特殊字符会被转码，这里需解码
    if (title) {
      title = decodeText(title);
    }
    // 路由切换
    browserHistory.push({
      pathname: OLDPAGEBASE,
      query: { src: url, title },
    });
  };

  /**
   * 关闭当前标签页
   */
  closeActivedTab = () => {
    const {
      dispatch,
      main: { activeTabId },
    } = this.props;
    this.onTabRemove(activeTabId);
  };

  /**
   * 切换标签页
   * @param {string} activeKey 标签页ID
   */
  onTabChange = (activeKey) => {
    const {
      history,
      main: { tabpanes },
    } = this.props;
    const tabpane = tabpanes.find((m) => m.id === activeKey);

    if (!tabpane) {
      return;
    }

    if (tabpane.newUrl) {
      history.push({
        pathname: tabpane.newUrl,
        query: {
          mirrorId: tabpane.mirrorId,
        },
      });
    } else if (this.nextCompactReg.test(tabpane.rurl)) {
      history.push(tabpane.rurl);
    } else {
      history.push(`${OLDPAGEBASE}?src=${encodeURIComponent(tabpane.rurl)}`);
    }
  };

  /**
   * 关闭tab时的回调
   */
  onTabRemove = (targetKey) => {
    // 如果关闭的是当前tab, 要激活哪个tab?
    // 首先尝试激活右边的, 再尝试激活左边的
    const {
      dispatch,
      main: { activeTabId, tabpanes },
    } = this.props;
    let nextTabKey = activeTabId;
    if (activeTabId === targetKey) {
      let currentTabIndex = -1;
      currentTabIndex = tabpanes.findIndex((pane) => pane.id === targetKey);
      if (tabpanes.length - 1 > currentTabIndex) {
        nextTabKey = tabpanes[currentTabIndex + 1].id;
      } else {
        nextTabKey = tabpanes[currentTabIndex - 1].id;
      }
      this.onTabChange(nextTabKey);
      // 其实还有一种情况, 就是只剩最后一个tab, 但这里不用处理
    }
    dispatch({
      type: 'main/removeTab',
      payload: {
        id: targetKey,
      },
    });
  };

  /**
   * 刷新当前tab
   */
  refreshCurrentTab = () => {
    const {
      dispatch,
      main: { activeTabId },
    } = this.props;
    dispatch({
      type: 'main/refreshTab',
      payload: { id: activeTabId },
    });
  };

  menuExpandBtnClick = () => {
    const { menuExpand, dispatch } = this.props;
    dispatch({
      type: 'main/updateState',
      payload: { menuExpand: !menuExpand },
    });
    focusResize(1000);
  };

  // 定时`10小时`退出登录
  scheduledLogoutTasks() {
    setTimeout(function () {
      logoutSystem().then(() => {
        gioReport.clearUserId();
        logout();
      });
    }, 36000000);
  }

  render() {
    const {
      main: { activeTabId, tabpanes, menus },
      account: { isLogin },
    } = this.props;
    const { menuExpand } = this.props;

    return (
      <div className="Main-root">
        <div className={'Main-container' + (!menuExpand ? ' Main-container-menuCollpased' : '')}>
          <div className="Main-leftArea">
            <div className={styles.logoArea}>
              <div className={styles.logo + (menuExpand ? '' : ' ' + styles.logoHide)} />
              <div className={styles.logoSmall + (!menuExpand ? '' : ' ' + styles.logoSmallHide)} />
            </div>
            {menus.length === 0 ? (
              <Loading loading={true} style={{ background: 'transparent' }} />
            ) : (
              <SideMenu expand={menuExpand} />
            )}
          </div>
          <div className="Main-rightArea">
            <Header expand={menuExpand} onExpandBtnClick={this.menuExpandBtnClick} />
            <ToolBar refreshCurrentTab={this.refreshCurrentTab} />
            {tabpanes.length === 0 ? (
              <Loading loading={true} style={{ background: 'transparent' }} />
            ) : (
              <Tabs
                activeKey={activeTabId}
                type="editable-card"
                onEdit={this.onTabRemove}
                onChange={this.onTabChange}
                hideAdd
                className="ant-layout-tab"
                className={styles.tabsArea}
              >
                {tabpanes.map((pane, Index) => (
                  <TabPane tab={pane.name} key={pane.id} closable={Index === 0 ? false : !pane.tabClosable}>
                    <PageBook menuExpand={menuExpand} pages={pane.pages} activePageId={pane.activePageId} />
                  </TabPane>
                ))}
              </Tabs>
            )}
          </div>
          {!isLogin && <Loading loading={true} size="large" style={{ background: '#fff' }} />}
        </div>
      </div>
    );
  }
}

const mapStateToProps = (state) => {
  return {
    main: state.main,
    account: state.account,
    menuExpand: state.main.menuExpand,
  };
};

export default connect(mapStateToProps)(Main);
