import React from 'react';
import { connect } from 'dva';
import { loadTemplate } from '../utils';
import { CONTRACT, PRINT_CONTRACT } from '../config';
import Index from '../Index';
import { getTemplateCode, getCampusContractPrint, saveCampusContractPrint } from 'SERVICE/print';
import { errorsHandle } from 'UTIL/common';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { parseUrl } from 'UTIL/url';
import GuideContent from './GuideContent';

class ContractPrint extends React.Component {
  constructor(props) {
    super(props);
    this.init = this.init.bind(this);
    this.initBtnAuth = this.initBtnAuth.bind(this);
    this.getTemplateCode = this.getTemplateCode.bind(this);
    this.initContractData = this.initContractData.bind(this);
    this.changeData = this.changeData.bind(this);
    this.saveData = this.saveData.bind(this);
    this.handleCancel = this.handleCancel.bind(this);
    this.state = {
      loading: false,
      data: null,
      template: undefined,
      isAllPage: '0',
    };
    this.tempType = 1;
    this.templateCode = null;
    this.natureGuidePriceFlag = '0';
  }

  componentDidMount() {
    this.init();
  }

  async init() {
    this.setState({ loading: true });

    const urlParams = parseUrl(window.location).params;
    /**
     * tempType 1: 标准合同 2: 家长合同
     * isAllPage 0: 单页打印 1：全打印
     * */
    this.tempType = urlParams.tempType || 1;
    this.natureGuidePriceFlag = urlParams.guideType || '0';
    this.setState({ isAllPage: urlParams.isAllPage || '0' });

    let data = {};
    const hadAuth = await new Promise(this.initBtnAuth);
    if (hadAuth) {
      // 有权限
      this.templateCode = await new Promise(this.getTemplateCode);
      if (this.templateCode) {
        const template = await loadTemplate(CONTRACT, this.templateCode);
        this.setState({ template });
        data = await new Promise(this.initContractData);
      }
    } else {
      // 没权限
      showMessage('警告：当前用户没有使用权限！', 'error');
    }
    this.setState({
      data,
      loading: false,
    });
  }

  initBtnAuth(resolve) {
    this.props.dispatch({
      type: 'account/updateBtnAuth',
      payload: {
        callback(btnAuth) {
          resolve((btnAuth || []).indexOf(PRINT_CONTRACT) > -1);
        },
      },
    });
  }

  async getTemplateCode(resolve) {
    if (this.props.params.templateCode) {
      resolve(this.props.params.templateCode);
    } else {
      const rsp = await getTemplateCode(
        {
          contractId: this.props.params.contractId,
          flag: this.tempType,
        },
        true,
      ).catch(errorsHandle());
      if (rsp && rsp.resultCode === 0 && typeof rsp.data === 'string') {
        resolve(rsp.data);
      } else {
        showMessage('抱歉，您所在的学习中心暂无合同模板', 'error');
        resolve(null);
      }
    }
  }

  async initContractData(resolve) {
    const rsp = await getCampusContractPrint(
      {
        contractId: this.props.params.contractId,
        templateCode: this.templateCode,
        natureGuidePriceFlag: this.natureGuidePriceFlag,
      },
      true,
    ).catch(errorsHandle());
    let data = {};
    if (rsp && rsp.resultCode === 0) {
      data = rsp.data;
      data.payFund = data.payFund || [];
      let payFundLen = data.payFund.length;
      const addNum = 2 - payFundLen;
      if (addNum > 0) {
        for (let i = 0; i < addNum; i++) {
          data.payFund.push({
            payYear: '',
            payMonth: '',
            payDay: '',
            payMoney: '',
          });
        }
      }

      data = this.adapterContract(data);
    }
    resolve(data);
  }

  /**
   * 接送方式
   * json字符串：{"parent":"true","other":"接送方式的文本"} parent 默认为true
   */
  adapterContract(data) {
    let { takeBringtWays } = data;
    if (takeBringtWays) {
      takeBringtWays = JSON.parse(takeBringtWays);
    } else {
      takeBringtWays = {
        parent: true,
        other: false,
        otherDetail: '',
      };
    }

    data = Object.assign({}, data, {
      takeBringtWaysParent: takeBringtWays.parent,
      takeBringtWaysOther: takeBringtWays.other,
      takeBringtWaysOtherDetail: takeBringtWays.otherDetail,
      classStartTime: data.classStartTime || '/',
      openingDate: data.openingDate || '/',
      finishDate: data.finishDate || '/',
      segFirstClassNatureHour: data.segFirstClassNatureHour || '/',
      segFirstClassFee: data.segFirstClassFee || '/',
      segSecondClassFee: data.segSecondClassFee || '/',
    });

    return data;
  }

  parseSaveData(data) {
    let { takeBringtWays, takeBringtWaysParent, takeBringtWaysOther, takeBringtWaysOtherDetail } = data;
    takeBringtWays = {
      parent: takeBringtWaysParent,
      other: takeBringtWaysOther,
      otherDetail: takeBringtWaysOtherDetail,
    };

    data.takeBringtWays = JSON.stringify(takeBringtWays);

    return data;
  }

  changeData(key, val) {
    this.setState((prevState) => ({
      data: {
        ...prevState.data,
        [key]: val,
      },
    }));
  }

  saveData() {
    return new Promise(async (resolve) => {
      const data = this.parseSaveData(this.state.data);

      let apiParams = {
        ...data,
        contractId: this.props.params.contractId,
      };
      const rsp = await saveCampusContractPrint(apiParams, true).catch(errorsHandle());
      if (rsp && rsp.resultCode === 0) {
        showMessage('保存成功！', 'success');
      }
      resolve();
    });
  }

  async handleCancel() {
    const data = await new Promise(this.initContractData);
    this.setState({
      data: data || {},
    });
  }

  render() {
    return (
      <Index
        loading={this.state.loading}
        data={this.state.data || {}}
        template={this.state.template}
        handleChange={this.changeData}
        handleSave={this.saveData}
        guideContent={<GuideContent />}
        handleCancel={this.handleCancel}
        templateProps={{
          isAllPage: this.state.isAllPage || '0',
        }}
      />
    );
  }
}

const mapStateToProps = (state) => {
  return {};
};

export default connect(mapStateToProps)(ContractPrint);
