import React from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';
import style from './index.less';
import Loading from 'COMPONENT/Loading';
import Button from 'COMPONENT/Button';
import GuideBtn from './GuideBtn';
import ModalGuide from './ModalGuide';

const editBtnText = '编辑';
const saveBtnText = '保存';

const classPrefix = 'Print';

class Print extends React.Component {
  constructor(props) {
    super(props);
    this.btnAction = this.btnAction.bind(this);
    this.edit = this.edit.bind(this);
    this.save = this.save.bind(this);
    this.cancel = this.cancel.bind(this);
    this.switchModalGuide = this.switchModalGuide.bind(this);
    this.state = {
      loading: false,
      btnText: editBtnText,
      isEdit: false,
      modalGuideVisible: false,
    };
  }

  static propTypes = {
    title: PropTypes.string,
    loading: PropTypes.bool,
    data: PropTypes.oneOfType([PropTypes.object, PropTypes.array]),
    // reactNode
    template: PropTypes.any,
    handleChange: PropTypes.func,
    handleSave: PropTypes.func,
    editBtnVisible: PropTypes.bool,
    customizeButtons: PropTypes.oneOfType([PropTypes.element, PropTypes.arrayOf(PropTypes.element)]),
    guideContent: PropTypes.oneOfType([PropTypes.element, PropTypes.node, PropTypes.string]),
    handleCancel: PropTypes.func,
    templateProps: PropTypes.object,
  };

  static defaultProps = {
    title: 'BOSS打印管理',
    loading: false,
    handleChange: () => {},
    editBtnVisible: true,
    customizeButtons: null,
    guideContent: null,
    templateProps: {},
  };

  static print() {
    window.print();
  }

  static renderEleOrArrEle(component) {
    if (React.isValidElement(component) || Array.isArray(component)) {
      return component;
    }
    return null;
  }

  btnAction() {
    if (this.state.isEdit) {
      this.save();
    } else {
      this.edit();
    }
  }

  cancel() {
    if (this.props.handleCancel && typeof this.props.handleCancel === 'function') {
      this.props.handleCancel();
    }

    this.setState({
      btnText: editBtnText,
      isEdit: false,
    });
  }

  edit() {
    this.setState({
      btnText: saveBtnText,
      isEdit: true,
    });
  }

  async save() {
    this.setState({ loading: true });
    if (this.props.handleSave && typeof this.props.handleSave === 'function') {
      await this.props.handleSave();
    }
    this.setState({
      loading: false,
      btnText: editBtnText,
      isEdit: false,
    });
  }

  switchModalGuide() {
    this.setState((prevState) => ({ modalGuideVisible: !prevState.modalGuideVisible }));
  }

  render() {
    const Template = this.props.template;

    return (
      <div className={style[classPrefix]}>
        <div className={classNames(style[`${classPrefix}-header`], 'print-hide')}>
          <div className={style[`${classPrefix}-title`]}>
            <span className={style[`${classPrefix}-title-text`]}>{this.props.title}</span>
            {this.props.subTitle && <span className={style[`${classPrefix}-title-sub`]}>({this.props.subTitle})</span>}
          </div>
          <div className={style[`${classPrefix}-tools`]}>
            {this.state.isEdit && (
              <Button type="default" onClick={this.cancel}>
                取消
              </Button>
            )}
            {this.props.customizeButtons && Print.renderEleOrArrEle(this.props.customizeButtons)}
            {this.props.editBtnVisible && (
              <Button type={this.state.isEdit ? 'primary' : 'default'} onClick={this.btnAction}>
                {this.state.btnText}
              </Button>
            )}
            <Button type="primary" onClick={Print.print}>
              打印
            </Button>
          </div>
        </div>
        <div className={classNames(style[`${classPrefix}-body`], 'print-no-padding-t-b')}>
          {!this.state.isEdit && <div className={style[`${classPrefix}-body-cover`]} />}
          {this.props.template && (
            <Template data={this.props.data} handleChange={this.props.handleChange} {...this.props.templateProps} />
          )}
        </div>
        <Loading loading={this.state.loading || this.props.loading} />
        {this.props.guideContent && <GuideBtn text="打印配置指引" onClick={this.switchModalGuide} />}
        {this.state.modalGuideVisible && this.props.guideContent && (
          <ModalGuide onCancel={this.switchModalGuide}>{this.props.guideContent}</ModalGuide>
        )}
      </div>
    );
  }
}

export default Print;
