import React from 'react';
import Index from '../Index';
import { getTemplateCodeNew, getCampusContractPrint } from 'SERVICE/print';
import { PRINT_CONTRACT } from '../config';
import { errorsHandle } from 'UTIL/common';
import showMessage from 'COMPONENT/utils/globalTips/showMessage';
import { parseUrl } from 'UTIL/url';
import GuideContent from './GuideContent';
import ServiceBook from '../template/ServiceBook';
import { connect } from 'dva';
import { CATEGORY_B } from 'CONST/dicts/product';

const errorsHandler = errorsHandle();

class ServiceBookPrint extends React.Component {
  constructor(props) {
    super(props);
    this.init = this.init.bind(this);
    this.initBtnAuth = this.initBtnAuth.bind(this);
    this.getTemplateCode = this.getTemplateCode.bind(this);
    this.initContractData = this.initContractData.bind(this);
    this.state = {
      loading: false,
      data: null,
      template: undefined,
    };
    // 1: 标准合同 2: 家长合同
    this.tempType = 1;
    this.templateCode = '';
  }

  componentDidMount() {
    this.init();
  }

  async init() {
    this.setState({ loading: true });
    let data = {};
    const hadAuth = await new Promise(this.initBtnAuth);
    if (hadAuth) {
      this.templateCode = await new Promise(this.getTemplateCode);
      if (this.templateCode) {
        data = await new Promise(this.initContractData);
      }
    } else {
      showMessage('警告：当前用户没有使用权限！', 'error');
    }
    this.setState({
      data,
      loading: false,
    });
  }

  initBtnAuth(resolve) {
    this.props.dispatch({
      type: 'account/updateBtnAuth',
      payload: {
        callback(btnAuth) {
          resolve((btnAuth || []).indexOf(PRINT_CONTRACT) > -1);
        },
      },
    });
  }

  async getTemplateCode(resolve) {
    if (this.props.params.templateCode) {
      resolve(this.props.params.templateCode);
    } else {
      const rsp = await getTemplateCodeNew({
        contractId: this.props.params.contractId,
        flag: this.tempType,
      }).catch(errorsHandler);
      if (rsp && rsp.resultCode === 0 && rsp.data.contractPayCategory === CATEGORY_B) {
        resolve(rsp.data.templateCode);
      } else {
        showMessage('抱歉，您所在的学习中心暂无该模板', 'error');
        resolve(null);
      }
    }
  }

  async initContractData(resolve) {
    const rsp = await getCampusContractPrint({
      contractId: this.props.params.contractId,
      templateCode: this.templateCode,
    }).catch(errorsHandler);
    resolve((rsp && rsp.data) || {});
  }

  render() {
    return (
      <Index
        loading={this.state.loading}
        data={this.state.data || {}}
        template={ServiceBook}
        editBtnVisible={false}
        guideContent={<GuideContent />}
      />
    );
  }
}

export default connect()(ServiceBookPrint);
