import React from 'react';
import ReactDOM from 'react-dom';
import style from './index.less';
import PropTypes from 'prop-types';
import classNames from 'classnames';
import { onInputChange, TextUtil } from '../../utils';

const classPrefix = 'InputPlus';

const Content = (props) => {
  // 添加占位input
  let data = [...props.data];
  const placeHoderLine = props.fixedRowsNumber - data.length;

  if (placeHoderLine > 0) {
    const arr = Array(placeHoderLine).fill(' ');
    data = [...data, ...arr];
  }

  return (
    <div className={style[`${classPrefix}-content`]} onClick={props.onClick}>
      {(data && data.length ? data : ['']).map((item, index) => {
        return (
          <Text active={props.active} key={index}>
            {item}
          </Text>
        );
      })}
    </div>
  );
};

const Text = (props) => {
  return (
    <div className={style[`${classPrefix}-text`]}>
      <p
        className={classNames(
          style[`${classPrefix}-text-paragraph`],
          { active: props.active },
          'InputPlus-text-paragraph',
        )}
      >
        {props.children}
      </p>
      <div className={style[`${classPrefix}-text-underline`]} />
    </div>
  );
};

/**
 * @param props.maxFontNum {number} 根据输入框宽度手动计算每一行能填充多少个字符
 */
class InputPlus extends React.Component {
  constructor(props) {
    super(props);
    this.switchActive = this.switchActive.bind(this);
    this.initText = this.initText.bind(this);
    this.onChange = this.onChange.bind(this);
    this.calcText = this.calcText.bind(this);

    this.printTextUtil = new TextUtil({
      fontSize: props.fontSize,
    });

    this.state = {
      isActive: false,
      textVal: '',
      textArr: [],
    };
    this.$textarea = null;
  }

  static propTypes = {
    value: PropTypes.string,
    maxFontNum: PropTypes.number,
    onChange: PropTypes.func,
    width: PropTypes.number,
    disabled: PropTypes.bool,
    fixedRowsNumber: PropTypes.number,
    maxRowsNumber: PropTypes.number,
    fontSize: PropTypes.number,
  };

  static defaultProps = {
    width: null,
    fixedRowsNumber: 0,
    disabled: false,
    fontSize: 14,
  };

  switchActive() {
    this.setState(
      (prevState) => ({ isActive: !prevState.isActive }),
      () => {
        if (this.state.isActive) {
          this.$textarea.focus();
        }
      },
    );
  }

  initText() {
    const textResult = this.calcText(this.props.value);
    const { textVal, textArr } = textResult;

    this.setState({
      textVal,
      textArr,
    });
  }

  onChange(val) {
    const textResult = this.calcText(val);
    const { textVal, textArr } = textResult;

    this.setState(
      {
        textVal,
        textArr,
      },
      () => {
        this.props.onChange(val);
      },
    );
  }

  calcText(value) {
    if (!value) {
      return {
        textVal: '',
        textArr: [],
      };
    }

    const { fixedRowsNumber, maxFontNum } = this.props;
    let maxRowsNumber = this.props.maxRowsNumber || fixedRowsNumber;
    let textVal = value;
    let textArr = this.printTextUtil.splitByFontNum(value, maxFontNum);

    if (maxRowsNumber && textArr.length > maxRowsNumber) {
      textArr = textArr.slice(0, maxRowsNumber);
      textVal = textArr.join('');
    }

    return {
      textVal,
      textArr,
    };
  }

  componentDidUpdate(prevProps, prevState, snapshot) {
    if (this.state.textArr.length === 0 && !prevProps.value && prevProps.value !== this.props.value) {
      this.initText();
    }
  }

  render() {
    return (
      <div className={style[classPrefix]} style={this.props.width && { width: this.props.width }}>
        <Content
          active={!this.state.isActive}
          data={this.state.textArr}
          fixedRowsNumber={this.props.fixedRowsNumber}
          onClick={this.switchActive}
        />
        <textarea
          ref={(c) => (this.$textarea = this.$textarea || ReactDOM.findDOMNode(c))}
          className={classNames(style[`${classPrefix}-textarea`], { active: this.state.isActive })}
          value={this.state.textVal}
          onChange={onInputChange(this.onChange)}
          onBlur={this.switchActive}
        />
        {this.props.disabled && <div className={style[`${classPrefix}-disabled`]} />}
      </div>
    );
  }
}

export default InputPlus;
