import React from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';
import style from './index.less';
import { Row, Col } from 'antd';
import Content from '../Content';
import Input from '../Input';

const classPrefix = 'SignedSealed';

const SignedSealed = (props) => {
  return (
    <div className={style[classPrefix]}>
      <Row>
        <Col span={10}>
          <div className={style[`${classPrefix}-row`]}>
            <Content label="甲方（盖章）：">
              {props.showPartyASeal && (
                <Input
                  plus
                  maxFontNum={12}
                  value={props.partyASeal}
                  onChange={(val) => {
                    props.handleChange('partyASeal', val);
                  }}
                />
              )}
            </Content>
          </div>
          <div className={style[`${classPrefix}-row`]}>
            <Content label="甲方代表（经办人签字）：">
              <Input disabled />
            </Content>
          </div>
          <div className={classNames(style[`${classPrefix}-row`], 'align-right')}>
            <Input
              value={props.contractYear}
              onChange={(val) => {
                props.handleChange('contractYear', val);
              }}
              width={37}
            />
            年
            <Input
              value={props.contractMonth}
              onChange={(val) => {
                props.handleChange('contractMonth', val);
              }}
              width={22}
            />
            月
            <Input
              value={props.contractDay}
              onChange={(val) => {
                props.handleChange('contractDay', val);
              }}
              width={22}
            />
            日
          </div>
        </Col>
        <Col span={4}> </Col>
        <Col span={10}>
          <div className={style[`${classPrefix}-row`]}>
            <Content label="乙方（监护人签字）：">
              <Input disabled />
            </Content>
          </div>
          <div className={style[`${classPrefix}-row`]}>
            <Content label="乙方（学习者签字）：">
              <Input disabled />
            </Content>
          </div>
          <div className={classNames(style[`${classPrefix}-row`], 'align-right')}>
            <Input width={37} disabled />年<Input width={22} disabled />月<Input width={22} disabled />日
          </div>
        </Col>
      </Row>
    </div>
  );
};
SignedSealed.propTypes = {
  showPartyASeal: PropTypes.oneOfType([PropTypes.bool]),
  partyASeal: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  contractYear: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  contractMonth: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  contractDay: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  handleChange: PropTypes.func,
};

export default SignedSealed;
