import React from 'react';
import PropTypes from 'prop-types';
import './print.less';
import style from './index.less';
import { Row, Col } from 'antd';
import A4 from '../../../components/A4';
import Contract from '../../../components/Contract';
import Paragraph from '../../../components/Paragraph';
import Content from '../../../components/Content';
import Input from '../../../components/Input';
import CheckBox from '../../../components/CheckBox';
import SignedSealed from '../../../components/SignedSealed';

const A4WrapStyle = {
  padding: '30px 50px',
};

const contractWrapStyle = {
  fontSize: '13px',
  fontWeight: 'normal',
};

const A4wrapClass = 'ss-min-height';

const title = (
  <div style={{ fontSize: '18px', fontFamily: '仿宋', textAlign: 'center' }}>
    <b>中小学生校外培训服务合同</b>
  </div>
);

const SPARK_STANDARD = (props) => {
  return (
    <div>
      <A4 wrapStyle={A4WrapStyle} wrapClass={A4wrapClass}>
        <Contract title={title} wrapStyle={contractWrapStyle}>
          <Paragraph style={{ textAlign: 'right' }}>
            合同编号：
            <Input
              value={props.data.contractId}
              onChange={(val) => {
                props.handleChange('contractId', val);
              }}
              width={150}
            />
          </Paragraph>
          <Paragraph indent={0}>甲方（提供培训方）：</Paragraph>
          <Paragraph indent={0}>
            <Content label="机构名称：">
              <Input
                value={props.data.institutionName}
                onChange={(val) => {
                  props.handleChange('institutionName', val);
                }}
              />
            </Content>
          </Paragraph>
          <Paragraph indent={0}>
            <Content label="办学地址：" flex>
              <Input
                plus
                maxFontNum={47}
                maxRowsNumber={2}
                value={props.data.schoolAddress}
                onChange={(val) => {
                  props.handleChange('schoolAddress', val);
                }}
              />
            </Content>
          </Paragraph>
          <Paragraph indent={0} style={{ marginTop: '2px' }}>
            <Row>
              <Col span={11}>
                <Content label="审批机关 ：">
                  <Input
                    value={props.data.examineOrganization}
                    onChange={(val) => {
                      props.handleChange('examineOrganization', val);
                    }}
                  />
                </Content>
              </Col>
              <Col span={13}>
                <Content label="登记注册机关：">
                  <Input
                    value={props.data.registrationAuthority}
                    onChange={(val) => {
                      props.handleChange('registrationAuthority', val);
                    }}
                  />
                </Content>
              </Col>
            </Row>
          </Paragraph>

          <Paragraph indent={0}>
            <Row>
              <Col span={11}>
                <Content label="办学许可证编号：">
                  <Input
                    value={props.data.schoolLicenseNumber}
                    onChange={(val) => {
                      props.handleChange('schoolLicenseNumber', val);
                    }}
                  />
                </Content>
              </Col>
              <Col span={13}>
                <Content label="办学许可证有效期：">
                  <Input
                    value={props.data.schoolLicenseTime}
                    onChange={(val) => {
                      props.handleChange('schoolLicenseTime', val);
                    }}
                  />
                </Content>
              </Col>
            </Row>
          </Paragraph>

          <Paragraph indent={0}>
            <Row>
              <Col span={11}>
                <Content label="统一社会信用代码：">
                  <Input
                    value={props.data.socialCode}
                    onChange={(val) => {
                      props.handleChange('socialCode', val);
                    }}
                  />
                </Content>
              </Col>
              <Col span={13}>
                <Content label="民非登记证/营业执照有效期：">
                  <Input
                    value={props.data.businessLicenseTime}
                    onChange={(val) => {
                      props.handleChange('businessLicenseTime', val);
                    }}
                  />
                </Content>
              </Col>
            </Row>
          </Paragraph>
          <Paragraph indent={0}>
            <Row>
              <Col span={11}>
                <Content label="联系人：">
                  <Input
                    value={props.data.contactPerson}
                    onChange={(val) => {
                      props.handleChange('contactPerson', val);
                    }}
                  />
                </Content>
              </Col>
              <Col span={13}>
                <Content label="联系电话：">
                  <Input
                    value={props.data.contactPhoneNumber}
                    onChange={(val) => {
                      props.handleChange('contactPhoneNumber', val);
                    }}
                  />
                </Content>
              </Col>
            </Row>
          </Paragraph>

          <Paragraph indent={0}>乙方（接受培训方）：</Paragraph>
          <Paragraph indent={0}>
            <Row>
              <Col span={7}>
                <Content label="学员姓名(必填)：">
                  <Input
                    value={props.data.studentName}
                    onChange={(val) => {
                      props.handleChange('studentName', val);
                    }}
                  />
                </Content>
              </Col>
              <Col span={5}>
                <Content label="性别(必填)：">
                  <Input
                    value={props.data.studentGender}
                    onChange={(val) => {
                      props.handleChange('studentGender', val);
                    }}
                  />
                </Content>
              </Col>
              <Col span={6}>
                <Content label="出生日期：">
                  <Input
                    value={props.data.studentBirthday}
                    onChange={(val) => {
                      props.handleChange('studentBirthday', val);
                    }}
                  />
                </Content>
              </Col>
              <Col span={6}>
                <Content label="年级(必填)：">
                  <Input
                    value={props.data.studentGrade}
                    onChange={(val) => {
                      props.handleChange('studentGrade', val);
                    }}
                  />
                </Content>
              </Col>
            </Row>
          </Paragraph>

          <Paragraph indent={0}>
            <Row>
              <Col span={12}>
                <Content label="就读学校：">
                  <Input
                    value={props.data.schoolName}
                    onChange={(val) => {
                      props.handleChange('schoolName', val);
                    }}
                  />
                </Content>
              </Col>
              <Col span={12}>
                <Content label="身份证号：">
                  <Input
                    value={props.data.studentIdNumber}
                    onChange={(val) => {
                      props.handleChange('studentIdNumber', val);
                    }}
                  />
                </Content>
              </Col>
            </Row>
          </Paragraph>

          <Paragraph indent={0}>
            <Row>
              <Col span={12}>
                <Content label="付款人姓名（必填）：">
                  <Input
                    value={props.data.payerName}
                    onChange={(val) => {
                      props.handleChange('payerName', val);
                    }}
                  />
                </Content>
              </Col>
              <Col span={12}>
                <Content label="联系电话（必填）：">
                  <Input
                    disabled
                    value={props.data.payerPhone}
                    onChange={(val) => {
                      props.handleChange('payerPhone', val);
                    }}
                  />
                </Content>
              </Col>
            </Row>
          </Paragraph>
          <Paragraph indent={0}>
            <Row>
              <Col span={16}>
                <Content label="付款人身份证号（必填）：">
                  <Input
                    value={props.data.payerIdCardNo}
                    onChange={(val) => {
                      props.handleChange('payerIdCardNo', val);
                    }}
                  />
                </Content>
              </Col>
              <Col span={8}>
                <Content label="与学员关系：">
                  <Input
                    value={props.data.payerStudentRelation}
                    onChange={(val) => {
                      props.handleChange('payerStudentRelation', val);
                    }}
                  />
                </Content>
              </Col>
            </Row>
          </Paragraph>
          <Paragraph indent={0}>
            <Row>
              <Col span={8}>
                <Content label="监护人姓名：">
                  <Input
                    value={props.data.customer}
                    onChange={(val) => {
                      props.handleChange('customer', val);
                    }}
                  />
                </Content>
              </Col>
              <Col span={16}>
                <Content label="联系电话：">
                  <Input
                    value={props.data.customerPhone}
                    onChange={(val) => {
                      props.handleChange('customerPhone', val);
                    }}
                  />
                </Content>
              </Col>
            </Row>
          </Paragraph>
          <Paragraph indent={0}>
            <Row>
              <Col span={12}>
                <Content label="监护人身份证号：">
                  <Input
                    value={props.data.customerIdNumber}
                    onChange={(val) => {
                      props.handleChange('customerIdNumber', val);
                    }}
                  />
                </Content>
              </Col>
              <Col span={12}>
                <Content label="联系地址：" flex>
                  <Input
                    value={props.data.customerAdress}
                    onChange={(val) => {
                      props.handleChange('customerAdress', val);
                    }}
                  />
                </Content>
              </Col>
            </Row>
          </Paragraph>

          <Paragraph>
            根据《中华人民共和国教育法》《中华人民共和国民办教育促进法》等有关法律、法规的规定，甲乙双方遵循平等、自愿、公平、诚实、守信的原则，遵循教育规律和青少年健康成长规律，经协商一致，签订本合同。
          </Paragraph>
          <Paragraph>
            <b>第一条 适用对象</b>
          </Paragraph>
          <Paragraph>本合同适用受培训者（学员）一般为在校的中小学生。</Paragraph>
          <Paragraph>
            <b>第二条 培训服务</b>
          </Paragraph>
          <Paragraph indent={0}>
            <Content label="课程名称：" flex>
              <Input
                plus
                maxFontNum={47}
                maxRowsNumber={5}
                value={props.data.className}
                onChange={(val) => {
                  props.handleChange('className', val);
                }}
              />
            </Content>
          </Paragraph>
          <Paragraph indent={0} style={{ marginTop: '2px' }}>
            <Row>
              <Col span={12}>
                <Content label="课程顾问（经办人）：">
                  <Input
                    value={props.data.classCounselor}
                    onChange={(val) => {
                      props.handleChange('classCounselor', val);
                    }}
                  />
                </Content>
              </Col>
              <Col span={12}>
                <Content label="总课时数：（节）">
                  <Input
                    value={props.data.classHour}
                    onChange={(val) => {
                      props.handleChange('classHour', val);
                    }}
                  />
                </Content>
              </Col>
            </Row>
          </Paragraph>
          <Paragraph indent={0}>
            <Row>
              <Col span={12}>
                <Content label="每次培训课时：（节）">
                  <Input
                    value={props.data.onceClassHour}
                    onChange={(val) => {
                      props.handleChange('onceClassHour', val);
                    }}
                  />
                </Content>
              </Col>
              <Col span={12}>
                <Content label="上课时间：">
                  <Input
                    value={props.data.classStartTime}
                    onChange={(val) => {
                      props.handleChange('classStartTime', val);
                    }}
                    onBlur={(val) => {
                      props.handleChange('classStartTime', val || '/');
                    }}
                  />
                </Content>
              </Col>
            </Row>
          </Paragraph>
          <Paragraph indent={0}>
            <Row>
              <Col span={12}>
                <Content label="开课日期：">
                  <Input
                    value={props.data.openingDate}
                    onChange={(val) => {
                      props.handleChange('openingDate', val);
                    }}
                    onBlur={(val) => {
                      props.handleChange('openingDate', val || '/');
                    }}
                  />
                </Content>
              </Col>
              <Col span={12}>
                <Content label="预计结课日期：">
                  <Input
                    value={props.data.finishDate}
                    onChange={(val) => {
                      props.handleChange('finishDate', val);
                    }}
                    onBlur={(val) => {
                      props.handleChange('finishDate', val || '/');
                    }}
                  />
                </Content>
              </Col>
            </Row>
          </Paragraph>
          <Paragraph indent={0}>
            <CheckBox
              value={props.data.pxOneToOne}
              onClick={() => {
                props.handleChange('pxOneToOne', !props.data.pxOneToOne);
              }}
            >
              一对一面授
            </CheckBox>
            <CheckBox
              value={props.data.pxSmallClass}
              onClick={() => {
                props.handleChange('pxSmallClass', !props.data.pxSmallClass);
              }}
            >
              小班面授
            </CheckBox>
            <CheckBox
              value={props.data.pxOneToMany}
              onClick={() => {
                props.handleChange('pxOneToMany', !props.data.pxOneToMany);
              }}
            >
              双师课程
            </CheckBox>
            <CheckBox
              value={props.data.pxWebucation}
              onClick={() => {
                props.handleChange('pxWebucation', !props.data.pxWebucation);
              }}
            >
              线上教育
            </CheckBox>
            <CheckBox
              value={props.data.pxOtherStudyWay}
              onClick={() => {
                props.handleChange('pxOtherStudyWay', !props.data.pxOtherStudyWay);
              }}
            >
              其他方式：
              <Input
                value={props.data.pxOtherStudeWayDetails}
                onChange={(val) => {
                  props.handleChange('pxOtherStudeWayDetails', val);
                }}
                width={280}
              />
            </CheckBox>
          </Paragraph>
          <Paragraph indent={0}>
            <Row>
              <Col span={12}>
                <CheckBox
                  value={props.data.specifyTheTeacher}
                  onClick={() => {
                    props.handleChange('specifyTheTeacher', !props.data.specifyTheTeacher);
                  }}
                >
                  指定教师姓名：
                  <Input
                    value={props.data.specifyTheTeacherName}
                    onChange={(val) => {
                      props.handleChange('specifyTheTeacherName', val);
                    }}
                    width={134}
                  />
                </CheckBox>
                <CheckBox
                  value={props.data.notSpecifyTheTeacher}
                  onClick={() => {
                    props.handleChange('notSpecifyTheTeacher', !props.data.notSpecifyTheTeacher);
                  }}
                >
                  不指定
                </CheckBox>
              </Col>
              <Col span={12}>
                <span style={{ marginRight: '14px' }}>是否有教师资格证</span>
                <CheckBox
                  value={props.data.hasTeacherCertification}
                  onClick={() => {
                    props.handleChange('hasTeacherCertification', !props.data.hasTeacherCertification);
                  }}
                >
                  有
                </CheckBox>
                <CheckBox
                  value={props.data.noTeacherCertification}
                  onClick={() => {
                    props.handleChange('noTeacherCertification', !props.data.noTeacherCertification);
                  }}
                >
                  没有
                </CheckBox>
              </Col>
            </Row>
          </Paragraph>
          <Paragraph indent={0}>
            <Content label="授课地点：">
              <Input
                value={props.data.trainingAddress}
                onChange={(val) => {
                  props.handleChange('trainingAddress', val);
                }}
                width={234}
              />
              学员接送方式：
              <CheckBox
                value={props.data.takeBringtWaysParent}
                onClick={() => {
                  props.handleChange('takeBringtWaysParent', !props.data.takeBringtWaysParent);
                }}
              >
                家长负责接送
              </CheckBox>
              <CheckBox
                value={props.data.takeBringtWaysOther}
                onClick={() => {
                  props.handleChange('takeBringtWaysOther', !props.data.takeBringtWaysOther);
                }}
              >
                其他：
                <Input
                  value={props.data.takeBringtWaysOtherDetail}
                  onChange={(val) => {
                    props.handleChange('takeBringtWaysOtherDetail', val);
                  }}
                  width={144}
                />
              </CheckBox>
            </Content>
          </Paragraph>

          <Paragraph>
            <b>第三条 培训收费</b>
          </Paragraph>
          <Paragraph>
            <Content label="1. 培训费用合计：">
              <Input
                value={props.data.trainingCostMajuscule}
                onChange={(val) => {
                  props.handleChange('trainingCostMajuscule', val);
                }}
                width={180}
              />
              （大写） 其中课时费：
              <Input
                value={props.data.classFee}
                onChange={(val) => {
                  props.handleChange('classFee', val);
                }}
                width={70}
              />
              元，
              <Input
                value={props.data.classFeeForOne}
                onChange={(val) => {
                  props.handleChange('classFeeForOne', val);
                }}
                width={50}
              />
              元/节；
            </Content>
          </Paragraph>
          <Paragraph>
            <Content label="">
              培训资料费：
              <Input
                value={props.data.trainingMaterialsFee}
                onChange={(val) => {
                  props.handleChange('trainingMaterialsFee', val);
                }}
                width={150}
              />
              元; 其他费用：
              <Input
                value={props.data.otherFee}
                onChange={(val) => {
                  props.handleChange('otherFee', val);
                }}
                width={150}
              />
              元。
            </Content>
          </Paragraph>
          <Paragraph>
            2. 乙方应于
            <Input
              value={props.data.payDeadline}
              onChange={(val) => {
                props.handleChange('payDeadline', val);
              }}
              width={250}
            />
            （时间）向甲方付清培训费用。
          </Paragraph>
          <Paragraph>
            3. 乙方采用
            <CheckBox
              value={props.data.cash}
              onClick={() => {
                props.handleChange('cash', !props.data.cash);
              }}
            >
              现金
            </CheckBox>
            <CheckBox
              value={props.data.bankTransfer}
              onClick={() => {
                props.handleChange('bankTransfer', !props.data.bankTransfer);
              }}
            >
              银行卡
            </CheckBox>
            <CheckBox
              value={props.data.aliPayWay}
              onClick={() => {
                props.handleChange('aliPayWay', !props.data.aliPayWay);
              }}
            >
              支付宝
            </CheckBox>
            <CheckBox
              value={props.data.wxPayWay}
              onClick={() => {
                props.handleChange('wxPayWay', !props.data.wxPayWay);
              }}
            >
              微信
            </CheckBox>
            <CheckBox
              value={props.data.otherPayWay}
              onClick={() => {
                props.handleChange('otherPayWay', !props.data.otherPayWay);
              }}
            >
              其他：
              <Input
                value={props.data.otherPayWayContent}
                onChange={(val) => {
                  props.handleChange('otherPayWayContent', val);
                }}
                width={100}
              />
              方式支付培训费用。
            </CheckBox>
          </Paragraph>
          <Paragraph>
            4.
            甲方按国家及行业制度规范收取合同相关费用。甲方应当依照相关规定开设并使用学杂费专用账户，足额留存学习保障金，保障其收取的学杂费主要用于教学活动。甲方收取培训费用后应当及时向乙方开具合法票据。
          </Paragraph>
          <Paragraph>
            5.
            如遇自然灾害、政府行为、突发疫情等不可抗力因素，导致乙方需通过线上进行同时间同老师课程的，按正常课时计费。
          </Paragraph>

          <Paragraph>
            <b>第四条 教学管理</b>
          </Paragraph>
          <Paragraph>
            （一）甲方应在其办学场所醒目位置公示教育培训资质证书、收费项目、收费标准，并提供与其资质相符的教育培训项目。
          </Paragraph>
          <Paragraph>
            （二）甲方应当加强消防、公共卫生等安全管理，落实安全防范措施，建立健全安全管理制度和应急预警处理机制，防范各类安全责任事故发生，保障学生安全。
          </Paragraph>
          <Paragraph>
            （三）甲方应当设置处理合同和服务争议的内设部门或者专员，甲方的客服电话为：
            <Input disabled value={props.data.contactPhoneNumber} width={120} />。
          </Paragraph>
          <Paragraph>（四）合同执行期间，甲方如需更改上课时间，须提前24小时通知学员，并另行安排辅导时间。</Paragraph>
          <Paragraph>（五）合同执行期间，学员因自身原因迟到，迟到时间将计入当次上课时间。</Paragraph>
          <Paragraph>（六）请假说明</Paragraph>
          <Paragraph>
            学员应尽量保证不旷课，不请假、迟到、早退。学员因自身原因确实无法正常上课的，需向老师请假说明。但该次课程按正常扣费处理，且不予另行安排补课。
          </Paragraph>

          <Paragraph>（七）插班说明</Paragraph>
          <Paragraph>1.学员经过甲方的入学测评并达到入学线后方可插班。</Paragraph>
          <Paragraph>2.春秋课程8次课后、寒暑课程开课后与及班级人数已满时不能插班。</Paragraph>
          <Paragraph>3.插班学费计算方法：插班学费=(总课时-已上课时)x课时单价+教材费原价。</Paragraph>

          <Paragraph>（八）调课、转班说明</Paragraph>
          <Paragraph>1.不接受不受理个人原因调课。</Paragraph>
          <Paragraph>
            2.学员因个人原因提出转班，需经学习中心审核通过后方可转班。学生从报名后开始计算，春秋限2次转班/学生/期，寒暑限1次转班/学生/期。
          </Paragraph>
        </Contract>
      </A4>

      <A4 wrapClass={A4wrapClass} wrapStyle={Object.assign({}, A4WrapStyle)}>
        <Contract wrapStyle={contractWrapStyle}>
          <Paragraph style={{ paddingTop: '20px' }}>
            3.转班前后课时费用可能产生的价差，多退少补。开课后转班的，原教材不接受退回。转班后需要配套新教材的，需要另外购买。
          </Paragraph>

          <Paragraph>（九）安全责任</Paragraph>
          <Paragraph>
            1.家长应当教育学员基础安全防范知识，学员在上学途中和离开上课地点后的安全完全由家长负责。
          </Paragraph>
          <Paragraph>
            2.家长应当要求学员遵守校内秩序，不在校内追逐打闹，课间未经老师同意不得离开学习中心，全力配合做好校内监护工作。
          </Paragraph>
          <Paragraph>
            3.学员有特异体质、疾病或者异常心理状况的（如先天性心脏病、癫痫、肺结核、哮喘、肺炎、肾炎、精神病、伤残或其它严重疾病），应当及时书面告知学习中心，不宜在学习中心学习的学员，应当休学由监护人安排治疗、休养。
          </Paragraph>
          <Paragraph>
            4.若学习中心购买公众责任险，学员及家长在学习中心发生的所有人身、财产安全的费用，则以保险赔付为限。如因学员未经许可擅自离校或家长有意隐瞒特异体质、疾病或异常心理状况而导致的人身安全问题，完全由家长负责。
          </Paragraph>
          <Paragraph>
            （十）因教学是一个长期的过程，如乙方后期未能达到乙方监护人预期分数，乙方不得以此为由要求甲方退还已经完成的课时费用。如乙方需提前终止本合同，未消耗的课时可以按照第五条条款要求甲方退费。
          </Paragraph>

          <Paragraph>
            <b>第五条 培训退费</b>
          </Paragraph>
          <Paragraph>1.开课前办理退费申请，退还全部学费（不含资料费及赠送部分）。</Paragraph>
          <Paragraph>2.因甲方原因造成学员所报班级不能正常开班，全额退还学费。</Paragraph>
          <Paragraph>
            3.开课后退费需在下次课程开始前一天提出申请，如课程当天提出申请则当天课程费用计入课时消耗。
          </Paragraph>
          <Paragraph>
            4.公益课程、超低价课程不受理退费；低折扣课程在课程进度达1/2(一半）时不再受理退费；常规折扣课程在课程进度达2/3（三分之二）时不再受理退费。
          </Paragraph>
          <Paragraph>
            5.退费流程：学员法定监护人到学习中心填写《提前结课申请表》，并提供缴费收据及合同原件、身份证复印件、退费银行卡复印件等。经财务审核后，于15个工作日左右将退款汇入所提供银行帐号中。
          </Paragraph>
          <Paragraph>
            6.退费标准：应退金额=所交总费用-课时消耗费-资料费-其它费用。不论交费时是否享受优惠，“课时消耗费”一律按原课时单价计算，课时消耗费=已上课时x原课时单价。赠送课时或现金优惠一律不予退还。
          </Paragraph>
          <Paragraph>7.退费期限：学员退费截止时间为本课程结束后的180天内，过期按自动放弃处理。</Paragraph>
          <Paragraph>
            8.学生因突发疾病须住院或长期休养、举家迁移至非本学区范围内的可办理退费。退费流程如第5点。
          </Paragraph>

          <Paragraph>
            <b>第六条 保密事项</b>
          </Paragraph>
          <Paragraph>
            1.甲方不得泄露、散布乙方的个人隐私和信息。甲方因公益或商业需要使用涉及乙方相关信息时，甲方应与乙方协商，并征得乙方同意。
          </Paragraph>
          <Paragraph>
            2.乙方应当尊重甲方的知识产权，不得擅自对教学培训活动进行拍照、录像、录音。对于甲方拥有知识产权的教学材料或课件，乙方不得擅自复制、散发、销售，及通过互联网进行扩散和传播。
          </Paragraph>

          <Paragraph>
            <b>第七条 违约责任</b>
          </Paragraph>
          <Paragraph>
            （一）甲方因自身原因要求解除培训合同，或没有按合同约定提供服务，导致乙方提前解除合同的，甲方应退还剩余培训费用。
          </Paragraph>
          <Paragraph>
            （二）乙方有下列情形之一的，甲方有权单方面解除合同，按本合同第五条第6点退还剩余学费。造成甲方或他人人身、财产损害的，乙方应承担损害赔偿责任。
          </Paragraph>
          <Paragraph>1.对辅导老师、学员或相关人员进行侮辱、威胁、恐吓、严重侵害人身安全或其他暴力行为的。</Paragraph>
          <Paragraph>2.偷窃、盗用、藏匿或未经允许从学习中心取出属于学习中心或其它人员的财物且数额较大的。</Paragraph>
          <Paragraph>3.故意损害、破坏老师、学员、相关人员及学习中心财物，与及学习中心责任范畴内公共财物的。</Paragraph>
          <Paragraph>
            4.在学习中心期间或上课过程中无心上课、玩手机、睡觉，不服从老师管教，严重影响教学秩序，甲方第一次提醒，第二次通知学员法定监护人，第三次教育后仍不改正的。
          </Paragraph>
          <Paragraph>5.故意隐瞒不适宜到甲方学习中心上课的身心疾病的。</Paragraph>
          <Paragraph>6.其他严重过失或不当行为、其它导致严重不良后果的违法或刑事行为。</Paragraph>
          <Paragraph>
            （三）因战争、自然灾害、传染性疾病等不可抗力致使本合同无法继续履行的，双方互不承担违约责任，受不可抗力影响的一方应及时书面通知对方，双方协商结算剩余培训费用。
          </Paragraph>

          <Paragraph>
            <b>第八条 争议处理</b>
          </Paragraph>
          <Paragraph>
            本合同在履行过程中发生争议，双方可协商解决，协商不成的，任何一方均有权向
            <span className={style.underline}>甲方所在地</span>人民法院提起诉讼。任何口头承诺无效。
          </Paragraph>

          <Paragraph>
            <b>第九条 其他约定事项</b>
          </Paragraph>
          <Paragraph>
            <Input
              plus
              maxFontNum={50}
              fixedRowsNumber={2}
              value={props.data.otherThings}
              onChange={(val) => {
                props.handleChange('otherThings', val);
              }}
            />
          </Paragraph>

          <Paragraph>
            <b>第十条 未尽事宜</b>
          </Paragraph>
          <Paragraph>
            <b>本合同及其附件以及补充协议中未约定事宜，均按照国家有关法律、法规和规章执行。</b>
          </Paragraph>
          <Paragraph>
            本合同自甲方盖章乙方签字或双方采用合法有效的电子签名方式签署之日起生效。本合同一式{' '}
            <span className={style.underline}>2</span> 份，甲方留存 <span className={style.underline}>1</span>{' '}
            份，乙方留存 <span className={style.underline}>1</span> 份，各份具有同等法律效力。
          </Paragraph>

          <Paragraph>
            <SignedSealed
              contractYear={props.data.contractYear}
              contractMonth={props.data.contractMonth}
              contractDay={props.data.contractDay}
              handleChange={props.handleChange}
            />
          </Paragraph>
        </Contract>
      </A4>
    </div>
  );
};

SPARK_STANDARD.propTypes = {
  data: PropTypes.oneOfType([PropTypes.object, PropTypes.array]),
  handleChange: PropTypes.func,
};

export default SPARK_STANDARD;
