import React from 'react';
import PropTypes from 'prop-types';
import './print.less';
import styles from './index.less';
import { Row, Col } from 'antd';
import A4 from '../../../components/A4';
import Contract from '../../../components/Contract';
import Paragraph from '../../../components/Paragraph';
import Content from '../../../components/Content';
import Input from '../../../components/Input';
import CheckBox from '../../../components/CheckBox';
import SignedSealed from '../../../components/SignedSealed';

const A4WrapStyle = {
  padding: '30px 65px',
};
const contractWrapStyle = {
  fontSize: '13px',
  fontWeight: 'normal',
};
const marginRight_0 = { marginRight: 0 };
const marginRight_8 = { marginRight: 8 };
const A4wrapClass = 'xh-min-height';
const title = (
  <div style={{ fontSize: '18px', fontFamily: '仿宋', textAlign: 'center' }}>
    <b>中小学生校外培训服务合同</b>
  </div>
);

function formatGrade(gradeName) {
  return gradeName ? String(gradeName).replace(/高一年级/, '高一') : '';
}

const SPARK_STANDARD = (props) => {
  return (
    <div>
      <A4 wrapStyle={A4WrapStyle} wrapClass={A4wrapClass}>
        <Contract title={title} wrapStyle={contractWrapStyle}>
          <Paragraph style={{ textAlign: 'right' }}>
            合同编号：
            <Input
              value={props.data.contractId}
              onChange={(val) => {
                props.handleChange('contractId', val);
              }}
              width={150}
            />
          </Paragraph>
          <Paragraph indent={0}>甲方（提供培训方）：</Paragraph>
          <Paragraph indent={0}>
            <Content label="机构名称（与民非登记证/营业执照或办学许可证一致）：">
              <Input
                value={props.data.institutionName}
                onChange={(val) => {
                  props.handleChange('institutionName', val);
                }}
              />
            </Content>
          </Paragraph>
          <Paragraph indent={0}>
            <Content label="办学地址：" flex>
              <Input
                plus
                maxRowsNumber={2}
                maxFontNum={45}
                value={props.data.schoolAddress}
                onChange={(val) => {
                  props.handleChange('schoolAddress', val);
                }}
              />
            </Content>
          </Paragraph>
          <Paragraph indent={0} style={{ marginTop: '2px' }}>
            <Row>
              <Col span={11}>
                <Content label="审批机关 ：">
                  <Input
                    value={props.data.examineOrganization}
                    onChange={(val) => {
                      props.handleChange('examineOrganization', val);
                    }}
                  />
                </Content>
              </Col>
              <Col span={13}>
                <Content label="登记注册机关：">
                  <Input
                    value={props.data.registrationAuthority}
                    onChange={(val) => {
                      props.handleChange('registrationAuthority', val);
                    }}
                  />
                </Content>
              </Col>
            </Row>
          </Paragraph>

          <Paragraph indent={0}>
            <Row>
              <Col span={11}>
                <Content label="办学许可证编号：">
                  <Input
                    value={props.data.schoolLicenseNumber}
                    onChange={(val) => {
                      props.handleChange('schoolLicenseNumber', val);
                    }}
                  />
                </Content>
              </Col>
              <Col span={13}>
                <Content label="办学许可证有效期：">
                  <Input
                    value={props.data.schoolLicenseTime}
                    onChange={(val) => {
                      props.handleChange('schoolLicenseTime', val);
                    }}
                  />
                </Content>
              </Col>
            </Row>
          </Paragraph>

          <Paragraph indent={0}>
            <Row>
              <Col span={11}>
                <Content label="线上机构ICP备案号：">
                  <Input
                    value={props.data.recordNumber}
                    onChange={(val) => {
                      props.handleChange('recordNumber', val);
                    }}
                  />
                </Content>
              </Col>
              <Col span={13}>
                <Content label="统一社会信用代码：">
                  <Input
                    value={props.data.socialCode}
                    onChange={(val) => {
                      props.handleChange('socialCode', val);
                    }}
                  />
                </Content>
              </Col>
            </Row>
          </Paragraph>

          <Paragraph indent={0}>
            <Row>
              <Col span={24}>
                <Content label="民非登记证/营业执照有效期：">
                  <Input
                    value={props.data.businessLicenseTime}
                    onChange={(val) => {
                      props.handleChange('businessLicenseTime', val);
                    }}
                  />
                </Content>
              </Col>
            </Row>
            <Row>
              <Col span={12}>
                <Content label="联系人：">
                  <Input
                    value={props.data.contactPerson}
                    onChange={(val) => {
                      props.handleChange('contactPerson', val);
                    }}
                  />
                </Content>
              </Col>
              <Col span={12}>
                <Content label="联系电话：">
                  <Input
                    value={props.data.contactPhoneNumber}
                    onChange={(val) => {
                      props.handleChange('contactPhoneNumber', val);
                    }}
                  />
                </Content>
              </Col>
            </Row>
          </Paragraph>

          <Paragraph indent={0}>乙方（接受培训方监护人）：</Paragraph>
          {/*<Paragraph indent={0}>
            <Row>
              <Col span={8}>
                <Content label="付款人姓名（必填）：">
                  <Input
                    value={props.data.payerName}
                    onChange={(val) => { props.handleChange('payerName', val); }}
                  />
                </Content>
              </Col>
              <Col span={16}>
                <Content label="联系电话（必填）：">
                  <Input
                    disabled
                    value={props.data.payerPhone}
                    onChange={(val) => { props.handleChange('payerPhone', val); }}
                  />
                </Content>
              </Col>
            </Row>
          </Paragraph>
          <Paragraph indent={0}>
            <Row>
              <Col span={16}>
                <Content label="付款人身份证号（必填）：">
                  <Input
                    value={props.data.payerIdCardNo}
                    onChange={(val) => { props.handleChange('payerIdCardNo', val); }}
                  />
                </Content>
              </Col>
              <Col span={8}>
                <Content label="与学员关系">
                  <Input
                    value={props.data.payerStudentRelation}
                    onChange={(val) => { props.handleChange('payerStudentRelation', val); }}
                  />
                </Content>
              </Col>
            </Row>
          </Paragraph>*/}
          <Paragraph indent={0}>
            <Row>
              <Col span={7}>
                <Content label="学员姓名(必填)：">
                  <Input
                    value={props.data.studentName}
                    onChange={(val) => {
                      props.handleChange('studentName', val);
                    }}
                  />
                </Content>
              </Col>
              <Col span={5}>
                <Content label="性别(必填)：">
                  <Input
                    value={props.data.studentGender}
                    onChange={(val) => {
                      props.handleChange('studentGender', val);
                    }}
                  />
                </Content>
              </Col>
              <Col span={6}>
                <Content label="出生日期：">
                  <Input
                    value={props.data.studentBirthday}
                    onChange={(val) => {
                      props.handleChange('studentBirthday', val);
                    }}
                  />
                </Content>
              </Col>
              <Col span={6}>
                <Content label="年级：">
                  <Input
                    value={formatGrade(props.data.studentGrade)}
                    onChange={(val) => {
                      props.handleChange('studentGrade', val);
                    }}
                  />
                </Content>
              </Col>
            </Row>
          </Paragraph>

          <Paragraph indent={0}>
            <Row>
              <Col span={12}>
                <Content label="就读学校：">
                  <Input
                    value={props.data.schoolName}
                    onChange={(val) => {
                      props.handleChange('schoolName', val);
                    }}
                  />
                </Content>
              </Col>
              <Col span={12}>
                <Content label="身份证号：">
                  <Input
                    value={props.data.studentIdNumber}
                    onChange={(val) => {
                      props.handleChange('studentIdNumber', val);
                    }}
                  />
                </Content>
              </Col>
            </Row>
          </Paragraph>

          <Paragraph indent={0}>
            <Row>
              <Col span={12}>
                <Content label="联系电话：">
                  <Input
                    value={props.data.studentPhone}
                    onChange={(val) => {
                      props.handleChange('studentPhone', val);
                    }}
                  />
                </Content>
              </Col>
            </Row>
          </Paragraph>

          <Paragraph indent={0}>
            <Row>
              <Col span={8}>
                <Content label="监护人姓名：">
                  <Input
                    value={props.data.customer}
                    onChange={(val) => {
                      props.handleChange('customer', val);
                    }}
                  />
                </Content>
              </Col>
              <Col span={8}>
                <Content label="联系电话：">
                  <Input
                    value={props.data.customerPhone}
                    onChange={(val) => {
                      props.handleChange('customerPhone', val);
                    }}
                  />
                </Content>
              </Col>
              <Col span={8}>
                <Content label="与学员关系">
                  <Input
                    value={props.data.payerStudentRelation}
                    onChange={(val) => {
                      props.handleChange('payerStudentRelation', val);
                    }}
                  />
                </Content>
              </Col>
            </Row>
          </Paragraph>
          <Paragraph indent={0}>
            <Row>
              <Col span={12}>
                <Content label="监护人身份证号：">
                  <Input
                    value={props.data.customerIdNumber}
                    onChange={(val) => {
                      props.handleChange('customerIdNumber', val);
                    }}
                  />
                </Content>
              </Col>
              <Col span={12}>
                <Content label="联系地址：" flex>
                  <Input
                    value={props.data.customerAdress}
                    onChange={(val) => {
                      props.handleChange('customerAdress', val);
                    }}
                  />
                </Content>
              </Col>
            </Row>
          </Paragraph>

          <Paragraph>
            根据《中华人民共和国民法典》《中华人民共和国教育法》《中华人民共和国义务教育法》《中华人民共和国民办教育促进法》《中华人民共和国民办教育促进法实施条例》《中华人民共和国未成年人保护法》《中华人民共和国行政许可法》等有关法律、法规的规定，甲乙双方遵循平等、自愿、公平、诚实、守信的原则，遵循教育规律和青少年健康成长规律，经协商一致，签订本合同。
          </Paragraph>

          {/*<Paragraph indent={0}><b>第一条  适用对象</b></Paragraph>
          <Paragraph>本合同适用受培训者（学员）一般为在校的中小学生。</Paragraph>*/}

          <Paragraph indent={0}>
            <b>第一条 培训服务</b>
          </Paragraph>
          <Paragraph indent={1}>
            <span>本培训项目属于（单选）</span>
            <CheckBox
              value={props.data.pxOffLine}
              onClick={() => {
                props.handleChange('pxOffLine', !props.data.pxOffLine);
              }}
            >
              线下学科类培训
            </CheckBox>
            <CheckBox
              value={props.data.pxOnLine}
              onClick={() => {
                props.handleChange('pxOnLine', !props.data.pxOnLine);
              }}
            >
              线上学科类培训
            </CheckBox>
            <CheckBox
              value={props.data.pxOffLineOtherSubject}
              onClick={() => {
                props.handleChange('pxOffLineOtherSubject', !props.data.pxOffLineOtherSubject);
              }}
            >
              线下非学科类培训
            </CheckBox>
            <CheckBox
              value={props.data.pxOnLineOtherSubject}
              onClick={() => {
                props.handleChange('pxOnLineOtherSubject', !props.data.pxOnLineOtherSubject);
              }}
            >
              线上非学科类培训
            </CheckBox>
          </Paragraph>
          <Paragraph indent={0}>
            <b>（一）培训项目</b>
          </Paragraph>
          <Paragraph indent={0}>
            <Content label="课程名称：" flex>
              <Input
                plus
                maxRowsNumber={5}
                maxFontNum={45}
                value={props.data.className}
                onChange={(val) => {
                  props.handleChange('className', val);
                }}
              />
            </Content>
          </Paragraph>
          <Paragraph indent={0} style={{ marginTop: '2px' }}>
            <Row>
              <Col span={8}>
                <Content label="课程顾问（经办人）：">
                  <Input
                    value={props.data.classCounselor}
                    onChange={(val) => {
                      props.handleChange('classCounselor', val);
                    }}
                  />
                </Content>
              </Col>
              <Col span={16}>
                <Content label="总培训时长（小时）">
                  <Input
                    disabled
                    value={props.data.classNatureHour}
                    onChange={(val) => {
                      props.handleChange('classNatureHour', val);
                    }}
                    width={110}
                  />
                  （其中上阶段时长为
                  <Input
                    disabled
                    value={props.data.segFirstClassNatureHour}
                    onChange={(val) => {
                      props.handleChange('segFirstClassNatureHour', val);
                    }}
                    width={55}
                  />
                  小时）
                </Content>
              </Col>
            </Row>
          </Paragraph>
          <Paragraph indent={0}>
            <Row>
              <Col span={12}>
                <Content label="每次培训时长（小时）：">
                  <Input
                    value={props.data.onceClassNatureHour}
                    onChange={(val) => {
                      props.handleChange('onceClassNatureHour', val);
                    }}
                  />
                </Content>
              </Col>
              <Col span={12}>
                <Content label="上课时间：">
                  <Input
                    value={props.data.classStartTime}
                    onChange={(val) => {
                      props.handleChange('classStartTime', val);
                    }}
                    onBlur={(val) => {
                      props.handleChange('classStartTime', val || '/');
                    }}
                  />
                </Content>
              </Col>
            </Row>
          </Paragraph>
          <Paragraph indent={0}>
            <Row>
              <Col span={12}>
                <Content label="开课日期：">
                  <Input
                    value={props.data.openingDate}
                    onChange={(val) => {
                      props.handleChange('openingDate', val);
                    }}
                    onBlur={(val) => {
                      props.handleChange('openingDate', val || '/');
                    }}
                  />
                </Content>
              </Col>
              <Col span={12}>
                <Content label="预计结课日期：">
                  <Input
                    value={props.data.finishDate}
                    onChange={(val) => {
                      props.handleChange('finishDate', val);
                    }}
                    onBlur={(val) => {
                      props.handleChange('finishDate', val || '/');
                    }}
                  />
                </Content>
              </Col>
            </Row>
          </Paragraph>

          <Paragraph indent={0}>
            <b>（二）培训要求</b>
          </Paragraph>
          <Paragraph indent={1}>1.培训方式：</Paragraph>
          <Paragraph indent={1}>
            <CheckBox
              value={props.data.pxOneToOne}
              onClick={() => {
                props.handleChange('pxOneToOne', !props.data.pxOneToOne);
              }}
            >
              一对一
            </CheckBox>

            <span>( 或一对</span>
            <Input
              value={props.data.pxOneToManyPersonNumber}
              onChange={(val) => {
                props.handleChange('pxOneToManyPersonNumber', val);
              }}
              width={20}
            />
            <span style={{ marginRight: 10 }}>) 面授</span>

            <CheckBox
              value={props.data.pxBigClass}
              onClick={() => {
                props.handleChange('pxBigClass', !props.data.pxBigClass);
              }}
              style={{ marginRight: 0 }}
            >
              大班额面授课
            </CheckBox>
            <span>（标准：</span>
            <Input
              value={props.data.pxBigClassPersonNumberStart}
              onChange={(val) => {
                props.handleChange('pxBigClassPersonNumberStart', val);
              }}
              width={20}
            />
            <span>人--</span>
            <Input
              value={props.data.pxBigClassPersonNumberEnd}
              onChange={(val) => {
                props.handleChange('pxBigClassPersonNumberEnd', val);
              }}
              width={20}
            />
            <span> 人）</span>

            <CheckBox
              value={props.data.pxSmallClass}
              onClick={() => {
                props.handleChange('pxSmallClass', !props.data.pxSmallClass);
              }}
              style={{ marginRight: 0 }}
            >
              小班额面授课
            </CheckBox>
            <span>（班级限额≤</span>
            <Input
              value={props.data.pxSmallClassPersonNumber}
              onChange={(val) => {
                props.handleChange('pxSmallClassPersonNumber', val);
              }}
              width={20}
            />
            <span style={{ marginRight: 10 }}>人）</span>

            <CheckBox
              value={props.data.pxOtherStudyWay}
              onClick={() => {
                props.handleChange('pxOtherStudyWay', !props.data.pxOtherStudyWay);
              }}
            >
              其他方式：
              <Input
                value={props.data.pxOtherStudeWayDetails}
                onChange={(val) => {
                  props.handleChange('pxOtherStudeWayDetails', val);
                }}
                width={80}
              />
            </CheckBox>
            <br />

            <CheckBox
              value={props.data.minOpenClassPersonNumberFlag}
              onClick={() => {
                props.handleChange('minOpenClassPersonNumberFlag', !props.data.minOpenClassPersonNumberFlag);
              }}
              style={{ marginRight: 0 }}
            >
              最低开班人数
            </CheckBox>
            <Input
              value={props.data.minOpenClassPersonNumber}
              onChange={(val) => {
                props.handleChange('minOpenClassPersonNumber', val);
              }}
              width={20}
            />
            <span className={styles.mr_10}>，低于此人数可不开班；</span>
            <CheckBox
              value={props.data.unlimitedOpenClassPersonNumber}
              onClick={() => {
                props.handleChange('unlimitedOpenClassPersonNumber', !props.data.unlimitedOpenClassPersonNumber);
              }}
            >
              本班开班不受最低人数限制。
            </CheckBox>
          </Paragraph>
          <Paragraph indent={1}>
            <Row>
              <Col span={24}>
                2.是否指定授课教师：
                <CheckBox
                  value={props.data.notSpecifyTheTeacher}
                  onClick={() => {
                    props.handleChange('notSpecifyTheTeacher', !props.data.notSpecifyTheTeacher);
                  }}
                >
                  否
                </CheckBox>
                <CheckBox
                  value={props.data.specifyTheTeacher}
                  onClick={() => {
                    props.handleChange('specifyTheTeacher', !props.data.specifyTheTeacher);
                  }}
                  style={{ marginRight: 0 }}
                >
                  是
                </CheckBox>
                <span>（指定教师姓名：</span>
                <Input
                  value={props.data.specifyTheTeacherName}
                  onChange={(val) => {
                    props.handleChange('specifyTheTeacherName', val);
                  }}
                  width={100}
                />
                <span>，指定教师未经乙方同意不得更换）；</span>
              </Col>
              <Col span={24}>
                <span style={{ marginRight: '14px' }}>是否有教师资格证</span>
                <CheckBox
                  value={props.data.hasTeacherCertification}
                  onClick={() => {
                    props.handleChange('hasTeacherCertification', !props.data.hasTeacherCertification);
                  }}
                >
                  有
                </CheckBox>
                <CheckBox
                  value={props.data.noTeacherCertification}
                  onClick={() => {
                    props.handleChange('noTeacherCertification', !props.data.noTeacherCertification);
                  }}
                >
                  没有
                </CheckBox>
              </Col>
            </Row>
          </Paragraph>
          <Paragraph indent={1}>
            <Content label="3.实际授课地点：">
              <Input
                value={props.data.trainingAddress}
                onChange={(val) => {
                  props.handleChange('trainingAddress', val);
                }}
                width={200}
                className={styles.mr_10}
              />
              <span>4.学员接送方式：</span>
              <CheckBox
                value={props.data.takeBringtWaysParent}
                onClick={() => {
                  props.handleChange('takeBringtWaysParent', !props.data.takeBringtWaysParent);
                }}
                style={marginRight_8}
              >
                家长负责接送
              </CheckBox>
              <CheckBox
                value={props.data.takeBringtWaysOther}
                onClick={() => {
                  props.handleChange('takeBringtWaysOther', !props.data.takeBringtWaysOther);
                }}
                style={marginRight_0}
              >
                其他
              </CheckBox>
              <Input
                value={props.data.takeBringtWaysOtherDetail}
                onChange={(val) => {
                  props.handleChange('takeBringtWaysOtherDetail', val);
                }}
                width={70}
              />
            </Content>
          </Paragraph>

          <Paragraph indent={0}>
            <b>第二条 培训收费</b>
          </Paragraph>

          <Paragraph indent={0}>
            <b>（一）收费标准（人民币）</b>
          </Paragraph>
          <Paragraph>
            <Content label="培训费用合计：">
              <Input
                value={props.data.trainingCostMajuscule}
                onChange={(val) => {
                  props.handleChange('trainingCostMajuscule', val);
                }}
                width={180}
              />
              <span>（大写）</span>
              <Input
                value={props.data.trainingCost}
                onChange={(val) => {
                  props.handleChange('trainingCost', val);
                }}
                width={180}
              />
              <span>（小写）元。</span>
            </Content>
            <Content label="1.培训课程费：共计">
              <Input
                value={props.data.classFee}
                onChange={(val) => {
                  props.handleChange('classFee', val);
                }}
                width={180}
              />
              <span>元，其中上阶段培训课程费为</span>
              <Input
                value={props.data.segFirstClassFee}
                onChange={(val) => {
                  props.handleChange('segFirstClassFee', val);
                }}
                onBlur={(val) => {
                  props.handleChange('segFirstClassFee', val || '/');
                }}
                width={180}
              />
              <span>元、下阶段培训课程费为</span>
              <Input
                value={props.data.segSecondClassFee}
                onChange={(val) => {
                  props.handleChange('segSecondClassFee', val);
                }}
                onBlur={(val) => {
                  props.handleChange('segSecondClassFee', val || '/');
                }}
                width={180}
              />
              <span>元（</span>
              <Input
                value={props.data.classFeeForNatureOne}
                onChange={(val) => {
                  props.handleChange('classFeeForNatureOne', val);
                }}
                width={80}
              />
              <span>元/小时）。</span>
            </Content>
            <Content label="2.培训服务费：">
              <Input
                value={props.data.trainingMaterialsFee}
                onChange={(val) => {
                  props.handleChange('trainingMaterialsFee', val);
                }}
                width={180}
              />
              <span>元，培训服务包括：</span>
              <Input value="/" width={180} disabled />
              <span>。</span>
            </Content>

            <Content label="3.其他费用：" />
            <Row>
              <span>名称：</span>
              <Input
                value={props.data.otherFee1Name}
                onChange={(val) => {
                  props.handleChange('otherFee1Name', val);
                }}
                width={130}
              />
              <span>金额：</span>
              <Input
                value={props.data.otherFee1Amount}
                onChange={(val) => {
                  props.handleChange('otherFee1Amount', val);
                }}
                width={130}
              />
              <span>元，收费依据：</span>
              <Input
                value={props.data.otherFee1Evidence}
                onChange={(val) => {
                  props.handleChange('otherFee1Evidence', val);
                }}
                width={130}
              />
            </Row>
            <Row>
              <span>名称：</span>
              <Input
                value={props.data.otherFee2Name}
                onChange={(val) => {
                  props.handleChange('otherFee2Name', val);
                }}
                width={130}
              />
              <span>金额：</span>
              <Input
                value={props.data.otherFee2Amount}
                onChange={(val) => {
                  props.handleChange('otherFee2Amount', val);
                }}
                width={130}
              />
              <span>元，收费依据：</span>
              <Input
                value={props.data.otherFee2Evidence}
                onChange={(val) => {
                  props.handleChange('otherFee2Evidence', val);
                }}
                width={130}
              />
            </Row>
          </Paragraph>

          <Paragraph indent={0}>
            <b>（二）付费方式（人民币）</b>
          </Paragraph>
          <Paragraph>
            经甲乙双方协商，乙方采取以下第
            <Input
              value={props.data.payMethodNum}
              onChange={(val) => {
                props.handleChange('payMethodNum', val);
              }}
              width={80}
            />
            种方式付款：
          </Paragraph>
          <Paragraph>
            1.
            <Input
              value={props.data.payMethodByOnceLastPayYear}
              onChange={(val) => {
                props.handleChange('payMethodByOnceLastPayYear', val);
              }}
              width={60}
            />
            年
            <Input
              value={props.data.payMethodByOnceLastPayMonth}
              onChange={(val) => {
                props.handleChange('payMethodByOnceLastPayMonth', val);
              }}
              width={60}
            />
            月
            <Input
              value={props.data.payMethodByOnceLastPayDay}
              onChange={(val) => {
                props.handleChange('payMethodByOnceLastPayDay', val);
              }}
              width={60}
            />
            日之前一次性付清培训费用；
          </Paragraph>
          <Paragraph>
            <span>2.</span>
            <CheckBox
              value={props.data.payMethodByPxCycle}
              onClick={() => {
                props.handleChange('payMethodByPxCycle', !props.data.payMethodByPxCycle);
              }}
              style={marginRight_0}
            >
              培训周期超过
            </CheckBox>
            <Input
              value={props.data.pxCycleExceed}
              onChange={(val) => {
                props.handleChange('pxCycleExceed', val);
              }}
              width={40}
            />
            <span>个</span>
            <CheckBox
              value={props.data.pxCycleMonth}
              onClick={() => {
                props.handleChange('pxCycleMonth', !props.data.pxCycleMonth);
              }}
              style={marginRight_0}
            >
              月/
            </CheckBox>
            <CheckBox
              value={props.data.pxCycleClassHour}
              onClick={() => {
                props.handleChange('pxCycleClassHour', !props.data.pxCycleClassHour);
              }}
              style={marginRight_0}
            >
              小时的，
            </CheckBox>
            <CheckBox
              value={props.data.payMethodByPxFee}
              onClick={() => {
                props.handleChange('payMethodByPxFee', !props.data.payMethodByPxFee);
              }}
              style={marginRight_0}
            >
              培训费用金额超过
            </CheckBox>
            <Input
              value={props.data.pxFeeExceed}
              onChange={(val) => {
                props.handleChange('pxFeeExceed', val);
              }}
              width={120}
            />
            <span>元的，</span>
          </Paragraph>
          <Paragraph>
            <Input
              value={props.data.payMethodByPx1Year}
              onChange={(val) => {
                props.handleChange('payMethodByPx1Year', val);
              }}
              width={60}
            />
            年
            <Input
              value={props.data.payMethodByPx1Month}
              onChange={(val) => {
                props.handleChange('payMethodByPx1Month', val);
              }}
              width={60}
            />
            月
            <Input
              value={props.data.payMethodByPx1Day}
              onChange={(val) => {
                props.handleChange('payMethodByPx1Day', val);
              }}
              width={60}
            />
            日 之前支付培训费用的
            <Input
              value={props.data.payMethodByPx1Percentage}
              onChange={(val) => {
                props.handleChange('payMethodByPx1Percentage', val);
              }}
              width={60}
            />
            %，计
            <Input
              value={props.data.payMethodByPx1Money}
              onChange={(val) => {
                props.handleChange('payMethodByPx1Money', val);
              }}
              width={60}
            />
            元；
          </Paragraph>
          <Paragraph>
            <Input
              value={props.data.payMethodByPx2Year}
              onChange={(val) => {
                props.handleChange('payMethodByPx2Year', val);
              }}
              width={60}
            />
            年
            <Input
              value={props.data.payMethodByPx2Month}
              onChange={(val) => {
                props.handleChange('payMethodByPx2Month', val);
              }}
              width={60}
            />
            月
            <Input
              value={props.data.payMethodByPx2Day}
              onChange={(val) => {
                props.handleChange('payMethodByPx2Day', val);
              }}
              width={60}
            />
            日 之前支付培训费用的
            <Input
              value={props.data.payMethodByPx2Percentage}
              onChange={(val) => {
                props.handleChange('payMethodByPx2Percentage', val);
              }}
              width={60}
            />
            %，计
            <Input
              value={props.data.payMethodByPx2Money}
              onChange={(val) => {
                props.handleChange('payMethodByPx2Money', val);
              }}
              width={60}
            />
            元；
          </Paragraph>
          <Paragraph>
            <Input
              value={props.data.payMethodByPx3Year}
              onChange={(val) => {
                props.handleChange('payMethodByPx3Year', val);
              }}
              width={60}
            />
            年
            <Input
              value={props.data.payMethodByPx3Month}
              onChange={(val) => {
                props.handleChange('payMethodByPx3Month', val);
              }}
              width={60}
            />
            月
            <Input
              value={props.data.payMethodByPx3Day}
              onChange={(val) => {
                props.handleChange('payMethodByPx3Day', val);
              }}
              width={60}
            />
            日 之前支付剩余
            <Input
              value={props.data.payMethodByPx3Percentage}
              onChange={(val) => {
                props.handleChange('payMethodByPx3Percentage', val);
              }}
              width={60}
            />
            %，计
            <Input
              value={props.data.payMethodByPx3Money}
              onChange={(val) => {
                props.handleChange('payMethodByPx3Money', val);
              }}
              width={100}
            />
            元；
          </Paragraph>
          <Paragraph>
            3.其他方式
            <Input
              value={props.data.payMethodByOtherContent}
              onChange={(val) => {
                props.handleChange('payMethodByOtherContent', val);
              }}
              width={256}
            />
            （说明）。
          </Paragraph>

          <Paragraph indent={0}>
            <b>（三）付费渠道</b>
          </Paragraph>

          <Paragraph indent={0}>
            <span style={{ marginRight: 5 }}>乙方采取</span>
            <CheckBox
              value={props.data.cash}
              onClick={() => {
                props.handleChange('cash', !props.data.cash);
              }}
              style={marginRight_8}
            >
              现金
            </CheckBox>
            <CheckBox
              value={props.data.bankTransfer}
              onClick={() => {
                props.handleChange('bankTransfer', !props.data.bankTransfer);
              }}
              style={marginRight_8}
            >
              银行卡
            </CheckBox>
            {/*<CheckBox
              value={props.data.aliPayWay}
              onClick={() => { props.handleChange('aliPayWay', !props.data.aliPayWay); }}
              style={marginRight_8}
            >支付宝</CheckBox>
            <CheckBox
              value={props.data.wxPayWay}
              onClick={() => { props.handleChange('wxPayWay', !props.data.wxPayWay); }}
              style={marginRight_8}
            >微信</CheckBox>*/}
            <CheckBox
              value={props.data.otherPayWay}
              onClick={() => {
                props.handleChange('otherPayWay', !props.data.otherPayWay);
              }}
              style={marginRight_0}
            >
              其他
            </CheckBox>
            <Input
              value={props.data.otherPayWayContent}
              onChange={(val) => {
                props.handleChange('otherPayWayContent', val);
              }}
              width={40}
            />
            方式支付培训费用。甲方的培训费用收款账户信息如下:
          </Paragraph>
          <Paragraph indent={1}>
            开户银行：
            <Input
              value={props.data.accountBank}
              onChange={(val) => {
                props.handleChange('accountBank', val);
              }}
              width={300}
            />
            银行账号：
            <Input
              value={props.data.account}
              onChange={(val) => {
                props.handleChange('account', val);
              }}
              width={180}
            />
          </Paragraph>
          {/*<Paragraph indent={1}>
            支付宝账户：
            <Input
              value={props.data.aliAccount}
              onChange={(val) => { props.handleChange('aliAccount', val); }}
              width={290}
            />
            微信账户：
            <Input
              value={props.data.wechatAccount}
              onChange={(val) => { props.handleChange('wechatAccount', val); }}
              width={180}
            />
          </Paragraph>
          <Paragraph>
            其他方式：
            <Input
              value={props.data.payChannelByOtherContent}
              onChange={(val) => { props.handleChange('payChannelByOtherContent', val); }}
              width={300}
            />
          </Paragraph>*/}

          <Paragraph indent={0}>
            <b>（四）预收费监管方式</b>
          </Paragraph>
          <Paragraph indent={1}>
            <CheckBox
              value={props.data.bankHosting}
              onClick={() => {
                props.handleChange('bankHosting', !props.data.bankHosting);
              }}
              style={marginRight_8}
            >
              银行托管
            </CheckBox>
            <CheckBox
              value={props.data.riskMargin}
              onClick={() => {
                props.handleChange('riskMargin', !props.data.riskMargin);
              }}
              style={marginRight_8}
            >
              风险保证金
            </CheckBox>
          </Paragraph>

          <Paragraph indent={0}>
            <b>第三条 甲方的权利和义务</b>
          </Paragraph>
          <Paragraph indent={0}>
            （一）甲方有权按照国家有关政策规定和合同约定收取培训费用。甲方收取培训费用后应当及时向乙方开具正规发票。（按照国家有关政策要求，培训机构不得一次性向乙方收取或变相收取时间跨度超过3个月的费用；线上培训机构按课时收费的，每科不得一次性收取超过60课时的费用。）
          </Paragraph>
          <Paragraph indent={0}>
            （二）甲方应当向乙方明示培训机构有效证明文件、收费项目、收费标准、收退费办法、培训范围、培训时间、教学人员资格和服务承诺等内容，公开透明培训，接受社会监督，甲方不得在公示的项目和标准外向乙方收取其他费用。
          </Paragraph>
          <Paragraph indent={0}>
            （三）甲方可以依照相关法律法规制定适合其机构自身的培训管理制度并在甲方培训场所醒目位置进行公示，甲方有权要求乙方遵照执行，以确保培训活动顺利进行。
          </Paragraph>
          <Paragraph indent={0}>
            （四）甲方开设培训项目须符合国家及培训场所所在地有关规定。甲方须选用与其培训项目及培训计划相匹配的培训材料，培训材料应当符合《校外培训机构培训材料管理办法（试行）》和当地有关实施细则规定。
          </Paragraph>
          <Paragraph indent={0}>
            （五）甲方保证，按照国家有关政策要求配备与培训项目及规模相适应的培训场所及设施设备，配备充足的教学人员、教研人员、培训管理人员、安全管理人员、助教、带班人员等辅助人员。同时，根据《校外培训机构从业人员管理办法（试行）》规定，加强对所聘用人员的管理，确保不出现打骂、猥亵、虐待等损害学员身心健康或合法权益的行为。
          </Paragraph>
          <Paragraph indent={0}>
            （六）甲方应做好消防、抗震、食品、公共卫生等安全管理，配备安全技术防范系统，建立健全安全管理制度和应急预警处理机制，防范各类安全责任事故发生。每次培训课程结束后，甲方应确保学员被乙方安全接走，双方另有约定的除外。甲方如使用校车接送培训学员，须按《校车安全管理条例》管理，审批时须提供校车使用许可。
          </Paragraph>
          <Paragraph indent={0}>（七）甲方若改变培训方式，需双方协商一致。</Paragraph>
          <Paragraph indent={0}>
            （八）甲方应当保护乙方个人信息，确保在收集、存储、使用、加工、公开等个人信息处理活动中严格遵守《中华人民共和国个人信息保护法》《中华人民共和国未成年人保护法》的规定。
          </Paragraph>
          <Paragraph indent={0}>
            （九）未经乙方书面同意，甲方不得将本合同约定的培训服务转让给第三方，不得擅自将学员转交给第三方机构进行培训。
          </Paragraph>
          <Paragraph indent={0}>
            （十）甲方应当设置处理合同和服务争议的内设部门或者专员，甲方的客服电话为：
            <Input
              value={props.data.customerServicePhone}
              onChange={(val) => {
                props.handleChange('customerServicePhone', val);
              }}
              width={120}
            />
            。
          </Paragraph>

          <Paragraph indent={0}>
            <b>第四条 乙方的权利和义务</b>
          </Paragraph>
          <Paragraph indent={0}>（一）乙方有按照本合同的约定接受甲方培训服务的权利。</Paragraph>
          <Paragraph indent={0}>
            （二）乙方对培训过程以及培训人员的从业背景和执教信息享有知情权。乙方可以通过公开课、学习报告等适当方式了解学员的学习状况，甲方应当为乙方提供方便，接受乙方监督。
          </Paragraph>
          <Paragraph indent={0}>
            （三）乙方应当按时足额向甲方缴纳培训费用。如甲方采用银行托管方式进行预收费监管，乙方应在托管协议规定的时间内对甲方授课完成和资金拨付予以确认；超过规定时限未确认的，视为确认同意。
          </Paragraph>
          <Paragraph indent={0}>
            （四）乙方及学员应当自觉遵守甲方的各项培训管理制度和课堂纪律，不得妨碍其他学员的正常学习活动。乙方应当自觉遵守培训场所的各种安全规定，不从事危害自身或者他人人身、财产安全的不当行为。培训期间如因乙方或学员的原因造成甲方或他人人身、财产损害的，乙方应承担损害赔偿责任。
          </Paragraph>
          <Paragraph indent={0}>
            （五）乙方及学员应当尊重甲方知识产权，不得擅自对培训过程进行录音、录像。对于甲方拥有知识产权的培训材料或者课件，乙方及学员除在正常学习过程中合理使用外，不得私自复制、散发、销售，不得通过互联网进行分享、扩散和传播。
          </Paragraph>
          <Paragraph indent={0}>
            （六）未经甲方同意，乙方不得擅自将本合同课程转让给第三方，或者将听课凭证转让、出借给他人使用。
          </Paragraph>
          <Paragraph indent={0}>
            （七）如学员身体健康状况有特殊情形不再适合参与培训的，乙方应及时告知甲方，甲方可按照相关培训管理制度处理。
          </Paragraph>

          <Paragraph indent={0}>
            <b>第五条 培训退费</b>
          </Paragraph>
          <Paragraph indent={0}>
            （一）乙方在培训班正式开班前[
            <Input
              value={props.data.refundBeforeOpenClassNum}
              onChange={(val) => {
                props.handleChange('refundBeforeOpenClassNum', val);
              }}
              width={20}
              className={styles.noUnderlineInput}
            />
            ]天或开班后[
            <Input
              value={props.data.refundAfterOpenClassNum}
              onChange={(val) => {
                props.handleChange('refundAfterOpenClassNum', val);
              }}
              width={20}
              className={styles.noUnderlineInput}
            />
            ]
            <CheckBox
              value={props.data.refundAfterOpenClassDay}
              onClick={() => {
                props.handleChange('refundAfterOpenClassDay', !props.data.refundAfterOpenClassDay);
              }}
            >
              天
            </CheckBox>
            前提出退学的，有权要求全额退费。
          </Paragraph>
          <Paragraph indent={0}>
            （二）由于乙方的原因申请提前退学的，双方同意按照第
            <Input
              value={props.data.refundMethodNum}
              onChange={(val) => {
                props.handleChange('refundMethodNum', val);
              }}
              width={40}
            />
            种约定方式办理退费。
          </Paragraph>
          <Paragraph indent={0}>1.退还乙方未消耗课程费所对应的培训费余额。</Paragraph>
          <Paragraph indent={0}>
            2.参加课程培训未达[
            <Input
              value={props.data.secondRefundNotReached}
              onChange={(val) => {
                props.handleChange('secondRefundNotReached', val);
              }}
              width={20}
              className={styles.noUnderlineInput}
            />
            ]%者，退还乙方未消耗课程费所对应的培训费余额；参加课程培训超过[
            <Input
              value={props.data.secondRefundExceed}
              onChange={(val) => {
                props.handleChange('secondRefundExceed', val);
              }}
              width={20}
              className={styles.noUnderlineInput}
            />
            ]%者，退还乙方未消耗课程费所对应培训费余额的[
            <Input
              value={props.data.secondRefundBalancePercentage}
              onChange={(val) => {
                props.handleChange('secondRefundBalancePercentage', val);
              }}
              width={20}
              className={styles.noUnderlineInput}
            />
            ]%。
          </Paragraph>
          <Paragraph indent={0}>
            3.其他方式
            <Input
              value={props.data.refundMethodByOtherContent}
              onChange={(val) => {
                props.handleChange('refundMethodByOtherContent', val);
              }}
              width={180}
            />
          </Paragraph>
          <Paragraph indent={0}>
            （三）在办理退费时，对于已发放给乙方的培训资料的费用、已转付给第三方并无法索回的代收代支费用以及已向银行（第三方）支付的合理手续费用等，由甲方出示相关证明材料后，经协商，由
            {/*<Input
              value={props.data.refundResponsibleParty}
              onChange={(val) => {
                props.handleChange('refundResponsibleParty', val);
              }}
              width={60}
            />*/}
            乙方承担。
          </Paragraph>
          <Paragraph indent={0}>
            （四）乙方所报班次低于最低开班人数不能开班的，甲方应退还乙方已缴付的全部费用。
          </Paragraph>
          <Paragraph indent={0}>
            （五）甲方应在收到乙方书面退费申请后
            <Input
              value={props.data.refundWorkday}
              onChange={(val) => {
                props.handleChange('refundWorkday', val);
              }}
              width={40}
            />
            （≤20）个工作日内，将相应退费款项支付给乙方。
          </Paragraph>
          <Paragraph indent={0}>（六）退费方式：按乙方缴费原路径或双方协商一致路径退回。</Paragraph>

          <Paragraph indent={0}>
            <b>第六条 违约责任</b>
          </Paragraph>
          <Paragraph indent={0}>
            （一）甲方未达到合同约定的场所、教师等培训条件的，或甲方未经乙方同意，擅自变更培训方式或培训教师的，乙方有权要求解除合同，要求甲方退还剩余培训费并支付剩余培训费[
            <Input
              value={props.data.liquidatedDamages1Percentage}
              onChange={(val) => {
                props.handleChange('liquidatedDamages1Percentage', val);
              }}
              width={20}
            />
            ]%金额的违约金。
          </Paragraph>
          <Paragraph indent={0}>
            （二）由于甲方的原因，包括但不限于甲方办学许可证过期，被吊销办学许可证、营业执照（或事业单位法人证书、民办非企业单位登记证书），被责令停业整顿等原因，无法继续向乙方提供培训服务的，乙方有权要求解除合同，要求甲方退还剩余培训费并支付剩余培训费[
            <Input
              value={props.data.liquidatedDamages2Percentage}
              onChange={(val) => {
                props.handleChange('liquidatedDamages2Percentage', val);
              }}
              width={20}
              className={styles.noUnderlineInput}
            />
            ]%金额的违约金。
          </Paragraph>
          <Paragraph indent={0}>
            （三）甲方招生简章或者招生广告中对培训师资和效果等所作的说明和允诺具体确定，并对培训合同的订立以及课程价格的确定有重大影响的，应当视为要约。相关说明和允诺即使未载入本合同，亦应当视为合同内容，甲方所提供服务与上述相关说明和允诺不相符的，乙方有权要求解除合同，要求甲方退还剩余培训费并支付剩余培训费[
            <Input
              value={props.data.liquidatedDamages3Percentage}
              onChange={(val) => {
                props.handleChange('liquidatedDamages3Percentage', val);
              }}
              width={20}
              className={styles.noUnderlineInput}
            />
            ]%金额的违约金。
          </Paragraph>
          <Paragraph indent={0}>
            （四）未经乙方同意，甲方擅自将本合同约定的服务转给第三方或将学员转交给第三方机构进行培训的，乙方有权要求解除合同，要求甲方退还剩余培训费并支付剩余培训费[
            <Input
              value={props.data.liquidatedDamages4Percentage}
              onChange={(val) => {
                props.handleChange('liquidatedDamages4Percentage', val);
              }}
              width={20}
              className={styles.noUnderlineInput}
            />
            ]%金额的违约金。
          </Paragraph>
          <Paragraph indent={0}>
            （五）因甲方违约，双方就退费事宜书面达成一致后，甲方应于
            <Input
              value={props.data.liquidatedDamages5Workday}
              onChange={(val) => {
                props.handleChange('liquidatedDamages5Workday', val);
              }}
              width={40}
            />
            （≤20）个工作日内将各项相关费用支付给乙方，每逾期一日应按逾期金额[
            <Input
              value={props.data.liquidatedDamages5Percentage}
              onChange={(val) => {
                props.handleChange('liquidatedDamages5Percentage', val);
              }}
              width={20}
              className={styles.noUnderlineInput}
            />
            ]%的标准（不超过万分之六点五）向乙方支付违约金。
          </Paragraph>
          <Paragraph indent={0}>
            （六）乙方逾期未支付培训费用的，甲方有权终止或中止培训服务，乙方需支付实际已培训天数的课程费，每逾期一日应按逾期金额[
            <Input
              value={props.data.partyBLiquidatedDamages6Percentage}
              onChange={(val) => {
                props.handleChange('partyBLiquidatedDamages6Percentage', val);
              }}
              width={20}
              className={styles.noUnderlineInput}
            />
            ]%的标准（不超过万分之六点五）向甲方支付违约金。
          </Paragraph>
          <Paragraph indent={0}>
            （七）乙方擅自将本合同课程转让给第三方，或者将听课凭证转让、出借给他人使用的，甲方有权拒绝为非学员提供培训服务。
          </Paragraph>
          <Paragraph indent={0}>（八）由于乙方的原因，无法继续接受培训服务的，甲方不承担违约责任。</Paragraph>
          <Paragraph indent={0}>
            （九）因不可抗力(如战争、自然灾害、政策、政府行为、突发疫情等)导致课程无法正常开展的，甲方将通过线上上课或其它形式课程保障课程合同的履行。乙方同意并配合甲方在不可抗力影响下的合理课程安排，并且同意不因不可抗力因素影响课程安排而单方面提出退费主张。
          </Paragraph>

          <Paragraph indent={0}>
            <b>第七条 争议处理</b>
          </Paragraph>
          <Paragraph indent={0}>
            本合同在履行过程中发生争议，双方可协商解决，协商不成的，任何一方有权向
            <Input
              value={props.data.disputeContent2}
              onChange={(val) => {
                props.handleChange('disputeContent2', val);
              }}
              width={120}
            />
            人民法院提起诉讼。任何口头承诺无效。
          </Paragraph>

          <Paragraph indent={0}>
            <b>第八条 补充协议</b>
          </Paragraph>
          <Paragraph indent={1}>
            本合同未尽事宜，由下列补充条款进行约定。补充条款与本合同其他条款不一致的，以补充条款为准。
          </Paragraph>
          <Paragraph indent={1}>
            <Content label="1.">
              <Input
                plus
                maxRowsNumber={2}
                maxFontNum={23}
                value={props.data.otherThings1}
                onChange={(val) => {
                  props.handleChange('otherThings1', val);
                }}
                width={280}
              />
            </Content>
          </Paragraph>
          <Paragraph indent={1}>
            <Content label="2.">
              <Input
                plus
                maxRowsNumber={2}
                maxFontNum={23}
                value={props.data.otherThings2}
                onChange={(val) => {
                  props.handleChange('otherThings2', val);
                }}
                width={280}
              />
            </Content>
          </Paragraph>
          <Paragraph indent={1}>
            <Content label="3.">
              <Input
                plus
                maxRowsNumber={2}
                maxFontNum={23}
                value={props.data.otherThings3}
                onChange={(val) => {
                  props.handleChange('otherThings3', val);
                }}
                width={280}
              />
            </Content>
          </Paragraph>
          <Paragraph indent={1}>
            <Content label="4.">
              <Input
                plus
                maxRowsNumber={2}
                maxFontNum={23}
                value={props.data.otherThings4}
                onChange={(val) => {
                  props.handleChange('otherThings4', val);
                }}
                width={280}
              />
            </Content>
          </Paragraph>
          <Paragraph indent={1}>
            <Content label="5.">
              <Input
                plus
                maxRowsNumber={2}
                maxFontNum={23}
                value={props.data.otherThings5}
                onChange={(val) => {
                  props.handleChange('otherThings5', val);
                }}
                width={280}
              />
            </Content>
          </Paragraph>

          <Paragraph indent={0}>
            <b>第九条 生效方式</b>
          </Paragraph>
          <Paragraph indent={0}>本合同自甲方盖章乙方签字或双方采用合法有效的电子签名方式签署之日起生效。</Paragraph>
          <Paragraph indent={0}>
            合同正本连同补充条款共
            <Input
              value={props.data.contractTotalPage}
              onChange={(val) => {
                props.handleChange('contractTotalPage', val);
              }}
              width={20}
            />
            页，一式
            <Input
              value={props.data.contractTotal}
              onChange={(val) => {
                props.handleChange('contractTotal', val);
              }}
              width={20}
            />
            份，甲乙双方各执
            <Input
              value={props.data.everyTotal}
              onChange={(val) => {
                props.handleChange('everyTotal', val);
              }}
              width={20}
            />
            份，各份具有同等法律效力。
          </Paragraph>

          <Paragraph indent={0}>
            <b>第十条 合同附件</b>
          </Paragraph>
          <Paragraph indent={1}>1.甲方服务项目说明与教学安排；</Paragraph>
          <Paragraph indent={1}>2.甲方培训收费标准；</Paragraph>
          <Paragraph indent={1}>3.甲方培训退费办法；</Paragraph>
          <Paragraph indent={1}>···</Paragraph>

          <Paragraph>
            <SignedSealed
              showPartyASeal={true}
              partyASeal={props.data.partyASeal}
              contractYear={props.data.contractYear}
              contractMonth={props.data.contractMonth}
              contractDay={props.data.contractDay}
              handleChange={props.handleChange}
            />
          </Paragraph>
        </Contract>
      </A4>
    </div>
  );
};

SPARK_STANDARD.propTypes = {
  data: PropTypes.oneOfType([PropTypes.object, PropTypes.array]),
  handleChange: PropTypes.func,
};

export default SPARK_STANDARD;
