import React from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';
import A4 from 'VIEW/Print/components/A4';
import Paragraph from 'VIEW/Print/components/Paragraph';
import { Row, Col } from 'antd';
import Input from 'VIEW/Print/components/Input';
import Content from 'VIEW/Print/components/Content';
import styles from './styles.module.less';

const SignedSealed = () => {
  return (
    <Row>
      <Col span={10}>
        <Paragraph className={styles.paragraph}>
          <Content label="甲方：">
            <Input disabled />
          </Content>
        </Paragraph>
        <Paragraph className={styles.paragraph}>
          <Row type="flex" justify="end">
            <Input width={80} disabled />
            年
            <Input width={38} disabled />
            月
            <Input width={38} disabled />日
          </Row>
        </Paragraph>
      </Col>
      <Col span={2} />
      <Col span={10}>
        <Paragraph className={styles.paragraph}>
          <Content label="乙方：">
            <Input disabled />
          </Content>
        </Paragraph>
        <Paragraph className={styles.paragraph}>
          <Row type="flex" justify="end">
            <Input width={80} disabled />
            年
            <Input width={38} disabled />
            月
            <Input width={38} disabled />日
          </Row>
        </Paragraph>
      </Col>
    </Row>
  );
};

const ServiceBook = (props) => {
  const { data } = props;
  let tips = '';
  if (data.customer && data.trainingCost) {
    tips = ` ${data.customer} 交来的咨询服务费，共 ${data.paidAmountCost} 元`;
  }

  return (
    <A4 wrapClass={styles.serviceBook}>
      <h2 className={styles.title}>
        <b>服务书</b>
      </h2>

      <Paragraph className={styles.paragraph} indent={0}>
        <Content label={<b className={styles.label}>今收到 </b>} isAlignCenter flex>
          <Input fontSize={20} maxFontNum={40} value={tips} plus />
        </Content>
      </Paragraph>

      <Paragraph className={styles.paragraph} indent={0}>
        <Content label={<b>金额（大写）：</b>} isAlignCenter>
          <Input value={data.paidAmountCostMajuscule} />
        </Content>
      </Paragraph>

      <Paragraph className={classNames(styles.paragraph, styles.separate)} indent={0}>
        就甲方为乙方提供服务事宜，双方协商一致，达成如下条款
      </Paragraph>

      <Paragraph className={styles.paragraph} indent={0}>
        <b>一、服务内容</b>
      </Paragraph>
      <Paragraph className={styles.paragraph} indent={0}>
        甲方根据甲乙双方的约定提供服务。
      </Paragraph>

      <Paragraph className={styles.paragraph} indent={0}>
        <b>二、费用</b>
      </Paragraph>
      <Paragraph className={styles.paragraph} indent={0}>
        费用总额为
        <Input value={data.trainingCost} width={160} disabled />
        元人民币。
      </Paragraph>

      <Paragraph className={styles.paragraph} indent={0}>
        <b>三、支付方式</b>
      </Paragraph>
      <Paragraph className={styles.paragraph} indent={0}>
        服务费用由乙方一次性支付至甲方指定账户。
      </Paragraph>

      <Paragraph className={styles.paragraph} indent={0}>
        <b>四、其他</b>
      </Paragraph>
      <Paragraph className={styles.paragraph} indent={0}>
        1、甲乙双方均有保密义务，违约方赔偿相应的经济损失。
      </Paragraph>
      <Paragraph className={styles.paragraph} indent={0}>
        2、本服务书从双方签字之日起生效，履行完毕即告终止。
      </Paragraph>
      <Paragraph className={styles.paragraph} indent={0}>
        3、在服务履行过程中，若需变更服务内容，须经双方协商同意。
      </Paragraph>

      <Paragraph className={styles.signedSealed} indent={0}>
        <SignedSealed />
      </Paragraph>
    </A4>
  );
};

ServiceBook.propTypes = {
  data: PropTypes.oneOfType([PropTypes.object, PropTypes.array]),
};

export default ServiceBook;
