const loadTemplate = (type, code) => {
  return new Promise((resolve) => {
    require.ensure([], () => {
      resolve(require(`VIEW/Print/template/${type}/${code}`));
    });
  });
};

const onInputChange = (callback) => {
  return (ele) => {
    callback && callback(ele.target.value);
  };
};

class TextUtil {
  constructor(option = {}) {
    this.option = {
      fontSize: option.fontSize || 14,
      halfSize: (option.fontSize && option.fontSize / 2) || 7,
    };
    this.zhReg = new RegExp(/[\u4E00-\u9FA5]+/);
    this.enReg = new RegExp(/[A-Za-z]+/);
    this.numReg = new RegExp(/[0-9]+/);
    this.blankReg = new RegExp(/^\s+$/);
    this.zhSymbolReg = new RegExp(
      /[\uff0c|\u3002|\u300a|\u300b|\uff1f|\uff1b|\uff1a|\u2018|\u2019|\u201c|\u201d|\u3010|\u3011|\u3001|\u00b7|\uff5e|\uff01|\uffe5|\u2026|\uff08|\uff09|\u2014]/,
    );
    this.enSymbolReg = new RegExp(
      /[\u002c|\u002e|\u002f|\u003c|\u003e|\u003f|\u003b|\u003a|\u0027|\u0022|\u005b|\u005d|\u007b|\u007d|\u005c|\u007c|\u0060|\u007e|\u0021|\u0040|\u0023|\u0024|\u0025|\u005e|\u0026|\u002a|\u0028|\u0029|\u002d|\u003d|\u002b|\u005f]/,
    );
  }

  isZH(char) {
    return this.zhReg.test(char);
  }
  isEN(char) {
    return this.enReg.test(char);
  }
  isNum(char) {
    return this.numReg.test(char);
  }
  isZHSymbol(char) {
    return this.zhSymbolReg.test(char);
  }
  isENSymbol(char) {
    return this.enSymbolReg.test(char);
  }
  isBlank(char) {
    return this.blankReg.test(char);
  }

  isFullSize(char) {
    return this.isZH(char) || this.isZHSymbol(char);
  }
  isHalfSize(char) {
    return this.isEN(char) || this.isNum(char) || this.isENSymbol(char) || this.isBlank(char);
  }

  getTotalWidth(str) {
    let totalWidth = 0;
    if (TextUtil.isString(str)) {
      str.split('').forEach((char) => {
        totalWidth += this.isHalfSize(char) ? this.option.halfSize : this.option.fontSize;
      });
    }
    return totalWidth;
  }

  splitByFontNum(str, maxNum) {
    if (TextUtil.isString(str)) {
      let fontArr = str.split('').map((char) => ({
        char,
        width: this.isHalfSize(char) ? this.option.halfSize : this.option.fontSize,
      }));
      let result = [''];
      let tempWidth = 0;
      let index = 0;
      const maxWidth = this.option.fontSize * maxNum;

      fontArr.forEach((font) => {
        if (tempWidth + font.width > maxWidth) {
          tempWidth = 0;
          ++index;
          result[index] = '';
        }
        tempWidth += font.width;
        result[index] += font.char;
      });
      return result;
    }
    return str;
  }

  static isString(str) {
    return typeof str === 'string';
  }
}

const printTextUtil = new TextUtil({ fontSize: 14 });

export { loadTemplate, onInputChange, TextUtil, printTextUtil };
