import React, { Component } from 'react';
import styles from './index.less';

const PDFJSViewer = require('pdfjs-dist/web/pdf_viewer.js');
const DEFAULT_SCALE_VALUE = 'page-width';

/**
 * 处理预览容器逻辑
 */
class Viewer extends Component {
  static defaultProps = {
    onInit: () => {},
    onScaleChanged: () => {},
    doc: null,
  };

  constructor(props) {
    super(props);

    this.state = {
      scale: (window.innerWidth / window.innerHeight) * 1.1,
    };

    this.pdfViewer = null;
  }

  componentDidMount() {
    this.initViewer();
    this.initDoc();
  }

  /**
   * 初始化预览容器
   * 监听 resize 事件
   * 监听 print 事件
   */
  initViewer = () => {
    const { onInit, onScaleChanged } = this.props;
    const { scale } = this.state;
    const eventBus = new PDFJSViewer.EventBus();
    const viewerContainer = this.ref_element;

    this.pdfViewer = new PDFJSViewer.PDFViewer({
      container: viewerContainer,
      eventBus: eventBus,
      textLayerMode: 0,
      useOnlyCssZoom: true,
    });

    eventBus.on('pagesinit', (_e) => {
      this.pdfViewer.currentScaleValue = DEFAULT_SCALE_VALUE;
      this.setState({
        scale: this.pdfViewer.currentScale,
      });

      if (onInit) {
        onInit({});
      }

      if (onScaleChanged) {
        onScaleChanged({ scale });
      }
    });

    eventBus.on('scalechange', (e) => {
      if (onScaleChanged) {
        onScaleChanged({ scale: e.scale });
      }
    });

    window.addEventListener(
      'resize',
      () => {
        if (this.pdfViewer) {
          const currentScaleValue = this.pdfViewer.currentScaleValue;

          if (currentScaleValue === 'auto' || currentScaleValue === 'page-fit' || currentScaleValue === 'page-width') {
            this.pdfViewer.currentScaleValue = currentScaleValue;
          } else if (!currentScaleValue) {
            this.pdfViewer.currentScaleValue = DEFAULT_SCALE_VALUE;
          }

          this.pdfViewer.update();
        }
      },
      false,
    );

    const mediaQueryList = window.matchMedia('print');
    mediaQueryList.addListener((mql) => {
      if (mql.matches) {
        // 'before print dialog open'
      } else {
        // 'after print dialog closed'
      }
    });
  };

  /**
   * 渲染文档
   */
  initDoc = () => {
    const { doc } = this.props;

    if (doc && this.pdfViewer) {
      this.pdfViewer.setDocument(doc);
    }
  };

  render() {
    return (
      <div
        className={styles.Viewer}
        ref={(c) => {
          this.ref_element = c;
        }}
      >
        <div className="pdfViewer" />
      </div>
    );
  }
}

export default Viewer;
