import React, { Component } from 'react';
import Viewer from './Viewer';
import styles from './index.less';

const pdfjsLib = require('pdfjs-dist/webpack');

/**
 * 处理PDF下载逻辑
 */
class PDFViewer extends Component {
  static defaultProps = {
    url: '', // 文档的完整地址
    onLoadStart: () => {}, // 初始化加载
    onLoadEnd: () => {}, // param: {doc, error}
  };

  constructor(props) {
    super(props);

    this.state = {
      doc: null, // 下载的本地文档
    };
  }

  componentDidMount() {
    this.initFile();
  }

  /**
   * 初始化加载文件
   * pdfjs-dist版本为@2.2.228
   * cMapUrl: pdfjs字库的路径
   * cMapPacked: 字体模糊处理
   * withCredentials: 是否开启跨域验证
   */
  initFile = () => {
    const { url, onLoadStart, onLoadEnd } = this.props;

    if (!url) {
      return;
    }

    const CMAP_URL = 'https://cdn.jsdelivr.net/npm/pdfjs-dist@2.0.288/cmaps/';
    const loadingTask = pdfjsLib.getDocument({
      url,
      cMapUrl: CMAP_URL,
      cMapPacked: true,
    });

    onLoadStart();
    loadingTask.promise.then(
      (doc) => {
        onLoadEnd({ doc });

        this.setState({
          doc,
        });
      },
      (e) => {
        onLoadEnd({ error: e });
      },
    );
  };

  render() {
    const { doc } = this.state;

    return <section className={styles.PDFViewer}>{doc && <Viewer doc={doc} />}</section>;
  }
}

export default PDFViewer;
