import React from 'react';
import printJS from 'print-js';
import styles from './index.less';
import Button from 'COMPONENT/Button';
import Loading from 'COMPONENT/Loading';
import NoDataPlus from 'COMPONENT/NoDataPlus';
import GuideBtn from './component/GuideBtn';
import ModalGuide from './component/ModalGuide';
// import PDFViewer from './component/PDFViewer';
import PDFViewer from './component/OnlinePDFViewer';

class PrintFile extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      fileUrl: '', // 文件地址
      fileName: '', // 文件名
      loading: false, // loading
      canDoPrint: false, // 是否允许打印
      modalGuideShow: false, // 是否显示配置指引
    };
  }

  componentDidMount() {
    this.initUrlQuery();
  }

  /**
   * 初始化URL参数
   */
  initUrlQuery = () => {
    const { query } = this.props.location;
    const { url = '' } = query || {};
    const decodedUrl = window.decodeURIComponent(url);
    const fileName = this.formatFileName(decodedUrl);

    this.setState({
      fileUrl: decodedUrl,
      fileName,
    });
  };

  /**
   * 格式化pdf文件名
   * @param url
   * @return {string}
   */
  formatFileName(url = '') {
    if (!url) {
      return '';
    }

    const start = url.lastIndexOf('/');
    const end = url.lastIndexOf('.');

    return url.substring(start + 1, end);
  }

  /**
   * 加载文件
   */
  onFileLoadStart = () => {
    this.setState({ loading: true });
  };

  onFileLoadEnd = (result = {}) => {
    const { doc } = result;

    this.setState({
      loading: false,
      canDoPrint: !!doc,
    });
  };

  /**
   * 打印
   */
  onPrint = () => {
    const { fileUrl = '' } = this.state;

    if (fileUrl) {
      printJS({
        printable: fileUrl,
        type: 'pdf',
      });
    }
  };

  /**
   * 切换弹窗提示
   */
  switchModalGuide = () => {
    this.setState((prevState) => ({ modalGuideShow: !prevState.modalGuideShow }));
  };

  render() {
    const { fileUrl, loading, canDoPrint, modalGuideShow } = this.state;

    return (
      <article className={styles.print}>
        <section className={styles.print_header + ' print-hide'}>
          <div className={styles.print_header_title}>
            <span className={styles.header_title_text}>BOSS打印管理</span>
          </div>
          <div className={styles.print_header_tools}>
            <Button type="primary" disabled={!canDoPrint} onClick={this.onPrint}>
              打印
            </Button>
          </div>
        </section>

        <section className={styles.print_body + ' print-no-padding-t-b'}>
          <GuideBtn text="打印配置指引" onClick={this.switchModalGuide} />

          {
            // fileUrl && <PDFViewer url={fileUrl} onLoadStart={this.onFileLoadStart} onLoadEnd={this.onFileLoadEnd}/>
            fileUrl && (
              <PDFViewer
                url={fileUrl}
                loading={loading}
                onLoadStart={this.onFileLoadStart}
                onLoad={this.onFileLoadEnd}
              />
            )
          }
        </section>

        <Loading loading={loading} />

        {!canDoPrint && <NoDataPlus text="文档加载失败" style={{ marginTop: 200 }} />}

        {modalGuideShow && <ModalGuide onCancel={this.switchModalGuide} />}
      </article>
    );
  }
}

export default PrintFile;
